/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "bman.h"
#include "services/data_pool_mng_ext.h"
#include "etc/mem_ext.h"
#include "e500v2_ext.h"

/*	Use only one pool of 2048 size buffers 
	It is safer to use a single pool now as we do not support scatter
	gather. In case the FM cannot get a buffer large enough, it will 
	use multiple small packets and create a scatter-gather frame descriptor. 
	We dont want that to happen now.
*/

static t_DataPoolSettings dataPoolSettings[] = 
#ifdef SINGLE_POOL
#define BPID1_NBUFS	32760
#define BPID0_NBUFS	16000

#define PER_PORTAL_SGT_BUFS \
{                                            \
    .partitionId    = 10,                     \
    .numOfBuffers   = BPID0_NBUFS/10,       \
    .bufferSize     = (512-DPAA_BMAN_PRIVATE_OFFSET),      \
    .alignment      = 64, \
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                   \
    .postfix        = 0,                     \
}, 

{ 
/*For scatter gather lists. Always make sure it is the first entry in the list because
 * it shall have a static BPID of 0*/
{                                           
    .partitionId    = 10,                    
#if !defined(PER_PORTAL_SGT_BUFS)
    .numOfBuffers   = BPID0_NBUFS,      
#else
    .numOfBuffers   = 0,      
#endif
    .bufferSize     = (512-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
}, 
/* remaining pools get dynamic BPID #'s */
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
PER_PORTAL_SGT_BUFS
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = BPID1_NBUFS/10,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
#ifdef NO_TX_CONFIRMATION_FLSHBUF
/*special entry will be mapped to IGNORED bpid (63)*/
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = 0,      
    .bufferSize     = (2048-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
#endif

};

#define INITIAL_NUM_OF_BUFFERS (BPID0_NBUFS+BPID1_NBUFS)

#else /* Multiple Pool */
{
/*For scatter gather lists. Always make sure it is the first entry in the list because
 * it shall have a static BPID of 0*/
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = 16384,      
    .bufferSize     = (512-DPAA_BMAN_PRIVATE_OFFSET),     
    .alignment      = 64,
    .prefix         = DPAA_BMAN_PRIVATE_OFFSET,                  
    .postfix        = 0,                    
},
/* remaining pools get dynamic BPID #'s */
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = 4096,      
    .bufferSize     = 256,     
    .alignment      = 0x80,	
    .prefix         = 32,                  
    .postfix        = 0,                    
},
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = 2048,      
    .bufferSize     = 512,     
    .alignment      = 0x80,
    .prefix         = 32,                  
    .postfix        = 0,                    
},
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = 1024,      
    .bufferSize     = 1024,     
    .alignment      = 0x80,
    .prefix         = 32,                  
    .postfix        = 0,                    
},
{                                           
    .partitionId    = 10,                    
    .numOfBuffers   = 512,      
    .bufferSize     = 2048,    
    .alignment      = 0x80,
    .prefix         = 32,                  
    .postfix        = 0,                    
}

};

#define INITIAL_NUM_OF_BUFFERS (16384+4096+2048+1024+512)

#endif /* SINGLE_POOL */

#define TOTAL_BPID_COUNT	 (sizeof(dataPoolSettings)/sizeof(dataPoolSettings[0]))
#define TOTAL_NUM_OF_BUFFERS (1048576>INITIAL_NUM_OF_BUFFERS?1048576:INITIAL_NUM_OF_BUFFERS)

int InitializeMemoryManager(bman_t *bman)
{
	int rc = E_OK;

	t_DataPoolMngParam poolParams;
	int i;

	poolParams.numOfBufferObjects = TOTAL_NUM_OF_BUFFERS;
	poolParams.numOfFrameObjects =  TOTAL_NUM_OF_BUFFERS;


	
	bman->h_BufPoolManager = DATA_POOL_MNG_Init(&poolParams);
	if(bman->h_BufPoolManager ==NULL){
        DPAA_LOG(LOG_CRIT_ERROR, "DATA_POOL_MNG_Init failed");
		return -1;
	}
	bman->numPoolParams = TOTAL_BPID_COUNT;
    DPAA_LOG(LOG_DEBUG_CONFIG, "%d / %d bman->numPoolParams=%d",sizeof(dataPoolSettings), sizeof(dataPoolSettings[0]), bman->numPoolParams);

	for (i = 0; i < bman->numPoolParams;i++) {
		if ( (rc = DataPoolMngPoolInit(bman->h_BufPoolManager,& dataPoolSettings[i], i)) != E_OK) {
        	DPAA_LOG(LOG_CRIT_ERROR, "DataPoolMngInit failed");
		}	 
	}

	return rc;
}

static void BmExceptionsCB(t_Handle h_App, e_BmExceptions exception)
{
    UNUSED(h_App);

    switch(exception)
    {
        case(e_BM_EX_INVALID_COMMAND):
            DPAA_LOG(LOG_RUNTIME_FAULT, "BM got exception e_BM_EX_INVALID_COMMAND");
            break;
        case(e_BM_EX_FBPR_THRESHOLD):
            DPAA_LOG(LOG_RUNTIME_FAULT, "BM got exception e_BM_EX_FBPR_THRESHOLD");
            break;
        case(e_BM_EX_SINGLE_ECC):
            DPAA_LOG(LOG_RUNTIME_FAULT, "BM got exception e_BM_EX_SINGLE_ECC");
            break;
        case(e_BM_EX_MULTI_ECC):
            DPAA_LOG(LOG_RUNTIME_FAULT, "BM got exception e_BM_EX_MULTI_ECC");
            break;
        default:
            DPAA_LOG(LOG_RUNTIME_FAULT, "BM got invalid exception %d", exception);
            break;
    }
}



/*****************************************************************************/
/*  Function Name	: bman_init
 *	Input			: Pointer to bman handler, member of dpaa_system_t
 *					  Pointer to DPAA configuration parameter
 *	Description		: Initializes BMAN  as per the parameters. 
 *	Returns			: NULL if error, else a valid BMAN handle. 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_init(dpaa_system_t *dpaa, IN dpaa_startup_params_t* cfg){
	int32_t rc=EOK;
	bman_t * bman=NULL;
	t_BmParam  bmParam;
#if defined(BPID1_NBUFS)
	uint32_t enb_macs, enb_gbits;
	uint32_t fm_index, mac_index;
#endif

	LOG_FUNCTION_ENTRY;
	dpaa->bman = (bman_handle_t )NULL;
	
	if(  (bman = malloc(sizeof(bman_t)))  == NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "Can not allocate memory to bman_t");
		bman=NULL;
		rc= ENOMEM;
		goto exit_cleanup;
	}
    memset(bman, 0, sizeof(bman_t));

    memset(&bmParam, 0, sizeof(t_BmParam));

	bmParam.guestId = NCSW_MASTER_ID;
    bmParam.baseAddress = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_BM, 0);
	bmParam.errIrq = NO_IRQ;
	bmParam.totalNumOfBuffers=TOTAL_NUM_OF_BUFFERS  ; /*Let DPAA adjust*/
	bmParam.fbprMemPartitionId=QNX_PARTITION_ID_NON_CACHED ;
	bmParam.partBpidBase = 0;
	bmParam.partNumOfPools = TOTAL_BPID_COUNT;
	bmParam.h_App= (t_Handle)bman;
	bmParam.f_Exception=BmExceptionsCB ;

	bman->mpool_initialized=0;

    bman->h_Bm = BM_Config(&bmParam);
	if(bman->h_Bm==NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "BM_Config Failure. ");
		rc= -1;
		goto exit_cleanup;
	}

	bman->master_if= cfg->master_if;

	dpaa->bman= (bman_handle_t)bman;

	if(BM_Init(bman->h_Bm) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "BM_Init Failure. ");
		rc= -1;
		BM_Free(bman->h_Bm);
		goto exit_cleanup;
	}
	
	DPAA_LOG(LOG_NORMAL, "BMAN Init Successful!");

#if defined(BPID1_NBUFS)
#ifdef PER_PORTAL_SGT_BUFS
	DPAA_LOG(LOG_NORMAL, "%s: distributing %d sgt buffers across active interfaces", __FUNCTION__, BPID0_NBUFS);
#endif
	DPAA_LOG(LOG_NORMAL, "%s: distributing %d data buffers across active interfaces", __FUNCTION__, BPID1_NBUFS);
	enb_macs  = 0;
	enb_gbits = 0;
	for(fm_index=0; fm_index<MAX_FMAN_PER_DPAA; fm_index++) {
		for(mac_index=0; mac_index<MAX_MACDEV_PER_FMAN; mac_index++) {
			if(cfg->macdev_enable[fm_index][mac_index]) {
				enb_macs++;
				if(mac_index == MAX_MACDEV_PER_FMAN-1)
					enb_gbits += 10;
				else
					enb_gbits += 1;
			}
		}
	}
	DPAA_LOG(LOG_NORMAL, "%s: found %d active interfaces with %d Gbits total", __FUNCTION__, enb_macs, enb_gbits);
	for(fm_index=0; fm_index<MAX_FMAN_PER_DPAA; fm_index++) {
		for(mac_index=0; mac_index<MAX_MACDEV_PER_FMAN; mac_index++) {
			uint32_t nbuf0 = 0, nbuf1 = 0;
			if(cfg->macdev_enable[fm_index][mac_index]) {
				if(mac_index == MAX_MACDEV_PER_FMAN-1) {
					nbuf0 = (1+10)*BPID0_NBUFS/(enb_macs+enb_gbits);
					nbuf1 = (1+10)*BPID1_NBUFS/(enb_macs+enb_gbits);
				} else {
					nbuf0 = (1+1)*BPID0_NBUFS/(enb_macs+enb_gbits);
					nbuf1 = (1+1)*BPID1_NBUFS/(enb_macs+enb_gbits);
				}
#ifdef PER_PORTAL_SGT_BUFS
				DPAA_LOG(LOG_NORMAL, "%s: allocating %d sgt buffers to portal %d, fm %d, mac %d", __FUNCTION__, nbuf0, fm_index*MAX_MACDEV_PER_FMAN + mac_index, fm_index, mac_index);
#else
				UNUSED(nbuf0);
#endif
				DPAA_LOG(LOG_NORMAL, "%s: allocating %d data buffers to portal %d, fm %d, mac %d", __FUNCTION__, nbuf1, fm_index*MAX_MACDEV_PER_FMAN + mac_index, fm_index, mac_index);
			}
#ifdef PER_PORTAL_SGT_BUFS
			dataPoolSettings[1+2*(fm_index*MAX_MACDEV_PER_FMAN + mac_index)+0].numOfBuffers = nbuf0;
			dataPoolSettings[1+2*(fm_index*MAX_MACDEV_PER_FMAN + mac_index)+1].numOfBuffers = nbuf1;
#else
			dataPoolSettings[1+(fm_index*MAX_MACDEV_PER_FMAN + mac_index)].numOfBuffers = nbuf1;
#endif
		}
	}
#endif

	if(InitializeMemoryManager(bman)==E_OK){
        DPAA_LOG(LOG_DEBUG_CONFIG, "Successful init of Memory manager");
	}
	else{
        DPAA_LOG(LOG_CRIT_ERROR, "Failure in init of Memory manager");
		BM_Free(bman->h_Bm);
		rc=-1;
		goto exit_cleanup;
	}

	rc=EOK;
	goto exit_ok;

exit_cleanup:

	
	if(bman!=NULL)
	free(bman);
	
	DPAA_LOG(LOG_CRIT_ERROR, "BMAN Init FAILURE!");

exit_ok:

	LOG_FUNCTION_EXIT;
	return rc;
}

/*****************************************************************************/
/*  Function Name	: bman_shutdown
 *	Input			: Bman handle
 *	Description		: Shutsdown the BMAN, clean memory allocated for bman_t 
 *	Returns			: ZERO if OK, else NON zero 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_shutdown(bman_handle_t bman_hdl ) {

	bman_t * bman= (bman_t*)bman_hdl;
	int i =0;
	LOG_FUNCTION_ENTRY;

    for (i=0; i<bman->numPoolParams; i++){
		BM_POOL_Free(bman->h_BmPool[i]);
	}
	
	for(i=0;i<4;i++){
		BM_PORTAL_Free(bman->h_BmPortal[i]);
	}
	
	BM_Free(bman->h_Bm);

	DPAA_LOG(LOG_NORMAL, "BMAN Shutdown");

	LOG_FUNCTION_EXIT;
	return 0;
}
/*****************************************************************************/
/*  Function Name	: bman_portal_init
 *	Input			: Bman handle
 *					  Request pointer
 *					  Response Pointer
 *	Description		: Initialize  
 *	Returns			: ZERO if OK, else NON zero 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_portal_init(bman_handle_t bman_hdl, IN init_bman_portal_t *req, OUT init_bman_portal_resp_t* rsp){
	bman_t * bman= (bman_t*)bman_hdl;
	t_BmPortalParam     bmPortalParam;
	int rc=EOK;

    memset(&bmPortalParam, 0, sizeof(t_BmPortalParam));

	bmPortalParam.ceBaseAddress = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_BM_CE_PORTAL,req->portal );
	bmPortalParam.ciBaseAddress = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_BM_CI_PORTAL,req->portal );
	bmPortalParam.h_Bm = bman->h_Bm;
	bmPortalParam.swPortalId = req->portal;
	bmPortalParam.irq = NO_IRQ; 

	if((bman->h_BmPortal[req->portal] = BM_PORTAL_Config(&bmPortalParam))==NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "BMAN  Portal %d Config FAILURE!",req->portal);
		rc=-1;
		goto err_exit;
	}
	if(BM_PORTAL_Init(bman->h_BmPortal[req->portal])!=E_OK){
		DPAA_LOG(LOG_CRIT_ERROR, "BMAN  Portal %d Init FAILURE!",req->portal);
		rc=-1;
		goto err_exit;
	}
	pthread_mutex_init(&bman->lock[req->portal], NULL);
	
err_exit:
	return rc;
}

/*****************************************************************************/
/*  Function Name	: bman_buf_release
 *	Input			: Bman handle
 *					  Request pointer
 *					  Response Pointer
 *	Description		: Releases an entry into RCR for portal.  
 *	Returns			: ZERO if OK, else NON zero 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_buf_release(bman_handle_t bman_hdl, IN buf_release_t *req, OUT buf_release_resp_t* resp ){
	uint32_t pool_id=0;
	int rc=EOK;
	bman_t * bman= (bman_t*)bman_hdl;
	LOG_FUNCTION_ENTRY;
	PORTAL_LOCK(req->flags, &bman->lock[req->portal]);
	

	for(pool_id=0;pool_id<bman->numPoolParams;pool_id++){
		if(BM_POOL_GetId(bman->h_BmPool[pool_id])==req->bpid)
		break;
	}
	if(pool_id==bman->numPoolParams){
		DPAA_LOG(LOG_RUNTIME_FAULT, "Buffer Pool id %d can not be found ",req->bpid);
		rc=-1;
		goto exit_func;
	}

	DPAA_LOG(LOG_DEBUG_DATA_FLOW, "release Buffer intoPool %d",
			 req->bpid);
	
	if(BM_POOL_PutBuf(bman->h_BmPool[pool_id], bman->h_BmPortal[req->portal], req->addr)!=E_OK){
		DPAA_LOG(LOG_RUNTIME_FAULT, "Buffer Pool id %d can not release vaddr %p",pool_id,req->addr);
		rc=-1;
		goto exit_func;
	}


exit_func:
	PORTAL_UNLOCK(req->flags, &bman->lock[req->portal]);
	LOG_FUNCTION_EXIT;

	return rc;
}

/*****************************************************************************/
/*  Function Name	: bman_buf_acquire
 *	Input			: Bman handle
 *					  Buffer acquire Request
 *					  Buffer acquire response
 *	Description		: Acquires a buffer from given Buffer ID  
 *	Returns			: ZERO if OK, else NON zero 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_buf_acquire( bman_handle_t bman_hdl, IN buf_acquire_t *req, OUT buf_acquire_resp_t* resp ){

	uint32_t pool_id=0;
	int rc=EOK;
	bman_t * bman= (bman_t*)bman_hdl;
	void *vaddr=NULL;

	LOG_FUNCTION_ENTRY;
	PORTAL_LOCK(req->flags, &bman->lock[req->portal]);

	for(pool_id=0;pool_id<bman->numPoolParams;pool_id++){
		if(BM_POOL_GetId(bman->h_BmPool[pool_id])==req->bpid){
			break;
		}
	}
	if(pool_id==bman->numPoolParams){
		DPAA_LOG(LOG_RUNTIME_FAULT, "Buffer Pool id %d can not be found ",req->bpid);
		rc=-1;
		goto exit_func;
	}

	DPAA_LOG(LOG_DEBUG_DATA_FLOW, "Acquire Buffer from Pool %d ",req->bpid);

	vaddr= BM_POOL_GetBuf(bman->h_BmPool[pool_id], bman->h_BmPortal[req->portal]);

	if(vaddr==NULL){
		DPAA_LOG(LOG_RUNTIME_FAULT, "Buffer Pool id %d No buffer found ",req->bpid);
		rc=-1;
		goto exit_func;
	}
	resp->count= 1;
	resp->bpid= req->bpid  ;
	resp->addr[0]= (uintptr_t)vaddr ;

exit_func:
	PORTAL_UNLOCK(req->flags, &bman->lock[req->portal]);
	/*Invert CR VBit*/	
	LOG_FUNCTION_EXIT;
	return rc;
}

/*****************************************************************************/
/*  Function Name	: bman_bufs_flush
 *	Input			: Bman handle
 *					  Buffer acquire Request
 *					  Buffer acquire response
 *	Description		: Flushes buffers from given Buffer ID  
 *	Returns			: ZERO if OK, else NON zero 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_bufs_flush( bman_handle_t bman_hdl, IN buf_acquire_t *req, OUT buf_acquire_resp_t* resp ){

	uint32_t pool_id=0;
	int rc=EOK;
	bman_t * bman= (bman_t*)bman_hdl;

	LOG_FUNCTION_ENTRY;
	PORTAL_LOCK(req->flags, &bman->lock[req->portal]);

	for(pool_id=0;pool_id<bman->numPoolParams;pool_id++){
		if(BM_POOL_GetId(bman->h_BmPool[pool_id])==req->bpid){
			break;
		}
	}
	if(pool_id==bman->numPoolParams){
		DPAA_LOG(LOG_RUNTIME_FAULT, "Buffer Pool id %d can not be found ",req->bpid);
		rc=-1;
		goto exit_func;
	}

	DPAA_LOG(LOG_DEBUG_DATA_FLOW, "Flush %d Buffers from Pool %d ", req->num, req->bpid);

	resp->count = BM_POOL_FlushBufs(bman->h_BmPool[pool_id], bman->h_BmPortal[req->portal], req->num);
	resp->bpid= req->bpid  ;
	resp->addr[0]= (uintptr_t)NULL ;

exit_func:
	PORTAL_UNLOCK(req->flags, &bman->lock[req->portal]);
	/*Invert CR VBit*/	
	LOG_FUNCTION_EXIT;
	return rc;
}

/*****************************************************************************/
/*  Function Name	: bman_portal_def_cbfn
 *	Input			: 
 *					  
 *					  
 *	Description		:   
 *	Returns			: void 
 *	Note			:				
 */
/*****************************************************************************/
void bman_portal_def_cbfn(void *token,uint32_t* size){
	LOG_FUNCTION_ENTRY;
	LOG_FUNCTION_EXIT;
}
/*****************************************************************************/
/*  Function Name	: bman_error_def_cbfn
 *	Input			: 
 *					  
 *					  
 *	Description		:   
 *	Returns			: void 
 *	Note			:				
 */
/*****************************************************************************/
void bman_error_def_cbfn(void *token,uint32_t* size){
	LOG_FUNCTION_ENTRY;

	LOG_FUNCTION_EXIT;
}
/*****************************************************************************/
/*  Function Name	: bman_bpool_init
 *	Input			: bman Handle
 *					  Init Request
 *					  Init Response
 *	Description		: Populates a Buffer Pool structure and allocates an empty
 *					  Buffer pool if requested.
 *	Returns			: void 
 *	Note			:				
 */
/*****************************************************************************/
int 
bman_bpool_init	(bman_handle_t bman_hdl, IN init_bpool_t *req, OUT init_bpool_resp_t *resp ){
	int i=0;
	int rc=EOK;
	bman_t * bman= (bman_t*)bman_hdl;
	t_BmPoolParam       bmPoolParam;
	t_DataPoolInfo      dataPoolInfo;

	if(!bman->mpool_initialized){
		assert(bman->numPoolParams > 0);
   		for (i=0; i<bman->numPoolParams; i++)
    	{
        	memset(&bmPoolParam, 0, sizeof(bmPoolParam));
			if(DATA_POOL_MNG_GetPoolInfo(bman->h_BufPoolManager,i, &dataPoolInfo)!=E_OK){
				DPAA_LOG(LOG_CRIT_ERROR, "DATA_POOL_MNG_GetPoolInfo() failure. Unable to retrieve pool index %d info",i);
				rc=-1;
				break;
			}
	       	bmPoolParam.h_Bm      					= bman->h_Bm;
	       	bmPoolParam.h_BmPortal      			= bman->h_BmPortal[req->portal];
			bmPoolParam.numOfBuffers    			= MEM_GetNumOfBlocks(dataPoolInfo.p_BufMemAccess->h_MemMng);
    	    bmPoolParam.bufferPoolInfo.h_BufferPool = dataPoolInfo.p_BufMemAccess; 
        	bmPoolParam.bufferPoolInfo.f_GetBuf     = DATA_POOL_DefaultGetBuf;
        	bmPoolParam.bufferPoolInfo.f_PutBuf     = DATA_POOL_DefaultPutBuf;
        	bmPoolParam.bufferPoolInfo.bufferSize   = dataPoolInfo.bufferSize;
    		bmPoolParam.bufferPoolInfo.f_PhysToVirt = NULL;  /* Let DPAA fill it internally */
    		bmPoolParam.bufferPoolInfo.f_VirtToPhys = NULL;  /* Let DPAA fill it internally */

        	bman->h_BmPool[i] = BM_POOL_Config(&bmPoolParam);
			if(bman->h_BmPool[i] ==NULL){
				DPAA_LOG(LOG_CRIT_ERROR, "BM_POOL_Config failure. Unable to intialize pool index %d",i);
				rc=-1;
				break;
			}
			if(i == 0) {
				/*Special code to set the first BPID as SGT pool (no longer used)*/
				BM_POOL_ConfigBpid(bman->h_BmPool[i], DPAA_BMAN_SGT_BPID);
				BM_POOL_ConfigStockpile(bman->h_BmPool[i], bmPoolParam.numOfBuffers/10, bmPoolParam.numOfBuffers/20);
#ifdef NO_TX_CONFIRMATION_FLSHBUF
			} else if(i == bman->numPoolParams-1) {
				/*Special code to set the last BPID as IGNORED pool*/
				BM_POOL_ConfigBpid(bman->h_BmPool[i], DPAA_BMAN_IGNORED_BPID);
				//BM_POOL_ConfigStockpile(bman->h_BmPool[i], 64, 32);
#endif
			} else if(bmPoolParam.numOfBuffers > 0 && (i & 1) != 0) {
				/* odd numbered bpid's are for sgt buffers, we need a stockpile of these for Tx scatter gather lists */
				DPAA_LOG(LOG_NORMAL, "BM_POOL_Init: pool %d stockpile %d-%d", i, bmPoolParam.numOfBuffers/20, bmPoolParam.numOfBuffers/10);
				BM_POOL_ConfigStockpile(bman->h_BmPool[i], bmPoolParam.numOfBuffers/10, bmPoolParam.numOfBuffers/20);
			}
			if(BM_POOL_Init(bman->h_BmPool[i])!=EOK){
				DPAA_LOG(LOG_CRIT_ERROR, "BM_POOL_Init failure. Unable to intialize pool index %d",i);
				rc=-1;
				break;
			}
			DPAA_LOG(LOG_DEBUG_CONFIG, "BM_POOL_Init success for pool index %d",i);
		}
		
		if(rc==EOK){
			bman->mpool_initialized = TRUE;
		}
	}
	
	if(rc==EOK){
		DPAA_LOG(LOG_CRIT_ERROR, "%s: Total buffer pools %d", __FUNCTION__, bman->numPoolParams);
		for(i=0;i<bman->numPoolParams;i++){
			resp->bpid[i]= BM_POOL_GetId(bman->h_BmPool[i]);
			resp->size[i]= (uint32_t)BM_POOL_GetBufferSize(bman->h_BmPool[i]);
			DPAA_LOG(LOG_CRIT_ERROR, "%s: pool %d bpid %d size %d", __FUNCTION__, i,resp->bpid[i],resp->size[i]);
		}
		/*Donot touch i*/
		resp->count= i;
		for( ;i<DPAA_BMAN_MAX_BPOOL;i++){
			resp->bpid[i]= 0;
			resp->size[i]= 0;
		}
	}
	
	return rc;
}
/*****************************************************************************/
/*  Function Name	: bman_attach_err_intr_cb 
 *	Input			: BMAN handle
 *					  Request Pointer
 *					  Response Pointer
 *	Description		: This function attaches the BMAN Error Interrupt handler   
 *					  Function, which BMAN would call before masking out an interrupt.
 *	Returns			: ZERO if success. 
 *	Note			: BMAN doesnt enables error interrupts before application supplying
 *					  a valid callback function, or a NULL callback (which tells BMAN
 *					  to use its own ISR handler)
 */
/*****************************************************************************/
int 
bman_attach_err_intr_cb(bman_handle_t bman_hdl,IN attach_callback_func_t *req, OUT detach_callback_func_t *resp){
	return 0;
}

/*****************************************************************************/
/*  Function Name	: bman_handle_err_interrupt
 *	Input			: 
 *	Description		: Calls the BM Portal API 
 *	Returns			: 
 *	Note			:				
 */
/*****************************************************************************/
void bman_handle_err_interrupt(bman_handle_t hdl, clear_irq_cause_t *req){

	bman_t *bman= (bman_t*)hdl;
	LOG_FUNCTION_ENTRY;
 	
	BM_ErrorIsr(bman->h_Bm);
	
	LOG_FUNCTION_EXIT;
		
}


int
bman_get_counter(bman_handle_t hdl, uint32_t cntr) {
	return BM_GetCounter(((bman_t *)hdl)->h_Bm, cntr);
}

int
bman_pool_get_counter(bman_handle_t hndl, uint32_t bpid, uint32_t cntr) {
	bman_t *bman = (bman_t *)hndl;
	int i;

	/* optimization: if BmPool indexes match allocated bpid then we can avoid loop */
	if(bpid < bman->numPoolParams && BM_POOL_GetId(bman->h_BmPool[bpid]) == bpid)
		return BM_POOL_GetCounter(bman->h_BmPool[bpid], cntr);

	for(i=0; i < bman->numPoolParams; i++)
	    if(BM_POOL_GetId(bman->h_BmPool[i]) == bpid)
			return BM_POOL_GetCounter(bman->h_BmPool[i], cntr);

	/* special case to allow dumping of non-depletion counts for uninitialized BPIDs */
	if(bpid < 64 && bman->h_BmPool[0])
		return BM_POOL_GetCounterX(bman->h_BmPool[0], cntr, bpid);

	return 0;
}
