/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "fman.h"



#define BUP_FM_HALT_SIG_ERRATA
#define FM_RAM_LIST_ERR_IRQ_ERRATA
uint32_t pcd_port=0;
uint32_t base_fq=1; 
uint32_t num_of_qid=8;
uint32_t fm_index=0;
uint32_t pcd_enable=0;
uint32_t one_call_only=1;

/* The FMan firmware */
static uint32_t       P4080_R1_0_FmCtrlCode[] = P4080_R1_0_UC_IMG;
static uint32_t       P4080_R2_0_FmCtrlCode[] = P4080_R2_0_UC_IMG;


static int fman_port_init(fm_interface_t *intf, config_port_t* req, config_port_resp_t* resp);
#ifndef DISABLE_UNUSED_CODE
static int fman_port_shutdown(fm_interface_t *intf); 
#endif

static int fm_reset_init(fm_t *);

int	
fman_init( dpaa_system_t *dpaa,const dpaa_startup_params_t *cfg){

	int32_t rc=EOK;
	int fm_index=0, mac_index;
	fman_t *fman= NULL;


	LOG_FUNCTION_ENTRY;
	
	if((fman = malloc(sizeof(fman_t)))== NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "Can not allocate memory to fman_t");
		fman=NULL;
		return ENOMEM;
	}
	memset(fman,0,sizeof(fman_t));

	for (fm_index=0;fm_index<INTG_MAX_NUM_OF_FM ;fm_index++){

		memset(&fman->fm[fm_index],0,sizeof(fm_t));
		fman->fm[fm_index].fm_index= fm_index;
		/*TODO: The Fman structures should be cleaned up. So many levels are not
		required now as we're using NetComm APIs.
		*/
		fman->fm[fm_index].parent = fman;
		
		if (cfg->fm_enabled[fm_index]==0){
			DPAA_LOG(LOG_DEBUG_CONFIG,"Skipping config of FMAN%d ",fm_index);
			/*Donot configure this FM*/
			continue;
		}
	
		if((rc=fm_reset_init(&fman->fm[fm_index]))!=EOK){
			DPAA_LOG(LOG_CRIT_ERROR,"FMAN%d can not be reset or Microcode can not be loaded",fm_index);
			rc=-1;
			break;
		}
		for(mac_index=0; mac_index < MAX_MACDEV_PER_FMAN ; mac_index++ ){

			/*Calculate the skip condition*/
			uint32_t no_enable=0;

			if( (cfg->fm0_1x4gig!=0 ||cfg->fm1_1x4gig!=0) &&(fm_index==0 && mac_index==0) ){
				/*FMO DSEC0 needs to be enabled because of MII access and PCD*/
				if(cfg->macdev_enable[fm_index][mac_index]==0 ){
					no_enable= 1;
				}
			}
			if( (cfg->fm0_10gig!=0 ||cfg->fm1_10gig!=0) && (fm_index==0 && mac_index==4) ){
				/*Currently we are using emu_phy for 10G. Once the real MDIO comes for 10G, we need to 
				 * enable FMO TGEC0 here in all cases.*/
				if(cfg->macdev_enable[fm_index][mac_index]==0 ){
					no_enable= 1;
				}
			}

			if(!no_enable && (cfg->macdev_enable[fm_index][mac_index] == 0) ) 
			{
				/* Do not configure this MAC */
				DPAA_LOG(LOG_DEBUG_CONFIG,"Skipping config of FMAN%d MAC Device %d",fm_index,mac_index);
				continue;
			}
			if((rc=fman_interface_init(&fman->fm[fm_index],dpaa->h_Platform,mac_index,cfg,no_enable))!=EOK){
				DPAA_LOG(LOG_CRIT_ERROR,"Failure Enabling FMAN%d MAC Device %d",fm_index,mac_index);
				rc=-1;
				/*Donot return error but continue to initialize MAC further*/
			}
			else{
				DPAA_LOG(LOG_DEBUG_CONFIG,"Successfully Enabled FMAN%d MAC Device %d",fm_index,mac_index);
			}
		}
	}
	if(rc==EOK){
		fman->parent=dpaa;
		INIT_FLAG(fman->flags);
		fman->state= 1;
		dpaa->fman= (fman_handle_t)fman;

	}
	else {

	}
	LOG_FUNCTION_EXIT;
	return rc;
}
static void FmPcdExceptionsCB(t_Handle h_FmDev, e_FmPcdExceptions exception)
{
		switch(exception)
		{
			case(e_FM_PCD_KG_EXCEPTION_DOUBLE_ECC):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_KG_EXCEPTION_DOUBLE_ECC");
				break;
			case(e_FM_PCD_PLCR_EXCEPTION_DOUBLE_ECC):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_PLCR_EXCEPTION_DOUBLE_ECC");
				break;
			case(e_FM_PCD_KG_EXCEPTION_KEYSIZE_OVERFLOW):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_KG_EXCEPTION_KEYSIZE_OVERFLOW");
				break;
			case(e_FM_PCD_PLCR_EXCEPTION_INIT_ENTRY_ERROR):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_PLCR_EXCEPTION_INIT_ENTRY_ERROR");
				break;
			case(e_FM_PCD_PLCR_EXCEPTION_PRAM_SELF_INIT_COMPLETE):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_PLCR_EXCEPTION_PRAM_SELF_INIT_COMPLETE");
				break;
			case(e_FM_PCD_PLCR_EXCEPTION_ATOMIC_ACTION_COMPLETE):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_PLCR_EXCEPTION_ATOMIC_ACTION_COMPLETE");
				break;
			case(e_FM_PCD_PRS_EXCEPTION_DOUBLE_ECC):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_PRS_EXCEPTION_DOUBLE_ECC");
				break;
			case(e_FM_PCD_PRS_EXCEPTION_SINGLE_ECC):
				DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception e_FM_PCD_PRS_EXCEPTION_SINGLE_ECC");
				break;
			default:
				break;
     	}
}

static void FmIndexedExceptionCB(t_Handle           h_FmDev,
                                 e_FmPcdExceptions  exception,
                                 uint16_t           index)
{

    DPAA_LOG(LOG_RUNTIME_FAULT, "FM-PCD got exception %d",exception);

    UNUSED(index);
}

//Callbacks, copied from init.c of NetComm Test Application
static void FmExceptionsCB(t_Handle h_FmDev, e_FmExceptions exception)
{
	fm_t * fm = (fm_t*) h_FmDev;
	ASSERT_COND(fm);
	switch(exception)
	{
		case(e_FM_EX_DMA_BUS_ERROR):
		DPAA_LOG(LOG_RUNTIME_FAULT, "FM%d got exception e_FM_EX_DMA_BUS_ERROR", fm->fm_index);
		break;
		case(e_FM_EX_DMA_READ_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_DMA_READ_ECC", fm->fm_index);
		break;
		case(e_FM_EX_DMA_SYSTEM_WRITE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_DMA_SYSTEM_WRITE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_DMA_FM_WRITE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_DMA_FM_WRITE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_FPM_STALL_ON_TASKS):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_FPM_STALL_ON_TASKS", fm->fm_index);
		break;
		case(e_FM_EX_FPM_SINGLE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_FPM_SINGLE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_FPM_DOUBLE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_FPM_DOUBLE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_QMI_SINGLE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_QMI_SINGLE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_QMI_DOUBLE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_QMI_DOUBLE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID", fm->fm_index);
		break;
		case(e_FM_EX_BMI_LIST_RAM_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_BMI_LIST_RAM_ECC", fm->fm_index);
		break;
		case(e_FM_EX_BMI_PIPELINE_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_BMI_PIPELINE_ECC", fm->fm_index);
		break;
		case(e_FM_EX_BMI_STATISTICS_RAM_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_BMI_STATISTICS_RAM_ECC", fm->fm_index);
		break;
		case(e_FM_EX_BMI_DISPATCH_RAM_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_BMI_DISPATCH_RAM_ECC", fm->fm_index);
		break;
		case(e_FM_EX_IRAM_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_IRAM_ECC", fm->fm_index);
		break;
		case(e_FM_EX_MURAM_ECC):
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got exception e_FM_EX_MURAM_ECC", fm->fm_index);
		break;
		default:
		DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got unknown exception %d", fm->fm_index, exception);
		break;
	}
	if(exception == e_FM_EX_FPM_SINGLE_ECC)
	{
		DPAA_LOG(LOG_WARNING, "FM%d DIsabled exception e_FM_EX_FPM_SINGLE_ECC", fm->fm_index);
		FM_SetException(fm->h_Fm, e_FM_EX_FPM_SINGLE_ECC, FALSE);
	}
	if(exception == e_FM_EX_FPM_DOUBLE_ECC)
	{
		DPAA_LOG(LOG_WARNING, "FM%d DIsabled exception e_FM_EX_FPM_DOUBLE_ECC", fm->fm_index);
		FM_SetException(fm->h_Fm, e_FM_EX_FPM_DOUBLE_ECC, FALSE);
	}
}

static void FmBusErrorCB(t_Handle       h_FmDev,
                         e_FmPortType   portType,
                         uint8_t        portId,
                         uint64_t       addr,
                         uint8_t        tnum,
                         uint16_t       liodn)
{
	fm_t * fm = (fm_t*) h_FmDev;
	ASSERT_COND(fm);

    /* do nothing */
    UNUSED(portType);
    DPAA_LOG(LOG_RUNTIME_FAULT,"FM%d got BUS error exception on port %d: addr 0x%08x, tnum %d, liodn %d",
            fm->fm_index, portId, (uint32_t)addr, tnum, liodn);
}


// Need to reset FM, and load Microcode
static int 
fm_reset_init(fm_t * fm){
	
	int rc=EOK, i;
	t_FmParams		fmParam;
	t_FmPcdParams   fmPcdParams;

	t_ChipRevInfo	chipInfo;
	 uint16_t		liodnPerPart[][FM_MAX_NUM_OF_PARTITIONS] =
        {{ 0,33,34,35,36,37,38,39,
          40,41,42,43,44,45,46,47,
          48,49,50,51,52,53,54,55,
          56,57,58,59, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0},
         { 0,65,66,67,68,69,70,71,
          72,73,74, 0, 0, 0, 0, 0,
          80,81,82,83,84,85,86,87,
          88,89,90,91, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0,
           0, 0, 0, 0, 0, 0, 0, 0},
         };
	  char img_info[256];

	LOG_FUNCTION_ENTRY;

	if ((fm->h_MuramDev = 
			FM_MURAM_ConfigAndInit(SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_MURAM, fm->fm_index),
					FM_MURAM_SIZE)) == NULL) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_MURAM_ConfigAndInit failed");
		return -1;
	}

	memset(&fmParam, 0, sizeof(fmParam));
	fmParam.guestId = NCSW_MASTER_ID;
	fmParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM, fm->fm_index);
	fmParam.fmId = fm->fm_index;
	fmParam.h_App = fm;
	fmParam.fmClkFreq = 600; // get this from RCW/processor regs (should be 600Mhz)
	fmParam.h_FmMuram = fm->h_MuramDev;
	fmParam.f_Exception = FmExceptionsCB; 
	fmParam.f_BusError   = FmBusErrorCB;
	fmParam.irq = NO_IRQ;
	fmParam.errIrq = NO_IRQ;
	SYS_GetChipRevInfo(&chipInfo);
	memcpy(fmParam.liodnBasePerPort, liodnPerPart[fm->fm_index], sizeof(liodnPerPart[fm->fm_index]));
	XX_Print("%s: FM%d Using fmClkFreq %dMhz",__FUNCTION__, fm->fm_index, fmParam.fmClkFreq);
	switch(chipInfo.revMajor)
	{
		case (1):
			fmParam.firmware.p_Code = P4080_R1_0_FmCtrlCode;
			fmParam.firmware.size = sizeof(P4080_R1_0_FmCtrlCode);
			strncpy(img_info, P4080_R1_0_UC_IMG_INFO, sizeof(img_info));
			break;
		case (2):
			fmParam.firmware.p_Code = P4080_R2_0_FmCtrlCode;
			fmParam.firmware.size = sizeof(P4080_R2_0_FmCtrlCode);
			strncpy(img_info, P4080_R2_0_UC_IMG_INFO, sizeof(img_info));
			break;
		default:
			rc = -1;	
			img_info[0] = '\0';
	}
	img_info[sizeof(img_info)-1] = '\0';
	/* strip trailing '\r', '\n', and ' ' */
	for(i=strlen(img_info); i > 0 && (img_info[i-1]=='\r' || img_info[i-1]=='\n' || img_info[i-1]==' '); i--)
	    img_info[i-1] = '\0';
	XX_Print("%s: Loading %s",__FUNCTION__, img_info);

	if(rc == EOK) {
		fm->h_Fm =  FM_Config(&fmParam);
		if (fm->h_Fm == NULL) {
			rc = ENOMEM;
			DPAA_LOG(LOG_CRIT_ERROR, "FM_Config failed with %08x", rc);
		}
	}

	FM_ConfigTotalFifoSize(fm->h_Fm, 110*KILOBYTE);
	FM_ConfigResetOnInit(fm->h_Fm, TRUE);
	FM_ConfigEnableCounters(fm->h_Fm);
#if !defined (__QNXNTO__)	
//	#ifdef BUP_FM_HALT_SIG_ERRATA
	/* Workaround for silicon! not relevant for simulator */
	FM_ConfigHaltOnExternalActivation(fm->h_Fm, FALSE);
	FM_ConfigHaltOnUnrecoverableEccError(fm->h_Fm, FALSE);
#endif /* BUP_FM_HALT_SIG_ERRATA */


	/* final init */
	if ((rc = FM_Init(fm->h_Fm)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_Init failed with %08x", rc);
		FM_Free(fm->h_Fm);
	} else {

		#ifdef FM_RAM_LIST_ERR_IRQ_ERRATA
		FM_SetException(fm->h_Fm, e_FM_EX_BMI_LIST_RAM_ECC, FALSE);
		FM_SetException(fm->h_Fm, e_FM_EX_MURAM_ECC, FALSE);
		FM_SetException(fm->h_Fm, e_FM_EX_IRAM_ECC, FALSE);
		#endif /* FM_RAM_LIST_ERR_IRQ_ERRATA */
	}

	/***************************/
	/* FM PCD Initialization   */
	/***************************/
	fm->kgActive = TRUE;
	fm->prsActive = TRUE;
	fm->activatePlcr = TRUE;
	fm->activateCc= TRUE;
	fm->useHc=FALSE;

	memset(&fmPcdParams, 0, sizeof(fmPcdParams));
	fmPcdParams.h_App = fm;
	//fmPcdParams.h_FmMuram = fm->h_MuramDev;
	fmPcdParams.prsSupport = fm->prsActive;
	fmPcdParams.kgSupport = fm->kgActive;
	if(fmPcdParams.kgSupport)
	    fmPcdParams.numOfSchemes = 5 * 2; /* 5 ports/FM, 2 schemes/port */
	fmPcdParams.plcrSupport = fm->activatePlcr;
	fmPcdParams.ccSupport = fm->activateCc;

	fmPcdParams.h_Fm = fm->h_Fm;
	fmPcdParams.f_Exception = FmPcdExceptionsCB;
	if((fm->kgActive) || (fm->activatePlcr))
			fmPcdParams.f_ExceptionId = FmIndexedExceptionCB;
	fmPcdParams.useHostCommand = FALSE; //fm->useHc;

	fm->h_FmPcd = FM_PCD_Config(&fmPcdParams);
	if(!fm->h_FmPcd)
			RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("FM PCD!"));

	if(fm->activatePlcr)
			FM_PCD_ConfigPlcrNumOfSharedProfiles(fm->h_FmPcd, 16);

	if((FM_PCD_Init(fm->h_FmPcd))!= E_OK)
			RETURN_ERROR(MAJOR,E_INVALID_HANDLE , NO_MSG);

	if(fm->activatePlcr)
			FM_PCD_SetPlcrStatistics(fm->h_FmPcd, TRUE);

	LOG_FUNCTION_EXIT;
	return rc;
}


int	
fman_runtime(fman_handle_t fman_hndl, dpaa_runtime_t *rtm){
	fman_t *fman= (fman_t*)fman_hndl;
	int rc=EOK;
	fm_interface_t *intf=NULL;
	switch(rtm->code){
		case DPAA_RUNTIME_CODE_SET_MAC_CONFIG:
		{	
			set_mac_config_t *config_mac_req=NULL;
		
			config_mac_req= &rtm->set_mac_config;
			if(config_mac_req->port_id>MAX_MACDEV_PER_FMAN-1) {
				intf= &fman->fm[1].interface[config_mac_req->port_id%MAX_MACDEV_PER_FMAN];
			}
			else {
				intf= &fman->fm[0].interface[config_mac_req->port_id%MAX_MACDEV_PER_FMAN];
			}
			rc=fman_mac_configure(intf,config_mac_req->speed,config_mac_req->duplex );
		}
		break;
		
		case DPAA_RUNTIME_CODE_GET_MAC_STATS:
		{
			get_mac_stats_t *get_stats=NULL;
			get_mac_stats_resp_t *get_stats_resp=NULL;
			get_stats= &rtm->get_mac_stats;
			get_stats_resp= 	&rtm->get_mac_stats_resp;
			t_FmMacStatistics fmMacStats;
			if(get_stats->port_id>MAX_MACDEV_PER_FMAN-1) {
				intf= &fman->fm[1].interface[get_stats->port_id%MAX_MACDEV_PER_FMAN];
			}
			else {
				intf= &fman->fm[0].interface[get_stats->port_id%MAX_MACDEV_PER_FMAN];
			}
	
			if ( (rc = FM_MAC_GetStatistics(intf->h_MacDev,&fmMacStats)) != E_OK ) {
			DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_GetStatistics failed %08x", rc);
			} else {
			   	/* There must be a better way of doing this. */
				get_stats_resp->stats.rpkt = fmMacStats.ifInPkts;
    			get_stats_resp->stats.rbyt = fmMacStats.ifInOctets;
			  //  get_stats_resp->stats.rfcs  
			    get_stats_resp->stats.rmca = fmMacStats.ifInMcastPkts;
			    get_stats_resp->stats.rbca = fmMacStats.ifInBcastPkts;
			    get_stats_resp->stats.raln = fmMacStats.eStatCRCAlignErrors;
			    //get_stats_resp->stats.rflr;
			    //get_stats_resp->stats.rcse;
			    get_stats_resp->stats.rund = fmMacStats.eStatUndersizePkts;
			    get_stats_resp->stats.rovr = fmMacStats.eStatOversizePkts;
			    get_stats_resp->stats.rfrg = fmMacStats.eStatsDropEvents;
			    get_stats_resp->stats.rjbr = fmMacStats.eStatJabbers;
			    get_stats_resp->stats.rdrp = 0;
			
    			get_stats_resp->stats.tpkt = fmMacStats.ifOutPkts;
			    get_stats_resp->stats.tbyt = fmMacStats.ifOutOctets;
			    get_stats_resp->stats.tmca = fmMacStats.ifOutMcastPkts;
			    get_stats_resp->stats.tbca = fmMacStats.ifOutBcastPkts;
			   // get_stats_resp->stats.tdfr;
			   // get_stats_resp->stats.tscl;
			   // get_stats_resp->stats.tmcl;
			   // get_stats_resp->stats.tlcl;
			    //get_stats_resp->stats.txcl;
			   // get_stats_resp->stats.tncl;
			   // get_stats_resp->stats.tjbr;
			}
		}
		break;
			
		case DPAA_RUNTIME_CODE_MII_READ:
		{
			mii_read_t *mii_read=NULL;
			mii_read_resp_t *mii_read_resp=NULL;
			mii_read= &rtm->mii_read;
			mii_read_resp= 	&rtm->mii_read_resp;
			if(mii_read->port_id>MAX_MACDEV_PER_FMAN-1) {
				intf= &fman->fm[1].interface[mii_read->port_id%MAX_MACDEV_PER_FMAN];
			}
			else {
				intf= &fman->fm[0].interface[mii_read->port_id%MAX_MACDEV_PER_FMAN];
			}

			if(intf) {
				if((rc = FM_MAC_MII_ReadPhyReg(intf->h_MacDev, mii_read->phy_add, mii_read->reg_add,&mii_read_resp->reg_value))!=EOK){
					DPAA_LOG(LOG_RUNTIME_FAULT, "FM_MAC_MII_ReadPhyReg Failure for PHY add %d, reg %d", mii_read->phy_add, mii_read->reg_add);
				}
				
			} else {
				DPAA_LOG(LOG_CRIT_ERROR, "Interface not found");
			}
		}
		break;
		case DPAA_RUNTIME_CODE_MII_WRITE:
		{
			mii_write_t *mii_write=NULL;
			mii_write= &rtm->mii_write;
			if(mii_write->port_id>MAX_MACDEV_PER_FMAN-1) {
				intf= &fman->fm[1].interface[mii_write->port_id%MAX_MACDEV_PER_FMAN];
			}
			else {
				intf= &fman->fm[0].interface[mii_write->port_id%MAX_MACDEV_PER_FMAN];
			}

			if(intf) {
				rc = FM_MAC_MII_WritePhyReg(intf->h_MacDev, mii_write->phy_add, mii_write->reg_add,mii_write->data);
			} else {
				DPAA_LOG(LOG_CRIT_ERROR, "Interface not found");
			}
		}
		break;
	default:
		break;
	}
	return rc;
}



int	
fman_shutdown(fman_handle_t fman_hndl){
	fman_t* fman=(fman_t*) fman_hndl;
	int fm_index;
	//TODO: Iterate through all the Ports and disable/Free them first.

	for (fm_index=0;fm_index<INTG_MAX_NUM_OF_FM ; fm_index++){
		if(fman->fm[fm_index].h_Fm) {
			FM_Free(fman->fm[fm_index].h_Fm);
			FM_MURAM_Free(fman->fm[fm_index].h_MuramDev);
		}
	}
	return 0;
}

int 
fman_config_port(fman_handle_t fman_hndl, config_port_t* req, OUT config_port_resp_t* resp){
	int rc=0;
	fm_interface_t *intf=NULL;
	fman_t* fman=(fman_t*) fman_hndl;

	LOG_FUNCTION_ENTRY;

	if(req->port_id>MAX_MACDEV_PER_FMAN-1) {
		intf= &fman->fm[1].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	else {
		intf= &fman->fm[0].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}

	if( (rc = fman_port_init(intf, req, resp)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "fman_port_init failed");
	}


	LOG_FUNCTION_EXIT;
	return rc;
}

int 
fman_enable_port(fman_handle_t fman_hndl, enable_port_t* req, OUT enable_port_resp_t* resp){
	int rc=0;
	fm_interface_t *intf=NULL;
	fman_t* fman=(fman_t*) fman_hndl;

	LOG_FUNCTION_ENTRY;

	if(req->port_id>MAX_MACDEV_PER_FMAN-1) {
		intf= &fman->fm[1].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	else {
		intf= &fman->fm[0].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}

	if( (rc = FM_MAC_Enable(intf->h_MacDev, e_COMM_MODE_RX_AND_TX)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "fman_port_enable failed");
	}

	LOG_FUNCTION_EXIT;
	return rc;
}

int 
fman_disable_port(fman_handle_t fman_hndl,IN disable_port_t* req, OUT disable_port_resp_t* resp){
	int rc=0;
	fm_interface_t *intf=NULL;
	fman_t* fman=(fman_t*) fman_hndl;

	LOG_FUNCTION_ENTRY;

	if(req->port_id>MAX_MACDEV_PER_FMAN-1) {
		intf= &fman->fm[1].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	else {
		intf= &fman->fm[0].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	
	if(intf->h_MacDev != NULL){
		rc=FM_MAC_Disable (intf->h_MacDev,e_COMM_MODE_RX_AND_TX);
	}

	LOG_FUNCTION_EXIT;
	return rc; 
	}

#ifndef DISABLE_UNUSED_CODE
static int fman_port_shutdown(fm_interface_t *intf) {
	int rc=0;
	LOG_FUNCTION_ENTRY;
	if(intf->rx.h_Dev != NULL) {
		FM_PORT_Disable(intf->rx.h_Dev);
		FM_PORT_Free(intf->rx.h_Dev);
		intf->rx.h_Dev = NULL;
	} else {
		DPAA_LOG(LOG_CRIT_ERROR, "FM Port handle is invalid (rx)");
		rc = -1;
	}

	if(intf->tx.h_Dev != NULL) {
		FM_PORT_Disable(intf->tx.h_Dev);
		FM_PORT_Free(intf->tx.h_Dev);
		intf->tx.h_Dev = NULL;
	} else {
		DPAA_LOG(LOG_CRIT_ERROR, "FM Port handle is invalid (tx)");
		rc = -1;
	}
	LOG_FUNCTION_EXIT;
	return rc;
}
#endif

int 
fman_set_promiscuous(fman_handle_t fman_hndl, set_promiscuous_t* req, OUT set_promiscuous_resp_t* resp){
	int rc=0;
	fm_interface_t *intf=NULL;
	fman_t* fman=(fman_t*) fman_hndl;
	bool set_unipromisc = FALSE;
	bool set_multipromisc = FALSE;

	LOG_FUNCTION_ENTRY;
	
	switch(req->promisc){
		case SET_PROMISCUOUS_ALL:
			set_unipromisc = TRUE;
			set_multipromisc = TRUE;
			break;
		case SET_PROMISCUOUS_MULTI:
			set_unipromisc = FALSE;
			set_multipromisc = TRUE;
			break;
		case SET_PROMISCUOUS_UNI:
			set_unipromisc = TRUE;
			set_multipromisc = FALSE;
			break;
		default:
			set_unipromisc = FALSE;
			set_multipromisc = FALSE;
	}
			
	if(req->port_id>MAX_MACDEV_PER_FMAN-1) {
		intf= &fman->fm[1].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	else {
		intf= &fman->fm[0].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}

	if( (rc = FM_MAC_SetPromiscuous(intf->h_MacDev, set_unipromisc, set_multipromisc)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "fmand FM_MAC_SetPromiscuous failed");
	}

	LOG_FUNCTION_EXIT;
	return rc;
}

int 
fman_add_hashmacaddr(fman_handle_t fman_hndl, hashmacaddr_t* req, OUT hashmacaddr_resp_t* resp){
	int rc=0;
	fm_interface_t *intf=NULL;
	fman_t* fman=(fman_t*) fman_hndl;

	LOG_FUNCTION_ENTRY;
	
	if(req->port_id>MAX_MACDEV_PER_FMAN-1) {
		intf= &fman->fm[1].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	else {
		intf= &fman->fm[0].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}

	if( (rc = FM_MAC_AddHashMacAddr(intf->h_MacDev, (t_EnetAddr *)req->ethaddr)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "fmand FM_MAC_AddHashMacAddr failed");
	}

	LOG_FUNCTION_EXIT;
	return rc;
}

int 
fman_del_hashmacaddr(fman_handle_t fman_hndl, hashmacaddr_t* req, OUT hashmacaddr_resp_t* resp){
	int rc=0;
	fm_interface_t *intf=NULL;
	fman_t* fman=(fman_t*) fman_hndl;

	LOG_FUNCTION_ENTRY;
	
	if(req->port_id>MAX_MACDEV_PER_FMAN-1) {
		intf= &fman->fm[1].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}
	else {
		intf= &fman->fm[0].interface[req->port_id%MAX_MACDEV_PER_FMAN];
	}

	if( (rc = FM_MAC_RemoveHashMacAddr(intf->h_MacDev, (t_EnetAddr *)req->ethaddr)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "fmand FM_MAC_RemoveHashMacAddr failed");
	}

	LOG_FUNCTION_EXIT;
	return rc;
}

static void config_buffer_prefix(t_FmPortBufferPrefixContent * content, uint32_t privateDataSize) {
	assert(content != NULL);
	memset(content, 0, sizeof(t_FmPortBufferPrefixContent));
	content->privDataSize = privateDataSize;
}


/* Callback for Port errors */

static void FM_PORT_ExceptionCB (t_Handle handle, uint32_t ex)
{
	fm_interface_t *intf = (fm_interface_t*) handle;
	fm_t * fm = NULL;

	assert(intf != NULL);
	fm = (fm_t*) intf->parent;
	assert(fm != NULL);	
	
	DPAA_LOG_CRIT_ERROR("FM%d PORT%d Exception %08x",fm->fm_index, intf->intf_id, ex);


}

/* 
Initialize a PORT. Both RX and TX Ports are created and initialized. This is a helper function for fman_enable_port

 */
static int fman_port_init(fm_interface_t *intf, config_port_t* req, config_port_resp_t* resp) {
	t_FmPortParams fmPortParam;	
	int rc = E_OK;
	fm_t * fm = (fm_t*) intf->parent;
	int mac_index = req->port_id % FM_MAX_NUM_OF_RX_PORTS; /*From 0 to 5*/
	int j;
	t_FmPortBufferPrefixContent bufferPrefixContent;
	//uint8_t	rxParts[][FM_MAX_NUM_OF_RX_PORTS] =
    //    						{{11,12,13,14,15},{16,17,18,19,20}};
	LOG_FUNCTION_ENTRY;
	assert(fm != NULL);
	assert(intf != NULL);
	assert(req != NULL);


	intf->pcdBaseQ= req->pcdBaseQ;
	intf->pcdNumOfQs= req->pcdNumOfQs;
	DPAA_LOG(LOG_DEBUG_CONFIG, "fman Interface initialized with Base FQID %d, Range %d",intf->pcdBaseQ,intf->pcdNumOfQs);
	/* The caller may have requested to have a private area before the packet data
		starts. Check for that
	*/
	config_buffer_prefix(&bufferPrefixContent, req->rx_pvt_data_size);

	//First check if the ports are already initialized, shouldnt happen
	if( (intf->rx.h_Dev != NULL)  || (intf->tx.h_Dev != NULL) ) {
		DPAA_LOG(LOG_CRIT_ERROR, "fman ports already initialized");
		rc = -1;
	} else { // Sanity Check Ok

		memset(&fmPortParam, 0, sizeof(fmPortParam));

		//First the RX Port
		if( mac_index < MAX_1GMAC_PER_FMAN) {
			// 1 G RX Port
			fmPortParam.portType = e_FM_PORT_TYPE_RX;	
			fmPortParam.portId = mac_index;
			fmPortParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_PORT_1GRx,
				mac_index + (fm->fm_index*FM_MAX_NUM_OF_1G_RX_PORTS));
		} else {
			// 10G RX Port
			fmPortParam.portType = e_FM_PORT_TYPE_RX_10G;	
			fmPortParam.portId = 0;
			fmPortParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_PORT_10GRx,
				fmPortParam.portId + (fm->fm_index*FM_MAX_NUM_OF_10G_RX_PORTS));
		}
		fmPortParam.h_Fm = fm->h_Fm;
		fmPortParam.f_Exception=  (t_FmPortExceptionCallback*)FM_PORT_ExceptionCB;
		fmPortParam.h_App = intf;

		fmPortParam.specificParams.rxParams.errFqid = req->rx_errqid;
		fmPortParam.specificParams.rxParams.dfltFqid = req->rx_defqid;
		//fmPortParam.specificParams.rxParams.rxPartitionId = rxParts[fm->fm_index][mac_index];
		fmPortParam.specificParams.rxParams.extBufPools.numOfPoolsUsed = req->rx_pool_count;

		// We get the id and the size from the actual BM Pool
		for(j=0;j<fmPortParam.specificParams.rxParams.extBufPools.numOfPoolsUsed;j++)
		{
			fmPortParam.specificParams.rxParams.extBufPools.extBufPool[j].id =  req->rx_bpid[j];
			fmPortParam.specificParams.rxParams.extBufPools.extBufPool[j].size = req->rx_buf_size[j];
			DPAA_LOG(LOG_WARNING, "FM_PORT_Config extBufPool%d: FM%d port_id %d bpid %d size %d:", j, fm->fm_index, req->port_id, req->rx_bpid[j], req->rx_buf_size[j]);
		}
		
		if( (intf->rx.h_Dev = FM_PORT_Config(&fmPortParam)) == NULL) {
			DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_Config failed for rx");
		} else if ( (rc=FM_PORT_ConfigBufferPrefixContent(intf->rx.h_Dev,&bufferPrefixContent)) != E_OK ) {  
			DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_ConfigBufferPrefixContent failed for rx (%08x)", rc);
		} else if ((rc=FM_PORT_ConfigRxL4ChecksumVerify(intf->rx.h_Dev, TRUE)) != E_OK ) {  
			DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_ConfigRxL4ChecksumVerify failed for rx (%08x)", rc);
		} else {
		
			intf->rx.type = fmPortParam.portType;
			// Now the TX Port
			if( mac_index < MAX_1GMAC_PER_FMAN) {
				// 1 G TX Port
				fmPortParam.portType = e_FM_PORT_TYPE_TX;	
				fmPortParam.portId = mac_index;
				fmPortParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_PORT_1GTx,
					fmPortParam.portId + (fm->fm_index*FM_MAX_NUM_OF_1G_TX_PORTS));
				fmPortParam.specificParams.nonRxParams.qmChannel= ((fm->fm_index==0)?CHID_FMAN0_TX_1GIG0:CHID_FMAN1_TX_1GIG0)+mac_index;
			} else {
				// 10G TX Port
				fmPortParam.portType = e_FM_PORT_TYPE_TX_10G;	
				fmPortParam.portId = 0;
				fmPortParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_PORT_10GTx,
					fmPortParam.portId + (fm->fm_index*FM_MAX_NUM_OF_10G_TX_PORTS));
				fmPortParam.specificParams.nonRxParams.qmChannel= (fm->fm_index==0)?CHID_FMAN0_TX_10_GIG :CHID_FMAN1_TX_10_GIG;
			}
			fmPortParam.h_Fm = fm->h_Fm;
			fmPortParam.f_Exception= (t_FmPortExceptionCallback*) FM_PORT_ExceptionCB;
			fmPortParam.h_App = intf;

			fmPortParam.specificParams.nonRxParams.errFqid = req->tx_errqid;
			fmPortParam.specificParams.nonRxParams.dfltFqid = req->tx_cfqid;

			if( (intf->tx.h_Dev = FM_PORT_Config(&fmPortParam)) == NULL) {
				DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_Config failed for tx");
				FM_PORT_Free(intf->rx.h_Dev); // Free the RX Port
				intf->rx.h_Dev = NULL;
#ifdef NO_TX_CONFIRMATION
			} else if ((rc=FM_PORT_ConfigDontReleaseTxBufToBM(intf->tx.h_Dev)) != E_OK ) {  
				DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_ConfigDontReleaseTxBufToBM failed for tx (%08x)", rc);
#endif
			} else { //Init
				// Ok, the RX and TX ports have been created, now Init them
				if( (rc = FM_PORT_Init(intf->rx.h_Dev)) != E_OK) {
					DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_init failed for rx (%08x)",rc);
				} else if( (rc = FM_PORT_Init(intf->tx.h_Dev)) != E_OK) {
					DPAA_LOG(LOG_CRIT_ERROR, "FM_PORT_init failed for tx (%08x)", rc);
				} else {
				// Do other initialization here
					intf->tx.type = fmPortParam.portType;
					/* Read back what the actual buffer prefix is */
					resp->rx_pvt_data_size = FM_PORT_GetBufferDataOffset(intf->rx.h_Dev);
					DPAA_LOG(LOG_DEBUG_CONFIG, "FM Buffer prefix size:%d",req->rx_pvt_data_size);

					FM_PORT_Enable(intf->rx.h_Dev);
					FM_PORT_Enable(intf->tx.h_Dev);
					DPAA_LOG(LOG_DEBUG_CONFIG, "Port Configuration done and OK FM:%d if:%d",fm->fm_index,mac_index);
				}
			} // Init
		} //TX Config
	} //Sanity Check ok

	LOG_FUNCTION_EXIT;
	return rc;
} // end function

//Callbacks
static void FM_MAC_ExceptionCB(t_Handle handle, e_FmMacExceptions exceptions) {
	fm_interface_t *intf = (fm_interface_t*) handle;
	fm_t * fm = NULL;

	assert(intf != NULL);
	fm = (fm_t*) intf->parent;
	assert(fm != NULL);	
	
	DPAA_LOG_CRIT_ERROR("FM%d MAC%d Exception %d",fm->fm_index, intf->intf_id, exceptions);

}


/*This function is responsible for initialzing an interface*/
int 
fman_interface_init(fm_t* fm, t_Handle h_Platform, uint32_t if_id,const dpaa_startup_params_t *cfg,uint32_t no_enable){
	int rc = EOK;
	fm_interface_t *intf=NULL;
	t_FmMacParams	fmMacParam;
	uint32_t fm_if= if_id ;
	
	LOG_FUNCTION_ENTRY;
	intf= &fm->interface[if_id];
	intf->parent= fm;
	intf->intf_id= if_id;

	memset(&fmMacParam, 0, sizeof(fmMacParam));
    fmMacParam.h_App           = intf;
    fmMacParam.h_Fm            = fm->h_Fm;

    fmMacParam.f_Exception     =  FM_MAC_ExceptionCB;
    fmMacParam.f_Event         = NULL; //FmMacEventsCB;
    memcpy(fmMacParam.addr,&cfg->mac_addr[fm->fm_index][fm_if][0],MAC_ADDR_SIZE);
	fmMacParam.mdioIrq = NO_IRQ;

	if(fm_if< MAX_MACDEV_PER_FMAN-1){
	// 1G Interface
		e_EnetInterface interfaceType;
		switch (cfg->phy_if[fm->fm_index][fm_if]) {
			case PHY_IF_MII: interfaceType = e_ENET_IF_MII; break;
			case PHY_IF_GMII: interfaceType = e_ENET_IF_GMII; break;
			case PHY_IF_SGMII: interfaceType = e_ENET_IF_SGMII; break;
			case PHY_IF_RGMII: interfaceType = e_ENET_IF_RGMII; break;
			case PHY_IF_XGMII: interfaceType = e_ENET_IF_XGMII; break;
			default:
				interfaceType = 0; //force error	

		}
		fmMacParam.enetMode =  MAKE_ENET_MODE(interfaceType, e_ENET_SPEED_1000);
		fmMacParam.macId    = fm_if;
		fmMacParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_PORT_1GMAC,
                                                                    fmMacParam.macId + (fm->fm_index*FM_MAX_NUM_OF_1G_MACS));
		assert(fmMacParam.baseAddr != NULL);
	}
	else{ /*The last interface in FMAN is 10Gig*/
		fmMacParam.enetMode = e_ENET_MODE_XGMII_10000;
		fmMacParam.macId    = 0;
		fmMacParam.baseAddr = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_FM_PORT_10GMAC,
                                                                    fmMacParam.macId + (fm->fm_index*FM_MAX_NUM_OF_10G_MACS));
		assert(fmMacParam.baseAddr != NULL);
	}
	intf->h_MacDev = FM_MAC_Config(&fmMacParam);
	if (intf->h_MacDev == NULL ) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_Config failed");
		rc = -1; 
		goto err_exit;
	} else {
		DPAA_LOG(LOG_DEBUG_CONFIG, "FM_MAC_Config complete for FM:%d interface:%d",fm->fm_index, fm_if);
	}
	
	 DPAA_LOG(LOG_WARNING, "FM_MAC_ConfigResetOnInit On for fm_index %d fm_if %d", fm->fm_index, fm_if);
	 if (FM_MAC_ConfigResetOnInit(intf->h_MacDev, TRUE) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_ConfigResetOnInit failed");
		rc = -1;
		goto err_exit;
	 }
#if 0
		if(FM_MAC_ConfigLengthCheck(intf->h_MacDev, FALSE) != E_OK ) {
			DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_ConfigLengthCheck failed");
			rc = -1;
			goto err_exit;
		}
#endif

#ifdef FM_10G_TX_ECC_FRMS_ERRATA_10GMAC_A004
	/* Disable workaround for now as it causes enable of second 10g to fail
    if ((fm_if >= FM_MAX_NUM_OF_1G_MACS) && no_enable &&
	*/
    if ((fm_if >= FM_MAX_NUM_OF_1G_MACS) &&
        FM_MAC_ConfigSkipFman11Workaround(intf->h_MacDev) != E_OK)
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, NO_MSG);
#endif /* FM_10G_TX_ECC_FRMS_ERRATA_10GMAC_A004 */

    if((cfg->jumbo_1g && fm_if < MAX_MACDEV_PER_FMAN-1) || (cfg->jumbo_10g && fm_if == MAX_MACDEV_PER_FMAN-1)){
		DPAA_LOG(LOG_WARNING, "Enabling FM %d, IF %d, with jumbo frames support", fm->fm_index, fm_if);
		if (FM_MAC_ConfigMaxFrameLength(intf->h_MacDev, 9600) != E_OK) {
			DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_ConfigMaxFrameLength failed for fm_index %d fm_if %d", fm->fm_index, fm_if);
			rc = -1;
			goto err_exit;
		}
	}

	DPAA_LOG(LOG_DEBUG_CONFIG, "Enabling FM %d, MAC %d",fm->fm_index,fmMacParam.macId);

	if(FM_MAC_Init(intf->h_MacDev) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_Init failed");
		rc = -1;
		goto err_exit;
	}

	if (  FM_MAC_SetStatistics(intf->h_MacDev, e_FM_MAC_FULL_STATISTICS) != E_OK ) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_SetStatistics failed");
		rc = -1;
		goto err_exit;
	}

	// Enable the Phys. First some hacks to get the platform_p4080.c code to work. Push the mac handle in. This us used later
	// by the SYS_InitEthernetPhy function. 
	//Warning!! The code in SYS_InitEthernetPhy assumes that dsec0 port 0 will be initialized first. This is used for all mdio
	//access.

	if ( fm_if == (MAX_MACDEV_PER_FMAN-1)) {
		t_Error err;


		SYS_ForceHandle(e_SYS_SUBMODULE_FM_PORT_10GMAC,
						fmMacParam.macId + (fm->fm_index *FM_MAX_NUM_OF_10G_MACS),intf->h_MacDev);
//#ifdef FM_SINGLE_MDIO_ERRATA_GEN8
		if ((fmMacParam.macId == 0) &&
				(fm->fm_index == 0) &&
				((err = PLATFORM_MdioForceMaster(h_Platform, e_SYS_SUBMODULE_FM_PORT_10GMAC, 0)) != E_OK))
			RETURN_ERROR(MAJOR, err, NO_MSG);
//#endif /* FM_SINGLE_MDIO_ERRATA_GEN8 */
		if (!no_enable) {
			if (SYS_InitEthernetPhy(e_SYS_SUBMODULE_FM_PORT_10GMAC,
							fmMacParam.macId + (fm->fm_index *FM_MAX_NUM_OF_10G_MACS),fmMacParam.enetMode,e_ENET_FULL_DUPLEX) != E_OK) {
				DPAA_LOG(LOG_CRIT_ERROR, "SYS_InitEthernetPhy failed (10G)");
			}
			else
				DPAA_LOG(LOG_DEBUG_CONFIG, "SYS_InitEthernetPhy succeded (10G) FM %d, MAC %d",fm->fm_index,fmMacParam.macId);
		} else {
				DPAA_LOG(LOG_DEBUG_CONFIG, "Skipping FM %d, MAC %d enabling",fm->fm_index,fmMacParam.macId);
		}

	} else {
		t_Error err;
		
		SYS_ForceHandle(e_SYS_SUBMODULE_FM_PORT_1GMAC,
						fmMacParam.macId + (fm->fm_index *FM_MAX_NUM_OF_1G_MACS),intf->h_MacDev);
//#ifdef FM_SINGLE_MDIO_ERRATA_GEN8
		if ((fmMacParam.macId == 0) &&
				(fm->fm_index == 0) &&
				((err = PLATFORM_MdioForceMaster(h_Platform, e_SYS_SUBMODULE_FM_PORT_1GMAC, 0)) != E_OK))
			RETURN_ERROR(MAJOR, err, NO_MSG);
//#endif /* FM_SINGLE_MDIO_ERRATA_GEN8 */
		if (!no_enable) {
			if (SYS_InitEthernetPhy(e_SYS_SUBMODULE_FM_PORT_1GMAC,
						fmMacParam.macId + (fm->fm_index *FM_MAX_NUM_OF_1G_MACS),fmMacParam.enetMode,e_ENET_FULL_DUPLEX) != E_OK) {
				DPAA_LOG(LOG_CRIT_ERROR, "SYS_InitEthernetPhy failed (1G)");
			}
			else
				DPAA_LOG(LOG_DEBUG_CONFIG, "SYS_InitEthernetPhy succeded (1G) FM %d, MAC %d",fm->fm_index,fmMacParam.macId);
		} else {
				DPAA_LOG(LOG_DEBUG_CONFIG, "Skipping FM %d, MAC %d enabling",fm->fm_index,fmMacParam.macId);
		}
	}

	
err_exit:
	// The port configuration is deferred until when it is actually required.
	LOG_FUNCTION_EXIT;
	return rc;
}

int 
fman_mac_configure(	fm_interface_t *intf,fm_mac_speed_t speed,fm_mac_duplex_t duplex ){
	e_EnetSpeed enetSpeed = e_ENET_SPEED_1000;
	bool fullDuplex = FALSE;
	int rc = E_OK;
	LOG_FUNCTION_ENTRY;
	switch(speed) {
	case MAC_SPEED_10GB: 	enetSpeed = e_ENET_SPEED_10000; break;
	case MAC_SPEED_1GB: 	enetSpeed = e_ENET_SPEED_1000; break;
	case MAC_SPEED_100MB:	enetSpeed = e_ENET_SPEED_100; break;
	case MAC_SPEED_10MB:	enetSpeed = e_ENET_SPEED_10; break;
	default:
		DPAA_LOG(LOG_CRIT_ERROR, "Invalid MAC_SPEED type %d",speed);
		LOG_FUNCTION_EXIT;
		return rc;
	}
	
	if (duplex == MAC_DUPLEX_FULL) {
		fullDuplex = TRUE;
	}

	if( (rc = FM_MAC_AdjustLink(intf->h_MacDev,enetSpeed, fullDuplex)) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "FM_MAC_AdjustLink failed %08x",rc);
	} else {
		DPAA_LOG_DEBUG_CONFIG("FM_MAC_AdjustLink OK");
	}
		LOG_FUNCTION_EXIT;
	return rc;
}


t_Error SetFmPcdFmc (fman_t* fman)
{
    t_Fmc   fmc;
	int fm_index=0;
	int port_index=0;

	memset(&fmc, 0, sizeof(fmc));
	
	for(fm_index=0;fm_index<INTG_MAX_NUM_OF_FM; fm_index++){

		fmc.fms[fm_index].h_Dev = fman->fm[fm_index].h_Fm ;
		fmc.fms[fm_index].h_PcdDev = fman->fm[fm_index].h_FmPcd;
		for(port_index=0;port_index<FM_MAX_NUM_OF_RX_PORTS;port_index++){
			fmc.fms[fm_index].rxPorts[port_index].h_Dev = fman->fm[fm_index].interface[port_index].rx.h_Dev;
		}
	}
	return fmc_config(&fmc);
}


int fman_dump_regs(fman_handle_t fman_hndl){
	int fm_index;
	int intf_index;
	fman_t* fman=(fman_t*) fman_hndl;

	for(fm_index=0; fm_index < INTG_MAX_NUM_OF_FM; fm_index++) {
		if(fman->fm[fm_index].h_Fm != NULL) {
			FM_DumpRegs(fman->fm[fm_index].h_Fm);

			/* Now all the ports for this FM */
			for(intf_index=0; intf_index<FM_MAX_NUM_OF_MACS; intf_index++) {
				DPAA_LOG_DEBUG_CONFIG(">>>------DUMP REGS FM:%d INTF:%d---------<<<",fm_index,intf_index);

				if(fman->fm[fm_index].interface[intf_index].h_MacDev != NULL) {
					DPAA_LOG_DEBUG_CONFIG(">>>>>------MAC REGS FM:%d INTF:%d-----<<<<<",fm_index,intf_index);
					FM_MAC_DumpRegs(fman->fm[fm_index].interface[intf_index].h_MacDev);
				}
				DPAA_LOG_DEBUG_CONFIG(">>>>>------PORT REGS FM:%d INTF:%d-----<<<<<",fm_index,intf_index);
				if(fman->fm[fm_index].interface[intf_index].rx.h_Dev != NULL) {
					FM_PORT_DumpRegs(fman->fm[fm_index].interface[intf_index].rx.h_Dev);
				}
				if(fman->fm[fm_index].interface[intf_index].tx.h_Dev != NULL) {
					FM_PORT_DumpRegs(fman->fm[fm_index].interface[intf_index].tx.h_Dev);
				}

			}
		}
	}
	return 0;
}

int fman_enable_pcd(fman_handle_t fman_hndl,IN enable_pcd_t* req, OUT enable_pcd_resp_t* resp){
	int rc=0;
	fman_t* fman=(fman_t*) fman_hndl;

	LOG_FUNCTION_ENTRY;

	pcd_port= req->pcd_port % (MAX_MACDEV_PER_FMAN);
	base_fq=req->base_fq; 
	num_of_qid=req->num_of_qid-1;
	fm_index= (req->pcd_port>(MAX_MACDEV_PER_FMAN-1)?1:0);
	pcd_enable = (req->fm_master != 0);
	if(req->fm_master == 1)
	   one_call_only = 1; /* cleared by fmc_config() if per port calls needed */
	else if(one_call_only)
	    return rc;
	DPAA_LOG_WARNING("%s: fm_master=%d,  pcd_port=%d, base_fq=%d,  num_of_qid=%d,  fm_index=%d,  pcd_enable=%d,  one_call_only=%d",__FUNCTION__,
		req->fm_master, pcd_port, base_fq, num_of_qid, fm_index, pcd_enable, one_call_only);

	if(SetFmPcdFmc(fman) != E_OK) {
		DPAA_LOG_WARNING("%s: SetFmPcdFmc FAILED", __FUNCTION__);
	}

	LOG_FUNCTION_EXIT;
	return rc; 

}

void fman_handle_event_irq(fman_handle_t h_fman, int fm_id){
 	fman_t *fman = (fman_t*) h_fman;
 	DPAA_ASSERT(fman != NULL);
 	DPAA_ASSERT(fm_id < INTG_MAX_NUM_OF_FM);
 	DPAA_ASSERT(fman->fm[fm_id].h_Fm != NULL);
 	FM_EventIsr(fman->fm[fm_id].h_Fm);
}
  
void fman_handle_error_irq(fman_handle_t h_fman, int fm_id) {
 	fman_t *fman = (fman_t*) h_fman;
 	DPAA_ASSERT(fm_id < INTG_MAX_NUM_OF_FM);
 	DPAA_ASSERT(fman->fm[fm_id].h_Fm != NULL);
 	FM_ErrorIsr(fman->fm[fm_id].h_Fm);
}

int
fman_get_counter(fman_handle_t h_fman, uint32_t fm_index, uint32_t cntr) {
	return FM_GetCounter(((fman_t *)h_fman)->fm[fm_index].h_Fm, cntr);
}

int
fman_port_get_counter(fman_handle_t h_fman, uint32_t index, uint32_t port, uint32_t cntr) {
	if(port==0)
		return FM_PORT_GetCounter(((fman_t *)h_fman)->fm[index/5].interface[index%5].rx.h_Dev, cntr);
	else
		return FM_PORT_GetCounter(((fman_t *)h_fman)->fm[index/5].interface[index%5].tx.h_Dev, cntr);
}

int
fman_pcd_get_counter(fman_handle_t h_fman, uint32_t index, uint32_t pcd, uint32_t cntr) {
	return FM_PCD_GetCounter(((fman_t *)h_fman)->fm[index/5].h_FmPcd, cntr);
}
