/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#ifndef NIC_fman_H_INCLUDED
#define NIC_fman_H_INCLUDED



#include "dpaa_priv.h"

/* NetcommSW Includes */
#include "Peripherals/fm_ext.h"
#include "Peripherals/fm_pcd_ext.h"
#include "integrations/P4080/part_integration_ext.h"
#include "fmc_config.h"
#include "Peripherals/fm_muram_ext.h"
#include "Peripherals/fm_mac_ext.h"
#include "Peripherals/fm_port_ext.h"
#include "Peripherals/bm_ext.h"
#include "fman_ctrl_code/p4080_r1.0.h"
#include "fman_ctrl_code/p4080_r2.0.h"


/*This is a Frame Manager port, which could be Offline/RX/TX port*/
typedef struct  {
    void  *parent;
	e_FmPortType type;
	t_Handle h_Dev; //NetComm Port handle
}fm_port_t;

/*This is an interface which consists of a FMAN RX Port+TX Port+DSEC/10GSEC*/
typedef struct {
    void * parent;

    uint32_t intf_id; // from 0 to 9
    uint32_t sub_portal; // from 0 to 9

	t_Handle h_MacDev; // Netcomm MAC Handle
	fm_port_t rx;
	fm_port_t tx;
	uint32_t pcdNumOfQs;
	uint32_t pcdBaseQ;
	uint32_t totalNumOfSchemes;
	uint32_t numOfSchemesUsed;
	uint8_t schemesBase;
	t_Handle h_DefNetEnv;
	t_Handle h_Schemes[FM_PCD_KG_NUM_OF_SCHEMES];
	t_Handle h_ClsPlanGrp;

} fm_interface_t;


typedef struct {
	void * parent;
	uint32_t flags;
	uint32_t fm_index;
	t_Handle h_Fm; // NetComm FM Handle 	
	t_Handle h_MuramDev;
	t_Handle h_FmPcd;

	uint32_t usedSchemes;
	fm_interface_t interface[FM_MAX_NUM_OF_MACS];
    /* Independent-Mode parameters(Not supported) */
    bool imode_enable;
	bool kgActive;
	bool prsActive;
	bool activatePlcr;
	bool activateCc;
	bool useHc;

}fm_t ;

typedef struct {
    void * parent;
    uint32_t flags;
    uint32_t state;
	fm_t fm[INTG_MAX_NUM_OF_FM];
}fman_t ;

int 
fman_interface_init(fm_t* fm, t_Handle h_Platform, uint32_t if_id,const dpaa_startup_params_t *cfg,uint32_t no_enable);
int 
fman_mac_configure(	fm_interface_t *intf,fm_mac_speed_t speed,fm_mac_duplex_t duplex );
t_Error fm_port_pcd(fm_interface_t *intf,enable_port_t* req);
t_Error SetFmPcdFmc (fman_t* fman);

#endif

