/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
#include <string.h>
#include "fmc_config.h"
#include "net_ext.h"
#include "types_ext.h"
#include "Peripherals/fm_pcd_ext.h"
#include "Peripherals/fm_port_ext.h"

/***************************************/
/*   Beginning of QNX declaration */
/* These externs variables are specifically included 
 * so that this configuration works with both 1G and 10G.
 * They can be safely removed and this file can be replaced 
 * with FMC tool output drop, provided all the values
 * given here matches the configuration.
 * A Mismatch in configuration would result in crash*/
extern uint32_t pcd_port; /*@QNX*/
extern uint32_t base_fq;  /*@QNX*/
extern uint32_t num_of_qid; /*@QNX*/
extern uint32_t fm_index; /*@QNX*/
extern uint32_t pcd_enable; /*@QNX*/
extern uint32_t one_call_only; /*@QNX*/
/*   END of QNX declaration          */
/***************************************/

t_Error fmc_config(t_Fmc *p_Fmc)
{
    t_Handle* engines [INTG_MAX_NUM_OF_FM];
    t_Handle* pcds    [INTG_MAX_NUM_OF_FM];
    t_Handle* ports   [INTG_MAX_NUM_OF_FM*FM_MAX_NUM_OF_RX_PORTS];
    t_Handle* env_ids [INTG_MAX_NUM_OF_FM*FM_MAX_NUM_OF_RX_PORTS];
    t_Handle* schemes [INTG_MAX_NUM_OF_FM*FM_MAX_NUM_OF_RX_PORTS*FM_PCD_KG_NUM_OF_SCHEMES];
    t_FmcPortDev* p_port_dev;

    t_FmPcdNetEnvParams       distinctionUnits;
    t_FmPcdKgSchemeParams     scheme;
    t_FmPortPcdParams         pcdParam;
    t_FmPortPcdPrsParams      prsParam;
    t_FmPortPcdKgParams       kgParam;

    one_call_only = 0; /*@QNX*/

    /* Open/Assign FM device: fm0 */
    engines[0] = &p_Fmc->fms[fm_index].h_Dev; /*@QNX*/

    /* Open FM PCD device: fm0 */
    pcds[0] = &p_Fmc->fms[fm_index].h_PcdDev; /*@QNX*/
    if(pcd_enable) /*@QNX*/
    FM_PCD_Enable( *pcds[0] );

    /* Open/Assign Port's device: e_FM_PORT_TYPE_RX #0 */
    ports[0] = &p_Fmc->fms[fm_index].rxPorts[pcd_port].h_Dev;   /*@QNX*/
    p_port_dev = &p_Fmc->fms[fm_index].rxPorts[pcd_port];       /*@QNX*/

    memset( &distinctionUnits, 0, sizeof( distinctionUnits ) );
    distinctionUnits.numOfDistinctionUnits = 2;
    distinctionUnits.units[0].hdrs[0].hdr = HEADER_TYPE_ETH;
    distinctionUnits.units[1].hdrs[0].hdr = HEADER_TYPE_IPv4;
    env_ids[0] = &p_port_dev->h_NetEnv;
    *env_ids[0] = FM_PCD_SetNetEnvCharacteristics( *pcds[0], &distinctionUnits );
    if ( 0 == *env_ids[0] ) {
        return E_INVALID_HANDLE;
    }

    /* Distribution: three_tuple_dist */
    memset( &scheme, 0, sizeof( scheme ) );
    scheme.modify = 0;
    scheme.id.relativeSchemeId =0+pcd_port*2; /*@QNX*/
    scheme.alwaysDirect = 0;
    scheme.netEnvParams.h_NetEnv = *env_ids[0];
    scheme.netEnvParams.numOfDistinctionUnits =2;
    scheme.netEnvParams.unitIds[0] =0;
    scheme.netEnvParams.unitIds[1] =1;
    scheme.useHash = 1;
    scheme.keyExtractAndHashParams.numOfUsedExtracts =1;
    /* Extract field:ipv4.src */
    scheme.keyExtractAndHashParams.extractArray[0].type = e_FM_PCD_EXTRACT_BY_HDR;
    scheme.keyExtractAndHashParams.extractArray[0].extractByHdr.hdr = HEADER_TYPE_IPv4;
    scheme.keyExtractAndHashParams.extractArray[0].extractByHdr.hdrIndex = e_FM_PCD_HDR_INDEX_NONE;
    scheme.keyExtractAndHashParams.extractArray[0].extractByHdr.ignoreProtocolValidation = 0;
    scheme.keyExtractAndHashParams.extractArray[0].extractByHdr.type = e_FM_PCD_EXTRACT_FULL_FIELD;
    scheme.keyExtractAndHashParams.extractArray[0].extractByHdr.extractByHdrType.fullField.ipv4 = NET_HEADER_FIELD_IPv4_SRC_IP;
    scheme.baseFqid =base_fq;            /*@QNX*/
    scheme.numOfUsedExtractedOrs =0;
    scheme.nextEngine =e_FM_PCD_DONE;
    scheme.schemeCounter.update = 1;
    scheme.schemeCounter.value = 0;
    scheme.keyExtractAndHashParams.hashDistributionNumOfFqids =num_of_qid;   /*@QNX*/
    scheme.keyExtractAndHashParams.numOfUsedMasks = 0;
    scheme.keyExtractAndHashParams.hashShift =0;
    schemes[1] = &p_port_dev->h_Schemes[1];
    *schemes[1] = FM_PCD_KgSetScheme( *pcds[0], &scheme );
    if ( 0 == *schemes[1] ) {
        return E_INVALID_HANDLE;
    }

    /* Distribution: default_dist */
    memset( &scheme, 0, sizeof( scheme ) );
    scheme.modify = 0;
    scheme.id.relativeSchemeId =1+pcd_port*2; /*@QNX*/
    scheme.alwaysDirect = 0;
    scheme.netEnvParams.h_NetEnv = *env_ids[0];
    scheme.netEnvParams.numOfDistinctionUnits =0;
    scheme.useHash = 0;
    scheme.keyExtractAndHashParams.numOfUsedExtracts =0;
    scheme.baseFqid =base_fq+num_of_qid;  /*@QNX*/
    scheme.numOfUsedExtractedOrs =0;
    scheme.nextEngine =e_FM_PCD_DONE;
    scheme.schemeCounter.update = 1;
    scheme.schemeCounter.value = 0;
    scheme.keyExtractAndHashParams.hashDistributionNumOfFqids =1;
    scheme.keyExtractAndHashParams.numOfUsedMasks = 0;
    scheme.keyExtractAndHashParams.hashShift =0;
    schemes[0] = &p_port_dev->h_Schemes[0];
    *schemes[0] = FM_PCD_KgSetScheme( *pcds[0], &scheme );
    if ( 0 == *schemes[0] ) {
        return E_INVALID_HANDLE;
    }

    /* Initialize PCD parameters */
    memset( &pcdParam, 0, sizeof( pcdParam ) );
    pcdParam.h_NetEnv = *env_ids[0];
    pcdParam.pcdSupport = e_FM_PORT_PCD_SUPPORT_PRS_AND_KG;
    pcdParam.p_PrsParams = &prsParam;
    pcdParam.p_KgParams = &kgParam;

    /* Initialize parser parameters */
    memset( &prsParam, 0, sizeof( prsParam ) );
    prsParam.parsingOffset = 0;
    prsParam.prsResultPrivateInfo =0;
    prsParam.firstPrsHdr = HEADER_TYPE_ETH;

    /* Initialize KeyGen parameters */
    memset( &kgParam, 0, sizeof( kgParam ) );
    kgParam.numOfSchemes =2;
    kgParam.h_Schemes[0] = *schemes[0];
    kgParam.h_Schemes[1] = *schemes[1];

    /* Apply the configuration */
    FM_PORT_SetPCD( *ports[0], &pcdParam );

    /* Close handles  */
    return E_OK;
}

