/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#ifndef NIC_dpaa_priv_H_INCLUDED
#define NIC_dpaa_priv_H_INCLUDED


#include <semaphore.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>

#include "dpaa_exp.h"
#include "types_qnx.h"

#include "platform_p4080_ds.h"
#include "services/data_pool_mng_ext.h"



#define DPAA_SEM_DEF_PERMISSION 0666
#define DPAA_SYSTEM_MASTER_SEM_NAME "/dpaa_master"
#define DPAA_SYSTEM_MASTER_SEM_INIT_VAL 1

#define DPAA_SYSTEM_PORTAL_SEM_NAME "/dpaa_portal_"
#define DPAA_SYSTEM_PORTAL_SEM_INIT_VAL 1
#define CORENET_TRANSACTION_SIZE 64


/************* Queue Manager specific Datastructures *************/
struct qman_t;
typedef struct qman_t* qman_handle_t;

/************* Frame Manager specific Datastructures *************/
struct fman_t;
typedef struct fman_t* fman_handle_t;

/************* Buffer Manager specific Datastructures *************/
struct bman_t;
typedef struct bman_t* bman_handle_t;


#define DPAA_MAX_NO_OF_BM_POOLS 4
/************* DPAA Master Datastructures *************/
typedef struct {
    
    bool                        imEn;
    bool                        masterMode;
    bman_handle_t              	bman;
    qman_handle_t               qman;
    fman_handle_t               fman;
    sem_t                       *master_sem;
    uint8_t                     dying;
    uint32_t                    verbose;
	t_Handle					h_Platform;

} dpaa_system_t;

uint64_t dpaa_vtophys (void *addr);
	
static inline uint32_t calc_size_exponent(uint32_t memory){
	uint32_t size=0;
	for(size=30;(size>0 ) && (!(memory & (2<<size))); size--);
	return size;
}

#define DPAA_CALC_SIZE_EXPONENT(x)  calc_size_exponent(x)
void * mmap_aligned( void * addr, size_t len, uint32_t alignment, int prot, int flags, int fildes,off_t off );

/*BMAN Exported functions*/
int bman_init(dpaa_system_t* , IN dpaa_startup_params_t* cfg);
int bman_portal_init(bman_handle_t , IN init_bman_portal_t *in, OUT init_bman_portal_resp_t*  );
int bman_bpool_init (bman_handle_t , IN init_bpool_t *req, OUT init_bpool_resp_t *resp );
int bman_buf_release(bman_handle_t , IN buf_release_t *, OUT buf_release_resp_t*  );
int bman_buf_acquire(bman_handle_t , IN buf_acquire_t *, OUT buf_acquire_resp_t*  );
int bman_bufs_flush(bman_handle_t , IN buf_acquire_t *, OUT buf_acquire_resp_t*  );
int bman_shutdown(bman_handle_t );
void bman_portal_def_cbfn(void *token,uint32_t* size);
int bman_attach_err_intr_cb(bman_handle_t ,IN attach_callback_func_t *, OUT detach_callback_func_t * );
void bman_handle_err_interrupt(bman_handle_t bman_hdl,clear_irq_cause_t *req);
int bman_get_counter(bman_handle_t h_bman, uint32_t cntr);
int bman_pool_get_counter(bman_handle_t h_bman, uint32_t bpool, uint32_t cntr);
void qman_handle_err_interrupt(qman_handle_t hdl, clear_irq_cause_t *req);



/*QMAN Exported functions*/
int qman_init(dpaa_system_t*, IN dpaa_startup_params_t* cfg);
int qman_portal_init(qman_handle_t, IN init_qman_portal_t *in, OUT init_qman_portal_resp_t*  );
void qman_portal_def_cbfn(void *token,uint32_t* size);
int qman_frame_enqueue(qman_handle_t, IN pkt_enqueue_t *req, OUT pkt_enqueue_resp_t* resp );
int qman_frame_dequeue(qman_handle_t, IN pkt_dequeue_t *req, OUT pkt_dequeue_resp_t* resp );
int qman_clear_irq(qman_handle_t,uint32_t portal);
int qman_portal_init_fqid(qman_handle_t hdl, init_fq_t *req, init_fq_resp_t *resp);
uint32_t qman_get_free_fqid(qman_handle_t hdl);
int qman_get_counter(qman_handle_t hdl, uint32_t cntr);
int qman_query_fq_p(qman_handle_t hdl, uint32_t index, uint32_t fqid, uint32_t nbuf, void *buf);
int qman_query_fq_np(qman_handle_t hdl, uint32_t index, uint32_t fqid, uint32_t nbuf, void *buf);
int qman_query_wq(qman_handle_t hdl, uint32_t index, uint16_t wqid, uint32_t nbuf, void *buf);

/*FMAN Exported function*/
int fman_init(dpaa_system_t * , IN dpaa_startup_params_t* cfg);
int fman_runtime(fman_handle_t fman_hndl, dpaa_runtime_t *rtm);
int fman_config_port( fman_handle_t, config_port_t*, OUT config_port_resp_t*);
int fman_enable_port( fman_handle_t, enable_port_t*, OUT enable_port_resp_t*);
int fman_disable_port(fman_handle_t,IN disable_port_t*, OUT disable_port_resp_t*);
int fman_enable_pcd(fman_handle_t,IN enable_pcd_t*, OUT enable_pcd_resp_t*);
void fman_handle_event_irq(fman_handle_t, int fm_id);
void fman_handle_error_irq(fman_handle_t, int fm_id);
int fman_dump_regs(fman_handle_t fman_hndl);
int fman_get_counter(fman_handle_t hdl, uint32_t fm_index, uint32_t cntr);
int fman_port_get_counter(fman_handle_t h_fman, uint32_t index, uint32_t port, uint32_t cntr);
int fman_pcd_get_counter(fman_handle_t h_fman, uint32_t index, uint32_t pcd, uint32_t cntr);
int fman_set_promiscuous(fman_handle_t fman_hndl, set_promiscuous_t* req, OUT set_promiscuous_resp_t* resp);
int fman_add_hashmacaddr(fman_handle_t fman_hndl, hashmacaddr_t* req, OUT hashmacaddr_resp_t* resp);
int fman_del_hashmacaddr(fman_handle_t fman_hndl, hashmacaddr_t* req, OUT hashmacaddr_resp_t* resp);

#define LOG_LEVEL_MIN _SLOG_CRITICAL
#define LOG_SYS_FAILURE _SLOG_CRITICAL       /*Always ON             LOG_ERR*/
#define LOG_CRIT_ERROR _SLOG_CRITICAL		/*Always ON             LOG_ERR*/
#define LOG_RUNTIME_FAULT _SLOG_ERROR     /*Always ON             LOG_ERR*/
#define LOG_WARNING _SLOG_WARNING           /*Always ON             LOG_WARNING*/
#define LOG_NORMAL  _SLOG_NOTICE           /*Always ON             LOG_WARNING*/
#define LOG_DEBUG_CONFIG _SLOG_INFO      /*Always ON             LOG_WARNING*/
#define LOG_DEBUG_FUNC_CALL _SLOG_DEBUG1   /*Always ON             LOG_ERR*/
#define LOG_DEBUG_DATA_FLOW _SLOG_DEBUG1
#define LOG_DEBUG_PKT_CONTENT _SLOG_DEBUG2
#define LOG_LEVEL_MAX _SLOG_DEBUG2

extern uint32_t dpaa_log_level;
/*DPAA log control function*/

#define DPAA_LOG_SYS_FAILURE(fmt,...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_SYS_FAILURE, fmt, ##__VA_ARGS__)

#define DPAA_LOG_CRIT_ERROR(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_CRIT_ERROR, fmt, ##__VA_ARGS__)

#define DPAA_LOG_RUNTIME_FAULT(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_RUNTIME_FAULT, fmt, ##__VA_ARGS__)

#define DPAA_LOG_WARNING(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_WARNING, fmt, ##__VA_ARGS__)

#define DPAA_LOG_NORMAL(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_NORMAL, fmt, ##__VA_ARGS__)

#define DPAA_LOG_DEBUG_CONFIG(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_DEBUG_CONFIG, fmt, ##__VA_ARGS__)

#define DPAA_LOG_DEBUG_FUNC_CALL(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_DEBUG_FUNC_CALL, fmt, ##__VA_ARGS__)

#define DPAA_LOG_DEBUG_DATA_FLOW(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_DEBUG_DATA_FLOW, fmt, ##__VA_ARGS__)

#define DPAA_LOG_DEBUG_PKT_CONTENT(fmt, ...) \
slogf(_SLOG_SETCODE(_SLOGC_NETWORK,10), LOG_DEBUG_PKT_CONTENT, fmt, ##__VA_ARGS__)

#define DPAA_LOG_IF(level)\
if(dpaa_log_level>=level) 

#define DPAA_LOG(level, fmt, ...) \
	do{		\
		DPAA_LOG_IF(level) \
		DPAA_##level(fmt,##__VA_ARGS__);\
	}while(0)
	

extern uint32_t func_depth_count;

#define MAX_FUNC_DEPTH 8
static const char func_entry_str[MAX_FUNC_DEPTH][15]= {
	">",
	"->",
	"-->",
	"--->",
	"---->",
	"----->",
	"------>",
	"------->",
};
#if 0
static const char* func_exit_str[MAX_FUNC_DEPTH]= {
	"<",
	"-<",
	"--<",
	"---<",
	"----<",
	"-----<",
	"------<",
	"-------<",
};
#endif
static const char func_exit_str[MAX_FUNC_DEPTH][15]= {
	"<",
	"<-",
	"<--",
	"<---",
	"<----",
	"<-----",
	"<------",
	"<-------",
};

#define LOG_FUNCTION_ENTRY	\
	func_depth_count++; \
	DPAA_LOG(LOG_DEBUG_FUNC_CALL, "%s%s():Entry",func_entry_str[func_depth_count% MAX_FUNC_DEPTH],__FUNCTION__)

#define LOG_FUNCTION_EXIT	\
	DPAA_LOG(LOG_DEBUG_FUNC_CALL, "%s%s():Exit",func_exit_str[func_depth_count% MAX_FUNC_DEPTH],__FUNCTION__);\
	func_depth_count-- 

#define DPAA_ASSERT(x) do { if(!(x)){ DPAA_LOG(LOG_CRIT_ERROR,"ASSERT(%s) Failure, FUNC %s LINE %d", #x, __FUNCTION__,__LINE__ ); }\
	assert(x);\
	}while(0)

#endif
