/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#ifndef NIC_dpaa_exp_H_INCLUDED
#define NIC_dpaa_exp_H_INCLUDED

// Freescale P4080 DPAA
#define MAC_ADDR_SIZE 6

//#include <io-pkt/iopkt_driver.h>
#include <stdint.h>
#include <stdlib.h>
//#include <sys/syslog.h>
//#include <sys/slogcodes.h>
#include <ioctl.h>

#define DPAA_MAX_MACDEV 10
#define DPAA_MAX_WQ_PER_CHANNEL 8
#define DPAA_MAX_FQ_PER_WQ 8


#define MAX_FMAN_PER_DPAA 2
#define MAX_1GMAC_PER_FMAN 4
#define MAX_10GSEC_PER_FMAN 1
#define MAX_MACDEV_PER_FMAN (MAX_1GMAC_PER_FMAN+MAX_10GSEC_PER_FMAN) 


/*Global Flag lists*/
/**************************************************************/
/*These are bit-flip flags default value of flag is 0*/

#define SET_BIT(pos) (1<<pos)
#define CLEAR_BIT(pos) ~(1<<pos)

#define FLAG_ALLOW_EXT_ISR     0
#define FLAG_DQRR_PUSH_MODE   1
#define FLAG_PORTAL_LOCK 2

#define SET_FLAG(value, flags) \
	value |= SET_BIT(flags)

#define INIT_FLAG(value) \
	value = 0

#define CLEAR_FLAG(value, flags) \
	value &= CLEAR_BIT(flags)

#define TEST_FLAG(value, flags) \
	(value & SET_BIT(flags))


/*Portal identifiers*/
/**************************************************************/
#define DPAA_MAX_SW_PORTAL 10
#define DPAA_SW_PORTAL_0   (0x1  << 0 )
#define DPAA_SW_PORTAL_1   (0x1  << 1 )
#define DPAA_SW_PORTAL_2   (0x1  << 2 )
#define DPAA_SW_PORTAL_3   (0x1  << 3 )
#define DPAA_SW_PORTAL_4   (0x1  << 4 )
#define DPAA_SW_PORTAL_5   (0x1  << 5 )
#define DPAA_SW_PORTAL_6   (0x1  << 6 )
#define DPAA_SW_PORTAL_7   (0x1  << 7 )
#define DPAA_SW_PORTAL_8   (0x1  << 8 )
#define DPAA_SW_PORTAL_9   (0x1  << 9 )
#define DPAA_SW_PORTAL_ALL  ((0x1  << DPAA_MAX_SW_PORTAL) -1 )

/*A portal can be affine to only one CPU, so no bitmap in these defines*/
typedef enum {
	DPAA_PORTAL_NOT_AFFINE=0,
	DPAA_PORTAL_CPU0_AFFINE= 1,
	DPAA_PORTAL_CPU1_AFFINE= 2,
	DPAA_PORTAL_CPU2_AFFINE= 3,
	DPAA_PORTAL_CPU3_AFFINE= 4,
	DPAA_PORTAL_CPU4_AFFINE= 5,
	DPAA_PORTAL_CPU5_AFFINE= 6,
	DPAA_PORTAL_CPU6_AFFINE= 7,
	DPAA_PORTAL_CPU7_AFFINE=8
} dpaa_cpu_affinity_t ;

#define PORTAL_LOCK(flag, x) 	if(TEST_FLAG(flag,FLAG_PORTAL_LOCK)){ \
	pthread_mutex_lock(x); \
}

#define PORTAL_UNLOCK(flag, x) 	if(TEST_FLAG(flag,FLAG_PORTAL_LOCK)){ \
	pthread_mutex_unlock(x); \
}

/**************************************************************/
/*QMAN specific*/

/*Pool channel Information, to be shareable between DPAA and user
 * These are the real Numbers, they are not used as it is in Channel MAP*/
#define POOL_CHNUMBER_OFFSET 0x21
#define CHID_POOLCH_1 (POOL_CHNUMBER_OFFSET+0)
#define CHID_POOLCH_2 (POOL_CHNUMBER_OFFSET+1)
#define CHID_POOLCH_3 (POOL_CHNUMBER_OFFSET+2)
#define CHID_POOLCH_4 (POOL_CHNUMBER_OFFSET+3)
#define CHID_POOLCH_5 (POOL_CHNUMBER_OFFSET+4)
#define CHID_POOLCH_6 (POOL_CHNUMBER_OFFSET+5)
#define CHID_POOLCH_7 (POOL_CHNUMBER_OFFSET+6)
#define CHID_POOLCH_8 (POOL_CHNUMBER_OFFSET+7)
#define CHID_POOLCH_9 (POOL_CHNUMBER_OFFSET+8)
#define CHID_POOLCH_10 (POOL_CHNUMBER_OFFSET+9)
#define CHID_POOLCH_11 (POOL_CHNUMBER_OFFSET+10)
#define CHID_POOLCH_12 (POOL_CHNUMBER_OFFSET+11)
#define CHID_POOLCH_13 (POOL_CHNUMBER_OFFSET+12)
#define CHID_POOLCH_14 (POOL_CHNUMBER_OFFSET+13)
#define CHID_POOLCH_15 (POOL_CHNUMBER_OFFSET+14)

#define MAX_NUM_POOL_CHANNEL 15

#define FMAN0_TX_CHANNEL_OFFSET 0x40
#define CHID_FMAN0_TX_10_GIG (FMAN0_TX_CHANNEL_OFFSET+0) /*FMAN1 SP0*/
#define CHID_FMAN0_TX_1GIG0 (FMAN0_TX_CHANNEL_OFFSET+1)  /*FMAN1 SP1*/
#define CHID_FMAN0_TX_1GIG1 (FMAN0_TX_CHANNEL_OFFSET+2)  /*FMAN1 SP2*/
#define CHID_FMAN0_TX_1GIG2 (FMAN0_TX_CHANNEL_OFFSET+3)  /*FMAN1 SP3*/
#define CHID_FMAN0_TX_1GIG3 (FMAN0_TX_CHANNEL_OFFSET+4)  /*FMAN1 SP4*/


#define FMAN1_TX_CHANNEL_OFFSET 0x60
#define CHID_FMAN1_TX_10_GIG (FMAN1_TX_CHANNEL_OFFSET+0) /*FMAN2 SP0*/
#define CHID_FMAN1_TX_1GIG0 (FMAN1_TX_CHANNEL_OFFSET+1) /*FMAN2 SP1*/
#define CHID_FMAN1_TX_1GIG1 (FMAN1_TX_CHANNEL_OFFSET+2) /*FMAN2 SP2*/
#define CHID_FMAN1_TX_1GIG2 (FMAN1_TX_CHANNEL_OFFSET+3) /*FMAN2 SP3*/
#define CHID_FMAN1_TX_1GIG3 (FMAN1_TX_CHANNEL_OFFSET+4) /*FMAN2 SP4*/


/*Since SW portal starts from 0 in DPAA, it matches exactly with channel ID*/
#define MAP_PORTAL_TO_CHID(portal_id) (portal_id)

/*Channel MAP macros. They are used to create a channel MAP and convey 
 * channel subscription information to DPAA library.*/


static inline uint32_t __dpaa_chmap_add(uint32_t map, uint32_t chid){
	/*Check if it is a dedicated channel ID*/
	if(chid<DPAA_MAX_SW_PORTAL)
	map|= 1<<15; /*Set Bit 16*/
	
	else if ((chid>= CHID_POOLCH_1) && (chid<= CHID_POOLCH_15))
	map|= 1<<(chid-POOL_CHNUMBER_OFFSET); /*Set Bit corresponding to pool channel*/

	else {
	//log(LOG_ERR,"%s():Invalid channel ID 0x%x supplied",__FUNCTION__,chid);
	}

	return map;
}

/*Add a channel ID to the map. Use bitwise OR to add multiple channel*/
#define CHMAP_ADD_CHID(map,chid)  __dpaa_chmap_add(map,chid) 
/*Macro to test whether a given bitmap contains the channel ID or not*/
#define CHMAP_TEST_POOL_CHID_PRESENT(map, x)   (map & CHMAP_ADD_CHID(0,x))
#define CHMAP_TEST_DED_CHID_PRESENT(map, x)   (map & CHMAP_ADD_CHID(0,1))


/*Default and Invalid values of Channel ID. */
#define CHANNELID_DEFAULT 255   /*Note: 0 is a valid channel ID. Dont use it as a default.*/
#define CHANNELID_INVALID CHANNELID_DEFAULT

#define WQID_DEFAULT 0
#define WQID_INVALID 8

#define FQID_DEFAULT 0 
#define FQID_INVALID 1<<24 /*FQID is 24 bit value*/

/**************************************************************/


/**************************************************************/
/*BMAN specific*/


#define DPAA_BMAN_PRIVATE_OFFSET 64    /*This is the private offset reserved before each of the buffer.*/

#define DPAA_BMAN_MAX_BPOOL 64    /*As there are only 64 pool*/
#define DPAA_BMAN_MAX_BUF_PER_CMD 8    /*each acquire/release command can operate upto 8 buffers at max*/

#define DPAA_BMAN_SGT_BPID 0    
#define DPAA_BMAN_DEFAULT_BPID 1    
#define DPAA_BMAN_IGNORED_BPID 63    /*We never create this BPID so buffers with this BPID are never freed to BM hardware*/
#define DPAA_BMAN_INVALID_BPID 64    /*As there are only 64 pools*/

/*Currently enabling all interrupts in BMAN CCSR*/
#define BMAN_CCSR_INTR_BSCN  0x1
#define BMAN_CCSR_INTR_SBEI  0x2
#define BMAN_CCSR_INTR_MBEI  0x4
#define BMAN_CCSR_INTR_FLWI  0x8
#define BMAN_CCSR_INTR_IVCI  0x10

#define BMAN_CCSR_ENABLE_INTR_ALL \
		(BMAN_CCSR_INTR_BSCN| \
		BMAN_CCSR_INTR_SBEI|  \
		BMAN_CCSR_INTR_MBEI|  \
		BMAN_CCSR_INTR_FLWI|  \
		BMAN_CCSR_INTR_IVCI)

/*Currently enabling all interrupts in BMAN SWPORTAL*/
#define BMAN_SW_PORTAL_INTR_BSCN 0x1
#define BMAN_SW_PORTAL_INTR_RCRI 0x2

#define BMAN_SW_PORTAL_ENABLE_INTR_ALL \
		(BMAN_SW_PORTAL_INTR_BSCN |\
		BMAN_SW_PORTAL_INTR_RCRI )


/**************************************************************/
/*FMAN specific*/
#define DPAA_MAX_BPID_PER_TX_PORT 1
#define DPAA_MAX_BPID_PER_RX_PORT 0x8

#ifndef __packed
#define __packed __attribute__ ((packed)) 
#endif


/*This is our handle of DPAA system. This is a singleton instance and
 * library doesnt allow to create more than one instance of this per process.*/
struct dpaa_system_t;
typedef struct dpaa_system_t* dpaa_handle_t;
typedef enum 
{
    PHY_IF_NOPHY,
    PHY_IF_MII,
    PHY_IF_GMII,
    PHY_IF_SGMII,
    PHY_IF_RGMII,
    PHY_IF_XGMII

}fm_phy_interface_t;


typedef struct {
    uint64_t        immr;
    uint32_t        verbose;
    uint32_t        portal_ownership;
    
	uint64_t		pixis_base;
	uint32_t		pixis_size;
    uint64_t        bman_portal_base;
    uint32_t        bman_portal_size;
    uint64_t        qman_portal_base;
    uint32_t        qman_portal_size;
    /*memory pool*/
    uint32_t        txbuf_memsize;
    uint32_t        rxbuf_memsize;
    int            loopback;
    uint32_t        emu_phy;

    /*Buffer Manager specific*/
    uint32_t        fbpr_memsize;

    /*Queue Manager specific*/
    uint32_t        fqdr_memsize;
    uint32_t        pfdr_memsize;
	uint32_t		fqid_base;
	uint32_t		total_fqid;


    /*Frame Manager specific*/
	uint32_t 		fm_enabled[MAX_FMAN_PER_DPAA];
    uint8_t         macdev_enable[MAX_FMAN_PER_DPAA][MAX_MACDEV_PER_FMAN];
    uint8_t         mac_addr[MAX_FMAN_PER_DPAA][MAX_MACDEV_PER_FMAN][MAC_ADDR_SIZE];
	uint32_t 		phy_addr[MAX_FMAN_PER_DPAA][MAX_MACDEV_PER_FMAN];
	uint32_t 		tbi_addr[MAX_FMAN_PER_DPAA][MAX_MACDEV_PER_FMAN];

    fm_phy_interface_t  phy_if[MAX_FMAN_PER_DPAA][MAX_MACDEV_PER_FMAN];

    //nic_config_t    nic_cfg;
    int            imode;
    uint32_t        master_if;
    uint32_t        secondary_if;
	int 			fm0_1x4gig ;
	int 			fm0_10gig ;
	int 			fm1_1x4gig;
	int 			fm1_10gig ;
	int 			no_phy ;
	int				dump_regs; /* Dump hardware registers on each stat call */
	int 			rx_loop ;
	int				jumbo_1g;
	int				jumbo_10g;

}dpaa_startup_params_t;


/*Structure denoting a frame descriptor. This is a common structure shared between DPAA
 * library and user process, thats why it needs to be defined here.*/
typedef struct {
    uint8_t dd:2;   
    uint8_t pid:6;  
    uint8_t bpid;   
    uint16_t addr_high; 
    uint32_t addr_low;  
    
    union {
    /*Only one type of union supported this time*/
        uint32_t opaque;
        struct {
            uint8_t format:3;
            uint16_t offset:9;
            uint32_t length20:20;
        } __packed;
    } __packed;

    union {
        uint32_t cmd;
        uint32_t status;
    };
} __packed fd_t;

typedef void (*fq_handler_funcptr_t)(void* token, uint32_t fqid, fd_t *fd_ptr);


typedef struct{
   /*  RMON MIB REGISTERS  */
    /*  TRANSMIT and RECEIVE COUNTERS   */

    volatile uint32_t  tr64;            /* 0x200 transmit and receive 64 byte frame counter */
    volatile uint32_t  tr127;           /* 0x204 transmit and receive 65 to 127 byte frame counter */
    volatile uint32_t  tr255;           /* 0x208 transmit and receive 128 to 255 byte frame counter */
    volatile uint32_t  tr511;           /* 0x20C transmit and receive 256 to 511 byte frame counter */
    volatile uint32_t  tr1k;            /* 0x210 transmit and receive 512 to 1023 byte frame counter */
    volatile uint32_t  trmax;           /* 0x214 transmit and receive 1024 to 1518 byte frame counter */
    volatile uint32_t  trmgv;           /* 0x218 transmit and receive 1519 to 1522 byte good VLAN frame count */

    /* RECEIVE COUNTERS */
    volatile uint32_t  rbyt;            /* 0x21C receive byte counter */
    volatile uint32_t  rpkt;            /* 0x220 receive packet counter */
    volatile uint32_t  rfcs;            /* 0x224 receive FCS error counter */
    volatile uint32_t  rmca;            /* 0x228 RMCA receive multicast packet counter */
    volatile uint32_t  rbca;            /* 0x22C receive broadcast packet counter */
    volatile uint32_t  rxcf;            /* 0x230 receive control frame packet counter */
    volatile uint32_t  rxpf;            /* 0x234 receive PAUSE frame packet counter */
    volatile uint32_t  rxuo;            /* 0x238 receive unknown OP code counter */
    volatile uint32_t  raln;            /* 0x23C receive alignment error counter */
    volatile uint32_t  rflr;            /* 0x240 receive frame length error counter */
    volatile uint32_t  rcde;            /* 0x244 receive code error counter */
    volatile uint32_t  rcse;            /* 0x248 receive carrier sense error counter */
    volatile uint32_t  rund;            /* 0x24C receive undersize packet counter */
    volatile uint32_t  rovr;            /* 0x250 receive oversize packet counter */
    volatile uint32_t  rfrg;            /* 0x254 receive fragments counter */
    volatile uint32_t  rjbr;            /* 0x258 receive jabber counter */
    volatile uint32_t  rdrp;            /* 0x25C receive drop */

    /* TRANSMIT COUNTERS */
    volatile uint32_t  tbyt;            /* 0x260 transmit byte counter */
    volatile uint32_t  tpkt;            /* 0x264 transmit packet counter */
    volatile uint32_t  tmca;            /* 0x268 transmit multicast packet counter */
    volatile uint32_t  tbca;            /* 0x26C transmit broadcast packet counter */
    volatile uint32_t  txpf;            /* 0x270 transmit PAUSE control frame counter */
    volatile uint32_t  tdfr;            /* 0x274 transmit deferral packet counter */
    volatile uint32_t  tedf;            /* 0x278 transmit excessive deferral packet counter */
    volatile uint32_t  tscl;            /* 0x27C transmit single collision packet counter */
    volatile uint32_t  tmcl;            /* 0x280 transmit multiple collision packet counter */
    volatile uint32_t  tlcl;            /* 0x284 transmit late collision packet counter */
    volatile uint32_t  txcl;            /* 0x288 transmit excessive collision packet counter */
    volatile uint32_t  tncl;            /* 0x28C transmit total collision counter */
    volatile uint32_t  DTSEC_RESERVED9; /* 0x290 */
    volatile uint32_t  tdrp;            /* 0x294 transmit drop frame counter */
    volatile uint32_t  tjbr;            /* 0x298 transmit jabber frame counter */
    volatile uint32_t  tfcs;            /* 0x29C transmit FCS error counter */
    volatile uint32_t  txcf;            /* 0x2A0 transmit control frame counter */
    volatile uint32_t  tovr;            /* 0x2A4 transmit oversize frame counter */
    volatile uint32_t  tund;            /* 0x2A8 transmit undersize frame counter */
    volatile uint32_t  tfrg;            /* 0x2AC transmit fragments frame counter */

    /* GENERAL REGISTERS */
    volatile uint32_t  car1;            /* 0x2B0 carry register one register* */
    volatile uint32_t  car2;            /* 0x2B4 carry register two register* */
    volatile uint32_t  cam1;            /* 0x2B8 carry register one mask register */
    volatile uint32_t  cam2;            /* 0x2BC carry register two mask register */
    volatile uint32_t  DTSEC_RESERVED10[16]; /* 0x2C0-0x2FC */

}__packed dsec_mac_stats_t;


#define SYNC \
    __asm__ __volatile__("sync")

#define ISYNC \
    __asm__ __volatile__("isync")

#define EIEIO \
    __asm__ __volatile__("eieio")

#define DCBZL(x) \
    __asm__ __volatile__ ("dcbz 0,%0" : : "r" (x))

#define LWSYNC \
	__asm__ __volatile__ ("lwsync" : : : "memory")

#define DCBF(x) \
    __asm__ __volatile__ ("dcbf 0,%0" : : "r" (x))

#define DCBT(x) \
    __asm__ __volatile__ ("dcbt 0,%0" : : "r" (x))

#define DCBI(x) \
    __asm__ __volatile__ ("dcbi 0,%0" : : "r" (x))

/*These functions are used to access Cache Inhibited regions of memory */
static inline unsigned get32(const volatile unsigned *addr,unsigned offset,uint32_t flags)
{
    unsigned result=0;
    addr+= offset>>2;

    __asm__ __volatile__("lwz%U1%X1 %0,%1; twi 0,%0,0; isync"
                 : "=r" (result) : "m" (*addr));
    return result;
}

static inline void set32(volatile unsigned *addr,unsigned offset, unsigned val,uint32_t flags )
{
    addr+= offset>>2;
    __asm__ __volatile__("stw%U0%X0 %1,%0; sync": "=m" (*addr) : "r" (val));
}

static inline void getset32_or(volatile unsigned *addr,unsigned offset, unsigned or_val,uint32_t flags )
{
	unsigned result=0;
    addr+= offset>>2;
   __asm__ __volatile__("lwz%U1%X1 %0,%1; twi 0,%0,0; isync"
                 : "=r" (result) : "m" (*addr));
	result |= or_val;
    __asm__ __volatile__("stw%U0%X0 %1,%0; sync": "=m" (*addr) : "r" (result));
}

static inline void getset32_and(volatile unsigned *addr,unsigned offset, unsigned and_val,uint32_t flags )
{
	unsigned result=0;
    addr+= offset>>2;
   __asm__ __volatile__("lwz%U1%X1 %0,%1; twi 0,%0,0; isync"
                 : "=r" (result) : "m" (*addr));
	result &= and_val;
    __asm__ __volatile__("stw%U0%X0 %1,%0; sync": "=m" (*addr) : "r" (result));
}

#define CCSR_RMW32_AND( base, offset, and_val, flags) \
        getset32_and((uint32_t*)base,offset,and_val,flags)

#define CCSR_RMW32_OR( base, offset, or_val, flags) \
        getset32_or((uint32_t*)base,offset,or_val,flags)

#define CCSR_SET32( base, offset, value, flags) \
        set32((uint32_t*)base,offset,value,flags)

#define CCSR_GET32(base, offset,flags) \
        get32((uint32_t*)base,offset,flags)    

/*Simple memory access routine, works with both Cache Inhibited and Cache enabled.
 * However care must be taken to ensure cache coherency while using them in Cache
 * enabled region. Usually not recommended for Cache enabled areas.*/

#define MEM_GET32(base, offset,flags) \
        *((base) + (((offset)>>2)))  

#define MEM_SET32(base, offset,value,flags) \
        *(base + ((offset)>>2))= value 

#define IS_MEM_ALIGNED(x) 	((x)&((x)-1))
/*Align given memory with a power of 2*/
#define MEM_ALIGN( extmem )		/*Align with power of 2*/ \
		do {    \
			(extmem) &= (extmem)-1; \
		} while((extmem) & ((extmem)-1));\
		/*Allocate the next bigger chunk if its less than already 32 bit value*/ \
		extmem = (extmem & 0x80000000 )? extmem : extmem<<1 

/*Instruction which would replace all above stuff but kill portability
		 * and may be understandability ....*/
		//__asm__ ("cntlzw %0  %1 ":"=r"(size) :"r"(extmem));
		//size = 31- size; 
		//extmem = 1 << size; 

/***********************************************************************************
 *
 *                                 Configuration Space
 *
 ***********************************************************************************/
typedef enum {
	MODE_PI_CACHE_INHIBITED,
	MODE_PI_CACHE_ENABLED,
	MODE_VBIT_MODE
} swportal_producer_mode_t;

typedef enum {
	MODE_CI_CACHE_INHIBITED,
	MODE_CI_CACHE_ENABLED,
	MODE_CI_DCA
} swportal_consumer_mode_t;

typedef enum {
 MAC_SPEED_INVALID,
 MAC_SPEED_10MB,
 MAC_SPEED_100MB,
 MAC_SPEED_1GB,
 MAC_SPEED_10GB
} fm_mac_speed_t;

typedef enum {
 MAC_DUPLEX_INVALID,
 MAC_DUPLEX_HALF,
 MAC_DUPLEX_FULL,
} fm_mac_duplex_t;

typedef enum {
 SET_PROMISCUOUS_OFF,
 SET_PROMISCUOUS_UNI,
 SET_PROMISCUOUS_MULTI,
 SET_PROMISCUOUS_ALL,
} fm_promiscuous_t;

/*DPAA configuration Codes. */

typedef enum {
  DPAA_CFG_CODE_MIN,
  /*Donot add anything before this*/

  DPAA_CFG_CODE_INIT_QMAN_SWPORTAL,
  DPAA_CFG_CODE_INIT_FQ,
  DPAA_CFG_CODE_INIT_BMAN_SWPORTAL,
  DPAA_CFG_CODE_INIT_BPID,
  DPAA_CFG_CODE_CONFIG_PORT,

  /*Donot add anything beyond that*/
  DPAA_CFG_CODE_MAX

} DPAA_CFG_CODE_T;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t chmap;
	dpaa_cpu_affinity_t affinity;
	uint32_t fqid_base;
	uint32_t fqid_num;
    void *owner;
    void (*default_cbfn)(void *token,uint32_t* size);
    void *token;
    uint32_t token_size;
}init_qman_portal_t;
typedef struct {
    uint32_t result;
}init_qman_portal_resp_t;


typedef struct {
    uint32_t portal;
    uint32_t flags;
    void *owner;
    void (*default_cbfn)(void *token,uint32_t* size);
    void *token;
    uint32_t token_size;
}init_bman_portal_t;
typedef struct {
    uint32_t result;
}init_bman_portal_resp_t;

typedef struct {
    uint32_t portal;
	uint32_t flags;
    uint32_t channel;
    uint32_t workq;
    uint32_t fqid;
    fq_handler_funcptr_t fq_handler;
	void *token;
    uint32_t token_size;
    uint32_t num_of_fqid;
	uint32_t align;
}init_fq_t;
typedef struct {
    uint32_t result;
    uint32_t fqid;
    uint32_t chid;
    void*    handle;
}init_fq_resp_t;


typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t block_size; /*size of each memory block*/
    uint32_t block_count; /*how many blocks*/
    void *owner;
}init_mempool_t;
typedef struct {
    uint32_t result;
    uint32_t mempool_id;
}init_mempool_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
}init_bpool_t;
typedef struct {
    uint32_t result;
    uint32_t count;
    uint32_t bpid[DPAA_BMAN_MAX_BPOOL];
    uint32_t size[DPAA_BMAN_MAX_BPOOL];
}init_bpool_resp_t;

typedef struct {
    uint32_t result;
}resp_t;   /*Default response*/


typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t port_id;   
    uint32_t tx_cfqid;
    uint32_t tx_defqid;
    uint32_t tx_errqid;
    uint32_t rx_pool_count;
    uint32_t rx_bpid[DPAA_MAX_BPID_PER_RX_PORT];
    uint32_t rx_buf_size[DPAA_MAX_BPID_PER_RX_PORT];
    uint32_t rx_defqid;
    uint32_t rx_errqid;
	uint32_t pcdNumOfQs;
	uint32_t pcdBaseQ;
	uint32_t rx_pvt_data_size; 
}config_port_t;

typedef struct {
    uint32_t result;   
	uint32_t rx_pvt_data_size; 

}config_port_resp_t;


typedef struct {
    DPAA_CFG_CODE_T code;

    union{
        init_qman_portal_t  init_qman_portal;
        init_bman_portal_t  init_bman_portal;
        init_fq_t           init_fq;
        init_mempool_t      init_mempool;
        init_bpool_t        init_bpool;
		config_port_t		config_port;
    };
    /*These are the response sections which may be filled by config API*/
    union {
        resp_t tesp;   /*Default response*/
        init_qman_portal_resp_t init_qman_portal_resp;
        init_bman_portal_resp_t init_bman_portal_resp;
        init_fq_resp_t init_fq_resp;
        init_mempool_resp_t init_mempool_resp;
        init_bpool_resp_t init_bpool_resp;
		config_port_resp_t config_port_resp;
    };
} dpaa_cfg_t;


/***********************************************************************************
 *
 *                                 Runtime Space
 *
 ***********************************************************************************/


typedef enum {
  DPAA_RUNTIME_CODE_MIN,
  /*Donot add anything before this*/
  DPAA_RUNTIME_CODE_ENABLE_PORT,
  DPAA_RUNTIME_CODE_DISABLE_PORT,
  
  DPAA_RUNTIME_CODE_ATTACH_IRQ_QMSWPORTAL,
  DPAA_RUNTIME_CODE_ATTACH_IRQ_BMSWPORTAL,
  DPAA_RUNTIME_CODE_DETACH_IRQ_QMSWPORTAL,
  DPAA_RUNTIME_CODE_DETACH_IRQ_BMSWPORTAL,

  DPAA_RUNTIME_CODE_FAST_ENQUEUE,
  DPAA_RUNTIME_CODE_FAST_DEQUEUE,
  DPAA_RUNTIME_CODE_BUF_RELEASE,
  DPAA_RUNTIME_CODE_BUF_ACQUIRE,
  DPAA_RUNTIME_CODE_BUF_FLUSH,
  
  DPAA_RUNTIME_CODE_CLEAR_BMAN_IRQ_CAUSE,
  DPAA_RUNTIME_CODE_CLEAR_QMAN_IRQ_CAUSE,
  DPAA_RUNTIME_CODE_CLEAR_FMAN_IRQ_CAUSE,

  DPAA_RUNTIME_CODE_SET_MAC_CONFIG,
  DPAA_RUNTIME_CODE_GET_MAC_STATS,
  DPAA_RUNTIME_CODE_MII_READ,
  DPAA_RUNTIME_CODE_MII_WRITE,
  
  DPAA_RUNTIME_CODE_ENABLE_PCD,
  DPAA_RUNTIME_CODE_DISABLE_PCD,
  DPAA_RUNTIME_CODE_DUMP_REGS, 
  
  DPAA_RUNTIME_CODE_SET_PROMISCUOUS,

  DPAA_RUNTIME_CODE_ADD_HASHMACADDR,
  DPAA_RUNTIME_CODE_DEL_HASHMACADDR,
  
  /*Donot add anything beyond that*/
  DPAA_RUNTIME_CODE_MAX

} DPAA_RUNTIME_CODE_T;



typedef struct {
    uint32_t portal;
    uint32_t flags;
	uint32_t pcd_port;
	uint32_t base_fq; 
	uint32_t num_of_qid;
	uint32_t fm_master;
}enable_pcd_t;
typedef struct {
    uint32_t result;    
}enable_pcd_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t port_id;   
}enable_port_t;
typedef struct {
    uint32_t result;    
}enable_port_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t port_id;   
}disable_port_t;
typedef struct {
    uint32_t result;    
}disable_port_resp_t;

typedef struct {
    uint32_t portal;
    void* owner;
    void (*intr_callback_fn)(void* token,uint32_t* size);
    void *token;
    uint32_t token_size;
}attach_callback_func_t;
typedef struct {
    uint32_t result;
}attach_callback_func_resp_t;

typedef struct {
    uint32_t portal;
    void* owner;
    void* func_ptr;
}detach_callback_func_t;
typedef struct {
    uint32_t result;
}detach_callback_func_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t fqid;
    void* 	 tx_handle;
	uint32_t tx_fqid_offset;
    fd_t* fd_ptr;
}pkt_enqueue_t;
typedef struct {
    uint32_t result;
}pkt_enqueue_resp_t ;

typedef struct {
    uint32_t portal;
    uint32_t flags;
}pkt_dequeue_t;
typedef struct {
    uint32_t result;
    fd_t* fd_ptr;
    uint32_t contextB;
    uint32_t fqid;
    uint32_t frames_to_dequeue;
}pkt_dequeue_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t bpid;
    void* addr;
}buf_release_t;
typedef struct {
    uint32_t result;
}buf_release_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    uint32_t bpid;
    uint32_t num;
}buf_acquire_t;
typedef struct {
    uint32_t result;
    uint32_t bpid;
    uint64_t count;
    uint64_t addr[DPAA_BMAN_MAX_BUF_PER_CMD];
}buf_acquire_resp_t;


typedef struct {
    uint32_t portal;
    uint32_t flags;
    void* owner;
    uint32_t id;
}clear_irq_cause_t;
typedef struct {
    uint32_t result;
}clear_irq_cause_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    void* owner;
	uint32_t port_id;   
	fm_mac_speed_t speed;
	fm_mac_duplex_t duplex;
}set_mac_config_t;
typedef struct {
    uint32_t result;
}set_mac_config_resp_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
    void* owner;
	uint32_t port_id;   
}get_mac_stats_t;
typedef struct {
    uint32_t result;
	dsec_mac_stats_t stats;
}get_mac_stats_resp_t;

/* MII Read */
typedef struct {
    uint32_t portal;
    uint32_t flags;
    void* owner;
	uint32_t port_id;   
	uint8_t phy_add;
	uint8_t reg_add;   
}mii_read_t;
typedef struct {
    uint32_t result;
	uint16_t reg_value;
}mii_read_resp_t;


/* MII Write */
typedef struct {
    uint32_t portal;
    uint32_t flags;
    void* owner;
	uint32_t port_id;   
	uint8_t phy_add;
	uint8_t reg_add;   
	uint16_t data;
}mii_write_t;

typedef struct {
    uint32_t portal;
    uint32_t flags;
	uint32_t port_id;   
    fm_promiscuous_t promisc;
}set_promiscuous_t;
typedef struct {
    uint32_t result;
}set_promiscuous_resp_t;

typedef struct {
	uint32_t portal;
	uint32_t flags;
	uint32_t port_id;   
	uint8_t  ethaddr[6];
}hashmacaddr_t;
typedef struct {
    uint32_t result;
}hashmacaddr_resp_t;

typedef struct {
    uint32_t result;    
}def_resp_t;    /*Default Response*/


typedef struct {
    DPAA_RUNTIME_CODE_T code;

    union{
        enable_port_t enable_port;
        enable_pcd_t enable_pcd;
        disable_port_t disable_port;
        attach_callback_func_t attach_cb_func;
        detach_callback_func_t detach_cb_func;
        pkt_enqueue_t pkt_enqueue;
        pkt_dequeue_t pkt_dequeue;
        buf_release_t buf_release;
        buf_acquire_t buf_acquire;
        clear_irq_cause_t clear_irq_cause;
		set_mac_config_t set_mac_config;
		get_mac_stats_t get_mac_stats;
		mii_read_t mii_read;
		mii_write_t mii_write;
		set_promiscuous_t set_promiscuous;
		hashmacaddr_t hashmacaddr;
    };
    
    /*These are the response sections which may be filled by runtime API*/
    union{
        def_resp_t def_resp;    /*Default Response*/
        enable_port_resp_t enable_port_resp;
        enable_pcd_resp_t enable_pcd_resp;
        disable_port_resp_t disable_port_resp;
        attach_callback_func_resp_t attach_irq_resp;
        detach_callback_func_resp_t detach_irq_resp;
        pkt_enqueue_resp_t pkt_enqueue_resp;
        pkt_dequeue_resp_t pkt_dequeue_resp;
        buf_release_resp_t buf_release_resp;
        buf_acquire_resp_t buf_acquire_resp;
        clear_irq_cause_resp_t clear_irq_cause_resp;
		set_mac_config_resp_t set_mac_config_resp;
		get_mac_stats_resp_t get_mac_stats_resp;
		mii_read_resp_t mii_read_resp;
		set_promiscuous_resp_t set_promiscuous_resp;
		hashmacaddr_resp_t hashmacaddr_resp;
    };
} dpaa_runtime_t;

/***********************************************************************************
 *
 *                                 Ioctl Space
 *
 ***********************************************************************************/
#include <sys/platform.h>
#ifndef _U_INT8_T_DEFINED
#define _U_INT8_T_DEFINED
typedef _Uint8t         u_int8_t;
#endif
#ifndef _U_INT16_T_DEFINED
#define _U_INT16_T_DEFINED
typedef _Uint16t        u_int16_t;
#endif
#ifndef _U_INT32_T_DEFINED
#define _U_INT32_T_DEFINED
typedef _Uint32t        u_int32_t;
#endif
#include <net/if.h>

#if 0
struct dpaa_fqd {
	uint32_t fq_link;

	uint8_t  orprws:3;
	uint8_t  oa:1;
	uint16_t odp_seq:14;
	uint16_t orp_nesn:14;

	uint8_t  olws:2;
	uint16_t orp_ea_hseq:15, orp_ea_tseq:15;

	uint32_t orp_ea_hptr:24, orp_ea_tptr:24;
	uint32_t pfdr_hptr:24, pfdr_tptr:24;

	uint64_t context_a;

	uint32_t context_b;

	uint16_t fq_ctrl:11;
	uint8_t  fe:1;
	uint8_t  r:1;
	uint8_t	 state:3;
	uint16_t dest_wq;

	uint16_t ics_surp;
	uint8_t  is:1;
	uint16_t ics_cred:15;

	uint32_t byte_cnt;

	uint8_t  cong_id:8;
	uint32_t frm_cnt:24;

	uint8_t  nra:2, oac:2, rsrv1:1;
	uint16_t ra1_sfdr:11;
	uint8_t  rsrv2:1, it:1, rsrv3:3;
	uint16_t ra2_sfdr:11;

	uint8_t  nod:2, oal0:3;
	uint16_t od1_sfdr:11;
	uint8_t  oal1:5;
	uint16_t od2_sfdr:11;

	uint8_t  npc:2, rsrv4:3;
	uint16_t od3_sfdr:11;
	uint8_t  rsrv5:3;
	uint16_t td_mant:8;
	uint8_t  td_exp:5;
} __packed;
#else
struct dpaa_fqd {
	uint8_t data[64];
};
#endif

struct dpaa_fqd_req {
	struct ifdrv ifd;
	union {
		uint32_t args[2];
		uint8_t data[64];
		struct dpaa_fqd fqd;
	};
};

#define SIOCDPAAQM		 _IOWR('X', 0x00, struct ifreq)
#define SIOCDPAABM		 _IOWR('X', 0x01, struct ifreq)
#define SIOCDPAABMPOOL	 _IOWR('X', 0x02, struct ifreq)
#define SIOCDPAAFM		 _IOWR('X', 0x03, struct ifreq)
#define SIOCDPAAFMPORT	 _IOWR('X', 0x04, struct ifreq)
#define SIOCDPAAFMPCD	 _IOWR('X', 0x05, struct ifreq)
#define SIOCDPAAQRYFQ_P	 _IOWR('X', 0x06, struct dpaa_fqd_req)
#define SIOCDPAAQRYFQ_NP _IOWR('X', 0x07, struct dpaa_fqd_req)
#define SIOCDPAAQRYWQ    _IOWR('X', 0x08, struct dpaa_fqd_req)

typedef enum e_bman_counters {
    BMAN_COUNTERS_FBPR = 0                    /* Total Free Buffer Proxy Record (FBPR) Free Pool Count in external memory */
} e_bman_counters;

typedef enum e_bman_pool_counters {
    BMAN_POOL_COUNTERS_CONTENT = 0,         /**< number of free buffers for a particular pool */
    BMAN_POOL_COUNTERS_SW_DEPLETION,        /**< number of times pool entered sw depletion */
    BMAN_POOL_COUNTERS_HW_DEPLETION,        /**< number of times pool entered hw depletion */
    BMAN_POOL_COUNTERS_HDPTR                /**< index of first entry of linked list of free pool buffer addresses */
} e_bman_pool_counters;

typedef enum e_qman_counters {
    QMAN_COUNTERS_SFDR_IN_USE = 0,            /* Total Single Frame Descriptor Record (SFDR) currently in use */
    QMAN_COUNTERS_PFDR_IN_USE,                /* Total Packed Frame Descriptor Record (PFDR) currently in use */
    QMAN_COUNTERS_PFDR_FREE_POOL              /* Total Packed Frame Descriptor Record (PFDR) Free Pool Count in external memory */
} e_qman_counters;

typedef enum e_fman_counters {
    FMAN_COUNTERS_ENQ_TOTAL_FRAME = 0,        /* QMI total enqueued frames counter */
    FMAN_COUNTERS_DEQ_TOTAL_FRAME,            /* QMI total dequeued frames counter */
    FMAN_COUNTERS_DEQ_0,                      /* QMI 0 frames from QMan counter */
    FMAN_COUNTERS_DEQ_1,                      /* QMI 1 frames from QMan counter */
    FMAN_COUNTERS_DEQ_2,                      /* QMI 2 frames from QMan counter */
    FMAN_COUNTERS_DEQ_3,                      /* QMI 3 frames from QMan counter */
    FMAN_COUNTERS_DEQ_FROM_DEFAULT,           /* QMI dequeue from default queue counter */
    FMAN_COUNTERS_DEQ_FROM_CONTEXT,           /* QMI dequeue from FQ context counter */
    FMAN_COUNTERS_DEQ_FROM_FD,                /* QMI dequeue from FD command field counter */
    FMAN_COUNTERS_DEQ_CONFIRM,                /* QMI dequeue confirm counter */
    FMAN_COUNTERS_SEMAPHOR_ENTRY_FULL_REJECT, /* DMA semaphor reject due to full entry counter */
    FMAN_COUNTERS_SEMAPHOR_QUEUE_FULL_REJECT, /* DMA semaphor reject due to full CAM queue counter */
    FMAN_COUNTERS_SEMAPHOR_SYNC_REJECT        /* DMA semaphor reject due to sync counter */
} e_fman_counters;

typedef enum e_fman_port_counters {
    FMAN_PORT_COUNTERS_CYCLE,                       /**< BMI performance counter */
    FMAN_PORT_COUNTERS_TASK_UTIL,                   /**< BMI performance counter */
    FMAN_PORT_COUNTERS_QUEUE_UTIL,                  /**< BMI performance counter */
    FMAN_PORT_COUNTERS_DMA_UTIL,                    /**< BMI performance counter */
    FMAN_PORT_COUNTERS_FIFO_UTIL,                   /**< BMI performance counter */
    FMAN_PORT_COUNTERS_RX_PAUSE_ACTIVATION,         /**< BMI Rx only performance counter */
    FMAN_PORT_COUNTERS_FRAME,                       /**< BMI statistics counter */
    FMAN_PORT_COUNTERS_DISCARD_FRAME,               /**< BMI statistics counter */
    FMAN_PORT_COUNTERS_DEALLOC_BUF,                 /**< BMI deallocate buffer statistics counter */
    FMAN_PORT_COUNTERS_RX_BAD_FRAME,                /**< BMI Rx only statistics counter */
    FMAN_PORT_COUNTERS_RX_LARGE_FRAME,              /**< BMI Rx only statistics counter */
    FMAN_PORT_COUNTERS_RX_OUT_OF_BUFFERS_DISCARD,   /**< BMI Rx only statistics counter */
    FMAN_PORT_COUNTERS_RX_FILTER_FRAME,             /**< BMI Rx & OP only statistics counter */
    FMAN_PORT_COUNTERS_RX_LIST_DMA_ERR,             /**< BMI Rx, OP & HC only statistics counter */
    FMAN_PORT_COUNTERS_WRED_DISCARD,                /**< BMI OP & HC only statistics counter */
    FMAN_PORT_COUNTERS_LENGTH_ERR,                  /**< BMI non-Rx statistics counter */
    FMAN_PORT_COUNTERS_UNSUPPRTED_FORMAT,           /**< BMI non-Rx statistics counter */
    FMAN_PORT_COUNTERS_DEQ_TOTAL,                   /**< QMI counter */
    FMAN_PORT_COUNTERS_ENQ_TOTAL,                   /**< QMI counter */
    FMAN_PORT_COUNTERS_DEQ_FROM_DEFAULT,            /**< QMI counter */
    FMAN_PORT_COUNTERS_DEQ_CONFIRM                  /**< QMI counter */
} e_fman_port_counters;

typedef enum e_fman_pcd_counters {
    FMAN_PCD_KG_COUNTERS_TOTAL,                                 /**< Policer counter */
    FMAN_PCD_PLCR_COUNTERS_YELLOW,                              /**< Policer counter */
    FMAN_PCD_PLCR_COUNTERS_RED,                                 /**< Policer counter */
    FMAN_PCD_PLCR_COUNTERS_RECOLORED_TO_RED,                    /**< Policer counter */
    FMAN_PCD_PLCR_COUNTERS_RECOLORED_TO_YELLOW,                 /**< Policer counter */
    FMAN_PCD_PLCR_COUNTERS_TOTAL,                               /**< Policer counter */
    FMAN_PCD_PLCR_COUNTERS_LENGTH_MISMATCH,                     /**< Policer counter */
    FMAN_PCD_PRS_COUNTERS_PARSE_DISPATCH,                       /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_L2_PARSE_RESULT_RETURNED,             /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_L3_PARSE_RESULT_RETURNED,             /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_L4_PARSE_RESULT_RETURNED,             /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_SHIM_PARSE_RESULT_RETURNED,           /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_L2_PARSE_RESULT_RETURNED_WITH_ERR,    /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_L3_PARSE_RESULT_RETURNED_WITH_ERR,    /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_L4_PARSE_RESULT_RETURNED_WITH_ERR,    /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_SHIM_PARSE_RESULT_RETURNED_WITH_ERR,  /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_SOFT_PRS_CYCLES,                      /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_SOFT_PRS_STALL_CYCLES,                /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_HARD_PRS_CYCLE_INCL_STALL_CYCLES,     /**< Parser counter */
    FMAN_PCD_PRS_COUNTERS_MURAM_READ_CYCLES,                    /**< MURAM counter */
    FMAN_PCD_PRS_COUNTERS_MURAM_READ_STALL_CYCLES,              /**< MURAM counter */
    FMAN_PCD_PRS_COUNTERS_MURAM_WRITE_CYCLES,                   /**< MURAM counter */
    FMAN_PCD_PRS_COUNTERS_MURAM_WRITE_STALL_CYCLES,             /**< MURAM counter */
    FMAN_PCD_PRS_COUNTERS_FPM_COMMAND_STALL_CYCLES              /**< FPM counter */
} e_fman_pcd_counters;


typedef struct {
    uint32_t code;
	uint8_t iface;
    uint8_t idx;
    uint16_t inst;
	uint32_t value;
	uint32_t nresbuf;
	void *resbuf;
} dpaa_ioctl_action_t;



#define IN const
#define OUT  
#define INOUT 


/*****************************************************************************/
/*  Function Name   : dpaa_system_init
 *  Input           : Pointer to startup parameters
 *  Description     : Initializes the DPAA system and library. 
 *  Returns         : NonZERO handle if successful, ZERO otherwise. 
 *  Note            :               
 */
/*****************************************************************************/
dpaa_handle_t dpaa_system_init(IN dpaa_startup_params_t *cfg);

/*****************************************************************************/
/*  Function Name   : dpaa_system_config
 *  Input           : Pointer to dpaa handle
 *                    Pointer to configuration parameter
 *  Description     : Configures the DPAA system as per the parameters. 
 *  Returns         : ZERO if successful, Non ZERO otherwise. 
 *  Note            :               
 */
/*****************************************************************************/
int dpaa_system_config(dpaa_handle_t hdl, dpaa_cfg_t *cfg);

/*****************************************************************************/
/*  Function Name   : dpaa_system_runtime
 *  Input           : Pointer to dpaa handle
 *                    Pointer to runtime parameters
 *  Description     : Process actions once the library has been fully configured. 
 *  Returns         : ZERO if successful, Non ZERO otherwise. 
 *  Note            :               
 */
/*****************************************************************************/
int dpaa_system_runtime(dpaa_handle_t hdl, dpaa_runtime_t *action);


/*****************************************************************************/
/*  Function Name   : dpaa_system_shutdown
 *  Input           : Pointer to dpaa handle
 *                    shutdown mode
 *  Description     : shutsdown the DPAA system as per the given mode. 
 *  Returns         : ZERO if successful, Non ZERO otherwise. 
 *  Note            :               
 */
/*****************************************************************************/
int dpaa_system_shutdown(dpaa_handle_t hdl, int mode);

/*****************************************************************************/
/*  Function Name   : dpaa_system_ioctl
 *  Input           : Pointer to dpaa handle
 *                    Pointer to DPAA ioctl action 
 *  Description     : Processes the IOCTL commands issued to DPAA  
 *  Returns         : ZERO if successful, Non ZERO otherwise. 
 *  Note            :               
 */
/*****************************************************************************/
int dpaa_system_ioctl(dpaa_handle_t hdl, dpaa_ioctl_action_t* action);


__SRCVERSION( "$URL: http://community.qnx.com/svn/repos/core_networking/trunk/sys/dev_qnx/dpaa/dpaa.h $ $Rev: 1120 $" )
#endif
