/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
#include "qman.h"


static void inline dump_fd(const fd_t *fd);

static frame_queue_t *
lookup_fqid(frame_queue_t *start, uint32_t id){
	
	frame_queue_t *ptr=NULL;
	LOG_FUNCTION_ENTRY;

	if(start==NULL){
		DPAA_LOG(LOG_WARNING, "%s(): Nullvalue passed at head",__FUNCTION__);
		LOG_FUNCTION_EXIT;
		return NULL;
	}
	for(ptr=start;; ptr=ptr->next ){
		if((id>=ptr->fqid) && (id < (ptr->fqid+ ptr->offset))){
			DPAA_LOG(LOG_DEBUG_DATA_FLOW, "Found FQID %d at %p",id,ptr);
			LOG_FUNCTION_EXIT;
			return ptr;
		}
		if (!ptr->next){
			break;
		}
	}
	LOG_FUNCTION_EXIT;
	return NULL;
}

static void QmExceptionsCB(t_Handle h_App, e_QmExceptions exception)
{
    t_QmErrorInfo   errorInfo;
    qman_t           *qman = (qman_t *)h_App;

    switch(exception)
    {
        case(e_QM_EX_CORENET_INITIATOR_DATA):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_CORENET_INITIATOR_DATA");
            break;
        case(e_QM_EX_CORENET_TARGET_DATA):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_CORENET_TARGET_DATA");
            break;
        case(e_QM_EX_CORENET_INVALID_TARGET_TRANSACTION):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_CORENET_INVALID_TARGET_TRANSACTION");
            break;
        case(e_QM_EX_PFDR_THRESHOLD):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_PFDR_THRESHOLD");
            break;
        case(e_QM_EX_PFDR_ENQUEUE_BLOCKED):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_PFDR_ENQUEUE_BLOCKED");
            break;
        case(e_QM_EX_SINGLE_ECC):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_SINGLE_ECC");
            break;
        case(e_QM_EX_MULTI_ECC):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_MULTI_ECC");
            break;
        case(e_QM_EX_INVALID_COMMAND):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_INVALID_COMMAND");
            break;
        case(e_QM_EX_DEQUEUE_DCP):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_DEQUEUE_DCP");
            break;
        case(e_QM_EX_DEQUEUE_FQ):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_DEQUEUE_FQ");
            break;
        case(e_QM_EX_DEQUEUE_SOURCE):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_DEQUEUE_SOURCE");
            break;
        case(e_QM_EX_DEQUEUE_QUEUE):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_DEQUEUE_QUEUE");
            break;
        case(e_QM_EX_ENQUEUE_OVERFLOW):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_ENQUEUE_OVERFLOW");
            break;
        case(e_QM_EX_ENQUEUE_STATE):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_ENQUEUE_STATE");
            break;
        case(e_QM_EX_ENQUEUE_CHANNEL):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_ENQUEUE_CHANNEL");
            break;
        case(e_QM_EX_ENQUEUE_QUEUE):
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got exception e_QM_EX_ENQUEUE_QUEUE");
            break;
        default:
            DPAA_LOG(LOG_RUNTIME_FAULT, "QM got invalid exception %d", exception);
            break;
    }
    memset(&errorInfo, 0 , sizeof(errorInfo));
    if (QM_GetErrorInformation(qman->h_Qm, &errorInfo) == E_OK)
    {
        if (errorInfo.portalValid)
        {
            if (errorInfo.hwPortal)
		DPAA_LOG(LOG_WARNING, "Qm Error Information: Dcp Id: %d", errorInfo.dcpId);
            else
		DPAA_LOG(LOG_WARNING, "Qm Error Information: Sw Portal Id: %d", errorInfo.swPortalId);
        }
        if (errorInfo.fqidValid)
			DPAA_LOG(LOG_WARNING, "Qm Error Information: Fqid: %d", errorInfo.fqid);
    }
}

/*Exported functions*/
int 
qman_init(dpaa_system_t* dpaa,IN dpaa_startup_params_t* cfg  )
{
	int rc= EOK;
	qman_t *qman=NULL;
	t_QmParam qmParam;
	
	LOG_FUNCTION_ENTRY;

	if((qman = malloc(sizeof(qman_t)))== NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "Can not allocate memory to qman_t");
		qman=NULL;
		rc= ENOMEM;
		goto exit_cleanup;
	}
	
	memset(qman,0,sizeof(qman_t));

	memset(&qmParam, 0, sizeof(qmParam));
	qmParam.guestId = NCSW_MASTER_ID;
    qmParam.baseAddress = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_QM, 0);
    qmParam.swPortalsBaseAddress = SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_QM_CE_PORTAL, 0);
	qmParam.partFqidBase = cfg->fqid_base;
    qmParam.partNumOfFqids = cfg->total_fqid;
	qmParam.totalNumOfFqids = cfg->total_fqid;
	qmParam.fqdMemPartitionId= QNX_PARTITION_ID_NON_CACHED;
	qmParam.pfdrMemPartitionId= QNX_PARTITION_ID_NON_CACHED;
	qmParam.h_App= (t_Handle)qman;
	qmParam.f_Exception= QmExceptionsCB;
	qmParam.errIrq = NO_IRQ;

    qman->h_Qm = QM_Config(&qmParam);

    if(!qman->h_Qm){
		DPAA_LOG(LOG_CRIT_ERROR, "QM_Config failed");
		rc=-1;	
		goto exit_cleanup;
	}

	if( QM_Init(qman->h_Qm) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "QM_Init failed");
		rc = -1;	
		QM_Free(qman->h_Qm);
		goto exit_cleanup;
	}
	
	qman->fq_list=NULL;
	qman->fq_count=0;
	dpaa->qman= (qman_handle_t)qman;
	goto exit_ok;

exit_cleanup:

	if(qman!=NULL){
		free(qman);
	}
	dpaa->qman=NULL;

exit_ok:
	LOG_FUNCTION_EXIT;
	return rc;

}




int 
qman_runtime(dpaa_system_t *dpaa){
	LOG_FUNCTION_ENTRY;
	LOG_FUNCTION_EXIT;
	return 0;
}

int 
qman_shutdown(dpaa_system_t *dpaa ){
	LOG_FUNCTION_ENTRY;
	LOG_FUNCTION_EXIT;
	return 0;
}

uint32_t 
qman_get_free_fqid(qman_handle_t hdl){
	qman_t *qman= (qman_t*)hdl;
	uint32_t fqid=0;

	LOG_FUNCTION_ENTRY;
	
	fqid= qman->fq_count+1;
	while((lookup_fqid(qman->fq_list, fqid)!=NULL))
	fqid++;

	LOG_FUNCTION_EXIT;
	return fqid;

}

int 
qman_portal_init_fqid(qman_handle_t hdl, init_fq_t *req, init_fq_resp_t *resp)
{

	frame_queue_t *fq=NULL,*tempq=NULL;
	int rc=EOK;
	qman_t *qman= (qman_t*)hdl;
    t_QmFqrParams   fqrParam;

	LOG_FUNCTION_ENTRY;

	if((fq=lookup_fqid(qman->fq_list,req->fqid))==NULL){
		/*Ok, this FQ hasnt been yet initialized*/
		/*Query QMAN for FQID state to ascertain that FQ is not in use by
		 * another process or portal*/
	}
	else{
		DPAA_LOG(LOG_CRIT_ERROR, "FQ id %d already initialized. Owner %p", fq->fqid,fq->owner);
		LOG_FUNCTION_EXIT;
		return -1;
	}

	DPAA_LOG(LOG_DEBUG_CONFIG, "Initializing FQ id %d, portal %d", req->fqid, req->portal);

	if((fq= malloc(sizeof(frame_queue_t)))==NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "Cannot allocate memory for new FQ");
		rc=ENOMEM;
		fq=NULL;
		goto exit_cleanup;
	}

	fq->next=NULL;
	fq->portal= req->portal;
	fq->fqid =req->fqid ;
	fq->wqid=req->workq;
    fq->chid= req->channel;
	fq->fq_handler= req->fq_handler;
	fq->token= req->token;
    fq->token_size= req->token_size;
	fq->offset= req->num_of_fqid;

   
	memset(&fqrParam, 0, sizeof(fqrParam));

	fqrParam.h_Qm  = qman->h_Qm;
	fqrParam.h_QmPortal  = qman->h_QmPortal[req->portal];

	fqrParam.canBeParked= FALSE;
	fqrParam.qs.nonFrcQs.align = req->align ? req->align : 1;
	//fqrParam.contextA = 0xabcd;
	fqrParam.channel = (e_QmFQChannel)req->channel;
	fqrParam.wq = 0;
	fqrParam.useForce = FALSE;
	fqrParam.qs.nonFrcQs.numOfFqids= req->num_of_fqid;
	fq->h_QmFqr= QM_FQR_Create(&fqrParam);

	if(fq->h_QmFqr==NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "QM_FQR_Create() failure");
		rc=-1;
		goto exit_cleanup;

	}
	DPAA_LOG(LOG_DEBUG_CONFIG, "QM_FQR_Create() %p", fq->h_QmFqr);
	fq->fq_handler = req->fq_handler;
	fq->token= req->token;
    resp->handle= fq;
	fq->fqid= QM_FQR_GetFqid(fq->h_QmFqr);
	resp->fqid =fq->fqid;
	resp->chid =req->channel;

	QM_FQR_RegisterCB(fq->h_QmFqr,qman_fq_cb,fq);

	DPAA_LOG(LOG_DEBUG_CONFIG, "Creating new FQ id %d Range %d", fq->fqid,req->num_of_fqid);
	qman->fq_count+= req->num_of_fqid;
	/*This function shall initialize the given frame queue ID*/
	if(qman->fq_list==NULL){
		qman->fq_list= fq;
	}
	else{
		for(tempq= qman->fq_list; tempq->next!=NULL; tempq=tempq->next);
		tempq->next= fq;
	}
	DPAA_LOG(LOG_DEBUG_CONFIG,"FQ[%d] successfully Created for channel %d", fq->fqid,fq->chid);
	rc=EOK;

	goto exit_ok;

exit_cleanup:
	if(fq!=NULL){
		free(fq);
	}
exit_ok:
	
	LOG_FUNCTION_EXIT;
	return rc;
}

e_RxStoreResponse qman_fq_cb(t_Handle h_Arg, t_Handle h_QmFqr, t_Handle h_QmPortal, uint32_t fqidOffset, t_DpaaFD *p_Frame){

	frame_queue_t *fq= (frame_queue_t *)h_Arg;

	if(fq->fq_handler){  
		DPAA_LOG_IF(LOG_DEBUG_DATA_FLOW){
			dump_fd((fd_t*)p_Frame);
		}
		fq->fq_handler(fq->token,(fq->fqid+ fqidOffset),(fd_t*)p_Frame);

	}
	else{
		DPAA_LOG(LOG_CRIT_ERROR, "No handler registered for FQID %d",fq->fqid);
	}
	return e_RX_STORE_RESPONSE_CONTINUE;
}


int 
qman_portal_init(qman_handle_t hdl, IN init_qman_portal_t *req, OUT init_qman_portal_resp_t* rsp){

	int rc=EOK;
	qman_t *qman= (qman_t*)hdl;
   	t_QmPortalParam         qmPortalParam;
	int i=0;
	LOG_FUNCTION_ENTRY;


   	memset(&qmPortalParam, 0, sizeof(qmPortalParam));
	qmPortalParam.ceBaseAddress=SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_QM_CE_PORTAL, req->portal) ;
	qmPortalParam.ciBaseAddress=SYS_GetMemoryMappedModuleBase(e_SYS_SUBMODULE_QM_CI_PORTAL, req->portal) ;

	qmPortalParam.h_Qm = qman->h_Qm;
	qmPortalParam.swPortalId = req->portal;
    qmPortalParam.irq           = 1;
    qmPortalParam.f_DfltFrame   = DefaultCB;
    qmPortalParam.h_App         = hdl;

    qman->h_QmPortal[req->portal] = QM_PORTAL_Config(&qmPortalParam);

    if(!qman->h_QmPortal[req->portal]){
		DPAA_LOG(LOG_CRIT_ERROR, "Cannot configure portal %d",req->portal);
		rc=-1;
		goto exit_cleanup;

	}

	if(QM_PORTAL_Init(qman->h_QmPortal[req->portal]) != E_OK) {
		DPAA_LOG(LOG_CRIT_ERROR, "Cannot init portal %d",req->portal);
		rc=-1;
		QM_PORTAL_Free(qman->h_QmPortal);
		goto exit_cleanup;

	}
	if(req->chmap){
		for(i=0;i<MAX_NUM_POOL_CHANNEL;i++){
			if(CHMAP_TEST_POOL_CHID_PRESENT(req->chmap,CHID_POOLCH_1+i)){
				DPAA_LOG(LOG_DEBUG_CONFIG, "Found Pool Channel %d in Portal ",i);
				if(QM_PORTAL_AddPoolChannel(qman->h_QmPortal[req->portal], i)!=EOK){
					DPAA_LOG(LOG_CRIT_ERROR, "Failed to add Pool channel %d to Portal %d ",i,req->portal);
				}
			}
		}
 	}
	pthread_mutex_init(&qman->rxLock[req->portal],NULL);
	pthread_mutex_init(&qman->txLock[req->portal],NULL);
exit_cleanup:

	LOG_FUNCTION_EXIT;
	return rc;
}

e_RxStoreResponse DefaultCB(t_Handle h_Arg, t_Handle h_QmFqr, t_Handle h_QmPortal, uint32_t fqidOffset, t_DpaaFD *p_Frame){
    DPAA_LOG(LOG_CRIT_ERROR, "Default Call Back from Q 0x%x", fqidOffset);
    return e_RX_STORE_RESPONSE_CONTINUE;
}

void qman_portal_def_cbfn(void *token,uint32_t* size){
	LOG_FUNCTION_ENTRY;
	
	LOG_FUNCTION_EXIT;
}

int 
qman_frame_enqueue(qman_handle_t hdl, IN pkt_enqueue_t *req, OUT pkt_enqueue_resp_t* resp ){
	
	int rc=EOK;
	qman_t *qman= (qman_t*)hdl;
	frame_queue_t *fq=NULL;

	LOG_FUNCTION_ENTRY;
	PORTAL_LOCK(req->flags, &qman->txLock[req->portal]);	

	if(req->tx_handle!=NULL){
		fq = (frame_queue_t *)req->tx_handle;
	}
	else if((fq=lookup_fqid(qman->fq_list,req->fqid))==NULL){
		DPAA_LOG(LOG_CRIT_ERROR, "Cannot lookup FQID %d",req->fqid);
		rc=-1;
		goto exit_cleanup;
	}

	DPAA_LOG_IF(LOG_DEBUG_DATA_FLOW){
		dump_fd(req->fd_ptr);
	}

    if((rc=QM_FQR_Enqueue(fq->h_QmFqr, qman->h_QmPortal[req->portal], req->tx_fqid_offset, (t_DpaaFD*)req->fd_ptr ))!=E_OK){
		//DPAA_LOG(LOG_CRIT_ERROR, "Failed to Enqueue Packet. Error 0x%x",rc);
		goto exit_cleanup;
	}
exit_cleanup:
	PORTAL_UNLOCK(req->flags, &qman->txLock[req->portal]);	
	LOG_FUNCTION_EXIT;
	return rc;
}

int 
qman_frame_dequeue(qman_handle_t hdl, IN pkt_dequeue_t *req, OUT pkt_dequeue_resp_t* resp ){

	int rc=EOK;
	qman_t *qman= (qman_t*)hdl;
//	LOG_FUNCTION_ENTRY;
 	PORTAL_LOCK(req->flags, &qman->rxLock[req->portal]);	

	if(QM_PORTAL_Poll(qman->h_QmPortal[req->portal],e_QM_PORTAL_POLL_SOURCE_BOTH)!=E_OK){
		DPAA_LOG(LOG_CRIT_ERROR, "Failed to Dequeue packets");
		rc=-1;
		goto exit_cleanup;

	}
exit_cleanup:
	PORTAL_UNLOCK(req->flags, &qman->rxLock[req->portal]);	

//	LOG_FUNCTION_EXIT;
	return rc;
}

static void inline 
dump_fd(const fd_t *fd){

	DPAA_LOG(LOG_DEBUG_DATA_FLOW,"============ BEGINNING FD DUMP ============");
	DPAA_LOG(LOG_DEBUG_DATA_FLOW,"FD DD=0x%x PID=0x%x BPID=0x%x ",fd->dd,fd->pid,fd->bpid );
	DPAA_LOG(LOG_DEBUG_DATA_FLOW,"FD ADDR HI=0x%4x LOW=0x%8x ", fd->addr_high,fd->addr_low);
	DPAA_LOG(LOG_DEBUG_DATA_FLOW,"FD FORMAT=0x%x OFFSET=0x%x LENGTH=%d ",fd->format,fd->offset, fd->length20 );
	DPAA_LOG(LOG_DEBUG_DATA_FLOW,"FD CMD=%x  STATUS=%x ",fd->cmd, fd->status );
	DPAA_LOG(LOG_DEBUG_DATA_FLOW,"============ END OF FD DUMP ============");
}

/*****************************************************************************/
/*  Function Name	: qman_handle_err_interrupt
 *	Input			: 
 *	Description		: Calls the QM Portal API 
 *	Returns			: 
 *	Note			:				
 */
/*****************************************************************************/
void qman_handle_err_interrupt(qman_handle_t hdl, clear_irq_cause_t *req){

	qman_t *qman= (qman_t*)hdl;
	LOG_FUNCTION_ENTRY;
 	
	QM_ErrorIsr(qman->h_Qm);
	
	LOG_FUNCTION_EXIT;
		
}

int
qman_get_counter(qman_handle_t hdl, uint32_t cntr) {
	return QM_GetCounter(((qman_t *)hdl)->h_Qm, cntr);
}

int
qman_query_fq_p(qman_handle_t hdl, uint32_t index, uint32_t fqid, uint32_t nbuf, void *buf) {
	return QM_FQR_QueryFq_P(((qman_t *)hdl)->h_QmPortal[index], fqid, buf);
}

int
qman_query_fq_np(qman_handle_t hdl, uint32_t index, uint32_t fqid, uint32_t nbuf, void *buf) {
	return QM_FQR_QueryFq_NP(((qman_t *)hdl)->h_QmPortal[index], fqid, buf);
}

int
qman_query_wq(qman_handle_t hdl, uint32_t index, uint16_t wqid, uint32_t nbuf, void *buf) {
	return QM_FQR_QueryWq(((qman_t *)hdl)->h_QmPortal[index], wqid, buf);
}
