/*
 * $QNXLicenseC:
 * Copyright 2010, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/

#include <sys/mman.h>
#include <stdio.h>
#include "platform_p4080_ds.h"

static t_Platform *g_p_Platform = NULL;

#define GET_ETH_PORT_ID(module,id)                                      \
    ((module == e_SYS_SUBMODULE_FM_PORT_10GMAC) && (id == 0)) ? 4 :     \
	    ( ((module == e_SYS_SUBMODULE_FM_PORT_10GMAC) && (id == 1)) ? 9 :   \
		    ( (id >= 4 && id <= 7) ? (id + 1) : id))     


static uint32_t GetVirtualBaseAddr(t_Platform *p_Platform, e_ModuleId module)
{
		/* Default handling for others using offsets from CCSR base */
		return p_Platform->baseAddress.ccsrBaseAddress + (P4080_GetModulePhysBase(p_Platform->h_Part, module) - p_Platform->baseAddress.ccsrPhysBase);

}

t_Error SYS_ForceHandle(e_SysModule module, uint32_t id, t_Handle h_Module)
{
	if ( ( module == e_SYS_SUBMODULE_FM_PORT_10GMAC )  &&
		 ( id < FM_MAX_NUM_OF_10G_MACS*INTG_MAX_NUM_OF_FM ) ) {
		if(g_p_Platform) {
			g_p_Platform->h_10GMiiAcc[id] = h_Module;
		} else {
	       RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("g_p_Platform"));
		}
	} else if ( ( module == e_SYS_SUBMODULE_FM_PORT_1GMAC )  &&
		 ( id < FM_MAX_NUM_OF_1G_MACS*INTG_MAX_NUM_OF_FM ) ) {
		if(g_p_Platform) {
			g_p_Platform->h_1GMiiAcc[id] = h_Module;
		} else {
	       RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("g_p_Platform"));
		}
	} else {
	       RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("invalid module or id"));
	}
    return E_OK;
}

/*****************************************************************************/
static void PlatformFreeResources(t_Platform *p_Platform)
{

    ASSERT_COND(p_Platform);


	munmap_device_memory((void*)p_Platform->baseAddress.ccsrBaseAddress, 16*MEGABYTE);
	munmap_device_memory((void*)p_Platform->baseAddress.portalsBMCEBaseAddress, 1*MEGABYTE);
	munmap_device_memory((void*)p_Platform->baseAddress.portalsBMCIBaseAddress, 1*MEGABYTE);
	munmap_device_memory((void*)p_Platform->baseAddress.portalsQMCEBaseAddress, 1*MEGABYTE);
	munmap_device_memory((void*)p_Platform->baseAddress.portalsQMCIBaseAddress, 1*MEGABYTE);

    /* Free MDS structure */
    if (p_Platform->h_Mds)
    {
        P4080DS_Free(p_Platform->h_Mds);
        p_Platform->h_Mds = NULL;
    }

    /* Free P4080 structure */
    if (p_Platform->h_Part)
    {
        P4080_Free(p_Platform->h_Part);
        p_Platform->h_Part = NULL;
    }

    /* Finally, free the platform structure */
    XX_Free(p_Platform);
}

/*****************************************************************************/
static t_Error PlatformInitGpio(t_Platform *p_Platform)
{
    t_GpioParam    gpioParam;

    ASSERT_COND(p_Platform);

	gpioParam.baseAddr = GetVirtualBaseAddr(p_Platform, e_MODULE_ID_GPIO);

    //#warning "add here irq for the GPIO from the QNX sys"
    gpioParam.irq[0] = NO_IRQ;

    p_Platform->h_Gpio = GPIO_ConfigAndInit(&gpioParam);
    if (!p_Platform->h_Gpio)
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("GPIO object"));
    }

    return E_OK;
}

/*****************************************************************************/
static void PlatformQueryEthernetSetup(t_Platform   *p_Platform,
                                       e_SysModule  module,
                                       uint32_t     id,
                                       bool         *p_UseAutoNegotiation,
                                       e_DiagMode   *p_LoopbackMode,
                                       bool         *p_IsForceMaster)
{
    t_PlatformBoardSettings *p_BoardSettings;

    ASSERT_COND(p_Platform);

    p_BoardSettings = &(p_Platform->boardSettings);
    *p_UseAutoNegotiation = p_BoardSettings->ethPhySettings.useAutoNegotiation;
    *p_IsForceMaster = (id ? FALSE : TRUE);
    *p_LoopbackMode = p_BoardSettings->diagModes[GET_ETH_PORT_ID(module, id)];
}

/*****************************************************************************/
t_Error PlatformSetIoPinForEthernet(t_Platform      *p_Platform,
                                           e_SysModule     module,
                                           uint32_t        id,
                                           bool            sgmii)
{
    t_P4080DevParams    *p_Devs;

    ASSERT_COND(p_Platform);

    p_Devs = P4080_GetDevicesConfiguration(p_Platform->h_Part);
	
    if ((module == e_SYS_SUBMODULE_FM_PORT_1GMAC && (id >= 0 && id <= 3)) && !sgmii &&
        ((p_Devs->fms[0].dtsecs[id].en && !p_Devs->fms[0].dtsecs[id].sgmii) ||
         (p_Devs->fms[0].dtsecs[id].en && !p_Devs->fms[0].dtsecs[id].sgmii)))
    {
        /* ---------------------- */
        /* Configure address pins */
        /* ---------------------- */
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        /* ----------------------- */
        /* Set address pins output */
        /* ----------------------- */
        /* On board FM1 - RGMII */
        IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
        IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
    }
    else if ((module == e_SYS_SUBMODULE_FM_PORT_1GMAC && (id >= 4 && id <= 7)) && !sgmii &&
             p_Devs->fms[1].dtsecs[id-4].en && !p_Devs->fms[1].dtsecs[id-4].sgmii)
    {
        /* ---------------------- */
        /* Configure address pins */
        /* ---------------------- */
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        /* ----------------------- */
        /* Set address pins output */
        /* ----------------------- */
        /* On board FM2 - RGMII */
        IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
        IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
    }
    else if ((module == e_SYS_SUBMODULE_FM_PORT_1GMAC && (id >= 0 && id <= 3)) && sgmii &&
             p_Devs->fms[0].dtsecs[id].en && p_Devs->fms[0].dtsecs[id].sgmii)
    {
        /* ---------------------- */
        /* Configure address pins */
        /* ---------------------- */
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        /* ----------------------- */
        /* Set address pins output */
        /* ----------------------- */
        /* SLOT 5 - FM1 SGMII */
        IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
        IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
    }
    else if ((module == e_SYS_SUBMODULE_FM_PORT_1GMAC && (id >= 4 && id <= 7)) && sgmii &&
             p_Devs->fms[1].dtsecs[id-4].en && p_Devs->fms[1].dtsecs[id-4].sgmii)
    {
        /* ---------------------- */
        /* Configure address pins */
        /* ---------------------- */
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        /* ----------------------- */
        /* Set address pins output */
        /* ----------------------- */
        if (p_Devs->fms[1].dtsecs[id-4].serdesBank == 1)
        {
            /* SLOT 3 - FM2 SGMII */
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
        }
        else if (p_Devs->fms[1].dtsecs[id-4].serdesBank == 2)
        {
            /* SLOT 4 - FM2 SGMII */
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
        }
        else
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("serdes bank"));
    }
    else if ((module == e_SYS_SUBMODULE_FM_PORT_10GMAC) && (id == 0) &&
             p_Devs->fms[0].tgec.en)
    {
        /* ---------------------- */
        /* Configure address pins */
        /* ---------------------- */
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 2, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 3, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        /* ----------------------- */
        /* Set address pins output */
        /* ----------------------- */
        if (p_Devs->fms[0].tgec.serdesBank == 3)
        {
            /* SLOT 5 - FM1 XAUI */
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 2, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 3, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
        }
        else
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("serdes bank"));
    }
    else if ((module == e_SYS_SUBMODULE_FM_PORT_10GMAC) && (id == 1) &&
            p_Devs->fms[1].tgec.en)
    {
        /* ---------------------- */
        /* Configure address pins */
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 2, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ConfigurePin(p_Platform->h_Gpio, IO_PIN(0, 3, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        /* ----------------------- */
        /* Set address pins output */
        /* ----------------------- */
        if (p_Devs->fms[1].tgec.serdesBank == 2)
        {
            /* SLOT 4 - FM2 XAUI */
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 2, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),0);
            IO_PORT_SetPinOutput(p_Platform->h_Gpio, IO_PIN(0, 3, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE),1);
        }
        else
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("serdes bank"));
    }
    else{

        RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, NO_MSG);
		}

    XX_UDelay(200000);

    return E_OK;
}

/*****************************************************************************/
t_Error PlatformClearIoPinForEthernet(t_Platform      *p_Platform,
                                             e_SysModule     module,
                                             uint32_t        id)
{
    ASSERT_COND(p_Platform);

    if (module == e_SYS_SUBMODULE_FM_PORT_1GMAC)
    {
        /* ---------------------- */
        /* Release address pins */
        /* ---------------------- */
        IO_PORT_ReleasePin(p_Platform->h_Gpio, IO_PIN(0, 0, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ReleasePin(p_Platform->h_Gpio, IO_PIN(0, 1, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
    }
    else if (module == e_SYS_SUBMODULE_FM_PORT_10GMAC)
    {
        /* ---------------------- */
        /* Release address pins */
        /* ---------------------- */
        IO_PORT_ReleasePin(p_Platform->h_Gpio, IO_PIN(0, 2, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
        IO_PORT_ReleasePin(p_Platform->h_Gpio, IO_PIN(0, 3, e_IO_PIN_ASSIGN_GPIO, e_IO_PIN_DIR_OUTPUT, FALSE));
    }
    else
        RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, NO_MSG);

    return E_OK;
}

/*****************************************************************************/
static t_Error PlatformSetSerdesLoopbackForEthernet(t_Platform *p_Platform, e_SysModule module, uint32_t id)
{
    /* Init SerDes for loopback */
    t_P4080DevParams    *p_Devs;
    uint64_t            srdsBaseAddr;
    uint8_t             serdesBank;
    int                 i;

    ASSERT_COND(p_Platform);

    p_Devs = P4080_GetDevicesConfiguration(p_Platform->h_Part);
    if (module == e_SYS_SUBMODULE_FM_PORT_10GMAC)
        serdesBank = p_Devs->fms[id].tgec.serdesBank;
    else if ((id >= 0) && (id <= 3))
        serdesBank = p_Devs->fms[0].dtsecs[id].serdesBank;
    else
        serdesBank = p_Devs->fms[1].dtsecs[id-4].serdesBank;

    switch (serdesBank)
    {
        case 1:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x500;
            break;
        case 2:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x800;
            break;
        case 3:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x900;
            break;
        default:
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("SerDes bank"));
    }
    for (i = 0; i < 4; i++, srdsBaseAddr += 0x40) {
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x34)),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x34))) & ~0x07000000) | 0x01000000));
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x10)),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x10))) & ~0x0000203f) | 0x00002000));
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x3c)),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x3c))) & ~0x30000000) | 0x10000000));
    }

    XX_UDelay(200000);

    return E_OK;
}

/*****************************************************************************/
static t_Error PlatformResetSerdesLaneReceivers(t_Platform *p_Platform, uint8_t serdesBank)
{
    uint64_t    srdsBaseAddr,tmpSerdesAddr;
    int         i;

    ASSERT_COND(p_Platform);

    switch (serdesBank)
    {
        case 1:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x500;
            break;
        case 2:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x800;
            break;
        case 3:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x900;
            break;
        default:
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("SerDes bank"));
    }

    for (i = 0,tmpSerdesAddr = srdsBaseAddr; i < 4; i++, tmpSerdesAddr += 0x40)
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, tmpSerdesAddr),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, tmpSerdesAddr )) & ~0x00400000)));

    XX_UDelay(200000);

    for (i = 0,tmpSerdesAddr = srdsBaseAddr; i < 4; i++, tmpSerdesAddr += 0x40)
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, tmpSerdesAddr),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, tmpSerdesAddr)) & ~0x00400000) | 0x00400000));

    return E_OK;
}

/*****************************************************************************/
static t_Error PlatformClearSerdesLoopbackForEthernet(t_Platform *p_Platform, e_SysModule module, uint32_t id)
{
    /* Clear SerDes internal loopback */
    t_P4080DevParams    *p_Devs;
    uint64_t            srdsBaseAddr;
    uint8_t             serdesBank;
    int                 i;

    ASSERT_COND(p_Platform);

    p_Devs = P4080_GetDevicesConfiguration(p_Platform->h_Part);
    if (module == e_SYS_SUBMODULE_FM_PORT_10GMAC)
        serdesBank = p_Devs->fms[id].tgec.serdesBank;
    else if ((id >= 0) && (id <= 3))
        serdesBank = p_Devs->fms[0].dtsecs[id].serdesBank;
    else
        serdesBank = p_Devs->fms[1].dtsecs[id-4].serdesBank;

    switch (serdesBank)
    {
        case 1:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x500;
            break;
        case 2:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x800;
            break;
        case 3:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x900;
            break;
        default:
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("SerDes bank"));
    }
    for (i = 0; i < 4; i++, srdsBaseAddr += 0x40)
    {
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x34)),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x34))) & ~0x07000000)));
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x10)),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x10))) & ~0x0000203f) | 0x0000001f));
        WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x3c)),
                     ((GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x3c))) & ~0x30000000)));
    }

    PlatformResetSerdesLaneReceivers(p_Platform,serdesBank);

    XX_UDelay(200000);

    return E_OK;
}

static __inline__ void PlatformSetXAUIErrata(t_Platform *p_Platform, uint8_t fmIndex)
{
    uint64_t    srdsBaseAddr;

    ASSERT_COND(p_Platform);

    PlatformSetSerdesLoopbackForEthernet(p_Platform,e_SYS_SUBMODULE_FM_PORT_10GMAC,fmIndex);
    do {
        srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x0e8;
        if (fmIndex == 0)
            WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr),
                         (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr)) & ~0x00800000));
        else
            WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr),
                         (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr)) & ~0x00400000));

        XX_UDelay(1000);
        if (fmIndex == 0)
            WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr),
                         (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr)) & ~0x00800000) | 0x00800000);
        else
            WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr),
                         (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr)) & ~0x00400000) | 0x00400000);
        XX_UDelay(1);
        if (fmIndex == 0)
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x250;
        else
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x260;
    } while (!(GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr)) & 0x40000000));

    PlatformClearSerdesLoopbackForEthernet(p_Platform,e_SYS_SUBMODULE_FM_PORT_10GMAC,fmIndex);

    PlatformResetSerdesLaneReceivers(p_Platform,(fmIndex?2:3));
}

#ifndef DISABLE_UNUSED_CODE
/*****************************************************************************/
static t_Error PlatformSetSerdesErratas(t_Platform *p_Platform)
{
    t_P4080DevParams    *p_Devs;
    uint64_t            srdsBaseAddr;
    int                 i;

    ASSERT_COND(p_Platform);

    p_Devs = P4080_GetDevicesConfiguration(p_Platform->h_Part);

    /* ERRATA : [SERDES2,SERDES6],[SERDES3] */
#define SGMII_ERRATA    \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x04)),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x04))) & ~0x041f0000) | 0x40000); \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x18)),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x18))) & ~0x30000000) | 0x10000000);

/* ERRATA : [SERDES2,SERDES6],[SERDES3] */
#define XAUI_ERRATA     \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x04)),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x04))) & ~0x041f0000));  \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x18)),    \
                  (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x18))) & ~0x30000000) | 0x10000000);

#define SERDES4_MULTILINE(bank)   \
    srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0);   \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x840:0x940))),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x840:0x940)))) & ~0x00010000));  \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x880:0x980))),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x880:0x980)))) & ~0x00010000));  \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x8c0:0x9c0))),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x8c0:0x9c0)))) & ~0x00010000));  \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x800:0x900))),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x800:0x900)))) & ~0x00010000) | 0x00010000);  \
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x20:0x40))),    \
                (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x20:0x40)))) & ~0x80000000) | 0x80000000);  \
    while (!(GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + ((bank==2)?0x20:0x40)))) & 0x40000000));

/* ERRATA: SERDES - check pll lock */
#define CHECK_PLL_LOCK     \
    srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0);   \
    if (!(GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x4))) & 0x00800000))  \
        RETURN_ERROR(MINOR,E_INVALID_STATE,("Bank 1 : no PLL lock")); \
    if (!(GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x24))) & 0x00800000)) \
        RETURN_ERROR(MINOR,E_INVALID_STATE,("Bank 2 : no PLL lock")); \
    if (!(GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, (srdsBaseAddr + 0x44))) & 0x00800000)) \
        RETURN_ERROR(MINOR,E_INVALID_STATE,("Bank 3 : no PLL lock"));

    CHECK_PLL_LOCK
    if (p_Devs->fms[0].dtsecs[0].en &&
         p_Devs->fms[0].dtsecs[0].sgmii)
    {
        /* workaround for bank 3 - SGMII*/
        srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x900;
        for (i = 0; i < 4; i++, srdsBaseAddr += 0x40)
            SGMII_ERRATA;
    }
    if (p_Devs->fms[0].tgec.en)
    {
        /* workaround for bank 3 - XAUI*/
        srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x900;
        for (i = 0; i < 4; i++, srdsBaseAddr += 0x40)
            XAUI_ERRATA;
        SERDES4_MULTILINE(3);
        PlatformSetXAUIErrata(p_Platform,0);
    }

    if ((p_Devs->fms[1].dtsecs[2].en &&
         p_Devs->fms[1].dtsecs[2].sgmii))
    {
        if ((p_Devs->fms[1].dtsecs[2].en) &&
            (p_Devs->fms[1].dtsecs[2].serdesBank == 1))
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x500;
        else if (((p_Devs->fms[1].dtsecs[2].en) &&
                  (p_Devs->fms[1].dtsecs[2].serdesBank == 2)))
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x800;
        else
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("serdes bank"));
        for (i = 0; i < 4; i++, srdsBaseAddr += 0x40)
        {
            if ((p_Devs->fms[1].dtsecs[2].serdesLane == e_SERDES_LANE_G_H) && (i<2))
                continue;
            SGMII_ERRATA;
        }
    }
    if (p_Devs->fms[1].tgec.en)
    {
        /* workaround for bank 2 - XAUI*/
        srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x800;
        for (i = 0; i < 4; i++, srdsBaseAddr += 0x40)
            XAUI_ERRATA;
        SERDES4_MULTILINE(2);
        PlatformSetXAUIErrata(p_Platform,1);
    }
    return E_OK;
}

/*****************************************************************************/
static t_Error PlatformSerdesBankDisable(t_Platform *p_Platform, uint8_t serdesBank, bool disable)
{
    uint64_t            srdsBaseAddr;

    ASSERT_COND(p_Platform);

    switch (serdesBank)
    {
        case 1:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0);
            break;
        case 2:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x20;
            break;
        case 3:
            srdsBaseAddr = PLATFORM_GetMemoryMappedModuleBase(p_Platform, e_SYS_SUBMODULE_SERDES, 0) + 0x40;
            break;
        default:
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("SerDes bank"));
    }
    WRITE_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr),
                 (GET_UINT32(*CAST_UINT64_TO_POINTER_TYPE(volatile uint32_t, srdsBaseAddr)) & ~0x00000020) |
                ((disable) ? 0x00000020 : 0x0));

    XX_UDelay(200000);

    return E_OK;
}

/*****************************************************************************/
static t_Error PlatformDisableUnusedSerdesBanks(t_Platform *p_Platform)
{
    ASSERT_COND(p_Platform);

    PlatformSerdesBankDisable(p_Platform,1,TRUE);

    return E_OK;
}
#endif

/*****************************************************************************/
static t_Handle PlatformGetMiiAcc(t_Platform        *p_Platform,
                                  e_SysModule       module,
                                  uint32_t          id,
                                  bool              force)
{
    t_Handle h_MiiAcc = NULL;

    ASSERT_COND(p_Platform);

    if ( (module == e_SYS_SUBMODULE_FM_PORT_1GMAC && (id >= 0 && id <= 3)) ||
         (module == e_SYS_SUBMODULE_FM_PORT_1GMAC && (id >= 4 && id <= 7)) )
    {
        h_MiiAcc = p_Platform->h_the1GMiiAcc;
        if (!h_MiiAcc || force)
        {
            ASSERT_COND(force || id == 0);
            h_MiiAcc = p_Platform->h_1GMiiAcc[id];
            p_Platform->h_the1GMiiAcc = h_MiiAcc;
        }
    }
    else if ( (module == e_SYS_SUBMODULE_FM_PORT_10GMAC && id == 0) ||
              (module == e_SYS_SUBMODULE_FM_PORT_10GMAC && id == 1) )
    {
        h_MiiAcc = p_Platform->h_the10GMiiAcc;
        if (!h_MiiAcc || force)
        {
            ASSERT_COND(force || id == 0);
            h_MiiAcc = p_Platform->h_10GMiiAcc[id];
            p_Platform->h_the10GMiiAcc = h_MiiAcc;
        }
    }
    ASSERT_COND(h_MiiAcc);

    return h_MiiAcc;
}


t_Error PLATFORM_GetChipRevInfo(t_Handle h_Platform, t_ChipRevInfo *p_ChipRevInfo)
{
    t_Platform          *p_Platform = (t_Platform *)h_Platform;
    e_P4080DeviceName deviceName;

    SANITY_CHECK_RETURN_ERROR(p_Platform, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_ChipRevInfo, E_NULL_POINTER);

    deviceName = P4080_GetRevInfo(p_Platform->h_Part);

    switch (deviceName)
    {
        case e_P4080E_REV_1_0:
            sprintf(p_ChipRevInfo->chipName, "P4080");
            p_ChipRevInfo->revMajor = 1;
            p_ChipRevInfo->revMinor = 0;
            break;
        case e_P4080E_REV_2_0:
            sprintf(p_ChipRevInfo->chipName, "P4080");
            p_ChipRevInfo->revMajor = 2;
            p_ChipRevInfo->revMinor = 0;
            break;
        default:
            RETURN_ERROR(MAJOR, E_INVALID_ID, ("Unknown chip ID"));
    }

    return E_OK;
}


#if 0
bool P4080DS_IsValidSerDesConfiguration (uint8_t val)
{
    switch (val)
	{
	   case 0x06:
	   case 0x1B:
	   case 0x07:
	   case 0x1C:
	   case 0x08:
	   case 0x1d:
	   case 0x10:

	   case 0x24:   /* SJ: I see this configuration on bootup, adding this here, but not sure  of the implications, yet.*/
	       return TRUE;
	   default:
	       REPORT_ERROR(MAJOR, E_NOT_SUPPORTED, NO_MSG);
	   }
    return FALSE;
}
#endif

uint32_t SYS_GetMemoryMappedModuleBase(e_SysModule module, uint32_t id)
{
	assert(g_p_Platform != NULL);
    return PLATFORM_GetMemoryMappedModuleBase(g_p_Platform,
	                                              module,
                                               id);
}


t_Error SYS_InitEthernetPhy(e_SysModule module, uint32_t id,e_EnetMode ethMode,e_EnetDuplexMode  duplexMode)
{
	assert(g_p_Platform != NULL);
	return PLATFORM_InitEthernetPhy(g_p_Platform, module, id, ethMode, duplexMode);
}

t_Error SYS_GetChipRevInfo(t_ChipRevInfo *p_ChipRevInfo)
{
	assert(g_p_Platform != NULL);
	PLATFORM_GetChipRevInfo(g_p_Platform,p_ChipRevInfo);
	return E_OK;
}

t_Handle PLATFORM_Init(uint64_t ccsrbase, uint64_t bm_portalbase, uint64_t qm_portalbase, uint64_t pixisbase)
{
	t_Platform *p_Platform;
	t_P4080Params p4080Params;
	t_P4080DsParams p4080DsParams;
	t_Error errCode;

	if(g_p_Platform  != NULL) {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Platform already initialized"));
		return NULL;
	}
	/* Allocate the platform's control structure */
	p_Platform = XX_Malloc(sizeof(t_Platform));
	if (!p_Platform)
	{
		REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Platform structure"));
		return NULL;
	}
	memset(p_Platform, 0, sizeof(t_Platform));
	//p_Platform->rmiiPhyResetDone = FALSE;

	p_Platform->baseAddress.ccsrPhysBase = ccsrbase;
	p_Platform->baseAddress.pixisPhysBase = pixisbase;
	p_Platform->baseAddress.bmPortalBase = bm_portalbase;
	p_Platform->baseAddress.qmPortalBase = qm_portalbase;

	/*------------------------------------------------------*/
	/* Initialize Part                                      */
	/*------------------------------------------------------*/

	p_Platform->baseAddress.ccsrBaseAddress = CAST_POINTER_TO_UINT64(mmap_device_memory(NULL, 16*MEGABYTE, 
		PROT_READ|PROT_WRITE|PROT_NOCACHE,MAP_SHARED,ccsrbase));
	p_Platform->baseAddress.pixisBaseAddress = CAST_POINTER_TO_UINT64(mmap_device_memory(NULL, 4*KILOBYTE, 
		PROT_READ|PROT_WRITE|PROT_NOCACHE,MAP_SHARED,pixisbase));

	/* Map the portals */

	/* BM Cache enabled */	
	p_Platform->baseAddress.portalsBMCEBaseAddress = CAST_POINTER_TO_UINT64(mmap_device_memory(NULL, 1*MEGABYTE, 
		PROT_READ|PROT_WRITE,MAP_SHARED,bm_portalbase)); 
	/* BM Cache inhibited */	
        p_Platform->baseAddress.portalsBMCIBaseAddress = CAST_POINTER_TO_UINT64(mmap_device_memory(NULL, 1*MEGABYTE, 
		PROT_READ|PROT_WRITE|PROT_NOCACHE,MAP_SHARED,bm_portalbase + 1*MEGABYTE)); 
		
	/* QM Cache enabled */	
        p_Platform->baseAddress.portalsQMCEBaseAddress = CAST_POINTER_TO_UINT64(mmap_device_memory(NULL, 1*MEGABYTE, 
		PROT_READ|PROT_WRITE,MAP_SHARED,qm_portalbase)); 
		
	/* QM Cache inhibited */	
        p_Platform->baseAddress.portalsQMCIBaseAddress = CAST_POINTER_TO_UINT64(mmap_device_memory(NULL, 1*MEGABYTE, 
		PROT_READ|PROT_WRITE|PROT_NOCACHE,MAP_SHARED,qm_portalbase + 1*MEGABYTE)); 
		

	if(	p_Platform->baseAddress.ccsrBaseAddress    == NULL || 
		p_Platform->baseAddress.pixisBaseAddress   == NULL ||
		p_Platform->baseAddress.portalsBMCEBaseAddress == NULL ||
		p_Platform->baseAddress.portalsBMCIBaseAddress == NULL ||
		p_Platform->baseAddress.portalsQMCEBaseAddress == NULL ||
		p_Platform->baseAddress.portalsQMCIBaseAddress == NULL) {

		REPORT_ERROR(MAJOR, E_NULL_POINTER, ("mmap_device_memory "));
	}
	
	p4080Params.ccsrBaseAddress = p_Platform->baseAddress.ccsrBaseAddress;
	p4080Params.bmPortalsBaseAddress = p_Platform->baseAddress.portalsBMCEBaseAddress;
	p4080Params.qmPortalsBaseAddress = p_Platform->baseAddress.portalsQMCEBaseAddress;
	p4080Params.f_BoardIsValidSerDesConfigurationCB = P4080DS_IsValidSerDesConfiguration;
	p_Platform->boardSettings.ethPhySettings.useAutoNegotiation = TRUE;
	
	p_Platform->boardSettings.diagModes[0] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[1] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[2] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[3] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[4] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[5] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[6] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[7] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[8] = e_DIAG_MODE_NONE;
	p_Platform->boardSettings.diagModes[9] = e_DIAG_MODE_NONE;

	#if 0
	p_Platform->clockInFreqHz= 100000000;
	p_Platform->l1CacheMode= e_CACHE_MODE_DATA_AND_INST ;
	p_Platform->l2CacheMode = e_CACHE_MODE_DATA_AND_INST;
	p_Platform->l3CacheMode= e_CACHE_MODE_DISABLED ;
	p_Platform->l3SramSize= 0 ;
	p_Platform->l3StashMode= e_L2_L2CTL_NO_STASH ;
	#endif
    /* P4080 database initialization */
	p_Platform->h_Part = P4080_ConfigAndInit(&p4080Params);
	if (!p_Platform->h_Part)
	{
		REPORT_ERROR(MAJOR, E_NOT_AVAILABLE, ("P4080 object"));
		PlatformFreeResources(p_Platform);
		return NULL;
	}

    /*------------------------------------------------------*/
    /* Initialize MDS                                       */
    /*------------------------------------------------------*/
    memset(&p4080DsParams, 0, sizeof(p4080DsParams));
    p4080DsParams.pixisBaseAddr = p_Platform->baseAddress.pixisBaseAddress;
    p_Platform->h_Mds = P4080DS_ConfigAndInit(&p4080DsParams);
    if (!p_Platform->h_Mds)
    {
        REPORT_ERROR(MAJOR, E_NOT_AVAILABLE, ("P4080-MDS"));
        PlatformFreeResources(p_Platform);
        return NULL;
    }

    /*------------------------------------------------------*/
    /* Initialize GPIO                                      */
    /*------------------------------------------------------*/
    errCode = PlatformInitGpio(p_Platform);
    if (errCode != E_OK)
    {
        REPORT_ERROR(MAJOR, errCode, NO_MSG);
        PlatformFreeResources(p_Platform);
        return NULL;
    }

#if !defined (__QNXNTO__)
#ifdef SERDES_ERRATA
    errCode = PlatformSetSerdesErratas(p_Platform);
    PlatformDisableUnusedSerdesBanks(p_Platform);
    if (errCode != E_OK)
    {
        REPORT_ERROR(MAJOR, errCode, NO_MSG);
        PlatformFreeResources(p_Platform);
        return NULL;
    }
#endif /* SERDES_ERRATA */
#endif
	g_p_Platform = p_Platform;
	return p_Platform;
}

t_Error PLATFORM_Free(t_Handle h_Platform)
{
    t_Platform  *p_Platform = (t_Platform *)h_Platform;
	PlatformFreeResources(p_Platform);
	return E_OK;
}

/*****************************************************************************/
uint32_t PLATFORM_GetMemoryMappedModuleBase(t_Handle    h_Platform,
                                            e_SysModule module,
                                            uint32_t    id)
{
    t_Platform  *p_Platform = (t_Platform *)h_Platform;
    uint64_t    baseAddress;
    int         i;


    t_SysToPartModuleMap sysToPartModuleMap[] =
    {
        { e_SYS_SUBMODULE_DUART,        0,  e_MODULE_ID_DUART_1         },
        { e_SYS_SUBMODULE_DUART,        1,  e_MODULE_ID_DUART_2         },
        { e_SYS_SUBMODULE_DUART,        2,  e_MODULE_ID_DUART_3         },
        { e_SYS_SUBMODULE_DUART,        3,  e_MODULE_ID_DUART_4         },
        { e_SYS_SUBMODULE_EPIC,         0,  e_MODULE_ID_MPIC            },
        { e_SYS_SUBMODULE_GPIO,         0,  e_MODULE_ID_GPIO            },
        { e_SYS_SUBMODULE_SERDES,       0,  e_MODULE_ID_SERDES          },
        { e_SYS_SUBMODULE_QM,           0,  e_MODULE_ID_QM              },
        { e_SYS_SUBMODULE_BM,           0,  e_MODULE_ID_BM              },
#if 0
        { e_SYS_SUBMODULE_BM_CE_PORTAL, 0,  e_MODULE_ID_BM_CE_PORTAL    },
        { e_SYS_SUBMODULE_BM_CI_PORTAL, 0,  e_MODULE_ID_BM_CI_PORTAL    },
        { e_SYS_SUBMODULE_QM_CE_PORTAL, 0,  e_MODULE_ID_QM_CE_PORTAL    },
        { e_SYS_SUBMODULE_QM_CI_PORTAL, 0,  e_MODULE_ID_QM_CI_PORTAL    },
#endif
        { e_SYS_SUBMODULE_FM,           0,  e_MODULE_ID_FM1             },
        { e_SYS_SUBMODULE_FM_RTC,       0,  e_MODULE_ID_FM1_RTC         },
        { e_SYS_SUBMODULE_FM_MURAM,     0,  e_MODULE_ID_FM1_MURAM       },
        { e_SYS_SUBMODULE_FM_PORT_HO,   0,  e_MODULE_ID_FM1_PORT_HO0    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   1,  e_MODULE_ID_FM1_PORT_HO1    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   2,  e_MODULE_ID_FM1_PORT_HO2    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   3,  e_MODULE_ID_FM1_PORT_HO3    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   4,  e_MODULE_ID_FM1_PORT_HO4    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   5,  e_MODULE_ID_FM1_PORT_HO5    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   6,  e_MODULE_ID_FM1_PORT_HO6    },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 0,  e_MODULE_ID_FM1_PORT_1GRx0  },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 1,  e_MODULE_ID_FM1_PORT_1GRx1  },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 2,  e_MODULE_ID_FM1_PORT_1GRx2  },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 3,  e_MODULE_ID_FM1_PORT_1GRx3  },
        { e_SYS_SUBMODULE_FM_PORT_10GRx,0,  e_MODULE_ID_FM1_PORT_10GRx0 },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 0,  e_MODULE_ID_FM1_PORT_1GTx0  },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 1,  e_MODULE_ID_FM1_PORT_1GTx1  },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 2,  e_MODULE_ID_FM1_PORT_1GTx2  },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 3,  e_MODULE_ID_FM1_PORT_1GTx3  },
        { e_SYS_SUBMODULE_FM_PORT_10GTx,0,  e_MODULE_ID_FM1_PORT_10GTx0 },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,0,  e_MODULE_ID_FM1_1GMAC0      },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,1,  e_MODULE_ID_FM1_1GMAC1      },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,2,  e_MODULE_ID_FM1_1GMAC2      },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,3,  e_MODULE_ID_FM1_1GMAC3      },
        { e_SYS_SUBMODULE_FM_PORT_10GMAC,0, e_MODULE_ID_FM1_10GMAC0     },
        { e_SYS_SUBMODULE_FM,           1,  e_MODULE_ID_FM2             },
        { e_SYS_SUBMODULE_FM_MURAM,     1,  e_MODULE_ID_FM2_MURAM       },
        { e_SYS_SUBMODULE_FM_RTC,       1,  e_MODULE_ID_FM2_RTC         },
        { e_SYS_SUBMODULE_FM_PORT_HO,   7,  e_MODULE_ID_FM2_PORT_HO0    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   8,  e_MODULE_ID_FM2_PORT_HO1    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   9,  e_MODULE_ID_FM2_PORT_HO2    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   10, e_MODULE_ID_FM2_PORT_HO3    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   11, e_MODULE_ID_FM2_PORT_HO4    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   12, e_MODULE_ID_FM2_PORT_HO5    },
        { e_SYS_SUBMODULE_FM_PORT_HO,   13, e_MODULE_ID_FM2_PORT_HO6    },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 4,  e_MODULE_ID_FM2_PORT_1GRx0  },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 5,  e_MODULE_ID_FM2_PORT_1GRx1  },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 6,  e_MODULE_ID_FM2_PORT_1GRx2  },
        { e_SYS_SUBMODULE_FM_PORT_1GRx, 7,  e_MODULE_ID_FM2_PORT_1GRx3  },
        { e_SYS_SUBMODULE_FM_PORT_10GRx,1,  e_MODULE_ID_FM2_PORT_10GRx0 },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 4,  e_MODULE_ID_FM2_PORT_1GTx0  },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 5,  e_MODULE_ID_FM2_PORT_1GTx1  },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 6,  e_MODULE_ID_FM2_PORT_1GTx2  },
        { e_SYS_SUBMODULE_FM_PORT_1GTx, 7,  e_MODULE_ID_FM2_PORT_1GTx3  },
        { e_SYS_SUBMODULE_FM_PORT_10GTx,1,  e_MODULE_ID_FM2_PORT_10GTx0 },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,4,  e_MODULE_ID_FM2_1GMAC0      },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,5,  e_MODULE_ID_FM2_1GMAC1      },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,6,  e_MODULE_ID_FM2_1GMAC2      },
        { e_SYS_SUBMODULE_FM_PORT_1GMAC,7,  e_MODULE_ID_FM2_1GMAC3      },
        { e_SYS_SUBMODULE_FM_PORT_10GMAC,1, e_MODULE_ID_FM2_10GMAC0     },
    };
	
	switch(module){

		case e_SYS_SUBMODULE_BM_CE_PORTAL:
			baseAddress= p_Platform->baseAddress.portalsBMCEBaseAddress + id* 0x4000; 
            	return baseAddress;
		break;

		case e_SYS_SUBMODULE_BM_CI_PORTAL:
			baseAddress= p_Platform->baseAddress.portalsBMCIBaseAddress + id* 0x1000;
            	return baseAddress;
		break;

		case e_SYS_SUBMODULE_QM_CE_PORTAL:
			baseAddress= p_Platform->baseAddress.portalsQMCEBaseAddress + id* 0x4000;
            	return baseAddress;
		break;

		case e_SYS_SUBMODULE_QM_CI_PORTAL:
			baseAddress= p_Platform->baseAddress.portalsQMCIBaseAddress + id* 0x1000;
            	return baseAddress;
		break;

    	default:

		for (i = 0; i < ARRAY_SIZE(sysToPartModuleMap); i++)
    	{
        	if ((sysToPartModuleMap[i].module == module) &&
            (sysToPartModuleMap[i].id == id))
        	{	
            	baseAddress = GetVirtualBaseAddr(p_Platform, sysToPartModuleMap[i].partModule);
                                                
            	return baseAddress;
        	}
    	}
		break;
	}

    REPORT_ERROR(MAJOR, E_NOT_FOUND, ("Module base"));
    return 0;
}

t_Error PLATFORM_GetDevicesConfiguration(t_Handle h_Platform, t_PlatformDevicesConf *p_DevicesConf)
{
    t_Platform          *p_Platform = (t_Platform *)h_Platform;
    t_P4080DevParams    *p_Devs = P4080_GetDevicesConfiguration(p_Platform->h_Part);
    int                 i, j;

    SANITY_CHECK_RETURN_ERROR(p_Platform, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DevicesConf, E_NULL_POINTER);

    memset(p_DevicesConf, 0, sizeof(t_PlatformDevicesConf));
    for (i=0; i<INTG_MAX_NUM_OF_FM; i++)
    {
        for (j=0; j<FM_MAX_NUM_OF_1G_MACS; j++)
            if (p_Devs->fms[i].dtsecs[j].en)
            {
                p_DevicesConf->fms[i].dtsecs[j].en          = p_Devs->fms[i].dtsecs[j].en;
                p_DevicesConf->fms[i].dtsecs[j].diagMode    = p_Platform->boardSettings.diagModes[(i * FM_MAX_NUM_OF_MACS) + j];
                p_DevicesConf->fms[i].dtsecs[j].phyModes    = (p_Devs->fms[i].dtsecs[j].sgmii) ?
                                                                PLATFORM_PHY_MODE_ANY :
                                                                (PLATFORM_PHY_MODE_ANY & ~PLATFORM_PHY_MODE_SGMII);
            }

        for (j=0; j<FM_MAX_NUM_OF_10G_MACS; j++)
            if (p_Devs->fms[i].tgec.en)
            {
                p_DevicesConf->fms[i].tgecs[j].en          = p_Devs->fms[i].tgec.en;
                p_DevicesConf->fms[i].tgecs[j].diagMode    = p_Platform->boardSettings.diagModes[(i * FM_MAX_NUM_OF_MACS) + FM_MAX_NUM_OF_1G_MACS + j];
            }
    }

    return E_OK;
}



/*****************************************************************************/
t_Error PLATFORM_MdioForceMaster(t_Handle h_Platform, e_ModuleId module, uint32_t id)
{
    t_Platform          *p_Platform = (t_Platform *)h_Platform;
    t_Handle            h_MiiAcc;

    h_MiiAcc = PlatformGetMiiAcc(p_Platform,module,id,TRUE);
    if (!h_MiiAcc)
        RETURN_ERROR(MINOR, E_INVALID_HANDLE, ("MII not found!"));
    return E_OK;
}

/*****************************************************************************/
t_Error PLATFORM_InitEthernetPhy(t_Handle           h_Platform,
                                 e_SysModule        module,
                                 uint32_t           id,
                                 e_EnetMode         ethMode,
                                 e_EnetDuplexMode   duplexMode)
{
    t_Platform          *p_Platform = (t_Platform *)h_Platform;
    t_Error             errCode = E_OK;
    t_Handle            h_MiiAcc;
    bool                autoNegotiation, forceMaster;
    e_DiagMode          loopbackMode;

    SANITY_CHECK_RETURN_ERROR(p_Platform, E_INVALID_HANDLE);

    if (p_Platform->boardSettings.diagModes[GET_ETH_PORT_ID(module, id)] == e_DIAG_MODE_CTRL_LOOPBACK)
        return E_OK;

    h_MiiAcc = PlatformGetMiiAcc(p_Platform,module,id,FALSE);

    p_Platform->ethPhyCounter++;

    if (p_Platform->boardSettings.diagModes[GET_ETH_PORT_ID(module, id)] == e_DIAG_MODE_CHIP_LOOPBACK)
        /* Init SerDes for loopback */
        return PlatformSetSerdesLoopbackForEthernet(p_Platform, module, id);

    if ((errCode = PlatformSetIoPinForEthernet(p_Platform, module, id, (ENET_INTERFACE_FROM_MODE(ethMode) == e_ENET_IF_SGMII))) != E_OK)
        RETURN_ERROR(MINOR, errCode, NO_MSG);

    /* Query board settings for relevant information */
    PlatformQueryEthernetSetup(p_Platform,
                               module,
                               id,
                               &autoNegotiation,
                               &loopbackMode,
                               &forceMaster);

    errCode = P4080DS_InitEthernetPhy(p_Platform->h_Mds,
                                      GET_ETH_PORT_ID(module,id),
                                      ethMode,
                                      autoNegotiation,
                                      duplexMode,
                                      forceMaster,
                                      loopbackMode,
                                      h_MiiAcc,
                                      (t_MiiPhyWriteFunc*)FM_MAC_MII_WritePhyReg,
                                      (t_MiiPhyReadFunc*)FM_MAC_MII_ReadPhyReg);

    PlatformClearIoPinForEthernet(p_Platform, module, id);

    if (errCode != E_OK)
        p_Platform->ethPhyCounter--;

    return errCode;
}

/*****************************************************************************/
t_Error PLATFORM_FreeEthernetPhy(t_Handle       h_Platform,
                                 e_SysModule    module,
                                 uint32_t       id)
{
    t_Platform          *p_Platform = (t_Platform *)h_Platform;
    e_EnetMode          ethMode;
    t_Error             errCode = E_OK;

    SANITY_CHECK_RETURN_ERROR(p_Platform, E_INVALID_HANDLE);

    ethMode = P4080DS_GetEnetMode(p_Platform->h_Mds, GET_ETH_PORT_ID(module,id));
    if (ethMode == e_ENET_MODE_INVALID)
        RETURN_ERROR(MAJOR, E_ALREADY_FREE, ("Ethernet PHY"));

    if ((errCode = PlatformSetIoPinForEthernet(p_Platform, module, id, (ENET_INTERFACE_FROM_MODE(ethMode) == e_ENET_IF_SGMII))) != E_OK)
        RETURN_ERROR(MINOR, errCode, NO_MSG);

    /* Free the matching PHY */
    ethMode = P4080DS_FreeEthernetPhy(p_Platform->h_Mds, GET_ETH_PORT_ID(module,id));

    PlatformClearIoPinForEthernet(p_Platform, module, id);

    p_Platform->ethPhyCounter--;

    return E_OK;
}

/*****************************************************************************/
bool PLATFORM_GetEthernetLinkStatus(t_Handle    h_Platform,
                                    e_SysModule module,
                                    uint32_t    id)
{
    t_Platform          *p_Platform = (t_Platform *)h_Platform;
    e_EnetMode          ethMode;
    t_Error             errCode = E_OK;

    SANITY_CHECK_RETURN_VALUE(p_Platform, E_INVALID_HANDLE, FALSE);

    ethMode = P4080DS_GetEnetMode(p_Platform->h_Mds, GET_ETH_PORT_ID(module,id));
    if (ethMode == e_ENET_MODE_INVALID)
        RETURN_ERROR(MAJOR, E_ALREADY_FREE, ("Ethernet PHY"));

    if ((errCode = PlatformSetIoPinForEthernet(p_Platform, module, id, (ENET_INTERFACE_FROM_MODE(ethMode) == e_ENET_IF_SGMII))) != E_OK)
        RETURN_ERROR(MINOR, errCode, NO_MSG);

    errCode = P4080DS_GetEthernetLinkStatus(p_Platform->h_Mds,
                                            GET_ETH_PORT_ID(module,id),
                                            PlatformGetMiiAcc(p_Platform,module,id,FALSE),
                                            (t_MiiPhyWriteFunc*)FM_MAC_MII_WritePhyReg,
                                            (t_MiiPhyReadFunc*)FM_MAC_MII_ReadPhyReg);

    PlatformClearIoPinForEthernet(p_Platform, module, id);

    return errCode;
}
