/*
 * $QNXLicenseC:
 * Copyright 2010, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/

#ifndef __PLATFORM_P4080_DS_H
#define __PLATFORM_P4080_DS_H

#include "std_ext.h"
#include "error_ext.h"
#include "enet_ext.h"
#include "sys_ext_enum.h"
#include "integrations/P4080/part_integration_ext.h"
#include "p4080_ds_ext.h"
#include "Peripherals/gpio_ext.h"
#include "Peripherals/fm_mac_ext.h"
#include "platform_settings.h"

#define __ERR_MODULE__  MODULE_UNKNOWN

#define IO_MODE_ANY     (-1)

#define PLATFORM_MAX_NUM_OF_CONNECTORS  10

#define PLATFORM_PHY_MODE_SGMII         0x80
#define PLATFORM_PHY_MODE_RGMII         0x40
#define PLATFORM_PHY_MODE_RTBI          0x20
#define PLATFORM_PHY_MODE_ANY           (PLATFORM_PHY_MODE_SGMII | \
                                         PLATFORM_PHY_MODE_RGMII | \
								            PLATFORM_PHY_MODE_RTBI)

typedef struct t_PlatformBoardSettings
{
	struct
	{
		bool                useAutoNegotiation;
	} ethPhySettings;
	e_DiagMode	diagModes[PLATFORM_MAX_NUM_OF_CONNECTORS];
} t_PlatformBoardSettings;

typedef struct t_MappedAddress
{
	uint32_t ccsrBaseAddress;
	uint32_t pixisBaseAddress;
	uint32_t portalsBMCEBaseAddress;
	uint32_t portalsBMCIBaseAddress;
	uint32_t portalsQMCEBaseAddress;
	uint32_t portalsQMCIBaseAddress;
	uint64_t ccsrPhysBase;
	uint64_t pixisPhysBase;
	uint64_t bmPortalBase;
	uint64_t qmPortalBase;
} t_MappedBaseAddress;


typedef struct t_Platform {
	t_Handle h_Part;
	t_Handle h_Mds;
	t_Handle h_Gpio;
	t_MappedBaseAddress baseAddress;
   /* MII-related variables */
   	t_Handle				h_the1GMiiAcc;
   	t_Handle				h_the10GMiiAcc;
    t_Handle                h_1GMiiAcc[FM_MAX_NUM_OF_1G_MACS*INTG_MAX_NUM_OF_FM];
	t_Handle                h_10GMiiAcc[FM_MAX_NUM_OF_10G_MACS*INTG_MAX_NUM_OF_FM];
	uint32_t                ethPhyCounter;
	t_PlatformBoardSettings boardSettings;
	} t_Platform;

typedef struct t_SysToPartModuleMap {
    e_SysModule module;
    uint32_t    id;
    e_ModuleId  partModule;
	} t_SysToPartModuleMap;


typedef struct t_PlatformDevicesConf{
    struct
    {
        struct
        {
            bool                    en;
            e_DiagMode              diagMode;
            uint8_t                 phyModes;
        } dtsecs[FM_MAX_NUM_OF_1G_MACS];
        struct
        {
            bool                    en;
            e_DiagMode              diagMode;
        } tgecs[FM_MAX_NUM_OF_10G_MACS];
    } fms[INTG_MAX_NUM_OF_FM];
} t_PlatformDevicesConf;

#define MAX_CHIP_NAME_LEN   10   /**< Maximum chip name length (including null character) */

/**************************************************************************//**
 @Description   Chip Type and Revision Information Structure
*//***************************************************************************/
typedef struct t_ChipRevInfo
{
    char        chipName[MAX_CHIP_NAME_LEN];
                /**< Chip name (e.g. "MPC8360E", "MPC8360") */
    uint16_t    revMajor;
                /**< Major chip revision */
    uint16_t    revMinor;
                /**< Minor chip revision */
} t_ChipRevInfo;


t_Error PLATFORM_GetChipRevInfo(t_Handle h_Platform, t_ChipRevInfo *p_ChipRevInfo);
t_Handle PLATFORM_Init(uint64_t ccsrbase, uint64_t bm_portalbase, uint64_t qm_portalbase, uint64_t pixisbase);
t_Error PLATFORM_Free(t_Handle h_Platform);
t_Error SYS_GetChipRevInfo(t_ChipRevInfo *p_ChipRevInfo);
t_Error SYS_ForceHandle(e_SysModule module, uint32_t id, t_Handle h_Module);

uint32_t SYS_GetMemoryMappedModuleBase(e_SysModule module, uint32_t id);
t_Error SYS_InitEthernetPhy(e_SysModule module, uint32_t id,e_EnetMode ethMode,e_EnetDuplexMode  duplexMode);

uint32_t PLATFORM_GetMemoryMappedModuleBase(t_Handle h_Platform, e_SysModule module, uint32_t id);
t_Error PLATFORM_MdioForceMaster(t_Handle h_Platform, e_ModuleId module, uint32_t id);
t_Error PLATFORM_InitEthernetPhy(t_Handle           h_Platform,
                                 e_SysModule        module,
                                 uint32_t           id,
                                 e_EnetMode         ethMode,
                                 e_EnetDuplexMode   duplexMode);

t_Error PLATFORM_GetDevicesConfiguration(t_Handle h_Platform, t_PlatformDevicesConf *p_DevicesConf);

t_Error PlatformSetIoPinForEthernet(t_Platform      *p_Platform,
                                           e_SysModule     module,
                                           uint32_t        id,
                                           bool            sgmii);

t_Error PlatformClearIoPinForEthernet(t_Platform      *p_Platform,
                                             e_SysModule     module,
                                             uint32_t        id);


#endif

