/*
 * $QNXLicenseC:
 * Copyright 2010, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

/* Derived from sys_ext.h. This is only to pull in the e_SysModule enum */


/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_EXT_H
#define __SYS_EXT_H



/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         sys_gen_grp     System General Interface

 @Description   General definitions, structures and routines of the bare-board
                system programming interface.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   System Modules and Sub-Modules Enumeration

                This enumeration type is expected by most system routines in
                order to identify the type of the referenced module or sub-module.
                The lower values of this enumeration type are modules, and the
                higher values are sub-modules.
*//***************************************************************************/
typedef enum e_SysModule
{
    e_SYS_MODULE_NONE = 0,

    /* ----------- Modules ----------- */
    e_SYS_MODULE_PLATFORM,
    e_SYS_MODULE_DEV_MNG,
    e_SYS_MODULE_DATA_POOL_MNG,
    e_SYS_MODULE_TRAFFIC_MNG,
    e_SYS_MODULE_APP,
    e_SYS_MODULE_EXTERNAL,
    /* Stacks */
    e_SYS_MODULE_PCI_STACK,
    e_SYS_MODULE_ESDHC_STACK,
    e_SYS_MODULE_RIO_STACK,
    e_SYS_MODULE_I2C_STACK,
    e_SYS_MODULE_SPI_STACK,
    e_SYS_MODULE_USB_STACK,
    e_SYS_MODULE_IMA_STACK,
    /* Wrappers */
    e_SYS_MODULE_WRP_ECM,
    e_SYS_MODULE_WRP_ARBITER,
    e_SYS_MODULE_WRP_DUART,
    e_SYS_MODULE_WRP_UART,
    e_SYS_MODULE_WRP_QE,
    e_SYS_MODULE_WRP_UPC,
    e_SYS_MODULE_WRP_UCC_ATM,
    e_SYS_MODULE_WRP_UCC_POS,
    e_SYS_MODULE_WRP_UCC_GETH,
    e_SYS_MODULE_WRP_UCC_HDLC,
    e_SYS_MODULE_WRP_UCC_TRNS,
    e_SYS_MODULE_WRP_UCC_UART,
    e_SYS_MODULE_WRP_QMC,
    e_SYS_MODULE_WRP_QMC_HDLC,
    e_SYS_MODULE_WRP_QMC_TRANS,
    e_SYS_MODULE_WRP_L2_SWITCH,
    e_SYS_MODULE_WRP_SI,
    e_SYS_MODULE_WRP_TDM,
    e_SYS_MODULE_WRP_MCC,
    e_SYS_MODULE_WRP_MCC_HDLC,
    e_SYS_MODULE_WRP_MCC_TRANS,
    e_SYS_MODULE_WRP_MCC_SS7,
    e_SYS_MODULE_WRP_PPPOHT,
    e_SYS_MODULE_WRP_IW_PPPOHT,
    e_SYS_MODULE_WRP_PCI,
    e_SYS_MODULE_WRP_ESDHC,
    e_SYS_MODULE_WRP_RIO,
    e_SYS_MODULE_WRP_I2C,
    e_SYS_MODULE_WRP_SPI,
    e_SYS_MODULE_WRP_ETSEC,
    e_SYS_MODULE_WRP_SRIO,
    e_SYS_MODULE_WRP_IMA,
    e_SYS_MODULE_WRP_MTC,
    e_SYS_MODULE_WRP_QE_IW,
    e_SYS_MODULE_WRP_QE_FILTERING,
    e_SYS_MODULE_WRP_QE_RTC,
    e_SYS_MODULE_WRP_VP,
    e_SYS_MODULE_WRP_GTIMERS,
    e_SYS_MODULE_WRP_RTC83,
    e_SYS_MODULE_WRP_DMA,
    e_SYS_MODULE_WRP_PTP,
    e_SYS_MODULE_WRP_IPSEC,

    /* Bus Device Drivers */
    e_SYS_MODULE_MPC8568_RIO_DEV,
    e_SYS_MODULE_MPC8568_PCI_DEV,
    e_SYS_MODULE_MPC836X_PCI_DEV,
    e_SYS_MODULE_MPC837x_PCI_DEV,
    e_SYS_MODULE_MPC832X_PCI_DEV,
    e_SYS_MODULE_MPC8568_PCIE_DEV,
    e_SYS_MODULE_MPC8569_PCIE_DEV,
    e_SYS_MODULE_MPC837x_PCIE_DEV,
    e_SYS_MODULE_MPC8568_SPI_FLASH_DEV,
    e_SYS_MODULE_MPC8568_SPI_LB_DEV,
    e_SYS_MODULE_SD_DEV,
    e_SYS_MODULE_SPD_I2C_DEV,
    e_SYS_MODULE_DS1374_I2C_DEV,
    e_SYS_MODULE_PCA9555_I2C_DEV,
    e_SYS_MODULE_M24256_I2C_DEV,
    e_SYS_MODULE_AT24C01A_I2C_DEV,

    /* Must close the modules list and open the sub-modules list */
    e_SYS_SUBMODULE_DUMMY_FIRST,

    /* --------- Sub-modules --------- */
    e_SYS_SUBMODULE_DATA_POOL,
    e_SYS_SUBMODULE_PG,
    e_SYS_SUBMODULE_TG,
    e_SYS_SUBMODULE_TA,
    e_SYS_SUBMODULE_DEC,
    e_SYS_SUBMODULE_PART,
    e_SYS_SUBMODULE_IPIC,
    e_SYS_SUBMODULE_EPIC,
    e_SYS_SUBMODULE_QE_IC,
    e_SYS_SUBMODULE_LAW,
    e_SYS_SUBMODULE_DDR,
    e_SYS_SUBMODULE_LBC,
    e_SYS_SUBMODULE_ECM,
    e_SYS_SUBMODULE_ARBITER,
    e_SYS_SUBMODULE_L2,
    e_SYS_SUBMODULE_QE,
    e_SYS_SUBMODULE_QE_GTIMERS,
    e_SYS_SUBMODULE_GTIMERS,
    e_SYS_SUBMODULE_RTC83,
    e_SYS_SUBMODULE_PAR_IO,
    e_SYS_SUBMODULE_GPIO,
    e_SYS_SUBMODULE_SI,
    e_SYS_SUBMODULE_TDM,
    e_SYS_SUBMODULE_TDM_RX_FRAME,
    e_SYS_SUBMODULE_TDM_TX_FRAME,
    e_SYS_SUBMODULE_MCC,
    e_SYS_SUBMODULE_MCC_HDLC_CH,
    e_SYS_SUBMODULE_MCC_TRANS_CH,
    e_SYS_SUBMODULE_MCC_SS7_CH,
    e_SYS_SUBMODULE_UPC,
    e_SYS_SUBMODULE_DUART,
    e_SYS_SUBMODULE_I2C_CTRL,
    e_SYS_SUBMODULE_DMA,
    e_SYS_SUBMODULE_DMA_CH,
    e_SYS_SUBMODULE_PM,
    e_SYS_SUBMODULE_TLU,
    e_SYS_SUBMODULE_SEC,
    e_SYS_SUBMODULE_PCI_CTRL,
    e_SYS_SUBMODULE_PCIE_CTRL,
    e_SYS_SUBMODULE_ESDHC,
    e_SYS_SUBMODULE_SRIO_PORT,
    e_SYS_SUBMODULE_SRIO_MU,
    e_SYS_SUBMODULE_SPI_CTRL,
    e_SYS_SUBMODULE_ETSEC,
    e_SYS_SUBMODULE_USB,
    e_SYS_SUBMODULE_UCC_GETH,
    e_SYS_SUBMODULE_UCC_ATM_COMMON,
    e_SYS_SUBMODULE_UCC_ATM_AAL2_COMMON,
    e_SYS_SUBMODULE_UCC_ATM_CTRL,
    e_SYS_SUBMODULE_UCC_POS_COMMON,
    e_SYS_SUBMODULE_UCC_POS_CTRL,
    e_SYS_SUBMODULE_EOS_LINK,
    e_SYS_SUBMODULE_PPPOS_LINK,
    e_SYS_SUBMODULE_UTOPIA_PORT,
    e_SYS_SUBMODULE_UCC_ATM_POLICER,
    e_SYS_SUBMODULE_UCC_ATM_AAL0,
    e_SYS_SUBMODULE_UCC_ATM_AAL5,
    e_SYS_SUBMODULE_UCC_ATM_AAL1,
    e_SYS_SUBMODULE_UCC_ATM_AAL2,
    e_SYS_SUBMODULE_UCC_ATM_AAL2_TQD,
    e_SYS_SUBMODULE_UCC_ATM_AAL2_CID,
    e_SYS_SUBMODULE_UCC_HDLC,
    e_SYS_SUBMODULE_UCC_TRNS,
    e_SYS_SUBMODULE_UCC_UART,
    e_SYS_SUBMODULE_QMC,
    e_SYS_SUBMODULE_QMC_HDLC_CH,
    e_SYS_SUBMODULE_QMC_TRANS_CH,
    e_SYS_SUBMODULE_L2_SWITCH,
    e_SYS_SUBMODULE_L2_SWITCH_PORT,
    e_SYS_SUBMODULE_PPPOHT,
    e_SYS_SUBMODULE_PPPOHT_BUNDLE,
    e_SYS_SUBMODULE_PPPOHT_LINK,
    e_SYS_SUBMODULE_PPPOHT_RX_FBP,
    e_SYS_SUBMODULE_PPPOHT_TX_FBPS,
    e_SYS_SUBMODULE_IW_PPPOHT,
    e_SYS_SUBMODULE_IW_PPPOHT_BUNDLE,
    e_SYS_SUBMODULE_IW_PPPOHT_LINK,
    e_SYS_SUBMODULE_IMA,
    e_SYS_SUBMODULE_IMA_GROUP_PORT,
    e_SYS_SUBMODULE_IMA_GROUP,
    e_SYS_SUBMODULE_IMA_LINK,
    e_SYS_SUBMODULE_MTC_COMMON,
    e_SYS_SUBMODULE_MTC,
    e_SYS_SUBMODULE_MTC_UCC,
    e_SYS_SUBMODULE_MTC_MCC,
    e_SYS_SUBMODULE_MTC_MCC_COMMON,
    e_SYS_SUBMODULE_IW_COMMON,
    e_SYS_SUBMODULE_IW_DEV,
    e_SYS_SUBMODULE_IW_CONN,
    e_SYS_SUBMODULE_IW_CDESC,
    e_SYS_SUBMODULE_IW_MCAST_GROUP,
    e_SYS_SUBMODULE_IW_IP_REASS,
    e_SYS_SUBMODULE_IW_PLCR,
    e_SYS_SUBMODULE_IW_QM,
    e_SYS_SUBMODULE_QE_FILTER,
    e_SYS_SUBMODULE_QE_TABLE,
    e_SYS_SUBMODULE_QE_RTC,
    e_SYS_SUBMODULE_QE_PTP,
    e_SYS_SUBMODULE_VP_DEV,
    e_SYS_SUBMODULE_VP_PORT,
    e_SYS_SUBMODULE_MII_MNG,
    e_SYS_SUBMODULE_PTP,
    e_SYS_SUBMODULE_SATA,
    e_SYS_SUBMODULE_SERDES,
    e_SYS_SUBMODULE_BM,
    e_SYS_SUBMODULE_BM_PORTAL,
    e_SYS_SUBMODULE_BM_CE_PORTAL,
    e_SYS_SUBMODULE_BM_CI_PORTAL,
    e_SYS_SUBMODULE_QM,
    e_SYS_SUBMODULE_QM_PORTAL,
    e_SYS_SUBMODULE_QM_CE_PORTAL,
    e_SYS_SUBMODULE_QM_CI_PORTAL,
    e_SYS_SUBMODULE_FM,
    e_SYS_SUBMODULE_FM_RTC,
    e_SYS_SUBMODULE_FM_MURAM,
    e_SYS_SUBMODULE_FM_PORT_HO,
    e_SYS_SUBMODULE_FM_PORT_10GRx,
    e_SYS_SUBMODULE_FM_PORT_1GRx,
    e_SYS_SUBMODULE_FM_PORT_10GTx,
    e_SYS_SUBMODULE_FM_PORT_1GTx,
    e_SYS_SUBMODULE_FM_PORT_10GMAC,
    e_SYS_SUBMODULE_FM_PORT_1GMAC,
    e_SYS_SUBMODULE_IW_IPHC_HC,
    e_SYS_SUBMODULE_IW_IPHC_HDEC,
    e_SYS_SUBMODULE_IW_IPSEC,
    e_SYS_SUBMODULE_EXTERNAL,
    /* Must close the sub-modules list */
    e_SYS_SUBMODULE_DUMMY_LAST

} e_SysModule;




#endif
