/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "p40xx.h"

#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <device_qnx.h>


/*****************************************************************************/
/*  Function Name	: bsd_mii_mediastatus
 *	Input			: Pointer to ifnet
 *					  Pointer to interface media request structure 
 *	Description		: this is a callback, made by the bsd media code.  We passed
					  a pointer to this function during the ifmedia_init() call
					  in bsd_mii_initmedia() 
 *					  Responsible for updating ifmr with media actual status 
 *	Returns			: Nothing. 
 *	Note			: None
*/
/*****************************************************************************/
void
bsd_mii_mediastatus(struct ifnet *ifp, struct ifmediareq *ifmr)
{
	p40xx_dev_t *p40xx = ifp->if_softc;

	p40xx->bsd_mii.mii_media_active = IFM_ETHER;
	p40xx->bsd_mii.mii_media_status = IFM_AVALID;
	

	if (p40xx->force_advertise != -1) {	// link is forced
		log(LOG_ERR,"%s() %s: Link is forced",__FUNCTION__, ifp->if_xname);

		if (p40xx->cfg.flags & NIC_FLAG_LINK_DOWN) {
			log(LOG_ERR,"%s() %s: NIC is currently down",__FUNCTION__, ifp->if_xname);
			p40xx->bsd_mii.mii_media_active |= IFM_NONE;
			p40xx->bsd_mii.mii_media_status  = 0;

		} else {	// link is up
			p40xx->bsd_mii.mii_media_status |= IFM_ACTIVE;

			switch(p40xx->cfg.media_rate) {
				case 0:
				p40xx->bsd_mii.mii_media_active |= IFM_NONE;
				break;	
	
				case 1000*10:
				p40xx->bsd_mii.mii_media_active |= IFM_10_T;
				break;
	
				case 1000*100:
				p40xx->bsd_mii.mii_media_active |= IFM_100_TX;
				break;
	
				case 1000*1000:
				p40xx->bsd_mii.mii_media_active |= IFM_1000_T;
				break;
				
				case 10*1000*1000:
#ifdef IFM_10G_T
				p40xx->bsd_mii.mii_media_active |= IFM_10G_T;
#else
				p40xx->bsd_mii.mii_media_active |= IFM_10G_CX4;
#endif
				break;
	
				default:	// this shouldnt really happen, but ...
				log(LOG_ERR,"%s() %s: p40xx->cfg.media_rate unknown %d",__FUNCTION__, ifp->if_xname,p40xx->cfg.media_rate);
				p40xx->bsd_mii.mii_media_active |= IFM_NONE;
				break;
			}
	
			if (p40xx->cfg.duplex) {
				p40xx->bsd_mii.mii_media_active |= IFM_FDX;
			}
	#if 0
			// could set flow bits
			p40xx->bsd_mii.mii_media_active |= IFM_FLOW;
			p40xx->bsd_mii.mii_media_active |= IFM_ETH_TXPAUSE;
			p40xx->bsd_mii.mii_media_active |= IFM_ETH_RXPAUSE;
	#endif
		}

	} else if (!(p40xx->cfg.flags & NIC_FLAG_LINK_DOWN)) {  // link is auto-detect and up
		log(LOG_INFO,"%s() %s: link is auto-detect and up (lan %d idx %d)",__FUNCTION__, ifp->if_xname, p40xx->cfg.lan, p40xx->cfg.device_index);

		p40xx->bsd_mii.mii_media_status |= IFM_ACTIVE;

		switch(p40xx->cfg.media_rate) {
			case 1000*10:
			p40xx->bsd_mii.mii_media_active |= IFM_10_T;
			break;

			case 1000*100:
			p40xx->bsd_mii.mii_media_active |= IFM_100_TX;
			break;

			case 1000*1000:
			p40xx->bsd_mii.mii_media_active |= IFM_1000_T;
			break;

			case 10*1000*1000:
#ifdef IFM_10G_T
			p40xx->bsd_mii.mii_media_active |= IFM_10G_T;
#else
			p40xx->bsd_mii.mii_media_active |= IFM_10G_CX4;
#endif
			break;


			default:	// this shouldnt really happen, but ...
			log(LOG_ERR,"%s() %s: p40xx->cfg.media_rate unknown %d",__FUNCTION__, ifp->if_xname,p40xx->cfg.media_rate);
			p40xx->bsd_mii.mii_media_active |= IFM_NONE;
			break;
		}

		if (p40xx->cfg.duplex) {
			p40xx->bsd_mii.mii_media_active |= IFM_FDX;
		}
#if 0
		// could set flow bits
		p40xx->bsd_mii.mii_media_active |= IFM_FLOW;
		p40xx->bsd_mii.mii_media_active |= IFM_ETH_TXPAUSE;
		p40xx->bsd_mii.mii_media_active |= IFM_ETH_RXPAUSE;
#endif

		// could move this to mii.c so there was no lag
		ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_AUTO);

	} else {	// link is auto-detect and down
		log(LOG_ERR,"%s() %s: link is auto-detect and down (lan %d idx %d)",__FUNCTION__, ifp->if_xname, p40xx->cfg.lan, p40xx->cfg.device_index);
		p40xx->bsd_mii.mii_media_active |= IFM_NONE;
		p40xx->bsd_mii.mii_media_status = 0;

		// could move this to mii.c so there was no lag
		ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_NONE);
	}

	// stuff parameter values with hoked-up bsd values
    ifmr->ifm_status = p40xx->bsd_mii.mii_media_status;
    ifmr->ifm_active = p40xx->bsd_mii.mii_media_active;
}

/*****************************************************************************/
/*  Function Name	: bsd_mii_mediachange
 *	Input			: Pointer to ifnet device of our interface
 *	Description		: Responsible for setting PHY media specific things. 
 *	Returns			: Nothing. 
 *	Note			: this is a callback, made by the bsd media code.  We passed
					  a pointer to this function during the ifmedia_init() call
					  in bsd_mii_initmedia().  This function is called when
					  someone makes an ioctl into us, we call into the generic
					  ifmedia source, and it make this callback to actually 
					  force the speed and duplex, just as if the user had
					  set the cmd line options
*/
/*****************************************************************************/
int
bsd_mii_mediachange(struct ifnet *ifp) {

	p40xx_dev_t	*p40xx			= ifp->if_softc;
	int				old_media_rate		= p40xx->cfg.media_rate;
	int				old_duplex			= p40xx->cfg.duplex;  
	struct ifmedia	*ifm				= &p40xx->bsd_mii.mii_media;
	int				user_duplex			= ifm->ifm_media & IFM_FDX ? 1 : 0;
	int				user_media			= ifm->ifm_media & IFM_TMASK;
	log(LOG_INFO,"%s()invoked",__FUNCTION__);

    if (!(ifp->if_flags & IFF_UP)) {
		slogf( _SLOGC_NETWORK, _SLOG_WARNING,
		  "%s() %s: p40xx interface isn't up, ioctl ignored", __FUNCTION__, ifp->if_xname);
	    return 0;
	}

	if (!(ifm->ifm_media & IFM_ETHER)) {
		slogf( _SLOGC_NETWORK, _SLOG_WARNING,"%s() %s: p40xx interface - bad media: 0x%X", 
		  __FUNCTION__, ifp->if_xname, ifm->ifm_media);
		return 0;	// should never happen
	}

	switch (user_media) {
		case IFM_AUTO:		// auto-select media
		if (p40xx->force_advertise == -1) {
			// we are already in auto-select mode, noop
			return 0;
		}
		// we are currently in a forced mode (or disabled) - now go auto-select
		p40xx->force_advertise = -1;
		p40xx->cfg.media_rate	 = -1;
		p40xx->cfg.duplex		 = -1;
		ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_AUTO);
		break;

		case IFM_NONE:		// disable media
		if (p40xx->force_advertise == 0) {
			// we are already disabled, noop
			return 0;
		}
		// we are currently either auto or forced - now disable link
		p40xx->force_advertise = 0;
		p40xx->cfg.media_rate	 = 0;
		p40xx->cfg.duplex		 = 0;
		ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_NONE);
		break;

		case IFM_10_T:		// force 10baseT
		p40xx->force_advertise = user_duplex ? MDI_10bTFD : MDI_10bT;
		p40xx->cfg.media_rate	 = 10 * 1000;
		p40xx->cfg.duplex		 = user_duplex;
		ifmedia_set(&p40xx->bsd_mii.mii_media, 
		  user_duplex ? IFM_ETHER|IFM_10_T|IFM_FDX : IFM_ETHER|IFM_10_T);
		break;

		case IFM_100_TX:	// force 100baseTX
		p40xx->force_advertise = user_duplex ? MDI_100bTFD : MDI_100bT;
		p40xx->cfg.media_rate	 = 100 * 1000;
		p40xx->cfg.duplex		 = user_duplex;
		ifmedia_set(&p40xx->bsd_mii.mii_media, 
		  user_duplex ? IFM_ETHER|IFM_100_TX|IFM_FDX : IFM_ETHER|IFM_100_TX);
		break;

		case IFM_1000_T:	// force 1000baseT
		//
		// N.B.  I have not had good luck, trying to get gige to work half
		// duplex.  Even with different gige switches, I can only force full duplex
		//
		p40xx->force_advertise = user_duplex ? MDI_1000bTFD : MDI_1000bT;
		p40xx->cfg.media_rate	 = 1000 * 1000;
		p40xx->cfg.duplex		 = user_duplex;
		ifmedia_set(&p40xx->bsd_mii.mii_media, 
		  user_duplex ? IFM_ETHER|IFM_1000_T|IFM_FDX : IFM_ETHER|IFM_1000_T);
		break;

		case IFM_10G_CX4:	// force 10Gig Copper
		
		p40xx->force_advertise =  MDI_1000bTFD ;
		p40xx->cfg.media_rate	 = 10*1000 * 1000;
		p40xx->cfg.duplex		 = 1;
		ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_10G_CX4|IFM_FDX );
		break;

#ifdef IFM_10G_T
		case IFM_10G_T:  	// force 10Gig Copper (cat 6A twisted-pair)
		
		p40xx->force_advertise =  MDI_1000bTFD ;
		p40xx->cfg.media_rate	 = 10*1000 * 1000;
		p40xx->cfg.duplex		 = 1;
		ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_10G_T|IFM_FDX );
		break;
#endif


		default:			// should never happen
		slogf( _SLOGC_NETWORK, _SLOG_WARNING,
		  "%s() %s: p40xx interface - unknown media: 0x%X", 
		  __FUNCTION__, ifp->if_xname, user_media);
		return 0;
		break;
	}

	// does the user want something different than it already is?
	if ((p40xx->cfg.media_rate  != old_media_rate) || 
		(p40xx->cfg.duplex      != old_duplex) ||
		(p40xx->cfg.flags       &  NIC_FLAG_LINK_DOWN) ){

		// must re-initialize hardware with new parameters
		ifp->if_init(ifp);
	}
    return 0;
}

/*****************************************************************************/
/*  Function Name	: bsd_mii_initmedia
 *	Input			: Pointer to parent device
 *	Description		: Called by p40xx_attach, triggered from dev_attach() function call. 
 *					  Responsible for initializing PHY media specific things. 
 *	Returns			: Nothing. 
 *	Note			: */
/*****************************************************************************/
void
bsd_mii_initmedia(p40xx_dev_t *p40xx)
{
    p40xx->bsd_mii.mii_ifp = &p40xx->ecom.ec_if;
	log(LOG_INFO,"%s()invoked",__FUNCTION__);

	ifmedia_init(&p40xx->bsd_mii.mii_media, IFM_IMASK, bsd_mii_mediachange,
	  bsd_mii_mediastatus);

	// we do NOT call mii_attach() - we do our own link management

	//
	// must create these entries to make ifconfig media work
	// see lib/socket/public/net/if_media.h for defines
	//

	// ifconfig wm0 none (x22)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_NONE, 0, NULL);

	// ifconfig wm0 auto (x20)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_AUTO, 0, NULL);

	// ifconfig wm0 10baseT (x23 - half duplex)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_10_T, 0, NULL);

	// ifconfig wm0 10baseT-FDX (x100023)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_10_T|IFM_FDX, 0, NULL);

	// ifconfig wm0 100baseTX (x26 - half duplex)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_100_TX, 0, NULL);

	// ifconfig wm0 100baseTX-FDX (x100026 - full duplex)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_100_TX|IFM_FDX, 0, NULL);

	// ifconfig wm0 1000baseT (x30 - half duplex)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_1000_T, 0, NULL);

	// ifconfig wm0 1000baseT mediaopt fdx (x100030 - full duplex)
    ifmedia_add(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_1000_T|IFM_FDX, 0, NULL);

	// would add more here for flow control via ifconfig media

	// link is initially down
	ifmedia_set(&p40xx->bsd_mii.mii_media, IFM_ETHER|IFM_NONE);
}



__SRCVERSION( "$URL:  $Rev: " );
