/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "p40xx.h"
#include <device_qnx.h>
#include <drvr/hwinfo.h>
#include <sys/mbuf.h>



/*Portal 0, dsec0 becomes Master in our case*/
#define DPAA_MASTER_IF 0
#define DPAA_DEF_RXQID_RANGE 8
#define DPAA_DEF_TXQID_RANGE 8
#define DPAA_DEF_PORTAL_FQID_BASE 1 // should this be 8?
#define DPAA_DEF_PORTAL_QID_RANGE 128
#define IF_NAME_1GIG   "dsec"
#define IF_NAME_10GIG  "tgec"
#define MAX_FQIDS (256*1024)
/*TBD Interrupt information should be part of HWINFO . As of now, lets keep it local*/
#define QMAN_PORTAL_IID_BASE 18
#define BMAN_PORTAL_IID_BASE 28
#define FMAN1_IID 38
#define FMAN2_IID 39
#define COMMON_ERR_IID 40

/*Declaration of local functions not exposed outside.*/


static void p40xx_get_syspage_params(dpaa_startup_params_t *cfg);
static void force_default_param(dpaa_startup_params_t *dpaa_cfg);
static int p40xx_parse_options (dpaa_startup_params_t *cfg, nic_config_t *nic_cfg, char *options);
static int p40xx_validate_params (dpaa_startup_params_t *cfg);
static void p40xx_stop(struct ifnet *ifp, int disable);
static void p40xx_destroy(p40xx_dev_t *dpaa);
static int p40xx_init(struct ifnet *ifp);
static int p40xx_attach(struct device *, struct device *, void *);
static int p40xx_detach(struct device *, int);
static void p40xx_shutdown(void *);
static void p40xx_qman_default(void *arg, uint32_t*);
static void p40xx_bman_default(void *arg,uint32_t*);
static int p40xx_defconfig(dpaa_startup_params_t *dpaa_cfg, int autocfg);
static int fmt_4x1g_lanes(int enables, char *buf /* char buf[8] */);
static int calc_4x1g_lanes(int enable12, int enable34, int lpd, int *enables, char *buf /* char buf[8] */);
#ifdef SERDES8_ERRATA
static void p40xx_serdes8_errata(dpaa_startup_params_t *dpaa_cfg);
#endif


const uint32_t portal_fqid_map[][2] = {
/*This is a map explaining base FQID and Ranges for each of the SW portal */
 /*FQID Base*/   /* Total number of FQID to be created*/
{(DPAA_DEF_PORTAL_FQID_BASE ),           	                  DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 0*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 1*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 1*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 2*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 2*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 3*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 3*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 4*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 4*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 5*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 5*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 6*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 6*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 7*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 7*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 8*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 8*/
{(DPAA_DEF_PORTAL_FQID_BASE+ 9*DPAA_DEF_PORTAL_QID_RANGE),    DPAA_DEF_PORTAL_QID_RANGE}, /*Portal 9*/
};


const uint32_t mac_to_portal_map[]= {
 /*Assigning Portal IDs for each MAC IF*/

	0,     /*MAC ID 0 FM1 DSEC0 is assigned Portal 0*/
	1,     /*MAC ID 1 FM1 DSEC1 is assigned Portal 1*/
	2,     /*MAC ID 2 FM1 DSEC2 is assigned Portal 2*/
	3,     /*MAC ID 3 FM1 DSEC3 is assigned Portal 3*/
	4,     /*MAC ID 4 FM1 TGEC0 is assigned Portal 4*/
	5,     /*MAC ID 5 FM2 DSEC0 is assigned Portal 5*/
	6,     /*MAC ID 6 FM2 DSEC1 is assigned Portal 6*/
	7,     /*MAC ID 7 FM2 DSEC2 is assigned Portal 7*/
	8,     /*MAC ID 8 FM2 DSEC3 is assigned Portal 8*/
	9      /*MAC ID 9 FM2 TGEC0 is assigned Portal 9*/
};

/*This has to be sufficiently larger to accomodate max #  of FQIDs possible*/


#ifdef VARIANT_a
#include <nw_dl.h>
/* This is what gets specified in the stack's dl.c */
struct nw_dll_syms p40xx_syms[] = {
        {"iopkt_drvr_entry", &IOPKT_DRVR_ENTRY_SYM(p40xx)},
        {NULL, NULL}
};
#endif

struct _iopkt_drvr_entry IOPKT_DRVR_ENTRY_SYM(p40xx) = IOPKT_DRVR_ENTRY_SYM_INIT(p40xx_detect);

static  char *p40xx_opts [] = {
      "phy_addr_fm1",    // 0
      "phy_addr_fm2",    // 1
      "emu_phy",		  // 2
      "verbose",		  // 3
      "10g_fm1",		  // 4
      "4x1g_fm1",		  // 5
      "10g_fm2",		  // 6
      "4x1g_fm2",		  // 7
      "no_phy",		  	  // 8
      "rx_loop",	  	  // 9
	  "dump_regs",		  // 10
      "2x1g_fm2",		  // 11
	  "jumbo_1g",		  // 12
	  "jumbo_10g",		  // 13
	  "jumbo",		 	  // 14
      NULL
};


/*This Map gives us channel ID for transmit Queue for each SW interface(Portal)*/
/*Members should be accessed using p40xx->idx instead of p40xx->portal*/
static uint32_t TX_IF_TO_CHANNEL_MAP[10] = {
	CHID_FMAN0_TX_1GIG0,  /*desc0 */
	CHID_FMAN0_TX_1GIG1,  /*desc1 */
	CHID_FMAN0_TX_1GIG2,  /*desc2 */
	CHID_FMAN0_TX_1GIG3,  /*desc3 */
	CHID_FMAN0_TX_10_GIG,  /*desc4 */
	CHID_FMAN1_TX_1GIG0,  /*desc5 */
	CHID_FMAN1_TX_1GIG1,  /*desc6 */
	CHID_FMAN1_TX_1GIG2,  /*desc7 */
	CHID_FMAN1_TX_1GIG3,  /*desc8 */
	CHID_FMAN1_TX_10_GIG  /*desc9 */
};

CFATTACH_DECL(p40xx,
	sizeof(p40xx_dev_t),
	NULL,
	p40xx_attach,
	p40xx_detach,
	NULL);

#ifdef LOG_BMAN_BUF_ADDR
void p40xx_buf_dump(p40xx_dev_t *p40xx){
	uint64_t paddr=0;
	void* vaddr=0;
	FILE* buf_info_file=NULL;
	int i=0;
	
	buf_info_file = fopen("/tmp/buf_release_info","w");
	for(i=0;i<p40xx->bp_count;i++){
		while(p40xx_acquire_buf(p40xx->dpaa,p40xx->bpid[i], p40xx->portal[0], &vaddr)==EOK){
			paddr= XX_VirtToPhys(vaddr);	
	    	fprintf(buf_info_file,"\nBPID %d, ADDR 0x%llx",p40xx->bpid[i], paddr);
			fflush(buf_info_file);
		}
	}
	fclose(buf_info_file);
}
#endif

/*****************************************************************************/
/*  Function Name	: p40xx_detect
 *	Input			: Pointer to DLL handle opened by io-pkt
 *					  Self pointer of io-pkt
 *					  option string passed by command line(comma seperated)
 *	Description		: Called while initializing driver library.First entry point 
 *					  of driver.
 *	Returns			: ZERO if successful, Non ZERO otherwise. 
 *	Note			:				
 */
/*****************************************************************************/
int 
p40xx_detect(void *dll_hdl, struct _iopkt_self *iopkt, char *options){

	dpaa_startup_params_t *dpaa_cfg=NULL;
	int				single=0;
	int				retVal=-1;
	int				i=0;
	int 			nic_id=0;
	struct device	*dev=NULL;
	p40xx_arg_t 	args;
	uint32_t fm_index=0;
	uint32_t mac_index=0;
	char *name=NULL;
	dpaa_handle_t 	dpaa_hndl=NULL; /*Interface should save this handle*/
    nic_config_t    *nic_cfg = NULL;

       
#if defined(ENABLE_INSTR) && defined(ENABLE_INSTR_STARTUP_CONFIG)
    TraceEvent(_NTO_TRACE_DELALLCLASSES);
    TraceEvent(_NTO_TRACE_CLRCLASSPID, _NTO_TRACE_KERCALL);
    TraceEvent(_NTO_TRACE_CLRCLASSTID, _NTO_TRACE_KERCALL);
    TraceEvent(_NTO_TRACE_CLRCLASSPID, _NTO_TRACE_THREAD);
    TraceEvent(_NTO_TRACE_CLRCLASSTID, _NTO_TRACE_THREAD);
    TraceEvent(_NTO_TRACE_SETALLCLASSESFAST);
    TraceEvent(_NTO_TRACE_ADDALLCLASSES);
    TraceEvent(_NTO_TRACE_START);
#endif
	if((dpaa_cfg= (dpaa_startup_params_t *)\
		 malloc(sizeof(dpaa_startup_params_t),M_TEMP, M_NOWAIT | M_ZERO))==NULL){
		 log(LOG_ERR, "%s(): cannot allocate memory for dpaa_cfg!", __FUNCTION__);
		 retVal= ENOMEM;
		 goto label_exit;
	}

	if((nic_cfg= (nic_config_t *)\
		 malloc(sizeof(nic_config_t),M_TEMP, M_NOWAIT | M_ZERO))==NULL){
		 log(LOG_ERR, "%s(): cannot allocate memory for nic_config!", __FUNCTION__);
		 retVal= ENOMEM;
		 goto label_exit;
	}

	
	/*Get System Parameters from Syspage first. Ignore if not found*/
	p40xx_get_syspage_params(dpaa_cfg);

	nic_cfg->device_index= 0xffffffff;
    nic_cfg->media_rate = -1;
    nic_cfg->duplex = -1;

	/*Override the startup with command line arguments, if any*/
	if (options){
		if ((retVal=p40xx_parse_options(dpaa_cfg, nic_cfg, options))!=EOK) {
            log(LOG_ERR, "%s(): p40xx_parse_options() failed! Return code %d", __FUNCTION__,retVal);
			retVal= EINVAL;
			goto label_exit_freemem;
		}
	}
	
	if ((dpaa_cfg->fm1_10gig ||dpaa_cfg->fm0_10gig) && (nic_cfg->media_rate != -1 || nic_cfg->duplex != -1)) {
			log(LOG_INFO, "%s(): XAUI(10G) port doesn't support the force speed and duplex setting", __FUNCTION__);
			retVal= EINVAL;
			goto label_exit_freemem;
		}
		
	if (nic_cfg->duplex == 0) {
			log(LOG_ERR, "%s(): SGMII(1G) and XAUI(10G) ports don't support the half-duplex mode. ", __FUNCTION__);
			retVal= EINVAL;
			goto label_exit_freemem;
	}
	
	/*We got the runtime parameters. Parse and validate them before proceeding further*/
	if((retVal=p40xx_validate_params(dpaa_cfg))!=EOK){
			log(LOG_ERR, "%s(): Invalid operational parameters for DPAA system!", __FUNCTION__);
			retVal= EINVAL;
			goto label_exit_freemem;
	}
	
	if (nic_cfg->device_index != 0xffffffff) {
		/*Ok, only one MAC needed as specified by user*/
		single=1;
		dpaa_cfg->master_if= mac_to_portal_map[nic_cfg->device_index];
		log(LOG_INFO, "%s(): Only 1 MAC index=%d getting initialized", __FUNCTION__,nic_cfg->device_index);
		/*Override the parsed options with command line force*/
		for(i=0;i<DPAA_MAX_MACDEV;i++){
			if(i==nic_cfg->device_index )
			dpaa_cfg->macdev_enable[i/MAX_MACDEV_PER_FMAN][i%MAX_MACDEV_PER_FMAN]=1;
			else
			dpaa_cfg->macdev_enable[i/MAX_MACDEV_PER_FMAN][i%MAX_MACDEV_PER_FMAN]=0;
		}
	}
	log(LOG_INFO, "%s(): Master IF %d", __FUNCTION__,dpaa_cfg->master_if);
	if(dpaa_cfg->secondary_if != -1)
	    log(LOG_INFO, "%s(): Secondary IF %d", __FUNCTION__,dpaa_cfg->secondary_if);
	

	/*Intialize the DPAA System, save the handle*/
	if((dpaa_hndl = dpaa_system_init(dpaa_cfg))==NULL){
			log(LOG_ERR, "%s(): Couldnot initialize DPAA system!", __FUNCTION__);
			retVal= -1; 
			dpaa_hndl=0;
			goto label_exit_freemem;
	}
#ifdef SERDES8_ERRATA
	p40xx_serdes8_errata(dpaa_cfg);
#endif
	/*Supply operational arguments to p40xx_attach() functions from dpaa_cfg*/
	memset(&args,0,sizeof(p40xx_arg_t));
	args.dll_hdl = dll_hdl;
	args.iopkt= iopkt;
	args.options = options;
	args.master = dpaa_cfg->master_if; /*Tell who is the Master interface*/
	args.secondary = dpaa_cfg->secondary_if; /*Tell who is the Secondary FM interface*/
	args.dpaa = dpaa_hndl; /*Pass on DPAA handle*/
	args.nic_cfg= nic_cfg; 
	args.emu_phy= dpaa_cfg->emu_phy; 
	args.dump_regs = dpaa_cfg->dump_regs;
	args.jumbo_1g  = dpaa_cfg->jumbo_1g;
	args.jumbo_10g = dpaa_cfg->jumbo_10g;
	/*Sync verbose params here*/
	args.nic_cfg->verbose= dpaa_cfg->verbose;
	args.immr=dpaa_cfg->immr; 

	/*Set this to detect whether there is at least 1 call to dev_attach */
	retVal= -1; 
	/*Attach all required interfaces here*/
	for(nic_id= single?nic_cfg->device_index:0;nic_id<DPAA_MAX_MACDEV;nic_id++){
		
		fm_index= nic_id/MAX_MACDEV_PER_FMAN; /*Which FMAN?*/
		mac_index=nic_id % MAX_MACDEV_PER_FMAN; /*Which MAC inside that FMAN? From 0 to 4*/

		if(dpaa_cfg->macdev_enable[fm_index][mac_index]==1){
			
			args.idx=nic_id;  /*idx corresponds to MAC on FM1 FM2*/
			args.phy_addr= dpaa_cfg->phy_addr[fm_index][mac_index];
			/*MAC address for all 10 interfaces should be processed and parsed into config array */
			memcpy(&args.nic_cfg->current_address[0],&dpaa_cfg->mac_addr[fm_index][mac_index][0],MAC_ADDR_SIZE);
			args.portal= mac_to_portal_map[nic_id];
			args.rx_loop = dpaa_cfg->rx_loop;
			if(mac_index==(MAX_MACDEV_PER_FMAN-1)){
				args.emu_phy= nic_id; /*Set emuphy fixed for 10Gig*/
				name=IF_NAME_10GIG;
			}
			else{
				name=IF_NAME_1GIG;
			}
			if(dpaa_cfg->no_phy==1){
				args.emu_phy= nic_id; /*Set emuphy for all*/
			}
			/*Call all interfaces dsecX initially (0<=X<=9)*/
			if ((retVal = dev_attach(name, options, &p40xx_ca, &args, &single, &dev, NULL)) != EOK) {
				log(LOG_ERR, "%s(): dev_attach failed for id%d: return code %d", __FUNCTION__, nic_id, retVal);
				retVal= ENODEV;
				goto label_exit_freemem;
            }

		}
		if(single)
		break;
	}

	/*Have a clean exit if you reach here*/
	if(retVal== EOK)
	goto label_exit;

	/*All initialization paramters have been conveyed to their intended users*/
label_exit_freemem:

	if(dpaa_hndl!=NULL)
		dpaa_system_shutdown(dpaa_hndl,0);
	
label_exit:
	if(nic_cfg) {
		free(nic_cfg,M_TEMP);
	}
	if(dpaa_cfg!=NULL) {
		free(dpaa_cfg,M_TEMP);
	}
	return retVal;
}

/*****************************************************************************/
/*  Function Name	: p40xx_attach
 *	Input			: Pointer to parent device (None in our case)
 *					  struct device pointer as initialized by io-pkt
 *					  Argument pointer (p40xx_arg_t*) passed by caller p40xx_detect()
 *	Description		: Called by io-pkt, triggered from dev_attach() function call. 
 *					  Responsible for attaching device with framework, assigning 
 *					  callbacks, registering interrupt functions. 
 *	Returns			: ZERO if successful, Non ZERO otherwise. 
 *	Note			: This function should not use area allocated for aux parameter
 *					  and should copy any required parameter to self pointer as 
 *					  aux shall be destroyed by caller of this function. */
/*****************************************************************************/
static int 
p40xx_attach(struct device *parent, struct device *self, void *aux){

	/*Iopkt library allocated a structure device for us. Lets fill it further*/
	p40xx_dev_t *p40xx= (p40xx_dev_t *)self;
	struct ifnet *ifp 	= &p40xx->ecom.ec_if;
	p40xx_arg_t *arg= (p40xx_arg_t*)aux;
	nic_config_t	*nic_cfg = &p40xx->cfg;
	dpaa_handle_t dpaa = arg->dpaa;
	dpaa_cfg_t cfg;
#ifdef EARLY_PCD_INIT
	dpaa_runtime_t req;
#endif
	int		rc=0;
	int		i=0;
	/*Let check what arguments have been passed to us.*/
	/*This is our interface id which makes us unique in among 10*/
	struct _iopkt_self	*iopkt =arg->iopkt ;
		
	/*Config Request-Response structure to talk with DPAA*/
	p40xx->dev.dv_dll_hdl = arg->dll_hdl;

	/*Register shutdown hook to system*/
	if ((p40xx->sdhook = shutdownhook_establish(p40xx_shutdown,p40xx)) == NULL) {
		log(LOG_ERR, "%s(): shutdownhook_establish failed for id%d: ",__FUNCTION__,  arg->idx);
		return ENOMEM;
	}

	p40xx->iopkt =iopkt; 
	p40xx->mdi = NULL;

	p40xx->rx_buffer_prefix_size = 64; /* Request atleast 64 bytes of private area before each Rxed packet.
				The system layer may overwrite this value, so we need to update it back */

	/*IDx is the MAC index varies from 0 to 9. Dont intermix with portal ID*/
	p40xx->idx= arg->idx ;
#ifdef MULTI_PORTAL
	if(IS_MAC_10GIG(p40xx->idx))
	p40xx->num_of_portals=4;
	else
#endif
	p40xx->num_of_portals=1;
	
	for(i=0;i<p40xx->num_of_portals;i++){
		/*Portal ID is assigned by caller, stick to it.*/
		p40xx->portal[i]=arg->portal+i;
		p40xx->token[i].tid=i;
		p40xx->token[i].p40xx=p40xx;
	}

	/*DPAA system handle as passed by detect. All interfaces have same dpaa handle*/
	p40xx->dpaa= dpaa;
	p40xx->dying=0;
	p40xx->rx_loop = arg->rx_loop;	
	p40xx->rx_fq_num=0;
	/*Save memory pool reference.
	 * Reason why we do it via arg is that application might plan to assign seperate Rx pools to 
	 * seperate interface. In that case, each arg would contain a seperate Mpool address*/
	if(arg->master==p40xx->portal[0]){
		p40xx->master=1;
	}
	else{
		p40xx->master=0;
	}
	if(arg->secondary==p40xx->portal[0]){
		p40xx->secondary=1;
	}
	else{
		p40xx->secondary=0;
	}
	for(i=0;i<p40xx->num_of_portals;i++){
		p40xx->iid_qman_portal[i]= QMAN_PORTAL_IID_BASE+arg->portal+i;
		p40xx->iid_bman_portal[i]= BMAN_PORTAL_IID_BASE+arg->portal+i;
	}
	if(p40xx->master){
		p40xx->iid_fman1= FMAN1_IID;
		p40xx->iid_fman2= FMAN2_IID;
		p40xx->iid_err=COMMON_ERR_IID;
	}
	else{
		p40xx->iid_fman1= -1;
		p40xx->iid_fman2= -1;
		p40xx->iid_err=-1;
	}


	/*Copy the parsed nic_cfg to our own structure*/
	memcpy(nic_cfg, arg->nic_cfg,sizeof(nic_config_t));
	p40xx->emu_phy= arg->emu_phy;

	p40xx->verbose=nic_cfg->verbose;
	p40xx->dump_regs=arg->dump_regs;

	nic_cfg->revision = NIC_CONFIG_REVISION;
	nic_cfg->flags = NIC_FLAG_MULTICAST|NIC_FLAG_LINK_DOWN;
  	nic_cfg->mtu = nic_cfg->mru = ETH_MAX_PKT_LEN;
	nic_cfg->lan = p40xx->dev.dv_unit;
	nic_cfg->mac_length = ETH_MAC_LEN;
	nic_cfg->media = NIC_MEDIA_802_3;
	nic_cfg->mac_length = ETH_MAC_LEN;
	nic_cfg->phy_addr = arg->phy_addr;
	nic_cfg->device_index = arg->idx;
	nic_cfg->num_mem_windows=0;
    nic_cfg->num_io_windows=0;
    nic_cfg->num_irqs=0; 
    nic_cfg->num_dma_channels=0;
	strcpy((char *)nic_cfg->device_description, "P4080 DPAA device");
	strcpy((char *)nic_cfg->uptype, "en");
	nic_cfg->iftype = IFT_ETHER;
	nic_cfg->priority = IRUPT_PRIO_DEFAULT;

	/*MAC address for nic were already provided inside nic_cfg*/
	nic_dump_config(nic_cfg);

	p40xx->force_advertise = -1;


	if(!(IS_MAC_10GIG(p40xx->idx))){
		// did user specify either of speed or duplex on the cmd line?
		if ((nic_cfg->media_rate != -1) || (nic_cfg->duplex != -1)) {
			if (nic_cfg->duplex == 0) {
				log(LOG_ERR, "%s(): SGMII(1G) and XAUI(10G) ports don't support the half-duplex mode. Setting to use the full duplex mode ", __FUNCTION__);
				nic_cfg->duplex = 1;
			}

			if (nic_cfg->media_rate != -1) {
				switch(nic_cfg->media_rate) {
				case 10*1000:
					p40xx->force_advertise = nic_cfg->duplex ? MDI_10bTFD : MDI_10bT;
					p40xx->cfg.media_rate=10000;
					break;

				case 100*1000:
					p40xx->force_advertise = nic_cfg->duplex ? MDI_100bTFD : MDI_100bT;
					p40xx->cfg.media_rate=100000;
					break;

				case 1000*1000:
					p40xx->force_advertise = nic_cfg->duplex ? MDI_1000bTFD : MDI_1000bT;
					p40xx->cfg.media_rate=1000000;
					break;

				default:
					log(LOG_ERR, "%s(): invalid speed: %d", __FUNCTION__, nic_cfg->media_rate/1000);
					return EINVAL;
					break;			
				}
			}
		}
}else
{
		log(LOG_INFO, "%s(): tgecX port doesn't support the force speed and duplex setting", __FUNCTION__);
}
	p40xx->rxd_pkts = 0; 

	p40xx->isr_fman1_funcid= -1;
	p40xx->isr_fman2_funcid= -1;

	/*Following interrupts shall only be used by master*/
	if(p40xx->master){
		/*Initialize FMAN1 interrupt mechanism*/
		if ((rc = interrupt_entry_init(&p40xx->inter_fman1, 0, NULL,IRUPT_PRIO_DEFAULT)) != EOK) {
			log(LOG_ERR, "%s(): interrupt_entry_init(inter_fman1) failed: return code %d", 
			__FUNCTION__, rc);
			return -1;
		}
		/*Using kernel masking by default*/
		p40xx->inter_fman1.func = p40xx_process_fman1_intr;
		p40xx->inter_fman1.enable = p40xx_enable_fman1_intr;
		p40xx->inter_fman1.arg = p40xx;
		p40xx->isrp_fman1 = p40xx_fman1_isr;
		
		
		/*Initialize FMAN2 interrupt mechanism*/
		if ((rc = interrupt_entry_init(&p40xx->inter_fman2, 0, NULL,IRUPT_PRIO_DEFAULT)) != EOK) {
			log(LOG_ERR, "%s(): interrupt_entry_init(inter_fman2) failed: return code %d", 
			__FUNCTION__, rc);
			return -1;
		}
		/*Using kernel masking by default*/
		p40xx->inter_fman2.func = p40xx_process_fman2_intr;
		p40xx->inter_fman2.enable = p40xx_enable_fman2_intr;
		p40xx->inter_fman2.arg = p40xx;
		p40xx->isrp_fman2 = p40xx_fman2_isr;
	}

	for(i=0;i<p40xx->num_of_portals;i++){
			/*Configure DPAA to initialize QMAN SW portal for this interface*/
			memset((void *)&cfg, '\0', sizeof(cfg));
			cfg.code=DPAA_CFG_CODE_INIT_QMAN_SWPORTAL;
			
			INIT_FLAG(cfg.init_qman_portal.flags);
			SET_FLAG(cfg.init_qman_portal.flags,FLAG_ALLOW_EXT_ISR);
			SET_FLAG(cfg.init_qman_portal.flags,FLAG_DQRR_PUSH_MODE);
			cfg.init_qman_portal.portal=p40xx->portal[i]; 
#ifdef MULTI_PORTAL
			cfg.init_qman_portal.chmap=CHMAP_ADD_CHID(0, CHID_POOLCH_1+p40xx->portal[0]);
#else
			cfg.init_qman_portal.chmap=0;
#endif
			cfg.init_qman_portal.fqid_base=portal_fqid_map[p40xx->portal[i]][0] ;
			cfg.init_qman_portal.fqid_num= portal_fqid_map[p40xx->portal[i]][1] ;

			cfg.init_qman_portal.owner= p40xx;
			/*This should handle Error and unknown FQID dequeue*/
			cfg.init_qman_portal.default_cbfn= &p40xx_qman_default; 	
			cfg.init_qman_portal.token= p40xx; 
			cfg.init_qman_portal.token_size= sizeof(p40xx_dev_t); 

			if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
					log(LOG_ERR, "%s():dpaa_system_config failed for CODE_INIT_QMAN_SWPORTAL %d: return code %d\n",
									__FUNCTION__,p40xx->portal[i],rc);
					log(LOG_ERR, "Error Cause %d",cfg.init_qman_portal_resp.result);
					return -1;
			}
			
			/*Configure DPAA to initialize BMAN SW portal for this interface*/
			memset((void *)&cfg, '\0', sizeof(cfg));
			cfg.code=DPAA_CFG_CODE_INIT_BMAN_SWPORTAL;

			INIT_FLAG(cfg.init_bman_portal.flags);
			SET_FLAG(cfg.init_bman_portal.flags,FLAG_ALLOW_EXT_ISR);
			cfg.init_bman_portal.portal=p40xx->portal[i];
			/*This portal subscribes to its default channel only*/
			cfg.init_bman_portal.owner= p40xx;
			/*This should handle Error and unknown FQID dequeue*/
			cfg.init_bman_portal.default_cbfn= &p40xx_bman_default; 	
			cfg.init_bman_portal.token= p40xx; 
			cfg.init_bman_portal.token_size= sizeof(p40xx_dev_t); 

			if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
					log(LOG_ERR, "%s():dpaa_system_config failed for CODE_INIT_BMAN_SWPORTAL %d: return code %d\n",
									__FUNCTION__,p40xx->portal[i],rc);
					log(LOG_ERR, "Error Cause %d",cfg.init_bman_portal_resp.result);
					return -1;
			}

			/*Initialize QMAN interrupt mechanism*/
			if ((rc = interrupt_entry_init(&p40xx->inter_qman[i], 0, NULL,IRUPT_PRIO_DEFAULT)) != EOK) {
					log(LOG_ERR, "%s(): interrupt_entry_init(inter_qman) failed: return code %d", __FUNCTION__, rc);
					return -1;
			}
			/*Using kernel masking by default*/
			p40xx->inter_qman[i].func = p40xx_process_interrupt;
			p40xx->inter_qman[i].enable = p40xx_enable_qman_intr;
			p40xx->inter_qman[i].arg = &p40xx->token[i];
			p40xx->isrp_qman = p40xx_qman_isr;
			p40xx->isr_qman_funcid[i]= -1;

			/*Initialize BMAN interrupt mechanism*/
			if ((rc = interrupt_entry_init(&p40xx->inter_bman[i], 0, NULL,IRUPT_PRIO_DEFAULT)) != EOK) {
					log(LOG_ERR, "%s(): interrupt_entry_init(inter_bman) failed: return code %d", __FUNCTION__, rc);
					return -1;
			}
			/*Using kernel masking by default*/
			p40xx->inter_bman[i].func = p40xx_process_bman_intr;
			p40xx->inter_bman[i].enable = p40xx_enable_bman_intr;
			p40xx->inter_bman[i].arg = &p40xx->token[i];
			p40xx->isrp_bman = p40xx_bman_isr;
			p40xx->isr_bman_funcid[i]= -1;

			/*Configure DPAA to create and attach to our default RX FQID*/
			memset((void *)&cfg, '\0', sizeof(cfg));
			cfg.code=DPAA_CFG_CODE_INIT_FQ;
			INIT_FLAG(cfg.init_fq.flags);
			cfg.init_fq.portal= p40xx->portal[i]; /*Portal idx starts from 0*/
#ifdef MULTI_PORTAL		
			cfg.init_fq.channel= CHID_POOLCH_1+p40xx->portal[0];
#else
			cfg.init_fq.channel= CHANNELID_DEFAULT; /*Bind to default channel for this portal*/
#endif
			cfg.init_fq.workq= WQID_DEFAULT; /*Enqueue from single WQ as of now*/
			cfg.init_fq.fqid= FQID_DEFAULT; /*Let DPAA assign the correct default FQID for us. We will save it later */
			/*Do not attach any handler with FrameQueue. */
			if(p40xx->rx_loop){
				cfg.init_fq.fq_handler = &p40xx_loop;
			}
			else{
				cfg.init_fq.fq_handler = &p40xx_rx;
			}
			cfg.init_fq.token =&p40xx->token[i];
			cfg.init_fq.token_size = 0;
			if(i==p40xx->num_of_portals-1){ 
				/*Add one more Default QID for last portal. This serves as default FQID for PCD*/
				cfg.init_fq.num_of_fqid = DPAA_DEF_RXQID_RANGE/p40xx->num_of_portals +1;
			}
			else{
				cfg.init_fq.num_of_fqid = DPAA_DEF_RXQID_RANGE/p40xx->num_of_portals ;
			}
			cfg.init_fq.align= DPAA_DEF_RXQID_RANGE;

			if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
					log(LOG_ERR, "%s():dpaa_system_config failed for CODE_INIT_FQ id%d: return code %d",
									__FUNCTION__,arg->idx,rc);
					log(LOG_ERR, "Error creating rx_fqid Cause %d",cfg.init_fq_resp.result);
					return -1;
			}
			/*If we passed DEFAULT_FQID or CHANNEL ID, DPAA shall return us a valid assigned ID. 
			 * Else we get the same number as was passed so no harm in overwriting it.
			 * Please note that if you requested more than one FQIds to be created, all of them would
			 * have got attached with your callback. So callback function needs to take care of the
			 * passed offset field, to distinguish between seperate FQIDs*/
			p40xx->rx_fqid[i]= cfg.init_fq_resp.fqid;
			if(i==p40xx->num_of_portals-1){ /*Add one more Default QID for last*/
				p40xx->rx_fq_num+=  DPAA_DEF_RXQID_RANGE/p40xx->num_of_portals+1;
			}
			else {
				p40xx->rx_fq_num+=  DPAA_DEF_RXQID_RANGE/p40xx->num_of_portals;
			}

			p40xx->rx_chid[i]= cfg.init_fq_resp.chid; /*Channel ID returned is portal's default channel ID*/
	}

/*This simply means that if Tx confirmation is expected*/
#if !defined (NO_TX_CONFIRMATION)

	/*Create a Transmit Confirmation Queue ID*/
	memset((void *)&cfg, '\0', sizeof(cfg));
	cfg.code=DPAA_CFG_CODE_INIT_FQ;
	INIT_FLAG(cfg.init_fq.flags);

	cfg.init_fq.portal= p40xx->portal[0]; 
#ifdef MULTI_PORTAL		
	cfg.init_fq.channel= CHID_POOLCH_1+p40xx->portal[0];
#else
	cfg.init_fq.channel= CHANNELID_DEFAULT; /*Bind to default channel for this portal*/
#endif
	cfg.init_fq.workq= WQID_DEFAULT; /*Enqueue from single WQ as of now*/
	cfg.init_fq.fqid= FQID_DEFAULT; /*Let DPAA assign the correct default FQID for us. We will save it later */
	cfg.init_fq.fq_handler = &p40xx_tx_complete;
	cfg.init_fq.token =&p40xx->token[0];
	cfg.init_fq.token_size = 0;
	cfg.init_fq.num_of_fqid=1;

	if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
			log(LOG_ERR, "%s():dpaa_system_config failed for CODE_INIT_FQ id%d: return code %d\n",
							__FUNCTION__,arg->idx,rc);
			log(LOG_ERR, "Error creating tcfqid Cause %d",cfg.init_fq_resp.result);
			return -1;
	}
	p40xx->tcfqid[0]= cfg.init_fq_resp.fqid;

/*If TX confirmation is not required*/
#else
	/*
	 *  When dontReleaseTxBufs is enabled:
	 *	fqid 0 means put buffers back, without tx confirmation
	 *	fqid undef (e.g. 0xffffff below) means don't put buffers back, without tx confirmation
	 *	fqid defined means don't put buffers back, with tx confirmation
	 */
	p40xx->tcfqid[0]= 0xffffff;
#endif

	/*Create a Rx Error Queue ID*/
	memset((void *)&cfg, '\0', sizeof(cfg));
	cfg.code=DPAA_CFG_CODE_INIT_FQ;
	INIT_FLAG(cfg.init_fq.flags);
	cfg.init_fq.portal= p40xx->portal[0]; /*Portal idx starts from 0*/
#ifdef MULTI_PORTAL		
	cfg.init_fq.channel= CHID_POOLCH_1+p40xx->portal[0];
#else
	cfg.init_fq.channel= CHANNELID_DEFAULT; /*Bind to default channel for this portal*/
#endif
	cfg.init_fq.workq= WQID_DEFAULT; /*Enqueue from single WQ as of now*/
	cfg.init_fq.fqid= FQID_DEFAULT; /*Let DPAA assign the correct default FQID for us. We will save it later */
	cfg.init_fq.fq_handler = &p40xx_rx_err;
	cfg.init_fq.token =&p40xx->token[0];
	cfg.init_fq.token_size = 0;
	cfg.init_fq.num_of_fqid=1;

	if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
			log(LOG_ERR, "%s():dpaa_system_config failed for CODE_INIT_FQ id%d: return code %d\n",
							__FUNCTION__,arg->idx,rc);
			log(LOG_ERR, "Error creating rx_errqid Cause %d",cfg.init_fq_resp.result);
			return -1;
	}
	p40xx->rx_errqid[0]= cfg.init_fq_resp.fqid;

	/*Create our Tx frame Queue ID*/
	memset((void *)&cfg, '\0', sizeof(cfg));
	cfg.code=DPAA_CFG_CODE_INIT_FQ;
	INIT_FLAG(cfg.init_fq.flags);

	cfg.init_fq.portal= p40xx->portal[0]; /*Portal idx starts from 0*/
	cfg.init_fq.channel= TX_IF_TO_CHANNEL_MAP[p40xx->idx]; /*Find the channel corresponding to MAC*/
	cfg.init_fq.workq= WQID_DEFAULT; /*Enqueue from single WQ as of now*/
	cfg.init_fq.fqid= FQID_DEFAULT; /*Let DPAA assign the correct default FQID for us. We will save it later */
	cfg.init_fq.fq_handler = NULL;
	cfg.init_fq.token =NULL;
	cfg.init_fq.token_size = 0;
	cfg.init_fq.num_of_fqid=DPAA_DEF_TXQID_RANGE;

	if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
			log(LOG_ERR, "%s():dpaa_system_config failed for CODE_INIT_FQ id%d: return code %d\n",
							__FUNCTION__,arg->idx,rc);
			log(LOG_ERR, "Error creating transmitQ Cause %d",cfg.init_fq_resp.result);
			return -1;
	}
	p40xx->tx_fqid= cfg.init_fq_resp.fqid;
	/*In order to save the lookup inside DPAA library for each transmitted frame,
	 *driver should use this handle while transmitting the frame*/
	p40xx->tx_handle= cfg.init_fq_resp.handle;
	p40xx->tx_fq_num = DPAA_DEF_TXQID_RANGE;

	/*This section would initialize the Buffer Pools inside the Netcomm Library. The pool information
	 * is currently hardcoded inside the library and we basically just get it. If you want to change the
	 * buffer pool settings, look at the file /lib/dpaa/bman/bman.c #DATA_POOL_SETTINGS*/
	memset((void *)&cfg, '\0', sizeof(cfg));
	cfg.code=DPAA_CFG_CODE_INIT_BPID;
	INIT_FLAG(cfg.init_bpool.flags);
	cfg.init_bpool.portal= p40xx->portal[0];

	if((rc=dpaa_system_config(dpaa, &cfg))!=NULL){
		log(LOG_ERR, "%s():dpaa_system_config failed for DPAA_CFG_CODE_INIT_BPID id%d: return code %d\n",
						__FUNCTION__,arg->idx,rc);
		log(LOG_ERR, "Error Cause %d",cfg.init_bpool_resp.result);
		return -1;
	}
	p40xx->bp_count= cfg.init_bpool_resp.count;

	for(i=0;(i<DPAA_BMAN_MAX_BPOOL) && (i<p40xx->bp_count);i++){
		p40xx->bpid[i]= cfg.init_bpool_resp.bpid[i];
		p40xx->bp_size[i]= cfg.init_bpool_resp.size[i];
	}
	/*Cleanup rest of the information*/
	for(;i<DPAA_BMAN_MAX_BPOOL;i++){
		p40xx->bpid[i]= 0;
		p40xx->bp_size[i]= 0;
	}


	/*Config FMAN Port*/
	memset((void *)&cfg, '\0', sizeof(cfg));
	cfg.code= DPAA_CFG_CODE_CONFIG_PORT;
	INIT_FLAG(cfg.config_port.flags);

	cfg.config_port.portal = p40xx->portal[0];
	cfg.config_port.port_id = arg->idx; /*This is the MAC index of the port*/
	cfg.config_port.tx_cfqid = p40xx->tcfqid[0];
	cfg.config_port.tx_defqid = p40xx->tx_fqid;
	cfg.config_port.rx_defqid = p40xx->rx_fqid[0];
	cfg.config_port.rx_errqid = p40xx->rx_errqid[0]; 
	cfg.config_port.tx_errqid = p40xx->rx_errqid[0];

	/*Note: The first BPID retrieved from DPAA is always the SGT buffer pool
	 * about which we don't send Rx HW any info. HW shall not use that BPID for 
	 * Rx frames.*/
	cfg.config_port.rx_pool_count = 2; 
	cfg.config_port.rx_pvt_data_size = p40xx->rx_buffer_prefix_size;

	for(i=0;i<cfg.config_port.rx_pool_count;i++){
		cfg.config_port.rx_bpid[i]=    p40xx->bpid[   1 + cfg.config_port.rx_pool_count*cfg.config_port.portal + i] ;
		cfg.config_port.rx_buf_size[i]=p40xx->bp_size[1 + cfg.config_port.rx_pool_count*cfg.config_port.portal + i];
	}
	/*For PCD load spreading on RX queues, we tell how many queues we need to create*/
	cfg.config_port.pcdNumOfQs = p40xx->rx_fq_num;
	cfg.config_port.pcdBaseQ = p40xx->rx_fqid[0];

	if((rc=dpaa_system_config(dpaa, &cfg))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_config CODE_CONFIG_PORT failed for id%d: return code %d\n",
						__FUNCTION__,arg->idx,rc);
		log(LOG_ERR, "Error Cause %d",cfg.config_port_resp.result);
		return -1;
	}

	/* Write back the actual private data size. */
	p40xx->rx_buffer_prefix_size = cfg.config_port_resp.rx_pvt_data_size;
	
	p40xx->phy_regs= NULL;
	p40xx->probe_phy=0;
	
	p40xx->isr_err_funcid= -1;

	/* hook up so media devctls work */
	bsd_mii_initmedia(p40xx);

	/*Setup all IFP related stuff right here*/
	ifp->if_softc = p40xx;
	strcpy(ifp->if_xname, p40xx->dev.dv_xname);
	ifp->if_capabilities_rx = IFCAP_CSUM_IPv4 | IFCAP_CSUM_TCPv4 | IFCAP_CSUM_UDPv4;
	ifp->if_capabilities_tx = 0;
	ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | ((p40xx->cfg.flags & NIC_FLAG_MULTICAST) ? IFF_MULTICAST : 0x00);
	ifp->if_ioctl = p40xx_ioctl;
	ifp->if_start = p40xx_start;
	ifp->if_init = p40xx_init;
	ifp->if_stop = p40xx_stop;
	IFQ_SET_READY(&ifp->if_snd);
	if_attach(ifp);
	if((arg->jumbo_1g && !IS_MAC_10GIG(p40xx->idx)) || (arg->jumbo_10g && IS_MAC_10GIG(p40xx->idx)))
		p40xx->ecom.ec_capabilities |= ETHERCAP_JUMBO_MTU;
	ether_ifattach(ifp, nic_cfg->current_address);


	/*Cache will be used to save and lookup PADDR to mbuf pointers*/
	p40xx->cache_handle= cache_map_init();
	
	/*Following interrupts shall only be used by master*/
	if(p40xx->master){
		/*Initialize Common Error interrupt mechanism*/
		if ((rc = interrupt_entry_init(&p40xx->inter_err, 0, NULL,IRUPT_PRIO_DEFAULT)) != EOK) {
			log(LOG_ERR, "%s(): interrupt_entry_init(inter_err) failed: return code %d", 
			__FUNCTION__, rc);
			return -1;
		}
		/*Using kernel masking by default*/
		p40xx->inter_err.func = p40xx_process_common_err_intr;
		p40xx->inter_err.enable = p40xx_enable_common_err_intr;
		p40xx->inter_err.arg = p40xx;
		p40xx->isrp_err = p40xx_common_err_isr;

		/* MAP EISR Register */
		if((p40xx->eisr_reg = mmap_device_memory (NULL,0x1000, PROT_READ | PROT_WRITE | PROT_NOCACHE,
    		MAP_SHARED,arg->immr+CCSR_EISR0_OFFSET)) == MAP_FAILED) {
    		log(LOG_ERR, "%s(): mmap EISR0 regs failed:", __FUNCTION__);
    		return ENOMEM;
    	}	

	}
	else{
		p40xx->eisr_reg=NULL;
	}

#ifdef EARLY_PCD_INIT
	log(LOG_WARNING, "%s(): FM interface %d initializing PCD, portal %d pcd_port %d base_fq %d num_of_qid %d", __FUNCTION__,
		   p40xx->idx, p40xx->portal[0], p40xx->idx, p40xx->rx_fqid[0], p40xx->rx_fq_num);
	/*Enable PCD*/
	/*Master interface is responsible for initializing PCD.
	 * Make absolutely sure that if you are not using the default fmc_config.c
	 * supplied with the BSP, your PCD configuration and port should match the 
	 * actual ports initialized by system. e.g. You can not initialize PCD for 
	 * 10G FM1 port, if you have 4x1SGMII connected with FM1. That is going to 
	 * certainly cause a SIGSEGV as PCD routines donot return an error. */
	req.code=DPAA_RUNTIME_CODE_ENABLE_PCD;
	INIT_FLAG(req.enable_pcd.flags);
	req.enable_pcd.portal = p40xx->portal[0];
	req.enable_pcd.pcd_port = p40xx->idx;
	req.enable_pcd.base_fq = p40xx->rx_fqid[0];
	req.enable_pcd.num_of_qid = p40xx->rx_fq_num;
	if (p40xx->secondary)
		req.enable_pcd.fm_master = 2;
	else if (p40xx->master)
		req.enable_pcd.fm_master = 1;
	else
		req.enable_pcd.fm_master = 0;

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_ENABLE_PCD failed for id%d: return code %d",
					__FUNCTION__,p40xx->idx,rc);
		log(LOG_ERR, "Error Cause %d",req.enable_pcd_resp.result);
		return -1;
	}
#endif

	p40xx_mbuf_list_init(p40xx);

	/*All done, nicely attached */
	return EOK;
}

/*****************************************************************************/
/*  Function Name	: p40xx_qman_default
 *	Input			: Address of p40xx device structure 
 *	Description		: Invoked for default handling of ISRs in QMAN SW portal
 *	Returns			: void 
 *	Note			: TBD  
 */
/*****************************************************************************/
static void 
p40xx_qman_default(void *arg , uint32_t *size){

	p40xx_dev_t *p40xx= arg;
	log(LOG_ERR, "Unhandled interrupt invoked %s for idx %d\n",__FUNCTION__,p40xx->idx);

}

/*****************************************************************************/
/*  Function Name	: p40xx_bman_default
 *	Input			: Address of p40xx device structure 
 *	Description		: Invoked for default handling of ISRs in BMAN SW Portal 
 *	Returns			: void 
 *	Note			: What this function should do ?
 *					  1. Check if ISR is triggered because of buffer depletion Entry/Exit.
 *					  2. TBD
 */
/*****************************************************************************/
static void 
p40xx_bman_default(void *arg, uint32_t *size){

	p40xx_dev_t *p40xx= arg;
	log(LOG_ERR, "Unhandled interrupt invoked %s for idx %d\n",__FUNCTION__,p40xx->idx);

}


/*****************************************************************************/
/*  Function Name	: vtophys
 *	Input			: Virtual address in 32 bits
 *	Description		: converts 32 bit vaddr to 64 bit PADDR 
 *	Returns			: PADDR if successful, -1 otherwise. 
 *	Note			: Physical Addresses in Corenet are 48bit long.	Upper 16 
 *					  Bits are mostly ZERO but not guaranteed. 
 */
/*****************************************************************************/
uint64_t 
vtophys (void *addr) {
    off64_t  offset;

    if (mem_offset64 (addr, NOFD, 1, &offset, 0) == -1) {
        return (-1);
    }
    return (offset);
}

/*****************************************************************************/
/*  Function Name	: p40xx_destroy
 *  Input			: Pointer to P40xx structure for the interface
 *  				  destroy mode
 *	Description		: This function is responsible for taking out the interface. 
 *	Returns			: void
 *	Note			: Its different than shutdown, where you expect some of the 
 *					  species left to recreate your world.
 */
/*****************************************************************************/
static void 
p40xx_destroy(p40xx_dev_t *p40xx){

	struct ifnet		*ifp;
	struct _iopkt_self	*iopkt;
	bool dpaa_shutdown_needed=false;
	ifp = &p40xx->ecom.ec_if;
	iopkt = p40xx->iopkt;
	int i=0;

	/*
	 * Stop further init() calls while we're dying.  Yes it can happen:
	 * ether_ifdetach() calls bridge_ifdetach() which
	 * tries to take us out of promiscuous mode with an
	 * init().
	 */
	p40xx->dying=1;
	log(LOG_INFO, "%s invoked for idx %d\n",__FUNCTION__,p40xx->idx);
	/*Reuse the stop function to stop interface*/
	p40xx_stop(ifp,0);
#ifdef LOG_BMAN_BUF_ADDR
	p40xx_buf_dump(p40xx);
#endif	
	cache_map_destroy(p40xx->cache_handle);

	ether_ifdetach(ifp);
	if_detach(ifp);
	for(i=0;i<p40xx->num_of_portals;i++){
		interrupt_entry_remove(&p40xx->inter_qman[i], NULL);
		interrupt_entry_remove(&p40xx->inter_bman[i], NULL);
	}
	if(p40xx->master){
		interrupt_entry_remove(&p40xx->inter_fman1, NULL);
		interrupt_entry_remove(&p40xx->inter_fman2, NULL);
		interrupt_entry_remove(&p40xx->inter_err, NULL);
		dpaa_shutdown_needed=true;
		munmap_device_memory(p40xx->eisr_reg,0x1000);
	}
	shutdownhook_disestablish(p40xx->sdhook);
	/*Finally kill the DPAA if there is no user*/
	if(dpaa_shutdown_needed){
		dpaa_system_shutdown(p40xx->dpaa,0);
	}
	p40xx_mbuf_list_finish(p40xx);

	return;

}

/*****************************************************************************/
/*  Function Name	: p40xx_shutdown
 *  Input			: Pointer to P40xx structure for the interface
 *	Description		: Shutdown the interface and kill everything
 *	Returns			: Empty void 
 *	Note			: 
 */
/*****************************************************************************/
static void 
p40xx_shutdown(void *arg){
	
	p40xx_dev_t *p40xx= (p40xx_dev_t *)arg;
	p40xx_destroy(p40xx);

}

/*****************************************************************************/
/*  Function Name	: p40xx_detach
 *	Input			: Pointer to parent device
 *					  flags
 *	Description		: Detaches your driver from io-pkt.  
 *	Returns			: ZERO if successful, Non ZERO otherwise. 
 *	Note			: Though detach is never invoked by io-pkt, 
 *					  still provide a cleanup implementation
 */
/*****************************************************************************/
static int 
p40xx_detach(struct device *dev, int flags){
	p40xx_dev_t	*p40xx = ( p40xx_dev_t	*)dev;
	p40xx_destroy(p40xx);	
	return EOK;
}


/*****************************************************************************/
/*  Function Name	: p40xx_init
 *	Input			: Pointer to ifnet
 *	Description		: This is called when ifconfig is invoked on interface.				
 *	Returns			: ZERO if OK, else NONZERO 
 *	Note			: Must ensure that this function, and everything it calls
					  is REALLY ok to call over and over again, because the stack
					  will call us MORE THAN ONCE. 
 */
/*****************************************************************************/
static int 
p40xx_init (struct ifnet *ifp)
{
	p40xx_dev_t		*p40xx = ifp->if_softc;
	nic_config_t		*cfg = &p40xx->cfg;
	int32_t rc=0;
#ifndef EARLY_PCD_INIT
	uint32_t idx=p40xx->idx;
	dpaa_handle_t dpaa= p40xx->dpaa;	
#endif	
	dpaa_runtime_t req;
	int i=0;
	
	log(LOG_INFO, "%s(): idx %d dev_idx %d", __FUNCTION__, p40xx->idx, p40xx->cfg.device_index);

	if (ifp->if_capenable_rx & ~(IFCAP_CSUM_IPv4 | IFCAP_CSUM_TCPv4 | IFCAP_CSUM_UDPv4) ) {
			log(LOG_ERR, "Not supported Rx Capability %llx\n",ifp->if_capenable_rx);
			return 1;

    }
    if (ifp->if_capenable_tx & ~(IFCAP_CSUM_IPv4 | IFCAP_CSUM_TCPv4 | IFCAP_CSUM_UDPv4) ) {
			log(LOG_ERR, "Not supported Tx capability %llx\n",ifp->if_capenable_tx);
			return 1;
    }

	if(p40xx->dying){
		log(LOG_ERR, "%s():Already dying", __FUNCTION__);
		return 1;
	}
	/*Gracefully stop the Rx and Tx transmission*/
	p40xx_stop(ifp,0);
	
	/*First update our parameter list as provided by IFP */
	cfg->mtu = cfg->mru = ifp->if_mtu;
   	
	p40xx->rx_cap_mask = 0;
    if (ifp->if_capenable_rx & IFCAP_CSUM_IPv4) {
        p40xx->rx_cap_mask |= M_CSUM_IPv4 | M_CSUM_IPv4_BAD;
    }
    if (ifp->if_capenable_rx & IFCAP_CSUM_TCPv4) {
        p40xx->rx_cap_mask |= M_CSUM_TCPv4 | M_CSUM_TCP_UDP_BAD;
    }
    if (ifp->if_capenable_rx & IFCAP_CSUM_UDPv4) {
        p40xx->rx_cap_mask |= M_CSUM_UDPv4 | M_CSUM_TCP_UDP_BAD;
    }
  	
	p40xx->tx_cap_mask = 0;
    if (ifp->if_capenable_tx & IFCAP_CSUM_IPv4) {
        p40xx->tx_cap_mask |= M_CSUM_IPv4 | M_CSUM_IPv4_BAD;
    }
    if (ifp->if_capenable_tx & IFCAP_CSUM_TCPv4) {
        p40xx->tx_cap_mask |= M_CSUM_TCPv4 | M_CSUM_TCP_UDP_BAD;
    }
    if (ifp->if_capenable_tx & IFCAP_CSUM_UDPv4) {
        p40xx->tx_cap_mask |= M_CSUM_UDPv4 | M_CSUM_TCP_UDP_BAD;
    }
	


	p40xx_clear_stats(p40xx);

    p40xx_init_phy(p40xx);

	for(i=0;i<p40xx->num_of_portals;i++){
			/*This should have enabled transmission and reception from that port */
			/*Attach directly to Interrupt lines.*/
			if (p40xx->isr_qman_funcid[i] == -1) {
					if (( p40xx->isr_qman_funcid[i] = InterruptAttach_r(p40xx->iid_qman_portal[i], p40xx->isrp_qman,
													&p40xx->token[i], sizeof(p40xx_dev_t), _NTO_INTR_FLAGS_TRK_MSK)) < 0) {
							log(LOG_ERR, "%s(): InterruptAttach_r(qman) failed: %d", 
											__FUNCTION__, p40xx->isr_qman_funcid[i]);
							return -1;
					}
			}

			if (p40xx->isr_bman_funcid[i] == -1) {
					if (( p40xx->isr_bman_funcid[i] = InterruptAttach_r(p40xx->iid_bman_portal[i], p40xx->isrp_bman,
													&p40xx->token[i], sizeof(p40xx_dev_t), _NTO_INTR_FLAGS_TRK_MSK)) < 0) {
							log(LOG_ERR, "%s(): InterruptAttach_r(bman) failed: %d", 
											__FUNCTION__, p40xx->isr_bman_funcid[i]);
							return -1;
					}
			}
	}

	req.code= DPAA_RUNTIME_CODE_ENABLE_PORT;
	INIT_FLAG(req.enable_port.flags);
	req.enable_port.portal = p40xx->portal[0];
	req.enable_port.port_id = p40xx->idx;

	if((rc=dpaa_system_runtime(p40xx->dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_ENABLE_PORT failed for id%d: return code %d",
						__FUNCTION__,p40xx->idx,rc);
		log(LOG_ERR, "Error Cause %d\n",req.enable_port_resp.result);
		return -1;
	}
	
#ifndef EARLY_PCD_INIT
	log(LOG_WARNING, "%s(): FM interface %d initializing PCD, portal %d pcd_port %d base_fq %d num_of_qid %d", __FUNCTION__,
		   p40xx->idx, p40xx->portal[0], p40xx->idx, p40xx->rx_fqid[0], p40xx->rx_fq_num);
	/*Enable PCD*/
	/*Master interface is responsible for initializing PCD.
	 * Make absolutely sure that if you are not using the default fmc_config.c
	 * supplied with the BSP, your PCD configuration and port should match the 
	 * actual ports initialized by system. e.g. You can not initialize PCD for 
	 * 10G FM1 port, if you have 4x1SGMII connected with FM1. That is going to 
	 * certainly cause a SIGSEGV as PCD routines donot return an error. */
	req.code=DPAA_RUNTIME_CODE_ENABLE_PCD;
	INIT_FLAG(req.enable_pcd.flags);
	req.enable_pcd.portal = p40xx->portal[0];
	req.enable_pcd.pcd_port = p40xx->idx;
	req.enable_pcd.base_fq = p40xx->rx_fqid[0];
	req.enable_pcd.num_of_qid = p40xx->rx_fq_num;
	if (p40xx->secondary)
		req.enable_pcd.fm_master = 2;
	else if (p40xx->master)
		req.enable_pcd.fm_master = 1;
	else
		req.enable_pcd.fm_master = 0;

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_ENABLE_PCD failed for id%d: return code %d",
					__FUNCTION__,idx,rc);
		log(LOG_ERR, "Error Cause %d",req.enable_pcd_resp.result);
		return -1;
	}
#endif

	if(p40xx->master){
		if (p40xx->isr_fman1_funcid == -1) {
			if (( p40xx->isr_fman1_funcid = InterruptAttach_r(p40xx->iid_fman1, p40xx->isrp_fman1,
		    p40xx, sizeof(p40xx_dev_t), _NTO_INTR_FLAGS_TRK_MSK)) < 0) {
				log(LOG_ERR, "%s(): InterruptAttach_r(fman1) failed: %d", 
				__FUNCTION__, p40xx->isr_fman1_funcid);
				return -1;
			}
		}
		if (p40xx->isr_fman2_funcid == -1) {
			if (( p40xx->isr_fman2_funcid = InterruptAttach_r(p40xx->iid_fman2, p40xx->isrp_fman2,
		    p40xx, sizeof(p40xx_dev_t), _NTO_INTR_FLAGS_TRK_MSK)) < 0) {
				log(LOG_ERR, "%s(): InterruptAttach_r(fman2) failed: %d", 
				__FUNCTION__, p40xx->isr_fman2_funcid);
				return -1;
			}
		}
		if (p40xx->isr_err_funcid == -1) {
			if (( p40xx->isr_err_funcid = InterruptAttach_r(p40xx->iid_err, p40xx->isrp_err,
		    p40xx, sizeof(p40xx_dev_t), _NTO_INTR_FLAGS_TRK_MSK)) < 0) {
				log(LOG_ERR, "%s(): InterruptAttach_r(Common Error) failed: %d", 
				__FUNCTION__, p40xx->isr_err_funcid);
				return -1;
			}
		}
	}

	/* set promiscuous or multicast mode */
 	p40xx_filter(p40xx, 0, NULL, NULL);

	NW_SIGLOCK(&ifp->if_snd_ex, p40xx->iopkt);
	ifp->if_flags_tx |= IFF_RUNNING;
	NW_SIGUNLOCK(&ifp->if_snd_ex, p40xx->iopkt);
	ifp->if_flags |= IFF_RUNNING;
	

	return EOK;
}

/*****************************************************************************/
/*  Function Name	: p40xx_stop
 *	Input			: Pointer to ifnet
 *					  Disable mode
 *	Description		: stops transmission and reception on interface gracefully				
 *	Returns			: void 
 *	Note			: Look at p40xx_init and try undo everything done in init.
 *					  Init would call this function everytime so both of these 
 *					  functions should be in sync wrt DPAA usage
 */
/*****************************************************************************/
static void 
p40xx_stop(struct ifnet *ifp, int disable){
	
	p40xx_dev_t	*p40xx = ifp->if_softc;
	dpaa_runtime_t req;
	int i=0;
	int rc=0;
	
	log(LOG_INFO, "%s(): idx %d dev_idx %d", __FUNCTION__, p40xx->idx, p40xx->cfg.device_index);

	callout_stop(&p40xx->mii_callout);


	/**/
	if (p40xx->mdi) {
		MDI_DeRegister(&p40xx->mdi);
		p40xx->mdi = NULL;
	}

	if ((p40xx->emu_phy != -1) && (p40xx->emu_phy == p40xx->cfg.device_index)) {
		emu_phy_dinit (p40xx);
		}

	NW_SIGLOCK(&ifp->if_snd_ex, p40xx->iopkt);

	for (i = 0; i < 10; i++) {
		if ((ifp->if_flags_tx & IFF_OACTIVE) == 0)
			break;
		NW_SIGUNLOCK(&ifp->if_snd_ex, p40xx->iopkt);
		delay(50);
		NW_SIGLOCK(&ifp->if_snd_ex, p40xx->iopkt);
	}

	if (i < 10) {
		ifp->if_flags_tx &= ~IFF_RUNNING;
	} else {
		/* Heavy load or bad luck.  Try the big gun. */
		quiesce_all();
		ifp->if_flags_tx &= ~IFF_RUNNING;
		unquiesce_all();
	}
	NW_SIGUNLOCK(&ifp->if_snd_ex, p40xx->iopkt);
	
	/*As per intended use case, DPAA ports are not designed to be shutdown and restarted at runtime.
	 * So we dont disable the transmission actually. We remove our interrupts so that no further 
	 * packets can be received, and hopefully stack doesnt sends any thing after we clear IFF_RUNNING
	 * flag.*/
	/*Disable transmission of frames*/
	req.code= DPAA_RUNTIME_CODE_DISABLE_PORT;
	INIT_FLAG(req.disable_port.flags);
	req.disable_port.portal = p40xx->portal[0];
	req.disable_port.port_id = p40xx->idx;

	if((rc=dpaa_system_runtime(p40xx->dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_DISABLE_PORT failed for id%d: return code %d",
						__FUNCTION__,p40xx->idx,rc);
		log(LOG_ERR, "Error Cause %d\n",req.disable_port_resp.result);
		/*Donot return but do rest of the cleanup*/
	}

    /*This should have disabled transmission and reception */
	/*Now check if there are any pending packets in the Rx queue before detaching our ISR*/
	for(i=0;i<p40xx->num_of_portals;i++){
	
			if(p40xx->isr_bman_funcid[i]!=-1){
					InterruptDetach(p40xx->isr_bman_funcid[i]);
					p40xx->isr_bman_funcid[i] =-1;
			}
			if(p40xx->isr_qman_funcid[i]!=-1){
					InterruptDetach(p40xx->isr_qman_funcid[i]);
					p40xx->isr_qman_funcid[i] =-1;
			}
	}
	if(p40xx->master){

		if(p40xx->isr_fman1_funcid!=-1){
			InterruptDetach(p40xx->isr_fman1_funcid);
			p40xx->isr_fman1_funcid =-1;
		}
		if(p40xx->isr_fman2_funcid!=-1){
			InterruptDetach(p40xx->isr_fman2_funcid);
			p40xx->isr_fman2_funcid =-1;
		}
		if(p40xx->isr_err_funcid!=-1){
			InterruptDetach(p40xx->isr_err_funcid);
			p40xx->isr_err_funcid =-1;
		}
	}	

	/* Mark the interface as down and cancel the watchdog timer. */
	ifp->if_flags &= ~(IFF_RUNNING | IFF_OACTIVE);
	ifp->if_timer = 0;

	return;

}

/*****************************************************************************/
/*  Function Name	: p40xx_get_syspage_params
 *	Input			: Pointer to dpaa_startup_params_t
 *	Description		: Read system page and put the parameters into cfg				
 *	Returns			: void  
 *	Note			: 
 */
/*****************************************************************************/
void 
p40xx_get_syspage_params(dpaa_startup_params_t *dpaa_cfg) {
    int32_t     pixis_off, bman_off, qman_off, off;

	force_default_param(dpaa_cfg);

    pixis_off = hwi_find_device("pixis", 0);
    off = hwi_find_tag(pixis_off, 1, HWI_TAG_NAME_location); /* PIXIS hw registers */
    if(pixis_off != HWI_NULL_OFF && off != HWI_NULL_OFF)
		log(LOG_INFO, "%s(): pixis: regs 0x%llx len %d", __FUNCTION__, ((struct hwi_location *)(hwi_off2tag(off)))->base, ((struct hwi_location *)(hwi_off2tag(off)))->len);
    if(off != HWI_NULL_OFF) off = hwi_find_tag(off, 1, HWI_TAG_NAME_location); /* PIXIS registers */
    if(pixis_off != HWI_NULL_OFF && off != HWI_NULL_OFF) {
		log(LOG_INFO, "%s(): pixis: base 0x%llx len %d", __FUNCTION__, ((struct hwi_location *)(hwi_off2tag(off)))->base, ((struct hwi_location *)(hwi_off2tag(off)))->len);
		dpaa_cfg->pixis_base = ((struct hwi_location *)(hwi_off2tag(off)))->base;
		dpaa_cfg->pixis_size = ((struct hwi_location *)(hwi_off2tag(off)))->len;
	}

	bman_off = hwi_find_device("bman", 0);
	off = hwi_find_tag(bman_off, 1, HWI_TAG_NAME_location); /* BMAN hw registers */
	if(bman_off != HWI_NULL_OFF && off != HWI_NULL_OFF)
		log(LOG_INFO, "%s(): bman: regs 0x%llx len %d", __FUNCTION__, ((struct hwi_location *)(hwi_off2tag(off)))->base, ((struct hwi_location *)(hwi_off2tag(off)))->len);
	if(off != HWI_NULL_OFF) off = hwi_find_tag(off, 1, HWI_TAG_NAME_location); /* BMAN portals API */
	if(bman_off != HWI_NULL_OFF && off != HWI_NULL_OFF) {
		log(LOG_INFO, "%s(): bman: base 0x%llx len %d", __FUNCTION__, ((struct hwi_location *)(hwi_off2tag(off)))->base, ((struct hwi_location *)(hwi_off2tag(off)))->len);
		dpaa_cfg->bman_portal_base = ((struct hwi_location *)(hwi_off2tag(off)))->base;
		dpaa_cfg->bman_portal_size = ((struct hwi_location *)(hwi_off2tag(off)))->len;
	}

	qman_off = hwi_find_device("qman", 0);
	off = hwi_find_tag(qman_off, 1, HWI_TAG_NAME_location); /* QMAN hw registers */
	if(qman_off != HWI_NULL_OFF && off != HWI_NULL_OFF)
		log(LOG_INFO, "%s(): qman: regs 0x%llx len %d", __FUNCTION__, ((struct hwi_location *)(hwi_off2tag(off)))->base, ((struct hwi_location *)(hwi_off2tag(off)))->len);
	if(off != HWI_NULL_OFF) off = hwi_find_tag(off, 1, HWI_TAG_NAME_location); /* QMAN portals API */
	if(qman_off != HWI_NULL_OFF && off != HWI_NULL_OFF) {
		log(LOG_INFO, "%s(): qman: base 0x%llx len %d", __FUNCTION__, ((struct hwi_location *)(hwi_off2tag(off)))->base, ((struct hwi_location *)(hwi_off2tag(off)))->len);
		dpaa_cfg->qman_portal_base = ((struct hwi_location *)(hwi_off2tag(off)))->base;
		dpaa_cfg->qman_portal_size = ((struct hwi_location *)(hwi_off2tag(off)))->len;
	}

    return ;
}

/*TBD: This is an auxillary function which should be removed once startup is populated. */
void 
force_default_param(dpaa_startup_params_t *dpaa_cfg){

    struct      asinfo_entry  *asinfo;
    int32_t     i,j, num, cnt;
	uint8_t mac_addr_base[6];

    num = _syspage_ptr->asinfo.entry_size / sizeof(*asinfo);
    asinfo = SYSPAGE_ENTRY (asinfo);

    for (i = cnt = 0; i < num; ++i) {
        char *name;
        name = __hwi_find_string (asinfo->name);
        if (strcmp (name, "immr")) {
            asinfo++;
            continue;
        }
		else {
         	dpaa_cfg->immr= asinfo->start;
			break;
		}
    }
	if(i==num)
   	{	
		log(LOG_ERR, "%s(): couldn't retreive immr base from startup hwinfo", __FUNCTION__);
		return ;
	}
	dpaa_cfg->verbose=LOG_WARNING ;
	dpaa_cfg->loopback= 0;

	dpaa_cfg->emu_phy= -1;
	dpaa_cfg->portal_ownership= DPAA_SW_PORTAL_ALL; /*This is a bitmap specifying which portal to bind with*/

	/* Write all default values for startup parameters here */
	
	dpaa_cfg->pixis_base = 0xfffdf0000ULL;  /*Hardcoded in LAW*/
	dpaa_cfg->pixis_size = 0x00001000;

	dpaa_cfg->bman_portal_base = 0xff4000000ULL;  /*Hardcoded in LAW*/
	dpaa_cfg->bman_portal_size = 0x00200000;
	
	dpaa_cfg->qman_portal_base = 0xff4200000ULL;  /*Hardcoded in LAW*/
	dpaa_cfg->qman_portal_size = 0x00200000;

	dpaa_cfg->txbuf_memsize= 64*1024; /*Allocate 64k for Transmit(Not used currently so keep it minimal)*/
	dpaa_cfg->rxbuf_memsize= 32*1024*1024; /*Allocate 32MB for Receive*/

	/*BMAN specific configuration*/
	dpaa_cfg->fbpr_memsize = 32*1024*1024; /*32M*/


	/*QMAN specific configuration*/
	dpaa_cfg->fqdr_memsize=32*1024*1024; /*32M*/
    dpaa_cfg->pfdr_memsize=32*1024*1024; /*32M*/

    dpaa_cfg->fqid_base=DPAA_DEF_PORTAL_FQID_BASE; 
    dpaa_cfg->total_fqid=MAX_FQIDS; 
	
	if(nic_get_syspage_mac((char *)mac_addr_base) < 0) {
		struct timespec time;

		clock_gettime( CLOCK_REALTIME, &time);
		time.tv_nsec += time.tv_sec; /* include seconds in hash */
		log(LOG_ERR, "%s(): Rand %lx", __FUNCTION__, time.tv_nsec);

		/*Use a random MAC address here if no hwinfo (network/nicaddr)*/
		mac_addr_base[0]=0 ;
		mac_addr_base[1]=4;
		mac_addr_base[2]=0x9f ;
		mac_addr_base[3]=(0x31 + ((time.tv_nsec & 0xFF0000)>>16))%0xff;
		mac_addr_base[4]=(0x45 + ((time.tv_nsec & 0xFF00)>>8))%0xff;
		mac_addr_base[5]=(0xab + (time.tv_nsec & 0xFF))%0xff;
		log(LOG_INFO,"No syspage nicaddr found, using generated base MAC %02x:%02x:%02x:%02x:%02x:%02x",
			mac_addr_base[0],
			mac_addr_base[1],
			mac_addr_base[2],
			mac_addr_base[3],
			mac_addr_base[4],
			mac_addr_base[5]);
	}

	dpaa_cfg->fm_enabled[0]=1; /*FM0 Enabled, SGMII Riser slot #5 connected to FM0 in P4080DS*/
	dpaa_cfg->fm_enabled[1]=1; /*FM1 enabled as well*/
   
	for (i = 0 ; i < MAX_FMAN_PER_DPAA; ++i) {
		for (j = 0 ; j < MAX_MACDEV_PER_FMAN; ++j) {
			memcpy(&dpaa_cfg->mac_addr[i][j][0],&mac_addr_base[0],MAC_ADDR_SIZE);
			dpaa_cfg->mac_addr[i][j][5] =  (dpaa_cfg->mac_addr[i][j][5] + i*MAX_MACDEV_PER_FMAN+j)% 0xff;

			dpaa_cfg->macdev_enable[i][j]=0;

			if( j==(MAX_MACDEV_PER_FMAN-1) ){
				dpaa_cfg->phy_if[i][j]= PHY_IF_XGMII;
			}
			else{
				dpaa_cfg->phy_if[i][j]= PHY_IF_SGMII;

			}

		}
	}
	/* This depends on the board configuration */
	/*We currently donot support RGMII PHY*/
	/*dpaa_cfg->phy_if[1][0]= PHY_IF_RGMII; */

	/* Phy information for P4080DS Board */
	dpaa_cfg->phy_addr[0][0]= 0x1c;
	dpaa_cfg->phy_addr[0][1]= 0x1d;
	dpaa_cfg->phy_addr[0][2]= 0x1e;
	dpaa_cfg->phy_addr[0][3]= 0x1f;
	dpaa_cfg->phy_addr[0][4]= 0x04;
	dpaa_cfg->phy_addr[1][0]= 0x1c;
	dpaa_cfg->phy_addr[1][1]= 0x1d;
	dpaa_cfg->phy_addr[1][2]= 0x1e;
	dpaa_cfg->phy_addr[1][3]= 0x1f;
	dpaa_cfg->phy_addr[1][4]= 0x00;

	dpaa_cfg->tbi_addr[0][0]=0x8;
	dpaa_cfg->tbi_addr[1][0]=0x14;

	dpaa_cfg->imode= false;
	dpaa_cfg->master_if=DPAA_MASTER_IF;
	dpaa_cfg->secondary_if=-1;

 	dpaa_cfg->fm0_1x4gig =0;
	dpaa_cfg->fm0_10gig  =0;
	dpaa_cfg->fm1_1x4gig =0;
	dpaa_cfg->fm1_10gig  =0;
	
	dpaa_cfg->no_phy =0;
	dpaa_cfg->rx_loop =0;
	dpaa_cfg->dump_regs = 0;


	/*Populate interrupt ID information*/
	return ;
}
/*****************************************************************************/
/*  Function Name	: p40xx_parse_options
 *	Input			: Pointer to the configuration parameters of DPAA 
 *					  character string passed by user on command line 
 *	Description		: Read any override parameters from command line and put
 *					  into cfg structure.
 *	Returns			: ZERO if ok, else nonZERO  
 *	Note			: Index should match the string literal on top of this file.
 */
/*****************************************************************************/
static int 
p40xx_parse_options (dpaa_startup_params_t *cfg, nic_config_t *nic_cfg, char *options){
    int32_t     rc = EOK;
    char    	*value=NULL;
	char 		*restore=NULL;
	char 		*c=NULL;
    int32_t     opt;
	int32_t     i,j;
	uint8_t mac_addr_base[6];

	restore = NULL;

   
	while (options && *options != '\0') {
	
	c = options;
        
		restore = strchr(options, ',');
        opt = getsubopt (&options, p40xx_opts, &value);
        switch (opt) {
          case 0:
            if (cfg) {
				if(value) {
					int i;
                	cfg->phy_addr[0][0] = strtoul (value, 0, 0);
					/* 	If the phy address is specified by the command line
					 	it is assumed that the Phys are addressed linearly */
					for(i = 1 ; i < MAX_MACDEV_PER_FMAN; i++) {
                		cfg->phy_addr[0][i] = 1 + cfg->phy_addr[0][i-1];
					}
				}
            }
            break;
          case 1:
            if (cfg) {
				if(value) {
					int i;
                	cfg->phy_addr[1][0] = strtoul (value, 0, 0);
					/* 	If the phy address is specified by the command line
					 	it is assumed that the Phys are addressed linearly */
					for(i = 1 ; i < MAX_MACDEV_PER_FMAN; i++) {
                		cfg->phy_addr[1][i] = 1 + cfg->phy_addr[1][i-1];
					}
				}
            }
            break;

        case 2:
            if (cfg) {
				if (value) {
					cfg->emu_phy = strtoul(value, 0, 0);
				}
            }
            break;
        case 3:
            if (cfg) {
				if (value) {
					cfg->verbose = strtoul(value, 0, 0);
				}
            }
            break;
        case 4: /*10g_fm1*/

            if (cfg) {
					cfg->fm0_10gig =1;
            }
            break;
        case 5: /*4x1g_fm1*/

            if (cfg) {
					cfg->fm0_1x4gig =0xf;
            }
            break;
        case 6: /*10g_fm2*/
            if (cfg) {
					cfg->fm1_10gig =1;
            }
            break;

        case 7:  /*4x1g_fm2*/
            if (cfg) {
					cfg->fm1_1x4gig =0xf;
            }
            break;

        case 11: /*2x1g_fm2*/
            if (cfg) {
					cfg->fm1_1x4gig =0xc; // DSEC 2-3
            }
            break;

        case 8:  /*no_phy*/
            if (cfg) {
					cfg->no_phy =1;
            }
            break;
        case 9:  /*rx_loop*/
            if (cfg) {
					cfg->rx_loop =1;
            }
            break;

        case 10:  /*dump_regs*/
            if (cfg) {
					cfg->dump_regs = 1;
            }
            break;

        case 12:  /*jumbo_1g*/
            if (cfg) {
					cfg->jumbo_1g = 1;
            }
            break;

        case 13:  /*jumbo_10g*/
            if (cfg) {
					cfg->jumbo_10g = 1;
            }
            break;

        case 14:  /*jumbo*/
            if (cfg) {
					cfg->jumbo_1g  = 1;
					cfg->jumbo_10g = 1;
            }
            break;

         default:
	    	if (nic_parse_options (nic_cfg, value) != EOK) {
                log(LOG_ERR, "%s(): unknown option %s", __FUNCTION__, c);
                rc = EINVAL;
	    	}
            break;
        }
        if (restore != NULL)
            *restore = ',';

    }
 
 	/* The following new MAC settings are only used for p40xx_detect()*/
	if((nic_cfg->current_address[0] ||\
		 nic_cfg->current_address[1] ||\
		 nic_cfg->current_address[2] ||\
		 nic_cfg->current_address[3] ||\
		 nic_cfg->current_address[4] ||\
		 nic_cfg->current_address[5])) {

		log(LOG_INFO,"Overriding MAC with %02x:%02x:%02x:%02x:%02x:%02x",
			nic_cfg->current_address[0],
			nic_cfg->current_address[1],
			nic_cfg->current_address[2],
			nic_cfg->current_address[3],
			nic_cfg->current_address[4],
			nic_cfg->current_address[5]);

			/*MAC address are hardcoded here. Change and read from HWInfo*/
			mac_addr_base[0]=nic_cfg->current_address[0];
			mac_addr_base[1]=nic_cfg->current_address[1];
			mac_addr_base[2]=nic_cfg->current_address[2];
			mac_addr_base[3]=nic_cfg->current_address[3];
			mac_addr_base[4]=nic_cfg->current_address[4];
			mac_addr_base[5]=nic_cfg->current_address[5];

			for (i = 0 ; i < MAX_FMAN_PER_DPAA; ++i) {
				for (j = 0 ; j < MAX_MACDEV_PER_FMAN; ++j) {
					memcpy(&cfg->mac_addr[i][j][0],&mac_addr_base[0],MAC_ADDR_SIZE);
					cfg->mac_addr[i][j][5]=  (cfg->mac_addr[i][j][5]+ i*MAX_MACDEV_PER_FMAN+j)% 0xff;

				}
			}
 	}
    return (rc);
}


/*****************************************************************************/
/*  Function Name	: p40xx_validate_params
 *	Input			: DPAA configuration parameters to be validated
 *	Description		: Validate the startup parameters and correct them if they can 
 *					  still be usable. Else return error so that user can be warned
 *					  that it is trying to configure system incorrectly.
 *	Returns			: ZERO if ok, else nonZERO  
 *	Note			:
 */
/*****************************************************************************/
static int 
p40xx_validate_params (dpaa_startup_params_t *cfg){
	/*Currently no validation. Add checks here to avoid starting up with some 
	 * misisng or incorrect critical parameters. */

    int32_t     rc = EOK;
	int			i, autocfg=0, nlane;
	char		buf[8];

	if(cfg->fm0_1x4gig==0 && cfg->fm0_10gig==0 && cfg->fm1_1x4gig==0 && cfg->fm1_10gig==0){
		/*User has not specified any thing. Switching to autoconfig mode*/
        log(LOG_INFO, "%s(): No configuration specified. Switching to autoconfig mode.", __FUNCTION__);
		autocfg = 1;
	}
	if(p40xx_defconfig(cfg, autocfg)!=EOK){
		log(LOG_ERR, "%s(): Unable to determine autoconfig. Quitting!", __FUNCTION__);
		return EINVAL;
	}
	if(cfg->fm0_1x4gig==0 && cfg->fm0_10gig==0 && cfg->fm1_1x4gig==0 && cfg->fm1_10gig==0){
        log(LOG_ERR, "%s(): No interfaces found to configure. Quitting!", __FUNCTION__);
		return EINVAL;
	}

	cfg->secondary_if = -1;
	if (cfg->fm1_10gig) {
			log(LOG_INFO, "%s(): Enabling fm2_10gig MAC", __FUNCTION__);
			cfg->macdev_enable[1][4]=1;
			cfg->master_if=mac_to_portal_map[9];
			cfg->secondary_if=cfg->master_if;
	}
	if (cfg->fm1_1x4gig) {
			nlane = fmt_4x1g_lanes(cfg->fm1_1x4gig, buf);
			log(LOG_INFO, "%s(): Enabling fm2_%dx1gig MACs (%s)", __FUNCTION__, nlane, buf);
			for(i=3; i>=0; i--) {
				if(cfg->fm1_1x4gig & (1<<i)) {
					cfg->macdev_enable[1][i] = 1;
					cfg->master_if=mac_to_portal_map[i+5];
					cfg->secondary_if=cfg->master_if;
				}
			}
			cfg->secondary_if=cfg->master_if;
	}
	if (cfg->fm0_10gig) {
			log(LOG_INFO, "%s(): Enabling fm1_10gig MAC", __FUNCTION__);
			cfg->macdev_enable[0][4]=1;
			cfg->master_if=mac_to_portal_map[4];
	}
	if (cfg->fm0_1x4gig) {
			nlane = fmt_4x1g_lanes(cfg->fm0_1x4gig, buf);
			log(LOG_INFO, "%s(): Enabling fm1_%dx1gig MACs (%s)", __FUNCTION__, nlane, buf);
			for(i=3; i>=0; i--) {
				if(cfg->fm0_1x4gig & (1<<i)) {
					cfg->macdev_enable[0][i] = 1;
					cfg->master_if=mac_to_portal_map[i];
				}
			}
	}
	if(cfg->master_if == cfg->secondary_if)
		cfg->secondary_if = -1;

	log(LOG_INFO, "%s(): Selected master_if portal %d", __FUNCTION__, cfg->master_if);
	if(cfg->secondary_if != -1)
		log(LOG_INFO, "%s(): Selected secondary_if portal %d", __FUNCTION__, cfg->secondary_if);

	return rc;
}
/*****************************************************************************/
/*  Function Name	: p40xx_speeduplex
 *	Input			: Pointer to device structure
 *	Description		: Sets the MAC speed as per received PHY autonegotiation result 
 *	Returns			: None  
 *	Note			:
 */
/*****************************************************************************/
void    
p40xx_speeduplex (p40xx_dev_t *p40xx){
	dpaa_runtime_t req;
	int32_t rc;
	dpaa_handle_t dpaa=p40xx->dpaa;
	nic_config_t        *cfg  = &p40xx->cfg;
	if(	p40xx->verbose>=LOG_DEBUG_CONFIG)	
	log(LOG_INFO, "%s, Called with Duplex %d Media rate %d",__FUNCTION__, cfg->duplex, cfg->media_rate);
	/*Set MAC speed and duplex accordingly*/

	req.code= DPAA_RUNTIME_CODE_SET_MAC_CONFIG;
	req.set_mac_config.portal = p40xx->portal[0];
	req.set_mac_config.flags = 0;
	req.set_mac_config.owner = p40xx;
	req.set_mac_config.port_id =p40xx->idx;

	if( cfg->duplex==1)
	req.set_mac_config.duplex= MAC_DUPLEX_FULL;
	else
	req.set_mac_config.duplex= MAC_DUPLEX_HALF;

	switch(cfg->media_rate){
		case (10*1000):
			req.set_mac_config.speed=MAC_SPEED_10MB ;
		break;
		case (100*1000):
			req.set_mac_config.speed=MAC_SPEED_100MB ;
		break;
		case (1000*1000):
			req.set_mac_config.speed=MAC_SPEED_1GB ;
		break;
		case (10*1000*1000):
			req.set_mac_config.speed= MAC_SPEED_10GB;
		break;
		default:
			req.set_mac_config.speed= MAC_SPEED_100MB;
		break;
	}

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime SET_MAC_CONFIG  failed for id%d: return code %d",
						__FUNCTION__,	p40xx->idx,rc);
		log(LOG_ERR, "Error Cause %d\n",req.set_mac_config_resp.result);
	}

}


static int fmt_4x1g_lanes(int enables, char *buf /* char buf[8] */){
	int i, nlane;
	char *p;
	int	startrange, nrange;

	/*
	 * Calculate number of enabled lanes and a string
	 * listing the enabled lanes for logging.
	 */
	nlane=0;
	startrange = -1;
	nrange = 0;
	p = buf;
	for(i=0; i<4; i++) {
		if(enables & (1<<i)) {
			nlane++;
			if(buf) {
				if(startrange+nrange == i)
					nrange++;
				else {
					if(nrange > 1) {
						*p++ = '-';
						*p++ = '0' + startrange + nrange - 1;
					}
					if(p != buf) *p++ = ',';
					*p++ = '0' + i;
					startrange = i;
					nrange = 1;
				}
			}
		}
	}
	if(buf) {
		if(nrange > 1) {
			*p++ = '-';
			*p++ = '0' + startrange + nrange - 1;
		}
		*p++ = '\0';
	}

	return nlane;
}

static int calc_4x1g_lanes(int enable12, int enable34, int lpd, int *enables, char *buf /* char buf[8] */){
	int	lpd_4x1g = 0;

	/*
	 * Calculate effective lane power downs
	 */
	if (enable12)
		lpd_4x1g |= lpd & 0x3;
	else
		lpd_4x1g |= 0x3;
	if (enable34)
		lpd_4x1g |= lpd & 0xc;
	else
		lpd_4x1g |= 0xc;

	/*
	 * Clear enables for powered down lanes
	 */
	*enables &= ~lpd_4x1g;

	return fmt_4x1g_lanes(~lpd_4x1g & 0xf, buf);
}

/*****************************************************************************/
/*  Function Name	: p40xx_defconfig
 *	Input			: Pointer to DPAA startup parameters
 *	Description		: Determines the autoconfiguration using the RCW settings 
 *	Returns			: None  
 *	Note			:
 */
/*****************************************************************************/
static int p40xx_defconfig(dpaa_startup_params_t *dpaa_cfg, int autocfg){
#define MEMORY_SIZE 0xB000

	uint8_t *memory=NULL;
	uint32_t serdes_prot=0, serdes_lpd_b1=0, serdes_lpd_b2=0, serdes_lpd_b3=0;
	int nlane, mac_enables;
	char buf[8];

typedef enum e_SrdsPrtcl {
    PCIE1,
    PCIE2,
    PCIE3,
    SRIO1,
    SRIO2,
    SGMII_FM1,
    SGMII_FM2,
    XAUI_FM1,
    XAUI_FM2,
    DEBUG
} e_SrdsPrtcl;

enum e_SrdsPrtcl serdesBank1[][5] = {
    [0] = {PCIE1, PCIE1, PCIE1, PCIE1, DEBUG},
    [1] = {PCIE1, PCIE1, PCIE1, PCIE1, DEBUG},
    [2] = {PCIE1, PCIE1, PCIE1, PCIE1, DEBUG},
    [3] = {PCIE1, PCIE1, PCIE2, PCIE2, DEBUG},
    [4] = {PCIE1, PCIE1, PCIE2, PCIE2, DEBUG},
    [5] = {PCIE1, PCIE1, PCIE2, PCIE2, DEBUG},
    [6] = {PCIE1, PCIE3, PCIE2, PCIE2, DEBUG},
    [7] = {PCIE1, PCIE3, PCIE2, PCIE2, DEBUG},
    [8] = {PCIE1, PCIE3, PCIE2, PCIE2, DEBUG},
    [9] = {PCIE1, PCIE1, PCIE2, PCIE2, DEBUG},
    [10] = {PCIE1, PCIE1, PCIE2, PCIE2, DEBUG},
    [11] = {PCIE1, PCIE1, PCIE2, SGMII_FM2, DEBUG},
    [12] = {PCIE1, PCIE1, PCIE2, SGMII_FM2, DEBUG},
    [13] = {PCIE1, PCIE1, PCIE2, SGMII_FM2, DEBUG},
    [14] = {PCIE1, PCIE3, PCIE2, SGMII_FM2, DEBUG},
    [15] = {PCIE1, PCIE1, SGMII_FM2, SGMII_FM2, DEBUG},
    [16] = {PCIE1, PCIE3, SGMII_FM2, SGMII_FM2, DEBUG},
    [17] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [18] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [19] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [20] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [21] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [22] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [23] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [24] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [25] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [26] = {SRIO2, SRIO2, SRIO1, SRIO1, DEBUG},
    [27] = {PCIE1, PCIE3, SRIO2, SRIO1, DEBUG},
    [28] = {PCIE1, PCIE3, SRIO2, SRIO1, DEBUG},
    [29] = {PCIE1, PCIE3, SRIO2, SRIO1, DEBUG},
    [30] = {PCIE1, PCIE1, SRIO1, SRIO1, DEBUG},
    [31] = {PCIE1, PCIE1, SRIO1, SRIO1, DEBUG},
    [32] = {PCIE1, PCIE1, SRIO1, SRIO1, DEBUG},
    [33] = {PCIE1, PCIE1, SRIO1, SRIO1, DEBUG},
    [34] = {PCIE1, PCIE1, SRIO1, SRIO1, DEBUG},
    [35] = {PCIE1, PCIE3, SRIO1, SRIO1, DEBUG},
    [36] = {PCIE1, PCIE3, SRIO1, SRIO1, DEBUG},
    [37] = {PCIE1, PCIE3, SRIO1, SRIO1, DEBUG}
};

static enum e_SrdsPrtcl serdesBank2[] = {
    [0] = SGMII_FM2,
    [1] = XAUI_FM2,
    [2] = XAUI_FM2,
    [3] = SGMII_FM2,
    [4] = XAUI_FM2,
    [5] = XAUI_FM2,
    [6] = SGMII_FM2,
    [7] = XAUI_FM2,
    [8] = XAUI_FM2,
    [9] = PCIE3,
    [10] = PCIE3,
    [11] = PCIE3,
    [12] = PCIE3,
    [13] = XAUI_FM2,
    [14] = XAUI_FM2,
    [15] = XAUI_FM2,
    [16] = XAUI_FM2,
    [17] = SGMII_FM2,
    [18] = XAUI_FM2,
    [19] = XAUI_FM2,
    [20] = PCIE3,
    [21] = PCIE3,
    [22] = SGMII_FM2,
    [23] = XAUI_FM2,
    [24] = XAUI_FM2,
    [25] = PCIE3,
    [26] = PCIE3,
    [27] = SGMII_FM2,
    [28] = XAUI_FM2,
    [29] = XAUI_FM2,
    [30] = PCIE3,
    [31] = PCIE3,
    [32] = SGMII_FM2,
    [33] = XAUI_FM2,
    [34] = XAUI_FM2,
    [35] = SGMII_FM2,
    [36] = XAUI_FM2,
    [37] = XAUI_FM2
};

static enum e_SrdsPrtcl serdesBank3[] = {
    [0] = SGMII_FM1,
    [1] = SGMII_FM1,
    [2] = XAUI_FM1,
    [3] = SGMII_FM1,
    [4] = SGMII_FM1,
    [5] = XAUI_FM1,
    [6] = SGMII_FM1,
    [7] = SGMII_FM1,
    [8] = XAUI_FM1,
    [9] = SGMII_FM1,
    [10] = XAUI_FM1,
    [11] = SGMII_FM1,
    [12] = XAUI_FM1,
    [13] = XAUI_FM1,
    [14] = XAUI_FM1,
    [15] = SGMII_FM1,
    [16] = SGMII_FM1,
    [17] = SGMII_FM1,
    [18] = SGMII_FM1,
    [19] = XAUI_FM1,
    [20] = SGMII_FM1,
    [21] = XAUI_FM1,
    [22] = SGMII_FM1,
    [23] = SGMII_FM1,
    [24] = XAUI_FM1,
    [25] = SGMII_FM1,
    [26] = XAUI_FM1,
    [27] = SGMII_FM1,
    [28] = SGMII_FM1,
    [29] = XAUI_FM1,
    [30] = SGMII_FM1,
    [31] = XAUI_FM1,
    [32] = SGMII_FM1,
    [33] = SGMII_FM1,
    [34] = XAUI_FM1,
    [35] = SGMII_FM1,
    [36] = SGMII_FM1,
    [37] = XAUI_FM1
};

    if ((memory = mmap_device_memory (NULL, MEMORY_SIZE, PROT_READ | PROT_WRITE | PROT_NOCACHE,
    		MAP_SHARED,dpaa_cfg->immr+ 0x000E0000)) == MAP_FAILED) {
    	log(LOG_ERR, "%s(): mmap GUTIL regs failed:", __FUNCTION__);
    	return ENOMEM;
    }	
	serdes_prot = (*(uint32_t*)(memory+ 0x110)) >> 26;
	serdes_lpd_b1 = (((*(uint32_t*)(memory+ 0x110))&0xff)<<2) | (((*(uint32_t*)(memory+ 0x114))>>30)&0x3);	/* Bank1 power down (10 lanes) */
	serdes_lpd_b2 = (((*(uint32_t*)(memory+ 0x114))>>26)&0xf);  /* Bank2 power down (4 lanes) */
	serdes_lpd_b3 = (((*(uint32_t*)(memory+ 0x114))>>18)&0xf);  /* Bank3 power down (4 lanes) */
   	log(LOG_INFO, "%s(): RCW: Serdes protocol 0x%x (pwrdn b1=%x, b2=%x, b3=%x) RCW4 0x%x RCW5 0x%x", __FUNCTION__,
		serdes_prot, serdes_lpd_b1, serdes_lpd_b2, serdes_lpd_b3,
		(*(uint32_t*)(memory+ 0x110)), (*(uint32_t*)(memory+ 0x114))
	   );

	if(autocfg) {
		dpaa_cfg->fm0_1x4gig = 0xf;
		dpaa_cfg->fm0_10gig  = 1;
		dpaa_cfg->fm1_1x4gig = 0xf;
		dpaa_cfg->fm1_10gig  = 1;
	}

	mac_enables = dpaa_cfg->fm0_1x4gig;
	if(serdesBank3[serdes_prot]==SGMII_FM1){
		/* calculate bank3 enabled FM1 SGMII lanes */
		nlane = calc_4x1g_lanes(TRUE, TRUE, serdes_lpd_b3&0xf, &dpaa_cfg->fm0_1x4gig, buf);
		if(nlane== 4)
			log(LOG_INFO, "%s():FM1 has 4x1Gig at SerDes3", __FUNCTION__);
		else if(nlane > 0)
			log(LOG_INFO, "%s():FM1 has %dx1Gig at SerDes3 (%s)", __FUNCTION__, nlane, buf);
	} else
		dpaa_cfg->fm0_1x4gig = 0;
	if(!autocfg && mac_enables!=0 && dpaa_cfg->fm0_1x4gig==0)
		log(LOG_WARNING, "%s(): WARNING: Requested FM1 4x1Gig but no requested SerDes lanes enabled", __FUNCTION__);

	if(serdesBank3[serdes_prot]==XAUI_FM1 && serdes_lpd_b3 == 0){
   		log(LOG_INFO, "%s():FM1 has 10Gig at SerDes3", __FUNCTION__);
	} else {
		if(!autocfg && dpaa_cfg->fm0_10gig) {
			if(serdesBank3[serdes_prot]!=XAUI_FM1)
				log(LOG_WARNING, "%s(): WARNING: Requested FM1 10Gig is not available in this config", __FUNCTION__);
			else
				log(LOG_WARNING, "%s(): WARNING: Requested FM1 10Gig but < 4 SerDes lanes enabled", __FUNCTION__);
		}
		dpaa_cfg->fm0_10gig = 0;
	}

	mac_enables = dpaa_cfg->fm1_1x4gig;
	if(serdesBank2[serdes_prot]==SGMII_FM2){
		/* calculate bank2 enabled FM2 SGMII lanes */
		nlane = calc_4x1g_lanes(TRUE, TRUE, serdes_lpd_b2&0xf, &dpaa_cfg->fm1_1x4gig, buf);
		if(nlane== 4)
			log(LOG_INFO, "%s():FM2 has 4x1Gig at SerDes2", __FUNCTION__);
		else if(nlane > 0)
			log(LOG_INFO, "%s():FM2 has %dx1Gig at SerDes2 (%s)", __FUNCTION__, nlane, buf);
	} else {
		/* when no bank2 FM2 SGMII then calculate bank1 enabled FM2 SGMII lanes */
		nlane = calc_4x1g_lanes(serdesBank1[serdes_prot][2]==SGMII_FM2, serdesBank1[serdes_prot][3]==SGMII_FM2, (serdes_lpd_b1>>4)&0xf, &dpaa_cfg->fm1_1x4gig, buf);
		if(nlane== 4)
			log(LOG_INFO, "%s():FM2 has 4x1Gig at SerDes1", __FUNCTION__);
		else if(nlane > 0)
			log(LOG_INFO, "%s():FM2 has %dx1Gig at SerDes1 (%s)", __FUNCTION__, nlane, buf);
	}
	if(!autocfg && mac_enables!=0 && dpaa_cfg->fm1_1x4gig==0)
		log(LOG_WARNING, "%s(): WARNING: Requested FM2 4x1Gig but no requested SerDes lanes enabled", __FUNCTION__);

	if(serdesBank2[serdes_prot]==XAUI_FM2 && serdes_lpd_b2 == 0){
   		log(LOG_INFO, "%s():FM2 has 10Gig at SerDes2", __FUNCTION__);
	} else {
		if(!autocfg && dpaa_cfg->fm1_10gig) {
			if(serdesBank2[serdes_prot]!=XAUI_FM2)
				log(LOG_WARNING, "%s(): WARNING: Requested FM2 10Gig is not available in this config", __FUNCTION__);
			else
				log(LOG_WARNING, "%s(): WARNING: Requested FM2 10Gig but < 4 SerDes lanes enabled", __FUNCTION__);
		}
		dpaa_cfg->fm1_10gig = 0;
	}


#ifdef SERDES8_ERRATA
	switch(get_serdes8_disable_bank(serdes_prot)){
			case SERDES_BANK_3:
				if(dpaa_cfg->fm0_1x4gig) log(LOG_INFO, "%s():FM1 4x1Gig at Serdes3 disabled for SERDES8 Errata workaround", __FUNCTION__);
				if(dpaa_cfg->fm0_10gig)  log(LOG_INFO, "%s():FM1 10Gig at Serdes3 disabled for SERDES8 Errata workaround", __FUNCTION__);
				dpaa_cfg->fm0_1x4gig=0;
				dpaa_cfg->fm0_10gig=0;
			break;
			case SERDES_BANK_2:
				dpaa_cfg->fm1_10gig=0;
			break;
			case SERDES_BANK_1:
				dpaa_cfg->fm1_1x4gig=0;
				dpaa_cfg->fm1_10gig=0;
			break;
	}

#endif
	munmap_device_memory(memory,MEMORY_SIZE);
	return EOK;
}
#ifdef SERDES8_ERRATA
static void p40xx_serdes8_errata(dpaa_startup_params_t *dpaa_cfg){
#define MEMORY_SIZE 0xB000

	uint8_t *memory=NULL;
	uint32_t serdes_prot=0;
	uint32_t *word=NULL;

    if ((memory = mmap_device_memory (NULL, MEMORY_SIZE, PROT_READ | PROT_WRITE | PROT_NOCACHE,
    		MAP_SHARED,dpaa_cfg->immr+ 0x000E0000)) == MAP_FAILED) {
    	log(LOG_ERR, "%s(): mmap GUTIL regs failed:", __FUNCTION__);
    	return;
    }	
	serdes_prot = (*(uint32_t*)(memory+ 0x110)) >> 26;
   	log(LOG_INFO, "%s(): RCW4 Serdes protocol read as 0x%x", __FUNCTION__,serdes_prot);

	uint32_t bank= get_serdes8_disable_bank(serdes_prot);
	if(bank!=SERDES_BANK_INVALID){
		word= ((uint32_t*)(memory+ 0xA000 + (bank-1)*0x20 ));
#if 0
		*word|= 0x20;
		log(LOG_INFO,"Disabling Serdes Bank %d value 0x%x",bank,*word);
#endif
	}
	munmap_device_memory(memory,MEMORY_SIZE);
	return;



}

#endif
