/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "p40xx.h"

/*****************************************************************************/
/* This routine emulates the read from a PHY. The routine reads the          */
/* information from the phy_regs structure, setup in the emu_phy_init        */
/* routine.                                                                  */
/*****************************************************************************/

uint16_t emu_phy_read (p40xx_dev_t * p40xx, uint8_t phy_add, uint8_t reg_add)

{
	if (reg_add > MII_NUM_REGS)
		return (0xffff);
	//if (p40xx->cfg.verbose > 8)
	//	log(LOG_INFO, "%s - reg %d - data 0x%x", __FUNCTION__, reg_add, p40xx->phy_regs [reg_add]);
	return (p40xx->phy_regs [reg_add]);
}

/*****************************************************************************/
/* Setup the phy_regs structure with the information necessary to emulate    */
/* a gigabit PHY. This is required for an interface that is connected to a   */
/* switch.                                                                   */
/*****************************************************************************/

int		emu_phy_init (p40xx_dev_t *p40xx)

{
nic_config_t	*cfg = &p40xx->cfg;
uint16_t		bmsr, bmcr;
int				i;
log(LOG_ERR, "%s(): invoked for interface %d", __FUNCTION__,p40xx->idx);

	bmsr = BMSR_AN_ABILITY | BMSR_LINK_STATUS | BMSR_EXT_STATUS;
	bmcr = 0;
	p40xx->phy_regs = malloc (MII_NUM_REGS*sizeof(*p40xx->phy_regs), M_DEVBUF, M_NOWAIT);
	if (p40xx->phy_regs == NULL) {
		log(LOG_ERR, "%s(): malloc phy_regs failed", __FUNCTION__);
		return (-1);
		}
	memset ((char *) p40xx->phy_regs, 0, MII_NUM_REGS*sizeof(*p40xx->phy_regs));
	//cfg->phy_addr = cfg->device_index;
	if (p40xx->force_advertise == -1) {
		bmcr = BMCR_FULL_DUPLEX | BMCR_SPEED_1000;
		bmsr |= (BMSR_EXTENDED_CAP | BMSR_100bXFD | BMSR_100bX | BMSR_10FD | BMSR_10);
		p40xx->phy_regs [MDI_MSSR] = (MSSR_CONFIG_RESULT | MSSR_LOCAL_RX_STATUS | MSSR_REMOTE_RX_STATUS);
		p40xx->phy_regs [MDI_MSSR] |= MSSR_LP_1000bTFD;
		}
	else {
		if (cfg->duplex)
			bmcr |= BMCR_FULL_DUPLEX;
		p40xx->phy_regs [MDI_ANAR] = (p40xx->force_advertise & 0x1f) << 5;
		switch (cfg->media_rate) {
			case	10*1000:
				bmsr |= (cfg->duplex) ? BMSR_10FD : BMSR_10;
				break;
			case	100*1000:
				bmsr |= (cfg->duplex) ? BMSR_100bXFD : BMSR_100bX;
				bmcr |= BMCR_SPEED_100;
				break;
			case	1000*1000:
				bmsr |= BMSR_EXTENDED_CAP;
				bmcr |= BMCR_SPEED_1000;
				p40xx->phy_regs [MDI_MSSR] = (MSSR_CONFIG_RESULT | MSSR_LOCAL_RX_STATUS | MSSR_REMOTE_RX_STATUS);
				p40xx->phy_regs [MDI_MSSR] |= (cfg->duplex) ? MSSR_LP_1000bTFD : MSSR_LP_1000bT;
				break;
			default:
				bmsr |= BMSR_EXTENDED_CAP;
				bmcr |= BMCR_SPEED_1000;
				p40xx->phy_regs [MDI_MSSR] = (MSSR_CONFIG_RESULT | MSSR_LOCAL_RX_STATUS | MSSR_REMOTE_RX_STATUS);
				p40xx->phy_regs [MDI_MSSR] |= (cfg->duplex) ? MSSR_LP_1000bTFD : MSSR_LP_1000bT;
				break;
	
			}
		}
	bmsr |= (BMSR_LINK_STATUS | BMSR_AN_COMPLETE);
	p40xx->phy_regs [MDI_BMSR] = bmsr;
	p40xx->phy_regs [MDI_BMCR] = bmcr;
	p40xx->phy_regs [MDI_PHYID_1] = 0x0f;
	p40xx->phy_regs [MDI_PHYID_2] = 0xc6c2;
	p40xx->phy_regs [MDI_EMSR] = (EMSR_1000bTFD | EMSR_1000bT);
	p40xx->phy_regs [MDI_ANAR] = (ANAR_100bTFD | ANAR_100bT | ANAR_10bTFD | ANAR_10bT | 1);
	p40xx->phy_regs [MDI_MSCR] = (MSCR_PORT_TYPE | MSCR_ADV_1000bTFD | MSCR_ADV_1000bT);
	p40xx->phy_regs [MDI_ANLPAR] = (ANLPAR_NEXT_PAGE | ANLPAR_ACKNOWLEDGE);
	p40xx->phy_regs [MDI_ANLPAR] |= p40xx->phy_regs [MDI_ANAR];
	p40xx->phy_regs [MDI_ANAE] = (ANAE_LP_NP_ABLE | ANAE_NP_ABLE | ANAE_LP_AN_ABLE);
		for (i = 0; i < MII_NUM_REGS; i++)
			log(LOG_INFO,"Phy REG %d - 0x%04x", i, p40xx->phy_regs [i]);
	return (0);
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		emu_phy_dinit (p40xx_dev_t *p40xx)

{
	if(!p40xx->phy_regs)
	free (p40xx->phy_regs, M_DEVBUF);
	return (0);
}
