/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "p40xx.h"

//static int p40xx_clear_interrupt(void *arg, struct nw_work_thread *wtp) ;
static int p40xx_fman_interrupt(void *arg, int fm_index, struct nw_work_thread *wtp); 

const struct sigevent * p40xx_qman_isr(void *arg, int iid) {

	p40xx_token_t		*token=arg;
	p40xx_dev_t		*p40xx=token->p40xx;
	struct _iopkt_inter	*ient;
	int i= token->tid;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_qman_portal[i], "QM_ISR_START" );
#endif
	ient = &p40xx->inter_qman[i];
	InterruptMask(p40xx->iid_qman_portal[i],p40xx->isr_qman_funcid[i]);
	return interrupt_queue(p40xx->iopkt, ient);
}


const struct sigevent * p40xx_bman_isr(void *arg, int iid) {

	p40xx_token_t		*token=arg;
	p40xx_dev_t		*p40xx=token->p40xx;
	struct _iopkt_inter	*ient;
	p40xx	= (p40xx_dev_t*)arg;
	volatile int i=token->tid;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_bman_portal[i], "BM_ISR_START" );
#endif
	ient = &p40xx->inter_bman[i];
	InterruptMask(p40xx->iid_bman_portal[i],p40xx->isr_bman_funcid[i]);

	return interrupt_queue(p40xx->iopkt, ient);
}
const struct sigevent * p40xx_fman1_isr(void *arg, int iid) {

	p40xx_dev_t		*p40xx;
	struct _iopkt_inter	*ient;
	p40xx	= (p40xx_dev_t*)arg;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_fman1, "FM1_ISR_START" );
#endif
	ient = &p40xx->inter_fman1;
	InterruptMask(p40xx->iid_fman1,p40xx->isr_fman1_funcid);
	return interrupt_queue(p40xx->iopkt, ient);
}
const struct sigevent * p40xx_fman2_isr(void *arg, int iid) {

	p40xx_dev_t		*p40xx;
	struct _iopkt_inter	*ient;
	p40xx	= (p40xx_dev_t*)arg;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_fman2, "FM2_ISR_START" );
#endif
	ient = &p40xx->inter_fman2;
	InterruptMask(p40xx->iid_fman2,p40xx->isr_fman2_funcid);
	return interrupt_queue(p40xx->iopkt, ient);
}
const struct sigevent * p40xx_common_err_isr(void *arg, int iid) {

	p40xx_dev_t		*p40xx;
	struct _iopkt_inter	*ient;
	p40xx	= (p40xx_dev_t*)arg;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_err, "CERR_ISR_START" );
#endif
	ient = &p40xx->inter_err;
	InterruptMask(p40xx->iid_err,p40xx->isr_err_funcid);
	return interrupt_queue(p40xx->iopkt, ient);
}

int p40xx_enable_qman_intr(void *arg) {
	p40xx_token_t *token=arg;
	p40xx_dev_t *p40xx = token->p40xx;
	int i = token->tid;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_qman_portal[i], "QM_ISR_FINISH" );
#endif
	InterruptUnmask(p40xx->iid_qman_portal[i],p40xx->isr_qman_funcid[i]);
	return 0;
}
int p40xx_enable_bman_intr(void *arg) {
	p40xx_token_t *token=arg;
	p40xx_dev_t *p40xx = token->p40xx;
	int i = token->tid;

#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_bman_portal[i], "BM_ISR_FINISH" );
#endif
	InterruptUnmask(p40xx->iid_bman_portal[i],p40xx->isr_bman_funcid[i]);
	return 0;
}

int p40xx_enable_fman1_intr(void *arg) {
	p40xx_dev_t *p40xx = arg;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_fman1, "FM1_ISR_FINISH" );
#endif
	InterruptUnmask(p40xx->iid_fman1,p40xx->isr_fman1_funcid);
	return 0;
}
int p40xx_enable_fman2_intr(void *arg) {
	p40xx_dev_t *p40xx = arg;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_fman2, "FM2_ISR_FINISH" );
#endif
	InterruptUnmask(p40xx->iid_fman2,p40xx->isr_fman2_funcid);
	return 0;
}
int p40xx_enable_common_err_intr(void *arg) {
	p40xx_dev_t *p40xx = arg;
#if 	defined(ENABLE_INSTR) && defined(ENABLE_INSTR_ISR_MASKING)
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,p40xx->iid_err, "CERR_ISR_FINISH" );
#endif
	InterruptUnmask(p40xx->iid_err,p40xx->isr_err_funcid);
	return 0;
}


int p40xx_process_bman_intr(void *arg, struct nw_work_thread *nwt){
	p40xx_dev_t *p40xx = arg;

	dpaa_runtime_t req;
	int32_t rc;
	uint32_t idx=p40xx->idx;
	dpaa_handle_t dpaa=p40xx->dpaa;
	
	/*Tell DPAA to mask the remaining interrupt causes */
	req.code=DPAA_RUNTIME_CODE_CLEAR_BMAN_IRQ_CAUSE;
	req.clear_irq_cause.portal= p40xx->portal[0]; 
	req.clear_irq_cause.owner= p40xx; 
	req.clear_irq_cause.id= p40xx->portal[0]; /*ID is same as Portal*/

	if((rc=dpaa_system_runtime(dpaa, &req))!=0){
		log(LOG_ERR, "%s():dpaa_system_runtime failed for CODE_CLEAR_BMAN_IRQ_CAUSE id%d: return code %d",
						__FUNCTION__,idx,rc);
		log(LOG_ERR, "Error Cause %d",req.clear_irq_cause_resp.result);
	}

	/*Return success in all case*/
	return 1;

}

int p40xx_process_qman_intr(void *arg, struct nw_work_thread *nwt){
	p40xx_dev_t *p40xx = arg;

	dpaa_runtime_t req;
	int32_t rc;
	uint32_t idx=p40xx->idx;
	dpaa_handle_t dpaa=p40xx->dpaa;
	
	/*Tell DPAA to mask the remaining interrupt causes */
	req.code=DPAA_RUNTIME_CODE_CLEAR_QMAN_IRQ_CAUSE;
	req.clear_irq_cause.portal= p40xx->portal[0]; 
	req.clear_irq_cause.owner= p40xx; 
	req.clear_irq_cause.id= p40xx->portal[0]; /*ID is same as Portal*/

	if((rc=dpaa_system_runtime(dpaa, &req))!=0){
		log(LOG_ERR, "%s():dpaa_system_runtime failed for CODE_CLEAR_QMAN_IRQ_CAUSE id%d: return code %d",
						__FUNCTION__,idx,rc);
		log(LOG_ERR, "Error Cause %d",req.clear_irq_cause_resp.result);
	}
	/*Return success in all case*/
	return 1;

}



int p40xx_process_fman1_intr(void *arg, struct nw_work_thread *nwt){
	p40xx_dev_t		*p40xx	= arg;

	p40xx_fman_interrupt(p40xx,0,nwt);
	return 1;
}



int p40xx_process_fman2_intr(void *arg, struct nw_work_thread *nwt){
	p40xx_dev_t		*p40xx	= arg;

	p40xx_fman_interrupt(p40xx,1,nwt);
	return 1;
}


int p40xx_process_common_err_intr(void *arg, struct nw_work_thread *nwt){
	p40xx_dev_t		*p40xx	= arg;
	uint32_t eisr=  *p40xx->eisr_reg;
	static unsigned int eisr_cnt, unexp_eisr_cnt;

	eisr_cnt++;
	if(eisr & ~(EISR0_INT_SOURCE_FM1|EISR0_INT_SOURCE_FM2|EISR0_INT_SOURCE_BMAN|EISR0_INT_SOURCE_QMAN) && unexp_eisr_cnt++ <= 5) {
	    log(LOG_ERR, "%s: Unexpected Common Error interrupt 0x%x (from eisr 0x%x) eisr_cnt %d", __FUNCTION__,
			eisr & ~(EISR0_INT_SOURCE_FM1|EISR0_INT_SOURCE_FM2|EISR0_INT_SOURCE_BMAN|EISR0_INT_SOURCE_QMAN),
			eisr, eisr_cnt);
	    if(unexp_eisr_cnt > 5) {
		log(LOG_ERR, "%s: Disabling further unexpected Common Error interrupts from sources 0x%x", __FUNCTION__,
			eisr & ~(EISR0_INT_SOURCE_FM1|EISR0_INT_SOURCE_FM2|EISR0_INT_SOURCE_BMAN|EISR0_INT_SOURCE_QMAN));
		p40xx->eisr_reg[0x10/sizeof(uint32_t)] |= eisr & ~(EISR0_INT_SOURCE_FM1|EISR0_INT_SOURCE_FM2|EISR0_INT_SOURCE_BMAN|EISR0_INT_SOURCE_QMAN);
		unexp_eisr_cnt = 0;
	    }
	} else if(eisr_cnt >= 1500000 && eisr_cnt < 1500000+5) {
	    log(LOG_ERR, "%s: High err intr count %d, error interrupt 0x%x", __FUNCTION__, eisr_cnt, eisr);
	}

	/*Check which of the common error interrupt sources triggered the interrupt*/
	if(eisr & EISR0_INT_SOURCE_FM1 ){
		log(LOG_ERR, "Processing EISR0_INT_SOURCE_FM1 ");
		p40xx_fman_interrupt(p40xx,0,nwt);
	}
	if(eisr & EISR0_INT_SOURCE_FM2 ){
		log(LOG_ERR, "Processing  EISR0_INT_SOURCE_FM2");
		p40xx_fman_interrupt(p40xx,1,nwt);
	}
	if(eisr & EISR0_INT_SOURCE_BMAN ){
		log(LOG_ERR, "Processing EISR0_INT_SOURCE_BMAN ");
		p40xx_process_bman_intr(p40xx,nwt);
	}
	if(eisr & EISR0_INT_SOURCE_QMAN ){
		log(LOG_ERR, "Processing EISR0_INT_SOURCE_QMAN ");
		p40xx_process_qman_intr(p40xx,nwt);
	}

	return 1;
}
 
/*****************************************************************************/
/*  Function Name	: p40xx_process_interrupt
 *	Input			: p40xx device pointer
 *					  Pointer to WTP thread who invokes this function
 *	Description		: Called by io-pkt Interrupt service thread, once it has been 
 *					  triggered by our very fast interrupt handler.We should receive  
 *					  all packets from all FQs associated here here and pass it upto the stack.
 *					  Remember that interrupts are masked at the kernel level, so
 *					  We should do all we can to make sure interrupt conditions are 
 *					  satisified once the call to this function exits. 
 *	Returns			: 1 always. 
 *	Note			: This function doesnt run in ISR context so there is no restriction
 *					  on its capability, except that it should be fast enough.
 */
/*****************************************************************************/
int p40xx_process_interrupt(void *arg, struct nw_work_thread *wtp) {
	/*Receive call should clear the receive interrupt cause */
	uint32_t loop=2;
	uint32_t tloop=0;
	p40xx_token_t *token= arg;
	p40xx_dev_t *p40xx= token->p40xx;
	uint64_t snap1=0;
	uint64_t delta=0;

	while(loop--){
		snap1= 	p40xx->rx_counter;
		p40xx_receive(arg);
		delta= p40xx->rx_counter-snap1;
		
		if(delta > 8){
			//Many packets are being received. Lets be greedy and loop more
			loop+=2;
		}
		else if(delta>0){
			// Getting more than a packet. Give some grace marks
			loop+=1;
		}
		if(loop > 5) loop = 5; //Never let loopcount grow too much
		if(++tloop > 64) break;
		/*For receiving  0 packet, the loop doesnt increment */
	}
    //if(tloop >= 64) log(LOG_WARNING, "%s: receive loop hit max %d interations", __FUNCTION__, tloop);
	return 1;
}

static int p40xx_fman_interrupt(void *arg, int fm_index, struct nw_work_thread *wtp) {
	p40xx_dev_t		*p40xx	= arg;
	dpaa_runtime_t req;
	int32_t rc=EOK;
	uint32_t idx=p40xx->idx;
	dpaa_handle_t dpaa=p40xx->dpaa;
	
	req.code=DPAA_RUNTIME_CODE_CLEAR_FMAN_IRQ_CAUSE;
	req.clear_irq_cause.portal= p40xx->portal[0]; 
	req.clear_irq_cause.owner= p40xx; 
	req.clear_irq_cause.id= fm_index;  /* Which FMan caused the interrupt */

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime failed for CODE_CLEAR_FMAN_IRQ_CAUSE id%d: return code %d",
						__FUNCTION__,idx,rc);
		log(LOG_ERR, "Error Cause %d",req.clear_irq_cause_resp.result);
	}

	/*Return success in all case*/
	return EOK;

}

