/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#ifndef NIC_P40XX_H_INCLUDED
#define NIC_P40XX_H_INCLUDED

#include <io-pkt/iopkt_driver.h>
#include "ppc/e500cpu.h"
#include "ppc/inline.h"
#include <ppc/cpu.h>
#include <ppc/85xxintr.h>
#include <ppc/83xxintr.h>
#include <hw/sysinfo.h>
#include <pthread.h>
#include <atomic.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/syslog.h>
#include <sys/slogcodes.h>
#include <sys/cache.h>
#include <sys/param.h>
#include <sys/syspage.h>
#include <sys/malloc.h>
#include <sys/mman.h>
#include <sys/io-pkt.h>
#include <sys/callout.h>
#include <sys/mbuf.h>
#include <sys/ioctl.h>

#include <drvr/mdi.h>
#include <drvr/eth.h>
#include <drvr/nicsupport.h>
#include <drvr/common.h>

#include <hw/nicinfo.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_ether.h>
#include <net/if_types.h>

#include <quiesce.h>
#include <siglock.h>

#include <sys/device.h>
#include <net/if_media.h>
#include <dev/mii/miivar.h>

#include <_pack1.h>
#include <_packpop.h>
#include "dpaa_exp.h"
#ifdef ENABLE_INSTR
#include <sys/trace.h>
#endif
#define	P40XX_MAX_BLOCK_PER_MEMPOOL	10

#define	P40XX_TX_BPID_BASE	41
#define	P40XX_RX_BPID_BASE	1

#define	P40XX_INTERRUPT_MASK_QMAN_PORTAL	(1 <<0)
#define	P40XX_INTERRUPT_MASK_BMAN_PORTAL	(1 <<1)
#define	P40XX_INTERRUPT_MASK_FMAN_1			(1 <<2)
#define	P40XX_INTERRUPT_MASK_FMAN_2			(1 <<3)
#define	P40XX_INTERRUPT_MASK_COMMON_ERR		(1 <<4)

#define	MII_NUM_REGS	29
#define MPC_TIMEOUT     1000

#define	MII_NUM_REGS	29
#define PHY_ADDR_BASE  0xFE4E1120
#define PHY_ADDR_SIZE  0x1000
#define PHY_ADDRESS_OFFSET 0x1c


#define DPAA_MAX_FRAGS 17
#define DPAA_QMAN_CCSR_BASE_OFFSET 0x318000
#define DPAA_QMAN_CCSR_BLOCK_SIZE  0x001000
#define DPAA_BMAN_CCSR_BASE_OFFSET 0x31A000
#define DPAA_BMAN_CCSR_BLOCK_SIZE 0x1000

#define DPAA_FMAN_CCSR_BASE_OFFSET 0x400000
#define DPAA_FMAN_CCSR_BLOCK_SIZE  0x100000

#define BPID_HW_RELEASE 0x8000
#define DPAA_ERR_EBUSY 0xD0020 /* MODULE_QM | E_BUSY */

/*ESIR 0 Interrupt sources*/
#define CCSR_EISR0_OFFSET 0x43900
#define EISR0_INT_SOURCE_FM2  0x80000000
#define EISR0_INT_SOURCE_FM1  0x40000000
#define EISR0_INT_SOURCE_BMAN 0x20000000
#define EISR0_INT_SOURCE_QMAN 0x10000000

#define MAX_SGT_ENTRIES 2
#define FM_MAX_NUM_OF_SG_TABLE_ENTRY 16
#define FM_NUM_OF_SG_TABLE_ENTRY ((MLEN/sizeof(sgt_entry_t)) < FM_MAX_NUM_OF_SG_TABLE_ENTRY ? (MLEN/sizeof(sgt_entry_t)) : FM_MAX_NUM_OF_SG_TABLE_ENTRY)


/*Cache lookup routines*/
typedef void* cache_handle_t;
typedef struct {
	void    *p40xx;
	int	    tid;
}p40xx_token_t ;

/**************************************************************************//**
 @Description   Frame Scatter/Gather Table Entry
*//***************************************************************************/
typedef struct {
    volatile uint32_t    addrh;        /**< Buffer Address high */
    volatile uint32_t    addrl;        /**< Buffer Address low */
    volatile uint32_t    length;       /**< Buffer length */
    volatile uint32_t    offset;       /**< SGTE offset */
} __packed sgt_entry_t;


/**************************************************************************//**
 @Description   Frame Scatter/Gather Table
*//***************************************************************************/
/*A SGT table can have upto 16 entries. */
typedef struct {
  sgt_entry_t  entry[FM_NUM_OF_SG_TABLE_ENTRY];
} __packed dpaa_sgt_t;

#define SGT_LIST_ELEMENTS 0x100
typedef struct {
  struct mbuf* m_ptr;
  void *next;
  void *prev;
} __packed p40xx_mlist_element;

#define DPAA_SIGNATURE 0xDEADBEEF



/*This is our custom header which we prefix on each incoming frame with an offset.
 * */
typedef struct {
  uint32_t ref_count;
  uint32_t signature;
  uint32_t entry_cnt;
  uint64_t info[MAX_SGT_ENTRIES];
} __packed dpaa_frame_prefix_t;

#define MATCH_SIGNATURE(x)   (((dpaa_frame_prefix_t*)(x))->signature==DPAA_SIGNATURE)
#define ERASE_SIGNATURE(x)   (((dpaa_frame_prefix_t*)(x))->signature=0)

/*Input to this MACRO should be actual VADDR which is seeded into BMAN*/
#define DPAA_GET_PREFIX(x)   (   (dpaa_frame_prefix_t*)( ((uint8_t*)(x) -DPAA_BMAN_PRIVATE_OFFSET) )  )

#define BUF_INFO_GET_BPID(token) ((uint64_t)((token)>>48) & 0xFFFF) 
#define BUF_INFO_GET_VADDR(token) ((uint64_t)(token) & 0xFFFFFFFFFFFFULL) 

#define BUF_INFO_SET_BPID(b) (((b) & 0xFFFF)<< 48) 

#define BUF_INFO_SET_PADDR(p) ((p)& 0xFFFFFFFFFFFFULL)

#define BUF_INFO_ADD(ptr, b, p) \
	((dpaa_frame_prefix_t*)ptr)->info[((dpaa_frame_prefix_t*)ptr)->entry_cnt++]= \
		((uint64_t)((b)& 0xFFFF)<< 48) | ((uint64_t)(p)& 0xFFFFFFFFFFFFULL);


#define INIT_FRAME_PREFIX(ptr) \
	((dpaa_frame_prefix_t*)ptr)->signature= DPAA_SIGNATURE;\
	((dpaa_frame_prefix_t*)ptr)->entry_cnt=0;

#define NUM_OF_PORTAL 10
/********* SW portal specific Datastructures *****************/
/*This is main device structure per interface. */
typedef struct {
	/*First element must be struct device*/
	struct device				dev;
	/*Add custom fields here onwards*/
	struct ethercom         	ecom;
	struct _iopkt_inter     	inter_qman[NUM_OF_PORTAL];
	void                        *isrp_qman;
	uint32_t					iid_qman_portal[NUM_OF_PORTAL];
	uint32_t					isr_qman_funcid[NUM_OF_PORTAL];
	
	struct _iopkt_inter     	inter_bman[NUM_OF_PORTAL];
	void                        *isrp_bman;
	uint32_t					iid_bman_portal[NUM_OF_PORTAL];
	uint32_t					isr_bman_funcid[NUM_OF_PORTAL];

	/*Interrupt below would be subscribed by Master application only*/
	struct _iopkt_inter     	inter_fman1;
	void                        *isrp_fman1;
	uint32_t					iid_fman1;
	uint32_t					isr_fman1_funcid;
	
	struct _iopkt_inter     	inter_fman2;
	void                        *isrp_fman2;
	uint32_t					iid_fman2;
	uint32_t					isr_fman2_funcid;

	struct _iopkt_inter     	inter_err;
	void                        *isrp_err;
	uint32_t					iid_err;
	uint32_t					isr_err_funcid;

	nic_config_t				cfg;
	nic_stats_t     			stats;
	struct _iopkt_self			*iopkt;
	int32_t						idx;
	uint32_t 					rx_cap_mask;
	uint32_t 					tx_cap_mask;
	uint32_t 					dying;
	uint32_t 					verbose;
	uint32_t 					rxd_pkts;  

	// mii
	struct callout          	mii_callout;
	int							force_advertise;
	mdi_t						*mdi;
    uint32_t        			probe_phy;  
    uint32_t        			emu_phy; 
	uint32_t        			*phy_regs; /* Used only for emu_phy */
	uint32_t 					dump_regs; /*flag to enable dumping of hardware registers to Syslog */

	struct mii_data				bsd_mii;       	
	void                        *sdhook;

/**************************************************************************************/
/* DPAA specific Data  */
/**************************************************************************************/
    /*DPAA specific paramters which should be stored by an application*/	
	dpaa_handle_t 				dpaa;
	/*Common portal ID*/
	uint32_t 					portal[NUM_OF_PORTAL]; /*An interface can attach to more than one portal*/
	uint32_t 					num_of_portals; 
//qman
	uint32_t 					rx_chid[NUM_OF_PORTAL]; /*This could be an array of CHID later*/
	uint32_t 					rx_fqid[NUM_OF_PORTAL]; /*This could be an array of FQ later*/
	uint32_t 					rx_fq_num; /*This could be an array of FQ later*/
	uint32_t 					rx_errqid[NUM_OF_PORTAL]; /*This could be an array of FQ later*/
	uint32_t 					tx_fqid;
	void* 						tx_handle; /*This could be an array of FQ later*/
	uint32_t					tx_fq_num; 
	uint32_t 					tcfqid[NUM_OF_PORTAL];  /*Transmit Confirmation QID for each Tx channel*/
	uint32_t					rx_buffer_prefix_size; /* Size of private area in rx buffer */
	
    uint32_t 					bp_count;
    uint32_t 					bpid[DPAA_BMAN_MAX_BPOOL];
    uint32_t 					bp_size[DPAA_BMAN_MAX_BPOOL];
/**************************************************************************************/
/*  End of DPAA specific Data  */
/**************************************************************************************/
	
	uint32_t 					master;  /*If an interface owns DPAA and 1st FM*/
	uint32_t 					secondary;  /*If an interface owns 2nd FM*/
	cache_handle_t				cache_handle;
	uint64_t 					tx_counter;
	uint64_t 					rx_counter;
	int		 					rx_loop; /*Donot pass packets up and loopback immediately*/
	p40xx_token_t				token[NUM_OF_PORTAL];
	p40xx_mlist_element*        mlist_start;
	p40xx_mlist_element*        mlist_end;
	uint32_t*					eisr_reg;

}p40xx_dev_t ;

#define MBUF_LIST_ELEMENTS 64

#define MLIST_INIT(elem) \
	do {((p40xx_mlist_element*)(elem))->m_ptr =NULL;\
	((p40xx_mlist_element*)(elem))->next=NULL;\
	((p40xx_mlist_element*)(elem))->prev =NULL;\
	} while(0)

/*Add to the tail of cur*/
#define MLIST_ADD_ELEM(tail, elem, token) \
	do { \
	((p40xx_mlist_element*)(tail))->next =elem;\
	((p40xx_mlist_element*)(elem))->next=NULL;\
	((p40xx_mlist_element*)(elem))->prev =tail;\
	((p40xx_mlist_element*)(elem))->m_ptr =token;\
	tail =(p40xx_mlist_element*)(elem);\
	} while(0)

	/*Remove from the front*/ 
#define MLIST_REM_ELEM(start, elem) \
	do { \
	elem = start;\
	start= (p40xx_mlist_element*)((p40xx_mlist_element*)(elem))->next;\
	((p40xx_mlist_element*)(start))->prev= NULL;\
	((p40xx_mlist_element*)(elem))->prev=NULL;\
	((p40xx_mlist_element*)(elem))->next=NULL;\
	} while(0)


/*Argument passed to dev_attach() by detect function are packed inside this*/
typedef struct {
	void			*dll_hdl;
	struct _iopkt_self *iopkt;
	char			*options;
	int32_t			idx;
	int32_t			portal;
	uint32_t		emu_phy;
	uint32_t		phy_addr;
	uint32_t		master;
	uint32_t		secondary;
	dpaa_handle_t	dpaa;
	nic_config_t	*nic_cfg;
	int				dump_regs;
	int				rx_loop;
	uint64_t		immr;
	int				jumbo_1g;
	int				jumbo_10g;
}p40xx_arg_t ;


// event.c
const struct sigevent * p40xx_qman_isr(void *, int);
const struct sigevent * p40xx_bman_isr(void *, int);
const struct sigevent * p40xx_fman1_isr(void *, int);
const struct sigevent * p40xx_fman2_isr(void *, int);
const struct sigevent * p40xx_common_err_isr(void *, int);

int p40xx_enable_qman_intr(void *arg);
int p40xx_enable_bman_intr(void *arg);
int p40xx_enable_fman1_intr(void *arg);
int p40xx_enable_fman2_intr(void *arg);
int p40xx_enable_common_err_intr(void *arg);

int p40xx_process_interrupt(void *arg, struct nw_work_thread *);
int p40xx_process_bman_intr(void *arg, struct nw_work_thread *);
int p40xx_process_qman_intr(void *arg, struct nw_work_thread *);
int p40xx_process_fman1_intr(void *arg, struct nw_work_thread *);
int p40xx_process_fman2_intr(void *arg, struct nw_work_thread *);
int p40xx_process_common_err_intr(void *arg, struct nw_work_thread *);

// multicast.c
void p40xx_set_multicast(p40xx_dev_t *);

// detect.c
uint64_t vtophys (void *addr) ;

int p40xx_detect(void *dll_hdl, struct _iopkt_self *iopkt, char *options);
void p40xx_speeduplex(p40xx_dev_t *);

/* devctl.c */
int p40xx_ioctl(struct ifnet *, unsigned long, caddr_t);
int p40xx_filter (p40xx_dev_t *p40xx, uint8_t oper, uint8_t *addrlo, uint8_t *addrhi);

// transmit.c
void p40xx_start(struct ifnet *);
int p40xx_transmit_complete (p40xx_dev_t *,uint64_t);
void p40xx_mbuf_list_init(p40xx_dev_t *);
void p40xx_mbuf_list_finish(p40xx_dev_t *);
void p40xx_mbuf_list_manage(p40xx_dev_t *, struct mbuf *);

// receive.c
void p40xx_receive(p40xx_token_t *);
void p40xx_mbuf_release(struct mbuf *m, caddr_t buf, size_t size, void *arg);

// mii.c
void p40xx_MDI_MonitorPhy(void *);
int p40xx_init_phy(p40xx_dev_t *);

// stats.c
void p40xx_update_stats(p40xx_dev_t *);
void p40xx_clear_stats(p40xx_dev_t *);

// bsd_media.c
void bsd_mii_initmedia(p40xx_dev_t *);

// emu_phy.c
uint16_t emu_phy_read (p40xx_dev_t *, uint8_t, uint8_t);
int emu_phy_init (p40xx_dev_t *);
int	emu_phy_dinit (p40xx_dev_t *);


/*Gives a handle to the database*/
cache_handle_t cache_map_init(void);  
/*Store entry into hash*/
int cache_map_store(uint64_t paddr, void *vaddr, cache_handle_t);
/*Delete entry into hash*/
int cache_map_delete(uint64_t paddr, void *vaddr, cache_handle_t);
/*Looks up given paddr*/
void* cache_paddr_lookup(uint64_t paddr, cache_handle_t);  
/*Looks up given vaddr*/
uint64_t cache_vaddr_lookup(void* vaddr, cache_handle_t);  
/*Destroy cache*/
int cache_map_destroy(cache_handle_t);


int p40xx_release_buf (dpaa_handle_t ,uint32_t bpid,uint32_t portal, void *);
int p40xx_acquire_buf (dpaa_handle_t ,uint32_t bpid,uint32_t portal, void*);
int p40xx_flush_bufs (dpaa_handle_t dpaa,uint32_t bpid,uint32_t portal, uint32_t nbuf);
void p40xx_rx(void* token, uint32_t,fd_t *fd_ptr );
void p40xx_tx_complete(void* token,uint32_t, fd_t *fd_ptr);
void p40xx_rx_err(void* token,uint32_t,fd_t *fd_ptr);
void p40xx_loop(void* token, uint32_t fqid,fd_t *fd_ptr);
extern void * XX_PhysToVirt(uint64_t addr);
extern uint64_t XX_VirtToPhys(void* addr);

void test_harness(p40xx_dev_t *);

#define LOG_LEVEL_MIN 0
#define LOG_SYS_FAILURE 1       /*Always ON             LOG_ERR*/
#define LOG_CRIT_ERROR 2		/*Always ON             LOG_ERR*/
#define LOG_RUNTIME_FAULT 3     /*Always ON             LOG_ERR*/
#define LOG_WARNING 4           /*Always ON             LOG_WARNING*/
#define LOG_NORMAL  5           /*Always ON             LOG_WARNING*/
#define LOG_DEBUG_CONFIG 6      /*Always ON             LOG_WARNING*/
#define LOG_DEBUG_FUNC_CALL 7   /*Always ON             LOG_ERR*/
#define LOG_DEBUG_DATA_FLOW 8
#define LOG_DEBUG_PKT_CONTENT 9
#define LOG_LEVEL_MAX 10

#ifdef SERDES8_ERRATA

typedef enum{
	SERDES_BANK_INVALID,
	SERDES_BANK_1,
	SERDES_BANK_2,
	SERDES_BANK_3,
	SERDES_BANK_MAX
}SERDES_BANK;

static inline uint32_t get_serdes8_disable_bank(uint32_t prctl){
	uint32_t retval=SERDES_BANK_INVALID;

	switch(prctl){ 
		case 0xf:
		case 0x10:
			log(LOG_INFO,"Bank 3 needs to be disabled for SERDES8 Errata workaround");
			retval=SERDES_BANK_3;
			break;
		default:
		break;
	}
	return retval;
}
#endif

static void inline 
dump_fd(const fd_t *fd){

	log(LOG_ERR,"============ BEGINNING FD DUMP ============");
	log(LOG_ERR,"FD DD=0x%x PID=0x%x BPID=0x%x ",fd->dd,fd->pid,fd->bpid );
	log(LOG_ERR,"FD ADDR HI=0x%4x LOW=0x%8x ", fd->addr_high,fd->addr_low);
	log(LOG_ERR,"FD FORMAT=0x%x OFFSET=0x%x LENGTH=%d ",fd->format,fd->offset, fd->length20 );
	log(LOG_ERR,"FD CMD=%x  STATUS=%x ",fd->cmd, fd->status );
	log(LOG_ERR,"============ END OF FD DUMP ============");
}

#define IS_MAC_10GIG(mac_id)  (mac_id==4 || mac_id==9)

#endif

