/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
#include "p40xx.h"
/*****************************************************************************/
/*  Function Name	: p40xx_update_stats
 *	Input			: Pointer to device structure
 *	Description		: gets stats for interface				
 *	Returns			: void 
 *	Note			: 
 */
/*****************************************************************************/
void    
p40xx_update_stats (p40xx_dev_t *p40xx)
{
	dpaa_runtime_t req;
    nic_ethernet_stats_t    *estats = &p40xx->stats.un.estats;
	dsec_mac_stats_t *regs=NULL;


	memset(&req,0, sizeof(dpaa_runtime_t));

	req.code= DPAA_RUNTIME_CODE_GET_MAC_STATS;

	req.get_mac_stats.portal= p40xx->portal[0];
	req.get_mac_stats.flags = 0  ;
	req.get_mac_stats.owner =  p40xx ;
	req.get_mac_stats.port_id = p40xx->idx ;
	
	if(dpaa_system_runtime(p40xx->dpaa, &req)!=EOK){
		/*Stats couldnt be retrieved*/	
		log(LOG_ERR, "%s(): Stats retrieval failed.",__FUNCTION__);
		return;
	}
	
	regs= &req.get_mac_stats_resp.stats;


    p40xx->stats.octets_rxed_ok = regs->rbyt;

    p40xx->stats.rxed_ok = regs->rpkt;
    estats->fcs_errors = regs->rfcs;
    p40xx->stats.rxed_multicast = regs->rmca;
    p40xx->stats.rxed_broadcast = regs->rbca;
    estats->align_errors = regs->raln;
    estats->length_field_mismatch = regs->rflr;
    estats->no_carrier = regs->rcse;
    estats->short_packets = regs->rund;
    estats->oversized_packets = regs->rovr;
    estats->internal_rx_errors = regs->rfrg;
    estats->jabber_detected = regs->rjbr;
    estats->internal_rx_errors = regs->rdrp;

    p40xx->stats.octets_txed_ok = regs->tbyt;
    p40xx->stats.txed_ok = regs->tpkt;
    p40xx->stats.txed_multicast = regs->tmca;
    p40xx->stats.txed_broadcast = regs->tbca;
    estats->excessive_deferrals = regs->tdfr;
    estats->single_collisions = regs->tscl;
    estats->multi_collisions = regs->tmcl;
    estats->late_collisions = regs->tlcl;
    estats->xcoll_aborted = regs->txcl;
    estats->total_collision_frames = regs->tncl;
    estats->jabber_detected = regs->tjbr;
	// refresh for devctl
	p40xx->stats.media = p40xx->cfg.media;

	/* Dump registers for debug */
	if(p40xx->dump_regs == 1) {
		memset(&req,0, sizeof(dpaa_runtime_t));
		req.code= DPAA_RUNTIME_CODE_DUMP_REGS;
		if(dpaa_system_runtime(p40xx->dpaa, &req)!=EOK){
			log(LOG_ERR, "%s(): dump regs failed.",__FUNCTION__);
			return;
		}
	}
}


/*****************************************************************************/
/*  Function Name	: p40xx_clear_stats
 *	Input			: Pointer to device structure
 *	Description		: clear stats for interface				
 *	Returns			: void 
 *	Note			: 
 */
/*****************************************************************************/
void    
p40xx_clear_stats (p40xx_dev_t *p40xx)
{
	// first, clear hardware register values by reading them
	p40xx_update_stats(p40xx);

	// now clear counters in our data structure
	memset(&p40xx->stats, 0, sizeof(p40xx->stats));
	
    // reset stats stuff for devctl
    p40xx->stats.revision = NIC_STATS_REVISION;

    p40xx->stats.valid_stats =
      NIC_STAT_TXED_MULTICAST |
      NIC_STAT_RXED_MULTICAST |
      NIC_STAT_TXED_BROADCAST |
      NIC_STAT_RXED_BROADCAST |
      NIC_STAT_TX_FAILED_ALLOCS |
      NIC_STAT_RX_FAILED_ALLOCS;

    p40xx->stats.un.estats.valid_stats =
      NIC_ETHER_STAT_FCS_ERRORS |
      NIC_ETHER_STAT_ALIGN_ERRORS |
      NIC_ETHER_STAT_LENGTH_FIELD_MISMATCH |
      NIC_ETHER_STAT_NO_CARRIER |
      NIC_ETHER_STAT_SHORT_PACKETS |
      NIC_ETHER_STAT_OVERSIZED_PACKETS |
      NIC_ETHER_STAT_INTERNAL_RX_ERRORS |
      NIC_ETHER_STAT_JABBER_DETECTED |
      NIC_ETHER_STAT_EXCESSIVE_DEFERRALS |
      NIC_ETHER_STAT_SINGLE_COLLISIONS |
      NIC_ETHER_STAT_MULTI_COLLISIONS |
      NIC_ETHER_STAT_LATE_COLLISIONS |
      NIC_ETHER_STAT_XCOLL_ABORTED |
      NIC_ETHER_STAT_TOTAL_COLLISION_FRAMES |
      NIC_ETHER_STAT_JABBER_DETECTED |
      NIC_ETHER_STAT_INTERNAL_TX_ERRORS;
}



__SRCVERSION( "$URL: http://community.qnx.com/svn/repos/core_networking/trunk/sys/dev_qnx/p40xx/stats.c $ $Rev: 904 $" );
