/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */
 
#include "proto.h"
#include <sys/mman.h>
#include <unistd.h>
#include <hw/inout.h>

static struct sigevent event;

static const struct sigevent* i2c_isr (void *area, int id) {
	mpc8572_dev_t *dev = area;

	if (dev->iid != id) {
		return NULL;
	}

	dev->i2c_status = in8(dev->unitbase + MPC8572_I2C_MSR);
	/* check if the interrupt come from the proper interface */
	if (dev->i2c_status & MPC8572_I2C_MSR_MIF) {
		/* clean the I2C MSR register interrupt flag */
		out8(dev->unitbase + MPC8572_I2C_MSR, 0);
		return &event;
	} else {
		return NULL;
	}
}

void *
mpc8572_init(int argc, char *argv[])
{
	mpc8572_dev_t   *dev;

	if (-1 == ThreadCtl(_NTO_TCTL_IO, 0))
	{
        perror("ThreadCtl");
        return NULL;
    }
	
	dev = malloc(sizeof(mpc8572_dev_t));
    if (!dev)
    {
        return NULL;
    }
	
	if (-1 == mpc8572_parseopts(dev, argc, argv))
        goto fail;

	dev->unitbase = mmap_device_io(dev->reglen, dev->i2cbase);
    if (dev->unitbase == (uintptr_t)MAP_FAILED)
    {
        perror("mmap_device_io");
        goto fail;
    }
	
    dev->restart = 0;
    
    /* Stop the I2C Controller */
    out8(dev->unitbase + MPC8572_I2C_MCR, 0);
    
    mpc8572_set_bus_speed(dev, 100000, NULL);
    
    /* Clear arbitration */
    out8(dev->unitbase + MPC8572_I2C_MSR, 0);
    
    /* Start with MEN */
    out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN);
    
    /* Check if the bus is idle */
    while (in8(dev->unitbase + MPC8572_I2C_MSR) & MPC8572_I2C_MSR_MBB)
    {
    	out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN | MPC8572_I2C_MCR_MSTA);
      	in8(dev->unitbase + MPC8572_I2C_MDR);
      	out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN);
      	out8(dev->unitbase + MPC8572_I2C_MCR, 0);
      	out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN);
	}
		 
	SIGEV_INTR_INIT(&event);
    dev->iid = InterruptAttach(dev->intr, i2c_isr, dev, sizeof(*dev), _NTO_INTR_FLAGS_TRK_MSK);
    if (dev->iid == -1) {
        perror("InterruptAttach failed!!");
        goto fail;
    }

    return dev;

fail:
    free(dev);
    return NULL;
}

void
mpc8572_fini(void *hdl)
{
    mpc8572_dev_t   *dev = hdl;

    InterruptDetach(dev->iid);
    /* Reset unit */
    out8(dev->unitbase + MPC8572_I2C_MCR, 0);
    munmap_device_io(dev->unitbase, dev->reglen);
    free(dev);
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/hardware/i2c/mpc8572/init.c $ $Rev: 580669 $" );
