/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */
 
#include "proto.h"
#include <assert.h>
#include <string.h>
#include <unistd.h>

uint8_t
mpc8572_poll_ready(mpc8572_dev_t *dev)
{
	uint8_t status;
	uint64_t ntime = 1e9;
	int intrerr = EOK;
	
	while (intrerr != ETIMEDOUT)
	{
		TimerTimeout(CLOCK_MONOTONIC, _NTO_TIMEOUT_INTR, NULL, &ntime, NULL);
		intrerr = InterruptWait_r(0,NULL);

		if(intrerr == EOK) {
			status = dev->i2c_status;

			/* check if it is the proper interrupt, dev->i2c_status has been updated in interrupt handler */ 
			if (status & MPC8572_I2C_MSR_MIF) {
				dev->i2c_status = 0;
				return status;
			}
			else {
				slogf(_SLOGC_I2C, _SLOG_CRITICAL, "I2C interrupt: Wrong interrupt");
			}
		}
	}

	slogf(_SLOGC_I2C, _SLOG_CRITICAL, "I2C interrupt timeout");
	return 0;
}

i2c_status_t
mpc8572_sendbyte(mpc8572_dev_t *dev,uint8_t byte)
{
	uint8_t status;
		
	out8(dev->unitbase + MPC8572_I2C_MDR, byte);
		
	status = mpc8572_poll_ready(dev);
	if (!(status & MPC8572_I2C_MSR_MIF))
	{
		out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN);
		return I2C_STATUS_ERROR; /* Timeout */
	}
 		
	if (!(in8(dev->unitbase + MPC8572_I2C_MCR) & MPC8572_I2C_MCR_MSTA)) 
	{   
		if (status & MPC8572_I2C_MSR_MAL)
        {
        	return I2C_STATUS_ARBL;
        }
        if (status & MPC8572_I2C_MSR_MAAS)
        {
        	return I2C_STATUS_ERROR;
        }
    }
  
	if (status & MPC8572_I2C_MSR_RXAK)
	{
		out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN);
        return I2C_STATUS_NACK;
    }

    return 0;
}	

i2c_status_t	
mpc8572_recvbyte(mpc8572_dev_t *dev, uint8_t *byte, int nack, int stop)
{
	uint8_t status;
	
	status = mpc8572_poll_ready(dev);
	
	/* Check for the data transfer bit */
	if (!(status & MPC8572_I2C_MSR_MCF))
	{
		out8(dev->unitbase + MPC8572_I2C_MCR,
			 MPC8572_I2C_MCR_MEN | MPC8572_I2C_MCR_MIEN);
		return I2C_STATUS_ERROR;
    }
    
    /* Generate ack on next to last byte */
    if (nack)
    {
    	out8(dev->unitbase + MPC8572_I2C_MCR,
              MPC8572_I2C_MCR_MEN | MPC8572_I2C_MCR_MIEN | MPC8572_I2C_MCR_MSTA |
              MPC8572_I2C_MCR_TXAK);
    }
    /* Generate stop on last byte */
    else if (stop)
    {
    	out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MIEN |MPC8572_I2C_MCR_MEN | MPC8572_I2C_MCR_TXAK);
    }

    *byte = in8(dev->unitbase + MPC8572_I2C_MDR);
     return 0;
}

i2c_status_t
mpc8572_sendaddr(mpc8572_dev_t *dev, unsigned addr, int dir, int restart)
{
	/* Enable the I2C module, module interrupt enable, start as master, trasmit mode */
	if (!restart)
		out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN);

	out8(dev->unitbase + MPC8572_I2C_MCR, MPC8572_I2C_MCR_MEN | 
	 		 MPC8572_I2C_MCR_MIEN | MPC8572_I2C_MCR_MSTA | (restart ? MPC8572_I2C_MCR_RSTA : 0) | MPC8572_I2C_MCR_MTX);
	
	/* Write target byte */
    return mpc8572_sendbyte(dev, (addr << 1) | dir);
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/hardware/i2c/mpc8572/poll.c $ $Rev: 580669 $" );
