/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */
 
#ifndef _mpc8572_I2C_PROTO_H_INCLUDED
#define _mpc8572_I2C_PROTO_H_INCLUDED

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdint.h>
#include <sys/neutrino.h>
#include <hw/inout.h>
#include <hw/i2c.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>
#include "mpc8572-i2c.h"


typedef struct {
    uintptr_t           unitbase;
    paddr64_t            i2cbase;
    unsigned            reglen;
    uint8_t             *buf;
    unsigned            scl_freq; 
    int                 intr;
    int                 iid;
    unsigned            slave_addr;
    i2c_addrfmt_t       slave_addr_fmt;
    unsigned            restart;
    uint8_t             i2c_status;
} mpc8572_dev_t;

#define MPC8572_I2C_ADDR_WR         0
#define MPC8572_I2C_ADDR_RD         1

#define _SLOGC_I2C	23

int mpc8572_parseopts(mpc8572_dev_t *dev, int argc, char *argv[]);

void *mpc8572_init(int argc, char *argv[]);
void mpc8572_fini(void *hdl);
int mpc8572_set_slave_addr(void *hdl, unsigned int addr, i2c_addrfmt_t fmt);
int mpc8572_set_bus_speed(void *hdl, unsigned int speed, unsigned int *ospeed);
i2c_status_t mpc8572_send(void *hdl, void *buf, unsigned int len, unsigned int stop);
i2c_status_t mpc8572_recv(void *hdl, void *buf, unsigned int len, unsigned int stop);
int mpc8572_version_info(i2c_libversion_t *version);
int mpc8572_driver_info(void *hdl, i2c_driver_info_t *info);

i2c_status_t mpc8572_sendaddr(mpc8572_dev_t *dev, 
                              unsigned addr, int read, int restart);
i2c_status_t mpc8572_sendbyte(mpc8572_dev_t *dev, uint8_t byte);
i2c_status_t mpc8572_recvbyte(mpc8572_dev_t *dev, uint8_t *byte,int nack, int stop);
uint8_t mpc8572_poll_ready(mpc8572_dev_t *dev);


#endif

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/hardware/i2c/mpc8572/proto.h $ $Rev: 580669 $" )
