/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */



#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <unistd.h>
#include <gulliver.h>
#include <sys/mman.h>
#include <inttypes.h>
#include <signal.h>
#include <setjmp.h>
#include <fcntl.h>
#include <assert.h>

#include <sys/platform.h>
#include <sys/rsrcdbmgr.h>
#include <sys/slogcodes.h>
#include <sys/slog.h>

#include <hw/inout.h>
#include "p4080.h"
#include <drvr/hwinfo.h>
#include <hw/hwinfo_85xx.h>
#include <hw/hwinfo_p40xx.h>



pdrvr_entry_t p4080_entry = {
	10,
	p4080_attach,
	p4080_detach,
	p4080_cnfg_bridge,
	p4080_read_cnfg,
	p4080_write_cnfg,
	p4080_special_cycle,
	p4080_map_irq,
	p4080_avail_irq_ads,
	p4080_map_addr,
	p4080_bus_info
};  
 
typedef struct pci_window_structure { 
	uint64_t	base_addr; 
	uint32_t	size; 
	uint64_t	length; 
	uint64_t	pci_addr; 
} pci_window_t;

pci_window_t	pci_windows[8]; 

/* default to PEX #3 */
unsigned	pex_inst = 3;
uint32_t	pex_register_offset = p4080_PCIE_REGISTER_OFFSET;
uint32_t	pex_targetif_mask = 0;
paddr64_t	pex_membase = p4080_PCIE3_MEM_BASE;
paddr64_t	pex_iobase = p4080_PCIE3_IO_BASE;


static void dump_rsrcdbmgr1(char *name, int type)
{
	rsrc_alloc_t rsrc[256];
	int i, cnt;

	cnt =  rsrcdbmgr_query(rsrc, sizeof(rsrc)/sizeof(rsrc[0]), 0, type|RSRCDBMGR_FLAG_USED);

	slogf (_SLOGC_PCI, _SLOG_DEBUG1, "Resource manager database for %s:",name);	
	slogf (_SLOGC_PCI, _SLOG_DEBUG1, "%-4s %-10s %-10s %s", "Idx", "Start", "End", "Flags");	
	for(i=0; i<cnt && i<sizeof(rsrc)/sizeof(rsrc[0]); i++)
		slogf (_SLOGC_PCI, _SLOG_DEBUG1, "%3d: %010llx-%010llx 0x%x", i, rsrc[i].start, rsrc[i].end, rsrc[i].flags);	
	if(cnt > sizeof(rsrc)/sizeof(rsrc[0]))
		slogf (_SLOGC_PCI, _SLOG_DEBUG1, "...");	
}

static void dump_rsrcdbmgr_db()
{

	dump_rsrcdbmgr1("Memory",   RSRCDBMGR_MEMORY);
	dump_rsrcdbmgr1("IRQs",     RSRCDBMGR_IRQ);
	dump_rsrcdbmgr1("PCI Mem",  RSRCDBMGR_PCI_MEMORY);
	dump_rsrcdbmgr1("IO Ports", RSRCDBMGR_IO_PORT);
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_pex_select(int inst)
{
	if (inst < 1 || inst > 3)
		return -1;

	pex_register_offset = p4080_PCIE_REGISTER_OFFSET +
		((PCIEX_NUM_INST - inst) * p4080_PCIE_REGISTER_SIZE);

	pex_targetif_mask = (PCIEX_NUM_INST - inst) << 20;

	switch (inst) {
		case 1:
			pex_membase = p4080_PCIE1_MEM_BASE;
			pex_iobase = p4080_PCIE1_IO_BASE;
			break;
		case 2:
			pex_membase = p4080_PCIE2_MEM_BASE;
			pex_iobase = p4080_PCIE2_IO_BASE;
			break;
		case 3:
			pex_membase = p4080_PCIE3_MEM_BASE;
			pex_iobase = p4080_PCIE3_IO_BASE;
			break;
	}

	return 0;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

uint64_t	p4080_find_ccsr_info ()
{
	struct	asinfo_entry  *asinfo;
	int32_t	i, num;

	num = _syspage_ptr->asinfo.entry_size / sizeof(*asinfo);
	asinfo = SYSPAGE_ENTRY (asinfo);

	for (i = 0; i < num; ++i) {
		char *name;
		name = __hwi_find_string (asinfo->name);
		if (strcmp (name, "immr")) {
			asinfo++;
			continue;
			}
		slogf (_SLOGC_PCI, _SLOG_DEBUG1,"Returning CCSR immr asinfo as 0x%llx",asinfo->start);	
		return (asinfo->start);
		}
	return 0;
} 

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

uint32_t	p4080_find_mem_info (uint64_t* mem_start, uint64_t* mem_size)

{
struct	asinfo_entry  *asinfo;
int		i, j, num, found = 0;
struct	{
	uint64_t	start;
	uint64_t	end;
	} ram_info [8];
	
	num = _syspage_ptr->asinfo.entry_size / sizeof (*asinfo);
	asinfo = SYSPAGE_ENTRY (asinfo);

	for (i = j = 0; i < num; ++i) {
		char *name;
		name = __hwi_find_string (asinfo->name);
		if (strcmp (name, "ram")) {
			asinfo++;
			continue;
			}
		ram_info [j].start = asinfo->start;
		ram_info [j++].end = asinfo->end;
		found = 1;
		asinfo++;
		}
	if (found) {
		j--;
		*mem_start = ram_info [0].start;
		*mem_size = ram_info [j].end - ram_info [0].start + 1;
		}
	return (found);
} 
 
/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int  p4080_get_pci_window (pci_window_t* pci_window_buf) 

{ 
	int			pci_count = 0; 
	unsigned 	pci_idx;
	unsigned	idx, lg2len;
	uint64_t 	len;
	unsigned	hwi_off;
	hwi_tag		*mem_busattr_tag;
	hwi_tag		*io_busattr_tag;

   	for(pci_idx=0; pci_idx<3; pci_idx++) {
		hwi_off = hwi_find_bus(PPCP40xx_HWI_PCIe, pci_idx);
		idx = 0;
		mem_busattr_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_busattr, &idx);
		io_busattr_tag  = hwi_tag_find(hwi_off, HWI_TAG_NAME_busattr, &idx);

		pci_window_buf[pci_idx*2+0].base_addr = mem_busattr_tag->busattr.addr;
		pci_window_buf[pci_idx*2+0].length    = mem_busattr_tag->busattr.size;
		len = pci_window_buf[pci_idx*2+0].length-1;
		for(lg2len=0; len != 0; lg2len++) len >>= 1;
		pci_window_buf[pci_idx*2+0].size = lg2len-1;
#if notyet
		if(mem_busattr_tag->busattr.flags & hwi_busattr_PCI_mem_BAR64)
			pci_window_buf[pci_idx*2+0].pci_addr = pci_window_buf[pci_idx*2+0].base_addr;
		else
#endif
			pci_window_buf[pci_idx*2+0].pci_addr = pci_window_buf[pci_idx*2+0].base_addr & 0xffffffffULL; /* translate to 32-bit by truncation */
		pci_count++;
		slogf (_SLOGC_PCI, _SLOG_DEBUG1, "PCIe %d - base %llx - length %llx (%llx)", pci_idx+1, pci_window_buf[pci_idx*2+0].base_addr, pci_window_buf[pci_idx*2+0].length, 1ULL<<(pci_window_buf[pci_idx*2+0].size+1));

		pci_window_buf[pci_idx*2+1].base_addr = io_busattr_tag->busattr.addr;
		pci_window_buf[pci_idx*2+1].length    = io_busattr_tag->busattr.size;
		len = pci_window_buf[pci_idx*2+1].length-1;
		for(lg2len=0; len != 0; lg2len++) len >>= 1;
		pci_window_buf[pci_idx*2+1].size = lg2len-1;
		pci_window_buf[pci_idx*2+1].pci_addr = 0; /* IO space always starts at 0 */
		pci_count++;
		slogf (_SLOGC_PCI, _SLOG_DEBUG1, "PCIe %d - base %llx - length %llx (%llx)", pci_idx+1, pci_window_buf[pci_idx*2+1].base_addr, pci_window_buf[pci_idx*2+1].length, 1ULL<<(pci_window_buf[pci_idx*2+1].size+1));
	}

	return pci_count; 
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_seed()

{
	rsrc_alloc_t	ralloc;

	// seed memory
	ralloc.start	= 0x003ff0000ULL;
	ralloc.end		= (1ULL<<36) - 1; /* 36-bit PHYS range */
	ralloc.flags	= RSRCDBMGR_MEMORY;

	if (rsrcdbmgr_create(&ralloc, 1) == -1) {
		perror("Unable to seed resource memory: ");
		return ENOMEM;
	}

	// seed irqs
	ralloc.start = 1;		
	ralloc.end	 = 30;
	ralloc.flags = RSRCDBMGR_IRQ;

	if (rsrcdbmgr_create(&ralloc, 1) == -1) {
		perror("Unable to seed resource irqs: ");
		return ENOMEM;
	}

	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_avail_irq_ads(void *hdl, uint32_t bus, uint32_t devfunc, uint32_t *list, uint32_t *nelm)

{
	uint_t	dev;
	uint_t  func;
	
	dev = ( devfunc >> 3 ) & 0x1F;
	func = devfunc & 0x07;

	/*
	 * Route IRQ depending on which PEX we are on.
	 */
	switch (bus) {
		case	0:
		case	1:
		{
			unsigned extirq_dev = hwi_find_bus(PPCP40xx_HWI_PCIe, 0);
			unsigned irq = func;
			list[0] = hwitag_find_ivec(extirq_dev, &irq);
			slogf(_SLOGC_PCI, _SLOG_ERROR, "pci_server: Returning IRQ %d, For PCI Express1 Function%d", list[0], func);
			assert(list[0] != HWI_ILLEGAL_VECTOR);
			break;
		}
		case	2:
		case	3:
		{
			unsigned extirq_dev = hwi_find_bus(PPCP40xx_HWI_PCIe, 2);
			unsigned irq = func;
			list[0] = hwitag_find_ivec(extirq_dev, &irq);
			slogf(_SLOGC_PCI, _SLOG_ERROR, "pci_server: Returning IRQ %d, For PCI Express3 Function%d", list[0], func);
			assert(list[0] != HWI_ILLEGAL_VECTOR);
			break;
		}
		case	4:
		case	5:
		{
			unsigned extirq_dev = hwi_find_bus(PPCP40xx_HWI_PCIe, 1);
			unsigned irq = func;
			list[0] = hwitag_find_ivec(extirq_dev, &irq);
			slogf(_SLOGC_PCI, _SLOG_ERROR, "pci_server: Returning IRQ %d, For PCI Express2 Function%d", list[0], func);
			assert(list[0] != HWI_ILLEGAL_VECTOR);
			break;
		}

		break;
		default:
			slogf(_SLOGC_PCI, _SLOG_ERROR, "pci_server: Invalid bus %d, interrupt not assigned", bus);
			*nelm = 0;
			return (-1);
		}

	*nelm = 1;
	return EOK;
}	

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_avail_irq_gen(void *hdl, uint32_t bus, uint32_t devfunc, uint32_t *list, uint32_t *nelm)
{
	p4080_dev_t *pdev = (p4080_dev_t *)(hdl);
	uint_t	dev;
	uint_t  func;

	dev = ( devfunc >> 3 ) & 0x1F;
	func = devfunc & 0x07;

	if (bus == 0) {
		int dev_off = dev - pdev->devbase;
		if ((dev_off >= 0) && (dev_off < MAX_SLOTS)) {
			int irq_off;

			if (pdev->irqrotate) {
				irq_off = (func + dev_off) % MAX_SLOTS;
			} else {
				irq_off = func;
			}
			list[0] = pdev->irqbase + irq_off;

			*nelm = 1;
			return EOK;
		} else {
			slogf(_SLOGC_PCI, _SLOG_ERROR, "pci_server: Invalid device %d, interrupt not assigned", dev);
		}
	} else {
			slogf(_SLOGC_PCI, _SLOG_ERROR, "pci_server: Invalid bus %d, interrupt not assigned", bus);

	}
	
	*nelm = 0;
	return (-1);
}	

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_config(p4080_dev_t *pdev)

{
	rsrc_request_t 	rreq;
	rsrc_alloc_t	ralloc; 
	int				i, pci_count = 0;  
	uint32_t		temp, mem_cnt;
	uint64_t		mem_size; 
	char			*reg_base;
	extern int		verbose;

	p4080_seed();

	for (i = 0; i < 8; i++)
		memset ((char *) &pci_windows [i], 0, sizeof (pci_window_t));

	for (i = 0; i < 3; i++) {
		memset ((char *) &rreq, 0, sizeof (rreq));
		rreq.length = p4080_PCIE_REGISTER_LEN;
		rreq.align = 0;
		rreq.start = pdev->ccsr_base + p4080_PCIE_REGISTER_OFFSET + (i * 0x1000);
		rreq.end = rreq.start + p4080_PCIE_REGISTER_LEN - 1;
		rreq.flags = RSRCDBMGR_MEMORY | RSRCDBMGR_FLAG_RANGE;
		if (rsrcdbmgr_attach (&rreq, 1)) {
			perror ("Unable to allocate PCI REGISTER memory space: ");
			return ENOMEM;
			}
	
		pdev->pci_register_base [i] = pdev->ccsr_base + p4080_PCIE_REGISTER_OFFSET + (i * 0x1000);

		if ((pdev->register_base [i] = mmap_device_memory (0, p4080_PCIE_REGISTER_LEN, PROT_READ | PROT_WRITE | PROT_NOCACHE, 0, pdev->pci_register_base[i])) == MAP_FAILED) {
			perror ("Unable to mmap PCI CNF device memory: ");
			return (ENODEV);
			}
		}
	
	pci_count = p4080_get_pci_window (pci_windows); 
	
	if (pci_count <= 1) { 
		perror ("No PCI window allocated: "); 
		return (ENODEV); 
		} 

	// allocate pci space memory
	for (i = 0; i < pci_count; i += 2) {
		rreq.align = 0; 
		rreq.flags = RSRCDBMGR_MEMORY | RSRCDBMGR_FLAG_RANGE; 
		rreq.length = pci_windows[i].length; 
		rreq.start = pci_windows[i].base_addr; 
		rreq.end = pci_windows[i].base_addr + pci_windows[i].length - 1;
	
		if (rsrcdbmgr_attach(&rreq, 1)) {
			perror("Unable to allocate PCI memory space: ");
			return ENOMEM;
			} 

		pdev->pci_mem [i >> 1] = rreq.start;
		pdev->pci_mem_xlate [i >> 1] = pci_windows[i].base_addr - pci_windows[i].pci_addr;
		ralloc.start = pdev->pci_mem [i >> 1];
		ralloc.end = pdev->pci_mem [i >> 1] + pci_windows[i].length - 1;
		ralloc.flags = RSRCDBMGR_PCI_MEMORY | RSRCDBMGR_FLAG_RANGE;
		if (rsrcdbmgr_create(&ralloc, 1) == -1) {
			perror("Unable to create PCI memory space type: ");
			return ENOMEM;
			}
		}
 
	for (i = 1; i < pci_count; i += 2) {
		rreq.align	= 0; 
		rreq.flags	= RSRCDBMGR_MEMORY | RSRCDBMGR_FLAG_RANGE; 
		rreq.length = pci_windows[i].length; 
		rreq.start = pci_windows[i].base_addr; 
		rreq.end = pci_windows[i].base_addr + pci_windows[i].length - 1;
	
		if (rsrcdbmgr_attach(&rreq, 1)) {
			perror("Unable to allocate PCI IO space: ");
			return ENOMEM;
			}
	
		pdev->pci_io [i >> 1] = rreq.start;
		ralloc.start = pdev->pci_io [i >> 1]; 
		ralloc.end = pdev->pci_io [i >> 1] + pci_windows[i].length - 1;
		ralloc.flags = RSRCDBMGR_IO_PORT | RSRCDBMGR_FLAG_RANGE;
		if( rsrcdbmgr_create( &ralloc, 1 ) == -1 ) {
			perror( "Unable to create PCI IO space type: " );
			return( ENOMEM );
			}
		}

	/* disable all windows */
	for (i = 0 ; i < 3; i++) {
		reg_base = pdev->register_base [i];
		*(uint32_t *)(reg_base + p4080_PCI_POWAR1) = 0;
		*(uint32_t *)(reg_base + p4080_PCI_POWAR2) = 0;
		*(uint32_t *)(reg_base + p4080_PCI_POWAR3) = 0;
		*(uint32_t *)(reg_base + p4080_PCI_POWAR4) = 0;
		*(uint32_t *)(reg_base + p4080_PCI_PIWAR1) = 0;
		*(uint32_t *)(reg_base + p4080_PCI_PIWAR2) = 0;
		*(uint32_t *)(reg_base + p4080_PCI_PIWAR3) = 0;
		out32((uintptr_t)(reg_base + p4080_PCI_CFG_ADDR), 0x84000004);
		pdev->link_trained [i] = in8 ((uintptr_t) reg_base + p4080_PCI_CFG_DATA);
		if (pdev->link_trained [i] != 0x16) {
			pdev->link_trained [i] = 0;
			}
		}

	mem_cnt = 0;
	mem_size = pdev->mem_size;
	while (mem_size - 1) {
		mem_size >>= 1;
		mem_cnt++;
		}

	for (i = 0; i < pci_count; i += 2) { 
		reg_base = pdev->register_base [i >> 1];

		/* outbound memory window */
		temp  = (uint32_t)(pci_windows[i].base_addr >> 12);
	    *(uint32_t *)(reg_base + p4080_PCI_POTAR1)  = (uint32_t)(pci_windows[i].pci_addr >> 12);
	    *(uint32_t *)(reg_base + p4080_PCI_POTEAR1) = 0x00000000; 
	    *(uint32_t *)(reg_base + p4080_PCI_POWBAR1) = temp; 
	    *(uint32_t *)(reg_base + p4080_PCI_POWAR1) = 0x80044000 | pci_windows[i].size; 

		/* outbound I/O window */
		temp = (uint32_t)(pci_windows[i + 1].base_addr >> 12); 
	    *(uint32_t *)(reg_base + p4080_PCI_POTAR2)  = 0x00000000;
	    *(uint32_t *)(reg_base + p4080_PCI_POTEAR2) = 0x00000000; 
	    *(uint32_t *)(reg_base + p4080_PCI_POWBAR2) = temp; 
	    *(uint32_t *)(reg_base + p4080_PCI_POWAR2) = 0x80088000 | pci_windows[i + 1].size; 

		/* inbound */
		*(uint32_t *)(reg_base + p4080_PCI_PITAR3)  = (uint32_t)((pdev->mem_start & 0xffffff000ULL) >> 12); 
	    *(uint32_t *)(reg_base + p4080_PCI_PIWBAR3) = (uint32_t)((pdev->mem_start & 0xffffff000ULL) >> 12); 
	    *(uint32_t *)(reg_base + p4080_PCI_PIWBEAR3) = 0x00000000; 
	    *(uint32_t *)(reg_base + p4080_PCI_PIWAR3) = 0xa0f55000 | ((mem_cnt) & 0x0000003f);
		}

	/* configure as master, I/O enable, memory enable */
	p4080_read_cnfg (pdev, 0, 0, 4, 4, &temp);
	temp |= 0x07000000;
	p4080_write_cnfg (pdev, 0, 0, 4, 4, temp);
    
	if(verbose > 2) dump_rsrcdbmgr_db();

	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_attach(char *options, void **hdl)

{
	p4080_dev_t		*pdev;	
	int					opt;
	char				*value; 
	static char	*pci_drvr_opts[] = { 
		                    "auto",
							"base", 
							"pcibase",
							"irqbase",
							"irqrotate",
							"devbase",
							"x2b",
							"x3",
							"pex",
							NULL
						};

	if ((pdev = calloc(1, sizeof(p4080_dev_t))) == NULL) {
		return ENOMEM;
		}

	pdev->irqbase = -1;
	pdev->irqrotate = 1;
	pdev->devbase = -1;

	while (options && *options != '\0') {
		if ((opt = getsubopt(&options, pci_drvr_opts, &value)) == -1 ) {
			continue;
		}
		switch (opt) { 
		    case 0:     
						// obsolete auto_detect option
						continue;
			case 1:
						pdev->ccsr_base= strtoull( value, 0, 0 );

						continue;
			case 2:
						pdev->pci_membase= strtoull( value, 0, 0 );
						continue;
			case 3:
						pdev->irqbase = strtoul( value,0,0 );
						p4080_entry.avail_irq = p4080_avail_irq_gen;
						continue;
			case 4:
						pdev->irqrotate = strtoul( value,0,0 );
						p4080_entry.avail_irq = p4080_avail_irq_gen;
						continue;
			case 5:
						pdev->devbase = strtoul( value,0,0 );
						p4080_entry.avail_irq = p4080_avail_irq_gen;
						continue;
			case 6:
			{
						unsigned extirq_dev = hwi_find_device(PPC85xx_HWI_EXT_IRQ, 0);
						unsigned irq = 0;
						pdev->irqbase = hwitag_find_ivec(extirq_dev, &irq); // PPC85xx_INTR_IRQ0;
						pdev->irqrotate = 1;
						pdev->devbase = 18;
						p4080_entry.avail_irq = p4080_avail_irq_gen;
						continue;
			}
			case 7:
			{
						unsigned extirq_dev = hwi_find_device(PPC85xx_HWI_EXT_IRQ, 0);
						unsigned irq = 0;
						pdev->irqbase = hwitag_find_ivec(extirq_dev, &irq); // PPC85xx_INTR_IRQ0;
						pdev->irqrotate = 0;
						pdev->devbase = 19;
						p4080_entry.avail_irq = p4080_avail_irq_gen;
						continue;
			}
			case 8:
						pex_inst = strtoul( value,0,0 );
						if (p4080_pex_select(pex_inst) == -1) {
							fprintf(stderr, "Invalid PCI-E controller instance\n");
							return -1;
						}
						continue;
			default:
						break;
		}
	}
	
	if ((pdev->irqbase == -1) || (pdev->devbase == -1)) {
		if ((pdev->irqbase != -1) || (pdev->devbase != -1)) {
			/* Error condition. Must specify both irq and dev base for
			configuration to be valid. */
			fprintf(stderr, "Both the interrupt base and device base must be specified\n" );
			free(pdev);
			return -1;
		}
	}

	if (!pdev->pci_membase) {
       pdev->pci_membase = pex_membase;
	}
	pdev->pci_iobase = pex_iobase;
	
	if ((!pdev->ccsr_base) &&
       (pdev->ccsr_base = p4080_find_ccsr_info ()) == 0) {       
		perror ("Unable to find base address ");
		free(pdev);
		return -1;
	    }
    
    if (! p4080_find_mem_info (&pdev->mem_start, &pdev->mem_size)) {
    	perror ("Unable to find memory ");
    	free (pdev);
    	return -1;
	    }	

	if (!p4080_config (pdev)) {
		*hdl = pdev;
		return (EOK);
		}
	free (pdev);
	return (-1);
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_detach(void *hdl)

{
	p4080_dev_t	*pdev;
	int	i;

	pdev = (p4080_dev_t *)hdl;	

	// free memory
	for (i = 0; i < 3; i++) {
		munmap_device_memory(pdev->register_base [i], p4080_PCIE_REGISTER_LEN);
		}
	free(pdev);

	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_cnfg_bridge(void *hdl, uint32_t bus, uint32_t devfunc, pci_bus_t *pbus)

{
p4080_dev_t	*pdev = (p4080_dev_t *)hdl;

	pbus->pciex_addr = pdev->register_base [0];
	pbus->pciex_size = 0xe00;
	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_read_cnfg(void *hdl, unsigned bus, unsigned devfunc, unsigned reg, unsigned width, void *buf)

{
	uint_t			addr;
	uint_t			dev;
	uint_t			func;
	void			*addrp;
	char			*reg_base;
	p4080_dev_t	*pdev = (p4080_dev_t *)hdl;


	addrp = NULL;
	dev = (devfunc >> 3) & 0x1F;
	func = devfunc & 0x07;
	
	if (bus > 255 || dev > 31 || func > 7) {
		return (-1);
		}
	switch (bus) {
		case	0:
		case	1:
			reg_base = pdev->register_base [0];
			if (bus && !pdev->link_trained [0])
				return (-1);
			break;
		case	2:
			reg_base = pdev->register_base [2];
			bus = 0;
			break;
		case	3:
			reg_base = pdev->register_base [2];
			bus = 1;
			if (!pdev->link_trained [2])
				return (-1);
			break;
		case	4:
			reg_base = pdev->register_base [1];
			bus = 0;
			break;
		case	5:
			reg_base = pdev->register_base [1];
			bus = 1;
			if (!pdev->link_trained [1])
				return (-1);
			break;

		default:
			return (-1);
		}
	addr = (bus << 16) | (dev << 11) | (func << 8) | (reg & 0xff);
	if (reg > 0xff)
		addr |= ((reg & 0xf00) << 16);
	out32 ((uintptr_t)(reg_base + p4080_PCI_CFG_ADDR), p4080_PCI_ENABLE_CONFIG | (addr & ~0x3));
	addrp = reg_base + p4080_PCI_CFG_DATA ;
	
	switch (width) {
		case 1:
			*((uint8_t *)buf) = in8((uintptr_t)addrp);
			break;
		case 2:
			*((uint16_t *)buf) = in16((uintptr_t)addrp);
			break;
		case 4:
			*((uint32_t *)buf) = in32((uintptr_t)addrp);
			break;
	}
	return PCI_SUCCESS;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_write_cnfg(void *hdl, unsigned bus, unsigned devfunc, unsigned reg, unsigned width,  const uint32_t data)

{
	uint_t			addr;
	uint_t			dev;
	uint_t			func;
	void			*addrp;
	char			*reg_base;
	p4080_dev_t	*pdev = (p4080_dev_t *)hdl;

	
	addrp = NULL;
	dev = (devfunc >> 3) & 0x1F;
	func = devfunc & 0x07;
	
	if (bus > 255 || dev > 31 || func > 7) {
		return( -1);
		}
	switch (bus) {
		case	0:
		case	1:
			reg_base = pdev->register_base [0];
			if (bus && !pdev->link_trained [0])
				return (-1);
			break;
		case	2:
			reg_base = pdev->register_base [2];
			bus = 0;
			break;
		case	3:
			reg_base = pdev->register_base [2];
			bus = 1;
			if (!pdev->link_trained [2])
				return (-1);
			break;
		case	4:
			reg_base = pdev->register_base [1];
			bus = 0;
			break;
		case	5:
			reg_base = pdev->register_base [1];
			if (!pdev->link_trained [1])
				return (-1);
			bus = 1;
			break;

		default:
			return (-1);
		}

	addr = (bus << 16) | (dev << 11) | (func << 8) | (reg & 0xff);
	if (reg > 0xff)
		addr |= ((reg & 0xf00) << 16);
	out32((uintptr_t)(reg_base + p4080_PCI_CFG_ADDR), p4080_PCI_ENABLE_CONFIG | (addr & ~0x3));
	addrp = reg_base + p4080_PCI_CFG_DATA ;
	
	switch (width) {
		case 1:
			out8((uintptr_t)addrp, data);
			break;
		case 2:
			out16((uintptr_t)addrp, data);
			break;
		case 4:
			out32((uintptr_t)addrp, data);
			break;
	}
	return( PCI_SUCCESS );
}

/*****************************************************************************/
/* Map cpu address to pci address / pci address to cpu address               */
/*****************************************************************************/

int		p4080_map_addr(void *hdl, uint64_t iaddr, uint64_t *oaddr, uint32_t type, uint8_t bus)

{
p4080_dev_t	*pdev = (p4080_dev_t *)hdl;

	if (type & PCI_MAP_ADDR_PCI) {
		/*
		 * Map CPU to PCI
		 */
		if (type & PCI_IO_SPACE) {
			if (iaddr < pdev->pci_io [1])
				*oaddr = iaddr - pdev->pci_io [0];
			else
				if (iaddr < pdev->pci_io [2])
					*oaddr = iaddr - pdev->pci_io [1];
				else
					*oaddr = iaddr - pdev->pci_io [2];
			}
		if (type & (PCI_MEM_SPACE | PCI_ROM_SPACE)) {
			if (iaddr < pdev->pci_mem [1])
				*oaddr = iaddr - pdev->pci_mem_xlate [0];
			else
				if (iaddr < pdev->pci_mem [2])
					*oaddr = iaddr - pdev->pci_mem_xlate [1];
				else
					*oaddr = iaddr - pdev->pci_mem_xlate [2];
			}
		if (type & PCI_ISA_SPACE) {
			*oaddr = iaddr;
		}
		if (type & PCI_BMSTR_SPACE) {
			*oaddr = iaddr;
		}
	}
	if (type & PCI_MAP_ADDR_CPU) {
		/*
		 * Map PCI to CPU
		 */
		if (type & PCI_IO_SPACE) {
			switch (bus) {
				case	0:
				case	1:
					*oaddr = iaddr + pdev->pci_io [0];
					break;
				case	2:
				case	3:
					*oaddr = iaddr + pdev->pci_io [2];
					break;
				case	4:
				case	5:
					*oaddr = iaddr + pdev->pci_io [1];
					break;
				default:
					*oaddr = 0;
					break;
				}
			}
		if (type & (PCI_MEM_SPACE | PCI_ROM_SPACE)) {
			switch (bus) {
				case	0:
				case	1:
					*oaddr = iaddr + pdev->pci_mem_xlate [0];
					break;
				case	2:
				case	3:
					*oaddr = iaddr + pdev->pci_mem_xlate [2];
					break;
				case	4:
				case	5:
					*oaddr = iaddr + pdev->pci_mem_xlate [1];
					break;
				default:
					*oaddr = 0;
					break;
				}
			}
		if (type & PCI_ISA_SPACE) {
			*oaddr = iaddr;
		}
		if (type & PCI_BMSTR_SPACE) {
			*oaddr = iaddr;
		}
	}
	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_map_irq(void *hdl, uint32_t bus, uint32_t devfunc, uint32_t line, uint32_t pin, uint32_t flags)

{
	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_special_cycle(void *hdl, uint32_t bus, uint32_t data)

{
	hdl = hdl, bus = bus, data = data;
	return EOK;
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int		p4080_bus_info (void *hdl, uint8_t bus, uint64_t *io_base, uint64_t *io_size, uint64_t *mem_base, uint64_t *mem_size)

{
	switch (bus) {
		case	0:
		case	1:
			*io_base = pci_windows [1].base_addr+0x10000 ;
			*io_size = pci_windows [1].length;
			*mem_base = pci_windows [0].base_addr;
			*mem_size = pci_windows [0].length;
			break;
		case	2:
		case	3:
			*io_base = pci_windows [5].base_addr;
			*io_size = pci_windows [5].length;
			*mem_base = pci_windows [4].base_addr;
			*mem_size = pci_windows [4].length;
			break;
		case	4:
		case	5:
			*io_base = pci_windows [3].base_addr;
			*io_size = pci_windows [3].length;
			*mem_base = pci_windows [2].base_addr;
			*mem_size = pci_windows [2].length;
			break;

		default:
			return (-1);
		}
	return (PCI_SUCCESS);
}

__SRCVERSION( "$URL: http://svn/product/tags/internal/bsp/ntotrunk/freescale_p4080_ds/1.0.0/latest/hardware/pci/p4080/p4080.c $ $Rev: 223788 $" );
