/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */



#ifndef _p4080_H_INCLUDED
#define _p4080_H_INCLUDED

#ifndef __PLATFORM_H_INCLUDED
#include <sys/platform.h>
#endif

#ifndef _INTTYPES_H_INCLUDED
 #include <inttypes.h>
#endif

#ifndef __TYPES_H_INCLUDED
 #include <sys/types.h>
#endif

#include <hw/pci.h>
#include <hw/pci_devices.h>
#include <pcidrvr.h>

#include <sys/syspage.h>
#include <hw/sysinfo.h>

#ifndef BIT
#define BIT(x)	(1<<(x))
#endif

#define PCIEX_NUM_INST		3

/*
 * PCI Express Controller Register Bases
 */
#define p4080_PCIE1_MEM_BASE			0xc0000000
#define p4080_PCIE1_IO_BASE			0xe2000000

#define p4080_PCIE2_MEM_BASE			0xa0000000
#define p4080_PCIE2_IO_BASE			0xe2800000

#define p4080_PCIE3_MEM_BASE			0x80000000
#define p4080_PCIE3_IO_BASE			0xe3000000

/*
 * PCI Express Register Base
 */
#define p4080_PCIE_REGISTER_OFFSET	0x00200000
#define p4080_PCIE_REGISTER_SIZE		0x1000
#define p4080_PCIE_REGISTER_LEN		0x0E38
#define p4080_LAW_REGISTER_OFFSET		0x00000c00
#define p4080_LAW_REGISTER_LEN		0x1FC

/*
 * PCI Express Memory Mapped Regisers
 */
#define p4080_PCI_CFG_ADDR				0x00
#define p4080_PCI_CFG_DATA				0x04
#define p4080_PCI_POTAR0					0xC00
#define p4080_PCI_POTAR1					0xC20
#define p4080_PCI_POTAR2					0xC40
#define p4080_PCI_POTAR3					0xC60
#define p4080_PCI_POTAR4					0xC80
#define p4080_PCI_POTEAR0					0xC04
#define p4080_PCI_POTEAR1					0xC24
#define p4080_PCI_POTEAR2					0xC44
#define p4080_PCI_POTEAR3					0xC64
#define p4080_PCI_POTEAR4					0xC84
#define p4080_PCI_POWBAR0					0xC08
#define p4080_PCI_POWBAR1					0xC28
#define p4080_PCI_POWBAR2					0xC48
#define p4080_PCI_POWBAR3					0xC68
#define p4080_PCI_POWBAR4					0xC88
#define p4080_PCI_POWAR0				    0xC10
#define p4080_PCI_POWAR1				    0xC30
#define p4080_PCI_POWAR2				    0xC50
#define p4080_PCI_POWAR3				    0xC70
#define p4080_PCI_POWAR4				    0xC90
#define p4080_PCI_PITAR3				    0xDA0
#define p4080_PCI_PITAR2				    0xDC0
#define p4080_PCI_PITAR1				    0xDE0
#define p4080_PCI_PIWBAR3					0xDA8
#define p4080_PCI_PIWBAR2					0xDC8
#define p4080_PCI_PIWBAR1					0xDE8
#define p4080_PCI_PIWBEAR3				0xDAC
#define p4080_PCI_PIWBEAR2				0xDCC
#define p4080_PCI_PIWAR3				    0xDB0
#define p4080_PCI_PIWAR2				    0xDD0
#define p4080_PCI_PIWAR1				    0xDF0

#define p4080_PCI_ENABLE_CONFIG        BIT(31)

#define MAX_SLOTS	2

typedef struct _p4080_dev {
	paddr64_t		ccsr_base;
	paddr64_t		mem_start;
	uint64_t		mem_size;
	paddr64_t		pci_membase;
	paddr64_t		pci_iobase;
	char			*register_base [3];
	unsigned		flags;
	paddr64_t		pci_register_base [3];
	paddr64_t		pci_isa;
	paddr64_t		pci_io [3];
	paddr64_t		pci_mem [3];
	uint64_t		pci_mem_xlate [3];
	uint32_t		devbase;
	uint32_t		irqbase;
	uint8_t			revision;
	uint8_t			irqrotate;
	uint8_t			link_trained [3];
	uint8_t			rsvd[3];
} p4080_dev_t;

int		p4080_attach( char *options, void **hdl );
int		p4080_detach( void *hdl );
int		p4080_cnfg_bridge( void *hdl, uint32_t bus, uint32_t devfunc, pci_bus_t *pbus );
int		p4080_read_cnfg( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t reg, uint32_t width, void *buf );
int		p4080_write_cnfg( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t reg, uint32_t width, const uint32_t data );
int		p4080_special_cycle( void *hdl, uint32_t bus, uint32_t data );
int 	p4080_map_irq( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t line, uint32_t pin, uint32_t flags );
int 	p4080_avail_irq_ads( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t *list, uint32_t *nelm );
int 	p4080_avail_irq_gen( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t *list, uint32_t *nelm );
int 	p4080_map_addr( void *hdl, uint64_t iaddr, uint64_t *oaddr, uint32_t type, uint8_t bus);
int 	p4080_bus_info(void *hdl, uint8_t bus, uint64_t *io_addr, uint64_t *io_size, uint64_t *mem_addr, uint64_t *mem_size);

#endif

__SRCVERSION( "$URL: http://svn/product/tags/internal/bsp/ntotrunk/freescale_p4080_ds/1.0.0/latest/hardware/pci/p4080/p4080.h $ $Rev: 223788 $" )
