/*
 * $QNXLicenseC:
 * Copyright 2011 QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "ppc_espi.h"


/*
 * We use the same buffer for transmit and receive
 * For exchange, that's exactly what we wanted
 * For Read, it doesn't matter what we write to SPI, so we are OK.
 * For transmit, the receive data is put at the buffer we just transmitted, we are still OK.
 */
static const struct sigevent *spi_intr(void *area, int id)
{
    ppc_espi_dev_t  *dev = area;
    uintptr_t       base = dev->vbase;
    uint32_t        event, i, cnt;

    event = in32(base + PPC_ESPI_INT_EVENT);

    if (event & PPC_ESPI_SPIM_LT) {
        /* whole frame transaction done, Read rest data from RxFIFO */
        cnt = (event >> 24) & 0x003F;
        switch (dev->dlen) {
            case 1:
                for (i=0; i<cnt; i++) {
                    dev->pbuf[dev->rlen++] = in8(base + PPC_ESPI_RXD);
                }
                break;
            case 2:
                for (i=0; i<cnt; i+=2) {
                    *(uint16_t *)(&dev->pbuf[dev->rlen]) = in16(base + PPC_ESPI_RXD);
                    dev->rlen += 2;
                }
                break;
            default:        // can not be here
                break;
        }

        /*clear the Events*/
        out32(base + PPC_ESPI_INT_EVENT, event);

        /* disable interrupt */
        out32(base + PPC_ESPI_MASK, 0);
        return (&dev->spievent);

    } else if (event & PPC_ESPI_SPIE_RXT) {
        /* read 16(PPC_ESPI_FIFO_THR) bytes data from RxFIFO */
        for(cnt=0; cnt<PPC_ESPI_FIFO_THR; cnt+=4) {
            *(uint32_t *)(&dev->pbuf[dev->rlen]) = in32(base + PPC_ESPI_RXD);
            dev->rlen += 4;
        }

        /*
         * More to transmit?
         * Fill transmit character to TxFIFO as PPC_ESPI_FIFO_THR bytes or rest of the data
         */
        cnt = 0;
        while ((dev->tlen < dev->xlen) && (cnt < PPC_ESPI_FIFO_THR)) {
            out32(base + PPC_ESPI_TXD, *(uint32_t *)(&dev->pbuf[dev->tlen]));
            dev->tlen += 4;
            cnt += 4;
        }
    }

    /* clear the Events */
    out32(base + PPC_ESPI_INT_EVENT, event);
    return 0;

}

int ppc_espi_attach_intr(ppc_espi_dev_t *ppc_espi)
{
    if ((ppc_espi->chid = ChannelCreate(_NTO_CHF_DISCONNECT | _NTO_CHF_UNBLOCK)) == -1) {
        return -1;
    }

    if ((ppc_espi->coid = ConnectAttach(0, 0, ppc_espi->chid, _NTO_SIDE_CHANNEL, 0)) == -1) {
        goto fail0;
    }

    ppc_espi->spievent.sigev_notify   = SIGEV_PULSE;
    ppc_espi->spievent.sigev_coid     = ppc_espi->coid;
    ppc_espi->spievent.sigev_code     = PPC_ESPI_EVENT;
    ppc_espi->spievent.sigev_priority = PPC_ESPI_PRIORITY;

    /*
     * Attach SPI interrupt
     */
    ppc_espi->iid = InterruptAttach(ppc_espi->irq, spi_intr, ppc_espi, 0, _NTO_INTR_FLAGS_TRK_MSK);

    if (ppc_espi->iid != -1) {
        return 0;
    }

    ConnectDetach(ppc_espi->coid);
fail0:
    ChannelDestroy(ppc_espi->chid);

    return -1;
}

__SRCVERSION( "$URL$ $Rev$" );
