/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "startup.h"
#include "ppc/p4080cpu.h"
#include <hw/inout.h>

extern uint32_t is_ddr_interleaved();
extern void	mpc85xx_smp_boot_code(void);
extern void	(*mpc85xx_smp_entry_point)(void);
extern struct callout_rtn	sendipi_mpc85xx;

unsigned
board_smp_num_cpu()
{
	if (debug_flag) {
		kprintf("board_smp_init: 8 cpu\n");
	}
	return 8;
}

void
board_smp_init(struct smp_entry *smp, unsigned num_cpus)
{
	uintptr_t	ccsr;

	/* TODO: Change to appropriate callout
	 */
	smp->send_ipi = (void *)&sendipi_mpc85xx;

	// Reset cpus 1 through 7.  u-boot is trying to implement the ePAPR
	// standard, but doesn't seem to do it completely.  The various cpu
	// properties necessary to release the secondary cpus don't seem to
	// be present.  So, resort to a sledge hammer...
	ccsr = startup_io_map(MEG(1), immr_paddr);

	// First, reset the boot release register so that the CPUs won't start
	// running again immediately.
	out32(ccsr + PPCP4080_CCSR_OFF_BRR, 0x1);
	// Next, use the PIR register to reset everything except core 0
	out32(ccsr + PPCP4080_CCSR_OFF_PIR, 0xfe);

	startup_io_unmap(ccsr);
}

int
board_smp_start(unsigned cpu, void (*start)(void))
{
	uintptr_t	ccsr;
	uintptr_t	bptr;

	/*
	 * Tell mpc85xx_smp_boot_code where to jump to
	 */
	if (debug_flag > 1) {
		kprintf("board_smp_start: cpu%d -> %x\n", cpu, start);
	}

	mpc85xx_smp_entry_point = start;

	// The P4080 and 8572 share the same boot mechanism (though they use
	// registers in different locations to accomplish this).
	// Set the boot configuration so that secondary cpu execution begins
	// with mpc85xx_smp_boot_code.  Execution begins at the end of the page
	// specified in the boot translation register.  The mpc85xx_smp_boot_code
	// is set up so that the page ends with a jump to mpc85xx_smp_boot_code.
	ccsr = startup_io_map(MEG(1), immr_paddr);
	bptr = (uintptr_t)mpc85xx_smp_boot_code;
	out32(ccsr + PPCP4080_CCSR_OFF_BSTRH, 0);
	out32(ccsr + PPCP4080_CCSR_OFF_BSTRL, bptr);

	// Define the boot translation parameters.
	// Magic number 0x8100000B:
	//    0x80000000 - enable boot space translation
	//    0x01000000 - DDR controller 1 -- FIXME: 1 or 2?  Does it depend on boot code paddr?
	//    0x0000000B - 4K page
	if(is_ddr_interleaved()){
		out32(ccsr + PPCP4080_CCSR_OFF_BSTAR, 0x8140000B);
	}
	else{
		out32(ccsr + PPCP4080_CCSR_OFF_BSTAR, 0x8100000B);
	}

	// Bring CPUs out of reset by setting the appropriate bits in the boot release register.
	// The BRR has one bit per CPU.
	out32(ccsr + PPCP4080_CCSR_OFF_BRR, in32(ccsr + PPCP4080_CCSR_OFF_BRR) | 1<<cpu);


	startup_io_unmap(ccsr);

	return 1;
}

unsigned
board_smp_adjust_num(unsigned cpu)
{
	return cpu;
}

__SRCVERSION( "$URL: http://svn.ott.qnx.com/product/trunk/hardware/startup/boards/mpc8572ds/board_smp.c $ $Rev: 217585 $" );
