/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include "startup.h"
#include "hwinfo_private.h"
#include "hwinfo_85xx_proto.h"
#include "hwinfo_p40xx_proto.h"
#include <hw/hwinfo_p40xx.h>
#include <drvr/hwinfo.h>
#include "ppcp40xx_lbc.h"

//#define INIT_DEBUG

extern paddr_t pixis_paddr ;
	
static struct {
    int	id;
    char *name;
} target_names[] = {
    {ppcp40xx_law_target_id_SDRAM_1, " (SDRAM_1)"},
    {ppcp40xx_law_target_id_SDRAM_2, " (SDRAM_2)"},
    {ppcp40xx_law_target_id_SDRAM_I, " (SDRAM_I)"},
    {ppcp40xx_law_target_id_PCIe_1,  " (PCIe_1)"},
    {ppcp40xx_law_target_id_PCIe_2,  " (PCIe_2)"},
    {ppcp40xx_law_target_id_PCIe_3,  " (PCIe_3)"},
    {ppcp40xx_law_target_id_SRIO1,   " (SRIO1)"},
    {ppcp40xx_law_target_id_SRIO2,   " (SRIO2)"},
    {ppcp40xx_law_target_id_LBUS,    " (LBUS)"},
    {ppcp40xx_law_target_id_BMSP,    " (BMSP)"},
    {ppcp40xx_law_target_id_QMSP,    " (QMSP)"},
    {ppcp40xx_law_target_id_DCSR,    " (DCSR)"},
    {0, 0},
};

static char *
target_id2name(int target_id) {
    int i;

    for(i=0; target_names[i].name; i++)
	if(target_names[i].id == target_id)
	    return target_names[i].name;
    return "";
}

#ifdef INIT_DEBUG
/*
 * Skip leading '0' chars in a string.
 * Return pointer to first non-zero digit to
 * a max offset of n.
 */
static char *strip0(char *p, int n)
{
	int	i;

	for(i=0; i<n; i++)
		if(p[i] != '0')
			break;
	return p+i;
}

static void pp_paddr36(uint64_t paddr)
{
    char buf1[8+1], buf2[8+1], buf3[8+1];

	ksprintf(buf1, "%x", (uint32_t)((paddr >> 32) & 0xf));
	ksprintf(buf2, "%x", (uint32_t)((paddr >> 16) & 0xffff));
	ksprintf(buf3, "%x", (uint32_t)((paddr >>  0) & 0xffff));
	kprintf("%s_%s_%s", strip0(buf1,7), strip0(buf2,4), strip0(buf3,4));
}

static void pp_law_setup(char *msg)
{
	const unsigned num_law_regs  = ppcp4080_law_numreg();
	int i, n=0;
	char     buf[16];

	kprintf("\n");
	if(msg) kprintf("%s:\n", msg);

	kprintf("CCSR ");
	pp_paddr36((uint64_t)immr_paddr); kprintf(" - "); pp_paddr36((uint64_t)immr_paddr+(16*1024*1024)-1);
	ksprintf(buf, "%d", (16*1024*1024)/1024);
	kprintf(" %s%sK\n", "        "+strlen(buf), buf);

	for (i=0; i<num_law_regs; i++) {
		if(ppcp4080_law_is_enabled(i) || ppcp4080_law_get_size64(i) > 4 || ppcp4080_law_get_target(i) == ppcp40xx_law_target_id_SDRAM_1 || ppcp4080_law_get_target(i) == ppcp40xx_law_target_id_SDRAM_2 || ppcp4080_law_get_target(i) == ppcp40xx_law_target_id_SDRAM_I) {
			uint64_t addr   = ppcp4080_law_get_addr(i);
			uint64_t size   = ppcp4080_law_get_size64(i);
			int      target = ppcp4080_law_get_target(i);

			if(i<10) kprintf(" %d:  ", i);
			else     kprintf("%d:  ", i);
			pp_paddr36(addr); kprintf(" - "); pp_paddr36(addr+size-1);
			ksprintf(buf, "%d", (uint32_t)(size/1024));
			kprintf(" %s%sK", "        "+strlen(buf), buf);
			ksprintf(buf, "%x", target);
			kprintf(" 0x%s%s", strip0(buf,6), target_id2name(target));
			if(!ppcp4080_law_is_enabled(i)) kprintf("  *DISABLED*");
			kprintf("\n");
			n++;
		}
	}

	kprintf("\n");
	kprintf("Total %d of %d LAW registers enabled\n", n, num_law_regs);
}
#else
#define pp_law_setup(msg)	/**/
#endif

paddr_t old_ccsr_paddr;
paddr_t new_ccsr_paddr;
volatile uint32_t *old_ccsr_base;
volatile uint32_t *new_ccsr_base;
int law_ccsr;

void fixup_ccsrbar32(paddr_t *paddr) {
	extern paddr_t immr_paddr;
	extern void ppcp4080_law_unmap(void);
	uint32_t temp;

	old_ccsr_paddr = *paddr;
	new_ccsr_paddr = *paddr & 0xffffffffULL;

	if(old_ccsr_paddr == new_ccsr_paddr)
	    return; /* nothing to do, already 32-bit */

	/* Setup a temp 16MB LAW for new CCSR */
	law_ccsr = ppcp4080_law_alloc(); /* sets up internal syscfg_base mapping of immr_paddr for law IN32/OUT32 macros */
	ASSERT(law_ccsr != -1);


	/* setup temp LAW register */
	ppcp4080_law_set_addr(law_ccsr, new_ccsr_paddr);
	ppcp4080_law_set_size(law_ccsr, 0x1000);
	ppcp4080_law_set_target(law_ccsr, 0x1E); /* reserved */
	ppcp4080_law_enable(law_ccsr);
	(void)ppcp4080_law_get_size(law_ccsr); /* flush writes to CCSR space */

	old_ccsr_base = (void *)startup_io_map(2*sizeof(*old_ccsr_base), *paddr);
	new_ccsr_base = (void *)startup_io_map(3*sizeof(*new_ccsr_base), new_ccsr_paddr);

	/* Read old CCSRBAR */
	temp = (paddr_t)old_ccsr_base[0];
	temp = (paddr_t)old_ccsr_base[1];
	asm volatile ("isync");

	/* Use temp LAW to write new CCSRBAR */
	new_ccsr_base[0] = new_ccsr_paddr>>32;
	new_ccsr_base[1] = new_ccsr_paddr;

	/* Commit write of new CCSRBAR */
	new_ccsr_base[2] = 0x80000000;
	temp = new_ccsr_base[2]; /* flush the write to CCSRAR */
	asm volatile ("sync");
	/* at this point CCSR has been moved, old CCSR is no longer valid */

	/* update immr_paddr to new location and clear syscfg_base, local mappings */
	*paddr = immr_paddr = new_ccsr_paddr;
	ppcp4080_law_unmap();
	startup_io_unmap((uintptr_t)old_ccsr_base);
	startup_io_unmap((uintptr_t)new_ccsr_base);

	(void)ppcp4080_law_get_size(law_ccsr); /* flush writes to CCSR space */

	/* temp LAW now longer needed */
	ppcp4080_law_disable(law_ccsr);
	(void)ppcp4080_law_get_size(law_ccsr); /* flush writes to CCSR space */

}

/*******************************************************************************
* init_hwinfo()
* 
* Add devices to the 'hwinfo' section of the syspage.
* 
* The startup library contains hwi_xxx() support routines for adding most common
* devices to the 'hwinfo' section (uarts, ethernet devices etc). Additionally,
* some SOC's have a support function that will add all of the individual on-chip
* devices for you. An example is hwi_p4080.c which will add the on-chip
* P4080 device.
* 
* The interface definition for all devices that can be added and the attributes
* (ie. hwi tags) they support can be found in hwiattr.h
* 
* init_hwinfo() should be called BEFORE init_intrinfo() so that as interrupt
* vectors are assigned to the devices, the corresponding IRQ tags for the device
* can be filled in.
* 
*/

/*******************************************************************************
* Address Map for P4080DS (36-bit paddr)
*
* 0x000000000 - 0x07FFFFFFF - DDR RAM                 --2GB
* 0xf80000000 - 0xf9FFFFFFF - PCIe1 Memory            --512MB
* 0xfA0000000 - 0xfBFFFFFFF - PCIe2 Memory            --512MB
* 0xfC0000000 - 0xfDFFFFFFF - PCIe3 Memory            --512MB
* 0xfE0000000 - 0xfE7FFFFFF - Promjet                 --128MB    --eLBC Chip Select 1 --No PROMJET on the board
* 0xfE8000000 - 0xfEFFFFFFF - Nor Flash               --128MB    --eLBC Chip Select 0
* 0xfF4000000 - 0xfF5FFFFFF - BMAN                    --2MB
* 0xfF6000000 - 0xfF7FFFFFF - QMAN                    --2MB
* 0xfF8000000 - 0xfF800FFFF - PCIe1 IO                --64KB
* 0xfF8010000 - 0xfF801FFFF - PCIe2 IO                --64KB
* 0xfF8020000 - 0xfF802FFFF - PCIe3 IO                --64KB
* 0xfF8030000 - 0xfF812FFFF - NAND Flash Bank 1       --1MB      --eLBC Chip Select 2 --No NAND flash on the board
* 0xfF8130000 - 0xfF822FFFF - NAND Flash Bank 2       --1MB      --eLBC Chip Select 4 --No NAND flash on the board
* 0xfF8230000 - 0xfF832FFFF - NAND Flash Bank 3       --1MB      --eLBC Chip Select 5 --No NAND flash on the board
* 0xfF8330000 - 0xfF842FFFF - NAND Flash Bank 4       --1MB      --eLBC Chip Select 6 --No NAND flash on the board
* 0xfFFDF0000 - 0xfFFDF7FFF - PIXIS Registers         --32KB     --eLBC Chip Select 3
* 0x100000000 - 0x17FFFFFFF - DDR RAM				  --2GB
*
*/
extern uint32_t is_ddr_interleaved();
void init_hwinfo()
{
	unsigned hwi_bus_local;
	unsigned last_law_id = 0;	
	const unsigned num_law_regs  = ppcp4080_law_numreg();
	unsigned elbc_law_id   = ppcp4080_law_find_target(ppcp40xx_law_target_id_LBUS, NULL);
	const unsigned pixis_law_id  = ppcp4080_law_find_target(ppcp40xx_law_target_id_LBUS, &elbc_law_id);
	const unsigned sdram_law_id1 = ppcp4080_law_find_target(ppcp40xx_law_target_id_SDRAM_1, NULL);
	const unsigned sdram_law_id2 = ppcp4080_law_find_target(ppcp40xx_law_target_id_SDRAM_2, NULL);
	const unsigned sdram_law_id3 = ppcp4080_law_find_target(ppcp40xx_law_target_id_SDRAM_I, NULL);
	unsigned pci_mem_law_ids[3];
	paddr64_t pci_addr_prefix = 0;

	/* Call the hwi_p4080() which adds the P4080 device to the hwinfo section of the syspage */
	
	pp_law_setup("LAW config from bootloader");
	hwi_p4080();
	/*
	 * Do any other board specific additions and Configure LAW's and eLBC
	*/
	{
		/* 	Disable the LAW registers we are going to reconfigure so that we start in a known state.
		 *	We do not touch the SDRAM, eLBC window as we are running from RAM
		 *  and we currently only reconfigure PCIe spaces (the rest we take from the boot loader).
		 */
		unsigned i;
		for (i=0; i<3; i++) pci_mem_law_ids[i] = -1;
		for (i=0; i<num_law_regs; i++) {
			switch(ppcp4080_law_get_target(i)) {
			case ppcp40xx_law_target_id_LBUS:
			case ppcp40xx_law_target_id_SDRAM_1:
			case ppcp40xx_law_target_id_SDRAM_2:
			case ppcp40xx_law_target_id_SDRAM_I:
			case ppcp40xx_law_target_id_DCSR:
			case ppcp40xx_law_target_id_BMSP:
			case ppcp40xx_law_target_id_QMSP:
				/* leave these alone */
				break;
			case ppcp40xx_law_target_id_PCIe_1:
			case ppcp40xx_law_target_id_PCIe_2:
			case ppcp40xx_law_target_id_PCIe_3:
				/*
				 * To handle 32-bit and 36-bit paddr versions of U-boot we remember the upper 4
				 * paddr bits used by U-boot for the PCIe mem and io address spaces for use in
				 * our own PCIe bus definitions.
				 */
				if(ppcp4080_law_is_enabled(i)) {
				    pci_addr_prefix = ppcp4080_law_get_addr(i) & 0xf00000000ULL;
					/*
					 * Track which buses were enabled by U-Boot and only remove aliased memory
					 * for those buses.
					 */
				    if(ppcp4080_law_get_target(i) == ppcp40xx_law_target_id_PCIe_1) pci_mem_law_ids[0] = -2;
				    if(ppcp4080_law_get_target(i) == ppcp40xx_law_target_id_PCIe_2) pci_mem_law_ids[1] = -2;
				    if(ppcp4080_law_get_target(i) == ppcp40xx_law_target_id_PCIe_3) pci_mem_law_ids[2] = -2;
				}
				/* and fall though to... */
			default:
				ppcp4080_law_disable(i);
			}
		}
	}

#ifndef ALLOW_SDRAM_IO_OVERLAP
	if(!is_ddr_interleaved())
	{
		uint64_t sdram2_addr = GIG(4);
		const unsigned sdram_law_id2 = ppcp4080_law_find_target(ppcp40xx_law_target_id_SDRAM_2, NULL);
		ppcp4080_law_disable(sdram_law_id2);
		ppcp4080_law_set_addr(sdram_law_id2, sdram2_addr);
		ppcp4080_law_enable(sdram_law_id2);
		last_law_id = sdram_law_id2;
	}
#endif
		
	ppcp40xx_configure_elbc_reset(); //Resets all the eLBC base and options registers

	{
	/* Add the local bus for board devices */
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
		unsigned hwi_bus_internal = hwi_find_bus(HWI_ITEM_BUS_INTERNAL,0);

		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x124000, sizeof(paddr_t), 0, hwi_find_as(immr_paddr + 0x124000, 1));
		HWIATTR_SET_NUM_IRQ(&attr,1);
		hwi_bus_local = hwibus_add(PPCP40xx_HWI_LBC, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_bus_local) == 0);
		hwitag_add_common(hwi_bus_local, &attr);
	}
	
	/* Initialize RTC device attributes */
	{
		/* RTC is on second I2C bus */
		unsigned hwi_bus = hwi_find_bus(PPCP40xx_HWI_I2C, 1);
		hwiattr_rtc_t rtcattr = HWIATTR_RTC_T_INITIALIZER;
		
		ASSERT(hwi_bus != HWI_NULL_OFF);
		HWIATTR_RTC_SET_LOCATION(&rtcattr, 0x68, 0x14, 0, 0); // Address of RTC DS3232 in the 2nd I2C bus
		HWIATTR_RTC_SET_NUM_IRQ(&rtcattr, 1);
		/* there is a separate century register */
		HWIATTR_RTC_SET_CENTURY_REG(&rtcattr, "century", 0x05);
		hwidev_add_rtc("rtc", &rtcattr, hwi_bus);
		/* can't set the interrupt vector yet since they have not been assigned */
	}
#if 0 //Not configuring the NAND Flash on eLBC as of now (As there is no NAND flash installed on the board)
	/* add NAND flash */
	{
		unsigned i;
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
		unsigned hwi_off, law_nand; 
		
		_uint32 bframaddr_array[4] = {0xF8030000, 0xF8130000, 0xF8230000, 0xF8330000}; /*MAX_ETFS_BANKS is 4*/

		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x124000, sizeof(paddr_t), 0, hwi_find_as(immr_paddr + 0x124000, 1));
		
		ppcp40xx_configure_elbc(eLBC_DEVICE_NAND, bframaddr_array[0], BYTE_LENGTH, eLBC_MACHINE_FCM, 1, 12, 0, LARGE_PAGE, 2);
		ppcp40xx_configure_elbc(eLBC_DEVICE_NAND, bframaddr_array[1], BYTE_LENGTH, eLBC_MACHINE_FCM, 1, 12, 0, LARGE_PAGE, 4);
		ppcp40xx_configure_elbc(eLBC_DEVICE_NAND, bframaddr_array[2], BYTE_LENGTH, eLBC_MACHINE_FCM, 1, 12, 0, LARGE_PAGE, 5);
		ppcp40xx_configure_elbc(eLBC_DEVICE_NAND, bframaddr_array[3], BYTE_LENGTH, eLBC_MACHINE_FCM, 1, 12, 0, LARGE_PAGE, 6);

		/* the NAND flash device tag originally used was "mpc85xx_nand". Not sure why the mpc85xx was needed but
		 * ragardless we must create the original device by that name since ETFS uses hwi_find_item() and not
		 * hwi_find_device() */
		hwi_off = hwidev_add("nand_flash", 0, hwi_bus_local);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);

		/* Setup an 4MB window for NAND Flash */
		law_nand = ppcp4080_law_alloc();
		ASSERT(law_nand != -1);
		
 		ppcp4080_law_set_target(law_nand, ppcp40xx_law_target_id_LBUS);
		ppcp4080_law_set_addr(law_nand, bframaddr_array[0]);
		ppcp4080_law_set_size(law_nand, 0x400000);
		ppcp4080_law_enable(law_nand);
		last_law_id = law_nand;
	
		/* add 4 more location tags for the bank addresses */
		for(i=0; i<4; i++) {
			hwitag_add_location(hwi_off, bframaddr_array[i], 0x100000, 0, hwi_find_as(bframaddr_array[i], 1));
		}
	}
#endif
        /* add BMAN, QMAN if not present */
	if(ppcp4080_law_find_target(ppcp40xx_law_target_id_BMSP, NULL) == -1)
	{
		unsigned law_bman; 
		
		/* Setup an 2MB window for BMAN */
		law_bman = ppcp4080_law_alloc();
		ASSERT(law_bman != -1);
		
 		ppcp4080_law_set_target(law_bman, ppcp40xx_law_target_id_BMSP);
		ppcp4080_law_set_addr(law_bman,0xF4000000 );
		ppcp4080_law_set_size(law_bman,   0x200000);
		ppcp4080_law_enable(law_bman);
	}
	if(ppcp4080_law_find_target(ppcp40xx_law_target_id_QMSP, NULL) == -1)
	{
		unsigned law_qman; 

		/* Setup an 2MB window for QMAN */
		law_qman = ppcp4080_law_alloc();
		ASSERT(law_qman != -1);
		
 		ppcp4080_law_set_target(law_qman, ppcp40xx_law_target_id_QMSP);
		ppcp4080_law_set_addr(law_qman,0xF4200000 );
		ppcp4080_law_set_size(law_qman,   0x200000);
		ppcp4080_law_enable(law_qman);
	}

	/* add NOR flash */
	{
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
		unsigned hwi_off;
		paddr_t nor_addr = ppcp4080_law_get_addr(elbc_law_id)+0x08000000 /*0xE8000000*/;

		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x124000, sizeof(paddr_t), 0, hwi_find_as(immr_paddr + 0x124000, 0));
		ppcp40xx_configure_elbc(eLBC_DEVICE_NOR, nor_addr, WORD_LENGTH, eLBC_MACHINE_GPCM, 1, 5, 1, 0, 0);
		
		/* We are not configuring the law for Nor flash, as u-boot is already setting it up*/
		hwi_off = hwidev_add("nor_flash", 0, hwi_bus_local);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);

		/* 	add a location TAG for the flash. The NOR flash driver will use this to configure the ELBC to use
		 	these addresses. We want the second 128Mb of the 256Mb space.
		*/
		hwitag_add_location(hwi_off, nor_addr, ppcp4080_law_get_size(elbc_law_id)-0x08000000, 0, hwi_find_as(nor_addr, 1));
	}
	
#if 0 //Not configuring the PROMJET on eLBC as of now (As there is no PROMJET installed on the board)
	/* add PROMJET */
	{
		ppcp40xx_configure_elbc(eLBC_DEVICE_PROMJET, 0xE0000000, WORD_LENGTH, eLBC_MACHINE_GPCM, 1, 5, 1, 0, 1);
	}
#endif
	/* add PIXIS Registers */
	{
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
		unsigned hwi_off;
		paddr_t pixis_size = ppcp4080_law_get_size(pixis_law_id) /*4k*/;
		
#ifdef PR92451_WORKAROUND
		/*workaround for callout_io_map_indirect working incorrectly in callout_reboot_p40xx.s*/
		ppcp4080_law_disable(pixis_law_id);
		ppcp4080_law_set_addr(pixis_law_id, 0x0ffdf0000);
		ppcp4080_law_enable(pixis_law_id);
#endif

		pixis_paddr = ppcp4080_law_get_addr(pixis_law_id) /*0xfFFDF0000*/;
		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x124000, sizeof(paddr_t), 0, hwi_find_as(immr_paddr + 0x124000, 0));

		ppcp40xx_configure_elbc(eLBC_DEVICE_PIXIS, (uint32_t)pixis_paddr, BYTE_LENGTH, eLBC_MACHINE_GPCM, 1, 17, 1, 0, 3);		

		/* We are not configuring the law for Pixis, as u-boot is already setting it up*/		
		hwi_off = hwidev_add("pixis", 0, hwi_bus_local);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);

		/* 	add a location TAG for the PIXIS registers. The Pixis registers will use this to configure the ELBC to use
		 	these addresses. 
		*/
		hwitag_add_location(hwi_off, pixis_paddr, pixis_size, 0, hwi_find_as(pixis_paddr, 1));	
	}

	/* Configure PCIe memory and I/O space apertures */
	{
		unsigned i;
			struct {	// table of desired PCIe configuration values
				unsigned target_id;
				struct {
					uint64_t addr;
					uint64_t size;
				} mem;
				struct {
					uint64_t addr;
					uint64_t size;
				} io;
			} pci_params[] = /* default values for PCIe1, PCIe2 and PCIe3 respectively */
			{
				{
					.target_id = ppcp40xx_law_target_id_PCIe_1,
					.mem = {.addr = 0x80000000, .size = 0x20000000},
					.io = {.addr = 0xF8000000, .size = 0x10000}
				},
				{
					.target_id = ppcp40xx_law_target_id_PCIe_2,
					.mem = {.addr = 0xA0000000, .size = 0x20000000},
					.io = {.addr = 0xF8010000, .size = 0x10000}
				},
				{
					.target_id = ppcp40xx_law_target_id_PCIe_3,
					.mem = {.addr = 0xC0000000, .size = 0x20000000},
					.io = {.addr = 0xF8020000, .size = 0x10000}
				},
			};
		int autoconf = 1;
		uint64_t pcimem_base;
		extern uint32_t pci_default_bus_window;

		/*
		 * Reduce PCI MEM window sizes from 512Mb to save SDRAM <4Gb loss due to address aliasing
		 */
		for (i=0; i<3; i++)
		    pci_params[i].mem.size = MEG(pci_default_bus_window);

		pcimem_base = pci_params[0].mem.addr;
		for (i=0; i<3; i++) {
		    pci_params[i].mem.addr = pcimem_base;
		    pcimem_base += pci_params[i].mem.size;
		}

		for (i=0; i<3; i++)
		{
			if(ppcp4080_law_find_target(pci_params[i].target_id, NULL) >= 0)
				autoconf = 0; /* don't autoconf if any PCI LAW registers already defined */
		}
		/*
		 * 	Write the LAW registers for PCIe with the values we want or
		 * 	retrieve LAW values if not autoconf.
		*/

		/* set/get the memory and I/O base addresses and sizes for the 3 PCIe controllers */
		for (i=0; i<3; i++)
		{
			unsigned law_id_1, law_id_2;
			hwi_tag *mem_busattr_tag;
			hwi_tag *io_busattr_tag;
			const unsigned hwi_off = hwi_find_bus(PPCP40xx_HWI_PCIe, pci_params[i].target_id);
			unsigned idx = 0;

			ASSERT(hwi_off != HWI_NULL_OFF);

			mem_busattr_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_busattr, &idx);
			io_busattr_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_busattr, &idx);
			ASSERT(mem_busattr_tag != NULL);
			ASSERT(io_busattr_tag != NULL);

			idx = 0;
			if(autoconf) {
				/* configure memory aperture */
				law_id_1 = ppcp4080_law_alloc();
				ASSERT(law_id_1 != -1);
				ppcp4080_law_set_target(law_id_1, pci_params[i].target_id);
				ppcp4080_law_set_addr(law_id_1, pci_params[i].mem.addr | pci_addr_prefix);
				ppcp4080_law_set_size(law_id_1, pci_params[i].mem.size);
				ppcp4080_law_enable(law_id_1);
				mem_busattr_tag->busattr.addr = pci_params[i].mem.addr | pci_addr_prefix;
				mem_busattr_tag->busattr.size = pci_params[i].mem.size;
				mem_busattr_tag->busattr.flags |= hwi_busattr_USE_ADDR | hwi_busattr_isPCIe | hwi_busattr_PCI_memtype_MEM;
				last_law_id = law_id_1;
				if(pci_mem_law_ids[i] == -2) pci_mem_law_ids[i] = law_id_1;
			} else if((law_id_1 = ppcp4080_law_find_target(pci_params[i].target_id, NULL)) != -1) {
				mem_busattr_tag->busattr.addr = ppcp4080_law_get_addr(law_id_1);
				mem_busattr_tag->busattr.size = ppcp4080_law_get_size(law_id_1);
				mem_busattr_tag->busattr.flags |= hwi_busattr_USE_ADDR | hwi_busattr_isPCIe | hwi_busattr_PCI_memtype_MEM;
				last_law_id = law_id_1;
				pci_mem_law_ids[i] = law_id_1;
			} else
				continue;

			if(autoconf) {
				/* configure I/O aperture */
				law_id_2 = ppcp4080_law_alloc();
				ASSERT(law_id_2 != -1);
				ppcp4080_law_set_target(law_id_2, pci_params[i].target_id);
				ppcp4080_law_set_addr(law_id_2, pci_params[i].io.addr | pci_addr_prefix);
				ppcp4080_law_set_size(law_id_2, pci_params[i].io.size);
				ppcp4080_law_enable(law_id_2);
				io_busattr_tag->busattr.addr = pci_params[i].io.addr | pci_addr_prefix;
				io_busattr_tag->busattr.size = pci_params[i].io.size;
				io_busattr_tag->busattr.flags |= hwi_busattr_USE_ADDR | hwi_busattr_isPCIe | hwi_busattr_PCI_memtype_IO;
				last_law_id = law_id_2;
			} else if((law_id_2 = ppcp4080_law_find_target(pci_params[i].target_id, &law_id_1)) != -1) {
				io_busattr_tag->busattr.addr = ppcp4080_law_get_addr(law_id_2);
				io_busattr_tag->busattr.size = ppcp4080_law_get_size(law_id_2);
				io_busattr_tag->busattr.flags |= hwi_busattr_USE_ADDR | hwi_busattr_isPCIe | hwi_busattr_PCI_memtype_IO;
				last_law_id = law_id_2;
			}
		}
	}
	/*
	2.2.2 Configuring Local Access Windows
	Once a local access window is enabled, it should not be modified while any 
	device in the system may be using the window. Neither should a new window
	be used until the effect of the write to the window is visible to all
	blocks that use the window. This can be guaranteed by completing a read of
	the last LAW configuration register before enabling any other devices to
	use the window. For example, if LAWs 03 are being configured in order
	during the initialization process, the last write (to LAWAR3) should be
	followed by a read of LAWAR3 before any devices try to use any of these
	windows. If the configuration is being performed by the e500 core, the
	read of LAWAR3 should be followed by an isync instruction.
	*/
	(void)ppcp4080_law_get_size(last_law_id);
	ppc_isync();

	pp_law_setup("Final LAW config");

#ifdef ALLOW_SDRAM_IO_OVERLAP
	/*
	 * Fixup any overlaps of LAW regs with SDRAM.  The SDRAM LAWs have
	 * the lowest priority so any other LAW region access will take precedence.
	 * This means the corresponding RAM is unusable so we remove it from the memory pool.
	 * NOTE: This assumes that all non-SDRAM LAW ids are < the SDRAM ids which is
	 * how they need to be configured.
	 */
	{
		int i;
		paddr_t end_sdram = 0;

		if(sdram_law_id1 >= 0 && ppcp4080_law_get_addr(sdram_law_id1)+ppcp4080_law_get_size64(sdram_law_id1) > end_sdram)
			end_sdram = ppcp4080_law_get_addr(sdram_law_id1)+ppcp4080_law_get_size64(sdram_law_id1);
		if(sdram_law_id2 >= 0 && ppcp4080_law_get_addr(sdram_law_id2)+ppcp4080_law_get_size64(sdram_law_id2) > end_sdram)
			end_sdram = ppcp4080_law_get_addr(sdram_law_id2)+ppcp4080_law_get_size64(sdram_law_id2);
		if(sdram_law_id3 >= 0 && ppcp4080_law_get_addr(sdram_law_id3)+ppcp4080_law_get_size64(sdram_law_id3) > end_sdram)
			end_sdram = ppcp4080_law_get_addr(sdram_law_id3)+ppcp4080_law_get_size64(sdram_law_id3);
		for (i=0; i<num_law_regs; i++) {
			if (ppcp4080_law_is_enabled(i) && (i != sdram_law_id1) && (i != sdram_law_id2) && (i != sdram_law_id3)) {
				uint64_t size = ppcp4080_law_get_size64(i);
				paddr_t range_start = ppcp4080_law_get_addr(i), range_end;
				range_end = range_start + size;
				if(find_ram_in_range(range_start, range_end, size, 1, 0, 0) != NULL_PADDR) {
					kprintf("Removing IO shadowed SDRAM from available memory: %P-%P%s\n", range_start, range_end-1, target_id2name(ppcp4080_law_get_target(i)));
					alloc_ram(range_start, size, 1);
				} else if(range_start < end_sdram) {
					/*
					 * ERROR: Couldn't find LAW region in sdram pool but address is in SDRAM range
					 * so someone is already using this memory.
					 */
					kprintf("ERROR: Cannot remove IO shadowed SDRAM %P-%P%s from available memory (already in use?)\n", range_start, range_end-1, target_id2name(ppcp4080_law_get_target(i)));
				}
#ifdef INIT_DEBUG
			   	else
					kprintf("Skipping IO region that doesn't shadow SDRAM:     %P-%P%s\n", range_start, range_end-1, target_id2name(ppcp4080_law_get_target(i)));
#endif
				/*
				 * Remove PCI MEM translated 36 to 32 bit window aliases from available memory
				 */
				if((i == pci_mem_law_ids[0] || i == pci_mem_law_ids[1] || i == pci_mem_law_ids[2])
						&& (range_start & ~0xffffffffULL) != 0) {
					range_start &= 0xffffffffULL; /* map to 32-bits */
					range_end = range_start + size;
					ASSERT(range_end <= 0x100000000ULL);
					if(find_ram_in_range(range_start, range_end, size, 1, 0, 0) != NULL_PADDR) {
						kprintf("Removing PCI MEM shadowed SDRAM from available memory: %P-%P%s\n", range_start, range_end-1, target_id2name(ppcp4080_law_get_target(i)));
						alloc_ram(range_start, size, 1);
					} else if(range_start < end_sdram) {
						/*
						 * ERROR: Couldn't find LAW region in sdram pool but address is in SDRAM range
						 * so someone is already using this memory.
						 */
						kprintf("ERROR: Cannot remove PCI MEM shadowed SDRAM %P-%P%s from available memory (already in use?)\n", range_start, range_end-1, target_id2name(ppcp4080_law_get_target(i)));
					}
#ifdef INIT_DEBUG
					else
						kprintf("Skipping PCI MEM region that doesn't shadow SDRAM:     %P-%P%s\n", range_start, range_end-1, target_id2name(ppcp4080_law_get_target(i)));
#endif
				}
			}
		}
	}
#endif
}

__SRCVERSION( "$URL:$ $Rev:$" );
