/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"
#include <ppc/e500cpu.h>

extern struct callout_rtn	reboot_p40xx;
paddr_t pixis_paddr;
	
uint32_t 	platform_clock;
uint32_t	pci_default_bus_window = 64;

const struct ppcv_chip *ppcv_list[] = {
	 &ppcv_chip_e500mc,
	 NULL
};

const struct callout_slot callouts[] = {
	{ CALLOUT_SLOT( reboot, _p40xx ) },
};


#define SER8250_LEN		64
char ser8250_init0[SER8250_LEN];
char ser8250_init1[SER8250_LEN];

const struct debug_device debug_devices_4080[] = {
	{ 	"8250",
		{	ser8250_init0,
			ser8250_init1
		},
		init_8250,
		put_8250,
		{	&display_char_8250,
			&poll_key_8250,
			&break_detect_8250,
		}
	},
};

extern void
ppc4080_set_get_clock_freqs(unsigned sys_clk, unsigned rtc,
						unsigned *platform_p, unsigned *core_p);

extern void
ppc4080_set_clock_freqs(unsigned sys_clk, unsigned rtc);

/*
 * main()
 *	Startup program executing out of RAM
 *
 * 1. It gathers information about the system and places it in a structure
 *    called the system page. The kernel references this structure to
 *    determine everything it needs to know about the system. This structure
 *    is also available to user programs (read only if protection is on)
 *    via _syspage->.
 *
 * 2. It (optionally) turns on the MMU and starts the next program
 *    in the image file system.
 */
int
main(int argc, char **argv, char **envv) {
	int 		opt;
	paddr_t		immr_loc;
	uint32_t	clock_rate;
	uint32_t 	svr;
	unsigned	use_l3_cache = 1;
	unsigned	fixup_ccsr32 = 0;
	extern void fixup_ccsrbar32(paddr_t *addr);

	// 0xFE000000 is where U-BOOT puts the CCSR.
	immr_loc = 0xFE000000; /*0xFE000000 CCSRBAR space (internal P4080 registers)*/

	// Default board clock is assumed to be 100MHz.
	// If the clock rate is set to 100 MHz, a "-t100000000" should be added
	// to startup's command line.
	clock_rate = 100000000; /*100Mhz Sysclk(Default)*/

	while((opt = getopt(argc, argv, COMMON_OPTIONS_STRING "c:Ct:3W:")) != -1) {
		switch(opt) {
		case 'c':
			immr_loc = strtopaddr(optarg, NULL, 0x10);
			break;
		case 't':
			clock_rate = strtoul(optarg, NULL, 10);
			break;
		case '3':
			use_l3_cache = 0;
			break;
		case 'C':
			fixup_ccsr32 = 1;
			break;
		case 'W':
			pci_default_bus_window = strtoul(optarg, NULL, 10);
			break;
		default:
			handle_common_option(opt);
			break;
		}
	}

	ppc_set_immr_paddr(immr_loc);
	if(fixup_ccsr32) fixup_ccsrbar32(&immr_loc);

	void (**call)(void) = (void (**)(void))lsp.callout.p;
	add_callout_array(callouts, sizeof(callouts));
	call += callouts[0].offset / sizeof(*call);
	callout_register_data( call, &pixis_paddr );
	//Note:pixis_paddr is set in init_hwinfo() and is used when init_system_private() is called to patch all callouts
	

	ppc4080_set_get_clock_freqs(clock_rate, 0, &platform_clock, NULL);

	/* Read the CCSR SVR field to determine the system version. */
	svr = get_spr(PPCE500_SPR_SVR);
	switch (svr) {
		case 0x82080010:	/* P4080DS rev1 with security*/
		case 0x82000010:	/* P4080DS rev1 without security*/
		case 0x82080020:	/* P4080DS rev2 with security*/
		case 0x82000020:	/* P4080DS rev2 without security*/
			add_typed_string(_CS_MACHINE, "P4080");

			/* Select DUART debug callouts. */
			ksprintf(ser8250_init0, "0x%P^0.0.%d.16",
						immr_loc + 0x11c500, platform_clock);
			ksprintf(ser8250_init1, "0x%P^0.0.%d.16",
						immr_loc + 0x11c600, platform_clock);
			select_debug(debug_devices_4080, sizeof(debug_devices_4080));
			break;

		default:
			crash("What processor is this? (svr = 0x%x)\n", svr);
 			break;
	}
	kprintf("\nP4080 console configured\n");
	kprintf("   spr_svr = 0x%x\n", svr);
	{   extern paddr_t old_ccsr_paddr, new_ccsr_paddr;
		if(new_ccsr_paddr != old_ccsr_paddr)
			kprintf("   old_ccsr_paddr = %P, new_ccsr_paddr = %P\n", old_ccsr_paddr, new_ccsr_paddr);
	}
	kprintf("   ccsr/immr_paddr = %P\n", immr_paddr);
	kprintf("   ser8250_init0   = %s\n", ser8250_init0);
	kprintf("   ser8250_init1   = %s\n", ser8250_init1);
	kprintf("\n");

	if(pci_default_bus_window < 4 || pci_default_bus_window > 512) {
	    kprintf("Error: Specified default PCIe bus window size %dMb out of range (4-512Mb), reset to 64Mb", pci_default_bus_window);
	    pci_default_bus_window = 64;
	}

	/*
	 * Collect information on all free RAM in the system.
	 */
	init_raminfo();

	/*
	 * Remove the RAM used by the modules in the image.
	 */
	alloc_ram(shdr->ram_paddr, shdr->ram_size, 1);

	/*
	 * Initialise SMP
	 */
	init_smp();

	/* In a virtual system we need to initialize the page tables */
	if(shdr->flags1 & STARTUP_HDR_FLAGS1_VIRTUAL) init_mmu();
	/*
	 * The following routines have hardware or system dependancies which
	 * may need to be changed.
	 *
	 * We do init_hwinfo() first so that interrupt vectors can be added to any
	 * configured devices by init_intrinfo()
	 */
	init_hwinfo();
	init_intrinfo();
	ppc_booke_dec_init_qtime();
	init_cacheattr();
	init_cpuinfo();


	kprintf("\nCpu_clk=%d, Sys_clk=%d, CCB=%d\n", cpu_freq, clock_rate, platform_clock);

	if (use_l3_cache) {
		ppcp40xx_init_l3_cache();
	} else {
		kprintf("\n\nNOTE!!!!   L3 cache not enabled by startup.\nAre you sure this is what you want?\n\n");
	}

	/*
	 * Load the bootstrap executables in the image filesystem and
	 * initialize various syspage pointers. This must be the *last*
	 * initialization done before transfering to the next program.
	 *
	*/
	init_system_private();

	/*
	 * This is real handy for debugging a new version of this startup
	 * program. Commenting this line out will save a whack of code.
	 */
	print_syspage();
	return 0;
}

__SRCVERSION( "$URL: $ $Rev: $" );
