#
# Copyright 2007, 2008, QNX Software Systems. 
# 
# Licensed under the Apache License, Version 2.0 (the "License"). You 
# may not reproduce, modify or distribute this software except in 
# compliance with the License. You may obtain a copy of the License 
# at: http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OF ANY KIND, either express or implied.
#
# This file may contain contributions from others, either as 
# contributors under the License or as licensors under other terms.  
# Please review this entire file for other proprietary rights or license 
# notices, as well as the QNX Development Suite License Guide at 
# http://licensing.qnx.com/license-guide/ for other information.
#


#
#
# An entry point for a hardware kernel callout.
# It may be called by an interrupt handler.
# This code MUST be position independant.
#

	.include "callout.ah"
	
# initialize space for the vector register offset and config flags tables. These
# values are necessarily 85xx variant specific and so the actual values are
# defined in the corresponding boards/startup/<variant>/vec_cfg_table.c file.

	.extern vec_regoffset_table_size
	.extern vec_config_flags_table_size

#
# Interrupts are always off in this code.
#
# interrupt_id_* and interrupt_eoi_*
#
# Note that interrupt_id_* and interrupt_eoi_* are not actually
# called by the kernel. For performance reasons, they are instead
# copied into place and intermixed with other kernel code. As such,
# they don't follow normal calling conventions. They should just fall out 
# their bottoms when done rather than attempting to do a return 
# instruction. The 'id' routine should return the interrupt level in
# the r14 register. The 'eoi' routine will find it still there. 
# By turning on various INTR_GENFLAG_* bits in the intrinfo_entry,
# the following values can be loaded before entry to this code.
# 
#		cpu number			=> r15
#		syspage_ptr			=> r16
#		intrinfo_entry		=> r17
#		intr mask count		=> r18 (only available in EOI routine)
#
# If the INTR_FLAG_CPU_FAULT bit is on, the eoi routine should set
# the r3 register to the following value to indicate the exception
# that has occurred:
# 		R3 = (fault_number << 16) | (signal_code << 8) | (signal_number)
# Set R3 to zero if no exception has occured. If the exception is memory
# related, set R4 to referenced address that caused the exception, zero
# if there is no address.
#
# interrupt_mask_* and interrupt_unmask_* are called as routines as follows
#
# int interrupt_mask(struct syspage_entry *sysp, int intr)
# int interrupt_unmask(struct syspage_entry *sysp, int intr)
#
#	r3 = sysp
#	r4 = intr
#
# Patching routine(s) are called with the following arguments
#	
# R3: syspage paddr
# R4: syspage vaddr
# R5: rtn dst offset
# R6: r/w offset
# R7: patch data
# R8: rtn src
#
# The patch data (in r7) is a pointer to the following
#
#	struct
#	{
#		paddr_t			*immr;
#		vector_tbl_t	reg_vectors; 
#	} *patch_data
#
# IMPORTANT: when calculating patch offsets best to compile and disassemble in
#			 order to make sure that assembly macros are expanded.
#			 for example 'loadi' translates into 2 assembly instructions
#
# devbase_patcher
#
# This patcher needs r3, r5 and r7 parameters
# The immr_paddr to be patched into DEVBASE is found at 0(r7). This address is
# patched into the callout at r3 + r5 
devbase_patcher:
	stwu	%r1, -32(%r1)
	mflr	%r0
	stw		%r31, 28(%r1)
	stw		%r0, 36(%r1)
	
	# r31 points to the start of the routine to be patched
	add		%r31, %r3, %r5
	
	# get the immr and map it
   	lwz		%r4, 0(%r7)
	loadi	%r3, 1024*1024
	bl		callout_io_map_indirect

	# patch the DEVBASE macro with the mapped address
	lhz		%r4, 6(%r31)
	lhz		%r5, 2(%r31)
	slwi	%r5, %r5, 16
	or		%r4, %r4, %r5
	add		%r3, %r3, %r4
	sth		%r3, 6(%r31)
	srwi	%r3, %r3, 16
	sth		%r3, 2(%r31)
	
	lwz		%r0, 36(%r1)
	mtlr	%r0
	lwz		%r31, 28(%r1)
	addi	%r1, %r1, 32
	blr
#
# id_patcher
#
# This patcher needs r3, r5 and r7 parameters
# The first patch done is for the maximum vector to be expected. This value is
# the sum of vector_tbl_t.num and vector_tbl_t.start (found at 4(r7) and 6(r7)).
# This value is also used for the third patch which normalizes the PIC vector value
# relative the the OS vector value with a subtraction
# The second patch is for the minimum vector expected, This is the vector_tbl_t.start
# value (6(r7))
# The second patch is the same as devbase_patcher
id_patcher:
	stwu	%r1, -32(%r1)
	mflr	%r0
	stw		%r31, 28(%r1)
	stw		%r0, 36(%r1)
	
	# r31 points to the start of the routine to be patched
	add		%r31, %r3, %r5
	
	# patch the max (start + num) expected vector check
	lhz		%r3, 4(%r7)
	lhz		%r4, 6(%r7)
	add		%r3, %r3, %r4
	sth		%r3, 42(%r31)
	# patch the min (start) expected vector check
	sth		%r4, 50(%r31)
	# patch the vector normalization value (start + num)
	neg		%r4, %r4
	sth		%r4, 54(%r31)

	# get the immr and map it
   	lwz		%r4, 0(%r7)
	loadi	%r3, 1024*1024
	bl		callout_io_map_indirect

	# patch the DEVBASE macro with the mapped address
	lhz		%r4, 6(%r31)
	lhz		%r5, 2(%r31)
	slwi	%r5, %r5, 16
	or		%r4, %r4, %r5
	add		%r3, %r3, %r4
	sth		%r3, 6(%r31)
	srwi	%r3, %r3, 16
	sth		%r3, 2(%r31)
	
	lwz		%r0, 36(%r1)
	mtlr	%r0
	lwz		%r31, 28(%r1)
	addi	%r1, %r1, 32
	blr
#
# mask_patcher
# 
# This patcher needs r3, r5, r6 and r7 parameters
# The first patch is for the maximum expected OS vector. The <intr> parameter passed
# in will always be normalized to the OS vector base for the callouts and so the
# minimum expected value is 0. We need only check for the maximum vector handled
# by this callout.
# The second patch done is to copy patch_data->reg_vectors.num register offsets
# from patch_data->reg_vectors.offset to the location pointed to by
# r6 + r3 + patch_data->reg_vectors.dst_tbl_off (ie. r/w offset location within
# the syspage). Because we can have only one vec_regoffset_table for all of the
# PIC register offsets but we may have multiple mask/unmask callouts that need
# patching, the patch_data->reg_vectors.dst_tbl_off gives the offset into the
# table to be used in calculating the destination of the copy.
# The third patch is for the offset into the vec_regoffset_table for this
# intrinfo_entry and is used for looking up the register offset based on the <intr>
# parameter
# The final patch is for the device base address. It is the same as devbase_patcher.
mask_patcher:
	stwu	%r1, -32(%r1)
	mflr	%r0
	stw		%r31, 28(%r1)
	stw		%r0, 36(%r1)
	
	# r31 points to the start of the routine to be patched
	add		%r31, %r3, %r5
	clrlwi	%r6, %r6, 16

	# patch the max expected vector check (patch_data->reg_vectors.num)
	lhz		%r8, 4(%r7)
	sth		%r8, 10(%r31)

	# copy patch_data->reg_vectors (12(r7)) to r/w offset (r3 + r6)
	# r4 will be used as the source (patch_data->reg_vectors)
	# r3 will be used as the destination (it is not required for any other patches)
	# r8 contains the number of 4 bytes words to copy
	# both r3 and r4 are setup to be predecremented by 4
	lwz		%r4, 12(%r7)
	addi	%r4, %r4, -4

	# setup the destination of the copy
	add		%r3, %r3, %r6
	lhz		%r5, 8(%r7)		/* dst_tbl_off (in vector entries) */
	slwi	%r5, %r5, 2		/* each vector table register offset is a uint32_t */
	add		%r3, %r3, %r5
	addi	%r3, %r3, -4
	
	# copy the vectors
	mtctr	%r8
loop:
	lwzu	%r8, 4(%r4)
	stwu	%r8, 4(%r3)
	bdnz	loop

	# now patch the vector table address (r6 + 8(r7))
	add		%r4, %r5, %r6
	sth		%r4, 22(%r31)

	# get the immr and map it
   	lwz		%r4, 0(%r7)
	loadi	%r3, 1024*1024
	bl		callout_io_map_indirect

	# patch the DEVBASE macro with the mapped address
	lhz		%r4, 6(%r31)
	lhz		%r5, 2(%r31)
	slwi	%r5, %r5, 16
	or		%r4, %r4, %r5
	add		%r3, %r3, %r4
	sth		%r3, 6(%r31)
	srwi	%r3, %r3, 16
	sth		%r3, 2(%r31)
	
	lwz		%r0, 36(%r1)
	mtlr	%r0
	lwz		%r31, 28(%r1)
	addi	%r1, %r1, 32
	blr

#
# config_patcher
# 
# This patcher needs r3, r5, r6 and r7 parameters
# The first patch is for the maximum expected OS vector. The <intr> parameter passed
# in will always be normalized to the OS vector base for the callouts and so the
# minimum expected value is 0. We need only check for the maximum vector handled
# by this callout.
# The second patch done is to copy patch_data->reg_vectors.num config flags from
# patch_data->reg_vectors.config_flags to the location pointed to by
# r6 + r3 + patch_data->reg_vectors.dst_tbl_off (ie. r/w offset location within
# the syspage). Because we can have only one vec_config_flags_table for all of
# the PIC register offsets but we may have multiple config callouts that need
# patching, the patch_data->reg_vectors.dst_tbl_off gives the offset into the
# table to be used in calculating the destination of the copy.
# The third patch is for the offset into the vec_config_flags_table for this
# intrinfo_entry and is used for looking up the flags offset based on the <intr>
# parameter
config_patcher:
	stwu	%r1, -32(%r1)
	mflr	%r0
	stw		%r31, 28(%r1)
	stw		%r0, 36(%r1)
	
	# r31 points to the start of the routine to be patched
	add		%r31, %r3, %r5
	clrlwi	%r6, %r6, 16

	# patch the max expected vector check (patch_data->reg_vectors.num)
	lhz		%r8, 4(%r7)
	sth		%r8, 6(%r31)

	# copy patch_data->reg_vectors.config_flags (16(r7)) to r/w offset (r3 + r6)
	# r4 will be used as the source (patch_data->reg_vectors.config_flags)
	# r3 will be used as the destination (it is not required for any other patches)
	# r8 contains the number of 4 bytes words to copy
	# both r3 and r4 are setup to be predecremented by 4
	lwz		%r4, 16(%r7)
	mr.		%r4, %r4	/* no copy if config_flags == NULL */
	beq		1f
	addi	%r4, %r4, -4

	# setup the destination of the copy
	add		%r3, %r3, %r6
	lhz		%r5, 8(%r7)		/* dst_tbl_off (in vector entries) */
	slwi	%r5, %r5, 2		/* each vector table register offset is a uint32_t */
	add		%r3, %r3, %r5
	addi	%r3, %r3, -4
	
	# copy the vectors
	mtctr	%r8
loop2:
	lwzu	%r8, 4(%r4)
	stwu	%r8, 4(%r3)
	bdnz	loop2

1:
	# now patch the vector table address (r6 + 8(r7))
	add		%r4, %r5, %r6
	sth		%r4, 18(%r31)

	lwz		%r0, 36(%r1)
	mtlr	%r0
	lwz		%r31, 28(%r1)
	addi	%r1, %r1, 32
	blr



.macro DEVBASE reg,dev_reg_off
	# the instruction immediate values are patched by the above routine
	lis		&reg,(&dev_reg_off) >> 16
	ori		&reg,&reg,(&dev_reg_off) & 0xffff
.endm


# 
# int interrupt_id(void)
#
# Returns: interrupt level
#
CALLOUT_START	interrupt_id_ppc85xx, 0, id_patcher
	DEVBASE	%r9, PPC85xx_CCSR_OFF_IACK

	mfmsr	%r12
	loadi	%r11, PPC_MSR_EE|PPC_MSR_CE|PPC_MSR_DR
	andc	%r11, %r12, %r11
	mtmsr	%r11
	isync

	# read the IACK register (get the PIC vector)
	lwz		%r14, 0(%r9)
	extsh	%r14, %r14
	cmplwi	%r14, 0x1234	/* patched to max expected PIC vector */
	bge		1f
	cmplwi	%r14, 0x1234		/* patched with the min expected PIC vector */
	addi	%r14, %r14, 0		/* patched to normalize the PIC vector */
	bge		2f
1:
	# Invalid vector. EOI the interrupt so it gets re-asserted and can be picked
	# up by the next handler. If we don't EOI, then the next handler will read
	# the spurious vector value from the IACK register instead of the vector
	# last read.
	loadi	%r0, 0
	stw		%r0, (PPC85xx_CCSR_OFF_EOI - PPC85xx_CCSR_OFF_IACK)(%r9)
	li 		%r14, -1
2:
	mtmsr	%r12
	isync
CALLOUT_END	interrupt_id_ppc85xx
		
# 
# void interrupt_eoi(struct syspage_entry *sysp, int intr)
#
# EOI the interrupt
#
CALLOUT_START	interrupt_eoi_ppc85xx, 0, devbase_patcher
	DEVBASE	%r9, PPC85xx_CCSR_OFF_EOI
	loadi	%r0, 0
	mfmsr	%r12
	loadi	%r11, PPC_MSR_EE|PPC_MSR_CE|PPC_MSR_DR
	andc	%r11, %r12, %r11
	mtmsr	%r11
	isync
	stw		%r0, 0(%r9)
	mtmsr	%r12
	isync
CALLOUT_END	interrupt_eoi_ppc85xx

#
# int interrupt_unmask(struct syspage_entry *sysp, int intr)
#
# Unmask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_unmask_ppc85xx, vec_regoffset_table_size, mask_patcher
	DEVBASE	%r5, 0

	cmplwi	%r4, 1234	/* patched with the max expected vector */
	bge		1f

	/* register offset corresponding to the vector to unmask. The vector offset
	 * table is found at the patched offset from syspage_entry (r3) */
	slwi	%r4, %r4, 2			/* vector offsets are 4 bytes */
	addi	%r3, %r3, 0x1234	/* patched with the r/w offset */
	lwzx	%r6, %r3, %r4
	
	/* if the offset (r6) == -1, then ignore */
	addic.	%r3, %r6, 1
	beq		1f

	mfmsr	%r12
	loadi	%r11, PPC_MSR_EE|PPC_MSR_CE|PPC_MSR_DR
	andc	%r11, %r12, %r11
	mtmsr	%r11
	isync
	
	/* clear the mask bit */
	lwzx	%r0, %r5, %r6
	bitclr	%r0, %r0, 0x80000000
	stwx	%r0, %r5, %r6

	mtmsr	%r12
	isync

	li		%r3, 3
	blr
1:
	li		%r3, -1
	blr
CALLOUT_END	interrupt_unmask_ppc85xx
	
#
# void interrupt_mask(struct syspage_entry *sysp, int intr)
#
# Mask the passed hardware interrupt number.
#
#
CALLOUT_START	interrupt_mask_ppc85xx, vec_regoffset_table_size, mask_patcher
	DEVBASE	%r5, 0
	
	cmplwi	%r4, 0x1234	/* patched with the max expected vector */
	bge		1f

	/* register offset corresponding to the vector to unmask. The vector offset
	 * table is found at the patched offset from syspage_entry (r3) */
	slwi	%r4, %r4, 2			/* vector offsets are 4 bytes */
	addi	%r3, %r3, 0x1234	/* patched with the r/w offset */
	lwzx	%r6, %r3, %r4

	/* if the offset (r6) == -1, then ignore */
	addic.	%r3, %r6, 1
	beq		1f

	mfmsr	%r12
	loadi	%r11, PPC_MSR_EE|PPC_MSR_CE|PPC_MSR_DR
	andc	%r11, %r12, %r11
	mtmsr	%r11
	isync
	
	/* clear the mask bit */
	lwzx	%r0, %r5, %r6
	bitset	%r0, %r0, 0x80000000
	stwx	%r0, %r5, %r6

	mtmsr	%r12
	isync

	li		%r3,3
	blr
1:
	li		%r3, -1
	blr
CALLOUT_END	interrupt_mask_ppc85xx

#
# int interrupt_config(volatile struct syspage_entry *syspage, struct intrinfo_entry *info, int intr)
#
# Return special config for the passed hardware interrupt number.
#
CALLOUT_START   interrupt_config_ppc85xx, vec_config_flags_table_size, config_patcher
	li		%r6, 0
	cmplwi	%r5, 0x1234	/* patched with the max expected vector */
	bge		1f

	/* register offset corresponding to the vector to config. The vector offset
	 * table is found at the patched offset from syspage_entry (r3) */
	slwi	%r5, %r5, 2			/* vector flags are 4 bytes */
	addi	%r3, %r3, 0x1234	/* patched with the r/w offset */
	lwzx	%r6, %r3, %r5
1:
	mr		%r3, %r6
    blr
CALLOUT_END interrupt_config_ppc85xx

