#
# Copyright 2007, 2008, QNX Software Systems.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not reproduce, modify or distribute this software except in
# compliance with the License. You may obtain a copy of the License
# at: http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTIES OF ANY KIND, either express or implied.
#
# This file may contain contributions from others, either as
# contributors under the License or as licensors under other terms.
# Please review this entire file for other proprietary rights or license
# notices, as well as the QNX Development Suite License Guide at
# http://licensing.qnx.com/license-guide/ for other information.
#

#

#

#
#
# An entry point for a hardware kernel callout.
# It may be called by an interrupt handler.
# This code MUST be position independant.
#

	.include "callout.ah"

#
# Interrupts are always off in this code.
#
# Note that interrupt_id_* and interrupt_eoi_* are not actually
# called by the kernel. For performance reasons, they are instead
# copied into place and intermixed with other kernel code. As such,
# they don't follow normal calling conventions. They should just fall out
# their bottoms when done rather than attempting to do a return
# instruction. The 'id' routine should return the interrupt level in
# the r14 register. The 'eoi' routine will find it still there.
# By turning on various INTR_GENFLAG_* bits in the intrinfo_entry,
# the following values can be loaded before entry to this code.
#
#		cpu number			=> r15
#		syspage_ptr			=> r16
#		intrinfo_entry		=> r17
#		intr mask count		=> r18 (only available in EOI routine)
#
# If the INTR_FLAG_CPU_FAULT bit is on, the eoi routine should set
# the R3 register to the following value to indicate the exception
# that has occurred:
# 		R3 = (fault_number << 16) | (signal_code << 8) | (signal_number)
# Set R3 to zero if no exception has occured. If the exception is memory
# related, set R4 to referenced address that caused the exception, zero
# if there is no address.
#

#
# int interrupt_id(void)
#
# Returns: interrupt level, masks it off
#
CALLOUT_START	interrupt_id_booke_dec, 0, 0
	# Clear the interrupt indicator
	loadi	%r0,PPCBKE_TSR_DIS
	mtspr	PPCBKE_SPR_TSR,%r0
	# Mask the interrupt (clear TCR.DIE)
	loadi	%r3,PPCBKE_TCR_DIE
	mfspr	%r0,PPCBKE_SPR_TCR
	andc	%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r14,0
CALLOUT_END		interrupt_id_booke_dec

#
# void interrupt_eoi(struct syspage_entry *sysp, int level)
#
# Send an end-of-interrupt, then unmask the level (interrupt_id masked it)
#
CALLOUT_START	interrupt_eoi_booke_dec, 0, 0
	# unmask if mask count (in r18) is zero
	cmplwi	%r18,0
	bgt		1f
	loadi	%r3,PPCBKE_TCR_DIE
	mfspr	%r0,PPCBKE_SPR_TCR
	or		%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
1:
CALLOUT_END		interrupt_eoi_booke_dec

#
# int interrupt_unmask(struct syspage_entry *sysp, int level)
#
# Unmask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_unmask_booke_dec, 0, 0
	loadi	%r3,PPCBKE_TCR_DIE
	mfspr	%r0,PPCBKE_SPR_TCR
	or		%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r3,0
	blr
CALLOUT_END		interrupt_unmask_booke_dec

#
# int interrupt_mask(struct syspage_entry *sysp, int level)
#
# Mask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_mask_booke_dec, 0, 0
	loadi	%r3,PPCBKE_TCR_DIE
	mfspr	%r0,PPCBKE_SPR_TCR
	andc	%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r3,0
	blr
CALLOUT_END		interrupt_mask_booke_dec

#
# int interrupt_id(void)
#
# Returns: interrupt level, masks it off
#
CALLOUT_START	interrupt_id_booke_fit, 0, 0
	# Clear the interrupt indicator
	loadi	%r0,PPCBKE_TSR_FIS
	mtspr	PPCBKE_SPR_TSR,%r0
	# Mask the interrupt (clear TCR.FIE)
	loadi	%r3,PPCBKE_TCR_FIE
	mfspr	%r0,PPCBKE_SPR_TCR
	andc	%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r14,0
CALLOUT_END		interrupt_id_booke_fit

#
# void interrupt_eoi(struct syspage_entry *sysp, int level)
#
# Send an end-of-interrupt, then unmask the level (interrupt_id masked it)
#
CALLOUT_START	interrupt_eoi_booke_fit, 0, 0
	# unmask if mask count (in r18) is zero
	cmplwi	%r18,0
	bgt		1f
	loadi	%r3,PPCBKE_TCR_FIE
	mfspr	%r0,PPCBKE_SPR_TCR
	or		%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
1:
CALLOUT_END		interrupt_eoi_booke_fit

#
# int interrupt_unmask(struct syspage_entry *sysp, int level)
#
# Unmask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_unmask_booke_fit, 0, 0
	loadi	%r3,PPCBKE_TCR_FIE
	mfspr	%r0,PPCBKE_SPR_TCR
	or		%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r3,0
	blr
CALLOUT_END		interrupt_unmask_booke_fit

#
# int interrupt_mask(struct syspage_entry *sysp, int level)
#
# Mask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_mask_booke_fit, 0, 0
	loadi	%r3,PPCBKE_TCR_FIE
	mfspr	%r0,PPCBKE_SPR_TCR
	andc	%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r3,0
	blr
CALLOUT_END		interrupt_mask_booke_fit

#
# int interrupt_id(void)
#
# Returns: interrupt level, masks it off
#
CALLOUT_START	interrupt_id_booke_watchdog, 0, 0
	# Clear the interrupt indicator
	loadi	%r0,PPCBKE_TSR_WIS
	mtspr	PPCBKE_SPR_TSR,%r0
	# Mask the interrupt (clear TCR.WIE)
	loadi	%r3,PPCBKE_TCR_WIE
	mfspr	%r0,PPCBKE_SPR_TCR
	andc	%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r14,0
CALLOUT_END		interrupt_id_booke_watchdog

#
# void interrupt_eoi(struct syspage_entry *sysp, int level)
#
# Send an end-of-interrupt, then unmask the level (interrupt_id masked it)
#
CALLOUT_START	interrupt_eoi_booke_watchdog, 0, 0
	# unmask if mask count (in r18) is zero
	cmplwi	%r18,0
	bgt		1f
	loadi	%r3,PPCBKE_TCR_WIE
	mfspr	%r0,PPCBKE_SPR_TCR
	or		%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
1:
CALLOUT_END		interrupt_eoi_booke_watchdog

#
# int interrupt_unmask(struct syspage_entry *sysp, int level)
#
# Unmask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_unmask_booke_watchdog, 0, 0
	loadi	%r3,PPCBKE_TCR_WIE
	mfspr	%r0,PPCBKE_SPR_TCR
	or		%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r3,0
	blr
CALLOUT_END		interrupt_unmask_booke_watchdog

#
# int interrupt_mask(struct syspage_entry *sysp, int level)
#
# Mask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_mask_booke_watchdog, 0, 0
	loadi	%r3,PPCBKE_TCR_WIE
	mfspr	%r0,PPCBKE_SPR_TCR
	andc	%r0,%r0,%r3
	mtspr	PPCBKE_SPR_TCR,%r0
	li		%r3,0
	blr
CALLOUT_END		interrupt_mask_booke_watchdog
