#
# Copyright 2007, 2008, QNX Software Systems. 
# 
# Licensed under the Apache License, Version 2.0 (the "License"). You 
# may not reproduce, modify or distribute this software except in 
# compliance with the License. You may obtain a copy of the License 
# at: http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OF ANY KIND, either express or implied.
#
# This file may contain contributions from others, either as 
# contributors under the License or as licensors under other terms.  
# Please review this entire file for other proprietary rights or license 
# notices, as well as the QNX Development Suite License Guide at 
# http://licensing.qnx.com/license-guide/ for other information.
#

#

# 

#
#
# An entry point for a hardware kernel callout.
# It may be called by an interrupt handler.
# This code MUST be position independant.
#

	.include "callout.ah"

.global	dec_mask_slot	
dec_mask_slot: 
		.long	4	

.global patch_dec_intr		
patch_dec_intr:
	    add     %r3,%r3,%r5
		sth     %r6,2(%r3)
		blr

	
#
# Interrupts are always off in this code.
#
# Note that interrupt_id_* and interrupt_eoi_* are not actually
# called by the kernel. For performance reasons, they are instead
# copied into place and intermixed with other kernel code. As such,
# they don't follow normal calling conventions. They should just fall out 
# their bottoms when done rather than attempting to do a return 
# instruction. The 'id' routine should return the interrupt level in
# the r14 register. The 'eoi' routine will find it still there. 
# By turning on various INTR_GENFLAG_* bits in the intrinfo_entry,
# the following values can be loaded before entry to this code.
# 
#		cpu number			=> r15
#		syspage_ptr			=> r16
#		intrinfo_entry		=> r17
#		intr mask count		=> r18 (only available in EOI routine)
#
# If the INTR_FLAG_CPU_FAULT bit is on, the eoi routine should set
# the R3 register to the following value to indicate the exception
# that has occurred:
# 		R3 = (fault_number << 16) | (signal_code << 8) | (signal_number)
# Set R3 to zero if no exception has occured. If the exception is memory
# related, set R4 to referenced address that caused the exception, zero
# if there is no address.
#

# 
# int interrupt_id(void)
#
# Returns: interrupt level, masks it off
#

#	Since we can't individually mask the PPC decrementer on the 600 family,
#	it's possible that overly long timer tick processing, or having interrupts
#	disabled for a long period of time might cause us to get a nested timer
#	tick. 
#	We use a variable to tell if we're still handling a previous decrementer
#	interrupt when a new one comes in. If so, the interrupt_id returns -1 to
#	cause the kernel to just skip processing of this interrupt. 

CALLOUT_START	interrupt_id_dec, dec_mask_slot, patch_dec_intr
		la		%r3,0x1234(%r16)	# get R/W base (patched)
		lwz		%r4,0(%r3)
		cmplwi	%r4,0
		beq+	1f
		li		%r14,-1
		b		2f
1:		
		li		%r14,0
		li		%r4,1
		stw		%r4,0(%r3)
2:
CALLOUT_END		interrupt_id_dec
		
# 
# void interrupt_eoi(struct syspage_entry *sysp, int level)
#
# Send an end-of-interrupt, then unmask the level (interrupt_id masked it)
#
CALLOUT_START	interrupt_eoi_dec, dec_mask_slot, patch_dec_intr
		la		%r3,0x1234(%r16)	# get R/W base (patched)
		li		%r4,0
		stw		%r4,0(%r3)
CALLOUT_END		interrupt_eoi_dec
		
#
# int interrupt_unmask(struct syspage_entry *sysp, int level)
#
# Unmask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_unmask_dec, 0, 0
		li		%r3, 0
		blr
CALLOUT_END		interrupt_unmask_dec
		
#
# int interrupt_mask(struct syspage_entry *sysp, int level)
#
# Mask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_mask_dec, 0, 0
		li		%r3,22	# return EINVAL
		blr
CALLOUT_END		interrupt_mask_dec
