/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */



/******************************************************************************
*
*       COPYRIGHT (c) 2002 MOTOROLA INC.
*       ALL RIGHTS RESERVED
*
*       This code is the property of Motorola, Inc. and is
*       Confidential Proprietary Information.
*
*       The copyright notice above does not evidence any
*       actual or intended publication of such source code.
*
* Filename:     $Source$
* Author:       $Author: builder@qnx.com $
* Locker:       $Locker$
* State:        $State$
* Revision:     $Revision: 580669 $
*
* Functions:	
*
* History:      Use the RCS command rlog to display revision history
*               information.
*
* Description:	Kernel interrupt handler callouts
*				
*
* Notes:		
*
*	Interrupts are always off in this code.
*
*	interrupt_id_* and interrupt_eoi_* are not actually
* 	called by the kernel. For performance reasons, they are instead
* 	copied into place and intermixed with other kernel code. As such,
* 	they don't follow normal calling conventions. They should just fall out 
* 	their bottoms when done rather than attempting to do a return 
* 	instruction. The 'id' routine should return the interrupt level in
* 	the r14 register. The 'eoi' routine will find it still there. 
* 	By turning on various INTR_GENFLAG_* bits in the intrinfo_entry,
*	the following values can be loaded before entry to this code.
*  
* 		syspage_ptr			=> r16
* 		intrinfo_entry		=> r17
* 		intr mask count		=> r18 (only available in EOI routine)
*
*****************************************************************************/

/*
 * An entry point for a hardware kernel callout.
 * It may be called by an interrupt handler.
 * This code MUST be position independant.
 */

.include "callout.ah"

/*
 * int interrupt_unmask(struct syspage_entry *sysp, int intr)
 *
 * Unmask the passed hardware interrupt number.
 */
.set MGT5200_PIMR,            0x0500
.set MGT5200_PP_HLSR_1,       0x0504
.set MGT5200_PP_HLSR_2,       0x0508
.set MGT5200_PP_HLSR_3,       0x050C
.set MGT5200_EE_ETR,          0x0510
.set MGT5200_CP_MIMR,         0x0514
.set MGT5200_MIP_SMI_1,       0x0518
.set MGT5200_MIP_SMI_2,       0x051C
.set MGT5200_ALL_STAT_ENC,    0x0524
.set MGT5200_CISR,            0x0528
.set MGT5200_MISR,            0x052C
.set MGT5200_PISR,            0x0530

.set MGT5200_GPIOSIE,		  0x0B20
.set MGT5200_GPIOSIIE,		  0x0B30
.set MGT5200_GPIOSSR,		  0x0B3C
.set MGT5200_GPIOWIE,		  0x0C14
.set MGT5200_GPIOWSR,		  0x0C24

.set MGT5200_SCIMR,           0x1218
.set MGT5200_SCIPR,           0x1214

.set MGT5200_SLC_0_TERM,	  0x0700
.set MGT5200_SLC_0_CTRL,	  0x0704
.set MGT5200_SLC_0_STAT,	  0x070c

.set MGT5200_SCTMR,			  0x0400
.set MGT5200_SCTSR,			  0x0400

patcher:
	# R3: syspage paddr
	# R4: syspage vaddr
	# R5: rtn dst offset
	# R6: r/w offset
	# R7: patch data
	# R8: rtn src

	stwu    %r1,-32(%r1)
	mflr    %r0
	stw     %r31,28(%r1)
	stw     %r0,36(%r1)

	add     %r31,%r3,%r5

	mr      %r4,%r7
	li		%r3, 0x08
	bl      callout_io_map_indirect

	sth     %r3,6(%r31)
	srwi    %r3,%r3,16
	sth     %r3,2(%r31)

	lwz     %r0,36(%r1)
	mtlr    %r0
	lwz     %r31,28(%r1)
	addi    %r1,%r1,32
	blr


.macro DEVBASE reg,dev_reg_off
# the instruction immediate values are patched by the above routine
	lis     &reg,(&dev_reg_off) >> 16
	ori     &reg,&reg,(&dev_reg_off) & 0xffff
.endm

/***************************************************************************
 * Callout : 	interrupt_id_mgt5200
 *
 * Description: Determine critical or main interrupt level and mask it off
 *				(if appropriate)
 *
 * On Entry:	Nothing
 *
 * On Exit:		r14 = interrupt level
 *
 * Notes:
 *
 *	Test for core critical interrupts first: 
 * 	If the interrupt is 0, (UART1_4), mask it by clearing bit 20 of EE_ETR.
 * 	If the interrupt is 1 (slice timer 0), then we mask it by clearing the
 * 	interrupt enable bit of Slice Timer 0 Control Register. If it's 2, it's 
 * 	a cascade from some peripheral that's programmed to generate a HI int,
 * 	and we can't mask it directly, but it'll be taken care of by the
 *	peripheral callouts.
 *
 **************************************************************************/
CALLOUT_START	interrupt_id_mgt5200, 0, patcher
	DEVBASE %r6, 0x12345678
	li		%r14,0

	/* get all interrupt status, encoded */
	lwz		%r5,MGT5200_ALL_STAT_ENC(%r6)		

	li		%r0, 0x0400		/* Critical Status flag */
	and		%r0, %r0, %r5	/* clear all bits except status */

	cmplwi	%r0,0
	beq		4f				/* no critical interrupt, proceed to main interrupt id */

	/* bits 22 and 23 represent the level. Load it into r14. */

	srawi	%r14, %r5, 8
	andi.	%r14, %r14, 3

	cmplwi	%r14,0
	bgt		1f

	/* interrupt is 0, clear bit 20 of EE_ETR */
    lwz     %r5,MGT5200_EE_ETR(%r6)

    li      %r4,0x800               /* bit 20 is IRQ 0 enable bit */
    andc    %r5,%r5,%r4             /* mask (clear) bit by ANDing with one's compliment of r4 */
    stw     %r5, MGT5200_EE_ETR(%r6)             /* store new value */
	b		6f	/* done */
1:
	cmplwi	%r14,1
	bgt		3f		 

	/* interrupt is 1, mask slice timer 0 interrupts */
    lwz     %r5,MGT5200_SLC_0_CTRL(%r6)                 
	lis		%r0,0x0200
	andc	%r5, %r5, %r0
	stw		%r5,MGT5200_SLC_0_CTRL(%r6)

    lwz     %r5,MGT5200_SLC_0_STAT(%r6)

	b		6f	/* done */

2:
    cmplwi  %r14,2
    bgt     3f

    /* For now, we'll pretend that this is a LO peripheral interrupt, since there seems to
     * be a problem cascading from the HI source.
     */
    li      %r14,4  /* LO int is main level 4 (QNX vector 8) */
    b       5f
	
3:
	b		6f		/* placeholder for CCS module wakeup from sleep. */

4:	
	/* main interrupt ID - r5 still contains Interrupt Status Register (encoded) */
    /* bits 15 - 11 represent the level. Load it into r14. */

	li	%r14, -1 
	
    srawi   %r8, %r5, 16
    andi.   %r8, %r8, 0x3f
    
    cmplwi	%r8,0	
	beq		6f
	andi.	%r14, %r8, 0x1f
	
5:
   /*
    * now, load the main interrupt mask register, and set the appropriate bit
    * to mask the interrupt
    *
    * Note: May want to prevent LO-int from being masked here and leave it to the peripheral
    *       handler to mask only the peripheral causing the LO-int.
    */
   lwz     %r5,MGT5200_CP_MIMR(%r6)
   lis     %r4, 1              /* puts 0x00010000 into r4 (bit 15) */
   srw     %r4, %r4, %r14      /* shift the bit to the proper position */
   or      %r5, %r5, %r4
   stw     %r5, MGT5200_CP_MIMR(%r6)         /* write new mask value */

    /*
     * Master interrupts start at offset 4 from the base of the critical interrupts, but
     * the value in r14 represents the relative master interrupt, so we need to compensate
     */
    addi    %r14, %r14, 4	/* done */

6:
CALLOUT_END	interrupt_id_mgt5200

/***************************************************************************
 * Callout : 	interrupt_eoi_mgt5200
 *
 * Description: Clear the interrupt, then unmask the level (it was masked in
 *				interrupt_id)
 *
 * On Entry:	r14 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_eoi_mgt5200, 0, patcher

/*
 * r14 = 0 - write a 1 to bit 20 of EE_ETR, to unmask
 * r14 = 1 - write a 1 to slice timer 0 status bit to clear, and enable timer interrupts
 * r14 = 2 or 3 - do nothing for now
 * r14 = 4 - 20 - unmask the appropriate main interrupt
 */
	DEVBASE %r6, 0x12345678

	cmplwi	%r18,0
	bgt		3f

	cmplwi	%r14,0
	bgt		1f
	/* UART 1_4 interrupt */

  	lwz     %r5,MGT5200_EE_ETR(%r6)

    ori     %r5, %r5, 0x800 /* unmask interrupt 0 (bit 20) */
	stw		%r5,MGT5200_EE_ETR(%r6)
    b       3f	/* done */

1:
	cmplwi	%r14, 1
	bgt		2f
	/* slice timer 0 interrupt */

    lwz     %r5,MGT5200_SLC_0_STAT(%r6)

/* clear int */
	lis		%r0,0x0100
	stw		%r0,MGT5200_SLC_0_STAT(%r6)

/* unmask the interrupt */
	lis		%r0,0x0700
	stw		%r0,MGT5200_SLC_0_CTRL(%r6)
	b		3f	/* done */

2:
	cmplwi	%r14,3
	ble		3f		/* if it's 2 or 3, do nothing */

	/* main interrupt */
	cmplwi	%r14, 20
	bgt		3f

    lwz         %r5,MGT5200_CP_MIMR(%r6)

    lis         %r4, 0x10 /* int 4 == main interrupt 0, this is so srw by r14 puts bit in proper position */
    srw         %r4, %r4, %r14  /* r14 will be 4 or greater */
    andc        %r5, %r5, %r4   /* clear bit to unmask */
    stw         %r5, MGT5200_CP_MIMR(%r6)
3:
/* done */
CALLOUT_END	interrupt_eoi_mgt5200

/***************************************************************************
 * Callout : 	interrupt_mask_mgt5200
 *
 * Description: Mask the given interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_mask_mgt5200, 0, patcher
/*
 * If the level passed here in r4 is 0, unmask external INT 0 by setting bit 20
 * of the External Enable and External Types register.
 * If it's 1, enable slice timer 0 interrupts.
 * Otherwise, load the Critical Priority and Main Interrupt Mask register,
 * and clear the appropriate bit for the level to be unmasked.
 */
 	DEVBASE %r6, 0x12345678
 	
	cmplwi	%r4, 0
	bgt		1f
	lwz %r5, MGT5200_EE_ETR(%r6)                 /* load contents of EE_ETR into r5 */
	li      %r8,0x800				/* bit 20 is IRQ 0 enable bit */
	andc    %r5,%r5,%r8             /* mask  (clear) bit by ANDing with one's compliment of r6 */
	stw     %r5, MGT5200_EE_ETR(%r6)             /* store new value */
	b		3f
1:
	cmplwi	%r4,1
	bgt		2f

    /* interrupt is 1, mask slice timer 0 interrupts */
    lwz     %r5,MGT5200_SLC_0_CTRL(%r6)
    lis     %r0,0x0200
    andc    %r5, %r5, %r0
    stw     %r5,MGT5200_SLC_0_CTRL(%r6)
	b		3f

2:
	cmplwi	%r4, 3					/* check for level 2 or 3 */
	ble		3f						/* can't mask */

	/* set bit to mask in MIMR */
	lwz %r5, MGT5200_CP_MIMR(%r6)                 /* load contents of CP_MIMR into r5 */
									/* bits 15 - 31 represent interrupt levels 4 - 20, respectively */
	lis		%r8, 0x10				/* bit 11 */
	srw     %r8,%r8,%r4             /* shift mask bit to proper position in r6 */
	or      %r5,%r5,%r8             
	eieio
	stw     %r5, MGT5200_CP_MIMR(%r6)             /* load new interrupt mask */
3:
	sync
	blr
CALLOUT_END	interrupt_mask_mgt5200

	
/***************************************************************************
 * Callout : 	interrupt_unmask_mgt5200
 *
 * Description: Unmask the given interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_unmask_mgt5200, 0, patcher

/*
 * If the level passed here is 0, unmask external INT 0 by setting bit 20
 * of the External Enable and External Types register.
 * If it's 1, enable slice timer 0 interrupts.
 * Otherwise, load the Critical Priority and Main Interrupt Mask register,
 * and clear the appropriate bit for the level to be unmasked.
 */
 	DEVBASE %r6, 0x12345678
 	
	cmplwi	%r4, 0
	bgt		1f
	lwz %r5, MGT5200_EE_ETR(%r6)                 /* load contents of EE_ETR into r5 */
	li      %r8,0x800				/* bit 20 is IRQ 0 enable bit */
	or      %r5,%r5,%r8             /* unmask bit */
	stw     %r5, MGT5200_EE_ETR(%r6)             /* store new value */
	b		3f
1:
	cmplwi	%r4, 1
	bgt		2f

/* unmask slice timer 0 interrupt */
    lis     %r0,0x0700
    stw     %r0,MGT5200_SLC_0_CTRL(%r6)
	b		3f


2:
	cmplwi	%r4, 3					/* check for level 2 or 3 */
	ble		3f						/* can't unmask */

	/* unmask a main interrupt - clear bit to unmask in MIMR */
	lwz %r5, MGT5200_CP_MIMR(%r6)                 /* load contents of CP_MIMR into r5 */
									/* bits 15 - 31 represent interrupt levels 4 - 20, respectively */
	lis		%r8, 0x10				/* bit 11 */
	srw     %r8,%r8,%r4             /* shift mask bit to proper position in r6 */
	andc    %r5,%r5,%r8             /* and r5 with one's compliment of r6 to unmask interrupt */
	eieio
	stw     %r5, MGT5200_CP_MIMR(%r6)             /* load new interrupt mask */
3:
	sync
	blr
CALLOUT_END	interrupt_unmask_mgt5200

/***************************************************************************
 *
 * Peripheral Interrupt Callouts
 *
 ***************************************************************************/

/***************************************************************************
 * Callout : 	interrupt_unmask_mgt5200_per_hi
 *
 * Description: Unmask the given peripheral interrupt level for interrupts 
 *              configured as HI
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_unmask_mgt5200_per_hi, 0, patcher

/*
 * Setting a bit in the Peripheral Interrupt Mask Register masks the interrupt
 * Clearing the bit unmasks.
 */
 	DEVBASE %r6, 0x12345678
 	
	lwz %r5, MGT5200_PIMR(%r6)                 /* load contents of Peripheral Interrupt Mask Register into r5 */
	lis     %r8,0x8000				
	srw		%r8, %r8, %r4			/* shift mask bit to proper position */
	andc    %r5,%r5,%r8             /* unmask bit */
	stw     %r5, MGT5200_PIMR(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_unmask_mgt5200_per_hi

/***************************************************************************
 * Callout : 	interrupt_mask_mgt5200_per_hi
 *
 * Description: Mask the given peripheral interrupt level for HI ints
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_mask_mgt5200_per_hi, 0, patcher

/*
 * Setting a bit in the Peripheral Interrupt Mask Register masks the interrupt
 * Clearing the bit unmasks.
 */
 	DEVBASE %r6, 0x12345678
 	
	lwz %r5, MGT5200_PIMR(%r6)                 /* load contents of Peripheral Interrupt Mask Register into r5 */
	lis     %r8,0x8000				
	srw		%r8, %r8, %r4			/* shift mask bit to proper position */
	or      %r5,%r5,%r8             /* mask bit */
	stw     %r5, MGT5200_PIMR(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_mask_mgt5200_per_hi


/***************************************************************************
 * Callout : 	interrupt_id_mgt5200_per_hi
 *
 * Description: Determine the HI peripheral interrupt level and mask it off.
 *
 * On Entry:	Nothing
 *
 * On Exit:		r14 = peripheral interrupt level
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_id_mgt5200_per_hi, 0, patcher

/*
 *  Make a mask out of the bits in each of the three HLSR registers that
 *  define whether a peripheral interrupt is HI or LO, and mask off all
 *  that are programmed LO 
 */
	DEVBASE %r6, 0x12345678

	li      %r14, -1

	lwz		%r5,MGT5200_ALL_STAT_ENC(%r6)

	lis     %r4, 0x2000     /* Peripheral Status flag */
	and     %r4, %r4, %r5   /* clear all bits except status */
	cmplwi  %r4,0
	beq     1f              /* No peripheral interrupt, return -1 */

	andi.   %r4,%r5,0x300   /* check if HI_int is the source */
	cmplwi  %r4,0x200
	bne     1f              /* not a HI int */

	andis.  %r5,%r5,0x1f00  /* Mask and shift PerStat field */
	li      %r4,24
	srw     %r14,%r5,%r4

	lwz     %r4,MGT5200_PIMR(%r6)   /* put Peripheral Interrupt Mask Register in r4 */

	ori     %r4, %r4, 0xff          /* set reserved bits to make them appear masked */

	lis     %r5,0x8000
	srw     %r5,%r5,%r14            /* shift mask bit right to position of interrupt we wish to mask */
	or      %r4,%r4,%r5             /* set relevant bit to mask it */
	eieio
	stw     %r4, MGT5200_PIMR(%r6)             /* load new enable mask into PIMR */
	sync
1:	
CALLOUT_END	interrupt_id_mgt5200_per_hi

/***************************************************************************
 * Callout : 	interrupt_eoi_mgt5200_per_hi
 *
 * Description: Clear the interrupt, then unmask the level (it was masked in
 *				interrupt_id)
 *
 * On Entry:	r14 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_eoi_mgt5200_per_hi, 0, patcher

/*
 * All peripheral interrupts are level triggered, active high. When the ISR
 * clears the interrupt at the source, the level goes low again, and the
 * updated status is reflected in the PISR. So, it's not necessary to clear
 * the interrupt here, we just have to unmask it.
 */
	DEVBASE %r6, 0x12345678 
	
	cmplwi	%r18, 0
	bgt		1f

    lwz     %r5,MGT5200_PIMR(%r6)                  /* put Peripheral Interrupt Mask Register in r5 */

	lis		%r4, 0x8000
	srw		%r4, %r4, %r14				/* shift the bit to the proper position */
	andc	%r5, %r5, %r4				/* unmask by clearing the appropriate bit */
    stw     %r5, MGT5200_PIMR(%r6)
1:	
CALLOUT_END		interrupt_eoi_mgt5200_per_hi

/***************************************************************************
 * Callout : 	interrupt_unmask_mgt5200_per_lo
 *
 * Description: Unmask the given peripheral interrupt level, for LO interrupts
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_unmask_mgt5200_per_lo, 0, patcher

/*
 * Setting a bit in the Peripheral Interrupt Mask Register masks the interrupt
 * Clearing the bit unmasks.
 */
 	DEVBASE %r6, 0x12345678 
 	
	lwz %r5, MGT5200_PIMR(%r6)                 /* load contents of Peripheral Interrupt Mask Register into r5 */
	lis     %r7,0x8000				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	andc    %r5,%r5,%r7             /* unmask bit */
	stw     %r5, MGT5200_PIMR(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_unmask_mgt5200_per_lo

/***************************************************************************
 * Callout : 	interrupt_mask_mgt5200_per_lo
 *
 * Description: Mask the given peripheral interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_mask_mgt5200_per_lo, 0, patcher

/*
 * Setting a bit in the Peripheral Interrupt Mask Register masks the interrupt
 * Clearing the bit unmasks.
 */
 	DEVBASE %r6, 0x12345678
 	
	lwz %r5, MGT5200_PIMR(%r6)                 /* load contents of Peripheral Interrupt Mask Register into r5 */
	lis     %r7,0x8000				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	or      %r5,%r5,%r7             /* mask bit */
	stw     %r5, MGT5200_PIMR(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_mask_mgt5200_per_lo


/***************************************************************************
 * Callout : 	interrupt_id_mgt5200_per_lo
 *
 * Description: Determine the peripheral interrupt level and mask it off.
 *
 * On Entry:	Nothing
 *
 * On Exit:		r14 = peripheral interrupt level
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_id_mgt5200_per_lo, 0, patcher

/*
 *  Make a mask out of the bits in each of the three HLSR registers that
 *  defines whether a peripheral interrupt is HI or LO, and mask off all
 *  that are programmed HI 
 */
	DEVBASE %r6, 0x12345678 

	li      %r14, -1

	lwz		%r5,MGT5200_ALL_STAT_ENC(%r6)

	lis     %r4, 0x2000     /* Peripheral Status flag */
	and     %r4, %r4, %r5   /* clear all bits except status */
	cmplwi  %r4,0
	beq     1f              /* No peripheral interrupt, return -1 */

	andi.   %r4,%r5,0x300   /* check if HI_int is the source */
	cmplwi  %r4,0x200
	beq     1f              /* this is a HI int */

	andis.  %r5,%r5,0x1f00  /* Mask and shift PerStat field */
	li      %r4,24
	srw     %r14,%r5,%r4

	lwz     %r4,MGT5200_PIMR(%r6)   /* put Peripheral Interrupt Mask Register in r4 */

	ori     %r4, %r4, 0xff          /* set reserved bits to make them appear masked */

	lis     %r5,0x8000
	srw     %r5,%r5,%r14            /* shift mask bit right to position of interrupt we wish to mask */
	or      %r4,%r4,%r5             /* set relevant bit to mask it */
	eieio
	stw     %r4, MGT5200_PIMR(%r6)             /* load new enable mask into PIMR */
	sync
1:	
CALLOUT_END	interrupt_id_mgt5200_per_lo

/***************************************************************************
 * Callout : 	interrupt_eoi_mgt5200_per_lo
 *
 * Description: Clear the interrupt, then unmask the level (it was masked in
 *				interrupt_id)
 *
 * On Entry:	r14 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_eoi_mgt5200_per_lo, 0, patcher

/*
 * All peripheral interrupts are level triggered, active high. When the ISR
 * clears the interrupt at the source, the level goes low again, and the
 * updated status is reflected in the PISR. So, it's not necessary to clear
 * the interrupt here, we just have to unmask it.
 */
 	DEVBASE %r6, 0x12345678 

	cmplwi	%r18, 0
	bgt		1f

    lwz     %r5,MGT5200_PIMR(%r6)                  /* put Peripheral Interrupt Mask Register in r5 */

	lis		%r4, 0x8000
	srw		%r4, %r4, %r14				/* shift the bit to the proper position */
	andc	%r5, %r5, %r4				/* unmask by clearing the appropriate bit */
    stw     %r5, MGT5200_PIMR(%r6)
1:	
CALLOUT_END		interrupt_eoi_mgt5200_per_lo


/*************************************************************************************
 *
 * Simple GPIO callouts
 *
 *************************************************************************************/
/***************************************************************************
 * Callout : 	interrupt_unmask_mgt5200_gpio
 *
 * Description: Unmask the given GPIO interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_unmask_mgt5200_gpio, 0, patcher

/*
 * Unmask the interrupt level that gets passed in r4. Setting a bit 
 * in the Wakeup GPIO Individual Interrupt Enable Register unmasks the interrupt.
 */
 	DEVBASE %r6, 0x12345678 
 	
	lwz     %r5, MGT5200_GPIOSIIE(%r6)
	lis     %r7,0x8000				
	srw	    %r7, %r7, %r4			/* shift mask bit to proper position */
	or      %r5,%r5,%r7             /* set bit to unmask */
	stw     %r5, MGT5200_GPIOSIIE(%r6)             /* store new value */
	li      %r3,0
	sync
	blr
CALLOUT_END	interrupt_unmask_mgt5200_gpio

/***************************************************************************
 * Callout : 	interrupt_mask_mgt5200_gpio
 *
 * Description: Mask the given GPIO interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_mask_mgt5200_gpio, 0, patcher

/*
 * Mask the interrupt level that gets passed in r4. Clearing a bit in the 
 * WakeUP GPIO Individual Interrupt Enable Register masks the interrupt.
 */
	DEVBASE %r6, 0x12345678 
	
	lwz %r5, MGT5200_GPIOSIIE(%r6)        
	lis     %r7,0x8000				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	andc    %r5,%r5,%r7             /* clear bit to mask */
	stw     %r5, MGT5200_GPIOSIIE(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_mask_mgt5200_gpio


/***************************************************************************
 * Callout : 	interrupt_id_mgt5200_gpio
 *
 * Description: Determine the GPIO interrupt level and mask it off.
 *
 * On Entry:	Nothing
 *
 * On Exit:		r14 = interrupt level
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_id_mgt5200_gpio, 0, patcher

/*
 * Identify the interrupt level, mask it, and return the level in r14.
 */
 	DEVBASE %r6, 0x12345678 

	li		%r14, -1

    lwz     %r5,MGT5200_GPIOSSR(%r6)     

	lis		%r4, 0xff00
	and		%r5, %r5, %r4	/* clear input value bits */

	cmplwi	%r5,0
	beq		1f		        /* no interrupt bits set, spurious int */

	cntlzw  %r14,%r5      /* count leading zeros in r5, place resulting interrupt value in r14 */

    /* Masking the corresponding GPIO bit in GPIO IE Register */
    lwz     %r4,MGT5200_GPIOSIIE(%r6) 

	lis     %r5,0x8000
	srw     %r5,%r5,%r14            /* shift mask bit right to position of interrupt we wish to mask */
	stw		%r5, MGT5200_GPIOSSR(%r6)				/* clear the interrupt by writing a 1 to the bit */
	andc    %r4,%r4,%r5             /* clear relevant bit to mask it */
	eieio
	stw     %r4, MGT5200_GPIOSIIE(%r6)         
	sync
1:
CALLOUT_END	interrupt_id_mgt5200_gpio

/***************************************************************************
 * Callout : 	interrupt_eoi_mgt5200_gpio
 *
 * Description: Clear the interrupt, then unmask the level (it was masked in
 *				interrupt_id)
 *
 * On Entry:	r14 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_eoi_mgt5200_gpio, 0, patcher

/*
 * clear the interrupt by writing a 1 to the appropriate bit in the status register, 
 * and unmask. r14 still holds the level.
 */
 	DEVBASE %r6, 0x12345678 

    lwz     %r5,MGT5200_GPIOSSR(%r6)               

	lis		%r4, 0x8000
	srw		%r4, %r4, %r14				/* shift the bit to the proper position */
	stw		%r4, MGT5200_GPIOSSR(%r6)					/* write appropriate bit into r6. Don't read/modify/write, */
										/* or we might clear interrupts that haven't been serviced yet */

	cmplwi	%r18, 0
	bgt		1f

	lwz 	%r5, MGT5200_GPIOSIIE(%r6)
	or      %r5,%r4,%r5             /* set bit to unmask */
	stw     %r5, MGT5200_GPIOSIIE(%r6)             /* store new value */
1:
CALLOUT_END		interrupt_eoi_mgt5200_gpio


/*************************************************************************************
 *
 * Wake-up GPIO callouts
 *
 *************************************************************************************/
/***************************************************************************
 * Callout : 	interrupt_unmask_mgt5200_gpiow
 *
 * Description: Unmask the given GPIO interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_unmask_mgt5200_gpiow, 0, patcher

/*
 * Unmask the interrupt level that gets passed in r4. Setting a bit 
 * in the Wakeup GPIO Individual Interrupt Enable Register unmasks the interrupt.
 */
 	DEVBASE %r6, 0x12345678 

	lwz %r5, MGT5200_GPIOWIE(%r6)
	lis     %r7,0x8000				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	or      %r5,%r5,%r7             /* set bit to unmask */
	stw     %r5, MGT5200_GPIOWIE(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_unmask_mgt5200_gpiow

/***************************************************************************
 * Callout : 	interrupt_mask_mgt5200_gpiow
 *
 * Description: Mask the given GPIO interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_mask_mgt5200_gpiow, 0, patcher

/*
 * Mask the interrupt level that gets passed in r4. Clearing a bit in the 
 * WakeUP GPIO Individual Interrupt Enable Register masks the interrupt.
 */
	DEVBASE %r6, 0x12345678 
	
	lwz %r5, MGT5200_GPIOWIE(%r6)        
	lis     %r7,0x8000				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	andc    %r5,%r5,%r7             /* clear bit to mask */
	stw     %r5, MGT5200_GPIOWIE(%r6)             /* store new value */
	li      %r3,0
	blr
CALLOUT_END	interrupt_mask_mgt5200_gpiow


/***************************************************************************
 * Callout : 	interrupt_id_mgt5200_gpiow
 *
 * Description: Determine the GPIO interrupt level and mask it off.
 *
 * On Entry:	Nothing
 *
 * On Exit:		r14 = interrupt level
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_id_mgt5200_gpiow, 0, patcher

/*
 * Identify the interrupt level, mask it, and return the level in r14.
 */
 	DEVBASE %r6, 0x12345678 

	li		%r14, -1

    lwz     %r5,MGT5200_GPIOWSR(%r6)     

	lis		%r4, 0xff00
	and		%r5, %r5, %r4	/* clear bus error bits */

	cmplwi	%r5,0
	beq		1f		/* no interrupt bits set, spurious int */

	cntlzw  %r14,%r5                /* count leading zeros in r5, place resulting interrupt value in r14 */

  	lwz     %r4,MGT5200_GPIOWIE(%r6) 

	lis     %r5,0x8000
	srw     %r5,%r5,%r14            /* shift mask bit right to position of interrupt we wish to mask */
	stw		%r5, MGT5200_GPIOWSR(%r6)				/* write a 1 to clear the interrupt */
	andc    %r4,%r4,%r5             /* clear relevant bit to mask it */
	eieio
	stw     %r4, MGT5200_GPIOWIE(%r6)         
	sync
1:
CALLOUT_END	interrupt_id_mgt5200_gpiow

/***************************************************************************
 * Callout : 	interrupt_eoi_mgt5200_gpiow
 *
 * Description: Clear the interrupt, then unmask the level (it was masked in
 *				interrupt_id)
 *
 * On Entry:	r14 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_eoi_mgt5200_gpiow, 0, patcher

/*
 * clear the interrupt by writing a 1 to the appropriate bit in the status register, 
 * and unmask. r14 still holds the level.
 */
	DEVBASE %r6, 0x12345678 
	
    lwz     %r5,MGT5200_GPIOWSR(%r6)               

	lis		%r4, 0x8000
	srw		%r4, %r4, %r14				/* shift the bit to the proper position */
	stw		%r4, MGT5200_GPIOWSR(%r6)					/* write appropriate bit into r6. Don't read/modify/write, */
										/* or we might clear interrupts that haven't been serviced yet */

	cmplwi	%r18, 0
	bgt		1f

	lwz 	%r5, MGT5200_GPIOWIE(%r6)
	or      %r5,%r4,%r5             /* set bit to unmask */
	stw     %r5, MGT5200_GPIOWIE(%r6)             /* store new value */
1:
CALLOUT_END		interrupt_eoi_mgt5200_gpiow

/***************************************************************************
 *
 * Smartcomm Interrupt Callouts
 *
 ***************************************************************************/

/***************************************************************************
 * Callout : 	interrupt_unmask_mgt5200_sc
 *
 * Description: Unmask the given smartcomm interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_unmask_mgt5200_sc, 0, patcher

/*
 * Unmask the interrupt level that gets passed in r4.
 */
 	DEVBASE %r6, 0x12345678 
 	
	cmplwi	%r4,24
	blt		1f						/* If it's less than 24, it's a task/EU int */

	/* Timer interrupt.  Need to enable int in the timer's control register */	
	li		%r7,7
	sub		%r4,%r7,%r4		/* 7 - r4 = timer number 1:7 */
	li		%r7,4
	slw		%r4,%r4,%r7		/* Get the offset of the individual timer control register */
	addi	%r4,%r4,4
	lis		%r5,MGT5200_SCTMR@ha
	la		%r5,MGT5200_SCTMR@l(%r5)
	add		%r5,%r5,%r4
	add		%r5,%r5,%r6
	lwz		%r7,0(%r5)		/* Load the timer control register */
	ori		%r7,%r7,0x0010			/* Set the IE bit to unmask the individual timer */
	eieio
	stw		%r7,0(%r5)	/* Store the timer control value back to the register */
	sync
	b		2f
	
/* Task or EU interrupt. Setting a bit in the Smartcomm Interrupt Mask
 * Register masks the interrupt, clearing the bit unmasks.
 */
1:
	lwz %r5, MGT5200_SCIMR(%r6)                 /* load contents of Smartcomm Interrupt Mask Register into r5 */
	lis     %r7,0x0080				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	andc    %r5,%r5,%r7             /* unmask bit */
	stw     %r5, MGT5200_SCIMR(%r6)             /* store new value */
	li      %r3,0
2:	
	blr
CALLOUT_END	interrupt_unmask_mgt5200_sc

/***************************************************************************
 * Callout : 	interrupt_mask_mgt5200_sc
 *
 * Description: Mask the given smartcomm interrupt level
 *
 * On Entry:	r4 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_mask_mgt5200_sc, 0, patcher

/*
 * Mask the interrupt level that gets passed in r4.
 */
 	DEVBASE %r6, 0x12345678 
 	
	cmplwi	%r4,24
	blt		1f						/* If it's less than 24, it's a task/EU int */

	/* Timer interrupt.  Need to disable int in the timer's control register */	
	li		%r7,7
	sub		%r4,%r7,%r4		/* 7 - r4 = timer number 1:7 */
	li		%r7,4
	slw		%r4,%r4,%r7		/* Get the offset of the individual timer control register */
	addi	%r4,%r4,4
	lis		%r5,MGT5200_SCTMR@ha
	la		%r5,MGT5200_SCTMR@l(%r5)
	add		%r5,%r5,%r4
	add		%r5,%r5,%r6
	lwz		%r7,0(%r5)		/* Load the timer control register */
	andi.	%r7,%r7,0xffef			/* Clear the IE bit to mask the individual timer */
	eieio
	stw		%r7,0(%r5)	/* Store the timer control value back to the register */
	sync
	b		2f
	
/* Task or EU interrupt. Setting a bit in the Smartcomm Interrupt Mask
 * Register masks the interrupt, clearing the bit unmasks.
 */
1:
	lwz %r5, MGT5200_SCIMR(%r6)                 /* load contents of Smartcomm Interrupt Mask Register into r5 */
	lis     %r7,0x0080				
	srw		%r7, %r7, %r4			/* shift mask bit to proper position */
	or      %r5,%r5,%r7             /* mask bit */
	stw     %r5, MGT5200_SCIMR(%r6)             /* store new value */
	li      %r3,0
2:
	blr
CALLOUT_END	interrupt_mask_mgt5200_sc


/***************************************************************************
 * Callout : 	interrupt_id_mgt5200_sc
 *
 * Description: Determine the smartcomm interrupt level and mask it off.
 *
 * On Entry:	Nothing
 *
 * On Exit:		r14 = smartcomm interrupt level
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_id_mgt5200_sc, 0, patcher

/*
 * Identify the interrupt level, mask it, and return the level in r14.
 */
 	DEVBASE %r6, 0x12345678 
 
 	lwz		%r5,MGT5200_SCTSR(%r6)
 	
 	andis.	%r5,%r5,0x0100	/* Check SC_int */
 	bne		1f
 	
 	lwz		%r5,MGT5200_SCTSR(%r6)
 	andis.	%r5,%r5,0x00fe	/* Check timer interrupt status */
	cntlzw  %r14,%r5        /* count leading zeros in r5, place resulting interrupt value in r14 */
	subi	%r4,%r14,8		/* account for 8 leading zero bits before int status field */
	addi	%r14,%r4,24		/* Add 24 to place timer int vectors after EU and Task ints */	
	
	/* r4 contains relative timer interrupt vector number (7:1) */
	li		%r7,7
	sub		%r4,%r6,%r4		/* 7 - r4 = timer number 1:7 */
	li		%r7,4
	slw		%r4,%r4,%r7		/* Get the offset of the individual timer control register */
	addi	%r4,%r4,4
	lis		%r5,MGT5200_SCTMR@ha
	la		%r5,MGT5200_SCTMR@l(%r5)
	add		%r5,%r5,%r4
	add		%r5,%r5,%r6
	lwz		%r7,0(%r5)		/* Load the timer control register */
	andi.	%r7,%r7,0xffef			/* Clear the IE bit to mask the individual timer */
	eieio
	stw		%r7,0(%r5)	/* Store the timer control value back to the register */
	sync
	b		2f
 	
1:
    lwz     %r5,MGT5200_SCIPR(%r6)                  /* put Smartcomm Interrupt Pending Register in r5 */

    lwz     %r4,MGT5200_SCIMR(%r6)                  /* put Smartcomm Interrupt Mask Register in r4 */

	andc	%r5,%r5,%r4				/* consider only unmasked pending interrupts */
	cntlzw  %r14,%r5                /* count leading zeros in r5, place resulting interrupt value in r14 */
	subi	%r14, %r14, 8			/* account for 8 reserved bits at beginning of SCIPR */

	lis     %r5,0x0080
	srw     %r5,%r5,%r14            /* shift mask bit right to position of interrupt we wish to mask */
	or      %r4,%r4,%r5             /* set relevant bit to mask it */
	eieio
	stw     %r4, MGT5200_SCIMR(%r6)             /* load new enable mask into SCIMR */
	sync
2:
CALLOUT_END	interrupt_id_mgt5200_sc

/***************************************************************************
 * Callout : 	interrupt_eoi_mgt5200_sc
 *
 * Description: Clear the interrupt, then unmask the level (it was masked in
 *				interrupt_id)
 *
 * On Entry:	r14 = interrupt level
 *
 * On Exit:		Nothing
 *
 * Notes:
 *
 **************************************************************************/
CALLOUT_START	interrupt_eoi_mgt5200_sc, 0, patcher

/*
 * Clear and unmask the interrupt level that gets passed in r4.
 */
	DEVBASE %r6, 0x12345678 
	
 	cmplwi	%r14,24
	blt		1f						/* If it's less than 24, it's a task/EU int */
	
	/* 
	 * For timer ints, we just enable the interrupt again.  User ISR will have
	 * to be responsible for clearing the interrupt condition by resetting
	 * the timer.
	 */

	/* Timer interrupt.  Need to enable int in the timer's control register */	
	li		%r7,7
	sub		%r4,%r7,%r14		/* 7 - r14 = timer number 1:7 */
	li		%r7,4
	slw		%r4,%r4,%r7		/* Get the offset of the individual timer control register */
	addi	%r4,%r4,4
	lis		%r5,MGT5200_SCTMR@ha
	la		%r5,MGT5200_SCTMR@l(%r5)
	add		%r5,%r5,%r4
	add      %r5,%r5,%r6
	lwz		%r7,0(%r5)		/* Load the timer control register */
	ori		%r7,%r7,0x0010			/* Set the IE bit to unmask the individual timer */
	eieio
	stw		%r7,0(%r5)	/* Store the timer control value back to the register */
	sync
	b		2f
	
1:

/*
 * Task or EU interrupt.
 */
	cmplwi	%r18, 0
	bgt		2f

    lwz     %r5,MGT5200_SCIMR(%r6)                  /* put Smartcomm Interrupt Mask Register in r5 */

	lis		%r4, 0x0080
	srw		%r4, %r4, %r14				/* shift the bit to the proper position */
	andc	%r5, %r5, %r4				/* unmask by clearing the appropriate bit */
    stw     %r5, MGT5200_SCIMR(%r6)
2:	
CALLOUT_END		interrupt_eoi_mgt5200_sc
