/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include "cpc925.h"

paddr_t	cpc925_mpic_base;


void
config_cpc925(paddr_t mpic_paddr) {
	uintptr_t	mpic;
	uintptr_t	toggle;
	unsigned	tog_val;
	unsigned 	i, j, val;

	cpc925_mpic_base = mpic_paddr;

	// Use the CPC 925 TOGGLE register to reset the MPIC and disable
	// interrupt outputs from it
	toggle = startup_io_map(KILO(4), mpic_paddr - 0x40000) + 0xe0;
	tog_val = in32(toggle);
	out32(toggle, tog_val & ~0x6);
	out32(toggle, tog_val |  0x2);

	mpic = startup_io_map(KILO(512), mpic_paddr);

	// reset the PIC
	out32(mpic + CPC925_CFG_REG, 0x80000000);
	in32(toggle);
	do {
		// wait for reset to complete
	} while(in32(mpic + CPC925_CFG_REG) & 0x80000000);

	// Program task prio regs 
	out32(mpic + CPC925_TASKPRIO_0, 0x00);
	// Prio 0 for CPU1, for IPI's
	out32(mpic + CPC925_TASKPRIO_1, 0x00);

	// Set up external interrupt vectors & priorities
	i = mpic + CPC925_INT_REG_START;
	for(j = 0; j < CPC925_INT_NUM; ++j) {
		// mask interrupt, priority 1
		val = (0x80000000 + (1 << 16)) + j;
		out32(i + CPC925_INT_VEC_OFF, val);
		out32(i + CPC925_INT_DST_OFF, 0x01);
		i += CPC925_INT_REG_SPACING;
	}

	// Set up IPI's. 
	i = mpic + CPC925_IPI_REG_START;
	for(j = 0; j < CPC925_IPI_NUM; j ++) {
		// unmask interrupt, priority 1
		val = ((1 << 16) + CPC925_INT_NUM) + j;
		out32(i + CPC925_IPI_VEC_OFF, val);
		i += CPC925_IPI_REG_SPACING;
	}

	// Program the spurious interrupt register
	out32(mpic + CPC925_SPUR_VECT, 0xff);

	// Program the cfg register to enable cascade mode
	out32(mpic + CPC925_CFG_REG, 0x20000000);

	startup_io_unmap(mpic);

	// Enable interrupt outputs
	out32(toggle, in32(toggle) | 0x4);
	startup_io_unmap(toggle);
}

__SRCVERSION("cpc925.c $Rev: 580669 $");
