/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
	
unsigned	original_msr;

//
// Perform CPU specific startup initialization.
// This code is hardware independant and should not have to be changed
// changed by end users.
//

static void
ppcv_cpuconfig_dummy(int cpu) {
}

static void *
ppcv_map_dummy(unsigned size, paddr_t phys, unsigned prot_flags) {
	return((void *)(uintptr_t)phys);
}

static void
ppcv_unmap_dummy(void *p) {
}

static void
ppcv_nosupport(void) {
	crash("Support for CPU PVR 0x%x not present in startup.\n", get_pvr());
}

static void
ppcv_no_e500v2_support(void) {
	crash("\n"
			"*** PVR: 0x%8x (e500 Version 2, Revision 1)\n"
			"*** Sorry, but the e500v2 revision 1 core is not currently supported by procnto\n"
			"*** Please use Rev 2 (or higher)\n\n", get_pvr()
		 );
}

#define PPC_SETUP(_x_) \
		do { \
			ppcv_config.cpuconfig2 = (void (*)(int))(_x_); \
			ppcv_config.cpuinfo = (void (*)(struct cpuinfo_entry *cpu))(_x_); \
			ppcv_config.qtime = (void (*)(void))(_x_); \
		} while(0)

static void
ppcv_setup_unknown(void) {
	PPC_SETUP(&ppcv_nosupport);
}

static void
ppcv_setup_e500v2_unsupported(void) {
	PPC_SETUP(&ppcv_no_e500v2_support);
}

const struct ppcv_chip	*ppcv;

struct ppcv_config		ppcv_config = {
	-1,
	&ppcv_cpuconfig_dummy,
	&ppcv_cpuconfig_dummy,
	NULL,	// always have a cpuinfo routine set
	NULL,	// always have a qtime routine set
	&ppcv_map_dummy,
	&ppcv_unmap_dummy,
};

static const struct ppcv_chip ppcv_chip_unknown = {
	-1,
	0, 0, 0, 0,
	0,
	0,
	"unknown",
	&ppcv_setup_unknown
};

static const struct ppcv_chip ppcv_chip_e500v2_unsupported = {
	-1,
	0, 0, 0, 0,
	0,
	0,
	"unsupported",
	&ppcv_setup_e500v2_unsupported
};

void
cpu_one_startup(int cpu) {

	// Make sure address translation is off.
	ppc_sync(); 
	set_msr(get_msr() & ~(PPC_MSR_DR |PPC_MSR_IR));
	ppc_isync();

	ppcv_config.cpuconfig1(cpu);

	board_cpuconfig1(cpu);
}

void
cpu_startup() {
	unsigned				ppc_chip;
	const struct ppcv_chip	**tryp;
	const struct ppcv_chip	*try;

	ppc_chip = PPC_GET_FAM_MEMBER(get_pvr());

	/*
	 * Freescale was imprecise in its documentation of the the e500v2 TLB config
	 * in that it indicated that a v2 core has a 512 entry 4-way TLB. In fact,
	 * it is only the version 2, rev 2 (and higher I guess) that have the 512/4
	 * TLB, the rev 1 has a 256/2 TLB. At the time of coding for this processor
	 * we made the mistake of coding the TLB handling code based on processor
	 * version and not using the TLBCFG registers which describe the TLB
	 * arrangement. The version 2 designation indicates more than just a TLB
	 * configuration difference. Because of this and the fact that it was desired
	 * not to put a TLBCFG check in procnto, and it "appears" that not many of
	 * this core are in circulation, it was decided to just fail the startup for
	 * v2r1 cores. 
	*/
	if ((ppc_chip == PPC_E500V2) && (PPC_GET_REVISION(get_pvr()) == 0x10)) {
		try = &ppcv_chip_e500v2_unsupported;
	} else {
		
		tryp = ppcv_list;
		for(;;) {
			try = *tryp;
			if(try == NULL) {
				// Delay reporting the problem that we don't support this
				// chip until we have a hope of the error message coming out.
				try = &ppcv_chip_unknown;
				break;
			}
			if(try->chip == ppc_chip) break;
			++tryp;
		}
	}
	ppcv = try;
	paddr_bits = try->paddr_bits;
	try->setup();
	original_msr = get_msr();
	cpu_one_startup(0);
}

__SRCVERSION("cpu_startup.c $Rev: 580669 $");
