/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





/*
 * falcon_system_clock()
 * Find the system clock speed on a system with a falcon/raven chipset.
 *
 */

#include "startup.h"

unsigned
falcon_system_clock(paddr_t falcon_base) {
	uint32_t 		sys_cfg;
	unsigned char 	sysclk;
	uintptr_t		falcon;

	falcon = startup_io_map(0x10, falcon_base + 0x0400);
	sys_cfg = in32(falcon);
	startup_io_unmap(falcon);

	// Determine bus speed
	sysclk = (sys_cfg >> 20) & 0x0f;
	switch(sysclk) {
	case 0x0e:
		return 60000000;
	case 0x0f:
		return 66666666;
	}
	// default to slowest
	return 50000000;
}

__SRCVERSION("falcon_system_clock.c $Rev: 580669 $");
