/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





/*
 * init_raminfo.c
 *	Tell syspage about our RAM configuration
 */
#include "startup.h"

// Initialize RAM information on a system with a Hawk memory controller
// located at 'hawk_base'

void
hawk_init_raminfo(paddr_t hawk_base) {
	uint32_t 		ram_attr;
	uint32_t		ram_base;
	unsigned char	ram_bits;
	unsigned		size;
	int				i;
	uintptr_t		hawk;

	hawk = startup_io_map(0x20, hawk_base);
	ram_attr = in32(hawk + 0x10);
	ram_base = in32(hawk + 0x18);
	startup_io_unmap(hawk);

	size = 0;	
	for(i = 0; i < 4; i++) {
		if((ram_attr >> (8 * i)) & 0x80) {
			ram_bits = ((ram_attr >> (8 * i)) & 0x0f);
			if(ram_bits == 0x01) {
				size += 0x02000000;
			} else if(ram_bits == 0x02 || ram_bits == 0x03) {
				size += 0x04000000;
			} else if(ram_bits >= 0x04 && ram_bits <= 0x06) {
				size += 0x08000000;
			} else if(ram_bits == 0x7 || ram_bits == 0x08) {
				size += 0x10000000;
			} else if(ram_bits == 0x09) {
				size += 0x20000000;
			}
		}
	}
	add_ram(0, size);
}

__SRCVERSION("hawk_init_raminfo.c $Rev: 580669 $");
