/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include <ppc/403spu.h>

static void
parse_line(unsigned channel, const char *line, unsigned long *baud,
			unsigned long *clk, unsigned long *div) {
	
    if(*line != '.' && *line != '\0') {
		dbg_device[channel].base = strtopaddr(line, (char **)&line, 16);
		if(*line == '^') dbg_device[channel].shift = strtoul(line+1, (char **)&line, 0);
	}
    if(*line == '.') ++line;
    if(*line != '.' && *line != '\0') *baud = strtoul(line, (char **)&line, 0);
    if(*line == '.') ++line;
    if(*line != '.' && *line != '\0') *clk = strtoul(line, (char **)&line, 0);
    if(*line == '.') ++line;
    if(*line != '.' && *line != '\0') *div = strtoul(line, (char **)&line, 0);
}

void
init_403(unsigned channel, const char *init, const char *defaults) {
	unsigned long 	baud;
	unsigned long	div;
	unsigned long	clk;
	unsigned 		cnt;
	paddr_t			base;

	baud = 0;
	parse_line(channel, defaults, &baud, &clk, &div);
	parse_line(channel, init, &baud, &clk, &div);

	base = dbg_device[channel].base;
	if(baud != 0) {
		cnt = clk / (baud * div) - 1;
	
		out8(base+PPC403_BRDL, cnt);
		out8(base+PPC403_BRDH, cnt >> 8);
	}

	// 8 bits no parity
	out8(base+PPC403_SPCTL,
		PPC403_SPCTL_LM_NORMAL
		| PPC403_SPCTL_DTR
		| PPC403_SPCTL_RTS
		| PPC403_SPCTL_DB);
}

void
put_403(int c) {
	paddr_t	base = dbg_device[0].base;

#ifdef NEED_EARLY_OUTPUT
	if(base==0) base=0x40000000;
#endif
	do {
	} while( !(in8(base+PPC403_SPLS) & PPC403_SPLS_TBR) );
	out8(base+PPC403_SPTB, c);
}

__SRCVERSION("hw_ser403.c $Rev: 580669 $");
