/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include "startup.h"
#include "hwinfo_private.h"
#include "ppc/hwinfo_85xx_proto.h"
#include <hw/hwinfo_85xx.h>
#include <drvr/hwinfo.h>		// for hwi support routines in libdrvr

/*
 * Add 8543 specific devices to the hardware info section of the syspage
*/

extern uint32_t platform_clock;

void hwi_8543(hwi85xx_flags_t flags)
{
	unsigned hwi_bus_internal;

	ASSERT(platform_clock != 0);

	/* add an internal bus to which all SOC devices will be attached. We handle
	 * being called multiple times and will only add 1 instance */
	if ((hwi_bus_internal = hwi_find_bus(HWI_ITEM_BUS_INTERNAL, 0)) == HWI_NULL_OFF)
	{
		hwi_bus_internal = hwibus_add(HWI_ITEM_BUS_INTERNAL, HWI_NULL_OFF);
		ASSERT(hwi_find_unit(hwi_bus_internal) == 0);
	}

	/* add I2C (unless directed not to) */
	if ((flags & hwi85xx_flags_t_I2C_NONE) == 0)
	{
		unsigned hwi_off;
		hwiattr_i2c_t attr = HWIATTR_I2C_T_INITIALIZER;
		// I2C source clock is 1/2 the CCB (ie. plaform_clock)
		struct hwi_inputclk clksrc = {.clk = platform_clock / 2, .div = 1};
		struct hwi_busattr busattr = {.speed = 100000, .addr = 0, .width = 0, .flags = 0};

		HWIATTR_I2C_SET_NUM_IRQ(&attr, 1);
		HWIATTR_I2C_SET_NUM_CLK(&attr, 1);
		HWIATTR_I2C_SET_NUM_BUSATTR(&attr, 1);
			
		/* create i2c0 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x3000, 0x18, 0, hwi_find_as(immr_paddr + 0x3000, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* create i2c1 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x3100, 0x18, 0, hwi_find_as(immr_paddr + 0x3100, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* we are done if the hwi85xx_flags_t_I2C_ONLY flag is set */
		if (flags & hwi85xx_flags_t_I2C_ONLY) return;
	}

	/* add 3 PCI ports */
	{
		unsigned hwi_off;
		hwiattr_pci_t attr = HWIATTR_PCI_T_INITIALIZER;
		struct hwi_busattr busattr = {.speed = 0, .addr = 0, .width=0, .flags = 0};

		HWIATTR_PCI_SET_NUM_IRQ(&attr, 1);
		HWIATTR_PCI_SET_DLL(&attr, "mpc8543");
		HWIATTR_PCI_SET_NUM_BUSATTR(&attr, 2);	// memory and I/O base addresses
		HWIATTR_PCI_SET_NUM_IRQBLKS(&attr, 1);

		/* create PCIe device 0 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x8000, 0xE28, 0, hwi_find_as(immr_paddr + 0x8000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* initialize bus attributes */
		busattr.addr = 0xc0000000ULL;	// memory space base address
		hwitag_set_busattr(hwi_off, 0, &busattr);
		busattr.addr = 0xe2000000ULL;	// I/O space base address
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe device 1 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x9000, 0xE28, 0, hwi_find_as(immr_paddr + 0x9000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* initialize bus attributes */
		busattr.addr = 0xa0000000ULL;	// memory space base address
		hwitag_set_busattr(hwi_off, 0, &busattr);
		busattr.addr = 0xe2800000ULL;	// I/O space base address
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe device 2 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0xA000, 0xE38, 0, hwi_find_as(immr_paddr + 0xA000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		/* initialize bus attributes */
		busattr.addr = 0x80000000ULL;	// memory space base address
		hwitag_set_busattr(hwi_off, 0, &busattr);
		busattr.addr = 0xe3000000ULL;	// I/O space base address
		hwitag_set_busattr(hwi_off, 1, &busattr);
	}

	/* add 2 UART's */
	{
		unsigned hwi_off;
		hwiattr_uart_t attr = HWIATTR_UART_T_INITIALIZER;
		// UART source clock is the CCB (ie. plaform_clock)
		// FIX ME - not sure why we just don't do the division first and make div = 1?
		struct hwi_inputclk clksrc = {.clk = platform_clock, .div = 16};

		HWIATTR_UART_SET_NUM_IRQ(&attr, 1);
		HWIATTR_UART_SET_NUM_CLK(&attr, 1);
		
		/* create uart0 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x4500, 0x14, 0, hwi_find_as(immr_paddr + 0x4500, 1));
		hwi_off = hwidev_add_uart(PPC85xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* create uart1 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x4600, 0x14, 0, hwi_find_as(immr_paddr + 0x4600, 1));
		hwi_off = hwidev_add_uart(PPC85xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* historically the UART's were called '8250' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_UART, 0), "8250");
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_UART, 1), "8250");
	}

	/* add the 2 eTSEC devices */
	{
		unsigned hwi_off;
		hwiattr_enet_t attr = HWIATTR_ENET_T_INITIALIZER;
		uint8_t mac[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0xFF};	// a non zero mac
		/* Each TSEC device has a set of MII management registers in order to
		 * communicate with its PHY however only the MII registers for TSEC1 can be
		 * used to access this bus regardless of which physical TSEC is being managed.
		 * Because of this, the mpc85xx TSEC driver has been written to look for an
		 * extra location tag for each configured instance which specifies the address
		 * of the MII registers. It will be set to the appropriate offset within the TSEC1
		 * register map.
		 * Note that because the driver is currently written expecting the PHY base value
		 * to be the base address of the TSEC registers (ie. it has the offset of 0x520
		 * hardcoded), we must not include this value in the setting of mii_reg.base. For
		 * this reason, the mii_reg.len value must necessarily now include this offset */
		const struct hwi_location mii_reg = {.base = (immr_paddr + 0x24000) /*+ 0x520*/, .len = 0x18 + 0x520};

		/* we configure for 4 interrupts, one each for Tx, Rx, Err and 1 PHY */
		if (flags & hwi85xx_flags_t_USE_MAC) HWIATTR_ENET_SET_MAC(&attr, mac);
		HWIATTR_ENET_SET_NUM_IRQ(&attr, 4);
		HWIATTR_ENET_SET_NUM_PHY(&attr, 1);
		HWIATTR_ENET_SET_DLL(&attr, "mpc85xx");

		/* create eTSEC0 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, immr_paddr + 0x24000, 0xE00, 0, hwi_find_as(immr_paddr + 0x24000, 1));
		hwi_off = hwidev_add_enet(PPC85xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 0);
		hwitag_add_location(hwi_off, mii_reg.base, mii_reg.len, 0, AS_NULL_OFF);

		/* create eTSEC1 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, immr_paddr + 0x26000, 0xE00, 0, hwi_find_as(immr_paddr + 0x26000, 1));
		hwi_off = hwidev_add_enet(PPC85xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 1);
		hwitag_add_location(hwi_off, mii_reg.base, mii_reg.len, 0, AS_NULL_OFF);

		/* historically the eTSEC's were called 'tsec' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_eTSEC, 0), PPC85xx_HWI_LEGACY_TSEC);
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_eTSEC, 1), PPC85xx_HWI_LEGACY_TSEC);
	}
	
	/* add 1 dma controller (4 channels) */
	{
		unsigned hwi_off;
		hwiattr_dma_t attr = HWIATTR_DMA_T_INITIALIZER;

		HWIATTR_DMA_SET_NUM_IRQ(&attr, 4);	// each of 4 channels can interrupt
		
		/* create DMA controller 0 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x21000, 0x304, 0, hwi_find_as(immr_paddr + 0x21100, 1));
		hwi_off = hwidev_add_dma(PPC85xx_HWI_DMA, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
	}

	/* add performance monitor */
	hwidev_add_85xx_pmon(PPC85xx_HWI_PMON, hwi_bus_internal, immr_paddr + 0xE1000);
	
	/* add security device */
	hwidev_add_85xx_security(PPC85xx_HWI_SECURITY, hwi_bus_internal, immr_paddr + 0x30000);
	
	/* add GPIO */
	hwidev_add_85xx_gpio(PPC85xx_HWI_GPIO, hwi_bus_internal, immr_paddr + 0xE0030, 1);
	
	/* add Serial RapidIO devices - FIX ME - what about the architecturally defined registers at 0xC0000 */
	{
		unsigned hwi_off;

		hwi_off = hwidev_add(PPC85xx_HWI_SRIO_EWP, 0, hwi_bus_internal);
		hwitag_add_irq(hwi_off, HWI_ILLEGAL_VECTOR);
		hwidev_add_85xx_srio_doorbell(PPC85xx_HWI_SRIO_DOORBELL, hwi_bus_internal, immr_paddr + 0xD3400);
		hwidev_add_85xx_srio_msgunit(PPC85xx_HWI_SRIO_MSGUNIT, hwi_bus_internal, immr_paddr + 0xD3000);
		hwidev_add_85xx_srio_msgunit(PPC85xx_HWI_SRIO_MSGUNIT, hwi_bus_internal, immr_paddr + 0xD3100);
	}

	/* add group A timers (4 timers per group) */
	hwidev_add_85xx_timer_group(PPC85xx_HWI_GTIMER, hwi_bus_internal, immr_paddr + 0x410F0, 4);
	
	/* add 8 message devices */
	hwidev_add_85xx_message(PPC85xx_HWI_MSG, hwi_bus_internal, immr_paddr + 0x41400, 4);

	/* add 8 shared message devices */
	hwidev_add_85xx_shared_message(PPC85xx_HWI_SMSG, hwi_bus_internal, immr_paddr + 0x41600, 8);

	/* add 12 external interrupt devices (board specific) */
	hwidev_add_85xx_ext_irq(PPC85xx_HWI_EXT_IRQ, hwi_bus_internal, 12);
}

__SRCVERSION("$URL$ $Rev$");
