/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include "startup.h"
#include "hwinfo_private.h"
#include "ppc/hwinfo_85xx_proto.h"
#include <hw/hwinfo_85xx.h>
#include <drvr/hwinfo.h>		// for hwi support routines in libdrvr

/*
 * Add 8544 specific devices to the hardware info section of the syspage
*/
#define MPC8544_REG_BASE_I2C0		(immr_paddr + 0x3000)
#define MPC8544_REG_LEN_I2C0		0x18
#define MPC8544_REG_BASE_I2C1		(immr_paddr + 0x3100)
#define MPC8544_REG_LEN_I2C1		0x18

#define MPC8544_REG_BASE_UART1		(immr_paddr + 0x4500)
#define MPC8544_REG_LEN_UART1		0x14
#define MPC8544_REG_BASE_UART2		(immr_paddr + 0x4600)
#define MPC8544_REG_LEN_UART2		0x14

#define MPC8544_REG_BASE_PCI		(immr_paddr + 0x8000)
#define MPC8544_REG_LEN_PCI			0xE28
#define MPC8544_REG_BASE_PCIe1		(immr_paddr + 0xA000)
#define MPC8544_REG_LEN_PCIe1		0xE28
#define MPC8544_REG_BASE_PCIe2		(immr_paddr + 0x9000)
#define MPC8544_REG_LEN_PCIe2		0xE28
#define MPC8544_REG_BASE_PCIe3		(immr_paddr + 0xB000)
#define MPC8544_REG_LEN_PCIe3		0xE28

#define MPC8544_REG_BASE_DMA		(immr_paddr + 0x21100)
#define MPC8544_REG_LEN_DMA			0x304

#define MPC8544_REG_BASE_TSEC1		(immr_paddr + 0x24000)
#define MPC8544_REG_LEN_TSEC1		0xE00
#define MPC8544_REG_BASE_TSEC3		(immr_paddr + 0x26000)
#define MPC8544_REG_LEN_TSEC3		0xE00

#define MPC8544_REG_BASE_SEC		(immr_paddr + 0x31008)
#define MPC8544_REG_BASE_TIMER		(immr_paddr + 0x410F0)
#define MPC8544_REG_BASE_MSG		(immr_paddr + 0x41400)
#define MPC8544_REG_BASE_SMSG		(immr_paddr + 0x41600)
#define MPC8544_REG_BASE_GPIO		(immr_paddr + 0xE0030)
#define MPC8544_REG_BASE_PMON		(immr_paddr + 0xE1000)


extern uint32_t platform_clock;

void hwi_8544(hwi85xx_flags_t flags, unsigned num_pci_slots)
{
	unsigned hwi_bus_internal;

	ASSERT(platform_clock != 0);

	/* add an internal bus to which all SOC devices will be attached. We handle
	 * being called multiple times and will only add 1 instance */
	if ((hwi_bus_internal = hwi_find_bus(HWI_ITEM_BUS_INTERNAL, 0)) == HWI_NULL_OFF)
	{
		hwi_bus_internal = hwibus_add(HWI_ITEM_BUS_INTERNAL, HWI_NULL_OFF);
		ASSERT(hwi_find_unit(hwi_bus_internal) == 0);
	}

	/* add I2C (unless directed not to) */
	if ((flags & hwi85xx_flags_t_I2C_NONE) == 0)
	{
		unsigned hwi_off;
		hwiattr_i2c_t attr = HWIATTR_I2C_T_INITIALIZER;
		// I2C source clock is 1/2 the CCB (ie. plaform_clock)
		struct hwi_inputclk clksrc = {.clk = platform_clock / 2, .div = 1};
		struct hwi_busattr busattr = {.speed = 100000, .addr = 0, .size = 0, .width = 0, .flags = 0};

		HWIATTR_I2C_SET_NUM_IRQ(&attr, 1);
		HWIATTR_I2C_SET_NUM_CLK(&attr, 1);
		HWIATTR_I2C_SET_NUM_BUSATTR(&attr, 1);
			
		/* create i2c0 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, MPC8544_REG_BASE_I2C0, MPC8544_REG_LEN_I2C0, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* create i2c1 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, MPC8544_REG_BASE_I2C1, MPC8544_REG_LEN_I2C1, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* we are done if the hwi85xx_flags_t_I2C_ONLY flag is set */
		if (flags & hwi85xx_flags_t_I2C_ONLY) return;
	}

	/* add the 1 PCI and 3 PCIe contollers */
	{
#define SET_BUSATTR_PCI_MEMTYPE(f, mt)	((f) = (((f) & ~hwi_busattr_PCI_memtype_mask) | (mt)))

		unsigned hwi_off;
		hwiattr_pci_t attr = HWIATTR_PCI_T_INITIALIZER;
		struct hwi_busattr busattr = {.speed = 0, .addr = 0, .size = 0, .width=0, .flags = 0};

		HWIATTR_PCI_SET_DLL(&attr, "mpc8544");
		HWIATTR_PCI_SET_NUM_BUSATTR(&attr, 2);	// memory and I/O base addresses (values set in BSP)

		/* for the PCI controllers, we have 1 interrupt for the controller and 1 for each of
		 * the INT A/B/C/D lines for each slot on the board */
		HWIATTR_PCI_SET_NUM_IRQ(&attr, 1 + 4 * num_pci_slots);

		/* create PCI controller 0 */
		HWIATTR_PCI_SET_LOCATION(&attr, MPC8544_REG_BASE_PCI, MPC8544_REG_LEN_PCI, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* initialize PCI bus attributes */
		busattr.speed = 66000000ULL;	// 66MHz
		busattr.width = 32;
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);
		
		/* for the PCIe controllers, we have 5 interrupts. 1 for the controller and 1 for each of
		 * the INT A/B/C/D lines when used in root complex mode */
		HWIATTR_PCI_SET_NUM_IRQ(&attr, 5);

		/* 
		 * NOTE: The bus width for the PCIe controllers is configurable and should be read from
		 * the SERDES configuration. For now we will use the default values of x4 for controllers
		 * 1 and 2 and x1 for controller 3
		*/

		/* create PCIe controller 1 */
		HWIATTR_PCI_SET_LOCATION(&attr, MPC8544_REG_BASE_PCIe1, MPC8544_REG_LEN_PCIe1, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* initialize PCIe bus attributes */
		busattr.speed = 2500000000ULL;	// 2.5GHz
		busattr.width = 4;				// x4, x2 and x1
		busattr.flags = hwi_busattr_isPCIe;
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe controller 2 */
		HWIATTR_PCI_SET_LOCATION(&attr, MPC8544_REG_BASE_PCIe2, MPC8544_REG_LEN_PCIe2, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		/* initialize PCIe bus attributes */
		busattr.speed = 2500000000ULL;	// 2.5GHz
		busattr.width = 4;				// x4, x2 and x1
		busattr.flags = hwi_busattr_isPCIe;
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe controller 3 */
		HWIATTR_PCI_SET_LOCATION(&attr, MPC8544_REG_BASE_PCIe3, MPC8544_REG_LEN_PCIe3, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 3);
		/* initialize PCIe bus attributes */
		busattr.speed = 2500000000ULL;	// 2.5GHz
		busattr.width = 1;				// x1
		busattr.flags = hwi_busattr_isPCIe;
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);
	}

	/* add 2 UART's */
	{
		unsigned hwi_off;
		hwiattr_uart_t attr = HWIATTR_UART_T_INITIALIZER;
		// UART source clock is the CCB (ie. plaform_clock)
		// FIX ME - not sure why we just don't do the division first and make div = 1?
		struct hwi_inputclk clksrc = {.clk = platform_clock, .div = 16};

		HWIATTR_UART_SET_NUM_IRQ(&attr, 1);
		HWIATTR_UART_SET_NUM_CLK(&attr, 1);
		
		/* create uart0 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, MPC8544_REG_BASE_UART1, MPC8544_REG_LEN_UART1, 0, AS_NULL_OFF);
		hwi_off = hwidev_add_uart(PPC85xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* create uart1 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, MPC8544_REG_BASE_UART2, MPC8544_REG_LEN_UART2, 0, AS_NULL_OFF);
		hwi_off = hwidev_add_uart(PPC85xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* historically the UART's were called '8250' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_UART, 0), "8250");
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_UART, 1), "8250");
	}

	/* add the 2 eTSEC devices */
	{
		unsigned hwi_off;
		hwiattr_enet_t attr = HWIATTR_ENET_T_INITIALIZER;
		uint8_t mac[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0xFF};	// a non zero mac
		/* Each TSEC device has a set of MII management registers in order to
		 * communicate with its PHY however only the MII registers for TSEC1 can be
		 * used to access this bus regardless of which physical TSEC is being managed.
		 * Because of this, the mpc85xx TSEC driver has been written to look for an
		 * extra location tag for each configured instance which specifies the address
		 * of the MII registers. It will be set to the appropriate offset within the TSEC1
		 * register map.
		 * Note that because the driver is currently written expecting the PHY base value
		 * to be the base address of the TSEC registers (ie. it has the offset of 0x520
		 * hardcoded), we must not include this value in the setting of mii_reg.base. For
		 * this reason, the mii_reg.len value must necessarily now include this offset */
		const struct hwi_location mii_reg = {.base = MPC8544_REG_BASE_TSEC1 /*+ 0x520*/, .len = 0x18 + 0x520};

		/* we configure for 4 interrupts, one each for Tx, Rx, Err and 1 PHY */
		if (flags & hwi85xx_flags_t_USE_MAC) HWIATTR_ENET_SET_MAC(&attr, mac);
		HWIATTR_ENET_SET_NUM_IRQ(&attr, 4);
		HWIATTR_ENET_SET_NUM_PHY(&attr, 1);
		HWIATTR_ENET_SET_DLL(&attr, "mpc85xx");

		/* create eTSEC0 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, MPC8544_REG_BASE_TSEC1, MPC8544_REG_LEN_TSEC1, 0, AS_NULL_OFF);
		hwi_off = hwidev_add_enet(PPC85xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 0);
		hwitag_add_location(hwi_off, mii_reg.base, mii_reg.len, 0, AS_NULL_OFF);

		/* create eTSEC1 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, MPC8544_REG_BASE_TSEC3, MPC8544_REG_LEN_TSEC3, 0, AS_NULL_OFF);
		hwi_off = hwidev_add_enet(PPC85xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 1);
		hwitag_add_location(hwi_off, mii_reg.base, mii_reg.len, 0, AS_NULL_OFF);

		/* historically the eTSEC's were called 'tsec' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_eTSEC, 0), PPC85xx_HWI_LEGACY_TSEC);
		hwi_add_synonym(hwi_find_device(PPC85xx_HWI_eTSEC, 1), PPC85xx_HWI_LEGACY_TSEC);
	}

	/* add 1 dma controller (4 channels) */
	{
		unsigned hwi_off;
		hwiattr_dma_t attr = HWIATTR_DMA_T_INITIALIZER;

		HWIATTR_DMA_SET_NUM_IRQ(&attr, 4);	// each of 4 channels can interrupt
		
		/* create DMA controller */
		HWIATTR_DMA_SET_LOCATION(&attr, MPC8544_REG_BASE_DMA, MPC8544_REG_LEN_DMA, 0, AS_NULL_OFF);
		hwi_off = hwidev_add_dma(PPC85xx_HWI_DMA, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
	}

	/* add performance monitor */
	hwidev_add_85xx_pmon(PPC85xx_HWI_PMON, hwi_bus_internal, MPC8544_REG_BASE_PMON);
	
	/* add security device */
	hwidev_add_85xx_security(PPC85xx_HWI_SECURITY, hwi_bus_internal, MPC8544_REG_BASE_SEC);
	
	/* add GPIO */
	hwidev_add_85xx_gpio(PPC85xx_HWI_GPIO, hwi_bus_internal, MPC8544_REG_BASE_GPIO, 1);

	/* add group A timers (4 timers per group) */
	hwidev_add_85xx_timer_group(PPC85xx_HWI_GTIMER, hwi_bus_internal, MPC8544_REG_BASE_TIMER, 4);
	
	/* add 4 message devices */
	hwidev_add_85xx_message(PPC85xx_HWI_MSG, hwi_bus_internal, MPC8544_REG_BASE_MSG, 4);

	/* add 8 shared message devices */
	hwidev_add_85xx_shared_message(PPC85xx_HWI_SMSG, hwi_bus_internal, MPC8544_REG_BASE_SMSG, 8);

	/* add 12 external interrupt devices (board specific) */
	hwidev_add_85xx_ext_irq(PPC85xx_HWI_EXT_IRQ, hwi_bus_internal, 12);
}

__SRCVERSION("$URL$ $Rev$");
