/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include "startup.h"
#include "hwinfo_private.h"
#include <hw/hwinfo_85xx.h>
#include <pic_85xx.h>

/*
 * Common functions for getting mpc85xx family devices into the 'hwinfo' section
 * of the syspage
 * 
 * Note that additional tags may be added based on the device however this
 * requires more discussions with device driver owners
 * 
 * FIX ME - probably should have each of these functions be in its own file
 */

void hwidev_add_85xx_pmon(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 1);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0xCC, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_security(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 2);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x820, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_gpio(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 1);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x18, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_srio_doorbell(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 1);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x7C, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}
	
void hwidev_add_85xx_srio_msgunit(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 2);	// outbound/inbound interrupts
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x7C, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_pme(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 5);	// 4 channels + a general interrupt
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x4800, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_tlu(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 1);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x820, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_timer_group(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, num);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x214, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, num);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x40, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);

	/* the message boxes are discontigous. Two groups of 4 reside at different locations */
	if (num > 4) {
		HWIATTR_SET_LOCATION(&attr, base_addr + 0x1000, 0x40, 0, hwi_find_as(base_addr, 1));
		
		hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);
	}
}

void hwidev_add_85xx_shared_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, num);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x150, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_ext_irq(const char *device_name, unsigned bus_hwi_off, unsigned num)
{
	/* there is no location information for external IRQ's. These interrupts are
	 * used to wire up external device (which this rotuine can't know about) or
	 * for PCI interrupts */
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, num);
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_sata(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 1);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x414, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

void hwidev_add_85xx_sdhc(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, 1);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x410, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}

__SRCVERSION("$URL$ $Rev$");

