/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include "startup.h"
#include "hwinfo_private.h"
#include <hw/hwinfo_p20xx.h>

/*
 * Common functions for getting P2020 devices into the 'hwinfo' section
 * of the syspage. The p2020 is very similar to the 85xx and most of the
 * functions in hwi_85xx.c can be reused, except for the functions in this file.
 */
void hwidev_add_p20xx_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, num);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x34, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);

	/* the message boxes are discontigous. Two groups of 4 reside at different locations */
	if (num > 4) {
		HWIATTR_SET_LOCATION(&attr, base_addr + 0x1000, 0x34, 0, hwi_find_as(base_addr, 1));
		
		hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);
	}
}

void hwidev_add_p20xx_shared_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num)
{
	hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
	unsigned hwi_off;

	HWIATTR_SET_NUM_IRQ(&attr, num);
	HWIATTR_SET_LOCATION(&attr, base_addr, 0x144, 0, hwi_find_as(base_addr, 1));
	
	hwi_off = hwidev_add(device_name, 0, bus_hwi_off);
	ASSERT(hwi_off != HWI_NULL_OFF);
	hwitag_add_common(hwi_off, &attr);
}



__SRCVERSION("$URL:$ $Rev:$");

