/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include "startup.h"
#include "hwinfo_private.h"
#include "ppc/hwinfo_85xx_proto.h" /* Pull in the 85xx protos */
#include "ppc/hwinfo_p40xx_proto.h"
#include <hw/hwinfo_p40xx.h>
#include <drvr/hwinfo.h>		// for hwi support routines in libdrvr

/*
 * Add P4080 specific devices to the hardware info section of the syspage
*/

extern uint32_t platform_clock;

void hwi_p4080()
{
	unsigned hwi_bus_internal;

	ASSERT(platform_clock != 0);

	/* add an internal bus to which all SOC devices will be attached. We handle
	 * being called multiple times and will only add 1 instance */
	if ((hwi_bus_internal = hwi_find_bus(HWI_ITEM_BUS_INTERNAL, 0)) == HWI_NULL_OFF)
	{
		hwi_bus_internal = hwibus_add(HWI_ITEM_BUS_INTERNAL, HWI_NULL_OFF);
		ASSERT(hwi_find_unit(hwi_bus_internal) == 0);
	}

	/* add I2C */
	{
		unsigned hwi_off;
		hwiattr_i2c_t attr = HWIATTR_I2C_T_INITIALIZER;
		// I2C source clock is equal to the CCB (ie. plaform_clock)
		struct hwi_inputclk clksrc = {.clk = platform_clock, .div = 1};
		struct hwi_busattr busattr = {.speed = 100000, .addr = 0, .width = 0, .flags = 0};

		HWIATTR_I2C_SET_NUM_IRQ(&attr, 1);
		HWIATTR_I2C_SET_NUM_CLK(&attr, 1);
		HWIATTR_I2C_SET_NUM_BUSATTR(&attr, 1);
			
		/* create i2c0 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x118000, 0x18, 0, hwi_find_as(immr_paddr + 0x118000, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* create i2c1 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x118100, 0x18, 0, hwi_find_as(immr_paddr + 0x118100, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		
		/* create i2c2 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x119000, 0x18, 0, hwi_find_as(immr_paddr + 0x119000, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* create i2c3 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x119100, 0x18, 0, hwi_find_as(immr_paddr + 0x119100, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 3);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);		
	}

	/* add 3 PCI ports */
	{
		unsigned hwi_off;
		hwiattr_pci_t attr = HWIATTR_PCI_T_INITIALIZER;
		struct hwi_busattr busattr = {.speed = 0, .addr = 0, .width=0, .flags = 0};

		HWIATTR_PCI_SET_NUM_IRQ(&attr, 4);
		HWIATTR_PCI_SET_DLL(&attr, "p4080_pci");
		HWIATTR_PCI_SET_NUM_BUSATTR(&attr, 2);	// memory and I/O base addresses

		/* create PCIe device 0 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x200000, 0xE38, 0, hwi_find_as(immr_paddr + 0x200000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* initialize bus attributes */
		busattr.addr = 0x80000000ULL;	// memory space base address
		hwitag_set_busattr(hwi_off, 0, &busattr);
		busattr.addr = 0xf8000000ULL;	// I/O space base address
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe device 1 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x201000, 0xE38, 0, hwi_find_as(immr_paddr + 0x201000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* initialize bus attributes */
		busattr.addr = 0xa0000000ULL;	// memory space base address
		hwitag_set_busattr(hwi_off, 0, &busattr);
		busattr.addr = 0xf8010000ULL;	// I/O space base address
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe device 2 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x202000, 0xE38, 0, hwi_find_as(immr_paddr + 0x202000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		/* initialize bus attributes */
		busattr.addr = 0xc0000000ULL;	// memory space base address
		hwitag_set_busattr(hwi_off, 0, &busattr);
		busattr.addr = 0xf8020000ULL;	// I/O space base address
		hwitag_set_busattr(hwi_off, 1, &busattr);
	}

	/* add 4 UART's */
	{
		unsigned hwi_off;
		hwiattr_uart_t attr = HWIATTR_UART_T_INITIALIZER;
		// UART source clock is the 1/2 of the CCB (ie. plaform_clock/2)
		struct hwi_inputclk clksrc = {.clk = platform_clock, .div = 2};

		HWIATTR_UART_SET_NUM_IRQ(&attr, 1);
		HWIATTR_UART_SET_NUM_CLK(&attr, 1);
		
		/* create uart0 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x11C500, 0x14, 0, hwi_find_as(immr_paddr + 0x11C500, 1));
		hwi_off = hwidev_add_uart(PPCP40xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* create uart1 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x11C600, 0x14, 0, hwi_find_as(immr_paddr + 0x11C600, 1));
		hwi_off = hwidev_add_uart(PPCP40xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		
		/* create uart2 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x11D500, 0x14, 0, hwi_find_as(immr_paddr + 0x11D500, 1));
		hwi_off = hwidev_add_uart(PPCP40xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* create uart3 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x11D600, 0x14, 0, hwi_find_as(immr_paddr + 0x11D600, 1));
		hwi_off = hwidev_add_uart(PPCP40xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 3);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);		

		/* historically the UART's were called '8250' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPCP40xx_HWI_UART, 0), "8250");
		hwi_add_synonym(hwi_find_device(PPCP40xx_HWI_UART, 1), "8250");
		hwi_add_synonym(hwi_find_device(PPCP40xx_HWI_UART, 2), "8250");
		hwi_add_synonym(hwi_find_device(PPCP40xx_HWI_UART, 3), "8250");		
	}

	/* add 2 USB controllers */
	{
		unsigned hwi_off;
		hwiattr_usb_t attr = HWIATTR_USB_T_INITIALIZER;

		HWIATTR_USB_SET_NUM_IRQ(&attr, 1);
		HWIATTR_USB_SET_DLL(&attr, "ehci-p4080");

		/* create usb controller 1 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x210000, 0x504, 0, hwi_find_as(immr_paddr + 0x210000, 1));
		hwi_off = hwibus_add_usb(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		
		/* create usb controller 2 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x211000, 0x504, 0, hwi_find_as(immr_paddr + 0x211000, 1));
		hwi_off = hwibus_add_usb(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);		
	}	
	
	/* add SPI */
	{
		unsigned hwi_off;
		hwiattr_spi_t attr = HWIATTR_SPI_T_INITIALIZER;
		// SPI source clock is 1/2 the CCB (ie. plaform_clock)
		struct hwi_inputclk clksrc = {.clk = platform_clock, .div = 1};
		struct hwi_busattr busattr = {.speed = 0, .addr = 0, .size = 0, .width = 0, .flags = hwi_busattr_MASTER};

		HWIATTR_SPI_SET_NUM_IRQ(&attr, 1);
		HWIATTR_SPI_SET_NUM_CLK(&attr, 1);
		HWIATTR_SPI_SET_NUM_BUSATTR(&attr, 1);
		HWIATTR_SPI_SET_DLL(&attr, "ppc_espi");

		/* create spi0 and set the clock source */
		HWIATTR_SPI_SET_LOCATION(&attr,  immr_paddr + 0x110000, 0x30, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_spi(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);
	}	
	
	/* add 2 dma controllers (4 channels each) */
	{
		unsigned hwi_off;
		hwiattr_dma_t attr = HWIATTR_DMA_T_INITIALIZER;

		HWIATTR_DMA_SET_NUM_IRQ(&attr, 4);	// each of 4 channels can interrupt
		
		/* create DMA controller 0 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x100000, 0x304, 0, hwi_find_as(immr_paddr + 0x100000, 1));
		hwi_off = hwidev_add_dma(PPCP40xx_HWI_DMA, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		
		/* create DMA controller 1 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x101000, 0x304, 0, hwi_find_as(immr_paddr + 0x101000, 1));
		hwi_off = hwidev_add_dma(PPCP40xx_HWI_DMA, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
	}
	/* add the buffer manager */	
	{
		unsigned hwi_off;		
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
		const unsigned bman_law_id   = ppcp4080_law_find_target(ppcp40xx_law_target_id_BMSP, NULL);
		paddr_t bman_addr = ppcp4080_law_get_addr(bman_law_id);

		ASSERT(bman_law_id != -1);

		HWIATTR_SET_NUM_IRQ(&attr, 10);
		
		/* create buffer manager */
		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x31A000, 0xE10, 0, hwi_find_as(immr_paddr + 0x31A000, 1));
		hwi_off = hwidev_add(PPCP40xx_HWI_BMAN, 0, hwi_bus_internal);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);
		hwitag_add_location(hwi_off, bman_addr, ppcp4080_law_get_size(bman_law_id), 0, hwi_find_as(bman_addr, 1));	
	}
	
	/* add the queue manager */	
	{
		unsigned hwi_off;		
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;
		const unsigned qman_law_id   = ppcp4080_law_find_target(ppcp40xx_law_target_id_QMSP, NULL);
		paddr_t qman_addr = ppcp4080_law_get_addr(qman_law_id);

		ASSERT(qman_law_id != -1);

		HWIATTR_SET_NUM_IRQ(&attr, 10);
		
		/* create queue manager */
		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x318000, 0xE18, 0, hwi_find_as(immr_paddr + 0x318000, 1));
		hwi_off = hwidev_add(PPCP40xx_HWI_QMAN, 0, hwi_bus_internal);
		ASSERT(hwi_off != HWI_NULL_OFF);
		hwitag_add_common(hwi_off, &attr);
		hwitag_add_location(hwi_off, qman_addr, ppcp4080_law_get_size(qman_law_id), 0, hwi_find_as(qman_addr, 1));	
	}	

	/* add the frame manager */	
	{
		unsigned hwi_off;		
		hwiattr_common_t attr = HWIATTR_COMMON_INITIALIZER;

		HWIATTR_SET_NUM_IRQ(&attr, 1);
		
		/* create frame manager 0 */
		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x400000, 0x100000, 0, hwi_find_as(immr_paddr + 0x400000, 1));
		hwi_off = hwidev_add(PPCP40xx_HWI_FMAN, 0, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_add_common(hwi_off, &attr);
		
		/* create frame manager 1 */
		HWIATTR_SET_LOCATION(&attr, immr_paddr + 0x500000, 0x100000, 0, hwi_find_as(immr_paddr + 0x500000, 1));
		hwi_off = hwidev_add(PPCP40xx_HWI_FMAN, 0, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_add_common(hwi_off, &attr);		
	}

	/* add common error interrupt */
	hwidev_add_p40xx_ext_irq(PPCP40xx_HWI_CERR, hwi_bus_internal, 1);

	/* add 4 IPI interrupts */
	hwidev_add_p40xx_ext_irq(PPCP40xx_HWI_IPI,  hwi_bus_internal, 4);
	
	/* add group A and B timers (4 timers per group) */
	hwidev_add_p40xx_timer_group(PPCP40xx_HWI_GTIMER, hwi_bus_internal, immr_paddr + 0x410F0, 4);
	hwidev_add_p40xx_timer_group(PPCP40xx_HWI_GTIMER, hwi_bus_internal, immr_paddr + 0x420F0, 4);
	
	/* add group A and B message devices (4 message devices per group) */
	hwidev_add_p40xx_message(PPCP40xx_HWI_MSG, hwi_bus_internal, immr_paddr + 0x41400, 4);
	hwidev_add_p40xx_message(PPCP40xx_HWI_MSG, hwi_bus_internal, immr_paddr + 0x42400, 4);

	/* add bank A, B and C shared message devices (8 shared message devices per group) */
	hwidev_add_p40xx_shared_message(PPCP40xx_HWI_SMSG, hwi_bus_internal, immr_paddr + 0x41600, 8);
	hwidev_add_p40xx_shared_message(PPCP40xx_HWI_SMSG, hwi_bus_internal, immr_paddr + 0x41800, 8);
	hwidev_add_p40xx_shared_message(PPCP40xx_HWI_SMSG, hwi_bus_internal, immr_paddr + 0x41A00, 8);

	/* add 12 external interrupt devices (board specific) */
	hwidev_add_p40xx_ext_irq(PPCP40xx_HWI_EXT_IRQ, hwi_bus_internal, 12);
}

__SRCVERSION("$URL$ $Rev$");
