/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


/*
* PPC85xx SOC devices
*
* This file contains names for all of the devices that may exists on any 85xx
* processor variant. Specifically, these are the internal devices that are part
* of the MPC85xx SOC and not board specific devices. Board specific devices
* are defined in a similarly named board specific file
* 
* The names are added to the hwinfo section of the syspage (at the discretion of
* the startup writer) so that drivers can use hwi_find_device() to determine
* the existence of SOC device instances in a portable fashion and optionally
* retrieve device specific information like interrupt vectors, etc.
* 
* Not all of the devices are on every processor so hwi_find_device() may return
* HWI_NULL_OFF if a device, or device instance (ie. unit) is not found.
*
*/

#ifndef __HWINFO_85xx_PROTO_H_INCLUDED
#define __HWINFO_85xx_PROTO_H_INCLUDED


/* FIX ME - consider a single second parameter as a pointer to a structure containing relevant attributes */
/* FIX ME - some of these go away as common devices are in sys/hwinfo.h */

extern void hwidev_add_85xx_sata(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_sdhc(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);

/* FIX ME - I think what follows will be considered 85xx specific devices */
extern void hwidev_add_85xx_pmon(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_security(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_gpio(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_85xx_srio_doorbell(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_srio_msgunit(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_pme(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_tlu(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr);
extern void hwidev_add_85xx_timer_group(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_85xx_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_85xx_shared_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_85xx_ext_irq(const char *device_name, unsigned bus_hwi_off, unsigned num);

typedef enum
{
	hwi85xx_flags_t_NONE		= 0,
	hwi85xx_flags_t_USE_MAC		= (1UL << 0),		// create a NIC address tag for ENET devices (must be set in BSP)
	hwi85xx_flags_t_I2C_ONLY	= (1UL << 1),		// only create I2C devices (this is for EEPROM reads on Freescale boards)
	hwi85xx_flags_t_I2C_NONE	= (1UL << 2),		// don't create I2C devices (so that hwi_85xx() can be called twice without duplicates)
	hwi85xx_flags_t_SEC_NONE	= (1UL << 3),		// don't create SECURITY devices

} hwi85xx_flags_t;

extern void hwi_8572(hwi85xx_flags_t flags);
extern void hwi_8536(hwi85xx_flags_t flags, unsigned num_pci_slots);
extern void hwi_8548(hwi85xx_flags_t flags);
extern void hwi_8547(hwi85xx_flags_t flags);
extern void hwi_8545(hwi85xx_flags_t flags);
extern void hwi_8544(hwi85xx_flags_t flags, unsigned num_pci_slots);
extern void hwi_8543(hwi85xx_flags_t flags);

/* prototypes for LAW register access */
extern unsigned mpc8536_law_numreg(void);
extern unsigned mpc8536_law_is_enabled(unsigned law_id);
extern int mpc8536_law_get_target(unsigned law_id);
extern size_t mpc8536_law_get_size(unsigned law_id);
extern uint64_t mpc8536_law_get_addr(unsigned law_id);
extern int mpc8536_law_find_target(unsigned target_id, unsigned *law_id_p);
extern int mpc8536_law_enable(unsigned law_id);
extern int mpc8536_law_disable(unsigned law_id);
extern uint64_t mpc8536_law_set_addr(unsigned law_id, uint64_t addr);
extern size_t mpc8536_law_set_size(unsigned law_id, size_t size);
extern size_t mpc8536_law_set_target(unsigned law_id, unsigned target_id);
extern int mpc8536_law_alloc(void);
extern void mpc8536_law_dump(int law_id);

extern unsigned mpc8544_law_numreg(void);
extern unsigned mpc8544_law_is_enabled(unsigned law_id);
extern int mpc8544_law_get_target(unsigned law_id);
extern size_t mpc8544_law_get_size(unsigned law_id);
extern uint64_t mpc8544_law_get_addr(unsigned law_id);
extern int mpc8544_law_find_target(unsigned target_id, unsigned *law_id_p);
extern int mpc8544_law_enable(unsigned law_id);
extern int mpc8544_law_disable(unsigned law_id);
extern uint64_t mpc8544_law_set_addr(unsigned law_id, uint64_t addr);
extern size_t mpc8544_law_set_size(unsigned law_id, size_t size);
extern size_t mpc8544_law_set_target(unsigned law_id, unsigned target_id);
extern int mpc8544_law_alloc(void);
extern void mpc8544_law_dump(int law_id);

extern unsigned mpc8572_law_numreg(void);
extern unsigned mpc8572_law_is_enabled(unsigned law_id);
extern int mpc8572_law_get_target(unsigned law_id);
extern size_t mpc8572_law_get_size(unsigned law_id);
extern uint64_t mpc8572_law_get_addr(unsigned law_id);
extern int mpc8572_law_find_target(unsigned target_id, unsigned *law_id_p);
extern int mpc8572_law_enable(unsigned law_id);
extern int mpc8572_law_disable(unsigned law_id);
extern uint64_t mpc8572_law_set_addr(unsigned law_id, uint64_t addr);
extern size_t mpc8572_law_set_size(unsigned law_id, size_t size);
extern size_t mpc8572_law_set_target(unsigned law_id, unsigned target_id);
extern int mpc8572_law_alloc(void);
extern void mpc8572_law_dump(int law_id);

extern unsigned mpc8641_law_numreg(void);
extern unsigned mpc8641_law_is_enabled(unsigned law_id);
extern int mpc8641_law_get_target(unsigned law_id);
extern size_t mpc8641_law_get_size(unsigned law_id);
extern uint64_t mpc8641_law_get_addr(unsigned law_id);
extern int mpc8641_law_find_target(unsigned target_id, unsigned *law_id_p);
extern int mpc8641_law_enable(unsigned law_id);
extern int mpc8641_law_disable(unsigned law_id);
extern uint64_t mpc8641_law_set_addr(unsigned law_id, uint64_t addr);
extern size_t mpc8641_law_set_size(unsigned law_id, size_t size);
extern size_t mpc8641_law_set_target(unsigned law_id, unsigned target_id);
extern int mpc8641_law_alloc(void);
extern void mpc8641_law_dump(int law_id);

/* LAW register target ID's */
#define mpc8536_law_target_id_PCI		0x0
#define mpc8536_law_target_id_PCIe_1	0x2
#define mpc8536_law_target_id_PCIe_2	0x1
#define mpc8536_law_target_id_PCIe_3	0x3
#define mpc8536_law_target_id_LBUS		0x4
#define mpc8536_law_target_id_SDRAM		0xF

#define mpc8544_law_target_id_PCI		0x0
#define mpc8544_law_target_id_PCIe_1	0x1
#define mpc8544_law_target_id_PCIe_2	0x2
#define mpc8544_law_target_id_PCIe_3	0x3
#define mpc8544_law_target_id_LBUS		0x4
#define mpc8544_law_target_id_SDRAM		0xF

#define mpc8572_law_target_id_PCIe_1	0x2
#define mpc8572_law_target_id_PCIe_2	0x1
#define mpc8572_law_target_id_PCIe_3	0x0
#define mpc8572_law_target_id_LBUS		0x4
#define mpc8572_law_target_id_SDRAM_IL	0xB		// Interleaved
#define mpc8572_law_target_id_SDRAM_1	0xF
#define mpc8572_law_target_id_SDRAM_2	0x16

#define mpc8641_law_target_id_PCIe_1	0x0
#define mpc8641_law_target_id_PCIe_2	0x1
#define mpc8641_law_target_id_LBUS		0x4
#define mpc8641_law_target_id_SDRAM_IL	0xB		// Interleaved
#define mpc8641_law_target_id_SRIO		0xC
#define mpc8641_law_target_id_SDRAM_1	0xF
#define mpc8641_law_target_id_SDRAM_2	0x16

#endif	/* __HWINFO_85xx_PROTO_H_INCLUDED */

__SRCVERSION("hwinfo_85xx_proto.h $Rev: 169799 $");

