/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


/*
* PPCP20xx SOC devices
*
* This file contains names for all of the devices that may exists on P20xx
* processor. Specifically, these are the internal devices that are part
* of the P20xx SOC and not board specific devices. Board specific devices
* are defined in a similarly named board specific file. Since the P20xx
* family of processors are very similar to the 85xx family, we reuse all
* the code we can, from the 85xx. As a result of this the P2020 code will
* contain of a lot of calls to the 85xx functions.
* 
* The names are added to the hwinfo section of the syspage (at the discretion of
* the startup writer) so that drivers can use hwi_find_device() to determine
* the existence of SOC device instances in a portable fashion and optionally
* retrieve device specific information like interrupt vectors, etc.
* 
* Not all of the devices are on every processor so hwi_find_device() may return
* HWI_NULL_OFF if a device, or device instance (ie. unit) is not found.
*
*/

#ifndef __HWINFO_P20xx_PROTO_H_INCLUDED
#define __HWINFO_P20xx_PROTO_H_INCLUDED

#include "hwinfo_85xx_proto.h"

extern void hwidev_add_p20xx_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_p20xx_shared_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);

extern void hwi_p2020(hwi85xx_flags_t flags);

/* LAW register target ID's */
#define ppcp20xx_law_target_id_PCIe_1	0x2
#define ppcp20xx_law_target_id_PCIe_2	0x1
#define ppcp20xx_law_target_id_PCIe_3	0x0
#define ppcp20xx_law_target_id_LBUS	0x4
#define ppcp20xx_law_target_id_SDRAM	0xF
#define ppcp20xx_law_target_id_SRIO1	0xC
#define ppcp20xx_law_target_id_SRIO2	0xD


#endif	/* __HWINFO_P20xx_PROTO_H_INCLUDED */

__SRCVERSION("$URL:$ $Rev: 169799 $");

