/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


/*
* Prototypes for adding P40xx SOC devices to hwinfo section of the Syspage
* This file contains the prototypes for adding the P40xx SOC devices to the 
* hwinfo section of the syspage. This file also gives the Law Target ID's
* for the devices which are accessed through the LAW's and also the prototypes
* for accessing the LAW registers
*
*/

#ifndef __HWINFO_P40xx_PROTO_H_INCLUDED
#define __HWINFO_P40xx_PROTO_H_INCLUDED

#include "hwinfo_85xx_proto.h"

/* Adding p40xx specific devices */
extern void hwidev_add_p40xx_timer_group(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_p40xx_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_p40xx_shared_message(const char *device_name, unsigned bus_hwi_off, paddr_t base_addr, unsigned num);
extern void hwidev_add_p40xx_ext_irq(const char *device_name, unsigned bus_hwi_off, unsigned num);

extern void hwi_p4080();

/* LAW register target ID's */
#define ppcp40xx_law_target_id_PCIe_1	0x00
#define ppcp40xx_law_target_id_PCIe_2	0x01
#define ppcp40xx_law_target_id_PCIe_3	0x02
#define ppcp40xx_law_target_id_SRIO1	0x08
#define ppcp40xx_law_target_id_SRIO2	0x09
#define ppcp40xx_law_target_id_SDRAM_1	0x10
#define ppcp40xx_law_target_id_SDRAM_2	0x11
#define ppcp40xx_law_target_id_SDRAM_I	0x14
#define ppcp40xx_law_target_id_BMSP	    0x18 //Buffer Manager Software Portal
#define ppcp40xx_law_target_id_DCSR	    0x1D
#define ppcp40xx_law_target_id_LBUS	    0x1F
#define ppcp40xx_law_target_id_QMSP	    0x3C //Queue Manager Software Portal

/* prototypes for LAW register access */
extern unsigned ppcp4080_law_numreg(void);
extern unsigned ppcp4080_law_is_enabled(unsigned law_id);
extern int ppcp4080_law_get_target(unsigned law_id);
extern size_t ppcp4080_law_get_size(unsigned law_id);
extern uint64_t ppcp4080_law_get_size64(unsigned law_id);
extern uint64_t ppcp4080_law_get_addr(unsigned law_id);
extern int ppcp4080_law_find_target(unsigned target_id, unsigned *law_id_p);
extern int ppcp4080_law_enable(unsigned law_id);
extern int ppcp4080_law_disable(unsigned law_id);
extern uint64_t ppcp4080_law_set_addr(unsigned law_id, uint64_t addr);
extern size_t ppcp4080_law_set_size(unsigned law_id, size_t size);
extern size_t ppcp4080_law_set_target(unsigned law_id, unsigned target_id);
extern int ppcp4080_law_alloc(void);
extern void ppcp4080_law_dump(int law_id);


#endif	/* __HWINFO_P40xx_PROTO_H_INCLUDED */

__SRCVERSION("$URL:$ $Rev: $");

