#
# Copyright 2007, 2008, QNX Software Systems. 
# 
# Licensed under the Apache License, Version 2.0 (the "License"). You 
# may not reproduce, modify or distribute this software except in 
# compliance with the License. You may obtain a copy of the License 
# at: http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OF ANY KIND, either express or implied.
#
# This file may contain contributions from others, either as 
# contributors under the License or as licensors under other terms.  
# Please review this entire file for other proprietary rights or license 
# notices, as well as the QNX Development Suite License Guide at 
# http://licensing.qnx.com/license-guide/ for other information.
#
#
# Bootstrap secondary cpu on 85xx processors
#
# board_smp_start() brings the secondary cpu out of reset with BPTR
# set to execute at the last word of this code page.
#
# The secondary cpu bootstrap needs to set up a TLB entry to cover the
# startup memory then jump to the libstartup SMP entry point.
#

	.include "asmoff.def"

	.data
	.global mpc85xx_smp_entry_point
mpc85xx_smp_entry_point:	.long	0

	.text
.ifdef PPC_CPUOP_ENABLED	
	.cpu booke32
.else	
	.machine "booke32"
.endif	

	.global mpc85xx_smp_boot_code
	.align 12
mpc85xx_smp_boot_code:

	#
	# Enable and invalidate icache
	#
	mfspr	%r0,PPCE500_SPR_L1CSR1
	ori		%r0,%r0,(PPCE500_L1CSR1_ICFI|PPCE500_L1CSR1_ICE)
	oris	%r0,%r0,PPCE500_L1CSR1_ICPE@h
	mtspr	PPCE500_SPR_L1CSR1,%r0
	isync

	#
	# Enable and invalidate dcache
	#
	mfspr	%r0,PPCE500_SPR_L1CSR0
	ori		%r0,%r0,(PPCE500_L1CSR0_CFI|PPCE500_L1CSR0_CE)
	oris	%r0,%r0,PPCE500_L1CSR0_CPE@h
	mtspr	PPCE500_SPR_L1CSR0,%r0
	isync

	#
	# Set up TLB to access first 256MB of SDRAM
	#
	lis		%r6,0x1001				# TLB=1, ESEL=1
	mtspr	PPCBKEM_SPR_MAS0,%r6

	lis		%r6,0xc000				# MAS1_V | MAS1_IPROT
	ori		%r6,%r6,0x0900			# 256MB TSIZE
	mtspr	PPCBKEM_SPR_MAS1,%r6

	lis		%r6,0					# EPN is 0
	ori		%r6,%r6,0x0004			# MAS2_M
	mtspr	PPCBKEM_SPR_MAS2,%r6

	lis		%r7,0					# RPN is 0
	ori		%r7,%r7,0x15			# SX|SW|SR
	mtspr	PPCBKEM_SPR_MAS3,%r7
	tlbwe	%r0,%r0,%r0				# operands ignored
	isync

	#
	# Jump to entry point set by board_smp_start()
	#
	lis		%r1,mpc85xx_smp_entry_point@ha
	lwz		%r0,mpc85xx_smp_entry_point@l(%r1)
	mtlr	%r0
	blrl

mpc85xx_smp_boot_end:
	.space 4092 - (mpc85xx_smp_boot_end - mpc85xx_smp_boot_code)
	b	mpc85xx_smp_boot_code
