/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





/*
 * Get memory size information from the open firmware tree
 */

#include "startup.h"


static uint64_t
get_val(void *p, unsigned size) {
	if(size == 2) {
		return *(uint64_t *)p;
	}
	return *(uint32_t *)p;
}


void
of_init_raminfo(void) {
	void		*node;
	char		prop[128];
	unsigned	i;
	unsigned	len;
	uint32_t	addr_size;
	uint32_t	size_size;
	int			first = 1;
	paddr_t		ram_start;
	paddr_t		ram_len;

	node = NULL;
	for( ;; ) {
		node = of_next(node);
		if(node == NULL) break;
		if(first) {
			first = 0;
			of_getprop(node, "#address-cells", &addr_size, sizeof(addr_size));
			of_getprop(node, "#size-cells",    &size_size, sizeof(size_size));
		}
		of_getprop(node, "device_type", prop, sizeof(prop));
		if(prop[0] == '\0') {
			// Some OF's don't set the device_type, so use the name
			of_getprop(node, "name", prop, sizeof(prop));
		}
		if(strcmp(prop, "memory") == 0) {
			len = of_getprop(node, "reg", prop, sizeof(prop));
			// property is too long, just use the first bit
			if(len > sizeof(prop)) len = sizeof(prop);
			i = 0;
			while(i < len) {
			   ram_start = get_val(&prop[i], addr_size);
			   i += addr_size*sizeof(uint32_t);
			   ram_len = get_val(&prop[i], size_size);
			   i += size_size*sizeof(uint32_t);
			   add_ram(ram_start, ram_len);
		   }
		}
	}
}
