/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





/*
 * SMP startup file for the boards with Open Firmware
 */

#include "startup.h"



unsigned
of_smp_num_cpu(void) {
	unsigned	num_cpus;
	void		*node;
	char		prop[40];

	num_cpus = 0;
	node = NULL;
	for( ;; ) {
		node = of_next(node);
		if(node == NULL) break;
		of_getprop(node, "device_type", prop, sizeof(prop));
		if(strcmp(prop, "cpu") == 0) {
			++num_cpus;
		}
	}
	return num_cpus;
}


// Perform board specific initialization of the smp_entry section of
// the system page.
void
of_smp_init(struct smp_entry *smp, unsigned num_cpus) {
}


// Get the 'cpu' processor executing at the 'start' address.
int
of_smp_start(unsigned cpu, void (*start)(void)) {
	void		*node;
	char		prop[40];
	unsigned	id;

	id = 0;
	node = NULL;
	for( ;; ) {
		node = of_next(node);
		if(node == NULL) break;
		of_getprop(node, "device_type", prop, sizeof(prop));
		if(strcmp(prop, "cpu") == 0) {
			if(cpu == id) {
				of_call("start-cpu", 3, 0, node, start, 0);
				return 1;
			}
			++id;
		}
	}
	return 0;
}


// Perform any necessary adjustment of the cpu number
unsigned
of_smp_adjust_num(unsigned cpu) {
	return cpu;
}

__SRCVERSION("of_smp.c $Rev: 580669 $");
