/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

unsigned
of_call(const char *service, unsigned nargs, unsigned nret, ...) {
	struct of_parms {
		unsigned	service;
		unsigned	nargs;
		unsigned	nret;
		unsigned	args[10];
	}		req;
	unsigned	save_msr;
	unsigned	save_sdr1;
	unsigned	r;
	va_list		va;
	unsigned	i;

	req.service = (uintptr_t)service;
	req.nargs   = nargs;
	req.nret    = nret;
	va_start(va, nret);
	for(i = 0; i < nargs; ++i) {
		req.args[i] = va_arg(va, unsigned);
	}
	va_end(va);
	memset(&req.args[nargs], 0, sizeof(req.args[0])*nret);

	if(original_sdr1 != -1) {
		save_sdr1 = get_spr(PPC_SPR_SDR1);
		set_spr(PPC_SPR_SDR1, original_sdr1);
	} else {
		save_sdr1 = 0;
	}
	save_msr = get_msr();
	set_msr(original_msr);
	ppc_isync();

	r = ((int (*)(void *))boot_regs[5])(&req);

	set_msr(save_msr);
	ppc_isync();
	if(original_sdr1 != -1) {
		set_spr(PPC_SPR_SDR1, save_sdr1);
		ppc_isync();
	}

	if((int)r < 0) {
		r = -1;
	} else if(nret > 0) {
		r = req.args[nargs];
	}
	return r;
}


int
of_getprop(void *node, const char *name, void *prop, unsigned max) {
	return of_call("getprop", 4, 1, node, name, prop, max);
}


void *
of_next(void *node) {
	void	*next;

	if(node != NULL) {
		next = (void *)of_call("child", 1, 1, node);
		if(next != NULL) return next;
	}
	for( ;; ) {
		next = (void *)of_call("peer", 1, 1, node);
		if(next != NULL) return next;
		node = (void *)of_call("parent", 1, 1, node);
		if(node == NULL) return NULL;
	}
}

__SRCVERSION("openfirmware.c $Rev: 580669 $");
