/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include "pa6t_1682.h"

paddr_t	pa6t_1682_mpic_base;


void
config_pa6t_1682(paddr_t mpic_paddr) {
	uintptr_t	mpic;
	unsigned 	i, j, val;

	pa6t_1682_mpic_base = mpic_paddr;

	mpic = startup_io_map(0x30000, mpic_paddr);

#if 0	//NYI: don't init right now - have to implement workaround
	// reset the PIC
	out32(mpic + PA6T_1682_CFG_REG, 0x80000000);
	in32(toggle);
	do {
		// wait for reset to complete
	} while(in32(mpic + PA6T_1682_CFG_REG) & 0x80000000);
#endif	

	// Program task prio regs 
	outle32(mpic + PA6T_1682_TASKPRIO_0, 0x00);
	// Prio 0 for CPU1, for IPI's
	outle32(mpic + PA6T_1682_TASKPRIO_1, 0x00);

	// Set up external interrupt vectors & priorities
	i = mpic + PA6T_1682_INT_REG_START;
	for(j = 0; j < PA6T_1682_INT_NUM; ++j) {
		// mask interrupt, priority 1, level sensitive, active low
		val = 0x80410001 + j;
		outle32(i + PA6T_1682_INT_VEC_OFF, val);
		outle32(i + PA6T_1682_INT_DST_OFF, 0x01);
		i += PA6T_1682_INT_REG_SPACING;
	}

	// Set up MSI interrupts	
	i = mpic + PA6T_1682_MSI_REG_START;
	for(j = 0; j < PA6T_1682_MSI_NUM; ++j) {
		// mask interrupt, priority 1 
		val = (0x80010001 + PA6T_1682_INT_NUM) + j;
		outle32(i + PA6T_1682_INT_VEC_OFF, val);
		outle32(i + PA6T_1682_INT_DST_OFF, 0x01);
		i += PA6T_1682_MSI_REG_SPACING;
	}

	// Set up timers
	i = mpic + PA6T_1682_TIMER_REG_START;
	for(j = 0; j < PA6T_1682_TIMER_NUM; ++j) {
		// mask interrupt, priority 1 
		val = (0x80010001 + PA6T_1682_INT_NUM+PA6T_1682_MSI_NUM) + j;
		outle32(i + PA6T_1682_INT_VEC_OFF, val);
		outle32(i + PA6T_1682_INT_DST_OFF, 0x01);
		i += PA6T_1682_TIMER_REG_SPACING;
	}

	// Set up IPI's. 
	i = mpic + PA6T_1682_IPI_REG_START;
	for(j = 0; j < PA6T_1682_IPI_NUM; j ++) {
		// mask interrupt, priority 1 
		val = (0x80010001 + PA6T_1682_INT_NUM+PA6T_1682_MSI_NUM+PA6T_1682_TIMER_NUM) + j;
		// Leave IPI0 unmasked, since we're using that procnto is using that
		// one and chip errata 4111 means that we can't unmask it with the
		// callouts
		if(j == 0) val &= ~0x80000000;
		outle32(i + PA6T_1682_INT_VEC_OFF, val);
		i += PA6T_1682_IPI_REG_SPACING;
	}

	// Program the spurious interrupt register
	outle32(mpic + PA6T_1682_SPUR_VECT, 0);

	startup_io_unmap(mpic);
}

__SRCVERSION("pa6t_1682.c $Rev: 580669 $");
