/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "startup.h"
#include <pic_85xx.h>

/* FIX ME - it's possible that this could be more generic than 85xx but I leave it
			named that way for now
*/
/*
 * =============================================================================
 *                                M A C R O S
 * =============================================================================
*/

/*
 * =============================================================================
 *                   F O R W A R D   D E C L A R A T I O N S
 * =============================================================================
*/

/*
 * =============================================================================
 *                               A P I 's
 * =============================================================================
*/


/*******************************************************************************
* set_config_flags
* clr_config_flags
* 
* set or clear <flags> in the optional config flags table.
* 
* Return: the new flags value. Note that 0 will be returned if rv->config_flags
* 			is NULL
*/
cfg_flags_t set_config_flags(vector_tbl_t *rv, unsigned n, cfg_flags_t flags)
{
	if (rv->config_flags != NULL) rv->config_flags[n] |= flags;
	return rv->config_flags[n];
}
cfg_flags_t clr_config_flags(vector_tbl_t *rv, unsigned n, cfg_flags_t flags)
{
	if (rv->config_flags != NULL) rv->config_flags[n] &= ~flags;
	return rv->config_flags[n];
}

/*******************************************************************************
* set_regoffset
* 
* enter the register offset value <reg_off> in to the vector table <rv> at entry
* <n>. If a config_flags table is provided, make sure that the
* INTR_CONFIG_FLAG_DISALLOWED flag is turned off allowing the vector to be
* attached too.
*/
void set_regoffset(vector_tbl_t *rv, unsigned n, reg_off_t reg_off)
{
	rv->offset[n] = reg_off;
	clr_config_flags(rv, n, INTR_CONFIG_FLAG_DISALLOWED);
}



__SRCVERSION("pic_85xx.c $Rev: 580669 $");
