/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */







#include "startup.h"
#include <ppc/86xxcpu.h>
#include <ppc/86xxintr.h>
#include "pic_86xx.h"


#define PIC_IN(r)		in32( base+ (PPC86xx_CCSR_OFF_##r))
#define PIC_OUT(r,v)	out32(base+ (PPC86xx_CCSR_OFF_##r),v)
#define VP_IPI ((1 << PPC86xx_xIVPR_PRIORITY_SHIFT))
#define VP_DEFAULT (PPC86xx_xIVPR_MSK|(1 << PPC86xx_xIVPR_PRIORITY_SHIFT))

#define PPC86xx_CTPR_PCPU_0		0x60080
#define PPC86xx_CTPR_PCPU_1		0x61080

paddr_t		mpc8641d_pic_base;

void ppc86xx_config_pic(void) {
	uintptr_t	base;
	unsigned	i;
	unsigned	vector;

	mpc8641d_pic_base = 0xF8040000;
	base = startup_io_map(MEG(1), immr_paddr);

	PIC_OUT(GCR,PPC86xx_GCR_RST);
	// Wait for the reset to complete
	do {
	} while(PIC_IN(GCR) & PPC86xx_GCR_RST);
	PIC_OUT(GCR,PPC86xx_GCR_M);

	// Enable interrupts on both CPUs
	out32(base + PPC86xx_CTPR_PCPU_0, 0);
	out32(base + PPC86xx_CTPR_PCPU_1, 0);

	vector = 0;

	// 48 Internal interrupts 
	for(i = 0; i < 48; ++i) {
		PIC_OUT(IIVPR(i), (VP_DEFAULT|PPC86xx_xIVPR_P) | vector);
		PIC_OUT(IIDR(i), PPC86xx_xIDR_P0 );
		++vector;
	}

	// 12 External interrupts 
	for(i = 0; i < 12; ++i) {
		if ( i < 8 || i == 9 )
			PIC_OUT(EIVPR(i), (VP_DEFAULT|PPC86xx_xIVPR_S|PPC86xx_xIVPR_P) | vector);
		else
			PIC_OUT(EIVPR(i), (VP_DEFAULT|PPC86xx_xIVPR_S) | vector);
			
		PIC_OUT(EIDR(i), PPC86xx_xIDR_P0 );
		++vector;
	}

	// 4 IPI's 
	for(i = 0; i < 4; ++i) {
		if ( i == 0 )
			PIC_OUT(IPIVPR(i), VP_IPI | vector);
		else
			PIC_OUT(IPIVPR(i), VP_DEFAULT | vector);
		
		++vector;
	}

	// 4 Global Timers - bank A
	for(i = 0; i < 4; ++i) {
		PIC_OUT(GTVPRA(i), VP_DEFAULT | vector);
		PIC_OUT(GTDRA(i), PPC86xx_xIDR_P0);
		++vector;
	}

	// 4 Global Timers - bank B
	for(i = 0; i < 4; ++i) {
		PIC_OUT(GTVPRB(i), VP_DEFAULT | vector);
		PIC_OUT(GTDRB(i), PPC86xx_xIDR_P0);
		++vector;
	}

	// 4 Message interrupts
	for(i = 0; i < 4; ++i) {
		PIC_OUT(MIVPR(i), VP_DEFAULT | vector);
		PIC_OUT(MIDR(i), PPC86xx_xIDR_P0);
		++vector;
	}

	/* 8 Message shared interrupts */
	for(i = 0; i < 8; ++i) {
		PIC_OUT(MSIVPR(i), VP_DEFAULT | vector);
		PIC_OUT(MSIDR(i), PPC86xx_xIDR_P0);
		++vector;
	}
	startup_io_unmap(base);
}

__SRCVERSION("pic_86xx.c $Rev: 580669 $");
