/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





/*
 * ppc4080_set_clock_freqs.c
 * Set the cpu_freq, cycles_freq, and timer_freq variables, based on
 * the main system clock
 *
 */

#include "startup.h"
#include <ppc/p4080cpu.h>

void
ppc4080_set_get_clock_freqs(unsigned sys_clk, unsigned rtc,
						unsigned *platform_p, unsigned *core_p) {

	unsigned 	platform_clock;
	unsigned	cluster_clock1, cluster_clock2, cluster_clock3, cluster_clock4; // There are 4 Core Cluster Clocks
	unsigned 	ccm; // core clock multiplier
	unsigned 	core_clock = 0; // Based on the Core Cluster Clocks
	unsigned	platform_pll, cluster_pll, core_pll;
	uintptr_t	base;

	base = startup_io_map(MEG(1), immr_paddr);

	/* This value contains clock ratios for platform and memory controller */
	platform_pll = in32(base + PPCP4080_CCSR_OFF_RCWSR1);
	cluster_pll = in32(base + PPCP4080_CCSR_OFF_RCWSR3);
	core_pll = in32(base + PPCP4080_CCSR_OFF_RCWSR4);

	platform_clock = sys_clk * ((platform_pll & PPCP4080_SYS_PLL_RAT_MASK) >> PPCP4080_SYS_PLL_RAT_SHIFT);
	
	cluster_clock1 = sys_clk * ((cluster_pll & PPCP4080_CC1_PLL_RAT_MASK) >> PPCP4080_CC1_PLL_RAT_SHIFT);
	cluster_clock2 = sys_clk * ((cluster_pll & PPCP4080_CC2_PLL_RAT_MASK) >> PPCP4080_CC2_PLL_RAT_SHIFT);
	cluster_clock3 = sys_clk * ((cluster_pll & PPCP4080_CC3_PLL_RAT_MASK) >> PPCP4080_CC3_PLL_RAT_SHIFT);
	cluster_clock4 = sys_clk * ((cluster_pll & PPCP4080_CC4_PLL_RAT_MASK) >> PPCP4080_CC4_PLL_RAT_SHIFT);
	
	/* As of now, reading only the core 0's clock multiplier value */
	ccm = ((core_pll & PPCP4080_C0_PLL_SEL_MASK) >> PPCP4080_C0_PLL_SEL_SHIFT);
	
	switch(ccm)
	{
		case 0:
			core_clock = cluster_clock1;
			break;
		case 1:
			core_clock = (cluster_clock1) / 2;
			break;
		case 2:
			core_clock = (cluster_clock1) / 4;
			break;
		case 4:
			core_clock = cluster_clock2;
			break;
		case 5:
			core_clock = (cluster_clock2) / 2;
			break;
		case 6:
			core_clock = (cluster_clock2) / 4;
			break;
		case 8:
			core_clock = cluster_clock3;
			break;
		default:
			crash("What multiplier is this ?\n");
			break;
	}
		
	if(cpu_freq == 0) {
		cpu_freq = core_clock;
	}

	/* The time base clock is clocked by, either through RTC or Platform_clock/16  */
	if(!(get_spr(PPCE500_SPR_HID0) & PPCE500_HID0_SEL_TBCLK)) {
		rtc = platform_clock / 16;
	}

	if(timer_freq == 0)  timer_freq  = rtc;
	if(cycles_freq == 0) cycles_freq = rtc;

	startup_io_unmap(base);

	if (platform_p != NULL) {
		*platform_p = platform_clock;
	}
	if (core_p != NULL) {
		*core_p = core_clock;
	}
}


void
ppc4080_set_clock_freqs(unsigned sys_clk, unsigned rtc) {

	ppc4080_set_get_clock_freqs(sys_clk, rtc, NULL, NULL);

}

__SRCVERSION("ppc4080_set_clock_freqs.c $Rev: $");
