/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

/*
 * ppc800_set_clock_freqs()
 * Set the cpu_freq, cycles_freq, and timer_freq variables based on
 * the main system clocks if they haven't already been set.
 */

void
ppc800_set_clock_freqs(unsigned extclk_freq, unsigned extal_freq, int is_extclk) {
	unsigned 		oscclk_freq;
	unsigned long	freq;
	unsigned long	div;
	unsigned long	sccr;
	unsigned long	plprcr;
	unsigned		mult;

	oscclk_freq = is_extclk ? extclk_freq : extal_freq;

	plprcr = PPC800_IMMR(PLPRCR,32);
	mult = (plprcr >> PPC800_PLPRCR_MF_SHIFT)+1;
	freq = oscclk_freq * mult;
	if(cpu_freq == 0) cpu_freq = freq;
	sccr = PPC800_IMMR(SCCR,32);
	if(cycles_freq == 0) {
		if(sccr & PPC800_SCCR_TBS) {
			/* GCLK2 */
			cycles_freq = freq / 16;
		} else {
			/* oscclk clock */
			cycles_freq = oscclk_freq / ((mult > 2) ? 4 : 16);
		}
	}
	//
	// Note: the following assumes that we're using the PIT for the timer. If
	// the decrementer is used instead, cycles_freq should be used for
	// the input frequency.
	//
	freq = (sccr & PPC800_SCCR_RTSEL) ? extclk_freq : extal_freq;
	div = (sccr & PPC800_SCCR_RTDIV) ? 512 : 4; 
	if(timer_freq == 0) timer_freq = freq / div;
}

__SRCVERSION("ppc800_set_clock_freqs.c $Rev: 580669 $");
