/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include <ppc/8260cpm.h>

/*
 * ppc8260_pit_init_qtime(void)
 *	Initialize Neutrino time data structures for use with the PPC8260 PIT
 *  unit.
 * Note: pit and tmcnt use brg0 as its default clock.
 */

static unsigned
timer_start_pit(void) {
	return(PPC8260_IMMR(PITR,16));
}

static unsigned
timer_diff_pit(unsigned start) {
	unsigned	now = PPC8260_IMMR(PITR,16);

	if(now > start) start += 0x10000;
	return(start - now);
}

static const struct callout_slot timer_callouts[] = {
	{ CALLOUT_SLOT( timer_load, _8260 ) },
	{ CALLOUT_SLOT( timer_value, _8260 ) },
	{ CALLOUT_SLOT( timer_reload, _8260 ) },
};

void
ppc8260_pit_init_qtime(void) {
	struct qtime_entry *qtime = alloc_qtime();

	qtime->intr = PPC8260_INTR_GEN_PIT;

	/* Set brg0: 4M */
	PPC8260_IMMR(BRGC1, 32) = PPC8260CPM_BRG_RST;
	PPC8260_IMMR(BRGC1, 32) = PPC8260CPM_BRG_EN | PPC8260CPM_BRG_EXTC(0) | PPC8260CPM_BRG_CD(timer_freq/(4*1024*1024));

	timer_freq = PPC8260_CLOCK_TIMERSCLK;

	PPC8260_IMMR(PISCR,16) = PPC8260_PISCR_PIE | PPC8260_PISCR_PTE;

	/* Set the PIT for maximum countdown */
	PPC8260_IMMR(PITC,16) = 0xffff;

	qtime->cycles_per_sec = cycles_freq;
	invert_timer_freq(qtime, timer_freq);

	add_callout_array(timer_callouts, sizeof(timer_callouts));

	timer_start = timer_start_pit;
	timer_diff  = timer_diff_pit;
}

__SRCVERSION("ppc8260_pit_init_qtime.c $Rev: 580669 $");
