/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

/* Current list of MPC85XX covered by this module
 *
 * Name       SVR           Num of DDR     Device Identity (DI from DIDCAR)
 * MPC8533E   0x803c        1               N/A
 * MPC8533    0x8034        1               N/A
 * MPC8540    0x8030        1               0x0002
 * MPC8541E   0x807A        1               N/A
 * MPC8541    0x8072        1               N/A
 * MPC8543E   0x803A        1               0x0014
 * MPC8543    0x8033        1               0x0015
 * MPC8544E   0x803C        1               N/A
 * MPC8544    0x8034        1               N/A
 * MPC8545E   0x8039        1               0x0019
 * MPC8545    0x8031        1               0x001A
 * MPC8547E   0x8039        1               0x0018
 * MPC8548E   0x8039        1               0x0012
 * MPC8548    0x8031        1               0x0013
 * MPC8555E   0x8079        1               N/A
 * MPC8555    0x8071        1               N/A
 * MPC8560    0x8070        1               0x0003
 * MPC8567E   0x807D        1               0x0020
 * MPC8567    0x8075        1               0x0021
 * MPC8568E   0x807D        1               0x0020
 * MPC8568    0x8075        1               0x0021
 * MPC8572E   0x80E8        2               0x0040
 * MPC8572    0x80E0        2               0x0041
 */


#include "startup.h"
#include <ppc/85xxcpu.h>

#define CCSR_OFF_CSn_DNDS(offset, n)        (offset + (n)*8)
#define CCSR_OFF_CSn_CONFIG(offset, n)      (offset + 0x80 + (n)*4)

 

/*
 * ppc85xx_init_raminfo_extended.c
 * Tell syspage about our RAM configuration
 */

void 
ppc85xx_init_raminfo_extended(unsigned immr_offset) {
    paddr64_t   start;
    paddr64_t   end;
    uintptr_t   base;
    unsigned    cs;
    unsigned    bounds;

    /* Initialize the additional controller currently available on the new 
     * MPC85xx (such as MPC8572). Since we can't predict if new controller will
     * be added or if the offset within the IMMR space will remain the same.
     * We keep this function as generic as possible and it will get invoke for 
     * any additionnal memory controller add by the BSP startup ram 
     * initialization function.
     */
     
    /* Go through all device attach to this controller */
    base = startup_io_map(MEG(1), immr_paddr);
    for(cs = 0; cs < 4; cs++) {
		unsigned	config;
       
		config = in32(base + CCSR_OFF_CSn_CONFIG(immr_offset, cs));
        if(config & PPC85xx_CSn_CONFIG_CS_EN) {
            bounds = in32(base + CCSR_OFF_CSn_DNDS(immr_offset, cs));
            start = (paddr64_t)(bounds & PPC85xx_CSn_BNDS_SA_MASK)
                        << (24 - PPC85xx_CSn_BNDS_SA_SHIFT);
            end = (paddr64_t)(bounds & PPC85xx_CSn_BNDS_EA_MASK)
                        << (24 - PPC85xx_CSn_BNDS_EA_SHIFT);

			if(config & PPC85xx_CSn_CONFIG_CS_INTLV_EN) {
				// Memory is interleaved
				if((end - start + MEG(16)) > GIG(2)){
					if(start < (GIG(4) - 1)){
						crash("Please don't allocate RAM between 2Gig and 4Gig\n");
					}
				}
			}

			// Don't add memory above 4G if we're using the 32-bit paddr_t
			// version of the library.
			#define MAX_PADDR (~(paddr_t)0 & ~(paddr_t)MEG(16))
			if(start <= MAX_PADDR) {
				if((end > MAX_PADDR)) {
					end = MAX_PADDR;
				}
				kprintf("Adding   %um ram at 0x%P for cntrlr 0x%P, cs%d\n",
				       (uint32_t)((end-start+MEG(16))/(1024*1024)), start, immr_paddr+immr_offset, cs);
				add_ram(start, end - start + MEG(16));
			} else {
				kprintf("Ignoring %um ram at 0x%P for cntrlr 0x%P, cs%d (>4Gb with 32-bit phys addrs)\n",
				       (uint32_t)((end-start+MEG(16))/(1024*1024)), start, immr_paddr+immr_offset, cs);
			}
        }
    }
    startup_io_unmap(base);
}

