/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

/*
 * ppc_booke_dec_init_qtime(void)
 *	Initialize Neutrino time data structures for use with the PPC Book E
 *  decrementer register.
 */

static unsigned
timer_start_dec(void) {
	return(get_spr(PPC_SPR_DEC));
}

static unsigned
timer_diff_dec(unsigned start) {
	return(start - get_spr(PPC_SPR_DEC));
}

static const struct callout_slot timer_callouts[] = {
	{ CALLOUT_SLOT(timer_load, _dec_booke) },
	{ CALLOUT_SLOT(timer_value, _dec_booke) },
};

void
ppc_booke_dec_init_qtime(void) {
	struct qtime_entry *qtime = alloc_qtime();

	qtime->intr = PPC_INTR_DECREMENTER;

	qtime->cycles_per_sec = cycles_freq;
	invert_timer_freq(qtime, timer_freq);

	add_callout_array(timer_callouts, sizeof(timer_callouts));
	if(lsp.syspage.p->num_cpu > 1) {
		qtime->flags |= QTIME_FLAG_TIMER_ON_CPU0;
	}

	timer_start = timer_start_dec;
	timer_diff  = timer_diff_dec;
}

__SRCVERSION("ppc_booke_dec_init_qtime.c $Rev: 580669 $");
