/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

static void inline
loop(unsigned count) {
	asm volatile( "mtctr	%0; 1: bdnz 1b" :: "r" (count) );
}

unsigned long
ppc_cpuspeed() {
	unsigned	speed;
	unsigned	adjust;
	int			change;
	unsigned	start;
	unsigned	diff;
	unsigned	one_ms;
	unsigned	upwards;

	//
	// Calculate CPU speed
	//
	upwards = 0;
	adjust = 1000;
	speed = 1000;
	change = 1000;
	one_ms = timer_ns2tick(1000000000UL/1000);
	for( ;; ) {
		start = timer_start();
		loop(speed);
		diff = timer_diff(start);
		if(diff <= one_ms) {
			if(change < 0) adjust /= 2;
			change = adjust;
			if(diff < one_ms && change == 0) change = 1;
			++upwards;
		} else {
			if(change > 0) adjust /= 2;
			change = -adjust;
			upwards = 0;
		}
		if(change == 0) break;
		if((diff == 0) && (upwards > 100)) {
			crash("Timer not running. Possible configuration problem?\n");
		}
		speed += change;
	}
	/* Multiply by 2.01 and divide by 1000 to get MHz. */
	/* 2.01 is composed of 2.00 => cycles per loop, 0.01 => error correction */
	return(((speed*2)+(speed/100)) / 1000);
}

__SRCVERSION("ppc_cpuspeed.c $Rev: 580669 $");
