/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

// Initialise the CoreNet Platform caches on P40xx family chips. 

#include "startup.h"
#include <ppc/p4080cpu.h>

void 
ppcp40xx_init_l3_cache(void) {
	uint32_t l3c;
	uintptr_t l3c_vloc;
	
	/* Enable the CoreNet Platform Cache 0, after invalidating and unlocking */  
	l3c_vloc = startup_io_map(4, (immr_paddr + PPCP4080_CCSR_OFF_CPCSR0));
	
	if(!(in32(l3c_vloc) & PPCP4080_CPC_EN)){
	   	ppc_eieio();
		ppc_isync();
		l3c = in32(l3c_vloc);
		out32(l3c_vloc, l3c | PPCP4080_CPC_FI | PPCP4080_CPC_LFC);
		while(in32(l3c_vloc) & PPCP4080_CPC_FI){
		// do nothing while we wait for FI bit to clear
		}
	   	ppc_eieio();

	   	ppc_eieio();
		ppc_isync();
		l3c = in32(l3c_vloc);
		out32(l3c_vloc, l3c | PPCP4080_CPC_EN | PPCP4080_CPC_PE);
		in32(l3c_vloc);
	   	ppc_eieio();
	} else
		kprintf("L3 cache0 already enabled by bootloader, skipping init.\n");
	startup_io_unmap(l3c_vloc);

	/* Enable the CoreNet Platform Cache 1, after invalidating and unlocking */ 
	l3c_vloc = startup_io_map(4, (immr_paddr + PPCP4080_CCSR_OFF_CPCSR1));
	
	if(!(in32(l3c_vloc) & PPCP4080_CPC_EN)){
	   	ppc_eieio();
		ppc_isync();
		l3c = in32(l3c_vloc);
		out32(l3c_vloc, l3c | PPCP4080_CPC_FI | PPCP4080_CPC_LFC);
		while(in32(l3c_vloc) & PPCP4080_CPC_FI){
		// do nothing while we wait for FI bit to clear
		}
	   	ppc_eieio();

	   	ppc_eieio();
		ppc_isync();
		l3c = in32(l3c_vloc);
		out32(l3c_vloc, l3c | PPCP4080_CPC_EN | PPCP4080_CPC_PE);
		in32(l3c_vloc);
	   	ppc_eieio();
	} else
		kprintf("L3 cache1 already enabled by bootloader, skipping init.\n");
	startup_io_unmap(l3c_vloc);
}

__SRCVERSION("ppcp40xx_init_l3_cache.c $Rev: $");
