/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"
#include <ppc/85xxcpu.h>

#define CCSR_OFF_CSn_DNDS(offset, n)        (offset + (n)*8)
#define CCSR_OFF_CSn_CONFIG(offset, n)      (offset + 0x80 + (n)*4)

//#define RAMINFO_DEBUG 

/*
 * ppcp40xx_init_raminfo_extended.c
 * Tell syspage about our RAM configuration
 */

void 
ppcp40xx_init_raminfo_extended(unsigned immr_offset) {
    paddr64_t   start;
    paddr64_t   end;
    uintptr_t   base;
    unsigned    cs;
    unsigned    bounds;

    /* Initialize the additional controller currently available on the P4080 which is
     * similar to the MPC85xx (such as MPC8572). Since we can't predict if new controller will
     * be added or if the offset within the IMMR space will remain the same.
     * We keep this function as generic as possible and it will get invoke for 
     * any additionnal memory controller add by the BSP startup ram 
     * initialization function.
     */
     
    /* Go through all device attach to this controller */
    base = startup_io_map(MEG(1), immr_paddr);
    for(cs = 0; cs < 4; cs++) {
		unsigned	config;
       
		config = in32(base + CCSR_OFF_CSn_CONFIG(immr_offset, cs));
        if(config & PPC85xx_CSn_CONFIG_CS_EN) {
            bounds = in32(base + CCSR_OFF_CSn_DNDS(immr_offset, cs));
            start = (paddr64_t)(bounds & PPC85xx_CSn_BNDS_SA_MASK)
                        << (24 - PPC85xx_CSn_BNDS_SA_SHIFT);
            end = (paddr64_t)(bounds & PPC85xx_CSn_BNDS_EA_MASK)
                        << (24 - PPC85xx_CSn_BNDS_EA_SHIFT);
#ifdef RAMINFO_DEBUG
			if(config & PPC85xx_CSn_CONFIG_CS_INTLV_EN) {
				kprintf("Config 0x%x for cntrlr 0x%P, cs%d (interleave)\n", config, immr_paddr+immr_offset, cs);
			} else
				kprintf("Config 0x%x for cntrlr 0x%P, cs%d (nointerleave)\n", config, immr_paddr+immr_offset, cs);
#endif

			if(end == start) continue;

			// Don't add memory above 4G if we're using the 32-bit paddr_t
			// version of the library.
			#define MAX_PADDR (~(paddr_t)0 & ~(paddr_t)MEG(16))
			if(start < MAX_PADDR) {
				if((end > MAX_PADDR)) {
					end = MAX_PADDR;
				}
				kprintf("Adding   %um ram at 0x%P for cntrlr 0x%P, cs%d%s\n",
				       (uint32_t)((end-start+MEG(16))/(1024*1024)), start, immr_paddr+immr_offset, cs,
				       (config & PPC85xx_CSn_CONFIG_CS_INTLV_EN) ? " (interleaved)" : "");
				add_ram(start, end - start + MEG(16));
			} else {
				kprintf("Ignoring %um ram at 0x%P for cntrlr 0x%P, cs%d (>4Gb with 32-bit phys addrs)\n",
				       (uint32_t)((end-start+MEG(16))/(1024*1024)), start, immr_paddr+immr_offset, cs);
			}
        }
    }
    startup_io_unmap(base);
}

