/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */

/*******************************************************************************
 * support routines for accessing PPCP40xx LAW registers
 * 
 * This file is only compiled when included from an ppcp40xx specific
 * implementation (see ppcp4080_law.c) which set up the
 * appropriate defines
*/

#ifdef	LAW_FUNC

#define PPCP40xx_REGOFF_LAWBARH(n)	    	(0xC00U + (0x10 * (n % LAWDEFN_NUM_LAW_REGS)))
#define PPCP40xx_REGOFF_LAWBARL(n)	    	(0xC04U + (0x10 * (n % LAWDEFN_NUM_LAW_REGS)))

/*
 * =============================================================================
 *                             G E T   A P I's
 * =============================================================================
*/

/*******************************************************************************
 * PPCP40XX_law_get_addr
 * 
 * Returns: the window base address for the LAW register <law_id>
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
 *
 * IMPORTANT 
 * Since the smallest window size is 4KB, the least significant 12 bits are fixed.
 * The LAWBAR contains the most significant 24 bits which together represent the
 * 36 bit internal address. This is why we have a 12 bit shift 
*/
uint64_t LAW_FUNC(law_get_addr)(unsigned law_id)
{
	volatile uint64_t reg, regh, regl;
	reg = regh = regl = 0;
	MAP_REG;

	if (law_id >= LAWDEFN_NUM_LAW_REGS) return 0;
	regh = (IN32(PPCP40xx_REGOFF_LAWBARH(law_id)) & LAWDEFN_LAWBARH_ADDR_MASK);
	regl = (IN32(PPCP40xx_REGOFF_LAWBARL(law_id)) & LAWDEFN_LAWBARL_ADDR_MASK);
	
	reg = ((regh << 32) | (regl));
	return (uint64_t)(reg);
}

/*******************************************************************************
 * PPCP40XX_law_set_addr
 * 
 * Set the address of the LAW register identified by <law_id> to <addr>.
 * 
 * Returns: the previous window base address for the LAW register <law_id>
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
 * 
 * IMPORTANT 
 * Since the smallest window size is 4KB, the least significant 12 bits are fixed.
 * The LAWBAR contains the most significant 24 bits which together represent the
 * 36 bit internal address. This is why we have a 12 bit shift 
*/
uint64_t LAW_FUNC(law_set_addr)(unsigned law_id, uint64_t addr)
{
	uint64_t prev_addr = LAW_FUNC(law_get_addr)(law_id);
	uint32_t regh, regl;
	regh = regl = 0;

	if (law_id >= LAWDEFN_NUM_LAW_REGS) return 0;
	if (prev_addr != addr)
	{
		unsigned is_enabled = LAW_FUNC(law_is_enabled)(law_id);

		LAW_FUNC(law_disable)(law_id);
		regl = ((addr ) & LAWDEFN_LAWBARL_ADDR_MASK);
		regh = ((addr >> 32) & LAWDEFN_LAWBARH_ADDR_MASK);
		OUT32(PPCP40xx_REGOFF_LAWBARH(law_id), regh);
		OUT32(PPCP40xx_REGOFF_LAWBARL(law_id), regl);
		if (is_enabled) LAW_FUNC(law_enable)(law_id);
	}
	return prev_addr;
}

#endif	/* LAW_FUNC */
