/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */

#include "startup.h"
#include "ppcp40xx_lbc.h"

/* Configuring the eLBC for different memories */
extern paddr_t immr_paddr;

#define PPCP40xx_ELBC_NUM_CHIP_SELECTS  8
#define PPCP40xx_REGOFF_ELBC_BR(n)	    (0x124000U + (0x8 * (n % PPCP40xx_ELBC_NUM_CHIP_SELECTS)))
#define PPCP40xx_REGOFF_ELBC_OR(n)		(0x124004U + (0x8 * (n % PPCP40xx_ELBC_NUM_CHIP_SELECTS)))

/* Reset eLBC chip selects before configuring them */
void ppcp40xx_configure_elbc_reset(void)
{
	int chip_select = 0;
	uintptr_t	base, elbc_base_reg_addr, elbc_options_reg_addr;
	
	base = startup_io_map(MEG(2), immr_paddr);
	
	for(; chip_select < PPCP40xx_ELBC_NUM_CHIP_SELECTS; chip_select++)
	{
		elbc_base_reg_addr = (base + PPCP40xx_REGOFF_ELBC_BR(chip_select));
	    out32(elbc_base_reg_addr, PPCP40xx_eLBC_BR_DEFAULT_VALUE);
		elbc_options_reg_addr = (base + PPCP40xx_REGOFF_ELBC_OR(chip_select));
		out32(elbc_options_reg_addr, PPCP40xx_eLBC_OR_DEFAULT_VALUE);
	}
	
	startup_io_unmap(base);
}

/* Configure the eLBC chip selects to connect to devices like NOR, NAND etc. */
void ppcp40xx_configure_elbc(eLBC_device_t device, uint32_t base_addr, eLBC_port_size_t port_size, 
							eLBC_machine_select_t machine_select, uint8_t valid, uint8_t bank_size, 
							uint8_t boot_src, eLBC_page_size_t page_size, uint8_t chip_select)
{
	uint8_t i, size;
    uint32_t br_value, or_value, mask;
	uintptr_t	base, elbc_base_reg_addr, elbc_options_reg_addr;	

	br_value = PPCP40xx_eLBC_BR_DEFAULT_VALUE;
	or_value = PPCP40xx_eLBC_OR_DEFAULT_VALUE;
	
	base = startup_io_map(MEG(2), immr_paddr);

    size = 	bank_size;
	
	for (i = 0; i < size; i++)
	{
	    mask = (1 << (31-i));
		or_value |= mask;
	}

	if (device != eLBC_DEVICE_NAND)
	{
        if (boot_src)
        {
            or_value |= PPCP40xx_eLBC_OR_GPCM_BOOT_SRC;
        }
        else
        {
            or_value |= PPCP40xx_eLBC_OR_GPCM_NOT_BOOT_SRC;
        }
	}
	
	if (device == eLBC_DEVICE_NAND)
	{
	    switch(page_size)
	    {
	        case SMALL_PAGE:
		        or_value |= PPCP40xx_eLBC_OR_FCM_SMALL_PAGE;
			    break;
		    case LARGE_PAGE:
                or_value |= PPCP40xx_eLBC_OR_FCM_LARGE_PAGE;
			    break;
            default:
                break;
	    }
	}
	
    br_value |= (PPCP40xx_eLBC_BR_PHYS_ADDR(base_addr));
	
	switch(machine_select)
	{
	    case eLBC_MACHINE_GPCM:
		    br_value |= PPCP40xx_eLBC_BR_MSEL_GPCM;
		    break;
		case eLBC_MACHINE_FCM:
		    br_value |= PPCP40xx_eLBC_BR_MSEL_FCM;		
            break;
        case eLBC_MACHINE_UPMA:
		    br_value |= PPCP40xx_eLBC_BR_MSEL_UPMA;		
            break;
        case eLBC_MACHINE_UPMB:
		    br_value |= PPCP40xx_eLBC_BR_MSEL_UPMB;		
            break;
        case eLBC_MACHINE_UPMC:
		    br_value |= PPCP40xx_eLBC_BR_MSEL_UPMC;		
            break;			
        default:
            crash("What Machine is this ?\n");
            break;			
	}
	
	switch(port_size)
	{
	    case BYTE_LENGTH:
		    br_value |= PPCP40xx_eLBC_BR_PS_8;
			break;
		case WORD_LENGTH:
            br_value |= PPCP40xx_eLBC_BR_PS_16;
			break;
        default:
            crash("Undefined Port size\n");
            break;
	}
 
    if(valid)
        br_value |= PPCP40xx_eLBC_BR_V;
		
	elbc_base_reg_addr = (base + PPCP40xx_REGOFF_ELBC_BR(chip_select));
    out32(elbc_base_reg_addr, br_value);
	
	elbc_options_reg_addr = (base + PPCP40xx_REGOFF_ELBC_OR(chip_select));
	out32(elbc_options_reg_addr, or_value);
	
	startup_io_unmap(base);	
}

__SRCVERSION( "$URL$ $Rev$" );
