/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */

/*
 * Local Bus Definitions
 */

#define PPCP40xx_eLBC_BR_PHYS_ADDR(x)                  (x & 0xffff8000)
 
#define PPCP40xx_eLBC_BR_PS_8                          0x00000800	/* Port Size 8 bit */
#define PPCP40xx_eLBC_BR_PS_16                         0x00001000	/* Port Size 16 bit */
#define PPCP40xx_eLBC_BR_V                             0x00000001    /* Bank is Valid */
#define PPCP40xx_eLBC_BR_MSEL_GPCM                     0x00000000	/* GPCM */
#define PPCP40xx_eLBC_BR_MSEL_FCM                      0x00000020	/* FCM */
#define PPCP40xx_eLBC_BR_MSEL_UPMA                     0x00000080	/* UPMA */
#define PPCP40xx_eLBC_BR_MSEL_UPMB                     0x000000A0	/* UPMB */
#define PPCP40xx_eLBC_BR_MSEL_UPMC                     0x000000C0	/* UPMC */
#define PPCP40xx_eLBC_BR_DEFAULT_VALUE                 0x00000000    /* Reset Value */

#define PPCP40xx_eLBC_OR_DEFAULT_VALUE                 0x00000000    /* Reset Value */
#define PPCP40xx_eLBC_OR_GPCM_BOOT_SRC                 0x00000FF7
#define PPCP40xx_eLBC_OR_GPCM_NOT_BOOT_SRC             0x00000F07
#define PPCP40xx_eLBC_OR_FCM_SMALL_PAGE                0x000003AE
#define PPCP40xx_eLBC_OR_FCM_LARGE_PAGE                0x000007AE

typedef enum {
	eLBC_DEVICE_NAND = 0,		
	eLBC_DEVICE_NOR,
	eLBC_DEVICE_PIXIS,
	eLBC_DEVICE_PROMJET
} eLBC_device_t;

typedef enum {
	eLBC_MACHINE_GPCM = 0,		
	eLBC_MACHINE_FCM = 1,
	eLBC_MACHINE_UPMA = 4,
	eLBC_MACHINE_UPMB = 5,
	eLBC_MACHINE_UPMC = 6
} eLBC_machine_select_t;

typedef enum {
	BYTE_LENGTH = 1,		
    WORD_LENGTH
} eLBC_port_size_t;

typedef enum {
	SMALL_PAGE = 0,		
    LARGE_PAGE
} eLBC_page_size_t;

/* Reset eLBC chip selects before configuring them */
void ppcp40xx_configure_elbc_reset(void);

/**
* @brief                              Configures the eLBC chip selects to 
*                                     connect to specific devices on the local bus (eg: NAND, NOR, PROMJET, PIXIS etc.)
*
* @param device                       NAND/NOR/PIXIS/PROMJET
* @param base_addr                    Base Address of the Device
* @param port_size                    8 bit/16 bit (FCM does not support 16 bit)
* @param machine_select               GPCM (for NOR/PIXIS/PROMJET), FCM (for NAND), UPM (for user specific devices)
* @param valid                        0 if bank is not valid, 1 if bank is valid
* @param bank_size                    size of the bank (0 represents 4GB, 1 represents 2GB, ....., 
*                                     17 represents 32KB which is the minimum size)
* @param boot_src                     whether eLBC is used as boot source or not 
*                                     (if GPCM is used as boot source, the reset value of OR is 0x00000FF7)
* @param page_size                    0 if the NAND device supports small pages (512 byte pages), 
*                                     1 if the NAND device supports large pages (1KB or 2KB pages)
* @param chip_select                  chip select for the bank
*
* @return NULL
*/

void ppcp40xx_configure_elbc(eLBC_device_t device, uint32_t base_addr, eLBC_port_size_t port_size, 
							eLBC_machine_select_t machine_select, uint8_t valid, uint8_t bank_size, 
							uint8_t boot_src, eLBC_page_size_t page_size, uint8_t chip_select);

__SRCVERSION( "$URL$ $Rev$" );
							
