/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */

#include "startup.h"


#define iccci(addr)          	\
	__asm__ __volatile__(           \
	".ifdef PPC_CPUOP_ENABLED;"     \
	".cpu 405;"                     \
	".else;"                        \
	".machine \"405\";"             \
	".endif;"                       \
	"iccci 0,%0" 	                \
	:: "r" (addr) ); 

#define dccci(addr)          	\
	__asm__ __volatile__(           \
	".ifdef PPC_CPUOP_ENABLED;"     \
	".cpu 405;"                     \
	".else;"                        \
	".machine \"405\";"             \
	".endif;"                       \
	"dccci 0,%0" 	                \
	:: "r" (addr) ); 


static paddr_t	top_ram = MEG(128)-1;

static void
ppc400_cache_watcher(paddr_t start, paddr_t size) {
	// more ram has been added, see if we need to turn on more
	// bits in the ICCR and DCCR registers
	if(size != 0) {
		paddr_t		new = start + size - 1;
		if(new > top_ram) {
			unsigned	bits = ~0;

			for( ;; ) {
				bits >>= 1;
				if(new <= MEG(128)-1) break;
				new -= MEG(128);
			}
			set_spr(PPC400_SPR_ICCR, ~bits);
			set_spr(PPC400_SPR_DCCR, ~bits);
			top_ram = new;
		}
	}
}
 

void
ppcv_cpuconfig1_400(int cpu) {
	unsigned	ccr;
	unsigned	i;
	unsigned	lsize;

	/* Put the exception table at the bottom of memory. */
	set_spr(PPC400_SPR_EVPR, 0);

	/* 
	   Make sure that the lower 128M region is cacheable.
	*/   
	lsize = 1 << ppcv->cache_lsize;

	/* I cache */
	ccr = get_spr(PPC400_SPR_ICCR);
	if(ccr == 0) {
		// Cache isn't enabled, invalidate all the lines first
		for(i = 0; i < ppcv->icache_lines; ++i) {
			iccci(i << lsize);
		}
		ppc_sync();
	}
	if(!(ccr & PPC400_ICCR_S0)) {
		set_spr(PPC400_SPR_ICCR, ccr | PPC400_ICCR_S0);
	}

	/* D cache */
	ccr = get_spr(PPC400_SPR_DCCR);
	if(ccr == 0) {
		// Cache isn't enabled, invalidate all the lines first
		// Note: the 400's DCCCI instruction actually invaldates both
		// ways, so we're actually doing twice as many as we need to,
		// but better safe than sorry in case it changes in the future.
		for(i = 0; i < ppcv->dcache_lines; ++i) {
			dccci(i << lsize);
		}
		ppc_sync();
	}
	if(!(ccr & PPC400_DCCR_S0)) {
		set_spr(PPC400_SPR_DCCR, ccr | PPC400_DCCR_S0);
	}
	watch_add_ram(ppc400_cache_watcher);
	set_spr(PPC400_SPR_DCWR, 0);
}
