/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include <ppc/e500cpu.h>

void
ppcv_cpuconfig1_e500_work(int cpu, int hid0_extra, int hid1_extra) {
	unsigned	npids;
	unsigned	hid0;
	unsigned	l1c;

	//Init the HID registers
	hid0 = get_spr(PPCE500_SPR_HID0) & PPCE500_HID0_SEL_TBCLK;
	set_spr(PPCE500_SPR_HID0, hid0
				| PPCE500_HID0_EMCP
				| PPCE500_HID0_DOZE
				| PPCE500_HID0_TBEN
				| hid0_extra);
	// e500 and e500v2 use the HID1 register, so pass the configuration
	// in hid1_extra.  The e500mc doesn't have a HID1 register, so passes
	// hid1_extra=0
	if (hid1_extra) {
		set_spr(PPCE500_SPR_HID1, get_spr(PPCE500_SPR_HID1) | hid1_extra);
	}

	//Init the L1 caches
	l1c = get_spr(PPCE500_SPR_L1CSR0);
	if(!(l1c & PPCE500_L1CSR0_CE)) {
		// flash invalidate data cache
		set_spr(PPCE500_SPR_L1CSR0, l1c | PPCE500_L1CSR0_CFI);
		do {
		} while(get_spr(PPCE500_SPR_L1CSR0) & PPCE500_L1CSR0_CFI);
		set_spr(PPCE500_SPR_L1CSR0, l1c | PPCE500_L1CSR0_CE);
	}
	l1c = get_spr(PPCE500_SPR_L1CSR1);
	if(!(l1c & PPCE500_L1CSR1_ICE)) {
		// flash invalidate instruction cache
		set_spr(PPCE500_SPR_L1CSR1, l1c | PPCE500_L1CSR1_ICFI);
		do {
		} while(get_spr(PPCE500_SPR_L1CSR1) & PPCE500_L1CSR1_ICFI);
		set_spr(PPCE500_SPR_L1CSR1, l1c | PPCE500_L1CSR1_ICE);
	}

	set_spr(PPCBKEM_SPR_MAS4, PPCBKEM_MAS4_MD
			| (PPCBKE_TLB_SIZE_4K << PPCBKEM_MAS4_TSIZED_SHIFT));
	npids = (get_spr(PPCBKEM_SPR_MMUCFG) & PPCBKEM_MMUCFG_NPIDS_MASK) >> PPCBKEM_MMUCFG_NPIDS_SHIFT;
	switch(npids) {
	default:
	case 3:
		set_spr(PPCBKEM_SPR_PID2, 0);
		// fall through
	case 2:
		set_spr(PPCBKEM_SPR_PID1, 0);
		// fall through
	case 1:
		// e500mc doesn't have a PID1 or PID2 register
		break;
	}
	set_spr(PPCE500_SPR_BUCSR, PPCE500_BUCSR_BPEN);

	ppcv_cpuconfig1_booke(cpu);
}



void ppcv_cpuconfig1_e500(int cpu) {
	ppcv_cpuconfig1_e500_work(cpu, 0, PPCE500_HID1_MSHARS|PPCE500_HID1_SSHAR);
	// Turn on the SHAREND bit
	set_spr(PPCBKEM_SPR_MAS4, PPCBKEM_MAS4_SHAREND|PPCBKEM_MAS4_MD
			| (PPCBKE_TLB_SIZE_4K << PPCBKEM_MAS4_TSIZED_SHIFT));
}

void
ppcv_cpuconfig2_e500(int cpu) {
	ppcv_cpuconfig2_booke(cpu);
	if (lsp.syspage.p->num_cpu > 1) {
		// Need to enable cache/TLB broadcast for SMP
		set_spr(PPCE500_SPR_HID1, get_spr(PPCE500_SPR_HID1) | PPCE500_HID1_ABE);
	}
}


void ppcv_cpuconfig1_e500v2(int cpu) {
	ppcv_cpuconfig1_e500_work(cpu, PPCE500_HID0_EN_MAS7_UPDATE, 0);
}

void ppcv_cpuconfig2_e500v2(int cpu) {
	ppcv_cpuconfig2_e500(cpu);
}


void ppcv_cpuconfig1_e500mc(int cpu) {
	ppcv_cpuconfig1_e500_work(cpu, PPCE500_HID0_EN_MAS7_UPDATE, 0);

	if(!(get_spr(PPCE500_SPR_L2CSR0) & PPCE500_L2CSR0_L2E)) {
		/* Set the Backside L2 Cache for the e500mc cores */
		set_spr(PPCE500_SPR_L2CSR0, PPCE500_L2CSR0_L2FI);
		while(get_spr(PPCE500_SPR_L2CSR0) & PPCE500_L2CSR0_L2FI) {
			// do nothing while we wait for L2FI bit to clear
		}
		set_spr(PPCE500_SPR_L2CSR0, PPCE500_L2CSR0_L2E);
		while(!(get_spr(PPCE500_SPR_L2CSR0) & PPCE500_L2CSR0_L2E)) {
			// Loop here if L2CSR0 is not enabled
		}
	}
}


void ppcv_cpuconfig2_e500mc(int cpu) {
	ppcv_cpuconfig2_booke(cpu);
	// The e500mc doesn't use the HID1.ABE register of the e500v2 -- it does
	// the equivalent of the address broadcast at all times.
}

__SRCVERSION("ppcv_cpuconfig_e500.c $Rev: 580669 $");
