/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

// Temp definition until all the header files get caught up
#ifndef PPC700_SPR_HID0_HBE
	#define	PPC700_SPR_HID0_HBE		_ONEBIT32B(8)
#endif


#define FIRST_VADDR	(GIG(1) - MEG(256))

struct bat_info {
		uintptr_t		vaddr;
		paddr32_t		paddr;
		unsigned		size;
		unsigned 		size_needed;
		unsigned short	flags;
		unsigned short	ref;
};

// Can't use PROT_CALLOUT because it's too big to fit in 'flags' field
#define MYCALLOUT_FLAG		1

static struct bat_info	bat[8] = {
	// Start with one entry for kernel 1-1 area.  We don't know how big RAM is at
	// this point, so we'll start small and adjust later.  Note that we use the
	// PROT_EXEC flag to make sure we get an IBAT as well as a BAT.  We also use
	// the MYCALLOUT_FLAG to make sure the mapping survives transfer to procnto,
	// even though this isn't really a callout mapping.
	{ 0, 0, MEG(256), MEG(1), PROT_READ|PROT_WRITE|PROT_EXEC|MYCALLOUT_FLAG, 1} // initial 1-1 mapping area
};

static unsigned num_bats = 4;

static void
invalidate_device_bats(void) {
	set_spr(PPC_SPR_IBAT1U, 0);
	set_spr(PPC_SPR_IBAT2U, 0);
	set_spr(PPC_SPR_IBAT3U, 0);
	set_spr(PPC_SPR_DBAT1U, 0);
	set_spr(PPC_SPR_DBAT2U, 0);
	set_spr(PPC_SPR_DBAT3U, 0);
	if(num_bats > 4) {
		set_spr(PPC_SPR_IBAT4U, 0);
		set_spr(PPC_SPR_IBAT5U, 0);
		set_spr(PPC_SPR_IBAT6U, 0);
		set_spr(PPC_SPR_IBAT7U, 0);
		set_spr(PPC_SPR_DBAT4U, 0);
		set_spr(PPC_SPR_DBAT5U, 0);
		set_spr(PPC_SPR_DBAT6U, 0);
		set_spr(PPC_SPR_DBAT7U, 0);
	}
	ppc_isync();
}

#define BAT_FLAG_VALID	0x01
#define BAT_FLAG_CFG2	0x02

static void
load_bat(unsigned bi, unsigned ti, int flags) {
	unsigned		up;
	unsigned		lo;
	struct bat_info	*bp;
	unsigned		adj;

	bp = &bat[ti];

	up = bp->vaddr & PPC_BATU_BEPI_MASK;
	if(flags & BAT_FLAG_VALID) {
		up |= PPC_BATU_VS;
		if(bp->flags & PROT_USER) up |= PPC_BATU_VP;
	}
	up |= ((bp->size-1) >> (29-14)) & PPC_BATU_BL_MASK;

	lo = (bp->paddr & PPC_BATL_BRPN_MASK) | PPC_BATL_PP_RW;
	if(bp->flags & PROT_NOCACHE) {
		lo |= PPC_BATL_I | PPC_BATL_G;
	} else {
		lo |= PPC_BATL_M;
		// If we're not transfering to the bootstraps, we want a
		// write-through cache for consistency in an SMP system
		if(!(flags & BAT_FLAG_CFG2)) lo |= PPC_BATL_W;
	}
	adj = bi * 2;
	if(bi >= 4) {
		// massage the adjustment for the 4 high order bats
		adj += PPC_SPR_IBAT4U - PPC_SPR_IBAT0U - 8;
	}
	if (bp->flags & PROT_EXEC) {
		set_spr_indirect(PPC_SPR_IBAT0L + adj, lo & ~(PPC_BATL_G|PPC_BATL_W));
		set_spr_indirect(PPC_SPR_IBAT0U + adj, up);
	} else {
		set_spr_indirect(PPC_SPR_IBAT0L + adj, 0);
		set_spr_indirect(PPC_SPR_IBAT0U + adj, 0);
	}
	set_spr_indirect(PPC_SPR_DBAT0L + adj, lo);
	set_spr_indirect(PPC_SPR_DBAT0U + adj, up);
	ppc_isync(); ppc_sync();
}
		
void
ppcv_mmuconfig1_600(int cpu) {
	unsigned	i;
	
	if(ppcv->cpu_flags & PPC_CPU_EXTRA_BAT) {
		num_bats = 8;
	}
	// Invalidate BATs
	set_spr(PPC_SPR_IBAT0U, 0);
	set_spr(PPC_SPR_DBAT0U, 0);
	invalidate_device_bats();

	// clear segment registers
	for(i = 0; i < 16; i++) {
		set_sreg(i << 28, 0);
	}

	//Invalidate the TLB's
	for(i = 0; i < _ONEBIT32B(14); i+= _ONEBIT32B(19)) {
		asm volatile ( "tlbie %0; eieio; tlbsync;"::"b"(i));
	}

	// Set a 1-1 translation map, so we can do non-cached accesses later.
	// Load any device mappings (for SMP on non-bootstrap CPU's).
	for(i = 0; i < num_bats; ++i) {
		if(bat[i].ref != 0) {
			load_bat(i, i, BAT_FLAG_VALID);
		}
	}

	set_msr(get_msr() | (PPC_MSR_IR|PPC_MSR_DR));

	ppc_isync();
	ppc_sync();
}

static paddr32_t ram_size = MEG(256);

/* some chips (like early E300 cores) don't like the 1-1 mapping
 * to be larger than real memory.
 *
 * So, before compressing the bats in mmuconfig2, we update the
 * limit of the 1-1
 */
void
ppc600_update_one_to_one_limit(paddr32_t limit)
{
	ram_size = min(MEG(256),limit);
}

void
ppcv_mmuconfig2_600(int cpu) {
	unsigned	i;
	unsigned	j;

	// Determine the real sizes needed.
	if (bat[0].size_needed < ram_size) {
		bat[0].size_needed = ram_size;
	}
	for(i=0; i<num_bats; i++) {
		// calculate the size of the BAT from the size_needed field
		unsigned new = MEG(128);
		while( new > KILO(64) ) {
			if(bat[i].size_needed > new) {
				break;
			}
			new >>= 1;
		}
		bat[i].size = new << 1;
	}
	
	// compress the bats.  Note that invalidate_device_bats leaves bat0 & ibat0
	// alone.  We can't mess with that bat now, since we depend on it for running
	// this code.  We'll overwrite it when we traverse the bat[] table though.
	invalidate_device_bats();
	for(i = 0, j = 0; i < num_bats; ++i) {
		if(bat[i].flags & MYCALLOUT_FLAG) {
			load_bat(j++, i, BAT_FLAG_CFG2|BAT_FLAG_VALID);
		}
	}
	set_spr(PPC_SPR_DBAT0L, get_spr(PPC_SPR_DBAT0L) & ~PPC_BATL_W);
	ppc_isync();
}

void *
ppcv_map_600(unsigned size, paddr_t phys, unsigned prot_flags) {
	unsigned		i;
	struct bat_info	*bp;
	uintptr_t		free_vaddr = FIRST_VADDR;
	int				free_bat = -1;

	i = 0;
	for(;;) {
		if(i >= num_bats) {
			if(free_bat >= 0) break;
			return NULL;
		}
		bp = &bat[i];
		if(bp->ref != 0) {
			if((phys >= bp->paddr) && ((phys+size-1) <= (bp->paddr+bp->size-1))) {
				if((prot_flags & PROT_NOCACHE) == (bp->flags & PROT_NOCACHE)) {
					// We can reuse this entry.  But first check to see if we need
					// to set up an IBAT.
					if( (prot_flags & PROT_EXEC) && !(bp->flags & PROT_EXEC) ) {
						// We need prot_exec but this entry didn't have it before.
						// Turn it on and reload the BAT.
						bp->flags |= PROT_EXEC;
						load_bat(i, i, BAT_FLAG_VALID);
					}
					// Next adjust the size if necessary
					if (phys - bp->paddr + size > bp->size_needed) {
						bp->size_needed = phys - bp->paddr + size;
					}
					goto reuse_one;
				}
			}
			if(free_vaddr < (bp->vaddr + bp->size)) {
				free_vaddr = bp->vaddr + bp->size;
			}
		} else {
			free_bat = i;
		}
		++i;
	}
	//set up a new bat
	#define INITIAL_BAT_SIZE	MEG(64)

	bp = &bat[free_bat];
	bp->vaddr = free_vaddr & ~(INITIAL_BAT_SIZE-1);
	bp->paddr = phys & ~(INITIAL_BAT_SIZE-1);
	bp->size = INITIAL_BAT_SIZE;
	bp->size_needed = phys - bp->paddr + size; 
	bp->flags = prot_flags;
	load_bat(free_bat, free_bat, BAT_FLAG_VALID);
reuse_one:
	bp->ref++;
	if(prot_flags & PROT_CALLOUT) {
		bp->ref++; 	// so it never gets released
		bp->flags |= MYCALLOUT_FLAG;
	}
	return (void *)(bp->vaddr + (uintptr_t)(phys - bp->paddr));
}

void
ppcv_unmap_600(void *p) {
	uintptr_t		vaddr = (uintptr_t)p;
	unsigned		i;
	struct bat_info	*bp;

	for(i = 0; i < num_bats; ++i) {
		bp = &bat[i];
		if(bp->ref != 0) {
			if((vaddr >= bp->vaddr) && (vaddr <= (bp->vaddr+bp->size-1))) {
				if(--bp->ref == 0) {
					load_bat(i, i, 0);
				}
				break;
			}
		}
	}
}

__SRCVERSION("ppcv_map_600.c $Rev: 580669 $");
