/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

#define N_PTES	8
struct pte64 {
	uint64_t		hi;
	uint64_t		lo;
};

#define FIRST_FREE_VADDR	(GIG(1)-MEG(256))

struct mapping {
	struct mapping		*next;
	uintptr_t			vaddr;
	paddr64_t			paddr;
	unsigned			size;
	unsigned			flags;
	unsigned			count;
};

struct mapping			*maps_free;
struct mapping			*maps_list;
static uint64_t			total_ram;
static unsigned 		lp_bits;
static unsigned 		va_bits;
static unsigned			sdr1;


static void
mapping_free(struct mapping *map) {
	map->next = maps_free;
	maps_free = map;
}


static struct mapping *
mapping_alloc(void) {
	struct mapping	*new;

	if(maps_free == NULL) {
		unsigned		i;
		#define MAP_GROUP	10

		new =  ws_alloc(sizeof(*new) * MAP_GROUP);
		for(i = 1; i < MAP_GROUP; ++i) mapping_free(&new[i]);
		return new;
	}
	new = maps_free;
	maps_free = new->next;
	return new;
}



#define PROT_LARGE		0x00000001


#define CALC_HASH(vaddr, large, avpn, pte1, pte2) 					\
	{																\
		uint32_t	hash1, hash2;									\
		uint32_t	hash1_lo, hash1_hi, hash2_lo, hash2_hi;			\
		uintptr_t	hash_base;										\
		unsigned	hash_mask;										\
		unsigned	vpn;											\
																	\
		if(large) {													\
			vpn = ((vaddr) >> 24) & 0x000ff;						\
			hash1 = (vpn >> 4) ^ (vpn & 0xf);						\
		} else {													\
			vpn = ((vaddr) >> 12) & 0xfffff;						\
			hash1 = (vpn >> 16) ^ (vpn & 0xffff);					\
		}															\
		(avpn) = (vaddr) >> 23;										\
																	\
		hash2 = ~hash1;												\
																	\
		hash1_lo = (hash1 & 0x000007ff) << 7;						\
		hash1_hi = (hash1 & 0x0007f800) >> 11;						\
																	\
		hash2_lo = (hash2 & 0x000007ff) << 7;						\
		hash2_hi = (hash2 & 0x0007f800) >> 11;						\
																	\
		hash_mask = (1 << (sdr1 & 0x1f)) - 1;						\
		hash1_hi = (hash1_hi & hash_mask) << 18;					\
		hash2_hi = (hash2_hi & hash_mask) << 18;					\
																	\
		hash_base = (uintptr_t)MAKE_1TO1_PTR(sdr1 & ~((1 << 18) - 1));\
		pte1 = (struct pte64 *)(hash_base | hash1_lo | hash1_hi);	\
		pte2 = (struct pte64 *)(hash_base | hash2_lo | hash2_hi);	\
	}

		

void
ppcv_mmuconfig1_900(int cpu) {
	unsigned	i;

	// Remember original mapping table in case we have to call back into
	// rom monitor code (e.g. open firmware).
	if((cpu == 0) && (original_msr & PPC_MSR_DR)) {
		original_sdr1 = get_spr(PPC_SPR_SDR1);
			// keep from overwriting hash table while we're running startup
			avoid_ram(original_sdr1 & ~((1 << 18) - 1), 
					0x40000 << (original_sdr1 & 0x1f));
	}

	// clear segment registers
	ppc_slbia();
	// set up one-to-one mapping area 
	ppc_slbmte(0x0000 + (PPC64_SLB0_KP|PPC64_SLB0_C|PPC64_SLB0_L), 
				(0 << PPC64_SLB1_ESID_SHIFT) | PPC64_SLB1_V | 0);
	// set up device mapping area 
	ppc_slbmte(0x3000 + (PPC64_SLB0_KP|PPC64_SLB0_C), 
				(3 << PPC64_SLB1_ESID_SHIFT) | PPC64_SLB1_V | 3);

#if 0	
//TEMP: Add in something so that OpenFirmware still works	
ppc_slbmte(0xf000, 0xf800000f);
kprintf("\n\n\n**************extra openfirmware SLB entry added*********\n\n\n"); 
//END TEMP
#endif

	//Invalidate the TLB
	for(i = 0; i < (1024*0x1000); i += 0x1000) {
		ppc_tlbie(i);
	}
	ppc_eieio();
	ppc_tlbsync();
	ppc_ptesync();
}


void
ppcv_mmuconfig2_900(int cpu) {
	unsigned	msr;

	if(sdr1 == 0) {
		crash("hash table never allocated");
	}

	if(cpu == 0) {
		// Scan page table looking for startup entries and clean them out.
		while(maps_list != NULL) {
			if(maps_list->flags & PROT_CALLOUT) {
				// Can just throw callout map information away, we don't
				// need it anymore.
				maps_list = maps_list->next;
			} else {
				startup_memory_unmap((void *)maps_list->vaddr);
			}
		}
	}

	set_spr(PPC_SPR_SDR1, sdr1);
	msr = get_msr();
	if(!(msr & PPC_MSR_IR)) {
		set_msr(msr | (PPC_MSR_IR|PPC_MSR_DR));
	}
	ppc_isync();
}


static int
add_mapping(uintptr_t vaddr, paddr_t paddr, unsigned flags) {
	struct pte64   	*pte1;
	struct pte64   	*pte2;
	struct pte64   	*pte;
	unsigned		i;
	uintptr_t		avpn;
	unsigned		hash;

	CALC_HASH(vaddr, (flags & PROT_LARGE), avpn, pte1, pte2);

#if 0
if(flags & PROT_CALLOUT) {
	kprintf("vaddr=%x, flags=%x, avpn=%x, pte1/2=%x/%x\n", vaddr, flags, avpn, pte1, pte2);
}
#endif
	// Look for a free PTE. Scan backwards so that we put the entries
	// at the end of a group. That way, the miss code in procnto is 
	// less likely to have to avoid removing them.
	i = N_PTES;
	for( ;; ) {
		--i;
		pte = &pte1[i];
		if(!(pte->hi & PPC64_TLBHI_V)) {
			hash = 0;
			break;
		}
		pte = &pte2[i];
		if(!(pte->hi & PPC64_TLBHI_V)) {
			hash = PPC64_TLBHI_H;
			break;
		}
		if(i == 0) {
			return 0;
		}
	}

	pte->lo = paddr | PPC_TLBLO_M;
	if(!(flags & PROT_EXEC)) {
		pte->lo |= PPC64_TLBLO_N;
	}
	if(flags & PROT_WRITE) {
		 pte->lo |= PPC_TLBLO_PP_NA; 	// supervisor only r/w
	} else {
		 pte->lo |= PPC_TLBLO_PP_ROA;	// supervisor only readonly
	}
	if(flags & PROT_NOCACHE) {
		pte->lo |= PPC_TLBLO_I;
		if(!(flags & PROT_EXEC)) {
			pte->lo |= PPC_TLBLO_G;
		}
	}

	if(flags & PROT_CALLOUT) {
		// Turn on a bit in the software field so that procnto knows
		// to never throw out this entry
		hash |= 1 << PPC64_TLBHI_SW_SHIFT;
	}
	if(flags & PROT_LARGE) {
		hash |= PPC64_TLBHI_L;
	}
	pte->hi = (avpn << PPC64_TLBHI_AVPN_SHIFT) | hash | PPC64_TLBHI_V;

	ppc_tlbsync();
	ppc_sync();
	return 1;
}


static unsigned
size_hashtable(void) {
	uint64_t	check;
	unsigned	size_idx;

	//look at size of ram and allocate hash table
	size_idx = 0;
	check = MEG(16); // minimum size is 16M
	for( ;; ) {
		if(check >= total_ram) break;
		++size_idx;
		if(size_idx >= 0x1f) break; // hit maximum size;
		check <<= 1;
	}

	// This avoids a problem with possibly missing some PTEGs when
	// searching. See Programming note on page 33 of Book III V2.01 (OEA).
	if((va_bits < 67) && (size_idx > (va_bits-39))) size_idx = va_bits-39;

	return size_idx;
}


static int
do_map(uintptr_t vaddr, paddr_t paddr, unsigned flags, unsigned size) {
	do {
		if(!add_mapping(vaddr, paddr, flags)) {
			return 0;
		}
		vaddr  += __PAGESIZE;
		paddr  += __PAGESIZE;
		size   -= __PAGESIZE;
	} while(size != 0);
	return 1;
}


static int
init_hashtable(void) {
	unsigned	size_idx;
	paddr_t		size_bytes;
	paddr32_t	hash_paddr;
	uintptr_t	vaddr;

	size_idx = size_hashtable();
	size_bytes = 0x40000 << size_idx;

	hash_paddr = calloc_ram(size_bytes, size_bytes);
	if(hash_paddr == NULL_PADDR32) return 0;
	sdr1 = hash_paddr | size_idx;

	//init hash table with big pages mapping first 256M
	for(vaddr = 0; vaddr < MEG(256); vaddr += 1 << lp_bits) {
		if(!add_mapping(vaddr, vaddr, PROT_CALLOUT|PROT_READ|PROT_WRITE|PROT_EXEC|PROT_LARGE)) {
			crash("can not create 1-to-1 mapping for 0x%x\n", vaddr);
		}
	}

	set_spr(PPC_SPR_SDR1, sdr1);

	ppc_sync();

	//enable ins/data translation (do we have to flush caches?)
	set_msr(get_msr() | (PPC_MSR_IR|PPC_MSR_DR));
	ppc_sync();
	return 1;
}


static void *
ppcv_map_900(unsigned size, paddr_t phys, unsigned prot_flags) {
	unsigned		off;
	uintptr_t		free_vaddr;
	struct mapping	*map;

	if((!(get_msr() & PPC_MSR_IR) || (phys < MEG(256)))
			&& !(prot_flags & PROT_NOCACHE)) {
		// can use the one-to-one mapping
		return (void *)(uintptr_t)phys;
	}
	if(!(get_msr() & PPC_MSR_IR)) {
		if(!init_hashtable()) return NULL;
	}

	off = phys & (__PAGESIZE-1);
	phys -= off;
	size = ROUNDPG(size+off);

	// look for an existing mapping
	#define PROT_CHECK (PROT_READ|PROT_WRITE|PROT_EXEC|PROT_NOCACHE)
	free_vaddr = FIRST_FREE_VADDR;
	for(map = maps_list; map != NULL; map = map->next) {
		uintptr_t		end;

		if((map->paddr == phys) 
		  && ((map->paddr + map->size) == (phys+size))
	 	  && ((map->flags & PROT_CHECK) == (prot_flags & PROT_CHECK))) {
			// can reuse this mapping
			map->count++;
			map->flags |= prot_flags & PROT_CALLOUT;
			return (void *)(map->vaddr + off);
		}
		end = map->vaddr + map->size;
		if(end > free_vaddr) free_vaddr = end;
	}

	map = mapping_alloc();
	map->vaddr = free_vaddr;
	map->paddr = phys;
	map->size  = size;
	map->flags = prot_flags;
	map->count = 1;
	map->next  = maps_list;
	maps_list  = map;

	if(!do_map(free_vaddr, phys, prot_flags, size)) {
		return NULL;
	}

	return (void *)(map->vaddr + off);
}


static void
ppcv_unmap_900(void *p) {
	uintptr_t		vaddr = (uintptr_t)p & ~(__PAGESIZE-1);
	struct mapping	*map;
	struct mapping	**owner;

	if(vaddr >= MEG(256)) {
		owner = &maps_list;
		for( ;; ) {
			map = *owner;
			if(map == NULL) return;
			if(map->vaddr == vaddr) break;
			owner = &map->next;
		}
		map->count--; 
		if(!(map->flags & PROT_CALLOUT) && (map->count == 0)) {
			do {
				struct pte64   	*pte1;
				struct pte64   	*pte2;
				struct pte64   	*pte;
				unsigned		i;
				unsigned		avpn;
				uint64_t		hi;

				ppc_tlbie(vaddr);

				CALC_HASH(vaddr, 0, avpn, pte1, pte2);

				/* look for a matching PTE */
				hi = (avpn << PPC64_TLBHI_AVPN_SHIFT) | PPC64_TLBHI_V;
				for(i = 0; i < N_PTES; i++) {
					pte = &pte1[i];
					if(pte->hi == hi) {
						pte->hi = 0;
						break;
					}
					pte = &pte2[i];
					if(pte->hi == (hi | PPC64_TLBHI_H)) {
						pte->hi = 0;
						break;
					}
				}
				vaddr      += __PAGESIZE;
				map->size  -= __PAGESIZE;
			} while(map->size != 0);
			*owner = map->next;
			mapping_free(map);
			ppc_eieio();
			ppc_tlbsync();
			ppc_ptesync();
		}
	}
}


static void
ram_added(paddr_t start, paddr_t size) {
	unsigned		old_sdr1;
	unsigned		old_size;
	struct mapping	*map;

	if(size != 0) {
		total_ram += size;
		return;
	}

	// No more RAM can be added, make sure that we've got the hash
	// table allocated (or properly sized for the total amount of ram).
	old_sdr1 = sdr1;
	if(old_sdr1 == 0) {
		if(!init_hashtable()) {
			crash("not enough memory for hash table");
		}
	} else if((old_sdr1 & 0x1f) != size_hashtable()) {
		// Need to grow the hash table
		if(!init_hashtable()) {
			crash("not enough memory for hash table");
		}
		old_size = 0x40000 << (old_sdr1 & 0x1f);
		// free the memory used by the old hash table
		add_ram(old_sdr1 & ~((1 << 18) - 1), old_size);
		// re-establish all the active mappings in the new hash table
		for(map = maps_list; map != NULL; map = map->next) {
			do_map(map->vaddr, map->paddr, map->flags, map->size);
		}
	}
}

void
ppc_900_map_setup(unsigned large_page_bits, unsigned virt_addr_bits, unsigned slb_entries) {
	lp_bits = large_page_bits;
	va_bits = virt_addr_bits;
	ppcv_config.map = &ppcv_map_900;
	ppcv_config.unmap = &ppcv_unmap_900;
	watch_add_ram(ram_added);
}

__SRCVERSION("ppcv_map_900.c $Rev: 580669 $");
