/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          bman_private.h

 @Description   BM header
*//***************************************************************************/
#ifndef __BMAN_PRIV_H
#define __BMAN_PRIV_H

#include "fsl_bman.h"


#ifdef DEBUG
/* Optionally compile-in assertion-checking */
#define BM_CHECKING
#endif /* DEBUG */

/* TODO - check the following defines */
#define cpu_relax()

/* TODO: NB, we currently assume that CORE_MemoryBarier() and lwsync() imply compiler barriers
 * and that dcbzl(), dcbfl(), and dcbi() won't fall victim to compiler or
 * execution reordering with respect to other code/instructions that manipulate
 * the same cacheline. */
#if defined (__QNXNTO__)
#include <sys/neutrino.h>
#endif

#if defined (CORE_E500MC) && (_NTO_VERSION > 640)
#define lwsync()    __asm__ __volatile__ ("lwsync" : : : "memory")
#define dcbzl(p)    __asm__ __volatile__ ("dcbzl 0,%0" : : "r" (p))
#define dcbf(p)     __asm__ __volatile__ ("dcbf 0,%0" : : "r" (p))
#define dcbt_ro(p)  __asm__ __volatile__ ("dcbt 0,%0" : : "r" (p))
#define dcbt_rw(p)  __asm__ __volatile__ ("dcbtst 0,%0" : : "r" (p))

#else
#define lwsync()    __asm__ __volatile__ ("msync" : : : "memory")
#define dcbzl(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbz 0,%0" : : "r" ((uint32_t)p+0x20)); \
    __asm__ __volatile__ ("dcbz 0,%0" : : "r" (p))
#define dcbf(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbf 0,%0" : : "r" (p))
#define dcbt_ro(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbt 0,%0" : : "r" (p))
#define dcbt_rw(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbtst 0,%0" : : "r" (p))
#endif /* CORE_E500MC */

#define dcbi(p) dcbf(p)

struct bm_addr {
    void  *addr_ce;    /* cache-enabled */
    void  *addr_ci;    /* cache-inhibited */
};

/* RCR state */
struct bm_rcr {
    struct bm_rcr_entry *ring, *cursor;
    uint8_t ci, available, ithresh, vbit;
#ifdef BM_CHECKING
    uint32_t busy;
    e_BmPortalProduceMode pmode;
    e_BmPortalRcrConsumeMode cmode;
#endif /* BM_CHECKING */
};

/* MC state */
struct bm_mc {
    struct bm_mc_command *cr;
    struct bm_mc_result *rr;
    uint8_t rridx, vbit;
#ifdef BM_CHECKING
    enum {
        /* Can only be _mc_start()ed */
        mc_idle,
        /* Can only be _mc_commit()ed or _mc_abort()ed */
        mc_user,
        /* Can only be _mc_retry()ed */
        mc_hw
    } state;
#endif /* BM_CHECKING */
};

/********************/
/* Portal structure */
/********************/

struct bm_portal {
    struct bm_addr addr;
    struct bm_rcr rcr;
    struct bm_mc mc;
};


#endif /* __BMAN_PRIV_H */
