/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          dpa_port.c

 @Description   DPAA routines implementation
*//***************************************************************************/
#include "std_ext.h"
#include "error_ext.h"
#include "sprint_ext.h"
#include "string_ext.h"
#include "bm_ext.h"
#include "qm_ext.h"
#include "sec_ext.h"
#include "fm_ext.h"
#include "fm_port_ext.h"
#include "fm_mac_ext.h"
#include "dpaa_ext.h"
#include "dpa_port.h"


//#define UNDER_CONSTRUCTION_DPAA
//#define DPAA_FAST_FWD
//#define DPAA_SUPER_FAST_FWD



static t_Error CheckInitParameters(t_DpaPort *p_DpaPort)
{
    switch(p_DpaPort->type)
    {
    case e_DPA_PORT_T_VIRTUAL:
        if (p_DpaPort->h_RxFmPortDev || p_DpaPort->h_TxFmPortDev || p_DpaPort->h_FmMacDev)
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("can't have ports/mac handles for virtual-port!"));
        if (p_DpaPort->f_TxConf)
            RETURN_ERROR(MINOR, E_INVALID_VALUE, ("No Tx-confirmation for virtual-port!"));
        break;
    case e_DPA_PORT_T_SEC:
        if (p_DpaPort->h_RxFmPortDev || p_DpaPort->h_TxFmPortDev)
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("can't have ports handles for SEC-port!"));
        if (!p_DpaPort->h_Sec)
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("missing SEC handle for SEC-port!"));
        if (p_DpaPort->f_TxConf)
            RETURN_ERROR(MINOR, E_INVALID_VALUE, ("No Tx-confirmation for SEC-port!"));
        if (!p_DpaPort->autoBmAlloc)
            RETURN_ERROR(MINOR, E_INVALID_VALUE, ("SEC-port must work with \"auto-bm-allocate\" enabled!"));
        break;
    case e_DPA_PORT_T_RX_TX:
    case e_DPA_PORT_T_IM_RX_TX:
        if (!(p_DpaPort->h_RxFmPortDev || p_DpaPort->h_TxFmPortDev || p_DpaPort->h_FmMacDev))
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("missing ports/mac handles for rxtx-port!"));
        if ((p_DpaPort->type == e_DPA_PORT_T_IM_RX_TX) &&
            (p_DpaPort->rxBuffInfo.parserResults ||
             p_DpaPort->rxBuffInfo.timeStamp ||
             p_DpaPort->txBuffInfo.parserResults ||
             p_DpaPort->txBuffInfo.timeStamp))
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("can't set parser-results/time-stamp for FM IM port!"));
        break;
    case e_DPA_PORT_T_OP:
        if (!p_DpaPort->h_TxFmPortDev)
            RETURN_ERROR(MINOR, E_INVALID_STATE, ("missing port handle for op-port!"));
        if (p_DpaPort->f_TxConf)
            RETURN_ERROR(MINOR, E_INVALID_VALUE, ("No Tx-confirmation for op-port!"));
        if (p_DpaPort->autoBmAlloc)
            RETURN_ERROR(MINOR, E_INVALID_VALUE, ("op-port must not work with \"auto-bm-allocate\" enabled!"));
        break;
    default:
        RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("port type"));
    }

    if ((p_DpaPort->type != e_DPA_PORT_T_IM_RX_TX) &&
        (GetBuffPoolHandleBySize(p_DpaPort , (uint16_t)(sizeof(t_DpaaSGT) + p_DpaPort->txBuffInfo.dataOffset)) == NULL))
        RETURN_ERROR(MINOR, E_INVALID_STATE,
                     ("At least one of the BM pools buffer size must be equal or greater than %d",
                      sizeof(t_DpaaSGT) + p_DpaPort->txBuffInfo.dataOffset));
    return E_OK;
}

static t_Error FillFmBuffInfo(t_DpaPort *p_DpaPort, bool tx)
{
    t_Handle                    h_PortDev;
    t_DpaPortBuffPrefixInfo     *p_Info;
    uint8_t                     *p_TmpBuff;

    ASSERT_COND(p_DpaPort);
    if (tx)
    {
        h_PortDev = p_DpaPort->h_TxFmPortDev;
        p_Info    = &p_DpaPort->txBuffInfo;
    }
    else
    {
        h_PortDev = p_DpaPort->h_RxFmPortDev;
        p_Info    = &p_DpaPort->rxBuffInfo;
    }

    p_TmpBuff = (uint8_t *)FM_PORT_GetBufferPrsResult(h_PortDev, NULL);
    if (p_TmpBuff)
    {
        p_Info->parserResults           = TRUE;
        p_Info->parserResultsOffset     = CAST_POINTER_TO_UINT32(p_TmpBuff);
    }

    p_TmpBuff = (uint8_t *)FM_PORT_GetBufferHashResult(h_PortDev, NULL);
    if (p_TmpBuff)
    {
        p_Info->hashResult              = TRUE;
        p_Info->hashResultOffset        = CAST_POINTER_TO_UINT32(p_TmpBuff);
    }

    p_TmpBuff = (uint8_t *)FM_PORT_GetBufferTimeStamp(h_PortDev, NULL);
    if (p_TmpBuff)
    {
        p_Info->timeStamp               = TRUE;
        p_Info->timeStampOffset         = CAST_POINTER_TO_UINT32(p_TmpBuff);
    }

    p_TmpBuff = (uint8_t *)FM_PORT_GetBufferICInfo(h_PortDev, NULL);
    if (p_TmpBuff)
    {
        p_Info->passAllOtherPCDInfo     = TRUE;
        p_Info->allOtherPCDInfoOffset   = CAST_POINTER_TO_UINT32(p_TmpBuff);
    }

#ifdef DEBUG
    p_TmpBuff = (uint8_t *)FM_PORT_GetBufferDebugInfo(h_PortDev, NULL);
    if (p_TmpBuff)
    {
        p_Info->passDebugInfo           = TRUE;
        p_Info->debugInfoOffset         = CAST_POINTER_TO_UINT32(p_TmpBuff);
    }
#endif /* DEBUG */

    p_Info->dataOffset = FM_PORT_GetBufferDataOffset(h_PortDev);

    return E_OK;
}

static t_Error FillSecBuffInfo(t_DpaPort *p_DpaPort)
{
    ASSERT_COND(p_DpaPort);
    p_DpaPort->txBuffInfo.dataOffset = p_DpaPort->txBuffInfo.privDataSize;
    if (p_DpaPort->rxBuffInfo.privDataSize)
        p_DpaPort->rxBuffInfo.dataOffset =
            (uint32_t)ROUND_UP(p_DpaPort->rxBuffInfo.privDataSize,
                               SEC_GetBurstSize(p_DpaPort->h_Sec));

    return E_OK;
}

static t_Error  DoFwd (t_DpaPort             *p_DpaPort,
                       uint8_t               fwdTblIndx,
                       uint32_t              rxQueueId,
                       uint8_t               *p_Data,
                       uint16_t              length,
                       uint8_t               position,
                       t_DpaaFD              *p_DpaaFd,
                       t_Handle              h_BufContext,
                       t_DpaPortBuffContext  *p_Context)
{
    t_Error err = E_OK;

    p_Context->dataOffset   = (uint16_t)DPAA_FD_GET_OFFSET(p_DpaaFd);
    p_Context->length       = length;

    if (p_DpaPort->p_FwdTbl[fwdTblIndx].f_RxPreFwd)
        p_DpaPort->p_FwdTbl[fwdTblIndx].f_RxPreFwd (p_DpaPort->p_FwdTbl[fwdTblIndx].h_App,
                                                    rxQueueId,
                                                    p_Data,
                                                    p_Context->length,
                                                    0,
                                                    position,
                                                    h_BufContext,
                                                    p_Context);

    DBG(TRACE, ("Forwarding:"
                "DpaPort 0x%x, type %d, Queue 0x%x -> "
                "DpaPort 0x%x type %d, Queue 0x%x.",
                p_DpaPort, p_DpaPort->type, rxQueueId,
                p_DpaPort->p_FwdTbl[fwdTblIndx].h_FwdDpaPort,
                ((t_DpaPort*)(p_DpaPort->p_FwdTbl[fwdTblIndx].h_FwdDpaPort))->type,
                (uint32_t)(rxQueueId -
                           p_DpaPort->p_FwdTbl[fwdTblIndx].rxBaseQueueId +
                           p_DpaPort->p_FwdTbl[fwdTblIndx].txBaseQueueId)));

    if (position & LAST_BUF)
    {
        t_DpaaFD                txFrame;
        uint8_t                 *p_Buff = p_Data;
        uint32_t                fdStatus = 0;
        uint32_t                queueId =
            (uint32_t)(rxQueueId -
                       p_DpaPort->p_FwdTbl[fwdTblIndx].rxBaseQueueId +
                       p_DpaPort->p_FwdTbl[fwdTblIndx].txBaseQueueId);

        p_DpaPort = p_DpaPort->p_FwdTbl[fwdTblIndx].h_FwdDpaPort;
        SANITY_CHECK_RETURN_ERROR((p_Context->h_BmPool || p_DpaPort->f_TxConf), E_NOT_SUPPORTED);

        p_Buff = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Buff)-
                                                       DPAA_FD_GET_OFFSET(p_DpaaFd)));

        /* Build the buffer Internal-Context */
        if (p_Context->h_UserPriv)
        {
            ASSERT_COND(p_DpaPort->txBuffInfo.privDataSize);
            ((t_Handle *)p_Buff)[0] = p_Context->h_UserPriv;
        }

        if (p_DpaPort->txBuffInfo.parserResults && p_Context->p_FmPrsResult)
        {
            t_FmPrsResult   *p_FmPrsResult;
            int             i;

            ASSERT_COND(p_DpaPort->txBuffInfo.parserResults);
            if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
                p_FmPrsResult = FM_PORT_GetBufferPrsResult(p_DpaPort->h_TxFmPortDev, (char*)p_Buff);
            else
                p_FmPrsResult =
                    CAST_UINT64_TO_POINTER_TYPE(t_FmPrsResult,
                                                (CAST_POINTER_TO_UINT64(p_Buff) + p_DpaPort->txBuffInfo.parserResultsOffset));

            ASSERT_COND(p_FmPrsResult);
            if (p_Context->p_FmPrsResult != p_FmPrsResult)
                for (i=0; i<(sizeof(t_FmPrsResult)>>2); i++)
                    ((uint32_t *)p_FmPrsResult)[i] = ((uint32_t *)p_Context->p_FmPrsResult)[i];

            /* Tx port - set UPD to enable copying from IC and DTC to enable checksum */
            fdStatus |= (FM_FD_CMD_RPD | FM_FD_CMD_DTC);
        }
        else if (p_DpaPort->type == e_DPA_PORT_T_OP)
            fdStatus |= FM_FD_CMD_DCL4C;

        if (p_DpaPort->txBuffInfo.timeStamp)
            fdStatus |= FM_FD_CMD_UPD;

        /* Build FD */
        txFrame.id = p_DpaaFd->id;
        DPAA_FD_SET_ADDR(&txFrame, p_Buff);
        DPAA_FD_SET_OFFSET(&txFrame, p_Context->dataOffset);
        DPAA_FD_SET_LENGTH(&txFrame, p_Context->length);
        if (p_DpaPort->p_TxSGTFrm)
        {
            DPAA_FD_SET_FORMAT(&txFrame, (uint32_t)e_DPAA_FD_FORMAT_TYPE_SHORT_MBSF);
            p_DpaPort->p_TxSGTFrm = NULL;
            DBG(TRACE, ("S/G frame sent."));
        }
        else
        {
            DPAA_FD_SET_FORMAT(&txFrame, (uint32_t)e_DPAA_FD_FORMAT_TYPE_SHORT_SBSF);
            DBG(TRACE, ("STD frame sent."));
        }

        DPAA_FD_SET_STATUS(&txFrame, fdStatus);

        err = QM_FQR_Enqueue(p_DpaPort->h_TxsQmFqr[queueId], p_Context->h_QmPortal, 0, &txFrame);

        if (err != E_OK)
        {
            if (p_DpaPort->f_TxConf)
                p_DpaPort->f_TxConf(p_DpaPort->h_App,
                                    p_Data,
                                    0x40000000,
                                    p_Context);
            else
                BM_POOL_PutBuf(p_Context->h_BmPool, p_Context->h_BmPortal, p_Buff);
        }

        return err;
    }

    return err;
}

static t_Error  DoImFwd (t_DpaPort             *p_DpaPort,
                         uint8_t               *p_Data,
                         uint16_t              length,
                         uint8_t               position,
                         t_Handle              h_BufContext,
                         t_DpaPortBuffContext  *p_Context)
{
    t_Error err;

    if (p_DpaPort->p_FwdTbl[0].f_RxPreFwd)
        p_DpaPort->p_FwdTbl[0].f_RxPreFwd (p_DpaPort->p_FwdTbl[0].h_App,
                                           0,
                                           p_Data,
                                           length,
                                           0,
                                           position,
                                           h_BufContext,
                                           p_Context);

    DBG(TRACE, ("Forwarding (IM):"
                "DpaPort 0x%x Queue 0 -> DpaPort 0x%x Queue 0.",
                p_DpaPort, p_DpaPort->p_FwdTbl[0].h_FwdDpaPort));
    err =  DPAPORT_Tx (p_DpaPort->p_FwdTbl[0].h_FwdDpaPort,
                       0,
                       p_Data,
                       length,
                       (bool)(position & LAST_BUF),
                       p_Context);
    if ((err != E_OK) && p_DpaPort->f_TxConf)
        p_DpaPort->f_TxConf(p_DpaPort->h_App,
                            p_Data,
                            0x40000000,
                            p_Context);

    return err;
}

static t_Error HandleTxSGT(t_DpaPort            *p_DpaPort,
                           uint8_t              **p_Buff,
                           uint16_t             *p_Length,
                           bool                 lastBuffer,
                           t_DpaPortBuffContext *p_Context,
                           t_Handle             *p_BmPool)
{
    /* If it is the first buffer of S/G frame, we need to allocate buffer for the SGT */
    if (!lastBuffer && (p_DpaPort->txSGTFrmEntryPos == 0))
    {
        ASSERT_COND(!p_DpaPort->p_TxSGTFrm);
        p_DpaPort->h_SGTBmPool = GetBuffPoolHandleBySize(p_DpaPort , (uint16_t)(sizeof(t_DpaaSGT) + p_DpaPort->txBuffInfo.dataOffset));
        ASSERT_COND(p_DpaPort->h_SGTBmPool);
        p_DpaPort->p_TxSGTFrm = (t_DpaaSGT *)BM_POOL_GetBuf(p_DpaPort->h_SGTBmPool, p_Context->h_BmPortal);
        if (!p_DpaPort->p_TxSGTFrm)
            RETURN_ERROR(MINOR, E_NO_MEMORY, ("Scatter-Gather List object"));

        memset(p_DpaPort->p_TxSGTFrm, 0, sizeof(t_DpaaSGT) + p_DpaPort->txBuffInfo.dataOffset);
        p_DpaPort->p_TxSGTFrm = CAST_UINT64_TO_POINTER_TYPE(t_DpaaSGT, (CAST_POINTER_TO_UINT64(p_DpaPort->p_TxSGTFrm)+
                                                                        p_DpaPort->txBuffInfo.dataOffset));
    }

    if (p_DpaPort->txSGTFrmEntryPos == DPAA_NUM_OF_SG_TABLE_ENTRY)
    {
        BM_POOL_PutBuf(p_DpaPort->h_SGTBmPool, p_Context->h_BmPortal, (void *)p_DpaPort->p_TxSGTFrm);
        p_DpaPort->h_SGTBmPool = NULL;
        RETURN_ERROR(MAJOR,E_INVALID_STATE,("Scatter-Gather List exceeded num of entries"));
    }
    DPAA_SGTE_SET_ADDR(&p_DpaPort->p_TxSGTFrm->tableEntry[p_DpaPort->txSGTFrmEntryPos], *p_Buff);
    DPAA_SGTE_SET_OFFSET(&p_DpaPort->p_TxSGTFrm->tableEntry[p_DpaPort->txSGTFrmEntryPos], 0);
    DPAA_SGTE_SET_LENGTH(&p_DpaPort->p_TxSGTFrm->tableEntry[p_DpaPort->txSGTFrmEntryPos], *p_Length);
    DPAA_SGTE_SET_BPID(&p_DpaPort->p_TxSGTFrm->tableEntry[p_DpaPort->txSGTFrmEntryPos],
                       (*p_BmPool ? BM_POOL_GetId(*p_BmPool) : (uint32_t)0xff));
    p_DpaPort->txSGTFrmAccumLength += *p_Length;
    if (lastBuffer)
    {
        DPAA_SGTE_SET_FINAL(&p_DpaPort->p_TxSGTFrm->tableEntry[p_DpaPort->txSGTFrmEntryPos], TRUE);
        *p_Buff = (uint8_t*)p_DpaPort->p_TxSGTFrm;
        *p_Length = p_DpaPort->txSGTFrmAccumLength;
        p_DpaPort->txSGTFrmEntryPos = 0;
        p_DpaPort->txSGTFrmAccumLength = 0;
        *p_BmPool = p_DpaPort->h_SGTBmPool;
        p_DpaPort->h_SGTBmPool = NULL;
    }
    else
        p_DpaPort->txSGTFrmEntryPos++;

    return E_OK;
}

static t_Error HandleRxSGT(t_DpaPort            *p_DpaPort,
                           uint32_t             fqidOffset,
                           uint8_t              *p_Data,
                           t_DpaaFD             *p_DpaaFd,
                           t_DpaPortBuffContext *p_Context)
{
    t_DpaaSGT               *p_FmSGT = NULL;
    t_Handle                h_BufContext = NULL;
    physAddress_t           physBuffAddr;
    t_Error                 err;
    uint32_t                status = DPAA_FD_GET_STATUS(p_DpaaFd) & FM_FD_RX_STATUS_ERR_MASK;
    uint16_t                length;
    uint8_t                 fwdTblIndx, entryPos = 0, buffPos = 0;

    DBG(TRACE, ("S/G frame recv."));
    p_FmSGT = CAST_UINT64_TO_POINTER_TYPE(t_DpaaSGT, (CAST_POINTER_TO_UINT64(p_Data)+
                                                     DPAA_FD_GET_OFFSET(p_DpaaFd)));
    buffPos = FIRST_BUF;
    p_Context->h_BmPortal = NULL;

    while(buffPos != LAST_BUF)
    {
        physBuffAddr = DPAA_SGTE_GET_PHYS_ADDR(&p_FmSGT->tableEntry[entryPos]);
        p_Context->h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_SGTE_GET_BPID(&p_FmSGT->tableEntry[entryPos]));
        ASSERT_COND((p_DpaPort->type == e_DPA_PORT_T_OP)  || (p_DpaPort->type == e_DPA_PORT_T_VIRTUAL) || p_Context->h_BmPool);

        if (p_Context->h_BmPool)
        {
            p_Data = BM_POOL_PhysToVirt(p_Context->h_BmPool, physBuffAddr);
            h_BufContext = BM_POOL_GetBufferContext(p_Context->h_BmPool, p_Data);
            if (p_DpaPort->autoBmFill)
            {
                if (p_DpaPort->f_FillBmPortal)
                {
                    p_DpaPort->f_FillBmPortal(p_DpaPort->h_App, p_Context);
                    SANITY_CHECK_RETURN_VALUE(p_Context->h_BmPortal, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
                }
                BM_POOL_FillBufs(p_Context->h_BmPool, p_Context->h_BmPortal, 1);
            }
        }
        else
            p_Data = XX_PhysToVirt(physBuffAddr);
        p_Data = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data)+
                DPAA_SGTE_GET_OFFSET(&p_FmSGT->tableEntry[entryPos])));
        length = (uint16_t)DPAA_SGTE_GET_LENGTH(&p_FmSGT->tableEntry[entryPos]);

        if (DPAA_SGTE_GET_FINAL(&p_FmSGT->tableEntry[entryPos]))
            buffPos = LAST_BUF;

        if (!p_DpaPort->numOfFwdDescs || status ||
            (fwdTblIndx = GetFwdTblIndx(p_DpaPort, fqidOffset)) == DPAPORT_ILLEGAL_FWD_TBL_INDX)
            p_DpaPort->f_RxStore(p_DpaPort->h_App,
                                 fqidOffset,
                                 p_Data,
                                 length,
                                 status,
                                 buffPos,
                                 h_BufContext,
                                 p_Context);
        else if ((err = DoFwd (p_DpaPort,
                               fwdTblIndx,
                               fqidOffset,
                               p_Data,
                               length,
                               buffPos,
                               p_DpaaFd,
                               h_BufContext,
                               p_Context)) != E_OK)
        {
            REPORT_ERROR(MINOR, err, NO_MSG);
            //return e_RX_STORE_RESPONSE_PAUSE;
        }

        buffPos = (bool)((buffPos != LAST_BUF) ? MIDDLE_BUF : LAST_BUF);
        entryPos++;
    }

    /* Release the buffer of the SGT itself; it must be from BM-pools */
    physBuffAddr = DPAA_FD_GET_PHYS_ADDR(p_DpaaFd);
    p_Context->h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_FD_GET_BPID(p_DpaaFd));
    ASSERT_COND(p_Context->h_BmPool);
    if (p_DpaPort->f_FillBmPortal)
    {
        p_DpaPort->f_FillBmPortal(p_DpaPort->h_App, p_Context);
        SANITY_CHECK_RETURN_VALUE(p_Context->h_BmPortal, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
    }
    BM_POOL_PutBuf(p_Context->h_BmPool, p_Context->h_BmPortal, BM_POOL_PhysToVirt(p_Context->h_BmPool, physBuffAddr));

    return E_OK;
}

static t_Error HandleTxCSGT(t_DpaPort            *p_DpaPort,
                            uint32_t             fqidOffset,
                            uint8_t              *p_Data,
                            t_DpaaFD             *p_DpaaFd,
                            t_DpaPortBuffContext *p_Context)
{
    t_DpaaSGT               *p_FmSGT = NULL;
    physAddress_t           physBuffAddr;
    uint8_t                 entryPos = 0;

    UNUSED(fqidOffset);
    DBG(TRACE, ("S/G frame conf."));
    p_FmSGT = CAST_UINT64_TO_POINTER_TYPE(t_DpaaSGT, (CAST_POINTER_TO_UINT64(p_Data)+
                                                     DPAA_FD_GET_OFFSET(p_DpaaFd)));
    p_Context->h_BmPortal = NULL;

    do
    {
        physBuffAddr = DPAA_SGTE_GET_PHYS_ADDR(&p_FmSGT->tableEntry[entryPos]);
        p_Context->h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_SGTE_GET_BPID(&p_FmSGT->tableEntry[entryPos]));
        if (p_Context->h_BmPool)
            p_Data = BM_POOL_PhysToVirt(p_Context->h_BmPool, physBuffAddr);
        else
            p_Data = XX_PhysToVirt(physBuffAddr);
        p_Data = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data)+
                                                      DPAA_SGTE_GET_OFFSET(&p_FmSGT->tableEntry[entryPos])));

        if (p_DpaPort->f_TxConf)
            p_DpaPort->f_TxConf(p_DpaPort->h_App,
                                p_Data,
                                (DPAA_FD_GET_STATUS(p_DpaaFd) & FM_FD_TX_STATUS_ERR_MASK),
                                p_Context);
        else if (p_Context->h_BmPool)
        {
            if (p_DpaPort->f_FillBmPortal)
            {
                p_DpaPort->f_FillBmPortal(p_DpaPort->h_App, p_Context);
                SANITY_CHECK_RETURN_VALUE(p_Context->h_BmPortal, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
            }
            BM_POOL_PutBuf(p_Context->h_BmPool, p_Context->h_BmPortal, p_Data);
        }
    } while (!DPAA_SGTE_GET_FINAL(&p_FmSGT->tableEntry[entryPos++]));

    physBuffAddr = DPAA_FD_GET_PHYS_ADDR(p_DpaaFd);
    p_Context->h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_FD_GET_BPID(p_DpaaFd));
    ASSERT_COND(p_Context->h_BmPool);
    if (p_DpaPort->f_FillBmPortal)
    {
        p_DpaPort->f_FillBmPortal(p_DpaPort->h_App, p_Context);
        SANITY_CHECK_RETURN_VALUE(p_Context->h_BmPortal, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
    }
    BM_POOL_PutBuf(p_Context->h_BmPool, p_Context->h_BmPortal, BM_POOL_PhysToVirt(p_Context->h_BmPool, physBuffAddr));

    return E_OK;
}

static t_Error HandleInFmBuffContext(t_DpaPort              *p_DpaPort,
                                     uint8_t                *p_Data,
                                     t_DpaPortBuffContext   *p_Context)
{
    if (p_DpaPort->rxBuffInfo.parserResults)
    {
        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
            p_Context->p_FmPrsResult = FM_PORT_GetBufferPrsResult(p_DpaPort->h_RxFmPortDev, (char*)p_Data);
        else
            p_Context->p_FmPrsResult = CAST_UINT64_TO_POINTER_TYPE(t_FmPrsResult, (CAST_POINTER_TO_UINT64(p_Data) + p_DpaPort->rxBuffInfo.parserResultsOffset));
        SANITY_CHECK_RETURN_ERROR(p_Context->p_FmPrsResult, E_NULL_POINTER);
    }

    if (p_DpaPort->rxBuffInfo.timeStamp)
    {
        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
            p_Context->p_FmTimeStamp = FM_PORT_GetBufferTimeStamp(p_DpaPort->h_RxFmPortDev, (char*)p_Data);
        else
            p_Context->p_FmTimeStamp = CAST_UINT64_TO_POINTER_TYPE(uint64_t, (CAST_POINTER_TO_UINT64(p_Data) + p_DpaPort->rxBuffInfo.timeStampOffset));
        SANITY_CHECK_RETURN_ERROR(p_Context->p_FmTimeStamp, E_NULL_POINTER);
    }

    if (p_DpaPort->rxBuffInfo.hashResult)
    {
        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
            p_Context->p_FmKgHashResult = FM_PORT_GetBufferHashResult(p_DpaPort->h_RxFmPortDev, (char*)p_Data);
        else
            p_Context->p_FmKgHashResult = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data) + p_DpaPort->rxBuffInfo.hashResultOffset));
        SANITY_CHECK_RETURN_ERROR(p_Context->p_FmKgHashResult, E_NULL_POINTER);
    }

    if (p_DpaPort->rxBuffInfo.passAllOtherPCDInfo)
    {
        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
            p_Context->p_FmIntContext = FM_PORT_GetBufferICInfo(p_DpaPort->h_RxFmPortDev, (char*)p_Data);
        else
            p_Context->p_FmIntContext = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data) + p_DpaPort->rxBuffInfo.allOtherPCDInfoOffset));
    }

#ifdef DEBUG
    if (p_DpaPort->rxBuffInfo.passDebugInfo)
    {
        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
            p_Context->p_FmDebugInfo = FM_PORT_GetBufferDebugInfo(p_DpaPort->h_RxFmPortDev, (char*)p_Data);
        else
            p_Context->p_FmDebugInfo = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data) + p_DpaPort->rxBuffInfo.debugInfoOffset));
        SANITY_CHECK_RETURN_ERROR(p_Context->p_FmDebugInfo, E_NULL_POINTER);
    }
#endif /* DEBUG */

    return E_OK;
}

static t_Error DpaFmTxCB (t_Handle              h_DpaPort,
                          uint32_t              queueId,
                          uint8_t               *p_Data,
                          uint16_t              length,
                          bool                  lastBuffer,
                          t_DpaPortBuffContext  *p_Context)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_DpaaFD                txFrame;
    t_Handle                h_BmPool = p_Context->h_BmPool;
    uint8_t                 *p_Buff = p_Data;
    uint32_t                fdStatus = 0;
    t_Error                 err = E_OK;

    ASSERT_COND(p_DpaPort);
    SANITY_CHECK_RETURN_ERROR(p_Context, E_NULL_POINTER);

    /* if there is no tx-confirmation, we need send the data with
       BM buffer so the HW can release it to the BM pool */
    if (p_DpaPort->autoBmAlloc &&
        !p_DpaPort->f_TxConf &&
        !h_BmPool)
    {
        uint8_t *p_NewData;
        /* This is not the first buffer of S/G frame */
        if (p_DpaPort->p_TxSGTFrm)
            h_BmPool = GetBuffPoolHandleBySize(p_DpaPort, length);
        else /* This is first buffer of S/G frame or single-buffer-frame */
            h_BmPool = GetBuffPoolHandleBySize(p_DpaPort, (uint16_t)(length + p_DpaPort->txBuffInfo.dataOffset));
        if (!h_BmPool)
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("Need to build scatter-gather frame!"));
        p_NewData = (uint8_t *)BM_POOL_GetBuf(h_BmPool, p_Context->h_BmPortal);
        if (!p_NewData)
            RETURN_ERROR(MINOR, E_NO_MEMORY, ("New data buffer for no-tx-conf!"));
        if (lastBuffer && !p_DpaPort->p_TxSGTFrm)
            p_NewData = CAST_UINT64_TO_POINTER_TYPE(uint8_t,(CAST_POINTER_TO_UINT64(p_NewData)+
                                                             p_DpaPort->txBuffInfo.dataOffset));
        MemCpy32(p_NewData, p_Data, length);
        p_Buff = p_Data = p_NewData;
    }

    if (!lastBuffer)
        return HandleTxSGT(p_DpaPort, &p_Buff, &length, lastBuffer, p_Context, &h_BmPool);
    else if (p_DpaPort->p_TxSGTFrm &&
             ((err = HandleTxSGT(p_DpaPort, &p_Buff, &length, lastBuffer, p_Context, &h_BmPool)) != E_OK))
        RETURN_ERROR(MINOR, err, NO_MSG);

    p_Buff = CAST_UINT64_TO_POINTER_TYPE(uint8_t,(CAST_POINTER_TO_UINT64(p_Buff)-
                                                  p_DpaPort->txBuffInfo.dataOffset));

    /* Build the buffer Internal-Context */
    if (p_Context->h_UserPriv)
    {
        ASSERT_COND(p_DpaPort->txBuffInfo.privDataSize);
        ((t_Handle *)p_Buff)[0] = p_Context->h_UserPriv;
    }

    if (p_Context->p_FmPrsResult)
    {
        t_FmPrsResult   *p_FmPrsResult;
        int             i;

        ASSERT_COND(p_DpaPort->txBuffInfo.parserResults);
        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
            p_FmPrsResult = FM_PORT_GetBufferPrsResult(p_DpaPort->h_TxFmPortDev, (char*)p_Buff);
        else
            p_FmPrsResult =
                CAST_UINT64_TO_POINTER_TYPE(t_FmPrsResult,
                                            (CAST_POINTER_TO_UINT64(p_Buff) + p_DpaPort->txBuffInfo.parserResultsOffset));

        ASSERT_COND(p_FmPrsResult);
        for (i=0; i<(sizeof(t_FmPrsResult)>>2); i++)
            ((uint32_t *)p_FmPrsResult)[i] = ((uint32_t *)p_Context->p_FmPrsResult)[i];

        /* Tx port - set UPD to enable copying from IC and DTC to enable checksum */
        fdStatus |= (FM_FD_CMD_RPD | FM_FD_CMD_DTC);
    }
    else if (p_DpaPort->type == e_DPA_PORT_T_OP)
        fdStatus |= FM_FD_CMD_DCL4C;

    if (p_DpaPort->txBuffInfo.timeStamp)
        fdStatus |= FM_FD_CMD_UPD;

    /* Build FD */
    txFrame.id = 0; /* set to zero to clear the LIODN-offset fields */
    DPAA_FD_SET_DD(&txFrame, (uint32_t)0x3);
    DPAA_FD_SET_PID(&txFrame, (uint32_t)0x3f);
    DPAA_FD_SET_BPID(&txFrame, (h_BmPool ? BM_POOL_GetId(h_BmPool) : (uint32_t)0xff));

    DPAA_FD_SET_ADDR(&txFrame, p_Buff);
    DPAA_FD_SET_OFFSET(&txFrame, p_DpaPort->txBuffInfo.dataOffset);
    DPAA_FD_SET_LENGTH(&txFrame, length);

    if (p_DpaPort->p_TxSGTFrm)
    {
        DPAA_FD_SET_FORMAT(&txFrame, (uint32_t)e_DPAA_FD_FORMAT_TYPE_SHORT_MBSF);
        p_DpaPort->p_TxSGTFrm = NULL;
        DBG(TRACE, ("S/G frame sent."));
    }
    else
    {
        DPAA_FD_SET_FORMAT(&txFrame, (uint32_t)e_DPAA_FD_FORMAT_TYPE_SHORT_SBSF);
        DBG(TRACE, ("STD frame sent."));
    }

    DPAA_FD_SET_STATUS(&txFrame, fdStatus);

    err = QM_FQR_Enqueue(p_DpaPort->h_TxsQmFqr[queueId], p_Context->h_QmPortal, 0, &txFrame);

    return err;
}

static t_Error DpaFmImTxCB (t_Handle              h_DpaPort,
                            uint32_t              queueId,
                            uint8_t               *p_Data,
                            uint16_t              length,
                            bool                  lastBuffer,
                            t_DpaPortBuffContext  *p_Context)
{
    t_DpaPort   *p_DpaPort = (t_DpaPort*)h_DpaPort;

    ASSERT_COND(p_DpaPort);
    SANITY_CHECK_RETURN_ERROR((queueId == 0), E_INVALID_VALUE);

    UNUSED(queueId);
    return FM_PORT_ImTx (p_DpaPort->h_TxFmPortDev,
                         p_Data,
                         length,
                         lastBuffer,
                         p_Context->h_UserPriv);
}

static e_RxStoreResponse DpaFmRxCB(t_Handle h_Arg,
                                   t_Handle h_QmFqr,
                                   t_Handle h_QmPortal,
                                   uint32_t fqidOffset,
                                   t_DpaaFD *p_DpaaFd)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_Arg;
    t_Handle                h_BufContext = NULL;
    t_DpaPortBuffContext    context;
    physAddress_t           physBuffAddr;
    uint8_t                 *p_Data;
    uint8_t                 fwdTblIndx;
    t_Error                 err;

    SANITY_CHECK_RETURN_VALUE(p_DpaPort, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);

    if (h_QmFqr == p_DpaPort->h_RxDfltQmFqr)
        fqidOffset = DPAPORT_RX_DFLT_Q;
    else if (h_QmFqr == p_DpaPort->h_RxPcdQmFqr)
        /* Do nothing */;
    else if (h_QmFqr == p_DpaPort->h_RxErrQmFqr)
        fqidOffset = DPAPORT_RX_ERR_Q;
    else if (p_DpaPort->type == e_DPA_PORT_T_OP)
    {
        int i;
        for (i=0; i<p_DpaPort->numOfTxFqs; i++)
        {
            if (p_DpaPort->qmTxQueuesParams[i].params.valid &&
                p_DpaPort->qmTxQueuesParams[i].params.fmOpParams.dedicatedRxQ &&
                (h_QmFqr == p_DpaPort->qmTxQueuesParams[i].h_Fqr))
            {
                fqidOffset = DPAPORT_SET_DEDICATED_Q(i);
                break;
            }
        }
        ASSERT_COND(i!=p_DpaPort->numOfTxFqs);
    }

    physBuffAddr = DPAA_FD_GET_PHYS_ADDR(p_DpaaFd);
    context.h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_FD_GET_BPID(p_DpaaFd));
    ASSERT_COND((p_DpaPort->type == e_DPA_PORT_T_OP) ||
                (p_DpaPort->type == e_DPA_PORT_T_VIRTUAL) ||
                context.h_BmPool);
    if (context.h_BmPool)
        p_Data = BM_POOL_PhysToVirt(context.h_BmPool, physBuffAddr);
    else
        p_Data = XX_PhysToVirt(physBuffAddr);

    context.h_UserPriv = ((t_Handle *)p_Data)[0];
    context.h_QmPortal = h_QmPortal;

    if (p_DpaPort->rxBuffInfo.hasFmBuffContext)
    {
        if ((err = HandleInFmBuffContext(p_DpaPort, p_Data, &context)) != E_OK)
        {
            REPORT_ERROR(MINOR, err, NO_MSG);
            return e_RX_STORE_RESPONSE_PAUSE;
        }
    }
    else
    {
        context.p_FmTimeStamp       = NULL;
        context.p_FmIntContext      = NULL;
        context.p_FmPrsResult       = NULL;
        context.p_FmKgHashResult    = NULL;
#ifdef DEBUG
        context.p_FmDebugInfo       = NULL;
#endif /* DEBUG */
    }

    if (DPAA_FD_GET_FORMAT(p_DpaaFd) != e_DPAA_FD_FORMAT_TYPE_SHORT_MBSF)
    {
        uint16_t length = (uint16_t)DPAA_FD_GET_LENGTH(p_DpaaFd);
        uint32_t status = DPAA_FD_GET_STATUS(p_DpaaFd) & FM_FD_RX_STATUS_ERR_MASK;

        DBG(TRACE, ("STD frame recv."));
        if (context.h_BmPool)
        {
            h_BufContext = BM_POOL_GetBufferContext(context.h_BmPool, p_Data);
            if (p_DpaPort->autoBmFill)
            {
                if (p_DpaPort->f_FillBmPortal)
                {
                    p_DpaPort->f_FillBmPortal(p_DpaPort->h_App, &context);
                    SANITY_CHECK_RETURN_VALUE(context.h_BmPortal, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
                }
                else
                    context.h_BmPortal = NULL;
                BM_POOL_FillBufs(context.h_BmPool, context.h_BmPortal, 1);
            }
        }
        p_Data = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data)+
                                                      DPAA_FD_GET_OFFSET(p_DpaaFd)));

        if (!p_DpaPort->numOfFwdDescs ||
            status ||
            (fwdTblIndx = GetFwdTblIndx(p_DpaPort, fqidOffset)) == DPAPORT_ILLEGAL_FWD_TBL_INDX)
            p_DpaPort->f_RxStore(p_DpaPort->h_App,
                                 fqidOffset,
                                 p_Data,
                                 length,
                                 status,
                                 SINGLE_BUF,
                                 h_BufContext,
                                 &context);
        else if ((err = DoFwd (p_DpaPort,
                               fwdTblIndx,
                               fqidOffset,
                               p_Data,
                               length,
                               SINGLE_BUF,
                               p_DpaaFd,
                               h_BufContext,
                               &context)) != E_OK)
        {
            REPORT_ERROR(MINOR, err, NO_MSG);
            //return e_RX_STORE_RESPONSE_PAUSE;
        }
    }
    else if ((err = HandleRxSGT(p_DpaPort,
                                fqidOffset,
                                p_Data,
                                p_DpaaFd,
                                &context)) != E_OK)
    {
        REPORT_ERROR(MINOR, err, NO_MSG);
        return e_RX_STORE_RESPONSE_PAUSE;
    }

    return e_RX_STORE_RESPONSE_CONTINUE;
}

static e_RxStoreResponse DpaFmTxConfCB(t_Handle h_Arg,
                                       t_Handle h_QmFqr,
                                       t_Handle h_QmPortal,
                                       uint32_t fqidOffset,
                                       t_DpaaFD *p_DpaaFd)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_Arg;
    t_DpaPortBuffContext    context;
    physAddress_t           physBuffAddr;
    uint8_t                 *p_Data;
    t_Error                 err;

    SANITY_CHECK_RETURN_VALUE(p_DpaPort, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);

    if (h_QmFqr == p_DpaPort->h_TxConfQmFqr)
        fqidOffset = DPAPORT_TX_CONF_Q;
    else if (h_QmFqr == p_DpaPort->h_TxErrQmFqr)
        fqidOffset = DPAPORT_TX_ERR_Q;
    else
        ASSERT_COND(FALSE);

    physBuffAddr = DPAA_FD_GET_PHYS_ADDR(p_DpaaFd);
    context.h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_FD_GET_BPID(p_DpaaFd));
    if (context.h_BmPool)
        p_Data = BM_POOL_PhysToVirt(context.h_BmPool, physBuffAddr);
    else
        p_Data = XX_PhysToVirt(physBuffAddr);

    memset(&context, 0, sizeof(context));
    context.h_UserPriv = ((t_Handle *)p_Data)[0];
    context.h_QmPortal = h_QmPortal;

    if (p_DpaPort->txBuffInfo.hasFmBuffContext)
    {
        if ((err = HandleInFmBuffContext(p_DpaPort, p_Data, &context)) != E_OK)
        {
            REPORT_ERROR(MINOR, err, NO_MSG);
            return e_RX_STORE_RESPONSE_PAUSE;
        }
    }
    else
    {
        context.p_FmTimeStamp       = NULL;
        context.p_FmIntContext      = NULL;
        context.p_FmPrsResult       = NULL;
        context.p_FmKgHashResult    = NULL;
#ifdef DEBUG
        context.p_FmDebugInfo       = NULL;
#endif /* DEBUG */
    }

    if (DPAA_FD_GET_FORMAT(p_DpaaFd) != e_DPAA_FD_FORMAT_TYPE_SHORT_MBSF)
    {
        DBG(TRACE, ("STD frame conf."));
        p_Data = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data)+
                                                      DPAA_FD_GET_OFFSET(p_DpaaFd)));

        if (p_DpaPort->f_TxConf)
            p_DpaPort->f_TxConf(p_DpaPort->h_App,
                                p_Data,
                                (DPAA_FD_GET_STATUS(p_DpaaFd) & FM_FD_TX_STATUS_ERR_MASK),
                                &context);
        else if (context.h_BmPool)
        {
            if (p_DpaPort->f_FillBmPortal)
            {
                p_DpaPort->f_FillBmPortal(p_DpaPort->h_App, &context);
                SANITY_CHECK_RETURN_VALUE(context.h_BmPortal, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
            }
            else
                context.h_BmPortal = NULL;
            BM_POOL_PutBuf(context.h_BmPool, context.h_BmPortal, p_Data);
        }
    }
    else if ((err = HandleTxCSGT(p_DpaPort,
                                 fqidOffset,
                                 p_Data,
                                 p_DpaaFd,
                                 &context)) != E_OK)
    {
        REPORT_ERROR(MINOR, err, NO_MSG);
        return e_RX_STORE_RESPONSE_PAUSE;
    }

    return e_RX_STORE_RESPONSE_CONTINUE;
}


static t_Error DpaSecTxCB (t_Handle              h_DpaPort,
                           uint32_t              queueId,
                           uint8_t               *p_Data,
                           uint16_t              length,
                           bool                  lastBuffer,
                           t_DpaPortBuffContext  *p_Context)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_DpaaFD                txFrame;
    t_Handle                h_BmPool = p_Context->h_BmPool;
    uint8_t                 *p_Buff = p_Data;
    uint32_t                fdStatus = 0;
    t_Error                 err = E_OK;

    UNUSED(lastBuffer);
    ASSERT_COND(p_DpaPort);
    SANITY_CHECK_RETURN_ERROR(p_Context, E_NULL_POINTER);

    /* if the data was not allocated from bm-pool,
       we need to reallocated and copy the data to bm-pool buffer
       so the HW can release it to the BM pool */
    if (!h_BmPool)
    {
        uint8_t *p_NewData;
        h_BmPool = GetBuffPoolHandleBySize(p_DpaPort, length);
        if (!h_BmPool)
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("Need to build scatter-gather frame!"));
        p_NewData = (uint8_t *)BM_POOL_GetBuf(h_BmPool, p_Context->h_BmPortal);
        if (!p_NewData)
            RETURN_ERROR(MINOR, E_NO_MEMORY, ("New data buffer for no-tx-conf!"));
        MemCpy32(p_NewData, p_Data, length);
        p_Buff = p_Data = p_NewData;
    }

    ASSERT_COND(lastBuffer);

    /* Build FD */
    txFrame.id = 0; /* set to zero to clear the LIODN-offset fields */
    DPAA_FD_SET_DD(&txFrame, (uint32_t)0x3);
    DPAA_FD_SET_PID(&txFrame, (uint32_t)0x3f);
    DPAA_FD_SET_BPID(&txFrame, (h_BmPool ? BM_POOL_GetId(h_BmPool) : (uint32_t)0xff));

    DPAA_FD_SET_ADDR(&txFrame, p_Buff);
    DPAA_FD_SET_OFFSET(&txFrame, 0);
    DPAA_FD_SET_LENGTH(&txFrame, length);

    if (p_DpaPort->p_TxSGTFrm)
    {
        DPAA_FD_SET_FORMAT(&txFrame, (uint32_t)e_DPAA_FD_FORMAT_TYPE_SHORT_MBSF);
        p_DpaPort->p_TxSGTFrm = NULL;
        DBG(TRACE, ("S/G frame sent."));
    }
    else
    {
        DPAA_FD_SET_FORMAT(&txFrame, (uint32_t)e_DPAA_FD_FORMAT_TYPE_SHORT_SBSF);
        DBG(TRACE, ("STD frame sent."));
    }

    DPAA_FD_SET_STATUS(&txFrame, fdStatus);

    err = QM_FQR_Enqueue(p_DpaPort->h_TxsQmFqr[queueId], p_Context->h_QmPortal, 0, &txFrame);

    return err;
}

static e_RxStoreResponse DpaSecRxCB(t_Handle h_Arg,
                                    t_Handle h_QmFqr,
                                    t_Handle h_QmPortal,
                                    uint32_t fqidOffset,
                                    t_DpaaFD *p_DpaaFd)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_Arg;
    t_Handle                h_BufContext = NULL;
    t_DpaPortBuffContext    context;
    physAddress_t           physBuffAddr;
    uint8_t                 *p_Data = NULL;
    uint8_t                 fwdTblIndx;
    t_Error                 err;
    int                     i;

    for (i=0; i<p_DpaPort->numOfTxFqs; i++)
    {
        ASSERT_COND(p_DpaPort->qmTxQueuesParams[i].params.valid);
        if (h_QmFqr == p_DpaPort->qmTxQueuesParams[i].h_Fqr)
        {
            fqidOffset = DPAPORT_SET_DEDICATED_Q(i);
            break;
        }
    }
    ASSERT_COND(i!=p_DpaPort->numOfTxFqs);

    SANITY_CHECK_RETURN_VALUE(p_DpaPort, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);

    memset(&context, 0, sizeof(context));

    if (DPAA_FD_GET_FORMAT(p_DpaaFd) != e_DPAA_FD_FORMAT_TYPE_SHORT_MBSF)
    {
        uint16_t length = (uint16_t)DPAA_FD_GET_LENGTH(p_DpaaFd);
        uint32_t status = DPAA_FD_GET_STATUS(p_DpaaFd);

        DBG(TRACE, ("STD frame recv."));

        if (!(status & 0x50000008))
        {
            context.h_BmPool = GetBuffPoolHandleById(p_DpaPort, (uint8_t)DPAA_FD_GET_BPID(p_DpaaFd));
            ASSERT_COND(context.h_BmPool);
            physBuffAddr = DPAA_FD_GET_PHYS_ADDR(p_DpaaFd);
            p_Data = BM_POOL_PhysToVirt(context.h_BmPool, physBuffAddr);
            p_Data = CAST_UINT64_TO_POINTER_TYPE(uint8_t, (CAST_POINTER_TO_UINT64(p_Data)+
                                                          DPAA_FD_GET_OFFSET(p_DpaaFd)));
        }
        context.h_QmPortal = h_QmPortal;

        if (!p_DpaPort->numOfFwdDescs ||
            status ||
            (fwdTblIndx = GetFwdTblIndx(p_DpaPort, fqidOffset)) == DPAPORT_ILLEGAL_FWD_TBL_INDX)
            p_DpaPort->f_RxStore(p_DpaPort->h_App,
                                 fqidOffset,
                                 p_Data,
                                 length,
                                 status,
                                 SINGLE_BUF,
                                 h_BufContext,
                                 &context);
        else if ((err = DoFwd (p_DpaPort,
                               fwdTblIndx,
                               fqidOffset,
                               p_Data,
                               length,
                               SINGLE_BUF,
                               p_DpaaFd,
                               h_BufContext,
                               &context)) != E_OK)
        {
            REPORT_ERROR(MINOR, err, NO_MSG);
            //return e_RX_STORE_RESPONSE_PAUSE;
        }
    }
    else
        REPORT_ERROR(MINOR, E_NOT_SUPPORTED, ("SGT for SEC!"));

    return e_RX_STORE_RESPONSE_CONTINUE;
}

static t_Error InitTxQueue(t_DpaPort    *p_DpaPort,
                           uint32_t     indx,
                           uint32_t     fqid,
                           uint32_t     forcedDedicateFqid)
{
    bool            useCA = FALSE;
    t_QmFqrParams   fqrParam;
    uint32_t        tmpBaseFqid=0;

    memset(&fqrParam, 0, sizeof(fqrParam));
    fqrParam.h_Qm                   = p_DpaPort->h_Qm;
    fqrParam.h_QmPortal             = p_DpaPort->h_QmPortal;
    fqrParam.channel                = (e_QmFQChannel)p_DpaPort->qmTxChannel;
    fqrParam.qs.nonFrcQs.numOfFqids = 1;
    fqrParam.useForce               = TRUE;
    fqrParam.qs.frcQ.fqid           = fqid;
    fqrParam.wq                     = (uint8_t)(indx % QM_MAX_NUM_OF_WQ);

    if (p_DpaPort->qmTxQueuesParams[indx].params.valid)
    {
        switch (p_DpaPort->type)
        {
            case e_DPA_PORT_T_OP:
            {
                t_FmContextA    fmContextA;

                memset(&fmContextA, 0, sizeof(t_FmContextA));

                if (p_DpaPort->qmTxQueuesParams[indx].params.fmOpParams.setFlow)
                {
                    FM_CONTEXTA_SET_A1_VALID(&fmContextA, TRUE);
                    FM_CONTEXTA_SET_A1(&fmContextA,
                                       (p_DpaPort->qmTxQueuesParams[indx].params.fmOpParams.flowId << 4));
                    if (p_DpaPort->qmTxQueuesParams[indx].params.fmOpParams.dedicatedRxQ)
                    {
                        t_FmContextB        fmContextB;
                        t_QmRsrvFqrParams   rsrvFqrParam;

                        memset(&rsrvFqrParam, 0, sizeof(rsrvFqrParam));
                        rsrvFqrParam.useForce               = FALSE;
                        rsrvFqrParam.qs.nonFrcQs.numOfFqids = 1;
                        rsrvFqrParam.qs.nonFrcQs.align      = 1;
                        if (QM_ReserveQueues(p_DpaPort->h_Qm, &rsrvFqrParam, &tmpBaseFqid) != E_OK)
                        {
                            DPAPORT_Free(p_DpaPort);
                            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port TX FQRs!"));
                        }

                        if (!forcedDedicateFqid)
                        {
                            t_QmFqrParams       tmpFqrParam;
                            memset(&tmpFqrParam, 0, sizeof(tmpFqrParam));
                            tmpFqrParam.h_Qm                    = p_DpaPort->h_Qm;
                            tmpFqrParam.h_QmPortal              = p_DpaPort->h_QmPortal;
                            tmpFqrParam.wq                      = 0;
                            tmpFqrParam.channel                 = (e_QmFQChannel)p_DpaPort->qmRxChannel;
                            tmpFqrParam.useForce                = TRUE;
                            tmpFqrParam.qs.frcQ.fqid            = tmpBaseFqid;
                            p_DpaPort->qmTxQueuesParams[indx].h_Fqr = QM_FQR_Create(&tmpFqrParam);
                            if (!p_DpaPort->qmTxQueuesParams[indx].h_Fqr)
                            {
                                DPAPORT_Free(p_DpaPort);
                                RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port TX FQR!"));
                            }
                            QM_FQR_RegisterCB(p_DpaPort->qmTxQueuesParams[indx].h_Fqr,DpaFmRxCB,p_DpaPort);
                            forcedDedicateFqid = QM_FQR_GetFqid(p_DpaPort->qmTxQueuesParams[indx].h_Fqr);
                        }
                        memset(&fmContextB, 0, sizeof(t_FmContextB));
                        FM_CONTEXTB_SET_FQID(&fmContextB, forcedDedicateFqid);
                        FM_CONTEXTA_SET_OVVERIDE(&fmContextA, TRUE);
                        fqrParam.p_ContextB = (t_QmContextB *)&fmContextB;
                    }
                    useCA = TRUE;
                }
                if (useCA)
                    fqrParam.p_ContextA = (t_QmContextA *)&fmContextA;
                break;
            }

            case e_DPA_PORT_T_RX_TX:
            {
                t_FmContextA    fmContextA;

                memset(&fmContextA, 0, sizeof(t_FmContextA));

                if (p_DpaPort->qmTxQueuesParams[indx].params.fmRxTxParams.dedicatedTxConfQ)
                {
                    t_FmContextB        fmContextB;
                    t_QmRsrvFqrParams   rsrvFqrParam;

                    memset(&rsrvFqrParam, 0, sizeof(rsrvFqrParam));
                    rsrvFqrParam.useForce               = FALSE;
                    rsrvFqrParam.qs.nonFrcQs.numOfFqids = 1;
                    rsrvFqrParam.qs.nonFrcQs.align      = 1;
                    if (QM_ReserveQueues(p_DpaPort->h_Qm, &rsrvFqrParam, &tmpBaseFqid) != E_OK)
                    {
                        DPAPORT_Free(p_DpaPort);
                        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port TX FQRs!"));
                    }

                    if (!forcedDedicateFqid)
                    {
                        t_QmFqrParams       tmpFqrParam;
                        memset(&tmpFqrParam, 0, sizeof(tmpFqrParam));
                        tmpFqrParam.h_Qm                    = p_DpaPort->h_Qm;
                        tmpFqrParam.h_QmPortal              = p_DpaPort->h_QmPortal;
                        tmpFqrParam.wq                      = 0;
                        tmpFqrParam.channel                 = (e_QmFQChannel)p_DpaPort->qmRxChannel;
                        tmpFqrParam.useForce                = TRUE;
                        tmpFqrParam.qs.frcQ.fqid            = tmpBaseFqid;
                        p_DpaPort->qmTxQueuesParams[indx].h_Fqr = QM_FQR_Create(&tmpFqrParam);
                        if (!p_DpaPort->qmTxQueuesParams[indx].h_Fqr)
                        {
                            DPAPORT_Free(p_DpaPort);
                            RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port TX FQR!"));
                        }
                        QM_FQR_RegisterCB(p_DpaPort->qmTxQueuesParams[indx].h_Fqr,DpaFmTxConfCB,p_DpaPort);
                        forcedDedicateFqid = QM_FQR_GetFqid(p_DpaPort->qmTxQueuesParams[indx].h_Fqr);
                    }
                    memset(&fmContextB, 0, sizeof(t_FmContextB));
                    FM_CONTEXTB_SET_FQID(&fmContextB, forcedDedicateFqid);
                    FM_CONTEXTA_SET_OVVERIDE(&fmContextA, TRUE);
                    fqrParam.p_ContextB = (t_QmContextB *)&fmContextB;
                    useCA = TRUE;
                }
                if (useCA)
                    fqrParam.p_ContextA = (t_QmContextA *)&fmContextA;
                break;
            }

            case e_DPA_PORT_T_SEC:
            {
                t_SecContextA       secContextA;
                t_SecContextB       secContextB;
                t_QmRsrvFqrParams   rsrvFqrParam;
                uint32_t            sharedLength = 0;

                memset(&rsrvFqrParam, 0, sizeof(rsrvFqrParam));
                rsrvFqrParam.useForce               = FALSE;
                rsrvFqrParam.qs.nonFrcQs.numOfFqids = 1;
                rsrvFqrParam.qs.nonFrcQs.align      = 1;
                if (QM_ReserveQueues(p_DpaPort->h_Qm, &rsrvFqrParam, &tmpBaseFqid) != E_OK)
                {
                    DPAPORT_Free(p_DpaPort);
                    RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port TX FQRs!"));
                }

                if (!forcedDedicateFqid)
                {
                    t_QmFqrParams       tmpFqrParam;
                    memset(&tmpFqrParam, 0, sizeof(tmpFqrParam));
                    tmpFqrParam.h_Qm                    = p_DpaPort->h_Qm;
                    tmpFqrParam.h_QmPortal              = p_DpaPort->h_QmPortal;
                    tmpFqrParam.wq                      = 0;
                    tmpFqrParam.channel                 = (e_QmFQChannel)p_DpaPort->qmRxChannel;
                    tmpFqrParam.useForce                = TRUE;
                    tmpFqrParam.qs.frcQ.fqid            = tmpBaseFqid;
                    p_DpaPort->qmTxQueuesParams[indx].h_Fqr = QM_FQR_Create(&tmpFqrParam);
                    if (!p_DpaPort->qmTxQueuesParams[indx].h_Fqr)
                    {
                        DPAPORT_Free(p_DpaPort);
                        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port TX FQR!"));
                    }
                    QM_FQR_RegisterCB(p_DpaPort->qmTxQueuesParams[indx].h_Fqr,DpaSecRxCB,p_DpaPort);
                    forcedDedicateFqid = QM_FQR_GetFqid(p_DpaPort->qmTxQueuesParams[indx].h_Fqr);
                }

                memset(&secContextA, 0, sizeof(t_SecContextA));
                memset(&secContextB, 0, sizeof(t_SecContextB));
                SEC_CONTEXTB_SET_FQID(&secContextB, forcedDedicateFqid);
                fqrParam.p_ContextB = (t_QmContextB *)&secContextB;

                if (p_DpaPort->qmTxQueuesParams[indx].params.secParams.p_SharedDesc)
                    sharedLength = p_DpaPort->qmTxQueuesParams[indx].params.secParams.sharedDescSize;

                p_DpaPort->qmTxQueuesParams[indx].p_PreHeader =
                    (t_SecPreHeader*)XX_MallocSmart(sizeof(t_SecPreHeader) + sharedLength, 0, 64);
                if (!p_DpaPort->qmTxQueuesParams[indx].p_PreHeader)
                {
                    DPAPORT_Free(p_DpaPort);
                    RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port SEC pre-hdr!"));
                }
                memset(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader,
                       0,
                       sizeof(t_SecPreHeader) + sharedLength);

                if (sharedLength)
                    memcpy(CAST_UINT64_TO_POINTER(CAST_POINTER_TO_UINT64(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader) +
                                                  sizeof(t_SecPreHeader)),
                           p_DpaPort->qmTxQueuesParams[indx].params.secParams.p_SharedDesc,
                           sharedLength);
                SEC_PH_SET_SDLEN(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader, sharedLength>>2);
                SEC_PH_SET_POOLID(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader,
                                  BM_POOL_GetId(p_DpaPort->p_Pools[p_DpaPort->qmTxQueuesParams[indx].params.secParams.bmPoolIndx]));
                SEC_PH_SET_BUFSIZE(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader,
                                   BM_POOL_GetBufferSize(p_DpaPort->p_Pools[p_DpaPort->qmTxQueuesParams[indx].params.secParams.bmPoolIndx]));
                SEC_PH_SET_OFFSET(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader,
                                  p_DpaPort->rxBuffInfo.dataOffset/
                                  SEC_GetBurstSize(p_DpaPort->h_Sec));
                secContextA.addr_hi = (uint32_t)((uint64_t)XX_VirtToPhys(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader) >> 32);
                secContextA.addr_lo = (uint32_t)XX_VirtToPhys(p_DpaPort->qmTxQueuesParams[indx].p_PreHeader);
                fqrParam.p_ContextA = (t_QmContextA *)&secContextA;
                break;
            }

            default:
                DPAPORT_Free(p_DpaPort);
                RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("DPA-Port Type!"));
        }
    }

    p_DpaPort->h_TxsQmFqr[indx] = QM_FQR_Create(&fqrParam);
    if (!p_DpaPort->h_TxsQmFqr[indx])
    {
        DPAPORT_Free(p_DpaPort);
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port TX FQR!"));
    }

    return E_OK;
}

static t_Error InitTxQueues(t_DpaPort *p_DpaPort, uint8_t align)
{
    t_QmFqrParams   fqrParam;
    uint32_t        baseFqid=0;
    int             i;

    if (!p_DpaPort->numOfTxFqs)
        return E_OK;

    memset(&fqrParam, 0, sizeof(fqrParam));
    fqrParam.h_Qm                   = p_DpaPort->h_Qm;
    fqrParam.h_QmPortal             = p_DpaPort->h_QmPortal;
    fqrParam.useForce               = FALSE;
    fqrParam.qs.nonFrcQs.align      = align;
    fqrParam.channel                = (e_QmFQChannel)p_DpaPort->qmTxChannel;
    fqrParam.qs.nonFrcQs.numOfFqids = 1;
    p_DpaPort->h_TxsQmFqr = (t_Handle *)XX_Malloc(p_DpaPort->numOfTxFqs*sizeof(t_Handle));
    if (!p_DpaPort->h_TxsQmFqr)
    {
        DPAPORT_Free(p_DpaPort);
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port TX FQRs!"));
    }
    memset(p_DpaPort->h_TxsQmFqr, 0, (p_DpaPort->numOfTxFqs*sizeof(t_Handle)));

    if (p_DpaPort->consecutiveTxQueues)
    {
        t_QmRsrvFqrParams   rsrvFqrParam;

        memset(&rsrvFqrParam, 0, sizeof(rsrvFqrParam));
        rsrvFqrParam.useForce               = FALSE;
        rsrvFqrParam.qs.nonFrcQs.numOfFqids = p_DpaPort->numOfTxFqs;
        rsrvFqrParam.qs.nonFrcQs.align      = fqrParam.qs.nonFrcQs.align;
        if (QM_ReserveQueues(p_DpaPort->h_Qm, &rsrvFqrParam, &baseFqid) != E_OK)
        {
            DPAPORT_Free(p_DpaPort);
            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port TX FQRs!"));
        }
    }

    for (i=0; i<p_DpaPort->numOfTxFqs; i++)
    {
        if (p_DpaPort->consecutiveTxQueues)
            InitTxQueue(p_DpaPort, (uint32_t)i, baseFqid+i, 0);
        else
        {
            t_QmRsrvFqrParams   rsrvFqrParam;

            memset(&rsrvFqrParam, 0, sizeof(rsrvFqrParam));
            rsrvFqrParam.useForce               = FALSE;
            rsrvFqrParam.qs.nonFrcQs.numOfFqids = 1;
            rsrvFqrParam.qs.nonFrcQs.align      = 1;
            if (QM_ReserveQueues(p_DpaPort->h_Qm, &rsrvFqrParam, &baseFqid) != E_OK)
            {
                DPAPORT_Free(p_DpaPort);
                RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port TX FQRs!"));
            }
            InitTxQueue(p_DpaPort, (uint32_t)i, baseFqid, 0);
        }
    }

    return E_OK;
}


t_Handle DPAPORT_Config(t_DpaPortParams *p_DpaPortParams)
{
    t_DpaPort       *p_DpaPort;

    SANITY_CHECK_RETURN_VALUE(p_DpaPortParams, E_INVALID_HANDLE, NULL);
    SANITY_CHECK_RETURN_VALUE(p_DpaPortParams->numOfQmTxQueues <= DPAPORT_MAX_NUM_OF_TX_QUEUES, E_INVALID_HANDLE, NULL);

    p_DpaPort = (t_DpaPort*)XX_Malloc(sizeof(t_DpaPort));
    if (!p_DpaPort)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port obj"));
        return NULL;
    }
    memset(p_DpaPort, 0, sizeof(t_DpaPort));
    p_DpaPort->p_DpaPortDriverParams = (t_DpaPortDriverParams *)XX_Malloc(sizeof(t_DpaPortDriverParams));
    if (!p_DpaPort->p_DpaPortDriverParams)
    {
        XX_Free(p_DpaPort);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Qm driver parameters"));
        return NULL;
    }
    memset(p_DpaPort->p_DpaPortDriverParams, 0, sizeof(t_DpaPortDriverParams));

    p_DpaPort->type             = p_DpaPortParams->type;
    p_DpaPort->f_RxStore        = p_DpaPortParams->f_RxStore;
    p_DpaPort->f_TxConf         = p_DpaPortParams->f_TxConf;
    p_DpaPort->h_App            = p_DpaPortParams->h_App;
    p_DpaPort->h_Qm             = p_DpaPortParams->h_Qm;
    p_DpaPort->numOfTxFqs       = p_DpaPortParams->numOfQmTxQueues;
    p_DpaPort->qmTxChannel      = p_DpaPortParams->qmTxChannel;
    p_DpaPort->qmRxChannel      = p_DpaPortParams->qmRxChannel;
    p_DpaPort->autoBmFill       = DEFAULT_autoBmFill;
    p_DpaPort->autoBmAlloc      = DEFAULT_autoBmAlloc;
    p_DpaPort->numOfBmPools     = p_DpaPortParams->numOfBmPools;
    p_DpaPort->userBmPools      = p_DpaPortParams->userBmPools;
    if (p_DpaPort->type == e_DPA_PORT_T_SEC)
    {
        p_DpaPort->h_Sec        = p_DpaPortParams->h_Sec;
        p_DpaPort->autoBmAlloc  = TRUE;
    }
    else if ((p_DpaPort->type == e_DPA_PORT_T_RX_TX) ||
             (p_DpaPort->type == e_DPA_PORT_T_IM_RX_TX))
        p_DpaPort->h_FmMacDev   = p_DpaPortParams->h_FmMacDev;
    else if (p_DpaPort->type == e_DPA_PORT_T_OP)
        p_DpaPort->autoBmAlloc  = FALSE;

    p_DpaPort->stash            = DEFAULT_stashingEn;

    p_DpaPort->p_DpaPortDriverParams->txQsAlign     = DEFAULT_txFqidsAlign;
    p_DpaPort->p_DpaPortDriverParams->h_Bm          = p_DpaPortParams->h_Bm;
    memcpy(&p_DpaPort->p_DpaPortDriverParams->bmPoolsParams, &p_DpaPortParams->bmPoolsParams, sizeof(p_DpaPortParams->bmPoolsParams));

    return (t_Handle)p_DpaPort;
}

t_Error  DPAPORT_Init(t_Handle h_DpaPort)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;
    t_BmPoolParam   bmPoolParam;
    t_QmFqrParams   fqrParam;
    int             i;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);

    if (p_DpaPort->type != e_DPA_PORT_T_IM_RX_TX)
    {
        p_DpaPort->p_Pools = (t_Handle*)XX_Malloc(p_DpaPort->numOfBmPools*sizeof(t_Handle));
        if (!p_DpaPort->p_Pools)
        {
            DPAPORT_Free(p_DpaPort);
            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port pool array"));
        }
        memset(p_DpaPort->p_Pools, 0, (p_DpaPort->numOfBmPools*sizeof(t_Handle)));

        for (i=0; i<p_DpaPort->numOfBmPools;i++)
        {
            if (!p_DpaPort->userBmPools)
            {
                memset(&bmPoolParam, 0, sizeof(bmPoolParam));
                bmPoolParam.h_Bm            = p_DpaPort->p_DpaPortDriverParams->h_Bm;
                bmPoolParam.numOfBuffers    = DPAPORT_NUM_BUFFERS;
                /* TODO - check non user pool params
                bmPoolParam.bufferPoolInfo.h_BufferPool = p_App->p_BuffPool;
                bmPoolParam.bufferPoolInfo.f_GetBuf     = DATA_POOL_DefaultGetBuf;
                bmPoolParam.bufferPoolInfo.f_PutBuf     = DATA_POOL_DefaultPutBuf;
                bmPoolParam.bufferPoolInfo.bufferSize   = p_DpaPortParams->bufferSizeMEM_GetSize(p_App->p_BuffPool->h_MemMng);
                bmPoolParam.bufferPoolInfo.f_PhysToVirt = XX_PhysToVirt;
                bmPoolParam.bufferPoolInfo.f_VirtToPhys = XX_VirtToPhys;
*/              p_DpaPort->p_Pools[i] = BM_POOL_Config(&bmPoolParam);
                if(!p_DpaPort->p_Pools[i])
                {
                    DPAPORT_Free(p_DpaPort);
                    RETURN_ERROR(MAJOR, E_NO_MEMORY, ("DPA-Port BM pool"));
                }
                BM_POOL_Init(p_DpaPort->p_Pools[i]);
            }
            else
                p_DpaPort->p_Pools[i] = p_DpaPort->p_DpaPortDriverParams->bmPoolsParams[i].h_BmPool;

            p_DpaPort->bpidToPoolIndx[BM_POOL_GetId(p_DpaPort->p_Pools[i])] = (uint8_t)(i+1);
        }

        p_DpaPort->rxBuffInfo.privDataSize    = 8;
        p_DpaPort->txBuffInfo.privDataSize    = 8;

        if (p_DpaPort->type == e_DPA_PORT_T_SEC)
            FillSecBuffInfo(p_DpaPort);

        if (p_DpaPort->type != e_DPA_PORT_T_VIRTUAL)
        {
            InitTxQueues(p_DpaPort, p_DpaPort->p_DpaPortDriverParams->txQsAlign);

            memset(&fqrParam, 0, sizeof(fqrParam));
            fqrParam.h_Qm                   = p_DpaPort->h_Qm;
            fqrParam.h_QmPortal             = p_DpaPort->h_QmPortal;
            fqrParam.useForce               = FALSE;
            fqrParam.wq                     = 0;
            fqrParam.channel                = (e_QmFQChannel)p_DpaPort->qmRxChannel;
            fqrParam.qs.nonFrcQs.numOfFqids = 1;
            fqrParam.qs.nonFrcQs.align      = 1;
            if (p_DpaPort->f_TxConf)
            {
                p_DpaPort->h_TxConfQmFqr    = QM_FQR_Create(&fqrParam);
                if (!p_DpaPort->h_TxConfQmFqr)
                {
                    DPAPORT_Free(p_DpaPort);
                    RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port TX Conf FQR!"));
                }
                QM_FQR_RegisterCB(p_DpaPort->h_TxConfQmFqr,DpaFmTxConfCB,p_DpaPort);
            }

            if (p_DpaPort->type == e_DPA_PORT_T_RX_TX)
            {
                p_DpaPort->h_TxErrQmFqr     = QM_FQR_Create(&fqrParam);
                if (!p_DpaPort->h_TxErrQmFqr)
                {
                    DPAPORT_Free(p_DpaPort);
                    RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port TX Err FQR!"));
                }
                QM_FQR_RegisterCB(p_DpaPort->h_TxErrQmFqr,DpaFmTxConfCB,p_DpaPort);
            }

            p_DpaPort->h_RxErrQmFqr     = QM_FQR_Create(&fqrParam);
            if (!p_DpaPort->h_RxErrQmFqr)
            {
                DPAPORT_Free(p_DpaPort);
                RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port RX Err FQR!"));
            }
            if (p_DpaPort->stash)
            {
                //fqrParam.useContextAForStash = TRUE;
                memset(&fqrParam.stashingParams, 0, sizeof(fqrParam.stashingParams));
                //fqrParam.stashingParams.frameDataSize = 64;
                fqrParam.stashingParams.fqContextSize = 64;
            }
            p_DpaPort->h_RxDfltQmFqr    = QM_FQR_Create(&fqrParam);
            if (!p_DpaPort->h_RxDfltQmFqr)
            {
                DPAPORT_Free(p_DpaPort);
                RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("DPA-Port RX Dflt FQR!"));
            }
            QM_FQR_RegisterCB(p_DpaPort->h_RxDfltQmFqr,DpaFmRxCB,p_DpaPort);
            QM_FQR_RegisterCB(p_DpaPort->h_RxErrQmFqr,DpaFmRxCB,p_DpaPort);
        }

        if (p_DpaPort->type == e_DPA_PORT_T_SEC)
            p_DpaPort->f_DpaTx = DpaSecTxCB;
        else
            p_DpaPort->f_DpaTx = DpaFmTxCB;
    }
    else
        p_DpaPort->f_DpaTx = DpaFmImTxCB;

    XX_Free(p_DpaPort->p_DpaPortDriverParams);
    p_DpaPort->p_DpaPortDriverParams = NULL;

    return E_OK;
}

t_Error DPAPORT_Free(t_Handle h_DpaPort)
{
    t_DpaPort   *p_DpaPort = (t_DpaPort*)h_DpaPort;
    int         i;

    if (!p_DpaPort)
        return ERROR_CODE(E_INVALID_HANDLE);

    if (p_DpaPort->p_Pools)
    {
        if (!p_DpaPort->userBmPools)
            for (i=0; i<p_DpaPort->numOfBmPools; i++)
                if (p_DpaPort->p_Pools[i])
                    BM_POOL_Free(p_DpaPort->p_Pools[i]);

        XX_Free(p_DpaPort->p_Pools);
    }

    if (p_DpaPort->h_RxPcdQmFqr)
        QM_FQR_Free(p_DpaPort->h_RxPcdQmFqr);
    if (p_DpaPort->h_TxsQmFqr)
    {
        for (i=0; i<p_DpaPort->numOfTxFqs; i++)
        {
            if (p_DpaPort->h_TxsQmFqr[i])
                QM_FQR_Free(p_DpaPort->h_TxsQmFqr[i]);
            if (p_DpaPort->qmTxQueuesParams[i].h_Fqr)
                QM_FQR_Free(p_DpaPort->qmTxQueuesParams[i].h_Fqr);
            if (p_DpaPort->qmTxQueuesParams[i].p_PreHeader)
                XX_FreeSmart(p_DpaPort->qmTxQueuesParams[i].p_PreHeader);
        }
        XX_Free(p_DpaPort->h_TxsQmFqr);
    }
    if (p_DpaPort->h_TxConfQmFqr && !p_DpaPort->dummyTxCDefQ)
        QM_FQR_Free(p_DpaPort->h_TxConfQmFqr);
    if (p_DpaPort->h_TxErrQmFqr && !p_DpaPort->dummyTxErrDefQ)
        QM_FQR_Free(p_DpaPort->h_TxErrQmFqr);
    if (p_DpaPort->h_RxDfltQmFqr && !p_DpaPort->dummyRxDefQ)
        QM_FQR_Free(p_DpaPort->h_RxDfltQmFqr);
    if (p_DpaPort->h_RxErrQmFqr && !p_DpaPort->dummyRxErrDefQ)
        QM_FQR_Free(p_DpaPort->h_RxErrQmFqr);

    if (p_DpaPort->p_FwdTbl)
        XX_Free(p_DpaPort->p_FwdTbl);
    XX_Free(p_DpaPort);

    return E_OK;
}



t_Error  DPAPORT_ConfigQmTxQueueParams(t_Handle h_DpaPort, t_DpaPortQmTxQueueParams *p_QmTxQueuesParams)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;
    int             i;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_QmTxQueuesParams, E_INVALID_HANDLE);

    for (i=0; i<p_DpaPort->numOfTxFqs; i++)
    {
        memcpy(&p_DpaPort->qmTxQueuesParams[i].params,
               &p_QmTxQueuesParams[i],
               sizeof(t_DpaPortQmTxQueueParams));
    }

    return E_OK;
}

t_Error  DPAPORT_ConfigConsecutiveQmTxQueues(t_Handle h_DpaPort, bool val, uint8_t align)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);

    p_DpaPort->consecutiveTxQueues              = val;
    p_DpaPort->p_DpaPortDriverParams->txQsAlign = align;

    return E_OK;
}

t_Error  DPAPORT_ConfigFillBmSwPortalCallBack(t_Handle h_DpaPort, t_DpaPortFillBmSwPortalCallback *f_FillBmPortal)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(f_FillBmPortal, E_INVALID_HANDLE);

    p_DpaPort->f_FillBmPortal = f_FillBmPortal;

    return E_OK;
}

t_Error  DPAPORT_ConfigQmPortal(t_Handle h_DpaPort, t_Handle h_QmPortal)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(h_QmPortal, E_INVALID_HANDLE);

    p_DpaPort->h_QmPortal = h_QmPortal;

    return E_OK;
}

t_Error  DPAPORT_ConfigNoAutoBmFill(t_Handle h_DpaPort, bool noAutoBmFill)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);

    p_DpaPort->autoBmFill = !noAutoBmFill;

    return E_OK;
}

t_Error  DPAPORT_ConfigAutoBmAlloc(t_Handle h_DpaPort, bool en)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);

    p_DpaPort->autoBmAlloc = en;

    return E_OK;
}

t_Error  DPAPORT_ConfigStashingEn(t_Handle h_DpaPort, bool en)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);

    p_DpaPort->stash = en;

    return E_OK;
}

t_Error  DPAPORT_GetDpaParams(t_Handle h_DpaPort, t_DpaPortInternalParams *p_Params)
{
    t_DpaPort   *p_DpaPort = (t_DpaPort*)h_DpaPort;
    int         i;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);
    SANITY_CHECK_RETURN_ERROR((p_DpaPort->type != e_DPA_PORT_T_IM_RX_TX), E_INVALID_HANDLE);

    p_Params->rxPrivDataSize    = p_DpaPort->rxBuffInfo.privDataSize;
    p_Params->txPrivDataSize    = p_DpaPort->txBuffInfo.privDataSize;

    for (i=0; i<p_DpaPort->numOfTxFqs; i++)
        p_Params->txFqids[i]    = QM_FQR_GetFqid(p_DpaPort->h_TxsQmFqr[i]);
    if (p_DpaPort->h_TxErrQmFqr)
        p_Params->txErrFqid     = QM_FQR_GetFqid(p_DpaPort->h_TxErrQmFqr);
    if (p_DpaPort->h_TxConfQmFqr)
        p_Params->txConfFqid    = QM_FQR_GetFqid(p_DpaPort->h_TxConfQmFqr);
    p_Params->rxErrFqid         = QM_FQR_GetFqid(p_DpaPort->h_RxErrQmFqr);
    p_Params->rxDfltFqid        = QM_FQR_GetFqid(p_DpaPort->h_RxDfltQmFqr);

    for (i=0; i<p_DpaPort->numOfBmPools; i++)
    {
        p_Params->bmPoolsParams[i].poolId     = BM_POOL_GetId(p_DpaPort->p_Pools[i]);
        p_Params->bmPoolsParams[i].bufferSize = BM_POOL_GetBufferSize(p_DpaPort->p_Pools[i]);
    }

    return E_OK;
}

t_Error DPAPORT_SetFmDevHandle(t_Handle h_DpaPort, t_DpaPortFmParams *p_Params)
{
    t_DpaPort    *p_DpaPort = (t_DpaPort*)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    if (p_DpaPort->type == e_DPA_PORT_T_VIRTUAL)
    {
        p_DpaPort->txBuffInfo.dataOffset          = p_Params->virt.txDataOffset;
        p_DpaPort->txBuffInfo.parserResultsOffset = p_Params->virt.txParserResultsOffset;
        p_DpaPort->rxBuffInfo.parserResultsOffset = p_Params->virt.rxParserResultsOffset;
        p_DpaPort->txBuffInfo.timeStampOffset     = p_Params->virt.txTimeStampOffset;
        p_DpaPort->rxBuffInfo.timeStampOffset     = p_Params->virt.rxTimeStampOffset;
    }
    else if (p_DpaPort->type == e_DPA_PORT_T_SEC)
        /* ???? */;
    else /* This is probably FM port (OP/IM/RXTX) */
    {
        t_Error err;
        p_DpaPort->h_TxFmPortDev = p_Params->h_TxFmPortDev;
        p_DpaPort->h_RxFmPortDev = p_Params->h_RxFmPortDev;
        if(p_DpaPort->type == e_DPA_PORT_T_OP)
            p_DpaPort->h_RxFmPortDev = p_DpaPort->h_TxFmPortDev;

        if ((err = FillFmBuffInfo(p_DpaPort, FALSE)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if ((err = FillFmBuffInfo(p_DpaPort, TRUE)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
    }

    p_DpaPort->rxBuffInfo.hasFmBuffContext =
        (bool)(p_DpaPort->rxBuffInfo.parserResults
               | p_DpaPort->rxBuffInfo.hashResult
               | p_DpaPort->rxBuffInfo.timeStamp
               | p_DpaPort->rxBuffInfo.passAllOtherPCDInfo
#ifdef DEBUG
               | p_DpaPort->rxBuffInfo.passDebugInfo
#endif /* DEBUG */
              );

    p_DpaPort->txBuffInfo.hasFmBuffContext =
        (bool)(p_DpaPort->txBuffInfo.parserResults
               | p_DpaPort->txBuffInfo.hashResult
               | p_DpaPort->txBuffInfo.timeStamp
               | p_DpaPort->txBuffInfo.passAllOtherPCDInfo
#ifdef DEBUG
               | p_DpaPort->txBuffInfo.passDebugInfo
#endif /* DEBUG */
              );

    CHECK_INIT_PARAMETERS(p_DpaPort, CheckInitParameters);

    return E_OK;
}

uint32_t DPAPORT_GetBufferDataOffset(t_Handle h_DpaPort, bool tx)
{
    t_DpaPort   *p_DpaPort = (t_DpaPort*)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    if (tx)
        return p_DpaPort->txBuffInfo.dataOffset;
    return p_DpaPort->rxBuffInfo.dataOffset;
}

t_Error DPAPORT_Enable(t_Handle h_DpaPort)
{
    t_DpaPort   *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_Error     err;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    if ((p_DpaPort->type == e_DPA_PORT_T_RX_TX) ||
        (p_DpaPort->type == e_DPA_PORT_T_IM_RX_TX))
    {
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_FmMacDev, E_INVALID_STATE);
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_TxFmPortDev, E_INVALID_STATE);
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_RxFmPortDev, E_INVALID_STATE);

        if ((err = FM_MAC_Enable(p_DpaPort->h_FmMacDev, e_COMM_MODE_TX)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if ((err = FM_PORT_Enable(p_DpaPort->h_TxFmPortDev)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if ((err = FM_PORT_Enable(p_DpaPort->h_RxFmPortDev)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if ((err = FM_MAC_Enable(p_DpaPort->h_FmMacDev, e_COMM_MODE_RX)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
    }
    else if(p_DpaPort->type == e_DPA_PORT_T_OP)
    {
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_TxFmPortDev, E_INVALID_STATE);
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_RxFmPortDev, E_INVALID_STATE);
        if ((err = FM_PORT_Enable(p_DpaPort->h_TxFmPortDev)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
    }

    return E_OK;
}

t_Error     DPAPORT_Disable(t_Handle h_DpaPort)
{
    t_DpaPort   *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_Error     err;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    if ((p_DpaPort->type == e_DPA_PORT_T_RX_TX) ||
        (p_DpaPort->type == e_DPA_PORT_T_IM_RX_TX))
    {
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_FmMacDev, E_INVALID_STATE);
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_TxFmPortDev, E_INVALID_STATE);
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_RxFmPortDev, E_INVALID_STATE);

        if ((err = FM_MAC_Disable(p_DpaPort->h_FmMacDev, e_COMM_MODE_RX)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if ((err = FM_PORT_Disable(p_DpaPort->h_RxFmPortDev)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if ((err = FM_PORT_Disable(p_DpaPort->h_TxFmPortDev)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if ((err = FM_MAC_Disable(p_DpaPort->h_FmMacDev, e_COMM_MODE_TX)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
    }
    else if(p_DpaPort->type == e_DPA_PORT_T_OP)
    {
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_TxFmPortDev, E_INVALID_STATE);
        SANITY_CHECK_RETURN_ERROR(p_DpaPort->h_RxFmPortDev, E_INVALID_STATE);
        if ((err = FM_PORT_Disable(p_DpaPort->h_TxFmPortDev)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
    }

    return E_OK;
}

uint32_t  DPAPORT_AllocRxQueues(t_Handle h_DpaPort, uint16_t numOfQueues, uint8_t align)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_QmFqrParams   fqrParam;

    SANITY_CHECK_RETURN_VALUE(p_DpaPort, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE, 0);
    SANITY_CHECK_RETURN_VALUE((p_DpaPort->type != e_DPA_PORT_T_IM_RX_TX), E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(numOfQueues, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(!p_DpaPort->h_RxPcdQmFqr, E_INVALID_STATE, 0);

    memset(&fqrParam, 0, sizeof(t_QmFqrParams));
    fqrParam.h_Qm                   = p_DpaPort->h_Qm;
    fqrParam.wq                     = 0;
    fqrParam.useForce               = FALSE;
    fqrParam.qs.nonFrcQs.align      = align;
    fqrParam.channel                = (e_QmFQChannel)p_DpaPort->qmRxChannel;
    fqrParam.qs.nonFrcQs.numOfFqids = numOfQueues;

    if (p_DpaPort->stash)
    {
        fqrParam.useContextAForStash = TRUE;
        //fqrParam.stashingParams.frameDataSize = 64;
        fqrParam.stashingParams.fqContextSize = 64;
    }

    p_DpaPort->h_RxPcdQmFqr         = QM_FQR_Create(&fqrParam);
    if (!p_DpaPort->h_RxPcdQmFqr)
    {
        REPORT_ERROR(MINOR, E_INVALID_HANDLE, ("rx-pcd-queues!"));
        return 0;
    }
    QM_FQR_RegisterCB(p_DpaPort->h_RxPcdQmFqr,DpaFmRxCB,p_DpaPort);

    return QM_FQR_GetFqid(p_DpaPort->h_RxPcdQmFqr);
}

uint32_t DPAPORT_AllocHwForwardQueues(t_Handle                  h_DpaPort,
                                      t_DpaPortHwFwdParams      *p_Params)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;
    t_DpaPort       *p_FwdDpaPort;

    SANITY_CHECK_RETURN_VALUE(p_DpaPort, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(p_Params->h_FwdDpaPort, E_INVALID_HANDLE, 0);

    p_FwdDpaPort = (t_DpaPort *)p_Params->h_FwdDpaPort;
    SANITY_CHECK_RETURN_VALUE(((p_Params->txBaseQueueId+p_Params->numOfQueues) <=
                               p_FwdDpaPort->numOfTxFqs),
                              E_INVALID_VALUE,
                              0);

    if (p_Params->rxBaseQueueId == DPAPORT_RX_DFLT_Q)
    {
        t_Handle    h_TmpFqr        = p_DpaPort->h_RxDfltQmFqr;
        SANITY_CHECK_RETURN_VALUE((p_Params->numOfQueues == 1), E_INVALID_VALUE, 0);
        p_DpaPort->h_RxDfltQmFqr    = p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId];
        p_DpaPort->dummyRxDefQ      = TRUE;
        QM_FQR_Free(h_TmpFqr);
    }
    else if (p_Params->rxBaseQueueId == DPAPORT_RX_ERR_Q)
    {
        t_Handle    h_TmpFqr        = p_DpaPort->h_RxErrQmFqr;
        SANITY_CHECK_RETURN_VALUE((p_Params->numOfQueues == 1), E_INVALID_VALUE, 0);
        p_DpaPort->h_RxErrQmFqr     = p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId];
        p_DpaPort->dummyRxErrDefQ   = TRUE;
        QM_FQR_Free(h_TmpFqr);
    }
    else if (p_Params->rxBaseQueueId == DPAPORT_TX_CONF_Q)
    {
        t_Handle    h_TmpFqr        = p_DpaPort->h_TxConfQmFqr;
        SANITY_CHECK_RETURN_VALUE(h_TmpFqr, E_INVALID_STATE, 0);
        SANITY_CHECK_RETURN_VALUE((p_Params->numOfQueues == 1), E_INVALID_VALUE, 0);
        p_DpaPort->h_TxConfQmFqr    = p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId];
        p_DpaPort->dummyTxCDefQ     = TRUE;
        QM_FQR_Free(h_TmpFqr);
    }
    else if (p_Params->rxBaseQueueId == DPAPORT_TX_ERR_Q)
    {
        t_Handle    h_TmpFqr        = p_DpaPort->h_TxErrQmFqr;
        SANITY_CHECK_RETURN_VALUE(h_TmpFqr, E_INVALID_STATE, 0);
        SANITY_CHECK_RETURN_VALUE((p_Params->numOfQueues == 1), E_INVALID_VALUE, 0);
        p_DpaPort->h_TxErrQmFqr     = p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId];
        p_DpaPort->dummyTxErrDefQ   = TRUE;
        QM_FQR_Free(h_TmpFqr);
    }
    else if (DPAPORT_IS_DEDICATED_Q(p_Params->rxBaseQueueId))
    {
        uint32_t        indx, fqid;

        SANITY_CHECK_RETURN_VALUE((p_Params->numOfQueues == 1), E_INVALID_VALUE, 0);

        indx    = DPAPORT_GET_DEDICATED_Q(p_Params->rxBaseQueueId);
        SANITY_CHECK_RETURN_VALUE(p_DpaPort->qmTxQueuesParams[indx].h_Fqr, E_INVALID_HANDLE, 0);

        /* Free the current dedicated rx-queue and the Tx queue associeted with it */
        fqid    = QM_FQR_GetFqid(p_DpaPort->h_TxsQmFqr[indx]);
        QM_FQR_Free(p_DpaPort->h_TxsQmFqr[indx]);
        p_DpaPort->h_TxsQmFqr[indx] = NULL;
        QM_FQR_Free(p_DpaPort->qmTxQueuesParams[indx].h_Fqr);
        p_DpaPort->qmTxQueuesParams[indx].h_Fqr = NULL;

        if (p_DpaPort->type == e_DPA_PORT_T_SEC)
        {
            SANITY_CHECK_RETURN_VALUE(!(p_FwdDpaPort->txBuffInfo.dataOffset %
                                        SEC_GetBurstSize(p_DpaPort->h_Sec)),
                                      E_INVALID_STATE, 0);
            SANITY_CHECK_RETURN_VALUE(((p_FwdDpaPort->txBuffInfo.dataOffset /
                                        SEC_GetBurstSize(p_DpaPort->h_Sec)) <= 4),
                                      E_INVALID_STATE, 0);
            p_DpaPort->rxBuffInfo.dataOffset  = p_FwdDpaPort->txBuffInfo.dataOffset;
        }

        /* Re-initilaize the tx-queue with new dedicated-queue */
        InitTxQueue(p_DpaPort,
                    indx,
                    fqid,
                    QM_FQR_GetFqid(p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId]));
    }
    else if (DPAPORT_IS_DEDICATED_Q(p_Params->txBaseQueueId))
        RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("hw-fwd to a dedicated q!"));
    else if (p_Params->numOfQueues > 1)
    {
        SANITY_CHECK_RETURN_VALUE(!p_DpaPort->numOfHwRxFqs, E_INVALID_STATE, 0);
        SANITY_CHECK_RETURN_VALUE(p_FwdDpaPort->consecutiveTxQueues, E_INVALID_STATE, 0);
        p_DpaPort->hwRxBaseFqids    =
            QM_FQR_GetFqid(p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId]);
        p_DpaPort->numOfHwRxFqs = p_Params->numOfQueues;
    }
    else
    {
        /* Do nothing */
    }

    return QM_FQR_GetFqid(p_FwdDpaPort->h_TxsQmFqr[p_Params->txBaseQueueId]);
}

t_Error  DPAPORT_AddForwardQueues(t_Handle                  h_DpaPort,
                                  t_DpaPortFwdParams        *p_Params)
{
    t_DpaPort       *p_DpaPort = (t_DpaPort *)h_DpaPort;
    t_DpaPort       *p_FwdDpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Params->h_FwdDpaPort, E_INVALID_HANDLE);

    p_FwdDpaPort = (t_DpaPort *)p_Params->h_FwdDpaPort;
    SANITY_CHECK_RETURN_ERROR(((p_Params->numOfQueues <= p_FwdDpaPort->numOfTxFqs) &&
                              (p_Params->txBaseQueueId < p_FwdDpaPort->numOfTxFqs)),
                              E_INVALID_VALUE);

    if (!p_DpaPort->p_FwdTbl)
    {
        p_DpaPort->p_FwdTbl = (t_DpaPortFwdEntry*)XX_Malloc(sizeof(t_DpaPortFwdEntry)*DPAPORT_MAX_NUM_OF_FWD_DESCS);
        if (!p_DpaPort->p_FwdTbl)
            RETURN_ERROR(MINOR, E_NO_MEMORY, ("DPA-port forward-table object!"));
        memset(p_DpaPort->p_FwdTbl, 0, sizeof(t_DpaPortFwdEntry)*DPAPORT_MAX_NUM_OF_FWD_DESCS);
    }

    if (p_DpaPort->numOfFwdDescs == DPAPORT_MAX_NUM_OF_FWD_DESCS)
        RETURN_ERROR(MINOR, E_INVALID_STATE, ("max number of fwd descriptor exceeded!"));

    ASSERT_COND(!p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].h_FwdDpaPort);

    p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].rxBaseQueueId = p_Params->rxBaseQueueId;
    p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].numOfQueues   = p_Params->numOfQueues;
    p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].h_FwdDpaPort  = p_Params->h_FwdDpaPort;
    p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].txBaseQueueId = p_Params->txBaseQueueId;
    p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].f_RxPreFwd    = p_Params->f_RxPreFwd;
    p_DpaPort->p_FwdTbl[p_DpaPort->numOfFwdDescs].h_App         = p_Params->h_App;
    p_DpaPort->numOfFwdDescs++;

    return E_OK;
}

t_Error  DPAPORT_Tx ( t_Handle              h_DpaPort,
                      uint32_t              queueId,
                      uint8_t               *p_Data,
                      uint16_t              length,
                      bool                  lastBuffer,
                      t_DpaPortBuffContext  *p_Context)
{
    t_DpaPort    *p_DpaPort = (t_DpaPort*)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    return p_DpaPort->f_DpaTx(h_DpaPort,queueId,p_Data,length,lastBuffer,p_Context);
}

void DPAPORT_TxConf(t_Handle h_DpaPort)
{
    t_DpaPort    *p_DpaPort = (t_DpaPort*)h_DpaPort;

    SANITY_CHECK_RETURN(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    if (p_DpaPort->type == e_DPA_PORT_T_IM_RX_TX)
        FM_PORT_ImTxConf(p_DpaPort->h_TxFmPortDev);
    else
        QM_Poll(p_DpaPort->h_Qm, e_QM_PORTAL_POLL_SOURCE_DATA_FRAMES);
}

t_Error DPAPORT_Rx(t_Handle h_DpaPort)
{
    t_DpaPort    *p_DpaPort = (t_DpaPort*)h_DpaPort;

    SANITY_CHECK_RETURN_ERROR(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    if (p_DpaPort->type == e_DPA_PORT_T_IM_RX_TX)
        return FM_PORT_ImRx(p_DpaPort->h_RxFmPortDev);
    else
        return QM_Poll(p_DpaPort->h_Qm, e_QM_PORTAL_POLL_SOURCE_DATA_FRAMES);
}

e_RxStoreResponse DPAPORT_ImRxStoreCB (t_Handle h_DpaPort,
                                       uint8_t  *p_Data,
                                       uint16_t length,
                                       uint16_t status,
                                       uint8_t  position,
                                       t_Handle h_BufContext)
{
    t_DpaPort    *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_Error      err;

    SANITY_CHECK_RETURN_VALUE(p_DpaPort, E_INVALID_HANDLE, e_RX_STORE_RESPONSE_PAUSE);
    SANITY_CHECK_RETURN_VALUE(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE, e_RX_STORE_RESPONSE_PAUSE);

    if (!p_DpaPort->numOfFwdDescs || status)
        p_DpaPort->f_RxStore(p_DpaPort->h_App,
                             0,
                             p_Data,
                             length,
                             (uint32_t)status,
                             position,
                             h_BufContext,
                             NULL);
    else if ((err = DoImFwd (p_DpaPort,
                             p_Data,
                             length,
                             position,
                             h_BufContext,
                             NULL)) != E_OK)
    {
        REPORT_ERROR(MINOR, err, NO_MSG);
        //return e_RX_STORE_RESPONSE_PAUSE;
    }

    return e_RX_STORE_RESPONSE_CONTINUE;
}

void DPAPORT_ImTxConfCB (t_Handle   h_DpaPort,
                         uint8_t    *p_Data,
                         uint16_t   status,
                         t_Handle   h_BufContext)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_DpaPort;
    t_DpaPortBuffContext    context;

    SANITY_CHECK_RETURN(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    memset(&context, 0, sizeof(context));
    context.h_UserPriv = h_BufContext;
    if (p_DpaPort->f_TxConf)
        p_DpaPort->f_TxConf(p_DpaPort->h_App,
                            p_Data,
                            (uint32_t)status,
                            &context);
}

void DPAPORT_ExceptionCB (t_Handle   h_DpaPort, uint32_t ex)
{
    t_DpaPort               *p_DpaPort = (t_DpaPort*)h_DpaPort;

    SANITY_CHECK_RETURN(p_DpaPort, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_DpaPort->p_DpaPortDriverParams, E_INVALID_STATE);

    UNUSED(p_DpaPort);
    DBG(INFO, ("DPA-port got exception 0x%08x", ex));
}
