/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          dpa_port.h

 @Description   DPAA internal structures and definitions.
*//***************************************************************************/
#ifndef __DPAA_H
#define __DPAA_H

#include "std_ext.h"
#include "error_ext.h"
#include "bm_ext.h"
#include "sec_qi_ext.h"
#include "dpa_port_ext.h"


#define __ERR_MODULE__  MODULE_FM

#define DPAPORT_NUM_BUFFERS             256
#define DPAPORT_MAX_NUM_OF_FWD_DESCS    16
#define DPAPORT_ILLEGAL_FWD_TBL_INDX    0xff

/**************************************************************************//**
 @Description       defaults
*//***************************************************************************/
#define DEFAULT_autoBmFill              TRUE
#define DEFAULT_txFqidsAlign            1
#define DEFAULT_stashingEn              FALSE
#define DEFAULT_autoBmAlloc             FALSE


typedef enum
{
    e_DPAPORT_CONFIG = 0,
    e_DPAPORT_INIT,
    e_DPAPORT_DEF_Q_FWD,
    e_DPAPORT_GET_PARAM,
    e_DPAPORT_SET_HNDLS,
    e_DPAPORT_PCD_Q_FWD,
    e_DPAPORT_PCD_Q_ALLOC
} e_DpaPortState;

typedef t_Error  (t_DpaTx) ( t_Handle              h_DpaPort,
                             uint32_t              queueId,
                             uint8_t               *p_Data,
                             uint16_t              length,
                             bool                  lastBuffer,
                             t_DpaPortBuffContext  *p_Context);

typedef struct t_DpaPortFwdEntry {
    uint32_t                    rxBaseQueueId;
    uint16_t                    numOfQueues;
    t_Handle                    h_FwdDpaPort;
    uint32_t                    txBaseQueueId;
    t_DpaPortRxStoreCallback    *f_RxPreFwd;
    t_Handle                    h_App;
} t_DpaPortFwdEntry;

typedef struct t_DpaPortDriverParams {
    t_Handle                    h_Bm;
    uint8_t                     txQsAlign;
    union {
        uint32_t                size;               /**< valid if userBmPools=FALSE: size of the BM pool */
        t_Handle                h_BmPool;           /**< valid if userBmPools=TRUE: user defined pool handle */
    } bmPoolsParams[DPAPORT_MAX_NUM_OF_BM_POOLS];   /**< An array of numOfBmPools entries of BM pool
                                                         sizes/handles - Depending on 'userBmPools'. */
} t_DpaPortDriverParams;

typedef struct t_DpaPortBuffPrefixInfo {
    uint32_t                    dataOffset;
    uint8_t                     privDataSize;
    bool                        hasFmBuffContext;
    bool                        parserResults;
    uint32_t                    parserResultsOffset;
    bool                        hashResult;
    uint32_t                    hashResultOffset;
    bool                        timeStamp;
    uint32_t                    timeStampOffset;
    bool                        passAllOtherPCDInfo;
    uint32_t                    allOtherPCDInfoOffset;
#ifdef DEBUG
    bool                        passDebugInfo;
    uint32_t                    debugInfoOffset;
#endif /* DEBUG */
} t_DpaPortBuffPrefixInfo;

typedef struct t_DpaPortQmTxQueueInfo {
    t_DpaPortQmTxQueueParams    params;
    t_Handle                    h_Fqr;
    t_SecPreHeader              *p_PreHeader;
} t_DpaPortQmTxQueueInfo;

typedef struct t_DpaPort {
    e_DpaPortType               type;
    t_Handle                    h_TxFmPortDev;
    t_Handle                    h_RxFmPortDev;
    t_Handle                    h_FmMacDev;
    t_Handle                    h_Sec;

    bool                        autoBmFill;
    bool                        autoBmAlloc;
    t_DpaPortRxStoreCallback    *f_RxStore;
    t_DpaPortTxConfCallback     *f_TxConf;
    t_DpaPortFillBmSwPortalCallback *f_FillBmPortal;       /**< TODO */
    t_Handle                    h_App;
    t_DpaTx                     *f_DpaTx;

    t_Handle                    h_Qm;
    t_Handle                    h_QmPortal;
    t_Handle                    h_FmMacsecSecY;
    uint32_t                    qmRxChannel;
    uint32_t                    qmTxChannel;
    uint32_t                    numOfTxFqs;
    bool                        consecutiveTxQueues;
    t_DpaPortQmTxQueueInfo      qmTxQueuesParams[DPAPORT_MAX_NUM_OF_TX_QUEUES];
    t_Handle                    *h_TxsQmFqr;
    bool                        dummyTxErrDefQ;
    t_Handle                    h_TxErrQmFqr;
    bool                        dummyTxCDefQ;
    t_Handle                    h_TxConfQmFqr;
    bool                        dummyRxErrDefQ;
    t_Handle                    h_RxErrQmFqr;
    bool                        dummyRxDefQ;
    t_Handle                    h_RxDfltQmFqr;
    uint32_t                    numOfRxFqs;
    t_Handle                    h_RxPcdQmFqr;
    uint32_t                    numOfHwRxFqs;
    uint32_t                    hwRxBaseFqids;

    uint8_t                     numOfBmPools;
    bool                        userBmPools;
    t_Handle                    *p_Pools;
    uint8_t                     bpidToPoolIndx[256];

    uint8_t                     numOfFwdDescs;
    t_DpaPortFwdEntry           *p_FwdTbl;

    t_DpaPortBuffPrefixInfo     txBuffInfo;
    t_DpaPortBuffPrefixInfo     rxBuffInfo;

    t_DpaaSGT                   *p_TxSGTFrm;
    t_Handle                    h_SGTBmPool;
    uint8_t                     txSGTFrmEntryPos;
    uint16_t                    txSGTFrmAccumLength;

    bool                        stash;

    t_DpaPortDriverParams       *p_DpaPortDriverParams;
} t_DpaPort;


static __inline__ t_Handle GetBuffPoolHandleById(t_DpaPort *p_DpaPort, uint8_t bpid)
{
    if (p_DpaPort->bpidToPoolIndx[bpid])
        return p_DpaPort->p_Pools[p_DpaPort->bpidToPoolIndx[bpid]-1];
    else
        return NULL;
}

static __inline__ t_Handle GetBuffPoolHandleBySize(t_DpaPort *p_DpaPort, uint16_t size)
{
    int i;
    for(i=0;i<p_DpaPort->numOfBmPools;i++)
        if (BM_POOL_GetBufferSize(p_DpaPort->p_Pools[i]) >= size)
            return p_DpaPort->p_Pools[i];
    return NULL;
}

static __inline__ uint8_t GetFwdTblIndx(t_DpaPort *p_DpaPort, uint32_t rxQueueId)
{
    uint8_t i;

    for (i=0; i<DPAPORT_MAX_NUM_OF_FWD_DESCS; i++)
        if ((rxQueueId >= p_DpaPort->p_FwdTbl[i].rxBaseQueueId) &&
            (rxQueueId < (p_DpaPort->p_FwdTbl[i].rxBaseQueueId +
                          p_DpaPort->p_FwdTbl[i].numOfQueues)))
            return i;
    return DPAPORT_ILLEGAL_FWD_TBL_INDX;
}


#endif /* __DPAA_H */
