/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          dtsec_mii_acc.c

 @Description   FM dtsec MII register access MAC ...
*//***************************************************************************/

#include "error_ext.h"
#include "std_ext.h"
#include "fm_mac.h"
#include "dtsec.h"

//#define MII_DEBUG

/*****************************************************************************/
t_Error DTSEC_MII_WritePhyReg(t_Handle    h_Dtsec,
                              uint8_t     phyAddr,
                              uint8_t     reg,
                              uint16_t    data)
{
    t_Dtsec             *p_Dtsec = (t_Dtsec *)h_Dtsec;
    t_MiiAccessMemMap   *p_MiiAccess;
    uint32_t            tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_Dtsec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Dtsec->p_MiiMemMap, E_INVALID_HANDLE);

    p_MiiAccess = p_Dtsec->p_MiiMemMap;

#ifdef MII_DEBUG
	XX_Print("%s(h_Dtsec=%x, phyAddr=%d, reg=%x, data=%x) p_MiiAccess=%p (phys=0x%llx)", __FUNCTION__,
			h_Dtsec, phyAddr, reg, data, p_MiiAccess, (uint64_t)XX_VirtToPhys(p_MiiAccess));
#endif
    /* Stop the MII management read cycle */
    WRITE_UINT32(p_MiiAccess->miimcom, 0);
    /* Dummy read to make sure MIIMCOM is written */
    tmpReg = GET_UINT32(p_MiiAccess->miimcom);

    /* Setting up MII Management Address Register */
    tmpReg = (uint32_t)((phyAddr << MIIMADD_PHY_ADDR_SHIFT) | reg);
    WRITE_UINT32(p_MiiAccess->miimadd, tmpReg);

    /* Setting up MII Management Control Register with data */
    WRITE_UINT32(p_MiiAccess->miimcon, (uint32_t)data);
    /* Dummy read to make sure MIIMCON is written */
    tmpReg = GET_UINT32(p_MiiAccess->miimcon);

    /* Wait till MII management write is complete */
    while ((GET_UINT32(p_MiiAccess->miimind)) & MIIMIND_BUSY) ;

    return E_OK;
}

/*****************************************************************************/
t_Error DTSEC_MII_ReadPhyReg(t_Handle h_Dtsec,
                             uint8_t  phyAddr,
                             uint8_t  reg,
                             uint16_t *p_Data)
{
    t_Dtsec             *p_Dtsec = (t_Dtsec *)h_Dtsec;
    t_MiiAccessMemMap   *p_MiiAccess;
    uint32_t            tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_Dtsec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Dtsec->p_MiiMemMap, E_INVALID_HANDLE);

    p_MiiAccess = p_Dtsec->p_MiiMemMap;

#ifdef MII_DEBUG
	XX_Print("%s(h_Dtsec=%x, phyAddr=%d, reg=%x, p_Data=%p) p_MiiAccess=%p (phys=0x%llx)", __FUNCTION__,
			h_Dtsec, phyAddr, reg, p_Data, p_MiiAccess, (uint64_t)XX_VirtToPhys(p_MiiAccess));
#endif

    /* Setting up the MII Management Address Register */
    tmpReg = (uint32_t)((phyAddr << MIIMADD_PHY_ADDR_SHIFT) | reg);
    WRITE_UINT32(p_MiiAccess->miimadd, tmpReg);

    /* Perform an MII management read cycle */
    WRITE_UINT32(p_MiiAccess->miimcom, MIIMCOM_READ_CYCLE);
    /* Dummy read to make sure MIIMCOM is written */
    tmpReg = GET_UINT32(p_MiiAccess->miimcom);

    /* Wait till MII management read is complete */
    while ((GET_UINT32(p_MiiAccess->miimind)) & MIIMIND_BUSY) ;

    /* Read MII management status  */
    *p_Data = (uint16_t)GET_UINT32(p_MiiAccess->miimstat);

    WRITE_UINT32(p_MiiAccess->miimcom, 0);
    /* Dummy read to make sure MIIMCOM is written */
    tmpReg = GET_UINT32(p_MiiAccess->miimcom);

    if (*p_Data == 0xffff)
        RETURN_ERROR(MINOR, E_NO_DEVICE,
                     ("Read wrong data (0xffff): phyAddr 0x%x, reg 0x%x",
                      phyAddr, reg));

#ifdef MII_DEBUG
	XX_Print("%s(...) *p_Data=%x", __FUNCTION__, *p_Data);
#endif

    return E_OK;
}
