/******************************************************************************

 © 1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#if defined (__QNXNTO__)

#include "error_ext.h"
#include "std_ext.h"
#include "fm_mac.h"
#include "tgec.h"
#include "xx_ext.h"




/*These functions are used to access Cache Inhibited regions of memory */
static inline unsigned get32(const volatile unsigned *addr,unsigned offset,uint32_t flags)
{
    unsigned result=0;
    addr+= offset>>2;

    __asm__ __volatile__("lwz%U1%X1 %0,%1; twi 0,%0,0; isync"
                 : "=r" (result) : "m" (*addr));
    return result;
}

static inline void set32(volatile unsigned *addr,unsigned offset, unsigned val,uint32_t flags )
{
    addr+= offset>>2;
    __asm__ __volatile__("stw%U0%X0 %1,%0; sync": "=m" (*addr) : "r" (val));
}


#define CCSR_SET32( base, offset, value, flags) \
        set32((uint32_t*)base,offset,value,flags)

#define CCSR_GET32(base, offset,flags) \
        get32((uint32_t*)base,offset,flags)    



#define MIIMSTAT 0x0
#define MIIMCTRL 0x4
#define MIIMDATA 0x8
#define MIIMADDR 0xc

#define MDIO_STAT_CLKDIV(x)	(((x>>1) & 0xff) << 8)
#define MDIO_STAT_BSY		(1 << 0)
#define MDIO_STAT_RD_ER		(1 << 1)
#define MDIO_CTL_DEV_ADDR(x) 	(x & 0x1f)
#define MDIO_CTL_PORT_ADDR(x)	((x & 0x1f) << 5)
#define MDIO_CTL_PRE_DIS	(1 << 10)
#define MDIO_CTL_SCAN_EN	(1 << 11)
#define MDIO_CTL_POST_INC	(1 << 14)
#define MDIO_CTL_READ		(1 << 15)

#define MDIO_DATA(x)		(x & 0xffff)
#define MDIO_DATA_BSY		(1 << 31)

#define MIIMCOM_READ 1
#define MIIMIND_NOT_VALID 4
#define MIIMCFG_RESET 1<<31

uint16_t    
mii_read (uint32_t *regs, uint8_t port_addr, uint8_t dev_addr, uint8_t reg_addr)
{

	uint32_t mdio_ctl;

	CCSR_SET32(regs, MIIMSTAT, MDIO_STAT_CLKDIV(100) | 0xc,0);
	/* Wait till the bus is free */
	while ((CCSR_GET32(regs,MIIMSTAT,0)) & MDIO_STAT_BSY);

	/* Set the Port and Device Addrs */
	mdio_ctl = MDIO_CTL_PORT_ADDR(port_addr) | MDIO_CTL_DEV_ADDR(dev_addr);
	CCSR_SET32(regs,MIIMCTRL, mdio_ctl,0);

	/* Set the register address */
	CCSR_SET32(regs, MIIMADDR, reg_addr & 0xffff,0);

	/* Wait till the bus is free */
	while ((CCSR_GET32(regs,MIIMSTAT,0)) & MDIO_STAT_BSY);

	/* Initiate the read */
	mdio_ctl |= MDIO_CTL_READ;
	CCSR_SET32(regs,MIIMCTRL, mdio_ctl,0);

	/* Wait till the MDIO write is complete */
	while ((CCSR_GET32(regs,MIIMDATA,0)) & MDIO_DATA_BSY);
	
	return (CCSR_GET32(regs,MIIMDATA,0) & 0xffff);

}


//
// drvr lib callback to write phy register
//
void    
mii_write (uint32_t* regs, uint8_t port_addr, uint8_t dev_addr, uint8_t reg_addr, uint16_t data)
{

	uint32_t mdio_ctl;

	CCSR_SET32(regs, MIIMSTAT, MDIO_STAT_CLKDIV(100) | 0xc,0);
	/* Wait till the bus is free */
	while ((CCSR_GET32(regs,MIIMSTAT,0)) & MDIO_STAT_BSY);

	/* Set the Port and Device Addrs */
	mdio_ctl = MDIO_CTL_PORT_ADDR(port_addr) | MDIO_CTL_DEV_ADDR(dev_addr);
	CCSR_SET32(regs,MIIMCTRL, mdio_ctl,0);

	/* Set the register address */
	CCSR_SET32(regs, MIIMADDR, reg_addr & 0xffff,0);

	/* Wait till the bus is free */
	while ((CCSR_GET32(regs,MIIMSTAT,0)) & MDIO_STAT_BSY);

	CCSR_SET32(regs,MIIMDATA, data,0);

	/* Wait till the MDIO write is complete */
	while ((CCSR_GET32(regs,MIIMDATA,0)) & MDIO_DATA_BSY);
	

}




/*****************************************************************************/
t_Error TGEC_MII_WritePhyReg(t_Handle   h_Tgec,
                             uint8_t    phyAddr,
                             uint8_t    reg,
                             uint16_t   data)
{
    t_Tgec                  *p_Tgec = (t_Tgec *)h_Tgec;
    t_TgecMiiAccessMemMap   *p_MiiAccess;

    SANITY_CHECK_RETURN_ERROR(p_Tgec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Tgec->p_MiiMemMap, E_INVALID_HANDLE);

    p_MiiAccess = p_Tgec->p_MiiMemMap;

	mii_write((uint32_t*) p_MiiAccess, 0x1f & (phyAddr>>5), (0x1f & phyAddr), reg, data);

    return E_OK;
}

/*****************************************************************************/
t_Error TGEC_MII_ReadPhyReg(t_Handle h_Tgec,
                            uint8_t  phyAddr,
                            uint8_t  reg,
                            uint16_t *p_Data)
{
    t_Tgec                  *p_Tgec = (t_Tgec *)h_Tgec;
    t_TgecMiiAccessMemMap   *p_MiiAccess;

    SANITY_CHECK_RETURN_ERROR(p_Tgec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Tgec->p_MiiMemMap, E_INVALID_HANDLE);

    p_MiiAccess = p_Tgec->p_MiiMemMap;
 
	*p_Data= mii_read ((uint32_t*) p_MiiAccess, 0x1f & (phyAddr>>5), (0x1f & phyAddr), reg);
	if(*p_Data==0xFFFF){
		return -1;
	}
	else{
    	return E_OK;
	}
}
#endif
