/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          fm_manip.h

 @Description   FM PCD manip...
*//***************************************************************************/
#ifndef __FM_MANIP_H
#define __FM_MANIP_H

#include "std_ext.h"
#include "error_ext.h"
#include "list_ext.h"

#include "fm_cc.h"


/***********************************************************************/
/*          Header manipulations defines                              */
/***********************************************************************/

#define HMAN_OC_RMV_N_OR_INSRT_INT_FRM_HDR                      0x2e
#define HMAN_OC_INSRT_HDR_BY_TEMPL_N_OR_FRAG_AFTER              0x31
#define HMAN_OC_CAPWAP_FRAGMENTATION                            0x33
#define HMAN_OC_MV_INT_FRAME_HDR_FROM_FRM_TO_BUFFER_PREFFIX     0x2f
#define HMAN_OC_CAPWAP_RMV_DTLS_IF_EXIST                        0x30
#define HMAN_OC_CAPWAP_REASSEMBLY                               0x11 // dummy
#define HMAN_OC_CAPWAP_INDEXED_STATS                            0x32 // dummy


#define HMAN_RMV_HDR                               0x80000000
#define HMAN_INSRT_INT_FRM_HDR                     0x40000000

#define UDP_UDPHECKSUM_FIELD_OFFSET_FROM_UDP        6
#define UDP_UDPCHECKSUM_FIELD_SIZE                  2

#define IP_DSCECN_FIELD_OFFSET_FROM_IP              1
#define IP_TOTALLENGTH_FIELD_OFFSET_FROM_IP         2
#define IP_HDRCHECKSUM_FIELD_OFFSET_FROM_IP         10
#define VLAN_TAG_FIELD_OFFSET_FROM_ETH              12
#define IP_ID_FIELD_OFFSET_FROM_IP                  4

#define FM_PCD_MANIP_CAPWAP_REASM_TABLE_SIZE               80
#define FM_PCD_MANIP_CAPWAP_REASM_TABLE_ALIGN              8
#define FM_PCD_MANIP_CAPWAP_REASM_RFD_SIZE                 32
#define FM_PCD_MANIP_CAPWAP_REASM_AUTO_LEARNING_HASH_ENTRY_SIZE 4
#define FM_PCD_MANIP_CAPWAP_REASM_TIME_OUT_ENTRY_SIZE      8


#define FM_PCD_MANIP_CAPWAP_REASM_TIME_OUT_BETWEEN_FRAMES          0x40000000
#define FM_PCD_MANIP_CAPWAP_REASM_HALT_ON_DUPLICATE_FRAG           0x10000000
#define FM_PCD_MANIP_CAPWAP_REASM_AUTOMATIC_LEARNIN_HASH_8_WAYS    0x08000000
#define FM_PCD_MANIP_CAPWAP_REASM_PR_COPY                          0x00800000

#define FM_PCD_MANIP_CAPWAP_FRAG_COMPR_OPTION_FIELD_EN             0x80000000

#define FM_PCD_MANIP_INDEXED_STATS_ENTRY_SIZE              4
#define FM_PCD_MANIP_INDEXED_STATS_CNIA                    0x20000000
#define FM_PCD_MANIP_INDEXED_STATS_DPD                    0x10000000

/***********************************************************************/
/*          Memory map                                                 */
/***********************************************************************/
#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(push,1)
#endif /* defined(__MWERKS__) && ... */

typedef _Packed struct {
    volatile uint32_t mode;
    volatile uint32_t autoLearnHashTblPtr;
    volatile uint32_t intStatsTblPtr;
    volatile uint32_t reasmFrmDescPoolTblPtr;
    volatile uint32_t reasmFrmDescIndexPoolTblPtr;
    volatile uint32_t timeOutTblPtr;
    volatile uint32_t bufferPoolIdAndRisc1SetIndexes;
    volatile uint32_t risc23SetIndexes;
    volatile uint32_t risc4SetIndexesAndExtendedStatsTblPtr;
    volatile uint32_t extendedStatsTblPtr;
    volatile uint32_t expirationDelay;
    volatile uint32_t totalProcessedFragCounter;
    volatile uint32_t totalUnsuccessfulReasmFramesCounter;
    volatile uint32_t totalDuplicatedFragCounter;
    volatile uint32_t totalMalformdFragCounter;
    volatile uint32_t totalTimeOutCounter;
    volatile uint32_t totalSetBusyCounter;
    volatile uint32_t totalRfdPoolBusyCounter;
    volatile uint32_t totalDiscardedFragsCounter;
    volatile uint32_t totalMoreThan16FramesCounter;
} _PackedType t_CapwapReasmPram;

#define MEM_MAP_END
#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(pop)
#endif /* defined(__MWERKS__) && ... */

/***********************************************************************/
/*  Driver's internal structures                                       */
/***********************************************************************/

typedef struct
{
    t_Handle p_AutoLearnHashTbl;
    t_Handle p_ReassmFrmDescrPoolTbl;
    t_Handle p_ReassmFrmDescrIndxPoolTbl;
    t_Handle p_TimeOutTbl;
    uint8_t  maxNumFramesInProcess;
    uint8_t  numOfTasks;
    uint8_t  poolId;
    uint8_t  prOffset;
    uint16_t  dataOffset;
    uint8_t  poolIndx;
}t_FragParams;

typedef struct{
    bool            muramAllocate;
    t_Handle        h_Ad;
    uint32_t        type;
    bool            rmv;
    bool            insrt;
    uint8_t         *p_Template;
    t_Handle        h_Frag;
    bool            frag;
    uint16_t        sizeForFragmentation;
    uint8_t         owner;
    uint32_t        updateParams;
    uint32_t        shadowUpdateParams;
    t_FragParams    fragParams;
    uint8_t         icOffset;
    uint16_t        ownerTmp;
    bool            cnia;
    t_Handle        p_StatsTbl;
    t_Handle        h_FmPcd;
} t_FmPcdManip;

typedef struct t_FmPcdCcSavedManipParams
{
    union
    {
        struct
        {
            uint16_t    dataOffset;
            uint8_t     poolId;
        }capwapParams;
    };
}t_FmPcdCcSavedManipParams;

#define     e_FM_MANIP_CAPWAP_INDX      0

#endif /* __FM_MANIP_H */
