/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/

/**************************************************************************//**

 @File          gpio.h

 @Description   GPIO internal structures and definitions.
*//***************************************************************************/
#ifndef __GPIO_H
#define __GPIO_H

#include "std_ext.h"
#include "io_port.h"
#include "gpio_ext.h"


/**************************************************************************//**
 @Description   GPIO memory mapped registers
*//***************************************************************************/
#ifdef __MWERKS__
#pragma pack(push,1)
#endif
#define MEM_MAP_START

typedef _Packed struct t_GpioMemMap
{
    _Packed struct
    {
        volatile uint32_t gpdir;    /**< Direction register */
        volatile uint32_t gpodr;    /**< Open drain register */
        volatile uint32_t gpdat;    /**< Data register */
        volatile uint32_t gpier;    /**< Event register */
        volatile uint32_t gpimr;    /**< Mask register */
        volatile uint32_t gpicr;    /**< Interrupt control register */
        volatile uint8_t  RESERVED[0xE8];
    } _PackedType io_regs[GPIO_NUM_OF_PORTS];
} _PackedType t_GpioMemMap;

#define MEM_MAP_END
#ifdef __MWERKS__
#pragma pack(pop)
#endif



/**************************************************************************//**
 @Description   GPIO structure
*//***************************************************************************/
typedef struct t_Gpio
{
    t_IoPortModule  ioPortModule;   /**< Generic I/O Port structure */
    t_GpioMemMap    *p_MemMap;      /**< Pointer to the GPIO registers in memory */
    int             irq[GPIO_NUM_OF_PORTS];
                                     /**< Interrupt source ID of each port -
                                          (-1) if interrupts are not required */
} t_Gpio;



#endif /* __GPIO_H */

