/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          io_port.h

 @Description   Internal definitions for the I/O port module.
*//***************************************************************************/
#ifndef __IO_PORT_H
#define __IO_PORT_H


#include "std_ext.h"
#include "io_port_ext.h"


#define __ERR_MODULE__  MODULE_PIO


#define NUM_OF_PINS_PER_PORT    32

#define PIN_0_MASK              0x80000000

#define IO_PIN_CONFIG_MASK \
    (IO_PIN_DIR_MASK | IO_PIN_SHARED_MASK | IO_PIN_ASSIGN_MASK)

#define NO_INTR                 0xFF


typedef struct t_IoPinIntrInfo
{
    t_Handle            h_App;
    t_IoPinIntrCallback *f_IntrCallback;

} t_IoPinIntrInfo;


typedef struct t_IoPort
{
    uint32_t            validPinsMask;      /**< */
    uint32_t            validIntrMask;      /**< */

    struct
    {
        uint8_t         configuration;
        uint8_t         numOfUsers;
        uint8_t         intrId;
    } pins[NUM_OF_PINS_PER_PORT];

} t_IoPort;


/**************************************************************************//**
 @Description   I/O ports module control structure
 *//***************************************************************************/
typedef struct t_IoPortModule
{
    t_Error         (*f_ConfigurePin)(t_Handle h_IoModule, int ioPin);
    uint8_t         (*f_ReadPinInput)(t_Handle h_IoModule, int ioPin);
    t_Error         (*f_SetPinOutput)(t_Handle h_IoModule, int ioPin, uint8_t output);
    t_Error         (*f_SetPinIntrMode)(t_Handle h_IoModule, int ioPin, e_IoPinIntrMode intrMode);
    uint8_t         numOfPorts;
    t_IoPort        *p_Ports;
    t_IoPinIntrInfo *p_IntrInfo;

} t_IoPortModule;



#endif /* __IO_PORT_H */

