/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          qman_private.h

 @Description   QM private header
*//***************************************************************************/
#ifndef __QMAN_PRIVATE_H
#define __QMAN_PRIVATE_H

#include "fsl_qman.h"


#define __ERR_MODULE__  MODULE_QM

#ifdef DEBUG
/* Optionally compile-in assertion-checking */
#define QM_CHECKING
#endif /* DEBUG */

/* TODO - check the following defines */
#define cpu_relax()

/* TODO: NB, we currently assume that CORE_MemoryBarier() and lwsync() imply compiler barriers
 * and that dcbzl(), dcbfl(), and dcbi() won't fall victim to compiler or
 * execution reordering with respect to other code/instructions that manipulate
 * the same cacheline. */
#if defined (__QNXNTO__)
#include <sys/neutrino.h>
#endif

#if defined (CORE_E500MC) && (_NTO_VERSION > 640)
#define hwsync()    __asm__ __volatile__ ("sync" : : : "memory")
#define lwsync()    __asm__ __volatile__ ("lwsync" : : : "memory")
#define dcbzl(p)    __asm__ __volatile__ ("dcbzl 0,%0" : : "r" (p))
#define dcbf(p)     __asm__ __volatile__ ("dcbf 0,%0" : : "r" (p))
#define dcbt_ro(p)  __asm__ __volatile__ ("dcbt 0,%0" : : "r" (p))
#define dcbt_rw(p)  __asm__ __volatile__ ("dcbtst 0,%0" : : "r" (p))

#else
#define hwsync()    __asm__ __volatile__ ("msync" : : : "memory")
#define lwsync      hwsync
#define dcbzl(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbz 0,%0" : : "r" ((uint32_t)p+0x20)); \
    __asm__ __volatile__ ("dcbz 0,%0" : : "r" (p))
#define dcbf(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbf 0,%0" : : "r" (p))
#define dcbt_ro(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbt 0,%0" : : "r" (p))
#define dcbt_rw(p) \
    __asm__ __volatile__ ("li 0,0"); \
    __asm__ __volatile__ ("dcbtst 0,%0" : : "r" (p))
#endif /* CORE_E500MC */

#define dcbi(p) dcbf(p)

struct qm_addr {
    void  *addr_ce;    /* cache-enabled */
    void  *addr_ci;    /* cache-inhibited */
};

/* EQCR state */
struct qm_eqcr {
    struct qm_eqcr_entry *ring, *cursor;
    uint8_t ci, available, ithresh, vbit;

#ifdef QM_CHECKING
    uint32_t busy;
    e_QmPortalProduceMode       pmode;
    e_QmPortalEqcrConsumeMode   cmode;
#endif /* QM_CHECKING */
};

/* DQRR state */
struct qm_dqrr {
    struct qm_dqrr_entry *ring, *cursor;
    uint8_t pi, ci, fill, ithresh, vbit, flags;

#ifdef QM_CHECKING
    e_QmPortalDequeueMode       dmode;
    e_QmPortalProduceMode       pmode;
    e_QmPortalDqrrConsumeMode   cmode;
#endif /* QM_CHECKING */
};
#define QM_DQRR_FLAG_RE 0x01 /* Stash ring entries */
#define QM_DQRR_FLAG_SE 0x02 /* Stash data */

/* MR state */
struct qm_mr {
    struct qm_mr_entry *ring, *cursor;
    uint8_t pi, ci, fill, ithresh, vbit;

#ifdef QM_CHECKING
    e_QmPortalProduceMode       pmode;
    e_QmPortalMrConsumeMode     cmode;
#endif /* QM_CHECKING */
};

/* MC state */
struct qm_mc {
    struct qm_mc_command *cr;
    struct qm_mc_result *rr;
    uint8_t rridx, vbit;
#ifdef QM_CHECKING
    enum {
        /* Can be _mc_start()ed */
        mc_idle,
        /* Can be _mc_commit()ed or _mc_abort()ed */
        mc_user,
        /* Can only be _mc_retry()ed */
        mc_hw
    } state;
#endif /* QM_CHECKING */
};

/********************/
/* Portal structure */
/********************/

struct qm_portal {
    /* In the non-QM_CHECKING case, everything up to and
     * including 'mc' fits in a cacheline (yay!). The 'config' part is setup-only, so isn't a
     * cause for a concern. In other words, don't rearrange this structure
     * on a whim, there be dragons ... */
    struct qm_addr addr;
    struct qm_eqcr eqcr;
    struct qm_dqrr dqrr;
    struct qm_mr mr;
    struct qm_mc mc;
    struct qm_portal_config config;
    t_Handle bind_lock;
    /* Logical index (not cell-index) */
    int index;
};

#endif /* __QMAN_PRIVATE_H */
