
#define INCLUDE_FROM_ASM 1

#include "e500v2_asm.h"
#include "e500v2_asm_ext.h"

/* .section .text ,4,1,6 - align (4) ,progbits ,flags (6) */
.section .text,"ax",@progbits
.align 2
.extern __start


/*****************************************************************************
    MSR Manipulation
*****************************************************************************/
.global GetMsr
.global MSR_Restore
.global MSR_EnableEE
.global MSR_DisableEE
.global MSR_ClearEE
.global MSR_EnableME
.global MSR_DisableME
.global MSR_EnableCE
.global MSR_DisableCE
.global MSR_EnableFP
.global MSR_DisableFP
.global MSR_ClearEEandCE
.global MSR_EnableDE
.global MSR_DisableDE
.global MSR_EnablePMM
.global MSR_DisablePMM
.global MSR_RestoreEEandCE

/* E500MC MSR */
.global MSR_EnableGS
.global MSR_DisableGS
.global MSR_EnablePR
.global MSR_DisablePR

/*  Get MSR */
/*  uint32_t GetMsr(void) */
GetMsr:
    mfmsr       3
    bclr        ALWAYS, 0

/*  Restore MSR */
/*  void MSR_Restore(uint32_t msr) */
MSR_Restore:
    msync
    mtmsr       3
    isync
    bclr        ALWAYS, 0

/*  Restore MSR EE and CE bits */
/*  void MSR_RestoreEEandCE(uint32_t msr) */
MSR_RestoreEEandCE:
    rlwinm      3, 3, 0, MSR_CE_BN,MSR_EE_BN    /* create a mask from the EE and CE bits */
    rlwinm      3, 3, 0, MSR_EE_BN,MSR_CE_BN
    mfmsr       4                               /* Read MSR */
    or          4, 3, 4                         /* Set CE and EE based on mask */
    mtmsr       4                               /* Write to MSR */
    bclr        ALWAYS, 0

/*  Set EE bit in the MSR */
/*  void MSR_EnableEE(void) */
MSR_EnableEE:
    mfmsr       3
    ori         4, 3, MSR_EE
    mtmsr       4
    bclr        ALWAYS, 0

/*  Clear EE bit in MSR and return old MSR value */
/*  unsigned int MSR_DisableEE(void) */
MSR_DisableEE:
MSR_ClearEE:
    mfmsr       3
    addis       4, 0, not_MSR_EE@h      /* load not_MSR_EE mask to 4 */
    ori         4, 4, not_MSR_EE@l
    and         4, 3, 4
    mtmsr       4
    bclr        ALWAYS, 0

/*  Clear EE and CE bits in MSR and return old MSR value */
/*  unsigned int MSR_ClearEEandCE(void) */
MSR_ClearEEandCE:
    mfmsr       3
    addis       4, 0, not_MSR_EECE@h    /* load not_MSR_EECE mask to 4 */
    ori         4, 4, not_MSR_EECE@l
    and         4, 3, 4
    mtmsr       4
    bclr        ALWAYS, 0

/*  Set ME bit in the MSR */
/*  void MSR_EnableME(void) */
MSR_EnableME:
    mfspr       3, HID0                 /* Read HID0 */
    oris        3, 3, HID0_EMCP_SHIFTED /* Set EMCP */
    isync
    mtspr       HID0,3                  /* Write HID0 */
    isync

    mfmsr       3
    ori         4, 3, MSR_ME
    mtmsr       4
    isync
    msync
    blr

/*  Clear ME bit in the MSR */
/*  void MSR_DisableME(void) */
MSR_DisableME:
    mfmsr       3
    addis       4, 0, not_MSR_ME@h      /* load not_MSR_ME mask to 4 */
    ori         4, 4, not_MSR_ME@l
    and         4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Set CE bit in the MSR */
/*  void MSR_EnableCE(void) */
MSR_EnableCE:
    mfmsr       3
    addis       4, 0, MSR_CE@h          /* load MSR_CE mask to 4 */
    ori         4, 4, MSR_CE@l
    or          4, 3, 4
    mtmsr       4
    blr

/*  Clear CE bit in the MSR */
/*  void MSR_DisableCE(void) */
MSR_DisableCE:
    mfmsr       3
    addis       4, 0, not_MSR_CE@h      /* load not_MSR_CE mask to 4 */
    ori         4, 4, not_MSR_CE@l
    and         4, 3, 4
    mtmsr       4
    blr

/*  Set FP bit in the MSR */
/*  void MSR_EnableFP(void) */
MSR_EnableFP:
    mfmsr       3
    addis       4, 0, MSR_FP@h          /* load MSR_FP mask to 4 */
    ori         4, 4, MSR_FP@l
    or          4, 3, 4
    mtmsr       4
    blr

/*  Clear FP bit in the MSR */
/*  void MSR_DisableFP(void) */
MSR_DisableFP:
    mfmsr       3
    addis       4, 0, not_MSR_FP@h      /* load not_MSR_FP mask to 4 */
    ori         4, 4, not_MSR_FP@l
    and         4, 3, 4
    mtmsr       4
    blr

/*  Set DE bit in the MSR */
/*  void MSR_EnableDE(void) */
MSR_EnableDE:
    mfmsr       3
    addis       4, 0, MSR_DE@h          /* load MSR_DE mask to 4 */
    ori         4, 4, MSR_DE@l
    or          4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Clear DE bit in the MSR */
/*  void MSR_DisableDE(void) */
MSR_DisableDE:
    mfmsr       3
    addis       4, 0, not_MSR_DE@h      /* load not_MSR_CE mask to 4 */
    ori         4, 4, not_MSR_DE@l
    and         4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Set PMM bit in the MSR */
/*  void MSR_EnablePMM(void) */
MSR_EnablePMM:
    isync
    msync
    mfmsr       3
    addis       4, 0, MSR_PMM@h         /* load MSR_PMM mask to 4 */
    ori         4, 4, MSR_PMM@l
    or          4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Clear PMM bit in the MSR */
/*  void MSR_DisablePMM(void) */
MSR_DisablePMM:
    mfmsr       3
    addis       4, 0, not_MSR_PMM@h     /* load not_MSR_PMM mask to 4 */
    ori         4, 4, not_MSR_PMM@l
    and         4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Clear GS bit in the E500MC MSR */
/*  void MSR_DisableGS(void) */
MSR_DisableGS:
    mfmsr       3
    addis       4, 0, not_MSR_GS@h     /* load not_MSR_GS mask to 4 */
    ori         4, 4, not_MSR_GS@l
    and         4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Clear PR bit in the E500MC MSR */
/*  void MSR_DisablePR(void) */
MSR_DisablePR:
    mfmsr       3
    addis       4, 0, not_MSR_PR@h     /* load not_MSR_PR mask to 4 */
    ori         4, 4, not_MSR_PR@l
    and         4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Set GS bit in the E500MC MSR*/
/*  void MSR_EnableGS(void) */
MSR_EnableGS:
    mfmsr       3
    addis       4, 0, MSR_GS@h          /* load MSR_GS mask to 4 */
    ori         4, 4, MSR_GS@l
    or          4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*  Set PR bit in the E500MC MSR*/
/*  void MSR_EnablePR(void) */
MSR_EnablePR:
    mfmsr       3
    addis       4, 0, MSR_PR@h          /* load MSR_PR mask to 4 */
    ori         4, 4, MSR_PR@l
    or          4, 3, 4
    mtmsr       4
    isync
    msync
    blr

/*****************************************************************************
    L1 Cache manipulation
*****************************************************************************/
.global L1DCache_Enable,        L1DCache_Disable
.global L1ICache_Enable,        L1ICache_Disable
.global L1DCache_Invalidate,    L1ICache_Invalidate
.global L1DCache_Flush,         L1ICache_Flush

.global L1DCache_LineLock,      L1DCache_LineUnLock
.global L1ICache_LineLock,      L1ICache_LineUnLock
.global L1DCache_Lock,          L1ICache_Lock
.global L1Cache_BroadCastEnable,L1Cache_BroadCastDisable

.global L1CtlWrite


/* Data cache enable */
L1DCache_Enable:
    msync
    isync
    mfspr       3, L1CSR0
    ori         3, 3, L1CSR0_CE       /* set enable bit at L1CSR0 */
    mtspr       L1CSR0, 3
    isync
    blr

/* Data cache disable */
L1DCache_Disable:
    msync
    isync
    mfspr       3, L1CSR0

    addis       4, 0, not_L1CSR0_CE@h   /* load not_MSR_EECE mask to 4 */
    ori         4, 4, not_L1CSR0_CE@l
    and         3, 3, 4

    mtspr       L1CSR0, 3
    isync
    blr

/* Instruction cache enable */
L1ICache_Enable:
    mfspr       3, L1CSR1
    ori         3, 3, L1CSR1_ICE      /* set enable bit at L1CSR1 */
    mtspr       L1CSR1, 3
    isync
    msync
    blr

/* Instruction cache disable */
L1ICache_Disable:
    mfspr       3, L1CSR1

    addis       4, 0, not_L1CSR1_ICE@h
    ori         4, 4, not_L1CSR1_ICE@l
    and         3, 3, 4

    mtspr       L1CSR1, 3
    isync
    blr

/* Data cache invalidate */
L1DCache_Invalidate:
    msync
    isync
    mfspr       3, L1CSR0
    ori         3, 3, L1CSR0_CFI       /* set invalidate bit at L1CSR0 */
    mtspr       L1CSR0, 3
    isync
    blr

/* Instruction cache invalidate */
L1ICache_Invalidate:
    mfspr       3, L1CSR1
    ori         3, 3, L1CSR1_ICFI      /* set invalidate bit at L1CSR1 */
    mtspr       L1CSR1, 3
    isync
    blr

/* Data Cache Flush */
L1DCache_Flush:
    li          4, (2 * CACHE_NWAYS * CACHE_NLINES)
    mtctr       4
    lis         5, __start@h
L1DCache_Flush_l1:
    lwz         3, 0(5)               /* Load one word from every line */
    addi        5, 5, L1_CACHE_LINE_SIZE
    bdnz        L1DCache_Flush_l1
    blr

/* Instruction Cache Flush */
L1ICache_Flush:
    mfspr       3,L1CSR1                       /* Get L1CSR1 */
    ori         3,3,L1CSR1_ICFI|L1CSR1_ICLFR  /* Set the Flahs bits */
    mtspr       L1CSR1,3                       /* Dp The Flash */
    isync                                       /* Sync Required */
    blr

/* Enable broadcast */
L1Cache_BroadCastEnable:
    mfspr       3,HID1
    ori         3, 3, HID1_ABE            /* set Broadcast enable bit at HID1 */
    msync
    mtspr       HID1,3
    isync
    blr

/* Disable broadcast */
L1Cache_BroadCastDisable:
    mfspr       3, HID1
    addis       4, 0, not_HID1_ABE@h        /* clear Broadcast enable bit at HID1 */
    ori         4, 4, not_HID1_ABE@l
    and         3, 3, 4

/*  andi.       3, 3, not_HID1_ABE */     /* clear Broadcast enable bit at HID1 */
    msync
    mtspr       HID1,3
    isync
    blr


/* Data cache single line lock */
/* 3 must contain the data address as passed by user.
   3 returns 0 on success, or lock failure bits. */
L1DCache_LineLock:
    dcbtls      0, 0, 3       /* lock line in L1 */
    msync
/*    mfL1CSR0    3*/              /* check if cache is full */

    addis       4, 0, L1CSR_ALL_LOCK@h
    ori         4, 4, L1CSR_ALL_LOCK@l
    and         3, 3, 4

    cmplwi      3, 0
    bne         DLockFailed
    blr                         /* return 3 = 0 */
DLockFailed:
    not         4, 3
/*    mfL1CSR0    5*/
    and.        4, 4, 5
/*    mtL1CSR0    4*/              /* clear sticky bits of lock failure */
    blr                         /* return - 3 contains lock failure bits */


/* Data cache unlock a single line */
/* 3 must contain the data address to be locked, as passed by user */
L1DCache_LineUnlock:
    dcblc       0, 0, 3       /* lock line in L1 */
    msync
    blr


/* Instruction cache single line lock */
/* 3 must contain the data address as passed by user.
   3 returns 0 on success, or lock failure bits. */
L1ICache_LineLock:
    icbtls      0, 0, 3       /* lock line in L1 */
    msync
    mfspr       3, L1CSR1      /* check if cache is full */

    addis       4, 0, L1CSR_ALL_LOCK@h
    ori         4, 4, L1CSR_ALL_LOCK@l
    and         3, 3, 4

    cmplwi      3, 0
    bne         ILockFailed
    blr                         /* return 3 = 0 */
ILockFailed:
    not         4, 3
    mfspr       5, L1CSR1
    and.        4, 4, 5
    mtspr       L1CSR1, 4      /* clear sticky bits of lock failure */
    msync
    blr                         /* return - 3 contains lock failure bits */

/* Instruction cache unlock a single line */
/* 3 must contain the instruction address */
L1ICache_LineUnlock:
    icblc       0, 0, 3       /* lock line in L1  */
    msync
    blr

L1CtlWrite:
/* 3 Value, 4 Address */
    mbar        0
    isync
    stw         4, 0(3)       /* Write to Memory */
    lwz         4, 0(3)       /* Load one word back */

    mbar        0
    blr


/*****************************************************************************
    MMU Manipulation
*****************************************************************************/
.global E500_TlbWrite
.global E500_TlbWrite5
.global E500MC_TlbWrite
.global E500_TlbRead
.global E500_TlbRead5
.global E500_Tlb1Invalidate

/* TLB entry configuration */
/* Used by C routine MMU_TlbEntryWrite, mmu.c.
   User must pass 3 (MAS0), 4 (MAS1), 5 (MAS2), and 6 (MAS3) values. */
E500_TlbWrite:
    /* configure masX registers values */
    mtspr       MAS0, 3
    isync
    mtspr       MAS1, 4
    isync
    mtspr       MAS2, 5
    isync
    mtspr       MAS3, 6
    tlbwe
    isync
    blr

/* Used by C routine MMU_TlbEntryWrite, mmu.c.
   User must pass 3 (MAS0), 4 (MAS1), 5 (MAS2), and 6 (MAS3) 7 (MAS7) values. */
E500_TlbWrite5:
    /* configure masX registers values */
    mtspr       MAS0, 3
    isync
    mtspr       MAS1, 4
    isync
    mtspr       MAS2, 5
    isync
    mtspr       MAS3, 6
    isync
    mtspr       MAS7, 7
    tlbwe
    isync
    blr

/* Used by C routine MMU_TlbEntryWrite, mmu.c.*/
E500MC_TlbWrite:
    /* configure masX registers values */
    lwz         4, 0(3)
    mtspr       MAS0, 4
    isync
    lwz         4, 4(3)
    mtspr       MAS1, 4
    isync
    lwz         4, 8(3)
    mtspr       MAS2, 4
    isync
    lwz         4, 12(3)
    mtspr       MAS3, 4
    isync
    lwz         4, 16(3)
    mtspr       MAS4, 4
    isync
    lwz         4, 20(3)
    mtspr       MAS5, 4
    isync
    lwz         4, 24(3)
    mtspr       MAS6, 4
    isync
    lwz         4, 28(3)
    mtspr       MAS7, 4
    isync
    lwz         4, 32(3)
    mtspr       MAS8, 4
    tlbwe
    isync
    blr

/* TLB entry read */

/* Used by C routine MMU_TlbEntryWrite, mmu.c.
   User must pass 3 (by address) 4 (MAS0) and 5 (MAS2).
   Routine uses 3 to return an array of the required TLB entry
   parameters as defined at the masX registers. */
E500_TlbRead:
    /* set MAS0 and MAS2 to define the specified TLB entry */
    mtspr       MAS0, 4
    isync
    mtspr       MAS2, 5
    isync
    tlbre
    /* store in the address speified by 3 the TLB parameters */
    mfspr       4, MAS0
    isync
    stw         4, 0(3)
    mfspr       4, MAS1
    isync
    stw         4, 4(3)
    mfspr       4, MAS2
    isync
    stw         4, 8(3)
    mfspr       4, MAS3
    isync
    stw         4, 12(3)
    isync
    blr

E500_TlbRead5:
    /* set MAS0 and MAS2 to define the specified TLB entry */
    mtspr       MAS0, 4
    isync
    mtspr       MAS2, 5
    isync
    tlbre
    /* store in the address speified by 3 the TLB parameters */
    mfspr       4, MAS0
    isync
    stw         4, 0(3)
    mfspr       4, MAS1
    isync
    stw         4, 4(3)
    mfspr       4, MAS2
    isync
    stw         4, 8(3)
    mfspr       4, MAS3
    isync
    stw         4, 12(3)
    mfspr       4, MAS7
    isync
    stw         4, 16(3)
    isync
    blr

/* TLB1 Invalidate */
/* Invalidates all entries in TLB1 table. */
E500_Tlb1Invalidate:
    li          3, 0x0c
    tlbivax     0,3
    isync
    blr


/*****************************************************************************
    Miscellaneous
*****************************************************************************/
.global GetDec
.global GetTcr
.global GetHid0
.global GetCpuId

/* Get DEC */
/* void GetDec(void) */
GetDec:
    mfdec       3
    bclr        ALWAYS, 0

/* Get TCR */
/* void GetTcr(void) */
GetTcr:
    mfspr       3, TCR
    bclr        ALWAYS, 0

/* Get HID0 */
/* void GetHid0(void) */
GetHid0:
    mfspr       3, HID0
    bclr        ALWAYS, 0

/* Get CPU Id */
/* uint32_t GetCpuId(void) */
GetCpuId:
    mfpir       3
    blr


/* Enable broadcast */
.global    E500_AddressBusStreamingEnable
E500_AddressBusStreamingEnable:
    mfspr       3,HID1
    ori         3, 3, HID1_ASTME
    msync
    mtspr       HID1,3
    isync
    blr

/* Enable address bus streaming */
.global    E500_AddressBroadcastEnable
E500_AddressBroadcastEnable:
    mfspr       3,HID1
    ori         3, 3, HID1_ABE
    msync
    mtspr       HID1,3
    isync
    blr

/* Disable broadcast */
.global    E500_AddressBroadcastDisable
E500_AddressBroadcastDisable:
    mfspr       3, HID1
    addis       4, 0, not_HID1_ABE@h
    ori         4, 4, not_HID1_ABE@l
    and         3, 3, 4
    msync
    mtspr       HID1,3
    isync
    blr

/* Check if tasklets are supported by the e500 interrupt handler */
.global E500_IsTaskletSupported
E500_IsTaskletSupported:
#ifndef DISABLE_TASKLETS
    li          3, 1
#else  /* DISABLE_TASKLETS */
    li          3, 0
#endif /* DISABLE_TASKLETS */
    bclr        ALWAYS, 0

.global E500_TestAndSet
E500_TestAndSet:
	li      4, 1            /* prepare non-zero value */
tas_loop:
    lwarx   5, 0, 3         /* load and reserve */
	cmpwi   5, 0            /* check loaded value */
	bne-    tas_end         /* not equal to 0 - already set */
	stwcx.  4, 0, 3         /* try to store non-zero */
	bne-    tas_loop        /* lost reservation */
	li      3, 1            /* return new (non-zero) value */
	blr
tas_end:
	li      3, 0            /* return previous (zero) value */
	blr



/*****************************************************************************
    SPRs Manipulation

    The SPRs functions where build by a macro definitions based on the SPRs
    table in the user manual.
*****************************************************************************/

/* Book E Special-Purpose Registers (by SPR Abbreviation) */

.global    E500_GetSPR_CSRR0
E500_GetSPR_CSRR0:
    mfspr   3,CSRR0         /* 58 */    /* Critical save/restore register 0 */
    blr
.global    E500_SetSPR_CSRR0
E500_SetSPR_CSRR0:
    mtspr   CSRR0,3         /* 58 */    /* Critical save/restore register 0 */
    blr

.global    E500_GetSPR_CSRR1
E500_GetSPR_CSRR1:
    mfspr   3,CSRR1         /* 59 */    /* Critical save/restore register 1 */
    blr
.global    E500_SetSPR_CSRR1
E500_SetSPR_CSRR1:
    mtspr   CSRR1,3         /* 59 */    /* Critical save/restore register 1 */
    blr

.global    E500_GetSPR_CTR
E500_GetSPR_CTR:
    mfspr   3,CTR           /* 9 */     /* Count register */
    blr
.global    E500_SetSPR_CTR
E500_SetSPR_CTR:
    mtspr   CTR,3           /* 9 */     /* Count register */
    blr

.global    E500_GetSPR_DAC1
E500_GetSPR_DAC1:
    mfspr   3,DAC1          /* 316 */   /* Data address compare 1 */
    blr
.global    E500_SetSPR_DAC1
E500_SetSPR_DAC1:
    mtspr   DAC1,3          /* 316 */   /* Data address compare 1 */
    blr

.global    E500_GetSPR_DAC2
E500_GetSPR_DAC2:
    mfspr   3,DAC2          /* 317 */   /* Data address compare 2 */
    blr
.global    E500_SetSPR_DAC2
E500_SetSPR_DAC2:
    mtspr   DAC2,3          /* 317 */   /* Data address compare 2 */
    blr

.global    E500_GetSPR_DBCR0
E500_GetSPR_DBCR0:
    mfspr   3,DBCR0         /* 308 */   /* Debug control register 0 1 */
    blr
.global    E500_SetSPR_DBCR0
E500_SetSPR_DBCR0:
    mtspr   DBCR0,3         /* 308 */   /* Debug control register 0 1 */
    blr

.global    E500_GetSPR_DBCR1
E500_GetSPR_DBCR1:
    mfspr   3,DBCR1         /* 309 */   /* Debug control register 1 1 */
    blr
.global    E500_SetSPR_DBCR1
E500_SetSPR_DBCR1:
    mtspr   DBCR1,3         /* 309 */   /* Debug control register 1 1 */
    blr

.global    E500_GetSPR_DBCR2
E500_GetSPR_DBCR2:
    mfspr   3,DBCR2         /* 310 */   /* Debug control register 2 1 */
    blr
.global    E500_SetSPR_DBCR2
E500_SetSPR_DBCR2:
    mtspr   DBCR2,3         /* 310 */   /* Debug control register 2 1 */
    blr

.global    E500_GetSPR_DBSR
E500_GetSPR_DBSR:
    mfspr   3,DBSR          /* 304 */   /* Debug status register */
    blr
.global    E500_SetSPR_DBSR
E500_SetSPR_DBSR:
    mtspr   DBSR,3          /* 304 */   /* Debug status register */
    blr

.global    E500_GetSPR_DEAR
E500_GetSPR_DEAR:
    mfspr   3,DEAR          /* 61 */    /* Data exception address register */
    blr
.global    E500_SetSPR_DEAR
E500_SetSPR_DEAR:
    mtspr   DEAR,3          /* 61 */    /* Data exception address register */
    blr

.global    E500_GetSPR_DEC
E500_GetSPR_DEC:
    mfspr   3,DEC           /* 22 */    /* Decrementer */
    blr
.global    E500_SetSPR_DEC
E500_SetSPR_DEC:
    mtspr   DEC,3           /* 22 */    /* Decrementer */
    blr

.global    E500_GetSPR_DECAR
E500_GetSPR_DECAR:
    mfspr   3,DECAR         /* 54 */    /* Decrementer auto-reload */
    blr
.global    E500_SetSPR_DECAR
E500_SetSPR_DECAR:
    mtspr   DECAR,3         /* 54 */    /* Decrementer auto-reload */
    blr

.global    E500_GetSPR_ESR
E500_GetSPR_ESR:
    mfspr   3,ESR           /* 62 */    /* Exception syndrome register */
    blr
.global    E500_SetSPR_ESR
E500_SetSPR_ESR:
    mtspr   ESR,3           /* 62 */    /* Exception syndrome register */
    blr

.global    E500_GetSPR_IAC1
E500_GetSPR_IAC1:
    mfspr   3,IAC1          /* 312 */   /* Instruction address compare 1 */
    blr
.global    E500_SetSPR_IAC1
E500_SetSPR_IAC1:
    mtspr   IAC1,3          /* 312 */   /* Instruction address compare 1 */
    blr

.global    E500_GetSPR_IAC2
E500_GetSPR_IAC2:
    mfspr   3,IAC2          /* 313 */   /* Instruction address compare 2 */
    blr
.global    E500_SetSPR_IAC2
E500_SetSPR_IAC2:
    mtspr   IAC2,3          /* 313 */   /* Instruction address compare 2 */
    blr

.global    E500_GetSPR_IVOR0
E500_GetSPR_IVOR0:
    mfspr   3,IVOR0         /* 400 */   /* Critical input */
    blr
.global    E500_SetSPR_IVOR0
E500_SetSPR_IVOR0:
    mtspr   IVOR0,3         /* 400 */   /* Critical input */
    blr

.global    E500_GetSPR_IVOR1
E500_GetSPR_IVOR1:
    mfspr   3,IVOR1         /* 401 */   /* Machine check interrupt offset */
    blr
.global    E500_SetSPR_IVOR1
E500_SetSPR_IVOR1:
    mtspr   IVOR1,3         /* 401 */   /* Machine check interrupt offset */
    blr

.global    E500_GetSPR_IVOR2
E500_GetSPR_IVOR2:
    mfspr   3,IVOR2         /* 402 */   /* Data storage interrupt offset */
    blr
.global    E500_SetSPR_IVOR2
E500_SetSPR_IVOR2:
    mtspr   IVOR2,3         /* 402 */   /* Data storage interrupt offset */
    blr

.global    E500_GetSPR_IVOR3
E500_GetSPR_IVOR3:
    mfspr   3,IVOR3         /* 403 */   /* Instruction storage interrupt offset */
    blr
.global    E500_SetSPR_IVOR3
E500_SetSPR_IVOR3:
    mtspr   IVOR3,3         /* 403 */   /* Instruction storage interrupt offset */
    blr

.global    E500_GetSPR_IVOR4
E500_GetSPR_IVOR4:
    mfspr   3,IVOR4         /* 404 */   /* External input interrupt offset */
    blr
.global    E500_SetSPR_IVOR4
E500_SetSPR_IVOR4:
    mtspr   IVOR4,3         /* 404 */   /* External input interrupt offset */
    blr

.global    E500_GetSPR_IVOR5
E500_GetSPR_IVOR5:
    mfspr   3,IVOR5         /* 405 */   /* Alignment interrupt offset */
    blr
.global    E500_SetSPR_IVOR5
E500_SetSPR_IVOR5:
    mtspr   IVOR5,3         /* 405 */   /* Alignment interrupt offset */
    blr

.global    E500_GetSPR_IVOR6
E500_GetSPR_IVOR6:
    mfspr   3,IVOR6         /* 406 */   /* Program interrupt offset */
    blr
.global    E500_SetSPR_IVOR6
E500_SetSPR_IVOR6:
    mtspr   IVOR6,3         /* 406 */   /* Program interrupt offset */
    blr

.global    E500_GetSPR_IVOR8
E500_GetSPR_IVOR8:
    mfspr   3,IVOR8         /* 408 */   /* System call interrupt offset */
    blr
.global    E500_SetSPR_IVOR8
E500_SetSPR_IVOR8:
    mtspr   IVOR8,3         /* 408 */   /* System call interrupt offset */
    blr

.global    E500_GetSPR_IVOR10
E500_GetSPR_IVOR10:
    mfspr   3,IVOR10        /* 410 */   /* Decrementer interrupt offset */
    blr
.global    E500_SetSPR_IVOR10
E500_SetSPR_IVOR10:
    mtspr   IVOR10,3        /* 410 */   /* Decrementer interrupt offset */
    blr

.global    E500_GetSPR_IVOR11
E500_GetSPR_IVOR11:
    mfspr   3,IVOR11        /* 411 */   /* Fixed-interval timer interrupt offset */
    blr
.global    E500_SetSPR_IVOR11
E500_SetSPR_IVOR11:
    mtspr   IVOR11,3        /* 411 */   /* Fixed-interval timer interrupt offset */
    blr

.global    E500_GetSPR_IVOR12
E500_GetSPR_IVOR12:
    mfspr   3,IVOR12        /* 412 */   /* Watchdog timer interrupt offset */
    blr
.global    E500_SetSPR_IVOR12
E500_SetSPR_IVOR12:
    mtspr   IVOR12,3        /* 412 */   /* Watchdog timer interrupt offset */
    blr

.global    E500_GetSPR_IVOR13
E500_GetSPR_IVOR13:
    mfspr   3,IVOR13        /* 413 */   /* Data TLB error interrupt offset */
    blr
.global    E500_SetSPR_IVOR13
E500_SetSPR_IVOR13:
    mtspr   IVOR13,3        /* 413 */   /* Data TLB error interrupt offset */
    blr

.global    E500_GetSPR_IVOR14
E500_GetSPR_IVOR14:
    mfspr   3,IVOR14        /* 414 */   /* Instruction TLB error interrupt offset */
    blr
.global    E500_SetSPR_IVOR14
E500_SetSPR_IVOR14:
    mtspr   IVOR14,3        /* 414 */   /* Instruction TLB error interrupt offset */
    blr

.global    E500_GetSPR_IVOR15
E500_GetSPR_IVOR15:
    mfspr   3,IVOR15        /* 415 */   /* Debug interrupt offset */
    blr
.global    E500_SetSPR_IVOR15
E500_SetSPR_IVOR15:
    mtspr   IVOR15,3        /* 415 */   /* Debug interrupt offset */
    blr

.global    E500_GetSPR_IVPR
E500_GetSPR_IVPR:
    mfspr   3,IVPR          /* 63 */    /* Interrupt vector */
    blr
.global    E500_SetSPR_IVPR
E500_SetSPR_IVPR:
    mtspr   IVPR,3          /* 63 */    /* Interrupt vector */
    blr

.global    E500_GetSPR_LR
E500_GetSPR_LR:
    mfspr   3,LR            /* 8 */     /* Link register */
    blr
.global    E500_SetSPR_LR
E500_SetSPR_LR:
    mtspr   LR,3            /* 8 */     /* Link register */
    blr

.global    E500_GetSPR_PID
E500_GetSPR_PID:
    mfspr   3,PID           /* 48 */    /* Process ID register 3 */
    blr
.global    E500_SetSPR_PID
E500_SetSPR_PID:
    mtspr   PID,3           /* 48 */    /* Process ID register 3 */
    blr

.global    E500_GetSPR_PIR
E500_GetSPR_PIR:
    mfspr   3,PIR           /* 286 */   /* Processor ID register */
    blr
.global    E500_SetSPR_PIR
E500_SetSPR_PIR:
    mtspr   PIR,3           /* 286 */   /* Processor ID register */
    blr

.global    E500_GetSPR_PVR
E500_GetSPR_PVR:
    mfspr   3,PVR           /* 287 */   /* Processor version register */
    blr
.global    E500_SetSPR_PVR
E500_SetSPR_PVR:
    mtspr   PVR,3           /* 287 */   /* Processor version register */
    blr

.global    E500_GetSPR_SPRG0
E500_GetSPR_SPRG0:
    mfspr   3,SPRG0         /* 272 */   /* SPR general 0 */
    blr
.global    E500_SetSPR_SPRG0
E500_SetSPR_SPRG0:
    mtspr   SPRG0,3         /* 272 */   /* SPR general 0 */
    blr

.global    E500_GetSPR_SPRG1
E500_GetSPR_SPRG1:
    mfspr   3,SPRG1         /* 273 */   /* SPR general 1 */
    blr
.global    E500_SetSPR_SPRG1
E500_SetSPR_SPRG1:
    mtspr   SPRG1,3         /* 273 */   /* SPR general 1 */
    blr

.global    E500_GetSPR_SPRG2
E500_GetSPR_SPRG2:
    mfspr   3,SPRG2         /* 274 */   /* SPR general 2 */
    blr
.global    E500_SetSPR_SPRG2
E500_SetSPR_SPRG2:
    mtspr   SPRG2,3         /* 274 */   /* SPR general 2 */
    blr

.global    E500_GetSPR_SPRG3
E500_GetSPR_SPRG3:
    mfspr   3,SPRG3         /* 275 */   /* SPR general 3 */
    blr
.global    E500_SetSPR_SPRG3
E500_SetSPR_SPRG3:
    mtspr   SPRG3,3         /* 275 */   /* SPR general 3 */
    blr

.global    E500_GetSPR_SPRG4
E500_GetSPR_SPRG4:
    mfspr   3,SPRG4         /* 276 */   /* SPR general 4 */
    blr
.global    E500_SetSPR_SPRG4
E500_SetSPR_SPRG4:
    mtspr   SPRG4,3         /* 276 */   /* SPR general 4 */
    blr

.global    E500_GetSPR_SPRG5
E500_GetSPR_SPRG5:
    mfspr   3,SPRG5         /* 277 */   /* SPR general 5 */
    blr
.global    E500_SetSPR_SPRG5
E500_SetSPR_SPRG5:
    mtspr   SPRG5,3         /* 277 */   /* SPR general 5 */
    blr

.global    E500_GetSPR_SPRG6
E500_GetSPR_SPRG6:
    mfspr   3,SPRG6         /* 278 */   /* SPR general 6 */
    blr
.global    E500_SetSPR_SPRG6
E500_SetSPR_SPRG6:
    mtspr   SPRG6,3         /* 278 */   /* SPR general 6 */
    blr

.global    E500_GetSPR_SPRG7
E500_GetSPR_SPRG7:
    mfspr   3,SPRG7         /* 279 */   /* SPR general 7 */
    blr
.global    E500_SetSPR_SPRG7
E500_SetSPR_SPRG7:
    mtspr   SPRG7,3         /* 279 */   /* SPR general 7 */
    blr

.global    E500_GetSPR_SRR0
E500_GetSPR_SRR0:
    mfspr   3,SRR0          /* 26 */    /* Save/restore register 0 */
    blr
.global    E500_SetSPR_SRR0
E500_SetSPR_SRR0:
    mtspr   SRR0,3          /* 26 */    /* Save/restore register 0 */
    blr

.global    E500_GetSPR_SRR1
E500_GetSPR_SRR1:
    mfspr   3,SRR1          /* 27 */    /* Save/restore register 1 */
    blr
.global    E500_SetSPR_SRR1
E500_SetSPR_SRR1:
    mtspr   SRR1,3          /* 27 */    /* Save/restore register 1 */
    blr

.global    E500_GetSPR_TBL
E500_GetSPR_TBL:
    mfspr   3,TBL           /* 284 */   /* Time base lower */
    blr
.global    E500_SetSPR_TBL
E500_SetSPR_TBL:
    mtspr   TBL,3           /* 284 */   /* Time base lower */
    blr

.global    E500_GetSPR_TBU
E500_GetSPR_TBU:
    mfspr   3,TBU           /* 285 */   /* Time base upper */
    blr
.global    E500_SetSPR_TBU
E500_SetSPR_TBU:
    mtspr   TBU,3           /* 285 */   /* Time base upper */
    blr

.global    E500_GetSPR_TCR
E500_GetSPR_TCR:
    mfspr   3,TCR           /* 340 */   /* Timer control register */
    blr
.global    E500_SetSPR_TCR
E500_SetSPR_TCR:
    mtspr   TCR,3           /* 340 */   /* Timer control register */
    blr

.global    E500_GetSPR_TSR
E500_GetSPR_TSR:
    mfspr   3,TSR           /* 336 */   /* Timer status register */
    blr
.global    E500_SetSPR_TSR
E500_SetSPR_TSR:
    mtspr   TSR,3           /* 336 */   /* Timer status register */
    blr

.global    E500_GetSPR_USPRG0
E500_GetSPR_USPRG0:
    mfspr   3,USPRG0        /* 256 */   /* User SPR general 06 */
    blr
.global    E500_SetSPR_USPRG0
E500_SetSPR_USPRG0:
    mtspr   USPRG0,3        /* 256 */   /* User SPR general 06 */
    blr

.global    E500_GetSPR_XER
E500_GetSPR_XER:
    mfspr   3,XER           /* 1 */     /* Integer exception register */
    blr
.global    E500_SetSPR_XER
E500_SetSPR_XER:
    mtspr   XER,3           /* 1 */     /* Integer exception register */
    blr


/* Implementation-Specific SPRs (by SPR Abbreviation) */

.global    E500_GetSPR_BBEAR
E500_GetSPR_BBEAR:
    mfspr   3,BBEAR         /* 513 */   /* Branch buffer entry address register 1 */
    blr
.global    E500_SetSPR_BBEAR
E500_SetSPR_BBEAR:
    mtspr   BBEAR,3         /* 513 */   /* Branch buffer entry address register 1 */
    blr

.global    E500_GetSPR_BBTAR
E500_GetSPR_BBTAR:
    mfspr   3,BBTAR         /* 514 */   /* Branch buffer target address register 1 */
    blr
.global    E500_SetSPR_BBTAR
E500_SetSPR_BBTAR:
    mtspr   BBTAR,3         /* 514 */   /* Branch buffer target address register 1 */
    blr

.global    E500_GetSPR_BUCSR
E500_GetSPR_BUCSR:
    mfspr   3,BUCSR         /* 1013 */  /* Branch unit control and status register 1 */
    blr
.global    E500_SetSPR_BUCSR
E500_SetSPR_BUCSR:
    mtspr   BUCSR,3         /* 1013 */  /* Branch unit control and status register 1 */
    blr

.global    E500_GetSPR_HID0
E500_GetSPR_HID0:
    mfspr   3,HID0          /* 1008 */  /* Hardware implementation dependent reg 0 1 */
    blr
.global    E500_SetSPR_HID0
E500_SetSPR_HID0:
    isync
    mtspr   HID0,3          /* 1008 */  /* Hardware implementation dependent reg 0 1 */
    isync
    blr

.global    E500_GetSPR_HID1
E500_GetSPR_HID1:
    mfspr   3,HID1          /* 1009 */  /* Hardware implementation dependent reg 1 1 */
    blr
.global    E500_SetSPR_HID1
E500_SetSPR_HID1:
    msync
    mtspr   HID1,3          /* 1009 */  /* Hardware implementation dependent reg 1 1 */
    isync
    blr

.global    E500_GetSPR_IVOR32
E500_GetSPR_IVOR32:
    mfspr   3,IVOR32        /* 528 */   /* SPE APU unavailable interrupt offset */
    blr
.global    E500_SetSPR_IVOR32
E500_SetSPR_IVOR32:
    mtspr   IVOR32,3        /* 528 */   /* SPE APU unavailable interrupt offset */
    blr

.global    E500_GetSPR_IVOR33
E500_GetSPR_IVOR33:
    mfspr   3,IVOR33        /* 529 */   /* Floating-point data exception interrupt offset */
    blr
.global    E500_SetSPR_IVOR33
E500_SetSPR_IVOR33:
    mtspr   IVOR33,3        /* 529 */   /* Floating-point data exception interrupt offset */
    blr

.global    E500_GetSPR_IVOR34
E500_GetSPR_IVOR34:
    mfspr   3,IVOR34        /* 530 */   /* Floating-point round exception interrupt offset */
    blr
.global    E500_SetSPR_IVOR34
E500_SetSPR_IVOR34:
    mtspr   IVOR34,3        /* 530 */   /* Floating-point round exception interrupt offset */
    blr

.global    E500_GetSPR_IVOR35
E500_GetSPR_IVOR35:
    mfspr   3,IVOR35        /* 531 */   /* Performance monitor */
    blr
.global    E500_SetSPR_IVOR35
E500_SetSPR_IVOR35:
    mtspr   IVOR35,3        /* 531 */   /* Performance monitor */
    blr

.global    E500_GetSPR_L1CFG0
E500_GetSPR_L1CFG0:
    mfspr   3,L1CFG0        /* 515 */   /* L1 cache configuration register 0 */
    blr
.global    E500_SetSPR_L1CFG0
E500_SetSPR_L1CFG0:
    mtspr   L1CFG0,3        /* 515 */   /* L1 cache configuration register 0 */
    blr

.global    E500_GetSPR_L1CFG1
E500_GetSPR_L1CFG1:
    mfspr   3,L1CFG1        /* 516 */   /* L1 cache configuration register 1 */
    blr
.global    E500_SetSPR_L1CFG1
E500_SetSPR_L1CFG1:
    mtspr   L1CFG1,3        /* 516 */   /* L1 cache configuration register 1 */
    blr

.global    E500_GetSPR_L1CSR0
E500_GetSPR_L1CSR0:
    mfspr   3,L1CSR0        /* 1010 */  /* L1 cache control and status register 0 1 */
    blr
.global    E500_SetSPR_L1CSR0
E500_SetSPR_L1CSR0:
    mtspr   L1CSR0,3        /* 1010 */  /* L1 cache control and status register 0 1 */
    blr

.global    E500_GetSPR_L1CSR1
E500_GetSPR_L1CSR1:
    mfspr   3,L1CSR1        /* 1011 */  /* L1 cache control and status register 1 1 */
    blr
.global    E500_SetSPR_L1CSR1
E500_SetSPR_L1CSR1:
    mtspr   L1CSR1,3        /* 1011 */  /* L1 cache control and status register 1 1 */
    blr

.global    E500_GetSPR_L1CSR2
E500_GetSPR_L1CSR2:
    mfspr   3,L1CSR2        /* 606 */  /* L1 cache control and status register 2 1 */
    blr
.global    E500_SetSPR_L1CSR2
E500_SetSPR_L1CSR2:
    mtspr   L1CSR2,3        /* 606 */  /* L1 cache control and status register 2 1 */
    blr

.global    E500MC_GetSPR_L2CFG0
E500MC_GetSPR_L2CFG0:
    mfspr   3,L2CFG0        /* 519 */   /* L2 cache configuration register 0 */
    blr

.global    E500MC_SetSPR_L2CFG0
E500MC_SetSPR_L2CFG0:
    mtspr   L2CFG0,3        /* 519 */   /* L2 cache configuration register 0 */
    blr

.global    E500MC_GetSPR_L2CSR0
E500MC_GetSPR_L2CSR0:
    mfspr   3,L2CSR0        /* 1017 */  /* L2 cache control and status register 0 1 */
    blr

.global    E500MC_SetSPR_L2CSR0
E500MC_SetSPR_L2CSR0:
    mtspr   L2CSR0,3        /* 1017 */  /* L2 cache control and status register 0 1 */
    blr

.global    E500MC_GetSPR_L2CSR1
E500MC_GetSPR_L2CSR1:
    mfspr   3,L2CSR1        /* 1018 */  /* L2 cache control and status register 1 1 */
    blr

.global    E500MC_SetSPR_L2CSR1
E500MC_SetSPR_L2CSR1:
    mtspr   L2CSR1,3        /* 1018 */  /* L2 cache control and status register 1 1 */
    blr


.global    E500_GetSPR_MAS0
E500_GetSPR_MAS0:
    mfspr   3,MAS0          /* 624 */   /* MMU assist register 0 1 */
    blr
.global    E500_SetSPR_MAS0
E500_SetSPR_MAS0:
    mtspr   MAS0,3          /* 624 */   /* MMU assist register 0 1 */
    blr

.global    E500_GetSPR_MAS1
E500_GetSPR_MAS1:
    mfspr   3,MAS1          /* 625 */   /* MMU assist register 1 1 */
    blr
.global    E500_SetSPR_MAS1
E500_SetSPR_MAS1:
    mtspr   MAS1,3          /* 625 */   /* MMU assist register 1 1 */
    blr

.global    E500_GetSPR_MAS2
E500_GetSPR_MAS2:
    mfspr   3,MAS2          /* 626 */   /* MMU assist register 2 1 */
    blr
.global    E500_SetSPR_MAS2
E500_SetSPR_MAS2:
    mtspr   MAS2,3          /* 626 */   /* MMU assist register 2 1 */
    blr

.global    E500_GetSPR_MAS3
E500_GetSPR_MAS3:
    mfspr   3,MAS3          /* 627 */   /* MMU assist register 3 1 */
    blr
.global    E500_SetSPR_MAS3
E500_SetSPR_MAS3:
    mtspr   MAS3,3          /* 627 */   /* MMU assist register 3 1 */
    blr

.global    E500_GetSPR_MAS4
E500_GetSPR_MAS4:
    mfspr   3,MAS4          /* 628 */   /* MMU assist register 4 1 */
    blr
.global    E500_SetSPR_MAS4
E500_SetSPR_MAS4:
    mtspr   MAS4,3          /* 628 */   /* MMU assist register 4 1 */
    blr

.global    E500_GetSPR_MAS6
E500_GetSPR_MAS6:
    mfspr   3,MAS6          /* 630 */   /* MMU assist register 6 1 */
    blr
.global    E500_SetSPR_MAS6
E500_SetSPR_MAS6:
    mtspr   MAS6,3          /* 630 */   /* MMU assist register 6 1 */
    blr

.global    E500_GetSPR_MAS7
E500_GetSPR_MAS7:
    mfspr   3,MAS7          /* 944 */   /* MMU assist register 7 1 */
    blr
.global    E500_SetSPR_MAS7
E500_SetSPR_MAS7:
    mtspr   MAS7,3          /* 944 */   /* MMU assist register 7 1 */
    blr

.global    E500_GetSPR_MCAR
E500_GetSPR_MCAR:
    mfspr   3,MCAR          /* 573 */   /* Machine check address register */
    blr
.global    E500_SetSPR_MCAR
E500_SetSPR_MCAR:
    mtspr   MCAR,3          /* 573 */   /* Machine check address register */
    blr

.global    E500_GetSPR_MCSR
E500_GetSPR_MCSR:
    mfspr   3,MCSR          /* 572 */   /* Machine check syndrome register */
    blr
.global    E500_SetSPR_MCSR
E500_SetSPR_MCSR:
    mtspr   MCSR,3          /* 572 */   /* Machine check syndrome register */
    blr

.global    E500_GetSPR_MCSRR0
E500_GetSPR_MCSRR0:
    mfspr   3,MCSRR0        /* 570 */   /* Machine check save/restore register 0 */
    blr
.global    E500_SetSPR_MCSRR0
E500_SetSPR_MCSRR0:
    mtspr   MCSRR0,3        /* 570 */   /* Machine check save/restore register 0 */
    blr

.global    E500_GetSPR_MCSRR1
E500_GetSPR_MCSRR1:
    mfspr   3,MCSRR1        /* 571 */   /* Machine check save/restore register 1 */
    blr
.global    E500_SetSPR_MCSRR1
E500_SetSPR_MCSRR1:
    mtspr   MCSRR1,3        /* 571 */   /* Machine check save/restore register 1 */
    blr

.global    E500_GetSPR_MMUCFG
E500_GetSPR_MMUCFG:
    mfspr   3,MMUCFG        /* 1015 */  /* MMU configuration register */
    blr
.global    E500_SetSPR_MMUCFG
E500_SetSPR_MMUCFG:
    mtspr   MMUCFG,3        /* 1015 */  /* MMU configuration register */
    blr

.global    E500_GetSPR_MMUCSR0
E500_GetSPR_MMUCSR0:
    mfspr   3,MMUCSR0       /* 1012 */  /* MMU control and status register 0 1 */
    blr
.global    E500_SetSPR_MMUCSR0
E500_SetSPR_MMUCSR0:
    mtspr   MMUCSR0,3       /* 1012 */  /* MMU control and status register 0 1 */
    blr



/* #define PID 48 */        /* Book E refers to this as instead of PID0 */
.global    E500_GetSPR_PID0
E500_GetSPR_PID0:
    mfspr   3,PID0          /* 48 */    /* Process ID register 0. */
    blr
.global    E500_SetSPR_PID0
E500_SetSPR_PID0:
    mtspr   PID0,3          /* 48 */    /* Process ID register 0. */
    blr

.global    E500_GetSPR_PID1
E500_GetSPR_PID1:
    mfspr   3,PID1          /* 633 */   /* Process ID register 1 1 */
    blr
.global    E500_SetSPR_PID1
E500_SetSPR_PID1:
    mtspr   PID1,3          /* 633 */   /* Process ID register 1 1 */
    blr

.global    E500_GetSPR_PID2
E500_GetSPR_PID2:
    mfspr   3,PID2          /* 634 */   /* Process ID register 2 1 */
    blr
.global    E500_SetSPR_PID2
E500_SetSPR_PID2:
    mtspr   PID2,3          /* 634 */   /* Process ID register 2 1 */
    blr


.global    E500_GetSPR_TLB0CFG
E500_GetSPR_TLB0CFG:
    mfspr   3,TLB0CFG       /* 688 */   /* TLB0CFG */
    blr

.global    E500_GetSPR_TLB1CFG
E500_GetSPR_TLB1CFG:
    mfspr   3,TLB1CFG       /* 689 */   /* TLB1CFG */
    blr


.global    E500_GetSPR_SPEFSCR
E500_GetSPR_SPEFSCR:
    mfspr   3,SPEFSCR       /* 512 */   /* Signal processing and embedded floating-point status and control register 1 */
    blr
.global    E500_SetSPR_SPEFSCR
E500_SetSPR_SPEFSCR:
    mtspr   SPEFSCR,3       /* 512 */   /* Signal processing and embedded floating-point status and control register 1 */
    blr

.global    E500_GetSPR_SVR
E500_GetSPR_SVR:
    mfspr   3,SVR           /* 1023 */  /* System version register */
    blr
.global    E500_SetSPR_SVR
E500_SetSPR_SVR:
    mtspr   SVR,3           /* 1023 */  /* System version register */
    blr


/*****************************************************************************
    Performance Monitoring
*****************************************************************************/

.global E500_GetPMC0
E500_GetPMC0:
    mfpmr   3,PMC0
    blr
.global E500_SetPMC0
E500_SetPMC0:
    mtpmr   PMC0,3
    blr
.global E500_GetPMC1
E500_GetPMC1:
    mfpmr   3,PMC1
    blr
.global E500_SetPMC1
E500_SetPMC1:
    mtpmr   PMC1,3
    blr
.global E500_GetPMC2
E500_GetPMC2:
    mfpmr   3,PMC2
    blr
.global E500_SetPMC2
E500_SetPMC2:
    mtpmr   PMC2,3
    blr
.global E500_GetPMC3
E500_GetPMC3:
    mfpmr   3,PMC3
    blr
.global E500_SetPMC3
E500_SetPMC3:
    mtpmr   PMC3,3
    blr
.global E500_GetPMLCa0
E500_GetPMLCa0:
    mfpmr   3,PMLCa0
    blr
.global E500_SetPMLCa0
E500_SetPMLCa0:
    mtpmr   PMLCa0,3
    blr
.global E500_GetPMLCa1
E500_GetPMLCa1:
    mfpmr   3,PMLCa1
    blr
.global E500_SetPMLCa1
E500_SetPMLCa1:
    mtpmr   PMLCa1,3
    blr
.global E500_GetPMLCa2
E500_GetPMLCa2:
    mfpmr   3,PMLCa2
    blr
.global E500_SetPMLCa2
E500_SetPMLCa2:
    mtpmr   PMLCa2,3
    blr
.global E500_GetPMLCa3
E500_GetPMLCa3:
    mfpmr   3,PMLCa3
    blr
.global E500_SetPMLCa3
E500_SetPMLCa3:
    mtpmr   PMLCa3,3
    blr
.global E500_GetPMLCb0
E500_GetPMLCb0:
    mfpmr   3,PMLCb0
    blr
.global E500_SetPMLCb0
E500_SetPMLCb0:
    mtpmr   PMLCb0,3
    blr
.global E500_GetPMLCb1
E500_GetPMLCb1:
    mfpmr   3,PMLCb1
    blr
.global E500_SetPMLCb1
E500_SetPMLCb1:
    mtpmr   PMLCb1,3
    blr
.global E500_GetPMLCb2
E500_GetPMLCb2:
    mfpmr   3,PMLCb2
    blr
.global E500_SetPMLCb2
E500_SetPMLCb2:
    mtpmr   PMLCb2,3
    blr
.global E500_GetPMLCb3
E500_GetPMLCb3:
    mfpmr   3,PMLCb3
    blr
.global E500_SetPMLCb3
E500_SetPMLCb3:
    mtpmr   PMLCb3,3
    blr
.global E500_GetPMGC0
E500_GetPMGC0:
    mfpmr   3,PMGC0
    blr
.global E500_SetPMGC0
E500_SetPMGC0:
    mtpmr   PMGC0,3
    blr


/*****************************************************************************
    Test functions
*****************************************************************************/
.global E500_Test_SC
E500_Test_SC:
    sc
    blr


.global E500_GetSPR_ILLEGAL
E500_GetSPR_ILLEGAL:
    mfspr   3,ILLEGAL       /* 512 */   /* Signal processing and embedded floating-point status and control register 1 */
    blr

.global E500_SetSPR_ILLEGAL
E500_SetSPR_ILLEGAL:
    mtspr   ILLEGAL,3       /* 512 */   /* Signal processing and embedded floating-point status and control register 1 */
    blr

.global E500_ILLEGAL_Inst
E500_ILLEGAL_Inst:
    .long   0x0
    blr

