/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          e500v2.c

 @Description   Upper Level ionterfaces to the ASM functions

 @Cautions

Author: Donio Ron


*//***************************************************************************/
#define __ERR_MODULE__  MODULE_UNKNOWN

#include "error_ext.h"
#include "std_ext.h"
#include "string_ext.h"
#include "core_ext.h"
#include "e500v2_ext.h"
#include "e500v2_asm_ext.h"


#if !defined(__QNXNTO__)
#ifndef CORE_E500MC
extern void saveCoreRegs(uint32_t bptrAddr);
extern void E500_RecoverDeepSleepMode();
void loadCoreRegsORG();

void E500_SetDozeMode(void)
{
    uint32_t tmpReg;

    /* Disable Decrementer interrupt */
    tmpReg = E500_GetSPR_TCR();
    tmpReg &= ~TCR_DIE;
    E500_SetSPR_TCR(tmpReg);
}

void E500_SetNapMode(void)
{
    uint32_t tmpReg;

    /* Disable Decrementer interrupt */
    tmpReg = E500_GetSPR_TCR();
    tmpReg &= ~TCR_DIE;
    E500_SetSPR_TCR(tmpReg);

    /* set the core in NAP mode */
    tmpReg = E500_GetSPR_HID0();
    tmpReg |= HID0_NAP;
    E500_SetSPR_HID0(tmpReg);

    /* The L1 caches do not respond to snoops in nap mode,
    the L1 cache must be flushed before entering nap mode. */
    L1DCache_Flush();
    L1ICache_Flush();

    MSR_EnableWE();
}

void E500_SetSleepMode(void)
{

    uint32_t tmpReg;

    /* Disable Decrementer interrupt */
    tmpReg = E500_GetSPR_TCR();
    tmpReg &= ~TCR_DIE;
    E500_SetSPR_TCR(tmpReg);

    /* The L1 caches do not respond to snoops in sleep mode,
    the L1 cache must be flushed before entering nap mode. */
    L1DCache_Flush();
    L1ICache_Flush();
}

t_Error E500_SetDeepSleepMode(uint32_t bptrAddress)
{

    uint32_t    tmpReg;

    saveCoreRegs(bptrAddress);

    MSR_DisableCE();
    MSR_DisableME();

    /* The L1 caches do not respond to snoops in Deep Sleep mode,
    the L1 cache must be flushed before entering nap mode. */
    E500_ICacheDisable();
    E500_DCacheDisable();

    E500_ICacheEnable();

    /* update Boot Page Translation Register */
    tmpReg = (uint32_t)E500_RecoverDeepSleepMode;
    if((tmpReg & 0xFFF) != 0xFFC)
    {
        RETURN_ERROR(MAJOR, E_INVALID_ADDRESS, ("E500_SetDeepSleepMode alignment"));
    }
    tmpReg >>= 12;
    tmpReg |= 0x80000000;
    WRITE_UINT32(*(uint32_t*)bptrAddress,tmpReg);

    /* disable TCR */
    tmpReg = E500_GetSPR_TCR();
    tmpReg &= ~TCR_DIE;
    E500_SetSPR_TCR(tmpReg);

    tmpReg = E500_GetSPR_TSR();
    tmpReg |= TSR_DIS;
    E500_SetSPR_TSR(tmpReg);

    return E_OK;
}

void E500_SetJogMode(void)
{
    uint32_t tmpReg;

    tmpReg = E500_GetSPR_TCR();
    tmpReg &= ~TCR_DIE;
    E500_SetSPR_TCR(tmpReg);

    L1DCache_Flush();
    L1ICache_Flush();

    MSR_DisableEE();
}


void E500_RecoverJogMode(void)
{
    uint32_t tmpReg;

    tmpReg = E500_GetSPR_TCR();
    tmpReg |= TCR_DIE;
    E500_SetSPR_TCR(tmpReg);

    MSR_EnableEE();
}

void E500_RecoverDozeMode(void)
{
    uint32_t tmpReg;

    tmpReg = E500_GetSPR_TCR();
    tmpReg |= TCR_DIE;
    E500_SetSPR_TCR(tmpReg);
}

void E500_RecoverNapMode(void)
{
    uint32_t tmpReg;

    MSR_DisableWE();

    tmpReg = E500_GetSPR_HID0();
    tmpReg &= ~HID0_NAP;
    E500_SetSPR_HID0(tmpReg);

    tmpReg = E500_GetSPR_TCR();
    tmpReg |= TCR_DIE;
    E500_SetSPR_TCR(tmpReg);
}

void E500_RecoverSleepMode(void)
{
    uint32_t tmpReg;

    tmpReg = E500_GetSPR_TCR();
    tmpReg |= TCR_DIE;
    E500_SetSPR_TCR(tmpReg);
}
#endif /* !CORE_E500MC */

/*----------------------------------*/
/*  Data Cache Enable               */
/*----------------------------------*/
void E500_DCacheEnable(void)
{
    L1DCache_Enable();
}

/*----------------------------------*/
/*  Instruction Cache Enable        */
/*----------------------------------*/
void E500_ICacheEnable(void)
{
    L1ICache_Enable();
}

/*----------------------------------*/
/*  Data Cache Disable              */
/*----------------------------------*/
void E500_DCacheDisable(void)
{
   uint32_t       intFlags;

   intFlags = XX_DisableAllIntr();
   L1DCache_Flush();
   L1DCache_Disable();
   XX_RestoreAllIntr(intFlags);
}

/*----------------------------------*/
/*  Instruction Cache Disable       */
/*----------------------------------*/
void E500_ICacheDisable(void)
{
    uint32_t       intFlags;

    intFlags = XX_DisableAllIntr();
    L1ICache_Flush();
    L1ICache_Disable();
    XX_RestoreAllIntr(intFlags);
}

/*----------------------------------*/
/*  Data Cache Flush                */
/*----------------------------------*/
void E500_DCacheFlush(void)
{
    L1DCache_Flush();
}

/*----------------------------------*/
/*  Instruction Cache Flush         */
/*----------------------------------*/
void E500_ICacheFlush(void)
{
    L1ICache_Flush();
}

/*----------------------------------*/
/*  Data Cache Set Stash Id         */
/*----------------------------------*/
void E500_DCacheSetStashId(uint8_t stashId)
{
    uint32_t    reg = (E500_GetSPR_L1CSR2() & not_L1CSR2_STASHID);

    reg |= stashId;

    E500_SetSPR_L1CSR2(reg);
}

/*****************************************************************************/
void E500_EnableTimeBase(void)
{
    uint32_t tHid0;

    tHid0 = E500_GetSPR_HID0();
    tHid0 |= HID0_TBEN;
    E500_SetSPR_HID0(tHid0);
}

/*****************************************************************************/
void E500_DisableTimeBase(void)
{
    uint32_t hid0;

    hid0 = E500_GetSPR_HID0();
    hid0 &= ~(HID0_TBEN);
    E500_SetSPR_HID0(hid0);
}

/*****************************************************************************/
uint64_t E500_GetTimeBaseTime(void)
{
    uint64_t time;
    uint32_t timel, timeh1, timeh2;

    do
    {
        timeh1 = E500_GetSPR_TBU();
        timel = E500_GetSPR_TBL();
        timeh2 = E500_GetSPR_TBU();
    } while (timeh1 != timeh2); /* Verify that no carry took place between
                                   reads of high and low*/

    time = ((uint64_t)timeh1 << 32) | timel;

    return time;
}
#endif

#if defined(__QNXNTO__)
/*
 * Define this here so we can avoid linking e500_asm.o and e500_intr_asm.o
 * and thereby avoid the startup warning about an undefined __start symbol.
 */
uint32_t GetCpuId()
{
    unsigned int pir;
    //__asm__ __volatile__ ("mfpir   %0" : "=r" (pir));
    __asm__ __volatile__ ("mfspr %0, %1" : "=r" (pir): "i"(286)); /* SPRN_PIR */
    return pir;
}
#endif

/*****************************************************************************/
uint32_t E500_GetId(void)
{
    uint32_t cpuId;
#ifdef CORE_E500MC
    cpuId = GetCpuId()>>5;
#else
    cpuId = GetCpuId();
#endif /* CORE_E500MC */

    ASSERT_COND(cpuId < INTG_MAX_NUM_OF_CORES);
    return cpuId;
}


#if !defined(__QNXNTO__)
#ifdef CORE_E500MC
/*****************************************************************************/
/*          E500mc L2 cache                                                  */
/*****************************************************************************/
void E500_L2CacheEnable(e_E500mcL2CacheMode mode)
{

    uint32_t    reg = (E500MC_GetSPR_L2CSR0() & (not_L2CSR0_L2IO & not_L2CSR0_L2DO));

    /*  On boot prior to enabling the L2 cache, the processor should set this bit to clear
        any lock state bits that may be randomly set out of reset. This differs from the
        behavior defined in EREF.
        */

    reg |= L2CSR0_L2FI | L2CSR0_L2LFC;

    E500MC_SetSPR_L2CSR0(reg);

    while (E500MC_GetSPR_L2CSR0() != reg) ;

    reg &= not_L2CSR0_L2FI & not_L2CSR0_L2LFC;
    /* Cache enable */
    reg |= L2CSR0_L2E;

    switch (mode)
    {
        case (e_L2_CACHE_MODE_DATA_ONLY):
            reg |= L2CSR0_L2DO;
            break;
        case (e_L2_CACHE_MODE_INST_ONLY):
            reg |= L2CSR0_L2IO;
            break;
        default:
            break;
    }

    E500MC_SetSPR_L2CSR0(reg);

    while (E500MC_GetSPR_L2CSR0() != reg) ;
}

void E500_L2CacheDisable(void)
{

    uint32_t    intFlags;
    uint32_t    reg = (E500MC_GetSPR_L2CSR0() & not_L2CSR0_L2E);

    intFlags = XX_DisableAllIntr();             //Lock Interrupt!!!

    E500MC_SetSPR_L2CSR0(reg);

    while (E500MC_GetSPR_L2CSR0() != reg) ;
    XX_RestoreAllIntr(intFlags);

}

void E500_L2CacheFlush(void)
{
    uint32_t    reg = (E500MC_GetSPR_L2CSR0() | L2CSR0_L2FL);

    E500MC_SetSPR_L2CSR0(reg);

    while (E500MC_GetSPR_L2CSR0() != reg) ;


}

void E500_L2SetStashId(uint8_t stashId)
{
    uint32_t    reg = (E500MC_GetSPR_L2CSR1() & not_L2CSR1_STASHID);

    reg |= stashId;

    E500MC_SetSPR_L2CSR1(reg);
}

#endif /* CORE_E500MC */
#endif
