/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc., 
 and its use is subject to the NetComm Device Drivers EULA. 
 The copyright notice above does not evidence any actual or intended 
 publication of such source code. 

 **************************************************************************/

#define INCLUDE_FROM_ASM 1

.include "e500v2_asm.h"
.include "e500v2_asm_ext.h"

#define SS_TB		0x00
#define SS_HID		0x08 /* 2 HIDs */
#define SS_IAC		0x10 /* 2 IACs */
#define SS_DAC		0x18 /* 2 DACs */
#define SS_DBCR		0x20 /* 3 DBCRs */
#define SS_PID		0x2c /* 3 PIDs */
#define SS_SPRG		0x38 /* 8 SPRGs */
#define SS_IVOR		0x58 /* 20 interrupt vectors */
#define SS_TCR		0xa8
#define SS_BUCSR	0xac
#define SS_L1CSR	0xb0 /* 2 L1CSRs */
#define SS_MSR		0xb8
#define SS_USPRG	0xbc
#define SS_GPREG	0xc0 /* r12-r31 */
#define SS_LR		0x110
#define SS_CR		0x114
#define SS_SP		0x118
#define SS_CURRENT	0x11c
#define SS_IVPR		0x120
#define SS_BPTR		0x124
#define SS_TLB1     0x12C

#define STATE_SAVE_SIZE 0x256 

.global GetCr
.global GetSp
.global GetCurrent
.global loadCoreRegs
.global saveCoreRegs
.global loadCoreRegsORG
.global E500_RecoverDeepSleepMode

.section .text, 0x1000, 1, 7
dataSaveCoreRegs:
.space  STATE_SAVE_SIZE

.align  0x4
/**********************************************/
E500_RecoverWindow:
    mfspr	r4, PIR
	andi.	r4, r4, 1
	
	lis	r4, 0x1008
	mtspr	MAS0, r4
	
	lis	r4, 0xc000
	ori	r4, r4, 0x0A00
	mtspr	MAS1, r4
	
	lis	r4, 0x0000
	ori	r4, r4, 0x0000
	mtspr	MAS2, r4
	
	li	r4, 0x003F
	mtspr	MAS3, r4
	
	li	r4, 0
	mtspr	MAS7, r4
	
	isync
	tlbwe
	isync
	
	lis	r3, loadCoreRegs@h
	ori	r3, r3, loadCoreRegs@l
	mtctr	r3
	bctr

.org 0xFFC    
E500_RecoverDeepSleepMode:

    b	E500_RecoverWindow    

saveCoreRegs:
    
	lis	r10, dataSaveCoreRegs@h
	ori	r10, r10, dataSaveCoreRegs@l

    lwz	r4, 0(r3)
    stw	r3, SS_BPTR(r10)
    stw	r4, SS_BPTR+4(r10)

	mfspr	r5, HID0
	mfspr	r6, HID1

	stw	r5, SS_HID+0(r10)
	stw	r6, SS_HID+4(r10)

	mfspr	r4, IAC1
	mfspr	r5, IAC2
	mfspr	r6, DAC1
	mfspr	r7, DAC2

	stw	r4, SS_IAC+0(r10)
	stw	r5, SS_IAC+4(r10)
	stw	r6, SS_DAC+0(r10)
	stw	r7, SS_DAC+4(r10)

	mfspr	r4, DBCR0
	mfspr	r5, DBCR1
	mfspr	r6, DBCR2

	stw	r4, SS_DBCR+0(r10)
	stw	r5, SS_DBCR+4(r10)
	stw	r6, SS_DBCR+8(r10)

	mfspr	r4, PID0
	mfspr	r5, PID1
	mfspr	r6, PID2

	stw	r4, SS_PID+0(r10)
	stw	r5, SS_PID+4(r10)
	stw	r6, SS_PID+8(r10)

	mfspr	r4, SPRG0
	mfspr	r5, SPRG1
	mfspr	r6, SPRG2
	mfspr	r7, SPRG3

	stw	r4, SS_SPRG+0x00(r10)
	stw	r5, SS_SPRG+0x04(r10)
	stw	r6, SS_SPRG+0x08(r10)
	stw	r7, SS_SPRG+0x0c(r10)

	mfspr	r4, SPRG4
	mfspr	r5, SPRG5
	mfspr	r6, SPRG6
	mfspr	r7, SPRG7

	stw	r4, SS_SPRG+0x10(r10)
	stw	r5, SS_SPRG+0x14(r10)
	stw	r6, SS_SPRG+0x18(r10)
	stw	r7, SS_SPRG+0x1c(r10)

	mfspr	r4, IVPR
	stw	r4, SS_IVPR(r10)

	mfspr	r4, IVOR0
	mfspr	r5, IVOR1
	mfspr	r6, IVOR2
	mfspr	r7, IVOR3

	stw	r4, SS_IVOR+0x00(r10)
	stw	r5, SS_IVOR+0x04(r10)
	stw	r6, SS_IVOR+0x08(r10)
	stw	r7, SS_IVOR+0x0c(r10)

	mfspr	r4, IVOR4
	mfspr	r5, IVOR5
	mfspr	r6, IVOR6
	mfspr	r7, IVOR7

	stw	r4, SS_IVOR+0x10(r10)
	stw	r5, SS_IVOR+0x14(r10)
	stw	r6, SS_IVOR+0x18(r10)
	stw	r7, SS_IVOR+0x1c(r10)

	mfspr	r4, IVOR8
	mfspr	r5, IVOR9
	mfspr	r6, IVOR10
	mfspr	r7, IVOR11

	stw	r4, SS_IVOR+0x20(r10)
	stw	r5, SS_IVOR+0x24(r10)
	stw	r6, SS_IVOR+0x28(r10)
	stw	r7, SS_IVOR+0x2c(r10)

	mfspr	r4, IVOR12
	mfspr	r5, IVOR13
	mfspr	r6, IVOR14
	mfspr	r7, IVOR15

	stw	r4, SS_IVOR+0x30(r10)
	stw	r5, SS_IVOR+0x34(r10)
	stw	r6, SS_IVOR+0x38(r10)
	stw	r7, SS_IVOR+0x3c(r10)

	mfspr	r4, IVOR32
	mfspr	r5, IVOR33
	mfspr	r6, IVOR34
	mfspr	r7, IVOR35

	stw	r4, SS_IVOR+0x40(r10)
	stw	r5, SS_IVOR+0x44(r10)
	stw	r6, SS_IVOR+0x48(r10)
	stw	r7, SS_IVOR+0x4c(r10)

	mfspr	r4, TCR
	mfspr	r5, BUCSR
	mfspr	r6, L1CSR0
	mfspr	r7, L1CSR1
	mfspr	r8, USPRG0

	stw	r4, SS_TCR(r10)
	stw	r5, SS_BUCSR(r10)
	stw	r6, SS_L1CSR+0(r10)
	stw	r7, SS_L1CSR+4(r10)
	stw	r8, SS_USPRG+0(r10)

	stmw	r12, SS_GPREG(r10)

	mfmsr	r4
	mflr	r5
	mfcr	r6

	stw	r4, SS_MSR(r10)
	stw	r5, SS_LR(r10)
	stw	r6, SS_CR(r10)
	stw	r1, SS_SP(r10)
	stw	r2, SS_CURRENT(r10)

	li	r4, 15
	mtctr	r4
	
	// 14 TLB * 20 each = 118
	li  r4, 0x118
	add r4, r4, r10

saveTLB:

	mfctr   r3
	slwi    r5, r3, 16
	oris    r5, r5, 0x1000
	mtspr	MAS0, r5
	
	tlbre
	
	mfspr	r5, MAS0
    stw	r5, SS_TLB1+0(r4)

	mfspr	r5, MAS1
    stw	r5, SS_TLB1+4(r4)

	mfspr	r5, MAS2
    stw	r5, SS_TLB1+8(r4)

	mfspr	r5, MAS3
    stw	r5, SS_TLB1+12(r4)

	mfspr	r5, MAS7
    stw	r5, SS_TLB1+16(r4)
    
    subi r4, r4, 0x14

    bdnz    saveTLB
	
	blr


loadCoreRegs:

	/* Access the MEM(r10) with TLB1[0] */
	lis	r10, dataSaveCoreRegs@h
	ori	r10, r10, dataSaveCoreRegs@l

	lwz	r5, SS_HID+0(r10)
	lwz	r6, SS_HID+4(r10)

	isync
	mtspr	HID0, r5
	isync

	msync
	mtspr	HID1, r6
	isync

	lwz	r4, SS_IAC+0(r10)
	lwz	r5, SS_IAC+4(r10)
	lwz	r6, SS_DAC+0(r10)
	lwz	r7, SS_DAC+4(r10)

	mtspr	IAC1, r4
	mtspr	IAC2, r5
	mtspr	DAC1, r6
	mtspr	DAC2, r7

	lwz	r4, SS_DBCR+0(r10)
	lwz	r5, SS_DBCR+4(r10)
	lwz	r6, SS_DBCR+8(r10)

	mtspr	DBCR0, r4
	mtspr	DBCR1, r5
	mtspr	DBCR2, r6

	lwz	r4, SS_PID+0(r10)
	lwz	r5, SS_PID+4(r10)
	lwz	r6, SS_PID+8(r10)

	mtspr	PID0, r4
	mtspr	PID1, r5
	mtspr	PID2, r6

	lwz	r4, SS_SPRG+0x00(r10)
	lwz	r5, SS_SPRG+0x04(r10)
	lwz	r6, SS_SPRG+0x08(r10)
	lwz	r7, SS_SPRG+0x0c(r10)

	mtspr	SPRG0, r4
	mtspr	SPRG1, r5
	mtspr	SPRG2, r6
	mtspr	SPRG3, r7

	lwz	r4, SS_SPRG+0x10(r10)
	lwz	r5, SS_SPRG+0x14(r10)
	lwz	r6, SS_SPRG+0x18(r10)
	lwz	r7, SS_SPRG+0x1c(r10)

	mtspr	SPRG4, r4
	mtspr	SPRG5, r5
	mtspr	SPRG6, r6
	mtspr	SPRG7, r7

	lwz	r4, SS_IVPR(r10)
	mtspr	IVPR, r4

	lwz	r4, SS_IVOR+0x00(r10)
	lwz	r5, SS_IVOR+0x04(r10)
	lwz	r6, SS_IVOR+0x08(r10)
	lwz	r7, SS_IVOR+0x0c(r10)

	mtspr	IVOR0, r4
	mtspr	IVOR1, r5
	mtspr	IVOR2, r6
	mtspr	IVOR3, r7

	lwz	r4, SS_IVOR+0x10(r10)
	lwz	r5, SS_IVOR+0x14(r10)
	lwz	r6, SS_IVOR+0x18(r10)
	lwz	r7, SS_IVOR+0x1c(r10)

	mtspr	IVOR4, r4
	mtspr	IVOR5, r5
	mtspr	IVOR6, r6
	mtspr	IVOR7, r7

	lwz	r4, SS_IVOR+0x20(r10)
	lwz	r5, SS_IVOR+0x24(r10)
	lwz	r6, SS_IVOR+0x28(r10)
	lwz	r7, SS_IVOR+0x2c(r10)

	mtspr	IVOR8, r4
	mtspr	IVOR9, r5
	mtspr	IVOR10, r6
	mtspr	IVOR11, r7

	lwz	r4, SS_IVOR+0x30(r10)
	lwz	r5, SS_IVOR+0x34(r10)
	lwz	r6, SS_IVOR+0x38(r10)
	lwz	r7, SS_IVOR+0x3c(r10)

	mtspr	IVOR12, r4
	mtspr	IVOR13, r5
	mtspr	IVOR14, r6
	mtspr	IVOR15, r7

	lwz	r4, SS_IVOR+0x40(r10)
	lwz	r5, SS_IVOR+0x44(r10)
	lwz	r6, SS_IVOR+0x48(r10)
	lwz	r7, SS_IVOR+0x4c(r10)

	mtspr	IVOR32, r4
	mtspr	IVOR33, r5
	mtspr	IVOR34, r6
	mtspr	IVOR35, r7

	lwz	r4, SS_TCR(r10)
	lwz	r5, SS_BUCSR(r10)
	lwz	r6, SS_L1CSR+0(r10)
	lwz	r7, SS_L1CSR+4(r10)
	lwz	r8, SS_USPRG+0(r10)

	mtspr	TCR, r4
	mtspr	BUCSR, r5

	msync
	isync
	mtspr	L1CSR0, r6
	isync

	mtspr	L1CSR1, r7
	isync

	mtspr	USPRG0, r8

	lmw	r12, SS_GPREG(r10)

	lwz	r1, SS_SP(r10)
	lwz	r2, SS_CURRENT(r10)
	lwz	r4, SS_MSR(r10)
	lwz	r5, SS_LR(r10)
	lwz	r6, SS_CR(r10)

	msync
	mtmsr	r4
	isync

	mtlr	r5
	mtcr	r6

	li	r4, 0
	mtspr	TBL, r4

	lwz	r4, SS_TB+0(r10)
	lwz	r5, SS_TB+4(r10)

	mtspr	TBU, r4
	mtspr	TBL, r5

	lis	r3, 1
	mtdec	r3
    
    li	r4, 15
	mtctr	r4
	
	mr r4, r10
	addi  r4, r4, SS_TLB1

loadTLB:

	lwz	r5, 0(r4)
	lwz	r6, 4(r4)
	lwz	r7, 8(r4)
	lwz	r8, 12(r4)
	lwz	r9, 16(r4)

	mtspr	MAS0, r5
	mtspr	MAS1, r6
	mtspr	MAS2, r7
	mtspr	MAS3, r8
	mtspr	MAS7, r9

    isync
	tlbwe
	isync
    
    addi r4, r4, 0x14

    bdnz    loadTLB
	

    lwz	r3, SS_BPTR(r10)
    lwz	r4, SS_BPTR+4(r10)
    stw r4, 0(r3)  
   
    li       r3,1
    lwz      r31,28(rsp)
    lwz      r30,24(rsp)
    lwz      r4, 36(rsp)
    mr       r0, r4
    mtlr     r0
    addi     rsp,rsp,32
    blr      
    
/******************************************/



