/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc., 
 and its use is subject to the NetComm Device Drivers EULA. 
 The copyright notice above does not evidence any actual or intended 
 publication of such source code. 

 **************************************************************************/
/*

 External interrupt handler code
 Saves all and passes control to ExternalIntrHandler routine

 WARNING #1
    This code will be moved by initialization routine
    Do not use here instructions which depend on its
    placement (for example b/bl).

 WARNING #2
    Length of each handler here cannot exceed 256 bytes (64 instructions)
*/

#define INCLUDE_FROM_ASM
#include "e500v2_asm_ext.h"
#include "e500v2_asm.h"


.extern E500_GenericIntrHandler    /* void E500_GenericIntrHandler(unsigned long exceptNumber) */
.extern E500_MachineCheckIntr
.extern EPIC_Intr
.extern CriticalIntr
.extern DEC_Intr
.extern XX_BottomHalf
/* .extern __reset */


/*****************************************************************************


    isr_reduced_gprs_store

    Save the general purpose registers for an interrupt service routine

*****************************************************************************/
isr_reduced_gprs_store:     .macro
    /* Saving registers (store registers 0, 3-12 in stack)
    store  rx,     offset_y     from    rz  */
    stw     0,  (2 + 0)*  LEN_OF_REG    (1)
    stw     3,  (2 + 1)*  LEN_OF_REG    (1)
    stw     4,  (2 + 2)*  LEN_OF_REG    (1)
    stw     5,  (2 + 3)*  LEN_OF_REG    (1)
    stw     6,  (2 + 4)*  LEN_OF_REG    (1)
    stw     7,  (2 + 5)*  LEN_OF_REG    (1)
    stw     8,  (2 + 6)*  LEN_OF_REG    (1)
    stw     9,  (2 + 7)*  LEN_OF_REG    (1)
    stw     10, (2 + 8)*  LEN_OF_REG    (1)
    stw     11, (2 + 9)*  LEN_OF_REG    (1)
    stw     12, (2 + 10)* LEN_OF_REG    (1)
    .endm

/*****************************************************************************


    isr_reduced_gprs_restore

    Restores the general purpose registers for an interrupt service routine

*****************************************************************************/
isr_reduced_gprs_restore:   .macro
    /* load registers 0, 3-12 from stack where they were saved.
    load   rx,     offset_y     from    rz */
    lwz     0,  (2 + 0)*  LEN_OF_REG    (1)
    lwz     3,  (2 + 1)*  LEN_OF_REG    (1)
    lwz     4,  (2 + 2)*  LEN_OF_REG    (1)
    lwz     5,  (2 + 3)*  LEN_OF_REG    (1)
    lwz     6,  (2 + 4)*  LEN_OF_REG    (1)
    lwz     7,  (2 + 5)*  LEN_OF_REG    (1)
    lwz     8,  (2 + 6)*  LEN_OF_REG    (1)
    lwz     9,  (2 + 7)*  LEN_OF_REG    (1)
    lwz     10, (2 + 8)*  LEN_OF_REG    (1)
    lwz     11, (2 + 9)*  LEN_OF_REG    (1)
    lwz     12, (2 + 10)* LEN_OF_REG    (1)
    .endm

/*****************************************************************************


    isr_sprs_store

    Save the special purpose registers for an interrupt service routine

*****************************************************************************/
isr_sprs_store:     .macro
    mfspr   7,  SRR0
    stw     7,  (2 + NUM_OF_GREG + 0)* LEN_OF_REG(1)
    mfspr   8,  SRR1
    stw     8,  (2 + NUM_OF_GREG + 1)* LEN_OF_REG(1)
    mfcr    9
    stw     9,  (2 + NUM_OF_GREG + 2)* LEN_OF_REG(1)
    mfspr   10, LR
    stw     10, (2 + NUM_OF_GREG + 3)* LEN_OF_REG(1)
    mfspr   11, CTR
    stw     11, (2 + NUM_OF_GREG + 4)* LEN_OF_REG(1)
    mfspr   12,  XER
    stw     12, (2 + NUM_OF_GREG + 5)* LEN_OF_REG(1)
    .endm

/*****************************************************************************

    isr_sprs_restore

    Restores the special purpose registers for an interrupt service routine

*****************************************************************************/
isr_sprs_restore:     .macro
    lwz     7,     (2 + NUM_OF_GREG + 0)* LEN_OF_REG(1)
    mtspr   SRR0,   7     
    lwz     8,     (2 + NUM_OF_GREG + 1)* LEN_OF_REG(1)
    mtspr   SRR1,   8
    lwz     9,     (2 + NUM_OF_GREG + 2)* LEN_OF_REG(1)
    mtcr    9
    lwz     10,    (2 + NUM_OF_GREG + 3)* LEN_OF_REG(1)
    mtspr   LR,     10
    lwz     11,    (2 + NUM_OF_GREG + 4)* LEN_OF_REG(1)
    mtspr   CTR,    11
    lwz     12,    (2 + NUM_OF_GREG + 5)* LEN_OF_REG(1)
    mtspr   XER,    12
    .endm

/*****************************************************************************

    isr_critical_sprs_store

    Save the special purpose registers for a critical interrupt service routine

*****************************************************************************/
isr_critical_sprs_store:    .macro
    mfspr   7,  CSRR0
    stw     7,  (2 + NUM_OF_GREG + 0)* LEN_OF_REG(1)
    mfspr   8,  CSRR1
    stw     8,  (2 + NUM_OF_GREG + 1)* LEN_OF_REG(1)
    mfcr    9
    stw     9,  (2 + NUM_OF_GREG + 2)* LEN_OF_REG(1)
    mfspr   10, LR
    stw     10, (2 + NUM_OF_GREG + 3)* LEN_OF_REG(1)
    mfspr   11, CTR
    stw     11, (2 + NUM_OF_GREG + 4)* LEN_OF_REG(1)
    mfspr   12, XER
    stw     12, (2 + NUM_OF_GREG + 5)* LEN_OF_REG(1)
    .endm

/*****************************************************************************

    isr_critical_sprs_restore

    Restores the special purpose registers for an interrupt service routine

*****************************************************************************/
isr_critical_sprs_restore:  .macro
    lwz     7,      (2 + NUM_OF_GREG + 0)* LEN_OF_REG(1)
    mtspr   CSRR0,  7     
    lwz     8,      (2 + NUM_OF_GREG + 1)* LEN_OF_REG(1)
    mtspr   CSRR1,  8
    lwz     9,      (2 + NUM_OF_GREG + 2)* LEN_OF_REG(1)
    mtcr    9
    lwz     10,     (2 + NUM_OF_GREG + 3)* LEN_OF_REG(1)
    mtspr   LR,     10
    lwz     11,     (2 + NUM_OF_GREG + 4)* LEN_OF_REG(1)
    mtspr   CTR,    11
    lwz     12,     (2 + NUM_OF_GREG + 5)* LEN_OF_REG(1)
    mtspr   XER,    12
    .endm

/*****************************************************************************

    isr_machine_check_sprs_store

    Save the special purpose registers for a machine check interrupt service routine

*****************************************************************************/
isr_machine_check_sprs_store:   .macro
    mfspr   7,  MCSRR0
    stw     7,  (2 + NUM_OF_GREG + 0)* LEN_OF_REG(1)
    mfspr   8,  MCSRR1
    stw     8,  (2 + NUM_OF_GREG + 1)* LEN_OF_REG(1)
    mfcr    9
    stw     9,  (2 + NUM_OF_GREG + 2)* LEN_OF_REG(1)
    mfspr   10, LR
    stw     10, (2 + NUM_OF_GREG + 3)* LEN_OF_REG(1)
    mfspr   11, CTR
    stw     11, (2 + NUM_OF_GREG + 4)* LEN_OF_REG(1)
    mfspr   12, XER
    stw     12, (2 + NUM_OF_GREG + 5)* LEN_OF_REG(1)
    .endm

/*****************************************************************************

    isr_machine_check_sprs_restore

    Restores the special purpose registers for a machine check interrupt service routine

*****************************************************************************/
isr_machine_check_sprs_restore:     .macro
    lwz     7,     (2 + NUM_OF_GREG + 0)* LEN_OF_REG(1)
    mtspr   MCSRR0,   7     
    lwz     8,     (2 + NUM_OF_GREG + 1)* LEN_OF_REG(1)
    mtspr   MCSRR1,   8
    lwz     9,     (2 + NUM_OF_GREG + 2)* LEN_OF_REG(1)
    mtcr    9
    lwz     10,    (2 + NUM_OF_GREG + 3)* LEN_OF_REG(1)
    mtspr   LR,     10
    lwz     11,    (2 + NUM_OF_GREG + 4)* LEN_OF_REG(1)
    mtspr   CTR,    11
    lwz     12,    (2 + NUM_OF_GREG + 5)* LEN_OF_REG(1)
    mtspr   XER,     12
    .endm

/*****************************************************************************
    isr_prologue

    Saves the necessary registers for an interrupt service routine
*****************************************************************************/
isr_prologue:   .macro
    /* move stack pointer (R1) to leave space for saved registers.
       and for the two registers the called function is going to save 
       (LR and SP) */
    addi    1, 1, -((NUM_OF_GREG + NUM_OF_SREG + 2) * LEN_OF_REG )

    isr_reduced_gprs_store
    isr_sprs_store
    .endm


/*****************************************************************************
    isr_epilogue

    Restores the necessary registers for an interrupt service routine
*****************************************************************************/
isr_epilogue:   .macro
    isr_sprs_restore        
    isr_reduced_gprs_restore

    /* advance the stack pointer to point to the valid stack frame! */
    addi    1, 1, ((NUM_OF_GREG + NUM_OF_SREG + 2) * LEN_OF_REG )
    rfi
    .endm


/*****************************************************************************
    isr_prologue_ci

    Saves the necessary registers for a critical interrupt service routine
*****************************************************************************/
isr_prologue_ci:    .macro
    /* move stack pointer (R1) to leave space for saved registers.
       and for the two registers the called function is going to save 
       (LR and SP) */
    addi    1, 1, -(( NUM_OF_GREG + NUM_OF_SREG + 2) * LEN_OF_REG)

    isr_reduced_gprs_store       
    isr_critical_sprs_store

    # Enable Debug interrupts So we can work with CW Jtag debugger 
    mfmsr   3
    ori     4, 3, MSR_DE
    mtmsr   4
    .endm


/*****************************************************************************
    isr_epilogue_ci

    Restores the necessary registers for a critical interrupt service routine
*****************************************************************************/
isr_epilogue_ci:   .macro
    isr_critical_sprs_restore        
    isr_reduced_gprs_restore

    /* advance the stack pointer to point to the valid stack frame! */
    addi    1, 1, (( NUM_OF_GREG + NUM_OF_SREG + 2) * LEN_OF_REG)
    rfci
    .endm

/*****************************************************************************
    isr_prologue_mc

    Saves the necessary registers for a machine check interrupt service routine
*****************************************************************************/
isr_prologue_mc:     .macro
    
    /* move stack pointer (R1) to leave space for saved registers.
       and for the two registers the called function is going to save 
       (LR and SP) */
    addi    1, 1, -(( NUM_OF_GREG + NUM_OF_SREG + 2) * LEN_OF_REG)

    isr_reduced_gprs_store       
    isr_machine_check_sprs_store

    .endm


/*****************************************************************************
    isr_epilogue_mc

    Restores the necessary registers for machine check interrupt service routine
*****************************************************************************/
isr_epilogue_mc:   .macro


        isr_machine_check_sprs_restore        
        isr_reduced_gprs_restore
        
        /* advance the stack pointer to point to the valid stack frame! */
        addi    1, 1, (( NUM_OF_GREG + NUM_OF_SREG + 2) * LEN_OF_REG)

        rfmci
                .endm


/*****************************************************************************
    run_bottom_halves

    Runs Bottom-Halves queues
*****************************************************************************/
run_bottom_halves:     .macro

        /* C Bottom-Half handler call */
    addis   10, 0, XX_BottomHalf@h
    ori     10, 10, XX_BottomHalf@l
    mtspr   LR, 10
    addi    rsp, rsp, -(2 * LEN_OF_REG) /* C-routine writes old LR and
                                           stack frame back chain here */
    bclrl   ALWAYS, 0
    addi    rsp, rsp, (2 * LEN_OF_REG)  /* But we do not need them */

    .endm



.section .text,4,1,6

/*****************************************************************************
    Init the vector SPRs
*****************************************************************************/
.global  E500_InitInterruptVector
.align  8
E500_InitInterruptVector:

    li      3, 0x0000
    lis     3,interrupt_base@ha
    mtspr   IVPR,3

    li      3, 0x0000
    ori     3,3,critical_intr@l
    mtspr   IVOR0,3         /* 400 */   /* critical_intr */

    li      3, 0x0000
    ori     3,3,machine_check_intr@l
    mtspr   IVOR1,3         /* 401 */   /* machine_check_intr */

    li      3, 0x0000
    ori     3,3,data_storage_intr@l
    mtspr   IVOR2,3         /* 402 */   /* data_storage_intr */

    li      3, 0x0000
    ori     3,3,instruction_storage_intr@l
    mtspr   IVOR3,3         /* 403 */   /* instruction_storage_intr*/

    li      3, 0x0000
    ori     3,3,external_intr@l
    mtspr   IVOR4,3         /* 404 */   /* external_intr */

    li      3, 0x0000
    ori     3,3,alignment_intr@l
    mtspr   IVOR5,3         /* 405 */   /* alignment_intr */

    li      3, 0x0000
    ori     3,3,program_intr@l
    mtspr   IVOR6,3         /* 406 */   /* program_intr */

#if 0
    /* Not Supported on e500 */
    li      3, 0x0000
    ori     3,3,floating_point_intr@l
    mtspr   IVOR7,3         /* 407 */   /* floating_point_intr */
#endif /* 0 */

    li      3, 0x0000
    ori     3,3,system_call_intr@l
    mtspr   IVOR8,3         /* 408 */   /* system_call_intr */

    li      3, 0x0000
    ori     3,3,auxiliary_intr@l
    mtspr   IVOR9,3         /* 409 */   /* auxiliary_intr */

    li      3, 0x0000
    ori     3,3,decrementer_intr@l
    mtspr   IVOR10,3        /* 410 */   /* decrementer_intr */

    li      3, 0x0000
    ori     3,3,fixed_interval_intr@l
    mtspr   IVOR11,3        /* 411 */   /* fixed_interval_intr */

    li      3, 0x0000
    ori     3,3,watchdog_timer_intr@l
    mtspr   IVOR12,3        /* 412 */   /* watchdog_timer_intr */

    li      3, 0x0000
    ori     3,3,d_tlb_error_intr@l
    mtspr   IVOR13,3        /* 413 */   /* d_tlb_error_intr */

    li      3, 0x0000
    ori     3,3,i_tlb_error_intr@l
    mtspr   IVOR14,3        /* 414 */   /* i_tlb_error_intr */

    li      3, 0x0000
    ori     3,3,spe_apu_intr@l
    mtspr   IVOR32,3        /* 528 */   /* spe_apu_intr */

    li      3, 0x0000
    ori     3,3,spe_flt_data_intr@l
    mtspr   IVOR33,3        /* 529 */   /* spe_flt_data_intr */

    li      3, 0x0000
    ori     3,3,spe_flt_round_intr@l
    mtspr   IVOR34,3        /* 530 */   /* spe_flt_round_intr */

    li      3, 0x0000
    ori     3,3,perf_monitor_intr@l
    mtspr   IVOR35,3        /* 531 */   /* perf_monitor_intr */

    blr

.section .interruptVector,4,1,6

/*****************************************************************************
    0x0100 System Reset (not used in e500)
*****************************************************************************/
/*
.org    SYSTEM_RESET_INTR
reset:
        lis        4,__reset@h
        ori        4,4,__reset@l
        mtlr    4
        blr
*/

/* Must be first */
interrupt_base:
        .long   0

/*****************************************************************************
    0x0100 Critical interrupt
*****************************************************************************/
.align  0x100
critical_intr:
    isr_prologue_ci

    li      3,CRITICAL_INTR
    addis   3,0,0
    ori     3,3,CRITICAL_INTR

    lis     4,CriticalIntr@h
    ori     4,4,CriticalIntr@l

    mtlr    4
    blrl

    isr_epilogue_ci

/*****************************************************************************
    0x200 Machine Check
*****************************************************************************/
.align  0x100
machine_check_intr:
    isr_prologue_mc

    li      3,MACHINE_CHECK_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue_mc

/*****************************************************************************
    0x0300 Data Storage
*****************************************************************************/
.align  0x100
data_storage_intr:
    isr_prologue

    li      3,DATA_STORAGE_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x0400 Instruction Storage
*****************************************************************************/
.align  0x100
instruction_storage_intr:
    isr_prologue

    li      3,INSTRUCTION_STORAGE_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x500 External interrupt
*****************************************************************************/
.align  0x100
external_intr:
    isr_prologue

    /* Fast handler call directly to the C function without going
       through the generic interrupt handler. */
    addis   10, 0,  EPIC_Intr@h
    ori     10, 10, EPIC_Intr@l
    mtlr    10
    blrl

#ifndef DISABLE_TASKLETS
    run_bottom_halves
#endif /* DISABLE_TASKLETS */

    isr_epilogue

/*****************************************************************************
    0x0600 Alignment
*****************************************************************************/
.align  0x100
alignment_intr:
    isr_prologue

    li      3,ALIGNMENT_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x0700 Program
*****************************************************************************/
.align  0x100
program_intr:
    isr_prologue

    li      3,PROGRAM_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x0800 Floating Point Unavailable
*****************************************************************************/
.align  0x100
floating_point_intr:
    isr_prologue

    li      3,FLOATING_POINT_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x900 Decrementer
*****************************************************************************/
.align  0x100
decrementer_intr:
    isr_prologue

    li      3,DECREMENTER_INTR
    lis     4,DEC_Intr@h
    ori     4,4,DEC_Intr@l
     mtlr   4
    blrl

#ifndef DISABLE_TASKLETS
    run_bottom_halves
#endif /* DISABLE_TASKLETS */

    isr_epilogue
/*****************************************************************************
    0x0A00 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
    0x0B00 Watchdog timer interrupt offset
*****************************************************************************/
.align  0x100
watchdog_timer_intr:
    isr_prologue

    li      3,WATCHDOG_TIMER_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue
/*****************************************************************************
    0x0C00 System Call
*****************************************************************************/
.align  0x100
system_call_intr:
    isr_prologue

    li      3,SYSTEM_CALL_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue
/*****************************************************************************
    0x0D00 Reserved
*****************************************************************************/
.align  0x100
    nop
/*****************************************************************************
    0x0E00 Reserved
*****************************************************************************/
.align  0x100
    nop
/*****************************************************************************
    0x0F00 Fixed-interval timer interrupt offset
*****************************************************************************/
.align  0x100
fixed_interval_intr:
    isr_prologue

    li      3,FIXED_INTERVAL_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue
/*****************************************************************************
0x1000     Instruction TLB error interrupt offset
*****************************************************************************/
.align  0x100
i_tlb_error_intr:
    isr_prologue

    li      3,I_TLB_ERROR_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x1100 Data TLB error interrupt offset
*****************************************************************************/
.align  0x100
d_tlb_error_intr:
    isr_prologue
    
    li      3,D_TLB_ERROR_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue


/*****************************************************************************
    0x1200 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
    0x1300 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
    0x1400 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
0x1500    Debug interrupt offset
*****************************************************************************/
.align  0x100
debug_intr:
    isr_prologue

    li      3,DEBUG_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
0x1600    SPE APU unavailable interrupt offset
*****************************************************************************/
.align  0x100
spe_apu_intr:
    isr_prologue

    li      3,SPE_APU_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
0x1700    SPE floating-point data exception interrupt offset
*****************************************************************************/
.align  0x100
spe_flt_data_intr:
    isr_prologue

    li      3,SPE_FLT_DATA_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
    0x1800 SPE floating-point round exception interrupt offset
*****************************************************************************/
.align  0x100
spe_flt_round_intr:
    isr_prologue

    li      3,SPE_FLT_ROUND_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue

/*****************************************************************************
0x1900 Performance monitor
*****************************************************************************/
.align  0x100
perf_monitor_intr:
    isr_prologue

    li      3,PERF_MONITOR_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue


/*****************************************************************************
    0x1A00 Auxiliary processor unavailable interrupt offset
*****************************************************************************/
.align  0x100
auxiliary_intr:
    isr_prologue

    li      3,AUXILIARY_INTR
    lis     4,E500_GenericIntrHandler@h
    ori     4,4,E500_GenericIntrHandler@l
    mtlr    4
    blrl

    isr_epilogue


/*****************************************************************************
    0x1B00 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
    0x1C00 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
    0x1D00 Reserved
*****************************************************************************/
.align  0x100
    nop

/*****************************************************************************
    0x1E00 Reserved
*****************************************************************************/
.align  0x100
    nop
