/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/

#ifndef __E500V2_L2_H
#define __E500V2_L2_H


#define __ERR_MODULE__  MODULE_L2_CACHE


#define NUMBER_STASH_REGIONS    0x4


typedef _Packed struct t_L2MemMap
{
    /* L2 Cache/SRAM memory mapped registers. */
    volatile uint32_t l2ctl; /* L2 control register */
    volatile uint8_t RESERVED0[0xC];

    struct l2_ext_write_regs
    {
        volatile uint32_t l2cewar;      /* L2 cache external write address register x */
        volatile uint32_t l2cewarea;    /* L2 cache external extended write address register x */
        volatile uint32_t l2cewcr;      /* L2 cache external write control register x */
        volatile uint8_t RESERVED2[0x4];
    } l2_ext_write_regs[NUMBER_STASH_REGIONS];

    volatile uint8_t RESERVED3[0xB0];
    volatile uint32_t l2srbar0;         /* L2 memory-mapped SRAM base address register 0 */
    volatile uint32_t l2srbarea0;       /* L2 memory-mapped SRAM base address register extended address 0 */
    volatile uint32_t l2srbar1;         /* L2 memory-mapped SRAM base address register 1 */
    volatile uint32_t l2srbarea1;       /* L2 memory-mapped SRAM base address register extended address 1 */

    volatile uint8_t RESERVED4[0xCF0];

    volatile uint32_t l2errinjhi;   /*    L2 error injection mask high register */
    volatile uint32_t l2errinjlo;   /*    L2 error injection mask low register */
    volatile uint32_t l2errinjctl;  /*    L2 error injection tag/ECC control register */
    volatile uint8_t RESERVED6[0x14];
    volatile uint32_t l2captdatahi; /*    L2 error data high capture register */
    volatile uint32_t l2captdatalo; /*    L2 errordata low capture register */
    volatile uint32_t l2captecc;    /*    L2 error ECC capture register */
    volatile uint8_t RESERVED7[0x14];
    volatile uint32_t l2errdet;        /* L2 error detect register */
    volatile uint32_t l2errdis;        /* L2 error disable register */
    volatile uint32_t l2errinten;    /* L2 error interrupt enable register */
    volatile uint32_t l2errattr;     /* L2 error attributes capture register */
/*    volatile uint32_t l2erraddr; */   /* L2 error address capture register */
/*    volatile uint8_t RESERVED8[0x4]; */
    volatile uint32_t l2erraddrh;    /* L2 error address capture register high */
    volatile uint32_t l2erraddrl;    /* L2 error address capture register low */
    volatile uint32_t l2errctl;        /* L2 error control register */

} _PackedType t_L2MemMap;


/* L2CTL register */
#define L2CTL_L2E               0x80000000
#define L2CTL_L2I               0x40000000
#define L2CTL_L2BLKSZ_256       0x10000000
#define L2CTL_L2BLKSZ_512       0x20000000
#define L2CTL_L2BLKSZ_1024      0x30000000
#define L2CTL_L2DO              0x00400000
#define L2CTL_L2IO              0x00200000
#define L2CTL_L2INTDIS          0x00080000
#define L2CTL_L2LO              0x00002000
#define L2CTL_L2SLC             0x00001000
#define L2CTL_L2LFR             0x00000400
#define L2CTL_L2LFRID_D         0x00000100
#define L2CTL_L2LFRID_I         0x00000200
#define L2CTL_L2STASHDIS        0x00000008

#define L2CTL_L2SIZ_MASK        0x30000000
#define L2CTL_L2STASHCTL_MASK   0x00000200
#define L2CTL_L2LFRID_MASK      0x00000300

#define L2CTL_MODE_MASK         0x0C030000
#define L2CTL_MODE_SHIFT        16

#define L2CTL_STASH_MASK        0x00000003
#define L2CTL_STASH_SHIFT       0


#define ALIGNED_64KB            0x0000FFFF
#define ALIGNED_128KB           0x0001FFFF
#define ALIGNED_256KB           0x0003FFFF
#define ALIGNED_512KB           0x0007FFFF

#define LL_ADDRESS_LOW_32_BIT_MASK 0x00000000FFFFFFFFLL
#define LL_ADDRESS_EXT_4_BIT_MASK  0x0000000F00000000LL

#define L2CEWCR_EN              0x80000000
#define L2CEWCR_LOCK            0x40000000
#define MIN_REGION_SIZE         256

/* Error registers */
#define L2ERRINJCTL_TERRIEN         0x00010000
#define L2ERRINJCTL_ECCMB           0x00000200
#define L2ERRINJCTL_DERRIEN         0x00000100
#define L2ERRINJCTL_ECCERRIM        0x000000FF

#define L2CAPTECC_ECCSYND           0xFF000000
#define L2CAPTECC_ECCSYND_SHIFT     24
#define L2CAPTECC_ECCCHKSUM         0x000000FF
#define L2CAPTECC_ECCCHKSUM_SHIFT   0

#define L2ERRDET_MULL2ERR           0x80000000
#define L2ERRDET_ALL                L2ERRDET_MULL2ERR | \
                                    e_L2_TAG_PARITY_ERR | \
                                    e_L2_SINGLE_BIT_ERR | \
                                    e_L2_MULT_BIT_ERR | \
                                    e_L2_CONFIG_ERROR

#define L2ERRINTEN_ALL              e_L2_TAG_PARITY_ERR | \
                                    e_L2_SINGLE_BIT_ERR | \
                                    e_L2_MULT_BIT_ERR | \
                                    e_L2_CONFIG_ERROR

#define L2ERRATTR_DWNUM             0x30000000
#define L2ERRATTR_DWNUM_SHIFT       28
#define L2ERRATTR_BURST             0x00800000
#define L2ERRATTR_TRANSSRC          0x001F0000
#define L2ERRATTR_TRANSTYPE         0x00003000
#define L2ERRATTR_VALINFO           0x00000001

/* L2 Detected error transaction size.     */
/* if a burst read, only 1 0f 16 0r 32 bytes sizes are valid. */
#define L2ERRATTR_TRANSSIZ              0x07000000
#define L2ERRATTR_TRANSSIZ_SHIFT        24
#define L2ERRATTR_TRANSSIZ_BURST_32B    0x02000000
#define L2ERRATTR_TRANSSIZ_BURST_16B    0x01000000

#define GET_SINGLE_BEAT(size)   ((size) ?\
                                ((size)>>L2ERRATTR_TRANSSIZ_SHIFT) : 8)
#define GET_BURST(size)        \
        ((((size)>>L2ERRATTR_TRANSSIZ_SHIFT) == L2ERRATTR_TRANSSIZ_BURST_16B) ? \
        16 : ((((size)>>L2ERRATTR_TRANSSIZ_SHIFT) == L2ERRATTR_TRANSSIZ_BURST_32B) ? \
        32:0))


#define L2ERRCTL_L2CTHRESH          0x00FF0000
#define L2ERRCTL_L2CTHRESH_SHIFT    16


typedef struct t_L2DriverParam
{
    bool        l2Active;   /* TRUE to activale L2/SRAM                     */
    e_L2Mode    l2Mode;     /* defines the Cache/SRAM mode.                 */
    e_L2Stash   l2Stash;    /* defines the Stash size.                      */
    bool        dataOnly;   /* Cache mode only: TRUE for allocating only data lines. */
    bool        instructionOnly;
                            /* Cache mode only: TRUE for allocating only instruction lines. */
    bool        cacheInterventionDisable;
                            /* Cache mode only: TRUE to disable cache read intervention. */
    uint64_t    sramBaseAddrBlock0;
    uint64_t    sramBaseAddrBlock1;
                            /* SRAM mode only: Depending of L2Mode, 1 or 2  */
                            /* blocks of SRAM memory must be defined:       */
                            /* e_L2_128Bx2_SRAM: 2 base addresses must be   */
                            /* specified, each of size 128 byte.            */
                            /* e_L2_256B_SRAM: only sramBaseAddrBlock0 must */
                            /* be defined for a 256B block.                 */
                            /* For other SRAM modes, only sramBaseAddrBlock0*/
                            /* must be defined for a 128B block.            */
} t_L2DriverParam;



typedef struct t_L2
{
    t_L2MemMap      *p_MemMap;          /**< a pointer to L2 memory mapped registers.    */
    int             irq;                /**< L2 interrupt ID (platform-specific number) */
    t_L2DriverParam *p_DriverParam;     /**< Pointer to the driver initialization parameters */
    uint8_t         cacheRegionCurr;    /**< current index of externak write region.       */
    bool            L2RegionAllocated[NUMBER_STASH_REGIONS];
                                        /**< Array of Busy/Free regions. */

} t_L2;



#endif /* __E500V2_L2_H */

