/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          hash.c

 @Description   p_HashTable file implements the hash class

 @Cautions      None.

*//***************************************************************************/

#include "hash_ext.h"
#include "xx_ext.h"


/**********************************************************************************/
t_HashTable * HASH_Create(t_HashEntry * (*f_HashFunc)(t_Handle, t_HashEntry*),
                          uint32_t tableSize)
{
    t_HashTable *p_HashTable = NULL;
    uint32_t     binIndex = 0;

    p_HashTable = (t_HashTable *)XX_Malloc(sizeof(t_HashTable));
    if (!p_HashTable)
    {
        return NULL;
    }

    p_HashTable->tableSize = tableSize;
    p_HashTable->numOfItems = 0;
    p_HashTable->f_HashFunc = f_HashFunc;

    p_HashTable->hashTable = (t_HashEntry *)XX_Malloc(tableSize * sizeof(t_HashEntry));
    if (!p_HashTable->hashTable)
    {
        XX_Free(p_HashTable);
        p_HashTable = NULL;
    }

    for (binIndex = 0; binIndex < p_HashTable->tableSize; binIndex++)
    {
        INIT_LIST( &p_HashTable->hashTable[binIndex] );
    }

    return p_HashTable;
}


/*****************************************************************************/
void HASH_Terminate(t_HashTable* p_HashTable)
{
    if (p_HashTable)
    {
        if (p_HashTable->hashTable)
        {
            XX_Free(p_HashTable->hashTable);
        }
        XX_Free(p_HashTable);
    }
}


/*****************************************************************************/
void HASH_Insert(t_HashTable* p_HashTable, t_HashEntry* p_NewEntry)
{
    t_HashEntry *p_Head = p_HashTable->f_HashFunc(p_HashTable, p_NewEntry);

    LIST_Add(p_NewEntry, p_Head);
    p_HashTable->numOfItems++;
}


/*****************************************************************************/
void HASH_DelInit(t_HashTable * p_HashTable, t_HashEntry *p_HashEntry)
{
    LIST_DelAndInit(p_HashEntry);
    p_HashTable->numOfItems--;
}


/*****************************************************************************/
int32_t HASH_IsEmpty(t_HashTable *p_HashTable)
{
    return (p_HashTable->numOfItems == 0);
}


/*****************************************************************************/
t_HashEntry * HASH_Find(t_HashTable *p_HashTable,
                        t_HashEntry *p_LookFor,
                        int32_t (*f_Compare)(t_HashEntry*, t_HashEntry*))
{
    t_HashEntry *p_Pos = NULL;
    t_HashEntry *p_Head =  p_HashTable->f_HashFunc(p_HashTable, p_LookFor);

    LIST_FOR_EACH(p_Pos, p_Head)
    {
        if (f_Compare(p_LookFor, p_Pos))
        {
            return p_Pos;
        }
    }
    return NULL;
}
