/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/****************************************************************
 *
 * File:  mm.h
 *
 *
 * Description:
 *  MM (Memory Management) object definitions.
 *  It also includes definitions of the Free Block, Busy Block
 *  and Memory Block structures used by the MM object.
 *
 ****************************************************************/

#ifndef __MM_H
#define __MM_H


#include "mm_ext.h"

#define __ERR_MODULE__  MODULE_MM


#define MAKE_ALIGNED(addr, align)    \
    (((uint64_t)(addr) + ((align) - 1)) & (~((align) - 1)))


/* t_MemBlock data stucutre defines parameters of the Memory Block */
typedef struct t_MemBlock
{
    struct t_MemBlock *p_Next;    /* Pointer to the next memory block */

    uint64_t  base;             /* Base address of the memory block */
    uint64_t  end;              /* End address of the memory block */
} t_MemBlock;


/* t_FreeBlock data stucutre defines parameters of the Free Block */
typedef struct t_FreeBlock
{
    struct t_FreeBlock *p_Next;   /* Pointer to the next free block */

    uint64_t  base;             /* Base address of the block */
    uint64_t  end;              /* End address of the block */
} t_FreeBlock;


/* t_BusyBlock data stucutre defines parameters of the Busy Block  */
typedef struct t_BusyBlock
{
    struct t_BusyBlock *p_Next;       /* Pointer to the next free block */

    uint64_t  base;                 /* Base address of the block */
    uint64_t  end;                  /* End address of the block */
    char  name[MM_MAX_NAME_LEN];    /* That block of memory was allocated for
                                       something specified by the Name */
} t_BusyBlock;


/* t_MM data structure defines parameters of the MM object */
typedef struct t_MM
{
    t_MemBlock      *memBlocks;     /* List of memory blocks (Memory list) */
    t_BusyBlock     *busyBlocks;    /* List of busy blocks (Busy list) */
    t_FreeBlock     *freeBlocks[MM_MAX_ALIGNMENT + 1];
                                    /* Alignment lists of free blocks (Free lists) */
    t_Handle        h_Spinlock;
} t_MM;


#endif /* __MM_H */
