/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          ddr_ext.h

 @Description   DDR Application Programming Interface
*//***************************************************************************/
#ifndef __DDR_EXT_H
#define __DDR_EXT_H

#include "ddr_std_ext.h"
#include "std_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         ddr_grp DDR Application Programming Interface

 @Description   DDR API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ddr_init_grp DDR Initialization Unit

 @Description   DDR initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   ODT_RD_CFG ODT for reads configuration.
*//***************************************************************************/
typedef enum e_DdrOdtRD
{
    e_DDR_ODT_READ_NEVER        = 0x00000000,   /**< Never assert ODT for reads */
    e_DDR_ODT_READ              = 0x00100000,   /**< Assert ODT only during reads to CSn */
    e_DDR_ODT_READ_OTHER        = 0x00200000,   /**< Assert ODT only during reads to other chip selects */
    e_DDR_ODT_READ_OTHER_DIMM   = 0x00300000,   /**< Assert ODT only during reads to other DIMM modules */
    e_DDR_ODT_READ_ALL          = 0x00400000    /**< Assert ODT for all reads */
} e_DdrOdtRD;

/**************************************************************************//**
 @Description   ODT_WR_CFG ODT for writes configuration.
*//***************************************************************************/
typedef enum e_DdrOdtWR
{
    e_DDR_ODT_WRITE_NEVER       = 0x00000000,   /**< Never assert ODT for writes */
    e_DDR_ODT_WRITE             = 0x00010000,   /**< Assert ODT only during writes to CSn */
    e_DDR_ODT_WRITE_OTHER       = 0x00020000,   /**< Assert ODT only during writes to other chip selects */
    e_DDR_ODT_WRITE_OTHER_DIMM  = 0x00030000,   /**< Assert ODT only during writes to other DIMM modules */
    e_DDR_ODT_WRITE_ALL         = 0x00040000    /**< Assert ODT for all writes */
} e_DdrOdtWR;

/**************************************************************************//**
 @Description   Chip select external parameters structure
*//***************************************************************************/
typedef struct t_DdrCsParam
{
    uint8_t     id;                     /**< 0-3, chip select ID */
    bool        active;                 /**< TRUE if CS is active */
    uint64_t    startAddr;              /**< 64-bit chip select start address
                                             (needed for e500 36-bit address)*/
    uint64_t    size;                   /**< Size of chip select */
    bool        autoPrechargeAlways;    /**< TRUE to enable auto precharge even when
                                             global precharge mode is not set        */
    uint8_t     numOfBankBits;          /**< Number of bank bits, 2 or 3 */
    uint8_t     numOfRows;              /**< Number of rows, 12-16 */
    uint8_t     numOfCols;              /**< Number of columns, 8-11 */
} t_DdrCsParam;

/**************************************************************************//**
 @Description   WRLVL_EN Write Leveling Enable
*//***************************************************************************/
typedef enum e_DdrWritelvlEn
{
    e_DDR_WRITE_LVL_DISABLE        = 0,   /**< Write leveling will not be used */
    e_DDR_WRITE_LVL_ENABLE         = 1    /**< Write leveling will be used */
} e_DdrWritelvlEn;

/**************************************************************************//**
 @Description   ZQ Calibration Time
*//***************************************************************************/
typedef enum e_DdrCalibrationTime
{
    e_DDR_CALIBRATION_TIME_128        = 7,    /**< 128 clocks     */
    e_DDR_CALIBRATION_TIME_256        = 8,    /**< 256 clocks     */
    e_DDR_CALIBRATION_TIME_512        = 9,    /**< 512 clocks     */
    e_DDR_CALIBRATION_TIME_1024       = 10    /**< 1024 clocks    */

} e_DdrCalibrationTime;

/**************************************************************************//**
 @Description   ZQ Full Calibration Time
*//***************************************************************************/
typedef enum e_DdrFullCalibrationTime
{
    e_DDR_FULL_CALIBRATION_TIME_128        = 7,    /**< 128 clocks     */
    e_DDR_FULL_CALIBRATION_TIME_256        = 8,    /**< 256 clocks     */
    e_DDR_FULL_CALIBRATION_TIME_512        = 9,    /**< 512 clocks     */
    e_DDR_FULL_CALIBRATION_TIME_1024       = 10    /**< 1024 clocks    */

} e_DdrFullCalibrationTime;

/**************************************************************************//**
 @Description    ZQ Short Calibration Time
*//***************************************************************************/
typedef enum e_DdrShortCalibrationTime
{
    e_DDR_SHORT_CALIBRATION_1         = 0,    /**< 1 clock    */
    e_DDR_SHORT_CALIBRATION_2         = 1,    /**< 2 clocks   */
    e_DDR_SHORT_CALIBRATION_4         = 2,    /**< 4 clocks   */
    e_DDR_SHORT_CALIBRATION_8         = 3,    /**< 8 clocks   */
    e_DDR_SHORT_CALIBRATION_16        = 4,    /**< 16 clocks  */
    e_DDR_SHORT_CALIBRATION_32        = 5,    /**< 32 clocks  */
    e_DDR_SHORT_CALIBRATION_64        = 6,    /**< 64 clocks  */
    e_DDR_SHORT_CALIBRATION_128       = 7,    /**< 128 clocks */
    e_DDR_SHORT_CALIBRATION_256       = 8,    /**< 256 clocks */
    e_DDR_SHORT_CALIBRATION_512       = 9     /**< 512 clocks */

} e_DdrShortCalibrationTime;

/**************************************************************************//**
 @Description    First DQS pulse rising edge after margining mode is programmed -
                 number of cycles to wait after margining mode has been programmed
                 before the first DQS pulse may be issued.
*//***************************************************************************/
typedef enum e_DdrWrlvlMrd
{
    e_DDR_WRLVL_MRD_1         = 0,    /**< 1 clock    */
    e_DDR_WRLVL_MRD_2         = 1,    /**< 2 clocks   */
    e_DDR_WRLVL_MRD_4         = 2,    /**< 4 clocks   */
    e_DDR_WRLVL_MRD_8         = 3,    /**< 8 clocks   */
    e_DDR_WRLVL_MRD_16        = 4,    /**< 16 clocks  */
    e_DDR_WRLVL_MRD_32        = 5,    /**< 32 clocks  */
    e_DDR_WRLVL_MRD_64        = 6,    /**< 64 clocks  */
    e_DDR_WRLVL_MRD_128       = 7     /**< 128 clocks */

} e_DdrWrlvlMrd;

/**************************************************************************//**
 @Description    ODT delay after margining mode is programmed -
                 number of cycles to wait after margining mode has been programmed until
                 ODT may be asserted.
*//***************************************************************************/
typedef enum e_DdrWrlvlOdten
{
    e_DDR_WRLVL_ODTEN_1         = 0,    /**< 1 clock    */
    e_DDR_WRLVL_ODTEN_2         = 1,    /**< 2 clocks   */
    e_DDR_WRLVL_ODTEN_4         = 2,    /**< 4 clocks   */
    e_DDR_WRLVL_ODTEN_8         = 3,    /**< 8 clocks   */
    e_DDR_WRLVL_ODTEN_16        = 4,    /**< 16 clocks  */
    e_DDR_WRLVL_ODTEN_32        = 5,    /**< 32 clocks  */
    e_DDR_WRLVL_ODTEN_64        = 6,    /**< 64 clocks  */
    e_DDR_WRLVL_ODTEN_128       = 7     /**< 128 clocks */

} e_DdrWrlvlOdten;

/**************************************************************************//**
 @Description    DQS/DQS delay after margining mode is programmed -
                 number of cycles to wait after margining mode has been programmed
                 until DQS may be actively driven.
*//***************************************************************************/
typedef enum e_DdrWrlvlDqsen
{
    e_DDR_WRLVL_DQSEN_1         = 0,    /**< 1 clock    */
    e_DDR_WRLVL_DQSEN_2         = 1,    /**< 2 clocks   */
    e_DDR_WRLVL_DQSEN_4         = 2,    /**< 4 clocks   */
    e_DDR_WRLVL_DQSEN_8         = 3,    /**< 8 clocks   */
    e_DDR_WRLVL_DQSEN_16        = 4,    /**< 16 clocks  */
    e_DDR_WRLVL_DQSEN_32        = 5,    /**< 32 clocks  */
    e_DDR_WRLVL_DQSEN_64        = 6,    /**< 64 clocks  */
    e_DDR_WRLVL_DQSEN_128       = 7     /**< 128 clocks */

} e_DdrWrlvlDqsen;

/**************************************************************************//**
 @Description    Write leveling sample time
*//***************************************************************************/
typedef enum e_DdrWrlvlSmpl
{
    e_DDR_WRLVL_SMPL_1         = 1,    /**< 1 clock    */
    e_DDR_WRLVL_SMPL_2         = 2,    /**< 2 clocks   */
    e_DDR_WRLVL_SMPL_3         = 3,    /**< 3 clocks   */
    e_DDR_WRLVL_SMPL_4         = 4,    /**< 4 clocks   */
    e_DDR_WRLVL_SMPL_5         = 5,    /**< 5 clocks   */
    e_DDR_WRLVL_SMPL_6         = 6,    /**< 6 clocks   */
    e_DDR_WRLVL_SMPL_7         = 7,    /**< 7 clocks   */
    e_DDR_WRLVL_SMPL_8         = 8,    /**< 8 clocks   */
    e_DDR_WRLVL_SMPL_9         = 9,    /**< 9 clock    */
    e_DDR_WRLVL_SMPL_10        = 10,   /**< 10 clocks  */
    e_DDR_WRLVL_SMPL_11        = 11,   /**< 11 clocks  */
    e_DDR_WRLVL_SMPL_12        = 12,   /**< 12 clocks  */
    e_DDR_WRLVL_SMPL_13        = 13,   /**< 13 clocks  */
    e_DDR_WRLVL_SMPL_14        = 14,   /**< 14 clocks  */
    e_DDR_WRLVL_SMPL_15        = 15    /**< 15 clocks  */

} e_DdrWrlvlSmpl;

/**************************************************************************//**
 @Description    Write leveling repetition time.
*//***************************************************************************/
typedef enum e_DdrWrlvlWlr
{
    e_DDR_WRLVL_WLR_1         = 0,    /**< 1 clock    */
    e_DDR_WRLVL_WLR_2         = 1,    /**< 2 clocks   */
    e_DDR_WRLVL_WLR_4         = 2,    /**< 4 clocks   */
    e_DDR_WRLVL_WLR_8         = 3,    /**< 8 clocks   */
    e_DDR_WRLVL_WLR_16        = 4,    /**< 16 clocks  */
    e_DDR_WRLVL_WLR_32        = 5,    /**< 32 clocks  */
    e_DDR_WRLVL_WLR_64        = 6,    /**< 64 clocks  */
    e_DDR_WRLVL_WLR_128       = 7     /**< 128 clocks */

} e_DdrWrlvlWlr;

/**************************************************************************//**
 @Description    Write leveling start time
*//***************************************************************************/
typedef enum e_DdrWrlvlStart
{
    e_DDR_WRLVL_START_0,           /**< 0 clock      */
    e_DDR_WRLVL_START_1_8,         /**< 1/8 clocks   */
    e_DDR_WRLVL_START_1_4,         /**< 1/4 clocks   */
    e_DDR_WRLVL_START_3_8,         /**< 3/8 clocks   */
    e_DDR_WRLVL_START_1_2,         /**< 1/2 clocks   */
    e_DDR_WRLVL_START_5_8,         /**< 5/8 clocks   */
    e_DDR_WRLVL_START_3_4,         /**< 3/4 clocks   */
    e_DDR_WRLVL_START_7_8,         /**< 7/8 clocks   */
    e_DDR_WRLVL_START_1,           /**< 1 clock      */
    e_DDR_WRLVL_START_9_8,         /**< 9/8 clocks   */
    e_DDR_WRLVL_START_5_4,         /**< 5/4 clocks   */
    e_DDR_WRLVL_START_11_8,        /**< 11/8 clocks  */
    e_DDR_WRLVL_START_3_2,         /**< 3/2 clocks   */
    e_DDR_WRLVL_START_13_8,        /**< 13/8 clocks  */
    e_DDR_WRLVL_START_7_4,         /**< 7/4 clocks   */
    e_DDR_WRLVL_START_15_8,        /**< 15/8 clocks  */
    e_DDR_WRLVL_START_2,           /**< 2 clocks     */
    e_DDR_WRLVL_START_17_8,        /**< 17/8 clocks  */
    e_DDR_WRLVL_START_9_4,         /**< 9/4 clocks   */
    e_DDR_WRLVL_START_19_8,        /**< 19/8 clocks  */
    e_DDR_WRLVL_START_5_2          /**< 5/2 clocks   */

} e_DdrWrlvlStart;


/**************************************************************************//**
 @Description   MCAS-to-preamble override.
*//***************************************************************************/
typedef enum e_DdrCpo
{
    e_DDR_CPO_READ_LAT_1         = 0x00000000,  /**< READ_LAT + 1 */
    e_DDR_CPO_READ_LAT           = 0x01000000,  /**< READ_LAT */
    e_DDR_CPO_READ_LAT_PLUS_1_4  = 0x01800000,  /**< READ_LAT + 1/4 */
    e_DDR_CPO_READ_LAT_PLUS_1_2  = 0x02000000,  /**< READ_LAT + 1/2 */
    e_DDR_CPO_READ_LAT_PLUS_3_4  = 0x02800000,  /**< READ_LAT + 3/4 */
    e_DDR_CPO_READ_LAT_PLUS_1    = 0x03000000,  /**< READ_LAT + 1 */
    e_DDR_CPO_READ_LAT_PLUS_5_4  = 0x03800000,  /**< READ_LAT + 5/4 */
    e_DDR_CPO_READ_LAT_PLUS_3_2  = 0x04000000,  /**< READ_LAT + 3/2 */
    e_DDR_CPO_READ_LAT_PLUS_7_4  = 0x04800000,  /**< READ_LAT + 7/4 */
    e_DDR_CPO_READ_LAT_PLUS_2    = 0x05000000,  /**< READ_LAT + 2 */
    e_DDR_CPO_READ_LAT_PLUS_9_4  = 0x05800000,  /**< READ_LAT + 9/4 */
    e_DDR_CPO_READ_LAT_PLUS_5_2  = 0x06000000,  /**< READ_LAT + 5/2 */
    e_DDR_CPO_READ_LAT_PLUS_11_4 = 0x06800000,  /**< READ_LAT + 11/4 */
    e_DDR_CPO_READ_LAT_PLUS_3    = 0x07000000,  /**< READ_LAT + 3 */
    e_DDR_CPO_READ_LAT_PLUS_13_4 = 0x07800000,  /**< READ_LAT + 13/4 */
    e_DDR_CPO_READ_LAT_PLUS_7_2  = 0x08000000,  /**< READ_LAT + 7/2 */
    e_DDR_CPO_READ_LAT_PLUS_15_4 = 0x08800000,  /**< READ_LAT + 15/4 */
    e_DDR_CPO_READ_LAT_PLUS_4    = 0x09000000,  /**< READ_LAT + 4 */
    e_DDR_CPO_READ_LAT_PLUS_17_4 = 0x09800000,  /**< READ_LAT + 17/4 */
    e_DDR_CPO_READ_LAT_PLUS_9_2  = 0x0A000000,  /**< READ_LAT + 9/2 */
    e_DDR_CPO_READ_LAT_PLUS_19_4 = 0x0A800000,  /**< READ_LAT + 19/4 */
    e_DDR_CPO_READ_LAT_AUTO      = 0x0F800000   /**< AUTO Calibrate  */
} e_DdrCpo;

/**************************************************************************//**
 @Description   Write command to write data strobe timing adjustment select
*//***************************************************************************/
typedef enum e_DdrWrDataDelay
{
    e_DDR_WR_DATA_DELAY_0       = 0x00000000,   /**< 0 clock delay */
    e_DDR_WR_DATA_DELAY_1_4     = 0x00000400,   /**< 1/4 clock delay */
    e_DDR_WR_DATA_DELAY_1_2     = 0x00000800,   /**< 1/2 clock delay */
    e_DDR_WR_DATA_DELAY_3_4     = 0x00000C00,   /**< 3/4 clock delay */
    e_DDR_WR_DATA_DELAY_1       = 0x00001000,   /**< 1 clock delay */
    e_DDR_WR_DATA_DELAY_5_4     = 0x00001400,   /**< 5/4 clock delay */
    e_DDR_WR_DATA_DELAY_3_2     = 0x00001800    /**< 3/2 clock delay */
} e_DdrWrDataDelay;

/**************************************************************************//**
 @Description   Bank (chip select) interleaving control.
*//***************************************************************************/
typedef enum e_DdrBankInterleave
{
    e_DDR_INTERLEAVE_NO            = 0x00000000,    /**< No external memory banks are interleaved */
    e_DDR_INTERLEAVE_0AND1         = 0x00004000,    /**< External memory banks 0 and 1 are interleaved */
    e_DDR_INTERLEAVE_2AND3         = 0x00002000,    /**< External memory banks 2 and 3 are interleaved */
    e_DDR_INTERLEAVE_0AND1_2AND3   = 0x00006000,    /**< External memory banks 0 and 1 are interleaved together and
                                                         banks 2 and 3 are interleaved together */
    e_DDR_INTERLEAVE_0AND1AND2AND3 = 0x00000400     /**< External memory banks 0 through 3 are all interleaved together */
} e_DdrBankInterleave;


/* SDRAM Timing Configuration Reg 2 */

/**************************************************************************//**
 @Description   Clock Adjustment select
*//***************************************************************************/
typedef enum e_DdrClkAdjust
{
    e_DDR_CLK_ADJUST_0      = 0x00000000,   /**< Clock will be launched aligned with address/command */
    e_DDR_CLK_ADJUST_1_8    = 0x01000000,   /**< Clock will be launched 1/8 applied cycle after address/command */
    e_DDR_CLK_ADJUST_1_4    = 0x02000000,   /**< Clock will be launched 1/4 applied cycle after address/command */
    e_DDR_CLK_ADJUST_3_8    = 0x03000000,   /**< Clock will be launched 3/8 applied cycle after address/command */
    e_DDR_CLK_ADJUST_1_2    = 0x04000000,   /**< Clock will be launched 1/2 applied cycle after address/command */
    e_DDR_CLK_ADJUST_5_8    = 0x05000000,   /**< Clock will be launched 5/8 applied cycle after address/command */
    e_DDR_CLK_ADJUST_3_4    = 0x06000000,   /**< Clock will be launched 3/4 applied cycle after address/command */
    e_DDR_CLK_ADJUST_7_8    = 0x07000000,   /**< Clock will be launched 7/8 applied cycle after address/command */
    e_DDR_CLK_ADJUST_1      = 0x08000000    /**< Clock will be launched 1 applied cycle after address/command */
} e_DdrClkAdjust;

/**************************************************************************//**
 @Description   ODT configuration select
*//***************************************************************************/
typedef enum e_DdrOdtCfg
{
    e_DDR_ODT_CFG_NEVER     = 0x00000000,   /**< Never assert ODT to internal IOs */
    e_DDR_ODT_CFG_IO_WR     = 0x00200000,   /**< Assert ODT to internal IOs only during writes to DRAM */
    e_DDR_ODT_CFG_IO_RD     = 0x00400000,   /**< Assert ODT to internal IOs only during reads to DRAM */
    e_DDR_ODT_CFG_ALWAYS    = 0x00600000    /**< Always keep ODT asserted to internal IOs */
} e_DdrOdtCfg;

/**************************************************************************//**
 @Description   Error types select
*//***************************************************************************/
typedef enum e_DdrErrType
{
    e_DDR_ADDRESS_PARITY_ERR    = 0x00000100,   /**< Address parity bit error */
    e_DDR_AUTO_CALIBRATION_ERR  = 0x00000080,   /**< Address calibration bit error */
    e_DDR_MULT_BIT_ERR          = 0x00000008,   /**< Multi bit error */
    e_DDR_SINGLE_BIT_ERR        = 0x00000004,   /**< Single bit error */
    e_DDR_MEM_SELECT_ERR        = 0x00000001    /**< Memory select bit error */
} e_DdrErrType;

/**************************************************************************//**
 @Description   Error policy
*//***************************************************************************/
typedef enum e_DdrErrPolicy
{
    e_DDR_ERR_DISABLE,              /**< Error policy disable   */
    e_DDR_ERR_ENABLE_CAPTURE,       /**< Error enable capture   */
    e_DDR_ERR_ENABLE_INTERRUPT      /**< Error enable interrupt */
} e_DdrErrPolicy;

/**************************************************************************//**
 @Description   ZQ Calibration Enable
*//***************************************************************************/
typedef enum e_DdrZqEn
{
    e_DDR_ZQ_DISABLE        = 0,   /**< ZQ Calibration will not be used. */
    e_DDR_ZQ_ENABLE         = 1    /**< ZQ Calibration will be used */
} e_DdrZqEn;

/**************************************************************************//**
 @Description   DDR transaction type - for error management.
*//***************************************************************************/
typedef enum e_DdrTransType
{
    e_DDR_TRANS_TYPE_WR         = 1,    /**< Transaction type: write */
    e_DDR_TRANS_TYPE_RD         = 2,    /**< Transaction type: read */
    e_DDR_TRANS_TYPE_RD_MOD_WR  = 3     /**< Transaction type: read-modify-write */
} e_DdrTransType;

/**************************************************************************//**
 @Description   DDR Control Adjust - Controls the amount of delay to add to the lightly loaded control signals w/ respect to
                all other DRAM address and command signals.
*//***************************************************************************/
typedef enum e_DdrCtrlAdj
{
    e_DDR_LAUNCHED_ALIGNED          = 0,    /**< MODT[0:3], MCS[0:3], and MCKE[0:3] will be launched aligned with the other DRAM address
                                                 and control signals. */
    e_DDR_LAUNCHED_0_5_CYC_LATER    = 1,    /**< launched 0.5 platform cycle later than the other
                                                 RAM address and control signals. */
    e_DDR_LAUNCHED_1_CYC_LATER      = 2,    /**< launched 1 platform cycle later than the other
                                                 RAM address and control signals. */
    e_DDR_LAUNCHED_1_5_CYC_LATER    = 3,    /**< launched 1.5 platform cycle later than the other
                                                 RAM address and control signals. */
    e_DDR_LAUNCHED_2_CYC_LATER      = 4,    /**< launched 2 platform cycle later than the other
                                                 RAM address and control signals. */
    e_DDR_LAUNCHED_3_5_CYC_LATER    = 5     /**< launched 2.5 platform cycle later than the other
                                                 RAM address and control signals. */
} e_DdrCtrlAdj;

/**************************************************************************//**
 @Description   DDR SDRAM DLL Lock Time - This provides the number of cycles that it
                takes for the DRAMs DLL to lock at POR and after exiting self refresh.
                The controller waits the specified number of cycles before
                issuing any commands after exiting POR or self refresh.
*//***************************************************************************/
typedef enum e_DdrDllLock
{
    e_DDR_DLL_LOCK_200              = 0,    /**< 200 clocks. */
    e_DDR_DLL_LOCK_512              = 1     /**< 512 clocks. */
} e_DdrDllLock;

/**************************************************************************//**
 @Description   Data structure to report error attributes to user.

                The exceptions callback routine fills a user structure with the
                current values. Parameters marked with '%' (in their comment)
                are valid only when the attrVal member is TRUE.
*//***************************************************************************/
typedef struct t_DdrErr
{
    bool            multErr;        /**< If TRUE, multiple errors of the same type
                                         detected; Otherwise - single error */
    bool            addrParityErr;  /**< TRUE if Addrss parity error detected;
                                         FALSE otherwise */
    bool            autoCalibErr;   /**< TRUE if ACE Automatic calibration error detected;
                                         FALSE otherwise */
    bool            multBitErr;     /**< TRUE if multi bit ECC error detected;
                                         FALSE otherwise */
    bool            singleBitErr;   /**< TRUE if single bit ECC error detected;
                                         FALSE otherwise */
    bool            memSelErr;      /**< TRUE if memory select error detected;
                                         FALSE otherwise */

    uint32_t        dataCaptureHi;  /**< Memory data path error capture high word */
    uint32_t        dataCaptureLow; /**< Memory data path error injection low word */
    uint32_t        eccCapture;     /**< ECC syndrome bits */

    uint8_t         beatNum;        /**< '%' Data beat error detected on */
    uint8_t         transSize;      /**< '%' Transaction size (in double words) */
    e_TransSrc      transSrc;       /**< '%' Id of the transaction source for the error */
    e_DdrTransType  transType;      /**< '%' DDR Transaction type for the error */
    uint32_t        transAddr;      /**< Address of the transaction */
    bool            attrVal;        /**< TRUE indicates that attribute are valid */
} t_DdrErr;


/**************************************************************************//**
 @Description   General interrupt-callback function.

                Will be used if there is no available specific callback function
                that matches the interrupt type.
*//***************************************************************************/
typedef void (t_DdrExceptions)(t_Handle appId, t_DdrErr *p_DdrError);

/**************************************************************************//**
 @Description   DDR external parameters structure
*//***************************************************************************/
typedef struct t_DdrParam
{
    uint32_t        baseAddress;          /**< Base address of DDR registers in memory */
    int             irq;                  /**< DDR interrupt ID (system-specific number) */
    t_Handle        h_App;                /**< Handle to the upper layer handle */
    bool            fullInit;             /**< FALSE if initialization was done at boot */
    bool            ddr64Bit;             /**< TRUE - for 32-bit bus initialization; FALSE for 64-bit bus */

/* TIMING_CFG_0 */
    uint8_t         tXARD;                /**< Active powerdown exit timing     (tXARD) [# clocks] DDR2 only */
    uint8_t         tXP;                  /**< Precharge powerdown exit timing  (tXP)   [# clocks] DDR2 only */
    uint8_t         tAXPD;                /**< powerdown exit timing            (tAXPD) [# clocks] DDR2 only */
    uint8_t         tMRD;                 /**< Mode register set cycle time     (tMRD)  [# clocks] DDR2 only */

/* TIMING_CFG_1 */
    uint32_t        tRP;                  /**< Precharge to activate interval   (tRP)   [ps] */
    uint32_t        tRAS;                 /**< Activate to Precharge interval   (tRAS)  [ps] */
    uint32_t        tRCD;                 /**< Activate to read/write interval  (tRCD)  [ps] */
    uint8_t         tCL;                  /**< MCAS latency from READ command  (tCL)   [# clocks]*10 */
    uint32_t        tRFC;                 /**< Refresh to activate interval     (tRFC)  [ps] */
    uint32_t        tWR;                  /**< Last data to precharge interval  (tWR)   [ps] */
    uint32_t        tRRD;                 /**< Activate to activate interval    (tRRD)  [ps] */
    uint32_t        tWTR;                 /**< Last write data pair to read command interval (tWTR) [ps] */

/* TIMING_CFG_2 */
    uint32_t        tRTP;                 /**< Read to precharge                (tRTP)  [ps] */
    uint8_t         tCKE;                 /**< Minimum CKE pulse width          (tCKE)  [# clocks] */
    uint32_t        tFAW;                 /**< Window for four activates        (tFAW)  [ps] */

/* DDR_SDRAM_CFG */
    bool            sren;                 /**< TRUE if Self refresh enable (during sleep) */
    bool            eccEn;                /**< TRUE if ECC enable */
    bool            rdEn;                 /**< TRUE if Registered DIMM enable FALSE if unbuffered DIMMs */
    e_DdrType       ddrType ;             /**< SDRAM_TYPE Type of SDRAM is DDR1/DDR2/DDR3  device to be used. */
    bool            dynPwr;               /**< Dynamic power management mode */
    bool            bus32BitEnable;       /**< TRUE for 32 bit bus, FALSE for 64 bit bus */
    bool            burst8BitEnable;      /**< TRUE for 8 bit burst, FALSE for 4 bit burst */
    bool            ncap;                 /**< TRUE if old system don't support concurrent auto-precharge*/
    bool            timing2TEnable;       /**< TRUE for 2T timing enabled, FALSE for 1T timing */
    e_DdrBankInterleave bankInterleave;   /**< BA_INTLV_CTL Bank (chip select) interleaving control */
    bool            x32En;                /**< TRUE for x32 discrete DRAM chips are used
                                               FALSE Either x8 or x16 discrete DRAM chips are used */
    bool            pchb8;                /**< TRUE for Precharge bit 8 enable (only if x32_EN enabled) */
    bool            hse;                  /**< TRUE for  Global half-strength override */

/* DDR_SDRAM_CFG_2 */
    bool            dqsCfg;               /**< DQS configuration */
    e_DdrOdtCfg     odtCfg;               /**< ODT configuration */
    uint8_t         numPr;                /**< Number of posted refreshes, 1-8 times [# clocks] */
    bool            addressParityEn;      /**< Address Parity Enable. Determines if address parity is generated and checked for the address and
                                               control signals when using registered DIMMs. */
    bool            mirroredDimmEn;       /**< Mirrored DIMM Enable */
    bool            dllReset;             /**< Dll reset enable */

/* DDR_SDRAM_MODE */
    bool            prechargePd;                /**< Precharge Power-Down */
    uint32_t        tCK;                        /**< This byte defines the minimum cycle time for the SDRAM module */
    bool            asrSupport;                 /**< Auto self refresh support */
    bool            extTemperatureRange;        /**< Extended operating temperature range support */
    bool            useExtTemperatureRange;     /**< Extended operating temperature range usage */
    bool            extTemperatureRefreshRate;  /**< Double refreshe rate for Extended operating temperature */
    uint8_t         sdmodeBl;                   /**< Burst Length */
/* DDR_SDRAM_INTERVAL */
    uint32_t        tREFI;                /**< Refresh interval [ps] */

/* DDR_SDRAM_CLK_CNTL */
    e_DdrClkAdjust  clkAdjust;            /**< Clock adjustment select */

/* TIMING_CFG_4 */
    uint8_t         rwt;                  /**< Read-to-write turnaround for same chip select.*/
    uint8_t         wrt;                  /**< Write-to-read turnaround for same chip select.*/
    uint8_t         rrt;                  /**< Read-to-read turnaround for same chip select.*/
    uint8_t         wwt;                  /**< Write-to-write turnaround for same chip select.*/
    e_DdrDllLock    dllLock;              /**< DDR SDRAM DLL Lock Time.*/

/* TIMING_CFG_5 */
    uint32_t        rodtOn;               /**< Read to ODT on. Specifies the number of cycles that
                                               passes from when a read command is placed on
                                               the DRAM bus until the assertion of the relevant ODT signal(s).*/
    uint16_t        rodtOff;              /**< Read to ODT off. Specifies the number of cycles that the relevant
                                               ODT signal(s) remains asserted for each read transaction.*/
    uint32_t        wodtOn;               /**< Write to ODT On Specifies the number of cycles that passes from
                                               when a write command is placed on the DRAM bus until the assertion
                                               of the relevant ODT signal(s)*/
    uint16_t        wodtOff;              /**< Write to ODT Off. Specifies the number of cycles that the relevant
                                               ODT signal(s) remains asserted for each write transaction.*/

/* DDR_ZQ_CNTL */
    bool                        zqEn;     /**< ZQ Calibration Enable */
    e_DdrCalibrationTime        zqInit;   /**< POR ZQ Calibration Time. Determines the number of cycles that
                                               must be allowed for DRAM ZQ calibration at POR.*/
    e_DdrFullCalibrationTime    zqOper;   /**< Normal Operation Full Calibration Time. Determines the number
                                               of cycles that must be allowed for DRAM ZQ calibration when
                                               exiting self refresh.*/
    e_DdrShortCalibrationTime   zqCs;     /**< Normal Operation Short Calibration Time */

/* DDR_WRLVL_CNTL */
    e_DdrWritelvlEn             wrlvlEn;  /**< Write Leveling Enable.*/

    uint8_t                     tWLMRD;   /**< First DQS, DQS# rising edge [CK]*/
    uint8_t                     tWLO;     /**< Write leveling output delay [ps]*/
    uint8_t                     tWLDQSEN; /**< DQS, DQS# delay [CK]*/

    e_DdrWrlvlStart             wrlvlStarts;        /**< Write leveling start time. Determines the value to use for the
                                                         DQS_ADJUST for the first sample when write leveling is enabled.*/

    bool                        emodeDLLEnable;     /**< Extended mode register:
                                                         TRUE to enable DLL, FALSE to disable */
    t_DdrCsParam                csParam[4];         /**< Chip select parameters array */
    t_DdrExceptions             *f_Exceptions;      /**< User's general interrupt callback routine  */
    uint32_t                    ddrClk;             /**< Ddr clock in MHz units */

} t_DdrParam;

/**************************************************************************//**
 @Description   Clock enable control structure
*//***************************************************************************/
typedef enum e_DdrClockEnableControl
{
    e_DDR_CKE_CNTL_NOT_FORCED       =   0,          /**< CKE signals are not forced by software. */
    e_DDR_CKE_CNTL_FORCED_TO_LOW    =   1,          /**< CKE signals are forced to a low value by software. */
    e_DDR_CKE_CNTL_FORCED_TO_HIGH   =   2           /**< CKE signals are forced to a high value by software. */

}e_DdrClockEnableControl;


/**************************************************************************//**
 @Function      DDR_Config

 @Description   Creation of DDR object.

                This routine must be called befor any other DDR routine may be called.

                This routine may be called at boot sequence or
                when DDR boot configuration should be changed.
                It may also be called assuming boot code was already
                run, only for initializing the DDR object. In this case
                FALSE must be passed as the 'fullInit' parameter at the
                parameters structure.

@Param[in]      p_DdrParam  - A structure of parameters for the DDR.
                              All parameters should be defined, including existing
                              values that shouldn't be changed.
                              NULL for only creating DDR object.

@Return         Handle to the initialized DDR object.
*//***************************************************************************/
t_Handle DDR_Config(t_DdrParam *p_DdrParam);

/**************************************************************************//**
 @Function      DDR_Init

 @Description   Init of the DDR Controler hardware.

                Init all relevant DDR registers.

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_Init(t_Handle h_Ddr);

/**************************************************************************//**
 @Group         ddr_adv_init_grp DDR Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DDR_Config_tRWT

 @Description   Advanced configuration of tRWT value.

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     tRWT    - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tRWT(t_Handle h_Ddr, uint8_t tRWT);

/**************************************************************************//**
 @Function      DDR_Config_tWRT

 @Description   Advanced configuration of tWRT value

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     tWRT    - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tWRT(t_Handle h_Ddr, uint8_t tWRT);

/**************************************************************************//**
 @Function      DDR_Config_tRRT

 @Description   Advanced configuration of tRRT value

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     tRRT    - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tRRT(t_Handle h_Ddr, uint8_t tRRT);

/**************************************************************************//**
 @Function      DDR_Config_tWWT

 @Description   Advanced configuration of tWWT value

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     tWWT    - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tWWT(t_Handle h_Ddr, uint8_t tWWT);

/**************************************************************************//**
 @Function      DDR_Config_tADD_LAT

 @Description   Advanced configuration of tADD_LAT value

 @Param[in]     h_Ddr       - Handle to the initialized DDR object.
 @Param[in]     tADD_LAT    - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tADD_LAT(t_Handle h_Ddr, uint8_t tADD_LAT);

/**************************************************************************//**
 @Function      DDR_Config_tCPO

 @Description   Advanced configuration of tCPO value

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     tCPO    - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tCPO(t_Handle h_Ddr, e_DdrCpo  tCPO);

/**************************************************************************//**
 @Function      DDR_Config_tWR_LAT

 @Description   Advanced configuration of tWR_LAT value

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     tWR_LAT - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tWR_LAT(t_Handle h_Ddr, uint8_t tWR_LAT);

/**************************************************************************//**
 @Function      DDR_Config_tWR_DATA_DELAY

 @Description   Advanced configuration of tWR_DATA_DELAY value

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     tWR_DATA_DELAY  - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_Config_tWR_DATA_DELAY(t_Handle h_Ddr, e_DdrWrDataDelay tWR_DATA_DELAY);

/**************************************************************************//**
 @Function      DDR_Config_BSTOPRE

 @Description   Advanced configuration of BSTOPRE value

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     BSTOPRE - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigBstopre(t_Handle h_Ddr, uint16_t BSTOPRE);

/**************************************************************************//**
 @Function      DDR_ConfigRwtForSameCs

 @Description   Advanced configuration of Read-to-write turnaround for same chip select.

                Configure the number of cycles are added between a read
                to write turnaround for transactions to the same chip select.
                If a value of 0000 is chosen, then the DDR controller uses the
                value used for transactions to different chip selects,

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     rwt             - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigRwtForSameCs(t_Handle h_Ddr, uint8_t rwt);

/**************************************************************************//**
 @Function      DDR_ConfigWrtForSameCs

 @Description   Advanced configuration of Write-to-read turnaround for same chip select.

                Configure the number of cycles are added between a write
                to read turnaround for transactions to the same chip select.
                If a value of 0000 is chosen, then the DDR controller uses the
                value used for transactions to different chip selects,

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     wrt             - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigWrtForSameCs(t_Handle h_Ddr, uint8_t wrt);

/**************************************************************************//**
 @Function      DDR_ConfigRrtForSameCs

 @Description   Advanced configuration of Read-to-read turnaround for same chip select.

                Configure the number of cycles are added between reads
                to the same chip select. If a value of 0000 is chosen, then 2 cycles are required
                between read commands to the same chip select if 4-beat bursts are used (4 cycles
                are required if 8-beat bursts are used).

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     rrt             - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigRrtForSameCs(t_Handle h_Ddr, uint8_t rrt);

/**************************************************************************//**
 @Function      DDR_ConfigWwtForSameCs

 @Description   Advanced configuration of Write-to-write turnaround for same chip select.

                Specifies how many cycles are added between writes
                to the same chip select. If a value of 0000 is chosen,
                then 2 cycles are required between write commands to the same chip select
                if 4-beat bursts are used (4 cycles are required if 8-beat bursts are used).

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     wwt             - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigWwtForSameCs(t_Handle h_Ddr, uint8_t wwt);

/**************************************************************************//**
 @Function      DDR_ConfigEsdmode

 @Description   Advanced configuration of Extended SDRAM mode.

                This routine allows to configure the extended SDRAM mode
                without driver's interference

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     esdmode         - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigEsdmode(t_Handle h_Ddr, uint16_t esdmode);

/**************************************************************************//**
 @Function      DDR_ConfigSdmode

 @Description   Advanced configuration of SDRAM mode.

                This routine allows to configure the SDRAM mode
                without driver's interference

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     sdmode          - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigSdmode(t_Handle h_Ddr, uint16_t sdmode);

/**************************************************************************//**
 @Function      DDR_ConfigEsdmode2

 @Description   Advanced configuration of Extended SDRAM mode 2.

                This routine allows to configure the extended SDRAM mode 2
                without driver's interference

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     esdmode2        - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigEsdmode2(t_Handle h_Ddr, uint16_t esdmode2);

/**************************************************************************//**
 @Function      DDR_ConfigEsdmode3

 @Description   Advanced configuration of Extended SDRAM mode 3.

                This routine allows to configure the extended SDRAM mode 3
                without driver's interference

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.
 @Param[in]     esdmode3        - value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
*//***************************************************************************/
t_Error DDR_ConfigEsdmode3(t_Handle h_Ddr, uint16_t esdmode3);

/**************************************************************************//**
 @Function      DDR_ConfigObcCfg

 @Description   Advanced configuration of On-The-Fly Burst Chop Configuration.

                configures if on-the-fly burst Chop is used.
                This mode should only be used if DDR3 memories are used.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().

 @Cautions      If on-the-fly Burst Chop mode is not used with DDR3
                memories, then fixed Burst Chop mode may be used if the proper turnaround
                times are programmed into TIMING_CFG_0 and TIMING_CFG_4. DDR_SDRAM_CFG[8_BE]
                should be cleared for both on-the-fly Burst Chop mode or fixed Burst Chop mode
                when using a 64-bit data bus with DDR3 memories.
*//***************************************************************************/
t_Error DDR_ConfigObcCfg(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_ConfigCntlAdj

 @Description   Advanced configuration of Control Adjust

                configure the amount of delay to add to the lightly loaded control
                signals w/ respect to all other DRAM address and command signals.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Param[in]     ctrlAdj         - Control Adjust value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ConfigCntlAdj(t_Handle h_Ddr, e_DdrCtrlAdj ctrlAdj);

/**************************************************************************//**
 @Function      DDR_ConfigApEn

 @Description   Advanced configuration of Address Parity Enable.

                Determines if address parity is generated and checked for the
                address and control signals when using registered DIMMs.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ConfigApEn(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_ConfigRcwEn

 @Description   Advanced configuration of Register Control Word Enable.

                If Register Control Word is enabled, the controller will write the
                register control words after the ddr is enabled, unless Initialization
                routine is bypassed.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().

 Cautions       If DDR3 registered DIMMs are used, it may be necessary to write the
                register control words before issuing commands to DRAM.
 *//***************************************************************************/
t_Error DDR_ConfigRcwEn(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_ConfigZqEn

 @Description   Advanced configuration of ZQ Calibration.

                This routine enables the ZQ calibration.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().

 @Cautions      This mode should only be used if DDR3 memory is used
 *//***************************************************************************/
t_Error DDR_ConfigZqEn(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_ConfigWrlvlEn

 @Description   Advanced configuration of Write Leveling.

                This routine determines if write leveling is used.
                If t Write Leveling is used, then the DDR controller performs
                write leveling immediately after initializing the DRAM.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().

 @Cautions      Write Leveling should only be used if DDR3 memory is used
 *//***************************************************************************/
t_Error DDR_ConfigWrlvlEn(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_ConfigRodtOn

 @Description   Advanced configuration of Read to ODT on.

                configutes the number of cycles that passes from when a read command
                is placed on the DRAM bus until the assertion of the relevant ODT signal(s).

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Param[in]     clocks          - number of cycles

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ConfigRodtOn(t_Handle h_Ddr, uint32_t clocks);

/**************************************************************************//**
 @Function      DDR_ConfigRodtOff

 @Description   Advanced configuration of Read to ODT off.

                configutes the number of cycles that the relevant ODT signal(s)
                remains asserted for each read transaction.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Param[in]     clocks          - number of cycles

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ConfigRodtOff(t_Handle h_Ddr, uint8_t clocks);

/**************************************************************************//**
 @Function      DDR_ConfigWodtOn

 @Description   Advanced configuration of Write to ODT On.

                configutes the number of cycles that passes from when a write command
                is placed on the DRAM bus until the assertion of the relevant ODT signal(s).

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Param[in]     clocks          - number of cycles

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ConfigWodtOn(t_Handle h_Ddr, uint32_t clocks);

/**************************************************************************//**
 @Function      DDR_ConfigWodtOff

 @Description   Advanced configuration of Write to ODT Off.

                configutes the number of cycles that the relevant ODT signal(s)
                remains asserted for each write transaction.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Param[in]     clocks          - number of cycles

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ConfigWodtOff(t_Handle h_Ddr, uint8_t clocks);

/**************************************************************************//**
 @Function      DDR_ClockEnableControl

 @Description   Advanced configuration of Clock enable control.

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Param[in]     clockCntl       - Clock enable control value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_ClockEnableControl(t_Handle h_Ddr, e_DdrClockEnableControl  clockCntl);

/**************************************************************************//**
 @Function      DDR_AddrParityErrInjectEnable

 @Description   Advanced configuration of Address parity error injection.

                This routine enables the address parity error injection

 @Param[in]     h_Ddr           - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after DDR_Config() and before DDR_Init().
 *//***************************************************************************/
t_Error DDR_AddrParityErrInjectEnable(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_Free

 @Description   Frees all memory associated with the DDR.

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_Free(t_Handle h_Ddr);


/** @} */ /* end of ddr_adv_init_grp */
/** @} */ /* end of ddr_init_grp group */

/**************************************************************************//**
 @Group         ddr_control_grp DDR Control Unit

 @Description   DDR control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DDR_ErrPolicy

 @Description   Enables/disables/allows interrupts on one or more error types
                indication (disabled errors remain disabled).

                Note that by default, all errors are ENABLED !!

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Param[in]     errType             - Indicates which error types are disabled.
                                      May be any OR combination of the following:
                                      e_DDR_MULT_BIT_ERR,
                                      e_DDR_SINGLE_BIT_ERR,
                                      e_DDR_MEM_SELECT_ERROR.

 @Param[in]     errPolicy           - Indicates the error policy
                                      May be any OR combination of the following
                                      e_ERR_DISABLE ('errType'  will not be detected),
                                      e_ERR_ENABLE_CAPTURE ('errType'  will be
                                      detected and captured (default) - no interrupt),
                                      e_ERR_ENABLE_INTERRUPT (an interrupt will
                                      be generated when 'errType' occure).

 @Param[in]     singleBitTrigger    - Set as threshold for single bit error reporting.
                                      Relevant only when enabling e_DDR_SINGLE_BIT_ERR.
                                      if 0 - remain unchanged, otherwise,

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_ErrPolicy(t_Handle          h_Ddr,
                      e_DdrErrType      errType,
                      e_DdrErrPolicy    errPolicy,
                      uint8_t           singleBitTrigger);

/**************************************************************************//**
 @Function      DDR_ErrInjEnable

 @Description   Cause the injection of errors decribed in the data and ECC mask
                bits and ECC mirror bit on writes to memory.

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_ErrInjEnable(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_ErrInjDisable

 @Description   Stops error injection.

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_ErrInjDisable(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_MirrorByteEnable

 @Description   Cause the mirroring of most significant data path byte onto the
                ECC byte.

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_MirrorByteEnable(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_MirrorByteDisable

 @Description   Stops byte mirroring.

 @Param[in]     h_Ddr - Handle to the initialized DDR object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_MirrorByteDisable(t_Handle h_Ddr);

/**************************************************************************//**
 @Function      DDR_SetErrInjectionMask

 @Description   Sets and clears error injection masks for data and ECC.

                On each call to this routine, all masks will be modified
                (i.e. no previous value may be assumed).

                A set bit in one of the masks, causes the corresponding
                data/ECC bit to be inverted on memory bus writes. To clear the
                mask, the corresponding argument must be '0'.

 @Param[in]     h_Ddr   - Handle to the initialized DDR object.
 @Param[in]     high    - 32 bits that correspond to the high word of data path.
 @Param[in]     low     - 32 bits that correspond to the low word of data path.
 @Param[in]     ecc     - 8 bits that correspond to ECC bits.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DDR_SetErrInjectionMask(t_Handle h_Ddr, uint32_t high, uint32_t low, uint8_t ecc);


/** @} */ /* end of ddr_control_grp group */
/** @} */ /* end of ddr_grp group */


#endif /* __DDR_EXT_H */

