/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          PPPoHT_ext.h

 @Description   PPPoHT Application Programming Interface.
*//***************************************************************************/
#ifndef __PPPOHT_EXT_H
#define __PPPOHT_EXT_H

#include "error_ext.h"
#include "std_ext.h"


/**************************************************************************//**
 @Group         pppoht_grp PPPoHT Application Programming Interface

 @Description   PPPoHT API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection   External defines
 
               May be used, on demand, by an application needing this
               PPPoHT fixed information
@{
*//***************************************************************************/

#define PPPOHT_FBP_SIZE                         0x10    /**< FBP parameter table size, defined for application information. */
#define PPPOHT_QD_SIZE                          0x10    /**< Queue descriptor size, defined for application information. */

#define PPPOHT_WFQ_STRICT_PRIORITY_WEIGHT       101     /**< Use this value as the WFQ weight in case that specific queue is a Strict Priority queue */
#define PPPOHT_WFQ_NOT_USED                     102     /**< Use this value as the WFQ weight in case TwoLevel WFQ is used, and the queue is configured on a different level  */

/* @} */



/**************************************************************************//**
 @Description   Exception Select.
                Used for indicating which errors the application wants reported.
                See comments, and according to LINK/BUNDLE/GENERAL, these enum (OR)
                may be passed to:
                GENERAL:    PPPoHT_ConfigExceptions
                LINK:       PPPoHT_LINK_ConfigExceptions
                BUNDLE:     PPPoHT_BUNDLE_ConfigExceptions
*//***************************************************************************/
typedef enum
{
    e_EX_NONE                   = 0x00000000
    , e_EX_INTRQ_OVERFLOW       = 0x80000000    /**< GENERAL:
                                                     Interrupt queue overflow. At least one event was lost */
    , e_EX_DEMUX_INTRQ_OVERFLOW = 0x40000000    /**< GENERAL:
                                                     Relevant only when demux enabled: : demux interrupt queue is full, at least one event was lost */
    , e_EX_MRU                  = 0x20000000    /**< LINK:
                                                     MRU violation. Occures when a frame length exceeds MRU value. */
    , e_EX_RXFBP_BSY            = 0x10000000    /**< LINK:
                                                     Buffer Pool Busy. Occures when a free buffer pool Busy
                                                     condition occurs. It means no buffer is available
                                                     for reception.*/
    , e_EX_RXFBP_THRESHOLD      = 0x08000000    /**< LINK:
                                                     Buffer Pool RLI. Set when a free buffer pool has reached
                                                     the level of the RLI threshold. */
    , e_EX_TX_UNDERRUN          = 0x04000000    /**< LINK: Tx no data. The RISC sets this flag if there is no data
                                                     available to be sent to the transmitter.*/
    , e_EX_TXFBP_BSY            = 0x02000000    /**< LINK/BUNDLE:
                                                     Tx Buffer Pool Busy. Set when a Transmitter free buffer
                                                     pool Busy condition occurs. It means no room is available
                                                     for the buffer return on transmission. */
    , e_EX_RX_BSY               = 0x01000000    /**< LINK/BUNDLE:
                                                     Busy condition. The RxBD table is busy. Packets were
                                                     discarded due to this condition.
                                                     For IW mode: This exception indicates a sub-frame which is
                                                     spread over more then four fragments which the QE is unable to
                                                     process. */
    , e_EX_MRRU                 = 0x00800000    /**< BUNDLE:
                                                     MRRU exceed.
                                                     Occures when the frame length exceeds MRRU value. */
    , e_EX_FRAG_LOSS_THRESHOLD  = 0x00400000    /**< BUNDLE:
                                                     Fragment loss
                                                     due to threshold reached. A fragment loss
                                                     was detected by the receiver. The packet was discarded. */
    , e_EX_WBD_BSY              = 0x00200000    /**< BUNDLE:
                                                     Busy condition
                                                     in WBD ring.Packets were discarded due to this condition. */
    , e_EX_PCKT_RECONSTRCTR_BSY = 0x00100000    /**< BUNDLE:
                                                     Packet Reconstruction
                                                     Table Busy condition. The Packet Reconstruction table is busy.
                                                     Packets were discarded due to this condition. */
    , e_EX_FRAG_LOSS            = 0x00080000    /**< BUNDLE:
                                                     Fragment loss occurance. There was a fragment loss
                                                     detected by the receiver.Packet wasdiscarded. */
    , e_EX_MUX_TX_ERROR         = 0x00040000    /**< LINK/BUNDLE:
                                                     Relevant only for links/bundles with a mux queue, and may occure
                                                     on MUX sub frames only: Transmit Error indication. Occures when
                                                     some error conditions which prevent this BD from being
                                                     transmitted. */
    , e_EX_MUX_AGE              = 0x00020000    /**< LINK/BUNDLE:
                                                     Relevant only for links/bundles with a mux queue: Age Discard.
                                                     Occures when a BD was not transmitted since it was too old.
                                                     Valid for MUX subframes BDs only. The discarded BD
                                                     will alse be marked on Tx confirmation. */
    , e_EX_MUX_TX_LENGTH_ERROR  = 0x00010000    /**< LINK/BUNDLE:
                                                     Relevant only for links/bundles with a mux queue: Length error.
                                                     The length of the sub frame exceeds the max allowed length of a
                                                     sub-frame. */
    , e_EX_MUX_TXFBP_BSY        = 0x00008000    /**< LINK/BUNDLE:
                                                     Relevant only for links/bundles with a mux queue:Tx Buffer Pool Busy.
                                                     Set when a Transmitter free buffer pool Busy condition occurs. It means
                                                     no room is available for the buffer return on transmission. */
    , e_EX_DEMUX_RX_LENGTH_ERROR= 0x00004000    /**< LINK/BUNDLE:
                                                     Relevant only when demux enabled:
                                                     Length Error. Indicates a Length Error condition on a sub-frame.
                                                     It happens when a sub-frame length exceeds the length
                                                     of the whole super-frame. */
    , e_EX_DEMUX_BSY            = 0x00002000    /**< LINK/BUNDLE:
                                                     Relevant only when demux enabled: De-mux Busy condition. The Demux table is busy.
                                                     Fragments/frames were discarded due to this condition.*/
    , e_EX_DEMUX_RX_BSY         = 0x00001000    /**< LINK/BUNDLE:
                                                     Relevant only when demux enabled: : Busy condition. The RxBD table associated with this
                                                     sub-frame is busy. sub-frames were discarded due to this condition. */
    , e_EX_DEMUX_RXFBP_BSY      = 0x00000800    /**< LINK/BUNDLE:
                                                     Relevant only when demux enabled and in IW mode only:In case Interworking mode is enabled the de-Mux
                                                     operation performs data copying of each sub-frame from the original
                                                     buffer to a new buffer which are taken from the FBP. This bit indicates
                                                     a situation where there are no available buffers in the FBP and the
                                                     sub-frames has to be discarded due to this event.*/
    , e_EX_IW_LENGTH_VIOLATION  = 0x00000400    /**< LINK/BUNDLE:
                                                     Relevant only for IW mode: Length Violation. In IW mode each sub-frame is copied to
                                                     a buffer which is taken from the Rx FBP. The size of the sub-frame
                                                     must fit into a single buffer from this pool. The size is specified
                                                     in the MCC global parameter page. This exception occures in case the
                                                     sub-frame exceeds this size, and the sub-frame is discarded. */
    , e_EX_IW_MAXBD3            = 0x00000200    /**< LINK/BUNDLE:
                                                     Relevant only for IW mode: MAXBD3 event. This error indicates that the frame was truncated
                                                     due to number of BDs in frame, that exceeds MAXBD3 (IW parameters).
                                                     The following data will be discarded. */
    , e_EX_IW_GEN_INVAL         = 0x00000100    /**< LINK/BUNDLE:
                                                     Relevant only for IW mode: This frame has an invalid generation value. */
    , e_EX_IW_MINFLR            = 0x00000080    /**< LINK/BUNDLE:
                                                     Relevant only for IW mode: This frame has a violated the minimal length required for
                                                     IW processing and is dropped. */
    , e_EX_IW_FRG_NUMBER        = 0x00000040    /**< LINK/BUNDLE:
                                                     Relevant only for IW mode: Received a sub-frame which is spread over more then four
                                                     fragments and the QE is unable to process it. */
} e_ExceptionsSelect;

/**************************************************************************//**
 @Description   Transmit-confirmation error flags
*//***************************************************************************/
#define TX_CONF_STATUS_UNSENT 0xCAFE

/**************************************************************************//**
 @Description   An enum defining the handshake protocol action options
                in the IW-PPPoHT communication
*//***************************************************************************/
typedef enum
{
    e_SET_IW_GLOBAL_PRAM,                   /**< SET - IW global parameter */
    e_SET_FILTER_PIDS_TABLE_SIZE,           /**< SET - PIDs to PCDs table size - Only the PID's portion of the table */
    e_SET_FILTER_TABLE_ADDR,                /**< SET - PIDs to PCDs table address */
    e_SET_TXQD_BASE,                        /**< SET - Tx QD base address */
    e_SET_COMM_DIR,                         /**< SET - Interworking Communication direction */
    e_SET_IW_TXQUEUE,                       /**< SET - Tx Queue as an interworking queue. */
    e_SET_IW_P2P_SUPPORT,                   /**< SET - Link support for P2P IW - Rx FBP=Tx FBP */
    e_GET_STRUCTS_MEM_ATTRIBUTES,           /**< GET - Bd's, external QD's and Interrupt queue memory type */
    e_GET_DATA_MEM_ATTRIBUTES,              /**< GET - Data memory type */
    e_GET_TX_QUEUES,                        /**< GET - Link TX Queues number */
    e_GET_NUM_OF_CLASSES,                   /**< GET - Bundle number of classes */
    e_GET_TXQD_IS_EXT,                      /**< GET - Queue Tx QD memory location */
    e_GET_TX_ENABLED,                       /**< GET - Tx queue state (enabled/disabled) */
    e_GET_RX_ENABLED,                       /**< GET - Rx/RxDemux queue state (enabled/disabled) */
    e_GET_APP_HANDLE,                       /**< GET - Application handle */
    e_GET_ID,                               /**< GET - Bundle Id */
    e_GET_TX_FBPS,                          /**< GET - TX FBP's handle */
    e_GET_LINK_F_RX,                        /**< GET - Link Rx routine */
    e_GET_LCP_QUEUE_ID,                     /**< GET - Tx queue used for LCP transmission. */
    e_GET_BUNDLE_F_RX,                      /**< GET - Bundle  Rx routine */
    e_GET_IS_HANDSHAKE_INIT,                /**< GET - IW handshake type - Init or Free */
    e_GET_SINGLE_CLASS_FAIRNESS_ENALBED,    /**< GET - Bundle Single Class Fairness Alg state */
    e_GET_CLASS_PART_OF_SINGLE_CLASS_WFQ    /**< GET - Bundle[Class] Single Class Fairness Alg */
} e_PppohtIwOpcode;

/**************************************************************************//**
 @Collection   Callbacks typedefs
@{
*//***************************************************************************/

/**************************************************************************//**
 @Function      t_PppohtExceptionsCallback

 @Description   Exceptions user callback routine, will be called upon an
                exception on general PPPoHT or per link/bundle, passing parameters
                describing the errors that occured.

 @Param[in]     h_AppId - User's application descriptor.
 @Param[in]     errors  - The exception.
 @Param[in]     flags   - TBD
*//***************************************************************************/
typedef void (t_PppohtExceptionsCallback)(t_Handle              h_AppId,
                                          e_ExceptionsSelect    errors,
                                          uint32_t              flags );

/**************************************************************************//**
 @Function      t_PppohtTxConfCallback

 @Description   Tx confirmation user callback routine, will be called, passing
                the frame or buffer transmitted, upon transmission of a frame per
                link/bundle.

 @Param[in]     h_AppId         - User's application descriptor.
 @Param[in]     p_Data          - The data pointer.
 @Param[in]     h_BufContext    - The buffer's context, as provided by the user.
 @Param[in]     status          - Unused.
                                  (Note: previous API's used this for error reporting.
                                   This API uses f_Exceptions for this purpose).

 @Param[in]     index   - Class (for Multilink) or Queue (for Plain PPPoHT) index.
*//***************************************************************************/
typedef void (t_PppohtTxConfCallback)(t_Handle  h_AppId,
                                      uint8_t   *p_Data,
                                      t_Handle  h_BufContext,
                                      uint16_t  status,
                                      uint8_t   index);

/**************************************************************************//**
 @Function      t_PppohtRxStoreCallback

 @Description   Receive user callback routine, will be called upon reception of
                a frame or buffer, passing the frame/buffer and other info.

 @Param[in]     h_AppId      - User's application descriptor.
 @Param[in]     p_Data       - A data pointer.
 @Param[in]     status       - Receive status and errors (relevant only on last buffer).
 @Param[in]     length       - Length of the buffer or the frame.
 @Param[in]     buffPos      - Buffer poistion - can be a single buffer, first,
                               middle or last buffer of a frame.
 @Param[in]     pid          - The PID that was received with the frame (relevant only on first buffer).
 @Param[in]     indx         - Class (for Multilink) or Queue (for Plain PPPoHT) index.
 @Param[in]     virtPtrsLeftInBuf - Indicates whether the buffer has virtual pointers in it. I.e. if the
                                    data is not located in the beginning of the buffer it is considered a
                                    virtual pointer and this parameter will reflect it.
                                    Each call back this number will reduce thus sparing the user from the need
                                    to keep track over the numbers of buffers already received.
                                    The user should use some methode of cloning in order to be able to release
                                    the virtual pointer without interfering with the other pointers.
 
 @Param[in]     h_BufContext - The buffer's context, as provided by the user.

 @Retval        e_RX_STORE_RESPONSE_CONTINUE - order the driver to continue Rx
                                               operation for all ready data.
 @Retval        e_RX_STORE_RESPONSE_PAUSE    - order the driver to stop Rx operation.
*//***************************************************************************/
typedef e_RxStoreResponse (t_PppohtRxStoreCallback)(t_Handle    h_AppId,
                                                    uint8_t     *p_Data,
                                                    uint16_t    status,
                                                    uint32_t    length,
                                                    uint8_t     buffPos,
                                                    uint16_t    pid,
                                                    uint8_t     indx,
                                                    uint8_t     virtPtrsLeftInBuf,
                                                    t_Handle    h_BufContext);

/**************************************************************************//**
 @Function      t_IwHandshakeCallback

 @Description   A routine pointer typedef, defining a handshake routine,
                passed from PPPoHT module to the IW-PPPoHT module. It will be
                called by the IW-PPPoHT registration routine to communicate (get/set)
                parameters between those 2 modules.

 @Param[in]     h_PppohtHandle  -   PPPoHT, link or bundle handle.
 @Param[in]     pppohtIwOpcode  -   An opcode defining the action needs taking.
 @Param[in]     p_Value         -  (In/Out) in GET opcodes, the address of the return
                                    parameter orstructure of parameters,
                                    in SET opcodes, a pointer to the passed
                                    parameter or structure of parameter.
*//***************************************************************************/
typedef t_Error (t_IwHandshakeCallback) (t_Handle h_PppohtHandle, e_PppohtIwOpcode pppohtIwOpcode, void *p_Value);
 
/**************************************************************************//**
 @Function      t_IwRegistrationCallback

 @Description   A routine pointer typedef, defining a registration routine called
                by the PPPoHT module to register in the IW-PPPoHT.

 @Param[in]     h_iwPppohtHandle -  the IW-PPPoHT, link or bundle handle..
 @Param[in]     f_Handshake      -  A callback routine to be called by the
                                    IW PPPoHT routine.
 @Param[in]     h_PppohtHandle   -  PPPoHT, link or bundle handle, to be passed
                                    to the handshake routine.
*//***************************************************************************/
typedef t_Error (t_IwRegistrationCallback) (t_Handle h_iwPppohtHandle,t_IwHandshakeCallback f_Handshake, t_Handle h_PppohtHandle);

/**************************************************************************//**
 @Function      t_LinkRxCallback

 @Description   A routine pointer typedef, defining a link receive routine.

 @Param[in]     h_Link -            the PPPoHT link handle..
*//***************************************************************************/
typedef t_Error (t_LinkRxCallback) (t_Handle h_Link);

/**************************************************************************//**
 @Function      t_BundleRxCallback

 @Description   A routine pointer typedef, defining a bundle receive routine.

 @Param[in]     h_Bundle -            the PPPoHT bundle handle..
*//***************************************************************************/
typedef t_Error (t_BundleRxCallback) (t_Handle h_Bundle, uint8_t clsIndex);

/* @} */


/**************************************************************************//**
 @Description   Data buffers managment parameters structure.
*//***************************************************************************/
typedef struct t_DataMemMngParam
{
    uint8_t             dataMemId;          /**< Memory partition ID for data buffers */
    uint32_t            dataMemAttributes;  /**< Memory attributes for data buffers */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffers pool information */

} t_DataMemMngParam;

/**************************************************************************//**
 @Description   A structure defining the use of free buffer pool.
*//***************************************************************************/
typedef struct t_FbpUsed
{
    t_Handle            h_Fbp;              /**< A handle of an initialized free buffer pool. */
} t_FbpUsed;

/**************************************************************************//**
 @Description   A sructure defining the data management parameters
                when free buffer pool is not used.
*//***************************************************************************/
typedef struct t_FbpNotUsed
{
    t_DataMemMngParam   dataMemMngParam;    /**< a structure of parameters defining
                                                 data management parameters. */
} t_FbpNotUsed;

/**************************************************************************//**
 @Description   A union for the selection of free buffer pool mode.
*//***************************************************************************/
typedef union
{
    t_FbpUsed           fbpParams;          /**< a Free Buffer Pool structure */
    t_FbpNotUsed        noFbpParams;        /**< an alternative data management parameters structure. */
} u_MemMngParam;


/**************************************************************************//**
 @Group         pppoht_init_grp PPPoHT Initialization Unit

 @Description   PPPoHT initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define PPPOHT_MAX_NUM_OF_MUX_QUEUES            32      /**< Number of maximum possible number of muxed queues */
#define PPPOHT_MAX_NUM_OF_PPPOHT_IW_THREADS     32      /**< Number of maximum threads that may be used for PPPoHT interworking application. */
#define PPPOHT_IW_DATA_ALIGN_MASK               0xFF    /**< Data buffer alignment resctriction in IW mode */

/**************************************************************************//**
 @Description   A structure used for expanding the return value 'void*' of the
                handshake routine
*//***************************************************************************/
typedef struct t_IwPppohtHandshakeExpander
{
    uint32_t    val;            /**< Multiple use parameter */
    void        *p_Val;         /**< Multiple use parameter */
} t_IwPppohtHandshakeExpander;

/**************************************************************************//**
 @Description   A structure for defining interrupt queue parameters.
*//***************************************************************************/
typedef struct t_IntrQueueParam
{
    uint8_t                 intrQThreshold;         /**< Interrupt threshold; After how many interrupts, the
                                                         interrupt queue will be serviced. */
    uint32_t                intrQNumOfEntries;      /**< Interrupt queue size, in terms of entries number. */
} t_IntrQueueParam;

/**************************************************************************//**
 @Description   A structure for configuring IW support for the PPPoHT module.
*//***************************************************************************/
typedef struct t_PppohtIwSupportParams
{
    t_IwRegistrationCallback    *f_Register;        /**< A callback to the registration routine of
                                                         the IW PPPoHT module. */
    t_Handle                    h_IwPppoht;         /**< A handle of the IW PPPoHT module. */
    uint8_t                     totalNumOfIwThreads;/**< number of threads for the PPPoHT interworking */
    uint16_t                    maxSubFrmSize;      /**< Should be defined if at list one of the links/bundles in
                                                         the system has an enabled De-mux. This value is the smallest
                                                         buffer size among all demux FBP's */
} t_PppohtIwSupportParams;

/**************************************************************************//**
 @Description   Main PPPoHT parameters structure,
                used for initializing the PPPoHT module.
*//***************************************************************************/
typedef struct t_PppohtParam
{
    /* PPPoHT basic parameters */
    bool                        muxTxSupport;   /**< TRUE to support muxed transmission, FALSE otherwize. */
    bool                        demuxRxSupport; /**< TRUE to support demuxed reception, FALSE otherwize. */
    bool                        MLMCTxSupport;  /**< TRUE to support multilinked transmission, FALSE otherwize. */
    bool                        MLMCRxSupport;  /**< TRUE to support multilinked reception, FALSE otherwize. */

    /* System integration parameters */
    t_Handle                    h_Qe;           /**< An initialized QE module handle. */
    t_Handle                    h_Mcc;          /**< An initialized MCC module handle. */
    uint8_t                     pramMemId;      /**< Parameter RAM partition ID */
 
    /* Application interface */
    t_Handle                    h_AppId;        /**< User's application descriptor. */
    t_PppohtExceptionsCallback  *f_Exceptions;  /**< A user callback routine for handling PPPoHT exceptions. */
} t_PppohtParam;


/**************************************************************************//**
 @Function      PPPoHT_Config

 @Description   Configures a PPPoHT device.
                This routine allocates the PPPoHT structure and sets user
                and default parameters in the PPPoHT driver parameters structure.

 @Param[in]     p_PppohtParam - A pointer to the initializing parameter
                                structure for the PPPoHT.

 @Return        A handle (descriptor) of the newly created PPPoHT structure.
*//***************************************************************************/
t_Handle PPPoHT_Config(t_PppohtParam *p_PppohtParam);

/**************************************************************************//**
 @Function      PPPoHT_Init

 @Description   Initializes a PPPoHT device.
                This routine initializes the t_Pppoht structure, the global
                parameters at the parameter ram and the memory mapped
                PPPoHT registers.

 @Param[in]     h_Pppoht - A handle to a PPPoHT device.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_Init(t_Handle h_Pppoht);

/**************************************************************************//**
 @Function      PPPoHT_Free

 @Description   Frees all PPPoHT device resources.

 @Param[in]     h_Pppoht - A handle to a PPPoHT device.
*//***************************************************************************/
void     PPPoHT_Free(t_Handle h_Pppoht);


/**************************************************************************//**
 @Group         pppoht_advanced_init_grp PPPoHT Advanced Configuration Unit

 @Description   PPPoHT Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PPPoHT_ConfigAdaptiveSqn

 @Description   When enabled, this feature identify gaps in reveived fragments
                sequence numbers for bundles [FALSE].

 @Param[in]     h_Pppoht         - A handle to a PPPoHT device.
 @Param[in]     enable           - Enable of disable this feature.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      When using this mode it is recomended to set
                PPPoHT_ConfigNumOfRxPcktReconstructionTblEntries for the value of:
                (TotalNumberOfClasses * 2) + 1 .
*//***************************************************************************/
t_Error PPPoHT_ConfigAdaptiveSqn(t_Handle h_Pppoht, bool  enable);

/**************************************************************************//**
 @Function      PPPoHT_ConfigIwSupport

 @Description   Enable IW support on the PPPoHT.
                Further IW mode selection will be made per plain link or bundle.

 @Param[in]     h_Pppoht                - A handle to a PPPoHT device.
 @Param[in]     p_PppohtIwSupportParams - A structure of interworking parameters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigIwSupport(t_Handle h_Pppoht, t_PppohtIwSupportParams *p_PppohtIwSupportParams);

/**************************************************************************//**
 @Function      PPPoHT_ConfigDemuxIw

 @Description   Defines the demux module of the PPPoHT to work in interwarking mode.
                There is only one demux per PPPoHT and it may work in either IW or non-IW mode.
                All links/bundles using the demux must comply with the selected mode.
 
 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigDemuxIw(t_Handle h_Pppoht);

/**************************************************************************//**
 @Function      PPPoHT_ConfigExceptions

 @Description   Selects the exceptions that will be reported. Upon an occurance of one
                of the selected events, f_Exception will be called, passing the bundle
                handle and the event.
                Calling this routine changes the internal driver data base
                from its default configuration - all errors and irregular
                behaviour is reported.

 @Param[in]     h_Pppoht                    - A handle to a PPPoHT device.
 @Param[in]     exceptionsSelect            - An OR value of all exceptions that the user
                                              whould like to have reported

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_ConfigExceptions(t_Handle h_Pppoht, e_ExceptionsSelect exceptionsSelect);

/**************************************************************************//**
 @Function      PPPoHT_ConfigIwIphcExtendedThreadPage

 @Description   Defines if the driver should use an addtional 128 bytes for ThreadPage,
                these bytes being needed by IPHC.

 @Param[in]     h_Pppoht                    - A handle to a PPPoHT device.
 @Param[in]     iwIphcExtendedThreadPage    - A boolean value to indicate if the driver
                                              should use an aditional 128 bytes for the
                                              thread page needed for IPHC
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigIwIphcExtendedThreadPage(t_Handle h_Pppoht,
                                              bool     iwIphcExtendedThreadPage);


/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfRxPcktReconstructionTblEntries

 @Description   Define number of entries in the Receive-Packet-Reconstruction-Table.
                Calling this routine changes the internal driver data base
                from its default configuration [16].

 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.
 @Param[in]     numOfEntries - Number of entries.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfRxPcktReconstructionTblEntries(t_Handle h_Pppoht, uint16_t numOfEntries);

/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfIntrQEntries

 @Description   Define the number of entries in the interrupt table.
                Calling this routine changes the internal driver data base
                from its default configuration [64].

 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.
 @Param[in]     numOfEntries - Number of entries.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfIntrQEntries(t_Handle h_Pppoht, uint16_t numOfEntries);

/**************************************************************************//**
 @Function      PPPoHT_ConfigIntrThreshold

 @Description   Define interrupt threshold, after this number of occurances
                interrupts will be serviced.
                Calling this routine changes the internal driver data base
                from its default configuration [1].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     intrThreshold   - Interrupt threshold.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigIntrThreshold(t_Handle h_Pppoht, uint8_t intrThreshold);

/**************************************************************************//**
 @Function      PPPoHT_ConfigIntrQueueParameters

 @Description   Define interrupt queue parameters.
                Calling this routine changes the internal driver data base
                from its default configuration [64, 1].

 @Param[in]     h_Pppoht          - A handle to a PPPoHT device.
 @Param[in]     p_IntrQueueParam  - Interrupt queue parameters structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigIntrQueueParameters(t_Handle h_Pppoht, t_IntrQueueParam *p_IntrQueueParam);

/**************************************************************************//**
 @Function      PPPoHT_ConfigRxLcpBdRingSize

 @Description   Define the number of BD's in the Rx LCP queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.
 @Param[in]     bdRingSize   - Number of BD's in the Rx LCP queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigRxLcpBdRingSize(t_Handle h_Pppoht, uint16_t bdRingSize);

/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfMuxParameterTblEntries

 @Description   Define the number of entries in the Mux-parameter-Table.
                Calling this routine changes the internal driver data base
                from its default configuration [PPPOHT_MAX_NUM_OF_MUX_QUEUES].

 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.
 @Param[in]     numOfEntries - Number of entries.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfMuxParameterTblEntries(t_Handle h_Pppoht, uint8_t numOfEntries);

/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfMuxThreads

 @Description   Define the number of threads which will be used for the muxing proccess.
                The driver will take care of the balancing automaticaly.
                Calling this routine changes the internal driver data base
                from its default configuration [1].

 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.
 @Param[in]     numOfThreads - Number of threads.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfMuxThreads(t_Handle h_Pppoht, uint8_t numOfThreads);

/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfMuxIntrQEntries

 @Description   Define the number of entries in the Mux interrupt table.
                Calling this routine changes the internal driver data base
                from its default configuration [8].

 @Param[in]     h_Pppoht     - A handle to a PPPoHT device.
 @Param[in]     numOfEntries - Number of entries.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfMuxIntrQEntries(t_Handle h_Pppoht, uint16_t numOfEntries);

/**************************************************************************//**
 @Function      PPPoHT_ConfigMuxIntrThreshold

 @Description   Define interrupt threshold for the mux, after this number of occurances
                interrupts will be serviced.
                Calling this routine changes the internal driver data base
                from its default configuration [1].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     intrThreshold   - Interrupt threshold.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigMuxIntrThreshold(t_Handle h_Pppoht, uint8_t intrThreshold);

/**************************************************************************//**
 @Function      PPPoHT_ConfigMuxIntrQueueMemory

 @Description   Defines the memory type for the mux interrupt queue.
 
                Calling this routine changes the internalPPPoHT_ext.h driver data base
                from its default configuration.
                [0 (default heap), MEMORY_ATTR_CACHEABLE].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigMuxIntrQueueMemory(t_Handle h_Pppoht, uint8_t memId, uint32_t memAttributes);

/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfDemuxFragmentTblEntries

 @Description   Define the number of entries in the demux fragment table.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     numOfEntries    - Number of entries.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfDemuxFragmentTblEntries(t_Handle h_Pppoht, uint16_t numOfEntries);

/**************************************************************************//**
 @Function      PPPoHT_ConfigNumOfDemuxIntrQEntries

 @Description   Define the number of entries in the demux interrupt queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     numOfEntries    - Number of entries.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigNumOfDemuxIntrQEntries(t_Handle h_Pppoht, uint16_t numOfEntries);

/**************************************************************************//**
 @Function      PPPoHT_ConfigDemuxIntrThreshold

 @Description   Define the interrupt threshold for the demux, after this number of occurances
                interrupts will be serviced.
                Calling this routine changes the internal driver data base
                from its default configuration [1].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     intrThreshold   - Interrupt threshold.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigDemuxIntrThreshold(t_Handle h_Pppoht, uint8_t intrThreshold);

/**************************************************************************//**
 @Function      PPPoHT_ConfigDemuxIntrQueueAndDftMemory

 @Description   Define the memory type for the mux interrupt queue.
                Calling this routine changes the internal driver data base
                from its default configuration
                [0 (default heap), MEMORY_ATTR_CACHEABLE].

 @Param[in]     h_Pppoht        - A handle to a PPPoHT device.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigDemuxIntrQueueAndDftMemory(t_Handle h_Pppoht, uint8_t memId, uint32_t memAttributes);

/**************************************************************************//**
 @Function      PPPoHT_ConfigDisableTxConf

 @Description   Disables Tx confirmation on transmitted frames/buffers.
                By default, the application should initialize a transmit
                confirmation callback (f_TxConf) which will be called for each
                transmitted frame/buffer befor this pointer may be reused/released.
                For performance reasons, the application may choose to estimate the
                time a transmitted frame/buffer had been transmitted, and avoid the
                driver calls to the callback routine.

 @Param[in]     h_Pppoht        -             A handle to a PPPoHT device.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_ConfigDisableTxConf(t_Handle h_Pppoht);

/** @} */ /* end of pppoht_advanced_init_grp group */
/** @} */ /* end of pppoht_init_grp group */


/**************************************************************************//**
 @Group         pppoht_control_grp PPPoHT Control Unit

 @Description   PPPoHT control unit API functions, definitions and enums.

 @{
*//***************************************************************************/


#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      PPPoHT_DumpRegs

 @Description   Prints

 @Param[in]     h_Pppoht - A handle to a PPPoHT device.
*//***************************************************************************/
void    PPPoHT_DumpRegs(t_Handle h_Pppoht);
#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of pppoht_control_grp group */


/**************************************************************************//**
 @Group         pppoht_fbp_grp PPPoHT FBP Application Programming Interface

 @Description   PPPoHT Free Buffer Pool API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         pppoht_txfbps_grp PPPoHT Tx FBPs Application Programming Interface

 @Description   PPPoHT Transmit Free Buffer Pools API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         pppoht_txfbps_init_grp PPPoHT Tx FBPs Initialization Unit

 @Description   PPPoHT Tx FBPs initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define PPPOHT_MAX_NUM_OF_TX_FBPS               4   /**< Number of maximum transmit buffer pools per link/bundle */

/**************************************************************************//**
 @Description   A structure for configuring a set of Tx FBP's.
*//***************************************************************************/
typedef struct t_PppohtTxFbpsParam
{
    uint8_t     numOfPools; /**< Number of FBP's */
    uint8_t     pramMemId;  /**< Parameter RAM partition ID */
    t_Handle    h_AppId;    /**< Application descriptor */
} t_PppohtTxFbpsParam;


/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_Config

 @Description   Configures a set of free buffer pools.
                This routine allocates the FBP structure and sets user
                and default parameters in the FBP driver parameters structure.

 @Param[in]     p_TxFbpsParam - A pointer to the initializing parameter
                                structure for the FBP.

 @Return        A handle (descriptor) of the newly created FBP structure.
*//***************************************************************************/
t_Handle PPPoHT_TxFBPs_Config(t_PppohtTxFbpsParam *p_TxFbpsParam);

/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_Init

 @Description   Initializes a set of FBP's.
                This routine allocates and initializes the t_Fbps structure
                and the required number of free buffer pools according to default
                values and user's requests.

 @Param[in]     h_TxFbps - A handle to the FBPs structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_TxFBPs_Init(t_Handle h_TxFbps);

/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_Free

 @Description   Frees all FBPs resources.

 @Param[in]     h_TxFbps - A handle to the FBPs structure.
*//***************************************************************************/
t_Error  PPPoHT_TxFBPs_Free(t_Handle h_TxFbps);


/**************************************************************************//**
 @Group         pppoht_txfbps_advanced_init_grp PPPoHT Tx FBPs Advanced Configuration Unit

 @Description   PPPoHT Tx FBPs Configuration functions used to change default values.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_ConfigIw

 @Description   This routine should be called if the Free Buffer Pool is used for
                Interworking (Tx side).

 @Param[in]     h_TxFbps            - A handle to the FBPs structure.
 @Param[in]     iwFbpIndexes        - An array of booleans declaring for each FBP in
                                      the FBP set, whether it is an IW pool.
 @Param[in]     p_DataMemMngParam   - A pointer to data management parameters.

 @Return        E_OK on success; Error code otherwise.
 
 
*//***************************************************************************/
t_Error PPPoHT_TxFBPs_ConfigIw(t_Handle  h_TxFbps, bool iwFbpIndexes[], t_DataMemMngParam *p_DataMemMngParam);

/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_ConfigNumOfBuffers

 @Description   Define the number of buffers in the selected FBP.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_TxFbps        - A handle to the FBPs structure.
 @Param[in]     i               - FBP index.
 @Param[in]     numOfBuffers    - Number of buffers in the FBP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_TxFBPs_ConfigNumOfBuffers(t_Handle h_TxFbps, int i, uint32_t numOfBuffers);

/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for all FBP in the set.
                Calling this routine changes the internal driver data base
                from its default configuration
                [0 (default heap), MEMORY_ATTR_CACHEABLE].

 @Param[in]     h_TxFbps        - A handle to the FBPs structure.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_TxFBPs_ConfigFirmwareExternalStructsMemory(t_Handle h_TxFbps,
                                                           uint8_t  memId,
                                                           uint32_t memAttributes);

/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_ConfigDataMemory

 @Description   Configures memory partition and attributes for all FBP in the set.
                Calling this routine changes the internal driver data base
                from its default configuration
                [0 (default heap), MEMORY_ATTR_CACHEABLE].
                NOTE: MUST be called prior to PPPoHT_TxFBPs_ConfigIw - if needed.

 @Param[in]     h_TxFbps        - A handle to the FBPs structure.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_TxFBPs_ConfigDataMemory(t_Handle    h_TxFbps,
                                        uint8_t     memId,
                                        uint32_t    memAttributes);

/** @} */ /* end of pppoht_txfbps_advanced_init_grp group */
/** @} */ /* end of pppoht_txfbps_init_grp group */


/**************************************************************************//**
 @Group         pppoht_txfbps_control_grp PPPoHT TxFBPs Control Unit

 @Description   PPPoHT Transmit Free Buffer Pools control unit API functions,
                definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_GetFbpsBaseAddr

 @Description   Returns the parameter table base address of the first FBP in the group.

 @Param[in]     h_TxFbps - A handle to the FBPs structure.

 @Return        Parameter table base address
*//***************************************************************************/
uint32_t PPPoHT_TxFBPs_GetFbpsBaseAddr(t_Handle h_TxFbps);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      PPPoHT_TxFBPs_DumpRegs

 @Description   Prints

 @Param[in]     h_TxFbps - A handle to the FBPs structure.
*//***************************************************************************/
void     PPPoHT_TxFBPs_DumpRegs(t_Handle h_TxFbps);
#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of pppoht_txfbps_control_grp group */
/** @} */ /* end of pppoht_txfbps_grp group */


/**************************************************************************//**
 @Group         pppoht_rxfbps_grp PPPoHT Rx FBPs Application Programming Interface

 @Description   PPPoHT Receive Free Buffer Pools API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         pppoht_rxfbps_init_grp PPPoHT Rx FBPs Initialization Unit

 @Description   PPPoHT Rx FBPs initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   A structure for configuring Rx FBP.
*//***************************************************************************/
typedef struct t_PppohtRxFbpParam
{
    t_DataMemMngParam   dataMemMngParam;    /**< Data memory management parameters */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID */
    t_Handle            h_AppId;            /**< Application descriptor */
} t_PppohtRxFbpParam;


/**************************************************************************//**
 @Function      PPPoHT_RxFBP_Config

 @Description   Configures a free buffer pool.
                This routine allocates the FBP structure and sets user
                and default parameters in the FBP driver parameters structure.

 @Param[in]     p_RxFbpParam - A pointer to the initializing parameter
                               structure for the FBP.

 @Return        A handle (descriptor) of the newly created FBP structure.
*//***************************************************************************/
t_Handle PPPoHT_RxFBP_Config(t_PppohtRxFbpParam *p_RxFbpParam);

/**************************************************************************//**
 @Function      PPPoHT_RxFBP_Init

 @Description   Initializes a FBP.
                This routine allocates and initializes the FBP structure
                according to default values and user's requests.

 @Param[in]     h_RxFbp - A handle to the FBP structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_RxFBP_Init(t_Handle h_RxFbp);

/**************************************************************************//**
 @Function      PPPoHT_RxFBP_Free

 @Description   Frees all Rx FBPs resources.

 @Param[in]     h_RxFbp - A handle to the FBP structure.
*//***************************************************************************/
t_Error  PPPoHT_RxFBP_Free(t_Handle h_RxFbp);


/**************************************************************************//**
 @Group         pppoht_rxfbps_advanced_init_grp PPPoHT Rx FBPs Advanced Configuration Unit

 @Description   PPPoHT Rx FBPs Configuration functions used to change default values.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      PPPoHT_RxFBP_ConfigFbpThreshold

 @Description   Define the allowed min number of available buffers in FBP.
                When the FBP reaches this number, an interrupt is issued.
                Calling this routine changes the internal driver data base
                from its default configuration [32/2].
 
                Note: fbpThreshold should be no more than half the FBP size

 @Param[in]     h_RxFbp             - A handle to the FBP structure.
 @Param[in]     fbpThreshold        - When the number of available buffers in the
                                      FBP reaches this threshold an interupt will
                                      be issued and driver will fill the FBP if
                                      possible and needed.
 @Param[in]     numOfBuffers        - This is the amount of buffers the driver will
                                      fill each time.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Threshold must be less than half number of buffers in this FBP.
 @Cautions      numOfBuffers must be less than the Threshold.
 @Cautions      When the FBP is in IW mode the FBP will not be filled.
*//***************************************************************************/
t_Error  PPPoHT_RxFBP_ConfigFbpThreshold(t_Handle h_RxFbp, uint16_t fbpThreshold, uint16_t numOfBuffers);

/**************************************************************************//**
 @Function      PPPoHT_RxFBP_ConfigNumOfBuffers

 @Description   Define the number of buffers in the selected FBP.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_RxFbp             - A handle to the FBP structure.
 @Param[in]     numOfBuffers        - Number of buffers in the FBP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_RxFBP_ConfigNumOfBuffers(t_Handle h_RxFbp, uint32_t numOfBuffers);

/**************************************************************************//**
 @Function      PPPoHT_RxFBP_ConfigFirmwareExternalStructsMemory

 @Description   Define the BD's memory type for the FBP.
                Calling this routine changes the internal driver data base
                from its default configuration
                [0 (default heap), MEMORY_ATTR_CACHEABLE].

 @Param[in]     h_RxFbp         - A handle to the FBPs structure.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_RxFBP_ConfigFirmwareExternalStructsMemory(t_Handle  h_RxFbp,
                                                          uint8_t   memId,
                                                          uint32_t  memAttributes);

/**************************************************************************//**
 @Function      PPPoHT_RxFBP_ConfigIw

 @Description   This routine should be called if the Free Buffer Pool is used for
                Interworking (Rx side).

 @Param[in]     h_RxFbp             - A handle to the FBPs structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_RxFBP_ConfigIw(t_Handle  h_RxFbp);

/** @} */ /* end of pppoht_rxfbp_advanced_init_grp group */
/** @} */ /* end of pppoht_rxfbp_init_grp group */


/**************************************************************************//**
 @Group         pppoht_rxfbp_control_grp PPPoHT RxFBP Control Unit

 @Description   PPPoHT Receive Free Buffer Pool control unit API functions,
                definitions and enums.

 @{
*//***************************************************************************/

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      PPPoHT_RxFBP_DumpRegs

 @Description   Prints

 @Param[in]     h_RxFbp - A handle to the FBP structure.
*//***************************************************************************/
void     PPPoHT_RxFBP_DumpRegs(t_Handle h_RxFbp);
#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of pppoht_rxfbp_control_grp group */
/** @} */ /* end of pppoht_rxfbp_grp group */
/** @} */ /* end of pppoht_fbp_grp group */


/**************************************************************************//**
 @Group         pppoht_bundle_grp PPPoHT Bundle Application Programming Interface

 @Description   PPPoHT Bundle API functions, definitions and enums

 @{
*//***************************************************************************/

#define PPPOHT_MAX_NUM_OF_BUNDLES              8        /**< Total maximum bundles in PPPoHT */
#define PPPOHT_MAX_NUM_OF_CLASSES_PER_BUNDLE   8        /**< Maximum number of classes per bundle in ML/MC mode */
#define PPPOHT_MAX_NUM_OF_LINKS_PER_BUNDLE     16       /**< Maximum number of links per bundle in Multilink mode */


/**************************************************************************//**
 @Group         pppoht_bundle_init_grp PPPoHT Bundle Initialization Unit

 @Description   PPPoHT Bundle initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   A structure for configuring each class for ML/MC PPPoHT.
*//***************************************************************************/
typedef struct t_ClassParam
{
    bool                    txQueueEn;              /**< Enable Tx queue */
    uint8_t                 txFreeBufferPoolIndex;  /**< Tx FBP index [0-3] - USED ONLY IF Tx queue enabled. */
    bool                    rxQueueEn;              /**< Enable Rx queue */
    bool                    rxDemuxQueueEn;         /**< Enable demux queue */
} t_ClassParam;

/**************************************************************************//**
 @Description   A structure for configuring bundle parameters
*//***************************************************************************/
typedef struct t_PppohtBundleParam
{
    t_Handle                    h_Pppoht;               /**< A handle to the PPPoHT module */
    t_Handle                    h_AppId;                /**< A handle to the user's module */
    t_DataMemMngParam           lcpDataMemMngParam;     /**< A data management parameters
                                                             structure for LCP management. */
    bool                        useRxFbp;               /**< TRUE to use Rx FBP, false to work in swap mode,
                                                             effects the processing of the following union. */
    u_MemMngParam               rxMemMng;               /**< Rx memory management union, supports paremeters
                                                             for working with or without FBP; FBP mode is
                                                             selected by the previous parameter */
    t_Handle                    h_TxFbps;               /**< Tx FBPs - Must be initialized if there is any Tx on
                                                             any class */
    uint8_t                     fwExtStructsMemId;      /**< Memory partition ID for external firmware structures -
                                                             USED ONLY IF NO FBPs ARE DEFINED (Rx and Tx). */
    uint32_t                    fwExtStructsMemAttr;    /**< Memory attributes for external firmware structures -
                                                             USED ONLY IF NO FBPs ARE DEFINED (Rx and Tx). */
    uint16_t                    mrru;                   /**< Maximum reconstructed receive frame size. */
    uint16_t                    maxFragSize;            /**< Maximum fragement size. */
    uint8_t                     numOfClasses;           /**< Number of classes under this bundle. */
    uint16_t                    muxDefPid;              /**< Default pid - relevant only for demux. */
 
    t_PppohtRxStoreCallback     *f_RxStore;             /**< A pointer to the upper layer's callback routine -
                                                             called to handle a received frame.*/
    t_PppohtTxConfCallback      *f_TxConf;              /**< A pointer to the upper layer's callback routine -
                                                             called to handle a transmitted frame.*/
    t_PppohtExceptionsCallback  *f_Exceptions;          /**< A user callback routine for handling PPPoHT bundle exceptions. */
    t_ClassParam                classParam[PPPOHT_MAX_NUM_OF_CLASSES_PER_BUNDLE]; /**< An array of parameters
                                                             structures for each class, according to numOfClasses */
} t_PppohtBundleParam;


/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_Config

 @Description   Configures a new bundle.
                This routine allocates the bundle structure and sets user
                and default parameters in the bundle driver parameters structure.

 @Param[in]     p_BundleParam - A pointer to the initializing parameters
                                structure for the bundle.

 @Return        A handle (descriptor) of the newly created bundle structure.
*//***************************************************************************/
t_Handle PPPoHT_BUNDLE_Config(t_PppohtBundleParam *p_BundleParam);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_Init

 @Description   Initializes a bundle module.
                This routine initializes the bundle structur and the global
                parameters at the parameter ram.

 @Param[in]     h_Bundle - A handle to a bundle module.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_Init(t_Handle h_Bundle);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_Free

 @Description   Frees all Rx bundle resources.

 @Param[in]     h_Bundle - A handle to the bundle module.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_Free(t_Handle h_Bundle);


/**************************************************************************//**
 @Group         pppoht_bundle_advanced_init_grp PPPoHT Bundle Advanced Configuration Unit

 @Description   PPPoHT Bundle Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   A structure for configuring IW on a specific bundle.
*//***************************************************************************/
typedef struct t_BundleIwParams
{
    t_IwRegistrationCallback    *f_Register;        /**< A callback to the registration
                                                         routine of the IW PPPoHT bundle module. */
    t_Handle                    h_IwPppohtBundle;   /**< A handle of the IW PPPoHT bundle module. */
    uint8_t                     classNumOfThreads[PPPOHT_MAX_NUM_OF_CLASSES_PER_BUNDLE];
                                                    /**< number of threads for each class in the bundle */
} t_BundleIwParams;

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigIw

 @Description   Configures interworking mode for this bundle. This configuration effects all
                links and classes connected to this bundle.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     p_BundleIwParams            - A pointer to a structure of interworking parameters..

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigIw(t_Handle h_Bundle, t_BundleIwParams *p_BundleIwParams);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigExceptions

 @Description   Selects the exceptions that will be reported. Upon an occurance of one
                of the selected events, f_Exception will be called, passing the bundle
                handle and the event.
                Calling this routine changes the internal driver data base
                from its default configuration - all errors and irregular
                behaviour is reported.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     exceptionsSelect            - An OR value of all exceptions that the user
                                              whould like to have reported

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigExceptions(t_Handle h_Bundle, e_ExceptionsSelect exceptionsSelect);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigEnMLPFC


 @Description   May be called by application to compress ML header PID on
                transmission (0x3D instead of 0x003D).
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigEnMLPFC(t_Handle h_Bundle);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigEnACFC


 @Description   May be called by application to support address and control
                character compressed on reception.
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      This routine effects all the Links associated with this Bundle.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigEnACFC(t_Handle h_Bundle);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigEnPFC


 @Description   May be called by application to support PID field compressed on
                transmission and reception (1 byte instead of 2 bytes).
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      This routine effects all the Links associated with this Bundle.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigEnPFC(t_Handle h_Bundle);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigExtendedSqnMode

 @Description   May be called by application to configure extended sequence number
                mode. By default, this mode is set only when the number of
                classes is larger than 4. In this mode, sequence number
                is 24 bits, and CLS id is 4, instead of 12 bits and 2 for CLS.
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigExtendedSqnMode(t_Handle h_Bundle);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassesFairnessAlg

 @Description   Configures fairness algorithm between the classes. The user should
                allocate an array in which each entry (one per class) contains a
                value which represents the precentage of this class in relation to
                the other classes. The user should make sure that the sum of all
                entries is 100.
                Calling this routine changes the internal driver data base
                from its default configuration - equal weights.
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     wfq             - An array of relative weight for each class,
                                  in precentage.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassesFairnessAlg(t_Handle h_Bundle, uint8_t *wfq);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigSingleClassFairnessAlg

 @Description   Enables fairness algorithm for single-class systems.
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     numOfTxQueues   - Number of Tx Queues.
 @Param[in]     wfq             - An array of relative weight for each queue,
                                  in precentage.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_BUNDLE_ConfigSingleClassFairnessAlg(t_Handle h_Bundle, uint8_t numOfTxQueues, uint8_t *wfq);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigTwoLevelsClassesFairnessAlg

 @Description   Enables spreading the classes ( TxQueues in case SingleClassFairnessAlg is used ) in two
                priority groups.
                Each priority group may contain three types of information:
                - strict-priority queues (queues that are scheduled for transmission as soon as they have data)
                - weighted queues (queues that are scheduled for transmission based on their priority related to the same group they are part of)
                - queues marked as NOT USED IN THIS GROUP, that must be used in the second group
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     highPriorityWfq - An array of relative weight for each queue in the high priority group,
                                  in precentage.
 @Param[in]     lowPriorityWfq  - An array of relative weight for each queue in the low priority group,
                                  in precentage.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_BUNDLE_ConfigTwoLevelsClassesFairnessAlg(t_Handle h_Bundle, uint8_t *highPriorityWfq, uint8_t * lowPriorityWfq);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigMaxNumOfLinks

 @Description   Define the number of possible total links in this bundle.
                Calling this routine changes the internal driver data base
                from its default configuration [PPPOHT_MAX_NUM_OF_LINKS_PER_BUNDLE].

 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     maxNumOfLinks   - Number of possible total links in this bundle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigMaxNumOfLinks(t_Handle h_Bundle, uint16_t maxNumOfLinks);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigMaxNumOfBdsInFragment

 @Description   Define the number of BD's that can accomodate one fragement (may not exceed 127).
                Calling this routine changes the internal driver data base
                from its default configuration [127].

 @Param[in]     h_Bundle                - A handle to the bundle module.
 @Param[in]     maxNumOfBdsInFragment   - Number of BD's that can accomodate one
                                          fragement (may not exceed 127).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigMaxNumOfBdsInFragment(t_Handle h_Bundle, uint16_t maxNumOfBdsInFragment);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassRxSqNThreshold

 @Description   Define the Rx sequence number threshold, for fragement loss detection.
                Calling this routine changes the internal driver data base
                from its default configuration [64].

 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     clsIndex        - Class on which fuction operates.
 @Param[in]     rxSqNThreshold  - Rx sequence number threshold.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassRxSqNThreshold(t_Handle h_Bundle, uint8_t clsIndex, uint16_t rxSqNThreshold);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassNumOfFragmentsStoredInWbd

 @Description   Define the maximum number of fragments stored in the WBD, used for allocating WBD status ring.
                Calling this routine changes the internal driver data base
                from its default configuration [64].

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     numOfFragmentsStoredInWbd   - Number of fragments stored in the WBD.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassNumOfFragmentsStoredInWbd(t_Handle h_Bundle, uint8_t clsIndex,  uint16_t numOfFragmentsStoredInWbd);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassRxBdRingSize

 @Description   Define the number of BD's in the class Rx queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     bdRingSize                  - Number of BD's in the class Rx queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassRxBdRingSize(t_Handle h_Bundle, uint8_t clsIndex, uint16_t bdRingSize);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassRxDemuxBdRingSize

 @Description   Define the number of BD's in the class Rx demux queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     bdRingSize                  - Number of BD's in the class Rx demux queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassRxDemuxBdRingSize(t_Handle h_Bundle, uint8_t clsIndex, uint16_t bdRingSize);


/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassNullFrag

 @Description   TBD

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     burstPeriod                 - TBD.
 @Param[in]     burstSize                   - TBD.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_BUNDLE_ConfigClassNullFrag(t_Handle h_Bundle, uint8_t clsIndex, uint32_t burstPeriod, uint8_t burstSize);


/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassTxBdRingSize

 @Description   Define the number of BD's in the class Tx queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     bdRingSize                  - Number of BD's in the class Tx queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassTxBdRingSize(t_Handle h_Bundle, uint8_t clsIndex, uint16_t bdRingSize);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassTxMux

 @Description   Configures class to work in mux mode.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     superFrameSize              - MUX super frame size; Defines the maximum transmitted size of a
                                              PPPoHT MUX superframe; The size specified in this field does not include
                                              the PPPoHT MUX PID which could be of one or two bytes depending on the
                                              mode in which the bundle is programmed;

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassTxMux(t_Handle h_Bundle, uint8_t clsIndex, uint16_t superFrameSize);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassTxMuxMaxSubfSize

 @Description   Enables maximal sub-frame size for the specified class. This means that any frame larger than this value
                will be sent unmuxed while the previous super frame being built will be closed and sent prior to that.
                If this function is not called, by default, the class will mux every sub frame.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     maxSubfSize                 - Max sub frame size vto be muxed.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      When setting this feature, user must make sure that the class on the other side can receive both muxed
                and non muxed data.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassTxMuxMaxSubfSize(t_Handle h_Bundle, uint8_t clsIndex, uint16_t maxSubfSize);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassTxMuxAgeAlgEn

 @Description   Enables aging algorithm for the specified class.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     agePeriod                   - Aging period in nano seconds.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassTxMuxAgeAlgEn(t_Handle h_Bundle, uint8_t clsIndex, uint32_t agePeriod);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassRxIntrQ

 @Description   Selects the Rx interrupt queue to be used.
                By default, the driver selects the next available and suitable interrupt
                queue. This routine should be called by applications that require
                an explicit selection of an interrupt queue. The routine will fail
                if this queue is already used by bundles that have conflicting
                parameters.
 
 @Param[in]     h_Bundle                - A handle to the bundle module.
 @Param[in]     clsIndex                - Class on which fuction operates.
 @Param[in]     intrQNum                - A legal Rx interrupt queue number
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PPPoHT_BUNDLE_ConfigClassRxIntrQ(t_Handle h_Bundle, uint8_t clsIndex, uint8_t intrQNum);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassTxIntrQ

 @Description   Selects the Tx interrupt queue to be used.
                By default, the driver selects the next available and suitable interrupt
                queue. This routine should be called by applications that require
                an explicit selection of an interrupt queue. The routine will fail
                if this queue is already used by bundles that have conflicting
                parameters.
 
 @Param[in]     h_Bundle                - A handle to the bundle module.
 @Param[in]     clsIndex                - Class on which fuction operates.
 @Param[in]     intrQNum                - A legal Tx interrupt queue number
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PPPoHT_BUNDLE_ConfigClassTxIntrQ(t_Handle h_Bundle, uint8_t clsIndex, uint8_t intrQNum);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigTxQueueMuxDisableSubFrmPFC

 @Description   Disables sub-frame PID compression.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigTxQueueMuxDisableSubFrmPFC(t_Handle h_Bundle, uint8_t clsIndex);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClassTxMuxTimeoutAlgEn

 @Description   Enables timeout algorithm for the specified class.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     timeoutPeriod               - Timeout period in nano seconds.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClassTxMuxTimeoutAlgEn(t_Handle h_Bundle, uint8_t clsIndex, uint32_t timeoutPeriod);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigDisableRxIntr

 @Description   Disables interrupts on reception.
                For non-muxed data this will disable only interrupts on frames reception,
                for muxed data it will disable fragment reception interrupts and length error
                exception.
                Should be used when reception is done by polling.
 
 @Param[in]     h_Bundle                - A handle to the bundle module.
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PPPoHT_BUNDLE_ConfigDisableRxIntr(t_Handle h_Bundle);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_ConfigClsExtendedStat

 @Description   Add more statistics to the classes of this Bundle.
 
 @Param[in]     h_Bundle                - A handle to the bundle module.
                enable                  - To enable or disable this feature.
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_ConfigClsExtendedStat(t_Handle h_Bundle, bool enable);
/** @} */ /* end of pppoht_bundle_advanced_init_grp group */
/** @} */ /* end of pppoht_bundle_init_grp group */


/**************************************************************************//**
 @Group         pppoht_bundle_control_grp PPPoHT Bundle Control Data Unit

 @Description   PPPoHT Bundle control data unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   A structure for reading class statistics counters
*//***************************************************************************/
typedef struct t_PppohtClassStatistics
{
    uint32_t    rxFrmCnt;       /**< Received frames counter. */
    uint32_t    rxFrgLossCnt;   /**< Fragment Loss counter- Counts the number of times window
                                     moved due to fragment lost. */
    uint32_t    rxWinFullCnt;   /**< Window Full counter- Counts the number of fragments
                                     that were discarded due to lack of space in WBD. */
    uint32_t    rxClassQBsyCnt; /**< Class Queue Busy counter: discarded frames counter due
                                     to lack of empty BDs in the Core BD Ring. */
    uint32_t    rxMrruCnt;      /**< Longer than MRRU frame counter. */
    uint32_t    demuxBsyCnt;    /**< Demux busy counter. */
} t_PppohtClassStatistics;

/**************************************************************************//**
 @Description   A structure for returning all classes statistics counters
*//***************************************************************************/
typedef struct t_PppohtBundleStatistics
{
    t_PppohtClassStatistics pppohtClassStatistics[PPPOHT_MAX_NUM_OF_CLASSES_PER_BUNDLE];
                                            /**< An array of class statistics structures. */
} t_PppohtBundleStatistics;


/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_SetClassMuxSuperFrameSize

 @Description   Set class to work with a new super-frame-size.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     clsIndex                    - Class on which fuction operates.
 @Param[in]     superFrameSize              - MUX super frame size; Defines the maximum transmitted size of a
                                              PPPoHT MUX superframe; The size specified in this field does not include
                                              the PPPoHT MUX PID which could be of one or two bytes depending on the
                                              mode in which the bundle is programmed;

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_SetClassMuxSuperFrameSize(t_Handle h_Bundle, uint8_t clsIndex, uint16_t superFrameSize);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_SetNumOfClasses

 @Description   Set bundle to work with different num of classes.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     numOfClasses                - New number of class.

 @Return        E_OK on success; Error code otherwise.
 
  @Cautions     Number of classes can only be equal or smaller to the number of classes configured for the bundle.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_SetNumOfClasses(t_Handle h_Bundle, uint8_t numOfClasses);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_SetClassesFairnessAlg

 @Description   Set class to work with a new super-frame-size.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     wfq                         - An array of relative weight for each queue,
                                              in precentage.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_SetClassesFairnessAlg(t_Handle h_Bundle, uint8_t *wfq);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_SetExtendedSqnMode

 @Description   Set class to work with a new super-frame-size.

 @Param[in]     h_Bundle                    - A handle to the bundle module.
 @Param[in]     enable                      - Use extended mode or not.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_SetExtendedSqnMode(t_Handle h_Bundle, bool enable);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_GetStatistics

 @Description   Gets values of bundle statistics counters.

 @Param[in]     h_Bundle                        - A handle to the bundle module.
 @Param[in]     p_PppohtBundleStatistics        - A structure into which to put statistics.
 
 @Return        None.

 @Cautions      Does not zero counters. does not return statistics if the
                provided structure pointer is NULL.

*//***************************************************************************/
void PPPoHT_BUNDLE_GetStatistics(t_Handle h_Bundle, t_PppohtBundleStatistics   *p_PppohtBundleStatistics);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_DumpRegs

 @Description   Prints

 @Param[in]     h_Bundle - A handle to the bundle module.
*//***************************************************************************/
void     PPPoHT_BUNDLE_DumpRegs(t_Handle h_Bundle);
#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of pppoht_bundle_control_grp group */


/**************************************************************************//**
 @Group         pppoht_bundle_runtime_grp PPPoHT Bundle Runtime Data Unit

 @Description   PPPoHT Bundle runtime data unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_Tx

 @Description   Tx function, called to transmit a data buffer on a specific class (queue).

 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     clsIndex        - Class on which fuction operates.
 @Param[in]     p_Data          - A pointer to a data buffer.
 @Param[in]     h_BufContext    - User-assigned buffer context.
 @Param[in]     length          - Size of data for transmission.
 @Param[in]     lastBuffer      - Buffer poistion - TRUE for the last buffer
                                  of a frame, including a single buffer frame
 @Param[in]     pidInBuffer     - TRUE if PID is includded in the data frame,
                                  FALSE otherwise.
                                  NOTE - This field is relevant only for
                                  the first buffer.
 @Param[in]     pid             - Will be used only if pidInBuffer=FALSE;
                                  Will be written to the Tx BD.
                                  NOTE - This field is relevant only for
                                  the first buffer.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_Tx(t_Handle  h_Bundle,
                          uint16_t  clsIndex,
                          uint8_t   *p_Data,
                          t_Handle  h_BufContext,
                          uint16_t  length,
                          bool      lastBuffer,
                          bool      pidInBuffer,
                          uint16_t  pid);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_TxConf

 @Description   Tx confirmation routine, optional, may be called to verify
                transmission of all frames. The procedure performed by this
                routine will be performed automatically on next frame
                transmission, but if desired, calling this routine will invoke
                this action on demand.

 @Param[in]     h_Bundle  - A handle to the bundle module.
 @Param[in]     clsIndex  - Class on which fuction operates.
*//***************************************************************************/
void     PPPoHT_BUNDLE_TxConf(t_Handle h_Bundle, uint8_t clsIndex);

/**************************************************************************//**
 @Function      PPPoHT_BUNDLE_Rx

 @Description   Rx function, may be called to poll for received buffers.
                Normally, Rx proccess is invoked by the driver on Rx interrupt.
                Alternatively, this routine may be called on demand.

 @Param[in]     h_Bundle  - A handle to the bundle module.
 @Param[in]     clsIndex  - Class on which fuction operates.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_BUNDLE_Rx(t_Handle h_Bundle, uint8_t clsIndex);

/** @} */ /* end of pppoht_bundle_runtime_grp group */
/** @} */ /* end of pppoht_bundle_grp group */


/**************************************************************************//**
 @Group         pppoht_link_grp PPPoHT Link Application Programming Interface

 @Description   PPPoHT Link API functions, definitions and enums

 @{
*//***************************************************************************/

#define PPPOHT_MAX_NUM_OF_LINKS             64          /**< Maximum number of links on PPPoHT */
#define PPPOHT_MAX_NUM_OF_TX_QS_PER_LINK    8           /**< Maximun number of Tx queues per link */


/**************************************************************************//**
 @Group         pppoht_link_init_grp PPPoHT Link Initialization Unit

 @Description   PPPoHT Link initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define PPPOHT_LINK_INTR_MASK_MRU            0x2000      /**< Interrupt mask for MRU event */
#define PPPOHT_LINK_INTR_MASK_NIDL           0x0800      /**< Interrupt mask for No idle event */
#define PPPOHT_LINK_INTR_MASK_IDL            0x0400      /**< Interrupt mask for Idle event */
#define PPPOHT_LINK_INTR_MASK_UN             0x0020      /**< Interrupt mask for Underrun event */

/**************************************************************************//**
 @Description   A structure for configuring specific parameters for
                a link as a part of multilink
*//***************************************************************************/
typedef struct t_LinkTxQueuesFbpParam
{
    bool                        useFbp;                 /**< Use Tx FBP for this Tx queue or not -
                                                             USED ONLY IF Tx FBPS ARE DEFINED. */
    uint8_t                     fbpIndex;               /**< Tx FBP index [0-3] -
                                                             USED ONLY IF Tx FBPS ARE DEFINED. */
} t_LinkTxQueuesFbpParam;

/**************************************************************************//**
 @Description   A structure for configuring specific parameters for
                a link as a part of multilink
*//***************************************************************************/
typedef struct t_LinkPartOfMultiLink
{
    t_Handle                    h_Bundle;
    t_LinkTxQueuesFbpParam      txQueuesFbpParam[PPPOHT_MAX_NUM_OF_TX_QS_PER_LINK];
                                                        /**< A structure of parameters per Tx queue, used
                                                             for FBP mode indication */
} t_LinkPartOfMultiLink;

/**************************************************************************//**
 @Description   A structure for configuring specific parameters for
                a link as a part of multilink
*//***************************************************************************/
typedef struct t_LinkTxFbpsParam
{
    t_Handle                    h_TxFbps;               /**< A handle to the FBPs module accossiated with this
                                                             link. */
    t_LinkTxQueuesFbpParam      txQueuesFbpParam[PPPOHT_MAX_NUM_OF_TX_QS_PER_LINK];
                                                        /**< Used only when the link's Tx queues are also used
                                                             for transmission (as plain link).
                                                             FBP parameters per Tx queue, used
                                                             for mode selection and FBP pool choice. */
} t_LinkTxFbpsParam;

/**************************************************************************//**
 @Description   A structure for configuring specific parameters for
                a plain link (non-multilink)
*//***************************************************************************/
typedef struct t_LinkNotPartOfMultiLink
{
    t_Handle                    h_Pppoht;               /**< A handle to the PPoHT module */
    t_DataMemMngParam           dataMemMngParamForLcp;  /**< A structure of data management
                                                             parameters for the LCP queue. */
    bool                        useRxFbp;               /**< TRUE to use Rx FBP, false to work in swap mode,
                                                             effects the processing of the following union. */
    u_MemMngParam               rxMemMng;               /**< Rx memory management union, supports paremeters
                                                             for working with or without FBP; FBP mode is
                                                             selected by the previous parameter */
    bool                        useTxFbps;              /**< TRUE to use Tx FBP, false to work in swap mode;
                                                             Determines whether to use the following structure. */
    t_LinkTxFbpsParam           txFbpsParam;            /**< A structure of parameters for working with FBP. */
    uint8_t                     fwExtStructsMemId;      /**< Memory partition ID for external firmware structures -
                                                             USED ONLY IF NO FBPs ARE DEFINED (Rx and Tx). */
    uint32_t                    fwExtStructsMemAttr;    /**< Memory attributes for external firmware structures -
                                                             USED ONLY IF NO FBPs ARE DEFINED (Rx and Tx). */
    uint16_t                    muxDefPid;              /**< Default pid - relevant only for demux. */
} t_LinkNotPartOfMultiLink;

/**************************************************************************//**
 @Description   A union for configuring link parameters that distinguish
                plain links and multilinks
*//***************************************************************************/
typedef union
{
    t_LinkPartOfMultiLink       linkPartOfMultiLink;    /**< Multilink specific parameters. */
    t_LinkNotPartOfMultiLink    linkNotPartOfMultiLink; /**< Plain link specific parameters. */
} u_AdditionalLinkParam;

/**************************************************************************//**
 @Description   A structure for configuring link TDM parameters.
*//***************************************************************************/
typedef struct t_LinkTdmParam
{
    t_Handle                    h_TdmRxFrame;       /**< A handle to the predefined Tx TDM frame. */
    t_Handle                    h_TdmTxFrame;       /**< A handle to the predefined Rx TDM frame. */
    uint16_t                    numOfTdmTxSlots;    /**< Number of TDM Tx slots for this link. */
    uint8_t                     *txSlotsIndexes;    /**< An array of Tx slot indexes, for each TDM slot. */
    uint16_t                    numOfTdmRxSlots;    /**< Number of TDM Rx slots for this link. */
    uint8_t                     *rxSlotsIndexes;    /**< An array of Tx slot indexes, for each TDM slot. */
} t_LinkTdmParam;

/**************************************************************************//**
 @Description   A structure for configuring a link
*//***************************************************************************/
typedef struct t_PppohtLinkParam
{
    uint8_t                     linkId;             /**< The link index 0-63 */
    t_Handle                    h_AppId;            /**< A handle to the user's module */
    bool                        rxQueueEn;          /**< Enable Rx queue */
    bool                        rxDemuxQueueEn;     /**< Enable Rx demux queue */
                                                    /**< NOTE: If muxed data is received and rxDemuxQueueEn
                                                         is FALSE, behaviour is unknown!!! */
    uint8_t                     numOfTxQueues;      /**< Number of Tx queue to be used for the link, 1-8. */
    bool                        linkPartOfMlBundle; /**< TRUE for multilinks, FALSE for plain link. */
    uint16_t                    mru;                /**< Maximum receive unit size - in bytes. */
    u_AdditionalLinkParam       additionalLinkParam;/**< A union configuring specific parameters
                                                         for either multilink or plain link parameters. */
    t_LinkTdmParam              tdmParams;
    t_PppohtRxStoreCallback     *f_RxStore;         /**< A pointer to the upper layer's callback routine -
                                                         called to handle a received frame.*/
    t_PppohtTxConfCallback      *f_TxConf;          /**< A pointer to the upper layer's callback routine -
                                                         called to handle a transmitted frame.*/
    t_PppohtExceptionsCallback  *f_Exceptions;      /**< A user callback routine for handling PPPoHT link exceptions. */
} t_PppohtLinkParam;


/**************************************************************************//**
 @Function      PPPoHT_LINK_Config

 @Description   Configures a new link.
                This routine allocates the link structure and sets user
                and default parameters in the link driver parameters structure.

 @Param[in]     p_LinkParam - A pointer to the initializing parameters
                              structure for the link.

 @Return        A handle (descriptor) of the newly created link structure.
*//***************************************************************************/
t_Handle PPPoHT_LINK_Config(t_PppohtLinkParam *p_LinkParam);

/**************************************************************************//**
 @Function      PPPoHT_LINK_Init

 @Description   Initializes a link module.
                This routine initializes the link structure and the global
                parameters at the parameter ram.

 @Param[in]     h_Link - A handle to a link module.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_Init(t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_Free

 @Description   Frees a link and its resources if needed.

 @Param[in]     h_Link           - A handle to a link module.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_Free(t_Handle h_Link);


/**************************************************************************//**
 @Group         pppoht_link_advanced_init_grp PPPoHT Link Advanced Configuration Unit

 @Description   PPPoHT Link Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   A structure for configuring link IW parameters.
*//***************************************************************************/
typedef struct t_LinkIwParams
{
    t_IwRegistrationCallback    *f_Register;    /**< A callback to the registration
                                                     routine of the IW PPPoHT link module. */
    t_Handle                    h_IwPppohtLink; /**< A handle of the IW PPPoHT link module. */
    uint8_t                     numOfThreads;   /**< number of threads for this link */
} t_LinkIwParams;


/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigIw

 @Description   Configures interworking mode for this link.
                NOTE: this routine should be called ONLY if the link has its own
                queues, and not when it is only a part of a bundle.

 @Param[in]     h_Link                    - A handle to the link module.
 @Param[in]     p_LinkIwParams            - A pointer to a structure of interworking parameters..

 @Return        E_OK on success; Error code otherwise.

 @Cautions      If this Link is part of Multilink, use this routine ONLY if link
                has its own queue, i.e. if used as plain as well as as Multilink.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigIw(t_Handle h_Link, t_LinkIwParams *p_LinkIwParams);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigExceptions

 @Description   Selects the exceptions that will be reported. Upon an occurance of one
                of the selected events, f_Exception will be called, passing the bundle
                handle and the event.
                Calling this routine changes the internal driver data base
                from its default configuration - all errors and irregular
                behaviour is reported.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     exceptionsSelect            - An OR value of all exceptions that the user
                                              whould like to have reported

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigExceptions(t_Handle h_Link, e_ExceptionsSelect exceptionsSelect);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxLcpQueueIndex

 @Description   Define which Tx queue is used for transmitting LCP packets.
                Calling this routine changes the internal driver data base
                from its default configuration [0].

 @Param[in]     h_Link                    - A handle to the link module.
 @Param[in]     lcpQueueIndex             - The Tx queue that will be used
                                            for transmitting LCP packets (0-(numOfTxQueues-1)).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigTxLcpQueueIndex(t_Handle h_Link, uint8_t lcpQueueIndex);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigRxQueueBdRingSize

 @Description   Define the number of BD's in the Rx queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     bdRingSize                  - Number of BD's in the Rx queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigRxQueueBdRingSize(t_Handle h_Link, uint16_t bdRingSize);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigRxDemuxBdRingSize

 @Description   Define the number of BD's in the class Rx demux queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32].

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     bdRingSize                  - Number of BD's in the class Rx demux queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigRxDemuxBdRingSize(t_Handle h_Link, uint16_t bdRingSize);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxQueueBdRingSize

 @Description   Define tumber of BD's in the Tx queue.
                Calling this routine changes the internal driver data base
                from its default configuration [32]

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 @Param[in]     bdRingSize                  - Number of BD's in the Tx queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTxQueueBdRingSize(t_Handle h_Link, uint8_t txQueueIndex, uint16_t bdRingSize);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigQueuesFairnessAlg

 @Description   Configures fairness algorithm between the queues. The user should
                allocate an array in which each entry (one per tx queue) contains a
                value which represents the precentage of this queue in relation to
                the other queues. The user should make sure that the sum of all
                entries is 100.
                Calling this routine changes the internal driver data base
                from its default configuration - equal weights.
 
 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     wfq                         - An array of relative weight for each queue,
                                              in precentage.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigQueuesFairnessAlg(t_Handle h_Link, uint8_t *wfq);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTwoLevelsClassesFairnessAlg

 @Description   Enables spreading the queues in two priority groups.
                Each priority group may contain three types of information:
                - strict-priority queues (queues that are scheduled for transmission as soon as they have data)
                - weighted queues (queues that are scheduled for transmission based on their priority related to the same group they are part of)
                - queues marked as NOT USED IN THIS GROUP, that must be used in the second group
 
 @Param[in]     h_Bundle        - A handle to the bundle module.
 @Param[in]     highPriorityWfq - An array of relative weight for each queue in the high priority group,
                                  in precentage.
 @Param[in]     lowPriorityWfq  - An array of relative weight for each queue in the low priority group,
                                  in precentage.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTwoLevelsClassesFairnessAlg(t_Handle h_Bundle, uint8_t *highPriorityWfq, uint8_t * lowPriorityWfq);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxQueueMux

 @Description   Configures queue to work in mux mode.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 @Param[in]     superFrameSize              - MUX super frame size; Defines the maximum transmitted size of a
                                              PPPoHT MUX superframe; The size specified in this field does not include
                                              the PPPoHT MUX PID which could be of one or two bytes depending on the
                                              mode in which the bundle is programmed;
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTxQueueMux(t_Handle h_Link, uint8_t txQueueIndex, uint16_t superFrameSize);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxQueueMuxMaxSubfSize

 @Description   Enables maximal sub-frame size for the specified queue. This means that any frame larger than this value
                will be sent unmuxed while the previous super frame being built will be closed and sent prior to that.
                If this function is not called, by default, the queue will mux every sub frame.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 @Param[in]     maxSubfSize                 - Max sub frame size vto be muxed.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      When setting this feature, user must make sure that the link on the other side can receive both muxed
                and non muxed data.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTxQueueMuxMaxSubfSize(t_Handle h_Link, uint8_t txQueueIndex, uint16_t maxSubfSize);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxQueueMuxAgeAlgEn

 @Description   Enables aging algorithm for the specified queue.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 @Param[in]     agePeriod                   - Aging period in nano seconds.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTxQueueMuxAgeAlgEn(t_Handle h_Link, uint8_t txQueueIndex, uint32_t agePeriod);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxQueueMuxDisableSubFrmPFC

 @Description   Disables sub-frame PID compression.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTxQueueMuxDisableSubFrmPFC(t_Handle h_Link, uint8_t txQueueIndex);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxQueueMuxTimeoutAlgEn

 @Description   Enables timeout algorithm for the specified queue.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 @Param[in]     timeoutPeriod               - Timeout period in nano seconds.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigTxQueueMuxTimeoutAlgEn(t_Handle h_Link, uint8_t txQueueIndex, uint32_t timeoutPeriod);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigNumOfFlags

 @Description   Chooses how many flags to put between frames. When not in precise mode
                this number replies to minimum number of flags.
 
 @Param[in]     h_Link                    - A handle to the link module.
 @Param[in]     numOfFlags                - Number of flags between frames.
 @Param[in]     preciseMode               - Whther the number of flags be exact.
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      Percise mode will "cost" extra 48 bytes in MURAM space.
*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigNumOfFlags(t_Handle h_Link, uint8_t numOfFlags, bool preciseMode);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigIdleMode

 @Description   Should the link transmit an idle between two frames.
 
 @Param[in]     h_Link                    - A handle to the link module.
 @Param[in]     enable                    - Enable idle mode.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigIdleMode(t_Handle h_Link, bool enable);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigEnPFC

 @Description   Enables PID field compression (1 byte instead of 2 bytes).
 
 @Param[in]     h_Link                    - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      If this Link is part of Multilink, do not use this routine, use
                the Bundle API routine to set the PFC.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ConfigEnPFC(t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigEnACFC

 @Description   Enables address and controlcharacter compression on reception.
 
 @Param[in]     h_Link                  - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      If this Link is part of Multilink, do not use this routine, use
                the Bundle API routine to set the ACFC.
*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigEnACFC(t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigDisableRxIntr

 @Description   Disables interrupts on reception.
                For non-muxed data this will disable only interrupts on frames reception,
                for muxed data it will disable fragment reception interrupts and length error
                exception.
                Should be used when reception is done by polling.
 
 @Param[in]     h_Link                  - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigDisableRxIntr(t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigRxIntrQ

 @Description   Selects the Rx interrupt queue to be used.
                By default, the driver selects the next available and suitable interrupt
                queue. This routine should be called by applications that require
                an explicit selection of an interrupt queue. The routine will fail
                if this queue is already used by bundles that have conflicting
                parameters.
 
 @Param[in]     h_Link                  - A handle to the link module.
 @Param[in]     intrQNum                - A legal Rx interrupt queue number
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigRxIntrQ(t_Handle h_Link, uint8_t intrQNum);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ConfigTxIntrQ

 @Description   Selects the Tx interrupt queue to be used.
                By default, the driver selects the next available and suitable interrupt
                queue. This routine should be called by applications that require
                an explicit selection of an interrupt queue. The routine will fail
                if this queue is already used by bundles that have conflicting
                parameters.
 
 @Param[in]     h_Link                  - A handle to the link module.
 @Param[in]     intrQNum                - A legal Tx interrupt queue number
 
 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PPPoHT_LINK_ConfigTxIntrQ(t_Handle h_Link, uint8_t intrQNum);

/** @} */ /* end of pppoht_link_advanced_init_grp group */
/** @} */ /* end of pppoht_link_init_grp group */


/**************************************************************************//**
 @Group         pppoht_link_control_grp PPPoHT Link Control Data Unit

 @Description   PPPoHT Link control data unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   A structure for reading link statistics counters
*//***************************************************************************/
typedef struct t_PppohtLinkStatistics
{
    uint32_t    rxFcsErrCnt;        /**< FCS error counter. Initialized to 0.   */
    uint32_t    rxMruCnt;           /**< Longer than MRU frame/fragment counter. */
    uint32_t    rxLinkQBsyCnt;      /**< Link Queue Busy counter: Discarded frames counter
                                         due to lack of empty BDs in the Host BD Ring */
    uint32_t    rxFrmCnt;           /**< PPPoHT plain Frames counter. */
    uint32_t    rxFrgCnt;           /**< Multi Link Fragments counter. */
    uint32_t    rxByteCnt;          /**< Byte Count - counts the number of bytes received on link. */
    uint32_t    txFrmCnt;           /**< Transmitter Frame counter. Counts the number of
                                         transmitted frames on this link. */
    uint32_t    txFrgCnt;           /**< Transmitter Fragment counter. Counts the number of
                                         transmitted fragments on this link. */
    uint32_t    txByteCnt;          /**< Transmitter bytes counter. */
    uint32_t    rxAbortFCnt;        /**< Counts the number of frames received with abort occurrence. */
    uint32_t    rxIllegalFrmCnt;    /**< Counts the number of illegal frames. Illegal frame example
                                         include: short fragment with less than 13 bytes. */
    uint32_t    rxInvalidAcCnt;     /**< Invalid Address & Control character counter. */
} t_PppohtLinkStatistics;


/**************************************************************************//**
 @Function      PPPoHT_LINK_Terminate

 @Description   This function makes the link unavailable for transmiting data.
                The function will wait for all the data to sent first.

 @Param[in]     h_Link   - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_LINK_Terminate (t_Handle h_Link);
 
/**************************************************************************//**
 @Function      PPPoHT_LINK_Establish

 @Description   This function makes it possible to send data over the link (plain link)

 @Param[in]     h_Link   - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_LINK_Establish (t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_AttachToMultilink

 @Description   Attach a link to a bundle. The function may also be called in order to change the current
                bundle of a link to a different one. Note that the link will become a a part of a multilink
                bundle instead of a plain link (if it was plain). This routine may be
                called even when the link is up although the application should
                be careful with transmission on the link or the bundle during the attachment
                process (in general it is not advise - data may be lost).

 @Param[in]     h_Link   - A handle to the link module.
 @Param[in]     h_Bundle - A handle to the bundle module to attach.
 @Param[in]     direction - The direction in which the link will be used in the bundle.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      It is highly important that this move will be done with full collaboration of the other side.
                Such coordination should be made by the application level.
*//***************************************************************************/
t_Error  PPPoHT_LINK_AttachToMultilink(t_Handle h_Link, t_Handle h_Bundle, e_CommMode direction);

/**************************************************************************//**
 @Function      PPPoHT_LINK_DetachFromMultilink

 @Description   Detach a link from a bundle. After calling this function the link
                will become a plain link. This routine may be called even when the link is up
                although the application should be careful with transmission on the link or the bundle
                during the detachment process (in general it is not advise - data may be lost).
 
 @Param[in]     h_Link   - A handle to the link module.
 @Param[in]     direction - The direction which the link will stop function for the bundle.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      It is highly important that this move will be done with full collaboration of the other side.
                Such coordination should be made by the application level.
*//***************************************************************************/
t_Error  PPPoHT_LINK_DetachFromMultilink(t_Handle h_Link, e_CommMode direction);

/**************************************************************************//**
 @Function      PPPoHT_LINK_Down

 @Description   Control Function. The user should call this function in order to
                stop a link from its operation. All the buffers in the tx Queues
                will be confirmed to the user by the given call back. The routine
                disables all the link associated slots.
                This function will wait for all transmitted buffer to be coffered.
                Unless confUnsent was marked as TRUE the function will busy wait for all the
                buffers to be sent. When the confUnsent marked as TRUE the function will confirm
                non transmitted buffer with an error code.
 
 @Param[in]     h_Link     - A handle to the link module.
 @Param[in]     confUnsent - A boolean value indicating whether the user wishes to confirm all
                             the buffers even if they were not sent. This mode can be used when
                             shutting down the link quickly is vital (physical disconnection for example).
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Be sure the link is able to transmit data when using the confUnsent as FALSE. If there is a problem
                with the link it is advised to use the confUnsent flag.
*//***************************************************************************/
t_Error  PPPoHT_LINK_Down(t_Handle h_Link, bool confUnsent);

/**************************************************************************//**
 @Function      PPPoHT_LINK_Up

 @Description   Control Function. The user should call this function in order to
                re-start a link operation. After calling this function the link will
                be operative as a plane link. If the link was part of a multilink bundle,
                after this function it will be connected to the bundle only on the
                receive side. In order to send fragments over this link once more, the
                user must call PPPoHT_LINK_AttachToMultilink(). Separating the Rx and Tx
                of the bundle is necessary in order to be able to stable the link connection
                first and then to coordinate the attachment to the bundle.
                Note that if the link wasn't used as a plane link but only as part of a bundle,
                it will only be able to send and receive LCP frames before attached to the
                bundle.

 @Param[in]     h_Link    - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_Up(t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_ChangeSlots
 
 @Description   Change the link TDM slots.
 
 @Param[in]     h_Link      - A handle to the link module.
 @Param[in]     p_TdmParams - The new TDM parameters.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The link must be down while calling this routine.
                TDM can not be changed, only slots indexes and TDM frames.
                Beware of slot sharing between links.
                It is highly important that this change will be made for both sides of the connection.
*//***************************************************************************/
t_Error  PPPoHT_LINK_ChangeSlots(t_Handle h_Link, t_LinkTdmParam *p_TdmParams);

/**************************************************************************//**
 @Function      PPPoHT_LINK_SetTxQueueMuxSuperFrameSize

 @Description   Set queue to work with a new super-frame-size.

 @Param[in]     h_Link                      - A handle to the link module.
 @Param[in]     txQueueIndex                - Queue on which fuction operates.
 @Param[in]     superFrameSize              - MUX super frame size; Defines the maximum transmitted size of a
                                              PPPoHT MUX superframe; The size specified in this field does not include
                                              the PPPoHT MUX PID which could be of one or two bytes depending on the
                                              mode in which the bundle is programmed;
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_SetTxQueueMuxSuperFrameSize(t_Handle h_Link, uint8_t txQueueIndex, uint16_t superFrameSize);

/**************************************************************************//**
 @Function      PPPoHT_LINK_GetStatistics

 @Description   Gets values of link statistics counters.

 @Param[in]     h_Link                          - A handle to the link module.
 @Param[in]     p_PppohtLinkStatistics          - A structure into which to put statistics.
 
 @Return        None.

 @Cautions      Does not zero counters. does not return statistics if the
                provided structure pointer is NULL.
*//***************************************************************************/
void PPPoHT_LINK_GetStatistics(t_Handle h_Link, t_PppohtLinkStatistics   *p_PppohtLinkStatistics);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      PPPoHT_LINK_DumpRegs

 @Description   Prints

 @Param[in]     h_Link - A handle to the link module.
*//***************************************************************************/
void     PPPoHT_LINK_DumpRegs(t_Handle h_Link);
#endif /* (defined(DEBUG_ERRORS) && ... */
 
/** @} */ /* end of pppoht_link_control_grp group */


/**************************************************************************//**
 @Group         pppoht_link_runtime_grp PPPoHT Link Runtime Data Unit

 @Description   PPPoHT Link runtime data unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PPPoHT_LINK_Tx

 @Description   Tx function, called to transmit a data buffer on a specific queue.

 @Param[in]     h_Link          - A handle to the link module.
 @Param[in]     queueIndex      - Queue on which fuction operates.
 @Param[in]     p_Data          - A pointer to a data buffer.
 @Param[in]     h_BufContext    - User-assigned buffer context.
 @Param[in]     length          - Size of data for transmission.
 @Param[in]     lastBuffer      - Buffer poistion - TRUE for the last buffer
                                  of a frame, including a single buffer frame
 @Param[in]     pidInBuffer     - TRUE if PID is includded in the data frame,
                                  FALSE otherwise.
                                  NOTE - This field is relevant only for
                                  the first buffer.
 @Param[in]     pid             - Will be used only if pidInBuffer=FALSE.
                                  Will be written to the Tx BD.
                                  NOTE - This field is relevant only for
                                  the first buffer.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPoHT_LINK_Tx(t_Handle     h_Link,
                       uint16_t     queueIndex,
                       uint8_t      *p_Data,
                       t_Handle     h_BufContext,
                       uint16_t     length,
                       bool         lastBuffer,
                       bool         pidInBuffer,
                       uint16_t     pid);

/**************************************************************************//**
 @Function      PPPoHT_LINK_TxLcp

 @Description   Tx function, called to transmit an LCP data buffer on the
                specially designated queue.

 @Param[in]     h_Link          - A handle to the link module.
 @Param[in]     p_Data          - A pointer to an LCP data buffer.
 @Param[in]     h_BufContext    - User-assigned buffer context.
 @Param[in]     length          - Size of data for transmission.
 @Param[in]     lastBuffer      - Buffer poistion - TRUE for the last buffer
                                  of a frame, including a single buffer frame
 @Param[in]     pidInBuffer     - TRUE if PID is includded in the data frame,
                                  FALSE otherwise.
                                  NOTE - This field is relevant only for
                                  the first buffer.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_TxLcp(t_Handle     h_Link,
                           uint8_t      *p_Data,
                           t_Handle     h_BufContext,
                           uint16_t     length,
                           bool         lastBuffer,
                           bool         pidInBuffer);

/**************************************************************************//**
 @Function      PPPoHT_LINK_TxConf

 @Description   Tx confirmation routine, optional, may be called to verify
                transmission of all frames. The procedure performed by this
                routine will be performed automatically on next frame transmission,
                but if desired, calling this routine will invoke this action on
                demand.

 @Param[in]     h_Link      - A handle to the link module.
 @Param[in]     queueIndex  - Queue on which fuction operates.

 @Return        None.
*//***************************************************************************/
void     PPPoHT_LINK_TxConf(t_Handle h_Link, uint8_t queueIndex);

/**************************************************************************//**
 @Function      PPPoHT_LINK_TxLcpConf

 @Description   Tx LCP queue confirmation routine, optional, may be called to verify
                transmission of all frames. The procedure performed by this
                routine will be performed automatically on next LCP frame transmission,
                but if desired, calling this routine will invoke this action on
                demand.

 @Param[in]     h_Link  - A handle to the link module.

 @Return        None.
*//***************************************************************************/
void PPPoHT_LINK_TxLcpConf(t_Handle h_Link);

/**************************************************************************//**
 @Function      PPPoHT_LINK_Rx

 @Description   Rx function, may be called to poll for received buffers.
                Normally, Rx proccess is invoked by the driver on Rx interrupt.
                Alternatively, this routine may be called on demand.

 @Param[in]     h_Link  - A handle to the link module.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  PPPoHT_LINK_Rx(t_Handle h_Link);

/** @} */ /* end of pppoht_link_runtime_grp group */
/** @} */ /* end of pppoht_link_grp group */
/** @} */ /* end of pppoht_grp group */


#endif /*__PPPOHT_EXT_H */
