/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          arc4_ext.h

 @Description   structures and definitions for operating ARC4 algorithm.

*//***************************************************************************/

#ifndef __ARC4_EXT_H
#define __ARC4_EXT_H

#define RC4_CTX_OUT_LEN 259

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_arc4 SEC ARC4

 @Description   SEC ARC4 functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_arc4_init SEC ARC4 Initialization Unit

 @Description   SEC ARC4 initialization functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      arc4_alloc_ctx

 @Description   Allocate memory for the arc4 ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Return        ctx - ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle arc4_alloc_ctx();

/**************************************************************************//**
 @Function      arc4_setkey

 @Description   The routine initializes the key for the arc4 calculation.

				The routine sets the specific key and its length in the
                private context structure.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         key -    (In) handler to the key.
 @Param         keylen - (In) the length of the key handler.
 @Param         flags -  (In) Not relevant.

 @Return        None.
*//***************************************************************************/
void     arc4_setkey (void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/**************************************************************************//**
 @Function      arc4_setiv

 @Description   The function sets the iv - the src pointer in the context structure.
 
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) handler to the initialization vector.
 @Param         len -  (In) the length of the initialization vector.

 @Return        None.
*//***************************************************************************/
void     arc4_setiv (void *ctx, void *src, unsigned int len);

/**************************************************************************//**
 @Function      arc4_set_outiv

 @Description   The function sets the output iv- the outsrc pointer in the context structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         outsrc - (In) handler to the output initialization vector.
 @Param         outlen - (In) the length of the output initialization vector.

 @Return        None.
*//***************************************************************************/
void     arc4_set_outiv (void *ctx, void *outsrc, unsigned int outlen);

/**************************************************************************//**
 @Function      arc4_set_srclen

 @Description   The routine sets the data source length in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -  (In) the length of the input.

 @Return        None.

*//***************************************************************************/
void     arc4_set_srclen (void *ctx, unsigned int len);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_arc4_runtime SEC ARC4 Runtime Unit

 @Description   SEC ARC4 Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      arc4_encrypt

 @Description   initialize and send the arc4 request to the Crypto hardware.
                This routine initializes the arc4 request structure
                by the fields in the arc4 ctx and than call to the Ioctl
                function in order to process the request.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the output destination handler.
 @Param         src -  (In) the input source handler.

 @Return        None.
*//***************************************************************************/
void     arc4_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec arc4 */
/** @} */ /* end of group sec  */

#endif
