/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __ESDHC_EXT_H
#define __ESDHC_EXT_H

#include "error_ext.h"
#include "std_ext.h"
#include "sdhc_std_ext.h"


/**************************************************************************//**
 @Group         esdhc_grp eSDHC Application Programming Interface

 @Description   eSDHC API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         esdhc_exception_grp eSDHC Exception Unit

 @Description   eSDHC exception functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Defines the error status
*//***************************************************************************/
typedef enum e_EsdhcErrorStatus{
    e_ESDHC_NON_RECOVERABLE_ERROR,               /**< Non recoverable error                                                                  */
    e_ESDHC_CMD_WO_DAT_ERROR_NO_TRANSFER_ERROR,  /**< An error occurred in CMD_wo_DAT, but not in the SD memory transfer                     */
    e_ESDHC_CMD_WO_DAT_ERROR_TRANSFER_ERROR,     /**< An error occurred in CMD_wo_DAT, and also occurred in the SD memory transfer           */
    e_ESDHC_NO_CMD_WO_DAT_ERROR_TRANSFER_ERROR,  /**< An error did not occur in CMD_wo_DAT, but an error occurred in the SD memory transfer  */
    e_ESDHC_CMD_WO_DAT_NOT_ISSUED_TRANSFER_ERROR /**< CMD_wo_DAT was not issued, and an error occurred in the SD memory transfer             */
}e_EsdhcErrorStatus;

/** @} */ /* esdhc_exception_grp */

/**************************************************************************//**
 @Group         esdhc_control_grp eSDHC Control Unit

 @Description   eSDHC control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   eSDHC transfer parameters
 *//***************************************************************************/
typedef struct t_EsdhcTransferParam
{
    uint32_t        srcAddr;                    /**< Transfer source address                                 */
    uint32_t        destAddr;                   /**< Transfer destination address                            */
    uint32_t        size;                       /**< Transfer size                                           */
    uint32_t        transferredSize;            /**< The last source address that transferred successfully   */
 
} t_EsdhcTransferParam;

/**************************************************************************//**
 @Description   Describes the version of the CSD structure for MMC card.
*//***************************************************************************/
typedef enum e_EsdhcMmcCsdVer
{
    e_ESDHC_CSD_MMC_VERSION_1_0,                /**< CSD version 1.0 */
    e_ESDHC_CSD_MMC_VERSION_1_1,                /**< CSD version 1.1 */
    e_ESDHC_CSD_MMC_VERSION_1_2                 /**< CSD version 1.2 */
} e_EsdhcMmcCsdVer;

/**************************************************************************//**
 @Description   Describes the version of the CSD structure for SD card.
*//***************************************************************************/
typedef enum e_EsdhcSdCsdVer
{
    e_ESDHC_CSD_SD_VERSION_1_0,                 /**< CSD Version 1.0 */
    e_ESDHC_CSD_SD_VERSION_2_0                  /**< CSD Version 2.0 */
} e_EsdhcSdCsdVer;

/**************************************************************************//**
 @Description   Defines the MultiMediaCard protocol version supported by the card.
*//***************************************************************************/
typedef enum e_EsdhcMmcSpecVer
{
    e_ESDHC_MMC_SPEC_VERSION_1_0_TO_1_2,        /**<Version 1.0-1.2         */
    e_ESDHC_MMC_SPEC_VERSION_1_4,               /**<Version 1.4             */
    e_ESDHC_MMC_SPEC_VERSION_2_0_TO_2_2,        /**<Version 2.0 - 2.2       */
    e_ESDHC_MMC_SPEC_VERSION_3_1__3_2__3_31,    /**<Version 3.1 - 3.2 -3.31 */
    e_ESDHC_MMC_SPEC_VERSION_4                  /**<Version 4.0             */
}e_EsdhcMmcSpecVer;

/**************************************************************************//**
 @Description   Defines the asynchronous part (relative to the MultiMediaCard
                 clock (CLK)) of the read access time.
*//***************************************************************************/
typedef enum e_EsdhcTaacTimeUnit
{
    e_ESDHC_TAAC_TIME_UNIT_1NS,                 /**< 1ns                   */
    e_ESDHC_TAAC_TIME_UNIT_10NS,                /**< 10ns                  */
    e_ESDHC_TAAC_TIME_UNIT_100NS,               /**< 100ns                 */
    e_ESDHC_TAAC_TIME_UNIT_1MMS,                /**< 1 micro second        */
    e_ESDHC_TAAC_TIME_UNIT_10MMS,               /**< 10 micro second       */
    e_ESDHC_TAAC_TIME_UNIT_100MMS,              /**< 100 micro second      */
    e_ESDHC_TAAC_TIME_UNIT_1MS,                 /**< 1ms                   */
    e_ESDHC_TAAC_TIME_UNIT_10MS                 /**< 10ms                  */
}e_EsdhcTaacTimeUnit;

/**************************************************************************//**
 @Description   Defines the asynchronous part (relative to the MultiMediaCard
                 clock (CLK)) of the read access time.
*//***************************************************************************/
typedef enum e_EsdhcTaacTimeValue
{
    e_ESDHC_TAAC_TIME_VALUE_1_0 = 0x1,          /**< 1.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_1_2 = 0x2,          /**< 1.2 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_1_3 = 0x3,          /**< 1.3 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_1_5 = 0x4,          /**< 1.5 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_2_0 = 0x5,          /**< 2.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_2_5 = 0x6,          /**< 2.5 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_3_0 = 0x7,          /**< 3.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_3_5 = 0x8,          /**< 3.5 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_4_0 = 0x9,          /**< 4.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_4_5 = 0xA,          /**< 4.5 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_5_0 = 0xB,          /**< 5.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_5_5 = 0xC,          /**< 5.5 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_6_0 = 0xD,          /**< 6.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_7_0 = 0xE,          /**< 7.0 Multiplier factor */
    e_ESDHC_TAAC_TIME_VALUE_8_0 = 0xF           /**< 8.0 Multiplier factor */

}e_EsdhcTaacTimeValue;

/**************************************************************************//**
 @Description   Transfer's speed frequency unit -
                 defines the maximum data transfer rate TRAN_SPEED
*//***************************************************************************/
typedef enum e_EsdhcTransferSpeedUnit
{
    e_ESDHC_TRANS_SPEED_UNIT_100kBIT_S,         /**< 100KHz transfer speed rate */
    e_ESDHC_TRANS_SPEED_UNIT_1MBIT_S,           /**< 1MHz transfer speed rate   */
    e_ESDHC_TRANS_SPEED_UNIT_10MBIT_S,          /**< 10MHz transfer speed rate  */
    e_ESDHC_TRANS_SPEED_UNIT_100MBIT_S          /**< 100MHz transfer speed rate */
}e_EsdhcTransferSpeedUnit;

/**************************************************************************//**
 @Description   Transfer's speed Multiplier factor for SD card -
                 defines the maximum data transfer rate TRAN_SPEED
*//***************************************************************************/
typedef enum e_EsdhcSdTransferSpeedValue
{
    e_ESDHC_SD_TRANS_SPEED_VALUE_1_0 = 0x1,     /**< 1.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_1_2 = 0x2,     /**< 1.2 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_1_3 = 0x3,     /**< 1.3 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_1_5 = 0x4,     /**< 1.5 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_2_0 = 0x5,     /**< 2.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_2_5 = 0x6,     /**< 2.5 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_3_0 = 0x7,     /**< 3.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_3_5 = 0x8,     /**< 3.5 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_4_0 = 0x9,     /**< 4.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_4_5 = 0xA,     /**< 4.5 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_5_0 = 0xB,     /**< 5.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_5_5 = 0xC,     /**< 5.5 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_6_0 = 0xD,     /**< 6.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_7_0 = 0xE,     /**< 7.0 Multiplier factor */
    e_ESDHC_SD_TRANS_SPEED_VALUE_8_0 = 0xF      /**< 8.0 Multiplier factor */
}e_EsdhcSdTransferSpeedValue;

/**************************************************************************//**
 @Description   Transfer's speed Multiplier factor for MMC card -
                 defines the maximum data transfer rate TRAN_SPEED
*//***************************************************************************/
typedef enum e_EsdhcMmcTransferSpeedValue
{
    e_ESDHC_MMC_TRANS_SPEED_VALUE_1_0 = 0x1,    /**< 1.0 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_1_2 = 0x2,    /**< 1.2 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_1_3 = 0x3,    /**< 1.3 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_1_5 = 0x4,    /**< 1.5 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_2_0 = 0x5,    /**< 2.0 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_2_5 = 0x6,    /**< 2.6 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_3_0 = 0x7,    /**< 3.0 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_3_5 = 0x8,    /**< 3.5 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_4_0 = 0x9,    /**< 4.0 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_4_5 = 0xA,    /**< 4.5 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_5_0 = 0xB,    /**< 5.2 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_5_5 = 0xC,    /**< 5.5 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_6_0 = 0xD,    /**< 6.0 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_7_0 = 0xE,    /**< 7.0 Transfer speed time value - Multiplier factor */
    e_ESDHC_MMC_TRANS_SPEED_VALUE_8_0 = 0xF     /**< 8.0 Transfer speed time value - Multiplier factor */
}e_EsdhcMmcTransferSpeedValue;

/**************************************************************************//**
 @Description   The maximum read data block length
*//***************************************************************************/
typedef enum e_EsdhcBlockLength
{
    e_ESDHC_BLOCK_LEN_1_BYTE    = 0x0,          /**< 1 byte block length    */
    e_ESDHC_BLOCK_LEN_2_BYTES   = 0x1,          /**< 2 byte block length    */
    e_ESDHC_BLOCK_LEN_4_BYTES   = 0x2,          /**< 4 byte block length    */
    e_ESDHC_BLOCK_LEN_8_BYTES   = 0x3,          /**< 8 byte block length    */
    e_ESDHC_BLOCK_LEN_16_BYTES  = 0x4,          /**< 16 byte block length   */
    e_ESDHC_BLOCK_LEN_32_BYTES  = 0x5,          /**< 32 byte block length   */
    e_ESDHC_BLOCK_LEN_64_BYTES  = 0x6,          /**< 64 byte block length   */
    e_ESDHC_BLOCK_LEN_128_BYTES = 0x7,          /**< 128 byte block length  */
    e_ESDHC_BLOCK_LEN_256_BYTES = 0x8,          /**< 256 byte block length  */
    e_ESDHC_BLOCK_LEN_512_BYTES = 0x9,          /**< 512 byte block length  */
    e_ESDHC_BLOCK_LEN_1_M       = 0xA,          /**< 1 Mbyte block length   */
    e_ESDHC_BLOCK_LEN_2_M       = 0xB           /**< 2 Mbyte block length   */

}e_EsdhcBlockLength;

/**************************************************************************//**
 @Description  The minimum values for read and write currents at the minimal power supply VDD
*//***************************************************************************/
typedef enum e_EsdhcVddCurrMin
{
    e_ESDHC_VDD_MIN_0_5MA       = 0,            /**< 0.5mA minimal power supply  */
    e_ESDHC_VDD_MIN_1MA         = 1,            /**< 1.0mA minimal power supply  */
    e_ESDHC_VDD_MIN_5MA         = 2,            /**< 5.0mA minimal power supply  */
    e_ESDHC_VDD_MIN_10MA        = 3,            /**< 10mA minimal power supply   */
    e_ESDHC_VDD_MIN_25MA        = 4,            /**< 25mA minimal power supply   */
    e_ESDHC_VDD_MIN_35MA        = 5,            /**< 355mA minimal power supply  */
    e_ESDHC_VDD_MIN_60MA        = 6,            /**< 605mA minimal power supply  */
    e_ESDHC_VDD_MIN_100MA       = 7             /**< 1005mA minimal power supply */
}e_EsdhcVddCurrMin;

/**************************************************************************//**
 @Description   The maximum values for read and write currents at the maximal power supply VDD
*//***************************************************************************/
typedef enum e_EsdhcVddCurrMax
{
    e_ESDHC_VDD_MAX_1_MA        = 0,            /**< 1mA maximal power supply    */
    e_ESDHC_VDD_MAX_5_MA        = 1,            /**< 5mA maximal power supply    */
    e_ESDHC_VDD_MAX_10_MA       = 2,            /**< 10mA maximal power supply   */
    e_ESDHC_VDD_MAX_25_MA       = 3,            /**< 25mA maximal power supply   */
    e_ESDHC_VDD_MAX_35_MA       = 4,            /**< 35mA maximal power supply   */
    e_ESDHC_VDD_MAX_45_MA       = 5,            /**< 45mA maximal power supply   */
    e_ESDHC_VDD_MAX_80_MA       = 6,            /**< 80mA maximal power supply   */
    e_ESDHC_VDD_MAX_200_MA      = 7             /**< 200mAmaximal power supply   */
}e_EsdhcVddCurrMax;

/**************************************************************************//**
 @Description   Defines the typical block program time as a multiple of the read access time.
*//***************************************************************************/
typedef enum e_EsdhcR2wFactor
{
    e_ESDHC_R2W_FACTOR_1         = 0,           /**< 1 Multiples of read access time  */
    e_ESDHC_R2W_FACTOR_2         = 1,           /**< 2 Multiples of read access time  */
    e_ESDHC_R2W_FACTOR_4         = 2,           /**< 4 Multiples of read access time  */
    e_ESDHC_R2W_FACTOR_8         = 3,           /**< 8 Multiples of read access time  */
    e_ESDHC_R2W_FACTOR_16        = 4,           /**< 16 Multiples of read access time */
    e_ESDHC_R2W_FACTOR_32        = 5,           /**< 32 Multiples of read access time */
    e_ESDHC_R2W_FACTOR_64        = 6,           /**< 64 Multiples of read access time */
    e_ESDHC_R2W_FACTOR_128        = 7           /**< 128 Multiples of read access time*/

}e_EsdhcR2wFactor;

/**************************************************************************//**
 @Description   Indicates the file format on the card.
*//***************************************************************************/
typedef enum e_EsdhcFileFormat
{
    e_ESDHC_FILE_FORMAT_HARD_DISK              = 0, /**< Hard disk-like file system with partition table */
    e_ESDHC_FILE_FORMAT_HARD_DOS_FAT           = 1, /**< DOS FAT (floppy-like) with boot sector only (no partition table) */
    e_ESDHC_FILE_FORMAT_HARD_UNIVERSAL_FORMAT  = 2, /**< Universal File Format */
    e_ESDHC_FILE_FORMAT_OTHER                  = 3  /**< Others / Unknown */
}e_EsdhcFileFormat;

/**************************************************************************//**
 @Description   Defines the ECC code that was used for storing data on the card.
*//***************************************************************************/
typedef enum e_EsdhcEccType
{
    e_ESDHC_ECC_TYPE_NONE       = 0,  /**< Maximum number of correctable bits per block is 0 */
    e_ESDHC_ECC_TYPE_BCH        = 1,  /**< Maximum number of correctable bits per block is 3 */
    e_ESDHC_ECC_TYPE_NOT_APPLICABLE
}e_EsdhcEccType;

/**************************************************************************//**
 @Description   MMC Card Specific Data structure
 *//***************************************************************************/
typedef struct t_EsdhcMmcCardDetails
{
    e_EsdhcMmcCsdVer                    mmcCsdVersion;      /**<MMC csd version  */
    e_EsdhcMmcSpecVer                   mmcVer;             /**<MMC spec version */
    e_EsdhcTaacTimeUnit                 taacTimeUnit;       /**<asynchronous part of read access time unit */
    e_EsdhcTaacTimeValue                taacTimeValue;      /**<asynchronous part of read access Multiplier factor*/
    uint32_t                            nsac;               /**<NSACDefines the worst case for the
                                                                 clock dependent factor of the data access time.
                                                                 The unit for NSAC is 100 clock cycles */
 
    e_EsdhcTransferSpeedUnit            transferSpeedUnit;  /**<Transfer speed rate unit */
    e_EsdhcMmcTransferSpeedValue        mmcTransferSpeedValue;/**<Transfer speed rate Multiplier factor */

    uint32_t                            ccc;                /**<The card command class register
                                                                 CCC defines which command classes are supported by
                                                                 this card. A value of 1 in a CCC bit means that the
                                                                corresponding command class is supported. */
    e_EsdhcBlockLength                  readBlockLength;    /**<Max block length for read transaction */
    bool                                readBlockPartial;   /**<Read partial blocks capability */
    bool                                writeBlockMisalign; /**<Defines if the data block to be written by one command can be spread over more than one physical
                                                                block of the memory device. The size of the memory block is defined in writeBlockLength */
    bool                                readBlockMisalign;  /**<Defines if the data block to be read by one command can be spread over more than one physical block of the
                                                                memory device. The size of the memory block is defined in readBlockLength */
    bool                                dsrImp;             /**<Defines if the configurable driver stage is integrated on the card */
    uint32_t                            cardMemCapacity;    /**<C_SIZE (Device Size)This parameter is used to compute the card capacity. The memory capacity of the card is
                                                                computed from the entries C_SIZE, C_SIZE_MULT and READ_BL_LEN as follows:
                                                                memory capacity = BLOCKNR * BLOCK_LEN */
    e_EsdhcVddCurrMin                   vddReadCurrMin;     /**<Minimal power supply for read  transaction */
    e_EsdhcVddCurrMax                   vddReadCurrMax;     /**<Maximal power supply for read  transaction */
    e_EsdhcVddCurrMin                   vddWriteCurrMin;    /**<Minimal power supply for write  transaction */
    e_EsdhcVddCurrMax                   vddWriteCurrMax;    /**<Maximal power supply for write  transaction */

    uint32_t                            sizeOfErasableUnit; /**<This size is the minimum number of write blocks that can be erased in a single erase command */
    uint32_t                            wpGrpSize;          /**<The size of a write protected group. defining the
                                                                 number of erase groups which can be write protected */
    bool                                wpGrpEnable;        /**<Group write protection possibility */
    e_EsdhcEccType                      defaultEcc;         /**<Defines the ECC code which is recommended for use */
    e_EsdhcR2wFactor                    r2wFactor;          /**<Defines the typical block program time as a multiple of the read access time */
    e_EsdhcBlockLength                  writeBlockLength;   /**<Max block length for write transaction */
    bool                                writeBlockPartial;  /**<Write partial blocks capability */
    e_EsdhcFileFormat                   fileFormatGroup;    /**<Indicates the selected group of file formats */
 
    uint32_t                            copy;               /**<Defines if the contents is original (=0) or has been copied (=1). The COPY bit for OTP and MTP
                                                                devices, sold to end consumers, is set to 1,
                                                                which identifies the card contents as a copy.  */
    bool                                fermWriteProtect;   /**<Permanently protects the whole card content against overwriting or erasing */
    bool                                tmpWriteProtect;    /**<Temporarily protects the whole card content from being overwritten or erased */
    bool                                contentProtApp;     /**<This field in the CSD indicates whether the content protection application is supported */
    e_EsdhcFileFormat                   fileFormat;         /**<Indicates the file format on the card */
    e_EsdhcEccType                      ecc;                /**<Defines the ECC code that was used for storing data on the card*/
 
} t_EsdhcMmcCardDetails;

/**************************************************************************//**
 @Description   SD Card Specific Data structure
 *//***************************************************************************/
typedef struct t_EsdhcSdCardDetails
{
    e_EsdhcSdCsdVer                      sdCsdVersion;              /**<SD csd version */
    e_EsdhcTaacTimeUnit                  taacTimeUnit;              /**<asynchronous part of read access time unit */
    e_EsdhcTaacTimeValue                 taacTimeValue;             /**<asynchronous part of read access Multiplier factor*/
    uint32_t                             nsac;                      /**<NSACDefines the worst case for the
                                                                        clock dependent factor of the data access time.
                                                                        The unit for NSAC is 100 clock cycles*/
 
    e_EsdhcTransferSpeedUnit             transferSpeedUnit;          /**<Transfer speed rate unit */
    e_EsdhcSdTransferSpeedValue          sdTransferSpeedValue;       /**<Transfer speed rate Multiplier factor */
    uint32_t                             ccc;                        /**<The card command class register
                                                                         CCC defines which command classes are supported by
                                                                         this card. A value of 1 in a CCC bit means that the
                                                                         corresponding command class is supported. */
    e_EsdhcBlockLength                   readBlockLength;            /**<Max block length for read transaction */
    bool                                 readBlockPartial;           /**<Read partial blocks capability */
    bool                                 writeBlockMisalign;         /**<Defines if the data block to be written by one command can be spread over more than one physical
                                                                         block of the memory device. The size of the memory block is defined in writeBlockLength*/
    bool                                 readBlockMisalign;          /**<Defines if the data block to be read by one command can be spread over more than one physical block of the
                                                                         memory device. The size of the memory block is defined in readBlockLength*/
    bool                                 dsrImp;                     /**<Defines if the configurable driver stage is integrated on the card*/
    uint32_t                             cardMemCapacity;            /**<C_SIZE (Device Size)This parameter is used to compute the card capacity. The memory capacity of the card is
                                                                         computed from the entries C_SIZE, C_SIZE_MULT and READ_BL_LEN as follows:
                                                                         memory capacity = BLOCKNR * BLOCK_LEN*/
    e_EsdhcVddCurrMin                    vddReadCurrMin;             /**<Minimal power supply for read  transaction*/
    e_EsdhcVddCurrMax                    vddReadCurrMax;             /**<Maximal power supply for read  transaction*/
    e_EsdhcVddCurrMin                    vddWriteCurrMin;            /**<Minimal power supply for write  transaction*/
    e_EsdhcVddCurrMax                    vddWriteCurrMax;            /**<Maximal power supply for write  transaction*/

    bool                                 eraseBlkEnable;             /**<Defines the granularity of the unit size of the data to be erased.
                                                                         if the value is FALSE, the host can erase one or multiple units of sectorSize.
                                                                         if the value is TRUE, the host can erase one or multiple units of 512 bytes.*/
    uint32_t                             sectorSize;                 /**<The size of an erasable sector. */

    uint32_t                             wpGrpSize;                  /**<The size of a write protected group. defining the
                                                                         number of erase groups which can be write protected*/
    bool                                 wpGrpEnable;                /**<Group write protection possibility*/
    e_EsdhcR2wFactor                     r2wFactor;                  /**<Defines the typical block program time as a multiple of the read access time */
    e_EsdhcBlockLength                   writeBlockLength;           /**<Max block length for write transaction */
    bool                                 writeBlockPartial;          /**<Write partial blocks capability */
    e_EsdhcFileFormat                    fileFormatGroup;            /**<Indicates the selected group of file formats*/
    uint32_t                             copy;                       /**<Defines if the contents is original (=0) or has been copied (=1). The COPY bit for OTP and MTP
                                                                         devices, sold to end consumers, is set to 1,
                                                                         which identifies the card contents as a copy.  */
    bool                                 fermWriteProtect;           /**<Permanently protects the whole card content against overwriting or erasing*/
    bool                                 tmpWriteProtect;            /**<Temporarily protects the whole card content from being overwritten or erased*/
    e_EsdhcFileFormat                    fileFormat;                 /**<Indicates the file format on the card*/
 
} t_EsdhcSdCardDetails;

/**************************************************************************//**
 @Function      ESDHC_SetBusWidth

 @Description   Change bus width.
 
                Set the bus width in the controller and the connected card
 
 @Param[in]     h_Esdhc - The handle to the eSDHC object.
 @Param[in]     width   - The required bus width.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().
*//***************************************************************************/
t_Error ESDHC_SetBusWidth(t_Handle h_Esdhc, e_SdhcBusWidth width);

/**************************************************************************//**
 @Function      ESDHC_Free

 @Description   This routine is called to shut down the eSDHC Controller.
 
                It releases all previously allocated resources.

 @Param[in]     h_Esdhc - Handle to the eSDHC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
*//***************************************************************************/
t_Error ESDHC_Free(t_Handle h_Esdhc);

/**************************************************************************//**
 @Function      ESDHC_GetSdCardDetails

 @Description   Returns card details.
 
                 This routine returns the connected card SD details like card capacity, max frequency ect.
 
 @Param[in]     h_Esdhc         - The handle to the eSDHC object.
 @Param[in,out] p_CardDetails   - Pointer to a structure for connected card details.

 @Return        E_OK - if action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().

*//***************************************************************************/
t_Error ESDHC_GetSdCardDetails(t_Handle h_Esdhc, t_EsdhcSdCardDetails* p_CardDetails);

/**************************************************************************//**
 @Function      ESDHC_GetMmcCardDetails

 @Description   Returns card details.
 
                 This routine returns the connected MMC card details like card capacity, max frequency ect.
 
 @Param[in]     h_Esdhc         - The handle to the eSDHC object.
 @Param[in,out] p_CardDetails   - Pointer to a structure for connected card details.

 @Return        E_OK - if action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().

*//***************************************************************************/
t_Error ESDHC_GetMmcCardDetails(t_Handle h_Esdhc, t_EsdhcMmcCardDetails* p_CardDetails);

/**************************************************************************//**
 @Function      ESDHC_IsReadyForTransfer

 @Description   Check if the controller is ready for data transfer.
 
                 This routine should be called before ESDHC_write or ESDHC_read
                 in order to verify that the eSDHC can handle this routine (the DAT lines are free)
 
 @Param[in]     h_Esdhc - The handle to the eSDHC object.

 @Return        TRUE    - If the controller is ready for a new transfer. FALSE if it doesn't.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().

*//***************************************************************************/
bool ESDHC_IsReadyForTransfer(t_Handle h_Esdhc);

/**************************************************************************//**
 @Function      ESDHC_Abort

 @Description   Abort the current transaction.
 
                 If this routine is called when executing a read transfer,
                 eSDHC will stop the reading to the buffer. If this command is set
                 when executing a write transfer, eSDHC will stop driving the DAT line.
                 After issuing the abort command, the eSDHC controller is reset.
 
 @Param[in]     h_Esdhc   -  The handle to the eSDHC object.
 @Param[in]     abortType -  Synchronous abort or asynchronous abort.

 @Return        E_OK - if action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().

*//***************************************************************************/
t_Error ESDHC_Abort(t_Handle h_Esdhc, e_SdhcAbortType abortType);

/**************************************************************************//**
 @Function      ESDHC_Reset

 @Description   Reset the eSDHC controller and the connected card.
 
 @Param[in]     h_Esdhc - The handle to the eSDHC object.

 @Return        E_OK    - If action was successful, the occurred error otherwise.

 @Cautions      This routine must be called after ESDHC_Init().

*//***************************************************************************/
t_Error ESDHC_Reset(t_Handle h_Esdhc);

/**************************************************************************//**
 @Function      ESDHC_HighSpeedEnable

 @Description   Change the controller SDCLK clock frequency to high speed.
 
 @Param[in]     h_Esdhc - The handle to the eSDHC object.

 @Return        E_OK - if action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().
 @Cautions      Use this routine for high speed SD and MMC only.
 @Cautions      Call ESDHC_IsReadyForTransfer() before using this routine.
*//***************************************************************************/
t_Error ESDHC_HighSpeedEnable(t_Handle h_Esdhc);

/**************************************************************************//**
 @Function      ESDHC_HighSpeedDisable

 @Description   Change the controller SDCLK clock frequency to low speed.
 
 @Param[in]     h_Esdhc - The handle to the eSDHC object.

 @Return        E_OK    - If action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().
 @Cautions      Use this routine for high speed SD only
 @Cautions      Call ESDHC_IsReadyForTransfer() before using this routine.
*//***************************************************************************/
t_Error ESDHC_HighSpeedDisable(t_Handle h_Esdhc);

/**************************************************************************//**
 @Function      ESDHC_IsCardConnected

 @Description   Check if there is a memory card connected to the controller's slot

 @Param[in]     h_Esdhc - Handle to the eSDHC object.

 @Return        TRUE    - If card is currently connected. FALSE - otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
*//***************************************************************************/
bool ESDHC_IsCardConnected (t_Handle h_Esdhc);

/**************************************************************************//**
 @Function      ESDHC_GetCardSize

 @Description   Returnes the car'd memory capacity

 @Param[in]     h_Esdhc - Handle to the eSDHC object.
 @Param[out]    p_Size  - Pointer to the returned size.

 @Return        E_OK    - If action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
*//***************************************************************************/
t_Error ESDHC_GetCardSize(t_Handle h_Esdhc, uint32_t *p_Size);

/**************************************************************************//**
 @Function      ESDHC_GetCardType

 @Description   Returnes the car'd type

 @Param[in]     h_Esdhc - Handle to the eSDHC object.

 @Return        e_SdhcCardType - Card's type.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
*//***************************************************************************/
e_SdhcCardType ESDHC_GetCardType (t_Handle h_Esdhc);


/**************************************************************************//**
 @Function      ESDHC_GetMaxBlockSize

 @Description   Returnes the maximum block size

 @Param[in]     h_Esdhc - Handle to the eSDHC object.

 @Return        uint32_t - Maximum block size.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
*//***************************************************************************/
uint32_t ESDHC_GetMaxBlockSize (t_Handle h_Esdhc);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      ESDHC_DumpRegs

 @Description   Dumps the memory of ESDHC module

 @Param[in]     h_Esdhc - Handle to the eSDHC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ESDHC_DumpRegs(t_Handle h_Esdhc);
#endif /* DEBUG_ERRORS */

/** @} */ /* end of esdhc_control_grp */

/**************************************************************************//**
 @Group         esdhc_init_grp eSDHC Initialization Unit

 @Description   eSDHC initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   eSDHC configuration parameters structure.
*//***************************************************************************/
typedef struct t_EsdhcParam
{
    /* System integration parameters */
    uint32_t    baseAddress;
                /**< Base address of eSDHC registers in memory. */
    uint32_t    baseClk;
                /**< eSDHC controller input clock frequency (in MHz) */
    int         irq;
                /**< eSDHC interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle    h_App;
                /**< Handle to the upper layer handle */
    t_Error     (*f_TransferComplete)(t_Handle h_App, t_EsdhcTransferParam t_Data);
                /**< Callback routine for transfer complete interrupts in non blocking mode */
    t_Error     (*f_Exception)(t_Handle h_App, e_EsdhcErrorStatus status);
                /**< Callback invoked by driver on exception events on non
                     recoverable errors; if NULL, no exceptions are reported */
    t_Error     (*f_TransferError)(t_Handle h_App, t_EsdhcTransferParam t_Data);
                /**< Callback invoked by driver on data transfer error
                     If NULL, no exceptions are reported */

} t_EsdhcParam;

/**************************************************************************//**
 @Description   Defines the clocks operation mode.
 
                The clocks are: Peripheral clock, hclk for the DMA engine and
                to the CSB interface logic and the Controller clock.
*//***************************************************************************/
typedef enum e_EsdhcClocksMode
{
    e_ESDHC_INTERNALLY_GATED_OFF,       /**< The clocks will be automatically off when no
                                             transaction is on SD bus                           */
    e_ESDHC_NO_AUTOMATICALLY_GATED_OFF  /**< The clocks will always be active and no
                                             automatic gating is applied, thus SDCLK is active
                                             only except auto gating-off during buffer danger.  */
} e_EsdhcClocksMode;

/**************************************************************************//**
 @Description   Defines the Card detect polarity mode.
 
                Card detection pin By default the SDHC_CD signal should be tied
                high (meaning card present). If the card detect is programmed to 1,
                the detect signal should be tied low (card present, according to the standard).
*//***************************************************************************/
typedef enum e_EsdhcCardDetectPolarity
{
    e_ESDHC_DETECT_HIGH,        /**< Card detection pin high polarity. By default the SDHC_CD signal
                                     should be tied high (meaning card present)  */
    e_ESDHC_DETECT_LOW          /**< ard detection pin high polarity */
} e_EsdhcCardDetectPolarity;

/**************************************************************************//**
 @Function      ESDHC_Config

 @Description   Create the eSDHC object and returns its handle.
 
                Some parameters are set implicitly to driver's default values.
                This routine must be called prior to any other eSDHC routine.

 @Param[in]     p_EsdhcParam - Pointer to the configuration parameters structure.

 @Return        Handle to the initialized eSDHC object.
                This handle uniquely identifies the eSDHC object and shall be
                used when calling other eSDHC API routines.
*//***************************************************************************/
t_Handle ESDHC_Config(t_EsdhcParam  *p_EsdhcParam);

/**************************************************************************//**
 @Function      ESDHC_ConfigControllerClockMode

 @Description   Configure the eSDHC controller clock to work in the requested clock mode.[e_ESDHC_NO_AUTOMATICALLY_GATED_OFF]
 
                eSDHC's controller clock can work in 2 modes.
                e_ESDHC_INTERNALLY_GATED_OFF - The controller clock will be internally gated off
                e_ESDHC_NO_AUTOMATICALLY_GATED_OFF - the peripheral clock will
                always be active and no automatic gating is applied.
 
 @Param[in]     h_Esdhc   - The handle to the eSDHC object.
 @Param[in]     clockMode - Requested clock mode.

 @Return        E_OK - if action was successful, the occurred error otherwise.
 
 @Cautions      The controller clock will not be auto gated off if the peripheral clock is not gated off.

*//***************************************************************************/
t_Error ESDHC_ConfigControllerClockMode (t_Handle h_Esdhc, e_EsdhcClocksMode clockMode);

/**************************************************************************//**
 @Function      ESDHC_ConfigDmaClockMode

 @Description   Configure the eSDHC DMA clock to work in the requested clock mode.[e_ESDHC_NO_AUTOMATICALLY_GATED_OFF]
 
                eSDHC's DMA clock can work in 2 modes.
                e_ESDHC_INTERNALLY_GATED_OFF - hclk will be automatically off when no data transfer is on SD bus
                e_ESDHC_NO_AUTOMATICALLY_GATED_OFF - hclk will always be active and no automatic gating is applied.
 
 @Param[in]     h_Esdhc   - The handle to the eSDHC object.
 @Param[in]     clockMode - Requested clock mode.

 @Return        E_OK - if action was successful, the occurred error otherwise.
 
*//***************************************************************************/
t_Error ESDHC_ConfigDmaClockMode (t_Handle h_Esdhc, e_EsdhcClocksMode clockMode);

/**************************************************************************//**
 @Function      ESDHC_ConfigPeripheralClockMode

 @Description   Configure the eSDHC DMA clock to work in the requested clock mode.[e_ESDHC_NO_AUTOMATICALLY_GATED_OFF]
 
                eSDHC's DMA clock can work in 2 modes.
                e_ESDHC_INTERNALLY_GATED_OFF - the peripheral clock will be automatically off when no
                transaction is on SD bus.
                e_ESDHC_NO_AUTOMATICALLY_GATED_OFF - the peripheral clock will always be active and no
                automatic gating is applied
 
 @Param[in]     h_Esdhc   - The handle to the eSDHC object.
 @Param[in]     clockMode - Requested clock mode.

 @Return        E_OK - if action was successful, the occurred error otherwise.
 
*//***************************************************************************/
t_Error ESDHC_ConfigPeripheralClockMode (t_Handle h_Esdhc, e_EsdhcClocksMode clockMode);

/**************************************************************************//**
 @Function      ESDHC_ConfigCardDetectPolarity

 @Description   Configure thr polarity of the card detect signal [e_ESDHC_DETECT_HIGH]
 
 @Param[in]     h_Esdhc   - The handle to the eSDHC object.
 @Param[in]     polarity  - card detect signal polarity.

 @Return        E_OK - if action was successful, the occurred error otherwise.
 
*//***************************************************************************/
t_Error ESDHC_ConfigCardDetectPolarity (t_Handle h_Esdhc, e_EsdhcCardDetectPolarity polarity);

/**************************************************************************//**
 @Function      ESDHC_Init

 @Description   Initialization of eSDHC controller.
 
                This routine initializes the eSDHC controller according to the
                selected configuration.

 @Param[in]     h_Esdhc - Handle to the eSDHC object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
*//***************************************************************************/
t_Error ESDHC_Init(t_Handle h_Esdhc);

/** @} */ /* end of esdhc_init_grp */

/**************************************************************************//**
 @Group         esdhc_runtime_grp eSDHC Runtime Unit

 @Description   eSDHC runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ESDHC_write

 @Description   Write data from the local memory space to the card.
 
                 Write data from specific address at local memory space to a specific adress
                 at the connected card. the write can be done using DMA or CPU polling
                 in blocking or non-blocking mode. When using blocking mode,
                 when this routine retrun, the transfer is completed.

 
 @Param[in]     h_Esdhc     - The handle to the eSDHC object.
 @Param[in]     srcAddr     - Local memory address. The first address that contains
                              the data to write the card.
 @Param[in]     destAddr    - Card address. The first address that contains
                              the written data.
 @Param[in]     length      - The length of the data to write.
 @Param[in]     blockingMode- Indicates the blocking mode. TRUE for blocking
                              mode and FALSE for non-blocking mode.
 @Param[in]     transferMode- Indicates the transfer mode. use e_ESDHC_TRANS_MODE_INTERNAL_DMA for DMA transfer
                              and e_ESDHC_TRANS_MODE_CPU for CPU polling.
 @Param[in]     snoopEn     - Indicates if snooping is needed for the write transfer.
 
 @Return        E_OK - if action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().
 @Cautions      Call ESDHC_IsReadyForTransfer() before using this routine.
*//***************************************************************************/
t_Error ESDHC_Write(t_Handle             h_Esdhc,
                    uint32_t             srcAddr,
                    uint32_t             destAddr,
                    uint32_t             length,
                    bool                 blockingMode,
                    e_SdhcTransferMode transferMode,
                    bool                snoopEn);


/**************************************************************************//**
 @Function      ESDHC_read

 @Description   Read data from the card to local memory space.
 
                Read data from specific address at the card to a specific adress
                at the local memory. the read can be done using DMA or CPU polling
                in blocking or non-blocking mode.
 
 @Param[in]     h_Esdhc      - The handle to the eSDHC object.
 @Param[in]     startAddr    - Card address to start reading from
 @Param[in]     destAddr     - Local memory address. The first address that contains
                               the data from the card after the reading.
 @Param[in]     length       - The length of the data to read.
 @Param[in]     blockingMode - Indicates the blocking mode. TRUE for blocking
                               mode and FALSE for non-blocking mode.
 @Param[in]     transferMode - Indicates the transfer mode. use e_ESDHC_TRANS_MODE_INTERNAL_DMA for DMA transfer
                               and e_ESDHC_TRANS_MODE_CPU for CPU polling.
 @Param[in]     snoopEn      - Indicates if snooping is needed for the read transfer.
 
 @Return        E_OK - if action was successful, the occurred error otherwise.

 @Cautions      h_Esdhc must have been previously created with ESDHC_Config().
 @Cautions      This routine must be called after ESDHC_Init().
 @Cautions      Call ESDHC_IsReadyForTransfer() before using this routine.
*//***************************************************************************/
t_Error ESDHC_Read(t_Handle             h_Esdhc,
                   uint32_t             startAddr,
                   uint32_t             destAddr,
                   uint32_t             length,
                   bool                 blockingMode,
                   e_SdhcTransferMode   transferMode,
                   bool                 snoopEn);

 
/** @} */ /* end of esdhc_runtime_grp */
/** @} */ /* end of esdhc_grp */

#endif /*__ESDHC_EXT_H*/
