/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          hmac_ext.h

 @Description   structures and definitions for operating HMAC creation.

*//***************************************************************************/
#ifndef __HMAC_EXT_H
#define __HMAC_EXT_H


#include "md5_ext.h"
#include "sha1_ext.h"
#include "sha256_ext.h"
#include "crypt_ext.h"

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_hmac SEC HMAC

 @Description   SEC HMAC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_hmac_init SEC HMAC Initialization Unit

 @Description   SEC HMAC initialization functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      hmac_alloc_ctx

 @Description   Allocate memory for the hmac ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Return        ctx - ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle hmac_alloc_ctx();

/**************************************************************************//**
 @Function      hmac_init

 @Description   Initialize some fields in the ctx strucutre.

				The routine initializes the ctx AlgName field with
                the required hash algorithm name and initializes the
                padIdgs field to default.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx         - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     hashAlgType - the required hash algorithm type.

 @Return        None.
*//***************************************************************************/
void     hmac_init(void *ctx, e_HashAlgType hashAlgType);

/**************************************************************************//**
 @Function      hmac_setpad

 @Description   The routine set the AutoPad variable in the ctx structure.

				The routine set the AutoPad variable in the ctx structure
                to indicate that an Auto Pad option is required.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void     hmac_setpad(void *ctx);

/**************************************************************************//**
 @Function      hmac_update

 @Description   The routine set the data source and its length in the hmac context structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     data    - the input source handler.
 @Param[in]     len     - the input length.

 @Return        None.
*//***************************************************************************/
void     hmac_update(void *ctx, const unsigned char *data, unsigned int len);

/**************************************************************************//**
 @Function      hmac_update_ctx

 @Description   The routine sets the key and its length in the hmac ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     key     - key handler.
 @Param[in]     keyLen  - key length.

 @Return        None.
*//***************************************************************************/
void     hmac_update_ctx(void *ctx, unsigned char *key, unsigned int keyLen);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_hmac_runtime SEC HMAC Runtime Unit

 @Description   SEC HMAC Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      hmac_final

 @Description   The routine fills in and operate a specific request.

			    The routine fills in a specific request (according
                to the hash algorithm type) with the hmac ctx
                structure fields and then calls to ioctl function
                in order to process the specific request.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     out - Handler of the result of the HMAC calculation.

 @Return        None.
*//***************************************************************************/
void     hmac_final(void *ctx, unsigned char *out);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec hmac */
/** @} */ /* end of group sec */

#endif
