/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          ima_ext.h

 @Description   IMA Unit API

 @Cautions      None.
 *//***************************************************************************/

#ifndef __IMA_EXT_H
#define __IMA_EXT_H

#ifdef __cplusplus
extern "C"{
#endif

#include "std_ext.h"
#include "error_ext.h"
#include "ucc_atm_ext.h"
#include "ima_lld_ext.h"




/**************************************************************************//**
@Group         ima_api IMA Stack Application Programming Interface

@Description   IMA API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
@Group         ima_obj IMA Object Application Programming Interface

@Description   IMA object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
@Group         ima_obj_init IMA Object Initialization Unit

@Description   IMA object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Typedefs for function callbacks
*//***************************************************************************/
/** typedef for one argument callback functions */
typedef void (* t_ImaCallbackOneArg)(t_Handle h_Arg);

/** typedef for two arguments callback functions */
typedef void (* t_ImaCallbackTwoArgs)(t_Handle h_Arg0, t_Handle h_Arg1);

/** typedef for the MTC event handler*/
typedef void (* t_ImaMtcEventHandler)(t_Handle h_ImaTc, uint16_t event);

/** typedef for the TC sublayer SYNC IN/OUT event handler */
typedef void (* t_ImaTcRxSyncInOutEventHandler)(t_Handle h_Link);

/** typedef for the PMD sublayer LOS ON/OFF event handler */
typedef void (* t_ImaPmdRxLosOnOffEventHandler)(t_Handle h_Link);

/**************************************************************************//**
 @Description   IMA Physical Link Type
*//***************************************************************************/
typedef enum {
	e_IMA_PHYSICAL_LINK_DS1 = 0,
	e_IMA_PHYSICAL_LINK_E1_MULTIFRAME,
	e_IMA_PHYSICAL_LINK_E1_G704
} e_ImaPhysicalLinkType;

/**************************************************************************//**
 @Description   IMA LINK Failure Alarm Type enum
*//***************************************************************************/
typedef enum {
	e_LINK_FAILURE_ALARM_LIF             = 1 << 0,
	e_LINK_FAILURE_ALARM_LODS            = 1 << 1,
	e_LINK_FAILURE_ALARM_RFI_IMA         = 1 << 2,
	e_LINK_FAILURE_ALARM_TX_MISCONNECTED = 1 << 3,
	e_LINK_FAILURE_ALARM_RX_MISCONNECTED = 1 << 4,
	e_LINK_FAILURE_ALARM_TX_FAULT        = 1 << 5,
	e_LINK_FAILURE_ALARM_RX_FAULT        = 1 << 6,
	e_LINK_FAILURE_ALARM_TX_UNUSABLE_FE  = 1 << 7,
	e_LINK_FAILURE_ALARM_RX_UNUSABLE_FE  = 1 << 8
} e_ImaLinkFailureAlarmType;

/**************************************************************************//**
 @Description   IMA GROUP Failure Alarm Type enum
*//***************************************************************************/
typedef enum {
	e_GROUP_FAILURE_ALARM_STARTUP_FE            = 1 << 0,
	e_GROUP_FAILURE_ALARM_CONFIG_ABORTED        = 1 << 1,
	e_GROUP_FAILURE_ALARM_CONFIG_ABORTED_FE     = 1 << 2,
	e_GROUP_FAILURE_ALARM_INSUFFICIENT_LINKS    = 1 << 3,
	e_GROUP_FAILURE_ALARM_INSUFFICIENT_LINKS_FE = 1 << 4,
	e_GROUP_FAILURE_ALARM_BLOCKED_FE            = 1 << 5,
	e_GROUP_FAILURE_ALARM_GR_TIMING_MISMATCH    = 1 << 6
} e_ImaGroupFailureAlarmType;

/**************************************************************************//**
 @Description   IMA LINK PM Indication Type enum
*//***************************************************************************/
typedef enum {
	e_LINK_PM_INDICATION_LINK_RX_ACTIVATED = 0,
	e_LINK_PM_INDICATION_LINK_RX_DEACTIVATED,
	e_LINK_PM_INDICATION_LINK_TX_ACTIVATED,
	e_LINK_PM_INDICATION_LINK_TX_DEACTIVATED
} e_ImaLinkPmIndicationType;

/**************************************************************************//**
 @Description   IMA GROUP PM Indication Type enum
*//***************************************************************************/
typedef enum {
	e_GROUP_PM_INDICATION_GTSM_UP = 0,
	e_GROUP_PM_INDICATION_GTSM_DOWN
} e_ImaGroupPmIndicationType;

/** typedef for group and link failure alarms callback functions */
typedef t_Error (* t_ImaLinkFailureAlarmCallback)(t_Handle h_Link, e_ImaLinkFailureAlarmType Alarm, bool OnOff, t_Handle h_UserArg);
typedef t_Error (* t_ImaGroupFailureAlarmCallback)(t_Handle h_Group, e_ImaGroupFailureAlarmType Alarm, bool OnOff, t_Handle h_UserArg);

/** typedef for group and link PM indications callback functions */
typedef t_Error (* t_ImaLinkPmIndicationCallback)(t_Handle h_Link, e_ImaLinkPmIndicationType Indication, t_Handle h_UserArg);
typedef t_Error (* t_ImaGroupPmIndicationCallback)(t_Handle h_Group, e_ImaGroupPmIndicationType Indication, t_Handle h_UserArg);

/**************************************************************************//**
 @Description   IMA PMD Sublayer Interface
*//***************************************************************************/
typedef bool (* t_ImaPmdRxInSync)(t_Handle h_Pmd);

typedef struct {
	t_ImaPmdRxInSync f_PmdRxInSync; /**< Returns FALSE if the PMD sublayer currently has the Loss Of Signal (LOS) defect, otherwise it returns TRUE */
} t_ImaPmdInterface;

/**************************************************************************//**
 @Description   IMA object initialization structure
*//***************************************************************************/
typedef struct
{
	e_ImaPhysicalLinkType   LinkType;               /**< Type of the physical links of the IMA unit */
	uint8_t                 NrGroups;               /**< Number of groups of the IMA unit (it should not exceed the number of groups supported by the IMA microcode) */
	uint8_t                 NrPhyLinks;             /**< Number of links of the IMA unit (it should not exceed the number of links supported by the IMA microcode) */
	t_Handle                p_ChipId;               /**< Handle to the QE object */
	t_Handle                p_UccAtmId;             /**< Handle to the UCC object used by the IMA unit */
    e_UccAtmIntQ            IcpChannelsIntQueue;    /**< UCC ATM interrupt queue used by the ATM channels that are bound to the IMA groups of this IMA unit */
    uint32_t                icpMemAttributes;       /**< Memory attributes mask for ICP data buffers */
    uint8_t                 pramMemId;              /**< Parameter RAM partition ID */
    uint8_t     			imaExtStructMemId; 		/**< Memory Id that will be used by the LLD in order to define the external memory used in drivers */
    uint32_t    			imaExtStructMemAttr; 	/**< Memory Attribute ID defines if the BUS beeing in use */
    t_BufferPoolInfo        icpBufferPoolInfo;      /**< Buffer pool information for ICP buffers */
	t_ImaTcInterface        *p_TcInterface;         /**< Interface to the external TC sublayer (should be set to NULL if the internal TC sublayer (i.e. Serial ATM microcode) is used) */
	t_ImaPmdInterface       *p_PmdInterface;        /**< Interface to the PMD sublayer, if it is exercised on an interrupt request basis (otherwise, this should be set to NULL) */\
	bool					isTcOverUcc;			/**< Interface to Internal TC sublayer. If not set the TC layer will work on top of MCC */\
} t_ImaParams;

/**************************************************************************//**
 @Description   IMA status structure
*//***************************************************************************/
typedef struct {
	e_ImaPhysicalLinkType LinkType; /**< Type of the physical links of the IMA unit */
    uint32_t NrConfiguredLinks;     /**< Number of groups of the IMA unit */
    uint32_t NrConfiguredGroups;    /**< Number of links of the IMA unit */
    bool InternalTcSublayer;        /**< Internal / External TC sublayer */
} t_ImaStatus;


/**************************************************************************//**
    @par Description:
    IMA_Config - Configure an IMA unit

    @param[in]  p_Params - User supplied params for the IMA unit
 
    @return     Handle to the IMA unit
*//***************************************************************************/
t_Handle IMA_Config (t_ImaParams *p_Params);


/**************************************************************************//**
@Group         ima_obj_init_advanced IMA Object Advanced Initialization Unit

@Description   IMA object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_ConfigLinkFailureAlarmCallback - Configure the function to be called
              on each ocurrence of a link failure alarm

    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  Cb - Link failure alarm callback function
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigLinkFailureAlarmCallback(t_Handle h_Ima, t_ImaLinkFailureAlarmCallback Cb);


/**************************************************************************//**
    @par Description:
    IMA_ConfigLinkPmIndicationCallback - Configure the function to be called
              for each Plane Management (PM) indication related to an IMA link

    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  Cb - Link related PM indication callback function
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigLinkPmIndicationCallback(t_Handle h_Ima, t_ImaLinkPmIndicationCallback Cb);


/**************************************************************************//**
    @par Description:
    IMA_ConfigGroupFailureAlarmCallback - Configure the function to be called
              on each ocurrence of a group failure alarm

    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  Cb - Group failure alarm callback function
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigGroupFailureAlarmCallback(t_Handle h_Ima, t_ImaGroupFailureAlarmCallback Cb);


/**************************************************************************//**
    @par Description:
    IMA_ConfigGroupPmIndicationCallback - Configure the function to be called
              for each Plane Management (PM) indication related to an IMA group

    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  Cb - Group related PM indication callback function
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigGroupPmIndicationCallback(t_Handle h_Ima, t_ImaGroupPmIndicationCallback Cb);


/**************************************************************************//**
    @par Description:
    IMA_ConfigTxTestPatternIterationsPerLink - Configure the number of Tx Test
              Pattern procedure iterations to be performed for each link that
              is added to the IMA group or recovered within the IMA group
 
    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  TxTppIterations - number of iterations for Tx Test Pattern Procedure
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigTxTestPatternIterationsPerLink(t_Handle h_Ima, uint8_t TxTppIterations);


/**************************************************************************//**
    @par Description:
    IMA_ConfigTxQueueCellLength - Configure the length of the TX side Queue to
              be created for each IMA link (measured in number of ATM cells)
 
    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  TxQueueLength - Length of the TxQueue
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigTxQueueCellLength(t_Handle h_Ima, uint32_t TxQueueLength);


/**************************************************************************//**
    @par Description:
    IMA_ConfigRxDcbCellLength - Configure the length of the RX side Delay
              Compensation Buffer (DCB) to be created fpr each IMA link
              (measured in number of ATM cells)
 
    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  RxDcbLength - Length of the RxDCB
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_ConfigRxDcbCellLength(t_Handle h_Ima, uint32_t RxDcbLength);


/** @} */ /* end of ima_obj_init_advanced */


/**************************************************************************//**
    @par Description:
    IMA_Init - Initialize the IMA unit

    @param[in]  h_Ima - Handle to the IMA unit
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_Init(t_Handle h_Ima);


/**************************************************************************//**
    @par Description:
    IMA_Free - Free the IMA unit object

    @param[in]  h_Ima - Handle to the IMA unit
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_Free(t_Handle h_Ima);


/** @} */ /* end of ima_obj_init */


/**************************************************************************//**
@Group         ima_obj_ctl IMA Object Control Unit

@Description   IMA object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_GetMtcEventHandler -  IMA - TC Hidden API - Get the TC sublayer event
                                 handler. This function should be called only when
                                 the internal TC sublayer (Serial ATM ucode) is used

    @return     Event handler
*//***************************************************************************/
t_ImaMtcEventHandler IMA_GetMtcEventHandler(void);


/**************************************************************************//**
    @par Description:
    IMA_GetTcRxSyncInOutEventHandler -  IMA - TC Hidden API - Get the TC sublayer event
                                 handler for the event signaling that the Cell Delineation
                                 State Machine tranitioned in or out of the SYNC state.
                                 This function should be called only when an external
                                 TC sublayer (external DS1/E1 PHY with TC sublayer
                                 capability) is used.
 
    @return     Event handler
*//***************************************************************************/
t_ImaTcRxSyncInOutEventHandler IMA_GetTcRxSyncInOutEventHandler(void);


/**************************************************************************//**
    @par Description:
    IMA_GetPmdRxLosOnOffEventHandler -  IMA - PMD Hidden API - Get the PMD sublayer event
                                 handler for the LOS ON/OFF event when the PMD sublayer is
                                 handled on an interrupt request basis
 
    @return     Event handler
*//***************************************************************************/
t_ImaPmdRxLosOnOffEventHandler IMA_GetPmdRxLosOnOffEventHandler(void);


/**************************************************************************//**
    @par Description:
    IMA_GetStatus -  Status and Statistics API - Get the IMA unit status

    @param[in]  h_Ima - Handle to the IMA unit
    @param[out] p_ImaStatus - Pointer to a structure where the status will be copied
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GetStatus(t_Handle h_Ima, t_ImaStatus *p_ImaStatus);
/**************************************************************************//**
    @par Description:
    IMA_SetImaVersion -  Configuration of the IMA version. The user can specify
    which IMA version should be used at the NE and FE.

    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  ImaVer - The Ima version that should be configured
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_SetImaVersion(t_Handle h_Ima, e_ImaVersion ImaVer);

t_Error IMA_ConfigDifferentialDelay(t_Handle h_Ima, uint32_t differentialDelay);

/** @} */ /* end of ima_obj_ctl */


/** @} */ /* end of ima_obj */


/**************************************************************************//**
@Group         ima_group_obj IMA GROUP Object Application Programming Interface

@Description   IMA GROUP object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
@Group         ima_group_obj_init IMA GROUP Object Initialization Unit

@Description   IMA GROUP object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA GROUP initialization structure
*//***************************************************************************/
typedef struct {
	uint8_t ImaId;                  /**< IMA ID for this IMA group */
	e_ImaTxClockMode TxClockMode;   /**< TX Clock Mode */
    t_Handle h_AtmPort;             /**< ATM port of this IMA group */
} t_ImaGroupParams;


/**************************************************************************//**
 @Description   IMA GROUP state enum
*//***************************************************************************/
typedef enum {
	e_IMA_GROUP_STATE_NOT_CONFIGURED = 0,
	e_IMA_GROUP_STATE_STARTUP,
	e_IMA_GROUP_STATE_STARTUP_ACK,
	e_IMA_GROUP_STATE_CONFIG_ABORTED,
	e_IMA_GROUP_STATE_INSUFFICIENT_LINKS,
	e_IMA_GROUP_STATE_BLOCKED,
	e_IMA_GROUP_STATE_OPERATIONAL
} e_ImaGroupState;


/**************************************************************************//**
 @Description   IMA GTSM state enum
*//***************************************************************************/
typedef enum {
	e_IMA_GROUP_TRAFFIC_STATE_DOWN = 0,
	e_IMA_GROUP_TRAFFIC_STATE_UP
} e_ImaGroupTrafficState;


/**************************************************************************//**
 @Description   IMA GROUP acceptance params
*//***************************************************************************/
typedef struct {
	e_ImaVersion ImaVersion;          /**< IMA Version (it should be set to IMA version 1.1) */
	e_ImaGroupSymMode SymmMode;       /**< Group Symmetry Mode (it should be set to Symmetrical Configuration and Operation) */
	e_ImaFrameLength TxM;             /**< IMA Frame Length (it should be set to M=128) */
	uint8_t ImaId;                    /**< IMA ID */
	e_ImaTxClockMode TxClockMode;     /**< TX Clock Mode */
} t_ImaGroupAcceptanceParams;


/**************************************************************************//**
 @Description   IMA GROUP status
*//***************************************************************************/
typedef struct {
	t_Handle h_Ima;                             /**< Handle to the IMA unit this group belongs to */
	t_Handle h_Links[IMA_MAX_LINKS_PER_GROUP];  /**< Links that are part of this IMA group */
	uint8_t NrLinks;                            /**< Number of links that are part of this IMA group */
	
	/* Ne */
	e_ImaGroupState NeGroupState;               /**< NE GSM state */
	e_ImaGroupTrafficState GroupTrafficState;   /**< NE GTSM state */
   	t_ImaGroupAcceptanceParams NeParams;        /**< NE Group Parameters */
	bool Inhibited;                             /**< TRUE is this group is currently inhibited */
	uint8_t Ptx;                                /**< Current value of Ptx */
	uint32_t FailureAlarms;                     /**< Bitmask of the currenly asserted Failure Alarms */
    uint8_t TxLinkTrl;                          /**< The link ID of the current TxTRL link of this group */
 
	/* Fe */
	e_ImaIcpGroupState FeGroupState;            /**< FE GSM state */
	t_ImaGroupAcceptanceParams FeParams;        /**< FE Group Parameters */
} t_ImaGroupStatus;


/**************************************************************************//**
 @Description   IMA GROUP statistics
*//***************************************************************************/
typedef struct {
	uint32_t GR_UAS_IMA;
	uint32_t GR_FC;
	uint32_t GR_FC_FE;
} t_ImaGroupStatistics;


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Config - Configure an IMA group

    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  p_GroupParams - User supplied params for the IMA group
 
    @return     Handle to the IMA group
*//***************************************************************************/
t_Handle IMA_GROUP_Config(t_Handle h_Ima, t_ImaGroupParams *p_GroupParams);


/**************************************************************************//**
@Group         ima_group_obj_init_advanced IMA GROUP Object Advanced Initialization Unit

@Description   IMA GROUP object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_GROUP_ConfigFailureAlarmsUserArg - Configure the user argument to be supplied
                    to the Failure Alarms callback function when called for this IMA
                    group

    @param[in]  h_Group - Handle to the IMA group
    @param[in]  h_UserArg - User argument for this IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_ConfigFailureAlarmsUserArg(t_Handle h_Group, t_Handle h_UserArg);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_ConfigPmIndicationsUserArg - Configure the user argument to be supplied
                    to the Plane Management (PM) Indications callback function when
                    called for this IMA group

    @param[in]  h_Group - Handle to the IMA group
    @param[in]  h_UserArg - User argument for this IMA group
 
    @return     Handle to the IMA group
*//***************************************************************************/
t_Error IMA_GROUP_ConfigPmIndicationsUserArg(t_Handle h_Group, t_Handle h_UserArg);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_ConfigIfsm - Configure the IMA Frame Synchronization Mechanism (IFSM)
              parameters to be used for this IMA group

    @param[in]  h_Group - Handle to the IMA group
    @param[in]  p_ImaGroupIfsmParam - User supplied params for the IMA group IFSM
 
    @return     Handle to the IMA group
*//***************************************************************************/
t_Error IMA_GROUP_ConfigIfsm(t_Handle h_Group, t_ImaGroupIfsmParam *p_ImaGroupIfsmParam);


/** @} */ /* end of ima_group_obj_init_advanced */


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Init - Initialize the IMA group

    @param[in]  h_Group - Handle to the IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_Init(t_Handle h_Group);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Free - Free the IMA group
 
    @param[in]  h_Group - Handle to the IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_Free(t_Handle h_Group);


/** @} */ /* end of ima_group_obj_init */


/**************************************************************************//**
@Group         ima_group_obj_run IMA GROUP Object Runtime Unit

@Description   IMA GROUP object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_GROUP_AddLinks -  Management API - Add links to an IMA group

    @param[in] 	h_Group - Handle to the IMA group
    @param[in]	LinkArray - Array of handlers to the IMA links to be added to the group
    @param[in]	LinkArraySize - Number of LinkArray elements
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_AddLinks(t_Handle h_Group, t_Handle *LinkArray, uint8_t LinkArraySize);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_DeleteLinks -  Management API - Delete links from an IMA group

    @param[in]  LinkArray - Array of handlers to the IMA links to be deleted from the group
   	@param[in]  LinkArraySize - Number of LinkArray elements

    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_DeleteLinks(t_Handle *LinkArray, uint8_t LinkArraySize);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_InhibitLinks - Management API - Inihibit links within an IMA group

    @param[in]  LinkArray - Array of handlers to the IMA links to be inhibited within the group
    @param[in]	LinkArraySize - Number of LinkArray elements
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_InhibitLinks(t_Handle *LinkArray, uint8_t LinkArraySize);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_UninhibitLinks - Management API - Uninihibit links within an IMA group

    @param[in]  LinkArray - Array of handlers to the IMA links to be uninhibited within the group
    @param[in]	LinkArraySize - Number of LinkArray elements
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_UninhibitLinks(t_Handle *LinkArray, uint8_t LinkArraySize);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Inhibit -  Management API - Inhibit an IMA group

    @param[in]  h_Group - Handle to the IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_Inhibit(t_Handle h_Group);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Uninhibit - Management API - Uninhibit an IMA group
 
    @param[in]  h_Group - Handle to the IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_Uninhibit(t_Handle h_Group);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Restart - Management API - Restart an IMA group

    @param[in]  h_Group - Handle to the IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_Restart(t_Handle h_Group);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_Restart - Management API - Set the Ptx parameter of the IMA group

    @param[in]  h_Group - Handle to the IMA group
    @param[in]  Ptx - the minimum number of links required for the group operational state
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_UpdatePtx(t_Handle h_Group, uint8_t Ptx);


/** @} */ /* end of ima_group_obj_run */


/**************************************************************************//**
@Group         ima_group_obj_ctl IMA GROUP Object Control Unit

@Description   IMA GROUP object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_GROUP_GetStatus -  Status and Statistics API - Get the IMA group status

    @param[in]  h_Group - Handle to the IMA group
    @param[out] p_GroupStatus - Pointer to a structure where the status will be copied
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_GetStatus(t_Handle h_Group, t_ImaGroupStatus *p_GroupStatus);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_GetStatistics -  Status and Statistics API - Get the IMA group statistics

    @param[in]  h_Group - Handle to the IMA group
    @param[out]	p_GroupStatistics - Pointer to a structure where the statistics will be copied
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_GetStatistics(t_Handle h_Group, t_ImaGroupStatistics *p_GroupStatistics);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_ResetStatistics -  Status and Statistics API - Reset the IMA group statistics

    @param[in]  h_Group - Handle to the IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_GROUP_ResetStatistics(t_Handle h_Group);


/** @} */ /* end of ima_group_obj_ctl */


/** @} */ /* end of ima_group_obj */


/**************************************************************************//**
@Group         ima_group_port_obj IMA GROUP PORT Object Application Programming Interface

@Description   IMA GROUP PORT object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
@Group         ima_group_port_obj_init IMA GROUP PORT Object Initialization Unit

@Description   IMA GROUP PORT object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA GROUP PORT initialization structure
*//***************************************************************************/
typedef t_ImaLldGroupPortParam t_ImaGroupPortParam;


/**************************************************************************//**
    @par Description:
        IMA Group Port configuration function

    @par Important Notes:
        This function must be called before any other routine of this object
 
    @param[in]  p_PortParams - IMA Group Port parameters provided by the upper layer

    @return     the handle of the IMA GROUP PORT object
*//***************************************************************************/
t_Handle IMA_GROUP_PORT_Config(t_ImaGroupPortParam *p_PortParams);


/**************************************************************************//**
    @par Description:
        IMA Group Port initialization function

    @par Important Notes:
        This function must be called after the Group Port config function

    @param[in]  h_ImaGroupPort - the handle of the IMA GROUP PORT object

    @return     error code
*//***************************************************************************/
t_Error IMA_GROUP_PORT_Init(t_Handle h_ImaGroupPort);


/**************************************************************************//**
    @par Description:
        Release memory and physical resources of an IMA group port
 
    @param[in]  h_ImaGroupPort - handle of the IMA GROUP PORT to be released
 
    @return     error code
*//***************************************************************************/
t_Error IMA_GROUP_PORT_Free(t_Handle h_ImaGroupPort);


/** @} */ /* end of ima_group_port_obj_init */


/** @} */ /* end of ima_group_port_obj */


/**************************************************************************//**
@Group         ima_link_obj IMA LINK Object Application Programming Interface

@Description   IMA LINK object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
@Group         ima_link_obj_init IMA LINK Object Initialization Unit

@Description   IMA object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA LINK initialization structure
*//***************************************************************************/
typedef struct {
    uint32_t PhyLinkIndex;           /**< Physical Link Index */
} t_ImaLinkParams;


/**************************************************************************//**
 @Description   IMA LINK Defect Type
*//***************************************************************************/
typedef enum {
	e_IMA_DEFECT_TYPE_PMD  = 0x01,
	e_IMA_DEFECT_TYPE_TC   = 0x02,
	e_IMA_DEFECT_TYPE_IMA  = 0x04
} e_ImaDefectType;


/**************************************************************************//**
 @Description   IMA LINK Tx-side defect type
*//***************************************************************************/
typedef enum {
	/* PMD */
	e_IMA_TX_LINK_DEFECT_PMD_FAULT = 1 << 0,
	
	/* TC */
	e_IMA_TX_LINK_DEFECT_TC_FAULT  = 1 << 1,
	
	/* IMA */
	e_IMA_TX_LINK_DEFECT_IMA_FAULT = 1 << 2
} e_ImaTxLinkDefect;


/**************************************************************************//**
 @Description   IMA LINK Rx-side defect type
*//***************************************************************************/
typedef enum {
	/* PMD */
	e_IMA_RX_LINK_DEFECT_LOS       = 1 << 0,
	e_IMA_RX_LINK_DEFECT_PMD_FAULT = 1 << 1,

	/* TC */
	e_IMA_RX_LINK_DEFECT_LCD       = 1 << 2,
	e_IMA_RX_LINK_DEFECT_TC_FAULT  = 1 << 3,
	
	/* IMA */
	e_IMA_RX_LINK_DEFECT_LIF       = 1 << 4,
	e_IMA_RX_LINK_DEFECT_LODS	   = 1 << 5,
	e_IMA_RX_LINK_DEFECT_IMA_FAULT = 1 << 6
} e_ImaRxLinkDefect;


/**************************************************************************//**
 @Description   IMA LINK Tx state
*//***************************************************************************/
typedef enum {
	e_IMA_TX_LINK_STATE_UNASSIGNED = 0,
	e_IMA_TX_LINK_STATE_DELETED,
	e_IMA_TX_LINK_STATE_UNUSABLE,
	e_IMA_TX_LINK_STATE_USABLE,
	e_IMA_TX_LINK_STATE_ACTIVE
} e_ImaTxLinkState;


/**************************************************************************//**
 @Description   IMA LINK Tx-side action type
*//***************************************************************************/
typedef enum {
    e_TX_LINK_OFF = 0, /* Link not in Ucode */
    e_TX_LINK_FILLER,  /* Link in Ucode, Tx=Filler */
    e_TX_LINK_ON       /* Link in Ucode, Tx=On */
} e_ImaTxLinkAction;


/**************************************************************************//**
 @Description   IMA LINK Rx state
*//***************************************************************************/
typedef enum {
	e_IMA_RX_LINK_STATE_UNASSIGNED = 0,
	e_IMA_RX_LINK_STATE_DELETED,
	e_IMA_RX_LINK_STATE_UNUSABLE,
	e_IMA_RX_LINK_STATE_BLOCKING,
	e_IMA_RX_LINK_STATE_USABLE,
	e_IMA_RX_LINK_STATE_ACTIVE
} e_ImaRxLinkState;


/**************************************************************************//**
 @Description   IMA LINK Rx-side action type
*//***************************************************************************/
typedef enum {
    e_RX_LINK_OFF = 0, /* Link not in Ucode */
    e_RX_LINK_MONITOR, /* Link in Ucode, Rx=Monitor */
    e_RX_LINK_ON       /* Link in Ucode, Rx=On */
} e_ImaRxLinkAction;


/**************************************************************************//**
 @Description   IMA LINK status
*//***************************************************************************/
typedef struct {
    /* Link information */
    uint32_t PhyLinkIndex;             /**< Physical Link Index */
    t_Handle h_Group;                  /**< Handle to the IMA group this link belongs to */
	bool Inhibited;                    /**< TRUE if this link is currently inhibited */
	uint32_t FailureAlarms;            /**< Mask of the currently asserted Failure Alarms for this link */
	
	/* Link Tx-side information */
	e_ImaTxLinkState TxLinkState;      /**< NE TxLSM state */
	uint8_t TxLid;                     /**< TX Link ID */
	uint8_t TxIcpOffset;               /**< TX ICP Cell Offset */
	uint32_t TxDefects;                /**< Mask of the currently asserted TX-side link defects */
	
	/* Link Rx-side information */
	e_ImaRxLinkState RxLinkState;      /**< NE RxLSM state */
	uint8_t RxLid;                     /**< RX Link ID */
	uint8_t RxIcpOffset;               /**< RX ICP Cell Offset */
	uint32_t RxDefects;                /**< Mask of the currently asserted RX-side link defects */
	
	/* Link Fe information */
	e_ImaIcpLinkState FeTxLinkState;   /**< NE TxLSM state */
	e_ImaIcpLinkState FeRxLinkState;   /**< NE RxLSM state */
} t_ImaLinkStatus;


/**************************************************************************//**
 @Description   IMA LINK statistics
*//***************************************************************************/
typedef struct {
	uint32_t IV_IMA;
	uint32_t OIF_IMA;
	uint32_t SES_IMA;
	uint32_t SES_IMA_FE;
	uint32_t UAS_IMA;
	uint32_t UAS_IMA_FE;
	uint32_t Tx_UUS_IMA;
	uint32_t Rx_UUS_IMA;
	uint32_t Tx_UUS_IMA_FE;
	uint32_t Rx_UUS_IMA_FE;
	uint32_t Tx_FC;
	uint32_t Rx_FC;
	uint32_t Tx_FC_FE;
	uint32_t Rx_FC_FE;
	uint32_t Tx_Stuff_IMA;
	uint32_t Rx_Stuff_IMA;
	uint32_t Tx_IcpCnt;
	uint32_t Tx_FillerCnt;
	uint32_t Tx_UserCnt;
	uint32_t Rx_IcpCnt;
	uint32_t Rx_FillerCnt;
	uint32_t Rx_UserCnt;
} t_ImaLinkStatistics;


/**************************************************************************//**
    @par Description:
    IMA_LINK_Config - Configure an IMA link
 
    @param[in]  h_Ima - Handle to the IMA unit
    @param[in]  p_LinkParams - User supplied parameters for the IMA link
 
    @return     Handle to the IMA link
*//***************************************************************************/
t_Handle IMA_LINK_Config(t_Handle h_Ima, t_ImaLinkParams *p_LinkParams);


/**************************************************************************//**
@Group         ima_link_obj_init_advanced IMA LINK Object Advanced Initialization Unit

@Description   IMA object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_GROUP_ConfigFailureAlarmsUserArg - Configure the user argument to be supplied
                    to the Failure Alarms callback function when called for this IMA
                    group

    @param[in]  h_Link - Handle to the IMA link
    @param[in]  h_UserArg - User argument for this IMA group
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_ConfigFailureAlarmsUserArg(t_Handle h_Link, t_Handle h_UserArg);


/**************************************************************************//**
    @par Description:
    IMA_GROUP_ConfigPmIndicationsUserArg - Configure the user argument to be supplied
                    to the Plane Management (PM) Indications callback function when
                    called for this IMA group

    @param[in]  h_Link - Handle to the IMA link
    @param[in]  h_UserArg - User argument for the IMA link
 
    @return     Handle to the IMA group
*//***************************************************************************/
t_Error IMA_LINK_ConfigPmIndicationsUserArg(t_Handle h_Link, t_Handle h_UserArg);


/**************************************************************************//**
    @par Description:
    IMA_LINK_ConfigTcHandle - Get the handle to the TC sublayer entity (either internal
                              or external) associated to this IMA link
 
    @param[in]  h_Link - Handle to the IMA link
    @param[in]  h_Tc - Handle to the TC sublayer entity to be associated with the IMA link
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_ConfigTcHandle(t_Handle h_Link, t_Handle h_Tc);


/**************************************************************************//**
    @par Description:
    IMA_LINK_ConfigTcHandle - Get the handle to the PMD sublayer entity
                              associated to this IMA link
 
    @param[in]  h_Link - Handle to the IMA link
    @param[in]  h_Pmd - Handle to the PMD sublayer entity to be associated with the IMA link
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_ConfigPmdHandle(t_Handle h_Link, t_Handle h_Pmd);


/** @} */ /* end of ima_link_obj_init_advanced */


/**************************************************************************//**
    @par Description:
    IMA_LINK_Init -  Initialize an IMA link

    @param[in]  h_Link - Handle to the Ima link
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_Init(t_Handle h_Link);


/**************************************************************************//**
    @par Description:
    IMA_LINK_Free -  Free an IMA link

    @param[in]  h_Link - Handle to the Ima link
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_Free(t_Handle h_Link);


/** @} */ /* end of ima_link_obj_init */


/**************************************************************************//**
@Group         ima_link_obj_ctl IMA LINK Object Control Unit

@Description   IMA LINK object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
    IMA_LINK_GetStatus -  Status and Statistics API - Get the IMA link status
 
    @param[in]  h_Link - Handle to the IMA link
    @param[out]	p_LinkStatus - Pointer to a structure where the status will be copied
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_GetStatus(t_Handle h_Link, t_ImaLinkStatus* p_LinkStatus);


/**************************************************************************//**
    @par Description:
    IMA_LINK_GetStatistics -  Status and Statistics API - Get the IMA link statistics

    @param[in]  h_Link - Handle to the IMA link
    @param[out] p_LinkStatistics - Pointer to a structure where the link statistics will be copied
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_GetStatistics(t_Handle h_Link, t_ImaLinkStatistics *p_LinkStatistics);


/**************************************************************************//**
    @par Description:
    IMA_LINK_ResetStatistics -  Status and Statistics API - Reset the IMA link statistics

    @param[in]  h_Link - Handle to the IMA link
 
    @return     Error code
*//***************************************************************************/
t_Error IMA_LINK_ResetStatistics(t_Handle h_Link);


/** @} */ /* end of ima_link_obj_ctl */


/** @} */ /* end of ima_link_obj */


/** @} */ /* end of ima_api */


#ifdef __cplusplus
}
#endif

#endif
