/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          ima_lld_ext.h

 @Description   IMA LLD API - structures and functions.

 @Cautions      None.
*//***************************************************************************/

#ifndef __IMA_LLD_EXT_H
#define __IMA_LLD_EXT_H

#ifdef __cplusplus
extern "C"{
#endif


#include "std_ext.h"
#include "ucc_atm_ext.h"
#include "ucc_atm_port_ext.h"

/**************************************************************************//**
@Group         ima_lld_api IMA LLD Application Programming Interface

@Description   IMA LLD API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
@Group         ima_lld_obj IMA LLD Object Application Programming Interface

@Description   IMA LLD object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ima_lld_obj_init_unit IMA LLD Object Initialization Unit

 @Description   IMA LLD Object API Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Macros
*//***************************************************************************/
#define    IMA_MAX_LINKS_PER_GROUP  32 /** Maximum number of links per group supported by the IMA protocol */
#define    IMA_MAX_LINKS_IN_GROUP	16
#define    IMA_ENABLE_LINK_COUNTERS	1  /** Enable / Disable Link Counters for User/ICP/Filler Cells */
/**************************************************************************//**
 @Description   IMA ucode events
*//***************************************************************************/
typedef enum
{
    e_IMA_EVENT_IFSW = 0,
    e_IMA_EVENT_IFSD,
    e_IMA_EVENT_GDS,
    e_IMA_EVENT_LDS,
    e_IMA_EVENT_DCBO,
    e_IMA_EVENT_LS,
    e_IMA_EVENT_DSL,
    e_IMA_EVENT_TQO,
    e_IMA_EVENT_TQU
}e_ImaUcodeEvent;


/**************************************************************************//**
 @Description   IMA link IESM states
*//***************************************************************************/
typedef enum
{
	e_IMA_LINK_IESM_STATE_WORKING = 0,
	e_IMA_LINK_IESM_STATE_OIF,
	e_IMA_LINK_IESM_STATE_LIF
}e_ImaLinkIesmState;

/**************************************************************************//**
 @Description   IMA Version
*//***************************************************************************/
typedef enum
{
	e_IMA_VER_1_0 = 0,
	e_IMA_VER_1_1,
 	e_IMA_VER_UNKNOWN
 }e_ImaVersion;
/**************************************************************************//**
 @Description   IMA Group Config aborted reasons
*//***************************************************************************/
 typedef enum {
     e_IMA_CONFIG_ABORTED_NONE = 0,
     e_IMA_CONFIG_ABORTED_UNSUPPORTED_M,
     e_IMA_CONFIG_ABORTED_INCOMPATIBLE_GROUP_SYM,
     e_IMA_CONFIG_ABORTED_UNSUPPORTED_IMA_VER,
     e_IMA_CONFIG_ABORTED_OTHER
 } e_ImaGroupConfigAbortedReason;
/**************************************************************************//**
 @Description   IMA Group symetry
*//***************************************************************************/
 typedef enum {
     e_IMA_SYM_CFG_SYM_OP = 0,
     e_IMA_SYM_CFG_ASYM_OP,
     e_IMA_ASYM_CFG_ASYM_OP,
     e_IMA_GROUP_SYM_MODE_RESERVED
 } e_ImaGroupSymMode;
/**************************************************************************//**
 @Description   IMA frame length
*//***************************************************************************/
typedef enum {
     e_IMA_M_32  = 32,
     e_IMA_M_64  = 64,
     e_IMA_M_128 = 128,
     e_IMA_M_256 = 256,
     e_IMA_M_Default = e_IMA_M_128
 } e_ImaFrameLength;
/**************************************************************************//**
 @Description   IMA transmmit clock Independent / Common
*//***************************************************************************/
typedef enum {
     e_IMA_TX_CLOCK_ITC = 0,
     e_IMA_TX_CLOCK_CTC
 } e_ImaTxClockMode;
/**************************************************************************//**
 @Description   IMA FE Group States
*//***************************************************************************/
typedef enum {
     e_IMA_ICP_GROUP_START_UP = 0,
     e_IMA_ICP_GROUP_START_UP_ACK,
     e_IMA_ICP_GROUP_CONFIG_ABORTED,
     e_IMA_ICP_GROUP_INSUFFICIENT_LINKS,
     e_IMA_ICP_GROUP_BLOCKED,
     e_IMA_ICP_GROUP_OPERATIONAL
 } e_ImaIcpGroupState;
 /**************************************************************************//**
 @Description   IMA FE Link States
*//***************************************************************************/
 typedef enum {
     e_IMA_ICP_LINK_NOT_IN_GROUP = 0,
     e_IMA_ICP_LINK_UNUSABLE,
     e_IMA_ICP_LINK_USABLE,
     e_IMA_ICP_LINK_ACTIVE
 } e_ImaIcpLinkState;
/**************************************************************************//**
 @Description   IMA FE Unusable Link states
*//***************************************************************************/
 typedef enum {
     e_IMA_ICP_LINK_UNUSABLE_NONE = 0,
     e_IMA_ICP_LINK_UNUSABLE_NO_REASON,
     e_IMA_ICP_LINK_UNUSABLE_FAULT,
     e_IMA_ICP_LINK_UNUSABLE_MIS_CONNECTED,
     e_IMA_ICP_LINK_UNUSABLE_INHIBITED,
     e_IMA_ICP_LINK_UNUSABLE_FAILED
 } e_ImaIcpLinkUnusableReason;
 /**************************************************************************//**
 @Description   IMA FE Unusable Link states defects reporting
*//***************************************************************************/
typedef enum {
     e_IMA_RX_DEFECT_NONE = 0,
     e_IMA_RX_DEFECT_PHYSICAL,
     e_IMA_RX_DEFECT_LIF,
     e_IMA_RX_DEFECT_LODS
 } e_ImaIcpRxLinkDefect;
  /**************************************************************************//**
 @Description   IMA Stuffing information
*//***************************************************************************/
 typedef enum {
     e_IMA_LINK_STUFFING_NONE = 0,
     e_IMA_LINK_STUFFING0,
     e_IMA_LINK_STUFFING1,
     e_IMA_LINK_STUFFING2,
     e_IMA_LINK_STUFFING3,
     e_IMA_LINK_STUFFING4
 } e_ImaLinkStuffIndication;
/**************************************************************************//**
 @Description   IMA LLD Callback Function Types
*//***************************************************************************/
typedef void (*t_ImaLldEventHandler)(t_Handle h_Upper, e_ImaUcodeEvent event);
typedef t_Error (* t_ImaTcTxStart)(t_Handle h_Tc);
typedef t_Error (* t_ImaTcTxStop)(t_Handle h_Tc);
typedef t_Error (* t_ImaTcRxStart)(t_Handle h_Tc);
typedef t_Error (* t_ImaTcRxStop)(t_Handle h_Tc);
typedef bool (* t_ImaTcRxInSync)(t_Handle h_Tc);
typedef t_Error (* t_ImaTcGetUtopiaPhyAddr)(t_Handle h_Tc, uint32_t *p_UtopiaPhyAddr);
typedef t_Error (* t_ImaTcGetRxCellFifoSize)(t_Handle h_Tc, uint32_t *p_RxCellFifoSize);
/**************************************************************************//**
 @Description   IMA TC Sublayer Interface
*//***************************************************************************/
typedef struct {
	t_ImaTcTxStart f_TcTxStart;         /**< start the Tx side of the TC sublayer */
	t_ImaTcTxStop f_TcTxStop;           /**< stop the Tx side of the TC sublayer */
	t_ImaTcRxStart f_TcRxStart;         /**< start the Rx side of the TC sublayer */
	t_ImaTcRxStop f_TcRxStop;           /**< start the Rx side of the TC sublayer */
	t_ImaTcRxInSync f_TcRxInSync;       /**< check the SYNC state of the Rx side of the TC */
	t_ImaTcGetUtopiaPhyAddr f_TcGetUtopiaPhyAddr;   /**< get the PHY address of the TC */
	t_ImaTcGetRxCellFifoSize f_TcGetRxCellFifoSize; /**< get the size of the Rx FIFO of the TC */
} t_ImaTcInterface;

/**************************************************************************//**
 @Description   IMA LLD object initialization structure
*//***************************************************************************/
typedef struct
{
	t_Handle        h_Chip;         /**< a handle to the chip descriptor */
	t_Handle        h_ImaUpper;     /**< ptr to the IMA stuct of the upper layer (e.g. PM) */
	t_Handle        p_UccAtmId;     /**< a handle to the UCC */
    uint8_t         nPhyLinks;      /**< number of physical links to be alocated */
    uint8_t         nGroups;        /**< number of groups to be alocated */
	e_ImaVersion    ImaVer;         /**< IMA version 1.0 or 1.1 */

    t_ImaLldEventHandler    f_GroupEventHandler;    /**< RxLink Ucode Event Handler */
    t_ImaLldEventHandler    f_LinkEventHandler;     /**< RxLink Ucode Event Handler */
    t_RxStoreFunction       *f_RxIcpStore;          /**< callback for passing a received ICP cell to the upper layer */
    e_UccAtmIntQ            IcpChIntQueue;          /**< interrupt queue used by the AAL0 ICP channel */
    uint8_t                 pramMemId;              /**< Parameter RAM partition ID */
    uint32_t                icpMemAttributes;       /**< Memory attributes mask for ICP data buffers */
    t_BufferPoolInfo        icpBufferPoolInfo;      /**< Buffer pool information for ICP buffers */
    uint8_t     			imaLldExtStructMemId; 	/**< Memory Id that will be used by the LLD in order to define the external memory used in drivers */
    uint32_t    			imaLldExtStructMemAttr; /**< Memory Attribute ID defines if the BUSS beeing in use */
	t_ImaTcInterface        *p_TcInterface;
}t_ImaLldParam;


/**************************************************************************//**
 @Description   IMA ICP Reporting
*//***************************************************************************/
 #define IMA_ICP_MAX_LINKS_PER_GROUP 32
 #define IMA_ICP_CELL_BUF_SIZE       52  /* ATM cell buffer size in bytes */
 
 typedef struct {
     uint32_t AtmCellHeader;		/**< Atm Cell Header */
 	uint8_t OamLabel;				/**< Used to specify the version of IMA reported v1.0 or v1.1 */
 	uint8_t CellIdLinkId;			/**< Ima link Id reportd on IMA speciffic link */
 	uint8_t IFSN;
 	uint8_t IcpCellOffset;			/**< ICP cell offset reported for the IMA link*/
 
 	uint8_t LinkStuffIndication;	/**< Stuffing indication field */
 	uint8_t SCCI;					/**< SCCI value reported */
 	uint8_t ImaId;					/**< IMA ID reported by FE */
 	uint8_t GroupInfo;				/**< Group state information reported in the ICP cell*/
 
 	uint8_t TxTimingInfo;			/**< Timing information report CTC/ITC */
 	uint8_t TxTestCtl;				/**< Test Pattern control*/
 	uint8_t TxTestPattern;			/**< Test Pattern - Tx */
 	uint8_t RxTestPattern;			/**< Test Pattern - Rx */
 
 	uint8_t LinkInfo[IMA_ICP_MAX_LINKS_PER_GROUP]; /**< Link Array - where link info is reported */
 
 	uint8_t Unused;					/**< Unused ICP field */
 	uint8_t EndToEndChannel;		/**< End to End Channel */
 	uint16_t Crc10;					/**< CRC 10 data */
 } t_ImaIcpCell;
 
/**************************************************************************//**
    @par Description:
        IMA LLD configuration function

    @par Important Notes:
        This function must be called before any other routine for this module

    @param[in]  p_ImaLldParam - IMA LLD parameters provided by the upper layer

    @return     handle of the IMA LLD object
*//***************************************************************************/
t_Handle IMA_LLD_Config(t_ImaLldParam* p_ImaLldParam);


/**************************************************************************//**
 @Group         ima_lld_obj_init_advanced_unit IMA LLD Object Advanced Initialization Unit

 @Description   IMA LLD Object API Advanced Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Set memory partition ID and attributes for the external structures
        of IMA LLD, such as IMA Tx/Rx Queues. Default values are:
        [0 (default heap), MEMORY_ATTR_CACHEABLE]

    @par Important Notes:
        This function must be called after the config function

    @param[in]  h_ImaLld    - the handle of the IMA LLD object
    @param[in]  memId       - Memory partition ID
    @param[in]  memAttributes       - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

    @return     error or E_OK if success.
*//***************************************************************************/
t_Error IMA_LLD_ConfigFirmwareExternalStructsMemory(t_Handle    h_ImaLld,
                                                    uint8_t     memId,
                                                    uint32_t    memAttributes);


/**************************************************************************//**
    @par Description:
        Config IMA LLD Tx queues length

    @par Important Notes:
        This function must be called after the config function

    @param[in]  h_ImaLld - the handle of the IMA LLD object
    @param[in]  TxQueueLength - the length in no of cells of the Tx queues

    @return     error or E_OK if success.
*//***************************************************************************/
t_Error IMA_LLD_ConfigTxQueueCellLength(t_Handle h_ImaLld, uint32_t TxQueueLength);


/**************************************************************************//**
    @par Description:
        Config IMA LLD DCBs length

    @par Important Notes:
        This function must be called after the config function

    @param[in]  h_ImaLld - the handle of the IMA LLD object
    @param[in]  DcbLength - the length in no of cells of the DCBs

    @return     error or E_OK if success.
*//***************************************************************************/
t_Error IMA_LLD_ConfigDcbCellLength(t_Handle h_ImaLld, uint32_t DcbLength);


/** @} */ /* end of ima_lld_obj_init_advanced_unit */


/**************************************************************************//**
    @par Description:
        IMA LLD initialization function

    @par Important Notes:
        This function must be called after the config function

    @param[in]  h_ImaLld - the handle of the IMA LLD object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_Init(t_Handle h_ImaLld);


/**************************************************************************//**
    @par Description:
        Release memory and physical resources of the IMA LLD object

    @par Important Notes:
        This function must be called after the Group free function

    @param[in]  h_ImaLld - handle of the IMA LLD object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_Free(t_Handle h_ImaLld);


/** @} */ /* end of ima_lld_obj_init_unit */


/**************************************************************************//**
 @Group         ima_lld_obj_ctl_unit IMA LLD Object Control Unit

 @Description   IMA LLD Object API Control Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Get IMA LLD DCBs length

    @param[in]  h_ImaLld - the handle of the IMA LLD object

    @return     the DCB length measured in number of cells
*//***************************************************************************/
uint32_t IMA_LLD_GetDcbCellLength(t_Handle h_ImaLld);

/**************************************************************************//**
    @par Description:
        Get IMA LLD Set IMA Version

    @param[in]  h_ImaLld - the handle of the IMA LLD object
	@param[in]  ImaVer   - The IMA version of the IMA unit. Can be v1.0 of v1.1
*//***************************************************************************/
t_Error IMA_LLD_SetImaVersion(t_Handle h_ImaLld, e_ImaVersion ImaVer);



/** @} */ /* end of ima_lld_obj_ctl_unit */


/** @} */ /* end of ima_lld_obj */


/**************************************************************************//**
@Group         ima_lld_group_obj IMA LLD GROUP Object Application Programming Interface

@Description   IMA LLD GROUP object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ima_lld_group_obj_init_unit IMA LLD GROUP Object Initialization Unit

 @Description   IMA LLD GROUP Object API Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA LLD GROUP initialization structure
*//***************************************************************************/
typedef struct
{
	t_Handle    h_GroupUpper;       /**< ptr to the group stuct of the upper layer */
	bool        TxCTC;              /**< if group works in CTC mode or not */
	t_Handle    h_AtmPort;
}t_ImaLldGroupParam;


/**************************************************************************//**
 @Description   IMA LLD GROUP IFSM parameters initialization structure
*//***************************************************************************/
typedef struct
{
    uint8_t    alpha;   /**< IFSM alpha parameter */
    uint8_t    beta;    /**< IFSM beta parameter */
    uint8_t    gamma;   /**< IFSM gamma parameter */
}t_ImaGroupIfsmParam;


/**************************************************************************//**
    @par Description:
        IMA LLD Group configuration function

    @par Important Notes:
        This function must be called before any other routine of this object

    @param[in]  h_ImaLld - handle of the IMA LLD object
    @param[in]  p_ImaLldGroupParam - IMA LLD Group parameters provided by the upper layer

    @return     the handle of the IMA LLD GROUP object
*//***************************************************************************/
t_Handle IMA_LLD_GROUP_Config(t_Handle h_ImaLld, t_ImaLldGroupParam* p_ImaLldGroupParam);


/**************************************************************************//**
 @Group         ima_lld_group_obj_init_advanced_unit IMA LLD GROUP Object Advanced Initialization Unit

 @Description   IMA LLD GROUP Object API Advanced Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        IMA LLD Group optional configuration function for IFSM parameters

    @par Important Notes:
        This function must be called after IMA_LLD_GROUP_Config

    @param[in]  h_ImaLldGroup - the handle of the IMA LLD GROUP object
    @param[in]  p_ImaGroupIfsmParam - IFSM parameters

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_ConfigIfsm(t_Handle h_ImaLldGroup, t_ImaGroupIfsmParam *p_ImaGroupIfsmParam);
/**************************************************************************//**
    @par Description:
        IMA LLD Group optional configuration function for IMA version Rx

    @par Important Notes:
        This function must be called after IMA_LLD_GROUP_Config

    @param[in]  h_ImaLldGroup - the handle of the IMA LLD GROUP object
    @param[in]  rxImaId - Ima Version

    @return     error code
*//***************************************************************************/
 t_Error IMA_LLD_GROUP_SetRxImaVersion(t_Handle h_ImaLldGroup, uint8_t rxImaId);
/** @} */ /* end of ima_lld_group_obj_init_advanced_unit */


/**************************************************************************//**
    @par Description:
        IMA LLD Group initialization function

    @par Important Notes:
        This function must be called after the Group config function

    @param[in]  h_ImaLldGroup - the handle of the IMA LLD GROUP object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_Init(t_Handle h_ImaLldGroup);


/**************************************************************************//**
    @par Description:
        Release memory and physical resources of an IMA LLD group

    @par Important Notes:
        This function must be called after the TxLink and RxLink free functions

    @param[in]  h_ImaLldGroup - handle of the IMA LLD GROUP to be released

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_Free(t_Handle h_ImaLldGroup);


/** @} */ /* end of ima_lld_group_obj_init_unit */


/**************************************************************************//**
 @Group         ima_lld_group_obj_run_unit IMA LLD GROUP Object Runtime Unit

 @Description   IMA LLD GROUP Object API Runtime Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Enables the GROUP to transmit and receive ATM data cells (set group to active mode)

    @par Important Notes:
        This function must be called after the GDS event has been obtained for the group

    @param[in]  h_ImaLldGroup - handle of the GROUP object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_Enable(t_Handle h_ImaLldGroup);


/**************************************************************************//**
    @par Description:
        Disables the GROUP to transmit and receive ATM data cells (set group to filler mode)

    @par Important Notes:
        This function must be called after the GDS event has been obtained for the group

    @param[in]  h_ImaLldGroup - handle of the GROUP object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_Disable(t_Handle h_ImaLldGroup);


/**************************************************************************//**
    @par Description:
        Add a Tx link in a group

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  h_ImaLldTxLink - handle of the TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_AddTxLink(t_Handle h_ImaLldGroup, t_Handle h_ImaLldTxLink);


/**************************************************************************//**
    @par Description:
        Add an Rx link in a group

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_AddRxLink(t_Handle h_ImaLldGroup, t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Delete one Tx link from a group

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_DeleteTxLink(t_Handle h_ImaLldTxLink);


/**************************************************************************//**
    @par Description:
        Delete one Rx link from a group

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_DeleteRxLink(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Remove one Rx link from the DCB

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_RemoveRxLinkFromDcb(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Start the Delay Compensation Buffering (DCB) for all the links of the group

    @param[in]  h_ImaLldGroup - handle of the GROUP object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_StartDcb(t_Handle h_ImaLldGroup);


/**************************************************************************//**
    @par Description:
        Start the Delay Compensation Buffering (DCB) for the selected links of the group

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  RxLinkArray - the array with the handles of the links which are to be delay compendated
    @param[in]  size - the size of the array

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_StartDcbSelective(t_Handle h_ImaLldGroup, t_Handle *RxLinkArray, uint8_t size);


/**************************************************************************//**
    @par Description:
        Start the Group Delay Synchronization(GDS) process

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[out] h_LinkUpper - handle of the upper layer link object which caused the GDS failure

    @return     error code
*//***************************************************************************/
bool IMA_LLD_GROUP_GdsSync(t_Handle h_ImaLldGroup, t_Handle *h_LinkUpper);


/**************************************************************************//**
    @par Description:
        Prepare a new ICP cell for transmitting to the FE

    @param[in]  h_ImaLldGroup - the handle of the group that has to send a modified ICP cell
    @param[in]  p_Buffer - the new ICP cell buffer

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_ChangeTxIcpCell(t_Handle h_ImaLldGroup, uint8_t *p_Buffer);

t_Error IMA_LLD_GROUP_RxLinkRemoval(t_Handle h_ImaLldRxLink);
/** @} */ /* end of ima_lld_group_obj_run_unit */


/**************************************************************************//**
 @Group         ima_lld_group_obj_ctl_unit IMA LLD GROUP Object Control Unit

 @Description   IMA LLD GROUP Object API Control Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Get the IFSM parameters of the IMA LLD GROUP

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  p_Params - storage area for the IFSM parameters

    @return     handle of the port of the IMA LLD GROUP
*//***************************************************************************/
t_Error IMA_LLD_GROUP_GetIfsmParams(t_Handle h_ImaLldGroup, t_ImaGroupIfsmParam *p_Params);


/**************************************************************************//**
    @par Description:
        Set GROUP Rx IMA ID (IMA ID of the FE)

    @par Important Notes:
        This function must be called after the M parameter has been negotiated with the FE

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  rxImaId - IMA ID of the FE

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_SetRxImaId(t_Handle h_ImaLldGroup, uint8_t rxImaId);


/**************************************************************************//**
    @par Description:
        Set GROUP Rx IMA frame size (M parameter received from the FE)

    @par Important Notes:
        This function must be called after the M parameter has been negotiated with the FE

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  imaFrameSize - IMA frame size

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_SetRxImaFrameSize(t_Handle h_ImaLldGroup, e_ImaFrameLength imaFrameSize);


/**************************************************************************//**
    @par Description:
        Set GROUP Tx IMA frame size (M parameter)

    @par Important Notes:
        This function must be called after the M parameter has been negotiated with the FE

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  imaFrameSize - IMA frame size

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_SetTxImaFrameSize(t_Handle h_ImaLldGroup, e_ImaFrameLength imaFrameSize);

/**************************************************************************//**
    @par Description:
        Get the status of the GDSS register. This will alow the veriffication of various params

    @par Important Notes:
        This function can be called after the Rx Group has been initialized

    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @return     Status - TRUE if the Rx Group is not in Sync Process
    			FALSE - If the Rx Group has started the SYNC process or is in SYNC
*//***************************************************************************/
 bool IMA_LLD_GROUP_RxGroupNotInSyncProcess(t_Handle h_ImaLldGroup);
/**************************************************************************//**
    @par Description:
        UPdate the SCCI of the Group. Function Needed in order to update the SCCI if the
        ICP cell recieved has a corrupted SCCI

    @par Important Notes:
    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  NewScci - The new SCCI value
    @return     Error
*//***************************************************************************/
t_Error IMA_LLD_GROUP_UpdateGroupScci(t_Handle h_ImaLldGroup, uint8_t NewScci);
/**************************************************************************//**
    @par Description:
		Update of the SCCI increment needed when FE is expecting other SCCI
    @par Important Notes:
    @param[in]  h_ImaLldGroup - handle of the GROUP object
    @param[in]  IcpTxScciIncrement - Scci Increment that will be used for control
*//***************************************************************************/
t_Error IMA_LLD_GROUP_UpdateIcpScciIncrement(t_Handle h_ImaLldGroup, uint8_t IcpTxScciIncrement);
/** @} */ /* end of ima_lld_group_obj_ctl_unit */


/** @} */ /* end of ima_lld_group_obj */


/**************************************************************************//**
@Group         ima_lld_group_port_obj IMA LLD GROUP PORT Object Application Programming Interface

@Description   IMA LLD GROUP PORT object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ima_lld_group_port_obj_init_unit IMA LLD GROUP PORT Object Initialization Unit

 @Description   IMA LLD GROUP PORT Object API Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA LLD GROUP port initialization structure
*//***************************************************************************/
typedef struct
{
    t_Handle   h_UccAtm;
    uint8_t    schedLevels;/**< number of the required Apc levels */
    uint32_t   fastChannelRateBitPerSec;/**< the rate of the fast channel - bits per second */
    uint32_t   slowChannelRateBitPerSec[e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS];/**< the rate of the slow channel - bits per second per priority level */
    uint32_t   lineRateBitPerSec; /**< the line rate */
}t_ImaLldGroupPortParam;


/**************************************************************************//**
    @par Description:
        IMA LLD Group Port configuration function

    @par Important Notes:
        This function must be called before any other routine of this object
 
    @param[in]  p_PortParam - IMA LLD Group Port parameters provided by the upper layer

    @return     the handle of the IMA LLD GROUP PORT object
*//***************************************************************************/
t_Handle IMA_LLD_GROUP_PORT_Config(t_ImaLldGroupPortParam *p_PortParam);


/**************************************************************************//**
    @par Description:
        IMA LLD Group Port initialization function

    @par Important Notes:
        This function must be called after the Group Port config function

    @param[in]  h_ImaGroupPort - the handle of the IMA LLD GROUP PORT object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_PORT_Init(t_Handle h_ImaGroupPort);


/**************************************************************************//**
    @par Description:
        Release memory and physical resources of an IMA LLD group port
 
    @param[in]  h_ImaGroupPort - handle of the IMA LLD GROUP PORT to be released
 
    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_GROUP_PORT_Free(t_Handle h_ImaGroupPort);


/** @} */ /* end of ima_lld_group_port_obj_init_unit */


/** @} */ /* end of ima_lld_group_port_obj */


/**************************************************************************//**
@Group         ima_lld_txlink_obj IMA LLD TXLINK Object Application Programming Interface

@Description   IMA LLD TXLINK object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ima_lld_txlink_obj_init_unit IMA LLD TXLINK Object Initialization Unit

 @Description   IMA LLD TXLINK Object API Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA LLD Tx link initialization structure
*//***************************************************************************/
typedef struct
{
    t_Handle    h_TxLinkUpper;  /**< ptr to the Tx Link struct of the upper layer (e.g. PM), if any */
    uint8_t     PhyLinkIndex;   /**< index of the T1/E1 physical link */
}t_ImaLldTxLinkParam;


/**************************************************************************//**
    @par Description:
        IMA LLD Tx link configuration function

    @par Important Notes:
        This function must be called before any other routine of this object

    @param[in]  h_ImaLld - handle of the IMA LLD object
    @param[in]  p_ImaLldTxLinkParam - IMA Tx link parameters provided by the upper layer

    @return     the handle of the IMA LLD TXLINK object
*//***************************************************************************/
t_Handle IMA_LLD_TXLINK_Config(t_Handle h_ImaLld, t_ImaLldTxLinkParam* p_ImaLldTxLinkParam);


/**************************************************************************//**
 @Group         ima_lld_txlink_obj_init_advanced_unit IMA LLD TXLINK Object Advanced Initialization Unit

 @Description   IMA LLD TXLINK Object API Advanced Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        IMA LLD TXLINK advanced configuration function

    @par Important Notes:
        This function is mandatory and must be called after the TC has been configured and before TXLINK initialization

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object
    @param[in]  h_Tc - IMA Tx link parameters provided by the upper layer

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_ConfigTcHandle(t_Handle h_ImaLldTxLink, t_Handle h_Tc);
/**************************************************************************//**
    @par Description:
        IMA LLD TXLINK advanced configuration function. If called the TC will work on UCC

    @par Important Notes:
    	Important if TC works on UCC controler and not on MCC controler. Default is TC over MCC
 
    @param[in]  h_ImaLldTxLink - handle of the RXLINK object
 
    @return     error code
*//***************************************************************************/
  t_Error IMA_LLD_TXLINK_ConfigTcOverUcc(t_Handle h_ImaLldTxLink);

/** @} */ /* end of ima_lld_txlink_obj_init_advanced_unit */


/**************************************************************************//**
    @par Description:
        IMA LLD Tx link initialization function

    @par Important Notes:
        This function must be called after the Tx Link config function

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_Init(t_Handle h_ImaLldTxLink);


/**************************************************************************//**
    @par Description:
        Release memory and physical resources of an IMA LLD Tx link

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_Free(t_Handle h_ImaLldTxLink);


/** @} */ /* end of ima_lld_txlink_obj_init_unit */


/**************************************************************************//**
 @Group         ima_lld_txlink_obj_run_unit IMA LLD TXLINK Object Runtime Unit

 @Description   IMA LLD TXLINK Object API Runtime Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Enables the Tx link (set the Tx Link to active mode)

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_Enable(t_Handle h_ImaLldTxLink);


/**************************************************************************//**
    @par Description:
        Disables the Tx link (set the Tx Link to filler mode)

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_Disable(t_Handle h_ImaLldTxLink);


/**************************************************************************//**
    @par Description:
        Change the TRL link of the group

    @param[in]  h_ImaLldTxLink - handle of the IMA LLD TXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_ChangeTrl(t_Handle h_ImaLldTxLink);


/** @} */ /* end of ima_lld_txlink_obj_run_unit */


/**************************************************************************//**
 @Group         ima_lld_txlink_obj_ctl_unit IMA LLD TXLINK Object Control Unit

 @Description   IMA LLD TXLINK Object API Control Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Set TXLINK linkId

    @par Important Notes:
        This function must be called after the first received ICP cell on this link

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object
    @param[in]  linkId - linkId value

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_SetLinkId(t_Handle h_ImaLldTxLink, uint8_t linkId);


/**************************************************************************//**
    @par Description:
        Set TXLINK ICP Cell Offset

    @param[in]  h_ImaLldTxLink - handle of the TXLINK object
    @param[in]  icpOffset - ICP cell offset value

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_SetIcpCellOffset(t_Handle h_ImaLldTxLink, uint8_t icpOffset);


/**************************************************************************//**
    @par Description:
        Get stuff event counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link
    @param[out]  p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_GetIcpStuff(t_Handle h_ImaLldTxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset stuff event counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_ResetIcpStuff(t_Handle h_ImaLldTxLink);
/**************************************************************************//**
    @par Description:
        Get ICP counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_GetIcpCellCounter(t_Handle h_ImaLldTxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset ICP counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_ResetIcpCellCounter(t_Handle h_ImaLldTxLink);
/**************************************************************************//**
    @par Description:
        Get Filler Cell counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_GetFillerCellCounter(t_Handle h_ImaLldTxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset Filler Cell counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_ResetFillerCellCounter(t_Handle h_ImaLldTxLink);

/**************************************************************************//**
    @par Description:
        Get User Cell counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_GetUserCellCounter(t_Handle h_ImaLldTxLink, uint32_t *p_Counter);
/**************************************************************************//**
    @par Description:
        Reset User Cell counter for the selected Tx link

    @param[in]  h_ImaLldTxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_TXLINK_ResetUserCellCounter(t_Handle h_ImaLldTxLink);
/** @} */ /* end of ima_lld_txlink_obj_ctl_unit */


/** @} */ /* end of ima_lld_txlink_obj */


/**************************************************************************//**
@Group         ima_lld_rxlink_obj IMA LLD RXLINK Object Application Programming Interface

@Description   Ima LLD RXLINK object API functions, structures, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ima_lld_rxlink_obj_init_unit IMA LLD RXLINK Object Initialization Unit

 @Description   IMA LLD RXLINK Object API Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IMA LLD Rx link initialization structure
*//***************************************************************************/
typedef struct
{
    t_Handle    h_RxLinkUpper;  /**< ptr to the Rx Link struct of the upper layer (e.g. PM), if any */
    uint8_t     PhyLinkIndex;   /**< index of the T1/E1 physical link */
}t_ImaLldRxLinkParam;


/**************************************************************************//**
    @par Description:
        IMA LLD Rx link initialization function

    @par Important Notes:
        This function must be called before any other routine of this object

    @param[in]  h_ImaLld - handle of the IMA LLD object
    @param[in]  p_ImaLldRxLinkParam - IMA Rx link parameters provided by the upper layer

    @return     the handle of the IMA LLD RXLINK object
*//***************************************************************************/
t_Handle IMA_LLD_RXLINK_Config(t_Handle h_ImaLld, t_ImaLldRxLinkParam* p_ImaLldRxLinkParam);


/**************************************************************************//**
 @Group         ima_lld_rxlink_obj_init_advanced_unit IMA LLD RXLINK Object Advanced Initialization Unit

 @Description   IMA LLD RXLINK Object API Advanced Initialization Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        IMA LLD RXLINK advanced configuration function. Configures the TC handle.

    @par Important Notes:
        This function is mandatory and must be called after the TC has been configured and before RXLINK initialization

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object
    @param[in]  h_Tc - IMA Tx link parameters provided by the upper layer

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ConfigTcHandle(t_Handle h_ImaLldRxLink, t_Handle h_Tc);
/**************************************************************************//**
    @par Description:
        IMA LLD RXLINK advanced configuration function. If called the TC will work on UCC

    @par Important Notes:
    	Important if TC works on UCC controler and not on MCC controler. Default is TC over MCC
 
    @param[in]  h_ImaLldRxLink - handle of the RXLINK object
 
    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ConfigTcOverUcc(t_Handle h_ImaLldRxLink);

/** @} */ /* end of ima_lld_rxlink_obj_init_advanced_unit */


/**************************************************************************//**
    @par Description:
        IMA LLD Rx link initialization function

    @par Important Notes:
        This function must be called after the Rx Link config function

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_Init(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Release memory and physical resources of an IMA LLD Rx link

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_Free(t_Handle h_ImaLldRxLink);


/** @} */ /* end of ima_lld_rxlink_obj_init_unit */


/**************************************************************************//**
 @Group         ima_lld_rxlink_obj_run_unit IMA LLD RXLINK Object Runtime Unit

 @Description   IMA LLD RXLINK Object API Runtime Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Enables the Rx link (set the Rx Link to active mode)

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_Enable(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Disables the Rx link (set the Rx Link to filler mode)

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_Disable(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Enables the reception of ATM traffic from the TC

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_StartTc(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Disables the reception of ATM traffic from the TC

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_StopTc(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Start the IMA Frame Synchronization Mechanism (IFSM)

    @par Important Notes:
        This function must be called after IMA_LLD_RXLINK_SetLinkId and IMA_LLD_RXLINK_SetIcpCellOffset functions

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_StartIfsm(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Stop the IMA Frame Synchronization Mechanism (IFSM)

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_StopIfsm(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Start the Delay Compensation Buffering for this link

    @par Important Notes:
        This function must be called after IMA_LLD_RXLINK_StartIfsm function

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_StartDcb(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Stop the Delay Compensation Buffering (DBC) for this link

    @par Important Notes:
        This function must be called before IMA_LLD_RXLINK_StartIfsm function

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_StopDcb(t_Handle h_ImaLldRxLink);


/** @} */ /* end of ima_lld_rxlink_obj_run_unit */


/**************************************************************************//**
 @Group         ima_lld_rxlink_obj_ctl_unit IMA LLD RXLINK Object Control Unit

 @Description   IMA LLD RXLINK Object API Control Unit

 @{
*//***************************************************************************/


/**************************************************************************//**
    @par Description:
        Set RXLINK linkId

    @par Important Notes:
        This function must be called after the first received ICP cell on this link

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object
    @param[in]  linkId - linkId value

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_SetLinkId(t_Handle h_ImaLldRxLink, uint8_t linkId);


/**************************************************************************//**
    @par Description:
        Get RXLINK ICP Cell Offset

    @par Important Notes:
        This function must be called after the first received ICP cell on this link

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object
    @param[in]  icpOffset - ICP cell offset value

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_SetIcpCellOffset(t_Handle h_ImaLldRxLink, uint8_t icpOffset);

/**************************************************************************//**
    @par Description:
       Turn the RXLINK Monitor Off

    @par Important Notes:
		The Link should be allready Initialized
    @param[in]  h_ImaLldRxLink - handle of the RXLINK object
    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_TurnMonitorOff(t_Handle h_ImaLldRxLink);
/**************************************************************************//**
    @par Description:
       Turn the RXLINK Monitor On

    @par Important Notes:
		The Link should be allready Initialized
    @param[in]  h_ImaLldRxLink - handle of the RXLINK object
    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_TurnMonitorOn(t_Handle h_ImaLldRxLink);
/**************************************************************************//**
    @par Description:
        Get the current state of the IMA Error State Machine (IESM) for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_State - Current IESM state

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetIesmState(t_Handle h_ImaLldRxLink, e_ImaLinkIesmState *p_State);


/**************************************************************************//**
    @par Description:
        Get ICP violation event counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_Counter - ICP violations counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetIcpViolCounter(t_Handle h_ImaLldRxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset ICP violation event counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ResetIcpViolCounter(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Get Out-of-IMA frame counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_Counter - OIF counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetOifCounter(t_Handle h_ImaLldRxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset Out-of-IMA frame counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ResetOifCounter(t_Handle h_ImaLldRxLink);


/**************************************************************************//**
    @par Description:
        Get stuff event counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetIcpStuffCounter(t_Handle h_ImaLldRxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset stuff event counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ResetIcpStuffCounter(t_Handle h_ImaLldRxLink);
/**************************************************************************//**
    @par Description:
        Get ICP counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetIcpCellCounter(t_Handle h_ImaLldRxLink, uint32_t *p_Counter);
/**************************************************************************//**
    @par Description:
        Reset ICP counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ResetIcpCellCounter(t_Handle h_ImaLldRxLink);
/**************************************************************************//**
    @par Description:
        Get Filler Cell counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetFillerCellCounter(t_Handle h_ImaLldRxLink, uint32_t *p_Counter);


/**************************************************************************//**
    @par Description:
        Reset Filler Cell counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ResetFillerCellCounter(t_Handle h_ImaLldRxLink);

/**************************************************************************//**
    @par Description:
        Get User Cell counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link
    @param[out] p_Counter - stuff counter

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_GetUserCellCounter(t_Handle h_ImaLldRxLink, uint32_t *p_Counter);
/**************************************************************************//**
    @par Description:
        Reset User Cell counter for the selected Rx link

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
t_Error IMA_LLD_RXLINK_ResetUserCellCounter(t_Handle h_ImaLldRxLink);
/**************************************************************************//**
    @par Description:
        Get RXLINK received IFSN

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     error code
*//***************************************************************************/
uint8_t IMA_LLD_RXLINK_GetRxIfsn(t_Handle h_ImaLldRxLink);

/**************************************************************************//**
    @par Description:
        Get RXLINK received M counter

    @param[in]  h_ImaLldRxLink - handle of the link

    @return     M counter
*//***************************************************************************/
uint8_t IMA_LLD_RXLINK_GetMCounter(t_Handle h_ImaLldRxLink);

/**************************************************************************//**
    @par Description:
        Get RXLINK ICP Cell Offset

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     icpOffset - ICP cell offset value
*//***************************************************************************/
uint8_t IMA_LLD_RXLINK_GetIcpCellOffset(t_Handle h_ImaLldRxLink);
/**************************************************************************//**
    @par Description:
        Get the group Assigned state

    @param[in]  h_ImaLldRxLink - handle of the RXLINK object

    @return     True if the Link is allready assigned
*//***************************************************************************/
bool IMA_LLD_RXLINK_GetGroupAssigned(t_Handle h_ImaLldRxLink);
/** @} */ /* end of ima_lld_rxlink_obj_ctl_unit */


/** @} */ /* end of ima_lld_rxlink_obj */


/** @} */ /* end of ima_lld_api */
#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      IMA_LLD_TXLINK_DumpRegs

 @Description   Dumps IMA LLD Tx Link registers.

 @Param         h_ImaLldTxLink - (In) handle to Tx Link

 @Return        None.
*//***************************************************************************/

void IMA_LLD_TXLINK_DumpRegs(t_Handle h_ImaLldTxLink);
/**************************************************************************//**
 @Function      IMA_LLD_RXLINK_DumpRegs

 @Description   Dumps IMA LLD Rx Link registers.

 @Param         h_ImaLldRxLink - (In) handle to Rx Link

 @Return        None.
*//***************************************************************************/

void IMA_LLD_RXLINK_DumpRegs(t_Handle h_ImaLldRxLink);

#endif /* (DEBUG_ERRORS > 0) */

#ifdef __cplusplus
}
#endif
#endif  /* __IMA_LLD_EXT_H */
