/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          l2switch_ext.h

 @Description   l2 switch external API
*//***************************************************************************/
#ifndef __L2_SWITCH_EXT_H
#define __L2_SWITCH_EXT_H

#include "std_ext.h"
#include "qe_ext.h"
#include "enet_ext.h"
#include "ucc_fast_ext.h"
#include "xx_ext.h"


/**************************************************************************//**
 @Group         l2switch_qe_id L2 SWITCH Application Programming Interface

 @Description   L2 SWITCH functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         l2switch_qe_init_id L2 SWITCH Initialization Unit

 @Description   L2SW Initialization routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   L2 SWITCH tag mode options.
*//***************************************************************************/
typedef enum e_L2swTagMode
{
    e_L2SW_TAG_MODE_BASIC   = 0x00000000,   /**< VLAN-unaware mode: switch is NOT configured
                                                 to handle frames according to their VLAN tag. */
    e_L2SW_TAG_MODE_TAGGED  = 0x80000000    /**< VLAN-aware mode: switch is configured to handle
                                                 frames according to their VLAN tag. */
} e_L2swTagMode;

/**************************************************************************//**
 @Description   L2 SWITCH  debug mode.
 
                Discard / forward to cpu error/special meaning frames.
*//***************************************************************************/
typedef enum e_L2swDebugMode
{
    e_L2SW_DEBUG_MODE_ERROR_FRAME_DISCARD                   = 0x00000000,   /**< Erroneous frames are discarded. */
    e_L2SW_DEBUG_MODE_ERROR_FRAME_FORWARD_TO_CPU            = 0x20000000,   /**< Erroneous frames are forwarded to CPU. */
    e_L2SW_DEBUG_MODE_FLOW_CONTROL_FRAME_NO_CPU_NOTIFICATION = 0x00000000,  /**< Handle Flow Control Frames without forwarding to CPU */
    e_L2SW_DEBUG_MODE_FLOW_CONTROL_FORWARD_TO_CPU           = 0x10000000,   /**< Handle Control Frames and forward them to CPU. */
    e_L2SW_DEBUG_MODE_SECURITY_PROBLEM_FRAME_DISCARD        = 0x00000000,   /**< Incoming frames with unknown VLAN ID or not belonging
                                                                                 to port's VLAN group are discarded.  */
    e_L2SW_DEBUG_MODE_SECURITY_PROBLEM_FRAME_FORWARD_TO_CPU = 0x08000000,   /**< Incoming frames with unknown VLAN ID or not belonging
                                                                                 to port's VLAN group are forwarded to CPU.  */
    e_L2SW_DEBUG_MODE_NO_DESTINATION_PORT_DISCARD           = 0x00000000,   /**< Discard if no destination port is found. */
    e_L2SW_DEBUG_MODE_NO_DESTINATION_PORT_FORWARD_TO_CPU    = 0x04000000    /**< Forward to CPU if destination port was not found. */
} e_L2swDebugMode;


/**************************************************************************//**
 @Description   L2 SWITCH  hash conflict mode.
 
                Hash conflict occurs when all four entries that correspond to
                the same column in hash table are full.
*//***************************************************************************/
typedef enum e_L2swHashingConflictMode
{
    e_L2SW_HASH_CONFLICT_DISCARD_NEW          = 0x00000000, /**< Keep the old entry, ignore the new one. */
    e_L2SW_HASH_CONFLICT_DISCARD_OLDEST_ENTRY = 0x00080000  /**< Insert the new data and discard the old one. */
} e_L2swHashingConflictMode;

/**************************************************************************//**
 @Description   L2 SWITCH  RMON Statistics counters mode.
*//***************************************************************************/
typedef enum e_L2RmonCountersMode
{
    e_L2SW_RMON_COUNTERS_DISABLED = 0x00000000, /**< Statistics are disabled. */
    e_L2SW_RMON_COUNTERS_ENABLED  = 0x00040000  /**< Statistics are enabled. */
} e_L2RmonCountersMode;

/**************************************************************************//**
 @Description   L2 SWITCH  aging mechanism mode.
*//***************************************************************************/
typedef enum e_L2swAgingMechanismMode
{
    e_L2SW_AGING_MECHANISM_DISABLED = 0x00000000,   /**< Adlt entries aging is disabled. */
    e_L2SW_AGING_MECHANISM_ENABLED  = 0x00020000    /**< Adlt entries aging is enabled. */
} e_L2swAgingMechanismMode;


/**************************************************************************//**
 @Description   L2 SWITCH PORT Table Selection.
 
                Indicates which table should be used for address resolution.
*//***************************************************************************/
typedef enum e_L2swPortTableSelectionMode
{
    e_L2SW_PORT_TABLE_USE_VDT_ELSE_DISCARD = 0,          /**< Use VDT, if not found discard.
                                                              Relevant in tag mode only. */
    e_L2SW_PORT_TABLE_USE_VDT_ELSE_PDT     = 0x40000000, /**< Use VDT, if not found use Ports Destination table;
                                                              Port destination table is maintained per port;
                                                              Relevant in tag mode only. */
    e_L2SW_PORT_TABLE_USE_PDT              = 0x80000000  /**< Use Ports Destination Table maintained per port;
                                                              Relevant in both tagged and untagged modes. */
} e_L2swPortTableSelectionMode;

/**************************************************************************//**
 @Description   L2 SWITCH PORT 1Q trunk.
 
                In 1Q trunk mode port is able to receive and transmit all VIDs.
                This mode is relevant only for VLAN-aware switch.
*//***************************************************************************/
typedef enum e_L2swPort1qTrunkMode
{
    e_L2SW_PORT_1Q_NORMAL = 0,          /**< Non-1Q Trunk mode: port is not accepting a tagged
                                             frame if it is not a member of its VLAN. */
    e_L2SW_PORT_1Q_TRUNK  = 0x20000000  /**< 1Q Trunk mode: port accepts a tagged frame even
                                             though it is not a member of its VLAN. */
} e_L2swPort1qTrunkMode;

/**************************************************************************//**
 @Description   L2 SWITCH PORT 1D trunk.
 
                In 1D trunk mode all frames are transmitted untagged.
                Relevant only for VLAN-aware switch.
*//***************************************************************************/
typedef enum e_L2swPort1dTrunkMode
{
    e_L2SW_PORT_1D_NORMAL = 0,          /**< Pass data traffic without tampering.*/
    e_L2SW_PORT_1D_TRUNK  = 0x10000000  /**< Pass only untagged traffic, remove tag if tagged frame;
                                             replace also CRC. */
} e_L2swPort1dTrunkMode;

/**************************************************************************//**
 @Description   L2 SWITCH  transparent mode.
*//***************************************************************************/
typedef enum e_L2swPortTransparentMode
{
    e_L2SW_PORT_TRANSPARENT_MODE_DISABLED = 0x00000000, /**< Normal behavior.                                   */
    e_L2SW_PORT_TRANSPARENT_MODE_ENABLED  = 0x00004000  /**< Frame kept as is, no addition or removal of tags.   */
} e_L2swPortTransparentMode;

/**************************************************************************//**
 @Description   L2 SWITCH PORT state.
 
                Port states to conform with the spanning tree protocol.
*//***************************************************************************/
typedef enum e_L2swPortState
{
    e_L2SW_PORT_STATE_DISABLED             = 0,             /**< No table updating, no frame forwarding. */
    e_L2SW_PORT_STATE_FORWARDING           = 0x04000000,    /**< Normal operation, forwarding learning and updating tables. */
    e_L2SW_PORT_STATE_BLOCKING_LISTENING   = 0x08000000,    /**< Only BPDU frames are handled.                 */
    e_L2SW_PORT_STATE_DISABLED_LEARNING    = 0x0C000000     /**< Similar to listening only tables are updated.*/
} e_L2swPortState;


/**************************************************************************//**
 @Description   L2 SWITCH PORT Pad and CRC Mode.
 
                Padding without CRC is not possible.
*//***************************************************************************/
typedef enum e_L2swPortMaccfg2PadAndCrcMode
{
    e_L2SW_PORT_MACCFG2_PAD_AND_CRC_MODE_NONE           = 0x00000000,   /**< Neither Padding short frames nor CRC. */
    e_L2SW_PORT_MACCFG2_PAD_AND_CRC_MODE_CRC_ONLY       = 0x00000002,   /**< Append CRC only. */
    e_L2SW_PORT_MACCFG2_PAD_AND_CRC_MODE_PAD_AND_CRC    = 0x00000004    /**< Pad short frames and append CRC;
                                                                             Resulting frame is at least 64 bytes. */
} e_L2swPortMaccfg2PadAndCrcMode;

/**************************************************************************//**
 @Description   L2 SWITCH PORT untagged frames policy.
 
                Drop or add default tag. Relevant only in tag mode.
*//***************************************************************************/
typedef enum e_L2swPortUntaggedPolicy
{
    e_L2SW_PORT_DROP_UNTAGGED   = 0,          /**< Untagged frames will be dropped. */
    e_L2SW_PORT_ADD_DEFAULT_TAG = 0x02000000  /**< Default tag will be added to untagged frames. */
} e_L2swPortUntaggedPolicy;

/**************************************************************************//**
 @Description   L2 SWITCH PORT force default tag.
 
                Relevant only in tag mode.
*//***************************************************************************/
typedef enum e_L2swPortForceDefaultTag
{
    e_L2SW_PORT_NOT_FORCING_DEFAULT_TAG = 0,         /**< Leave tag intact. */
    e_L2SW_PORT_FORCING_DEFAULT_TAG     = 0x01000000 /**< Insert default tag instead of the tag
                                                          containing in incoming frame. */
} e_L2swPortForceDefaultTag;

/**************************************************************************//**
 @Description   L2 SWITCH PORT Protocol frames handling.
*//***************************************************************************/
typedef enum e_L2swPortProtocolFramesTreatment
{
    e_L2SW_PORT_IGMP_NOT_PROMPTING_TO_CPU       = 0,            /**< No special treatment for IGMP packets. */
    e_L2SW_PORT_IGMP_PROMPTING_TO_CPU           = 0x00800000,   /**< Forward IGMP packets to CPU. */
    e_L2SW_PORT_BPDU_NOT_PROMPTING_TO_CPU       = 0,            /**< No special treatment for BPDU packets. */
    e_L2SW_PORT_BPDU_PROMPTING_TO_CPU           = 0x00400000,   /**< Forward BPDU packets to CPU. */
    e_L2SW_PORT_BPDU_AND_IGMP_PROMPTING_TO_CPU  = 0x00C00000    /**< Forward both BPDU and IGMP packets to CPU. */
} e_L2swPortProtocolFramesTreatment;

/**************************************************************************//**
 @Description   L2 SWITCH  PORT Add extra tag.
*//***************************************************************************/
typedef enum e_L2swPortAddExtraTag
{
    e_L2SW_PORT_AET_NO_EXTRA_TAG_ADDITION   = 0,            /**< Not adding a 2nd tag. */
    e_L2SW_PORT_AET_ADD_EXTRA_TAG           = 0x00200000    /**< Adding an extra tag. */
} e_L2swPortAddExtraTag;

/**************************************************************************//**
 @Description   L2 SWITCH  PORT remove extra tag.
*//***************************************************************************/
typedef enum e_L2swPortRemoveExtraTag
{
    e_L2SW_PORT_RET_NO_EXTRA_TAG_REMOVAL    = 0,         /**< Not addressing extra tag situation. */
    e_L2SW_PORT_RET_REMOVE_EXTRA_TAG        = 0x00008000 /**< Removing an extra tag if presents. */
} e_L2swPortRemoveExtraTag;

/**************************************************************************//**
 @Description   L2 SWITCH  PORT broadcast disable.
*//***************************************************************************/
typedef enum e_L2swPortBroadcastDisable
{
    e_L2SW_PORT_BROADCAST_ENABLE   = 0,         /**< Forward broadcast frames (normal behavior).*/
    e_L2SW_PORT_BROADCAST_DISABLE  = 0x00100000 /**< Avoid broadcast transmission, thus preventing a broadcast storm.*/
} e_L2swPortBroadcastDisable;

/**************************************************************************//**
 @Description   L2 SWITCH  PORT IP packets priority.
*//***************************************************************************/
typedef enum e_L2swPortIPPriority
{
    e_L2SW_PORT_IP_PRIORITY                                        = 0,         /**< Use IP packet priority. */
    e_L2SW_PORT_IP_PRIORITY_FOR_UNTAGGED_TAG_PRIORITY_FOR_TAGGED   = 0x00040000,/**< Use IP packet priority only for untagged frames
                                                                                     and VLAn priority for tagged frames. */
    e_L2SW_PORT_IGNORE_IP_PRIORITY                                 = 0x00080000,/**< Ignore IP priority. */
    e_L2SW_PORT_FORCE_DEFAULT_PRIORITY                             = 0x000C0000 /**< Force default port priority. */
} e_L2swPortIPPriority;

/**************************************************************************//**
 @Description   L2 SWITCH  General priority.
*//***************************************************************************/
typedef enum e_L2swPriority
{
    e_L2SW_PRIORITY_LOW      = 0,         /**< Lowest priority. */
    e_L2SW_PRIORITY_NORMAL   = 0x1,       /**< Normal priority. */
    e_L2SW_PRIORITY_MEDIUM   = 0x2,       /**< Medium priority. */
    e_L2SW_PRIORITY_HIGH     = 0x3        /**< High   priority. */
} e_L2swPriority;


/**************************************************************************//**
 @Description   L2 SWITCH  PORT Port priority policy.
*//***************************************************************************/
typedef enum e_L2swPortPriorityPolicy
{
    e_L2SW_PORT_FIXED_PRIORITY             = 0,          /**< Invalid - not in use! */
    e_L2SW_PORT_INVALID_PRIORITY           = 0x00010000, /**< Fixed priority (weights are equal). */
    e_L2SW_PORT_WEIGHTED_PRIORITY          = 0x00020000, /**< Weighted round robin. */
    e_L2SW_PORT_COMBINED_WRR_AND_FIXED     = 0x00030000  /**< Mixed priority. */
} e_L2swPortPriorityPolicy;


/**************************************************************************//**
  L2 SWITCH general definitions.
*//***************************************************************************/
#define L2SW_PORT_MAX_NUM_RX_BD_RINGS   4           /**< Maximum number of port Rx queues. */
#define L2SW_PORT_MAX_NUM_OF_TX_QUEUES  4           /**< Maximum number of Tx queues from switch. */

#define L2SW_DESTINATION_PORTS_PORT_0 0x100

/**************************************************************************//**
 @Description   Destination port.
*//***************************************************************************/
typedef enum e_L2swDestinationPorts
{
    e_L2SW_DESTINATION_PORTS_PORT_0 = L2SW_DESTINATION_PORTS_PORT_0 >> 0,  /**<  Destination port 0. */
    e_L2SW_DESTINATION_PORTS_PORT_1 = L2SW_DESTINATION_PORTS_PORT_0 >> 1,  /**<  Destination port 1. */
    e_L2SW_DESTINATION_PORTS_PORT_2 = L2SW_DESTINATION_PORTS_PORT_0 >> 2,  /**<  Destination port 2. */
    e_L2SW_DESTINATION_PORTS_PORT_3 = L2SW_DESTINATION_PORTS_PORT_0 >> 3,  /**<  Destination port 3. */
    e_L2SW_DESTINATION_PORTS_PORT_4 = L2SW_DESTINATION_PORTS_PORT_0 >> 4,  /**<  Destination port 4. */
    e_L2SW_DESTINATION_PORTS_PORT_5 = L2SW_DESTINATION_PORTS_PORT_0 >> 5,  /**<  Destination port 5. */
    e_L2SW_DESTINATION_PORTS_PORT_6 = L2SW_DESTINATION_PORTS_PORT_0 >> 6,  /**<  Destination port 6. */
    e_L2SW_DESTINATION_PORTS_PORT_7 = L2SW_DESTINATION_PORTS_PORT_0 >> 7,  /**<  Destination port 7. */
    e_L2SW_DESTINATION_PORTS_PORT_8 = L2SW_DESTINATION_PORTS_PORT_0 >> 8,  /**<  Destination port 8 (CPU port). */
    e_L2SW_DESTINATION_PORTS_PORT_NONE = 0
} e_L2swDestinationPorts;

#define L2SW_SOURCE_PORTS_PORT_0 0x100

/**************************************************************************//**
 @Description   Source port.
*//***************************************************************************/
typedef enum e_L2swSourcePorts
{
    e_L2SW_SOURCE_PORTS_PORT_0 = L2SW_SOURCE_PORTS_PORT_0 >> 0,	  /**<  Source port 0. */
    e_L2SW_SOURCE_PORTS_PORT_1 = L2SW_SOURCE_PORTS_PORT_0 >> 1,   /**<  Source port 1. */
    e_L2SW_SOURCE_PORTS_PORT_2 = L2SW_SOURCE_PORTS_PORT_0 >> 2,   /**<  Source port 2. */
    e_L2SW_SOURCE_PORTS_PORT_3 = L2SW_SOURCE_PORTS_PORT_0 >> 3,   /**<  Source port 3. */
    e_L2SW_SOURCE_PORTS_PORT_4 = L2SW_SOURCE_PORTS_PORT_0 >> 4,   /**<  Source port 4. */
    e_L2SW_SOURCE_PORTS_PORT_5 = L2SW_SOURCE_PORTS_PORT_0 >> 5,   /**<  Source port 5. */
    e_L2SW_SOURCE_PORTS_PORT_6 = L2SW_SOURCE_PORTS_PORT_0 >> 6,   /**<  Source port 6. */
    e_L2SW_SOURCE_PORTS_PORT_7 = L2SW_SOURCE_PORTS_PORT_0 >> 7,   /**<  Source port 7. */
    e_L2SW_SOURCE_PORTS_PORT_8 = L2SW_SOURCE_PORTS_PORT_0 >> 8    /**<  Source port 8 (CPU port). */
} e_L2swSourcePorts;


/**************************************************************************//**
 @Description   User callback function for event handling.
 
                User provides this function, driver invokes it.

 @Param[in]     h_App     - Application handle provided by the user at the driver
                            initialization time; either ports' h_App or the switche's
                            h_App.
 @Param[in]     errors    - Events causing the exception; may be any combination
                            of either e_L2swEventBits or e_L2swPortEventBits.
 @Param[in]     flags     - Protocol dependent flag, may contain additional information.

 @Return        None.
 *//***************************************************************************/
typedef void (t_L2swException) (t_Handle h_App, uint32_t errors, uint32_t flags);

/**************************************************************************//**
 @Description   L2 SWITCH PORT configuration parameters.
*//***************************************************************************/
typedef struct t_L2SwPortParam
{
    /* L2 switch port basic parameters */
    e_EnetMode          enetMode;       /**< Speed and interface */
    bool                fullDuplex;     /**< TRUE for full duplex; FALSE for half duplex */
    e_L2swPortState     portState;      /**< The state the port should be in; states are designed
                                             to support the spanning tree protocol. */
    uint16_t            portDestinationTable;
                                        /**< Port destination table. */

    /* System integration parameters */
    uint32_t            baseAddress;    /**< Pointer to L2 switch port registers in memory. */
    t_Handle            h_L2sw;         /**< Handle of the associated l2switch. */
    e_Ucc               uccId;          /**< UCC ID */
    e_QeClock           rxClock;        /**< Rx clock to use */
    e_QeClock           txClock;        /**< Tx clock to use */
    int                 irq;            /**< Interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle            h_App;          /**< Handle to application layer information;
                                             is passed as is to application in port's
                                             callback functions. */
    t_L2swException     *f_Exceptions;  /**< Exception callback routine the user wishes
                                             invocation of upon port event; In this case
                                             event register will be any combination of
                                             e_L2swPortEventBits. */
} t_L2SwPortParam;

/**************************************************************************//**
 @Description   L2 SWITCH configuration parameters.
 
                This structure includes switch general parameters as
                well as parameter relevant for CPU port configuration.
*//***************************************************************************/
typedef struct t_L2SwParam
{
    /* L2 switch basic parameters */
    e_L2swPortTableSelectionMode    tableSelectionMode;
                                            /**< Specifies if PDT (Port Destination Table ) or VDT
                                                 (VLAN Destination table) should be used for multicast
                                                 and unknown unicast frames arriving from CPU port
                                                 with destination address not found in ADLT
                                                 (ADdress Learning Table); PDT table is always used in
                                                 VLAN-unaware switch while VDT table is usually used in
                                                 VLAN-aware switch; It is also possible to force using
                                                 of PDT table in all cases by setting this parameter to
                                                 e_L2SW_PORT_TABLE_USE_PDT. */
    uint16_t                        pdt;    /**< Port destination table (PDT) for CPU port. */

    /* System integration parameters */
    t_Handle            h_Qe;               /**< Handle to the controlling QE. */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID. */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information;
                                                 Note restrictions on the f_GetBuf; disregarding them will
                                                 result in erronous behavior:
                                                 - Buffer must reside in the specified bus;
                                                 - Every buffer stores one full frame, thus
                                                   buffer must be sufficient to include max frame length + CRC size;
                                                 - Buffer must be 8 byte aligned. */
    uint32_t            dataMemAttributes;  /**< Memory attributes mask for data buffers. */

    /* Application interface */
    t_Handle            h_App;              /**< Handle to application layer information;
                                                 is passed as is to application in callback
                                                 functions. */
    t_RxStoreFunction   *f_RxStore;         /**< User callback function invoked by driver with received data;
                                                 "status" parameter passed to the function reflects Rx BD
                                                 status field and is combination of the following:
                                                      - e_L2swRxBD - reception reason indication (optional);
                                                      - e_L2swCpuRxBdFrameType - masks bits 4-6 of Rx BD and specifies
                                                        the type of received frame;
                                                      - e_L2swCpuRxCpuBdSourcePort - masks bits 7-10 of Rx BD and determines
                                                        the incoming port of received frame. */
    t_L2swException     *f_Exceptions;      /**< Exception callback routine the user wishes
                                                 invocation of upon switch event; In this case
                                                 event register will be any combination of
                                                 e_L2swEventBits. */
} t_L2SwParam;


/**************************************************************************//**
 @Function      L2SW_Config

 @Description   Constructs an L2 Switch based on user parameters.
 
                The Switch can then be initialized using L2SW_Init()
                and enabled using L2SW_Enable().
                Ports that are added to switch using L2SW_PORT_Config
                are internally registered by the L2Switch module.

 @Param[in]     p_L2SwParam - User parameters for l2 switch
 
 @Return        Handle of switch object on success, NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.
*//***************************************************************************/
t_Handle L2SW_Config(t_L2SwParam *p_L2SwParam);

/**************************************************************************//**
 @Function      L2SW_Init

 @Description   Initializes driver and hardware for the L2 Switch
                and its ports.
 
                Implicitly initializes all ports.
                Switch can then be enabled using L2SW_Enable().

 @Param[in]     h_L2sw - Switch handle to be initialized

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_L2sw must have been previously created using L2SW_Config().
*//***************************************************************************/
t_Error L2SW_Init(t_Handle h_L2sw);

/**************************************************************************//**
 @Function      L2SW_Free

 @Description   Frees the switch and all its associated resources including ports.

 @Param[in]     h_L2sw - Switch handle

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_Free(t_Handle h_L2sw);


/**************************************************************************//**
 @Function      L2SW_PORT_Config

 @Description   Constructs a new port adding it to the existing ports in the switch.
 
                If the switch has not been initialized yet, the L2SW_Init()
                and L2SW_Enable() routines will also initialize and enable
                the port's hardware. However in case port is added dynamically
                to already functional switch an explicit call to L2SW_PORT_Init()
                and L2SW_PORT_Enable() is required to enable port's operation.
 
 @Param[in]     p_L2SwPortParam - Port configuration parameters.

 @Return        Handle to the new port, or NULL in case of a failure.

 @Cautions      L2SW_Config must precede this call.
*//***************************************************************************/
t_Handle L2SW_PORT_Config(t_L2SwPortParam *p_L2SwPortParam);

/**************************************************************************//**
 @Function      L2SW_PORT_Init

 @Description   Initializes hardware and firmware port's resources.
 
                This function should be used only for ports added to switch
                after switch initialization, i.e. after L2SW_Init() function
                has been issued. Ports added to switch prior to L2SW_Init()
                time are initialized automatically by driver from within
                L2SW_Init() routine.
                In addition, if switch has already been enabled by L2SW_Enable()
                function, an explicit call to L2SW_EnablePort() is mandatory
                for bringing the port to operational state.
 
 @Param[in]     h_L2swPort - Port handle.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_L2swPort must have been previously created using L2SW_Config().
*//***************************************************************************/
t_Error L2SW_PORT_Init(t_Handle h_L2swPort);

/**************************************************************************//**
 @Function      L2SW_PORT_Free

 @Description   Deletes an existing port from the switch.
 
                Note: This routine is responsible of making sure that pending frames
                that have already been dispatched and that also have been
                transmitted on the referred port will not cause a deadlock.
                These frames are NOT guarantied to be sent on the referred port
                prior to port disablement and deletion.
 
 @Param[in]     h_L2swPort - Port handle.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_PORT_Free(t_Handle h_L2swPort);

/**************************************************************************//**
 @Group         l2switch_qe_advanced_init_grp L2 Switch Advanced Configuration Unit
 

 @Description   Configuration functions used to change default values.
 

 @{
*//***************************************************************************/
 

/**************************************************************************//**
 @Function      L2SW_ConfigDebugMode

 @Description   Configures frame types that will be forwarded to the host
                for debug purposes.
 
                Any combination of e_L2swDebugMode is possible.
 
                Default value: [e_L2SW_DEBUG_MODE_ERROR_FRAME_DISCARD].

 @Param[in]     h_L2sw  - Handle to L2 Switch
 @Param[in]     newVal  - Debug mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigDebugMode(t_Handle h_L2sw, uint32_t newVal);

/**************************************************************************//**
 @Function      L2SW_ConfigRxBdRingLen

 @Description   Configures RX CPU BD ring length for specific Rx queue.
 
                Default value: [16].

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - New BD length value
 @Param[in]     index    - Rx queue index

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigRxBdRingLen(t_Handle h_L2sw, uint16_t newVal, uint8_t index);


/**************************************************************************//**
 @Function      L2SW_ConfigTxBdRingLen

 @Description   Sets the number of entries in transmit BD ring of CPU port.
 
                Frames are placed into Tx BD ring when are required
                to be transmitted from CPU port.
 
                Default value: [16]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Tx BD ring length

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTxBdRingLen(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigQueueWeightAndLength

 @Description   Configures Tx CPU queues size and queues weight.
 
                Each transmission to CPU port is done via one of its
                L2SW_PORT_MAX_NUM_OF_TX_QUEUES queues. The queue is
                chosen according to the priority given to the frame.
                Later on frames are copied to CPU Rx BD ring.
 
                Note that as opposed to L2SW_ConfigTxBdRingLen()
                function dealing with transmit from CPU direction,
                this routine is relevant for receive to CPU side.
 
                The size of queue along with its Fair Queuing Weight,
                has an influence on its appropriate QOS.
                Frames are dropped by switch if queue they should
                have been forwarded to is full.
                A queue of size 0 is an unused queue.
                A queue with a higher weight is a better/more
                frequently served queue.
 
                Default weight: [2]
                Default size:   [16]
 
 @Param[in]     h_L2sw          - Handle to L2 Switch
 @Param[in]     newQueueSize    - New value for queue size
 @Param[in]     newQueueWeight  - New value for queue weight
 @Param[in]     index           - Tx queue index

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigQueueWeightAndLength(t_Handle h_L2sw, uint8_t newQueueWeight, uint8_t newQueueSize, uint8_t index);


/**************************************************************************//**
 @Function      L2SW_ConfigInterruptUponReceive

 @Description   Configures interrupt/polling mode for the specified Rx queue.
 
                Default value: [TRUE].

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Interrupt mode: TRUE for interrupt, FALSE for polling
 @Param[in]     index    - Rx queue index

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigInterruptUponReceive(t_Handle h_L2sw, bool newVal, uint8_t index);


/**************************************************************************//**
 @Function      L2SW_ConfigDeferredDriverRxRoutineInvocation

 @Description   Assigns the associated Rx queue an interrupt manager ID for
                a delayed Rx handling.
 
                If the interrupt manager ID is non-zero, the driver defers
                the invocation of L2SW_Rx() routine to the user. The driver
                notifies the user through XX_Call(), and the user is responsible
                for calling L2SW_Rx() eventually (usually not from within XX_Call()
                itself).

                XX_Call() will not be called again for the same receive queue
                until the user issues L2SW_Rx() function.

                If the interrupt manager ID is zero (default mode), the driver
                invokes the receive routine of the queue directly from the interrupt
                routine.

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Rx interrupt manager ID
 @Param[in]     index    - Queue index (0 through 3)

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigDeferredDriverRxRoutineInvocation(t_Handle h_L2sw, uint32_t newVal, uint8_t index);


/**************************************************************************//**
 @Function      L2SW_ConfigEmptyBdListsSizes

 @Description   Configures fast and regular size of Empty BD list.
 
                Internal Empty BD list buffers are used by switch when new
                frame is received. Up to 256 frames may be stored by switch
                internally.
 
                Empty BD list is separated into fast and regular parts.
                Fast BD list includes first M BDs of empty BD list while
                regular list includes the rest of BDs.
 
                Fast BDs point to M frames, the first 64 bytes of
                which are allocated in internal memory; the rest of the
                frames are in external memory.
 
                Regular BDs point to frames that are all allocated
                in external memory.

                The sum of both lists may not exceed 256.
 
                The default values of the fast and slow empty BD lists sizes are:
                [128] and [128]
                respectively.

 @Param[in]     h_L2sw                  - Handle to L2 Switch
 @Param[in]     newFastEmptyBdListSize  - New value for the fast empty BD list size
 @Param[in]     newSlowEmptyBdListSize  - New value for the slow empty BD list size

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigEmptyBdListsSizes(t_Handle h_L2sw, uint16_t newFastEmptyBdListSize, uint16_t newSlowEmptyBdListSize);

/**************************************************************************//**
 @Function      L2SW_ConfigTagMode

 @Description   Sets the tag mode of the switch.
 
                This function configures switch to operate in VLAN-aware or
                VLAN-unaware mode. This mode affects switch egress mode
                of operation. In VLAN-unaware mode frames forwarding is done
                based only on destination MAC address, while in VLAN-aware mode
                forwarding is done according to VLAN ID and destination address.
 
                Default value: [e_L2SW_TAG_MODE_BASIC]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Tag mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTagMode(t_Handle h_L2sw, e_L2swTagMode newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigVlanPriorityMatchingTable

 @Description   Sets the priority maching table for VLAN priority.
 
                L2 Switch supports up to 4 priorities. Thus, when QoS
                is based on VLAN priority, 8 existing VLAN priorities
                should be mapped to available 4 switch priorities.
                The priorities should be arranged two bits per priority
                in the following manner: two most significant bits for
                priority 0, followed by the two bits for priority 1 and
                so forth.
 
                Default value: [0x5Af]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - VLAN priority matching

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigVlanPriorityMatchingTable(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigIpPriorityTable

 @Description   Configures IP priority table.
 
                L2 switch supports up to 4 priorities. Thus, when QoS
                is implemented based on IP priority it is required to
                map 64 available IP priorities to 4 available switch
                priorities.

                For ease of use two routines are available: one that
                receives the entire table and copies it (this one);
                and another (L2SW_ConfigIpPriorityEntry())that
                configures a single table entry.

 @Param[in]     h_L2sw                  - Handle to L2 Switch
 @Param[in]     p_NewIpPriorityTable    - Pointer to IP priority table to copy from

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigIpPriorityTable(t_Handle h_L2sw, uint32_t* p_NewIpPriorityTable);

/**************************************************************************//**
 @Function      L2SW_ConfigIpPriorityEntry

 @Description   Configures one entry in IP priority table.
 
                L2 switch supports up to 4 priorities. Thus, when QoS
                is implemented based on IP priority it is required to
                map 64 available IP priorities to 4 available switch
                priorities.

                For ease of use two routines are available: one that
                receives the entire table and copies it (L2SW_ConfigIpPriorityTable());
                and another (this one) that configures a single table entry.

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - New table entry value
 @Param[in]     index    - Entry index (IP priority) between 0 and 63

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigIpPriorityEntry(t_Handle h_L2sw, e_L2swPriority newVal, uint8_t index);


/**************************************************************************//**
 @Function      L2SW_ConfigHashConflictMode

 @Description   Sets the hash conflict behavior.
 
                This function specifies the expected behavior in case of
                a hashing conflict. Hash conflict occurs when newly
                learned MAC address maps to already full hash entry.
 
                Hash conflict mode defines if new or old entry should
                be discarded upon conflict encountering.
 
                Default value: [e_L2SW_HASH_CONFLICT_DISCARD_NEW]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Hash conflict mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigHashConflictMode(t_Handle h_L2sw, e_L2swHashingConflictMode newVal);

/**************************************************************************//**
 @Function      L2SW_ConfigRmonCountersMode

 @Description   Sets the RMON counters mode.
 
                Default value: [e_L2SW_RMON_COUNTERS_ENABLED]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - RMON counters mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigRmonCountersMode(t_Handle h_L2sw, e_L2RmonCountersMode newVal);

/**************************************************************************//**
 @Function      L2SW_ConfigAgingMode

 @Description   Sets the aging mode.
 
                Aging mechanism mode indicates whether or not the aging
                mechanism should be active. In case of aging, entries will
                be deleted from address learning table if not being
                addressed after aging period of time.
 
                Default value: [e_L2SW_AGING_MECHANISM_DISABLED]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Aging mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigAgingMode(t_Handle h_L2sw, e_L2swAgingMechanismMode newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigAgingPeriod

 @Description   Sets the aging period for address learning table [sec].
 
                Indicate the period in seconds for each aging round.
                Any entry with MAC address that has not been referred
                during one round will age.
 
                Default value: [100]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Aging period

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigAgingPeriod(t_Handle h_L2sw, uint32_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigVdtSize

 @Description   Sets the size of VLAN Destination Table (VDT).
 
                Number of VLAN entries in the VLAN Destination Table.
 
                Default value: [64]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - VLAN Destination Table size

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigVdtSize(t_Handle h_L2sw, uint16_t newVal);

/**************************************************************************//**
 @Function      L2SW_ConfigAdltSize

 @Description   Set the size of the address learning table (ADLT).
 
                Number of entries in the Address Learning Table.
                Should be multiple of 16.
 
                Default value: [128]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Address learning table size

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigAdltSize(t_Handle h_L2sw, uint16_t newVal);

/**************************************************************************//**
 @Function      L2SW_ConfigLowestPriority

 @Description   Sets the priority viewed by switch as lowest.
 
                The lowest priority is the priority that will be assigned to
                unknown unicast frames or error frames.
 
                Default value: [e_L2SW_PRIORITY_LOW]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Switch lowest priority

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigLowestPriority(t_Handle h_L2sw, e_L2swPriority newVal);

/**************************************************************************//**
 @Function      L2SW_ConfigHighestPriority

 @Description   Sets the priority viewed by switch as highest.
 
                The highest priority is the priority that will be assigned to
                protocol frames (BPDU or IGMP snooping).
 
                Default value: [e_L2SW_PRIORITY_HIGH]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Switch highest priority

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigHighestPriority(t_Handle h_L2sw, e_L2swPriority newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigDestPortsIgmp

 @Description   Sets the ports that are valid IGMP destination ports.
 
                Destination Ports for IGMP protocol packets.
                May be any combination of any one of
                e_L2swDestinationPorts.
 
                Default value: [0x1]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - IGMP destination ports

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigDestPortsIgmp(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigHighGlobalThreshold

 @Description   Sets the high global threshold for Flow Control management.
 
                This parameter indicates what should be the high global threshold.
                If the number of frames in memory is equal to or greater
                than this value, the switch will send pause frames to all port.
 
                Individual threshold for every port limiting number of
                frames in memory from specific port is set by
                L2SW_PORT_ConfigHighPortThreshold() routine.
 
                Default value: [255]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - High global threshold

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigHighGlobalThreshold(t_Handle h_L2sw, uint8_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigLowGlobalThreshold

 @Description   Sets the low global threshold for Flow Control management.
 
                This parameter indicates what should be the low global threshold.
                If the number of frames in memory is equal to or less than
                this value, the switch will send pause frames to all ports with pause
                value 0 thus notifiyng that the switch is ready to accept frames again.
 
                Individual threshold for every port limiting number of
                frames in memory from specific port is set by
                L2SW_PORT_ConfigLowPortThreshold() routine.

                Default value: [180]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Low global threshold

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigLowGlobalThreshold(t_Handle h_L2sw, uint8_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigMaxFrameLength

 @Description   Sets the Maximum frame length.
 
                Maximum frame length excluding 4 CRC bytes.
 
                Default value: [1518]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Maximum frame length

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigMaxFrameLength(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigMinFrameLength

 @Description   Sets the Minimum frame length.
 
                Minimum frame length excluding 4 CRC bytes.
 
                Default value: [60]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Minimum frame length

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigMinFrameLength(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigExceptions

 @Description   Sets exception events to be received by user in interrupt.
 
                Expected events may be any combination of e_L2swEventBits.
 
                Default value: [L2SW_SWE_OTHER]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Exception events

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigExceptions(t_Handle h_L2sw, uint32_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortMirrorPorts

 @Description   Sets ports that should behave as mirror ports.
 
                A bit set for one port makes it a mirror port.
                A combination of ports enumerated in e_L2swDestinationPorts.
 
                Default value: [0]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Mirror ports

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortMirrorPorts(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortTableSelectionMode

 @Description   Determines table selection mode for CPU port.
 
                Indicates which table should be used for address resolution
                of frames received from CPU port.
 
                PDT (Port Destination Table ) or VDT (VLAN Destination table)
                may be used for multicast and unknown unicast frames with
                destination address not found in ADLT (ADdress Learning Table).
                PDT table is always used in VLAN-unaware switchwhile VDT table
                is usually used in VLAN-aware switch.It is also possible to force
                using of PDT table in all cases by setting this parameter to
                e_L2SW_PORT_TABLE_USE_PDT.
 
                Default value: [e_L2SW_PORT_TABLE_USE_PDT]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Table selection mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortTableSelectionMode(t_Handle h_L2sw, e_L2swPortTableSelectionMode newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPort1QTrunkMode

 @Description   Sets the termination to 1Q trunk mode.
 
                When operating in 1Q trunk mode CPU port accepts also frames
                with VLAN ID that termination port doesn't belong to.
                This mode is relevant only for VLAN-aware switch.
 
                Default value: [e_L2SW_PORT_1Q_NORMAL]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - 1Q trunk mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPort1QTrunkMode(t_Handle h_L2sw, e_L2swPort1qTrunkMode newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPort1DTrunkMode

 @Description   Sets the termination port to 1D trunk mode.
 
                In 1D trunk mode all frames are passed untagged. Tag is
                removed for tagged frames.
                This mode is relevant only for VLAN-aware switch.
 
                Default value: [e_L2SW_PORT_1D_NORMAL]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - 1D trunk mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPort1DTrunkMode(t_Handle h_L2sw, e_L2swPort1dTrunkMode newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortUntaggedPolicy

 @Description   Sets policy for CPU port upon receiving untagged frame.
 
                Indicates what is the expected behavior in case of
                of an incoming untagged frame from host CPU port.
                This functionality is relevant only for VLAN-aware switch.
                Untagged frames can be either discarded or default tag
                can be attached.
 
                Default value: [e_L2SW_PORT_DROP_UNTAGGED]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Untagged policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortUntaggedPolicy(t_Handle h_L2sw, e_L2swPortUntaggedPolicy newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortForceDefaultTag

 @Description   Forces default tag policy for CPU port.
 
                In forced default tag mode default tag is attached
                to all frames. If tagged frame is received its original
                tag is replaced by default one.
                This mode is relevant only for VLAN-aware switch.
 
                Default value: [e_L2SW_PORT_DROP_UNTAGGED]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Default tag policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortForceDefaultTag(t_Handle h_L2sw, e_L2swPortForceDefaultTag newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortAddExtraTag

 @Description   Determines add extra tag policy for CPU port.
 
                In add extra tag mode, extra tag is added to all frames
                transmitted by CPU port. If frames are already tagged
                an additional tag is added before the original one.
                This mode is relevant only for VLAN-aware switch.
 
                Default value: [e_L2SW_PORT_AET_NO_EXTRA_TAG_ADDITION]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Extra tag policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortAddExtraTag(t_Handle h_L2sw, e_L2swPortAddExtraTag newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortRemoveExtraTag

 @Description   Sets remove extra tag mode for CPU port.
 
                In this mode first/external tag is removed from all frames
                transmitted by CPU if presents. Second/inner tag is used
                then for determining forwarding policy.
                This mode is relevant only for VLAN-aware switch.
 
                Default value: [e_L2SW_PORT_RET_NO_EXTRA_TAG_REMOVAL]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Remove extra tag policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortRemoveExtraTag(t_Handle h_L2sw, e_L2swPortRemoveExtraTag newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigExtraTagType

 @Description   Sets the type of the extra tag.
 
                This function configures VLAN extra tag type  to a value
                different from 0x8100.
                Relevant when extra tag addition functionality is
                required for CPU port, see
                L2SW_ConfigTerminationPortAddExtraTag()
                routine.
 
                Default value: [0]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Extra tag VLAN type

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigExtraTagType(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigExtraTagId

 @Description   Sets the extra tag ID.
 
                This function configures extra tag VLAN ID.
                Relevant when extra tag addition functionality is
                required for CPU port, see
                L2SW_ConfigTerminationPortAddExtraTag() routines.
 
                Default value: [0]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Extra tag VLAN ID

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigExtraTagId(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortBroadcastDisable


 @Description   Sets termination port broadcast disable mode.
 
                Discard broadcast frame arriving from CPU port.
                This mode is used for broadcast storm avoidance.
 
                Default value: [e_L2SW_PORT_BROADCAST_ENABLE]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Broadcast disable mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortBroadcastDisable(t_Handle h_L2sw, e_L2swPortBroadcastDisable newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortIpPriority

 @Description   Sets priority mapping behavior for CPU port.
 
                L2 Switch QoS may be implemented based on VLAN
                or IP priority. Switch supports up to 4 priorities,
                thus 8 VLAN priorities or 64 IP priorities should
                be mapped to the available 4. See L2SW_ConfigIpPriorityTable(),
                L2SW_ConfigIpPriorityEntry() and L2SW_ConfigVlanPriorityMatchingTable()
                routines.
 
                This function determines QoS criteria to be applied
                first: may be VLAN or IP priority based.
 
                Default value: [e_L2SW_PORT_IGNORE_IP_PRIORITY]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Priority mapping behavior

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortIpPriority(t_Handle h_L2sw, e_L2swPortIPPriority newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortPriorityPolicy

 @Description   Sets Tx scheduling policy for CPU port.
 
                Determines Tx scheduling policy for 4 CPU port Tx queues,
                either fixed, WRR or mixed.
 
                Default value: [e_L2SW_PORT_WEIGHTED_PRIORITY]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Tx scheduling policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortPriorityPolicy(t_Handle h_L2sw, e_L2swPortPriorityPolicy newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortTransparent

 @Description   Sets CPU port to operate in transparent mode.
 
                In transparemt mode frames are kept as is, i.e.
                without tag removal/addition.
 
                Default value: [e_L2SW_PORT_TRANSPARENT_MODE_DISABLED]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Transparent mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortTransparent(t_Handle h_L2sw, e_L2swPortTransparentMode newVal);


/**************************************************************************//**
 @Function      L2SW_ConfigTerminationPortDefaultTag

 @Description   Sets default tag for CPU port.
 
                Default tag to be inserted to frames arriving from
                CPU port.
                This function is relevant when default tag addition
                was required by L2SW_ConfigTerminationPortUntaggedPolicy()
                routine or force default tag mode was set by
                L2SW_ConfigTerminationPortForceDefaultTag() routine.
 
                Default value: [0]

 @Param[in]     h_L2sw   - Handle to L2 Switch
 @Param[in]     newVal   - Default tag value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_Config() and before L2SW_Init().
*//***************************************************************************/
t_Error L2SW_ConfigTerminationPortDefaultTag(t_Handle h_L2sw, uint16_t newVal);


/**************************************************************************//**
 @Description   L2 Switch receiver decoding mode
*//***************************************************************************/
typedef enum e_L2swPortGUMRReceiverDecodingMethodRENC
{
    e_L2SW_PORT_RX_ENCODING_NRZ          = e_UCC_FAST_RX_ENCODING_NRZ      , /**< nrz  */
    e_L2SW_PORT_RX_ENCODING_NRZI         = e_UCC_FAST_RX_ENCODING_NRZI     , /**< nrzi */
    e_L2SW_PORT_RX_ENCODING_RESERVED0    = e_UCC_FAST_RX_ENCODING_RESERVED0, /**<      */
    e_L2SW_PORT_RX_ENCODING_RESERVED1    = e_UCC_FAST_RX_ENCODING_RESERVED1  /**<      */
} e_L2swPortGUMRReceiverDecodingMethodRENC;

/**************************************************************************//**
 @Description   L2 Switch transmitter encoding mode
*//***************************************************************************/
typedef enum e_L2swPortGUMRTransmitterEncodingMethodTENC
{
    e_L2SW_PORT_TX_ENCODING_NRZ          = e_UCC_FAST_TX_ENCODING_NRZ      , /**< nrz  */
    e_L2SW_PORT_TX_ENCODING_NRZI         = e_UCC_FAST_TX_ENCODING_NRZI     , /**< nrzi */
    e_L2SW_PORT_TX_ENCODING_RESERVED0    = e_UCC_FAST_TX_ENCODING_RESERVED0, /**<      */
    e_L2SW_PORT_TX_ENCODING_RESERVED1    = e_UCC_FAST_TX_ENCODING_RESERVED1  /**<      */
} e_L2swPortGUMRTransmitterEncodingMethodTENC;

/**************************************************************************//**
 @Description   L2 Switch diagnostic mode
*//***************************************************************************/
typedef enum e_L2swPortGUMRDiagnosticModeDIAG
{
    e_L2SW_PORT_DIAGNOSTIC_NORMAL            = e_UCC_FAST_DIAGNOSTIC_NORMAL            , /**< normal             */
    e_L2SW_PORT_DIAGNOSTIC_LOCAL_LOOP_BACK   = e_UCC_FAST_DIAGNOSTIC_LOCAL_LOOP_BACK   , /**< loop back          */
    e_L2SW_PORT_DIAGNOSTIC_AUTO_ECHO         = e_UCC_FAST_DIAGNOSTIC_AUTO_ECHO         , /**< echo               */
    e_L2SW_PORT_DIAGNOSTIC_LOOP_BACK_AND_ECHO= e_UCC_FAST_DIAGNOSTIC_LOOP_BACK_AND_ECHO  /**< loop back and echo */
} e_L2swPortGUMRDiagnosticModeDIAG;

/**************************************************************************//**
 @Description   L2 Switch CRC length
*//***************************************************************************/
typedef enum e_L2swPortGUMRTransparentCRCTypeTCRC
{
    e_L2SW_PORT_16_BIT_CRC     = e_UCC_FAST_16_BIT_CRC   , /**< 16 bit */
    e_L2SW_PORT_CRC_RESERVED0  = e_UCC_FAST_CRC_RESERVED0, /**<        */
    e_L2SW_PORT_32_BIT_CRC     = e_UCC_FAST_32_BIT_CRC   , /**< 32 bit */
    e_L2SW_PORT_CRC_RESERVED1  = e_UCC_FAST_CRC_RESERVED1  /**<        */
} e_L2swPortGUMRTransparentCRCTypeTCRC;

/**************************************************************************//**
 @Description   L2 Switch Sync length (transparent mode only)
*//***************************************************************************/
typedef enum e_L2swPortGUMRSyncLenSYNL
{
    e_L2SW_PORT_SYNC_LEN_NOT_USED  = e_UCC_FAST_SYNC_LEN_NOT_USED , /**< not used  */
    e_L2SW_PORT_SYNC_LEN_AUTOMATIC = e_UCC_FAST_SYNC_LEN_AUTOMATIC, /**< automatic */
    e_L2SW_PORT_SYNC_LEN_8_BIT     = e_UCC_FAST_SYNC_LEN_8_BIT    , /**< 8 bit     */
    e_L2SW_PORT_SYNC_LEN_16_BIT    = e_UCC_FAST_SYNC_LEN_16_BIT     /**< 16 bit    */
} e_L2swPortGUMRSyncLenSYNL;


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPortDestinationTable

 @Description   Sets valid destination ports for this port.
 
                Should be combination of ports enumerated in
                e_L2swDestinationPorts.
 
                Port destination table (PDT) or VLAN destination table (VDT)
                can be used for multicast or unknown unicast frames.
                When PDT is used multicast or unknown unicast frames are
                forwarded only to ports specified by this table.
 
 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Valid destination ports for this port

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPortDestinationTable(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigQueueWeightAndSize

 @Description   Configures port Tx queues size and queues weight.
 
                Each port transmission is done via one of its
                L2SW_PORT_MAX_NUM_OF_TX_QUEUES queues. The queue
                is chosen according to the priority given to the frame.
 
                Each queue size is limited by its size.  The size of
                queue along with its Fair Queuing Weight have an
                influence on its appropriate QOS.
                It is recommended that Queue sizes get a larger value
                than their proportion  to enable flexibility:
 
                proportion = 256 / (number_of_UCCs * number_of_queues_in_UCC)
 
                Frames are dropped by switch if queue they should
                have been forwarded to is full.
                A queue of size 0 is an unused queue.
                A queue with a higher weight is a better/more
                frequently served queue.
 
                Default weight: [2]
                Default size:   [64]
 
 @Param[in]     h_L2swPort      - Handle to L2 Switch Port
 @Param[in]     newQueueSize    - New value for queue size
 @Param[in]     newQueueWeight  - New value for queue weight
 @Param[in]     index           - Tx queue index

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigQueueWeightAndLength(t_Handle h_L2swPort, uint8_t newQueueWeight, uint8_t newQueueSize, uint8_t index);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigMirrorPort

 @Description   Sets the ports that are expected to serve as mirror ports.
 
                A bit set for one port makes it a mirror port.
                Should be combination of ports enumerated in e_L2swDestinationPorts.
 
                Default value: [0]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Mirror ports

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigMirrorPort(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTableSelectionMode

 @Description   Indicates which table should be used for address resolution
                of frames received from this port.
 
                PDT (Port Destination Table ) or VDT (VLAN Destination table)
                may be used for multicast and unknown unicast frames with
                destination address not found in ADLT (ADdress Learning Table).
                PDT table is always used in VLAN-unaware switchwhile VDT table
                is usually used in VLAN-aware switch.It is also possible to force
                using of PDT table in all cases by setting this parameter to
                e_L2SW_PORT_TABLE_USE_PDT.
 
                Default value: [e_L2SW_PORT_TABLE_USE_PDT]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Table selection mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTableSelectionMode(t_Handle h_L2swPort, e_L2swPortTableSelectionMode newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPort1qTrunkMode

 @Description   Sets the port to 1Q trunk mode.
 
                When operating in 1Q trunk mode port accepts also
                frames with VLAN ID that it doesn't belong to.
                This mode is relevant only for VLAN-aware switch.
 
                Default value: [e_L2SW_PORT_1Q_NORMAL]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - 1Q trunk mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPort1qTrunkMode(t_Handle h_L2swPort, e_L2swPort1qTrunkMode newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPort1dTrunkMode

 @Description   Sets the port to 1D trunk mode.
 
                In 1D trunk mode all frames are passed untagged. Tag is
                removed for tagged frames.
                This mode is relevant only for VLAN-aware switch.

                Default value: [e_L2SW_PORT_1D_NORMAL]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - 1D trunk mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPort1dTrunkMode(t_Handle h_L2swPort, e_L2swPort1dTrunkMode newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPortState

 @Description   Sets the ports state.
 
                For Spanning tree protocol support, some ports
                need to be kept disabled or updating tables yet
                not forwarding to other ports or CPU.
 
                The value specified here is initial port state.
                It may be changed later on in runtime through
                L2SW_PORT_SetState() routine invokation.
 
                Default value: [e_L2SW_PORT_STATE_FORWARDING]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Initial port state

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPortState(t_Handle h_L2swPort, e_L2swPortState newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigUntaggedPolicy

 @Description   Sets policy for the port upon receiving untagged frame.
 
                Indicates what is the expected behavior in case of
                of an incoming untagged frame from this port.
                This functionality is relevant only for VLAN-aware switch.
                Untagged frames can be either discarded or default tag
                can be attached.
 
                Default value: [e_L2SW_PORT_DROP_UNTAGGED]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Untagged policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigUntaggedPolicy(t_Handle h_L2swPort, e_L2swPortUntaggedPolicy newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigForceDefaultTag

 @Description   Forces default tag policy for the port.
 
                In forced default tag mode default tag is attached
                to all frames. If tagged frame is received its original
                tag is replaced by default one.
                This mode is relevant only for VLAN-aware switch.

                Default value: [e_L2SW_PORT_NOT_FORCING_DEFAULT_TAG]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Default tag policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigForceDefaultTag(t_Handle h_L2swPort, e_L2swPortForceDefaultTag newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigAddExtraTag

 @Description   Determines add extra tag policy for the port.
 
                In add extra tag mode, extra tag is added to all frames
                transmitted by this port. If frames are already tagged
                an additional tag is added before the original one.
                This mode is relevant only for VLAN-aware switch.

                Default value: [e_L2SW_PORT_AET_NO_EXTRA_TAG_ADDITION]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Extra tag policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigAddExtraTag(t_Handle h_L2swPort, e_L2swPortAddExtraTag newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRemoveExtraTag

 @Description   Sets remove extra tag mode for the port.
 
                In this mode first/external tag is removed from all frames
                transmitted by this port if presents. Second/inner tag is used
                then for determining forwarding policy.
                This mode is relevant only for VLAN-aware switch.

                Default value: [e_L2SW_PORT_RET_NO_EXTRA_TAG_REMOVAL]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Remove extra tag policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRemoveExtraTag(t_Handle h_L2swPort, e_L2swPortRemoveExtraTag newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigExtraTagType

 @Description   Sets the type of the extra tag.
 
                This function configures VLAN extra tag type  to a value
                different from 0x8100.
                Relevant when extra tag addition functionality is
                required for this port, see L2SW_PORT_ConfigAddExtraTag()
                routine.
 
                Default value: [0x0]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Extra tag VLAN type

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigExtraTagType(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigExtraTagId

 @Description   Sets the extra tag ID.
 
                This function configures extra tag VLAN ID.
                Relevant when extra tag addition functionality is
                required for this port, see
                L2SW_PORT_ConfigAddExtraTag() routine.
 
                Default value: [0]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Extra tag VLAN ID

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigExtraTagId(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigDefaultTag

 @Description   Sets default tag for the port.
 
                Default tag to be inserted to frames arriving from
                this port.
                This function is relevant when default tag addition
                was required by L2SW_PORT_ConfigUntaggedPolicy ()
                routine or force default tag mode was set by
                L2SW_PORT_ConfigForceDefaultTag () routine.

                Default value: [0x0555]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Default tag value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigDefaultTag(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigProtocolFramesTreatment

 @Description   Sets Protocol frames treatment, i.e. IGMP or BPDU frames.
 
                Indicates whether or not they should be forwarded
                to the CPU port.
 
                Default value: [e_L2SW_PORT_BPDU_NOT_PROMPTING_TO_CPU]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Protocol frames policy.

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigProtocolFramesTreatment(t_Handle h_L2swPort, e_L2swPortProtocolFramesTreatment newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigBroadcastDisable

 @Description   Sets the port to broadcast disable mode.
 
                Discard broadcast frame arriving from this port.
                This mode is used for broadcast storm avoidance.

                Default value: [e_L2SW_PORT_BROADCAST_ENABLE].

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Broadcast disable mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigBroadcastDisable(t_Handle h_L2swPort, e_L2swPortBroadcastDisable newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigIPPriority

 @Description   Sets priority mapping behavior for the port.
 
                L2 Switch QoS may be implemented based on VLAN
                or IP priority. Switch supports up to 4 priorities,
                thus 8 VLAN priorities or 64 IP priorities should
                be mapped to the available 4. See L2SW_ConfigIpPriorityTable(),
                L2SW_ConfigIpPriorityEntry() and L2SW_ConfigVlanPriorityMatchingTable()
                routines.
 
                This function determines QoS criteria to be applied
                first for this port: may be VLAN or IP priority based.

                Default value: [e_L2SW_PORT_IGNORE_IP_PRIORITY]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Priority mapping behavior

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigIPPriority(t_Handle h_L2swPort, e_L2swPortIPPriority newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPriorityPolicy

 @Description   Sets Tx scheduling policy for the port.
 
                Determines Tx scheduling policy for 4 port Tx queues,
                either fixed, WRR or mixed.

                Default value: [e_L2SW_PORT_FIXED_PRIORITY]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Tx scheduling policy

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPriorityPolicy(t_Handle h_L2swPort, e_L2swPortPriorityPolicy newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTransparent

 @Description   Sets the port to operate in transparent mode.
 
                In transparemt mode frames are kept as is, i.e.
                without tag removal/addition.
 
                Default value: [e_L2SW_PORT_TRANSPARENT_MODE_DISABLED]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Transparent mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTransparent(t_Handle h_L2swPort, e_L2swPortTransparentMode newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRisc

 @Description   Configure the active RISC.
 
                As several RISCs are available, a balanced switch behavior is reached
                if the expected processing time is divided between RISCs in a
                balanced manner. This decision is done offline therefore the
                balance is done based on expected Flows.
 
                Allocation of more than one RISC is not permitted for an L2 switch port.

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     riscNum     - RISC selection

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init(). RISC number must be between 1 and QE_NUM_OF_RISCS.
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRisc(t_Handle h_L2swPort, uint8_t riscNum);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigHighPortThreshold

 @Description   Sets the high threshold for Flow Control management for this port.
 
                This parameter indicates what should be the high global threshold.
                If the number of frames in memory from this port is equal to or greater
                than this value, the switch will send pause frames on the port.
 
                In addition, there is also switch global threshold limiting
                total number of frames in memory, see L2SW_ConfigHighGlobalThreshold()
                routine.

                Default value: [255]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - High global threshold

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigHighPortThreshold(t_Handle h_L2swPort, uint8_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigLowPortThreshold

 @Description   Sets the low global threshold for Flow Control management for this port.
 
                This parameter indicates what should be the low global threshold.
                If the number of frames in memory from this port is equal to or less than
                this value, the switch will send pause frames on the ports with pause
                value 0 thus notifiyng that the switch is ready to accept frames again.
 
                In addition, there is also switch global threshold limiting
                total number of frames in memory, see L2SW_ConfigLowGlobalThreshold()
                routine.

                Default value: [48]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Low global threshold

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigLowPortThreshold(t_Handle h_L2swPort, uint8_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigJumboFrames

 @Description   Allows jumbo frame reception.
 
                TRUE to enable.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Jumbo frame acceptance mode
 
 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigJumboFrames(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigReceiveShortFrames

 @Description   Sets the receive short frames mode.
 
                TRUE to receive short frames,
                FALSE to discard short frames.
 
                Short frames are frames with length shorter
                than minimal frame length set by
                L2SW_ConfigMinFrameLength() routine.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Short frame acceptance mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigReceiveShortFrames(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigExceptions

 @Description   Sets exception events from this port to be received by
                user in interrupt.
 
                Expected events may be any combination of e_L2swPortEventBits.

                Default value: [L2SW_PORT_UCCE_OTHER]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Exception events

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigExceptions(t_Handle h_L2swPort, uint32_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRxFlow

 @Description   Handles flow control frames when set to TRUE.
 
                Default value: [TRUE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Rx flow control mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRxFlow(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTxFlow

 @Description   Allows Flow Control (pause) frames transmission if TRUE.
 
                Default value: [TRUE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Tx flow control mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTxFlow(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigStatistics

 @Description   Sets the HW statistics gathering mode.
 
                TRUE to keep statistics counters for Tx and Rx.
                Note that microcode statistics mode is copied
                from L2Switch RMON mode configured through
                L2SW_ConfigRmonCountersMode() routine.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Statistics mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigStatistics(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPreambleLength

 @Description   Sets the length of the preamble field in the frame (3-7).
 
                Default value: [3]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Preamble length

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPreambleLength(t_Handle h_L2swPort, uint8_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigLengthCheck

 @Description   Enables frame length check by MAC layer.
 
                The MAC checks the frames length field to ensure it
                matches the actual data field length.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Length check mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigLengthCheck(t_Handle h_L2swPort, bool newVal);

/**************************************************************************//**
 @Function      L2SW_PORT_ConfigPadCrc

 @Description   Enables padding and/or CRC appending mode.
 
                Pad short frames and append CRC at end.
 
                Default value: [e_L2SW_PORT_MACCFG2_PAD_AND_CRC_MODE_CRC_ONLY]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - Pad/CRC mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigPadCrc(t_Handle h_L2swPort, e_L2swPortMaccfg2PadAndCrcMode newVal);

/**************************************************************************//**
 @Function      L2SW_PORT_ConfigCrcEnable

 @Description   Defines if CRCshould be appended to the frame.
 
                Default value: [TRUE]

 @Param[in]     h_L2swPort  - Handle to L2 Switch Port
 @Param[in]     newVal      - CRC enable mode

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigCrcEnable(t_Handle h_L2swPort, bool newVal);

/**************************************************************************//**
 @Function      L2SW_PORT_ConfigGrantSupport

 @Description   TRUE to provide grant support.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigGrantSupport(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTSA
 
 @Description   Configures UCC port to connect to the Time-Slot Assigner
                (TSA) instead of the normal pins.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTSA(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRTSM

 @Description   FALSE to send idles between frames,
                TRUE to send flags/syncs between frames.
 
                Default value: [FALSE]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRTSM(t_Handle h_L2swPort, bool newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRxVirtFifoSize

 @Description   UCC Rx virtual FIFO Size register.
 
                Default value: [0x200]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRxVirtFifoSize(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRxVirtFifoEmergThresh

 @Description   UCC Rx virtual FIFO Emergency Threshold.
 
                Default value: [0x100]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRxVirtFifoEmergThresh(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigRxVirtFifoSpecEmergThresh

 @Description   UCC Rx virtual FIFO Special Emergency Threshold.
 
                Default value: [0x180]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigRxVirtFifoSpecEmergThresh(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTxVirtFifoSize

 @Description   UCC Tx virtual FIFO Size register.
 
                Default value: [0x200]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTxVirtFifoSize(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTxVirtFifoEmergThresh

 @Description   UCC Tx virtual FIFO Emergency Threshold.
 
                Default value: [0x100]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTxVirtFifoEmergThresh(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTxVirtFifoTxThresh

 @Description   UCC Tx virtual FIFO Tx Threshold.
 
                Default value: [0x80]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTxVirtFifoTxThresh(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTxPollingTimer

 @Description   UCC Tx Polling Time.
 
                Default value: [256]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTxPollingTimer(t_Handle h_L2swPort, uint16_t newVal);


/**************************************************************************//**
 @Function      L2SW_PORT_ConfigDiagnosticMode

 @Description   Set diagnostic mode for this UCC port.
 
                Default value: [e_UCC_FAST_DIAGNOSTIC_NORMAL]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigDiagnosticMode(t_Handle h_L2swPort, e_L2swPortGUMRDiagnosticModeDIAG newVal);

/**************************************************************************//**
 @Function      L2SW_PORT_ConfigTbiPhysicalAddress

 @Description   Configures the TBI physical address base for this UCC.
 
                UCC's TBI address is set to the number given here plus
                this UCC number's offset from UCC1.
 
                For example, if the address given here is 5, and the UccId is
                e_UCC_3, then this UCC will be assigned TBI address equal to
                5 + (e_UCC_3 - e_UCC_1) = 5 + 2 = 7.
 
                Default value: [0x10]

 @Param[in]     h_L2swPort  - Handle to UCC L2SW_PORT
 @Param[in]     newVal      - New value

 @Return        E_OK on success, error code otherwise.

 @Cautions      Allowed only following L2SW_PORT_Config() and before L2SW_Init()
                or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_ConfigTbiPhysicalAddress(t_Handle h_L2swPort, uint8_t newVal);


/** @} */ /* end of l2switch_qe_advanced_init_grp group */
/** @} */ /* end of group L2 SWITCH Initialization Unit.*/

/**************************************************************************//**
 @Group         l2switch_qe_control_id L2 SWITCH Control Unit

 @Description   L2 SWITCH Control functions.

 @{
*//***************************************************************************/
/** packed structures **/
#ifdef __MWERKS__
#pragma pack(push,1)
#endif /* __MWERKS__ */

/**************************************************************************//**
 @Description   L2 SWITCH PORT RX RMON COUNTER
*//***************************************************************************/
typedef _Packed struct t_L2swRxRmonCounter{
    uint32_t  RxByteCnt;      /**< The total number of received octets.                       */
    uint32_t  Rx64;           /**< The total number of received packets exactly 64 octets long. */
    uint32_t  RxL64;          /**< The total number of received packets
                                   of length < 64 with valid CRC.                             */
    uint32_t  RxL64Crce;      /**< The total number of received packets
                                   of length < 64 with erroneous CRC.                         */
    uint32_t  Rx65t127;       /**< The total number of received packets with length
                                   within the range: [65,127]                                 */
    uint32_t  Rx128t255;      /**< The total number of received packets with length
                                   within the range: [128,255]                                */
    uint32_t  Rx256t511;      /**< The total number of received packets with length
                                   within the range: [256,511]                                */
    uint32_t  Rx512t1023;     /**< The total number of received packets with length
                                   within the range: [512,1023]                               */
    uint32_t  Rx1024t1522;    /**< The total number of received packets with length
                                   within the range: [1024,1522]                              */
    uint32_t  RxJabber;       /**< The total number of received packets with length
                                   greater than 1522, and a valid CRC.                        */
    uint32_t  RxJabberErr;    /**< The total number of received packets with length
                                   greater than 1522, and an invalid CRC.                     */
    uint32_t  RxBC;           /**< The total number of broadcast packets received, including bad ones.    */
    uint32_t  RxUC;           /**< The total number of unicast packets received, including bad ones.      */
    uint32_t  RxMC;           /**< The total number of multicast packets received, including bad ones.    */
    uint32_t  RxFC;           /**< The total number of flow control packets received, including bad ones. */
} _PackedType t_L2swRxRmonCounter;

/**************************************************************************//**
 @Description   L2 SWITCH PORT TX RMON COUNTER
*//***************************************************************************/
typedef _Packed struct t_L2swTxRmonCounter{
    uint32_t  TxByteCnt;      /**< The total number of transmitted octets.                        */
    uint32_t  Tx64;           /**< The total number of transmitted packets exactly 64 octets long.*/
    uint32_t  TxL64;          /**< The total number of transmitted packets
                                   of length < 64 .                                       */
    uint32_t  Tx65t127;       /**< The total number of transmitted packets with length
                                   within the range: [65,127]                             */
    uint32_t  Tx128t255;      /**< The total number of transmitted packets with length
                                   within the range: [128,255]                            */
    uint32_t  Tx256t511;      /**< The total number of transmitted packets with length
                                   within the range: [256,511]                            */
    uint32_t  Tx512t1023;     /**< The total number of transmitted packets with length
                                   within the range: [512,1023]                           */
    uint32_t  Tx1024t1522;    /**< The total number of transmitted packets with length
                                   within the range: [1024,1522]                          */
    uint32_t  TxJabber;       /**< The total number of transmitted packets with length
                                   greater than 1522.                                     */
    uint32_t  TxBC;           /**< The total number of broadcast packets transmitted.     */
    uint32_t  TxUC;           /**< The total number of unicast packets transmitted.       */
    uint32_t  TxMC;           /**< The total number of multicast packets transmitted.     */
    uint32_t  TxFC;           /**< The total number of flow control packets transmitted.  */
} _PackedType t_L2swTxRmonCounter;

/**************************************************************************//**
 @Description   L2 SWITCH PORT Rx error counters
*//***************************************************************************/
typedef _Packed struct t_L2swRxErrorCounter{
    uint32_t  secec;  /**< Security error counter.                                        */
    uint32_t  missc;  /**< Miss counter, no destination port found.                       */
    uint32_t  mflec;  /**< Memory full counter.                                              */
    uint32_t  crcec;  /**< Rx FCS error counter.                                          */
    uint32_t  alec;   /**< Rx alignment error counter (nonoctet aligned frame arrived).   */
    uint32_t  orec;   /**< Rx overrun occurred during frame reception.                    */
    uint32_t  colc;   /**< Rx collision counter.                                          */
}_PackedType t_L2swRxErrorCounter;

/**************************************************************************//**
 @Description   L2 SWITCH PORT Tx error counters
*//***************************************************************************/
typedef _Packed struct t_L2swTxErrorCounter{
    uint32_t  TxDftCnt;   /**< Tx defer counter, deferring while trying to transmit.      */
    uint32_t  TxCrsCnt;   /**< Tx carrier sense loss counter (during frame transmission). */
    uint32_t  TxUnCnt;    /**< Tx underrun counter, transmitter ran into underrun condition,
                               while sending.                                             */
    uint32_t  TxLcCnt;    /**< Tx late collision counter.                                  */
    uint32_t  TxRlCnt;    /**< Tx excessive collision counter; transmit failure due
                               to maximum allowed repeated colisions number.                         */
    uint32_t  TxClCnt;    /**< Tx collision counter.   */
    uint32_t  TxExdCnt;   /**< Tx excessive defer counter.                                */
}_PackedType t_L2swTxErrorCounter;
#ifdef __MWERKS__
#pragma pack(pop)
#endif /* __MWERKS__ */

/**************************************************************************//**
 @Description   L2 SWITCH ADLT optional statuses.
*//***************************************************************************/
typedef enum e_L2swAdltStatus
{
    e_L2SW_ADLT_STATUS_INVALID_ENTRY   = 0,    /**< Entry is invalid.*/
    e_L2SW_ADLT_STATUS_AGING_BIT_ZERO  = 0x1,  /**< Entry has aged a 'bit'.*/
    e_L2SW_ADLT_STATUS_LOCKED_ENTRY    = 0x2,  /**< Locked entry: will not age.*/
    e_L2SW_ADLT_STATUS_AGING_BIT_SET   = 0x3   /**< Entry is as young as can be.*/
}e_L2swAdltStatus;


/**************************************************************************//**
 @Function      L2SW_Enable

 @Description   Enables switch and all its ports configured so far.

 @Param[in]     h_L2sw  - Switch handle

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_L2sw must have been previously created using L2SW_Config()
                and initialized using L2SW_Init().
*//***************************************************************************/
t_Error L2SW_Enable(t_Handle h_L2sw);

/**************************************************************************//**
 @Function      L2SW_Disable

 @Description   Disables switch and all its ports configured so far.

 @Param[in]     h_L2sw  - Switch handle

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_Disable(t_Handle h_L2sw);

/**************************************************************************//**
 @Function      L2SW_TransmittedBDsRelease

 @Description   Releases buffers of already transmitted frames sent from CPU.
 
                Invoking this routine is optional. Buffers will
                eventually be released when new buffers will be sent
                (at most Size_Of_Tx_Bd_Ring are pending for release).
                But if no transmition is going to take place in the
                near future, and the user would like to make use of
                the pending buffers, user may choose to invoke this
                routine.
 
 @Param[in]     h_L2sw  - Switch handle

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_TransmittedBDsRelease(t_Handle h_L2sw);

/**************************************************************************//**
 @Function      L2SW_GetRmonCounters

 @Description   Retrieve statistics counters for CPU port.
 
                Counters will be retrieved only for an uncleared counter argument.
                Setting either argument to 0 (NULL), will avoid the retrievement
                of the associatede counter but will NOT be regarded as an error.
 
                Note: RxRmonCounter will eventually hold the counters of frames
                received by the switch from the CPU while TxRmonCounter will eventually
                hold the counters of frames transmitted to the CPU.

 @Param[in]     h_L2sw          - Switch handle
 @Param[out]    p_RxRmonCounter - Pointer to Rx statistics structure
 @Param[out]    p_TxRmonCounter - Pointer to Tx statistics structure

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error  L2SW_GetRmonCounters(t_Handle            h_L2sw,
                              t_L2swRxRmonCounter *p_RxRmonCounter,
                              t_L2swTxRmonCounter *p_TxRmonCounter );

/**************************************************************************//**
 @Function      L2SW_SetAdltEntry

 @Description   Inserts an entry to the Address Learning Table.

 @Param[in]     h_L2sw           - Switch handle
 @Param[in]     enetAddr         - MAC address of the inserted entry
 @Param[in]     destinationPorts - Destination ports frames should be forwarded to;
                                   Combination of e_L2swDestinationPorts
 @Param[in]     status           - Entry status

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_SetAdltEntry(t_Handle          h_L2sw,
                          t_EnetAddr        enetAddr,
                          uint16_t          destinationPorts,
                          e_L2swAdltStatus  status);

/**************************************************************************//**
 @Function      L2SW_GetAdltEntry

 @Description   Lookups the entry with a given MAC address and
                retrieves the associated destination ports.

 @Param[in]     h_L2sw              - Switch handle
 @Param[in]     enetAddr            - MAC address of the looked up entry
 @Param[in]     p_DestinationPorts  - Destination ports: combination of
                                      e_L2swDestinationPorts if found, zero
                                      otherwise

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_GetAdltEntry(t_Handle   h_L2sw,
                          t_EnetAddr enetAddr,
                          uint16_t   *p_DestinationPorts);
 

/**************************************************************************//**
 @Function      L2SW_ClearAdltEntry

 @Description   Deletes Address Learning Table entry content
                of a given MAC address.

 @Param[in]     h_L2sw           - Switch handle
 @Param[in]     enetAddr         - MAC address of the deleted entry
 
 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_ClearAdltEntry(t_Handle h_L2sw, t_EnetAddr enetAddr);

/**************************************************************************//**
 @Function      L2SW_ClearAdlt

 @Description   Clears the entire Address Learning Table.

 @Param[in]     h_L2sw           - Switch handle

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_ClearAdlt(t_Handle h_L2sw);

/**************************************************************************//**
 @Function      L2SW_SetVdtEntry

 @Description   Set VLAN Destination Entry in the VLAN Destination Table.

 @Param[in]     h_L2sw           - Switch handle
 @Param[in]     vid              - VLAN ID of the inserted entry
 @Param[in]     sourcePorts      - Source ports belonging to this VLAN ID group;
                                   conbination of e_L2swSourcePorts
 @Param[in]     destinationPorts - Destination ports belonging to this VLAN ID group;
                                   conbination of e_L2swDestinationPorts

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_SetVdtEntry(t_Handle h_L2sw,
                         uint16_t vid,
                         uint16_t sourcePorts,
                         uint16_t destinationPorts);
 

/**************************************************************************//**
 @Function      L2SW_GetVdtEntry

 @Description   Lookups entry in the VLAN Destination Table with
                specified VLAN ID.
 
 @Param[in]     h_L2sw             - Switch handle
 @Param[in]     vid                - VLAN ID of the looked up entry
 @Param[in]     p_SourcePorts      - Source ports belonging to this VLAN ID group;
                                     conbination of e_L2swSourcePorts if entry is
                                     found, zero otherwise
 @Param[in]     p_DestinationPorts - Destination ports belonging to this VLAN ID group;
                                     conbination of e_L2swDestinationPorts if entry is
                                     found, zero otherwise

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_GetVdtEntry(t_Handle h_L2sw,
                         uint16_t vid,
                         uint16_t *p_SourcePorts,
                         uint16_t *p_DestinationPorts);
 

/**************************************************************************//**
 @Function      L2SW_ClearVdtEntry

 @Description   Clears VLAN ID entry in VLAN Destination Table.
 
 @Param[in]     h_L2sw           - Switch handle
 @Param[in]     vid              - VLAN ID of the deleted entry

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_ClearVdtEntry(t_Handle h_L2sw, uint16_t vid);
 

/**************************************************************************//**
 @Function      L2SW_ClearVdt

 @Description   Clears entire VLAN Destination Table.

 @Param[in]     h_L2sw           - Switch handle

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_ClearVdt(t_Handle h_L2sw);
 

/**************************************************************************//*
 @Function      L2SW_GetLeastAllocatedRiscNum

 @Description   Returns the number of the RISC that is
                allocated to the least number of ports.
 
 @Param[in]     h_L2sw           - Switch handle

 @Return        RISC number (between 1 and QE_NUM_OF_PORTS).
*//***************************************************************************/
uint8_t L2SW_GetLeastAllocatedRiscNum(t_Handle h_L2sw);


/**************************************************************************//*
 @Function      L2SW_IncAllocatedRiscNum

 @Description   Increments the number of ports that a RISC is allocated to.
 
 @Param[in]     h_L2sw      - Switch handle
 @Param[in]     riscNum     - The number of the RISC that is allocated to some port.

 @Cautions      RISC number must be between 1 and QE_NUM_OF_RISCS.
*//***************************************************************************/
void L2SW_IncAllocatedRiscNum(t_Handle h_L2sw, uint8_t riscNum);


/**************************************************************************//*
 @Function      L2SW_DecAllocatedRiscNum

 @Description   Decrements the number of ports that a RISC is allocated to.
 
 @Param[in]     h_L2sw      - Switch handle
 @Param[in]     riscNum     - The number of the RISC that is no longer allocated to some port.

 @Cautions      RISC number must be between 1 and QE_NUM_OF_RISCS.
*//***************************************************************************/
void L2SW_DecAllocatedRiscNum(t_Handle h_L2sw, uint8_t riscNum);


#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      L2SW_DumpRegs

 @Description   Dumps L2Switch and CPU Port parameters RAM.

 @Param[in]     h_L2sw - Handle to L2Switch object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_L2sw must have been previously created using L2SW_Config()
                and initialized using L2SW_Init().
*//***************************************************************************/
t_Error L2SW_DumpRegs(t_Handle h_L2sw);
#endif

/**************************************************************************//**
 @Function      L2SW_PORT_Enable

 @Description   Enables the port.
 
                This routine will enable port that has either been added after
                the entire switch was already enabled (using L2SW_Enable())
                or port that was disabled using L2SW_PORT_Disable().

 @Param[in]     h_L2swPort  - Port handle.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_PORT_Enable(t_Handle h_L2swPort);

/**************************************************************************//**
 @Function      L2SW_PORT_Disable

 @Description   Disables the port.
 
                This routine is responsible of making sure that pending frames
                that have already been dispatched and that would also have been
                transmitted on the referred port, will not cause a deadlock.
                These frames are NOT guarantied to be sent on the referred port
                prior to port disablement.

 @Param[in]     h_L2swPort  - Port handle.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_PORT_Disable(t_Handle h_L2swPort);

/**************************************************************************//**
 @Function      L2SW_PORT_GetRmonCounters

 @Description   Retrieves statistics counters.
 
                Counters will be retrieved only for an uncleared counter argument.
                Setting either argument to 0 (NULL), will avoid the retrievement
                of the associatede counter but will NOT be regarded as an error.

 @Param[in]     h_L2swPort      - Port handle
 @Param[out]    p_RxRmonCounter - Pointer to Rx statistics structure
 @Param[out]    p_TxRmonCounter - Pointer to Tx statistics structure

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error  L2SW_PORT_GetRmonCounters(t_Handle            h_L2swPort,
                                   t_L2swRxRmonCounter *p_RxRmonCounter,
                                   t_L2swTxRmonCounter *p_TxRmonCounter);

/**************************************************************************//**
 @Function      L2SW_PORT_GetErrorCounters

 @Description   Retrieves error counters.
 
                Note counters will be retrieved only for an uncleared
                counter argument. Setting either argument to 0 (NULL),
                will avoid the retrievement of the associatede counter
                but will NOT be regarded as an error.

 @Param[in]     h_L2swPort              - Port handle
 @Param[out]    p_L2swRxErrorCounter    - Pointer to Rx errors statistics structure
 @Param[out]    p_L2swTxErrorCounter    - Pointer to Tx errors statistics structure

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error  L2SW_PORT_GetErrorCounters(t_Handle h_L2swPort,
                                    t_L2swRxErrorCounter *p_L2swRxErrorCounter,
                                    t_L2swTxErrorCounter *p_L2swTxErrorCounter);

/**************************************************************************//**
 @Function      L2SW_PORT_SetPdt

 @Description   Sets Port Destination Table of a specified port.
 
 @Param[in]     h_L2swPort       - Port handle
 @Param[in]     destinationPorts - Destination ports that are to be available
                                   from the referred port; combination of
                                   e_L2swDestinationPorts

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error L2SW_PORT_SetPdt(t_Handle h_L2swPort, uint16_t destinationPorts);
 

/**************************************************************************//**
 @Function      L2SW_PORT_SetState

 @Description   Sets the working state of a specified port.
 
 @Param[in]     h_L2swPort      - Port handle
 @Param[in]     state           - New port state

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error   L2SW_PORT_SetState(t_Handle h_L2swPort, e_L2swPortState state);


#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      L2SW_PORT_DumpRegs

 @Description   Dumps L2Switch Port parameters RAM.

 @Param[in]     h_L2swPort - Handle to L2Switch Port object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_L2swPort must have been previously created using L2SW_PORT_Config()
                and initialized using L2SW_Init() or L2SW_PORT_Init().
*//***************************************************************************/
t_Error L2SW_PORT_DumpRegs(t_Handle h_L2swPort);
#endif  /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of group L2 SWITCH Control Unit*/

/**************************************************************************//**
 @Group         l2switch_qe_runtime_id L2 SWITCH Runtime Unit

 @Description   L2 SWITCH Runtime functions.

 @{
*//***************************************************************************/
#define L2SW_RX_FRAME_TYPE_MASK         0x0E000000  /**< Frame type, indicates the type of received frame. */
#define L2SW_RX_FRAME_TYPE_OFFSET       25          /**< Location of the type, offset from LSB */

/**************************************************************************//**
  L2 SWITCH termination Port Rx BD
*//***************************************************************************/
typedef enum e_L2swRxBD
{
    e_L2SW_BD_MISS                  = 0x4000,   /**< Miss - destination port was not found. */
    e_L2SW_BD_VLAN_SECURITY_PROBLEM = 0x2000,   /**< Vlan Security Problem.                 */
    e_L2SW_BD_MISS_OR_VLAN_MASK     = 0x6000    /**< Mask both bits for easy comparison.*/
} e_L2swRxBD;

/**************************************************************************//**
 @Description   Incoming frames type
*//***************************************************************************/
typedef enum e_L2swCpuRxBdFrameType
{
    e_L2SW_RX_FRAME_TYPE_UNICAST    = 0,								/**< Incoming frame type: unicast.*/
    e_L2SW_RX_FRAME_TYPE_MULTICAST  = 0x1 << L2SW_RX_FRAME_TYPE_OFFSET, /**< Incoming frame type: multicast.*/
    e_L2SW_RX_FRAME_TYPE_BROADCAST  = 0x2 << L2SW_RX_FRAME_TYPE_OFFSET, /**< Incoming frame type: broadcast.*/
    e_L2SW_RX_FRAME_TYPE_IGMP       = 0x3 << L2SW_RX_FRAME_TYPE_OFFSET, /**< Incoming frame type: igmp.*/
    e_L2SW_RX_FRAME_TYPE_FLOW_CTRL  = 0x4 << L2SW_RX_FRAME_TYPE_OFFSET, /**< Incoming frame type: flow control.*/
    e_L2SW_RX_FRAME_TYPE_BPDU       = 0x5 << L2SW_RX_FRAME_TYPE_OFFSET, /**< Incoming frame type: BPDU (spanning tree protocol).*/
    e_L2SW_RX_FRAME_TYPE_MASK       = 0x7 << L2SW_RX_FRAME_TYPE_OFFSET  /**< Incoming frame type mask.*/
} e_L2swCpuRxBdFrameType;

#define L2SW_RX_SP_MASK     0x01E00000  /**< Source Port mask. */
#define L2SW_RX_SP_OFFSET   21          /**< Location of the source port, offset from LSB. */

/**************************************************************************//**
 @Description   Received Frame Source port BD status indicator
*//***************************************************************************/
typedef enum e_L2swCpuRxCpuBdSourcePort
{
    e_L2SW_RX_BD_SOURCE_PORT_0 = 0,							/**< Incoming frame arrived from source port 0.*/
    e_L2SW_RX_BD_SOURCE_PORT_1 = 0x1 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 1.*/
    e_L2SW_RX_BD_SOURCE_PORT_2 = 0x2 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 2.*/
    e_L2SW_RX_BD_SOURCE_PORT_3 = 0x3 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 3.*/
    e_L2SW_RX_BD_SOURCE_PORT_4 = 0x4 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 4.*/
    e_L2SW_RX_BD_SOURCE_PORT_5 = 0x5 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 5.*/
    e_L2SW_RX_BD_SOURCE_PORT_6 = 0x6 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 6.*/
    e_L2SW_RX_BD_SOURCE_PORT_7 = 0x7 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 7.*/
    e_L2SW_RX_BD_SOURCE_PORT_8 = 0x8 << L2SW_RX_SP_OFFSET,  /**< Incoming frame arrived from source port 8 (cpu port).*/
    e_L2SW_RX_BD_SOURCE_PORT_MASK = 0xF << L2SW_RX_SP_OFFSET/**< Mask in the bits regarding the arriving frame port attributes.*/
} e_L2swCpuRxCpuBdSourcePort;

/**************************************************************************//**
 @Description   L2 SWITCH  PORT Event Register.
*//***************************************************************************/
typedef enum e_L2swPortEventBits
{
    e_L2SW_PORT_UCCE_GRA  = 0x00008000, /**< The port has gracefuly stopped as requested. */
    e_L2SW_PORT_UCCE_HC   = 0x00004000, /**< An ADLT Hash conflict occurred. */
    e_L2SW_PORT_UCCE_CTQF = 0x00002000, /**< CPU transmit queue is full. */
    e_L2SW_PORT_UCCE_TXE  = 0x00001000, /**< Tx Error has been encountered. */
    e_L2SW_PORT_UCCE_GFC  = 0x00000200, /**< Global Flow control event. */
    e_L2SW_PORT_UCCE_MF   = 0x00000100, /**< Memory is full, no room for new frames. */
    e_L2SW_PORT_UCCE_TQF0 = 0x00000080, /**< Port 0 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF1 = 0x00000040, /**< Port 1 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF2 = 0x00000020, /**< Port 2 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF3 = 0x00000010, /**< Port 3 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF4 = 0x00000008, /**< Port 4 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF5 = 0x00000004, /**< Port 5 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF6 = 0x00000002, /**< Port 6 transmit queue - full. */
    e_L2SW_PORT_UCCE_TQF7 = 0x00000001  /**< Port 7 transmit queue - full. */
} e_L2swPortEventBits;

/**************************************************************************//**
 @Description   L2 SWITCH  Event Register.
*//***************************************************************************/
typedef enum e_L2swEventBits
{
    e_L2SW_SWE_BSY  = 0x00008000,   /**< Busy - lack of empty Rx BDs caused frame discard.         */
    e_L2SW_SWE_HC   = 0x00004000,   /**< Frame arriving from CPU caused a hashing conflict. */
    e_L2SW_SWE_CTQF = 0x00002000,   /**< CPU transmit queue is full.                              */
    e_L2SW_SWE_RXF0 = 0x00001000,   /**< A frame has arrived on pririty queue 0.                                     */
    e_L2SW_SWE_RXF1 = 0x00000800,   /**< A frame has arrived on pririty queue 1.                                     */
    e_L2SW_SWE_RXF2 = 0x00000400,   /**< A frame has arrived on pririty queue 2.                                     */
    e_L2SW_SWE_RXF3 = 0x00000200,   /**< A frame has arrived on pririty queue 3.                                     */
    e_L2SW_SWE_MF   = 0x00000100,   /**< Memory is full, current incoming frame from CPU was dropped. */
    e_L2SW_SWE_TQF0 = 0x00000080,   /**< Port 0 transmit queue - full. */
    e_L2SW_SWE_TQF1 = 0x00000040,   /**< Port 1 transmit queue - full. */
    e_L2SW_SWE_TQF2 = 0x00000020,   /**< Port 2 transmit queue - full. */
    e_L2SW_SWE_TQF3 = 0x00000010,   /**< Port 3 transmit queue - full. */
    e_L2SW_SWE_TQF4 = 0x00000008,   /**< Port 4 transmit queue - full. */
    e_L2SW_SWE_TQF5 = 0x00000004,   /**< Port 5 transmit queue - full. */
    e_L2SW_SWE_TQF6 = 0x00000002,   /**< Port 6 transmit queue - full. */
    e_L2SW_SWE_TQF7 = 0x00000001    /**< Port 7 transmit queue - full. */
} e_L2swEventBits;

/**************************************************************************//**
 @Function      L2SW_Tx

 @Description   Transmits a data buffer.
 
                Buffer must include the entire frame, multi-buffer
                frames are not supported by switch.
 
                Ports for transmition may be specified explicitly
                (exp indication is set) or alternatively, determined
                by switch ingress rule.

                Note: The transmitted buffer is 'consumed' by the switch
                and the user may NOT regain ownership on the buffer.
                The buffer will continue to be available for switch use.
 
                Following limitations regarding buffers should be met:
                 - buffer containing the frame must be as long as a maximum frame
                   length including CRC
                 - buffer's pointer to data must be 8 byte aligned
                 - additional 4 bytes should be allocated before data to allow
                   storing buffer context

                User may avoid the constraints above by using L2SW_TxCopiedData()
                that copies Tx data prior to transmition and returns immediately
                user buffer back to application.

 @Param[in]     h_L2sw                  - Handle to the L2 Switch.
 @Param[in]     p_Data                  - Pointer to data to transmit
 @Param[in]     h_BufContext            - User's context for the given data buffer
 @Param[in]     length                  - Length of data to transmit
 @Param[in]     exp                     - Explicit ports and priority (yes or no)
 @Param[in]     portsForTransmission    - An indication of which ports to transmit
                                          this frame to; relevant only if exp is set
                                          to TRUE, otherwise this parameter is ignored;
                                          Should be combination of e_L2swDestinationPorts
 @Param[in]     priority                - The explicit priority requested for transmission;
                                          relevant only when exp is set to TRUE.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Not meeting described above buffer demands may results in
                erratic behavior.
                h_L2sw must have been previously created using L2SW_Config()
                and initialized using L2SW_Init().
*//***************************************************************************/
t_Error L2SW_Tx(t_Handle        h_L2sw,
                uint8_t         *p_Data,
                t_Handle        h_BufContext,
                uint16_t        length,
                bool            exp,
                uint16_t        portsForTransmission,
                e_L2swPriority  priority);

/**************************************************************************//**
 @Function      L2SW_TxCopiedData

 @Description   Copies and transmits a data buffer.
 
                Buffer must include the entire frame, multi-buffer
                frames are not supported by switch.

                Ports for transmition may be specified explicitly
                (exp indication is set) or alternatively, determined
                by switch ingress rule.

                Note: buffer will be copied by this routine prior to
                transmition thus enabling reuse of the buffer by user.
                This also avoids the need for meeting buffer constraints
                as explained in L2SW_Tx() description. However, due to
                buffer copying this routine is less efficient than
                alternative L2SW_Tx() function.
 
 @Param[in]     h_L2sw                  - Handle to the L2 Switch
 @Param[in]     p_Data                  - Pointer to data to transmit
 @Param[in]     length                  - Length of data to transmit.
 @Param[in]     exp                     - Explicit ports and priority (yes or no)
 @Param[in]     portsForTransmission    - An indication of which ports to transmit
                                          this frame to; relevant only if exp is set
                                          to TRUE, otherwise this parameter is ignored;
                                          Should be combination of e_L2swDestinationPorts
 @Param[in]     priority                - The explicit priority requested for transmission;
                                          relevant only when exp is set to TRUE.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_L2sw must have been previously created using L2SW_Config()
                and initialized using L2SW_Init().
*//***************************************************************************/
t_Error L2SW_TxCopiedData(t_Handle         h_L2sw,
                          uint8_t          *p_Data,
                          uint16_t         length,
                          bool             exp,
                          uint16_t         portsForTransmission,
                          e_L2swPriority   priority);
 

/**************************************************************************//**
 @Function      L2SW_Rx

 @Description   Handles data received in the specified receive queue.
 
                Buffer will include the entire frame, multi-buffer
                frames are not supported by switch.

                Receive data are passed to user via user-defined f_RxStore
                callback routine provided during L2SW_Config() time.
 
                When working in polling mode, this routine must be invoked
                often enough to keep BD ring from overflowing.
                In interrupt mode this routine is invoked by driver
                from within interrupt context.
 
 @Param[in]     h_L2sw  - channel on which to receive.
 @Param[in]     rxQ     - Rx queue on which to receive.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_L2sw must have been previously created using L2SW_Config()
                and initialized using L2SW_Init().
*//***************************************************************************/
t_Error L2SW_Rx(t_Handle h_L2sw, uint8_t rxQ);

/** @} */ /* end of group L2 SWITCH Runtime Unit*/
/** @} */ /* end of group L2 SWITCH */


#endif /* __L2_SWITCH_EXT_H */
