/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          md5_ext.h

 @Description   structures and definitions for operating MD5 algorithm.

*//***************************************************************************/
#ifndef __MD5_EXT_H
#define __MD5_EXT_H

#define MD5_OUTPUT 16

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_md5 SEC MD5

 @Description   SEC MD5 functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_md5_init SEC MD5 Initialization Unit

 @Description   SEC MD5 initialization functions.

 @{
*//***************************************************************************/

/* accessory routine */
/**************************************************************************//**
 @Function      md5_alloc_ctx

 @Description   Allocate memory for the md5 ctx structure.
 
                The routine is not a part of the Crypto Api standard routines.

 @Return        ctx - ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle md5_alloc_ctx();

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      md5_init

 @Description   The function initializes the fields in the md5 ctx structure.

				The function initializes the fields in the md5 ctx
                structure.
                In crypto API the function initializes a default
                context value. However, in Crypto hash calculation,
                this initialization is not relevant because it's
                part of the hardware accelerators operation.
                The routine is a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void md5_init(void *ctx);

/**************************************************************************//**
 @Function      md5_update

 @Description   The routine sets the data source and its length in the md5 ctx structure.

				The routine sets the data source and its length in
                the md5 ctx structure. Those fields are important
                for initiating the md5 request.
                The routine is a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     data    - source input data.
 @Param[in]     len     - the source input length.

 @Return        None.
*//***************************************************************************/
void md5_update(void *ctx, const unsigned char *data, unsigned int len);

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      md5_update_ctx

 @Description   The routine initiates some fields in the ctx structure.

				The routine sets the src that means the algorithm
                context and its length in the specific md5 context
                field(p_HashCtx). Those fields are important for
                initiating the md5 request.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     src - hash algorithm context.
 @Param[in]     len - hash context length.

 @Return        None.
*//***************************************************************************/
void md5_update_ctx(void *ctx, unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      md5_setpad

 @Description   The routine initiates the PAD option.

				The function sets the Pad variable in the ctx
                structure to indicate that the user asked for padding.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void md5_setpad(void *ctx);

/**************************************************************************//**
 @Function      md5_setidgs

 @Description   The routine indicates the idgs - Initialize digest operation.

				The function sets the idgs variable in the ctx
	            structure to indicate that the user asked for idgs possibility.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void md5_setidgs(void *ctx);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_md5_runtime SEC MD5 Runtime Unit

 @Description   SEC MD5 Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      md5_final

 @Description   The routine operate the MD5 request.

				The routine fills in a specific request with the
	            md5 ctx fields and then calls to ioctl function in order
                to process the specific request.
                The routine is a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     out - the destination output data.

 @Return        None.
*//***************************************************************************/
void md5_final(void *ctx, unsigned char *out);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec md5 */
/** @} */ /* end of group sec */

#endif /* __MD5_EXT_H */

