/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          mtc_mcc_ext.h

 @Description   MTC MCC Application Programming Interface
*//***************************************************************************/
#ifndef __MTC_MCC_EXT_H
#define __MTC_MCC_EXT_H

#include "std_ext.h"
#include "mtc_ext.h"
#include "ucc_atm_port_ext.h"

/**************************************************************************//**
 @Group         mtc_grp             MTC Application Programming Interface

 @Description   MTC API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mtc_mcc_grp    MTC MCC Application Programming Interface

 @Description   MTC MCC API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mtc_mcc_common_grp    MTC MCC COMMON Application Programming Interface

 @Description   MTC MCC API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mtc_mcc_common_init_grp    MTC MCC COMMON Initialization Unit

 @Description   MTC MCC COMMON initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define MTC_MCC_EVENT_QOV1 0x80000000
#define MTC_MCC_EVENT_QOV2 0x20000000
#define MTC_MCC_EVENT_QOV3 0x08000000
#define MTC_MCC_EVENT_QOV4 0x02000000
#define MTC_MCC_EVENT_QOV5 0x00800000
#define MTC_MCC_EVENT_QOV6 0x00200000
#define MTC_MCC_EVENT_QOV9 0x00008000
#define MTC_MCC_EVENT_QOV10 0x00002000
#define MTC_MCC_EVENT_QOV11 0x00000800
#define MTC_MCC_EVENT_QOV12 0x00000200
#define MTC_MCC_EVENT_QOV13 0x00000080
#define MTC_MCC_EVENT_QOV14 0x00000020
#define MTC_MCC_EVENT_QOV15 0x00000008
#define MTC_MCC_EVENT_QOV16 0x00000002

#define MTC_MCC_EVENT_QOV (MTC_MCC_EVENT_QOV1|MTC_MCC_EVENT_QOV2|MTC_MCC_EVENT_QOV3|MTC_MCC_EVENT_QOV4|MTC_MCC_EVENT_QOV5|MTC_MCC_EVENT_QOV6|MTC_MCC_EVENT_QOV9|MTC_MCC_EVENT_QOV10|\
                      MTC_MCC_EVENT_QOV11|MTC_MCC_EVENT_QOV12|MTC_MCC_EVENT_QOV13|MTC_MCC_EVENT_QOV14| MTC_MCC_EVENT_QOV15|MTC_MCC_EVENT_QOV16)


typedef enum
{
    e_MCC_EVENT_Q1      = 0x40000000
    ,e_MCC_EVENT_Q2     = 0x10000000
    ,e_MCC_EVENT_Q3     = 0x04000000
    ,e_MCC_EVENT_Q4     = 0x01000000
    ,e_MCC_EVENT_Q5     = 0x00400000
    ,e_MCC_EVENT_Q6     = 0x00100000
    ,e_MCC_EVENT_Q9     = 0x00004000
    ,e_MCC_EVENT_Q10    = 0x00001000
    ,e_MCC_EVENT_Q11    = 0x00000400
    ,e_MCC_EVENT_Q12    = 0x00000100
    ,e_MCC_EVENT_Q13    = 0x00000040
    ,e_MCC_EVENT_Q14    = 0x00000010
    ,e_MCC_EVENT_Q15    = 0x00000004
    ,e_MCC_EVENT_Q16    = 0x00000001
    ,e_MCC_EVENT_NONE   = 0x00000000
}e_MCC_INT_Q;

typedef enum
{
    e_MTC_MCC_RX_STATE_HUNT  = 0x00
    ,e_MTC_MCC_RX_STATE_PRESYNC = 0x01
    ,e_MTC_MCC_RX_STATE_SYNC = 0x3
}e_MtcMccRxState;

typedef void (t_MtcMccCommonExceptions) ( t_Handle appId, uint32_t errors, uint32_t flags );

typedef struct
{
    uint16_t mtcIntQNum;            /**<1 - 256. important that in init MTC_COMMON
                                    peripheralId of this IntQ will match mccId*/
    e_MCC_INT_Q  mapMtcIntQToMccIntQ;     /**match MtcIntQ(1 of 256) to Mcc Interrupt Queue(1 of 16)*/
}t_MapIntQToMcce;

#define MAX_NUM_OF_MCCS 2
/**************************************************************************//**
 @Description   MTC MCC COMMON initialization structure, used when calling MCC_Config.
*//***************************************************************************/
typedef struct
{
    e_Mcc                       mccId[MAX_NUM_OF_MCCS];          /**< MCC ID */
    t_Handle                    h_App;          /**< Handle to the upper layer handler */
    t_Handle                    h_Qe;           /**< Handle to the QE object */
    t_Handle                    h_Mcc[MAX_NUM_OF_MCCS];		/**< Handle to the Fisrt MCC object */
    t_Handle                    h_MtcCommon;    /**<handler to MTC_MCC_COMMON*/
    t_MapIntQToMcce             *p_MapMtcIntQToMccIntQ[MAX_NUM_OF_MCCS];
                                                /**< array of mapping IntQToMcce (1 of 256 to 1 of 16)*/
    uint16_t                    numOfIntQMappedToMcc;
    t_MtcMccCommonExceptions    *f_Exceptions;
}t_MtcMccCommonParam;


/**************************************************************************//**
 @Function      MTC_MCC_COMMON_Config

 @Description   Configures a global MTC_MCC object shared by all MTC's on MCC.
                This routine allocates the MtcMccCommon structure and sets user
                and default parameters in the MTC_MCC_COMMON driver parameters structure.

 @Param[in]     p_MtcMccCommonParam - A handle to the initializing parameter
                                      structure for the MTC_MCC_COMMON.
 @Return        A handle (descriptor) of the newly created MTC_MCC_COMMON structure.
*//***************************************************************************/
t_Handle MTC_MCC_COMMON_Config(t_MtcMccCommonParam * p_MtcMccCommonParam);

/**************************************************************************//**
 @Function      MTC_MCC_COMMON_Init

 @Description   Initializes a global MTC_MCC_COMMON object.
 @Param[in]     h_MtcMccCommon  - A handle of the MTC_MCC_COMMON object

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_MCC_COMMON_Init(t_Handle h_MtcMccCommon);

/**************************************************************************//**
 @Function      MTC_MCC_COMMON_Free

 @Description   Frees all global MTC_MCC_COMMON object
 @Param[in]     h_MtcMccCommon  - A handle of the MTC_MCC_COMMON object

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_MCC_COMMON_Free(t_Handle h_MtcMccCommon);


/**************************************************************************//**
 @Group         mtc_mcc_common_advanced_init_grp MTC MCC COMMON Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      MTC_MCC_COMMON_ConfigExceptions

 @Description   an interrupt will be generated for the events
                represented by this mask (an 'or' of MCC_EVENT_XXX
                definitions. [MCC_COMMON_EVENT_TQOV]

 @Param[in]     h_MtcMccCommon  - A handle of the MTC_MCC_COMMON object
 @Param[in]     newVal          - New value

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MTC_MCC_COMMON_ConfigExceptions(t_Handle h_MtcMccCommon, uint32_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_COMMON_ConfigChannelUnderrun

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                received channel underrun indication. [TRUE]

 @Param[in]     h_MtcMccCommon  - A handle of the MTC_MCC_COMMON object
 @Param[in]     newVal          - New value

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MTC_MCC_COMMON_ConfigChannelUnderrun(t_Handle h_MtcMccCommon, bool newVal);

/** @} */ /* end of mtc_mcc_common_advanced_init_grp group */
/** @} */ /* end of mtc_mcc_common_init_grp group */
/** @} */ /* end of mtc_mcc_common_grp group */


/**************************************************************************//**
 @Group         mtc_mcc_init_grp    MTC MCC Initialization Unit

 @Description   MTC MCC initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/
typedef enum
{
      e_MTC_MCC_PORT_SCHED_LEVEL_1  = e_UCC_ATM_PORT_SCHED_LEVEL_1
    , e_MTC_MCC_PORT_SCHED_LEVEL_2  = e_UCC_ATM_PORT_SCHED_LEVEL_2
    , e_MTC_MCC_PORT_SCHED_LEVEL_3   = e_UCC_ATM_PORT_SCHED_LEVEL_3
    , e_MTC_MCC_PORT_SCHED_LEVEL_4   = e_UCC_ATM_PORT_SCHED_LEVEL_4
    , e_MTC_MCC_PORT_SCHED_LEVEL_5    = e_UCC_ATM_PORT_SCHED_LEVEL_5
    , e_MTC_MCC_PORT_SCHED_LEVEL_6  = e_UCC_ATM_PORT_SCHED_LEVEL_6
    , e_MTC_MCC_PORT_SCHED_LEVEL_7  = e_UCC_ATM_PORT_SCHED_LEVEL_7
    , e_MTC_MCC_PORT_SCHED_LEVEL_8  = e_UCC_ATM_PORT_SCHED_LEVEL_8
    , e_MTC_MCC_PORT_TOTAL_SCHED_LEVELS = e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS/* Must be last */
}e_MtcMccPortSchedLevels;

#define MTC_MCC_MAX_GCRA_LEVELS 4

typedef struct
{
    uint8_t    schedLevels;/**< number of the required Apc levels */
    uint32_t   fastChannel;/**< the rate of the fast channel - bits per second */
    uint32_t   slowChannel[e_MTC_MCC_PORT_TOTAL_SCHED_LEVELS];/**< the rate of the slow channel - bits per second per priority level */
    uint32_t   lineRate; /**< the line rate */
 
}t_MtcMccPortParam;

typedef struct
{
    e_MtcMccPortSchedLevels      firstUbrPlusSchedLevel;/**< the first scheduling level of UBR Plus traffic type */
    uint8_t                      numOfUbrPlusPriorityLevels; /**< in how many priority levels the user is going to work with ubr+ */
    uint8_t                      thresholdVal[e_MTC_MCC_PORT_TOTAL_SCHED_LEVELS];/**< the UBR+ MDA value for a specific priority level */
}t_MtcMccPortApcPriUbrPlusParam;

/**************************************************************************//**
 @Description   Information regarding GBR traffic of channels on a APC port last priority
*//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt[e_MTC_MCC_PORT_TOTAL_SCHED_LEVELS];
                                            /**< If true - the control slot indicates that TCTE should be read for each channel;
                                                 Must be false in case of working with apc scalable mode or with traffic type which is different then CBR */

    uint8_t     maxIteration;               /**< Max iteration allowed */
    bool        burstInhibit;               /**< Indicates if enable the burst inhibit option in the scheduler_MODE field */
    uint8_t     schedScaleFactor;           /**< Scheduler scaling factor */

} t_MtcMccQeApcSchedParam;

/**************************************************************************//**
 @Description   Information regarding GBR traffic of channels on a GCRA port last priority
 *//***************************************************************************/
typedef enum
{
    e_MTC_MCC_GCRA_NO_GBR_MODE      = e_UCC_ATM_GCRA_NO_GBR_MODE,     /**< There are no GBR channels in this GCRA port */
    e_MTC_MCC_GCRA_MIXED_GBR_MODE   = e_UCC_ATM_GCRA_MIXED_GBR_MODE,  /**< Some of the channels on last priority work with GBR traffic */
    e_MTC_MCC_GCRA_FULL_GBR_MODE    = e_UCC_ATM_GCRA_FULL_GBR_MODE    /**< All channels on last priority work with GBR traffic */
 
} e_MtcMccQeGcraGbrMode;

/**************************************************************************//**
 @Description   GCRA Scheduler initialization structure.
 
                For MTC_MCC_PORT_ConfigVpShapingScheduler and MTC_MCC_PORT_ConfigGcraScheduler.
 *//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt;      /**< No extended connection table needed for this port's channels */
    bool        isInternal;         /**< Indication if the user asked to work with internal/external channels in this port */
    uint8_t     gcraChannels;       /**< The actual number of gcra channels per the specific port */
    bool        workConservingMode; /**< Work Conserving Mode - Vps on the last priority are scheduled according to their
                                         weighted PCR/PCR Fraction regardless of the system clock */
    uint8_t     *p_WrrPriPercentages;/**< Array of up to 4 WRR (Weighted Round Robin) priorities percentages;
                                         Relevant only for WRR Mode, in which selection between the up to 4 priorities
                                         is done according to their relative weights; Note: For regular fixed priority mode
                                         (non-WRR) this pointer should be NULL. */
    e_MtcMccQeGcraGbrMode gbrMode;    /**< Full GBR mode / Mixed GBR mode / No GBR mode */

} t_MtcMccQeGcraSchedParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level APC initialization structure.

                This structurte includes the parameters needed for a single
                APC parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    vpi;                        /**< Virtual Path Identifier of this VP group channels */
 
   	/* Scheduling level */
	e_MtcMccPortSchedLevels	schedLevel;		/**< Scheduling level in high level GCRA scheduler */
    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[e_MTC_MCC_PORT_TOTAL_SCHED_LEVELS];
                                            /**< The rate of the slow channel - bits per second per priority level */

    /* APC specific parameters */
    t_MtcMccQeApcSchedParam apcSchedParam;

} t_MtcMccQeLowLevelApcParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level GCRA initialization structure.

                This structurte includes the parameters needed for a single
                GCRA parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    vpi;                        /**< Virtual Path Identifier of this VP group channels */
	/* Scheduling level */
	e_MtcMccPortSchedLevels	schedLevel;		/**< Scheduling level in high level GCRA scheduler */
    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[MTC_MCC_MAX_GCRA_LEVELS];
                                            /**< The rate of the slow channel per priority level (in bits per second) */

    /* GCRA specific parameters */
    t_MtcMccQeGcraSchedParam gcraSchedParam;

} t_MtcMccQeLowLevelGcraParam;

/**************************************************************************//**
 @Description   VP Shaping Scheduling initialization structure.
 *//***************************************************************************/
typedef struct
{
    uint8_t                         vpsNum;             /**< Number of Vps under this Vp Shaping port */
    bool                            autoVpOnOff;        /**< Auto Vp On Off enable */
    t_MtcMccQeLowLevelApcParam      *p_LowLevelApcSchedParam;
                                                        /**< If not NULL => GCRA over APC Vp Shaping (Low level
                                                        scheduling type is APC) and this is the array of all
                                                        low level APC schedulers parameters */
    t_MtcMccQeLowLevelGcraParam     *p_LowLevelGcraSchedParam;
                                                        /**< If not NULL => GCRA over GCRA Vp Shaping (Low level
                                                        scheduling type is GCRA) and this is the array of all
                                                        low level GCRA schedulers parameters */
} t_MtcMccPortQeVpShapingParam;

/**************************************************************************//**
 @Description   MiniCam Structure
 *//***************************************************************************/
typedef struct
{
    bool                        uniMode;        /**< Indicates if to take the GFC value into the CAM search or mask it
                                                     FALSE - take the GFC field,
                                                     TRUE - mask the GFC field in the incoming cell */
    bool                        isOverlapping;  /**< Indication if the specific port belongs to an overlap group */
    uint8_t                     overlapGroup;   /**< The number of the overlap group ,
                                                     NOTE: This parameter is relevant only in case the isOverlapping parameter is set */
    e_MaxChannelsPerMiniCamPort numOfChannels;  /**< The actual number of how many channels are going to work under this MiniCam port
                                                     NOTE: The parameter is important to improve performance
                                                     and also to search for an available channel code */
} t_MtcMccPortMiniCamParam;

/**************************************************************************//**
 @Description   MtcMcc initialization structure, used when calling MTC_MCC
*//***************************************************************************/
typedef struct
{
    t_Handle            h_App;              /**< Handle to the upper layer handler             */
    t_Handle            h_AtmController;    /**< Handle to the UccAtm            */
    uint16_t            txIntQ;             /**< Rx queue number   */
    uint16_t            rxIntQ;             /**< Tx queue number.                   */

    t_Handle            h_MtcMccCommon;     /**<handler to MTC MCC COMMON*/
    t_MtcMccPortParam   portParam;          /**<ATM port user defined parameters*/
    void (*f_Isr)(t_Handle, uint16_t);
    uint16_t numOfTdmRxSlots;
    uint16_t numOfTdmTxSlots;
      uint8_t *rxSlots;
    uint8_t *txSlots;
    t_Handle h_TdmRxFrame;
    t_Handle h_TdmTxFrame;
    e_Tdm tdmId;
    e_Mcc mccId;
    t_Handle h_Mcc;
    t_Handle h_Qe;
    t_Handle h_MtcCommon;
} t_MtcMccParam;


/**************************************************************************//**
 @Function      MTC_MCC_Config

 @Description   Configures a  MTC_MCC object.
 
                This routine allocates the MtcMcc structure and sets user
                and default parameters in the MTC_MCC driver parameters structure.

 @Param[in]     p_MtcMccParam  - Pointer to the MTC_MCC configuration
                                 parameters structure.

 @Return        A handle (descriptor) of the newly created MTC_MCC structure.
*//***************************************************************************/
t_Handle MTC_MCC_Config(t_MtcMccParam *p_MtcMccParam);

/**************************************************************************//**
 @Function      MTC_MCC_Init

 @Description   Initializes a MTC_MCC object.
 @Param[in]     h_MtcMcc - A handle of the MTC_MCC object

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_MCC_Init(t_Handle h_MtcMcc);

/**************************************************************************//**
 @Function      MTC_MCC_Free

 @Description   Frees a memory allocated by MTC_MCC object.
 @Param[in]     h_MtcMcc - A handle of the MTC_MCC object

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_MCC_Free(t_Handle h_MtcMcc);

/**************************************************************************//**
 @Function      MTC_MCC_Enable

 @Description   Enables MTC_MCC object according to requested mode
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     activeMode  - Direction (RX, TX, RX_TX)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_MCC_Enable(t_Handle h_MtcMcc, e_CommMode activeMode);

/**************************************************************************//**
 @Function      MTC_MCC_Disable

 @Description   Disables MTC_MCC object according to requested mode
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     activeMode  - Direction (RX, TX, RX_TX)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_Disable(t_Handle h_MtcMcc, e_CommMode activeMode);

/**************************************************************************//**
 @Function      MTC_MCC_RxAtmEnable

 @Description   Enables passing up the cells to the ATM microcode
 
 @Param[in]     h_MtcMcc - A handle of the MTC_MCC object

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_RxAtmEnable(t_Handle h_MtcMcc);

/**************************************************************************//**
 @Function      MTC_MCC_RxAtmDisable

 @Description   Disables passing up the cells to the ATM microcode
 
 @Param[in]     h_MtcMcc - A handle of the MTC_MCC object

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_RxAtmDisable(t_Handle h_MtcMcc);

/**************************************************************************//**
 @Group         mtc_mcc_advanced_init_grp MTC MCC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      MTC_MCC_ConfigCellFiltering

 @Description   Configures Cell Filtering according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of Cell Filtering

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigCellFiltering(t_Handle h_MtcMcc, e_CF newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigThd

 @Description   Configures Transmit HEC disabled according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of Transmit HEC disabled

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigThd(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigSynchMode

 @Description   Configures Syncronization mode according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of Syncronization mode

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigSynchMode(t_Handle h_MtcMcc, e_SYNC_MODE newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigExceptions

 @Description   Configures event reister  according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of event register

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigExceptions(t_Handle h_MtcMcc, uint16_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxCosetVal

 @Description   Configures Recieve coset value  according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of RxCosetVal

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigRxCosetVal(t_Handle h_MtcMcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxCosetVal

 @Description   Configures Transmit coset value  according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of TxCosetVal

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigTxCosetVal(t_Handle h_MtcMcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxCfSize

 @Description   Configures Receive Cell Filtering FIFO  according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of RxCfSize

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigRxCfSize(t_Handle h_MtcMcc, uint32_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxCfSize

 @Description   Configures Transmit Cell Filtering FIFO  according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of TxCfSize

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigTxCfSize(t_Handle h_MtcMcc, uint32_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigDelta

 @Description   Configures Delta according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of Delte

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigDelta(t_Handle h_MtcMcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigAlpha

 @Description   Configures Alpha according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of Alpha

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigAlpha(t_Handle h_MtcMcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxIma

 @Description   Configures if MTC works in IMA mode in receive direction according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value RxIMA mode(TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigRxIma(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxIma

 @Description   Configures if MTC works in IMA mode in transmit direction according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value TxIMA mode(TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigTxIma(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigSbc

 @Description   Configures single bit correction procedure according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of Sbc

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigSbc(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxCoset

 @Description   Configures transmit coset procedure according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of TxCoset (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigTxCoset(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxCoset

 @Description   Configures receive coset procedure according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of TxCoset (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigRxCoset(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxPScr

 @Description   Configures transmit payload scrambling procedure according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of TxPScr (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigTxPScr(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxPScr

 @Description   Configures receive payload scrambling procedure according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of RxPScr (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigRxPScr(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxEn

 @Description   Configures if MTC will be initialized in receive direction according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of RxEn (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigRxEn(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxEn

 @Description   Configures if MTC will be initialized in transmit direction according to newVal
 
 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object
 @Param[in]     newVal      - New value of TxEn (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_MCC_ConfigTxEn(t_Handle h_MtcMcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcPortMaxIteration

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Max iteration allowed

 @Param[in]     h_MtcMcc    - A handle of the MTC_MCC object.
 @Param[in]     newVal      - New value

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_MCC_ConfigMtcPortMaxIteration (t_Handle h_MtcMcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcPortGcraScheduler

 @Description   Configures a Gcra APC scheduler.

 @Param[in]     h_MtcMcc                    - A handle of the MTC_MCC object.
 @Param[in]     p_MtcMccQeGcraSchedParam    - New value.

 @Return        E_OK in case of success otherwise error Code.
*//***************************************************************************/
t_Error MTC_MCC_ConfigMtcPortGcraScheduler(t_Handle h_MtcMcc, t_MtcMccQeGcraSchedParam *p_MtcMccQeGcraSchedParam);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcVpShapingScheduler

 @Description   Configures VP Shapping scheduler.

 @Param[in]     h_MtcMcc                    - A handle of the MTC_MCC object.
 @Param[in]     p_MtcMccPortVpShapingParam  - New value for VP shapping params.

 @Return        E_OK in case of success otherwise error Code.
*//***************************************************************************/
t_Error MTC_MCC_ConfigMtcVpShapingScheduler(t_Handle h_MtcMcc, t_MtcMccPortQeVpShapingParam *p_MtcMccPortVpShapingParam);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcPortNonExtendedCt

 @Description   Configures a nonExtendedCt parameter for a specific scheduling table.

 @Param[in]     h_MtcMcc        - A handle of the MTC_MCC object.
 @Param[in]     schedLevel      - Which scheduling level to update.
 @Param[in]     nonExtendedCt   - New value to set to nonExtendedCt parameter.

 @Return        E_OK in case of success otherwise error Code.
*//***************************************************************************/
t_Error MTC_MCC_ConfigMtcPortNonExtendedCt(t_Handle h_MtcMcc, e_MtcMccPortSchedLevels schedLevel, bool nonExtendedCt);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMiniCam

 @Description   Configures the minicam table for an ATM port on top of MTC.

 @Param[in]     h_MtcMcc                    - A handle of the MTC_MCC object.
 @Param[in]     p_MtcMccPortMiniCamParam    - The params for minicam.
 
 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Does not affect hardware, just internal driver structures.
                May not be used after UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error MTC_MCC_ConfigMiniCam(t_Handle h_MtcMcc, t_MtcMccPortMiniCamParam *p_MtcMccPortMiniCamParam);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcPortBurstInhibit

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcMcc            - A handle of the MTC_MCC object.
 @Param[in]     enableBurstInhibit  - New value to set to burst inhibit parameter.

 @Return        E_OK in case of success otherwise error Code.

 *//***************************************************************************/
t_Error MTC_MCC_ConfigMtcPortBurstInhibit(t_Handle h_MtcMcc, bool enableBurstInhibit);

t_Error MTC_MCC_GetPortNum(t_Handle p_MtcMccId, uint32_t *port);

t_Error MTC_MCC_GetRxCellFifoSize(t_Handle h_MtcMcc, uint32_t *val);

t_Error MTC_MCC_GetCellDelineationState(t_Handle h_MtcMcc, uint32_t *val);
/**************************************************************************//**
 @Function      MTC_MCC_RegisterAppHandle

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcMcc            - A handle of the MTC_MCC object.
 @Param[in]     h_App 				- Handle to Application.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_MCC_RegisterAppHandle(t_Handle h_MtcMcc, t_Handle h_App);
/**************************************************************************//**
 @Function      MTC_MCC_RegisterAppHandle

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcMcc            - A handle of the MTC_MCC object.
 @Param[in]     f_EventHandler  	- Interrupt Service provided
									format: t_Handle, event
 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_MCC_RegisterEventHandler(t_Handle h_MtcMcc, void *f_EventHandler);
/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcMccPortAsNonShaper

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcMcc - A handle of the MTC MCC object.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_MCC_ConfigMtcMccPortAsNonShaper(t_Handle h_MtcMcc);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigMtcPortSchedulerScaleFactor

 @Description   Configures the port's scheduler scale factor.
 
                This routine is important when the MTC is used as an IMA port,
                because the ATM scheduler scales the rate of the ATM channel
                according to the number of active links. In this case, set
                the scale factor to be the maximum number of links expected
                in the IMA group.

 @Param[in]     h_MtcMcc            - A handle of the MTC MCC object.
 @Param[in]     schedScaleFactor    - New scheduler scale factor.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_MCC_ConfigMtcPortSchedulerScaleFactor(t_Handle h_MtcMcc, uint8_t schedScaleFactor);


/** @} */ /* end of mtc_mcc_advanced_init_grp group */
/** @} */ /* end of mtc_mcc_init_grp group */
/** @} */ /* end of mtc_mcc_grp group */
/** @} */ /* end of mtc_grp group */

#endif /* __MTC_MCC_EXT_H */
