/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          mtc_ucc_ext.h

 @Description   MTC UCC Application Programming Interface
*//***************************************************************************/
#ifndef __MTC_UCC_EXT_H
#define __MTC_UCC_EXT_H

#include "std_ext.h"
#include "mtc_ext.h"
#include "ucc_atm_port_ext.h"

/**************************************************************************//**
 @Group         mtc_grp             MTC Application Programming Interface

 @Description   MTC API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group        mtc_ucc_grp    MTC UCC Application Programming Interface

 @Description   MTC UCC API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mtc_ucc_init_grp    MTC UCC Initialization Unit

 @Description   MTC UCC initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define UCC_EVENT_QOV0   0x8000
#define UCC_EVENT_QOV1     0x2000
#define UCC_EVENT_QOV2     0x0800
#define UCC_EVENT_QOV3     0x0200
#define UCC_EVENT_QOV4     0x0080
#define UCC_EVENT_QOV5     0x0020
#define UCC_EVENT_QOV6     0x0008

#define UCC_EVENT_GUN 0x0002
#define UCC_EVENT_GOV 0x0001
#define UCC_EVENT_QOV (UCC_EVENT_QOV0|UCC_EVENT_QOV1|UCC_EVENT_QOV2|UCC_EVENT_QOV3|UCC_EVENT_QOV4|UCC_EVENT_QOV5|UCC_EVENT_QOV6)
#define UCC_EVENT_ALL	(UCC_EVENT_QOV0|UCC_EVENT_QOV1|UCC_EVENT_QOV2|UCC_EVENT_QOV3|UCC_EVENT_QOV4|UCC_EVENT_QOV5|UCC_EVENT_QOV6 | UCC_EVENT_GUN | UCC_EVENT_GOV)

/**************************************************************************//**
 @Description   User callback function called by driver for event handling.

                User provides this function. Driver invokes it.

 @Param[in]     appId   - The initial appId given by the user at driver
                          initialization.
 @Param[in]     errors  - The events that caused the exception to be invoked by
                          the driver. These events are divided to two some of
                          them are UCCE events and some of them are the interrupt
                          queue events (flags is the type of the errors).
 @Param[in]     flags   - Determine the errors source - is it from the UCCE
                          register or from the Interrupt queue;
                          0 - the UCCE is the source of the errors
 
 @Return         None.
 *//***************************************************************************/
typedef void (t_MtcUccExceptions) ( t_Handle appId, uint32_t errors, uint32_t flags );
typedef enum
{
      e_MTC_UCC_PORT_SCHED_LEVEL_1      = e_UCC_ATM_PORT_SCHED_LEVEL_1
    , e_MTC_UCC_PORT_SCHED_LEVEL_2     = e_UCC_ATM_PORT_SCHED_LEVEL_2
    , e_MTC_UCC_PORT_SCHED_LEVEL_3     = e_UCC_ATM_PORT_SCHED_LEVEL_3
    , e_MTC_UCC_PORT_SCHED_LEVEL_4     = e_UCC_ATM_PORT_SCHED_LEVEL_4
    , e_MTC_UCC_PORT_SCHED_LEVEL_5    =  e_UCC_ATM_PORT_SCHED_LEVEL_5
    , e_MTC_UCC_PORT_SCHED_LEVEL_6     = e_UCC_ATM_PORT_SCHED_LEVEL_6
    , e_MTC_UCC_PORT_SCHED_LEVEL_7    = e_UCC_ATM_PORT_SCHED_LEVEL_7
    , e_MTC_UCC_PORT_SCHED_LEVEL_8  = e_UCC_ATM_PORT_SCHED_LEVEL_8
    , e_MTC_UCC_PORT_TOTAL_SCHED_LEVELS= e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS /* Must be last */
}e_MtcUccPortSchedLevels;

#define MTC_UCC_MAX_GCRA_LEVELS 4

typedef struct
{
    uint8_t    schedLevels;/**< number of the required Apc levels */
    uint32_t   fastChannel;/**< the rate of the fast channel - bits per second */
    uint32_t   slowChannel[e_MTC_UCC_PORT_TOTAL_SCHED_LEVELS];/**< the rate of the slow channel - bits per second per priority level */
    uint32_t   lineRate; /**< the line rate */
 
}t_PortParam;


typedef struct
{
    e_MtcUccPortSchedLevels      firstUbrPlusSchedLevel;/**< the first scheduling level of UBR Plus traffic type */
    uint8_t                      numOfUbrPlusPriorityLevels; /**< in how many priority levels the user is going to work with ubr+ */
    uint8_t                      thresholdVal[e_MTC_UCC_PORT_TOTAL_SCHED_LEVELS];/**< the UBR+ MDA value for a specific priority level */
}t_MtcUccPortApcPriUbrPlusParam;
/**************************************************************************//**
 @Description   Information regarding GBR traffic of channels on a APC port last priority
*//***************************************************************************/

typedef struct
{
    bool        nonExtendedCt[e_MTC_UCC_PORT_TOTAL_SCHED_LEVELS];
                                            /**< If true - the control slot indicates that TCTE should be read for each channel;
                                                 Must be false in case of working with APC scalable mode or with traffic type which is different then CBR */

    uint8_t     maxIteration;               /**< Max iteration allowed */
    bool        burstInhibit;               /**< Indicates if enable the burst inhibit option in the scheduler_MODE field */
    uint8_t     schedScaleFactor;           /**< Scheduler scaling factor */

} t_MtcUccQeApcSchedParam;
/**************************************************************************//**
 @Description   Information regarding GBR traffic of channels on a GCRA port last priority
 *//***************************************************************************/
typedef enum
{
    e_MTC_UCC_GCRA_NO_GBR_MODE      = e_UCC_ATM_GCRA_NO_GBR_MODE,     /**< There are no GBR channels in this GCRA port */
    e_MTC_UCC_GCRA_MIXED_GBR_MODE   = e_UCC_ATM_GCRA_MIXED_GBR_MODE,  /**< Some of the channels on last priority work with GBR traffic */
    e_MTC_UCC_GCRA_FULL_GBR_MODE    = e_UCC_ATM_GCRA_FULL_GBR_MODE    /**< All channels on last priority work with GBR traffic */
 
} e_MtcUccQeGcraGbrMode;

/**************************************************************************//**
 @Description   GCRA Scheduler initialization structure.
 
                For MTC_MCC_PORT_ConfigVpShapingScheduler and MTC_MCC_PORT_ConfigGcraScheduler.
 *//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt;      /**< No extended connection table needed for this port's channels */
    bool        isInternal;         /**< Indication if the user asked to work with internal/external channels in this port */
    uint8_t     gcraChannels;       /**< The actual number of gcra channels per the specific port */
    bool        workConservingMode; /**< Work Conserving Mode - Vps on the last priority are scheduled according to their
                                         weighted PCR/PCR Fraction regardless of the system clock */
    uint8_t     *p_WrrPriPercentages;/**< Array of up to 4 WRR (Weighted Round Robin) priorities percentages;
                                         Relevant only for WRR Mode, in which selection between the up to 4 priorities
                                         is done according to their relative weights; Note: For regular fixed priority mode
                                         (non-WRR) this pointer should be NULL. */
    e_MtcUccQeGcraGbrMode gbrMode;    /**< Full GBR mode / Mixed GBR mode / No GBR mode */

} t_MtcUccQeGcraSchedParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level APC initialization structure.

                This structurte includes the parameters needed for a single
                APC parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    vpi;                        /**< Virtual Path Identifier of this VP group channels */
  	/* Scheduling level */
	e_MtcUccPortSchedLevels	schedLevel;		/**< Scheduling level in high level GCRA scheduler */
    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;                 /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[e_MTC_UCC_PORT_TOTAL_SCHED_LEVELS];
                                            /**< The rate of the slow channel - bits per second per priority level */

    /* APC specific parameters */
    t_MtcUccQeApcSchedParam apcSchedParam;

} t_MtcUccQeLowLevelApcParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level GCRA initialization structure.

                This structurte includes the parameters needed for a single
                GCRA parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    vpi;                        /**< Virtual Path Identifier of this VP group channels */
  	/* Scheduling level */
	e_MtcUccPortSchedLevels	schedLevel;		/**< Scheduling level in high level GCRA scheduler */
    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[MTC_UCC_MAX_GCRA_LEVELS];
                                            /**< The rate of the slow channel per priority level (in bits per second) */

    /* GCRA specific parameters */
    t_MtcUccQeGcraSchedParam gcraSchedParam;

} t_MtcUccQeLowLevelGcraParam;

/**************************************************************************//**
 @Description   VP Shaping Scheduling initialization structure.
 *//***************************************************************************/
typedef struct
{
    uint8_t                         vpsNum;             /**< Number of Vps under this Vp Shaping port */
    bool                            autoVpOnOff;        /**< Auto Vp On Off enable */
    t_MtcUccQeLowLevelApcParam      *p_LowLevelApcSchedParam;
                                                        /**< If not NULL => GCRA over APC Vp Shaping (Low level
                                                        scheduling type is APC) and this is the array of all
                                                        low level APC schedulers parameters */
    t_MtcUccQeLowLevelGcraParam     *p_LowLevelGcraSchedParam;
                                                        /**< If not NULL => GCRA over GCRA Vp Shaping (Low level
                                                        scheduling type is GCRA) and this is the array of all
                                                        low level GCRA schedulers parameters */
} t_MtcUccPortQeVpShapingParam;

/**************************************************************************//**
 @Description   MiniCam Structure
 *//***************************************************************************/
typedef struct
{
    bool                        uniMode;        /**< Indicates if to take the GFC value into the CAM search or mask it
                                                     FALSE - take the GFC field,
                                                     TRUE - mask the GFC field in the incoming cell */
    bool                        isOverlapping;  /**< Indication if the specific port belongs to an overlap group */
    uint8_t                     overlapGroup;   /**< The number of the overlap group ,
                                                     NOTE: This parameter is relevant only in case the isOverlapping parameter is set */
    e_MaxChannelsPerMiniCamPort numOfChannels;  /**< The actual number of how many channels are going to work under this MiniCam port
                                                     NOTE: The parameter is important to improve performance
                                                     and also to search for an available channel code */
} t_MtcUccPortMiniCamParam;

/**************************************************************************//**
 @Description   MtcUcc initialization structure, used when calling MTC_UCC
*//***************************************************************************/
typedef struct {
    t_Handle            h_App;              /**< Handle to the upper layer handler             */
    t_Handle            h_AtmController;    /**< Handle to the UccAtm            */
    uint16_t            txIntQ;             /**< Tx queue number   */
    uint16_t            rxIntQ;             /**< Rx queue number.                   */
    int					irq;				/**< Irq number needed for the UCC      */
    t_Handle            h_MtcCommon;        /**< Handler to MTC_COMMON*/
    t_PortParam         portParam;          /**< ATM port user defined parameters*/
    void (*f_Isr)(t_Handle, uint16_t);
    e_Ucc               peripheral;         /**< Peripheral MTC works on - can be (UCC1 - UCC8)*/
    t_MtcUccExceptions  *f_Exceptions;      /**< The exception invoked by driver upon events.
                                                 User may choose to set it to NULL by that avoiding
                                                 recption of events (no harm done). */
    uint16_t            numOfTdmRxSlots;
    uint16_t            numOfTdmTxSlots;
    uint8_t             *rxSlots;
    uint8_t             *txSlots;
    t_Handle            h_TdmRxFrame;
    t_Handle            h_TdmTxFrame;
    e_Tdm               tdmId;
    uint8_t 			pramMemId;
    uint32_t			dataMemAttributes;
    uint32_t			bdMemAttributes;
 
}t_MtcUccParam;

/**************************************************************************//**
 @Function      MTC_UCC_Config

 @Description   Configures a  MTC_UCC object
 
                This routine allocates the MtcUcc structure and sets user
                and default parameters in the MTC_UCC driver parameters structure.

 @Param[in]     p_MtcUccParam   - A handle to the initializing parameter
                                  structure for the MTC_UCC
 @Param[in]     baseAddress     - Base address of MTC UCC  registers.

 @Return         A handle (descriptor) of the newly created MTC_UCC structure.
*//***************************************************************************/
t_Handle MTC_UCC_Config(t_MtcUccParam *p_MtcUccParam, uint32_t baseAddress );

/**************************************************************************//**
 @Function      MTC_UCC_Init

 @Description   Initializes a MTC_UCC object.
 
 @Param[in]     h_MtcUcc - A handle of the MTC_UCC object

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_Init(t_Handle h_MtcUcc);

/**************************************************************************//**
 @Function      MTC_UCC_Enable

 @Description   Enables MTC_UCC object according to requested mode.
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     activeMode  - Direction (RX, TX, RX_TX)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_Enable(t_Handle h_MtcUcc,  e_CommMode activeMode);

/**************************************************************************//**
 @Function      MTC_UCC_Disable

 @Description   Disables MTC_UCC object according to requested mode.
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     activeMode  - Direction (RX, TX, RX_TX)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_Disable(t_Handle h_MtcUcc,  e_CommMode activeMode);

/**************************************************************************//**
 @Function      MTC_UCC_Free

 @Description   Frees a memory allocated by MTC_UCC object.
 
 @Param[in]     h_MtcUcc - A handle of the MTC_UCC object

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_Free(t_Handle h_MtcUcc);

t_Error MTC_UCC_GetRxCellFifoSize(t_Handle h_MtcUcc, uint32_t *val);
t_Error MTC_UCC_GetPortNum(t_Handle p_MtcUccId, uint32_t *port);
t_Error MTC_UCC_GetCellDelineationState(t_Handle h_MtcUcc, uint32_t *val);

/**************************************************************************//**
 @Group         mtc_ucc_advanced_init_grp MTC UCC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      MTC_UCC_ConfigTxEn

 @Description   Configures if MTC will be initialized in transmit direction
                according to newVal.
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of TxEn (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigTxEn(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
@Function      MTC_UCC_ConfigMiniCam

 @Description   Configures the minicam table for an ATM port on top of MTC.

 @Param[in]     h_MtcUcc                    - A handle of the MTC_UCC object.
 @Param[in]     p_MtcUccPortMiniCamParam    - The params for minicam.
 
 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Does not affect hardware, just internal driver structures.
                May not be used after UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error MTC_UCC_ConfigMiniCam(t_Handle h_MtcUcc, t_MtcUccPortMiniCamParam *p_MtcUccPortMiniCamParam);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigRxEn

 @Description   Configures if MTC will be initialized in receive direction
                according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of RxEn (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigRxEn(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigRxPScr

 @Description   Configures receive payload scrambling procedure according
                to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of RxPScr (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigRxPScr(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigTxPScr

 @Description   Configures transmit payload scrambling procedure according
                to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of TxPScr (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigTxPScr(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigRxCoset

 @Description   Configures receive coset procedure according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of TxCoset (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigRxCoset(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigTxCoset

 @Description   Configures transmit coset procedure according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of TxCoset (TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigTxCoset(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigSbc

 @Description   Configures single bit correction procedure according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of Sbc

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigSbc(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigTxIma

 @Description   Configures if MTC works in IMA mode in transmit direction
                according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value TxIMA mode(TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigTxIma(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigRxIma

 @Description   Configures if MTC works in IMA mode in receive direction
                according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value RxIMA mode(TRUE or FALSE)

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigRxIma(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigAlpha

 @Description   Configures Alpha according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of Alpha

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigAlpha(t_Handle h_MtcUcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigDelta

 @Description   Configures Delta according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of Delte

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigDelta(t_Handle h_MtcUcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigSynchMode

 @Description   Configures Syncronization mode according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of Syncronization mode

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigSynchMode(t_Handle h_MtcUcc, e_SYNC_MODE newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigCellFiltering

 @Description   Configures Cell Filtering according to newVal
 
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of Cell Filtering

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigCellFiltering(t_Handle h_MtcUcc, e_CF newVal);

/**************************************************************************//**
 @Function      MTC_MCC_ConfigTxCfSize

 @Description   Configures Transmit Cell Filtering FIFO  according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of TxCfSize

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigTxCfSize(t_Handle h_MtcUcc, uint32_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigRxCfSize

 @Description   Configures Receive Cell Filtering FIFO  according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of RxCfSize

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigRxCfSize(t_Handle h_MtcUcc, uint32_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigTxCosetVal

 @Description   Configures Transmit coset value  according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of TxCosetVal

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigTxCosetVal(t_Handle h_MtcUcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigRxCosetVal

 @Description   Configures Recieve coset value  according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of RxCosetVal

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigRxCosetVal(t_Handle h_MtcUcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigThd

 @Description   Configures Transmit HEC disabled according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of Transmit HEC disabled

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigThd(t_Handle h_MtcUcc, bool newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigExceptions

 @Description   Configures event reister  according to newVal
 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object
 @Param[in]     newVal      - New value of event register

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigExceptions(t_Handle h_MtcUcc, uint16_t newVal);


/**************************************************************************//**
 @Function      MTC_UCC_ConfigMtcPortMaxIteration

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Max iteration allowed

 @Param[in]     h_MtcUcc    - A handle of the MTC_UCC object.
 @Param[in]     newVal      - New value

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigMtcPortMaxIteration (t_Handle h_MtcUcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigMtcPortGcraScheduler

 @Description   Configures a Gcra APC scheduler.

 @Param[in]     h_MtcUcc - A handle of the MTC_UCC object.
 @Param[in]     p_UtopiaPortGcraSchedParam  - New value to set to parameter ParamXXX.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigMtcPortGcraScheduler(t_Handle h_MtcUcc, t_MtcUccQeGcraSchedParam *p_UtopiaPortGcraSchedParam);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigMtcPortNonExtendedCt

 @Description   Configures a nonExtendedCt parameter for a specific scheduling table.

 @Param[in]     h_MtcUcc        - A handle of the MTC_UCC object.
 @Param[in]     schedLevel      - Which scheduling level to update.
 @Param[in]     nonExtendedCt   - New value to set to nonExtendedCt parameter.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_UCC_ConfigMtcPortNonExtendedCt(t_Handle h_MtcUcc, e_MtcUccPortSchedLevels schedLevel, bool nonExtendedCt);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigMtcPortBurstInhibit

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcUcc            - A handle of the MTC_UCC object.
 @Param[in]     enableBurstInhibit  - New value to set to burst inhibit parameter.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_UCC_ConfigMtcPortBurstInhibit(t_Handle h_MtcUcc, bool enableBurstInhibit);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigMtcUccPortAsNonShaper

 @Description   Configures the port as non-shaper (without scheduler)

 @Param[in]     h_MtcUcc - A handle of the MTC_UCC object.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_UCC_ConfigMtcUccPortAsNonShaper(t_Handle h_MtcUcc);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigMtcPortSchedulerScaleFactor

 @Description   Configures the port's scheduler scale factor.
 
                This routine is important when the MTC is used as an IMA port,
                because the ATM scheduler scales the rate of the ATM channel
                according to the number of active links. In this case, set
                the scale factor to be the maximum number of links expected
                in the IMA group.

 @Param[in]     h_MtcUcc            - A handle of the MTC UCC object.
 @Param[in]     schedScaleFactor    - New scheduler scale factor.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_UCC_ConfigMtcPortSchedulerScaleFactor(t_Handle h_MtcUcc, uint8_t schedScaleFactor);

/**************************************************************************//**
 @Function      MTC_UCC_ConfigTimeOutMicroSec

 @Description   Configures the port's time out(in micro seconds).
 
                The time out indicates how much time to wait to
                a channel stop request to be finished.
 
 @Param[in]     h_MtcUcc        - A handle of the MTC UCC object.
 @Param[in]     timeOutMicroSec - New time out.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_UCC_ConfigTimeOutMicroSec(t_Handle h_MtcUcc, uint16_t timeOutMicroSec);
/**************************************************************************//**
 @Function      MTC_UCC_RegisterAppHandle

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcUcc            - A handle of the MTC_UCC object.
 @Param[in]     h_App 				- Handle to Application.

 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_UCC_RegisterAppHandle(t_Handle h_MtcUcc, t_Handle h_App);
/**************************************************************************//**
 @Function      MTC_UCC_RegisterEventHandler

 @Description   Configures the burst inhibit option.

 @Param[in]     h_MtcUcc            - A handle of the MTC_UCC object.
 @Param[in]     f_EventHandler  	- Interrupt Service provided
									format: t_Handle, event
 @Return        E_OK in case of success otherwise error Code.
 *//***************************************************************************/
t_Error MTC_UCC_RegisterEventHandler(t_Handle h_MtcUcc, void *f_EventHandler);

t_Error MTC_UCC_RxAtmEnable(t_Handle h_MtcUcc);
t_Error MTC_UCC_RxAtmDisable(t_Handle h_MtcUcc);
/** @} */ /* end of mtc_ucc_advanced_init_grp group */
/** @} */ /* end of mtc_ucc_init_grp group */
/** @} */ /* end of mtc_ucc_grp group */
/** @} */ /* end of mtc_grp */

#endif /* __MTC_UCC_EXT_H */
