/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ops_ext.h

 @Description   structures and definitions for operating different complex mathematical operations.

*//***************************************************************************/
#ifndef __OPS_EXT_H
#define __OPS_EXT_H

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_ops SEC OPS

 @Description   SEC OPS functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         sec_ops_init SEC OPS Initialization Unit

 @Description   SEC OPS Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   enum of the different Mod2Op possibilities.
*//***************************************************************************/
typedef enum e_Mod2Op{
         e_MUL1,
         e_MUL2,
         e_ADD,
         e_SUB,
         e_A0_B0_MUL1,
         e_A0_B0_MUL2,
         e_A0_B0_ADD,
         e_A1_B0_MUL1,
         e_A1_B0_MUL2,
         e_A1_B0_ADD,
         e_A2_B0_MUL1,
         e_A2_B0_MUL2,
         e_A2_B0_ADD,
         e_A3_B0_MUL1,
         e_A3_B0_MUL2,
         e_A3_B0_ADD,
         e_A0_B1_MUL1,
         e_A0_B1_MUL2,
         e_A0_B1_ADD,
         e_A1_B1_MUL1,
         e_A1_B1_MUL2,
         e_A1_B1_ADD,
         e_A2_B1_MUL1,
         e_A2_B1_MUL2,
         e_A2_B1_ADD,
         e_A3_B1_MUL1,
         e_A3_B1_MUL2,
         e_A3_B1_ADD,
         e_A0_B2_MUL1,
         e_A0_B2_MUL2,
         e_A0_B2_ADD,
         e_A1_B2_MUL1,
         e_A1_B2_MUL2,
         e_A1_B2_ADD,
         e_A2_B2_MUL1,
         e_A2_B2_MUL2,
         e_A2_B2_ADD,
         e_A3_B2_MUL1,
         e_A3_B2_MUL2,
         e_A3_B2_ADD,
         e_A0_B3_MUL1,
         e_A0_B3_MUL2,
         e_A0_B3_ADD,
         e_A1_B3_MUL1,
         e_A1_B3_MUL2,
         e_A1_B3_ADD,
         e_A2_B3_MUL1,
         e_A2_B3_MUL2,
         e_A2_B3_ADD,
         e_A3_B3_MUL1,
         e_A3_B3_MUL2,
         e_A3_B3_ADD
}e_Mod2Op;


/**************************************************************************//**
 @Description   enum of the different EccPoint possibilities.
*//***************************************************************************/
typedef enum e_EccPoint{
         e_FP_AFF_KP,
         e_FP_PROJ_KP,
         e_F2M_AFF_KP,
         e_F2M_PROJ_KP,
         e_FP_ADD,
         e_FP_DOUBLE,
         e_F2M_ADD,
         e_F2M_DOUBLE
}e_EccPoint;

/**************************************************************************//**
 @Description   structure of the parameters for modExp operation.
*//***************************************************************************/
typedef struct
{
    unsigned char *p_AData;
    uint32_t         aDataLength;
    unsigned char *p_ExpData;
    uint32_t         expDataLength;
    unsigned char *p_ModData;
    uint32_t         modDataLength;
}modExpParam;

/**************************************************************************//**
 @Description   structure of the parameters for modR2ModN operation.
*//***************************************************************************/
typedef struct
{
    unsigned char *p_ModData;
    uint32_t         modDataLength;
}modR2ModNParam;

/**************************************************************************//**
 @Description   structure of the parameters for modRRModP operation.
*//***************************************************************************/
typedef struct
{
    uint32_t         nbytes;
    unsigned char *p_PData;
    uint32_t         pDataLength;
}modRRModPParam;

/**************************************************************************//**
 @Description   structure of the parameters for mod2Op operation.
*//***************************************************************************/
typedef struct
{
    unsigned char *p_BData;
    unsigned int   bDataLength;
    unsigned char *p_AData;
    unsigned int   aDataLength;
    unsigned char *p_ModData;
    unsigned int   modDataLength;
}mod2OpParam;

#ifdef SEC2
/**************************************************************************//**
 @Description   structure of the parameters for eccPoint operation.
*//***************************************************************************/
typedef struct
{
    uint32_t         nDataBytes;
    unsigned char *p_NData;
    uint32_t         eDataBytes;
    unsigned char *p_EData;
    uint32_t         buildDataBytes;
    unsigned char *p_BuildData;
    uint32_t         b1DataBytes;
    unsigned char* p_B1Data;
    uint32_t         b2DataBytes;
    unsigned char* p_B2Data;
    uint32_t         b3DataBytes;
    unsigned char* p_B3Data;
}eccPointParam;

#else /* not SEC2 */

/**************************************************************************//**
 @Description   structure of the parameters for eccPoint operation.
*//***************************************************************************/
typedef struct
{
    unsigned char *p_Par2Data;
    uint32_t         par2Length;
    unsigned char *p_Par1Data;
    uint32_t         par1Length;
    unsigned char *p_ExpData;
    uint32_t         expDataLength;
    unsigned char *p_DataIn;
    uint32_t         dataInLength;
}eccPointParam;

#endif /* SEC2 */
/**************************************************************************//**
 @Description   structure of the parameters for ecc2Op operation.
*//***************************************************************************/
typedef struct
{
    unsigned char *p_BData;
    uint32_t         bDataLength;
    unsigned char *p_AData;
    uint32_t         aDataLength;
    unsigned char *p_ModData;
    uint32_t         modDataLength;
}ecc2OpParam;

#ifdef SEC2
/**************************************************************************//**
 @Description   structure of the parameters for build public key structure operation.
*//***************************************************************************/
typedef struct
{
    unsigned char *p_A0Data;
    uint32_t         a0DataLength;
    unsigned char *p_A1Data;
    uint32_t         a1DataLength;
    unsigned char *p_A2Data;
    uint32_t         a2DataLength;
    unsigned char *p_A3Data;
    uint32_t         a3DataLength;
    unsigned char *p_B0Data;
    uint32_t         b0DataLength;
    unsigned char *p_B1Data;
    uint32_t         b1DataLength;
}eccSpkParam;
#endif


/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Group         sec_mod_exp_init SEC Modular Exponent Initialization Unit

 @Description   SEC Modular Exponent initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_exp_alloc_ctx

 @Description   Allocate memory for the mod_exp ctx structure.

				The routine is not part of the Crypto Api standard routines.
 
 @Return        ctx -  ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  mod_exp_alloc_ctx();

/**************************************************************************//**
 @Function      mod_exp_update

 @Description   update the mod_exp ctx structure.

				The routine sets the aData, expData, modData and
                their length in the ctx structure according to the
                values in the input structure.
				The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx   - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input - handler to param structure

 @Return        None.
*//***************************************************************************/
void      mod_exp_update(void *ctx, void *input);

/** @} */ /* end of group modular_exponent  */

/**************************************************************************//**
 @Group         sec_mod_ss_init SEC Single Step Exponent Initialization Unit

 @Description   SEC Single Step Exponent initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_ss_exp_alloc_ctx

 @Description   Allocate memory for the ss_exp ctx structure.

				The routine is not part of the Crypto Api standard routines.
 
 @Return        ctx -   ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  mod_ss_exp_alloc_ctx();

/**************************************************************************//**
 @Function      mod_ss_exp_update

 @Description   update the ss_exp ctx structure.

				The routine sets the aData, expData, modData and
                their length in the ctx structure according to the
                values in the input structure.
				The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx   - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input - handler to param structure

 @Return        None.
*//***************************************************************************/
void      mod_ss_exp_update(void *ctx, void *input);

/** @} */ /* end of group ss_exponent  */

/**************************************************************************//**
 @Group         sec_r2modn_init SEC R2MODN Initialization Unit

 @Description   SEC R2modN initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_r2modn_alloc_ctx

 @Description   operate the request for mod_exp operation.

				Allocate memory for the mod_r2modn ctx structure.
				The routine is not part of the Crypto Api standard routines.
 
 @Return        ctx  - mod r2modn ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  mod_r2modn_alloc_ctx();

/**************************************************************************//**
 @Function      mod_r2modn_update

 @Description   update the mod_r2modn ctx structure.

				The routine sets modData and its length in the ctx structure
                according to the values in the input structure.
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx   - mod exp ctx handler  - holds the relevant parameters for the operation.
 @Param[in]     input - handler to param structure.

 @Return        None.
*//***************************************************************************/
void      mod_r2modn_update(void *ctx, void *input);

/** @} */ /* end of group r2modn  */

/**************************************************************************//**
 @Group         sec_rrmodn_init SEC RRMODN Initialization Unit

 @Description   SEC RRModN initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_rrmodp_alloc_ctx

 @Description   Allocate memory for the mod_rrmodp ctx structure.
 
                The routine is not part of the Crypto Api standard routines.
 
 @Return        ctx   -  mod rrmodn ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  mod_rrmodp_alloc_ctx();

/**************************************************************************//**
 @Function      mod_rrmodp_update

 @Description   update the mod_rrmodp ctx structure.
 
				The routine sets pData, pData Length and nbytes fields in
                the ctx structure according to the values in the input
                structure.
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx   - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input - handler to param structure.

 @Return        None.
*//***************************************************************************/
void      mod_rrmodp_update(void *ctx, void *input);

/** @} */ /* end of group rRmodn  */

/**************************************************************************//**
 @Group         sec_2op_init SEC 2OP Initialization Unit

 @Description   SEC 2op initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_2op_alloc_ctx

 @Description   Allocate memory for the mod_2op ctx structure.
 
                The routine is not part of the Crypto Api standard routines.
 
 @Return        ctx -   mod 2op ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  mod_2op_alloc_ctx();

/**************************************************************************//**
 @Function      mod_2op_setoperation

 @Description   The routine sets the operationType in the ctx structure according to the input opType value.
 
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx    - mod 2op ctx handler - holds the relevant parameters for the operation.
 @Param[in]     opType - the operation type.

 @Return        None.
*//***************************************************************************/
void      mod_2op_setoperation(void *ctx, e_Mod2Op opType);

/**************************************************************************//**
 @Function      mod_2op_update

 @Description   The routine updates the mod_2op ctx structure.
 
                The routine sets bData,aData and modData pointers and their
                lengths in the ctx structure according to the values in the
                input structure.
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx    - mod 2op ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input  - input parameters structure.

 @Return        None.
*//***************************************************************************/
void      mod_2op_update(void *ctx, void *input);

/** @} */ /* end of group 2op  */

/**************************************************************************//**
 @Group         sec_ecc_pointit SEC ECC Point Initialization Unit

 @Description   SEC Ecc Point initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ecc_point_alloc_ctx

 @Description   Allocate memory for the mod_2op ctx structure.
 
                The routine is not a part of the Crypto Api standard routines.
 
 @Return        ctx - ecc point ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  ecc_point_alloc_ctx();

/**************************************************************************//**
 @Function      ecc_point_setoperation

 @Description   The routine sets the operationType in the ctx structure according to the input opType value.
 
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx    - ecc point ctx handler - holds the relevant parameters for the operation.
 @Param[in]     opType - the operation type.

 @Return        None.
*//***************************************************************************/
void      ecc_point_setoperation(void *ctx, e_EccPoint opType);

/**************************************************************************//**
 @Function      ecc_point_update

 @Description   The routine updates the ecc_point ctx structure.
 
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx    - mod 2op ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input  - input parameters structure.

 @Return        None.
*//***************************************************************************/
void      ecc_point_update(void *ctx, void *input);

/** @} */ /* end of group ecc_point  */

/**************************************************************************//**
 @Group         sec_ecc_2op_init SEC ECC 2OP Initialization Unit

 @Description   SEC Ecc 2op initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ecc_2op_alloc_ctx

 @Description   Allocate memory for the ecc_2op ctx structure.
 
                The routine is not a part of the Crypto Api standard routines.
 
 @Return        ctx -   ecc 2op ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  ecc_2op_alloc_ctx();

/**************************************************************************//**
 @Function      ecc_2op_update

 @Description   The routine updates the ecc_2op ctx structure.
 
 				The routine sets bData, aData, modData pointers and their
                lengths in the ctx structure according to the values in the
                input structure.
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx    - mod 2op ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input  - input parameters structure.

 @Return        None.
*//***************************************************************************/
void      ecc_2op_update(void *ctx, void *input);

/** @} */ /* end of group ecc_2op  */

#ifdef SEC2
/**************************************************************************//**
 @Group         sec_spk_init SEC ECC SPK Initialization Unit

 @Description   SEC Ecc Spk initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ecc_spk_alloc_ctx

 @Description   Allocate memory for the ecc_spk ctx structure.
 
                The routine is not a part of the Crypto Api standard routines.
 
 @Return        ctx - ecc spk ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle  ecc_spk_alloc_ctx();

/**************************************************************************//**
 @Function      ecc_spk_update

 @Description   The routine updates the ecc_spk ctx structure.
 
 				The routine sets the relevant pointers and their
                lengths in the ctx structure according to the values in the
                input structure.
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx     - mod spk ctx handler - holds the relevant parameters for the operation.
 @Param[in]     input   - input parameters structure.

 @Return        None.
*//***************************************************************************/
void      ecc_spk_update(void *ctx, void *input);
/** @} */ /* end of group ecc_spk  */

#endif /* SEC2 */

/** @} */ /* end of group sec ops init */

/**************************************************************************//**
 @Group         sec_ops_runtime SEC OPS Runtime Unit

 @Description   SEC OPS Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Group         sec_mod_exp_runtime SEC Modular Exponent Runtime Unit

 @Description   SEC Modular Exponent Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_exp_final

 @Description   operate the request for mod_exp operation.

				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
				The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      mod_exp_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group modular_exponent  */

/**************************************************************************//**
 @Group         sec_ss_exp_runtime SEC Single Step Exponent Runtime Unit

 @Description   SEC Single Step Exponent Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_ss_exp_final

 @Description   operate the request for mod_ss_exp operation.

				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
				The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      mod_ss_exp_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group ss_exponent  */

/**************************************************************************//**
 @Group         sec_r2modn_runtime SEC R2MODN Runtime Unit

 @Description   SEC R2ModN Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_r2modn_final

 @Description   operate the request for mod_r2modn operation.

				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      mod_r2modn_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group_r2modn */

/**************************************************************************//**
 @Group         sec_rrmodn_runtime SEC RRMODN Runtime Unit

 @Description   SEC RRModN Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_rrmodp_final

 @Description   operate the mod_rrmodp request.
 
				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - mod exp ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      mod_rrmodp_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group rRmodn  */

/**************************************************************************//**
 @Group         sec_2op_runtime SEC 2OP Runtime Unit

 @Description   SEC 2op Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      mod_2op_final

 @Description   The routine operates the mod_2op request.
 
				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - mod 2op ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      mod_2op_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group 2op  */

/**************************************************************************//**
 @Group         sec_ecc_point_runtime SEC ECC Point Runtime Unit

 @Description   SEC Ecc Point Run Time functions.

 @{
*//***************************************************************************/

#ifdef SEC2
/**************************************************************************//**
 @Function      ecc_point_final

 @Description   The routine operates the ecc_point request.
 
 				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - ecc point ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void ecc_point_final(void *ctx);

#else /* NOT SEC2 */

/**************************************************************************//**
 @Function      ecc_point_final

 @Description   The routine operates the ecc_point request.
 
 				The routine initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - ecc point ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void      ecc_point_final(void *ctx, unsigned char *dst, unsigned int len);

#endif /* SEC2 */

/** @} */ /* end of group ecc_point  */

/**************************************************************************//**
 @Group         sec_ecc_2op_runtime SEC ECC 2OP Runtime Unit

 @Description   SEC Ecc 2op Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ecc_2op_final

 @Description   The routine operatess the ecc_2op request.
 
 				The routine sets the dst and its length in the
                dataOut and dataOutLength variables in the ctx
                structure. Then it initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - ecc 2op ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      ecc_2op_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group ecc_2op */

#ifdef SEC2
/**************************************************************************//**
 @Group         sec_spk_runtime SEC SPK Runtime Unit

 @Description   SEC Spk Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ecc_spk_final

 @Description   The routine operatess the ecc_spk request.
 
 				The routine initializes a specific request
                with the fields in the ctx structure and calls to
                the ioctl function in order to process the specific
                request.
                The routine is not a part of the Crypto Api standard routines.
 
 @Param[in]     ctx  - ecc spk ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst  - output handler.
 @Param[in]     len  - output length.

 @Return        None.
*//***************************************************************************/
void      ecc_spk_final(void *ctx, unsigned char *dst, unsigned int len);

/** @} */ /* end of group spk  */

#endif /* SEC2 */

/** @} */ /* end of group sec ops runtime */
/** @} */ /* end of group sec ops  */
/** @} */ /* end of group sec */

#endif /* __OPS_EXT_H */

