/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          par_io_ext.h

 @Description   Parallel IO Application Programming Interface
*//***************************************************************************/
#ifndef __PAR_IO_EXT_H
#define __PAR_IO_EXT_H

#include "std_ext.h"
#include "par_io_integration_ext.h"


/**************************************************************************//**
 @Group         par_io_grp Parallel IO Application Programming Interface

 @Description   Parallel IO API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         par_io_init_grp Parallel IO Initialization Unit

 @Description   Parallel IO initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Parallel I/O Configuration Parameters.
 
                This structure contains the configuration parameters for the
                Parallel I/O module. It is used with PAR_IO_ConfigAndInit()
                routine.
*//***************************************************************************/
typedef struct t_ParIoParam
{
    uint32_t    baseAddress;        /**< Base address of Parallel I/O registers */
    uint32_t    qePortsBaseAddr;    /**< Address of QE ports interrupt memory map */
    int         irq;                /**< Interrupt source ID - pass (-1) if
                                         interrupts are not required */
} t_ParIoParam;

/**************************************************************************//**
 @Function      PAR_IO_ConfigAndInit
 
 @Description   Configures and initializes the I/O Port manager.
 
                This routine must be called prior to any I/O ports pin access.
 
 @Param[in]     p_ParIoParam - Pointer to Parallel I/O configuration structure.
 
 @Return        A handle to the Parallel I/O module, to be passed on subsequent
                API routines.
*//***************************************************************************/
t_Handle PAR_IO_ConfigAndInit(t_ParIoParam *p_ParIoParam);

/**************************************************************************//**
 @Function      PAR_IO_Free
 
 @Description   Frees the I/O port manager and all allocated resources.
 
 @Param[in]     h_ParIo - The handle to the I/O port, as returned
                          by PAR_IO_ConfigAndInit() routine.
 
 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error PAR_IO_Free(t_Handle h_ParIo);

/** @} */ /* end of par_io_init_grp group */


/**************************************************************************//**
 @Group         par_io_control_grp Parallel IO Control Unit

 @Description   Parallel IO control unit API functions, definitions and enums

 @{
*//***************************************************************************/

#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      PAR_IO_DumpRegs

 @Description   Dumps PAR_IO registers.

 @Param[in]     h_ParIo - Handle to PAR_IO object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PAR_IO_DumpRegs(t_Handle h_ParIo);

#endif /* DEBUG_ERRORS > 0 */


/** @} */ /* end of par_io_control_grp group */
/** @} */ /* end of par_io_grp  */


#endif /* __PAR_IO_EXT_H */

