/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          ptp_etsec_ext.h

 @Description   External definitions and API for PTP (IEEE 1588).
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __PTP_ETSEC_EXT_H__
#define __PTP_ETSEC_EXT_H__



/**************************************************************************//**
 @Group         ptp_grp PTP Application Programming Interface

 @Description   PTP functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_exceptions_grp PTP Exceptions Unit

 @Description   PTP exceptions API.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Anchor        ptp_ETSEC_tsu_events
 
 @Collection    PTP eTSEC TSU Events
 
                Exceptions and events reported through the PTP eTSEC TSU events
                callback routine. These events can be masked/unmasked using
                the PTP_ConfigEventsMask() routine.
 
                Each event is reported separately (values are not combined).
                Callback flags have no meaning in all these events.
 @{
*//***************************************************************************/
#define PTP_ETSEC_EVENT_PTP        0x00000001  /**< PTP frame was timestamped on Rx */
#define PTP_ETSEC_EVENT_TX_TS1     0x00000100  /**< PTP frame was timestamped on Tx and stored in TXTS1 */
#define PTP_ETSEC_EVENT_TX_TS2     0x00000200  /**< PTP frame was timestamped on Tx and stored in TXTS2 */

#define PTP_ETSEC_EVENT_ALL              (PTP_ETSEC_EVENT_PTP | PTP_ETSEC_EVENT_TX_TS1 | \
                                         PTP_ETSEC_EVENT_TX_TS2)
                                         /**< All events */

/* @} */

/** @} */ /* end of ptp_exceptions_grp */
/** @} */ /* end of ptp_grp */


#endif /* __PTP_ETSEC_EXT_H__ */
