/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          ptp_qe_ext.h

 @Description   External definitions and API for PTP (IEEE 1588).
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __PTP_QE_EXT_H
#define __PTP_QE_EXT_H



/**************************************************************************//**
 @Group         ptp_grp PTP Application Programming Interface

 @Description   PTP functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_init_grp PTP Initialization Unit

 @Description   PTP initialization API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_adv_config_grp  PTP Advanced Configuration Unit

 @Description   PTP advanced configuration functions.

 @{
*//***************************************************************************/

#ifdef PTP_QE_TX_ACCELERATION
/**************************************************************************//**
 @Description   PTP QE Tx acceleration parameters structure.
 
                This structure should be passed to PTP_ConfigQeTxAccelerator().
*//***************************************************************************/
typedef struct t_PtpTxAccelerationParam
{
    bool        enable;             /**< Enable/disable Tx acceleration;
                                         TRUE to enable */
    t_Handle    h_Qe;               /**< Handle to QE object */
    uint32_t    txIntrThreshold;    /**< Tx Interrupt coalescing threshold */
    uint8_t     pramMemId;          /**< Parameter RAM partition ID */
    uint8_t     tsMemId;            /**< Timestamps queue memory partition ID */
    uint32_t    tsMemAttributes;    /**< Timestamps queue memory attributes mask
                                         (a combination of MEMORY_ATTR_x flags) */
} t_PtpTxAccelerationParam;

/**************************************************************************//**
 @Function      PTP_ConfigQeTxAccelerator

 @Description   Enables Tx timestamp acceleration by the QE, and configures
                the Tx interrupt threshold (common to all PTP ports).
                [FALSE]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     p_Param         - Tx acceleration parameters.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigQeTxAccelerator(t_Handle                  h_Ptp,
                                  t_PtpTxAccelerationParam  *p_Param);
#endif /* PTP_QE_TX_ACCELERATION */

/**************************************************************************//**
 @Function      PTP_ConfigEthernetType

 @Description   Configures the value of Ethernet type field for PTP frames.
                [0x0800]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     ethTypeValue    - Ethernet type field value.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigEthernetType(t_Handle h_Ptp, uint16_t ethTypeValue);

/**************************************************************************//**
 @Function      PTP_ConfigEthernetVlanType

 @Description   Configures the value of Ethernet VLAN type field for PTP frames.
                [0x8100]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     vlanTypeValue   - Ethernet VLAN type field value.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigEthernetVlanType(t_Handle h_Ptp, uint16_t vlanTypeValue);

/**************************************************************************//**
 @Function      PTP_ConfigIpType

 @Description   Configures the value of IP type field for PTP frames.
                [0x11]
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     ipTypeValue - IP type field value.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigIpType(t_Handle h_Ptp, uint8_t ipTypeValue);

/**************************************************************************//**
 @Function      PTP_ConfigUdpEventsPort

 @Description   Configures the value of UDP destination port field for PTP
                Sync/Delay_Request frames. [319]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     udpEventsPort   - UDP destination port field value for PTP
                                  event messages.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigUdpEventsPort(t_Handle h_Ptp, uint16_t udpEventsPort);

/**************************************************************************//**
 @Function      PTP_ConfigUdpGeneralPort

 @Description   Configures the value of UDP destination port field for general
                PTP frames. [320]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     udpGeneralPort  - UDP destination port field value for PTP
                                  general messages.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigUdpGeneralPort(t_Handle h_Ptp, uint16_t udpGeneralPort);

/**************************************************************************//**
 @Function      PTP_ConfigEthernetTypeOffset

 @Description   Configures the offset of Ethernet type field for PTP frames.
                [12]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     ethTypeOffset   - Ethernet type field offset.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigEthernetTypeOffset(t_Handle h_Ptp, uint8_t ethTypeOffset);

/**************************************************************************//**
 @Function      PTP_ConfigIpTypeOffset

 @Description   Configures the offset of IP type field for PTP frames.
                [23]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     ipTypeOffset    - IP type field offset.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigIpTypeOffset(t_Handle h_Ptp, uint8_t ipTypeOffset);

/**************************************************************************//**
 @Function      PTP_ConfigUdpDestPortOffset

 @Description   Configures the offset of UDP destination port field for PTP frames.
                [36]
 
 @Param[in]     h_Ptp               - Handle to PTP object.
 @Param[in]     udpDestPortOffset   - UDP destination port field offset.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigUdpDestPortOffset(t_Handle h_Ptp, uint8_t udpDestPortOffset);

/**************************************************************************//**
 @Function      PTP_ConfigPtpTypeOffset

 @Description   Configures the offset of PTP message type field for PTP frames.
                [74]
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     ptpTypeOffset   - PTP message type field offset.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigPtpTypeOffset(t_Handle h_Ptp, uint8_t ptpTypeOffset);

/**************************************************************************//**
 @Function      PTP_ConfigMessageTypeCode

 @Description   Configures the values of PTP message types for PTP frames.
                [-IEEE1588 V1 message type defaults
                 -Sync: 0;
                 -Delay_Request: 1;
                 -Follow_Up: 2;
                 -Delay_Response: 3;
                 -Management: 4;
 
                 -IEEE1588 V2 message type defaults
                 -Sync :            0;
                 -Delay_Request :   1;
                 -Follow_Up :       0x8;
                 -Delay_Response :  0x9;
                 -Management :      0xD;
                 -Pdelay_Request :  0x2;
                 -Pdelay_Response : 0x3;
                 -Announce :        0xB;
                 -Signaling :       0xC;
                 -Pdelay follow up :0xA;]
 
 @Param[in]     h_Ptp   - Handle to PTP object.
 @Param[in]     msgType - PTP message type (Sync, Delay_req, etc.).
 @Param[in]     code    - Value for PTP message type.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigMessageTypeCode(t_Handle h_Ptp, e_PtpMessage msgType, uint8_t code);

#ifdef PTP_V2
/**************************************************************************//**
 @Function      PTP_ConfigMessageTimestampPoint

 @Description   Configures the Message timestamp point.
 
 @Param[in]     h_Ptp   - Handle to PTP object.
 @Param[in]     timestampPoint - Time stamp point shall correspond to internal SFD(Default)
                                 ,or that the timestamp point shall correspond to external SOF.
                                 All port are sampled according to internal SFD.
                                 If external SFD is chosen a specific port should be provided ,this port will
                                 sample the RTC by an external time stamp lock trigger.
                                 Other ports in the same PTP unit will use the internal SFD.
                                 When choosing internal SFD - it applies on both units.

 @Param[in]     port    - The port which will work in external SOF mode.
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigMessageTimestampPoint(t_Handle h_Ptp,e_PtpMessageTimestampPoint timestampPoint,uint8_t port);

/**************************************************************************//**
 @Function      PTP_ConfigTransportSpecific

 @Description   Configures checking the transportSpecific field, and the value expected.
                This field is checked only when working according to IEEE1588 V2 PTP over IEEE 802.3/Ethernet(not over UDP)
 @Param[in]     h_Ptp   - Handle to PTP object.
 @Param[in]     check   - check for transportSpecific value.
 @Param[in]     transportSpecificVal   - The value of the field.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_ConfigTransportSpecific(t_Handle h_Ptp,bool check, uint8_t transportSpecificVal);
#endif /*PTP_V2*/

/** @} */ /* end of ptp_adv_config_grp */
/** @} */ /* end of ptp_init_grp */

#ifdef PTP_QE_TX_ACCELERATION
/**************************************************************************//**
 @Group         ptp_control_grp PTP Control Unit

 @Description   PTP runtime control API.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PTP_TSU_EnableCoalescing

 @Description   Enable interrupt coalescing on specific port.
 
                Re-enable interrupt coalescing back after it has
                been disabled through PTP_TSU_DisableCoalescing()
                routine.
 
                This function is valid only when Tx acceleration
                has been configured through PTP_ConfigQeTxAccelerator()
                routine.

 @Param[in]     h_Ptp   - Handle to PTP object.
 @Param[in]     port    - Port to enable interrupt coalescing on.

*//***************************************************************************/
void PTP_TSU_EnableCoalescing(t_Handle h_Ptp, uint8_t port);

/**************************************************************************//**
 @Function      PTP_TSU_DisableCoalescing

 @Description   Disable TSU interrupt coalescing on specific port.
 
                Interrupt coalescing may be re-enabled later on
                through PTP_TSU_EnableCoalescing() routine.

                This function is valid only when Tx acceleration
                has been configured through PTP_ConfigQeTxAccelerator()
                routine.

 @Param[in]     h_Ptp   - Handle to PTP object.
 @Param[in]     port    - Port to disable interrupt coalescing on.

*//***************************************************************************/
void PTP_TSU_DisableCoalescing(t_Handle h_Ptp, uint8_t port);

/** @} */ /* end of ptp_control_grp */
#endif /* PTP_QE_TX_ACCELERATION */


/**************************************************************************//**
 @Group         ptp_exceptions_grp PTP Exceptions Unit

 @Description   PTP exceptions API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        ptp_qe_tsu_events
 
 @Collection    PTP QE TSU Events
 
                Exceptions and events reported through the PTP QE TSU events
                callback routine. These events can be masked/unmasked using
                the PTP_ConfigEventsMask() routine.
 
                Each event is reported separately (values are not combined).
                Callback flags have no meaning in all these events.
 @{
*//***************************************************************************/
#ifndef PTP_V2
#define PTP_QE_EVENT_RX_TS_OVR          0x80000000   /**< Rx timestamp overrun - lost Rx timestamps synchronization(override timestamp)
                                                          (cannot be masked in out-of-band mode). */
#define PTP_QE_EVENT_TX_TS_OVR          0x40000000   /**< Tx timestamp overrun - lost Tx timestamps synchronization
                                                          (cannot be masked) */
#define PTP_QE_EVENT_SYNC_RX_TS         0x20000000   /**< PTP Sync frame was timestamped on Rx */
#define PTP_QE_EVENT_DELAY_REQ_RX_TS    0x10000000   /**< PTP Delay_Request frame was timestamped on Rx */
#define PTP_QE_EVENT_TX_TS              0x08000000   /**< PTP frame was timestamped on Tx */
#define PTP_QE_EVENT_ALL                (PTP_QE_EVENT_RX_TS_OVR | PTP_QE_EVENT_TX_TS_OVR | \
                                         PTP_QE_EVENT_SYNC_RX_TS | PTP_QE_EVENT_DELAY_REQ_RX_TS | \
                                         PTP_QE_EVENT_TX_TS)
                                                     /**< All events */
#else /*PTP_V2*/

#define PTP_QE_EVENT_RX_TS_OVR          0x40000000   /**< Rx timestamp overrun - lost Rx timestamps synchronization
                                                          (cannot be masked in out-of-band mode). */
#define PTP_QE_EVENT_TX_TS_OVR          0x20000000   /**< Tx timestamp overrun - lost Tx timestamps synchronization
                                                          (cannot be masked) */
#define PTP_QE_EVENT_SYNC_RX_TS         0x10000000   /**< PTP Sync frame was timestamped on Rx */
#define PTP_QE_EVENT_DELAY_REQ_RX_TS    0x08000000   /**< PTP Delay_Request frame was timestamped on Rx */
#define PTP_QE_EVENT_TX_TS              0x04000000   /**< PTP frame was timestamped on Tx */
#define PTP_QE_EVENT_PDELAY_REQ_RX_TS   0x02000000   /**< PTP PDelay_Request frame was timestamped on Rx */
#define PTP_QE_EVENT_PDELAY_RES_RX_TS   0x01000000   /**< PTP PDelay_Response frame was timestamped on Rx */
#define PTP_QE_EVENT_RX_EXT             0x00800000   /**< External trigger was not received to transmitter from PHY
                                                          No timestamp taken */
#define PTP_QE_EVENT_RX_EXR             0x80000000   /**< External trigger was not received to receiver from PHY
                                                          No timestamp taken */
#define PTP_QE_EVENT_ALL                (PTP_QE_EVENT_RX_TS_OVR | PTP_QE_EVENT_TX_TS_OVR | \
                                         PTP_QE_EVENT_SYNC_RX_TS | PTP_QE_EVENT_DELAY_REQ_RX_TS | \
                                         PTP_QE_EVENT_TX_TS |PTP_QE_EVENT_PDELAY_REQ_RX_TS |\
                                         PTP_QE_EVENT_PDELAY_RES_RX_TS|PTP_QE_EVENT_RX_EXT|\
                                         PTP_QE_EVENT_RX_EXR)
                                                   /**< All events */

#endif /*PTP_V2*/




/* @} */

/** @} */ /* end of ptp_exceptions_grp */
/** @} */ /* end of ptp_grp */



#endif /* __PTP_QE_EXT_H */
