/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_CDESC_EXT_H
#define __QE_IW_CDESC_EXT_H




/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_cdesc_grp IW CDESC Object Application Programming Interface

 @Description    IW CDESC Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_cdesc_init_grp IW CDESC Object Initialization Unit

 @Description   IW CDESC Object API Initialization Unit

 @{
*//***************************************************************************/

		/* GLOBAL CONSTANTS */
/* Defines used for sizing the driver */
#define MAX_IW_VLAN_VPRI_MAP_TABLE 			8

/**
 @Description   Destination Protocol IDs
*/
typedef enum {
 
      e_QE_IW_DEST_PROTO_NONE               = e_QE_FLTR_DEST_PROTO_NONE             /**< CPU */
    , e_QE_IW_DEST_PROTO_ETH_UNICAST        = e_QE_FLTR_DEST_PROTO_ETH_UNICAST      /**< Ethernet */
    , e_QE_IW_DEST_PROTO_AAL5_UNICAST       = e_QE_FLTR_DEST_PROTO_AAL5_UNICAST     /**< AAL5 unicast */
    , e_QE_IW_DEST_PROTO_AAL5_MULTICAST     = e_QE_FLTR_DEST_PROTO_AAL5_MULTICAST   /**< AAL5 multicast */
    , e_QE_IW_DEST_PROTO_AAL2_SSSAR         = e_QE_FLTR_DEST_PROTO_AAL2_SSSAR       /**< AAL2 SSAR */
    , e_QE_IW_DEST_PROTO_ATM_PW             = e_QE_FLTR_DEST_PROTO_ATM_PW           /**< ATM Pseudo Wire */
    , e_QE_IW_DEST_PROTO_PPPoHT             = e_QE_FLTR_DEST_PROTO_PPPoHT           /**< PPP over HDLC over TDM */
    , e_QE_IW_DEST_PROTO_VP                 = e_QE_FLTR_DEST_PROTO_VP               /**< VP recirculation */

} e_QeIwDestProtocol;


/**
@Description   Connection Descriptor User Parameters
*/
typedef struct {

    t_Handle                h_AppId;        /**< Application-provided ID for the Connection Descriptor */
    t_Handle                h_Device;       /**< Handle to the Device object to which this Connection Descriptor will belong */
    e_QeIwDestProtocol      destProtocol;   /**< Identifies the destination protocol of the outgoing port */
    uint8_t                 aal2Cid;        /**< The CID number (8-255), the same with the CID of the outgoing AAL2 SSSAR */
    bool                    internal;       /**< Whether to allocate the Connection Descriptor in internal or in external memory */
    uint16_t                saptidx;        /**< SAPT table index for connection using IPSec decryption or encryption */
} t_QeIwConnectionDescriptorParam;


/**
 @Function      QE_IW_CDESC_Config

 @Description   Creates a new Connection Descriptor object and initializes its internal data
                with default values and with the values specified by the user in p_Params.
 
 @Param         p_Params    - (In)    Pointer to user params structure to use for initialization.

 @Return        Handle to a new Connection Descriptor object.
 @Return        NULL in case of failure to create object or invalid parameters.
*/
t_Handle QE_IW_CDESC_Config (t_QeIwConnectionDescriptorParam *p_Params);


/**
 @Function      QE_IW_CDESC_Init

 @Description   Initializes microcode structures associated with this Connection Descriptor
                based on already configured information.

 @Param         h_ConnDescr     - (In) Handle to the Header manipulator.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_ConnDescr must have been previously created using QE_IW_CDESC_Config().
*/
t_Error    QE_IW_CDESC_Init (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_Free

 @Description   Frees the memory associated with this Connection Descriptor object.

 @Param         h_ConnDescr     - (In) Handle to the Connection Descriptor.

 @Return        E_OK if success. Error code otherwise.

 @Cautions      h_ConnDescr must have been previously created using QE_IW_CDESC_Config().
*/
t_Error    QE_IW_CDESC_Free (t_Handle h_ConnDescr);


/**************************************************************************//**
 @Group         iw_cdesc_init_advanced_grp IW CDESC Object Advanced Initialization Unit

 @Description   IW CDESC Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description   Identifiers for Statistics Mode
*/
typedef enum
{
      e_QE_IW_CDESC_STATISTICS_MODE_NONE            = 0 /**< No statistics */
    , e_QE_IW_CDESC_STATISTICS_MODE_SIMPLE_INT      = 1 /**< Use simple statistics counters into internal memory */
    , e_QE_IW_CDESC_STATISTICS_MODE_EXTENDED_INT    = 2 /**< Use extended statistics counters into internal memory */
    , e_QE_IW_CDESC_STATISTICS_MODE_SIMPLE_EXT      = 3 /**< Use simple statistics counters into external memory */
    , e_QE_IW_CDESC_STATISTICS_MODE_EXTENDED_EXT    = 4 /**< Use extended statistics counters into external memory */
 
} e_QeIwStatisticsMode;

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Description   IP Reassembly Full identifiers for per flow Statistics Mode
*/
typedef enum
{
      e_QE_IW_CDESC_STATISTICS_MODE_IPREASS_FULL_NONE = 0 /**< No statistics */
    , e_QE_IW_CDESC_STATISTICS_MODE_IPREASS_FULL_INT  = 1 /**< Use statistics counters into internal memory */
    , e_QE_IW_CDESC_STATISTICS_MODE_IPREASS_FULL_EXT  = 2 /**< Use statistics counters into external memory */
 
} e_QeIwStatisticsModeIpReassFull;

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */
 
/**
 @Description   Connection Descriptor VLAN Tag Indexes for Ethernet
 
                These QTags indexes must be set according to the GETH Termination transmit
                routines for performing VLAN operations on the transmitted frame.
*/
typedef enum
{
      e_QE_IW_CDESC_ETH_INSERT_NONE        = 0x00
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_0  = 0x20
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_1  = 0x24
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_2  = 0x28
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_3  = 0x2c
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_4  = 0x30
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_5  = 0x34
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_6  = 0x38
    , e_QE_IW_CDESC_ETH_INSERT_VLAN_TAG_7  = 0x3c

} e_QeIwConnDescrEthVlanIndex;


/**
 @Description   Connection Descriptor IP Header Offset for Ethernet
 
                These offsets must be set according to the GETH Termination transmit
                routines for performing IP header checksum calculation on the transmitted frame.
*/
typedef enum
{
      e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_0 = 0x00
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_1 = 0x01
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_2 = 0x02
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_3 = 0x03
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_4 = 0x40
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_5 = 0x41
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_6 = 0x42
    , e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_7 = 0x43
 
} e_QeIwConnDescrEthIpOffsetIndex;


/**
 @Description   Connection Descriptor AAL5 CLP and CNG bits
 
                These offsets must be set according to the ATM Termination transmit
                routines.
*/
typedef enum
{
      e_QE_IW_CDESC_ATM_AAL5_CLP            = 0x20   /**< ATM AAL5 Tx CLP (Cell Loss Priority) indication */
    , e_QE_IW_CDESC_ATM_AAL5_CNG            = 0x10   /**< ATM AAL5 Tx CNG (Congestion) indication */
    , e_QE_IW_CDESC_ATM_AAL5_CLP_CNG        = 0x30   /**< ATM AAL5 Tx CLP and CNG indication */
    , e_QE_IW_CDESC_ATM_AAL5_NO_CLP_CNG     = 0x00   /**< ATM AAL5 Tx No CLP and No CNG */

} e_QeIwConnDescrAal5HdrFlag;


/**
@Description    VLAN Operations; these values should be ORed together in order
                to achieve the desired VLAN functionality.
 
@Cautions       There are groups of values that are mutually exclusive,
                i.e. values which are visibly grouped together in this listing.
*/
typedef enum {

      e_QE_IW_CDESC_VLAN_NO_OPERATION                                   = 0x00000000    /**< No VLAN Operation - has no effect if ORed with other values */

    , e_QE_IW_CDESC_VLAN_UNTAGGED_INSERT_QTAG1                          = 0x40000000    /**< Insert QTag1 in untagged frames */
    , e_QE_IW_CDESC_VLAN_UNTAGGED_INSERT_QTAG1_AND_QTAG2                = 0x80000000    /**< Insert QTag1 and QTag2 in untagged frames */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_REMOVE_QTAG1                     = 0x20000000    /**< Remove the QTag from single tagged frames */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_INSERT_QTAG1                     = 0x08000000    /**< Insert QTag1 in single tagged frame */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_INSERT_QTAG1_AND_QTAG2           = 0x10000000    /**< Insert QTag1 and QTag2 in single tagged frame */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_UPDATE_VID                       = 0x01000000    /**< Replace VID portion of QTag in single tagged frames */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_UPDATE_VID_ZERO_WITH_DEFAULT     = 0x02000000    /**< If VID=0 replace VID with default value in single tagged frames */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_UPDATE_QTAG1                     = 0x03000000    /**< Replace QTag (with QTag1) in single tagged frames */
    , e_QE_IW_CDESC_VLAN_SINGLE_TAGGED_UPDATE_QTAG1_IF_VID_ZERO         = 0x04000000    /**< If VID=0 replace QTag (with QTag1) in single tagged frames */

    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_REMOVE_QTAG1                      = 0x00400000    /**< Remove outermost QTag from multiple-tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_REMOVE_QTAG1_AND_QTAG2            = 0x00600000    /**< Remove outermost two QTags from multiple-tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_INSERT_QTAG1                      = 0x00080000    /**< Insert QTag1 in multiple tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_INSERT_QTAG1_AND_QTAG2            = 0x00100000    /**< Insert QTag1 and QTag2 in multiple tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_UPDATE_VID_QTAG1                  = 0x00010000    /**< Replace VID of outermost QTag with that in QTag 1 in multiple-tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_UPDATE_VID_ZERO_WITH_DEFAULT      = 0x00020000    /**< If VID=0 replace VID of outermost QTag with VID of QTag1 (?) in multiple-tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_UPDATE_QTAG1                      = 0x00030000    /**< Replace outermost QTag with QTag1 in multiple-tagged frames */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_UPDATE_QTAG1_IF_VID_ZERO          = 0x00040000    /**< If VID=0 replace outermost QTag in multiple-tagged frames. The new QTag value must be specified in qType2 and qTci2 */
    , e_QE_IW_CDESC_VLAN_MULTI_TAGGED_UPDATE_QTAG1_AND_QTAG2            = 0x00050000    /**< Replace two outermost QTags with QTag1 and QTag2 */

    , e_QE_IW_CDESC_VLAN_QTAG_VPRI_MARK                                 = 0x00800000    /**< Mark priority bits in the outermost VLAN tag wit value taken from IP TOS  field Mapping Table or Diffserv  DSCP[0-2] field (same bits). Valid only for VLAN-tagged frames. */

    , e_QE_IW_CDESC_VLAN_SPECIFIC_REMOVE_QTAG1                          = 0x00000100    /**< Remove 4 bytes of QTag from Ethernet header */

    , e_QE_IW_CDESC_VLAN_SPECIFIC_UPDATE_QTAG1                          = 0x00000002    /**< Replace 4 bytes of QTag (QType and Tci) in Ethernet header */
    , e_QE_IW_CDESC_VLAN_SPECIFIC_UPDATE_QTYPE_AND_VPRI_MAP             = 0x00000082    /**< Replace QType and Tci in Ethernet header; the VPri field is replaced from provided Vpri mapping table, the index to this VPri mapping operation  is the incoming frame Vpri field.*/
 
    , e_QE_IW_CDESC_VLAN_SPECIFIC_INSERT_QTAG1                          = 0x00000001    /**< Insert QTag (QType and Tci) to Ethernet header */
    , e_QE_IW_CDESC_VLAN_SPECIFIC_INSERT_QTYPE_AND_VPRI_MAP             = 0x00000081    /**< Insert QType and Tci to Ethernet header; the VPri field is inserted from provided Vpri mapping table, the index to this VPri mapping operation is the incoming frame Vpri field.*/

} e_QeIwVlanOperation;


/**
@Description   Specifies the General type operation to be performed on the frame
*/
typedef enum {

      e_QE_IW_CDESC_HEADER_INSERT   = 0x80000000    /**< Insert Header */
    , e_QE_IW_CDESC_HEADER_REMOVE   = 0x40000000    /**< Remove Header */
    , e_QE_IW_CDESC_HEADER_REPLACE  = 0x20000000    /**< Replace Header */
    , e_QE_IW_CDESC_TRAILER_REMOVE  = 0x10000000    /**< Remove Trailer */

} e_QeIwHmGeneralOperation;


/**
@Description   Header Manipulation Operation Descriptor
*/
typedef enum {

      e_QE_IW_CDESC_HEADER_AND_LENGTH_INSERT            = 0x8000    /**< Insert a header and the length of the frame in the last two bytes of the header to be inserted
                                                                    (it includes all the HMANIPs performed up to this point).
                                                                    The size of the header must include the 2 bytes of the length filed.
                                                                    This mode is useful for inserting PPPoE headers. Trailer remove operations will be accounted if present */
} e_QeIwHmGeneralFlags;


/**
@Description   Instructs which type of Header Insert/Replace HMCD to generate for its containing HM Operation Descriptor
*/
typedef enum {

      e_QE_IW_CDESC_INTERNAL    = 0x01    /**< Header is located in Multiuser RAM */
    , e_QE_IW_CDESC_EXTERNAL    = 0x02    /**< Header is located in external RAM */
    , e_QE_IW_CDESC_LOCAL       = 0x03    /**< Header is located inside of this HMCD */

} e_QeIwHeaderType;


/**
@Description   Protocol Specific Operations to be performed on the frame.
               Should be used on flows after a parse capable filter has been invoked.
*/
typedef enum {

 
      e_QE_IW_CDESC_UPDATE_IP_TUP_HEADER        = 0x00000001    /**< Update the IP/TCP/UDP headers. Selection of the updated field is mandatory through the provided e_QeIwHmProtocolSpecificFlags */
    , e_QE_IW_CDESC_REMOVE_ETH_HEADER           = 0x00000002    /**< Remove Ethernet/802.3 MAC header (assuming it is first in the frame) */
    , e_QE_IW_CDESC_REMOVE_ETH_TRAILER          = 0x00000004    /**< Remove Ethernet/802.3 FCS and padding if exist. Default mode assumes the frame is received on the ETH interface. */
    , e_QE_IW_CDESC_COMPRESS_IP_HEADER          = 0x00000008    /**< Compress IP header. It assumes the an IP packet reaches this HMANIP. All preceiding headers should be removed. */
    , e_QE_IW_CDESC_DECOMPRESS_IP_PPPoE_HEADER  = 0x00000010    /**< Decompress compressed IP header on incoming PPPoE frames. */
    , e_QE_IW_CDESC_INSERT_PID_HEADER           = 0x00000020    /**< Insert PPP PID header. Useful on protocol-to-PPP IW flows. */
    , e_QE_IW_CDESC_INSERT_IP_UDP_HEADER        = 0x00000040    /**< Insert IP/UDP header in ETH frame. */
    , e_QE_IW_CDESC_INSERT_ETH_AND_PPPoE_HEADER = 0x00000080    /**< Insert ETH and PPPoE headers. Valid only on PPP-to-ETH IW flows */
    , e_QE_IW_CDESC_REMOVE_IPSEC_ESP_HEADER     = 0x00000100    /**< IPSEC remove ESP header */
    , e_QE_IW_CDESC_INSERT_IPSEC_ESP_HEADER     = 0x00000200    /**< IPSEC add ESP header */
 
} e_QeIwHmProtocolSpecificOperation;


/**
@Description   Protocol Specific Operations modes.
*/
typedef enum {

      e_QE_IW_CDESC_HMFLAG_NONE                             = 0x00000000    /**< Default mode of the selected operations is used.*/
    , e_QE_IW_CDESC_UPDATE_TCP_UDP_DST                      = 0x08000000    /**< Update the TCP/UDP destination port from the TCP/UDP header */
    , e_QE_IW_CDESC_UPDATE_TCP_UDP_SRC                      = 0x04000000    /**< Update the TCP/UDP source port from the TCP/UDP header */
    , e_QE_IW_CDESC_UPDATE_IP_DST                           = 0x02000000    /**< Update the IP destination address from the IP header */
    , e_QE_IW_CDESC_UPDATE_IP_SRC                           = 0x01000000    /**< Update the IP source address from the IP header */
    , e_QE_IW_CDESC_UPDATE_IP_TTL                           = 0x00800000    /**< Decrement by 1 the Time to Live field from the IP header */
    , e_QE_IW_CDESC_UPDATE_IP_TOS                           = 0x00400000    /**< Update the IP TOS from the IP header. Default setting is using the insertParam.ipTos value. */
    , e_QE_IW_CDESC_UPDATE_TCP_UDP_CHECKSUM                 = 0x00200000    /**< Update the TCP/UDP checksum from the TCP/UDP header */
    , e_QE_IW_CDESC_UPDATE_IP_TOS_BY_POLICER                = 0x00100000    /**< Update the IP TOS from the IP header based on policer ETH output */
    , e_QE_IW_CDESC_REMOVE_ETH_AND_PPPoE_HEADER             = 0x00080000    /**< Remove Ethernet/802.3 MAC header (assuming it is first in the frame) */
    , e_QE_IW_CDESC_REMOVE_ETH_TRAILER_2684_PAD_AND_FCS     = 0x00040000    /**< Remove 2684 bridged frame trailer */
    , e_QE_IW_CDESC_REMOVE_ETH_TRAILER_PAD                  = 0x00020000    /**< Remove Ethernet/802.3 FCS and padding if exist */
    , e_QE_IW_CDESC_REMOVE_ETH_TRAILER_REASSEMBLED_FCS      = 0x00010000    /**< Remove Ethernet/802.3 FCS and padding if exist */
    , e_QE_IW_CDESC_REMOVE_ETH_TRAILER_PPPoE_PAD_AND_FCS    = 0x00008000    /**< Remove Ethernet/802.3 FCS and padding if exist */
    , e_QE_IW_CDESC_REMOVE_ETH_TRAILER_PPPoE_PAD            = 0x00004000    /**< Remove Ethernet/802.3 FCS and padding if exist */
    , e_QE_IW_CDESC_INSERT_IP_UDP_IPSEC_MODE                = 0x00002000    /**< Update ESP header is in transport mode else it is in tunnel mode */
    , e_QE_IW_CDESC_UPDATE_IPSEC_ESP_HEADER_TRANSPORT_MODE  = 0x00001000    /**< Update ESP header is in transport mode else it is in tunnel mode */
    , e_QE_IW_CDESC_UPDATE_IPSEC_ESP_HEADER_IV_SIZE8        = 0x00000800    /**< Update ESP header IV size is 8 bytes else it is none  */
    , e_QE_IW_CDESC_UPDATE_IPSEC_ESP_HEADER_IV_SIZE16       = 0x00000400    /**< Update ESP header IV size is 16 bytes else it is none */
    , e_QE_IW_CDESC_REMOVE_IPSEC_ESP_HEADER_COPY_TTL        = 0x00000200    /**< Remove ESP header copy the ttl field in the IP header */
    , e_QE_IW_CDESC_INSERT_IP_UDP_HDR_IPSEC_MODE_COPY_TTL   = 0x00000200   /**< Insert IP header for IPSec tunneled mode  copy the ttl field in the IP header */
    , e_QE_IW_CDESC_REMOVE_IPSEC_ESP_HEADER_DECREMENT_TTL   = 0x00000100    /**< Remove ESP header decrement the ttl field in the IP header */
    , e_QE_IW_CDESC_INSERT_IP_UDP_HDR_IPSEC_MODE_DECREMENT_TTL  = 0x00000100    /**< Insert IP header for tunneled mode decrement the ttl field in the IP header */
    , e_QE_IW_CDESC_INSERT_IP_UDP_HDR_IPSEC_MODE_COPY_TOS       = 0x00000080    /**< Insert IP header for tunneled mode copy TOS field in the IP header */
    , e_QE_IW_CDESC_INSERT_IP_UDP_HDR_IPSEC_MODE_COPY_DF        = 0x00000040    /**< Insert IP header for tunneled mode copy DF field in the IP header */
    , e_QE_IW_CDESC_INSERT_IPSEC_ESP_HEADER_CALC_IP_CKSM    = 0x00000020    /**< Insert ESP header calculate IP checksum for the new frame  */
    , e_QE_IW_CDESC_COMPRESS_THRESHOLD_ENABLE               = 0x00000010    /**< Enable threshold check on IP Header compression. Packets larger than MTU (that should be fragmented) must not be compressed. */
    , e_QE_IW_CDESC_DECOMPRESS_LENGTH_REMOVE_CRC            = 0x00000008    /**< Remove 4 bytes of crc when calculating IP Length. Unless this mode is selected, IP Length is calculated according to original frame length and header/trailer removal. */
} e_QeIwHmProtocolSpecificFlags;


/**
 @Description   Connection Descriptor IP/UDP insert mode
*/
typedef enum
{
      e_QE_IW_CDESC_INSERT_MODE_IP          = 0x01        /**< Insert IP header; set up the length and generate the checksum of IP */
    , e_QE_IW_CDESC_INSERT_MODE_UDP         = 0x02        /**< Insert UDP header; set up the length and generate the checksum of UDP */
    , e_QE_IW_CDESC_INSERT_MODE_IP_AND_UDP  = 0x03        /**< Insert IP/UDP headers; set up the lengths and generate the checksums of IP and UDP */

} e_QeIwPspInsertMode;


/**
@Description   VLAN Operation Descriptor
*/
typedef struct {

    uint32_t    vlanOps;            /**< As many of the ::e_QeIwVlanOperation enum values as needed, ORed together (restrictions apply - some values are mutually exclusive) */

    uint16_t    qType1;             /**< Type of the first of the two user-specified QTags used for VLAN replacements */
    uint16_t    qTci1;              /**< TCI (VPri & VID) of the first of the two user-specified QTags used for VLAN replacements */

    uint16_t    qType2;             /**< Type of the second of the two user-specified QTags used for VLAN replacements */
    uint16_t    qTci2;              /**< TCI (VPri & VID) of the second of the two user-specified QTags used for VLAN replacements */

    uint8_t     vlanHdrManOffset;   /**< The offset of the first byte in the Ethernet header from the beginning of the frame */

    uint8_t     vPriMapTable[MAX_IW_VLAN_VPRI_MAP_TABLE];    /**< Type of the second of the two user-specified QTags used for VLAN replacements */

} t_QeIwVlanOperation;


/**
@Description   Header Manipulation Operation Descriptor
*/
typedef struct {

    uint32_t                    hmOperation;        /**< Operation specifier */
    uint16_t                    hmFlags;            /**< Option to automatically update the length field of the frame to be transmitted in the last two bytes which are being inserted. The length will be calculated as the original frame length received and will take into account all the header manipulation done on the frame up to this point */
    uint8_t                     hmSize;             /**< Size of header to insert/remove/replace or size of trailer to remove */
    uint8_t                     hmOffset;           /**< Offset from start of frame where to insert header/where header to remove/replace starts */
    e_QeIwHeaderType            hmType;             /**< Type of memory where is the header to insert/replace: internal/external/local */
    uint8_t                     *p_Header;          /**< Pointer to header to be inserted/replaced */

} t_QeIwGeneralHeaderManipulation;


/**
@Description   Protocol Specific Header Replacement Data
*/
typedef struct {

    uint32_t                ipSourceAddress;        /**< IP Encapsulation - IP Source Address */
    uint32_t                ipDestinationAddress;   /**< IP Encapsulation - IP Destination Address */
    uint8_t                 ipTos;                  /**< IP Encapsulation - IP TOS */
    uint16_t                sourcePort;             /**< TCP or UDP Source Port */
    uint16_t                destinationPort;        /**< TCP or UDP Destination Port */

} t_QeIwPspHeaderReplace;


/**
@Description   Protocol Specific Header Insert Data
*/
typedef struct {

    uint8_t                 size;                   /**< Size of header, in bytes, to be inserted in frame. */
    uint8_t                 ipUdpOffset;            /**< Offset from the beginning of the frame in bytes, where the header is being inserted. Only used for IP/UDP header insert */
    e_QeIwPspInsertMode     pspInsertMode;          /**< Only used for IP/UDP header insert */
    uint8_t                 *p_InsertHeader;        /**< Pointer to header to be inserted. */
    uint16_t                pid;                    /**< PPP PID to be inserted */
 
} t_QeIwPspHeaderInsert;


/**
@Description   Protocol Specific Header Compress
*/
typedef struct {

    uint16_t                cid;                /**< Context Identifier */
    uint16_t                maxHcFrameSize;     /**< Valid when e_QE_IW_CDESC_COMPRESS_THRESHOLD_ENABLE is used. Frame is compressed only when IPTotalLength in IP header =< maxHcFrameSiz.*/
    t_Handle                h_Compressor;       /**< Handle to the Compressor object */
    t_Handle                h_Decompressor;     /**< Handle to the Decompressor object */

} t_QeIwIpHcDc;

/**
@Description   Protocol Specific Header Manipulation Operation Descriptor
*/
typedef struct {

    uint32_t                    hmOperation;    /**< Protocol-specific opetations selection. Combination of flags from e_QeIwHmProtocolSpecificOperation according.*/
    uint32_t                    hmFlags;        /**< Protocol-specific flags used for header manipulation opetations. Combination of flags from e_QeIwHmProtocolSpecificFlag according to the protocol;*/
    e_QeIwHeaderType            hmType;         /**< Type of memory where for data required by this HMANIP set will be stored */
    t_QeIwPspHeaderReplace      replaceParam;   /**< Header Replace Parameters */
    t_QeIwPspHeaderInsert       insertParam;    /**< Header Insert Parameters */
    t_QeIwIpHcDc                hmIpHcDcParams; /**< IP Header Compressor specific parameters */
    uint16_t                    saptIndex;      /**< IPSEC specific - Index of the SA in the SAPT */
} t_QeIwProtocolSpecificOperation;


/**
 @Function      QE_IW_CDESC_ConfigStatisticsMode

 @Description   Configures and activates the Statistics to be gathered for this Connection Descriptor;
                default: no statistics.
                [NONE]
 
                The user can select statistic counters requested in another CDESC object
                for gathering statistics from both CDESCs. This can be achived by calling the
                QE_IW_CDESC_ConfigStatisticsIndex function.

 @Param         h_ConnDescr        - (In)    Handle of the Connection Descriptor object.
 @Param         newStatsMode    - (In)    New Mode for allocating the Statistics Counter/s

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      This function should not be called together with QE_IW_CDESC_ConfigStatisticsIndex
                function for the same CDESC (it will overwrite the settings).
                Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigStatisticsMode (t_Handle h_ConnDescr, e_QeIwStatisticsMode newStatsMode);

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Function      QE_IW_CDESC_ConfigStatisticsModeIpReassSupport

 @Description   Enables the IP Reassembly Full - Per Flow Statistics to be gathered
                for this Connection Descriptor;
                Default: no IPREASS statistics are available.
                [NONE]
 
                The user is able to gather IPREASS FULL specific statistics, through
                the use of QE_IW_CDESC_GetIpReassFullPerFlowStatistics().
                IPREASS support is available only for EXTENDED statistics modes
                (internal or external).
                can select statistic counters requested in another CDESC object
                for gathering statistics from both CDESCs. This can be achived by calling the
                QE_IW_CDESC_ConfigStatisticsIndex function.

 @Param         h_ConnDescr     - (In)    Handle of the Connection Descriptor object.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      This function should not be called together with QE_IW_CDESC_ConfigStatisticsIndex
                function for the same CDESC (it will overwrite the settings).
                Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_ConfigStatisticsModeIpReassSupport(t_Handle h_ConnDescr);

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Function      QE_IW_CDESC_ConfigStatisticsIndex
 
 @Description   Configures the same statistics index as in an already exisiting CDESC object for
                for gathering statistics from both CDESCs.
 
 @Param         h_ConnDescr     - (In)    Handle of the Connection Descriptor object
 @Param         h_ConnDescrSrc  - (In)    Handle to an already created Connection Descriptor object
 
 @Return        E_OK in case of    success. Error code otherwise.

 @Cautions      This function should not be called together with QE_IW_CDESC_ConfigStatisticsMode
                function for the same CDESC (it will overwrite the settings).
                Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init() for the destination Conection Descriptor object.
*/
t_Error QE_IW_CDESC_ConfigStatisticsIndex (t_Handle h_ConnDescr, t_Handle h_ConnDescrSrc);


/**
 @Function      QE_IW_CDESC_ConfigEthTxVlanIndex

 @Description   [Ethernet] Configures the use of the Termination VLAN tag insertion
                for all frames sent from Ethernet to CPU (a mechanism distinct from VLAN operation!).
                [e_QE_IW_CDESC_ETH_INSERT_NONE]

 @Param         h_ConnDescr        - (In)    Handle of the Connection Descriptor object.
 @Param         newVlanIndex    - (In)    Index of the VLAN tag in the VLAN table (configured by Termination)
                                         to insert in all outgoing frames.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().

*/
t_Error    QE_IW_CDESC_ConfigEthTxVlanIndex (t_Handle h_ConnDescr, e_QeIwConnDescrEthVlanIndex newVlanIndex);


/**
 @Function      QE_IW_CDESC_ConfigEthTxIpChecksumOffsetIndex

 @Description   [Ethernet] Configures the index of one of four offsets of IP header in the frame (configured in Termination)
                to be used for automatic computation of IP checksum for all outgoing frames.
                [e_QE_IW_CDESC_ETH_IP_HEADER_OFFSET_0]

 @Param         h_ConnDescr            - (In)    Handle of the Connection Descriptor object.
 @Param         newIpOffsetIndex    - (In)    Index of the IP offset in the Termination-configured table of IP offsets.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigEthTxIpChecksumOffsetIndex ( t_Handle h_ConnDescr,
                                                       e_QeIwConnDescrEthIpOffsetIndex newIpOffsetIndex);

/**
 @Function      QE_IW_CDESC_ConfigAal5HdrFlags

 @Description   [AAL5] Configures the CLP and CNG bits for outgoing cells.
                [e_QE_IW_CDESC_ATM_AAL5_NO_CLP_CNG]

 @Param         h_ConnDescr     - (In)    Handle of the Connection Descriptor object,
 @Param         newFlag            - (In)    The new value for the CLP and CNG bits.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigAal5HeaderFlags (t_Handle h_ConnDescr, e_QeIwConnDescrAal5HdrFlag newFlag);
 
/**
 @Function      QE_IW_CDESC_ConfigAddVlanOperation

 @Description   Adds a VLAN Operation descriptor to this Connection Descriptor object's
                internally-maintained list of operations.

 @Param         h_ConnDescr     - (In)    Handle of the Connection Descriptor object,
 @Param         p_VlanOp        - (In)    Pointer to user-supplied operation parameters.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      At most one VLAN operation per Connection Descriptor is allowed!
 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigAddVlanOperation (t_Handle h_ConnDescr, t_QeIwVlanOperation *p_VlanOp);


/**
 @Function      QE_IW_CDESC_ConfigAddHeaderManipulation

 @Description   Adds a specific header manipulation operation to the
                list of operations maintained internally by this Connection Descriptor object.

 @Param         h_ConnDescr     - (In) Handle of the Connection Descriptor object;
 @Param         p_Operation     - (In) Pointer to user-supplied operation parameters.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigAddHeaderManipulation (t_Handle h_ConnDescr, t_QeIwGeneralHeaderManipulation *p_Operation);


/**
 @Function      QE_IW_CDESC_ConfigAddProtocolSpecific

 @Description   Adds protocol specific header manipulation operations to the
                list of operations maintained internally by this Connection Descriptor object.

 @Param         h_ConnDescr     - (In) Handle of the Connection Descriptor object;
 @Param         p_Operation     - (In) Pointer to user-supplied operation parameters.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_ConfigAddProtocolSpecific (t_Handle h_ConnDescr, t_QeIwProtocolSpecificOperation *p_Operation);


/**
 @Function      QE_IW_CDESC_ConfigQueueManager

 @Description   Configures a Queue Manager (QM) object to use with this Connection Descriptor object.

 @Param         h_ConnDescr     - (In) Handle to the Connection Descriptor object.
 @Param         h_QueueManager  - (In) Handle to the QM object.

 @Return        E_OK if    success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigQueueManager (t_Handle h_ConnDescr, t_Handle h_QueueManager);


/**
 @Function      QE_IW_CDESC_ConfigPolicer

 @Description   Configures the policer profile to use with this Connection Descriptor object

 @Param         h_ConnDescr     - (In) Handle to the Connection Descriptor object;
 @Param         h_Policer       - (In) Handle to the policer object.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
                h_Profile should be obtained previously as a free profile from the policer(PLCR) object

*/
t_Error    QE_IW_CDESC_ConfigPolicer (t_Handle h_ConnDescr, t_Handle h_Policer);


/**
 @Function      QE_IW_CDESC_ConfigForwardCpuErrorFrames

 @Description   Configures this Connection Descriptor to be used for termination of packets which are forwarded to the CPU as a result of an error.
                [FALSE]
 
 @Param         h_ConnDescr         - (In) Handle to the Connection Descriptor object;
 @Param         enabled             - (In) TRUE, if this Connection Descriptor is used for error frames sent to CPU

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_ConfigForwardCpuErrorFrames(t_Handle h_ConnDescr, bool enabled);


/**
 @Function      QE_IW_CDESC_ConfigIpAlignment

 @Description   Activates IP Alignment for this Connection Descriptor object.
                [FALSE]
 
 @Param         h_ConnDescr     - (In) Handle to the Connection Descriptor object.
 @Param         enabled         - (In) TRUE to enable IP Alignment mechanism.

 @Return        E_OK if    success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config() and before QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_ConfigIpAlignment (t_Handle h_ConnDescr, bool enabled);

/**
 @Function      QE_IW_CDESC_ModifyIpsecSapt

 @Description   Modify the SAPT index for HMCD IPSec operation.
 
 @Param         h_ConnDescr     - (In) Handle to the Connection Descriptor object.
 @Param         saptIdx         - (In) Index of entry IN SAPT table accepted from QE_IW_IPSEC_AddSaEntry ( ) .

 @Return        E_OK if    success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_CDESC_Config().
*/
t_Error	QE_IW_CDESC_ModifyIpsecSapt (t_Handle h_ConnDescr, uint16_t saptIdx);



/** @} */ /* end of iw_cdesc_init_advanced_grp group */
/** @} */ /* end of iw_cdesc_init_grp group */


/**************************************************************************//**
 @Group         iw_cdesc_control_grp IW CDESC Object Control Unit

 @Description   IW CDESC Object API Control Unit

 @{
*//***************************************************************************/

/**
@Description   Connection Descriptor Statistics
*/
typedef struct {

    bool        extendedMode;       /**< Indicates if the extended statistic counters (described below) are valid or not */
    uint32_t    general_Cnt;        /**< This counter is always incremented, regardless extended counters mode */
    uint32_t    noTci_Cnt;          /**< Incremented for frames with no TCI fields                 - valid only if extendedMode is TRUE */
    uint32_t    tci1_Cnt;           /**< Incremented for frames with 1 TCI                         - valid only if extendedMode is TRUE */
    uint32_t    tci2_Cnt;           /**< Incremented for frames with 2 TCIs                     - valid only if extendedMode is TRUE */
    uint32_t    PPPoE_Cnt;          /**< Incremented for frames with PPPoE session header         - valid only if extendedMode is TRUE */
    uint32_t    IPv4_Cnt;           /**< Incremented for frames with IPv4 header                 - valid only if extendedMode is TRUE */
    uint32_t    TCP_Cnt;            /**< Incremented for frames with TCP header                 - valid only if extendedMode is TRUE */
    uint32_t    UDP_Cnt;            /**< Incremented for frames with UDP header                 - valid only if extendedMode is TRUE */
    uint32_t    TotalBytes_Cnt;     /**< Incremented for every frame as the total byte count     - valid only if extendedMode is TRUE */
    uint32_t    IPBytes_Cnt;        /**< Incremented for every IP frame as the total byte count    - valid only if extendedMode is TRUE */
    uint16_t    enqueueBusy_Cnt;    /**< Incremented for each enqueue busy condition encountered          - valid only if extendedMode is TRUE */
    uint16_t    IW_MC_OV_Cnt;       /**< Incremented for each overrun condition detected for Multicast Tx - valid only if extendedMode is TRUE */
    uint16_t    WR_GDrop_Cnt;       /**< Incremented for each WRED Green colored frame          - valid only if extendedMode is TRUE */
    uint16_t    WR_YDrop_Cnt;       /**< Incremented for each WRED Yellow colored frame         - valid only if extendedMode is TRUE */
    uint16_t    WR_RDrop_Cnt;       /**< Incremented for each WRED Red colored frame            - valid only if extendedMode is TRUE */
 
} t_QeIwConnectionDescriptorStatistics;

/**
 @Description	IP Reassembly Full Per Flow Statistics
*/
typedef struct
{
    uint32_t    fragTotal_Cnt;                      /**< Incremented when a valid fragment is received */
    uint32_t    reassembledBytes_Cnt;               /**< Incremented when a reassembled frame is forwarded to a destination queue */
    uint32_t    fragOverlap_Cnt;                    /**< Represents the number of overlap fragments for reassembled frames */
    uint32_t    fragDuplicate_Cnt;                  /**< Represents the number of duplicate fragments for reassembled frames */
    uint32_t    fragError_Cnt;                      /**< Represents the number of error fragments for reassembled frames */
    uint32_t    reassembledFrame_Cnt;               /**< Represents the number of reassembled frames that are forwarded to destination queue */
    uint32_t    reassembledFrameDrop_Cnt;           /**< Represents the number of reassembled frames that are droped due to fragment duplicate/overlap/error received for this frame */
    uint32_t    reassembledFrameToCpu_Cnt;          /**< Represents the number of reassembled frames that are forwarded to CPU due to fragment duplicate/overlap/error received for this frame */
    uint32_t    reassembledFrameDropBusyWred_Cnt;   /**< Represents the number of reassembled frames that are dropped due to busy condition in destionation queue or due to WRED*/
    uint32_t    resserved[7];                       /**< Reserved. Should be cleared */
} t_QeIwIpReassFullPerFlowStatistics;


/**
 @Function      QE_IW_CDESC_ModifyQueueManager

 @Description   Modifies the Queue Manager (QM) object for this Connection Descriptor object.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 @Param         h_QueueManager  - (In)    Handle to the new QM object to be used in this Connection descriptor.
                                        To remove the QM functionality from this Connection descriptor, use NULL value.
 
 @Return        E_OK if success; Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
 @Cautions      The QM mechanism must be disabled for this Connection descriptor before calling this function.
*/
t_Error QE_IW_CDESC_ModifyQueueManager (t_Handle h_ConnDescr, t_Handle h_QueueManager);


/**
 @Function      QE_IW_CDESC_QueueManagerEnable

 @Description   Enables the Queue Manager (QM) for this Connection descriptor.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 
 @Return        E_OK if success; Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
 @Cautions      The QM should be previously configured at design time by calling the QE_IW_CDESC_ConfigQueueManager function
                or at run time by calling the QE_IW_CDESC_ModifyQueueManager function.
*/
t_Error QE_IW_CDESC_QueueManagerEnable (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_QueueManagerDisable

 @Description   Disables the Queue Manager (QM) for this Connection descriptor.

 @Param         h_ConnDescr        - (In)    Handle to the Connection descriptor.
 
 @Return        E_OK if success, error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_QueueManagerDisable (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_IpAlignmentEnable

 @Description   Enables the IP Header Alignment for this Connection descriptor.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 
 @Return        E_OK if success; Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_IpAlignmentEnable (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_IpAlignmentDisable

 @Description   Disables the IP Header Alignment for this Connection descriptor.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 
 @Return        E_OK if success, error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_IpAlignmentDisable (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_ModifyPolicer

 @Description   Modifies the profile used for this Connection descriptor.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 @Param         h_Policer       - (In)    Handle to the new profile to be used in this Connection descriptor.
                                        To remove the Policer functionality from this Connection descriptor, use NULL value.
 
 @Return        E_OK if success, error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
 @Cautions      The policer mechanism must be disabled for this Connection descriptor before calling this function.
*/
t_Error QE_IW_CDESC_ModifyPolicer(t_Handle h_ConnDescr, t_Handle h_Policer);


/**
 @Function      QE_IW_CDESC_PolicerEnable

 @Description   Enables the policer mechanism for this Connection descriptor.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 
 @Return        E_OK if success, error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_PolicerEnable (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_PolicerDisable

 @Description   Disables the policer mechanism for this Connection descriptor.

 @Param         h_ConnDescr        - (In)    Handle to the Connection descriptor.
 
 @Return        E_OK if success, error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_PolicerDisable (t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_GetStatistics

 @Description   Retrieves at runtime the statistics gathered for this Connection Descriptor.

 @Param         h_ConnDescr     - (In)  Handle to the Connection Descriptor,
 @Param         p_Stats            - (Out)    Pointer to user-supplied structure where to place the statistics gathered

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_GetStatistics (t_Handle h_ConnDescr, t_QeIwConnectionDescriptorStatistics *p_Stats);

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Function      QE_IW_CDESC_GetIpReassFullPerFlowStatistics

 @Description   Retrieves at runtime the IP Reassembly Full Per Flow statistics gathered for this Connection Descriptor.

 @Param         h_ConnDescr - (In)  Handle to the Connection Descriptor,
 @Param         p_Stats     - (Out)    Pointer to user-supplied structure where to place the statistics gathered

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error	QE_IW_CDESC_GetIpReassFullPerFlowStatistics(t_Handle h_ConnDescr, t_QeIwIpReassFullPerFlowStatistics *p_Stats);

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Function      QE_IW_CDESC_StopStatistics

 @Description   Stop the statistics gathering for this Connection Descriptor.
 
 @Param         h_ConnDescr     - (In)  Handle to the Connection Descriptor,

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_StopStatistics ( t_Handle h_ConnDescr);


/**
 @Function      QE_IW_CDESC_RestartStatistics

 @Description   Restart the statistics gathering for this Connection Descriptor.
 
                Restarting the statistics gathering for this Connection Descriptor,
                is valid operation only if were configured as enabled prior to QE_IW_CDESC_Init() call.
 

 @Param         h_ConnDescr     - (In)  Handle to the Connection Descriptor,

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_RestartStatistics ( t_Handle h_ConnDescr );


/**
 @Function      QE_IW_CDESC_ResetStatistics

 @Description   Clear the statistics counters for this Connection Descriptor.
 
                In order to safely clear the statistics counters for this Connection Descriptor,
                this routine disable the statistics gathering, clear counters, and enable the statistics if
                were enabled prior to QE_IW_CDESC_ResetStatistics call.
                Reset the statistics counters for for this Connection Descriptor,
                is valid operation only if statistics gathering was configured as enabled prior to QE_IW_CDESC_Init() call.
 

 @Param         h_ConnDescr     - (In)  Handle to the Connection Descriptor,

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CDESC_Init.
*/
t_Error QE_IW_CDESC_ResetStatistics ( t_Handle h_ConnDescr );

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Function      QE_IW_CDESC_ResetIpReassFullPerFlowStatistics

 @Description   Clear the statistics counters for this Connection Descriptor.
 
                In order to safely clear the statistics counters for this Connection Descriptor,
                this routine disable the statistics gathering, clear counters, and enable the statistics if
                were enabled prior to QE_IW_CDESC_ResetStatistics call.
                Reset the statistics counters for for this Connection Descriptor,
                is valid operation only if statistics gathering was configured as enabled prior to QE_IW_CDESC_Init() call.
 

 @Param         h_ConnDescr     - (In)  Handle to the Connection Descriptor,

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CDESC_Init.
*/
t_Error QE_IW_CDESC_ResetIpReassFullPerFlowStatistics ( t_Handle h_ConnDescr );

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */


/**
 @Function      QE_IW_CDESC_GetAppId

 @Description   Retrieves the App handle.

 @Param         h_ConnDescr         - (In)  Handle to the Connection Descriptor,
 @Param         h_AppId             - (Out)    Pointer to user-supplied App handle
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error    QE_IW_CDESC_GetAppId (t_Handle h_ConnDescr, t_Handle *h_AppId);


/**
 @Function      QE_IW_CDESC_GetIwctIndex

 @Description   Retrieves the IWCT index of a Connection Descriptor.

 @Param         h_ConnDescr     - (In)    Handle to the Connection descriptor.
 @Param         p_IwctIndex     - (Out)    Pointer to user-supplied variable where to place the IWCT index of CDESC.
 
 @Return        E_OK if success
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_GetIwctIndex (t_Handle h_ConnDescr, uint16_t *p_IwctIndex);


#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**
 @Function      QE_IW_CDESC_DumpRegs

 @Description   Dumps QE IW CDESC registers and parameter RAM.

 @Param         h_ConnDescr - (In) Handle to the Connection descriptor.
 
 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_ConnDescr must have been previously created using QE_IW_CDESC_Config()
                and initialized using QE_IW_CDESC_Init().
*/
t_Error QE_IW_CDESC_DumpRegs(t_Handle h_ConnDescr);
#endif /* (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0)) */

/** @} */ /* end of iw_cdesc_control_grp group */
/** @} */ /* end of iw_cdesc_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_CDESC_EXT_H */
