/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_COMMON_EXT_H
#define __QE_IW_COMMON_EXT_H




/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_common_grp IW COMMON Object Application Programming Interface

 @Description    IW COMMON Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_common_init_grp IW COMMON Object Initialization Unit

 @Description   IW COMMON Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    User Parameters for the Interworking Common Object
*/
typedef struct
{
    t_Handle    h_QeId;         /**< Handle to the controlling QE. Taken from Termination UCC GETH or ATM initialization. */
    uint8_t     pramMemId;      /**< Parameter RAM partition ID */
    uint16_t    maxMcastGroups; /**< Max number of multicast groups for the system */
    uint16_t    maxConnections; /**< Max number of external interworking connections for the system */

} t_QeIwCommonParam;


/**
 @Function      QE_IW_COMMON_Config

 @Description   Creates an uninitialized Interworking Common object,
                used for maintaining the global state of the Interworking machine.

 @Param         p_Params - (In) Required non-default parameters for the Interworking Common object.

 @Return        Handle to the newly created Common object to be later passed in API calls.
 @Return        NULL in case of failure to create object or invalid parameters.
 
 @Cautions      At most one such object may exist in the system at any time,
                rule that is enforced by this function.
*/
t_Handle QE_IW_COMMON_Config (t_QeIwCommonParam *p_Params);


/**
 @Function      QE_IW_COMMON_Init

 @Description   Init the Interworking Common object,
                used for maintaining the global state of the Interworking machine.

 @Param         h_Common    - (In)    Handle to the Interworking Common object.

 @Return        E_OK in case of    success. Error code otherwise.
 
 @Cautions      Common object must have been previously created using QE_IW_COMMON_Config().
*/
t_Error    QE_IW_COMMON_Init (t_Handle h_Common);


/**
 @Function      QE_IW_COMMON_Free

 @Description   Frees the resources associated to the Common object.
                The rule is that only resources allocated during previous calls
                to QE_IW_COMMON_Config and QE_IW_COMMON_Init are freed.
 
 @Param         h_Common     - (In)    Handle to the IW Common object as returned by QE_IW_COMMON_Config function.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Any other resources that may be referenced from the IW Common object and that
                were allocated using other APIs must be freed using the same APIs that allocated them.
*/
t_Error    QE_IW_COMMON_Free (t_Handle h_Common);


/**************************************************************************//**
 @Group         iw_common_init_advanced_grp IW COMMON Object Advanced Initialization Unit

 @Description   IW COMMON Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Function      QE_IW_COMMON_ConfigNumOfRulesPerConnection

 @Description   Configures the number of rules that can be configured per Interworking connection.
                [5]

 @Param         h_Common           - (In) Handle to the Interworking Common object.
 @Param         numOfRules         - (In) The number of rules that can be configured per connection

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_COMMON_Config() and before QE_IW_COMMON_Init().
*/
t_Error QE_IW_COMMON_ConfigNumOfRulesPerConnection(t_Handle h_Common, uint8_t numOfRules);

#ifndef QE_IW_PPPOHT_UNAVAILABLE
/**
 @Function      QE_IW_COMMON_ConfigIwToMultiMcc

 @Description   Disables or enables the IwToMultiMCC

 @Param         h_Common      - (In) Handle to the Interworking Common object.
 @Param         iwToMultiMcc  - (In) Value for marking the feature as disabled or enabled
                                     default is FALSE (disabled), if TRUE feature is enabled
 
 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_COMMON_Config() and before QE_IW_COMMON_Init().
*/

t_Error QE_IW_COMMON_ConfigIwToMultiMcc(t_Handle h_Common, bool iwToMultiMcc);

#endif /* !QE_IW_PPPOHT_UNAVAILABLE */

/** @} */ /* end of iw_common_init_advanced_grp group */
/** @} */ /* end of iw_common_init_grp group */
/** @} */ /* end of iw_common_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_COMMON_EXT_H */
