/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_DEV_EXT_H
#define __QE_IW_DEV_EXT_H



#include "qe_ext.h"
#include "qe_iw_dev_ext.h"

        /* MACROS */

/**
 @Description    Interworking Interrupt Codes
*/

typedef enum {
	
	  e_QE_IW_INTRQ_RX_FRAME_ENTRY			= 0x0001
	, e_QE_IW_INTRQ_FRAME_ERROR				= 0x0002
	, e_QE_IW_INTRQ_ENQUEUE_BUSY			= 0x0003
	, e_QE_IW_INTRQ_CPU_BUSY				= 0x0004
	, e_QE_IW_INTRQ_MCAST_TX_OVERWRITTEN	= 0x0005
	, e_QE_IW_INTRQ_MCAST_RX_OVERWRITTEN	= 0x0006
	, e_QE_IW_INTRQ_PPP_DEMULTIPLEXER_BUSY  = 0x0007
	, e_QE_IW_INTRQ_MLMC_PPP_FBP_BUSY       = 0x0008
	, e_QE_IW_INTRQ_PW_SEQ_NUM_ZERO_ERROR   = 0x0009
	, e_QE_IW_INTRQ_IPSEC_ERROR   			= 0x0020
} e_QeIwInterworkingIntrQueueEntryFlags;

/**
 @Description    Interworking Parse and Lookup Interrupt Codes
*/
typedef enum {
	
	  e_QE_PL_INTRQ_LAST_PCD			            = 0x0080

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

	, e_QE_IW_INTRQ_IP_REASS_NUM_SIM_FLOWS_EXCEEDED = 0x0010
	, e_QE_IW_INTRQ_IP_REASS_TIME_OUT               = 0x0011
	, e_QE_IW_INTRQ_IP_REASS_FQP_BUSY               = 0x0014
	, e_QE_IW_INTRQ_IP_REASS_REQUEST_TABLE_BUSY     = 0x0015

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

	, e_QE_IW_INTRQ_IP_REASS_FULL_NEW_BUCKET        = 0x0030
	, e_QE_IW_INTRQ_IP_REASS_FULL_BUCKET_BUSY       = 0x0031
	, e_QE_IW_INTRQ_IP_REASS_FULL_TIMEOUT           = 0x0032
	, e_QE_IW_INTRQ_IP_REASS_FULL_FRAGMENT_BUSY     = 0x0033
	, e_QE_IW_INTRQ_IP_REASS_FULL_BD_BUSY           = 0x0034
	, e_QE_IW_INTRQ_IP_REASS_FULL_FRAME_BUSY        = 0x0035

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */
 
	, e_QE_PL_INTRQ_LOOKUP_HIT			            = 0x0081
	, e_QE_PL_INTRQ_LOOKUP_MISS			            = 0x0082
	, e_QE_PL_INTRQ_UNRECOGNIZED_FRAME	            = 0x0083
	, e_QE_PL_INTRQ_IPHC_NEW_ADL                    = 0x0090
	, e_QE_PL_INTRQ_IPHC_GENERATION_CHANGE          = 0x0091
	
} e_QeIwParseLookupIntrQueueEntryFlags;

/**
 @Description    Interworking CPU Swap Interrupt Codes
*/
typedef enum {

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

    e_QE_IW_CPU_SWAP_INTRQ_IP_REASS_ERROR	= 0x0036,

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

    e_QE_IW_CPU_SWAP_INTRQ_IP_SEC_ERROR   = 0x0020
	
} e_QeIwCpuSwapIntrQueueEntryFlags;

/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_dev_grp IW DEV Object Application Programming Interface

 @Description    IW DEV Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_dev_init_grp IW DEV Object Initialization Unit

 @Description   IW DEV Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Types of Devices for Interworking
*/
typedef enum {
 
      e_QE_IW_DEV_TYPE_ETH          = 1 /**< Ethernet Device */
    , e_QE_IW_DEV_TYPE_PPPoHT       = 2 /**< PPPoHT Device */
    , e_QE_IW_DEV_TYPE_ATM          = 3 /**< ATM Device */
    , e_QE_IW_DEV_TYPE_VP           = 4 /**< VP Device */
} e_QeIwDeviceType;


/**
 @Description    Identifiers for possible exceptions in Interworking
*/
typedef enum {

      e_QE_IW_EXCEPTION_INTERWORKING    = 1    /**< Interworking Exception */
    , e_QE_IW_EXCEPTION_PARSE_LOOKUP    = 2    /**< Parse and Lookup Exception */
 
} e_QeIwException;


/**
 @Description    Identifiers for Interworking Events
*/
typedef enum {

      e_QE_IW_INTERWORKING_EVENT_UNKNOWN                                                        /**< Unknown IW Interrupt event */
    , e_QE_IW_INTERWORKING_EVENT_INTERRUPT_QUEUE_OVERFLOW                                       /**< The IW Interrupt Queue is full (try increase the number of entries using QE_IW_DEV_ConfigIwInterruptQueue function) */
    , e_QE_IW_INTERWORKING_EVENT_FRAME_ERROR            = e_QE_IW_INTRQ_FRAME_ERROR             /**< Erroneous Frame */
    , e_QE_IW_INTERWORKING_EVENT_ENQUEUE_BUSY           = e_QE_IW_INTRQ_ENQUEUE_BUSY            /**< Enqueue Busy */
    , e_QE_IW_INTERWORKING_EVENT_CPU_BUSY               = e_QE_IW_INTRQ_CPU_BUSY                /**< CPU Busy */
    , e_QE_IW_INTERWORKING_EVENT_MCAST_TX_OVERWRITTEN   = e_QE_IW_INTRQ_MCAST_TX_OVERWRITTEN    /**< Multicast Transmitter Queue Overwritten (Transmitter-generated event) */
    , e_QE_IW_INTERWORKING_EVENT_MCAST_RX_OVERWRITTEN   = e_QE_IW_INTRQ_MCAST_RX_OVERWRITTEN    /**< Multicast Transmitter Queue Overwritten (Receiver-generated event) */
    , e_QE_IW_INTERWORKING_EVENT_PPP_DEMULTIPLEXER_BUSY = e_QE_IW_INTRQ_PPP_DEMULTIPLEXER_BUSY  /**< Fragments/frames are discarded due to demultiplexer busy condition */
    , e_QE_IW_INTERWORKING_EVENT_MLMC_PPP_FBP_BUSY      = e_QE_IW_INTRQ_MLMC_PPP_FBP_BUSY       /**< Indicates that the receiver does not have valid buffer pointers available in the ML MC PPP FBP (issued for ATM and ETH receiver) */
    , e_QE_IW_INTERWORKING_EVENT_PW_SEQ_NUM_ZERO_ERROR  = e_QE_IW_INTRQ_PW_SEQ_NUM_ZERO_ERROR   /**< Pseudo Wire frame sequence number is expected to be 0, but is not (issued for ETH receiver) */

} e_QeIwEventInterworking;


/**
 @Description    Identifiers for Parse and Lookup Events
*/
typedef enum {

      e_QE_IW_PARSE_LOOKUP_EVENT_INTERRUPT_QUEUE_OVERFLOW                                  /**< The Parse Lookup Interrupt Queue is full (try increase the number of entries using QE_IW_DEV_ConfigParseLookupInterruptQueue function) */
    , e_QE_IW_PARSE_LOOKUP_EVENT_GENERAL_MISS           = e_QE_PL_INTRQ_LAST_PCD           /**< At the end of the filtering process, no filter was able to generate a lookup hit on a certain frame */
    , e_QE_IW_PARSE_LOOKUP_EVENT_LOOKUP_HIT             = e_QE_PL_INTRQ_LOOKUP_HIT         /**< A filter has generated a lookup hit on a certain frame */
    , e_QE_IW_PARSE_LOOKUP_EVENT_LOOKUP_MISS            = e_QE_PL_INTRQ_LOOKUP_MISS        /**< A lookup miss has occurred on a certain frame for a certain filter */
    , e_QE_IW_PARSE_LOOKUP_EVENT_UNRECOGNIZED_FRAME     = e_QE_PL_INTRQ_UNRECOGNIZED_FRAME /**< The format of the frame was unrecognized by the filtering subsystem */
    , e_QE_IW_PARSE_LOOKUP_EVENT_IPHC_NEW_ADL           = e_QE_PL_INTRQ_IPHC_NEW_ADL       /**< Only in Automatic Decompression Learning Mode (ADL): A frame with context that is not valid has arrived and the QE validated and update the context */
    , e_QE_IW_PARSE_LOOKUP_EVENT_IPHC_GENERATION_CHANGE = e_QE_PL_INTRQ_IPHC_GENERATION_CHANGE   /**< A Frame with a change in generation number has been received. */

} e_QeIwEventParseLookup;

/**
 @Description    Exception-specific data for Parse and Lookup Exceptions. It is used to identify detected protocols ( provided in the parseFlags field of the t_QeIwExceptionParseLookup struct.
*/
typedef enum {

      e_QE_IW_PL_EXCEPTION_DETECTED_TCI1                = 0x80000000    /**< Indicates that at least 1 TCI header was found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_TCI2                = 0x40000000    /**< Indicates that at least 2 TCI headers were found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_TCI3                = 0x20000000    /**< Indicates that at least 3 TCI headers were found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_TCI4                = 0x10000000    /**< Indicates that at least 4 TCI headers were found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_TCI5                = 0x08000000    /**< Indicates that at least 5 TCI headers were found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_TCI6                = 0x04000000    /**< Indicates that at least 6 TCI headers were found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_IP_OPITONS       = 0x02000000    /**< The IP header does not contain IP OPTIONS */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_IP_FRAGMENTATION = 0x01000000    /**< The packet is not an IP fragment */
    , e_QE_IW_PL_EXCEPTION_DETECTED_PPPoE               = 0x00800000    /**< Indicates that a PPPoE header was found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_IP                  = 0x00400000    /**< Indicates that an IP header was found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_TCP                 = 0x00200000    /**< Indicates that a TCP header was found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_UDP                 = 0x00100000    /**< Indicates that an UDP header was found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_SCTP                = 0x00080000    /**< Indicates that an SCTP header was found. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_ICMP             = 0x00000200    /**< Indicates that the frame is not ICMP */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_IGMP             = 0x00000100    /**< Indicates that the frame is not IGMP */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_ARP              = 0x00000080    /**< Indicates that the frame is not ARP */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_DHCP             = 0x00000040    /**< Indicates that the frame is not DHCP */
    , e_QE_IW_PL_EXCEPTION_DETECTED_NO_TCP_CONTROL      = 0x00000020    /**< Indicates that the TCP header does not contain any set control flags */
 
}e_QeIwExceptionParseLookupDetectedProtocols;


/**
 @Description    Exception-specific data for Parse and Lookup Exceptions. It is used to identify detected AAL5 RFC encapsulations ( provided in the rfcFlags field of the t_QeIwExceptionParseLookup struct.
*/
typedef enum {

      e_QE_IW_PL_EXCEPTION_DETECTED_RFC_2684B           = 0x00008000    /**< Indicates that the frame is encapsulated according to RFC2684 Bridged mode. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_RFC_2684R           = 0x00004000    /**< Indicates that the frame is encapsulated according to RFC2684 Routed mode. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_RFC_2364            = 0x00002000    /**< Indicates that the frame is encapsulated according to RFC2364. */
    , e_QE_IW_PL_EXCEPTION_DETECTED_RFC_2364_NON_COMP   = 0x00001000    /**< Indicates that the frame is encapsulated according to RFC2364, with non-compressed PID (2 bytes). */
    , e_QE_IW_PL_EXCEPTION_DETECTED_RFC_2364_COMP       = 0x00000800    /**< Indicates that the frame is encapsulated according to RFC2364, with compressed PID (1 byte). */
 
}e_QeIwExceptionParseLookupDetectedRfcEncapsulation;


/**
 @Description    IW Interrupt Queue Entry for ETHERNET Receive Protocol
*/
typedef struct {

    t_Handle                h_AppId;        /**< Handle to the upper layer */
    e_QeFltrDestProtocol    destProtocol;   /**< This field identifies the destination protocol of the outgoing port */
    uint8_t                 flags;          /**< Flags - error dependent configuration */
    uint16_t                iwctIndex;      /**< Index to Interworking Connection Table; valid only for PW Sequence Number Zero Interrupt */
	uint16_t                tqdIndex;           /**< TxQD Index taken from IWAD. Use TERMINATION DRIVER API functions to make the conversion to significant information. Valid for VP-to-destProtocol flows. */
	bool                    isExtTQD;           /**< Indicates if the TQD Index is the one of an internal of external Tx QD */

} t_QeIwExceptionInterworkingEth;

/**
 @Description    IW Interrupt Queue Entry for Virtual Port
*/
typedef struct {

    t_Handle                h_AppId;        /**< Handle to the upper layer */
    e_QeFltrDestProtocol    destProtocol;   /**< This field identifies the destination protocol of the outgoing port */
    uint8_t                 flags;          /**< Flags - error dependent configuration */
	uint16_t                tqdIndex;       /**< TxQD Index taken from IWAD. Use TERMINATION DRIVER API functions to make the conversion to significant information. Valid for VP-to-destProtocol flows. */
	bool                    isExtTQD;       /**< Indicates if the TQD Index is the one of an internal of external Tx QD */
 
} t_QeIwExceptionInterworkingVp;

/**
 @Description    IW Interrupt Queue Entry for ATM Receive Protocol
*/
typedef struct {

    t_Handle                h_AppId;        /**< Handle to the upper layer */
    uint8_t                 cid;            /**< CID - valid only for AAL2 */
    uint8_t                 cpuQueue;       /**< The number of the Busy CPU Queue - valid only for e_QE_IW_INTERWORKING_EVENT_ENQUEUE_BUSY interrupt  */
    e_QeFltrDestProtocol    destProtocol;   /**< This field identifies the destination protocol of the outgoing port */
    uint8_t                 flags;          /**< Flags - error dependent configuration */
	uint16_t                tqdIndex;       /**< TxQD Index taken from IWAD. Use TERMINATION DRIVER API functions to make the conversion to significant information. Valid for ATM-to-destProtocol flows.*/
	bool                    isExtTQD;       /**< Indicates if the TQD Index is the one of an internal of external Tx QD */
 
} t_QeIwExceptionInterworkingAtm;

/**
 @Description    IW Interrupt Queue Entry for PPPoHT Receive Protocol
*/
typedef struct {

    t_Handle                h_AppId;         /**< Handle to the upper layer */
    bool                    muxFrame;        /**< FALSE if non MUX PPP frame was received TRUE if MUX PPP frame was received */
    bool                    mlMcFrame;       /**< FALSE if Plain PPP frame was received TRUE if ML/MC PPP frame was received */
    uint8_t                 classID;         /**< Class ID - valid only mlMcFrame is TRUE */
    uint8_t                 bundleID;        /**< Bundle ID - valid only mlMcFrame is TRUE */
    uint8_t                 linkID;          /**< Link ID - valid only mlMcFrame is FALSE */
    e_QeFltrDestProtocol    destProtocol;    /**< This field identifies the destination protocol of the outgoing port */
    uint8_t                 flags;           /**< Flags - error dependent configuration */
	uint16_t                tqdIndex;        /**< TxQD Index taken from IWAD. Use TERMINATION DRIVER API functions to make the conversion to significant information. Valid for PPP-to-destProtocol flows. */
	bool                    isExtTQD;        /**< Indicates if the TQD Index is the one of an internal of external Tx QD */
 
} t_QeIwExceptionInterworkingPppoht;

/**
 @Description    IW Interrupt Queue Entry
*/
typedef struct {

    e_QeIwDeviceType            devType;    /**< Receive protocol */
    e_QeIwEventInterworking     type;       /**< Type of interrupt */
 
    union _dev{

        t_QeIwExceptionInterworkingEth      eth;    /**< Ethernet specific interrupt */
        t_QeIwExceptionInterworkingVp       vp;     /**< Virtual Port specific interrupt */
        t_QeIwExceptionInterworkingAtm      atm;    /**< ATM specific interrupt */
        t_QeIwExceptionInterworkingPppoht   pppoht; /**< PPPoHT specific interrupt */
 
    } dev;
 
} t_QeIwExceptionInterworking;


/**
 @Description    Exception-specific data for Parse and Lookup Exceptions
*/
typedef struct {

    uint32_t            parseFlags; /**< Selection of flags indicatind the detected protocols and options. Used e_QeIwExceptionParseLookupDetectedProtocols values to make the identification. */
    uint32_t            rfcFlags;   /**< Selection of flags indicatind the detected AAL5 encapsulation. Used e_QeIwExceptionParseLookupDetectedRfcEncapsulation values to make the identification. */
    uint8_t             userId;     /**< User-provided ID of the filter that caused the exception */
    t_QeFltrLookupKey   lookupKey;  /**< Pointer to the lookup key that was generated during filtering process (valid only for some events, i.e. LookupHit, LookupMiss, GeneralMiss; NULL in all other cases) */
} t_QeIwExceptionParseLookup;

/**
 @Description    Exception-specific data for IPHC Exceptions
*/
typedef struct {

    uint8_t     userId;             /**< User-provided ID of the filter that caused the exception */
    bool        fullHeader;         /**< Indicates whether the interrupt is for Full Header (TRUE) or Compressed Header (FALSE) */
    bool        contextInvalidate;  /**< Indicates that the context was invalidated by the QE due to generation change that is greater than DiffGen. This bit can be set by the QE only when ADL=0.*/
    uint8_t     generationNumber;   /**< Generation number of the frame. Valid only for Generation Change interrupt.*/
    uint16_t    cid;                /**< Context ID */

}t_QeIwExceptionIphc;

/**
 @Description    Exception-specific data for Parser Exceptions
*/
typedef struct {

    e_QeIwEventParseLookup    type;    /**< Combination of flags (defined in ::e_QeIwEventParseLookup) indicating the event(s) that have occurred */
 
    union _parser{
 
        t_QeIwExceptionParseLookup  parseLookup;    /**< Exceptions for parse and lookup events */
        t_QeIwExceptionIphc         iphc;           /**< Exceptions for IPHC events */
    } parser;
 
}t_QeIwExceptionParser;


/**
 @Description    Exception-specific data for Parser and IW Exceptions
*/
typedef struct {

    e_QeIwException exception;              /**< Exception type. Allows differentiating between IW and Parser interrupts */
 
    union _data{

        t_QeIwExceptionInterworking     iw; /**< Interworking exception. */
        t_QeIwExceptionParser           pl; /**< Parser exception. */

    } data;

} t_QeIwException;


/**
 @Description   Type definition for user callback function called by the driver for reporting exceptions.

 @Param         h_AppId        (In)    The AppId of the dev object that caused the exception,
 @Param         p_Exception    (In)    Pointer to a structure containing detailed exception information.
 
 @Return        N/A

 @Cautions      The user may not retain the pointer passed in p_Exception after returning
                control to the driver. The pointer will become invalid after the callback.
*/
typedef void (t_QeIwDevExceptions) (t_Handle h_AppId, t_QeIwException *p_Exception);


/**
 @Description    User Parameters for configuring a Device for Interworking
*/
typedef struct {

    t_Handle                h_AppId;                /**< Handle to the upper layer */
    e_QeIwDeviceType        devType;                /**< The type of the Termination Device (ATM or Ethernet) */
    t_Handle                h_DevTerm;              /**< Handle to the Termination Device (ATM, Ethernet, PPP and VP instance #0) */
    t_Handle                h_Common;               /**< Handle to the Interworking Common object */
    t_QeIwDevExceptions    *f_IwException;          /**< User-provided callback for interworking exception reporting */
    t_QeIwDevExceptions    *f_IwPlException;        /**< User-provided callback for interworking parse and lookup exception reporting */
    uint16_t                numIntConnDescriptors;  /**< Expected number of internal Connection Descriptors for the device. Must be in range [0..254]. In case IP REASSEMBLY is requested for this device, the range is [0..251] */
    uint16_t                numExtConnDescriptors;  /**< Expected number of external Connection Descriptors for the device. Must be in range [0..65280]. In case IP REASSEMBLY is requested for this device, the range is [0..65277] */
} t_QeIwDeviceParam;


/**
 @Function      QE_IW_DEV_Config

 @Description   Creates (and configures its default parameters) an interworking-specific object
                corresponding to a Termination Device that is to be used in Interworking.

 @Param         p_Params    - (In)    Pointer to a structure containing user-provided parameters.
 
 @Return        Handle to the newly-created device.
 @Return        NULL in case of failure to create object or invalid parameters.
*/
t_Handle    QE_IW_DEV_Config (t_QeIwDeviceParam * p_Params);


/**
 @Function      QE_IW_DEV_Init

 @Description   This function initializes driver and hardware for IW.

 @Param         h_Device    - (In)  Handle to the Interworking Device object.
 
 @Return        E_OK in case of    success. Error code otherwise.

 @Cautions      h_Device must have been previously created using QE_IW_DEV_Config().
*/
t_Error    QE_IW_DEV_Init (t_Handle h_Device);


/**
 @Function      QE_IW_DEV_Free

 @Description   Frees the resources associated to the Dev object.
                The rule is that only resources allocated during previous calls
                to QE_IW_DEV_Config are freed.
 
 @Param         h_Dev   - (In)  Handle to the IW DEV object as returned by QE_IW_DEV_Config function.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Any other resources that may be referenced from the IW DEV object and that
                were allocated using other APIs must be freed using the same APIs that allocated them.
*/
t_Error     QE_IW_DEV_Free (t_Handle h_Dev);


/**************************************************************************//**
 @Group         iw_dev_init_advanced_grp IW DEV Object Advanced Initialization Unit

 @Description   IW DEV Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Identifiers for  IP TOS - Drop Preference association mask
*/
typedef enum {

      e_QE_IW_IP_TOS_TO_COLOR_MASK_0        = 0     /**< The identifier represents the 0x03 mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_1        = 1     /**< The identifier represents the 0x06 mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_2        = 2     /**< The identifier represents the 0x0c mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_3        = 3     /**< The identifier represents the 0x18 mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_4        = 4     /**< The identifier represents the 0x30 mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_5        = 5     /**< The identifier represents the 0x60 mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_6        = 6     /**< The identifier represents the 0xc0 mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_GREEN    = 16    /**< The identifier represents green drop prefference mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_YELLOW   = 17    /**< The identifier represents yellow drop prefference mask */
    , e_QE_IW_IP_TOS_TO_COLOR_MASK_RED      = 18    /**< The identifier represents the red drop prefference mask */
 
} e_QeIwTOSToColorMask;


/**
 @Description    Identifiers for Drop Preference - IP TOS association mask
*/
typedef enum {

      e_QE_IW_COLOR_TO_IP_TOS_MASK_0        = 0     /**< The identifier represents the 0x03 mask */
    , e_QE_IW_COLOR_TO_IP_TOS_MASK_1        = 1     /**< The identifier represents the 0x06 mask */
    , e_QE_IW_COLOR_TO_IP_TOS_MASK_2        = 2     /**< The identifier represents the 0x0c mask */
    , e_QE_IW_COLOR_TO_IP_TOS_MASK_3        = 3     /**< The identifier represents the 0x18 mask */
    , e_QE_IW_COLOR_TO_IP_TOS_MASK_4        = 4     /**< The identifier represents the 0x30 mask */
    , e_QE_IW_COLOR_TO_IP_TOS_MASK_5        = 5     /**< The identifier represents the 0x60 mask */
    , e_QE_IW_COLOR_TO_IP_TOS_MASK_6        = 6     /**< The identifier represents the 0xc0 mask */
 
} e_QeIwColorToIpToSMask;


/**
 @Description    Combination of flags indicating discard / forward to CPU on certain frames
*/
typedef enum {

      e_QE_IW_DEBUG_MODE_ERROR_FRAME_DISCARD        = 0             /**< Discard Frames */
    , e_QE_IW_DEBUG_MODE_ERROR_FRAME_FORWARD_TO_CPU = 0x80000000    /**< Forward Error Frames to CPU */
    , e_QE_IW_DEBUG_MODE_FRAME_FORWARD_TO_CPU       = 0x40000000    /**< Forward All Frames to CPU */
 
} e_QeIwDebugMode;


/**
 @Description    Flags indicating interworking exception events
*/
typedef enum {

      e_QE_IW_EXCEPTION_EVENT_NONE              = 0             /**< No exception */
    , e_QE_IW_EXCEPTION_EVENT_CPU_QUEUE_NO_RX   = 0x00000040    /**< No Event on Rx CPU Queue  */
    , e_QE_IW_EXCEPTION_EVENT_FRAME_ERROR       = 0x00000020    /**< Frame Error Event */
    , e_QE_IW_EXCEPTION_EVENT_ENQUEUE_BUSY      = 0x00000010    /**< Enqueue Busy Event */
    , e_QE_IW_EXCEPTION_EVENT_CPU_QUEUE_BUSY    = 0x00000008    /**< CPU Queue Busy Event */
    , e_QE_IW_EXCEPTION_EVENT_TX_OVERWRITTEN    = 0x00000004    /**< Multicast Transmitter Overflow Event */
    , e_QE_IW_EXCEPTION_EVENT_RX_OVERWRITTEN    = 0x00000002    /**< Multicast Receiver Overflow Event */

} e_QeIwExceptionSelect;

/**
 @Description  PPP Filter flags
*/
typedef enum
{
      e_QE_IW_PPP_NO_FLAG               = 0x00  /**< No flags are used for this filter */
    , e_QE_IW_PPP_TUNNELED_PID          = 0x01  /**< Specify that the PPP PID is encapsulated over L2TP */
    , e_QE_IW_PPP_SUBFRAME              = 0x01  /**< Specify that the PPP PID is one of a Subframe */
    , e_QE_IW_PPP_IPHC_FULL_HEADER      = 0x02  /**< Specify that the coresponding PPP PID is a IPHC FULL HEADER (RFC 2507) */
    , e_QE_IW_PPP_IPHC_DECOMPRESS       = 0x04  /**< Specify to enable IPHC Decompression mechanism */

} e_QeIwPppFilterFlags;


/**
 @Description    PPP Params to associate a Filter with a PID
*/
typedef struct {
    t_Handle    h_Filter;       /**< Handle to Filter */
    uint16_t    pid;            /**< PID to be associated with the Filter */
    uint8_t     filterFlags;    /**< Filter flags - use combinations from e_QeIwPppFilterFlags */
 
}t_QeIwPppFilterTableParam;

/**
 @Function      QE_IW_DEV_ConfigMinFrameLength

 @Description   Configures the minimum ATM frame length in bytes.
                [0]
                If the receiver detects an incoming frame shorter than this value
                it discards that frame unless the Frame Error Event was enabled.

 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         minFrameLength  - (In)    Requested minimum frame length.
                                          Must be in range [0..65535]
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error        QE_IW_DEV_ConfigMinFrameLength (t_Handle h_Device, uint16_t minFrameLength);


/**
 @Function      QE_IW_DEV_ConfigMaxFrameLength

 @Description   Configures the maximum ATM frame length in bytes.
                [0xFFFF]
                If the receiver detects an incoming frame longer than this value
                it discards that frame unless the Frame Error Event was enabled.

 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         maxFrameLength  - (In)    Requested maximum frame length.
                                        Must be in range [0..65535]
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error        QE_IW_DEV_ConfigMaxFrameLength (t_Handle h_Device, uint16_t maxFrameLength);


/**
 @Function      QE_IW_DEV_ConfigMinIpTTLValue

 @Description   Configures the minimum IP TTL value
                [0]
                If the TTL in the incoming frame is < MinTTL the parser considers this frame as a parse miss

 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         minIpTTLValue   - (In)    Requested minimum TTL value.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigMinIpTTLValue (t_Handle h_Device, uint8_t minIpTTLValue);


/**
 @Function      QE_IW_DEV_ConfigMaxIpTTLValue

 @Description   Configures the maximum IP TTL value
                [255]
                If the TTL in the incoming frame is > MaxTTL-1 the parser considers this frame as a parse miss

 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         maxIpTTLValue   - (In)    Requested maximum TTL value.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigMaxIpTTLValue (t_Handle h_Device, uint8_t maxIpTTLValue);


/**
 @Function      QE_IW_DEV_ConfigColorInsertionMode

 @Description   Configures the color insertion mode for the policer (clears the IP TOS bits)
                [FALSE]

 @Param         h_Device    - (In)    Handle to the Interworking Device object,
 @Param         clearIpTos  - (In)    The color insertion mode
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigColorInsertionMode (t_Handle h_Device, bool clearIpTos);


/**
 @Function      QE_IW_DEV_ConfigIpTosToColorMask

 @Description   Configures the mask value used for insertion of drop preference (color) to IP TOS (or DSCP) field.
                Drop preference is the output of the policer.
                [e_QE_IW_IP_TOS_TO_COLOR_MASK_GREEN]

 @Param         h_Device    - (In)    Handle to the Interworking Device object,
 @Param         mask        - (In)    The value of the desired mask
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigIpTosToColorMask (t_Handle h_Device, e_QeIwTOSToColorMask mask);


/**
 @Function      QE_IW_DEV_ConfigColorToIpToSMask

 @Description   Mask value applied to IP TOS (or DSCP) field to extract the drop preference (color) bits.
                The extracted drop preference is used as input to the policer.
                [e_QE_IW_COLOR_TO_IP_TOS_MASK_3]

 @Param         h_Device    - (In)    Handle to the Interworking Device object,
 @Param         mask        - (In)    The value of the desired mask
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigColorToIpToSMask (t_Handle h_Device, e_QeIwColorToIpToSMask mask);


/**
 @Function      QE_IW_DEV_ConfigFwdCpuMaxDataLen

 @Description   Configures the length of the frames that are sent to CPU.
                [0x80]
                The interworking function could forwards frames of size up to
                maxDataLen to the CPU queue.

 @Param         h_Device    - (In)    Handle to the Interworking Device object,
 @Param         maxDataLen  - (In)    The length of the frames that are sent to CPU.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error        QE_IW_DEV_ConfigFwdCpuMaxDataLen (t_Handle h_Device, uint16_t maxDataLen);


/**
 @Function      QE_IW_DEV_ConfigFwdCpuErrMaxDataLen

 @Description   Configures the length of error frames that are sent to CPU.
                [64]
                The interworking function could forwards error frames of size up to
                maxDataLen to the CPU queue.

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         maxDataLen  - (In)    The length of error frames that are sent to CPU.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error        QE_IW_DEV_ConfigFwdCpuErrMaxDataLen (t_Handle h_Device, uint16_t maxDataLen);


/**
 @Function      QE_IW_DEV_ConfigFwdCpuMaxBdNum

 @Description   Configures the number of Buffer Descriptors that are used for frames
                that are sent to CPU.The interworking functions forwards maxBdNunm
                number of BDs to the CPU queue.
                [1]
 
 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         maxBdNunm   - (In)    Number of BDs that are used for frames sent to CPU.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error        QE_IW_DEV_ConfigFwdCpuMaxBdNum (t_Handle h_Device, uint8_t maxBdNunm);


/**
 @Function      QE_IW_DEV_ConfigFrameDepth
 
 @Description   Configures a new frame depth used by parse, lookup and header manipulations mechanisms.
                [0x7d]
 
 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         frameDepth  - (In)    The new value for the maximum depth for the parse, lookup and header manipulations.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigFrameDepth (t_Handle h_Device, uint8_t frameDepth);

#ifndef QE_IW_IPSEC_UNAVAILABLE
/**
 @Function      QE_IW_DEV_ConfigIpsecSupport
 
 @Description   Configures support of IPSec IW , when the device is over VP this mode can be used.
 
 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         h_Ipsec     - (In)    handle to the ipsec object used by the device.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error	QE_IW_DEV_ConfigIpsecSupport(t_Handle h_Device,t_Handle h_Ipsec);
#endif

/**
 @Function      QE_IW_DEV_ConfigDebugMode
 
 @Description   Configures which Frames should also be sent to CPU for debug purposes.
                 [e_QE_IW_DEBUG_MODE_ERROR_FRAME_DISCARD]
 
 @Param         h_Device    - (In)   Handle to the Interworking Device object.
 @Param         debugMode   - (In)   The new value for the Debug Mode settings.
                                     This value must be a combination of e_QeIwDebugMode values.

 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Use a combination of the values defined in ::e_QeIwDebugMode.
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigDebugMode (t_Handle h_Device, uint32_t debugMode);


/**
 @Function      QE_IW_DEV_ConfigExceptionEvents
 
 @Description   Masks/unmasks the generation of interrupts in case of exception events
                described in e_QeIwExceptionSelect. By default CPU queue RX interupt is enabled.
                [e_QE_IW_EXCEPTION_EVENT_NONE]
 
 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         flags       - (In)  Combination of exception events defined in e_QeIwExceptionSelect.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error QE_IW_DEV_ConfigExceptionEvents ( t_Handle h_Device, uint32_t flags );


/**
 @Function      QE_IW_DEV_ConfigTypeOrLenThreshold
 
 @Description   Configures the Ethernet Length/Type field for the incoming frames to
                be compared against. Useful for distinction between Ethernet and 802.3.
                [0x600]
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object.
 @Param         etherLenType    - (In)    The new expected EtherLen/Type field of incoming frames.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error        QE_IW_DEV_ConfigTypeOrLenThreshold (t_Handle h_Device, uint16_t etherLenType);


/**
 @Function      QE_IW_DEV_ConfigQTagTypes
 
 @Description   Changes the VLAN QTag Types from the defaults.
                QTag Type fields to be compared to the type field of stacked VLAN tags.
                [0x8100]
 
 @Param         h_Device    - (In)  Handle to the Interworking Device object,
 @Param         qTagIndex   - (In)  Index in QTagType table.
 @Param         qTagType    - (In)  QTag type.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigQTagTypes (t_Handle h_Device, uint8_t qTagIndex, uint16_t qTagType);
 

/**
 @Function      QE_IW_DEV_ConfigDefaultVlanPriority
 
 @Description   Changes the value of the VLAN Priority field to insert in outgoing
                VLAN-tagged frames in the absence of another indication (IP TOS for instance).
                [0x00]
 
 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         newVpri     - (In)    The new value for the VLAN priority.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigDefaultVlanPriority (t_Handle h_Device, uint8_t newVpri);


/**
 @Function      QE_IW_DEV_ConfigExtConnDescriptorMemory

 @Description   Configures the type of the external memory where the Connection Descriptors table will reside.
                [0, MEMORY_ATTR_CACHEABLE]

 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         memId           - (In)    Memory partition ID.
 @Param         memAttributes   - (In)    Memory attributes mask.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtConnDescriptorMemory(t_Handle h_Device, uint8_t memId, uint32_t memAttributes);


/**
 @Function      QE_IW_DEV_ConfigExtPolicerMemory

 @Description   Configures the type of the external memory where the policer profile table will reside.
                [0, MEMORY_ATTR_CACHEABLE]
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         memId           - (In)    Memory partition ID.
 @Param         memAttributes   - (In)    Memory attributes mask.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtPolicerMemory(t_Handle h_Device, uint8_t memId, uint32_t memAttributes);


/**
 @Function      QE_IW_DEV_ConfigExtStatisticsMemory

 @Description   Configures the type of the external memory where the statistics counters will reside.
                [0, MEMORY_ATTR_CACHEABLE]
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         memId           - (In)    Memory partition ID.
 @Param         memAttributes   - (In)    Memory attributes mask.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtStatisticsMemory(t_Handle h_Device, uint8_t memId, uint32_t memAttributes);


/**
 @Function      QE_IW_DEV_ConfigExtHeaderManipulationMemory

 @Description   Configures the type of the external memory where the buffers
                used by header manipulation operations will reside.
                [0, MEMORY_ATTR_CACHEABLE]
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         memId           - (In)    Memory partition ID.
 @Param         memAttributes   - (In)    Memory attributes mask.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtHeaderManipulationMemory(t_Handle h_Device, uint8_t memId, uint32_t memAttributes);

/**
 @Function      QE_IW_DEV_ConfigIntHeaderManipulationPool

 @Description   Configures an header manipulation pool in internal memory. You will
                need this pool if you are working with internal header manipulations
                which are using data buffers (operations such as "insert" or "replace").
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         numOfBuffers    - (In)    Number of buffers to reserve in this pool.
 @Param         bufferSize      - (In)    The size of the buffers in the pool.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigIntHeaderManipulationPool(t_Handle h_Device, uint32_t numOfBuffers, uint16_t bufferSize);

/**
 @Function      QE_IW_DEV_ConfigExtHeaderManipulationPool

 @Description   Configures an header manipulation pool in external memory. You will
                need this pool if you are working with external header manipulations
                which are using data buffers (operations such as "insert" or "replace").
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object,
 @Param         numOfBuffers    - (In)    Number of buffers to reserve in this pool.
 @Param         bufferSize      - (In)    The size of the buffers in the pool.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtHeaderManipulationPool(t_Handle h_Device, uint32_t numOfBuffers, uint16_t bufferSize);

/**
 @Function      QE_IW_DEV_ConfigNumIntStatsCounters
 
 @Description   Changes the number of internal Statistics Counters from its default value.
                [For simple statistics counters: 0,
                For extended statistics counters: 0]
 
 @Param         h_Device            - (In)  Handle to the Interworking Device object.
 @Param         extendedCounters    - (In)  Flag indicating whether the function should allocated space
                                            for extended or single statistics counters.
 @Param         numIntStatsCounters - (In)  The new value of the number of internal Statistics Counters.
                                            Simple Statistics Counters must be in range [0..239]
                                            Extended Statistics Counters must be in range [0..255]
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error QE_IW_DEV_ConfigNumIntStatsCounters (t_Handle h_Device, bool extendedCounters, uint16_t numIntStatsCounters);


/**
 @Function      QE_IW_DEV_ConfigNumExtStatsCounters
 
 @Description   Changes the number of external Statistics Counters from its default value.
                [For simple statistics counters: 0,
                For extended statistics counters: 0]
 
 @Param         h_Device            - (In)  Handle to the Interworking Device object,
 @Param         extendedCounters    - (In)  Flag indicating whether the function should allocated space
                                            for extended or single statistics counters.
 @Param         numExtStatsCounters - (In)  The new value of the number of external Statistics Counters.
                                            Simple Statistics Counters must be in range [0..65535]
                                            Extended Statistics Counters must be in range [0..65535]
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error QE_IW_DEV_ConfigNumExtStatsCounters (t_Handle h_Device, bool extendedCounters, uint16_t numExtStatsCounters);
 

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Function      QE_IW_DEV_ConfigNumIntIpReassStatsCounters
 
 @Description   Changes the number of internal IP Reassembly Statistics Counters from its default value.
                [IP Reassembly statistics counters: 0]
 
 @Param         h_Device            - (In)  Handle to the Interworking Device object.
 @Param         numIntIpReassStatsCounters - (In)  The new value of the number of internal IP Reassembly Statistics Counters.
                        IP Reassembly Internal Statistics Counters must be in range [0..255]
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init(). Only for FULL_IPREASS type.
*/
t_Error	QE_IW_DEV_ConfigNumIntIpReassStatsCounters (t_Handle h_Device,
                                                    uint16_t numIntIpReassStatsCounters);

/**
 @Function      QE_IW_DEV_ConfigNumExtIpReassStatsCounters
 
 @Description   Changes the number of external IP Reassembly Statistics Counters from its default value.
                [IP Reassembly statistics counters: 0]
 
 @Param         h_Device            - (In)  Handle to the Interworking Device object.
 @Param         numExtIpReassStatsCounters - (In)  The new value of the number of external IP Reassembly Statistics Counters.
                        IP Reassembly External Statistics Counters must be in range [0..65535]
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init(). Only for FULL_IPREASS type.
*/
t_Error	QE_IW_DEV_ConfigNumExtIpReassStatsCounters (t_Handle h_Device,
                                                    uint16_t numExtIpReassStatsCounters);

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Function      QE_IW_DEV_ConfigAtmFilter

 @Description   Configures a Filter that will be used by ATM Rx channels in IW mode.
                This function should be called to set all the filters that will be used
                by ATM Rx channels.

 @Param         h_Device    -  (In)    Handle to the Interworking Device object,
 @Param         h_Filter    -  (In)    Handle to the Filter object (ATM type).
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error QE_IW_DEV_ConfigAtmFilter (t_Handle h_Device, t_Handle h_Filter);

#if (!defined(QE_IW_PPPOHT_UNAVAILABLE))
/**
 @Function      QE_IW_DEV_ConfigPppFilter

 @Description   Configures a Filter that will be used by PPP Rx in IW mode.
                This function should be called to set all the filters that will be used
                by PPPoHT Rx associated with PIDs.

 @Param         h_Device           - (In)    Handle to the Interworking Device object,
 @Param         tableIndex         - (In)    Index of the table in which the filter is inserted
 @Param         p_PppFltrTblParams - (In)    Adress of the Filter Table params structure
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigPppFilter ( t_Handle h_Device,
                                    uint8_t  tableIndex,
                                    t_QeIwPppFilterTableParam *p_PppFltrTblParams);


/**
 @Function      QE_IW_DEV_ConfigPppDefaultFilter

 @Description   Configures a Default Filter that will be used by PPP Rx in IW mode
                in case of no PID match in filter table
 
 @Param         h_Device      - (In)    Handle to the Interworking Device object,
 @Param         h_Filter      - (In)    Handle to the Filter object
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config(), and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigPppDefaultFilter (t_Handle h_Device, t_Handle h_Filter);
#endif /* !QE_IW_PPPOHT_UNAVAILABLE */

#if (!defined(QE_IW_IPHC_UNAVAILABLE))
/**
 @Function      QE_IW_DEV_ConfigIphcMaxHeaderSize
                [28]

 @Description   Configure the maximum header size for the Header decompression
 

 @Param         h_Device            - (In)  Handle to the Interworking Device object.
 @Param         iphcMaxHeaderSize   - (In)  Maximum header size for the Header decompression.
                                            This parameter must be the actual header being
                                            decompressed by the QE.
 
 @Return        E_OK if    success, Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigIphcMaxHeaderSize (t_Handle h_Device, uint8_t iphcMaxHeaderSize );
#endif /* !QE_IW_IPHC_UNAVAILABLE */

/**
 @Function      QE_IW_DEV_ConfigExtParseLookupMemory

 @Description   Configures the type of external memory where will reside all the lookup tables
                used in parse and lookup operations for this device.
                [0, MEMORY_ATTR_CACHEABLE]
 
 @Param         h_Device        - (In)    Handle to the Interworking Device object.
 @Param         memId           - (In)    Memory partition ID.
 @Param         memAttributes   - (In)    Memory attributes mask.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtParseLookupMemory(t_Handle h_Device, uint8_t memId, uint32_t memAttributes);


/**
 @Function      QE_IW_DEV_ConfigIpTosToVlanVPriTable

 @Description   Configures the VLAN Priorities to be inserted in outgoing frames
                based on the values of the IP TOS field in the incoming IP-containing frames.
                [0x0].

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         ipTos       - (In)    IP TOS field value to which to associate the Vpri provided here.
 @Param         vPri        - (In)    Vpri to insert in the VLAN tags of frames with specified IP TOS.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      This function must be used only on ATM side. On GETH side, the
                 IP TOS to VLAN VPri Table is configured through UCC_GETH_ConfigIpPriorityTable function.
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigIpTosToVlanVPriTable (t_Handle h_Device, uint8_t ipTos, uint8_t    vPri);


/**
 @Function      QE_IW_DEV_ConfigIwInterruptQueue

 @Description   Configures the Interworking Interrupt Queue for this Device.
                [For number of entries: 100,
                For initial counter: 1]
 
 @Param         h_Device                - (In)   Handle to the Interworking Device object,
 @Param         numIntrQueueEntries     - (In)   The total number of entries in the Interworking Interrupt Queue,
                                                 Must be in range [1..8192]
 @Param         interruptThreshold      - (In)   The number of interrupt queue entries required to be filled before
                                                 the CP issues a global interrupt.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigIwInterruptQueue (t_Handle    h_Device,
                                          uint32_t    numIntrQueueEntries,
                                          uint16_t    interruptThreshold);


/**
 @Function      QE_IW_DEV_ConfigParseLookupInterruptQueue

 @Description   Configures the Parse and Lookup Interrupt Queue for this Device.
                [For number of entries: 100,
                For initial counter: 1]

 @Param         h_Device                - (In)  Handle to the Interworking Device object,
 @Param         numIntrQueueEntries     - (In)  The total number of entries in the Parse and Lookup Interrupt Queue,
                                                Must be in range [1..2048]
 @Param         interruptThreshold      - (In)  The number of interrupt queue entries required to be filled before
                                                the CP issues a global interrupt.
 
 @Return        E_OK if    success, Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigParseLookupInterruptQueue (t_Handle    h_Device,
                                                   uint32_t    numIntrQueueEntries,
                                                   uint16_t    interruptThreshold);


/**
 @Function      QE_IW_DEV_ConfigQueueManagersNum

 @Description   Configure the number of Queue Manager objects.
                [0]

 @Param         h_Device    - (In)  Handle to the Interworking Device object.
 @Param         qmNum       - (In)  The total number of Queue Manager objects.
                                    Must be in range [0..16]
 
 @Return        E_OK if    success; Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
 @Cautions      Available for e_QE_IW_DEV_TYPE_ETH Device type.
*/
t_Error    QE_IW_DEV_ConfigQueueManagersNum (t_Handle h_Device, uint8_t qmNum);


/**
 @Function      QE_IW_DEV_ConfigIntPolicersNum

 @Description   Configure the number of Policer objects in internal memory
                [0]

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         intPlcrNum  - (In)    The maximum number of policer objects in internal memory
 
 @Return        E_OK if    success, Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigIntPolicersNum (t_Handle h_Device, uint16_t intPlcrNum);


/**
 @Function      QE_IW_DEV_ConfigExtPolicersNum

 @Description   Configure the number of Policer objects in external memory
                [0]

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         extPlcrNum  - (In)    The maximum number of policer objects in external memory
                                      Must be in range [0..65280]
 @Return        E_OK if    success, Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ConfigExtPolicersNum (t_Handle h_Device, uint16_t extPlcrNum);

/**
 @Function      QE_IW_DEV_ConfigEthRedundantConnection

 @Description   Configure a Redundant Connection for this Dev.
                [No Redundant Connection]
 
 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         uccDest     - (In)    The UCC destination to be used as Redundant Connection.
 
 @Return        E_OK if    success; Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
 @Cautions      Available for e_QE_IW_DEV_TYPE_ETH Device type.
*/
t_Error    QE_IW_DEV_ConfigEthRedundantConnection (t_Handle h_Device, e_Ucc uccDest);


#if (!defined(QE_IW_IPREASS_CUSTOM_UNAVAILABLE) || !defined(QE_IW_IPREASS_FULL_UNAVAILABLE))

/**
 @Function      QE_IW_DEV_ConfigIpReass

 @Description   Configures IP Reassembly functionality for this device.
 
 @Param         h_Device            - (In) Handle to the Interworking Device object.
 @Param         h_IpReass           - (In) Handle to the IP Reassembly object.
 @Param         isInternal          - (In) Indication of the header manipulations objects, internally
                                           used by the IPREASS object
 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only after QE_IW_DEV_Config() and before QE_IW_DEV_Init().
 @Cautions      Available only for Ethernet receiver.
*/
t_Error QE_IW_DEV_ConfigIpReassSupport ( t_Handle h_Device, t_Handle h_IpReass, bool isInternal );
#endif  /* not QE_IW_IPREASS_CUSTOM_UNAVAILABLE || not QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Function      QE_IW_DEV_ConfigVlan1PriorityQueueSelectionTable

 @Description   Configures the Logical Port and Logical Queue on which the frames will be sent
                based on the values of the VPri of the first VLAN field in the incoming frames.
                [0x0].

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         vPri        - (In)    VLAN 1 VPri field value for which the provided destination queue will be selected.
 @Param         queueId     - (In)    Destination Queue for frames with specified VLAN 1 VPri.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
 @Cautions      Available only for Ethernet, PPP and VP receiver.
*/
t_Error    QE_IW_DEV_ConfigVlan1PriorityQueueSelectionTable (t_Handle h_Device, uint8_t vPri, uint8_t    queueId);


/**
 @Function      QE_IW_DEV_ConfigVlan2PriorityQueueSelectionTable

 @Description   Configures the Logical Port and Logical Queue on which the frames will be sent
                based on the values of the VPri of the second VLAN field in the incoming frames.
                [0x0].

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         vPri        - (In)    VLAN 2 VPri field value for which the provided destination queue will be selected.
 @Param         queueId     - (In)    Destination Queue for frames with specified VLAN 2 VPri.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
 @Cautions      Available only for Ethernet, PPP and VP receiver.
*/
t_Error    QE_IW_DEV_ConfigVlan2PriorityQueueSelectionTable (t_Handle h_Device, uint8_t vPri, uint8_t    queueId);


/**
 @Function      QE_IW_DEV_ConfigIpPriorityQueueSelectionTable

 @Description   Configures the Logical Port and Logical Queue on which the frames will be sent
                based on the values of the IP TOS field in the incoming frames.
                [0x0].

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         ipTos       - (In)    IP TOS field value for which the provided destination queue will be selected.
 @Param         queueId     - (In)    Destination Queue for frames with specified IP TOS.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Config() and before QE_IW_DEV_Init().
 @Cautions      Available only for Ethernet, PPP and VP receiver.
*/
t_Error    QE_IW_DEV_ConfigIpPriorityQueueSelectionTable (t_Handle h_Device, uint8_t ipTos, uint8_t    queueId);

/** @} */ /* end of iw_dev_init_advanced_grp group */
/** @} */ /* end of iw_dev_init_grp group */


/**************************************************************************//**
 @Group         iw_dev_control_grp IW DEV Object Control Unit

 @Description   IW DEV Object API Control Unit

 @{
*//***************************************************************************/

/**
@Description   Device Special Statistics
*/
typedef struct {

    uint16_t    rxBusy_Cnt;                 /**< For ATM and Ethernet receiver this counter is incremented if a frame is not received due to receive buffer busy condition */
    uint16_t    iwEnqueueBusy_Cnt;          /**< For ATM and Ethernet receiver this counter is incremented if a frame is not placed in the Transmit queue */
    uint16_t    iwCpuEnqueueBusy_Cnt;       /**< For ATM and Ethernet receiver this counter is incremented if a frame is not placed in the CPU queue by the IW function due to buffer busy condition (ATM and Ethernet) */
    uint16_t    iwMcTxSlow_Cnt;             /**< For ATM multicast transmitter this counter is incremented if an overrun condition is detected in the multicast transmitter buffer signalling that the multicast receiver has overridden the frame to be transmited (slow Tx) */
    uint16_t    iwMcRxOver_Cnt;             /**< For Ethernet receiver to ATM transmitter this counter is incremented if an overrun condition is detected in the multicast receiver buffer signalling that the multicast receiver has overridden the frame to be transmitted (slow or stuck Tx) */
    uint16_t    iwMcEnqBusy_Cnt;            /**< For Ethernet receiver to ATM multicast transmitter this counter is incremented if a busy condition is detected in MCTxBD ring. In this case the frame is discarded and this counter is incremented. */
    uint16_t    iwUnrecognizedFrame_Cnt;    /**< For Ethernet and ATM receiver this counter is incremented if the automatic parser has detected an unrecognized frame */
    uint16_t    iwFrameRej_Cnt;             /**< For Ethernet and ATM receiver this counter is incremented if the frame is rejected due to the rejectFrame configured for Connection Debug Mode (use function QE_IW_CONN_ConfigDebugMode) */
    uint16_t    iwErrorCondition_Cnt;       /**< For ATM receiver this counter is incremented if a ABRT or LNE or CRE (for AAL5) or TE-Ras_Cnt or US_Cnt or OS_Cnt (for AAL2) error condition is detected at the reception of the frame; For Ethernet receiver this counter is incremented if a OV error condition is detected at the reception of the frame; For PPP receiver this counter indicates the number of MRRU errors encountered */
    uint16_t    iwPppDemuxBusy_Cnt;         /**< For PPP receiver this counter is incremented if a frame is dropped due to unavailable Demux Fragment Table (DFT) entries */
    uint16_t    iwGeneralMiss_Cnt;          /**< For ATM, Ethernet and PPP receivers this counter is incremented if a ParserLookup General Miss or Passthtough Fitler is encountered */
    uint16_t    iwTTLDrop_Cnt;              /**< For ATM, Ethernet and PPP receivers this counter is incremented if the IP TTL field in the frame is not in the [MinTTL, MaxTTL] interval as programmed at the DEV level */
    uint16_t    iwIPChecksumErr_Cnt;        /**< This counter is incremented when the IP Checksum value in the IP header of the incoming frame does not match the IP checksum calculated on the same header. */
    uint16_t    iwPppFbpBusy_Cnt;           /**< This counter is incremented when interworking to MLMC PPP transmitter. The Ethenet or the ATM doesnt have a valid buffer in the pool to be taken back to the receiver BD ring. The frame is dropped in this case and an interrupt in issued. */
    uint16_t    threadBusyCnt;              /**< This counter is incremented when a 'thread-busy' condition occurs for the IW available threads*/

} t_QeIwDevSpecialStatistics;


/**
 @Function      QE_IW_DEV_ModifyEthRedundantConnection

 @Description   Modifies a Redundant Connection for this Dev.

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         uccDest     - (In)    The new UCC destination to be used as Redundant Connection.
 
 @Return        E_OK if    success; Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_ModifyEthRedundantConnection (t_Handle    h_Device,
                                                e_Ucc       uccDest);

/**
 @Function      QE_IW_DEV_GetAtmFilterIndex

 @Description   Retrives the index and the fast pass mode to the Filter that will be linked to the ATM Channel.

 @Param         h_Device        - (In)    Handle to the Interworking Device object.
 @Param         h_Filter        - (In)    Handle to the ATM Filter Object.
 @Param         p_FilterIndex   - (Out)   Index to the filter in the ATM Filter Table.
 @Param         p_FastPass      - (Out)   True, if the filter is configured for ATM fast pass mode.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_GetAtmFilterIndex (t_Handle    h_Device,
                                     t_Handle    h_Filter,
                                     uint8_t    *p_FilterIndex,
                                     bool        *p_FastPass );


/**
 @Function      QE_IW_DEV_GetSpecialStatistics

 @Description   Retrieves at runtime the special statistics gathered for this Connection Descriptor.

 @Param         h_Device    - (In)    Handle to the Interworking Device object.
 @Param         p_Stats     - (Out)   Pointer to user-supplied structure where to place the special statistics gathered.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_GetSpecialStatistics (t_Handle h_Device, t_QeIwDevSpecialStatistics *p_Stats);


/**
 @Function      QE_IW_DEV_ResetSpecialStatistics

 @Description   Clear the special statistics counters.
 
                The Reset of the special statistics counters is valid operation only if
                statistics gathering was configured as enabled prior to QE_IW_DEV_Init call.
 

 @Param         h_Device    - (In) Handle to the Interworking Device object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_DEV_Init().
*/
t_Error QE_IW_DEV_ResetSpecialStatistics (t_Handle h_Device);

/**
 @Function      QE_IW_DEV_GetIntStatsCounterIndex

 @Description   Get an available internal memory counter.
                Internal Memory counters were allocated using QE_IW_DEV_ConfigNumIntStatsCounters

 @Param         h_Device            - (In) Handle to the Interworking Device object.
 @Param         extendedCounters    - (In) Flag indicating whether the returned index will be
                                            for extended or single statistics counters.
 @Param         p_CounterIndex      - (Out) Address where the index will be returned.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_DEV_Init().
 
*/
t_Error    QE_IW_DEV_GetIntStatsCounterIndex (t_Handle h_Device, bool extendedCounters, uint16_t *p_CounterIndex);

/**
 @Function      QE_IW_DEV_GetExtStatsCounterIndex

 @Description   Get an available external memory counter.
                External Memory counters were allocated using QE_IW_DEV_ConfigNumExtStatsCounters

 @Param         h_Device    - (In) Handle to the Interworking Device object.
 @Param         extendedCounters    - (In) Flag indicating whether the returned index will be
                                            for extended or single statistics counters.
 @Param         p_CounterIndex      - (Out) Address where the index will be returned.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_DEV_Init().
*/
t_Error    QE_IW_DEV_GetExtStatsCounterIndex (t_Handle h_Device, bool extendedCounters, uint16_t *p_CounterIndex);


#if (DEBUG_ERRORS > 0)
/**
 @Function      QE_IW_DEV_DumpRegs

 @Description   Dumps QE IW DEV registers and parameter RAM.

 @Param         h_Device - (In) Handle to the Interworking Device object.
 
 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Device must have been previously created using QE_IW_DEV_Config()
                and initialized using QE_IW_DEV_Init().
*/
t_Error QE_IW_DEV_DumpRegs(t_Handle h_Device);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of iw_dev_control_grp group */
/** @} */ /* end of iw_dev_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_DEV_EXT_H */
