/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_IPREASS_EXT_H
#define __QE_IW_IPREASS_EXT_H


#if (!defined(QE_IW_IPREASS_CUSTOM_UNAVAILABLE) || !defined(QE_IW_IPREASS_FULL_UNAVAILABLE))

/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_ipReass_grp IW IP REASSEMBLY Object Application Programming Interface

 @Description    IW IP REASSEMBLY Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_ipReass_init_grp IW IP REASSEMBLY Object Initialization Unit

 @Description   IW IP REASSEMBLY Object API Initialization Unit

 @{
*//***************************************************************************/

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

#define QE_IPREASS_CUSTOM_EVENT_KEY_SIZE           15

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

#define QE_IPREASS_FULL_EVENT_KEY_SIZE             11

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Description   IP Reassembly Types
*/
typedef enum
{

      e_QE_IW_IPREASS_TYPE_CUSTOM         = 1,   /**< The IP Reassembly is custom */

      e_QE_IW_IPREASS_TYPE_FULL           = 2    /**< The IP Reassembly is full */

} e_QeIwIpReassType;

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
 @Description    Identifiers for IP Reassembly Custom events
*/
typedef enum
{
      e_QE_IW_IPREASS_EVENT_NONE                    = 0x0000    /**< Unknown event from an IP Reassembly instance */
    , e_QE_IW_IPREASS_EVENT_NUM_SIM_FLOWS_EXCEEDED  = 0x8000    /**< A new flow should be learned by the IP Reassembly process, but the total number of flows has reached its maximum (32). This interrupt is issued whenever the Dynamic Lookup table is full and the first fragment of an IP packet to be reassembled is received. The discarded frames and discarded fragments counters are incremented. If the timeout threshold is set to zero, the oldest flow will be discarded and replaced by the new flow. */
    , e_QE_IW_IPREASS_EVENT_TIME_OUT                = 0x4000    /**< In case the number of simultaneous flows is exceeded, the oldest flow is discarded only if its older than timeoutThreshold configured with QE_IW_DEV_ConfigIpReassemblyTimeoutThreshold() API function. The discarded frames counter is incremented. */
    , e_QE_IW_IPREASS_EVENT_BD_RINGS_POOL_BUSY      = 0x2000    /**< This event signals that an IP frame is dropped because no Queue Descriptor is available in the Free Queue Pool(not enough BD Rings were allocated). The discarded frames counter is incremented. */
    , e_QE_IW_IPREASS_EVENT_REQUEST_TABLE_BUSY      = 0x1000    /**< This event signals that an IP frame is dropped because the IP Reassembly Request Table is full. The discarded frames counter is incremented. */
    , e_QE_IW_IPREASS_EVENT_RX_OOO_FRAGMENT         = 0x0800    /**< This event signals that an IP fragment ( first or last only ) is received in the CPU queue under processing. */

} e_QeIwIpReassCustomEvent;

/**
 @Description    Parser and Lookup Ip Reassembly Custom Interrupt Queue Entry
*/
typedef struct
{

    e_QeIwIpReassCustomEvent code;     /**< The type of interrupt in this entry (defined in
                                            ::e_QeIwIpReassEvent) */
    uint8_t                  key[QE_IPREASS_CUSTOM_EVENT_KEY_SIZE]; /**< Generated Key of the
                                            fragment/regular frame that caused this interrupt */

} t_QeIwIpreassCustomEventEntry;


#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Description    Identifiers for IP Reassembly Full events
*/
typedef enum
{
      e_QE_IW_IPREASS_FULL_EVENT_NONE          = 0xFFFF  /**< Unknown event */
    , e_QE_IW_IPREASS_FULL_EVENT_NEW_BUCKET    = 0x0400  /**< New bucket from the IPR Bucket Pool
                                                              is taken */
    , e_QE_IW_IPREASS_FULL_EVENT_BUCKET_BUSY   = 0x0200  /**< New Bucket is needed but no available buckets are found. */
    , e_QE_IW_IPREASS_FULL_EVENT_TIMEOUT       = 0x0100  /**< TimeOut condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_FRAGMENT_BUSY = 0x0080  /**< Fragment Busy condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_BD_BUSY       = 0x0040  /**< BD Busy condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_FRAME_BUSY    = 0x0020  /**< Frame Busy condition occurs */
} e_QeIwIpReassFullEvent;

/**
 @Description    Identifiers for IP Reassembly Full CPU Swap events
*/
typedef enum
{
      e_QE_IW_IPREASS_FULL_EVENT_OVERLAP_FRAG               = 0x0000  /**< Overlap fragment condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_DUPLICATE_FRAG             = 0x0001  /**< Duplicate fragment condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_ERROR_FRAG                 = 0x0002  /**< Error fragment condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_SHORT_FRAG                 = 0x0003  /**< Short fragment condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_FRAME_WITH_OVERLAP_FRAG    = 0x0004  /**< Reassembled frame with overlap fragment condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_FRAME_WITH_DUPLICATE_FRAG  = 0x0005  /**< Reassembled frame with duplicate fragment condition occurs */
    , e_QE_IW_IPREASS_FULL_EVENT_FRAME_WITH_ERROR_FRAG      = 0x0006  /**< Reassembled frame with erroneous fragment condition occurs */
} e_QeIwCpuSwapIpReassFullEvent;

/**
 @Description    Identifiers for IP Reassembly Full Error Code for the frame received:
*/
typedef enum
{
      e_QE_IW_IPREASS_FULL_ERROR_CODE_OVERLAP_FRAG               = 0x0000  /**< Overlap fragment */
    , e_QE_IW_IPREASS_FULL_ERROR_CODE_DUPLICATE_FRAG             = 0x0001  /**< Duplicate fragment */
    , e_QE_IW_IPREASS_FULL_ERROR_CODE_ERROR_FRAG                 = 0x0002  /**< Erroneous fragment */
    , e_QE_IW_IPREASS_FULL_ERROR_CODE_SHORT_FRAG                 = 0x0003  /**< Short fragment */
    , e_QE_IW_IPREASS_FULL_ERROR_CODE_FRAME_WITH_OVERLAP_FRAG    = 0x0004  /**< Reassembled frame with overlap fragment */
    , e_QE_IW_IPREASS_FULL_ERROR_CODE_FRAME_WITH_DUPLICATE_FRAG  = 0x0005  /**< Reassembled frame with duplicate fragment */
    , e_QE_IW_IPREASS_FULL_ERROR_CODE_FRAME_WITH_ERROR_FRAG      = 0x0006  /**< Reassembled frame with erroneous fragment */
} e_QeIwCpuSwapIpReassFullErrorCode;

/**
 @Description    Identifiers for IP Reassembly Full events
*/
typedef enum
{
      e_QE_IW_IPREASS_FULL_TIMEOUT_ERR_FRAG         = 0x01  /**< Valid for e_QE_IW_IPREASS_FULL_EVENT_TIMEOUT event.
                                                                 Timeout condition with one of the following
                                                                 errors occured:
                                                                 - Last fragment received (MF=0) with (IP Total Length +
                                                                 Offset * 8) greater than 64KBytes.
                                                                 - First/Middle fragment received with size not a
                                                                 multiple of 8 Bytes.
                                                                 - Length of First/Middle is less than a user
                                                                 configurable size. */
    , e_QE_IW_IPREASS_FULL_TIMEOUT_OVERLAP_FRAG     = 0x02  /**< Valid for e_QE_IW_IPREASS_FULL_EVENT_TIMEOUT event.
                                                                 Timeout condition with Ovelap occured. */
    , e_QE_IW_IPREASS_FULL_TIMEOUT_DUPLICATE_FRAG   = 0x04  /**< Valid for e_QE_IW_IPREASS_FULL_EVENT_TIMEOUT event.
                                                                 Timeout condition with Duplication occured. */
    , e_QE_IW_IPREASS_FULL_BUCKET_BUSY_POOL_EMPTY   = 0x08  /**< Valid for e_QE_IW_IPREASS_FULL_EVENT_BUCKET_BUSY event.
                                                                 There is a demand for a new bucket, but there is no free
                                                                 bucket in the IPR Bucket Pool. In case of
                                                                 e_QE_IW_IPREASS_FULL_EVENT_BUCKET_BUSY without
                                                                 this flag set, it means that the maximum number of buckets for the entry was reached */

} e_QeIwIpReassFullEventFlags;

/**
 @Description    Parser and Lookup IP Reassembly Full Interrupt Queue Entry
*/
typedef struct
{

    uint32_t                parseFlags;     /**< Selection of flags indicating the detected
                                                 protocols and options.
                                                 Used e_QeIwExceptionParseLookupDetectedProtocols
                                                 values to make the identification.
                                                 Valid only for e_QeIwIpReassFullEvent interrupts */
    uint8_t                 userId;         /**< User-provided ID of the filter that caused the
                                                 exception; valid only for e_QeIwIpReassFullEvent interrupts */
    uint16_t                code;           /**< The type of interrupt in this entry (defined in
                                                 e_QeIwIpReassFullEvent or in e_QeIwCpuSwapIpReassFullEvent) */
    uint8_t                 key[QE_IPREASS_FULL_EVENT_KEY_SIZE]; /**< Generated Key of the
                                                  fragment/regular frame that caused this interrupt;
                                                  valid only for e_QeIwIpReassFullEvent interrupts */
    uint8_t                 bucketNum;
    uint8_t                 flags;          /**< Flags from e_QeIwIpReassFullEventFlags indicating various IPREASS events' type. */
} t_QeIwIpreassFullEventEntry;

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Description    Selection of flags identifying selected events. Custom/Full will be used, considering the selected e_QeIwIpReassType.
*/
typedef union t_QeIwIpReassEvent
{

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

    uint16_t        customFlags;    /**< Selection of e_QeIwIpReassCustomEvent. Used in case e_QE_IW_IPREASS_TYPE_CUSTOM was selected */

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

    uint16_t        fullFlags;      /**< Selection of e_QeIwIpReassFullEvent. Used in case e_QE_IW_IPREASS_TYPE_FULL was selected */

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

}t_QeIwIpReassEvent;

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
 @Description    Identifiers for IP Reassembly CPU Queue selection
*/
typedef enum
{
      e_QE_IW_IPREASS_CPU_QUEUE_0   = 0x01  /**< IP Reassembly CPU Queue Index 0: used to forward packets in case only forwarding to CPU is selected. */
    , e_QE_IW_IPREASS_CPU_QUEUE_1   = 0x02  /**< IP Reassembly CPU Queue Index 1: used to copy packets sent to a destination port */

}e_QeIwIpReassCpuQueue;


#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

/**
 @Description    Identifiers for IP Reassembly events
*/
typedef struct t_QeIwIpReassEventEntry
{
    e_QeIwIpReassType   type; /**< The type of the event */
 
    union
    {

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

        t_QeIwIpreassCustomEventEntry   customEntry;    /**< Valid when type is e_QE_IW_IPREASS_TYPE_CUSTOM */

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

        t_QeIwIpreassFullEventEntry     fullEntry;      /**< Valid when type is e_QE_IW_IPREASS_TYPE_FULL */

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

    };

}t_QeIwIpReassEventEntry;

/**
 @Description   Type definition for user callback function used by the driver for reporting exceptions.

 @Param         h_AppId - (In)  The AppId of the QE_IW_IPREASS Object that caused the exception,
 @Param         event   - (Out) IP Reassembly Events Identifier. Provides a t_QeIwIpReassEventEntry
 
 @Return        N/A
*/
typedef void (t_QeIwIpReassEventsCallback) (t_Handle h_AppId, uint32_t eventEntry);

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Description   User Parameters for configuring an IP Reassembly Full Object
*/
typedef struct
{
    uint16_t                        maxNumOfFrames;     /**< Maximum number of open frames waiting to be reassembled */
    uint8_t                         numOfVpThreads;     /**< The sum of threads for all Virtual Port instances that have IP Reassembly enabled */
    uint32_t                        compressKey;        /**< Mask for compression of lookup key of IPR hash table */
    uint32_t                        dataMemAttributes;  /**< Memory attributes mask for data buffers */

} t_QeIwIpReassFullParam;

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */
 
/**
@Description   IP Reassembly Initialization Structure
*/
typedef struct {

    e_QeIwIpReassType   type;                   /**< The type of IP Reassembly to create - used as selector for the union fields */
 
#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

    t_QeIwIpReassFullParam          full;       /**< Specific user params for the IP REASSEMBLY full (type = e_QE_IW_IPREASS_TYPE_FULL) */

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

    uint8_t                         pramMemId;
    t_Handle                        h_QeId;
    t_Handle                        h_AppId;            /**< Handle to the upper layer */
    t_BufferPoolInfo                bufferPoolInfo;     /**< Data buffer pool information */
    t_QeIwIpReassEventsCallback     *f_Events;          /**< User-provided callback for IP Reassembly specific
                                                         exception reporting (e_QeIwIpReassEvent exception type) */
    t_RxStoreFunction               *f_RxStore;         /**< User-provided callback for processing the buffers received in the available CPU Swap queues */
    uint8_t                         maxNumOfFrags;      /**< The maximum number of frags (of length = Rx[MaxFrameLength])
                                                        that will be reassembled.
                                                        NOTE: When IW is performed, the destination has to support a MaxFrameLength enough to
                                                        fit the reassembled frame. */
 
} t_QeIwIpReassParam;

/**
 @Function      QE_IW_IPREASS_Config

 @Description   Creates an uninitialized QE_IW_IPREASS object, used to configure
                the IP Reassembly functionality.

 @Param         p_Params - (In)    Required non-default parameters for the QE_IW_IPREASS object.

 @Return        Handle to the newly created QE_IW_IPREASS object to be later passed in API calls.
 @Return        NULL in case of failure to create object or invalid parameters.
*/
t_Handle QE_IW_IPREASS_Config ( t_QeIwIpReassParam *p_Params );


/**
 @Function      QE_IW_IPREASS_Init

 @Description   This function initializes driver and hardware for IP Reassembly functionality.
 				It also enables the IP Reassembly functionality in the QuiccEngine block.

 @Param         h_IpReass    - (In)    Handle to the QE_IW_IPREASS object.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      h_IpReass must have been previously created using QE_IW_IPREASS_Config().
*/
t_Error QE_IW_IPREASS_Init ( t_Handle h_IpReass );


/**
 @Function      QE_IW_IPREASS_Free

 @Description   Frees the resources associated to the QE_IW_IPREASS object.
  				It also disables the IP Reassembly functionality in the QuiccEngine block.
 
 @Param         h_IpReass    - (In)    Handle to the QE_IW_IPREASS object.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      h_IpReass must have been previously created using QE_IW_IPREASS_Config().
*/
t_Error QE_IW_IPREASS_Free ( t_Handle h_IpReass );


/**************************************************************************//**
 @Group         iw_ipReass_init_advanced_grp IW IP REASSEMBLY Object Advanced Initialization Unit

 @Description   IW IP REASSEMBLY Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
 @Description    Combination of flags indicating the action performed on the middle
                 fragments of an in-reassembly packet when an exception event occurs
*/
typedef enum
{
      e_QE_IW_IPREASS_DEBUG_MODE_REJECT_FRAGMENTS   = 0x01  /**< Discard middle fragments, */
    , e_QE_IW_IPREASS_DEBUG_MODE_FORWARD_FRAGMENTS  = 0x02  /**< Forward middle fragments in to one of the available CPU queues */
}e_QeIwIpReassCustomFlags;


/**
 @Description    Definition of the debug mode in custom IP reassembly
*/
typedef struct
{
      uint8_t                       queueIndex; /**< ETH CPU Queue index, for forwarding middle fragments*/
      e_QeIwIpReassCustomFlags      flags;      /**< Debug mode ( IP Reass Custom is able to forward or reject the middle-fragments ) */
}t_QeIwIpReassCustomDebugMode;

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Description    Combination of flags indicating the action performed on a fragment or
                 on a frame when an exception event occurs
*/
typedef enum
{
      e_QE_IW_IPREASS_FULL_FRAG_DISCARD                              = 0x0000   /**< Discard duplicate/overlap/error/short fragment */
    , e_QE_IW_IPREASS_FULL_DUPLICATE_FRAG_FORWARD_FRAG_TO_CPU_SWAP   = 0x8000   /**< Forward duplicate fragment to CPU Swap */
    , e_QE_IW_IPREASS_FULL_OVERLAP_FRAG_FORWARD_FRAG_TO_CPU_SWAP     = 0x4000   /**< Forward overlap fragment to CPU Swap */
    , e_QE_IW_IPREASS_FULL_ERROR_FRAG_FORWARD_FRAG_TO_CPU_SWAP       = 0x2000   /**< Forward error fragment to CPU Swap */
    , e_QE_IW_IPREASS_FULL_SHORT_FRAG_FORWARD_FRAG_TO_CPU_SWAP       = 0x1000   /**< Forward short fragment to CPU Swap */
    , e_QE_IW_IPREASS_FULL_FRAME_REASSEMBLE                          = 0x0000   /**< Forward short fragment to CPU Swap */
    , e_QE_IW_IPREASS_FULL_DUPLICATE_FRAG_FRAME_DISCARD 			 = 0x0040   /**< Drop all fragments belonging to the frame when
    																		        a duplicated fragment was received */
    , e_QE_IW_IPREASS_FULL_DUPLICATE_FRAG_FRAME_FORWARD_TO_CPU_SWAP  = 0x0080   /**< Forward the reassembled frame to CPU Swap queue
    																		        when a duplicated fragment was received */
    , e_QE_IW_IPREASS_FULL_OVERLAP_FRAG_FRAME_DISCARD     			 = 0x0010   /**< Drop all fragments belonging to the frame when
    																		        an overlap fragment was received  */
    , e_QE_IW_IPREASS_FULL_OVERLAP_FRAG_FRAME_FORWARD_TO_CPU_SWAP    = 0x0020   /**< Forward the reassembled frame to CPU Swap queue
    																		        when an overlap fragment was received */
    , e_QE_IW_IPREASS_FULL_ERROR_FRAG_FRAME_DISCARD     			 = 0x0004   /**< Drop all fragments belonging to the frame when
    																		        an error fragment was received */
    , e_QE_IW_IPREASS_FULL_ERROR_FRAG_FRAME_FORWARD_TO_CPU_SWAP      = 0x0008   /**< Forward the reassembled frame to CPU Swap queue
    																		        when an error fragment was received */
} e_QeIwIpReassFullDebugFlags;


typedef struct
{
      uint16_t      flags;      /* Debug mode ( any combination of flags from e_QeIwIpReassFullDebugFlags) */

}t_QeIwIpReassFullDebugMode;

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Description   User Parameters for configuring the IP Reassembly Object's debug mode
*/
typedef struct {
 
    union _ipreassMode{

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

       t_QeIwIpReassCustomDebugMode   custom;       /**< IP Reassembly Custom Debug Mode */

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

       t_QeIwIpReassFullDebugMode     full;         /**< IP Reassembly Full Debug Mode */

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

    } ipreassMode;                                  /**< Debug mode parameters */

} t_QeIwIpReassDebugMode;

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Description   Timeout Mode
*/
typedef enum
{
      e_QE_IW_IPREASS_FULL_TIMEOUT_MODE_FRAGMENT = 0 /**< Timeout calculated for each fragment */
    , e_QE_IW_IPREASS_FULL_TIMEOUT_MODE_FRAME    = 1 /**< Timeout calculated per frame */
} e_QeIwIpReassFullTimeoutMode;

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Description   User parameters for Timeout
*/
typedef struct {

    uint32_t                        timeoutInterval;    /**< Timeout in miliseconds */
#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE
    uint32_t                        accuracy;           /**< Timeout accuracy in miliseconds */
    e_QeIwIpReassFullTimeoutMode    timeoutMode;        /**< Timeout mode */
#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

} t_QeIwIpReassTimeoutParam;

/**
 @Function      QE_IW_IPREASS_ConfigExceptionEvents

 @Description   Configures the events to be triggered by IP Reassembly functionality.
                [e_QE_IW_IPREASS_EVENT_NONE]
 
 @Param         h_IpReass  - (In) Handle to the QE_IW_IPREASS object.
 @Param         flags      - (In) Combination of exception events
                             (use values from e_QeIwIpReassFullEvent - IP Reassembly Full;
                              use values from e_QeIwIpReassCustomEvent - IP Reassembly Custom).

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigExceptionEvents ( t_Handle h_IpReass, uint32_t flags );


/**
 @Function      QE_IW_IPREASS_ConfigDebugMode
 
 @Description   Configures the debug mode for handling the middle fragments of an in-reassembly packet,
                when an exception event occurs.
                [e_QE_IW_IPREASS_DEBUG_MODE_REJECT_FRAGMENTS]
 
 @Param         h_IpReass - (In) Handle to the QE_IW_IPREASS object.
 @Param         debugMode - (In) The new value for the Debug Mode settings. This value
                                 must be a combination of e_QeIwIpReassFullDebugMode or
                                 t_QeIwIpReassCustomDebugMode values.

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
 
*/
t_Error QE_IW_IPREASS_ConfigDebugMode ( t_Handle               h_IpReass,
                                        t_QeIwIpReassDebugMode* debugMode );


/**
 @Function      QE_IW_IPREASS_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param         h_IpReass       - (In) Handle to IP Reassembly object.
 @Param         memId           - (In) Memory partition ID.
 @Param         memAttributes   - (In) Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/

t_Error QE_IW_IPREASS_ConfigFirmwareExternalStructsMemory(t_Handle h_IpReass,
                                                          uint8_t  memId,
                                                          uint32_t memAttributes);
 
/**
 @Function      QE_IW_IPREASS_ConfigMaxFrameLength

 @Description   Configures the maximum Ethernet frame length in bytes.
                [1518]
 
 @Param         h_IpReass       - (In) Handle to IP Reassembly object.
 @Param         maxFrameLength  - (In) Requested maximum frame length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().

*/
t_Error QE_IW_IPREASS_ConfigMaxFrameLength(t_Handle h_IpReass, uint16_t maxFrameLength);

/**
 @Function      QE_IW_IPREASS_ConfigTimeout

 @Description   Configures the IP Reassembly Timeout feature
 
 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         p_TimeoutParams - (In) User parameters to configure the timeout;

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigTimeout ( t_Handle                  h_IpReass,
                                      t_QeIwIpReassTimeoutParam *p_TimeoutParams );

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**************************************************************************//**
 @Group         iw_ipReass_init_advanced_custom_grp IW IP REASSEMBLY Object CUSTOM Advanced Initialization Unit

 @Description   IW IP REASSEMBLY Object CUSTOM API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description   User Parameters for configuring the operational mode of the IPREASS module
*/
typedef enum
{
      e_QE_IW_IPREASS_MODE_IPoE     = 0x01  /**< Configure the IP Reassembly object to enable specific IPoPPPoEthernet reassembly */
    , e_QE_IW_IPREASS_MODE_IPoPPPoE = 0x02  /**< Configure the IP Reassembly object to enable specific IPoEthernet reassembly */

}t_QeIwIpReassMode;


/**
 @Function      QE_IW_IPREASS_ConfigMode

 @Description   Configures the IP Reassembly to support reassembly of either IP-over-Ethernet or IP-over-PPP-over-Ethernet encapsulation.
                [e_QE_IW_IPREASS_MODE_IPoE]
 
 @Param         h_IpReass   - (In) Handle to the QE_IW_IPREASS object.
 @Param         ipReassMode - (In) Indication of the protocol-stack to be reassembled

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigMode ( t_Handle h_IpReass, t_QeIwIpReassMode ipReassMode );

/**
 @Function      QE_IW_IPREASS_ConfigNumOfThreads

 @Description   Configures the number of threads used for IP Reassembly functionality.
                [1]
 
 @Param         h_IpReass           - (In) Handle to the QE_IW_IPREASS object.
 @Param         numOfStaticThreads  - (In) Number of threads.
                                           Must be in range [1..32]
 @Param         iwExtendedThreadPage- (In) Indicate if support for Extended IW features is required (e.g. IPHC)
 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigNumOfThreads ( t_Handle h_IpReass,
                                           uint8_t numOfStaticThreads,
                                           bool iwExtendedThreadPage);

/**
 @Function      QE_IW_IPREASS_ConfigNumOfBdRings
 
 @Description   Configures the number of BD Rings used for IP Reassembly functionality.
                [17]

 @Param         h_IpReass      - (In) Handle to the QE_IW_IPREASS object.
 @Param         numOfBdRings   - (In) Number of BD Rings.
                                      Must be in range [8..248] and also a multiple of 8.
 
 @Return        E_OK if success; Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigNumOfBdRings ( t_Handle h_IpReass, uint8_t numOfBdRings );

/**
 @Function      QE_IW_IPREASS_ConfigCpuBdRingLen
 
 @Description   Configures the length of the CPU BD Ring available to the IP Reassembly functionality.
                [32]
 
 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         iprQueue        - (In) Selected CPU Queue to be modified
 @Param         cpuBdRingLen    - (In) CPU BD Ring length.
                                       Must be in range [16..65535]

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigCpuBdRingLen ( t_Handle              h_IpReass,
                                           e_QeIwIpReassCpuQueue iprQueue,
                                           uint16_t              cpuBdRingLen );

/**
 @Function      QE_IW_IPREASS_ConfigExtParseLookupMemory

 @Description   Configures the type of external memory where will reside all the lookup tables
                used in parse and lookup operations for this device.
                [0, MEMORY_ATTR_CACHEABLE]
 
 @Param         h_IpReass       - (In) Handle to IP Reassembly object.
 @Param         memId           - (In) Memory partition ID.
 @Param         memAttributes   - (In) Memory attributes mask.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error    QE_IW_IPREASS_ConfigExtParseLookupMemory(t_Handle h_IpReass,
                                            uint8_t memId, uint32_t memAttributes);

/** @} */ /* end of iw_ipReass_init_advanced_custom_grp group */

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**************************************************************************//**
 @Group         iw_ipReass_init_advanced_full_grp IW IP REASSEMBLY Object FULL Advanced Initialization Unit

 @Description   IW IP REASSEMBLY Object FULL API Advanced Initialization Unit

 @{
*//***************************************************************************/


/**
 @Description  Parameters for the configuration of the emergency timeout feature
*/
typedef struct {

    uint16_t                        minThreshold;               /**< Minimum threshold, in number of open frames, below which the emergency situation ends and timeout returns to the value configured via the QE_IW_IPREASS_ConfigTimeout function. */
    uint16_t                        maxThreshold;               /**< Maximum threshold, in number of open frames, starting from which the emergency timeout value (specified here) takes effect */
    uint32_t                        emergencyTimeoutValue;      /**< Emergency timeout interval in miliseconds */
 
} t_QeIwIpReassFullEmergencyTimeoutParam;

/**
 @Description  CPU Swap Queue configuration parameters
*/
typedef struct t_IpReassCpuSwapQParam
{
	bool     isExternal;             /**< Queue descriptor is on external or internal memory */
	uint8_t  interruptQueue;         /**< Index of interrupt queue to be used*/
    uint16_t bdRingLen;              /**< BD ring size in number of BDs */
    uint8_t  expWeightFactor;        /**< Exponential weight factor for WRED */

} t_IpReassCpuSwapQParam;

/**
 @Function      QE_IW_IPREASS_ConfigTimeoutEmergency

 @Description   Configures the IP Reassembly Timeout thresholds. This allows
                the system to automatically switch to an emergency timeout value if
                it is running out of frame descriptors for reassembly. This may
                prevent Denial of Service attacks.
 
 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         p_EmergencyTimeoutParams - (In) the parameters for the emergency timeout.

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
 @Cautions      Allowed only for IP Reassembly, type FULL.
*/
t_Error QE_IW_IPREASS_ConfigTimeoutEmergency(t_Handle h_IpReass,
                t_QeIwIpReassFullEmergencyTimeoutParam *p_EmergencyTimeoutParams);

/**
 @Function      QE_IW_IPREASS_ConfigTimeoutTimeStamper

 @Description   Configures Time Stamp Register to use when monitoring the reassembled
                frame/fragments timeout.
                [e_QE_TIME_STAMPER_1]
 
 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         timeStampRegister - (In) the identifier of the Time Stamper to use.

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
 @Cautions      Allowed only for IP Reassembly, type FULL.
*/
t_Error QE_IW_IPREASS_ConfigTimeoutTimeStamper(t_Handle h_IpReass, e_QeTimeStamper timeStampRegister);

/**
 @Function      QE_IW_IPREASS_FULL_ConfigMinFragmentLength

 @Description   Configures the minimum fragment size in bytes, for first and
                middle fragments.
                [68]
 
                If a fragment with a size shorter than this value is detected and
                the timeout has not expired, the Ip Reassembly will first reassemble
                the frame and then drop it. Otherwise the timeout mechanism will drop
                the already received fragments.
 
 @Param         h_IpReass         - (In) Handle to the QE_IW_IPREASS object.
 @Param[in]     minFragmentLength - (In) Requested minimum fragment length. The size is
                                       defined by IP Total Length value (IP Header
                                       Length + IP Data Length). The IP Data Length
                                       must be a multiple of 8.

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and
                before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigMinFragmentLength (t_Handle h_IpReass,
                                               uint16_t minFragmentLength);


/**
 @Function      QE_IW_IPREASS_ConfigHashBuckets

 @Description   Configures the size of the Hash Bucket Pool and the maximum number of
                linked buckets per entry
                [1]
                [1]
 
 @Param         h_IpReass                 - (In) Handle to IP Reassembly object.
 @Param[in]     hashBucketPoolSize        - (In) Number of buckets in the Hash Bucket Pool
 @Param[in]     numOfHashBucketsPerEntry  - (In) Maximum number of hash buckets per entry

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigHashBuckets ( t_Handle h_IpReass,
                            uint16_t hashBucketPoolSize, uint16_t numOfHashBucketsPerEntry );


/**
 @Function      QE_IW_IPREASS_ConfigCpuSwapQueue

 @Description   Configures the CPU Swap Queue Parameters
 
 @Param         h_IpReass - (In) Handle to IP Reassembly object.
 @Param[in]     p_Params  - (In) CPU Swap Queue parameters structure

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigCpuSwapQueue (t_Handle h_IpReass, t_IpReassCpuSwapQParam *p_Params);


/**
 @Function      QE_IW_IPREASS_ConfigPreserveIdentificationField

 @Description   Configures the mode the identification field in IP header of fragments, is
                is preserved or cleared
                [FALSE]
 
 @Param         h_IpReass - (In) Handle to IP Reassembly object.
 @Param[in]     preserveIdField  - (In) Identification field mode

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigPreserveIdentificationField (t_Handle h_IpReass,
                                                         bool     preserveIdField);

 
/**
 @Function      QE_IW_IPREASS_ConfigPerFlowStatisticsMode

 @Description   Configures the IP Reassembly Per Flow Statistics
                [FALSE]
 
 @Param         h_IpReass      - (In) Handle to IP Reassembly object.
 @Param         statisticsMode - (In) Per Flow Statistics mode to be
                                used by the IP Reassembly Object

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
 @Cautions      Allowed only for IP Reassembly, type FULL.
*/
t_Error QE_IW_IPREASS_ConfigPerFlowStatisticsMode ( t_Handle h_IpReass,
                                                    bool     statisticsMode );

/**
 @Function      QE_IW_IPREASS_ConfigFwdOnFilterLast

 @Description   Configures a reserved CPU Swap queue to which the reassembled frame
                is forwarded when its PCD chain ends with last PCD.
 
 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         resQueueId      - (In) Reserved queue Id;
 @Param         isExternal      - (In) Queue is in external or internal memory

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_IPREASS_Config() and before QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ConfigFwdOnFilterLast(t_Handle h_IpReass,
                                                        uint16_t resQueueId, bool isExternal);

/** @} */ /* end of iw_ipReass_init_advanced_full_grp group */

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/** @} */ /* end of iw_ipReass_init_advanced_grp group */
/** @} */ /* end of iw_ipReass_init_grp group */

/**************************************************************************//**
 @Group         iw_ipReass_control_grp IW IP REASSEMBLY Object Control Unit

 @Description   IW IP REASSEMBLY Object API Control Unit

 @{
*//***************************************************************************/

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
@Description   IP Reassembly Custom Statistics
*/
typedef struct {

    uint32_t    discardedPartialyReassembledFrames_Cnt; /**< This counter is incremented if a frame is discarded without reassembly being completed, because one of the following conditions: a wrong fragment offset was received; a regular IP frame was received before the last fragment of the previous frame was received; a FCS error occurred on one of the fragments; a time-out condition occurred; IP Reassembly busy process occurred. */
    uint32_t    discardedIpFragments_Cnt;               /**< This counter is incremented if a fragment is discarded because one of the following conditions: the fragment offset is not expected  (out-of -order fragment); IP Reassembly busy process occurred */
} t_QeIwIpReassCustomStatistics;

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
@Description   IP Reassembly Full Statistics
*/
typedef struct {
    uint32_t    timeout_Cnt;        /**< This counter is incremented when a timeout condition occured */
    uint32_t    fragDrop_Cnt;       /**< This counter is incremented when a fragment is dropped */
    uint32_t    fragOverlap_Cnt;    /**< This counter is incremented when a fragment overlap is received */
    uint32_t    fragDuplicate_Cnt;  /**< This counter is incremented when a fragment duplicate is received */
    uint32_t    fragError_Cnt;      /**< This counter is incremented when a fragment error is received */
    uint32_t    fragShort_Cnt;      /**< This counter is incremented when a fragment short is received */
    uint32_t    frameBusy_Cnt;      /**< This counter is incremented when a frame is received and the number of open frames was exceeded  */
    uint32_t    fragBusy_Cnt;       /**< This counter is incremented when a fragment is received and the total number of fragments allowed was exceeded*/
    uint32_t    bdBusy_Cnt;         /**< This counter is incremented when the number of BDs for a frame was exceeded */
    uint32_t    iprFrag_Cnt;        /**< This counter is incremented when a fragment is received */
    uint32_t    iprFrame_Cnt;       /**< This counter is incremented when a frame is reassembled and forwarded to a destination queue */
    uint32_t    toFragDropCnt;      /**< Counts the number of total fragments that are dropped due to timeout condition */
} t_QeIwIpReassFullStatistics;

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
@Description   IP Reassembly Statistics
*/
typedef struct {
    e_QeIwIpReassType   type;                              /**< The type of IP reassembly control object */

    union _ipreassStats{

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

        t_QeIwIpReassCustomStatistics   customStats;       /**< IP Reassembly Custom Statistics */

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

        t_QeIwIpReassFullStatistics     fullStats;         /**< IP Reassembly Full Statistics */

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */
 
    } ipreassStats;

} t_QeIwIpReassStatistics;

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Description   Callback routine for storing the Reassembly filter.

 @Param         h_IpReass	- (In) Handle to the QE_IW_IPREASS object
 @Param         h_Filter  	- (In) Filter with Reassembly support

 @Return        None.
*/
typedef t_Error (t_QeIwIpReassStoreFilter)(t_Handle h_IpReass, t_Handle h_Filter);

/**
 @Description  IP REASSEMBLY parameters required by the FLTR_FILTER object, with REASSEMBLY support
*/
typedef struct
{
	uint32_t                    compressKey;              	/**< IP Reassembly Full Compress Key */
	uint32_t                    hashTableBasePtr;        	/**< IP Reassembly Hash Table Base Pointer */
	t_QeIwIpReassStoreFilter    *f_QeIwIpReassStoreFilter; 	/**< Callback routine to store the Filter */

} QeIwIpReassFilterService;

#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
 @Function      QE_IW_IPREASS_Rx

 @Description   Handles data buffers received in the specified CPU receive queue.

                Received data is passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific queue.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. The routine should
                be called by the user as often as necessary, as no interrupt is generated
                upon reception of a packet.

 @Param[in]     h_IpReass   - Handle to UCC GETH object.
 @Param[in]     rxQueue     - CPU Rx queue on which to receive.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_Rx( t_Handle h_IpReass, e_QeIwIpReassCpuQueue rxQueue);

/**
 @Function      QE_IW_IPREASS_ModifyFilter

 @Description   Modify the destination FILTER for REASSEMBLED packets.

 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         h_Filter        - (In) Handle to the new FILTER that will be set as the destination for the reassembled packets

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ModifyFilter( t_Handle h_IpReass, t_Handle h_Filter );

#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Function      QE_IW_IPREASS_Enable

 @Description   This function enables the IP Reassembly in the QuiccEngine block.

 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Return        E_OK in case of success. Error code otherwise.
 
 @Cautions      h_IpReass must have been previously created using QE_IW_IPREASS_Config()
                and initialized using QE_IW_IPREASS_Init(). Should be used only when IP
                Reassembly is disabled.
*/
t_Error  QE_IW_IPREASS_Enable(t_Handle h_IpReass);

/**
 @Function      QE_IW_IPREASS_Disable

 @Description   This function disables the IP Reassembly in the QuiccEngine block.
				IP Reassembly can be re-enabled using QE_IW_IPREASS_Enable
	
 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Return        E_OK in case of success. Error code otherwise.
 
 @Cautions      h_IpReass must have been previously created using QE_IW_IPREASS_Config()
                and initialized using QE_IW_IPREASS_Init().Should be used only when IP
                Reassembly is enabled.
*/
t_Error  QE_IW_IPREASS_Disable(t_Handle h_IpReass);

/**
 @Function      QE_IW_IPREASS_GetFilterService

 @Description   Get the IP Reassembly parameters related to Filter
 
 @Param         h_IpReass  - (In) Handle to the QE_IW_IPREASS object.
 @Param         p_QeIwIpReassFilterService - (Out) FLTR_FILTER (w/ REASSEMBLY action) necessary parameters
 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_GetFilterService(t_Handle 			  	h_IpReass,
                              		   QeIwIpReassFilterService	*p_QeIwIpReassFilterService);

#endif	/* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Function      QE_IW_IPREASS_GetStatistics

 @Description   Retrieves at runtime the statistics gathered for this IP Reassembly object.

 @Param         h_IpReass       - (In) Handle to the QE_IW_IPREASS object.
 @Param         p_Stats         - (Out)Pointer to user-supplied structure where to place the statistics gathered.

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_GetStatistics( t_Handle h_IpReass, t_QeIwIpReassStatistics *p_Stats );


/**
 @Function      QE_IW_IPREASS_ResetStatistics

 @Description   Clear the IP Reassembly statistics counters.
 
 @Param         h_IpReass  - (In) Handle to the QE_IW_IPREASS object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_IPREASS_Init().
*/
t_Error QE_IW_IPREASS_ResetStatistics ( t_Handle h_IpReass );

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**
 @Function      QE_IW_IPREASS_DumpRegs

 @Description   Dump IP Reassembly Parameter Table regs.

 @Param         h_IpReass  - (In) Handle to the QE_IW_IPREASS object.
*/
t_Error QE_IW_IPREASS_DumpRegs ( t_Handle h_IpReass );


#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
 @Function      QE_IW_IPREASS_DumpQds

 @Description   Dump IP Reassembly Queue Descriptors

 @Param         h_IpReass  - (In) Handle to the QE_IW_IPREASS object.
*/
t_Error QE_IW_IPREASS_DumpQds ( t_Handle h_IpReass );


#endif  /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

/**
 @Function      QE_IW_IPREASS_DumpBdRings

 @Description   Dump IP Reassembly BD Rings

 @Param         h_IpReass  - (In) Handle to the QE_IW_IPREASS object.
*/
t_Error QE_IW_IPREASS_DumpBdRings ( t_Handle h_IpReass );
#endif /* (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0)) */

/** @} */ /* end of iw_ipReass_control_grp group */
/** @} */ /* end of iw_ipReass_grp group */
/** @} */ /* end of iw_grp group */

#endif  /* not QE_IW_IPREASS_CUSTOM_UNAVAILABLE || not QE_IW_IPREASS_FULL_UNAVAILABLE */

#endif /* __QE_IW_IPREASS_EXT_H */
