/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          qmc_trns_ext.h QMC TRANSPARENT channel external API.

 @Description   QMC TRANSPARENT  external functions structures and definitions.

 @Cautions      None.
 
*//***************************************************************************/
#ifndef __QMC_TRNS_EXT_H
#define __QMC_TRNS_EXT_H

#include "std_ext.h"
#include "error_ext.h"
#include "qmc_ext.h"

/**************************************************************************//**
 @Group         qmc_grp QMC Application Programming Interface

 @Description   QMC API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         qmc_trns_grp QMC TRANSPARENT Application Programming Interface

 @Description   QMC TRANSPARENT API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         qmc_trns_init_grp QMC TRANSPARENT Initialization Unit.

 @Description   Configuration functions used to allocate resources and to set user and default
 				parameters.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   QMC TRANSPARENT channel user parameters structure.
*//***************************************************************************/
typedef t_QmcChParam t_QmcTrnsParam;

/**************************************************************************//**
 @Function      QMC_TRNS_Config

 @Description   Configures a QMC TRANSPARENT channel.
 
                This routine allocates the QMC channel data structures and sets
                user and default parameters in the QMC channel driver
                parameters structure.

 @Param[in]     p_QmcTrnsParam - Pointer to QMC transparent channel parametrs structure.

 @Return        Handle to new QMC TRANSPARENT channel.
*//***************************************************************************/
t_Handle QMC_TRNS_Config(t_QmcTrnsParam *p_QmcTrnsParam);

/**************************************************************************//**
 @Function      QMC_TRNS_Init

 @Description   Transparent channel initilization.
 
                This routine initializes the transparent channel's specific
                parameters at the channel's parameter RAM.
	
 				Global channel initilization:
 				1. Assignment t_Qmc parameters out of the driver parameters.
 				2. Allocation and Initialization of the channel's Tx & Rx BD ring.
 				3. Initialization of SI RAM & QMC TSA slots.
 				4. Initialization of global parameters at the channel's specific
 				   parameter ram.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT channel.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_TRNS_Init(t_Handle h_QmcTrns);

/**************************************************************************//**
 @Function      QMC_TRNS_Enable

 @Description   Enables the given QMC channel.

				This routine enables Tx and/or Rx for this channel through the rstate and
 				tstate registers in the channel's specific paraneter ram.

 @Param[in]     h_QmcTrns   - Handle to QMC TRANSPARENT channel.
 @Param			mode        - Tx/Rx communication mode.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_TRNS_Enable(t_Handle h_QmcTrns, e_CommMode mode);

/**************************************************************************//**
 @Function      QMC_TRNS_Free

 @Description   Free QMC TRANSPARENT channel resources.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT channel.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_TRNS_Free(t_Handle h_QmcTrns);

/***************************************************************************
					ADVANCED CONFIG FUNCTIONS
****************************************************************************/
/**************************************************************************//**
 @Group         qmc_trns_advanced_init_grp QMC TRANSPARENT Advanced Configuration Unit.

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QMC_TRNS_ConfigPad

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Number of pad chars, 0-15, to transmit after the frame's data transmission
                is done [0].

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.
 @Param[in]     newVal    - New value..

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config().
                May be called only before QMC_TRNS_Init().
                Change is applied only upon calling QMC_TRNS_Init().
*//***************************************************************************/
t_Error QMC_TRNS_ConfigPad(t_Handle h_QmcTrns, uint8_t newVal);

/**************************************************************************//**
 @Function      QMC_TRNS_ConfigrxInterrupts

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Flag indicating Rx method - interupt / polling. [TRUE].

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config().
                May be called only before QMC_TRNS_Init().
                Change is applied only upon calling QMC_TRNS_Init().
*//***************************************************************************/
t_Error QMC_TRNS_ConfigRxInterrupts ( t_Handle h_QmcTrns, bool newVal );

/**************************************************************************//**
 @Function      QMC_TRNS_ConfigIntrManagerId

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                If zero, driver invokes driver receive routine
                directly inside interrupt.
                If non-zero, driver instead defers invocation of
                driver receive routine to user. Driver informs user by invoking
                XX_Call(). It is then responsibility of user to ensure
                that drive receive routine is eventually called. XX_Call()
                will not be called again until user calls driver receive routine.
                Driver calls XX_Call() as follows:
                XX_Call(this value, NULL, appId, 0, 0).
                [0]

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config().
                May be called only before QMC_TRNS_Init().
                Change is applied only upon calling QMC_TRNS_Init().
*//***************************************************************************/
t_Error QMC_TRNS_ConfigIntrManagerId ( t_Handle h_QmcTrns, uint32_t newVal );

/**************************************************************************//**
 @Function      QMC_TRNS_ConfigintrMask

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Channel's interupts mask. [QMC_INTR_MRF].

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config().
                May be called only before QMC_TRNS_Init().
                Change is applied only upon calling QMC_TRNS_Init().
*//***************************************************************************/
t_Error QMC_TRNS_ConfigIntrMask ( t_Handle h_QmcTrns, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_TRNS_ConfigRxMaxBufLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Channel's maximum receive buffer length. defines the number of
                bytes written by the QE to each BD's data buffer before moving to
                the next BD. [0x40].

 @Param[in]     h_QmcTrns   - Handle to QMC TRANSPARENT Channel.
 @Param[in]     newVal      - New value.

 @Return        E_OK on success, error code otherwise.
 
 @Cautions      h_QmcTrns must have been previously created using QMC_CH_Config().
                May be called only before QMC_CH_Init().
                Change is applied only upon calling QMC_CH_Init().
*//***************************************************************************/
t_Error QMC_TRNS_ConfigRxMaxBufLen ( t_Handle h_QmcTrns, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_TRNS_ConfigReverseData

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Flag indicating that the bit oredr on the channel is reversed. [e_QMC_NORMAL].

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config().
                May be called only before QMC_TRNS_Init().
                Change is applied only upon calling QMC_TRNS_Init().
*//***************************************************************************/
t_Error QMC_TRNS_ConfigReverseData ( t_Handle h_QmcTrns, e_QmcTrnsReverse newVal );

/** @} */ /* end of qmc_trns_advanced_init_grp group */
/** @} */ /* end of qmc_trns_init_grp group */

/**************************************************************************//**
 @Group         qmc_trns_control_grp QMC TRANSPARENT Control Unit.

 @Description   Control unit functions.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      QMC_TRNS_StopRx

 @Description   Stop transmission on a given transparent channel.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config
                and initialized using QMC_TRNS_Init.
*//***************************************************************************/
t_Error QMC_TRNS_StopTx(t_Handle h_QmcTrns);

/**************************************************************************//**
 @Function      QMC_TRNS_StopRx

 @Description   Stop transmission on a given transparent channel.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config
                and initialized using QMC_TRNS_Init.
*//***************************************************************************/
t_Error QMC_TRNS_StopRx(t_Handle h_QmcTrns);

/**************************************************************************//**
 @Function      QMC_TRNS_RestartTx

 @Description   Restart transmission on a given transparent channel.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config
                and initialized using QMC_TRNS_Init.
*//***************************************************************************/
t_Error QMC_TRNS_RestartTx (t_Handle h_QmcTrns);

/**************************************************************************//**
 @Function      QMC_TRNS_RestartRx

 @Description   Restart transmission on a given channel.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcTrns must have been previously created using QMC_TRNS_Config
                and initialized using QMC_TRNS_Init.
*//***************************************************************************/
t_Error QMC_TRNS_RestartRx (t_Handle h_QmcTrns);

/**************************************************************************//**
 @Function      QMC_TRNS_TxConf

 @Description   QMC_TRNS channel function for forced transmit confirmation.
 
                This routine collects the transmitted buffers and returns them
                to the application. It also prepares the transmit BDs for new
                buffers. For each buffer the application callback routine will
                be called (f_TxConf in t_QmcChParam).
 
                Normally, transmit confirmation is done automatically by the
                transmit routine. This routine can be called after transmission
                was stopped, to collect all remaining buffers from the BD ring.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT Channel.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_TRNS_TxConf(t_Handle h_QmcTrns);

/** @} */ /* end of qmc_trns_control_grp group */

/**************************************************************************//**
 @Group         qmc_trns_runtime_grp QMC TRANSPARENT Runtime Unit.

 @Description   Rx / Tx functions.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      QMC_TRNS_Tx

 @Description   Transmit data over QMC TRANSPARENT channel.

 @Param[in]     h_QmcTrns 	    - Handle to a QMC TRANSPARENT channel.
 @Param[in]     p_Data 		    - Pointer to a single data buffer.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length 		    - Data length.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_TRNS_Tx(t_Handle    h_QmcTrns,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint32_t    length);

/**************************************************************************//**
 @Function      QMC_TRNS_Rx

 @Description   Receive data over QMC TRANSPARENT channel.

                This function is called by the upper application to instigate
                receive handling. Upon interrupt on receive, depending on queueId
                parameter, either this routine is called directly by the driver or
                by the user.
                QMC_TRNS_Rx routine collects the data buffers from the port BD's and
                passes them to the application by calling the application's
                f_RxStore routine.

 @Param[in]     h_QmcTrns - Handle to QMC TRANSPARENT channel.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_TRNS_Rx(t_Handle h_QmcTrns);

/** @} */ /* end of qmc_trns_runtime_grp group  */
/** @} */ /* end of qmc_trns_grp group  */
/** @} */ /* end of qmc_grp group */

#endif  /* __QMC_TRNS_EXT_H */
