/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          rng_ext.h

 @Description   structures and definitions for generating random number.

*//***************************************************************************/
#ifndef __RNG_EXT_H
#define __RNG_EXT_H

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_rng SEC RNG

 @Description   SEC RNG functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_rng_init SEC RNG Initialization Unit

 @Description   SEC RNG initialization functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      rng_alloc_ctx

 @Description   Allocate memory for the rng ctx structure.
 
				The routine is not a part of the Crypto Api standard routines.

 @Return        ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
void *rng_alloc_ctx();

/**************************************************************************//**
 @Function      rng_init

 @Description   Initialization of rng ctx.

				This routine Initializes the Length in the rng ctx to zero.
				The routine is not a part of the Crypto Api standard routines.

 @Param         p_Ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void   rng_init(t_Handle p_Ctx);

/**************************************************************************//**
 @Function      rng_update

 @Description   update the length field in the rng ctx.

				This routine updates the Length in the rng ctx
                with the len value.
				The routine is not a part of the Crypto Api standard routines.

 @Param         p_Ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len   -  (In) the required random number length

 @Return        None.
*//***************************************************************************/
void   rng_update(t_Handle p_Ctx, unsigned int len);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_rng_runtime SEC RNG RunTime Unit

 @Description   SEC RNG Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      rng_final

 @Description   initialize and send the rng request to the Crypto hardware.

				This routine initializes the rng request structure
                by the fields in the rng ctx and then call to the Ioctl
                function in order to process the request.
				The routine is not a part of the Crypto Api standard routines.

 @Param         p_Ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         p_Out -  (In) the handler to the output random number.

 @Return        None.
*//***************************************************************************/
void   rng_final(t_Handle p_Ctx, unsigned char *p_Out);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec rng */
/** @} */ /* end of group sec */


#endif
