/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          sha1_ext.h

 @Description   structures and definitions for operating SHA1 algorithm.

*//***************************************************************************/
#ifndef __SHA1_EXT_H
#define __SHA1_EXT_H

/**************************************************************************//**
 @Description   the hash output size is 20 bytes.
*//***************************************************************************/
#define SHA1_OUTPUT 20

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_sha1 SEC SHA1

 @Description   SEC SHA1 functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_sha1_init SEC SHA1 Initialization Unit

 @Description   SEC SHA1 initialization functions.

 @{
*//***************************************************************************/

/* accessory routine */
/**************************************************************************//**
 @Function      sha1_alloc_ctx

 @Description   Allocate memory for the sha1 ctx structure.
                The routine is not a part of the Crypto Api standard routines.

 @Return        ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle sha1_alloc_ctx();

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      sha1_init

 @Description   The function initializes the fields in the sha1 ctx structure.
 
				In crypto API the function initializes a default
                context value. However, in Crypto hash calculation,
                this initialization is not relevant because it's
                part of the hardware accelerators operation.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void sha1_init(void *ctx);

/**************************************************************************//**
 @Function      sha1_update

 @Description   The routine sets the data source and its length in the sha1 ctx structure.
 
				The routine sets the data source and its length in
                the sha1 ctx structure. Those fields are important
                for initiating the sha1 request.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         data - (In) source input data.
 @Param         len -  (In) the source input length.

 @Return        None.
*//***************************************************************************/
void sha1_update(void *ctx, const unsigned char *data, unsigned int len);

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      sha1_update_ctx

 @Description   The routine updates the ctx structure.
 
				The routine sets the src that means the algorithm
                context and its length in the specific sha1 context
                field(p_HashCtx). Those fields are important for
                initiating the sha1 request.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) hash algorithm context.
 @Param         len -  (In) hash context length.

 @Return        None.
*//***************************************************************************/
void     sha1_update_ctx(void *ctx, unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      sha1_setpad

 @Description   The routine sets the PAD option.
 
				The function sets the Pad variable in the ctx
                structure to indicate that the user asked for padding.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void     sha1_setpad(void *ctx);

/**************************************************************************//**
 @Function      sha1_setidgs

 @Description   The routine sets the idgs - Init digest option.
 
				The function sets the idgs(init digest) variable in the ctx
                structure to indicate that the user asked for idgs possibility.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void     sha1_setidgs(void *ctx);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_sha1_runtime SEC SHA1 Runtime Unit

 @Description   SEC SHA1 Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      sha1_final

 @Description   The routine operates the request for SHA1 operation.
 
				The routine fills in a specific request with the
                sha1 ctx fields and then calls to ioctl function in order
                to process the specific request.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx - (In) ctx handler - holds the relevant parameters for the operation.
 @Param         out - (Out) the destination output data.

 @Return        None.
*//***************************************************************************/
void sha1_final(void *ctx, unsigned char *out);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec sha1 */
/** @} */ /* end of group sec */

#endif
