/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          sha256_ext.h

 @Description   structures and definitions for operating SHA256 algorithm.

*//***************************************************************************/
#ifndef __SHA256_EXT_H
#define __SHA256_EXT_H

/**************************************************************************//**
 @Description   the hash output size is 32 bytes.
*//***************************************************************************/
#define SHA256_OUTPUT 32

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_sha256 SEC SHA256

 @Description   SEC SHA256 functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_sha256_init SEC SHA256 Initialization Unit

 @Description   SEC SHA256 initialization functions.

 @{
*//***************************************************************************/

/* accessory routine */
/**************************************************************************//**
 @Function      sha256_alloc_ctx

 @Description   Allocate memory for the sha256 ctx structure.
 
                The routine is not a part of the Crypto Api standard routines.

 @Return        ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle sha256_alloc_ctx();

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      sha256_init

 @Description   The function initializes the fields in the sha256 ctx structure.

				In crypto API the function initializes a default
                context value. However, in Crypto hash calculation,
                this initialization is not relevant because it's
                part of the hardware accelerators operation.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void sha256_init(void *ctx);

/**************************************************************************//**
 @Function      sha256_update

 @Description   The function updates the fields in the sha256 ctx structure.

				The routine sets the data source and its length in
    	        the sha256 ctx structure. Those fields are important
                for initiating the sha256 request.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         data - (In) source input data.
 @Param         len -  (In) the source input length.

 @Return        None.
*//***************************************************************************/
void sha256_update(void *ctx, const unsigned char *data, unsigned int len);

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      sha256_update_ctx

 @Description   The function updates the sha256 ctx structure.

				The routine sets the src that means the algorithm
                context and its length in the specific sha256 context
                field(p_HashCtx). Those fields are important for
                initiating the sha256 request.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) hash algorithm context.
 @Param         len -  (In) hash context length.

 @Return        None.
*//***************************************************************************/
void     sha256_update_ctx(void *ctx, unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      sha256_setpad

 @Description   The function sets the PAD option in the sha256 ctx structure.

				The function sets the Pad variable in the ctx
                structure to indicate that the user asked for padding.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void     sha256_setpad(void *ctx);

/**************************************************************************//**
 @Function      sha256_setidgs

 @Description   The function sets the idgs option in the sha256 ctx structure.

				The function sets the idgs variable in the ctx
                structure to indicate that the user asked for idgs possibility.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void     sha256_setidgs(void *ctx);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_sha256_runtime SEC SHA256 Runtime Unit

 @Description   SEC SHA256 Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      sha256_final

 @Description   The function operates the request of SHA256 operation

				The routine fills in a specific request with the
                sha256 ctx fields and then calls to ioctl function in order
                to process the specific request.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx - (In) ctx handler - holds the relevant parameters for the operation.
 @Param         out - (In) the destination output data.

 @Return        None.
*//***************************************************************************/
void sha256_final(void *ctx, unsigned char *out);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec sha256 */
/** @} */ /* end of group sec */

#endif
