/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          srio_ext.h

 @Description   Header file for SRIO module routines

 @Cautions      None.

*//***************************************************************************/
#ifndef __SRIO_EXT_H
#define __SRIO_EXT_H

#include "std_ext.h"
#include "list_ext.h"
#include "rio_std_ext.h"
#include "part_integration_ext.h"

/**************************************************************************//**
 @Group         srio_port_id SRIO Port Application Programming Interface

 @Description   SRIO Port API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         srio_port_init SRIO Port Initialization Unit

 @Description   SRIO Port initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    General SRIO port definitions

 @{
*//***************************************************************************/
#define SRIO_MAX_NUM_OF_PORTS                16
#define SRIO_MAX_PORT_ID                     SRIO_NUM_OF_PORTS - 1

#define SRIO_NUM_OF_MSG_UNITS                2                              /**< Number of messaging units. */

#define SRIO_NUM_OF_MAILBOXES                SRIO_NUM_OF_MSG_UNITS          /**< Number of mailboxes in this SRIO port */
#define SRIO_MAX_MAILBOX_ID                  SRIO_NUM_OF_MAILBOXES - 1

#define SRIO_NUM_OF_OUT_QUEUES               SRIO_NUM_OF_MSG_UNITS          /**< Number of outbound queues unit in this SRIO port */
#define SRIO_MAX_OUT_QUEUE_ID                SRIO_NUM_OF_OUT_QUEUES - 1

 /* @} */

/**************************************************************************//**
 @Collection    SRIO ATMU windows definitions

 @{
*//***************************************************************************/
#define SRIO_MAX_IN_WIN_SIZE                 0x400000000    /**< Maximum size of an inbound window is 16GB  */
#define SRIO_MIN_IN_WIN_SIZE                 0x1000         /**< Minimum size of an inbound window is 4KB   */
#define SRIO_NUM_OF_IN_WINDOWS               5              /**< Number of inbound windows of the SRIO port */

#define SRIO_MAX_OUT_WIN_SIZE                0x1000000000   /**< Maximum size of an outbound window is 64GB */
#define SRIO_MIN_OUT_WIN_SIZE                0x1000         /**< Minimum size of an outbound window is 4KB  */
#define SRIO_NUM_OF_OUT_WINDOWS              9              /**< Number of outbound windows of the SRIO port*/

#define SRIO_MAX_NUM_OF_OUT_WINDOW_SEGS      4              /**< Max segments number of an outbound window  */
#define SRIO_MAX_NUM_OF_OUT_WINDOW_SUB_SEGS  8              /**< Max sub-segments number of an outbound window's segment */
/* @} */

 /**************************************************************************//**
 @Description   Supported address size in bits.
 *//***************************************************************************/
typedef enum e_SrioSupportedAddressSize
{
    e_SRIO_66_50_34_BIT_ADDRESS,
    e_SRIO_66_34_BIT_ADDRESS,
    e_SRIO_50_34_BIT_ADDRESS,
    e_SRIO_34_BIT_ADDRESS_ONLY
} e_SrioSupportedAddressSize;

 /**************************************************************************//**
 @Description   Possible origins of an occured error.
 
                For each error origin, there is a set of error masks above, to be
                used in user error callback routine to retrieve the errors from
                the received error register value.
 *//***************************************************************************/
typedef enum e_SrioErrOrigin
{
    e_SRIO_ERR_PORT_GENERAL,
    e_SRIO_INPUT_OR_OUTPUT_PORT_NOT_INITIALIZED,
    e_SRIO_ERR_LOG_TRANS_LAYER,
    e_SRIO_ERR_PHYSICAL_LAYER,
    e_SRIO_ERR_MAILBOX_0,
    e_SRIO_ERR_MAILBOX_1,
    e_SRIO_ERR_OUT_QUEUE_0,
    e_SRIO_ERR_OUT_QUEUE_1,
    e_SRIO_ERR_IN_DOORBELL,
    e_SRIO_ERR_OUT_DOORBELL,
    e_SRIO_ERR_IN_PORT_WRITE
} e_SrioErrOrigin;

/**************************************************************************//**
@Description   Soft port configuration to override hardware settings.
*//***************************************************************************/
typedef enum e_SrioForceSrioLaneNum
{
    e_SRIO_NO_OVERRIDE,
    e_SRIO_FORCE_LANE_0,
    e_SRIO_FORCE_LANE_2
} e_SrioForceSrioLaneNum;

/**************************************************************************//**
@Description   SRIO Error Rate Bias Value.
*//***************************************************************************/
typedef enum e_SrioErrRateBiasValue
{
    e_SRIO_DECREMENT_DISABLE       = 0x00,
    e_SRIO_DECREMENT_EVERY_1_MS    = 0x01,
    e_SRIO_DECREMENT_EVERY_10_MS   = 0x02,
    e_SRIO_DECREMENT_EVERY_100_MS  = 0x04,
    e_SRIO_DECREMENT_EVERY_1_S     = 0x08,
    e_SRIO_DECREMENT_EVERY_10_S    = 0x10,
    e_SRIO_DECREMENT_EVERY_100_S   = 0x20,
    e_SRIO_DECREMENT_EVERY_1000_S  = 0x40,
    e_SRIO_DECREMENT_EVERY_10000_S = 0x80
} e_SrioErrRateBiasValue;

/**************************************************************************//**
@Description   SRIO Error Rate Counter Limit Value.
*//***************************************************************************/
typedef enum e_SrioErrRateCountLimit
{
    e_SRIO_ERRORS_COUNT_ABOVE_TRESH_LIMIT_2    = 0x00,
    e_SRIO_ERRORS_COUNT_ABOVE_TRESH_LIMIT_4    = 0x01,
    e_SRIO_ERRORS_COUNT_ABOVE_TRESH_LIMIT_16   = 0x02,
    e_SRIO_ERRORS_COUNT_ABOVE_TRESH_LIMIT_NONE = 0x03
} e_SrioErrRateCountLimit;

/**************************************************************************//**
@Description   SRIO CRC Check definitions.
*//***************************************************************************/
typedef enum e_SrioEnableCrcCheck
{
    e_SRIO_CRC_CHECK_DISABLED,
    e_SRIO_CRC_CHECK_CONTROL_SYMBOL_ONLY,
    e_SRIO_CRC_CHECK_PACKET_ONLY,
    e_SRIO_CRC_CHECK_ALL
} e_SrioEnableCrcCheck;

/**************************************************************************//**
@Description   SRIO Serial link error injection control
*//***************************************************************************/
typedef enum e_SrioErrInjectControl
{
    e_SRIO_ERR_INJECT_DISABLED                = 0x00,
    e_SRIO_ERR_INJECT_LANE_0_ONLY             = 0x10,
    e_SRIO_ERR_INJECT_LANE_1_ONLY             = 0x08,
    e_SRIO_ERR_INJECT_LANE_2_ONLY             = 0x04,
    e_SRIO_ERR_INJECT_LANE_3_ONLY             = 0x02,
    e_SRIO_ERR_INJECT_ALL_4_LANES             = 0x1E,
    e_SRIO_ERR_INJECT_RANDOM_OVER_ALL_4_LANES = 0x1F
} e_SrioErrInjectControl;

/**************************************************************************//**
 @Description  ATMU Window target interface.
 
               If this field is set to anything other than local address space,
               the attributes for the transaction must be assigned in a
               corresponding outbound window at the target.
*//***************************************************************************/
typedef enum e_SrioTargetInterface
{
    e_SRIO_PCI_INTERFACE = 0x0,
    e_SRIO_PCIE          = 0x2,
    e_SRIO_LOCAL_MEM     = 0xF
} e_SrioTargetInterface;

/**************************************************************************//**
 @Description  Supported read transaction types for ATMU inbound window.
*//***************************************************************************/
typedef enum e_SrioInWinReadType
{
    e_SRIO_IN_WIN_READ_SNOOP_DISABLE        = 0x4,
    e_SRIO_IN_WIN_READ_SNOOP_ENABLE         = 0x5,
    e_SRIO_IN_WIN_READ_UNLOCK_L2_CACHE_LINE = 0x7
} e_SrioInWinReadType;

/**************************************************************************//**
 @Description  Supported write transaction types for ATMU inbound window.
*//***************************************************************************/
typedef enum e_SrioInWinWriteType
{
    e_SRIO_IN_WIN_WRITE_SNOOP_DISABLE             = 0x4,
    e_SRIO_IN_WIN_WRITE_SNOOP_ENABLE              = 0x5,
    e_SRIO_IN_WIN_WRITE_ALLOC_CACHE_LINE          = 0x6,
    e_SRIO_IN_WIN_WRITE_ALLOC_AND_LOCK_CACHE_LINE = 0x7
} e_SrioInWinWriteType;

/**************************************************************************//**
 @Description   Mailbox queue / outbound queue status
 *//***************************************************************************/
typedef enum e_SrioQueueStatus
{
    e_SRIO_QUEUE_FULL,
    e_SRIO_QUEUE_NOT_FULL
} e_SrioQueueStatus;

/**************************************************************************//**
 @Description   User callback function on message reception.
 
 @Param[in]     h_App        - Handle to the user application structure
 @Param[in]     mailboxId    - ID of the receiving mailbox
 @Param[in]     p_Data       - Pointer to the received data
 @Param[in]     h_BufContext - The user's private context associated with
                               the given data buffer
 @Param[in]     queueStatus  - Mailbox queue status after reception.
*//***************************************************************************/
typedef e_RxStoreResponse (t_SrioRxStore)(t_Handle          h_App,
                                          uint8_t           mailboxId,
                                          uint8_t           *p_Data,
                                          t_Handle          h_BufContext,
                                          e_SrioQueueStatus queueStatus);
 
/**************************************************************************//**
 @Description   User callback function on doorbell reception.
 
 @Param[in]     h_App       - Handle to the user application structure
 @Param[in]     sourceId    - Device ID of the sender
 @Param[in]     targetId    - Device ID of the receiver
 @Param[in]     data        - The received data
 @Param[in]     queueStatus - Doorbell queue status after reception
*//***************************************************************************/
typedef e_RxStoreResponse (t_SrioDoorbellRxStore)(t_Handle          h_App,
                                                  uint16_t          sourceId,
                                                  uint16_t          targetId,
                                                  uint16_t          data,
                                                  e_SrioQueueStatus queueStatus);

/**************************************************************************//**
 @Description   User callback function on port-write reception.
 
 @Param[in]     h_App  - Handle to the user application structure
 @Param[in]     p_Data - Pointer to the received data
*//***************************************************************************/
typedef void (t_SrioPortWriteCallback)(t_Handle h_App,
                                       uint8_t  *p_Data);

/**************************************************************************//**
 @Description   User callback function on error.
 
 @Param[in]     h_App          - Handle to the user application structure
 @Param[in]     srioErrOrigin  - Description of the object where the error occured
 @Param[in]     srioErrRegMask - Value of the relevant register describing the error.
*//***************************************************************************/
typedef void (t_SrioErrCallback)(t_Handle        h_App,
                                 e_SrioErrOrigin srioErrOrigin,
                                 uint32_t        srioErrRegValue);

/**************************************************************************//**
 @Description   Parameters of a protocol defined by the application.
*//***************************************************************************/
typedef struct t_SrioQeProtocolParams
{
    uint8_t     numOfQueues;                    /**< Number of Rx queues (Bd Rings) to hold
                                                     messages of the appropriate mailbox */
    uint32_t    lenFieldOffset;                 /**< Offset from the beggining of a received
                                                     frame to the leght field            */
    uint32_t    queueNumFieldOffset;            /**< Offset from the beggining of a received
                                                     frame to the RxQueue number field   */
} t_SrioQeProtocolParams;

/**************************************************************************//**
 @Description   Parameters for initialization of the QE mode firmware and for
                the driver's runtime phase when working in QE mode.
*//***************************************************************************/
typedef struct t_SrioQeModeParams
{
    t_Handle                h_Qe;               /**< Handle to the QE structure */
    uint8_t                 pramMemId;          /**< Parameter RAM partition ID */
    uint16_t                rxQueueSize;        /**< Rx bd ring size */
    t_BufferPoolInfo        bufferPoolInfo;     /**< Data buffer pool information */
    t_SrioQeProtocolParams  *p_ProtocolParams;  /**< Parameters of a protocol defined by the application */
} t_SrioQeModeParams;

/**************************************************************************//**
 @Description   User-defined parameters for SRIO Mailbox
*//***************************************************************************/
typedef struct t_SrioMailboxParam
{
    uint16_t    mailboxSize;            /**< Max number of message to be placed in the mailbox      */
    uint16_t    maxMsgSize;             /**< Max message size to be received by the mailbox         */
    uint8_t     memId;                  /**< Memory partition ID for allocating the mailbox queue   */
    uint32_t    memAttributes;          /**< Memory attributes for the mailbox queue                */
    int         irq;                    /**< Mailbox Rx interrupt ID (system-specific number)       */
} t_SrioMailboxParam;

/**************************************************************************//**
 @Description   User-defined parameters for SRIO Outbound Queue
*//***************************************************************************/
typedef struct t_SrioOutQueueParam
{
    uint16_t    queueSize;              /**< Max number of messages to be placed in the outbound queue  */
    uint8_t     memId;                  /**< Memory partition ID for allocating the outbound queue      */
    uint32_t    memAttributes;          /**< Memory attributes for the outbound queue                   */
} t_SrioOutQueueParam;

/**************************************************************************//**
 @Description   User-defined parameters for SRIO Inbound Doorbell
*//***************************************************************************/
typedef struct t_SrioInDoorbellParam
{
    uint16_t    queueSize;              /**< Max number of message to be placed in the inbound doorbell queue   */
    uint8_t     memId;                  /**< Memory partition ID for allocating the inbound doorbell queue      */
    uint32_t    memAttributes;          /**< Memory attributes for the inbound doorbell queue                   */
    int         irq;                    /**< Inbound doorbell Rx interrupt ID (system-specific number)          */
} t_SrioInDoorbellParam;

/**************************************************************************//**
 @Description   User-defined parameters for SRIO Inbound Port-Write
*//***************************************************************************/
typedef struct t_SrioInPortWriteParam
{
    uint8_t     memId;                  /**< Memory partition ID for allocating the inbound port-write queue    */
    uint32_t    memAttributes;          /**< Memory attributes for the inbound port-write queue                 */
} t_SrioInPortWriteParam;

/**************************************************************************//**
 @Description  Attributes of an ATMU inbound window.
*//***************************************************************************/
typedef struct
{
    bool                   isWriteProtected;  /**< Indicates that this window cannot be written to.  */
    e_SrioTargetInterface  targetInterface;   /**< If not set to local address space, the attributes
                                                   for the transaction must be assigned in a
                                                   corresponding out window at the target.           */
    e_SrioInWinReadType    transReadType;     /**< Transaction type to run on the I/O interface or
                                                   the local memory if access is a read.             */
    e_SrioInWinWriteType   transWriteType;    /**< Transaction type to run on the I/O interface or
                                                   the local memory if access is a write.            */
    uint64_t               windowSize;        /**< Must be a power of 2, from 4Kb up to 16Gb.        */
 
    uint64_t               baseAddr;          /**< 34-bit base window address on RapidIO   */
    uint64_t               transAddr;         /**< 36-bit on-chip translation address      */
 
} t_SrioInWinParams;

/**************************************************************************//**
 @Description  Attributes of an ATMU outbound window segment.
*//***************************************************************************/
typedef struct
{
    uint16_t                targetDeviceId;    /**< Device Id of this window's target device.         */
    e_RioTransFlowLevel     transFlowLevel;    /**< Priority of transaction request flow.             */
    e_RioIoOpReadType       transReadType;     /**< Transaction type to run on the I/O interface or
                                                    the local memory if access is a read.             */
    e_RioIoOpWriteType      transWriteType;    /**< Transaction type to run on the I/O interface or
                                                    the local memory if access is a write.            */
} t_SrioOutWinSegParams;

/**************************************************************************//**
 @Description   Attributes of an ATMU outbound window.
*//***************************************************************************/
typedef struct
{
    bool                    pciWindow;         /**< If TRUE, this window follows PCI ordering rules.             */
    uint8_t                 numOfSegs;         /**< Number of segments for this window, can equal 1,2 or 4.      */
    uint8_t                 numOfSubSegs;      /**< Number of subsegments for this segment, can equal 1,2,4 or 8.*/
    uint64_t                windowSize;        /**< Must be a power of 2, from 4Kb up to 16Gb.                   */
    t_SrioOutWinSegParams   *p_SegParams;      /**< A list of parameters for this window's segments              */
} t_SrioOutWinParams;

/**************************************************************************//**
 @Description   User-defined parameters for SRIO Port
 
                Passed by the user to SRIO_PortConfig() routine.
*//***************************************************************************/
typedef struct t_SrioPortParam
{
    /* SRIO port basic parameters */
    uint8_t             portId;             /**< ID of the physical port to be initialized */
    t_Handle            h_SrioMU;           /**< Handle to the SRIO messaging unit.
                                                 SRIO MU is optional to SRIO port operation.
                                                 If SRIO MU was not initialized, the port will
                                                 support ATMU operations only.             */
    /* System integration parameters */
    uint32_t            baseAddress;        /**< Base address of SRIO port registers.      */
    int                 irq;                /**< Errors and port-write  interrupt ID
                                                 (system-specific number)                  */
    uint32_t            localAccessWinAddr; /**< Base address of SRIO local access window  */
    uint32_t            localAccessWinSize; /**< Size of SRIO local access window          */

    /* Application interface */
    t_Handle            h_App;              /**< Handle to the application                 */
    t_SrioErrCallback   *f_ErrCallback;     /**< User callback on error reception          */

} t_SrioPortParam;
 
/**************************************************************************//**
 @Description   User-defined parameters for SRIO Messaging Unit
 
                Passed by the user to SRIO_MU_Config() routine.
*//***************************************************************************/
typedef struct t_SrioMUParam
{
    /* SRIO message unit basic parameters */
    t_SrioMailboxParam      *p_MailboxParam[SRIO_NUM_OF_MAILBOXES];   /**< Mailbox user-defined parameters               */
    t_SrioOutQueueParam     *p_OutQueueParam[SRIO_NUM_OF_OUT_QUEUES]; /**< Outbound Queue user-defined parameters        */
    t_SrioInDoorbellParam   *p_InDoorbellParam;                       /**< Inbound Doorbell user-defined parameters      */
    bool                    outDoorbellInUse;                         /**< TRUE will create an outbound doorbell object  */
    t_SrioInPortWriteParam  *p_InPortWriteParam;                      /**< Inbound Port-Write user-defined parameters    */

    /* System integration parameters */
    uint32_t                baseAddress;                              /**< Base address of SRIO message unit registers.  */

    /* Application interface */
    t_Handle                h_App;                                    /**< Handle to the application                     */
    t_SrioRxStore           *f_RxStore;                               /**< User callback on message reception            */
    t_SrioDoorbellRxStore   *f_DoorbellRxStore;                       /**< User callback on doorbell reception           */
    t_SrioErrCallback       *f_ErrCallback;                           /**< User callback on error reception              */
    t_SrioPortWriteCallback *f_PortWriteCallback;                     /**< User callback on port-write reception         */
    t_TxConfFunction        *f_TxConf;                                /**< User confirmation routine                     */

} t_SrioMUParam;

/**************************************************************************//**
 @Collection    SRIO Port Initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_PORT_Config

 @Description   Creates and configures an SRIO port object.

 @Param[in]     p_SrioPortParam - User defined parameters.
 
 @Retval        Handle to SRIO port object : on success
 @Retval        NULL: on error
*//***************************************************************************/
t_Handle SRIO_PORT_Config (t_SrioPortParam *p_SrioPortParam);

/**************************************************************************//**
 @Function      SRIO_MU_Config

 @Description   Creates and configures an SRIO messaging unit object.

 @Param[in]     p_SrioMUParam - User defined parameters.
 
 @Retval        Handle to SRIO messaging unit object : on success
 @Retval        NULL: on error
*//***************************************************************************/
t_Handle SRIO_MU_Config(t_SrioMUParam *p_SrioMUParam);


/**************************************************************************//**
 @Function      SRIO_PORT_Init

 @Description   Initializes SRIO port object.
 
 @Param[in]     h_SrioPort - Handle to SRIO port object
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_Init (t_Handle h_SrioPort);

/**************************************************************************//**
 @Function      SRIO_MU_Init

 @Description   Initializes SRIO messaging unit.
 
                Memory map of the messaging unit and all sub-object will be initialized.
                Initialization routines will be invoked for all objects
                created in configuration stage.
                The initialized objects need to be enabled in order to enable
                transmission / reception.
 
 @Param[in]     h_SrioMU - Handle to SRIO messaging unit object
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_Init(t_Handle h_SrioMU);

/**************************************************************************//**
 @Function      SRIO_PORT_IsDiscovered

 @Description   Returns whether this SRIO port was discovered by system host.
 
 @Param[in]     h_SrioPort - Handle to SRIO port object
 
 @Retval        TRUE  : If the device was discovered by system host
 @Retval        FALSE : otherwise
*//***************************************************************************/
bool SRIO_PORT_IsDiscovered(t_Handle h_SrioPort);

/* @} */

/**************************************************************************//**
 @Group         srio_port_advanced_init SRIO Port Advanced Configuration Unit

 @Description   SRIO port configuration functions used to change default values.
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SRIO Outbound queue service control modes
 
                 States the number of descriptors to be processed from an
                 outbound queue before continuing to the next queue.
                 If service control of one of the queues was set to
                 "Fixed Priority", service control definitions for all other
                 queues are ignored.
*//***************************************************************************/
typedef enum e_SrioOutQServiceControl
{
    e_SRIO_FIXED_PRIORITY        = 0x00,
    e_SRIO_SERVICE_1_DESCRIPTORS,
    e_SRIO_SERVICE_2_DESCRIPTORS,
    e_SRIO_SERVICE_4_DESCRIPTORS,
    e_SRIO_SERVICE_8_DESCRIPTORS,
    e_SRIO_SERVICE_16_DESCRIPTORS,
    e_SRIO_SERVICE_32_DESCRIPTORS,
    e_SRIO_SERVICE_64_DESCRIPTORS
} e_SrioOutQServiceControl;

/**************************************************************************//**
 @Description   SRIO physical layer errors control parameters structure
 
                A structure of parameters that control the reporting of physical
                layer errors.
 *//***************************************************************************/
typedef struct t_SrioPhysErrParams
{
    uint32_t                physLayerErrRateMask;       /**< Mask of errors that will increase error counter        */
    e_SrioErrRateBiasValue  errRateBiasValue;           /**< Error rate bias value                                  */
    e_SrioErrRateCountLimit errRateCountLimit;          /**< Error rate counter limit value                         */
    uint8_t                 errRateFailedThresh;        /**< Threshold value for reporting an error condition due
                                                             to a possibly broken link, can range from 0 upto 0xFF  */
    uint8_t                 errRateDegradedThresh;      /**< The threshold value for reporting an error condition due
                                                             to a degrading link, can range from 0 upto 0xFF.       */
} t_SrioPhysErrParams;

 
/**************************************************************************//**
 @Collection    SRIO Port Advanced Initialization routines
 
                SRIO port configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigLinkTimeout

 @Description   Configures the link time-out value for all ports on a device.
 
                This timeout is for link events such as sending a packet to
                receiving the corresponding acknowledge, etc.
                This value can range from 0 upto 0xFFFFFF.
                [PLTOCCSR_LINK_TIMEOUT_MAX]
 
 @Param[in]     h_SrioPort  - Handle to SRIO port object.
 @Param[in]     linkTimeout - link time-out value
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigLinkTimeout(t_Handle h_SrioPort, uint32_t linkTimeout);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigPortResponseTimeout

 @Description   Configures port response timeout value.
 
                Timeout timer count for all ports on a device. This timeout is
                for sending a request packet to receiving the corresponding
                response packet.
                This value can range from 0 upto 0xFFFFFF.
                [PRTOCCSR_RESPONSE_TIMEOUT_MAX]
 
 @Param[in]     h_SrioPort          - Handle to SRIO port object.
 @Param[in]     portResponseTimeout - port response timeout value
 

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigPortResponseTimeout(t_Handle  h_SrioPort,
                                            uint32_t  portResponseTimeout);

/**************************************************************************//**
 @Function      SRIO_PORT_ForceSingleLane

 @Description   Configure port's width after POR.
                [0x0]
 
 @Param[in]     h_SrioPort - Handle to SRIO port object.
 @Param[in]     laneNum    - New port configuration to override
                             hardware settings.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ForceSingleLane(t_Handle h_SrioPort, e_SrioForceSrioLaneNum laneNum);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigStopOnPortFail

 @Description   Configures port behaviour when the error rate failed threshold
                has been met or exceeded.
                [CCSR_STOP_ON_PORT_FAIL_ENABLE]
 
 @Param[in]     h_SrioPort           - Handle to SRIO port object.
 @Param[in]     enableStopOnPortFail - TRUE of the port will be stopped when error
                                       rate failed threshold has been met,
                                       FALSE otherwise.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigStopOnPortFail(t_Handle h_SrioPort, bool enableStopOnPortFail);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigPacketDrop

 @Description   Configures port behaviour when the error rate failed threshold
                has been met or exceeded.
                [CCSR_PACKET_DROP_ENABLE]
 
 @Param[in]     h_SrioPort       - Handle to SRIO port object.
 @Param[in]     enablePacketDrop - TRUE if packetes will be dropped when error
                                   rate failed threshold has been met,
                                   FALSE otherwise.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigPacketDrop(t_Handle h_SrioPort, bool enablePacketDrop);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigLogTransLayerErrEnable

 @Description   Configures logical/trasport errors that will cause error interrupt
                generation.
                [LTLEDCSR_LOG_TRANS_LAYER_ERR]
 
 @Param[in]     h_SrioPort           - Handle to SRIO port object.
 @Param[in]     logTransLayerErrMask - Mask if errors that will cause error
                                       interrupt generation.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigLogTransLayerErrEnable(t_Handle h_SrioPort,
                                               uint32_t logTransLayerErrMask);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigPhysLayerErrRateEnable

 @Description   Configures monitoring and reporting of physical layer errors.
 
                Also controls the behaviour of the error rate counter and sets
                approprarite thresholds.
                [LTLEDCSR_PHYS_LAYER_ERR]
 
 @Param[in]     h_SrioPort          - Handle to SRIO port object.
 @Param[in]     p_SrioPhysErrParams - Pointer to a structure of parameters that control
                                      the reporting of physical layer errors.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigPhysLayerErrRateEnable(t_Handle             h_SrioPort,
                                               t_SrioPhysErrParams  *p_SrioPhysErrParams);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigLogicalRetryErrThreshold

 @Description   Configure the retry error threshold value for the logical layer.
 
                When the number of consecutive logical retries for a given packet
                is greater than to this value, an error interrupt is generated.
                This value can range from 0 upto 0xFF.
                [LRETCR_LOG_RETRY_ERR_THRESHOLD_MAX]
 
 @Param[in]     h_SrioPort        - Handle to SRIO port object.
 @Param[in]     logRetryErrThresh - logical retry error threshold value
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigLogicalRetryErrThreshold (t_Handle   h_SrioPort,
                                                  uint32_t   logRetryErrThresh);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigPhysicalRetryErrThreshold

 @Description   Configures the retry error threshold for the physical layer.
 
                When the number of consecutive ACK-retries is greater than or
                equal to this value, an error interrupt is generated.
                This value can range from 0 upto 0xFF.
                [PRETCR_PHYS_RETRY_ERR_THRESHOLD_MAX]
 
 @Param[in]     h_SrioPort         - Handle to SRIO port object.
 @Param[in]     physRetryErrThresh - physical retry error threshold value.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigPhysicalRetryErrThreshold (t_Handle   h_SrioPort,
                                                   uint32_t   physRetryErrThresh);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigOutPacketTimeToLive

 @Description   Configures contains the time-to-live count for all ports
 
                This packet time-to-live counter starts when a packet is ready
                to be transmitted. If the packet is not successfully transmitted
                before the timer expires, the packet is discarded.
                This value can range from 0 upto 0xFFFFFF.
                [LOPTTLCR_OUT_PACKET_TIME_TO_LIVE_MAX]
 
 @Param[in]     h_SrioPort - Handle to SRIO port object.
 @Param[in]     timeToLive - maximum time-to-live counter value.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigOutPacketTimeToLive (t_Handle   h_SrioPort,
                                             uint32_t   timeToLive);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigCrcCheck

 @Description   Configures CRC checks.
 
                CRC can be checked on control; symbols only, on packets only
                or on both.
                [PCR_CONTROL_SYMBOL_CRC_CHECK_ENABLE]
 
 @Param[in]     h_SrioPort     - Handle to SRIO port object.
 @Param[in]     enableCrcCheck - CRC check value

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigCrcCheck (t_Handle               h_SrioPort,
                                  e_SrioEnableCrcCheck   enableCrcCheck);

/* @} */

/**************************************************************************//**
 @Collection    SRIO Mailbox Advanced Initialization routines
 
                SRIO Mailbox configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_ConfigMailboxQeMode

 @Description   Enables the use of QE on message reception.
 
                Enables the copying of a received message to a QE Rx queue.
                The application can set firmware parameters, such as maximum
                lenght of the received data, umber of Rx queues to contain
                messages received to a mailbox and the size of these queues.
                The application can also define a protocol for the messages,
                such as each message will include a header with its size and
                destination queue.

 @Param         h_SrioMU           - Handle to SRIO message unit object.
 @Param         mailboxId          - Id of the mailbox to be configured
 @Param         p_SrioQeModeParams - Parameters for initialization of QE mode
                                     firmware
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigMailboxQeMode(t_Handle            h_SrioMU,
                                    uint16_t            mailboxId,
                                    t_SrioQeModeParams  *p_SrioQeModeParams);

/**************************************************************************//**
 @Function      SRIO_MU_ConfigMailboxInQueueThreshold

 @Description   Configure message-in-queue threshold.
 
                Configuring the number of messages to be accumulated in the
                mailbox before Rx interrupt is generated.
                This value can be any power of 2 from 1 upto 1024.
                [1]

 @Param[in]     h_SrioMU            - Handle to SRIO message unit object
 @Param[in]     mailboxId           - Id of the mailbox to be configured.
 @Param[in]     msgInQueueThreshold - Message-in-queue threshold value.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigMailboxInQueueThreshold (t_Handle   h_SrioMU,
                                               uint8_t    mailboxId,
                                               uint32_t   msgInQueueThreshold);

/**************************************************************************//**
 @Function      SRIO_MU_ConfigMailboxRxIntr

 @Description   Configure Rx interrupt.
 
                Controls Rx interrupt generation when message-in-queue threshold
                is reached.
                [TRUE]

 @Param[in]     h_SrioMU     - Handle to SRIO message unit object
 @Param[in]     mailboxId    - Id of the mailbox to be configured.
 @Param[in]     rxIntrEnable - TRUE if Rx interrupt is enabled and the user will be
                               informed, FALSE otherwise.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigMailboxRxIntr (t_Handle   h_SrioMU,
                                     uint8_t    mailboxId,
                                     bool       rxIntrEnable);

/**************************************************************************//**
 @Function      SRIO_MU_ConfigMailboxMaxReportInterval

 @Description   Configure Rx maximum report interval.
 
                Configure the maximum time interval from message reception until
                Rx interrupt is generated, without message-in-queue threshold being
                reached.
                This value can range from 0 upto 0xFFFFFF.
                [IMMIRIR_MAX_REPORT_INTERVAL]

 @Param[in]     h_SrioMU       - Handle to SRIO message unit object
 @Param[in]     mailboxId      - Id of the mailbox to be configured.
 @Param[in]     reportInterval - Maximum report interval.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigMailboxMaxReportInterval (t_Handle   h_SrioMU,
                                                uint8_t    mailboxId,
                                                uint32_t   reportInterval);

/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound queue Advanced Initialization routines
 
                SRIO Outbound queue configuration functions used to change
                default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_ConfigOutQueueServiceControl

 @Description   Configure Outbound Queue service control.
 
                Determines the number of descriptors to process before servicing
                the next outbound queue.
                If one queue was set with "Fixed priority" service control,
                configurations of all other queues are ignored.
                [e_SRIO_FIXED_PRIORITY]

 @Param[in]     h_SrioMU            - Handle to SRIO message unit object
 @Param[in]     outQueueId          - Id of the outbound queue to be configured.
 @Param[in]     queueServiceControl - Outbound Queue service control.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigOutQueueServiceControl (t_Handle                 h_SrioMU,
                                              uint8_t                  outQueueId,
                                              e_SrioOutQServiceControl queueServiceControl);

/**************************************************************************//**
 @Function      SRIO_MU_ConfigOutQueueRetryErrThreshold

 @Description   Configure retry error threshold value.
 
                Configures the number of times the oubound queue will attempt
                to transfer a message before error will be generated.
                This value can range from 0 upto 0xFF.
                [0x000000FF]

 @Param[in]     h_SrioMU          - Handle to SRIO message unit object
 @Param[in]     outQueueId        - Id of the outbound queue to be configured.
 @Param[in]     retryErrThreshold - Retry error threshold value.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigOutQueueRetryErrThreshold (t_Handle h_SrioMU,
                                                 uint8_t  outQueueId,
                                                 uint8_t  retryErrThreshold);
 
/* @} */

/**************************************************************************//**
 @Collection    SRIO Inbound Doorbell queue Advanced Initialization routines
 
                SRIO Inbound Doorbell configuration functions used to change
                default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_ConfigInDoorbellInQueueThreshold

 @Description   Configure doorbell-in-queue threshold.
 
                Configuring the number of doorbells to be accumulated in the
                inbound doorbell queue before Rx interrupt is generated.
                This value can be any power of 2 from 1 upto 1024.
                [1]

 @Param[in]     h_SrioMU             - Handle to SRIO message unit object
 @Param[in]     drblInQueueThreshold - Doorbell-in-queue threshold value.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigInDoorbellInQueueThreshold (t_Handle   h_SrioMU,
                                                  uint32_t   drblInQueueThreshold);

/**************************************************************************//**
 @Function      SRIO_MU_ConfigInDoorbellRxIntr

 @Description   Configure Rx interrupt.
 
                Controls Rx interrupt generation when doorbell-in-queue threshold
                is reached.
                [TRUE]

 @Param[in]     h_SrioMU     - Handle to SRIO message unit object
 @Param[in]     rxIntrEnable - TRUE if Rx interrupt is enabled and the user will be
                               informed, FALSE otherwise.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigInDoorbellRxIntr(t_Handle h_SrioMU, bool rxIntrEnable);

/**************************************************************************//**
 @Function      SRIO_MU_ConfigInDoorbellMaxReportInterval

 @Description   Configure Rx maximum report interval.
 
                Configure the maximum time interval from doorbell reception until
                Rx interrupt is generated, without doorbell-in-queue threshold
                being reached.
                This value can range from 0 upto 0xFFFFFF.
                [0x00FFFFFF]

 @Param[in]     h_SrioMU          - Handle to SRIO message unit object
 @Param[in]     maxReportInterval - Maximum report interval.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigInDoorbellMaxReportInterval(t_Handle   h_SrioMU,
                                                  uint32_t   maxReportInterval);

/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound Doorbell queue Advanced Initialization routines
 
                SRIO Outbound Doorbell configuration functions used to change
                default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_ConfigOutDoorbellRetryErrThreshold

 @Description   Configure retry error threshold value.
 
                Configures the number of times the oubound doorbell will attempt
                to transfer a doorbell before error will be generated.
                This value can range from 0 upto 0xFF.
                [ODRETCR_RETRY_ERR_THRESHOLD_MAX]

 @Param[in]     h_SrioMU          - Handle to SRIO message object
 @Param[in]     retryErrThreshold - Retry error threshold value.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_ConfigOutDoorbellRetryErrThreshold (t_Handle   h_SrioMU,
                                                    uint8_t    retryErrThreshold);
 
/* @} */

/** @} */ /* end of srio_port_advanced_init group */

/** @} */ /* end of srio_port_init group */

/**************************************************************************//**
 @Group         srio_port_runtime SRIO Port Runtime Unit

 @Description   SRIO port runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    Doorbell unit definitions

 @{
*//***************************************************************************/
#define SRIO_SIZE_OF_DOORBELL_DATA_IN_BYTES       2
/* @} */

/**************************************************************************//**
 @Collection    Inbound Port-Write unit definitions

 @{
*//***************************************************************************/
#define SRIO_IN_PORT_WRITE_MSG_SIZE  64  /**< Inbound Port-Write message size in bytes */
/* @} */

/**************************************************************************//**
 @Description  SRIO parameters for message multicasting.
 
                A multicast group in combination with the multicast devices
                mask indicate which deviceIDs are targets of a multicast operation.
                To enable multicasting, a non-NULL multicast parameters structure
                must be passed in a relevant message descriptor.
*//***************************************************************************/
typedef struct t_SrioMulticastParams
{
    uint16_t multicastGroup;        /**< The 3 msb bits of the target deviceIDs
                                         when working in small transport system,
                                         The 11 msb bits of the target deviceIDs
                                         when working in large transport system. */
    uint32_t multicastDevicesMask;  /**< Setting bit n corresponds to multicasting
                                         to a device with deviceId n in group
                                         defined by multicastGroup value.
                                         If none of the bits are set, bit 0 is
                                         assumed to be set. */
} t_SrioMulticastParams;

/**************************************************************************//**
 @Description   SRIO User-defined parameters for an outgoing message
*//***************************************************************************/
typedef struct t_SrioMsgDesc
{
    uint64_t              dataAddr;           /**< Pointer to data to be sent, should be aligned to 8 bytes */
    uint32_t              dataSize;           /**< Size of the data to be sent        */
    uint16_t              targetDeviceId;     /**< DeviceId of the target device      */
    uint8_t               targetMailboxId;    /**< Target mailbox Id                  */
    t_SrioMulticastParams *p_MulticastParams; /**< Pointer to multicast parameters structure     */
    e_RioTransFlowLevel   transFlowLevel;     /**< Transaction request flow level     */
    uint8_t               portId;             /**< Specifies from which port on the device the message will be transmitted */
} t_SrioMsgDesc;

/**************************************************************************//**
 @Description   SRIO descriptor for outbound doorbell to be sent
*//***************************************************************************/
typedef struct t_SrioDoorbellDesc
{
    uint16_t              destDeviceId;       /**< DeviceId of the target device      */
    e_RioTransFlowLevel   transFlowLevel;     /**< Transaction request flow level     */
    uint16_t              data;               /**< 16-bit data to be sent             */
    uint8_t               portId;             /**< Specifies from which port on the device the doorbell will be transmitted */
} t_SrioDoorbellDesc;
 
/**************************************************************************//**
 @Collection    SRIO Mailbox Runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_MailboxRx

 @Description   This routine handles messages received to a specified mailbox.
                This routine is called by the user when working in Polling mode.
 

 @Param[in]     h_SrioPort - Handle to SRIO port object
 @Param[in]     mailboxId  - Id of relevant the mailbox.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_MailboxRx (t_Handle h_SrioPort, uint8_t mailboxId);

/**************************************************************************//**
 @Function      SRIO_MU_MailboxQueueRx

 @Description   This routine handles messages received to a specific RxQueue of
                the specified mailbox.
                This routine is called by the user when working in Polling mode.
 

 @Param[in]     h_SrioPort - Handle to SRIO port object
 @Param[in]     mailboxId  - Id of relevant the mailbox.
 @Param[in]     queueNum   - Id of the RxQueue of the specified mailbox.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      QE mode must have been enabled for the appropriate mailbox at
                advanced configuration stage.
*//***************************************************************************/
t_Error SRIO_MU_MailboxQueueRx(t_Handle h_SrioPort, uint8_t mailboxId, uint16_t queueNum);

/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound Queue Runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_SendMessage

 @Description   Send a message represented by the given descriptor.
 
                This routine is called by the user on transmission of a message.
                Message parameters like the data to be sent, target device id, etc
                are set in message descriptor by the user.
                If currently the outbound queue is full, this descriptor will
                not be added to the queue and will not be sent. Error indication
                will be returned and error message will be displayed.

 @Param[in]     h_SrioPort   - Handle to SRIO port object.
 @Param[in]     outQueueId   - Id of the outbound queue.
 @Param[in]     msgDesc      - Descriptor of the message to be sent.
 @Param[in]     h_BufContext - User's context for the given data buffer
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      Multicasting available only for messages sized upto 256 bytes
*//***************************************************************************/
t_Error SRIO_MU_SendMessage (t_Handle        h_SrioPort,
                             uint8_t         outQueueId,
                             t_SrioMsgDesc   msgDesc,
                             t_Handle        h_BufContext);

/**************************************************************************//**
 @Function      SRIO_MU_TxConf

 @Description   SRIO port routine for transmit confirmation.
 
                This routine returns transmitted buffers to the application.
                Called by the driver during transmission.

 @Param[in]     h_SrioPort - Handle to SRIO port object
 @Param[in]     outQueueId - Id of the outbound queue.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
void    SRIO_MU_TxConf(t_Handle          h_SrioPort,
                       uint8_t           outQueueId);

/* @} */

/**************************************************************************//**
 @Collection    SRIO Inbound Doorbell Runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_DoorbellRx

 @Description   This routine is called by the user when working in Polling mode.
                Handles doorbell received to the inbound doorbell unit.

 @Param[in]     h_SrioPort - Handle to SRIO port object

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_DoorbellRx (t_Handle h_SrioPort);

/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound Doorbell Runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_SendDoorbell

 @Description   Send a doorbell represented by the given descriptor.
 
                This routine is called by the user on transmission of a doorbell.
                Doorbell parameters like the data to be sent, target device id, etc
                are set in the doorbell descriptor by the user.

 @Param[in]     h_SrioPort   - Handle to SRIO port object
 @Param[in]     doorbellDesc - Descriptor of the doorbell to be sent.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_SendDoorbell(t_Handle              h_SrioPort,
                             t_SrioDoorbellDesc    doorbellDesc);
 
/* @} */

/** @} */ /* end of srio_port_runtime group */

/**************************************************************************//**
 @Group         srio_port_control SRIO Port Control Unit

 @Description   SRIO port control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    SRIO set operations that can be issued by a proccessing element
 
                Mask for source operation capability value, passed to the user
                in t_SrioDeviceDesc. A proccessing element can be the source of
                these operations.

 @{
*//***************************************************************************/
#define SRIO_SOURCE_OPER_READ                   0x80000000  /**< Read logical I/O operation */
#define SRIO_SOURCE_OPER_IREAD                  0x40000000  /**< IRead logical I/O operation */
#define SRIO_SOURCE_OPER_READ_TO_OWN            0x20000000  /**< Read_To_Own logical I/O operation */
#define SRIO_SOURCE_OPER_DATA_CACHE_INVALID     0x10000000  /**< Data Cache Invalidate logical I/O operation */
#define SRIO_SOURCE_OPER_CASTOUT                0x08000000  /**< Castout logical I/O operation */
#define SRIO_SOURCE_OPER_FLUSH                  0x04000000  /**< Flush logical I/O operation */
#define SRIO_SOURCE_OPER_IO_READ                0x02000000  /**< I/O-Read logical I/O operation */
#define SRIO_SOURCE_OPER_INST_CAHCE_INVALID     0x01000000  /**< Instruction Cache Invalidate logical I/O operation */
#define SRIO_SOURCE_OPER_TLBIE                  0x00800000  /**< TLBIE logical I/O operation */
#define SRIO_SOURCE_OPER_TLBSYNC                0x00400000  /**< TLBSYNC logical I/O operation */
/* @} */

/**************************************************************************//**
 @Collection    SRIO set operations that can be supported by a proccessing element.
 
                Mask for destination operation capability value, passed to the
                user in t_SrioDeviceDesc. A proccessing element can be the
                target of these operations.
 @{
*//***************************************************************************/
#define SRIO_DEST_OPER_READ                   0x80000000  /**< Read logical I/O operation */
#define SRIO_DEST_OPER_IREAD                  0x40000000  /**< IRead logical I/O operation */
#define SRIO_DEST_OPER_READ_TO_OWN            0x20000000  /**< Read_To_Own logical I/O operation */
#define SRIO_DEST_OPER_DATA_CACHE_INVALID     0x10000000  /**< Data Cache Invalidate logical I/O operation */
#define SRIO_DEST_OPER_CASTOUT                0x08000000  /**< Castout logical I/O operation */
#define SRIO_DEST_OPER_FLUSH                  0x04000000  /**< Flush logical I/O operation */
#define SRIO_DEST_OPER_IO_READ                0x02000000  /**< I/O-Read logical I/O operation */
#define SRIO_DEST_OPER_INST_CAHCE_INVALID     0x01000000  /**< Instruction Cache Invalidate logical I/O operation */
#define SRIO_DEST_OPER_TLBIE                  0x00800000  /**< TLBIE logical I/O operation */
#define SRIO_DEST_OPER_TLBSYNC                0x00400000  /**< TLBSYNC logical I/O operation */
/* @} */

/**************************************************************************//**
 @Collection    SRIO Port Control routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_PortFree

 @Description   Frees all resources of the SRIO port
 
 
 @Param[in]     h_SrioPort - Handle to SRIO port object.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_Free(t_Handle h_SrioPort);

/**************************************************************************//**
 @Function      SRIO_MU_Free

 @Description   Frees all resources of the SRIO messaging unit
 
 
 @Param[in]     h_SrioMU - Handle to SRIO messaging unit object.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_Free(t_Handle h_SrioMU);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigurationRead

 @Description   Issues a maintenance read request to the destination device.
                The target of this operation is always the configuration space of
                the destination device (used to access the RapidIO capability and
                status registers).
 
 @Param[in]     h_SrioPort   - Handle to SRIO port object.
 @Param[in]     destDeviceId - Destination device id.
 @Param[in]     hopCount     - Number of switches between the request initiator
                               and the target device.
 @Param[in]     offset       - Offset from registers map base to requested data.
 @Param[out]    p_Data       - Pointer to store the requested data.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigurationRead(t_Handle    h_SrioPort,
                                    uint16_t    destDeviceId,
                                    uint8_t     hopCount,
                                    uint32_t    offset,
                                    uint32_t    *p_Data);

/**************************************************************************//**
 @Function      SRIO_PORT_ConfigurationWrite

 @Description   Issues a maintenance write request to the destination device.
                The target of this operation is always the configuration space of
                the destination device (used to access the RapidIO capability and
                status registers).
 
 @Param[in]     h_SrioPort   - Handle to SRIO port object.
 @Param[in]     destDeviceId - Destination device id.
 @Param[in]     hopCount     - Number of switches between the request initiator
                               and the target device.
 @Param[in]     offset       - Offset from registers map base to the write operation.
 @Param[in]     data         - Data to be wiritten.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_ConfigurationWrite (t_Handle    h_SrioPort,
                                      uint16_t    destDeviceId,
                                      uint8_t     hopCount,
                                      uint32_t    offset,
                                      uint32_t    data);
 
/**************************************************************************//**
 @Function      SRIO_PORT_GetMyDeviceId

 @Description   Returns self device Id of the device at this SRIO port.
 
 @Param[in]     h_SrioPort - Handle to SRIO port object.
 @Param[out]    p_DeviceId - Local device Id value.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_GetMyDeviceId(t_Handle h_SrioPort, uint16_t *p_DeviceId);

/**************************************************************************//**
 @Function      SRIO_GetPeakErrorRate

 @Description   Returns the peak value attained by the physical layer error
                rate counter.
 
 @Param[in]     h_SrioPort      - Handle to SRIO port object.
 @Param[out]    p_PeakErrorRate - Pointer to the error rate peak value.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_GetPeakErrorRate (t_Handle h_SrioPort, uint8_t *p_PeakErrorRate);

/**************************************************************************//**
 @Function      SRIO_PORT_ModifyPortLockout

 @Description   Lockes or unlocks the port for communication.
 
                When locked, this port is not enabled to issue or receive any
                packets. All received packets return packet-not-accepted control
                symbols to force an error condition to be signaled by the
                sending device.
                [FALSE]
 
 @Param[in]     h_SrioPort - Handle to SRIO port object.
 @Param[in]     enable     - TRUE if the port is to be locked, FALSE otherwise.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      This routine can be used both during initialization phase and
                during runtime.
*//***************************************************************************/
t_Error SRIO_PORT_ModifyPortLockout (t_Handle h_SrioPort, bool enable);

/**************************************************************************//**
 @Function      SRIO_PORT_ModifyExternalConfigBlock

 @Description   Blocks the SRIO port memory map to external configuration.
 
                When enabled, all maintenance requests and accesses will be
                blocked. Reads will return all 0s and writes will be ignored.
                When disabled, any external RapidIO device can access registers.
 
                This routine can be called during port initialization and during
                runtime.
                If called during intialization, the port will be initialized
                accordinglly.
                [FALSE]
 
 @Param[in]     h_SrioPort  - Handle to SRIO port object.
 @Param[in]     accessBlock - TRUE if internal registers will be blocked from any
                              external configuration,
                              FALSE otherwise.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      This routine can be used both during initialization phase and
                during runtime.
*//***************************************************************************/
t_Error SRIO_PORT_ModifyExternalConfigBlock (t_Handle h_SrioPort, bool accessBlock);

/**************************************************************************//**
 @Function      SRIO_PORT_SetAlternateDeviceId

 @Description   Defines alternate device id for this port
 
                When enabled, the port will accept packets sent with this id or
                port's base id value.
                Requests will be generated with this device id value.
                Responses will be generated with the device id of the appropriate
                request - alternate or base device id.
                By default this option is disabled by the driver.
 
 @Param[in]     h_SrioPort     - Handle to SRIO port object.
 @Param[in]     alternateDevId - alternate device id value

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      This routine can be used both during initialization phase and
                during runtime.
*//***************************************************************************/
t_Error SRIO_PORT_SetAlternateDeviceId (t_Handle  h_SrioPort,
                                        uint16_t  alternateDevId);

/**************************************************************************//**
 @Function      SRIO_PORT_ClearAlternateDeviceId

 @Description   Clears alternate device id for this port
 
                Disables reception based on alternate device id. After this
                rouitne is invoked, reception is based on base device id only.
 
 @Param[in]     h_SrioPort - Handle to SRIO port object.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      This routine can be used both during initialization phase and
                during runtime.
*//***************************************************************************/
t_Error SRIO_PORT_ClearAlternateDeviceId (t_Handle  h_SrioPort);

/**************************************************************************//**
 @Function      SRIO_PORT_AcceptAllEnable

 @Description   Enables/Disables Accept All mode.
 
                When enabled, all packets are accepted without checking the target id.
                When disabled, acceptance is based on device id only.
                [FALSE]
 
 @Param[in]     h_SrioPort      - Handle to SRIO port object.
 @Param[in]     enableAcceptAll - TRUE of accept-all mode is enabled, FALSE otherwise.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
 
 @Cautions      This routine can be used both during initialization phase and
                during runtime.
*//***************************************************************************/
t_Error SRIO_PORT_AcceptAllEnable (t_Handle h_SrioPort, bool enableAcceptAll);

#ifdef DEBUG
/**************************************************************************//**
 @Function      SRIO_PORT_SetErrorInjection

 @Description   Enables/disables error injection into the outbound serial
                RapidIO data stream.
                By default this option is disabled by the driver.
 
 @Param[in]     h_SrioPort       - Handle to SRIO port object.
 @Param[in]     errInjectControl - Enables/diasbles error injection.
                                   Determines specific lanes for error injection.
 @Param[in]     errInjectRange   - (This value)*32 determines the max value of
                                   the between errors.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_SetErrorInjection (t_Handle                h_SrioPort,
                                     e_SrioErrInjectControl  errInjectControl,
                                     uint32_t                errInjectRange);
#endif /* DEBUG */
 
/**************************************************************************//**
 @Function      SRIO_PORT_VerifyStatus

 @Description   Verifies the state of the link between this device and its
                link partner.
 
                Checks whether the physical link exists (synchronization and
                alignment with the adjacent device). If so, continues to check
                whether the link partner's port needs to be recovered from an
                error condition due to link reset.

 @Param[in]     h_SrioPort - Handle to SRIO port object.


 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_VerifyStatus(t_Handle h_SrioPort);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      SRIO_PORT_DumpGeneralRegs

 @Description   Dumps SRIO port general registers.

 @Param[in]     h_SrioPort - Handle to SRIO port object

 @Return        None.
*//***************************************************************************/
void SRIO_PORT_DumpGeneralRegs(t_Handle h_SrioPort);

/**************************************************************************//**
 @Function      SRIO_PORT_DumpAtmuRegs

 @Description   Dumps SRIO ATMU unit registers.

 @Param[in]     h_SrioPort - Handle to SRIO port object

 @Return        None.
*//***************************************************************************/
void SRIO_PORT_DumpAtmuRegs(t_Handle h_SrioPort);

/**************************************************************************//**
 @Function      SRIO_MU_DumpRegs

 @Description   Dumps SRIO message units registers.

 @Param[in]     h_SrioMU - Handle to SRIO messaging unit object

 @Return        None.
*//***************************************************************************/
void SRIO_MU_DumpRegs(t_Handle h_SrioMU);
#endif /* (DEBUG_ERRORS > 0) */

/* @} */

/**************************************************************************//**
 @Collection    SRIO Mailbox Control routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_MailboxEnable

 @Description   Enables SRIO Mailbox object for reception.

 @Param[in]     h_SrioMU  - Handle to SRIO message unit object
 @Param[in]     mailboxId - Id of the mailbox to be enabled.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_MailboxEnable (t_Handle h_SrioMU, uint8_t mailboxId);

/**************************************************************************//**
 @Function      SRIO_MU_MailboxDisable

 @Description   Disables reception to SRIO Mailbox object.

 @Param[in]     h_SrioMU  - Handle to SRIO message unit object
 @Param[in]     mailboxId - Id of the mailbox to be disabled.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_MailboxDisable (t_Handle h_SrioMU, uint8_t mailboxId);
/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound Queue Control routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_OutQueueEnable

 @Description   Enables SRIO Outbound Queue object for transmission.

 @Param[in]     h_SrioMU   - Handle to SRIO message unit object
 @Param[in]     outQueueId - Id of the outbound queue to be enabled.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_OutQueueEnable (t_Handle h_SrioMU, uint8_t outQueueId);

/**************************************************************************//**
 @Function      SRIO_MU_OutQueueDisable

 @Description   Disables transmission from SRIO Outbound Queue object.

 @Param[in]     h_SrioMU   - Handle to SRIO message unit object
 @Param[in]     outQueueId - Id of the outbound queue to be disabled.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_OutQueueDisable (t_Handle h_SrioMU, uint8_t outQueueId);

/* @} */

/**************************************************************************//**
 @Collection   SRIO Inbound Doorbell control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_InDoorbellEnable

 @Description   Enables SRIO Inbound Doorbell object for reception.

 @Param[in]     h_SrioMU - Handle to SRIO message unit object

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_InDoorbellEnable (t_Handle h_SrioMU);

/**************************************************************************//**
 @Function      SRIO_MU_InDoorbellDisable

 @Description   Disables reception to SRIO Inbound Doorbell unit.

 @Param[in]     h_SrioMU - Handle to SRIO message unit object

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_InDoorbellDisable (t_Handle h_SrioMU);

/* @} */

/**************************************************************************//**
 @Collection   SRIO Inbound Port-Write control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_MU_InPortWriteEnable

 @Description   Enables SRIO Inbound Port-Write object for reception.

 @Param[in]     h_SrioMU - Handle to SRIO message unit object

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_InPortWriteEnable (t_Handle h_SrioMU);

/**************************************************************************//**
 @Function      SRIO_MU_InPortWriteDisable

 @Description   Disables reception to SRIO Inbound Port-Write object.

 @Param[in]     h_SrioMU - Handle to SRIO message unit object

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_MU_InPortWriteDisable (t_Handle h_SrioMU);

/* @} */

/**************************************************************************//**
 @Collection    SRIO ATMU Windows Control routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SRIO_PORT_CreateOutboundWin

 @Description   Initializes an outbound window.
 
                The driver allocates a window on RapidIO virtual memory space.
                The attributes for the window are assigned by the user. The
                driver allocates window base address and for the system host
                device this address equals to the translation address.
                For agent device a valid translation adress must be passed
                to the routine.
                All addresses will be returned to the user to use on setting
                a proper inbound window.
                The driver will assign a free window Id for this window. The
                user should pass the id as a parameter for setting or getting
                window or window's segments parameters or to get window
                mapping addresses (base and translation addresses).

 @Param[in]     h_SrioPort  - Handle to SRIO port object
 @Param[in]     p_WinParams - Attributes for the window.
 @Param[out]    p_WinId     - Assigned window id.
 @Param[out]    p_BaseAddr  - 36-bit on-chip base window address.
 @Param[in,out] transAddr   - 34-bit translation address on RapidIO assgined
                              to this window.
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_CreateOutboundWin(t_Handle           h_SrioPort,
                                    t_SrioOutWinParams *p_WinParams,
                                    uint8_t            *p_WinId,
                                    uint64_t           *p_BaseAddr,
                                    uint64_t           transAddr);

/**************************************************************************//**
 @Function      SRIO_PORT_FreeOutboundWin

 @Description   Disables the specified outbound window and releases all
                allocated resources.

 @Param[in]     h_SrioPort - Handle to SRIO port object
 @Param[in]     winId      - Window id.
 
 @Retval        None.
*//***************************************************************************/
void SRIO_PORT_FreeOutboundWin(t_Handle h_SrioPort, uint8_t winId);


/**************************************************************************//**
 @Function      SRIO_PORT_CreateInboundWin

 @Description   Initializes an inbound window.
 
                The driver allocates a window on RapidIO virtual memory space.
                The attributes for the window and window translation address are
                assigned by the user.
                For the system host if no base address is given, the driver will
                allocated one on the SRIO virtual space.
                For an agent this address must be passed to the routine.
                The driver will assign a free window Id for this window. The
                user should pass the id as a parameter for setting or getting
                window attributes or to get window mapping addresses (base and
                translation addresses).

 @Param[in]     h_SrioPort  - Handle to SRIO port object
 @Param[in]     p_WinParams - Attributes for the window.
 @Param[out]    p_WinId     - Assigned window id.
 
 
 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error SRIO_PORT_CreateInboundWin (t_Handle          h_SrioPort,
                                    t_SrioInWinParams *p_WinParams,
                                    uint8_t           *p_WinId);
 
/**************************************************************************//**
 @Function      SRIO_PORT_FreeInboundWin

 @Description   Disables the specified inbound window and releases all
                allocated resources.

 @Param[in]     h_SrioPort - Handle to SRIO port object
 @Param[in]     winId      - Window id.
 
 @Retval        None.
*//***************************************************************************/
void SRIO_PORT_FreeInboundWin(t_Handle  h_SrioPort, uint8_t   winId);

/**************************************************************************//**
 @Function      SRIO_PORT_GetSerialLinkStats

 @Description   Prints the status of the port's serial link.

 @Param[in]     h_SrioPort - Handle to SRIO port object
 
 @Retval        None.
*//***************************************************************************/
void SRIO_PORT_GetSerialLinkStats(t_Handle h_SrioPort);

/* @}*/

/** @} */ /* end of srio_port_control group */


/**************************************************************************//**
 @Group         srio_port_exception SRIO Port Exception Unit

 @Description   SRIO port exception functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    SRIO Logical Tansport Layer Error masks

 @{
*//***************************************************************************/
#define SRIO_PORT_IO_ERR_RESPONSE                       0x80000000  /**< Error response received for an IO logical layer request. */
#define SRIO_PORT_MSG_ERR_RESPONSE                      0x40000000  /**< Error response received for a MSG logical layer request. */
#define SRIO_PORT_GSM_ERR_RESPONSE                      0x20000000  /**< Error response received for for a GSM logical layer request. */
#define SRIO_PORT_MSG_FORMAT_ERR                        0x10000000  /**< Received MESSAGE packet data payload with an invalid size or segment. */
#define SRIO_PORT_ILLEGAL_TRANSACTION_DECODE            0x08000000  /**< Received illegal fields in the request/response packet for a supported transaction. */
#define SRIO_PORT_ILLEGAL_TRANSACTION_TARGET_ERR        0x04000000  /**< Received a packet with unknown destination ID. */
#define SRIO_PORT_MSG_REQUEST_TIMEOUT                   0x02000000  /**< A required message request has not been received within the specified time-out interval. */
#define SRIO_PORT_PACKET_RESPONSE_TIMEOUT               0x01000000  /**< A required response has not been received within the specified time out interval. */
#define SRIO_PORT_UNSOLICITED_RESPONSE                  0x00800000  /**< An unexpected response packet was received */
#define SRIO_PORT_UNSUPPORTED_TRANSACTION               0x00400000  /**< Unsupported transaction received. */
#define SRIO_PORT_INBOUND_ATMU_CROSSED_BOUNDARY         0x00000080  /**< Inbound ATMU crossed boundary. */
#define SRIO_PORT_OUTBOUND_ATMU_CROSSED_BOUNDARY        0x00000040  /**< Outbound ATMU crossed boundary. */
#define SRIO_PORT_RETRY_ERR_THRESHOLD_EXCEEDED          0x00000010  /**< The allowed number of logical retries has been exceeded. */
#define SRIO_PORT_TRANS_SIZE_ERR                        0x00000008  /**< Received a packet with a common transport system that is unsupported by this device. */
#define SRIO_PORT_PACKET_TIME_TO_LIVE_ERR               0x00000004  /**< A packet could not be successfully transmitted before the packet time-to-live counter expired. */
/* @} */

/**************************************************************************//**
 @Collection    SRIO Physical Layer error masks

 @{
*//***************************************************************************/
#define SRIO_PORT_IMPLEMENT_ERR                         0x80000000  /**< Physical layer retry error threshold exceeded. */
#define SRIO_PORT_CNTRL_SYMBOL_WITH_BAD_CRC             0x00400000  /**< Received a control symbol with a bad CRC value. */
#define SRIO_PORT_CNTRL_SYMBOL_WITH_UNEXPECTED_ACK_ID   0x00200000  /**< Received acknowledge control symbol with unexpected ackID. */
#define SRIO_PORT_PACKET_NOT_ACCEPTED                   0x00100000  /**< Received packet-not-accepted acknowledge control symbol. */
#define SRIO_PORT_PACKET_WITH_UNEXPECTED_ACK_ID         0x00080000  /**< Received packet with unexpected ackID value. */
#define SRIO_PORT_PACKET_WITH_BAD_CRC                   0x00040000  /**< Received a packet with a bad CRC value.*/
#define SRIO_PORT_PACKET_MAX_SIZE_EXCEED                0x00020000  /**< Received packet which exceed the maximum allowed size (276 bytes).*/
#define SRIO_PORT_NOT_OUTSTANDING_ACKID                 0x00000020  /**< Link-response received with an ackID that is not outstanding. */
#define SRIO_PORT_UNEXPECTED_PACKET_OR_CNTRL_SYMBOL     0x00000010  /**< An unexpected packet or control symbol was received. */
#define SRIO_PORT_UNDEFINED_CODE_GROUP                  0x00000004  /**< Received unaligned /SC/ or /PD/ or undefined code-group. */
#define SRIO_PORT_UNEXPECTED_ACK_CNTRL_SYMBOL           0x00000002  /**< An unexpected acknowledge control symbol was received. */
#define SRIO_PORT_ACK_OR_LINK_RESPONSE_TIMEOUT          0x00000001  /**< An acknowledge or link-response control symbol is not received within the specified time-out interval. */
/* @} */

/**************************************************************************//**
 @Collection    SRIO Physical Layer Error Rate Enable masks

 @{
*//***************************************************************************/
#define SRIO_CORRUPT_CONTROL_SYMBOL_ERR_COUNT_ENABLE    0x00400000   /**< Enable error rate counting of a corrupt control symbol. */
#define SRIO_UNEXPECTED_ACKID_ERR_COUNT_ENABLE          0x00200000   /**< Enable error rate counting of an acknowledge control symbol with an unexpected ackID */
#define SRIO_PACKET_NOT_ACCEPTED_ERR_COUNT_ENABLE       0x00100000   /**< Enable error rate counting of received packet-not-accepted control symbols. */
#define SRIO_UNEXPECTED_ACKID_VALUE_ERR_COUNT_ENABLE    0x00080000   /**< Enable error rate counting of packet with unexpected ackID value. */
#define SRIO_BAD_CRC_VALUE_ERR_COUNT_ENABLE             0x00040000   /**< Enable error rate counting of packet with a bad CRC value. */
#define SRIO_MAX_SIZE_EXCEEDED_ERR_COUNT_ENABLE         0x00020000   /**< Enable error rate counting of packet which exceeds the maximum allowed size */
#define SRIO_UNSOLICITED_LINK_RESPONSE_ERR_COUNT_ENABLE 0x00000020   /**< Enable error rate counting of link-responses received with an ackID that is not outstanding. */
#define SRIO_PROTOCOL_ERR_COUNT_ENABLE                  0x00000010   /**< Enable error rate counting of protocol errors. */
#define SRIO_DELINEATION_ERR_COUNT_ENABLE               0x00000004   /**< Enable error rate counting of delineation errors. */
#define SRIO_UNSOLICITED_ACKNOLEDGE_ERR_COUNT_ENABLE    0x00000002   /**< Enable error rate counting of unsolicited acknowledge control symbol errors.*/
#define SRIO_LINK_TIME_OUT_ERR_COUNT_ENABLE             0x00000001   /**< Enable error rate counting of link time-out errors. */
/* @} */

/**************************************************************************//**
 @Collection    SRIO General Port error masks

 @{
*//***************************************************************************/
#define SRIO_PORT_OUTPUT_PACKET_DROPPED                 0x04000000  /**< Output port has discarded a packet. */
#define SRIO_PORT_OUTPUT_FAILED_COND                    0x02000000  /**< The error rate counter has met or exceeded the ports failed error threshold. */
#define SRIO_PORT_OUTPUT_DEGRADED_COND                  0x01000000  /**< The error rate counter has met or exceeded the ports degraded error threshold. */
#define SRIO_PORT_OUTPUT_RETRY_COND                     0x00100000  /**< Output port has encountered a retry condition. */
#define SRIO_PORT_OUTPUT_PACKET_RETRY_RECEIVED          0x00080000  /**< Output port has received a packet retry control symbol and cannot make forward progress. */
#define SRIO_PORT_OUTPUT_STOP_DUE_TO_RETRY              0x00040000  /**< Output port is stopped due to a retry. */
#define SRIO_PORT_OUTPUT_TX_ERR                         0x00020000  /**< Output port has encountered (and possibly recovered from) a transmission error. */
#define SRIO_PORT_OUTPUT_STOP_DUE_TO_TX_ERR             0x00010000  /**< Output port is stopped due to a transmission error. */
#define SRIO_PORT_INPUT_STOP_DUE_TO_RETRY               0x00000400  /**< Input port is stopped due to a retry. */
#define SRIO_PORT_INPUT_TX_ERR                          0x00000200  /**< Input port has encountered (and possibly recovered from) a transmission error. */
#define SRIO_PORT_INPUT_STOP_DUE_TO_TX_ERR              0x00000100  /**< Input port is stopped due to a transmission error. */
#define SRIO_PORT_INPUT_OR_OUTPUT_UNRCVRBLE_ERR         0x00000004  /**< Input or output port has encountered an error from which hardware was unable to recover. */
#define SRIO_PORT_INPUT_OUTPUT_NOT_INITIALIZED          0x00000001  /**< Input and output ports are not initialized. */
/* @} */

/**************************************************************************//**
 @Collection    SRIO Mailbox error masks

 @{
*//***************************************************************************/
#define SRIO_PORT_MAILBOX_MSG_REQUEST_TIMEOUT           0x00000400  /**< Message request time-out. */
#define SRIO_PORT_MAILBOX_TRANSACTION_ERROR             0x00000080  /**< An internal error condition occurs during the message operation.*/
/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound Queue error masks

 @{
*//***************************************************************************/
#define SRIO_PORT_OUT_QUEUE_MSG_ERR_RESPONSE            0x00001000  /**< Message error response is received from the message target.*/
#define SRIO_PORT_OUT_QUEUE_RETRY_ERR_THRESH_EXCEEDED   0x00000800  /**< The retry error threshold value has been exceeded due to RapidIO retry response.*/
#define SRIO_PORT_OUT_QUEUE_PACKET_RESPONSE_TIMEOUT     0x00000400  /**< The message unit has been unable to complete a message operation and a packet response time-out occurred. */
#define SRIO_PORT_OUT_QUEUE_TRANSACTION_ERR             0x00000080  /**< An internal error condition occurs during the message operation.*/
/* @} */

/**************************************************************************//**
 @Collection    SRIO Inbound Doorbell error mask

 @{
*//***************************************************************************/
#define SRIO_PORT_IN_DOORBELL_TRANSACTION_ERR           0x00000080  /**< An internal error condition occurs during the doorbell operation.*/
/* @} */

/**************************************************************************//**
 @Collection    SRIO Outbound Doorbell error masks

 @{
*//***************************************************************************/
#define SRIO_PORT_OUT_DOORBELL_MSG_ERR_RESPONSE         0x00001000  /**< Message error response is received from the doorbell target.*/
#define SRIO_PORT_OUT_DOORBELL_RETRY_ERR_THRES_EXCEEDED 0x00000800  /**< The retry error threshold value has been exceeded due to RapidIO retry response.*/
#define SRIO_PORT_OUT_DOORBELL_PACKET_RESPONSE_TIMEOUT  0x00000400  /**< The doorbell unit has been unable to complete a doorbell operation and a packet response time-out occurred.*/
/* @} */

/**************************************************************************//**
 @Collection    SRIO Inbound Port-Write error mask

 @{
*//***************************************************************************/
#define SRIO_PORT_IN_PORT_WRITE_TRANSACTION_ERR         0x00000080  /**< An internal error condition occurs during the port-write operation.*/
/* @} */

/** @} */ /* end of srio_port_exception group */

/** @} */ /* end of srio_port_id group */
 
#endif /* __SRIO_EXT_H */
