/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_aal2_ext.h

 @Description   UCC Atm Aal2 external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_ATM_AAL2_EXT_H
#define __UCC_ATM_AAL2_EXT_H

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

#include "ucc_atm_ch_ext.h"
#include "std_ext.h"

/**************************************************************************//**
 @Group         ucc_atm_common_grp UCC ATM Common

 @Description   Ucc Atm Common

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_common_init_grp UCC ATM Common Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_aal2_common_init_grp UCC ATM AAL2 Common Initialization Unit

 @Description   AAL2 Common Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   structure representing ucc atm common aal2 parameters
*//***************************************************************************/
typedef struct t_UccAtmAal2CommonParam
{
    uint32_t            totalCids;              /**< Total CIDs for all the system */
    uint32_t            intTxQd;                /**< the required number of internal TxQDs in the system; TxQD is actually a priority */
    uint32_t            extTxQd;                /**< the required number of external TxQDs in the system; TxQD is actually a priority */
    uint32_t            sarOrOffset32ModeIntRxQd;/**< The required number of SAR internal RxQDs / internal RxQDs that work with offset32 mode, in the system */
    uint32_t            sarOrOffset32ModeExtRxQd;/**< The required number of SAR external RxQDs / external RxQDs that work with offset32 mode, in the system */
    uint32_t            cpsIntRxQd;             /**< the required number of CPS internal RxQDs in the system */
    uint32_t            cpsExtRxQd;             /**< the required number of CPS external RxQDs in the system */
    uint32_t            swAvconIntRxQdType1;    /**< the required number of Switch Auto VcOn internal RxQDs in the system that are working with internal TxQds */
    uint32_t            swAvconExtRxQdType1;    /**< the required number of Switch Auto VcOn external RxQDs in the system that are working with external TxQds */
    uint32_t            swAvconIntRxQdType2;    /**< the required number of Switch Auto VcOn internal RxQDs in the system that are working with internal TxQds(Type2) */
    uint32_t            swAvconExtRxQdType2;    /**< the required number of Switch Auto VcOn external RxQDs in the system that are working with external TxQds(Type2) */
    uint8_t             lastCid;                /**< The last CID number (range between 8 to 255) */
    bool                rxStatistics;           /**< indication if the work is with receive statistics in the system; the rx Statistics which has some influence on the size of the CID Map table */
    bool                txStatistics;           /**< indication if the work is with transmit statistics in the system */
    t_BufferPoolInfo    bufferPoolInfo;         /**< Data buffer pool information */
    t_Handle            h_UccAtmCommon;         /**< Handle to the ATM Common level; this is the output from the call to the UCC_ATM_COMMON_Config() routine */
} t_UccAtmAal2CommonParam;


/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_Config

 @Description   Config the AAL2 structures which are common to all the UCCs

 @Param[in]     p_UccAtmAal2CommonParam - UCC ATM AAL2 Common parameters.

 @Return        handle to the AAL2 Common structure.
*//***************************************************************************/
t_Handle UCC_ATM_AAL2_COMMON_Config(t_UccAtmAal2CommonParam *p_UccAtmAal2CommonParam);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_Init

 @Description   Initialize the AAL2 structure which is common to all the UCCs.

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_Init(t_Handle h_UccAtmAal2Common);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_Free

 @Description   Free the AAL2 structures which are common to all the UCCs.

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_Free(t_Handle h_UccAtmAal2Common);

/** @} */ /* end of ucc_atm_aal2_common_init_grp group */
/** @} */ /* end of ucc_atm_common_init_grp group */
/** @} */ /* end of ucc_atm_common_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal2_channel_grp UCC ATM AAL2 Channel

 @Description   Ucc Atm Aal2 Channel

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_aal2_channel_init_grp UCC ATM AAL2 Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   AAL2 Types.
 *//***************************************************************************/
typedef enum e_UccAtmAal2Type
{
      e_UCC_ATM_AAL2_CPS    = 1 /**< AAL2 CPS Type - relevant for CID/RxQd/TxQd initialization */
    , e_UCC_ATM_AAL2_SAR    = 2 /**< AAL2 SAR Type - relevant for CID/RxQd/TxQd initialization */
    , e_UCC_ATM_AAL2_SWITCH = 3 /**< AAL2 Switch Type - relevant for CID/RxQd/TxQd initialization */
} e_UccAtmAal2Type;

/****************************************************************//**
 @Collection    Queue Management interrupt masks.
 
                The required queue management interrupts
                (relevant only in case of switch WFQ); important to fulfill in queueManagementInterrupts field in #t_UccAtmAal2TxQdParam.
 @{
*//*****************************************************************/
#define AAL2_WFQ_FULL_SPEED_TO_DECELERATE_INTERRUPT       0x20  /**< Full speed to decelerate interrupt mask */
#define AAL2_WFQ_DECELERATE_TO_FULL_STOP_INTERRUPT        0x10  /**< Decelerate to full stop interrupt mask */
#define AAL2_WFQ_ACCELERATE_TO_DECELERATE_INTERRUPT       0x08  /**< accelerate to decelerate interrupt mask */
#define AAL2_WFQ_FULL_STOP_TO_ACCELERATE_INTERRUPT        0x04  /**< Full stop to accelerate interrupt mask */
#define AAL2_WFQ_ACCELERATE_TO_FULL_SPEED_INTERRUPT       0x02  /**< accelerate to full stop interrupt mask */
#define AAL2_WFQ_DECELERATE_TO_ACCELERATE_INTERRUPT       0x01  /**< decelerate to accelerate interrupt mask */

/* @} */

/**************************************************************************//**
 @Description   Max Weighted Fair Queue members.
 *//***************************************************************************/
#define MAX_AAL2_WFQS_NUM         4 /**< the maximum number of WFQ TxQDs under a specific channel */


/**************************************************************************//**
 @Description   Aal2 channel user-defined parameters.
 *//***************************************************************************/
typedef struct t_UccAtmAal2Param
{
    t_UccAtmChannelParam  uccAtmChParam;     /**< ATM channel general driver parameters */
    t_BufferPoolInfo      bufferPoolInfo;    /**< Data buffer pool information */
    uint32_t              dataMemAttributes; /**< Data memory partition attributes  */

    e_CommMode            statistics;        /**< if to work with statistics for receive operation, transmit operation or both of them */
    /*****************/
	/* Tx parameters */
    /*****************/
    bool                  txNoStf;           /**< If True - the cell doesn't include the
                                                  STF byte while transmitted;
                                                  Note: The rxNoStf fields must be configured to be in the same mode in case of full duplex */
    bool                  partFillMode;      /**< If True - partially filled mode is used;
                                                  False - otherwise. */
    uint8_t               partFillThreshold; /**< The cell is sent when there are at least
                                                  PartFillThreshold bytes in the user data
                                                  field; relevant only if Partially filled mode is set. */
    uint8_t               uncompletedCellDelay;
                                             /**< If the channel was scheduled uncompletedCellDelay iterations and
                                                  still the cell is uncompleted, the uncompleted cell is padded with
                                                  zeros and sent; If uncompletedCellDelay==0, this mode is disabled. */
    uint8_t               maxStep;           /**< The number of Tx Queues the transmitter
                                                  will visit for each transmitted cell;
                                                  In Fixed priority mode this parameter should
                                                  be set to the number of Tx queue descriptors
                                                  of the channel. */
    bool                  onePacketFromQueue;/**< If True - only one packet will be transmitted
                                                  from each Tx queue of the channel and then
                                                  advanced to the next queue. */
    bool                  fixedPriority;     /**< If True - the transmitter will visit the
                                                  Tx queues in the Fixed priority mode;
                                                  Otherwise Roun-Robin will be performed. */
    bool                  cidMultiplexing;   /**< If True - CID Multiplexing Mode is used;
                                                  Relevant only for IW AAL2 channels with Auto VC On mode.*/
    uint8_t               cidMultiplexingPrioritiesNum;
                                             /**< Relevant only if CID Multiplexing Mode is used;
                                                  Number of priorities used for theis CID Multiplexing channel. */
	bool                  enableTxStatsInt;  /**< If True - the Statistics interrupts are enabled */
    bool                  externalTxQd;      /**< If True - the TxQD structure reside in
                                                  the external memory else in the internal memory */
    /*****************/
	/* Rx parameters */
    /*****************/
    bool                  rxNoStf;           /**< If true, the cell doesn't include the
                                                  STF byte while received;
                                                  Note: The txNoStf fields must be configured to be
                                                  in the same mode in case of full duplex */
	bool                  enableRxStatsInt;  /**< If True - the Rx Statistics interrupts
	                                              are enabled */
	bool                  enableCidMask;     /**< If True - the CID Mask feature is enabled */
	uint8_t               cidMask;           /**< The value of CID Mask - relevant only
	                                              if CID Mask mode is enabled */

    uint8_t               maxCpsSduLength;   /**< Maximum CPS SDU length (ITU-T I-363-2)
                                                  the correct values are 45 or 64;
                                                  Note: In case of SSSAR, this parameter should be
                                                  equal or greater than the sarSegLen of the Transmit side */

    bool                  rxQdOffset32;      /**< If this mode is set, the offset between rxQds is multipled
                                                  by 32 and not 4 as usual, which enables up to 64K SAR RxQDs
                                                  to be used instead of 8K SAR RxQDs in the usual mode */

    /*********************/
    /* Switch parameters */
    /*********************/
    uint8_t               tbnrTimeOutCnt;    /**< Describes the amount of attempts the
                                                  transmitter tries to transmit a packet
                                                  on switch BD which is currently marked
                                                  as partially filled */
	bool                  enablePprs;        /**< indicates the PPRS (Partial Packet Receive Storage) mode */
    e_CommMode            wfqPriority;       /**< Is this channel is configured to work in Wfq mode
                                                  only for receive, only for Transmit or both of them */
    uint16_t              overloadLimitSlots_MicSec;/**<  the maximum time that behind it the
                                                  scheduler would not send any best effort cell;
                                                  If the channel is scheduled by the APC, the value is in slots,
                                                  If the channel is scheduled by the GCRA, the value is in micro seconds */
    uint16_t              winc[MAX_AAL2_WFQS_NUM];/**< increment value ,0 - means fixed priority, all the inc are the same means round robin,
                                                       otherwise give different values (There is no limitation that the sum of all the Winc is 100%)
                                                       Note: A smaller increment means higher prioritization */
    uint8_t               numOfWfqs;         /**< How many weighted fair queues are related to the
                                                  specific channel ; range between 1 to 4 */
} t_UccAtmAal2Param;

/**************************************************************************//**
 @Description   Aal2 TxQd user-defined parameters.
 *//***************************************************************************/
typedef struct t_UccAtmAal2TxQdParam
{
    t_Handle           h_UccAtmAal2;            /**< AAL2 channel that the specific TxQd belongs to */
    e_UccAtmAal2Type   txQdType;                /**< AAL2 type (CPS, SSSAR, SWITCH) */
    bool               enableTbnrInterrupts;    /**< Enable or mask Buffer Not Ready interrupts on this TxQD;
                                                     relevant for SAR and CPS */
    uint16_t           bdRingLength;            /**< BD ring length */

    /*********************************************/
    /* CPS/Switch Tx Queue descriptor parameters */
    /*********************************************/

    bool               countPackD;              /**< Relevant only for switch mode; If true ,the field Number of packet in queue counts the discard packet; Relevant only in switch Type-1 */

    /*******************************/
    /* WFQ(Switch) mode parameters */
    /*******************************/
    uint8_t            thresholdSet;            /**< which threshold set to use between 0-255 */
    uint8_t            queueManagementInterrupts;/**< enable the required interrupts - according to the defines like :
                                                     #AAL2_WFQ_FULL_SPEED_TO_DECELERATE_INTERRUPT
                                                     NOTE:The interrupts can be ored and the user
                                                     can ask one or more of the list */
    bool               realTimeTraffic;         /**< indicates if this TxQD contains a real
                                                     time traffic or best effort */
    bool               thresholdEnable;         /**< indicates if to enable the queue management */

    /****************************************/
    /* SSSAR Tx Queue descriptor parameters */
    /****************************************/
    bool               uuiMode;                 /**< For SSSAR Tx queue descriptor:if true, the UUI information is set at the end of the SSSAR frame
                                                     Note:The user responsibility to allocate one more byte for the last buffer in order
                                                     to have space for the UUI value */
    uint8_t            sarSegLen;               /**< For SSSAR Tx queue descriptor:
                                                     Maximum length of SSSAR PDU; If
                                                     NoStf=1, this parameter have to be 45
                                                     Note: This parameter can't be more than Mrblr and
                                                     more than maxCpsSduLength (in the receiver side) */
} t_UccAtmAal2TxQdParam;

/**************************************************************************//**
 @Description   Aal2 RxQd user-defined parameters.
 *//***************************************************************************/
typedef struct t_UccAtmAal2RxQdParam
{
    /* NOTE: The type of the rxqd is the same as the type of cid */
    bool              externalRxQd;        /**< TRUE if RxQD is external, FALSE otherwise */
    uint32_t          intrManagerId;       /**< Id of queue for messages
                                                from the driver to the application layer */
    bool              rxInterrupts;        /**< working with Rx interrupts */
    uint16_t          bdRingLength;        /**< BD ring length */
 
    /*******************************************************************************/
    /* SWITCH Rx Queue descriptor parameters (in addition to the above parameters) */
    /*******************************************************************************/
    uint8_t           txCid;              /**< Translation CID for switching. */
    bool              partPacketDiscard;  /**< If true, packet will be discarded
                                               due to buffer not ready event. */
    t_Handle          h_SwitchTxQd;       /**< Handler of the Switch TxQD that the RxQd work with;
                                               The pointer is returned by the driver
                                               after TxQD initialization. */
    bool              rxQdType2;          /**< indicates if the specific rxQd contains a pointer which
                                               resides in the external memory (external TxQd) */
 
    bool              txVconMode;         /**< TEMPORARY - for now please ignore;
                                               enable the auto vc on in AAL2 switch mode */

    /******************************************************************************/
    /* SSSAR Rx Queue descriptor parameters (in addition to the above parameters) */
    /******************************************************************************/
    bool              rasTimerEnable;     /**< If true, the RAS timer is enabled */
    uint16_t          mrblr;              /**< Maximum received buffer length
                                             ( for each Rx BD );
                                               Note:In case of SSSAR, this parameter
                                               should be equal or greater than the sarSegLen
                                               parameter in the Tx side */
    uint16_t          maxSarSduLength;    /**< Maximum SSSAR SDU length. */
} t_UccAtmAal2RxQdParam;

/**************************************************************************//*
 @Description   AAL2 Channel Interworking Parameters.
 
                This structure is relevant only for Aal5 IW channels.
*//***************************************************************************/
typedef struct t_UccAtmAal2IwParam
{
    e_CommMode  iwCommMode;
    bool        rxFastPass;
    uint8_t     rxFilterIndex;
    uint16_t    rxCodeKey;
	
} t_UccAtmAal2IwParam;

#define IW_AAL2_CID_MUX_PRI_1   0
#define IW_AAL2_CID_MUX_PRI_2   1
#define IW_AAL2_CID_MUX_PRI_3   2
#define IW_AAL2_CID_MUX_PRI_4   3


/**************************************************************************//**
 @Description   Aal2 Cid user-defined parameters.
 *//***************************************************************************/
typedef struct t_UccAtmAal2CidParam
{
    uint8_t                 cidNumber;      /**< Valid CID number (8-255) */
    t_Handle                h_App;          /**< Application layer handler of the CID */
    t_Handle                h_UccAtmAal2;   /**< Driver's handler of the AAL2 channel */
    e_UccAtmAal2Type        cidType;        /**< AAL2 type (CPS, SSSAR, SWITCH) */
    e_CommMode              commMode;    	/**< CID communication mode */

    /*****************/
    /* Tx Parameters */
    /*****************/
    t_Handle                h_TxQd;         /**< Driver's handler of the Tx queue
                                                 descriptor table; If NULL,
                                                 the CID will not be activated to transmit.*/
    t_TxConfFunction        *f_TxConf;      /**< Pointer to application layer's routine -
                                                 called to handle transmitted frames/packets/datas.*/

    uint8_t                 cidMuxPriority; /**< IW AAL2 CID Multiplexing Priority (0..3);
                                                 Relevant only if the relative channel works in CID Multiplexing mode;
                                                 In addition, relevant only if Single Priority was not requested in the channel level. */

    /*****************/
    /* Rx Parameters */
    /*****************/
    t_RxStoreFunction       *f_RxStore;     /**< Pointer to application layer's routine - called to
                                                 pass a received frame/packet/data to the application. */
    t_UccAtmAal2RxQdParam   *p_UccAtmAal2RxQdParam;
                                            /**< Structure that holds the required
                                                 parameters for RxQD initialization
                                                 NOTE:the structure needs to be fulfilled
                                                 only if the CID initialized to receive data */
    t_UccAtmAal2IwParam     *p_IwParams;    /**< Pointer to IW parameters structure;
                                                 NULL for non-IW mode */
} t_UccAtmAal2CidParam;

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_ConfigAndInit

 @Description   Initializes driver and hardware for an AAL2 ATM channel.
 
 @Param[in]     p_UccAtmAal2Param - AAL2 channel user defined parameters.

 @Return        Handle to created channel if success. NULL otherwise.
*//***************************************************************************/
t_Handle UCC_ATM_AAL2_ConfigAndInit(t_UccAtmAal2Param *p_UccAtmAal2Param);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_Free

 @Description   Free memory of the AAL2 ATM channel handler.

 @Param[in]     h_UccAtmAal2 - AAL2 channel handler to be freed.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccAtmAal2 must have been previously initialized using UCC_ATM_AAL2_ConfigAndInit().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_Free(t_Handle h_UccAtmAal2);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_ConfigAndInitTxQd

 @Description   Tx Queue Descriptor configuration and initialization.

 @Param[in]     p_UccAtmAal2TxQDParam - TxQD Aal2 user defined parameters.

 @Return        Handle to the created TxQD if success; NULL otherwise.
*//***************************************************************************/
t_Handle UCC_ATM_AAL2_ConfigAndInitTxQd(t_UccAtmAal2TxQdParam *p_UccAtmAal2TxQDParam);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_FreeTxQd

 @Description   Free memory of a Tx Queue Descriptor.

 @Param[in]     h_UccAtmAal2TxQd - TxQD handler that was initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The routine must be called only after UCC_ATM_AAL2_ConfigAndInitTxQd()
                There is no support of remove TxQD from the list of the TxQDs
                Needs to remember that in order to remove one TxQD from the list
                there is a need to call to UCC_ATM_CH_Disable()

*//***************************************************************************/
t_Error UCC_ATM_AAL2_FreeTxQd ( t_Handle h_UccAtmAal2TxQd );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_ConfigAndInitCid

 @Description   AAL2 CID initialization and possible initialization of RxQd.

		        It checks and saves various parameters required by
		        Tx and Rx flows. It also initializes the CID internal data
		        structure, Initialized the RxQD, sets the rx queue descriptor
		        offset into CID mapping table and sets data buffers
		        into the Rx BD ring.

 @Param[in]     p_UccAtmAal2CidParam - Ucc Atm Aal2 Cid parameters.

 @Return        Handle to created CID if success. NULL otherwise.
 
 @Cautions      The routine must be called only after UCC_ATM_AAL2_ConfigAndInit()
                routine was called and if the CID is configured to transmit the routine
                UCC_ATM_AAL2_ConfigAndInitAal2TxQd() should be called.
*//***************************************************************************/
t_Handle   UCC_ATM_AAL2_ConfigAndInitCid( t_UccAtmAal2CidParam *p_UccAtmAal2CidParam );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_FreeCid

 @Description   Free AAL2 CID structure.

 @Param[in]     h_UccAtmAal2Cid - CID handler that was already initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The routine must be called only after UCC_ATM_ConfigAndInitCid() was called.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_FreeCid( t_Handle h_UccAtmAal2Cid );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_RxCidDisable

 @Description   Disable AAL2 Rx CID Synchronically.

 @Param[in]     h_UccAtmAal2Cid - CID handler that was already initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The routine must be called only after UCC_ATM_ConfigAndInitCid() was called.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_RxCidDisable(t_Handle h_UccAtmAal2Cid);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_RxCidDisableAsync

 @Description   Disable AAL2 Rx CID Asynchronically.

                The routine works asynchronically which means that it doesn't wait
                until the disable operation will be finished.
 
 @Param[in]     h_UccAtmAal2Cid - CID handler that was already initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The routine must be called only after UCC_ATM_ConfigAndInitCid() was called.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_RxCidDisableAsync(t_Handle h_UccAtmAal2Cid);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_RxCidEnable

 @Description   Enable AAL2 Rx CID.

                There is no separation to synchronic and asynchronic enable
                (like in the disable case) bcause there is nothing to wait
                for so the synchronic option does not block.
 
 @Param[in]     h_UccAtmAal2Cid - CID handler that was already initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The routine must be called only after UCC_ATM_ConfigAndInitCid() was called.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_RxCidEnable(t_Handle h_UccAtmAal2Cid);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_IsRxCidStopped

 @Description   Checks if Rx CID is disabled.
 
                If yes - returns TRUE and if necessary updates the CID status,
                If no - returns FALSE.
 
 @Param[in]     h_UccAtmAal2Cid - handle to UCC ATM AAL2 CID object.

 @Return        TRUE is Rx CID is disabled; FALSE otherwise.
*//***************************************************************************/
bool UCC_ATM_AAL2_IsRxCidStopped(t_Handle h_UccAtmAal2Cid);


/** @} */ /* end of ucc_atm_aal2_channel_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal2_channel_runtime_grp UCC ATM AAL2 Runtime Unit

 @Description   Runtime Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_CpsTx

 @Description   Transmits CPS packet on a specific CID.
 
                Accepts data as a pointer to a single data buffer (array of
                bytes).

                This routine eventually causes the user's transmit confirmation
                call-back routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                UCC_ATM_AAL2_CpsTxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.
 
 @Param[in]     h_UccAtmAal2Cid  - driver's handler of the CID that wants to transmit data.
 @Param[in]     p_Data - pointer to the beginning of CPS packet payload.
 @Param[in]     h_BufContext  - User's context for the given data buffer.
 @Param[in]     packetLength - The size of the packet.
@if UNDER_CONSTRUCTION
 @Param[in]     hec - the hec value which the user asked to use.
@endif
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                UCC_ATM_AAL2_CpsTxConf()) simultaneously at task level and at interrupt
                level (for example, inside the Rx user callback routine) may lead
                to unpredictable results.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_CpsTx(t_Handle h_UccAtmAal2Cid,
                           uint8_t  *p_Data,
                           t_Handle h_BufContext,
                           uint8_t  packetLength
                           );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_CpsRx

 @Description   Handles data received in the specific AAL2 CID.

                Receive data are passed to the user via the user receive call
                back routine corresponding to the cid.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.

                That routine is called either from the driver's interrupt
                routine or from an application's routine to instigate
                receive handling of packets.
 
 @Param[in]     h_UccAtmAal2Cid  - driver's handler of the CID.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      In receiver polling mode Rx interrupt is disabled, and the user
                calls the receive routines directly.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_CpsRx(t_Handle h_UccAtmAal2Cid);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_SarTx

 @Description   Transmit data in an AAL2 SSSAR CID.
 
 				Accepts data as a pointer to a single data buffer (array of
                bytes). If uuiMode is set for this TxQd,
                the UUI Data should be set at the end of the data
                ('length' bytes from the start).

                This routine eventually causes the user's transmit confirmation
                call-back routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                UCC_ATM_AAL2_SarTxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.

 
 @Param[in]     h_UccAtmAal2Cid          - driver's handler of the CID.
 @Param[in]     p_Data         - the address of the data for sending.
 @Param[in]     h_BufContext   - User's context for the given data buffer.
 @Param[in]     length         - the number of octets for transmitting.
 @Param[in]     lastBuffer 	   - is last buffer in frame.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The user must take into account that the uui value is written
                at the end of the data buffer. Therefore, he must allocate one more byte for
                this value.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                UCC_ATM_AAL2_SarTxConf()) simultaneously at task level and at interrupt
                level (for example, inside the Rx user callback routine) may lead
                to unpredictable results.

*//***************************************************************************/
t_Error UCC_ATM_AAL2_SarTx(t_Handle h_UccAtmAal2Cid,
                           uint8_t  *p_Data,
                           t_Handle h_BufContext,
                           uint16_t length,
                           bool     lastBuffer);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_SarRx

 @Description   Handles data buffers received in a specific AAL2 CID.

                Received data are passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific CID.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.
 
 @Param[in]     h_UccAtmAal2Cid  - driver's handler of the CID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_SarRx(t_Handle h_UccAtmAal2Cid);

/**************************************************************************//**
 @Collection    UCC ATM AAL2 Rx errors.
 
                Those errors are set in the receive bd; They are returned via
                the f_RxStore callback.
 @{
*//***************************************************************************/
#define UCC_ATM_AAL2_RX_ERRORS_UP      0x0100  /**< UP error;
                                                    a CPS/Switch receiver error occured that caused this
                                                    packet to be uncomleted. */

#define UCC_ATM_AAL2_RX_ERRORS_TE      0x0200  /**< TE error; Ras Timer Expired (SSSAR error) -
                                                    The Ras timer expired before tis buffer could be completed,
                                                    so the SDU stored in this buffer is not completed */
#define UCC_ATM_AAL2_RX_ERRORS_US      0x0400  /**< US error; Uncompleted SDU (SSSAR error) - A receiver error
                                                    caused a packet belonging to this SSSAR SDU to be lost;
                                                    The receiver discarded the rest of the SSSAR SDU. */
#define UCC_ATM_AAL2_RX_ERRORS_OS      0x0600  /**< OS error; Oversized (SSSAR error) - The size of the SSSAR SDU
                                                    has exceeded the Max SSSAR SDU size allowed;
                                                    The rest of the SDU was discarded */
/* @} */

/**************************************************************************//**
 @Collection    UCC ATM AAL2 Rx flags.
 
                The AAL2 flags returned via f_RxStore callback.
 
                Note that the low 5 bits of the flags parameter are reserved
                for the UUI value of the received frame.
 @{
*//***************************************************************************/
#define UCC_ATM_AAL2_RX_FLAG_IW_QUEUE   0x80000000  /**< SAR frame was received through one of the IW CPU queues;
                                                         The reported queue ID represents the IW CPU queues index. */
/* @} */

/** @} */ /* end of ucc_atm_aal2_channel_runtime_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal2_channel_control_grp UCC ATM AAL2 Control Unit

 @Description   Control Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_CpsTxConf

 @Description   Forces confirmation on transmitted CPS packets.
 
                Invokes user's tx confirmation call back routine for previously
                transmitted data on the specific AAL2 CID.

                Normally, this functionality is performed in the transmit
                routines for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.

 @Param[in]     h_UccAtmAal2Cid  - driver's handler of the CID.

 @Cautions      The txConf routine actually cleans a specific BD Ring which
                is related to the specific cid. Please note that the routine
                cleans a BD Ring that there is a possibility that some
                CIDs worked with it, so the routine reads in every entry of
                the BD ring ,the actual CID number and according to the
                actual CID number, it calls to the f_TxConf user routine.
 
 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                the transmit routines) simultaneously at task level and at
                interrupt level may lead to unpredictable results.
*//***************************************************************************/
void    UCC_ATM_AAL2_CpsTxConf ( t_Handle h_UccAtmAal2Cid );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_SarTxConf

 @Description   Forces confirmation on transmitted frames or data buffers.
 
		        Invokes user's tx confirmation call back routine for previously
                transmitted data on the specific AAL2 CID.

                Normally, this functionality is performed in the transmit
                routines for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.
 
 @Param[in]     h_UccAtmAal2Cid  - driver's handler of the CID.
*//***************************************************************************/
void    UCC_ATM_AAL2_SarTxConf ( t_Handle h_UccAtmAal2Cid );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_GetChannelTxStat

 @Description   Returns the channel transmitted cell counter.

 @Param[in]     h_UccAtmAal2 - Ucc Atm Aal2 handler.
 @Param[out]    p_ChCnt      - Channel Tx Statistics counter value.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_GetChannelTxStat( t_Handle h_UccAtmAal2, uint16_t *p_ChCnt);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_GetChannelRxStat

 @Description   Returns the channel received cell counter.

 @Param[in]     h_UccAtmAal2 - Ucc Atm Aal2 handler.
 @Param[out]    p_ChCnt      - Channel Rx Statistics counter value.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_GetChannelRxStat( t_Handle h_UccAtmAal2, uint16_t *p_ChCnt);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_GetCidTxStat

 @Description   Returns the cid transmitted packets/frames counter.

 @Param[in]     h_UccAtmAal2Cid - Ucc Atm Aal2 Cid handler.
 @Param[out]    p_CidCnt        - The cid counter value.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_GetCidTxStat( t_Handle h_UccAtmAal2Cid, uint16_t *p_CidCnt);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_GetCidRxStat

 @Description   Returns the cid receiveded packets/frames counter.

 @Param[in]     h_UccAtmAal2Cid - Ucc Atm Aal2 Cid handler.
 @Param[out]    p_CidCnt        - The cid counter value.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_GetCidRxStat( t_Handle h_UccAtmAal2Cid, uint16_t *p_CidCnt);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_ModifyTxQdThresholdSet

 @Description   Update a threshold set for a specific TxQd.

 @Param[in]     h_UccAtmAal2TxQd    - TxQd handler.
 @Param[in]     thresholdSetNum     - the threshold set number.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_ModifyTxQdThresholdSet( t_Handle h_UccAtmAal2TxQd, uint16_t thresholdSetNum);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_GetAvailableCidNum

 @Description   Get an available cid number in the given range.
 
 @Param[in]     h_UccAtmAal2 - handle to UCC ATM AAL2 channel object.
 @Param[in]     minCid - min cid number in range.
 @Param[in]     maxCid - max cid number in range.

 @Return        An available cid number in the given range.
*//***************************************************************************/
int8_t UCC_ATM_AAL2_GetAvailableCidNum(t_Handle h_UccAtmAal2, uint8_t minCid, uint8_t maxCid);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_AAL2_DumpRegs

 @Description   Dumps ATM AAL2 Channel registers and parameters RAM.

 @Param[in]     h_UccAtmAal2 - Handle to the AAL2 channel object.

 @Return        None.

 @Cautions      AAL2 Channel must have been previously initialized using UCC_ATM_AAL2_ConfigAndInit().

*//***************************************************************************/
void UCC_ATM_AAL2_DumpRegs(t_Handle h_UccAtmAal2);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_DumpRegsCid

 @Description   Dumps ATM AAL2 CID registers and parameters RAM.

 @Param[in]     h_UccAtmAal2Cid - Handle to the AAL2 channel object.

 @Return        None.

 @Cautions      AAL2 CID must have been previously initialized using UCC_ATM_AAL2_ConfigAndInitCid().

*//***************************************************************************/
void UCC_ATM_AAL2_DumpRegsCid(t_Handle h_UccAtmAal2Cid);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_DumpRegsTxQd

 @Description   Dumps ATM AAL2 Tx QD registers and parameters RAM.

 @Param[in]     h_UccAtmAal2TxQd - Handle to the AAL2 channel object.

 @Return        None.

 @Cautions      AAL2 Tx QD must have been previously initialized using UCC_ATM_AAL2_ConfigAndInitTxQd().

*//***************************************************************************/
void UCC_ATM_AAL2_DumpRegsTxQd(t_Handle h_UccAtmAal2TxQd);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_aal2_channel_control_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal2_channel_exception_grp UCC ATM AAL2 Exception Unit

 @Description   Exception Structures and Routines

 @{
*//***************************************************************************/

/****************************************************************//**
 @Collection    AAL2 Interrupt Queue exception bit mask
 
                The status is returned via f_Exception callback.
 @{
*//*****************************************************************/

/****************************************************************//**
 @Collection    AAL2 Interrupts Queue exception bit mask when CID = 0.

 @{
*//*****************************************************************/
/* Interrupts when CID = 0 */
#define UCC_ATM_AAL2_INTR_PARITY_ERROR_OF_OSF       0x00000000 /**< Parity error of the OSF.   */
#define UCC_ATM_AAL2_INTR_INCORRECT_STF             0x00000001 /**< The STF sequence number is incorrect.   */
#define UCC_ATM_AAL2_INTR_OSF_NO_MATCH              0x00000002 /**< The number of octets expected to overlap
                                                                    into this cell doesn't match the OSF.   */
#define UCC_ATM_AAL2_INTR_OSF_GREATER               0x00000003 /**< the OSF is greater than 47.   */
#define UCC_ATM_AAL2_INTR_HEC_ERROR                 0x00000004 /**< A packet HEC error was detected.   */
#define UCC_ATM_AAL2_INTR_EXCEED_MAX_SDU_LENGTH     0x00000005 /**< The length of the CPS packet exceeds the Max_SDU_Length.   */
#define UCC_ATM_AAL2_INTR_HEC_ERR_IN_SPLIT          0x00000007 /**< A packet HEC error was detected in a split header packet.   */

/* @} */

/****************************************************************//**
 @Collection    AAL2 Interrupts Queue exception bit mask for CID statistics interrupts.

 @{
*//*****************************************************************/
/* Interrupts for CID statistics */
#define UCC_ATM_AAL2_INTR_STATS_PACKD               0x00000004 /**< Rx Packet discard counter overflow;
                                                                    Valid only for switch mode when TxQd[packD] bit is set */
#define UCC_ATM_AAL2_INTR_STATS_RX_OVERFLOW         0x00000002 /**< Rx CID/ATM cell counter overflow ;
                                                                    if CID = 0,indicates that it's a Rx ATM cell counter
                                                                    else ,indicates the CID value in which the RX CID counter overflow appears */
#define UCC_ATM_AAL2_INTR_STATS_TX_OVERFLOW         0x00000001 /**< Tx CID/ATM cell counter overflow ;
                                                                    if CID = 0,indicates that it's a Tx ATM cell counter
                                                                    else ,indicates the CID value in which the TX CID counter overflow appears */
/* @} */

/****************************************************************//**
 @Collection    AAL2 Interrupts Queue exception bit mask for WFQ management.

 @{
*//*****************************************************************/
/* Interrupts for WFQ queue management */
#define UCC_ATM_AAL2_INTR_WFQ_FULL_SPEED_TO_DECELERATE  0x0000000 /**< Full speed to decelerate interrupt */
#define UCC_ATM_AAL2_INTR_WFQ_DECELERATE_TO_FULL_STOP   0x0000003 /**< Decelerate to full stop interrupt */
#define UCC_ATM_AAL2_INTR_WFQ_ACCELERATE_TO_DECELERATE  0x0000004 /**< Accelerate to decelerate interrupt */
#define UCC_ATM_AAL2_INTR_WFQ_FULL_STOP_TO_ACCELERATE   0x0000005 /**< Full stop to accelerate interrupt */
#define UCC_ATM_AAL2_INTR_WFQ_ACCELERATE_TO_FULL_SPEED  0x0000006 /**< Accelerate to full spped interrupt */
#define UCC_ATM_AAL2_INTR_WFQ_DECELERATE_TO_ACCELERATE  0x0000007 /**< Decelerate to accelerate interrupt */
/* @} */

/* @} */

/** @} */ /* end of ucc_atm_aal2_channel_exception_grp group */
/** @} */ /* end of ucc_atm_aal2_channel_grp group */
/** @} */ /* end of group ucc_atm_grp */


/**************************************************************************//*
 @Description   AAL2 IW details structure (interface with IW driver).
*//***************************************************************************/
typedef struct t_UccAtmAal2IwDetails
{
	uint16_t    channelCode;        /**< Tx channel code */
	uint8_t     cidNumber;          /**< CID number */
	e_CommMode  iwCommMode;         /**< Indicates whether this CID is used
	                                     for IW in Tx mode, Rx mode, or both */
    uint16_t    txQdOffset;         /**< Offset of the CID's TxQD */
	bool        isExternalTxQd;     /**< Indicates if the TxQD is external */
} t_UccAtmAal2IwDetails;

/**************************************************************************//*
 @Function      UCC_ATM_AAL2_GetIwDetails

 @Description   Gets the IW details of this channel (interface with IW driver).

 @Param[in]     h_UccAtmAal2Cid     - AAL2 CID handle.
 @Param[in]     p_IwDetails         - Pointer to AAL2 IW details structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_GetIwDetails(t_Handle                h_UccAtmAal2Cid,
                                  t_UccAtmAal2IwDetails   *p_IwDetails);


/**************************************************************************//*
 @Function      UCC_ATM_AAL2_SarGetIwIndex

 @Description   Get the IW index for Interworking Action Descriptor (IWAD).

 @Param[in]     h_UccAtmAal2Cid - AAL2 CID handle.
 @Param[out]    p_IwIndex       - Will get the IW TQD index (Offset of the
                                  channel's first TxQD).
 @Param[out]    p_IsExtTqd      - Will be TRUE if TxQD is external.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL2_SarGetIwIndex(t_Handle h_UccAtmAal2Cid,
                                   uint16_t *p_IwIndex,
                                   bool     *p_IsExtTqd);


/**************************************************************************//**
 @Group         ucc_atm_common_grp UCC ATM Common

 @Description   Ucc Atm Common

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_common_init_grp UCC ATM Common Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_common_config_grp UCC ATM Common Advanced Configuration Unit

 @Description   Configuration routines and structures used to change default values

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ucc_atm_aal2_common_advanced_init_grp UCC ATM AAL2 Common Advanced Configuration Unit

 @Description   AAL2 Common Configuration Routines and Structures used to change default values.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigWfqIntTxQdGroups

 @Description   Changes the number of Weighted Fair Queue(switch mode) internal TxQDs from its default configuration.

                WFQ firmware constraint forces the WFQ TxQds to be in a consecutive order;
                Default value: [0].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     wfqIntTxQdGroups   - number of groups of WFQ internal TxQDs.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigWfqIntTxQdGroups ( t_Handle h_UccAtmAal2Common, uint32_t wfqIntTxQdGroups );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigWfqExtTxQdGroups

 @Description   Changes the number of Weighted Fair Queue(switch mode) external TxQDs from its default configuration.

                WFQ firmware constraint forces the WFQ TxQds to be in a consecutive order;
                Default value: [0].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     wfqExtTxQdGroups   - number of groups of WFQ external TxQDs.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigWfqExtTxQdGroups ( t_Handle h_UccAtmAal2Common, uint32_t wfqExtTxQdGroups );


/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition information for AAL2 firmware's
                external structures, to override the default configuration.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

                This configuration does not include memory allocation for
                Partial Packet Receive Storage (PPRS), which may be set using
                the UCC_ATM_AAL2_COMMON_ConfigPprsMemory() routine.

 @Param[in]     h_UccAtmAal2Common  - handle to UCC ATM AAL2 Common object.
 @Param[in]     memId               - Memory partition ID.
 @Param[in]     memAttributes       - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigFirmwareExternalStructsMemory(t_Handle    h_UccAtmAal2Common,
                                                                uint8_t     memId,
                                                                uint32_t    memAttributes);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigSwIntRxQdType1

 @Description   Changes the number of switch internal RxQDs from type1 from its default configuration.
 
                The routine changes the number of Switch internal RxQDs Type1(which means that point to an internal TxQD) in the system
                Default value: [0].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     swIntRxQdType1     - number of switch internal RxQds from type1

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigSwIntRxQdType1 ( t_Handle h_UccAtmAal2Common, uint32_t swIntRxQdType1 );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigSwExtRxQdType1

 @Description   Changes the number of switch external RxQDs from type1 from its default configuration.
 
                The routine changes the number of Switch external RxQDs Type1(which means that point to an internal TxQD) in the system
                Default value: [0].
 
 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     swExtRxQdType1     - number of switch external RxQds from type1.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigSwExtRxQdType1 ( t_Handle h_UccAtmAal2Common, uint32_t swExtRxQdType1 );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigSwIntRxQdType2

 @Description   Changes the number of switch internal RxQDs from type2 from its default configuration.
 
                The routine changes the number of Switch internal RxQDs Type2(which means that point to an external TxQD) in the system
                Default value: [0].
 
 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     swIntRxQdType2     - number of switch internal RxQds from type2.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigSwIntRxQdType2 ( t_Handle h_UccAtmAal2Common, uint32_t swIntRxQdType2 );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigSwExtRxQdType2

 @Description   Changes the number of switch external RxQDs from type2 from its default configuration.
 
                The routine changes the number of Switch external RxQDs Type2(which means that point to an external TxQD) in the system
                Default value: [0].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     swExtRxQdType2     - number of switch external RxQds from type2.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigSwExtRxQdType2 ( t_Handle h_UccAtmAal2Common, uint32_t swExtRxQdType2 );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigPprsMode

 @Description   Changes the configuration to operate the PPRS(Partial Packet Receive Storage) mode.
 
                The PPRS mode enables the situation that when partial packet is received
                it is written to external memory  until the packet is completely received.
                Default value: [TRUE].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     pprsMode           - TRUE to enable PPRS mode
                                          FALSE to disable PPRS mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigPprsMode ( t_Handle h_UccAtmAal2Common, bool pprsMode );

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigPprsMemory

 @Description   Configures memory partition information for partial packets storage,
                to override the default configuration.

 @Param[in]     h_UccAtmAal2Common  - handle to UCC ATM AAL2 Common object.
 @Param[in]     memId               - Memory partition ID.
 @Param[in]     memAttributes       - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigPprsMemory(t_Handle   h_UccAtmAal2Common,
                                             uint8_t    memId,
                                             uint32_t   memAttributes);

/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigRasTimerDuration

 @Description   Changes the RAS timer duration value from its default configuration.
 
                RAS timer duration in nano seconds ; important for the SAR sublayer
                Default value: [0].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     rasTimerDuration   - RasTimerDuration ; 0 - means no duration.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigRasTimerDuration ( t_Handle h_UccAtmAal2Common, uint32_t rasTimerDuration );

#ifndef UCC_ATM_AAL2_UNAVAILABLE
/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigCidMapTableAllocation

 @Description   Changes the CID map table allocation type from its default configuration.
 
                The CID map table allocation type is common to all the AAL2 channels in the system;
                It can internal or external or combined(Internal allocation for internal channels, external allocation for external channels)
                Default value: [e_UCC_ATM_COMMON_AAL2_CID_MAP_COMBINED_ALLOCATION].

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     cidMapTableAllocation   - CID map table allocation type.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigCidMapTableAllocation ( t_Handle h_UccAtmAal2Common, e_UccAtmAal2CommonCidMapAlloc cidMapTableAllocation );
/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_ConfigThresholdSets

 @Description   Configures threshold sets in the system.

 @Param[in]     h_UccAtmAal2Common - handle to UCC ATM AAL2 Common object.
 @Param[in]     queueManagementThresholdSets - how many queue management threshold sets are required in the system.
 @Param[in]     p_ThresholdSetsParam - queue management parameters structure

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_AAL2_COMMON_Config() and before UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL2_COMMON_ConfigThresholdSets (t_Handle h_UccAtmAal2Common, uint16_t queueManagementThresholdSets, t_UccAtmAal2CommonThresholdSetsParam **p_ThresholdSetsParam);
#endif /* UCC_ATM_AAL2_UNAVAILABLE */

/** @} */ /* end of ucc_atm_aal2_common_advanced_init_grp group */
/** @} */ /* end of ucc_atm_common_config_grp group */
/** @} */ /* end of ucc_atm_common_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_common_control_grp UCC ATM Common Control Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_AAL2_COMMON_DumpRegs

 @Description   Dumps UCC ATM AAL2 COMMON registers and parameters RAM.

 @Param[in]     h_UccAtmAal2Common - Handle to UCC ATM AAL2 Common object.

 @Return        None.

 @Cautions      h_UccAtmAal2Common must have been previously created using UCC_ATM_AAL2_COMMON_Config()
 and initialized using UCC_ATM_AAL2_COMMON_Init().
*//***************************************************************************/
void UCC_ATM_AAL2_COMMON_DumpRegs(t_Handle h_UccAtmAal2Common);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_common_control_grp group */

/** @} */ /* end of ucc_atm_common_grp group */

#endif /* __UCC_ATM_AAL2_EXT_H */
