/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_common_ext.h

 @Description   UCC Atm Common external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_ATM_COMMON_EXT_H
#define __UCC_ATM_COMMON_EXT_H

#include "part_ext.h"
#include "xx_ext.h"
#include "cq_ext.h"
#include "qe_ext.h"


/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/


/**************************************************************************//**
 @Group         ucc_atm_common_grp UCC ATM Common

 @Description   Ucc Atm Common

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_common_init_grp UCC ATM Common Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Ucc Atm Common initialization structure, used when calling UCC_ATM_COMMON_Config().
 *//***************************************************************************/
typedef struct t_UccAtmCommonParam
{
    t_Handle    h_Qe;           /**< Handle to the controlling QE */
    uint8_t     pramMemId;      /**< Parameter RAM partition ID */
    uint16_t    rxIntChannels;  /**< Max receive internal channels in the system */
    uint16_t    rxExtChannels;  /**< Max receive external channels in the system (not including switched channels) */
    uint16_t    txIntChannels;  /**< Max transmit internal channels in the system;
                                     This number does not include AAL5 hierarchical channels and AAL0 AVCON switched channels. */
    uint16_t    txExtChannels;  /**< Max transmit external channels in the system;
                                     This number does not include AAL5 hierarchical channels and switched channels */
    uint16_t    aal0Channels;   /**< Max Aal0 channels in the system; Influence on the number of AAL0 channel driver structures in the system;
                                     Note: Half/Full duplex channels are counted as one */
    uint16_t    aal1Channels;   /**< Max Aal1 channels for the system;Influence on the number of AAL1 channel driver structures in the system;
                                     Note: Half/Full duplex channels are counted as one */
    uint16_t    aal2Channels;   /**< Max Aal2 channels for the system;Influence on the number of AAL2 channel driver structures in the system;
                                     Note: Half/Full duplex channels are counted as one */
    uint16_t    aal5Channels;   /**< Max Aal5 channels for the system;Influence on the number of AAL5 channel driver structures in the system;
                                     Note: Half/Full duplex channels are counted as one;
                                     This number does not include AAL5 hierarchical channels. */

} t_UccAtmCommonParam;


/**************************************************************************//**
 @Function      UCC_ATM_COMMON_Config

 @Description   Configures a Ucc Atm Common level based on user parameters.

                The UCC ATM Common handle can then be initialized using
                UCC_ATM_COMMON_Init().

 @Param[in]     p_UccAtmCommonParam - user parameters for Ucc Atm Common.

 @Return        Ucc Atm Common handler on success; NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.

*//***************************************************************************/
t_Handle UCC_ATM_COMMON_Config(t_UccAtmCommonParam *p_UccAtmCommonParam);

/**************************************************************************//**
 @Group         ucc_atm_common_config_grp UCC ATM Common Advanced Configuration Unit

 @Description   Configuration routines and structures used to change default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   structure representing Gcra scheduler user parameters
*//***************************************************************************/
typedef struct t_UccAtmCommonGcraSchedParam
{
    uint8_t maxInternalChannelsPerPort;     /**< The maximum number of internal channels for each port that work with the GCRA scheduler
                                                 Note: In case of GBR, each channel should be counted as two */
    uint8_t numOfPortsUseInternalChannels;  /**< Number of ports in the system that work with internal channels and work with GCRA scheduler */
    uint8_t maxExternalChannelsPerPort;     /**< The maximum number of external channels for each port that work with the GCRA scheduler
                                                 Note: In case of GBR, each channel should be counted as two */
    uint8_t numOfPortsUseExternalChannels;  /**< Number of ports in the system that work with external channels and work with GCRA scheduler */

} t_UccAtmCommonGcraSchedParam;

/**************************************************************************//**
 @Description   structure representing Vp Shaping Over Gcra user parameters
 
                Note : The parameters in this structure should regard to all VPs
                in the system, inclusing VPs of different ports.
*//***************************************************************************/
typedef struct t_UccAtmCommonVpShapingOverGcraParam
{
    uint8_t     maxInternalChannelsPerVp;       /**< The maximum number of internal channels for each vp that work with the VP Shaping over GCRA
                                                     Note: In case of GBR, each channel should be counted as two */
    uint8_t     numOfVpsUseInternalChannels;    /**< Number of vps in the system that work with internal channels and work with VP Shaping over GCRA */
    uint8_t     maxExternalChannelsPerVp;       /**< The maximum number of external channels for each vp that work with the VP Shaping over GCRA
                                                     Note: In case of GBR, each channel should be counted as two */
    uint8_t     numOfVpsUseExternalChannels;    /**< Number of vps in the system that work with external channels and work with VP Shaping over GCRA */

} t_UccAtmCommonVpShapingOverGcraParam;

/**************************************************************************//**
 @Description   structure representing Hierarchical channels user parameters
*//***************************************************************************/
typedef struct t_UccAtmCommonHierarchicalParam
{
    uint16_t    numOfInternalHierarchicalWithFrameBasedOnly;
                    /**< How many Internal Hierarchical channels in the system that have Frame Based (HFB) queues */

    uint16_t    numOfExternalHierarchicalWithFrameBasedOnly;
                    /**< How many External Hierarchical channels in the system that have Frame Based (HFB) queues */

    uint16_t    numOfInternalHierarchicalWithCellBased;
                    /**< How many Internal Hierarchical channels in the system that have Cell Based (HCB) queues or
                         Cell based and Frame based queues combined */

    uint16_t    numOfExternalHierarchicalWithCellBased;
                    /**< How many Internal Hierarchical channels in the system that have Cell Based (HCB) queues or
                         Cell based and Frame based queues combined */

    uint8_t     maxQueuesPerHierarchicalWithCellBased;
                    /**< The max number of queues per hierarchical channel with Cell Based queues
                         (possibly combined with Frame based); should be 1-8;
                         this parameter actually determine the size of the internal WFQ table */
 
}  t_UccAtmCommonHierarchicalParam;

/**************************************************************************//**
 @Description   Number of overlapping groups in MiniCam
 *//***************************************************************************/
#define MAX_MINI_CAM_OVERLAP_GROUPS      64

/**************************************************************************//**
 @Description   optional values of maximum channels per minCam port
*//***************************************************************************/
/* NOTE: The values of the enums is the relevant values that need to be written
         in the Mini-Cam parameter table */
typedef enum e_MaxChannelsPerMiniCamPort
{
    e_MAX_CHANNELS_PER_MINI_CAM_PORT_NONE   = 0
    , e_MAX_CHANNELS_PER_MINI_CAM_PORT_8    = 8
    , e_MAX_CHANNELS_PER_MINI_CAM_PORT_16   = 16
    , e_MAX_CHANNELS_PER_MINI_CAM_PORT_32   = 32
    , e_MAX_CHANNELS_PER_MINI_CAM_PORT_64   = 64
} e_MaxChannelsPerMiniCamPort;

/**************************************************************************//**
 @Description   structure representing Mini Cam Overlap VPI+VCI pair user parameters
*//***************************************************************************/
typedef struct t_OverlapVpiVciParam
{
    uint16_t vpi;             /**< The Vpi value, which is common to all the overlap ports in the overlap group */
    uint16_t vci;             /**< The Vci value, which is common to all the overlap ports in the overlap group */
    uint8_t  policerIndex;    /**< Indicates the policer table index;
                                   0 - means no policer is required for this Vpi/Vci
                                   1 - 15 - means the policer index
                                   There is a possibility to cooperate the same policer table by using the same
                                   index to different Vpis/Vcis; The max index should not exceed the number of
                                   policer tables per miniCam port in the UCC level
                                   (maxPolicerTablesPerMiniCamPort in t_UccAtmPolicerParam). */
} t_OverlapVpiVciParam;

/**************************************************************************//**
 @Description   structure representing Mini Cam UCC Overlap Group user parameters
*//***************************************************************************/
typedef struct t_MiniCamOverlapGroupParam
{
    uint8_t numOfOverlapVpiVciValues;   /**< The number of different VPIs and VCIs in this overlap group. */
 
    t_OverlapVpiVciParam *p_OverlapVpiVciValues;
                                        /**< The different values of channels (VPIs and VCIs) that belong to ports of
                                             a specific overlap group (up to 64 different values per overlap group) */
} t_MiniCamOverlapGroupParam;

/**************************************************************************//**
 @Description   structure representing Mini Cam user parameters
 
                The structure needs to be initialized per UCC in the system
                that requires to work with MiniCam as address resolution mode
*//***************************************************************************/
typedef struct t_UccAtmCommonMiniCamParam
{
    e_MaxChannelsPerMiniCamPort     maxChannelsPerMiniCamPort;  /**< The maximum number of channels for each port that
                                                                     work with the MiniCam address resolution mode */
    uint16_t                        nonOverlapPorts;            /**< Number of ports in the system that requires the
                                                                     work with MiniCam or Address Compression address
                                                                     resolution mode and are not overlapping with other
                                                                     ports (which means that they don't have an
                                                                     overlapping in the VPI/VCI values with other ports)
                                                                     NOTE:although there are some ports between the
                                                                     firstMiniCam port and the lastMiniCam port that
                                                                     are not intend to work with Mini-Cam, they must be
                                                                     included in the number of ports since the channel
                                                                     codes that are allocated are consecutive */
    uint8_t                         miniCamOverlapGroups;       /**< How many overlapping groups for the specific ucc
                                                                     NOTE:The limitation is 64 overlap groups */
    bool                            internalMiniCamChannels;    /**< Indicates if the Mini Cam channels for the
                                                                     specific UCC should be internal or external;
                                                                     NOTE: it's a driver restriction that there is
                                                                     a possibility to work with internal or external
                                                                     Mini-Cam channels under a specific ucc and not
                                                                     both of the types */
    bool                            useVpSwitchFilter;          /**< Indicates if VP Switch filter will be used for the
                                                                     specific UCC;
                                                                     NOTE: Relevant only if VP Switch mode is enabled
                                                                     for the UCC */
    uint16_t                        firstMiniCamPort;           /**< The number of the first MiniCam port of the
                                                                     specific UCC; NOTE: The number needs to take into
                                                                     account the device number. */
    uint16_t                        lastMiniCamPort;            /**< The number of the last MiniCam port of the
                                                                     specific UCC; NOTE: The number needs to take into
                                                                     account the device number. */
    uint16_t                        firstAddrCompPort;          /**< The number of the first A.C. port of the
                                                                     specific UCC; NOTE: The number needs to take into
                                                                     account the device number. */
    uint16_t                        lastAddrCompPort;           /**< The number of the last A.C. port of the
                                                                     specific UCC; NOTE: The number needs to take into
                                                                     account the device number. */
    uint16_t                        addressCompressionPorts;    /**< How many ports of this UCC are going to use
                                                                     classical address compression;
                                                                     =0 - means no ports work with address compression
                                                                     although the UCC is configured to Mini-Cam.
                                                                     >0 - means there are some ports under this UCC
                                                                     that work with address compression altough the
                                                                     UCC is configured to work in Mini-Cam. */
    t_MiniCamOverlapGroupParam     *p_MiniCamOverlapGroupParam; /**< Structure holds all the required parameter for
                                                                     an overlap group */
} t_UccAtmCommonMiniCamParam;


/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigConnectionTablesMemory

 @Description   Configures memory partition information for external connection
                tables to override the default configuration.
 
 @Param[in]     h_UccAtmCommon  - Handle to ATM COMMON object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigConnectionTablesMemory(t_Handle    h_UccAtmCommon,
                                                    uint8_t     memId,
                                                    uint32_t    memAttributes);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigNonExtendedCt

 @Description   Changes the indication if the work is with non extended connection tables.
 
                set 'nonExtendedCt' to TRUE if there is no need of extended connection tables.
                Extended connection tables are required for ATM channels from service types of: VBR ,
                UBR+, GBR, or when Scalable mode is enabled.
                Default value: [FALSE].

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     nonExtendedCt  - TRUE to work without extended connection tables
                                      FALSE to work with extended connection tables.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigNonExtendedCt (t_Handle h_UccAtmCommon, bool nonExtendedCt);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigTxExtSwitchChannels

 @Description   Changes the number of external switched channels from its default configuration.
 
                Switched channels are channels that are added to the
                scheduler in an automatic way without core intervention when there is a
                data to transmit. The number of switch channels include channels that are not
                Hcb/Hfb channels with auto VcOn mode.
                Default value: [0]

 @Param[in]     h_UccAtmCommon      	- Handle to ATM COMMON object.
 @Param[in]     txExtSwitchedChannels - how many transmit external switched channels are required in the system.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigTxExtSwitchChannels (t_Handle h_UccAtmCommon, uint16_t txExtSwitchedChannels);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigaAal0AvconSwitchIntChannels

 @Description   Changes the number of ATM AAL0 switched internal channels with Auto VC On feature enabled from its default configuration.
 
                Switched channels are channels that are added to the
                scheduler in an automatic way without core intervention when there is a
                data to transmit.
                Default value: [0]

 @Param[in]     h_UccAtmCommon      		- Handle to ATM COMMON object.
 @Param[in]     aal0AvconSwitchIntChannels 	- How many ATM AAL0 switched internal channels with Auto VC On feature enabled are required in the system.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigaAal0AvconSwitchIntChannels (t_Handle h_UccAtmCommon, uint8_t aal0AvconSwitchIntChannels);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigaAal0AvconSwitchExtChannels

 @Description   Changes the number of ATM AAL0 switched external channels with Auto VC On feature enabled from its default configuration.
 
                Switched channels are channels that are added to the
                scheduler in an automatic way without core intervention when there is a
                data to transmit.
                Default value: [0]

 @Param[in]     h_UccAtmCommon      		- Handle to ATM COMMON object.
 @Param[in]     aal0AvconSwitchExtChannels 	- How many ATM AAL0 switched external channels with Auto VC On feature enabled are required in the system.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigaAal0AvconSwitchExtChannels (t_Handle h_UccAtmCommon, uint16_t aal0AvconSwitchExtChannels);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigStaticThreadsNum

 @Description   Changes the number of static threads from its default configuration.
 
                Static threads are important to the multi-thread mode and the routine
                configures the required number of static threads for the system.
                Default value: [10]

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     staticThreadsNum  - how many static threads are required for the system.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigStaticThreadsNum (t_Handle h_UccAtmCommon, uint8_t staticThreadsNum);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigDynamicThreadsNum

 @Description   Changes the number of dynamic threads from its default configuration.
 
                Dynamic threads are important to the multi-thread mode and the routine
                configures the required number of dynamic threads for the system.
                Default value: [10]

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     dynamicThreadsNum  - how many dynamic threads are required for the system.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigDynamicThreadsNum (t_Handle h_UccAtmCommon, uint8_t dynamicThreadsNum);


#if (!defined(UCC_ATM_AAL2_UNAVAILABLE)) && (!defined(UCC_ATM_AAL1_UNAVAILABLE))
/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigAal2AndCesDataMemory

 @Description   Configures memory partition information for AAL2 and AAL1-CES
                channels to override the default configuration.

 @Param[in]     h_UccAtmCommon  - Handle to ATM COMMON object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigAal2AndCesDataMemory(t_Handle  h_UccAtmCommon,
                                                  uint8_t   memId,
                                                  uint32_t  memAttributes);
#endif /* (!defined UCC_ATM_AAL2_UNAVAILABLE) && (!defined UCC_ATM_AAL1_UNAVAILABLE) */

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigGcraScheduler

 @Description   Configures GCRA scheduler parameters.
 
                Should be called by the user application for working with the GCRA.

 @Param[in]     h_UccAtmCommon                  - Handle to ATM COMMON object.
 @Param[in]     p_UccAtmCommonGcraSchedParam    - gcra parameters

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigGcraScheduler (
            t_Handle h_UccAtmCommon, t_UccAtmCommonGcraSchedParam *p_UccAtmCommonGcraSchedParam);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigVpShapingOverGcra

 @Description   Configures Vp Shaping Over GCRA parameters.
 
                Should be called by the user application for working with Vp Shaping Over GCRA.

 @Param[in]     h_UccAtmCommon                          - Handle to ATM COMMON object.
 @Param[in]     p_UccAtmCommonVpShapingOverGcraParam    - Vp Shaping over GCRA parameters.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigVpShapingOverGcra (
            t_Handle h_UccAtmCommon, t_UccAtmCommonVpShapingOverGcraParam *p_UccAtmCommonVpShapingOverGcraParam);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigApcTxHierarchicalChannels

 @Description   Configures the number of APC internal and external Hierarchical parent channels.
 
                Relevant only in case of Hierarchical + GCRA / Vp Shaping over GCRA
                configuration in the common level, and some of the Hierarchical parent channels are
                included in an APC scheduler.

 @Param[in]     h_UccAtmCommon             - Handle to ATM COMMON object.
 @Param[in]     apcIntHierarchicalChannels - APC Internal Hierarchical parent channels number.
 @Param[in]     apcExtHierarchicalChannels - APC External Hierarchical parent channels number.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigApcTxHierarchicalChannels (  t_Handle h_UccAtmCommon,
                                                        uint16_t apcIntHierarchicalChannels,
                                                        uint16_t apcExtHierarchicalChannels);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigMiniCam

 @Description   Configures MiniCam address resolution mode for a specific UCC.

 @Param[in]     h_UccAtmCommon             - Handle to ATM COMMON object.
 @Param[in]     uccNum                     - the number of UCC which is
                                going to work with MiniCam address mapping mode.
 @Param[in]     p_UccAtmCommonMiniCamParam - MiniCam parameters.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigMiniCam (t_Handle h_UccAtmCommon, e_Ucc uccNum, t_UccAtmCommonMiniCamParam *p_UccAtmCommonMiniCamParam);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigTxHierarchicalChannels

 @Description   Configures AAL5 Hierarchical parameters.
 
 @Param[in]     h_UccAtmCommon - handle to UCC ATM COMMON
 @Param[in]     p_HierarchicalParam - Hierarchical Frame/Cell base parameters.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Change is applied only upon calling UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigTxHierarchicalChannels (t_Handle h_UccAtmCommon, t_UccAtmCommonHierarchicalParam *p_HierarchicalParam);

 
#ifndef UCC_ATM_AAL1_UNAVAILABLE
/**************************************************************************//**
 @Group         ucc_atm_common_config_aal1_grp UCC ATM AAL1 Common Advanced Configuration Unit

 @Description   AAL1 Configuration routines and structures used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   User callback function called by driver for event handling.

                User provides this function. Driver invokes it.
                The user also allocates the memory for the updated CAS block
                and passed its pointer in the common configuration phase.
                The same pointer is passed to the user here with the updated block.

 @Param[in]     appId - AAL1 channel application Id.
 @Param[in]     p_UpdatedCasBlock - Pointer to the updated CAS block.
 @Param[in]     tdmId - The TDM Index (0-7).
 
 @Return        None.

 *//***************************************************************************/
typedef void (t_Aal1CesCasBlockRoutine) ( t_Handle appId, uint8_t *p_UpdatedCasBlock, uint8_t tdmId );

/**************************************************************************//**
 @Description   structure representing CES user parameters
*//***************************************************************************/
typedef struct t_UccAtmCommonCesParam
{
    bool        casMode;            /**< CAS mode selection */
 
    t_Aal1CesCasBlockRoutine
                *f_CasBlockRoutine; /**< User callback for getting the updated Cas block;
                                         Will be called by the driver only for Aal1 Ces channels
                                         with Core Cas Mode enabled, in case of CAS Updated interrupt;
                                         If set to NULL, the CAS block won't be passed to the user. */
 
    t_Handle    p_CasUpdatedBlock;  /**< A pointer to an allocated space for the updated cas block;
                                         Will be passed to the user through f_CasBlockRoutine. */

    bool        crtFirstNibble;     /**< Indicates that the signaling information is in the first nibble
                                         in the CAS block; Relevant only for CAS mode. */

    uint8_t     lowSuperChannel;    /**< Lowest MCC super channel number (0-255). */
    uint8_t     highSuperChannel;   /**< Highest MCC super channel number (0-255). */

    uint8_t     lowTDM;             /**< Lowest TDM index (0-7). */
    uint8_t     highTDM;            /**< Highest TDM index (0-7). */
 
} t_UccAtmCommonCesParam;


/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigAal1Srts

 @Description   Configures SRTS (Synchronous Residual Time Stamp) mode.
 
                Relevant only if there are ATM AAL1 Unstructured channels with SRTS mode.

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     srtsBase            - SRTS base address

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigAal1Srts(t_Handle h_UccAtmCommon, uint32_t srtsBase);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigAal1BdMemory

 @Description   Configures memory partition information for AAL1 buffer descriptors
                to override the default configuration.

 @Param[in]     h_UccAtmCommon  - Handle to ATM COMMON object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigAal1BdMemory(t_Handle    h_UccAtmCommon,
                                          uint8_t     memId,
                                          uint32_t    memAttributes);

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigAal1Ces

 @Description   Configures the parameters for working with CES (Circuit Emulation Service).

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     p_UccAtmCommonCesParam - pointer to CES parameters

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigAal1Ces (t_Handle h_UccAtmCommon, t_UccAtmCommonCesParam *p_UccAtmCommonCesParam);

/** @} */ /* end of ucc_atm_common_aal1_config_grp group */
#endif /* UCC_ATM_AAL1_UNAVAILABLE */
/** @} */ /* end of ucc_atm_common_config_grp group */

/**************************************************************************//**
 @Function      UCC_ATM_COMMON_Init

 @Description   Initializes driver and hardware for Ucc Atm Common.

                Initialize the free Transmit/Receive Internal/External channels
                queues, initialize the Global Atm Param table and initialize
                the multi-thread required fields if this option was chosen.
 
 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmCommon must have been previously created using UCC_ATM_COMMON_Config().

*//***************************************************************************/
t_Error UCC_ATM_COMMON_Init(t_Handle h_UccAtmCommon);


/**************************************************************************//**
 @Function      UCC_ATM_COMMON_Free

 @Description   Free memory of the uccAtmCommon handler.

                Remove multi thread option from the UccAtmCommon(In case it was initialized),
                Free memory of the Global ATM Parameter Table and free the
                channels structures.
 
 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmCommon must have been previously created using UCC_ATM_COMMON_Config().

*//***************************************************************************/
t_Error UCC_ATM_COMMON_Free(t_Handle h_UccAtmCommon);

/** @} */ /* end of ucc_atm_common_init_grp group */
/**************************************************************************//**
 @Group         ucc_atm_common_control_grp UCC ATM Common Control Unit

 @Description   Control Structures and Routines

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      UCC_ATM_COMMON_CheckRasTimeout

 @Description    Goes over all active AAL5 channels of the given Ucc and
                checks if the reassembly timeout of the specific AAL5 channel
                is greater then the given one. If yes, calls application's
                callback routine to inform it about reassembly timeout.
                Also, it updates the reassembly timeout statistics.
 
 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     f_RasTimeout   - user callback to be called in timeout case.

 @Return        None.
*//***************************************************************************/
void    UCC_ATM_COMMON_CheckRasTimeout(t_Handle h_UccAtmCommon,
                                       void (*f_RasTimeout)(t_Handle, uint32_t));

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_IsUsingThreadsForIwOnly

 @Description   Tells whether ATM threads are used for IW only (and not
                for termination).

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.

 @Return        TRUE if ATM threads are used for IW only; FALSE otherwise.
*//***************************************************************************/
bool UCC_ATM_COMMON_IsUsingThreadsForIwOnly(t_Handle h_UccAtmCommon);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_COMMON_DumpRegs

 @Description   Dumps UCC ATM COMMON registers and parameters RAM.

 @Param[in]     h_UccAtmCommon - Handle to UCC ATM Common object.

 @Return        None.

 @Cautions      h_UccAtm must have been previously created using UCC_ATM_COMMON_Config()
                and initialized using UCC_ATM_COMMON_Init().
*//***************************************************************************/
void UCC_ATM_COMMON_DumpRegs(t_Handle h_UccAtmCommon);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_common_control_grp group */
/** @} */ /* end of ucc_atm_common_grp group */
/** @} */ /* end of group ucc_atm_grp */


/**************************************************************************//**
 @Function      UCC_ATM_COMMON_ConfigImaLinksNum

 @Description   Configures the total number of IMA links in the system.

 @Param[in]     h_UccAtmCommon - Handle to UCC ATM COMMON object.
 @Param[in]     imaLinksNum    - Total number of IMA links in the system.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigImaLinksNum(t_Handle   h_UccAtmCommon,
                                         uint8_t    imaLinksNum);


/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwSupport

 @Description   Configures IW support in the UCC ATM COMMON object.

 @Param[in]     h_UccAtmCommon  - Handle to ATM COMMON object.
 @Param[in]     iwSupport       - TRUE to enable IW support;
                                  FALSE for no IW support.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwSupport(t_Handle h_UccAtmCommon, bool iwSupport);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwThreadPagesPoolSize

 @Description   Configures the pool size for the IW thread pages.
 
                If this routine is not used, the driver sets the pool size to
                half the number of threads (rounded up). In the worst case,
                each thread shall have its own page (big memory waste).

 @Param[in]     h_UccAtmCommon          - Handle to ATM COMMON object.
 @Param[in]     iwThreadPagePoolSize    - Pool size for the IW thread pages.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwThreadPagesPoolSize(t_Handle h_UccAtmCommon,
                                                   uint8_t  iwThreadPagePoolSize);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigUseThreadsForIwOnly

 @Description   Determines whether ATM threads are used for IW only (and not
                for termination).

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     useThreadsForIwOnly - TRUE to use threads only for IW (not termination);
                                      FALSE to use threads for IW and termination.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigUseThreadsForIwOnly(t_Handle   h_UccAtmCommon,
                                                 bool       useThreadsForIwOnly);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwAal2AvconIntTxChannels

 @Description   Configures the total number of internal AAL2 IW Tx channels.

                Note : The passed parameter should be the total internal AAL2 IW Tx channels,
                including internal AAL2 IW Tx channels that use the CID multiplexing mode.

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     totalChannels       - Total number of internal AAL2 IW Tx channels.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwAal2AvconIntTxChannels(t_Handle h_UccAtmCommon,
                                                      uint16_t totalChannels);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwAal2AvconExtTxChannels

 @Description   Configures the total number of external AAL2 IW Tx channels.

                Note : The passed parameter should be the total external AAL2 IW Tx channels,
                including external AAL2 IW Tx channels that use the CID multiplexing mode.
 
 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     totalChannels       - Total number of external AAL2 IW Tx channels.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwAal2AvconExtTxChannels(t_Handle h_UccAtmCommon,
                                                      uint16_t totalChannels);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwAal2AvconCidMuxTxChannels

 @Description   Configures the total number of external AAL2 IW Tx channels.

 
 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     cidMuxIntChannels   - Number of internal AAL2 IW Tx channels with CID Multiplexing Mode.
 @Param[in]     cidMuxExtChannels   - Number of external AAL2 IW Tx channels with CID Multiplexing Mode.
 @Param[in]     cidMuxMultiPriority - Are there going to be channels with more than a single CIDs priority.
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwAal2AvconCidMuxTxChannels(t_Handle h_UccAtmCommon,
                                                            uint16_t cidMuxIntChannels,
                                                            uint16_t cidMuxExtChannels,
                                                            bool cidMuxMultiPriority);
 
/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwAal5IntTxChannels

 @Description   Configures the total number of internal AAL5 IW Tx channels.

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     iwAal5IntTxChannels - Number of internal AAL5 IW Tx channels.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwAal5IntTxChannels(t_Handle   h_UccAtmCommon,
                                                 uint16_t   iwAal5IntTxChannels);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwAal5ExtTxChannels

 @Description   Configures the total number of external AAL5 IW Tx channels.
 
                When hierarchical scheduling (either frame-based or cell-based)
                is used, this number must include the number of hierarchical
                queues ("children") used for IW, and NOT the number of
                hierarchical "parents". Note, that hierarchical children channels
                are always external.

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     iwAal5ExtTxChannels - Number of external AAL5 IW Tx channels.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwAal5ExtTxChannels(t_Handle   h_UccAtmCommon,
                                                 uint16_t   iwAal5ExtTxChannels);

/**************************************************************************//*
 @Description   PW parameter structure.
*//***************************************************************************/
typedef struct t_UccAtmCommonPwParams
{
    uint16_t numOfRxTxChannels;
    uint16_t maxNumOfVpiVciPerRxChannel;
    uint16_t numOfInternalIwTxChannels;
    uint16_t numOfExternalIwTxChannels;
}   t_UccAtmCommonPwParams;

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigPwParameters

 @Description   Configures the PW parameters.

 @Param[in]     h_UccAtmCommon - Handle to ATM COMMON object.
 @Param[in]     p_PwParameters - Pointer to PW parameter structure:
                    numOfRxTxChannels          - Number of channels (Rx and Tx).
                    maxNumOfVpiVciPerRxChannel - Maximum number of VPI/VCI per Rx channel.
                    numOfInternalIwTxChannels  - Number of internal IW Tx channels.
                    numOfExternalIwTxChannels  - Number of external IW Tx channels.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigPwParameters(t_Handle h_UccAtmCommon,t_UccAtmCommonPwParams* p_PwParameters);


/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigNoGcraForIwAal5

 @Description   Determines whether AAL5 IW channels are limited to the
                APC scheduler.

 @Param[in]     h_UccAtmCommon  - Handle to ATM COMMON object.
 @Param[in]     noGcraForIwAal5 - TRUE if AAL5 IW channels do not use GCRA scheduler;
                                  FALSE if AAL5 IW channels use GCRA scheduler.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigNoGcraForIwAal5(t_Handle   h_UccAtmCommon,
                                             bool       noGcraForIwAal5);


/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwMulticastGroupsNum

 @Description   Configures the number of AAL5 multicast groups (IW mode only).
                [Total: 0, Internal: 0]
 
                The first internalMulticastGroups multicast groups shall use
                internal Tx queue descriptors, and the rest shall be external.

 @Param[in]     h_UccAtmCommon          - Handle to ATM COMMON object.
 @Param[in]     totalMulticastGroups    - Number of multicast groups.
 @Param[in]     internalMulticastGroups - Number of internal multicast groups.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
 
 @Cautions      internalMulticastGroups must not exceed totalMulticastGroups.
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwMulticastGroupsNum(t_Handle  h_UccAtmCommon,
                                                  uint16_t  totalMulticastGroups,
                                                  uint16_t  internalMulticastGroups);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_ConfigIwMulticastBdRingLen

 @Description   Configures the length of AAL5 multicast BD ring (IW mode only).

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     multicastBdRingLen  - Length of multicast BD ring.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_COMMON_Config() and before UCC_ATM_COMMON_Init().
*//***************************************************************************/
t_Error UCC_ATM_COMMON_ConfigIwMulticastBdRingLen(t_Handle  h_UccAtmCommon,
                                                  uint16_t  multicastBdRingLen);

/**************************************************************************//*
 @Description   IW details structure (interface with IW driver).
*//***************************************************************************/
typedef struct t_UccAtmCommonIwDetails
{
    uint32_t    intTctBaseAddr;         /**< The base address of the internal TCT */
    uint32_t    extTctBaseAddr;         /**< The base address of the external TCT */
    uint32_t    intAal5TxQdBaseAddr;    /**< The base address of the internal AAL5 TxQD */
    uint32_t    extAal5TxQdBaseAddr;    /**< The base address of the external AAL5 TxQD */
    uint32_t    extAal2TxQdBaseAddr;    /**< The base address of the external AAL2 TxQD (ILLEGAL_BASE if none) */
    uint32_t    extAal2TxQdExtensionBaseAddr;/**< The base address of the external AAL2 SAR Extension TxQD,
                                             Relevant for IW (ILLEGAL_BASE if none) */
    uint32_t    autoVconBase;           /**< The base address of the Auto-VC-On bits array */
    uint32_t    multicastBdBase;        /**< The base address of the multicast BDs */
} t_UccAtmCommonIwDetails;

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_GetIwDetails

 @Description   Gets the IW details from UCC ATM COMMON (interface with IW driver).

 @Param[in]     h_UccAtmCommon  - Handle to ATM COMMON object.
 @Param[in]     p_IwDetails     - Pointer to IW details structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_COMMON_GetIwDetails(t_Handle                h_UccAtmCommon,
                                    t_UccAtmCommonIwDetails *p_IwDetails);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_GetRxChannelAppId

 @Description   Returns the application ID of an Rx channel, according to the
                channel code.
 
                This function is used by IW interrupt handling routine.

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     rxChannelCode    - Rx channel code.

 @Return        The application ID of the Rx channel.
*//***************************************************************************/
t_Handle UCC_ATM_COMMON_GetRxChannelAppId(t_Handle h_UccAtmCommon, uint16_t rxChannelCode);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_GetTxChannelAppId

 @Description   Returns the application ID of a Tx channel, according to the
                channel code.
 
                This function is used by IW interrupt handling routine.

 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.
 @Param[in]     txChannelCode    - Tx channel code.

 @Return        The application ID of the Rx channel.
*//***************************************************************************/
t_Handle UCC_ATM_COMMON_GetTxChannelAppId(t_Handle h_UccAtmCommon, uint16_t txChannelCode);

/**************************************************************************//*
 @Function      UCC_ATM_COMMON_GetGlobalPramBase

 @Description   Returns the global parameters base address.
 
 @Param[in]     h_UccAtmCommon      - Handle to ATM COMMON object.

 @Return        The global parameters base address.
*//***************************************************************************/
uint32_t UCC_ATM_COMMON_GetGlobalPramBase(t_Handle h_UccAtmCommon);

/**************************************************************************//**
 @Group         ucc_atm_common_grp UCC ATM Common

 @Description   Ucc Atm Common

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_common_init_grp UCC ATM Common Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_common_config_grp UCC ATM Common Advanced Configuration Unit

 @Description   Configuration routines and structures used to change default values

 @{
*//***************************************************************************/

#ifndef UCC_ATM_AAL2_UNAVAILABLE
/**************************************************************************//**
 @Group         ucc_atm_aal2_common_advanced_init_grp UCC ATM AAL2 Common Advanced Configuration Unit

 @Description   AAL2 Common Configuration Routines and Structures used to change default values.
 
 @{
*//***************************************************************************/
/**************************************************************************//**
 @Description   enum representing the different ways to allocate the CID Map table in the system
*//***************************************************************************/
typedef enum e_UccAtmAal2CommonCidMapAlloc
{
      e_UCC_ATM_COMMON_AAL2_CID_MAP_COMBINED_ALLOCATION /**< map allocation - combined - internal(for internal channels) + external(for external channels) */
    , e_UCC_ATM_COMMON_AAL2_CID_MAP_INTERNAL_ALLOCATION /**< map allocation - internal */
    , e_UCC_ATM_COMMON_AAL2_CID_MAP_EXTERNAL_ALLOCATION /**< map allocation - external */
}e_UccAtmAal2CommonCidMapAlloc;

/**************************************************************************//**
 @Description   structure representing ucc atm common aal2 Threshold sets parameters
*//***************************************************************************/
typedef struct t_UccAtmAal2CommonThresholdSetsParam
{
    uint16_t    higherOfHighThreshold; /**< Number of packets in queue that corresponds to full stop scenario - queue is too full */
    uint16_t    lowerOfHighThreshold;  /**< Number of packets in queue that corresponds to decelerate scenario - queue is about to be too full */
    uint16_t    higherOfLowThreshold;  /**< Number of packets in queue that corresponds to accelerate scenario - queue is about to be empty */
    uint16_t    lowerOfLowThreshold;   /**< Number of packets in queue that corresponds to full speed scenario - queue is too empty */
} t_UccAtmAal2CommonThresholdSetsParam;

/** @} */ /* end of ucc_atm_aal2_common_advanced_init_grp group */
#endif /* UCC_ATM_AAL2_UNAVAILABLE */
/** @} */ /* end of ucc_atm_common_config_grp group */
/** @} */ /* end of ucc_atm_common_init_grp group */
/** @} */ /* end of ucc_atm_common_grp group */

#endif /* __UCC_ATM_COMMON_EXT_H */
