/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_ext.h

 @Description   UCC Atm external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_ATM_EXT_H
#define __UCC_ATM_EXT_H

#include "xx_ext.h"
#include "qe_ext.h"
#include "ucc_atm_common_ext.h"
#include "ucc_fast_ext.h"


/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
@Group         ucc_atm_lvl_grp UCC ATM

@Description   Ucc Atm

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_init_grp UCC ATM Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/***********/
/* Defines */
/***********/

#define STANDARD_CELL_LENGTH    52
#define PAYLOAD_LENGTH          48

#define AAL1_DUMMY_CELL_SIZE    47  /* AAL1 Dummy Cell Size */



/**************************************************************************//**
 @Description   UCC ATM Operation mode.
 *//***************************************************************************/
typedef enum e_UccAtmOperationMode
{
      e_UCC_ATM_OP_MODE_AAL0   = 0x00000000       /**< Operation Mode - AAL0 */
    , e_UCC_ATM_OP_MODE_AAL1   = 0x00000001       /**< Operation Mode - AAL1 */
    , e_UCC_ATM_OP_MODE_CES    = 0x00000002       /**< Operation Mode - CES  */
    , e_UCC_ATM_OP_MODE_AAL1_CES =
        e_UCC_ATM_OP_MODE_AAL1 | e_UCC_ATM_OP_MODE_CES /** Operation Mode - AAL1 and CES */
    , e_UCC_ATM_OP_MODE_AAL2   = 0x00000004       /**< Operation Mode - AAL2 */
    , e_UCC_ATM_OP_MODE_AAL5   = 0x00000008       /**< Operation Mode - AAL5 */
    , e_UCC_ATM_OP_MODE_IMA    = 0x00000010       /**< Operation Mode - IMA  */
    , e_UCC_ATM_OP_MODE_MSP    = 0x00000020       /**< Operation Mode - MSP  */
    , e_UCC_ATM_OP_MODE_SATM   = 0x00000040       /**< Operation Mode - SATM */
    , e_UCC_ATM_OP_MODE_IW     = 0x00000080       /**< Operation Mode - IW   */
} e_UccAtmOperationMode;

/**************************************************************************//**
 @Description   UCC ATM OAM Filtering VCIs.
                The VCIs that need to be filtered should be ORed.
 *//***************************************************************************/
typedef enum e_UccAtmOamFiltering
{
      e_UCC_ATM_OAM_FILTER_VCI_3  = 0x1000     /**< VCI 3 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_4  = 0x0800     /**< VCI 4 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_6  = 0x0200     /**< VCI 6 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_7  = 0x0100     /**< VCI 7 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_8  = 0x0080     /**< VCI 8 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_9  = 0x0040     /**< VCI 9 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_10 = 0x0020     /**< VCI 10 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_11 = 0x0010     /**< VCI 11 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_12 = 0x0008     /**< VCI 12 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_13 = 0x0004     /**< VCI 13 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_14 = 0x0002     /**< VCI 14 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_15 = 0x0001     /**< VCI 15 should be passed to the Raw Cell queue */
    , e_UCC_ATM_OAM_FILTER_VCI_ALL = 0x1bff     /**< All possible VCIs should be passed to the Raw Cell queue */
 
} e_UccAtmOamFiltering;

/**************************************************************************//**
 @Description   Interrupt Queues
 *//***************************************************************************/
typedef enum e_UccAtmIntQ
{
    e_UCC_ATM_INTQ_1,   /**< Interrupt queue number 1 */
    e_UCC_ATM_INTQ_2,   /**< Interrupt queue number 2 */
    e_UCC_ATM_INTQ_3,   /**< Interrupt queue number 3 */
    e_UCC_ATM_INTQ_4,   /**< Interrupt queue number 4 */
    UCC_ATM_INTQ_MAX
} e_UccAtmIntQ;

/**************************************************************************//**
 @Description   Interrupt Queues type
 *//***************************************************************************/
typedef enum e_UccAtmIntQType
{
    e_UCC_ATM_INTQ_TYPE_RX,     /**< interrupt queue for receive interrupts */
    e_UCC_ATM_INTQ_TYPE_TX,     /**< interrupt queue for transmit interrupts */
    e_UCC_ATM_INTQ_TYPE_IMA     /**< interrupt queue for IMA interrupts */
} e_UccAtmIntQType;

/**************************************************************************//**
 @Description   structure representing BD ring length parameters
*//***************************************************************************/
typedef struct t_UccAtmBdRingLengthParam
{
    uint16_t bdRingLen;     /**< BD ring length */
    uint32_t channels;      /**< Max channels with the specific BD ring length belong to this Ucc;
                                 In case of using full duplex channel, there is a need to add two to this variable to take into account -
                                 one chanel for Rx and one channel for Tx;
                                 NOTE:  1: This parameter relevant only in case of AAL0,AAL1,AAL5
                                        2: In case of hierarchical mode , don't count the hierarchical channel but count the
                                        number of queues of the hierarchical channel under this UCC */
} t_UccAtmBdRingLengthParam;

/**************************************************************************//**
 @Description   structure representing Aal2 BD ring length parameters
*//***************************************************************************/
typedef struct t_UccAtmAal2BdRingLengthParam
{
    uint16_t bdRingLen;         /**< BD ring length */
    uint32_t queueDescriptors;  /**< Max AAL2 Queue Descriptors (Rx + Tx queue descriptors) with the specific BD ring length belong to this Ucc. */
 
} t_UccAtmAal2BdRingLengthParam;

/**************************************************************************//**
 @Description   structure representing raw cell parameters
*//***************************************************************************/
typedef struct t_UccAtmRawCellParam
{
    /* System integration parameters */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information */
    uint32_t            dataMemAttributes;  /**< Memory partition information for data buffers */

    /* Application interface */
    t_RxStoreFunction   *f_RxStore;         /**< The Rx store routine */

} t_UccAtmRawCellParam;

/**************************************************************************//**
 @Description   Types of AAL connections .
 *//***************************************************************************/
typedef enum e_UccAtmAal
{
      e_UCC_ATM_AAL0           /**< Adaptation Layer 0 */
    , e_UCC_ATM_AAL1           /**< Adaptation Layer 1 */
    , e_UCC_ATM_AAL1_CES       /**< Adaptation Layer 1 - CES Mode */
    , e_UCC_ATM_AAL2           /**< Adaptation Layer 2 */
    , e_UCC_ATM_AAL5           /**< Adaptation Layer 5 */
    , e_UCC_ATM_PW             /**< PW Emulation Layer */
    , MAX_UCC_ATM_AAL_TYPES
} e_UccAtmAal;

/**************************************************************************//**
 @Description   User callback function called by driver for event handling.

                User provides this function. Driver invokes it.

 @Param[in]     h_App     - the initial h_App given by the user at driver
                            initialization.
 @Param[in]     errors    - the events that caused the exception to be invoked by
                            the driver; These events are divided to two some of them are
                            UCCE events and some of them are the interrupt queue events (flags is the type of the errors)
 @Param[in]     flags     - Determine the errors source - is it from the UCCE register or from the Interrupt queue
                            0 - the UCCE is the source of the errors
                            1 and higher - the Interrupt Queue is the source of the errors; there are different types of interrupt queue like policer, AAL2 and so on.
 
 @Return        None.
 *//***************************************************************************/
typedef void (t_UccAtmExceptions)(t_Handle h_App, uint32_t errors, uint32_t flags);

/**************************************************************************//**
 @Description   Ucc Atm initialization structure, used when calling UCC_ATM_Config().
*//***************************************************************************/
typedef struct t_UccAtmParam
{
    /* ATM basic parameters */
    e_UccAtmOperationMode   atmOpMode;          /**< different atm modes:AAL5,AAL2,AAL0,AAL1,MSP , the modes can be ORed */
    uint32_t                channels;           /**< Max channels with the default/configured BD ring length belong to this Ucc;
                                                     In case of using full duplex channel, there is a need to add two to this variable to take into account one chanel for Rx and one channel for Tx;
                                                     NOTE: 1: This parameter relevant only in case of AAL0,AAL1,AAL5
                                                           2: In case of hierarchical mode , don't count the hierarchical channel but count the
                                                           number of queues of the hierarchical channel under this UCC */
    uint16_t                vpMask;             /**< VP mask which determines which available address information bits are used for vpi in address compression */
    uint8_t                 portMask;           /**< port mask which determines which available address information bits are used for port in address compression */
    uint8_t                 atmCtlPhyDevMask;   /**< Atm Controller phy device mask which determines which available address information bits are used for phy device in address compression */
    uint32_t                maxVcPerVp;         /**< 2^[Max number of bits in VC masks] */
    bool                    OC12;               /**< indication if there is an intention to work in oc12 speed */
    t_UccAtmRawCellParam    rawCellParam;       /**< raw cell param */

    /* System integration parameters */
    uint32_t                baseAddress;        /**< Base address of UCC ATM registers in memory. */
    t_Handle                h_UccAtmCommon;     /**< Handle to the UCC ATM Common handler */
    e_Ucc                   uccId;              /**< UCC ID */
    int                     irq;                /**< UCC interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle                h_App;              /**< Handle to the application */
    t_UccAtmExceptions      *f_Exceptions;      /**< The exception invoked by driver upon events;
                                                     User may choose to set it to NULL by that avoiding
                                                     recption of events (no harm done). */
} t_UccAtmParam;



/**************************************************************************//**
 @Description   address look-up mechanism
 *//***************************************************************************/
typedef enum e_UccAtmAddressMapping
{
      e_UCC_ATM_ADDRESS_COMPRESSION                 /**< Address Compression address mapping method */
    , e_UCC_ATM_EXTRA_HEADER_CC                     /**< Extract channel code from header address mapping method  */
    , e_UCC_ATM_EXTRA_HEADER_CC_PROTECTED           /**< Extract channel code from header (With protection) address mapping method  */
    , e_UCC_ATM_MINI_CAM                            /**< Mini Cam address mapping method */
    , e_UCC_ATM_ADDRESS_COMPRESSION_AND_MINI_CAM    /**< Combination of Address compression and Mini Cam address mapping method */
} e_UccAtmAddressMapping;

/**************************************************************************//**
 @Description   UCC ATM receiver decoding mode
*//***************************************************************************/
typedef enum e_UccAtmGUMRReceiverDecodingMethodRENC
{
    e_UCC_ATM_RX_ENCODING_NRZ          = e_UCC_FAST_RX_ENCODING_NRZ      , /**< nrz  */
    e_UCC_ATM_RX_ENCODING_NRZI         = e_UCC_FAST_RX_ENCODING_NRZI     , /**< nrzi */
    e_UCC_ATM_RX_ENCODING_RESERVED0    = e_UCC_FAST_RX_ENCODING_RESERVED0, /**<      */
    e_UCC_ATM_RX_ENCODING_RESERVED1    = e_UCC_FAST_RX_ENCODING_RESERVED1  /**<      */
} e_UccAtmGUMRReceiverDecodingMethodRENC;

/**************************************************************************//**
 @Description   UCC ATM transmitter encoding mode

*//***************************************************************************/
typedef enum e_UccAtmGUMRTransmitterEncodingMethodTENC
{
    e_UCC_ATM_TX_ENCODING_NRZ          = e_UCC_FAST_TX_ENCODING_NRZ      , /**< nrz  */
    e_UCC_ATM_TX_ENCODING_NRZI         = e_UCC_FAST_TX_ENCODING_NRZI     , /**< nrzi */
    e_UCC_ATM_TX_ENCODING_RESERVED0    = e_UCC_FAST_TX_ENCODING_RESERVED0, /**<      */
    e_UCC_ATM_TX_ENCODING_RESERVED1    = e_UCC_FAST_TX_ENCODING_RESERVED1  /**<      */
} e_UccAtmGUMRTransmitterEncodingMethodTENC;

/**************************************************************************//**
 @Description   UCC ATM diagnostic mode
*//***************************************************************************/
typedef enum e_UccAtmGUMRDiagnosticModeDIAG
{
    e_UCC_ATM_DIAGNOSTIC_NORMAL            = e_UCC_FAST_DIAGNOSTIC_NORMAL            , /**< normal             */
    e_UCC_ATM_DIAGNOSTIC_LOCAL_LOOP_BACK   = e_UCC_FAST_DIAGNOSTIC_LOCAL_LOOP_BACK   , /**< loop back          */
    e_UCC_ATM_DIAGNOSTIC_AUTO_ECHO         = e_UCC_FAST_DIAGNOSTIC_AUTO_ECHO         , /**< echo               */
    e_UCC_ATM_DIAGNOSTIC_LOOP_BACK_AND_ECHO= e_UCC_FAST_DIAGNOSTIC_LOOP_BACK_AND_ECHO  /**< loop back and echo */
} e_UccAtmGUMRDiagnosticModeDIAG;


/**************************************************************************//**
 @Function      UCC_ATM_Config

 @Description   Configures a UCC for ATM mode based on user parameters.

                The UCC ATM handle can then be initialized using UCC_ATM_Init()
                and enabled using UCC_ATM_Enable().

 @Param[in]     p_UccAtmParam - user parameters for UCC ATM.

 @Return        Handle to created UCC ATM if success; NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.
*//***************************************************************************/
t_Handle UCC_ATM_Config(t_UccAtmParam *p_UccAtmParam);

/**************************************************************************//**
 @Group         ucc_atm_config_grp UCC ATM Advanced Configuration Unit

 @Description   Configuration Routines and Structures used to change default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Interrupt Queue initialization structure, used when calling UCC_ATM_ConfigIntQ().
 *//***************************************************************************/
typedef struct t_UccAtmIntQueueParam
{
    uint32_t         intqNumOfEntries;/**< The number of entries in the interrupt queue */
    uint16_t         intInitCnt;      /**< Interrupt counter initial value */
    e_UccAtmIntQType intqType;        /**< interrupt queue type; indication if the interrupt queueu is for Transmit event/Receive events/IMA events; There is no possibility to get receive and transmit interrupts on the same interrupt queue */
} t_UccAtmIntQueueParam;

/**************************************************************************//**
 @Description   Free buffer pool initialization structure.
 *//***************************************************************************/
typedef struct t_UccAtmFbpParam
{
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information;
                                                 NOTE: Due to firmware limitation, all FBP buffers must share
                                                 an identical value of their 4 most significant bits;
                                                 For better bus performance this pointer should be 32-bytes aligned. */
    uint32_t            dataMemAttributes;  /**< Memory attributes for FBP buffers */
    uint16_t            numOfBuffers;       /**< The number of the buffers in the pool */
    uint16_t            redLineBuffers;     /**< The number of the buffers in the pool for Red line interrupt */
 
} t_UccAtmFbpParam;

/**************************************************************************//**
 @Description   structure representing raw cell configuration parameters
*//***************************************************************************/
typedef struct t_UccAtmRawCellConfigParam
{
    uint16_t            bdRingLength;       /**< the required raw cell BD ring length; The default is the first UCC BD ring length. */
    bool                interruptEnable;    /**< the option to work with interrupts [TRUE] */
    bool                timeStampEnable;    /**< enable time stamp [FALSE]*/
 
} t_UccAtmRawCellConfigParam;

/**************************************************************************//**
 @Description   PM block size (number of cells between two FMC cells)
*//***************************************************************************/
typedef enum e_PmBlockSize
{
    e_BLOCK_128_CELLS   = 128,      /**< PM block size is 128 cells */
    e_BLOCK_256_CELLS   = 256,      /**< PM block size is 256 cells */
    e_BLOCK_512_CELLS   = 512,      /**< PM block size is 512 cells */
    e_BLOCK_1024_CELLS  = 1024      /**< PM block size is 1024 cells */
 
} e_PmBlockSize;

/**************************************************************************//**
 @Description   structure representing PM Table configuration parameters
*//***************************************************************************/
typedef struct t_PmTableParam
{
    uint8_t         pmTableIndex;           /**< PM Table Index : 0 - <UCC total PMTs-1> */
    e_PmBlockSize   blockSize;              /**< Number of cells between two FMC cells  */
    uint32_t        pmCellHeader;           /**< PM cell header                         */
    bool            fmcTimeStampEnable;     /**< FMC time stamp enable/disable          */
 
} t_PmTableParam;

 /**************************************************************************//**
 @Description   Policer initialization structure, used when calling UCC_ATM_ConfigPolicer().
 *//***************************************************************************/
typedef struct t_UccAtmPolicerParam
{
    uint8_t     intPolicerTables;               /**< Number of internal policer tables
                                                     (not relevant in case of Mini-Cam address resolution mode) */
    uint16_t    extPolicerTables;               /**< Number of external policer tables
                                                     (not relevant in case of Mini-Cam address resolution mode) */
    uint8_t     maxPolicerTablesPerMiniCamPort; /**< The maximum Policer tables number
                                                     that can be used for every MiniCam port
                                                     NOTE:The number is limited to 15,
                                                     if set to zero ,it means that there is no need to initialize
                                                     policer table for the Mini-Cam feature */
    bool        isInternalMiniCamPolicerTables; /**< Indicates if the policer tables
                                                     that are related to Mini-Cam port resides in the internal
                                                     memory or in the external memory */
} t_UccAtmPolicerParam;

 /**************************************************************************//**
 @Description   Address Compression initialization structure, used when calling UCC_ATM_COMMON_ConfigAddrCompression().
 *//***************************************************************************/
typedef struct t_UccAtmAddrCompressionParam
{
    uint8_t     addrTblMemId;           /**< Memory partition ID for address look-up tables */
    uint32_t    addrTblMemAttributes;   /**< Memory partition attributes for address look-up tables */
    bool        internalVpTables;       /**< Indication if the Vp tables will be allocated in the internal memory */
    bool        checkUnAllocBits;       /**< Enable/disable unallocated bits checking (relevant only in address compression):
                                             FALSE - Do not check unallocated bits;
                                             TRUE - Check unallocated bits */
} t_UccAtmAddrCompressionParam;


/**************************************************************************//**
 @Function      UCC_ATM_ConfigRxStaticThreadsNum

 @Description   Changes the number of receive static threads from its default configuration.
 
                The number of receive static threads for the specific UCC; The number is relevant in case of working in multi-thread mode;
                Default value: [1].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     rxStaticThreadsNum     - the number of receive static threads (for the specific UCC).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigRxStaticThreadsNum (t_Handle h_UccAtm, uint32_t rxStaticThreadsNum);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigTxStaticThreadsNum

 @Description   Changes the number of transmit static threads from its default configuration.
 
                The number of transmit static threads for the specific UCC; The number is relevant in case of working in multi-thread mode;
                Default value: [1].
 
 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     txStaticThreadsNum     - the number of transmit static threads (for the specific UCC).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigTxStaticThreadsNum (t_Handle h_UccAtm, uint32_t txStaticThreadsNum);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigMultiThreadOperationMode

 @Description   Changes the operation mode of multi-thread from its default configuration.
 
                Default value:[TRUE].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     isMultiThread     - TRUE to work in multi-thread operation mode
                                         FALSE not to work in multi-thread operation mode.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigMultiThreadOperationMode (t_Handle h_UccAtm, bool isMultiThread);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigPolicer

 @Description   Configures policer parameters.
 
 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     p_UccAtmPolicerParam  - Policer parameters to be config.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().

 @Cautions      In case the user asked to work with:
                Address compression and MiniCam address resolution modes and
                operate policer , there is a need to add the required
                number of policer tables to the internal/external policer tables numbers.
*//***************************************************************************/
t_Error UCC_ATM_ConfigPolicer(t_Handle h_UccAtm, t_UccAtmPolicerParam *p_UccAtmPolicerParam);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigBdRingLen

 @Description   Changes the BD ring length from its default configuration.
 
                The length of the BD ring is common to the Tx and Rx BD rings for
                the channels under the specific UCC;
                Default value: [16].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     bdRingLen     - the length of the BD ring.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigBdRingLen (t_Handle h_UccAtm, uint16_t bdRingLen);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigAdditionalBdRingLen

 @Description   Adds BD ring lengths, in addition to the first (default/configured) one.
 
 @Param[in]     h_UccAtm                - Handle to UCC ATM object.
 @Param[in]     additionalBdRingLengths  - Number of additional (to the default one) BD ring lengths.
 @Param[in]     p_UccAtmBdRingLengthsParam    - BD ring lengths parameters array.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigAdditionalBdRingLengths(t_Handle h_UccAtm, uint8_t additionalBdRingLengths, t_UccAtmBdRingLengthParam *p_UccAtmBdRingLengthsParam);


/**************************************************************************//**
 @Function      UCC_ATM_ConfigVciFiltering

 @Description   Changes the VCI filtering value from its default configuration.
 
                The VCI filtering determines if to send cells with this VCI value
                to the raw cell queue.
                Note : vciFilteringValue should be built by ORing values
                out of the e_UccAtmOamFiltering enum.
                Default value: [0].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     vciFilteringValue     - the value of the VCI filtering.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigVciFiltering (t_Handle h_UccAtm, uint16_t vciFilteringValue);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigAal1DummyCell

 @Description   Changes the AAL1 dummy cell content from its default configuration.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     p_Aal1DummyCell - AAL1 dummy cell content.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Change is applied only upon calling UCC_ATM_Init.
*//***************************************************************************/
t_Error UCC_ATM_ConfigAal1DummyCell(t_Handle h_UccAtm, uint8_t *p_Aal1DummyCell);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigHierarchicalWithFrameBasedOnly

 @Description   Changes the Tx Hierarchical channels with only frame based queues
                from its default configuration.
                Default value: [0].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     hierarchicalWithFrameBasedOnly - Tx Hierarchical channels with only frame based queues Number.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Change is applied only upon calling UCC_ATM_Init.
*//***************************************************************************/
t_Error UCC_ATM_ConfigHierarchicalWithFrameBasedOnly(t_Handle h_UccAtm, uint16_t hierarchicalWithFrameBasedOnly);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigVpSwitchMode

 @Description   Changes the VP Switch mode from its default configuration.
 
                Currently supported only for Mini-Cam address resolution.
                Default value: [FALSE].

 @Param[in]     h_UccAtm                - Handle to UCC ATM object
 @Param[in]     enableVpSwitchMode      - TRUE to work in VP switch mode
                                          FALSE not to work in VP switch mode

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigVpSwitchMode (t_Handle h_UccAtm, bool enableVpSwitchMode);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigExceptions

 @Description   Changes the different UCC exceptions from its default configuration.

                The routine configures the events user expects to receive an interrupt upon occurrence.
                A combination of the following available events.
                #UCC_ATM_UCCE_TIRU, #UCC_ATM_UCCE_GRLI, #UCC_ATM_UCCE_GBPB, #UCC_ATM_UCCE_INTO0, #UCC_ATM_UCCE_INTO1 , #UCC_ATM_UCCE_INTO2, #UCC_ATM_UCCE_INTO3
                (TIRU = Transmit Internal Underrun, GRLI = Global Red Line Interrupt(Fpb), GBPB = Global Buffer Pool Busy, INTO - Interrupt Queue Overflow.)
                Default value: [UCC_ATM_UCCE_INTO0].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     exceptions      - the events user expects to receive an interrupt upon occurrence.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigExceptions (t_Handle h_UccAtm, uint32_t exceptions);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigIdleCellMode

 @Description   Changes the Idle Cell Mode from its default configuration.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     idleCellMode  - Idle Cell Mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigIdleCellMode(t_Handle h_UccAtm, bool idleCellMode);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigAddrCompression

 @Description   Configures the parameters for working with Address Compression as an address mapping method.

 @Param[in]     h_UccAtm                - Handle to UCC ATM object.
 @Param[in]     p_AddrCompressionParam  - address compression parameters structure.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
 
 @Cautions      The routine should be called in case that the user ask to initialize
                the specific UCC to work in Mini-Cam and classical address compression
                together and he requires some parameter changes.
*//***************************************************************************/
t_Error UCC_ATM_ConfigAddrCompression(t_Handle                      h_UccAtm,
                                      t_UccAtmAddrCompressionParam  *p_AddrCompressionParam);


/**************************************************************************//**
 @Function      UCC_ATM_ConfigIntQ

 @Description   Configures interrupt queue parameters for one or more interrupt queues.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     totalIntq  - how many interrupt queues to config(1-4).
 @Param[in]     p_UccAtmIntQueueParam  - Intq params to config.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigIntQ(t_Handle h_UccAtm, uint8_t totalIntq, t_UccAtmIntQueueParam *p_UccAtmIntQueueParam);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigFbp

 @Description   Configures Free Buffer Pool parameters for one or more Free Buffer Pools.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     totalFbp  -how many FBPs to config.
 @Param[in]     p_UccAtmFbpParam  - Fbp params to config.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigFbp(t_Handle h_UccAtm, uint8_t totalFbp, t_UccAtmFbpParam *p_UccAtmFbpParam);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigRawCell

 @Description   Configures raw cell parameters.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     p_RawCellParam  - raw cell parameters to be config.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigRawCell (t_Handle h_UccAtm, t_UccAtmRawCellConfigParam *p_RawCellParam);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigPerformanceMonitoringTablesNum

 @Description   Configures Performance Monitoring Tables Number.
 
                If the user application needs to use PM in this ATM UCC,
                he should call this function.

 @Param[in]     h_UccAtm    - Handle to UCC ATM object.
 @Param[in]     numOfPmTables - Performance Monitoring tables number.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigPerformanceMonitoringTablesNum(t_Handle h_UccAtm, uint8_t numOfPmTables);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_UccAtm        - Handle to UCC ATM object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigFirmwareExternalStructsMemory(t_Handle    h_UccAtm,
                                                    uint8_t     memId,
                                                    uint32_t    memAttributes);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigInitTx

 @Description   Changes the UCC to be initialized to transmit operation.
 
                Configure the UCC to be initialized to transmit operation.
                Default value: [TRUE].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     initTx     - TRUE to initialize for transmission
                                  FALSE to initialize not to work for transmission.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigInitTx(t_Handle h_UccAtm, bool initTx);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigInitRx

 @Description   Changes the UCC to be initialized to receive operation.
 
                Configure the UCC to be initialized to receive operation.
                Default value: [TRUE].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     initRx     - TRUE to initialize for reception
                                  FALSE to initialize not to work for reception.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigInitRx(t_Handle h_UccAtm, bool initRx);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigGrantSupport

 @Description   Changes the grant support from its default configuration.

                Default value: [FALSE].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     grantSupport     - TRUE to provide grant support
                                        FALSE not to provide frant support.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigGrantSupport(t_Handle h_UccAtm, bool grantSupport);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigTSA

 @Description   Changes the TSA field from its default configuration.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     useTsa     - TRUE to use TSA,
                                  FALSE to use NMSI.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigTSA(t_Handle h_UccAtm, bool useTsa);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigRTSM

 @Description   Changes the RTSM from its default configuration.

                Default value: [FALSE].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     workWithRtsm     - TRUE to send flags/syncs between frames.
                                        FALSE to send idles between frames.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigRTSM(t_Handle h_UccAtm, bool workWithRtsm);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigRxVirtFifoSize

 @Description   Changes the receive virtual Fifo size from its default configuration.

                Rx virtual FIFO Size register, [0x100]
                For OC12, [0x400].
                For OC3, [0x100].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     rxVirtFifosize     - receive virtual fifo size.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigRxVirtFifoSize(t_Handle h_UccAtm, uint16_t rxVirtFifosize);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigRxVirtFifoEmergThresh

 @Description   Changes the receive virtual Fifo Emergency Threshold from its default configuration.
 
                Rx virtual FIFO Emergency Threshold, [0x88]
                For OC12, [0x200].
                For OC3, [0x80].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     rxVirtFifoEmergThresh     - receive virtual fifo emergency threshold size.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigRxVirtFifoEmergThresh(t_Handle h_UccAtm, uint16_t rxVirtFifoEmergThresh);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigRxVirtFifoSpecEmergThresh

 @Description   Changes the Rx virtual FIFO Special Emergency Threshold from its default configuration.

                Rx virtual FIFO Special Emergency Threshold, [0xD0]
                For OC12, [0x300].
                For OC3, [0xC0].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     rxVirtFifoSpecEmergThresh     - Rx virtual FIFO Special Emergency Threshold value.

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigRxVirtFifoSpecEmergThresh(t_Handle h_UccAtm, uint16_t rxVirtFifoSpecEmergThresh);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigTxVirtFifoSize

 @Description   Changes the transmit virtual fifo size from its default configuration.

                Tx virtual FIFO Size register, [0x100]
                For OC12, [0x400].
                For OC3, [0x100].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     txVirtFifoSize     -transmit virtual fifo size value

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigTxVirtFifoSize(t_Handle h_UccAtm, uint16_t txVirtFifoSize);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigTxVirtFifoEmergThresh

 @Description   Changes the transmit virtual fifo emergency threshold from its default configuration.

                Tx virtual FIFO Emergency Threshold, [0x88]
                For OC12, [0x200].
                For OC3, [0x80].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     txVirtFifoEmergThresh     - Tx virtual FIFO Emergency Threshold value.

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigTxVirtFifoEmergThresh(t_Handle h_UccAtm, uint16_t txVirtFifoEmergThresh);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigTxVirtFifoTxThresh

 @Description   Changes the transmit virtual fifo threshold size from its default configuration.

                Tx virtual FIFO Tx Threshold, [0x40]
                For OC12, [0x40].
                For OC3, [0x40].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     txVirtFifoTxThresh     - Tx virtual FIFO Tx Threshold value.

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigTxVirtFifoTxThresh(t_Handle h_UccAtm, uint16_t txVirtFifoTxThresh);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigTxPollingTimer

 @Description   Changes the transmit polling timer from its default configuration.

                Default value: [256].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     txPollingTimer     - Tx Polling Timer value

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigTxPollingTimer(t_Handle h_UccAtm, uint16_t txPollingTimer);

/**************************************************************************//**
 @Function      UCC_ATM_ConfigDiagnosticMode

 @Description   Changes the diagnostic mode from its default configuration.
                Please set 'newVal' according to the following description:
                Default value: [e_UCC_FAST_DIAGNOSTIC_NORMAL].

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     diagMode     - diagnostic mode value.

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigDiagnosticMode(t_Handle h_UccAtm, e_UccAtmGUMRDiagnosticModeDIAG diagMode);

/**************************************************************************//**
 @Function      UCC_ATM_RawCellRx

 @Description   Calls the Rx function with the handle of the Raw Cell channel.
 
                For raw cell polling mode, should be called only if
                interruptEnable in the raw cell parameters is set to false.
 
 @Param[in]     h_UccAtm      - Handle to UCC ATM object.

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only after UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_RawCellRx(t_Handle h_UccAtm);

/* End of prototypes generated by /cygdrive/d/NetCommSw/gen/fungen/fastslowext.pl at Tue May 17 12:29:00 2005 */

/**************************************************************************//**
 @Function      UCC_ATM_InitTcellTmp

 @Description   Calls the common level for init of Tcell Tmp
                and updates the base in subpage0.

                Called by AAL2 channels, CES case is handled from the Common and UCC levels.
 
 @Param[in]     h_UccAtm - Handle to UCC ATM object.

 @Return        E_OK if okay, Error indication other wise
*//***************************************************************************/
t_Error UCC_ATM_InitTcellTmp(t_Handle h_UccAtm);

/**************************************************************************//**
 @Group         ucc_atm_aal2_advanced_config UCC ATM AAL2 Advanced Configuration Unit

 @Description   Configuration functions used to change default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_ConfigAal2BdRingLengths

 @Description   Changes the number of AAL2 total number of BD ring lengths from the default
                and configures the number of CIDs for each BD ring length.
 
                In case of using full duplex CID, there is a need to add two to the count
                of CIDs per a specific BD ring length.
                Default value: [0].
 
 @Param[in]     h_UccAtm                  - Handle to UCC ATM object.
 @Param[in]     totalAal2BdRingLengths    - Number of AAL2 BD ring lengths.
 @Param[in]     p_BdRingLengthsParam      - AAL2 BD ring lengths parameters.

 @Return        E_OK if okay, Error indication other wise
 
 @Cautions      Allowed only following UCC_ATM_Config() and before UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ConfigAal2BdRingLengths ( t_Handle h_UccAtm, uint16_t totalAal2BdRingLengths, t_UccAtmAal2BdRingLengthParam *p_BdRingLengthsParam);

/** @} */ /* end of ucc_atm_aal2_advanced_config grp group */
/** @} */ /* end of ucc_atm_config_grp group */

/**************************************************************************//**
 @Function      UCC_ATM_Init

 @Description   Initializes driver and hardware for ucc atm.

                The initialization includes:
                   - UCC global and protocol-specific
                     registers initialization;
                   - ATM parameter RAM initialization:
                        Distributer Local page
                        Sub-page0
                        Sub-page1
                   - handle the multi-thread option
                   - GUMR initialization (ATM mode)
                   - Interrupt queues initialization;
                   - Setting address mapping method;
                   - Free buffer pools (FBP) initialization;
                   - Allocation memory segment for BD rings management.
 
 @Param[in]     h_UccAtm      - Handle to UCC ATM object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtm must have been previously created using UCC_ATM_Config().

*//***************************************************************************/
t_Error UCC_ATM_Init(t_Handle h_UccAtm);

/**************************************************************************//**
 @Function      UCC_ATM_Free

 @Description   Free memory of the uccAtm handler.

                Free memory allocated for ATM parameters,
                queues and UCC.
 
 @Param[in]     h_UccAtm    - Handle to UCC ATM object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtm must have been previously created using UCC_ATM_Config().

*//***************************************************************************/
t_Error UCC_ATM_Free(t_Handle h_UccAtm);

/**************************************************************************//**
 @Function      UCC_ATM_SetVcMaskInVpTable

 @Description   Sets VC mask to one VP entry in the VP-level addressing table.
 
                According to the VPI, sets VC_MASK to one VP entry in
                the VP-level addressing table (VPT) and initializes the
                VC-level addressing tables (VCT) of the given VP.
                That routine should be called by the application for
                each VPI. The VC Mask Should not include OAM filtered VCIs values.
                The routine uses the ATM controller number, ATM controller phy Device number and the
                port number in order to get the specific VP entry.
                The routine is relevant only in case of that the user asked to work
                with address compression as the address resolution method.

 @Param[in]     h_UccAtm    - Handle to UCC ATM object.
 @Param[in]     h_Port      - port handle.
 @Param[in]     vpi         - VPI(Virtual Path Identifier) value.
 @Param[in]     vcMask      - for masking the VCI field of the incoming cell's.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_SetVcMaskInVpTable(t_Handle h_UccAtm,
                                   t_Handle h_Port,
                                   uint16_t vpi,
                                   uint16_t vcMask);

/**************************************************************************//**
 @Function      UCC_ATM_SetPerformanceMonitoringTableParam

 @Description   Sets Performance Monitoring Table parameters.

                This function can be used for PM Tables initialization
                both in initialization phase (for setting a table parameters
                for the first time) and in run-time (for modifying a table
                parameters).

 @Param[in]     h_UccAtm        - Handle to UCC ATM object.
 @Param[in]     p_PmTableParam  - PM Table parameters.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      UCC ATM must have been previously initialized using UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_SetPerformanceMonitoringTableParam( t_Handle h_UccAtm,
                                                    t_PmTableParam *p_PmTableParam );

/**************************************************************************//**
 @Function      UCC_ATM_ClearPerformanceMonitoringTableCounters

 @Description   Clears Performance Monitoring Table counters.

 @Param[in]     h_UccAtm        - Handle to UCC ATM object.
 @Param[in]     pmTableIndex    - PM Table Index : 0 - <UCC total PMTs-1>.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      UCC ATM must have been previously initialized using UCC_ATM_Init().
*//***************************************************************************/
t_Error UCC_ATM_ClearPerformanceMonitoringTableCounters( t_Handle h_UccAtm,
                                                         uint8_t pmTableIndex );

/** @} */ /* end of ucc_atm_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_control_grp UCC ATM Control Unit

 @Description   Control Structures and Routines

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      UCC_ATM_Enable

 @Description   Enable the UCC to work in ATM mode.

 @Param[in]     h_UccAtm      - Handle to UCC ATM object.
 @Param[in]     mode       - communication mode (Receive or Transmit or Both).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtm must have been previously initialized using UCC_ATM_Init().
                The enable operation can be done only after the initialization of all
                the ports under the specific UCC.

*//***************************************************************************/
t_Error UCC_ATM_Enable(t_Handle h_UccAtm, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_ATM_Disable

 @Description   Disable the UCC to work in ATM mode.

 @Param[in]     h_UccAtm   - Handle to UCC ATM object.
 @Param[in]     mode       - communication mode (Receive or Transmit or Both).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtm must have been previously initialized using UCC_ATM_Init().

*//***************************************************************************/
t_Error UCC_ATM_Disable(t_Handle h_UccAtm, e_CommMode mode);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_DumpRegs

 @Description   Dumps UCC ATM registers and parameters RAM.

 @Param[in]     h_UccAtm - Handle to UCC ATM object.

 @Return        None.

 @Cautions      h_UccAtm must have been previously created using UCC_ATM_Config()
                and initialized using UCC_ATM_Init().
*//***************************************************************************/
void UCC_ATM_DumpRegs(t_Handle h_UccAtm);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_control_grp group */

/**************************************************************************//**
 @Group         atm_runtime_grp UCC ATM Runtime Unit

 @Description   Runtime Structures and Routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description    ATM Exception types
 
                 relevant for all the ATM exception types that are exist in the system.
*//***************************************************************************/
typedef enum e_UccAtmIntrType
{
      e_UCC_ATM_UCCE_INTR_TYPE                            /**< indicates that an exception was occured(the indication is in the UCCE register) */
    , e_UCC_ATM_INTQ_INTR_TYPE                            /**< indicates that an exception was occured (the indication is in the interrupt queue entry) */
    , e_UCC_ATM_INTQ_PLCR_INTR_TYPE                       /**< indicates that a policer exception was occured (the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_INTR_TYPE                       /**< indicates that an aal2 exception was occured (the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_ZERO_CID_INTR_TYPE              /**< indicates that an aal2 exception was occured where the cid number is zero(the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_CELL_STATS_INTR_TYPE            /**< indicates that an aal2 cell statistics exception was occured (the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_CID_STATS_INTR_TYPE             /**< indicates that an aal2 cid statistics exception was occured (the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_WFQ_QUEUE_MANAGE_0_INTR_TYPE    /**< indicates that a wfq queue management exception was occured in queue number 0(the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_WFQ_QUEUE_MANAGE_1_INTR_TYPE    /**< indicates that a wfq queue management exception was occured in queue number 1(the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_WFQ_QUEUE_MANAGE_2_INTR_TYPE    /**< indicates that a wfq queue management exception was occured in queue number 2(the indication is in the interrupt queue entry)*/
    , e_UCC_ATM_INTQ_AAL2_WFQ_QUEUE_MANAGE_3_INTR_TYPE    /**< indicates that a wfq queue management exception was occured in queue number 3(the indication is in the interrupt queue entry)*/
} e_UccAtmIntrType;

/****************************************************************//**
 @Collection    Interrupt Queue exception bit mask
 
                The different exception bits in an interrupt queue.
 @{
*//*****************************************************************/

#define UCC_ATM_INTR_TBNR             0x00000010 /**< Transmit Buffer Not Ready            */
#define UCC_ATM_INTR_BSY              0x00000004 /**< Busy; No available Receive Buffers   */

/* CES interrupts */
#define UCC_ATM_CES_INTR_SLIPE        0x00000080 /**< CES Slip End   */
#define UCC_ATM_CES_INTR_SLIPS        0x00000040 /**< CES Slip Start */
#define UCC_ATM_CES_INTR_CASUP        0x00000020 /**< CES CAS Update */

/* Policer interrupts */
#define UCC_ATM_INTR_PLCR_CLP0        0x00001000 /**< CLP0 counter overflow     */
#define UCC_ATM_INTR_PLCR_CLP1        0x00000800 /**< CLP1 counter overflow     */
#define UCC_ATM_INTR_PLCR_NCCLP0      0x00000400 /**< NCCLP0 counter overflow   */
#define UCC_ATM_INTR_PLCR_NCCLP1      0x00000200 /**< NCCLP1 counter overflow   */
#define UCC_ATM_INTR_PLCR_FC          0x00000100 /**< Frame counter overflow    */
#define UCC_ATM_INTR_PLCR_DC          0x00000080 /**< Cell/Frame discard counter overflow */

/* @} */

/** @} */ /* end of atm_runtime_grp group */

/**************************************************************************//**
 @Group         atm_exceptions_grp UCC ATM Exceptions Unit

 @Description   Exceptions Structures and Routines.

 @{
*//***************************************************************************/
/************************************************************************************************/
/* UCC ATM exception bit mask                                                                  */
/************************************************************************************************/
/****************************************************************//**
 @Collection    UCC ATM exception bit mask
 
                The different exception bits in the UCCE register.
 @{
*//*****************************************************************/

#define    UCC_ATM_UCCE_TIRU  0x04000000  /**< Transmit Internal rate underrun.   */
#define    UCC_ATM_UCCE_GRLI  0x02000000  /**< Global Red-line interrupt.         */
#define    UCC_ATM_UCCE_GBPB  0x01000000  /**< Global Buffer pool busy interrupt. */
#define    UCC_ATM_UCCE_INTO0 0x00010000  /**< Interrupt queue0 overflow.         */
#define    UCC_ATM_UCCE_INTO1 0x00020000  /**< Interrupt queue1 overflow.         */
#define    UCC_ATM_UCCE_INTO2 0x00040000  /**< Interrupt queue2 overflow.         */
#define    UCC_ATM_UCCE_INTO3 0x00080000  /**< Interrupt queue3 overflow.         */

/* @} */

/** @} */ /* end of atm_exceptions_grp group */
/** @} */ /* end of ucc_atm_lvl_grp group */
/** @} */ /* end of ucc_atm_grp group */

/**************************************************************************//*
 @Function      UCC_ATM_GetQeHandle

 @Description   Gets a handle to the QE.

                For the use of utopia port.
 
 @Param[in]     h_UccAtm  - Handle to UCC ATM object.
 
 @Return        A handle to the QE.
*//***************************************************************************/
t_Handle UCC_ATM_GetQeHandle(t_Handle h_UccAtm);

/**************************************************************************//*
 @Function      UCC_ATM_EnableChannelMiniCamVpSwitch

 @Description   Sets the bit that enables the specified channel for Mini-Cam VP Switch mode.

                For the use of Port level in Run-Time call to enable a VP Switch mode for a specific VPI,
                Called for all the Rx Channels of this port with that VPI value.
 
 @Param[in]     h_UccAtm            - Handle to UCC ATM object.
 @Param[in]     portNum             - The relevant absolute port number (0..255).
 @Param[in]     channelIndexInPort  - The channel index of that Mini-Cam port (0..63).
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_EnableChannelMiniCamVpSwitch(t_Handle h_UccAtm, uint16_t phyIndex, uint8_t channelIndexInPort);

/**************************************************************************//*
 @Function      UCC_ATM_DisableChannelMiniCamVpSwitch

 @Description   Unsets the bit that enables the specified channel for Mini-Cam VP Switch mode.

                For the use of Port level in Run-Time call to disable a VP Switch mode for a specific VPI,
                Called for all the Rx Channels of this port with that VPI value.
 
 @Param[in]     h_UccAtm            - Handle to UCC ATM object.
 @Param[in]     portNum             - The relevant absolute port number (0..255).
 @Param[in]     channelIndexInPort  - The channel index of that Mini-Cam port (0..63).
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_DisableChannelMiniCamVpSwitch(t_Handle h_UccAtm, uint16_t phyIndex, uint8_t channelIndexInPort);

/**************************************************************************//*
 @Function      UCC_ATM_ConfigImaLinksNum

 @Description   Configures the number of IMA links for this UCC.

 @Param[in]     h_UccAtm  - Handle to UCC ATM object.
 @Param[in]     imaLinksNum - Number of IMA links for this UCC.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_ConfigImaLinksNum(t_Handle h_UccAtm, uint8_t imaLinksNum);

typedef void (t_UccAtmImaExceptions)(t_Handle h_Ima);

/**************************************************************************//*
 @Function      UCC_ATM_SetImaExceptionCallback

 @Description   Sets the IMA exception callback (interface with IMA driver).

 @Param[in]     h_UccAtm        - Handle to UCC ATM object.
 @Param[in]     h_Ima           - Handle to IMA device driver.
 @Param[in]     f_ImaExceptions - Callback routine for IMA interrupts.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_SetImaExceptionCallback(t_Handle                h_UccAtm,
                                        t_Handle                h_Ima,
                                        t_UccAtmImaExceptions   *f_ImaExceptions);


/**************************************************************************//*
 @Description   IMA details structure (interface with IMA driver).
*//***************************************************************************/
typedef struct t_UccAtmImaDetails
{
    uint32_t    imaRootTableBase;       /**< Base address of IMA root table */
    uint32_t    imaIntrQueueBase;       /**< Base address of IMA interrupt queue */
    uint32_t    imaIntrQueueMemAttr;    /**< memory attributes for IMA interrupt queue */
    uint8_t     imaIntrQueueNum;        /**< IMA interrupt queue number */
} t_UccAtmImaDetails;

/**************************************************************************//*
 @Function      UCC_ATM_GetImaDetails

 @Description   Gets the IMA details of this UCC ATM (interface with IMA driver).

 @Param[in]     h_UccAtm        - Handle to UCC ATM object.
 @Param[in]     p_ImaDetails    - Pointer to IMA details structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_GetImaDetails(t_Handle h_UccAtm, t_UccAtmImaDetails *p_ImaDetails);

/************************************************************/
/* API for CES ( Circuit Emulation Service ) Module */
/************************************************************/

/**************************************************************************//*
 @Function      UCC_ATM_CES_GetAdaptiveThresholdsBase

 @Description   Gets the CES Adaptive Thresholds base address.

                Used by CES_UccMccBind for passing the value to
                the associated MCC for CES bind.
 
 @Param[in]     h_UccAtm      - Handle to UCC ATM object.

 @Return        The CES Adaptive Thresholds base address.
 
 @Cautions      Assuming the ATM UCC was already initialized.

*//***************************************************************************/
uint32_t UCC_ATM_CES_GetAdaptiveThresholdsBase(t_Handle h_UccAtm);

/**************************************************************************//*
 @Function      UCC_ATM_ConfigIwCpuRxQueuesDataMemAttributes

 @Description   Configures the memory attributes for data buffers in IW CPU
                receive queues (IW mode only).

 @Param[in]     h_UccAtm            - Handle to UCC ATM object.
 @Param[in]     dataMemAttributes   - Memory attributes for data buffers in
                                      IW CPU receive queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_ConfigIwCpuRxQueuesDataMemAttributes(t_Handle h_UccAtm, uint32_t dataMemAttributes);

/**************************************************************************//*
 @Function      UCC_ATM_ConfigAal5IwCpuRxQueuesNum

 @Description   Configures the number of AAL5 IW CPU receive queues (IW mode only).

 @Param[in]     h_UccAtm                - Handle to UCC ATM object.
 @Param[in]     aal5IwCpuRxQueuesNum    - Number of AAL5 IW CPU receive queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_ConfigAal5IwCpuRxQueuesNum(t_Handle h_UccAtm, uint8_t aal5IwCpuRxQueuesNum);

/**************************************************************************//*
 @Function      UCC_ATM_ConfigAal2IwCpuRxQueuesNum

 @Description   Configures the number of AAL2 IW CPU receive queues (IW mode only).

 @Param[in]     h_UccAtm                - Handle to UCC ATM object.
 @Param[in]     aal2IwCpuRxQueuesNum    - Number of AAL2 IW CPU receive queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_ConfigAal2IwCpuRxQueuesNum(t_Handle h_UccAtm, uint8_t aal2IwCpuRxQueuesNum);

typedef void (t_UccAtmIwExceptions)(t_Handle h_IwDev, uint32_t events);

/**************************************************************************//*
 @Function      UCC_ATM_SetIwExceptionsCallback

 @Description   Sets the IW exception callbacks (interface with IW driver).
 
                If the callback routine parameter is NULL, or both iwEvents
                and parserEvents are FALSE, the relevant exceptions will be
                masked (disabled).

 @Param[in]     h_UccAtm        - Handle to UCC ATM object.
 @Param[in]     h_IwDev         - Handle to IW device driver.
 @Param[in]     f_IwExceptions  - Callback routine for IW interrupts.
 @Param[in]     iwEvents        - TRUE if IW events are required.
 @Param[in]     parserEvents    - TRUE if parse and lookup events are required.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_SetIwExceptionsCallback(t_Handle                h_UccAtm,
                                        t_Handle                h_IwDev,
                                        t_UccAtmIwExceptions    *f_IwExceptions,
                                        bool                    iwEvents,
                                        bool                    parserEvents);


typedef void (t_IwRxCallback)(t_Handle h_UccAtm, uint16_t channelCode, uint8_t cid, uint8_t queueId);

/**************************************************************************//*
 @Description   IW details structure (interface with IW driver).
*//***************************************************************************/
typedef struct t_UccAtmIwDetails
{
    e_Ucc           uccId;                      /**< The UCC number */
    t_Handle        h_UccAtmCommon;             /**< Handle to UCC ATM COMMON object */
    uint32_t        iwGlobalPramBase;           /**< Base address of IW global parameters */
    uint32_t        atmGlobalPramBase;          /**< Base address of ATM global parameters */
    uint8_t         aal5CpuRxQueuesNum;         /**< Number of AAL5 CPU receive queues */
    uint8_t         aal2CpuRxQueuesNum;         /**< Number of AAL2 CPU receive queues */
    uint32_t        aal5CpuRxQdBase;            /**< Base address of AAL5 CPU RxQDs (ILLEGAL_BASE if none) */
    uint32_t        aal2CpuRxQdBase;            /**< Base address of AAL2 CPU RxQDs (ILLEGAL_BASE if none) */
    t_IwRxCallback  *f_IwRxCallback;            /**< Callback for receiving IW CPU frames */
    uint8_t         intrQueueMemId;             /**< Memory partition for interrupt queues */
    uint32_t        intrQueueMemAttributes;     /**< Memory attributes for interrupt queues */
    uint32_t        cpuQueuesDataMemAttributes; /**< Memory attributes for buffers in IW CPU queues */
} t_UccAtmIwDetails;

/**************************************************************************//*
 @Function      UCC_ATM_GetIwDetails

 @Description   Gets the IW details from this UCC ATM (interface with IW driver).

 @Param[in]     h_UccAtm    - Handle to UCC ATM object.
 @Param[in]     p_IwDetails - Pointer to IW details structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_GetIwDetails(t_Handle h_UccAtm, t_UccAtmIwDetails *p_IwDetails);


#endif /* __UCC_ATM_EXT_H */

