/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          ucc_atm_port_ext.h

 @Description   UCC Atm Port external structures and definitions.
*//***************************************************************************/

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   UCC ATM API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_port_grp UCC ATM Port

 @Description   UCC ATM Port

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_port_init_grp UCC ATM Port Initialization Unit

 @Description   UCC ATM Port initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/
#ifndef __UCC_ATM_PORT_EXT_H
#define __UCC_ATM_PORT_EXT_H

#include "ucc_atm_common_ext.h"

/**************************************************************************//**
 @Description   Scheduler levels in the scheduling table.
 *//***************************************************************************/
typedef enum
{
      e_UCC_ATM_PORT_SCHED_LEVEL_1 = 0       /**< Scheduling level 1 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_2           /**< Scheduling level 2 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_3           /**< Scheduling level 3 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_4           /**< Scheduling level 4 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_5           /**< Scheduling level 5 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_6           /**< Scheduling level 6 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_7           /**< Scheduling level 7 */
    , e_UCC_ATM_PORT_SCHED_LEVEL_8           /**< Scheduling level 8 */
    , e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS
} e_UccAtmPortSchedLevels;

#define UCC_ATM_MAX_GCRA_LEVELS 4

/**************************************************************************//**
 @Description   UPC Port Priority
 *//***************************************************************************/
typedef enum e_UccAtmPortUpcPriority
{
    e_UCC_ATM_PORT_UPC_HIGH_PRIORITY, /**< High priority */
    e_UCC_ATM_PORT_UPC_LOW_PRIORITY   /**< Low priority  */
 
} e_UccAtmPortUpcPriority;

/**************************************************************************//**
 @Description   APC Scheduler initialization structure
 
                For UCC_ATM_PORT_ConfigVpShapingScheduler.
 *//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt[e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS];
                                            /**< If true - the control slot indicates that TCTE should be read for each channel;
                                                 Must be false in case of working with apc scalable mode or with traffic type which is different then CBR */

    uint8_t     maxIteration;               /**< Max iteration allowed */
    bool        burstInhibit;               /**< Indicates if enable the burst inhibit option in the scheduler_MODE field */
    uint8_t     schedScaleFactor;           /**< Scheduler scaling factor */

} t_UccAtmApcSchedParam;

/**************************************************************************//**
 @Description   Information regarding GBR traffic of channels on a GCRA port
 *//***************************************************************************/
typedef enum
{
    e_UCC_ATM_GCRA_NO_GBR_MODE,      /**< There are no GBR channels in this GCRA port */
    e_UCC_ATM_GCRA_MIXED_GBR_MODE,   /**< Some of the channels on last priority work with GBR traffic */
    e_UCC_ATM_GCRA_FULL_GBR_MODE     /**< All channels on last priority work with GBR traffic */
 
} e_UccAtmGcraGbrMode;

/**************************************************************************//**
 @Description   GCRA Scheduler initialization structure.
 
                For UCC_ATM_PORT_ConfigVpShapingScheduler and UCC_ATM_PORT_ConfigGcraScheduler.
 *//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt;      /**< No extended connection table needed for this port's channels */
    bool        isInternal;         /**< Indication if the user asked to work with internal/external channels in this port */
    uint8_t     gcraChannels;       /**< The actual number of gcra channels per the specific port */
    bool        workConservingMode; /**< Work Conserving Mode - Vps on the last priority are scheduled according to their
                                         weighted PCR/PCR Fraction regardless of the system clock */
    uint8_t     *p_WrrPriPercentages;/**< Array of up to 4 WRR (Weighted Round Robin) priorities percentages;
                                         Relevant only for WRR Mode, in which selection between the up to 4 priorities
                                         is done according to their relative weights; Note: For regular fixed priority mode
                                         (non-WRR) this pointer should be NULL. */
    e_UccAtmGcraGbrMode gbrMode;    /**< Full GBR mode / Mixed GBR mode / No GBR mode */

} t_UccAtmGcraSchedParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level APC initialization structure.

                This structurte includes the parameters needed for a single
                APC parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    			vpi;			/**< Virtual Path Identifier of this VP group channels */
 
	/* Scheduling level */
	e_UccAtmPortSchedLevels	schedLevel;		/**< Scheduling level in high level GCRA scheduler */

    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS];
                                            /**< The rate of the slow channel - bits per second per priority level */
    /* APC specific parameters */
    t_UccAtmApcSchedParam apcSchedParam;

} t_UccAtmLowLevelApcParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level GCRA initialization structure.

                This structurte includes the parameters needed for a single
                GCRA parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    			vpi;			/**< Virtual Path Identifier of this VP group channels */

	/* Scheduling level */
	e_UccAtmPortSchedLevels	schedLevel;		/**< Scheduling level in high level GCRA scheduler */

    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[UCC_ATM_MAX_GCRA_LEVELS];
                                            /**< The rate of the slow channel per priority level (in bits per second) */

    /* GCRA specific parameters */
    t_UccAtmGcraSchedParam gcraSchedParam;

} t_UccAtmLowLevelGcraParam;

/**************************************************************************//**
 @Description   VP Shaping Scheduling initialization structure.
 *//***************************************************************************/
typedef struct
{
    uint8_t                         vpsNum;             /**< Number of Vps under this Vp Shaping port */
    bool                            autoVpOnOff;        /**< Auto Vp On Off enable */
    t_UccAtmLowLevelApcParam        *p_LowLevelApcSchedParam;
                                                        /**< If not NULL => GCRA over APC Vp Shaping (Low level
                                                        scheduling type is APC) and this is the array of all
                                                        low level APC schedulers parameters */
    t_UccAtmLowLevelGcraParam       *p_LowLevelGcraSchedParam;
                                                        /**< If not NULL => GCRA over GCRA Vp Shaping (Low level
                                                        scheduling type is GCRA) and this is the array of all
                                                        low level GCRA schedulers parameters */
} t_UccAtmPortVpShapingParam;

/**************************************************************************//**
 @Description   MiniCam parameters structure.
 *//***************************************************************************/
typedef struct
{
    bool                        uniMode;        /**< Indicates if to take the GFC value into the CAM search or mask it
                                                     FALSE - take the GFC field,
                                                     TRUE - mask the GFC field in the incoming cell */
    bool                        isOverlapping;  /**< Indication if the specific port belongs to an overlap group */
    uint8_t                     overlapGroup;   /**< The number of the overlap group ,
                                                     NOTE: This parameter is relevant only in case the isOverlapping parameter is set */
    e_MaxChannelsPerMiniCamPort numOfChannels;  /**< The actual number of how many channels are going to work under this MiniCam port
                                                     NOTE: The parameter is important to improve performance
                                                     and also to search for an available channel code */
} t_UccAtmPortMiniCamParam;


/**************************************************************************//**
 @Description   Shaper Type.
 *//***************************************************************************/
typedef enum e_UccAtmPortShaperType
{
       e_UCC_ATM_PORT_NON_SHAPER_TYPE = 0
     , e_UCC_ATM_PORT_APC_SHAPER_TYPE
     , e_UCC_ATM_PORT_GCRA_SHAPER_TYPE
     , e_UCC_ATM_PORT_VP_SHAPING_TYPE
} e_UccAtmPortShaperType;


/**************************************************************************//**
 @Function      UCC_ATM_PORT_Config

 @Description   Configures a UCC ATM Port based on user parameters.

            	Initialize the t_UccAtmPort driver handle and zero all its fields.
            	Set default values to configuration parameters.

 @Return        Handle to created UCC ATM Port if success; NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.
*//***************************************************************************/
t_Handle UCC_ATM_PORT_Config(void);

/**************************************************************************//**
 @Group         ucc_atm_port_config_grp UCC ATM Port Advanced Configuration Unit

 @Description   Configuration functions used to change default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_port_config_apc_grp Port APC Advanced Configuration Unit

 @Description   Configuration functions used to change APC default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigMaxIteration

 @Description   Configures the maximum iteration parameter for the port.
 

 @Param[in]     h_UccAtmPort    - UCC ATM Port handle.
 @Param[in]     newVal          - Maximum iteration allowed.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigMaxIteration (t_Handle h_UccAtmPort, uint8_t newVal);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigBurstInhibit

 @Description   Configures burst inhibit parameter for the APC.

 @Param[in]     h_UccAtmPort        - UCC ATM Port handle.
 @Param[in]     enableBurstInhibit  - Enable Burst Inhibit in the scheduler mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigBurstInhibit(t_Handle h_UccAtmPort, bool enableBurstInhibit);


/** @} */ /* end of ucc_atm_port_config_apc_grp group */

/**************************************************************************//**
 @Group         ucc_atm_port_config_gcra_grp Port GCRA Advanced Configuration Unit

 @Description   Configuration functions used to change GCRA default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigGcraScheduler

 @Description   Configures a GCRA scheduler.

 @Param[in]     h_UccAtmPort                - UCC ATM Port handle.
 @Param[in]     p_UccAtmPortGcraSchedParam  - GCRA scheduler parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigGcraScheduler(t_Handle                   h_UccAtmPort,
                                         t_UccAtmGcraSchedParam     *p_UccAtmPortGcraSchedParam);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigVpShapingScheduler

 @Description   Configures Vp Shaping mode.

 @Param[in]     h_UccAtmPort                - UCC ATM Port handle.
 @Param[in]     p_UccAtmPortVpShapingParam  - Vp Shaping parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigVpShapingScheduler(t_Handle h_UccAtmPort, t_UccAtmPortVpShapingParam *p_UccAtmPortVpShapingParam);

/** @} */ /* end of ucc_atm_port_config_gcra_grp group */

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigNonExtendedCt

 @Description   Configures if to work with extended connection table for a specific scheduling table.

 @Param[in]     h_UccAtmPort    - UCC ATM Port handle.
 @Param[in]     schedLevel      - which apc level to update.
 @Param[in]     nonExtendedCt   - new value to set to nonExtendedCt parameter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigNonExtendedCt(t_Handle                   h_UccAtmPort,
                                         e_UccAtmPortSchedLevels    schedLevel,
                                         bool                       nonExtendedCt);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigMiniCam

 @Description   Configures Mini-CAM parameters for the specific port.

 @Param[in]     h_UccAtmPort                - UCC ATM Port handle.
 @Param[in]     p_UccAtmPortMiniCamParam    - Mini-CAM parameters structure.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigMiniCam(t_Handle                 h_UccAtmPort,
                                   t_UccAtmPortMiniCamParam *p_UccAtmPortMiniCamParam);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigNonShaperType

 @Description   Configures the burst inhibit option.

 @Param[in]     h_UccAtmPort    - UCC ATM Port handle.

 @Return        E_OK on success; Error code otherwise.

 *//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigNonShaperType(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigSchedulerScaleFactor

 @Description   Configures the scheduler scale factor of the port.

 @Param[in]     h_UccAtmPort        - UCC ATM Port handle.
 @Param[in]     schedScaleFactor    - Scheduler scale factor.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigSchedulerScaleFactor(t_Handle h_UccAtmPort, uint8_t schedScaleFactor);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ConfigTimeOutInMicroSec

 @Description   Configures the time to wait until a stop request is finished.

 @Param[in]     h_UccAtmPort        - UCC ATM Port handle.
 @Param[in]     timeOutMicroSec     - time in micro seconds to wait until a stop request is finished .

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_ATM_PORT_Config() and before
                UCC_ATM_PORT_Init().
*//***************************************************************************/
t_Error UCC_ATM_PORT_ConfigTimeOutMicroSec (t_Handle h_UccAtmPort, uint16_t timeOutMicroSec);

/** @} */ /* end of ucc_atm_port_config_grp group */

/**************************************************************************//**
 @Function      UCC_ATM_PORT_Init

 @Description   Initializes driver and hardware for UCC ATM Port.

                The initialization includes:
                   - Initialize the scheduler
                   - registers initialization
 
 @Param[in]     h_UccAtmPort - UCC ATM Port handle to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccAtmPort must have been previously created using UCC_ATM_PORT_Config().
*//***************************************************************************/
t_Error UCC_ATM_PORT_Init(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_Free

 @Description   Free the UCC ATM Port object.

 @Param[in]     h_UccAtmPort - UCC ATM Port handle to be freed.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccAtmPort must have been previously created using UCC_ATM_PORT_Config().
*//***************************************************************************/
t_Error UCC_ATM_PORT_Free(t_Handle h_UccAtmPort);

/** @} */ /* end of ucc_atm_port_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_port_control_grp UCC ATM Port Control Unit

 @Description   UCC ATM Port initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_PORT_EnableMiniCamVpSwitch

 @Description   Enables VP Mini-Cam VP Switch mode for all of the channels of 'h_UccAtmPort'
                that are configured with the passed vpi value.

 @Param[in]     h_UccAtmPort - handle to Port object.
 @Param[in]     vpi - the vpi value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only after UCC_ATM_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UCC_ATM_PORT_EnableMiniCamVpSwitch(t_Handle h_UccAtmPort, uint16_t vpi);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_DisableMiniCamVpSwitch

 @Description   Disables VP Mini-Cam VP Switch mode for all of the channels of 'h_UccAtmPort'
                that are configured with the passed vpi value.

 @Param[in]     h_UccAtmPort - handle to Port object.
 @Param[in]     vpi - the vpi value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only after UCC_ATM_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UCC_ATM_PORT_DisableMiniCamVpSwitch(t_Handle h_UccAtmPort, uint16_t vpi);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_EnablePolicer

 @Description   Enable the work with policer for a specific port.
 
                Note : Relevant only for Mini Cam ports.

 @Param[in]     h_UccAtmPort - handle to Port object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only after UCC_ATM_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UCC_ATM_PORT_EnablePolicer(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_DisablePolicer

 @Description   Disable the work with policer for a specific port.

                Note : Relevant only for Mini Cam ports.
 
 @Param[in]     h_UccAtmPort - handle to Port object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only after UCC_ATM_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UCC_ATM_PORT_DisablePolicer(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_Enable

 @Description   Enable the port.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UCC_ATM_PORT_Init().

*//***************************************************************************/
t_Error UCC_ATM_PORT_Enable(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_Disable

 @Description   Disable the port.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UCC_ATM_PORT_Init().

*//***************************************************************************/
t_Error UCC_ATM_PORT_Disable(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_SetPortPriority

 @Description   Set the port Rx priority in the UPC;
                Default value for all ports at init point is High Priority.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     rxPriority   - The Rx priority in UPC.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UCC_ATM_PORT_Init().

*//***************************************************************************/
t_Error UCC_ATM_PORT_SetPortPriority(t_Handle h_UccAtmPort, e_UccAtmPortUpcPriority rxPriority);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_GetPortUnderrunEventIndication

 @Description   Returns the TRUE if underrun occurred and clears the register.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        TRUE if port Tx underrun occurred; FALSE otherwise.

 @Cautions      Allowed only after UCC_ATM_PORT_Init().

*//***************************************************************************/
bool UCC_ATM_PORT_GetPortUnderrunEventIndication(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_IsAlive

 @Description   Checks if the phy is working properly.
 
                To be used by the user application for verifying the phy is still
                alive in case of Channel transmit operation or Tx Channel stop failure.

 @Param[in]     h_UccAtmPort - handle to Port object.

 @Return        TRUE if the phy is working, FALSE - otherwise.

*//***************************************************************************/
bool UCC_ATM_PORT_IsAlive(t_Handle h_UccAtmPort);

/**************************************************************************//**
 @Function      UCC_ATM_PORT_ForceTxChannelsDisable

 @Description   Force stop of all the channels of the specific port.
 
                To be used by the user application in case there is a phy error
                (for example - if UCC_ATM_PORT_IsAlive returned FALSE) for termination of the
                channels of this port. Note : A regular disable routine will not
                work in case of a phy error, because the microcode stop command depends
                on the APC/GCRA scheduling.

 @Param[in]     h_UccAtmPort - handle to Port object.

 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error UCC_ATM_PORT_ForceTxChannelsDisable(t_Handle h_UccAtmPort);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_PORT_DumpRegs

 @Description   Dumps ATM Port registers and parameters RAM.

 @Param[in]     h_UccAtmPort - Handle to port object.

 @Return        None.

 @Cautions      h_UccAtmPort must have been previously created using UCC_ATM_PORT_Config()
                and initialized using UCC_ATM_PORT_Init().
*//***************************************************************************/
void UCC_ATM_PORT_DumpRegs(t_Handle h_UccAtmPort);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_port_control_grp group */
/** @} */ /* end of ucc_atm_port_grp group */
/** @} */ /* end of group ucc_atm_grp */

#endif /* __UCC_ATM_PORT_EXT_H */

