/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_fast_ext.h

 @Description   UCC Fast external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_FAST_EXT
#define __UCC_FAST_EXT

#include "std_ext.h"
#include "ucc_ext.h"
#include "xx_ext.h"

/**************************************************************************//**
 @Group         ucc_fast_id UCC Fast

 @Description   Fast UCC functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UCC fast diagnostic mode
*//***************************************************************************/
typedef enum e_UccFastGUMRDiagnosticModeDIAG
{
    e_UCC_FAST_DIAGNOSTIC_NORMAL            = 0x0,        /**< normal             */
    e_UCC_FAST_DIAGNOSTIC_LOCAL_LOOP_BACK   = 0x40000000, /**< loop back          */
    e_UCC_FAST_DIAGNOSTIC_AUTO_ECHO         = 0x80000000, /**< echo               */
    e_UCC_FAST_DIAGNOSTIC_LOOP_BACK_AND_ECHO= 0xC0000000  /**< loop back and echo */
}e_UccFastGUMRDiagnosticModeDIAG;


/**************************************************************************//**
 @Description   UCC fast Sync length (transparent mode only)
*//***************************************************************************/
typedef enum e_UccFastGUMRSyncLenSYNL
{
    e_UCC_FAST_SYNC_LEN_NOT_USED  = 0x0,        /**< not used  */
    e_UCC_FAST_SYNC_LEN_AUTOMATIC = 0x00004000, /**< automatic */
    e_UCC_FAST_SYNC_LEN_8_BIT     = 0x00008000, /**< 8 bit     */
    e_UCC_FAST_SYNC_LEN_16_BIT    = 0x0000C000  /**< 16 bit    */
}e_UccFastGUMRSyncLenSYNL;

/**************************************************************************//**
 @Description   UCC fast RTS mode
*//***************************************************************************/
typedef enum e_UccFastGUMRReadyToSendRTSM
{
    e_UCC_FAST_SEND_IDLES_BETWEEN_FRAMES = 0x00000000, /**< idles between frames */
    e_UCC_FAST_SEND_FLAGS_BETWEEN_FRAMES = 0x00002000  /**< flags between frames */
}e_UccFastGUMRReadyToSendRTSM;

/**************************************************************************//**
 @Description   UCC fast receiver decoding mode
*//***************************************************************************/
typedef enum e_UccFastGUMRReceiverDecodingMethodRENC
{
    e_UCC_FAST_RX_ENCODING_NRZ          = 0x00000000, /**< nrz  */
    e_UCC_FAST_RX_ENCODING_NRZI         = 0x00000800, /**< nrzi */
    e_UCC_FAST_RX_ENCODING_RESERVED0    = 0x00001000, /**<      */
    e_UCC_FAST_RX_ENCODING_RESERVED1    = 0x00001800  /**<      */
}e_UccFastGUMRReceiverDecodingMethodRENC;

/**************************************************************************//**
 @Description   UCC fast transmitter encoding mode
*//***************************************************************************/
typedef enum e_UccFastGUMRTransmitterEncodingMethodTENC
{
    e_UCC_FAST_TX_ENCODING_NRZ          = 0x00000000, /**< nrz  */
    e_UCC_FAST_TX_ENCODING_NRZI         = 0x00000100, /**< nrzi */
    e_UCC_FAST_TX_ENCODING_RESERVED0    = 0x00000200, /**<      */
    e_UCC_FAST_TX_ENCODING_RESERVED1    = 0x00000300  /**<      */
}e_UccFastGUMRTransmitterEncodingMethodTENC;

/**************************************************************************//**
 @Description   UCC fast CRC length
*//***************************************************************************/
typedef enum e_UccFastGUMRTransparentCRCTypeTCRC
{
    e_UCC_FAST_16_BIT_CRC     = 0x00000000, /**< 16 bit */
    e_UCC_FAST_CRC_RESERVED0  = 0x00000040, /**<        */
    e_UCC_FAST_32_BIT_CRC     = 0x00000080, /**< 32 bit */
    e_UCC_FAST_CRC_RESERVED1  = 0x000000C0  /**<        */
}e_UccFastGUMRTransparentCRCTypeTCRC;



/**************************************************************************//**
 @Function      UCC_FAST_ConfigInitTx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to initialize for transmission. [TRUE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigInitTx ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigInitRx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to initialize for reception.  [TRUE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigInitRx ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigGrantSupport

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to provide grant support. [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigGrantSupport ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTSA

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to use TSA, FALSE to use NMSI. [FALSE]
 
                If this routine is called with TRUE, the driver will set CDP, CDS,
                CTSP and CTSS to TRUE, as if the following functions were also
                called with TRUE:
                - UCC_FAST_ConfigCDP(),
                - UCC_FAST_ConfigCDSynchronous(),
                - UCC_FAST_ConfigCTSP(),
                - UCC_FAST_ConfigCTSSynchronous().
 
 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTSA ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigCDP

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for envelope mode (normal operation),
                TRUE for pulse mode.
                [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigCDP ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigCDSynchronous

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for CD input that is asynchronous with data,
                TRUE for synchronous.
                [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigCDSynchronous ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigCTSP

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for envelope mode (normal operation),
                TRUE for pulse mode.
                [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigCTSP ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigCTSSynchronous

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for CTS input that is asynchronous with data,
                TRUE for synchronous.
                [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigCTSSynchronous ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigInvertTxClock

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to invert internal transmit clock. [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigInvertTxClock ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigRxTxSync

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE for receiver-transmitter synchronization. [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigRxTxSync ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigRTSM

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE to send idles between frames,
                TRUE to send flags/syncs between frames. [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigRTSM ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigReverseDataTransparent

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to reverse data on transparent channels. [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigReverseDataTransparent ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigReceiveSyncTiming

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE for receive synchronization timing. [FALSE]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigReceiveSyncTiming ( t_Handle h_UccFast, bool newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigDataSyncPattern1

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                data synchronization pattern 1. [0x7E]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigDataSyncPattern1 ( t_Handle h_UccFast, uint8_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigDataSyncPattern2

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                data synchronization pattern 2. [0x7E]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigDataSyncPattern2 ( t_Handle h_UccFast, uint8_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigRxVirtFifoSize

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Rx virtual FIFO Size register. [0x100]
 
 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigRxVirtFifoSize ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigRxVirtFifoEmergThresh

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Rx virtual FIFO Emergency Threshold. [0x88]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigRxVirtFifoEmergThresh ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigRxVirtFifoSpecEmergThresh

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Rx virtual FIFO Special Emergency Threshold. [0xD0]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigRxVirtFifoSpecEmergThresh ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTxVirtFifoSize

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Tx virtual FIFO Size register. [0x100]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTxVirtFifoSize ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTxVirtFifoEmergThresh

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Tx virtual FIFO Emergency Threshold. [0x88]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTxVirtFifoEmergThresh ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTxVirtFifoTxThresh

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Tx virtual FIFO Tx Threshold. [0x40]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTxVirtFifoTxThresh ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTxPollingTimer

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Tx Polling Timer [256]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTxPollingTimer ( t_Handle h_UccFast, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTxEncoding

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Transmitter encoding [e_UCC_FAST_TX_ENCODING_NRZ]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTxEncoding ( t_Handle h_UccFast, e_UccFastGUMRTransmitterEncodingMethodTENC newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigRxDecoding

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Receiver decoding [e_UCC_FAST_RX_ENCODING_NRZ]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigRxDecoding ( t_Handle h_UccFast, e_UccFastGUMRReceiverDecodingMethodRENC newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigDiagnosticMode

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Diagnostic mode. [e_UCC_FAST_DIAGNOSTIC_NORMAL]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigDiagnosticMode ( t_Handle h_UccFast, e_UccFastGUMRDiagnosticModeDIAG newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTransparentCRCType

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Transparent CRC type [e_UCC_FAST_16_BIT_CRC]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTransparentCRCType ( t_Handle h_UccFast, e_UccFastGUMRTransparentCRCTypeTCRC newVal );


/**************************************************************************//**
 @Function      UCC_FAST_ConfigTransparentSyncLenRx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                transparent sync length for receiver [e_UCC_FAST_SYNC_LEN_NOT_USED]

 @Param         h_UccFast - (In) handle to UCC FAST
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccFast handle passed

 @Cautions      Change is applied only upon calling UCC_FAST_Init.
*//***************************************************************************/
t_Error UCC_FAST_ConfigTransparentSyncLenRx ( t_Handle h_UccFast, e_UccFastGUMRSyncLenSYNL newVal );


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      UCC_FAST_DumpRegs

 @Description   Dumps UCC Fast registers.

 @Param         h_UccFast - (In) handle to UCC FAST

 @Return        None.
*//***************************************************************************/
void UCC_FAST_DumpRegs(t_Handle h_UccFast);

#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of group */

#endif /* __UCC_FAST_EXT */



