/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_pos_common_ext.h

 @Description   UCC POS common external structures and definitions.
*//***************************************************************************/
#ifndef __UCC_POS_COMMON_EXT_H
#define __UCC_POS_COMMON_EXT_H


#include "qe_ext.h"
#include "upc_ext.h"
#include "ucc_pos_def_ext.h"


/**************************************************************************//**
 @Group         ucc_pos_grp UCC POS Application Programming Interface

 @Description   UCC POS functions, definitions and enumerators

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         ucc_pos_common_grp UCC POS Common API

 @Description   UCC POS Common functions, definitions and enumerators

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         ucc_pos_common_initialization_grp UCC POS Common Initialization Unit

 @Description   UCC POS Common initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Configuration parameters
*//***************************************************************************/
typedef struct t_UccPosCommonParam
{
    t_Handle h_Qe;
    /**< Handle to the QE object */
 
    uint8_t pramMemId;
    /**< Parameter RAM partition ID */
 
    t_Handle p_UpcArray[QE_NUM_OF_UPCS];
    /**< Array of UPC handles: If any of the ports running over UPC-i is expected to run POS,
         then UPC-i should be initialized, and p_UpcArray[i] should be set to UPC-i's handle */
 
    uint8_t numOfResources;
    /**< Total number of resources (each resource can be used by a single UCC or by several UCCs) */
 
    struct
    {
        bool     spMode;                 /**< Single/Multi port mode */
        uint8_t  numOfUccs;              /**< Number of UCCs         */
        e_Ucc    p_Uccs[QE_NUM_OF_UCCS]; /**< List of UCCs           */
        uint8_t  txNumOfThreads;         /**< Tx number of threads   */
        uint8_t  rxNumOfThreads;         /**< Rx number of threads   */
    }
    p_ResourceParam[QE_NUM_OF_UCCS]; /**< Parameters of each resource */
}
t_UccPosCommonParam;


/**************************************************************************//**
 @Function      UCC_POS_COMMON_Config

 @Description   Configures the common object.

 @Param[in]     p_UccPosCommonParam - Common configuration parameters.

 @Return        Handle to the common object on success; NULL otherwise.
*//***************************************************************************/
t_Handle UCC_POS_COMMON_Config(t_UccPosCommonParam* p_UccPosCommonParam);


/**************************************************************************//**
 @Function      UCC_POS_COMMON_Init

 @Description   Initializes the common object.

 @Param[in]     h_UccPosCommon - Handle to the common object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_Init(t_Handle h_UccPosCommon);


/**************************************************************************//**
 @Function      UCC_POS_COMMON_Free

 @Description   Deallocates the common object + memory allocated during its configuration and initialization.

 @Param[in]     h_UccPosCommon - Handle to the common object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_Free(t_Handle h_UccPosCommon);


/**************************************************************************//**
 @Group         ucc_pos_common_advance_config_grp UCC POS Common Advanced Configuration Unit

 @Description   UCC POS Common advanced configuration routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      UCC_POS_COMMON_ConfigResourceTxThreadsRisc

 @Description   Configures the Tx threads RISC of a resource.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     resourceId     - ID of the resource.
 @Param[in]     txThreadsRisc  - Tx threads RISC of the resource.
                The default Tx threads RISC of each resource is [QE_RISC_ALLOC_ALL_RISCS].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_ConfigResourceTxThreadsRisc(t_Handle h_UccPosCommon,
                                                   uint8_t  resourceId,
                                                   uint8_t  txThreadsRisc);


/**************************************************************************//**
 @Function      UCC_POS_COMMON_ConfigResourceRxThreadsRisc

 @Description   Configures the Rx threads RISC of a resource.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     resourceId     - ID of the resource.
 @Param[in]     rxThreadsRisc  - Rx threads RISC of the resource.
                The default Rx threads RISC of each resource is [QE_RISC_ALLOC_ALL_RISCS].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_ConfigResourceRxThreadsRisc(t_Handle h_UccPosCommon,
                                                   uint8_t  resourceId,
                                                   uint8_t  rxThreadsRisc);


/**************************************************************************//**
 @Function      UCC_POS_COMMON_ConfigResourceRxThreadTableLength

 @Description   Configures the Rx thread table length of a resource.

 @Param[in]     h_UccPosCommon      - Handle to the common object.
 @Param[in]     resourceId          - ID of the resource.
 @Param[in]     rxThreadTableLength - Rx thread table length of the resource.
                The default Rx thread table length of each resource is [32].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_ConfigResourceRxThreadTableLength(t_Handle h_UccPosCommon,
                                                         uint8_t  resourceId,
                                                         uint8_t  rxThreadTableLength);


/**************************************************************************//**
 @Function      UCC_POS_COMMON_ConfigResourceRxExtFramePages

 @Description   Configures the Rx external frame pages of a resource.

 @Param[in]     h_UccPosCommon         - Handle to the common object.
 @Param[in]     resourceId             - ID of the resource.
 @Param[in]     rxExtFramePagesPercent - Rx external frame pages percentage of the resource.
 @Param[in]     rxExtFramePagesMemId   - Rx external frame pages memory partition ID of the resource.
 @Param[in]     rxExtFramePagesMemAttr - Rx external frame pages memory partition attributes of the resource.
                The default Rx external frame pages percentage of each resource is [0].
                The default Rx external frame pages memory partition ID of each resource is [0].
                The default Rx external frame pages memory partition attributes of each resource is [0].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_ConfigResourceRxExtFramePages(t_Handle h_UccPosCommon,
                                                     uint8_t  resourceId,
                                                     uint8_t  rxExtFramePagesPercent,
                                                     uint8_t  rxExtFramePagesMemId,
                                                     uint32_t rxExtFramePagesMemAttr);


/** @} */ /* end of ucc_pos_common_advance_config_grp */
/** @} */ /* end of ucc_pos_common_initialization_grp */


/**************************************************************************//*
 @Function      UCC_POS_COMMON_GetQeHandle

 @Description   Returns a handle to the QE object.

 @Param[in]     h_UccPosCommon - Handle to the common object.

 @Return        Handle to the QE object.
*//***************************************************************************/
t_Handle UCC_POS_COMMON_GetQeHandle(t_Handle h_UccPosCommon);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_GetPramPartitionId

 @Description   Returns the selected partition ID for parameter RAM.

 @Param[in]     h_UccPosCommon - Handle to the common object.

 @Return        Parameter RAM partition ID.
*//***************************************************************************/
uint8_t UCC_POS_COMMON_GetPramPartitionId(t_Handle h_UccPosCommon);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_GetUccTotalNumOfPorts

 @Description   Returns the total number of ports in a specific UCC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     uccId          - ID of the UCC.

 @Return        Total number of ports in a specific UCC.
*//***************************************************************************/
uint8_t UCC_POS_COMMON_GetUccTotalNumOfPorts(t_Handle h_UccPosCommon,e_Ucc uccId);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_GetUccNumOfPortsPerUpcDev

 @Description   Returns the number of ports in a specific UCC, that are running over a specific device of a specific UPC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     uccId          - ID of the UCC.
 @Param[in]     upcId          - ID of the UPC.
 @Param[in]     devId          - ID of the device.

 @Return        Number of ports in a specific UCC, that are running over a specific device of a specific UPC.
*//***************************************************************************/
uint8_t UCC_POS_COMMON_GetUccNumOfPortsPerUpcDev(t_Handle h_UccPosCommon,e_Ucc uccId,e_Upc upcId,e_UpcDevice devId);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_AcquireResource

 @Description   Acquires a resource that is configured for a given UCC.
                The resource is possibly shared by several UCCs.
                It is allocated only when the first UCC acquires it.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     uccId          - ID of the UCC requiring the resource.
 @Param[out]    p_Resource     - Pointer to the handle of the resource.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_AcquireResource(t_Handle h_UccPosCommon,e_Ucc uccId,t_Handle* p_Resource);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_ReleaseResource

 @Description   Releases a resource that is configured for a given UCC.
                The resource is possibly shared by several UCCs.
                It is dellocated only when the last UCC releases it.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     uccId          - ID of the UCC requiring the resource.
 @Param[out]    p_Resource     - Pointer to the handle of the resource.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_ReleaseResource(t_Handle h_UccPosCommon,e_Ucc uccId,t_Handle* p_Resource);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_ResourceIsInSpMode

 @Description   Indicates whether a resource is configured in Single-Port mode or in Multi-Port mode.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     uccId          - ID of one of the UCCs that are sharing the resource.

 @Return        TRUE - The resource is configured in Single-Port mode.
                FALSE - The resource is configured in Multi-Port mode.
*//***************************************************************************/
bool UCC_POS_COMMON_ResourceIsInSpMode(t_Handle h_UccPosCommon,e_Ucc uccId);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_EnablePort

 @Description   Enables a port that is running over a specific device of a specific UPC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     upcId          - ID of the UPC.
 @Param[in]     devId          - ID of the device.
 @Param[in]     portId         - ID of the port.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_EnablePort(t_Handle h_UccPosCommon,e_Upc upcId,e_UpcDevice devId,uint8_t portId);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_DisablePort

 @Description   Disable a port that is running over a specific device of a specific UPC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     upcId          - ID of the UPC.
 @Param[in]     devId          - ID of the device.
 @Param[in]     portId         - ID of the port.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_DisablePort(t_Handle h_UccPosCommon,e_Upc upcId,e_UpcDevice devId,uint8_t portId);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_SetPortPriority

 @Description   Changes the priority of a port that is running over a specific device of a specific UPC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     upcId          - ID of the UPC.
 @Param[in]     devId          - ID of the device.
 @Param[in]     portId         - ID of the port.
 @Param[in]     rxPriority     - Rx priority.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_SetPortPriority(t_Handle h_UccPosCommon,e_Upc upcId,e_UpcDevice devId,uint8_t portId,e_UpcPortPriority rxPriority);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_SetPortIntRate

 @Description   Changes the internal rate of a port that is running over a specific device of a specific UPC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     upcId          - ID of the UPC.
 @Param[in]     devId          - ID of the device.
 @Param[in]     portId         - ID of the port.
 @Param[in]     txIntRate      - Tx internal rate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_COMMON_SetPortIntRate(t_Handle h_UccPosCommon,e_Upc upcId,e_UpcDevice devId,uint8_t portId,uint32_t txIntRate);


/**************************************************************************//*
 @Function      UCC_POS_COMMON_GetPortUnderrunEventIndication

 @Description   Reports whether an underrun event has occurred for a port that is running over a specific device of a specific UPC.

 @Param[in]     h_UccPosCommon - Handle to the common object.
 @Param[in]     upcId          - ID of the UPC.
 @Param[in]     devId          - ID of the device.
 @Param[in]     portId         - ID of the port.

 @Return        TRUE if an underrun event has occurred; FALSE otherwise.
*//***************************************************************************/
bool UCC_POS_COMMON_GetPortUnderrunEventIndication(t_Handle h_UccPosCommon,e_Upc upcId,e_UpcDevice devId,uint8_t portId);


/**************************************************************************//**
 @Group         ucc_pos_common_runtime_control_grp UCC POS Common Runtime Control Unit

 @Description   UCC POS Common runtime control routines

 @{
*//***************************************************************************/


#if (DEBUG_ERRORS > 0)


/**************************************************************************//**
 @Function      UCC_POS_COMMON_DumpRegisters

 @Description   Dumps the current values of the common registers.

 @Param[in]     h_UccPosCommon - Handle to the common object.

 @Return        None.
*//***************************************************************************/
void UCC_POS_COMMON_DumpRegisters(t_Handle h_UccPosCommon);


#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of ucc_pos_common_runtime_control_grp */
/** @} */ /* end of ucc_pos_common_grp */
/** @} */ /* end of ucc_pos_grp */


#endif /* __UCC_POS_COMMON_EXT_H */
