/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_pos_ext.h

 @Description   UCC POS external structures and definitions.
*//***************************************************************************/
#ifndef __UCC_POS_EXT_H
#define __UCC_POS_EXT_H


#include "qe_ext.h"
#include "upc_ext.h"
#include "ucc_pos_def_ext.h"


/**************************************************************************//**
 @Group         ucc_pos_grp UCC POS Application Programming Interface

 @Description   UCC POS functions, definitions and enumerators

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         ucc_pos_controller_grp UCC POS Controller API

 @Description   UCC POS controller functions, definitions and enumerators

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         ucc_pos_controller_exception_grp UCC POS Controller Exception Unit

 @Description   UCC POS controller exception routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Tx and Rx event flags
*//***************************************************************************/
#define UCC_POS_EVENT_TX_UNDERRUN                (1<<31)
#define UCC_POS_EVENT_RX_PROTOCOL_VIOLATION      (1<<30)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_3_THRESHOLD (1<<23)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_2_THRESHOLD (1<<22)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_1_THRESHOLD (1<<21)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_0_THRESHOLD (1<<20)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_3_OVERFLOW  (1<<19)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_2_OVERFLOW  (1<<18)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_1_OVERFLOW  (1<<17)
#define UCC_POS_EVENT_TX_FRAME_QUEUE_0_OVERFLOW  (1<<16)
#define UCC_POS_EVENT_RX_ERROR_QUEUE_0_THRESHOLD (1<<14)
#define UCC_POS_EVENT_RX_ERROR_QUEUE_0_OVERFLOW  (1<<10)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_3_THRESHOLD (1<< 7)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_2_THRESHOLD (1<< 6)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_1_THRESHOLD (1<< 5)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_0_THRESHOLD (1<< 4)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_3_OVERFLOW  (1<< 3)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_2_OVERFLOW  (1<< 2)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_1_OVERFLOW  (1<< 1)
#define UCC_POS_EVENT_RX_FRAME_QUEUE_0_OVERFLOW  (1<< 0)


/**************************************************************************//**
 @Description   Rx error flags
*//***************************************************************************/
#define UCC_POS_RX_BUSY (1<<0)


/**************************************************************************//**
 @Description   Callback routine, invoked by the driver upon exception events.

 @Param[in]     h_App          - Handle to an application context.
 @Param[in]     exceptionFlags - A bitwise-or between any of the following flags:
                                 UCC_POS_EVENT_TX_UNDERRUN
                                 UCC_POS_EVENT_RX_PROTOCOL_VIOLATION
                                 UCC_POS_EVENT_TX_FRAME_QUEUE_0_OVERFLOW
                                 UCC_POS_EVENT_TX_FRAME_QUEUE_1_OVERFLOW
                                 UCC_POS_EVENT_TX_FRAME_QUEUE_2_OVERFLOW
                                 UCC_POS_EVENT_TX_FRAME_QUEUE_3_OVERFLOW
                                 UCC_POS_EVENT_RX_FRAME_QUEUE_0_OVERFLOW
                                 UCC_POS_EVENT_RX_FRAME_QUEUE_1_OVERFLOW
                                 UCC_POS_EVENT_RX_FRAME_QUEUE_2_OVERFLOW
                                 UCC_POS_EVENT_RX_FRAME_QUEUE_3_OVERFLOW
                                 UCC_POS_EVENT_RX_ERROR_QUEUE_0_OVERFLOW

 @Return        None.
 *//***************************************************************************/
typedef void (t_UccPosExceptionRoutine)(t_Handle h_App,uint32_t exceptionFlags);


/** @} */ /* end of ucc_pos_controller_exception_grp */
/**************************************************************************//**
 @Group         ucc_pos_controller_initialization_grp UCC POS Controller Initialization Unit

 @Description   UCC POS controller initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   POS Protocols
*//***************************************************************************/
typedef enum e_UccPosProtocol
{
    e_UCC_POS_ETH_PROTOCOL, /**< Layer 2 Ethernet protocol */
    e_UCC_POS_PPP_PROTOCOL  /**< Layer 2 PPP      protocol */
}
e_UccPosProtocol;


/**************************************************************************//**
 @Description   Configuration parameters
*//***************************************************************************/
typedef struct t_UccPosParam
{
    /* POS basic parameters */
    e_UccPosProtocol protocol; /**< Protocol of all ports in the UCC */
 
    /* System integration parameters */
    uint32_t baseAddress; /**< Base address of the UCC registers         */
    t_Handle h_Common;    /**< Handle to the POS common object           */
    e_Ucc    uccId;       /**< ID of the UCC                             */
    e_Upc    upcId;       /**< ID of the associated UPC                  */
    int      irq;         /**< UCC interrupt ID (system-specific number) */
 
    /* Application interface */
    t_Handle                  h_App;              /**< Handle to an application context,
                                                       passed to the application through
                                                       the user's callback routine */
    t_UccPosExceptionRoutine* f_ExceptionRoutine; /**< User's callback routine, invoked by
                                                       the driver upon exception events */
}
t_UccPosParam;


/**************************************************************************//**
 @Function      UCC_POS_Config

 @Description   Configures a UCC object.

 @Param[in]     p_UccPosParam - UCC configuration parameters.

 @Return        Handle to the UCC object on success; NULL otherwise.
*//***************************************************************************/
t_Handle UCC_POS_Config(t_UccPosParam* p_UccPosParam);


/**************************************************************************//**
 @Function      UCC_POS_Init

 @Description   Initializes the UCC object.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_Init(t_Handle h_UccPos);


/**************************************************************************//**
 @Function      UCC_POS_Free

 @Description   Deallocates the UCC object + memory allocated during its configuration and initialization.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_Free(t_Handle h_UccPos);


/**************************************************************************//**
 @Group         ucc_pos_controller_advance_config_grp UCC POS Controller Advanced Configuration Unit

 @Description   UCC POS controller advanced configuration routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxVfifoSize

 @Description   Configures the size of the UCC's Tx virtual FIFO.

 @Param[in]     h_UccPos    - Handle to the UCC object.
 @Param[in]     txVfifoSize - Size of the UCC's Tx virtual FIFO.
                The default size of the UCC's Tx virtual FIFO is [2048].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxVfifoSize(t_Handle h_UccPos,uint16_t txVfifoSize);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxVfifoSize

 @Description   Configures the size of the UCC's Rx virtual FIFO.

 @Param[in]     h_UccPos    - Handle to the UCC object.
 @Param[in]     rxVfifoSize - Size of the UCC's Rx virtual FIFO.
                The default size of the UCC's Rx virtual FIFO is [2048].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxVfifoSize(t_Handle h_UccPos,uint16_t rxVfifoSize);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxVfifoThreshold

 @Description   Configures the threshold of the UCC's Tx virtual FIFO.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     txVfifoThreshold - Threshold of the UCC's Tx virtual FIFO.
                The default threshold of the UCC's Tx virtual FIFO is [1024].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxVfifoThreshold(t_Handle h_UccPos,uint16_t txVfifoThreshold);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxVfifoEmerThreshold

 @Description   Configures the emergency threshold of the UCC's Tx virtual FIFO.

 @Param[in]     h_UccPos             - Handle to the UCC object.
 @Param[in]     txVfifoEmerThreshold - Emergency threshold of the UCC's Tx virtual FIFO.
                The default emergency threshold of the UCC's Tx virtual FIFO is [1024].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxVfifoEmerThreshold(t_Handle h_UccPos,uint16_t txVfifoEmerThreshold);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxVfifoEmerThreshold

 @Description   Configures the emergency threshold of the UCC's Rx virtual FIFO.

 @Param[in]     h_UccPos             - Handle to the UCC object.
 @Param[in]     rxVfifoEmerThreshold - Emergency threshold of the UCC's Rx virtual FIFO.
                The default emergency threshold of the UCC's Rx virtual FIFO is [1024].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxVfifoEmerThreshold(t_Handle h_UccPos,uint16_t rxVfifoEmerThreshold);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxVfifoSpecEmerThreshold

 @Description   Configures the special emergency threshold of the UCC's Rx virtual FIFO.

 @Param[in]     h_UccPos                 - Handle to the UCC object.
 @Param[in]     rxVfifoSpecEmerThreshold - Special emergency threshold of the UCC's Rx virtual FIFO.
                The default special emergency threshold of the UCC's Rx virtual FIFO is [1024].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxVfifoSpecEmerThreshold(t_Handle h_UccPos,uint16_t rxVfifoSpecEmerThreshold);


/**************************************************************************//**
 @Function      UCC_POS_ConfigExceptionMask

 @Description   Configures the UCC's exception mask.
                Exception events included in this mask shall be
                passed to the application through the user's callback routine.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     exceptionMask - A bitwise-or between any of the following flags:
                                UCC_POS_EVENT_TX_UNDERRUN
                                UCC_POS_EVENT_RX_PROTOCOL_VIOLATION
                The default UCC's exception mask is [uint32_t].

 @Cautions      If exceptionMask is not 0, then f_ExceptionRoutine must not be NULL.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigExceptionMask(t_Handle h_UccPos,uint32_t exceptionMask);


/**************************************************************************//**
 @Function      UCC_POS_ConfigPppAccValue

 @Description   Configures the value of the ACC (Address-And-Control Character) for each port in the UCC.

 @Param[in]     h_UccPos    - Handle to the UCC object.
 @Param[in]     pppAccValue - Value of the ACC generated upon Tx and verified upon Rx for each port in the UCC.
                The default value of the ACC (Address-And-Control Character) for each port in the UCC is [0xFF03].

 @Cautions      Relevant for PPP frames only.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigPppAccValue(t_Handle h_UccPos,uint16_t pppAccValue);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxSegmentSize

 @Description   Configures the size of the Tx segment associated within each port in the UCC.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     txSegmentSize - Size of the Tx segment associated within each port in the UCC.
                The default size of the Tx segment associated within each port in the UCC is [128].

 @Cautions      txSegmentSize must be a power of 2.
                txSegmentSize cannot be less than [MIN_VAL_txSegmentSize].
                txSegmentSize cannot be more than [MAX_VAL_txSegmentSize].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxSegmentSize(t_Handle h_UccPos,uint8_t txSegmentSize);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxMaxInProcess

 @Description   Configures the maximum number of ports in the UCC that can be in Tx process simultaneously.

 @Param[in]     h_UccPos       - Handle to the UCC object.
 @Param[in]     txMaxInProcess - Maximum number of ports in the UCC that can be in Tx process simultaneously.
                The default maximum number of ports in the UCC that can be in Tx process simultaneously is [255].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxMaxInProcess(t_Handle h_UccPos,uint8_t txMaxInProcess);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxAutoRecoveryMode

 @Description   Configures the Tx Automatic-Recovery mode of the UCC.

 @Param[in]     h_UccPos           - Handle to the UCC object.
 @Param[in]     txAutoRecoveryMode - Determines whether to automatically resume transmission after error or not.
                The default Tx Automatic-Recovery mode of the UCC is [TRUE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxAutoRecoveryMode(t_Handle h_UccPos,bool txAutoRecoveryMode);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxIdleOnLastMode

 @Description   Configures the Tx Idle-On-Last mode of the UCC.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     txIdleOnLastMode - Determines whether every transmission of a frame is followed by an IDLE command or not.
                The default Tx Idle-On-Last mode of the UCC is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxIdleOnLastMode(t_Handle h_UccPos,bool txIdleOnLastMode);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxIdleOnEmergencyMode

 @Description   Configures the Tx Idle-On-Emergency mode of the UCC.

 @Param[in]     h_UccPos              - Handle to the UCC object.
 @Param[in]     txIdleOnEmergencyMode - Determines whether every special emergency condition in the Rx virtual FIFO is followed by an IDLE data or not.
                The default Tx Idle-On-Emergency mode of the UCC is [TRUE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxIdleOnEmergencyMode(t_Handle h_UccPos,bool txIdleOnEmergencyMode);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxSegmentSize

 @Description   Configures the size of the Rx segment associated within each port in the UCC.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     rxSegmentSize - Size of the Rx segment associated within each port in the UCC.
                The default size of the Rx segment associated within each port in the UCC is [128].

 @Cautions      rxSegmentSize must be a power of 2.
                rxSegmentSize cannot be less than [MIN_VAL_rxSegmentSize].
                rxSegmentSize cannot be more than [MAX_VAL_rxSegmentSize].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxSegmentSize(t_Handle h_UccPos,uint8_t rxSegmentSize);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxMaxBufferLength

 @Description   Configures the UCC's maximum Rx buffer length.

 @Param[in]     h_UccPos          - Handle to the UCC object.
 @Param[in]     rxMaxBufferLength - UCC's maximum Rx buffer length.
                The default UCC's maximum Rx buffer length is [256].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxMaxBufferLength(t_Handle h_UccPos,uint16_t rxMaxBufferLength);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxMinFrameLength

 @Description   Configures the UCC's minimum Rx frame length.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     rxMinFrameLength - UCC's minimum Rx frame length.
                The default minimum Rx frame length for Ethernet frames is [64].
                The default minimum Rx frame length for PPP      frames is [40].

 @Cautions      rxMinFrameLength should specify the minimum possible length
                of an entire Ethernet / PPP frame (data + additional fields).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxMinFrameLength(t_Handle h_UccPos,uint16_t rxMinFrameLength);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxMaxFrameLength

 @Description   Configures the UCC's maximum Rx frame length.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     rxMaxFrameLength - UCC's maximum Rx frame length.
                The default maximum Rx frame length for Ethernet frames is [1518].
                The default maximum Rx frame length for PPP      frames is [1500].

 @Cautions      rxMaxFrameLength should specify the maximum possible length
                of an entire Ethernet / PPP frame (data + additional fields).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxMaxFrameLength(t_Handle h_UccPos,uint16_t rxMaxFrameLength);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxEthVlanType

 @Description   Configures the type value for Ethernet frames in which the next header will be regarded as VLAN.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     rxEthVlanType - Type value for Ethernet frames.
                The default type value for Ethernet frames is [0x8100].

 @Cautions      Relevant for Ethernet frames only.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxEthVlanType(t_Handle h_UccPos,uint16_t rxEthVlanType);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxEthTypeLenThreshold

 @Description   Configures the threshold value for type/length field of Ethernet frames:
                If the type/length field is smaller than the specified value, it is regarded as a length field.
                If the type/length field is not smaller than the specified value, it is regarded as a type field.

 @Param[in]     h_UccPos              - Handle to the UCC object.
 @Param[in]     rxEthTypeLenThreshold - Threshold value for type/length field of Ethernet frames.
                The default threshold value for type/length field of Ethernet frames is [1536].

 @Cautions      Relevant for Ethernet frames only.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxEthTypeLenThreshold(t_Handle h_UccPos,uint16_t rxEthTypeLenThreshold);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxVfifoPolicerLimit

 @Description   Configures the limit of the UCC's Rx virtual FIFO policer.

 @Param[in]     h_UccPos            - Handle to the UCC object.
 @Param[in]     rxVfifoPolicerLimit - Limit of the UCC's Rx virtual FIFO policer.
                The default limit of the UCC's Rx virtual FIFO policer is [0].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxVfifoPolicerLimit(t_Handle h_UccPos,uint16_t rxVfifoPolicerLimit);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxTermThreadRisc

 @Description   Configures the RISC of the UCC's Tx terminator thread.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     txTermThreadRisc - The RISC of the UCC's Tx terminator thread.
                The default RISC of the UCC's Tx terminator thread is [QE_RISC_ALLOC_ALL_RISCS].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxTermThreadRisc(t_Handle h_UccPos,uint8_t txTermThreadRisc);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxTermThreadRisc

 @Description   Configures the RISC of the UCC's Rx terminator thread.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     rxTermThreadRisc - The RISC of the UCC's Rx terminator thread.
                The default RISC of the UCC's Rx terminator thread is [QE_RISC_ALLOC_ALL_RISCS].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxTermThreadRisc(t_Handle h_UccPos,uint8_t rxTermThreadRisc);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxDistThreadRisc

 @Description   Configures the RISC of the UCC's Tx distributor thread.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     txDistThreadRisc - The RISC of the UCC's Tx distributor thread.
                The default RISC of the UCC's Tx distributor thread is [QE_RISC_ALLOC_ALL_RISCS].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxDistThreadRisc(t_Handle h_UccPos,uint8_t txDistThreadRisc);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxDistThreadRisc

 @Description   Configures the RISC of the UCC's Rx distributor thread.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     rxDistThreadRisc - The RISC of the UCC's Rx distributor thread.
                The default RISC of the UCC's Rx distributor thread is [QE_RISC_ALLOC_ALL_RISCS].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxDistThreadRisc(t_Handle h_UccPos,uint8_t rxDistThreadRisc);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxThreadJobQueueLength

 @Description   Configures the length of the UCC's Tx thread job queue.

 @Param[in]     h_UccPos               - Handle to the UCC object.
 @Param[in]     txThreadJobQueueLength - Length of the UCC's Tx thread job queue.
                The default length of the UCC's Tx thread job queue is [32].

 @Cautions      txThreadJobQueueLength cannot be less than [MIN_VAL_txThreadJobQueueLength].
                txThreadJobQueueLength cannot be more than [MAX_VAL_txThreadJobQueueLength].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxThreadJobQueueLength(t_Handle h_UccPos,uint8_t txThreadJobQueueLength);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxThreadJobQueueLength

 @Description   Configures the length of the UCC's Rx thread job queue.

 @Param[in]     h_UccPos               - Handle to the UCC object.
 @Param[in]     rxThreadJobQueueLength - Length of the UCC's Rx thread job queue.
                The default length of the UCC's Rx thread job queue is [32].

 @Cautions      rxThreadJobQueueLength cannot be less than [MIN_VAL_rxThreadJobQueueLength].
                rxThreadJobQueueLength cannot be more than [MAX_VAL_rxThreadJobQueueLength].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxThreadJobQueueLength(t_Handle h_UccPos,uint8_t rxThreadJobQueueLength);


/**************************************************************************//**
 @Description   Event queue configuration parameters
*//***************************************************************************/
typedef struct t_UccPosEventQueueParam
{
    uint16_t length;            /**< Maximum number of events that the queue can store              */
    uint16_t countThreshold;    /**< Number of events occurring, before an interrupt is issued      */
    uint32_t timeThreshold;     /**< Time elapsed after first event, before an interrupt is issued  */
    bool     overflowException; /**< Indicates whether to interrupt the CPU upon overflow exception */
}
t_UccPosEventQueueParam;


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxFrameEventQueuesMemory

 @Description   Configures memory information for the UCC's Tx frame event queues.
 
                The default location of the UCC's Tx frame event queues is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     memId         - Memory partition ID for UCC's Tx frame event queues.
 @Param[in]     memAttributes - Memory attributes for UCC's Tx frame event queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxFrameEventQueuesMemory(t_Handle h_UccPos,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxFrameEventQueuesMemory

 @Description   Configures memory information for the UCC's Rx frame event queues.

                The default location of the UCC's Rx frame event queues is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     memId         - Memory partition ID for UCC's Rx frame event queues.
 @Param[in]     memAttributes - Memory attributes for UCC's Rx frame event queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxFrameEventQueuesMemory(t_Handle h_UccPos,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxErrorEventQueuesMemory

 @Description   Configures memory information for the UCC's Rx error event queues.

                The default location of the UCC's Rx error event queues is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPos      - Handle to the UCC object.
 @Param[in]     memId         - Memory partition ID for UCC's Rx error event queues.
 @Param[in]     memAttributes - Memory attributes for UCC's Rx error event queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxErrorEventQueuesMemory(t_Handle h_UccPos,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxNumOfFrameEventQueues

 @Description   Configures the number of the UCC's Tx frame event queues.

 @Param[in]     h_UccPos                - Handle to the UCC object.
 @Param[in]     txNumOfFrameEventQueues - Number of the UCC's Tx frame event queues.
                The default number of the UCC's Tx frame event queues is [0].

 @Cautions      txNumOfFrameEventQueues cannot be less than [MIN_VAL_txNumOfFrameEventQueues].
                txNumOfFrameEventQueues cannot be more than [MAX_VAL_txNumOfFrameEventQueues].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxNumOfFrameEventQueues(t_Handle h_UccPos,uint8_t txNumOfFrameEventQueues);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxNumOfFrameEventQueues

 @Description   Configures the number of the UCC's Rx frame event queues.

 @Param[in]     h_UccPos                - Handle to the UCC object.
 @Param[in]     rxNumOfFrameEventQueues - Number of the UCC's Rx frame event queues.
                The default number of the UCC's Rx frame event queues is [0].

 @Cautions      rxNumOfFrameEventQueues cannot be less than [MIN_VAL_rxNumOfFrameEventQueues].
                rxNumOfFrameEventQueues cannot be more than [MAX_VAL_rxNumOfFrameEventQueues].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxNumOfFrameEventQueues(t_Handle h_UccPos,uint8_t rxNumOfFrameEventQueues);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxNumOfErrorEventQueues

 @Description   Configures the number of the UCC's Rx error event queues.

 @Param[in]     h_UccPos                - Handle to the UCC object.
 @Param[in]     rxNumOfErrorEventQueues - Number of the UCC's Rx error event queues.
                The default number of the UCC's Rx error event queues is [1].

 @Cautions      rxNumOfErrorEventQueues cannot be less than [MIN_VAL_rxNumOfErrorEventQueues].
                rxNumOfErrorEventQueues cannot be more than [MAX_VAL_rxNumOfErrorEventQueues].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxNumOfErrorEventQueues(t_Handle h_UccPos,uint8_t rxNumOfErrorEventQueues);


/**************************************************************************//**
 @Function      UCC_POS_ConfigTxFrameEventQueueParam

 @Description   Configures one of the UCC's Tx frame event queues.

 @Param[in]     h_UccPos                 - Handle to the UCC object.
 @Param[in]     txFrameEventQueueId      - ID of the Tx frame event queue.
 @Param[in]     p_TxFrameEventQueueParam - Parameters of the Tx frame event queue.
                The default for each one of the UCC's Tx frame event queues is [t_UccPosEventQueueParam].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigTxFrameEventQueueParam(t_Handle h_UccPos,uint8_t txFrameEventQueueId,t_UccPosEventQueueParam* p_TxFrameEventQueueParam);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxFrameEventQueueParam

 @Description   Configures one of the UCC's Rx frame event queues.

 @Param[in]     h_UccPos                 - Handle to the UCC object.
 @Param[in]     rxFrameEventQueueId      - ID of the Rx frame event queue.
 @Param[in]     p_RxFrameEventQueueParam - Parameters of the Rx frame event queue.
                The default for each one of the UCC's Rx frame event queues is [t_UccPosEventQueueParam].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxFrameEventQueueParam(t_Handle h_UccPos,uint8_t rxFrameEventQueueId,t_UccPosEventQueueParam* p_RxFrameEventQueueParam);


/**************************************************************************//**
 @Function      UCC_POS_ConfigRxErrorEventQueueParam

 @Description   Configures one of the UCC's Rx error event queues.

 @Param[in]     h_UccPos                 - Handle to the UCC object.
 @Param[in]     rxErrorEventQueueId      - ID of the Rx error event queue.
 @Param[in]     p_RxErrorEventQueueParam - Parameters of the Rx error event queue.
                The default for each one of the UCC's Rx error event queues is [t_UccPosEventQueueParam].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_ConfigRxErrorEventQueueParam(t_Handle h_UccPos,uint8_t rxErrorEventQueueId,t_UccPosEventQueueParam* p_RxErrorEventQueueParam);


/** @} */ /* end of ucc_pos_controller_advance_config_grp */
/** @} */ /* end of ucc_pos_controller_initialization_grp */
/**************************************************************************//**
 @Group         ucc_pos_controller_runtime_control_grp UCC POS Controller Runtime Control Unit

 @Description   UCC POS controller runtime control routines

 @{
*//***************************************************************************/


/**************************************************************************//*
 @Function      UCC_POS_GetId

 @Description   Returns the ID of the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        ID of the UCC.
*//***************************************************************************/
e_Ucc UCC_POS_GetId(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_IsUsingSpResource

 @Description   Indicates whether the UCC is using a Single-Port resource or a Multi-Port resource.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        TRUE - The UCC is using a Single-Port resource; FALSE - The UCC is using a Multi-Port resource.
*//***************************************************************************/
bool UCC_POS_IsUsingSpResource(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetQeHandle

 @Description   Returns a handle to the QE object (common to all UCCs).

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Handle to the QE object (common to all UCCs).
*//***************************************************************************/
t_Handle UCC_POS_GetQeHandle(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetPramPartitionId

 @Description   Returns the Parameter RAM partition ID, taken from POS common object.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Parameter RAM partition ID.
*//***************************************************************************/
uint8_t UCC_POS_GetPramPartitionId(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetProtocol

 @Description   Returns the protocol of all ports in the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Protocol of all ports in the UCC.
*//***************************************************************************/
e_UccPosProtocol UCC_POS_GetProtocol(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetTxSegmentSize

 @Description   Returns the size of the Tx segment associated within each port in the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Size of the Tx segment associated within each port in the UCC.
*//***************************************************************************/
uint8_t UCC_POS_GetTxSegmentSize(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetRxSegmentSize

 @Description   Returns the size of the Rx segment associated within each port in the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Size of the Rx segment associated within each port in the UCC.
*//***************************************************************************/
uint8_t UCC_POS_GetRxSegmentSize(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetRxMaxBufferLength

 @Description   Returns the UCC's maximum Rx buffer length.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        UCC's maximum Rx buffer length.
*//***************************************************************************/
uint16_t UCC_POS_GetRxMaxBufferLength(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetTxNumOfFrameEventQueues

 @Description   Returns the number of the UCC's Tx frame event queues.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Number of the UCC's Tx frame event queues.
*//***************************************************************************/
uint8_t UCC_POS_GetTxNumOfFrameEventQueues(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetRxNumOfFrameEventQueues

 @Description   Returns the number of the UCC's Rx frame event queues.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Number of the UCC's Rx frame event queues.
*//***************************************************************************/
uint8_t UCC_POS_GetRxNumOfFrameEventQueues(t_Handle h_UccPos);


/**************************************************************************//*
 @Function      UCC_POS_GetRxNumOfErrorEventQueues

 @Description   Returns the number of the UCC's Rx error event queues.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Number of the UCC's Rx error event queues.
*//***************************************************************************/
uint8_t UCC_POS_GetRxNumOfErrorEventQueues(t_Handle h_UccPos);


/**************************************************************************//*
 @Description   Callback routine, invoked by the driver upon Tx frame events.

 @Param[in]     h_PosPort - Handle to the port that has caused the event.
 @Param[in]     txDataQueueId - ID of the data queue which the event refers to.

 @Return        None.
 *//***************************************************************************/
typedef void (t_UccPosTxFrameRoutine)(t_Handle h_PosPort,uint8_t txDataQueueId);


/**************************************************************************//*
 @Function      UCC_POS_SetTxFrameRoutine

 @Description   Sets the callback routine, invoked by the driver upon Tx frame events.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     f_TxFrameRoutine - Tx frame event callback routine.

 @Return        None.
*//***************************************************************************/
void UCC_POS_SetTxFrameRoutine(t_Handle h_UccPos,t_UccPosTxFrameRoutine* f_TxFrameRoutine);


/**************************************************************************//*
 @Description   Callback routine, invoked by the driver upon Rx frame events.

 @Param[in]     h_PosPort     - Handle to the port that has caused the event.
 @Param[in]     rxDataQueueId - ID of the data queue which the event refers to.

 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
typedef t_Error (t_UccPosRxFrameRoutine)(t_Handle h_PosPort,uint8_t rxDataQueueId);


/**************************************************************************//*
 @Function      UCC_POS_SetRxFrameRoutine

 @Description   Sets the callback routine, invoked by the driver upon Rx frame events.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     f_RxFrameRoutine - Rx frame event callback routine.

 @Return        None.
*//***************************************************************************/
void UCC_POS_SetRxFrameRoutine(t_Handle h_UccPos,t_UccPosRxFrameRoutine* f_RxFrameRoutine);


/**************************************************************************//*
 @Description   Callback routine, invoked by the driver upon Rx error events.

 @Param[in]     h_PosPort     - Handle to the port that has caused the event.
 @Param[in]     rxDataQueueId - ID of the data queue which the event refers to.
 @Param[in]     errorFlags    - A bitwise-or between any of the following flags:
                                UCC_POS_RX_BUSY

 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
typedef t_Error (t_UccPosRxErrorRoutine)(t_Handle h_PosPort,uint8_t rxDataQueueId,uint16_t errorFlags);


/**************************************************************************//*
 @Function      UCC_POS_SetRxErrorRoutine

 @Description   Sets the callback routine, invoked by the driver upon Rx error events.

 @Param[in]     h_UccPos         - Handle to the UCC object.
 @Param[in]     f_RxErrorRoutine - Rx error event callback routine.

 @Return        None.
*//***************************************************************************/
void UCC_POS_SetRxErrorRoutine(t_Handle h_UccPos,t_UccPosRxErrorRoutine* f_RxErrorRoutine);


/**************************************************************************//*
 @Function      UCC_POS_GetTimeStampPeriod

 @Description   Returns the period of the QE's 2nd time stamp register in nanoseconds.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        Period of the QE's 2nd time stamp register in nanoseconds.
*//***************************************************************************/
uint32_t UCC_POS_GetTimeStampPeriod(t_Handle h_UccPos);


/**************************************************************************//**
 @Function      UCC_POS_Enable

 @Description   Enables the UCC for Tx and Rx.

 @Param[in]     h_UccPos - Handle to the UCC object.
 @Param[in]     commMode - Communication mode to enable in the UCC (Tx, Rx or both).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_Enable(t_Handle h_UccPos,e_CommMode commMode);


/**************************************************************************//**
 @Function      UCC_POS_Disable

 @Description   Disables the UCC for Tx and Rx.

 @Param[in]     h_UccPos - Handle to the UCC object.
 @Param[in]     commMode - Communication mode to disable in the UCC (Tx, Rx or both).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_Disable(t_Handle h_UccPos,e_CommMode commMode);


/**************************************************************************//*
 @Function      UCC_POS_RegisterPort

 @Description   Registers a port in the UCC.

 @Param[in]     h_UccPos  - Handle to the UCC object.
 @Param[in]     devId     - ID of the port's device.
 @Param[in]     portId    - ID of the port.
 @Param[in]     h_PosPort - Handle to the port object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_RegisterPort(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId,t_Handle h_PosPort);


/**************************************************************************//*
 @Function      UCC_POS_UnregisterPort

 @Description   Unregisters a port in the UCC.

 @Param[in]     h_UccPos  - Handle to the UCC object.
 @Param[in]     devId     - ID of the port's device.
 @Param[in]     portId    - ID of the port.
 @Param[in]     h_PosPort - Handle to the port object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_UnregisterPort(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId,t_Handle h_PosPort);


/**************************************************************************//*
 @Function      UCC_POS_AcquirePortTxPage

 @Description   Acquires the Tx page of a port in the UCC.

 @Param[in]     h_UccPos        - Handle to the UCC object.
 @Param[in]     devId           - ID of the port's device.
 @Param[in]     portId          - ID of the port.
 @Param[out]    p_PosPortTxPage - Pointer to the port's Tx page.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_AcquirePortTxPage(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId,void** p_PosPortTxPage);


/**************************************************************************//*
 @Function      UCC_POS_AcquirePortRxPage

 @Description   Acquires the Rx page of a port in the UCC.

 @Param[in]     h_UccPos        - Handle to the UCC object.
 @Param[in]     devId           - ID of the port's device.
 @Param[in]     portId          - ID of the port.
 @Param[out]    p_PosPortRxPage - Pointer to the port's Rx page.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_AcquirePortRxPage(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId,void** p_PosPortRxPage);


/**************************************************************************//*
 @Function      UCC_POS_EnablePort

 @Description   Enables a port in the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.
 @Param[in]     devId    - ID of the port's device.
 @Param[in]     portId   - ID of the port.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_EnablePort(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId);


/**************************************************************************//*
 @Function      UCC_POS_DisablePort

 @Description   Disables a port in the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.
 @Param[in]     devId    - ID of the port's device.
 @Param[in]     portId   - ID of the port.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_DisablePort(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId);


/**************************************************************************//*
 @Function      UCC_POS_SetPortPriority

 @Description   Changes the priority of a port in the UCC.

 @Param[in]     h_UccPos   - Handle to the UCC object.
 @Param[in]     devId      - ID of the port's device.
 @Param[in]     portId     - ID of the port.
 @Param[in]     rxPriority - Rx priority.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_SetPortPriority(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId,e_UpcPortPriority rxPriority);


/**************************************************************************//*
 @Function      UCC_POS_SetPortIntRate

 @Description   Changes the internal rate of a port in the UCC.

 @Param[in]     h_UccPos  - Handle to the UCC object.
 @Param[in]     devId     - ID of the port's device.
 @Param[in]     portId    - ID of the port.
 @Param[in]     txIntRate - Tx internal rate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_SetPortIntRate(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId,uint32_t txIntRate);


/**************************************************************************//*
 @Function      UCC_POS_GetPortUnderrunEventIndication

 @Description   Reports whether an underrun event has occurred for a port in the UCC.

 @Param[in]     h_UccPos - Handle to the UCC object.
 @Param[in]     devId    - ID of the port's device.
 @Param[in]     portId   - ID of the port.

 @Return        TRUE if an underrun event has occurred; FALSE otherwise.
*//***************************************************************************/
bool UCC_POS_GetPortUnderrunEventIndication(t_Handle h_UccPos,e_UpcDevice devId,uint8_t portId);


#if (DEBUG_ERRORS > 0)


/**************************************************************************//**
 @Function      UCC_POS_DumpRegisters

 @Description   Dumps the current values of the UCC registers.

 @Param[in]     h_UccPos - Handle to the UCC object.

 @Return        None.
*//***************************************************************************/
void UCC_POS_DumpRegisters(t_Handle h_UccPos);


#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of ucc_pos_controller_runtime_control_grp */
/** @} */ /* end of ucc_pos_controller_grp */
/** @} */ /* end of ucc_pos_grp */


#endif /* __UCC_POS_EXT_H */
