/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_pos_port_ext.h

 @Description   UCC POS Port external structures and definitions.
*//***************************************************************************/
#ifndef __UCC_POS_PORT_EXT_H
#define __UCC_POS_PORT_EXT_H


#include "qe_ext.h"
#include "upc_ext.h"
#include "ucc_pos_def_ext.h"


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigDefaultParameters

 @Description   Configures the port default parameters.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        None.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigDefaultParameters(t_Handle h_UccPosPort);


/**************************************************************************//**
 @Function      UCC_POS_PORT_ConfigInternalLoopbackMode

 @Description   Configures the Internal Loopback mode of the port.

 @Param[in]     h_UccPosPort         - Handle to the port object.
 @Param[in]     internalLoopbackMode - When TRUE, internal loopback is enabled.
                The default Internal Loopback mode of the port is [FALSE].

 @Cautions      Internal loopback can be enabled only for port 0 on device 0.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigInternalLoopbackMode(t_Handle h_UccPosPort,bool internalLoopbackMode);


/**************************************************************************//**
 @Description   CRC types
*//***************************************************************************/
typedef enum e_UccPosPortCrcType
{
    e_UCC_POS_PORT_NO_CRC,     /**< No CRC     generation/verification */
    e_UCC_POS_PORT_16_BIT_CRC, /**< 16 bit CRC generation/verification */
    e_UCC_POS_PORT_32_BIT_CRC  /**< 32 bit CRC generation/verification */
}
e_UccPosPortCrcType;


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigCrcType

 @Description   Configures the CRC type of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.
 @Param[in]     crcType      - Type of CRC generated upon Tx and verified upon Rx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigCrcType(t_Handle h_UccPosPort,e_UccPosPortCrcType crcType);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigIpChecksumMode

 @Description   Configures the IP Checksum mode of the port.

 @Param[in]     h_UccPosPort   - Handle to the port object.
 @Param[in]     ipChecksumMode - When TRUE, IP checksum is generated upon Tx and verified upon Rx.

 @Cautions      IP Checksum can be generated upon Tx and verified upon Rx only when the port's Rx parser is enabled.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigIpChecksumMode(t_Handle h_UccPosPort,bool ipChecksumMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigPppAcfcMode

 @Description   Configures the ACFC (Address-And-Control Character Field Compression) mode of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.
 @Param[in]     pppAcfcMode  - When TRUE, ACFC is enabled upon Tx and enabled upon Rx.

 @Cautions      ACFC can be enabled upon Tx and enabled upon Rx only when the port's UCC is running PPP protocol.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigPppAcfcMode(t_Handle h_UccPosPort,bool pppAcfcMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigPppPfcMode

 @Description   Configures the PFC (Protocol-ID Field Compression) mode of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.
 @Param[in]     pppPfcMode   - When TRUE, PFC is enabled upon Tx and enabled upon Rx.

 @Cautions      PFC can be enabled upon Tx and enabled upon Rx only when the port's UCC is running PPP protocol.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigPppPfcMode(t_Handle h_UccPosPort,bool pppPfcMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxErrorSignalMode

 @Description   Configures the Error Signal mode of the port.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     txErrorSignalMode - When TRUE, an error signal is generated upon Tx (on the POS L2 bus, at the end of every frame).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxErrorSignalMode(t_Handle h_UccPosPort,bool txErrorSignalMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxEthPaddingMode

 @Description   Configures the Padding mode of the port.

 @Param[in]     h_UccPosPort     - Handle to the port object.
 @Param[in]     txEthPaddingMode - When TRUE, padding is generated upon Tx.

 @Cautions      Padding can be generated upon Tx only when the port's UCC is running Ethernet protocol.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxEthPaddingMode(t_Handle h_UccPosPort,bool txEthPaddingMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxExtendedSchedMode

 @Description   Configures the Extended Scheduling mode of the port.
                Extended Scheduling mode supports two features:
                1. Tx traffic scheduling (using weights for each data queue).
                2. Tx traffic shaping (using rate limits for each data queue).

 @Param[in]     h_UccPosPort        - Handle to the port object.
 @Param[in]     txExtendedSchedMode - When TRUE, extended scheduling is enabled upon Tx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxExtendedSchedMode(t_Handle h_UccPosPort,bool txExtendedSchedMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxRegulationMode

 @Description   Configures the Regulation mode of the port.
                Regulation can be applied when the port's UCC reaches the
                maximum number of ports that can be in Tx process simultaneously.

 @Param[in]     h_UccPosPort     - Handle to the port object.
 @Param[in]     txRegulationMode - When TRUE, regulation is enabled upon Tx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxRegulationMode(t_Handle h_UccPosPort,bool txRegulationMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxFrameEventsMode

 @Description   Configures the Frame Events mode of the port.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     txFrameEventsMode - When TRUE, frame events are enabled upon Tx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxFrameEventsMode(t_Handle h_UccPosPort,bool txFrameEventsMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxEthIpAddrAlignMode

 @Description   Configures the IP Address Alignment mode of the port.

 @Param[in]     h_UccPosPort         - Handle to the port object.
 @Param[in]     rxEthIpAddrAlignMode - When TRUE, IP address is aligned upon Rx.

 @Cautions      IP address can be aligned upon Rx only when the port's UCC is running Ethernet protocol.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxEthIpAddrAlignMode(t_Handle h_UccPosPort,bool rxEthIpAddrAlignMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxFrameEventsMode

 @Description   Configures the Frame Events mode of the port.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     rxFrameEventsMode - When TRUE, frame events are enabled upon Rx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxFrameEventsMode(t_Handle h_UccPosPort,bool rxFrameEventsMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxErrorEventsMode

 @Description   Configures the Error Events mode of the port.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     rxErrorEventsMode - When TRUE, error events are enabled upon Rx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxErrorEventsMode(t_Handle h_UccPosPort,bool rxErrorEventsMode);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxStatCounters

 @Description   Configures the port's Tx statistical counters.

                The default location of the Tx statistical counters is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPosPort       - Handle to the port object.
 @Param[in]     txStatCountersMode - When TRUE, the port's Tx statistical counters are allocated.
 @Param[in]     memId              - Memory partition ID for port's Tx statistical counters.
 @Param[in]     memAttributes      - Memory attributes for the port's Tx statistical counters.

 @Cautions      When the port's UCC is in Single-Port mode, Tx statistical counters must reside in internal memory.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxStatCounters(t_Handle h_UccPosPort,bool txStatCountersMode,uint8_t memId,uint32_t memAttributes);
 

/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxStatCounters

 @Description   Configures the port's Rx statistical counters.

                The default location of the Rx statistical counters is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPosPort       - Handle to the port object.
 @Param[in]     rxStatCountersMode - When TRUE, the port's Rx statistical counters are allocated.
 @Param[in]     memId              - Memory partition ID for port's Rx statistical counters.
 @Param[in]     memAttributes      - Memory attributes for the port's Rx statistical counters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxStatCounters(t_Handle h_UccPosPort,bool rxStatCountersMode,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxDataQueuesMemory

 @Description   Configures memory information for the port's Tx data queues.

                The default location of the Tx data queues is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPosPort  - Handle to the port object.
 @Param[in]     memId         - Memory partition ID for port's Tx data queues.
 @Param[in]     memAttributes - Memory attributes for the port's Tx data queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxDataQueuesMemory(t_Handle h_UccPosPort,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxDataQueuesMemory

 @Description   Configures memory information for the port's Rx data queues.

                The default location of the Rx data queues is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPosPort  - Handle to the port object.
 @Param[in]     memId         - Memory partition ID for port's Rx data queues.
 @Param[in]     memAttributes - Memory attributes for the port's Rx data queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxDataQueuesMemory(t_Handle h_UccPosPort,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxDataQueueDescTableMemory

 @Description   Configures memory information for the port's Rx data queue descriptors.

                The default location of the Rx data queue descriptors is according
                to the structs partition configured in POS common object.

 @Param[in]     h_UccPosPort  - Handle to the port object.
 @Param[in]     memId         - Memory partition ID for port's Rx data queue descriptors.
 @Param[in]     memAttributes - Memory attributes for the port's Rx data queue descriptors.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxDataQueueDescTableMemory(t_Handle h_UccPosPort,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxNumOfDataQueues

 @Description   Configures the number of the port's Tx data queues.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     txNumOfDataQueues - Number of the port's Tx data queues.

 @Cautions      txNumOfDataQueues cannot be less than [MIN_VAL_txNumOfDataQueues].
                txNumOfDataQueues cannot be more than [MAX_VAL_txNumOfDataQueues].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxNumOfDataQueues(t_Handle h_UccPosPort,uint8_t txNumOfDataQueues);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxNumOfDataQueues

 @Description   Configures the number of the port's Rx data queues.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     txNumOfDataQueues - Number of the port's Rx data queues.

 @Cautions      rxNumOfDataQueues cannot be less than [MIN_VAL_rxNumOfDataQueues].
                rxNumOfDataQueues cannot be more than [MAX_VAL_rxNumOfDataQueues].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxNumOfDataQueues(t_Handle h_UccPosPort,uint8_t rxNumOfDataQueues);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxDataQueueLength

 @Description   Configures the length of one of the port's Tx data queues.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     txDataQueueId     - ID of the Tx data queue.
 @Param[in]     txDataQueueLength - Length of the Tx data queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxDataQueueLength(t_Handle h_UccPosPort,uint8_t txDataQueueId,uint8_t txDataQueueLength);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxDataQueueLength

 @Description   Configures the length of one of the port's Rx data queues.

 @Param[in]     h_UccPosPort      - Handle to the port object.
 @Param[in]     rxDataQueueId     - ID of the Rx data queue.
 @Param[in]     rxDataQueueLength - Length of the Rx data queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxDataQueueLength(t_Handle h_UccPosPort,uint8_t rxDataQueueId,uint8_t rxDataQueueLength);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigTxDataQueueFrameEqn

 @Description   Associates one of the port's Tx data queues to one of the UCC's Tx frame event queues.

 @Param[in]     h_UccPosPort        - Handle to the port object.
 @Param[in]     txDataQueueId       - ID of the Tx data queue.
 @Param[in]     txDataQueueFrameEqn - ID of the Tx frame event queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigTxDataQueueFrameEqn(t_Handle h_UccPosPort,uint8_t txDataQueueId,uint8_t txDataQueueFrameEqn);


/**************************************************************************//*
 @Function      UCC_POS_PORT_ConfigRxDataQueueFrameEqn

 @Description   Associates one of the port's Rx data queues to one of the UCC's Rx frame event queues.

 @Param[in]     h_UccPosPort        - Handle to the port object.
 @Param[in]     rxDataQueueId       - ID of the Rx data queue.
 @Param[in]     rxDataQueueFrameEqn - ID of the Rx frame event queue.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_ConfigRxDataQueueFrameEqn(t_Handle h_UccPosPort,uint8_t rxDataQueueId,uint8_t rxDataQueueFrameEqn);


/**************************************************************************//*
 @Function      UCC_POS_PORT_GetTxDataQueueLength

 @Description   Retrieves the length of a Tx data queue.

 @Param[in]     h_UccPosPort  - Handle to the port object.
 @Param[in]     txDataQueueId - ID of the Tx data queue.

 @Cautions      This routine can be invoked only after configuration and before initialization.

 @Return        The length of the Tx data queue.
*//***************************************************************************/
uint8_t UCC_POS_PORT_GetTxDataQueueLength(t_Handle h_UccPosPort,uint8_t txDataQueueId);


/**************************************************************************//*
 @Function      UCC_POS_PORT_GetRxDataQueueLength

 @Description   Retrieves the length of an Rx data queue.

 @Param[in]     h_UccPosPort  - Handle to the port object.
 @Param[in]     rxDataQueueId - ID of the Rx data queue.

 @Cautions      This routine can be invoked only after configuration and before initialization.

 @Return        The length of the Rx data queue.
*//***************************************************************************/
uint8_t UCC_POS_PORT_GetRxDataQueueLength(t_Handle h_UccPosPort,uint8_t rxDataQueueId);


/**************************************************************************//*
 @Function      UCC_POS_PORT_Init

 @Description   Initializes the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_Init(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_Enable

 @Description   Enables the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_Enable(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_Disable

 @Description   Disables the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_Disable(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_SetPriority

 @Description   Changes the priority of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.
 @Param[in]     rxPriority   - Rx priority.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_SetPriority(t_Handle h_UccPosPort,e_UpcPortPriority rxPriority);


/**************************************************************************//*
 @Function      UCC_POS_PORT_SetIntRate

 @Description   Changes the internal rate of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.
 @Param[in]     txIntRate    - Tx internal rate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_SetIntRate(t_Handle h_UccPosPort,uint32_t txIntRate);


/**************************************************************************//*
 @Function      UCC_POS_PORT_GetUnderrunEventIndication

 @Description   Reports whether an underrun event has occurred for the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        TRUE if an underrun event has occurred; FALSE otherwise.
*//***************************************************************************/
bool UCC_POS_PORT_GetUnderrunEventIndication(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Description   Tx data queue service parameters
*//***************************************************************************/
typedef struct t_UccPosPortTxServiceParam
{
    uint8_t bwPercentage;
    /**< The relative bandwidth (weight) of the Tx data queue */
 
    bool strictPriority;
    /**< FALSE - The scheduling of the Tx data queue depends on its weight      */
    /**< TRUE  - The scheduling of the Tx data queue is done with high priority */
 
    bool transmitAsap;
    /**< FALSE - The Tx data queue might endure shaper stall for bit rate regulation          */
    /**< TRUE  - The Tx data queue should be serviced for transmission right after scheduling */
 
    bool extraBandwidth;
    /**< FALSE - The bandwidth of the Tx data queue is monitored by the shaper bandwidth mechanism     */
    /**< TRUE  - The bandwidth of the Tx data queue is not monitored by the shaper bandwidth mechanism */
}
t_UccPosPortTxServiceParam;


/**************************************************************************//*
 @Function      UCC_POS_PORT_SetTxQueueService

 @Description   Changes the service for the port's Tx data queues.

 @Param[in]     h_UccPosPort     - Handle to the port object.
 @Param[in]     p_TxServiceParam - Service parameters for each Tx data queue.

 @Cautions      The relative bandwidth of each Tx data queue must be 1-100.
                The total bandwidth of all Tx data queues must be exactly 100.
                The relative bandwidth of a Tx data queue is irrelevant in strict priority.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_SetTxQueueService(t_Handle h_UccPosPort,t_UccPosPortTxServiceParam p_TxServiceParam[]);


/**************************************************************************//*
 @Function      UCC_POS_PORT_SetTxShaping

 @Description   Changes the port's Tx traffic shaping.

 @Param[in]     h_UccPosPort   - Handle to the port object.
 @Param[in]     averageBitRate - The average bit rate of the port (bits per second).
 @Param[in]     maxBurstLength - The longest burst of the port (bytes per burst).

 @Remarks       A burst is a group of packets with minimal IPG.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_SetTxTrafficShaping(t_Handle h_UccPosPort,uint32_t averageBitRate,uint32_t maxBurstLength);


/**************************************************************************//*
 @Function      UCC_POS_PORT_EnableTxStatCounters

 @Description   Enables the Tx statistical counters of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_EnableTxStatCounters(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_EnableRxStatCounters

 @Description   Enables the Rx statistical counters of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_EnableRxStatCounters(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_DisableTxStatCounters

 @Description   Disables the Tx statistical counters of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_DisableTxStatCounters(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_DisableRxStatCounters

 @Description   Disables the Rx statistical counters of the port.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_DisableRxStatCounters(t_Handle h_UccPosPort);


/**************************************************************************//*
 @Function      UCC_POS_PORT_FreeInitResources

 @Description   Deallocates memory allocated during the port's configuration and initialization.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_POS_PORT_FreeInitResources(t_Handle h_UccPosPort);


#if (DEBUG_ERRORS > 0)


/**************************************************************************//*
 @Function      UCC_POS_PORT_DumpRegisters

 @Description   Dumps the current values of the port registers.

 @Param[in]     h_UccPosPort - Handle to the port object.

 @Return        None.
*//***************************************************************************/
void UCC_POS_PORT_DumpRegisters(t_Handle h_UccPosPort);


#endif /* (DEBUG_ERRORS > 0) */


#endif /* __UCC_POS_PORT_EXT_H */
