/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_slow_ext.h

 @Description   UCC Slow external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_SLOW_EXT
#define __UCC_SLOW_EXT

#include "std_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         ucc_slow_id UCC Slow

 @Description   UCC Slow functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UCC Slow transparent CRC (TCRC)
 *//***************************************************************************/
typedef enum
{
    e_UCC_SLOW_GUMR_H_TRANSPARENT_CRC_TCRC_CCITT_CRC16 = 0x00000000 /**< 16-bit CCITT CRC (HDLC). (X16 + X12 + X5 + 1) */
   ,e_UCC_SLOW_GUMR_H_TRANSPARENT_CRC_TCRC_CRC16       = 0x00004000 /**<  CRC16 (BISYNC). (X16 + X15 + X2 + 1) */
   ,e_UCC_SLOW_GUMR_H_TRANSPARENT_CRC_TCRC_CCITT_CRC32 = 0x00008000 /**< 32-bit CCITT CRC (Ethernet and HDLC).
                                                                         (X32 + X26 + X23 + X22 + X16 + X12 + X11 + X10 + X8 + X7 + X5 + X4 + X2 + X1 + 1) */
} e_UccSlowGUMRHTransparentCrcTCRC;

/**************************************************************************//**
 @Description   UCC Slow oversampling rate for transmitter (TDCR)
 *//***************************************************************************/
typedef enum
{
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_TX_TDCR_1  = 0x00000000, /**< 1x  clock mode */
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_TX_TDCR_8  = 0x00010000, /**< 8x  clock mode */
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_TX_TDCR_16 = 0x00020000, /**< 16x clock mode */
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_TX_TDCR_32 = 0x00030000  /**< 32x clock mode */
} e_UccSlowGUMRLOversamplingRateTxTDCR;

/**************************************************************************//**
 @Description   UCC Slow Oversampling rate for receiver (RDCR)
 *//***************************************************************************/
typedef enum
{
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_1  = 0x00000000,  /**< 1x  clock mode */
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_8  = 0x00004000,  /**< 8x  clock mode */
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_16 = 0x00008000,  /**< 16x clock mode */
    e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_32 = 0x0000c000   /**< 32x clock mode */
} e_UccSlowGUMRLOversamplingRateRxRDCR;

/**************************************************************************//**
 @Description   UCC Slow Transmitter encoding method (TENC)
 *//***************************************************************************/
typedef enum
{
    e_UCC_SLOW_GUMR_L_TRANSMITTER_ENCODING_METHOD_TENC_NRZ  = 0x00000000, /**< nrz  */
    e_UCC_SLOW_GUMR_L_TRANSMITTER_ENCODING_METHOD_TENC_NRZI = 0x00000100  /**< nrzi */
} e_UccSlowGUMRLTransmitterEncodingMethodTENC;

/**************************************************************************//**
 @Description   UCC Slow Receiver decoding method (RENC)
 *//***************************************************************************/
typedef enum
{
    e_UCC_SLOW_GUMR_L_RECEIVER_DECODING_METHOD_RENC_NRZ     = 0x00000000, /**< nrz  */
    e_UCC_SLOW_GUMR_L_RECEIVER_DECODING_METHOD_RENC_NRZI    = 0x00000800  /**< nrzi */
} e_UccSlowGUMRLReceiverDecodingMethodRENC;

/**************************************************************************//**
 @Description   UCC Slow Diagnostic mode (DIAG)
 *//***************************************************************************/
typedef enum
{
    e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_NORMAL        = 0x00000000, /**< normal             */
    e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_LOOPBACK      = 0x00000040, /**< loop back          */
    e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_ECHO          = 0x00000080, /**< echo               */
    e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_LOOPBACK_ECHO = 0x000000c0  /**< loop back and echo */
} e_UccSlowGUMRLDiagnosticModeDIAG;

/* Slow UCC API Routines. */

/* << Parameter config routines Start >> */
/*
 * These declarations generated automatically by FunGen 1.1
 * Time:             Jul  5 2005 11:41:53
 * Source file:      GenFuncs.c
 */

/**************************************************************************//**
 @Function      UCC_SLOW_ConfigInitTx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to initialize for transmission. [TRUE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigInitTx ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigInitRx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to initialize for reception     [TRUE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigInitRx ( t_Handle h_UccSlow, bool newVal );

/**************************************************************************//**
 @Function      UCC_SLOW_ConfigGrantSupport

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to provide grant support. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigGrantSupport ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigTSA

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to use TSA, FALSE to use NMSI. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigTSA ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigCDP

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for envelope mode [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigCDP ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigCDSynchronous

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for CD input that is asynchronous with data,
                TRUE for synchronous. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigCDSynchronous ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigCTSP

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for envelope mode [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigCTSP ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigCTSSynchronous

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE for CTS input that is asynchronous with data,
                TRUE for synchronous. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigCTSSynchronous ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigInvertRxData

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to invert receive  data. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigInvertRxData ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigInvertTxData

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to invert transmit data. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigInvertTxData ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigRTSM

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                FALSE to send idles between frames,
                TRUE to send flags/syncs between frames. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigRTSM ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigReduceRxFifoWidth

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to Reduce Fifo Width down to 8-bits,
                one quarter of its normal size. [FALSE].

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigReduceRxFifoWidth ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigInvertTxClock

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to invert internal transmit clock. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigInvertTxClock ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigEncodeTXDLine

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE for the TXD line to be always encoded. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigEncodeTXDLine ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigReduceTxFifoLength

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE to reduce transmit fifo length to 1 word.
                This option is used with character-oriented protocols
                such as UART, to ensure a minimum FIFO latency
                at the expense of performance.
                FALSE to leave transmit fifo length at 128 bytes. [FALSE].

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigReduceTxFifoLength ( t_Handle h_UccSlow, bool newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigRxTxSync

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                TRUE for receiver-transmitter synchronization. [FALSE]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigRxTxSync ( t_Handle h_UccSlow, bool newVal );

/**************************************************************************//**
 @Function      UCC_SLOW_ConfigDiagnosticMode

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Diagnostic mode. [e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_NORMAL]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigDiagnosticMode ( t_Handle h_UccSlow, e_UccSlowGUMRLDiagnosticModeDIAG newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigOversamplingRateTx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Oversampling rate Tx [e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_TX_TDCR_1].

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigOversamplingRateTx ( t_Handle h_UccSlow, e_UccSlowGUMRLOversamplingRateTxTDCR newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigOversamplingRateRx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Oversampling rate Rx [e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_1]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigOversamplingRateRx ( t_Handle h_UccSlow, e_UccSlowGUMRLOversamplingRateRxRDCR newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigTxEncoding

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Transmitter encoding [e_UCC_SLOW_GUMR_L_TRANSMITTER_ENCODING_METHOD_TENC_NRZ]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigTxEncoding ( t_Handle h_UccSlow, e_UccSlowGUMRLTransmitterEncodingMethodTENC newVal );


/**************************************************************************//**
 @Function      UCC_SLOW_ConfigRxDecoding

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Receiver decoding [e_UCC_SLOW_GUMR_L_RECEIVER_DECODING_METHOD_RENC_NRZ]

 @Param         h_UccSlow - (In) handle to UCC SLOW
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, E_BADPARM if NULL h_UccSlow handle passed

 @Cautions      Change is applied only upon calling UCC_SLOW_Init.
*//***************************************************************************/
t_Error UCC_SLOW_ConfigRxDecoding ( t_Handle h_UccSlow, e_UccSlowGUMRLReceiverDecodingMethodRENC newVal );

/*
 * End of automatically generated declarations
 */

/* << Parameter config routines End >> */

/** @} */ /* end of group */

#endif /* __UCC_SLOW_EXT */



