/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          carrier_ext.h

 @Description   Definitions related to telecommunication carrier lines.
*//***************************************************************************/
#ifndef __CARRIER_EXT_H
#define __CARRIER_EXT_H

#include "std_ext.h"


/**************************************************************************//**
 @Description   Carrier Line Rate Enumeration

                Provides line rate enumeration of various carrier classes.
*//***************************************************************************/
typedef enum e_LineRate
{
    /* T-Carrier */
    e_LINE_RATE_T1,                     /**< T1 line rate (1.544 Mbps)      */
    e_LINE_RATE_DS1 = e_LINE_RATE_T1,   /**< DS1 line rate (1.544 Mbps)     */
    e_LINE_RATE_T2,                     /**< T2 line rate (6.312 Mbps)      */
    e_LINE_RATE_DS2 = e_LINE_RATE_T2,   /**< DS2 line rate (6.312 Mbps)     */
    e_LINE_RATE_T3,                     /**< T3 line rate (44.736 Mbps)     */
    e_LINE_RATE_DS3 = e_LINE_RATE_T3,   /**< DS3 line rate (44.736 Mbps)    */
    e_LINE_RATE_T4,                     /**< T4 line rate (274.216 Mbps)    */
    e_LINE_RATE_DS4 = e_LINE_RATE_T4,   /**< DS4 line rate (274.216 Mbps)   */
    e_LINE_RATE_T5,                     /**< T5 line rate (400.352 Mbps)    */
    e_LINE_RATE_DS5 = e_LINE_RATE_T5,   /**< DS5 line rate (400.352 Mbps)   */

    /* E-Carrier */
    e_LINE_RATE_E1,                     /**< E1 line rate (2.048 Mbps)      */
    e_LINE_RATE_E2,                     /**< E2 line rate (8.448 Mbps)      */
    e_LINE_RATE_E3,                     /**< E3 line rate (34.368 Mbps)     */
    e_LINE_RATE_E4,                     /**< E4 line rate (139.264 Mbps)    */
    e_LINE_RATE_E5,                     /**< E5 line rate (565.148 Mbps)    */

    /* Optical Carrier */
    e_LINE_RATE_OC1,                    /**< OC-1 line rate (51.84 Mbps)    */
    e_LINE_RATE_OC3,                    /**< OC-3 line rate (155.52 Mbps)   */
    e_LINE_RATE_OC12,                   /**< OC-12 line rate (622.08 Mbps)  */
    e_LINE_RATE_OC24,                   /**< OC-24 line rate (1244.16 Mbps) */
    e_LINE_RATE_OC48                    /**< OC-48 line rate (2488.32 Mbps) */

} e_LineRate;


#define OC_MAX_LINE_RATE(ocLevel) (uint32_t)((ocLevel)*51840000)


/**************************************************************************//**
 @Description   Carrier Line Encoding Type

                Enumeration of line encoding types.
*//***************************************************************************/
typedef enum e_LineEncoding
{
    e_LINE_ENCODING_AMI,    /**< AMI encoding */
    e_LINE_ENCODING_B3ZS,   /**< B3ZS encoding */
    e_LINE_ENCODING_B6ZS,   /**< B6ZS encoding */
    e_LINE_ENCODING_B8ZS,   /**< B8ZS encoding */
    e_LINE_ENCODING_HDB3    /**< HDB3 encoding */
} e_LineEncoding;


/**************************************************************************//**
 @Description   TDM Line Framing Mode

                Enumeration of TDM line framing modes.
*//***************************************************************************/
typedef enum e_TdmFramingMode
{
    e_TDM_FRAMING_MODE_SF,  /**< Super-Frame mode */
    e_TDM_FRAMING_MODE_ESF  /**< Extended Super-Frame mode */

} e_TdmFramingMode;



#endif /* __CARRIER_EXT_H */
