/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          e300_ext.h

 @Description   e300 external definitions prototypes
                This file is not included by the e300
                source file as it is an assembly file. It is used
                only for prototypes exposure, for inclusion
                by user and other modules.
*//***************************************************************************/
#ifndef __E300_EXT_H
#define __E300_EXT_H

#include "std_ext.h"


/**************************************************************************//**
 @Group         e300_id e300 Application Programming Interface

 @Description   e300 API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         e300_init_grp e300 Initialization Unit

 @Description   e300 initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/* MSR - Machine State Register */
#ifndef MSR_EE
#define MSR_EE     0x8000
#endif /* MSR_EE */
#ifndef MSR_CE
#define MSR_CE     0x0080
#endif /* MSR_CE */
#ifndef MSR_ME
#define MSR_ME     0x1000
#endif /* MSR_ME */

#define not_MSR_EECE    0xFFFF7F7F
#define not_MSR_EE      0xFFFF7FFF
#define MSR_CE_BN       24
#define MSR_EE_BN       16


#define CORE_DCacheEnable       E300_DCacheEnable
#define CORE_ICacheEnable       E300_ICacheEnable
#define CORE_DCacheDisable      E300_DCacheDisable
#define CORE_ICacheDisable      E300_ICacheDisable
#define CORE_GetId              E300_GetId
#define CORE_TestAndSet         E300_TestAndSet
#define CORE_MemoryBarrier      E300_MemoryBarrier
#define CORE_InstructionSync    E300_InstructionSync


/**************************************************************************//**
 @Function      E300_InvalidateBats

 @Description   Invalidates the BAT array.

 @Return        None.
*//***************************************************************************/
void E300_InvalidateBats(void);

/**************************************************************************//**
 @Function      E300_InvalidateTlbs

 @Description   Invalidates the TLB array.

 @Return        None.
*//***************************************************************************/
void E300_InvalidateTlbs(void);

/**************************************************************************//**
 @Function      E300_DTranslationEnable

 @Description   Enables the address translation for data.

                After calling this routine, the core data access is done
                using the MMU translation.

 @Return        None.
*//***************************************************************************/
void E300_DTranslationEnable(void);

/**************************************************************************//**
 @Function      E300_ITranslationEnable

 @Description   Enables the address translation for instructions.

                After calling this routine, the core instruction access is
                done using the MMU translation.

 @Return        None.
*//***************************************************************************/
void E300_ITranslationEnable(void);

/**************************************************************************//**
 @Function      E300_DTranslationDisable

 @Description   Disables the address translation for data.


 @Return        None.
*//***************************************************************************/
void E300_DTranslationDisable(void);

/**************************************************************************//**
 @Function      E300_ITranslationDisable

 @Description   Disables the address translation for instructions.

 @Return        None.
*//***************************************************************************/
void E300_ITranslationDisable(void);

/**************************************************************************//**
 @Function      E300_DCacheEnable

 @Description   Enables the data cache for memory pages that are
                not cache inhibited.

 @Return        None.
*//***************************************************************************/
void E300_DCacheEnable(void);

/**************************************************************************//**
 @Function      E300_ICacheEnable

 @Description   Enables the instruction cache for memory pages that are
                not cache inhibited.

 @Return        None.
*//***************************************************************************/
void E300_ICacheEnable(void);

/**************************************************************************//**
 @Function      E300_DCacheDisable

 @Description   Disables the data cache.

 @Return        None.
*//***************************************************************************/
void E300_DCacheDisable(void);

/**************************************************************************//**
 @Function      E300_ICacheDisable

 @Description   Disables the instruction cache.

 @Return        None.
*//***************************************************************************/
void E300_ICacheDisable(void);

/**************************************************************************//**
 @Function      E300_DCacheFlush

 @Description   Flushes the data cache in the specified range.

 @Param         startAddr  - (In) Start address to flush
 @Param         endAddr  -   (In) End address to flush

 @Return        None.
*//***************************************************************************/
void E300_DCacheFlush(uint32_t startAddr, uint32_t endAddr);

/**************************************************************************//**
 @Function      E300_GetTimeBaseTime

 @Description   Get the current value of the Time base.

                The routine reads the time base value.

 @Return        Current value (in micro seconds).
*//***************************************************************************/
uint64_t E300_GetTimeBaseTime(void);

/**************************************************************************//**
 @Function      E300_IsTaskletSupported

 @Description   Checks if tasklets are supported by the e300 interrupt handler.

 @Retval        TRUE    - Tasklets are supported.
 @Retval        FALSE   - Tasklets are not supported.
*//***************************************************************************/
bool E300_IsTaskletSupported(void);

/** @} */ /* end of e300_init_grp group */

/**************************************************************************//**
 @Group         e300_ex_grp e300 Exception Unit

 @Description   e300 exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      E300_GenericIntrHandler

 @Description   This routine is the default interrupt handler of all
                e300 interrupts.

 @Param         intrEntry  - (In) Interrupt vector entry.

 @Return        None
*//***************************************************************************/
void E300_GenericIntrHandler(uint32_t intrEntry);


/**************************************************************************//**
 @Function      CriticalIntr

 @Description   This is the specific critical e300 interrupt handler.

                It is called by the main assembly interrupt handler
                when an critical interrupt.

 @Param         intrEntry   - (In) The exception interrupt vector entry.
*//***************************************************************************/
void CriticalIntr(uint32_t intrEntry);


#ifndef E300_PM_SUPPORT
/**************************************************************************//**
 @Function      E300_PM_Intr

 @Description   This is the specific performance monitoring e300 interrupt handler.

                It is called by the main assembly interrupt handler
                when a performance monitoring interrupt.

 @Param         intrEntry   - (In) The exception interrupt vector entry.
*//***************************************************************************/
void E300_PM_Intr(uint32_t intrEntry);

#endif /* #ifndef E300_PM_SUPPORT */

/** @} */ /* end of e300_ex_grp group */
/** @} */ /* end of e300_grp group */


/************************************************************/
/*  The following routines are inter-modules and are not    */
/*  designed for direct application use. They are used by   */
/*  other modules in the drivers.                           */
/************************************************************/
void     MSR_EnableEE(void);
void     MSR_EnableME(void);
void     MSR_EnableCE(void);
void     MSR_DisableME(void);
void     MSR_DisableCE(void);
uint32_t MSR_ClearEE(void);
void     MSR_EnablePOW(void);
void     MSR_DisablePOW(void);
void     MSR_Restore(uint32_t oldMSR);
uint32_t GetMsr(void);
uint32_t GetDec(void);
uint32_t GetTimeBaseLow(void);
uint32_t GetTimeBaseHigh(void);
uint32_t GetSrr0(void);
uint32_t GetSrr1(void);
uint32_t GetLr(void);

void E300_SetDozeMode(void);
void E300_SetNapMode(void);
void E300_SetSleepMode(void);

void E300_RecoverDozeMode(void);
void E300_RecoverNapMode(void);
void E300_RecoverSleepMode(void);

uint32_t E300_GetSPR_HID0(void);
void E300_SetSPR_HID0(uint32_t val);

void E300_DBat0Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat0Read(uint32_t *batRegs);
void E300_IBat0Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat0Read(uint32_t *batRegs);
void E300_DBat1Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat1Read(uint32_t *batRegs);
void E300_IBat1Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat1Read(uint32_t *batRegs);
void E300_DBat2Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat2Read(uint32_t *batRegs);
void E300_IBat2Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat2Read(uint32_t *batRegs);
void E300_DBat3Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat3Read(uint32_t *batRegs);
void E300_IBat3Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat3Read(uint32_t *batRegs);
void E300_DBat4Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat4Read(uint32_t *batRegs);
void E300_IBat4Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat4Read(uint32_t *batRegs);
void E300_DBat5Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat5Read(uint32_t *batRegs);
void E300_IBat5Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat5Read(uint32_t *batRegs);
void E300_DBat6Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat6Read(uint32_t *batRegs);
void E300_IBat6Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat6Read(uint32_t *batRegs);
void E300_DBat7Write(uint32_t batxUpper, uint32_t batxLower );
void E300_DBat7Read(uint32_t *batRegs);
void E300_IBat7Write(uint32_t batxUpper, uint32_t batxLower );
void E300_IBat7Read(uint32_t *batRegs);
void E300_SetHighBats(void);
void E300_SetEBPX(void);
bool E300_IsValidHighBats(void);


/**************************************************************************//**
 @Function      E300_GetId

 @Description   Returns the core ID in the system.

 @Return        Core ID.
*//***************************************************************************/
static __inline__ uint32_t E300_GetId(void)
{
    return 0;
}

/**************************************************************************//**
 @Function      E300_TestAndSet

 @Description   This routine tries to atomically test-and-set an integer
                in memory to a non-zero value.

                The memory will be set only if it is tested as zero, in which
                case the routine retunrs the new non-zero value; otherwise the
                routine returns zero.

 @Param[in]     p - pointer to a volatile int in memory, on which test-and-set
                    operation should be made.

 @Retval        Zero        - Operation failed - memory was already set.
 @Retval        Non-zero    - Operation succeeded - memory has been set.
*//***************************************************************************/
int E300_TestAndSet(volatile int *p);

/**************************************************************************//**
 @Function      E300_MemoryBarrier

 @Description   This routine will cause the core to stop executing any commands
                until all previous memory read/write commands are completely out
                of the core's pipeline.

 @Return        None.
*//***************************************************************************/
static __inline__ void E300_MemoryBarrier(void)
{
    __asm__ ("eieio");
}

/**************************************************************************//**
 @Function      E300_InstructionSync

 @Description   This routine will cause the core to wait for previous instructions
                (including any interrupts they generate) to complete before the
                synchronization command executes, which purges all instructions
                from the processor's pipeline and refetches the next instruction.

 @Return        None.
*//***************************************************************************/
static __inline__ void E300_InstructionSync(void)
{
    __asm__ ("isync");
}


#endif /* __E300_EXT_H */
