/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          e300_mmu_ext.h

 @Description   e300 MMU external definitions and structures.
*//***************************************************************************/
#ifndef __E300_MMU_EXT_H
#define __E300_MMU_EXT_H

/**************************************************************************//**
 @Group         e300_mmu_id e300 MMU Application Programming Interface

 @Description   e300 MMU API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         e300_mmu_init_grp e300 MMU Initialization Unit

 @Description   e300 MMU initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

#define E300_MMU_MAX_PAGE_SIZE      0x10000000
#define E300_MMU_MIN_PAGE_SIZE      0x20000

/**************************************************************************//**
 @Description   BAT type select
*//***************************************************************************/
typedef enum e_MmuBatType
{
    e_MMU_DATA_BAT,         /**< Data BAT */
    e_MMU_INSTRUCTION_BAT   /**< INSTRUCTION BAT */
} e_MmuBatType;

/**************************************************************************//**
 @Description   BAT access select
*//***************************************************************************/
typedef enum e_MmuPageProtect
{
    e_MMU_BAT_NO_ACCESS     = 0x00000000,   /**< No Read/Write */
    e_MMU_BAT_READ_ONLY     = 0x00000001,   /**< Read only */
    e_MMU_BAT_READ_WRITE    = 0x00000002    /**< Read/Write*/
} e_MmuPageProtect;

/**************************************************************************//**
 @Description   BAT parameters structure
*//***************************************************************************/
typedef struct t_MmuBatEntryParam
{
    uint8_t             batId;          /**< BAT number. */
    e_MmuBatType        batType;        /**< Instruction/Data BAT. */
    uint32_t            pageSize;       /**< 128KB - 256MB (power of 2).
                                             TLB1 only. Translation page size. */
    uint32_t            virtualBaseAddr;
                                        /**< Virtual/effective beginning address of this page. */
    uint32_t            physicalBaseAddr;
                                        /**< Physical/Real beginning address of this page. */
    bool                writeThrough;   /**< "W" of "WIMG". When TRUE all stores to this
                                             page are written through cache to memory. */
    bool                cacheEnable;    /**< "Not I" of "WIMG". TRUE to consider this
                                             page cacheacle. */
    bool                memCoherencyReq;
                                        /**< "M" of "WIMG". when TRUE, memory coherency is required. */
    bool                guarded;        /**< "G" of "WIMG". when TRUE this page is guarded. */
    bool                userMode;       /**< Set user (problem) mode for this page. */
    bool                supervisorMode; /**< Set supervisor mode for this page.     */
    e_MmuPageProtect    pageProtection; /**< one of the e_PageProt options.         */
} t_MmuBatEntryParam;


/**************************************************************************//**
 @Function      E300_MMU_BatEntryWrite

 @Description   Configure a BAT entry - new or overwrite an existing one.

 @Param         p_MmuBatEntryParam - (In) Configuration structure for the
                                          specific TLB entry.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error E300_MMU_BatEntryWrite(t_MmuBatEntryParam *p_MmuBatEntryParam);

/**************************************************************************//**
 @Function      E300_MMU_BatEntryRead

 @Description   Get an initialized BAT entry specification from the MMU.

 @Param         p_MmuBatEntryParam - (In/Out) A structure of parameters defining
                                              the requested BAT entry. User should
                                              define batType (data/instruction)
                                              and batId only. All other structure
                                              members shall be filled in.

 @Return        TRUE if entry was read; FALSE if entry cannot be read.
*//***************************************************************************/
bool E300_MMU_BatEntryRead(t_MmuBatEntryParam *p_MmuBatEntryParam);

/** @} */ /* end of e300_mmu_init_grp group */
/** @} */ /* end of e300_mmu_grp group */

#endif /* __E300_MMU_EXT_H */

