/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          e300_pm_ext.h

 @Description   e300 Performance Monitor Application Programming Interface
*//***************************************************************************/
#ifndef __E300_PM_EXT_H
#define __E300_PM_EXT_H


/**************************************************************************//**
 @Group         e300_performance_id e300 Performance Monitor Application Programming Interface

 @Description   e300 Performance Monitor functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         e300_performance_api e300 Performance Monitor Control Unit

 @Description   e300 Performance Monitor runtime control routines and defintions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Types of performance monitoring events.
*//***************************************************************************/
typedef enum e_E300PmEvent
{
      e_E300_PM_NOTHING                              = 0 /**< Nonspec Register counter holds current value */
    , e_E300_PM_PROCESSOR_CYCLES                     = 1 /**< Nonspec Every processor cycle */
    , e_E300_PM_INSTRUCTIONS_COMPLETED               = 2 /**< Nonspec Completed instructions. 0, 1, or 2 per cycle. */

    , e_E300_PM_INSTRUCTIONS_FETCHED                 = 4 /**< Spec Fetched instructions. 0, 1, 2, 3, or 4 per cycle. (instructions written to the IQ.) */

    , e_E300_PM_EVENT_TRANSITIONS                = 6 /**< Spec 0 to 1 transitions on the pm_event input. */
    , e_E300_PM_EVENT_CYCLES                     = 7 /**< Spec Processor cycles that occur when the pm_event input is asserted. */

    , e_E300_PM_BRANCH_INSTRUCTIONS_COMPLETED        = 8 /**< Nonspec Completed branch instructions. */
    , e_E300_PM_LOAD_COMPLETED                       = 9 /**< Completed load (l*, load-update (1 load)), load-multiple (1-32),
                                                              dcbt(L1, CT = 0), and dcbtst(L1, CT = 0) */
    , e_E300_PM_STORE_COMPLETED                      = 10 /**< Completed store (st*, store-update (1 store)),
                                                              store-multiple (1-32),icbi, dcbf, dcbst, dcbt(CT = 1),
                                                              dcbtst(CT = 1), dcbz, icbt(CT = 1) */

    , e_E300_PM_BRANCHES_FINISHED                    = 12 /**< Spec Includes all branch instructions */
    , e_E300_PM_TAKEN_BRANCHES_FINISHED              = 13 /**< Spec Includes all taken branch instructions */

    , e_E300_PM_BRANCHES_MISPREDICTED                = 15 /**< Spec Counts branch instructions mispredicted due to direction, target
                                                               (for example if the CTR contents change), or IAB prediction. Does not
                                                               count instructions that the branch predictor incorrectly predicted to be branches. */
    , e_E300_PM_CYCLES_DECODE_STALLED                = 18 /**< Spec Cycles the IQ is not empty but 0 instructions decoded */
    , e_E300_PM_CYCLES_ISSUE_STALLED                 = 19 /**< Spec Cycles the issue buffer is not empty but 0 instructions issued */

    , e_E300_PM_CACHE_INHIBITED_TRANSLATED           = 31 /**< Spec Cache inhibited accesses translated */

    , e_E300_PM_NUMBER_OF_FETCHES                    = 61 /**< Spec Counts fetches that write at least one instruction to the IQ. (With instruction fetched (com:4), can used to compute instructions-per-fetch) */

    , e_E300_PM_MMU_INSIDE_MISS                      = 62 /**< Spec Counts instruction TLB miss exceptions */

    , e_E300_PM_BIU_MASTER_REQUESTS                  = 67 /**< Spec Master transaction starts (assertions of ts */
    , e_E300_PM_BIU_MASTER_INSTRUCTION_SIDE_REQUESTS = 68 /**< Spec Master instruction-side assertions of ts */
    , e_E300_PM_BIU_MASTER_DATA_SIDE_REQUESTS        = 69 /**< Spec Master data-side assertions of ts */

    , e_E300_PM_BIU_MASTER_RETRIES                   = 71 /**< Spec Transactions initiated by this processor that were retried on the BIU
                                                               interface. (The core is master and another device retries the core transaction.) */

    , e_E300_PM_SNOOP_PUSHES                         = 74 /**< N/A Snoop pushes from all data-side resources. (Counts snoop ARTRYs and WOPs.) */

    , e_E300_PM_PMC0_OVERFLOW                        = 82 /**< N/A PMC0[32] transitions from 1 to 0. */
    , e_E300_PM_PMC1_OVERFLOW                        = 83 /**< N/A PMC1[32] transitions from 1 to 0. */
    , e_E300_PM_PMC2_OVERFLOW                        = 84 /**< N/A PMC2[32] transitions from 1 to 0. */
    , e_E300_PM_PMC3_OVERFLOW                        = 85 /**< N/A PMC3[32] transitioned from 1 to 0. */

    , e_E300_PM_INTERRUPTS_TAKEN                     = 86 /**< Nonspec  */
    , e_E300_PM_EXTERNAL_INPUT_INTERRUPTS_TAKEN      = 87 /**< Nonspec  */
    , e_E300_PM_CRITICAL_INPUT_INTERRUPTS_TAKEN      = 88 /**< Nonspec  */
    , e_E300_PM_SC_AND_TRAP_INTERRUPTS               = 89 /**< Nonspec  */
    , e_E300_PM_TRANSITIONS_OF_TBL                   = 90 /**< Nonspec Counts transitions of the TBL bit selected by PMGC0[TBSEL]. */

    , e_E300_PM_I_CACHE_HITS                         = 96 /**< Spec Number of fetches that hit in i-cache */
    , e_E300_PM_INSTRUCTIONS_FLODED                  = 97 /**< Spec Number of instructions folded (used to determine true number of
                                                               instructions completed) */

    , e_E300_PM_STALLS_DUE_TO_COMPLETION_BUFFER      = 100 /**< Spec Cycles issue stalled due to full completion buffer */

    , e_E300_PM_STALLED_COMPLETION                   = 104 /**< Spec Cycles that completion is stalled */
    , e_E300_PM_STALLS_DUE_TO_LOAD                   = 105 /**< Spec Cycles that completion is stalled due to load */
    , e_E300_PM_STALLS_DUE_TO_FLOATING_POINT         = 106 /**< Spec Cycles that completion is stalled due to floating point instruction */

    , e_E300_PM_LOAD_STORES_TO_CACHEABLE_SPACE       = 108 /**< Spec Number of loads and stores to cacheable space in the data cache */
    , e_E300_PM_LOAD_STORE_HIT_IN_CACHE              = 109 /**< Spec Number of loads and stores that hit  in the data cache */

}e_E300PmEvent;


/**************************************************************************//**
 @Function          E300_PM_SetupCounters

 @Description       The routine defines a PERFORMANCE counter

 @Param[in]         counter     -  counter number
 @Param[in]         event       -  event to count for this counter

 @Return            E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error E300_PM_SetupCounter(uint8_t counter, e_E300PmEvent event);

/**************************************************************************//**
 @Function          E300_PM_Start

 @Description       The routine start the PERFORMANCE counting.

 @Return            None
*//***************************************************************************/
void E300_PM_Start(void);

/**************************************************************************//**
 @Function          E300_PM_Stop

 @Description       The routine stop the PERFORMANCE counting.

 @Return            None
*//***************************************************************************/
void E300_PM_Stop(void);

/**************************************************************************//**
 @Function          E300_PM_GetCounters

 @Description       The routine return counter value.

 @Param[in]         counter -  counter number

 @Return            64 bit counter value.
*//***************************************************************************/
uint64_t E300_PM_GetCounter(uint8_t counter);

/**************************************************************************//**
 @Function          E300_PM_GetNumCounters

 @Description       The routine return number of counters avilable.

 @Return            number of available counters.
*//***************************************************************************/
uint32_t E300_PM_GetNumCounters(void);


/** @} */ /* end of e300_performance_ctrl group */
/** @} */ /* end of e300_performance_id group */


#endif /* __E300_PM_EXT_H */
