/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          e500v2_L2_ext.h

 @Description   L2 CACHE/SRAM external structures and definitions.
*//***************************************************************************/
#ifndef __E500_L2_EXT_H
#define __E500_L2_EXT_H

/**************************************************************************//**
 @Group         e500v2_l2_id L2 Application Programming Interface

 @Description   L2 functions,definitions, structures and enums

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         e500v2_L2_struct L2/Sram Data Structures ad variables


 @Description   L2/SRAM Data Structures ad variables.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   L2CTL mode bits
 *//***************************************************************************/
typedef enum e_L2Mode
{
    e_L2_L2CTL_NO_SRAM          = 0,  /**< No SRAM */
    e_L2_L2CTL_FULL_SRAM        = 1,  /**< Entire Array is SRAM */
    e_L2_L2CTL_1_HALF_SRAM      = 2,  /**< One half of array is an SRAM */
    e_L2_L2CTL_2_HALVES_SRAM    = 3,  /**< Both halves of array are SRAM */
    e_L2_L2CTL_1_QUARTER_SRAM   = 4,  /**< One quarter of the array is SRAM */
    e_L2_L2CTL_2_QUARTERS_SRAM  = 5,  /**< Two quarters of the array are SRAM */
    e_L2_L2CTL_1_EIGHTH_SRAM    = 6,  /**< One eighth of the array is an SRAM */
    e_L2_L2CTL_2_EIGHTHS_SRAM   = 7   /**< Two eighths of the array are SRAM */
} e_L2Mode;

/**************************************************************************//**
 @Description   L2CTL mode bits
 *//***************************************************************************/
typedef enum e_L2Stash
{
    e_L2_L2CTL_NO_STASH         = 0,  /**< No Stash */
    e_L2_L2CTL_HALF_STASH       = 1,  /**< One half of the array is a stash-only cache */
    e_L2_L2CTL_QUARTER_STASH    = 2,  /**< One quarter of the array is a stash-only cache */
    e_L2_L2CTL_EIGHTH_STASH     = 3   /**< One eighth of the array is a stash-only cache (way7 of each set) */
} e_L2Stash;


typedef enum e_L2StashSize
{
    e_L2_STASH_256bytes     = 0xFFFFFFF,
    e_L2_STASH_512bytes     = 0xFFFFFFE,
    e_L2_STASH_1Kbyte       = 0xFFFFFFC,
    e_L2_STASH_2Kbytes      = 0xFFFFFF8,
    e_L2_STASH_4Kbytes      = 0xFFFFFF0,
    e_L2_STASH_8Kbytes      = 0xFFFFFE0,
    e_L2_STASH_16Kbytes     = 0xFFFFFC0,
    e_L2_STASH_32Kbytes     = 0xFFFFF80,
    e_L2_STASH_64Kbytes     = 0xFFFFF00,
    e_L2_STASH_128Kbytes    = 0xFFFFE00,
    e_L2_STASH_256Kbytes    = 0xFFFFC00,
    e_L2_STASH_512Kbytes    = 0xFFFF800,
    e_L2_STASH_1Mbyte       = 0xFFFF000,
    e_L2_STASH_2Mbytes      = 0xFFFE000,
    e_L2_STASH_4Mbytes      = 0xFFFC000,
    e_L2_STASH_8Mbytes      = 0xFFF8000,
    e_L2_STASH_16Mbytes     = 0xFFF0000,
    e_L2_STASH_32Mbytes     = 0xFFE0000,
    e_L2_STASH_64Mbytes     = 0xFFC0000,
    e_L2_STASH_128Mbytes    = 0xFF80000,
    e_L2_STASH_256Mbytes    = 0xFF00000,
    e_L2_STASH_512Mbytes    = 0xFE00000,
    e_L2_STASH_1Gbyte       = 0xFC00000,
    e_L2_STASH_2Gbytes      = 0xF800000,
    e_L2_STASH_4Gbytes      = 0xF000000,
    e_L2_STASH_8Gbytes      = 0xE000000,
    e_L2_STASH_16Gbytes     = 0xC000000,
    e_L2_STASH_32Gbytes     = 0x8000000,
    e_L2_STASH_64Gbytes     = 0x0000000
} e_L2StashSize;


/**************************************************************************//**
 @Description   Cache type
 *//***************************************************************************/
typedef enum e_CacheType
{
    e_L2_D_CACHE,                 /**< Data Cache */
    e_L2_I_CACHE                  /**< Instruction Cache */
} e_CacheType;


/**************************************************************************//**
 @Description   Error types
 *//***************************************************************************/
typedef enum e_L2ErrType
{
    e_L2_TAG_PARITY_ERR     = 0x00000010,   /**< TAG PARITY ERROR */
    e_L2_MULT_BIT_ERR       = 0x00000008,   /**< MULTI BIT ERROR   */
    e_L2_SINGLE_BIT_ERR     = 0x00000004,   /**< SINGLE BIT ERROR */
    e_L2_CONFIG_ERROR       = 0x00000001    /**< CONFIG ERROR   */
} e_L2ErrType;

/**************************************************************************//**
 @Description   Error Policy
 *//***************************************************************************/
typedef enum
{
    e_ERR_DISABLE,              /**< ERROR DISABLE */
    e_ERR_ENABLE_CAPTURE,       /**< ERROR ENABLE CAPTURE */
    e_ERR_ENABLE_INTERRUPT,     /**< ERROR ENABLE INTERRUPT */
    e_ERR_ENABLE_MACHINE_CHECK  /**< ERROR ENABLE MACHINE CHECK */
} e_ErrPolicy;

/**************************************************************************//**
 @Description   L2 Detected error transaction type.
 *//***************************************************************************/
typedef enum e_L2TransType
{
    e_L2_TRANS_TYPE_SNOOP       = 0x00000000,   /**< TRANS TYPE SNOOP */
    e_L2_TRANS_TYPE_WR          = 0x00001000,   /**< TRANS TYPE WR */
    e_L2_TRANS_TYPE_RD          = 0x00002000,   /**< TRANS TYPE RD */
    e_L2_TRANS_TYPE_RD_MOD_WR   = 0x00003000    /**< TRANS TYPE RD MOD WR */
} e_L2TransType;

/**************************************************************************//**
 @Description   L2 Detected error transaction source.
 *//***************************************************************************/
typedef enum e_L2TransSrc
{
    e_L2_TRANS_SRC_EXTERNAL = 0x00000000,   /**< TRANS SRC EXTERNAL */
    e_L2_TRANS_SRC_PROC_INST = 0x00100000,  /**< TRANS SRC PROC INST */
    e_L2_TRANS_SRC_PROC_DATA = 0x00110000   /**< TRANS SRC PROC DATA */
} e_L2TransSrc;



/**************************************************************************//**
 @Description   L2 Input Parameter Structure.
 *//***************************************************************************/
typedef struct t_L2Param
{
    uint32_t    baseAddress;        /**< Base address of L2 registers in memory. */
    int         irq;                /**< L2 interrupt ID (system-specific number) */
    bool        l2Active;           /**< TRUE to activale L2/SRAM */
    bool        dataOnly;           /**< Cache mode only:
                                         TRUE for allocating only data lines. */
    bool        instructionOnly;    /**< Cache mode only:
                                         TRUE for allocating only instruction lines. */
    bool        cacheInterventionDisable;
                                    /**< Cache mode only:   TRUE to disable cache
                                         read intervention. */
    e_L2Mode    l2Mode;             /**< defines the Cache/SRAM mode. */

    e_L2Stash   l2Stash;            /**< defines the Stash size */
    uint64_t    sramBaseAddrBlock0; /**< In SRAM mode define block 1 base address */
    uint64_t    sramBaseAddrBlock1; /**< In SRAM mode define block 2 base address */

} t_L2Param;


/**************************************************************************//**
 @Description   Data structure to report error attributes to user.
                The ErrReport routine fills a user structure with the
                current values. Parameters marked with % (in their comment)
                are valid only when ErrReport routine returns TRUE.
 *//***************************************************************************/
typedef struct t_L2Err
{
    bool            multErr;        /**<  if TRUE, multiple errors of the same type
                                          detected. Otherwise - single error.      */
    bool            tagParityErr;   /**<  TRUE if tag parity error detected. FALSE otherwise. */
    bool            multBitErr;     /**<  TRUE if multi bit ECC error
                                          detected. FALSE otherwise.               */
    bool            singleBitErr;   /**< TRUE if single bit ECC error
                                         detected. FALSE otherwise.               */
    bool            configErr;      /**< TRUE if illegal configuration error. FALSE otherwise. */
    uint32_t        dataCaptureHi;  /**< Memory data path error capture high word.*/
    uint32_t        dataCaptureLow; /**< Memory data path error injection low word. */
    uint8_t         eccSyndrome;    /**< ECC syndrome bits.                       */
    uint8_t         eccChksum;      /**< The datapath ECC of the failing doubleword. */
    uint8_t         dwordNum;       /**<  % Double word number of the detected error. */
    bool            burst;          /**<  % TRUE if the detected error was on burst transaction,
                                            FALSE if single beat (<=64).*/
    uint8_t         transSize;      /**<  % Transaction size in bytes.          */
    e_L2TransSrc    transSrc;       /**<  % Id of the transaction source for the
                                            error. Returned as e_L2TransSrc type.    */
    e_L2TransType   transType;      /**<  % Transaction type for the error.
                                            Returned as e_L2TransType type           */
    uint32_t        transAddr;      /**<  Address of the transaction               */

} t_L2Err;


#define NUM_OF_CACHE_REGIONS    4
#define NO_AVAILABLE_REGION     NUM_OF_CACHE_REGIONS

/**************************************************************************//**
 @Description   L2 Cache  external write region.
 *//***************************************************************************/
typedef struct t_L2CacheRegionParam
{
    uint32_t    baseAddr;   /**< Base address of the external write region. */
    uint32_t    size;       /**< size of the external write region; must be
                                 at least 256 bytes and a power of 2. */
    bool        lock;       /**< If TRUE, external writes matching this region
                                 will be locked. */
} t_L2CacheRegionParam;

/** @} */ /* end of e500v2_L2_struct group */

/**************************************************************************//**
 @Group         e500v2_L2_init_config L2/Sram Initialization Unit

 @Description   L2/SRAM initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      L2_Config

 @Description   Creates L2 descriptor according to user parameters.

                User parameters are the minimum necessary for L@ init

 @Param         p_L2Param   - (In) pointer to structure of user-specified parameters.

 @Return        L2 module descriptor. NULL if failed.
*//***************************************************************************/
t_Handle L2_Config(t_L2Param *p_L2Param);

/**************************************************************************//**
 @Function      L2_Init

 @Description   Fully initializes L2 module enabling it.

 @Param         h_L2 - (In) the L2 descriptor returned by L2_Config.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  L2_Init(t_Handle h_L2);

/**************************************************************************//**
 @Function      L2_Free

 @Description   Disable and Frees all resources used by L2 module.

 @Param         h_L2 - (In) the L2 descriptor returned by L2_Config.

@Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error  L2_Free(t_Handle h_L2);

/**************************************************************************//**
 @Function      L2_MapStashRegion

 @Description   Define a region of memory address as stashed area.

 @Param         h_L2 - (In) the L2 descriptor returned by L2_Config.

 @Param         regionId - (In) one of 4 region ID for stash area

 @Param         baseAddr - (In) Start Address of the area to be stashed.

 @Param         size - (In) Size of the area to be stashed.

 @Param         lock - (In) Lock or no the data in the L2

@Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error L2_MapStashRegion(t_Handle      h_L2,
                          uint8_t       regionId,
                          uint32_t      baseAddr,
                          e_L2StashSize size,
                          bool          lock);

/** @} */ /* end of e500v2_L2_init_config group */





/** @} */ /* end of e500v2_l2_id Programming Interface group */
#endif

