/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          e500v2_asm_ext.h

 @Description   E500 asmbler external definitions prototypes

                This file is included by the E500 asm & C files.
                you must set INCLUDE_FROM_ASM when including in asm files
                This files defins BITS and function of CPU specific asmbler routines.

*//***************************************************************************/
#ifndef __E500v2_ASM_EXT_H
#define __E500v2_ASM_EXT_H

#ifndef INCLUDE_FROM_ASM
#include "std_ext.h"
#endif  /* INCLUDE_FROM_ASM */

/*
 *
 * Fields in special registers
 * ---------------------------
 */

/* HID0 bits */
#define HID0_EMCP               0x80000000
#define HID0_EMCP_SHIFTED       0x8000
#define HID0_DOZE               0x00800000
#define HID0_NAP                0x00400000
#define HID0_SLEEP              0x00200000
#define HID0_TBEN               0x00004000
#define HID0_SEL_TBCLK          0x00002000
#define HID0_EN_MAS7_UPDATE     0x00000080
#define HID0_DCFA               0x00000040
#define HID0_NOPTI              0x00000001

/* HID1 bits */
#define HID1_ASTME              0x00002000
#define not_HID1_ASTME          0xFFFFDFFF
#define HID1_ABE                0x00001000
#define not_HID1_ABE            0xFFFFEFFF


/* TSR bits */
#define TSR_ENW                 0x80000000
#define TSR_WIS                 0x40000000
#define TSR_DIS                 0x08000000
#define TSR_FIS                 0x04000000

/* TCR bits */
#define TCR_DIE                 0x04000000
#define TCR_ARE                 0x00400000

/* L1CSR0/1 bits */
#define L1CSR0_CE               0x00000001
#define not_L1CSR0_CE           ~(L1CSR0_CE)
#define L1CSR0_CFI              0x00000002
#define L1CSR0_CLFR             0x00000100

/* L1CSR1/1 bits */
#define L1CSR1_ICE              0x00000001
#define not_L1CSR1_ICE          ~(L1CSR1_ICE)
#define L1CSR1_ICFI             0x00000002
#define L1CSR1_ICLFR            0x00000100

/* L1CSR2/1 bits */
#define L1CSR2_STASHID          0x000000ff
#define not_L1CSR2_STASHID      ~(L1CSR2_STASHID)

#define L1CSR_UNABLE_TO_LOCK    0x00000400
#define L1CSR_OVER_LOCK         0x00000200
#define L1CSR_ALL_LOCK          0x00000600

/* L2CSR0/1 bits */
#define L2CSR0_L2E              0x80000000      /* cache enable */
#define not_L2CSR0_L2E          ~(L2CSR0_L2E)
#define L2CSR0_L2FI             0x00200000      /* cache flash invalidate */
#define not_L2CSR0_L2FI         ~(L2CSR0_L2FI)
#define L2CSR0_L2IO             0x00100000      /* I-cache only */
#define not_L2CSR0_L2IO         ~(L2CSR0_L2IO)
#define L2CSR0_L2DO             0x00010000      /* D-cache only */
#define not_L2CSR0_L2DO         ~(L2CSR0_L2DO)
#define L2CSR0_L2FL             0x00000800      /* L2 cache flush */
#define L2CSR0_L2LFC            0x00000400      /* L2 cache lock flash clear */
#define not_L2CSR0_L2LFC        ~(L2CSR0_L2LFC) /* L2 cache lock flash clear */

/* L2CSR1/1 bits */
#define L2CSR1_STASHID          0x000000ff
#define not_L2CSR1_STASHID      ~(L2CSR1_STASHID)

/*
 *
 * Cache Definitions
 * ------------------------
 */
#define CACHE_NWAYS         128
#define CACHE_NLINES        12
#define L1_CACHE_LINE_SIZE  32


/*
 *
 * Fields in special registers
 * ---------------------------
 */
/* MSR - Machine State Register */
#define MSR_UCLE        0x4000000
#define MSR_SPE         0x2000000
#define MSR_WE          0x40000
#define MSR_CE          0x20000
#define MSR_CE_BN       14
#define MSR_EE          0x8000
#define MSR_EE_BN       16
#define MSR_PR          0x4000
#define MSR_FP          0x2000
#define MSR_ME          0x1000
#define MSR_ME_BN       23
#define MSR_FE0         0x800
#define MSR_UBLE        0x400
#define MSR_DE          0x200
#define MSR_FE1         0x100
#define MSR_IS          0x20
#define MSR_DS          0x10
#define MSR_PMM         0x4
#define MSR_RI          0x2
#define MSR_LE          0x1
#define not_MSR_EE      0xFFFF7FFF
#define not_MSR_ME      0xFFFFEFFF
#define not_MSR_WE      0xFFFBFFFF
#define not_MSR_CE      0xFFFDFFFF
#define not_MSR_FP      (~MSR_FP)
#define not_MSR_IS      0xFFFFFFDF
#define not_MSR_DS      0xFFFFFFEF
#define not_MSR_RI      0xFFFFFFFB
#define not_MSR_DE      0xFFFFFDFF
#define not_MSR_EECE    0xFFFD7FFF
#define not_MSR_PMM     0xFFFFFFFB
#define not_MSR_PR      ~(MSR_PR)

#define MSR_GS          0x10000000
#define not_MSR_GS      ~(MSR_GS)

/*******************************************************
* interrupt vector
*
* IVORx is loaded with this value
* Also the handler is fixed to this address.
********************************************************/
#define CRITICAL_INTR               0x0100
#define MACHINE_CHECK_INTR          0x0200
#define DATA_STORAGE_INTR           0x0300
#define INSTRUCTION_STORAGE_INTR    0x0400
#define EXTERNAL_INTR               0x0500
#define ALIGNMENT_INTR              0x0600
#define PROGRAM_INTR                0x0700
#define FLOATING_POINT_INTR         0x0800
#define DECREMENTER_INTR            0x0900
#define WATCHDOG_TIMER_INTR         0x0B00
#define SYSTEM_CALL_INTR            0x0C00
#define FIXED_INTERVAL_INTR         0x0F00
#define I_TLB_ERROR_INTR            0x1000
#define D_TLB_ERROR_INTR            0x1100
#define DEBUG_INTR                  0x1500
#define SPE_APU_INTR                0x1600
#define SPE_FLT_DATA_INTR           0x1700
#define SPE_FLT_ROUND_INTR          0x1800
#define PERF_MONITOR_INTR           0x1900
#define AUXILIARY_INTR              0x1A00
/* Next should be +1 from last Interrupt, for table build */
#define LAST_INTR_INDEX             0x1B00


/*
 *
 * Book E Special-Purpose Registers (by SPR Abbreviation)
 */
#define CSRR0       58      /* Critical save/restore register 0 */
#define CSRR1       59      /* Critical save/restore register 1 */
#define CTR         9       /* Count register */
#define DAC1        316     /* Data address compare 1 */
#define DAC2        317     /* Data address compare 2 */
#define DBCR0       308     /* Debug control register 0 1 */
#define DBCR1       309     /* Debug control register 1 1 */
#define DBCR2       310     /* Debug control register 2 1 */
#define DBSR        304     /* Debug status register */
#define DEAR        61      /* Data exception address register */
#define DEC         22      /* Decrementer */
#define DECAR       54      /* Decrementer auto-reload */
#define ESR         62      /* Exception syndrome register */
#define IAC1        312     /* Instruction address compare 1 */
#define IAC2        313     /* Instruction address compare 2 */
#define IVOR0       400     /* Critical input */
#define IVOR1       401     /* Machine check interrupt offset */
#define IVOR2       402     /* Data storage interrupt offset */
#define IVOR3       403     /* Instruction storage interrupt offset */
#define IVOR4       404     /* External input interrupt offset */
#define IVOR5       405     /* Alignment interrupt offset */
#define IVOR6       406     /* Program interrupt offset */
#define IVOR7       407     /* Floating-point unavailable interrupt offset */
#define IVOR8       408     /* System call interrupt offset */
#define IVOR10      410     /* Decrementer interrupt offset */
#define IVOR11      411     /* Fixed-interval timer interrupt offset */
#define IVOR12      412     /* Watchdog timer interrupt offset */
#define IVOR13      413     /* Data TLB error interrupt offset */
#define IVOR14      414     /* Instruction TLB error interrupt offset */
#define IVOR15      415     /* Debug interrupt offset */
#define IVOR32      528     /* SPE APU unavailable interrupt offset */
#define IVOR33      529     /* Floating-point data exception interrupt offset */
#define IVOR34      530     /* Floating-point round exception interrupt offset */
#define IVOR35      531     /* Performance monitor */
#define IVPR        63      /* Interrupt vector */
#define LR          8       /* Link register */
#define PID         48      /* Process ID register 3 */
#define PIR         286     /* Processor ID register */
#define PVR         287     /* Processor version register */
#define SPRG0       272     /* SPR general 0 */
#define SPRG1       273     /* SPR general 1 */
#define SPRG2       274     /* SPR general 2 */
#define SPRG3       275     /* SPR general 3 */
#define SPRG4       276     /* SPR general 4 */
#define SPRG5       277     /* SPR general 5 */
#define SPRG6       278     /* SPR general 6 */
#define SPRG7       279     /* SPR general 7 */
#define SRR0        26      /* Save/restore register 0 */
#define SRR1        27      /* Save/restore register 1 */
#define TBL         284     /* Time base lower */
#define TBU         285     /* Time base upper */
#define TCR         340     /* Timer control register */
#define TSR         336     /* Timer status register */
#define USPRG0      256     /* User SPR general 06 */
#define XER         1       /* Integer exception register */


/*
 *
 * Implementation-Specific SPRs (by SPR Abbreviation)
 *
 */
#define BBEAR       513     /* Branch buffer entry address register 1 */
#define BBTAR       514     /* Branch buffer target address register 1 */
#define BUCSR       1013    /* Branch unit control and status register 1 */
#define HID0        1008    /* Hardware implementation dependent reg 0 1 */
#define HID1        1009    /* Hardware implementation dependent reg 1 1 */
#define L1CFG0      515     /* L1 cache configuration register 0 */
#define L1CFG1      516     /* L1 cache configuration register 1 */
#define L1CSR0      1010    /* L1 cache control and status register 0 */
#define L1CSR1      1011    /* L1 cache control and status register 1 */
#define L1CSR2      606     /* L1 cache control and status register 2 */
#define L2CFG0      519     /* L2 cache configuration register 0 */
#define L2CSR0      1017    /* L2 cache control and status register 0 */
#define L2CSR1      1018    /* L2 cache control and status register 1 */
#define MAS0        624     /* MMU assist register 0 1 */
#define MAS1        625     /* MMU assist register 1 1 */
#define MAS2        626     /* MMU assist register 2 1 */
#define MAS3        627     /* MMU assist register 3 1 */
#define MAS4        628     /* MMU assist register 4 1 */
#define MAS5        339     /* MMU assist register 5 1 */
#define MAS6        630     /* MMU assist register 6 1 */
#define MAS7        944     /* MMU assist register 7 1 */
#define MAS8        341     /* MMU assist register 8 1 */
#define MCAR        573     /* Machine check address register */
#define MCSR        572     /* Machine check syndrome register */
#define MCSRR0      570     /* Machine check save/restore register 0 */
#define MCSRR1      571     /* Machine check save/restore register 1 */
#define MMUCFG      1015    /* MMU configuration register */
#define MMUCSR0     1012    /* MMU control and status register 0 1 */

#define TLB0CFG     688     /* TLB0CFG register 0. */
#define TLB1CFG     689     /* TLB1CFG register 1. */

#define PID0        48      /* Process ID register 0. */
#define PID1        633     /* Process ID register 1 1 */
#define PID2        634     /* Process ID register 2 1 */
#define SPEFSCR     512     /* Signal processing and embedded floating-point status and control register 1 */
#define SVR         1023    /* System version register */
#define ILLEGAL     333     /* For test only */

/*
 * PM Implementation-Specific SPRs (by SPR Abbreviation)
 *
 */
#define PMC0        16      /* Performance monitor counter 0 */
#define PMC1        17      /* Performance monitor counter 1 */
#define PMC2        18      /* Performance monitor counter 2 */
#define PMC3        19      /* Performance monitor counter 3 */
#define PMLCa0      144     /* Performance monitor local control a0 */
#define PMLCa1      145     /* Performance monitor local control a1 */
#define PMLCa2      146     /* Performance monitor local control a2 */
#define PMLCa3      147     /* Performance monitor local control a3 */
#define PMLCb0      272     /* Performance monitor local control b0 */
#define PMLCb1      273     /* Performance monitor local control b1 */
#define PMLCb2      274     /* Performance monitor local control b2 */
#define PMLCb3      274     /* Performance monitor local control b3 */
#define PMGC0       400     /* Performance monitor global control 0 */


#ifndef INCLUDE_FROM_ASM

/* assembly routines prototypes for user reference only */

uint32_t    GetCr(void);
uint32_t    GetSp(void);
uint32_t    GetCurrent(void);

void        SetMsr(uint32_t);
void        SetCr(uint32_t);
void        SetSp(uint32_t);
void        SetCurrent(uint32_t);

/* MSR Manipulations
 *===================
 */
uint32_t    GetMsr(void);
void        MSR_Restore(uint32_t oldMSR);
void        MSR_RestoreEEandCE(uint32_t oldMSR);
void        MSR_EnableEE();
uint32_t    MSR_DisableEE();
void        MSR_EnableME(void);
void        MSR_DisableME(void);
void        MSR_EnableWE(void);
void        MSR_DisableWE(void);
void        MSR_EnableCE(void);
void        MSR_DisableCE(void);
void        MSR_EnableFP(void);
void        MSR_DisableFP(void);
uint32_t    MSR_ClearEE(void);
uint32_t    MSR_ClearEEandCE(void);
void        MSR_EnableDE(void);
void        MSR_DisableDE(void);
void        MSR_EnablePMM(void);
void        MSR_DisablePMM(void);

/* E500MC MSR */
void        MSR_EnableGS(void);
void        MSR_DisableGS(void);
void        MSR_EnablePR(void);
void        MSR_DisablePR(void);

void        E500_TlbWrite(uint32_t mas0, uint32_t mas1,uint32_t mas2,uint32_t mas3);
void        E500_TlbWrite5(uint32_t mas0, uint32_t mas1,uint32_t mas2,uint32_t mas3,uint32_t mas7);
void        E500MC_TlbWrite(uint32_t mas[]);
void        E500_TlbRead(uint32_t *arrayAddr, uint32_t mas0, uint32_t mas2);
void        E500_TlbRead5(uint32_t *arrayAddr, uint32_t mas0, uint32_t mas2);
void        E500MC_TlbRead(uint32_t *arrayAddr, uint32_t mas0, uint32_t mas2);
void        E500_Tlb1Invalidate(void);


uint32_t    L1CtlWrite(uint32_t address, uint32_t value);


uint32_t    GetDec(void);
uint32_t    GetTcr(void);
uint32_t    GetHid0(void);
uint32_t    GetCpuId(void);

/************************************************************/
/*  The following routines are inter-modules and are not    */
/*  designed for direct application use. They are used by   */
/*  other modules in the drivers.                           */
/************************************************************/
uint32_t  GetSrr0(void);
uint32_t  GetSrr1(void);
uint32_t  GetLr(void);


/*
 *
 * Book E Special-Purpose Registers (by SPR Abbreviation)
 */
uint32_t E500_GetSPR_CSRR0(void);           /* [58]         Critical save/restore register 0 */
void E500_SetSPR_CSRR0(uint32_t newvalue);  /* [58]         Critical save/restore register 0 */
uint32_t E500_GetSPR_CSRR1(void);           /* [59]         Critical save/restore register 1 */
void E500_SetSPR_CSRR1(uint32_t newvalue);  /* [59]         Critical save/restore register 1 */
uint32_t E500_GetSPR_CTR(void);             /* [9]          Count register */
void E500_SetSPR_CTR(uint32_t newvalue);    /* [9]          Count register */
uint32_t E500_GetSPR_DAC1(void);            /* [316]        Data address compare 1 */
void E500_SetSPR_DAC1(uint32_t newvalue);   /* [316]        Data address compare 1 */
uint32_t E500_GetSPR_DAC2(void);            /* [317]        Data address compare 2 */
void E500_SetSPR_DAC2(uint32_t newvalue);   /* [317]        Data address compare 2 */
uint32_t E500_GetSPR_DBCR0(void);           /* [308]        Debug control register 0 1 */
void E500_SetSPR_DBCR0(uint32_t newvalue);  /* [308]        Debug control register 0 1 */
uint32_t E500_GetSPR_DBCR1(void);           /* [309]        Debug control register 1 1 */
void E500_SetSPR_DBCR1(uint32_t newvalue);  /* [309]        Debug control register 1 1 */
uint32_t E500_GetSPR_DBCR2(void);           /* [310]        Debug control register 2 1 */
void E500_SetSPR_DBCR2(uint32_t newvalue);  /* [310]        Debug control register 2 1 */
uint32_t E500_GetSPR_DBSR(void);            /* [304]        Debug status register */
void E500_SetSPR_DBSR(uint32_t newvalue);   /* [304]        Debug status register */
uint32_t E500_GetSPR_DEAR(void);            /* [61]         Data exception address register */
void E500_SetSPR_DEAR(uint32_t newvalue);   /* [61]         Data exception address register */
uint32_t E500_GetSPR_DEC(void);             /* [22]         Decrementer */
void E500_SetSPR_DEC(uint32_t newvalue);    /* [22]         Decrementer */
uint32_t E500_GetSPR_DECAR(void);           /* [54]         Decrementer auto-reload */
void E500_SetSPR_DECAR(uint32_t newvalue);  /* [54]         Decrementer auto-reload */
uint32_t E500_GetSPR_ESR(void);             /* [62]         Exception syndrome register */
void E500_SetSPR_ESR(uint32_t newvalue);    /* [62]         Exception syndrome register */
uint32_t E500_GetSPR_IAC1(void);            /* [312]        Instruction address compare 1 */
void E500_SetSPR_IAC1(uint32_t newvalue);   /* [312]        Instruction address compare 1 */
uint32_t E500_GetSPR_IAC2(void);            /* [313]        Instruction address compare 2 */
void E500_SetSPR_IAC2(uint32_t newvalue);   /* [313]        Instruction address compare 2 */
uint32_t E500_GetSPR_IVOR0(void);           /* [400]        Critical input */
void E500_SetSPR_IVOR0(uint32_t newvalue);  /* [400]        Critical input */
uint32_t E500_GetSPR_IVOR1(void);           /* [401]        Machine check interrupt offset */
void E500_SetSPR_IVOR1(uint32_t newvalue);  /* [401]        Machine check interrupt offset */
uint32_t E500_GetSPR_IVOR2(void);           /* [402]        Data storage interrupt offset */
void E500_SetSPR_IVOR2(uint32_t newvalue);  /* [402]        Data storage interrupt offset */
uint32_t E500_GetSPR_IVOR3(void);           /* [403]        Instruction storage interrupt offset */
void E500_SetSPR_IVOR3(uint32_t newvalue);  /* [403]        Instruction storage interrupt offset */
uint32_t E500_GetSPR_IVOR4(void);           /* [404]        External input interrupt offset */
void E500_SetSPR_IVOR4(uint32_t newvalue);  /* [404]        External input interrupt offset */
uint32_t E500_GetSPR_IVOR5(void);           /* [405]        Alignment interrupt offset */
void E500_SetSPR_IVOR5(uint32_t newvalue);  /* [405]        Alignment interrupt offset */
uint32_t E500_GetSPR_IVOR6(void);           /* [406]        Program interrupt offset */
void E500_SetSPR_IVOR6(uint32_t newvalue);  /* [406]        Program interrupt offset */
uint32_t E500_GetSPR_IVOR8(void);           /* [408]        System call interrupt offset */
void E500_SetSPR_IVOR8(uint32_t newvalue);  /* [408]        System call interrupt offset */
uint32_t E500_GetSPR_IVOR10(void);          /* [410]        Decrementer interrupt offset */
void E500_SetSPR_IVOR10(uint32_t newvalue); /* [410]        Decrementer interrupt offset */
uint32_t E500_GetSPR_IVOR11(void);          /* [411]        Fixed-interval timer interrupt offset */
void E500_SetSPR_IVOR11(uint32_t newvalue); /* [411]        Fixed-interval timer interrupt offset */
uint32_t E500_GetSPR_IVOR12(void);          /* [412]        Watchdog timer interrupt offset */
void E500_SetSPR_IVOR12(uint32_t newvalue); /* [412]        Watchdog timer interrupt offset */
uint32_t E500_GetSPR_IVOR13(void);          /* [413]        Data TLB error interrupt offset */
void E500_SetSPR_IVOR13(uint32_t newvalue); /* [413]        Data TLB error interrupt offset */
uint32_t E500_GetSPR_IVOR14(void);          /* [414]        Instruction TLB error interrupt offset */
void E500_SetSPR_IVOR14(uint32_t newvalue); /* [414]        Instruction TLB error interrupt offset */
uint32_t E500_GetSPR_IVOR15(void);          /* [415]        Debug interrupt offset */
void E500_SetSPR_IVOR15(uint32_t newvalue); /* [415]        Debug interrupt offset */
uint32_t E500_GetSPR_IVPR(void);            /* [63]         Interrupt vector */
void E500_SetSPR_IVPR(uint32_t newvalue);   /* [63]         Interrupt vector */
uint32_t E500_GetSPR_LR(void);              /* [8]          Link register */
void E500_SetSPR_LR(uint32_t newvalue);     /* [8]          Link register */
uint32_t E500_GetSPR_PID(void);             /* [48]         Process ID register 3 */
void E500_SetSPR_PID(uint32_t newvalue);    /* [48]         Process ID register 3 */
uint32_t E500_GetSPR_PIR(void);             /* [286]        Processor ID register */
void E500_SetSPR_PIR(uint32_t newvalue);    /* [286]        Processor ID register */
uint32_t E500_GetSPR_PVR(void);             /* [287]        Processor version register */
void E500_SetSPR_PVR(uint32_t newvalue);    /* [287]        Processor version register */
uint32_t E500_GetSPR_SPRG0(void);           /* [272]        SPR general 0 */
void E500_SetSPR_SPRG0(uint32_t newvalue);  /* [272]        SPR general 0 */
uint32_t E500_GetSPR_SPRG1(void);           /* [273]        SPR general 1 */
void E500_SetSPR_SPRG1(uint32_t newvalue);  /* [273]        SPR general 1 */
uint32_t E500_GetSPR_SPRG2(void);           /* [274]        SPR general 2 */
void E500_SetSPR_SPRG2(uint32_t newvalue);  /* [274]        SPR general 2 */
uint32_t E500_GetSPR_SPRG3(void);           /* [275]        SPR general 3 */
void E500_SetSPR_SPRG3(uint32_t newvalue);  /* [275]        SPR general 3 */
uint32_t E500_GetSPR_SPRG4(void);           /* [276]        SPR general 4 */
void E500_SetSPR_SPRG4(uint32_t newvalue);  /* [276]        SPR general 4 */
uint32_t E500_GetSPR_SPRG5(void);           /* [277]        SPR general 5 */
void E500_SetSPR_SPRG5(uint32_t newvalue);  /* [277]        SPR general 5 */
uint32_t E500_GetSPR_SPRG6(void);           /* [278]        SPR general 6 */
void E500_SetSPR_SPRG6(uint32_t newvalue);  /* [278]        SPR general 6 */
uint32_t E500_GetSPR_SPRG7(void);           /* [279]        SPR general 7 */
void E500_SetSPR_SPRG7(uint32_t newvalue);  /* [279]        SPR general 7 */
uint32_t E500_GetSPR_SRR0(void);            /* [26]         Save/restore register 0 */
void E500_SetSPR_SRR0(uint32_t newvalue);   /* [26]         Save/restore register 0 */
uint32_t E500_GetSPR_SRR1(void);            /* [27]         Save/restore register 1 */
void E500_SetSPR_SRR1(uint32_t newvalue);   /* [27]         Save/restore register 1 */
uint32_t E500_GetSPR_TBL(void);             /* [284]        Time base lower */
void E500_SetSPR_TBL(uint32_t newvalue);    /* [284]        Time base lower */
uint32_t E500_GetSPR_TBU(void);             /* [285]        Time base upper */
void E500_SetSPR_TBU(uint32_t newvalue);    /* [285]        Time base upper */
uint32_t E500_GetSPR_TCR(void);             /* [340]        Timer control register */
void E500_SetSPR_TCR(uint32_t newvalue);    /* [340]        Timer control register */
uint32_t E500_GetSPR_TSR(void);             /* [336]        Timer status register */
void E500_SetSPR_TSR(uint32_t newvalue);    /* [336]        Timer status register */
uint32_t E500_GetSPR_USPRG0(void);          /* [256]        User SPR general 06 */
void E500_SetSPR_USPRG0(uint32_t newvalue); /* [256]        User SPR general 06 */
uint32_t E500_GetSPR_XER(void);             /* [1]          Integer exception register */
void E500_SetSPR_XER(uint32_t newvalue);    /* [1]          Integer exception register */


/*
*
* Implementation-Specific SPRs (by SPR Abbreviation)
*
*/

uint32_t E500_GetSPR_BBEAR(void);           /* [513]        Branch buffer entry address register 1 */
void E500_SetSPR_BBEAR(uint32_t newvalue);  /* [513]        Branch buffer entry address register 1 */
uint32_t E500_GetSPR_BBTAR(void);           /* [514]        Branch buffer target address register 1 */
void E500_SetSPR_BBTAR(uint32_t newvalue);  /* [514]        Branch buffer target address register 1 */
uint32_t E500_GetSPR_BUCSR(void);           /* [1013]       Branch unit control and status register 1 */
void E500_SetSPR_BUCSR(uint32_t newvalue);  /* [1013]       Branch unit control and status register 1 */
uint32_t E500_GetSPR_HID0(void);            /* [1008]       Hardware implementation dependent reg 0 1 */
void E500_SetSPR_HID0(uint32_t newvalue);   /* [1008]       Hardware implementation dependent reg 0 1 */
uint32_t E500_GetSPR_HID1(void);            /* [1009]       Hardware implementation dependent reg 1 1 */
void E500_SetSPR_HID1(uint32_t newvalue);   /* [1009]       Hardware implementation dependent reg 1 1 */
uint32_t E500_GetSPR_IVOR32(void);          /* [528]        SPE APU unavailable interrupt offset */
void E500_SetSPR_IVOR32(uint32_t newvalue); /* [528]        SPE APU unavailable interrupt offset */
uint32_t E500_GetSPR_IVOR33(void);          /* [529]        Floating-point data exception interrupt offset */
void E500_SetSPR_IVOR33(uint32_t newvalue); /* [529]        Floating-point data exception interrupt offset */
uint32_t E500_GetSPR_IVOR34(void);          /* [530]        Floating-point round exception interrupt offset */
void E500_SetSPR_IVOR34(uint32_t newvalue); /* [530]        Floating-point round exception interrupt offset */
uint32_t E500_GetSPR_IVOR35(void);          /* [531]        Performance monitor */
void E500_SetSPR_IVOR35(uint32_t newvalue); /* [531]        Performance monitor */
uint32_t E500_GetSPR_L1CFG0(void);          /* [515]        L1 cache configuration register 0 */
void E500_SetSPR_L1CFG0(uint32_t newvalue); /* [515]        L1 cache configuration register 0 */
uint32_t E500_GetSPR_L1CFG1(void);          /* [516]        L1 cache configuration register 1 */
void E500_SetSPR_L1CFG1(uint32_t newvalue); /* [516]        L1 cache configuration register 1 */
uint32_t E500_GetSPR_L1CSR0(void);          /* [1010]       L1 cache control and status register 0 1 */
void E500_SetSPR_L1CSR0(uint32_t newvalue); /* [1010]       L1 cache control and status register 0 1 */
uint32_t E500_GetSPR_L1CSR1(void);          /* [1011]       L1 cache control and status register 1 1 */
void E500_SetSPR_L1CSR1(uint32_t newvalue); /* [1011]       L1 cache control and status register 1 1 */
uint32_t E500_GetSPR_L1CSR2(void);          /* [606]        L1 cache control and status register 2 1 */
void E500_SetSPR_L1CSR2(uint32_t newvalue); /* [606]        L1 cache control and status register 2 1 */

uint32_t E500MC_GetSPR_L2CFG0(void);          /* [519]        L2 cache configuration register 0 */
void E500MC_SetSPR_L2CFG0(uint32_t newvalue); /* [519]        L2 cache configuration register 0 */
uint32_t E500MC_GetSPR_L2CSR0(void);          /* [1017]       L2 cache control and status register 0 1 */
void E500MC_SetSPR_L2CSR0(uint32_t newvalue); /* [1017]       L2 cache control and status register 0 1 */
uint32_t E500MC_GetSPR_L2CSR1(void);          /* [1018]       L2 cache control and status register 1 1 */
void E500MC_SetSPR_L2CSR1(uint32_t newvalue); /* [1018]       L2 cache control and status register 1 1 */

uint32_t E500_GetSPR_MAS0(void);            /* [624]        MMU assist register 0 1 */
void E500_SetSPR_MAS0(uint32_t newvalue);   /* [624]        MMU assist register 0 1 */
uint32_t E500_GetSPR_MAS1(void);            /* [625]        MMU assist register 1 1 */
void E500_SetSPR_MAS1(uint32_t newvalue);   /* [625]        MMU assist register 1 1 */
uint32_t E500_GetSPR_MAS2(void);            /* [626]        MMU assist register 2 1 */
void E500_SetSPR_MAS2(uint32_t newvalue);   /* [626]        MMU assist register 2 1 */
uint32_t E500_GetSPR_MAS3(void);            /* [627]        MMU assist register 3 1 */
void E500_SetSPR_MAS3(uint32_t newvalue);   /* [627]        MMU assist register 3 1 */
uint32_t E500_GetSPR_MAS4(void);            /* [628]        MMU assist register 4 1 */
void E500_SetSPR_MAS4(uint32_t newvalue);   /* [628]        MMU assist register 4 1 */
uint32_t E500_GetSPR_MAS6(void);            /* [630]        MMU assist register 6 1 */
void E500_SetSPR_MAS6(uint32_t newvalue);   /* [630]        MMU assist register 6 1 */
uint32_t E500_GetSPR_MAS7(void);            /* [944]        MMU assist register 7 1 */
void E500_SetSPR_MAS7(uint32_t newvalue);   /* [944]        MMU assist register 7 1 */
uint32_t E500_GetSPR_MCAR(void);            /* [573]        Machine check address register */
void E500_SetSPR_MCAR(uint32_t newvalue);   /* [573]        Machine check address register */
uint32_t E500_GetSPR_MCSR(void);            /* [572]        Machine check syndrome register */
void E500_SetSPR_MCSR(uint32_t newvalue);   /* [572]        Machine check syndrome register */
uint32_t E500_GetSPR_MCSRR0(void);          /* [570]        Machine check save/restore register 0 */
void E500_SetSPR_MCSRR0(uint32_t newvalue); /* [570]        Machine check save/restore register 0 */
uint32_t E500_GetSPR_MCSRR1(void);          /* [571]        Machine check save/restore register 1 */
void E500_SetSPR_MCSRR1(uint32_t newvalue); /* [571]        Machine check save/restore register 1 */
uint32_t E500_GetSPR_MMUCFG(void);          /* [1015]       MMU configuration register */
void E500_SetSPR_MMUCFG(uint32_t newvalue); /* [1015]       MMU configuration register */
uint32_t E500_GetSPR_MMUCSR0(void);         /* [1012]       MMU control and status register 0 1 */
void E500_SetSPR_MMUCSR0(uint32_t newvalue);/* [1012]       MMU control and status register 0 1 */

uint32_t E500_GetSPR_TLB0CFG(void);         /* [688]        TLB0CFG register 0. */
uint32_t E500_GetSPR_TLB1CFG(void);         /* [689]        TLB1CFG register 0. */


uint32_t E500_GetSPR_PID0(void);            /* [48]         Process ID register 0. */
void E500_SetSPR_PID0(uint32_t newvalue);   /* [48]         Process ID register 0. */
uint32_t E500_GetSPR_PID1(void);            /* [633]        Process ID register 1 1 */
void E500_SetSPR_PID1(uint32_t newvalue);   /* [633]        Process ID register 1 1 */
uint32_t E500_GetSPR_PID2(void);            /* [634]        Process ID register 2 1 */
void E500_SetSPR_PID2(uint32_t newvalue);   /* [634]        Process ID register 2 1 */
uint32_t E500_GetSPR_SPEFSCR(void);         /* [512]        Signal processing and embedded floating-point status and control register 1 */
void E500_SetSPR_SPEFSCR(uint32_t newvalue);/* [512]        Signal processing and embedded floating-point status and control register 1 */
uint32_t E500_GetSPR_SVR(void);             /* [1023]       System version register */
void E500_SetSPR_SVR(uint32_t newvalue);    /* [1023]       System version register */
uint32_t E500_GetSPR_ILLEGAL(void);         /* [999]        System version register */
void E500_SetSPR_ILLEGAL(uint32_t newvalue);/* [999]        System version register */
void E500_ILLEGAL_Inst(void);


/* Performance Monitoring */
uint32_t E500_GetPMC0(void);
void E500_SetPMC0(uint32_t newvalue);
uint32_t E500_GetPMC1(void);
void E500_SetPMC1(uint32_t newvalue);
uint32_t E500_GetPMC2(void);
void E500_SetPMC2(uint32_t newvalue);
uint32_t E500_GetPMC3(void);
void E500_SetPMC3(uint32_t newvalue);
uint32_t E500_GetPMLCa0(void);
void E500_SetPMLCa0(uint32_t newvalue);
uint32_t E500_GetPMLCa1(void);
void E500_SetPMLCa1(uint32_t newvalue);
uint32_t E500_GetPMLCa2(void);
void E500_SetPMLCa2(uint32_t newvalue);
uint32_t E500_GetPMLCa3(void);
void E500_SetPMLCa3(uint32_t newvalue);
uint32_t E500_GetPMLCb0(void);
void E500_SetPMLCb0(uint32_t newvalue);
uint32_t E500_GetPMLCb1(void);
void E500_SetPMLCb1(uint32_t newvalue);
uint32_t E500_GetPMLCb2(void);
void E500_SetPMLCb2(uint32_t newvalue);
uint32_t E500_GetPMLCb3(void);
void E500_SetPMLCb3(uint32_t newvalue);
uint32_t E500_GetPMGC0 (void);
void E500_SetPMGC0 (uint32_t newvalue);


#endif /* INCLUDE_FROM_ASM */
#endif /* __E500v2_ASM_EXT_H */
