/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          e500v2_mmu_ext.h

 @Description   MMU Application Programming Interface
*//***************************************************************************/
#ifndef __E500V2_MMU_EXT_H
#define __E500V2_MMU_EXT_H

/**************************************************************************//**
 @Group         e500v2_mmu_id MMU E500 V2 Application Programming Interface

 @Description   MMU E500 V2 MMU controller functions,definitions and enums.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         e500v2_mmu_struct MMU E500 V2 enums and data structures

 @Description   MMU E500 V2 MMU controller structures, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   TLB Select
*//***************************************************************************/
typedef enum e_MmuTlbSel
{
    e_MMU_MAS0_TLBSEL_TLB0      = 0x00000000,   /**< TLB 0 select*/
    e_MMU_MAS0_TLBSEL_TLB1      = 0x10000000    /**< TLB 1 select*/
} e_MmuTlbSel;

/**************************************************************************//**
 @Description   TLB Size Select
*//***************************************************************************/
/* The next enum is also physical definition! */
typedef enum e_MmuPageSize
{
    e_MMU_MAS1_TLB1_TSIZE_WRONG = 0x00000000,     /**< Wrong Memory page */
    e_MMU_MAS1_TLB1_TSIZE_4K    = 0x00000100,     /**< 4K Memory page       */
    e_MMU_MAS1_TLB1_TSIZE_16K   = 0x00000200,     /**< 16K Memory page */
    e_MMU_MAS1_TLB1_TSIZE_64K   = 0x00000300,     /**< 64K Memory page */
    e_MMU_MAS1_TLB1_TSIZE_256K  = 0x00000400,     /**< 256K Memory page */
    e_MMU_MAS1_TLB1_TSIZE_1M    = 0x00000500,     /**< 1M Memory page */
    e_MMU_MAS1_TLB1_TSIZE_4M    = 0x00000600,     /**< 4M Memory page */
    e_MMU_MAS1_TLB1_TSIZE_16M   = 0x00000700,     /**< 16M Memory page */
    e_MMU_MAS1_TLB1_TSIZE_64M   = 0x00000800,     /**< 64M Memory page */
    e_MMU_MAS1_TLB1_TSIZE_256M  = 0x00000900,     /**< 256M Memory page */
    e_MMU_MAS1_TLB1_TSIZE_1G    = 0x00000A00,     /**< 1GB Memory page */
    e_MMU_MAS1_TLB1_TSIZE_4G    = 0x00000B00      /**< 4GB Memory page */
} e_MmuPageSize;

/**************************************************************************//**
 @Description   Page Endianess Select
*//***************************************************************************/
/* The next enum is also physical definition! */
typedef enum e_MmuEndianess
{
    e_MMU_MAS2_LITTLE_ENDIAN    = 0x00000001,      /**< Little endian page */
    e_MMU_MAS2_BIG_ENDIAN       = 0x00000000       /**< Big endian page */
} e_MmuEndianess;

/**************************************************************************//**
 @Description   Page Permissions Select
*//***************************************************************************/
/* The next enum is also physical definition! */
typedef enum e_MmuPermission
{
    e_MMU_MAS3_USER_X       = 0x00000020,   /**< Enable user mode for execution  */
    e_MMU_MAS3_USER_W       = 0x00000008,   /**< Enable user mode for writing  */
    e_MMU_MAS3_USER_R       = 0x00000002,   /**< Enable user mode for reading  */
    e_MMU_MAS3_SUPERVISOR_X = 0x00000010,   /**< Enable superviser mode for execution  */
    e_MMU_MAS3_SUPERVISOR_W = 0x00000004,   /**< Enable superviser mode for writing  */
    e_MMU_MAS3_SUPERVISOR_R = 0x00000001    /**< Enable superviser mode for reading  */
} e_MmuPermission;

/**************************************************************************//**
 @Description   TLB Entry Parameters structure
*//***************************************************************************/
typedef struct t_MmuTlbEntryParam
{
    e_MmuTlbSel     tlbSel;             /**< TLB select                                 */
    uint32_t        entrySel;           /**< Entry select. 0-511 on TLB0, 0-15 on TLB1. */
    bool            invalidateProtect;  /**<  TLB1 only. TRUE to protect this TLB entry
                                              from invalidation by tlbvivax             */
    uint8_t         transId;            /**<  Transaction identity (0-255).             */
    e_MmuPageSize   pageSize;           /**<  TLB1 only. Translation page size.         */
    uint64_t        virtualBaseAddr;    /**<  Virtual/effective beginning address of page.*/
    uint64_t        physicalBaseAddr;   /**<  Physical/Real beginning address of this page.*/
    bool            x0;                 /**<  TRUE to set X0 user define bit */
    bool            x1;                 /**<  TRUE to set X1 user define bit */
    bool            writeThrough;       /**<  Write-through:
                                        FALSE: This page is considered write-back with respect to the caches in the system.
                                        TRUE: All stores performed to this page are written through the caches to main memory. */
    bool            cacheInhibited;     /**< Caching-inhibited:
                                            FALSE:  Accesses to this page are considered cacheable.
                                            TRUE: The page is considered caching-inhibited. */
    bool            memCoherencyReq;    /**< TRUE when "M" of "WIMGE" is required .TRUE memory coherency    */
    bool            guarded;            /**< TRUE when "G" of "WIMGE". this page isguarded. */
    bool            tlbValid;           /**< True=valid False=Invalidate */
    e_MmuEndianess  endianess;          /**< "E" of "WIMGE".                                */
    uint32_t        permission;         /**< Any OR combination of e_MmuPermission.     */
} t_MmuTlbEntryParam;

/**************************************************************************//**
 @Description   TLB Read structure
*//***************************************************************************/
typedef struct t_MmuTlbEntry
{
    bool        nextVictim;
    bool        invalidateProtect;  /**< TLB1 only. TRUE to protect this TLB entry
                                              from invalidation by tlbvivax             */
    uint8_t     transId;            /**< Transaction identity (0-255).              */
    e_MmuPageSize   pageSize;           /**< TLB1 only. Translation page size.          */
    uint64_t        virtualBaseAddr;    /**<  Virtual/effective beginning address of page.*/
    uint64_t        physicalBaseAddr;   /**<  Physical/Real beginning address of this page.*/
    bool            x0;                 /**<  TRUE to set X0 user define bit */
    bool            x1;                 /**<  TRUE to set X1 user define bit */
    bool        shareCache;         /**< TRUE to enable cache fills to use share
                                         cache state for this page.                 */
    bool        writeThrough;      /**<  Write-through:
                                        FALSE: This page is considered write-back with respect to the caches in the system.
                                        TRUE: All stores performed to this page are written through the caches to main memory. */
    bool        cacheEnable;        /**< Caching-inhibited:
                                            FALSE:  Accesses to this page are considered cacheable.
                                            TRUE: The page is considered caching-inhibited. */
    bool        memCoherencyReq;    /**< TRUE when "M" of "WIMGE" is required .TRUE memory coherency    */
    bool        guarded;            /**< TRUE when "G" of "WIMGE". this page isguarded. */
    e_MmuEndianess  endianess;      /**< "E" of "WIMGE".                                */
    e_MmuPermission permission;     /**< Any OR combination of e_MmuPermission.     */
    bool    transSpace;
} t_MmuTlbEntry;


/**************************************************************************//**
 @Description   TLB Identify structure
*//***************************************************************************/
typedef struct t_MmuIdent
{
    uint8_t     nPids;          /**< Number of PID registers */
    uint8_t     pidSize;        /**< The 5-bit value of PIDSIZE */
    uint8_t     nTlbs;          /**< Number of TLBs */
    uint8_t     mAvn;           /**< MMU architecture version number */
/* Tlb0Cfg */
    uint8_t     tlb0aSsoc;      /**< Associativity of TLB0 */
    uint16_t    tlb0nEntry;     /**< Number of entries in TLB0 */
    uint8_t     tlb0minSize;    /**< Minimum page size of TLB0 */
    uint8_t     tlb0maxSize;    /**< Maximum page size of TLB0 */
/* Tlb1Cfg */
    uint8_t     tlb1aSsoc;      /**< Associativity of TLB1 */
    uint16_t    tlb1nEntry;     /**< Number of entries in TLB1 */
    uint8_t     tlb1minSize;    /**< MINSIZE Minimum page size of TLB1 */
    uint8_t     tlb1maxSize;    /**< Maximum page size of TLB1 */
} t_MmuIdent;

/** @} */ /* end of e500v2_mmu_struct group */
/**************************************************************************//**
 @Group         e500v2_mmu_api MMU E500 V2 Application Programming Interface

 @Description   MMU E500 V2 MMU controller functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function          MMU_TlbEntryWrite

 @Description       The routine defines and enable an MMU TLB entry.

 @Param[in]         p_MmuTlbEntryParam -  Parameters for page initialization

 @Return            E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MMU_TlbEntryWrite(t_MmuTlbEntryParam *p_MmuTlbEntryParam);

/**************************************************************************//**
 @Function          MMU_Tlb1EntryRead

 @Description       The routine Read an MMU TLB entry.

 @Param[in]         entrySel        - MMU TLB1 Entry to read
 @Param[in]         virtualAddr     - Address of  TLB1 Entry
 @Param[out]        p_MmuTlbEntry   - TLB1 Parameters out from Read

 @Return            E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MMU_Tlb1EntryRead(uint8_t       entrySel,
                          uint32_t      virtualAddr,
                          t_MmuTlbEntry *p_MmuTlbEntry);

/**************************************************************************//**
 @Function          MMU_Identify

 @Description       The routine Read an MMU TLB entry.

 @Param[out]        p_MmuIdent - Structure with MMU information.

 @Return            None.
*//***************************************************************************/
void MMU_Identify(t_MmuIdent *p_MmuIdent);

/**************************************************************************//**
 @Function          MMU_Tlb1EntryPrint

 @Description       The routine Print all TLB1 entris.

 @Return            None.
*//***************************************************************************/
void MMU_Tlb1EntryPrint(void);

/**************************************************************************//**
 @Function          MMU_Tlb1ClearAll

 @Description       Clear all TLB1's excpete the 2 protected Entrys.

 @Return            None.
*//***************************************************************************/
void MMU_Tlb1ClearAll(void);

void MMU_Tlb1ClearAllExceptRunning(uint8_t protect1, uint8_t protect2);

/**************************************************************************//**
 @Function          MMU_Tlb1FindTlb

 @Description       Find TLB corresponding to start address and size.

 @Param[in]         startAddr   - Start Address (virtual).
 @Param[in]         size        - Page Size.

 @Return            TLB Entry number. 0xFF if not found.
*//***************************************************************************/
uint8_t MMU_Tlb1FindTlb(uint64_t startAddr, uint64_t size);

/**************************************************************************//**
 @Function          MMU_Tlb1ProtectTlb

 @Description       Protect a TLB Entry By re Writing it with TLBprotect ON.

 @Param[in]         entry - TLB Entry number.

 @Return            None.
*//***************************************************************************/
void MMU_Tlb1ProtectTlb(uint8_t entry);

/**************************************************************************//**
 @Function          MMU_Tlb1Size2Page

 @Description       The routine Convert 64 bit real address into a TLB1 MMU definition

 @Param[in]         pageSize - Real page size.

 @Return            e_MmuPageSize TLB1  Page Size. or e_MMU_MAS1_TLB1_TSIZE_WRONG for Error code.
*//***************************************************************************/
e_MmuPageSize MMU_Tlb1Size2Page(uint64_t pageSize);

/**************************************************************************//**
 @Function          MMU_Tlb1FindUnusedTlb

 @Description       Find first non-initialized TLB.

 @Return            TLB Entry number; 0xFF if not found.
*//***************************************************************************/
uint8_t MMU_Tlb1FindUnusedTlb(void);

/** @} */ /* end of e500v2_mmu_id group */



/** @} */ /* end of e500v2_mmu_api group */


#endif /* __E500V2_MMU_EXT_H */
