/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          e500v2_pm_ext.h

 @Description   E500 Performance Monitor Application Programming Interface
*//***************************************************************************/
#ifndef __E500V2_PM_EXT_H
#define __E500V2_PM_EXT_H


/**************************************************************************//**
 @Group         e500v2_performance_id E500 Performance Monitor Application Programming Interface

 @Description   E500 Performance Monitor functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         e500v2_performance_api E500 Performance Monitor Control Unit

 @Description   E500 Performance Monitor runtime control routines and defintions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Types of performance monitoring events.
*//***************************************************************************/
typedef enum e_E500PmEvent
{
      e_E500_PM_NOTHING                              = 0 /**< Nonspec Register counter holds current value */
    , e_E500_PM_PROCESSOR_CYCLES                     = 1 /**< Nonspec Every processor cycle */
    , e_E500_PM_INSTRUCTIONS_COMPLETED               = 2 /**< Nonspec Completed instructions. 0, 1, or 2 per cycle. */
    , e_E500_PM_MICRO_OPS_COMPLETED                  = 3 /**< Nonspec Completed micro-ops. 0, 1, or 2 per cycle. (1 for each standard
                                                              instruction, 2 for load/store-with-update. 1-32 for load or store multiple instructions) */
    , e_E500_PM_INSTRUCTIONS_FETCHED                 = 4 /**< Spec Fetched instructions. 0, 1, 2, 3, or 4 per cycle. (instructions written to the IQ.) */
    , e_E500_PM_MICRO_OPS_DECODED                    = 5 /**< Spec Micro-ops decoded. 0, 1, or 2 per cycle. (2 for load/store-with-update) */
    , e_E500_PM_PM_EVENT_TRANSITIONS                 = 6 /**< Spec 0 to 1 transitions on the pm_event input. */
    , e_E500_PM_PM_EVENT_CYCLES                      = 7 /**< Spec Processor cycles that occur when the pm_event input is asserted. */

    , e_E500_PM_BRANCH_INSTRUCTIONS_COMPLETED        = 8 /**< Nonspec Completed branch instructions. */
    , e_E500_PM_LOAD_MICRO_OPS_COMPLETED             = 9 /**< Nonspec Completed load micro-ops. (l*, evl*, load-update (1 load micro-op),
                                                              load-multiple (1-32 micro-ops), dcbt(L1, CT = 0), and dcbtst(L1, CT= 0) */
    , e_E500_PM_STORE_MICRO_OPS_COMPLETED            = 10 /**< Nonspec Completed store micro-ops. (st*, evst*, store-update (1 store
                                                              micro-op), store-multiple (1-32 micro-ops), tlbivax, icbi, icblc,icbtls, dcba, dcbf,
                                                              dcblc, dcbst, dcbt(L2, CT = 1), dcbtls, dcbtst(L2, CT = 1), dcbtstls, dcbz, icbt(L2,
                                                              CT = 1), mbar, and msync) */
    , e_E500_PM_NUMBER_OF_CQ_REDIRECTS               = 11 /**< Nonspec Fetch redirects initiated from the completion unit. (for example,
                                                               resulting from sc, rfi, rfci, rfmci, isync, and interrupts) */

    , e_E500_PM_BRANCHES_FINISHED                    = 12 /**< Spec Includes all branch instructions */
    , e_E500_PM_TAKEN_BRANCHES_FINISHED              = 13 /**< Spec Includes all taken branch instructions */
    , e_E500_PM_FINISHED_UNC_BRANCHES_T_M_BTB        = 14 /**< Spec Includes all taken branch instructions not allocated in the BTB */
    , e_E500_PM_BRANCHES_MISPREDICTED                = 15 /**< Spec Counts branch instructions mispredicted due to direction, target
                                                               (for example if the CTR contents change), or IAB prediction. Does not
                                                               count instructions that the branch predictor incorrectly predicted to be branches. */
    , e_E500_PM_BRANCHES_IN_BTB_MISPREDICTED         = 16 /**< Spec Counts branch instructions mispredicted due to direction prediction. */
    , e_E500_PM_BTB_HITS_AND_PSEUDO_HITS             = 17 /**< Spec Branch instructions that hit in the BTB or miss in the BTB and are
                                                               not-taken (a pseudo-hit). Characterizes upper bound on prediction rate. */
    , e_E500_PM_CYCLES_DECODE_STALLED                = 18 /**< Spec Cycles the IQ is not empty but 0 instructions decoded */
    , e_E500_PM_CYCLES_ISSUE_STALLED                 = 19 /**< Spec Cycles the issue buffer is not empty but 0 instructions issued */
    , e_E500_PM_CYCLES_BRANCH_ISSUE_STALLED          = 20 /**< Spec Cycles the branch buffer is not empty but 0 instructions issued */
    , e_E500_PM_CYCLES_SU1_SCHEDULE_STALLED          = 21 /**< Spec Cycles SU1 is not empty but 0 instructions scheduled */
    , e_E500_PM_CYCLES_SU2_SCHEDULE_STALLED          = 22 /**< Spec Cycles SU2 is not empty but 0 instructions scheduled */
    , e_E500_PM_CYCLES_MU_SCHEDULE_STALLED           = 23 /**< Spec Cycles MU is not empty but 0 instructions scheduled */
    , e_E500_PM_CYCLES_LRU_SCHEDULE_STALLED          = 24 /**< Spec Cycles LRU is not empty but 0 instructions scheduled */
    , e_E500_PM_CYCLES_BU_SCHEDULE_STALLED           = 25 /**< Spec Cycles BU is not empty but 0 instructions scheduled */

    , e_E500_PM_TOTAL_TRANSLATED                     = 26 /**< Spec Total of load and store micro-ops that reach the second stage of the LSU 1, 2 */
    , e_E500_PM_LOADS_TRANSLATED                     = 27 /**< Spec Cacheable l* or evl* micro-ops translated. (includes load micro-ops
                                                               from load-multiple and load-update instructions) 1,2 */
    , e_E500_PM_STORES_TRANSLATED                    = 28 /**< Spec Cacheable st* or evst* micro-ops translated. (includes micro-ops
                                                               from store-multiple, and store-update instructions) 1,2 */
    , e_E500_PM_TOUCHES_TRANSLATED                   = 29 /**< Spec Cacheable dcbt and dcbtst instructions translated (L1 only) (Doesnt
                                                               count touches that are converted to nops i.e. exceptions, noncacheable, HID0[NOPTI] is set.) */
    , e_E500_PM_CACHEOPS_TRANSLATED                  = 30 /**< Spec dcba, dcbf, dcbst, and dcbz instructions translated. */
    , e_E500_PM_CACHE_INHIBITED_TRANSLATED           = 31 /**< Spec Cache inhibited accesses translated */
    , e_E500_PM_GUARDED_LOADS_TRANSLATED             = 32 /**< Spec Guarded loads translated */
    , e_E500_PM_WRITE_THROUGH_STORES_TRANSLATED      = 33 /**< Spec Write-through stores translated */
    , e_E500_PM_MISALIGNED_L_S_TRANSLATED            = 34 /**< Spec Misaligned load or store accesses translated. */
    , e_E500_PM_TOTAL_ALLOCATED_TO_DLFB              = 35 /**< Spec   */
    , e_E500_PM_LOADS_TRAN_AND_ALLOC_TO_DLFB         = 36 /**< Spec Applies to same class of instructions as loads translated. */
    , e_E500_PM_STORES_COMP_AND_ALLOC_TO_DLFB        = 37 /**< Nonspec Applies to same class of instructions as stores translated. */
    , e_E500_PM_TOUCHES_TRAN_AND_ALLOC_TO_DLFB       = 38 /**< Spec Applies to same class of instructions as touches translated. */
    , e_E500_PM_STORES_COMPLETED                     = 39 /**< Nonspec Cacheable st* or evst* micro-ops completed. (Applies to the same
                                                               class of instructions as stores translated.) 1,2 */
    , e_E500_PM_DATA_L1_CACHE_LOCKS                  = 40 /**< Nonspec Cache lines locked in the data L1 cache. (Counts a lock even if an overlock condition occurs.) */
    , e_E500_PM_DATA_L1_CACHE_RELOADS                = 41 /**< Spec Counts cache reloads for any reason. Typically used to determine
                                                               data cache miss rate (along with loads/stores completed). */
    , e_E500_PM_DATA_L1_CACHE_CASTOUTS               = 42 /**< Spec Does not count castouts due to dcbf. */

    , e_E500_PM_ST_LOAD_MISS_WITH_DLFB_FULL          = 43 /**< Load miss with DLFB full. Spec Counts number of stalls; Com:51 counts cycles stalled. */
    , e_E500_PM_ST_LOAD_MISS_WITH_LD_Q_FULL          = 44 /**< Load miss with load queue full. Spec Counts number of stalls; Com:52 counts cycles stalled. */
    , e_E500_PM_ST_LOAD_G_MISS_WHEN_LD_NOT_BT_CQ     = 45 /**< Load guarded miss when the load is not yet at the bottom of the CQ. Spec Counts number of stalls; Com:53 counts cycles stalled. */
    , e_E500_PM_ST_TRAN_ST_WH_ST_Q_FULL              = 46 /**< Translate a store when the store queue is full. Spec Counts number of stalls; Com:54 counts cycles stalled. */
    , e_E500_PM_ST_ADDRESS_COLLISION                 = 47 /**< Address collision. Spec Counts number of stalls; Com:55 counts cycles stalled. */
    , e_E500_PM_ST_DATA_MMU_MISS                     = 48 /**< Data MMU miss. Spec Counts number of stalls; Com:56 counts cycles stalled. */
    , e_E500_PM_ST_DATA_MMU_BUSY                     = 49 /**< Data MMU busy. Spec Counts number of stalls; Com:57 counts cycles stalled. */
    , e_E500_PM_ST_SECOND_PART_MAWFPARTMC            = 50 /**< Second part of misaligned access when first part missed in cache. Spec Counts number of stalls; Com:58 counts cycles stalled. */

    , e_E500_PM_CY_LOAD_MISS_WITH_DLFB_FULL          = 51 /**< Load miss with DLFB full. Spec Counts cycles stalled; Com:43 counts number of stalls. */
    , e_E500_PM_CY_LOAD_MISS_WITH_LD_Q_FULL          = 52 /**< Load miss with load queue full. Spec Counts cycles stalled; Com:44 counts number of stalls. */
    , e_E500_PM_CY_LOAD_G_MISS_WHEN_LD_NOT_BT_CQ     = 53 /**< Load guarded miss when the load is not yet at the bottom of the CQ. Spec Counts cycles stalled; Com:45 counts number of stalls. */
    , e_E500_PM_CY_TRAN_ST_WH_ST_Q_FULL              = 54 /**< Translate a store when the store queue is full. Spec Counts cycles stalled; Com:46 counts number of stalls. */
    , e_E500_PM_CY_ADDRESS_COLLISION                 = 55 /**< Address collision. Spec Counts cycles stalled; Com:47 counts number of stalls. */
    , e_E500_PM_CY_DATA_MMU_MISS                     = 56 /**< Data MMU miss. Spec Counts cycles stalled; Com:48 counts number of stalls. */
    , e_E500_PM_CY_DATA_MMU_BUSY                     = 57 /**< Data MMU busy. Spec Counts cycles stalled; Com:49 counts number of stalls. */
    , e_E500_PM_CY_SECOND_PART_MAWFPARTMC            = 58 /**< Second part of misaligned access when first part missed in cache. Spec Counts cycles stalled; Com:50 counts number of stalls. */

    , e_E500_PM_INST_L1_CACHE_LOCKS                  = 59 /**< Nonspec Counts cache lines locked in the instruction L1 cache. (Counts a lock even if an overlock occurs.) */
    , e_E500_PM_INST_L1_CACHE_R_F_F                  = 60 /**< Spec Counts reloads due to demand fetch. Typically used to determine instruction cache miss rate (along with instructions completed) */
    , e_E500_PM_NUMBER_OF_FETCHES                    = 61 /**< Spec Counts fetches that write at least one instruction to the IQ. (With instruction fetched (com:4), can used to compute instructions-per-fetch) */

    , e_E500_PM_INSTRUCTION_MMU_TLB4K_RELOADS        = 62 /**< Spec Counts reloads in the level 1 instruction MMU TLB4K.A reload in th elevel 2 MMU TLB4Kis not counted */
    , e_E500_PM_INSTRUCTION_MMU_VSP_RELOADS          = 63 /**< Spec Counts reloads in the level 1 instruction MMU VSP.A reload in th elevel 2 MMU VSP is not counted */
    , e_E500_PM_DATA_MMU_TLB4K_RELOADS               = 64 /**< Spec Counts reloads in the level 1 data MMU TLB4K.A reload in the leve l2 MMU TLB4K is not counted.*/
    , e_E500_PM_DATA_MMU_VSP_RELOADS                 = 65 /**< Spec Counts reloads in the level 1 data MMU VSP.A reload in the level  2MMU VSP is not counted.*/
    , e_E500_PM_L2MMU_MISSES                         = 66 /**< Nonspec Counts instruction TLB/data TLB error interrupts BIU Interface Usage */
    , e_E500_PM_BIU_MASTER_REQUESTS                  = 67 /**< Spec Master transaction starts (assertions of ts */
    , e_E500_PM_BIU_MASTER_INSTRUCTION_SIDE_REQUESTS = 68 /**< Spec Master instruction-side assertions of ts */
    , e_E500_PM_BIU_MASTER_DATA_SIDE_REQUESTS        = 69 /**< Spec Master data-side assertions of ts */
    , e_E500_PM_BIU_MASTER_DATA_SIDE_CASTOUT_REQUESTS= 70 /**< Spec Includes replacement pushes and snoop pushes, but not DCBF castouts. (ts assertions caused by master data-side
                                                               non-program-demand castouts) */
    , e_E500_PM_BIU_MASTER_RETRIES                   = 71 /**< Spec Transactions initiated by this processor that were retried on the BIU
                                                               interface. (The e500 is master and another device retries the e500 transaction.) */

    , e_E500_PM_SNOOP_REQUESTS                       = 72 /**< N/A Externally generated snoop requests. (Counts snoop TSs.) */
    , e_E500_PM_SNOOP_HITS                           = 73 /**< N/A Snoop hits on all data-side resources regardless of the cache state (modified or exclusive) */
    , e_E500_PM_SNOOP_PUSHES                         = 74 /**< N/A Snoop pushes from all data-side resources. (Counts snoop ARTRYs and WOPs.) */
    , e_E500_PM_SNOOP_RETRIES                        = 75 /**< N/A Retried snoop requests. (Counts snoop ARTRYs.) (opposite of com 71another device drives artry). */

    , e_E500_PM_DATA_LINE_F_BUF_LD_M_CYC             = 76 /**< Spec Instances when the number of cycles between a load allocation in the
                                                               data line fill buffer (entry 0) and write-back to the data L1 cache exceeds the threshold. */
    , e_E500_PM_ILFB_FETCH_MISS_CYCLES               = 77 /**< Spec Instances when the number of cycles between allocation in the ILFB
                                                               (entry 0) and write-back to the instruction L1 cache exceeds the threshold. */
    , e_E500_PM_EXT_INPUT_INT_LATENCY_CYC            = 78 /**< N/A Instances when the number of cycles between request for interrupt
                                                               (int) asserted (but possibly masked/disabled) and redirecting
                                                               fetch to external interrupt vector exceeds threshold. */
    , e_E500_PM_CRITICAL_INPUT_INT_LATENCY_CYC       = 79 /**< N/A Instances when the number of cycles between request for critical
                                                               interrupt (cint) is asserted (but possibly masked/disabled) and redirecting
                                                               fetch to the critical interrupt vector exceeds threshold. */
    , e_E500_PM_EXT_INPUT_INT_PEND_LATENCY_CYC       = 80 /**< N/A Instances when the number of cycles between external interrupt
                                                               pending (enabled and pin asserted) and redirecting fetch to the
                                                               external interrupt vector exceeds the threshold. Note that this and the
                                                               next event may count multiple times for a single interrupt if the
                                                               threshold is very small and the interrupt is masked a few cycles after
                                                               it is asserted and later becomes unmasked. */
    , e_E500_PM_CRITICAL_INPUT_INT_PEND_LATENCY_CYC  = 81 /**< N/A Instances when the number of cycles between pin request for critical
                                                               interrupt pending (enabled and pin asserted) and redirecting fetch to
                                                               the critical interrupt vector exceeds the threshold. See note for previous event. */

    , e_E500_PM_PMC0_OVERFLOW                        = 82 /**< N/A PMC0[32] transitions from 1 to 0. */
    , e_E500_PM_PMC1_OVERFLOW                        = 83 /**< N/A PMC1[32] transitions from 1 to 0. */
    , e_E500_PM_PMC2_OVERFLOW                        = 84 /**< N/A PMC2[32] transitions from 1 to 0. */
    , e_E500_PM_PMC3_OVERFLOW                        = 85 /**< N/A PMC3[32] transitioned from 1 to 0. */

    , e_E500_PM_INTERRUPTS_TAKEN                     = 86 /**< Nonspec  */
    , e_E500_PM_EXTERNAL_INTERRUPTS                  = 87 /**< Nonspec External interrupts taken   */
    , e_E500_PM_CRITICAL_INTERRUPTS                  = 88 /**< Nonspec  */
    , e_E500_PM_SC_AND_TRAP_INTERRUPTS               = 89 /**< Nonspec  */
    , e_E500_PM_TRANSITIONS_TBL_BIT                  = 90 /**< Nonspec Counts transitions of the TBL bit selected by PMGC0[TBSEL]. (e500v2 only) */

}e_E500PmEvent;

/**************************************************************************//**
 @Description   Threshold multiple valid values.
*//***************************************************************************/
typedef enum e_E500PmThreshMul
{
      e_E500_PM_THRESHMUL_1   = 0x0
    , e_E500_PM_THRESHMUL_2   = 0x1
    , e_E500_PM_THRESHMUL_4   = 0x2
    , e_E500_PM_THRESHMUL_8   = 0x3
    , e_E500_PM_THRESHMUL_16  = 0x4
    , e_E500_PM_THRESHMUL_32  = 0x5
    , e_E500_PM_THRESHMUL_64  = 0x6
    , e_E500_PM_THRESHMUL_128 = 0x7
}e_E500PmThreshMul;

/**************************************************************************//**
 @Function          E500_PM_SetupCounters

 @Description       The routine defines a PERFORMANCE counter

 @Param[in]         counter     -  counter number
 @Param[in]         event       -  event to count for this counter
 @Param[in]         thresMul    -  threshold multiplier for this counter
 @Param[in]         thresHold   -  threshold for this counter

 @Return            E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error E500_PM_SetupCounter(uint8_t           counter,
                             e_E500PmEvent     event,
                             e_E500PmThreshMul thresMul,
                             uint8_t           thresHold);

/**************************************************************************//**
 @Function          E500_PM_Start

 @Description       The routine start the PERFORMANCE counting.

 @Return            None
*//***************************************************************************/
void E500_PM_Start(void);

/**************************************************************************//**
 @Function          E500_PM_Stop

 @Description       The routine stop the PERFORMANCE counting.

 @Return            None
*//***************************************************************************/
void E500_PM_Stop(void);

/**************************************************************************//**
 @Function          E500_PM_GetCounter

 @Description       The routine return counter value.

 @Param[in]         counter -  counter number

 @Return            64 bit counter value.
*//***************************************************************************/
uint64_t E500_PM_GetCounter(uint8_t counter);

/**************************************************************************//**
 @Function          E500_PM_GetNumCounters

 @Description       The routine return number of counters avilable.

 @Return            number of available counters.
*//***************************************************************************/
uint32_t E500_PM_GetNumCounters(void);


/** @} */ /* end of E500v2_performance_ctrl group */
/** @} */ /* end of E500v2_performance_id group */


#endif /* __E500V2_PM_EXT_H */
