/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @Description   IPIC sources table.
 
                This table provides integration-specific information on
                all IPIC sources.
*//***************************************************************************/

#ifndef __IPIC_SOURCES_TABLE_H
#define __IPIC_SOURCES_TABLE_H


/* IPIC mask registers */
#define IPIC_SIMSR_H_QE_HIGH    0x80000000
#define IPIC_SIMSR_H_QE_LOW     0x40000000
#define IPIC_SIMSR_H_DUART1     0x00000080
#define IPIC_SIMSR_H_DUART2     0x00000040
#define IPIC_SIMSR_H_SEC        0x00000020
#define IPIC_SIMSR_H_I2C1       0x00000004
#define IPIC_SIMSR_H_I2C2       0x00000002
#define IPIC_SIMSR_H_SPI        0x00000001

#define IPIC_SIMSR_L_RTC_SEC    0x80000000
#define IPIC_SIMSR_L_PIT        0x40000000
#define IPIC_SIMSR_L_PCI        0x20000000
#define IPIC_SIMSR_L_RTC_ALR    0x08000000
#define IPIC_SIMSR_L_MU         0x04000000
#define IPIC_SIMSR_L_SBA        0x02000000
#define IPIC_SIMSR_L_DMA        0x01000000
#define IPIC_SIMSR_L_GTM4       0x00800000
#define IPIC_SIMSR_L_GTM8       0x00400000
#define IPIC_SIMSR_L_QE_PORTS   0x00200000
#define IPIC_SIMSR_L_DDR1       0x00080000
#define IPIC_SIMSR_L_DDR2       0x00100000
#define IPIC_SIMSR_L_LBIU       0x00040000
#define IPIC_SIMSR_L_GTM2       0x00020000
#define IPIC_SIMSR_L_GTM6       0x00010000
#define IPIC_SIMSR_L_PMC        0x00008000
#define IPIC_SIMSR_L_GTM3       0x00000800
#define IPIC_SIMSR_L_GTM7       0x00000400
#define IPIC_SIMSR_L_GTM1       0x00000020
#define IPIC_SIMSR_L_GTM5       0x00000010

#define IPIC_SEMSR_IRQ0         0x80000000
#define IPIC_SEMSR_IRQ1         0x40000000
#define IPIC_SEMSR_IRQ2         0x20000000
#define IPIC_SEMSR_IRQ3         0x10000000
#define IPIC_SEMSR_IRQ4         0x08000000
#define IPIC_SEMSR_IRQ5         0x04000000
#define IPIC_SEMSR_IRQ6         0x02000000
#define IPIC_SEMSR_IRQ7         0x01000000

#define IPIC_SERMR_IRQ0         0x80000000
#define IPIC_SERMR_WDT          0x40000000
#define IPIC_SERMR_SBA          0x20000000
#define IPIC_SERMR_CIEE         0x10000000
#define IPIC_SERMR_CMEE         0x08000000
#define IPIC_SERMR_PCI          0x04000000
#define IPIC_SERMR_MU           0x01000000


static t_IpicSrcInfo ipicSourcesTable[] =
{
    /* intrSrc                  maskReg                 mask                   priCode  mcpMask         */
    { e_IPIC_INTR_DUART_1,      e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_DUART1,    0,      0               },
    { e_IPIC_INTR_DUART_2,      e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_DUART2,    1,      0               },
    { e_IPIC_INTR_SEC,          e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_SEC,       2,      0               },
    { e_IPIC_INTR_I2C_1,        e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_I2C1,      5,      0               },
    { e_IPIC_INTR_I2C_2,        e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_I2C2,      6,      0               },
    { e_IPIC_INTR_IRQ_1,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ1,        5,      0               },
    { e_IPIC_INTR_IRQ_2,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ2,        6,      0               },
    { e_IPIC_INTR_IRQ_3,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ3,        7,      0               },
    { e_IPIC_INTR_IRQ_4,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ4,        4,      0               },
    { e_IPIC_INTR_IRQ_5,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ5,        5,      0               },
    { e_IPIC_INTR_IRQ_6,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ6,        6,      0               },
    { e_IPIC_INTR_IRQ_7,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ7,        7,      0               },
    { e_IPIC_INTR_QE_HIGH,      e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_QE_HIGH,   0,      0               },
    { e_IPIC_INTR_QE_LOW,       e_IPIC_MASK_REG_HIGH,   IPIC_SIMSR_H_QE_LOW,    1,      0               },
    { e_IPIC_INTR_IRQ_0,        e_IPIC_MASK_REG_EXT,    IPIC_SEMSR_IRQ0,        4,      IPIC_SERMR_IRQ0 },
    { e_IPIC_INTR_RTC_COUNT,    e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_RTC_SEC,   0,      0               },
    { e_IPIC_INTR_PIT,          e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_PIT,       1,      0               },
    { e_IPIC_INTR_PCI,          e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_PCI,       2,      IPIC_SERMR_PCI  },
    { e_IPIC_INTR_RTC_ALARM,    e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_RTC_ALR,   0,      0               },
    { e_IPIC_INTR_MU,           e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_MU,        1,      IPIC_SERMR_MU   },
    { e_IPIC_INTR_ARBITER,      e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_SBA,       2,      IPIC_SERMR_SBA  },
    { e_IPIC_INTR_DMA,          e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_DMA,       3,      0               },
    { e_IPIC_INTR_GTM_4,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM4,      0,      0               },
    { e_IPIC_INTR_GTM_8,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM8,      0,      0               },
    { e_IPIC_INTR_QE_IO_PORTS,  e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_QE_PORTS,  0,      0               },
    { e_IPIC_INTR_DDR_2,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_DDR2,      0,      0               },
    { e_IPIC_INTR_DDR_1,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_DDR1,      0,      0               },
    { e_IPIC_INTR_LBC,          e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_LBIU,      0,      0               },
    { e_IPIC_INTR_GTM_2,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM2,      0,      0               },
    { e_IPIC_INTR_GTM_6,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM6,      0,      0               },
    { e_IPIC_INTR_PMC,          e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_PMC,       0,      0               },
    { e_IPIC_INTR_GTM_3,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM3,      0,      0               },
    { e_IPIC_INTR_GTM_7,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM7,      0,      0               },
    { e_IPIC_INTR_GTM_1,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM1,      0,      0               },
    { e_IPIC_INTR_GTM_5,        e_IPIC_MASK_REG_LOW,    IPIC_SIMSR_L_GTM5,      0,      0               }
};


#endif /* __IPIC_SOURCES_TABLE_H */

