/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PAR_IO_INTEGRATION_EXT_H
#define __PAR_IO_INTEGRATION_EXT_H

#include "io_port_ext.h"


/**************************************************************************//**
 @Description   Parallel I/O Port ID
*//***************************************************************************/
typedef enum e_ParIoPort
{
    e_PAR_IO_PORT_A,        /**< Parallel I/O port A */
    e_PAR_IO_PORT_B,        /**< Parallel I/O port B */
    e_PAR_IO_PORT_C,        /**< Parallel I/O port C */
    e_PAR_IO_PORT_D,        /**< Parallel I/O port D */
    e_PAR_IO_PORT_E,        /**< Parallel I/O port E */
    e_PAR_IO_PORT_F,        /**< Parallel I/O port F */
    e_PAR_IO_PORT_G         /**< Parallel I/O port G */
 
} e_ParIoPort;


#define PAR_IO_NUM_OF_PORTS     7
	
#define PAR_IO_VALID_PIN_MASKS  \
    { /* Port A */ 0xFFFFFFFF,  \
      /* Port B */ 0xFFFFFFF0,  \
      /* Port C */ 0xFFFFFFFC,  \
      /* Port D */ 0xFFFFFFF0,  \
      /* Port E */ 0xFFFFFFFF,  \
      /* Port F */ 0xFFFFFFFC,  \
      /* Port F */ 0xFFFFFFFF }

#define PAR_IO_VALID_INTR_MASKS \
    { /* Port A */ 0x00018006,  \
      /* Port B */ 0x140C0030,  \
      /* Port C */ 0x0000001C,  \
      /* Port D */ 0x000CC030,  \
      /* Port E */ 0x000C00F1,  \
      /* Port F */ 0x00000800,  \
      /* Port G */ 0x00000001 }


#define PAR_IO_INTR_ID_MAP \
    { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27 }

#define PAR_IO_PIN(_portId, _pinId, _assign, _dir, _shared) \
    IO_PIN(e_PAR_IO_PORT_##_portId, _pinId, e_IO_PIN_ASSIGN_##_assign, e_IO_PIN_DIR_##_dir, _shared)

/* This enum is automatically generated from the Reference Manual */
typedef enum e_ParIoFunction
{
    e_PAR_IO_FUNC_ENET1_RX_ER_SER1_CD_B                   = PAR_IO_PIN(A, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RX_ER_SER2_CD_B                   = PAR_IO_PIN(A, 30, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RX_ER_SER3_CD_B                   = PAR_IO_PIN(B, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RX_ER_SER4_CD_B_OPT1              = PAR_IO_PIN(B,  5, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RX_ER_SER4_CD_B_OPT2              = PAR_IO_PIN(B, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_ER_SER5_CD_B_OPT1              = PAR_IO_PIN(D, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_ER_SER5_CD_B_OPT2              = PAR_IO_PIN(E, 25, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RX_ER_SER6_CD_B                   = PAR_IO_PIN(D, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RX_ER_SER7_CD_B                   = PAR_IO_PIN(E, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RX_ER_SER8_CD_B                   = PAR_IO_PIN(E, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RX_DV_SER1_CTS_B                  = PAR_IO_PIN(A, 15, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RX_DV_SER2_CTS_B_OPT1             = PAR_IO_PIN(A, 29, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RX_DV_SER3_CTS_B                  = PAR_IO_PIN(B, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RX_DV_SER4_CTS_B_OPT1             = PAR_IO_PIN(B,  3, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RX_DV_SER4_CTS_B_OPT2             = PAR_IO_PIN(B, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_DV_SER5_CTS_B_OPT1             = PAR_IO_PIN(D, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_DV_SER5_CTS_B_OPT2             = PAR_IO_PIN(E, 24, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RX_DV_SER6_CTS_B                  = PAR_IO_PIN(D, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RX_DV_SER7_CTS_B                  = PAR_IO_PIN(E, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RX_DV_SER8_CTS_B                  = PAR_IO_PIN(E, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UART2_CTS_B_OPT1                        = PAR_IO_PIN(E, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UART2_CTS_B_OPT2                        = PAR_IO_PIN(F,  1, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA0_DACK_B_OPT1                        = PAR_IO_PIN(C,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA0_DACK_B_OPT2                        = PAR_IO_PIN(C, 29, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA1_DACK_B_OPT1                        = PAR_IO_PIN(F,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA1_DACK_B_OPT2                        = PAR_IO_PIN(F, 27, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA2_DACK_B_OPT1                        = PAR_IO_PIN(B,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA2_DACK_B_OPT2                        = PAR_IO_PIN(C, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA3_DACK_B_OPT1                        = PAR_IO_PIN(B, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA3_DACK_B_OPT2                        = PAR_IO_PIN(C,  0, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA0_DONE_B_OPT1                        = PAR_IO_PIN(A, 28, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA0_DONE_B_OPT2                        = PAR_IO_PIN(B, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA1_DONE_B_OPT1                        = PAR_IO_PIN(E, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA1_DONE_B_OPT2                        = PAR_IO_PIN(F,  1, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA2_DONE_B_OPT1                        = PAR_IO_PIN(B,  6, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA2_DONE_B_OPT2                        = PAR_IO_PIN(C, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA3_DONE_B_OPT1                        = PAR_IO_PIN(A, 30, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA3_DONE_B_OPT2                        = PAR_IO_PIN(C, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_DMA0_DREQ_B_OPT1                        = PAR_IO_PIN(C, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA0_DREQ_B_OPT2                        = PAR_IO_PIN(C, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA1_DREQ_B_OPT1                        = PAR_IO_PIN(F,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA1_DREQ_B_OPT2                        = PAR_IO_PIN(F, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA2_DREQ_B_OPT1                        = PAR_IO_PIN(B,  5, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA2_DREQ_B_OPT2                        = PAR_IO_PIN(D, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA3_DREQ_B_OPT1                        = PAR_IO_PIN(C, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_DMA3_DREQ_B_OPT2                        = PAR_IO_PIN(C, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_NUSB_OE_OPT1                            = PAR_IO_PIN(B,  2, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_NUSB_OE_OPT2                            = PAR_IO_PIN(G, 11, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_DEVSEL_B                            = PAR_IO_PIN(F, 16, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_FRAME_B                             = PAR_IO_PIN(F, 12, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_GNT_B0                              = PAR_IO_PIN(F, 23, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_GNT_B1                              = PAR_IO_PIN(F, 24, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_GNT_B2                              = PAR_IO_PIN(F, 25, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_INTA_B                                  = PAR_IO_PIN(F,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_IRDY_B                              = PAR_IO_PIN(F, 14, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_PERR_B                              = PAR_IO_PIN(F, 19, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_REQ_B0                              = PAR_IO_PIN(F, 20, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_REQ_B1                              = PAR_IO_PIN(F, 21, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_PCI_REQ_B2                              = PAR_IO_PIN(F, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_PCI_RESET_OUT_B                         = PAR_IO_PIN(F,  6, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_SERR_B                              = PAR_IO_PIN(F, 18, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_STOP_B                              = PAR_IO_PIN(F, 15, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_TRDY_B                              = PAR_IO_PIN(F, 13, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMA_REQ                                = PAR_IO_PIN(A,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_REQ                                = PAR_IO_PIN(A, 19, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_REQ                                = PAR_IO_PIN(B, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_REQ                                = PAR_IO_PIN(B, 25, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDME_REQ                                = PAR_IO_PIN(D, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMF_REQ                                = PAR_IO_PIN(D, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMG_REQ                                = PAR_IO_PIN(E, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMH_REQ                                = PAR_IO_PIN(E, 25, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TX_EN_SER1_RTS_B                  = PAR_IO_PIN(A,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TX_EN_SER2_RTS_B_OPT1             = PAR_IO_PIN(A, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TX_EN_SER3_RTS_B                  = PAR_IO_PIN(B,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TX_EN_SER4_RTS_B_OPT1             = PAR_IO_PIN(B, 11, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TX_EN_SER4_RTS_B_OPT2             = PAR_IO_PIN(B, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TX_EN_SER5_RTS_B_OPT1             = PAR_IO_PIN(D,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TX_EN_SER5_RTS_B_OPT2             = PAR_IO_PIN(E, 23, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TX_EN_SER6_RTS_B                  = PAR_IO_PIN(D, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TX_EN_SER7_RTS_B                  = PAR_IO_PIN(E,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TX_EN_SER8_RTS_B                  = PAR_IO_PIN(E, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UART2_RTS_B_OPT1                        = PAR_IO_PIN(E, 25, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UART2_RTS_B_OPT2                        = PAR_IO_PIN(F,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXEN_B                           = PAR_IO_PIN(B, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXEN_B_OPT1                      = PAR_IO_PIN(E, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXEN_B_OPT2                      = PAR_IO_PIN(G, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXEN_B0                          = PAR_IO_PIN(D,  6, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXEN_B0_OPT1                     = PAR_IO_PIN(E, 26, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXEN_B0_OPT2                     = PAR_IO_PIN(F, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B1_OPT1                       = PAR_IO_PIN(A, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B1_OPT2                       = PAR_IO_PIN(E, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXEN_B1                            = PAR_IO_PIN(F, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B2_OPT1                       = PAR_IO_PIN(C, 21, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B2_OPT2                       = PAR_IO_PIN(E, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXEN_B2                            = PAR_IO_PIN(G, 31, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B3_OPT1                       = PAR_IO_PIN(C, 19, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B3_OPT2                       = PAR_IO_PIN(E, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXEN_B3                            = PAR_IO_PIN(F, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPISEL_B                                = PAR_IO_PIN(E, 31, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE1_B_OPT1                      = PAR_IO_PIN(C,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE1_B_OPT2                      = PAR_IO_PIN(F,  0, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE2_B_OPT1                      = PAR_IO_PIN(A, 31, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE2_B_OPT2                      = PAR_IO_PIN(C, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE3_B_OPT1                      = PAR_IO_PIN(C, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE3_B_OPT2                      = PAR_IO_PIN(F, 28, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE4_B_OPT1                      = PAR_IO_PIN(C,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE4_B_OPT2                      = PAR_IO_PIN(C, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TGATE4_B_OPT3                      = PAR_IO_PIN(C, 29, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT1_B_OPT1                       = PAR_IO_PIN(A, 28, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT1_B_OPT2                       = PAR_IO_PIN(F,  2, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM2_TOUT1_B_OPT1                       = PAR_IO_PIN(D, 17, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM2_TOUT1_B_OPT2                       = PAR_IO_PIN(F,  3, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT2_B_OPT1                       = PAR_IO_PIN(D, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT2_B_OPT2                       = PAR_IO_PIN(F,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT3_B_OPT1                       = PAR_IO_PIN(C, 24, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT3_B_OPT2                       = PAR_IO_PIN(E, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM2_TOUT3_B_OPT1                       = PAR_IO_PIN(C, 18, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM2_TOUT3_B_OPT2                       = PAR_IO_PIN(C, 29, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT4_B_OPT1                       = PAR_IO_PIN(C, 10, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TOUT4_B_OPT2                       = PAR_IO_PIN(C, 28, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXEN_B                           = PAR_IO_PIN(D,  6, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXEN_B0                          = PAR_IO_PIN(B, 15, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXEN_B_OPT1                      = PAR_IO_PIN(E, 26, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXEN_B_OPT2                      = PAR_IO_PIN(F, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXEN_B0_OPT1                     = PAR_IO_PIN(E, 15, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXEN_B0_OPT2                     = PAR_IO_PIN(G, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B1_OPT1                       = PAR_IO_PIN(A, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B1_OPT2                       = PAR_IO_PIN(E, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXEN_B1                            = PAR_IO_PIN(G, 30, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B2_OPT1                       = PAR_IO_PIN(C, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B2_OPT2                       = PAR_IO_PIN(E, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXEN_B2                            = PAR_IO_PIN(F, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B3_OPT1                       = PAR_IO_PIN(C, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B3_OPT2                       = PAR_IO_PIN(E, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXEN_B3                            = PAR_IO_PIN(F, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_BRGO1                                   = PAR_IO_PIN(C,  0, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO10                                  = PAR_IO_PIN(C, 15, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO11                                  = PAR_IO_PIN(C, 16, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO12                                  = PAR_IO_PIN(C, 17, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO13                                  = PAR_IO_PIN(C, 18, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO14                                  = PAR_IO_PIN(C, 19, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO15                                  = PAR_IO_PIN(C, 20, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO16                                  = PAR_IO_PIN(C, 25, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO2                                   = PAR_IO_PIN(C,  1, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO3                                   = PAR_IO_PIN(C,  3, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO4                                   = PAR_IO_PIN(C,  4, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO5                                   = PAR_IO_PIN(C,  5, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO6                                   = PAR_IO_PIN(C,  7, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO7                                   = PAR_IO_PIN(C,  8, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO8                                   = PAR_IO_PIN(C, 11, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO9                                   = PAR_IO_PIN(C, 14, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_CLK1                                    = PAR_IO_PIN(C,  0, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK10                                   = PAR_IO_PIN(C,  9, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK11                                   = PAR_IO_PIN(C, 10, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK12                                   = PAR_IO_PIN(C, 11, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK13                                   = PAR_IO_PIN(C, 12, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK14                                   = PAR_IO_PIN(C, 13, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK15                                   = PAR_IO_PIN(C, 14, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK16                                   = PAR_IO_PIN(C, 15, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK17                                   = PAR_IO_PIN(C, 16, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK18                                   = PAR_IO_PIN(C, 17, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK19                                   = PAR_IO_PIN(C, 18, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK2                                    = PAR_IO_PIN(C,  1, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK20                                   = PAR_IO_PIN(C, 19, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK21                                   = PAR_IO_PIN(C, 20, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK22                                   = PAR_IO_PIN(C, 21, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK23                                   = PAR_IO_PIN(C, 22, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK24                                   = PAR_IO_PIN(C, 23, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK3                                    = PAR_IO_PIN(C,  2, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK4                                    = PAR_IO_PIN(C,  3, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK5                                    = PAR_IO_PIN(C,  4, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK6                                    = PAR_IO_PIN(C,  5, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK7                                    = PAR_IO_PIN(C,  6, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK8                                    = PAR_IO_PIN(C,  7, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK9                                    = PAR_IO_PIN(C,  8, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_TDMA_CLKO                               = PAR_IO_PIN(A, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_CLKO                               = PAR_IO_PIN(A, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_CLKO                               = PAR_IO_PIN(B,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_CLKO                               = PAR_IO_PIN(B, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDME_CLKO                               = PAR_IO_PIN(D,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMF_CLKO                               = PAR_IO_PIN(D, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMG_CLKO                               = PAR_IO_PIN(E,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMH_CLKO                               = PAR_IO_PIN(E, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_COL_RXD4_SER1_RXD4                = PAR_IO_PIN(A, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_COL_RXD4_SER2_RXD4                = PAR_IO_PIN(A, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_COL                               = PAR_IO_PIN(B, 10, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_COL                               = PAR_IO_PIN(B, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_COL                               = PAR_IO_PIN(D, 10, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_COL                               = PAR_IO_PIN(D, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_COL                               = PAR_IO_PIN(E, 10, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_COL                               = PAR_IO_PIN(E, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_COL                               = PAR_IO_PIN(C, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_COL                               = PAR_IO_PIN(C, 11, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_CPCI_HS_ENUM                            = PAR_IO_PIN(F, 25, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_CPCI_HS_ES                              = PAR_IO_PIN(F, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_CPCI_HS_LED                             = PAR_IO_PIN(F, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_CRS_SDET                          = PAR_IO_PIN(A, 14, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_CRS_SDET                          = PAR_IO_PIN(A, 28, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_CRS                               = PAR_IO_PIN(B, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_CRS                               = PAR_IO_PIN(B, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_CRS                               = PAR_IO_PIN(D, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_CRS                               = PAR_IO_PIN(D, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_CRS                               = PAR_IO_PIN(E, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_CRS                               = PAR_IO_PIN(E, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_GTXCLK                            = PAR_IO_PIN(C,  9, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_GTXCLK_OPT1                       = PAR_IO_PIN(C,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_GTXCLK_OPT2                       = PAR_IO_PIN(F, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_M66EN                                   = PAR_IO_PIN(F,  4, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SPI_MDC                                 = PAR_IO_PIN(A,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_MUX_MDC                              = PAR_IO_PIN(A,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI_MDIO                                = PAR_IO_PIN(A,  1, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_QE_MUX_MDIO                             = PAR_IO_PIN(A,  1, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD0                                 = PAR_IO_PIN(G,  0, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD10                                = PAR_IO_PIN(G, 10, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD11                                = PAR_IO_PIN(G, 11, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD12                                = PAR_IO_PIN(G, 12, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD13                                = PAR_IO_PIN(G, 13, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD14                                = PAR_IO_PIN(G, 14, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD15                                = PAR_IO_PIN(G, 15, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD16                                = PAR_IO_PIN(G, 16, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD17                                = PAR_IO_PIN(G, 17, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD18                                = PAR_IO_PIN(G, 18, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD19                                = PAR_IO_PIN(G, 19, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD1                                 = PAR_IO_PIN(G,  1, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD20                                = PAR_IO_PIN(G, 20, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD21                                = PAR_IO_PIN(G, 21, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD22                                = PAR_IO_PIN(G, 22, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD23                                = PAR_IO_PIN(G, 23, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD24                                = PAR_IO_PIN(G, 24, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD25                                = PAR_IO_PIN(G, 25, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD26                                = PAR_IO_PIN(G, 26, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD27                                = PAR_IO_PIN(G, 27, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD28                                = PAR_IO_PIN(G, 28, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD29                                = PAR_IO_PIN(G, 29, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD2                                 = PAR_IO_PIN(G,  2, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD30                                = PAR_IO_PIN(G, 30, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD31                                = PAR_IO_PIN(G, 31, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD3                                 = PAR_IO_PIN(G,  3, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD4                                 = PAR_IO_PIN(G,  4, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD5                                 = PAR_IO_PIN(G,  5, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD6                                 = PAR_IO_PIN(G,  6, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD7                                 = PAR_IO_PIN(G,  7, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD8                                 = PAR_IO_PIN(G,  8, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_AD9                                 = PAR_IO_PIN(G,  9, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_C_BE_B0                             = PAR_IO_PIN(F,  7, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_C_BE_B1                             = PAR_IO_PIN(F,  8, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_C_BE_B2                             = PAR_IO_PIN(F,  9, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_C_BE_B3                             = PAR_IO_PIN(F, 10, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_CLK0                                = PAR_IO_PIN(F, 26, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_CLK1                                = PAR_IO_PIN(F, 27, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_CLK2                                = PAR_IO_PIN(F, 28, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PCI_IDSEL                               = PAR_IO_PIN(F, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_PCI_PAR                                 = PAR_IO_PIN(F, 11, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_PCI_SYNC_OUT                            = PAR_IO_PIN(F, 29, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXCLAV                           = PAR_IO_PIN(B, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXCLAV_OPT1                      = PAR_IO_PIN(E, 16, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXCLAV_OPT2                      = PAR_IO_PIN(G, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXCLAV0                          = PAR_IO_PIN(D,  7, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXCLAV0_OPT1                     = PAR_IO_PIN(E, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXCLAV0_OPT2                     = PAR_IO_PIN(F, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV1_OPT1                       = PAR_IO_PIN(A,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV1_OPT2                       = PAR_IO_PIN(E, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXCLAV1                            = PAR_IO_PIN(G, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV2_OPT1                       = PAR_IO_PIN(A, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV2_OPT2                       = PAR_IO_PIN(E, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXCLAV2                            = PAR_IO_PIN(G, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV3_OPT1                       = PAR_IO_PIN(C, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV3_OPT2                       = PAR_IO_PIN(E, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXCLAV3                            = PAR_IO_PIN(F, 28, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXCLAV                           = PAR_IO_PIN(D,  7, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXCLAV_OPT1                      = PAR_IO_PIN(E, 27, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXCLAV_OPT2                      = PAR_IO_PIN(F, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXCLAV0                          = PAR_IO_PIN(B, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXCLAV0_OPT1                     = PAR_IO_PIN(E, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXCLAV0_OPT2                     = PAR_IO_PIN(G, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV1_OPT1                       = PAR_IO_PIN(A, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV1_OPT2                       = PAR_IO_PIN(E, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXCLAV1                            = PAR_IO_PIN(G, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV2_OPT1                       = PAR_IO_PIN(A, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV2_OPT2                       = PAR_IO_PIN(E, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXCLAV2                            = PAR_IO_PIN(G, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV3_OPT1                       = PAR_IO_PIN(C, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV3_OPT2                       = PAR_IO_PIN(E, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXCLAV3                            = PAR_IO_PIN(F, 29, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_RTC_ALARM1_OPT1                         = PAR_IO_PIN(C,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_ALARM1_OPT2                         = PAR_IO_PIN(E, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_ALARM2_OPT1                         = PAR_IO_PIN(C, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_ALARM2_OPT2                         = PAR_IO_PIN(E,  6, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PTP_CLK                                 = PAR_IO_PIN(C, 30, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN3_OPT1                          = PAR_IO_PIN(B,  4, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN3_OPT2                          = PAR_IO_PIN(C, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN4_OPT1                          = PAR_IO_PIN(C, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN4_OPT2                          = PAR_IO_PIN(C, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_RTC_PPS1_OPT1                           = PAR_IO_PIN(C, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PPS1_OPT2                           = PAR_IO_PIN(E, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PPS2_OPT1                           = PAR_IO_PIN(C, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PPS2_OPT2                           = PAR_IO_PIN(D, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PPS3_OPT1                           = PAR_IO_PIN(C,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PPS3_OPT2                           = PAR_IO_PIN(F, 29, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PRESCALE_OPT1                       = PAR_IO_PIN(C, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RTC_PRESCALE_OPT2                       = PAR_IO_PIN(E, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR0                          = PAR_IO_PIN(E,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR0                          = PAR_IO_PIN(E,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR0_OPT1                     = PAR_IO_PIN(B,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR0_OPT2                     = PAR_IO_PIN(F,  5, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR0_OPT1                     = PAR_IO_PIN(B,  4, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR0_OPT2                     = PAR_IO_PIN(G,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR1                          = PAR_IO_PIN(E,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR1                          = PAR_IO_PIN(E,  8, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR1_OPT1                     = PAR_IO_PIN(A,  8, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR1_OPT2                     = PAR_IO_PIN(F,  6, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR1_OPT1                     = PAR_IO_PIN(D, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR1_OPT2                     = PAR_IO_PIN(G,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR2                          = PAR_IO_PIN(E,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR2                          = PAR_IO_PIN(E, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR2_OPT1                     = PAR_IO_PIN(A, 22, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR2_OPT2                     = PAR_IO_PIN(F, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR2_OPT1                     = PAR_IO_PIN(D, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR2_OPT2                     = PAR_IO_PIN(G,  5, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR3                          = PAR_IO_PIN(E, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR3                          = PAR_IO_PIN(E,  1, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR3_OPT1                     = PAR_IO_PIN(A, 14, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR3_OPT2                     = PAR_IO_PIN(F, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR3_OPT1                     = PAR_IO_PIN(A, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR3_OPT2                     = PAR_IO_PIN(F, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR4                          = PAR_IO_PIN(E,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR4                          = PAR_IO_PIN(E, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR4_OPT1                     = PAR_IO_PIN(C, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RXADDR4_OPT2                     = PAR_IO_PIN(F, 24, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR4_OPT1                     = PAR_IO_PIN(E, 20, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RXADDR4_OPT2                     = PAR_IO_PIN(G,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXADDR5                            = PAR_IO_PIN(E,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXADDR5                            = PAR_IO_PIN(A, 28, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_RXD0_SER1_RXD0                    = PAR_IO_PIN(A,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD0_SER2_RXD0_OPT1               = PAR_IO_PIN(A, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD0_SER2_RXD0_OPT2               = PAR_IO_PIN(F, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD1_SER1_RXD1                    = PAR_IO_PIN(A, 10, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD1_SER2_RXD1_OPT1               = PAR_IO_PIN(A, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD1_SER2_RXD1_OPT2               = PAR_IO_PIN(F, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD2_SER1_RXD2                    = PAR_IO_PIN(A, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD2_SER2_RXD2_OPT1               = PAR_IO_PIN(A, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD2_SER2_RXD2_OPT2               = PAR_IO_PIN(G, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD3_SER1_RXD3                    = PAR_IO_PIN(A, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD3_SER2_RXD3_OPT1               = PAR_IO_PIN(A, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD3_SER2_RXD3_OPT2               = PAR_IO_PIN(F, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RX_DV_SER2_CTS_B_OPT2             = PAR_IO_PIN(G, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD5_SER1_RXD5                    = PAR_IO_PIN(B,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD5_SER2_RXD5                    = PAR_IO_PIN(B, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD6_SER1_RXD6                    = PAR_IO_PIN(B,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD6_SER2_RXD6                    = PAR_IO_PIN(B, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD7_SER1_RXD7                    = PAR_IO_PIN(B,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD7_SER2_RXD7                    = PAR_IO_PIN(B, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA0                            = PAR_IO_PIN(D, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA0                            = PAR_IO_PIN(D,  4, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA0                            = PAR_IO_PIN(F,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA0                            = PAR_IO_PIN(F, 10, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA10                           = PAR_IO_PIN(D, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA10                           = PAR_IO_PIN(B, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA10_OPT1                      = PAR_IO_PIN(E, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA10_OPT2                      = PAR_IO_PIN(F, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA10_OPT1                      = PAR_IO_PIN(B, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA10_OPT2                      = PAR_IO_PIN(G, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA11                           = PAR_IO_PIN(D, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA11                           = PAR_IO_PIN(B, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA11_OPT1                      = PAR_IO_PIN(B, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA11_OPT2                      = PAR_IO_PIN(F, 18, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA11_OPT1                      = PAR_IO_PIN(E, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA11_OPT2                      = PAR_IO_PIN(G,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA12                           = PAR_IO_PIN(D, 11, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA12                           = PAR_IO_PIN(B, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA12_OPT1                      = PAR_IO_PIN(B,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA12_OPT2                      = PAR_IO_PIN(F, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA12                           = PAR_IO_PIN(G, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA13                           = PAR_IO_PIN(D, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA13                           = PAR_IO_PIN(B, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA13_OPT1                      = PAR_IO_PIN(C, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA13_OPT2                      = PAR_IO_PIN(F, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA13_OPT1                      = PAR_IO_PIN(E, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA13_OPT2                      = PAR_IO_PIN(G, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA14                           = PAR_IO_PIN(D,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA14                           = PAR_IO_PIN(B, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA14_OPT1                      = PAR_IO_PIN(B,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA14_OPT2                      = PAR_IO_PIN(F, 15, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA14_OPT1                      = PAR_IO_PIN(B,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA14_OPT2                      = PAR_IO_PIN(G, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA15                           = PAR_IO_PIN(D,  8, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA15                           = PAR_IO_PIN(B, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA15_OPT1                      = PAR_IO_PIN(B,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA15_OPT2                      = PAR_IO_PIN(F, 14, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA15_OPT1                      = PAR_IO_PIN(E, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA15_OPT2                      = PAR_IO_PIN(G, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA1                            = PAR_IO_PIN(D, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA1                            = PAR_IO_PIN(D,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA1                            = PAR_IO_PIN(F,  1, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA1                            = PAR_IO_PIN(F,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA2                            = PAR_IO_PIN(D, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA2                            = PAR_IO_PIN(D,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA2                            = PAR_IO_PIN(G,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA2                            = PAR_IO_PIN(F,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA3                            = PAR_IO_PIN(D, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA3                            = PAR_IO_PIN(D,  1, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA3                            = PAR_IO_PIN(G,  2, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA3                            = PAR_IO_PIN(F,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA4                            = PAR_IO_PIN(D, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA4                            = PAR_IO_PIN(D,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA4                            = PAR_IO_PIN(G,  1, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA4_OPT1                       = PAR_IO_PIN(A, 13, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA4_OPT2                       = PAR_IO_PIN(G, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA5                            = PAR_IO_PIN(D, 21, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA5                            = PAR_IO_PIN(B, 27, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA5                            = PAR_IO_PIN(F,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA5                            = PAR_IO_PIN(G, 10, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA6                            = PAR_IO_PIN(D, 20, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA6                            = PAR_IO_PIN(B, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA6                            = PAR_IO_PIN(F,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA6                            = PAR_IO_PIN(G, 29, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA7                            = PAR_IO_PIN(D, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA7                            = PAR_IO_PIN(B, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA7                            = PAR_IO_PIN(G,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA7                            = PAR_IO_PIN(G, 28, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA8                            = PAR_IO_PIN(D, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA8                            = PAR_IO_PIN(B, 24, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA8_OPT1                       = PAR_IO_PIN(E, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA8_OPT2                       = PAR_IO_PIN(G,  0, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA8_OPT1                       = PAR_IO_PIN(E, 24, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA8_OPT2                       = PAR_IO_PIN(G, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA9                            = PAR_IO_PIN(D, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA9                            = PAR_IO_PIN(B, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA9_OPT1                       = PAR_IO_PIN(B,  1, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXDATA9_OPT2                       = PAR_IO_PIN(F,  3, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA9_OPT1                       = PAR_IO_PIN(E, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXDATA9_OPT2                       = PAR_IO_PIN(G, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_REOP                               = PAR_IO_PIN(C, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TEOP                               = PAR_IO_PIN(C, 25, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_REOP_OPT1                          = PAR_IO_PIN(C, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_REOP_OPT2                          = PAR_IO_PIN(G, 30, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TEOP_OPT1                          = PAR_IO_PIN(C, 20, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TEOP_OPT2                          = PAR_IO_PIN(F, 25, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ1                           = PAR_IO_PIN(B, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ2                           = PAR_IO_PIN(C, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ3                           = PAR_IO_PIN(D, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ4                           = PAR_IO_PIN(D, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RERR                               = PAR_IO_PIN(D, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TERR                               = PAR_IO_PIN(D, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RERR_OPT1                          = PAR_IO_PIN(A, 19, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RERR_OPT2                          = PAR_IO_PIN(G, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TERR_OPT1                          = PAR_IO_PIN(C, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TERR_OPT2                          = PAR_IO_PIN(G, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RMOD_OPT1                          = PAR_IO_PIN(C, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RMOD_OPT2                          = PAR_IO_PIN(F,  0, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TMOD_OPT1                          = PAR_IO_PIN(C, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TMOD_OPT2                          = PAR_IO_PIN(F,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RMOD_OPT1                          = PAR_IO_PIN(C, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RMOD_OPT2                          = PAR_IO_PIN(G, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TMOD_OPT1                          = PAR_IO_PIN(A, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TMOD_OPT2                          = PAR_IO_PIN(G, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_RN_OPT1                             = PAR_IO_PIN(B, 11, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_RN_OPT2                             = PAR_IO_PIN(F, 29, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_RP_OPT1                             = PAR_IO_PIN(B,  9, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_RP_OPT2                             = PAR_IO_PIN(G,  5, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXPRTY                             = PAR_IO_PIN(D, 19, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXPRTY                             = PAR_IO_PIN(D, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXPRTY_OPT1                        = PAR_IO_PIN(B, 12, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXPRTY_OPT2                        = PAR_IO_PIN(G, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXPRTY_OPT1                        = PAR_IO_PIN(B, 13, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXPRTY_OPT2                        = PAR_IO_PIN(G, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXSOC                              = PAR_IO_PIN(D,  5, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXSOC                              = PAR_IO_PIN(B, 14, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_RXSOC_OPT1                         = PAR_IO_PIN(E, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_RXSOC_OPT2                         = PAR_IO_PIN(F, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_TXSOC_OPT1                         = PAR_IO_PIN(A, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXSOC_OPT2                         = PAR_IO_PIN(G, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_RSYNC_OPT1                         = PAR_IO_PIN(A, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RSYNC_OPT2                         = PAR_IO_PIN(B, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RSYNC_OPT1                         = PAR_IO_PIN(A, 28, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RSYNC_OPT2                         = PAR_IO_PIN(C, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RSYNC_OPT1                         = PAR_IO_PIN(B,  2, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RSYNC_OPT2                         = PAR_IO_PIN(E, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RSYNC_OPT1                         = PAR_IO_PIN(A, 24, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RSYNC_OPT2                         = PAR_IO_PIN(B,  0, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RSYNC_OPT3                         = PAR_IO_PIN(B, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_RSYNC_OPT1                         = PAR_IO_PIN(A, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_RSYNC_OPT2                         = PAR_IO_PIN(D,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_RSYNC_OPT3                         = PAR_IO_PIN(E, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_RSYNC_OPT1                         = PAR_IO_PIN(A, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_RSYNC_OPT2                         = PAR_IO_PIN(C, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_RSYNC_OPT3                         = PAR_IO_PIN(D, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMG_RSYNC_OPT1                         = PAR_IO_PIN(D, 25, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMG_RSYNC_OPT2                         = PAR_IO_PIN(E,  2, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_RSYNC_OPT1                         = PAR_IO_PIN(E,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_RSYNC_OPT2                         = PAR_IO_PIN(E, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RVAL_OPT1                        = PAR_IO_PIN(E, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RVAL_OPT2                        = PAR_IO_PIN(F,  2, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RVAL_OPT1                        = PAR_IO_PIN(E, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RVAL_OPT2                        = PAR_IO_PIN(F,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_RVAL_OPT1                        = PAR_IO_PIN(A, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_RVAL_OPT2                        = PAR_IO_PIN(G, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_RVAL_OPT1                        = PAR_IO_PIN(A, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_RVAL_OPT2                        = PAR_IO_PIN(G, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_RXD_OPT1                            = PAR_IO_PIN(B, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_RXD_OPT2                            = PAR_IO_PIN(F, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_GRXCLK                            = PAR_IO_PIN(A,  0, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_GRXCLK_OPT1                       = PAR_IO_PIN(A, 31, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_GRXCLK_OPT2                       = PAR_IO_PIN(F, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_GRXCLK1                           = PAR_IO_PIN(C, 29, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_GRXCLK1                           = PAR_IO_PIN(C, 28, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD0_SER3_RXD0                    = PAR_IO_PIN(B,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD0_SER4_RXD0_OPT1               = PAR_IO_PIN(B,  8, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD0_SER4_RXD0_OPT2               = PAR_IO_PIN(B, 20, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD0_SER5_RXD0_OPT1               = PAR_IO_PIN(D,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD0_SER5_RXD0_OPT2               = PAR_IO_PIN(E, 22, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD0_SER6_RXD0                    = PAR_IO_PIN(D, 20, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD0_SER7_RXD0                    = PAR_IO_PIN(E,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD0_SER8_RXD0                    = PAR_IO_PIN(E, 20, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RXD0_OPT1                          = PAR_IO_PIN(A,  8, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMA_RXD0_OPT2                          = PAR_IO_PIN(B, 25, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMB_RXD0_OPT1                          = PAR_IO_PIN(A, 27, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMB_RXD0_OPT2                          = PAR_IO_PIN(C, 27, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_RXD0_OPT1                          = PAR_IO_PIN(B,  5, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_RXD0_OPT2                          = PAR_IO_PIN(E, 11, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_RXD0_OPT1                          = PAR_IO_PIN(A, 25, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_RXD0_OPT2                          = PAR_IO_PIN(B,  7, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_RXD0_OPT3                          = PAR_IO_PIN(B, 22, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDME_RXD0_OPT1                          = PAR_IO_PIN(A, 29, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDME_RXD0_OPT2                          = PAR_IO_PIN(D,  8, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDME_RXD0_OPT3                          = PAR_IO_PIN(E, 21, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMF_RXD0_OPT1                          = PAR_IO_PIN(D, 19, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMF_RXD0_OPT2                          = PAR_IO_PIN(E,  9, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMG_RXD0_OPT1                          = PAR_IO_PIN(D, 24, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMG_RXD0_OPT2                          = PAR_IO_PIN(E,  8, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMH_RXD0_OPT1                          = PAR_IO_PIN(E, 12, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMH_RXD0_OPT2                          = PAR_IO_PIN(E, 19, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_ENET3_RXD1_SER3_RXD1                    = PAR_IO_PIN(B,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD1_SER4_RXD1_OPT1               = PAR_IO_PIN(B,  2, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD1_SER4_RXD1_OPT2               = PAR_IO_PIN(B, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD1_SER5_RXD1_OPT1               = PAR_IO_PIN(D,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD1_SER5_RXD1_OPT2               = PAR_IO_PIN(E, 19, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD1_SER6_RXD1                    = PAR_IO_PIN(D, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD1_SER7_RXD1                    = PAR_IO_PIN(E,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD1_SER8_RXD1                    = PAR_IO_PIN(E, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RXD1                               = PAR_IO_PIN(A,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RXD1                               = PAR_IO_PIN(A, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RXD1                               = PAR_IO_PIN(B,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RXD1                               = PAR_IO_PIN(B, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_RXD1                               = PAR_IO_PIN(D,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_RXD1                               = PAR_IO_PIN(D, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_RXD1                               = PAR_IO_PIN(E, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD2_SER3_RXD2                    = PAR_IO_PIN(B,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD2_SER4_RXD2                    = PAR_IO_PIN(B, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD2_SER5_RXD2                    = PAR_IO_PIN(D,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD2_SER6_RXD2                    = PAR_IO_PIN(D, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD2_SER7_RXD2                    = PAR_IO_PIN(E,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD2_SER8_RXD2                    = PAR_IO_PIN(E, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RXD2                               = PAR_IO_PIN(A, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RXD2                               = PAR_IO_PIN(A, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RXD2                               = PAR_IO_PIN(B,  7, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RXD2                               = PAR_IO_PIN(B, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_RXD2                               = PAR_IO_PIN(D,  7, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_RXD2                               = PAR_IO_PIN(D, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_RXD2                               = PAR_IO_PIN(E, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD3_SER3_RXD3                    = PAR_IO_PIN(B,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD3_SER4_RXD3                    = PAR_IO_PIN(B, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD3_SER5_RXD3                    = PAR_IO_PIN(D,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD3_SER6_RXD3                    = PAR_IO_PIN(D, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD3_SER7_RXD3                    = PAR_IO_PIN(E,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD3_SER8_RXD3                    = PAR_IO_PIN(E, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RXD3                               = PAR_IO_PIN(A, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RXD3                               = PAR_IO_PIN(A, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RXD3                               = PAR_IO_PIN(B, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RXD3                               = PAR_IO_PIN(B, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_RXD3                               = PAR_IO_PIN(D, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_RXD3                               = PAR_IO_PIN(D, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_RXD3                               = PAR_IO_PIN(E, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UART2_SIN_OPT1                          = PAR_IO_PIN(E, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UART2_SIN_OPT2                          = PAR_IO_PIN(F,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UART2_SOUT_OPT1                         = PAR_IO_PIN(E, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UART2_SOUT_OPT2                         = PAR_IO_PIN(F,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPICLK                                  = PAR_IO_PIN(E, 30, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPIMISO                                 = PAR_IO_PIN(E, 29, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPIMOSI                                 = PAR_IO_PIN(E, 28, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_STPA                             = PAR_IO_PIN(E,  9, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_STPA                             = PAR_IO_PIN(E,  9, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_STPA_OPT1                        = PAR_IO_PIN(A, 20, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_STPA_OPT2                        = PAR_IO_PIN(F, 20, OPTION_0, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_STPA_OPT1                        = PAR_IO_PIN(A, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_STPA_OPT2                        = PAR_IO_PIN(F, 20, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE1_OPT1                         = PAR_IO_PIN(A, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE1_OPT2                         = PAR_IO_PIN(C,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE2_OPT1                         = PAR_IO_PIN(A, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE2_OPT2                         = PAR_IO_PIN(C, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE3_OPT1                         = PAR_IO_PIN(B, 10, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE3_OPT2                         = PAR_IO_PIN(C, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE4_OPT1                         = PAR_IO_PIN(B, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE4_OPT2                         = PAR_IO_PIN(C, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE5_OPT1                         = PAR_IO_PIN(C, 28, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE5_OPT2                         = PAR_IO_PIN(D, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE6_OPT1                         = PAR_IO_PIN(C, 29, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE6_OPT2                         = PAR_IO_PIN(D, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE7_OPT1                         = PAR_IO_PIN(E, 10, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE7_OPT2                         = PAR_IO_PIN(E, 31, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE8_OPT1                         = PAR_IO_PIN(C,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI_STROBE8_OPT2                         = PAR_IO_PIN(E, 24, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR0                          = PAR_IO_PIN(E,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR0                          = PAR_IO_PIN(E,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR0_OPT1                     = PAR_IO_PIN(B,  4, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR0_OPT2                     = PAR_IO_PIN(G,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR0_OPT1                     = PAR_IO_PIN(B,  0, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR0_OPT2                     = PAR_IO_PIN(F,  5, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR1                          = PAR_IO_PIN(E,  8, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR1                          = PAR_IO_PIN(E,  5, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR1_OPT1                     = PAR_IO_PIN(D, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR1_OPT2                     = PAR_IO_PIN(G,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR1_OPT1                     = PAR_IO_PIN(A,  8, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR1_OPT2                     = PAR_IO_PIN(F,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR2                          = PAR_IO_PIN(E, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR2                          = PAR_IO_PIN(E,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR2_OPT1                     = PAR_IO_PIN(D, 16, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR2_OPT2                     = PAR_IO_PIN(G,  5, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR2_OPT1                     = PAR_IO_PIN(A, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR2_OPT2                     = PAR_IO_PIN(F, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR3                          = PAR_IO_PIN(E,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR3                          = PAR_IO_PIN(E, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR3_OPT1                     = PAR_IO_PIN(A, 16, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR3_OPT2                     = PAR_IO_PIN(F, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR3_OPT1                     = PAR_IO_PIN(A, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR3_OPT2                     = PAR_IO_PIN(F, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR4                          = PAR_IO_PIN(E, 13, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR4                          = PAR_IO_PIN(E,  7, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR4_OPT1                     = PAR_IO_PIN(E, 20, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_M_TXADDR4_OPT2                     = PAR_IO_PIN(G,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR4_OPT1                     = PAR_IO_PIN(C, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC2_S_TXADDR4_OPT2                     = PAR_IO_PIN(F, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXADDR5                            = PAR_IO_PIN(E,  6, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC2_TXADDR5                            = PAR_IO_PIN(C, 22, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD0_SER1_TXD0                    = PAR_IO_PIN(A,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD0_SER2_TXD0_OPT1               = PAR_IO_PIN(A, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD0_SER2_TXD0_OPT2               = PAR_IO_PIN(F,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD1_SER1_TXD1                    = PAR_IO_PIN(A,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD1_SER2_TXD1_OPT1               = PAR_IO_PIN(A, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD1_SER2_TXD1_OPT2               = PAR_IO_PIN(F,  6, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD2_SER1_TXD2                    = PAR_IO_PIN(A,  5, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD2_SER2_TXD2_OPT1               = PAR_IO_PIN(A, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD2_SER2_TXD2_OPT2               = PAR_IO_PIN(F, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD3_SER1_TXD3                    = PAR_IO_PIN(A,  6, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD3_SER2_TXD3_OPT1               = PAR_IO_PIN(A, 20, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD3_SER2_TXD3_OPT2               = PAR_IO_PIN(G, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD4_SER1_TXD4_OPT1               = PAR_IO_PIN(B,  6, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD4_SER1_TXD4_OPT2               = PAR_IO_PIN(C, 25, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD4_SER2_TXD4                    = PAR_IO_PIN(B,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TX_EN_SER2_RTS_B_OPT2             = PAR_IO_PIN(F, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD5_SER1_TXD5_OPT1               = PAR_IO_PIN(B,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD5_SER1_TXD5_OPT2               = PAR_IO_PIN(C, 24, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD5_SER2_TXD5                    = PAR_IO_PIN(B,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD6_SER1_TXD6_OPT1               = PAR_IO_PIN(B,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD6_SER1_TXD6_OPT2               = PAR_IO_PIN(C, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD6_SER2_TXD6                    = PAR_IO_PIN(B,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD7_SER1_TXD7_OPT1               = PAR_IO_PIN(B, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD7_SER1_TXD7_OPT2               = PAR_IO_PIN(C, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD7_SER2_TXD7                    = PAR_IO_PIN(B,  8, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TX_ER                             = PAR_IO_PIN(A,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TX_ER                             = PAR_IO_PIN(A, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_GTM1_TIN1_OPT1                          = PAR_IO_PIN(C,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN1_OPT2                          = PAR_IO_PIN(F,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN2_OPT1                          = PAR_IO_PIN(D, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_GTM1_TIN2_OPT2                          = PAR_IO_PIN(E, 31, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_TN_OPT1                             = PAR_IO_PIN(B,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_TN_OPT2                             = PAR_IO_PIN(G, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_TP_OPT1                             = PAR_IO_PIN(B,  3, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_TP_OPT2                             = PAR_IO_PIN(G, 12, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TSYNC_OPT1                         = PAR_IO_PIN(A,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_TSYNC_OPT2                         = PAR_IO_PIN(A, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_TSYNC_OPT1                         = PAR_IO_PIN(A, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_TSYNC_OPT2                         = PAR_IO_PIN(A, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_TSYNC_OPT1                         = PAR_IO_PIN(B,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_TSYNC_OPT2                         = PAR_IO_PIN(E,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_TSYNC_OPT1                         = PAR_IO_PIN(A, 18, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_TSYNC_OPT2                         = PAR_IO_PIN(B,  6, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_TSYNC_OPT3                         = PAR_IO_PIN(B, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_TSYNC_OPT1                         = PAR_IO_PIN(A, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_TSYNC_OPT2                         = PAR_IO_PIN(D,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDME_TSYNC_OPT3                         = PAR_IO_PIN(E, 15, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_TSYNC_OPT1                         = PAR_IO_PIN(A, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_TSYNC_OPT2                         = PAR_IO_PIN(C, 28, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMF_TSYNC_OPT3                         = PAR_IO_PIN(D, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_TSYNC_OPT1                         = PAR_IO_PIN(E,  4, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMH_TSYNC_OPT2                         = PAR_IO_PIN(E, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMG_TSYNC_OPT1                         = PAR_IO_PIN(D, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMG_TSYNC_OPT2                         = PAR_IO_PIN(E,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_TX_ER                             = PAR_IO_PIN(B,  5, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TX_ER                             = PAR_IO_PIN(B, 19, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TX_ER                             = PAR_IO_PIN(D,  5, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TX_ER                             = PAR_IO_PIN(D, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TX_ER                             = PAR_IO_PIN(E,  5, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TX_ER                             = PAR_IO_PIN(E, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD0_SER3_TXD0                    = PAR_IO_PIN(B,  0, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD0_SER4_TXD0_OPT1               = PAR_IO_PIN(B,  9, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD0_SER4_TXD0_OPT2               = PAR_IO_PIN(B, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD0_SER5_TXD0_OPT1               = PAR_IO_PIN(D,  0, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD0_SER5_TXD0_OPT2               = PAR_IO_PIN(E, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD0_SER6_TXD0                    = PAR_IO_PIN(D, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD0_SER7_TXD0                    = PAR_IO_PIN(E,  0, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD0_SER8_TXD0                    = PAR_IO_PIN(E, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD0_OPT1                          = PAR_IO_PIN(A, 13, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD0_OPT2                          = PAR_IO_PIN(B, 26, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD0_OPT1                          = PAR_IO_PIN(A, 22, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD0_OPT2                          = PAR_IO_PIN(C, 28, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD0_OPT1                          = PAR_IO_PIN(B,  8, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD0_OPT2                          = PAR_IO_PIN(E,  7, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD0_OPT1                          = PAR_IO_PIN(A, 19, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD0_OPT2                          = PAR_IO_PIN(B,  1, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD0_OPT3                          = PAR_IO_PIN(B, 19, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDME_TXD0_OPT1                          = PAR_IO_PIN(A, 30, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDME_TXD0_OPT2                          = PAR_IO_PIN(D,  5, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDME_TXD0_OPT3                          = PAR_IO_PIN(E, 18, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMF_TXD0_OPT1                          = PAR_IO_PIN(D, 22, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMF_TXD0_OPT2                          = PAR_IO_PIN(E, 10, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMG_TXD0_OPT1                          = PAR_IO_PIN(D,  3, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMG_TXD0_OPT2                          = PAR_IO_PIN(E,  5, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMH_TXD0_OPT1                          = PAR_IO_PIN(E,  0, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMH_TXD0_OPT2                          = PAR_IO_PIN(E, 22, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD1_SER3_TXD1                    = PAR_IO_PIN(B,  1, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD1_SER4_TXD1_OPT1               = PAR_IO_PIN(B, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD1_SER4_TXD1_OPT2               = PAR_IO_PIN(B, 15, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD1_SER5_TXD1_OPT1               = PAR_IO_PIN(D,  1, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD1_SER5_TXD1_OPT2               = PAR_IO_PIN(E, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD1_SER6_TXD1                    = PAR_IO_PIN(D, 15, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD1_SER7_TXD1                    = PAR_IO_PIN(E,  1, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD1_SER8_TXD1                    = PAR_IO_PIN(E, 15, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD1                               = PAR_IO_PIN(A,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD1                               = PAR_IO_PIN(A, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD1                               = PAR_IO_PIN(B,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD1                               = PAR_IO_PIN(B, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDME_TXD1                               = PAR_IO_PIN(D,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMF_TXD1                               = PAR_IO_PIN(D, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMH_TXD1                               = PAR_IO_PIN(E, 14, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD2_SER3_TXD2                    = PAR_IO_PIN(B,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD2_SER4_TXD2                    = PAR_IO_PIN(B, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD2_SER5_TXD2                    = PAR_IO_PIN(D,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD2_SER6_TXD2                    = PAR_IO_PIN(D, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD2_SER7_TXD2                    = PAR_IO_PIN(E,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD2_SER8_TXD2                    = PAR_IO_PIN(E, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD2                               = PAR_IO_PIN(A,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD2                               = PAR_IO_PIN(A, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD2                               = PAR_IO_PIN(B,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD2                               = PAR_IO_PIN(B, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDME_TXD2                               = PAR_IO_PIN(D,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMF_TXD2                               = PAR_IO_PIN(D, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMH_TXD2                               = PAR_IO_PIN(E, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD3_SER3_TXD3                    = PAR_IO_PIN(B,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD3_SER4_TXD3                    = PAR_IO_PIN(B, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD3_SER5_TXD3                    = PAR_IO_PIN(D,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD3_SER6_TXD3                    = PAR_IO_PIN(D, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD3_SER7_TXD3                    = PAR_IO_PIN(E,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD3_SER8_TXD3                    = PAR_IO_PIN(E, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD3                               = PAR_IO_PIN(A, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD3                               = PAR_IO_PIN(A, 30, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD3                               = PAR_IO_PIN(B, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD3                               = PAR_IO_PIN(B, 26, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDME_TXD3                               = PAR_IO_PIN(D, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMF_TXD3                               = PAR_IO_PIN(D, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMH_TXD3                               = PAR_IO_PIN(E, 27, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC1_CLKO                               = PAR_IO_PIN(C,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC2_CLKO                               = PAR_IO_PIN(C,  2, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC3_CLKO                               = PAR_IO_PIN(C, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC4_CLKO                               = PAR_IO_PIN(C,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC5_CLKO                               = PAR_IO_PIN(C, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC6_CLKO                               = PAR_IO_PIN(C,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC7_CLKO                               = PAR_IO_PIN(C, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC8_CLKO                               = PAR_IO_PIN(C,  4, OPTION_2, OUTPUT, FALSE)

} e_ParIoFunction;



#endif /* __PAR_IO_INTEGRATION_EXT_H */
