/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          MPC836x_ext.h

 @Description   MPC836x external definitions and structures.
*//***************************************************************************/

#ifndef __PART_INTEGRATION_EXT_H
#define __PART_INTEGRATION_EXT_H

#include "la83_ext.h"
#include "ipic_ext.h"
#include "qe_ic_ext.h"
#include "ddr_std_ext.h"


/**************************************************************************//**
 @Group         836x_chip_id MPC836x Application Programming Interface

 @Description   836x Chip functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Module types.
*//***************************************************************************/
typedef enum e_ModuleId
{
    e_MODULE_ID_LA,             /**< Local Access module */
    e_MODULE_ID_DDR_1,          /**< Primary DDR memory controller */
    e_MODULE_ID_DDR_2,          /**< Secondary DDR memory controller */
    e_MODULE_ID_I2C_1,          /**< I2C 1 */
    e_MODULE_ID_I2C_2,          /**< I2C 2 */
    e_MODULE_ID_LBC,            /**< Local bus memory controller module */
    e_MODULE_ID_PCI,            /**< PCI controller */
    e_MODULE_ID_DMA,            /**< DMA module */
    e_MODULE_ID_IPIC,           /**< Programmable interrupt controller */
    e_MODULE_ID_QE,             /**< QUICC Engine module */
    e_MODULE_ID_IRAM,           /**< Instruction RAM */
    e_MODULE_ID_PRAM,           /**< Parameter RAM */
    e_MODULE_ID_QE_IC,          /**< Quicc engine interrupt controller */
    e_MODULE_ID_USB,            /**< USB module */
    e_MODULE_ID_PCI_CFG,        /**< PCI configuration space */
    e_MODULE_ID_PCI_IOS,        /**< PCI scheduler space */
    e_MODULE_ID_SPI_1,          /**< SPI module 1 */
    e_MODULE_ID_SPI_2,          /**< SPI module 2 */
    e_MODULE_ID_DUART_1,        /**< DUART module 1 */
    e_MODULE_ID_DUART_2,        /**< DUART module 2 */
    e_MODULE_ID_UCC_1,          /**< UCC module 1 */
    e_MODULE_ID_UCC_2,          /**< UCC module 2 */
    e_MODULE_ID_UCC_3,          /**< UCC module 3 */
    e_MODULE_ID_UCC_4,          /**< UCC module 4 */
    e_MODULE_ID_UCC_5,          /**< UCC module 5 */
    e_MODULE_ID_UCC_6,          /**< UCC module 6 */
    e_MODULE_ID_UCC_7,          /**< UCC module 7 */
    e_MODULE_ID_UCC_8,          /**< UCC module 8 */
    e_MODULE_ID_RTC,            /**< Real time clock module */
    e_MODULE_ID_MCC,            /**< MCC module */
    e_MODULE_ID_SI,             /**< SI module */
    e_MODULE_ID_SI_RAM,         /**< SI RAM module */
    e_MODULE_ID_PAR_IO,         /**< Parallel I/O ports module */
    e_MODULE_ID_PAR_IO_INTR,    /**< Parallel I/O interrupts module */
    e_MODULE_ID_SEC,            /**< Security (talitos2) module */
    e_MODULE_ID_QE_TIMER,       /**< QE Timer module */
    e_MODULE_ID_GTM1_TIMER,     /**< Timer module 1 */
    e_MODULE_ID_GTM2_TIMER,     /**< Timer module 2 */
    e_MODULE_ID_SYSTEM_ARBITER, /**< Arbiter module */
    e_MODULE_ID_UPC_1,          /**< UPC module 1 */
    e_MODULE_ID_UPC_2,          /**< UPC module 2 */
    e_MODULE_ID_SDMA,           /**< Serial DMA */
    e_MODULE_ID_DEBUG,          /**< QE Debug module */
    e_MODULE_ID_RSP_1,          /**< Risc 1 special registers */
    e_MODULE_ID_RSP_2,          /**< Risc 2 special registers */
    e_MODULE_ID_TRB,            /**< QE trace-buffer module */
    e_MODULE_ID_QE_RTC,         /**< RTC QE module */
    e_MODULE_ID_PTP_1,          /**< PTP module 1 */
    e_MODULE_ID_PTP_2,          /**< PTP module 2 */
    e_MODULE_ID_DUMMY_LAST
} e_ModuleId;

#define NUM_OF_MODULES  e_MODULE_ID_DUMMY_LAST



/**************************************************************************//**
 @Description   Transaction source ID (for memory conrollers error reporting).
*//***************************************************************************/
typedef enum e_TransSrc
{
    e_TRANS_SRC_CORE_DATA            = 0,       /**< Processor (data) */
    e_TRANS_SRC_CORE_INST            = 0x2,     /**< Processor (instruction) */
    e_TRANS_SRC_SEC                  = 0x8,     /**< Security engine */
    e_TRANS_SRC_BOOT_SEQ             = 0x9,     /**< Boot sequencer */
    e_TRANS_SRC_JTAG                 = 0xA,     /**< JTAG */
    e_TRANS_SRC_PCI                  = 0xD,     /**< PCI */
    e_TRANS_SRC_DMA                  = 0xF,     /**< DMA */
    e_TRANS_SRC_QE_RISC0_SNUM0_CETM0 = 0x10,    /**< QE RISC 0, SNUM lsb 0, CETM bit 0 */
    e_TRANS_SRC_QE_RISC0_SNUM0_CETM1 = 0x11,    /**< QE RISC 0, SNUM lsb 0, CETM bit 1 */
    e_TRANS_SRC_QE_RISC0_SNUM1_CETM0 = 0x12,    /**< QE RISC 0, SNUM lsb 1, CETM bit 0 */
    e_TRANS_SRC_QE_RISC0_SNUM1_CETM1 = 0x13,    /**< QE RISC 0, SNUM lsb 1, CETM bit 1 */
    e_TRANS_SRC_QE_RISC1_SNUM0_CETM0 = 0x14,    /**< QE RISC 1, SNUM lsb 0, CETM bit 0 */
    e_TRANS_SRC_QE_RISC1_SNUM0_CETM1 = 0x15,    /**< QE RISC 1, SNUM lsb 0, CETM bit 1 */
    e_TRANS_SRC_QE_RISC1_SNUM1_CETM0 = 0x16,    /**< QE RISC 1, SNUM lsb 1, CETM bit 0 */
    e_TRANS_SRC_QE_RISC1_SNUM1_CETM1 = 0x17     /**< QE RISC 1, SNUM lsb 1, CETM bit 1 */
} e_TransSrc;


/***************************************************************
    MPC836x general routines
****************************************************************/
/**************************************************************************//**
 @Group         836x_init_grp MPC836x Initialization Unit

 @Description   MPC836x initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Part ID and revision number
*//***************************************************************************/
typedef enum e_MPC836xDeviceName
{
    e_MPC836x_REV_INVALID   = 0x00000000,   /**< Invalid revision      */
    e_MPC8360E_REV_1_0      = 0x80480010,   /**< MPC8360E revision 1.0 */
    e_MPC8360E_REV_1_1      = 0x80480011,   /**< MPC8360E revision 1.1 */
    e_MPC8360E_REV_1_2      = 0x80480012,   /**< MPC8360E revision 1.2 */
    e_MPC8360E_REV_2_0      = 0x80480020,   /**< MPC8360E revision 2.0 */
    e_MPC8360E_REV_2_1      = 0x80480021,   /**< MPC8360E revision 2.1 */
    e_MPC8360_REV_1_0       = 0x80490010,   /**< MPC8360 revision 1.0 */
    e_MPC8360_REV_1_1       = 0x80490011,   /**< MPC8360 revision 1.1 */
    e_MPC8360_REV_1_2       = 0x80490012,   /**< MPC8360 revision 1.2 */
    e_MPC8360_REV_2_0       = 0x80490020,   /**< MPC8360 revision 2.0 */
    e_MPC8360_REV_2_1       = 0x80490021    /**< MPC8360 revision 2.1 */
} e_MPC836xDeviceName;

/**************************************************************************//**
 @Description   System bus priority functions
*//***************************************************************************/
typedef enum e_MPC836xPriFunc
{
    e_MPC836x_PRI_PCI   = 0x03000000,   /**< PCI priority id */
    e_MPC836x_PRI_E300  = 0x00300000    /**< E300 priority id */
} e_MPC836xPriFunc;


#define MPC836X_NUM_OF_PIN_MUX_FUNC 9   /**< Number of pin-mux functions */

/**************************************************************************//**
 @Description   Pin muxing functions
*//***************************************************************************/
typedef enum e_MPC836xMuxFuncGrp
{
    e_MPC836x_NO_FUNC           = 0x00000000,   /**< No function */
    e_MPC836x_MUX_LDP           = 0xC0000000,   /**< LDP mux id */
    e_MPC836x_MUX_LCLK1         = 0x20000000,   /**< LCLK1 mux id */
    e_MPC836x_MUX_LCLK2         = 0x10000000,   /**< LCLK2 mux id */
    e_MPC836x_MUX_SRCID_A       = 0x03000000,   /**< SRCID_A mux id */
    e_MPC836x_MUX_IRQ_CKSTP_A   = 0x00C00000,   /**< IRQ_CKSTP_A mux id */
    /* used bit 31 as a register id (high/low) */
    e_MPC836x_MUX_DDR           = 0x80000001,   /**< DDR mux id */
    e_MPC836x_MUX_SEC_DDR       = 0x40000001,   /**< Secondary DDR mux id */
    e_MPC836x_MUX_SDDRIOE       = 0x20000001,   /**< SDDRIOE mux id */
    e_MPC836x_MUX_IRQ3          = 0x10000001    /**< IRQ3 mux id */
} e_MPC836xMuxFuncGrp;


/**************************************************************************//**
 @Description   Voltage values
*//***************************************************************************/
typedef enum e_VoltageVal
{
    e_MPC836x_VOLTAGE_VAL_3_3 ,
    e_MPC836x_VOLTAGE_VAL_2_5
}e_VoltageVal;

/**************************************************************************//**
 @Function      MPC836x_ConfigAndInit

 @Description   General initiation of the chip registers.

 @Param[in]     baseAddress - Memory map start

 @Return        A handle to the MPC836x data structure.
*//***************************************************************************/
t_Handle MPC836x_ConfigAndInit(uint32_t baseAddress);

/**************************************************************************//**
 @Function      MPC836x_Free

 @Description   Free all resources.

 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_Free(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_GetDdrMode
 
 @Description   returns the multiplication factor of the clock in for the DDR clock .
 
 @Param         h_MPC836x - (In) a handle to the MPC836x object.
 @Param       	p_DdrMode   - (Out) returns DDR mode 32/64 bit.
 
 @Return    	E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_GetDdrMode(t_Handle h_MPC836x,e_DdrMode *p_DdrMode );

/**************************************************************************//**
 @Function      MPC836x_GetModuleBase
 
 @Description   returns the base address of a MPC836x module's
				memory mapped registers.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]    	module      - Id of module as defined in part_ext.h.
 
 @Return    	Base address of module's memory mapped registers.
                ILLEGAL_BASE in case of non-existent module
*//***************************************************************************/
uint64_t MPC836x_GetModuleBase(t_Handle h_MPC836x, e_ModuleId module);

/**************************************************************************//**
 @Function      MPC836x_GetRevInfo
 
 @Description   This routine enables access to chip and revision information.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.
 
 @Return    	Part ID and revision.
*//***************************************************************************/
e_MPC836xDeviceName MPC836x_GetRevInfo(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_SetSystemPriority
 
 @Description   The routine gives a new priority to a specified
				chip functionality.
	
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]   	func        - Specified functionality group.
 @Param[in]   	pri         - Priority level: 0 (lowest) - 3 (highest).

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_SetSystemPriority(t_Handle h_MPC836x, e_MPC836xPriFunc func, uint8_t pri);

/**************************************************************************//**
 @Function      MPC836x_SetPCIHighestPri
 
 @Description   The routine causes the PCI to request system bus at highest priority,
 				regardless of current PCI priority.
	
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_SetPCIHighestPri(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_E300CoreTimeBaseEnable
 
 @Description   The routine enables E300 PowerPC core time base unit.
	
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_E300CoreTimeBaseEnable(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_E300CoreTimeBaseDisable
 
 @Description   The routine disables E300 PowerPC core time base unit.
	
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_E300CoreTimeBaseDisable(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_QeSecOptimize
 
 @Description   The routine may enhance the performance of transactions issued
 				to the internal Coherent System Bus by the Security Engine
 				(SEC) and the Quicc Engine (QE)
	
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_QeSecOptimize(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_SetSystemPinMux
 
 @Description   The routine sets a choice of pins group for a specified
				chip functionality.
	
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]   	func        - Specified functionality.
 @Param[in]   	choice      - One of 2 to 4 choices (0-3) as specified in
                              the User Manual, sections 5.4.3.5 System I/O
                              Configuration Register Low, 5.4.3.6 System
                              I/O Configuration Register High.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_SetSystemPinMux(t_Handle h_MPC836x, e_MPC836xMuxFuncGrp func, uint8_t choice);

/**************************************************************************//**
 @Function      MPC836x_GetCoreClkFactors

 @Description   Retrieves the multiplication and division factors of the core clock.

 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[out]    p_MulFactor - Core clock multiplication factor.
 @Param[out]    p_DivFactor - Core clock division factor.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_GetCoreClkFactors(t_Handle h_MPC836x, uint8_t *p_MulFactor, uint8_t *p_DivFactor);

/**************************************************************************//**
 @Function      MPC836x_GetSysClkFactors

 @Description   Retrieves the multiplication and division factors of the system clock.

 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[out]    p_MulFactor - System clock multiplication factor.
 @Param[out]    p_DivFactor - System clock division factor.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_GetSysClkFactors(t_Handle h_MPC836x, uint8_t *p_MulFactor, uint8_t *p_DivFactor);

/**************************************************************************//**
 @Function      MPC836x_GetDdrClkFactor

 @Description   Retrieves the multiplication factor of the DDR clock.

 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[out]    p_MulFactor - DDR clock multiplication factor.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_GetDdrClkFactor(t_Handle h_MPC836x, uint8_t *p_MulFactor);

/**************************************************************************//**
 @Function      MPC836x_GetQeClkFactors

 @Description   Retrieves the multiplication and division factors of the QE clock.

 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[out]    p_MulFactor - QE clock multiplication factor.
 @Param[out]    p_DivFactor - QE clock division factor.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_GetQeClkFactors(t_Handle h_MPC836x, uint8_t *p_MulFactor, uint8_t *p_DivFactor);

/**************************************************************************//**
 @Function      MPC836x_EnablePciClocks
 
 @Description   Enables PCI clocks.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_EnablePciClocks(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_GetPramSize
 
 @Description   Returns the size of the Multi-User RAM.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.
 
 @Return    	The size of the Multi-User RAM.
*//***************************************************************************/
uint32_t MPC836x_GetPramSize(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_LAWGet

 @Description   Allocation of a local access window.
 
               	The routine allocates memory space window according to the
               	requested size and owner.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]     id          - Target interface id.
 @Param[in]   	size        - Size of window, at least 4KB and no more than
                                   2GB,	Must be a power of 2.
 @Param[out]    p_Addr      - On success, returns base address allocated
                              for the window.

 @Return        Number of allocated window (0-7) or LA83_ILLEGAL_LAW if either
                size is illegal, No window available or no free memory of
                required size.
				In such a case, user may still call MPC836x_LAWForce and overlap
				existing windows, in which case memory areas are prioritized
				according to LAW order. User may also call MPC836x_LAWReport and
				rearrange LAW's.
*//***************************************************************************/
uint8_t MPC836x_LAWGet(t_Handle h_MPC836x, uint8_t id, uint32_t size, uint32_t *p_Addr);

/**************************************************************************//**
 @Function      MPC836x_LAWForce
 
 @Description  	Allocation of a local access window in a specific address.
 
               	The routine allocates memory space window at the requested
               	address, according to the requested size and owner.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]     id          - Target interface id.
 @Param[in]   	size        - Size of window, at least 4KB and no more than
                              2GB, must be a power of 2.
 @Param[in]   	address     - Base address for the window.
 
 @Return        Number of allocated window (0-7) or LA83_ILLEGAL_LAW
                If either size or address are illegal, No window
                available or failed to allocate a block of the
                requested size.
*//***************************************************************************/
uint8_t MPC836x_LAWForce(t_Handle h_MPC836x, uint8_t id, uint32_t size, uint32_t address);

/**************************************************************************//**
 @Function      MPC836x_LAWFree
 
 @Description  	Release of a local access window.
 
                The routine releases the memory block associated with a specific
                local access window, and disables this window. On termination,
                the routine modifies the current available window index to point
                to the freed window. This is useful in case all windows are busy.
                It may be of no significance or a little inefficient in other cases.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]     id          - Target interface id, for verifying ownership.
 @Param[in]     lawNum      - window number (0-7) as returned by the window
                              allocation routine.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_LAWFree(t_Handle h_MPC836x, uint8_t id, uint8_t lawNum);

/**************************************************************************//**
 @Function      MPC836x_LAWReport
 
 @Description  	The routine reports the current configuration of the memory
                space, as reflected by the 8 local access memory windows.
 
                An Array of 8 structures of type t_LawAttr is allocated and
                filled with attributes of current status of memory space.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]  	p_LawReport - A pointer to an empty structure which is an
                              array of 8 other structures each holding
                              attributes of a single window as described at
                              MPC836x_ext.h. Note that the structures are
                              ordered according to their location at the
                              memory space. Unused windows (disabled) are
                              listed at the end of the array and marked
                              with lawNum MPC836x_ILLEGAL_LAW.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_LAWReport(t_Handle h_MPC836x, t_LawReport *p_LawReport);

/**************************************************************************//**
 @Function      MPC836x_LAWFindId
 
 @Description  	The routine Checks for a specific
           		address, which window it falls into.
               	It does that by going through all windows
               	and comparing the address
               	to their start address and size.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]  	address     - Address for testing.
 @Param[out]    p_LawNum    - Returns the window index within all of the found
                              target's windows.
 
 @Return        Target id code as defined in 'LA83_ext.h' or LA83_ID_NONE
                if no match.
*//***************************************************************************/
uint8_t MPC836x_LAWFindId(t_Handle h_MPC836x, uint32_t address, uint8_t *p_LawNum);

/**************************************************************************//**
 @Function      MPC836x_GetMiiOffset
 
 @Description   Returns the offset of MII management configuration control
                registers from the UCC memory map.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.
 
 @Return        Offset of MII management configuration control registers from
                the UCC memory map
*//***************************************************************************/
uint32_t MPC836x_GetMiiOffset(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_EnableQeSecBusLBIUWindow
 
 @Description   Enables the LBIU access window on the QE secondary bus.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]  	baseAddress - The base address of the access window.
                              The 12 least significant bits are zeroed.
 @Param[in]  	size        - The size of the access window (Minimum 4KB).
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_EnableQeSecBusLBIUWindow(t_Handle h_MPC836x, uint32_t baseAddress, uint32_t size);

/**************************************************************************//**
 @Function      MPC836x_EnableQeSecBusDDRWindow
 
 @Description   Enables the secondary DDR access window on the QE secondary bus.
 
 @Param[in]  	h_MPC836x   - The handle of the initialized MPC836x object.
 @Param[in]  	baseAddress - The base address of the access window.
                              The 12 least significant bits are zeroed.
 @Param[in]  	size        - The size of the access window (Minimum 4KB).
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_EnableQeSecBusDDRWindow(t_Handle h_MPC836x, uint32_t baseAddress, uint32_t size);

/**************************************************************************//**
 @Function      MPC836x_DisableQeSecBusLBIUWindow
 
 @Description   Disables the LBIU access window on the QE secondary bus.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_DisableQeSecBusLBIUWindow(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_DisableQeSecBusDDRWindow
 
 @Description   Disables the secondary DDR access window on the QE secondary bus.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC836x_DisableQeSecBusDDRWindow(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_Reset
 
 @Description   Reset the chip.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.
*//***************************************************************************/
void MPC836x_Reset(t_Handle h_MPC836x);

/**************************************************************************//**
 @Function      MPC836x_EnableIpsecExtRequest
 
 @Description   connect the the SEC to the QE in order to suppoert IPsec Ucode.
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.

 @Param[in]  	extReqNum - Number of external request.
*//***************************************************************************/
t_Error MPC836x_EnableIpsecExtRequest(t_Handle h_MPC836x,uint8_t extReqNum);

/**************************************************************************//**
 @Function      MPC836x_SetEthernetVoltage
 
 @Description   Set the correct ethernet voltage (important in case of reduced modes - RGMII and RTBI).
 
 @Param[in]  	h_MPC836x - The handle of the initialized MPC836x object.
 @Param[in]  	moduleId - module identifier.
 @Param[in]  	voltageVal - The required voltage values.
*//***************************************************************************/
void MPC836x_SetEthernetVoltage(t_Handle h_MPC836x, e_ModuleId moduleId, e_VoltageVal voltageVal);


/** @} */ /* end of 836x_init_grp group */
/** @} */ /* end of 836x_chip_id group */


#define CORE_E300

#define INTG_MAX_NUM_OF_CORES   1


/*****************************************************************************
 INTEGRATION-SPECIFIC MODULE CODES
******************************************************************************/
#define MODULE_UNKNOWN          0x00000000
#define MODULE_MPC836x          0x00010000
#define MODULE_QE               0x00020000
#define MODULE_MCC              0x00030000
#define MODULE_MCC_TRANS        0x00040000
#define MODULE_MCC_HDLC         0x00050000
#define MODULE_MCC_SS7          0x00060000
#define MODULE_UCC              0x00070000
#define MODULE_UCC_ATM          0x00080000
#define MODULE_UCC_ETH          0x00090000
#define MODULE_UCC_UART         0x000a0000
#define MODULE_UCC_TRANS        0x000b0000
#define MODULE_UCC_HDLC         0x000c0000
#define MODULE_UCC_QMC          0x000d0000
#define MODULE_UCC_POS          0x000e0000
#define MODULE_MTC              0x000f0000
#define MODULE_DUART            0x00100000
#define MODULE_SPI              0x00110000
#define MODULE_TDM              0x00120000
#define MODULE_SEC              0x00130000
#define MODULE_PCI              0x00140000
#define MODULE_USB              0x00150000
#define MODULE_I2C              0x00160000
#define MODULE_PIO              0x00170000
#define MODULE_UPC              0x00180000
#define MODULE_FLTR             0x00190000
#define MODULE_IW               0x001a0000
#define MODULE_IMA              0x001b0000
#define MODULE_PPPOHT           0x001c0000
#define MODULE_L2SWITCH         0x001d0000
#define MODULE_DMA              0x001e0000
#define MODULE_LA               0x001f0000
#define MODULE_DDR              0x00200000
#define MODULE_LBC              0x00210000
#define MODULE_IPIC             0x00220000
#define MODULE_QE_IC            0x00230000
#define MODULE_MEM              0x00240000
#define MODULE_MM               0x00250000
#define MODULE_MMU              0x00260000
#define MODULE_CES              0x00270000
#define MODULE_RTC              0x00280000
#define MODULE_QE_RTC           0x00290000
#define MODULE_PTP              0x002a0000
#define MODULE_IW_PPPOHT        0x002b0000
#define MODULE_IPHC             0x002c0000
#define MODULE_GTIMERS          0x002d0000
#define MODULE_MII              0x002e0000
#define MODULE_VP				0x002f0000
#define MODULE_PW               0x00300000
#define MODULE_ARBITER          0x00310000
#define MODULE_CORE             0x00320000
#define MODULE_IPSEC            0x00330000



/*****************************************************************************
 IPIC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/

#define IPIC_NUM_OF_EXTERNAL_IRQS   8
#define IPIC_PARTIAL_INTERNAL_PRIORITY_GROUPS
#define IPIC_NO_EXT_IRQ_POLARITY_SUPPORT

/**************************************************************************//**
 @Group         ipic_id IPIC Application Programming Interface

 @Description   IPIC API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ipic_init_grp IPIC Initialization Unit

 @Description   IPIC initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Enumeration of IPIC interrupts
*//***************************************************************************/
typedef enum e_IpicIntrSrc
{
	e_IPIC_INTR_DUART_1     = 9,        /**< IPIC interrupt source for DUART1           */
	e_IPIC_INTR_DUART_2     = 10,       /**< IPIC interrupt source for DUART2           */
	e_IPIC_INTR_SEC         = 11,       /**< IPIC interrupt source for SEC              */
	e_IPIC_INTR_I2C_1       = 14,       /**< IPIC interrupt source for I2C1             */
	e_IPIC_INTR_I2C_2       = 15,       /**< IPIC interrupt source for I2C2             */
	e_IPIC_INTR_IRQ_1       = 17,       /**< IPIC interrupt source for IRQ1             */
	e_IPIC_INTR_IRQ_2       = 18,       /**< IPIC interrupt source for IRQ2             */
	e_IPIC_INTR_IRQ_3       = 19,       /**< IPIC interrupt source for IRQ3             */
	e_IPIC_INTR_IRQ_4       = 20,       /**< IPIC interrupt source for IRQ4             */
	e_IPIC_INTR_IRQ_5       = 21,       /**< IPIC interrupt source for IRQ5             */
	e_IPIC_INTR_IRQ_6       = 22,       /**< IPIC interrupt source for IRQ6             */
    e_IPIC_INTR_IRQ_7       = 23,       /**< IPIC interrupt source for IRQ7             */
    e_IPIC_INTR_QE_HIGH     = 32,       /**< IPIC interrupt source for QE (High)        */
    e_IPIC_INTR_QE_LOW      = 33,       /**< IPIC interrupt source for QE (Low)         */
	e_IPIC_INTR_IRQ_0       = 48,       /**< IPIC interrupt source for IRQ0             */
	e_IPIC_INTR_RTC_COUNT   = 64,       /**< IPIC interrupt source for RTC seconds      */
	e_IPIC_INTR_PIT         = 65,       /**< IPIC interrupt source for PIT              */
    e_IPIC_INTR_PCI         = 66,       /**< IPIC interrupt source for PCI              */
	e_IPIC_INTR_RTC_ALARM   = 68,       /**< IPIC interrupt source for RTC alarm        */
	e_IPIC_INTR_MU          = 69,       /**< IPIC interrupt source for MU               */
	e_IPIC_INTR_ARBITER     = 70,       /**< IPIC interrupt source for SBA              */
	e_IPIC_INTR_DMA         = 71,       /**< IPIC interrupt source for DMA              */
	e_IPIC_INTR_GTM_4       = 72,       /**< IPIC interrupt source for GTM4             */
	e_IPIC_INTR_GTM_8       = 73,       /**< IPIC interrupt source for GTM8             */
	e_IPIC_INTR_QE_IO_PORTS = 74,       /**< IPIC interrupt source for QE I/O ports     */
	e_IPIC_INTR_DDR_2       = 75,       /**< IPIC interrupt source for Secondary DDR    */
    e_IPIC_INTR_DDR_1       = 76,       /**< IPIC interrupt source for Primary DDR      */
    e_IPIC_INTR_LBC         = 77,       /**< IPIC interrupt source for LBIU             */
	e_IPIC_INTR_GTM_2       = 78,       /**< IPIC interrupt source for GTM2             */
	e_IPIC_INTR_GTM_6       = 79,       /**< IPIC interrupt source for GTM6             */
	e_IPIC_INTR_PMC         = 80,       /**< IPIC interrupt source for PMC              */
	e_IPIC_INTR_GTM_3       = 84,       /**< IPIC interrupt source for GTM3             */
	e_IPIC_INTR_GTM_7       = 85,       /**< IPIC interrupt source for GTM7             */
	e_IPIC_INTR_GTM_1       = 90,       /**< IPIC interrupt source for GTM1             */
	e_IPIC_INTR_GTM_5       = 91,       /**< IPIC interrupt source for GTM5             */
    e_IPIC_INTR_DUMMY_LAST

} e_IpicIntrSrc;

/** @} */ /* end of ipic_init_grp group */

/**************************************************************************//**
 @Group         ipic_exception_grp IPIC Exception Unit

 @Description   IPIC Exception unit API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Anchor        ipic_mcpbm
 
 @Collection    IPIC Machine Check Events Bit Mask
 
                These are machine check events reported through the optional
                machine check callback routine. These events are non-maskable.
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define IPIC_MACHINE_CHECK_EVENT_IRQ0   0x80000000  /**< IRQ0 event (valid only if
                                                         configured as an external
                                                         MCP interrupt using the
                                                         IPIC_ModifyIrq0Mcp() routine). */
#define IPIC_MACHINE_CHECK_EVENT_WDT    0x40000000  /**< Watchdog timer event */
#define IPIC_MACHINE_CHECK_EVENT_SBA    0x20000000  /**< System bus arbiter event */
#define IPIC_MACHINE_CHECK_EVENT_CIEE   0x10000000  /**< QE instruction RAM double
                                                         ECC error */
#define IPIC_MACHINE_CHECK_EVENT_CMEE   0x08000000  /**< QE multi-user RAM double
                                                         ECC error */
#define IPIC_MACHINE_CHECK_EVENT_PCI    0x04000000  /**< PCI event */
#define IPIC_MACHINE_CHECK_EVENT_MU     0x01000000  /**< Messaging unit event */

/* @} */

/** @} */ /* end of ipic_exception_grp group */
/** @} */ /* end of ipic_id_grp group */

#define IPIC_OPTIMIZATION_FOR_QE


/*****************************************************************************
 QE IC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/

/**************************************************************************//**
 @Description   Enumeration of QE interrupts
*//***************************************************************************/
typedef enum e_QeIcIntrSrc
{
    e_QE_IC_INTR_SPI_2      = 1,    /**< QE interrupt source for SPI2               */
    e_QE_IC_INTR_SPI_1      = 2,    /**< QE interrupt source for SPI1               */
    e_QE_IC_INTR_RTT        = 3,    /**< QE interrupt source for the RISC timers    */
    e_QE_IC_INTR_SDMA       = 10,   /**< QE interrupt source for SDMA               */
    e_QE_IC_INTR_USB        = 11,   /**< QE interrupt source for USB                */
    e_QE_IC_INTR_TIMER_1    = 12,   /**< QE interrupt source for general timer 1    */
    e_QE_IC_INTR_TIMER_2    = 13,   /**< QE interrupt source for general timer 2    */
    e_QE_IC_INTR_TIMER_3    = 14,   /**< QE interrupt source for general timer 3    */
    e_QE_IC_INTR_TIMER_4    = 15,   /**< QE interrupt source for general timer 4    */
    e_QE_IC_INTR_PTP_1      = 17,   /**< QE interrupt source for PTP1               */
    e_QE_IC_INTR_PTP_2      = 19,   /**< QE interrupt source for PTP2               */
    e_QE_IC_INTR_VT         = 20,   /**< QE interrupt source for virtual tasks      */
    e_QE_IC_INTR_RTC        = 21,   /**< QE interrupt source for PTP RTC            */
    e_QE_IC_INTR_EXT_REQ_1  = 25,   /**< QE interrupt source for external request 1 */
    e_QE_IC_INTR_EXT_REQ_2  = 26,   /**< QE interrupt source for external request 2 */
    e_QE_IC_INTR_EXT_REQ_3  = 27,   /**< QE interrupt source for external request 3 */
    e_QE_IC_INTR_EXT_REQ_4  = 28,   /**< QE interrupt source for external request 4 */
    e_QE_IC_INTR_UCC_1      = 32,   /**< QE interrupt source for UCC1               */
    e_QE_IC_INTR_UCC_2      = 33,   /**< QE interrupt source for UCC2               */
    e_QE_IC_INTR_UCC_3      = 34,   /**< QE interrupt source for UCC3               */
    e_QE_IC_INTR_UCC_4      = 35,   /**< QE interrupt source for UCC4               */
    e_QE_IC_INTR_MCC        = 36,   /**< QE interrupt source for MCC                */
    e_QE_IC_INTR_UCC_5      = 40,   /**< QE interrupt source for UCC5               */
    e_QE_IC_INTR_UCC_6      = 41,   /**< QE interrupt source for UCC6               */
    e_QE_IC_INTR_UCC_7      = 42,   /**< QE interrupt source for UCC7               */
    e_QE_IC_INTR_UCC_8      = 43,   /**< QE interrupt source for UCC8               */
    e_QE_IC_INTR_DUMMY_LAST
 
} e_QeIcIntrSrc;


/*****************************************************************************
 LBC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/**************************************************************************//**
 @Group         lbc_exception_grp LBC Exception Unit

 @Description   LBC Exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        lbc_exbm
 
 @Collection    LBC Errors Bit Mask
 
                These errors are reported through the exceptions callback..
                The values can be or'ed in any combination in the errors mask
                parameter of the errors report structure.
 
                These errors can also be passed as a bit-mask to
                LBC_EnableErrorChecking() or LBC_DisableErrorChecking(),
                for enabling or disabling error checking.
 @{
*//***************************************************************************/
#define LBC_ERR_BUS_MONITOR     0x80000000  /**< Bus monitor error */
#define LBC_ERR_PARITY_ECC      0x20000000  /**< Parity error for GPCM/UPM */
#define LBC_ERR_WRITE_PROTECT   0x04000000  /**< Write protection error */
#define LBC_ERR_ATOMIC_WRITE    0x00800000  /**< Atomic write error */
#define LBC_ERR_ATOMIC_READ     0x00400000  /**< Atomic read error */
#define LBC_ERR_CHIP_SELECT     0x00080000  /**< Unrecognized chip select */

#define LBC_ERR_ALL             (LBC_ERR_BUS_MONITOR | LBC_ERR_PARITY_ECC | \
                                 LBC_ERR_WRITE_PROTECT | LBC_ERR_ATOMIC_WRITE | \
                                 LBC_ERR_ATOMIC_READ | LBC_ERR_CHIP_SELECT)
                                            /**< All possible errors */
/* @} */
/** @} */ /* end of lbc_exception_grp group */

#define LBC_NUM_OF_BANKS            8
#define LBC_MAX_CS_SIZE             0x80000000
#define LBC_SDRAM_MACHINE_AVAILABLE
#define LBC_32BIT_PORT_SIZE_SUPPORT
#define LBC_ATOMIC_OPERATION_SUPPORT
#define LBC_PARITY_SUPPORT


/*****************************************************************************
 SPI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SPI_QE_INTEGRATION
#define SPI_NUM_OF_CONTROLLERS          2


/*****************************************************************************
 UCC GETH INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define UCC_GETH_INTERFACES_MAP \
    { { e_ENET_IF_MII,      { 1, 1, 1, 1, 1, 1, 1, 1 } }, \
      { e_ENET_IF_RMII,     { 1, 1, 1, 1, 1, 1, 1, 1 } }, \
      { e_ENET_IF_RGMII,    { 1, 1, 0, 0, 0, 0, 0, 0 } }, \
      { e_ENET_IF_GMII,     { 1, 1, 0, 0, 0, 0, 0, 0 } }, \
      { e_ENET_IF_TBI,      { 1, 1, 0, 0, 0, 0, 0, 0 } }, \
      { e_ENET_IF_RTBI,     { 1, 1, 0, 0, 0, 0, 0, 0 } } }
 
#define UCC_GETH_PTP_SUPPORT


/*****************************************************************************
 UCC UART INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define UCC_UART_SW_WORKAROUND


/*****************************************************************************
 ARBITER INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/**************************************************************************//**
 @Group         arbiter_init_grp Arbiter Initialization Unit

 @Description   Arbiter initialization routines

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         arbiter_advanced_init_grp  Arbiter Advanced Configuration Unit

 @Description   Configuration functions used to change default values.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Parking Master
 
                Specifies arbiter parking master.
*//***************************************************************************/
typedef enum e_ArbiterParkingMaster
{
	e_ARBITER_PARK_MASTER_E300_CORE         = 0x0,   /**< Parking master e300 core */
	e_ARBITER_PARK_MASTER_PCI_DMA           = 0x1,   /**< Parking master DMA PCI */
	e_ARBITER_PARK_MASTER_I2C_JTAG_TPR      = 0x2,   /**< Parking master I2C, JTAG, TPR */
	e_ARBITER_PARK_MASTER_QUICC_ENGINE      = 0x3,   /**< Parking master QUICC Engine */
	e_ARBITER_PARK_MASTER_ENCRYPTION_CORE   = 0x4,   /**< Parking master Encryption core */
    e_ARBITER_PARK_MASTER_DUMMY_LAST        = 0x5
} e_ArbiterParkingMaster;

/** @} */ /* end of arbiter_advanced_init_grp */
/** @} */ /* end of arbiter_init_grp */

#define ARBITER_PARK_MASTER_DEFAULT_VALUE   e_ARBITER_PARK_MASTER_QUICC_ENGINE
 
#define ARBITER_TIMEOUT_GRANULARITY     128

/* Arbiter Timers Register Bit definitions
 * ====================================
 */
#define ARBITER_ATR_DTO_MASK        0xFFFF0000  /* Core disable */
#define ARBITER_ATR_DTO_SHIFT       16
#define ARBITER_ATR_ATO_MASK        0x0000FFFF	/* Pipeline depth */
#define ARBITER_ATR_MAX_TIMER       0xFFFFFFFF  /* Timers' maximum value */
#define ARBITER_ATR_MAX_VALUE       0xFFFF


/*****************************************************************************
 PCI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/* PCI modules registration */
void PCI_CTRL_Register(t_Handle h_PciCtrl);

#define PCI_MODULES_REGISTRATION \
{ \
    { e_MODULE_ID_PCI,  PCI_CTRL_Register, e_MODULE_ID_PCI_CFG, e_MODULE_ID_PCI_IOS, e_MODULE_ID_DUMMY_LAST } \
}

#define PCI_NUM_OF_BUSSES               1
#define PCI_MAX_CTRL_NUM                1
#define PCI_MAX_INBOUND_WINDOWS_NUM     4
#define PCI_MAX_OUTBOUND_WINDOWS_NUM    5

/**************************************************************************//**
 @Description   Target interface of an inbound window
*//***************************************************************************/
typedef enum e_PciTargetInterface
{
    e_PCI_TARGET_PCI            = 0x0,  /**<  PCI target interface                                       */
    e_PCI_TARGET_PCIE           = 0x2,  /**<  PCI Express target interface                               */
    e_PCI_TARGET_SRIO           = 0xC,  /**<  SRIO target interface                                      */
    e_PCI_TARGET_LOCAL_MEMORY   = 0xF   /**<  Local Memory (DDR SDRAM, Local Bus, SRAM) target interface */
 
} e_PciTargetInterface;


/*****************************************************************************
 DDR INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define DDR_NUM_OF_VALID_CS     4

/*****************************************************************************
 DMA INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define DMA_NUM_OF_CONTROLLERS  1

/*****************************************************************************
 GETH INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/* Note: the define is related to QE_ENET22 errata */
#define QE_ENET_RX_MAY_STUCK_ERRATA

/* Note: the define is related to QE_GENERAL_A003 errata */
#define QE_ENET_HIGH_TX_VFIFO_THRESHOLD_ERRATA

/*****************************************************************************
 UPC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define UPC_NUM_OF_DEVS         4

/*****************************************************************************
 QMC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define QE_QMC_REINIT_ERRATA
/* Note: the define is related to QE_QMC3 errata */
#define QE_QMC_NO_RX_INTR_AFTER_STOP_RX_ERRATA

/*****************************************************************************
SI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SI_MODULE_ID_MAP      \
    { { e_MODULE_ID_SI, 0 } } \
 
/*****************************************************************************
 TDM INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
//#define TDM_DISABLE_BUG_RTSYNC_WORKAROUND
#define TDM_DISABLE_BUG_NEUTRALIZE_WORKAROUND_ERRATA
#define TDM_HIGH_SPEED_MODE_SUPPORT

/*****************************************************************************
 MCC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define MCC_INIT_CMD_BUG_WORKAROUND_ERRATA

/*****************************************************************************
 QE IW INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define QE_IW_IPREASS_FULL_UNAVAILABLE
#define QE_IW_IPSEC_UNAVAILABLE

/*****************************************************************************
 SEC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SEC_NUM_OF_CHANNELS     4


/*****************************************************************************
 QE INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define PTP_QE_TX_ACCELERATION


#endif /* __PART_INTEGRATION_EXT_H */

