/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          qe_integration_ext.h

 @Description   Integration-specific header file for QE unit routines.
*//***************************************************************************/

#ifndef __QE_INTEGRATION_EXT_H
#define __QE_INTEGRATION_EXT_H

#include "std_ext.h"


#define QE_VALID_UID_MASK           0x0100
#define QE_VALID_DEV_MASK           0x0060
#define QE_VALID_PORT_MASK          0x001F

/**************************************************************************//**
 @Group         qe__grp QUICC Engine Application Programming Interface

 @Description   QE API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         qe_init_grp QE Initialization Unit

 @Description   QE Initialization Unit Functions and definitions

 @{
*//***************************************************************************/

#define QE_MURAM_SIZE                       0xC000  /**< Size of MURAM (in bytes) */

#define QE_SDMA_MAX_INTERNAL_BUFFER_BLOCKS  6       /**< Maximum number of
                                                         512-bytes blocks for
                                                         internal SDMA buffer */

#define QE_NUM_OF_MCCS                      1       /**< Number of MCC controllers */


/**************************************************************************//**
 @Description   Enumeration of UCC instances.
*//***************************************************************************/
typedef enum e_Ucc
{
    e_UCC_1 = 0     /**< UCC 1 */
    , e_UCC_2       /**< UCC 2 */
    , e_UCC_3       /**< UCC 3 */
    , e_UCC_4       /**< UCC 4 */
    , e_UCC_5       /**< UCC 5 */
    , e_UCC_6       /**< UCC 6 */
    , e_UCC_7       /**< UCC 7 */
    , e_UCC_8       /**< UCC 8 */
    , e_UCC_DUMMY_LAST
} e_Ucc;

/**************************************************************************//**
 @Description   Enumeration of SI instances.
*//***************************************************************************/
typedef enum e_Si
{
    e_SI_1 = 0      /**< SI 1 */
    , e_SI_DUMMY_LAST
} e_Si;

/**************************************************************************//**
 @Description   Enumeration of MCC instances.
*//***************************************************************************/
typedef enum e_Mcc
{
    e_MCC_1 = 0     /**< MCC 1 */
    , e_MCC_DUMMY_LAST
} e_Mcc;

/**************************************************************************//**
 @Description   Enumeration of UPC instances.
*//***************************************************************************/
typedef enum e_Upc
{
    e_UPC_1 = 0     /**< UPC 1 */
    , e_UPC_2       /**< UPC 2 */
    , e_UPC_DUMMY_LAST
} e_Upc;

/**************************************************************************//**
 @Description   Enumeration of USB instances.
*//***************************************************************************/
typedef enum e_Usb
{
    e_USB_1 = 0     /**< USB 1 */
    , e_USB_DUMMY_LAST
} e_Usb;

/**************************************************************************//**
 @Description   Enumeration of TDM instances.
*//***************************************************************************/
typedef enum e_Tdm
{
    e_TDM_A = 0     /**< TDM A */
    , e_TDM_B       /**< TDM B */
    , e_TDM_C       /**< TDM C */
    , e_TDM_D       /**< TDM D */
    , e_TDM_E       /**< TDM E */
    , e_TDM_F       /**< TDM F */
    , e_TDM_G       /**< TDM G */
    , e_TDM_H       /**< TDM H */
    , e_TDM_DUMMY_LAST
} e_Tdm;


/**************************************************************************//**
 @Collection    RISC allocation macros
 @{
*//***************************************************************************/
#define QE_NUM_OF_RISCS         2                       /**< Number of QE RISCs */

#define QE_RISC_ALLOC_RISC(Ri)  (1 << ((Ri)-1))         /**< Allocate RISC i */

#define QE_RISC_ALLOC_ALL_RISCS \
        (QE_RISC_ALLOC_RISC(1)|QE_RISC_ALLOC_RISC(2))   /**< Allocate all RISCs */
/* @} */


/**************************************************************************//**
 @Description   Clock Signals used by the QE MUX
*//***************************************************************************/
typedef enum e_QeClockSignal
{
    e_QE_CLK_SIGNAL_NONE,
    e_QE_CLK_SIGNAL_TDM_A_RX,
    e_QE_CLK_SIGNAL_TDM_A_TX,
    e_QE_CLK_SIGNAL_TDM_A_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_A_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_B_RX,
    e_QE_CLK_SIGNAL_TDM_B_TX,
    e_QE_CLK_SIGNAL_TDM_B_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_B_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_C_RX,
    e_QE_CLK_SIGNAL_TDM_C_TX,
    e_QE_CLK_SIGNAL_TDM_C_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_C_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_D_RX,
    e_QE_CLK_SIGNAL_TDM_D_TX,
    e_QE_CLK_SIGNAL_TDM_D_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_D_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_E_RX,
    e_QE_CLK_SIGNAL_TDM_E_TX,
    e_QE_CLK_SIGNAL_TDM_E_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_E_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_F_RX,
    e_QE_CLK_SIGNAL_TDM_F_TX,
    e_QE_CLK_SIGNAL_TDM_F_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_F_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_G_RX,
    e_QE_CLK_SIGNAL_TDM_G_TX,
    e_QE_CLK_SIGNAL_TDM_G_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_G_TX_SYNC,
    e_QE_CLK_SIGNAL_TDM_H_RX,
    e_QE_CLK_SIGNAL_TDM_H_TX,
    e_QE_CLK_SIGNAL_TDM_H_RX_SYNC,
    e_QE_CLK_SIGNAL_TDM_H_TX_SYNC,
    e_QE_CLK_SIGNAL_UCC_1_RX,
    e_QE_CLK_SIGNAL_UCC_1_TX,
    e_QE_CLK_SIGNAL_UCC_2_RX,
    e_QE_CLK_SIGNAL_UCC_2_TX,
    e_QE_CLK_SIGNAL_UCC_3_RX,
    e_QE_CLK_SIGNAL_UCC_3_TX,
    e_QE_CLK_SIGNAL_UCC_4_RX,
    e_QE_CLK_SIGNAL_UCC_4_TX,
    e_QE_CLK_SIGNAL_UCC_5_RX,
    e_QE_CLK_SIGNAL_UCC_5_TX,
    e_QE_CLK_SIGNAL_UCC_6_RX,
    e_QE_CLK_SIGNAL_UCC_6_TX,
    e_QE_CLK_SIGNAL_UCC_7_RX,
    e_QE_CLK_SIGNAL_UCC_7_TX,
    e_QE_CLK_SIGNAL_UCC_8_RX,
    e_QE_CLK_SIGNAL_UCC_8_TX,
    e_QE_CLK_SIGNAL_UPC_1_RX,
    e_QE_CLK_SIGNAL_UPC_1_TX,
    e_QE_CLK_SIGNAL_UPC_2_RX,
    e_QE_CLK_SIGNAL_UPC_2_TX,
    e_QE_CLK_SIGNAL_UPC_1_INT_RATE,
    e_QE_CLK_SIGNAL_UPC_2_INT_RATE,
    e_QE_CLK_SIGNAL_USB,
    e_QE_CLK_SIGNAL_RTC,
    e_QE_CLK_SIGNAL_DUMMY_LAST
} e_QeClockSignal;


 /**************************************************************************//**
 @Description   Clocks and BRG's
*//***************************************************************************/
typedef enum e_QeClock
{
    e_QE_CLK_NONE = 0
    , e_QE_BRG1             /**< Baud Rate Generator  1 */
    , e_QE_BRG2             /**< Baud Rate Generator  2 */
    , e_QE_BRG3             /**< Baud Rate Generator  3 */
    , e_QE_BRG4             /**< Baud Rate Generator  4 */
    , e_QE_BRG5             /**< Baud Rate Generator  5 */
    , e_QE_BRG6             /**< Baud Rate Generator  6 */
    , e_QE_BRG7             /**< Baud Rate Generator  7 */
    , e_QE_BRG8             /**< Baud Rate Generator  8 */
    , e_QE_BRG9             /**< Baud Rate Generator  9 */
    , e_QE_BRG10            /**< Baud Rate Generator 10 */
    , e_QE_BRG11            /**< Baud Rate Generator 11 */
    , e_QE_BRG12            /**< Baud Rate Generator 12 */
    , e_QE_BRG13            /**< Baud Rate Generator 13 */
    , e_QE_BRG14            /**< Baud Rate Generator 14 */
    , e_QE_BRG15            /**< Baud Rate Generator 15 */
    , e_QE_BRG16            /**< Baud Rate Generator 16 */
    , e_QE_CLK1             /**< Clock  1               */
    , e_QE_CLK2             /**< Clock  2               */
    , e_QE_CLK3             /**< Clock  3               */
    , e_QE_CLK4             /**< Clock  4               */
    , e_QE_CLK5             /**< Clock  5               */
    , e_QE_CLK6             /**< Clock  6               */
    , e_QE_CLK7             /**< Clock  7               */
    , e_QE_CLK8             /**< Clock  8               */
    , e_QE_CLK9             /**< Clock  9               */
    , e_QE_CLK10            /**< Clock 10               */
    , e_QE_CLK11            /**< Clock 11               */
    , e_QE_CLK12            /**< Clock 12               */
    , e_QE_CLK13            /**< Clock 13               */
    , e_QE_CLK14            /**< Clock 14               */
    , e_QE_CLK15            /**< Clock 15               */
    , e_QE_CLK16            /**< Clock 16               */
    , e_QE_CLK17            /**< Clock 17               */
    , e_QE_CLK18            /**< Clock 18               */
    , e_QE_CLK19            /**< Clock 19               */
    , e_QE_CLK20            /**< Clock 20               */
    , e_QE_CLK21            /**< Clock 21               */
    , e_QE_CLK22            /**< Clock 22               */
    , e_QE_CLK23            /**< Clock 23               */
    , e_QE_CLK24            /**< Clock 24               */
    , e_QE_CLK_EXT_RTC      /**< External RTC           */
    , e_QE_CLK_GRX          /**< GRX clock              */
    , e_QE_CLK_TBI_RX_CLK1  /**< TBI RX CLK1            */
    , e_QE_CLK_DUMMY_LAST
} e_QeClock;

#define QE_FIRST_CLK_SIGNAL     e_QE_CLK1
#define QE_CLK1_VALUE           e_QE_CLK1

/** @} */ /* end of qe_init_grp group */
/** @} */ /* end of qe__grp group */


/* Mapping of UCC index to module ID:
      --------------------------------
      UCC ID    Module ID (e_ModuleId)
      ------    ----------------------  */
#define QE_UCC_MODULES_MAPPING \
{ \
    { e_UCC_1,  e_MODULE_ID_UCC_1 },    \
    { e_UCC_2,  e_MODULE_ID_UCC_2 },    \
    { e_UCC_3,  e_MODULE_ID_UCC_3 },    \
    { e_UCC_4,  e_MODULE_ID_UCC_4 },    \
    { e_UCC_5,  e_MODULE_ID_UCC_5 },    \
    { e_UCC_6,  e_MODULE_ID_UCC_6 },    \
    { e_UCC_7,  e_MODULE_ID_UCC_7 },    \
    { e_UCC_8,  e_MODULE_ID_UCC_8 }     \
}

#define QE_TDM_SLOT_UCC_CSEL \
{ \
    { e_UCC_1,  9 },    \
    { e_UCC_2,  10 },    \
    { e_UCC_3,  11 },    \
    { e_UCC_4,  12 },    \
    { e_UCC_5,  1 },    \
    { e_UCC_6,  2 },    \
    { e_UCC_7,  3 },    \
    { e_UCC_8,  4 }     \
}

/* Mapping of MCC index to module ID:
      --------------------------------
      MCC ID    Module ID (e_ModuleId)
      ------    ----------------------  */
#define QE_MCC_MODULES_MAPPING \
{ \
    { e_MCC_1,  e_MODULE_ID_MCC }       \
}

#define ILLEGAL_SNUM    0xFF
#define QE_FREE_SNUMS   \
{ \
    0x88, 0x89, 0x98, 0x99, 0xa8, 0xa9, 0xb8, 0xb9, \
    0xc8, 0xc9, 0xd8, 0xd9, 0xe8, 0xe9, ILLEGAL_SNUM, ILLEGAL_SNUM, \
    0x04, 0x05, 0x0c, 0x0d, 0x14, 0x15, 0x1c, 0x1d, \
    0x24, 0x25, 0x2c, 0x2d, 0x34, 0x35, ILLEGAL_SNUM, ILLEGAL_SNUM \
}

/* in the INTERNAL_THREADS array -
   0 - external thread - has interrupt capabilities
   1 - internal thread - no interrupt capabilities */
#define INTERNAL_THREADS   \
{ \
    0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, \
}

/* Static SNUMs structure for I/O devices,
   contains the following fields:
  -----------------------------------------
  Module ID (e_ModuleId), Tx SNUM,  Rx SNUM
  ----------------------  -------   ------- */
#define QE_STATIC_SNUMS \
{ \
    { e_MODULE_ID_UCC_1,    0x00,   0x01 }, \
    { e_MODULE_ID_UCC_2,    0x10,   0x11 }, \
    { e_MODULE_ID_UCC_3,    0x20,   0x21 }, \
    { e_MODULE_ID_UCC_4,    0x30,   0x31 }, \
    { e_MODULE_ID_UCC_5,    0x40,   0x41 }, \
    { e_MODULE_ID_UCC_6,    0x50,   0x51 }, \
    { e_MODULE_ID_UCC_7,    0x60,   0x61 }, \
    { e_MODULE_ID_UCC_8,    0x70,   0x71 }, \
    { e_MODULE_ID_USB,      0x90,   0x91 }, \
    { e_MODULE_ID_SPI_1,    0xA0,   0xA1 }, \
    { e_MODULE_ID_SPI_2,    0xB0,   0xB1 }, \
    { e_MODULE_ID_MCC,      0xC0,   0xC1 }  \
}


/* defines for checking BRGs and Clocks for USB: */

#define IS_LEGAL_CLOCK_SRC_FOR_USB(boardClockSrc, internalClockSrc) \
    ((boardClockSrc == e_QE_CLK3)  ||          \
     (boardClockSrc == e_QE_CLK5)  ||          \
     (boardClockSrc == e_QE_CLK7)  ||          \
     (boardClockSrc == e_QE_CLK9)  ||          \
     (boardClockSrc == e_QE_CLK13) ||          \
     (boardClockSrc == e_QE_CLK17) ||          \
     (boardClockSrc == e_QE_CLK21) ||          \
     (boardClockSrc == e_QE_CLK19)) ||         \
    ((internalClockSrc != boardClockSrc) &&    \
     (boardClockSrc == e_QE_CLK11) &&          \
     ((internalClockSrc == e_QE_BRG9) ||       \
      (internalClockSrc == e_QE_BRG10)))

#define IS_LEGAL_BRG_FOR_USB(brg) \
    ((brg == e_QE_BRG9) || (brg == e_QE_BRG10))

#define IS_LEGAL_BRG_AND_CLK_PAIR_FOR_USB(clock, brg) \
    ((brg == e_QE_BRG9) || (brg == e_QE_BRG10)) &&    \
    ((clock == e_QE_CLK21) || (clock == e_QE_CLK11))

#define IS_LEGAL_CLK_WITH_OR_WITHOUT_BRG_FOR_USB(clock) \
    (clock == e_QE_CLK21)

/* QE clocks muxing table
   ------------------------*/

#define QE_CLOCKS_MUX_TABLE \
{ /* e_QeMuxGroup       Per. ID  e_CommMode             e_QeClockSignal */                      \
    {e_QE_MUX_SI1_CLK,  e_TDM_A, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_A_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_A, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_A_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_A, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_A_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_A, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_A_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_B, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_B_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_B, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_B_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_B, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_B_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_B, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_B_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_C, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_C_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_C, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_C_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_C, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_C_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_C, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_C_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_D, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_D_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_D, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_D_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_D, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_D_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_D, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_D_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_E, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_E_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_E, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_E_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_E, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_E_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_E, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_E_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_F, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_F_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_F, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_F_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_F, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_F_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_F, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_F_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_G, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_G_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_G, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_G_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_G, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_G_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_G, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_G_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_SI1_CLK,  e_TDM_H, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_H_RX       },\
    {e_QE_MUX_SI1_CLK,  e_TDM_H, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_H_TX       },\
    {e_QE_MUX_SI1_SYNC, e_TDM_H, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_TDM_H_RX_SYNC  },\
    {e_QE_MUX_SI1_SYNC, e_TDM_H, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_TDM_H_TX_SYNC  },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_1, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_1_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_1, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_1_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_2, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_2_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_2, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_2_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_3, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_3_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_3, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_3_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_4, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_4_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_4, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_4_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_5, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_5_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_5, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_5_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_6, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_6_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_6, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_6_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_7, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_7_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_7, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_7_TX       },\
                                                                                         \
    {e_QE_MUX_UCC_CLK,  e_UCC_8, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UCC_8_RX       },\
    {e_QE_MUX_UCC_CLK,  e_UCC_8, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UCC_8_TX       },\
                                                                                         \
    {e_QE_MUX_UPC_CLK,  e_UPC_1, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UPC_1_TX       },\
    {e_QE_MUX_UPC_CLK,  e_UPC_1, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UPC_1_RX       },\
    {e_QE_MUX_UPC_CLK,  e_UPC_1, e_COMM_MODE_NONE,      e_QE_CLK_SIGNAL_UPC_1_INT_RATE },\
                                                                                         \
    {e_QE_MUX_UPC_CLK,  e_UPC_2, e_COMM_MODE_TX,        e_QE_CLK_SIGNAL_UPC_2_TX       },\
    {e_QE_MUX_UPC_CLK,  e_UPC_2, e_COMM_MODE_RX,        e_QE_CLK_SIGNAL_UPC_2_RX       },\
    {e_QE_MUX_UPC_CLK,  e_UPC_2, e_COMM_MODE_NONE,      e_QE_CLK_SIGNAL_UPC_2_INT_RATE },\
                                                                                         \
    {e_QE_MUX_USB_CLK,  e_USB_1, e_COMM_MODE_RX_AND_TX, e_QE_CLK_SIGNAL_USB            },\
                                                                                         \
    {e_QE_MUX_RTC_CLK,  0,       e_COMM_MODE_NONE,      e_QE_CLK_SIGNAL_RTC            } \
}
 
#endif /* __QE_INTEGRATION_EXT_H */
