/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PAR_IO_INTEGRATION_EXT_H
#define __PAR_IO_INTEGRATION_EXT_H

#include "io_port_ext.h"

/**************************************************************************//**
 @Description   Parallel I/O Port ID
*//***************************************************************************/
typedef enum e_ParIoPortId
{
    e_PAR_IO_PORT_A = 0,        /**< Parallel IO port A */
    e_PAR_IO_PORT_B,            /**< Parallel IO port B */
    e_PAR_IO_PORT_C,            /**< Parallel IO port C */
    e_PAR_IO_PORT_D,            /**< Parallel IO port D */
    e_PAR_IO_PORT_E,            /**< Parallel IO port E */
    e_PAR_IO_PORT_F             /**< Parallel IO port F */
} e_ParIoPortId;

#define PAR_IO_NUM_OF_PORTS      6


#define PAR_IO_VALID_PIN_MASKS  \
    { /* Port A */ 0xFFFFFFFF,  \
      /* Port B */ 0xFFFFFFFF,  \
      /* Port C */ 0xFFFFFFFF,  \
      /* Port D */ 0xFFFFFFFF,  \
      /* Port E */ 0xFFFFFFFF,  \
      /* Port F */ 0xFFFFFE02 }


#define PAR_IO_VALID_INTR_MASKS \
    { /* Port A */ 0x00000300,  \
      /* Port B */ 0x000C000C,  \
      /* Port C */ 0x0000C060,  \
      /* Port D */ 0x00003000,  \
      /* Port E */ 0x00088002,  \
      /* Port F */ 0x00088002 }

#define PAR_IO_INTR_ID_MAP \
    { 12, 13, 8, 9, 4, 5, 14, 15, 6, 7, 10, 11, 16, 0, 1, 17, 2, 3}

#define PAR_IO_MEM_MAP_85XX


#define PAR_IO_PIN(_portId, _pinId, _assign, _dir, _shared) \
    IO_PIN(e_PAR_IO_PORT_##_portId, _pinId, e_IO_PIN_ASSIGN_##_assign, e_IO_PIN_DIR_##_dir, _shared)

/* This enum is automatically generated from the Reference Manual */

typedef enum e_ParIoFunction
{
 
    e_PAR_IO_FUNC_ENET1_RX_ER_SER1_CD_B                   = PAR_IO_PIN(A, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RX_ER_SER2_CD_B                   = PAR_IO_PIN(A, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RX_ER_SER3_CD_B                   = PAR_IO_PIN(B, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RX_ER_SER4_CD_B                   = PAR_IO_PIN(B, 25, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_ER_SER5_CD_B                   = PAR_IO_PIN(D, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RX_ER_SER6_CD_B                   = PAR_IO_PIN(D, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RX_ER_SER7_CD_B                   = PAR_IO_PIN(E,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RX_ER_SER8_CD_B                   = PAR_IO_PIN(E, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RX_DV_SER1_CTS_B                  = PAR_IO_PIN(A, 12, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RX_DV_SER2_CTS_B                  = PAR_IO_PIN(A, 26, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RX_DV_SER3_CTS_B                  = PAR_IO_PIN(B,  9, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RX_DV_SER4_CTS_B                  = PAR_IO_PIN(B, 24, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_DV_SER5_CTS_B                  = PAR_IO_PIN(D, 12, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RX_DV_SER6_CTS_B                  = PAR_IO_PIN(D, 26, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RX_DV_SER7_CTS_B                  = PAR_IO_PIN(E,  8, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RX_DV_SER8_CTS_B                  = PAR_IO_PIN(E, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UART1_CTS_B                             = PAR_IO_PIN(F, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_NUSB_OE                                 = PAR_IO_PIN(F,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_REQ                            = PAR_IO_PIN(A,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_REQ                            = PAR_IO_PIN(B, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_REQ                            = PAR_IO_PIN(D, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_REQ                            = PAR_IO_PIN(E,  0, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_REQ                            = PAR_IO_PIN(A, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_REQ                            = PAR_IO_PIN(A, 30, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_REQ                            = PAR_IO_PIN(E, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TX_EN_SER1_RTS_B                  = PAR_IO_PIN(A,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TX_EN_SER2_RTS_B                  = PAR_IO_PIN(A, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TX_EN_SER3_RTS_B                  = PAR_IO_PIN(B,  1, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TX_EN_SER4_RTS_B                  = PAR_IO_PIN(B, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TX_EN_SER5_RTS_B                  = PAR_IO_PIN(D,  4, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TX_EN_SER6_RTS_B                  = PAR_IO_PIN(D, 18, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TX_EN_SER7_RTS_B                  = PAR_IO_PIN(E,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TX_EN_SER8_RTS_B                  = PAR_IO_PIN(E, 14, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UART1_RTS_B                             = PAR_IO_PIN(F, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXEN_B                           = PAR_IO_PIN(D,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXEN_B0                          = PAR_IO_PIN(E,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B1                            = PAR_IO_PIN(E,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B2_OPT1                       = PAR_IO_PIN(B,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B2_OPT2                       = PAR_IO_PIN(B,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B3_OPT1                       = PAR_IO_PIN(B, 10, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXEN_B3_OPT2                       = PAR_IO_PIN(B, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPISEL_B                           = PAR_IO_PIN(E, 30, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SPI2_SPISEL_B                           = PAR_IO_PIN(B, 31, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE1_B_OPT1                    = PAR_IO_PIN(F,  1, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE1_B_OPT2                    = PAR_IO_PIN(F, 11, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE2_B_OPT1                    = PAR_IO_PIN(C,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE2_B_OPT2                    = PAR_IO_PIN(E, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE3_B_OPT1                    = PAR_IO_PIN(B, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE3_B_OPT2                    = PAR_IO_PIN(B, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE4_B_OPT1                    = PAR_IO_PIN(C, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE4_B_OPT2                    = PAR_IO_PIN(E, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT1_B_OPT1                     = PAR_IO_PIN(E, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT1_B_OPT2                     = PAR_IO_PIN(F, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT2_B_OPT1                     = PAR_IO_PIN(C, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT2_B_OPT2                     = PAR_IO_PIN(E, 25, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT3_B_OPT1                     = PAR_IO_PIN(F, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT3_B_OPT2                     = PAR_IO_PIN(F, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT4_B_OPT1                     = PAR_IO_PIN(C, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT4_B_OPT2                     = PAR_IO_PIN(E, 28, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXEN_B                           = PAR_IO_PIN(E,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXEN_B0                          = PAR_IO_PIN(D,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B1                            = PAR_IO_PIN(E,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B2_OPT1                       = PAR_IO_PIN(B,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B2_OPT2                       = PAR_IO_PIN(E, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B3_OPT1                       = PAR_IO_PIN(A, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXEN_B3_OPT2                       = PAR_IO_PIN(B,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_BRGO1                                   = PAR_IO_PIN(C,  0, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO10                                  = PAR_IO_PIN(C, 21, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO11                                  = PAR_IO_PIN(C, 22, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO12                                  = PAR_IO_PIN(C, 23, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO13                                  = PAR_IO_PIN(C, 26, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO14                                  = PAR_IO_PIN(C, 27, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO15                                  = PAR_IO_PIN(C,  4, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO16                                  = PAR_IO_PIN(C,  6, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO2                                   = PAR_IO_PIN(C,  1, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO3                                   = PAR_IO_PIN(C, 10, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO4                                   = PAR_IO_PIN(C, 11, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO5                                   = PAR_IO_PIN(C, 12, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO6                                   = PAR_IO_PIN(C, 14, OPTION_3, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO7                                   = PAR_IO_PIN(C, 15, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO8                                   = PAR_IO_PIN(C, 18, OPTION_2, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO9                                   = PAR_IO_PIN(C, 19, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_CLK1                                    = PAR_IO_PIN(C,  0, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK10                                   = PAR_IO_PIN(C,  9, OPTION_3, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK11                                   = PAR_IO_PIN(C, 10, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK12                                   = PAR_IO_PIN(C, 11, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK13                                   = PAR_IO_PIN(C, 12, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK14                                   = PAR_IO_PIN(C, 13, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK15                                   = PAR_IO_PIN(C, 14, OPTION_3, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK16                                   = PAR_IO_PIN(C, 15, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK17                                   = PAR_IO_PIN(C, 16, OPTION_3, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK18                                   = PAR_IO_PIN(C, 17, OPTION_3, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK19                                   = PAR_IO_PIN(C, 18, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK2                                    = PAR_IO_PIN(C,  1, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK20                                   = PAR_IO_PIN(C, 19, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK21                                   = PAR_IO_PIN(C, 20, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK22                                   = PAR_IO_PIN(C, 21, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK23                                   = PAR_IO_PIN(C, 22, OPTION_3, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK24                                   = PAR_IO_PIN(C, 23, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK25                                   = PAR_IO_PIN(C, 24, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK26                                   = PAR_IO_PIN(C, 25, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK27                                   = PAR_IO_PIN(C, 26, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK28                                   = PAR_IO_PIN(C, 27, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK3                                    = PAR_IO_PIN(C,  2, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK4                                    = PAR_IO_PIN(C,  3, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK5                                    = PAR_IO_PIN(C,  4, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK6                                    = PAR_IO_PIN(C,  5, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK7                                    = PAR_IO_PIN(C,  6, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK8                                    = PAR_IO_PIN(C,  7, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK9                                    = PAR_IO_PIN(C,  8, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_SI1_TDMA_CLKO                           = PAR_IO_PIN(A, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_CLKO                           = PAR_IO_PIN(B, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_CLKO                           = PAR_IO_PIN(D, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_CLKO                           = PAR_IO_PIN(E,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_CLKO                           = PAR_IO_PIN(A, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_CLKO                           = PAR_IO_PIN(B, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_CLKO                           = PAR_IO_PIN(D, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_CLKO                           = PAR_IO_PIN(E, 23, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_COL                               = PAR_IO_PIN(A, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_COL                               = PAR_IO_PIN(A, 24, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_COL                               = PAR_IO_PIN(B,  7, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_COL                               = PAR_IO_PIN(B, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_COL                               = PAR_IO_PIN(D, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_COL                               = PAR_IO_PIN(D, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_COL                               = PAR_IO_PIN(E,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_COL                               = PAR_IO_PIN(E, 20, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_CRS_SDET                          = PAR_IO_PIN(A, 11, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_CRS_SDET                          = PAR_IO_PIN(A, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_CRS                               = PAR_IO_PIN(B,  8, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_CRS_SDET                          = PAR_IO_PIN(B, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_CRS                               = PAR_IO_PIN(D, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_CRS                               = PAR_IO_PIN(D, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_CRS                               = PAR_IO_PIN(E,  7, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_CRS                               = PAR_IO_PIN(E, 21, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RX_DV_SER2_CTS_B_OPT2             = PAR_IO_PIN(B, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_GTXCLK                            = PAR_IO_PIN(C, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_GTXCLK                            = PAR_IO_PIN(C,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_GTXCLK                            = PAR_IO_PIN(C, 25, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_GTXCLK                            = PAR_IO_PIN(C, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_MUX_MDC                              = PAR_IO_PIN(C, 31, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI2_SPICLK_OPT1                        = PAR_IO_PIN(B, 30, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI2_SPICLK_OPT2                        = PAR_IO_PIN(C, 31, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_QE_MUX_MDIO                             = PAR_IO_PIN(C, 30, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI2_SPIMOSI_OPT1                       = PAR_IO_PIN(B, 28, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI2_SPIMOSI_OPT2                       = PAR_IO_PIN(C, 30, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXCLAV                           = PAR_IO_PIN(D,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXCLAV0                          = PAR_IO_PIN(E,  5, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV1_OPT1                       = PAR_IO_PIN(B,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV1_OPT2                       = PAR_IO_PIN(E, 18, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV2_OPT1                       = PAR_IO_PIN(B,  1, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV2_OPT2                       = PAR_IO_PIN(B,  2, OPTION_0, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV3_OPT1                       = PAR_IO_PIN(A, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXCLAV3_OPT2                       = PAR_IO_PIN(B,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXCLAV                           = PAR_IO_PIN(E,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXCLAV0                          = PAR_IO_PIN(D,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV1_OPT1                       = PAR_IO_PIN(B,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV1_OPT2                       = PAR_IO_PIN(E, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV2_OPT1                       = PAR_IO_PIN(A, 28, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV2_OPT2                       = PAR_IO_PIN(A, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV3_OPT1                       = PAR_IO_PIN(A,  5, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXCLAV3_OPT2                       = PAR_IO_PIN(A, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR0                          = PAR_IO_PIN(E, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR0                          = PAR_IO_PIN(E, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR1                          = PAR_IO_PIN(E, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR1                          = PAR_IO_PIN(E, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR2_OPT1                     = PAR_IO_PIN(E, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR2_OPT2                     = PAR_IO_PIN(F, 15, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR2_OPT1                     = PAR_IO_PIN(E, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR2_OPT2                     = PAR_IO_PIN(F, 18, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR3_OPT1                     = PAR_IO_PIN(E, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR3_OPT2                     = PAR_IO_PIN(F, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR3_OPT1                     = PAR_IO_PIN(E, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR3_OPT2                     = PAR_IO_PIN(F, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR4_OPT1                     = PAR_IO_PIN(E, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR4_OPT2                     = PAR_IO_PIN(F, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR4_OPT1                     = PAR_IO_PIN(E, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR4_OPT2                     = PAR_IO_PIN(F, 20, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXADDR5_OPT1                       = PAR_IO_PIN(E, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXADDR5_OPT2                       = PAR_IO_PIN(F, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_RXD0_SER1_RXD0                    = PAR_IO_PIN(A,  6, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD0_SER2_RXD0                    = PAR_IO_PIN(A, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD0_SER3_RXD0                    = PAR_IO_PIN(B,  3, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD0_SER4_RXD0                    = PAR_IO_PIN(B, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD1_SER1_RXD1                    = PAR_IO_PIN(A,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD1_SER2_RXD1                    = PAR_IO_PIN(A, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD1_SER3_RXD1                    = PAR_IO_PIN(B,  4, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD1_SER4_RXD1                    = PAR_IO_PIN(B, 19, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD2_SER1_RXD2                    = PAR_IO_PIN(A,  8, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD2_SER2_RXD2                    = PAR_IO_PIN(A, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD2_SER3_RXD2                    = PAR_IO_PIN(B,  5, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD2_SER4_RXD2                    = PAR_IO_PIN(B, 20, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD3_SER1_RXD3                    = PAR_IO_PIN(A,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD3_SER2_RXD3                    = PAR_IO_PIN(A, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_RXD3_SER3_RXD3                    = PAR_IO_PIN(B,  6, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_RXD3_SER4_RXD3                    = PAR_IO_PIN(B, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD4_SER1_RXD4                    = PAR_IO_PIN(B, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD4_SER2_RXD4                    = PAR_IO_PIN(A, 28, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD5_SER1_RXD5                    = PAR_IO_PIN(B,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD5_SER2_RXD5                    = PAR_IO_PIN(B, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD6_SER1_RXD6                    = PAR_IO_PIN(B,  2, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD6_SER2_RXD6                    = PAR_IO_PIN(B, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD7_SER1_RXD7                    = PAR_IO_PIN(A, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD7_SER2_RXD7                    = PAR_IO_PIN(B, 14, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA0_OPT1                       = PAR_IO_PIN(A,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA0_OPT2                       = PAR_IO_PIN(D, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA0_OPT1                       = PAR_IO_PIN(A, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA0_OPT2                       = PAR_IO_PIN(D, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA10                           = PAR_IO_PIN(D, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA10                           = PAR_IO_PIN(D,  8, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA11                           = PAR_IO_PIN(D, 30, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA11                           = PAR_IO_PIN(D,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA12                           = PAR_IO_PIN(D, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA12                           = PAR_IO_PIN(D,  6, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA13                           = PAR_IO_PIN(D, 28, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA13                           = PAR_IO_PIN(D,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA14                           = PAR_IO_PIN(D, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA14                           = PAR_IO_PIN(D,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA15                           = PAR_IO_PIN(D,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA15                           = PAR_IO_PIN(D, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA1_OPT1                       = PAR_IO_PIN(A,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA1_OPT2                       = PAR_IO_PIN(D, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA1_OPT1                       = PAR_IO_PIN(A,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA1_OPT2                       = PAR_IO_PIN(D, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA2_OPT1                       = PAR_IO_PIN(A,  5, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA2_OPT2                       = PAR_IO_PIN(D, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA2_OPT1                       = PAR_IO_PIN(A,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA2_OPT2                       = PAR_IO_PIN(D, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA3_OPT1                       = PAR_IO_PIN(A,  2, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA3_OPT2                       = PAR_IO_PIN(D, 19, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA3_OPT1                       = PAR_IO_PIN(A, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA3_OPT2                       = PAR_IO_PIN(D, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA4_OPT1                       = PAR_IO_PIN(A,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA4_OPT2                       = PAR_IO_PIN(D, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA4_OPT1                       = PAR_IO_PIN(A,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA4_OPT2                       = PAR_IO_PIN(D, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA5_OPT1                       = PAR_IO_PIN(A,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA5_OPT2                       = PAR_IO_PIN(D, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA5_OPT1                       = PAR_IO_PIN(A, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA5_OPT2                       = PAR_IO_PIN(D, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA6_OPT1                       = PAR_IO_PIN(A,  4, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA6_OPT2                       = PAR_IO_PIN(D, 22, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA6_OPT1                       = PAR_IO_PIN(A, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA6_OPT2                       = PAR_IO_PIN(D, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA7                            = PAR_IO_PIN(E,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA7                            = PAR_IO_PIN(D, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA8                            = PAR_IO_PIN(E,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA8                            = PAR_IO_PIN(D, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA9                            = PAR_IO_PIN(E,  0, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA9                            = PAR_IO_PIN(D,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_REOP_OPT1                          = PAR_IO_PIN(E, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_REOP_OPT2                          = PAR_IO_PIN(F, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TEOP_OPT1                          = PAR_IO_PIN(E, 11, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TEOP_OPT2                          = PAR_IO_PIN(F, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ1_OPT1                      = PAR_IO_PIN(B, 28, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ1_OPT2                      = PAR_IO_PIN(B, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ2_OPT1                      = PAR_IO_PIN(B, 29, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ2_OPT2                      = PAR_IO_PIN(E, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ3_OPT1                      = PAR_IO_PIN(B, 30, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ3_OPT2                      = PAR_IO_PIN(F, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ4_OPT1                      = PAR_IO_PIN(C, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ4_OPT2                      = PAR_IO_PIN(F, 19, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RERR_OPT1                          = PAR_IO_PIN(E, 16, OPTION_0, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RERR_OPT2                          = PAR_IO_PIN(F, 21, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TERR_OPT1                          = PAR_IO_PIN(E, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TERR_OPT2                          = PAR_IO_PIN(F, 20, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RMOD_OPT1                          = PAR_IO_PIN(E, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RMOD_OPT2                          = PAR_IO_PIN(F, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TMOD_OPT1                          = PAR_IO_PIN(E, 22, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TMOD_OPT2                          = PAR_IO_PIN(F, 15, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_RN                                  = PAR_IO_PIN(F,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_RP                                  = PAR_IO_PIN(F,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXPRTY                             = PAR_IO_PIN(E,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXPRTY                             = PAR_IO_PIN(E,  6, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXSOC                              = PAR_IO_PIN(E,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXSOC                              = PAR_IO_PIN(D,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_RSYNC                          = PAR_IO_PIN(A, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_RSYNC_OPT1                     = PAR_IO_PIN(A, 27, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_RSYNC_OPT2                     = PAR_IO_PIN(F,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMC_RSYNC                          = PAR_IO_PIN(B, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_RSYNC                          = PAR_IO_PIN(B, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDME_RSYNC_OPT1                     = PAR_IO_PIN(D, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDME_RSYNC_OPT2                     = PAR_IO_PIN(F, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_RSYNC                          = PAR_IO_PIN(D, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_RSYNC                          = PAR_IO_PIN(E,  7, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMH_RSYNC                          = PAR_IO_PIN(E, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_RSYNC_OPT1                     = PAR_IO_PIN(A, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_RSYNC_OPT2                     = PAR_IO_PIN(F, 18, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_RSYNC                          = PAR_IO_PIN(A, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RSYNC_OPT1                     = PAR_IO_PIN(B,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RSYNC_OPT2                     = PAR_IO_PIN(B,  8, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_RSYNC_OPT1                     = PAR_IO_PIN(B, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_RSYNC_OPT2                     = PAR_IO_PIN(D, 23, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RSYNC_OPT1                     = PAR_IO_PIN(D, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RSYNC_OPT2                     = PAR_IO_PIN(F,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMF_RSYNC                          = PAR_IO_PIN(D, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMG_RSYNC                          = PAR_IO_PIN(E,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_RSYNC                          = PAR_IO_PIN(E, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RVAL_OPT1                        = PAR_IO_PIN(E, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RVAL_OPT2                        = PAR_IO_PIN(F, 22, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RVAL_OPT1                        = PAR_IO_PIN(E, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RVAL_OPT2                        = PAR_IO_PIN(F, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_RXD0_SER6_RXD0_OPT2               = PAR_IO_PIN(E, 14, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD0_SER8_RXD0                    = PAR_IO_PIN(E, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_RXD                                 = PAR_IO_PIN(F,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_GRXCLK                            = PAR_IO_PIN(C,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_GRXCLK                            = PAR_IO_PIN(C,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET3_GRXCLK                            = PAR_IO_PIN(C,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET4_GRXCLK                            = PAR_IO_PIN(C, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_GRXCLK1                           = PAR_IO_PIN(C,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_GRXCLK1                           = PAR_IO_PIN(C, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD0_SER5_RXD0                    = PAR_IO_PIN(D,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD0_SER6_RXD0                    = PAR_IO_PIN(D, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD0_SER7_RXD0                    = PAR_IO_PIN(E,  2, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_RXD0                           = PAR_IO_PIN(A,  5, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_RXD0_OPT1                      = PAR_IO_PIN(A, 20, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_RXD0_OPT2                      = PAR_IO_PIN(F,  0, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMC_RXD0                           = PAR_IO_PIN(B,  3, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_RXD0                           = PAR_IO_PIN(B, 17, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDME_RXD0_OPT1                      = PAR_IO_PIN(D,  6, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDME_RXD0_OPT2                      = PAR_IO_PIN(F, 21, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_RXD0                           = PAR_IO_PIN(D, 19, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_RXD0                           = PAR_IO_PIN(E,  1, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMH_RXD0                           = PAR_IO_PIN(E, 16, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_RXD0_OPT1                      = PAR_IO_PIN(A,  6, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_RXD0_OPT2                      = PAR_IO_PIN(F, 17, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_RXD0                           = PAR_IO_PIN(A, 19, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RXD0_OPT1                      = PAR_IO_PIN(A, 31, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RXD0_OPT2                      = PAR_IO_PIN(B,  2, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_RXD0_OPT1                      = PAR_IO_PIN(B, 18, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_RXD0_OPT2                      = PAR_IO_PIN(D, 18, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RXD0_OPT1                      = PAR_IO_PIN(D,  5, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RXD0_OPT2                      = PAR_IO_PIN(F,  4, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMF_RXD0                           = PAR_IO_PIN(D, 20, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMG_RXD0                           = PAR_IO_PIN(E,  2, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_RXD0                           = PAR_IO_PIN(E, 15, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_ENET2_RXD0_SER2_RXD0_OPT2               = PAR_IO_PIN(B, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD1_SER5_RXD1                    = PAR_IO_PIN(D,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD1_SER6_RXD1                    = PAR_IO_PIN(D, 21, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD1_SER7_RXD1                    = PAR_IO_PIN(E,  3, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD1_SER8_RXD1                    = PAR_IO_PIN(E, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_RXD1                           = PAR_IO_PIN(A,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_RXD1                           = PAR_IO_PIN(B, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_RXD1                           = PAR_IO_PIN(D, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_RXD1                           = PAR_IO_PIN(D, 29, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_RXD1                           = PAR_IO_PIN(A, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RXD1                           = PAR_IO_PIN(B,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RXD1                           = PAR_IO_PIN(D,  1, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_RXD1                           = PAR_IO_PIN(E, 11, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET2_RXD1_SER2_RXD1_OPT2               = PAR_IO_PIN(B, 22, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD2_SER5_RXD2                    = PAR_IO_PIN(D,  8, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD2_SER6_RXD2                    = PAR_IO_PIN(D, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD2_SER7_RXD2                    = PAR_IO_PIN(E,  4, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD2_SER8_RXD2                    = PAR_IO_PIN(E, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_RXD2                           = PAR_IO_PIN(A,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_RXD2                           = PAR_IO_PIN(B, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_RXD2                           = PAR_IO_PIN(D, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_RXD2                           = PAR_IO_PIN(D, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_RXD2                           = PAR_IO_PIN(A, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RXD2                           = PAR_IO_PIN(A, 31, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RXD2                           = PAR_IO_PIN(D,  2, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_RXD2                           = PAR_IO_PIN(E, 12, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD3_SER5_RXD3                    = PAR_IO_PIN(D,  9, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_RXD3_SER6_RXD3                    = PAR_IO_PIN(D, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET7_RXD3_SER7_RXD3                    = PAR_IO_PIN(E,  5, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_RXD3_SER8_RXD3                    = PAR_IO_PIN(E, 19, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_RXD3                           = PAR_IO_PIN(A, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_RXD3                           = PAR_IO_PIN(B, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_RXD3                           = PAR_IO_PIN(D, 23, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_RXD3                           = PAR_IO_PIN(E,  8, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_RXD3                           = PAR_IO_PIN(A, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_RXD3                           = PAR_IO_PIN(B,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_RXD3                           = PAR_IO_PIN(D, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_RXD3                           = PAR_IO_PIN(E, 22, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UART1_SIN                               = PAR_IO_PIN(F, 12, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UART1_SOUT                              = PAR_IO_PIN(F,  9, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPICLK                             = PAR_IO_PIN(E, 29, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPIMISO                            = PAR_IO_PIN(E, 28, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI2_SPIMISO                            = PAR_IO_PIN(B, 29, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPIMOSI                            = PAR_IO_PIN(E, 27, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_STPA_OPT1                        = PAR_IO_PIN(E, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_STPA_OPT2                        = PAR_IO_PIN(F, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_STPA_OPT1                        = PAR_IO_PIN(E, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_STPA_OPT2                        = PAR_IO_PIN(F, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE1                             = PAR_IO_PIN(C, 27, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE1                             = PAR_IO_PIN(F,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE2                             = PAR_IO_PIN(C, 21, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE2                             = PAR_IO_PIN(F, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE3                             = PAR_IO_PIN(B,  6, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE3                             = PAR_IO_PIN(A, 28, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE4                             = PAR_IO_PIN(B, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE4                             = PAR_IO_PIN(B, 20, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE5                             = PAR_IO_PIN(D,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE5                             = PAR_IO_PIN(D,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE6                             = PAR_IO_PIN(D, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE6                             = PAR_IO_PIN(D, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE7                             = PAR_IO_PIN(D, 30, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE7                             = PAR_IO_PIN(E,  4, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_STROBE8                             = PAR_IO_PIN(E, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE8_OPT1                        = PAR_IO_PIN(E, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_STROBE8_OPT2                        = PAR_IO_PIN(E, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_SYNC_IN_OPT2                      = PAR_IO_PIN(E, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET8_SYNC_IN                           = PAR_IO_PIN(E, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_SYNC_OUT_OPT2                     = PAR_IO_PIN(E, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_SYNC_OUT                          = PAR_IO_PIN(E, 11, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR0                          = PAR_IO_PIN(E, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR0                          = PAR_IO_PIN(E, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR1                          = PAR_IO_PIN(E, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR1                          = PAR_IO_PIN(E, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR2_OPT1                     = PAR_IO_PIN(E, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR2_OPT2                     = PAR_IO_PIN(F, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR2_OPT1                     = PAR_IO_PIN(E, 10, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR2_OPT2                     = PAR_IO_PIN(F, 15, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR3_OPT1                     = PAR_IO_PIN(E, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR3_OPT2                     = PAR_IO_PIN(F, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR3_OPT1                     = PAR_IO_PIN(E, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR3_OPT2                     = PAR_IO_PIN(F, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR4_OPT1                     = PAR_IO_PIN(E, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR4_OPT2                     = PAR_IO_PIN(F, 20, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR4_OPT1                     = PAR_IO_PIN(E, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR4_OPT2                     = PAR_IO_PIN(F, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXADDR5_OPT1                       = PAR_IO_PIN(C, 13, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_TXADDR5_OPT2                       = PAR_IO_PIN(E, 12, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD0_SER1_TXD0                    = PAR_IO_PIN(A,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD0_SER2_TXD0                    = PAR_IO_PIN(A, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD0_SER3_TXD0                    = PAR_IO_PIN(A, 29, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD0_SER4_TXD0                    = PAR_IO_PIN(B, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD1_SER1_TXD1                    = PAR_IO_PIN(A,  1, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD1_SER2_TXD1                    = PAR_IO_PIN(A, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD1_SER3_TXD1                    = PAR_IO_PIN(A, 30, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD1_SER4_TXD1                    = PAR_IO_PIN(B, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD2_SER1_TXD2                    = PAR_IO_PIN(A,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD2_SER2_TXD2                    = PAR_IO_PIN(A, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD2_SER3_TXD2                    = PAR_IO_PIN(A, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD2_SER4_TXD2                    = PAR_IO_PIN(B, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD3_SER1_TXD3                    = PAR_IO_PIN(A,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD3_SER2_TXD3                    = PAR_IO_PIN(A, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TXD3_SER3_TXD3                    = PAR_IO_PIN(B,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TXD3_SER4_TXD3                    = PAR_IO_PIN(B, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD4_SER1_TXD4                    = PAR_IO_PIN(B,  8, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD4_SER2_TXD4                    = PAR_IO_PIN(B, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD5_SER1_TXD5                    = PAR_IO_PIN(B,  0, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD5_SER2_TXD5                    = PAR_IO_PIN(B, 15, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD6_SER1_TXD6                    = PAR_IO_PIN(B,  7, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD6_SER2_TXD6                    = PAR_IO_PIN(B, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD7_SER1_TXD7                    = PAR_IO_PIN(B,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD7_SER2_TXD7                    = PAR_IO_PIN(B, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TX_ER                             = PAR_IO_PIN(A,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TX_ER                             = PAR_IO_PIN(A, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN1_OPT1                        = PAR_IO_PIN(F,  0, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN1_OPT2                        = PAR_IO_PIN(F,  9, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN2_OPT1                        = PAR_IO_PIN(C,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN2_OPT2                        = PAR_IO_PIN(E, 24, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN3_OPT1                        = PAR_IO_PIN(B, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN3_OPT2                        = PAR_IO_PIN(F, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN4_OPT1                        = PAR_IO_PIN(B, 31, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN4_OPT2                        = PAR_IO_PIN(E, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_USB_TN                                  = PAR_IO_PIN(F,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_USB_TP                                  = PAR_IO_PIN(F,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_TSYNC_OPT1                     = PAR_IO_PIN(A,  3, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_TSYNC_OPT2                     = PAR_IO_PIN(A, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_TSYNC_OPT1                     = PAR_IO_PIN(B, 15, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_TSYNC_OPT2                     = PAR_IO_PIN(B, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_TSYNC                          = PAR_IO_PIN(D, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_TSYNC                          = PAR_IO_PIN(D, 31, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_TSYNC_OPT1                     = PAR_IO_PIN(A, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_TSYNC_OPT2                     = PAR_IO_PIN(F, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_TSYNC_OPT1                     = PAR_IO_PIN(A, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_TSYNC_OPT2                     = PAR_IO_PIN(A, 27, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TSYNC_OPT1                     = PAR_IO_PIN(B,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TSYNC_OPT2                     = PAR_IO_PIN(B, 10, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TSYNC_OPT1                     = PAR_IO_PIN(D,  3, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TSYNC_OPT2                     = PAR_IO_PIN(F,  3, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_TSYNC_OPT1                     = PAR_IO_PIN(E, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_TSYNC_OPT2                     = PAR_IO_PIN(E, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_TSYNC_OPT1                     = PAR_IO_PIN(A, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_TSYNC_OPT2                     = PAR_IO_PIN(E, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMC_TSYNC                          = PAR_IO_PIN(B,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDME_TSYNC_OPT1                     = PAR_IO_PIN(D, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDME_TSYNC_OPT2                     = PAR_IO_PIN(F, 19, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMH_TSYNC_OPT1                     = PAR_IO_PIN(E, 11, OPTION_0, INPUT , FALSE),
    e_PAR_IO_FUNC_SI1_TDMH_TSYNC_OPT2                     = PAR_IO_PIN(E, 22, OPTION_0, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_TSYNC_OPT1                     = PAR_IO_PIN(B, 24, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_TSYNC_OPT2                     = PAR_IO_PIN(D, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMF_TSYNC                          = PAR_IO_PIN(D, 26, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SI2_TDMG_TSYNC                          = PAR_IO_PIN(E,  8, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET6_TXD0_SER6_TXD0_OPT2               = PAR_IO_PIN(E, 17, OPTION_0, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD0_SER8_TXD0                    = PAR_IO_PIN(E, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TX_EN_SER2_RTS_B_OPT2             = PAR_IO_PIN(A, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET3_TX_ER                             = PAR_IO_PIN(B,  2, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET4_TX_ER                             = PAR_IO_PIN(B, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TX_ER                             = PAR_IO_PIN(D,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TX_ER                             = PAR_IO_PIN(D, 19, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TX_ER                             = PAR_IO_PIN(E,  1, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TX_ER                             = PAR_IO_PIN(E, 15, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD0_SER5_TXD0                    = PAR_IO_PIN(D,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD0_SER6_TXD0                    = PAR_IO_PIN(D, 14, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD0_SER7_TXD0                    = PAR_IO_PIN(D, 28, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_TXD0                           = PAR_IO_PIN(A, 10, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_TXD0_OPT1                      = PAR_IO_PIN(A, 14, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMB_TXD0_OPT2                      = PAR_IO_PIN(F,  1, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMC_TXD0                           = PAR_IO_PIN(A, 29, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDME_TXD0_OPT1                      = PAR_IO_PIN(D,  0, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDME_TXD0_OPT2                      = PAR_IO_PIN(F, 20, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_TXD0                           = PAR_IO_PIN(D, 24, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_TXD0                           = PAR_IO_PIN(E,  6, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMH_TXD0                           = PAR_IO_PIN(E, 10, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_TXD0_OPT1                      = PAR_IO_PIN(A,  0, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMA_TXD0_OPT2                      = PAR_IO_PIN(F, 16, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_TXD0                           = PAR_IO_PIN(A, 24, OPTION_2, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TXD0_OPT1                      = PAR_IO_PIN(B,  5, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TXD0_OPT2                      = PAR_IO_PIN(B,  7, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_TXD0_OPT1                      = PAR_IO_PIN(B, 12, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMD_TXD0_OPT2                      = PAR_IO_PIN(D, 22, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TXD0_OPT1                      = PAR_IO_PIN(D, 10, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TXD0_OPT2                      = PAR_IO_PIN(F,  5, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMF_TXD0                           = PAR_IO_PIN(D, 14, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMG_TXD0                           = PAR_IO_PIN(D, 28, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_TXD0                           = PAR_IO_PIN(E, 20, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_TXD0                           = PAR_IO_PIN(B, 22, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD0_SER2_TXD0_OPT2               = PAR_IO_PIN(A, 19, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD1_SER5_TXD1                    = PAR_IO_PIN(D,  1, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD1_SER6_TXD1                    = PAR_IO_PIN(D, 15, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD1_SER7_TXD1                    = PAR_IO_PIN(D, 29, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD1_SER8_TXD1                    = PAR_IO_PIN(E, 11, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_TXD1                           = PAR_IO_PIN(A,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_TXD1                           = PAR_IO_PIN(B, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_TXD1                           = PAR_IO_PIN(D, 21, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_TXD1                           = PAR_IO_PIN(E,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_TXD1                           = PAR_IO_PIN(A, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TXD1                           = PAR_IO_PIN(B,  4, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TXD1                           = PAR_IO_PIN(D,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_TXD1                           = PAR_IO_PIN(E, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET2_TXD1_SER2_TXD1_OPT2               = PAR_IO_PIN(A, 24, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD2_SER5_TXD2                    = PAR_IO_PIN(D,  2, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD2_SER6_TXD2                    = PAR_IO_PIN(D, 16, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD2_SER7_TXD2                    = PAR_IO_PIN(D, 30, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD2_SER8_TXD2                    = PAR_IO_PIN(E, 12, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_TXD2                           = PAR_IO_PIN(A,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_TXD2                           = PAR_IO_PIN(B, 20, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_TXD2                           = PAR_IO_PIN(D, 22, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_TXD2                           = PAR_IO_PIN(E,  4, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_TXD2                           = PAR_IO_PIN(A, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TXD2                           = PAR_IO_PIN(B,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TXD2                           = PAR_IO_PIN(D,  8, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_TXD2                           = PAR_IO_PIN(E, 18, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD3_SER5_TXD3                    = PAR_IO_PIN(D,  3, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET6_TXD3_SER6_TXD3                    = PAR_IO_PIN(D, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET7_TXD3_SER7_TXD3                    = PAR_IO_PIN(D, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET8_TXD3_SER8_TXD3                    = PAR_IO_PIN(E, 13, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMA_TXD3                           = PAR_IO_PIN(A,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMD_TXD3                           = PAR_IO_PIN(B, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMF_TXD3                           = PAR_IO_PIN(D, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI1_TDMG_TXD3                           = PAR_IO_PIN(E,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMB_TXD3                           = PAR_IO_PIN(A, 23, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMC_TXD3                           = PAR_IO_PIN(B,  6, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDME_TXD3                           = PAR_IO_PIN(D,  9, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SI2_TDMH_TXD3                           = PAR_IO_PIN(E, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC1_CLKO                               = PAR_IO_PIN(C, 20, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC2_CLKO                               = PAR_IO_PIN(C,  2, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC3_CLKO                               = PAR_IO_PIN(C, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC4_CLKO                               = PAR_IO_PIN(C, 24, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC5_CLKO                               = PAR_IO_PIN(C, 12, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC6_CLKO                               = PAR_IO_PIN(C,  7, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC7_CLKO                               = PAR_IO_PIN(C, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UCC8_CLKO                               = PAR_IO_PIN(C, 21, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_CLKO                               = PAR_IO_PIN(C, 26, OPTION_1, OUTPUT, FALSE)

 
}e_ParIoFunction;

#endif /* __PAR_IO_MDS8569_EXT */
