/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          part_integration_ext.h

 @Description   MPC8569 external definitions and structures.
*//***************************************************************************/
#ifndef __PART_INTEGRATION_EXT_H
#define __PART_INTEGRATION_EXT_H


#include "ddr_std_ext.h"
#include "epic_ext.h"
#include "qe_ic_ext.h"

/**************************************************************************//**
 @Group         8569_chip_id MPC8569 Application Programming Interface

 @Description   MPC8569 Chip functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Module types.
*//***************************************************************************/
typedef enum e_ModuleId
{
    e_MODULE_ID_LAW,            /**< Local Access module      */
    e_MODULE_ID_ECM,            /**< e500 Coherency Module    */
    e_MODULE_ID_DDR_1,          /**< Primary DDR memory controller */
    e_MODULE_ID_DDR_2,          /**< Secondary DDR memory controller */
    e_MODULE_ID_I2C_1,          /**< I2C 1                    */
    e_MODULE_ID_I2C_2,          /**< I2C 2                    */
    e_MODULE_ID_DUART_1,        /**< DUART module 1           */
    e_MODULE_ID_DUART_2,        /**< DUART module 2           */
    e_MODULE_ID_LBC,            /**< Local bus memory controller module */
    e_MODULE_ID_PCI,            /**< PCI controller */
    e_MODULE_ID_PCIE,           /**< PCI Express controller module*/
    e_MODULE_ID_PCIE_ATMU,      /**< PCI ATMU WINDOW */
    e_MODULE_ID_L2_SRAM,             /**< L2/SRAM Memory-Mapped controller module*/
    e_MODULE_ID_DMA,            /**< DMA module */
    e_MODULE_ID_EPIC,           /**< Programmable interrupt controller */
    e_MODULE_ID_MSI,            /**< MSI registers */
    e_MODULE_ID_SRIO_PORTS,     /**< RapidIO controller */
    e_MODULE_ID_SRIO_MU,        /**< RapidIO msg unit module */
    e_MODULE_ID_QE,             /**< QUICC Engine module */
    e_MODULE_ID_IRAM,           /**< Instruction RAM */
    e_MODULE_ID_PRAM,           /**< Parameter RAM */
    e_MODULE_ID_QE_IC,          /**< Quicc engine interrupt controller */
    e_MODULE_ID_SPI_1,          /**< SPI module 1 */
    e_MODULE_ID_SPI_2,          /**< SPI module 2 */
    e_MODULE_ID_UCC_1,          /**< UCC module 1 */
    e_MODULE_ID_UCC_2,          /**< UCC module 2 */
    e_MODULE_ID_UCC_3,          /**< UCC module 3 */
    e_MODULE_ID_UCC_4,          /**< UCC module 4 */
    e_MODULE_ID_UCC_5,          /**< UCC module 5 */
    e_MODULE_ID_UCC_6,          /**< UCC module 6 */
    e_MODULE_ID_UCC_7,          /**< UCC module 7 */
    e_MODULE_ID_UCC_8,          /**< UCC module 8 */
    e_MODULE_ID_MCC_1,          /**< MCC module 1 */
    e_MODULE_ID_MCC_2,          /**< MCC module 2 */
    e_MODULE_ID_SI_1,           /**< SI  module 1 */
    e_MODULE_ID_SI_2,           /**< SI  module 2 */
    e_MODULE_ID_SI_RAM_1,       /**< SI RAM module 1 */
    e_MODULE_ID_SI_RAM_2,       /**< SI RAM module 2 */
    e_MODULE_ID_PAR_IO,         /**< Parallel I/O ports module */
    e_MODULE_ID_PAR_IO_INTR,    /**< Parallel I/O interrupts module */
    e_MODULE_ID_SEC,            /**< Security (sec 3.1) module */
    e_MODULE_ID_USB_DR,         /**< USB 2.0 module */
    e_MODULE_ID_ESDHC,          /**< eSDHC module */
    e_MODULE_ID_QE_TIMER,       /**< QE Timers module */
    e_MODULE_ID_QE_RTC,         /**< QE RTC */
    e_MODULE_ID_UPC_1,          /**< UPC module 1 */
    e_MODULE_ID_SDMA,           /**< Serial DMA */
    e_MODULE_ID_DEBUG,          /**< QE Debug module */
    e_MODULE_ID_RSP_1,          /**< Risc 1 special registers */
    e_MODULE_ID_RSP_2,          /**< Risc 2 special registers */
    e_MODULE_ID_PTP_1,          /**< PTP 1 module */
    e_MODULE_ID_PTP_2,          /**< PTP 2 module */
    e_MODULE_ID_TRB,            /**< QE trace-buffer module */
    e_MODULE_ID_PM,             /**< Performance Monitor module */
    e_MODULE_ID_DUMMY_LAST
} e_ModuleId;

#define NUM_OF_MODULES  e_MODULE_ID_DUMMY_LAST



/**************************************************************************//**
 @Description   Transaction source ID (for memory conrollers error reporting).
*//***************************************************************************/
typedef enum e_TransSrc
{
    e_TRANS_SRC_PCIE            = 0x2,  /**< PCI Express                    */
    e_TRANS_SRC_LBC             = 0x4,  /**< Enhanced local bus controller  */
    e_TRANS_SRC_SEC             = 0x7,  /**< Security engine                */
    e_TRANS_SRC_BOOT_SEQ        = 0xA,  /**< Boot sequencer                 */
    e_TRANS_SRC_ESDHC           = 0xB,  /**< eSDHC                          */
    e_TRANS_SRC_SRIO1           = 0xC,  /**< Serial RapidIO 1               */
    e_TRANS_SRC_SRIO2           = 0xD,  /**< Serial RapidIO 2               */
    e_TRANS_SRC_DDR1            = 0xF,  /**< DDR controller 1               */
    e_TRANS_SRC_CORE_INST       = 0x10, /**< Processor (instruction)        */
    e_TRANS_SRC_CORE_DATA       = 0x11, /**< Processor (data)               */
    e_TRANS_SRC_QE              = 0x14, /**< QUICC Engine block             */
    e_TRANS_SRC_DMA             = 0x15, /**< DMA                            */
    e_TRANS_SRC_DDR2            = 0x16, /**< DDR controller 2               */
    e_TRANS_SRC_SAP             = 0x17, /**< System access port             */
    e_TRANS_SRC_SRIO_MSG        = 0x1C, /**< RapidIO Message Unit           */
    e_TRANS_SRC_SRIO_DOORBELL   = 0x1D, /**< RapidIO Doorbell Unit          */
    e_TRANS_SRC_SRIO_PORT_WRITE = 0x1E  /**< RapidIO Port-write Unit        */
} e_TransSrc;


/**************************************************************************//**
 @Description   Local Access Window Target interface ID
*//***************************************************************************/
typedef enum e_MPC8569LawTargetId
{
    e_MPC8569_LAW_TARGET_PCIE   = 0x02, /**< PCI Express target interface */
    e_MPC8569_LAW_TARGET_LBC    = 0x04, /**< Local bus target interface */
    e_MPC8569_LAW_TARGET_SRIO_1 = 0x0C, /**< SRIO 1 target interface */
    e_MPC8569_LAW_TARGET_SRIO_2 = 0x0D, /**< SRIO 2 target interface */
    e_MPC8569_LAW_TARGET_DDR_1  = 0x0F, /**< Primary DDR target interface */
    e_MPC8569_LAW_TARGET_DDR_2  = 0x16, /**< Secondary DDR target interface */
    e_MPC8569_LAW_TARGET_NONE   = 0xFF  /**< Invalid target interface */
} e_MPC8569LawTargetId;


/**************************************************************************//**
 @Group         8569_init_grp MPC8569 Initialization Unit

 @Description   MPC8569 initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Part ID and revision number
*//***************************************************************************/

typedef enum e_MPC8569DeviceName
{
    e_MPC8569_REV_INVALID   = 0x00000000,   /**< Invalid revision                  */
    e_MPC8569E_REV_1_0      = 0x80880010,   /**< MPC8569E rev 1.0 with    security */
    e_MPC8569_REV_1_0       = 0x80800010,   /**< MPC8569  rev 1.0 without security */
    e_MPC8569E_REV_2_0      = 0x80880020,   /**< MPC8569E rev 2.0 with    security */
    e_MPC8569_REV_2_0       = 0x80800020,   /**< MPC8569  rev 2.0 without security */
    e_MPC8569E_REV_2_1      = 0x80880021,   /**< MPC8569E rev 2.1 with    security */
    e_MPC8569_REV_2_1       = 0x80800021    /**< MPC8569  rev 2.1 without security */
} e_MPC8569DeviceName;

/**************************************************************************//**
 @Description   Device Disable Register
*//***************************************************************************/
typedef enum e_MPC8569DeviceDisable
{
    e_MPC8569_DEV_DISABLE_PCIE     = 0x20000000,      /**< PCI Express controller disable   */
    e_MPC8569_DEV_DISABLE_LBC      = 0x08000000,      /**< Local bus controller disable     */
    e_MPC8569_DEV_DISABLE_SEC      = 0x01000000,      /**< Security disable                 */
    e_MPC8569_DEV_DISABLE_ESDHC    = 0x00200000,      /**< SD controller disable            */
    e_MPC8569_DEV_DISABLE_SRIO1    = 0x00080000,      /**< Serial RapidIO 1 controller disable */
    e_MPC8569_DEV_DISABLE_SRIO2    = 0x00040000,      /**< Serial RapidIO 2 controller disable */
    e_MPC8569_DEV_DISABLE_DDR1     = 0x00010000,      /**< DDR1 SDRAM controller disable    */
    e_MPC8569_DEV_DISABLE_E500     = 0x00008000,      /**< e500 core disable                */
    e_MPC8569_DEV_DISABLE_TB       = 0x00004000,      /**< Time base (timer facilities) of the e500 core disable */
    e_MPC8569_DEV_DISABLE_DDR2     = 0x00002000,      /**< DDR2 SDRAM controller disable    */
    e_MPC8569_DEV_DISABLE_QE       = 0x00000800,      /**< QUICC Engine disable             */
    e_MPC8569_DEV_DISABLE_DMA      = 0x00000400,      /**< DMA controller disable           */
    e_MPC8569_DEV_DISABLE_I2C      = 0x00000004,      /**< I2C controllers disabled         */
    e_MPC8569_DEV_DISABLE_DUART    = 0x00000002       /**< Dual UART controller disable     */
} e_MPC8569DeviceDisable;

/**************************************************************************//**
 @Function      MPC8569_ConfigAndInit

 @Description   General initiation of the chip registers.

 @Param         baseAddress  - (In) memory map start

 @Return        A handle to the MPC8569 data structure.
*//***************************************************************************/
t_Handle MPC8569_ConfigAndInit(uint32_t baseAddress);

/**************************************************************************//**
 @Function      MPC8569_Free

 @Description   Free all resources.

 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_Free(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_GetModuleBase
 
 @Description   returns the base address of a MPC8569 module's
				memory mapped registers.
 
 @Param      	h_MPC8569   - (In) The handle of the initialized MPC8569 object.
 @Param        	module      - (In) The module ID.
 
 @Return    	Base address of module's memory mapped registers.
                ILLEGAL_BASE in case of non-existent module
*//***************************************************************************/
uint64_t MPC8569_GetModuleBase(t_Handle h_MPC8569, e_ModuleId module);

/**************************************************************************//**
 @Function      MPC8569_GetRevInfo
 
 @Description   This routine enables access to chip and revision information.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return    	Part ID and revision.
*//***************************************************************************/
e_MPC8569DeviceName MPC8569_GetRevInfo(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_GetQeFactor
 
 @Description   returns QE multiplication factors. (This value is returned using
                two parameters to avoid using float parameter )
 @Param       	p_QeMulFactor   - (Out) returns E500 to CCB multification factor.
 @Param       	p_QeDivFactor   - (Out) returns E500 to CCB division factor.
 
 
 @Param      	h_MPC8569     - (In) The handle of the initialized MPC8569 object.

 @Return    	E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error  MPC8569_GetQeFactor(t_Handle h_MPC8569,uint32_t *p_QeMulFactor, uint32_t *p_QeDivFactor);

/**************************************************************************//**
 @Function      MPC8569_GetE500Factor
 
 @Description   returns system multiplication factor.
 
 @Param         h_MPC8569 - (In) a handle to the MPC8569 object.
 @Param       	p_E500MulFactor   - (Out) returns E500 to CCB multification factor.
 @Param       	p_E500DivFactor   - (Out) returns E500 to CCB division factor.
 
 @Return        E_OK on success; Other value otherwise.
*
*//***************************************************************************/
t_Error MPC8569_GetE500Factor(t_Handle h_MPC8569, uint32_t *p_E500MulFactor, uint32_t *p_E500DivFactor);

/**************************************************************************//**
 @Function      MPC8569_GetCcbFactor
 
 @Description   returns system multiplication factor.
 
 @Param         h_MPC8569 - (In) a handle to the MPC8569 object.
 
 @Return    	System multiplication factor.
*//***************************************************************************/
uint32_t MPC8569_GetCcbFactor(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_GetDdrFactor
 
 @Description   returns the multiplication factor of the clock in for the DDR clock .
 
 @Param         h_MPC8569 - (In) a handle to the MPC8569 object.
 
 @Return    	System multiplication factor.
*//***************************************************************************/
uint32_t MPC8569_GetDdrFactor(t_Handle h_MPC8569);


/**************************************************************************//**
 @Function      MPC8569_GetDdrType
 
 @Description   returns the multiplication factor of the clock in for the DDR clock .
 
 @Param         h_MPC8569 - (In) a handle to the MPC8569 object.
 @Param       	p_DdrType   - (Out) returns DDR type DDR1/DDR2/DDR3.
 
 @Return    	E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_GetDdrType(t_Handle h_MPC8569,e_DdrType *p_DdrType );

/**************************************************************************//**
 @Function      MPC8569_GetDdrMode
 
 @Description   returns the multiplication factor of the clock in for the DDR clock .
 
 @Param         h_MPC8569 - (In) a handle to the MPC8569 object.
 @Param       	p_DdrMode   - (Out) returns DDR mode 32/64 bit.
 
 @Return    	E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_GetDdrMode(t_Handle h_MPC8569,e_DdrMode *p_DdrMode );

/**************************************************************************//**
 @Function      MPC8569_GetPramSize
 
 @Description   Returns the size of the Multi-User RAM.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return    	The size of the Multi-User RAM.
*//***************************************************************************/
uint32_t MPC8569_GetPramSize(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_GetMiiOffset
 
 @Description   Returns the offset of MII management configuration control
                registers from the UCC memory map.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        Offset of MII management configuration control registers from
                the UCC memory map
*//***************************************************************************/
uint32_t MPC8569_GetMiiOffset(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_SelectSd1bit
 
 @Description   Configure platform mux between I2c2 and SD(1 bit mode)
                , Pins are configured to be used as SD 1bit mode.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_SelectSd1bit(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_FreeSd1bit
 
 @Description   Free Platform mux for the specific bits.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_FreeSd1bit(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_SelectSd4bit
 
 @Description   Configure platform mux between both I2c2 and Duart 0  and  SD(4 bits mode)
                , Pins are configured to be used as SD 4 bits mode.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_SelectSd4bit(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_FreeSd4bit
 
 @Description   Free Platform mux for the specific bits.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_FreeSd4bit(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_SelectDuart1
 
 @Description   Configure platform mux between Duart 0  and  SD(4 bits mode)
                , Pins are configured to be used as Uart 0, SD can be still configured to be used in 1 bit mode.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_SelectDuart1(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_FreeDuart1
 
 @Description   Free Platform mux for the specific bits.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_FreeDuart1(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_SelectI2C2
 
 @Description   Configure platform mux between I2c2 and SD(1 bit mode)
                , Pins are configured to be used as SD 1bit mode.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_SelectI2C2(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_FreeI2C2
 
 @Description   Free Platform mux for the specific bits.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error MPC8569_FreeI2C2(t_Handle h_MPC8569);


t_Error MPC8569_DeviceDisableStatus(t_Handle h_MPC8569,e_MPC8569DeviceDisable device);
t_Error MPC8569_DeviceDisable(t_Handle h_MPC8569,e_MPC8569DeviceDisable device);
t_Error MPC8569_DeviceEnable(t_Handle h_MPC8569,e_MPC8569DeviceDisable device);
t_Error MPC8569_SetSdWriteProtectInvert(t_Handle h_MPC8569,bool invert);

/**************************************************************************//**
 @Function      MPC8569_Reset
 
 @Description   Reset the chip.
 
 @Param      	h_MPC8569 - (In) The handle of the initialized MPC8569 object.
*//***************************************************************************/
void    MPC8569_Reset(t_Handle h_MPC8569);

/**************************************************************************//**
 @Function      MPC8569_EnableIpsecExtRequest
 
 @Description   connect the the SEC to the QE in order to suppoert IPsec Ucode.
 
 @Param[in]  	h_MPC8569 - The handle of the initialized MPC8569 object.
 @Param[in]  	intrId    - The id of the SEC interrupt need to be connected(0 or 1) .
 @Param[in]  	extReqNum - Number of external request.

*//***************************************************************************/
t_Error MPC8569_EnableIpsecExtRequest(t_Handle h_MPC8569,uint32_t intrId,uint8_t extReqNum);

/**************************************************************************//**
 @Function      MPC8569_EnableSecFastRate
 
 @Description   Enable SEC to work in the same frequncy as the system bus, so as I2C controllers.
 
 @Param[in]  	h_MPC8569 - The handle of the initialized MPC8569 object.

*//***************************************************************************/
t_Error MPC8569_EnableSecFastRate(t_Handle h_MPC8569);

/** @} */ /* end of 8569_init_grp group */
/** @} */ /* end of 8569_grp group */


#define CORE_E500V2

#define INTG_MAX_NUM_OF_CORES   1


/*****************************************************************************
 INTEGRATION-SPECIFIC MODULE CODES
******************************************************************************/
#define MODULE_UNKNOWN          0x00000000
#define MODULE_MPC8569          0x00010000
#define MODULE_QE               0x00020000
#define MODULE_MCC              0x00030000
#define MODULE_MCC_TRANS        0x00040000
#define MODULE_MCC_HDLC         0x00050000
#define MODULE_MCC_SS7          0x00060000
#define MODULE_UCC              0x00070000
#define MODULE_UCC_ATM          0x00080000
#define MODULE_UCC_ETH          0x00090000
#define MODULE_UCC_UART         0x000a0000
#define MODULE_UCC_TRANS        0x000b0000
#define MODULE_UCC_HDLC         0x000c0000
#define MODULE_UCC_QMC          0x000d0000
#define MODULE_MTC              0x000e0000
#define MODULE_DUART            0x000f0000
#define MODULE_SPI              0x00100000
#define MODULE_TDM              0x00110000
#define MODULE_SEC              0x00120000
#define MODULE_PCI              0x00130000
#define MODULE_I2C              0x00140000
#define MODULE_PIO              0x00150000
#define MODULE_UPC              0x00160000
#define MODULE_FLTR             0x00170000
#define MODULE_IW               0x00180000
#define MODULE_IMA              0x00190000
#define MODULE_PPPOHT           0x001a0000
#define MODULE_L2SWITCH         0x001b0000
#define MODULE_DMA              0x001c0000
#define MODULE_LAW              0x001d0000
#define MODULE_DDR              0x001e0000
#define MODULE_LBC              0x001f0000
#define MODULE_EPIC             0x00200000
#define MODULE_QE_IC            0x00210000
#define MODULE_MEM              0x00220000
#define MODULE_MM               0x00230000
#define MODULE_MMU              0x00240000
#define MODULE_CES              0x00250000
#define MODULE_L2_CACHE         0x00260000
#define MODULE_USB              0x00270000
#define MODULE_ECM              0x00280000
#define MODULE_SRIO             0x00290000
#define MODULE_MII              0x002a0000
#define MODULE_GTIMERS          0x002b0000
#define MODULE_IW_PPPOHT        0x002c0000
#define MODULE_IPHC             0x002d0000
#define MODULE_CORE             0x002e0000
#define MODULE_PM               0x002f0000
#define MODULE_ESDHC            0x00300000
#define MODULE_UCC_POS          0x00310000
#define MODULE_VP		        0x00320000
#define MODULE_QE_RTC   		0x00330000
#define MODULE_PTP              0x00340000

/*****************************************************************************
 QE IC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/

/**************************************************************************//**
 @Description   Enumeration of QE interrupts
*//***************************************************************************/
typedef enum e_QeIcIntrSrc
{
    e_QE_IC_INTR_SPI_2      = 1,    /**< QE interrupt source for SPI2               */
    e_QE_IC_INTR_SPI_1      = 2,    /**< QE interrupt source for SPI1               */
    e_QE_IC_INTR_RTT        = 3,    /**< QE interrupt source for the RISC timers    */
    e_QE_IC_INTR_SDMA       = 10,   /**< QE interrupt source for SDMA               */
    e_QE_IC_INTR_USB        = 11,   /**< QE interrupt source for USB                */
    e_QE_IC_INTR_TIMER_1    = 12,   /**< QE interrupt source for general timer 1    */
    e_QE_IC_INTR_TIMER_2    = 13,   /**< QE interrupt source for general timer 2    */
    e_QE_IC_INTR_TIMER_3    = 14,   /**< QE interrupt source for general timer 3    */
    e_QE_IC_INTR_TIMER_4    = 15,   /**< QE interrupt source for general timer 4    */
    e_QE_IC_INTR_PTP_1      = 17,   /**< QE interrupt source for PTP1               */
    e_QE_IC_INTR_PTP_2      = 19,   /**< QE interrupt source for PTP2               */
    e_QE_IC_INTR_VT         = 20,   /**< QE interrupt source for virtual tasks      */
    e_QE_IC_INTR_RTC        = 21,   /**< QE interrupt source for RTC                */
    e_QE_IC_INTR_EXT_REQ_1  = 25,   /**< QE interrupt source for external request 1 */
    e_QE_IC_INTR_EXT_REQ_2  = 26,   /**< QE interrupt source for external request 2 */
    e_QE_IC_INTR_EXT_REQ_3  = 27,   /**< QE interrupt source for external request 3 */
    e_QE_IC_INTR_EXT_REQ_4  = 28,   /**< QE interrupt source for external request 4 */
    e_QE_IC_INTR_UCC_1      = 32,   /**< QE interrupt source for UCC1               */
    e_QE_IC_INTR_UCC_2      = 33,   /**< QE interrupt source for UCC2               */
    e_QE_IC_INTR_UCC_3      = 34,   /**< QE interrupt source for UCC3               */
    e_QE_IC_INTR_UCC_4      = 35,   /**< QE interrupt source for UCC4               */
    e_QE_IC_INTR_MCC_1      = 36,   /**< QE interrupt source for MCC1               */
    e_QE_IC_INTR_UCC_5      = 40,   /**< QE interrupt source for UCC5               */
    e_QE_IC_INTR_UCC_6      = 41,   /**< QE interrupt source for UCC6               */
    e_QE_IC_INTR_UCC_7      = 42,   /**< QE interrupt source for UCC7               */
    e_QE_IC_INTR_UCC_8      = 43,   /**< QE interrupt source for UCC8               */
    e_QE_IC_INTR_MCC_2      = 44,   /**< QE interrupt source for MCC2               */
    e_QE_IC_INTR_DUMMY_LAST
 
} e_QeIcIntrSrc;


/*****************************************************************************
 LBC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/**************************************************************************//**
 @Group         lbc_exception_grp LBC Exception Unit

 @Description   LBC Exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        lbc_exbm
 
 @Collection    LBC Errors Bit Mask
 
                These errors are reported through the exceptions callback..
                The values can be or'ed in any combination in the errors mask
                parameter of the errors report structure.
 
                These errors can also be passed as a bit-mask to
                LBC_EnableErrorChecking() or LBC_DisableErrorChecking(),
                for enabling or disabling error checking.
 @{
*//***************************************************************************/
#define LBC_ERR_BUS_MONITOR     0x80000000  /**< Bus monitor error */
#define LBC_ERR_PARITY_ECC      0x20000000  /**< Parity error for GPCM/UPM */
#define LBC_ERR_WRITE_PROTECT   0x04000000  /**< Write protection error */
#define LBC_ERR_ATOMIC_WRITE    0x00800000  /**< Atomic write error */
#define LBC_ERR_ATOMIC_READ     0x00400000  /**< Atomic read error */
#define LBC_ERR_CHIP_SELECT     0x00080000  /**< Unrecognized chip select */

#define LBC_ERR_ALL             (LBC_ERR_BUS_MONITOR | LBC_ERR_PARITY_ECC | \
                                 LBC_ERR_WRITE_PROTECT | LBC_ERR_ATOMIC_WRITE | \
                                 LBC_ERR_ATOMIC_READ | LBC_ERR_CHIP_SELECT)
                                            /**< All possible errors */
/* @} */
/** @} */ /* end of lbc_exception_grp group */

/*****************************************************************************
 LBC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define LBC_INCORRECT_ERROR_REPORT_ERRATA

#define LBC_NUM_OF_BANKS            8
#define LBC_MAX_CS_SIZE             0x0000000100000000LL
#define LBC_ATOMIC_OPERATION_SUPPORT
#define LBC_PARITY_SUPPORT
#define LBC_ADDRESS_HOLD_TIME_CTRL
#define LBC_HIGH_CLK_DIVIDERS
#define LBC_FCM_AVAILABLE



/*****************************************************************************
 LAW INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define LAW_ARCH_CCB
#define LAW_NUM_OF_WINDOWS      10
#define LAW_MIN_WINDOW_SIZE     0x0000000000001000LL    /**< 4KB */
#define LAW_MAX_WINDOW_SIZE     0x0000001000000000LL    /**< 32GB */


/*****************************************************************************
 EPIC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define EPIC_NUM_OF_EXT_INTRS           12
#define EPIC_NUM_OF_INT_INTRS           64
#define EPIC_NUM_OF_TIMERS              4
#define EPIC_NUM_OF_MSG_INTRS           4
#define EPIC_NUM_OF_SMSG_INTRS          8

#define EPIC_OPTIMIZATION_FOR_QE

/**************************************************************************//**
 @Description   Enumeration of EPIC interrupts
*//***************************************************************************/
typedef enum e_EpicIntrSrc
{
    e_EPIC_INTR_TIMER_0         = (EPIC_INTR_BASE_TIMER + 0),
    e_EPIC_INTR_TIMER_1         = (EPIC_INTR_BASE_TIMER + 1),
    e_EPIC_INTR_TIMER_2         = (EPIC_INTR_BASE_TIMER + 2),
    e_EPIC_INTR_TIMER_3         = (EPIC_INTR_BASE_TIMER + 3),

    e_EPIC_INTR_IRQ_0           = (EPIC_INTR_BASE_EXTERNAL + 0),
    e_EPIC_INTR_IRQ_1           = (EPIC_INTR_BASE_EXTERNAL + 1),
    e_EPIC_INTR_IRQ_2           = (EPIC_INTR_BASE_EXTERNAL + 2),
    e_EPIC_INTR_IRQ_3           = (EPIC_INTR_BASE_EXTERNAL + 3),
    e_EPIC_INTR_IRQ_4           = (EPIC_INTR_BASE_EXTERNAL + 4),
    e_EPIC_INTR_IRQ_5           = (EPIC_INTR_BASE_EXTERNAL + 5),
    e_EPIC_INTR_IRQ_6           = (EPIC_INTR_BASE_EXTERNAL + 6),
    e_EPIC_INTR_IRQ_7           = (EPIC_INTR_BASE_EXTERNAL + 7),
    e_EPIC_INTR_IRQ_8           = (EPIC_INTR_BASE_EXTERNAL + 8),
    e_EPIC_INTR_IRQ_9           = (EPIC_INTR_BASE_EXTERNAL + 9),
    e_EPIC_INTR_IRQ_10          = (EPIC_INTR_BASE_EXTERNAL + 10),
    e_EPIC_INTR_IRQ_11          = (EPIC_INTR_BASE_EXTERNAL + 11),

    e_EPIC_INTR_L2              = (EPIC_INTR_BASE_INTERNAL + 0),
    e_EPIC_INTR_ECM             = (EPIC_INTR_BASE_INTERNAL + 1),
    e_EPIC_INTR_DDR             = (EPIC_INTR_BASE_INTERNAL + 2),
    e_EPIC_INTR_LBC             = (EPIC_INTR_BASE_INTERNAL + 3),
    e_EPIC_INTR_DMA_0           = (EPIC_INTR_BASE_INTERNAL + 4),
    e_EPIC_INTR_DMA_1           = (EPIC_INTR_BASE_INTERNAL + 5),
    e_EPIC_INTR_DMA_2           = (EPIC_INTR_BASE_INTERNAL + 6),
    e_EPIC_INTR_DMA_3           = (EPIC_INTR_BASE_INTERNAL + 7),
    e_EPIC_INTR_PCIE            = (EPIC_INTR_BASE_INTERNAL + 10),
    e_EPIC_INTR_DUART           = (EPIC_INTR_BASE_INTERNAL + 26),
    e_EPIC_INTR_I2C             = (EPIC_INTR_BASE_INTERNAL + 27),
    e_EPIC_INTR_PM              = (EPIC_INTR_BASE_INTERNAL + 28),
    e_EPIC_INTR_SEC_PRIMARY     = (EPIC_INTR_BASE_INTERNAL + 29),
    e_EPIC_INTR_QE_LOW          = (EPIC_INTR_BASE_INTERNAL + 30),
    e_EPIC_INTR_QE_IO_PORTS     = (EPIC_INTR_BASE_INTERNAL + 31),
    e_EPIC_INTR_RIO_ERR_WR      = (EPIC_INTR_BASE_INTERNAL + 32),
    e_EPIC_INTR_RIO_DB_OUT      = (EPIC_INTR_BASE_INTERNAL + 33),
    e_EPIC_INTR_RIO_DB_IN       = (EPIC_INTR_BASE_INTERNAL + 34),
    e_EPIC_INTR_RIO_OUT_MSG0    = (EPIC_INTR_BASE_INTERNAL + 37),
    e_EPIC_INTR_RIO_IN_MSG0     = (EPIC_INTR_BASE_INTERNAL + 38),
    e_EPIC_INTR_RIO_OUT_MSG1    = (EPIC_INTR_BASE_INTERNAL + 39),
    e_EPIC_INTR_RIO_IN_MSG1     = (EPIC_INTR_BASE_INTERNAL + 40),
    e_EPIC_INTR_SEC_SECONDARY   = (EPIC_INTR_BASE_INTERNAL + 42),
    e_EPIC_INTR_QE_IRAM_ERR     = (EPIC_INTR_BASE_INTERNAL + 43),
    e_EPIC_INTR_QE_MURAM_ERR    = (EPIC_INTR_BASE_INTERNAL + 44),
    e_EPIC_INTR_QE_HIGH         = (EPIC_INTR_BASE_INTERNAL + 47),
    e_EPIC_INTR_ESDHC           = (EPIC_INTR_BASE_INTERNAL + 56),

    e_EPIC_INTR_MSG_0           = (EPIC_INTR_BASE_MSG + 0),
    e_EPIC_INTR_MSG_1           = (EPIC_INTR_BASE_MSG + 1),
    e_EPIC_INTR_MSG_2           = (EPIC_INTR_BASE_MSG + 2),
    e_EPIC_INTR_MSG_3           = (EPIC_INTR_BASE_MSG + 3),
 
    e_EPIC_INTR_IPI_0           = (EPIC_INTR_BASE_IPI + 0),
    e_EPIC_INTR_IPI_1           = (EPIC_INTR_BASE_IPI + 1),
    e_EPIC_INTR_IPI_2           = (EPIC_INTR_BASE_IPI + 2),
    e_EPIC_INTR_IPI_3           = (EPIC_INTR_BASE_IPI + 3),
 
    e_EPIC_INTR_SHARED_MSG_0    = (EPIC_INTR_BASE_SHARED_MSG + 0),
    e_EPIC_INTR_SHARED_MSG_1    = (EPIC_INTR_BASE_SHARED_MSG + 1),
    e_EPIC_INTR_SHARED_MSG_2    = (EPIC_INTR_BASE_SHARED_MSG + 2),
    e_EPIC_INTR_SHARED_MSG_3    = (EPIC_INTR_BASE_SHARED_MSG + 3),
    e_EPIC_INTR_SHARED_MSG_4    = (EPIC_INTR_BASE_SHARED_MSG + 4),
    e_EPIC_INTR_SHARED_MSG_5    = (EPIC_INTR_BASE_SHARED_MSG + 5),
    e_EPIC_INTR_SHARED_MSG_6    = (EPIC_INTR_BASE_SHARED_MSG + 6),
    e_EPIC_INTR_SHARED_MSG_7    = (EPIC_INTR_BASE_SHARED_MSG + 7),

    e_EPIC_INTR_SPURIOUS,
    e_EPIC_INTR_DUMMY_LAST,
 
    e_EPIC_INTR_RESERVED        = 0xFAFA
 
} e_EpicIntrSrc;


/*****************************************************************************
 SPI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SPI_QE_INTEGRATION
#define SPI_NUM_OF_CONTROLLERS          2


/*****************************************************************************
 UCC GETH INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define UCC_GETH_INTERFACES_MAP \
    { { e_ENET_IF_MII,      { 1, 1, 1, 1, 1, 1, 1, 1 } }, \
      { e_ENET_IF_RMII,     { 1, 1, 1, 1, 1, 1, 1, 1 } }, \
      { e_ENET_IF_GMII,     { 1, 1, 0, 0, 0, 0, 0, 0 } }, \
      { e_ENET_IF_RGMII,    { 1, 1, 1, 1, 0, 0, 0, 0 } }, \
      { e_ENET_IF_TBI,      { 1, 1, 0, 0, 0, 0, 0, 0 } }, \
      { e_ENET_IF_RTBI,     { 1, 1, 1, 1, 0, 0, 0, 0 } }, \
      { e_ENET_IF_SGMII,    { 0, 0, 0, 0, 0, 1, 0, 1 } }, \
      { e_ENET_IF_SMII,     { 0, 0, 0, 0, 0, 1, 0, 1 } } }
 
#define UCC_GETH_PTP_SUPPORT
#define UCC_GETH_CACHE_STASHING_SUPPORT
#define UCC_GETH_VP_MODE_SUPPORT


/*****************************************************************************
 SRIO INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SRIO_NUM_OF_PORTS   2
/* Note: the define is related to SRIO3 errata */
#define SRIO_MSG_PRIO_0_NOT_SUPPORTED_ERRATA

/*****************************************************************************
 PCI/PCIe INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/* PCI modules registration */
void PCIE_CTRL_Register(t_Handle h_PciCtrl);

#define PCI_MODULES_REGISTRATION \
    { { e_MODULE_ID_PCIE, PCIE_CTRL_Register, e_MODULE_ID_PCIE, e_MODULE_ID_PCIE_ATMU, e_MODULE_ID_MSI }}

#define PCI_MAX_INBOUND_WINDOWS_NUM     4
#define PCI_MAX_OUTBOUND_WINDOWS_NUM    5

/**************************************************************************//**
 @Description   Target interface of an inbound window
*//***************************************************************************/
typedef enum e_PciTargetInterface
{
    e_PCI_TARGET_PCI            = 0x0,  /**<  PCI target interface                                       */
    e_PCI_TARGET_PCIE           = 0x2,  /**<  PCI Express 1 target interface                               */
    e_PCI_TARGET_SRIO           = 0xC,  /**<  SRIO target interface                                      */
    e_PCI_TARGET_LOCAL_MEMORY   = 0xF   /**<  Local Memory (DDR SDRAM, Local Bus, SRAM) target interface */
 
} e_PciTargetInterface;

/*****************************************************************************
 DDR INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define DDR_NUM_OF_VALID_CS     4

/*****************************************************************************
 DMA INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define DMA_NUM_OF_CONTROLLERS  1

/*****************************************************************************
 SI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SI_MODULE_ID_MAP   \
    { { e_MODULE_ID_SI_1, 0 }, \
      { e_MODULE_ID_SI_2, 1 } }

/*****************************************************************************
 1588 INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define PTP_V2

/*****************************************************************************
 USB INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define USB11_MULTI_DATA_BUS
//#define CONFIG_USB11_HAS_NO_RT_SOF
//#define CONFIG_USB11_HAS_EOP_MISSING_BUG
//#define CONFIG_USB11_HAS_UC_RT_SOF

/*****************************************************************************
 UPC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define UPC_NUM_OF_DEVS         4

/*****************************************************************************
 TDM INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
//#define TDM_DISABLE_BUG_RTSYNC_WORKAROUND


#define TDM_HIGH_SPEED_MODE_SUPPORT

/*****************************************************************************
 MCC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define MCC_INIT_CMD_BUG_WORKAROUND_ERRATA

/*****************************************************************************
 General utilities INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/


/*****************************************************************************
 ESDHC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define ESDHC_INTERRUPT_NOT_NEGATED_ERRATA
#define ESDHC_ASYNC_CMD12_ERRATA

/*****************************************************************************
 GETH INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/

/*****************************************************************************
 QE IW INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/*****************************************************************************
 SEC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SEC_NUM_OF_CHANNELS     4

/*****************************************************************************
 QE INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/

#define PTP_QE_TX_ACCELERATION

#endif /* __PART_INTEGRATION_EXT_H */
