/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          pm_integration_ext.h

 @Description   MPC8569 Performance Monitor external structures and definitions.
*//***************************************************************************/

#ifndef __PM_INTEGRATION_EXT_H
#define __PM_INTEGRATION_EXT_H


/**************************************************************************//**
 @Group         pm_grp Performance Monitor Application Programming Interface

 @Description   Performance Monitor API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pm_control Performance Monitor Control Unit

 @Description   Performance Monitor Runtime Control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Performance Monitor Events.
*//***************************************************************************/
/* Note - the values are ordered in the following way:
   2 msb bits  - the clock divisor for the specific counter
   8 next bits - the counter number - 0xff means don't care about counter number
   8 lsb bits  - the event value (in decimal)
*/
typedef enum e_PmEvent
{
      /* General Events */
      e_PM_EV_SYSTEM_CYCLES                                                     = 0x10000 /**< System cycles CCB (platform) clock cycles */
 
      /* DDR 1 Memory Controller Events */
    , e_PM_EV_DDR1_CYCLES_IN_READ_TRANSFERS                                     = 0x1ff3c /**< Each data beat returned to the memory controller on the DRAM interface */
    , e_PM_EV_DDR1_CYCLES_IN_READ_OR_WRITE_TRANSFERS                            = 0x10139 /**< Each data beat transferred to or from the DRAM */
    , e_PM_EV_DDR1_PIPELINED_READ_MISSES_IN_ROW_OPEN_TABLE                      = 0x10200 /**< Row open table read misses issued while a read is outstanding */
    , e_PM_EV_DDR1_PIPELINED_READ_OR_WRITE_MISSES_IN_ROW_OPEN_TABLE             = 0x1033c /**< Row open table read or write misses issued while a read or write is outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_MISSES_IN_ROW_OPEN_TABLE                  = 0x10400 /**< Row open table read misses issued when no reads are outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_OR_WRITE_MISSES_IN_ROW_OPEN_TABLE         = 0x10538 /**< Row open table read or write misses issued when no reads or writes are outstanding */
    , e_PM_EV_DDR1_PIPELINED_READ_HITS_IN_ROW_OPEN_TABLE                        = 0x10600 /**< Row open table read hits issued when a read is outstanding */
    , e_PM_EV_DDR1_PIPELINED_READ_OR_WRITE_HITS_IN_ROW_OPEN_TABLE               = 0x10739 /**< Row open table read or write hits issued when a read or write is outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_HITS_IN_ROW_OPEN_TABLE                    = 0x10800 /**< Row open table read hits issued when no reads are outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_OR_WRITE_HITS_IN_ROW_OPEN_TABLE           = 0x10100 /**< Row open table read or write hits issued when no reads or writes are outstanding */
    , e_PM_EV_DDR1_FORCED_PAGE_CLOSINGS_NOT_CAUSED_BY_A_REFRESH                 = 0x10201 /**< Precharges issued to the DRAM for any reason except refresh */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_MISSES                                        = 0x10300 /**< Transactions that miss in the row open table */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS                                          = 0x10401 /**< Transaction that hit in the row open table */
    , e_PM_EV_DDR1_FORCED_PAGE_CLOSINGS                                         = 0x10500 /**< Forced page closings including those due to refreshes */
    , e_PM_EV_DDR1_READ_MODIFY_WRITE_TRANSACTIONS_DUE_TO_ECC_OR_FCRAM           = 0x1ff3b /**< If ECC is enabled, or FCRAM mode, and a transaction requires byte enables, a read-modify-write sequence is issued on the DRAM interface. */
    , e_PM_EV_DDR1_FORCED_PAGE_CLOSINGS_DUE_TO_COLLISION_WITH_BANK_AND_SUB_BANK = 0x1ff3e /**< Increments if a new transaction must be issued to an active bank and sub-bank that has a different row open */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_CORE                                    = 0x10301 /**< Total number of reads or writes from core */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_ETHERNET                                = 0x10403 /**< Total number of reads or writes from Ethernets 1-4 */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_HSI                                     = 0x1ff3f /**< Total number of reads or writes from high speed interfaces (PCIe/SRIO) */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_DMA                                     = 0x10601 /**< Total number of reads or writes from DMA */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_SEC                                     = 0x10602 /**< Total number of reads or writes from security engine */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_CORE            = 0x10701 /**< Total number of row open table hits for reads or writes from core */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_ETHERNET        = 0x10802 /**< Total number of row open table hits for reads or writes from Ethernet 1-4 */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_HSI             = 0x1ff13 /**< Total number of row open table hits for reads or writes from high speed interfaces (PCIe/SRIO) */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_DMA             = 0x1010c /**< Total number of row open table hits for reads or writes from DMA */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SEC             = 0x10211 /**< Total number of row open table hits for reads or writes from security engine */
    , e_PM_EV_DDR1_ALL_COUNTERS_VALID                                           = 0x1040f /**< All counters are valid */

      /* DDR 2 Memory Controller Events */
    , e_PM_EV_DDR2_CYCLES_IN_READ_TRANSFERS                                     = 0x1ff16 /**< Each data beat returned to the memory controller on the DRAM interface */
    , e_PM_EV_DDR2_CYCLES_IN_READ_OR_WRITE_TRANSFERS                            = 0x10202 /**< Each data beat transferred to or from the DRAM */
    , e_PM_EV_DDR2_PIPELINED_READ_MISSES_IN_ROW_OPEN_TABLE                      = 0x1033e /**< Row open table read misses issued while a read is outstanding */
    , e_PM_EV_DDR2_PIPELINED_READ_OR_WRITE_MISSES_IN_ROW_OPEN_TABLE             = 0x10425 /**< Row open table read or write misses issued while a read or write is outstanding */
    , e_PM_EV_DDR2_NON_PIPELINED_READ_MISSES_IN_ROW_OPEN_TABLE                  = 0x10528 /**< Row open table read misses issued when no reads are outstanding */
    , e_PM_EV_DDR2_NON_PIPELINED_READ_OR_WRITE_MISSES_IN_ROW_OPEN_TABLE         = 0x1060b /**< Row open table read or write misses issued when no reads or writes are outstanding */
    , e_PM_EV_DDR2_PIPELINED_READ_HITS_IN_ROW_OPEN_TABLE                        = 0x10735 /**< Row open table read hits issued when a read is outstanding */
    , e_PM_EV_DDR2_PIPELINED_READ_OR_WRITE_HITS_IN_ROW_OPEN_TABLE               = 0x1081b /**< Row open table read or write hits issued when a read or write is outstanding */
    , e_PM_EV_DDR2_NON_PIPELINED_READ_HITS_IN_ROW_OPEN_TABLE                    = 0x10109 /**< Row open table read hits issued when no reads are outstanding */
    , e_PM_EV_DDR2_NON_PIPELINED_READ_OR_WRITE_HITS_IN_ROW_OPEN_TABLE           = 0x1020d /**< Row open table read or write hits issued when no reads or writes are outstanding */
    , e_PM_EV_DDR2_FORCED_PAGE_CLOSINGS_NOT_CAUSED_BY_A_REFRESH                 = 0x1030d /**< Precharges issued to the DRAM for any reason except refresh */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_MISSES                                        = 0x10426 /**< Transactions that miss in the row open table */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_HITS                                          = 0x10508 /**< Transaction that hit in the row open table */
    , e_PM_EV_DDR2_FORCED_PAGE_CLOSINGS                                         = 0x1030e /**< Forced page closings including those due to refreshes */
    , e_PM_EV_DDR2_READ_MODIFY_WRITE_TRANSACTIONS_DUE_TO_ECC_OR_FCRAM           = 0x1ff17 /**< If ECC is enabled, or FCRAM mode, and a transaction requires byte enables, a read-modify-write sequence is issued on the DRAM interface. */
    , e_PM_EV_DDR2_FORCED_PAGE_CLOSINGS_DUE_TO_COLLISION_WITH_BANK_AND_SUB_BANK = 0x1ff18 /**< Increments if a new transaction must be issued to an active bank and sub-bank that has a different row open */
    , e_PM_EV_DDR2_READS_OR_WRITES_FROM_CORE                                    = 0x1030f /**< Total number of reads or writes from core */
    , e_PM_EV_DDR2_READS_OR_WRITES_FROM_ETHERNET                                = 0x1050f /**< Total number of reads or writes from Ethernets 1-4 */
    , e_PM_EV_DDR2_READS_OR_WRITES_FROM_HSI                                     = 0x1ff19 /**< Total number of reads or writes from high speed interfaces (PCIe/SRIO) */
    , e_PM_EV_DDR2_READS_OR_WRITES_FROM_DMA                                     = 0x10620 /**< Total number of reads or writes from DMA */
    , e_PM_EV_DDR2_READS_OR_WRITES_FROM_SEC                                     = 0x1071b /**< Total number of reads or writes from security engine */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_CORE            = 0x1081d /**< Total number of row open table hits for reads or writes from core */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_ETHERNET        = 0x1071e /**< Total number of row open table hits for reads or writes from Ethernet 1-4 */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_HSI             = 0x1ff3d /**< Total number of row open table hits for reads or writes from high speed interfaces (PCIe/SRIO) */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_DMA             = 0x10212 /**< Total number of row open table hits for reads or writes from DMA */
    , e_PM_EV_DDR2_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SEC             = 0x10329 /**< Total number of row open table hits for reads or writes from security engine */
    , e_PM_EV_DDR2_ALL_COUNTERS_VALID                                           = 0x10501 /**< All counters are valid */

      /* DMA Controller Events */
    , e_PM_EV_DMA_CHANNEL_0_READ_REQUESTS                                       = 0x20102 /**< DMA channel 0 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_1_READ_REQUESTS                                       = 0x20205 /**< DMA channel 1 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_2_READ_REQUESTS                                       = 0x20304 /**< DMA channel 2 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_3_READ_REQUESTS                                       = 0x20406 /**< DMA channel 3 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_0_WRITE_REQUESTS                                      = 0x20103 /**< DMA channel 0 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_1_WRITE_REQUESTS                                      = 0x20206 /**< DMA channel 1 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_2_WRITE_REQUESTS                                      = 0x20305 /**< DMA channel 2 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_3_WRITE_REQUESTS                                      = 0x20407 /**< DMA channel 3 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_0_DESCRIPTOR_REQUESTS                                 = 0x20529 /**< DMA channel 0 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_1_DESCRIPTOR_REQUESTS                                 = 0x2062c /**< DMA channel 1 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_2_DESCRIPTOR_REQUESTS                                 = 0x20729 /**< DMA channel 2 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_3_DESCRIPTOR_REQUESTS                                 = 0x20829 /**< DMA channel 3 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_0_READ_DWORD_OR_LESS                                  = 0x20104 /**< DMA channel 0 read double word valid */
    , e_PM_EV_DMA_CHANNEL_1_READ_DWORD_OR_LESS                                  = 0x20207 /**< DMA channel 1 read double word valid */
    , e_PM_EV_DMA_CHANNEL_2_READ_DWORD_OR_LESS                                  = 0x20306 /**< DMA channel 2 read double word valid */
    , e_PM_EV_DMA_CHANNEL_3_READ_DWORD_OR_LESS                                  = 0x20408 /**< DMA channel 3 read double word valid */
    , e_PM_EV_DMA_CHANNEL_0_WRITE_DWORD_OR_LESS                                 = 0x20105 /**< DMA channel 0 write double word valid */
    , e_PM_EV_DMA_CHANNEL_1_WRITE_DWORD_OR_LESS                                 = 0x20208 /**< DMA channel 1 write double word valid */
    , e_PM_EV_DMA_CHANNEL_2_WRITE_DWORD_OR_LESS                                 = 0x20307 /**< DMA channel 2 write double word valid */
    , e_PM_EV_DMA_CHANNEL_3_WRITE_DWORD_OR_LESS                                 = 0x20409 /**< DMA channel 3 write double word valid */

      /* e500 Coherency Module (ECM) Events */
    , e_PM_EV_ECM_REQUEST_WAIT_CORE                                             = 0x1080d /**< Asserted for every cycle in which core request occurs */
    , e_PM_EV_ECM_DISPATCH                                                      = 0x1ff0f /**< ECM dispatch (includes address onlys); Note: all ECM dispatch events are for committed dispatches */
    , e_PM_EV_ECM_DISPATCH_FROM_CORE                                            = 0x10110 /**< ECM dispatch from core (includes address onlys) */
    , e_PM_EV_ECM_DISPATCH_FROM_QE                                              = 0x10313 /**< ECM dispatch from QE (source ID = 10100) */
    , e_PM_EV_ECM_DISPATCH_FROM_SRIO_1                                          = 0x10511 /**< ECM dispatch from SRIO-1 (source ID = 01100) */
    , e_PM_EV_ECM_DISPATCH_FROM_SRIO_2                                          = 0x10415 /**< ECM dispatch from SRIO-2 (source ID = 01101) */
    , e_PM_EV_ECM_DISPATCH_FROM_PCIE                                            = 0x10710 /**< ECM dispatch from PCIe (source ID = 00010) */
    , e_PM_EV_ECM_DISPATCH_FROM_DMA                                             = 0x1070e /**< ECM dispatch from DMA (source ID = 10101) */
    , e_PM_EV_ECM_DISPATCH_FROM_SEC                                             = 0x10811 /**< ECM dispatch from security engine (source ID = 00111) */
    , e_PM_EV_ECM_DISPATCH_FROM_RIO_MU_OR_PW                                    = 0x10912 /**< ECM dispatch from RIO Message Unit, Door Bell, or Port Write (source ID = 111xx) */
    , e_PM_EV_ECM_DISPATCH_FROM_LBC                                             = 0x10918 /**< ECM dispatch from Enhanced Local Bus Controller (source ID = 00100) */
    , e_PM_EV_ECM_DISPATCH_TO_DDR_1                                             = 0x10416 /**< ECM dispatch to DDR 1 */
    , e_PM_EV_ECM_DISPATCH_TO_DDR_2                                             = 0x10711 /**< ECM dispatch to DDR 2 */
    , e_PM_EV_ECM_DISPATCH_TO_L2                                                = 0x1080f /**< ECM dispatch to L2 (excludes addr only) */
    , e_PM_EV_ECM_DISPATCH_TO_LBC                                               = 0x10612 /**< ECM dispatch to Enhanced Local Bus Controller */
    , e_PM_EV_ECM_DISPATCH_TO_SRIO_1                                            = 0x1070f /**< ECM dispatch to SRIO-1 */
    , e_PM_EV_ECM_DISPATCH_TO_SRIO_2                                            = 0x10910 /**< ECM dispatch to SRIO-2 */
    , e_PM_EV_ECM_DISPATCH_TO_PCIE                                              = 0x10611 /**< ECM dispatch to PCIe */
    , e_PM_EV_ECM_DISPATCH_WRITE                                                = 0x10111 /**< ECM dispatch write */
    , e_PM_EV_ECM_DISPATCH_READ                                                 = 0x10417 /**< ECM dispatch read */
    , e_PM_EV_ECM_DISPATCH_READ_ATOMIC_CLR_SET_DEC_INC                          = 0x10917 /**< ECM dispatch read clear atomics */

      /* Interrupt Controller (PIC) Events */
    , e_PM_EV_PIC_TOTAL_INTERRUPT_COUNT                                         = 0x1ff1a /**< Total number of interrupts serviced */
    , e_PM_EV_PIC_INTERRUPT_ACTIVE_CYCLES                                       = 0x1083e /**< Number of cycles there is an active interrupt */
    , e_PM_EV_PIC_INTERRUPT_SERVICE_CYCLES                                      = 0x10213 /**< Number of cycles there is an interrupt currently being serviced */
    , e_PM_EV_PIC_INTERRUPT_SELECT_0                                            = 0x10138 /**< Select 0: interrupt count over threshold (note that a request will be acknowledged only if it is not masked, and its priority is greater than 0) */
    , e_PM_EV_PIC_INTERRUPT_SELECT_1                                            = 0x1033b /**< Select 1: interrupt count over threshold */
    , e_PM_EV_PIC_INTERRUPT_SELECT_2                                            = 0x10537 /**< Select 2: interrupt count over threshold */
    , e_PM_EV_PIC_INTERRUPT_SELECT_3                                            = 0x1063c /**< Select 3: interrupt count over threshold */

      /* Local Bus Events */
    , e_PM_EV_LBC_CORRECTABLE_ECC_ERRORS                                        = 0x1090e /**< Number of correctable ECC errors */
    , e_PM_EV_LBC_UNCORRECTABLE_ECC_ERRORS                                      = 0x10a02 /**< Number of uncorrectable ECC errors */
    , e_PM_EV_LBC_ATOMIC_LOCK_TIMEOUTS_PORT_1                                   = 0x10531 /**< Number of atomic lock time-outs for port 1 */
    , e_PM_EV_LBC_ATOMIC_LOCK_TIMEOUTS_PORT_2                                   = 0x10636 /**< Number of atomic lock time-outs for port 2 */
    , e_PM_EV_LBC_FCM_BUFFER_READ_CYCLES                                        = 0x10838 /**< Number of cycles for a read from FCM buffer RAM */
    , e_PM_EV_LBC_FCM_BUFFER_WRITE_CYCLES                                       = 0x1092a /**< Number of cycles for a write to FCM buffer RAM */
    , e_PM_EV_LBC_GPCM_READ_CYCLES                                              = 0x10135 /**< Number of cycles a read is taking in GPCM */
    , e_PM_EV_LBC_UPM_READ_CYCLES                                               = 0x1023a /**< Number of cycles a read is taking in UPM */
    , e_PM_EV_LBC_GPCM_WRITE_CYCLES                                             = 0x10438 /**< Number of cycles a write is taking in GPCM */
    , e_PM_EV_LBC_UPM_WRITE_CYCLES                                              = 0x10532 /**< Number of cycles a write is taking in UPM */

      /* PCI Express Events */
    , e_PM_EV_PCIE_INBOUND_READ                                                 = 0x20837 /**< A single pulse to indicate an inbound PCIe read has occurred */
    , e_PM_EV_PCIE_INBOUND_WRITE                                                = 0x20925 /**< A single pulse to indicate an inbound PCIe write has occurred */
    , e_PM_EV_PCIE_INBOUND_DATA                                                 = 0x2053c /**< A level signal to indicate the amount of data transferred if any for inbound PCIe request; Active for every beat of PCIe data */
    , e_PM_EV_PCIE_OUTBOUND_READ                                                = 0x2063e /**< A single pulse to indicate an outbound PCIe read has occurred */
    , e_PM_EV_PCIE_OUTBOUND_WRITE                                               = 0x2073d /**< A single pulse to indicate an outbound PCIe write has occurred */
    , e_PM_EV_PCIE_OUTBOUND_DATA                                                = 0x2083c /**< A level signal to indicate the amount of data transferred if any for outbound PCIe request; Active for every beat of PCIe data */
    , e_PM_EV_PCIE_INBOUND_STATIC_QUEUE_0_START                                 = 0x2ff36 /**< Lifetime of ISQ entry 0 or 6 (duration threshold) */
    , e_PM_EV_PCIE_OUTBOUND_STATIC_QUEUE_0_START                                = 0x2ff37 /**< Lifetime of OSQ entry 0 (duration threshold) */
    , e_PM_EV_PCIE_P1_SYMBOL_DISPARITY_ERRORS                                   = 0x20820 /**< Number of times a receive PCIe symbol disparity error was detected */
    , e_PM_EV_PCIE_P1_SYMBOL_DECODE_ERRORS                                      = 0x2090b /**< Number of times an unrecognizable PCIe symbol (decode error) was detected */
    , e_PM_EV_PCIE_P2_SYMBOL_DISPARITY_ERRORS                                   = 0x2090a /**< Number of times a receive PCIe symbol disparity error was detected */
    , e_PM_EV_PCIE_P2_SYMBOL_DECODE_ERRORS                                      = 0x20229 /**< Number of times an unrecognizable PCIe symbol (decode error) was detected */
    , e_PM_EV_PCIE_P3_SYMBOL_DISPARITY_ERRORS                                   = 0x2022a /**< Number of times a receive PCIe symbol disparity error was detected */
    , e_PM_EV_PCIE_P3_SYMBOL_DECODE_ERRORS                                      = 0x2032e /**< Number of times an unrecognizable PCIe symbol (decode error) was detected */
    , e_PM_EV_PCIE_P4_SYMBOL_DISPARITY_ERRORS                                   = 0x2032f /**< Number of times a receive PCIe symbol disparity error was detected */
    , e_PM_EV_PCIE_P4_SYMBOL_DECODE_ERRORS                                      = 0x20431 /**< Number of times an unrecognizable PCIe symbol (decode error) was detected */

      /* SRIO-1 Events */
    , e_PM_EV_SRIO1_INBOUND_PACKETS_ACCEPTED                                    = 0x2013c /**< Packet accepted on RIO */
    , e_PM_EV_SRIO1_INBOUND_PACKETS_ACCEPTED_PRIORITY_0                         = 0x2023e /**< Packet accepted from RIO of priority 0 */
    , e_PM_EV_SRIO1_INBOUND_NON_IDLES_RECEIVED                                  = 0x2043e /**< Non idles received. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count */
    , e_PM_EV_SRIO1_INBOUND_PACKETS_RETRIED                                     = 0x20513 /**< Packet retry occurred, any cause */
    , e_PM_EV_SRIO1_INBOUND_BUFFER_FULL                                         = 0x20608 /**< Clock cycle occurred in which the inbound buffer is full to any priority (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true. */
    , e_PM_EV_SRIO1_INBOUND_BUFFER_FULL_PRIORITY_0                              = 0x2073b /**< Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true.*/
    , e_PM_EV_SRIO1_OUTBOUND_NON_IDLES_TRANSMITTED                              = 0x2ff1c /**< Non idles transmitted. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count */
    , e_PM_EV_SRIO1_OUTBOUND_PACKETS_SENT                                       = 0x2023f /**< Outbound packet sent to RapidIO interface */
    , e_PM_EV_SRIO1_OUTBOUND_PACKETS_MISALIGNED                                 = 0x2040c /**< Outbound packet was misaligned */
    , e_PM_EV_SRIO1_OUTBOUND_PACKETS_RETRIED                                    = 0x20514 /**< Outbound packet was retried */
    , e_PM_EV_SRIO1_OUTBOUND_PACKETS_REORDERED                                  = 0x20609 /**< Outbound packet was reordered */
    , e_PM_EV_SRIO1_OUTBOUND_PACKETS_SENT_PRIORITY_0                            = 0x2073c /**< Outbound packet sent to RIO of priority 0 */
    , e_PM_EV_SRIO1_OUTBOUND_BUFFER_FULL                                        = 0x2083b /**< Clock cycle occurred in which the outbound buffer is full to any priority. Event asserted for as many clock cycles as this is true. */
    , e_PM_EV_SRIO1_OUTBOUND_BUFFER_FULL_PRIORITY_0                             = 0x20927 /**< Clock cycle occurred in which the outbound buffer is full to priority 0. Event asserted for as many clock cycles as this is true. */

      /* SRIO-2 Events */
    , e_PM_EV_SRIO2_INBOUND_PACKETS_ACCEPTED                                    = 0x20137 /**< Packet accepted on RIO */
    , e_PM_EV_SRIO2_INBOUND_PACKETS_ACCEPTED_PRIORITY_0                         = 0x20236 /**< Packet accepted from RIO of priority 0 */
    , e_PM_EV_SRIO2_INBOUND_NON_IDLES_RECEIVED                                  = 0x2043b /**< Non idles received. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count */
    , e_PM_EV_SRIO2_INBOUND_PACKETS_RETRIED                                     = 0x2051e /**< Packet retry occurred, any cause */
    , e_PM_EV_SRIO2_INBOUND_BUFFER_FULL                                         = 0x2060f /**< Clock cycle occurred in which the inbound buffer is full to any priority (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true. */
    , e_PM_EV_SRIO2_INBOUND_BUFFER_FULL_PRIORITY_0                              = 0x2073e /**< Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when EOP received to EOP transferred on OCN). Event asserted for as many clock cycles as this is true.*/
    , e_PM_EV_SRIO2_OUTBOUND_NON_IDLES_TRANSMITTED                              = 0x2ff1b /**< Non idles transmitted. This can be used to determine the RIO link utilization. This is actually 1/2 of the actual count */
    , e_PM_EV_SRIO2_OUTBOUND_PACKETS_SENT                                       = 0x20225 /**< Outbound packet sent to RapidIO interface */
    , e_PM_EV_SRIO2_OUTBOUND_PACKETS_MISALIGNED                                 = 0x2043a /**< Outbound packet was misaligned */
    , e_PM_EV_SRIO2_OUTBOUND_PACKETS_RETRIED                                    = 0x20536 /**< Outbound packet was retried */
    , e_PM_EV_SRIO2_OUTBOUND_PACKETS_REORDERED                                  = 0x2063b /**< Outbound packet was reordered */
    , e_PM_EV_SRIO2_OUTBOUND_PACKETS_SENT_PRIORITY_0                            = 0x20738 /**< Outbound packet sent to RIO of priority 0 */
    , e_PM_EV_SRIO2_OUTBOUND_BUFFER_FULL                                        = 0x20839 /**< Clock cycle occurred in which the outbound buffer is full to any priority. Event asserted for as many clock cycles as this is true. */
    , e_PM_EV_SRIO2_OUTBOUND_BUFFER_FULL_PRIORITY_0                             = 0x2092b /**< Clock cycle occurred in which the outbound buffer is full to priority 0. Event asserted for as many clock cycles as this is true. */

      /* RapidIO Message Unit Events */
    , e_PM_EV_RIO_MU_INBOUND_PACKETS_RECEIVED                                   = 0x2ff0c /**< Packet received of any priority */
    , e_PM_EV_RIO_MU_INBOUND_PACKETS_RECEIVED_PRIORITY_0                        = 0x2050d /**< Packet received of priority 0 */
    , e_PM_EV_RIO_MU_INBOUND_BUFFER_FULL                                        = 0x20615 /**< CLock cycle occurred in which the inbound buffer is full to any priority (measured from when SOP received to buffer release transferred on OCN). Event asserted for as many clock cycles as this is true. */
    , e_PM_EV_RIO_MU_INBOUND_BUFFER_FULL_PRIORITY_0                             = 0x20708 /**< Clock cycle occurred in which the inbound buffer is full to priority 0 (measured from when SOP received to buffer release transferred on OCN). Event asserted for as many clock cycles as this is true. */

      /* Debug Events */
    , e_PM_EV_DEBUG_EXTERNAL_EVENT                                              = 0x1033d /**< Number of cycles trig_in pin is asserted */
    , e_PM_EV_DEBUG_WATCHPOINT_MONITOR_HITS                                     = 0x1023d /**< Number of watchpoint monitor hits */
    , e_PM_EV_DEBUG_TRACE_BUFFER_HITS                                           = 0x1013a /**< Number of trace buffer hits */

      /* DUART Events */
    , e_PM_EV_DUART1_BAUD_RATE                                                  = 0x1013f /**< DUART-1 real baud rate */
    , e_PM_EV_DUART2_BAUD_RATE                                                  = 0x1053f /**< DUART-2 real baud rate */

} e_PmEvent;

/** @} */ /* end of pm_control */
/** @} */ /* end of pm_grp */


/* Definition format:                           */
/*   { option, alternateEventValue },           */
/*     ------  -------------------              */
#define ALTERNATE_EVENT_OPTION \
    {{ e_PM_EV_DMA_CHANNEL_0_READ_DWORD_OR_LESS, 0x0535 }, \
     { e_PM_EV_DMA_CHANNEL_1_READ_DWORD_OR_LESS, 0x063a }, \
     { e_PM_EV_DMA_CHANNEL_2_READ_DWORD_OR_LESS, 0x0736 }, \
     { e_PM_EV_DMA_CHANNEL_3_READ_DWORD_OR_LESS, 0x0834 }}


#define PM_NUM_OF_COUNTERS      11


#endif /* __PM_INTEGRATION_EXT_H */

