/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          apm_integration_ext.h

 @Description   Integration-specific definitions and API for APM.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __APM_INTEGRATION_EXT_H
#define __APM_INTEGRATION_EXT_H


/* Contains external information specific to APM */
#include "apm_ext.h"


/**************************************************************************//**
 @Group         apm_grp Advanced Power Management Programming Interface

 @Description   APM functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         apm_control Advanced Power Management Control Unit

 @Description   Advanced Power Management Runtime Control functions

 @{
*//***************************************************************************/


//#define APM_JOG_MODE
//#define APM_POWER_DOWN_COUNTER
//#define APM_VOLTAGE_RAMPUP_COUNTER
//#define APM_DEEP_SLEEP_MODE
//#define APM_QUIESCE_MODE


/**************************************************************************//**
 @Description   Clock Disable
*//***************************************************************************/
typedef enum e_ApmClockEnable
{
    e_APM_CLOCK_DISABLE_USB_1           = 0x800000,     /**< USB 1 controller enable clock in low power modes */
    e_APM_CLOCK_DISABLE_USB_2           = 0x400000,     /**< USB 2 controller enable clock in low power modes */
    e_APM_CLOCK_DISABLE_SAP             = 0x020000,     /**< Enable SAP to memory DRAM and Local Bus */
    e_APM_CLOCK_DISABLE_eTSEC_1         = 0x000080,     /**< eTSEC 1 enable clock in sleep/deep-sleep mode */
    e_APM_CLOCK_DISABLE_eTSEC_2         = 0x000040      /**< eTSEC 2 enable clock in sleep/deep-sleep mode */
}e_ApmClockEnable;



#define APM_JOG_CORE_SPEED_MODE_SLOW    0       /**<  Core frequency at 400 - 533.34 MHz */
#define APM_JOG_CORE_SPEED_MODE_HIGH    1       /**<  Core frequency at 533.35 - 1066 MHz */

#define APM_JOG_MAX_RATIO_AVAILABLE     8       /**<  Core ratio between e500 core and CCB clock */

#define APM_VRCNT_PRE_MIN               0x0C    /**< default value for Voltage RampUp Counter Prescaler */
#define APM_VRCNT_VAL_MIN               0x83    /**< default value for Voltage RampUp Counter Value */

#define APM_JCNT_PRE_MIN                0x09    /**< default value for Jog Counter Prescaler */
#define APM_JCNT_VAL_MIN                0xD1    /**< default value for Jog Counter Value */

#define APM_PDCNT_PRE_MIN               0x08    /**< default value for Power Down Counter Prescaler */
#define APM_PDCNT_VAL_MIN               0xD1    /**< default value for Power Down Counter Value */

/** @} */ /* end of apm_grp */
/** @} */ /* end of apm_init_grp */


/*****************************************************************************
 INTERNAL INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/



#endif /* __APM_INTEGRATION_EXT_H */
