/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          xx_integration_ext.h

 @Description   Inline optimizations for the XX interface (bare-board only)
*//***************************************************************************/

#ifndef __XX_INTEGRATION_EXT_H
#define __XX_INTEGRATION_EXT_H

#include "std_ext.h"


/* Copied from e500v2_asm_ext.h */
#define not_MSR_EECE    0xFFFD7FFF
#define MSR_CE_BN       14
#define MSR_EE_BN       16


static __inline__ uint32_t XX_DisableAllIntr(void)
{
    register uint32_t   temp;

    asm
    {
        /* Disable interrupts in MSR */
        mfmsr   temp
        addis   r4,  0, not_MSR_EECE@h  /* load not_MSR_EECE mask to r4 */
        ori     r4, r4, not_MSR_EECE@l
        and     r4, temp, r4
        mtmsr   r4
    }

    return temp;
}


static __inline__ void XX_RestoreAllIntr(uint32_t flags)
{
    register uint32_t   temp = flags;
 
    asm
    {
        /* Restore interrupts state in MSR */
        rlwinm	temp, temp, 0, MSR_CE_BN, MSR_EE_BN /* Create a mask from the EE and CE bits */
        rlwinm	temp, temp, 0, MSR_EE_BN, MSR_CE_BN
        mfmsr   r4                                  /* Read MSR */
        or      r4, temp, r4                        /* Set CE and EE based on mask */
        mtmsr   r4                                  /* Write to MSR. */
    }
}


#endif /* __XX_INTEGRATION_EXT_H */
