/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PAR_IO_INTEGRATION_EXT_H
#define __PAR_IO_INTEGRATION_EXT_H

#include "io_port_ext.h"

/**************************************************************************//**
 @Description   Parallel I/O Port ID
*//***************************************************************************/
typedef enum e_ParIoPortId
{
    e_PAR_IO_PORT_A = 0,        /**< Parallel IO port A */
    e_PAR_IO_PORT_B,            /**< Parallel IO port B */
    e_PAR_IO_PORT_C             /**< Parallel IO port C */
} e_ParIoPortId;

#define PAR_IO_NUM_OF_PORTS      3


#define PAR_IO_VALID_PIN_MASKS  \
    { /* Port A */ 0xFFFFFFFF,  \
      /* Port B */ 0xFFFFFFFF,  \
      /* Port C */ 0x80000000 }


#define PAR_IO_VALID_INTR_MASKS \
    { /* Port A */ 0x08802208,  \
      /* Port B */ 0x88822000,  \
      /* Port C */ 0x00000000 }

#define PAR_IO_INTR_ID_MAP      { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 }

#define PAR_IO_MEM_MAP_85XX


#define PAR_IO_PIN(_portId, _pinId, _assign, _dir, _shared) \
    IO_PIN(e_PAR_IO_PORT_##_portId, _pinId, e_IO_PIN_ASSIGN_##_assign, e_IO_PIN_DIR_##_dir, _shared)

/* This enum is automatically generated from the Reference Manual */

typedef enum e_ParIoFunction
{
    e_PAR_IO_FUNC_ENET1_RX_ER_SER1_CD_B                   = PAR_IO_PIN(A,  8, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER3_CD_B                               = PAR_IO_PIN(B,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_ER_SER5_CD_B                   = PAR_IO_PIN(B,  8, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER7_CD_B_OPT1                          = PAR_IO_PIN(A, 22, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER7_CD_B_OPT2                          = PAR_IO_PIN(B, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RX_DV_SER1_CTS_B                  = PAR_IO_PIN(A,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER3_CTS_B                              = PAR_IO_PIN(A, 28, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RX_DV_SER5_CTS_B                  = PAR_IO_PIN(B,  4, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER7_CTS_B_OPT1                         = PAR_IO_PIN(A, 18, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER7_CTS_B_OPT2                         = PAR_IO_PIN(B, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_REQ_OPT1                           = PAR_IO_PIN(A,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_REQ_OPT2                           = PAR_IO_PIN(B, 27, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_REQ                                = PAR_IO_PIN(A, 28, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_REQ                                = PAR_IO_PIN(B,  8, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_REQ_OPT1                           = PAR_IO_PIN(A, 22, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMD_REQ_OPT2                           = PAR_IO_PIN(B, 18, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TX_EN_SER1_RTS_B                  = PAR_IO_PIN(A,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER3_RTS_B                              = PAR_IO_PIN(A, 29, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TX_EN_SER5_RTS_B                  = PAR_IO_PIN(B,  5, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER7_RTS_B_OPT1                         = PAR_IO_PIN(A, 19, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER7_RTS_B_OPT2                         = PAR_IO_PIN(B, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXEN_B                           = PAR_IO_PIN(A, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXEN_B0                          = PAR_IO_PIN(A, 27, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPISEL_B                           = PAR_IO_PIN(B, 20, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE1_B                         = PAR_IO_PIN(A, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE2_B                         = PAR_IO_PIN(B, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE3_B                         = PAR_IO_PIN(B, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TGATE4_B                         = PAR_IO_PIN(B, 31, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT1_B                          = PAR_IO_PIN(A, 28, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT2_B                          = PAR_IO_PIN(B, 22, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT3_B                          = PAR_IO_PIN(B, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TOUT4_B                          = PAR_IO_PIN(B, 29, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXEN_B                           = PAR_IO_PIN(A, 26, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXEN_B0                          = PAR_IO_PIN(A, 26, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_BRGO1_OPT1                              = PAR_IO_PIN(A, 23, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO1_OPT2                              = PAR_IO_PIN(A, 27, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO10_OPT1                             = PAR_IO_PIN(A,  7, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO10_OPT2                             = PAR_IO_PIN(B, 26, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO11_OPT1                             = PAR_IO_PIN(A, 21, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO11_OPT2                             = PAR_IO_PIN(B, 17, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO2_OPT1                              = PAR_IO_PIN(A, 24, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO2_OPT2                              = PAR_IO_PIN(B,  1, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO5                                   = PAR_IO_PIN(B, 10, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO6                                   = PAR_IO_PIN(B, 11, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO7_OPT1                              = PAR_IO_PIN(B,  2, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO7_OPT2                              = PAR_IO_PIN(B, 31, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO8_OPT1                              = PAR_IO_PIN(B,  3, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO8_OPT2                              = PAR_IO_PIN(C,  0, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO9_OPT1                              = PAR_IO_PIN(A,  6, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO9_OPT2                              = PAR_IO_PIN(A, 20, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO9_OPT3                              = PAR_IO_PIN(B, 16, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_BRGO9_OPT4                              = PAR_IO_PIN(B, 25, OPTION_1, OUTPUT, TRUE),
    e_PAR_IO_FUNC_CLK1                                    = PAR_IO_PIN(A, 23, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK10                                   = PAR_IO_PIN(B,  1, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK11                                   = PAR_IO_PIN(A, 27, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK12                                   = PAR_IO_PIN(A, 23, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK13                                   = PAR_IO_PIN(B, 11, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK14                                   = PAR_IO_PIN(C,  0, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK15                                   = PAR_IO_PIN(B, 31, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK16                                   = PAR_IO_PIN(B, 10, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK19                                   = PAR_IO_PIN(B,  3, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK20                                   = PAR_IO_PIN(B,  2, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK21                                   = PAR_IO_PIN(B, 18, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK3                                    = PAR_IO_PIN(B, 12, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK4                                    = PAR_IO_PIN(B, 13, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK5                                    = PAR_IO_PIN(B,  1, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK6                                    = PAR_IO_PIN(A, 27, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK7                                    = PAR_IO_PIN(B, 10, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK8                                    = PAR_IO_PIN(B, 27, OPTION_2, INPUT , TRUE),
    e_PAR_IO_FUNC_CLK9                                    = PAR_IO_PIN(A, 24, OPTION_1, INPUT , TRUE),
    e_PAR_IO_FUNC_ENET1_COL                               = PAR_IO_PIN(A, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_CRS                               = PAR_IO_PIN(A, 17, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_MUX_MDC                              = PAR_IO_PIN(B, 19, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPICLK                             = PAR_IO_PIN(B, 21, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_QE_MUX_MDIO                             = PAR_IO_PIN(B, 20, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPIMOSI                            = PAR_IO_PIN(B, 19, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_1588_TRIG_OUT_OPT1                      = PAR_IO_PIN(A, 20, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_1588_TRIG_OUT_OPT2                      = PAR_IO_PIN(B, 26, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_1588_CLK_OUT_OPT1                       = PAR_IO_PIN(A, 18, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_1588_CLK_OUT_OPT2                       = PAR_IO_PIN(B, 23, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_1588_TRIG_IN_OPT1                       = PAR_IO_PIN(A, 21, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_1588_TRIG_IN_OPT2                       = PAR_IO_PIN(B, 25, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_1588_PULSE_OUT1_OPT1                    = PAR_IO_PIN(A, 19, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_1588_PULSE_OUT1_OPT2                    = PAR_IO_PIN(B, 24, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_PTP_CLK                                 = PAR_IO_PIN(A, 22, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ1_OPT1                      = PAR_IO_PIN(A,  1, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ1_OPT2                      = PAR_IO_PIN(B, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ2_OPT1                      = PAR_IO_PIN(A,  2, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ2_OPT2                      = PAR_IO_PIN(B, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ3_OPT1                      = PAR_IO_PIN(A,  0, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ3_OPT2                      = PAR_IO_PIN(A, 28, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ4_OPT1                      = PAR_IO_PIN(A,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_RISC_EXT_REQ4_OPT2                      = PAR_IO_PIN(B,  9, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RSYNC_OPT1                         = PAR_IO_PIN(A,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RSYNC_OPT2                         = PAR_IO_PIN(B, 26, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RSYNC                              = PAR_IO_PIN(A, 31, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RSYNC                              = PAR_IO_PIN(B,  7, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RSYNC_OPT1                         = PAR_IO_PIN(A, 21, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_RSYNC_OPT2                         = PAR_IO_PIN(B, 17, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR0                          = PAR_IO_PIN(A, 31, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR0                          = PAR_IO_PIN(A, 31, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR1                          = PAR_IO_PIN(B,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR1                          = PAR_IO_PIN(B,  0, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR2_OPT1                     = PAR_IO_PIN(A, 28, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR2_OPT2                     = PAR_IO_PIN(B, 27, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR2_OPT1                     = PAR_IO_PIN(A, 28, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR2_OPT2                     = PAR_IO_PIN(B, 27, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR3_OPT1                     = PAR_IO_PIN(A,  3, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR3_OPT2                     = PAR_IO_PIN(B, 24, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR3_OPT1                     = PAR_IO_PIN(A,  3, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR3_OPT2                     = PAR_IO_PIN(B, 24, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR4_OPT1                     = PAR_IO_PIN(A,  2, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXADDR4_OPT2                     = PAR_IO_PIN(B, 25, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR4_OPT1                     = PAR_IO_PIN(A,  2, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXADDR4_OPT2                     = PAR_IO_PIN(B, 25, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_RXCLAV                           = PAR_IO_PIN(A, 24, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_RXCLAV0                          = PAR_IO_PIN(A, 24, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD0_SER1_RXD0                    = PAR_IO_PIN(A,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER3_RXD0                               = PAR_IO_PIN(A, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD0_SER5_RXD0                    = PAR_IO_PIN(B,  6, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER7_RXD0_OPT1                          = PAR_IO_PIN(A, 20, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER7_RXD0_OPT2                          = PAR_IO_PIN(B, 16, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_RXD0_OPT1                          = PAR_IO_PIN(A,  5, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMA_RXD0_OPT2                          = PAR_IO_PIN(B, 24, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMB_RXD0                               = PAR_IO_PIN(A, 29, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_RXD0                               = PAR_IO_PIN(B,  5, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_RXD0_OPT1                          = PAR_IO_PIN(A, 19, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_RXD0_OPT2                          = PAR_IO_PIN(B, 15, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_ENET1_RXD1                              = PAR_IO_PIN(A, 10, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER3_RXD1                               = PAR_IO_PIN(A, 11, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET5_RXD1_SER5_RXD1                    = PAR_IO_PIN(B,  9, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RXD1                               = PAR_IO_PIN(A, 11, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RXD1                               = PAR_IO_PIN(B, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD2                              = PAR_IO_PIN(A, 14, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER3_RXD2                               = PAR_IO_PIN(A, 12, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER5_RXD2                               = PAR_IO_PIN(B, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RXD2                               = PAR_IO_PIN(A, 12, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RXD2                               = PAR_IO_PIN(B, 18, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_RXD3                              = PAR_IO_PIN(A, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER3_RXD3                               = PAR_IO_PIN(A, 13, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_SER5_RXD3                               = PAR_IO_PIN(B, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_RXD3                               = PAR_IO_PIN(A, 13, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_RXD3                               = PAR_IO_PIN(B, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA8                            = PAR_IO_PIN(A, 12, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA8                            = PAR_IO_PIN(A,  4, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA9                            = PAR_IO_PIN(A, 13, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA9                            = PAR_IO_PIN(A,  5, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA10                           = PAR_IO_PIN(A, 14, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA10                           = PAR_IO_PIN(A,  6, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA11                           = PAR_IO_PIN(A, 15, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA11                           = PAR_IO_PIN(A,  7, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA12                           = PAR_IO_PIN(A, 16, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA12                           = PAR_IO_PIN(A,  8, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA13                           = PAR_IO_PIN(A, 17, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA13                           = PAR_IO_PIN(A,  9, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA14                           = PAR_IO_PIN(A, 18, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA14                           = PAR_IO_PIN(A, 10, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXDATA15                           = PAR_IO_PIN(A, 19, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXDATA15                           = PAR_IO_PIN(A, 11, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXPRTY                             = PAR_IO_PIN(A, 21, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXPRTY                             = PAR_IO_PIN(A, 20, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_RXSOC                              = PAR_IO_PIN(A, 22, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_TXSOC                              = PAR_IO_PIN(A, 23, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SPI1_SPIMISO                            = PAR_IO_PIN(B, 22, OPTION_3, IN_OUT, FALSE),
    e_PAR_IO_FUNC_STROBE1_OPT1                            = PAR_IO_PIN(A, 17, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_STROBE1_OPT2                            = PAR_IO_PIN(B, 29, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_STROBE2                                 = PAR_IO_PIN(A, 26, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_STROBE3                                 = PAR_IO_PIN(B, 30, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_STROBE4_OPT1                            = PAR_IO_PIN(A,  3, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_STROBE4_OPT2                            = PAR_IO_PIN(A, 25, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN1                             = PAR_IO_PIN(A, 29, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN2                             = PAR_IO_PIN(B, 19, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN3                             = PAR_IO_PIN(B, 15, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_QE_GTM_TIN4                             = PAR_IO_PIN(B, 30, OPTION_2, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMB_TSYNC                              = PAR_IO_PIN(A, 30, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMC_TSYNC                              = PAR_IO_PIN(B,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_TSYNC_OPT1                         = PAR_IO_PIN(A,  6, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMA_TSYNC_OPT2                         = PAR_IO_PIN(B, 25, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_TSYNC_OPT1                         = PAR_IO_PIN(A, 20, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_TDMD_TSYNC_OPT2                         = PAR_IO_PIN(B, 16, OPTION_1, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_TX_ER                             = PAR_IO_PIN(A, 13, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR0                          = PAR_IO_PIN(B,  1, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR0                          = PAR_IO_PIN(B,  1, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR1                          = PAR_IO_PIN(A, 29, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR1                          = PAR_IO_PIN(A, 29, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR2_OPT1                     = PAR_IO_PIN(A, 30, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR2_OPT2                     = PAR_IO_PIN(B, 26, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR2_OPT1                     = PAR_IO_PIN(A, 30, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR2_OPT2                     = PAR_IO_PIN(B, 26, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR3_OPT1                     = PAR_IO_PIN(A,  1, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR3_OPT2                     = PAR_IO_PIN(B, 23, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR3_OPT1                     = PAR_IO_PIN(A,  1, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR3_OPT2                     = PAR_IO_PIN(B, 23, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR4_OPT1                     = PAR_IO_PIN(A,  0, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXADDR4_OPT2                     = PAR_IO_PIN(B, 28, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR4_OPT1                     = PAR_IO_PIN(A,  0, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXADDR4_OPT2                     = PAR_IO_PIN(B, 28, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_UPC1_S_TXCLAV                           = PAR_IO_PIN(A, 25, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_UPC1_M_TXCLAV0                          = PAR_IO_PIN(A, 25, OPTION_3, INPUT , FALSE),
    e_PAR_IO_FUNC_ENET1_TXD0_SER1_TXD0                    = PAR_IO_PIN(A,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER3_TXD0                               = PAR_IO_PIN(A, 31, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD0_SER5_TXD0                    = PAR_IO_PIN(B,  7, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER7_TXD0_OPT1                          = PAR_IO_PIN(A, 21, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER7_TXD0_OPT2                          = PAR_IO_PIN(B, 17, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD0_OPT1                          = PAR_IO_PIN(A,  4, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMA_TXD0_OPT2                          = PAR_IO_PIN(B, 23, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD0                               = PAR_IO_PIN(B,  0, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD0                               = PAR_IO_PIN(B,  4, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD0_OPT1                          = PAR_IO_PIN(A, 18, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_TDMD_TXD0_OPT2                          = PAR_IO_PIN(B, 14, OPTION_1, IN_OUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD1                              = PAR_IO_PIN(A,  9, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER3_TXD1                               = PAR_IO_PIN(A, 14, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER5_TXD1                               = PAR_IO_PIN(B, 17, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD1                               = PAR_IO_PIN(A, 14, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD1                               = PAR_IO_PIN(B,  9, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET5_TXD1                              = PAR_IO_PIN(B, 10, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD2                              = PAR_IO_PIN(A, 11, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER3_TXD2                               = PAR_IO_PIN(A, 15, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER5_TXD2                               = PAR_IO_PIN(B, 18, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD2                               = PAR_IO_PIN(A, 15, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD2                               = PAR_IO_PIN(B, 15, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_ENET1_TXD3                              = PAR_IO_PIN(A, 12, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER3_TXD3                               = PAR_IO_PIN(A, 16, OPTION_2, OUTPUT, FALSE),
    e_PAR_IO_FUNC_SER5_TXD3                               = PAR_IO_PIN(B, 14, OPTION_3, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMB_TXD3                               = PAR_IO_PIN(A, 16, OPTION_1, OUTPUT, FALSE),
    e_PAR_IO_FUNC_TDMC_TXD3                               = PAR_IO_PIN(B, 16, OPTION_3, OUTPUT, FALSE)

}e_ParIoFunction;

#endif /* __PAR_IO_INTEGRATION_EXT_H */
