/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          pm_integration_ext.h

 @Description   P1021 Performance Monitor external structures and definitions.
*//***************************************************************************/

#ifndef __PM_INTEGRATION_EXT_H
#define __PM_INTEGRATION_EXT_H


/**************************************************************************//**
 @Group         pm_grp Performance Monitor Application Programming Interface

 @Description   Performance Monitor API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pm_control Performance Monitor Control Unit

 @Description   Performance Monitor Runtime Control functions

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Performance Monitor Events.
*//***************************************************************************/
/* Note - the values are ordered in the following way:
   2 msb bits  - the clock divisor for the specific counter
   8 next bits - the counter number - 0xff means don't care about counter number
   8 lsb bits  - the event value (in decimal)
*/
typedef enum e_PmEvent
{
      /* General Events */
      e_PM_EV_SYSTEM_CYCLES                                                         = 0x1003F   /**< CCB (platform) clock cycles */
 
      /* DDR Memory Controller Events */
    , e_PM_EV_DDR1_CYCLES_A_READ_IS_RETURNING_DATA_FROM_DRAM                        = 0x1FF13   /**< Each data beat returned to the memory controller on the DRAM interface */
    , e_PM_EV_DDR1_CYCLES_A_WRITE_TRANSFER_DATA_FROM_OR_TO_DRAM                     = 0x1FF1B   /**< Each data beat transferred to or from the DRAM */
    , e_PM_EV_DDR1_PIPELINED_READ_MISSES_IN_ROW_OPEN_TABLE                          = 0x10179   /**< Row open table read misses issued while a read is outstanding */
    , e_PM_EV_DDR1_PIPELINED_READ_OR_WRITE_MISSES_IN_ROW_OPEN_TABLE                 = 0x10240   /**< Row open table read or write misses issued while a read or write is outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_MISSES_IN_ROW_OPEN_TABLE                      = 0x1037C   /**< Row open table read misses issued when no reads are outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_OR_WRITE_MISSES_IN_ROW_OPEN_TABLE             = 0x10440   /**< Row open table read or write misses issued when no reads or writes are outstanding */
    , e_PM_EV_DDR1_PIPELINED_READ_HITS_IN_ROW_OPEN_TABLE                            = 0x10578   /**< Row open table read hits issued when a read is outstanding */
    , e_PM_EV_DDR1_PIPELINED_READ_OR_WRITE_HITS_IN_ROW_OPEN_TABLE                   = 0x10640   /**< Row open table read or write hits issued when a read or write is outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_HITS_IN_ROW_OPEN_TABLE                        = 0x10779   /**< Row open table read hits issued when no reads are outstanding */
    , e_PM_EV_DDR1_NON_PIPELINED_READ_OR_WRITE_HITS_IN_ROW_OPEN_TABLE               = 0x10840   /**< Row open table read or write hits issued when no reads or writes are outstanding */
    , e_PM_EV_DDR1_FORCED_PAGE_CLOSINGS_NOT_CAUSED_BY_A_REFRESH                     = 0x10140   /**< Precharges issued to the DRAM for any reason except refresh. */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_MISSES                                            = 0x10241   /**< Transactions that miss in the row open table */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS                                              = 0x10340   /**< Transaction that hit in the row open table */
    , e_PM_EV_DDR1_FORCE_PAGE_CLOSINGS                                              = 0x10441   /**< Forced page closings including those due to refreshes */
    , e_PM_EV_DDR1_READ_MODIFY_WRITE_TRANSACTIONS_DUE_TO_ECC                        = 0x10540   /**< If ECC is enabled and a transaction requires byte enables, a read-modify-write sequence is issued on the DRAM interface. */
    , e_PM_EV_DDR1_FORCED_PAGE_CLOSINGS_DUE_TO_COLLISION_WITH_BANK_AND_SUB_BANK     = 0x10940   /**< Increments if a new transaction must be issued to an active bank and sub-bank that has a different row open */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_CORES                                       = 0x1FF0D   /**< Reads or writes from core 0-1 */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_ETSEC                                       = 0x10341   /**< Reads or writes from eTSEC 1-3 */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_HSI                                         = 0x10443   /**< Reads or writes from high speed interfaces (PCI Express 1-2) */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_DMA                                         = 0x10542   /**< Reads or writes from DMA */
    , e_PM_EV_DDR1_READS_OR_WRITES_FROM_SEC                                         = 0x10645   /**< Reads or writes from Security */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_CORES               = 0x1FFE    /**< Row open table hits for reads or writes from core 0-1 */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_ETSEC               = 0x10641   /**< Row open table hits for reads or writes from eTSEC 1-4 */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_HSI                 = 0x10741   /**< Row open table hits for reads or writes from high speed interfaces (PCI Express 1-2) */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_DMA                 = 0x10842   /**< Row open table hits for reads or writes from DMA */
    , e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SEC                 = 0x10744   /**< Row open table hits for reads or writes from Security */

      /* DMA Controller Events */
    , e_PM_EV_DMA_CHANNEL_0_READ_REQUESTS                                           = 0x20142   /**< DMA channel 0 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_1_READ_REQUESTS                                           = 0x20245   /**< DMA channel 1 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_2_READ_REQUESTS                                           = 0x20344   /**< DMA channel 2 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_3_READ_REQUESTS                                           = 0x20446   /**< DMA channel 3 read request active in the system */
    , e_PM_EV_DMA_CHANNEL_0_WRITE_REQUESTS                                          = 0x20143   /**< DMA channel 0 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_1_WRITE_REQUESTS                                          = 0x20246   /**< DMA channel 1 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_2_WRITE_REQUESTS                                          = 0x20345   /**< DMA channel 2 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_3_WRITE_REQUESTS                                          = 0x20447   /**< DMA channel 3 write request active in the system */
    , e_PM_EV_DMA_CHANNEL_0_DESCRIPTOR_REQUESTS                                     = 0x20569   /**< DMA channel 0 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_1_DESCRIPTOR_REQUESTS                                     = 0x2066C   /**< DMA channel 1 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_2_DESCRIPTOR_REQUESTS                                     = 0x20769   /**< DMA channel 2 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_3_DESCRIPTOR_REQUESTS                                     = 0x20869   /**< DMA channel 3 descriptor request active in the system */
    , e_PM_EV_DMA_CHANNEL_0_READ_DWORD_OR_LESS                                      = 0x20144   /**< DMA channel 0 read double word valid */
    , e_PM_EV_DMA_CHANNEL_1_READ_DWORD_OR_LESS                                      = 0x20247   /**< DMA channel 1 read double word valid */
    , e_PM_EV_DMA_CHANNEL_2_READ_DWORD_OR_LESS                                      = 0x20346   /**< DMA channel 2 read double word valid */
    , e_PM_EV_DMA_CHANNEL_3_READ_DWORD_OR_LESS                                      = 0x20448   /**< DMA channel 3 read double word valid */
    , e_PM_EV_DMA_CHANNEL_0_WRITE_DWORD_OR_LESS                                     = 0x20145   /**< DMA channel 0 write double word valid */
    , e_PM_EV_DMA_CHANNEL_1_WRITE_DWORD_OR_LESS                                     = 0x20248   /**< DMA channel 1 write double word valid */
    , e_PM_EV_DMA_CHANNEL_2_WRITE_DWORD_OR_LESS                                     = 0x20347   /**< DMA channel 2 write double word valid */
    , e_PM_EV_DMA_CHANNEL_3_WRITE_DWORD_OR_LESS                                     = 0x20449   /**< DMA channel 3 write double word valid */
 
      /* e500 Coherency Module (ECM) Events */
    , e_PM_EV_ECM_REQUEST_WAIT_CORE_0                                               = 0x1084D   /**< */
    , e_PM_EV_ECM_REQUEST_WAIT_CORE_1                                               = 0x10B62   /**< */
    , e_PM_EV_ECM_REQUEST_WAIT_I2C_SECURITY_TEST_PORT_USB1                          = 0x1074D   /**< */
    , e_PM_EV_ECM_REQUEST_WAIT_PEX1_2_DMA                                           = 0x10550   /**< */
    , e_PM_EV_ECM_REQUEST_WAIT_ETSEC1_3                                             = 0x10650   /**< */
    , e_PM_EV_ECM_REQUEST_WAIT_SAP_USB0_ESDHC_TDM_QE                                = 0x10454   /**< */
    , e_PM_EV_ECM_DISPATCH                                                          = 0x1FF0F   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_CORE_0                                              = 0x10150   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_CORE_1                                              = 0x10A5F   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_USB1                                                = 0x10A57   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_ETSEC1                                              = 0x10455   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_ETSEC2                                              = 0x10850   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_ETSEC3                                              = 0x10950   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_TDM                                                 = 0x10A56   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_PEX1                                                = 0x10851   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_PEX2                                                = 0x10A62   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_DMA                                                 = 0x10952   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_SECURITY                                            = 0x10A64   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_USB0_HOST                                           = 0x10A42   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_USB0_DUAL_ROLE                                      = 0x10944   /**< */
    , e_PM_EV_ECM_DISPATCH_FROM_OTHER                                               = 0x10652   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_DDR                                                   = 0x1074F   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_L2                                                    = 0x10A66   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_SRAM                                                  = 0x1084F   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_LBC                                                   = 0x10953   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_PEX1                                                  = 0x10254   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_PEX2                                                  = 0x10151   /**< */
    , e_PM_EV_ECM_DISPATCH_TO_CCSR                                                  = 0x10957   /**< */
    , e_PM_EV_ECM_DISPATCH_WRITE                                                    = 0x10751   /**< */
    , e_PM_EV_ECM_DISPATCH_WRITE_ALLOCATE                                           = 0x10152   /**< */
    , e_PM_EV_ECM_DISPATCH_READ                                                     = 0x10256   /**< */
    , e_PM_EV_ECM_DISPATCH_READ_ATOMIC                                              = 0x10356   /**< */
    , e_PM_EV_ECM_DATA_BUS_GRANT_DDR                                                = 0x10153   /**< */
    , e_PM_EV_ECM_DATA_BUS_GRANT_I2C_SECURITY_TEST_PORT_USB1                        = 0x10357   /**< */
    , e_PM_EV_ECM_DATA_BUS_GRANT_PEX1_2_DMA                                         = 0x10459   /**< */
    , e_PM_EV_ECM_DATA_BUS_GRANT_LBC                                                = 0x1FF10   /**< */
    , e_PM_EV_ECM_DATA_BUS_GRANT_ETSEC1_3                                           = 0x1FF11   /**< */
    , e_PM_EV_ECM_DATA_BUS_GRANT_SAP_USB0_ESDHC_TDM_QE                              = 0x1FF12   /**< */
    , e_PM_EV_ECM_GLOBAL_DATA_BUS_BEAT                                              = 0x10556   /**< */
    , e_PM_EV_ECM_E500_DIRECT_READ_BUS_BEAT                                         = 0x10B5F   /**< */
    , e_PM_EV_ECM_E500_DIRECT_READ_BUS_BEAT_FORWARDED                               = 0x10754   /**< */
    , e_PM_EV_ECM_CANCEL                                                            = 0x10853   /**< */

    , e_PM_EV_BOOT_SEQUENCER_SECURITY_USB1_REQUESTS                                 = 0x20177   /**< */
    , e_PM_EV_BOOT_SEQUENCER_SECURITY_USB1_READ_REQUESTS                            = 0x2024C   /**< */
    , e_PM_EV_BOOT_SEQUENCER_SECURITY_USB1_DATA_BEATS                               = 0x2034B   /**< */
    , e_PM_EV_BOOT_SEQUENCER_SECURITY_USB1_READ_DATA_BEATS                          = 0x2044F   /**< */

    , e_PM_EV_SAP_USB0_ESDHC_TDM_QE_REQUESTS                                        = 0x20163   /**< */
    , e_PM_EV_SAP_USB0_ESDHC_TDM_QE_READ_REQUESTS                                   = 0x20A4E   /**< */
    , e_PM_EV_SAP_USB0_ESDHC_TDM_QE_DATA_BEATS                                      = 0x20B4C   /**< */
    , e_PM_EV_SAP_USB0_ESDHC_TDM_QE_READ_DATA_BEATS                                 = 0x20560   /**< */
 
      /* Interrupt Controller (PIC) Events */
    , e_PM_EV_PIC_TOTAL_INTERRUPT_COUNT                                             = 0x1FF1A   /**< */
    , e_PM_EV_PIC_INTERRUPT_WAIT_CYCLES                                             = 0x1087E   /**< */
    , e_PM_EV_PIC_INTERRUPT_SERVICE_CYCLES                                          = 0x10253   /**< */
    , e_PM_EV_PIC_INTERRUPT_SELECT_0_DURATION_THRESHOLD                             = 0x10178   /**< */
    , e_PM_EV_PIC_INTERRUPT_SELECT_1_DURATION_THRESHOLD                             = 0x1037B   /**< */
    , e_PM_EV_PIC_INTERRUPT_SELECT_2_DURATION_THRESHOLD                             = 0x10577   /**< */
    , e_PM_EV_PIC_INTERRUPT_SELECT_3_DURATION_THRESHOLD                             = 0x1067C   /**< */

      /* eTSEC 1 Events */
    , e_PM_EV_ETSEC1_DMA_WRITE_DATA_BEATS                                           = 0x2036D   /**< */
    , e_PM_EV_ETSEC1_DMA_READ_DATA_BEATS                                            = 0x2046E   /**< */
    , e_PM_EV_ETSEC1_DMA_WRITE_REQUEST                                              = 0x2056A   /**< */
    , e_PM_EV_ETSEC1_DMA_READ_REQUEST                                               = 0x2066D   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_DROPPED_FRAMES                                       = 0x20958   /**< */
    , e_PM_EV_ETSEC1_TXBD_READ_LIFETIME_DURATION_THRESHOLD                          = 0x2FF22   /**< */
    , e_PM_EV_ETSEC1_RXBD_READ_LIFETIME_DURATION_THRESHOLD                          = 0x2FF26   /**< */
    , e_PM_EV_ETSEC1_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD                         = 0x2FF2A   /**< */
    , e_PM_EV_ETSEC1_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD                         = 0x2FF2E   /**< */
    , e_PM_EV_ETSEC1_READ_DATA_LIFETIME_DURATION_THRESHOLD                          = 0x2FF32   /**< */
    , e_PM_EV_ETSEC1_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM                             = 0x2095C   /**< */
    , e_PM_EV_ETSEC1_TX_IP_PACKET_WITH_CHECKSUM                                     = 0x20169   /**< */
    , e_PM_EV_ETSEC1_TX_TCP_UDP_PACKET_WITH_CHECKSUM                                = 0x20271   /**< */
    , e_PM_EV_ETSEC1_TCP_UDP_PACKETS_CHECKED_FOR_CS                                 = 0x20372   /**< */
    , e_PM_EV_ETSEC1_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR                                = 0x20473   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_REJECTED_FRAMES_BY_FILER                             = 0x2056F   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR                   = 0x20672   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_1_4_FULL                    = 0x2056E   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_1_2_FULL                    = 0x20671   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_3_4_FULL                    = 0x2076E   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_CYCLES_RX_FIFO_EQUALS_FULL                           = 0x2086E   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_ACCEPTED_FRAMES_MATCH                                = 0x2096E   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_ACCEPTED_FRAMS_TO_STATION_ADDRESS                    = 0x20A50   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_ACCEPTED_UNICASET_FRAMES_VIA_HASH                    = 0x20B4E   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_ACCEPTED_GROUP_FRAMES_VIA_HASH                       = 0x20660   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_ACCEPTED_FRAMS_VIA_EXACT_MATCH                       = 0x20764   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_REJECTED_FRAMES_AT_LAYER_2                           = 0x2096F   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_RX_INTERRUPTS_SIGNALLED                              = 0x2085E   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_TX_INTERRUPTS_SIGNALLED                              = 0x20970   /**< */
    , e_PM_EV_ETSEC1_RX_DATA_WRITE_LIFETIME_DURATION_THRESHOLD                      = 0x20A51   /**< */
    , e_PM_EV_ETSEC1_NUMBER_OF_RX_PACKETS_RECEIVED_WHILE_RX_FIFO_IS_FULL            = 0x20B4F   /**< */

      /* eTSEC 2 Events */
    , e_PM_EV_ETSEC2_DMA_WRITE_DATA_BEATS                                           = 0x2056B   /**< */
    , e_PM_EV_ETSEC2_DMA_READ_DATA_BEATS                                            = 0x2066E   /**< */
    , e_PM_EV_ETSEC2_DMA_WRITE_REQUEST                                              = 0x2076B   /**< */
    , e_PM_EV_ETSEC2_DMA_READ_REQUEST                                               = 0x2086B   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_DROPPED_FRAMES                                       = 0x2026E   /**< */
    , e_PM_EV_ETSEC2_TXBD_READ_LIFETIME_DURATION_THRESHOLD                          = 0x2FF23   /**< */
    , e_PM_EV_ETSEC2_RXBD_READ_LIFETIME_DURATION_THRESHOLD                          = 0x2FF27   /**< */
    , e_PM_EV_ETSEC2_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD                         = 0x2FF2B   /**< */
    , e_PM_EV_ETSEC2_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD                         = 0x2FF2F   /**< */
    , e_PM_EV_ETSEC2_READ_DATA_LIFETIME_DURATION_THRESHOLD                          = 0x2FF33   /**< */
    , e_PM_EV_ETSEC2_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM                             = 0x20373   /**< */
    , e_PM_EV_ETSEC2_TX_IP_PACKET_WITH_CHECKSUM                                     = 0x20474   /**< */
    , e_PM_EV_ETSEC2_TX_TCP_UDP_PACKET_WITH_CHECKSUM                                = 0x20565   /**< */
    , e_PM_EV_ETSEC2_TCP_UDP_PACKETS_CHECKED_FOR_CS                                 = 0x20673   /**< */
    , e_PM_EV_ETSEC2_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR                                = 0x2076F   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_REJECTED_FRAMES_BY_FILER                             = 0x2086F   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR                   = 0x2095D   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_1_4_FULL                    = 0x20771   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_1_2_FULL                    = 0x20871   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_3_4_FULL                    = 0x20960   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_CYCLES_RX_FIFO_EQUALS_FULL                           = 0x2016C   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_ACCEPTED_FRAMES_MATCH                                = 0x20971   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_ACCEPTED_FRAMS_TO_STATION_ADDRESS                    = 0x20A52   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_ACCEPTED_UNICASET_FRAMES_VIA_HASH                    = 0x20B50   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_ACCEPTED_GROUP_FRAMES_VIA_HASH                       = 0x20661   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_ACCEPTED_FRAMS_VIA_EXACT_MATCH                       = 0x20765   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_REJECTED_FRAMES_AT_LAYER_2                           = 0x20972   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_RX_INTERRUPTS_SIGNALLED                              = 0x20A53   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_TX_INTERRUPTS_SIGNALLED                              = 0x20B51   /**< */
    , e_PM_EV_ETSEC2_RX_DATA_WRITE_LIFETIME_DURATION_THRESHOLD                      = 0x20860   /**< */
    , e_PM_EV_ETSEC2_NUMBER_OF_RX_PACKETS_RECEIVED_WHILE_RX_FIFO_IS_FULL            = 0x20973   /**< */
 
      /* eTSEC 3 Events */
    , e_PM_EV_ETSEC3_DMA_WRITE_DATA_BEATS                                           = 0x2076C   /**< */
    , e_PM_EV_ETSEC3_DMA_READ_DATA_BEATS                                            = 0x2086C   /**< */
    , e_PM_EV_ETSEC3_DMA_WRITE_REQUEST                                              = 0x2095A   /**< */
    , e_PM_EV_ETSEC3_DMA_READ_REQUEST                                               = 0x20167   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_DROPPED_FRAMES                                       = 0x20470   /**< */
    , e_PM_EV_ETSEC3_TXBD_READ_LIFETIME_DURATION_THRESHOLD                          = 0x2FF24   /**< */
    , e_PM_EV_ETSEC3_RXBD_READ_LIFETIME_DURATION_THRESHOLD                          = 0x2FF28   /**< */
    , e_PM_EV_ETSEC3_TXBD_WRITE_LIFETIME_DURATION_THRESHOLD                         = 0x2FF2C   /**< */
    , e_PM_EV_ETSEC3_RXBD_WRITE_LIFETIME_DURATION_THRESHOLD                         = 0x2FF30   /**< */
    , e_PM_EV_ETSEC3_READ_DATA_LIFETIME_DURATION_THRESHOLD                          = 0x2FF34   /**< */
    , e_PM_EV_ETSEC3_RX_IP_PACKETS_CHECKED_FOR_CHECKSUM                             = 0x20674   /**< */
    , e_PM_EV_ETSEC3_TX_IP_PACKET_WITH_CHECKSUM                                     = 0x20770   /**< */
    , e_PM_EV_ETSEC3_TX_TCP_UDP_PACKET_WITH_CHECKSUM                                = 0x20870   /**< */
    , e_PM_EV_ETSEC3_TCP_UDP_PACKETS_CHECKED_FOR_CS                                 = 0x2095E   /**< */
    , e_PM_EV_ETSEC3_IP_OR_TCP_UDP_RX_CHECKSUM_ERROR                                = 0x2016A   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_REJECTED_FRAMES_BY_FILER                             = 0x20272   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_REJECTED_FRAMES_DUE_TO_FILER_ERROR                   = 0x20374   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_1_4_FULL                    = 0x20961   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_1_2_FULL                    = 0x2016D   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_MORE_THEN_3_4_FULL                    = 0x20274   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_CYCLES_RX_FIFO_EQUALS_FULL                           = 0x20376   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_ACCEPTED_FRAMES_MATCH                                = 0x20974   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_ACCEPTED_FRAMS_TO_STATION_ADDRESS                    = 0x20A54   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_ACCEPTED_UNICASET_FRAMES_VIA_HASH                    = 0x20B52   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_ACCEPTED_GROUP_FRAMES_VIA_HASH                       = 0x20662   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_ACCEPTED_FRAMS_VIA_EXACT_MATCH                       = 0x20766   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_REJECTED_FRAMES_AT_LAYER_2                           = 0x20975   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_RX_INTERRUPTS_SIGNALLED                              = 0x20861   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_TX_INTERRUPTS_SIGNALLED                              = 0x20976   /**< */
    , e_PM_EV_ETSEC3_RX_DATA_WRITE_LIFETIME_DURATION_THRESHOLD                      = 0x20A55   /**< */
    , e_PM_EV_ETSEC3_NUMBER_OF_RX_PACKETS_RECEIVED_WHILE_RX_FIFO_IS_FULL            = 0x20B53   /**< */

      /* PCI Express 1 Events */
    , e_PM_EV_PCIE1_INBOUND_READ                                                    = 0x20877   /**< */
    , e_PM_EV_PCIE1_INBOUND_WRITE                                                   = 0x20965   /**< */
    , e_PM_EV_PCIE1_INBOUND_DATA                                                    = 0x2057C   /**< */
    , e_PM_EV_PCIE1_OUTBOUND_READ                                                   = 0x2067E   /**< */
    , e_PM_EV_PCIE1_OUTBOUND_WRITE                                                  = 0x2077D   /**< */
    , e_PM_EV_PCIE1_OUTBOUND_DATA                                                   = 0x2087C   /**< */
    , e_PM_EV_PCIE1_INBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD                 = 0x2FF36   /**< */
    , e_PM_EV_PCIE1_OUTBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD                = 0x2FF37   /**< */

      /* PCI Express 2 Events */
    , e_PM_EV_PCIE2_INBOUND_READ                                                    = 0x20366   /**< */
    , e_PM_EV_PCIE2_INBOUND_WRITE                                                   = 0x20467   /**< */
    , e_PM_EV_PCIE2_INBOUND_DATA                                                    = 0x20561   /**< */
    , e_PM_EV_PCIE2_OUTBOUND_READ                                                   = 0x2015F   /**< */
    , e_PM_EV_PCIE2_OUTBOUND_WRITE                                                  = 0x20265   /**< */
    , e_PM_EV_PCIE2_OUTBOUND_DATA                                                   = 0x20B57   /**< */
    , e_PM_EV_PCIE2_INBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD                 = 0x2FF20   /**< */
    , e_PM_EV_PCIE2_OUTBOUND_STATIC_QUEUE_0_START_DURATION_THRESHOLD                = 0x2FF21   /**< */

      /* Local Bus Events */
    , e_PM_EV_LBC_NUMBER_OF_CORRECTABLE_FCM_ECC_ERRORS                              = 0x10379   /**< */
    , e_PM_EV_LBC_NUMBER_OF_UNCORRECTABLE_FCM_ECC_ERROR                             = 0x10677   /**< */
    , e_PM_EV_LBC_ATOMIC_RESERVATION_TIMEOUTS_FOR_ECM_PORT                          = 0x10676   /**< */
    , e_PM_EV_LBC_CYCLES_FOR_A_READ_TO_FCM_BUFFER_RAM                               = 0x10773   /**< */
    , e_PM_EV_LBC_CYCLES_FOR_A_WRITE_TO_FCM_BUFFER_RAM                              = 0x10873   /**< */
    , e_PM_EV_LBC_CYCLES_A_READ_IS_TAKING_IN_GPCM                                   = 0x10175   /**< */
    , e_PM_EV_LBC_CYCLES_A_READ_IS_TAKING_IN_UPM                                    = 0x1027A   /**< */
    , e_PM_EV_LBC_CYCLES_A_WRITE_IS_TAKING_IN_GPCM                                  = 0x10478   /**< */
    , e_PM_EV_LBC_CYCLES_A_WRITE_IS_TAKING_IN_UPM                                   = 0x10572   /**< */

      /* L2 Cache/SRAM Events */
    , e_PM_EV_L2_SRAM_L2_CLEARING_OF_LOCKS                                          = 0x1FF16   /**< */
    , e_PM_EV_L2_SRAM_CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_HIT                      = 0x1027B   /**< */
    , e_PM_EV_L2_SRAM_CORE_INSTRUCTION_ACCESSES_TO_L2_THAT_MISS                     = 0x1FF17   /**< */
    , e_PM_EV_L2_SRAM_CORE_DATA_ACCESSES_TO_L2_THAT_HIT                             = 0x10479   /**< */
    , e_PM_EV_L2_SRAM_CORE_DATA_ACCESSES_TO_L2_THAT_MISS                            = 0x10573   /**< */
    , e_PM_EV_L2_SRAM_NONCORE_BURST_WRITE_TO_L2_CACHE_EXTERNAL_WRITE_OR_SRAM        = 0x10678   /**< */
    , e_PM_EV_L2_SRAM_NONCORE_NONBURST_WRITE_TO_L2                                  = 0x10774   /**< */
    , e_PM_EV_L2_SRAM_NONCORE_WRITE_MISSES_CACHE_EXT_WRITE_WIN_AND_SRAM_MEM_RANGE   = 0x1FF18   /**< */
    , e_PM_EV_L2_SRAM_NONCORE_READ_HIT_IN_L2                                        = 0x10176   /**< */
    , e_PM_EV_L2_SRAM_NONCORE_READ_MISS_IN_L2                                       = 0x1FF19   /**< */
    , e_PM_EV_L2_SRAM_L2_ALLOCATES_FROM_ANY_SOURCE                                  = 0x1027C   /**< */
    , e_PM_EV_L2_SRAM_L2_RETRIES_DUE_TO_FULL_WRITE_QUEUE                            = 0x1037A   /**< */
    , e_PM_EV_L2_SRAM_L2_RETRIES_DUE_TO_ADDRESS_COLLISION                           = 0x1047A   /**< */
    , e_PM_EV_L2_SRAM_L2_FAILED_LOCK_ATTEMPTS_DUE_TO_FULL_SET                       = 0x10574   /**< */
    , e_PM_EV_L2_SRAM_L2_VICTIMIZATIONS_OF_VALID_LINES                              = 0x10679   /**< */
    , e_PM_EV_L2_SRAM_L2_INVALIDATIONS_OF_LINES                                     = 0x10775   /**< */

      /* Debug Events */
    , e_PM_EV_DEBUG_EXTERNAL_EVENT                                                  = 0x1037D   /**< */
    , e_PM_EV_DEBUG_WATCHPOINT_MONITOR_HITS                                         = 0x1027D   /**< */
    , e_PM_EV_DEBUG_TRACE_BUFFER_HITS                                               = 0x1017A   /**< */
 
      /* DUART Events */
    , e_PM_EV_DUART1_BAUD_RATE                                                      = 0x1017F   /**< */
    , e_PM_EV_DUART2_BAUD_RATE                                                      = 0x1057F   /**< */

      /* Chaining Events */
    , e_PM_EV_CH_PMC0_CARRYOUT                                                      = 0x1FF01   /**< */
    , e_PM_EV_CH_PMC1_CARRYOUT                                                      = 0x1FF02   /**< */
    , e_PM_EV_CH_PMC2_CARRYOUT                                                      = 0x1FF03   /**< */
    , e_PM_EV_CH_PMC3_CARRYOUT                                                      = 0x1FF04   /**< */
    , e_PM_EV_CH_PMC4_CARRYOUT                                                      = 0x1FF05   /**< */
    , e_PM_EV_CH_PMC5_CARRYOUT                                                      = 0x1FF06   /**< */
    , e_PM_EV_CH_PMC6_CARRYOUT                                                      = 0x1FF07   /**< */
    , e_PM_EV_CH_PMC7_CARRYOUT                                                      = 0x1FF08   /**< */
    , e_PM_EV_CH_PMC8_CARRYOUT                                                      = 0x1FF09   /**< */
    , e_PM_EV_CH_PMC9_CARRYOUT                                                      = 0x1FF0A   /**< */
    , e_PM_EV_CH_PMC10_CARRYOUT                                                     = 0x1FF0B   /**< */
    , e_PM_EV_CH_PMC11_CARRYOUT                                                     = 0x1FF0C   /**< */
 
} e_PmEvent;

/** @} */ /* end of pm_control */
/** @} */ /* end of pm_grp */


/* Definition format:                           */
/*   { option, alternateEventValue },           */
/*     ------  -------------------              */
#define ALTERNATE_EVENT_OPTION                                                      \
    {{ e_PM_EV_SYSTEM_CYCLES,                                           0x1FF3F },  \
     { e_PM_EV_DDR1_ROW_OPEN_TABLE_MISSES,                              0x10B42 },  \
     { e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS,                                0x10A44 },  \
     { e_PM_EV_DDR1_ROW_OPEN_TABLE_HITS_FOR_READS_OR_WRITES_FROM_SEC,   0x10A41 },  \
     { e_PM_EV_DMA_CHANNEL_0_READ_DWORD_OR_LESS,                        0x20575 },  \
     { e_PM_EV_DMA_CHANNEL_1_READ_DWORD_OR_LESS,                        0x2067A },  \
     { e_PM_EV_DMA_CHANNEL_2_READ_DWORD_OR_LESS,                        0x20776 },  \
     { e_PM_EV_DMA_CHANNEL_3_READ_DWORD_OR_LESS,                        0x20874 }}


#define PM_NUM_OF_COUNTERS      10


#endif /* __PM_INTEGRATION_EXT_H */

