/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          part_integration_ext.h

 @Description   P1022 external definitions and structures.
*//***************************************************************************/
#ifndef __PART_INTEGRATION_EXT_H
#define __PART_INTEGRATION_EXT_H


#include "ddr_std_ext.h"
#include "epic_ext.h"

/**************************************************************************//**
 @Group         1022_grp P1022 Application Programming Interface

 @Description   P1022 Chip functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Module types.
*//***************************************************************************/
typedef enum e_ModuleId
{
    e_MODULE_ID_LAW,            /**< Local Access module                     */
    e_MODULE_ID_ECM,            /**< e500 Coherency Module                   */
    e_MODULE_ID_DDR,            /**< DDR memory controller                   */
    e_MODULE_ID_I2C_1,          /**< I2C 1                                   */
    e_MODULE_ID_I2C_2,          /**< I2C 2                                   */
    e_MODULE_ID_DUART_1,        /**< DUART module 1                          */
    e_MODULE_ID_DUART_2,        /**< DUART module 2                          */
    e_MODULE_ID_LBC,            /**< Local bus memory controller module      */
    e_MODULE_ID_PCIE_1,         /**< PCI Express 1 controller module         */
    e_MODULE_ID_PCIE_1_ATMU,    /**< PCI 1 ATMU WINDOW                       */
    e_MODULE_ID_PCIE_2,         /**< PCI Express 2 controller module         */
    e_MODULE_ID_PCIE_2_ATMU,    /**< PCI 2 ATMU WINDOW                       */
    e_MODULE_ID_PCIE_3,         /**< PCI Express 3 controller module         */
    e_MODULE_ID_PCIE_3_ATMU,    /**< PCI 3 ATMU WINDOW                       */
    e_MODULE_ID_MSI,            /**< MSI registers                           */
    e_MODULE_ID_L2_SRAM,        /**< L2/SRAM Memory-Mapped controller module */
    e_MODULE_ID_DMA_1,          /**< DMA controller 1                        */
    e_MODULE_ID_DMA_2,          /**< DMA controller 2                        */
    e_MODULE_ID_EPIC,           /**< Programmable interrupt controller       */
    e_MODULE_ID_ESPI,           /**< Enhanced SPI module                     */
    e_MODULE_ID_SSI,            /**< SSI module                              */
    e_MODULE_ID_GPIO,           /**< General Purpose I/O                     */
    e_MODULE_ID_DIU,            /**< DIU module                              */
    e_MODULE_ID_SEC,            /**< Security (sec 3.3) module               */
    e_MODULE_ID_USB_DR_1,       /**< USB 2.0 module 1                        */
    e_MODULE_ID_USB_DR_2,       /**< USB 2.0 module 2                        */
    e_MODULE_ID_ETSEC_1,        /**< ETSEC module 1                          */
    e_MODULE_ID_ETSEC_2,        /**< ETSEC module 2                          */
    e_MODULE_ID_ETSEC_1_PTP,    /**< ETSEC1 PTP module                       */
    e_MODULE_ID_ETSEC_2_PTP,    /**< ETSEC2 PTP module                       */
    e_MODULE_ID_ETSEC_MII_MNG,/**< ETSEC1 module 1 MII managment base      */
    e_MODULE_ID_ETSEC_RTC,      /**< ETSEC RTC module                        */
    e_MODULE_ID_ESDHC,          /**< eSDHC module                            */
    e_MODULE_ID_TDM,            /**< TDM module                              */
    e_MODULE_ID_TDM_DMA,        /**< TDM DMA module                          */
    e_MODULE_ID_SATA_1,         /**< SATA 1 controller module                */
    e_MODULE_ID_SATA_2,         /**< SATA 2 controller module                */
    e_MODULE_ID_PM,             /**< Performance Monitor module              */
    e_MODULE_ID_APM,			/**< Advanced Power Management module        */
    e_MODULE_ID_DUMMY_LAST
} e_ModuleId;

#define NUM_OF_MODULES  e_MODULE_ID_DUMMY_LAST

/**************************************************************************//**
 @Description   Transaction source ID (for memory conrollers error reporting).
*//***************************************************************************/
typedef enum e_TransSrc
{
    e_TRANS_SRC_PCIE_2          = 0x1,  /**< PCI Express 2                  */
    e_TRANS_SRC_PCIE_1          = 0x2,  /**< PCI Express 2                  */
    e_TRANS_SRC_PCIE_3          = 0x3,  /**< PCI Express 1                  */
    e_TRANS_SRC_USB_1           = 0x5,  /**< USB 1                          */
    e_TRANS_SRC_SEC             = 0x7,  /**< Security engine                */
    e_TRANS_SRC_SATA_2          = 0x8,  /**< SATA 2                         */
    e_TRANS_SRC_DIU             = 0x9,  /**< DIU                            */
	e_TRANS_SRC_BOOT_SEQ        = 0xA,  /**< Boot sequencer                 */
    e_TRANS_SRC_ESDHC           = 0xB,  /**< eSDHC                          */
    e_TRANS_SRC_SATA_1          = 0xD,  /**< SATA 1                         */
    e_TRANS_SRC_TDM             = 0x0F, /**< TDM               				*/
    e_TRANS_SRC_CORE_0_INST     = 0x10, /**< Processor 0 (instruction)      */
    e_TRANS_SRC_CORE_0_DATA     = 0x11, /**< Processor 0 (data)             */
    e_TRANS_SRC_CORE_1_INST     = 0x12, /**< Processor 1 (instruction)      */
    e_TRANS_SRC_CORE_1_DATA     = 0x13, /**< Processor 1 (data)             */
    e_TRANS_SRC_USB_2           = 0x14, /**< USB 2                          */
    e_TRANS_SRC_DMA_1           = 0x15, /**< DMA Controller 1               */
    e_TRANS_SRC_DMA_2           = 0x16, /**< DMA Controller 2               */
    e_TRANS_SRC_ETSEC_1         = 0x18, /**< eTsec 1                        */
    e_TRANS_SRC_ETSEC_2         = 0x19  /**< eTsec 2                        */
 } e_TransSrc;

/**************************************************************************//**
 @Description   Local Access Window Target interface ID
*//***************************************************************************/
typedef enum e_P1022LawTargetId
{
    e_P1022_LAW_TARGET_PCIE_2  = 0x01, /**< PCI Express 2 target interface */
    e_P1022_LAW_TARGET_PCIE_1  = 0x02, /**< PCI Express 1 target interface */
    e_P1022_LAW_TARGET_PCIE_3  = 0x03, /**< PCI Express 3 target interface */
    e_P1022_LAW_TARGET_LBC     = 0x04, /**< Local bus target interface */
    e_P1022_LAW_TARGET_DDR     = 0x0F, /**< DDR target interface */
    e_P1022_LAW_TARGET_NONE    = 0xFF  /**< Invalid target interface */
} e_P1022LawTargetId;

/***************************************************************
    P1022 general routines
****************************************************************/
/**************************************************************************//**
 @Group         1022_init_grp P1022 Initialization Unit

 @Description   P1022 initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Part ID and revision number
*//***************************************************************************/

typedef enum e_P1022DeviceName
{
    e_P1022_REV_INVALID          = 0x00000000, /**< Invalid revision                            */
    e_P1022E_DUAL_CORE_REV_1_0   = 0x80EE0010, /**< P1022E rev 1.0 with security, dual core     */
    e_P1022E_SINGLE_CORE_REV_1_0 = 0x80EF0010, /**< P1022E rev 1.0 with security, single core   */
    e_P1022_DUAL_CORE_REV_1_0    = 0x80E60010, /**< P1022 rev 1.0 without security, dual core   */
    e_P1022_SINGLE_CORE_REV_1_0  = 0x80E70010  /**< P1022 rev 1.0 without security, single core */

} e_P1022DeviceName;

/**************************************************************************//**
 @Description   Device Disable Register
*//***************************************************************************/
typedef enum e_P1022DeviceDisable
{
    e_P1022_DEV_DISABLE_PCIE1       = 0x20000000, /**< PCI Express 1 controller disable     */
    e_P1022_DEV_DISABLE_TDM         = 0x10000000, /**< TDM disable                          */
    e_P1022_DEV_DISABLE_LBC         = 0x08000000, /**< Local bus controller disable         */
    e_P1022_DEV_DISABLE_PCIE2       = 0x04000000, /**< PCI Express 2 controller disable     */
    e_P1022_DEV_DISABLE_PCIE3       = 0x02000000, /**< PCI Express 3 controller disable     */
    e_P1022_DEV_DISABLE_SEC         = 0x01000000, /**< Security disable                     */
    e_P1021_DEV_DISABLE_USB0        = 0x00800000, /**< USB 0 controller disable             */
    e_P1021_DEV_DISABLE_USB1        = 0x00400000, /**< USB 1 controller disable             */
    e_P1022_DEV_DISABLE_ESDHC       = 0x00200000, /**< SD controller disable                */
    e_P1022_DEV_DISABLE_SATA0       = 0x00080000, /**< SATA 0 disable                       */
    e_P1022_DEV_DISABLE_SATA1       = 0x00040000, /**< SATA 1 disable                       */
    e_P1022_DEV_DISABLE_DDR         = 0x00010000, /**< DDR controller disable               */
    e_P1022_DEV_DISABLE_E500_CORE_0 = 0x00008000, /**< E500 core 0 disable                  */
    e_P1022_DEV_DISABLE_TB_CORE_0   = 0x00004000, /**< Time base of the e500 core 0 disable */
    e_P1022_DEV_DISABLE_E500_CORE_1 = 0x00002000, /**< E500 core 1 disable                  */
    e_P1022_DEV_DISABLE_TB_CORE_1   = 0x00001000, /**< Time base of the e500 core 1 disable */
    e_P1022_DEV_DISABLE_DIU         = 0x00000800, /**< DIU disable                          */
    e_P1022_DEV_DISABLE_DMA1        = 0x00000400, /**< DMA 1 controller disable             */
    e_P1022_DEV_DISABLE_DMA2        = 0x00000200, /**< DMA 2 controller disable             */
    e_P1022_DEV_DISABLE_TSEC1       = 0x00000080, /**< TSEC1 controller disable             */
    e_P1022_DEV_DISABLE_TSEC2       = 0x00000040, /**< TSEC2 controller disable             */
    e_P1022_DEV_DISABLE_SSI         = 0x00000020, /**< SSI controller disable               */
    e_P1022_DEV_DISABLE_SPI         = 0x00000008, /**< SPI controller disable               */
    e_P1022_DEV_DISABLE_I2C         = 0x00000004, /**< I2C controllers disable              */
    e_P1022_DEV_DISABLE_DUART       = 0x00000002  /**< Dual controller disable              */
 
  } e_P1022DeviceDisable;

/**************************************************************************//**
 @Description   P1022 Signals Mux Selections
*//***************************************************************************/
typedef enum e_P1022MuxOption
{
    e_P1022_MUX_I2C1_UART1_4W,                  /**< I2C1 and UART1 4-wire signals */
    e_P1022_MUX_I2C1_UART1_2W,                  /**< I2C1 and UART1 2-wire signals */
    e_P1022_MUX_DMA_SSI_I2C1,                   /**< DMA Channel 0, SSI RCK and I2C1 signals */
    e_P1022_MUX_SSI_I2C1,                     	/**< SSI RCK and I2C1 signals */
    e_P1022_MUX_SSI_UART1_2W,                   /**< SSI and UART1 2-wire signals */
    e_P1022_MUX_TDM_UART1_2W,                   /**< TDM and UART1 2-wire signals */
    e_P1022_MUX_SSI,                   			/**< SSI signals */
    e_P1022_MUX_TDM,                   			/**< TDM signals */
    e_P1022_MUX_DMA_B1,
    e_P1022_MUX_I2C2,                           /**< I2C2 signals */
    e_P1022_MUX_UART2,                          /**< UART2 signals */
    e_P1022_MUX_UART2_I2C2,                     /**< UART2 and I2C2 signals */
    e_P1022_MUX_SDHC_1_BIT_MODE,                /**< SDHC 1-bit mode signals                    */
    e_P1022_MUX_SDHC_4_BIT_MODE,                /**< SDHC 4-bit mode signals                    */
    e_P1022_MUX_SDHC_8_BIT_MODE,                /**< SDHC 8-bit mode signals                    */
    e_P1022_MUX_ETSEC1,                         /**< eTSEC-1 signals */
    e_P1022_MUX_ETSEC2,                         /**< eTSEC-2 signals */
    e_P1022_MUX_SPI,                            /**< eSPI signals */
    e_P1022_MUX_DIU,                            /**< DIU selection */
    e_P1022_MUX_SSI_TX,                         /**< SSI TX signals */
    e_P1022_MUX_SSI1_DMA1_CH0,                  /**< SSI1 Tx connected to DMA1 channel 0 */
    e_P1022_MUX_SSI1_DMA1_CH1,                  /**< SSI1 Rx connected to DMA1 channel 1 */
    e_P1022_MUX_SSI2_DMA1_CH2,                  /**< SSI2 Tx connected to DMA1 channel 2 */
    e_P1022_MUX_SSI2_DMA1_CH3,                  /**< SSI2 Rx connected to DMA1 channel 3 */
    e_P1022_MUX_SSI1_DMA2_CH0,                  /**< SSI1 Tx connected to DMA2 channel 0 */
    e_P1022_MUX_SSI1_DMA2_CH1,                  /**< SSI1 Rx connected to DMA2 channel 1 */
    e_P1022_MUX_SSI2_DMA2_CH2,                  /**< SSI2 Tx connected to DMA2 channel 2 */
    e_P1022_MUX_SSI2_DMA2_CH3,                  /**< SSI2 Rx connected to DMA2 channel 3 */
    e_P1022_MUX_USB1_GPIO						/**< USB1 signals */
 

} e_P1022MuxOption;


/**************************************************************************//**
 @Description   DDR Clock Mode.
*//***************************************************************************/
typedef enum e_DdrClockMode
{
    e_DDR_NO_CLOCK,
    e_DDR_ASYNC_CLOCK,
    e_DDR_SYNC_CLOCK
} e_DdrClockMode;

/**************************************************************************//**
 @Description   SerDeser Control Registers.
*//***************************************************************************/
typedef enum e_SerDesCtrlReg
{
    e_SERDES_CTRL_REG_0,
    e_SERDES_CTRL_REG_1,
    e_SERDES_CTRL_REG_2
} e_SerDesCtrlReg;

/**************************************************************************//**
 @Function      P1022_ConfigAndInit

 @Description   General initiation of the chip registers.

 @Param         baseAddress  - (In) memory map start

 @Return        A handle to the P1022 data structure.
*//***************************************************************************/
t_Handle P1022_ConfigAndInit(uint32_t baseAddress);

/**************************************************************************//**
 @Function      P1022_Free

 @Description   Free all resources.

 @Param      	h_P1022 - (In) The handle of the initialized P1022 object.

 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error P1022_Free(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetModuleBase
 
 @Description   returns the base address of a P1022 module's
				memory mapped registers.
 
 @Param      	h_P1022   - (In) The handle of the initialized P1022 object.
 @Param        	module      - (In) The module ID.
 
 @Return    	Base address of module's memory mapped registers.
                ILLEGAL_BASE in case of non-existent module
*//***************************************************************************/
uint64_t P1022_GetModuleBase(t_Handle h_P1022, e_ModuleId module);

/**************************************************************************//**
 @Function      P1022_GetRevInfo
 
 @Description   This routine enables access to chip and revision information.
 
 @Param      	h_P1022 - (In) The handle of the initialized P1022 object.
 
 @Return    	Part ID and revision.
*//***************************************************************************/
e_P1022DeviceName P1022_GetRevInfo(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetE500Factor
 
 @Description   Returns E500 core clock multiplication factor.
 
 @Param[in]     h_P1022         - A handle to the P1022 object.
 @Param[in]     coreId          - Id of the requested core.
 @Param[out]    p_E500MulFactor - Returns E500 to CCB multification factor.
 @Param[out]    p_E500DivFactor - Returns E500 to CCB division factor.
 
 @Return        E_OK on success; Other value otherwise.
*
*//***************************************************************************/
t_Error P1022_GetE500Factor(t_Handle    h_P1022,
                            uint32_t    coreId,
                            uint32_t    *p_E500MulFactor,
                            uint32_t    *p_E500DivFactor);

/**************************************************************************//**
 @Function      P1022_GetCcbFactor
 
 @Description   returns system multiplication factor.
 
 @Param         h_P1022 - (In) a handle to the P1022 object.
 
 @Return    	System multiplication factor.
*//***************************************************************************/
uint32_t P1022_GetCcbFactor(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetDdrFactor
 
 @Description   returns the multiplication factor of the clock in for the DDR clock .
 
 @Param         h_P1022 - (In) a handle to the P1022 object.
 
 @Return    	System multiplication factor.
*//***************************************************************************/
uint32_t P1022_GetDdrFactor(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetDdrAsyncFactor
 
 @Description   Returns the multiplication factor of the clock in for the DDR clock.

 @Param[in]     h_P1022 - A handle to the P1022 object.
 
 @Return        DDR contoler division factor.
*//***************************************************************************/
uint32_t P1022_GetDdrAsyncFactor(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetDdrClockMode
 
 @Description   Get the DDR clock mode : synchronous or asynchronous

 @Param[in]     h_P1022 - A handle to the P1022 object.
 
 @Param[out]    p_DdrClockMode - Returns DDR clock mode
 
 @Return        E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error P1022_GetDdrClkMode(t_Handle h_P1022, e_DdrClockMode *p_DdrClockMode);

/**************************************************************************//**
 @Function      P1022_GetDdrType
 
 @Description   returns the multiplication factor of the clock in for the DDR clock .
 
 @Param         h_P1022 - (In) a handle to the P1022 object.
 @Param       	p_DdrType   - (Out) returns DDR type DDR1/DDR2/DDR3.
 
 @Return    	E_OK on success; Other value otherwise.
*//***************************************************************************/
t_Error P1022_GetDdrType(t_Handle h_P1022,e_DdrType *p_DdrType );
 
/**************************************************************************//**
 @Function      P1022_DeviceDisableStatus
 
 @Description   Returns whether a device is desabled in DEVDISR register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 @Param[in]     device  - Requested device
 
 @Return        TRUE  - if the specified device is disabled
                FALSE - if the specified device is enabled
*//***************************************************************************/
bool P1022_DeviceDisableStatus(t_Handle h_P1022, e_P1022DeviceDisable device);

/**************************************************************************//**
 @Function      P1022_DeviceDisable
 
 @Description   Disable device in DEVDISR register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 @Param[in]     device  - Requested device
 
 @Return        None
*//***************************************************************************/
t_Error P1022_DeviceDisable(t_Handle h_P1022, e_P1022DeviceDisable device);

/**************************************************************************//**
 @Function      P1022_DeviceEnable
 
 @Description   Enable device in DEVDISR register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 @Param[in]     device  - Requested device
 
 @Return        None
*//***************************************************************************/
t_Error P1022_DeviceEnable(t_Handle h_P1022, e_P1022DeviceDisable device);

/**************************************************************************//**
 @Function      P1022_Reset
 
 @Description   Reset the chip.
 
 @Param      	h_P1022 - (In) The handle of the initialized P1022 object.
*//***************************************************************************/
void    P1022_Reset(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_SetMuxOption
 
 @Description   Enables a selected signals MUX option.

 @Param[in]     h_P1022 - The handle of the initialized P1022 object
 @Param[in]     option  - Mux option to enable
 
 @Return        None
*//***************************************************************************/
t_Error P1022_SetMuxOption(t_Handle h_P1022, e_P1022MuxOption option);

/**************************************************************************//**
 @Function      P1022_FreeMuxOption
 
 @Description   Releases a selected signals MUX option.

 @Param[in]     h_P1022 - The handle of the initialized P1022 object
 @Param[in]     option  - Mux option to release
 
 @Return        None
*//***************************************************************************/
t_Error P1022_FreeMuxOption(t_Handle h_P1022, e_P1022MuxOption option);

/**************************************************************************//**
 @Function      P1022_GetDeviceDisableStatusRegister
 
 @Description   Return the value of Disable Status Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        Disable Status Register value
*//***************************************************************************/
uint32_t P1022_GetDeviceDisableStatusRegister(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_SetDeviceDisableStatusRegisterBits
 
 @Description   Set the value of Disable Status Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Param[in]     bits - Bits mask to set
 
 @Return        E_OK
 
*//***************************************************************************/
t_Error P1022_SetDeviceDisableStatusRegisterBits(t_Handle h_P1022, uint32_t bits);

/**************************************************************************//**
 @Function      P1022_GetPorDeviceStatusRegister
 
 @Description   Return the value of POR Device Status Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        POR Device Status Register
*//***************************************************************************/
uint32_t P1022_GetPorDeviceStatusRegister(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetSerDesControlRegister
 
 @Description   Return the value of SerDes Control Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Param[in]     reg - SerDes control register to read
 
 @Return        SerDes Control Register value
*//***************************************************************************/
uint32_t P1022_GetSerDesControlRegister(t_Handle h_P1022, e_SerDesCtrlReg reg);

/**************************************************************************//**
 @Function      P1022_GetBootPageTranslationRegisterPtr
 
 @Description   Return the value of Boot Page Translation Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        Boot Page Translation Register value
*//***************************************************************************/
uint32_t P1022_GetBootPageTranslationRegisterPtr(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetPorDeviceStatusRegister2
 
 @Description   Return the value of POR Device Status Register 2

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        POR Device Status Register 2
*//***************************************************************************/
uint32_t P1022_GetPorDeviceStatusRegister2(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetPorBootModeStatusRegister
 
 @Description   Return the value of POR Boot Mode Status Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        POR Boot Mode Status Register value
*//***************************************************************************/
uint32_t P1022_GetPorBootModeStatusRegister(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetClockDivideStatusRegisterPtr
 
 @Description   Return the address of the Clock Divide Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        Register address
*//***************************************************************************/
uint32_t P1022_GetClockDivideStatusRegisterPtr(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetClockDivideStatusRegister
 
 @Description   Return the value of Clock Divide Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Return        POR Boot Mode Status Register value
*//***************************************************************************/
uint32_t P1022_GetClockDivideStatusRegister(t_Handle h_P1022);

/**************************************************************************//**
 @Function      P1022_GetClockDivideStatusRegisterField
 
 @Description   Return the value of the intended field from the Clock Divide Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Param[in]     mask - The mask of the field.
 
 @Return        Field value
*//***************************************************************************/
uint32_t P1022_GetClockDivideStatusRegisterField(t_Handle h_P1022, uint32_t mask);

/**************************************************************************//**
 @Function      P1022_SetClockDivideStatusRegisterField
 
 @Description   Set the value of the intended field in the Clock Divide Register

 @Param[in]     h_P1022 - The handle of the initialized P1022 object.
 
 @Param[in]     mask - The mask of the field.

 @Param[in]     value - The value of the field.
 
 @Return        E_OK
*//***************************************************************************/
t_Error P1022_SetClockDivideStatusRegisterField(t_Handle h_P1022,
                                                 uint32_t mask, uint32_t value);

/** @} */ /* end of 1022_init_grp group */
/** @} */ /* end of 1022_grp group */

#define CORE_E500V2

#define INTG_MAX_NUM_OF_CORES   2


/*****************************************************************************
 INTEGRATION-SPECIFIC MODULE CODES
******************************************************************************/
#define MODULE_UNKNOWN          0x00000000
#define MODULE_P1022            0x00010000
#define MODULE_DUART            0x00020000
#define MODULE_ESPI             0x00030000
#define MODULE_TDM              0x00040000
#define MODULE_SEC              0x00050000
#define MODULE_PCI              0x00060000
#define MODULE_I2C              0x00070000
#define MODULE_PIO              0x00080000
#define MODULE_DMA              0x00090000
#define MODULE_LAW              0x000A0000
#define MODULE_DDR              0x000B0000
#define MODULE_LBC              0x000C0000
#define MODULE_EPIC             0x000D0000
#define MODULE_MEM              0x000E0000
#define MODULE_MM               0x000F0000
#define MODULE_MMU              0x00100000
#define MODULE_L2_CACHE         0x00110000
#define MODULE_USB              0x00120000
#define MODULE_ECM              0x00130000
#define MODULE_MII              0x00140000
#define MODULE_CORE             0x00150000
#define MODULE_PM               0x00160000
#define MODULE_PTP              0x00170000
#define MODULE_ESDHC            0x00180000
#define MODULE_ETSEC            0x00190000
#define MODULE_ETSEC_RTC        0x001A0000
#define MODULE_SATA             0x001B0000
#define MODULE_DIU              0x001C0000
#define MODULE_SSI              0x001D0000
#define MODULE_APM              0x001E0000

/*****************************************************************************
 GPIO INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define GPIO_NUM_OF_PORTS   3   /**< Number of ports in GPIO module;
                                     Each port contains up to 32 i/O pins. */
#define GPIO_VALID_PIN_MASKS    \
    { /* Port 1 */ 0xFFFFFFFF,  \
      /* Port 2 */ 0xFFFFFFFF,  \
      /* Port 3 */ 0xFFFFE000   \
    }

#define GPIO_VALID_INTR_MASKS   \
    { /* Port 1 */ 0x00000000,  \
      /* Port 2 */ 0x0000007F,  \
      /* Port 3 */ 0x00480000   \
    }

/*****************************************************************************
 LBC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/**************************************************************************//**
 @Group         lbc_exception_grp LBC Exception Unit

 @Description   LBC Exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        lbc_exbm
 
 @Collection    LBC Errors Bit Mask
 
                These errors are reported through the exceptions callback..
                The values can be or'ed in any combination in the errors mask
                parameter of the errors report structure.
 
                These errors can also be passed as a bit-mask to
                LBC_EnableErrorChecking() or LBC_DisableErrorChecking(),
                for enabling or disabling error checking.
 @{
*//***************************************************************************/
#define LBC_ERR_BUS_MONITOR     0x80000000  /**< Bus monitor error */
#define LBC_ERR_PARITY_ECC      0x20000000  /**< Parity error for GPCM/UPM */
#define LBC_ERR_WRITE_PROTECT   0x04000000  /**< Write protection error */
#define LBC_ERR_ATOMIC_WRITE    0x00800000  /**< Atomic write error */
#define LBC_ERR_ATOMIC_READ     0x00400000  /**< Atomic read error */
#define LBC_ERR_CHIP_SELECT     0x00080000  /**< Unrecognized chip select */

#define LBC_ERR_ALL             (LBC_ERR_BUS_MONITOR | LBC_ERR_PARITY_ECC | \
                                 LBC_ERR_WRITE_PROTECT | LBC_ERR_ATOMIC_WRITE | \
                                 LBC_ERR_ATOMIC_READ | LBC_ERR_CHIP_SELECT)
                                            /**< All possible errors */
/* @} */
/** @} */ /* end of lbc_exception_grp group */

#define LBC_NUM_OF_BANKS            8
#define LBC_MAX_CS_SIZE             0x0000000100000000LL
#define LBC_ATOMIC_OPERATION_SUPPORT
#define LBC_PARITY_SUPPORT
#define LBC_ADDRESS_SHIFT_SUPPORT
#define LBC_ADDRESS_HOLD_TIME_CTRL
#define LBC_HIGH_CLK_DIVIDERS
#define LBC_FCM_AVAILABLE


/*****************************************************************************
 LAW INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define LAW_ARCH_CCB
#define LAW_NUM_OF_WINDOWS      12
#define LAW_MIN_WINDOW_SIZE     0x0000000000001000LL    /**< 4KB */
#define LAW_MAX_WINDOW_SIZE     0x0000001000000000LL    /**< 32GB */


/*****************************************************************************
 EPIC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define EPIC_NUM_OF_EXT_INTRS           12
#define EPIC_NUM_OF_INT_INTRS           64
#define EPIC_NUM_OF_TIMERS              8
#define EPIC_NUM_OF_MSG_INTRS           8
#define EPIC_NUM_OF_SMSG_INTRS          8

/**************************************************************************//**
 @Description   Enumeration of EPIC interrupts
*//***************************************************************************/
typedef enum e_EpicIntrSrc
{
    e_EPIC_INTR_TIMER_A0        = (EPIC_INTR_BASE_TIMER + 0),
    e_EPIC_INTR_TIMER_A1        = (EPIC_INTR_BASE_TIMER + 1),
    e_EPIC_INTR_TIMER_A2        = (EPIC_INTR_BASE_TIMER + 2),
    e_EPIC_INTR_TIMER_A3        = (EPIC_INTR_BASE_TIMER + 3),
    e_EPIC_INTR_TIMER_B0        = (EPIC_INTR_BASE_TIMER + 4),
    e_EPIC_INTR_TIMER_B1        = (EPIC_INTR_BASE_TIMER + 5),
    e_EPIC_INTR_TIMER_B2        = (EPIC_INTR_BASE_TIMER + 6),
    e_EPIC_INTR_TIMER_B3        = (EPIC_INTR_BASE_TIMER + 7),

    e_EPIC_INTR_IRQ_0           = (EPIC_INTR_BASE_EXTERNAL + 0),
    e_EPIC_INTR_IRQ_1           = (EPIC_INTR_BASE_EXTERNAL + 1),
    e_EPIC_INTR_IRQ_2           = (EPIC_INTR_BASE_EXTERNAL + 2),
    e_EPIC_INTR_IRQ_3           = (EPIC_INTR_BASE_EXTERNAL + 3),
    e_EPIC_INTR_IRQ_4           = (EPIC_INTR_BASE_EXTERNAL + 4),
    e_EPIC_INTR_IRQ_5           = (EPIC_INTR_BASE_EXTERNAL + 5),
    e_EPIC_INTR_IRQ_6           = (EPIC_INTR_BASE_EXTERNAL + 6),
    e_EPIC_INTR_IRQ_7           = (EPIC_INTR_BASE_EXTERNAL + 7),
    e_EPIC_INTR_IRQ_8           = (EPIC_INTR_BASE_EXTERNAL + 8),
    e_EPIC_INTR_IRQ_9           = (EPIC_INTR_BASE_EXTERNAL + 9),
    e_EPIC_INTR_IRQ_10          = (EPIC_INTR_BASE_EXTERNAL + 10),
    e_EPIC_INTR_IRQ_11          = (EPIC_INTR_BASE_EXTERNAL + 11),

    e_EPIC_INTR_ERR             = (EPIC_INTR_BASE_INTERNAL +  0),
    e_EPIC_INTR_ETSEC1_GRP2_TX  = (EPIC_INTR_BASE_INTERNAL +  1),
    e_EPIC_INTR_ETSEC1_GRP2_RX  = (EPIC_INTR_BASE_INTERNAL +  2),
    e_EPIC_INTR_LBC             = (EPIC_INTR_BASE_INTERNAL +  3),
    e_EPIC_INTR_DMA1_1          = (EPIC_INTR_BASE_INTERNAL +  4),
    e_EPIC_INTR_DMA1_2          = (EPIC_INTR_BASE_INTERNAL +  5),
    e_EPIC_INTR_DMA1_3          = (EPIC_INTR_BASE_INTERNAL +  6),
    e_EPIC_INTR_DMA1_4          = (EPIC_INTR_BASE_INTERNAL +  7),
    e_EPIC_INTR_ETSEC1_GRP2_ERR = (EPIC_INTR_BASE_INTERNAL +  8),
 
    e_EPIC_INTR_PCIE_1 			= (EPIC_INTR_BASE_INTERNAL +  9),
    e_EPIC_INTR_PCIE_2 			= (EPIC_INTR_BASE_INTERNAL + 10),
    e_EPIC_INTR_PCIE_3 			= (EPIC_INTR_BASE_INTERNAL + 11),
 
    e_EPIC_INTR_USB             = (EPIC_INTR_BASE_INTERNAL + 12),
    e_EPIC_INTR_ETSEC1_GRP1_TX  = (EPIC_INTR_BASE_INTERNAL + 13),
    e_EPIC_INTR_ETSEC1_GRP1_RX  = (EPIC_INTR_BASE_INTERNAL + 14),
    e_EPIC_INTR_ETSEC1_GRP1_ERR = (EPIC_INTR_BASE_INTERNAL + 18),
    e_EPIC_INTR_ETSEC2_GRP1_TX  = (EPIC_INTR_BASE_INTERNAL + 19),
    e_EPIC_INTR_ETSEC2_GRP1_RX  = (EPIC_INTR_BASE_INTERNAL + 20),
    e_EPIC_INTR_ETSEC2_GRP1_ERR = (EPIC_INTR_BASE_INTERNAL + 24),
    e_EPIC_INTR_SATA2           = (EPIC_INTR_BASE_INTERNAL + 25),
    e_EPIC_INTR_DUART           = (EPIC_INTR_BASE_INTERNAL + 26),
    e_EPIC_INTR_I2C             = (EPIC_INTR_BASE_INTERNAL + 27),
    e_EPIC_INTR_PM              = (EPIC_INTR_BASE_INTERNAL + 28),
    e_EPIC_INTR_SEC1            = (EPIC_INTR_BASE_INTERNAL + 29),
    e_EPIC_INTR_USB2            = (EPIC_INTR_BASE_INTERNAL + 30),
    e_EPIC_INTR_GPIO            = (EPIC_INTR_BASE_INTERNAL + 31),
    e_EPIC_INTR_ETSEC2_GRP2_TX  = (EPIC_INTR_BASE_INTERNAL + 35),
    e_EPIC_INTR_ETSEC2_GRP2_RX  = (EPIC_INTR_BASE_INTERNAL + 36),
    e_EPIC_INTR_SEC2            = (EPIC_INTR_BASE_INTERNAL + 42),
    e_EPIC_INTR_ESPI            = (EPIC_INTR_BASE_INTERNAL + 43),
    e_EPIC_INTR_TDM             = (EPIC_INTR_BASE_INTERNAL + 46),
    e_EPIC_INTR_TDM_ERR         = (EPIC_INTR_BASE_INTERNAL + 47),
    e_EPIC_INTR_DIU             = (EPIC_INTR_BASE_INTERNAL + 48),
    e_EPIC_INTR_ETSEC2_GRP2_ERR = (EPIC_INTR_BASE_INTERNAL + 51),
    e_EPIC_INTR_PTP_ETSEC1      = (EPIC_INTR_BASE_INTERNAL + 52),
    e_EPIC_INTR_ESDHC           = (EPIC_INTR_BASE_INTERNAL + 56),
    e_EPIC_INTR_SATA1           = (EPIC_INTR_BASE_INTERNAL + 58),
    e_EPIC_INTR_SSI             = (EPIC_INTR_BASE_INTERNAL + 59),
    e_EPIC_INTR_DMA2_1          = (EPIC_INTR_BASE_INTERNAL + 60),
    e_EPIC_INTR_DMA2_2          = (EPIC_INTR_BASE_INTERNAL + 61),
    e_EPIC_INTR_DMA2_3          = (EPIC_INTR_BASE_INTERNAL + 62),
    e_EPIC_INTR_DMA2_4          = (EPIC_INTR_BASE_INTERNAL + 63),

    e_EPIC_INTR_MSG_0           = (EPIC_INTR_BASE_MSG + 0),
    e_EPIC_INTR_MSG_1           = (EPIC_INTR_BASE_MSG + 1),
    e_EPIC_INTR_MSG_2           = (EPIC_INTR_BASE_MSG + 2),
    e_EPIC_INTR_MSG_3           = (EPIC_INTR_BASE_MSG + 3),
    e_EPIC_INTR_MSG_4           = (EPIC_INTR_BASE_MSG + 4),
    e_EPIC_INTR_MSG_5           = (EPIC_INTR_BASE_MSG + 5),
    e_EPIC_INTR_MSG_6           = (EPIC_INTR_BASE_MSG + 6),
    e_EPIC_INTR_MSG_7           = (EPIC_INTR_BASE_MSG + 7),
 
    e_EPIC_INTR_IPI_0           = (EPIC_INTR_BASE_IPI + 0),
    e_EPIC_INTR_IPI_1           = (EPIC_INTR_BASE_IPI + 1),
    e_EPIC_INTR_IPI_2           = (EPIC_INTR_BASE_IPI + 2),
    e_EPIC_INTR_IPI_3           = (EPIC_INTR_BASE_IPI + 3),
 
    e_EPIC_INTR_SHARED_MSG_0    = (EPIC_INTR_BASE_SHARED_MSG + 0),
    e_EPIC_INTR_SHARED_MSG_1    = (EPIC_INTR_BASE_SHARED_MSG + 1),
    e_EPIC_INTR_SHARED_MSG_2    = (EPIC_INTR_BASE_SHARED_MSG + 2),
    e_EPIC_INTR_SHARED_MSG_3    = (EPIC_INTR_BASE_SHARED_MSG + 3),
    e_EPIC_INTR_SHARED_MSG_4    = (EPIC_INTR_BASE_SHARED_MSG + 4),
    e_EPIC_INTR_SHARED_MSG_5    = (EPIC_INTR_BASE_SHARED_MSG + 5),
    e_EPIC_INTR_SHARED_MSG_6    = (EPIC_INTR_BASE_SHARED_MSG + 6),
    e_EPIC_INTR_SHARED_MSG_7    = (EPIC_INTR_BASE_SHARED_MSG + 7),

    e_EPIC_INTR_SPURIOUS,
    e_EPIC_INTR_DUMMY_LAST,
 
    e_EPIC_INTR_RESERVED        = 0xFAFA
 
} e_EpicIntrSrc;
 
/*****************************************************************************
 PCI/PCIe INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
/* PCI modules registration */
void PCIE_CTRL_Register(t_Handle h_PciCtrl);

#define PCI_MODULES_REGISTRATION \
    { { e_MODULE_ID_PCIE_1, PCIE_CTRL_Register, e_MODULE_ID_PCIE_1, e_MODULE_ID_PCIE_1_ATMU, e_MODULE_ID_MSI },\
      { e_MODULE_ID_PCIE_2, PCIE_CTRL_Register, e_MODULE_ID_PCIE_2, e_MODULE_ID_PCIE_2_ATMU, e_MODULE_ID_MSI },\
      { e_MODULE_ID_PCIE_3, PCIE_CTRL_Register, e_MODULE_ID_PCIE_3, e_MODULE_ID_PCIE_3_ATMU, e_MODULE_ID_MSI }}

#define PCI_MAX_INBOUND_WINDOWS_NUM     4
#define PCI_MAX_OUTBOUND_WINDOWS_NUM    5

/**************************************************************************//**
 @Description   Target interface of an inbound window
*//***************************************************************************/
typedef enum e_PciTargetInterface
{
    e_PCI_TARGET_PCIE_1         = 0x2,  /**<  PCI Express target interface 1                                 */
    e_PCI_TARGET_PCIE_2         = 0x1,  /**<  PCI Express target interface 2                                      */
    e_PCI_TARGET_PCIE_3         = 0x3,  /**<  PCI Express target interface 3                                        */
    e_PCI_TARGET_LOCAL_MEMORY   = 0xF   /**<  Local Memory (DDR SDRAM, Local Bus, SRAM) target interface */
 
} e_PciTargetInterface;

/*****************************************************************************
 DDR INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define DDR_NUM_OF_VALID_CS     4

/*****************************************************************************
 DMA INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define DMA_NUM_OF_CONTROLLERS  2

/*****************************************************************************
 ETSEC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define ETSEC_NUM_OF_PORTS  2

#define ETSEC_REV2

#define ETSEC_INTERFACES_MAP         \
{                                    \
    { e_ENET_IF_RGMII,  { 1, 1} },   \
    { e_ENET_IF_SGMII,  { 1, 1} },   \
}

/*****************************************************************************
 1588 INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define PTP_V2

/*****************************************************************************
 SATA INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SATA_NUM_OF_DEVS   2

/*****************************************************************************
 SEC INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SEC_NUM_OF_CHANNELS     4
#define SEC_AFEU_UNAVAILABLE

/*****************************************************************************
 SPI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SPI_NUM_OF_CONTROLLERS  1

/*****************************************************************************
 SSI INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define SSI_NUM_OF_CONTROLLERS        2

/*****************************************************************************
 TDM INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define NUM_OF_TDM_CTRLS               1

/*****************************************************************************
 USB INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/
#define USB_NUM_OF_CONTROLLERS  2


/*****************************************************************************
 Registers Masks
 ******************************************************************************/

/* CLKDVDR */
#define CLKDVDR_PXCKEN_MASK     0x80000000
#define CLKDVDR_SSICKEN_MASK    0x20000000
#define CLKDVDR_PXCKINV_MASK    0x10000000
#define CLKDVDR_PXCKDLY_MASK    0x06000000
#define CLKDVDR_PXCLK_MASK      0x00FF0000
#define CLKDVDR_SSICLK_MASK     0x000000FF

#define CLKDVDR_PXCLK_DIV_MIN   1
#define CLKDVDR_PXCLK_DIV_MAX   255


#endif /* __PART_INTEGRATION_EXT_H */
