/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          ptp_integration_ext.h

 @Description   Integration-specific definitions and API for PTP (IEEE 1588).
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __PTP_INTEGRATION_EXT_H
#define __PTP_INTEGRATION_EXT_H


/* Contains external information specific to PTP eTSEC */
#include "ptp_etsec_ext.h"


/**************************************************************************//**
 @Group         ptp_grp PTP Application Programming Interface

 @Description   PTP functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_init_grp PTP Initialization Unit

 @Description   PTP initialization API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PTP Port ID.
*//***************************************************************************/
typedef enum e_PtpPort
{
    e_PTP_PORT_ETSEC_1,     /**< PTP Port on eTSEC1 */
    e_PTP_PORT_ETSEC_2,     /**< PTP Port on eTSEC2 */
    e_PTP_PORT_DUMMY_LAST
} e_PtpPort;

#define PTP_NUM_OF_PORTS    e_PTP_PORT_DUMMY_LAST   /**< Number of PTP ports */


/** @} */ /* end of ptp_grp */
/** @} */ /* end of ptp_init_grp */


/*****************************************************************************
 INTERNAL INTEGRATION-SPECIFIC DEFINITIONS
******************************************************************************/

t_Error PTP_ETSEC_Register(t_Handle h_Ptp, uint8_t ptpModuleId);

#define PTP_MODULES_REGISTRATION \
{ \
    { PTP_ETSEC_Register, e_MODULE_ID_ETSEC_1_PTP,   e_EPIC_INTR_PTP_ETSEC1          /* @@@@ temporary */ }, \
    { PTP_ETSEC_Register, e_MODULE_ID_ETSEC_2_PTP,   e_EPIC_INTR_PTP_ETSEC1          /* @@@@ temporary */ }, \
}

#define PTP_PORTS_MAPPING \
{ \
/*                       PTP Module     Internal ID */ \
/* e_PTP_PORT_ETSEC_1 */ { 0,              0}, \
/* e_PTP_PORT_ETSEC_2 */ { 1,              0}  \
}

#endif /* __PTP_INTEGRATION_EXT_H */
