/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "core_ext.h"
#include "string_ext.h"
#include "part_ext.h"
#include "xx_ext.h"
#include "P4080.h"


static int64_t GetModulePhysAddr(t_P4080 *p_P4080, e_ModuleId moduleId)
{
    switch (moduleId)
    {
        case (e_MODULE_ID_LAW):
            return p_P4080->ccsrPhysBaseAddr + LAW_OFFSET;
        case (e_MODULE_ID_LBC):
            return p_P4080->ccsrPhysBaseAddr + LBC_OFFSET;
        case (e_MODULE_ID_DUART_1):
            return p_P4080->ccsrPhysBaseAddr + DUART0_OFFSET;
        case (e_MODULE_ID_DUART_2):
            return p_P4080->ccsrPhysBaseAddr + DUART1_OFFSET;
        case (e_MODULE_ID_DUART_3):
            return p_P4080->ccsrPhysBaseAddr + DUART2_OFFSET;
        case (e_MODULE_ID_DUART_4):
            return p_P4080->ccsrPhysBaseAddr + DUART3_OFFSET;
        case (e_MODULE_ID_QM):
            return p_P4080->ccsrPhysBaseAddr + QM_OFFSET;
        case (e_MODULE_ID_BM):
            return p_P4080->ccsrPhysBaseAddr + BM_OFFSET;
        case (e_MODULE_ID_SEC_GEN):
            return p_P4080->ccsrPhysBaseAddr + SEC_GEN_OFFSET;
        case (e_MODULE_ID_SEC_QI):
            return p_P4080->ccsrPhysBaseAddr + SEC_QI_OFFSET;
        case (e_MODULE_ID_SEC_JQ0):
            return p_P4080->ccsrPhysBaseAddr + SEC_JQ0_OFFSET;
        case (e_MODULE_ID_SEC_JQ1):
            return p_P4080->ccsrPhysBaseAddr + SEC_JQ1_OFFSET;
        case (e_MODULE_ID_SEC_JQ2):
            return p_P4080->ccsrPhysBaseAddr + SEC_JQ2_OFFSET;
        case (e_MODULE_ID_SEC_JQ3):
            return p_P4080->ccsrPhysBaseAddr + SEC_JQ3_OFFSET;
        case (e_MODULE_ID_SEC_RTIC):
            return p_P4080->ccsrPhysBaseAddr + SEC_RTIC_OFFSET;
        case (e_MODULE_ID_SEC_DECO0_CCB0):
            return p_P4080->ccsrPhysBaseAddr + SEC_DECO0_CCB0_OFFSET;
        case (e_MODULE_ID_SEC_DECO1_CCB1):
            return p_P4080->ccsrPhysBaseAddr + SEC_DECO1_CCB1_OFFSET;
        case (e_MODULE_ID_SEC_DECO2_CCB2):
            return p_P4080->ccsrPhysBaseAddr + SEC_DECO2_CCB2_OFFSET;
        case (e_MODULE_ID_SEC_DECO3_CCB3):
            return p_P4080->ccsrPhysBaseAddr + SEC_DECO3_CCB3_OFFSET;
        case (e_MODULE_ID_SEC_DECO4_CCB4):
            return p_P4080->ccsrPhysBaseAddr + SEC_DECO4_CCB4_OFFSET;
        case (e_MODULE_ID_MPIC):
            return p_P4080->ccsrPhysBaseAddr + MPIC_OFFSET;
        case (e_MODULE_ID_GPIO):
            return p_P4080->ccsrPhysBaseAddr + GPIO_OFFSET;
        case (e_MODULE_ID_CPC_1):
            return p_P4080->ccsrPhysBaseAddr + CPC1_OFFSET;
        case (e_MODULE_ID_CPC_2):
            return p_P4080->ccsrPhysBaseAddr + CPC2_OFFSET;
        case (e_MODULE_ID_SERDES):
            return p_P4080->ccsrPhysBaseAddr + SERDES_OFFSET;
        case (e_MODULE_ID_PAMU):
            return p_P4080->ccsrPhysBaseAddr + PAMU_OFFSET;
        case (e_MODULE_ID_FM1):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET;
        case (e_MODULE_ID_FM1_RTC):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_RTC_OFFSET;
        case (e_MODULE_ID_FM1_MURAM):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_MURAM_OFFSET;
        case (e_MODULE_ID_FM1_BMI):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_BMI_OFFSET;
        case (e_MODULE_ID_FM1_QMI):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_QMI_OFFSET;
        case (e_MODULE_ID_FM1_PRS):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PRS_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO0_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO1):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO1_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO2):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO2_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO3):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO3_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO4):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO4_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO5):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO5_OFFSET;
        case (e_MODULE_ID_FM1_PORT_HO6):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_HO6_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GRx0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GRX0_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GRx1):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GRX1_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GRx2):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GRX2_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GRx3):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GRX3_OFFSET;
        case (e_MODULE_ID_FM1_PORT_10GRx0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_10GRX0_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GTx0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GTX0_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GTx1):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GTX1_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GTx2):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GTX2_OFFSET;
        case (e_MODULE_ID_FM1_PORT_1GTx3):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_1GTX3_OFFSET;
        case (e_MODULE_ID_FM1_PORT_10GTx0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PORT_10GTX0_OFFSET;
        case (e_MODULE_ID_FM1_PLCR):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PLCR_OFFSET;
        case (e_MODULE_ID_FM1_KG):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_KG_OFFSET;
        case (e_MODULE_ID_FM1_DMA):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_DMA_OFFSET;
        case (e_MODULE_ID_FM1_FPM):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_FPM_OFFSET;
        case (e_MODULE_ID_FM1_IRAM):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_IRAM_OFFSET;
        case (e_MODULE_ID_FM1_1GMDIO0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMDIO0_OFFSET;
        case (e_MODULE_ID_FM1_1GMDIO1):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMDIO1_OFFSET;
        case (e_MODULE_ID_FM1_1GMDIO2):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMDIO2_OFFSET;
        case (e_MODULE_ID_FM1_1GMDIO3):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMDIO3_OFFSET;
        case (e_MODULE_ID_FM1_10GMDIO):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_10GMDIO_OFFSET;
        case (e_MODULE_ID_FM1_PRS_IRAM):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_PRS_IRAM_OFFSET;
        case (e_MODULE_ID_FM1_1GMAC0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMAC0_OFFSET;
        case (e_MODULE_ID_FM1_1GMAC1):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMAC1_OFFSET;
        case (e_MODULE_ID_FM1_1GMAC2):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMAC2_OFFSET;
        case (e_MODULE_ID_FM1_1GMAC3):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_1GMAC3_OFFSET;
        case (e_MODULE_ID_FM1_10GMAC0):
            return p_P4080->ccsrPhysBaseAddr + FM1_OFFSET + FM_10GMAC0_OFFSET;

        case (e_MODULE_ID_FM2):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET;
        case (e_MODULE_ID_FM2_RTC):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_RTC_OFFSET;
        case (e_MODULE_ID_FM2_MURAM):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_MURAM_OFFSET;
        case (e_MODULE_ID_FM2_BMI):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_BMI_OFFSET;
        case (e_MODULE_ID_FM2_QMI):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_QMI_OFFSET;
        case (e_MODULE_ID_FM2_PRS):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PRS_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO0_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO1):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO1_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO2):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO2_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO3):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO3_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO4):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO4_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO5):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO5_OFFSET;
        case (e_MODULE_ID_FM2_PORT_HO6):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_HO6_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GRx0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GRX0_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GRx1):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GRX1_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GRx2):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GRX2_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GRx3):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GRX3_OFFSET;
        case (e_MODULE_ID_FM2_PORT_10GRx0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_10GRX0_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GTx0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GTX0_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GTx1):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GTX1_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GTx2):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GTX2_OFFSET;
        case (e_MODULE_ID_FM2_PORT_1GTx3):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_1GTX3_OFFSET;
        case (e_MODULE_ID_FM2_PORT_10GTx0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PORT_10GTX0_OFFSET;
        case (e_MODULE_ID_FM2_PLCR):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PLCR_OFFSET;
        case (e_MODULE_ID_FM2_KG):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_KG_OFFSET;
        case (e_MODULE_ID_FM2_DMA):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_DMA_OFFSET;
        case (e_MODULE_ID_FM2_FPM):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_FPM_OFFSET;
        case (e_MODULE_ID_FM2_IRAM):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_IRAM_OFFSET;
        case (e_MODULE_ID_FM2_1GMDIO0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMDIO0_OFFSET;
        case (e_MODULE_ID_FM2_1GMDIO1):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMDIO1_OFFSET;
        case (e_MODULE_ID_FM2_1GMDIO2):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMDIO2_OFFSET;
        case (e_MODULE_ID_FM2_1GMDIO3):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMDIO3_OFFSET;
        case (e_MODULE_ID_FM2_10GMDIO):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_10GMDIO_OFFSET;
        case (e_MODULE_ID_FM2_PRS_IRAM):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_PRS_IRAM_OFFSET;
        case (e_MODULE_ID_FM2_1GMAC0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMAC0_OFFSET;
        case (e_MODULE_ID_FM2_1GMAC1):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMAC1_OFFSET;
        case (e_MODULE_ID_FM2_1GMAC2):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMAC2_OFFSET;
        case (e_MODULE_ID_FM2_1GMAC3):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_1GMAC3_OFFSET;
        case (e_MODULE_ID_FM2_10GMAC0):
            return p_P4080->ccsrPhysBaseAddr + FM2_OFFSET + FM_10GMAC0_OFFSET;

        case (e_MODULE_ID_BM_CE_PORTAL_0):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(0);
        case (e_MODULE_ID_BM_CI_PORTAL_0):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(0);
        case (e_MODULE_ID_BM_CE_PORTAL_1):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(1);
        case (e_MODULE_ID_BM_CI_PORTAL_1):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(1);
        case (e_MODULE_ID_BM_CE_PORTAL_2):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(2);
        case (e_MODULE_ID_BM_CI_PORTAL_2):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(2);
        case (e_MODULE_ID_BM_CE_PORTAL_3):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(3);
        case (e_MODULE_ID_BM_CI_PORTAL_3):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(3);
        case (e_MODULE_ID_BM_CE_PORTAL_4):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(4);
        case (e_MODULE_ID_BM_CI_PORTAL_4):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(4);
        case (e_MODULE_ID_BM_CE_PORTAL_5):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(5);
        case (e_MODULE_ID_BM_CI_PORTAL_5):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(5);
        case (e_MODULE_ID_BM_CE_PORTAL_6):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(6);
        case (e_MODULE_ID_BM_CI_PORTAL_6):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(6);
        case (e_MODULE_ID_BM_CE_PORTAL_7):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(7);
        case (e_MODULE_ID_BM_CI_PORTAL_7):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(7);
        case (e_MODULE_ID_BM_CE_PORTAL_8):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(8);
        case (e_MODULE_ID_BM_CI_PORTAL_8):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(8);
        case (e_MODULE_ID_BM_CE_PORTAL_9):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(9);
        case (e_MODULE_ID_BM_CI_PORTAL_9):
            return p_P4080->bmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(9);
        case (e_MODULE_ID_QM_CE_PORTAL_0):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(0);
        case (e_MODULE_ID_QM_CI_PORTAL_0):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(0);
        case (e_MODULE_ID_QM_CE_PORTAL_1):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(1);
        case (e_MODULE_ID_QM_CI_PORTAL_1):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(1);
        case (e_MODULE_ID_QM_CE_PORTAL_2):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(2);
        case (e_MODULE_ID_QM_CI_PORTAL_2):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(2);
        case (e_MODULE_ID_QM_CE_PORTAL_3):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(3);
        case (e_MODULE_ID_QM_CI_PORTAL_3):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(3);
        case (e_MODULE_ID_QM_CE_PORTAL_4):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(4);
        case (e_MODULE_ID_QM_CI_PORTAL_4):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(4);
        case (e_MODULE_ID_QM_CE_PORTAL_5):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(5);
        case (e_MODULE_ID_QM_CI_PORTAL_5):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(5);
        case (e_MODULE_ID_QM_CE_PORTAL_6):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(6);
        case (e_MODULE_ID_QM_CI_PORTAL_6):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(6);
        case (e_MODULE_ID_QM_CE_PORTAL_7):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(7);
        case (e_MODULE_ID_QM_CI_PORTAL_7):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(7);
        case (e_MODULE_ID_QM_CE_PORTAL_8):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(8);
        case (e_MODULE_ID_QM_CI_PORTAL_8):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(8);
        case (e_MODULE_ID_QM_CE_PORTAL_9):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CE_OFFSET + PORTALS_OFFSET_CE(9);
        case (e_MODULE_ID_QM_CI_PORTAL_9):
            return p_P4080->qmPortalsPhysBaseAddr + PORTALS_CI_OFFSET + PORTALS_OFFSET_CI(9);
        default:
            return (int64_t)ILLEGAL_BASE;
    }
}

/*****************************************************************************/
static t_Error SetDevicesConfiguration(t_P4080 *p_P4080)
{
    uint32_t            prtcl;
    int                 i;

    ASSERT_COND(p_P4080);

    prtcl = RCW_GET_SRDS_PRTCL(p_P4080->p_GurMemMap);
    if (!p_P4080->f_BoardIsValidSerDesConfigurationCB(prtcl))
        RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("Ilegal SerDes protocol - 0x%x", prtcl));

    /* FM 1 DTSEC */
    if (P4080_IsDeviceConfiguredInRcw(p_P4080, SRDS_PRTCL_SGMII_FM1))
    {
        for (i = 0; i < 4; i++)
        {
            p_P4080->devs.fms[0].dtsecs[i].en = TRUE;
            p_P4080->devs.fms[0].dtsecs[i].serdesBank = 3;
            p_P4080->devs.fms[0].dtsecs[i].serdesLane = e_SERDES_LANE_A_B_C_D;
            p_P4080->devs.fms[0].dtsecs[i].sgmii = TRUE;
        }
    } else {
        if (RCW_GET_EC1(p_P4080->p_GurMemMap) == 0)
        {
            p_P4080->devs.fms[0].dtsecs[0].en = TRUE;
            p_P4080->devs.fms[0].dtsecs[0].sgmii = FALSE;
        }
        if (RCW_GET_EC2(p_P4080->p_GurMemMap) == 2)
        {
            p_P4080->devs.fms[0].dtsecs[1].en = TRUE;
            p_P4080->devs.fms[0].dtsecs[1].sgmii = FALSE;
        }
    }
    /* FM 1 TGEC */
    if (P4080_IsDeviceConfiguredInRcw(p_P4080, SRDS_PRTCL_XAUI_FM1))
    {
        p_P4080->devs.fms[0].tgec.en = TRUE;
        p_P4080->devs.fms[0].tgec.serdesBank = 3;
        p_P4080->devs.fms[0].tgec.serdesLane = e_SERDES_LANE_A_B_C_D;
    }
    /* FM 2 DTSEC */
    if (P4080_IsDeviceConfiguredInRcw(p_P4080, SRDS_PRTCL_SGMII_FM2))
    {
        if ((prtcl == 15) || (prtcl == 16))
        {
            for (i = 0; i < 4; i++)
            {
                p_P4080->devs.fms[1].dtsecs[i].en = TRUE;
                p_P4080->devs.fms[1].dtsecs[i].serdesBank = 1;
                p_P4080->devs.fms[1].dtsecs[i].serdesLane = e_SERDES_LANE_E_F_G_H;
                p_P4080->devs.fms[1].dtsecs[i].sgmii = TRUE;
            }
        }
        else if ((prtcl < 11) || (prtcl > 16))
        {
            for (i = 0; i < 4; i++)
            {
                p_P4080->devs.fms[1].dtsecs[i].en = TRUE;
                p_P4080->devs.fms[1].dtsecs[i].serdesBank = 2;
                p_P4080->devs.fms[1].dtsecs[i].serdesLane = e_SERDES_LANE_A_B_C_D;
                p_P4080->devs.fms[1].dtsecs[i].sgmii = TRUE;
            }
        } else
        {
            for (i = 2; i < 4; i++)
            {
                p_P4080->devs.fms[1].dtsecs[i].en = TRUE;
                p_P4080->devs.fms[1].dtsecs[i].serdesBank = 1;
                p_P4080->devs.fms[1].dtsecs[i].serdesLane = e_SERDES_LANE_G_H;
                p_P4080->devs.fms[1].dtsecs[i].sgmii = TRUE;
            }
        }
    } else if (RCW_GET_EC2(p_P4080->p_GurMemMap) == 0)
    {
        p_P4080->devs.fms[1].dtsecs[0].en = TRUE;
        p_P4080->devs.fms[1].dtsecs[0].sgmii = FALSE;
    }
    /* FM 2 TGEC */
    if (P4080_IsDeviceConfiguredInRcw(p_P4080, SRDS_PRTCL_XAUI_FM2))
    {
        p_P4080->devs.fms[1].tgec.en = TRUE;
        p_P4080->devs.fms[1].tgec.serdesBank = 2;
        p_P4080->devs.fms[1].tgec.serdesLane = e_SERDES_LANE_A_B_C_D;
    }

    return E_OK;
}

/*****************************************************************************/
static uint64_t GetModuleVirtAddrCB(t_Handle h_P4080, e_ModuleId module)
{
    t_P4080     *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, e_MODULE_ID_DUMMY_LAST);

    return CAST_POINTER_TO_UINT64(XX_PhysToVirt((physAddress_t)(GetModulePhysAddr(p_P4080, module))));
}

/*****************************************************************************/
static e_ModuleId GetModuleIdByBaseCB(t_Handle h_P4080, uint64_t baseAddress)
{
    t_P4080     *p_P4080 = (t_P4080 *)h_P4080;
    e_ModuleId  moduleId;
    uint64_t    physBbaseAddress = (physAddress_t)XX_VirtToPhys(CAST_UINT64_TO_POINTER(baseAddress));

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, e_MODULE_ID_DUMMY_LAST);

    for (moduleId = (e_ModuleId)0; moduleId < e_MODULE_ID_DUMMY_LAST; moduleId++)
        if (physBbaseAddress == GetModulePhysAddr(p_P4080, moduleId))
            return moduleId;

    return e_MODULE_ID_DUMMY_LAST;
}

/*****************************************************************************/
t_Handle P4080_ConfigAndInit(t_P4080Params *p_P4080Params)
{
    t_P4080     *p_P4080;

    p_P4080 = (t_P4080 *)XX_Malloc(sizeof(t_P4080));
    if (!p_P4080)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("P4080 driver structure"));
        return NULL;
    }

    memset(p_P4080, 0, sizeof(t_P4080));

    /* Initialize 'part' parameters with P4080 parameters and service routines. */
    p_P4080->part.f_GetModuleBase = GetModuleVirtAddrCB;
    p_P4080->part.f_GetModuleIdByBase = GetModuleIdByBaseCB;
    p_P4080->part.f_GetRevInfo = (t_RevInfoCallback *)P4080_GetRevInfo;

    p_P4080->p_GurMemMap            = CAST_UINT64_TO_POINTER(p_P4080Params->ccsrBaseAddress + GUTIL_OFFSET);
    p_P4080->p_RcpmMemMap           = CAST_UINT64_TO_POINTER(p_P4080Params->ccsrBaseAddress + RCPM_OFFSET);
    p_P4080->ccsrPhysBaseAddr       = (uint64_t)XX_VirtToPhys(CAST_UINT64_TO_POINTER(p_P4080Params->ccsrBaseAddress));
    p_P4080->qmPortalsPhysBaseAddr  = (uint64_t)XX_VirtToPhys(CAST_UINT64_TO_POINTER(p_P4080Params->qmPortalsBaseAddress));
    p_P4080->bmPortalsPhysBaseAddr  = (uint64_t)XX_VirtToPhys(CAST_UINT64_TO_POINTER(p_P4080Params->bmPortalsBaseAddress));
    p_P4080->f_BoardIsValidSerDesConfigurationCB = p_P4080Params->f_BoardIsValidSerDesConfigurationCB;
    p_P4080->serdesLanesBank2       = p_P4080Params->serdesLanesBank2;
    p_P4080->serdesLanesBank3       = p_P4080Params->serdesLanesBank3;

   SetDevicesConfiguration(p_P4080);

    return p_P4080;
}

/*****************************************************************************/
t_Error P4080_Free(t_Handle h_P4080)
{
    t_P4080 *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    XX_Free(p_P4080);

    return E_OK;
}

/*****************************************************************************/
uint64_t P4080_GetModulePhysBase(t_Handle h_P4080, e_ModuleId module)
{
    t_P4080 *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, 0);

    return GetModulePhysAddr(p_P4080, module);
}

/*****************************************************************************/
uint32_t P4080_GetPramSize(t_Handle h_P4080)
{
    SANITY_CHECK_RETURN_VALUE(h_P4080, E_INVALID_HANDLE, 0);

    UNUSED(h_P4080);
    return FM_MURAM_SIZE;
}

/*****************************************************************************/
e_P4080DeviceName P4080_GetRevInfo(t_Handle h_P4080)
{
    t_P4080  *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, e_P4080_REV_INVALID);

    return (e_P4080DeviceName)GET_UINT32(p_P4080->p_GurMemMap->svr);
}

/*****************************************************************************/
t_Error P4080_GetE500Factor(t_Handle h_P4080, uint8_t coreIndex, uint32_t *p_E500MulFactor, uint32_t *p_E500DivFactor)
{

    t_P4080  *p_P4080 = (t_P4080 *)h_P4080;
    t_P4080RCWCoreComplexDesc   CoreComplex[] = RCW_C_PLL_SEL;
    uint32_t            tmp;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    tmp = RCW_GET_C_PLL_SEL(p_P4080->p_GurMemMap,coreIndex);

    *p_E500MulFactor = RCW_GET_CC_PLL_RAT(p_P4080->p_GurMemMap,CoreComplex[tmp].ccSel);
    *p_E500DivFactor = CoreComplex[tmp].divider;
    return E_OK;
}

/*****************************************************************************/
uint32_t P4080_GetCcbFactor(t_Handle h_P4080)
{
    t_P4080  *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, 0);

    return RCW_GET_SYS_PLL_RAT(p_P4080->p_GurMemMap);
}

/*****************************************************************************/
uint32_t P4080_GetDdrFactor(t_Handle h_P4080)
{
    t_P4080  *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t            tmp;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, 0);

    tmp = RCW_GET_DDR_SYNC(p_P4080->p_GurMemMap);
    if(tmp == 1) /* synchronous mode */
        return P4080_GetCcbFactor(h_P4080);
    else
        return RCW_GET_MEM_PLL_RAT(p_P4080->p_GurMemMap);
}

/*****************************************************************************/
/*t_Error  P4080_GetDdrType(t_Handle h_P4080,e_DdrType *p_DdrType )
{
    t_P4080  *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t            tmpporsr2;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    tmpporsr2 = ((p_P4080->p_GurMemMap->porsr2 & PORSR2_DRAM_TYPE_MASK) >> PORSR2_DRAM_TYPE_SHIFT);

    if(tmpporsr2 == 0)
        *p_DdrType = e_DDR_DDR3;
    else
        *p_DdrType = e_DDR_DDR2;
    return E_OK;
}
*/

/*****************************************************************************/
t_Error  P4080_GetFmFactor(t_Handle h_P4080, uint8_t fmIndex, uint32_t *p_FmMulFactor, uint32_t *p_FmDivFactor)
{
    t_P4080  *p_P4080 = (t_P4080 *)h_P4080;
    uint8_t             tmp;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    tmp = RCW_GET_FM_CLK_SEL(p_P4080->p_GurMemMap,fmIndex);
    *p_FmDivFactor = 2;

    if (tmp)
        *p_FmMulFactor = RCW_GET_CC_PLL_RAT(p_P4080->p_GurMemMap,3);
    else
        *p_FmMulFactor = P4080_GetCcbFactor(h_P4080);
    return E_OK;
}

/*****************************************************************************/
void P4080_Reset(t_Handle h_P4080)
{
    t_P4080             *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN(p_P4080, E_INVALID_HANDLE);

    UNUSED(p_P4080);
    REPORT_ERROR(MINOR, E_NOT_SUPPORTED, NO_MSG);
}

/*****************************************************************************/
t_Error P4080_CoreTimeBaseEnable(t_Handle h_P4080)
{
    t_P4080             *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t            tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    tmpReg = GET_UINT32(p_P4080->p_RcpmMemMap->ctbenrl);
    WRITE_UINT32(p_P4080->p_RcpmMemMap->ctbenrl, tmpReg | (uint32_t)(0x1 << CORE_GetId()));

    return E_OK;
}

/*****************************************************************************/
t_Error P4080_CoreTimeBaseDisable(t_Handle h_P4080)
{
    t_P4080             *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t            tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    tmpReg = GET_UINT32(p_P4080->p_RcpmMemMap->ctbenrl);
    WRITE_UINT32(p_P4080->p_RcpmMemMap->ctbenrl, tmpReg & ~(uint32_t)(0x1 << CORE_GetId()));

    return E_OK;
}

/*****************************************************************************/
t_P4080DevParams * P4080_GetDevicesConfiguration(t_Handle h_P4080)
{
    t_P4080             *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, NULL);

    return &p_P4080->devs;
}

/************************************************************/
/* Enable device in DEVDISR register                        */
/************************************************************/
t_Error P4080_DeviceEnable(t_Handle h_P4080, e_ModuleId module, bool enable)
{
    t_P4080             *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t            tmpdevdisr,device, *p_Devdisr;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_P4080->p_GurMemMap, E_INVALID_HANDLE);

    switch (module)
    {
        case e_MODULE_ID_BM :
        case e_MODULE_ID_QM :
            device = P4080_DEV_DISABLE_QM_BM;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM1 :
            device = P4080_DEV_DISABLE_FM1;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM1_1GMAC0 :
            device = P4080_DEV_DISABLE_FM1_DTSEC1;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM1_1GMAC1 :
            device = P4080_DEV_DISABLE_FM1_DTSEC2;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM1_1GMAC2 :
            device = P4080_DEV_DISABLE_FM1_DTSEC3;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM1_1GMAC3 :
            device = P4080_DEV_DISABLE_FM1_DTSEC4;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM1_10GMAC0 :
            device = P4080_DEV_DISABLE_FM1_TGEC;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM2 :
            device = P4080_DEV_DISABLE_FM2;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM2_1GMAC0 :
            device = P4080_DEV_DISABLE_FM2_DTSEC1;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM2_1GMAC1 :
            device = P4080_DEV_DISABLE_FM2_DTSEC2;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM2_1GMAC2 :
            device = P4080_DEV_DISABLE_FM2_DTSEC3;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM2_1GMAC3 :
            device = P4080_DEV_DISABLE_FM2_DTSEC4;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        case e_MODULE_ID_FM2_10GMAC0 :
            device = P4080_DEV_DISABLE_FM2_TGEC;
            p_Devdisr = (uint32_t*)&p_P4080->p_GurMemMap->devdisr2;
            break;
        default :
            RETURN_ERROR(MINOR,E_INVALID_SELECTION,NO_MSG);
    }

    tmpdevdisr = GET_UINT32(*p_Devdisr);
    if (enable)
        tmpdevdisr &= ~((uint32_t)device);
    else
        tmpdevdisr |= ((uint32_t)device);
    WRITE_UINT32(*p_Devdisr, tmpdevdisr);

    return E_OK;
}

/************************************************************/
/* Disable PAMU bypass in PAMUBYPENR register               */
/************************************************************/
t_Error P4080_PamuDisableBypass(t_Handle h_P4080, uint8_t pamuId, bool disable)
{
    t_P4080         *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t        pamubypenr;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(pamuId < PAMU_NUM_OF_PARTITIONS, E_INVALID_VALUE);

    pamubypenr = GET_UINT32(p_P4080->p_GurMemMap->pamubypenr);
    if (disable)
        pamubypenr &= ~P4080_GET_PAMU_BYPASS_ID(pamuId);
    else
        pamubypenr |= P4080_GET_PAMU_BYPASS_ID(pamuId);
    WRITE_UINT32(p_P4080->p_GurMemMap->pamubypenr, pamubypenr);

    return RCW_GET_SRDS_PRTCL(p_P4080->p_GurMemMap);
}

bool P4080_IsDeviceConfiguredInRcw(t_Handle h_P4080, e_SrdsPrtcl device)
{
    t_P4080         *p_P4080 = (t_P4080 *)h_P4080;
    e_SrdsPrtcl     serdesLanes[][SRDS_MAX_LANES] = SRDS_PRTCL_OPTIONS;
    uint32_t        i,prtcl = RCW_GET_SRDS_PRTCL(p_P4080->p_GurMemMap);

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, FALSE);

    /* Is serdes enabled at all? */
    if (!(GET_UINT32(p_P4080->p_GurMemMap->rcwsr[5]) & 0x2000))
        return FALSE;

    for (i = 0; i < SRDS_MAX_LANES; i++) {
        if (P4080_IsSrdsLaneEnabledInRcw(p_P4080, i) &&
            serdesLanes[prtcl][i] == device)
            return TRUE;
    }
    return FALSE;
}

uint32_t P4080_GetRcwSrdsPrtcl(t_Handle h_P4080)
{
    t_P4080         *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, 0);

    return RCW_GET_SRDS_PRTCL(p_P4080->p_GurMemMap);
}

bool P4080_IsSrdsLaneEnabledInRcw(t_Handle h_P4080, uint32_t lane)
{
    t_P4080         *p_P4080 = (t_P4080 *)h_P4080;
    uint32_t        lanes[][3] = SRDS_LANES;
    uint32_t        word = lanes[lane][1] / 32;
    uint32_t        bit = lanes[lane][1] % 32;

    SANITY_CHECK_RETURN_VALUE(p_P4080, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(lane < SRDS_MAX_LANES, E_INVALID_HANDLE, 0);

#ifdef SERDES_PLL_JITTER_ERRATA_SERDES8
    if (P4080_GetRevInfo(p_P4080) != e_P4080E_REV_1_0) {
        if (lane >= 10 && lane <= 13) {
            return !(p_P4080->serdesLanesBank2 &
                 (8 >> (lane - 10)));
        }

        if (lane >= 14 && lane <= 18) {
            return !(p_P4080->serdesLanesBank3 &
                 (8 >> (lane - 14)));
        }
    }
#endif /* SERDES_PLL_JITTER_ERRATA_SERDES8 */

    return !(GET_UINT32(p_P4080->p_GurMemMap->rcwsr[word]) & (0x80000000 >> bit));
}

t_Error P4080_SetDevicesBankStatus(t_Handle h_P4080, uint8_t serdesBank, bool status)
{
    t_P4080         *p_P4080 = (t_P4080 *)h_P4080;
    uint8_t         i,j;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(serdesBank && serdesBank < 4, E_INVALID_VALUE);

    for (i=0; i<INTG_MAX_NUM_OF_FM; i++)
    {
        for (j=0; j<FM_MAX_NUM_OF_1G_MACS; j++)
            if (p_P4080->devs.fms[i].dtsecs[j].serdesBank == serdesBank)
                p_P4080->devs.fms[i].dtsecs[j].en = status;

        for (j=0; j<FM_MAX_NUM_OF_10G_MACS; j++)
            if (p_P4080->devs.fms[i].tgec.serdesBank == serdesBank)
                p_P4080->devs.fms[i].tgec.en = status;
    }
    if ((!p_P4080->devs.fms[0].dtsecs[0].en) &&
        (RCW_GET_EC1(p_P4080->p_GurMemMap) == 0))
        {
            p_P4080->devs.fms[0].dtsecs[0].en = TRUE;
            p_P4080->devs.fms[0].dtsecs[0].sgmii = FALSE;
        }
    if ((!p_P4080->devs.fms[0].dtsecs[1].en) &&
        (RCW_GET_EC2(p_P4080->p_GurMemMap) == 2))
        {
            p_P4080->devs.fms[0].dtsecs[1].en = TRUE;
            p_P4080->devs.fms[0].dtsecs[1].sgmii = FALSE;
        }
    if ((!p_P4080->devs.fms[1].dtsecs[0].en) &&
        (RCW_GET_EC2(p_P4080->p_GurMemMap) == 0))
        {
            p_P4080->devs.fms[1].dtsecs[0].en = TRUE;
            p_P4080->devs.fms[1].dtsecs[0].sgmii = FALSE;
        }

    return E_OK;
}

t_Error P4080_DumpDevicesConfiguration(t_Handle h_P4080)
{
    t_P4080         *p_P4080 = (t_P4080 *)h_P4080;

    SANITY_CHECK_RETURN_ERROR(p_P4080, E_INVALID_HANDLE);

    XX_Print("\nDevices Configuration:\n");
    if (p_P4080->devs.fms[0].dtsecs[0].en)
    {
        if (p_P4080->devs.fms[0].dtsecs[0].sgmii)
            XX_Print("FM1 SGMII Bank3\n");
        else
            XX_Print("FM1 MAC1 RGMII\n");
    }
    if ((p_P4080->devs.fms[0].dtsecs[1].en) &&
        (!p_P4080->devs.fms[0].dtsecs[1].sgmii))
        XX_Print("FM1 MAC2 RGMII\n");
    if (p_P4080->devs.fms[0].tgec.en)
        XX_Print("FM1 has 10G Bank3\n");
    if (p_P4080->devs.fms[1].dtsecs[0].en)
    {
        if (p_P4080->devs.fms[1].dtsecs[0].sgmii)
            XX_Print("FM2 SGMII (all 4) Bank%d\n",p_P4080->devs.fms[1].dtsecs[0].serdesBank);
        else
            XX_Print("FM2 MAC1 RGMII\n");
    }
    else if (p_P4080->devs.fms[1].dtsecs[2].en)
        XX_Print("FM2 SGMII dTSEC[3:4] Bank1\n");
    if (p_P4080->devs.fms[1].tgec.en)
        XX_Print("FM2 has 10G Bank2\n");
    return E_OK;
}
