/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          P4080.h

 @Description   P4080 object data structure declaration, definitions and
                internal prototypes.
*//***************************************************************************/

#ifndef __P4080_H
#define __P4080_H

#include "std_ext.h"
#include "part_ext.h"


#define __ERR_MODULE__  MODULE_P4080

#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(push,1)
#endif /* defined(__MWERKS__) && ... */
#define MEM_MAP_START

/* Global Utilities Registers */
typedef _Packed struct
{
/*Power-On Reset Configuration Values */
    volatile uint32_t porsr1;           /* POR status register 1*/
    volatile uint32_t porsr2;           /* POR status register 2*/
    volatile uint8_t  RESERVED1[0x18];
    volatile uint32_t gpporcr1;         /* General-purpose POR configuration register */
    volatile uint8_t  RESERVED2[0x4C];
    volatile uint32_t devdisr1;         /* Device disable register 1 */
    volatile uint32_t devdisr2;         /* Device disable register 2 */
    volatile uint8_t  RESERVED3[0x1C];
    volatile uint32_t coredisr;         /* Core disable register */
    volatile uint8_t  RESERVED4[0x8];
    volatile uint32_t pvr;              /* Processor version register */
    volatile uint32_t svr;              /* System version register */
    volatile uint8_t  RESERVED5[0x8];
    volatile uint32_t rstcr;            /* Reset Control register */
    volatile uint32_t rstrqpblsr;       /* Reset request preboot loader status register */
    volatile uint8_t  RESERVED6[0x8];
    volatile uint32_t rstrqmr1;         /* Reset request mask register */
    volatile uint8_t  RESERVED7[0x4];
    volatile uint32_t rstrqsr1;         /* Reset request status register */
    volatile uint8_t  RESERVED8[0x8];
    volatile uint32_t rstrqwdtmr;       /* Reset request WDT mask register */
    volatile uint8_t  RESERVED9[0x4];
    volatile uint32_t rstrqwdtsr;       /* Reset request WDT status register */
    volatile uint8_t  RESERVED10[0x4];
    volatile uint32_t brr;              /* Boot release register */
    volatile uint8_t  RESERVED11[0x18];
    volatile uint32_t rcwsr[16];        /* Reset Control word status registers 1-16  */
    volatile uint8_t  RESERVED12[0xc0];

/*Scratch Registers */
    volatile uint32_t scratchrw[4];      /* Scratch read/write registers 1-4  */
    volatile uint8_t  RESERVED13[0xf0];
    volatile uint32_t scratchw1r[4];     /* Scratch write once registers 1-4  */
    volatile uint8_t  RESERVED14[0xf0];

/*Core Reset Status */
    volatile uint32_t crstsr[8];         /* Core reset status registers 0-7 (one per core)  */
    volatile uint8_t  RESERVED15[0xe0];

/*Peripheral Logical I/O Device Number (LIODN) Registers */
    volatile uint32_t pex1liodnr;        /* PCI Express 1 LIODN register */
    volatile uint32_t pex2liodnr;        /* PCI Express 2 LIODN register */
    volatile uint32_t pex3liodnr;        /* PCI Express 3 LIODN register */
    volatile uint8_t  RESERVED16[0x4];
    volatile uint32_t rio1liodnr;        /* RIO 1 LIODN register */
    volatile uint32_t rio2liodnr;        /* RIO 2 LIODN register */
    volatile uint8_t  RESERVED17[0x8];
    volatile uint32_t usb1liodnr;        /* USB 1 LIODN register */
    volatile uint32_t usb2liodnr;        /* USB 2 LIODN register */
    volatile uint8_t  RESERVED18[0x8];
    volatile uint32_t esdhcliodnr;       /* eSDHC LIODN register */
    volatile uint8_t  RESERVED19[0xC];
    volatile uint32_t rmuliodnr;         /* RIO meessage unit (eRMU) LIODN register */
    volatile uint8_t  RESERVED20[0x3C];
    volatile uint32_t dma1liodnr;        /* DMA 1 LIODN register */
    volatile uint32_t dma2liodnr;        /* DMA 2 LIODN register */
    volatile uint8_t  RESERVED21[0x78];

/*Peripheral Registers */
    volatile uint8_t  RESERVED22[0x4];
    volatile uint32_t pamubypenr;        /* PAMU bypass enable register */
    volatile uint32_t dmacr1;            /* DMA control register */
    volatile uint8_t  RESERVED23[0x7f4];

/*General Pin Multiplexing Control */
    volatile uint32_t pmuxcr;            /* Pin Multiplexing control register */
    volatile uint8_t  RESERVED24[0x1c];

/* Debug Source ID */
    volatile uint32_t dsrcid1a;          /* Debug Source ID 1A register */
    volatile uint32_t dsrcid1b;          /* Debug Source ID 1B register */
    volatile uint32_t dsrcid1c;          /* Debug Source ID 1C register */
    volatile uint32_t dsrcid1d;          /* Debug Source ID 1D register */
    volatile uint32_t dsrcid2a;          /* Debug Source ID 2A register */
    volatile uint32_t dsrcid2b;          /* Debug Source ID 2B register */
    volatile uint8_t  RESERVED25[0x8];

/*I/O Voltage Selection */
    volatile uint32_t iovselsr;           /* I/O voltage selection status register */
    volatile uint8_t  RESERVED26[0x1c];

/*Clock Disable Control */
    volatile uint32_t ddrclkdr;           /* DDR clock disable register */
    volatile uint8_t  RESERVED27[0x4];
    volatile uint32_t elbcclkdr;          /* eLBC clock disable register */
    volatile uint8_t  RESERVED28[0x14];

/*Polarity Control */
    volatile uint32_t sdhcpcr;            /* eSDHC Polarity configuration register */
    volatile uint8_t  RESERVED29[0x17c];

} _PackedType t_P4080GurMemMap;

typedef _Packed struct
{
    /* system configuration module */
    volatile uint32_t tbd;    /**< tbd */
} _PackedType t_P4080SysCfgMemMap;

typedef _Packed struct {
    volatile uint8_t  res1[4];
    volatile uint32_t cdozsrl;
    volatile uint8_t  res2[4];
    volatile uint32_t cdozcrl;
    volatile uint8_t  res3[4];
    volatile uint32_t cnapsrl;
    volatile uint8_t  res4[4];
    volatile uint32_t cnapcrl;
    volatile uint8_t  res5[4];
    volatile uint32_t cdozpsrl;
    volatile uint8_t  res6[4];
    volatile uint32_t cdozpcrl;
    volatile uint8_t  res7[4];
    volatile uint32_t cwaitsrl;
    volatile uint8_t  res8[4];
    volatile uint32_t cwdtdsrl;
    volatile uint32_t powmgtcsr;
    volatile uint8_t  res9[0xc];
    volatile uint32_t ippdexpcrn;
    volatile uint8_t  res10[0x30];
    volatile uint32_t ctbenrl;
    volatile uint8_t  res11[4];
    volatile uint32_t ctbckselrl;
    volatile uint8_t  res12[4];
    volatile uint32_t ctbhltcrl;
} _PackedType t_P4080RcpmMemMap;

#define MEM_MAP_END
#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(pop)
#endif /* defined(__MWERKS__) && ... */


/* Offsets relative to larger memory map, with base IMMRBAR */
#define LAW_OFFSET              0x00000c00
#define DDR1_OFFSET             0x00008000
#define DDR2_OFFSET             0x00009000
#define CPC1_OFFSET             0x00010000
#define CPC2_OFFSET             0x00011000
#define CCF_OFFSET              0x00018000
#define PAMU_OFFSET             0x00020000
#define MPIC_OFFSET             0x00040000
#define GUTIL_OFFSET            0x000e0000
#define RCPM_OFFSET             0x000e2000
#define SERDES_OFFSET           0x000ea000
#define DUART0_OFFSET           0x0011c500
#define DUART1_OFFSET           0x0011c600
#define DUART2_OFFSET           0x0011d500
#define DUART3_OFFSET           0x0011d600
#define LBC_OFFSET              0x00124000
#define GPIO_OFFSET             0x00130000
#define SEC_GEN_OFFSET          0x00300000
#define SEC_JQ0_OFFSET          0x00301000
#define SEC_JQ1_OFFSET          0x00302000
#define SEC_JQ2_OFFSET          0x00303000
#define SEC_JQ3_OFFSET          0x00304000
#define SEC_RESERVED_OFFSET     0x00305000
#define SEC_RTIC_OFFSET         0x00306000
#define SEC_QI_OFFSET           0x00307000
#define SEC_DECO0_CCB0_OFFSET   0x00308000
#define SEC_DECO1_CCB1_OFFSET   0x00309000
#define SEC_DECO2_CCB2_OFFSET   0x0030a000
#define SEC_DECO3_CCB3_OFFSET   0x0030b000
#define SEC_DECO4_CCB4_OFFSET   0x0030c000
#define PME_OFFSET              0x00316000
#define QM_OFFSET               0x00318000
#define BM_OFFSET               0x0031a000
#define FM1_OFFSET              0x00400000
#define FM2_OFFSET              0x00500000

/* Offsets relative to FM_BASE_OFFSET off of base IMMRBAR */
#define FM_MURAM_OFFSET         0x00000000
#define FM_BMI_OFFSET           0x00080000
#define FM_QMI_OFFSET           0x00080400
#define FM_PRS_OFFSET           0x00080800
#define FM_PORT_HO0_OFFSET      0x00081000
#define FM_PORT_HO1_OFFSET      0x00082000
#define FM_PORT_HO2_OFFSET      0x00083000
#define FM_PORT_HO3_OFFSET      0x00084000
#define FM_PORT_HO4_OFFSET      0x00085000
#define FM_PORT_HO5_OFFSET      0x00086000
#define FM_PORT_HO6_OFFSET      0x00087000
#define FM_PORT_1GRX0_OFFSET    0x00088000
#define FM_PORT_1GRX1_OFFSET    0x00089000
#define FM_PORT_1GRX2_OFFSET    0x0008a000
#define FM_PORT_1GRX3_OFFSET    0x0008b000
#define FM_PORT_10GRX0_OFFSET   0x00090000
#define FM_PORT_1GTX0_OFFSET    0x000a8000
#define FM_PORT_1GTX1_OFFSET    0x000a9000
#define FM_PORT_1GTX2_OFFSET    0x000aa000
#define FM_PORT_1GTX3_OFFSET    0x000ab000
#define FM_PORT_10GTX0_OFFSET   0x000b0000
#define FM_PLCR_OFFSET          0x000c0000
#define FM_KG_OFFSET            0x000c1000
#define FM_DMA_OFFSET           0x000c2000
#define FM_FPM_OFFSET           0x000c3000
#define FM_IRAM_OFFSET          0x000c4000
#define FM_PRS_IRAM_OFFSET      0x000c7000
#define FM_1GMAC0_OFFSET        0x000e0000
#define FM_1GMDIO0_OFFSET       0x000e1000
#define FM_1GMAC1_OFFSET        0x000e2000
#define FM_1GMDIO1_OFFSET       0x000e3000
#define FM_1GMAC2_OFFSET        0x000e4000
#define FM_1GMDIO2_OFFSET       0x000e5000
#define FM_1GMAC3_OFFSET        0x000e6000
#define FM_1GMDIO3_OFFSET       0x000e7000
#define FM_10GMAC0_OFFSET       0x000f0000
#define FM_10GMDIO_OFFSET       0x000f1000
#define FM_RTC_OFFSET           0x000fe000

#define PORTALS_CE_OFFSET           0x000000
#define PORTALS_CI_OFFSET           0x100000
#define PORTALS_OFFSET_CE(portal)   (0x4000 * portal)
#define PORTALS_OFFSET_CI(portal)   (0x1000 * portal)


/* RCW */
/* PLL CONFIGURATION (bits 0-127) */
#define RCW_SYS_PLL_CFG_MASK 0xc0000000          /**< RCW SYS PLL CFG field mask */
#define RCW_SYS_PLL_RAT_MASK 0x3e000000          /**< RCW SYS PLL RAT field mask */
#define RCW_MEM_PLL_CFG_MASK 0x003e0000          /**< RCW MEM PLL CFG field mask */
#define RCW_MEM_PLL_RAT_MASK 0x3e000000          /**< RCW MEM PLL RAT field mask */

#define RCW_CC_PLL_RAT_MASK  0x0000001f          /**< RCW Core Cluster PLL RAT field mask */
#define RCW_C_PLL_SEL_MASK   0x0000000f          /**< RCW Core Complex PLL SEL field mask */

#define RCW_GET_SYS_PLL_CFG(p_P4080GurMemMap)   ((GET_UINT32(p_P4080GurMemMap->rcwsr[0]) & RCW_SYS_PLL_CFG_MASK) >> (31-1))     /**< Macro to get RCW SYS PLL CFG field */
#define RCW_GET_SYS_PLL_RAT(p_P4080GurMemMap)   ((GET_UINT32(p_P4080GurMemMap->rcwsr[0]) & RCW_SYS_PLL_RAT_MASK) >> (31-6))     /**< Macro to get RCW SYS PLL RAT field */
#define RCW_GET_MEM_PLL_CFG(p_P4080GurMemMap)   ((GET_UINT32(p_P4080GurMemMap->rcwsr[0]) & RCW_SYS_PLL_CFG_MASK) >> (31-9))     /**< Macro to get RCW MEM PLL CFG field */
#define RCW_GET_MEM_PLL_RAT(p_P4080GurMemMap)   ((GET_UINT32(p_P4080GurMemMap->rcwsr[0]) & RCW_MEM_PLL_CFG_MASK) >> (31-14))    /**< Macro to get RCW MEM PLL RAT field */

#define RCW_GET_CC_PLL_RAT(p_P4080GurMemMap,ccIndex)    ((GET_UINT32(p_P4080GurMemMap->rcwsr[2]) >> (25 - (ccIndex-1)*8)) & RCW_CC_PLL_RAT_MASK)    /**< Macro to get RCW CC PLL RAT field */
#define RCW_GET_C_PLL_SEL(p_P4080GurMemMap,cIndex)      ((GET_UINT32(p_P4080GurMemMap->rcwsr[3]) >> (28 - (4 * cIndex))) & RCW_C_PLL_SEL_MASK)              /**< Macro to get RCW C PLL SEL field */

#define RCW_C_PLL_SEL_RESERVED {0xff,0xff}
#define RCW_C_PLL_SEL {     \
    {1, 1},                 \
    {1, 2},                 \
    RCW_C_PLL_SEL_RESERVED, \
    RCW_C_PLL_SEL_RESERVED, \
    {2, 1},                 \
    {2, 2},                 \
    RCW_C_PLL_SEL_RESERVED, \
    RCW_C_PLL_SEL_RESERVED, \
    {3, 1},                 \
    {3, 2},                 \
    RCW_C_PLL_SEL_RESERVED, \
    RCW_C_PLL_SEL_RESERVED, \
    {4, 1},                 \
    {4, 2},                 \
    RCW_C_PLL_SEL_RESERVED, \
    RCW_C_PLL_SEL_RESERVED  \
}

typedef struct t_P4080RCWCoreComplexDesc {
    uint8_t     ccSel;
    uint8_t     divider;
} t_P4080RCWCoreComplexDesc;

/* SERDES PLL AND PROTOCOL CONFIGURATION (bits 128-183) */
#define RCW_SRDS_PRTCL_MASK   0xfc000000          /**< RCW Serdes Protocol field mask */
#define RCW_GET_SRDS_PRTCL(p_P4080GurMemMap)      ((GET_UINT32(p_P4080GurMemMap->rcwsr[4]) & RCW_SRDS_PRTCL_MASK) >> (31-5))    /**< Macro to get RCW SRDS PRTCL field */

/* MISCELLANEOUS PLL-RELATED CONFIGURATION (bits 184-191) */
#define RCW_DDR_SYNC_MASK    0x00000004          /**< RCW DDR SYNC field mask */
#define RCW_GET_DDR_SYNC(p_P4080GurMemMap)      ((GET_UINT32(p_P4080GurMemMap->rcwsr[5]) & RCW_DDR_SYNC_MASK) >> (31-29))    /**< Macro to get RCW DDR SYNC field */

/* CLOCKING CONFIGURATION (bits 224-255) */
#define RCW_PME_CLK_SEL_MASK 0x80000000
#define RCW_FM_CLK_SEL_MASK  0x00000001
#define RCW_GET_FM_CLK_SEL(p_P4080GurMemMap,fmIndex)    ((GET_UINT32(p_P4080GurMemMap->rcwsr[7]) >> (30 - (1 * fmIndex))) & RCW_FM_CLK_SEL_MASK)             /**< Macro to get RCW FM CLK SEL field */

/* PIN MULTIPLEXING CONFIGURATION (bits 352-415) */
#define RCW_EC1_MASK   0x00c00000          /**< RCW EC1 field mask */
#define RCW_EC2_MASK   0x001c0000          /**< RCW EC2 field mask */

#define RCW_GET_EC1(p_P4080GurMemMap)      ((GET_UINT32(p_P4080GurMemMap->rcwsr[11]) & RCW_EC1_MASK) >> (31-9))    /**< Macro to get RCW EC1 field */
#define RCW_GET_EC2(p_P4080GurMemMap)      ((GET_UINT32(p_P4080GurMemMap->rcwsr[11]) & RCW_EC2_MASK) >> (31-13))    /**< Macro to get RCW EC2 field */

/* PORSR2 */
#define PORSR2_DRAM_TYPE_MASK   0x20000000
#define PORSR2_DRAM_TYPE_SHIFT   29

/**************************************************************************//**
 Device Disable Register
*//***************************************************************************/
#define P4080_DEV_DISABLE_PME           0x80000000      /**< PME controller disable        */
#define P4080_DEV_DISABLE_SEC           0x40000000      /**< SEC controller disable        */
#define P4080_DEV_DISABLE_QM_BM         0x08000000      /**< QM/BM controller disable      */
#define P4080_DEV_DISABLE_FM1           0x02000000      /**< FM1 controller disable        */
#define P4080_DEV_DISABLE_FM1_TGEC      0x01000000      /**< FM1 TGEC controller disable   */
#define P4080_DEV_DISABLE_FM1_DTSEC1    0x00800000      /**< FM1 DTSEC1 controller disable */
#define P4080_DEV_DISABLE_FM1_DTSEC2    0x00400000      /**< FM1 DTSEC2 controller disable */
#define P4080_DEV_DISABLE_FM1_DTSEC3    0x00200000      /**< FM1 DTSEC3 controller disable */
#define P4080_DEV_DISABLE_FM1_DTSEC4    0x00100000      /**< FM1 DTSEC4 controller disable */
#define P4080_DEV_DISABLE_FM2           0x00020000      /**< FM2 controller disable        */
#define P4080_DEV_DISABLE_FM2_TGEC      0x00010000      /**< FM2 TGEC controller disable   */
#define P4080_DEV_DISABLE_FM2_DTSEC1    0x00008000      /**< FM2 DTSEC1 controller disable */
#define P4080_DEV_DISABLE_FM2_DTSEC2    0x00004000      /**< FM2 DTSEC2 controller disable */
#define P4080_DEV_DISABLE_FM2_DTSEC3    0x00002000      /**< FM2 DTSEC3 controller disable */
#define P4080_DEV_DISABLE_FM2_DTSEC4    0x00001000      /**< FM2 DTSEC4 controller disable */

/**************************************************************************//**
 PAMU Bypass Enable Register
*//***************************************************************************/

#define P4080_GET_PAMU_BYPASS_ID(pamuId) (0x80000000 >> pamuId)


/*--------------------------------------*/
/* Structure for the P4080 object.      */
/*--------------------------------------*/
typedef struct
{
    t_Part                  part;               /**< Common parameters for all parts */
    t_P4080GurMemMap        *p_GurMemMap;
    t_P4080RcpmMemMap       *p_RcpmMemMap;
    uint64_t                ccsrPhysBaseAddr;
    uint64_t                bmPortalsPhysBaseAddr;
    uint64_t                qmPortalsPhysBaseAddr;
    uint8_t                 serdesLanesBank2;
    uint8_t                 serdesLanesBank3;
    t_P4080DevParams        devs;
    bool                    (*f_BoardIsValidSerDesConfigurationCB) (uint8_t val);
} t_P4080;

#endif /* __P4080_H */
