/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/

#ifndef __DATA_POOL_MNG_EXT_H
#define __DATA_POOL_MNG_EXT_H

#include "std_ext.h"
#include "buf_ext.h"


/**************************************************************************//**
 @Description   Data Pools Manager parameters structure
*//***************************************************************************/
typedef struct t_DataPoolMngParam
{
    uint32_t    numOfBufferObjects;
    uint32_t    numOfFrameObjects;

} t_DataPoolMngParam;


/**************************************************************************//**
 @Description   Data Pool settings structure
*//***************************************************************************/
typedef struct t_DataPoolSettings
{
    int         partitionId;
    uint32_t    numOfBuffers;
    uint16_t    bufferSize;
    uint16_t    alignment;
    uint16_t    prefix;
    uint16_t    postfix;

} t_DataPoolSettings;


/**************************************************************************//**
 @Description   Buffer Pool Information Structure.
*//***************************************************************************/
typedef struct t_DataPoolInfo
{
    t_BufMemAccess  *p_BufMemAccess;    /**< Buffers manager access structure */
    int             memPartitionId;     /**< Memory partition ID */
    uint16_t        bufferSize;         /**< Buffer size */

} t_DataPoolInfo;


t_Handle DATA_POOL_MNG_Init(t_DataPoolMngParam *p_DataPoolMngParam);

t_Error DATA_POOL_MNG_Free(t_Handle h_DataPoolMng);

t_Error DATA_POOL_MNG_GetPoolInfo(t_Handle          h_DataPoolMng,
                                  uint32_t          poolId,
                                  t_DataPoolInfo    *p_DataPoolInfo);


uint8_t * DATA_POOL_DefaultGetBuf(t_Handle h_BufMemAccess, t_Handle *p_BufContextHandle);

t_Error DATA_POOL_DefaultPutBuf(t_Handle h_BufMemAccess, uint8_t *p_Buffer, t_Handle h_BufContext);

/* QNX Added */
t_Handle DataPoolMngGetPoolHandle(t_Handle       h_DataPoolMng,
                                         e_SysModule    module,
                                         uint32_t       id);

t_Error DataPoolMngPoolInit(t_Handle                 h_DataPoolMng,
                                   t_DataPoolSettings *p_DataPoolSettings,
								   uint32_t id);

t_Error  DataPoolMngPoolFree(t_Handle    h_DataPoolMng,
                                    e_SysModule module,
                                    uint32_t    id);



#endif /* __DATA_POOL_MNG_EXT_H */
