/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/**************************************************************************//**
 @File          device_manager_ext.h

 @Description
*//***************************************************************************/

#ifndef __DEVICE_MANAGER_EXT_H
#define __DEVICE_MANAGER_EXT_H

#include "std_ext.h"
#include "stdarg_ext.h"
#include "list_ext.h"


/**************************************************************************//**
 @Group         dev_mng_id Device Manager Application Programming Interface

 @Description   Device Manager routines, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         dev_mng_init Device Manager Initialization Unit

 @Description   Device Manager initialization routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DEVMNG_Init

 @Description   Initializes the Device Manager object.

 @Retval        Handle to Device Manager object : on success
 @Retval        NULL : on error
*//***************************************************************************/
t_Handle DEVMNG_Init(void);

/**************************************************************************//**
 @Function      DEVMNG_Free

 @Description   Frees the Device Manager object and all allocated resources.

 @Param[in]     h_DevMng - Handle to the Device Manager object.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error DEVMNG_Free(t_Handle h_DevMng);

/** @} */ /* end of dev_mng_init group */

/**************************************************************************//**
 @Group         dev_mng_runtime Device Manager Runtime Unit

 @Description   Device Manager runtime routines and definitions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    Device Manager ID related definitions and macros

 @{
*//***************************************************************************/
typedef uint32_t t_DevMngId;

#define DEV_CLASS_ANY               0xFF
#define DEV_CLASS_MASK              0xFF000000
#define DEV_CLASS_SHIFT             24

#define DEV_SUB_CLASS_ANY           0xFF
#define DEV_SUB_CLASS_MASK          0x00FF0000
#define DEV_SUB_CLASS_SHIFT         16

#define DEV_INSTANCE_ANY            0xFFFF
#define DEV_INSTANCE_MASK           0x0000FFFF

#define DEV_GET_CLASS(devMngId)     (uint8_t) ((devMngId & DEV_CLASS_MASK)     >> DEV_CLASS_SHIFT)
#define DEV_GET_SUB_CLASS(devMngId) (uint8_t) ((devMngId & DEV_SUB_CLASS_MASK) >> DEV_SUB_CLASS_SHIFT)
#define DEV_GET_INSTANCE(devMngId)  (uint16_t)((devMngId & DEV_INSTANCE_MASK))

#define DEVMNG_ID(_class, _subClass, _instance)                                                   \
     (t_DevMngId)((((uint32_t)(_class)    << DEV_CLASS_SHIFT)       & DEV_CLASS_MASK) |        \
                  (((uint32_t)(_subClass) << DEV_SUB_CLASS_SHIFT)   & DEV_SUB_CLASS_MASK) |    \
                  (((uint32_t)(_instance)                           & DEV_INSTANCE_MASK)))

/* @} */

/**************************************************************************//**
@Description    Device manager class identifiers

                Describes the protocol supported by the device
*//***************************************************************************/
typedef enum e_DeviceClass
{
    e_DEV_CLASS_ETHERNET,
    e_DEV_CLASS_ATM_AAL0,
    e_DEV_CLASS_ATM_AAL5,
    e_DEV_CLASS_ATM_AAL1,
    e_DEV_CLASS_ATM_AAL2_CID,
    e_DEV_CLASS_UTOPIA_PORT,
    e_DEV_CLASS_POS,
    e_DEV_CLASS_HDLC,
    e_DEV_CLASS_TRANS,
    e_DEV_CLASS_PPP,
    e_DEV_CLASS_L2_SWITCH,
    e_DEV_CLASS_RAW,
    e_DEV_CLASS_SERIAL,
    e_DEV_CLASS_RTC,
    e_DEV_CLASS_MEM,
    e_DEV_CLASS_BUS_CTRL,
    e_DEV_CLASS_POLICER,
    e_DEV_CLASS_FILTER,
    e_DEV_CLASS_LOOKUP_TABLE,
    e_DEV_CLASS_IW_FLOW,
    e_DEV_CLASS_IW_FLOW_OPS,
    e_DEV_CLASS_IW_IP_REASSEMBLY,
    e_DEV_CLASS_IW_DEV,
    e_DEV_CLASS_LINK,
    e_DEV_CLASS_BUNDLE,
    e_DEV_CLASS_QUEUE_MANAGER,
    e_DEV_CLASS_PTP,
    e_DEV_CLASS_SS7,
    e_DEV_CLASS_IPHC_HC,
    e_DEV_CLASS_IPHC_HDEC,
    e_DEV_CLASS_IPSEC

} e_DeviceClass;

/**************************************************************************//**
@Description    Device manager sub-class identifiers.

                Describes the represented controller.
*//***************************************************************************/
typedef enum e_DeviceSubClass
{
    e_DEV_SUB_CLASS_UCC,
    e_DEV_SUB_CLASS_MCC,
    e_DEV_SUB_CLASS_MCC_CH,
    e_DEV_SUB_CLASS_QMC,
    e_DEV_SUB_CLASS_QMC_CH,
    e_DEV_SUB_CLASS_TDM,
    e_DEV_SUB_CLASS_TDM_RX_FRAME,
    e_DEV_SUB_CLASS_TDM_TX_FRAME,
    e_DEV_SUB_CLASS_MTC,
    e_DEV_SUB_CLASS_ETSEC,
    e_DEV_SUB_CLASS_SRIO,
    e_DEV_SUB_CLASS_PCI,
    e_DEV_SUB_CLASS_PCIE,
    e_DEV_SUB_CLASS_ESDHC,
    e_DEV_SUB_CLASS_SPI,
    e_DEV_SUB_CLASS_DMA,
    e_DEV_SUB_CLASS_DMA_CH,
    e_DEV_SUB_CLASS_DUART,
    e_DEV_SUB_CLASS_PPPoHT,
    e_DEV_SUB_CLASS_PPPoHT_LINK,
    e_DEV_SUB_CLASS_EOS_LINK,
    e_DEV_SUB_CLASS_PPPOS_LINK,
    e_DEV_SUB_CLASS_PPPoHT_BUNDLE,
    e_DEV_SUB_CLASS_IW_PPPoHT,
    e_DEV_SUB_CLASS_IW_PPPoHT_LINK,
    e_DEV_SUB_CLASS_IW_PPPoHT_BUNDLE,
    e_DEV_SUB_CLASS_QE,
    e_DEV_SUB_CLASS_QE_IW,
    e_DEV_SUB_CLASS_QE_VP,
    e_DEV_SUB_CLASS_QE_IMA,
    e_DEV_SUB_CLASS_QE_IW_MCAST,
    e_DEV_SUB_CLASS_QE_GTIMERS,
    e_DEV_SUB_CLASS_GTIMERS,
    e_DEV_SUB_CLASS_RTC83,
    /* External vendors */
    e_DEV_SUB_CLASS_DS1374,
    e_DEV_SUB_CLASS_AT24C01A,
    e_DEV_SUB_CLASS_M24256,
    e_DEV_SUB_CLASS_M25PX,
    e_DEV_SUB_CLASS_USB_HOST,
    e_DEV_SUB_CLASS_USB_GADGET

} e_DeviceSubClass;


/**************************************************************************//**
@Description   Device types

               Device type determines the API routines needed to register a
               device to the Device Manager.
*//***************************************************************************/
typedef enum e_DeviceType
{
    e_DEV_TYPE_CTRL = 0,
    e_DEV_TYPE_CHAR,
    e_DEV_TYPE_BLOCK,
    e_DEV_TYPE_NET
} e_DeviceType;

/**************************************************************************//**
@Description   Device status, reported to the application upon device
               registration or removal using device using user-defined callback.
*//***************************************************************************/
typedef enum e_DeviceStatus
{
    e_DEV_STATUS_DISCONNECTED = 0,
    e_DEV_STATUS_CONNECTED
} e_DeviceStatus;

/**************************************************************************//**
@Description    Device events.

                Reported to the application via the f_AppEventCb
*//***************************************************************************/
/* General events */
#define DEV_EVENT_ERROR                     0x00000000
/* Data events */
#define DEV_EVENT_DATA_RECEIVED             0x00000010
#define DEV_EVENT_DATA_TRANSMITTED          0x00000011
#define DEV_EVENT_RECEIVE_OVERRUN           0x00000012
#define DEV_EVENT_TRANSMIT_UNDERRUN         0x00000013
/* RTC events */
#define DEV_EVENT_RTC_PERIODIC_SEC          0x00000020
#define DEV_EVENT_RTC_ALARM                 0x00000021

#define DEV_EVENT_DEVICE_SPECIFIC           0x10000000


/**************************************************************************//**
@Description   I/O request operation types.
*//***************************************************************************/
typedef enum e_DeviceIoReqType
{
    e_DEVMNG_IO_REQ_WRITE = 0,
    e_DEVMNG_IO_REQ_READ,
    e_DEVMNG_IO_REQ_CONTROL
} e_DeviceIoReqType;

/**************************************************************************//**
@Description   I/O request current state.
*//***************************************************************************/
typedef enum e_DeviceIoReqState
{
    e_DEVMNG_IO_REQ_QUEUED = 0,
    e_DEVMNG_IO_REQ_ONGOING,
    e_DEVMNG_IO_REQ_DONE
} e_DeviceIoReqState;

/**************************************************************************//**
@Description   Device add/remove callback prototype
*//***************************************************************************/
typedef void (f_DeviceAddRemove) (t_Handle ,e_DeviceType , e_DeviceStatus);

/**************************************************************************//**
 @Collection    Device Manager Application Callback related definitions and macros

 @{
*//***************************************************************************/

/**************************************************************************//**
@Description    Sets application handle for the specified device, passed to
                the application via application callback routines.
*//***************************************************************************/
#define DEV_SET_APP_HANDLE(p_DeviceDesc, _h_App)                \
    do {                                                        \
        switch (p_DeviceDesc->devType)                          \
        {                                                       \
            case e_DEV_TYPE_CTRL:                               \
                p_DeviceDesc->p_CtrlDevice->h_App = _h_App;     \
                break;                                          \
            case e_DEV_TYPE_CHAR:                               \
                p_DeviceDesc->p_CharDevice->h_App = _h_App;     \
                break;                                          \
            case e_DEV_TYPE_BLOCK:                              \
                p_DeviceDesc->p_BlockDevice->h_App = _h_App;    \
                break;                                          \
            case e_DEV_TYPE_NET:                                \
                p_DeviceDesc->p_NetDevice->h_App = _h_App;      \
                break;                                          \
            default:                                            \
                ASSERT_COND(FALSE);                             \
        }                                                       \
    } while (0)

/**************************************************************************//**
@Description    Sets callback routine for reporting driver events to the
                application.
*//***************************************************************************/
#define DEV_SET_APP_EVENT_CB(p_DeviceDesc, f_Cb)                    \
    do {                                                            \
        switch (p_DeviceDesc->devType)                              \
        {                                                           \
            case e_DEV_TYPE_CTRL:                                   \
                p_DeviceDesc->p_CtrlDevice->f_AppEventCb = f_Cb;    \
                break;                                              \
            case e_DEV_TYPE_CHAR:                                   \
                p_DeviceDesc->p_CharDevice->f_AppEventCb = f_Cb;    \
                break;                                              \
            case e_DEV_TYPE_BLOCK:                                  \
                p_DeviceDesc->p_BlockDevice->f_AppEventCb = f_Cb;   \
                break;                                              \
            case e_DEV_TYPE_NET:                                    \
                p_DeviceDesc->p_NetDevice->f_AppEventCb = f_Cb;     \
                break;                                              \
            default:                                                \
                ASSERT_COND(FALSE);                                 \
        }                                                           \
    } while (0)

/**************************************************************************//**
@Description    Sets application callback routine for message reception for
                specified network device.
*//***************************************************************************/
#define DEV_SET_APP_RX_CB(p_DeviceDesc, f_Cb)                   \
    do {                                                        \
        ASSERT_COND(p_DeviceDesc->devType == e_DEV_TYPE_NET);   \
        p_DeviceDesc->p_NetDevice->f_AppRxCb = f_Cb;     \
    } while (0)

/**************************************************************************//**
@Description    Sets application callback routine for trasnmitted frames
                confirmation for specified network device.
*//***************************************************************************/
#define DEV_SET_APP_TX_CONF_CB(p_DeviceDesc, f_Cb)              \
    do {                                                        \
        ASSERT_COND(p_DeviceDesc->devType == e_DEV_TYPE_NET);   \
        p_DeviceDesc->p_NetDevice->f_AppTxConfCb = f_Cb; \
    } while (0)

/* @} */

/**************************************************************************//**
@Description    Character device descriptor.

                Character devices are byte-oriented devices that can be read
                from or written to.
*//***************************************************************************/
typedef struct t_CharDevice
{
    t_Handle    h_Device;
                /**< Private handle for the use of the device driver */

    /* Application interface.
       The application can access the underlying device through these routines. */

    t_Error     (*f_IoCtl)          (t_Handle       h_Device,
                                     uint32_t       ioCtlCmd,
                                     void           *p_RetVal, ... /* args */);
                /**< Passes control commands to the device driver */

    int         (*f_DeviceWrite)    (t_Handle       h_Device,
                                     uint8_t        *p_Buffer,
                                     uint32_t       count);
                /**< Writes 'count' bytes from 'p_Buffer' to the device,
                     returns how many bytes were actually written */

    int         (*f_DeviceRead)     (t_Handle       h_Device,
                                     uint8_t        *p_Buffer,
                                     uint32_t       count);
                /**< Reads 'count' bytes from the device into 'p_Buffer'
                     returns how many bytes were actually read into 'p_Buffer' */

    /* Device driver interface.
       The device can report events to the application through these routines. */

    t_Handle    h_App;
                /**< A handle supplied by the application to enable
                     information sharing with the device driver */

    t_Error     (*f_AppEventCb)     (t_Handle       h_App,
                                     uint32_t       devEvents,
                                     void           *p_Arg);
                /**< A callback routine supplied by the application to be
                     invoked by the device driver, to report events upwards
                     to the application */
    t_List      node;
                /**< Linked list object */
} t_CharDevice;


typedef struct t_DeviceIoReq t_DeviceIoReq;

/**************************************************************************//**
@Description    I/O request descriptor.

                Used to pass a request for I/O operation from the application
                to the device. Possible to pass a linked list of I/O requests.
*//***************************************************************************/
struct t_DeviceIoReq
{
    t_Handle            h_Private;
                        /**< Application-defined private data */

    uint64_t            address;
                        /**< Target address for the read/write operation */

    e_DeviceIoReqType   requestType;
                        /**< The operation to preform - read, write or control */

    void*               requestParam;
                        /**< The operation parameters to preform */

    uint8_t             *p_Buff;
                        /**< Target buffer that holds the data to write or
                             will contain the read data */

    uint32_t            requestLength;
                        /**< The requested length for the current operation */

    uint32_t            actualLength;
                        /**< The number of bytes actually read or written by this operation */

    uint32_t            timeout;
                        /**< Timeout value for operation completion */

    e_DeviceIoReqState  currState;
                        /**< Current operation state */

    t_Error             completionStatus;
                        /**< Request status upon completion - success / error */

    void                (* f_IoComplete)(t_Handle       h_App,
                                         t_DeviceIoReq  *p_IoReq);
                        /**< Application callback routine to be invoked by the
                             device driver upon completion of the request */
    t_List              node;
                        /**< Linked list object */
};


/**************************************************************************//**
@Description    Block device descriptor.

                Block devices are devices that are block oriented, meaning they
                handle data in the form of blocks and allow seeking of data
                within the device.
*//***************************************************************************/
typedef struct t_BlockDevice
{
    t_Handle    h_Device;
                /**< A private handle for the use of the device driver */

    /* Application interface.
       The application can access the underlying device through these routines. */

    t_Error     (*f_IoCtl)          (t_Handle       h_Device,
                                     uint32_t       ioCtlCmd,
                                     void           *p_RetVal, ... /* args */);
                /**< Passes control commands to the device driver */

    t_Error     (*f_EnqueueRequest) (t_Handle       h_Device,
                                     t_DeviceIoReq  *p_IoReq);
                /**< Passes a I/O operation request (read, write or control)
                     from the application to the device driver */

    t_Error     (*f_DequeueRequest) (t_Handle       h_Device,
                                     t_DeviceIoReq  *p_IoReq);
                /**< The application removes an enqueued I/O request */

    /* Device driver interface.
       The device can report events to the application through these routines. */

    t_Handle    h_App;
                /**< A handle supplied by the application to enable information
                     sharing with the device driver */

    t_Error     (*f_AppEventCb)     (t_Handle       h_App,
                                     uint32_t       devEvents,
                                     void           *p_Arg);
                /**< A callback function supplied by the application to be called
                     by the device driver, to report events upwards to the application */

    t_List      node;
                /**< Linked list object */
} t_BlockDevice;


/**************************************************************************//**
@Description    Network device descriptor.

                Network devices support receive/transmit operations of packets.
*//***************************************************************************/
typedef struct t_NetDevice
{
    t_Handle            h_Device;
                        /**< A private handle for the use of the device driver */

    /* Application interface.
       The application can access the underlying device through these routines. */

    t_Error             (*f_IoCtl)          (t_Handle   h_Device,
                                             uint32_t   ioCtlCmd,
                                             void       *p_RetVal, ... /* args */);
                        /**< Passes control commands to the device driver */

    t_Error             (*f_Tx)             (t_Handle   h_Device,
                                             uint32_t   queueId,
                                             t_Handle   h_Frame);
                        /**< Application routine for frame transmission */

    void                (*f_TxConf)         (t_Handle   h_Device,
                                             uint32_t   queueId);
                        /**< Application routine for Tx frames confirmation */

    t_Error             (*f_Rx)             (t_Handle   h_Device,
                                             uint32_t   queueId);
                        /**< Application routine to invoke frame reception */

    t_Error             (*f_RxConf)         (t_Handle   h_Device,
                                             uint32_t   queueId,
                                             t_Handle   h_Frame);
                        /**< Application routine for reception confirmation */

    /* Device driver interface.
       The device can report events to the application through these routines. */

    t_Handle            h_App;
                        /**< A handle supplied by the application to enable
                             information sharing with the device driver */

    e_RxStoreResponse   (*f_AppRxCb)        (t_Handle   h_App,
                                             uint32_t   queueId,
                                             t_Handle   h_Frame);
                        /**< Application callback routine for frame reception */

    void                (*f_AppTxConfCb)    (t_Handle   h_App,
                                             uint32_t   queueId,
                                             t_Handle   h_Frame);
                        /**< Application callback for confirming of transmitted frames */

    t_Error             (*f_AppEventCb)     (t_Handle   h_App,
                                             uint32_t   devEvents,
                                             void       *p_Arg);
                        /**< A callback function supplied by the application to
                             be called by the device driver, to report events
                             upwards to the application */

    t_List              node;
                        /**< Linked list object */
} t_NetDevice;


/**************************************************************************//**
@Description    Control device descriptor

                Control device can only receive control commands from the
                application
*//***************************************************************************/
typedef struct t_CtrlDevice
{
    t_Handle    h_Device;                                                   /**< A private data for the use of the device driver */

    /* Application interface.
       The application can access the underlying device through these routines. */

    t_Error     (*f_IoCtl)      (t_Handle   h_Device,
                                 uint32_t   ioCtlCmd,
                                 void       *p_RetVal, ... /* args */);
                /**< Passes control commands to the device driver */

    /* Device driver interface.
       The device can report events to the application through these routines. */

    t_Handle    h_App;
                /**< A handle supplied by the application to enable information
                     sharing with the device driver */

    t_Error     (*f_AppEventCb) (t_Handle   h_App,
                                 uint32_t   devEvents,
                                 void       *p_Arg);
                /**< A callback function supplied by the application to be
                     called by the device driver, to report events upwards to
                     the application */

    t_List      node;
                /**< Linked list object */
} t_CtrlDevice;


/**************************************************************************//**
@Description    Device descriptor.
*//***************************************************************************/
typedef struct t_DeviceDesc
{
    t_DevMngId          devMngId;   /**< Device Manager ID */

    e_DeviceType        devType;    /**< Identifies the device as character, block,
                                         network or control device; Using this
                                         information we can interpret the next field. */
    union
    {
        t_CharDevice    *p_CharDevice;
        t_BlockDevice   *p_BlockDevice;
        t_NetDevice     *p_NetDevice;
        t_CtrlDevice    *p_CtrlDevice;
    };                              /**< Union that holds character, block, network or
                                         control device specific API routines of this device */

    t_List              node;       /**< Linked list object */

} t_DeviceDesc;



/**************************************************************************//**
 @Group         dev_mng_runtime_dev_drivers_api  Device Manager Runtime - Device Drivers Level API

 @Description   Device Manager routines, definitions and enums toward the device
                drivers layer.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DEVMNG_RegisterCharDevice

 @Description   Registers a new character device.

                The user can pass a instanceId to force a specific devMngId. The
                Device Manager will verify that it is unuque and no duplication
                of id will occur.
                If an "ANY" value is passed as the instanceId, the Device Manager
                will assign the next available instanceId for the specified classId
                and subClassId pair.


 @Param[in]     h_DevMng     - Handle to the device manager object.
 @Param[in]     p_CharDevice - Pointer to character device descriptor, holds
                               pointer to required API routines.
 @Param[in]     classId      - The class of the device being added.
 @Param[in]     subClassId   - The sub class of the device being added.
 @Param[in]     instanceId   - The requested instance id for the device or "ANY"
                               indication

 @Retval        Pointer to a descriptor of the registered device : on success
 @Retval        NULL : otherwise
*//***************************************************************************/
t_DeviceDesc * DEVMNG_RegisterCharDevice(t_Handle      h_DevMng,
                                         t_CharDevice  *p_CharDevice,
                                         uint8_t       classId,
                                         uint8_t       subClassId,
                                         uint16_t      instanceId);

/**************************************************************************//**
 @Function      DEVMNG_UnregisterCharDevice

 @Description   Unregisters a the specified character device.


 @Param[in]     h_DevMng     - Handle to the device manager object.
 @Param[in]     p_DeviceDesc - Descriptor of the device to remove, returned during
                               device registration.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error DEVMNG_UnregisterCharDevice  (t_Handle      h_DevMng,
                                      t_DeviceDesc  *p_DeviceDesc);

/**************************************************************************//**
 @Function      DEVMNG_RegisterBlockDevice

 @Description   Registers a new block device.

                The user can pass a instanceId to force a specific devMngId. The
                Device Manager will verify that it is unuque and no duplication
                of id will occur.
                If an "ANY" value is passed as the instanceId, the Device Manager
                will assign the next available instanceId for the specified classId
                and subClassId pair.


 @Param[in]     h_DevMng      - Handle to the device manager object.
 @Param[in]     p_BlockDevice - Pointer to block device descriptor, holds
                                pointer to required API routines.
 @Param[in]     classId       - The class of the device being added.
 @Param[in]     subClassId    - The sub class of the device being added.
 @Param[in]     instanceId    - The requested instance id for the device or "ANY"
                                indication

 @Retval        Pointer to a descriptor of the registered device : on success
 @Retval        NULL : otherwise
*//***************************************************************************/
t_DeviceDesc* DEVMNG_RegisterBlockDevice (t_Handle        h_DevMng,
                                          t_BlockDevice   *p_BlockDevice,
                                          uint8_t         classId,
                                          uint8_t         subClassId,
                                          uint16_t        instanceId);

/**************************************************************************//**
 @Function      DEVMNG_UnregisterBlockDevice

 @Description   Unregisters a the specified block device.


 @Param[in]     h_DevMng     - Handle to the device manager object.
 @Param[in]     p_DeviceDesc - Descriptor of the device to remove, returned during
                               device registration.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error DEVMNG_UnregisterBlockDevice (t_Handle      h_DevMng,
                                      t_DeviceDesc  *p_DeviceDesc);

/**************************************************************************//**
 @Function      DEVMNG_RegisterNetDevice

 @Description   Registers a new network device.

                The user can pass a instanceId to force a specific devMngId. The
                Device Manager will verify that it is unuque and no duplication
                of id will occur.
                If an "ANY" value is passed as the instanceId, the Device Manager
                will assign the next available instanceId for the specified classId
                and subClassId pair.


 @Param[in]     h_DevMng    - Handle to the device manager object.
 @Param[in]     p_NetDevice - Pointer to network device descriptor, holds
                              pointer to required API routines.
 @Param[in]     classId     - The class of the device being added.
 @Param[in]     subClassId  - The sub class of the device being added.
 @Param[in]     instanceId  - The requested instance id for the device or "ANY"
                              indication

 @Retval        Pointer to a descriptor of the registered device : on success
 @Retval        NULL : otherwise
*//***************************************************************************/
t_DeviceDesc* DEVMNG_RegisterNetDevice (t_Handle      h_DevMng,
                                        t_NetDevice   *p_NetDevice,
                                        uint8_t       classId,
                                        uint8_t       subClassId,
                                        uint16_t      instanceId);

/**************************************************************************//**
 @Function      DEVMNG_UnregisterNetDevice

 @Description   Unregisters a the specified network device.


 @Param[in]     h_DevMng     - Handle to the device manager object.
 @Param[in]     p_DeviceDesc - Descriptor of the device to remove, returned during
                               device registration.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error DEVMNG_UnregisterNetDevice   (t_Handle      h_DevMng,
                                      t_DeviceDesc  *p_DeviceDesc);

/**************************************************************************//**
 @Function      DEVMNG_RegisterCtrlDevice

 @Description   Registers a new control device.

                The user can pass a instanceId to force a specific devMngId. The
                Device Manager will verify that it is unuque and no duplication
                of id will occur.
                If an "ANY" value is passed as the instanceId, the Device Manager
                will assign the next available instanceId for the specified classId
                and subClassId pair.


 @Param[in]     h_DevMng     - Handle to the device manager object.
 @Param[in]     p_CtrlDevice - Pointer to control device descriptor, holds
                               pointer to required API routines.
 @Param[in]     classId      - The class of the device being added.
 @Param[in]     subClassId   - The sub class of the device being added.
 @Param[in]     instanceId   - The requested instance id for the device or "ANY"
                               indication

 @Retval        Pointer to a descriptor of the registered device : on success
 @Retval        NULL : otherwise
*//***************************************************************************/
t_DeviceDesc* DEVMNG_RegisterCtrlDevice (t_Handle      h_DevMng,
                                         t_CtrlDevice  *p_CtrlDevice,
                                         uint8_t       classId,
                                         uint8_t       subClassId,
                                         uint16_t      instanceId);

/**************************************************************************//**
 @Function      DEVMNG_UnregisterCtrlDevice

 @Description   Unregisters a the specified control device.


 @Param[in]     h_DevMng     - Handle to the device manager object.
 @Param[in]     p_DeviceDesc - Descriptor of the device to remove, returned during
                               device registration.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error DEVMNG_UnregisterCtrlDevice  (t_Handle      h_DevMng,
                                      t_DeviceDesc  *p_DeviceDesc);


/** @} */ /* end of dev_mng_runtime_dev_drivers_api group */

/**************************************************************************//**
 @Group         dev_mng_runtime_app_api  Device Manager Runtime - Application Level API

 @Description   Device Manager routines, definitions and enums towards the
                application layer.

 @{
*//***************************************************************************/


#define DEVMNG_DEVICE_DESC_ITEM(p_List)     LIST_OBJECT(p_List, t_DeviceDesc, node)


#define DEVMNG_FOR_EACH_DEVICE(p_List, p_DevDesc) \
    LIST_FOR_EACH_OBJECT(p_DevDesc, t_DeviceDesc, p_List, node)


/**************************************************************************//**
 @Function      DEVMNG_GetDevice

 @Description   Returns a pointer to the device descriptor that matches the
                received id.

                The user may pass a specific device id or a device id that
                contains "ANY" in its fields.
                If the user passed a specific device id, this routine will
                return a device desciptor of a device that exactly matches
                the specified id.
                If the user passed a device id that contains "ANY" in its
                fields, then a group of devices may match the specified id.
                In this case this routine will return a device desciptor of
                the first device from the specified group.

 @Param[in]     h_DevMng - Handle to the device manager object.
 @Param[in]     devMngId - Device manager device id, can include "ANY" fields.

 @Retval        Descriptor of the requested device : on success
 @Retval        NULL : otherwise
*//***************************************************************************/
t_DeviceDesc * DEVMNG_GetDevice(t_Handle h_DevMng, t_DevMngId devMngId);

/**************************************************************************//**
 @Function      DEVMNG_GetDevicesList

 @Description   Returns a list of device descriptors that match the received id.

                In this routine, the user can request a group of devices. A group
                can be chosen by setting one of the fields that construct the
                device id to the value "ANY".
                For example, to receive all instances of devices of a certain
                class and sub class, instance field should be passed as "ANY".
                Each node of the list is of the type t_DeviceDesc.

 @Param[in]     h_DevMng - Handle to the device manager object.
 @Param[in]     devMngId - Device manager device id, can include "ANY" fields.
 @Param[out]    p_Count  - Returns the number of devices in the list.

 @Retval        Pointer to a list of devices : on success
 @Retval        NULL : otherwise
*//***************************************************************************/
t_List * DEVMNG_GetDevicesList(t_Handle h_DevMng, t_DevMngId devMngId, uint32_t *p_Count);

/**************************************************************************//**
 @Function      DEVMNG_FreeDevicesList

 @Description   Frees a list of device descriptors that was retrieved using
                the DEVMNG_GetDevicesList() routine.

 @Param[in]     h_DevMng        - Handle to the device manager object.
 @Param[in]     p_DevicesList   - Pointer to the list of devices

 @Return        None.
*//***************************************************************************/
void DEVMNG_FreeDevicesList(t_Handle h_DevMng, t_List *p_DevicesList);

/**************************************************************************//**
 @Function      DEVMNG_RegisterDeviceStatusNotificationCB

 @Description   Registers a function callback for a client.

                The subClassId can be passed as "ANY", in that case the callback
                will be invoked for each device with the specified classId.


 @Param[in]     h_DevMng         - Handle to the device manager object.
 @Param[in]     classId          - The class id of the device being added.
 @Param[in]     subClassId       - The sub class id of the device being added,
                                   can be passed as "ANY".
 @Param[in]     p_DevAddRemoveCB - Application callback routine to invoke upon
                                   adding or removal of a device.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise
*//***************************************************************************/
t_Error DEVMNG_RegisterDeviceStatusNotificationCB
                                (t_Handle            h_DevMng,
                                 uint8_t             classId,
                                 uint8_t             subClassId,
                                 f_DeviceAddRemove   *p_DevAddRemoveCB);

/**************************************************************************//**
 @Function      DEVMNG_UnregisterDeviceStatusNotificationCB

 @Description   Removes the 1st specified callback from the callback list
 .

 @Param[in]     h_DevMng            - Handle to the device manager object.
 @Param[in]     p_DeviceAddRemoveCB - Application callback routine to be removed.

 @Retval        E_OK       : on success
 @Retval        Error code : otherwise .
*//***************************************************************************/
t_Error DEVMNG_UnregisterDeviceStatusNotificationCB
                                (t_Handle            h_DevMng,
                                 f_DeviceAddRemove   *p_DeviceAddRemoveCB);

#ifdef DEBUG
/**************************************************************************//**
 @Function      DEVMNG_PrintDevices

 @Description   Prints a list of all registered devices.


 @Param[in]     h_DevMng - Handle to the device manager object.

 @Retval        None.
*//***************************************************************************/
void DEVMNG_PrintDevices (t_Handle h_DevMng);

#endif

/** @} */ /* end of dev_mng_runtime_app_api group */

/** @} */ /* end of dev_mng_runtime group */

/** @} */ /* end of dev_mng_id group */


#endif /* __DEVICE_MANAGER_EXT_H */
