/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/**************************************************************************//**
 @File          ioctl_ext.h

 @Description
*//***************************************************************************/

#ifndef __IOCTL_EXT_H
#define __IOCTL_EXT_H

#include "std_ext.h"
#include "stdarg_ext.h"


/**************************************************************************//**
 @Group         ioctl_id Device I/O Control (IOCTL) Definitions

 @Description   Device I/O Control (IOCTL) Definitions.

 @{
*//***************************************************************************/


#define IOC_CODE(_num, ...)         ((uint32_t)(((_num) << 2) | 0x00000001))

#define IOC_FUNC(_func, ...)        ((uint32_t)(_func))

#define IS_IOC_CODE(_ioCtlCmd)      ((_ioCtlCmd) & 0x00000001)

#define IS_IOC_FUNC(_ioCtlCmd)      (!IS_IOC_CODE(_ioCtlCmd))


#define IOC_BASE_GEN                        0x00000001

#define IOC_BASE_CTRL_DEV                   0x00000100
#define IOC_BASE_CHAR_DEV                   0x00000200
#define IOC_BASE_BLOCK_DEV                  0x00000300
#define IOC_BASE_NET_DEV                    0x00000400

#define IOC_BASE_ETH                        0x00000500
#define IOC_BASE_RTC                        0x00000600
#define IOC_BASE_SERIAL                     0x00000700
#define IOC_BASE_MEMDEV                     0x00000800
#define IOC_BASE_RIO                        0x00000900
#define IOC_BASE_ARBITER                    0x00000A00
#define IOC_BASE_ESDHC                      0x00000B00
#define IOC_BASE_UART                       0x00000C00
#define IOC_BASE_ATM                        0x00000D00
#define IOC_BASE_SPI                        0x00000E00
#define IOC_BASE_PCI                        0x00000F00
#define IOC_BASE_DMA                        0x00001000


#define IOC_BASE_PRIVATE                    0x00010000


/* General device IOCTL commands */
#define IOC_DEV_ENABLE                      IOC_CODE(IOC_BASE_GEN + 1, t_Error, e_CommMode)
#define IOC_DEV_DISABLE                     IOC_CODE(IOC_BASE_GEN + 2, t_Error, e_CommMode)
#define IOC_DEV_CREATE                      IOC_CODE(IOC_BASE_GEN + 3, t_Error, uint32_t, void*)
#define IOC_DEV_FREE                        IOC_CODE(IOC_BASE_GEN + 4, t_Error, uint32_t)
#define IOC_DEV_GET_NUM_OBJECTS             IOC_CODE(IOC_BASE_GEN + 5, uint32_t)
#define IOC_DEV_GET_COMM_MODE               IOC_CODE(IOC_BASE_GEN + 6, e_CommMode)
#define IOC_DEV_GET_HANDLE                  IOC_CODE(IOC_BASE_GEN + 7, t_Handle)

/* General network device IOCTL commands */
#define IOC_NETDEV_GET_NUM_OF_TX_QUEUES     IOC_CODE(IOC_BASE_NET_DEV + 1, uint8_t)
#define IOC_NETDEV_GET_NUM_OF_RX_QUEUES     IOC_CODE(IOC_BASE_NET_DEV + 2, uint8_t)
#define IOC_NETDEV_TX_CONFIRM               IOC_CODE(IOC_BASE_NET_DEV + 4, void, uint8_t)

/* General Ethernet IOCTL commands */
#define IOC_ETH_GET_MODE                    IOC_CODE(IOC_BASE_ETH + 1, e_EnetMode)
#define IOC_ETH_GET_MTU                     IOC_CODE(IOC_BASE_ETH + 2, uint16_t)
#define IOC_ETH_GET_LINK_STATE              IOC_CODE(IOC_BASE_ETH + 3, bool)

/* General ATM IOCTL commands */
#define IOC_ATM_AAL2_CID_TYPE               IOC_CODE(IOC_BASE_ATM + 1, e_UccAtmAal2Type)
#define IOC_ATM_AAL2_CPS_MAX_FRAME          IOC_CODE(IOC_BASE_ATM + 2, uint8_t)
#define IOC_ATM_AAL5_GET_VPI                IOC_CODE(IOC_BASE_ATM + 3, uint16_t)
#define IOC_ATM_AAL5_HIER_QUEUES            IOC_CODE(IOC_BASE_ATM + 4, uint8_t)
#define IOC_ATM_AAL5_HIER_ADD_QUEUE         IOC_CODE(IOC_BASE_ATM + 5, t_Error)
#define IOC_ATM_AAL5_HIER_REMOVE_QUEUE      IOC_CODE(IOC_BASE_ATM + 6, t_Handle)
#define IOC_ATM_AAL5_HIER_FRAME_BASED_GROUPS IOC_CODE(IOC_BASE_ATM + 7, uint8_t*)
#define IOC_ATM_AAL1_STRUCT_FORMAT          IOC_CODE(IOC_BASE_ATM + 8, bool)
#define IOC_ATM_CHECK_POLICER               IOC_CODE(IOC_BASE_ATM + 9, bool)
#define IOC_ATM_CHECK_POLICER_ERRORS        IOC_CODE(IOC_BASE_ATM + 10, bool)
#define IOC_ATM_PORT_ENABLE                 IOC_CODE(IOC_BASE_ATM + 11, t_Error)
#define IOC_ATM_PORT_DISABLE                IOC_CODE(IOC_BASE_ATM + 12, t_Error)

/* General RTC IOCTL commands */
#define IOC_RTC_GET_TIME                    IOC_CODE(IOC_BASE_RTC + 1, t_Error, uint32_t*)
#define IOC_RTC_SET_TIME                    IOC_CODE(IOC_BASE_RTC + 2, t_Error, uint32_t)
#define IOC_RTC_SET_ALARM                   IOC_CODE(IOC_BASE_RTC + 3, t_Error, uint32_t)
#define IOC_RTC_START                       IOC_CODE(IOC_BASE_RTC + 4, t_Error)
#define IOC_RTC_STOP                        IOC_CODE(IOC_BASE_RTC + 5, t_Error)

/* General Serial IOCTL commands */
#define IOC_SERIAL_SET_BAUD_RATE            IOC_CODE(IOC_BASE_SERIAL + 1, t_Error, uint32_t)
#define IOC_SERIAL_TX_CONF                  IOC_CODE(IOC_BASE_SERIAL + 2, t_Error, uint32_t)

/* General ESDHC IOCTL commands */
#define IOC_SDHC_SET_BUS_WIDTH              IOC_CODE(IOC_BASE_ESDHC + 1, t_Error, e_EsdhcBusWidth)
#define IOC_SDHC_SD_HIGH_SPEED_EN           IOC_CODE(IOC_BASE_ESDHC + 2, t_Error, uint32_t)
#define IOC_SDHC_SD_LOW_SPEED_EN            IOC_CODE(IOC_BASE_ESDHC + 3, t_Error, uint32_t)
#define IOC_SDHC_ABORT                      IOC_CODE(IOC_BASE_ESDHC + 4, t_Error, uint32_t)
#define IOC_SDHC_GET_SD_CARD_DETAILS        IOC_CODE(IOC_BASE_ESDHC + 5, t_Error, uint32_t)
#define IOC_SDHC_GET_MMC_CARD_DETAILS       IOC_CODE(IOC_BASE_ESDHC + 6, t_Error, uint32_t)
#define IOC_SDHC_RESET                      IOC_CODE(IOC_BASE_ESDHC + 7, t_Error, uint32_t)
#define IOC_SDHC_IS_READY_FOR_TRANSFER      IOC_CODE(IOC_BASE_ESDHC + 8, bool)
#define IOC_SDHC_IS_CARD_CONNECTED          IOC_CODE(IOC_BASE_ESDHC + 9, bool)

/* General Memory device IOCTL commands */
#define IOC_MEMDEV_GET_SIZE                 IOC_CODE(IOC_BASE_MEMDEV + 1, uint32_t)

/* General RIO IOCTL commands */
#define IOC_RIO_DOORBELL_TX                 IOC_CODE(IOC_BASE_RIO + 1, t_Error, uint16_t, e_RioTransFlowLevel)

/* General SPI IOCTL commands */
#define IOC_SPI_MODIFY_CHAR_LENGTH          IOC_CODE(IOC_BASE_SPI + 1, t_Error)

/* General PCI IOCTL commands */
#define IOC_PCI_CHANGE_POWER_STATE          IOC_CODE(IOC_BASE_PCI + 1, t_Error)

/* General DMA IOCTL commands */
#define IOC_DMA_TRANSFER                    IOC_CODE(IOC_BASE_DMA + 1, t_Error, t_DmaTransferParams*)
#define IOC_DMA_TRANSFER_HALT               IOC_CODE(IOC_BASE_DMA + 2, t_Error)
#define IOC_DMA_TRANSFER_ABORT              IOC_CODE(IOC_BASE_DMA + 3, t_Error)
#define IOC_DMA_TRANSFER_RESUME             IOC_CODE(IOC_BASE_DMA + 4, t_Error)
#define IOC_DMA_ALLOC_CHANNEL               IOC_CODE(IOC_BASE_DMA + 5, t_Error, e_DmaBwcSize, int32_t)
#define IOC_DMA_FREE_CHANNEL                IOC_CODE(IOC_BASE_DMA + 6, t_Error, e_DmaBwcSize, int32_t)
#define IOC_DMA_CH_IS_BUSY                  IOC_CODE(IOC_BASE_DMA + 6, bool)

/* General ARBITER IOCTL commands */
#define IOC_ARBITER_SET_DATA_TIMEOUT        IOC_CODE(IOC_BASE_ARBITER + 1, bool)
#define IOC_ARBITER_SET_ADDRESS_TIMEOUT     IOC_CODE(IOC_BASE_ARBITER + 2, bool)



#define IOC_PARAM(_idx, _type)              (_type)args[(_idx)]

#define IOC_PARAMS(_num, _params)           _num, _params
#define IOC_NO_PARAMS                       0, (void)

#define IOCTL_FUNC_PARAMS_0(_type)

#define IOCTL_FUNC_PARAMS_1(_type) \
    , (_type)args[0]

#define IOCTL_FUNC_PARAMS_2(_type0, _type1) \
    , (_type0)args[0], (_type1)args[1]

#define IOCTL_FUNC_PARAMS_3(_type0, _type1, _type2) \
    , (_type0)args[0], (_type1)args[1], (_type2)args[2]

#define IOCTL_FUNC_PARAMS_4(_type0, _type1, _type2, _type3) \
    , (_type0)args[0], (_type1)args[1], (_type2)args[2], (_type3)args[3]

#define IOCTL_FUNC_PARAMS_5(_type0, _type1, _type2, _type3, _type4) \
    , (_type0)args[0], (_type1)args[1], (_type2)args[2], (_type3)args[3], (_type4)args[4]



/* IOCTL macros for single-object device */

#define IOC_VARS_DECLARE()  \
    int     args[5];

#define IOC_EXTRACT_ARGS(_num)                                  \
    do                                                          \
    {                                                           \
        va_list argsList;                                       \
        va_start(argsList, p_RetVal);                           \
        IOCTL_ExtractArgs(&argsList, _num, args);               \
        va_end(argsList);                                       \
    } while (0)

#define IOC_EXEC_RETURN(_handle, _func, _retType, _params)      \
    _EXEC_RETURN(_handle, _func, _retType, _params)

#define IOC_EXEC_NO_RET(_handle, _func, _params)                \
    _EXEC_NO_RET(_handle, _func, _params)


#define IOC_FUNC_CHECK_START    /* Nothing for now */

#define IOC_CHECK_RETURN(_handle, _func, _retType, _params)     \
    if (ioCtlCmd == (uint32_t)(_func))                          \
    {                                                           \
        _EXEC_RETURN(_handle, _func, _retType, _params);        \
    } else

#define IOC_CHECK_NO_RET(_handle, _func, _params)               \
    if (ioCtlCmd == (uint32_t)(_func))                          \
    {                                                           \
        _EXEC_NO_RET(_handle, _func, _params);                  \
    } else

#define IOC_FUNC_CHECK_END                                              \
    {                                                                   \
        RETURN_ERROR(MAJOR, E_INVALID_OPERATION, ("IOCTL operation"));  \
    }


#define _EXEC_RETURN(_handle, _func, _retType, _num, _params)   \
    do                                                          \
    {                                                           \
        IOC_EXTRACT_ARGS(_num);                                 \
        *(_retType *)p_RetVal =                                 \
            _func(_handle IOCTL_FUNC_PARAMS_##_num _params);    \
    } while (0)

#define _EXEC_NO_RET(_handle, _func, _num, _params)             \
    do                                                          \
    {                                                           \
        IOC_EXTRACT_ARGS(_num);                                 \
        _func(_handle IOCTL_FUNC_PARAMS_##_num _params);        \
    } while (0)



/* IOCTL macros for multiple-objects device */

#define IOC_MULTOBJ_DEV_VARS_DECLARE()                          \
    int      args[5];                                           \
    int      objId = 0;                                         \
    uint32_t objectsLimit;

#define IOC_MULTOBJ_DEV_EXTRACT_ARGS(_num)                      \
    do                                                          \
    {                                                           \
        va_list argsList;                                       \
        va_start(argsList, p_RetVal);                           \
        objId = va_arg(argsList, int);                          \
        IOCTL_ExtractArgs(&argsList, _num, args);               \
        va_end(argsList);                                       \
    } while (0)


#define IOC_MULTOBJ_DEV_FUNC_CHECK_START(_objLimit) \
    objectsLimit = _objLimit;


#define IOC_MULTOBJ_DEV_CHECK_RETURN(_handle, _func, _retType, _params) \
    if (ioCtlCmd == (uint32_t)(_func))                                  \
    {                                                                   \
        _MULTOBJ_DEV_EXEC_RETURN(_handle, _func, _retType, _params);    \
    } else


#define IOC_MULTOBJ_DEV_FUNC_CHECK_END  IOC_FUNC_CHECK_END


#define _MULTOBJ_DEV_VALIDATE_DEVICE(_handle)                           \
    if (objId >= objectsLimit)                                          \
    {                                                                   \
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,                             \
                     ("Device ID (should be 0-%d", objectsLimit - 1));  \
    }                                                                   \
    if (!_handle[objId])                                                \
    {                                                                   \
        RETURN_ERROR(MAJOR, E_NOT_FOUND, ("Device"));                   \
    }


#define _MULTOBJ_DEV_EXEC_RETURN(_handle, _func, _retType, _num, _params)   \
    do                                                                      \
    {                                                                       \
        IOC_MULTOBJ_DEV_EXTRACT_ARGS(_num);                                 \
        _MULTOBJ_DEV_VALIDATE_DEVICE(_handle);                              \
        *(_retType *)p_RetVal =                                             \
            _func(_handle[objId] IOCTL_FUNC_PARAMS_##_num _params);         \
    } while (0)



/**************************************************************************//**
 @Function      IOCTL_ExtractArgs

 @Description   Extracts the requested number of arguments of an IOCTL call.

 @Param[in]     p_VarArgsList   - Pointer to a variable arguments list.
 @Param[in]     numOfArgs       - Number of arguments to extract.
 @Param[out]    p_Args          - Pointer to an array of integers, which will be
                                  filled with the extracted arguments; The array
                                  must be at least the size of numOfArgs.

 @Return        None
*//***************************************************************************/
static __inline__ void IOCTL_ExtractArgs(va_list    *p_VarArgsList,
                                         int        numOfArgs,
                                         int        *p_Args)
{
    int i;

    for (i = 0; i < numOfArgs; i++)
    {
        p_Args[i] = va_arg(*p_VarArgsList, int);
    }
}


/** @} */ /* end of ioctl_id group */


#endif /* __IOCTL_EXT_H */
