/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_eth_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_ETH_EXT_H
#define __PLATFORM_ETH_EXT_H

#include "std_ext.h"
#include "sys_ext.h"
#include "enet_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PLATFORM_InitEthernetPhy

 @Description   Initializes the platform's Ethernet PHY matching a given
                Ethernet controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Ethernet controller object type (usually a sub-module type)
 @Param[in]     id          - Ethernet controller object identifier
 @Param[in]     ethMode     - Ethernet mode of operation (MAC-PHY interface)
 @Param[in]     duplexMode  - Duplex mode

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitEthernetPhy(t_Handle           h_Platform,
                                 e_SysModule        module,
                                 uint32_t           id,
                                 e_EnetMode         ethMode,
                                 e_EnetDuplexMode   duplexMode);

/**************************************************************************//**
 @Function      PLATFORM_FreeEthernetPhy

 @Description   Releases the platform's Ethernet PHY matching a given
                Ethernet controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Ethernet controller object type (usually a sub-module type)
 @Param[in]     id          - Ethernet controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeEthernetPhy(t_Handle       h_Platform,
                                 e_SysModule    module,
                                 uint32_t       id);

/**************************************************************************//**
 @Function      PLATFORM_GetEthernetLinkStatus

 @Description   Checks the link state of the Ethernet PHY matching a given
                Ethernet controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Ethernet controller object type (usually a sub-module type)
 @Param[in]     id          - Ethernet controller object identifier

 @Retval        TRUE    : Link is up.
 @Retval        FALSE   : Link is down or not detected.
*//***************************************************************************/
bool PLATFORM_GetEthernetLinkStatus(t_Handle    h_Platform,
                                    e_SysModule module,
                                    uint32_t    id);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_ETH_EXT_H */
