/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_p4080_ds_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_P4080_DS_EXT_H
#define __PLATFORM_P4080_DS_EXT_H

#include "std_ext.h"
#include "platform_ext.h"
#include "e500v2_L2_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @{
*//***************************************************************************/

#define PLATFORM_MAX_NUM_OF_CONNECTORS  10

#define PLATFORM_PHY_MODE_SGMII         0x80
#define PLATFORM_PHY_MODE_RGMII         0x40
#define PLATFORM_PHY_MODE_RTBI          0x20
#define PLATFORM_PHY_MODE_ANY           (PLATFORM_PHY_MODE_SGMII | \
                                         PLATFORM_PHY_MODE_RGMII | \
                                         PLATFORM_PHY_MODE_RTBI)


/**************************************************************************//**
 @Description   Platform Console Types
*//***************************************************************************/
typedef enum e_PlatformConsoleType
{
    e_PLATFORM_CONSOLE_NONE,        /**< Do not use platform console */
    e_PLATFORM_CONSOLE_DUART_1,     /**< Use DUART-1 as console port */

} e_PlatformConsoleType;

typedef struct t_PlatformBoardSettings
{
    struct
    {
        bool                useAutoNegotiation;
    } ethPhySettings;

    e_DiagMode              diagModes[PLATFORM_MAX_NUM_OF_CONNECTORS];

} t_PlatformBoardSettings;

/**************************************************************************//**
 @Description   P4080-DS Platform configuration parameters structure
*//***************************************************************************/
typedef struct t_PlatformParam
{
    uint32_t                clockInFreqHz;
    uint32_t                localBusFreqHz;
    e_CacheMode             l1CacheMode;
    e_CacheMode             l2CacheMode;
    uint32_t                l2SramSize;
    e_L2Stash               l2StashMode;
    bool                    useDdrBootSettings;
    e_PlatformConsoleType   consoleType;

    t_PlatformBoardSettings boardSettings;

    struct
    {
        t_Error (*f_InitMemoryRegions)(t_Handle h_Platform);
        t_Error (*f_InitMemoryPartitions)(t_Handle h_Platform);
        t_Error (*f_InitIntrCtrl)(t_Handle h_Platform);
        t_Error (*f_InitIoPorts)(t_Handle h_Platform);

    } userInitHooks;
} t_PlatformParam;

typedef struct t_PlatformDevicesConf{
    struct
    {
        struct
        {
            bool                    en;
            e_DiagMode              diagMode;
            uint8_t                 phyModes;
        } dtsecs[FM_MAX_NUM_OF_1G_MACS];
        struct
        {
            bool                    en;
            e_DiagMode              diagMode;
        } tgecs[FM_MAX_NUM_OF_10G_MACS];
    } fms[INTG_MAX_NUM_OF_FM];
} t_PlatformDevicesConf;

/**************************************************************************//**
 @Function      PLATFORM_GetDevicesConfiguration

 @Description   TODO

 @Param[in]     h_Platform      - Platform object handle.
 @Param[out]    p_DevicesConf   - A pointer to a structure where the platform will
                                  fill the information about the devices that supported
                                  in the current configuration.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_GetDevicesConfiguration(t_Handle h_Platform, t_PlatformDevicesConf *p_DevicesConf);

/** @} */ /* end of platform_grp */


#endif /* __PLATFORM_P4080_DS_EXT_H */

