/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_pci_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_PCI_EXT_H
#define __PLATFORM_PCI_EXT_H

#include "std_ext.h"
#include "pci_std_ext.h"
#include "sys_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Mapping of a physical slot to an IRQ line
*//***************************************************************************/
typedef struct t_PlatformPciIrqMapEntry
{
	uint32_t  slot; /**< Slot number */
	e_PciIrq  irq;  /**< IRQ line for the given slot */

} t_PlatformPciIrqMapEntry;

/**************************************************************************//**
 @Function      PLATFORM_InitPciPort

 @Description   Initializes the platform's PCI bus or PCIe port matching a
                given PCI/PCIe controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - PCI/PCIe controller object type (usually a sub-module type)
 @Param[in]     id          - PCI/PCIe controller object identifier
 @Param[in]     hostMode    - TRUE for host mode; FALSE for agent mode

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitPciPort(t_Handle       h_Platform,
                             e_SysModule    module,
                             uint32_t       id,
                             bool           hostMode);

/**************************************************************************//**
 @Function      PLATFORM_FreePciPort

 @Description   Releases the platform's PCI bus or PCIe port matching a
                given PCI/PCIe controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - PCI/PCIe controller object type (usually a sub-module type)
 @Param[in]     id          - PCI/PCIe controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreePciPort(t_Handle       h_Platform,
                             e_SysModule    module,
                             uint32_t       id);

/**************************************************************************//**
 @Function      PLATFORM_GetPciIrqMapping

 @Description   Retrieves platform's mapping of PCI IRQ signals matching a
                given PCI/PCIe controller object.

 @Param[in]     h_Platform      - Platform object handle.
 @Param[in]     module          - PCI/PCIe controller object type (usually a sub-module type)
 @Param[in]     id              - PCI/PCIe controller object identifier
 @Param[out]    p_IrqMapping    - Receives the mapping array of PCI IRQs to slots.
 @Param[out]    p_NumOfEntries  - Receives the number of entries in the PCI
                                  IRQ mapping array.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_GetPciIrqMapping(t_Handle                  h_Platform,
                                  e_SysModule               subModule,
                                  uint32_t                  id,
                                  t_PlatformPciIrqMapEntry  **p_IrqMapping,
                                  uint32_t                  *p_NumOfEntries);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_PCI_EXT_H */
