/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __PLATFORM_QE_EXT_H
#define __PLATFORM_QE_EXT_H

#include "std_ext.h"
#include "qe_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PLATFORM_GetQePeripheralInputClk

 @Description   Gets the input clock signal selection of a given QUICC Engine
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Controller object type (usually a sub-module type)
 @Param[in]     id          - Controller object identifier
 @Param[in]     mode        - Mode selection (varies for different controllers types)
 @Param[in]     commMode    - Indicates if the input clock is associated with
                              Rx, Tx or both.

 @Return        The input clock signal selection of a given QUICC Engine
                controller object.
*//***************************************************************************/
e_QeClock PLATFORM_GetQePeripheralInputClk(t_Handle     h_Platform,
                                           e_SysModule  module,
                                           uint32_t     id,
                                           int          mode,
                                           e_CommMode   commMode);

/**************************************************************************//**
 @Function      PLATFORM_GetQeBusAccessWindowInfo

 @Description   Retrieves the base address and size of a QUICC Engine memory
                access window.

                The information returned by this routine may be required for
                configuring the dynamic bus selection capabilities of the
                QUICC Engine.

 @Param[in]     h_Platform      - Platform object handle.
 @Param[in]     secondaryBus    - TRUE if the requested information relates to
                                  the QUICC Engine's secondary bus;
                                  FALSE if the requested information relates to
                                  the QUICC Engine's primary (system) bus.
 @Param[out]    p_BaseAddr      - Returns the base address of the access window
 @Param[out]    p_Size          - Returns the size of the access window

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_GetQeBusAccessWindowInfo(t_Handle  h_Platform,
                                          bool      secondaryBus,
                                          uint32_t  *p_BaseAddr,
                                          uint32_t  *p_Size);

/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_QE_EXT_H */

