/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_serial_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_SERIAL_EXT_H
#define __PLATFORM_SERIAL_EXT_H

#include "std_ext.h"
#include "sys_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitSerialPort

 @Description   Initializes the platform's serial (UART) port matching a given
                serial controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Serial controller object type (usually a sub-module type)
 @Param[in]     id          - Serial controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitSerialPort(t_Handle    h_Platform,
                                e_SysModule module,
                                uint32_t    id);

/**************************************************************************//**
 @Function      SYS_FreeSerialPort

 @Description   Releases the platform's serial (UART) port matching a given
                serial controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Serial controller object type (usually a sub-module type)
 @Param[in]     id          - Serial controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeSerialPort(t_Handle    h_Platform,
                                e_SysModule module,
                                uint32_t    id);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_SERIAL_EXT_H */
