/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_spi_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_SPI_EXT_H
#define __PLATFORM_SPI_EXT_H

#include "std_ext.h"
#include "sys_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PLATFORM_InitSpiPort

 @Description   Initializes the platform's SPI port matching a given SPI
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - SPI controller object type (usually a sub-module type)
 @Param[in]     id          - SPI controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitSpiPort(t_Handle    h_Platform,
                             e_SysModule module,
                             uint32_t    id);

/**************************************************************************//**
 @Function      PLATFORM_FreeSpiPort

 @Description   Releases the platform's SPI port matching a given SPI
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - SPI controller object type (usually a sub-module type)
 @Param[in]     id          - SPI controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeSpiPort(t_Handle    h_Platform,
                             e_SysModule module,
                             uint32_t    id);

t_Error PLATFORM_CreateSpiDevices(t_Handle  h_Platform,
                                  t_Handle  h_Spi,
                                  t_Handle  h_SpiBus,
                                  uint8_t   busId,
                                  bool      loopbackDev);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_SPI_EXT_H */
