/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/**

 @File          semaphore_ext.h

 @Description   structures and definitions for initialize and operate the
                semaphore module.

                This module ....

*//***************************************************************************/
#ifndef SEMAPHORE_EXT_H
#define SEMAPHORE_EXT_H


#include "std_ext.h"


/**************************************************************************//**
 @Group         Semaphore - Semaphore module

 @Description   Semaphore module functions,definitions and enums.

 @{
*//***************************************************************************/

typedef struct {
    /*
     * Note that any negative value of count is equivalent to 0,
     * but additionally indicates that some process(es) might be
     * sleeping on `wait'.
     */
    volatile int count;
} t_Semaphore;


/**************************************************************************//**
 @Function      SEMAPHORE_Init

 @Description   Initialize semaphore structure.

                The routine initializes the given semaphore structure with the
                given 'val' preameter.

 @Param         p_Sem  -  (In) Pointer to semaphore structure to be initialize.
 @Param         val    -  (In) The initializition value of the semaphore.
*//***************************************************************************/
void SEMAPHORE_Init (t_Semaphore *p_Sem, int val);

/**************************************************************************//**
 @Function      SEMAPHORE_InitMutex

 @Description   Initialize semaphore structure.

                The routine initializes the given semaphore structure with
                value of 1 - Unlocked.

 @Param         p_Sem  -  (In) Pointer to semaphore structure to be initialize.
*//***************************************************************************/
void SEMAPHORE_InitMutex (t_Semaphore *p_Sem);

/**************************************************************************//**
 @Function      SEMAPHORE_InitMutexLocked

 @Description   Initialize semaphore structure.

                The routine initializes the given semaphore structure with
                value of 0 - Locked.

 @Param         p_Sem  -  (In) Pointer to semaphore structure to be initialize.
*//***************************************************************************/
void SEMAPHORE_InitMutexLocked (t_Semaphore *p_Sem);

/**************************************************************************//**
 @Function      SEMAPHORE_Down

 @Description   Try to decrease the semaphore (Lock it if its was greater than 0).

                This routine tryes to decrease the semaphore value.

 @Param         p_Sem  -  (In) Pointer to semaphore structure to be lock.

 @Return        1 - Fail to lock. 0 - on success.
*//***************************************************************************/
int SEMAPHORE_TryDown(t_Semaphore * p_Sem);

/**************************************************************************//**
 @Function      SEMAPHORE_Down

 @Description   Decrease the semaphore (Lock it if its was greater than 0).

                This routine decreases the semaphore value.

 @Param         p_Sem  -  (In) Pointer to semaphore structure to be lock.
*//***************************************************************************/
void SEMAPHORE_Down(t_Semaphore * p_Sem);

/**************************************************************************//**
 @Function      SEMAPHORE_Up

 @Description   Increase the semaphore (Unlock it if its was nagative).

                This routine increases the semaphore value.

 @Param         p_Sem  -  (In) Pointer to semaphore structure to be unlock.
*//***************************************************************************/
void SEMAPHORE_Up(t_Semaphore * p_Sem);
/** @} */ /* end of group */


#endif /* SEMAPHORE_EXT_H */
