/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_CLI_EXT_H
#define __SYS_CLI_EXT_H

#include "std_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_cli_grp     System CLI Service

 @Description   Bare-board system interface for CLI (Command Line Interface)
                operations.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      SYS_IsCliAvailable

 @Description   Check if the system's CLI is enabled.

 @Return        TRUE if CLI is enabled in the system; FALSE if not.
*//***************************************************************************/
bool    SYS_IsCliAvailable(void);

/**************************************************************************//**
 @Function      SYS_RegisterCliCommand

 @Description   Register a new command to the system's CLI.

 @Param[in]     name        - CLI command name.
 @Param[in]     desc        - CLI command description.
 @Param[in]     format      - CLI command format string.
 @Param[in]     h_CommArg   - Argument to pass the to \c f_DoCommand callback.
 @Param[in]     f_DoCommand - Callback routine to execute the command.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SYS_RegisterCliCommand(char     *name,
                               char     *desc,
                               char     *format,
                               t_Handle h_CommArg,
                               t_Error  (*f_DoCommand)(t_Handle h_CommArg, int argc, char *argv[]));

/**************************************************************************//**
 @Function      SYS_RegisterCliCommand

 @Description   Unregister a command from the system's CLI.

                The CLI command must have been previously registered using the
                SYS_RegisterCliCommand() routine.

 @Param[in]     name - CLI command name.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SYS_UnregisterCliCommand(char *name);


/** @} */ /* end of sys_cli_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_CLI_EXT_H */
