/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_PCI_EXT_H
#define __SYS_PLATFORM_PCI_EXT_H

#include "std_ext.h"
#include "platform_pci_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitPciPort

 @Description   Initializes the platform's PCI bus or PCIe port matching a
                given PCI/PCIe controller object.

 @Param[in]     module      - PCI/PCIe controller object type (usually a sub-module type)
 @Param[in]     id          - PCI/PCIe controller object identifier
 @Param[in]     hostMode    - TRUE for host mode; FALSE for agent mode

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitPciPort(e_SysModule module, uint32_t id, bool hostMode)
{
    return PLATFORM_InitPciPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                    module,
                                                    id,
                                                    hostMode);
}

/**************************************************************************//**
 @Function      SYS_FreePciPort

 @Description   Releases the platform's PCI bus or PCIe port matching a
                given PCI/PCIe controller object.

 @Param[in]     module  - PCI/PCIe controller object type (usually a sub-module type)
 @Param[in]     id      - PCI/PCIe controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreePciPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_FreePciPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                    module,
                                                    id);
}

/**************************************************************************//**
 @Function      SYS_GetPciIrqMapping

 @Description   Retrieves platform's mapping of PCI IRQ signals matching a
                given PCI/PCIe controller object.

 @Param[in]     module          - PCI/PCIe controller object type (usually a sub-module type)
 @Param[in]     id              - PCI/PCIe controller object identifier
 @Param[out]    p_IrqMapping    - Receives the mapping array of PCI IRQs to slots.
 @Param[out]    p_NumOfEntries  - Receives the number of entries in the PCI
                                  IRQ mapping array.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_GetPciIrqMapping(e_SysModule              module,
                                               uint32_t                 id,
                                               t_PlatformPciIrqMapEntry **p_IrqMapping,
                                               uint32_t                 *p_NumOfEntries)
{
    return PLATFORM_GetPciIrqMapping(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id,
                                     p_IrqMapping,
                                     p_NumOfEntries);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_PCI_EXT_H */

