/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_SERIAL_EXT_H
#define __SYS_PLATFORM_SERIAL_EXT_H

#include "std_ext.h"
#include "platform_serial_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitSerialPort

 @Description   Initializes the platform's serial (UART) port matching a given
                serial controller object.

 @Param[in]     module  - Serial controller object type (usually a sub-module type)
 @Param[in]     id      - Serial controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitSerialPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_InitSerialPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                   module,
                                   id);
}

/**************************************************************************//**
 @Function      SYS_FreeSerialPort

 @Description   Releases the platform's serial (UART) port matching a given
                serial controller object.

 @Param[in]     module  - Serial controller object type (usually a sub-module type)
 @Param[in]     id      - Serial controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeSerialPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_FreeSerialPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                   module,
                                   id);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_SERIAL_EXT_H */

