/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_SPI_EXT_H
#define __SYS_PLATFORM_SPI_EXT_H

#include "std_ext.h"
#include "platform_spi_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitSpiPort

 @Description   Initializes the platform's SPI port matching a given SPI
                controller object.

 @Param[in]     module  - SPI controller object type (usually a sub-module type)
 @Param[in]     id      - SPI controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitSpiPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_InitSpiPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                    module,
                                                    id);
}

/**************************************************************************//**
 @Function      SYS_FreeSpiPort

 @Description   Releases the platform's SPI port matching a given SPI
                controller object.

 @Param[in]     module  - SPI controller object type (usually a sub-module type)
 @Param[in]     id      - SPI controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeSpiPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_FreeSpiPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                    module,
                                                    id);
}

//@@@@ re-think this one
static __inline__ t_Error SYS_CreateSpiDevices(t_Handle h_Spi, t_Handle h_SpiBus, uint8_t busId, bool isLoopbackDev)
{
    return PLATFORM_CreateSpiDevices(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     h_Spi,
                                     h_SpiBus,
                                     busId,
                                     isLoopbackDev);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_SPI_EXT_H */

