/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_UTOPIA_POS_EXT_H
#define __SYS_PLATFORM_UTOPIA_POS_EXT_H

#include "std_ext.h"
#include "platform_utopia_pos_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitUtopiaPosBus

 @Description   Initializes a UTOPIA/POS bus.

 @Param[in]     module              - Object type.
 @Param[in]     id                  - Bus ID.
 @Param[in]     lineRate            - Line rate.
 @Param[in]     protocol            - Protocol.
 @Param[in]     master              - TRUE for bus master mode; FALSE for bus slave mode.
 @Param[in]     numOfDevices        - Number of UTOPIA/POS devices per bus controller.
 @Param[in]     p_DevParamArray     - Array of UTOPIA/POS device parameters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitUtopiaPosBus(e_SysModule                  module,
                                               uint32_t                     id,
                                               e_LineRate                   lineRate,
                                               e_UtopiaPosProtocol          protocol,
                                               bool                         master,
                                               uint8_t                      numOfDevices,
                                               t_PlatformUtopiaPosDevParam  *p_DevParamArray)
{
    return PLATFORM_InitUtopiaPosBus(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id,
                                     lineRate,
                                     protocol,
                                     master,
                                     numOfDevices,
                                     p_DevParamArray);
}

/**************************************************************************//**
 @Function      SYS_FreeUtopiaPosBus

 @Description   Terminates a UTOPIA/POS bus.

 @Param[in]     module - Object type.
 @Param[in]     id     - Bus ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeUtopiaPosBus(e_SysModule module,
                                               uint32_t    id)
{
    return PLATFORM_FreeUtopiaPosBus(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id);
}

/**************************************************************************//**
 @Function      SYS_InitUtopiaPosPhy

 @Description   Initializes a UTOPIA/POS PHY.

 @Param[in]     module   - Object type.
 @Param[in]     id       - PHY ID.
 @Param[in]     phyAddr  - PHY address.
 @Param[in]     diagMode - Diagnostic mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitUtopiaPosPhy(e_SysModule module,
                                               uint32_t    id,
                                               uint8_t     phyAddr,
                                               e_DiagMode  diagMode)
{
    return PLATFORM_InitUtopiaPosPhy(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id,
                                     phyAddr,
                                     diagMode);
}

/**************************************************************************//**
 @Function      SYS_FreeUtopiaPosPhy

 @Description   Terminates a UTOPIA/POS PHY.

 @Param[in]     module - Object type.
 @Param[in]     id     - PHY ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeUtopiaPosPhy(e_SysModule module,
                                               uint32_t    id)
{
    return PLATFORM_FreeUtopiaPosPhy(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_UTOPIA_POS_EXT_H */

