/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __P4080_DS_H
#define __P4080_DS_H

#include "p4080_ds_ext.h"


#define __ERR_MODULE__ MODULE_UNKNOWN

#define P4080DS_NUM_OF_PIXIS_REGS    0x30

#define P4080DS_NUM_OF_ETH_PORTS    10

#define IO_GROUP_MARKER             -1

#define P4080DS_MAX_ETH_MODES       6

#define NO_CS                       0xFF

#define P4080DS_MEMORY_CTRLS_CSs {      \
    /* id                    CS    */   \
    {e_P4080DS_MEM_NONE,     NO_CS},    \
    {e_P4080DS_MEM_FLASH,    0},        \
    {e_P4080DS_MEM_PROMJET,  1},        \
    {e_P4080DS_MEM_NGPIXIS,  2}         \
}


#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(push,1)
#endif /* defined(__MWERKS__) && ... */
#define MEM_MAP_START

/* Local Configuration Control Register Map */
typedef _Packed struct
{
    volatile uint8_t   px_id;
    volatile uint8_t   px_arch;
    volatile uint8_t   px_scver;
    volatile uint8_t   px_csr;
    volatile uint8_t   px_rst;
    volatile uint8_t   res0;
    volatile uint8_t   px_aux;
    volatile uint8_t   px_spd;
    volatile uint8_t   px_brdcfg0;
    volatile uint8_t   px_dma;
    volatile uint8_t   px_addr;
    volatile uint8_t   res1[0x2];
    volatile uint8_t   px_data;
    volatile uint8_t   px_led;
    volatile uint8_t   res2;
    volatile uint8_t   px_vctl;
    volatile uint8_t   px_vstat;
    volatile uint8_t   px_vcfgen0;
    volatile uint8_t   res3;
    volatile uint8_t   px_ocmcsr;
    volatile uint8_t   px_ocmmsg;
    volatile uint8_t   px_gmdbg;
    volatile uint8_t   res4;
    volatile uint8_t   px_sclk0;
    volatile uint8_t   px_sclk1;
    volatile uint8_t   px_sclk2;
    volatile uint8_t   px_watch;
    volatile uint8_t   px_sw[8];
    volatile uint8_t   px_en[8];
} _PackedType t_P4080DsPixisMemMap;

#define MEM_MAP_END
#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(push,1)
#endif /* defined(__MWERKS__) && ... */


typedef struct t_P4080DsMemCtrlDesc {
    uint8_t     id;
    uint8_t     chipSelect;
} t_P4080DsMemCtrlDesc;

typedef struct t_P4080Ds {
    uint8_t                 gethPortsCount;
    e_EnetMode              ethModes[P4080DS_NUM_OF_ETH_PORTS];
    bool                    serialPortsEnabled[P4080DS_NUM_OF_SERIAL_PORTS];
    t_P4080DsPixisMemMap    *p_PixisMemMap;        /**<  */
} t_P4080Ds;

/****************************************/
/* PIXIS defines - in Hz                */
/****************************************/
#define PIXIS_SYS_CLK_66_666    (66.6660 * 1000000)
#define PIXIS_SYS_CLK_75        (75.0000 * 1000000)
#define PIXIS_SYS_CLK_83_333    (83.3330 * 1000000)
#define PIXIS_SYS_CLK_90        (90.0000 * 1000000)
#define PIXIS_SYS_CLK_100       (100.000 * 1000000)
#define PIXIS_SYS_CLK_111       (111.000 * 1000000)
#define PIXIS_SYS_CLK_125       (125.000 * 1000000)
#define PIXIS_SYS_CLK_133_333   (133.333 * 1000000)

typedef enum e_PixisSysClk {
    e_PIXIS_SYS_CLK_66_666 = 0,
    e_PIXIS_SYS_CLK_75,
    e_PIXIS_SYS_CLK_83_333,
    e_PIXIS_SYS_CLK_90,
    e_PIXIS_SYS_CLK_100,
    e_PIXIS_SYS_CLK_111,
    e_PIXIS_SYS_CLK_125,
    e_PIXIS_SYS_CLK_133_333
} e_PixisSysClk;

#define PX_SPD_SYSCLK_MASK  0x07


#endif /* __P4080_DS_H */
