/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          mds_1021_ext.h

 @Description   Prototypes, externals and typedefs for MDS1021 routines
*//***************************************************************************/
#ifndef __MDS_1021_EXT_H
#define __MDS_1021_EXT_H

#include "enet_ext.h"
#include "mii_mng_ext.h"
#include "utopia_pos_ext.h"
#include "pq_mds_t1_ext.h"
#include "carrier_ext.h"
#include "sdhc_std_ext.h"


/**************************************************************************//**
 @Group         mds1021_id_grp

 @Description   MDS1021 definitions and enums.

 @{
*//***************************************************************************/

#define MDS1021_NUM_OF_TDMS             4
#define MDS1021_NUM_OF_I2C_PORTS        2
#define MDS1021_NUM_OF_SERIAL_PORTS     e_MDS1021_SERIAL_PORT_DUMMY_LAST

/**************************************************************************//**
 @Description   P1021MDS Memory Regions
*//***************************************************************************/
typedef enum e_Mds1021MemRegion
{
    e_MDS1021_MEM_RGN_DDR,
    e_MDS1021_MEM_RGN_PCIE_1,
    e_MDS1021_MEM_RGN_CCSR,
    e_MDS1021_MEM_RGN_L2_SRAM,
    e_MDS1021_MEM_RGN_BCSR,
    e_MDS1021_MEM_RGN_PIB_PMC0,
    e_MDS1021_MEM_RGN_PIB_PMC1,
    e_MDS1021_MEM_RGN_NAND_FLASH

} e_Mds1021MemRegion;


/**************************************************************************//**
 @Description   MPC1021MDS Memory Regions
*//***************************************************************************/
typedef struct t_Mds1021MemRegionInfo
{
    uint32_t    startAddr;
    uint32_t    size;
    uint8_t     chipSelect;

} t_Mds1021MemRegionInfo;

/**************************************************************************//**
 @Description   Ethernet PHY selection
*//***************************************************************************/
typedef enum e_Mds1021EthPort
{
    e_MDS1021_ETH_PORT_UCC_1 = 0,
    e_MDS1021_ETH_PORT_UCC_5,
    e_MDS1021_ETH_PORT_ETSEC_1,
    e_MDS1021_ETH_PORT_ETSEC_2,
    e_MDS1021_ETH_PORT_ETSEC_3,
    e_MDS1021_ETH_PORT_DUMMY_LAST

} e_Mds1021EthPort;

typedef enum e_Mds1021UtopiaPosMode
{
    e_MDS1021_UTOPIA_POS_NOT_SUPPORTED,
    e_MDS1021_UTOPIA_SLAVE_SDEV_SPHY_8_BITS,
    e_MDS1021_UTOPIA_MASTER_SDEV_SPHY_8_BITS,
    e_MDS1021_UTOPIA_MASTER_SDEV_MPHY_8_BITS

} e_Mds1021UtopiaPosMode;

typedef enum e_Mds1021Led
{
   e_MDS1021_LED_GREEN = 0,
   e_MDS1021_LED_YELLOW,
   e_MDS1021_LED_RED

} e_Mds1021Led;

typedef enum e_Mds1021SerialPort
{
    e_MDS1021_SERIAL_PORT_DUART_1 = 0,
    e_MDS1021_SERIAL_PORT_DUART_2,
    e_MDS1021_SERIAL_PORT_UCC_UART_7,
    e_MDS1021_SERIAL_PORT_DUMMY_LAST

} e_Mds1021SerialPort;

typedef enum e_Mds1021SpiPort
{
    e_MDS1021_ESPI_PORT = 0,
    e_MDS1021_SPI_PORT

} e_Mds1021SpiPort;

typedef enum e_Mds1021IoGroup
{
    e_MDS1021_IO_GRP_TDM_A_SERIAL,
    e_MDS1021_IO_GRP_TDM_B_SERIAL,
    e_MDS1021_IO_GRP_TDM_C_SERIAL,
    e_MDS1021_IO_GRP_TDM_D_SERIAL,
    e_MDS1021_IO_GRP_UCC_1_MII,
    e_MDS1021_IO_GRP_UCC_1_RMII,
    e_MDS1021_IO_GRP_UCC_5_RMII,
    e_MDS1021_IO_GRP_UPC_1_UTOPIA_SLAVE_SDEV_SPHY_8_BITS,
    e_MDS1021_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_SPHY_8_BITS,
    e_MDS1021_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_MPHY_8_BITS,
    e_MDS1021_IO_GRP_SPI_1_MASTER,
    e_MDS1021_IO_GRP_SPI_1_SLAVE,
    e_MDS1021_IO_GRP_PTP,
    e_MDS1021_IO_GRP_UCC_7_UART,
    e_MDS1021_IO_GRP_MII_CFG_QE_MUX

} e_Mds1021IoGroup;

t_Handle MDS1021_ConfigAndInit(void);

t_Error MDS1021_Free(t_Handle h_Mds1021);

t_Error MDS1021_InitEthernetPhy(t_Handle            h_Mds1021,
                                e_Mds1021EthPort    ethPort,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                fiber,
                                bool                forceMaster,
                                bool                loopbackConnector,
                                t_Handle            h_MiiAccess,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg);

e_EnetMode MDS1021_FreeEthernetPhy(t_Handle         h_Mds1021,
                                   e_Mds1021EthPort ethPort);

bool MDS1021_GetEthernetLinkStatus(t_Handle             h_Mds1021,
                                   e_Mds1021EthPort     ethPort,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg);

t_Error MDS1021_SetUtopiaPos(t_Handle               h_Mds1021,
                             uint8_t                pmcId,
                             bool                   reset,
                             e_Mds1021UtopiaPosMode mdsUtopiaPosMode);

bool MDS1021_PqMdsQoc3IsConnected(t_Handle h_Mds1021,
                                  uint32_t baseAddr);

bool MDS1021_PqMdsE1T1IsConnected(t_Handle h_Mds1021,
                                  uint32_t baseAddr);

bool MDS1021_PqMds16E1T1IsConnected(t_Handle h_Mds1021,
                                  uint32_t baseAddr);

t_Error MDS1021_InitPqMdsQoc3(t_Handle              h_Mds1021,
                              uint32_t              baseAddr,
                              uint8_t               numOfDevs,
                              uint8_t               numOfPhys,
                              e_LineRate            lineRate,
                              e_UtopiaPosDevMode    allDevsOpMode,
                              e_UtopiaPosBusWidth   allDevsBusWidth);

t_Error MDS1021_FreePqMdsQoc3(t_Handle h_Mds1021,
                              uint32_t baseAddr);

t_Error MDS1021_InitPqMdsQoc3Phy(t_Handle               h_Mds1021,
                                 uint32_t               baseAddr,
                                 uint8_t                phyId,
                                 uint8_t                phyAddr,
                                 e_DiagMode             diagMode,
                                 e_UtopiaPosDevMode     allDevsOpMode,
                                 e_UtopiaPosBusWidth    allDevsBusWidth);

t_Error MDS1021_FreePqMdsQoc3Phy(t_Handle h_Mds1021,
                                 uint32_t baseAddr,
                                 uint8_t  phyIdOnQuad);

t_Error MDS1021_InitSerialPort(t_Handle h_Mds1021, e_Mds1021SerialPort serialPort);

t_Error MDS1021_FreeSerialPort(t_Handle h_Mds1021, e_Mds1021SerialPort serialPortId);

t_Error MDS1021_InitSdPort(t_Handle h_Mds1021, e_SdhcBusWidth busWidth);

t_Error MDS1021_FreeSdPort(t_Handle h_Mds1021);

t_Error MDS1021_InitSpiPort(t_Handle h_Mds1021, e_Mds1021SpiPort spiPort);

t_Error MDS1021_FreeSpiPort(t_Handle h_Mds1021, e_Mds1021SpiPort spiPort);

t_Error MDS1021_InitI2cPort(t_Handle h_Mds1021, uint8_t i2cPortId);

t_Error MDS1021_FreeI2cPort(t_Handle h_Mds1021, uint8_t i2cPortId);

t_Error MDS1021_InitPciePort(t_Handle h_Mds1021);

t_Error MDS1021_FreePciePort(t_Handle h_Mds1021);

t_Error MDS1021_InitTdmPhy(t_Handle             h_Mds1021,
                           uint8_t              phyId,
                           e_LineRate           lineRate,
                           e_LineEncoding       lineEncoding,
                           e_TdmFramingMode     framingMode,
                           e_DiagMode           diagMode,
                           int                  irq,
                           t_PqMdsT1EvCallback  f_Isr,
                           t_Handle             h_IsrArg);

t_Error MDS1021_Init16TdmPhy(t_Handle             h_Mds1021,
                             uint8_t              phyId,
                             e_LineRate           lineRate,
                             e_LineEncoding       lineEncoding,
                             e_TdmFramingMode     framingMode,
                             e_DiagMode           diagMode,
                             int                  irq,
                             t_PqMdsT1EvCallback  f_Isr,
                             t_Handle             h_IsrArg);

t_Error MDS1021_EnableTdmPhyIntr(t_Handle h_Mds1021, uint8_t phyId);

t_Error MDS1021_FreeTdmPhy(t_Handle h_Mds1021, uint8_t phyId);

t_Error MDS1021_InitNandFlash(t_Handle h_Mds1021);

t_Error MDS1021_FreeNandFlash(t_Handle h_Mds1021);

t_Error MDS1021_SetLed(t_Handle h_Mds1021, e_Mds1021Led led, bool ledOn);


t_Error MDS1021_GetIoGroupPins(t_Handle         h_Mds1021,
                               e_Mds1021IoGroup ioGroup,
                               int              **p_Pins,
                               uint16_t         *p_NumOfPins);

t_Error MDS1021_GetIoGroupInputClockPins(t_Handle           h_Mds1021,
                                         e_Mds1021IoGroup   ioGroup,
                                         int                *p_RxClkPin,
                                         int                *p_TxClkPin);

t_Error MDS1021_GetMemRegionInfo(e_Mds1021MemRegion     memRegion,
                                 t_Mds1021MemRegionInfo *p_MemRegionInfo);


/** @} *//* mds1021_id_grp */

#endif /* __MDS_1021_EXT_H */
