/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          phast12p_ext.h

 @Description   PHAST12P QUAD/PHYs external structures and definitions.
*//***************************************************************************/
#ifndef __PHAST12P_EXT_H
#define __PHAST12P_EXT_H


#include "std_ext.h"
#include "utopia_pos_ext.h"
#include "carrier_ext.h"


bool PHAST12P_IsConnected(uint32_t baseAddr);


void PHAST12P_EnableQuad(uint32_t               baseAddr,
                         uint8_t                numOfDevs,
                         uint8_t                numOfPhys,
                         e_LineRate             lineRate,
                         e_UtopiaPosProtocol    protocol,
                         e_UtopiaPosDevMode     allDevsOpMode,
                         e_UtopiaPosBusWidth    allDevsBusWidth);


void PHAST12P_DisableQuad(uint32_t baseAddr);


void PHAST12P_EnablePhy(uint32_t            baseAddr,
                        uint8_t             phyId,
                        uint8_t             phyAddr,
                        e_DiagMode          diagMode,
                        e_UtopiaPosProtocol protocol,
                        e_UtopiaPosDevMode  allDevsOpMode,
                        e_UtopiaPosBusWidth allDevsBusWidth);


void PHAST12P_DisablePhy(uint32_t baseAddr,uint8_t phyId);


typedef struct
{
    uint32_t txFrameFromFifoCounter ; /**< Number of good cells/frames received from Tx FIFO                                      */
    uint32_t txIdleCellInsertCounter; /**< Number of cells inserted                                                               */
    uint8_t  txHecErrorCounter      ; /**< Number of corrupted cells received from Tx FIFO                                        */
    uint8_t  txUnderflowCounter     ; /**< Number of frames affected by a TX FIFO underflow                                       */
    uint8_t  txErroredFrameCounter  ; /**< Number of frames for which the error signal was asserted during the last word transfer */
    uint32_t rxFrameToFifoCounter   ; /**< Number of cells/frames forwarded to Rx FIFO                                            */
    uint32_t rxCellFilteredCounter  ; /**< Number of cells discarded due to cell filtering                                        */
    uint16_t rxCorrHecErrorCounter  ; /**< Number of cells with corrected HEC error                                               */
    uint16_t rxUncorrHecErrorCounter; /**< Number of cells with uncorrected HEC error                                             */
    uint16_t rxOverflowCounter      ; /**< Number of cells/frames discarded due to Rx FIFO overflow                               */
    uint8_t  rxFrameDiscardedCounter; /**< Number of frames with an abort sequence (0x7D7E) or mismatched Address/Control fields  */
    uint8_t  rxMaxFrameLengthCounter; /**< Number of frames longer than the maximum frame length                                  */
    uint8_t  rxMinFrameLengthCounter; /**< Number of frames shorter than the minimum frame length                                 */
}
t_Phast12pStatCounters;


void PHAST12P_RetrievePhyStatCounters(uint32_t                baseAddr,
                                      uint8_t                 phyId,
                                      t_Phast12pStatCounters* p_Phast12pStatCounters);


#endif /* __PHAST12P_EXT_H */
