/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          PQ-MDS-T1_ext.h

 @Author        Zelda Langenmass

 @Description   Header of the initialization routines of the PQ-MDS-T1 E1 and T1 phys.
*//***************************************************************************/
#ifndef __PQMDST1_EXT_H__
#define __PQMDST1_EXT_H__

#include "error_ext.h"
#include "std_ext.h"
#include "carrier_ext.h"


/**************************************************************************//**
 @Description  PQ-MDS-T1 framer events for interrupt
*//***************************************************************************/
typedef uint16_t    t_PqMdsT1FramerTxEvent;
typedef uint64_t    t_PqMdsT1FramerRxEvent;
typedef uint8_t     t_PqMdsT1FramerLiuEvent;

/* Tx Latched status register 1 */
#define PQ_MDS_T1_EV_TX_ELASTIC_STORE_FULL                0x8000
#define PQ_MDS_T1_EV_TX_ELASTIC_STORE_EMPTY               0x4000
#define PQ_MDS_T1_EV_TX_ELASTIC_STORE_SLIP                0x2000
#define PQ_MDS_T1_EV_TX_TRANSMIT_SLC96                    0x1000
#define PQ_MDS_T1_EV_TX_TRANSMIT_TAF                      0x0800
#define PQ_MDS_T1_EV_TX_TRANSMIT_MULTIFRAME               0x0400
#define PQ_MDS_T1_EV_TX_LOSS_OF_TRANSMISION_CLOCK_CLEAR   0x0200
#define PQ_MDS_T1_EV_TX_LOSS_OF_TRANSMISION_CLOCK         0x0100

/* Tx Latched status register 2 */
#define PQ_MDS_T1_EV_TX_TRANSMIT_FDL_REGISTER_EMPTY       0x0080
#define PQ_MDS_T1_EV_TX_TRANSMIT_FIFO_UNDERRUN            0x0040
#define PQ_MDS_T1_EV_TX_TRANSMIT_MESSAGE_END              0x0020
#define PQ_MDS_T1_EV_TX_TRANSMIT_FIFO_BELOW               0x0010
#define PQ_MDS_T1_EV_TX_TRANSMIT_FIFO_NOT_FULL            0x0008

/* Tx Latched status register 3 */
#define PQ_MDS_T1_EV_TX_LOSS_OF_FRAME                     0x0004
#define PQ_MDS_T1_EV_TX_LOSS_OF_FRAME_SYNC                0x0002

/* Rx Latched status register 1 */
#define PQ_MDS_T1_EV_RX_REMOTE_ALARM_INDICATION_CLEAR     0x8000000000000000LL
#define PQ_MDS_T1_EV_RX_REMOTE_ALARM_SIGNAL_CLEAR         0x4000000000000000LL
#define PQ_MDS_T1_EV_RX_LOSS_OF_SIGNAL_CLEAR              0x2000000000000000LL
#define PQ_MDS_T1_EV_RX_LOSS_OF_FRAME_CLEAR               0x1000000000000000LL
#define PQ_MDS_T1_EV_RX_REMOTE_ALARM_INDICATION_DETECT    0x0800000000000000LL
#define PQ_MDS_T1_EV_RX_REMOTE_ALARM_SIGNAL_DETECT        0x0400000000000000LL
#define PQ_MDS_T1_EV_RX_LOSS_OF_SIGNAL_DETECT             0x0200000000000000LL
#define PQ_MDS_T1_EV_RX_LOSS_OF_FRAME_DETECT              0x0100000000000000LL

/* Rx Latched status register 2 T1 mode */
#define PQ_MDS_T1_EV_RX_PULSE_DENSITY_VIOLATION           0x0080000000000000LL
#define PQ_MDS_T1_EV_RX_CHANGE_OF_FRAME_ALIGNMENT         0x0040000000000000LL
#define PQ_MDS_T1_EV_RX_EIGHT_ZERO_DETECT                 0x0020000000000000LL
#define PQ_MDS_T1_EV_RX_SIXTEEN_ZERO_DETECT               0x0010000000000000LL
#define PQ_MDS_T1_EV_RX_SEVERLY_ERRORED_FRAMING           0x0008000000000000LL
#define PQ_MDS_T1_EV_RX_B8ZS_CODEWORD_DETECT              0x0004000000000000LL
#define PQ_MDS_T1_EV_RX_FRAME_BIT_ERROR                   0x0002000000000000LL

/* Rx Latched status register 2 E1 mode */
#define PQ_MDS_T1_EV_RX_CRC_RESYNC_MET                    0x0001000000000000LL
#define PQ_MDS_T1_EV_RX_CAS_RESYNC_MET                    0x0000800000000000LL
#define PQ_MDS_T1_EV_RX_FAS_RESYNC_MET                    0x0000400000000000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_SIGNALING_ALL_ONES        0x0000200000000000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_SIGNALING_ALL_ZEROS       0x0000100000000000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_CRC4_MULTIFRAME           0x0000080000000000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_ALIGN_FRAME               0x0000040000000000LL

/* Rx Latched status register 3 T1 mode */
#define PQ_MDS_T1_EV_RX_T1LOSS_OF_RECEIVE_CLOCK_CLEAR     0x0000020000000000LL
#define PQ_MDS_T1_EV_RX_SPARE_CODE_CLEAR                  0x0000010000000000LL
#define PQ_MDS_T1_EV_RX_LOOP_DOWN_CODE_CLEAR              0x0000008000000000LL
#define PQ_MDS_T1_EV_RX_LOOP_UP_CODE_CLEAR                0x0000004000000000LL
#define PQ_MDS_T1_EV_RX_T1LOSS_OF_RECEIVE_CLOCK_DETECT    0x0000002000000000LL
#define PQ_MDS_T1_EV_RX_SPARE_CODE_DETECT                 0x0000001000000000LL
#define PQ_MDS_T1_EV_RX_LOOP_DOWN_CODE_DETECT             0x0000000800000000LL
#define PQ_MDS_T1_EV_RX_LOOP_UP_CODE_DETECT               0x0000000400000000LL

/* Rx Latched status register 3 E1 mode */
#define PQ_MDS_T1_EV_RX_E1LOSS_OF_RECEIVE_CLOCK_CLEAR     0x0000000200000000LL
#define PQ_MDS_T1_EV_RX_V52_LINK_DETECTED_CLEAR           0x0000000100000000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_DISTANT_ALARM_CLEAR       0x0000000080000000LL
#define PQ_MDS_T1_EV_RX_E1LOSS_OF_RECEIVE_CLOCK_DETECT    0x0000000040000000LL
#define PQ_MDS_T1_EV_RX_V52_LINK_DETECTED_DETECT          0x0000000020000000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_DISTANT_ALARM_DETECT      0x0000000010000000LL

/* Rx Latched status register 4 */
#define PQ_MDS_T1_EV_RX_ELASTIC_STORE_FULL                0x0000000008000000LL
#define PQ_MDS_T1_EV_RX_ELASTIC_STORE_EMPTY               0x0000000004000000LL
#define PQ_MDS_T1_EV_RX_ELASTIC_STORE_SLIP                0x0000000002000000LL
#define PQ_MDS_T1_EV_RX_CHANGE_OF_STATE                   0x0000000001000000LL
#define PQ_MDS_T1_EV_RX_ONE_SECOND_TIMER                  0x0000000000800000LL
#define PQ_MDS_T1_EV_RX_TIMER_EVENT                       0x0000000000400000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_MULTIFRAME                0x0000000000200000LL

/* Rx Latched status register 5(HDLC) */
#define PQ_MDS_T1_EV_RX_HDLC_FIFO_OVERRUN                 0x0000000000100000LL
#define PQ_MDS_T1_EV_RX_HDLC_OPENING_BYTE                 0x0000000000080000LL
#define PQ_MDS_T1_EV_RX_HDLC_PACKET_END                   0x0000000000040000LL
#define PQ_MDS_T1_EV_RX_HDLC_PACKET_START                 0x0000000000020000LL
#define PQ_MDS_T1_EV_RX_HDLC_FIFO_ABOVE_HIGH              0x0000000000010000LL
#define PQ_MDS_T1_EV_RX_HDLC_FIFO_NOT_EMPTY               0x0000000000008000LL

/* Rx Latched status register 7 T1 mode */
#define PQ_MDS_T1_EV_RX_RECEIVE_RAICI_DETECT              0x0000000000004000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_AISCI_DETECT              0x0000000000002000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_SLC96_ALIGNMENT           0x0000000000001000LL
#define PQ_MDS_T1_EV_RX_RECEIVE_FDL_REGISTER_FULL         0x0000000000000800LL
#define PQ_MDS_T1_EV_RX_BOC_CLEAR                         0x0000000000000400LL
#define PQ_MDS_T1_EV_RX_BOC_DETECT                        0x0000000000000200LL

/* Rx Latched status register 7 E1 mode */
#define PQ_MDS_T1_EV_RX_SA6_CODEWORD_DETECT               0x0000000000000100LL
#define PQ_MDS_T1_EV_RX_SAX_BIT_CHANGE_DETECT             0x0000000000000080LL

/* LIU Latched status register */
#define PQ_MDS_T1_EV_LIU_JITTER_ATENUATOR_LIMIT_CLEAR     0x80
#define PQ_MDS_T1_EV_LIU_OPEN_CIRCUIT_CLEAR               0x40
#define PQ_MDS_T1_EV_LIU_SHORT_CIRCUIT_CLEAR              0x20
#define PQ_MDS_T1_EV_LIU_LOSS_OF_SIGNAL_CLEAR             0x10
#define PQ_MDS_T1_EV_LIU_JITTER_ATENUATOR_LIMIT_DETECT    0x08
#define PQ_MDS_T1_EV_LIU_OPEN_CIRCUIT_DETECT              0x04
#define PQ_MDS_T1_EV_LIU_SHORT_CIRCUIT_DETECT             0x02
#define PQ_MDS_T1_EV_LIU_LOSS_OF_SIGNAL_DETECT            0x01

typedef struct
{
    t_PqMdsT1FramerTxEvent  txEvents;
    t_PqMdsT1FramerRxEvent  rxEvents;
    t_PqMdsT1FramerLiuEvent liuEvents;
} t_PqMdsT1FramerEvents;

/**************************************************************************//**
 @Description  DS26528 framer link call back typedef
*//***************************************************************************/
/** call back needed to be used when configure the PHY link structure*/
typedef void (* t_PqMdsT1EvCallback)(t_Handle h_App, uint8_t pmcId, uint8_t linkId, t_PqMdsT1FramerEvents *p_Events);

typedef enum
{
    e_NORMAL = 0,
    e_FRAMER_LB,
    e_ALTERA_LB
} e_PqMdsT1Diag;


typedef struct
{
    uint8_t                 linkId;
} t_PqMdsT1SphyParam;

typedef struct
{
    uint8_t                 numOfTDMs;
    uint8_t                 numOfLinks;
} t_PqMdsT1MphyParam;


typedef struct
{
    int                     irq;
    t_PqMdsT1FramerEvents   events;
    t_PqMdsT1EvCallback     f_Isr;
    t_Handle                h_App;
    e_LineRate              mode;
    e_TdmFramingMode        framing;
    e_LineEncoding          encoding;
    e_PqMdsT1Diag           diag;
} t_PqMdsT1Param;

typedef enum
{
    e_PQMDS16T1_SIGNAL_ROUTE_8360 = 0x1,
    e_PQMDS16T1_SIGNAL_ROUTE_8568 = 0x2,
    e_PQMDS16T1_SIGNAL_ROUTE_8569 = 0x3,
    e_PQMDS16T1_SIGNAL_ROUTE_1021 = 0x4
} e_PqMds16T1SignalRoute;


bool PQMDST1_IsConnected(uint32_t baseAddr);

t_Error PQMDST1_InitFramer( uint32_t        baseAddr,
                            uint8_t         linkId,
                            t_PqMdsT1Param  *p_PqMdsT1Param );

t_Error PQMDST1_FreeFramer(uint8_t linkId);

t_Error PQMDST1_EnableIntr(uint8_t linkId);

t_Error PQMDST1_DisableIntr(uint8_t linkId);

bool PQMDS16T1_IsConnected(uint32_t baseAddr, e_PqMds16T1SignalRoute signalRoute);

t_Error PQMDS16T1_InitFramer (uint32_t               alteraBasrAddr,
                              uint8_t                framerId,
                              uint8_t                firstLinkId,
                              uint8_t                numOfLinks,
                              t_PqMdsT1Param         *p_PqMdsT1Param,
                              e_PqMds16T1SignalRoute signalRoute);

t_Error PQMDS16T1_FreeFramer (uint8_t         framerId,
                              uint8_t         firstLinkId,
                              uint8_t         numOfLinks);

t_Error PQMDS16T1_EnableIntr(uint8_t framerId, uint8_t firstlinkId);

t_Error PQMDS16T1_DisableIntr(uint8_t framerId, uint8_t firstlinkId);

t_Error PQMDST1_InitMPhysFramer( uint32_t       baseAddr,
                                 uint8_t        id,
                                 uint8_t        numOfTDMs,
                                 uint8_t        numOfLinks,
                                 t_PqMdsT1Param *p_PqMdsT1Param );

t_Error PQMDST1_FreeMPhysFramer(uint8_t id);

t_Error PQMDST1_EnableMPhysIntr(uint8_t id);

t_Error PQMDST1_DisableMPhysIntr(uint8_t id);

t_Error PQMDS16T1_InitDs3Framer (uint32_t               baseAddr,
                                 t_PqMdsT1Param         *p_PqMdsT1Param,
                                 e_PqMds16T1SignalRoute signalRoute,
                                 uint8_t                framerId,
                                 uint8_t                chId);

#endif /* __PQMDST1_EXT_H__ */
