/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          rdb_1020_ext.h

 @Description   Prototypes, externals and typedefs for RDB1020 routines
*//***************************************************************************/
#ifndef __RDB_1020_EXT_H
#define __RDB_1020_EXT_H

#include "enet_ext.h"
#include "mii_mng_ext.h"


/**************************************************************************//**
 @Group         rdb1020_id_grp

 @Description   RDB1020 definitions and enums.

 @{
*//***************************************************************************/

#define RDB1020_NUM_OF_TDMS             4
#define RDB1020_NUM_OF_I2C_PORTS        2
#define RDB1020_NUM_OF_SERIAL_PORTS     e_RDB1020_SERIAL_PORT_DUMMY_LAST
#define RDB1020_NUM_OF_SPI_PORTS        1

/**************************************************************************//**
 @Description   P1020RDB Memory Regions
*//***************************************************************************/
typedef enum e_Rdb1020MemRegion
{
    e_RDB1020_MEM_RGN_DDR,
    e_RDB1020_MEM_RGN_PEX1,
    e_RDB1020_MEM_RGN_PEX2,
    e_RDB1020_MEM_RGN_CCSR,
    e_RDB1020_MEM_RGN_PEX_IO,
    e_RDB1020_MEM_RGN_L2_SRAM,
    e_RDB1020_MEM_RGN_NAND_FLASH,
    e_RDB1020_MEM_RGN_VITESSE,
    e_RDB1020_MEM_RGN_NOR_FLASH

} e_Rdb1020MemRegion;


/**************************************************************************//**
 @Description   MPC1020RDB Memory Regions
*//***************************************************************************/
typedef struct t_Rdb1020MemRegionInfo
{
    uint32_t    startAddr;
    uint32_t    size;
    uint8_t     chipSelect;

} t_Rdb1020MemRegionInfo;

/**************************************************************************//**
 @Description   Ethernet PHY selection
*//***************************************************************************/
typedef enum e_Rdb1020EthPort
{
    e_RDB1020_ETH_PORT_ETSEC_1 = 0,
    e_RDB1020_ETH_PORT_ETSEC_2,
    e_RDB1020_ETH_PORT_ETSEC_3,
    e_RDB1020_ETH_PORT_DUMMY_LAST

} e_Rdb1020EthPort;

typedef enum e_Rdb1020Led
{
   e_RDB1020_LED_GREEN = 0,
   e_RDB1020_LED_YELLOW,
   e_RDB1020_LED_RED

} e_Rdb1020Led;

typedef enum e_Rdb1020SerialPort
{
    e_RDB1020_SERIAL_PORT_DUART_1 = 0,
    e_RDB1020_SERIAL_PORT_DUART_2,
    e_RDB1020_SERIAL_PORT_UCC_UART_7,
    e_RDB1020_SERIAL_PORT_DUMMY_LAST

} e_Rdb1020SerialPort;

typedef enum e_Rdb1020SpiPort
{
    e_RDB1020_ESPI_PORT = 0

} e_Rdb1020SpiPort;

typedef enum e_Rdb1020IoGroup
{
    e_RDB1020_IO_GRP_TDM,
    e_RDB1020_IO_GRP_SDHC,
    e_RDB1020_IO_GRP_TSEC1,
    e_RDB1020_IO_GRP_TSEC2,
    e_RDB1020_IO_GRP_TSEC3,
    e_RDB1020_IO_GRP_USB0,
    e_RDB1020_IO_GRP_USB1

} e_Rdb1020IoGroup;

t_Handle RDB1020_ConfigAndInit(void);

t_Error RDB1020_Free(t_Handle h_Rdb1020);

t_Error RDB1020_InitEthernetPhy(t_Handle            h_Rdb1020,
                                e_Rdb1020EthPort    ethPort,
                                t_Handle            h_MiiAccess,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                loopbackConnector,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg);

e_EnetMode RDB1020_FreeEthernetPhy(t_Handle         h_Rdb1020,
                                   e_Rdb1020EthPort ethPort);

bool RDB1020_GetEthernetLinkStatus(t_Handle             h_Rdb1020,
                                   e_Rdb1020EthPort     ethPort,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg);

t_Error RDB1020_InitSerialPort(t_Handle h_Rdb1020, e_Rdb1020SerialPort serialPort);

t_Error RDB1020_FreeSerialPort(t_Handle h_Rdb1020, e_Rdb1020SerialPort serialPortId);

t_Error RDB1020_InitSpiPort(t_Handle h_Rdb1020, e_Rdb1020SpiPort spiPort);

t_Error RDB1020_FreeSpiPort(t_Handle h_Rdb1020, e_Rdb1020SpiPort spiPort);

t_Error RDB1020_InitI2cPort(t_Handle h_Rdb1020, uint8_t i2cPortId);

t_Error RDB1020_FreeI2cPort(t_Handle h_Rdb1020, uint8_t i2cPortId);

t_Error RDB1020_InitTdmPort(t_Handle h_Rdb);

t_Error RDB1020_InitTdmPhy(t_Handle             h_Rdb,
                           uint8_t              phyId);

t_Error RDB1020_EnableTdmPhyIntr(t_Handle h_Rdb, uint8_t phyId);

t_Error RDB1020_FreeTdmPhy(t_Handle     h_Rdb,
                           uint8_t      phyId);

t_Error RDB1020_SetLed(t_Handle h_Rdb1020, e_Rdb1020Led led, bool ledOn);


t_Error RDB1020_GetIoGroupPins(t_Handle         h_Rdb1020,
                               e_Rdb1020IoGroup ioGroup,
                               int              **p_Pins,
                               uint16_t         *p_NumOfPins);

t_Error RDB1020_GetIoGroupInputClockPins(t_Handle           h_Rdb1020,
                                         e_Rdb1020IoGroup   ioGroup,
                                         int                *p_RxClkPin,
                                         int                *p_TxClkPin);

t_Error RDB1020_GetMemRegionInfo(e_Rdb1020MemRegion     memRegion,
                                 t_Rdb1020MemRegionInfo *p_MemRegionInfo);


/** @} *//* rdb1020_id_grp */

#endif /* __RDB_1020_EXT_H */
