/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "enet_ext.h"
#include "xx_ext.h"

#include "vsc8221_ext.h"
#include "vsc82X4.h"


#define __ERR_MODULE__ MODULE_UNKNOWN



/********************************************************************************/
t_Error VSC8221_Init(uint8_t           phyAddr,
                     e_EnetMode        ethMode,
                     bool              autoNegotiation,
                     e_EnetDuplexMode  duplexMode,
                     e_DiagMode        loopbackMode,
                     t_Handle          h_MiiAccess,
                     t_MiiPhyWriteFunc *f_SetPhyReg,
                     t_MiiPhyReadFunc  *f_GetPhyReg)
{
    uint16_t    val;

    if (f_GetPhyReg(h_MiiAccess, phyAddr, 0, &val) != E_OK)
    {
        REPORT_ERROR(MINOR, E_NO_DEVICE, ("PHY %x not found!", phyAddr));
        return E_OK;
    }


    f_SetPhyReg(h_MiiAccess, phyAddr, 0, 0x8000);

    XX_UDelay(1000000);

    val = 0;
    if (loopbackMode == e_DIAG_MODE_PHY_LOOPBACK)
        val = 0x4000;

    if (autoNegotiation)
        val |= 0x1000;

    if(duplexMode == e_ENET_FULL_DUPLEX)
        val |= 0x100;

    if(ethMode == e_ENET_MODE_SGMII_100)
        val |= 0x2000;
    else if(ethMode == e_ENET_MODE_SGMII_1000)
        val |= 0x40;

    f_SetPhyReg(h_MiiAccess, phyAddr, 0, val);


    XX_UDelay(100000);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
    {
        uint8_t     reg;
        uint16_t    val;

        XX_Print("\tPHY %d regs:\n", phyAddr);
        for (reg=0; reg < 25; reg++)
        {
            f_GetPhyReg(h_MiiAccess, phyAddr, reg, &val);
            XX_Print("Reg %2d, Value = 0x%04x\n", reg, val);
        }
    }
#endif /* (defined(DEBUG_ERRORS) && ... */

    return E_OK;
}
