/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "enet_ext.h"
#include "xx_ext.h"
#include "vsc8244_ext.h"

#include "vsc82X4.h"


#define __ERR_MODULE__ MODULE_UNKNOWN



/********************************************************************************/
t_Error VSC8244_Init(uint8_t           phyAddr,
                     e_EnetMode        ethMode,
                     bool              autoNegotiation,
                     e_EnetDuplexMode  duplexMode,
                     bool              forceMaster,
                     e_DiagMode        loopbackMode,
                     bool              addTxOutputDelay,   /* Relevant fot RGMII/RTBI only */
                     bool              addRxOutputDelay,   /* Relevant fot RGMII/RTBI only */
                     t_Handle          h_MiiAccess,
                     t_MiiPhyWriteFunc *f_SetPhyReg,
                     t_MiiPhyReadFunc  *f_GetPhyReg)
{
    uint16_t    reg16;
    uint32_t    phyId, speed = ENET_SPEED_FROM_MODE(ethMode);
    int         timeout = 5000; /* in mili-sces */
    uint32_t    startTime;

    UNUSED(addTxOutputDelay);
    UNUSED(addRxOutputDelay);

    if (f_GetPhyReg(h_MiiAccess, phyAddr, MII_82X4_ModeControl, &reg16) != E_OK)
        RETURN_ERROR(MINOR, E_NO_DEVICE, ("PHY %x not found!", phyAddr));

    /* Grab the bits from PHYIR1, and put them in the upper half */
    f_GetPhyReg(h_MiiAccess, phyAddr, MII_82X4_PHY_IdReg1, &reg16);
    phyId = (reg16 & 0xffff) << 16;

    /* Grab the bits from PHYIR2, and put them in the lower half */
    f_GetPhyReg(h_MiiAccess, phyAddr, MII_82X4_PHY_IdReg2, &reg16);
    phyId |= (reg16 & 0xffff);

    DBG(TRACE, ("VSC8244: PHY-ID is %x\n", phyId));

    f_SetPhyReg(h_MiiAccess, phyAddr, MII_82X4_ModeControl, BMCR_RESET);
    XX_UDelay(200000);

    reg16 = BBCR_MASTER_CONFIG_ENABLE | BBCR_MASTER_CONFIG_VALUE;
    if ((speed == e_ENET_SPEED_1000) && forceMaster)
        f_SetPhyReg(h_MiiAccess, phyAddr, MII_82X4_1000BaseTControl, reg16);

    reg16 = 0;
    if (speed == e_ENET_SPEED_1000)
        reg16 |= BMCR_SPEED_SELECT_1000;
    else if(speed == e_ENET_SPEED_100)
            reg16 |= BMCR_SPEED_SELECT_100;

    if(autoNegotiation)
        reg16 |= BMCR_AUTO_NEG_ENABLE;

    if(duplexMode == e_ENET_FULL_DUPLEX)
        reg16 |= BMCR_FULL_DUPLEX;

    if(loopbackMode != e_DIAG_MODE_NONE)
        reg16 |= BMCR_LOOPBACK;

    f_SetPhyReg(h_MiiAccess, phyAddr, MII_82X4_ModeControl, reg16);

    startTime = XX_CurrentTime();
    do
    {
        f_GetPhyReg(h_MiiAccess, phyAddr, MII_82X4_ModeStatus, &reg16);
        if (((loopbackMode == e_DIAG_MODE_PHY_LOOPBACK) || (loopbackMode == e_DIAG_MODE_EXT_LOOPBACK)) &&
            (reg16 & BMSR_LINK_STATUS))
            return E_OK;
        else if (reg16 & (BMSR_LINK_STATUS | BMSR_AUTO_NEG_COMPLETED))
            return E_OK;
    } while ((XX_CurrentTime() - startTime) < timeout);
    REPORT_ERROR(MINOR, E_TIMEOUT, ("No link on PHY %x!", phyAddr));

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
    {
        uint8_t     reg;
        uint16_t    val;

        XX_Print("\tPHY %d regs:\n", phyAddr);
        for (reg=0; reg < 25; reg++)
        {
            f_GetPhyReg(h_MiiAccess, phyAddr, reg, &val);
            XX_Print("Reg %2d, Value = 0x%04x\n", reg, val);
        }
    }
#endif /* (defined(DEBUG_ERRORS) && ... */

    return E_OK;
}
