/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
@File          pg_ext.h

@Description   Packet Generator data structures and definitions
*//***************************************************************************/

#ifndef __PG_EXT_H
#define __PG_EXT_H

#include "std_ext.h"
#include "part_ext.h"
#include "list_ext.h"
#include "buf_ext.h"
#include "itg_common_ext.h"

/**************************************************************************//**
 @Group         itg_package_grp Internal Traffic Generator

 @Description   Internal Traffic Generator routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pg_grp Packet Generator Application Programming Interface

 @Description   Packet Generator defines, structures and functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection   PG Defines

               General Packet Generator defines

 @{
*//***************************************************************************/
#define MAX_VLAN_TAG                            6   /**< Maximum number of VLAN tags */
#define MAX_MPLS_HEADER                         25  /**< Maximum number of MPLS headers */
#define MAX_PADDING_HEADER                      4   /**< Maximum number of Padding headers */
#define MAX_LLC_HEADER                          2   /**< Maximum number of LLC headers */
#define MAX_SNAP_HEADER                         2   /**< Maximum number of SNAP headers */
#define MAX_HEADER                              6   /**< Maximum number of headers of same type*/
#define L2TP_V3_COOKIE_SIZE                     8   /**< L2TPV3 Data Message cookie size */

#define ETH_ADDR_SIZE                           6   /**< Ethernet address size */
#define IPv6_ADDR_SIZE                          16  /**< IPv6 address size */
#define MAX_RTP_SRC_ID                          16  /**< Maximum number of RTP source identifiers */
#define RTP_EXTN_HEADER_IDENTIFIER_SIZE         4   /**< RTP extension header profile and length fields */
#define IEEE1588_V1_SUB_DOMIAN_NUM_SIZE         16  /**< Maximum bytes of Sub domain number */
#define IEEE1588_V1_SOURCE_UUID_SIZE            6   /**< Maximum bytes of Source UID */
#define IEEE1588_V2_CORRECTION_FIELD_SIZE       8   /**< Maximum bytes of Correction Field */
#define IEEE1588_V2_CLOCK_IDENTITY_SIZE         8   /**< Maximum bytes of Clock Identity */

#define MAX_NUM_OF_ENCAPSULATIONS               30  /**< Maximum number of header encapsulations */
#define MAX_FIXED_PATTERN_PAYLOAD_BYTES         4   /**< Maximum bytes of fixed pattern */
#define IP_MAX_OPTIONS_COUNT                    16  /**< Maximum number of IPv4 options */
#define MAX_TCP_OPTS_BYTES                      16  /**< Maximum number of TCP options */
#define MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER      8   /**< Maximum number of 32 bit words in the RTP extension header */

/** @} */

/**************************************************************************//**
 @Description   Ethernet Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_ETH_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify Ethernet next header type       */
    e_FIELD_ETH_VALID_FIELDS        = 1                                 , /**< Flag to identify the configured Ethernet fields  */
    e_FIELD_ETH_DA                  = e_FIELD_ETH_VALID_FIELDS << 1     , /**< Flag to identify source address field            */
    e_FIELD_ETH_SA                  = e_FIELD_ETH_DA << 1               , /**< Flag to identify destination address field       */
    e_FIELD_ETH_LENGTH              = e_FIELD_ETH_SA << 1               , /**< Flag to identify length field                    */
    e_FIELD_ETH_TYPE                = e_FIELD_ETH_LENGTH << 1           , /**< Flag to identify type field                      */
    e_FIELD_ETH_USE_FINAL_CKSUM     = e_FIELD_ETH_TYPE << 1             , /**< Flag to identify whether to use FCS field        */
    e_FIELD_ETH_FINAL_CKSUM         = e_FIELD_ETH_USE_FINAL_CKSUM << 1  , /**< Flag to identify FCS field                       */
    e_FIELD_ETH_PADDING             = e_FIELD_ETH_FINAL_CKSUM << 1      , /**< Flag to identify padding field                   */
    e_FIELD_ETH_ALL_FIELDS          = (e_FIELD_ETH_PADDING << 1) - 1      /**< Flag to identify all Ethernet fields             */
} e_PgFieldsEth;

/**************************************************************************//**
 @Description  VLAN Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_VLAN_NEXT_HEADER_TYPE   = 0                                 , /**< Flag to identify VLAN next header type               */
    e_FIELD_VLAN_VALID_FIELDS       = 1                                 , /**< Flag to identify the configured VLAN fields          */
    e_FIELD_VLAN_VPRI               = e_FIELD_VLAN_VALID_FIELDS << 1    , /**< Flag to identify VLAN priority field                 */
    e_FIELD_VLAN_CFI                = e_FIELD_VLAN_VPRI << 1            , /**< Flag to identify canonical Format Indicator field    */
    e_FIELD_VLAN_VID                = e_FIELD_VLAN_CFI << 1             , /**< Flag to identify VLAN id field                       */
    e_FIELD_VLAN_LENGTH             = e_FIELD_VLAN_VID << 1             , /**< Flag to identify length field                        */
    e_FIELD_VLAN_TYPE               = e_FIELD_VLAN_LENGTH << 1          , /**< Flag to identify type field                          */
    e_FIELD_VLAN_ALL_FIELDS         = (e_FIELD_VLAN_TYPE << 1) - 1        /**< Flag to identify all VLAN fields                     */
} e_PgFieldsVlan;

/**************************************************************************//**
 @Description   LLC Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_LLC_NEXT_HEADER_TYPE    = 0                             , /**< Flag to identify LLC next header type                    */
    e_FIELD_LLC_VALID_FIELDS        = 1                             , /**< Flag to identify the configured LLC fields               */
    e_FIELD_LLC_DSAP                = e_FIELD_LLC_VALID_FIELDS << 1 , /**< Flag to identify destination Service Access Point field  */
    e_FIELD_LLC_SSAP                = e_FIELD_LLC_DSAP << 1         , /**< Flag to identify source Service Access Point field       */
    e_FIELD_LLC_CTRL                = e_FIELD_LLC_SSAP << 1         , /**< Flag to identify control field                           */
    e_FIELD_LLC_ALL_FIELDS          = (e_FIELD_LLC_CTRL << 1) - 1     /**< Flag to identify all LLC fields                          */
} e_PgFieldsLlc;

/**************************************************************************//**
 @Description   NLP Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_NLPID_NEXT_HEADER_TYPE  = 0                                 , /**< Flag to identify VLAN next header type                   */
    e_FIELD_NLPID_VALID_FIELDS      = 1                                 , /**< Flag to identify the configured NLPID fields             */
    e_FIELD_NLPID_NLPID             = e_FIELD_NLPID_VALID_FIELDS << 1   , /**< Flag to identify network layer protocol identifier field */
    e_FIELD_NLPID_ALL_FIELDS        = (e_FIELD_NLPID_NLPID << 1) - 1      /**< Flag to identify all VLAN fields                         */
} e_PgFieldNlpid;

/**************************************************************************//**
 @Description   SNAP Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_SNAP_NEXT_HEADER_TYPE   = 0                             , /**< Flag to identify SNAP next header type                       */
    e_FIELD_SNAP_VALID_FIELDS       = 1                             , /**< Flag to identify the configured SNAP fields                  */
    e_FIELD_SNAP_OUI                = e_FIELD_SNAP_VALID_FIELDS << 1, /**< Flag to identify organizationally unique identifier field    */
    e_FIELD_SNAP_PID                = e_FIELD_SNAP_OUI << 1         , /**< Flag to identify protocol Identifier field                   */
    e_FIELD_SNAP_ALL_FIELDS         = (e_FIELD_SNAP_PID << 1) - 1     /**< Flag to identify all SNAP fields                             */
} e_PgFieldSnap;

/**************************************************************************//**
 @Description   PPPoE Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_PPPoE_NEXT_HEADER_TYPE  = 0                                 , /**< Flag to identify PPPoE next header type                          */
    e_FIELD_PPPoE_VALID_FIELDS      = 1                                 , /**< Flag to identify the configured PPPoE fields                     */
    e_FIELD_PPPoE_VER               = e_FIELD_PPPoE_VALID_FIELDS << 1   , /**< Flag to identify 4 bits - version field                          */
    e_FIELD_PPPoE_TYPE              = e_FIELD_PPPoE_VER << 1            , /**< Flag to identify 4 bits - type field                             */
    e_FIELD_PPPoE_CODE              = e_FIELD_PPPoE_TYPE << 1           , /**< Flag to identify code field                                      */
    e_FIELD_PPPoE_SID               = e_FIELD_PPPoE_CODE << 1           , /**< Flag to identify session identifier field                        */
    e_FIELD_PPPoE_LEN               = e_FIELD_PPPoE_SID << 1            , /**< Flag to identify length field                                    */
    e_FIELD_PPPoE_SESSION           = e_FIELD_PPPoE_LEN << 1            , /**< Flag to identify enable/disable status of PPPoE session field    */
    e_FIELD_PPPoE_ALL_FIELDS        = (e_FIELD_PPPoE_SESSION << 1) - 1    /**< Flag to identify all PPPoE fields                                */
} e_PgFieldsPppoe;

/**************************************************************************//**
 @Description   HDLC Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_HDLC_NEXT_HEADER_TYPE   = 0                                 , /**< Flag to identify HDLC next header type                           */
    e_FIELD_HDLC_VALID_FIELDS       = 1                                 , /**< Flag to identify the configured HDLC fields                      */
    e_FIELD_HDLC_ADDRESS            = e_FIELD_HDLC_VALID_FIELDS << 1    , /**< Flag to identify address field                                   */
    e_FIELD_HDLC_ADDRESS_TYPE       = e_FIELD_HDLC_ADDRESS << 1         , /**< Flag to identify the address type (8/16 bit)                     */
    e_FIELD_HDLC_CONTROL            = e_FIELD_HDLC_ADDRESS_TYPE << 1    , /**< Flag to identify control field                                   */
    e_FIELD_HDLC_USE_FCS            = e_FIELD_HDLC_CONTROL << 1         , /**< Flag to identify enable/disable status of FCS calculation field  */
    e_FIELD_HDLC_FCS                = e_FIELD_HDLC_USE_FCS << 1         , /**< Flag to identify FCS field                                       */
    e_FIELD_HDLC_FCS_TYPE           = e_FIELD_HDLC_FCS << 1             , /**< Flag to identify 16/32 bit FCS type field                        */
    e_FIELD_HDLC_ALL_FIELDS         = (e_FIELD_HDLC_FCS_TYPE << 1) - 1    /**< Flag to identify all HDLC fields                                 */
} e_PgFieldsHdlc;

/**************************************************************************//**
 @Description   PPP Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_PPP_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify VLAN next header type                           */
    e_FIELD_PPP_VALID_FIELDS        = 1                                 , /**< Flag to identify the configured PPP fields                       */
    e_FIELD_PPP_TYPE                = e_FIELD_PPP_VALID_FIELDS << 1     , /**< Flag to identify type field                                      */
    e_FIELD_PPP_COMPRESSED          = e_FIELD_PPP_TYPE << 1             , /**< Flag to identify enable/disable status of PPP compression field  */
    e_FIELD_PPP_ALL_FIELDS          = (e_FIELD_PPP_COMPRESSED << 1) - 1   /**< Flag to identify all VLAN fields                                 */
} e_PgFieldsPpp;

/**************************************************************************//**
 @Description   ML/MC Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_PPP_MLMC_NEXT_HEADER_TYPE   = 0                                         , /**< Flag to identify ML/MC next header type                  */
    e_FIELD_PPP_MLMC_VALID_FIELDS       = 1                                         , /**< Flag to identify the configured PPP MLMC fields          */
    e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT     = e_FIELD_PPP_MLMC_VALID_FIELDS << 1        , /**< Flag to identify first fragment of a PPP packet field    */
    e_FIELD_PPP_MLMC_END_FRAG_BIT       = e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT << 1      , /**< Flag to identify last fragment of a PPP packet field     */
    e_FIELD_PPP_MLMC_CLASS              = e_FIELD_PPP_MLMC_END_FRAG_BIT << 1        , /**< Flag to identify class identifier field                  */
    e_FIELD_PPP_MLMC_SEQ_NUM            = e_FIELD_PPP_MLMC_CLASS << 1               , /**< Flag to identify sequence number of each fragment field  */
    e_FIELD_PPP_MLMC_PID                = e_FIELD_PPP_MLMC_SEQ_NUM << 1             , /**< Flag to identify protocol identifier field               */
    e_FIELD_PPP_MLMC_SEQ_NUM_TYPE       = e_FIELD_PPP_MLMC_PID << 1                 , /**< Flag to identify sequence number type field              */
    e_FIELD_PPP_MLMC_ALL_FIELDS         = (e_FIELD_PPP_MLMC_SEQ_NUM_TYPE << 1) - 1    /**< Flag to identify all ML/MC fields                        */
} e_PgFieldsPppMlmc;

/**************************************************************************//**
 @Description   AAL0 Cell Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_AAL0_CELL_NEXT_HEADER_TYPE  = 0                                 , /**< Flag to identify AAL0 Cell next header type          */
    e_FIELD_AAL0_VALID_FIELDS           = 1                                 , /**< Flag to identify the configured AAL0 fields          */
    e_FIELD_AAL0_CELL_GFC               = e_FIELD_AAL0_VALID_FIELDS <<1     , /**< Flag to identify generic Flow Control field          */
    e_FIELD_AAL0_CELL_VPI               = e_FIELD_AAL0_CELL_GFC << 1        , /**< Flag to identify virtual Path Identifier field       */
    e_FIELD_AAL0_CELL_VCI               = e_FIELD_AAL0_CELL_VPI << 1        , /**< Flag to identify virtual Channel Identifier field    */
    e_FIELD_AAL0_CELL_PTI               = e_FIELD_AAL0_CELL_VCI << 1        , /**< Flag to identify payload Type Indication field       */
    e_FIELD_AAL0_CELL_CLP               = e_FIELD_AAL0_CELL_PTI << 1        , /**< Flag to identify cell Loss Priority field            */
    e_FIELD_AAL0_CELL_HEC               = e_FIELD_AAL0_CELL_CLP << 1        , /**< Flag to identify header Error Control field          */
    e_FIELD_AAL0_CELL_ALL_FIELDS        = (e_FIELD_AAL0_CELL_HEC << 1) - 1    /**< Flag to identify all AAL0 Cell fields                */
} e_PgFieldsAal0Cell;

/**************************************************************************//**
 @Description   IPv4 Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_IPv4_NEXT_HEADER_TYPE   = 0                                 , /**< Flag to identify IPv4 next header type           */
    e_FIELD_IPv4_VALID_FIELDS       = 1                                 , /**< Flag to identify the configured IPv4 fields      */
    e_FIELD_IPv4_VER                = e_FIELD_IPv4_VALID_FIELDS << 1    , /**< Flag to identify 4 bits - version field          */
    e_FIELD_IPv4_HDR_LEN            = e_FIELD_IPv4_VER << 1             , /**< Flag to identify 4 bits - header length field    */
    e_FIELD_IPv4_TOS                = e_FIELD_IPv4_HDR_LEN << 1         , /**< Flag to identify type Of service field           */
    e_FIELD_IPv4_TOTAL_LEN          = e_FIELD_IPv4_TOS << 1             , /**< Flag to identify total length field              */
    e_FIELD_IPv4_ID                 = e_FIELD_IPv4_TOTAL_LEN << 1       , /**< Flag to identify identification field            */
    e_FIELD_IPv4_FLAG_D             = e_FIELD_IPv4_ID << 1              , /**< Flag to identify 3 bits  - IPv4 flags field      */
    e_FIELD_IPv4_FLAG_M             = e_FIELD_IPv4_FLAG_D << 1          , /**< Flag to identify fragment offset field           */
    e_FIELD_IPv4_OFFSET             = e_FIELD_IPv4_FLAG_M << 1          , /**< Flag to identify fragment offset field           */
    e_FIELD_IPv4_TTL                = e_FIELD_IPv4_OFFSET << 1          , /**< Flag to identify time To live field              */
    e_FIELD_IPv4_PROTO              = e_FIELD_IPv4_TTL << 1             , /**< Flag to identify protocol identifier field       */
    e_FIELD_IPv4_CKSUM              = e_FIELD_IPv4_PROTO << 1           , /**< Flag to identify header checksum field           */
    e_FIELD_IPv4_SRC_IP             = e_FIELD_IPv4_CKSUM << 1           , /**< Flag to identify source IP addressy field        */
    e_FIELD_IPv4_DST_IP             = e_FIELD_IPv4_SRC_IP << 1          , /**< Flag to identify destination IP address field    */
    e_FIELD_IPv4_OPTS               = e_FIELD_IPv4_DST_IP << 1          , /**< Flag to identify option field                    */
    e_FIELD_IPv4_OPTS_COUNT         = e_FIELD_IPv4_OPTS << 1            , /**< Flag to identify number of options field         */
    e_FIELD_IPv4_ALL_FIELDS         = (e_FIELD_IPv4_OPTS_COUNT << 1) - 1  /**< Flag to identify all IPv4 fields                 */
} e_PgFieldsIpv4;

/**************************************************************************//**
 @Description   IPv6 Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_IPv6_NEXT_HEADER_TYPE   = 0                                 , /**< Flag to identify IPv6 next header type           */
    e_FIELD_IPv6_VALID_FIELDS       = 1                                 , /**< Flag to identify the configured IPv6 fields      */
    e_FIELD_IPv6_VER                = e_FIELD_IPv6_VALID_FIELDS << 1    , /**< Flag to identify 4 bits - IP version field       */
    e_FIELD_IPv6_TRAFFIC_CLASS      = e_FIELD_IPv6_VER << 1             , /**< Flag to identify 8 bits - traffic class field    */
    e_FIELD_IPv6_FLOW_LABEL         = e_FIELD_IPv6_TRAFFIC_CLASS << 1   , /**< Flag to identify 20 bits - flow label field      */
    e_FIELD_IPv6_PAYLOAD_LENGTH     = e_FIELD_IPv6_FLOW_LABEL << 1      , /**< Flag to identify length of IPv6 payload field    */
    e_FIELD_IPv6_NEXT_HEADER        = e_FIELD_IPv6_PAYLOAD_LENGTH << 1  , /**< Flag to identify next header type field          */
    e_FIELD_IPv6_HOP_LIMIT          = e_FIELD_IPv6_NEXT_HEADER << 1     , /**< Flag to identify hop limit field                 */
    e_FIELD_IPv6_SOURCE_IP          = e_FIELD_IPv6_HOP_LIMIT << 1       , /**< Flag to identify source IP address field         */
    e_FIELD_IPv6_DEST_IP            = e_FIELD_IPv6_SOURCE_IP << 1       , /**< Flag to identify destination IP address field    */
    e_FIELD_IPv6_ALL_FIELDS         = (e_FIELD_IPv6_DEST_IP << 1) - 1     /**< Flag to identify all IPv6 fields                 */
} e_PgFieldsIpv6;

/**************************************************************************//**
 @Description   TCP Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_TCP_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify TCP next header type                            */
    e_FIELD_TCP_VALID_FIELDS        = 1                                 , /**< Flag to identify the configured TCP fields                       */
    e_FIELD_TCP_PORT_SRC            = e_FIELD_TCP_VALID_FIELDS << 1     , /**< Flag to identify soruce port address field                       */
    e_FIELD_TCP_PORT_DST            = e_FIELD_TCP_PORT_SRC << 1         , /**< Flag to identify destination port address field                  */
    e_FIELD_TCP_SEQ                 = e_FIELD_TCP_PORT_DST << 1         , /**< Flag to identify sequence number field                           */
    e_FIELD_TCP_ACK                 = e_FIELD_TCP_SEQ << 1              , /**< Flag to identify acknowledgement number field                    */
    e_FIELD_TCP_OFFSET              = e_FIELD_TCP_ACK << 1              , /**< Flag to identify 4 bits - Data offset field                      */
    e_FIELD_TCP_FLAGS               = e_FIELD_TCP_OFFSET << 1           , /**< Flag to identify 6 bits - Flags: URG-ACK-PSH-RST-SYN-FIN field   */
    e_FIELD_TCP_WINDOW              = e_FIELD_TCP_FLAGS << 1            , /**< Flag to identify window field                                    */
    e_FIELD_TCP_CHECKSUM            = e_FIELD_TCP_WINDOW << 1           , /**< Flag to identify checksum field                                  */
    e_FIELD_TCP_URGPTR              = e_FIELD_TCP_CHECKSUM << 1         , /**< Flag to identify urgent pointer field                            */
    e_FIELD_TCP_OPTS                = e_FIELD_TCP_URGPTR << 1           , /**< Flag to identify options field                                   */
    e_FIELD_TCP_OPTS_COUNT          = e_FIELD_TCP_OPTS << 1             , /**< Flag to identify number of options field                         */
    e_FIELD_TCP_ALL_FIELDS          = (e_FIELD_TCP_OPTS_COUNT << 1) - 1   /**< Flag to identify all TCP fields                                  */
} e_PgFieldsTcp;

/**************************************************************************//**
 @Description   UDP Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_UDP_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify UDP next header type            */
    e_FIELD_UDP_VALID_FIELDS        = 1                                 , /**< Flag to identify the configured UDP fields       */
    e_FIELD_UDP_PORT_SRC            = e_FIELD_UDP_VALID_FIELDS << 1     , /**< Flag to identify soruce port address field       */
    e_FIELD_UDP_PORT_DST            = e_FIELD_UDP_PORT_SRC << 1         , /**< Flag to identify destination port address field  */
    e_FIELD_UDP_LEN                 = e_FIELD_UDP_PORT_DST << 1         , /**< Flag to identify datagram length field           */
    e_FIELD_UDP_CHECKSUM            = e_FIELD_UDP_LEN << 1              , /**< Flag to identify UDP Checksum field              */
    e_FIELD_UDP_ALL_FIELDS          = (e_FIELD_UDP_CHECKSUM << 1) - 1     /**< Flag to identify all UDP fields                  */
} e_PgFieldsUdp;

/**************************************************************************//**
 @Description   SCTP Fields
*//***************************************************************************/
 typedef enum {
     e_FIELD_SCTP_NEXT_HEADER_TYPE   = 0                                    , /**< Flag to identify SCTP next header type           */
     e_FIELD_SCTP_VALID_FIELDS       = 1                                    , /**< Flag to identify the configured SCTP fields      */
     e_FIELD_SCTP_PORT_SRC           = e_FIELD_SCTP_VALID_FIELDS << 1       , /**< Flag to identify source port address field       */
     e_FIELD_SCTP_PORT_DST           = e_FIELD_SCTP_PORT_SRC     << 1       , /**< Flag to identify destination port address field  */
     e_FIELD_SCTP_VER_TAG            = e_FIELD_SCTP_PORT_DST     << 1       , /**< Flag to identify SCTP verification tag field     */
     e_FIELD_SCTP_CKSUM              = e_FIELD_SCTP_VER_TAG      << 1       , /**< Flag to identify checksum fild                   */
     e_FIELD_SCTP_ALL_FIELDS         =(e_FIELD_SCTP_CKSUM        << 1) - 1    /**< Flag to identify all SCTP fields                 */
 } e_FieldsSctp;

/**************************************************************************//**
 @Description   SCTP Chunk Fields
*//***************************************************************************/
 typedef enum {
     e_FIELD_SCTP_CHUNK_DATA_NEXT_HEADER_TYPE     = 0                                           , /**< Flag to identify SCTP chunk next header type         */
     e_FIELD_SCTP_CHUNK_DATA_VALID_FIELDS         = 1                                           , /**< Flag to identify the configured SCTP chunk fields    */
     e_FIELD_SCTP_CHUNK_DATA_TYPE                 = e_FIELD_SCTP_CHUNK_DATA_VALID_FIELDS << 1   , /**< Flag to identify SCTP chunk data type fields         */
     e_FIELD_SCTP_CHUNK_DATA_FLAGS                = e_FIELD_SCTP_CHUNK_DATA_TYPE        << 1    , /**< Flag to identify flag field                          */
     e_FIELD_SCTP_CHUNK_DATA_LENGTH               = e_FIELD_SCTP_CHUNK_DATA_FLAGS       << 1    , /**< Flag to identify length field                        */
     e_FIELD_SCTP_CHUNK_DATA_TSN                  = e_FIELD_SCTP_CHUNK_DATA_LENGTH      << 1    , /**< Flag to identify Transmission Sequence Number field  */
     e_FIELD_SCTP_CHUNK_DATA_STREAM_ID            = e_FIELD_SCTP_CHUNK_DATA_TSN         << 1    , /**< Flag to identify stream id field                     */
     e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN           = e_FIELD_SCTP_CHUNK_DATA_STREAM_ID   << 1    , /**< Flag to identify stream sequence number field        */
     e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID          = e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN  << 1    , /**< Flag to identify payload pid field                   */
     e_FIELD_SCTP_CHUNK_DATA_UNORDERED            = e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID << 1    , /**< Flag to identify SCTP chunk unordered field          */
     e_FIELD_SCTP_CHUNK_DATA_BEGGINING            = e_FIELD_SCTP_CHUNK_DATA_UNORDERED   << 1    , /**< Flag to identify SCTP chunk beginning field          */
     e_FIELD_SCTP_CHUNK_DATA_END                  = e_FIELD_SCTP_CHUNK_DATA_BEGGINING   << 1    , /**< Flag to identify SCTP chunk end field                */
     e_FIELD_SCTP_CHUNK_DATA_ALL_FIELDS           = (e_FIELD_SCTP_CHUNK_DATA_END        << 1) - 1 /**< Flag to identify all SCTP chunk fields               */
 } e_FieldsSctpChunkData;

/**************************************************************************//**
 @Description   IPHC Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_IPHC_NEXT_HEADER_TYPE   = 0                                 , /**< Flag to identify IPHC next header type           */
    e_FIELD_IPHC_VALID_FIELDS       = 1                                 , /**< Flag to identify the configured IPHC fields      */
    e_FIELD_IPHC_CID                = e_FIELD_IPHC_VALID_FIELDS << 1    , /**< Flag to identify context identifier field        */
    e_FIELD_IPHC_CID_TYPE           = e_FIELD_IPHC_CID << 1             , /**< Flag to identify context type field              */
    e_FIELD_IPHC_HCINDEX            = e_FIELD_IPHC_CID_TYPE << 1        , /**< Flag to identify header compression index field  */
    e_FIELD_IPHC_GEN                = e_FIELD_IPHC_HCINDEX << 1         , /**< Flag to identify generator field                 */
    e_FIELD_IPHC_D_BIT              = e_FIELD_IPHC_GEN << 1             , /**< Flag to identify D bit field                     */
    e_FIELD_IPHC_ALL_FIELDS         = (e_FIELD_IPHC_D_BIT << 1) - 1       /**< Flag to identify all IPHC fields                 */
} e_PgFieldsIphc;

/**************************************************************************//**
 @Description   RTP Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_RTP_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify RTP next header type                */
    e_FIELD_RTP_VALID_FIELDS        = 1                                 , /**< Flag to identify the configured IPHC fields          */
    e_FIELD_RTP_VER                 = e_FIELD_RTP_VALID_FIELDS << 1     , /**< Flag to identify 2 bits - Version field              */
    e_FIELD_RTP_PADDING             = e_FIELD_RTP_VER << 1              , /**< Flag to identify padding field                       */
    e_FIELD_RTP_EXTENSION           = e_FIELD_RTP_PADDING << 1          , /**< Flag to identify extension field                     */
    e_FIELD_RTP_CSRC_COUNT          = e_FIELD_RTP_EXTENSION << 1        , /**< Flag to identify number of CSRC identifiers field    */
    e_FIELD_RTP_MARKER              = e_FIELD_RTP_CSRC_COUNT << 1       , /**< Flag to identify significant events field            */
    e_FIELD_RTP_PAYLOAD_TYPE        = e_FIELD_RTP_MARKER << 1           , /**< Flag to identify format of RTP payload field         */
    e_FIELD_RTP_SEQ_NUM             = e_FIELD_RTP_PAYLOAD_TYPE << 1     , /**< Flag to identify sequence number field               */
    e_FIELD_RTP_TIME_STAMP          = e_FIELD_RTP_SEQ_NUM << 1          , /**< Flag to identify time stamp field                    */
    e_FIELD_RTP_SSRC                = e_FIELD_RTP_TIME_STAMP << 1       , /**< Flag to identify synchronization source field        */
    e_FIELD_RTP_CSRC_LIST           = e_FIELD_RTP_SSRC << 1             , /**< Flag to identify contributing sources
                                                                               for the payload contained in the packet field        */
    e_FIELD_RTP_EXTN_PROFILE        = e_FIELD_RTP_CSRC_LIST << 1        , /**< Flag to identify extension header profile field      */
    e_FIELD_RTP_EXTN_LENGTH         = e_FIELD_RTP_EXTN_PROFILE << 1     , /**< Flag to identify extension header length field       */
    e_FIELD_RTP_EXTN_HEADER         = e_FIELD_RTP_EXTN_LENGTH << 1      , /**< Flag to identify extension header field              */
    e_FIELD_RTP_PAD_LENGTH          = e_FIELD_RTP_EXTN_HEADER << 1      , /**< Flag to identify number of bytes to be padded field  */
    e_FIELD_RTP_ALL_FIELDS          = (e_FIELD_RTP_PAD_LENGTH << 1) - 1   /**< Flag to identify all RTP fields                      */
} e_PgFieldsRtp;

/**************************************************************************//**
 @Description   MPLS Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_MPLS_NEXT_HEADER_TYPE   = 0                                 , /**< Flag to identify MPLS next header type               */
    e_FIELD_MPLS_VALID_FIELDS       = 1                                 , /**< Flag to identify the configured MPLS fields          */
    e_FIELD_MPLS_LABEL              = e_FIELD_MPLS_VALID_FIELDS << 1    , /**< Flag to identify label field                         */
    e_FIELD_MPLS_CLASSOFSERVICE     = e_FIELD_MPLS_LABEL << 1           , /**< Flag to identify class of service field              */
    e_FIELD_MPLS_STACKVALUE         = e_FIELD_MPLS_CLASSOFSERVICE<< 1   , /**< Flag to identify stack for tracking the labels field */
    e_FIELD_MPLS_TIMETOLIVE         = e_FIELD_MPLS_STACKVALUE << 1      , /**< Flag to identify time to live field                  */
    e_FIELD_MPLS_ALL_FIELDS         = (e_FIELD_MPLS_TIMETOLIVE << 1) - 1  /**< Flag to identify all MPLS fields                     */
} e_PgFieldsMpls;

/**************************************************************************//**
 @Description   Enumeration for L2TPv2 Fields (RFC2661)
*//***************************************************************************/
typedef enum {
    e_FIELD_L2TPv2_NEXT_HEADER_TYPE     = 0                                 , /**< Flag to identify L2TPv2 next header type                                     */
    e_FIELD_L2TPv2_VALID_FIELDS         = 1                                 , /**< Flag to identify the configured L2TPv2 fields                                */
    e_FIELD_L2TPv2_TBIT                 = e_FIELD_L2TPv2_VALID_FIELDS << 1  , /**< Flag to identify one bit T - control/data field                              */
    e_FIELD_L2TPv2_LBIT                 = e_FIELD_L2TPv2_TBIT << 1          , /**< Flag to identify one bit L - length present field                            */
    e_FIELD_L2TPv2_SBIT                 = e_FIELD_L2TPv2_LBIT << 1          , /**< Flag to identify one bit S - sequence numbers present field                  */
    e_FIELD_L2TPv2_OBIT                 = e_FIELD_L2TPv2_SBIT << 1          , /**< Flag to identify one bit O - offset present field                            */
    e_FIELD_L2TPv2_PBIT                 = e_FIELD_L2TPv2_OBIT << 1          , /**< Flag to identify one bit P - Priority field                                  */
    e_FIELD_L2TPv2_VER                  = e_FIELD_L2TPv2_PBIT << 1          , /**< Flag to identify version field                                               */
    e_FIELD_L2TPv2_LEN                  = e_FIELD_L2TPv2_VER << 1           , /**< Flag to identify length field                                                */
    e_FIELD_L2TPv2_TID                  = e_FIELD_L2TPv2_LEN << 1           , /**< Flag to identify Tunnel ID field                                             */
    e_FIELD_L2TPv2_SID                  = e_FIELD_L2TPv2_TID << 1           , /**< Flag to identify Session ID field                                            */
    e_FIELD_L2TPv2_NS                   = e_FIELD_L2TPv2_SID << 1           , /**< Flag to identify sequence number for data / control message field                    */
    e_FIELD_L2TPv2_NR                   = e_FIELD_L2TPv2_NS << 1            , /**< Flag to identify sequence number expected for next data / control message field      */
    e_FIELD_L2TPv2_OFFSIZE              = e_FIELD_L2TPv2_NR << 1            , /**< Flag to identify Offset Size field                                           */
    e_FIELD_L2TPv2_ALL_FIELDS           = (e_FIELD_L2TPv2_OFFSIZE << 1) - 1   /**< Flag to identify all L2TPv2 fields                                           */
} e_PgFieldsL2tpv2;

/**************************************************************************//**
 @Description   Enumeration for L2TPv3 Fields (RFC3931)
*//***************************************************************************/
typedef enum {
    e_FIELD_L2TPv3_NEXT_HEADER_TYPE     = 0                                         , /**< Flag to identify L2TPv3 next header type                                     */
    e_FIELD_L2TPv3_VALID_FIELDS         = 1                                         , /**< Flag to identify the configured L2TPv3 fields                                */
    e_FIELD_L2TPv3_TBIT                 = e_FIELD_L2TPv3_VALID_FIELDS << 1          , /**< Flag to identify one bit T - control/data field                              */
    e_FIELD_L2TPv3_LBIT                 = e_FIELD_L2TPv3_TBIT << 1                  , /**< Flag to identify one bit L - length present field                            */
    e_FIELD_L2TPv3_SBIT                 = e_FIELD_L2TPv3_LBIT << 1                  , /**< Flag to identify one bit S - sequence numbers present field                  */
    e_FIELD_L2TPv3_VER                  = e_FIELD_L2TPv3_SBIT << 1                  , /**< Flag to identify version field                                               */
    e_FIELD_L2TPv3_LEN                  = e_FIELD_L2TPv3_VER << 1                   , /**< Flag to identify length field                                                */
    e_FIELD_L2TPv3_CCID                 = e_FIELD_L2TPv3_LEN << 1                   , /**< Flag to identify Control Connection ID field                                 */
    e_FIELD_L2TPv3_NS                   = e_FIELD_L2TPv3_CCID << 1                  , /**< Flag to identify sequence number for control message field                   */
    e_FIELD_L2TPv3_NR                   = e_FIELD_L2TPv3_NS << 1                    , /**< Flag to identify sequence number expected for next control message field     */
    e_FIELD_L2TPv3_SID                  = e_FIELD_L2TPv3_NR << 1                    , /**< Flag to identify Session ID field                                            */
    e_FIELD_L2TPv3_COOKIE_SIZE          = e_FIELD_L2TPv3_SID << 1                   , /**< Flag to identify Cookie size field                                           */
    e_FIELD_L2TPv3_COOKIE               = e_FIELD_L2TPv3_COOKIE_SIZE << 1           , /**< Flag to identify Cookie field                                                */
    e_FIELD_L2TPv3_L2_SUBLAYER_SBIT     = e_FIELD_L2TPv3_COOKIE << 1                , /**< Flag to identify one bit L2 Sublayer S - validity of sequence number         */
    e_FIELD_L2TPv3_L2_SUBLAYER_NS       = e_FIELD_L2TPv3_L2_SUBLAYER_SBIT << 1      , /**< Flag to identify L2 Sublayer sequence number for data message field          */
    e_FIELD_L2TPv3_ALL_FIELDS           = (e_FIELD_L2TPv3_L2_SUBLAYER_NS << 1) - 1    /**< Flag to identify all L2TPv3 fields                                           */
} e_PgFieldsL2tpv3;

/**************************************************************************//**
 @Description   PWE3 Encapsulation Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_PWE3_ENCAP_NEXT_HEADER_TYPE = 0                                     , /**< Flag to identify PWE3Encap next header type      */
    e_FIELD_PWE3_ENCAP_VALID_FIELDS     = 1                                     , /**< Flag to identify the configured PWE3Encap fields */
    e_FIELD_PWE3_ENCAP_TYPE             = e_FIELD_PWE3_ENCAP_VALID_FIELDS << 1  , /**< Flag to identify type field                      */
    e_FIELD_PWE3_ENCAP_FLAGS            = e_FIELD_PWE3_ENCAP_TYPE << 1          , /**< Flag to identify flags field                     */
    e_FIELD_PWE3_ENCAP_FRG              = e_FIELD_PWE3_ENCAP_FLAGS << 1         , /**< Flag to identify framenting info field           */
    e_FIELD_PWE3_ENCAP_LEN              = e_FIELD_PWE3_ENCAP_FRG << 1           , /**< Flag to identify length field                    */
    e_FIELD_PWE3_ENCAP_SEQ_NUM          = e_FIELD_PWE3_ENCAP_LEN << 1           , /**< Flag to identify sequence number field           */
    e_FIELD_PWE3_ENCAP_ALL_FIELDS       = (e_FIELD_PWE3_ENCAP_SEQ_NUM << 1) - 1  /**< Flag to identify all PWE3Encap fields             */
} e_PgFieldsPwe3Encap;

/**************************************************************************//**
 @Description   Payload Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_PAYLOAD_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify payload next header type                            */
    e_FIELD_PAYLOAD_BUFFER              = 1                                 , /**< Flag to identify user defined payload data buffer field              */
    e_FIELD_PAYLOAD_SIZE                = e_FIELD_PAYLOAD_BUFFER << 1       , /**< Flag to identify size of the user defined payload field              */
    e_FIELD_MAX_FRM_SIZE                = e_FIELD_PAYLOAD_SIZE << 1         , /**< Flag to identify maximum size field                                  */
    e_FIELD_MIN_FRM_SIZE                = e_FIELD_MAX_FRM_SIZE << 1         , /**< Flag to identify minimum size field                                  */
    e_FIELD_PAYLOAD_TYPE                = e_FIELD_MIN_FRM_SIZE << 1         , /**< Flag to identify type of the payload field                           */
    e_FIELD_FRAME_SIZE                  = e_FIELD_PAYLOAD_TYPE << 1         , /**< Flag to identify frame size field                                    */
    e_FIELD_PRBS_POLYNOMIAL             = e_FIELD_FRAME_SIZE << 1           , /**< Flag to identify user defined polynomial for PRBS algorithm field    */
    e_FIELD_PRBS_NUM_BITS               = e_FIELD_PRBS_POLYNOMIAL << 1      , /**< Flag to identify number of bits in the sequence field                */
    e_FIELD_PRBS_INIT_VECTOR            = e_FIELD_PRBS_NUM_BITS << 1        , /**< Flag to identify initial seed value field                            */
    e_FIELD_FIXED_PATTERN_SIZE          = e_FIELD_PRBS_INIT_VECTOR << 1     , /**< Flag to identify pattern size field                                  */
    e_FIELD_FIXED_PATTERN_VALUE         = e_FIELD_FIXED_PATTERN_SIZE << 1   , /**< Flag to identify pattern value field                                 */
    e_FIELD_SEQ_NUM                     = e_FIELD_FIXED_PATTERN_VALUE << 1  , /**< Flag to identify sequence number field                               */
    e_FIELD_PAYLOAD_ALL_FIELDS          = (e_FIELD_SEQ_NUM << 1) - 1          /**< Flag to identify all Payload fields                                  */
} e_PgPayloadFields;

/**************************************************************************//**
 @Description   Padding Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_PADDING_NEXT_HEADER_TYPE    = 0                                 , /**< Flag to identify padding next header type        */
    e_FIELD_PADDING_VALID_FIELDS        = 1                                 , /**< Flag to identify the configured padding fields   */
    e_FIELD_PADDING_VALUE               = e_FIELD_PADDING_VALID_FIELDS << 1 , /**< Flag to identify padding value field             */
    e_FIELD_PADDING_SIZE                = e_FIELD_PADDING_VALUE << 1        , /**< Flag to identify padding size field              */
    e_FIELD_PADDING_SHIM_HEADER         = e_FIELD_PADDING_SIZE << 1             , /**< Flag to identify padding shim header field       */
    e_FIELD_PADDING_ALL_FIELDS          = (e_FIELD_PADDING_SHIM_HEADER << 1) - 1  /**< Flag to identify all padding fields              */
} e_PgPaddingFields;

/**************************************************************************//**
 @Description   IEEE1588 Version1 Message Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_IEEE1588_V1_NEXT_HEADER_TYPE        = 0                                         , /**< Flag to identify IEEE1588 v1 next header type                */
    e_FIELD_IEEE1588_V1_VALID_FIELDS            = 1                                         , /**< Flag to identify the configured IEEE1588 v1 fields           */
    e_FIELD_IEEE1588_V1_VERSION                 = e_FIELD_IEEE1588_V1_VALID_FIELDS << 1     , /**< Flag to identify IEEE1588 v1 version field                   */
    e_FIELD_IEEE1588_V1_VERSION_NETWORK         = e_FIELD_IEEE1588_V1_VERSION << 1          , /**< Flag to identify IEEE1588 v1 version network field           */
    e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM          = e_FIELD_IEEE1588_V1_VERSION_NETWORK << 1  , /**< Flag to identify IEEE1588 v1 sub domain number field         */
    e_FIELD_IEEE1588_V1_MESSAGE_TYPE            = e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM << 1   , /**< Flag to identify IEEE1588 v1 message type field              */
    e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH        = e_FIELD_IEEE1588_V1_MESSAGE_TYPE << 1     , /**< Flag to identify IEEE1588 v1 source communication tech field */
    e_FIELD_IEEE1588_V1_SOURCE_U_UID            = e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH << 1 , /**< Flag to identify IEEE1588 v1 source u uid field              */
    e_FIELD_IEEE1588_V1_SOURCE_PORT_ID          = e_FIELD_IEEE1588_V1_SOURCE_U_UID << 1     , /**< Flag to identify IEEE1588 v1 source port uid field           */
    e_FIELD_IEEE1588_V1_SEQUENCE_ID             = e_FIELD_IEEE1588_V1_SOURCE_PORT_ID << 1   , /**< Flag to identify IEEE1588 v1 sequence id field               */
    e_FIELD_IEEE1588_V1_CONTROL                 = e_FIELD_IEEE1588_V1_SEQUENCE_ID << 1      , /**< Flag to identify IEEE1588 v1 control field                   */
    e_FIELD_IEEE1588_V1_FLAGS                   = e_FIELD_IEEE1588_V1_CONTROL << 1          , /**< Flag to identify IEEE1588 v1 flags field                     */
    e_FIELD_IEEE1588_V1_ALL_FIELDS              = (e_FIELD_IEEE1588_V1_FLAGS << 1) - 1        /**< Flag to identify all IEEE1588 v1 fields                      */
} e_PgFieldsIeee1588MessageV1;

/**************************************************************************//**
 @Description  IEEE1588 Version2 Message Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_IEEE1588_V2_NEXT_HEADER_TYPE        = 0                                             , /**< Flag to identify IEEE1588 v2 next header type            */
    e_FIELD_IEEE1588_V2_VALID_FIELDS            = 1                                             , /**< Flag to identify the configured IEEE1588 v2 fields       */
    e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC      = e_FIELD_IEEE1588_V2_VALID_FIELDS << 1         , /**< Flag to identify IEEE1588 v2 transport specific field    */
    e_FIELD_IEEE1588_V2_MESSAGE_TYPE            = e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC << 1   , /**< Flag to identify IEEE1588 v2 message type field          */
    e_FIELD_IEEE1588_V2_VERSION                 = e_FIELD_IEEE1588_V2_MESSAGE_TYPE << 1         , /**< Flag to identify IEEE1588 v2 version field               */
    e_FIELD_IEEE1588_V2_MESSAGE_LENGTH          = e_FIELD_IEEE1588_V2_VERSION << 1              , /**< Flag to identify IEEE1588 v2 message length field        */
    e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM          = e_FIELD_IEEE1588_V2_MESSAGE_LENGTH << 1       , /**< Flag to identify IEEE1588 v2 sub domain number field     */
    e_FIELD_IEEE1588_V2_FLAGS                   = e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM << 1       , /**< Flag to identify IEEE1588 v2 flags field                 */
    e_FIELD_IEEE1588_V2_CORRECTION_FIELD        = e_FIELD_IEEE1588_V2_FLAGS << 1                , /**< Flag to identify IEEE1588 v2 correction field            */
    e_FIELD_IEEE1588_V2_CLOCK_IDENTITY          = e_FIELD_IEEE1588_V2_CORRECTION_FIELD << 1     , /**< Flag to identify IEEE1588 v2 clock identity field        */
    e_FIELD_IEEE1588_V2_SOURCE_PORT_ID          = e_FIELD_IEEE1588_V2_CLOCK_IDENTITY << 1       , /**< Flag to identify IEEE1588 v2 source port id field        */
    e_FIELD_IEEE1588_V2_SEQUENCE_ID             = e_FIELD_IEEE1588_V2_SOURCE_PORT_ID << 1       , /**< Flag to identify IEEE1588 v2 sequence id field           */
    e_FIELD_IEEE1588_V2_CONTROL                 = e_FIELD_IEEE1588_V2_SEQUENCE_ID << 1          , /**< Flag to identify IEEE1588 v2 control field               */
    e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD      = e_FIELD_IEEE1588_V2_CONTROL << 1              , /**< Flag to identify IEEE1588 v2 log message period field    */
    e_FIELD_IEEE1588_V2_TARGET_PORT_IDENTITY    = e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD << 1   , /**< Flag to identify IEEE1588 v2 target port identity field  */
    e_FIELD_IEEE1588_V2_TARGET_PORT_ID          = e_FIELD_IEEE1588_V2_TARGET_PORT_IDENTITY << 1 , /**< Flag to identify IEEE1588 v2 target port id field        */
    e_FIELD_IEEE1588_V2_TLV_TYPE                = e_FIELD_IEEE1588_V2_TARGET_PORT_ID << 1       , /**< Flag to identify IEEE1588 v2 tlv type field              */
    e_FIELD_IEEE1588_V2_LENGTH_FIELD            = e_FIELD_IEEE1588_V2_TLV_TYPE << 1             , /**< Flag to identify IEEE1588 v2 length field                */
    e_FIELD_IEEE1588_V2_ALL_FIELDS              = (e_FIELD_IEEE1588_V2_LENGTH_FIELD << 1) - 1     /**< Flag to identify all IEEE1588 v2 fields                  */
} e_PgFieldsIeee1588MessageV2;

/**************************************************************************//**
 @Description   User-defined Fields
*//***************************************************************************/
typedef enum
{
    e_FIELD_USER_DEFINED_SRCPORT    = 1                                     , /**< Flag to identify user defined source port field  */
    e_FIELD_USER_DEFINED_PCDID      = e_FIELD_USER_DEFINED_SRCPORT << 1     , /**< Flag to identify user defined pcdid field        */
    e_FIELD_USER_DEFINED_ALL_FIELDS = (e_FIELD_USER_DEFINED_PCDID << 1) - 1   /**< Flag to identify all user defined fields         */
} e_PgUserDefinedFields;

/**************************************************************************//**
 @Description   Header Types
*//***************************************************************************/
typedef enum
{
    e_HEADER_TYPE_NONE = 0                              , /**< No header                            */
    e_HEADER_TYPE_PAYLOAD                               , /**< Payload                              */
    e_HEADER_TYPE_ETH                                   , /**< Ethernet header                      */
    e_HEADER_TYPE_ETH_802_3                             , /**< Ethernet 802.3 header                */
    e_HEADER_TYPE_IEEE1588_ETH                          , /**< IEEE 1588 Over Ethernet header       */
    e_HEADER_TYPE_VLAN                                  , /**< VLAN header                          */
    e_HEADER_TYPE_VTAGGED_802_3                         , /**< Vtagged IEEE 802.3                   */
    e_HEADER_TYPE_MPLS                                  , /**< MPLS header                          */
    e_HEADER_TYPE_L2TPV2                                , /**< L2TP V2 header                       */
    e_HEADER_TYPE_L2TPV3                                , /**< L2TP V3 header                       */
    e_HEADER_TYPE_IPv4                                  , /**< IPv4 header                          */
    e_HEADER_TYPE_IPv6                                  , /**< IPv6 header                          */
    e_HEADER_TYPE_TCP                                   , /**< TCP header                           */
    e_HEADER_TYPE_UDP                                   , /**< UDP header                           */
    e_HEADER_TYPE_SCTP                                  , /**< SCTP header                          */
    e_HEADER_TYPE_SCTP_CHUNK_DATA                       , /**< SCTP Chunk header                    */
    e_HEADER_TYPE_IEEE1588_V1_GENERAL_UDP               , /**< IEEE 1588 V1 UDP general header      */
    e_HEADER_TYPE_IEEE1588_V2_GENERAL_UDP               , /**< IEEE 1588 V2 UDP general header      */
    e_HEADER_TYPE_IEEE1588_V1_EVENT_UDP                 , /**< IEEE 1588 V1 UDP event header        */
    e_HEADER_TYPE_IEEE1588_V2_EVENT_UDP                 , /**< IEEE 1588 V2 UDP event header        */
    e_HEADER_TYPE_IEEE1588_V1_MESSAGE                   , /**< IEEE 1588 V1 message info            */
    e_HEADER_TYPE_IEEE1588_V2_MESSAGE                   , /**< IEEE 1588 V2 message info            */
    e_HEADER_TYPE_IPHC                                  , /**< IPHC manipulation                    */
    e_HEADER_TYPE_PPPoE                                 , /**< PPPoE header                         */
    e_HEADER_TYPE_PPP                                   , /**< PPP header                           */
    e_HEADER_TYPE_PPP_MLMC                              , /**< ML/MC header                         */
    e_HEADER_TYPE_HDLC                                  , /**< HDLC header                          */
    e_HEADER_TYPE_LLC                                   , /**< LLC header                           */
    e_HEADER_TYPE_NLPID                                 , /**< NLP header                           */
    e_HEADER_TYPE_SNAP                                  , /**< SNAP header                          */
    e_HEADER_TYPE_AAL0                                  , /**< AAL0 cell header                     */
    e_HEADER_TYPE_RAW_DATA                              , /**< Raw data                             */
    e_HEADER_TYPE_RTP                                   , /**< RTP header                           */
    e_HEADER_TYPE_PWE3_ATM_MPLS_DEMUX                   , /**< ATM Pseudowire Demux Layer header    */
    e_HEADER_TYPE_PWE3_ATM_ENCAP                        , /**< ATM Pseudowire Encap Layer header    */
    e_HEADER_TYPE_PWE3_SAToP_MPLS_DEMUX                 , /**< SAToP Pseudowire Demux Layer header  */
    e_HEADER_TYPE_PWE3_SAToP_ENCAP                      , /**< SAToP Pseudowire Encap Layer header  */
    e_HEADER_TYPE_TUNNELED_IPv4                         , /**< Tunnelled IPv4 header                */
    e_HEADER_TYPE_TUNNELED_IPv6                         , /**< Tunnelled IPv6 header                */
    e_HEADER_TYPE_PADDING                               , /**< Padding header                       */
    e_MAX_HEADER_TYPE_COUNT                             , /**< Number of header types               */
    e_HEADER_TYPE_USER_DEFINED = e_MAX_HEADER_TYPE_COUNT  /**< User defined header                  */
} e_PgHeaderType;

/**************************************************************************//**
 @Description   CRC Type
*//***************************************************************************/
typedef enum
{
    e_FCS_16 = 1 /**< 16-bit FCS    */
  , e_FCS_32 = 2 /**< 32-bit FCS    */
} e_PgFcsType;

/**************************************************************************//**
 @Description   Frame Manipulation Type
*//***************************************************************************/
typedef enum
{
    e_IPv4_CHECKSUM_ERROR           = 0x00000001 /**< Insertion of IPv4 checksum error                  */
  , e_UDP_CHECKSUM_ERROR            = 0x00000002 /**< Insertion of UDP checksum error                   */
  , e_TCP_CHECKSUM_ERROR            = 0x00000004 /**< Insertion of TCP checksum error                   */
  , e_IPv4_VERSION_ERROR            = 0x00000008 /**< Insertion of IPv4 version error                   */
  , e_IPv6_VERSION_ERROR            = 0x00000010 /**< Insertion of IPv6 version error                   */
  , e_AAL0_HEC_ERROR                = 0x00000020 /**< Insertion of AAL0 HEC error                       */
  , e_RTP_VERSION_ERROR             = 0x00000040 /**< Insertion of RTP version error                    */
  , e_FCS_ERROR                     = 0x00000080 /**< Insertion of FCS error                            */
  , e_INSERT_HEADER                 = 0x00000100 /**< Insert header to the frame gen                    */
  , e_REMOVE_HEADER                 = 0x00000200 /**< Remove any existing header from the frame gen     */
  , e_REPLACE_HEADER                = 0x00000400 /**< Replace existing header in the frame gen
                                                         with the new information                       */
  , e_MANIPULATE_HEADER_FIELD       = 0x00000800 /**< Modify any fields in the frame header             */
  , e_SCTP_CHECKSUM_ERROR           = 0x00001000 /**< Insertion of SCTP checksum error                  */
  , e_ERROR_TYPE_NONE               = 0x00000000
} e_PgFrameManipulation;

/**************************************************************************//**
 @Description   Payload Type
*//***************************************************************************/
typedef enum
{
    e_PAYLOAD_PRBS          , /**< The payload will contain bytes with random values */
    e_PAYLOAD_PATTERN       , /**< The payload will contain a defined pattern : an initial value,
                                   a step, and the  number of steps                  */
    e_PAYLOAD_USER          , /**< The payload will be copied from a location in memory provided
                                   by the user                                       */
    e_PAYLOAD_FIXED_PATTERN   /**< The payload will contain the fixed patterns       */
} e_PgPayloadType;

/**************************************************************************//**
 @Description   IEEE 1588 Version 1 Control Type
*//***************************************************************************/
typedef enum
{
      e_IEEE1588_V1_MESSAGE_SYNC          = 0x00    , /**< Sync Message             */
      e_IEEE1588_V1_MESSAGE_DELAY_REQ               , /**< Delay Request Message    */
      e_IEEE1588_V1_MESSAGE_FOLLOW_UP               , /**< Follow up Message        */
      e_IEEE1588_V1_MESSAGE_DELAY_RESP                /**< Delay Response Message   */
} e_PgIeee1588V1ControlType;

/**************************************************************************//**
 @Description   IEEE 1588 Version 2 Message Type
*//***************************************************************************/
typedef enum
{
      e_IEEE1588_V2_MESSAGE_SYNC                    = 0x00  , /**< Sync Message                             */
      e_IEEE1588_V2_MESSAGE_DELAY_REQ                       , /**< Delay Request Message                    */
      e_IEEE1588_V2_MESSAGE_PATH_DELAY_REQ                  , /**< Path Delay Request Message               */
      e_IEEE1588_V2_MESSAGE_PATH_DELAY_RESP                 , /**< Path Delay Response Message              */
      e_IEEE1588_V2_MESSAGE_FOLLOW_UP               = 0x08  , /**< Follow up Message                        */
      e_IEEE1588_V2_MESSAGE_DELAY_RESP                      , /**< Delay Response Message                   */
      e_IEEE1588_V2_MESSAGE_PATH_DELAY_RESP_FOLLOW_UP       , /**< Path Delay Response Follow up Message    */
      e_IEEE1588_V2_MESSAGE_ANNOUNCE                        , /**< Announce Message                         */
      e_IEEE1588_V2_MESSAGE_SIGNALLING                      , /**< Signalling Message                       */
      e_IEEE1588_V2_MESSAGE_MANAGEMENT                        /**< Management Message                   */
} e_PgIeee1588V2MessageType;

/**************************************************************************//**
 @Description   HDLC Address Type
*//***************************************************************************/
typedef enum
{
    e_ADDRESS_8                     = 1 ,   /**< 8-bit HDLC address     */
    e_ADDRESS_16                    = 2     /**< 16-bit HDLC address    */
} e_PgHdlcAddrType;

/**************************************************************************//**
 @Description   ML/MC Sequence Number Type
*//***************************************************************************/
typedef enum
{
    e_SEQ_NUM_12_BIT     = 0 /**< 12 bit sequence number    */
  , e_SEQ_NUM_24_BIT     = 1 /**< 24 bit sequence number    */
} e_PgMlmcSeqNumType;

/**************************************************************************//**
 @Description   LLC Type
*//***************************************************************************/
typedef enum
{
    e_VAL_LLC_NLPID      = 0xFEFE03 /**< Value configured for llc, if next header is NLPID                  */
  , e_VAL_LLC_SNAP       = 0xAAAA03 /**< Value configured for llc, if next header is SNAP                   */
  , e_VAL_LLC_MASK       = 0xFFFFFF /**< Value configured for llc, if not followd by any protocol header    */
} e_PgValuesLlc;

/**************************************************************************//**
 @Description   NLP Type
*//***************************************************************************/
typedef enum
{
    e_VAL_NLPID_PPP       = 0xCF /**< Value is configured in NLPID represents PPP               */
  , e_VAL_NLPID_MASK      = 0xFF /**< Value is configured if not followd by any protocol header */
} e_PgValuesNlpid;

/**************************************************************************//**
 @Description   SNAP Type
*//***************************************************************************/
typedef enum
{
    e_VAL_SNAP_ROUTED      = 0x000000 /**< OUI for Routed Protocols     */
  , e_VAL_SNAP_BRIDGED     = 0x0080C2 /**< OUI for Bridged Protocols    */
  , e_VAL_SNAP_VPN         = 0x00005E /**< OUI for VPN                  */
  , e_VAL_SNAP_MASK        = 0xFFFFFF /**< OUI Masking Value            */
} e_PgValuesSnapOui;

/**************************************************************************//**
 @Description   SNAP PID Type
*//***************************************************************************/
typedef enum
{
    e_VAL_SNAP_PID_ETH_802_3        = 0x0001 /**< PID for Eth_802_3             */
  , e_VAL_SNAP_PID_802_4            = 0x0002 /**< PID for 802_4                 */
  , e_VAL_SNAP_PID_802_5            = 0x0003 /**< PID for 802_5                 */
  , e_VAL_SNAP_PID_FDDI             = 0x0004 /**< PID for FDDI                  */
  , e_VAL_SNAP_PID_802_6            = 0x0005 /**< PID for 802_6                 */

  , e_VAL_SNAP_PID_ETH_802_3_NO_FCS = 0x0007 /**< PID for Eth_802_3 with no FCS */
  , e_VAL_SNAP_PID_802_4_NO_FCS     = 0x0008 /**< PID for 802_4 with no FCS     */
  , e_VAL_SNAP_PID_802_5_NO_FCS     = 0x0009 /**< PID for 802_5 with no FCS     */
  , e_VAL_SNAP_PID_FDDI_NO_FCS      = 0x000A /**< PID for FDDI with no FCS      */
  , e_VAL_SNAP_PID_802_6_NO_FCS     = 0x000B /**< PID for 802_6 with no FCS     */
  , e_VAL_SNAP_PID_FRAGMENTS        = 0x000D /**< PID Fragments                 */
  , e_VAL_SNAP_PID_BPDUS            = 0x000E /**< PID for the BPDU              */

  , e_VAL_SNAP_PID_MASK             = 0xFFFF /**< PID Masking Value             */
} e_PgValuesSnapPidBridged;

/**************************************************************************//**
 @Description   TCP Flags
*//***************************************************************************/
typedef enum
{
    e_FLAG_TCP_FIN  = 1                         , /**< FIN flag                     */
    e_FLAG_TCP_SYN  = e_FLAG_TCP_FIN << 1       , /**< SYN flag                     */
    e_FLAG_TCP_RST  = e_FLAG_TCP_SYN << 1       , /**< Reset connection flag        */
    e_FLAG_TCP_PSH  = e_FLAG_TCP_RST << 1       , /**< Push flag                    */
    e_FLAG_TCP_ACK  = e_FLAG_TCP_PSH << 1       , /**< Ack number valid flag        */
    e_FLAG_TCP_URG  = e_FLAG_TCP_ACK << 1       , /**< Urgent pointer valid flag    */
    e_FLAG_TCP_ALL  =(e_FLAG_TCP_URG << 1) - 1    /**< All the above flags          */
} e_PgFlagsTcp;

/**************************************************************************//**
 @Description   SCTP Flags
*//***************************************************************************/
 typedef enum
 {
     e_FLAG_CHUNK_END                      = 1,                                   /**< E bit : indicates that this is the last chunk                                        */
     e_FLAG_CHUNK_BEGGINING                = e_FLAG_CHUNK_END        << 1,        /**< B bit : indicates that this is the first chunk                                       */
     e_FLAG_CHUNK_UNORDERED                = e_FLAG_CHUNK_BEGGINING  << 1,        /**< U bit : indicates that this is an UNORDERED chunk ( stream sequence number invalid ) */
     e_FLAG_CHUNK_ALL_FIELDS               = (e_FLAG_CHUNK_UNORDERED  << 1) - 1   /**< All the above flags                                                                  */
 } e_PgFlagsSctpChunk;

/**************************************************************************//**
 @Description   Fixed Pattern Payload Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             patternValue[MAX_FIXED_PATTERN_PAYLOAD_BYTES];   /**< Fixed pattern value   */
    uint8_t             patternSize;                                     /**< Pattern size          */
} t_PgPayloadFixedPattern;

/**************************************************************************//**
 @Description   User-defined Payload Structure
*//***************************************************************************/
typedef struct
{
    uint8_t     *p_Buffer;    /**< User defined payload data buffer */
    uint32_t     size;        /**< Size of the user defined payload */
} t_PgPayloadUser;

/**************************************************************************//**
 @Description   Payload Pattern Structure
*//***************************************************************************/
typedef struct
{
    uint8_t     startVal;    /**< First byte value of the payload                       */
    int8_t      stepVal;     /**< Determines the consecutive byte values of the payload */
    uint8_t     stepCount;   /**< Step count determines the number of steps after
                                  which the data bytes are to be repeated               */
} t_PgPayloadPattern;

/**************************************************************************//**
 @Description   PRBS Structure
*//***************************************************************************/
typedef struct
{
    uint32_t     polynomial;    /**< User defined polynomial for PRBS algorithm */
    uint8_t      numBits;       /**< Number of bits in the sequence             */
    uint32_t     initVector;    /**< Initial seed value                         */
} t_PgPayloadPrbs;

/**************************************************************************//**
 @Description   Payload Type Union
*//***************************************************************************/
typedef union
{
    t_PgPayloadPrbs             prbs;              /**< Random payload structure        */
    t_PgPayloadPattern          pattern;           /**< Payload pattern structure       */
    t_PgPayloadUser             user;              /**< User defined payload structure  */
    t_PgPayloadFixedPattern     fixedPattern;      /**< Fixed payload pattern structure */
} u_PgExtra;

/**************************************************************************//**
 @Description   IEEE1588 Version1 Message Structure
*//***************************************************************************/
typedef struct
{
    uint16_t        version;            /**< Version of IEEE 1588   */
    uint16_t        versionNetwork;     /**< Version of the network */
    uint8_t         subDomainNum[IEEE1588_V1_SUB_DOMIAN_NUM_SIZE];
                                        /**< Sub-domian Number      */
    uint8_t         messageType;        /**< Message Type           */
    uint8_t         sourceCommTech;     /**< Source Machines Communication Technology */
    uint8_t         sourceUuid[IEEE1588_V1_SOURCE_UUID_SIZE];
                                        /**< SourceUuid             */
    uint16_t        sourcePortID;       /**< Source Port ID         */
    uint16_t        sequenceID;         /**< sequence ID            */
    uint8_t         control;            /**< Control                */
    uint16_t        flags;              /**< Flags                  */

    uint32_t        validFields;        /**< Fields configured by the user */
    e_PgHeaderType  nextHeaderType;     /**< Next header type       */
} t_PgIeee1588V1Message;

/**************************************************************************//**
 @Description   IEEE1588 Version2 Message Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             transportSpecific:4;    /**< Transport Specific     */
    uint8_t             messageType:4;          /**< Message Type           */
    uint8_t             version;                /**< Version of IEEE 1588   */
    uint16_t            messageLength;          /**< Message Length         */
    uint8_t             subDomainNum;           /**< Sub-domian Number      */
    uint16_t            flags;                  /**< Flags                  */
    uint8_t             correctionField[IEEE1588_V2_CORRECTION_FIELD_SIZE];
                                                /**< Correction Field       */
    uint8_t             clockIdentity[IEEE1588_V2_CLOCK_IDENTITY_SIZE];
                                                /**< Reserved Field         */
    uint16_t            sourcePortID;           /**< Source Port ID         */
    uint16_t            sequenceID;             /**< sequence ID            */
    uint8_t             control;                /**< Control                */
    uint8_t             logMessagePeriod;       /**< Log Message Period     */

    uint32_t            validFields;            /**< Fields configured by the user */
    e_PgHeaderType      nextHeaderType;         /**< Next header type       */
} t_PgIeee1588V2Message;

/**************************************************************************//**
 @Description   Payload Structure
*//***************************************************************************/
typedef struct
{
    e_PgPayloadType     type;           /**< Type of the payload                        */
    u_PgExtra           value;          /**< Union of payload type structures           */
    uint32_t            minFrameLen;    /**< Minimum frame length,
                                             valid only for pattern and random types    */
    uint32_t            maxFrameLen;    /**< Maximum frame length,
                                             valid only for pattern and random types    */
    e_PgHeaderType      nextHeaderType; /**< Next header type                           */
} t_PgPayloadType;

/**************************************************************************//**
 @Description   Padding Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             value;          /**< User defined padding value         */
    uint16_t            size;           /**< Size of the user defined padding   */
    uint8_t             *p_ShimHeader;  /**< User defined header                */
    uint32_t            validFields;    /**< Fields configured by the user      */
    e_PgHeaderType      nextHeaderType; /**< Next header type                   */
} t_PgPaddingHeader;

/**************************************************************************//**
 @Description   Ethernet Header Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             da[ETH_ADDR_SIZE];  /**< Destination address                    */
    uint8_t             sa[ETH_ADDR_SIZE];  /**< Source address                         */
    uint16_t            typeLen;            /**< Type/Length                            */

    bool                bUseFinalCkSum;     /**< Enable/Disable the calculation of FCS  */
    uint32_t            finalCheckSum;      /**< Final checksum                         */

    bool                padding;            /**< Enable/Disable Ethernet padding        */

    uint32_t            validFields;        /**< Fields configured by the user          */
    e_PgHeaderType      nextHeaderType;     /**< Next header type                       */
} t_PgEthernetHeader;

/**************************************************************************//**
 @Description   VLAN Header Structure
*//***************************************************************************/
typedef struct
{
    uint16_t            vPri:3;         /**< VLAN priority                  */
    uint16_t            cfi:1;          /**< Canonical Format Indicator     */
    uint16_t            vid:12;         /**< VLAN id                        */
    uint16_t            typeLen;        /**< Type/Length                    */

    uint32_t            validFields;    /**< Fields configured by the user  */
    e_PgHeaderType      nextHeaderType; /**< Next header type               */
} t_PgVlanHeader;

/**************************************************************************//**
 @Description   LLC Header Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             dsap;           /**< Destination Service Access Point   */
    uint8_t             ssap;           /**< Source Service Access Point        */
    uint8_t             ctrl;           /**< Control                            */

    uint32_t            validFields;    /**< Fields configured by the user      */
    e_PgHeaderType      nextHeaderType; /**< Next header type                   */
} t_PgLlcHeader;

/**************************************************************************//**
 @Description   NLP Header Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             nlpid;          /**< Network layer protocol identifier  */

    uint32_t            validFields;    /**< Fields configured by the user      */
    e_PgHeaderType      nextHeaderType; /**< Next header type                   */
} t_PgNlpidHeader;

/**************************************************************************//**
 @Description   SNAP Header Structure
*//***************************************************************************/
typedef struct
{
    uint32_t            oui;            /**< Organizationally Unique Identifier */
    uint16_t            pid;            /**< Protocol Identifier                */

    uint32_t            validFields;    /**< Fields comfigured by the user      */
    e_PgHeaderType      nextHeaderType; /**< Next header type                   */
} t_PgSnapHeader;

/**************************************************************************//**
 @Description   PPPoE Header Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             ver:4;          /**< 4 bits - Version                   */
    uint8_t             type:4;         /**< 4 bits - Type                      */
    uint8_t             code;           /**< Code                               */
    uint16_t            sid;            /**< Session identifier                 */
    uint16_t            len;            /**< Length                             */

    bool                sessionStarted; /**< Enable/Disable the PPPoE session   */

    uint32_t            validFields;    /**< Fields configured by the user      */
    e_PgHeaderType      nextHeaderType; /**< Next header type                   */
} t_PgPppoeHeader;

/**************************************************************************//**
 @Description   HDLC Header Structure
*//***************************************************************************/
typedef struct
{
    uint16_t            address;            /**< Single octet, which contains the binary
                                                   sequence 11111111 (hexadecimal 0xff), the All-Stations address */
    e_PgHdlcAddrType    addressType;        /**< 8/16 bit HDLC address type     */
    uint8_t             control;            /**< Single octet, which contains the binary
                                                   sequence 00000011 (hexadecimal 0x03), the Unnumbered Information (UI) command with the Poll/Final (P/F) bit set to zero */

    bool                bUseFinalCkSum;     /**< Enable/Disable FCS calculation */
    uint32_t            finalCheckSum;      /**< Final Check Sum                */
    e_PgFcsType         finalCheckSumType;  /**< 16/32 bit FCS type             */

    uint32_t            validFields;        /**< Fields configured by the user  */
    e_PgHeaderType      nextHeaderType;     /**< Next header type               */
} t_PgHdlcHeader;

/**************************************************************************//**
 @Description   PPP Header Structure
*//***************************************************************************/
typedef struct
{
    uint16_t            type;               /**< PPP header                     */
    bool                compressedHeader;   /**< Enable/Disable PPP compression */
    uint32_t            validFields;        /**< Fields configured by the user  */
    e_PgHeaderType      nextHeaderType;     /**< Next header type               */
} t_PgPppHeader;

/**************************************************************************//**
 @Description   ML/MC Header Structure
*//***************************************************************************/
typedef struct
{
    uint32_t            beginFragmentBit:1; /**< 1 bit   - Set to 1 for the first fragment of a PPP packet  */
    uint32_t            endFragmentBit:1;   /**< 1 bit   - Set to 1 for the last fragment of a PPP packet   */
    uint32_t            classId:4;          /**< 4 bits  - Class identifier                                 */
    uint32_t            seqNum:24;          /**< 24 bits - Sequence number of each fragment                 */
    uint16_t            pid;                /**< Protocol identifier                                        */

    e_PgMlmcSeqNumType  seqNumType;         /**< Sequence number type                                       */

    uint32_t            validFields;        /**< Fields configured by the user                              */
    e_PgHeaderType      nextHeaderType;     /**< Next header type                                           */
} t_PgPppMlmcHeader;

/**************************************************************************//**
 @Description   AAL0 Cell Header Structure
*//***************************************************************************/
typedef struct
{
    uint32_t            gfc:4;          /**< 4 bits  - Generic Flow Control         */
    uint32_t            vpi:8;          /**< 8 bits  - Virtual Path Identifier      */
    uint32_t            vci:16;         /**< 16 bits - Virtual Channel Identifier   */
    uint32_t            pti:3;          /**< 3 bits  - Payload Type Indication      */
    uint32_t            clp:1;          /**< 1 bit   - Cell Loss Priority           */
    uint8_t             hec;            /**< Header Error Control                   */

    uint32_t            validFields;    /**< Fields configured by the user          */
    e_PgHeaderType      nextHeaderType; /**< Next header type                       */
} t_PgAal0CellHeader;

/**************************************************************************//**
 @Description   IPv4 Header Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             version:4;          /**< 4 bits - Version               */
    uint8_t             headerLength:4;     /**< 4 bits - Header length         */
    uint8_t             tos;                /**< Type Of Service                */
    uint16_t            totalLength;        /**< Total length                   */
    uint16_t            id;                 /**< Identification                 */
    uint16_t            flags:3;            /**< 3 bits  - IPv4 flags           */
    uint16_t            fragmentOffset:13;  /**< 13 bits - Fragment offset      */
    uint8_t             ttl;                /**< Time To Live                   */
    uint8_t             protocol;           /**< Protocol identifier            */
    uint16_t            headerCksum;        /**< Header checksum                */
    uint32_t            srcIp;              /**< Source IP address              */
    uint32_t            dstIp;              /**< Destination IP address         */

    uint8_t             options[IP_MAX_OPTIONS_COUNT];
                                            /**< Options                        */
    uint8_t             optionsCount;
                                            /**< Number of options              */

    uint32_t            validFields;        /**< Fields configured by the user  */
    e_PgHeaderType      nextHeaderType;     /**< Next header type               */
} t_PgIpv4Header;

/**************************************************************************//**
 @Description   IPv6 Header Structure
*//***************************************************************************/
typedef struct
{
    uint32_t            ipVersion:4;            /**< 4 bits - IP version field                                  */
    uint32_t            trafficClass:8;         /**< 8 bits - Traffic class field to distinguish
                                                     between different classes or priorities of IPv6 packets    */
    uint32_t            flowLabel:20;           /**< 20 bits - Flow label to request special handling           */
    uint16_t            payloadLength;          /**< Length of IPv6 payload (i.e., rest of
                                                     the packet following the IPv6 header)                      */
    uint8_t             nextHeader;             /**< Type of header immediately following the IPv6 header       */
    uint8_t             hopLimit;               /**< Decremented by 1 at each node that forwards the packet     */
    uint8_t             srcIp[IPv6_ADDR_SIZE];  /**< Source IP address                                          */
    uint8_t             dstIp[IPv6_ADDR_SIZE];  /**< Destination IP address                                     */

    uint32_t            validFields;            /**< Fields configured by the user                              */
    e_PgHeaderType      nextHeaderType;         /**< Next header type                                           */
} t_PgIpv6Header;

/**************************************************************************//**
 @Description   TCP Header Structure
*//***************************************************************************/
typedef struct
{
    uint16_t            srcPort;            /**< Source Port                                */
    uint16_t            dstPort;            /**< Destination Port                           */
    uint32_t            seq;                /**< Sequence number                            */
    uint32_t            ack;                /**< Acknowledgement number                     */
    uint8_t             offset:4;           /**< 4 bits - Data offset                       */
    uint8_t             flags:6;            /**< 6 bits - Flags: URG-ACK-PSH-RST-SYN-FIN    */
    uint16_t            window;             /**< Window                                     */
    uint16_t            checksum;           /**< Checksum                                   */
    uint16_t            urgentPtr;          /**< Urgent pointer                             */

    uint8_t             p_Opts[MAX_TCP_OPTS_BYTES]; /**< Options                            */

    uint8_t             optionsCount;       /**< Number of options                          */

    uint32_t            validFields;        /**< Fields configured by the user              */
    e_PgHeaderType      nextHeaderType;     /**< Next header type                           */
} t_PgTcpHeader;

/**************************************************************************//**
 @Description   UDP Header Structure
*//***************************************************************************/
typedef struct
{
    uint16_t            srcPort;        /**< Source port                                */
    uint16_t            dstPort;        /**< Destination port                           */
    uint16_t            len;            /**< Datagram length (including this header)    */
    uint16_t            checksum;       /**< UDP Checksum                               */

    uint32_t            validFields;    /**< Fields configured by the user              */
    e_PgHeaderType      nextHeaderType; /**< Next header type                           */
} t_PgUdpHeader;

/**************************************************************************//**
 @Description   SCTP Header Structure
*//***************************************************************************/
 typedef struct
 {
    uint16_t            srcPort;             /**< Source Port                       */
    uint16_t            dstPort;             /**< Destination Port                  */
    uint32_t            verTag;              /**< Verification Tag                  */
    uint32_t            checksum;            /**< Checksum                          */
    uint32_t            validFields;         /**< Fields configured by the user     */
    e_PgHeaderType      nextHeaderType;      /**< Next header type                  */
 } t_PgSctpCommonHeader;

 /**************************************************************************//**
 @Description   SCTP Chunk Header Structure
*//***************************************************************************/
 typedef struct
 {
     uint8_t                 type;                   /**< Chunk Type ( 0 - DATA )                                        */
     uint8_t                 flags;                  /**< 5bits reserved - UNORDERED | BEGGINING fragment | END fragment */
     uint16_t                len;                    /**< Chunk Length                                                   */
     uint32_t                tsn;                    /**< Transmission Sequence Number                                   */
     uint16_t                streamId;               /**< Stream Identifier                                              */
     uint16_t                streamSeq;              /**< Stream Sequence Number                                         */
     uint32_t                payloadPid;             /**< Payload Protocol Identifier                                    */
     uint32_t                validFields;            /**< Fields configured by the user                                  */
     e_PgHeaderType          nextHeaderType;         /**< Next header type                                               */
 } t_PgSctpChunkDataHeader;


/**************************************************************************//**
 @Description   IEEE 1588 Headers
*//***************************************************************************/
typedef union
{
    t_PgIeee1588V1Message       ieee1588V1MessageHeader;   /**< IEEE 1588 Version1 Message Header */
    t_PgIeee1588V2Message       ieee1588V2MessageHeader;   /**< IEEE 1588 Version2 Message Header */
} u_PgIeee1588MessageHeader;

/**************************************************************************//**
 @Description   IPHC Structure
*//***************************************************************************/
typedef struct
{
    uint16_t            cidNumber;      /**< Context identifier             */
    uint32_t            cidType;        /**< Context type                   */
    uint8_t             hcIndex;        /**< Header compression index       */
    uint8_t             gen;            /**< Generation number              */
    uint8_t             dBit;           /**< D-Bit frames                   */

    uint32_t            validFields;    /**< Fields configured by the user  */
    e_PgHeaderType      nextHeaderType; /**< Next header type               */
} t_PgIphcManip;

/**************************************************************************//**
 @Description   RTP Header Structure
*//***************************************************************************/
typedef struct
{
    uint8_t             version:2;                  /**< 2 bits - Version                           */
    uint8_t             padding:1;                  /**< 1 bit  -  If set to 1, the packet contains one or more
                                                         additional padding octets at the end which are not part
                                                         of the payload.                            */
    uint8_t             extension:1;                /**< 1 bit  - If set to 1, the fixed header must be followed
                                                         by exactly one header extension            */
    uint8_t             csrcCount:4;                /**< 4 bits - Number of CSRC identifiers that follow the
                                                         fixed header.                              */
    uint8_t             marker:1;                   /**< 1 bit  - To allow significant events       */
    uint8_t             payloadType:7;              /**< 7 bits - Format of RTP payload             */
    uint16_t            seqNum;                     /**< Sequence number                            */
    uint32_t            timeStamp;                  /**< Time stamp                                 */
    uint32_t            ssrc;                       /**< Identifies the synchronization source      */

    uint32_t            csrcList[MAX_RTP_SRC_ID];   /**< Identifies the contributing sources
                                                         for the payload contained in the packet    */

    uint16_t            extnProfile;                /**< Extension header profile                   */
    uint16_t            extnLength;                 /**< Extension header length                    */

    uint32_t            extnHeader[MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER];
                                                    /**< Extension header                           */

    uint8_t             padLength;                  /**< Number of bytes to be padded               */
    uint32_t            validFields;                /**< Fields configured by the user              */
    e_PgHeaderType      nextHeaderType;             /**< Next header type                           */
} t_PgRtpHeader;

/**************************************************************************//**
@Description    MPLS/PW Label Header Structure
*//***************************************************************************/
typedef struct
{
    uint32_t            label:20;           /**< 20 bits - Label for the MPLS header            */
    uint32_t            cos:3;              /**< 3 bits  - Class of service to set the priority */
    uint32_t            stackValue:1;       /**< 1 bit   - Stack for tracking the labels        */
    uint32_t            ttl:8;              /**< 8 bits  - Time to Live                         */
    uint32_t            validFields;        /**< Fields configured by the user                  */
    e_PgHeaderType      nextHeaderType;     /**< Next Header Type                               */
} t_PgMplsHeader;

/**************************************************************************//**
 @Description   L2TPv2 Header Structure
*//***************************************************************************/
typedef struct {
    uint8_t             type:1;             /**< 1 bit - Type                                   */
    uint8_t             lengthPresent:1;    /**< 1 bit - Length field present                   */
    uint8_t             res0:2;             /**< reserved                                       */
    uint8_t             sequencePresent:1;  /**< 1 bit - Ns and Nr fields present               */
    uint8_t             res1:1;             /**< reserved                                       */
    uint8_t             offsetPresent:1;    /**< 1 bit - Offset present                         */
    uint8_t             priority:1;         /**< 1 bit - Priority field                         */
    uint8_t             res2:4;             /**< reserved                                       */
    uint8_t             version:4;          /**< 4 bits - Version field                         */
    uint16_t            length;             /**< 16 bits - Length field                         */
    uint16_t            tunnelId;           /**< 16 bits - Tunnel ID field                      */
    uint16_t            sessionId;          /**< 16 bits - Session ID field                     */
    uint16_t            sequenceNo;         /**< 16 bits - Sequence Number field                */
    uint16_t            expectedSequenceNo; /**< 16 bits - Expected Sequence No. field          */
    uint16_t            offsetSize;         /**< 16 bits - Offset Size field                    */
    uint32_t            validFields;        /**< Fields configured by the user                  */
    e_PgHeaderType      nextHeaderType;     /**< Next header type                               */
} t_PgL2tpv2Header;

/**************************************************************************//**
 @Description   L2TPv3 Data Message - L2 Specific Sublayer Header Structure
*//***************************************************************************/
typedef struct {
    uint32_t                res0:1;                 /**< reserved                                       */
    uint32_t                sequenceValid:1;        /**< 1 bit - Sequence Number validity               */
    uint32_t                res1:6;                 /**< reserved                                       */
    uint32_t                sequenceNo:24;          /**< 24 bits - Sequence Number field                */
}t_L2SpecificSublayer;
/**************************************************************************//**
 @Description   L2TPv3 Data Message Header Structure
*//***************************************************************************/
typedef struct {
    uint16_t                type:1;                     /**< 1 bit - Type                                   */
    uint16_t                res0:11;                    /**< reserved                                       */
    uint16_t                version:4;                  /**< 4 bits - Version field                         */
    uint16_t                res1;                       /**< 16 bits - reserved                             */
    uint32_t                sessionId;                  /**< 32 bits - Session ID field                     */
    uint8_t                 cookieSize;                 /**< Determines the Cookie size                     */
    uint8_t                 cookie[L2TP_V3_COOKIE_SIZE];/**< 64 bits - Cookie field                         */
    t_L2SpecificSublayer    l2SpecificSublayer;         /**< L2 Specific Sublayer structure                 */
    uint32_t                validFields;                /**< Fields configured by the user                  */
    e_PgHeaderType          nextHeaderType;             /**< Next header type                               */
} t_PgL2tpv3DataMsgHeader;

/**************************************************************************//**
 @Description   L2TPv3 Control Message Header Structure
*//***************************************************************************/
typedef struct {
    uint8_t             type:1;             /**< 1 bit - Type                                   */
    uint8_t             lengthPresent:1;    /**< 1 bit - Length field present                   */
    uint8_t             res0:2;             /**< reserved                                       */
    uint8_t             sequencePresent:1;  /**< 1 bit - Ns and Nr fields present               */
    uint8_t             res1:3;             /**< reserved                                       */
    uint8_t             res2:4;             /**< reserved                                       */
    uint8_t             version:4;          /**< 4 bits - Version field                         */
    uint16_t            length;             /**< 16 bits - Length field                         */
    uint32_t            controlConnectionId;/**< 32 bits - Control Connection ID field          */
    uint16_t            sequenceNo;         /**< 16 bits - Sequence Number field                */
    uint16_t            expectedSequenceNo; /**< 16 bits - Expected Sequence No. field          */
    uint32_t            validFields;        /**< Fields configured by the user                  */
    e_PgHeaderType      nextHeaderType;     /**< Next header type                               */
} t_PgL2tpv3CtrlMsgHeader;

/**************************************************************************//**
 @Description   L2TPv3 Header Structure
*//***************************************************************************/
typedef union {
    t_PgL2tpv3DataMsgHeader     pgL2tpv3DataMsgHeader;      /**< L2TPv3 Data Message header */
    t_PgL2tpv3CtrlMsgHeader     pgL2tpv3CtrlMsgHeader;      /**< L2TPv3 Control Message header */
} u_PgL2tpv3Header;

/**************************************************************************//**
@Description    PWE3 Control Word Header Structure
*//***************************************************************************/
typedef struct
{
    uint32_t            type:4;         /**< 4 bits  - Indicate PW data or PW control frame                 */
    uint32_t            flags:4;        /**< 4 bits  - Per-Payload signaling                                */
    uint32_t            frg:2;          /**< 2 bits  - Used when fragmenting a PW payload                   */
    uint32_t            length:6;       /**< 6 bits  - Length of PW payload + length of the Control Word
                                             Must be set to zero if ETH frame >= MIN_FRAME_LEN_ETHERNET     */
    uint32_t            seqNum:16;      /**< 16 bits  -  PW Frame sequence number                           */
    uint32_t            validFields;    /**< Fields configured by the user                                  */
    e_PgHeaderType      nextHeaderType; /**< Next header type                                               */
} t_PgPwe3CtrlWordHeader;

/**************************************************************************//**
 @Description   Layer 5 Headers
*//***************************************************************************/
typedef union
{
    t_PgRtpHeader       rtpHeader;  /**< RTP header */
} u_PgL5Header;

/**************************************************************************//**
 @Description   Layer 4 SCTP Headers
*//***************************************************************************/

typedef struct
{
    t_PgSctpCommonHeader        sctpCommonHeader;   /**< SCTP Common Header */
    t_PgSctpChunkDataHeader     sctpChunkHeader;    /**< SCTP Chunk Data Header */
}t_PgSctpHeader;

/**************************************************************************//**
 @Description   Layer 4 Headers
*//***************************************************************************/
typedef union
{
    t_PgTcpHeader       tcpHeader;    /**< TCP header */
    t_PgUdpHeader       udpHeader;    /**< UDP header */
    t_PgSctpHeader      sctpHeader;   /**< SCTP header */
} u_PgL4Header;

/**************************************************************************//**
 @Description   Layer 3 Headers
*//***************************************************************************/
typedef union
{
    t_PgIpv4Header      ipv4Header;   /**< IPv4 header */
    t_PgIpv6Header      ipv6Header;   /**< IPv6 header */
} u_PgL3Header;

/**************************************************************************//**
 @Description   Layer 2 Headers
*//***************************************************************************/
typedef struct
{
    t_PgEthernetHeader  ethernetHeader;                 /**< Ethernet header    */
    t_PgVlanHeader      vlanHeader[MAX_VLAN_TAG];       /**< VLAN header        */
    t_PgMplsHeader      mplsHeader[MAX_MPLS_HEADER];    /**< MPLS header        */
    t_PgPppoeHeader     pppoeHeader;                    /**< PPPoE header       */
    t_PgLlcHeader       llcHeader[MAX_LLC_HEADER];      /**< LLC header         */
    t_PgNlpidHeader     nlpidHeader;                    /**< NLPID header       */
    t_PgSnapHeader      snapHeader[MAX_SNAP_HEADER];    /**< SNAP header        */
    t_PgAal0CellHeader  aal0CellHeader;                 /**< AAL0 cell header   */
    t_PgHdlcHeader      hdlcHeader;                     /**< HDLC header        */
    t_PgPppHeader       pppHeader;                      /**< PPP header         */
    t_PgPppMlmcHeader   pppMlmcHeader;                  /**< ML/MC header       */
} t_PgL2Header;

/**************************************************************************//**
 @Description   L2TP Headers
*//***************************************************************************/
typedef union {
    t_PgL2tpv2Header      l2tpv2Header;     /**< L2TPv2 header */
    u_PgL2tpv3Header      l2tpv3Header;     /**< L2TPv3 header */
} u_PgL2tpHeader;

/**************************************************************************//**
 @Description   Frame setting structure
*//***************************************************************************/
typedef struct
{
    e_PgHeaderType              initHeaderType;         /**< First header of the frame  */
    t_PgL2Header                l2Header;               /**< Layer 2 header             */
    u_PgL3Header                l3Header;               /**< Layer 3 header             */
    u_PgL4Header                l4Header;               /**< Layer 4 header             */
    u_PgL5Header                l5Header;               /**< Layer 5 header             */
    t_PgIphcManip               iphcManip;              /**< IPHC Manipulation          */
    t_PgPwe3CtrlWordHeader      pwe3CwHeader;           /**< PWE3 Header                */
    u_PgIeee1588MessageHeader   ieee1588Header;         /**< IEEE1588 header            */
    u_PgL2tpHeader              l2tpHeader;             /**< L2TP header */

    u_PgL3Header                tunnelledL3Header;      /**< Tunnelled Layer 3 header   */
    t_PgPayloadType             payload;                /**< Payload                    */

    t_PgPaddingHeader           paddingHeader[MAX_PADDING_HEADER];
                                                        /**< Padding header             */
} t_PgFrameSetting;

/**************************************************************************//**
 @Description   Header Compression Parameters Structure
*//***************************************************************************/
typedef struct
{
    uint8_t     genCh;          /**< Generation Channel                                     */
    uint16_t    maxNonTcpSpace; /**< Maximum CID value for Non-TCP.                         */
    uint16_t    fMaxPeriod;     /**< Largest number of compressed non-TCP headers that
                                     may be sent without sending a full header              */
    uint16_t    maxHeader;      /**< Largest header size in octets that may be compressed   */
    uint32_t    minWrap;        /**< Minimum time of generation value wrap around           */
    uint32_t    fMaxTime;       /**< Compressed headers may not be sent more than F_MAX_TIME
                                     seconds after sending last full header                 */
} t_PgIphcCompParam;

/**************************************************************************//**
 @Description   Traffic IPHC Paramaters Structure
*//***************************************************************************/
typedef struct
{
    uint8_t                    numOfIphcCompressors; /**< Number of IPHC compressors    */
    t_PgIphcCompParam          *p_IphcCompParams;     /**< IPHC compression parameters  */
} t_PgIphcParam;

/**************************************************************************//**
 @Description   Frame sub-list of frame list Structure

                This structure defines sub-list of TG or
                TA frame list. Entire frame list may consists
                of several sub-lists, each one of them specified
                by its id.
*//***************************************************************************/
typedef struct
{
    uint16_t     id;        /**< Frame identifier   */
    uint32_t     weight;    /**< Frame weightage    */
} t_PgFrameDescrParam;

/**************************************************************************//**
 @Description   Frame Descriptor parameters structure
*//***************************************************************************/
typedef struct
{
    uint32_t                        numOfFrameSettings; /**< Number of frame settings         */
    t_PgFrameSetting                *p_FrameSettings;   /**< Frame settings, array of size
                                                             numOfFrameSettings               */
    t_BufMemAccess                  bufMemAccess;       /**< Buffers memory access parameters */
} t_PgFrameDescriptorSetParam;

/**************************************************************************//*
 @Description   Traffic Frame structure
*//***************************************************************************/
typedef struct
{
    t_Handle     h_Frame;   /**< Frame handle */
    uint32_t     frameDescrSubSetIndex; /**< Frame Descriptor subset index */
    t_List       frmsNode;  /**< Frames node */
} t_PgFrame;

/**************************************************************************//**
 @Description   User's callback routine invoked by the PG module for frame-specific
                manipulations.

                This function is issued by PG module from within PG frame list
                API when serialized frame is created. It allows application
                to perform use case specific manipulations on the created
                frame, like e.g.
                - Modify different fields of the frame and
                create differentiation of frames for several IW connections
                when frames are still created from the same layout
                - Insert checksum errors into part of frames
                - Set frame info fields of t_Frame to be used later on
                by the driver.

 @Param[in]     h_Frame             - Handle of the Frame object (t_Frame structure)
 @Param[in]     h_UserManipParam    - Handle of user's parameters to be passed as is
                                      to callback function
 @Param[in]     frameId             - Frame ID
 @Param[in]     frameNum            - Sequential number of frame in the list

 @Return        E_OK on success, Error code otherwise
*//***************************************************************************/
typedef t_Error (t_PgUserFrameConfig)(t_Handle h_Frame,
                                      t_Handle h_UserManipParam,
                                      uint32_t frameId,
                                      uint32_t frameNum);

/**************************************************************************//*
 @Description   User manipulation callbacks mapping

                This structure specifies callback function per frame ID
*//***************************************************************************/
typedef struct
{
    uint32_t                        frameId;                /**< Frame Index                    */
    bool                            suppress;               /**< Don't create frames with frameId
                                                                 specified in the resulting list */
    t_PgUserFrameConfig             *f_UserFrameConfig;     /**< User-defined callback function
                                                                 for the specified frameId      */
} t_PgFrameCallBackMapSettings;

/**************************************************************************//*
 @Description   User Manipulation callback function and parameters structure
*//***************************************************************************/
typedef struct
{
    uint32_t                            numOfUserManipFrame;            /**< Number of map settings                                 */
    t_PgFrameCallBackMapSettings        *p_PgFrameCallBackMapSettings;  /**< Map settings for frame id and user callback function,
                                                                             array of size numOfUserManipFrame defining callback
                                                                             function per frame ID                                  */
    t_Handle                            h_UserManipParam;               /**< User manipulation parameters handle,
                                                                             passed as is to callback function                      */
} t_PgUserManipParam;

/**************************************************************************//*
 @Description   Packet Generator Frame List parameters structure
*//***************************************************************************/
typedef struct
{
    uint32_t                    numOfFrameListSubSet;       /**< Number of frame descriptors in the list                        */
    t_PgFrameDescrParam         *p_FrameListSettings;       /**< Pointer to frame list settings, array of size
                                                                 numOfFrameListSubSet                                           */
    t_PgUserManipParam          *p_UserManipParam;          /**< Pointer to user manipulation parameters                        */
    uint32_t                    *p_FrameSeqNum;             /**< Pointer to sequence number; user provides
                                                                 here starting sequence number for the frames
                                                                 in list and gets back the last used sequence
                                                                 number; sequence number of the first frame
                                                                 in list will be equal to (*p_FrameSeqNum + seqNumStep)         */
    uint32_t                    seqNumStep;                 /**< Step for increasing sequence number                            */
    uint32_t                    dataByteAlignment;          /**< First data byte alignment; if 0 random alignment will be used  */
} t_PgListParam;

 /**************************************************************************//*
 @Function      PG_CreateFrame

 @Description   Creates and intializes PG Frame object.

 @RetVal        Pointer to PG Frame object: Success.
 @RetVal        NULL: Failure.

 @Cautions      None.
*//***************************************************************************/
t_PgFrame* PG_CreateFrame(void);

/**************************************************************************//*
 @Function      PG_DestroyFrame

 @Description   Destroys the frame.

 @Param[in]     h_Frame - Handle to the frame.

 @RetVal        E_OK: Success.
 @RetVal        Error code: Failure.

 @Cautions      None.
*//***************************************************************************/
t_Error PG_DestroyFrame(t_Handle h_Frame);

/**************************************************************************//**
 @Function      PG_Init

 @Description   Creates and initializes Packet Generator object.

                The Packet Generator object holds the following:

                - Frame Descriptor object that defines structure/layout
                of frames for the use of TG and TA objects

                - IPHC object to be used for IP header compression, only
                when p_IphcParam differs from NULL

 @Param[in]     p_FrameDescriptorSettings   - Pointer to the frame descriptor settings.
 @Param[in]     p_IphcParam                 - Pointer to the IPHC parameter structure.

 @Return        Handle of packet generator object on success, NULL on failure

 @Cautions      None
*//***************************************************************************/
t_Handle PG_Init(t_PgFrameDescriptorSetParam *p_FrameDescriptorSettings,t_PgIphcParam  *p_IphcParam);

/**************************************************************************//**
 @Function      PG_Free

 @Description   Frees the Packet Generator object.

 @Param[in]     h_PacketGen - Handle to the packet generator.

 @Return        E_OK on success, Error code otherwise

 @Cautions      None.
*//***************************************************************************/
t_Error PG_Free(t_Handle h_PacketGen);

/**************************************************************************//**
 @Function      PG_IphcReset

 @Description   Resets IPHC information in the packet generator.

                This function must be used to reset IPHC context
                when multiple frame list are created for the same
                CID. It should be used prior to next frame list
                using the same CID creation to prevent recurring
                compression to be performed for the same CID.

 @Param[in]     h_PacketGen      - Handle to the packet generator.

 @Return        E_OK on success, Error code otherwise

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_IphcReset(t_Handle h_PacketGen);

/**************************************************************************//*
 @Function      PG_GetList

 @Description   Gets the handle to the list from the list map settings.

 @Param[in]     h_PgListSettings         - Handle to the list map settings.

 @Return        h_List                   - Handle to the frames list.

 @Cautions      None.
*//***************************************************************************/
 t_Handle PG_GetList(t_Handle h_PgListSettings);

/**************************************************************************//*
 @Function      PG_GetNumOfListSubSet

 @Description   Gets the number of id-weight pair.

 @Param[in]     h_PgListSettings         - Handle to the list map settings.

 @Param[in/out] p_NumOfFrameListSubSet   - Pointer to the number of id-weight pair.

 @Return        E_OK on success, Error code on failure.

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_GetNumOfListSubSet(t_Handle h_PgListSettings, uint32_t* p_NumOfFrameListSubSet);

/**************************************************************************//*
 @Function      PG_GetFrameWeights

 @Description   Gets the handle to the frame weights.

 @Param[in]     h_PgListSettings         - Handle to the list map settings.

 @Return        h_PgFrameWeights         - Handle to the frames weight.

 @Cautions      None.
*//***************************************************************************/
 t_Handle PG_GetFrameWeights(t_Handle h_PgListSettings);

/**************************************************************************//*
 @Function      PG_FragmentFrameList

 @Description   Takes an input frame list and IP fragments it according to
                a specific IP MTU.

                The frame list in transformed entirely by this function and,
                upon exit, will contain all the original frames but in IP
                fragments that relate to the specified MTU.

 @Param[in]     p_FrameList     - Address of the list to transform.
 @Param[in]     ipMtu           - The IP MTU that fragmentation will take into
                                  account.

 @Return        E_OK on success, an error code on failure

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_FragmentFrameList(t_List *p_FrameList, uint32_t ipMtu);

/**************************************************************************//**
 @Function      PG_CreateFrameList

 @Description   Creates frame list.

                This function performs the following set of operations:
                - Creates and initializes frame list object by frames of type
                specified by Frame Descriptor part PG object
                - The frames in the list are created in order of IDs as specified
                by user in p_FrameListSettings field of p_PgListParam
                - User specific frame configuration manipulations are possible
                through setting p_UserManipParam structure of p_PgListParam

 @Param[in]     h_PacketGen        - Handle to the packet generator object returned by
                                     PG_Init() function; Holds the informations needed
                                     for creating required frames.
 @Param[in]     p_PgListParam      - Pointer to list parameters.

 @Return        Handle to the Frame List on success, NULL on failure

 @Cautions      None.
*//***************************************************************************/
 t_Handle PG_CreateFrameList(t_Handle h_PacketGen, t_PgListParam *p_PgListParam);

/**************************************************************************//**
 @Function      PG_CreateFrameListFromExistingList

 @Description   Creates frame list from existing list.

                Creates a new frame list from existing list. Optional user
                manipulation on cloned frame may be set through p_UserManipParam
                parameter.

 @Param[in]     h_List              - Handle to the existing frames list
 @Param[in]     p_UserManipParam    - Pointer to the user defined frame manipulation parameters

 @Return        Handle to the new Frame List on success, NULL on failure

 @Cautions      None.
*//***************************************************************************/
 t_Handle PG_CreateFrameListFromExistingList(t_Handle h_List, t_PgUserManipParam *p_UserManipParam);

/**************************************************************************//**
 @Function      PG_MergeFrameLists

 @Description   Merges two or more existing frame lists.

                Appends one frame list to the tail of another frame list.

 @Param[in]     h_MergeLists        - Pointer to the array of list objects to be
                                      merged.
 @Param[in]     numOfMergeLists     - Number of list to be merged.

 @Return        Handle to the new merged Frame List on success, NULL on failure

 @Cautions      None
*//***************************************************************************/
 t_Handle PG_MergeFrameLists(t_Handle *h_MergeLists, uint32_t numOfMergeLists);

/**************************************************************************//**
 @Function      PG_FreeFrameList

 @Description   Frees the frames list.

 @Param[in]     h_List       - Handle to the frames list to be freed.

 @Return        None.

 @Cautions      None.
*//***************************************************************************/
 void PG_FreeFrameList(t_Handle h_List);

/**************************************************************************//**
 @Function      PG_GetFrameFromFrameList

 @Description   Returns the handle of the Nth frame from the frame list.

 @Param[in]     h_List       - Handle to the frame list
 @Param[in]     nthFrame     - Frame number for which frame handle has to be returned
                               (starting from 0)

 @Return        Handle to the Nth frame of the Frame list on success, NULL on failure

 @Cautions      None
*//***************************************************************************/
 t_Handle PG_GetFrameFromFrameList(t_Handle h_List, uint32_t nthFrame);

/**************************************************************************//**
 @Function      PG_HeaderInsert

 @Description   Inserts new header to the frame.

                This function may be optionally issued by user frame
                configuration callback function when header insertion
                is required. It adds one header to the frame.

                This function is one of several manipulation types
                available on h_Frame (see also PG_HeaderRemove(),
                PG_HeaderReplace() and PG_HeaderFieldManipulation()
                routines). Different manipulation can be performed
                sequentially one after the other.

                Serialized frame represented by h_Frame handle still
                isn't updated by the above manipulation functions. In
                order to confirm modifications and to create serialized
                frame including them PG_CreateManipFrame() routine
                should be issued by user frame configuration callback
                at the end of manipulations on the frame. During
                serializing of the modified frame all relevant protocol
                checksum fields are re-calculated.

 @Param[in]     h_Frame                   - Handle to the original serialized frame
 @Param[in]     operateOnHeader           - Type of the header to be inserted
 @Param[in]     insertAfterHeader         - Type of the header after that new one
                                            should be inserted
 @Param[in]     insertAfterHeaderIndex    - Index of the header of type specified
                                            by insertAfterHeader, within headers of
                                            the same type, after which new header should
                                            be inserted, starting from 0; For example,
                                            if IPv4 header should be added after 3rd VLAN
                                            header then operateOnHeader = e_HEADER_TYPE_IPv4,
                                            insertAfterHeader = e_HEADER_TYPE_VLAN and
                                            insertAfterHeaderIndex = 2
 @Param[in]     p_FrameSetting            - Pointer to frame layout including header
                                            to be inserted; only header to be inserted
                                            should be configured in this structure, all
                                            other fields are ignored

 @Return        E_OK on success, Error code on failure

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_HeaderInsert(t_Handle          h_Frame,
                         e_PgHeaderType    operateOnHeader,
                         e_PgHeaderType    insertAfterHeader,
                         uint8_t           insertAfterHeaderIndex,
                         t_PgFrameSetting  *p_FrameSetting);

/**************************************************************************//**
 @Function      PG_HeaderRemove

 @Description   Removes the specified existing frame header.

                This function may be optionally issued by user frame
                configuration callback function when header removal
                is required. It removes one header from the frame.

                This function is one of several manipulation types
                available on h_Frame (see also PG_HeaderInsert(),
                PG_HeaderReplace() and PG_HeaderFieldManipulation()
                routines). Different manipulation can be performed
                sequentially one after the other.

                Serialized frame represented by h_Frame handle still
                isn't updated by the above manipulation functions. In
                order to confirm modifications and to create serialized
                frame including them PG_CreateManipFrame() routine
                should be issued by user frame configuration callback
                at the end of manipulations on the frame. During
                serializing of the modified frame all relevant protocol
                checksum fields are re-calculated.

 @Param[in]     h_Frame                   - Handle to the original serialized frame
 @Param[in]     operateOnHeader           - Type of the header to be removed
 @Param[in]     operateOnHeaderIndex      - Sequential index of header to be removed,
                                            within header of the same operateOnHeader
                                            types, starting from 0; For example if
                                            3rd VLAN header should be removed then
                                            operateOnHeader = e_HEADER_TYPE_VLAN and
                                            operateOnHeaderIndex = 2

 @Return        E_OK on success, Error code on failure

 @Cautions      None.
*//***************************************************************************/
t_Error PG_HeaderRemove(t_Handle        h_Frame,
                        e_PgHeaderType  operateOnHeader,
                        uint8_t         operateOnHeaderIndex);

/**************************************************************************//**
 @Function      PG_HeaderReplace

 @Description   Replaces the specified existing frame header.

                This function may be optionally issued by user frame
                configuration callback function when header replacement
                is required. It replaces one header in the frame.

                This function is one of several manipulation types
                available on h_Frame (see also PG_HeaderInsert(),
                PG_HeaderRemove() and PG_HeaderFieldManipulation()
                routines). Different manipulation can be performed
                sequentially one after the other.

                Serialized frame represented by h_Frame handle still
                isn't updated by the above manipulation functions. In
                order to confirm modifications and to create serialized
                frame including them PG_CreateManipFrame() routine
                should be issued by user frame configuration callback
                at the end of manipulations on the frame. During
                serializing of the modified frame all relevant protocol
                checksum fields are re-calculated.

 @Param[in]     h_Frame                   - Handle to the original serialized frame
 @Param[in]     operateOnHeader           - Type of the header to be replaced
 @Param[in]     operateOnHeaderIndex      - Sequential index of header to be replaced,
                                            within header of the same operateOnHeader
                                            types, starting from 0; For example if
                                            3rd VLAN header should be replaced then
                                            operateOnHeader = e_HEADER_TYPE_VLAN and
                                            operateOnHeaderIndex = 2
 @Param[in]     p_FrameSetting            - Pointer to frame layout including header
                                            to be replaced by; only header for replacement
                                            should be configured in this structure, all
                                            other fields are ignored

 @Return        E_OK on success, Error code on failure

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_HeaderReplace(t_Handle          h_Frame,
                          e_PgHeaderType    operateOnHeader,
                          uint8_t           operateOnHeaderIndex,
                          t_PgFrameSetting  *p_FrameSetting);

/**************************************************************************//**
 @Function      PG_HeaderFieldManipulation

 @Description   Replaces the specified fields in the frame header.

                This function may be optionally issued by user frame
                configuration callback function when replacement of
                specific fields in the header is required. It replaces
                fields of one header in the frame.

                Functionality of this function is very similar to
                PG_HeaderReplace(). As opposed to PG_HeaderReplace()
                this function alters only header fields specified
                by validFields value without generating random
                values for others/unspecified fields.

                This function is one of several manipulation types
                available on h_Frame (see also PG_HeaderInsert(),
                PG_HeaderRemove() and PG_HeaderReplace() routines).
                Different manipulation can be performed sequentially
                one after the other.

                Serialized frame represented by h_Frame handle still
                isn't updated by the above manipulation functions. In
                order to confirm modifications and to create serialized
                frame including them PG_CreateManipFrame() routine
                should be issued by user frame configuration callback
                at the end of manipulations on the frame. During
                serializing of the modified frame all relevant protocol
                checksum fields are re-calculated.

 @Param[in]     h_Frame                   - Handle to the original serialized frame
 @Param[in]     operateOnHeader           - Type of the header to be modified
 @Param[in]     operateOnHeaderIndex      - Sequential index of header to be modified,
                                            within header of the same operateOnHeader
                                            types, starting from 0; For example if
                                            3rd VLAN header fileds should be changed then
                                            operateOnHeader = e_HEADER_TYPE_VLAN and
                                            operateOnHeaderIndex = 2
 @Param[in]     p_FrameSetting            - Pointer to frame layout including header
                                            to be modified; only header for modification
                                            should be configured in this structure, all
                                            other fields are ignored

 @Return        E_OK on success, Error code on failure

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_HeaderFieldManipulation(t_Handle            h_Frame,
                                    e_PgHeaderType      operateOnHeader,
                                    uint8_t             operateOnHeaderIndex,
                                    t_PgFrameSetting    *p_FrameSetting);

/**************************************************************************//**
 @Function      PG_CreateManipFrame

 @Description   Creates serialized manipulated frame.

                This function is last one in series of frame
                manipulation functions that may be optionally
                issued by user frame configuration callback
                (See also PG_HeaderInsert(), PG_HeaderRemove(),
                PG_HeaderReplace() and PG_HeaderFieldManipulation
                routines). It creates serialized frame including
                all previous modifications performed by the above
                functions. During serializing of the modified frame
                all relevant protocol checksum fields are re-calculated.

 @Param[in]     h_Frame         - Handle to the original serialized frame

 @Return        E_OK on success, Error code on failure

 @Cautions      None.
*//***************************************************************************/
 t_Error PG_CreateManipFrame(t_Handle h_Frame);

/** @} */ /* end of pg_grp */

/** @} */ /* end of itg_package_grp */

#endif /* __PG_EXT_H */

