/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          fmanip.c

  @Description   ITG Frame Manipulation services
*//***************************************************************************/
#include "std_ext.h"
#include "string_ext.h"

#include "fmanip.h"


static void* ConfigExtraParams_VLAN(void* confParams)
{
    t_ExtraParams_VLAN* vlanOpParams = NULL;

    UNUSED(confParams);
    vlanOpParams = XX_Malloc(sizeof(t_ExtraParams_VLAN));
    vlanOpParams->defaultVPri = 0;
    memset((void*)vlanOpParams->ipTosToVPriTable, 0, sizeof(uint8_t) * MAX_IP_TOS_VALUES);

    return vlanOpParams;
}

static void FreeExtraParams_VLAN(void* params)
{
    if(params)
        XX_Free(params);
}

static void* ConfigExtraParams_HM(void* confParams)
{
    t_ExtraParams_HM* hmParams = NULL;

    UNUSED(confParams);

    hmParams = XX_Malloc(sizeof(t_ExtraParams_HM));

    return hmParams;
}

static void FreeExtraParams_HM(void* params)
{
    if(params)
        XX_Free(params);
}

static void* ConfigExtraParams_PSP(void* confParams)
{
    UNUSED(confParams);
    return NULL;
}

static void FreeExtraParams_PSP(void* params)
{
    UNUSED(params);
}

static t_Error ConfigDefaultVPri(void* vlanParams, va_list argList)
{
    t_ExtraParams_VLAN* extraParams = (t_ExtraParams_VLAN*)vlanParams;
    uint8_t             newVPri     = (uint8_t)va_arg(argList, int);

    if(!extraParams) return E_INVALID_HANDLE;

    extraParams->defaultVPri = newVPri;

    return E_OK;
}

static t_Error  ConfigIpTosToVPriMapping(void* vlanParams,va_list argList)
{
    t_ExtraParams_VLAN* extraParams = (t_ExtraParams_VLAN*)vlanParams;

    uint8_t ipTos   = (uint8_t)va_arg(argList, int);
    uint8_t vPri    = (uint8_t)va_arg(argList, int);

    if(!extraParams) return E_INVALID_HANDLE;

    if(ipTos >= MAX_IP_TOS_VALUES) return E_INVALID_VALUE;

    extraParams->ipTosToVPriTable[ipTos] = vPri;

    return E_OK;
}

static t_Error GetDefaultVPri(t_ExtraParams_VLAN* vlanParams, uint8_t *p_VPri)
{
    if(!vlanParams) return E_INVALID_HANDLE;

    if(p_VPri) *p_VPri = vlanParams->defaultVPri;

    return E_OK;
}

static t_Error GetVPriForIpTos(t_ExtraParams_VLAN* vlanParams, uint8_t ipTos, uint8_t *p_VPri)
{
    if(!vlanParams)
        return E_INVALID_HANDLE;

    if(ipTos >= MAX_IP_TOS_VALUES)
        return E_INVALID_VALUE;

    if(p_VPri)
        *p_VPri = vlanParams->ipTosToVPriTable[ipTos];

    return E_OK;
}

static t_Error Execute_HEADER_INSERT(t_ItgHeaderManipulation* hmOp,t_Handle frame, void* extraParams )
{
    t_ExtraParams_HM* p_hmParams = (t_ExtraParams_HM*)extraParams;

    UNUSED(frame);

    if(!p_hmParams || !(p_hmParams->frameBuffer) )
        return E_INVALID_HANDLE;

    if(hmOp->hmSize > MAX_HEADER_INSERT)
    {
        XX_Print("the microcode does not support insert header of: %d", MAX_HEADER_INSERT);
    }

    if(p_hmParams->frame - hmOp->hmSize < p_hmParams->frameBuffer)
        return E_NO_MEMORY;

    return FRAME_BufferInsert(&p_hmParams->frame, &p_hmParams->frameSize, hmOp->p_Header,
                                                                hmOp->hmSize, hmOp->hmOffset, hmOp->hmDynamicInsMode);
    /*
    newFrmStart = p_hmParams->frame - hmOp->hmSize;

    memcpy(newFrmStart,p_hmParams->frame, hmOp->hmOffset);
    memcpy(newFrmStart + hmOp->hmOffset, hmOp->p_Header, hmOp->hmSize);

    p_hmParams->frame       = newFrmStart;

    if(hmOp->hmDynamicInsMode)
    {
        frmSize = (uint16_t)p_hmParams->frameSize;

        memcpy(p_hmParams->frame + hmOp->hmOffset + hmOp->hmSize - 2, &frmSize, 2 );
    }

    p_hmParams->frameSize   += hmOp->hmSize;

    return error;*/
}

static t_Error Execute_HEADER_REPLACE(t_ItgHeaderManipulation* hmOp,t_Handle frame, void* extraParams )
{
    t_ExtraParams_HM* p_hmParams = (t_ExtraParams_HM*)extraParams;

    UNUSED(frame);

    if(!p_hmParams || !(p_hmParams->frameBuffer) )
        return E_INVALID_HANDLE;

    /*if( p_hmParams->frameSize <= hmOp->hmOffset)
        return E_INVALID_VALUE;*/

    return FRAME_BufferReplace(&p_hmParams->frame, &p_hmParams->frameSize, hmOp->p_Header, hmOp->hmSize, hmOp->hmOffset);
    /*
    memcpy(p_hmParams->frame + hmOp->hmOffset, hmOp->p_Header,hmOp->hmSize);

    return E_OK;*/
}

static t_Error Execute_HEADER_REMOVE(t_ItgHeaderManipulation* hmOp,t_Handle frame, void* extraParams )
{
    t_ExtraParams_HM* p_hmParams = (t_ExtraParams_HM*)extraParams;

    UNUSED(frame);

    if(!p_hmParams || !(p_hmParams->frameBuffer) )
        return E_INVALID_HANDLE;

    if( p_hmParams->frameSize <= hmOp->hmOffset)
        return E_INVALID_VALUE;

    if(hmOp->hmSize >= MAX_HEADER_REMOVE)
    {
        XX_Print("the microcode does not support remove of headers greater than:%d",MAX_HEADER_REMOVE);
    }

    return FRAME_BufferRemove(&p_hmParams->frame, &p_hmParams->frameSize, hmOp->hmSize, hmOp->hmOffset);
    /*
    p_hmParams->frameSize -= hmOp->hmSize;

    memcpy(p_hmParams->frame + hmOp->hmOffset, p_hmParams->frame + hmOp->hmOffset + hmOp->hmSize, p_hmParams->frameSize);

    return E_OK;*/
}

static t_Error Execute_TRAILER_REMOVE(t_ItgHeaderManipulation* hmOp,t_Handle frame, void* extraParams )
{
    t_ExtraParams_HM* p_hmParams = (t_ExtraParams_HM*)extraParams;

    UNUSED(frame);

    if(!p_hmParams || !(p_hmParams->frameBuffer) )
        return E_INVALID_HANDLE;

    /*if( p_hmParams->frameSize <= hmOp->hmOffset)
        return E_INVALID_VALUE;

    p_hmParams->frameSize -= hmOp->hmSize;*/

    return FRAME_BufferRemove(&p_hmParams->frame, &p_hmParams->frameSize, hmOp->hmSize, (uint32_t)-1);

    return E_OK;

}

/*******************************************************************************************************/
/*removes all headers until the first after the mac header ( mac header = ETh , VLAN, SNAP)*/
/*******************************************************************************************************/
#define IS_MAC_HEADER(X) ( ((X) == e_HEADER_TYPE_ETH) || ((X) == e_HEADER_TYPE_VLAN) || ((X) == e_HEADER_TYPE_LLC) || ((X) == e_HEADER_TYPE_SNAP))

static t_Error Execute_REMOVE_MAC_HEADER(t_ItgProtocolSpecificOperation* pspOp, t_Handle frame, void* extraParams )
{
    t_Handle        hCurrentHeader = NULL;
    t_Error         error = E_OK;

    UNUSED(pspOp);
    UNUSED(extraParams);

    if( NULL != (hCurrentHeader = FRAME_GetNextHeaderByType( frame, NULL, e_HEADER_TYPE_ETH )) )
    {
        while ( (NULL != (hCurrentHeader = FRAME_GetNextHeaderByType(frame,NULL, HEADER_TYPE_ANY))) &&
                ( IS_MAC_HEADER(FRAME_GetHeaderType(frame, hCurrentHeader)) ) )
        {
            if( E_OK != (error = FRAME_ConfigRemoveHeader(frame,hCurrentHeader) ) )
            {
                XX_Print("\nError removing header in Execute_RM_MAC_HEADER : %d \n", error);

                return error;
            }
        }
    }
    else
    {
        XX_Print("\nEthernet header not found\n");
    }

    return error;

}

/*Remove PPPoE header. This HMCD removes the PPPoE header and all preceding headers.*/

static t_Error Execute_REMOVE_MAC_PPPoE_HEADER(t_ItgProtocolSpecificOperation* pspOp, t_Handle frame, void* extraParams )
{
    t_Handle        hCurrentHeader = NULL,
                    hPPPoEHeader = NULL;

    t_Error         error = E_OK;

    UNUSED(pspOp);
    UNUSED(extraParams);

    if( NULL != (hPPPoEHeader = FRAME_GetNextHeaderByType(frame,frame, e_HEADER_TYPE_PPPoE)) )
    {

        while ( (NULL != (hCurrentHeader = FRAME_GetPrevHeaderByType(frame,hPPPoEHeader, HEADER_TYPE_ANY))) )
        {
            if( E_OK != (error = FRAME_ConfigRemoveHeader(frame,hCurrentHeader)) )
            {
                XX_Print("\nError removing header in Execute_RM_MAC_PPPoE_HEADER : %d \n", error);

                return error;
            }
        }

    }
    else
    {
        XX_Print("\nPPPoE header not found\n");

    }

    return error;
}

static t_Error Execute_UPDATE_TCP_UDP_CHECKSUM(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{
    UNUSED(ipHeader);
    UNUSED(extraParams);
    UNUSED(pspOp);

    return E_NOT_AVAILABLE;
}

static t_Error Execute_UPDATE_TCP_UDP_SRC(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{
    t_Handle hTrsHeader = NULL;

    UNUSED(extraParams);

    if( NULL != (hTrsHeader = FRAME_GetNextHeaderByType(ipHeader, ipHeader, HEADER_TYPE_ANY)))
    {
        switch(FRAME_GetHeaderType(ipHeader,hTrsHeader))
        {
            case e_HEADER_TYPE_UDP:
            {
                if( E_OK == FRAME_SetHeaderField(  ipHeader,
                                                 hTrsHeader,
                                                 e_FIELD_UDP_PORT_SRC,
                                                 &(pspOp->replaceParam.sourcePort),
                                                 sizeof(pspOp->replaceParam.sourcePort) ) )
                {
                    return FRAME_ConfigHeaderAuto(  ipHeader,
                                                    hTrsHeader,
                                                    e_FIELD_UDP_CHECKSUM,
                                                    FALSE);
                }
            }
            case e_HEADER_TYPE_TCP:
            {
                if( E_OK == FRAME_SetHeaderField( ipHeader,
                                                hTrsHeader,
                                                e_FIELD_TCP_PORT_DST,
                                                &(pspOp->replaceParam.sourcePort),
                                                sizeof(pspOp->replaceParam.sourcePort)))
                {
                    return FRAME_ConfigHeaderAuto(  ipHeader,
                                                    hTrsHeader,
                                                    e_FIELD_TCP_CHECKSUM,
                                                    FALSE);
                }
            }
            default:
            {
                return E_NOT_AVAILABLE;
            }
        }
    }
    else
        return E_NOT_AVAILABLE;

}

static t_Error Execute_UPDATE_TCP_UDP_DST(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{
    t_Handle hTrsHeader = NULL;

    UNUSED(extraParams);

    if( NULL != (hTrsHeader = FRAME_GetNextHeaderByType(ipHeader, ipHeader, HEADER_TYPE_ANY)))
    {
        switch(FRAME_GetHeaderType(ipHeader,hTrsHeader))
        {
            case e_HEADER_TYPE_UDP:
            {
                return FRAME_SetHeaderField(  ipHeader,
                                                 hTrsHeader,
                                                 e_FIELD_UDP_PORT_DST,
                                                 &(pspOp->replaceParam.destinationPort),
                                                 sizeof(pspOp->replaceParam.destinationPort));
            }
            case e_HEADER_TYPE_TCP:
            {
                return FRAME_SetHeaderField( ipHeader,
                                                hTrsHeader,
                                                e_FIELD_TCP_PORT_DST,
                                                &(pspOp->replaceParam.destinationPort),
                                                sizeof(pspOp->replaceParam.destinationPort));
            }
            default:
            {
                return E_NOT_AVAILABLE;
            }
        }
    }
    else
        return E_NOT_AVAILABLE;

}
static t_Error Execute_UPDATE_IP_TOS(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{

    UNUSED(extraParams);

    return FRAME_SetHeaderField( ipHeader,
                                    ipHeader,
                                    e_FIELD_IPv4_TOS,
                                    &(pspOp->replaceParam.ipTos),
                                    sizeof(pspOp->replaceParam.ipTos));

}

static t_Error Execute_UPDATE_IP_SRC(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{
    UNUSED(extraParams);

    return FRAME_SetHeaderField( ipHeader,
                                    ipHeader,
                                    e_FIELD_IPv4_SRC_IP,
                                    &(pspOp->replaceParam.ipSourceAddress),
                                    sizeof(pspOp->replaceParam.ipSourceAddress));
}

static t_Error Execute_UPDATE_IP_DST(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{
    UNUSED(extraParams);

    return FRAME_SetHeaderField( ipHeader,
                                    ipHeader,
                                    e_FIELD_IPv4_DST_IP,
                                    &(pspOp->replaceParam.ipDestinationAddress),
                                    sizeof(pspOp->replaceParam.ipDestinationAddress));
}

static t_Error Execute_UPDATE_IP_TTL(t_ItgProtocolSpecificOperation* pspOp, t_Handle ipHeader, void* extraParams )
{
    uint8_t ipTos = 0;
    t_Error error = E_OK;

    UNUSED(extraParams);
    UNUSED(pspOp);

    if( E_OK == (error = FRAME_GetHeaderField(  ipHeader,
                                                ipHeader,
                                                e_FIELD_IPv4_TOS,
                                                &ipTos, sizeof(ipTos) )))
    {

        ipTos = (uint8_t)((ipTos > 0) ?(ipTos - 1):ipTos);
        if( E_OK == (error = FRAME_SetHeaderField(   ipHeader,
                                                        ipHeader,
                                                        e_FIELD_IPv4_TOS,
                                                        &ipTos,
                                                        sizeof(ipTos) ) ) )
            return E_OK;
    }

    return error;
}


/* specific execute function for a VLAN header manipulation operation*/
t_Error Execute_VLAN(t_Handle h_OpData, t_Handle h_Frame, void* p_extraParams)
{
    t_ItgVlanOperation  *p_VlanOp = (t_ItgVlanOperation*) h_OpData;
    uint16_t            numVTags = 0, tmpVlanOps, tmpVlanOps2, tmpVlanOps3, tmpFieldValue;
    uint8_t             tmpIpTos, tmpVPri;
    t_FrameStruct       *p_Hdr, *p_Eth = NULL, *p_VTag1 = NULL, *p_VTag2 = NULL, *p_IPv4 = NULL;
    t_Error             error;

    t_PgVlanHeader      vlanHeader;
    e_PgHeaderType      nextHeaderType;

   memset(&vlanHeader,0,sizeof(t_PgVlanHeader));

    if(!p_VlanOp) return E_INVALID_VALUE;

    /* Count the VTags in h_Frame */

    p_Hdr = (t_FrameStruct*)h_Frame;
    while(p_Hdr != NULL)
    {
        switch(p_Hdr->HeaderID)
        {
            case e_HEADER_TYPE_ETH:

                if(!p_Eth)
                {
                    p_Eth = p_Hdr;
                    p_Hdr = (t_FrameStruct*)FRAME_GetNextHeaderByType(h_Frame, p_Hdr, HEADER_TYPE_ANY);
                    if(p_Hdr && p_Hdr->HeaderID == e_HEADER_TYPE_VLAN)
                    {
                        p_VTag1 = p_Hdr; numVTags++;
                        p_Hdr = (t_FrameStruct*)FRAME_GetNextHeaderByType(h_Frame, p_Hdr, HEADER_TYPE_ANY);
                        if(p_Hdr && p_Hdr->HeaderID == e_HEADER_TYPE_VLAN)
                        {
                            p_VTag2 = p_Hdr;
                            numVTags++;
                        }
                    }
                }

                continue;

            case e_HEADER_TYPE_IPv4:
                if(!p_IPv4) p_IPv4 = p_Hdr;
                break;

            default:
                 return E_INVALID_VALUE;
        }

        p_Hdr = (t_FrameStruct*)FRAME_GetNextHeaderByType(h_Frame, p_Hdr, HEADER_TYPE_ANY);
    }


    if(!numVTags)   /* UNTAGGED FRAMES */
    {
        tmpVlanOps =  (uint16_t)(p_VlanOp->vlanOps & (
            e_ITG_CDESC_OP_VLAN_NO_QTAG_INS_QTAG1 |
            e_ITG_CDESC_OP_VLAN_NO_QTAG_INS_QTAG1_AND_QTAG2));
    }
    else if(numVTags == 1)  /* SINGLE-TAGGED FRAMES */
    {
        /* Stage 1: Validate the VLAN Ops */

        tmpVlanOps = (uint16_t)(p_VlanOp->vlanOps & (
            e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1 |
            e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1_AND_QTAG2));

        tmpVlanOps2 = (uint16_t)(p_VlanOp->vlanOps & (
            e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID |
            e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID_ZERO_WITH_DEFAULT |
            e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1 |
            e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1_IF_VID_ZERO));

        if( tmpVlanOps2 &&
            (p_VlanOp->vlanOps & e_ITG_CDESC_OP_VLAN_QTAG1_RM_QTAG1 ||
            tmpVlanOps == e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1_AND_QTAG2))
            return E_INVALID_VALUE;

        switch(tmpVlanOps2)
        {
            case 0:
            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID:
            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID_ZERO_WITH_DEFAULT:
            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1:
            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1_IF_VID_ZERO:
                break;

            default:
                return E_INVALID_VALUE;
        }

        switch(tmpVlanOps)
        {
            case 0:
            case e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1:
            case e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1_AND_QTAG2:
                break;

            default:
                return E_INVALID_VALUE;
        }


        /* Stage 2: Do the changes to the frame as indicated by the combination of VLAN Ops */

        if(p_VlanOp->vlanOps & e_ITG_CDESC_OP_VLAN_QTAG1_RM_QTAG1)
        {
            /*  Remove p_VTag1 */

            error = FRAME_ConfigRemoveHeader(h_Frame, p_VTag1);
            RETURN_IF_ERROR(error);
            p_VTag1 = NULL;
        }

        switch(tmpVlanOps2)
        {
            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID:

                /* Replace VID portion of p_VTag1 with that in QTag2 */

                tmpFieldValue = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);

                break;

            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID_ZERO_WITH_DEFAULT:

                /* If p_VTag1->VID == 0, replace VID portion of p_VTag1 with that in QTag2 */

                error = FRAME_GetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                if(!tmpFieldValue)
                {
                    tmpFieldValue = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
                }

                break;

            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1:

                /* Replace p_VTag1 with QTag2 */

                tmpFieldValue   = (uint16_t)(p_VlanOp->qTci2>>13);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VPRI,       &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<3)>>15);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_CFI,        &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VID,        &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
//              error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_LENGTH, &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
//              RETURN_IF_ERROR(error);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_TYPE,       &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
                RETURN_IF_ERROR(error);

                break;

            case e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1_IF_VID_ZERO:

                /* If p_VTag1->VID == 0, replace p_VTag1 with QTag2 */

                error = FRAME_GetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                if(!tmpFieldValue)
                {
                    tmpFieldValue   = (uint16_t)(p_VlanOp->qTci2>>13);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VPRI,       &tmpFieldValue,     sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
                    tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<3)>>15);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_CFI,        &tmpFieldValue,     sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
                    tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VID,        &tmpFieldValue,     sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
//                  error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_LENGTH, &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
//                  RETURN_IF_ERROR(error);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_TYPE,       &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
                    RETURN_IF_ERROR(error);
                }

                break;
        }

        /* Insertion of VTags is done after this if-else block */
    }
    else    /* MULTIPLE-TAGGED FRAMES */
    {
        /* Stage 1: Validate the VLAN Ops */

        tmpVlanOps = (uint16_t)(p_VlanOp->vlanOps & (
            e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1 |
            e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1_AND_QTAG2));

        tmpVlanOps2 = (uint16_t)(p_VlanOp->vlanOps & (
            e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1 |
            e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1_AND_QTAG2));

        tmpVlanOps3 = (uint16_t)(p_VlanOp->vlanOps & (
            e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_QTAG1 |
            e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_ZERO_WITH_DEFAULT |
            e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1 |
            e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_IF_VID_ZERO |
            e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_AND_QTAG2));

        if( (tmpVlanOps == e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1_AND_QTAG2 ||
             tmpVlanOps2 == e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1_AND_QTAG2)
         && (tmpVlanOps3 ||
             (tmpVlanOps == e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1 ||
              tmpVlanOps2 == e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1))
         && (tmpVlanOps3 == e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_AND_QTAG2))
         return E_INVALID_VALUE;

        switch(tmpVlanOps)
        {
            case 0:
            case e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1:
            case e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1_AND_QTAG2:
                break;

            default:
                 return E_INVALID_VALUE;
        }

        switch(tmpVlanOps2)
        {
            case 0:
            case e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1:
            case e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1_AND_QTAG2:
                break;

            default:
                return E_INVALID_VALUE;
        }

        switch(tmpVlanOps3)
        {
            case 0:
            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_QTAG1:
            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_ZERO_WITH_DEFAULT:
            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1:
            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_IF_VID_ZERO:
            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_AND_QTAG2:
                break;

            default:
                return E_INVALID_VALUE;
        }


        /* Stage 2: Do the changes to the frame as indicated by the combination of VLAN Ops */

        switch(tmpVlanOps)
        {

            case e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1_AND_QTAG2:

                /*  Remove pVTag2 */

                error = FRAME_ConfigRemoveHeader(h_Frame, p_VTag2);
                RETURN_IF_ERROR(error);
                p_VTag2 = NULL;

            case e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1:

                /*  Remove p_VTag1 */

                error = FRAME_ConfigRemoveHeader(h_Frame, p_VTag1);
                RETURN_IF_ERROR(error);
                p_VTag1 = NULL;

                HDR_Eth_Init(h_Frame); /*update the etype field accordingly */

                /* this is a hax, please redesign this area */
                    p_Hdr = (t_FrameStruct*)h_Frame;
                    p_Eth = NULL;
                    p_IPv4 = NULL;
                    while(p_Hdr != NULL)
                    {
                        switch(p_Hdr->HeaderID)
                        {
                            case e_HEADER_TYPE_ETH:

                                if(!p_Eth)
                                {
                                    p_Eth = p_Hdr;
                                    p_Hdr = (t_FrameStruct*)FRAME_GetNextHeaderByType(h_Frame, p_Hdr, HEADER_TYPE_ANY);
                                    if(p_Hdr && p_Hdr->HeaderID == e_HEADER_TYPE_VLAN)
                                    {
                                        p_VTag1 = p_Hdr; numVTags++;
                                        p_Hdr = (t_FrameStruct*)FRAME_GetNextHeaderByType(h_Frame, p_Hdr, HEADER_TYPE_ANY);
                                        if(p_Hdr && p_Hdr->HeaderID == e_HEADER_TYPE_VLAN)
                                        {
                                            p_VTag2 = p_Hdr;
                                            numVTags++;
                                        }
                                    }
                                }

                                continue;

                            case e_HEADER_TYPE_IPv4:
                                if(!p_IPv4) p_IPv4 = p_Hdr;
                                break;

                           default:
                                return E_INVALID_VALUE;
                        }

                        p_Hdr = (t_FrameStruct*)FRAME_GetNextHeaderByType(h_Frame, p_Hdr, HEADER_TYPE_ANY);
                    }
                /* end of ugly hax */


                break;
        }

        switch(tmpVlanOps3)
        {
            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_QTAG1:

                /* Replace VID portion of p_VTag1 with that in QTag2 */

                tmpFieldValue = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);

                break;

            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_ZERO_WITH_DEFAULT:

                /* If p_VTag1->VID == 0, replace VID portion of p_VTag1 with that in QTag2 */

                error = FRAME_GetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                if(!tmpFieldValue)
                {
                    tmpFieldValue = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
                }

                break;

            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_AND_QTAG2:

                /* Replace p_VTag2 with QTag2 */

                tmpFieldValue   = (uint16_t)(p_VlanOp->qTci2>>13);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VPRI,       &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<3)>>15);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_CFI,        &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VID,        &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
//              error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_LENGTH, &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
//              RETURN_IF_ERROR(error);
                error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_TYPE,       &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
                RETURN_IF_ERROR(error);

            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1:

                /* Replace p_VTag1 with QTag1 */

                tmpFieldValue   = (uint16_t)(p_VlanOp->qTci1>>13);
                error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VPRI,       &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                tmpFieldValue   = (uint16_t)((p_VlanOp->qTci1<<3)>>15);
                error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_CFI,        &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                tmpFieldValue   = (uint16_t)((p_VlanOp->qTci1<<4)>>4);
                error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID,        &tmpFieldValue,     sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
//              error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_LENGTH, &p_VlanOp->qType1,  sizeof(p_VlanOp->qType1));
//              RETURN_IF_ERROR(error);
                error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_TYPE,       &p_VlanOp->qType1,  sizeof(p_VlanOp->qType1));
                RETURN_IF_ERROR(error);

                break;

            case e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_IF_VID_ZERO:

                /* If p_VTag1->VID == 0, replace p_VTag1 with QTag2 */

                error = FRAME_GetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VID, &tmpFieldValue, sizeof(tmpFieldValue));
                RETURN_IF_ERROR(error);
                if(!tmpFieldValue)
                {
                    tmpFieldValue   = (uint16_t)(p_VlanOp->qTci2>>13);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VPRI,       &tmpFieldValue,     sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
                    tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<3)>>15);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_CFI,        &tmpFieldValue,     sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
                    tmpFieldValue   = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_VID,        &tmpFieldValue,     sizeof(tmpFieldValue));
                    RETURN_IF_ERROR(error);
//                  error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_LENGTH, &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
//                  RETURN_IF_ERROR(error);
                    error = FRAME_SetHeaderField(h_Frame, p_VTag2, e_FIELD_VLAN_TYPE,       &p_VlanOp->qType2,  sizeof(p_VlanOp->qType2));
                    RETURN_IF_ERROR(error);
                }

                break;
        }

        tmpVlanOps = tmpVlanOps2;
    }

    /* Insertion of VTags is done in one place (i.e. here) for all types of frames: */

    switch(tmpVlanOps)
    {
        case 0:

            //ASSERT_COND(!numVTags); /* Can't get here if the input frame is tagged! */
            break;

        case e_ITG_CDESC_OP_VLAN_NO_QTAG_INS_QTAG1_AND_QTAG2:
        case e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1_AND_QTAG2:
        case e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1_AND_QTAG2:

            /* Insert QTag2 after Ethernet SA (QTag1 to be inserted before it afterwards) */

             vlanHeader.vPri = (uint8_t)(p_VlanOp->qTci2>>13);
             vlanHeader.cfi = (uint8_t)((p_VlanOp->qTci2<<3)>>15);
             vlanHeader.vid = (uint16_t)((p_VlanOp->qTci2<<4)>>4);
             vlanHeader.typeLen = p_VlanOp->qType2;
             vlanHeader.validFields = e_FIELD_VLAN_VPRI | e_FIELD_VLAN_CFI | e_FIELD_VLAN_VID | e_FIELD_VLAN_TYPE;
             error = FRAME_GetHeaderField(h_Frame, p_Eth, e_FIELD_ETH_NEXT_HEADER_TYPE,&nextHeaderType,sizeof(nextHeaderType));
             RETURN_IF_ERROR(error);
             vlanHeader.nextHeaderType = nextHeaderType;

            p_VTag2 = (t_FrameStruct*)FRAME_ConfigInsertHeader(h_Frame, INSERT_AFTER, p_Eth, e_HEADER_TYPE_VLAN, &vlanHeader);
            if(!p_VTag2)
                return E_NO_MEMORY;

             nextHeaderType = e_HEADER_TYPE_VLAN;
             error = FRAME_SetHeaderField(h_Frame, p_Eth, e_FIELD_ETH_NEXT_HEADER_TYPE, &nextHeaderType, sizeof(nextHeaderType));
             RETURN_IF_ERROR(error);
        case e_ITG_CDESC_OP_VLAN_NO_QTAG_INS_QTAG1:
        case e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1:
        case e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1:

            /* Insert QTag1 right after Ethernet SA */
             vlanHeader.vPri = (uint8_t)(p_VlanOp->qTci1>>13);
             vlanHeader.cfi = (uint8_t)((p_VlanOp->qTci1<<3)>>15);
             vlanHeader.vid = (uint16_t)((p_VlanOp->qTci1<<4)>>4);
             vlanHeader.typeLen = p_VlanOp->qType1;
             vlanHeader.validFields = e_FIELD_VLAN_VPRI | e_FIELD_VLAN_CFI | e_FIELD_VLAN_VID | e_FIELD_VLAN_TYPE;
             error = FRAME_GetHeaderField(h_Frame, p_Eth, e_FIELD_ETH_NEXT_HEADER_TYPE,&nextHeaderType,sizeof(nextHeaderType));
             RETURN_IF_ERROR(error);
             vlanHeader.nextHeaderType = nextHeaderType;

            p_VTag1 = (t_FrameStruct*)FRAME_ConfigInsertHeader(h_Frame, INSERT_AFTER, p_Eth, e_HEADER_TYPE_VLAN,&vlanHeader);
            if(!p_VTag1)
            {
                if(p_VTag2) FRAME_ConfigRemoveHeader(h_Frame, p_VTag2);
                return E_NO_MEMORY;
            }
             nextHeaderType = e_HEADER_TYPE_VLAN;
             error = FRAME_SetHeaderField(h_Frame, p_Eth, e_FIELD_ETH_NEXT_HEADER_TYPE, &nextHeaderType, sizeof(nextHeaderType));
             RETURN_IF_ERROR(error);
            {
            /*b01108: fast workaround: must update the type field of the ethernet header after inserting the VLAN*/
            /* TO DO here: verify that the type/length field of the ethernet header is set to type not length */
            t_Handle h_Eth = FRAME_GetNextHeaderByType(h_Frame, h_Frame, e_HEADER_TYPE_ETH);
            if(!h_Eth)
                return E_INVALID_VALUE;
            else
            {
                uint16_t vlanType = 0x8100;
                error = FRAME_SetHeaderField(h_Frame, h_Eth, e_FIELD_ETH_TYPE, &vlanType,   sizeof(vlanType));
                RETURN_IF_ERROR(error);
            }
            }

            break;

        default:

            ASSERT_COND(!numVTags); /* Can't get here if the input frame is tagged! */
            return E_INVALID_VALUE;
    }

    if(p_VlanOp->vlanOps & e_ITG_CDESC_OP_VLAN_QTAG_VPRI_MARK)
    {
        /* Mark VPRI field in p_VTag1 with either the value of the VPRI Field
           from the IP_TOS_to_Vpri_Table or the default VPRI from the Global IW PRAM
         */
        if(p_IPv4)
        {
            error = FRAME_GetHeaderField(h_Frame, p_IPv4, e_FIELD_IPv4_TOS, &tmpIpTos, sizeof(tmpIpTos));
            RETURN_IF_ERROR(error);
            error = GetVPriForIpTos(p_extraParams, tmpIpTos, &tmpVPri);
            RETURN_IF_ERROR(error);
        }
        else
        {
            error = GetDefaultVPri(p_extraParams, &tmpVPri);
            RETURN_IF_ERROR(error);
        }

        error = FRAME_SetHeaderField(h_Frame, p_VTag1, e_FIELD_VLAN_VPRI, &tmpVPri, sizeof(tmpVPri));
        RETURN_IF_ERROR(error);
    }
    return E_OK;
}

/* specific execute function for a generic header manipulation operation*/
t_Error Execute_HM(t_Handle h_OpData, t_Handle h_Frame, void* p_extraParams)
{
    t_ItgHeaderManipulation *p_HmDesc   = (t_ItgHeaderManipulation*) h_OpData;
    t_ExtraParams_HM            *p_HmParams = (t_ExtraParams_HM*)p_extraParams;
    t_Error                     error       = E_OK;
    t_Error (*pfnFunc)(t_ItgHeaderManipulation* hmOp,t_Handle frame, void* extraParams) = NULL;

    if(!p_HmDesc || !p_HmParams || !h_Frame)
        return E_INVALID_VALUE;

    switch(p_HmDesc->hmOperation)
    {

        case e_ITG_CDESC_HEADER_INSERT:
            pfnFunc = Execute_HEADER_INSERT;
            break;
        case e_ITG_CDESC_HEADER_REMOVE:
            pfnFunc = Execute_HEADER_REMOVE;
            break;
        case e_ITG_CDESC_HEADER_REPLACE:
            pfnFunc = Execute_HEADER_REPLACE;
            break;
        case e_ITG_CDESC_TRAILER_REMOVE:
            pfnFunc = Execute_TRAILER_REMOVE;
            break;
        default:
            {
                XX_Print("Undefined generic header manipulation operation");
                XX_Exit(1);
            }break;
    }

    error = pfnFunc(p_HmDesc, h_Frame, p_extraParams);
    return error;
}

#define REPLACE_FLAGS  (e_ITG_CDESC_UPDATE_IP_DST | e_ITG_CDESC_UPDATE_IP_SRC | e_ITG_CDESC_UPDATE_IP_TTL \
                            | e_ITG_CDESC_UPDATE_IP_TOS | e_ITG_CDESC_UPDATE_TCP_UDP_DST | e_ITG_CDESC_UPDATE_TCP_UDP_SRC \
                            | e_ITG_CDESC_UPDATE_TCP_UDP_CHECKSUM)

t_Error Execute_PSP(t_Handle opData, t_Handle frame, void* extraParams)
{
    t_ItgProtocolSpecificOperation*    pspOpData   = (t_ItgProtocolSpecificOperation*)opData;
    t_Error                             error       = E_OK;
    t_Handle                            hHeader     = frame;
    t_Error (*pfnFunc)(t_ItgProtocolSpecificOperation* pspOpData, t_Handle hHeader, void* extraParams) = NULL;
    UNUSED(frame);
    UNUSED(extraParams);

    switch ( pspOpData->hmProtocol )
    {
        case e_ITG_CDESC_PROTOCOL_ETH:
        {
            hHeader = frame;

            /* check for remove flags (mutual exclusive)*/
            if (FLAG_CHECK(pspOpData->hmPspFlags, e_ITG_CDESC_REMOVE_MAC_HEADER))
                pfnFunc = Execute_REMOVE_MAC_HEADER;
            else if (FLAG_CHECK(pspOpData->hmPspFlags, e_ITG_CDESC_REMOVE_MAC_PPPoE_HEADER))
                pfnFunc = Execute_REMOVE_MAC_PPPoE_HEADER;
            if ( pfnFunc && (E_OK != pfnFunc(pspOpData, hHeader, extraParams)) )
                  {
                      XX_Print("Failed to apply the remove operation\n");
                      XX_Exit(1);
                  }

            /* check for replace flags (not mutual exclusive) */
            if(FLAG_CHECK(pspOpData->hmPspFlags, REPLACE_FLAGS))
                    {
                        /* ip specific header operations */
                        if( (e_HEADER_TYPE_IPv4 != FRAME_GetHeaderType(frame, hHeader)) &&
                            (NULL == (hHeader = FRAME_GetNextHeaderByType(frame,hHeader, e_HEADER_TYPE_IPv4)) ) )
                        {
                            XX_Print("\nNo IP header found for protocol specific operations\n");
                            XX_Exit(1);
                        }

                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_IP_DST )
                            if (E_OK != Execute_UPDATE_IP_DST(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_IP_DST operation\n");
                                XX_Exit(1);
                            }
                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_IP_SRC )
                            if (E_OK != Execute_UPDATE_IP_SRC(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_IP_SRC operation\n");
                                XX_Exit(1);
                            }
                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_IP_TTL )
                            if (E_OK != Execute_UPDATE_IP_TTL(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_IP_TTL operation\n");
                                XX_Exit(1);
                            }
                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_IP_TOS )
                            if (E_OK != Execute_UPDATE_IP_TOS(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_IP_TOS operation\n");
                                XX_Exit(1);
                            }
                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_TCP_UDP_DST )
                            if (E_OK != Execute_UPDATE_TCP_UDP_DST(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_TCP_UDP_DST operation\n");
                                XX_Exit(1);
                            }
                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_TCP_UDP_SRC )
                            if (E_OK != Execute_UPDATE_TCP_UDP_SRC(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_TCP_UDP_SRC operation\n");
                                XX_Exit(1);
                            }
                        if ( pspOpData->hmPspFlags & e_ITG_CDESC_UPDATE_TCP_UDP_CHECKSUM )
                            if (E_OK != Execute_UPDATE_TCP_UDP_CHECKSUM(pspOpData, hHeader, extraParams))
                            {
                                XX_Print("Failed to apply the e_ITG_CDESC_UPDATE_TCP_UDP_CHECKSUM operation\n");
                                XX_Exit(1);
                            }
                    }
        }
        break;
        default:
        {
            return E_NOT_AVAILABLE;
        }

    }

    return error;
}

/*********************************************************************/
/* t_Handle Config(t_FrameManipParams gpParams) initialize the GoldenPattern object */
/*********************************************************************/
t_Handle FMANIP_Config(t_Handle h_Params)
{
    t_FrameManip *frameManip      = (t_FrameManip*)XX_Malloc(sizeof(t_FrameManip));

    UNUSED(h_Params);

    if(!frameManip)
        return NULL;

    memset(frameManip, 0x00, sizeof(t_FrameManip));
    frameManip->hmLast = frameManip->hmChain = NULL;

    /*initialize the executing functions specific for the different header manipulation op types*/
    frameManip->exe[e_OP_VLAN] = Execute_VLAN;
    frameManip->exe[e_OP_PSP] = Execute_PSP;
    frameManip->exe[e_OP_HM] = Execute_HM;

    /* Register additional params specific to a type of header manip */
    frameManip->exeParams[e_OP_HM] = ConfigExtraParams_HM(NULL);
    frameManip->exeParams[e_OP_VLAN] = ConfigExtraParams_VLAN(NULL);
    frameManip->exeParams[e_OP_PSP] = ConfigExtraParams_PSP(NULL);

   return frameManip;
}

t_Error FMANIP_Free(t_Handle h_GpObj)
{
    t_FrameManip*           frameManip  = (t_FrameManip*)h_GpObj;
    t_HeaderManipChainOp*   currentPos  = NULL;

    while( frameManip->hmChain )
    {
        currentPos = frameManip->hmChain->p_Next;
        if(frameManip->hmChain->OpDescriptor.pOpParams != NULL)
        {
            XX_Free(frameManip->hmChain->OpDescriptor.pOpParams);
        }

        XX_Free(frameManip->hmChain);
        frameManip->hmChain = currentPos;
    }

    FreeExtraParams_VLAN(frameManip->exeParams[e_OP_VLAN]);
    FreeExtraParams_HM(frameManip->exeParams[e_OP_HM]);
    FreeExtraParams_PSP(frameManip->exeParams[e_OP_PSP]);

    XX_Free(frameManip);
    return E_OK;
}

t_Error FMANIP_GetGoldenPattern( t_Handle h_GpObject, t_Handle h_Frame, uint8_t** p_Frame, uint32_t* p_FrameSize)
{
    t_FrameManip*           frameManip  = (t_FrameManip*)h_GpObject;
    t_HeaderManipChainOp*   currPos     = frameManip->hmChain;
    t_ExtraParams_HM*       hmParams    = NULL;
    t_Error                 error       = E_OK;

    if(!frameManip)
        return E_INVALID_VALUE;

    if(!currPos)
    {
        // no header manipulation

        return FRAME_Serialize(h_Frame, p_Frame, p_FrameSize);
    }

    /*while there are still more operations in the header manipulation chain execute them*/
    while(  currPos &&
            (currPos->OpDescriptor.operationType != e_OP_HM) &&
            ( (currPos->OpDescriptor.operationType > e_OP_NO_OP) && (currPos->OpDescriptor.operationType < e_OP_LAST) ) &&
            ( E_OK == ( error = frameManip->exe[currPos->OpDescriptor.operationType](  (currPos->OpDescriptor.pOpParams),
                                                                                      h_Frame,
                                                                                      frameManip->exeParams[currPos->OpDescriptor.operationType]) ) ) )
    {
        currPos = currPos->p_Next;
    }

    /*we are at the end of the op chain but no error had occured*/
    if (error == E_OK)
    {
        hmParams                = frameManip->exeParams[e_OP_HM];

        hmParams->frameSize     = FRAME_GetFrameSize( h_Frame, h_Frame );

        if(!hmParams->frameSize)
            return E_INVALID_VALUE;

        hmParams->frameMaxSize  = hmParams->frameSize + MAX_HM_SIZE;

        hmParams->frameBuffer   = XX_Malloc(hmParams->frameMaxSize);

        if(!hmParams->frameBuffer)
            return E_NO_MEMORY;

        hmParams->frame         = hmParams->frameBuffer + MAX_HM_SIZE;


        memset(hmParams->frameBuffer, 0x00, hmParams->frameMaxSize);

        if (E_OK == (error = FRAME_Serialize(h_Frame, &(hmParams->frame), &(hmParams->frameSize) ) ) )
        {

            while(  currPos &&
                    (currPos->OpDescriptor.operationType == e_OP_HM) &&
                     ( E_OK == ( error = frameManip->exe[currPos->OpDescriptor.operationType](  (currPos->OpDescriptor.pOpParams),
                                                                                                 h_Frame,
                                                                                                 hmParams) ) ) )
            {
                currPos = currPos->p_Next;
}

            if(E_OK == error)
            {
                if(*p_FrameSize >= hmParams->frameSize)
                {
                    memcpy(*p_Frame, hmParams->frame, hmParams->frameSize );
                    *p_FrameSize = hmParams->frameSize;

                }
                else
                {
                    XX_Print("Buffer to small");
                    error = E_NO_MEMORY;
                }
            }

        }

        XX_Free(hmParams->frameBuffer);

        return error;
    }

   XX_Print("No such header manipulation\n");

   return error;
}

static t_Error FMANIP_AddOperationToList(t_FrameManip *p_FrameManip, t_HeaderManipChainOp *p_Operation)
{
    ASSERT_COND(p_FrameManip && p_Operation);

     if(p_FrameManip->hmLast)
            p_FrameManip->hmLast->p_Next  = p_Operation;

        p_FrameManip->hmLast          = p_Operation;

        if(!p_FrameManip->hmChain)
            p_FrameManip->hmChain = p_Operation;

        return E_OK;
}

t_Error FMANIP_ConfigAddHeaderManipulation(t_Handle h_FrameManip, t_ItgHeaderManipulation *p_Operation)
{
    t_FrameManip    *frameManip  = (t_FrameManip*)h_FrameManip;
    t_HeaderManipChainOp    *newElm      = NULL;

    ASSERT_COND(p_Operation);

    newElm  = XX_Malloc(sizeof(t_HeaderManipChainOp));
    if (!newElm)
        return E_NO_MEMORY;

    memset(newElm, 0x00, sizeof(t_HeaderManipChainOp));

    newElm->OpDescriptor.pOpParams = XX_Malloc(sizeof(t_ItgHeaderManipulation));
    if (!newElm->OpDescriptor.pOpParams)
        return E_NO_MEMORY;

    memcpy(newElm->OpDescriptor.pOpParams, p_Operation, sizeof(t_ItgHeaderManipulation));
    newElm->OpDescriptor.operationType  = e_OP_HM;

    /* add the operation to the end of the list */
    return FMANIP_AddOperationToList(frameManip, newElm);
}

t_Error FMANIP_ConfigAddProtocolSpecific(t_Handle h_FrameManip, t_ItgProtocolSpecificOperation *p_Operation)
{
    t_FrameManip    *frameManip  = (t_FrameManip*)h_FrameManip;
    t_HeaderManipChainOp    *newElm      = NULL;
    t_ItgProtocolSpecificOperation *pPSP = NULL;
    t_ItgHeaderManipulation pGeneralHM;

   ASSERT_COND(p_Operation);

   pPSP = (t_ItgProtocolSpecificOperation*)p_Operation;

   if ((pPSP->hmProtocol == e_ITG_CDESC_PROTOCOL_PPPoHT) &&
       (pPSP->hmPspFlags == e_ITG_CDESC_INSERT_MAC_PPPoE))
    {
        //change the type from psp to general header manip
        pGeneralHM.h_HeaderMemSeg = pPSP->h_HeaderMemSeg;
        pGeneralHM.hmOperation = e_ITG_CDESC_HEADER_INSERT;
        pGeneralHM.hmDynamicInsMode = TRUE;
        pGeneralHM.hmSize = pPSP->insertParam.size;
        pGeneralHM.p_Header = pPSP->insertParam.p_InsertHeader;
        pGeneralHM.hmOffset = 0;
        return FMANIP_ConfigAddHeaderManipulation(h_FrameManip,  &pGeneralHM);
    }
    else
    if (pPSP->hmProtocol == e_ITG_CDESC_PROTOCOL_ETH)
    {
        if (pPSP->hmPspFlags == e_ITG_CDESC_INSERT_PID_HEADER)
        {
            //change the type from psp to general header manip
            pGeneralHM.h_HeaderMemSeg = pPSP->h_HeaderMemSeg;
            pGeneralHM.hmOperation = e_ITG_CDESC_HEADER_INSERT;
            pGeneralHM.hmDynamicInsMode = FALSE;
            pGeneralHM.hmSize = pPSP->insertParam.size;
            pGeneralHM.p_Header = pPSP->insertParam.p_InsertHeader;
            pGeneralHM.hmOffset = 0;
            return FMANIP_ConfigAddHeaderManipulation(h_FrameManip,  &pGeneralHM);
        }

        if (pPSP->hmPspFlags == e_ITG_CDESC_REMOVE_MAC_HEADER)
        {
        }
    }

    /* add the operation as it is - no need to convert it */
    newElm  = XX_Malloc(sizeof(t_HeaderManipChainOp));
    if (!newElm)
        return E_NO_MEMORY;

    memset(newElm, 0x00, sizeof(t_HeaderManipChainOp));

    newElm->OpDescriptor.pOpParams = XX_Malloc(sizeof(t_ItgProtocolSpecificOperation));
    if (!newElm->OpDescriptor.pOpParams)
        return E_NO_MEMORY;

    memcpy(newElm->OpDescriptor.pOpParams, p_Operation, sizeof(t_ItgProtocolSpecificOperation));
    newElm->OpDescriptor.operationType  = e_OP_PSP;

    /* add the operation to the end of the list */
    return FMANIP_AddOperationToList(frameManip, newElm);
}


t_Error FMANIP_ConfigAddVlanOperation (t_Handle h_FrameManip, t_ItgVlanOperation *p_VlanOp)
{
    t_FrameManip*           frameManip  = (t_FrameManip*)h_FrameManip;
    t_HeaderManipChainOp*   newElm      = NULL;

   ASSERT_COND(p_VlanOp);

    newElm  = XX_Malloc(sizeof(t_HeaderManipChainOp));
    if (!newElm)
        return E_NO_MEMORY;

    memset(newElm, 0x00, sizeof(t_HeaderManipChainOp));

    newElm->OpDescriptor.pOpParams = XX_Malloc(sizeof(t_ItgVlanOperation));
    if (!newElm->OpDescriptor.pOpParams)
        return E_NO_MEMORY;

    memcpy(newElm->OpDescriptor.pOpParams, p_VlanOp, sizeof(t_ItgVlanOperation));
    newElm->OpDescriptor.operationType  = e_OP_VLAN;

    /* add the operation to the end of the list */
    return FMANIP_AddOperationToList(frameManip, newElm);
}

t_Error FMANIP_ConfigExtraParam(t_Handle h_FManip, e_HmType opType,...)
{
    t_FrameManip    *p_FManip = (t_FrameManip*)h_FManip;
    va_list         argList;
    t_Error         error    = E_OK;

    if(!p_FManip) return E_INVALID_HANDLE;

    va_start(argList, opType);

    switch(opType)
    {
        case e_OP_VLAN:
            {
                switch(va_arg(argList,e_AddParams_VLAN))
                {
                    case e_DEFAULT_VPRI:
                        {
                            error = ConfigDefaultVPri(p_FManip->exeParams[e_OP_VLAN],argList);
                        }
                    break;
                    case e_IPTOS_TO_VPRI_MAPPING:
                        {
                           error = ConfigIpTosToVPriMapping(p_FManip->exeParams[e_OP_VLAN],argList);
                        }
                    break;
                    default:
                    {
                        XX_Print("Unknown VLAN parameter");

                        error = E_INVALID_VALUE;

                        break;
                    }
                }
            }
        break;
        case e_OP_HM:
            {
                error = E_NOT_AVAILABLE;
            }
        break;
        case e_OP_PSP:
            {
                error = E_NOT_AVAILABLE;
            }
        break;
        default:
        {
            XX_Print("Operation not supported");

            error = E_INVALID_VALUE;

            break;
        }
    }

    va_end(argList);

    return error;

}

