/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          fmanip.h

  @Description   ITG Frame Manipulation Common Structures And Definitions
*//***************************************************************************/
#ifndef __FMANIP_H__
#define __FMANIP_H__

#include "std_ext.h"
#include "error_ext.h"
#include "stdarg_ext.h"
#include "endian_ext.h"
#include "fmanip_ext.h"

#include "frame.h"


#define MAX_IP_TOS_VALUES       0x40
#define RETURN_IF_ERROR(error)  if(error) return (error)
#define MAX_FRAME_BUFFER_SIZE   0xffff
#define MAX_HM_SIZE             500

//op specific limits(from the spec)
#define MAX_HEADER_INSERT       96
#define MAX_HEADER_REMOVE       120

typedef t_Error (*t_pfnExecute)(t_Handle h_OpData, t_Handle h_Frame, void* p_extraParams);

typedef enum
{
    GENERIC_HM_INSERT = 1,
    GENERIC_HM_REMOVE,
    GENERIC_HM_REPLACE,
    GENERIC_HM_TRAILER_REMOVE
} e_GenericHmOpType;

typedef struct
{
    t_Handle    hFrame;         /*handle to the input frame/frame generation object */
    t_Handle    hHeaderManipOp; /*handle to the list of header manipulation operations to apply on the input frame*/
    uint8_t*    pBuffer;        /* if not null this buffer will be used to store the resulting frame*/
    uint32_t    bufferSize;     /* if pBuffer not null this is his size */

} t_FrameManipParams;

typedef struct strOpHM
{
    struct
    {
        e_HmType operationType;
        void*   pOpParams;
    } OpDescriptor;

    struct strOpHM *p_Next;
} t_HeaderManipChainOp;


/***************************************************************************/
/* t_FrameManip: encapsulates internal data for a FrameManip Object */
/****************************************************************************/
typedef struct
{
    t_HeaderManipChainOp *hmChain;
    t_HeaderManipChainOp *hmLast;
    t_pfnExecute                          exe[e_OP_LAST];
    void                          *exeParams[e_OP_LAST];
}t_FrameManip;

typedef struct
{
    uint8_t                 defaultVPri;
    uint8_t                 ipTosToVPriTable[MAX_IP_TOS_VALUES];
}t_ExtraParams_VLAN;


typedef struct
{
    uint8_t*     frameBuffer;
    uint8_t*     frame;
    uint32_t     frameSize;
    uint32_t     frameMaxSize;
}t_ExtraParams_HM;

/* specific execute functions for generic, PSP and VLAN header manipulation operation*/
t_Error Execute_VLAN(t_Handle h_OpData, t_Handle h_Frame, void* p_extraParams);
t_Error Execute_HM(t_Handle h_OpData, t_Handle h_Frame, void* p_extraParams);
t_Error Execute_PSP(t_Handle h_OpData, t_Handle h_Frame, void* p_extraParams);


#endif //__FMANIP_H__

