/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          fmanip_ext.h

  @Description   ITG Frame Manipulation Data Structures And Definitions
*//***************************************************************************/
#ifndef __FMANIP_EXT_H__
#define __FMANIP_EXT_H__

#include "std_ext.h"
#include "error_ext.h"


/**************************************************************************//**
@Description   Specifies the operation to be performed on the frame
*//***************************************************************************/
typedef enum {
      e_ITG_CDESC_HEADER_INSERT     = 0x01    /**< Insert Header */
    , e_ITG_CDESC_HEADER_REMOVE     = 0x02    /**< Remove Header */
    , e_ITG_CDESC_HEADER_REPLACE    = 0x03    /**< Replace Header */
    , e_ITG_CDESC_TRAILER_REMOVE    = 0x04    /**< Remove Trailer */
} e_ItgHmOperation;

/**************************************************************************//**
@Description    Instructs which type of Header Insert/Replace HMCD to generate
                for its containing HM Operation Descriptor.
*//***************************************************************************/
typedef enum {
      e_ITG_CDESC_INTERNAL    = 0x01    /**< Header is located in Multiuser RAM */
    , e_ITG_CDESC_EXTERNAL    = 0x02    /**< Header is located in external RAM */
    , e_ITG_CDESC_LOCAL       = 0x03    /**< Header is located inside of this HMCD */
} e_ItgHeaderManipType;

/**************************************************************************//**
@Description   Protocol Types for Specific Header Manipulation
*//***************************************************************************/
typedef enum {
    e_ITG_CDESC_PROTOCOL_ETH  = 0 /**< Ethernet protocol is used for specific header manipulation operations. */
  , e_ITG_CDESC_PROTOCOL_PPPoHT   /**< PPP protocol is used for specific header manipulation operations. */
  , e_ITG_CDESC_PROTOCOL_ATM      /**< ATM protocol is used for specific header manipulation operations. */
} e_ItgProtocol;

/**************************************************************************//**
 @Description   Connection Descriptor IP/UDP insert mode
*//***************************************************************************/
typedef enum {
      e_ITG_CDESC_INSERT_MODE_NONE          = 0x00
    , e_ITG_CDESC_INSERT_MODE_IP            = 0x01        /**< Insert IP header; set up the length and generate the checksum of IP */
    , e_ITG_CDESC_INSERT_MODE_UDP           = 0x02        /**< Insert UDP header; set up the length and generate the checksum of UDP */
    , e_ITG_CDESC_INSERT_MODE_IP_AND_UDP    = 0x03        /**< Insert IP/UDP headers; set up the lengths and generate the checksums of IP and UDP */
} e_ItgIpUdpInsertMode;

typedef enum {
    e_OP_NO_OP  = 0,
    e_OP_VLAN   = 1,
    e_OP_HM     = 2,
    e_OP_PSP    = 3,
    e_OP_LAST
} e_HmType;

typedef enum {
    e_DEFAULT_VPRI,
    e_IPTOS_TO_VPRI_MAPPING
} e_AddParams_VLAN;

/**************************************************************************//**
@Description    VLAN Operations; these values should be ORed together in order
                to achieve the desired VLAN functionality.

@Cautions       There are groups of values that are mutually exclusive,
                i.e. values which are visibly grouped together in this listing.
*//***************************************************************************/
typedef enum {
      e_ITG_CDESC_OP_VLAN_NO_OPERATION                      = 0x0000    /**< No VLAN Operation - has no effect if ORed with other values \n <hr> */

    , e_ITG_CDESC_OP_VLAN_NO_QTAG_INS_QTAG1                 = 0x4000    /**< Insert QTag1 in untagged frames */
    , e_ITG_CDESC_OP_VLAN_NO_QTAG_INS_QTAG1_AND_QTAG2       = 0x8000    /**< Insert QTag1 and QTag2 in untagged frames \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG1_RM_QTAG1                    = 0x2000    /**< Remove the QTag from single tagged frames \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1                   = 0x0800    /**< Insert QTag1 in single tagged frame */
    , e_ITG_CDESC_OP_VLAN_QTAG1_INS_QTAG1_AND_QTAG2         = 0x1000    /**< Insert QTag1 and QTag2 in single tagged frame \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID                    = 0x0100    /**< Replace VID portion of QTag in single tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG1_REPL_VID_ZERO_WITH_DEFAULT  = 0x0200    /**< If VID=0 replace VID with default value in single tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1                  = 0x0300    /**< Replace QTag (with QTag1) in single tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG1_REPL_QTAG1_IF_VID_ZERO      = 0x0400    /**< If VID=0 replace QTag (with QTag1) in single tagged frames \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG_VPRI_MARK                    = 0x0080    /**< Mark priority bits in the outermost VLAN tag (applicable only to VLAN-tagged frames) \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1                    = 0x0040    /**< Remove outermost QTag from multiple-tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG2_RM_QTAG1_AND_QTAG2          = 0x0060    /**< Remove outermost two QTags from multiple-tagged frames \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1                   = 0x0008    /**< Insert QTag1 in multiple tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG2_INS_QTAG1_AND_QTAG2         = 0x0010    /**< Insert QTag1 and QTag2 in multiple tagged frames \n <hr> */

    , e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_QTAG1              = 0x0001    /**< Replace VID of outermost QTag with that in QTag 1 in multiple-tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG2_REPL_VID_ZERO_WITH_DEFAULT  = 0x0002    /**< If VID=0 replace VID of outermost QTag with VID of QTag1 (?) in multiple-tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1                  = 0x0003    /**< Replace outermost QTag with QTag1 in multiple-tagged frames */
    , e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_IF_VID_ZERO      = 0x0004    /**< If VID=0 replace outermost QTag in multiple-tagged frames. The new QTag value must be specified in qType2 and qTci2 */
    , e_ITG_CDESC_OP_VLAN_QTAG2_REPL_QTAG1_AND_QTAG2        = 0x0005    /**< Replace two outermost QTags with QTag1 and QTag2 */
} e_ItgVlanOperation;

/*********************************************************************************//**
@Description   Protocol Specific Operations to be performed on the frame.
               Should be used on flows after a parsing filter has been invoked.
*//**********************************************************************************/
typedef enum {
      e_ITG_CDESC_UPDATE_IP_DST           = 0x0001    /**< Update the IP destination address from the IP header */
    , e_ITG_CDESC_UPDATE_IP_SRC           = 0x0002    /**< Update the IP source address from the IP header */
    , e_ITG_CDESC_UPDATE_IP_TTL           = 0x0004    /**< Decrement by 1 the Time to Live field from the IP header */

    , e_ITG_CDESC_UPDATE_IP_TOS           = 0x0008    /**< Update the IP TOS from the IP header using the t_ItgHeaderReplace structure */
    , e_ITG_CDESC_UPDATE_IP_TOS_BY_POLICER= 0x0010    /**< Update the IP TOS from the IP header based on policer ETH output */

    , e_ITG_CDESC_UPDATE_TCP_UDP_DST      = 0x0020    /**< Update the TCP/UDP destination port from the TCP/UDP header */
    , e_ITG_CDESC_UPDATE_TCP_UDP_SRC      = 0x0040    /**< Update the TCP/UDP source port from the TCP/UDP header */
    , e_ITG_CDESC_UPDATE_TCP_UDP_CHECKSUM = 0x0080    /**< Update the TCP/UDP checksum from the TCP/UDP header */

    , e_ITG_CDESC_REMOVE_MAC_HEADER       = 0x1000    /**< Remove Ethernet/802.3 MAC header (assuming it is first in the frame) */
    , e_ITG_CDESC_REMOVE_MAC_PPPoE_HEADER = 0x2000    /**< Remove PPPoE header and all preceding headers */

    , e_ITG_CDESC_REMOVE_ETH_TRAILER      = 0x4000    /**< Remove Ethernet/802.3 FCS and padding if exist */

    , e_ITG_CDESC_INSERT_PID_HEADER       = 0x0100    /**< Insert PPP PID header */
    , e_ITG_CDESC_INSERT_IP_UDP_HEADER    = 0x0200    /**< Insert IP/UDP header in ETH frame */

    , e_ITG_CDESC_REMOVE_2684_ETH_TRAILER = 0x8000    /**< Remove 2684 bridged frame trailer */

    , e_ITG_CDESC_INSERT_MAC_PPPoE        = 0x0100    /**< Insert the ETHERNET MAC Header and PPPoE header */
    , e_ITG_CDESC_COMPRESS_IP_UDP_HEADER  = 0x0800    /**< Compress IP/UDP header */
} e_ItgHmProtocolSpecificFlag;


/**************************************************************************//**
@Description   Header Manipulation Operation Descriptor
*//***************************************************************************/
typedef struct {
    e_ItgHmOperation        hmOperation;          /**< Operation specifier */
    bool                    hmDynamicInsMode;     /**< Option to automatically update the length field of the frame to be transmitted in the last two bytes which are being inserted.
                                                       The length will be calculated as the original frame length received and will take into account all the header manipulation done on the frame up to this point */
    uint8_t                 hmSize;               /**< Size of header to insert/remove/replace or size of trailer to remove */
    uint8_t                 hmOffset;             /**< Offset from start of frame where to insert header/where header to remove/replace starts */
    e_ItgHeaderManipType    hmType;               /**< Type of memory where is the header to insert/replace: internal/external/local */
    t_Handle                h_HeaderMemSeg;       /**< Handle of preallocated memory segment used for storing the header (must be consistent with the header type and mem type) */
    uint8_t                 *p_Header;            /**< Pointer to header to be inserted/replaced */
} t_ItgHeaderManipulation;

/**************************************************************************//**
@Description   Protocol Specific Header Replacement Data
*//***************************************************************************/
typedef struct {
    uint32_t    ipSourceAddress;            /**< IP Encapsulation - IP Source Address */
    uint32_t    ipDestinationAddress;       /**< IP Encapsulation - IP Destination Address */
    uint8_t     ipTos;                      /**< IP Encapsulation - IP TOS */
    uint16_t    sourcePort;                 /**< TCP or UDP Source Port */
    uint16_t    destinationPort;            /**< TCP or UDP Destination Port */
} t_ItgHeaderReplace;

/**************************************************************************//**
@Description   Protocol Specific Header Insert Data
*//***************************************************************************/
typedef struct {
    uint8_t                 size;               /**< Size of header, in bytes, to be inserted in frame. */
    uint8_t                 ipUdpOffset;        /**< Offset from the beginning of the frame in bytes, where the header is being inserted. Only used for IP/UDP header insert */
    e_ItgIpUdpInsertMode    ipUdpMode;          /**< Only used for IP/UDP header insert */
    uint8_t                 *p_InsertHeader;    /**< Pointer to header to be inserted
                                                     For PPP protocol: - for e_ITG_CDESC_INSERT_MAC_PPPoE flag: pointer to the ETHERNET and PPPoE headers (the length field of the PPPoE header will be updated) */
} t_ItgHeaderInsert;

/**************************************************************************//**
@Description   Protocol Specific Header Compress
*//***************************************************************************/
typedef struct {
    uint16_t    cid;            /**< Context Identifier */
    t_Handle    h_Compressor;   /**< Handle to the Compressor object */
} t_ItgCompressIpUdpHeader;

/**************************************************************************//**
@Description   Protocol Specific Header Manipulation Operation Descriptor
*//***************************************************************************/
typedef struct {
    e_ItgProtocol              hmProtocol;         /**< Protocol type for specific header manipulation operations */
    uint16_t                   hmPspFlags;         /**< Protocol-specific flags used for header manipulation opetations. Combination of flags from e_ItgHmProtocolSpecificFlag according to the protocol;*/
    e_ItgHeaderManipType       hmType;             /**< Type of memory where is the header to replace: internal/local */
    t_Handle                   h_HeaderMemSeg;     /**< Handle of preallocated memory segment used for storing the header (must be consistent with the header type and mem type) */
    t_ItgHeaderReplace         replaceParam;       /**< Header Replace Parameters */
    t_ItgHeaderInsert          insertParam;        /**< Header Insert Parameters */
    t_ItgCompressIpUdpHeader   hmCompressorParams; /**< IP Header Compressor specific parameters */
} t_ItgProtocolSpecificOperation;

/**************************************************************************//**
@Description   VLAN Operation Descriptor
*//***************************************************************************/
typedef struct {
    uint16_t    vlanOps;              /**< As many of the ::e_ItgVlanOperation enum values as needed, ORed together (restrictions apply - some values are mutually exclusive) */
    uint8_t     vlanHdrManOffset;     /**< The offset of the first byte in the Ethernet header from the beginning of the frame */
    uint16_t    qType1;               /**< Type of the first of the two user-specified QTags used for VLAN replacements */
    uint16_t    qTci1;                /**< TCI (VPri & VID) of the first of the two user-specified QTags used for VLAN replacements */
    uint16_t    qType2;               /**< Type of the second of the two user-specified QTags used for VLAN replacements */
    uint16_t    qTci2;                /**< TCI (VPri & VID) of the second of the two user-specified QTags used for VLAN replacements */
} t_ItgVlanOperation;

/*********************************************************************/
/* t_Handle Config(t_FrameManipParams fmParams) initialize the FrameManipulator object */
/*********************************************************************/
t_Handle FMANIP_Config(t_Handle params);

/*********************************************************************/
/* t_Handle t_Error FManip_Free(t_Handle frameManip); frees the frame manipulator resources */
/*********************************************************************/
t_Error FMANIP_Free(t_Handle frameManip);

/********************************************************************************/
/* FMANIP_GetGoldenPattern returns address of the frame after header manipulation  *****/
/* if on Config the user provided a valid buffer and the size was greater or*****/
/* equal to the resulted frame that buffer is used, if the size was smaller *****/
/* the function returns error; if no buffer was provided a buffer is allocated***/
/********************************************************************************/
t_Error FMANIP_GetGoldenPattern( t_Handle h_GpObject, t_Handle h_Frame, uint8_t** p_Frame, uint32_t* p_FrameSize);

/********************************************************************************/
/* Some header manipulation operations need additional information in order *****/
/* to execute properly, The user must set this values after the Config and  *****/
/* before FMANIP_GetFrame call or else default values will be used.         *****/
/********************************************************************************/
t_Error FMANIP_ConfigExtraParam(t_Handle h_FManip, e_HmType opType,...);/* fieldType(enum), value(s) */

/********************************************************************************/
/* Use the following function to add a new header manipulation operation        */
/* in the chain of operations that will define the golden pattern               */
/* The generic header manipulation will take place after the protocol specific ones *****/
t_Error FMANIP_ConfigAddHeaderManipulation(t_Handle h_FrameManip, t_ItgHeaderManipulation *p_Operation);
t_Error FMANIP_ConfigAddProtocolSpecific(t_Handle h_FrameManip, t_ItgProtocolSpecificOperation *p_Operation);
t_Error FMANIP_ConfigAddVlanOperation (t_Handle h_FrameManip, t_ItgVlanOperation *p_VlanOp);


#endif /* __FMANIP_EXT_H__ */

