/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/***************************************************************************//**
@File          frame.h

@Description   Frame Data Structures And Definitions
*//***************************************************************************/

#ifndef _FRAME__H
#define _FRAME__H

#include "xx_ext.h"
#include "stdlib_ext.h"
#include "frame_ext.h"
/**************************************************************************//**
@Group          InternalFrame

@Description    Internal frame defines, structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Internal Frame Defines
*//***************************************************************************/
#define _BIG_ENDIAN_                        1           /* Enable/Disable big endian format */
#define CONVERT_ENDIANNESS                  1           /* Convert the endianness during serialize */
#define KEEP_ENDIANNESS                     0           /* Donot convert the endianness during serialize */

#define CRC_TABLE_SIZE                      256         /* CRC table size */
#define POLYNOMIAL                          0x04c11db7L /* Standard CRC-32 polynomial */
#define CRC16_POLYNOMIAL                    0x8408      /* CRC-16 polynomial */

#define MIN_FRAME_LEN_ETHERNET              64          /* Minimum Ethernet frame size */
#define MAX_FRAME_LEN_ETHERNET              1518        /* Maximum Ethernet frame size */
#define MAX_FRAME_LEN_PAYLOAD               1500        /* Maximum payload size */
#define MIN_FRAME_LEN_PAYLOAD               0           /* Minimum frame size */
#define MAX_FRAME_SIZE_IP                   65536       /* Maximum IP frame size */

#define FIXED_HEADER_SIZE_ETH               14          /* Ethernet header size */
#define FIXED_HEADER_SIZE_VLAN              4           /* VLAN header size */
#define FIXED_HEADER_SIZE_PPPoE             6           /* PPPoE header size */
#define FIXED_HEADER_SIZE_SNAP              5           /* SNAP header size */
#define HEADER_SIZE_ETH_IN_SNAP             2           /* Ethernet header size in SNAP header */
#define FIXED_HEADER_SIZE_PPP_MLMC_RFC2686  4           /* MLMC RFC 2686 frame size */
#define FIXED_HEADER_SIZE_PPP_MLMC_RFC1990  6           /* MLMC RFC 1990 frame size */
#define FIXED_HEADER_SIZE_ATM               5           /* ATM header size */
#define FIXED_HEADER_SIZE_IP                20          /* IPv4 header size */
#define FIXED_HEADER_SIZE_IPv6              40          /* IPv6 header size */
#define FIXED_HEADER_SIZE_TCP               20          /* TCP header size */
#define FIXED_HEADER_SIZE_UDP               8           /* UDP header size */
#define FIXED_HEADER_SIZE_SCTP              12          /* SCTP Common Header Size */
#define FIXED_HEADER_SIZE_SCTP_CHUNK_DATA   16          /* SCTP Chunk Data Header Size */
#define FIXED_HEADER_SIZE_PSEUDO_IP         12          /* Pseudo IP header size */
#define FIXED_HEADER_SIZE_IEEE1588_V1_MESSAGE       36  /* IEEE 1588 Version 1 message header size */
#define FIXED_HEADER_SIZE_IEEE1588_V2_MESSAGE       34  /* IEEE 1588 Version 2 message header size */
#define FIXED_HEADER_SIZE_MPLS              4                   /* MPLS header sixe*/
#define FIXED_HEADER_SIZE_L2TPv2            6           /* L2TPv2 header size - Flag bits + Version + TunnelId + SesssionId */
#define FIXED_HEADER_SIZE_L2TPv3_DATA_MESSAGE        12 /* L2TPv3 Data message header size - Flag bits + Version + SesssionId + L2 specific sublayer */
#define FIXED_HEADER_SIZE_L2TPv3_CONTROL_MESSAGE     6  /* L2TPv3 Control message header size - Flag bits + Version + ControlConnectionID */
#define FIXED_HEADER_SIZE_PWE3_ENCAP        4           /* PWE3 encap header size*/
#define FIXED_HEADER_SIZE_RTP               12          /* RTP header size */

#define DEFAULT_PADDING_VALUE               0x00        /* Default padding value if value is not configured by the user */
#define MAX_PADDING_LIMIT                   16          /* Maximum padding limit */

#define RECURSIVE                           1           /* Flag to free the frame headers recursively */
#define NON_RECURSIVE                       0           /* Flag to disable the recursive free */

#define ETH_ADDR_SIZE                       6           /* Ethernet address size */
#define MAX_ETH_LEN_VAL                     1500        /* Maximum Ethernet length value */
#define ETH_FCS_SIZE                        4           /* Ethernet 32 bit FCS size */

#define HDLC_16_BIT_FCS_SIZE                2           /* HDLC 16 bit FCS size */
#define HDLC_32_BIT_FCS_SIZE                4           /* HDLC 32 bit FCS size */

#define MAX_VLAN_LEN_VAL                    1500        /* Maximum VLAN length value */

#define COMPRESSED_PPP_HEADER_SIZE          1           /* Compressed PPP header size */
#define PPP_HEADER_SIZE                     2           /* PPP header size */

#define MASK_MLMC_4_BIT_CLASS_ID            0x0F        /* Allow only last 4 bits in the byte */
#define MASK_MLMC_2_BIT_CLASS_ID            0x03        /* Allow only last 2 bits in the byte */
#define MASK_MLMC_24_BIT_SEQ_NUM            0x00FFFFFF  /* Allow only last 24 bits in the word */
#define MASK_MLMC_12_BIT_SEQ_NUM            0x00000FFF  /* Allow only last 12 bits in the word */
#define SET_MLMC_RESERVED_2_BIT             0x000000FC  /* Allow only last 2 bits in the byte */
#define OFFSET_MLMC_END_FRAG_BIT            1           /* End fragement bit offset */
#define OFFSET_MLMC_4_BIT_CLASS_ID          4           /* 4-bit class id offset */
#define OFFSET_MLMC_2_BIT_CLASS_ID          2           /* 2-bit class id offset */
#define OFFSET_MLMC_24_BIT_SEQ_NUM          24          /* 24-bit sequence number offset */
#define OFFSET_MLMC_12_BIT_SEQ_NUM          12          /* 12-bit sequence number offset */
#define OFFSET_MLMC_RESERVED                2           /* 2-bit reserved field offset */

#define MASK_IPV4_FLAG_D                    0xFD        /* Allow only the first bit in the byte */
#define MASK_IPV4_FLAG_M                    0xFE        /* Allow only the second bit in the byte */
#define OFFSET_IPV4_FLAG_D                  1           /* IPv4 D flag offset */
#define OFFSET_IPV4_FLAG_M                  0           /* IPv4 M flag offset */
#define IPv4_ADDR_SIZE                      4           /* IPv4 address size */
#define IPv4_VERSION                        4           /* IPv4 version */
#define IPv6_VERSION                        6           /* IPv6 version */

#define MAX_SCTP_CHUNK_SIZE                 0xFFFF      /* length is stored on 2 bytes */
#define SCTP_UNORDERED_FLAG                 0x0004      /* U bit*/
#define SCTP_BEGINNING_FLAG                 0x0002      /* B bit*/
#define SCTP_END_FLAG                       0x0001      /* E bit*/

#define HDLC_8Bit_ADDRESS_SIZE              2           /* HDLC header size */
#define HDLC_16Bit_ADDRESS_SIZE             3           /* HDLC header size */

#define OFFSET_TCP_RESERVED                 4           /* Reseved field offset */

#define RTP_VERSION                         2           /* RTP version */
#define MAX_RTP_PAYLOAD_TYPES               128         /* Maximum RTP payload types */

#define PRBS_32BITS                         32
#define PRBS_16BITS                         16
#define PRBS_8BITS                          8

#define FLAG_SET(value, flag)               value |= flag
#define FLAG_CLEAR(value, flag)             value &= ~flag
#define FLAG_CHECK(value, flag)             ((value & flag) ? (1) : (0))

#define MEMALLOC                            XX_Malloc
#define MEMFREE                             XX_Free
#define SAFE_COPY_VALUE(src, dst, src_size, dst_size) {ASSERT_COND(dst_size>=src_size); memcpy(dst, src, src_size);}

/**************************************************************************//**
 @Description   Internal Frame Structure
*//***************************************************************************/
typedef struct {
    e_PgHeaderType    HeaderID;     /* Header ID */
    t_Handle           h_HeaderStruct;  /* Handle to the header */
    t_List             node;            /* */
} t_FrameStruct;

/**************************************************************************//**
 @Description   Node Structure
*//***************************************************************************/
typedef struct {
    uint64_t           ID;              /* */
    uint32_t           value;           /* */
    t_List             node;            /* */
}t_Assoc;

/**************************************************************************//**
 @Description   Internal Frame Functions
*//***************************************************************************/
typedef t_Handle        (*t_pfnConfig)(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);
typedef t_Handle        (*t_pfnClone)(t_Handle hObject);
typedef t_Error         (*t_pfnGetField)(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);
typedef t_Error         (*t_pfnConfigField)(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);
typedef t_Error         (*t_pfnConfigAuto)(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);
typedef t_Error         (*t_pfnConfigHeader)(t_Handle hHeader, t_Handle hValues, uint32_t t_FieldFlags);
typedef t_Error         (*t_pfnRandomizeHeader)(t_Handle hHeader);
typedef t_Error         (*t_pfnSerializeHeader)(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);
typedef t_Error         (*t_pfnInit)(t_Handle hHeader);
typedef uint32_t        (*t_pfnGetHeaderSize)(t_Handle hHeader);
typedef t_Handle        (*t_pfnGetHeader)(t_Handle hFrame);
typedef t_Error         (*t_pfnGetHeaderError)(t_Handle hHeader, t_Handle hError);

typedef struct {
    e_PgHeaderType          HeaderID;
    t_Assoc                 *FieldAssoc;
    t_pfnConfig             pfnConfig;
    t_pfnInit               pfnInit;
    t_pfnClone              pfnClone;
    t_pfnGetField           pfnGetField;
    t_pfnConfigField        pfnConfigField;
    t_pfnConfigAuto         pfnConfigAuto;
    t_pfnConfigHeader       pfnConfigHeader;
    t_pfnRandomizeHeader    pfnRandomizeHeader;
    t_pfnSerializeHeader    pfnSerializeHeader;
    t_pfnGetHeaderSize      pfnGetHeaderSize;
    t_pfnGetHeader          pfnGetHeader;
    t_pfnGetHeaderError     pfnGetHeaderError;
 } t_HeaderInfo;
/**************************************************************************//**
@Group          Frame

@Description    Frame defines, structures and functions

@{
*//***************************************************************************/
t_Error INTERNAL_StartFrameFactory(void);
t_Error INTERNAL_StopFrameFactory(void);
t_Error INTERNAL_RegisterFieldAssoc(e_PgHeaderType currentHeaderID, uint32_t fieldID, e_PgHeaderType nextHeaderID, uint32_t value);
uint32_t INTERNAL_RetrieveFieldAssoc(e_PgHeaderType currentHeaderID, uint32_t fieldID, e_PgHeaderType nextHeaderID);
t_Error INTERNAL_FreeFieldAssocArray(e_PgHeaderType currentHeaderID);
t_Error INTERNAL_ValidateHeaderArray(t_HeaderInfo* pArray);
t_Error INTERNAL_SerializeValue(uint8_t* pMember, uint32_t memberSize, uint8_t** pBuffer, uint32_t *sizeLeft, uint8_t endianness);
t_Error INTERNAL_SerializeNextHeader(t_Handle hFrame, uint8_t **pBuffer, uint32_t* pSize);
t_Error INTERNAL_Frame_Free(t_Handle hFrame, uint8_t recurse);
t_Error INTERNAL_ConfigPayloadType(t_Handle hFrame, e_PgPayloadType type, t_Handle hParams);
uint32_t INTERNAL_GetFrameSize(t_Handle hStartHeader, t_Handle hStopHeader);

t_Handle INTERNAL_GetNextHeader(t_Handle hFrame);
t_Handle INTERNAL_GetPrevHeader(t_Handle hFrame);
t_Handle INTERNAL_GetLastHeader(t_Handle hFrame);
t_Handle INTERNAL_GetFirstHeader(t_Handle hFrame);

void INTERNAL_ResetMultipleHeaderCounters(void);

/**************************************************************************//**
 @Description   CRC/Checksum Functions
*//***************************************************************************/
uint32_t INTERNAL_ComputeCkSum(uint8_t *pBuffer, uint32_t size, uint32_t offset, bool b16bitFold);
t_Error INTERNAL_InitCRC32Table(void);
t_Error INTERNAL_InitCRC8Table(void);
uint32_t INTERNAL_ComputeCRC(uint32_t crcAccum, uint8_t *pBuffer, uint32_t size);
uint16_t INTERNAL_Compute16BitCRC(uint16_t crcAccum, uint8_t *p_Buffer, uint32_t size);
uint32_t INTERNAL_MakeCRC(uint32_t crcAccum, uint8_t *pBuffer, uint32_t size);
uint8_t INTERNAL_ReverseBit(uint8_t in_byte);
uint8_t INTERNAL_ComputeHEC(uint8_t *p_Buffer, uint32_t size);

/**************************************************************************//**
 @Description   List Functions
*//***************************************************************************/
 t_Handle INTERNAL_GetLastNode(t_Handle hNode);
 t_Handle INTERNAL_GetFirstNode(t_Handle hNode);
 t_Handle INTERNAL_GetNextNode(t_Handle hNode);
 t_Handle INTERNAL_GetPrevNode(t_Handle hNode);
 t_Error INTERNAL_InsertAfterNode(t_Handle parentNode, t_Handle childList);
 t_Error INTERNAL_InsertBeforeNode(t_Handle parentNode, t_Handle childList);
 t_Error INTERNAL_AddToTail(t_Handle parentList, t_Handle childList);
 t_Error INTERNAL_RemoveNode(t_Handle pNode);
 t_Error INTERNAL_DebugPrintChain(t_Handle pNode);

 /** @} */ /* end of Frame group */

/**************************************************************************//**
@Group          Header

@Description    Header structures and functions

@{
*//***************************************************************************/
/**************************************************************************//**
@Group          Payload

@Description    Payload structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Payload Structure
*//***************************************************************************/
typedef struct {
    uint8_t             *p_Data;        /* Pointer to payload data */
    uint32_t            length;         /* Payload length */
    uint32_t            maxFrameLen;    /* Maximum frame length */
    uint32_t            minFrameLen;    /* Minimum frame length */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    e_PgPayloadType     payloadType;    /* Payload type */
    uint8_t             startVal;       /* Start value of the payload pattern */
    int8_t              stepVal;        /* Step value of the payload pattern */
    uint8_t             stepCount;      /* Step count of the payload pattern */
    uint32_t            polynomial;     /* PRBS polynomial */
    uint8_t             numBits;        /* Number of bits in the PRBS */
    uint32_t            initVector;     /* Initial value of the PRBS */
    uint8_t             patternSize;    /* Fixed pattern size */
    uint8_t             patternValue[MAX_FIXED_PATTERN_PAYLOAD_BYTES];/* Fixed pattern value */
    uint32_t            seqNumber;      /* Sequence number of the frame */
    e_PgHeaderType      nextHeaderType; /* Next header type */
}t_Payload;

/**************************************************************************//**
 @Function      HDR_Payload_Config

 @Description   Configures the payload

 @Param[in]     hHeader -  Handle to the input payload

 @Return        Payload handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Payload_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_Payload_Clone

 @Description   Takes a copy of the payload

 @Param[in]     hObject -  Handle to the payload

 @Return        Cloned payload handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Payload_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_Payload_GetField

 @Description   Extracts the specified field of the payload

 @param[in]     hHeader - handle to the payload
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_Payload_ConfigField

 @Description   Configures the specified field of the payload

 @param[out]    hHeader - handle to the payload
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_Payload_ConfigAuto

 @Description   Sets or clears the specified field of the payload in the auto flag

 @param[out]    hHeader - handle to the payload
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_Payload_ConfigHeader

 @Description   Configures the payload

 @param[out]    hHeader - handle to the configured payload
 @param[in]     hValues - handle to the payload
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_Payload_ConfigHeader

 @Description   Generates random values for the payload fields

 @param[out]    hHeader - handle to the payload
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Payload_Init

 @Description   Initializes the internal fields into the payload structure

 @param[in]     hHeader - handle to the payload

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Payload_Serialize

 @Description   Packs the payload fields into the data buffer

 @param[in]     hHeader - handle to the payload
 @param[out]    pBuffer - pointer to the buffer which holds the serialized payload
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_Payload_GetHeaderSize

 @Description   Returns the size of the payload

 @param[in]     hHeader - handle to the payload

 @Return        Payload size
*//***************************************************************************/
uint32_t    HDR_Payload_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Payload_GetHeader

 @Description   Returns the payload structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Payload_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_Payload_GetHeaderError

 @Description   Extracts the error in the payload

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Payload_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of Payload group */

/**************************************************************************//**
@Group          Padding

@Description    Padding functions

@{
*//***************************************************************************/


/**************************************************************************//**
 @Function      HDR_Padding_Config

 @Description   Configures the padding

 @Param[in]     hHeader -  Handle to the input padding

 @Return        Padding handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Padding_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_Padding_Clone

 @Description   Takes a copy of the padding

 @Param[in]     hObject -  Handle to the padding

 @Return        Cloned padding handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Padding_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_Padding_GetField

 @Description   Extracts the specified field of the Padding

 @param[in]     hHeader - handle to the Padding
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_Padding_ConfigField

 @Description   Configures the specified field of the Padding

 @param[out]    hHeader - handle to the Padding
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the Padding field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_Padding_ConfigAuto

 @Description   Sets or clears the specified field of the Padding in the auto flag

 @param[out]    hHeader - handle to the Padding
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_Padding_ConfigHeader

 @Description   Configures the Padding

 @param[out]    hHeader - handle to the configured payload
 @param[in]     hValues - handle to the Padding
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_Padding_ConfigHeader

 @Description   Generates random values for the Padding fields

 @param[out]    hHeader - handle to the Padding
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Padding_Init

 @Description   Initializes the internal fields into the Padding structure

 @param[in]     hHeader - handle to the Padding

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Padding_Serialize

 @Description   Packs the Padding fields into the data buffer

 @param[in]     hHeader - handle to the Padding
 @param[out]    pBuffer - pointer to the buffer which holds the serialized Padding
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_Padding_GetHeaderSize

 @Description   Returns the size of the Padding

 @param[in]     hHeader - handle to the Padding

 @Return        Padding size
*//***************************************************************************/
uint32_t    HDR_Padding_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Padding_GetHeader

 @Description   Returns the Padding structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Padding_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_Padding_GetHeaderError

 @Description   Extracts the error in the Padding

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Padding_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of Padding group */

/**************************************************************************//**
@Group          Ethernet

@Description    Ethernet structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Ethernet Structure
*//***************************************************************************/

typedef struct {
    t_PgEthernetHeader  data;           /* Ethernet header */
    bool                bIsType;        /* Type/Length flag */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_Ethernet;

/**************************************************************************//**
 @Function      HDR_Eth_Config

 @Description   Configures the Ethernet header

 @Param[in]     hHeader -  Handle to the input Ethernet header

 @Return        Ethernet header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Eth_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_Eth_Clone

 @Description   Takes a copy of the Ethernet header

 @Param[in]     hObject -  Handle to the Ethernet header

 @Return        Cloned Ethernet header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_Eth_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_Eth_GetField

 @Description   Extracts the specified field of the Ethernet header

 @param[in]     hHeader - handle to the Ethernet header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_Eth_ConfigField

 @Description   Configures the specified field of the Ethernet header

 @param[out]    hHeader - handle to the Ethernet header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_Eth_ConfigAuto

 @Description   Sets or clears the specified field of the Ethernet header in the auto flag

 @param[out]    hHeader - handle to the Ethernet header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_Eth_ConfigHeader

 @Description   Configures the Ethernet header

 @param[out]    hHeader - handle to the configured Ethernet header
 @param[in]     hValues - handle to the Ethernet header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_Eth_Randomize

 @Description   Generates random values for the Ethernet header fields

 @param[out]    hHeader - handle to the Ethernet header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Eth_Init

 @Description   Initializes the internal fields into the Ethernet header structure

 @param[in]     hHeader - handle to the Ethernet header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Eth_Serialize

 @Description   Packs the Ethernet header fields into the data buffer

 @param[in]     hHeader - handle to the Ethernet header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized Ethernet header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_Eth_GetHeaderSize

 @Description   Returns the size of the Ethernet header

 @param[in]     hHeader - handle to the Ethernet header

 @Return        Ethernet header size
*//***************************************************************************/
uint32_t HDR_Eth_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_Eth_GetHeader

 @Description   Returns the Ethernet header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_Eth_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_Eth_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_Eth_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of Ethernet group */

/**************************************************************************//**
@Group          VLAN

@Description    VLAN structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   VLAN Structure
*//***************************************************************************/

typedef struct {
    t_PgVlanHeader      data;           /* VLAN header */
    bool                bIsType;
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_VLAN;

/**************************************************************************//**
 @Function      HDR_VLAN_Config

 @Description   Configures the VLAN header

 @Param[in]     hHeader -  Handle to the input VLAN header

 @Return        VLAN header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_VLAN_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_VLAN_Clone

 @Description   Takes a copy of the VLAN header

 @Param[in]     hObject -  Handle to the VLAN header

 @Return        Cloned VLAN header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_VLAN_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_VLAN_GetField

 @Description   Extracts the specified field of the VLAN header

 @param[in]     hHeader - handle to the VLAN header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_VLAN_ConfigField

 @Description   Configures the specified field of the VLAN header

 @param[out]    hHeader - handle to the VLAN header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_VLAN_ConfigAuto

 @Description   Sets or clears the specified field of the VLAN header in the auto flag

 @param[out]    hHeader - handle to the VLAN header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_VLAN_ConfigHeader

 @Description   Configures the VLAN header

 @param[out]    hHeader - handle to the configured VLAN header
 @param[in]     hValues - handle to the VLAN header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_VLAN_Randomize

 @Description   Generates random values for the VLAN header fields

 @param[out]    hHeader - handle to the VLAN header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_VLAN_Init

 @Description   Generates random values for the VLAN header fields

 @param[out]    hHeader - handle to the VLAN header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_VLAN_Serialize

 @Description   Packs the VLAN header fields into the data buffer

 @param[in]     hHeader - handle to the VLAN header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized VLAN header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_VLAN_GetHeaderSize

 @Description   Returns the size of the VLAN header

 @param[in]     hHeader - handle to the VLAN header

 @Return        VLAN header size
*//***************************************************************************/
uint32_t HDR_VLAN_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_VLAN_GetHeader

 @Description   Returns the VLAN header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_VLAN_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_VLAN_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_VLAN_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of VLAN group */

/**************************************************************************//**
@Group          IPv4

@Description    IPv4 structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   IPv4 Structure
*//***************************************************************************/
typedef struct {
    t_PgIpv4Header      data;           /* IPv4 header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_IPv4;

/**************************************************************************//**
 @Function      HDR_IPv4_Config

 @Description   Configures the IPv4 header

 @Param[in]     hHeader -  Handle to the input IPv4 header

 @Return        IPv4 header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IPv4_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_IPv4_Clone

 @Description   Takes a copy of the IPv4 header

 @Param[in]     hObject -  Handle to the IPv4 header

 @Return        Cloned IPv4 header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_IPv4_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_IPv4_GetField

 @Description   Extracts the specified field of the IPv4 header

 @param[in]     hHeader - handle to the IPv4 header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IPv4_ConfigField

 @Description   Configures the specified field of the IPv4 header

 @param[out]    hHeader - handle to the IPv4 header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IPv4_ConfigAuto

 @Description   Sets or clears the specified field of the IPv4 header in the auto flag

 @param[out]    hHeader - handle to the IPv4 header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_IPv4_ConfigHeader

 @Description   Configures the IPv4 header

 @param[out]    hHeader - handle to the configured IPv4 header
 @param[in]     hValues - handle to the IPv4 header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_IPv4_Randomize

 @Description   Generates random values for the IPv4 header fields

 @param[out]    hHeader - handle to the IPv4 header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPv4_Init

 @Description   Initializes the internal fields into the IPv4 header structure

 @param[in]     hHeader - handle to the IPv4 header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPv4_Serialize

 @Description   Packs the IPv4 header fields into the data buffer

 @param[in]     hHeader - handle to the IPv4 header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized IPv4 header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_IPv4_GetHeaderSize

 @Description   Returns the size of the IPv4 header

 @param[in]     hHeader - handle to the IPv4 header

 @Return        IPv4 header size
*//***************************************************************************/
uint32_t HDR_IPv4_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPv4_GetHeader

 @Description   Returns the IPv4 header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IPv4_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_IPv4_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv4_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of IPv4 group */

/**************************************************************************//**
@Group          LLC

@Description    LLC structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   LLC Structure
*//***************************************************************************/
typedef struct {
    t_PgLlcHeader           data;           /* LLC header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_LLC;

/**************************************************************************//**
 @Function      HDR_LLC_Config

 @Description   Configures the LLC header

 @Param[in]     hHeader -  Handle to the input LLC header

 @Return        LLC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_LLC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_LLC_Clone

 @Description   Takes a copy of the LLC header

 @Param[in]     hObject -  Handle to the LLC header

 @Return        Cloned LLC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_LLC_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_LLC_GetField

 @Description   Extracts the specified field of the LLC header

 @param[in]     hHeader - handle to the LLC header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_LLC_ConfigField

 @Description   Configures the specified field of the LLC header

 @param[out]    hHeader - handle to the LLC header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_LLC_ConfigAuto

 @Description   Sets or clears the specified field of the LLC header in the auto flag

 @param[out]    hHeader - handle to the LLC header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_LLC_ConfigHeader

 @Description   Configures the LLC header

 @param[out]    hHeader - handle to the configured LLC header
 @param[in]     hValues - handle to the LLC header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_LLC_Randomize

 @Description   Generates random values for the LLC header fields

 @param[out]    hHeader - handle to the LLC header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_LLC_Init

 @Description   Initializes the internal fields into the LLC header structure

 @param[in]     hHeader - handle to the LLC header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_LLC_Serialize

 @Description   Packs the LLC header fields into the data buffer

 @param[in]     hHeader - handle to the LLC header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized LLC header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_LLC_GetHeaderSize

 @Description   Returns the size of the LLC header

 @param[in]     hHeader - handle to the LLC header

 @Return        LLC header size
*//***************************************************************************/
uint32_t HDR_LLC_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_LLC_GetHeader

 @Description   Returns the LLC header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_LLC_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_LLC_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_LLC_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of LLC group */

/**************************************************************************//**
@Group          NLP

@Description    NLP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   NLP Structure
*//***************************************************************************/
typedef struct {
    t_PgNlpidHeader     data;           /* NLP header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_NLPID;

/**************************************************************************//**
 @Function      HDR_NLPID_Config

 @Description   Configures the NLP header

 @Param[in]     hHeader -  Handle to the input NLP header

 @Return        NLP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_NLPID_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_NLPID_Clone

 @Description   Takes a copy of the NLP header

 @Param[in]     hObject -  Handle to the NLP header

 @Return        Cloned NLP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_NLPID_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_NLPID_GetField

 @Description   Extracts the specified field of the NLP header

 @param[in]     hHeader - handle to the NLP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_NLPID_ConfigField

 @Description   Configures the specified field of the NLP header

 @param[out]    hHeader - handle to the NLP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_NLPID_ConfigAuto

 @Description   Sets or clears the specified field of the NLP header in the auto flag

 @param[out]    hHeader - handle to the NLP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_NLPID_ConfigHeader

 @Description   Configures the NLP header

 @param[out]    hHeader - handle to the configured NLP header
 @param[in]     hValues - handle to the NLP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_NLPID_Randomize

 @Description   Generates random values for the NLP header fields

 @param[out]    hHeader - handle to the NLP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_NLPID_Init

 @Description   Initializes the internal fields into the NLP header structure

 @param[in]     hHeader - handle to the NLP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_NLPID_Serialize

 @Description   Packs the NLP header fields into the data buffer

 @param[in]     hHeader - handle to the NLP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized NLP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_NLPID_GetHeaderSize

 @Description   Returns the size of the NLP header

 @param[in]     hHeader - handle to the NLP header

 @Return        NLP header size
*//***************************************************************************/
uint32_t HDR_NLPID_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_NLPID_GetHeader

 @Description   Returns the NLP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_NLPID_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_NLPID_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hFrame - handle to the frame

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_NLPID_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of NLPID group */

/**************************************************************************//**
@Group          SNAP

@Description    SNAP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SNAP Structure
*//***************************************************************************/
typedef struct {
    t_PgSnapHeader      data;           /* SNAP header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_SNAP;

/**************************************************************************//**
 @Function      HDR_SNAP_Config

 @Description   Configures the SNAP header

 @Param[in]     hHeader -  Handle to the input SNAP header

 @Return        SNAP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_SNAP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_SNAP_Clone

 @Description   Takes a copy of the SNAP header

 @Param[in]     hObject -  Handle to the SNAP header

 @Return        Cloned SNAP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_SNAP_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_SNAP_GetField

 @Description   Extracts the specified field of the SNAP header

 @param[in]     hHeader - handle to the SNAP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_SNAP_ConfigField

 @Description   Configures the specified field of the SNAP header

 @param[out]    hHeader - handle to the SNAP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_SNAP_ConfigAuto

 @Description   Sets or clears the specified field of the SNAP header in the auto flag

 @param[out]    hHeader - handle to the SNAP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_SNAP_ConfigHeader

 @Description   Configures the SNAP header

 @param[out]    hHeader - handle to the configured SNAP header
 @param[in]     hValues - handle to the SNAP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_SNAP_Randomize

 @Description   Generates random values for the SNAP header fields

 @param[out]    hHeader - handle to the SNAP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SNAP_Init

 @Description   Initializes the internal fields into the SNAP header structure

 @param[in]     hHeader - handle to the SNAP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SNAP_Serialize

 @Description   Packs the SNAP header fields into the data buffer

 @param[in]     hHeader - handle to the SNAP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized SNAP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_SNAP_GetHeaderSize

 @Description   Returns the size of the SNAP header

 @param[in]     hHeader - handle to the SNAP header

 @Return        SNAP header size
*//***************************************************************************/
uint32_t HDR_SNAP_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SNAP_GetHeader

 @Description   Returns the SNAP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_SNAP_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_SNAP_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SNAP_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of SNAP group */

/**************************************************************************//**
@Group          PPPoE

@Description    PPPoE structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PPPoE Structure
*//***************************************************************************/
typedef struct {
    t_PgPppoeHeader         data;           /* PPPoE header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_PPPoE;

/**************************************************************************//**
 @Function      HDR_PPPoE_Config

 @Description   Configures the PPPoE header

 @Param[in]     hHeader -  Handle to the input PPPoE header

 @Return        PPPoE header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PPPoE_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_PPPoE_Clone

 @Description   Takes a copy of the PPPoE header

 @Param[in]     hObject -  Handle to the PPPoE header

 @Return        Cloned PPPoE header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_PPPoE_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_PPPoE_GetField

 @Description   Extracts the specified field of the PPPoE header

 @param[in]     hHeader - handle to the PPPoE header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_PPPoE_ConfigField

 @Description   Configures the specified field of the PPPoE header

 @param[out]    hHeader - handle to the PPPoE header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_PPPoE_ConfigAuto

 @Description   Sets or clears the specified field of the PPPoE header in the auto flag

 @param[out]    hHeader - handle to the PPPoE header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_PPPoE_ConfigHeader

 @Description   Configures the PPPoE header

 @param[out]    hHeader - handle to the configured PPPoE header
 @param[in]     hValues - handle to the PPPoE header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_PPPoE_Randomize

 @Description   Generates random values for the PPPoE header fields

 @param[out]    hHeader - handle to the PPPoE header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPPoE_Init

 @Description   Initializes the internal fields into the PPPoE header structure

 @param[in]     hHeader - handle to the PPPoE header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPPoE_Serialize

 @Description   Packs the PPPoE header fields into the data buffer

 @param[in]     hHeader - handle to the PPPoE header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized PPPoE header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_PPPoE_GetHeaderSize

 @Description   Returns the size of the PPPoE header

 @param[in]     hHeader - handle to the PPPoE header

 @Return        PPPoE header size
*//***************************************************************************/
uint32_t HDR_PPPoE_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPPoE_GetHeader

 @Description   Returns the PPPoE header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PPPoE_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_PPPoE_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPoE_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of PPPoE group */

/**************************************************************************//**
@Group          PPP

@Description    PPP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PPP Structure
*//***************************************************************************/
typedef struct {
    t_PgPppHeader       data;           /* PPP header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_PPP;

/**************************************************************************//**
 @Function      HDR_PPP_Config

 @Description   Configures the PPP header

 @Param[in]     hHeader -  Handle to the input PPP header

 @Return        PPP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PPP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_PPP_Clone

 @Description   Takes a copy of the PPP header

 @Param[in]     hObject -  Handle to the PPP header

 @Return        Cloned PPP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_PPP_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_PPP_GetField

 @Description   Extracts the specified field of the PPP header

 @param[in]     hHeader - handle to the PPP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_PPP_ConfigField

 @Description   Configures the specified field of the PPP header

 @param[out]    hHeader - handle to the PPP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_PPP_ConfigAuto

 @Description   Sets or clears the specified field of the PPP header in the auto flag

 @param[out]    hHeader - handle to the PPP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_PPP_ConfigHeader

 @Description   Configures the PPP header

 @param[out]    hHeader - handle to the configured PPP header
 @param[in]     hValues - handle to the PPP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_PPP_Randomize

 @Description   Generates random values for the PPP header fields

 @param[out]    hHeader - handle to the PPP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPP_Init

 @Description   Initializes the internal fields into the PPP header structure

 @param[in]     hHeader - handle to the PPP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPP_Serialize

 @Description   Packs the PPP header fields into the data buffer

 @param[in]     hHeader - handle to the PPP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized PPP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_PPP_GetHeaderSize

 @Description   Returns the size of the PPP header

 @param[in]     hHeader - handle to the PPP header

 @Return        PPP header size
*//***************************************************************************/
uint32_t HDR_PPP_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPP_GetHeader

 @Description   Returns the PPP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PPP_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_PPP_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPP_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of PPP group */

/**************************************************************************//**
@Group          TCP

@Description    TCP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   TCP Structure
*//***************************************************************************/
typedef struct {
    t_PgTcpHeader       data;           /* TCP header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_TCP;

/**************************************************************************//**
 @Function      HDR_TCP_Config

 @Description   Configures the TCP header

 @Param[in]     hHeader -  Handle to the input TCP header

 @Return        TCP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_TCP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_TCP_Clone

 @Description   Takes a copy of the TCP header

 @Param[in]     hObject -  Handle to the TCP header

 @Return        Cloned TCP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_TCP_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_TCP_GetField

 @Description   Extracts the specified field of the TCP header

 @param[in]     hHeader - handle to the TCP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_TCP_ConfigField

 @Description   Configures the specified field of the TCP header

 @param[out]    hHeader - handle to the TCP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_TCP_ConfigAuto

 @Description   Sets or clears the specified field of the TCP header in the auto flag

 @param[out]    hHeader - handle to the TCP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_TCP_ConfigHeader

 @Description   Configures the TCP header

 @param[out]    hHeader - handle to the configured TCP header
 @param[in]     hValues - handle to the TCP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_TCP_Randomize

 @Description   Generates random values for the TCP header fields

 @param[out]    hHeader - handle to the TCP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_TCP_Init

 @Description   Initializes the internal fields into the TCP header structure

 @param[in]     hHeader - handle to the TCP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_TCP_Serialize

 @Description   Packs the TCP header fields into the data buffer

 @param[in]     hHeader - handle to the TCP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized TCP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_TCP_GetHeaderSize

 @Description   Returns the size of the TCP header

 @param[in]     hHeader - handle to the TCP header

 @Return        TCP header size
*//***************************************************************************/
uint32_t HDR_TCP_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_TCP_GetHeader

 @Description   Returns the TCP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_TCP_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_TCP_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_TCP_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of TCP group */

/**************************************************************************//**
@Group          UDP

@Description    UDP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UDP Structure
*//***************************************************************************/
typedef struct {
    t_PgUdpHeader       data;           /* UDP header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_UDP;

/**************************************************************************//**
 @Function      HDR_UDP_Config

 @Description   Configures the UDP header

 @Param[in]     hHeader -  Handle to the input UDP header

 @Return        UDP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_UDP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_UDP_Clone

 @Description   Takes a copy of the UDP header

 @Param[in]     hObject -  Handle to the UDP header

 @Return        Cloned UDP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_UDP_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_UDP_GetField

 @Description   Extracts the specified field of the UDP header

 @param[in]     hHeader - handle to the UDP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_UDP_ConfigField

 @Description   Configures the specified field of the UDP header

 @param[out]    hHeader - handle to the UDP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_UDP_ConfigAuto

 @Description   Sets or clears the specified field of the UDP header in the auto flag

 @param[out]    hHeader - handle to the UDP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_UDP_ConfigHeader

 @Description   Configures the UDP header

 @param[out]    hHeader - handle to the configured UDP header
 @param[in]     hValues - handle to the UDP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_UDP_Randomize

 @Description   Generates random values for the UDP header fields

 @param[out]    hHeader - handle to the UDP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_UDP_Init

 @Description   Initializes the internal fields into the UDP header structure

 @param[in]     hHeader - handle to the UDP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_UDP_Serialize

 @Description   Packs the UDP header fields into the data buffer

 @param[in]     hHeader - handle to the UDP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized UDP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_UDP_GetHeaderSize

 @Description   Returns the size of the UDP header

 @param[in]     hHeader - handle to the UDP header

 @Return        UDP header size
*//***************************************************************************/
uint32_t HDR_UDP_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_UDP_GetHeader

 @Description   Returns the UDP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_UDP_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_UDP_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_UDP_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of UDP group */

/**************************************************************************//**
@Group          SCTP

@Description    SCTP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SCTP Structure
*//***************************************************************************/
typedef struct {
    t_PgSctpCommonHeader    data;           /* SCTP header */
    uint32_t                autoFlags;      /* Holds the fields configured internally */
    uint8_t                 errorFlag;      /* Holds the errors in the header */
} t_SCTP;

/**************************************************************************//**
 @Function      HDR_SCTP_Config

 @Description   Configures the SCTP header

 @Param[in]     hHeader -  Handle to the input SCTP header

 @Return        SCTP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_SCTP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_SCTP_Clone

 @Description   Takes a copy of the SCTP header

 @Param[in]     hObject -  Handle to the SCTP header

 @Return        Cloned SCTP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_SCTP_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_SCTP_GetField

 @Description   Extracts the specified field of the SCTP header

 @param[in]     hHeader - handle to the SCTP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_SCTP_ConfigField

 @Description   Configures the specified field of the SCTP header

 @param[out]    hHeader - handle to the SCTP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_SCTP_ConfigAuto

 @Description   Sets or clears the specified field of the SCTP header in the auto flag

 @param[out]    hHeader - handle to the SCTP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_SCTP_ConfigHeader

 @Description   Configures the SCTP header

 @param[out]    hHeader - handle to the configured SCTP header
 @param[in]     hValues - handle to the SCTP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_SCTP_Randomize

 @Description   Generates random values for the SCTP header fields

 @param[out]    hHeader - handle to the SCTP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SCTP_Init

 @Description   Initializes the internal fields into the SCTP header structure

 @param[in]     hHeader - handle to the SCTP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SCTP_Serialize

 @Description   Packs the SCTP header fields into the data buffer

 @param[in]     hHeader - handle to the SCTP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized SCTP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_SCTP_GetHeaderSize

 @Description   Returns the size of the SCTP header

 @param[in]     hHeader - handle to the SCTP header

 @Return        SCTP header size
*//***************************************************************************/
uint32_t HDR_SCTP_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SCTP_GetHeader

 @Description   Returns the SCTP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_SCTP_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_SCTP_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_SCTP_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of SCTP group */

/**************************************************************************//**
@Group          SCTP Chunk

@Description    SCTP Chunk structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SCTP Chunk Structure
*//***************************************************************************/
typedef struct {
    t_PgSctpChunkDataHeader data;          /* SCTP Chunk Data header */
    uint32_t                autoFlags;     /* Holds the fields configured internally */
} t_SCTPChunkData;

 /**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_Config

 @Description   Configures the SCTP Chunk header

 @Param[in]     hHeader -  Handle to the input SCTP Chunk header

 @Return        SCTP Chunk header handle on success, NULL on failure
*//***************************************************************************/
 t_Handle HDR_SCTP_CHUNK_DATA_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

  /**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_Clone

 @Description   Takes a copy of the SCTP Chunk header

 @Param[in]     hObject -  Handle to the SCTP Chunk header

 @Return        Cloned SCTP Chunk header handle on success, NULL on failure
*//***************************************************************************/
 t_Handle HDR_SCTP_CHUNK_DATA_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_GetField

 @Description   Extracts the specified field of the SCTP Chunk header

 @param[in]     hHeader - handle to the SCTP Chunk header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue  - handle to the extracted field
 @param[in]     size    - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t Size);

 /**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_ConfigField

 @Description   Configures the specified field of the SCTP Chunk header

 @param[out]    hHeader - handle to the SCTP  Chunk header
 @param[in]     fieldID - field to be set
 @param[in]     hValue  - handle to the payload field
 @param[in]     size    - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_ConfigAuto

 @Description   Sets or clears the specified field of the SCTP Chunk header in the auto flag

 @param[out]    hHeader  - handle to the SCTP Chunk header
 @param[in]     fieldID  - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_ConfigHeader

 @Description   Configures the SCTP Chunk header

 @param[out]    hHeader  - handle to the configured SCTP Chunk header
 @param[in]     hValues  - handle to the SCTP Chunk header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_Randomize

 @Description   Generates random values for the SCTP Chunk header fields

 @param[out]    hHeader - handle to the SCTP Chunk header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_Init

 @Description   Initializes the internal fields into the SCTP Chunk header structure

 @param[in]     hHeader - handle to the SCTP Chunk header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_Serialize

 @Description   Packs the SCTP Chunk header fields into the data buffer

 @param[in]     hHeader - handle to the SCTP Chunk header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized SCTP Chunk header
 @param[out]    pSize   - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

 /**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_GetHeaderSize

 @Description   Returns the size of the SCTP Chunk header

 @param[in]     hHeader - handle to the SCTP Chunk header

 @Return        SCTP Chunk header size
*//***************************************************************************/
 uint32_t HDR_SCTP_CHUNK_DATA_GetHeaderSize(t_Handle hHeader);

 /**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_GetHeader

 @Description   Returns the SCTP Chunk header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
 t_Handle HDR_SCTP_CHUNK_DATA_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_SCTP_CHUNK_DATA_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError  - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
 t_Error HDR_SCTP_CHUNK_DATA_GetHeaderError(t_Handle hHeader, t_Handle hError);


/** @} */ /* end of SCTP Chunk group */

/**************************************************************************//**
@Group          IPHC

@Description    IPHC structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   IPHC Structure
*//***************************************************************************/
typedef struct {
    t_PgIphcManip       data;           /* IPHC manipulation */
    uint32_t            hcHdrLength;    /* Compressed header length */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    t_Handle            h_TrafficIphc;  /* Handle to the Iphc object */
} t_IPHC;

/**************************************************************************//**
 @Function      HDR_IPHC_Config

 @Description   Configures the IPHC header

 @Param[in]     hHeader -  Handle to the input IPHC header

 @Return        IPHC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IPHC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_IPHC_Clone

 @Description   Takes a copy of the IPHC header

 @Param[in]     hObject -  Handle to the IPHC header

 @Return        Cloned IPHC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_IPHC_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_IPHC_GetField

 @Description   Extracts the specified field of the IPHC header

 @param[in]     hHeader - handle to the IPHC header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IPHC_ConfigField

 @Description   Configures the specified field of the IPHC header

 @param[out]    hHeader - handle to the IPHC header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IPHC_ConfigAuto

 @Description   Sets or clears the specified field of the IPHC header in the auto flag

 @param[out]    hHeader - handle to the IPHC header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_IPHC_ConfigHeader

 @Description   Configures the IPHC header

 @param[out]    hHeader - handle to the configured IPHC header
 @param[in]     hValues - handle to the IPHC header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_IPHC_Randomize

 @Description   Generates random values for the IPHC header fields

 @param[out]    hHeader - handle to the IPHC header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPHC_Init

 @Description   Initializes the internal fields into the IPHC header structure

 @param[in]     hHeader - handle to the IPHC header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPHC_Serialize

 @Description   Packs the IPHC header fields into the data buffer

 @param[in]     hHeader - handle to the IPHC header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized IPHC header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_IPHC_GetHeaderSize

 @Description   Returns the size of the IPHC header

 @param[in]     hHeader - handle to the IPHC header

 @Return        IPHC header size
*//***************************************************************************/
uint32_t HDR_IPHC_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPHC_GetHeader

 @Description   Returns the IPHC header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IPHC_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_IPHC_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPHC_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of IPHC group */

/**************************************************************************//**
@Group          IEEE 1588

@Description    IEEE 1588 structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   IEEE 1588 Message Header Structure
*//***************************************************************************/
typedef struct {
    u_PgIeee1588MessageHeader   data;           /* IEEE 1588 Message header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_IEEE1588Message;

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_Config

 @Description   Configures the IEEE 1588 Message header

 @Param[in]     hHeader -  Handle to the input IEEE 1588 Message header

 @Return        IEEE 1588 Message header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IEEE1588Message_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_Clone

 @Description   Takes a copy of the IEEE 1588 Message header

 @Param[in]     hObject -  Handle to the IEEE 1588 Message header

 @Return        Cloned IEEE 1588 Message header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_IEEE1588Message_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_GetField

 @Description   Extracts the specified field of the IEEE 1588 Message header

 @param[in]     hHeader - handle to the IEEE 1588 header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_ConfigField

 @Description   Configures the specified field of the IEEE 1588 Message header

 @param[out]    hHeader - handle to the IEEE 1588 Message header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_ConfigAuto

 @Description   Sets or clears the specified field of the IEEE 1588 Message header in the auto flag

 @param[out]    hHeader - handle to the IEEE 1588 Message header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_IEEE1588_V1_Message_ConfigHeader

 @Description   Configures the IEEE 1588 Message header

 @param[out]    hHeader - handle to the configured IEEE 1588 Message header
 @param[in]     hValues - handle to the IEEE 1588 Message header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_Randomize

 @Description   Generates random values for the IEEE 1588 Message header fields

 @param[out]    hHeader - handle to the IEEE 1588 Message header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_Init

 @Description   Initializes the internal fields into the IEEE 1588 header structure

 @param[in]     hHeader - handle to the IEEE 1588 header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_Serialize

 @Description   Packs the IEEE 1588 header fields into the data buffer

 @param[in]     hHeader - handle to the IEEE 1588 header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized IEEE 1588 header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_GetHeaderSize

 @Description   Returns the size of the IEEE 1588 header

 @param[in]     hHeader - handle to the IEEE 1588 header

 @Return        IEEE 1588 header size
*//***************************************************************************/
uint32_t HDR_IEEE1588Message_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_GetHeader

 @Description   Returns the IEEE 1588 Message header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IEEE1588Message_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_IEEE1588Message_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IEEE1588Message_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of IEEE 1588 Message group */

/**************************************************************************//**
@Group          RTP

@Description    RTP structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   RTP Structure
*//***************************************************************************/
typedef struct {
    t_PgRtpHeader           data;           /* RTP header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_RTP;

/**************************************************************************//**
 @Function      HDR_RTP_Config

 @Description   Configures the RTP header

 @Param[in]     hHeader -  Handle to the input RTP header

 @Return        RTP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_RTP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_RTP_Clone

 @Description   Takes a copy of the RTP header

 @Param[in]     hObject -  Handle to the RTP header

 @Return        Cloned RTP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_RTP_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_RTP_GetField

 @Description   Extracts the specified field of the RTP header

 @param[in]     hHeader - handle to the RTP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_RTP_ConfigField

 @Description   Configures the specified field of the RTP header

 @param[out]    hHeader - handle to the RTP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_RTP_ConfigAuto

 @Description   Sets or clears the specified field of the RTP header in the auto flag

 @param[out]    hHeader - handle to the RTP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_RTP_ConfigHeader

 @Description   Configures the RTP header

 @param[out]    hHeader - handle to the configured RTP header
 @param[in]     hValues - handle to the RTP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_RTP_Randomize

 @Description   Generates random values for the RTP header fields

 @param[out]    hHeader - handle to the RTP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_RTP_Init

 @Description   Initializes the internal fields into the RTP header structure

 @param[in]     hHeader - handle to the RTP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_RTP_Serialize

 @Description   Packs the RTP header fields into the data buffer

 @param[in]     hHeader - handle to the RTP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized RTP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_RTP_GetHeaderSize

 @Description   Returns the size of the RTP header

 @param[in]     hHeader - handle to the RTP header

 @Return        RTP header size
*//***************************************************************************/
uint32_t HDR_RTP_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_RTP_GetHeader

 @Description   Returns the RTP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_RTP_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_RTP_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_RTP_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of RTP group */

/**************************************************************************//**
@Group          IPv6

@Description    IPv6 structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   IPv6 Structure
*//***************************************************************************/
typedef struct {
    t_PgIpv6Header      data;           /* ML/MC header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_IPv6;

/**************************************************************************//**
 @Function      HDR_IPv6_Config

 @Description   Configures the IPv6 header

 @Param[in]     hHeader -  Handle to the input IPv6 header

 @Return        IPv6 header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IPv6_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_IPv6_Clone

 @Description   Takes a copy of the IPv6 header

 @Param[in]     hObject -  Handle to the IPv6 header

 @Return        Cloned IPv6 header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_IPv6_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_IPv6_GetField

 @Description   Extracts the specified field of the IPv6 header

 @param[in]     hHeader - handle to the IPv6 header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IPv6_ConfigField

 @Description   Configures the specified field of the IPv6 header

 @param[out]    hHeader - handle to the IPv6 header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_IPv6_ConfigAuto

 @Description   Sets or clears the specified field of the IPv6 header in the auto flag

 @param[out]    hHeader - handle to the IPv6 header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_IPv6_ConfigHeader

 @Description   Configures the IPv6 header

 @param[out]    hHeader - handle to the configured IPv6 header
 @param[in]     hValues - handle to the IPv6 header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_IPv6_Randomize

 @Description   Generates random values for the IPv6 header fields

 @param[out]    hHeader - handle to the IPv6 header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPv6_Init

 @Description   Initializes the internal fields into the IPv6 header structure

 @param[in]     hHeader - handle to the IPv6 header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPv6_Serialize

 @Description   Packs the IPv6 header fields into the data buffer

 @param[in]     hHeader - handle to the IPv6 header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized IPv6 header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_IPv6_GetHeaderSize

 @Description   Returns the size of the IPv6 header

 @param[in]     hHeader - handle to the IPv6 header

 @Return        IPv6 header size
*//***************************************************************************/
uint32_t HDR_IPv6_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_IPv6_GetHeader

 @Description   Returns the IPv6 header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_IPv6_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_IPv6_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_IPv6_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of IPv6 group */

/**************************************************************************//**
@Group          AAL0

@Description    AAL0 structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   AAL0 Cell Structure
*//***************************************************************************/
typedef struct {
    t_PgAal0CellHeader  data;           /* AAL0 cell header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_AAL0Cell;

/**************************************************************************//**
 @Function      HDR_AAL0Cell_Config

 @Description   Configures the AAL0 Cell header

 @Param[in]     hHeader -  Handle to the input AAL0 cell header

 @Return        AAL0 Cell header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_AAL0Cell_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_Clone

 @Description   Takes a copy of the AAL0 Cell header

 @Param[in]     hObject -  Handle to the AAL0 Cell header

 @Return        Cloned AAL0 Cell header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_AAL0Cell_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_GetField

 @Description   Extracts the specified field of the AAL0 Cell header

 @param[in]     hHeader - handle to the AAL0 Cell header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_ConfigField

 @Description   Configures the specified field of the AAL0 Cell header

 @param[out]    hHeader - handle to the AAL0 Cell header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_ConfigAuto

 @Description   Sets or clears the specified field of the AAL0 Cell header in the auto flag

 @param[out]    hHeader - handle to the AAL0 Cell header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_ConfigHeader

 @Description   Configures the AAL0 Cell header

 @param[out]    hHeader - handle to the configured AAL0 Cell header
 @param[in]     hValues - handle to the AAL0 Cell header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_Randomize

 @Description   Generates random values for the AAL0 Cell header fields

 @param[out]    hHeader - handle to the AAL0 Cell header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_Init

 @Description   Initializes the internal fields into the AAL0 Cell header structure

 @param[in]     hHeader - handle to the AAL0 Cell header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_Serialize

 @Description   Packs the AAL0 Cell header fields into the data buffer

 @param[in]     hHeader - handle to the AAL0 Cell header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized AAL0 Cell header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_GetHeaderSize

 @Description   Returns the size of the AAL0 Cell header

 @param[in]     hHeader - handle to the AAL0 Cell header

 @Return        AAL0 Cell header size
*//***************************************************************************/
uint32_t HDR_AAL0Cell_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_GetHeader

 @Description   Returns the AAL0 cell header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_AAL0Cell_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_AAL0Cell_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_AAL0Cell_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of AAL0 group */

/**************************************************************************//**
@Group          HDLC

@Description    HDLC structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   HDLC Structure
*//***************************************************************************/
typedef struct {
    t_PgHdlcHeader      data;           /* HDLC header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
    uint8_t             errorFlag;      /* Holds the errors in the header */
} t_HDLC;

/**************************************************************************//**
 @Function      HDR_HDLC_Config

 @Description   Configures the HDLC header

 @Param[in]     hHeader -  Handle to the input HDLC header

 @Return        HDLC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_HDLC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_HDLC_Clone

 @Description   Takes a copy of the HDLC header

 @Param[in]     hObject -  Handle to the HDLC header

 @Return        Cloned HDLC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_HDLC_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_HDLC_GetField

 @Description   Extracts the specified field of the HDLC header

 @param[in]     hHeader - handle to the HDLC header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_HDLC_ConfigField

 @Description   Configures the specified field of the HDLC header

 @param[out]    hHeader - handle to the HDLC header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_HDLC_ConfigAuto

 @Description   Sets or clears the specified field of the HDLC header in the auto flag

 @param[out]    hHeader - handle to the HDLC header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_HDLC_ConfigHeader

 @Description   Configures the HDLC header

 @param[out]    hHeader - handle to the configured HDLC header
 @param[in]     hValues - handle to the HDLC header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_HDLC_Randomize

 @Description   Generates random values for the HDLC header fields

 @param[out]    hHeader - handle to the HDLC header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_HDLC_Init

 @Description   Initializes the internal fields into the HDLC header structure

 @param[in]     hHeader - handle to the HDLC header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_HDLC_Serialize

 @Description   Packs the HDLC header fields into the data buffer

 @param[in]     hHeader - handle to the HDLC header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized HDLC header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_HDLC_GetHeaderSize

 @Description   Returns the size of the HDLC header

 @param[in]     hHeader - handle to the HDLC header

 @Return        HDLC header size
*//***************************************************************************/
uint32_t HDR_HDLC_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_HDLC_GetHeader

 @Description   Returns the HDLC header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_HDLC_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_HDLC_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_HDLC_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of HDLC group */

/**************************************************************************//**
@Group          ML/MC

@Description    ML/MC structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   ML/MC Structure
*//***************************************************************************/
typedef struct {
    t_PgPppMlmcHeader       data;           /* ML/MC header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_PPPMLMC;

/**************************************************************************//**
 @Function      HDR_PPPMLMC_Config

 @Description   Configures the PPP ML/MC header

 @Param[in]     hHeader -  Handle to the input PPP ML/MC header

 @Return        PPP ML/MC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PPPMLMC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_Clone

 @Description   Takes a copy of the PPP ML/MC header

 @Param[in]     hObject -  Handle to the PPP ML/MC header

 @Return        Cloned PPP ML/MC header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_PPPMLMC_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_GetField

 @Description   Extracts the specified field of the PPP ML/MC header

 @param[in]     hHeader - handle to the PPP ML/MC header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_ConfigField

 @Description   Configures the specified field of the PPP ML/MC header

 @param[out]    hHeader - handle to the PPP ML/MC header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_ConfigAuto

 @Description   Sets or clears the specified field of the PPP ML/MC header in the auto flag

 @param[out]    hHeader - handle to the PPP ML/MC header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_ConfigHeader

 @Description   Configures the PPP ML/MC header

 @param[out]    hHeader - handle to the configured PPP ML/MC header
 @param[in]     hValues - handle to the PPP ML/MC header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_Randomize

 @Description   Generates random values for the PPP ML/MC header fields

 @param[out]    hHeader - handle to the PPP ML/MC header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_Init

 @Description   Initializes the internal fields into the PPP ML/MC header structure

 @param[in]     hHeader - handle to the PPP ML/MC header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_Serialize

 @Description   Packs the PPP ML/MC header fields into the data buffer

 @param[in]     hHeader - handle to the PPP ML/MC header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized PPP ML/MC header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_GetHeaderSize

 @Description   Returns the size of the PPP ML/MC header

 @param[in]     hHeader - handle to the PPP ML/MC header

 @Return        PPP ML/MC header size
*//***************************************************************************/
uint32_t HDR_PPPMLMC_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_GetHeader

 @Description   Returns the PPP ML/MC header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PPPMLMC_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_PPPMLMC_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PPPMLMC_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of ML/MC group */
/** @} */ /* end of Header group */
/** @} */ /* end of InternalFrame group */



/**************************************************************************//**
@Group          MPLS

@Description    MPLS structures and functions

@{
*//***************************************************************************/


/**************************************************************************//**
 @Description   MPLS Structure
*//***************************************************************************/
typedef struct
{
    t_PgMplsHeader      data;
    uint32_t            autoFlags;
}t_MPLS;

/**************************************************************************//**
 @Function      HDR_MPLS_Config

 @Description   Configures the MPLS header

 @Param[in]     hHeader -  Handle to the input MPLS header

 @Return        MPLS header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_MPLS_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_MPLS_Clone

 @Description   Takes a copy of the MPLS header

 @Param[in]     hObject -  Handle to the MPLS header

 @Return        Cloned MPLS header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_MPLS_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_MPLS_GetField

 @Description   Extracts the specified field of the MPLS header

 @param[in]     hHeader - handle to MPLS header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);



/**************************************************************************//**
 @Function      HDR_MPLS_ConfigField

 @Description   Configures the specified field of the MPLS header

 @param[out]    hHeader - handle to the MPLS header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_MPLS_ConfigAuto

 @Description   Sets or clears the specified field of the MPLS header in the auto flag

 @param[out]    hHeader - handle to the MPLS header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/

t_Error HDR_MPLS_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);


/**************************************************************************//**
 @Function      HDR_MPLS_ConfigHeader

 @Description   Configures the MPLS header

 @param[out]    hHeader - handle to the configured MPLS header
 @param[in]     hValues - handle to the MPLS header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);


/**************************************************************************//**
 @Function      HDR_MPLS_Randomize

 @Description   Generates random values for the MPLS header fields

 @param[out]    hHeader - handle to the MPLS header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_Randomize(t_Handle hHeader);



/**************************************************************************//**
 @Function      HDR_MPLS_Init

 @Description   Initializes the internal fields into the MPLS header structure

 @param[in]     hHeader - handle to the MPLS header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_Init(t_Handle hHeader);



/**************************************************************************//**
 @Function      HDR_MPLS_Serialize

 @Description   Packs the MPLS header fields into the data buffer

 @param[in]     hHeader - handle to the MPLS header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized MPLS header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_Serialize(t_Handle hHeader, uint8_t **p_Buffer, uint32_t *pSize);


/**************************************************************************//**
 @Function      HDR_MPLS_GetHeaderSize

 @Description   Returns the size of the MPLS header

 @param[in]     hHeader - handle to the MPLS header

 @Return        MPLS header size
*//***************************************************************************/
uint32_t HDR_MPLS_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_MPLS_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_MPLS_GetHeaderError(t_Handle hHeader, t_Handle hError);




 /**************************************************************************//**
 @Function      HDR_MPLS_GetHeader

 @Description   Returns the MPLS header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_MPLS_GetHeader(t_Handle hFrame);


/** @} */ /* end of MPLS group */

/**************************************************************************//**
@Group          L2TPv2

@Description    L2TP v2 structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   L2TP Structure
*//***************************************************************************/
typedef struct {
    t_PgL2tpv2Header        data;           /* L2TP v2 header */
    uint32_t                autoFlags;      /* Holds the fields configured internally */
} t_L2TPv2;

/**************************************************************************//**
 @Function      HDR_L2TPv2_Config

 @Description   Configures the L2TP header

 @Param[in]     hHeader -  Handle to the input L2TP header

 @Return        L2TP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_L2TPv2_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_L2TPv2_Clone

 @Description   Takes a copy of the L2TP header

 @Param[in]     hObject -  Handle to the L2TP header

 @Return        Cloned L2TP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_L2TPv2_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_L2TPv2_GetField

 @Description   Extracts the specified field of the L2TP header

 @param[in]     hHeader - handle to the L2TP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_L2TPv2_ConfigField

 @Description   Configures the specified field of the L2TP header

 @param[out]    hHeader - handle to the L2TP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the L2TP field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_L2TPv2_ConfigAuto

 @Description   Sets or clears the specified field of the L2TP header in the auto flag

 @param[out]    hHeader - handle to the L2TP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_L2TPv2_ConfigHeader

 @Description   Configures the L2TP header

 @param[out]    hHeader - handle to the configured L2TP header
 @param[in]     hValues - handle to the L2TP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_L2TPv2_Randomize

 @Description   Generates random values for the L2TP header fields

 @param[out]    hHeader - handle to the L2TP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_L2TPv2_Init

 @Description   Initializes the internal fields into the L2TP header structure

 @param[in]     hHeader - handle to the L2TP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_L2TPv2_Serialize

 @Description   Packs the L2TP header fields into the data buffer

 @param[in]     hHeader - handle to the L2TP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized L2TP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_L2TPv2_GetHeaderSize

 @Description   Returns the size of the L2TP header

 @param[in]     hHeader - handle to the L2TP header

 @Return        L2TP header size
*//***************************************************************************/
uint32_t HDR_L2TPv2_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_L2TPv2_GetHeader

 @Description   Returns the L2TP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_L2TPv2_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_L2TPv2_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv2_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of L2TPv2 group */


/**************************************************************************//**
@Group          L2TPv3

@Description    L2TPv3 structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   L2TPv3 Structure
*//***************************************************************************/
typedef struct {
    u_PgL2tpv3Header        data;           /* L2TP v3 header */
    uint32_t            autoFlags;      /* Holds the fields configured internally */
} t_L2TPv3;

/**************************************************************************//**
 @Function      HDR_L2TPv3_Config

 @Description   Configures the L2TP header

 @Param[in]     hHeader -  Handle to the input L2TP header

 @Return        L2TP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_L2TPv3_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      HDR_L2TPv3_Clone

 @Description   Takes a copy of the L2TP header

 @Param[in]     hObject -  Handle to the L2TP header

 @Return        Cloned L2TP header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_L2TPv3_Clone(t_Handle hObject);

/**************************************************************************//**
 @Function      HDR_L2TPv3_GetField

 @Description   Extracts the specified field of the L2TP header

 @param[in]     hHeader - handle to the L2TP header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_L2TPv3_ConfigField

 @Description   Configures the specified field of the L2TP header

 @param[out]    hHeader - handle to the L2TP header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the L2TP field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);

/**************************************************************************//**
 @Function      HDR_L2TPv3_ConfigAuto

 @Description   Sets or clears the specified field of the L2TP header in the auto flag

 @param[out]    hHeader - handle to the L2TP header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      HDR_L2TPv3_ConfigHeader

 @Description   Configures the L2TP header

 @param[out]    hHeader - handle to the configured L2TP header
 @param[in]     hValues - handle to the L2TP header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      HDR_L2TPv3_Randomize

 @Description   Generates random values for the L2TP header fields

 @param[out]    hHeader - handle to the L2TP header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_Randomize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_L2TPv3_Init

 @Description   Initializes the internal fields into the L2TP header structure

 @param[in]     hHeader - handle to the L2TP header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_Init(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_L2TPv3_Serialize

 @Description   Packs the L2TP header fields into the data buffer

 @param[in]     hHeader - handle to the L2TP header
 @param[out]    pBuffer - pointer to the buffer which holds the serialized L2TP header
 @param[out]    pSize - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_Serialize(t_Handle hHeader, uint8_t **pBuffer, uint32_t *pSize);

/**************************************************************************//**
 @Function      HDR_L2TPv3_GetHeaderSize

 @Description   Returns the size of the L2TP header

 @param[in]     hHeader - handle to the L2TP header

 @Return        L2TP header size
*//***************************************************************************/
uint32_t HDR_L2TPv3_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_L2TPv3_GetHeader

 @Description   Returns the L2TP header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_L2TPv3_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_L2TPv3_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_L2TPv3_GetHeaderError(t_Handle hHeader, t_Handle hError);


/** @} */ /* end of L2TPv3 group */

/**************************************************************************//**
@Group           PWE3Encap

@Description     PWE3Encap structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description  PWE3Encap
*//***************************************************************************/
typedef struct
{
    t_PgPwe3CtrlWordHeader    data;             /* auxiliary data */
    uint32_t                autoFlags;
} t_PWE3Encap;

/**************************************************************************//**
 @Function     HDR_PWE3Encap_Config()

 @Description   Configures the PWE3Encap header

 @Param[in]     hHeader -  Handle to the input PWE3Encap header

 @Return        PWE3Encap header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_PWE3Encap_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo);


/**************************************************************************//**
 @Function      HDR_PWE3Encap_Clone()

 @Description   Takes a copy of the PWE3Encap header

 @Param[in]     hObject -  Handle to the PWE3Encap header

 @Return        Cloned PWE3Encap header handle on success, NULL on failure
*//***************************************************************************/
t_Handle    HDR_PWE3Encap_Clone(t_Handle hObject);



/**************************************************************************//**
 @Function      HDR_HDR_PWE3Encap_GetField()

 @Description   Extracts the specified field of the PWE3Encap header

 @param[in]     hHeader - handle to the PWE3Encap header
 @param[in]     fieldID - field to be extracted
 @param[out]    hValue - handle to the extracted field
 @param[in]     size - size of the field to be extracted

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_GetField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);


/**************************************************************************//**
 @Function      HDR_PWE3Encap_ConfigField()

 @Description   Configures the specified field of the PWE3Encap header

 @param[out]    hHeader - handle to the PWE3Encap header
 @param[in]     fieldID - field to be set
 @param[in]     hValue - handle to the payload field
 @param[in]     size - size of the field to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_ConfigField(t_Handle hHeader, uint32_t fieldID, t_Handle hValue, uint32_t size);



/**************************************************************************//**
 @Function      HDR_PWE3Encap_ConfigAuto

 @Description   Sets or clears the specified field of the PWE3Encap header in the auto flag

 @param[out]    hHeader - handle to the PWE3Encap header
 @param[in]     fieldID - field to be set or cleared
 @param[in]     bSetAuto - variable to set or clear the field

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_ConfigAuto(t_Handle hHeader, uint32_t fieldID, bool bSetAuto);



/**************************************************************************//**
 @Function      HDR_PWE3Encap_ConfigHeader()

 @Description   Configures the PWE3Encap header

 @param[out]    hHeader - handle to the configured PWE3Encap header
 @param[in]     hValues - handle to the PWE3Encap header
 @param[in]     fieldIDs - fields to be configured

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_ConfigHeader(t_Handle hHeader, t_Handle hValues, uint32_t fieldIDs);



/**************************************************************************//**
 @Function      HDR_PWE3Encap_Randomize

 @Description   Generates random values for the PWE3Encap header fields

 @param[out]    hHeader - handle to the PWE3Encap header
 @param[in]     fieldID - fields to be randomized

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_Randomize(t_Handle hHeader);



/**************************************************************************//**
 @Function      HDR_PWE3Encap_Init

 @Description   Initializes the internal fields into the PWE3Encap header structure

 @param[in]     hHeader - handle to the PWE3Encap header

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_Init(t_Handle hHeader);



/**************************************************************************//**
 @Function      HDR_PWE3Encap_Serialize

 @Description   Packs the PWE3Encap header fields into the data buffer

 @param[in]     hHeader - handle to the PWE3Encap header
 @param[out]    p_Buffer - pointer to the buffer which holds the serialized PWE3Encap header
 @param[out]    p_Size - size of the buffer

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error     HDR_PWE3Encap_Serialize(t_Handle hHeader, uint8_t **p_Buffer, uint32_t *p_Size);



/**************************************************************************//**
 @Function      HDR_PWE3Encap_GetHeaderSize

 @Description   Returns the size of the PWE3Encap header

 @param[in]     hHeader - handle to the PWE3Encap header

 @Return        PWE3Encap header size
*//***************************************************************************/
uint32_t    HDR_PWE3Encap_GetHeaderSize(t_Handle hHeader);

/**************************************************************************//**
 @Function      HDR_PWE3Encap_GetHeader

 @Description   Returns the PWE3Encap header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_PWE3Encap_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_PWE3Encap_GetHeaderError

 @Description   Extracts the error in the header

 @param[in]     hHeader - handle to the header
 @param[in]     hError - handle to the error

 @Return        E_OK on success , Appropriate error type on failure
*//***************************************************************************/
t_Error HDR_PWE3Encap_GetHeaderError(t_Handle hHeader, t_Handle hError);

/** @} */ /* end of PWE3Encap group */

/**************************************************************************//**
@Group           Tunnelled IP

@Description     Tunnelled IP functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Function      HDR_TunnelledIPv4_GetHeader

 @Description   Returns the Tunnelled IPv4 header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_TunnelledIPv4_GetHeader(t_Handle hFrame);

/**************************************************************************//**
 @Function      HDR_TunnelledIPv6_GetHeader

 @Description   Returns the Tunnelled IPv6 header structure from the input frame

 @param[in]     hFrame - handle to the input frame

 @Return        Header on success, NULL on failure
*//***************************************************************************/
t_Handle HDR_TunnelledIPv6_GetHeader(t_Handle hFrame);


/** @} */ /* end of Tunnelled IP group */
/** @} */ /* end of Header group */
/** @} */ /* end of InternalFrame group */

#endif /* _FRAME__H */

/******************************************************************************
**                                                                           **
**   END                                                                     **
**                                                                           **
** ***************************************************************************/

