/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/

#include "std_ext.h"
#include "memcpy_ext.h"
#include "traffic_generator.h"
#include "iphc_manip.h"



static t_Error CreateFullHeader(t_TrafficGenComp     *p_TrafficGenComp,
                              bool                   defChangeFlag,
                              bool                   noChangeFlag,
                              uint8_t                *p_IpStart,
                              uint8_t                genValue,
                              uint16_t               cidIndex,
                              uint32_t               cidSize,
                              uint8_t                dBitSet)
{
     t_TrafficGenContext     *p_TrafficGenContext= &p_TrafficGenComp->p_TrafficGenContext[cidIndex];
     uint32_t                currentTime  = 0;
     uint16_t                cid = cidIndex;
     static uint32_t         generationValue;
     uint32_t                prevGenValue     = 0;

     bool       isUDP          = TRUE;
     uint8_t    iphcHeaderSize = HC_IP_UDP_COMPRESSION_SIZE;
     uint8_t    dBitValue      = 0x00;

    prevGenValue = generationValue = genValue;

    if(!p_TrafficGenContext)
    {
        RETURN_ERROR(MAJOR, E_NULL_POINTER, (" Context PRAM memory allocation"));
    }

     /* Check whether there is a change in NOCHANGE fields  */
     if (noChangeFlag || defChangeFlag)
     {
         /* Wait for Min Wrap seconds while the number of generation changes exceeds 32 */
         p_TrafficGenContext->generationCounter--;
         if (p_TrafficGenContext->generationTimeStamp + p_TrafficGenComp->minWrap > currentTime)
         {
         }
         else
         {
             p_TrafficGenContext->generationTimeStamp = currentTime;
             p_TrafficGenContext->generationCounter = p_TrafficGenComp->genCh;

             /* Save the initial value of generation after compressor driver initialization */
            generationValue = (uint32_t)(GENERATION_MASK  & p_TrafficGenContext->fullHeader[GENERATION_FIELD]) ;
             prevGenValue = generationValue;
             generationValue = generationValue + 1;

             if (generationValue > MAX_GEN_NUMBER)
            {
                generationValue = MIN_GEN_NUMBER;
            }
         }
     }


     if(p_IpStart[IPV4_PROTOCOL_OFFSET] != IP_HEADER_UDP_PROTOCOL)
     {
         isUDP = FALSE;
         iphcHeaderSize = HC_IP_ONLY_COMPRESSION_SIZE;
     }

     p_TrafficGenContext->fullHeader[GENERATION_FIELD] = (uint8_t)generationValue;

     /* Update the fullHeader */
     memcpy (&(p_TrafficGenContext->fullHeader[FULL_HEADER_FIELD]), p_IpStart, iphcHeaderSize);

     /* The fullheader complies with RFC2507 */
     p_IpStart[IP_TOT_LEN_OFFSET] &= 0x00;

     if(dBitSet)
     {
         dBitValue = SET_D_BIT;
     }

     /* Check for 8 bit or 16 bit CID and modify the input packet accordingly */
     if(cid > 255)
     {

             p_IpStart[IP_TOT_LEN_OFFSET] |= SET_CID_FLAG | generationValue | dBitValue;
             p_IpStart[IP_TOT_LEN_OFFSET + 1]    = DATA_VALUE;
             if(isUDP)
             {
                p_IpStart[UDP_TOT_LEN_OFFSET]       = (uint8_t)((cid >> 8) & 0xff);
                p_IpStart[UDP_TOT_LEN_OFFSET + 1]   = (uint8_t)(cid & 0xff);
             }
     }
     /* Check for CID Size bit 8/16 */
     else
     {
         if (cidSize)
         {
             p_IpStart[IP_TOT_LEN_OFFSET]        |= SET_CID_FLAG | generationValue | dBitValue;
             p_IpStart[IP_TOT_LEN_OFFSET + 1]    = DATA_VALUE;
             if(isUDP)
             {
                p_IpStart[UDP_TOT_LEN_OFFSET]       = 0x00;
                p_IpStart[UDP_TOT_LEN_OFFSET + 1]   = (uint8_t)(cid & 0xff);
             }

         }
         else
         {
             p_IpStart[IP_TOT_LEN_OFFSET]       |=  generationValue | dBitValue;
             p_IpStart[IP_TOT_LEN_OFFSET + 1]    = (uint8_t)(cid & 0xff);
             if(isUDP)
             {
                p_IpStart[UDP_TOT_LEN_OFFSET]       = 0x00;
                p_IpStart[UDP_TOT_LEN_OFFSET + 1]   = DATA_VALUE;
             }
         }
     }

     /* Check whether the max threshold for the number of compressed
      * headers between two full headers has been reached */
     p_TrafficGenContext->fPeriodCnt = p_TrafficGenContext->fPeriod;

     /* Check the Number of Commpressed header with the
      *Largest number of compressed NON-TCP headers  */
     if (p_TrafficGenContext->fPeriodCnt >= p_TrafficGenComp->fMaxPeriod)
     {
       p_TrafficGenContext->fPeriodCnt = p_TrafficGenComp->fMaxPeriod;
     }

     /* Doubled the fPeriod each time a full header is sent */
     if ( p_TrafficGenContext->fPeriod < p_TrafficGenComp->fMaxPeriod )
     {
         p_TrafficGenContext->fPeriod *= 2;
     }

     /* Update the next Full header Time stamp with current time +
      * duration for which compressed headers could be sent after sending last Full Header  */
     p_TrafficGenContext->fMaxTimeCnt = currentTime + p_TrafficGenComp->fMaxTime;

     return E_OK;
}

static t_Error CreateCompressedNonTCPHeader(t_TrafficGenComp     *p_TrafficGenComp,
                                             uint8_t             *p_IpStart,
                                             uint16_t            cidIndex,
                                             uint32_t            cidSize,
                                             uint8_t             *p_RemainingSize,
                                             uint8_t             *p_HeaderSize)
{
     t_TrafficGenContext     *p_TrafficGenContext= &p_TrafficGenComp->p_TrafficGenContext[cidIndex];
     uint8_t                 *p_TempFrame = NULL;
     uint32_t                byteIndex = 0;
     uint16_t                cid = cidIndex, udpChecksum;
     uint8_t                 cidDataFlag = 0;

     bool       isUDP = TRUE;
     uint8_t    iphcHeaderSize = HC_IP_UDP_COMPRESSION_SIZE;

     /* Get the Context entry for the given CID */
     if(!p_TrafficGenContext)
     {
        RETURN_ERROR(MAJOR, E_NULL_POINTER, (" Context PRAM memory allocation"));
     }

     if(p_IpStart[IPV4_PROTOCOL_OFFSET] != IP_HEADER_UDP_PROTOCOL)
     {
         isUDP = FALSE;
         iphcHeaderSize = HC_IP_ONLY_COMPRESSION_SIZE;
     }

     /* Allocate memory for tempFrame of size FrameLength */
     p_TempFrame = (uint8_t *)XX_Malloc(iphcHeaderSize);

     if (!p_TempFrame)
     {
         XX_Print(" TempFrame memory allocation \n");
         return E_INVALID_VALUE;
     }

     memset ((char*)p_TempFrame,0,iphcHeaderSize);

     /* Check for 8 bit or 16 bit CID and modify the input packet accordingly */
     if (cid > 255)
     {
         /*Write the MSB of CID*/
         p_TempFrame[byteIndex++] = (uint8_t)((cid >> 8) & 0xff);
     }
     else
     {
         /*Check for CID Size bit*/
         if (cidSize)
         {
             /*Write 8 bit CID as 16 bit*/
             p_TempFrame[byteIndex++] = 0x00;
         }
         else
         {
             /*Write the CID*/
             p_TempFrame[byteIndex++] = (uint8_t)(cid & 0xff);
         }
     }

     if (cid > 255 || cidSize )
     {
         p_TempFrame[byteIndex++] = (uint8_t)(((cidDataFlag | 0x2) << 6) | (p_TrafficGenContext->fullHeader[GENERATION_FIELD] & GENERATION_MASK));
         p_TempFrame[byteIndex++] = (uint8_t)(cid & 0xFF);   /*Write the LSB of CID*/
     }
     else
     {
         p_TempFrame[byteIndex++] = (uint8_t)(((cidDataFlag) << 6) | (p_TrafficGenContext->fullHeader[GENERATION_FIELD] & GENERATION_MASK));
     }

     /* Writing 2 byte IPv4 ID */
     p_TempFrame[byteIndex++] = p_IpStart[IPV4_ID_OFFSET];
     p_TempFrame[byteIndex++] = p_IpStart[IPV4_ID_OFFSET + 1];

     if(isUDP)
     {
       /* Check if UDP Checksum in previous context is non-zero */
       udpChecksum = p_TrafficGenContext->fullHeader[UDP_CHECKSUM_OFFSET + FULL_HEADER_FIELD];
       udpChecksum = (uint16_t)((udpChecksum << 8) | p_TrafficGenContext->fullHeader[UDP_CHECKSUM_OFFSET + 1 + FULL_HEADER_FIELD]);

       if (udpChecksum)
       {
           /*Writing 2 byte UDP Checksum*/
           p_TempFrame[byteIndex++] = p_IpStart[UDP_CHECKSUM_OFFSET];
           p_TempFrame[byteIndex++] = p_IpStart[UDP_CHECKSUM_OFFSET + 1];
       }
     }

     /* Update the length of the constructed packet */

     memset((char*)p_IpStart,0,iphcHeaderSize);

     /*Copy the constructed Compressed packet back to the input frame array*/
     MemCpy32(p_IpStart, p_TempFrame, (uint32_t)byteIndex);
     *p_RemainingSize = (uint8_t)(iphcHeaderSize - byteIndex) ;
     *p_HeaderSize = (uint8_t)byteIndex  ;

     XX_Free(p_TempFrame);
     /* Update the value of fPeriodCnt*/
     p_TrafficGenContext->fPeriodCnt--;

     return E_OK;
}

t_Handle PG_IPHC_Init(t_PgIphcParam  *p_IphcParam)
{
     t_TrafficGenComp        *p_TrafficGenComp;
     t_TrafficGenContext     *p_TrafficGenContext = NULL;
     t_TrafficIphc           *p_TrafficIphc = NULL;
     int                      i = 0, j = 0;

     p_TrafficIphc = (t_TrafficIphc *)XX_Malloc(sizeof(t_TrafficIphc));
     if (!p_TrafficIphc)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficIphc"));
         return NULL;
     }
     memset(p_TrafficIphc, 0, sizeof(p_TrafficIphc));

     p_TrafficIphc->numOfIphcCompressors = p_IphcParam->numOfIphcCompressors;

     p_TrafficIphc->p_TrafficGenComp = XX_Malloc(p_IphcParam->numOfIphcCompressors * sizeof(t_TrafficGenComp));
     if (!p_TrafficIphc->p_TrafficGenComp)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficIphc->p_TrafficGenComp"));
         return NULL;
     }
     memset(p_TrafficIphc->p_TrafficGenComp, 0, p_IphcParam->numOfIphcCompressors * sizeof(t_TrafficGenComp));

     for(i = 0; i < p_TrafficIphc->numOfIphcCompressors; i++)
     {
         p_TrafficGenComp = &p_TrafficIphc->p_TrafficGenComp[i];

         p_TrafficGenComp->genCh               = p_IphcParam->p_IphcCompParams[i].genCh;
         p_TrafficGenComp->maxNonTcpSpace      = p_IphcParam->p_IphcCompParams[i].maxNonTcpSpace;
         p_TrafficGenComp->fMaxPeriod          = p_IphcParam->p_IphcCompParams[i].fMaxPeriod;
         p_TrafficGenComp->maxHeader           = p_IphcParam->p_IphcCompParams[i].maxHeader;
         p_TrafficGenComp->minWrap             = p_IphcParam->p_IphcCompParams[i].minWrap;
         p_TrafficGenComp->fMaxTime            = p_IphcParam->p_IphcCompParams[i].fMaxTime;

         p_TrafficGenComp->p_TrafficGenContext = (t_TrafficGenContext *)XX_Malloc((p_TrafficGenComp->maxNonTcpSpace + 1) * sizeof(t_TrafficGenContext));
         if(!p_TrafficGenComp->p_TrafficGenContext)
         {
            PG_IPHC_Free(p_TrafficIphc);
            REPORT_ERROR(MAJOR, E_NULL_POINTER, ("Memory could not be allocated for Compressor Golden Model \n"));
            return NULL;
         }
         memset(p_TrafficGenComp->p_TrafficGenContext, 0, (p_IphcParam->p_IphcCompParams[i].maxNonTcpSpace + 1) * sizeof(t_TrafficGenContext));

            for(j = 0; j < p_TrafficGenComp->maxNonTcpSpace + 1; j++)
            {
                p_TrafficGenContext = &p_TrafficGenComp->p_TrafficGenContext[j];

             /* Need to get populated from user data structure*/
                 p_TrafficGenContext->checkSum              = 0;
                 p_TrafficGenContext->generationCounter     = p_TrafficGenComp->genCh;
                 p_TrafficGenContext->fPeriodCnt            = 0x1;
                 p_TrafficGenContext->fPeriod               = 0x1;
                 p_TrafficGenContext->fMaxTimeCnt           = 0;
                 p_TrafficGenContext->generationTimeStamp   = 0;
                memset(p_TrafficGenContext->fullHeader, 0,FULL_HEADER_SIZE);
                 p_TrafficGenContext->fullHeader[1]         = 0;
             }
        }
    return (t_Handle)p_TrafficIphc;
}

t_Error PG_IPHC_Reset(t_Handle h_TrafficIphc)
{
     t_TrafficIphc           *p_TrafficIphc = (t_TrafficIphc *)h_TrafficIphc;
     t_TrafficGenComp        *p_TrafficGenComp;
     t_TrafficGenContext     *p_TrafficGenContext = NULL;
     int                     i = 0, j = 0;

    if(!p_TrafficIphc)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("p_TrafficIphc\n"));
    }

    for(i = 0; i < p_TrafficIphc->numOfIphcCompressors; i++)
    {
         p_TrafficGenComp = &p_TrafficIphc->p_TrafficGenComp[i];
         memset(p_TrafficGenComp->p_TrafficGenContext, 0, (p_TrafficIphc->p_TrafficGenComp[i].maxNonTcpSpace + 1) * sizeof(t_TrafficGenContext));

            for(j = 0; j < p_TrafficGenComp->maxNonTcpSpace + 1; j++)
            {
                p_TrafficGenContext = &p_TrafficGenComp->p_TrafficGenContext[j];

             /* Need to get populated from user data structure*/
                 p_TrafficGenContext->checkSum              = 0;
                 p_TrafficGenContext->generationCounter     = p_TrafficGenComp->genCh;
                 p_TrafficGenContext->fPeriodCnt            = 0x1;
                 p_TrafficGenContext->fPeriod               = 0x1;
                 p_TrafficGenContext->fMaxTimeCnt           = 0;
                 p_TrafficGenContext->generationTimeStamp   = 0;
                 memset(p_TrafficGenContext->fullHeader, 0,FULL_HEADER_SIZE);
                 p_TrafficGenContext->fullHeader[1]         = 0;
             }
    }
    return E_OK;
}

t_Error PG_IPHC_Free(t_Handle h_TrafficIphc)
{
    int i = 0;

    t_TrafficGenComp *p_TrafficGenComp = NULL ;
    t_TrafficIphc    *p_TrafficIphc = (t_TrafficIphc *)h_TrafficIphc;

    if(!p_TrafficIphc)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("p_TrafficIphc\n"));
    }

    for(i = 0; i < p_TrafficIphc->numOfIphcCompressors; i++)
    {
        p_TrafficGenComp = &p_TrafficIphc->p_TrafficGenComp[i];
        if(p_TrafficGenComp)
        {
            if(p_TrafficGenComp->p_TrafficGenContext)
            {
                XX_Free(p_TrafficGenComp->p_TrafficGenContext);
                p_TrafficGenComp->p_TrafficGenContext = NULL;
            }
        }
    }
    XX_Free(p_TrafficIphc->p_TrafficGenComp);
    p_TrafficIphc->p_TrafficGenComp = NULL;

    XX_Free(p_TrafficIphc);
    p_TrafficIphc = NULL;

    return E_OK;
}

t_Error IPHC_GoldenModel(  uint16_t         cidIndex,
                            uint8_t          compIndex,
                            uint8_t          *p_IpStart,
                            uint8_t          genValue,
                            uint32_t         cidSize,
                            uint8_t          dBitSet,
                            uint8_t          *p_RemainingSize,
                            uint8_t          *p_HeaderSize,
                            bool             *p_IsFH,
                            t_Handle         h_TrafficIphc)

{
     t_TrafficIphc           *p_TrafficIphc = (t_TrafficIphc *)h_TrafficIphc;
     t_TrafficGenComp        *p_TrafficGenComp = NULL;
     t_TrafficGenContext     *p_TrafficGenContext = NULL;
     t_Error                 errorVal =  E_INVALID_VALUE;
     bool                    noChangeFlag = FALSE;
     bool                    defChangeFlag = FALSE;
     uint32_t                currentTime = 0;

     if(!p_TrafficIphc)
     {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("p_TrafficIphc"));
     }

     p_TrafficGenComp = &p_TrafficIphc->p_TrafficGenComp[compIndex];
     p_TrafficGenContext = &p_TrafficGenComp->p_TrafficGenContext[cidIndex];

     /* Send FullHeader based on Slow Start refresh mechanism */

     /* Check number of compressed header reached zero */
     if (!p_TrafficGenContext->fPeriodCnt)
     {
         errorVal = CreateFullHeader(p_TrafficGenComp,defChangeFlag, noChangeFlag, p_IpStart, genValue ,cidIndex, cidSize, dBitSet);
         return errorVal;

     }
     /* Check the timestamp of the next fullheader is less than current time */
     else if (currentTime >= p_TrafficGenContext->fMaxTimeCnt)
     {
         errorVal = CreateFullHeader(p_TrafficGenComp, defChangeFlag, noChangeFlag, p_IpStart, genValue, cidIndex, cidSize, dBitSet);
         return errorVal;

     }
     /* IPHC_GoldenModel to generate compressed frames*/
     else
     {
         errorVal = CreateCompressedNonTCPHeader(p_TrafficGenComp, p_IpStart, cidIndex, cidSize, p_RemainingSize, p_HeaderSize);
         *p_IsFH = FALSE;
         return errorVal;
     }

     return E_OK;
}

