/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
  @File          iphc_manip.h

  @Description   IP Header Compression (HC) Golden Model related data structures.

******************************************************************************/

#ifndef __IPHC_MANIP_H
#define __IPHC_MANIP_H

#include "std_ext.h"
#include "string_ext.h"
#include "pg_ext.h"


/*IP Frame related defines */
#define IP_HEADER_VERSION                       4
#define IP_HEADER_UDP_PROTOCOL                  17
#define IP_HEADER_TCP_PROTOCOL                  6
#define IP_HEADER_LENGTH                        20

#define IP_LENGTH_OFFSET                         2      /* IP total length offset */
#define UDP_LENGTH_OFFSET                        24     /* UDP length offset */
#define CID_16BIT_MASK                           0x80   /* Bit mask for 16 bit cide size */
#define IP_VERSION_OFFSET                        0      /* IP Version Offset */
#define IP_IHL_VALUE                             5      /* IP IHL Value */
#define IP_HEADER_VERSION_VALUE                  4      /* IP Header version value */
#define FULL_HEADER_SIZE                         32     /* size of full header */
#define HEADER_SIZE_TO_DECOMPRESS                28     /* Header size to decompress */
#define IP_IHL_MASK                              0x0F   /* IP IHL Mask */
#define GENERATION_BIT_MASK                      0x3F   /* Generation value bit mask */

/*HC related defines*/
#define HC_IP_UDP_COMPRESSION                   1
#define HC_IP_UDP_COMPRESSION_SIZE              28
#define HC_IP_ONLY_COMPRESSION_SIZE             20
#define UDP_CHECKSUM_OFFSET                     26
#define IP_TOT_LEN_OFFSET                       2
#define UDP_TOT_LEN_OFFSET                      24
#define IP_VER_IHL_OFFSET                       0
#define DATA_VALUE                              0
#define IPV4_ID_OFFSET                          4
#define IPV4_PROTOCOL_OFFSET                    9
#define GENERATION_FIELD                        1
#define MIN_GEN_NUMBER                          0
#define MAX_GEN_NUMBER                          63
#define GENERATION_MASK                         0x3F
#define SET_CID_FLAG                            0x80
#define SET_D_BIT                               0x40
#define FULL_HEADER_FIELD                       GENERATION_FIELD + 1

#define HC_IP_UDP_COMPSIZE_8BITCID               6
#define HC_IP_UDP_COMPSIZE_16BITCID              8

/*PPP frame related defines*/
#define IPV4_HEADER_TYPE                        0x0021
#define FULL_HEADER_TYPE                        0x0061
#define COMPRESSED_NONTCP_HEADER_TYPE           0x0065

/*Attribute related defines*/
#define SET_COMP_DISABLE                        0x01
#define SET_INVALID_CID                         0x02
#define SET_INVALID_COMP_TYPE                   0x04
#define SET_INVALID_COMP_SIZE                   0x08
#define SET_NO_ADL                              0x10
#define SET_INVALID_VERSION                     0x20
#define SET_INVALID_IHL                         0x40
#define SET_INVALID_PROTOCOL                    0x80

/*Decompressor Interrupt related defines*/
#define SET_INTR_NEW_ADL                         0x01
#define SET_INTR_GEN_CHANGE_FH                   0x02
#define SET_INTR_GEN_CHANGE_CH                   0x04
#define SET_NOT_FHIPV4UDP                        0x08
#define SET_CONTEXT_RTP_ERR                      0x10
#define CLEAR_GEN_CHANGE_CH                      0x2b
#define SET_CONTEXT_VALID                        0x20
#define DECOMP_NEWADL_INTERRUPT_MASK             0x80000000
#define DECOMP_GENCH_INTERRUPT_MASK              0x40000000
#define DECOMP_FH_WITHDBIT_CONTEXTINVALID        0x30

/*Decompressor Interrupt Entry related defines*/
#define DECOMP_SET_FH_INTERRUPT_ENTRY            0x8000
#define DECOMP_SET_CI_INTERRUPT_ENTRY            0x4000
#define DECOMP_PCD_ID                            0x01

#define BMR_BIG_ENDIAN                           0x10
//#define BMR_DTB_SEC_BUS                        0x02           /* BMR - Data bus bit mask */
#define BMR_BTB_SEC_BUS                      0x01           /* BMR - BD bus bit mask */
#define BMR_GBL                              0x20           /* BMR - Global bit mask */

 /* WIPRO_14_12_2006 Start;Constants used by Golden Model */
 #define GM_COMPRESSOR_MODE                      0
 #define GM_DECOMPRESSOR_MODE                    1
 #define GM_COMPRESSOR_FAIL_MODE                 2 /* To test compressor fail statistics counter */
 /* WIPRO_14_12_2006 End */


typedef enum {
    e_GM_NO_ERROR,
    e_GM_IP_Version_Error,
    e_GM_IP_IHL_Error,
    e_GM_IP_ProtocolType_Error,
    e_GM_CH_GenerationChange_Error,
    e_GM_Diffgen_Error,
    e_GM_CID_OutofRange_Error,
    e_GM_FH_NonAdl_InvalidContext_Frame,
    e_GM_NOT_VALID,
    e_GM_DROP_FRAME,
    e_GM_Decompressor_Disable
} e_GM_Error_Cases;

typedef struct {
     uint16_t        frameLength;
     uint16_t        goldenModelCompLength;
     uint16_t        cid;            //CID for HC frames
     uint8_t         dBit;           //Used for RTP Payload Error frame generation
     uint8_t         attrErrorFlags;//Attribute variable for  Error Flags
                                     //bit 0 - CompressorDisable
                                     //bit 1 - Invalid CID
                                     //bit 2 - InvalidCompressionType
                                     //bit 3 - InvalidCompressionSize
                                     //bit 4 - invalid ADL
                                     //bit 5 - invalid version
                                     //bit 6 - invalid IHL
                                     //bit 7 - invalid protocol
     uint16_t        attrDecompInterruptFlags;// Set When an event raises an interrupt is identified.
                                              // bit 0 - For New ADL Interrupt
                                              // bit 1 - For Generation Change Interrupt raised for Full header
                                              // bit 2 - For Generation Change Interrupt raised for Compressed header
                                              // bit 3 - Indicates DROP state
                                              // bit 4 - Indicates MISS2 state
                                              // bit 5 - Indicates that RTPPayload error occurred when context was valid
                                              // bit 6 - Indicates whether the context was valid during current frame processing
     uint8_t         generationNumber;         // Generation number of the frame
                                                  // Attribute variables
     bool            attrDrop;
     bool            attrDecompress;
     bool            attrMisstoCPU;
     bool            attrCompress;
     bool            attrFullheader;
     bool            attrUncompress;
     uint8_t         errorFrameGen;            // error flag
} t_GoldenFrameStack;

typedef struct {
    uint8_t     checkSum;
    uint8_t     generationCounter;
    uint8_t     fullHeader[FULL_HEADER_SIZE];
    uint16_t    fPeriodCnt;
    uint16_t    fPeriod;
    uint32_t    fMaxTimeCnt;
    uint32_t    generationTimeStamp;
} t_TrafficGenContext;

typedef struct {
    uint8_t     genCh;
    uint16_t    maxNonTcpSpace;
    uint16_t    fMaxPeriod;
    uint16_t    maxHeader;
    uint32_t    minWrap;
    uint32_t    fMaxTime;

    t_TrafficGenContext  *p_TrafficGenContext;
} t_TrafficGenComp;

typedef struct {
    uint8_t                 numOfIphcCompressors;
    t_TrafficGenComp        *p_TrafficGenComp;
}t_TrafficIphc;

 /**************************************************************************//**
 @Function      PG_IPHC_Init

 @Description   Creates and intializes a IPHC object.

 @Param[in]     p_IphcParam - Pointer to the IPHC Param structure

 @Return        Handle to the IPHC object
*//***************************************************************************/
t_Handle PG_IPHC_Init(t_PgIphcParam  *p_IphcParam);

/**************************************************************************//**
 @Function      PG_IPHC_Free

 @Description   Frees the IPHC object created during the PG_IPHC_Init

 @Param[in]     h_TrafficIphc - Handle to the IPHC object

 @Return        E_OK on success, Error code otherwise
*//***************************************************************************/
 t_Error   PG_IPHC_Free(t_Handle h_TrafficIphc);

 /**************************************************************************//**
 @Function      PG_IPHC_Reset

 @Description   Resets the IPHC object created during the PG_IPHC_Init

 @Param[in]     h_TrafficIphc - Handle to the Iphc object

 @Return        E_OK on success, Error code otherwise

 @Cautions      Must be used to reset the context of IPHC when multiple frame list
                are created for the same Context Identifier (CID)
*//***************************************************************************/
 t_Error  PG_IPHC_Reset(t_Handle h_TrafficIphc);

 /************************************************************************************
 @Function IPHC_GoldenModel

 @Description Processes the generated input frames and produces Compressed frames

 @Param[in]     cidIndex      - Context identifier
 @Param[in]     compIndex     - Compression index
 @Param[in]     p_IpStart     - Pointer to IP start address
 @Param[in]     genValue      - Generation value
 @Param[in]     cidSize       - Context identifier size
 @Param[in]     dBitSet       - D-Bit set
 @Param[out]    p_RemainingSize - Remaining frame size
                                  (IPHC frame size - compressed header size)
 @Param[out]    p_HeaderSize    - Compressed header size
 @Param[out]    p_IsFH          - Flag to identify full header/compressed header
 @Param[in]     h_TrafficIphc   - Handle to the IPHC object

 @Return        E_OK on success, Error code otherwise

 @Cautions      None
 ************************************************************************************/
 t_Error IPHC_GoldenModel(uint16_t     cidIndex,
                          uint8_t      compIndex,
                          uint8_t      *p_IpStart,
                          uint8_t      genValue,
                          uint32_t     cidSize,
                          uint8_t      dBitSet,
                          uint8_t      *p_RemainingSize,
                          uint8_t      *p_HeaderSize,
                          bool         *p_IsFH,
                          t_Handle     h_TrafficIphc);


 #endif /* __IPHC_MANIP_H */

