/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          itg_common.c

  @Description   Traffic Generator and Traffic Analyzer common functions
*//***************************************************************************/
#include "std_ext.h"
#include "stdlib_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "mem_ext.h"
#include "buf_ext.h"
#include "mb_frame_ext.h"
#include "frame_ext.h"
#include "fmanip_ext.h"
#include "iphc_manip.h"
#include "itg_common.h"
#include "traffic_generator.h"
#include "traffic_analyzer.h"

/**************************************************************************//**
  @Group          ITG

  @Description    ITG Functions

  @{
*//***************************************************************************/


t_TrafficPort* ITG_CreateTrafficPort(void)
{
     t_TrafficPort *p_TrafficPort = NULL;

     p_TrafficPort = (t_TrafficPort *)XX_Malloc(sizeof(t_TrafficPort));
     if (!p_TrafficPort)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("Traffic port!"));
         return NULL;
     }
     memset(p_TrafficPort, 0 , sizeof(t_TrafficPort));

     INIT_LIST(&p_TrafficPort->portsNode);

     return p_TrafficPort;
}

void ITG_AddPortToList(t_TrafficPort *p_TrafficPort, t_List *p_PortsLst)
{
     uint32_t             intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_AddToTail(&p_TrafficPort->portsNode, p_PortsLst);
     XX_RestoreAllIntr(intFlags);
}

t_TrafficPort * ITG_RemovePortFromList(t_Handle h_Port, t_List *p_PortsLst)
{
     t_TrafficPort        *p_TrafficPort = NULL;
     uint32_t             intFlags;
     t_List               *p_Pos;
     bool                 found = FALSE;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(p_PortsLst))
     {
         LIST_FOR_EACH(p_Pos, p_PortsLst)
         {
            p_TrafficPort = CONFIG_P_TRAFFIC_PORT_OBJECT(p_Pos);

            if (p_TrafficPort->h_Port == h_Port)
            {
                LIST_DelAndInit(&p_TrafficPort->portsNode);
                found = TRUE;
                break;
            }
         }
     }

     if (!found)
     {
        p_TrafficPort = NULL;
     }

     XX_RestoreAllIntr(intFlags);
     return p_TrafficPort;
}


t_TrafficPort * ITG_GetPort(t_List *p_PortsLst, uint16_t nthPort)
{
     t_TrafficPort        *p_TrafficPort = NULL;
     uint32_t             intFlags;
     t_List               *p_Pos;
     uint32_t             i = 0;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(p_PortsLst))
     {
         p_Pos = p_PortsLst->p_Next;

         /* Get the nth node in the List */
         for(i = 0 ; i < nthPort ; p_Pos = LIST_NEXT(p_Pos), i++)
                ;
         p_TrafficPort = CONFIG_P_TRAFFIC_PORT_OBJECT(p_Pos);
     }

     XX_RestoreAllIntr(intFlags);
     return p_TrafficPort;
}

void ITG_RechainList(t_List *p_FramesLst, t_List *p_FirstNode)
{
     t_List *p_OrigTxLastNode = LIST_PREV(p_FirstNode);
     t_List *p_CurFirstNode   = LIST_FIRST(p_FramesLst);
     t_List *p_CurLastNode    = LIST_LAST(p_FramesLst);

     LIST_NEXT(p_CurLastNode)  = p_CurFirstNode;
     LIST_PREV(p_CurFirstNode) = p_CurLastNode;

     LIST_NEXT(p_OrigTxLastNode) = p_FramesLst;
     LIST_PREV(p_FramesLst)      = p_OrigTxLastNode;

     LIST_NEXT(p_FramesLst) = p_FirstNode;
     LIST_PREV(p_FirstNode) = p_FramesLst;
}

/** @} */ /* end of ITG group */

