/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/***************************************************************************//**
@File          itg_common.h

@Description   Traffic Generator and Analyzer Common Structures And Definitions
*//***************************************************************************/

#ifndef __ITG_COMMON_H
#define __ITG_COMMON_H

#include "error_ext.h"
#include "std_ext.h"
#include "list_ext.h"
#include "mb_frame_ext.h"
#include "itg_common_ext.h"

/**************************************************************************//**
 @Description       Macros
*//***************************************************************************/
#define __ERR_MODULE__                      MODULE_UNKNOWN

#define F_SetFrameInfo                      F_SetInfo1
#define F_GetFrameInfo                      F_GetInfo1

#define JUMBO_FRAME_SIZE                    9600    /* Ethernet jumbo frame size */
#define BYTE_TO_BITS                                8       /* One Byte = 8 bits */

#define PRINT_STAT(str,val) \
{                           \
    if(val)                 \
    {                       \
        XX_Print(str,val);  \
    }                       \
}

#define CONFIG_P_TRAFFIC_PORT_OBJECT(ptr)           LIST_OBJECT(ptr, t_TrafficPort, portsNode)

/**************************************************************************//**
 @Description   ITG Common Structure
*//***************************************************************************/
typedef struct t_TrafficFrameInfo
{
     t_Handle            h_FrameHeaderType;     /**< Handle to frame encapsulation */
     t_Handle            h_App;                 /**< Application frame handle */
     uint32_t            frameId;               /**< Frame Index */
     uint32_t            payloadLength;         /**< Payload length info */
     uint32_t            payloadOffset;         /**< Payload offset info */
     uint32_t            frameErrorType;        /**< Frame error info */
     t_Handle            h_FrameGen;            /**< Handle to frame layout information */
     t_Handle            h_BufMemAccess;       /**<  Handle to buffers memory access parameters */
     uint32_t            dataByteAlignment;     /**< Data byte alignment value */
     t_Handle            h_ProtocolInfo;        /**< Handle to protocol information */
} t_TrafficFrameInfo;


#endif /* __ITG_COMMON_H */

/******************************************************************************
**                                                                           **
**   END                                                                     **
**                                                                           **
** ***************************************************************************/
