/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************

@File          traffic_analyzer.h

@Description   Traffic Analyzer Structures And Definitions
*//***************************************************************************/

#ifndef __TRAFFIC_ANALYZER_H
#define __TRAFFIC_ANALYZER_H

#include "std_ext.h"
#include "error_ext.h"
#include "list_ext.h"
#include "traffic_analyzer_ext.h"


#define __ERR_MODULE__  MODULE_UNKNOWN


/**************************************************************************//**
 @Description       Macros
*//***************************************************************************/

#define RECV_F_TRAFFIC_ANALYZER_FRAME_OBJECT(ptr)       LIST_OBJECT(ptr, t_PgFrame, frmsNode)
#define COMPDEPTH_F_TRAFFIC_ANALYZER_FRAME_OBJECT(ptr)  LIST_OBJECT(ptr, t_PgFrame, frmsNode)

#define GET_NTH_ANALYZER_PORT_HANDLE(lst,portId)  (t_TrafficAnalyzerPort *)(GET_PORT_HANDLE(lst,portId))

#define READ_SEQ_NUM(h_Frame, p_SegNum, size)\
{\
    uint16_t offset;\
    offset = F_GetLength(h_Frame);\
    offset -= MAX_SEQ_NUM_SIZE;\
    size = F_Read(h_Frame, p_SegNum, offset, MAX_SEQ_NUM_SIZE);\
}

#define TA_BPS_TO_NANO_SEC_FACTOR               8000000000LL
#define OPTIMAL_ANALYZER_TIME_TO_WAIT           80000000


/**************************************************************************//**
 @Description   Traffic Analyzer Specific Structures
*//***************************************************************************/

#define MAX_NUM_FIXED_PATTERNS      6

#define PAYLOAD_FIXED_PATTERN_TYPES \
{                                   \
    { {0x5A, 0x5A},2 },             \
    { {0x55, 0xAA},2 },             \
    { {0x55, 0x55},2 },             \
    { {0x00, 0xFF},2 },             \
    { {0xF0, 0xF0},2 },             \
    { {0x00, 0x55, 0xAA, 0xFF},4 }, \
}

/**************************************************************************//**
 @Description   Traffic Analyzer Queue Structure
*//***************************************************************************/
typedef struct t_TrafficAnalyzerQueue
{
    uint32_t                    queueId;            /* Queue identifier */
    t_List                      *p_RxDataFramesLst; /* Receive frames list */
    t_List                      *p_FirstNode;       /* Pointer to first element in Rx list */
    t_List                      compDepthFramesLst; /* Stored receive frames list */
    uint32_t                    rxFrames;           /* Rx frames counter */
    uint32_t                    rxErrors;           /* Rx errors counter */
    uint32_t                    compDepthCnt;       /* Comparison depth count */
    uint32_t                    numOfFrameListSubSet;   /* Number of frame descriptors (id-weight pair) */
    uint32_t                    *p_PgFrameWeights;      /* Pointer to weight settings */
    uint32_t                    *p_RxFrameTypeCounter;  /* Received frame type counter */
    uint32_t                    rxTotalWeightFrames;    /* Sum of weight of all frame id's */

} t_TrafficAnalyzerQueue;

/**************************************************************************//**
 @Description   Traffic Analyzer Port Structure
*//***************************************************************************/
typedef struct t_TrafficAnalyzerPort
{
    struct t_TrafficAnalyzer    *p_TrafficAnalyzer;
    uint16_t                    portId;             /* Port identifier */
    bool                        inUse;              /* Port status */
    t_Handle                    h_Dev;              /* Device handle */
    uint32_t                    numOfFrames;        /* Number of frames */
    volatile bool               testPassed;         /* Test status */

    volatile uint32_t           rxBytes;            /* Number of bytes expected */
    uint32_t                    byteTime;           /* Byte time */

    t_Error                     (*f_DevRx) (t_Handle h_Dev, uint32_t queueId); /* Receive callback function */
    t_Error                     (*f_DevRxConf) (t_Handle h_Dev, uint32_t queueId, t_Handle h_Frame); /* Receive confirmation callback function */
    uint32_t                    numOfQueues;        /* Number of queues */
    t_TrafficAnalyzerQueue      *p_Queues;          /* Receive queue parameters */

    uint32_t                    *p_QueueIndexMap;   /* Map the queue index */
    t_TrafficAnalyzerRxStatistics rxStatistics;     /* Receive port statistics */
    bool                        *p_EnableFlexibleRxMode; /* Flexible receive analysis mode */
    uint32_t                    underSizeFrameValue;/* Undersize frame value */
    uint32_t                    overSizeFrameValue; /* Oversize frame value */
    uint32_t                    maxQueueIndex;      /* Maximal possible receive queue index number */

    t_Error                     (*f_UserInspection) (t_Handle h_Frame); /* Call back function for User Inspection on the received packets */
} t_TrafficAnalyzerPort;

/**************************************************************************//**
 @Description   Traffic Analyzer Structure
*//***************************************************************************/
typedef struct t_TrafficAnalyzer
{
    uint16_t                        numOfPorts;         /* Number of ports */
    t_List                          portsLst;           /* Rx Port list */

    t_TrafficAnalyzerRxStatistics   globalRxStatistics; /* Global receive statistics */
    e_FrameComparisonMode           frameComparisonMode;/* Frame Comparison mode */
    uint16_t                        comparisonDepth;    /* Comparison depth */
    t_PgPayloadFixedPattern         payloadFixedPattern[MAX_NUM_FIXED_PATTERNS];

    uint32_t                        timeToWait;         /* Time to wait for the packets to get received */

    bool                            analyzerStopped;
    bool                            pollingMode;        /* Polling mode or Interrupt Mode */
    uint32_t                        pollingPeriod;      /* Polling interval period  */
    t_Handle                        h_Timer;            /* Handle to the timer */

} t_TrafficAnalyzer;

#endif /* __TRAFFIC_ANALYZER_H */


