/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************

@File          traffic_generator.h

@Description   Traffic Generator Structures And Definitions
*//***************************************************************************/

#ifndef __TRAFFIC_GENERATOR_H
#define __TRAFFIC_GENERATOR_H

#include "error_ext.h"
#include "std_ext.h"
#include "list_ext.h"
#include "traffic_generator_ext.h"

#define __ERR_MODULE__  MODULE_UNKNOWN


/**************************************************************************//**
 @Description       Macros
*//***************************************************************************/
#define RDY_F_TRAFFIC_GEN_FRAME_OBJECT(ptr)             LIST_OBJECT(ptr, t_PgFrame, frmsNode)
#define CONF_F_TRAFFIC_GEN_FRAME_OBJECT(ptr)            LIST_OBJECT(ptr, t_PgFrame, frmsNode)

#define GET_NTH_GEN_PORT_HANDLE(lst,portId)  (t_TrafficGenPort *)(GET_PORT_HANDLE(lst,portId))

#define TG_BPS_TO_NANO_SEC_FACTOR               8000000000LL

/**************************************************************************//**
 @Description   Traffic Generator Specific Structures
*//***************************************************************************/

/**************************************************************************//**
 @Description   Traffic Generator Queue Structure
*//***************************************************************************/
typedef struct t_TrafficGenQueue
{
    uint32_t                    queueId;                /* Queue identifier */
    t_List                      *p_EmptyFramesLst;      /* Empty frames list */
    t_List                      confFramesLst;          /* Confirmation frames list */
    t_List                      *p_FirstNode;           /* Pointer to first element in Tx list */
    uint32_t                    txConfedFrames;         /* Tx confirmed frames counter */
    uint32_t                    txFrames;               /* Tx frames counter */
    uint32_t                    txErrors;               /* Tx errors counter */
    uint32_t                    txFramesInLst;          /* Tx frames counter in Empty frames list  -
                                                           initial number of frames in list */
    uint32_t                    txFrameCounter;         /* Volatile Tx frames counter in Empty frames list -
                                                           current number of frames in list, used in shuffle mode */
    uint32_t                    txConfFrameCounter;     /* Volatile Tx confirmation frames counter in Empty frames list,
                                                           used in shuffle mode */
    uint32_t                    txLeftoverCounter;      /* In shuffle mode: number of frames that were transmitted
                                                           but still not confirmed */
    bool                        txShuffleFrames;        /* Shuffle option for queue */
    uint32_t                    retransmitThreshold;    /* Counter to retransmit frame */
} t_TrafficGenQueue;

/**************************************************************************//**
 @Description   Traffic Generator Port Structure
*//***************************************************************************/
typedef struct t_TrafficGenPort
{
    struct t_TrafficGen         *p_TrafficGen;
    uint16_t                    portId;             /* Port identifier */
    bool                        inUse;              /* Port status */
    t_Handle                    h_Dev;              /* Device handle */
    uint32_t                    numOfFrames;        /* Number of frames */
    uint32_t                    byteTime;           /* Byte time */
    uint32_t                    burstSize;          /* Burst size */
    uint32_t                    tmpTxBytes;         /* */
    uint32_t                    counterHistory;     /* */
    uint32_t                    lastTimeStamp;      /* */
    bool                        keepGoing;          /* */
    volatile bool               testPassed;         /* Test status */
    uint32_t                    errorsThreshold;    /* Error threshold */
    volatile uint32_t           txBytes;            /* Number of bytes transmitted */
    uint32_t                    lastTxTimeStamp;    /* */
    t_Error                     (*f_DevTx) (t_Handle h_Dev, uint32_t queueId, t_Handle h_Frame); /* Transmit callback function */
    void                        (*f_DevTxConf) (t_Handle h_Dev, uint32_t queueId); /* Transmit confirmation callback function */
    uint32_t                    numOfQueues;        /* Number of transmit queues */
    t_TrafficGenQueue           *p_Queues;          /* Transmit queue parameters */

    uint32_t                    *p_QueueIndexMap;   /* Map the queue index */
    uint32_t                    queueCounter;       /* Queue counter incremented to achieve round robin scheduling */
    bool                        getNextQueueHandle; /* Flag to get next queue handle in poet slice scheduling mechanism */

    t_TrafficGenTxStatistics    txStatistics;       /* Transmit port statistics */
    uint32_t                    underSizeFrameValue;/* Undersize frame value */
    uint32_t                    overSizeFrameValue; /* Oversize frame value */
    bool                        noConfirmationMode; /* No Tx confirmation is required when TRUE,
                                                       frame object will be deleted upon successful transmission */
    uint32_t                    mtu;                /* The port's IP MTU - If non zero, this port does IP fragmentation */
} t_TrafficGenPort;

/**************************************************************************//**
 @Description   Traffic Generator Structure
*//***************************************************************************/
typedef struct t_TrafficGen
{
    uint16_t                    numOfPorts;         /* Number of ports */
    t_List                      portsLst;           /* Tx Port list */

    bool                        isTxStopped;        /* Tx stop flag */

    t_TrafficGenTxStatistics    globalTxStatistics; /* Global transmit statistics */

    e_TgOperationMode           txMode;             /* Transmission mode */

    uint32_t                    (*f_SchedMech) (t_TrafficGenPort  *p_TrafficGenPort); /* Scheduling callback function */
    uint32_t                    queueIterator;      /* Number of frames per queue per loop */

} t_TrafficGen;

#endif /* __TRAFFIC_GENERATOR_H */

