/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mem_mng.h

 @Description   Internal definitions for memory allocation management module.
*//***************************************************************************/
#ifndef __MEM_MNG_H
#define __MEM_MNG_H


#include "std_ext.h"
#include "mem_mng_ext.h"
#include "list_ext.h"


#define __ERR_MODULE__  MODULE_UNKNOWN


typedef struct t_MemMngDebugEntry
{
    void        *p_Memory;
    uint32_t    size;
    char        *info;
    char        *filename;
    int         line;
    t_List      node;

} t_MemMngDebugEntry;

#define MEM_MNG_DBG_OBJECT(p_List)  \
    LIST_OBJECT(p_List, t_MemMngDebugEntry, node)


/**************************************************************************//**
 @Description   Memory management partition control structure
 *//***************************************************************************/
typedef struct t_MemMngPartition
{
    int                     id;             /**< Partition ID */
    t_Handle                h_MemManager;   /**< Memory manager handle */
    void *                  (*f_UserMalloc)(uint32_t size, uint32_t alignment);
                                            /**< Memory allocation routine */
    void                    (*f_UserFree)(void *p_Addr);
                                            /**< Memory deallocation routine */
    bool                    enableDebug;    /**< TRUE to track malloc/free operations */
    t_List                  memDebugList;   /**< List of allocation entries (for debug) */
    t_List                  node;
    t_MemMngPartitionInfo   info;           /**< Partition information */
    t_Handle                h_Spinlock;
} t_MemMngPartition;

#define MEM_MNG_PARTITION_OBJECT(p_List)  \
    LIST_OBJECT(p_List, t_MemMngPartition, node)


/**************************************************************************//**
 @Description   Memory management module internal parameters
 *//***************************************************************************/
typedef struct t_MemMng
{
    void *      (*f_Malloc)(uint32_t size);
                /**< Memory allocation routine (for internal structures) */
    void        (*f_Free)(void *p_Addr);
                /**< Memory deallocation routine (for internal structures) */
    t_List      memPartitionsList;
                /**< List of partition control structures */

    /* Variables related to early memory partition */
    void *      (*f_EarlyMalloc)(uint32_t size, uint32_t alignment);
                /**< Early allocation routine (before partitions are registered) */
    void        (*f_EarlyFree)(void *p_Addr);
                /**< Early deallocation routine (before partitions are registered) */
    t_List      earlyMemDebugList;
                /**< List of early memory allocation entries (for debug) */

    t_Handle    h_Spinlock;
} t_MemMng;



#endif /* __MEM_MNG_H */

