/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          shared_memory.h

 @Description   Prototypes, externals and typedefs for shared_memory resource
*//***************************************************************************/

#ifndef __SHARED_MEMORY_H
#define __SHARED_MEMORY_H

#include "std_ext.h"
#include "shared_memory_ext.h"
#include "list_ext.h"
#include "xx_ext.h"

#define __ERR_MODULE__  MODULE_UNKNOWN

#define AVAILABLE_MSGS_QUEUE   0
#define QUEUE_LOCK_FLAG        1
#define QUEUE_UNLOCK_FLAG      0

typedef _Packed struct
{
    volatile uint64_t   nextMsgOffset;
    volatile uint32_t   msgLength;
    volatile char       msgBody[1];     /* The actual size will be determined according
                                           to the input msgLength parameter */
} _PackedType t_Msg;


typedef _Packed struct
{
    volatile uint64_t   headOffset;     /* Offset to the first message in the queue's
                                           messages list */
    volatile uint64_t   tailOffset;
    volatile int        lock;
} _PackedType t_MsgQueue;


/* The memory map of the shared memory is built as the following:



   Shared memory base address:
    t_SharedMemoryMap: a structure that defines that shared memory map addresses


    msgQueueOffset:            t_MsgQueue    -|
                               t_MsgQueue     |
                               .              |
                               .              |-> numOfMessageQueues
                               .              |
                               t_MsgQueue    -|
                (message queues[0 - numOfMessageQueues])

    availableMsgsOffset:       t_Msg         -|
                               t_Msg          |
                               t_Msg          |
                               .              |-> numOfMessages
                               .              |
                               .              |
                               t_Msg         -|
                (available messages[0 - numOfMessages])

    Upon initialization msgQueue with index AVAILABLE_MSGS_QUEUE points to
    all messages list residing at availableMsgsOffset. Other queues are initialized
    to be empty.
*/

typedef _Packed struct
{
    volatile uint64_t         msgQueueOffset;           /* Offset to queues that hold the messages */
    volatile uint64_t         availableMsgsOffset;      /* Offset to pool of free message buffers to be used
                                                           for send/write */
    volatile uint32_t         totalMessageLength;       /* Size of message buffer based on msgBodyLength
                                                           provided in SHARED_MEMORY_EarlyInit params, plus
                                                           additional field required by t_Msg struct */
    volatile uint32_t         totalMessageQueueLength;  /* Size of t_MsgQueue struct + queue alignment
                                                           provided in SHARED_MEMORY_EarlyInit params */
} _PackedType t_SharedMemoryMap;

/* Shared memory internal structure */
typedef struct t_SharedMemory
{
    uint32_t                 id;               /**< Shared memory identifier */
    t_Handle                 h_App;            /**< Handle to the application */
    t_TxConfRoutine          *f_TxConf;        /**< Pointer to upper layer's routine - called to handle transmitted frames */
    t_RxStoreRoutine         *f_RxStore;       /**< Pointer to upper layer's routine - called to pass a received frame to the application */
    uint64_t                 memoryMapAddress; /**< The base address of the shared memory */
    uint32_t                 irq;              /**< Interrupt request number - relevant only if interrupts field is TRUE */
    bool                     interrupts;       /**< Interrupt/polling mode; TRUE for interrupt mode */
} t_SharedMemory;


#endif

