/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          cli.h

 @Description   Command-Line-Interpreter (CLI) routine implementation.
*//***************************************************************************/
#ifndef __CLI_H
#define __CLI_H

#include "std_ext.h"
#include "error_ext.h"
#include "list_ext.h"
#include "cli_ext.h"


#define __ERR_MODULE__      MODULE_UNKNOWN

#define CLI_PRINT_BUF_SIZE      1024
#define CLI_MAX_LINE_LENGTH     256
#define CLI_TIMER_INTERVAL      128 /* 128 m-secs */
#define CLI_MAX_HISTORY         10


typedef struct t_CliCommand
{
    char        *name;
    char        *desc;
    char        *format;
    t_Handle    h_CommArg;
    t_Error     (*f_DoCommand)(t_Handle h_CommArg, int argc, char *argv[]);
    t_List      node;
} t_CliCommand;
#define CLI_COMMAND_OBJ(ptr)  LIST_OBJECT(ptr, t_CliCommand, node)

typedef struct t_CliHistoryLine
{
    char    line[CLI_MAX_LINE_LENGTH+5];
    int     length;
    t_List  node;
} t_CliHistoryLine;
#define CLI_HISTORY_LINE_OBJ(ptr)  LIST_OBJECT(ptr, t_CliHistoryLine, node)

typedef struct t_Cli
{
    char        *prompt;
    uint32_t    attributes;
    void        (*f_Print)   (char *str, ...);
    char        (*f_GetChar) (void);
    t_Handle    h_Timer;
    t_Handle    h_CommandsLock;
    t_List      commands;
    char        line[CLI_MAX_LINE_LENGTH+5];
    int         currPos;
    t_List      history;
    bool        historyReq;
    t_List      *p_HistoryPos;
} t_Cli;


#endif /* __CLI_H */
