#!/bin/bash
# Copy the NetCommSW public headers into the public/ncsw folder so that
# the QNX build system can copy it into the INSTALL folder.
# This prevents having to check in copies of headers in the public folder


if [ -r $QNX_HOST/usr/bin/pwd.exe ]; then
	CPWD=`$QNX_HOST/usr/bin/pwd.exe`
else
	CPWD=`pwd`
fi

if [ -r $QNX_HOST/usr/bin/find.exe ]; then
	FIND="$QNX_HOST/usr/bin/find.exe"
else
	FIND=find
fi

if [ -r $QNX_HOST/usr/bin/xargs.exe ]; then
	XARGS="$QNX_HOST/usr/bin/xargs.exe"
else
	XARGS=xargs
fi


if [ "$1" == "clean" ]; then
	echo "Cleaning out old NetComm SW Public headers"
	$FIND public/ncsw -name '*.h' -type f -print -o -name .svn -prune | $XARGS rm -f
fi

echo "Copying NetComm SW Public headers"

CP_HOST=$QNX_HOST/usr/bin/qnx_cp 

PROJECT_ROOT=$CPWD
NETCOMM_ROOT=$PROJECT_ROOT/NetCommSw

function fn_copy_top_level()  {
	cd $1
	EXT_FILES=`$FIND . -name "*_ext.h"`
	for file in $EXT_FILES
	do
		case $1/$file in
			$NETCOMM_ROOT/inc/./integrations/P4080/*)
				$CP_HOST -cfp $file $PROJECT_ROOT/public/ncsw/$file
				;;
			$NETCOMM_ROOT/inc/./Peripherals/*)
				case `basename $file` in
					bm_*.h|dpa*_ext.h|fm_*ext.h|gpio_ext.h|io_port_ext.h|mii_acc_ext.h|qm_*ext.h|sec_*ext.h|serdes_ext.h)
						$CP_HOST -cfp $file $PROJECT_ROOT/public/ncsw/$file
						;;
				esac
				;;

			$NETCOMM_ROOT/inc/./*Ctl/*) ;; # exclude
			$NETCOMM_ROOT/inc/./cores/e300*_ext.h) ;; # exclude
			$NETCOMM_ROOT/inc/./cores/dec_ext.h) ;; # exclude
			$NETCOMM_ROOT/inc/./cores/e500v2_*_ext.h) ;; # exclude
			$NETCOMM_ROOT/inc/./integrations/*/*) ;; # exclude

			*)
				$CP_HOST -cfp $file $PROJECT_ROOT/public/ncsw/$file
				;;
		esac
	done
}

## Copy the contents of the inc folders 
FOLDERS="inc Peripherals/FM/inc user/inc user/env/qnx/inc user/hardware/inc"
for folder in $FOLDERS
do
fn_copy_top_level $NETCOMM_ROOT/$folder
done

##Copy other headers not copied above
#FMan firmware
$CP_HOST -cfpr $NETCOMM_ROOT/integrations/P4080/fman_ctrl_code $PROJECT_ROOT/public/ncsw
$CP_HOST -cfpr $NETCOMM_ROOT/inc/types_dflt.h $PROJECT_ROOT/public/ncsw
$CP_HOST -cfpr $NETCOMM_ROOT/user/env/qnx/inc/types_qnx.h $PROJECT_ROOT/public/ncsw/
$CP_HOST -cfpr $NETCOMM_ROOT/user/env/qnx/inc/config_qnx.h $PROJECT_ROOT/public/ncsw/
