/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          fm_muram_ext.h

 @Description   FM MURAM Application Programming Interface.
*//***************************************************************************/
#ifndef __FM_MURAM_EXT
#define __FM_MURAM_EXT

#include "error_ext.h"
#include "std_ext.h"


/**************************************************************************//**

 @Group         FM_grp Frame Manager API

 @Description   FM API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         FM_muram_grp FM MURAM

 @Description   FM MURAM API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         FM_muram_init_grp FM MURAM Initialization

 @Description   FM MURAM initialization API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      FM_MURAM_ConfigAndInit

 @Description   Creates partition in the MURAM.

                The routine returns a handle (descriptor) to the MURAM partition.
                This descriptor must be passed as first parameter to all other
                FM-MURAM function calls.

                No actual initialization or configuration of FM_MURAM hardware is
                done by this routine.

 @Param[in]     baseAddress - Pointer to base of memory mapped FM-MURAM.
 @Param[in]     size        - Size of the FM-MURAM partition.

 @Return        Handle to FM-MURAM object, or NULL for Failure.
*//***************************************************************************/
t_Handle FM_MURAM_ConfigAndInit(uint64_t baseAddress, uint32_t size);

/**************************************************************************//**
 @Function      FM_MURAM_Free

 @Description   Frees all resources that were assigned to FM-MURAM module.

                Calling this routine invalidates the descriptor.

 @Param[in]     h_FmMuram - FM-MURAM module descriptor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  FM_MURAM_Free(t_Handle h_FmMuram);

/** @} */ /* end of FM_muram_init_grp group */


/**************************************************************************//**
 @Group         FM_muram_ctrl_grp FM MURAM Control

 @Description   FM MURAM control API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      FM_MURAM_AllocMem

 @Description   Allocate some memory from FM-MURAM partition.

 @Param[in]     h_FmMuram - FM-MURAM module descriptor.
 @Param[in]     size      - size of the memory to be allocated.
 @Param[in]     align     - Alignment of the memory.

 @Return        address of the allocated memory; NULL otherwise.
*//***************************************************************************/
void  * FM_MURAM_AllocMem(t_Handle h_FmMuram, uint32_t size, uint32_t align);

/**************************************************************************//**
 @Function      FM_MURAM_FreeMem

 @Description   Free an allocated memory from FM-MURAM partition.

 @Param[in]     h_FmMuram - FM-MURAM module descriptor.
 @Param[in]     ptr       - A pointer to an allocated memory.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error FM_MURAM_FreeMem(t_Handle h_FmMuram, void *ptr);

/** @} */ /* end of FM_muram_ctrl_grp group */
/** @} */ /* end of FM_muram_grp group */
/** @} */ /* end of FM_grp group */



#endif /* __FM_MURAM_EXT */
