/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          ds_1022_ext.h

 @Description   Prototypes, externals and typedefs for DS1022 routines
*//***************************************************************************/
#ifndef __DS_1022_EXT_H
#define __DS_1022_EXT_H

#include "enet_ext.h"
#include "mii_mng_ext.h"
#include "sdhc_std_ext.h"


/**************************************************************************//**
 @Group         ds1022_id_grp

 @Description   DS1022 definitions and enums.

 @{
*//***************************************************************************/

#define DS1022_NUM_OF_TDMS         16
#define DS1022_NUM_OF_I2C_PORTS    2
#define DS1022_NUM_OF_SERIAL_PORTS 2
#define DS1022_NUM_OF_SATA_PORTS   2
#define DS1022_NUM_OF_SPI_PORTS    1


/**************************************************************************//**
 @Description   P1022DS Memory Regions
*//***************************************************************************/
typedef enum e_Ds1022MemRegion
{
    e_DS1022_MEM_RGN_DDR,
    e_DS1022_MEM_RGN_PCIE1,
    e_DS1022_MEM_RGN_PCIE2,
    e_DS1022_MEM_RGN_PCIE3,
    e_DS1022_MEM_RGN_PCIEIO,
    e_DS1022_MEM_RGN_CCSR,
    e_DS1022_MEM_RGN_PIXIS,
    e_DS1022_MEM_RGN_L2_SRAM,
    e_DS1022_MEM_RGN_NAND_FLASH,
    e_DS1022_MEM_RGN_NOR_FLASH,
    e_DS1022_MEM_RGN_PROMJET

} e_Ds1022MemRegion;

/**************************************************************************//**
 @Description   P1022DS Memory Regions
*//***************************************************************************/
typedef struct t_Ds1022MemRegionInfo
{
    uint32_t    startAddr;
    uint32_t    size;
    uint8_t     chipSelect;

} t_Ds1022MemRegionInfo;

/**************************************************************************//**
 @Description   Ethernet PHY selection
*//***************************************************************************/
typedef enum e_Ds1022EthPort
{
    e_DS1022_ETH_PORT_ETSEC_1 = 0,
    e_DS1022_ETH_PORT_ETSEC_2,
    e_DS2020_ETH_PORT_DUMMY_LAST

} e_Ds1022EthPort;

typedef enum e_Ds1022Led
{
   e_DS1022_LED_GREEN = 0,
   e_DS1022_LED_YELLOW,
   e_DS1022_LED_RED

} e_Ds1022Led;

/**************************************************************************//**
 @Description   Serial Port selection
*//***************************************************************************/
typedef enum e_Ds1022SerialPort
{
    e_DS1022_SERIAL_PORT_DUART_1 = 0,
    e_DS1022_SERIAL_PORT_DUART_2

} e_Ds1022SerialPort;

/**************************************************************************//**
 @Description   ESDHC Port selection
*//***************************************************************************/
typedef enum e_Ds1022EsdhcPort
{
    e_DS1022_ESDHC_PORT_1 = 0,
    e_DS1022_ESDHC_PORT_2

} e_Ds1022EsdhcPort;

/**************************************************************************//**
 @Description   SPI Port selection
*//***************************************************************************/

typedef enum e_Ds1022SpiPort
{
    e_DS1022_ESPI_PORT = 0

} e_Ds1022SpiPort;

/**************************************************************************//**
 @Description   Video encoder
*//***************************************************************************/

typedef enum e_Ds1022VideoEncoder
{
    e_DS1022_NO_ENCODER = 0,
    e_DS1022_DVI_ENCODER,
    e_DS1022_LVDS_ENCODER

} e_Ds1022VideoEncoder;

typedef struct t_diuDevInfo
{
    e_Ds1022VideoEncoder    videoEncoder;
    bool                    useDdcEeprom;
    bool                    backlightOn;

    uint32_t                norFlashBA;
    uint32_t                promJetBA;

} t_diuDevInfo;

t_Handle DS1022_ConfigAndInit(void);

t_Error DS1022_Free(t_Handle h_Ds1022);

t_Error DS1022_InitEthernetPhy(t_Handle             h_Ds1022,
                                e_Ds1022EthPort     ethPort,
                                t_Handle            h_MiiAccess,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                loopbackConnector,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg);

e_EnetMode DS1022_FreeEthernetPhy(t_Handle 			h_Ds1022,
								  e_Ds1022EthPort 	ethPort);

bool DS1022_GetEthernetLinkStatus(t_Handle              h_Ds1022,
                                   e_Ds1022EthPort      ethPort,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg);

t_Error DS1022_InitSerialPort(t_Handle h_Ds1022, e_Ds1022SerialPort serialPort);

t_Error DS1022_FreeSerialPort(t_Handle h_Ds1022, e_Ds1022SerialPort serialPortId);

t_Error DS1022_InitI2cPort(t_Handle h_Ds1022, uint8_t i2cPortId);

t_Error DS1022_FreeI2cPort(t_Handle h_Ds1022, uint8_t i2cPortId);

t_Error DS1022_InitSdPort(t_Handle 			h_Ds1022,
						  e_Ds1022EsdhcPort esdhcPort,
						  e_SdhcBusWidth 	busWidth);

t_Error DS1022_FreeSdPort(t_Handle 			h_Ds1022,
						  e_Ds1022EsdhcPort esdhcPort,
						  e_SdhcBusWidth 	busWidth);

/* TDM SUPPORT */
t_Error DS1022_InitTdmPort(t_Handle h_Ds1022);
t_Error DS1022_FreeTdmPort(t_Handle h_Ds1022);

t_Error DS1022_InitSpiPort(t_Handle h_Ds1022, e_Ds1022SpiPort spiPort);

t_Error DS1022_FreeSpiPort(t_Handle h_Ds1022, e_Ds1022SpiPort spiPort);

t_Error DS1022_InitDiu(t_Handle h_Ds1022, t_diuDevInfo *p_Info);

t_Error DS1022_FreeDiu(t_Handle h_Ds1022, t_diuDevInfo *p_Info);

t_Error DS1022_GetMemRegionInfo(e_Ds1022MemRegion     memRegion,
                                t_Ds1022MemRegionInfo *p_MemRegionInfo);


/* SSI SUPPORT */
t_Error DS1022_InitSsiPort(t_Handle h_Ds1022);

t_Error DS1022_SetSsiClk(t_Handle h_Ds1022, uint32_t ssiClock);

t_Error DS1022_FreeSsiPort(t_Handle h_Ds1022);


#endif /* __DS_1022_EXT_H */
