/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*************************************************************************************//**
@File          itg_common_ext.h

@Description   Traffic Generator / Traffic Analyzer common data structures and definitions
*//**************************************************************************************/

#ifndef __ITG_COMMON_EXT_H
#define __ITG_COMMON_EXT_H

#include "std_ext.h"
#include "part_ext.h"
#include "list_ext.h"
#include "buf_ext.h"

/**************************************************************************//**
 @Group         itg_package_grp Internal Traffic Generator

 @Description   Internal Traffic Generator routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         itg_common_grp Internal Traffic Generator Common Application Programming Interface

 @Description   ITG common defines, structures and functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection   TG/TA Defines

               General Traffic Generator/Traffic Analyzer defines

 @{
*//***************************************************************************/

#define MAX_SEQ_NUM_SIZE      4     /**< Frames's sequence number size in bytes */
#define GET_PORT_HANDLE(lst,portId)   (((t_TrafficPort *)ITG_GetPort(lst,(uint16_t)portId))->h_Port)
                                    /**< Get the port handle */
/** @} */

/**************************************************************************//**
 @Description   Traffic Port Structure

                This structure is used internally by TG/TA modules
                for TG/TA port managing.
*//***************************************************************************/
typedef struct t_TrafficPort
{
    t_Handle     h_Port;        /**< Port handle    */
    t_List       portsNode;     /**< Port node      */
} t_TrafficPort;

/**************************************************************************//**
 @Function      ITG_CreateTrafficPort

 @Description   Creates and initializes Traffic Port object.

                It is used internally by TG_PORT_Config() and
                TA_PORT_Config() during TG/TA port configuration.

 @RetVal        Pointer to Traffic Port: Success.
 @RetVal        NULL: Failure.

 @Cautions      None.
*//***************************************************************************/
t_TrafficPort* ITG_CreateTrafficPort(void);

/**************************************************************************//**
 @Function      ITG_AddPortToList

 @Description   Adds Traffic Port to the Port list.

                It is used internally by TG_PORT_Config() and
                TA_PORT_Config() during TG/TA port configuration.

 @Param[in]     p_TrafficPort  - Pointer to Traffic Port.
 @Param[in]     p_PortsLst     - Pointer to the Port list.

 @Return        None.

 @Cautions      Traffic Port must have been previously created using ITG_CreateTrafficPort().
*//***************************************************************************/
void ITG_AddPortToList(t_TrafficPort *p_TrafficPort, t_List *p_PortsLst);

/**************************************************************************//**
 @Function      ITG_RemovePortFromList

 @Description   Removes first Traffic Port from the list of TG/TA ports.

                It is used internally by TG_Free() and TA_Free()
                to free all TG/TA Ports one by one.

 @Param[in]     h_Port         - Handle to the Port.
 @Param[in]     p_PortsLst     - Pointer to Port list.

 @RetVal        Pointer to Traffic Port removed from the Port list: Success.
 @RetVal        NULL, if the list is empty: Failure.

 @Cautions      None.
*//***************************************************************************/
t_TrafficPort * ITG_RemovePortFromList(t_Handle h_Port, t_List *p_PortsLst);

/**************************************************************************//**
 @Function      ITG_GetPort

 @Description   Gets the pointer to the Traffic port based on Port identification number.

                It is used internally by TG/TA functions to get
                TG/TA port.

 @Param[in]     p_PortsLst     - Pointer to Port list of TG/TA.
 @Param[in]     nthPort        - Corresponding port sequential number.

 @Return        Pointer to specified Traffic Port from list.

 @Cautions      None.
*//***************************************************************************/
t_TrafficPort * ITG_GetPort(t_List *p_PortsLst, uint16_t nthPort);

/**************************************************************************//**
 @Function      ITG_RechainList

 @Description   Rechains the frames list such that the node pointed by p_FirstNode
                is the first element in the re-chained list.

                It is used internally by TG/TA functions to rechain the Tx/Rx list
                to aid in rerunning the test.

 @Param[in]     p_FramesLst    - Pointer to TG/TA frames list.
 @Param[in]     p_FirstNode    - Pointer to the first node in the TG/TA frames list.

 @Return        None.

 @Cautions      None.
*//***************************************************************************/
void ITG_RechainList(t_List *p_FramesLst, t_List *p_FirstNode);

/** @} */ /* end of itg_common_grp */

/** @} */ /* end of itg_package_grp */

#endif /* __ITG_COMMON_EXT_H */

