/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          MDS836x_ext.h

 @Description   Prototypes, externals and typedefs for MDS836x routines
*//***************************************************************************/
#ifndef __MDS_836x_EXT_H
#define __MDS_836x_EXT_H

#include "enet_ext.h"
#include "mii_mng_ext.h"
#include "pq_mds_t1_ext.h"
#include "utopia_pos_ext.h"
#include "carrier_ext.h"


/**************************************************************************//**
 @Group         mds836x_id

 @Description   MDS836x definitions and enums.

 @{
*//***************************************************************************/

#define MDS836x_NUM_OF_TDMS         8
#define MDS836x_NUM_OF_I2C_PORTS    2
#define MDS836x_NUM_OF_SERIAL_PORTS 2

/**************************************************************************//**
 @Description   MPC836xMDS Memory Regions
*//***************************************************************************/
typedef enum e_Mds836xMemRegion
{
    e_MDS836x_MEM_RGN_DDR1,
    e_MDS836x_MEM_RGN_DDR2,
    e_MDS836x_MEM_RGN_PCI,
    e_MDS836x_MEM_RGN_IMMR,
    e_MDS836x_MEM_RGN_SDRAM,
    e_MDS836x_MEM_RGN_BCSR,
    e_MDS836x_MEM_RGN_PIB_PMC0,
    e_MDS836x_MEM_RGN_PIB_PMC1,
    e_MDS836x_MEM_RGN_FLASH
} e_Mds836xMemRegion;


/**************************************************************************//**
 @Description   MPC836xMDS Memory Regions
*//***************************************************************************/
typedef struct t_Mds836xMemRegionInfo
{
    uint32_t    startAddr;
    uint32_t    size;
    uint8_t     chipSelect;

} t_Mds836xMemRegionInfo;


/**************************************************************************//**
 @Description   Ethernet PHY selection
*//***************************************************************************/
typedef enum e_Mds836xEthPort
{
    e_MDS836x_ETH_PORT_UCC_1 = 0,
    e_MDS836x_ETH_PORT_UCC_2,
    e_MDS836x_ETH_PORT_UCC_3,
    e_MDS836x_ETH_PORT_UCC_4,
    e_MDS836x_ETH_PORT_UCC_5,
    e_MDS836x_ETH_PORT_UCC_6,
    e_MDS836x_ETH_PORT_UCC_7,
    e_MDS836x_ETH_PORT_UCC_8

} e_Mds836xEthPort;


typedef enum e_Mds836xUtopiaPosMode
{
    e_MDS836x_UTOPIA_POS_NOT_SUPPORTED,
    e_MDS836x_UTOPIA_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS836x_UTOPIA_MASTER_SDEV_SPHY_16_BITS,
    e_MDS836x_UTOPIA_MASTER_SDEV_MPHY_16_BITS,
    e_MDS836x_UTOPIA_MASTER_MDEV_SPHY_16_BITS,
    e_MDS836x_UTOPIA_MASTER_MDEV_MPHY_16_BITS,
    e_MDS836x_POS_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS836x_POS_MASTER_SDEV_SPHY_16_BITS,
    e_MDS836x_POS_MASTER_SDEV_MPHY_16_BITS,
    e_MDS836x_POS_MASTER_MDEV_SPHY_16_BITS,
    e_MDS836x_POS_MASTER_MDEV_MPHY_16_BITS

} e_Mds836xUtopiaPosMode;


typedef enum e_Mds836xLed
{
   e_MDS836x_LED_GREEN,
   e_MDS836x_LED_RED

} e_Mds836xLed;


typedef enum e_Mds836xIoGroup
{
    e_MDS836x_IO_GRP_USB,
    e_MDS836x_IO_GRP_USB_SW,
    e_MDS836x_IO_GRP_TDM_A_SERIAL,
    e_MDS836x_IO_GRP_TDM_A_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_B_SERIAL,
    e_MDS836x_IO_GRP_TDM_B_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_C_SERIAL,
    e_MDS836x_IO_GRP_TDM_C_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_D_SERIAL,
    e_MDS836x_IO_GRP_TDM_D_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_E_SERIAL,
    e_MDS836x_IO_GRP_TDM_E_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_F_SERIAL,
    e_MDS836x_IO_GRP_TDM_F_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_G_SERIAL,
    e_MDS836x_IO_GRP_TDM_G_SERIAL_MPHY,
    e_MDS836x_IO_GRP_TDM_H_SERIAL,
    e_MDS836x_IO_GRP_TDM_H_SERIAL_MPHY,
    e_MDS836x_IO_GRP_UCC_1_GMII,
    e_MDS836x_IO_GRP_UCC_1_RGMII,
    e_MDS836x_IO_GRP_UCC_1_TBI,
    e_MDS836x_IO_GRP_UCC_1_RTBI,
    e_MDS836x_IO_GRP_UCC_1_MII,
    e_MDS836x_IO_GRP_UCC_1_RMII,
    e_MDS836x_IO_GRP_UCC_2_GMII,
    e_MDS836x_IO_GRP_UCC_2_RGMII,
    e_MDS836x_IO_GRP_UCC_2_TBI,
    e_MDS836x_IO_GRP_UCC_2_RTBI,
    e_MDS836x_IO_GRP_UCC_2_MII,
    e_MDS836x_IO_GRP_UCC_2_RMII,
    e_MDS836x_IO_GRP_UCC_3_RMII,
    e_MDS836x_IO_GRP_UCC_4_RMII,
    e_MDS836x_IO_GRP_UCC_5_RMII,
    e_MDS836x_IO_GRP_UCC_6_RMII,
    e_MDS836x_IO_GRP_UCC_7_RMII,
    e_MDS836x_IO_GRP_UCC_8_RMII,
    e_MDS836x_IO_GRP_UPC_1_UTOPIA_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_UTOPIA_MASTER_MDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_UTOPIA_MASTER_MDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_POS_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_POS_MASTER_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_POS_MASTER_SDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_POS_MASTER_MDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_1_POS_MASTER_MDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_UTOPIA_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_UTOPIA_MASTER_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_UTOPIA_MASTER_SDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_UTOPIA_MASTER_MDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_UTOPIA_MASTER_MDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_POS_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_POS_MASTER_SDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_POS_MASTER_SDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_POS_MASTER_MDEV_SPHY_16_BITS,
    e_MDS836x_IO_GRP_UPC_2_POS_MASTER_MDEV_MPHY_16_BITS,
    e_MDS836x_IO_GRP_SPI_1_MASTER,
    e_MDS836x_IO_GRP_SPI_1_SLAVE,
/* @@@@ check
    e_MDS836x_IO_GRP_SPI_2_MASTER,
    e_MDS836x_IO_GRP_SPI_2_SLAVE,
*/
    e_MDS836x_IO_GRP_MII_CFG_QE_MUX,
    e_MDS836x_IO_GRP_DUART_2,
    e_MDS836x_IO_GRP_I2C_2

} e_Mds836xIoGroup;


t_Handle MDS836x_ConfigAndInit(void);

t_Error MDS836x_Free(t_Handle h_Mds836x);

t_Error MDS836x_InitEthernetPhy(t_Handle            h_Mds836x,
                                e_Mds836xEthPort    ethPort,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                fiber,
                                bool                forceMaster,
                                bool                loopbackConnector,
                                t_Handle            h_MiiAccess,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg);

e_EnetMode MDS836x_FreeEthernetPhy(t_Handle h_Mds836x, e_Mds836xEthPort ethPort);

bool MDS836x_GetEthernetLinkStatus(t_Handle             h_Mds836x,
                                   e_Mds836xEthPort     ethPort,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg);

t_Error MDS836x_SetUtopiaPos(t_Handle               h_Mds836x,
                             uint8_t                pmcId,
                             bool                   reset,
                             e_Mds836xUtopiaPosMode mdsUtopiaPosMode);

bool MDS836x_PqMdsQoc3IsConnected(t_Handle h_Mds836x,
                                  uint32_t baseAddr);

bool MDS836x_PqMdsQoc12IsConnected(t_Handle h_Mds836x,
                                   uint32_t baseAddr);

bool MDS836x_PqMds8E1T1IsConnected(t_Handle h_Mds836x,
                                   uint32_t baseAddr);

t_Error MDS836x_InitPqMdsQoc3(t_Handle              h_Mds836x,
                              uint32_t              baseAddr,
                              uint8_t               numOfDevs,
                              uint8_t               numOfPhys,
                              e_LineRate            lineRate,
                              e_UtopiaPosProtocol   protocol,
                              e_UtopiaPosDevMode    allDevsOpMode,
                              e_UtopiaPosBusWidth   allDevsBusWidth);

t_Error MDS836x_InitPqMdsQoc12(t_Handle             h_Mds836x,
                               uint32_t             baseAddr,
                               uint8_t              numOfDevs,
                               uint8_t              numOfPhys,
                               e_LineRate           lineRate,
                               e_UtopiaPosProtocol  protocol,
                               e_UtopiaPosDevMode   allDevsOpMode,
                               e_UtopiaPosBusWidth  allDevsBusWidth);

t_Error MDS836x_FreePqMdsQoc3(t_Handle h_Mds836x,
                              uint32_t baseAddr);

t_Error MDS836x_FreePqMdsQoc12(t_Handle h_Mds836x,
                               uint32_t baseAddr);

t_Error MDS836x_InitPqMdsQoc3Phy(t_Handle               h_Mds836x,
                                 uint32_t               baseAddr,
                                 uint8_t                phyId,
                                 uint8_t                phyAddr,
                                 e_DiagMode             diagMode,
                                 e_UtopiaPosProtocol    protocol,
                                 e_UtopiaPosDevMode     allDevsOpMode,
                                 e_UtopiaPosBusWidth    allDevsBusWidth);

t_Error MDS836x_InitPqMdsQoc12Phy(t_Handle              h_Mds836x,
                                  uint32_t              baseAddr,
                                  uint8_t               phyId,
                                  uint8_t               phyAddr,
                                  e_DiagMode            diagMode,
                                  e_UtopiaPosProtocol   protocol,
                                  e_UtopiaPosDevMode    allDevsOpMode,
                                  e_UtopiaPosBusWidth   allDevsBusWidth);

t_Error MDS836x_FreePqMdsQoc3Phy(t_Handle h_Mds836x,
                                 uint32_t baseAddr,
                                 uint8_t  phyIdOnQuad);

t_Error MDS836x_FreePqMdsQoc12Phy(t_Handle h_Mds836x,
                                  uint32_t baseAddr,
                                  uint8_t  phyIdOnQuad);


t_Error MDS836x_InitSerialPort(t_Handle h_Mds836x, uint8_t serialPortId);

t_Error MDS836x_FreeSerialPort(t_Handle h_Mds836x, uint8_t serialPortId);


t_Error MDS836x_InitI2cPort(t_Handle h_Mds836x, uint8_t i2cPortId);

t_Error MDS836x_FreeI2cPort(t_Handle h_Mds836x, uint8_t i2cPortId);


t_Error MDS836x_InitTdmPhy(t_Handle             h_Mds836x,
                           uint8_t              phyId,
                           e_LineRate           lineRate,
                           e_LineEncoding       lineEncoding,
                           e_TdmFramingMode     framingMode,
                           e_DiagMode           diagMode,
                           int                  irq,
                           t_PqMdsT1EvCallback  f_Isr,
                           t_Handle             h_IsrArg);

t_Error MDS836x_EnableTdmPhyIntr(t_Handle   h_Mds836x,
                                 uint8_t    phyId);

t_Error MDS836x_FreeTdmPhy(t_Handle h_Mds836x, uint8_t phyId);

t_Error MDS836x_InitTdmMultiPhy(t_Handle            h_Mds836x,
                                uint8_t             pmcId,
                                uint8_t             numOfTdms,
                                uint8_t             numOfPhys,
                                e_LineRate          lineRate,
                                e_LineEncoding      lineEncoding,
                                e_TdmFramingMode    framingMode,
                                e_DiagMode          diagMode,
                                int                 irq,
                                t_PqMdsT1EvCallback f_Isr,
                                t_Handle            h_IsrArg);

t_Error MDS836x_FreeTdmMultiPhy(t_Handle h_Mds836x, uint8_t pmcId);


t_Error MDS836x_SetLed(t_Handle h_Mds836x, e_Mds836xLed led, bool ledOn);


t_Error MDS836x_GetIoGroupPins(t_Handle         h_Mds836x,
                               e_Mds836xIoGroup ioGroup,
                               int              **p_Pins,
                               uint16_t         *p_NumOfPins);

t_Error MDS836x_GetIoGroupInputClockPins(t_Handle           h_Mds836x,
                                         e_Mds836xIoGroup   ioGroup,
                                         int                *p_RxClkPin,
                                         int                *p_TxClkPin);

t_Error MDS836x_GetMemRegionInfo(e_Mds836xMemRegion     memRegion,
                                 t_Mds836xMemRegionInfo *p_MemRegionInfo);

t_Error MDS836x_InitSpiPort(t_Handle h_Mds836x, uint8_t spiPortId);
t_Error MDS836x_FreeSpiPort(t_Handle h_Mds836x, uint8_t spiPortId);

t_Error MDS836x_InitPciPort(t_Handle h_Mds836x, uint8_t pciPortId);
t_Error MDS836x_FreePciPort(t_Handle h_Mds836x, uint8_t pciPortId);

#endif /* __MDS_836x_EXT_H */
