/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          mds_8569_ext.h

 @Description   Prototypes, externals and typedefs for MDS8569 routines
*//***************************************************************************/
#ifndef __MDS_8569_EXT_H
#define __MDS_8569_EXT_H

#include "enet_ext.h"
#include "mii_mng_ext.h"
#include "pq_mds_t1_ext.h"
#include "utopia_pos_ext.h"
#include "carrier_ext.h"
#include "sdhc_std_ext.h"


/**************************************************************************//**
 @Group         mds8569_id

 @Description   MDS8569 definitions and enums.

 @{
*//***************************************************************************/

#define MDS8569_NUM_OF_TDMS         16
#define MDS8569_NUM_OF_I2C_PORTS    2
#define MDS8569_NUM_OF_SERIAL_PORTS 3

/**************************************************************************//**
 @Description   MPC8569MDS Memory Regions
*//***************************************************************************/
typedef enum e_Mds8569MemRegion
{
    e_MDS8569_MEM_RGN_DDR1,
    e_MDS8569_MEM_RGN_DDR2,
    e_MDS8569_MEM_RGN_PCIE,
    e_MDS8569_MEM_RGN_SRIO1,
    e_MDS8569_MEM_RGN_SRIO2,
    e_MDS8569_MEM_RGN_CCSR,
    e_MDS8569_MEM_RGN_L2_SRAM,
    e_MDS8569_MEM_RGN_BCSR,
    e_MDS8569_MEM_RGN_PIB_PMC0,
    e_MDS8569_MEM_RGN_PIB_PMC1,
    e_MDS8569_MEM_RGN_FLASH,
    e_MDS8569_MEM_RGN_NAND_FLASH

} e_Mds8569MemRegion;


/**************************************************************************//**
 @Description   MPC8569MDS Memory Regions
*//***************************************************************************/
typedef struct t_Mds8569MemRegionInfo
{
    uint32_t    startAddr;
    uint32_t    size;
    uint8_t     chipSelect;

} t_Mds8569MemRegionInfo;


typedef enum e_Mds8569UtopiaPosMode
{
    e_MDS8569_UTOPIA_POS_NOT_SUPPORTED,
    e_MDS8569_UTOPIA_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS8569_UTOPIA_MASTER_SDEV_SPHY_16_BITS,
    e_MDS8569_UTOPIA_MASTER_SDEV_MPHY_16_BITS,
    e_MDS8569_UTOPIA_MASTER_MDEV_SPHY_16_BITS,
    e_MDS8569_UTOPIA_MASTER_MDEV_MPHY_16_BITS,
    e_MDS8569_UTOPIA_SLAVE_SDEV_SPHY_8_BITS,
    e_MDS8569_UTOPIA_MASTER_SDEV_SPHY_8_BITS,
    e_MDS8569_UTOPIA_MASTER_SDEV_MPHY_8_BITS,
    e_MDS8569_UTOPIA_MASTER_MDEV_SPHY_8_BITS,
    e_MDS8569_UTOPIA_MASTER_MDEV_MPHY_8_BITS,
    e_MDS8569_POS_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS8569_POS_MASTER_SDEV_SPHY_16_BITS,
    e_MDS8569_POS_MASTER_SDEV_MPHY_16_BITS,
    e_MDS8569_POS_MASTER_MDEV_SPHY_16_BITS,
    e_MDS8569_POS_MASTER_MDEV_MPHY_16_BITS

} e_Mds8569UtopiaPosMode;

typedef enum e_Mds8569Led
{
   e_MDS8569_LED_GREEN = 0,
   e_MDS8569_LED_YELLOW,
   e_MDS8569_LED_RED

} e_Mds8569Led;

typedef enum e_Mds8569SerialPort
{
    e_MDS8569_SERIAL_PORT_DUART_1 = 0,
    e_MDS8569_SERIAL_PORT_DUART_2,
    e_MDS8569_SERIAL_PORT_UCC_UART_2

} e_Mds8569SerialPort;

typedef enum e_Mds8569UsbMode
{
    e_MDS8569_USB_FS_DEVICE,
    e_MDS8569_USB_LS_DEVICE,
    e_MDS8569_USB_HOST,
    e_MDS8569_USB_DISCONNECT,
    e_MDS8569_USB_POWER_ON,
    e_MDS8569_USB_POWER_OFF,
    e_MDS8569_USB_NO_MODE

} e_Mds8569UsbMode;

typedef enum e_Mds8569IoGroup
{
    e_MDS8569_IO_GRP_TDM1_A_SERIAL,
    e_MDS8569_IO_GRP_TDM1_B_SERIAL,
    e_MDS8569_IO_GRP_TDM1_C_SERIAL,
    e_MDS8569_IO_GRP_TDM1_C_SERIAL_DS3,
    e_MDS8569_IO_GRP_TDM1_D_SERIAL,
    e_MDS8569_IO_GRP_TDM1_E_SERIAL,
    e_MDS8569_IO_GRP_TDM1_F_SERIAL,
    e_MDS8569_IO_GRP_TDM1_F_SERIAL_DS3,
    e_MDS8569_IO_GRP_TDM1_G_SERIAL,
    e_MDS8569_IO_GRP_TDM1_H_SERIAL,
    e_MDS8569_IO_GRP_TDM2_A_SERIAL,
    e_MDS8569_IO_GRP_TDM2_B_SERIAL,
    e_MDS8569_IO_GRP_TDM2_B_SERIAL_DS3,
    e_MDS8569_IO_GRP_TDM2_C_SERIAL,
    e_MDS8569_IO_GRP_TDM2_D_SERIAL,
    e_MDS8569_IO_GRP_TDM2_E_SERIAL,
    e_MDS8569_IO_GRP_TDM2_F_SERIAL,
    e_MDS8569_IO_GRP_TDM2_F_SERIAL_DS3,
    e_MDS8569_IO_GRP_TDM2_G_SERIAL,
    e_MDS8569_IO_GRP_TDM2_H_SERIAL,
    e_MDS8569_IO_GRP_UCC_1_RGMII,
    e_MDS8569_IO_GRP_UCC_2_RGMII,
    e_MDS8569_IO_GRP_UCC_3_RGMII,
    e_MDS8569_IO_GRP_UCC_4_RGMII,
    e_MDS8569_IO_GRP_UCC_1_RTBI,
    e_MDS8569_IO_GRP_UCC_2_RTBI,
    e_MDS8569_IO_GRP_UCC_3_RTBI,
    e_MDS8569_IO_GRP_UCC_4_RTBI,
    e_MDS8569_IO_GRP_UCC_1_RMII,
    e_MDS8569_IO_GRP_UCC_2_RMII,
    e_MDS8569_IO_GRP_UCC_3_RMII,
    e_MDS8569_IO_GRP_UCC_4_RMII,
    e_MDS8569_IO_GRP_UCC_5_RMII,
    e_MDS8569_IO_GRP_UCC_6_RMII,
    e_MDS8569_IO_GRP_UCC_7_RMII,
    e_MDS8569_IO_GRP_UCC_8_RMII,
    e_MDS8569_IO_GRP_UCC_6_SMII,
    e_MDS8569_IO_GRP_UCC_8_SMII,
    e_MDS8569_IO_GRP_USB,
    e_MDS8569_IO_GRP_SPI_2_MASTER,
    e_MDS8569_IO_GRP_SPI_1_MASTER,
    e_MDS8569_IO_GRP_DUART_2,
    e_MDS8569_IO_GRP_UCC_2_UART,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_SPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_MPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_MDEV_SPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_MDEV_MPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_SLAVE_SDEV_SPHY_8_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_SPHY_8_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_SDEV_MPHY_8_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_MDEV_SPHY_8_BITS,
    e_MDS8569_IO_GRP_UPC_1_UTOPIA_MASTER_MDEV_MPHY_8_BITS,
    e_MDS8569_IO_GRP_UPC_1_POS_SLAVE_SDEV_SPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_POS_MASTER_SDEV_SPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_POS_MASTER_SDEV_MPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_POS_MASTER_MDEV_SPHY_16_BITS,
    e_MDS8569_IO_GRP_UPC_1_POS_MASTER_MDEV_MPHY_16_BITS,
    e_MDS8569_IO_GRP_MII_CFG_QE_MUX,
/* @@@@ check
    e_MDS8569_IO_GRP_SPI_1_SLAVE,
    e_MDS8569_IO_GRP_SPI_2_SLAVE,
*/
    e_MDS8569_IO_GRP_I2C_2

} e_Mds8569IoGroup;

t_Handle MDS8569_ConfigAndInit(void);

t_Error MDS8569_Free(t_Handle h_Mds8569);

t_Error MDS8569_InitEthernetPhy(t_Handle            h_Mds8569,
                                uint32_t            ethPortId,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                fiber,
                                bool                forceMaster,
                                bool                loopbackConnector,
                                t_Handle            h_MiiAccess,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg);

e_EnetMode MDS8569_FreeEthernetPhy(t_Handle h_Mds8569, uint32_t ethPort);

bool MDS8569_GetEthernetLinkStatus(t_Handle             h_Mds8569,
                                   uint32_t             ethPortId,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg);

t_Error MDS8569_ResetRmiiPhy(t_Handle h_Mds8569);

t_Error MDS8569_SetUtopiaPos(t_Handle               h_Mds8569,
                             uint8_t                pmcId,
                             bool                   reset,
                             e_Mds8569UtopiaPosMode mdsUtopiaPosMode);

bool MDS8569_PqMdsQoc3IsConnected(t_Handle h_Mds8569,
                                  uint32_t baseAddr);

bool MDS8569_PqMdsQoc12IsConnected(t_Handle h_Mds8569,
                                   uint32_t baseAddr);

bool MDS8569_PqMds16E1T1IsConnected(t_Handle h_Mds8569,
                                    uint32_t baseAddr);

t_Error MDS8569_InitPqMdsQoc3(t_Handle              h_Mds8569,
                              uint32_t              baseAddr,
                              uint8_t               numOfDevs,
                              uint8_t               numOfPhys,
                              e_LineRate            lineRate,
                              e_UtopiaPosProtocol   protocol,
                              e_UtopiaPosDevMode    allDevsOpMode,
                              e_UtopiaPosBusWidth   allDevsBusWidth);

t_Error MDS8569_InitPqMdsQoc12(t_Handle             h_Mds8569,
                               uint32_t             baseAddr,
                               uint8_t              numOfDevs,
                               uint8_t              numOfPhys,
                               e_LineRate           lineRate,
                               e_UtopiaPosProtocol  protocol,
                               e_UtopiaPosDevMode   allDevsOpMode,
                               e_UtopiaPosBusWidth  allDevsBusWidth);

t_Error MDS8569_FreePqMdsQoc3(t_Handle h_Mds8569,
                              uint32_t baseAddr);

t_Error MDS8569_FreePqMdsQoc12(t_Handle h_Mds8569,
                               uint32_t baseAddr);

t_Error MDS8569_InitPqMdsQoc3Phy(t_Handle               h_Mds8569,
                                 uint32_t               baseAddr,
                                 uint8_t                phyId,
                                 uint8_t                phyAddr,
                                 e_DiagMode             diagMode,
                                 e_UtopiaPosProtocol    protocol,
                                 e_UtopiaPosDevMode     allDevsOpMode,
                                 e_UtopiaPosBusWidth    allDevsBusWidth);

t_Error MDS8569_InitPqMdsQoc12Phy(t_Handle              h_Mds8569,
                                  uint32_t              baseAddr,
                                  uint8_t               phyId,
                                  uint8_t               phyAddr,
                                  e_DiagMode            diagMode,
                                  e_UtopiaPosProtocol   protocol,
                                  e_UtopiaPosDevMode    allDevsOpMode,
                                  e_UtopiaPosBusWidth   allDevsBusWidth);

t_Error MDS8569_FreePqMdsQoc3Phy(t_Handle h_Mds8569,
                                 uint32_t baseAddr,
                                 uint8_t  phyIdOnQuad);

t_Error MDS8569_FreePqMdsQoc12Phy(t_Handle h_Mds8569,
                                  uint32_t baseAddr,
                                  uint8_t  phyIdOnQuad);


t_Error MDS8569_InitSerialPort(t_Handle h_Mds8569, e_Mds8569SerialPort serialPort);

t_Error MDS8569_FreeSerialPort(t_Handle h_Mds8569, e_Mds8569SerialPort serialPortId);

t_Error MDS8569_InitSdPort(t_Handle h_Mds8569, e_SdhcBusWidth busWidth);

t_Error MDS8569_FreeSdPort(t_Handle h_Mds8569);

t_Error MDS8569_InitI2cPort(t_Handle h_Mds8569, uint8_t i2cPortId);

t_Error MDS8569_FreeI2cPort(t_Handle h_Mds8569, uint8_t i2cPortId);

t_Error MDS8569_InitUsbPort(t_Handle h_Mds8569, e_Mds8569UsbMode usbMode);
t_Error MDS8569_FreeUsbPort(t_Handle h_Mds8569);


t_Error MDS8569_InitTdmPhy(t_Handle             h_Mds8569,
                           uint8_t              phyId,
                           e_LineRate           lineRate,
                           e_LineEncoding       lineEncoding,
                           e_TdmFramingMode     framingMode,
                           e_DiagMode           diagMode,
                           int                  irq,
                           t_PqMdsT1EvCallback  f_Isr,
                           t_Handle             h_IsrArg);

t_Error MDS8569_EnableTdmPhyIntr(t_Handle h_Mds8569, uint8_t phyId);

t_Error MDS8569_FreeTdmPhy(t_Handle     h_Mds8569,
                           uint8_t      phyId,
                           e_LineRate   lineRate);

t_Error MDS8569_SetLed(t_Handle h_Mds8569, e_Mds8569Led led, bool ledOn);


t_Error MDS8569_GetIoGroupPins(t_Handle         h_Mds8569,
                               e_Mds8569IoGroup ioGroup,
                               int              **p_Pins,
                               uint16_t         *p_NumOfPins);

t_Error MDS8569_GetIoGroupInputClockPins(t_Handle           h_Mds8569,
                                         e_Mds8569IoGroup   ioGroup,
                                         int                *p_RxClkPin,
                                         int                *p_TxClkPin);

t_Error MDS8569_GetMemRegionInfo(e_Mds8569MemRegion     memRegion,
                                 t_Mds8569MemRegionInfo *p_MemRegionInfo);



#endif /* __MDS_8569_EXT_H */
