/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          p4080ds_ext.h

 @Description   Prototypes, externals and typedefs for P4080-DS routines
*//***************************************************************************/
#ifndef __P4080DS_EXT_H
#define __P4080DS_EXT_H

#include "enet_ext.h"
#include "mii_mng_ext.h"


/**************************************************************************//**
 @Group         p4080ds_id

 @Description   P4080DS definitions and enums.

 @{
*//***************************************************************************/

#define P4080DS_NUM_OF_SERIAL_PORTS     4


typedef enum e_P4080DsSerialPort {
    e_P4080DS_SERIAL_PORT_DUART_1 = 0,
    e_P4080DS_SERIAL_PORT_DUART_2
} e_P4080DsSerialPort;

typedef enum e_P4080DsMemCtrl {
    e_P4080DS_MEM_NONE = 0,
    e_P4080DS_MEM_FLASH,
    e_P4080DS_MEM_PROMJET,
    e_P4080DS_MEM_NGPIXIS
} e_P4080DsMemCtrl;

typedef struct t_P4080DsParams {
    uint64_t    pixisBaseAddr;
} t_P4080DsParams;


t_Handle P4080DS_ConfigAndInit(t_P4080DsParams *p_P4080DsParams);

t_Error P4080DS_Free(t_Handle h_P4080Ds);

t_Error P4080DS_GetCSForMemCtrl(t_Handle            h_P4080Ds,
                                uint8_t             id,
                                uint8_t             *p_Cs);

t_Error P4080DS_InitEthernetPhy(t_Handle            h_P4080Ds,
                                uint32_t            ethPortId,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                forceMaster,
                                e_DiagMode          loopbackMode,
                                t_Handle            h_MiiAccess,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg);

e_EnetMode P4080DS_FreeEthernetPhy(t_Handle h_P4080Ds, uint32_t ethPortId);

bool P4080DS_GetEthernetLinkStatus(t_Handle             h_P4080Ds,
                                   uint32_t             ethPortId,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg);

t_Error P4080DS_InitSerialPort(t_Handle h_P4080Ds, e_P4080DsSerialPort  serialPort);

t_Error P4080DS_FreeSerialPort(t_Handle h_P4080Ds, e_P4080DsSerialPort  serialPortId);

e_EnetMode P4080DS_GetEnetMode(t_Handle h_P4080Ds, uint32_t ethPortId);

bool P4080DS_IsValidSerDesConfiguration (uint8_t val);

uint32_t P4080DS_GetSysClk(t_Handle h_P4080Ds);


#endif /* __P4080DS_EXT_H */
