/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PCI_STD_EXT_H
#define __PCI_STD_EXT_H

/**************************************************************************//**
 @Description   PCI external IRQ lines
*//***************************************************************************/
typedef enum e_PciIrq
{
    e_PCI_IRQ_A  = 0x00,    /**< IRQ A */
    e_PCI_IRQ_B,            /**< IRQ B */
    e_PCI_IRQ_C,            /**< IRQ C */
    e_PCI_IRQ_D             /**< IRQ D */
} e_PciIrq;


/**************************************************************************//**
 @Description   PCI configuration space registers offset
*//***************************************************************************/
#define    PCI_VENDOR_ID                    0x00  /**< Vendor ID                     */
#define    PCI_DEV_ID                       0x02  /**< Device ID                     */
#define    PCI_COMMAND                      0x04  /**< PCI Bus Command               */
#define    PCI_STATUS                       0x06  /**< PCI Bus Status                */
#define    PCI_REV_ID                       0x08  /**< Revision ID                   */
#define    PCI_STD_PROG                     0x09  /**< Bus Programming Interface     */
#define    PCI_SUB_CLASS_CODE               0x0A  /**< Subclass Code                 */
#define    PCI_BASE_CLASS_CODE              0x0B  /**< Bus Base Class Code           */
#define    PCI_CACHE_LINE                   0x0C  /**< Bus Cache Line Size           */
#define    PCI_LATENCY_TIMER                0x0D  /**< Bus Latency Timer             */
#define    PCI_HEADER_TYPE                  0x0E  /**< Header Type                   */
#define    PCI_BIST_CONTROL                 0x0F  /**< BIST Control                  */
#define    PCI_BASE_ADDRESS_0               0x10  /**< PCI Configuration and Status Register Base Address Register(PCSRBAR) */
#define    PCI_BASE_ADDRESS_1               0x14  /**< 32-Bit Memory Base Address Register       */
#define    PCI_BASE_ADDRESS_2               0x18  /**< 64-Bit Low Memory Base Address Register   */
#define    PCI_BASE_ADDRESS_3               0x1C  /**< 64-Bit High Memory Base Address Register  */
#define    PCI_SECONDARY_STATUS             0x1E  /**< Secondary status register                 */
#define    PCI_BASE_ADDRESS_4               0x20  /**< 64-Bit Low Memory Base Address Register   */
#define    PCI_BASE_ADDRESS_5               0x24  /**< 64-Bit High Memory Base Address Register  */
#define    PCI_SUBSYSTEM_VENDOR_ID          0x2C  /**< Subsystem Vendor ID                       */
#define    PCI_SUBSYSTEM_ID                 0x2E  /**< Subsystem ID                              */
#define    PCI_CAPABILITY_POINTER           0x34  /**< PCI Bus Capability Pointer                */
#define    PCI_INTERRUPT_LINE               0x3C  /**< PCI Bus Interrupt Line                    */
#define    PCI_INTERRUPT_PIN                0x3D  /**< PCI Bus Interrupt Pin                     */
#define    PCI_MIN_GNT                      0x3E  /**< PCI Bus MIN_GNT                           */
#define    PCI_MAX_LAT                      0x3F  /**< PCI Bus MAX_LAT                           */
#define    PCI_FUNCTION                     0x44  /**< PCI Bus Function                          */
#define    PCI_ARBITER_CONTROL              0x46  /**< PCI Bus Arbiter Configuration             */
#define    PCI_CAPABILITY                   0x4C  /**< PCI Express configuration                 */
#define    PCI_DEVICE_CAPABILITY            0x50  /**< PCI Express Device Capabilities           */
#define    PCI_DEVICE_CONTROL               0x54  /**< PCI Express Device Control                */
#define    PCI_DEVICE_STATUS                0x56  /**< PCI Express Device Status                 */
#define    PCI_LINK_CAPABILITY              0x58  /**< PCI Express Device Capabilities           */
#define    PCI_LINK_CONTROL                 0x5C  /**< PCI Express Device Control                */
#define    PCI_LINK_STATUS                  0x5E  /**< PCI Express Device Status                 */
#define    PCI_SLOT_CAPABILITY              0x60  /**< PCI Express Device Capabilities           */
#define    PCI_SLOT_CONTROL                 0x64  /**< PCI Express Device Control                */
#define    PCI_SLOT_STATUS                  0x66  /**< PCI Express Device Status                 */
#define    PCI_ROOT_STATUS                  0x6C  /**< PCI Express Root Status                   */

/**************************************************************************//**
 @Description   PCI MSI Registers
*//***************************************************************************/
#define MSI_MESSAGE_CONTROL_OFFSET          0               /**< MSI message control offset             */
#define MSI_MULT_MSG_ENABLE_SHIFT           20              /**< MSI multuple message enable offset     */
#define MSI_EN_MASK                         0x00010000      /**< MSI enable mask                        */
#define MSI_MESSAGE_ADDRESS_OFFSET          0x4             /**< MSI message address offset             */
#define MSI_64_BIT_ADDREES_MASK             0x00800000      /**< 64 bit address mask                    */
#define MSI_MESSAGE_UPPER_ADDRESS_OFFSET    0x8             /**< MSI message upper address offset       */
#define MSI_MESSAGE_DATA_OFFSET             0x8             /**< MSI message data offset                */
#define MSI_MULT_MSG_CAPABLE_MASK           0x000E0000      /**< MSI multiple message capability mask   */
#define MSI_MULT_MSG_CAPABLE_SHIFT          17              /**< MSI multiple message capability offset */
#define MSI_MULT_MSG_ENABLE_MASK            0x00700000      /**< MSI multiple message enable mask       */
#define MSI_MESSAGE_64_BIT_DATA_OFFSET      0xc             /**< MSI message 64 bit data offset         */


/**************************************************************************//**
 @Description   Write transaction type for PCI windows.
*//***************************************************************************/
typedef enum e_PciInboundWriteTransType
{
    e_PCI_INBOUND_WTT_WRITE_NO_SNOOP          = 0x4,  /**<  Write, dont snoop local processor      */
    e_PCI_INBOUND_WTT_WRITE_AND_SNOOP         = 0x5,  /**<  Write, snoop local processor            */
    e_PCI_INBOUND_WTT_WRITE_AND_ALLOC_L2      = 0x6,  /**<  Write, allocate L2 cache line           */
    e_PCI_INBOUND_WTT_WRITE_ALLOC_AND_LOCK_L2 = 0x7   /**<  Write, allocate and lock L2 cache line  */
} e_PciInboundWriteTransType;

/**************************************************************************//**
 @Description   Read transaction type for PCI windows.
*//***************************************************************************/
typedef enum e_PciInboundReadTransType
{
    e_PCI_INBOUND_RTT_READ_NO_SNOOP    = 0x4,  /**<  Do not snoop at read transaction   */
    e_PCI_INBOUND_RTT_READ_AND_SNOOP   = 0x5,  /**<  Snoop at read transaction          */
    e_PCI_INBOUND_RTT_READ_AND_LOCK_L2 = 0x7   /**<  Read, unlock L2 cache line         */
} e_PciInboundReadTransType;

/**************************************************************************//**
 @Description   Determine the required action upon an error interrupt.
*//***************************************************************************/
typedef enum e_PciErrorPolicy
{
    e_PCI_ERROR_DISABLE,        /**<  Disable the interrupts   */
    e_PCI_ERROR_INTERRUPT       /**<  Enable the interrupts    */
} e_PciErrorPolicy;

/**************************************************************************//**
 @Description   PCI Power states
*//***************************************************************************/
typedef enum e_PciPowerState
{
    e_PCI_POWER_STATE_D0     = 0x0,  /**< D0 - Full operation state     */
    e_PCI_POWER_STATE_D1     = 0x1,  /**< D1 - Doze mode                */
    e_PCI_POWER_STATE_D2     = 0x2,  /**< D2 - Nap mode                 */
    e_PCI_POWER_STATE_D3     = 0x3   /**< D3 - Sleep mode               */
} e_PciPowerState;

/**************************************************************************//**
 @Description   PCI Capabilities
*//***************************************************************************/
typedef enum e_PciCapability
{
    e_PCI_CAPABILITY_POWER_MANAGEMENT   = 0x01,     /**<  PCI Power Management capability   */
    e_PCI_CAPABILITY_MSI                = 0x05,     /**<  PCI MSI capability                */
    e_PCI_CAPABILITY_PCIE               = 0x10      /**<  PCI Express capability            */
} e_PciCapability;


#endif /* __PCI_STD_EXT_H */
