/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __SPI_STD_EXT_H
#define __SPI_STD_EXT_H



/**************************************************************************//**
 @Description   SPI Clock Polarity

                The clock polarity (CPOL) and phase (CPHA) pair is sometimes
                described as the "SPI mode", where mode is numbered as 0-3,
                according to the following options:
                - Mode-0: CPOL=0, CPHA=0
                - Mode-1: CPOL=0, CPHA=1
                - Mode-2: CPOL=1, CPHA=0
                - Mode-3: CPOL=1, CPHA=1

*//***************************************************************************/
typedef enum e_SpiClockPolarity
{
    e_SPI_CLK_POLARITY_ACTIVE_HIGH = 0, /**< SPI_CLK is active-high signal (CPOL = 0)*/
    e_SPI_CLK_POLARITY_ACTIVE_LOW  = 1  /**< SPI_CLK is active-low signal (CPOL = 1) */

} e_SpiClockPolarity;


/**************************************************************************//**
 @Description   SPI Clock Phase

                The clock polarity (CPOL) and phase (CPHA) pair is sometimes
                described as the "SPI mode", where mode is numbered as 0-3,
                according to the following options:
                - Mode-0: CPOL=0, CPHA=0
                - Mode-1: CPOL=0, CPHA=1
                - Mode-2: CPOL=1, CPHA=0
                - Mode-3: CPOL=1, CPHA=1

*//***************************************************************************/
typedef enum e_SpiClockPhase
{
    e_SPI_CLK_PHASE_MIDDLE  = 0,        /**< SPI_CLK starts toggling at the middle
                                             of a data transfer (CPHA = 0) */
    e_SPI_CLK_PHASE_START   = 1         /**< SPI_CLK starts toggling at the beginning
                                             of a data transfer (CPHA = 1) */
} e_SpiClockPhase;


/**************************************************************************//**
 @Description   SPI Bit Order

                Determines whether the most significant or least significant
                bit is sent first on the SPI bus.
*//***************************************************************************/
typedef enum e_SpiBitOrder
{
    e_SPI_BO_MSBIT_FIRST,               /**< Most significant bit of the word is sent
                                             and received first */
    e_SPI_BO_LSBIT_FIRST                /**< Least significant bit of the word is sent
                                             and received first */
} e_SpiBitOrder;


#endif /* __SPI_STD_EXT_H */


