/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*Stop Watch*/
#ifndef __STOP_WATCH_EXT
#define __STOP_WATCH_EXT


typedef struct t_StwParam
{
    uint64_t (*f_GetTime)( t_Handle h_TimeClk );
    t_Handle h_TimeClk;

}t_StwParam;

/**************************************************************************//**
 @Function      STW_Config

 @Description   Constructs a stop watch based on user parameters;

 @Param         p_StwParam - (in) user parameters for the stop watch;

 @Return        Handle to created stop watch if succeeded; NULL otherwise;

*//***************************************************************************/
t_Handle STW_Config( t_StwParam *p_StwParam );

/**************************************************************************//**
 @Function      STW_Init

 @Description   Initializes the stop watch;

 @Param         h_Stw - (in) a handle to the stop watch;

 @Return        E_OK - Success, other value otherwise;
*//***************************************************************************/
t_Error STW_Init( t_Handle h_Stw );

/**************************************************************************//**
 @Function      STW_GetId

 @Description   Retrieve an ID for the timing the referred section ;

 @Param         h_Stw - (in) a handle to the stop watch;
 @Param         p_SectionName - (in) the name of the referred section for timing;
 @Param         p_Id - (out) the value of the requested ID;

 @Return        E_OK - Success, other value otherwise;
*//***************************************************************************/
t_Error STW_GetId( t_Handle h_Stw, char* p_SectionName, uint32_t* p_Id );


/**************************************************************************//**
 @Function      STW_Start

 @Description   Starts the stop watch for the referred id;

 @Param         h_Stw - (in) a handle to the stop watch;
 @Param         id - (in) the id of the section for timing;

 @Return        E_OK - Success, other value otherwise;
*//***************************************************************************/
t_Error STW_Start( t_Handle h_Stw, uint32_t id );

/**************************************************************************//**
 @Function      STW_Stop

 @Description   Stops the stop watch and adds the calculated duration to
                former durations for the referred id.

 @Param         h_Stw - (in) a handle to the stop watch;
 @Param         id - (in) the id of the section for timing;

 @Return        E_OK - Success, other value otherwise;
*//***************************************************************************/
t_Error STW_Stop( t_Handle h_Stw, uint32_t id  );

/**************************************************************************//**
 @Function      STW_Print

 @Description   Calculates the absolute value of the average duration;

 @Param         h_Stw - (in) a handle to the stop watch;

 @Return        E_OK - Success, other value otherwise;
*//***************************************************************************/
t_Error STW_Print( t_Handle h_Stw );

/**************************************************************************//**
 @Function      STW_Clear

 @Description   Clears the stop watch counters;

 @Param         h_Stw - (in) a handle to the stop watch;
 @Param         id - (in) the id of the section for timing;

 @Return        E_OK - Success, other value otherwise;
*//***************************************************************************/
t_Error STW_Clear( t_Handle h_Stw, uint32_t id );

#if (TIMING_WITH_STW == 1)
#define USING_STOP_WATCH \
extern t_Handle h_StopWatch;

#define STOP_WATCH_ON(section_name)\
{ \
    static uint32_t stop_watch_id = 0;\
    if(h_StopWatch)\
    {\
        if (!stop_watch_id)\
        {\
            STW_GetId( h_StopWatch, section_name, &stop_watch_id );\
        }\
        STW_Start( h_StopWatch, stop_watch_id );\
    }

#define STOP_WATCH_OFF\
    if(h_StopWatch)\
    {\
        STW_Stop( h_StopWatch, stop_watch_id );\
    }\
}


#define STOP_WATCH_REPORT\
{\
    if(h_StopWatch)\
    {\
        STW_Print( h_StopWatch );\
    }\
}
#else
#define USING_STOP_WATCH

#define STOP_WATCH_ON(section_name)

#define STOP_WATCH_OFF

#define STOP_WATCH_REPORT
/**/
#endif

#endif /* __STOP_WATCH_EXT */
