/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_i2c_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_I2C_EXT_H
#define __PLATFORM_I2C_EXT_H

#include "std_ext.h"
#include "sys_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PLATFORM_InitI2cPort

 @Description   Initializes the platform's I2C bus matching a given I2C
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - I2C controller object type (usually a sub-module type)
 @Param[in]     id          - I2C controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitI2cPort(t_Handle       h_Platform,
                             e_SysModule    module,
                             uint32_t       id);

/**************************************************************************//**
 @Function      PLATFORM_FreeI2cPort

 @Description   Releases the platform's I2C bus matching a given I2C
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - I2C controller object type (usually a sub-module type)
 @Param[in]     id          - I2C controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeI2cPort(t_Handle       h_Platform,
                             e_SysModule    module,
                             uint32_t       id);

/**************************************************************************//**
 @Function      SYS_CheckI2cDeviceAddress

 @Description   Checks if a given I2C device address is valid for a given I2C
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - I2C controller object type (usually a sub-module type)
 @Param[in]     id          - I2C controller object identifier
 @Param[in]     deviceAddr  - I2C device address to check

 @Retval        TRUE:   The I2C address is valid in this platform.
 @Retval        FALSE:  The I2C address is not valid in this platform.
*//***************************************************************************/
bool PLATFORM_CheckI2cDeviceAddress(t_Handle    h_Platform,
                                    e_SysModule module,
                                    uint32_t    i2cBusId,
                                    uint32_t    deviceAddr);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_I2C_EXT_H */
