/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_mpc837x_mds_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_MPC837X_MDS_EXT_H
#define __PLATFORM_MPC837X_MDS_EXT_H

#include "std_ext.h"
#include "platform_ext.h"
#include "carrier_ext.h"
#include "e300_ext.h"

/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Platform Console Types
*//***************************************************************************/
typedef enum e_PlatformConsoleType
{
    e_PLATFORM_CONSOLE_NONE,        /**< Do not use platform console */
    e_PLATFORM_CONSOLE_DUART_1,     /**< Use DUART-1 as console port */
    e_PLATFORM_CONSOLE_DUART_2      /**< Use DUART-2 as console port */

} e_PlatformConsoleType;


#define PLATFORM_MAX_NUM_OF_CONNECTORS  8

typedef struct t_PlatformBoardSettings
{
    struct
    {
        bool                useAutoNegotiation;
    } ethPhySettings;

    t_BoardConnectorDesc    connectors[PLATFORM_MAX_NUM_OF_CONNECTORS];

} t_PlatformBoardSettings;


/**************************************************************************//**
 @Description   MPC837xMDS Platform configuration parameters structure
*//***************************************************************************/
typedef struct t_PlatformParam
{
    uint32_t                clockInFreqHz;
    uint32_t                localBusFreqHz;
    e_CacheMode             cacheMode;
    bool                    useDdrBootSettings;
    bool                    useLbcBootSettings;
    bool                    pci64BitBus;
    e_PlatformConsoleType   consoleType;
    bool                    simulator;

    t_PlatformBoardSettings boardSettings;

    struct
    {
        t_Error (*f_InitMemoryRegions)(t_Handle h_Platform);
        t_Error (*f_InitMemoryPartitions)(t_Handle h_Platform);
        t_Error (*f_InitIntrCtrl)(t_Handle h_Platform);
        t_Error (*f_InitDdr)(t_Handle h_Platform);
        t_Error (*f_InitLocalBus)(t_Handle h_Platform);
        t_Error (*f_InitIoPorts)(t_Handle h_Platform);
    } userInitHooks;

} t_PlatformParam;

/** @} */ /* end of platform_grp */


#endif /* __PLATFORM_MPC837X_MDS_EXT_H */

