/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_sd_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_SD_EXT_H
#define __PLATFORM_SD_EXT_H

#include "std_ext.h"
#include "sys_ext.h"
#include "sdhc_std_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PLATFORM_InitSdPort

 @Description   Initializes the platform's SD port matching a given SD
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - SD controller object type (usually a sub-module type)
 @Param[in]     id          - SD controller object identifier
 @Param[in]     busWidth    - SD bus width

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitSdPort(t_Handle        h_Platform,
                            e_SysModule     module,
                            uint32_t        id,
                            e_SdhcBusWidth  busWidth);

/**************************************************************************//**
 @Function      PLATFORM_FreeSdPort

 @Description   Releases the platform's SD port matching a given SD
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module  - SD controller object type (usually a sub-module type)
 @Param[in]     id      - SD controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeSdPort(t_Handle    h_Platform,
                            e_SysModule module,
                            uint32_t    id);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_SD_EXT_H */
