/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_tdm_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_TDM_EXT_H
#define __PLATFORM_TDM_EXT_H

#include "std_ext.h"
#include "sys_ext.h"
#include "carrier_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PLATFORM_InitTdmPhy

 @Description   Initializes the platform's TDM port matching a given TDM
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - TDM controller object type (usually a sub-module type)
 @Param[in]     id          - TDM controller object identifier
 @Param[in]     f_Events    - Callback routine for link events
 @Param[in]     h_Arg       - The argument to pass to the \c f_Events callback

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitTdmPhy(t_Handle                h_Platform,
                            e_SysModule             module,
                            uint32_t                id,
                            t_PlatformLinkEventsCb  f_Events,
                            t_Handle                h_Arg);

/**************************************************************************//**
 @Function      PLATFORM_FreeTdmPhy

 @Description   Releases the platform's TDM port matching a given TDM
                controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - TDM controller object type (usually a sub-module type)
 @Param[in]     id          - TDM controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeTdmPhy(t_Handle    h_Platform,
                            e_SysModule module,
                            uint32_t    id);

/**************************************************************************//**
 @Function      PLATFORM_EnableTdmPhyIntr

 @Description   Enables interrupts from the platform's TDM port matching a
                given TDM controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - TDM controller object type (usually a sub-module type)
 @Param[in]     id          - TDM controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_EnableTdmPhyIntr(t_Handle      h_Platform,
                                  e_SysModule   module,
                                  uint32_t      id);

/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_TDM_EXT_H */
