/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_utopia_pos_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_UTOPIA_POS_EXT_H
#define __PLATFORM_UTOPIA_POS_EXT_H

#include "std_ext.h"
#include "sys_ext.h"
#include "carrier_ext.h"
#include "utopia_pos_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UTOPIA/POS device parameters (a bus may have several devices).
*//***************************************************************************/
typedef struct t_PlatformUtopiaPosDevParam
{
    e_UtopiaPosDevMode  opMode;             /**< Operation mode (SPHY/MPHY) for the device */
    e_UtopiaPosBusWidth busWidth;           /**< Selected bus width for the device */
    uint8_t             numOfActivePorts;   /**< Number of active ports in the device */
} t_PlatformUtopiaPosDevParam;


/**************************************************************************//**
 @Function      PLATFORM_InitUtopiaPosBus

 @Description   Initializes a UTOPIA/POS bus.

 @Param[in]     h_Platform          - Platform object handle.
 @Param[in]     module              - Object type.
 @Param[in]     id                  - Bus ID.
 @Param[in]     lineRate            - Line rate.
 @Param[in]     protocol            - Protocol.
 @Param[in]     master              - TRUE for bus master mode; FALSE for bus slave mode.
 @Param[in]     numOfDevices        - Number of UTOPIA/POS devices per bus controller.
 @Param[in]     p_DevParamArray     - Array of UTOPIA/POS device parameters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitUtopiaPosBus(t_Handle                      h_Platform,
                                  e_SysModule                   module,
                                  uint32_t                      id,
                                  e_LineRate                    lineRate,
                                  e_UtopiaPosProtocol           protocol,
                                  bool                          master,
                                  uint8_t                       numOfDevices,
                                  t_PlatformUtopiaPosDevParam   *p_DevParamArray);

/**************************************************************************//**
 @Function      PLATFORM_FreeUtopiaPosBus

 @Description   Terminates a UTOPIA/POS bus.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Object type.
 @Param[in]     id          - Bus ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeUtopiaPosBus(t_Handle    h_Platform,
                                  e_SysModule module,
                                  uint32_t    id);

/**************************************************************************//**
 @Function      PLATFORM_InitUtopiaPosPhy

 @Description   Initializes a UTOPIA/POS PHY.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Object type.
 @Param[in]     id          - PHY ID.
 @Param[in]     phyAddr     - PHY address.
 @Param[in]     diagMode    - Diagnostic mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_InitUtopiaPosPhy(t_Handle    h_Platform,
                                  e_SysModule module,
                                  uint32_t    id,
                                  uint8_t     phyAddr,
                                  e_DiagMode  diagMode);

/**************************************************************************//**
 @Function      SYS_FreeUtopiaPosPhy

 @Description   Terminates a UTOPIA/POS PHY.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Object type.
 @Param[in]     id          - PHY ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_FreeUtopiaPosPhy(t_Handle    h_Platform,
                                  e_SysModule module,
                                  uint32_t    id);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_UTOPIA_POS_EXT_H */
