/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_CONSOLE_EXT_H
#define __SYS_CONSOLE_EXT_H

#include "std_ext.h"
#include "sys_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_console_grp     System Console Service

 @Description   Bare-board system interface for console operations.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      SYS_RegisterConsole

 @Description   Register a new system console.

 @Param[in]     h_ConsoleDev    - Handle to console device (passed as the first
                                  parameter to \c f_ConsolePrint and \c f_ConsoleGet).
 @Param[in]     f_ConsolePrint  - Console routine for printing characters.
 @Param[in]     f_ConsoleGet    - Console routine for reading characters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Only a single console is supported. The existing console must
                be unregistered before a new one is registered.
*//***************************************************************************/
t_Error SYS_RegisterConsole(t_Handle    h_ConsoleDev,
                            int         (*f_ConsolePrint)(t_Handle h_ConsoleDev, uint8_t *p_Data, uint32_t size),
                            int         (*f_ConsoleGet)(t_Handle h_ConsoleDev, uint8_t *p_Data, uint32_t size));

/**************************************************************************//**
 @Function      SYS_UnregisterConsole

 @Description   Unregister the existing system console.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SYS_UnregisterConsole(void);


/**************************************************************************//**
 @Function      SYS_Print

 @Description   Prints a characters string to the console device.

                If no console has been registered yet, the function prints the
                string into an internal buffer. The buffer will be printed to
                the first registered console.

 @Param[in]     str - The characters string to print.

 @Return        None.
*//***************************************************************************/
void SYS_Print(char *str);

/**************************************************************************//**
 @Function      SYS_GetChar

 @Description   Reads a single character from the console device.

 @Return        The character read from the console; Zero on failure or if no
                console has been registered yet.
*//***************************************************************************/
char SYS_GetChar(void);


/** @} */ /* end of sys_console_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_CONSOLE_EXT_H */
