/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_INTR_MNG_EXT_H
#define __SYS_INTR_MNG_EXT_H

#include "std_ext.h"


#define SYS_INTR_CTRL_ID_MASK   0xFF000000
#define SYS_INTR_CTRL_ID_SHIFT  24


#define SYS_MAKE_INTR_ID(_intrIdBase, _intrCtrlSrc) \
    ((int)((_intrIdBase) | ((_intrCtrlSrc) & ~(SYS_INTR_CTRL_ID_MASK))))


t_Error SYS_RegisterIntrController(t_Handle h_IntrCtrl,
                                   t_Error  (*f_SetIntr)(t_Handle h_IntrCtrl, int intrSrc, t_Isr *f_Isr, t_Handle handle),
                                   t_Error  (*f_FreeIntr)(t_Handle h_IntrCtrl, int intrSrc),
                                   t_Error  (*f_EnableIntr)(t_Handle h_IntrCtrl, int intrSrc),
                                   t_Error  (*f_DisableIntr)(t_Handle h_IntrCtrl, int intrSrc),
                                   int      *p_IntrIdBase);

t_Error SYS_UnregisterIntrController(t_Handle h_IntrCtrl);


/**************************************************************************//**
 @Function      SYS_SetIntr

 @Description   Set a specific interrupt to be handled by a given callback routine.

 @Param[in]     intrId  - Interrupt source ID.
 @Param[in]     f_Isr   - Interrupt service routine, which will be called when
                          the interupt occurs.
 @Param[in]     handle  - Argument for the interrupt service routine.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error SYS_SetIntr(int intrId, t_Isr *f_Isr, t_Handle handle);

/**************************************************************************//**
 @Function      SYS_FreeIntr

 @Description   Free a specific interrupt source.

 @Param[in]     intrId - Interrupt source ID.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error SYS_FreeIntr(int intrId);

/**************************************************************************//**
 @Function      SYS_EnableIntr

 @Description   Enable a specific interrupt source.

 @Param[in]     intrId - Interrupt source ID.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error SYS_EnableIntr(int intrId);

/**************************************************************************//**
 @Function      SYS_DisableIntr

 @Description   Disable a specific interrupt source.

 @Param[in]     intrId - Interrupt source ID.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error SYS_DisableIntr(int intrId);


#endif /* __SYS_INTR_MNG_EXT_H */

