/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_ETH_EXT_H
#define __SYS_PLATFORM_ETH_EXT_H

#include "std_ext.h"
#include "platform_eth_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitEthernetPhy

 @Description   Initializes the platform's Ethernet PHY matching a given
                Ethernet controller object.

 @Param[in]     module      - Ethernet controller object type (usually a sub-module type)
 @Param[in]     id          - Ethernet controller object identifier
 @Param[in]     ethMode     - Ethernet mode of operation (MAC-PHY interface)
 @Param[in]     duplexMode  - Duplex mode

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitEthernetPhy(e_SysModule       module,
                                              uint32_t          id,
                                              e_EnetMode        ethMode,
                                              e_EnetDuplexMode  duplexMode)
{
    return PLATFORM_InitEthernetPhy(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                    module,
                                    id,
                                    ethMode,
                                    duplexMode);
}

/**************************************************************************//**
 @Function      SYS_FreeEthernetPhy

 @Description   Releases the platform's Ethernet PHY matching a given
                Ethernet controller object.

 @Param[in]     module  - Ethernet controller object type (usually a sub-module type)
 @Param[in]     id      - Ethernet controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeEthernetPhy(e_SysModule module, uint32_t id)
{
    return PLATFORM_FreeEthernetPhy(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                    module,
                                    id);
}

/**************************************************************************//**
 @Function      SYS_GetEthernetLinkStatus

 @Description   Checks the link state of the Ethernet PHY matching a given
                Ethernet controller object.

 @Param[in]     module  - Ethernet controller object type (usually a sub-module type)
 @Param[in]     id      - Ethernet controller object identifier

 @Retval        TRUE    : Link is up.
 @Retval        FALSE   : Link is down or not detected.
*//***************************************************************************/
static __inline__ bool SYS_GetEthernetLinkStatus(e_SysModule module, uint32_t id)
{
    return PLATFORM_GetEthernetLinkStatus(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                          module,
                                          id);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_ETH_EXT_H */

