/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_I2C_EXT_H
#define __SYS_PLATFORM_I2C_EXT_H

#include "std_ext.h"
#include "platform_i2c_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitI2cPort

 @Description   Initializes the platform's I2C bus matching a given I2C
                controller object.

 @Param[in]     module  - I2C controller object type (usually a sub-module type)
 @Param[in]     id      - I2C controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitI2cPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_InitI2cPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                module,
                                id);
}

/**************************************************************************//**
 @Function      SYS_FreeI2cPort

 @Description   Releases the platform's I2C bus matching a given I2C
                controller object.

 @Param[in]     module  - I2C controller object type (usually a sub-module type)
 @Param[in]     id      - I2C controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeI2cPort(e_SysModule module, uint32_t id)
{
    return PLATFORM_FreeI2cPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                module,
                                id);
}

/**************************************************************************//**
 @Function      SYS_CheckI2cDeviceAddress

 @Description   Checks if a given I2C device address is valid for a given I2C
                controller object.

 @Param[in]     module      - I2C controller object type (usually a sub-module type)
 @Param[in]     id          - I2C controller object identifier
 @Param[in]     deviceAddr  - I2C device address to check

 @Retval        TRUE:   The I2C address is valid in this platform.
 @Retval        FALSE:  The I2C address is not valid in this platform.
*//***************************************************************************/
static __inline__ bool SYS_CheckI2cDeviceAddress(e_SysModule module, uint32_t id, uint32_t deviceAddr)
{
    return PLATFORM_CheckI2cDeviceAddress(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                          module,
                                          id,
                                          deviceAddr);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_I2C_EXT_H */

