/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_QE_EXT_H
#define __SYS_PLATFORM_QE_EXT_H

#include "std_ext.h"
#include "qe_ext.h"
#include "platform_qe_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_GetQePeripheralInputClk

 @Description   Gets the input clock signal selection of a given QUICC Engine
                controller object.

 @Param[in]     module      - Controller object type (usually a sub-module type)
 @Param[in]     id          - Controller object identifier
 @Param[in]     mode        - Mode selection (varies for different controllers types)
 @Param[in]     commMode    - Indicates if the input clock is associated with
                              Rx, Tx or both.

 @Return        The input clock signal selection of a given QUICC Engine
                controller object.
*//***************************************************************************/
static __inline__ e_QeClock SYS_GetQePeripheralInputClk(e_SysModule module,
                                                  uint32_t    id,
                                                  int         mode,
                                                  e_CommMode  commMode)
{
    return PLATFORM_GetQePeripheralInputClk(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                            module,
                                            id,
                                            mode,
                                            commMode);
}

/**************************************************************************//**
 @Function      SYS_GetQeBusAccessWindowInfo

 @Description   Retrieves the base address and size of a QUICC Engine memory
                access window.

                The information returned by this routine may be required for
                configuring the dynamic bus selection capabilities of the
                QUICC Engine.

 @Param[in]     secondaryBus    - TRUE if the requested information relates to
                                  the QUICC Engine's secondary bus;
                                  FALSE if the requested information relates to
                                  the QUICC Engine's primary (system) bus.
 @Param[out]    p_BaseAddr      - Returns the base address of the access window
 @Param[out]    p_Size          - Returns the size of the access window

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_GetQeBusAccessWindowInfo(bool     secondaryBus,
                                                       uint32_t *p_BaseAddr,
                                                       uint32_t *p_Size)
{
        return PLATFORM_GetQeBusAccessWindowInfo(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                 secondaryBus,
                                                 p_BaseAddr,
                                                 p_Size);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_QE_EXT_H */

