/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_SD_EXT_H
#define __SYS_PLATFORM_SD_EXT_H

#include "std_ext.h"
#include "platform_sd_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitSdPort

 @Description   Initializes the platform's SD port matching a given SD
                controller object.

 @Param[in]     module      - SD controller object type (usually a sub-module type)
 @Param[in]     id          - SD controller object identifier
 @Param[in]     busWidth    - SD bus width

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitSdPort(e_SysModule    module,
                                         uint32_t       id,
                                         e_SdhcBusWidth busWidth)
{
    return PLATFORM_InitSdPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                   module,
                                                   id,
                                                   busWidth);
}

/**************************************************************************//**
 @Function      SYS_FreeSdPort

 @Description   Releases the platform's SD port matching a given SD
                controller object.

 @Param[in]     module  - SD controller object type (usually a sub-module type)
 @Param[in]     id      - SD controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeSdPort(e_SysModule module,  uint32_t id)
{
    return PLATFORM_FreeSdPort(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                   module,
                                                   id);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_SD_EXT_H */

