/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_TDM_EXT_H
#define __SYS_PLATFORM_TDM_EXT_H

#include "std_ext.h"
#include "platform_tdm_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_InitTdmPhy

 @Description   Initializes the platform's TDM port matching a given TDM
                controller object.

 @Param[in]     module      - TDM controller object type (usually a sub-module type)
 @Param[in]     id          - TDM controller object identifier
 @Param[in]     f_Events    - Callback routine for link events
 @Param[in]     h_Arg       - The argument to pass to the \c f_Events callback

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_InitTdmPhy(e_SysModule            module,
                                         uint32_t               id,
                                         t_PlatformLinkEventsCb f_Events,
                                         t_Handle               h_Arg)
{
    return PLATFORM_InitTdmPhy(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                               module,
                               id,
                               f_Events,
                               h_Arg);
}

/**************************************************************************//**
 @Function      SYS_FreeTdmPhy

 @Description   Releases the platform's TDM port matching a given TDM
                controller object.

 @Param[in]     module  - TDM controller object type (usually a sub-module type)
 @Param[in]     id      - TDM controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_FreeTdmPhy(e_SysModule module, uint32_t id)
{
    return PLATFORM_FreeTdmPhy(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                               module,
                               id);
}

/**************************************************************************//**
 @Function      SYS_EnableTdmPhyIntr

 @Description   Enables interrupts from the platform's TDM port matching a
                given TDM controller object.

 @Param[in]     module      - TDM controller object type (usually a sub-module type)
 @Param[in]     id          - TDM controller object identifier

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_EnableTdmPhyIntr(e_SysModule            module,
                                               uint32_t               id)
{
    return PLATFORM_EnableTdmPhyIntr(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id);
}


/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_TDM_EXT_H */

