/*
 * $QNXLicenseC:
 * Copyright 2010, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

/* Copyright (c) 2008-2009 Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Freescale Semiconductor nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY Freescale Semiconductor ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Freescale Semiconductor BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __TYPES_QNX_H__
#define __TYPES_QNX_H__

#include <sys/types.h>
#include <stdint.h>
//#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "config_qnx.h"

typedef float               float_t;    /* Single precision floating point  */
typedef double              double_t;   /* Double precision floating point  */

#ifndef bool
typedef int bool;
#endif

#ifndef FALSE
#define FALSE 0
#endif


#ifndef TRUE
#define TRUE (-1)
#endif


typedef volatile int atomic_t;
typedef uint64_t            physAddress_t;

typedef uint32_t u32; /* needed by fsl_qman.h */
typedef uint8_t u8; /* needed by fsl_qman.h */
typedef uint16_t u16; /* needed by fsl_qman.h */
//#define __packed __attribute__ ((packed)) /* needed by fsl_qman.h */

//struct device {
//void * ptr;
//};


#define _Packed
#define _PackedType __attribute__ ((packed))

#define UINT8_MIN   0
#define UINT16_MIN  0
#define UINT32_MIN  0
#define UINT64_MIN  0


#define ON          1
#define OFF         0



/************************/
/* memory access macros */
/************************/
#define GET_UINT8(arg)              *(volatile uint8_t *)(&(arg))
#define GET_UINT16(arg)             *(volatile uint16_t*)(&(arg))
#define GET_UINT32(arg)             *(volatile uint32_t*)(&(arg))
#define GET_UINT64(arg)             *(volatile uint64_t*)(&(arg))

#ifdef VERBOSE_WRITE
#define WRITE_UINT8(arg, data)  \
    do { XX_Print("ADDR: 0x%08x, VAL: 0x%02x\r\n",    (uint32_t)&(arg), (data)); *(volatile uint8_t *)(&(arg)) = (data); } while (0)
#define WRITE_UINT16(arg, data) \
    do { XX_Print("ADDR: 0x%08x, VAL: 0x%04x\r\n",    (uint32_t)&(arg), (data)); *(volatile uint16_t*)(&(arg)) = (data); } while (0)
#define WRITE_UINT32(arg, data) \
    do { XX_Print("ADDR: 0x%08x, VAL: 0x%08x\r\n",    (uint32_t)&(arg), (data)); *(volatile uint32_t*)(&(arg)) = (data); } while (0)
#define WRITE_UINT64(arg, data) \
    do { XX_Print("ADDR: 0x%08x, VAL: 0x%016llx\r\n", (uint32_t)&(arg), (data)); *(volatile uint64_t*)(&(arg)) = (data); } while (0)

#else  /* not VERBOSE_WRITE */
#define WRITE_UINT8(arg, data)      *(volatile uint8_t *)(&(arg)) = (data)
#define WRITE_UINT16(arg, data)     *(volatile uint16_t*)(&(arg)) = (data)
#define WRITE_UINT32(arg, data)     *(volatile uint32_t*)(&(arg)) = (data)
#define WRITE_UINT64(arg, data)     *(volatile uint64_t*)(&(arg)) = (data)
#endif /* not VERBOSE_WRITE */


/*****************************************************************************/
/*                      General stuff                                        */
/*****************************************************************************/
#ifdef ARRAY_SIZE
#undef ARRAY_SIZE
#endif /* ARRAY_SIZE */

#ifdef MAJOR
#undef MAJOR
#endif /* MAJOR */

#ifdef MINOR
#undef MINOR
#endif /* MINOR */

#ifdef QE_SIZEOF_BD
#undef QE_SIZEOF_BD
#endif /* QE_SIZEOF_BD */

#ifdef BD_BUFFER_CLEAR
#undef BD_BUFFER_CLEAR
#endif /* BD_BUFFER_CLEAR */

#ifdef BD_BUFFER
#undef BD_BUFFER
#endif /* BD_BUFFER */

#ifdef BD_STATUS_AND_LENGTH_SET
#undef BD_STATUS_AND_LENGTH_SET
#endif /* BD_STATUS_AND_LENGTH_SET */

#ifdef BD_STATUS_AND_LENGTH
#undef BD_STATUS_AND_LENGTH
#endif /* BD_STATUS_AND_LENGTH */

#ifdef BD_BUFFER_ARG
#undef BD_BUFFER_ARG
#endif /* BD_BUFFER_ARG */

#ifdef BD_GET_NEXT
#undef BD_GET_NEXT
#endif /* BD_GET_NEXT */

#ifdef QE_SDEBCR_BA_MASK
#undef QE_SDEBCR_BA_MASK
#endif /* QE_SDEBCR_BA_MASK */

#ifdef BD_BUFFER_SET
#undef BD_BUFFER_SET
#endif /* BD_BUFFER_SET */

#ifdef UPGCR_PROTOCOL
#undef UPGCR_PROTOCOL
#endif /* UPGCR_PROTOCOL */

#ifdef UPGCR_TMS
#undef UPGCR_TMS
#endif /* UPGCR_TMS */

#ifdef UPGCR_RMS
#undef UPGCR_RMS
#endif /* UPGCR_RMS */

#ifdef UPGCR_ADDR
#undef UPGCR_ADDR
#endif /* UPGCR_ADDR */

#ifdef UPGCR_DIAG
#undef UPGCR_DIAG
#endif /* UPGCR_DIAG */

#ifdef PARAMS
#undef PARAMS
#endif /* PARAMS */

#ifdef NO_IRQ
#undef NO_IRQ
#endif /* NO_IRQ */

#define PRINT_LINE   XX_Print("%s:\n %s [%d]\n",__FILE__,__FUNCTION__,__LINE__);

#define QNX_PARTITION_ID_NON_CACHED 0
#define QNX_PARTITION_ID_CACHED 10

#endif /* __TYPES_QNX_H__ */
