# Copyright 2001, 2008, 2009, QNX Software Systems Ltd. All Rights Reserved
#  
# This source code has been published by QNX Software Systems Ltd. (QSSL).
# However, any use, reproduction, modification, distribution or transfer of
# this software, or any software which includes or is based upon any of this
# code, is only permitted under the terms of the QNX Realtime Plaform End User
# License Agreement (see licensing.qnx.com for details) or as otherwise
# expressly authorized by a written license agreement from QSSL. For more
# information, please email licensing@qnx.com.

ifndef QCONFIG
QCONFIG=qconfig.mk
endif
include $(QCONFIG)

.PHONY: first
first: all

EXTRA_INCVPATH = $(PROJECT_ROOT)/inc $(PROJECT_ROOT)/../../services/system/public $(PROJECT_ROOT)/../m/public $(PROJECT_ROOT)/../m/inc $(PROJECT_ROOT)/../pm/public $(PROJECT_ROOT)/../../services/registry/public

ASSEMBLER_TYPE_x86 = gcc
ASMOFF_FORMAT_x86 = cpp
DEFFILE = asmoff.def

OS = nto

#
# The mcount function is used for profiling, and needs the frame pointer
# to find the caller's adress
#

CCFLAGS_mcount = -fno-omit-frame-pointer

#
# Special compiler/assembler options for startup code
#

ASFLAGS_crt1 += -static
ASFLAGS_mcrt1 += -static
ASFLAGS_startup += -shared
ASFLAGS_startup_ion += -DNO_INIT_ARRAY
ASFLAGS_startup_iox += -DNO_INIT_ARRAY -DNO_CLEAR_ERRNO

CCFLAGS_startup_ppc_gcc_ = -G0
CCFLAGS_startup_ppc_gcc_qcc = -Wc,-G0
CCFLAGS_startup_mips_gcc_ = -G0
CCFLAGS_startup_mips_gcc_qcc = -Wc,-G0
CCFLAGS_startup_sh_gcc_ = -mprefergot
CCFLAGS_startup_sh_gcc_qcc = -mprefergot

CCFLAGS_startup = -fpic
CCFLAGS_startup += -finhibit-size-directive
CCFLAGS_startup += $(CCFLAGS_startup_$(CPU)_$(COMPILER_TYPE)_$(COMPILER_DRIVER))

#
# Shrink the library on PPC by avoiding unnecessary F.P. instructions
# on vararg functions that never do any floating point.
#

CCFLAGS_execl_ppc = -msoft-float
CCFLAGS_execlp_ppc = -msoft-float
CCFLAGS_execlpe_ppc = -msoft-float
CCFLAGS_fcntl_ppc = -msoft-float
CCFLAGS_iodir_ppc = -msoft-float
CCFLAGS_open_ppc = -msoft-float
CCFLAGS_mq_open_ppc = -msoft-float
CCFLAGS_sem_open_ppc = -msoft-float
CCFLAGS_typed_mem_open_ppc = -msoft-float
CCFLAGS_traceevent_ppc = -msoft-float
CCFLAGS_hwi_find_item_ppc = -msoft-float
CCFLAGS_sopen_ppc = -msoft-float
CCFLAGS_spawnl_ppc = -msoft-float
CCFLAGS_spawnle_ppc = -msoft-float
CCFLAGS_spawnlp_ppc = -msoft-float
CCFLAGS_spawnlpe_ppc = -msoft-float
CCFLAGS_mount_ppc = -msoft-float
CCFLAGS_ioctl_ppc = -msoft-float
CCFLAGS_open64_ppc = -msoft-float

#
# And try to shrink the literal pools...
#

CCFLAGS_sh += -mspace

CCFLAGS_ansi += -D__INLINE_FUNCTIONS__

#
# Configure gcc warnings
#

gcc-warn += -Wextra
gcc-warn += -Wshadow
gcc-warn += -Wpointer-arith
gcc-warn += -Wbad-function-cast
gcc-warn += -Wno-unused-parameter

CCF_gcc_ += $(gcc-warn)
CCF_gcc_qcc += $(gcc-warn)

#
# Don't display certain gcc warnings in Dinkum libs
#

CCFLAGS_gcc_ansi += -Wno-uninitialized
CCFLAGS_gcc_ansi += -Wno-switch
CCFLAGS_gcc_ansi += -Wno-missing-braces
CCFLAGS_gcc_ansi += -Wno-char-subscripts

CCFLAGS_gcc_stdio += -Wno-uninitialized
CCFLAGS_gcc_stdio += -Wno-char-subscripts

CCFLAGS_gcc_string += -Wno-uninitialized

lint-defer-rules := 1
include $(MKFILES_ROOT)/qrules.mk

ifndef LIBC_SO_VERSION
include $(PROJECT_ROOT)/lib/soversion.mk
-include $(PRODUCT_ROOT)/cpp/soversion.mk
ifndef LIBCPP_SO_VERSION
LIBCPP_SO_VERSION=2
endif
-include $(PRODUCT_ROOT)/m/soversion.mk
ifndef LIBM_SO_VERISION
LIBM_SO_VERSION=2
endif
endif

#
# If we're compiling the SH nofpu variant, add the appropriate -m flag.
#

CCVFLAG_nofpu_sh = -m4-nofpu
CCVFLAG_nofpu = $(CCVFLAG_nofpu_$(CPU))

ifeq ($(filter g, $(VARIANTS)),)
CCF_opt_gcc_    += -O2 -fomit-frame-pointer
CCF_opt_gcc_qcc += -O2 -fomit-frame-pointer
CCF_gcc_    	+= -O2 
CCF_gcc_qcc 	+= -O2 
endif

# We really should fix our source to not require -fno-strict-aliasing
CCF_opt_gcc_    += -fno-strict-aliasing
CCF_opt_gcc_qcc += -fno-strict-aliasing
CCF_gcc_        += -fno-strict-aliasing
CCF_gcc_qcc     += -fno-strict-aliasing

# Arrange for weak symbols where needed (and available)
CCF_gcc_        += -Wp,-include,$(PROJECT_ROOT)/inc/weak.h
CCF_gcc_qcc     += -Wp,-include,$(PROJECT_ROOT)/inc/weak.h

# Never want debug information for startup files - causes compiler
# to get confused about the sections.
ifeq ($(SECTION),startup)
override DEBUG=
CCOPTS:=$(filter-out -g,$(CCOPTS))
endif

CCF_wcc_	= -U__INLINE_FUNCTIONS__
CCF_wcc_	+= -fi$(PROJECT_ROOT)/inc/weak.h

# ION wants MALLOC_PC
CCFLAGS_alloc_ion  = -DMALLOC_PC
CCFLAGS_malloc_iox = -DMALLOC_ID

#what to do when someone does printf("%s",null). default behavior is to sigsegv
CCFLAGS_stdio_qss = -DSTDIO_OPT_PERCENT_S_NULL=NULL

#ION and IOX want printf("%s",NULL) to not sigsegv. 
CCFLAGS_stdio_ion = -DSTDIO_OPT_PERCENT_S_NULL=\"\(null\)\"
CCFLAGS_stdio_iox = -DSTDIO_OPT_PERCENT_S_NULL=\"\(null\)\" 

ifeq ($(BUILDENV),ion)
CCFLAGS_crtbegin_PIC_gcc__mips = -mabicalls
CCFLAGS_crtend_PIC_gcc__mips = -mabicalls
endif

CCFLAGS += $(CCF_$(COMPILER_TYPE)_$(COMPILER_DRIVER))
CCFLAGS += $(CCFLAGS_$(SECTION))
CCFLAGS += $(CCFLAGS_$(COMPILER_TYPE)_$(SECTION))
CCFLAGS += $(CCFLAGS_$(COMPILER_TYPE)_$(CPU))
CCFLAGS += $(CCFLAGS_$(CPU))
CCFLAGS += $(CCFLAGS_$(basename $@))
CCFLAGS += $(CCFLAGS_$(basename $@)_$(CPU))
CCFLAGS += $(CCFLAGS_$(basename $@)_$(COMPILER_TYPE)_$(COMPILER_DRIVER)_$(CPU))
CCFLAGS += $(CCFLAGS_$(SECTION)_$(BUILDENV))
CCFLAGS += -D_LIBC_SO_VERSION=$(LIBC_SO_VERSION)
CCFLAGS += -D_LIBCPP_SO_VERSION=$(LIBCPP_SO_VERSION)
CCFLAGS += -D_LIBM_SO_VERSION=$(LIBM_SO_VERSION)

ASFLAGS += $(ASFLAGS_$(SECTION))
ASFLAGS += $(ASFLAGS_$(COMPILER_TYPE)_$(CPU))
ASFLAGS += $(ASFLAGS_$(CPU))
ASFLAGS += $(ASFLAGS_$(SECTION)_$(BUILDENV))
ASFLAGS += $(ASFLAGS_$(basename $@))
ASFLAGS += $(ASFLAGS_$(basename $@)_$(CPU)) 

# Some sections we only want to build in certain environments	
BUILDENV_dllmgr = iox
BUILDENV_malloc = iox
BUILDENV_dllmisc = iox
BUILDENV_alloc = qss ion
BUILDENV_ldd = qss
BUILDENV_dllstubs = iox
BUILDENV_libdl = iox
BUILDENV_miscdll = iox
ifeq ($(BUILDENV),)
BUILDENV = qss
endif

ifeq ($(BUILDENV),iox)
CCVFLAG_dll += -D__DLL -D__SLIB
ASVFLAG_dll += -D__DLL -D__SLIB
endif

ifneq ($(filter $(BUILDENV), $(if $(BUILDENV_$(SECTION)),$(BUILDENV_$(SECTION)),$(BUILDENV))),$(BUILDENV))
OBJS=
endif

ifeq ($(COMPILER),wcc)
wcc_comp := $(firstword $(CC_nto_x86_wcc))
ifeq ($(and $(wcc_comp),$(wildcard $(wcc_comp))),)
OBJS =
endif
endif

#
# Rules
#

.PHONY: all
all: $(OBJS)

ifeq ($(BUILDENV),iox)
.PHONY: stubs
stubs:
ifneq ($(OBJS),)
	$(MKSTUBS) -v -D $(PROJECT_ROOT)/dllstubs -S -d libc.dll lib_$(SECTION).dll.a $(OBJS)
endif
endif

.PHONY: install
install: all qinstall

.PHONY: qinstall
qinstall: # Nothing to do

.PHONY: hinstall
hinstall: # Nothing to do

.PHONY: iclean
iclean: # Nothing to do

.PHONY: spotless clean
spotless clean:
	$(RM_HOST) *.o $(DEFFILE) $(EXTRA_CLEAN)

ldd.o: ldd.c $(firstword $(wildcard ../relocs.ci ../../relocs.ci))

#
# Lint Configuration
#

topdir-libc = $(PROJECT_ROOT)

include $(topdir-libc)/libc-qlint.mk
