/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <drvr/hwinfo.h>

/*******************************************************************************
* hwi_find_bus
* 
* if the <unit> instance of <name> has been added to the hwinfo section
* of the syspage then return its hwi offset.
* 
* This function ensures that the <name> found is in fact a bus.
* 
* This function is basically a wrapper for hwi_find_item() that allows for a
* specific item instance to be returned however it differs from hwi_find_item()
* int that it will resolve (all) synonym names (aliases) to a real bus.
* 
* Returns: the hwinfo section offset or HWI_NULL_OFF if the device instance
* 			does not exist
* 
* Note that although we used the name 'bus' to be consistent with previous
* nomenclature, from a topology perscpective, we actually mean any interconnect
* to which a device is connected
* 
*/
unsigned hwi_find_bus(const char *name, unsigned unit)
{
	unsigned hwi_off = hwi_find(name, unit, HWI_NULL_OFF);

	if (hwi_off != HWI_NULL_OFF)
	{
		hwi_tag *tag = hwi_off2tag(hwi_off);
		if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_bus) != 0) {
			hwi_off = HWI_NULL_OFF;
		}
	}
	return hwi_off;
}

__SRCVERSION("$URL$ $Rev$");
