/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>


/*******************************************************************************
* hwi_find_devicebus
* 
* Find the bus that the device <device_hwi_off> resides on. This function
* ensure that <device_hwi_off> is in fact a device and that its 'owner' is in
* fact a bus.
* 
* Returns: the hwinfo section offset or HWI_NULL_OFF if the bus/device instance
* 			does not exist
* 
* Note that although we used the name 'bus' to be consistent with previous
* nomenclature, from a topology perscpective, we actually mean any interconnect
* to which a device is connected
*/
unsigned hwi_find_devicebus(unsigned device_hwi_off)
{
	unsigned bus_hwi_off = HWI_NULL_OFF;

	if (device_hwi_off != HWI_NULL_OFF)
	{
		hwi_tag *tag = hwi_off2tag(device_hwi_off);
		if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_device) == 0) {
			hwi_tag *bus_tag = hwi_off2tag(tag->item.owner);
			if (strcmp(__hwi_find_string(bus_tag->prefix.name), HWI_TAG_NAME_bus) == 0) {
				bus_hwi_off = tag->item.owner;
			}
		}
	}
	return bus_hwi_off;
}

__SRCVERSION("$URL$ $Rev$");
