/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <drvr/hwinfo.h>

/*******************************************************************************
* hwi_find_unit
* 
* Given that multiple instances of a device or bus may exist in the hwinfo
* section of the sypage, determine which instance of the item <hwi_off> is.
* 
* Note that this routine will resolve all synonyms and return the instance of a
* real device or bus, not the instance of a synonym name. For example, if the 2nd
* instance of synonym name "foo" resolves to the 4th instance of real device
* "bar", 3 will be returned (base 0 instance numbering).
* 
* Returns: the device or bus instance (or unit #, base 0) or -1 on error
*  
*/
int hwi_find_unit(unsigned hwi_off)
{
	if (hwi_off != HWI_NULL_OFF)
	{
		unsigned unit_cnt = 0;
		unsigned hwi_off_unit = HWI_NULL_OFF;
		hwi_tag *tag = hwi_off2tag(hwi_off);
		const char *name = __hwi_find_string(tag->prefix.name);

		if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_bus) == 0)
		{
			name = __hwi_find_string(tag->item.itemname);
			while ((hwi_off_unit = hwi_find_bus(name, unit_cnt)) != HWI_NULL_OFF)
			{
				if (hwi_off_unit == hwi_off) return (int)unit_cnt;
				++unit_cnt;
			}
		}
		else if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_device) == 0)
		{
			name = __hwi_find_string(tag->item.itemname);
			while ((hwi_off_unit = hwi_find_device(name, unit_cnt)) != HWI_NULL_OFF)
			{
				if (hwi_off_unit == hwi_off) return (int)unit_cnt;
				++unit_cnt;
			}
		}
	}
	return -1;
}

__SRCVERSION("$URL$ $Rev$");
