/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>

/*******************************************************************************
 * hwi_tag_find
 * 
 * This function will locate a specific instance of the named tag <tag_name> for
 * the device specified by <hwi_off>.
 * 
 * <tag_idx> is pointer to the specific tag instance of interest. Some devices
 * may contain multiple tags of the same type. This parameter allows the caller
 * to obtain a specific instance or repeatedly call this function to obtain all
 * instances.
 * For example, to obtain the 2nd instance of <tag_name>, set *<tag_idx> = 1.
 * Repeatedly calling this function with the <tag_idx> parameter will sequentially
 * retrieve each one starting with the 2nd. Of course to get them all set
 * <tag_idx> = 0 for the first call. If you only want the first tag instance,
 * <tag_idx> can be set to NULL which is equivalent to *<tag_idx> = 0 and only
 * calling once.
 * 
 * Returns: the 'hwi_tag' or NULL on error
*/

hwi_tag *hwi_tag_find(unsigned hwi_off, const char *tag_name, unsigned *tag_idx)
{
	if (hwi_off != HWI_NULL_OFF)
	{
		unsigned _tag_idx = 0;
		unsigned tag_cnt = 0;
		unsigned off = hwi_off;

		if (tag_idx == NULL) tag_idx = &_tag_idx;
		/* while there are more (tag_name) tags for this device */
		while ((off = hwi_next_tag(off, 1)) != HWI_NULL_OFF)
		{
			hwi_tag *tag = hwi_off2tag(off);
			if (strcmp(__hwi_find_string(tag->prefix.name), tag_name) != 0)
				continue;
			if (*tag_idx == tag_cnt) {
				*tag_idx = tag_cnt + 1;
				return tag;
			}
			++tag_cnt;
		}
	}
	return NULL;
}


__SRCVERSION("$URL$ $Rev$");

