/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <sys/hwinfo.h>
#include <drvr/hwinfo.h>
#include <errno.h>


/*******************************************************************************
 * hwiattr_get_spi
 * 
 * this function will retrieve the attributes for the SPI bus identified
 * by <hwi_off> and fill in the <hwiattr> structure.
 *  
 * Returns: the EOK on success or errno
*/
int hwiattr_get_spi(unsigned hwi_off, hwiattr_spi_t *hwiattr)
{
	int r = EINVAL;

	if ((hwi_off != HWI_NULL_OFF) && (hwiattr != NULL))
	{
		hwi_tag *tag;
		unsigned cnt;

		memset(hwiattr, 0, sizeof(*hwiattr));

		/* we expect at least 1 location tag */
		tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_location, NULL);
		if (tag == NULL) return ENOENT;	// mandatory tag
		hwiattr->common.location = tag->location;

		/* get optional dll name */
		tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_dll, NULL);
		if (tag != NULL) hwiattr->common.dll = __hwi_find_string(tag->dll.name);
		
		/* get optional optstr */
		tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_optstr, NULL);
		if (tag != NULL) hwiattr->common.optstr = __hwi_find_string(tag->optstr.string);
		
		/* figure out the number of IRQ tags */
		cnt = 0;
		while(hwitag_find_ivec(hwi_off, &cnt) != HWI_ILLEGAL_VECTOR);
		hwiattr->common.num_irq = cnt;
		
		/* figure out the number of errata tags */
		cnt = 0;
		while(hwi_tag_find(hwi_off, HWI_TAG_NAME_errata, &cnt) != NULL);
		hwiattr->common.num_errata = cnt;
		
		/* now fill in the SPI specific tags */

		/* figure out the number of inputclk tags */
		cnt = 0;
		while(hwi_tag_find(hwi_off, HWI_TAG_NAME_inputclk, &cnt) != NULL);
		hwiattr->num_clks = cnt;

		/* figure out the number of busattr tags */
		cnt = 0;
		while(hwi_tag_find(hwi_off, HWI_TAG_NAME_busattr, &cnt) != NULL);
		hwiattr->num_busattr = cnt;
		
		/* success */
		r = EOK;
	}
	return r;
}


__SRCVERSION("$URL$ $Rev$");

