/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <drvr/hwinfo.h>


/*******************************************************************************
* hwitag_find_busattr
* 
* This routine can be used to find 1 or several bus attribute tags associated
* with the bus at <hwi_off> in the hwinfo section of the syspage.
* The <hwi_off> parameter is most easily obtained with hwi_find_bus().
* 
* If the specified bus attribute tag exists and <busattr> is not NULL, the
* retrieved bus attributes will be placed in <busattr>.  
* 
* Some buses contain multiple sets of attributes. The caller can retrieve all
* of them by using the <busattr_idx> parameter as follows.
* On the first call, *<busattr_idx> should be set to 0. The first bus attribute
* (if one exists) will be returned. The caller can then continue to call
* hwitag_find_busattr() with the same arguments until NULL is returned.
* If the caller knows that there is more than 1 set of bus attributes for the
* specified bus, *<busattr_idx> can be set to the entry number of the desired
* attribute tag and if that entry exists, it will be returned. For example, to
* obtain the 3rd bus attribute for <hwi_off>, set *<busattr_idx> == 2 (base 0
* indexing applies).
* If the caller is only interested in the first attribute, NULL can be passed for
* <busattr_idx> (same as setting *<busattr_idx> == 0)
* 
* Returns: 0 on success (independent of whether <busattr> is NULL) or -1 on error
* 
*/ 
int hwitag_find_busattr(unsigned hwi_off, unsigned *busattr_idx, struct hwi_busattr *busattr)
{
	hwi_tag *tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_busattr, busattr_idx);
	if ((tag != NULL) && (busattr != NULL)) {
		*busattr = tag->busattr;
	}
	return (tag == NULL) ? -1 : 0;
}


__SRCVERSION("$URL$ $Rev$");
