/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <drvr/hwinfo.h>


/*******************************************************************************
* hwitag_find_clkfreq
* 
* This routine can be used to find 1 or several clock sources associated
* with the device at <hwi_off> in the hwinfo section of the syspage.
* The <hwi_off> parameter is most easily obtained with hwi_find_device().
* 
* Some devices contain clock sources vectors. The caller can retrieve the base
* frequency for all of them by using the <clk_idx> parameter as follows.
* On the first call, *<clk_idx> should be set to 0. The first clock frequency
* (if one exists) will be returned. The caller can then continue to call
* hwitag_find_clkfreq() with the same arguments until -1 is returned.
* If the caller knows that there is more than 1 clock source for the
* specified device, *<clk_idx> can be set to the number of the desired clock and
* if that clock exists, the base frequency will be returned. For example, to
* obtain the 3rd clock frequency for <hwi_off>, set *<clk_idx> == 2 (base 0
* indexing applies).
* If the caller is only interested in the first clock, NULL can be passed for
* <clk_idx> (same as setting *<clk_idx> == 0)
* 
* Returns: the clock source frequency in Hz or -1 if the clock source is not found
* 
*/ 
long hwitag_find_clkfreq(unsigned hwi_off, unsigned *clk_idx)
{
	hwi_tag *tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_inputclk, clk_idx);
	return (tag == NULL) ? -1 : (long)tag->inputclk.clk / (long)tag->inputclk.div;
}


__SRCVERSION("$URL$ $Rev$");
