/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <drvr/hwinfo.h>


/*******************************************************************************
* hwitag_find_errata
* 
* This routine can be used to find 1 or several errata tags associated with
* the device at <hwi_off> in the hwinfo section of the syspage.
* The <hwi_off> parameter is most easily obtained with hwi_find_device().
* 
* The caller can retrieve all errata by using the <errata_idx> parameter as
* follows.
* On the first call, *<errata_idx> should be set to 0. The first address
* (if one exists) will be returned. The caller can then continue to call
* hwitag_find_errata() with the same arguments until -1 is returned.
* If the caller knows that there is more than 1 errata for the specified
* device, *<errata_idx> can be set to the instance number of the desired errata
* and if that instance exists, the address will be returned. For example, to
* obtain the 3rd errata for <hwi_off>, set *<errata_idx> == 2 (base 0 indexing
* applies). If the caller is only interested in the first errata, NULL can be
* passed for <errata_idx> (same as setting *<errata_idx> == 0)
* 
* Returns: the errata value on success or -1 on error
* 
* Note that by convention, startup will set the errata value to the vendors
* chosen errata number as obtained from the errata documents. Some vendors don't
* necessarily use a unique errata number and so the startup and driver must
* agree on a errata value.
* 
*/ 
int hwitag_find_errata(unsigned hwi_off, unsigned *errata_idx)
{
	hwi_tag *tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_errata, errata_idx);
	return (tag == NULL) ? -1 : tag->errata.num;
}

__SRCVERSION("$URL$ $Rev$");
