/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <drvr/hwinfo.h>


/*******************************************************************************
* hwitag_find_ivec
* 
* This routine can be used to find 1 or several interrupt vectors associated
* with the device at <hwi_off> in the hwinfo section of the syspage.
* The <hwi_off> parameter is most easily obtained with hwi_find_device().
* 
* Some devices contain multiple interrupt vectors. The caller can retrieve all
* of them by using the <ivec_idx> parameter as follows.
* On the first call, *<ivec_idx> should be set to 0. The first vector (if one exists)
* will be returned. The caller can then continue to call hwitag_find_ivec() with
* the same arguments until HWI_ILLEGAL_VECTOR is returned.
* If the caller knows that there is more than 1 interrupt vector for the
* specified device, *<ivec_idx> can be set to the number of the desired interrupt and
* if that interrupt exists, the vector will be returned. For example, to obtain
* the 3rd interrupt for <hwi_off>, set *<ivec_idx> == 2 (base 0 indexing applies).
* If the caller is only interested in the first vector, NULL can be passed for
* <ivec_idx> (same as setting *<ivec_idx> == 0)
* 
* Returns: the interrupt vector or HWI_ILLEGAL_VECTOR is the vector is not found
* 
*/ 
unsigned hwitag_find_ivec(unsigned hwi_off, unsigned *ivec_idx)
{
	hwi_tag *tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_irq, ivec_idx);
	return (tag == NULL) ? HWI_ILLEGAL_VECTOR : tag->irq.vector;
}


__SRCVERSION("$URL$ $Rev$");
