/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <drvr/hwinfo.h>


/*******************************************************************************
* hwitag_find_nicaddr
* 
* This routine can be used to find 1 or several NIC (mac) addresses associated
* with the device at <hwi_off> in the hwinfo section of the syspage.
* The <hwi_off> parameter is most easily obtained with hwi_find_device().
* 
* If the specified NIC address exists and <nicaddr> is not NULL, the retrieved
* address will be placed in <nicaddr>.  
* 
* Some devices contain multiple NIC addresses. The caller can retrieve all
* of them by using the <nicaddr_idx> parameter as follows.
* On the first call, *<nicaddr_idx> should be set to 0. The first address
* (if one exists) will be returned. The caller can then continue to call
* hwitag_find_nicaddr() with the same arguments until NULL is returned.
* If the caller knows that there is more than 1 NIC address for the specified
* device, *<nicaddr_idx> can be set to the entry number of the desired address
* and if that entry exists, the address will be returned. For example, to obtain
* the 3rd address for <hwi_off>, set *<nicaddr_idx> == 2 (base 0 indexing applies).
* If the caller is only interested in the first address, NULL can be passed for
* <nicaddr_idx> (same as setting *<nicaddr_idx> == 0)
* 
* Returns: 0 on success (independent of whether <nicaddr> is NULL) or -1 on error
* 
*/ 
int hwitag_find_nicaddr(unsigned hwi_off, unsigned *nicaddr_idx, _Uint8t *nicaddr)
{
	hwi_tag *tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_nicaddr, nicaddr_idx);
	if ((tag != NULL) && (nicaddr != NULL)) {
		memcpy(nicaddr, tag->nicaddr.addr, 6);	// mac addresses are 6 octets
	}
	return (tag == NULL) ? -1 : 0;
}


__SRCVERSION("$URL$ $Rev$");
