/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <drvr/hwinfo.h>


/*******************************************************************************
* hwitag_find_phyaddr
* 
* This routine can be used to find 1 or several PHY addresses associated with
* the device at <hwi_off> in the hwinfo section of the syspage.
* The <hwi_off> parameter is most easily obtained with hwi_find_device().
* 
* Some devices contain multiple PHY addresses. The caller can retrieve all
* of them by using the <phy_idx> parameter as follows.
* On the first call, *<phy_idx> should be set to 0. The first address
* (if one exists) will be returned. The caller can then continue to call
* hwitag_find_phyaddr() with the same arguments until -1 is returned.
* If the caller knows that there is more than 1 PHY address for the specified
* device, *<phy_idx> can be set to the entry number of the desired address
* and if that entry exists, the address will be returned. For example, to obtain
* the 3rd address for <hwi_off>, set *<phy_idx> == 2 (base 0 indexing applies).
* If the caller is only interested in the first address, NULL can be passed for
* <phy_idx> (same as setting *<phy_idx> == 0)
* 
* Returns: the PHY address on success or -1 on error
* 
*/ 
int hwitag_find_phyaddr(unsigned hwi_off, unsigned *phy_idx)
{
	hwi_tag *tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_nicphyaddr, phy_idx);
	return (tag == NULL) ? -1 : tag->nicphyaddr.addr;
}

__SRCVERSION("$URL$ $Rev$");
