/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <gulliver.h>


#include "f3s.h"

static void F3S_ENDIANISE(resmgr_context_t *ctp, int dcmd, void *msg)
{
	if (ctp->info.flags & _NTO_MI_ENDIAN_DIFF) {
		switch (dcmd) {
		case DCMD_F3S_ERASE:
			{ f3s_erase_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->offset);
				ENDIAN_SWAP32(&m->limit);
			} break;
		case DCMD_F3S_FORMAT:
			{ f3s_format_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->offset);
				ENDIAN_SWAP32(&m->limit);
				ENDIAN_SWAP32(&m->unit_spare);
				ENDIAN_SWAP32(&m->align_pow2);
				ENDIAN_SWAP32(&m->xip_pow2);
				ENDIAN_SWAP32(&m->name_length);
			} break;
		case DCMD_F3S_RECLAIM:
			{ f3s_reclaim_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->limit);
			} break;
		case DCMD_F3S_PARTINFO:
			{ f3s_partinfo_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->total_size);
				ENDIAN_SWAP32(&m->unit_size);
				ENDIAN_SWAP32(&m->spare_size);
				ENDIAN_SWAP32(&m->retire_size);
				ENDIAN_SWAP32(&m->head_size);
				ENDIAN_SWAP32(&m->pad_size);
				ENDIAN_SWAP32(&m->free_size);
				ENDIAN_SWAP32(&m->stale_size);
				ENDIAN_SWAP32(&m->resv_size);
			} break;
		case DCMD_F3S_ARRAYINFO:
			{ f3s_arrayinfo_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->total_size);
				ENDIAN_SWAP32(&m->unit_size);
				ENDIAN_SWAP32(&m->chip_size);
			} break;
		case DCMD_F3S_UNITINFO:
			{ f3s_unitinfo_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->offset);
				ENDIAN_SWAP32(&m->unit_size);
				ENDIAN_SWAP32(&m->erase_count);
			} break;
		case DCMD_F3S_MOUNT:
			{ f3s_name_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->mount_flags);
				ENDIAN_SWAP32(&m->name_length);
			} break;
		case DCMD_F3S_LOCK:
			{ f3s_unitlock_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->offset);
				ENDIAN_SWAP32(&m->limit);
			} break;
		case DCMD_F3S_UNLOCK:
			{ f3s_unitunlock_t *m = msg;
				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP32(&m->offset);
				ENDIAN_SWAP32(&m->limit);
			} break;
		case DCMD_F3S_UNLOCKALL:
			{ f3s_arrayunlock_t *m = msg;
				ENDIAN_SWAP32(&m->status);
			} break;
		case DCMD_F3S_RECLAIMCTL:
			{ f3s_reclaimctl_t *m = msg;
				ENDIAN_SWAP16(&m->super_count);
				ENDIAN_SWAP16(&m->stale_percent);
				ENDIAN_SWAP16(&m->reclaim_enable);
				ENDIAN_SWAP16(&m->reserved);
			} break;
		case DCMD_F3S_GEOINFO:
			{
				f3s_geoinfo_t *m = msg;
				int i;

				ENDIAN_SWAP32(&m->status);
				ENDIAN_SWAP64(&m->base);
				ENDIAN_SWAP32(&m->size);
				ENDIAN_SWAP16(&m->chipwidth);
				ENDIAN_SWAP16(&m->interleave);
				for (i=0; i < m->num_geo; i++){
					ENDIAN_SWAP16(&m->geo[i].unit_num);
					ENDIAN_SWAP16(&m->geo[i].unit_pow2);
				}
				ENDIAN_SWAP16(&m->num_geo);
			}break;
		}
	}
}

int f3s_raw_erase (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *		part = NULL;
	f3s_array_t *		array;
	f3s_attr_t *		attr;
	f3s_erase_t *		erase;
	struct _client_info	info;
	int32_t				phys;
	uint32_t			unit_first;
	uint32_t			unit_num;
	uint32_t			end;
	int					i;
	int					error;
	int					part_locked=0;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check for privileges */
	error = iofunc_client_info (ctp, 0, &info);
	if (error) return (error);

	error = iofunc_check_access (ctp, &attr->attr, S_IWRITE, &info);
	if (error) return (error);

	/* set erase pointer in message */
	erase = (f3s_erase_t *)(&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, erase);

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		int  error;
		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);
		part_locked = 1;

		f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
		/* detach file system attributes from partition */
		error = f3s_part_detach (part, NULL, attr->part_attr);
		f3s_assert(error == EOK);

		/* get array info pointer */
		array = part->array;

		/* calculate unit first and unit num */
		unit_first = part->unit_first + (erase->offset >> array->unit_pow2);
		unit_num = min(part->unit_num - (erase->offset >> array->unit_pow2),
		               erase->limit >> array->unit_pow2);

		/* sanity checks */
		end = part->unit_first + part->unit_num;
		if ((unit_first < part->unit_first) ||
		    (unit_first >= end) || ((unit_first + unit_num) > end))
		{
			F3S_TABLE_LEAVE(part);
			return (ERANGE);
		}

	/* else, attach id is for socket */
	} else
	{
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* detach array (all partitions) */
		f3s_array_detach (array, attr);

		/* set unit first and unit num */
		unit_first = erase->offset >> array->unit_pow2;
		unit_num = min(array->unit_total - (erase->offset >> array->unit_pow2),
		               erase->limit >> array->unit_pow2);

		/* sanity checks */
		end = array->unit_total;
		if ((unit_first >= end) || ((unit_first + unit_num) > end))
		{
			return (ERANGE);
		}
	}

	/* for all units in partition */
	for (phys = unit_first; phys < (unit_first + unit_num); phys++)
	{
		/* call flash erase function */
		F3S_RAW_ENTER(array, phys);
		for (i = 0; i <= f3s.retry; i++) {
			error = f3s_flash_erase (array, phys);
			if (error) {
				if (error == EIO) continue;
				else              break;
			} else {
				break;
			}
		}
		F3S_RAW_LEAVE(array, phys);
		if (error) break;
	}

	/* check if attach id is not for socket */
	if (part_locked)
	{
		F3S_TABLE_LEAVE(part);
		part_locked = 0;
	}

	return (error);
/*}}}1*/
}

int f3s_raw_format (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *		part = NULL;
	f3s_array_t *		array;
	f3s_attr_t *		attr;
	f3s_format_t *		format;
	struct _client_info	info;
	int32_t				phys;
	uint32_t			unit_first;
	uint32_t			unit_num;
	uint32_t			unit_spare;
	uint32_t			end;
	int					error;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check if attach id is for socket */
	if (ctp->id == attr->attr.mount->attach_id[F3S_ATTACH_SKT]) return (EINVAL);

	/* check for privileges */
	error = iofunc_client_info (ctp, 0, &info);
	if (error) return (error);

	error = iofunc_check_access (ctp, &attr->attr, S_IWRITE, &info);
	if (error) return (error);

	/* update times */
	iofunc_time_update (&attr->attr);

	/* set format pointer in message */
	format = (f3s_format_t *) (&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, format);

	/* get partition info pointer */
	part = attr->part;

	F3S_TABLE_ENTER(part);

	f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
	/* detach file system attributes from partition */
	error = f3s_part_detach (part, NULL, attr->part_attr);
	f3s_lock_assert(error == EOK);

	/* Reset endian to native */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	part->table.swap_flag = 0;

	/* get array info pointer */
	array = part->array;

	/* calculate unit first and unit num */
	unit_first = part->unit_first + (format->offset >> array->unit_pow2);
	unit_num   = min(part->unit_num - (format->offset >> array->unit_pow2),
	                 format->limit >> array->unit_pow2);

	/* sanity checks */
	end = part->unit_first + part->unit_num;
	if ((unit_first < part->unit_first) ||
	    (unit_first >= end) || ((unit_first + unit_num) > end))
	{
		F3S_TABLE_LEAVE(part);
		return (ERANGE);
	}

	/* set number of spare units */
	unit_spare = (unit_num == 1) ? 0 : format->unit_spare;

	/* for all physical units */
	for (phys = unit_first; phys < (unit_first + unit_num); phys++)
	{
		F3S_UNIT_ENTER(part, phys);
	}

	/* check if unit num is zero */
	if (!unit_num)
	{
		error = EINVAL;
		goto fail_shortcut;
	}

	/* for all physical units */
	for (phys = unit_first; phys < (unit_first + unit_num); phys++)
	{
		/* write a unit info extent */
		error = f3s_unit_write (part, phys, 0, f3s_boot_extptr);
		if (error) goto fail_shortcut;
	}

	/* for all physical units except spare units */
	for (phys = unit_first; phys < (unit_first + (unit_num - unit_spare)); phys++)
	{
		/* write logical unit number into unit */
		error = f3s_unit_logi (part, &phys,
		                       F3S_FIRST_LOGI + phys - unit_first, 0, 0);
		if (error) goto fail_shortcut;
	}

	/* write boot info into first unit */
	error = f3s_boot_write (part, unit_first, f3s_root_extptr,
	                        unit_num, unit_spare, format->align_pow2,
	                        0);
	if (error) goto fail_shortcut;

	/* write root directory entry */
	error = f3s_root_write (part, unit_first, f3s_first_extptr,
	                        (char*)(format + 1), format->name_length, 
	                        attr->attr.ctime, format->align_pow2);
	if (error) goto fail_shortcut;

fail_shortcut:
	/* for all units in partition */
	for (phys = unit_first; phys < (unit_first + unit_num); phys++)
	{
		F3S_UNIT_LEAVE(part, phys);
	}

	F3S_TABLE_LEAVE(part);
	return (error);
/*}}}1*/
}

int f3s_raw_mount (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *	part;
	f3s_array_t *	array;
	f3s_attr_t *	attr;
	f3s_name_t *	name;
	int				error;

	/* get attributes pointer */
	attr = ocb->ocb.attr;

	/* set name pointer in message */
	name = (f3s_name_t *) (&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, name);

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* get partition info pointer */
		part = attr->part;
		F3S_TABLE_ENTER(part);

		/* make sure partition was not mounted before */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		if (!(part->table.status & F3S_TABLE_NO_ATTACH))
		{
			F3S_TABLE_LEAVE(part);
			return _RESMGR_ERRNO(EOK);
		}

		f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
		/* check if name has a length */
		if (name->name_length > 1)
		{
			/* set name into mount structure */
			attr->part_attr->attr.mount->name = (char*)(name + 1);
		}

		/* set mount flags */
		attr->part_attr->attr.mount->mount.flags &= ~0xf;
		attr->part_attr->attr.mount->mount.flags |= name->mount_flags & 0xf;

		/* attach file system attributes to partition */
		f3s.force_mount = 1;
		error = f3s_part_attach (part, NULL, attr->part_attr, part->index);
		F3S_TABLE_LEAVE(part);

		/* else, attach id is for socket */
	} else
	{
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* check if there is no pair in array list */
		if (attr->attr.mount->mounted.pair_list.next ==
		    &attr->attr.mount->mounted.pair_list)
		{
			f3s.force_mount = 1;
			/* detach partition attributes from socket */
			error = f3s_array_attach (array, attr);

		/* there is something mounted */
		} else
		{
			error = EOK;
		}
	}

	f3s.force_mount = 0;
	return (error);
/*}}}1*/
}

int f3s_raw_mountedby (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *attr;
	f3s_part_t *part = NULL;
	char *		name;

	/* get attribute pointer */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check if attach id is not for socket */
	if (ctp->id == attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		return (ENXIO);
	}

	/* check if attributes are not attached */
	part = attr->part;
	F3S_TABLE_ENTER(part);
	if (part->table.status & F3S_TABLE_NO_ATTACH)
	{
		F3S_TABLE_LEAVE(part);
		return (ENXIO);
	}
	F3S_TABLE_LEAVE(part);

	/* set name pointer in message */
	name = (char *)(&msg->o + 1);

	f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
	/* copy mount name in message */
	strcpy (name, attr->part_attr->attr.mount->name);

	/* return result message */
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + strlen (name) + 1));
/*}}}1*/
}

int f3s_raw_mountedon (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *	attr;
	f3s_part_t *	part;
	f3s_array_t *	array;
	char *			name;

	/* get attribute pointer */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
	/* check if attach id is not for socket */
	if (ctp->id == attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		return (ENXIO);
	}

	/* get partition info pointer */
	part = attr->part;

	/* get array info pointer */
	array = part->array;

	/* set name pointer in message */
	name = (char *)(&msg->o + 1);

	/* build pathname for socket mountpoint */
	sprintf (name, _PATH_DEV "fs%X", array->index);

	/* return result message */
	msg->o.ret_val = 0;

	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + strlen (name) + 1));
/*}}}1*/
}

int f3s_raw_umount (resmgr_context_t *ctp, void *reserved, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *	part;
#if 0
	f3s_array_t *	array;
#endif
	f3s_attr_t *	attr;
	int				error;

	/* get attributes pointer */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);

		/* make sure partition was mounted before */
		if (part->table.status & F3S_TABLE_NO_ATTACH)
		{
			F3S_TABLE_LEAVE(part);
			return (ENOENT);
		}

		f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
		/* check if an ocb is opened for partition */
		if (attr->part_attr->share_list.next != &attr->part_attr->share_list)
		{
			F3S_TABLE_LEAVE(part);
			return (EBUSY);
		}

		/* detach file system attributes from partition */
		error = f3s_part_detach (part, NULL, attr->part_attr);
f3s_assert(error == EOK);

		F3S_TABLE_LEAVE(part);

	/* else, attach id is for socket */
	} else
	{
#if 0
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* check if there is a pair in array list */
		if (attr->attr.mount->mounted.pair_list.next !=
		    &attr->attr.mount->mounted.pair_list)
		{
			/* detach partition attributes from socket */
			error = f3s_array_detach (array, attr);

		} else
		{
			/* there is nothing mounted */
			error = ENOENT;
		}
#else
		return (ENOTSUP);
#endif
	}

	return (error);
/*}}}1*/
}

int f3s_raw_partinfo (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *	part;
	f3s_array_t *	array;
	f3s_partinfo_t *partinfo;
	f3s_attr_t *	attr;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check if attach id is for socket */
	if (ctp->id == attr->attr.mount->attach_id[F3S_ATTACH_SKT]) return (EINVAL);

	/* get partition info pointer */
	part = ocb->ocb.attr->part;

	/* get array info pointer */
	array = part->array;

	/* set info pointer in message */
	partinfo = (f3s_partinfo_t *) (&msg->o + 1);

	/* read info from partition */
	F3S_TABLE_ENTER(part);
	partinfo->unit_size  = array->unit_size;
	partinfo->total_size = part->unit_num << array->unit_pow2;
	partinfo->spare_size = 0;
	partinfo->head_size  = 0;
	partinfo->pad_size   = 0;
	partinfo->free_size  = 0;
	partinfo->stale_size = 0;
	partinfo->resv_size  = 0;
	F3S_TABLE_LEAVE(part);

	/* return result message */
	F3S_ENDIANISE(ctp, msg->i.dcmd, partinfo);
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*partinfo)));
/*}}}1*/
}

int f3s_raw_statvfs (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *	attr;
	f3s_part_t *	part;
	f3s_array_t *	array;
	struct statvfs *statvfs;
	uint32_t		unit_num;

	/* get attribute pointer */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* get partition info pointer */
		part = attr->part;

		/* get array info pointer */
		array = part->array;

		/* set unit num */
		F3S_TABLE_ENTER(part);
		unit_num = part->unit_num;
		F3S_TABLE_LEAVE(part);

	/* else, attach id is for socket */
	} else
	{
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* set unit num */
		unit_num = array->unit_total;
	}

	/* set statvfs pointer in message and clear structure */
	statvfs = (struct statvfs *) (&msg->o + 1);
	memset (statvfs, 0, sizeof (*statvfs));

	/* read statvfs from partition */
	statvfs->f_bsize   = 1;
	statvfs->f_frsize  = 1;
	statvfs->f_blocks  = unit_num << array->unit_pow2;
	statvfs->f_fsid    = attr->attr.mount->mount.dev;
	statvfs->f_flag    = attr->attr.flags;
	statvfs->f_namemax = F3S_NAME_MAX;
	strcpy (statvfs->f_basetype, F3S_BASE_TYPE);

	/* return result message */
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*statvfs)));
/*}}}1*/
}

int f3s_arrayinfo (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *		attr;
	f3s_part_t *		part = NULL;
	f3s_array_t *		array;
	f3s_arrayinfo_t *	arrayinfo;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* set info pointer in message */
	arrayinfo = (f3s_arrayinfo_t *) (&msg->o + 1);

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{

		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);

		/* get array info pointer */
		array = part->array;

	} else
	{
		/* get array pointer */
		array = attr->attr.mount->array;
	}

	/* read info from array */
	arrayinfo->total_size = array->unit_total << array->unit_pow2;
	arrayinfo->unit_size  = array->unit_size;
	arrayinfo->chip_size  = 1 << array->chip_pow2;

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		F3S_TABLE_LEAVE(part);
	}

	/* return result message */
	F3S_ENDIANISE(ctp, msg->i.dcmd, arrayinfo);
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*arrayinfo)));
/*}}}1*/
}

int f3s_partinfo (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *	part;
	f3s_array_t *	array;
	f3s_partinfo_t *partinfo;
	f3s_attr_t *	attr;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* get partition info pointer */
	part = ocb->ocb.attr->part;

	/* get array info pointer */
	array = part->array;

	/* set info pointer in message */
	partinfo = (f3s_partinfo_t *) (&msg->o + 1);

	/* read info from partition */
	partinfo->unit_size  = array->unit_size;
	partinfo->total_size = part->unit_num << array->unit_pow2;
	partinfo->spare_size = part->table.unit_spare << array->unit_pow2;

	F3S_ALLOC_ENTER(part);

	/* XXX - Is this correct? */
	partinfo->head_size  = (F3S_UNIT_ACTIVE(part) * (sizeof(f3s_unit_info_t) + sizeof(f3s_unit_logi_t))) +
	                       (sizeof(f3s_boot_t))                         +
	                       (part->table.head_num * sizeof(f3s_head_t))  +
	                       (part->table.root_size);

	partinfo->pad_size   = part->table.pad_total;
	partinfo->free_size  = part->table.free_total;
	partinfo->stale_size = part->table.stale_total;
	partinfo->resv_size  = part->table.resv_total;

	F3S_ALLOC_LEAVE(part);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* return result message */
	F3S_ENDIANISE(ctp, msg->i.dcmd, partinfo);
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*partinfo)));
/*}}}1*/
}

int f3s_unitinfo (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *	part;
	f3s_array_t *	array;
	f3s_map_t *		map;
	f3s_unitinfo_t *unitinfo;
	int32_t			phys;

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));

	/* get partition info pointer */
	part = ocb->ocb.attr->part;

	/* get array info pointer */
	array = part->array;

	/* set info pointer in message */
	unitinfo = (f3s_unitinfo_t *) (&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, unitinfo);

	F3S_TABLE_ENTER(part);

	/* get physical unit index */
	phys = part->unit_first + (unitinfo->offset >> array->unit_pow2);

	F3S_UNIT_ENTER(part, phys);

	/* get map pointer for physical unit */

	map = &array->map_vect[phys];

	/* read info from unit */
	unitinfo->unit_size  = array->unit_size;
	unitinfo->erase_count = map->erase_count;

	F3S_UNIT_LEAVE(part, phys);

	/* unlock partition table */
	F3S_TABLE_LEAVE(part);

	/* return result message */
	F3S_ENDIANISE(ctp, msg->i.dcmd, unitinfo);
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*unitinfo)));
/*}}}1*/
}

int f3s_statvfs (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *	attr;
	f3s_part_t *	part;
	struct statvfs *statvfs;

	/* get attribute pointer */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* get partition info pointer */
	part = attr->part;

	/* set statvfs pointer in message and clear structure */
	statvfs = (struct statvfs *)(&msg->o + 1);
	memset (statvfs, 0, sizeof (*statvfs));

	/* read statvfs from partition */
	statvfs->f_bsize  = 1;
	statvfs->f_frsize = 1;

	statvfs->f_blocks = (part->unit_num << part->array->unit_pow2) - (part->table.unit_spare << part->array->unit_pow2);
	F3S_ALLOC_ENTER(part);
	statvfs->f_bfree  = part->table.free_total + part->table.stale_total;
	statvfs->f_files  = part->table.dirent_num;
	F3S_ALLOC_LEAVE(part);

	statvfs->f_bavail = statvfs->f_bfree;
	statvfs->f_ffree  = (statvfs->f_bfree /	(sizeof(f3s_dirent_t) + F3S_NAME_ALIGN(1) +
				sizeof(f3s_stat_t) + (sizeof(f3s_head_t) * 2)));
	statvfs->f_favail = statvfs->f_ffree;
	
	statvfs->f_fsid = attr->attr.mount->mount.dev;
	statvfs->f_flag = attr->attr.mount->mount.flags;
	statvfs->f_namemax = F3S_NAME_MAX;

	strcpy (statvfs->f_basetype, F3S_BASE_TYPE);

	/* return result message */
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*statvfs)));
/*}}}1*/
}

int f3s_mountedon (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *attr;
	f3s_part_t *part;
	char *		name;

	/* get attribute pointer */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* get partition info pointer */
	part = attr->part;

	/* set name pointer in message */
	name = (char *) (&msg->o + 1);

	/* build pathname for raw mountpoint */
	sprintf (name, _PATH_DEV "fs%Xp%X", part->array->index, part->index);

	/* return result message */
	msg->o.ret_val = 0;
	return (_RESMGR_PTR (ctp, &msg->o, sizeof(msg->o) + strlen (name) + 1));
/*}}}1*/
}

int f3s_mountedat (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *attr;
	char *name;

	/* get attribute pointer */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* set name pointer in message */
	name = (char *)(&msg->o + 1);

	f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
	/* copy mount name in message */
	strcpy (name, attr->part_attr->attr.mount->name);

	/* return result message */
	msg->o.ret_val = 0;
	return (_RESMGR_PTR (ctp, &msg->o, sizeof(msg->o) + strlen (name) + 1));
/*}}}1*/
}

int f3s_exit (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	kill (getpid(), SIGTERM);
	return (_RESMGR_NOREPLY);
/*}}}1*/
}

int f3s_reclaim (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_reclaim_t *	reclaim;
	f3s_attr_t *	attr;
	f3s_part_t *	part;
	f3s_sort_t *	sort;
	f3s_map_t *		map;
	int32_t			reclaim_phys;
	int				error = EOK;

	/* get attributes pointer */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* set reclaim pointer in message */
	reclaim = (f3s_reclaim_t *)(&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, reclaim);

	/* get partition info pointer */
	part = attr->part;

	/* Check for a read-only filesystem */
	if (attr->attr.mount->mount.flags & _MOUNT_READONLY){
		return (EROFS);
	}

	F3S_TABLE_ENTER(part);

	/* check if there is something to reclaim */
	if(f3s_check_stale_total(part))
	{
		/* sort units for best allocation and worst reclaim order */
		f3s_table_sort (part, -1);

		/* get sort pointer */
		sort = &part->table.sort_vect[F3S_UNIT_ACTIVE (part) - 1];

		/* while there is stale space in partition */
		while ((sort >= part->table.sort_vect) && f3s_check_free_total(part, reclaim->limit))
		{
			F3S_LOGI_ENTER(part, sort->logi);
			F3S_UNIT_ENTER(part, sort->phys);

			/* get unit map pointer */
			f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[sort->phys].thread == pthread_self()));
			map = &part->array->map_vect[sort->phys];

			/* check if there is stale space on unit */
			if (map->stale_size)
			{
				/* foreground reclaim the unit */
				reclaim_phys = f3s_frgd_reclaim (part, sort->phys);
				if ((reclaim_phys < 0) ||
				    (reclaim_phys < part->unit_first) ||
				    (reclaim_phys > (part->unit_first + part->unit_num)))
				{
					F3S_UNIT_LEAVE(part, sort->phys);
					F3S_LOGI_LEAVE(part, sort->logi);
					error = EIO;
					break;
				}
				F3S_UNIT_LEAVE(part, reclaim_phys);
				F3S_LOGI_LEAVE(part, sort->logi);

				/* reflect physical unit swap in sort vector */
				sort->phys = reclaim_phys;

				/* else there is no stale space */
			} else
			{
				F3S_UNIT_LEAVE(part, sort->phys);
				F3S_LOGI_LEAVE(part, sort->logi);
			}

			/* decrement sort pointer */
			sort--;
		}
	}
	F3S_TABLE_LEAVE(part);
	return (error);
/*}}}1*/
}

int f3s_raw_lock (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *		part;
	f3s_array_t *		array;
	f3s_attr_t *		attr;
	f3s_unitlock_t *	lock;
	struct _client_info	info;
	int32_t				phys;
	uint32_t			unit_first;
	uint32_t			unit_num;
	uint32_t			end;
	int					error;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check for privileges */
	error = iofunc_client_info (ctp, 0, &info);
	if (error) return (error);

	error = iofunc_check_access (ctp, &attr->attr, S_IWRITE, &info);
	if (error) return (error);

	/* set lock pointer in message */
	lock = (f3s_unitlock_t *) (&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, lock);

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* get partition info pointer */
		part = attr->part;
		F3S_TABLE_ENTER(part);

		/* get array info pointer */
		array = part->array;

		/* calculate unit first and unit num */
		unit_first = part->unit_first + (lock->offset >> array->unit_pow2);
		unit_num = min(part->unit_num - (lock->offset >> array->unit_pow2),
		               lock->limit >> array->unit_pow2);

		/* sanity checks */
		end = part->unit_first + part->unit_num;
		if ((unit_first < part->unit_first) ||
		    (unit_first >= end) || ((unit_first + unit_num) > end))
		{
			F3S_TABLE_LEAVE(part);
			return (ERANGE);
		}

		/* prevent lock operations on mounted partitions */
		if (!(part->table.status & F3S_TABLE_NO_ATTACH))
		{
			F3S_TABLE_LEAVE(part);
			return (EBUSY);
		}

	/* else, attach id is for socket */
	} else
	{
		part = NULL;

		/* get array info pointer */
		array = attr->attr.mount->array;

		/* set unit first and unit num */
		unit_first = lock->offset >> array->unit_pow2;
		unit_num = min(array->unit_total - (lock->offset >> array->unit_pow2),
		               lock->limit >> array->unit_pow2);

		/* sanity checks */
		end = array->unit_total;
		if ((unit_first >= end) || ((unit_first + unit_num) > end))
		{
			return (ERANGE);
		}

		/* NOTE: Anything goes on the socket mountpoint */
	}

	/* for all units in partition */
	for (phys = unit_first; phys < (unit_first + unit_num); phys++)
	{
		/* call flash lock function */
		F3S_RAW_ENTER(array, phys);
		error = f3s_flash_lock (array, phys);
		F3S_RAW_LEAVE(array, phys);
		if (error) break;
	}

	if (part) F3S_TABLE_LEAVE(part);
	return (error);
/*}}}1*/
}

int f3s_raw_unlock (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_part_t *		part;
	f3s_array_t *		array;
	f3s_attr_t *		attr;
	f3s_unitunlock_t *	unlock;
	struct _client_info	info;
	int32_t				phys;
	uint32_t			unit_first;
	uint32_t			unit_num;
	uint32_t			end;
	int					error;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check for privileges */
	error = iofunc_client_info (ctp, 0, &info);
	if (error) return (error);

	error = iofunc_check_access (ctp, &attr->attr, S_IWRITE, &info);
	if (error) return (error);

	/* set unlock pointer in message */
	unlock = (f3s_unitunlock_t *) (&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, unlock);

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);

		/* get array info pointer */
		array = part->array;

		/* calculate unit first and unit num */
		unit_first = part->unit_first + (unlock->offset >> array->unit_pow2);
		unit_num = min(part->unit_num - (unlock->offset >> array->unit_pow2),
		               unlock->limit >> array->unit_pow2);

		/* sanity checks */
		end = part->unit_first + part->unit_num;
		if ((unit_first < part->unit_first) ||
		    (unit_first >= end) || ((unit_first + unit_num) > end))
		{
			F3S_TABLE_LEAVE(part);
			return (ERANGE);
		}

		/* prevent lock operations on mounted partitions */
		if (!(part->table.status & F3S_TABLE_NO_ATTACH))
		{
			F3S_TABLE_LEAVE(part);
			return (EBUSY);
		}

	/* else, attach id is for socket */
	} else
	{
		part = NULL;

		/* get array info pointer */
		array = attr->attr.mount->array;

		/* set unit first and unit num */
		unit_first = unlock->offset >> array->unit_pow2;
		unit_num = min(array->unit_total - (unlock->offset >> array->unit_pow2),
		               unlock->limit >> array->unit_pow2);

		/* sanity checks */
		end = array->unit_total;
		if ((unit_first >= end) || ((unit_first + unit_num) > end))
		{
			return (ERANGE);
		}

		/* NOTE: Anything goes on the socket mountpoint */
	}

	/* for all units in partition */
	for (phys = unit_first; phys < (unit_first + unit_num); phys++)
	{
		/* call flash unlock function */
		F3S_RAW_ENTER(array, phys);
		error = f3s_flash_unlock (array, phys);
		F3S_RAW_LEAVE(array, phys);
		if (error) break;
	}

	if (part) F3S_TABLE_LEAVE(part);
	return (error);
/*}}}1*/
}

int f3s_raw_unlockall (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_array_t *		array;
	f3s_attr_t *		attr;
	f3s_arrayunlock_t *	unlockall;
	struct _client_info	info;
	int					error;

	/* get attributes */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* check for privileges */
	error = iofunc_client_info (ctp, 0, &info);
	if (error) return (error);

	error = iofunc_check_access (ctp, &attr->attr, S_IWRITE, &info);
	if (error) return (error);

	/* set unlock pointer in message */
	unlockall = (f3s_arrayunlock_t *) (&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, unlockall);

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* Only works on the socket */
		return (ENOTSUP);

	/* else, attach id is for socket */
	} else
	{
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* NOTE: Anything goes on the socket mountpoint */
	}

	/* call flash unlock function */
	error = f3s_flash_unlockall (array);
	return (error);
/*}}}1*/
}

int f3s_reclaimctl (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
#ifdef ENABLE_RECLAIMCTL
	f3s_part_t *part;
	f3s_reclaimctl_t *reclaimctl;
	f3s_reclaimctl_t tmp_reclaimctl;

	/* Get partition pointer */
	part = ocb->ocb.attr->part;

	reclaimctl = (f3s_reclaimctl_t*)(&msg->o + 1);
	F3S_ENDIANISE(ctp, msg->i.dcmd, reclaimctl);
	
	F3S_TABLE_ENTER (part);
	
	if (part->table.status & F3S_TABLE_NO_ATTACH){
		fprintf(stderr, "DCMD_RECLAIMCTL sent to non-filesystem partition\n");
		F3S_TABLE_LEAVE(part);
		return ENOTSUP;
	}

	tmp_reclaimctl.super_count = part->table.reclaim.ethreshold;
	tmp_reclaimctl.stale_percent = part->table.reclaim.etrigger;
	tmp_reclaimctl.reclaim_enable = part->table.reclaim.enable;
	
	part->table.reclaim.ethreshold = (reclaimctl->super_count == -1)? part->table.reclaim.ethreshold:reclaimctl->super_count;
	part->table.reclaim.etrigger = (reclaimctl->stale_percent == -1)? part->table.reclaim.etrigger:reclaimctl->stale_percent;
	part->table.reclaim.enable = (reclaimctl->reclaim_enable == -1)? part->table.reclaim.enable:reclaimctl->reclaim_enable;

	/* Fill in the old values, to return from the devctl */
	reclaimctl->super_count = tmp_reclaimctl.super_count;
	reclaimctl->stale_percent = tmp_reclaimctl.stale_percent;
	reclaimctl->reclaim_enable = tmp_reclaimctl.reclaim_enable;

	/* check if there is more stale space than free space trigger */
	F3S_ALLOC_ENTER(part);
	if (part->table.reclaim.enable && part->table.free_total &&
	    ((uint64_t)part->table.stale_total * 100 / (uint64_t)part->table.free_total) > part->table.reclaim.etrigger)
	{
		/* signal reclaim background */
		f3s_bkgd_signal (part);
	}
	F3S_ALLOC_LEAVE(part);
	
	F3S_TABLE_LEAVE (part);

	F3S_ENDIANISE(ctp, msg->i.dcmd, reclaimctl);
	msg->o.ret_val = 0;
	return (_RESMGR_PTR (ctp, &msg->o, sizeof(msg->o) + sizeof (*reclaimctl)));
#else
	return ENOSYS;
#endif /* ENABLE_RECLAIMCTL */
/*}}}1*/
}

int f3s_geoinfo (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
	f3s_attr_t *		attr;
	f3s_part_t *		part = NULL;
	f3s_array_t *		array;
	f3s_geoinfo_t *		geoinfo;
	int					max_geos;
	int					part_locked = 0;

	/* get attributes */
	attr = ocb->ocb.attr;
	if (NULL == attr){
		return EINVAL;
	}
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* set info pointer in message */
	geoinfo = (f3s_geoinfo_t *) (&msg->o + 1);
	if (NULL == geoinfo){
		return EINVAL;
	}

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{

		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);
		part_locked = 1;

		/* get array info pointer */
		array = part->array;

	} else
	{
		/* get array pointer */
		array = attr->attr.mount->array;
	}

	/* read info from dbase */
	geoinfo->status = 0;
	geoinfo->chipwidth = array->dbase.chip_width;
	geoinfo->interleave = array->dbase.chip_inter;
	geoinfo->base = array->access_vect[0].socket.address;
	geoinfo->size = array->access_vect[0].socket.window_size;

	max_geos = sizeof(geoinfo->geo)/sizeof(*geoinfo->geo);
	geoinfo->num_geo = min(array->dbase.geo_num, max_geos);

	memcpy (geoinfo->geo, array->dbase.geo_vect, geoinfo->num_geo * sizeof (*geoinfo->geo));

	
	/* check if attach id is not for socket */
	if (part_locked)
	{
		F3S_TABLE_LEAVE(part);
		part_locked = 0;
	}

	/* return result message */
	F3S_ENDIANISE(ctp, msg->i.dcmd, geoinfo);
	msg->o.ret_val = 0;
	return (_RESMGR_PTR(ctp, &msg->o, sizeof(msg->o) + sizeof(*geoinfo)));
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/devctl.c $ $Rev: 580669 $" );
