/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_inline.h
**
** Description:
**
** This file contains the inline macro definitions for the f3s
** resource manager for Neutrino
**
** Ident: $Id: f3s_inline.h 580669 2011-09-01 15:18:50Z builder@qnx.com $
*/

/*
** Include Loop Prevention
*/

#ifndef __F3S_INLINE_H_INCLUDED
#define __F3S_INLINE_H_INCLUDED

#include <sys/neutrino.h>
#include <pthread.h>

/*
** Inline Macro Definitions
*/

#ifdef NDEBUG

#define f3s_assert(FLAG)
#define f3s_lock_assert(f)

#else // NDEBUG

/*
** Function: f3s_assert

void f3s_assert(int32_t flag);
*/
#define f3s_assert(FLAG) \
				do { \
					/* check if flag is not true */ \
					if (!(FLAG)) \
					{ \
						printf("(devf  t%d::%s:%d) ASSERT (%s)\n", pthread_self(), __func__, __LINE__, #FLAG); \
						DebugBreak(); \
					} \
				} while(0)

/*
** Function: f3s_lock_assert

void f3s_lock_assert(int32_t flag);
*/
#define f3s_lock_assert(f)		f3s_assert(f)

#endif // NDEBUG

/*
* Function: f3s_list_init
* 
* void f3s_list_init(f3s_list_t *list);
*/
#define f3s_list_init(LIST_PTR) \
			(LIST_PTR)->prev=(LIST_PTR)->next=(LIST_PTR)

/*
* Function: f3s_extptr_phys
* 
* int32_t f3s_extptr_phys(f3s_part_t *part, f3s_extptr_t extptr);
*/
#ifdef NDEBUG	// Lock Debug ?
#define f3s_extptr_phys(PART_PTR, EXTPTR) \
			((PART_PTR)->table.logi_vect[(EXTPTR).logi_unit])
#else	/* NDEBUG */
extern uint16_t  _f3s_extptr_phys();
/* the debug version uses a function that does some additional lock checking */
#define f3s_extptr_phys(PART_PTR, EXTPTR)	_f3s_extptr_phys(PART_PTR, EXTPTR)
#endif	/* NDEBUG */

/*
* Function: f3s_desc_init
* 
* void f3s_desc_init(f3s_desc_t *desc,f3s_extptr_t first, uint32_t mask)
*/
#define f3s_desc_init(DESC_PTR, FIRST, MASK) \
			do { \
				/* erase descriptor status */ \
				(DESC_PTR)->status = F3S_DESC_INIT; \
				/* put root first pointer into descriptor */ \
				(DESC_PTR)->first = (FIRST); \
				/* rewind descriptor */ \
				f3s_desc_rewind(DESC_PTR); \
				/* setup descriptor status with mask*/ \
				(DESC_PTR)->status &= (MASK); \
				(DESC_PTR)->last_offset = 0; \
			} while(0)

/*
* Function: f3s_desc_curr
* 
* int32_t f3s_desc_curr(f3s_desc_t *desc)
*/
#define f3s_desc_curr(DESC_PTR)		((DESC_PTR)->curr_offset)

/*
* Function: f3s_desc_end
* 
* int32_t f3s_desc_end(f3s_desc_t *desc)
*/
#define f3s_desc_end(DESC_PTR)	(!((DESC_PTR)->status&F3S_DESC_NO_LAST))

/*
* Function: f3s_desc_last
* 
* void f3s_desc_last(f3s_desc_t *desc, int32_t text_size)
*/
#define f3s_desc_last(DESC_PTR, TEXT_SIZE) \
			do { \
				/* set last extent pointer to current extent pointer */ \
				(DESC_PTR)->last = desc->curr; \
				/* set last extent offset to current extent offset */ \
				(DESC_PTR)->last_offset = desc->curr_offset; \
				/* keep size of last extent */ \
				(DESC_PTR)->last_size = (TEXT_SIZE); \
				/* set last extent pointer valid flag in desc status */ \
				(DESC_PTR)->status &= ~F3S_DESC_NO_LAST; \
			} while(0)

/*
* Function: f3s_desc_rewind
* 
* void f3s_desc_rewind(f3s_desc_t *desc)
*/
#define f3s_desc_rewind(DESC_PTR) \
			do { \
				/* set desc current extent pointer to first in file */ \
				(DESC_PTR)->curr = (DESC_PTR)->first; \
				/* clear current extent offset */ \
				(DESC_PTR)->curr_offset = 0; \
			} while(0)

/*
* Function: f3s_desc_size
* 
* int32_t f3s_desc_size(f3s_desc_t *desc)
*/
#define f3s_desc_size(DESC_PTR) \
			((DESC_PTR)->last_offset + (DESC_PTR)->last_size)

/*
* Function: f3s_desc_step
* 
* void f3s_desc_step(f3s_desc_t *desc, f3s_extptr_t curr, int32_t text_size)
*/
#define f3s_desc_step(DESC_PTR, CURR, TEXT_SIZE) \
			do { \
				/* set current extent pointer */ \
				(DESC_PTR)->curr = (CURR); \
				/* increment current extent offset */ \
				(DESC_PTR)->curr_offset += (TEXT_SIZE); \
			} while(0)

/*
* Function: f3s_malloc
* 
* void *f3s_malloc(unsigned size)
*/
#define f3s_malloc(NEW_SIZE) \
			/* use small malloc */ \
			malloc(NEW_SIZE), \
			(f3s.verbose>=4) ? \
				printf("(devf  t%d::%s:%d) malloc(%d)\n", pthread_self(), __func__, __LINE__, (NEW_SIZE)) : 0

/*
* Function: f3s_calloc
* 
* void *f3s_calloc(unsigned size)
*/
#define f3s_calloc(NEW_SIZE) \
			/* use small calloc */ \
			calloc(1, (NEW_SIZE)), \
			(f3s.verbose>=4) ? \
				printf("(devf  t%d::%s:%d) calloc(1,  %d)\n", pthread_self(), __func__, __LINE__, (NEW_SIZE)) : 0

/*
* Function: f3s_realloc
* 
* void *f3s_realloc(void *data, unsigned old_size, unsigned new_size)
*/

#define f3s_realloc(OLD_DATA, OLD_SIZE, NEW_SIZE) \
			/* use small realloc */ \
			realloc((OLD_DATA),/* (OLD_SIZE),*/ (NEW_SIZE)), \
			(f3s.verbose>=4) ? \
				printf("(devf  t%d::%s:%d) realloc(0x%p, %d to %d)\n", \
							pthread_self(), __func__, __LINE__, (OLD_DATA), (OLD_SIZE), (NEW_SIZE)) : 0

/*
* Function: f3s_free
* 
* void *f3s_free(unsigned size)
*/
#ifdef NDEBUG
#define f3s_free(DATA, OLD_SIZE) \
			do { \
				if (DATA) \
				{ \
					free((DATA));  \
					if (f3s.verbose>=4) \
						printf("(devf  t%d::%s:%d) free() %d bytes @ 0x%p)\n", \
									pthread_self(), __func__, __LINE__, (OLD_SIZE), (DATA)); \
					(DATA) = NULL; \
				} \
			} while(0)
#else
/* debug version - zero memory, NULL pointer */
#define f3s_free(DATA, OLD_SIZE) \
			do { \
				if (DATA) \
				{ \
					memset((DATA), 0, (OLD_SIZE)); \
					free((DATA));  \
					if (f3s.verbose>=4) \
						printf("(devf  t%d::%s:%d) free() %d bytes @ 0x%p)\n", \
									pthread_self(), __func__, __LINE__, (OLD_SIZE), (DATA)); \
					(DATA) = NULL; \
				} \
				else if (f3s.verbose>=4) \
					printf("(devf  t%d::%s:%d) Trying to free %d bytes at (null)\n", \
								pthread_self(), __func__, __LINE__, (OLD_SIZE)); \
			} while(0)
#endif	/* NDEBUG */

/* by default, verify functions are used on the debug load */
//#define INCLUDE_VERIFY_FUNCS		/* uncomment if desired */

#if (defined(INCLUDE_VERIFY_FUNCS) && !defined(NDEBUG))
/*
* Function: f3s_verify_counters
* 
* void f3s_verify_counters(f3s_part_t *part, int32_t logi, uint32_t extra_text)
*/
#define f3s_verify_counters(PART, LOGI, EXTRA_TEXT) \
			do { \
				F3S_TABLE_ENTER((PART)); \
				_f3s_verify_counters((PART), (LOGI), (EXTRA_TEXT)); \
				F3S_TABLE_LEAVE((PART)); \
			} while(0)
/*
* Function: f3s_verify_dir
* 
* void f3s_verify_dir(f3s_part_t *part, int32_t logi, int32_t index)
*/
#define f3s_verify_dir(PART, LOGI, INDEX) \
			_f3s_verify_dir((PART), (LOGI), (INDEX))

/*
* Function: f3s_verify_headers
* 
* void f3s_verify_headers(f3s_part_t *part, int32_t include_phys)
*/
#define f3s_verify_headers(PART, INC) \
			_f3s_verify_headers((PART), (INC))

#else	/* INCLUDE_VERIFY_FUNCS */

#define f3s_verify_counters(PART, LOGI, EXTRA_TEXT)
#define f3s_verify_dir(PART, LOGI, INDEX)
#define f3s_verify_headers(PART, INC)

#endif	/* INCLUDE_VERIFY_FUNCS */

#endif /* __F3S_INLINE_H_INCLUDED */

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/f3s_inline.h $ $Rev: 580669 $" )
