/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_iofunc.h
**
** Description:
**
** This file contains the iofunc definitions for the f3s resource manager for
** Neutrino
**
** Ident: $Id: f3s_iofunc.h 580669 2011-09-01 15:18:50Z builder@qnx.com $
*/

/*
** Include Loop Prevention
*/

#ifndef __F3S_IOFUNC_H_INCLUDED
#define __F3S_IOFUNC_H_INCLUDED

/*
** Option Defines
*/

#define _LARGEFILE64_SOURCE  1   /* add 64 bit stuff to namespace */
#define _IOFUNC_OFFSET_BITS  32  /* iofunc offsets are 32 bits */

/*
** Structure Definitions
*/

/* Mount Structure Definition */

typedef struct f3s_mount_s
{
  iofunc_mount_t mount;                /* resource manager mount structure */
  f3s_array_t *array;                  /* chip array description */
  union
  {
    f3s_list_t pair_list;              /* attr pair list for array */
    struct f3s_attr_s *raw_attr;       /* raw attr for f3s */
    struct f3s_attr_s *skt_attr;       /* skt attr for raw */
  }
  mounted;
  int32_t attach_id[F3S_ATTACH_NUM];   /* attach id vector for resmgr */
  char *name;	                       /* name pointer */
}
f3s_mount_t;

/* Attributes Definition */

typedef struct f3s_attr_s
{
  iofunc_attr_t attr;            /* resource manager attribute structure */
  struct f3s_attr_s *part_attr;  /* partition attributes */
  f3s_part_t *part;              /* partition info */
  f3s_list_t share_list;         /* share list of ocbs linked to attr */
  f3s_extptr_t parent_extptr;    /* directory first extent pointer */
  f3s_resv_t *resv_ptr;          /* reservation pointer for buffer */
  uint32_t depth;                /* depth of directory entry */
  uint32_t buffer_pos;           /* position in append buffer */
  uint8_t *buffer;               /* buffer for appends */
  uint8_t  pending_delete;	/* boolean indicating whether ot not the object should be deleted on last close.
													This flag is set in f3s_unlink() if any open()'s still exist on the object. It is cleared
													in f3s_ocb_attach() as soon as an ocb is added to the objects share_list hence
													creating a reference to the object. It is tested in f3s_close() to see if the deletion
													is necessary */
}
f3s_attr_t;

/* Open Control Block Definition */

typedef struct f3s_ocb_s
{
  iofunc_ocb_t ocb;        /* resource manager attribute structure */
  f3s_desc_t file_desc;    /* file descriptor structure cache */
#if 0
  int32_t buffer_offset;   /* offset of decompress buffer */
  int32_t buffer_pos;      /* position in decompress buffer */
  int32_t buffer_rewind;   /* rewind offset to get to previous buffer */
  uint16_t buffer_size;    /* size of decompress buffer */
  uint16_t buffer_max;     /* maximum size of buffer */
  uint8_t *buffer;         /* buffer for decompress */
#endif
}
f3s_ocb_t;

/*
** Function Prototypes
*/

void f3s_mount_init(f3s_mount_t *mount,
                    dev_t dev,
                    iofunc_funcs_t *funcs);

void f3s_attr_init(f3s_attr_t *attr,
                   mode_t mode,
                   f3s_attr_t *dattr,
                   f3s_desc_t *desc,
                   f3s_stat_t *stat,
                   struct _client_info *info);

int f3s_attr_sync(f3s_ocb_t *ocb,
                  f3s_attr_t *attr);

void f3s_attr_inval(f3s_ocb_t *ocb,
                    f3s_attr_t *attr,
                    uint32_t last,
                    int32_t offset,
                    int32_t adjust,
                    uint32_t rewind);

void f3s_attr_resv(f3s_attr_t *attr,
                   int32_t size);

int f3s_sync_meta(f3s_ocb_t *ocb,
                  f3s_attr_t *attr);

int f3s_dir_lock(resmgr_context_t *ctp,
                 void *reserved,
                 f3s_ocb_t *ocb);

int f3s_dir_unlock(resmgr_context_t *ctp,
                   void *reserved,
                   f3s_ocb_t *ocb);

int f3s_file_lock(resmgr_context_t *ctp,
                 void *reserved,
                 f3s_ocb_t *ocb);

int f3s_file_unlock(resmgr_context_t *ctp,
                   void *reserved,
                   f3s_ocb_t *ocb);

int f3s_rawio_lock (resmgr_context_t * ctp,
					void *reserved,
					f3s_ocb_t * ocb);
					
int f3s_rawio_unlock (resmgr_context_t * ctp,
					void *reserved,
					f3s_ocb_t * ocb);

f3s_ocb_t *f3s_ocb_calloc(resmgr_context_t *ctp,
                          f3s_attr_t *attr);

void f3s_ocb_free(f3s_ocb_t *ocb);

int f3s_ocb_attach(resmgr_context_t *ctp,
                   f3s_ocb_t *ocb,
                   f3s_desc_t *file_desc,
                   f3s_attr_t *file_attr);

int f3s_ocb_detach(resmgr_context_t *ctp,
                   f3s_ocb_t *ocb,
                   f3s_attr_t *attr);

int f3s_namei(resmgr_context_t *ctp,
              io_open_t *msg,
              f3s_attr_t *part_attr,
              f3s_attr_t *dir_attr,
              f3s_desc_t *dir_desc,
              struct _client_info *info);

int f3s_create(resmgr_context_t *ctp,
               io_open_t *msg,
               f3s_attr_t *dir_attr,
               f3s_attr_t *file_attr,
               f3s_desc_t *dir_desc,
               f3s_desc_t *file_desc,
               f3s_stat_t *file_stat,
               uint8_t moves);

int f3s_open(resmgr_context_t *ctp,
             io_open_t *msg,
             f3s_attr_t *part_attr,
             void *extra);

int f3s_unlink(resmgr_context_t *ctp,
               io_unlink_t *msg,
               f3s_attr_t *part_attr,
               void *extra);

int f3s_rename(resmgr_context_t *ctp,
               io_rename_t *msg,
               f3s_attr_t *part_attr,
               io_rename_extra_t *extra);

int f3s_mknod(resmgr_context_t *ctp,
              io_mknod_t *msg,
              f3s_attr_t *part_attr,
              void *extra);

int f3s_link(resmgr_context_t *ctp,
             io_link_t *msg,
             f3s_attr_t *part_attr,
             io_link_extra_t *extra);

int f3s_readlink(resmgr_context_t *ctp,
                 io_readlink_t *msg,
                 f3s_attr_t *part_attr,
                 void *extra);

int f3s_mnt_mount(resmgr_context_t *ctp,
                  io_mount_t *msg,
                  f3s_attr_t *part_attr,
                  io_mount_extra_t *extra);

int f3s_raw_unmount(resmgr_context_t *ctp,
                    io_mount_t *msg,
                    f3s_attr_t *part_attr,
                    io_mount_extra_t *extra);

int f3s_umount(resmgr_context_t *ctp,
               io_mount_t *msg,
               f3s_attr_t *part_attr,
               io_mount_extra_t *extra);

int f3s_link_resolve(resmgr_context_t *ctp,
                     io_readlink_t *msg,
                     f3s_attr_t *part_attr,
                     f3s_extptr_t first,
                     int16_t length,
                     int *ret_val);

int f3s_openfd(resmgr_context_t *ctp,
               io_openfd_t *msg,
               f3s_ocb_t *ocb);

int f3s_read(resmgr_context_t *ctp,
             io_read_t *msg,
             f3s_ocb_t *ocb);

int f3s_readdir(resmgr_context_t *ctp,
                io_read_t *msg,
                f3s_ocb_t *ocb);

int f3s_raw_read(resmgr_context_t *ctp,
                 io_read_t *msg,
                 f3s_ocb_t *ocb);

int f3s_cmp_read(resmgr_context_t *ctp,
                 io_read_t *msg,
                 f3s_ocb_t *ocb);

int f3s_write(resmgr_context_t *ctp,
              io_write_t *msg,
              f3s_ocb_t *ocb);

int f3s_raw_write(resmgr_context_t *ctp,
                  io_write_t *msg,
                  f3s_ocb_t *ocb);

int f3s_append(resmgr_context_t *ctp,
               io_write_t *msg,
               f3s_ocb_t *ocb);

int f3s_cmp_append(resmgr_context_t *ctp,
                   io_write_t *msg,
                   f3s_ocb_t *ocb);

int f3s_space(resmgr_context_t *ctp,
              io_space_t *msg,
              f3s_ocb_t *ocb);

int f3s_close(resmgr_context_t *ctp,
              f3s_ocb_t *ocb);

int f3s_close_ocb(resmgr_context_t *ctp,
                  void *reserved,
                  f3s_ocb_t *ocb);

int f3s_raw_close_ocb(resmgr_context_t *ctp,
                      void *reserved,
                      f3s_ocb_t *ocb);

int f3s_stat(resmgr_context_t *ctp,
             io_stat_t *msg,
             f3s_ocb_t *ocb);

int f3s_statdir(resmgr_context_t *ctp,
                io_stat_t *msg,
                f3s_ocb_t *ocb);

int f3s_cmp_stat(resmgr_context_t *ctp,
                 io_stat_t *msg,
                 f3s_ocb_t *ocb);

int f3s_devctl(resmgr_context_t *ctp,
               io_devctl_t *msg,
               f3s_ocb_t *ocb);

int f3s_raw_devctl(resmgr_context_t *ctp,
                   io_devctl_t *msg,
                   f3s_ocb_t *ocb);

int f3s_pathconf(resmgr_context_t *ctp,
                 io_pathconf_t *msg,
                 f3s_ocb_t *ocb);

int f3s_lseek(resmgr_context_t *ctp,
              io_lseek_t *msg,
              f3s_ocb_t *ocb);

int f3s_cmp_lseek(resmgr_context_t *ctp,
                  io_lseek_t *msg,
                  f3s_ocb_t *ocb);

int f3s_sync(resmgr_context_t *ctp,
             io_sync_t *msg,
             f3s_ocb_t *ocb);

int f3s_raw_mount(resmgr_context_t *ctp,
                  io_devctl_t *msg,
                  f3s_ocb_t *ocb);

int f3s_raw_umount(resmgr_context_t *ctp,
                   void *reserved,
                   f3s_ocb_t *ocb);

int f3s_reclaim(resmgr_context_t *ctp,
                io_devctl_t *msg,
                f3s_ocb_t *ocb);

int f3s_arrayinfo(resmgr_context_t *ctp,
                  io_devctl_t *msg,
                  f3s_ocb_t *ocb);

int f3s_partinfo(resmgr_context_t *ctp,
                 io_devctl_t *msg,
                 f3s_ocb_t *ocb);

int f3s_raw_partinfo(resmgr_context_t *ctp,
                     io_devctl_t *msg,
                     f3s_ocb_t *ocb);

int f3s_unitinfo(resmgr_context_t *ctp,
                 io_devctl_t *msg,
                 f3s_ocb_t *ocb);

int f3s_exit(resmgr_context_t *ctp,
             io_devctl_t *msg,
             f3s_ocb_t *ocb);

int f3s_statvfs(resmgr_context_t *ctp,
                io_devctl_t *msg,
                f3s_ocb_t *ocb);

int f3s_raw_statvfs(resmgr_context_t *ctp,
                    io_devctl_t *msg,
                    f3s_ocb_t *ocb);

int f3s_mountedat(resmgr_context_t *ctp,
                  io_devctl_t *msg,
                  f3s_ocb_t *ocb);

int f3s_mountedon(resmgr_context_t *ctp,
                  io_devctl_t *msg,
                  f3s_ocb_t *ocb);

int f3s_raw_mountedby(resmgr_context_t *ctp,
                      io_devctl_t *msg,
                      f3s_ocb_t *ocb);

int f3s_raw_mountedon(resmgr_context_t *ctp,
                      io_devctl_t *msg,
                      f3s_ocb_t *ocb);

int f3s_raw_erase(resmgr_context_t *ctp,
                  io_devctl_t *msg,
                  f3s_ocb_t *ocb);

int f3s_raw_format(resmgr_context_t *ctp,
                   io_devctl_t *msg,
                   f3s_ocb_t *ocb);

int f3s_getcmp(resmgr_context_t *ctp,
               io_devctl_t *msg,
               f3s_ocb_t *ocb);

int f3s_setcmp(resmgr_context_t *ctp,
               io_devctl_t *msg,
               f3s_ocb_t *ocb);

int f3s_clrcmp(resmgr_context_t *ctp,
               io_devctl_t *msg,
               f3s_ocb_t *ocb);

int f3s_raw_lock(resmgr_context_t *ctp,
                 io_devctl_t *msg,
                 f3s_ocb_t *ocb);

int f3s_raw_unlock(resmgr_context_t *ctp,
                   io_devctl_t *msg,
                   f3s_ocb_t *ocb);

int f3s_raw_unlockall(resmgr_context_t *ctp,
                      io_devctl_t *msg,
                      f3s_ocb_t *ocb);

int f3s_raw_lockdown(resmgr_context_t *ctp,
					  io_devctl_t *msg,
					  f3s_ocb_t *ocb);

int f3s_raw_unlockdown(resmgr_context_t *ctp,
					  io_devctl_t *msg,
					  f3s_ocb_t *ocb);

int f3s_reclaimctl (resmgr_context_t *ctp,
					  io_devctl_t *msg,
					  f3s_ocb_t *ocb);
int f3s_geoinfo(resmgr_context_t *ctp,
	                  io_devctl_t *msg,
    	              f3s_ocb_t *ocb);
#endif /* __F3S_IOFUNC_H_INCLUDED */

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/f3s_iofunc.h $ $Rev: 580669 $" )
