/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */



/*
** File: f3s_log.h
**
** Description:
**
** This file contains the log macro definitions for the f3s
** resource manager for Neutrino
**
** Ident: $Id: f3s_log.h 580669 2011-09-01 15:18:50Z builder@qnx.com $
*/

/*
** Include Loop Prevention
*/

#ifndef __F3S_LOG_H_INCLUDED
#define __F3S_LOG_H_INCLUDED

/*
** Log Macro Definitions
*/

#if 0
//#ifdef NDEBUG

#define f3s_log_retry(MAJOR, MINOR, LOGI, PHYS)
#define f3s_log_reclaim(S, MAJOR, MINOR, LOGI, PHYS, SPARE, STALE)
#define f3s_log_split(MAJOR, MINOR, LOGI, INDEX, ALLOC_SIZE, SPLIT_SIZE)
#define f3s_log_bad(MAJOR, MINOR, PHYS, INDEX, ALLOC_SIZE)
#define f3s_log_retire(MAJOR, MINOR, PHYS)
#define f3s_log_spare(MAJOR, MINOR, PHYS)
#define f3s_log_socket(MAJOR, NAME_PTR)
#define f3s_log_array(MAJOR, UNIT_NUM, UNIT_SIZE, NAME_PTR)
#define f3s_log_boot(MAJOR, MINOR, PHYS, UNIT_NUM)
#define f3s_log_swap(MAJOR, MINOR)
#define f3s_log_partial(MAJOR, MINOR, PHYS)
#define f3s_log_multi(MAJOR, MINOR, LOGI, PHYS, INDEX, MULTI_NUM)
#define f3s_log_dangle(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_bogus(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_dirty(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_damage(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_virgin(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_sticky(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_slip(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_outside(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_junk(MAJOR, MINOR, LOGI, PHYS, INDEX)
#define f3s_log_corrupt(MAJOR, MINOR, UNIT_NUM)
#define f3s_log_raw(MAJOR, MINOR, UNIT_NUM)
#define f3s_log_head_repair(MAJOR, MINOR, LOGI, PHYS, SPARE, STALE)

#else // NDEBUG

#define F3S_SLOG(MAJOR) _SLOG_SETCODE(_SLOGC_FS_FFS,(MAJOR))
#define stdout_log 0 == f3s.log_method || 2 == f3s.log_method
#define slog_log 1 == f3s.log_method || 2 == f3s.log_method

/*
 * Defines for verbosity thresholds.
 */
#define F3S_LOG_STARTUP_FATAL  0
#define F3S_LOG_STARTUP_ERROR  1
#define F3S_LOG_STARTUP_INFO   2
#define F3S_LOG_RUNTIME_ERROR  2
#define F3S_LOG_RUNTIME_INFO   3
#define F3S_LOG_RUNTIME_DEBUG  4
#define F3S_LOG_RUNTIME_DEBUG1 5 

/*
** Function: f3s_log_retry

void f3s_log_retry(uint32_t major,
                   uint32_t minor,
                   uint32_t logi,
                   int32_t phys)
*/

#define f3s_log_retry(MAJOR, MINOR, LOGI, PHYS) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X retry L[%02X] P[%02X]", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X retry L[%02X] P[%02X]", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS));\
				}\
			}

/*
** Function: f3s_log_reclaim

void f3s_log_reclaim(char *id_string,
					 uint32_t major,
                     uint32_t minor,
                     uint32_t logi,
                     int32_t phys,
                     uint32_t spare,
                     uint32_t stale)
*/

#define f3s_log_reclaim(S, MAJOR, MINOR, LOGI, PHYS, SPARE, STALE) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X " S " reclaim L[%02X] P[%02X] => P[%02X] * %06X\n", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS), (SPARE), (STALE));\
				}\
				if (slog_log){\
				slogf(F3S_SLOG((MAJOR)),_SLOG_INFO,"(devf  t%d::%s:%d) fs%Xp%X " S " reclaim L[%02X] P[%02X] => P[%02X] * %06X\n", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS), (SPARE), (STALE));\
				}\
			}
					

/*
** Function: f3s_log_split

void f3s_log_split(uint32_t major,
                   uint32_t minor,
                   uint32_t logi,
                   uint32_t index,
                   int32_t alloc_size,
                   int32_t split_size)
*/

#define f3s_log_split(MAJOR, MINOR, LOGI, INDEX, ALLOC_SIZE, SPLIT_SIZE) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X split H[%02X] L[%02X] # %06X | %06X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (ALLOC_SIZE), (SPLIT_SIZE));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)),_SLOG_INFO,"(devf  t%d::%s:%d) fs%Xp%X split H[%02X] L[%02X] # %06X | %06X\n", \
					pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (ALLOC_SIZE), (SPLIT_SIZE));\
				}\
			}

/*
** Function: f3s_log_bad

void f3s_log_bad(uint32_t major,
                 uint32_t minor,
                 int32_t phys,
                 uint32_t index,
                 int32_t alloc_size)
*/

#define f3s_log_bad(MAJOR, MINOR, PHYS, INDEX, ALLOC_SIZE, STATUS) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X bad H[%02X] P[%02X] # %06X # %08X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (PHYS), (ALLOC_SIZE), (STATUS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X bad H[%02X] P[%02X] # %06X # %08X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (PHYS), (ALLOC_SIZE), (STATUS));\
				}\
			}

/*
** Function: f3s_log_map

void f3s_log_map(uint32_t major,
		 uint32_t minor,
		 int32_t  logi,
		 int32_t  phys,
		 uint32_t text_size,
		 uint32_t free_size,
		 uint32_t stale_size,
		 uint32_t pad_size,
		 uint16_t head_last)
*/

#define f3s_log_map(MAJOR, MINOR, LOGI, PHYS, TEXT_SIZE, FREE_SIZE, STALE_SIZE, PAD_SIZE, HEAD_LAST) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X bad map L[%02X] P[%02X] #%08X #%08X #%08X #%08X #%04X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS), (TEXT_SIZE), (FREE_SIZE), (STALE_SIZE), (PAD_SIZE), (HEAD_LAST));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X bad map L[%02X] P[%02X] #%08X #%08X #%08X #%08X #%04X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS), (TEXT_SIZE), (FREE_SIZE), (STALE_SIZE), (PAD_SIZE), (HEAD_LAST));\
				}\
			}

/*
** Function: f3s_log_retire

void f3s_log_retire(uint32_t major,
                    uint32_t minor,
                    int32_t phys)
*/

#define f3s_log_retire(MAJOR, MINOR, PHYS) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X retire P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X retire P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS));\
				}\
			}

/*
** Function: f3s_log_spare

void f3s_log_spare(uint32_t major,
                   uint32_t minor,
                   int32_t phys)
*/

#define f3s_log_spare(MAJOR, MINOR, PHYS) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X spare P[%02X]\n", pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X spare P[%02X]\n", pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS));\
				}\
			}

/*
** Function: f3s_log_socket

void f3s_log_socket(uint32_t major,
                    uint8_t *name)
*/

#define f3s_log_socket(MAJOR, NAME_PTR) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%X socket %s\n", pthread_self(), __func__, __LINE__, (MAJOR), (NAME_PTR));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%X socket %s", pthread_self(), __func__, __LINE__, (MAJOR), (NAME_PTR));\
				}\
			}

/*
** Function: f3s_log_array

void f3s_log_array(uint32_t major,
                   uint32_t unit_num,
                   uint32_t unit_size,
                   uint8_t *name)
*/

#define f3s_log_array(MAJOR, UNIT_NUM, UNIT_SIZE, NAME_PTR) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%X array %s U: %02X S: %06X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (NAME_PTR), (UNIT_NUM), (UNIT_SIZE));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%X array %s U: %02X S: %06X", \
						pthread_self(), __func__, __LINE__, (MAJOR), (NAME_PTR), (UNIT_NUM), (UNIT_SIZE));\
				}\
			}

/*
** Function: f3s_log_boot

void f3s_log_boot(uint32_t major,
                  uint32_t minor,
                  int32_t phys,
                  uint32_t unit_num)
*/

#define f3s_log_boot(MAJOR, MINOR, PHYS, UNIT_NUM) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X boot P[%02X] U: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS), (UNIT_NUM));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X boot P[%02X] U: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS), (UNIT_NUM));\
				}\
			}

/*
** Function: f3s_log_swap

void f3s_log_swap(uint32_t major,
                  uint32_t minor)
*/

#define f3s_log_swap(MAJOR, MINOR) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X swap\n", pthread_self(), __func__, __LINE__, (MAJOR), (MINOR));\
				}\
				if(slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X swap\n", pthread_self(), __func__, __LINE__, (MAJOR), (MINOR));\
				}\
			}

/*
** Function: f3s_log_partial

void f3s_log_partial(uint32_t major,
                     uint32_t minor,
                     int32_t phys)
*/

#define f3s_log_partial(MAJOR, MINOR, PHYS) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X partial P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X partial P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (PHYS));\
				}\
			}

/*
** Function: f3s_log_dangle

void f3s_log_dangle(uint32_t major,
                    uint32_t minor,
                    uint32_t logi,
                    int32_t phys,
                    uint32_t index)
*/

#define f3s_log_dangle(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X dangle H[%02X] L[%02X] P[%02X]\n", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X dangle H[%02X] L[%02X] P[%02X]\n", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
			}

/*
** Function: f3s_log_bogus

void f3s_log_bogus(uint32_t major,
                   uint32_t minor,
                   uint32_t logi,
                   int32_t phys,
                   uint32_t index)
*/

#define f3s_log_bogus(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X bogus H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X bogus H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
			}



/*
** Function: f3s_log_multi

void f3s_log_multi(uint32_t major,
                   uint32_t minor,
                   uint32_t logi,
                   int32_t phys,
                   uint32_t index,
                   uint32_t multi_num)
*/

#define f3s_log_multi(MAJOR, MINOR, LOGI, PHYS, INDEX, MULTI_NUM) \
			if (f3s.verbose>=F3S_LOG_STARTUP_FATAL){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X multi H[%02X] L[%02X] P[%02X] M: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS), (MULTI_NUM));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X multi H[%02X] L[%02X] P[%02X] M: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS), (MULTI_NUM));\
				}\
			}

/*
** Function: f3s_log_dirty

void f3s_log_dirty(uint32_t major,
                   uint32_t minor,
                   uint32_t logi,
                   int32_t phys,
                   uint32_t index)
*/

#define f3s_log_dirty(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X dirty H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X dirty H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
			}\

			

/*
** Function: f3s_log_damage

void f3s_log_damage(uint32_t major,
                    uint32_t minor,
                    uint32_t logi,
                    int32_t phys,
                    uint32_t index)
*/

#define f3s_log_damage(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR) \
				printf("(devf  t%d::%s:%d) fs%Xp%X damage H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS))

/*
** Function: f3s_log_virgin

void f3s_log_virgin(uint32_t major,
                    uint32_t minor,
                    uint32_t logi,
                    int32_t phys,
                    uint32_t index)
*/

#define f3s_log_virgin(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO) \
				printf("(devf  t%d::%s:%d) fs%Xp%X virgin H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS))

/*
** Function: f3s_log_sticky

void f3s_log_sticky(uint32_t major,
                    uint32_t minor,
                    uint32_t logi,
                    int32_t phys,
                    uint32_t index)
*/

#define f3s_log_sticky(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO) \
				printf("(devf  t%d::%s:%d) fs%Xp%X sticky H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS))

/*
** Function: f3s_log_slip

void f3s_log_slip(uint32_t major,
                  uint32_t minor,
                  uint32_t logi,
                  int32_t phys,
                  uint32_t index)
*/

#define f3s_log_slip(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR) \
				printf("(devf  t%d::%s:%d) fs%Xp%X slip H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS))

/*
** Function: f3s_log_outside

void f3s_log_outside(uint32_t major,
                     uint32_t minor,
                     uint32_t logi,
                     int32_t phys,
                     uint32_t index)
*/

#define f3s_log_outside(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X outside H[%02X] L[%02X] P[%02X]\n", \
							pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X outside H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
			}

/*
** Function: f3s_log_junk

void f3s_log_junk(uint32_t major,
                  uint32_t minor,
                  uint32_t logi,
                  int32_t phys,
                  uint32_t index)
*/

#define f3s_log_junk(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X junk H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X junk H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS));\
				}\
			}

/*
** Function: f3s_log_corrupt

void f3s_log_corrupt(uint32_t major,
                     uint32_t minor,
                     uint32_t unit_num)
*/

#define f3s_log_corrupt(MAJOR, MINOR, UNIT_NUM) \
			if (f3s.verbose>=F3S_LOG_STARTUP_FATAL){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X corrupt U: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (UNIT_NUM));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X corrupt U: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (UNIT_NUM));\
				}\
			}

/*
** Function: f3s_log_raw

void f3s_log_raw(uint32_t major,
                 uint32_t minor,
                 uint32_t unit_num)
*/

#define f3s_log_raw(MAJOR, MINOR, UNIT_NUM) \
			if (f3s.verbose>=F3S_LOG_STARTUP_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X raw U: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (UNIT_NUM));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X raw U: %02X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (UNIT_NUM));\
				}\
			}
/*
** Function: f3s_log_head_repair

void f3s_log_head_repair(uint32_t major,
                         uint32_t minor,
                         uint32_t logi,
                         int32_t phys,
                         uint32_t spare,
                         uint32_t stale)
*/

#define f3s_log_head_repair(MAJOR, MINOR, LOGI, PHYS, SPARE, STALE) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X head_repair L[%02X] P[%02X] => P[%02X] * %06X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS), (SPARE), (STALE));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X head_repair L[%02X] P[%02X] => P[%02X] * %06X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (LOGI), (PHYS), (SPARE), (STALE));\
				}\
			}

/*
** Function: f3s_log_xcache

void f3s_log_xcache(uint32_t major,
                    uint8_t *string)
*/

#define f3s_log_xcache(MAJOR, STRING_PTR) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_INFO){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%X xcache %s\n", pthread_self(), __func__, __LINE__, (MAJOR), (STRING_PTR));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%X xcache %s", pthread_self(), __func__, __LINE__, (MAJOR), (STRING_PTR));\
				}\
			}

/*
** Function: f3s_log_overlap

void f3s_log_overlap(uint32_t major,
                 uint32_t minor,
                 int32_t phys,
                 uint32_t index,
                 int32_t text_size,
                 int32_t text_offset)
*/

#define f3s_log_overlap(MAJOR, MINOR, PHYS, INDEX, TEXT_SIZE, TEXT_OFFSET) \
			if (f3s.verbose>=F3S_LOG_STARTUP_FATAL){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X header H[%02X] P[%02X] overlapped with text # %06X # %08X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (PHYS), (TEXT_SIZE), (TEXT_OFFSET));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X header H[%02X] P[%02X] overlapped with text # %06X # %08X\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (PHYS), (TEXT_SIZE), (TEXT_OFFSET));\
				}\
			}

/*
** Function: f3s_log_head_last

void f3s_log_head_last(uint32_t major,
                  uint32_t minor,
                  uint32_t logi,
                  int32_t phys,
                  uint32_t index)
*/

#define f3s_log_head_last(MAJOR, MINOR, LOGI, PHYS, INDEX) \
			if (f3s.verbose>=F3S_LOG_STARTUP_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X missed head_last H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS)); \
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X missed head_last H[%02X] L[%02X] P[%02X]\n", \
						pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (INDEX), (LOGI), (PHYS)); \
				}\
			}

/*
** Function: f3s_log_error

void f3s_log_error(uint32_t major,
                  uint32_t minor,
                  int32 error)
*/
#define f3s_log_error(MAJOR, MINOR, ERROR) \
			if (f3s.verbose>=F3S_LOG_RUNTIME_ERROR){ \
				if (stdout_log){\
					printf("(devf  t%d::%s:%d) fs%Xp%X error %d \n", pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (ERROR));\
				}\
				if (slog_log){\
					slogf(F3S_SLOG((MAJOR)), _SLOG_INFO, "(devf  t%d::%s:%d) fs%Xp%X error %d \n", pthread_self(), __func__, __LINE__, (MAJOR), (MINOR), (ERROR));\
				}\
			}
#endif // NDEBUG

#endif /* __F3S_LOG_H_INCLUDED */

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/f3s_log.h $ $Rev: 580669 $" )
