/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_macro.h
**
** Description:
**
** This file contains the macros definitions for the f3s
** resource manager for Neutrino
**
** Ident: $Id: f3s_macro.h 580669 2011-09-01 15:18:50Z builder@qnx.com $
*/

/*
** Include Loop Prevention
*/

#ifndef __F3S_MACRO_H_INCLUDED
#define __F3S_MACRO_H_INCLUDED

/*
** Macros Definitions
*/

#define F3S_MAKE_MINOR(ARRAY_INDEX, PART_INDEX, PART_FLAG)\
\
  (((ARRAY_INDEX)<<5) | ((PART_INDEX)<<1) | (PART_FLAG))

#define F3S_TEXT_OFFSET(ARRAY_PTR, UNIT, OFFSET)\
\
  (((UNIT)<<(ARRAY_PTR)->unit_pow2)+(OFFSET))

#define F3S_HEAD_OFFSET(ARRAY_PTR, INDEX)\
\
  (((ARRAY_PTR)->unit_size-(((INDEX)+1)<<F3S_HEAD_POW2)))

#define F3S_TEXT_ALIGN(PART, SIZE, ALIGN)\
\
  (((SIZE)+((ALIGN)-1))&~((ALIGN)-1))

#ifdef NDEBUG	/* lock define ? */
#define F3S_UNIT_ACTIVE(PART_PTR) \
				((PART_PTR)->unit_num-(PART_PTR)->table.unit_spare)
#else	/* NDEBUG */
/* debug version does lock checking */
extern uint16_t _f3s_unit_active();
#define F3S_UNIT_ACTIVE(PART_PTR)		_f3s_unit_active((PART_PTR))
#endif	/* NDEBUG */

#define F3S_TABLE_ENTER(PART_PTR)\
\
  f3s_lock_enter(&(PART_PTR)->table_lock)

#define F3S_TABLE_LEAVE(PART_PTR)\
\
  f3s_lock_leave(&(PART_PTR)->table_lock)


#define F3S_UNIT_ENTER(PART_PTR, PHYS) \
				do { \
						f3s_lock_enter(&(PART_PTR)->array->map_lock_vect[(PHYS)]); \
				} while(0)

#define F3S_UNIT_LEAVE(PART_PTR, PHYS)\
\
  f3s_lock_leave(&(PART_PTR)->array->map_lock_vect[(PHYS)])

#define F3S_LOGI_ENTER(PART_PTR, LOGI) \
				do { \
						f3s_lock_enter(&(PART_PTR)->table.logi_lock_vect[(LOGI)]); \
				} while(0)

#define F3S_LOGI_LEAVE(PART_PTR, LOGI) \
\
  f3s_lock_leave(&(PART_PTR)->table.logi_lock_vect[(LOGI)]); 

#define F3S_SPARE_ENTER(PART_PTR) \
				do { \
					/* make sure that table lock is held by caller. This is the inferred lock hierarchy */ \
					f3s_lock_assert((PART_PTR)->table_lock.thread == pthread_self()); \
					f3s_lock_enter(&(PART_PTR)->table.spare_lock); \
				} while(0)

#define F3S_SPARE_LEAVE(PART_PTR)\
\
  f3s_lock_leave(&(PART_PTR)->table.spare_lock)

#define F3S_RAW_ENTER(ARRAY_PTR, PHYS) \
				do {\
						/* can cheat because I know there is a'part' local almost everywhere that this macro is used */ \
						if (part == NULL) if (attr != NULL) part = attr->part; \
						if (part != NULL) \
							f3s_lock_assert(part->table_lock.thread == pthread_self()); \
						f3s_lock_enter(&(ARRAY_PTR)->map_lock_vect[(PHYS)]); \
				} while(0)

#define F3S_RAW_LEAVE(ARRAY_PTR, PHYS)\
\
  f3s_lock_leave(&(ARRAY_PTR)->map_lock_vect[(PHYS)])

#define F3S_CACHE_ENTER(PART_PTR)\
\
  f3s_lock_enter(&(PART_PTR)->table.cache_lock)

#define F3S_CACHE_LEAVE(PART_PTR)\
\
  f3s_lock_leave(&(PART_PTR)->table.cache_lock)

#define F3S_ALLOC_ENTER(PART_PTR)\
\
  f3s_lock_enter(&(PART_PTR)->table.alloc_lock)

#define F3S_ALLOC_LEAVE(PART_PTR)\
\
  f3s_lock_leave(&(PART_PTR)->table.alloc_lock)

#define F3S_XCACHE_ENTER(PART_PTR)\
\
  f3s_lock_enter(&(PART_PTR)->xlock)

#define F3S_XCACHE_LEAVE(PART_PTR)\
\
  f3s_lock_leave(&(PART_PTR)->xlock)

#define F3S_XPOOL_ENTER(POOL_PTR)\
\
  f3s_lock_enter(&(POOL_PTR)->lock)

#define F3S_XPOOL_LEAVE(POOL_PTR)\
\
  f3s_lock_leave(&(POOL_PTR)->lock)


#define F3S_INODE_TO_EXTPTR(INODE,EXTPTR)\
	do{\
		(EXTPTR).logi_unit = (INODE) >> (sizeof((EXTPTR).index) * CHAR_BIT);\
		(EXTPTR).index = (INODE) & ((1<<(sizeof((EXTPTR).index) * CHAR_BIT))-1);\
	}while(0)


#define F3S_EXTPTR_TO_INODE(EXTPTR, INODE)\
	do{\
		(INODE) = ((EXTPTR).logi_unit << (sizeof((EXTPTR).index) * CHAR_BIT) | (EXTPTR).index);\
	}while(0)


#endif /* __F3S_MACRO_H_INCLUDED */

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/f3s_macro.h $ $Rev: 580669 $" )
