/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int f3s_part_attach (f3s_part_t *part,
                     f3s_attr_t *raw_attr, f3s_attr_t *f3s_attr,
                     uint32_t index)
{
/*{{{1*/
	char			name[F3S_NAME_MAX + 1];
	f3s_dirent_t	dirent;
	f3s_stat_t		stat;
	struct			sigevent event;
	int32_t			flag;
	int				err_size;
	int				error = EOK;
	unsigned		rflag = 0|_RESMGR_FLAG_DIR;
	uint32_t xpool_pages;
	int				name_match = 0;


	/* check if we have file system attributes */
	if (f3s_attr)
	{
		f3s_lock_assert((f3s_attr->attr.lock_tid == pthread_self()) && (f3s_attr->attr.lock_count > 0));
		
		/* setup partition index */
		part->index = index;

		/* setup file system attribute pointer in attribute (self) */
		f3s_attr->part_attr = f3s_attr;

		/* find partition mapping table */
		error = f3s_table_find (part);

		/* check for readonly filesystem. Note that f3s_mntopts_t_MOUNT_RO only applies
		 * at devf-xxx initialization */
		if ((error == EROFS) ||
			((f3s.mount_options & f3s_mntopts_t_INIT) &&
				((f3s.mount_options & f3s_mntopts_t_OPTION_MASK) == f3s_mntopts_t_MOUNT_RO)))
		{
			/* set readonly bit in mount structure */
			f3s_attr->attr.mount->mount.flags |= _MOUNT_READONLY;
			/* don't change error unless we got here because of EROFS */
			if (error == EROFS)
				error = EOK;
		}

		/* check if file system is good */
		if (!error)
		{
			/* copy partition root extent pointer in partition attributes */
			f3s_lock_assert(part->table_lock.thread == pthread_self());
			f3s_attr->parent_extptr = part->table.root_extptr;

			/* read root stat info */
			err_size = f3s_root_read (part, &dirent, name, &stat);
			if (err_size < 0) return (errno);

			/* check if there is no name defined */
			if (!(*name))
			{
				/* check if there is a pathname override defined */
				if (f3s.mount_over)
				{
					/* check if there is no format specifier */
					if (!f3s.mount_format)
					{
						strcpy (name, f3s.mount_over);

					/* else, check if there is one format specifier */
					} else if (f3s.mount_format == 1)
					{
						sprintf (name, f3s.mount_over, part->array->index);

					/* else, there are two format specifiers */
					} else
					{
						sprintf (name, f3s.mount_over, part->array->index, part->index);
					}
				} else
				{
					/* build default pathname for file system mountpoint */
					sprintf (name, "/fs%Xp%X", part->array->index, part->index);
				}
			}else{
				/* Check if the user specified a list of mountpoints to mount.
				 * If they did, check to see if this one is on the list.
				 */
				if (f3s.mountexact_num > 0){
					unsigned i;
					name_match = 0;
					for (i = 0; i<f3s.mountexact_num; i++){
						if (!strcmp(name, f3s.mountexact_list[i])){
							name_match = 1;
							break;
						}
					}
				}

			}

			if ((f3s.mountexact_num == 0) || name_match){
				/* check if there is a name override in mount */
				if (f3s_attr->attr.mount->name)
				{
					/* build pathname for file system mountpoint */
					strcpy (name, f3s_attr->attr.mount->name);
				}

				/* set root size for info */
				f3s_lock_assert(part->table_lock.thread == pthread_self());
				part->table.root_size = sizeof(f3s_dirent_t) + F3S_NAME_ALIGN(dirent.namelen) + sizeof(f3s_stat_t);

				/* reserve space for a header to garantee deletion */
				f3s_resv_alloc (part, &part->table.deletion_resv, 0);
	
				/* initialize pulse connection for partition */
				part->pulse_connect = message_connect (f3s.dpp, MSG_FLAG_SIDE_CHANNEL);
				if (part->pulse_connect < 0) perror ("unable to message connect");

				/* initialize sync pulse for partition */
				part->sync_code =
					pulse_attach (f3s.dpp, MSG_FLAG_ALLOC_PULSE, 0,
				              (int (*)(message_context_t *, int, unsigned, void *))f3s_sync_pulse,
				              f3s_attr);
				if (part->sync_code < 0) perror ("unable to pulse attach");

				SIGEV_PULSE_INIT (&event, part->pulse_connect, SIGEV_PULSE_PRIO_INHERIT, part->sync_code, NULL);

				flag = procmgr_event_notify (PROCMGR_EVENT_SYNC, &event);
				if (flag < 0) perror ("unable to event notify");

				/* initialize reclaim background for partition */
				part->bkgd_code = -1;
				f3s_bkgd_init (part);

				/* set inode number in attributes */
				f3s_lock_assert(part->table_lock.thread == pthread_self());
				f3s_attr->attr.inode = (ino_t) (*(uint32_t *) & part->table.root_extptr);

				/* keep a copy of mountpoint name */
				f3s_attr->attr.mount->name = malloc (strlen (name) + 1);
				if (!f3s_attr->attr.mount->name)
				{
					f3s_assert (0);
					return (ENOMEM);
				}

				strcpy (f3s_attr->attr.mount->name, name);

				/* initialize the reclaim triggers for this partition */
				part->table.reclaim.trigger = f3s.reclaim.trigger;
				part->table.reclaim.etrigger = f3s.reclaim.etrigger;
				part->table.reclaim.threshold = f3s.reclaim.threshold;
				part->table.reclaim.ethreshold = f3s.reclaim.ethreshold;

				/* attach pathname for filesys mountpoint */
#ifdef ENABLE_MOUNTBEHIND
				if (f3s.mount_behind){
					rflag |= _RESMGR_FLAG_AFTER;
				}
#endif
				f3s_attr->attr.mount->attach_id[F3S_ATTACH_F3S] = resmgr_attach (f3s.dpp, 
					&f3s.resmgr_attr, name, _FTYPE_ANY, rflag, &f3s.f3s_connect_funcs,
					&f3s.f3s_io_funcs, f3s_attr);
				if (f3s_attr->attr.mount->attach_id[F3S_ATTACH_F3S] < 0) return (errno);

				/* allocate memory for xcache table and create page pool */
				part->xcache = f3s_calloc(sizeof(f3s_xcache_t)*f3s.xcache_file_max);
				if (part->xcache == NULL) 
				{
					return (ENOMEM);
				}
				memset(part->xcache, 0, sizeof(f3s_xcache_t)*f3s.xcache_file_max);
				f3s_lock_init(&part->xlock);
				xpool_pages = (part->unit_num<<part->array->unit_pow2)/(f3s.xcache_blksz*XCACHE_ENTRIES_PER_PAGE) + 16;
				if (f3s_xpool_init(&part->xpool, sizeof(f3s_xcache_page_t), xpool_pages) != 0) 
				{
					return (ENOMEM);
				}

				/* clear not attached flag */
				f3s_lock_assert(part->table_lock.thread == pthread_self());
				part->table.status &= ~F3S_TABLE_NO_ATTACH;
			}
		}
	}

	/* check if we have raw attributes */
	if (raw_attr)
	{
		f3s_lock_assert((raw_attr->attr.lock_tid == pthread_self()) && (raw_attr->attr.lock_count > 0));
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		
		/* setup file system attribute pointer in raw attribute */
		raw_attr->part_attr = f3s_attr;

		/* setup size for attributes */
		raw_attr->attr.nbytes = part->unit_num << part->array->unit_pow2;

		/* build pathname for raw mountpoint */
		sprintf (name, _PATH_DEV "fs%Xp%X", part->array->index, part->index);

		/* set inode number in attributes */
		raw_attr->attr.inode = (ino_t) index;

		/* attach pathname for raw mountpoint */
		raw_attr->attr.mount->attach_id[F3S_ATTACH_RAW] =
			resmgr_attach (f3s.dpp, &f3s.resmgr_attr, name, _FTYPE_FILE,
		                   _RESMGR_FLAG_SELF, &f3s.raw_connect_funcs,
		                   &f3s.raw_io_funcs, raw_attr);
		if (raw_attr->attr.mount->attach_id[F3S_ATTACH_RAW] < 0) return (errno);
	}

	return (error);
/*}}}1*/
}

int f3s_part_detach (f3s_part_t *part,
                     f3s_attr_t *raw_attr, f3s_attr_t *f3s_attr)
{
/*{{{1*/
	int32_t i, flag;

	/* check if we have f3s attributes attached */
	f3s_lock_assert((f3s_attr->attr.lock_tid == pthread_self()) && (f3s_attr->attr.lock_count));
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->table_lock.count == 1));

	if (f3s_attr && !(part->table.status & F3S_TABLE_NO_ATTACH))
	{
		/* Need to give up the locks prior to calling resmgr_detach() so that any "in-progress"
		 * operations can actually proceed */
		F3S_TABLE_LEAVE(part);
		iofunc_attr_unlock(&f3s_attr->attr);

		/* NOTE: MUST BE DONE FIRST, WILL BLOCK UNTIL ACTIVE OPS DONE */
		flag = resmgr_detach (f3s.dpp,
							  f3s_attr->attr.mount->attach_id[F3S_ATTACH_F3S], _RESMGR_DETACH_ALL);
							  
		/* if resmgr_detach() fails, we are not going to recquire the locks. Just
		 * return the error. The unlock functions will not unlock an already unlocked lock */
		if (flag < 0)
			return (errno);
		else
		{
			/* all operations on this partition SHOULD be done. Grab the locks and cleanup the attached OCB's */
			f3s_share_t *	file_share;

			iofunc_attr_lock(&f3s_attr->attr);
			F3S_TABLE_ENTER(part);

			f3s_lock_assert((f3s_attr->attr.lock_tid == pthread_self()) && (f3s_attr->attr.lock_count == 1));
			f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->table_lock.count == 1));
			
			file_share = (f3s_share_t *)f3s_attr->share_list.next;

			while (file_share != (f3s_share_t *)&f3s_attr->share_list)
			{
				if (file_share->ocb != NULL)
					f3s_close(NULL, file_share->ocb);
				file_share = (f3s_share_t *)f3s_attr->share_list.next;
			}
		}

		/* free (internal) reserved space */
		f3s_resv_free (part, &part->table.deletion_resv);

		/* drain spare unit list */
		f3s_list_drain (&part->table.spare_list, sizeof(f3s_spare_t) - sizeof(f3s_list_t));

		/* free mutex of partition table */
		f3s_lock_free(&part->table.alloc_lock);
		f3s_lock_free(&part->table.cache_lock);
		f3s_lock_free(&part->table.spare_lock);
		for(i = 0; i < (part->unit_num + F3S_FIRST_LOGI); i++) {
			f3s_lock_free(&part->table.logi_lock_vect[i]);
		}

		/* free space for table vectors */
		f3s_free (part->table.logi_vect, (part->unit_num + F3S_FIRST_LOGI) * sizeof(*part->table.logi_vect));
		part->table.logi_vect = NULL;
		f3s_free (part->table.logi_lock_vect, (part->unit_num + F3S_FIRST_LOGI) * sizeof(f3s_lock_t));
		part->table.logi_lock_vect = NULL;
		f3s_free (part->table.sort_vect, F3S_UNIT_ACTIVE (part) * sizeof(f3s_sort_t));
		part->table.sort_vect = NULL;
		f3s_free (part->table.cache_vect, F3S_CACHE_MAX * sizeof(f3s_cache_t));
		part->table.cache_vect = NULL;

		f3s_free (part->table.copy_vect, part->table.copy_max * sizeof(*part->table.copy_vect));
		part->table.copy_vect = NULL;
		f3s_free (part->table.obsol_vect, part->table.obsol_max * sizeof(*part->table.obsol_vect));
		part->table.obsol_vect = NULL;

		/* free space for reclaim text buffer */
		f3s_free (part->table.text_buffer, part->table.text_size);
		part->table.text_buffer = NULL;

		/* detach sync pulse for partition */
		pulse_detach (f3s.dpp, part->sync_code, 0);
		part->sync_code = -1;

		/* disable background reclaim for partition */
		f3s_bkgd_kill (part);

		/* detach pulse connection for partition */
		ConnectDetach (part->pulse_connect);

		/* free mounted name */
		free (f3s_attr->attr.mount->name);
		f3s_attr->attr.mount->name = NULL;

		/* free xcache and pool */
		if(part->xcache != NULL) 
		{
			f3s_free(part->xcache, sizeof(f3s_xcache_t)*f3s.xcache_file_max);
			f3s_lock_free(&part->xlock);
		}
		f3s_xpool_delete(&part->xpool);

		/* set not attached flag */
		part->table.status |= F3S_TABLE_NO_ATTACH;
	}

	/* check if we have raw attributes */
	if (raw_attr)
	{
		f3s_lock_assert((raw_attr->attr.lock_tid == pthread_self()) && (raw_attr->attr.lock_count));
		
		rsrcdbmgr_devno_detach (raw_attr->attr.rdev, 0);
		/* detach raw mountpoint name */
		flag = resmgr_detach (f3s.dpp, raw_attr->attr.mount->attach_id[F3S_ATTACH_RAW], _RESMGR_DETACH_ALL);
		if (flag < 0) return (errno);
	}

	return (EOK);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/partition.c $ $Rev: 580669 $" );
