/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

static uint8_t FF32[32] =
{
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
};


/*
 * If we turn off the supersede pointer, it's critical to turn off the "next" pointer also.*/
static int f3s_fix_super(f3s_part_t *part, int32_t *phys, uint32_t index, f3s_head_t *head)
{
	if (f3s.recover) {
		head->next.index = 0;
		head->next.logi_unit = 0;
		f3s_head_next(part, phys, index, head->next, 0);
		head->super.index = 0;
		head->super.logi_unit = 0;
		f3s_head_super(part, phys, index, head->super, 0);
		return (EOK); //have to return EOK because we changed something
	}
	return (-1);
}

static int f3s_fix_next(f3s_part_t *part, int32_t *phys, const uint16_t index, f3s_head_t *head)
{
	if (f3s.recover) {
		head->next.index = 0;
		head->next.logi_unit = 0;
		f3s_head_next(part, phys, index, head->next, 0);
		return (EOK);
	}
	return (-1);
}

static int f3s_fix_dirent(f3s_part_t *part, const int32_t logi, const uint16_t index)
{
	f3s_desc_t dir_desc;
	f3s_extptr_t extptr;
	int32_t phys;

	if (f3s.recover) {
		extptr.logi_unit = logi;
		extptr.index = index;
		phys = part->table.logi_vect[logi];
		F3S_UNIT_LEAVE(part, phys);
		f3s_desc_init (&dir_desc, extptr, ~F3S_DESC_NO_DIR);
		f3s_resv_alloc (part, &part->table.deletion_resv, 0);
		f3s_entry_delete (part, &dir_desc, 0);
		f3s_resv_free (part, &part->table.deletion_resv);
		/* it's quite possible that f3s_head_repair() changed the value of phys*/
		phys = part->table.logi_vect[logi];
		F3S_UNIT_ENTER(part, phys);
		return (EOK);
	}
	return (-1);
}

static int f3s_fix_multi (f3s_part_t *part, const f3s_extptr_t multi, int32_t multi_count)
{
	f3s_head_t		head;
	const f3s_map_t* 	map;

	int32_t			logi;
	uint16_t		index;
	int32_t			phys;
	int32_t			text_size;
	int    			error;

	for (logi = F3S_FIRST_LOGI; logi <= F3S_UNIT_ACTIVE(part); logi++) {

		F3S_LOGI_ENTER(part, logi);
		phys = part->table.logi_vect[logi];
		F3S_UNIT_ENTER(part, phys);

		map = &part->array->map_vect[phys];

		for (index = 0; index <= map->head_last; index++) {
			text_size = f3s_head_read (part, phys, index, NULL, &head, 1);
			if (text_size < 0) {
				error = errno;
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE(part, logi);
				return (error);
			}

			if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC) {
				if (!(head.status[0] & F3S_EXT_NO_SUPER)) {
					if(!memcmp(&multi, &head.super, sizeof(f3s_extptr_t))) {
						error = f3s_fix_super(part, &phys, index, &head);
						map = &part->array->map_vect[phys];
						multi_count--;
					}
				} else if (!(head.status[0] & F3S_EXT_NO_NEXT)) {
					if(!memcmp(&multi, &head.next, sizeof(f3s_extptr_t))) {
						error = f3s_fix_next(part, &phys, index, &head);
						map = &part->array->map_vect[phys];
						multi_count--;
					}
				}
			}
		}
		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, logi);

		if(multi_count == 0) {
			return (EOK);
		}
	}
	
	return (-1);
}


/* XXX - Remeber to skip retired units */
int f3s_recover_boot (f3s_part_t *part)
{
/*{{{1*/
	f3s_unit_info_t	unit_info;
	f3s_unit_logi_t	unit_logi;
	f3s_boot_t		boot;
	int32_t			curr_phys;
	int				error;

	/* unit find loop */
	for (curr_phys = part->unit_first; curr_phys < part->array->unit_total; curr_phys++)
	{
		/* If not dealing with first unit and there is a break in array */
		if (curr_phys != part->unit_first && part->array->break_vect[curr_phys])
		{
			/* initialize unit num to current less first */
			part->unit_num = max(1, curr_phys - part->unit_first);
			return (f3s_ebadfsys (part->index, __func__, __LINE__));
		}

		F3S_UNIT_ENTER(part, curr_phys);

		/* Look for candidate endian */
		error = f3s_flash_read (part->array, curr_phys, 0,
                                sizeof(unit_info), (uint8_t *)&unit_info);
		if (error)
		{
			F3S_UNIT_LEAVE(part, curr_phys);
			return (error);
		}

		f3s_lock_assert(part->table_lock.thread == pthread_self());
#if defined(__BIGENDIAN__)
		if (unit_info.endian == 'B')      part->table.swap_flag = 0;
		else if (unit_info.endian == 'L') part->table.swap_flag = 1;
		else                              {F3S_UNIT_LEAVE(part, curr_phys); continue;}
#else
		if (unit_info.endian == 'B')      part->table.swap_flag = 1;
		else if (unit_info.endian == 'L') part->table.swap_flag = 0;
		else                              {F3S_UNIT_LEAVE(part, curr_phys); continue;}
#endif

		/* read unit info structure */
		error = f3s_unit_info_read (part, curr_phys, &unit_info, 1);
		if (error)
		{
			F3S_UNIT_LEAVE(part, curr_phys);
			if (error == ENOENT) continue;
			goto error;
		}

		/* If the unit is ready */
		error = f3s_unit_logi_read (part, curr_phys, &unit_logi);
		if (error)
		{
			F3S_UNIT_LEAVE(part, curr_phys);
			continue;
		}

		/* If boot extent is in same unit */
		if (unit_info.boot.logi_unit == unit_logi.logi)
		{
			/* read boot info structure */
			error = f3s_boot_read (part, curr_phys, &boot);
			if (error)
			{
				F3S_UNIT_LEAVE(part, curr_phys);
				if (error == ENOENT) continue;
				goto error;
			}

			/* If first is invalid */
			if (boot.unit_index != (curr_phys - part->unit_first))
			{
				/* make sure boot is valid */
				if ((curr_phys - part->unit_first) >= boot.unit_index)
				{
					/* We've found a candidate, make the previous part raw */
					F3S_UNIT_LEAVE(part, curr_phys);
					part->unit_num = max(1, (curr_phys - part->unit_first) -
					                        boot.unit_index);
					return (EBADFSYS);
				}

				F3S_UNIT_LEAVE(part, curr_phys);
				continue;
			}

			/* If swapping is enabled */
			f3s_lock_assert(part->table_lock.thread == pthread_self());
			if (part->table.swap_flag)
			{
				/* log swapping mode */
				f3s_log_swap (part->array->index, part->index);
			}

			/* setup number of units in partition table */
			part->unit_num          = max(1, boot.unit_total);
			part->table.unit_spare  = boot.unit_spare;
			part->table.unit_retire = 0;

			/* setup alignment sizes in partition table */
			part->table.align_size = 1 << boot.align_pow2;

			/* setup alignment powers in partition table */
			part->table.align_pow2 = boot.align_pow2;

			/* log boot extent found */
			f3s_log_boot (part->array->index, part->index, curr_phys, part->unit_num);

			/* get partition boot and root extent pointers */
			part->table.boot_extptr = unit_info.boot;
			part->table.root_extptr = boot.root;

			F3S_UNIT_LEAVE(part, curr_phys);
			break;
		}
		F3S_UNIT_LEAVE(part, curr_phys);
	}

	/* If no file system was found */
	if (curr_phys >= part->array->unit_total) goto error;

	return (EOK);
error:
	/* setup partition for raw mountpoint only */
	part->unit_num = max(1, curr_phys - part->unit_first);
	return (EBADFSYS);
/*}}}1*/
}

int f3s_recover_delete (f3s_part_t *part, uint8_t **link_matrix)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_dirent_t	dirent;
	f3s_extptr_t	multi;
	f3s_map_t *		map;
	int32_t			zero_found;
	int32_t			multi_found = 0;
	int32_t			text_size;
	int32_t			curr_phys;
	int32_t			link_phys;
	uint32_t		text_offset;
	int32_t			index;
	int32_t			logi_num;
	int				err_size;
	int				error;

	/* XXX - Does this deal with power failures during recovery */
	/* delete dangling extents loop */
	do
	{
		/* clear zero found flag */
		zero_found = 0;

		/* unit map loop */
		for (logi_num = F3S_FIRST_LOGI; logi_num <= F3S_UNIT_ACTIVE(part); logi_num++)
		{
			F3S_LOGI_ENTER(part, logi_num);
			curr_phys = part->table.logi_vect[logi_num];
			F3S_UNIT_ENTER(part, curr_phys);

			/* setup map pointer */
			f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
			map = &part->array->map_vect[curr_phys];

			/* header find loop */
			for (index = 0; index <= map->head_last; index++)
			{
				/* If link count for header is zero */
				if (!link_matrix[curr_phys][index])
				{
					/* set zero found flag */
					zero_found = 1;

					/* read header info structure */
					text_size = f3s_head_read (part, curr_phys, index, &text_offset, &head, 1);
					if (text_size < 0)
					{
						error = errno;
						F3S_UNIT_LEAVE(part, curr_phys);
						F3S_LOGI_LEAVE(part, logi_num);
						return (error);
					}

					/* log dangling extent */
					f3s_log_dangle (part->array->index, part->index,
					                part->array->phys_vect[curr_phys],
					                curr_phys, index);

					/* If recovery is wanted */
					if (f3s.recover && ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC))
					{
						/* mark header for deletion */
						err_size = f3s_head_status (part, &curr_phys, index,
						                            ~F3S_EXT_MASK | F3S_EXT_DELETE, 0);
						if (err_size < 0)
						{
							/* log sticky extent */
							f3s_log_sticky (part->array->index, part->index,
							                part->array->phys_vect[curr_phys],
							                curr_phys, index);
						}

						/* If header is not superseded */
						if (head.status[0] & F3S_EXT_NO_SUPER)
						{
							/* stale text size for unit */
							f3s_unit_stale (part, curr_phys, text_size);
						}

						/* increment number of deleted headers in map */
						map->del_num++;
					}

					/* mark header position in matrix */
					link_matrix[curr_phys][index]++;

					/* If super pointer is set */
					if (!(head.status[0] & F3S_EXT_NO_SUPER))
					{
						/* get link physical unit index */
						link_phys = f3s_extptr_phys (part, head.super);

						/* If super pointer is within range */
						if (head.super.index <= part->array->map_vect[link_phys].head_last)
						{
							/* unmark super header position in matrix */
							link_matrix[link_phys][head.super.index]--;
						}

					/* header is not superseded */
					}
					else
					{
						/* If next pointer is set */
						if (!(head.status[0] & F3S_EXT_NO_NEXT))
						{
							link_phys = f3s_extptr_phys (part, head.next);

							/* If next pointer is within range */
							if (head.next.index <= part->array->map_vect[link_phys].head_last)
							{
								/* unmark next header position in matrix */
								link_matrix[link_phys][head.next.index]--;
							}
						}

						/* If header is for a directory entry */
						if ((head.status[0] & F3S_EXT_TYPE) == F3S_EXT_DIR)
						{
							/* If dirent text has a size */
							if (text_size)
							{
								/* read dirent extent */
								error = f3s_flash_read (part->array, curr_phys, text_offset,
								                        sizeof(dirent), (uint8_t *)&dirent);
								if (error)
								{
									F3S_UNIT_LEAVE(part, curr_phys);
									F3S_LOGI_LEAVE(part, logi_num);
									return (error);
								}
								text_size = sizeof(dirent);

								/* If dirent structure needs swapping */
								f3s_lock_assert(part->table_lock.thread == pthread_self());
								if (part->table.swap_flag) f3s_dirent_swap (&dirent);

								/* If dirent is completely commited */
								/* get link physical unit index */
								link_phys = f3s_extptr_phys (part, dirent.first);

								/* If first pointer is within range */
								if (dirent.first.index <= part->array->map_vect[link_phys].head_last)
								{
									/* mark first header position in matrix */
									link_matrix[link_phys][dirent.first.index]--;
								}
							}
						}
					}
				}
			}
			F3S_UNIT_LEAVE(part, curr_phys);
			F3S_LOGI_LEAVE(part, logi_num);
		}

	/* while at least an extent has a zero link */
	} while (zero_found);

	/* unit map loop */
	for (curr_phys = part->unit_first;
	     curr_phys < (part->unit_first + part->unit_num);
	     curr_phys++)
	{
		F3S_UNIT_ENTER(part, curr_phys);

		/* setup map pointer */
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];

		/* header find loop */
		for (index = 0; index <= map->head_last; index++)
		{
			/* If link count for header is greater than one */
			if (link_matrix[curr_phys][index] > 1)
			{
				/* If recovery is wanted and link count is two */
				if (f3s.recover)
				{
					/* build multi extent pointer */
					f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
					multi.logi_unit = part->array->phys_vect[curr_phys];
					multi.index = index;

					F3S_UNIT_LEAVE(part, curr_phys);
					error = f3s_fix_multi(part, multi, link_matrix[curr_phys][index]);
					F3S_UNIT_ENTER(part, curr_phys);
					if (error == EOK)
					{
						link_matrix[curr_phys][index] = 0;
						continue;
					}

					/* clean up unfinished move */
					if (link_matrix[curr_phys][index] == 2) 
					{
						F3S_UNIT_LEAVE(part, curr_phys);
						error = f3s_cleanup_multi (part, multi);
						F3S_UNIT_ENTER(part, curr_phys);
						if (error == EOK) 
						{
							link_matrix[curr_phys][index] = 1;
							continue;
						}
					}
				}

				/* increment multi found counter */
				multi_found++;

				/* log multiply linked extent */
				f3s_log_multi (part->array->index, part->index,
				               part->array->phys_vect[curr_phys],
				               curr_phys, index, link_matrix[curr_phys][index]);
			}
		}
		F3S_UNIT_LEAVE(part, curr_phys);
	}

	/* If a multiple link was found */
	if (multi_found) return (f3s_ebadfsys (part->index, __func__, __LINE__));
	else             return (EOK);
/*}}}1*/
}

int f3s_recover_link (f3s_part_t *part, uint8_t **link_matrix)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_dirent_t	dirent;
	f3s_map_t *		map;
	int32_t			text_size;
	int32_t			curr_phys;
	int32_t			link_phys;
	uint32_t		text_offset;
	uint16_t		index;
	int32_t			logi_num;
	int				error;

	/* unit map loop */
	for (logi_num = F3S_FIRST_LOGI;
	     logi_num <= F3S_UNIT_ACTIVE(part);
	     logi_num++)
	{
		F3S_LOGI_ENTER(part, logi_num);
		curr_phys = part->table.logi_vect[logi_num];
		F3S_UNIT_ENTER(part, curr_phys);

		/* setup map pointer */
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];

		/* header find loop */
		for (index = 0; index <= map->head_last; index++)
		{
			/* read header info structure */
			text_size = f3s_head_read (part, curr_phys, index, &text_offset, &head, 1);
			if (text_size < 0)
			{
				error = errno;
				F3S_UNIT_LEAVE(part, curr_phys);
				F3S_LOGI_LEAVE(part, logi_num);
				return (error);
			}

			/* If header is good stuff */
			if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC)
			{
				/* If super pointer is set */
				if (!(head.status[0] & F3S_EXT_NO_SUPER))
				{
					/* If super pointer logical unit is within range */
					if (head.super.logi_unit >= F3S_FIRST_LOGI && head.super.logi_unit <= (F3S_FIRST_LOGI + F3S_UNIT_ACTIVE (part)))
					{
						/* get link physical unit index */
						link_phys = f3s_extptr_phys (part, head.super);

						/* If super pointer is within range */
						if (head.super.index <= part->array->map_vect[link_phys].head_last)
						{
							f3s_head_t thead;
							int32_t tsize;

							F3S_UNIT_ENTER(part, link_phys);
							tsize = f3s_head_read(part, link_phys, head.super.index, NULL, &thead, 1); 
							F3S_UNIT_LEAVE(part, link_phys);
							if ((thead.status[0] & F3S_EXT_MASK) != F3S_EXT_ALLOC) 
							{
								f3s_log_bad (part->array->index, part->index, curr_phys, index, text_size, head.status[0]);
								f3s_log_bad (part->array->index, part->index, link_phys, head.super.index, tsize, thead.status[0]);
								if (f3s_fix_super(part, &curr_phys, index, &head) != EOK) {
									link_matrix[link_phys][head.super.index]++;
								}
							} else 
							{
								/* mark super header position in matrix */
								link_matrix[link_phys][head.super.index]++;
							}
						} else
						{
							/* log outside pointing extent */
							f3s_log_outside (part->array->index, part->index,
								head.super.logi_unit, link_phys, head.super.index);
							error = f3s_fix_super(part, &curr_phys, index, &head);
						}
					} else
					{
						F3S_UNIT_LEAVE(part, curr_phys);
						F3S_LOGI_LEAVE(part, logi_num);

						/* log junk pointing extent */
						f3s_log_junk (part->array->index, part->index,
						              logi_num, curr_phys, index);
						if (f3s_fix_super(part, &curr_phys, index, &head) != EOK) {
							return (f3s_ebadfsys (part->index, __func__, __LINE__));
						}
					}

				/* header is not superseded */
				}
				else
				{
					/* If next pointer is set */
					if (!(head.status[0] & F3S_EXT_NO_NEXT))
					{
						/* If next pointer logical unit is within range */
						if (head.next.logi_unit >= F3S_FIRST_LOGI && head.next.logi_unit <= (F3S_FIRST_LOGI + F3S_UNIT_ACTIVE (part)))
						{
							/* get link physical unit index */
							link_phys = f3s_extptr_phys (part, head.next);

							/* If next pointer is within range */
							if (head.next.index <= part->array->map_vect[link_phys].head_last)
							{
								f3s_head_t thead;
								int32_t tsize;

								F3S_UNIT_ENTER(part, link_phys);
								tsize = f3s_head_read(part, link_phys, head.next.index, NULL, &thead, 1); 
								F3S_UNIT_LEAVE(part, link_phys);
								if ((thead.status[0] & F3S_EXT_MASK) != F3S_EXT_ALLOC) 
								{
									f3s_log_bad (part->array->index, part->index, curr_phys, index, text_size, head.status[0]);
									f3s_log_bad (part->array->index, part->index, link_phys, head.next.index, tsize, thead.status[0]);
									if (f3s_fix_next(part, &curr_phys, index, &head) != EOK) {
										link_matrix[link_phys][head.next.index]++;
									}
								} else
								{
									/* mark next header position in matrix */
									link_matrix[link_phys][head.next.index]++;
								}
							} else
							{
								/* log outside pointing extent */
								f3s_log_outside (part->array->index, part->index,
								                 logi_num, curr_phys, index);
								error = f3s_fix_next(part, &curr_phys, index, &head);
							}
						} else
						{
							F3S_UNIT_LEAVE(part, curr_phys);
							F3S_LOGI_LEAVE(part, logi_num);

							/* log junk pointing extent */
							f3s_log_junk (part->array->index, part->index,
							              logi_num, curr_phys, index);
							if (f3s_fix_next(part, &curr_phys, index, &head) != EOK) {
								return (f3s_ebadfsys (part->index, __func__, __LINE__));
							}
						}
					}

					/* If header type is system */
					if ((head.status[0] & F3S_EXT_TYPE) == F3S_EXT_SYS)
					{
						/* mark header position in matrix */
						link_matrix[curr_phys][index]++;

						/* If extent is boot info */
						if (index == F3S_BOOT_INDEX)
						{
							/* If root pointer logical unit is within range */
							f3s_lock_assert(part->table_lock.thread == pthread_self());
							if ((part->table.root_extptr.logi_unit >= F3S_FIRST_LOGI) &&
							    (part->table.root_extptr.logi_unit <= (F3S_FIRST_LOGI + F3S_UNIT_ACTIVE(part))))
							{
								/* get link physical unit index */
								link_phys = f3s_extptr_phys (part, part->table.root_extptr);

								/* If root pointer is within range */
								f3s_lock_assert(part->array->map_lock_vect[link_phys].thread == pthread_self());

								if (part->table.root_extptr.index <= part->array->map_vect[link_phys].head_last)
								{
									/* mark root header position in matrix */
									link_matrix[link_phys][part->table.root_extptr.index]++;
								} else
								{
									/* log outside pointing extent */
									f3s_log_outside (part->array->index, part->index,
									                 logi_num, curr_phys, index);
								}
							} else
							{
								F3S_UNIT_LEAVE(part, curr_phys);
								F3S_LOGI_LEAVE(part, logi_num);

								/* log junk pointing extent */
								f3s_log_junk (part->array->index, part->index,
								              logi_num, curr_phys, index);
								return (f3s_ebadfsys (part->index, __func__, __LINE__));
							}
						}

					/* If header type is directory */
					} else if ((head.status[0] & F3S_EXT_TYPE) == F3S_EXT_DIR)
					{
						/* If dirent text has a size */
						if (text_size)
						{
							/* read dirent extent */
							error = f3s_flash_read (part->array, curr_phys, text_offset, sizeof (dirent), (uint8_t *) & dirent);
							if (error)
							{
								F3S_UNIT_LEAVE(part, curr_phys);
								F3S_LOGI_LEAVE(part, logi_num);
								return (error);
							}
							text_size = sizeof(dirent);

							/* If dirent structure needs swapping */
							f3s_lock_assert(part->table_lock.thread == pthread_self());
							if (part->table.swap_flag) f3s_dirent_swap (&dirent);

							/* If dirent is completely commited */
							if (1)
							{
								/* If first pointer logical unit is within range */
								if (dirent.first.logi_unit >= F3S_FIRST_LOGI && dirent.first.logi_unit <= (F3S_FIRST_LOGI + F3S_UNIT_ACTIVE (part)))
								{
									/* get link physical unit index */
									link_phys = f3s_extptr_phys (part, dirent.first);

									/* If first pointer index is within range */
									if (dirent.first.index <= part->array->map_vect[link_phys].head_last)
									{
										f3s_head_t thead; 
										int32_t tsize;

										F3S_UNIT_ENTER(part, link_phys);
										tsize = f3s_head_read(part, link_phys, dirent.first.index, NULL, &thead, 1); 
										F3S_UNIT_LEAVE(part, link_phys);
										if ((thead.status[0] & F3S_EXT_MASK) != F3S_EXT_ALLOC) 
										{
											f3s_log_bad (part->array->index, part->index, link_phys, dirent.first.index, tsize, thead.status[0]);
											if (f3s_fix_dirent(part, logi_num, index) != EOK) {
												link_matrix[link_phys][dirent.first.index]++;
											}
										} else
										{
											/* mark first header position in matrix */
											link_matrix[link_phys][dirent.first.index]++;
										}
									} else
									{
										/* log outside pointing extent */
										f3s_log_outside (part->array->index, part->index,
										                 logi_num, curr_phys, index);
										error = f3s_fix_dirent(part, logi_num, index);
									}
								} else
								{
									F3S_UNIT_LEAVE(part, curr_phys);
									F3S_LOGI_LEAVE(part, logi_num);

									/* log junk pointing extent */
									f3s_log_junk (part->array->index, part->index,
									              logi_num, curr_phys, index);
									if (f3s_fix_dirent(part, logi_num, index) != EOK) {
										return (f3s_ebadfsys (part->index, __func__, __LINE__));
									}
								}

								/* increment dirent counters */
								F3S_ALLOC_ENTER(part);
								part->table.dirent_num++;
								F3S_ALLOC_LEAVE(part);

							}
						}
					}
				}

			/* header is unused stuff */
			}
			else
			{
				/* mark unused header position in matrix */
				link_matrix[curr_phys][index]++;
			}

			/* If header is last on unit */
			if (head.status[0] & F3S_EXT_LAST) break;
		}
		F3S_UNIT_LEAVE(part, curr_phys);
		F3S_LOGI_LEAVE(part, logi_num);
	}

	return (EOK);
/*}}}1*/
}

int f3s_recover_reclaim (f3s_part_t *part)
{
/*{{{1*/
	f3s_unit_info_t	unit_info;
	f3s_unit_logi_t	unit_logi;
	f3s_map_t*		map;
	int32_t			spare_flag;
	int32_t			curr_phys;
	int32_t			old_phys;
	int32_t			spare_num = 0;
	int				readonly = 0;
	int				error;
	uint32_t		avg_erasecount = 0;

	/* Reset physical unit index */
	curr_phys = part->unit_first;

	/* Unit find loop */
	do
	{
		F3S_UNIT_ENTER(part, curr_phys);
		spare_flag = 0;
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];
		memset (map, 0, sizeof (f3s_map_t));

		/* Read unit info structure */
		error = f3s_unit_info_read (part, curr_phys, &unit_info, 1);
		if (error)
		{
			/* If the unit has been retired */
			if (error == ENODATA) {
				spare_num++;
				f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
				part->array->phys_vect[curr_phys] = 0;

			/* Consider the unit partially erased */
			} else
			{
				/* Log partial reclaim */
				f3s_log_partial (part->array->index, part->index, curr_phys);

				spare_num++;
				f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
				part->array->phys_vect[curr_phys] = 0;

				if (f3s.recover)
				{
					/* XXX: Defer the erase until later.  Right now, just invalidate
					 * the block, incase it wasn't fully done the first time. */
					//error = f3s_unit_inval(part, curr_phys);
					spare_flag = 1;
				}
			}

			goto loop;
		}
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
		map->erase_count = unit_info.erase_count;	

		/* Get the total erase count, for averaging later. */
		avg_erasecount += unit_info.erase_count;

		/* If unit is read-only */
		error = f3s_flash_islock(part->array, curr_phys);
		if ((error) && (error != ENOTSUP)) readonly = 1;

		/* Read the logical unit number */
		error = f3s_unit_logi_read (part, curr_phys, &unit_logi);
		if (error)
		{
			if (error == EBADFSYS) {
				/* Log partial reclaim */
				f3s_log_partial (part->array->index, part->index, curr_phys);

				spare_num++;
				f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
				part->array->phys_vect[curr_phys] = 0;

				if (f3s.recover)
				{
					//f3s_unit_inval(part, curr_phys);
					spare_flag = 1;
				}
			} else
			{
				/* Consider the unit a spare */
				spare_num++;
				f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
				part->array->phys_vect[curr_phys] = 0;
				part->array->age_vect[curr_phys]  = 0xFF;
				spare_flag = 1;
			}

		} else
		{
		
			/* If logical unit number is not within range */
			if (unit_logi.logi > (F3S_FIRST_LOGI + part->unit_num))
			{
				return f3s_ebadfsys (part->index, __func__, __LINE__);
			}

			/* Register unit */
			f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
			part->array->phys_vect[curr_phys] = unit_logi.logi;
			part->array->age_vect[curr_phys]  = unit_logi.age;
			part->table.logi_vect[unit_logi.logi] = curr_phys;

			/* Search for duplicate logical block numbers */
			for (old_phys = part->unit_first; old_phys < curr_phys; old_phys++)
			{
				/* If we find a duplicate */
				if (part->array->phys_vect[old_phys] == unit_logi.logi)
				{
					if ((part->array->age_vect[old_phys] > unit_logi.age) &&
					    (unit_logi.age != -1))
					{
						/* Log partial reclaim */
						f3s_log_partial (part->array->index, part->index, old_phys);

						/* Make this unit a spare */
						spare_num++;
						f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[old_phys].thread == pthread_self()));
						part->array->phys_vect[old_phys] = 0;

						/* If recovery is allowed */
						if (!(readonly) && (f3s.recover))
						{
							F3S_UNIT_ENTER(part, old_phys);

							/*
							 * This code is not redundant, since we must
							 * retire the already registered block instead
							 * of the current one.
							 */
							map = &part->array->map_vect[old_phys];
								
							/* Setup the map correctly for the spare */
							f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
							map->text_size    = F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t), F3S_SYS_ALIGN);
							map->free_size    = part->array->unit_size;
							map->free_size   -= F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);
							map->head_last    = F3S_UNIT_INFO_INDEX;
							map->free_first   = F3S_UNIT_INFO_INDEX + 1;

							/* Log spare unit found */
							f3s_log_spare (part->array->index, part->index, old_phys);

							/* spare_drop does the UNIT_LEAVE */
							f3s_spare_drop (part, old_phys, F3S_DIRTY);

						/* Error detected, prevent writes */
						} else
						{
							readonly = 1;
						}

					/* current unit is oldest */
					} else
					{
						/* Log partial reclaim */
						f3s_log_partial (part->array->index, part->index, curr_phys);

						/* Make this unit a spare */
						spare_num++;
						f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
						part->array->phys_vect[curr_phys] = 0;
						part->table.logi_vect[unit_logi.logi] = old_phys;

						/* If recovery is allowed */
						if (!(readonly) && (f3s.recover))
						{
							/* spare_drop unlocks curr_phys,
							 * but we want it to remain locked,
							 * so we UNIT_ENTER an extra time, to
							 * keep the lock count > 0.
							 */
							spare_flag = 1;

						/* Error detected, prevent writes */
						} else
						{
							readonly = 1;
						}
					}
				}
			}
		}

loop:
		/* If spare flag is set */
		if (spare_flag)
		{
			f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
			/* Setup the map correctly for the spare */
			map->text_size    = F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t), F3S_SYS_ALIGN);
			map->free_size    = part->array->unit_size;
			map->free_size   -= F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);
			map->head_last    = F3S_UNIT_INFO_INDEX;
			map->free_first   = F3S_UNIT_INFO_INDEX + 1;
		
			/* log spare unit found */
			f3s_log_spare (part->array->index, part->index, curr_phys);

			f3s_spare_drop (part, curr_phys, F3S_DIRTY);

		}else{
			F3S_UNIT_LEAVE(part, curr_phys);
		}

		/* If number of spare units exceeds what's in boot info */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		if (spare_num > part->table.unit_spare)
		{
			/* this file system is not good */
			return (f3s_ebadfsys (part->index, __func__, __LINE__));
		}

	/* while unit is within partition table */
	} while (++curr_phys < (part->unit_first + part->unit_num));

	/* Scan through all spare sectors and, if they are dirty, set their
	 * erase count to average erase count.
	 */
	avg_erasecount /= part->unit_num;
	{
		int num_spare = part->table.unit_spare;
		while(num_spare--){
			int dirty;
			int32_t spare_blk;
			spare_blk = f3s_spare_grab(part, &dirty);
			if((F3S_DIRTY == dirty) && (part->array->map_vect[spare_blk].erase_count == 0)){
				part->array->map_vect[spare_blk].erase_count = avg_erasecount;
			}
			f3s_spare_drop(part, spare_blk, dirty);
		}
	}

	if (readonly) return (EROFS);
	return (EOK);
/*}}}1*/
}

int f3s_recover_write (f3s_part_t *part, int32_t *reclaim_vect)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_map_t *		map;
	uint32_t		text_size;
	int32_t			curr_phys;
	uint32_t		text_offset;
	uint16_t		index;
	int32_t			readonly = 0;
	uint32_t		align_size, head_size;
	uint32_t		sys_size;
	int32_t			logi_num;
	int				error;
	uint16_t		last_index = 0;

	/* unit map loop */
	for (logi_num = F3S_FIRST_LOGI; logi_num <= F3S_UNIT_ACTIVE (part); logi_num++)
	{
		F3S_LOGI_ENTER(part, logi_num);
		curr_phys = part->table.logi_vect[logi_num];
		
		/* Initialize map */
		F3S_UNIT_ENTER(part, curr_phys);
		f3s_lock_assert((part->table_lock.thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];
		
		/* Skip spare units */
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));


		/* Walk every extent header */
		sys_size = 0;
		index = 0;
		while (1)
		{
			text_size = 0;
			/* Read extent header */
			error = f3s_head_read (part, curr_phys, index, &text_offset, &head, 2);
			if (error < 0)
			{
				error = errno;
				goto error;
			}else{
				text_size = (uint32_t)error;
			}

			head_size = (index + 1) << F3S_HEAD_POW2;

			/* If header is allocated */
			if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC)
			{
				/* Must count the size regardless of written state */
				if (head.status[0] & F3S_EXT_NO_WRITE) 
				{
					text_size = head.text_size;
					f3s_lock_assert(part->table_lock.thread == pthread_self());
				}

				/* If we are not dealing with a system extent */
				if ((head.status[0] & F3S_EXT_TYPE) != F3S_EXT_SYS)
				{
					/* Get aligned size for regular extent */
					f3s_lock_assert(part->table_lock.thread == pthread_self());
					align_size = F3S_TEXT_ALIGN(part, text_size, part->table.align_size);

				} else
				{
					/* Get aligned size for system extent */
					align_size = F3S_TEXT_ALIGN(part, text_size, F3S_SYS_ALIGN);
					sys_size += align_size;
				}

				/* Check text_offset and text_size */
				if ((text_size   >= part->array->unit_size) ||
				    (text_offset >= part->array->unit_size) ||
				    ((text_offset + align_size) > (part->array->unit_size - head_size))) 
				{
					f3s_log_overlap (part->array->index, part->index, 
						curr_phys,index, text_size, text_offset);

					if(f3s.recover) 
					{
						f3s_head_status (part, &curr_phys, index, ~F3S_EXT_MASK | F3S_EXT_BAD, 0);
						map->del_num++;
						goto check_index;
					}
					error = f3s_ebadfsys (part->index, __func__, __LINE__);
					goto error;
				}

				/* Update the size of the text area */
				if (map->text_size < text_offset + align_size)
				{
					map->text_size = text_offset + align_size;
				}

				if (head.status[0] & F3S_EXT_NO_WRITE) 
				{
					map->stale_size += align_size;
				}

				/* If header is stale because superseded */
				if (!(head.status[0] & F3S_EXT_NO_SUPER))
				{
					map->stale_size += align_size;
					f3s_assert (map->stale_size < map->text_size);

				/* Header has good stuff */
				} else
				{
					map->pad_size += align_size - text_size;
				}

			/* If index is deleted */
			} else if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_DELETE)
			{
				/* Must count the size regardless of written state */
				if (head.status[0] & F3S_EXT_NO_WRITE) 
				{
					text_size = head.text_size;
				}

				f3s_lock_assert(part->table_lock.thread == pthread_self());
				align_size = F3S_TEXT_ALIGN(part, text_size, part->table.align_size);

				/* Check text_offset and text_size */
				if ((text_size   >= part->array->unit_size) ||
				    (text_offset >= part->array->unit_size) ||
				    ((text_offset + align_size) > (part->array->unit_size - head_size))) 
				{
					f3s_log_overlap (part->array->index, part->index, 
						curr_phys,index, text_size, text_offset);

					if(f3s.recover) 
					{
						f3s_head_status (part, &curr_phys, index, ~F3S_EXT_MASK | F3S_EXT_BAD, 0);
						map->del_num++;
						goto check_index;
					}
					error = f3s_ebadfsys (part->index, __func__, __LINE__);
					goto error;
				}

				/* Update the size of the text area */
				if (map->text_size < text_offset + align_size)
				{
					map->text_size = text_offset + align_size;
				}

				/* Add text size to unit stale space */
				map->stale_size += align_size;
				f3s_assert (map->stale_size < map->text_size);

				/* Increment deleted header counter */
				map->del_num++;

			/* If index is free */
			} else if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_FREE)
			{
				/* Make sure it's really free */
				if (!memcmp (((uint8_t *)&head) + sizeof(head.status), FF32,
				             sizeof(f3s_head_t) - sizeof(head.status)))
				{
					/* If free num is reset */
					if (!map->free_num)
					{
						/* setup index in free first */
						map->free_first = index;
					}

					/* "Holes" cannot be LAST */
					if (!(head.status[0] & F3S_EXT_LAST))
						map->free_num++;

					/* Count the free space */
					map->free_size -= sizeof(head);

				/* Header is partially allocated and needs repairing */
				} else if (f3s.recover)
				{
					/* Mark the header as bad */
					if (f3s_head_status (part, &curr_phys, index,
					                            ~F3S_EXT_MASK | F3S_EXT_BAD, 0) < 0)
					{
						readonly = 1;
					}

					/* Count the bad header as deleted */
					f3s_log_bad (part->array->index, part->index, curr_phys, index, text_size, head.status[0]);
					map->del_num++;
				}

			/* Must be bad */
			} else
			{
				/* Count the bad header as deleted */
				f3s_log_bad (part->array->index, part->index, curr_phys, index, text_size, head.status[0]);
				map->del_num++;
			}

check_index:
			/* If the text area overlaps with the extent headers */
			if (head_size > (part->array->unit_size - map->text_size)) {
				f3s_log_overlap (part->array->index, part->index, 
					curr_phys, index, map->text_size, text_offset);

				if(f3s.recover) break;
				error = f3s_ebadfsys (part->index, __func__, __LINE__);
				goto error;
			}

			/* Update last_index. The last_index is 1 at least, since f3s_recover_reclaim()
			 * has already verified the first two system extents. 
			 */
			last_index = index; 

			/* If header is last on unit */
			if (head.status[0] & F3S_EXT_LAST)
			{
				/* set last header entry to index */
				map->head_last = index;

				/* If free num is reset */
				if (!map->free_num)
				{
					/* setup index in free first */
					map->free_first = index;
				}

				break;
			}

			/* increment header index number */
			index++;
		}

		/* Check if the F3S_EXT_FLAG missed */ 
		if (map->head_last == 0) 
		{
			map->head_last = last_index;
			f3s_log_head_last(part->array->index, part->index, logi_num, curr_phys, map->head_last);
		}
		head_size = (map->head_last + 1) << F3S_HEAD_POW2;

		/* Align map size */
		//f3s_lock_assert(part->table_lock.thread == pthread_self());
		align_size = F3S_TEXT_ALIGN(part, sys_size, part->table.align_size);
		map->pad_size += align_size - sys_size;
		map->text_size = max(align_size, map->text_size);
		map->free_size = part->array->unit_size - (map->text_size + head_size);

		/* Verify map of current phys */
		if ((map->text_size  >= part->array->unit_size) ||
		    ((map->text_size + head_size) > part->array->unit_size))
		{
			f3s_log_map (part->array->index, part->index, logi_num, curr_phys, map->text_size, map->free_size, map->stale_size, map->pad_size, map->head_last);
			error = f3s_ebadfsys (part->index, __func__, __LINE__);
			goto error;
		} 
		if ((map->stale_size > map->text_size) || (map->pad_size >= part->array->unit_size)) 
		{
			f3s_log_map (part->array->index, part->index, logi_num, curr_phys, map->text_size, map->free_size, map->stale_size, map->pad_size, map->head_last);
		}

		/* Update table counters */
		F3S_ALLOC_ENTER(part);
		part->table.free_total  += map->free_size;
		part->table.stale_total += map->stale_size;
		part->table.pad_total   += map->pad_size;
		part->table.head_num    += map->head_last + 1;
		F3S_ALLOC_LEAVE(part);

		F3S_UNIT_LEAVE(part, curr_phys);
		F3S_LOGI_LEAVE(part, logi_num);
	}

	if (readonly) return (EROFS);
	else          return (EOK);

error:
	F3S_UNIT_LEAVE(part, curr_phys);
	F3S_LOGI_LEAVE(part, logi_num);
	return (error);
/*}}}1*/
}

int f3s_cleanup_multi (f3s_part_t *part, f3s_extptr_t multi)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_dirent_t	dirent[2];
	f3s_extptr_t	extptr[2];
	f3s_desc_t		dir_desc;
	f3s_map_t *		map;
	int32_t			curr_phys;
	int32_t			text_size;
	uint32_t		text_offset;
	int32_t			link_phys;
	int32_t			multi_count = 0;
	int32_t			logi_num;
	int				err_size;
	int				error;
	uint16_t		index;

	/* unit map loop */
	for (logi_num = F3S_FIRST_LOGI; logi_num <= F3S_UNIT_ACTIVE(part); logi_num++)
	{
		F3S_LOGI_ENTER(part, logi_num);
		curr_phys = part->table.logi_vect[logi_num];
		F3S_UNIT_ENTER(part, curr_phys);

		/* setup map pointer */
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];

		/* header find loop */
		for (index = 0; index <= map->head_last; index++)
		{
			/* read header info structure */
			text_size = f3s_head_read (part, curr_phys, index,
			                           &text_offset, &head, 1);

			/* if text size makes no sense */
			if (text_size < 0)
			{
				error = errno;
				F3S_UNIT_LEAVE(part, curr_phys);
				F3S_LOGI_LEAVE(part, logi_num);
				return (error);
			}

			/* if header is good stuff */
			if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC)
			{
				/* if super pointer is not set */
				if (head.status[0] & F3S_EXT_NO_SUPER)
				{
					/* if header type is directory */
					if ((head.status[0] & F3S_EXT_TYPE) == F3S_EXT_DIR)
					{
						/* if dirent text has a size */
						if (text_size)
						{
							/* read dirent extent */
							error = f3s_flash_read (part->array, curr_phys, text_offset,
							                        sizeof(*dirent), (uint8_t *)&dirent[multi_count]);
							if (error) {
								F3S_UNIT_LEAVE(part, curr_phys);
								F3S_LOGI_LEAVE(part, logi_num);
								return (error);
							}
							text_size = sizeof(*dirent);

							/* if dirent structure needs swapping */
							f3s_lock_assert(part->table_lock.thread == pthread_self());
							if (part->table.swap_flag) f3s_dirent_swap (&dirent[multi_count]);

							/* if dirent is completely commited */
							if (1)
							{
								/* get link physical unit index */
								link_phys = f3s_extptr_phys (part, dirent[multi_count].first);

								/* if first pointer index is within range */
								if (dirent[multi_count].first.index <=
								    part->array->map_vect[link_phys].head_last)
								{
									/* if dirent points to multi */
									if (!memcmp (&dirent[multi_count].first, &multi,
									             sizeof (f3s_extptr_t)))
									{
										/* setup extent pointer into buffer */
										f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
										extptr[multi_count].logi_unit = part->array->phys_vect[curr_phys];
										extptr[multi_count].index = index;

										/* increment multi count */
										multi_count++;
									}
								}
							}
						}
					}
				}
			}

			/* if multi count is bigger or equal to two */
			if (multi_count >= 2) break;
		}
		F3S_UNIT_LEAVE(part, curr_phys);
		F3S_LOGI_LEAVE(part, logi_num);

		/* if multi count is bigger or equal to two */
		if (multi_count >= 2) break;
	}

	/* if multi count is smaller than two */
	if (multi_count < 2) return (f3s_ebadfsys(part->index, __func__, __LINE__));

	/* if first dirent is older than the second */
	if ((uint8_t)(dirent[0].moves + 1) == dirent[1].moves)
	{
		/* setup directory descriptor with first entry */
		f3s_desc_init (&dir_desc, extptr[0], ~F3S_DESC_NO_DIR);

	/* if second dirent is older than the first */
	} else if ((uint8_t) (dirent[1].moves + 1) == dirent[0].moves)
	{
		/* setup directory descriptor with first entry */
		f3s_desc_init (&dir_desc, extptr[1], ~F3S_DESC_NO_DIR);

	/* this is a bad file system */
	} else
	{
		return (f3s_ebadfsys (part->index, __func__, __LINE__));
	}

	/* temporarily reserve a spot for a header */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	f3s_resv_alloc (part, &part->table.deletion_resv, 0);

	/* delete entry in directory descriptor */
	err_size = f3s_entry_delete (part, &dir_desc, 0);
	error = errno;
	if (err_size < 0) return (error);

	/* free temporary spot for a header */
	f3s_resv_free (part, &part->table.deletion_resv);
	return (EOK);
/*}}}1*/
}

#if 0
/*{{{1 - Junk we ripped out from f3s_recover_write()*/
			/* If header is allocated */
			if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC)
			{
				/* If no write bit is set */
				if (head.status[0] & F3S_EXT_NO_WRITE)
				{
					/* log dirty extent */
					f3s_log_dirty (part->array->index, part->index,
					               unit_logi.logi, curr_phys, index);

					/* If recovery is wanted */
					if (f3s.recover)
					{
						/* mark header for deletion */
						err_size = f3s_head_status (part, &curr_phys, index,
						                            ~F3S_EXT_MASK | F3S_EXT_DELETE,
						                            0);
						if (err_size < 0)
						{
							/* log sticky extent */
							f3s_log_sticky (part->array->index, part->index,
							                unit_logi.logi, curr_phys, index);
						}
					}

				/* header contents are valid */
				} else
				{
					/* If we are not dealing with a system extent */
					if ((head.status[0] & F3S_EXT_TYPE) != F3S_EXT_SYS)
					{
						/* get aligned size for regular extent */
						f3s_lock_assert(part->table_lock.thread == pthread_self());
						align_size = F3S_TEXT_ALIGN(part, text_size, part->table.align_size);

					} else
					{
						/* get aligned size for system extent */
						align_size = F3S_TEXT_ALIGN(part, text_size, F3S_SYS_ALIGN);

						/* add to sys size */
						sys_size += align_size;
					}

					/* If text offset plus size go past unit offset */
					if (map->text_size < text_offset + align_size)
					{
						/* set text size to header offset plus size */
						map->text_size = text_offset + align_size;

					}

					/* If header is stale because superseded */
					if (!(head.status[0] & F3S_EXT_NO_SUPER))
					{
						/* add text size to unit stale space */
						map->stale_size += align_size;
						f3s_assert (map->stale_size < map->text_size);

					/* header has good stuff */
					} else
					{
						/* If super pointer not erased */
						if ((head.super.logi_unit != f3s_null_extptr.logi_unit) || (head.super.index != f3s_null_extptr.index))
						{
							/* log damaged extent */
							f3s_log_damage (part->array->index, part->index,
							                unit_logi.logi, curr_phys, index);

							/* If recovery is wanted */
							if (f3s.recover)
							{
								/* set reclaim flag */
								reclaim_vect[curr_phys] = 1;

							} else
							{
								/* this partition can not be written */
								readonly = 1;
							}
						}

						/* If no next bit is set and next pointer not erased */
						if ((head.status[0] & F3S_EXT_NO_NEXT) && ((head.next.logi_unit != f3s_null_extptr.logi_unit) || (head.next.index != f3s_null_extptr.index)))
						{
							/* log damaged extent */
							f3s_log_damage (part->array->index, part->index,
							                unit_logi.logi, curr_phys, index);

							/* If recovery is wanted */
							if (f3s.recover)
							{
								/* set reclaim flag for unit */
								reclaim_vect[curr_phys] = 1;

							} else
							{
								/* this partition can not be written */
								readonly = 1;
							}
						}

						/* calculate padding size */
						map->pad_size += align_size - text_size;
					}
				}

			/* If index is deleted */
			} else if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_DELETE)
			{
				/* If header was written */
				if (!(head.status[0] & F3S_EXT_NO_WRITE))
				{
					/* get aligned size */
					f3s_lock_assert(part->table_lock.thread == pthread_self());
					align_size = F3S_TEXT_ALIGN(part, text_size, part->table.align_size);

					/* If text offset plus size go past unit offset */
					if (map->text_size < text_offset + align_size)
					{
						/* set text size to header offset plus size */
						map->text_size = text_offset + align_size;
					}

					/* add text size to unit stale space */
					map->stale_size += align_size;
					f3s_assert (map->stale_size < map->text_size);
				}

				/* increment deleted header counter */
				map->del_num++;

			/* If index is free */
			} else if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_FREE)
			{
				/* If free num is reset */
				if (!map->free_num)
				{
					/* setup index in free first */
					map->free_first = index;
				}

				/* increment number of free indexes */
				map->free_num++;
				map->free_size -= sizeof(head);
			}

			/* If header is last on unit */
			if (head.status[0] & F3S_EXT_LAST)
			{
				/* If header is free */
				if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_FREE)
				{
					/* log virgin extent */
					f3s_log_virgin (part->array->index, part->index,
					                unit_logi.logi, curr_phys, index);

					/* If recovery is wanted */
					if (f3s.recover)
					{
						/* set reclaim flag for unit */
						reclaim_vect[curr_phys] = 1;

						/* decrement index */
						index--;
					}
				}

				/* set last header entry to index */
				map->head_last = index;

				/* If free num is reset */
				if (!map->free_num)
				{
					/* setup index in free first */
					map->free_first = index;
				}

				break;
			}

			/* If there is room for a header */
			if ((index << F3S_HEAD_POW2) >= (part->array->unit_size - map->text_size))
			{
				error = f3s_ebadfsys (part->index, __func__, __LINE__);
				goto error;
			}

			/* increment header index number */
			index++;
/*}}}1*/
#endif

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/recover.c $ $Rev: 580669 $" );
