/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */


#include "f3s.h"

const f3s_extptr_t f3s_null_extptr  = {~0, ~0};
const f3s_extptr_t f3s_first_extptr = {F3S_FIRST_LOGI, F3S_FIRST_INDEX};
const f3s_extptr_t f3s_root_extptr  = {F3S_FIRST_LOGI, F3S_ROOT_INDEX};
const f3s_extptr_t f3s_boot_extptr  = {F3S_FIRST_LOGI, F3S_BOOT_INDEX};

f3s_t f3s; /* f3s global structure */
thread_pool_attr_t	pool_attr;
pthread_attr_t		pthread_attr; /* pthread attr for the thread pool
									 must always remain in scope */

#ifndef resmgr_unbind
/* missing proto in older resmgr.h */
extern int resmgr_unbind(resmgr_context_t *__ctp);
#endif	/* resmgr_unbind */

/* for backwards compatibility, provide an implementation for f3s_mtd_buildInfo */
extern const char * __attribute__((weak)) f3s_mtd_buildInfo(void)
{
	return("MTD Build: No Info available");
}



void f3s_init (int argc, char **argv, f3s_flash_t *flash_vect)
{
/*{{{1*/
	int err;
	sigset_t			set;
	/* process command line arguments */
	f3s_option (argc, argv, flash_vect);
	ThreadCtl(_NTO_TCTL_IO, 0);

	if (f3s.verbose >= 1)
		printf("Flash Development Library - HEAD\nBuild: %s %s\n%s\n",
					__DATE__, __TIME__, f3s_mtd_buildInfo());
		
	if (!f3s.exit_on_fault)
	{
		/* become a daemon */
		f3s.daemon_id = procmgr_daemon (EXIT_SUCCESS, PROCMGR_DAEMON_NODEVNULL);
		if (f3s.daemon_id < 0)
		{
			perror ("unable to become daemon");
			exit (EXIT_FAILURE);
		}
	}

	while ((err = nanospin_calibrate (0)) != EOK && err != EINTR);
	if (err != EOK) perror ("nanospin_calibrate failed");

	/* initialize resmgr dispatch */
	f3s.dpp = dispatch_create ();
	if (!f3s.dpp)
	{
		perror ("unable to create dispatch");
		exit (EXIT_FAILURE);
	}

	/* setup maximum sizes for resource manager */
	f3s.resmgr_attr.flags = RESMGR_FLAG_CROSS_ENDIAN;
	f3s.resmgr_attr.nparts_max = 1;
	f3s.resmgr_attr.msg_max_size = F3S_MSG_MAX;

	/* initialize connect function structure for mnt mountpoints */
	iofunc_func_init (_RESMGR_CONNECT_NFUNCS, &f3s.mnt_connect_funcs, 0, NULL);

	f3s.mnt_connect_funcs.open     = NULL;
	f3s.mnt_connect_funcs.rename   = NULL;
	f3s.mnt_connect_funcs.unlink   = NULL;
	f3s.mnt_connect_funcs.mknod    = NULL;
	f3s.mnt_connect_funcs.link     = NULL;
	f3s.mnt_connect_funcs.readlink = NULL;
	f3s.mnt_connect_funcs.mount    = f3s_mnt_mount;

	/* initialize connect and io function structures for raw mountpoints */
	iofunc_func_init (_RESMGR_CONNECT_NFUNCS, &f3s.raw_connect_funcs,
	                  _RESMGR_IO_NFUNCS,      &f3s.raw_io_funcs);

	f3s.raw_connect_funcs.mount = f3s_raw_unmount;

	f3s.raw_io_funcs.devctl    = f3s_raw_devctl;
	f3s.raw_io_funcs.read      = f3s_raw_read;
	f3s.raw_io_funcs.write     = f3s_raw_write;
	f3s.raw_io_funcs.close_ocb = f3s_raw_close_ocb;
	/* next 2 lines added to solve out of order locking issue */
	f3s.raw_io_funcs.lock_ocb   = f3s_rawio_lock;
	f3s.raw_io_funcs.unlock_ocb = f3s_rawio_unlock;

	/* initialize connect and io function structures for file system */
	iofunc_func_init (_RESMGR_CONNECT_NFUNCS, &f3s.f3s_connect_funcs,
	                  _RESMGR_IO_NFUNCS,      &f3s.f3s_io_funcs);

	f3s.f3s_connect_funcs.open     = f3s_open;
	f3s.f3s_connect_funcs.rename   = f3s_rename;
	f3s.f3s_connect_funcs.unlink   = f3s_unlink;
	f3s.f3s_connect_funcs.mknod    = f3s_mknod;
	f3s.f3s_connect_funcs.link     = f3s_link;
	f3s.f3s_connect_funcs.readlink = f3s_readlink;
	f3s.f3s_connect_funcs.mount    = f3s_umount;

	f3s.f3s_io_funcs.read      = f3s_read;
	f3s.f3s_io_funcs.write     = f3s_write;
	f3s.f3s_io_funcs.space     = f3s_space;
	f3s.f3s_io_funcs.close_ocb = f3s_close_ocb;
	f3s.f3s_io_funcs.lseek     = f3s_lseek;
	f3s.f3s_io_funcs.stat      = f3s_stat;
	f3s.f3s_io_funcs.devctl    = f3s_devctl;
	f3s.f3s_io_funcs.pathconf  = f3s_pathconf;
	f3s.f3s_io_funcs.sync      = f3s_sync;
	/* next 2 lines added to solve out of order locking issue */
	f3s.f3s_io_funcs.lock_ocb   = f3s_file_lock;
	f3s.f3s_io_funcs.unlock_ocb = f3s_file_unlock;

	/* initialize io function structure for restricted file system */
	iofunc_func_init (0, NULL, _RESMGR_IO_NFUNCS, &f3s.res_io_funcs);

	f3s.res_io_funcs.read      = f3s_read;
	f3s.res_io_funcs.write     = f3s_append;
	f3s.res_io_funcs.close_ocb = f3s_close_ocb;
	f3s.res_io_funcs.lseek     = f3s_lseek;
	f3s.res_io_funcs.stat      = f3s_stat;
	f3s.res_io_funcs.devctl    = f3s_devctl;
	f3s.res_io_funcs.pathconf  = f3s_pathconf;
	f3s.res_io_funcs.sync      = f3s_sync;

	/* initialize io function structures for directories */
	iofunc_func_init (0, NULL, _RESMGR_IO_NFUNCS, &f3s.dir_io_funcs);

	f3s.dir_io_funcs.read       = f3s_readdir;
	f3s.dir_io_funcs.close_ocb  = f3s_close_ocb;
	f3s.dir_io_funcs.lseek      = f3s_lseek;
	f3s.dir_io_funcs.stat       = f3s_statdir;
	f3s.dir_io_funcs.devctl     = f3s_devctl;
	f3s.dir_io_funcs.pathconf   = f3s_pathconf;
	f3s.dir_io_funcs.lock_ocb   = f3s_dir_lock;
	f3s.dir_io_funcs.unlock_ocb = f3s_dir_unlock;

	/* initialize iofunc functions */
	f3s.f3s_funcs.nfuncs = 2;
	f3s.f3s_funcs.ocb_calloc = f3s_ocb_calloc;
	f3s.f3s_funcs.ocb_free   = f3s_ocb_free;

	/* initialize mounter attributes */
	f3s_attr_init (&f3s.mounter_attr, S_IFBLK | S_IRUSR | S_IRGRP | S_IWUSR,
	               NULL, NULL, NULL, NULL);

	/* attach mount */
	f3s.mounter_id = resmgr_attach (f3s.dpp, &f3s.resmgr_attr, NULL, _FTYPE_MOUNT,
	                                _RESMGR_FLAG_DIR | _RESMGR_FLAG_FTYPEONLY,
	                                &f3s.mnt_connect_funcs, NULL, &f3s.mounter_attr);
	if (f3s.mounter_id < 0)
	{
		perror ("unable to attach mounter");
		exit (EXIT_FAILURE);
	}

	/* Create the thread pool.  since there aren't any attached names yet, the pool
	 * isn't actually going to do anything.  Also, the startup and scanning code
	 * is already properly locked, so being multi-threaded isn't going to hurt
	 * anything.
	 */

	/* Initialize resmgr thread pool */
	memset (&pool_attr, 0, sizeof(pool_attr));

	pool_attr.handle = f3s.dpp;

	pool_attr.context_alloc = dispatch_context_alloc;
	pool_attr.block_func    = dispatch_block;
	pool_attr.unblock_func  = dispatch_unblock;
	pool_attr.handler_func  = dispatch_handler;
	pool_attr.context_free  = dispatch_context_free;
	pool_attr.tid_name      = "I/O Handler";

	/*
	 * Note that if maximum and hiwater are equal and the caller is
	 * a thread from the thread pool, the caller will get stuck in SchedYield()
	 * (the reason for this is not yet clear) so allow the maximum number of
	 * threads to be 1 larger than the 'hiwater'
	*/
	pool_attr.lo_water  = f3s.thread_num - 1;
	pool_attr.hi_water  = f3s.thread_num;
	pool_attr.increment = 1;
	pool_attr.maximum   = F3S_THREAD_MAX + 1;

	pool_attr.attr = &pthread_attr;

	/* Initialize thread attributes */
	pthread_attr_init (&pthread_attr);

	/* Create thread pool */
	f3s.tpp = thread_pool_create (&pool_attr, 0);
	if (!f3s.tpp){
		resmgr_detach (f3s.dpp, f3s.mounter_id, _RESMGR_DETACH_ALL);
		exit (EXIT_FAILURE);
	}

	/* Mask out unecessary signals */
	sigfillset (&set);
	pthread_sigmask (SIG_BLOCK, &set, NULL);

	/* Start resource manager */
	thread_pool_start (f3s.tpp);
	
/*}}}1*/
}

void f3s_mount_init (f3s_mount_t *mount, dev_t dev, iofunc_funcs_t *funcs)
{
/*{{{1*/
	int id;

	/* initialize mount structure */
	mount->mount.flags = IOFUNC_MOUNT_32BIT;
	mount->mount.conf = IOFUNC_PC_CHOWN_RESTRICTED | IOFUNC_PC_NO_TRUNC | IOFUNC_PC_SYNC_IO;
	mount->mount.dev = dev;
	mount->mount.blocksize = 1;	// f3s.buffer_size

	mount->mount.funcs = funcs;

	/* initialize mount attach id vector */
	for (id = 0; id < F3S_ATTACH_NUM; id++)
		mount->attach_id[id] = -1;
/*}}}1*/
}

int f3s_start (f3s_service_t *service_vect, f3s_flash_t *flash_vect)
{
/*{{{1*/
	sigset_t			set;
	f3s_service_t *		service;
	f3s_mount_t *		mount[F3S_INDEX_MAX];
	f3s_access_t *		access;
	f3s_attr_t *		attr = NULL;
	f3s_socket_t *		socket;
	uint32_t			socket_index;
	uint32_t			window_index;
	uint32_t			window_num;
	int					error;

	/* initialize socket index */
	socket_index = f3s.array_index;

	/* Loop through the service vector */
	service = service_vect;

	/* Ensure that the service_vect has at least one good entry */
	if (service->struct_size == 0){
		/* Stop the thread pool */
		if (f3s.tpp){
			thread_pool_destroy (f3s.tpp);
			f3s.tpp = NULL;
		}
		if (f3s.mounter_id){
			resmgr_detach (f3s.dpp, f3s.mounter_id, _RESMGR_DETACH_ALL);
		}

		fprintf(stderr, "devf: No valid services entry found, check your main()\n");
		exit(EXIT_FAILURE);
	}
	while ((service->struct_size) && (socket_index < F3S_INDEX_MAX))
	{
		/* Allocate and initialize socket attributes */
		attr = f3s_calloc (sizeof (*attr));
		if (!attr) break;

		f3s_attr_init (attr, S_IFBLK | S_IRUSR | S_IRGRP | S_IWUSR, NULL,
		               NULL, NULL, NULL);

		/* Find a suitable device number */
		do
		{
			/* Initialize device number */
			attr->attr.rdev =
				rsrcdbmgr_devno_attach (F3S_BASE_TYPE,
				                        F3S_MAKE_MINOR(socket_index, 0, 0), 0);

			/* Stop when we find a good device number */
			if (((int32_t)attr->attr.rdev) >= 0) break;

			/* Try the next socket number */
			socket_index++;
		} while (socket_index < F3S_INDEX_MAX);

		/* check if socket index makes no sense */
		if (socket_index >= F3S_INDEX_MAX)
		{
			f3s_free (attr, sizeof (*attr));
			perror ("unable to attach devno");
			break;
		}

		/* Allocate and initialize socket structure */
		socket = f3s_calloc (sizeof (*socket));
		if (!socket)
		{
			f3s_free (attr, sizeof (*attr));
			attr = NULL;
			break;
		}
		socket->struct_size  = sizeof(*socket);
		socket->socket_index = socket_index;
		socket->option       = (uint8_t*)f3s.socket_option;

		/* open the socket */
		error = service->open (socket, F3S_OPER_SOCKET);
		if (error)
		{
			/* Increment service pointer and socket index */
			service++;
			socket_index++;

			f3s_free (attr,   sizeof(*attr));
			f3s_free (socket, sizeof(*socket));
			attr = NULL; socket = NULL;
			continue;
		}

		/* allocate a mount structure for socket */
		mount[socket_index] = f3s_calloc (sizeof(*mount[socket_index]));
		if (!mount[socket_index])
		{
			/* close the socket */
			service->close (socket, F3S_OPER_SOCKET);

			f3s_free (attr,   sizeof(*attr));
			f3s_free (socket, sizeof(*socket));
			attr = NULL; socket = NULL;
			break;
		}

		/* Initialize socket mount */
		f3s_mount_init (mount[socket_index], 0, NULL);

		/* Put mount in atribute structure */
		attr->attr.mount = mount[socket_index];

		/* Get number of windows from socket
		 * is this what is intended. I do not see where socket->window_num is being set. It is therefore
		 *  0 from f3s_calloc() above and threfore window_num will always be 1
		*/
		window_num = min(socket->window_num, f3s.thread_num);
		window_num = max(window_num, 1);

		/* Allocate an access vector for socket */
		access = f3s_calloc (window_num * sizeof(*access));
		if (!access)
		{
			/* close the socket */
			service->close (socket, F3S_OPER_SOCKET);

			/* free attribute, socket and mount memory */
			f3s_free (attr, sizeof (*attr));
			f3s_free (socket, sizeof (*socket));
			f3s_free (mount[socket_index], sizeof (*mount[socket_index]));
			attr = NULL; socket = NULL; mount[socket_index] = NULL;
			break;
		}

		/* allocate an array structure */
		mount[socket_index]->array = f3s_calloc (sizeof(*mount[socket_index]->array));
		if (!mount[socket_index]->array)
		{
			/* close the socket */
			service->close (socket, F3S_OPER_SOCKET);

			f3s_free (attr,                sizeof(*attr));
			f3s_free (socket,              sizeof(*socket));
			f3s_free (mount[socket_index], sizeof(*mount[socket_index]));
			f3s_free (access, window_num * sizeof(*access));
			attr = NULL; socket = NULL; mount[socket_index] = NULL; access = NULL;
			break;
		}

		/* setup database */
		mount[socket_index]->array->dbase.struct_size =
			sizeof(mount[socket_index]->array->dbase);

		/* initialize window key for array */
		error = pthread_key_create (&mount[socket_index]->array->window_key, NULL);

		/* put access into array */
		mount[socket_index]->array->access_num  = window_num;
		mount[socket_index]->array->access_vect = access;

		/* for all socket windows */
		for (window_index = 0; window_index < window_num; window_index++)
		{
			/* copy socket into access */
			access->socket = *socket;

			/* setup service pointer into access */
			access->service = service;

			/* setup window index into access */
			access->socket.window_index = window_index;

			/* open socket window */
			error = service->open (&access->socket, F3S_OPER_WINDOW);
			if (error) break;
			/* if multiple windows are opened, but one fails, do we just want to abort initialization of remaining? */
			/* increment access pointer */
			access++;
		}

		/* Check if no windows were opened */
		/* XXX - Is this check really valid? */
		/* check should be if (window_index < window_num) error handling. This break doesn't do any cleanup */
		if (!window_index) break;

		/* Attach socket mountpoint */
		error = f3s_skt_attach (mount[socket_index]->array, attr, flash_vect);
		if (error)
		{
			fprintf (stderr, "(devf  t%d::%s:%d) f3s_skt_attach failed (error = %d)\n",
						pthread_self(), __func__, __LINE__, error);
			exit (EXIT_FAILURE);
		}

		/* Increment service pointer and socket index */
		service++;
		socket_index++;
	}
	/* mark initialization complete (for mount options) */
	f3s.mount_options &= ~f3s_mntopts_t_INIT;
	
#if 0
	/* 
	 * This is already handled with the above pthread_sigmask.
	 * SignalWaitInfo will unblock the signals we are interested
	 * in automatically.
	 */
	 
	/* Mask out unecessary signals */
	sigfillset (&set);
	sigdelset (&set, SIGTERM);
	sigdelset (&set, SIGUSR1);
	sigdelset (&set, SIGUSR2);
	pthread_sigmask (SIG_BLOCK, &set, NULL);
#endif

	(void) pthread_setname_np(0, "Signal Handler");
	/* Wait for one of these signals */
	sigemptyset (&set);
	sigaddset (&set, SIGTERM);
	sigaddset (&set, SIGUSR1);
	sigaddset (&set, SIGUSR2);
	while (1) {
		int sig_handler_ret;
		/* Wait for a signal */
		switch (SignalWaitinfo (&set, NULL))
		{
			case SIGTERM:
				if (f3s.tpp != NULL)
				{
					/* Stop the thread pool */
					thread_pool_destroy (f3s.tpp);
					f3s.tpp = NULL;
				}
				if (attr){
					iofunc_attr_lock(&attr->attr);
					f3s_array_detach(attr->attr.mount->array, attr);
					iofunc_attr_unlock(&attr->attr);
				}
				return (EOK);

			case SIGUSR1:
				f3s_sigusr1_handler(mount[socket_index-1]->array);
				break;

			case SIGUSR2:
				sig_handler_ret = f3s_sigusr2_handler(mount[socket_index-1]->array, &pool_attr);
				if (sig_handler_ret != EOK){
					return sig_handler_ret;
				}
				break;

			default:
				break;
		}
	}

	return (errno);
/*}}}1*/
}

/*
 * Mounter Connect Functions {{{1
 */
int f3s_mnt_mount (resmgr_context_t *ctp, io_mount_t *msg,
                   f3s_attr_t *part_attr, io_mount_extra_t *extra)
{
/*{{{2*/
	f3s_part_t *	part;
	f3s_array_t *	array;
	f3s_attr_t *	attr;
	int				error;

	/* Validate flash type for mounts */
	if (msg->connect.file_type ==
		(_FTYPE_MOUNT && strcmp (F3S_BASE_TYPE, extra->extra.srv.type)))
	{
		return (ENOSYS);
	}

	/* We don't support attaching to non-existing flash file system nodes */
	if (msg->connect.extra_type != _IO_CONNECT_EXTRA_MOUNT_OCB) return (ENOSYS);

	/* Lock mounter attributes */
	iofunc_attr_lock (&part_attr->attr);

	/* Get the attributes for the partition we want to mount */
	attr = ((f3s_ocb_t *)extra->extra.srv.ocb)->ocb.attr;

	iofunc_attr_lock(&attr->attr);
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* If attach ID is not for socket */
	/* XXX - Shouldn't we want to look at ctp->id instead? */
	if (attr->attr.mount->attach_id[F3S_ATTACH_SKT] < 0)
	{
		/* get partition info pointer */
		part = attr->part;
		iofunc_attr_lock(&attr->part_attr->attr);
		/* Don't mount partitions that have been mounted before */
		F3S_TABLE_ENTER(part);
		if (!(part->table.status & F3S_TABLE_NO_ATTACH))
		{
			/* already mounted */
			error = EBUSY;

		} else
		{
			int savedflags;

			f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));
			/* Set name into mount structure */
			attr->part_attr->attr.mount->name = msg->connect.path;

			/* Save and set mount flags */
			savedflags = attr->part_attr->attr.mount->mount.flags;
			attr->part_attr->attr.mount->mount.flags &= ~0xf;
			attr->part_attr->attr.mount->mount.flags |= extra->flags & 0xf;

			f3s.force_mount = 1;
			/* Attach file system attributes to partition */
			error = f3s_part_attach (part, NULL, attr->part_attr, part->index);
			if (error) {
				/* Restore the mount structure */
				attr->part_attr->attr.mount->name = NULL;
				attr->part_attr->attr.mount->mount.flags = savedflags;
			}
		}
		F3S_TABLE_LEAVE(part);
		iofunc_attr_unlock(&attr->part_attr->attr);
	/* Otherwise, attach ID is for a socket */
	} else
	{
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* check if there is no pair in array list */
		if (attr->attr.mount->mounted.pair_list.next !=
		    &attr->attr.mount->mounted.pair_list)
		{
			/* there is something mounted */
			error = EBUSY;

		} else
		{
			/* attach partition attributes to socket */
			error = f3s_array_attach (array, attr);
		}
	}

	iofunc_attr_unlock(&attr->attr);
	/* Unlock mounter attributes */
	iofunc_attr_unlock (&part_attr->attr);

	f3s.force_mount = 0;
	return (error);
/*}}}2*/
}

/*}}}1
 */

/*
 * OCB Functions {{{1
 *
 * The way the OCB and attribute structures are arranged is really messy!!!
 * Every OCB is referenced by two lists:
 *   1) file_entry (aka vnode):
 *     Every vnode has a list of all OCBs sharing it.
 *
 *     Used in unmount to check for active references.
 *     Used as a vnode cache (very sloppy b/c of duplicates)
 *     Used to flush all cached meta-data (sloppy b/c of duplicates)
 *
 *   2) part_entry (aka vnode cache):
 *     Every partition vnode has a master list of all OCBs sharing any of its
 *     child vnodes.
 *
 *     Used to invalidate OCB's extent cache if the extent chain changes.
 *
 * Each OCB reference has a primary and secondary search key. The first is the
 * physical starting point of the file. The second is the pointer to the OCB
 * itself.
 */
f3s_ocb_t *f3s_ocb_calloc (resmgr_context_t *ctp, f3s_attr_t *attr)
{
/*{{{2*/
	f3s_desc_t	desc;
	f3s_ocb_t *	ocb;
	f3s_extptr_t inode;
	int			error = EOK;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	iofunc_attr_lock(&attr->part_attr->attr);
	iofunc_attr_lock(&attr->attr);

	/* create ocb for file */
	ocb = f3s_calloc (sizeof(*ocb));
	if (ocb) {
		F3S_INODE_TO_EXTPTR(attr->attr.inode, inode);

		/* setup descriptor with inode */
		f3s_desc_init (&desc, inode, F3S_DESC_INIT);

		/* attach newly created ocb */
		error = f3s_ocb_attach (ctp, ocb, &desc, attr);
		if (error) {
			f3s_free (ocb, sizeof(*ocb));
			ocb = NULL;
		}
	}

	iofunc_attr_unlock(&attr->attr);
	iofunc_attr_unlock(&attr->part_attr->attr);

	if (error) errno = error;
	return (ocb);
/*}}}2*/
}

void f3s_ocb_free (f3s_ocb_t *ocb)
{
/*{{{2*/
	f3s_attr_t* attr;

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));

	attr = ocb->ocb.attr;

	iofunc_attr_lock(&attr->part_attr->attr);
	iofunc_attr_lock(&attr->attr);

	/* detach the OCB from the F3S internal lists.  We don't call
	 * iofunc_ocb_detach() here because the iofunc library has already
	 * done this for us.
	 */
	f3s_ocb_detach(NULL, ocb, ocb->ocb.attr);

	iofunc_attr_unlock(&attr->attr);
	iofunc_attr_unlock(&attr->part_attr->attr);

	f3s_free (ocb, sizeof(*ocb));
/*}}}2*/
}

int f3s_ocb_attach (resmgr_context_t *ctp, f3s_ocb_t *ocb,
                    f3s_desc_t *file_desc, f3s_attr_t *file_attr)
{
/*{{{2*/
	f3s_share_t *	part_entry;
	f3s_share_t *	file_entry;

	/* copy file descriptor in ocb */
	ocb->file_desc = *file_desc;

	/* 
	 * Allocate list entries for both the vnode's OCB list and the partition's
	 * master OCB list.
	 */
	file_entry = f3s_malloc (sizeof(*file_entry) * 2);
	if (file_entry == NULL) return (ENOMEM);

	part_entry = &file_entry[1];

	/* Initialize both list nodes */
	file_entry->first = file_desc->first;
	file_entry->ocb = ocb;
	part_entry->first = file_desc->first;
	part_entry->ocb = ocb;

	f3s_lock_assert((file_attr->attr.lock_tid == pthread_self()) && (file_attr->attr.lock_count > 0));
	/* Add the list entry to the file_attr (aka vnode) */
	f3s_list_add (&file_attr->share_list, (f3s_list_t *)file_entry);
	file_attr->pending_delete = 0;		/* set the pending_delete flag to 'false' */

	f3s_lock_assert((file_attr->part_attr->attr.lock_tid == pthread_self()) && (file_attr->part_attr->attr.lock_count > 0));
	/* Add the list entry to the parition's master list */
	f3s_list_add (&file_attr->part_attr->share_list, (f3s_list_t *)part_entry);
	return (EOK);
/*}}}2*/
}

int f3s_ocb_detach (resmgr_context_t *ctp, f3s_ocb_t *ocb, f3s_attr_t *attr)
{
/*{{{2*/
	f3s_share_t *	file_share;
	uint8_t  detached = 0;		/* boolean indicating whether the ocb was actually detached */

	f3s_lock_assert(attr->part_attr->attr.lock_tid == pthread_self() && attr->part_attr->attr.lock_count >= 1);
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* Remove the OCB from the partition's master list */
	file_share = (f3s_share_t *)&attr->part_attr->share_list;
	while (file_share)
	{
		file_share = (f3s_share_t *)f3s_list_find ((f3s_list_t *)file_share,
		                                           sizeof(ocb->file_desc.first),
		                                           (uint8_t *)&ocb->file_desc.first);
		/* Exit the loop */
		f3s_assert(file_share != NULL);
		if (file_share == NULL)
			break;

		/* Resolve duplicates */
		if (file_share->ocb == ocb)
		{
			/* remove ocb share from list */
			f3s_list_rem ((f3s_list_t *)file_share);
			detached = 1;
			break;
		}

		/* look for share from next entry on */
		file_share = (f3s_share_t *)&file_share->list;
	}

	f3s_assert(attr->attr.lock_tid == pthread_self() && attr->attr.lock_count >= 1);
	/* Remove the OCB from the vnode's OCB list */
	file_share = (f3s_share_t *)&attr->share_list;
	while (file_share)
	{
		/* find ocb in file attribute share list */
		file_share = (f3s_share_t *)f3s_list_find ((f3s_list_t *)file_share,
		                                           sizeof(ocb->file_desc.first),
		                                           (uint8_t *)&ocb->file_desc.first);

		/* Exit the loop */
		f3s_assert(file_share != NULL);
		if (file_share == NULL)
			break;

		/* check if ocb pointer is same as in share */
		if (file_share->ocb == ocb)
		{
			/* remove ocb share from list */
			f3s_list_rem ((f3s_list_t *) file_share);
f3s_assert(detached == 1);	// can't be here if we weren't attached to the partition
			detached = 1;	// should be redundant
			break;
		}

		/* look for share from next entry */
		file_share = (f3s_share_t *)&file_share->list;
	}

	/* free memory for share entries */
	f3s_free (file_share, sizeof (*file_share) * 2);
	return ((int)detached);
/*}}}2*/
}

/*}}}1
 */

/*
 * Raw Connect Functions {{{1
 */
int f3s_raw_close_ocb (resmgr_context_t *ctp, void *reserved, f3s_ocb_t *ocb)
{
/*{{{2*/
	f3s_attr_t *	attr;
	f3s_array_t *	array;
	int				flag;

	/* Get attributes pointer */
	attr = ocb->ocb.attr;
	iofunc_attr_lock (&attr->attr);

	/* Cleanup the OCB */
	flag = iofunc_ocb_detach (ctp, &ocb->ocb);
	iofunc_ocb_free (ocb);

	/* If automount is wanted and last writer */
	if ((f3s.mount_options & f3s_mntopts_t_AUTOMNT) &&
		(flag & IOFUNC_OCB_LAST_WRITER))
	{
		/* If attach ID is for a socket */
		if (ctp->id == attr->attr.mount->attach_id[F3S_ATTACH_SKT])
		{
			/* get array info pointer */
			array = attr->attr.mount->array;

			/* If there is no pair in array list */
			if (attr->attr.mount->mounted.pair_list.next ==
			    &attr->attr.mount->mounted.pair_list)
			{
				/* Attach the array (all partitions) */
				f3s.force_mount = 1;
				f3s_array_attach (array, attr);
				f3s.force_mount = 0;
			}
		}
	}

	/* Unlock attributes */
	iofunc_attr_unlock (&attr->attr);
	return (EOK);
/*}}}2*/
}

int f3s_raw_unmount (resmgr_context_t *ctp, io_mount_t *msg,
                            f3s_attr_t *part_attr, io_mount_extra_t *extra)
{
/*{{{2*/
	f3s_pair_t *		pair;
	f3s_part_t *		part;
#if 0
	f3s_array_t *		array;
#endif
	struct _client_info	info;
	int					error = EOK;

	/* Stop non-root users */
	if ((iofunc_client_info (ctp, msg->connect.ioflag, &info) != EOK) ||
		(info.cred.euid != 0))
	{
		return (EPERM);
	}

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	/* check for unmount flag */
	if (extra->flags & _MOUNT_UNMOUNT)
	{
		/* check if attach id is not for socket */
		if (part_attr->attr.mount->attach_id[F3S_ATTACH_SKT] < 0)
		{
			/* get partition info pointer */
			part = part_attr->part;
			iofunc_attr_lock (&part_attr->part_attr->attr);

			/* check if an ocb is opened for partition */
			F3S_TABLE_ENTER(part);
			f3s_lock_assert((part_attr->part_attr->attr.lock_tid == pthread_self()) && (part_attr->part_attr->attr.lock_count > 0));

			/* if no other process is using the filesystem, or the force flag is specified, unmount */
			if ((part_attr->part_attr->share_list.next == &part_attr->part_attr->share_list) ||
				(extra->flags & _MOUNT_FORCE))
			{
				/* detach file system attributes from partition */
				error = f3s_part_detach (part, part_attr, part_attr->part_attr);
				if (error == EOK)
				{
					/* remove mountpoint pair pointer from list */
					pair = (f3s_pair_t *)part_attr->attr.mount->mounted.skt_attr->attr.mount->mounted.pair_list.next;
					f3s_list_rem ((f3s_list_t *) pair);
				}
			}
			else		/* filesystem is in use and force flag is not provided */
			{
				error = EBUSY;
			}
			F3S_TABLE_LEAVE(part);
			iofunc_attr_unlock (&part_attr->part_attr->attr);
		}
		else	/* else, attach id is for socket */
		{
#if 0
			/* get array info pointer */
			array = part_attr->attr.mount->array;

			/* detach partition attributes from socket */
			error = f3s_skt_detach (array, part_attr);
#else
			error = ENOTSUP;
#endif
		}
	}
	else
	{
		error = ENOTSUP;
	}

	iofunc_attr_unlock (&part_attr->attr);
	return (error);
/*}}}2*/
}

int f3s_raw_read (resmgr_context_t *ctp, io_read_t *msg, f3s_ocb_t *ocb)
{
/*{{{2*/
	f3s_attr_t *	attr;
	f3s_part_t *	part = NULL;
	f3s_array_t *	array;
	f3s_pair_t *	pair;
	uint8_t *		buffer;
	int32_t			buffer_size;
	int32_t			phys;
	int32_t			resume_flag;
	int32_t			read_size = 0;
	int32_t			chunk_size;
	int32_t			text_offset;
	int32_t			unit_num;
	int32_t			unit_first;
	int				err_size;
	int				error;
	int				part_locked = 0;
	int				has_read_callout = 0;

	/* reject unknown xtypes */
	if ((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_NONE) return (ENOSYS);

	/* check for privileges */
	error = iofunc_read_verify (ctp, msg, &ocb->ocb, NULL);
	if (error) return (error);

	/* get attributes pointer */
	attr = ocb->ocb.attr;

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);
		part_locked = 1;

		/* get array info pointer */
		array = part->array;

		/* set unit first and unit num */
		unit_first = part->unit_first;
		unit_num = part->unit_num;

	/* else, attach id is for socket */
	} else
	{
		/* setup mountpoint pair pointer */
		pair = (f3s_pair_t *) attr->attr.mount->mounted.pair_list.next;

		/* while there is a pair in array list */
		while (pair != (f3s_pair_t *)&attr->attr.mount->mounted.pair_list)
		{
			/* get partition pointer */
			part = pair->raw_attr->part;

			F3S_TABLE_ENTER(part);

			/* go to next pair */
			pair = (f3s_pair_t *) pair->list.next;
		}

		/* get array info pointer */
		array = attr->attr.mount->array;

		/* set unit first and unit num */
		unit_first = 0;
		unit_num = array->unit_total;
	}

	/* find read size for read buffer */
	buffer_size = msg->i.nbytes;

	/* check if we are attempting to read past end of partition */
	if ((ocb->ocb.offset + buffer_size) > (unit_num << array->unit_pow2))
	{
		/* adjust size accordingly */
		buffer_size = (unit_num << array->unit_pow2) - ocb->ocb.offset;

		/* check if buffer size makes no sense */
		if (buffer_size < 0) buffer_size = 0;
	}

	/* Check if there is either a v1 or v2 read callout. */
	if (array->flash->struct_size == sizeof(f3s_flash_v2_t)){
		const f3s_flash_v2_t * const tfp = (f3s_flash_v2_t*)array->flash;
		
		if (tfp->v2read || tfp->read){
		   	has_read_callout = 1;
		}
	}else{
		/* V1 callout */
		if (array->flash->read){
		   	has_read_callout = 1;
		}
	}

	/* while there is still something to read */
	while (read_size < buffer_size)
	{
		/* get physical unit index */
		phys = unit_first + (ocb->ocb.offset >> array->unit_pow2);

		/* get offset in unit */
		text_offset = ocb->ocb.offset & (array->unit_size - 1);

		/* fix size so it fits in unit */
		chunk_size = min(array->unit_size - text_offset,
		                 buffer_size - read_size);

		/* fix chunk size so it fits in message */
		chunk_size = min (chunk_size, F3S_READ_MAX);

		F3S_RAW_ENTER(array, phys);

		/* check if mtd has a read function */
		if (has_read_callout)
		{
			/* get buffer pointer */
			buffer = (uint8_t *)&msg->i;

			/* read from flash */
			error = f3s_flash_read (array, phys, text_offset,
			                        chunk_size, buffer);
			F3S_RAW_LEAVE(array, phys);
			if (error) break;

			/* write message to client buffer */
			err_size = resmgr_msgwrite (ctp, buffer, chunk_size, read_size);
			if (err_size < 0) {error = errno; break;}

		/* else, read is memory map */
		} else
		{
			/* map flash in memory */
			error = f3s_flash_map (array, phys, text_offset, chunk_size,
			                       &buffer, &resume_flag);
			if (error)
			{
				/* unmap flash from memory */
				f3s_flash_unmap (array, phys, text_offset, resume_flag);

				F3S_RAW_LEAVE(array, phys);
				break;
			}

			/* write into client message */
			err_size = resmgr_msgwrite (ctp, buffer, chunk_size, read_size);

			/* unmap flash from memory */
			f3s_flash_unmap (array, phys, text_offset, resume_flag);

			F3S_RAW_LEAVE(array, phys);
			if (err_size < 0) {error = errno; break;}
		}

		/* update ocb offset in file */
		ocb->ocb.offset += chunk_size;

		/* update read size */
		read_size += chunk_size;
	}

	/* check if attach id is not for socket */
	if (part_locked)
	{
		F3S_TABLE_LEAVE(part);
		part_locked = 0;

	/* else, attach id is for socket */
	} else
	{
		/* setup mountpoint pair pointer */
		pair = (f3s_pair_t *)attr->attr.mount->mounted.pair_list.next;

		/* while there is a pair in array list */
		while (pair != (f3s_pair_t *)&attr->attr.mount->mounted.pair_list)
		{
			/* get partition pointer */
			part = pair->raw_attr->part;

			F3S_TABLE_LEAVE(part);

			/* go to next pair */
			pair = (f3s_pair_t *)pair->list.next;
		}
	}

	/* check if read size makes no sense */
	if (error) return (error);
	/* return number of bytes read */
	_IO_SET_READ_NBYTES(ctp, read_size);
	return (EOK);
/*}}}2*/
}

int f3s_raw_write (resmgr_context_t *ctp, io_write_t *msg, f3s_ocb_t *ocb)
{
/*{{{2*/
	f3s_attr_t *	attr;
	f3s_array_t *	array;
	f3s_part_t *	part = NULL;
	uint8_t *		buffer;
	int32_t			buffer_size;
	int32_t			phys;
	int32_t			write_size = 0;
	int32_t			chunk_size;
	int32_t			text_offset = 0;
	int32_t			unit_num;
	int32_t			unit_first;
	int				error;
	int				part_locked = 0;

	/* reject unknown xtypes */
	if ((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_NONE) return (ENOSYS);

	/* check for privileges */
	error = iofunc_write_verify (ctp, msg, &ocb->ocb, NULL);
	if (error) return (error);

	/* get attributes pointer */
	attr = ocb->ocb.attr;

	/* check if attach id is not for socket */
	if (ctp->id != attr->attr.mount->attach_id[F3S_ATTACH_SKT])
	{
		int  error;	// added for part_detach check
		/* get partition info pointer */
		part = attr->part;

		F3S_TABLE_ENTER(part);
		part_locked = 1;

		/* detach file system attributes from partition */
		error = f3s_part_detach (part, NULL, attr->part_attr);
		f3s_assert(error == EOK);

		/* get array info pointer */
		array = part->array;

		/* set unit first and unit num */
		unit_first = part->unit_first;
		unit_num   = part->unit_num;

	/* else, attach id is for socket */
	}
	else
	{
		/* get array info pointer */
		array = attr->attr.mount->array;

		/* detach array (all partitions) */
		f3s_array_detach (array, attr);

		/* set unit first and unit num */
		unit_first = 0;
		unit_num = array->unit_total;
	}

	/* get size of write */
	buffer_size = msg->i.nbytes;

	/* check if we are attempting to write past end of partition */
	if ((ocb->ocb.offset + buffer_size) > (unit_num << array->unit_pow2))
	{
		/* adjust size accordingly */
		buffer_size = (unit_num << array->unit_pow2) - ocb->ocb.offset;

		/* check if buffer size makes no sense */
		if (buffer_size < 0) buffer_size = 0;
	}

	/* get buffer pointer */
	buffer = (uint8_t *)(&msg->i + 1);

	/* while there is still something to write */
	while (write_size < buffer_size)
	{
		/* get physical unit index */
		phys = unit_first + (ocb->ocb.offset >> array->unit_pow2);

		/* get offset in unit */
		text_offset = ocb->ocb.offset & (array->unit_size - 1);

		/* fix chunk size so it fits in message */
		chunk_size = min (buffer_size - write_size, F3S_WRITE_MAX);

		/* fix size so it fits in unit */
		chunk_size = min (array->unit_size - text_offset, chunk_size);

		/* write with flash write function */
		F3S_RAW_ENTER(array, phys);
		error = f3s_flash_write (array, phys, text_offset,
		                         chunk_size, buffer, 0);
		F3S_RAW_LEAVE(array, phys);
		if (error) break;

		/* update ocb offset in file */
		ocb->ocb.offset += chunk_size;

		/* update write size */
		write_size += chunk_size;

		/* check if there is still something to write */
		if (write_size < buffer_size)
		{
			/* get size of chunk */
			chunk_size = min(buffer_size - write_size, F3S_WRITE_MAX);

			/* fix size so it fits in unit */
			chunk_size = min((ocb->ocb.offset & ~(array->unit_size - 1)) +
			                 array->unit_size - text_offset, chunk_size);

			/* read message */
			chunk_size = resmgr_msgread (ctp, buffer, chunk_size,
			                             sizeof(msg->i) + write_size);
			if (chunk_size < 0) {error = errno; break;}
			if (chunk_size == 0) break;
		}
	}

	/* check if attach id is not for socket */
	if (part_locked)
	{
		F3S_TABLE_LEAVE(part);
		part_locked = 0;
	}

	/* check if write size makes no sense */
	if (error) return (error);

	_IO_SET_WRITE_NBYTES(ctp, write_size);
	return (EOK);
/*}}}2*/
}

int f3s_raw_devctl (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{2*/
	int		error;

	/* benchmarking early exit */
	if (!msg->i.dcmd) return (EOK);

	/* do standard devctl stuff */
	error = iofunc_devctl (ctp, msg, &ocb->ocb, &ocb->ocb.attr->attr);

	/* check if devctl was not processed */
	if (error == _RESMGR_DEFAULT)
	{
		/* do f3s specific stuff depending on message */
		switch (msg->i.dcmd)
		{
			case DCMD_F3S_ERASE:
				error = f3s_raw_erase (ctp, msg, ocb);
				break;

			case DCMD_F3S_FORMAT:
				error = f3s_raw_format (ctp, msg, ocb);
				break;

			case DCMD_F3S_MOUNT:
				error = f3s_raw_mount (ctp, msg, ocb);
				break;

			case DCMD_F3S_UMOUNT:
				error = f3s_raw_umount (ctp, msg, ocb);
				break;

			case DCMD_F3S_ARRAYINFO:
				error = f3s_arrayinfo (ctp, msg, ocb);
				break;

			case DCMD_F3S_PARTINFO:
				error = f3s_raw_partinfo (ctp, msg, ocb);
				break;

			case DCMD_FSYS_STATVFS:
				error = f3s_raw_statvfs (ctp, msg, ocb);
				break;

			case DCMD_FSYS_MOUNTED_BY:
				error = f3s_raw_mountedby (ctp, msg, ocb);
				break;

			case DCMD_FSYS_MOUNTED_ON:
				error = f3s_raw_mountedon (ctp, msg, ocb);
				break;

			case DCMD_F3S_EXIT:
				f3s_exit (ctp, msg, ocb);
				break;

			case DCMD_F3S_LOCK:
				error = f3s_raw_lock (ctp, msg, ocb);
				break;

			case DCMD_F3S_UNLOCK:
				error = f3s_raw_unlock (ctp, msg, ocb);
				break;

			case DCMD_F3S_UNLOCKALL:
				error = f3s_raw_unlockall (ctp, msg, ocb);
				break;
			case DCMD_F3S_LOCKDOWN:
				error = f3s_raw_lockdown (ctp, msg, ocb);
				break;

			case DCMD_F3S_ULOCKDOWN:
				error = f3s_raw_unlockdown (ctp, msg, ocb);
				break;
			
			case DCMD_F3S_GEOINFO:
				error = f3s_geoinfo (ctp, msg, ocb);
				break;

			default:
				error = ENOSYS;
		}
	}

	/* return result message */
	return (error);
/*}}}2*/
}

/*}}}1
 */

/* 
 * Flash File System Connect Functions {{{1
 */ 

int f3s_open (resmgr_context_t *ctp, io_open_t *msg, f3s_attr_t *part_attr,
              void *extra)
{
/*{{{2*/
	struct _client_info	info;
	f3s_desc_t			dir_desc;
	f3s_desc_t			file_desc;
	f3s_dirent_t		dirent;
	f3s_stat_t			stat;
	f3s_attr_t			dir_attr;
	f3s_attr_t *		file_attr;
	resmgr_io_funcs_t *	io_funcs = NULL;
	f3s_part_t *		part;
	f3s_share_t *		file_share;
	f3s_ocb_t *			ocb;
	char*				name;
	int32_t				found;
	int32_t				trunc_size;
	int32_t				file_size;
	int16_t				length;
	int					error;
uint8_t  pending_delete;


	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return _RESMGR_ERRNO(EBADF);
	}
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition pointer */
	part = part_attr->part;

	/* change to proper directory */
	error = f3s_namei (ctp, msg, part_attr, &dir_attr, &dir_desc, &info);
	if (error) goto part_shortcut;

	/* initialize name pointer */
	name = f3s_path_last (msg->connect.path, &length);

	/* find the file entry */
	error = f3s_entry_lookup (part, &dir_desc, &dirent, name, length, &stat,
	                          dir_attr.depth);

	// if lookup returns anything other than found or not found, exit with an error
	if ((error != EOK) && (error != ENOENT))
	{
		goto part_shortcut;
	}
	else if (error == EOK)		// entry found ?
	{
		found = 1;

		/* check if file needs to be handled by something else */
		if (S_ISFIFO(stat.mode) &&
			(msg->connect.ioflag & (_IO_FLAG_RD | _IO_FLAG_WR)))
		{
			/* let fifos be handled by resmgr */
			_IO_SET_CONNECT_RET (ctp, _IO_CONNECT_RET_FTYPE);

			((struct _io_connect_ftype_reply *) msg)->status = EOK;
			((struct _io_connect_ftype_reply *) msg)->file_type = _FTYPE_PIPE;

			error = _RESMGR_PTR(ctp, msg, sizeof(struct _io_connect_ftype_reply));
			goto part_shortcut;
		}

		/* setup file descriptor with dirent contents */
		f3s_desc_init (&file_desc, dirent.first, F3S_DESC_INIT);

		/* check if entry is a symbolic link to resolve */
		if (S_ISLNK(stat.mode) &&
			((msg->connect.eflag & _IO_CONNECT_EFLAG_DIR) ||
			 !S_ISLNK(msg->connect.mode)))
		{
			int  ret_val;
			
			// POSIX says that an attempt to open a symlink with O_CREAT | O_EXCL set should fail with EEXIST
			if ((msg->connect.ioflag & (O_CREAT | O_EXCL)) == (O_CREAT | O_EXCL))
			{
				error = EEXIST;
				goto part_shortcut;
			}
				
			/* set connect reply for resolved symbolic link */
			_IO_SET_CONNECT_RET (ctp, _IO_CONNECT_RET_LINK);

			/* resolve the link */
			error = f3s_link_resolve (ctp, (io_readlink_t *) msg, part_attr,
			                          dirent.first, msg->connect.path_len - length, &ret_val);
			if (error == EOK)
				error = ret_val;
				
			goto part_shortcut;
		}

		/* check if entry is not a directory but directory was wanted */
		if (!S_ISDIR (stat.mode) &&
			(msg->connect.eflag & _IO_CONNECT_EFLAG_DIR))
		{
			error = ENOTDIR;
			goto part_shortcut;
		}
		f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
		/* find an ocb list entry with the same first extent pointer */
		file_share =
			(f3s_share_t *)f3s_list_find (&part_attr->share_list,
			                              sizeof(dirent.first),
			                              (uint8_t *)&dirent.first);
	}
	else	// else, file was not found (error == ENOENT)
	{
			found = 0;
			
			if (!(msg->connect.ioflag & O_CREAT))		// entry not found and caller does not want to create
				goto part_shortcut;
			else	// setup for new entry creation
			{
				/* clear ocb list entry pointer */
				file_share = NULL;
			}
	}

	/* chech if ocb list entry was not found */
	if (!file_share)
	{
		/* create attributes for file */
		file_attr = f3s_calloc (sizeof(*file_attr));
		if (file_attr == NULL)
		{
			f3s_assert (file_attr);
			error = ENOMEM;
			goto part_shortcut;
		}

		/* copy directory descriptor in file attr */
		file_attr->parent_extptr = dir_desc.first;

		/* initialize ocb share list */
		f3s_list_init (&file_attr->share_list);

		/* copy default attribute fields */
		file_attr->part_attr = dir_attr.part_attr;
		file_attr->part = dir_attr.part;

		/* copy mount structure pointer */
		file_attr->attr.mount = dir_attr.attr.mount;

	} else
	{
		/* set file attributes pointer */
		file_attr = file_share->ocb->ocb.attr;
	}

	/* check if entry was found */
	if (found)
	{
		/* check if no share exists */
		if (!file_share)
		{
			/* initialize attributes */
			f3s_attr_init (file_attr, 0, file_attr, &file_desc, &stat, &info);

			/* setup depth in file attributes */
			file_attr->depth = dir_attr.depth + 1;
		}

		/* directories get their own special iofuncs */
		if (S_ISDIR(stat.mode))
			io_funcs = &f3s.dir_io_funcs;
		else
			io_funcs = &f3s.f3s_io_funcs;			/* regular files get different iofuncs */

		/* check if ioflag has truncate */
		if (msg->connect.ioflag & O_TRUNC)
		{
			/* go without filtering iofuncs */
			io_funcs = &f3s.f3s_io_funcs;
		}
		iofunc_attr_lock(&file_attr->attr);

		/* call resource manager open function (lock state is preserved with arg4 == NULL) */
		error = iofunc_open (ctp, msg, &file_attr->attr, NULL, &info);
		if (error)
		{
			iofunc_attr_unlock(&file_attr->attr);
			/* free the file_attr if nobody else has the file
			 * open.
			 */
			if (!file_share)
				f3s_free(file_attr, sizeof(*file_attr));
			goto part_shortcut;
		}
	}
	else	/* else, entry was not found */
	{
		/*
		 * file_share == NULL and hence file_attr was allocated (no lock required and
		 * iofunc_open() will clear it anyway)
		*/
		
		/* call resource manager open function for create */
		error = iofunc_open (ctp, msg, &file_attr->attr, &dir_attr.attr, &info);
		if (error)
		{
			if (!file_share)
				f3s_free(file_attr, sizeof(*file_attr));
			goto part_shortcut;
		}

		/* setup file descriptor */
		f3s_desc_init (&file_desc, f3s_null_extptr, F3S_DESC_INIT);

		/* create the file */
		file_size = f3s_file_create (part, &file_desc, NULL, 0, NULL, 0);
		if (file_size < 0)
		{
			error = errno;
			goto part_shortcut;
		}

		iofunc_attr_lock(&file_attr->attr);
		/* call create function */
		error = f3s_create (ctp, msg, &dir_attr, file_attr,
		                    &dir_desc, &file_desc, &stat, 0);

		if (error)
		{
			iofunc_attr_unlock(&file_attr->attr);
			goto part_shortcut;
		}

		/* initialize attributes */
		f3s_attr_init (file_attr, 0, file_attr, &file_desc, NULL, &info);

		/* go without filtering iofuncs */
		io_funcs = &f3s.f3s_io_funcs;

		/* knock down truncation */
		msg->connect.ioflag &= ~O_TRUNC;
	}

	/* at this point, the file_attr lock is held whether allocated or found */
	f3s_assert((file_attr->attr.lock_tid == pthread_self()) && (file_attr->attr.lock_count > 0));

	/* create ocb for file */
	ocb = f3s_calloc (sizeof(*ocb));
	if (ocb == NULL)
	{
		f3s_assert (ocb);
		iofunc_attr_unlock(&file_attr->attr);
		if (!file_share)
			f3s_free(file_attr, sizeof(*file_attr));
		error = ENOMEM;
		goto part_shortcut;
	}

	/* attach newly created ocb to file attributes */
	pending_delete = file_attr->pending_delete;	/* save pending_delete flag in case it needs restoring */
	error = f3s_ocb_attach (ctp, ocb, &file_desc, file_attr);
	if (error)
	{
		f3s_free (ocb, sizeof(*ocb));
		goto file_shortcut;
	}

	/* check if truncate is wanted */
	if (msg->connect.ioflag & O_TRUNC)
	{
		/* synchronize file attributes */
		error = f3s_attr_sync (ocb, file_attr);
		f3s_assert (!error);
		if (error)
		{
			file_attr->pending_delete = pending_delete;	/* restore pending_delete flag */
			f3s_ocb_detach(ctp, ocb, file_attr);

			f3s_free(ocb, sizeof(*ocb));
			goto file_shortcut;
		}

		/* truncate the file */
		trunc_size = f3s_file_trunc (part, &file_desc, 0);
		if (trunc_size < 0)
		{
			error = errno;
			f3s_assert(error != 0);	/* f3s_file_trunc() does not guarantee errno is set
														which could result in a leak on file_attr */
			file_attr->pending_delete = pending_delete;	/* restore pending_delete flag */
			f3s_ocb_detach(ctp, ocb, file_attr);

			f3s_free(ocb, sizeof(*ocb));
			goto file_shortcut;
		}

		/* invalidate file attributes */
		f3s_attr_inval (ocb, file_attr, F3S_DESC_NO_LAST, 0, 0, F3S_DESC_NO_REWIND);

		/* knock down dirty time flag */
		if (f3s.update_level < F3S_UPDATE_FILE)
			file_attr->attr.flags &= ~IOFUNC_ATTR_DIRTY_TIME;
	}

	/* attach ocb to resource manager with proper iofuncs. iofunc_ocb_attach
	 * calls f3s_ocb_free() on error, so we shouldn't deallocate the OCB again.
	 */
	error = iofunc_ocb_attach (ctp, msg, &ocb->ocb, &file_attr->attr, io_funcs);

file_shortcut:
	/* unlock file attributes */
	iofunc_attr_unlock (&file_attr->attr);

	/* free unattached attr */
	if (error && !file_share)
		f3s_free (file_attr, sizeof(*file_attr));

part_shortcut:
	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);
	return (error);
/*}}}2*/
}

int f3s_unlink (resmgr_context_t *ctp, io_unlink_t *msg, f3s_attr_t *part_attr,
                void *reserved)
{
/*{{{2*/
	f3s_desc_t			dir_desc;
	f3s_desc_t			dirent_desc;
	f3s_desc_t			file_desc;
	f3s_dirent_t		dirent;
	f3s_stat_t			stat;
	f3s_attr_t			dir_attr;
	f3s_attr_t *		file_attr;
	f3s_part_t *		part;
	f3s_share_t *		file_share;
	f3s_share_t *		dirent_share;
	struct _client_info	info;
	char *			name;
	int32_t				update_size;
	int32_t				forward_size;
	int32_t				delete_size;
	int32_t				reclen;
	int16_t				length;
	int					error;

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return _RESMGR_ERRNO(EBADF);
	}
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition pointer */
	part = part_attr->part;

	/* change to proper directory */
	error = f3s_namei (ctp, (io_open_t *) msg, part_attr, &dir_attr, &dir_desc, &info);
	if (error) goto part_shortcut;

	/* initialize name pointer */
	name = f3s_path_last (msg->connect.path, &length);

	f3s_lock_assert((dir_attr.attr.lock_tid == pthread_self()) && (dir_attr.attr.lock_count > 0));
	/* find the file entry */
	error = f3s_entry_lookup (part, &dir_desc, &dirent, name, length, &stat, dir_attr.depth);
	if (error != EOK)
		goto part_shortcut;

	/* error if treating a non directory object as a directory */
	if ((msg->connect.eflag & _IO_CONNECT_EFLAG_DIR) && (!S_ISDIR(stat.mode)))
	{
		error = ENOTDIR;
		goto part_shortcut;
	}

	/* setup file descriptor */
	f3s_desc_init (&file_desc, dirent.first, F3S_DESC_INIT);

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
	/* find file share in share list */
	file_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
	                                           sizeof(dirent.first),
	                                           (uint8_t *)&dirent.first);
	if (file_share)
	{
		/* set file attributes pointer */
		file_attr = file_share->ocb->ocb.attr;

	} else
	{
		/* create temporary attributes for file */
		file_attr = alloca (sizeof(*file_attr));

		/* check if file attribute pointer is not valid */
		if (!file_attr)
		{
			f3s_assert (file_attr);
			error = ENOMEM;
			goto part_shortcut;
		}

		/* clear file attributes */
		memset (file_attr, 0, sizeof (*file_attr));

		/* initialize attributes */
		f3s_attr_init (file_attr, 0, file_attr, &file_desc, &stat, &info);

		/* setup depth in file attributes */
		file_attr->depth = dir_attr.depth + 1;
	}

	/* make sure entry is not root */
	if (!memcmp (&dir_desc.curr, &part->table.root_extptr, sizeof(f3s_extptr_t)))
	{
		error = EBUSY;
		goto part_shortcut;
	}

	/* lock file attributes */
	iofunc_attr_lock (&file_attr->attr);

	/* call resource manager unlink function */
	f3s_lock_assert((file_attr->attr.lock_tid == pthread_self()) && (file_attr->attr.lock_count > 0));
	f3s_lock_assert((dir_attr.attr.lock_tid == pthread_self()) && (dir_attr.attr.lock_count > 0));
	error = iofunc_unlink (ctp, msg, &file_attr->attr, &dir_attr.attr, &info);
	if (error)
		goto file_shortcut;

	/* check if removing a directory */
	if (S_ISDIR (stat.mode))
	{
		/* check the size of directory file to make sure it is empty */
		forward_size = f3s_file_forward (part, &file_desc, 1);
		if (forward_size < 0)
		{
			error = errno;
			goto file_shortcut;

		} else if (forward_size)
		{
			error = ENOTEMPTY;
			goto file_shortcut;
		}
	}

	/* delete the directory entry */
	delete_size = f3s_entry_delete (part, &dir_desc, dir_attr.depth);
	if (delete_size < 0)
	{
		error = errno;
		goto file_shortcut;
	}

	/* check if file is opened */
	if (!file_share)
	{
		/* delete the file completely */
		f3s_file_delete (part, &file_desc, F3S_FILE_MAX);

	/* file is opened, do not delete */
	} else
	{
		f3s_lock_assert((file_share->ocb->ocb.attr->attr.lock_tid == pthread_self()) && (file_share->ocb->ocb.attr->attr.lock_count > 0));
		/* postpone the delete until file is last closed */
		file_share->ocb->ocb.attr->attr.nlink = 0;
		file_share->ocb->ocb.attr->pending_delete = 1;	// delete on close
	}

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
	dirent_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
	                                             sizeof(dir_desc.first),
	                                             (uint8_t *)&dir_desc.first);
	if (dirent_share)
	{
		/* calculate record length */
		reclen = sizeof (dirent) + F3S_NAME_ALIGN (dirent.namelen) + sizeof (stat);

		iofunc_attr_lock(&dirent_share->ocb->ocb.attr->attr);
		f3s_lock_assert((dirent_share->ocb->ocb.attr->attr.lock_tid == pthread_self()) && (dirent_share->ocb->ocb.attr->attr.lock_count > 0));
		/* invalidate sharing directories */
		f3s_attr_inval (NULL, dirent_share->ocb->ocb.attr,
						F3S_DESC_NO_LAST, dir_desc.curr_offset, -reclen,
						F3S_DESC_NO_REWIND);
		iofunc_attr_unlock(&dirent_share->ocb->ocb.attr->attr);
	}

file_shortcut:
	/* unlock file attributes */
	iofunc_attr_unlock (&file_attr->attr);

	/* check if parent time update is wanted */
	if (!error && f3s.update_level >= F3S_UPDATE_PARENT)
	{
		/* update times in parent attributes */
		iofunc_time_update (&dir_attr.attr);

		/* initialize dirent descriptor */
		f3s_desc_init (&dirent_desc, dir_attr.parent_extptr, ~F3S_DESC_NO_DIR);

		/* read parent directory entry */
		error = f3s_entry_lookup (part, &dirent_desc, NULL, "", 1, &stat, F3S_CACHE_MAX);
		if (error != EOK)
			goto part_shortcut;

		/* copy modification time */
		stat.mtime = dir_attr.attr.mtime;

		/* update directory entry */
		update_size = f3s_entry_update (part, &dirent_desc, &stat);
		if (update_size < 0) error = errno;
	}

part_shortcut:
	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);
	return (error);
/*}}}2*/
}

int f3s_rename (resmgr_context_t *ctp, io_rename_t *msg, f3s_attr_t *part_attr,
                io_rename_extra_t *extra)
{
/*{{{2*/
	f3s_desc_t			old_dir_desc;
	f3s_desc_t			new_dir_desc;
	f3s_desc_t			dirent_desc;
	f3s_desc_t			old_file_desc;
	f3s_desc_t			new_file_desc;
	f3s_dirent_t		old_dirent;
	f3s_dirent_t		new_dirent;
	f3s_stat_t			old_stat;
	f3s_stat_t			new_stat;
	f3s_attr_t			old_dir_attr;
	f3s_attr_t			new_dir_attr;
	struct _client_info	info;
	io_rename_t *		old_msg;
	f3s_attr_t *		old_file_attr;
	f3s_attr_t *		new_file_attr;
	f3s_part_t *		part;
	f3s_share_t *		old_share;
	f3s_share_t *		new_share;
	f3s_share_t *		dirent_share;
	char *				old_name;
	char *				new_name;
	int32_t				in_place = 1;
	int32_t				forward_size;
	int32_t				reclen;
	int16_t				old_length;
	int16_t				new_length;
	int					err_size;
	int					error;

	/* guard against moving a directory into itself */
	if ((msg->connect.extra_len <= msg->connect.path_len) &&
		(msg->connect.extra_len != 0) &&
		(msg->connect.path[msg->connect.extra_len - 1] == '/') &&
		(!memcmp (msg->connect.path, extra->path, msg->connect.extra_len - 1)))
	{
		return (EINVAL);
	}
	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);
	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return _RESMGR_ERRNO(EBADF);
	}
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition pointer */
	part = part_attr->part;

	/* allocate space for old message */
	old_length = msg->connect.extra_len;
	old_msg = alloca (sizeof(*msg) + old_length);
	if (!old_msg)
	{
		f3s_assert (old_msg);
		error = ENOMEM;
		goto part_shortcut;
	}

	/* copy message and setup for old entry */
	memcpy (old_msg, msg, sizeof(*msg));
	memcpy (old_msg->connect.path, extra->path, old_length);
	old_msg->connect.path_len = old_length;

	/* change to proper directory for old entry */
	error = f3s_namei (ctp, (io_open_t *)old_msg, part_attr, &old_dir_attr,
					   &old_dir_desc, &info);
	if (error) goto part_shortcut;

	/* initialize old name pointer */
	old_name = f3s_path_last (extra->path, &old_length);

	/* find the old file entry */
	error = f3s_entry_lookup (part, &old_dir_desc, &old_dirent, old_name,
								  old_length, &old_stat, old_dir_attr.depth);
	if (error != EOK)
		goto part_shortcut;

	/* error if treating a non directory object as a directory */
	if ((msg->connect.eflag & _IO_CONNECT_EFLAG_DIR) && (!S_ISDIR(old_stat.mode)))
	{
		error = ENOTDIR;
		goto part_shortcut;
	}

	/* make sure entry is not root */
	if (!memcmp (&old_dir_desc.curr, &part->table.root_extptr, sizeof(f3s_extptr_t)))
	{
		error = EBUSY;
		goto part_shortcut;
	}

	/* initialize new name pointer */
	new_name = f3s_path_last (msg->connect.path, &new_length);

	/* compare base paths, to check if rename is in place */
	if ((msg->connect.path_len - new_length != msg->connect.extra_len - old_length) ||
		(memcmp (msg->connect.path, extra->path, msg->connect.path_len - new_length)))
	{
		/* accross directories */
		in_place = 0;

	/* else, check if names are same */
	} else if (!strcmp (new_name, old_name))
	{
		/* rename is successfull */
		error = EOK;
		goto part_shortcut;
	}

	/* setup file descriptor */
	f3s_desc_init (&old_file_desc, old_dirent.first, F3S_DESC_INIT);

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
	/* find old file share in share list */
	old_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
											  sizeof(old_dirent.first),
											  (uint8_t *)&old_dirent.first);

	/* check if file is shared */
	if (old_share)
	{
		/* set old file attributes pointer */
		old_file_attr = old_share->ocb->ocb.attr;

	} else
	{
		/* create temporary attributes for old file */
		old_file_attr = alloca (sizeof(*old_file_attr));

		/* check if old file attribute pointer is not valid */
		if (!old_file_attr)
		{
			f3s_assert (old_file_attr);
			error = ENOMEM;
			goto part_shortcut;
		}

		/* clear file attributes */
		memset (old_file_attr, 0, sizeof(*old_file_attr));

		/* initialize attributes */
		f3s_attr_init (old_file_attr, 0, old_file_attr, &old_file_desc, &old_stat, &info);

		/* setup depth in file attributes */
		old_file_attr->depth = old_dir_attr.depth + 1;
	}

	/* lock old file attributes */
	iofunc_attr_lock (&old_file_attr->attr);

	/* change to proper directory for new entry */
	error = f3s_namei (ctp, (io_open_t *)msg, part_attr, &new_dir_attr,
	                   &new_dir_desc, &info);
	if (error) goto old_file_shortcut;

	/* find the new file entry */
	error = f3s_entry_lookup (part, &new_dir_desc, &new_dirent, new_name,
	                          new_length, &new_stat, new_dir_attr.depth);

	// if lookup returns anything other than found or not found, return the error
	if ((error != EOK) && (error != ENOENT))
	{
		goto old_file_shortcut;
	}
	else if (error == EOK)
	{
		/* make sure entry is not root */
		if (!memcmp (&new_dir_desc.curr, &part->table.root_extptr, sizeof(f3s_extptr_t)))
		{
			error = EBUSY;
			goto old_file_shortcut;
		}

		/* setup file descriptor */
		f3s_desc_init (&new_file_desc, new_dirent.first, F3S_DESC_INIT);

		f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
		/* find new file share in share list */
		new_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
												  sizeof(new_dirent.first),
												  (uint8_t *)&new_dirent.first);
		if (new_share)
		{
			/* set file attributes pointer */
			new_file_attr = new_share->ocb->ocb.attr;

		} else
		{
			/* create temporary attributes for file */
			new_file_attr = alloca (sizeof(*new_file_attr));

			/* check if file attribute pointer is not valid */
			if (!new_file_attr)
			{
				f3s_assert (new_file_attr);
				error = ENOMEM;
				goto old_file_shortcut;
			}

			/* clear file attributes */
			memset (new_file_attr, 0, sizeof(*new_file_attr));

			/* initialize attributes */
			f3s_attr_init (new_file_attr, 0, new_file_attr, &new_file_desc, &new_stat, &info);

			/* setup depth in file attributes */
			new_file_attr->depth = new_dir_attr.depth + 1;
		}

		/* lock new file attributes */
		iofunc_attr_lock (&new_file_attr->attr);

		/* call iofunc rename to check posix stuff */
		error = iofunc_rename (ctp, msg, &old_file_attr->attr, &old_dir_attr.attr,
		                       &new_file_attr->attr, &new_dir_attr.attr, &info);
		if (error) goto new_file_shortcut;

		/* check if removing a directory */
		if (S_ISDIR (new_stat.mode))
		{
			/* check the size of directory file to make sure it is empty */
			forward_size = f3s_file_forward (part, &new_file_desc, 1);
			if (forward_size < 0)
			{
				error = errno;
				goto new_file_shortcut;

			} else if (forward_size)
			{
				error = ENOTEMPTY;
				goto new_file_shortcut;
			}
		}

		/* update times in old file attributes */
		iofunc_time_update (&old_file_attr->attr);

		/* update status change time */
		old_stat.ctime = old_file_attr->attr.ctime;

		/* change directory entry */
		err_size = f3s_entry_change (part, &new_dir_desc, &old_dirent,
		                             new_name, new_length, &old_stat,
		                             (uint8_t)old_dirent.moves + 1);
		if (err_size < 0) {error = errno; goto new_file_shortcut;}

		/* delete the old directory entry */
		err_size = f3s_entry_delete (part, &old_dir_desc, old_dir_attr.depth);
		if (err_size < 0) {error = errno; goto new_file_shortcut;}

		/* check if file is opened */
		if (!new_share)
		{
			/* delete the new file completely */
			f3s_file_delete (part, &new_file_desc, F3S_FILE_MAX);

			/* file is opened, do not delete */
		} else
		{
			/* postpone the delete until new file is last closed */
			new_share->ocb->ocb.attr->attr.nlink = 0;
		}

		f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
		/* find directory share in share list */
		dirent_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
		                                             sizeof(old_dir_desc.first),
		                                             (uint8_t *)&old_dir_desc.first);
		if (dirent_share)
		{
			/* calculate record length */
			reclen = sizeof(old_dirent) + F3S_NAME_ALIGN(old_dirent.namelen) +
			         sizeof(old_stat);

			/* invalidate sharing directories */
			f3s_attr_inval (NULL, dirent_share->ocb->ocb.attr,
			                F3S_DESC_NO_LAST, old_dir_desc.curr_offset,
			                -reclen, F3S_DESC_NO_REWIND);
		}

		/* find directory share in share list */
		dirent_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
		                                             sizeof (new_dir_desc.first),
		                                             (uint8_t *)&new_dir_desc.first);
		if (dirent_share)
		{
			/* invalidate sharing directories */
			f3s_attr_inval (NULL, dirent_share->ocb->ocb.attr,
			                F3S_DESC_NO_LAST, new_dir_desc.curr_offset,
			                F3S_NAME_ALIGN(new_length) - F3S_NAME_ALIGN(old_length),
			                F3S_DESC_NO_REWIND);
		}

new_file_shortcut:
		/* unlock new file attributes */
		iofunc_attr_unlock (&new_file_attr->attr);

		/* check if no error and not in place and parent time update is wanted */
		if (!error && !in_place && f3s.update_level >= F3S_UPDATE_PARENT)
		{
			/* initialize dirent descriptor */
			f3s_desc_init (&dirent_desc, new_dir_attr.parent_extptr, ~F3S_DESC_NO_DIR);

			/* read parent directory entry */
			error = f3s_entry_lookup (part, &dirent_desc, NULL, "", 1, &new_stat, F3S_CACHE_MAX);
			f3s_assert (error == EOK);
			if (error == EOK)
			{
				/* copy modification time */
				new_stat.mtime = new_dir_attr.attr.mtime;

				/* update directory entry */
				err_size = f3s_entry_update (part, &dirent_desc, &new_stat);
				if (err_size < 0) error = errno;
			}
		}
	}
	else	// new file name does not exist (error == ENOENT)
	{
		/* call iofunc rename with null for new attributes */
		error = iofunc_rename (ctp, msg, &old_file_attr->attr, &old_dir_attr.attr,
		                       NULL, &new_dir_attr.attr, &info);
		if (error) goto old_file_shortcut;

		/* check if not in place */
		if (!in_place)
		{
			/* create the file entry */
			error = f3s_create (ctp, (io_open_t *)msg, &new_dir_attr,
			                    old_file_attr, &new_dir_desc, &old_file_desc,
			                    &old_stat, (uint8_t)old_dirent.moves + 1);
			if (error) goto old_file_shortcut;

			/* delete the old directory entry */
			err_size = f3s_entry_delete (part, &old_dir_desc, old_dir_attr.depth);
			if (err_size < 0) {error = errno; goto old_file_shortcut;}

			f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
			/* find directory share in share list */
			dirent_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
			                                             sizeof(old_dir_desc.first),
			                                             (uint8_t *)&old_dir_desc.first);
			if (dirent_share)
			{
				/* calculate record length */
				reclen = sizeof(old_dirent) + F3S_NAME_ALIGN(old_dirent.namelen) +
				         sizeof(old_stat);

				/* invalidate sharing directories */
				f3s_attr_inval (NULL, dirent_share->ocb->ocb.attr,
				                F3S_DESC_NO_LAST, old_dir_desc.curr_offset,
				                -reclen, F3S_DESC_NO_REWIND);
			}
		} else
		{
			/* update times in old file attributes */
			iofunc_time_update (&old_file_attr->attr);

			/* update status change time */
			old_stat.ctime = old_file_attr->attr.ctime;

			/* change directory entry */
			err_size = f3s_entry_change (part, &old_dir_desc, &old_dirent,
			                             new_name, new_length, &old_stat,
			                             old_dirent.moves);
			if (err_size < 0) {error = errno; goto old_file_shortcut;}

			f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
			/* find directory share in share list */
			dirent_share = (f3s_share_t *)f3s_list_find (&part_attr->share_list,
			                                             sizeof(old_dir_desc.first),
			                                             (uint8_t *)&old_dir_desc.first);
			if (dirent_share)
			{
				/* invalidate sharing directories */
				f3s_attr_inval (NULL, dirent_share->ocb->ocb.attr,
				                F3S_DESC_NO_LAST, old_dir_desc.curr_offset,
				                F3S_NAME_ALIGN(new_length) - F3S_NAME_ALIGN(old_length),
				                F3S_DESC_NO_REWIND);
			}
		}
	}

old_file_shortcut:
	/* unlock old file attributes */
	iofunc_attr_unlock (&old_file_attr->attr);

	/* check if there was no error and parent time update is wanted */
	if (!error && f3s.update_level >= F3S_UPDATE_PARENT)
	{
		/* initialize dirent descriptor */
		f3s_desc_init (&dirent_desc, old_dir_attr.parent_extptr, ~F3S_DESC_NO_DIR);

		/* read parent directory entry */
		error = f3s_entry_lookup (part, &dirent_desc, NULL, "", 1, &old_stat, F3S_CACHE_MAX);
		f3s_assert (error == EOK);
		if (error == EOK)
		{
			/* copy modification time */
			old_stat.mtime = old_dir_attr.attr.mtime;

			/* update directory entry */
			err_size = f3s_entry_update (part, &dirent_desc, &old_stat);
			if (err_size < 0) error = errno;
		}
	}

part_shortcut:
	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);
	return (error);
/*}}}2*/
}

int
f3s_mknod (resmgr_context_t * ctp,
		   io_mknod_t * msg,
		   f3s_attr_t * part_attr,
		   void *extra)
{
/*{{{2*/
	f3s_desc_t			dir_desc;
	f3s_desc_t			file_desc;
	f3s_attr_t			dir_attr;
	f3s_attr_t			file_attr;
	f3s_stat_t			stat;
	struct _client_info	info;
	f3s_part_t *		part;
	char*				name;
	uint32_t			mask;
	int16_t				length;
	int					err_size;
	int					error;

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);
	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return _RESMGR_ERRNO(EBADF);
	}
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition pointer */
	part = part_attr->part;

	/* climb to proper directory */
	error = f3s_namei (ctp, (io_open_t *) msg, part_attr, &dir_attr, &dir_desc, &info);
	if (error) goto part_shortcut;

	/* initialize name pointer */
	name = f3s_path_last (msg->connect.path, &length);

	/* find the file entry */
	error = f3s_entry_lookup (part, &dir_desc, NULL, name, length, &stat, dir_attr.depth);
	
	// if lookup fails for any reason other than found or not found, return the error
	if ((error != EOK) && (error != ENOENT))
	{
		goto part_shortcut;
	}
	else if (error == ENOENT)		// entry not found, create it
	{
		/* call iofunc mknod function */
		error = iofunc_mknod (ctp, msg, &file_attr.attr, &dir_attr.attr, &info);
		if (error) goto part_shortcut;

		/* check if mode is for a directory */
		if (S_ISDIR (file_attr.attr.mode))
		{
			/* setup mask with no directory bit cleared */
			mask = ~F3S_DESC_NO_DIR;

		} else
		{
			/* setup mask with no directory bit set */
			mask = F3S_DESC_INIT;
		}

		/* setup file descriptor */
		f3s_desc_init (&file_desc, f3s_null_extptr, mask);

		/* create the file */
		err_size = f3s_file_create (part, &file_desc, NULL, 0, NULL, 0);
		if (err_size < 0) {error = errno; goto part_shortcut;}

		iofunc_attr_lock(&file_attr.attr);
		/* call create function */
		error = f3s_create (ctp, (io_open_t *)msg, &dir_attr, &file_attr,
		                    &dir_desc, &file_desc, &stat, 0);
		iofunc_attr_unlock(&file_attr.attr);
		if (error)
		{
			/* rewind link descriptor */
			f3s_desc_rewind (&file_desc);

			/* delete link descriptor */
			f3s_file_delete (part, &file_desc, F3S_FILE_MAX);
			goto part_shortcut;
		}
	}
	else	/* object exists */
	{
		/* error if treating a non directory object as a directory */
		if ((msg->connect.eflag & _IO_CONNECT_EFLAG_DIR) && (!S_ISDIR(stat.mode)))
			error = ENOTDIR;
		else
			error = EEXIST;
	}

part_shortcut:
	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);
	return (error);
/*}}}2*/
}

int
f3s_readlink (resmgr_context_t * ctp,
			  io_readlink_t * msg,
			  f3s_attr_t * part_attr,
			  void *extra)
{
/*{{{2*/
	int error;
	int16_t length;
	char *name;
	struct _client_info info;
	f3s_desc_t dir_desc;
	f3s_dirent_t dirent;
	f3s_stat_t stat;
	f3s_attr_t file_attr, dir_attr;
	f3s_part_t *part;
	int  ret_val;

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return _RESMGR_ERRNO(EBADF);
	}
	f3s_lock_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition pointer */
	part = part_attr->part;

	/*
	 * if a trailing '/' was provided by the client on an object which is a link,
	 * (causing the _IO_CONNECT_EFLAG_DIR to be set), re-append the '/' here
	 * and let the code generate the appropriate error conditions (easiest way)
	*/
	if (msg->connect.eflag & _IO_CONNECT_EFLAG_DIR)
	{
		msg->connect.path[msg->connect.path_len - 1] = '/';
		msg->connect.path[msg->connect.path_len++] = '\0';
	}

	/* change to proper directory */
	error =
		f3s_namei (ctp, (io_open_t *) msg, part_attr, &dir_attr, &dir_desc,
				   &info);

	if (error)
		goto part_shortcut;

	/* initialize name pointer */
	name = f3s_path_last (msg->connect.path, &length);

	/* find the file entry */
	error = f3s_entry_lookup (part, &dir_desc, &dirent, name, length, &stat,
	                          dir_attr.depth);
	if (error != EOK)
		goto part_shortcut;

	/* clear file attributes */
	memset (&file_attr, 0, sizeof (file_attr));

	/* initialize file attributes */
	f3s_attr_init (&file_attr, 0, &file_attr, NULL, &stat, &info);

	/* call resource manager readlink */
	error = iofunc_readlink (ctp, msg, &file_attr.attr, &info);
	if (error) goto part_shortcut;

	/* resolve the link */
	error = f3s_link_resolve (ctp, msg, part_attr, dirent.first, 0, &ret_val);
	msg->link_reply.file_type = 0;	// only preserve eflag for readlink
	/* the next line is to compensate for a bug in readlink() which expects a null terminated
	 * path but does not want the '\0' character counted as part of link_reply.pathlen.
	 * f3s_link_resolve() ensures that the path returned is null terminated but counts the
	 * null character as expected by the connect code
	 * Unfortunately since all other filesystem resource managers also do this compensation
	 * the impetus to fix readlink() is low and so the flash filesystem has been made to conform.
	 */
	msg->link_reply.path_len -= 1;
	if (error == EOK)
	{
		ctp->status = EOK;
		error = ret_val;
	}

part_shortcut:
	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);

	return _RESMGR_ERRNO (error);
/*}}}2*/
}

int
f3s_link (resmgr_context_t * ctp,
		  io_link_t * msg,
		  f3s_attr_t * part_attr,
		  io_link_extra_t * extra)
{
/*{{{2*/
	int error;
	int16_t length;
	int32_t link_size;
	char *name;
	struct _client_info info;
	f3s_desc_t dir_desc, file_desc;
	f3s_attr_t dir_attr, file_attr;
	f3s_stat_t stat;
	f3s_part_t *part;

	/* make sure we are dealing with a symbolic link */
	if (msg->connect.extra_type != _IO_CONNECT_EXTRA_SYMLINK)
		return _RESMGR_ERRNO (ENOTSUP);
		
	/*
	 * the number of bytes passed to us as the link target includes the terminating NULL, so discount it
	* when doing range checking. The NULL will not be included when the link is actually created on flash.
	*/
	if (msg->connect.extra_len > (_POSIX_SYMLINK_MAX + 1))
		return _RESMGR_ERRNO(ENAMETOOLONG);

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return _RESMGR_ERRNO(EBADF);
	}
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition pointer */
	part = part_attr->part;

	/* change to proper directory */
	error =
		f3s_namei (ctp, (io_open_t *) msg, part_attr, &dir_attr, &dir_desc,
				   &info);

	if (error)
		goto part_shortcut;

	/* initialize name pointer */
	name = f3s_path_last (msg->connect.path, &length);

	/* find the file entry */
	error = f3s_entry_lookup (part, &dir_desc, NULL, name, length, NULL,
	                          dir_attr.depth);

	// if lookup returns anything other than found or not found, return the error
	if ((error != EOK) && (error != ENOENT))
	{
		goto part_shortcut;
	}
	else if (error == ENOENT)
	{
		/* call resource manager open function for create */
		error = iofunc_open (ctp, (io_open_t *)msg, &file_attr.attr,
		                     &dir_attr.attr, &info);
		if (error) goto part_shortcut;

		/* initialize link descriptor */
		f3s_desc_init (&file_desc, f3s_null_extptr, F3S_DESC_INIT);

		/* create the link pathname. Do not include the terminating NULL */
		link_size = f3s_file_create (part, &file_desc, NULL,
		                             msg->connect.extra_len - 1, (uint8_t*)extra->path, 1);

		/* check if link size makes no sense */
		if (link_size < 0)
		{
			error = errno;
			goto part_shortcut;
		}

		/* create the file */
		iofunc_attr_lock(&file_attr.attr);
		error = f3s_create (ctp, (io_open_t *)msg, &dir_attr, &file_attr,
		                    &dir_desc, &file_desc, &stat, 0);
		iofunc_attr_unlock(&file_attr.attr);
		if (error)
		{
			/* rewind link descriptor */
			f3s_desc_rewind (&file_desc);

			/* delete link descriptor */
			f3s_file_delete (part, &file_desc, F3S_FILE_MAX);

			goto part_shortcut;
		}
	}
	else	/* else, file was found */
	{
		error = EEXIST;
	}

part_shortcut:
	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);

	return _RESMGR_ERRNO (error);
/*}}}2*/
}

/*}}}1
 */

/*
 * Flash File System I/O Functions {{{1
 */
int
f3s_read (resmgr_context_t * ctp,
		  io_read_t * msg,
		  f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;
	int32_t desc_offset, buffer_size, message_size, read_size, done_size = 0;
	uint8_t *buffer;
	f3s_part_t *part;
	f3s_attr_t *attr;
	off64_t  read_offset;
	uint_t		pread;			// a boolean to record whether this is a pread or not (since msg->i gets reused

	/* reject unknown xtypes */
	if (((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_NONE) &&
		((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_OFFSET))
		return _RESMGR_ERRNO (ENOSYS);

	/* get attributes */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	if (attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* do posix read stuff and checks */
	error = iofunc_read_verify (ctp, msg, &ocb->ocb, NULL);

	if (error)
		return _RESMGR_ERRNO (error);

	/* synchronize file attributes */
	error = f3s_attr_sync (ocb, attr);

	f3s_assert (!error);

	if (error)
		return _RESMGR_ERRNO (error);

	/* get partition info pointer */
	part = attr->part;

	/* set message size */
	message_size = msg->i.nbytes;

	/* get buffer pointer */
	buffer = (uint8_t *) & msg->i;

	// setup the read position
	if ((msg->i.xtype & _IO_XTYPE_MASK) == _IO_XTYPE_OFFSET)
	{
		pread = 1;		// preserve whether or not this is a pread or not since msg->i is reused
		if ((read_offset = ((struct _xtype_offset *)&msg[1])->offset) < 0)
			return _RESMGR_ERRNO(EINVAL);		// current pread doc does not include EINVAL
	}
	else
	{
		pread = 0;		// preserve whether or not this is a pread or not since msg->i is reused
		read_offset = ocb->ocb.offset;
	}
	
	// set ocb->file_desc to the correct location for the read
	if (read_offset < f3s_desc_curr(&ocb->file_desc))
	{
		f3s_desc_rewind (&ocb->file_desc);
		f3s_file_forward (part, &ocb->file_desc, read_offset);
	}
	else if (read_offset > f3s_desc_curr(&ocb->file_desc))
	{
		f3s_file_forward (part, &ocb->file_desc, read_offset - f3s_desc_curr(&ocb->file_desc));
	}
	
	/* read complete message loop */
	while (done_size < message_size) {

		/* find read size for read buffer */
		buffer_size = min (F3S_READ_MAX, message_size - done_size);

		/* calculate descriptor offset */
		desc_offset = read_offset - f3s_desc_curr (&ocb->file_desc);

		/* read from file */
		read_size =
			f3s_file_read (part, &ocb->file_desc, desc_offset, buffer_size,
						   buffer);

		/* check if read size makes no sense */
		if (read_size < 0)
		{
			// FIX ME - ?
			// don't think we should adjust the offset if failing the read. Should probably return what has been
			// read so far. For now, keep the pre-existing behaviour
			if (!pread)
				ocb->ocb.offset = read_offset;
			return _RESMGR_ERRNO (errno);
		}

		/* write message to client buffer */
		read_size = resmgr_msgwrite (ctp, buffer, read_size, done_size);

		/* check if read size makes no sense */
		if (read_size < 0)
		{
			// FIX ME - ?
			// don't think we should adjust the offset if failing the read. Should probably return what has been
			// read so far. For now, keep the pre-existing behaviour
			if (!pread)
				ocb->ocb.offset = read_offset;
			return _RESMGR_ERRNO (errno);
		}

		/* update ocb offset in file */
		read_offset += read_size;

		/* update read size */
		done_size += read_size;

		/* check for end of file */
		if (read_size < buffer_size) {

			/* this is the end of file, break of loop */
			break;
		}
	}

	// if doing a regular read, update the read position
	if (!pread)
		ocb->ocb.offset = read_offset;
		
	/* return number of bytes read */
	_IO_SET_READ_NBYTES (ctp, done_size);

	return _RESMGR_ERRNO (EOK);
/*}}}2*/
}

int
f3s_write (resmgr_context_t * ctp,
		   io_write_t * msg,
		   f3s_ocb_t * ocb)
{
/*{{{2*/
	int32_t buffer_size, chunk_size, write_size = 0, append_flag =
		0, done_size, desc_offset, flush_flag =
		0, sync_flags, save_offset, forward_size;
	uint8_t *buffer;
	f3s_attr_t *attr;
	f3s_part_t *part;
	int		error;
	off64_t  write_offset;
	ssize_t	extra_hdr_bytes;

	/* reject unknown xtypes */
	if (((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_NONE) &&
		((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_OFFSET))
		return _RESMGR_ERRNO (ENOSYS);

	/* get attributes pointer */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	if (attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* do posix write stuff and checks */
	error = iofunc_write_verify (ctp, msg, &ocb->ocb, NULL);
	if (error)
		return _RESMGR_ERRNO (error);

	/*
	 * setup the write_offset. Also need to save the current ocb->ocb.offset since iofunc_write_verify
	 * will change it. Depending on the type of write, the ocb->ocb.offset will either be restored or updated
	*/
	save_offset = ocb->ocb.offset;
	if ((msg->i.xtype & _IO_XTYPE_MASK) == _IO_XTYPE_OFFSET)
	{
		if ((write_offset = ((struct _xtype_offset *)&msg[1])->offset) < 0)
			return _RESMGR_ERRNO(EINVAL);
	}
	else
		write_offset = save_offset;

	/* get partition information pointer */
	part = attr->part;

	/* check if file needs rewind */
	if (!(ocb->file_desc.status & F3S_DESC_NO_REWIND) ||
		(write_offset < f3s_desc_curr(&ocb->file_desc)))
	{
		/* rewind file descriptor */
		f3s_desc_rewind (&ocb->file_desc);

		/* forward file descriptor to offset */
		forward_size = f3s_file_forward (part, &ocb->file_desc, write_offset);
		if (forward_size < 0)
			return _RESMGR_ERRNO (errno);

		/* set not rewind and no forward flag in file descriptor */
		ocb->file_desc.status |= (F3S_DESC_NO_REWIND | F3S_DESC_NO_FORWARD);
	}
	/* else, check if file needs forward */
	else if (!(ocb->file_desc.status & F3S_DESC_NO_FORWARD) ||
				 (write_offset > f3s_desc_curr(&ocb->file_desc)))
	{
		/* forward file descriptor to ocb offset */
		forward_size = f3s_file_forward (part, &ocb->file_desc,
										 write_offset - ocb->file_desc.curr_offset);
		if (forward_size < 0)
			return _RESMGR_ERRNO (errno);

		/* set no forward flag in file descriptor */
		ocb->file_desc.status |= F3S_DESC_NO_FORWARD;
	}

	/* get sync flags */
	sync_flags = iofunc_sync (ctp, &ocb->ocb, O_DSYNC | _IO_FLAG_WR);

	/* get size of write */
	buffer_size = msg->i.nbytes;

	/* get size of chunk */
	chunk_size = min (buffer_size, F3S_WRITE_MAX);

	/* get buffer pointer */
	if ((msg->i.xtype & _IO_XTYPE_MASK) == _IO_XTYPE_OFFSET){
		buffer = (uint8_t *)(&msg->i + 1) + sizeof(struct _xtype_offset);
		extra_hdr_bytes = sizeof(struct _xtype_offset);
	}else{
		buffer = (uint8_t *) (&msg->i + 1);
		extra_hdr_bytes = 0;
	}

	/* while write size is smaller than size */
	while (write_size < buffer_size)
	{
		f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
		
		/* check if file flag is for append. Ignore it for pwrite() */
		if ((ocb->ocb.ioflag & O_APPEND) &&
			((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_OFFSET))
		{
			/* set append flag */
			append_flag = 1;
		}
		/* else, check if end of file is known */
		else if (f3s_desc_end (&ocb->file_desc)) {
			/* check if we are at end of file */
			if (write_offset ==
				(f3s_desc_size (&ocb->file_desc) + attr->buffer_pos)) {
				/* set append flag */
				append_flag = 1;
			}
		}

		/* check if we are appending */
		if ((append_flag)) {
			
			/* check if buffer pointer is null */
			/* If we have an append buffer, that means we'll get a delayed
			 * write, when the buffer is flushed.  To make sure we have
			 * space for this, we need to setup a reservation that's
			 * big enough to hold the append buffer
			 */
			if (!attr->buffer && f3s.buffer_size) {

				f3s_assert(attr->resv_ptr == NULL);
				/* initialize reservation pointer */
				attr->resv_ptr = f3s_malloc (sizeof (*attr->resv_ptr));

				/* check if reservation structure was allocated */
				if (attr->resv_ptr)
				{
					memset(attr->resv_ptr, 0, sizeof(*attr->resv_ptr));
					
					/* initialize append buffer */
					attr->buffer = f3s_malloc (f3s.buffer_size);

					/* check if buffer allcoation was successfull */
					if (attr->buffer) {
						/* attempt to reserve flash space */
						f3s_attr_resv (attr, f3s.buffer_size);
					} else {
						/* free reservation structure */
						f3s_free (attr->resv_ptr, sizeof (*attr->resv_ptr));
						attr->resv_ptr = NULL;
					}
				}
			}

			f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
			/* check if buffer to write is bigger than append buffer space */
			if (sync_flags || !attr->buffer ||
				chunk_size > (f3s.buffer_size - attr->buffer_pos)) {
				/* check if there is something in append buffer */
				if (attr->buffer_pos) {
					/* flush the append buffer to the file */
					done_size =
						f3s_file_append (part, &ocb->file_desc,
										 attr->resv_ptr, attr->buffer_pos,
										 attr->buffer, 0);

					/* attempt to reserve flash space again */
					f3s_attr_resv (attr, f3s.buffer_size);

					/* check if done size makes sense */
					if (done_size < 0) {
						error = errno;
						goto fail_shortcut;
					}

					/* set flush flag */
					flush_flag = 1;

					/* clear the append buffer position */
					attr->buffer_pos = 0;

					/* get current offset */
					write_offset =
						f3s_file_forward (part, &ocb->file_desc,
										  F3S_FILE_MAX);
					if (write_offset < 0) {
						error = errno;
						goto fail_shortcut;
					}
				}

				/* check if buffer could fit into append buffer */
				if (!sync_flags && attr->buffer &&
					chunk_size <= f3s.buffer_size) {
					/* copy buffer to append buffer */
					memcpy (attr->buffer, buffer, chunk_size);

					/* set append buffer position */
					attr->buffer_pos += chunk_size;

					/* set done size to chunk size */
					done_size = chunk_size;
				}
				else	/* else, buffer does not fit into append buffer */
				{
					/* write with file append function */
					done_size =
						f3s_file_append (part, &ocb->file_desc, NULL,
										 chunk_size, buffer, 0);
					if (done_size < 0) {
						error = errno;
						goto fail_shortcut;
					}

					/* set flush flag */
					flush_flag = 1;

					/* get current offset */
					write_offset =
						f3s_file_forward (part, &ocb->file_desc,
										  F3S_FILE_MAX) - done_size;
					if (write_offset < 0) {
						error = errno;
						goto fail_shortcut;
					}
				}
			}
			else	/* else, buffer is smaller than append buffer space */
			{
				/* copy buffer to append buffer */
				memcpy (attr->buffer + attr->buffer_pos, buffer, chunk_size);

				/* set append buffer position */
				attr->buffer_pos += chunk_size;

				/* set done size to chunk size */
				done_size = chunk_size;
			}
		}
		else	/* else, we are not appending */
		{
			/* check if there is something in append buffer */
			if (attr->buffer_pos) {
				
				f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
				/* flush the append buffer to the file */
				done_size = f3s_file_append (part, &ocb->file_desc, attr->resv_ptr,
											 attr->buffer_pos, attr->buffer, 0);

				/* attempt to reserve flash space again */
				f3s_attr_resv (attr, f3s.buffer_size);

				/* check if append size makes no sense */
				if (done_size < 0) {
					error = errno;
					goto fail_shortcut;
				}

				/* set flush flag */
				flush_flag = 1;

				/* clear the append buffer position */
				attr->buffer_pos = 0;

				/* setup last unknown in ocb descriptor */
				ocb->file_desc.status |= F3S_DESC_NO_LAST;

				/* check if file needs rewind */
				if (write_offset < f3s_desc_curr(&ocb->file_desc))
				{
					/* rewind file descriptor */
					f3s_desc_rewind (&ocb->file_desc);

					/* forward file descriptor to offset */
					forward_size = f3s_file_forward (part, &ocb->file_desc, write_offset);
					if (forward_size < 0)
						return _RESMGR_ERRNO (errno);

					/* set not rewind and no forward flag in file descriptor */
					ocb->file_desc.status |= (F3S_DESC_NO_REWIND | F3S_DESC_NO_FORWARD);
				}
			}

			/* calculate descriptor offset */
			desc_offset = write_offset - f3s_desc_curr (&ocb->file_desc);

			/* write with file write function */
			done_size =
				f3s_file_write (part, &ocb->file_desc, desc_offset,
								chunk_size, buffer);
			if (done_size < 0) {
				error = errno;
				goto fail_shortcut;
			}

			/* set flush flag */
			flush_flag = 1;
		}

		/* update offset in file */
		write_offset += done_size;

		/* update write size */
		write_size += done_size;

		/* check if there is still something to write */
		if (write_size < buffer_size) {
			/* get size of chunk */
			chunk_size = min (buffer_size - write_size, F3S_WRITE_MAX);

			/* read message */
			chunk_size = resmgr_msgread (ctp, buffer, chunk_size,
										 sizeof (msg->i) + write_size + extra_hdr_bytes);
			if (chunk_size < 0) {error = errno; goto fail_shortcut;}
			if (chunk_size == 0) break;
		}
	}

fail_shortcut:
	/* check if flush was performed */
	if (flush_flag) {
		/* invalidate sharing ocb decriptors */
		f3s_attr_inval (ocb, attr, F3S_DESC_NO_LAST, 0, 0, F3S_DESC_NO_REWIND);
	}

	// if doing a regular write, update the write position
	if ((msg->i.xtype & _IO_XTYPE_MASK) == _IO_XTYPE_OFFSET)
		ocb->ocb.offset = save_offset;
	else
		ocb->ocb.offset = write_offset;

	/* check for error other than no space and nothing was written */
	if ((error) && ((error != ENOSPC) || (!write_size))) return (error);

	/* everything went fine */
	attr->attr.flags |= IOFUNC_ATTR_MTIME;
	_IO_SET_WRITE_NBYTES (ctp, write_size);
	return (EOK);
/*}}}2*/
}

int f3s_close(resmgr_context_t *ctp,
              f3s_ocb_t *ocb)
{
/*{{{2*/
	int error = EOK, flags;
	f3s_part_t *part;
	f3s_attr_t *attr;
	uint8_t  attached;		// boolen indicating whether or not ocb was actually attached

	/* get attributes pointer */
	attr = ocb->ocb.attr;
  
	/* lock file attributes */
	iofunc_attr_lock(&attr->attr);

	/* get partition info pointer */
	part = attr->part;

	/* check if time update is not wanted */
	if (f3s.update_level < F3S_UPDATE_FILE)
	{
		/* knock down time flags */
		attr->attr.flags &= ~(IOFUNC_ATTR_MTIME |
				      IOFUNC_ATTR_CTIME |
   				      IOFUNC_ATTR_ATIME);
	}

	/* detach ocb */
	attached = f3s_ocb_detach(ctp, ocb, attr);
	f3s_assert(attached);
	flags = iofunc_ocb_detach (ctp, &ocb->ocb);

	/* check if there is no more ocb for attributes */
	if (flags & IOFUNC_OCB_LAST_INUSE)
	{
		f3s_assert(attr->attr.nlink >= 0);

		/* check if postponed delete is wanted and last client */
		if ((attr->attr.nlink == 0) && (attr->pending_delete))
		{
			/* rewind descriptor */
			f3s_desc_rewind(&ocb->file_desc);

   			/* delete the extents to end of file */
			f3s_file_delete(part, &ocb->file_desc, F3S_FILE_MAX);
    		}
		/* else, we keep the file */
		else
		{
			/* synchronize file attributes, error should not prevent close */
			error = f3s_attr_sync(ocb, attr);
			f3s_assert(!error);

			/* synchronize meta data, error should not prevent close */
			error = f3s_sync_meta(ocb, attr);

			/* change the status of xcache to 'expired' */
			f3s_xcache_expire(part, &ocb->file_desc);
		}

		/* check if we have a reservation */
		f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
		if (attr->resv_ptr)
		{
			/* free that reservation */
			F3S_TABLE_ENTER(part);
			f3s_resv_free(part, attr->resv_ptr);
			F3S_TABLE_LEAVE(part);

			/* free reservation memory */
			f3s_free(attr->resv_ptr, sizeof(*attr->resv_ptr));
		}

		/* free attributes, and append buffer */
		f3s_free(attr->buffer, f3s.buffer_size);
		f3s_free(attr, sizeof(*attr));
	}
	else	/* else, attributes still live */
	{
		/* unlock file attributes */
		iofunc_attr_unlock(&attr->attr);
	}

	/* free ocb memory and buffer */
	f3s_free(ocb, sizeof(*ocb));

	/* everything went fine */
	return error;
/*}}}2*/
}

int
f3s_close_ocb (resmgr_context_t * ctp,
			   void *reserved,
			   f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;
	f3s_attr_t *part_attr;

	/* get partition attributes pointer */
	part_attr = ocb->ocb.attr->part_attr;

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	/* call close function */
	error = f3s_close (ctp, ocb);

	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);

	/* return error code */
	return _RESMGR_ERRNO (error);
/*}}}2*/
}

int
f3s_stat (resmgr_context_t * ctp,
		  io_stat_t * msg,
		  f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;
	int32_t file_size, curr_offset;
	f3s_extptr_t curr;
	f3s_attr_t *attr;
	f3s_part_t *part;

	/* get attribute pointer */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	if (attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* synchronize file attributes */
	error = f3s_attr_sync (ocb, attr);

	f3s_assert (!error);

	if (error)
		return _RESMGR_ERRNO (error);

	/* get partition pointer */
	part = attr->part;

	/* remember current position in file descriptor */
	curr = ocb->file_desc.curr;
	curr_offset = ocb->file_desc.curr_offset;

	/* get the file size */
	file_size = f3s_file_forward (part, &ocb->file_desc, F3S_FILE_MAX);

	if (file_size < 0)
		return _RESMGR_ERRNO (errno);

	/* reset the file descriptor */
	ocb->file_desc.curr = curr;
	ocb->file_desc.curr_offset = curr_offset;

	/* check if file size makes no sense */
	if (file_size < 0)
		return _RESMGR_ERRNO (errno);

	/* fill up attribute nbytes field */
	attr->attr.nbytes = file_size;

	/* update attribute times */
	iofunc_time_update (&attr->attr);

	/* call iofunc stat */
	error = iofunc_stat (ctp, &attr->attr, &msg->o);

	if (error)
		return _RESMGR_ERRNO (error);

	/* return iov with data and length */
	return _RESMGR_PTR (ctp, &msg->o, sizeof (msg->o));
/*}}}2*/
}

int
f3s_devctl (resmgr_context_t * ctp,
			io_devctl_t * msg,
			f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;

	/* benchmarking early exit */
	if (!msg->i.dcmd)
		return _RESMGR_ERRNO (EOK);

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));

	if (ocb->ocb.attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((ocb->ocb.attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* do standard devctl stuff */
	error = iofunc_devctl (ctp, msg, &ocb->ocb, &ocb->ocb.attr->attr);

	/* check if devctl was not processed */
	if (error == _RESMGR_DEFAULT) {

		/* do f3s specific stuff depending on message */
		switch (msg->i.dcmd) {
			case DCMD_F3S_RECLAIM:

				error = f3s_reclaim (ctp, msg, ocb);

				break;

			case DCMD_F3S_ARRAYINFO:

				error = f3s_arrayinfo (ctp, msg, ocb);

				break;

			case DCMD_F3S_PARTINFO:

				error = f3s_partinfo (ctp, msg, ocb);

				break;

			case DCMD_F3S_UNITINFO:

				error = f3s_unitinfo (ctp, msg, ocb);

				break;

			case DCMD_FSYS_STATVFS:

				error = f3s_statvfs (ctp, msg, ocb);

				break;

			case DCMD_FSYS_MOUNTED_AT:

				error = f3s_mountedat (ctp, msg, ocb);

				break;

			case DCMD_FSYS_MOUNTED_ON:

				error = f3s_mountedon (ctp, msg, ocb);

				break;

			case DCMD_F3S_EXIT:

				f3s_exit (ctp, msg, ocb);

				break;
			case DCMD_F3S_RECLAIMCTL:
				error = f3s_reclaimctl(ctp, msg, ocb);
				break;
			
			case DCMD_F3S_GEOINFO:
				error = f3s_geoinfo(ctp, msg, ocb);
				break;

			default:

				error = ENOSYS;
		}
	}

	/* return result message */
	return _RESMGR_ERRNO (error);
/*}}}2*/
}

int
f3s_pathconf (resmgr_context_t * ctp,
			  io_pathconf_t * msg,
			  f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));

	if (ocb->ocb.attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((ocb->ocb.attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* do standard pathconf stuff */
	error = iofunc_pathconf (ctp, msg, &ocb->ocb, &ocb->ocb.attr->attr);

	/* check if pathconf was not processed */
	if (error == _RESMGR_DEFAULT) {

		/* clear error code */
		error = EOK;

		/* do f3s specific stuff depending on message */
		switch (msg->i.name) {
			case _PC_LINK_MAX:	/* maximum number of hard links */
				if (S_ISDIR (ocb->ocb.attr->attr.mode)) {

					/* hard links not available for directories */
					_IO_SET_PATHCONF_VALUE (ctp, 2);
				} else {

					/* hard links not available for files */
					_IO_SET_PATHCONF_VALUE (ctp, 1);
				}

				break;

			case _PC_NAME_MAX:	/* maximum length of a name */
				_IO_SET_PATHCONF_VALUE (ctp, F3S_NAME_MAX);

				break;

			case _PC_PATH_MAX:	/* maximum length of path  */
				_IO_SET_PATHCONF_VALUE (ctp, F3S_PATH_MAX);

				break;

			case _PC_SYMLINK_MAX:	/* maximum length of a symlink  */
				_IO_SET_PATHCONF_VALUE (ctp, _POSIX_SYMLINK_MAX);

				break;
				
			default:

				error = EINVAL;
		}
	}

	/* return result message */
	return _RESMGR_ERRNO (error);
/*}}}2*/
}

int
f3s_sync_meta (f3s_ocb_t * ocb,
			   f3s_attr_t * attr)
{
/*{{{2*/
	int error = EOK;
	int32_t update_size = 0;
	f3s_stat_t stat;
	f3s_desc_t dirent_desc;
	f3s_part_t *part;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
	f3s_lock_assert((attr->part_attr->attr.lock_tid == pthread_self()) && (attr->part_attr->attr.lock_count > 0));

	if (attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return(EBADF);
	f3s_assert((attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);
	
	/* get partition info pointer */
	part = attr->part;

	/* check for attr update */
	if (attr->attr.flags & (IOFUNC_ATTR_DIRTY_OWNER | IOFUNC_ATTR_DIRTY_MODE |
							IOFUNC_ATTR_DIRTY_TIME)) {

		/* initialize dirent descriptor */
		f3s_desc_init (&dirent_desc, attr->parent_extptr, ~F3S_DESC_NO_DIR);

		/* retrace directory entry */
		if (f3s_entry_retrace (part, &dirent_desc, ocb->file_desc.first) == 0)
		{
			//printf("Lost: Parent[0x%X,0x%X] First[0x%X,0x%X]\n",
			//       attr->parent_extptr.logi_unit, attr->parent_extptr.index,
			//       ocb->file_desc.first.logi_unit, ocb->file_desc.first.index);
			return (ENOENT);
		}

		/* update stat values */
		stat.struct_size = sizeof (stat);
		stat.mode        = attr->attr.mode;

		stat.uid = attr->attr.uid;
		stat.gid = attr->attr.gid;

		stat.mtime = attr->attr.mtime;
		stat.ctime = attr->attr.ctime;

		/* update directory entry */
		update_size = f3s_entry_update (part, &dirent_desc, &stat);

		/* check if there was not enough space */
		if (update_size < 0) error = errno;
	}

	/* return error code */
	return error;
/*}}}2*/
}

int
f3s_sync (resmgr_context_t * ctp,
		  io_sync_t * msg,
		  f3s_ocb_t * ocb)
{
/*{{{2*/
	int error, sync_flags;
	f3s_attr_t *part_attr, *file_attr;

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));

	/* verify sync stuff */
	error = iofunc_sync_verify (ctp, msg, &ocb->ocb);

	if (error)
		return _RESMGR_ERRNO (error);

	/* get attributes pointer */
	file_attr = ocb->ocb.attr;

	if (file_attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((file_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);
	
	/* get sync flags */
	sync_flags = iofunc_sync (ctp, &ocb->ocb, 0);

	/* check sync flags for data sync */
	if (sync_flags & O_DSYNC) {

		/* synchronize file attributes */
		error = f3s_attr_sync (ocb, file_attr);

		f3s_assert (!error);

		if (error)
			return _RESMGR_ERRNO (error);
	}

	/* check sync flags for meta data sync */
	if (sync_flags & O_SYNC) {
		f3s_lock_assert((file_attr->attr.lock_tid == pthread_self()) && (file_attr->attr.lock_count > 0));
		/* get partition attributes pointer */
		part_attr = file_attr->part_attr;

		/* lock partition attributes */
		iofunc_attr_lock (&part_attr->attr);

		/* lock file attributes */
		iofunc_attr_lock (&file_attr->attr);

		/* update attribute times */
		iofunc_time_update (&file_attr->attr);

		/* synchronize meta data */
		error = f3s_sync_meta (ocb, file_attr);

		/* unlock file attributes */
		iofunc_attr_unlock (&file_attr->attr);

		/* unlock partition attributes */
		iofunc_attr_unlock (&part_attr->attr);

		if (error)
			return _RESMGR_ERRNO (error);
	}

	/* everything went fine */
	return _RESMGR_ERRNO (error);
/*}}}2*/
}

int
f3s_lseek (resmgr_context_t * ctp,
		   io_lseek_t * msg,
		   f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;
	int32_t file_size, forward_size, file_offset;
	f3s_attr_t *attr;
	f3s_part_t *part;

	/* get attributes */
	attr = ocb->ocb.attr;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	if (attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* synchronize file attributes */
	error = f3s_attr_sync (ocb, attr);

	f3s_assert (!error);

	if (error)
		return _RESMGR_ERRNO (error);

	/* get partition info pointer */
	part = attr->part;

	/* check if seek to end is necessary */
	if (msg->i.whence == SEEK_END) {

		/* go to file end to get file size */
		file_size = f3s_file_forward (part, &ocb->file_desc, F3S_FILE_MAX);

		if (file_size < 0)
			return _RESMGR_ERRNO (errno);

		/* set nbytes into attributes */
		attr->attr.nbytes = file_size;
	}

	/* call iofunc lseek function */
	error = iofunc_lseek (ctp, msg, &ocb->ocb, &attr->attr);

	if (error > EOK)
		return _RESMGR_ERRNO (error);

	/* get file current offset */
	file_offset = f3s_desc_curr (&ocb->file_desc);

	/* check if displacement is positive or null */
	if (ocb->ocb.offset >= file_offset)

		/*
		 * set size of forward with offset in message relative to
		 * current
		 */
		forward_size = ocb->ocb.offset - file_offset;

	else {

		/* rewind file pointers */
		f3s_desc_rewind (&ocb->file_desc);

		/* set size of forward with offset in message */
		forward_size = ocb->ocb.offset;
	}

	/* check if seek is negative */
	if (forward_size < 0)
		return _RESMGR_ERRNO (EINVAL);

	/* forward file pointers */
	file_size = f3s_file_forward (part, &ocb->file_desc, forward_size);

	if (file_size < 0)
		return _RESMGR_ERRNO (errno);
		
	/* return iofunc lseek error */
	return _RESMGR_ERRNO (error);
/*}}}2*/
}

int
f3s_space (resmgr_context_t * ctp,
		   io_space_t * msg,
		   f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;
	int32_t forward_size, file_size, grow_size, file_offset;
	f3s_attr_t *attr;
	f3s_part_t *part;
	f3s_desc_t seek_desc, size_desc;

	/* get attributes */
	attr = ocb->ocb.attr;

	/* ensure size is less than 2^31 */
	if ((msg->i.start + msg->i.len) >= 0x80000000)
		return _RESMGR_ERRNO (EFBIG);
	if ((msg->i.start + msg->i.len) < msg->i.start)
		return _RESMGR_ERRNO (EFBIG);

	/* check for privileges */
	error = iofunc_space_verify (ctp, msg, &ocb->ocb, NULL);
	if (error)
		return _RESMGR_ERRNO (error);

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
	if (attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* synchronize file attributes */
	error = f3s_attr_sync (ocb, attr);
	f3s_assert (!error);
	if (error)
		return _RESMGR_ERRNO (error);

	/* keep a copy of descriptor for seeking */
	seek_desc = ocb->file_desc;

	/* get partition info pointer */
	part = attr->part;

	/* check what kind of seek we are doing */
	switch (msg->i.whence) {
			/* we are absolutely setting the start */
		case SEEK_SET:
			/* get file current offset */
			file_offset = f3s_desc_curr (&seek_desc);

			/* check if new position is behind current offset */
			if (msg->i.start < file_offset) {
				/* rewind file pointers */
				f3s_desc_rewind (&seek_desc);

				/* set size of forward with start in message */
				forward_size = msg->i.start;
			} else {
				/* set size of forward with start in message relative to current */
				forward_size = msg->i.start - file_offset;
			}

			/* check if seek is negative */
			if (forward_size < 0)
				return _RESMGR_ERRNO (EINVAL);

			/* forward file pointers */
			f3s_file_forward (part, &seek_desc, forward_size);

			/* setup new file offset */
			file_offset = msg->i.start;

			break;

			/* we are relatively setting the start */
		case SEEK_CUR:
			/* get file current start */
			file_offset = f3s_desc_curr (&seek_desc);

			/* check if displacement is positive */
			if (ocb->ocb.offset + msg->i.start >= file_offset) {
				/* set size of forward with start in message */
				forward_size = msg->i.start;
			}
			/* displacement is negative */
			else {
				/* rewind file pointers */
				f3s_desc_rewind (&seek_desc);

				/* set size of forward relative to previous offset */
				forward_size = ocb->ocb.offset + msg->i.start;
			}

			/* check if seek is negative */
			if (forward_size < 0)
				return _RESMGR_ERRNO (EINVAL);

			/* forward file pointers */
			f3s_file_forward (part, &seek_desc, forward_size);

			/* setup new file offset */
			file_offset += msg->i.start;

			break;

			/* we want to move to the end of the file */
		case SEEK_END:
			/* go to file end */
			file_offset = f3s_file_forward (part, &seek_desc, F3S_FILE_MAX);

			break;

			/* anything else */
		default:
			/* invalid seek */
			return _RESMGR_ERRNO (EINVAL);
	}

	/* get a copy of seek desc */
	size_desc = seek_desc;

	/* find size of file */
	file_size = f3s_file_forward (part, &size_desc, F3S_FILE_MAX);

	/* check what kind of space we are doing */
	switch (msg->i.subtype) {
			/* we are allocating space */
		case F_ALLOCSP:
		case F_ALLOCSP64:
			/* find grow size for file */
			grow_size = file_offset - file_size + msg->i.len;

			/* check if grow is necessary */
			if (grow_size > 0) {
				/* grow file with grow size */
				grow_size =
					f3s_file_grow (part, &size_desc, grow_size, 0, NULL);

				/* check if grow size is valid */
				if (grow_size < 0)
					return _RESMGR_ERRNO (errno);

				/* invalidate sharing ocb decriptors */
				f3s_attr_inval (NULL, attr, F3S_DESC_NO_LAST, file_size, 0,
								F3S_DESC_NO_FORWARD);

				/* adjust file size */
				file_size += grow_size;
			}

			break;

			/* we are freeing space */
		case F_FREESP:
		case F_FREESP64:
			/* check if truncate is possible */
			if (file_offset < file_size && (!msg->i.len ||
											file_offset + msg->i.len >=
											file_size)) {
				/* truncate file */
				file_size = f3s_file_trunc (part, &seek_desc, file_offset -
											f3s_desc_curr (&seek_desc));

				/* check if file size makes sense */
				if (file_size < 0)
					return _RESMGR_ERRNO (errno);

				/* invalidate sharing ocb decriptors */
				f3s_attr_inval (NULL, attr, F3S_DESC_NO_LAST, 0, 0,
								F3S_DESC_NO_REWIND);

				/* adjust file size */
				file_size = file_offset;
			}

			break;

			/* anything else */
		default:
			/* invalid space */
			return _RESMGR_ERRNO (EINVAL);
	}

	/* reflect size setting in message */
	msg->o = file_size;
	return _RESMGR_PTR (ctp, &msg->o, sizeof (msg->o));
/*}}}2*/
}

int
f3s_umount (resmgr_context_t * ctp,
			io_mount_t * msg,
			f3s_attr_t * part_attr,
			io_mount_extra_t * extra)
{
/*{{{2*/
	int error = EOK;
	f3s_part_t *part;
	struct _client_info info;

	//this is not a flash mount point
	if (*msg->connect.path != 0)
		return (ENOSYS);

	//the user isn 't root
	if ((iofunc_client_info (ctp, msg->connect.ioflag, &info) != EOK) ||
		(info.cred.euid != 0))
		return (EPERM);

	// go single threaded for the unmount
	thread_pool_limits(f3s.tpp, 0, 0, 1, 1, 0);

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	/* get partition info pointer */
	part = part_attr->part;
	F3S_TABLE_ENTER(part);

	/* check if remount is wanted */
	if (extra->flags & _MOUNT_REMOUNT) {
		/* set mount flags */
		part_attr->attr.mount->mount.flags &= ~0xf;
		part_attr->attr.mount->mount.flags |= extra->flags & 0xf;
	}
	/* else, check if unmount is wanted */
	else if (extra->flags & _MOUNT_UNMOUNT)
	{
		/* if no other process is using the filesystem or the force flag is provided, unmount */
		if ((part_attr->share_list.next == &part_attr->share_list) ||
			(extra->flags & _MOUNT_FORCE))
		{
			/* detach file system attributes from partition */
			error = f3s_part_detach (part, NULL, part_attr);
			f3s_assert(error == EOK);
		}
		else		/* filesystem is in use and the force flag has not been provided */
		{
			error = EBUSY;
		}
	}
	F3S_TABLE_LEAVE(part);

	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);
	
	/* return the thread pool to its initial state */
	thread_pool_limits(f3s.tpp, f3s.thread_num - 1, f3s.thread_num, f3s.thread_num + 1, 1,
										THREAD_POOL_CONTROL_NONBLOCK);
	return (error);
/*}}}2*/
}

/*}}}1
 */

/*
 * Directory Specific I/O Functions {{{1
 */
int
f3s_readdir (resmgr_context_t * ctp,
			 io_read_t * msg,
			 f3s_ocb_t * ocb)
{
/*{{{2*/
	#define ALIGN(_x)    (((_x) + 7) & ~7)
	int error;
	int32_t dirent_size, buffer_size, read_size, done_size = 0, desc_offset,
		reclen, name_size, forward_size;
	struct dirent *buffer = NULL;
	f3s_dirent_t dirent_buffer;
	f3s_part_t *part;
	f3s_attr_t *file_attr, *part_attr;

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));

	/* reject unknown xtypes */
	if ((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_NONE)
		return _RESMGR_ERRNO (ENOSYS);

	/* do posix read stuff and checks */
	error = iofunc_read_verify (ctp, msg, &ocb->ocb, NULL);

	if (error)
		return _RESMGR_ERRNO (error);

	/* get attributes */
	file_attr = ocb->ocb.attr;

	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));

	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	/* get partition info pointer */
	part = file_attr->part;

	/* find read size for read buffer */
	buffer_size = min(msg->i.nbytes, ctp->msg_max_size - ctp->offset);

	/* get buffer pointer */
	buffer = (struct dirent *) &msg->i;

	/* synchronize file attributes */
	error = f3s_attr_sync (ocb, file_attr);

	f3s_assert (!error);

	if (error)
		return _RESMGR_ERRNO (error);

	/* for all directory entries */
	while (1) {

		/* calculate descriptor offset */
		desc_offset = ocb->ocb.offset - f3s_desc_curr (&ocb->file_desc);

		/* read from file */
		dirent_size = f3s_dirent_read (part, &ocb->file_desc, desc_offset,
									   &dirent_buffer);
		if (dirent_size < 0)
		{
			/* rewind descriptor */
			f3s_desc_rewind (&ocb->file_desc);

			/* read from file */
			dirent_size = f3s_dirent_read (part, &ocb->file_desc, 0, &dirent_buffer);
			if (dirent_size < 0)
			{
				return _RESMGR_ERRNO (errno);
			}
		}

		/* check if there was something to read and we have space to read it */
		if ((dirent_size) &&
			(buffer_size >= ALIGN(offsetof(struct dirent, d_name) + dirent_buffer.namelen + 1)))
		{
			/* setup size of read */
			read_size = ALIGN(offsetof(struct dirent, d_name) + dirent_buffer.namelen + 1);

			/* calculate size of record with dirent, name length and stat */
			reclen = sizeof(dirent_buffer) + F3S_NAME_ALIGN(dirent_buffer.namelen) + sizeof(f3s_stat_t);

			/* fill buffer with dirent information, use memcpy to avoid "strict-aliasing" warning */
			memcpy (&buffer->d_ino, &dirent_buffer.first, 
					min(sizeof(buffer->d_ino), sizeof(dirent_buffer.first)));
			buffer->d_ino_hi = 0;

			buffer->d_offset = ocb->ocb.offset;
			buffer->d_offset_hi = 0;

			buffer->d_reclen = read_size;
			buffer->d_namelen = dirent_buffer.namelen;	// will be adjusted as required after name is read

			/* read file name */
			name_size = f3s_file_read (part, &ocb->file_desc, sizeof(dirent_buffer),
			                           F3S_NAME_ALIGN(dirent_buffer.namelen),
			                           (uint8_t*)buffer->d_name);
			if (name_size < 0)
			{
				return _RESMGR_ERRNO (errno);
			}
			/* PR18519
			 * if the entry name was F3S_NAME_MAX + 1 (including NULL termination)
			 * fix up the length for the struct dirent (which can handle names > 255) */
			if (buffer->d_name[dirent_buffer.namelen - 1] != '\0')
			{
				buffer->d_namelen += 1;
				f3s_assert(buffer->d_name[buffer->d_namelen - 1] == '\0');	// had better be terminated
			}
				
			/* forward file descriptor to next directory entry */
			forward_size = f3s_file_forward (part, &ocb->file_desc, reclen);
			if (forward_size < 0)
			{
				return _RESMGR_ERRNO (errno);
			}

			/* update ocb offset in file */
			ocb->ocb.offset += reclen;

			/* update done and buffer size */
			done_size += read_size;
			buffer_size -= read_size;
			buffer = (struct dirent *)((char *)buffer + read_size);
		}

		/* else, there is nothing to read or no space left to read */
		else if (dirent_size && !done_size)
			return _RESMGR_ERRNO (EMSGSIZE);
		else
			/* break of this loop */
			break;
	}

	/* setup iov with length */
	resmgr_endian_context (ctp, _IO_READ, S_IFDIR, 0);
	_IO_SET_READ_NBYTES (ctp, done_size);
	resmgr_msgreply (ctp, &msg->i, done_size);
	return _RESMGR_NOREPLY;
/*}}}2*/
}

int
f3s_statdir (resmgr_context_t * ctp,
			 io_stat_t * msg,
			 f3s_ocb_t * ocb)
{
/*{{{2*/
	int error;
	int32_t offset = 0, desc_offset, nlink = 0, dirent_size, text_size, phys;
	f3s_desc_t dir_desc;
	f3s_head_t head;
	f3s_dirent_t dirent;
	f3s_attr_t *file_attr, *part_attr;
	f3s_part_t *part;

	/* get file attribute pointer */
	file_attr = ocb->ocb.attr;

	f3s_lock_assert((file_attr->attr.lock_tid == pthread_self()) && (file_attr->attr.lock_count > 0));

	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
		return _RESMGR_ERRNO(EBADF);
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);
	
	/* get partition pointer */
	part = file_attr->part;

	/* get a copy of directory descriptor */
	dir_desc = ocb->file_desc;

	/* rewind directory descriptor */
	f3s_desc_rewind (&dir_desc);

	/* synchronize file attributes */
	error = f3s_attr_sync (ocb, file_attr);

	f3s_assert (!error);

	if (error)
		goto part_shortcut;

	/* for all directory entries */
	while (1) {

		/* calculate descriptor offset */
		desc_offset = offset - f3s_desc_curr (&dir_desc);

		/* read from file */
		dirent_size = f3s_dirent_read (part, &dir_desc, desc_offset, &dirent);

		if (dirent_size < 0) {
			error = errno;

			goto part_shortcut;
		}

		/* check if struct size makes no sense */
		if (dirent_size > 0 && dirent.struct_size != sizeof (dirent)) {
			error = f3s_ebadfsys (part->index, __func__, __LINE__);

			f3s_assert (0);

			goto part_shortcut;
		}

		/* check if there was something to read */
		if (dirent_size) {
			/* read basic current extent header and lock the unit */
			text_size = f3s_extptr_basic (part, dirent.first, NULL, &head);

			if (text_size < 0) {
				error = errno;
				goto part_shortcut;
			}

			/* get physical unit index */
			phys = f3s_extptr_phys (part, dirent.first);

			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, dirent.first.logi_unit);

			/* check if sibling is a directory */
			if ((head.status[0] & F3S_EXT_TYPE) == F3S_EXT_DIR)

				nlink++;

			/* calculate size of record with dirent, name length and stat */
			offset +=
				sizeof (dirent) + F3S_NAME_ALIGN (dirent.namelen) +
				sizeof (f3s_stat_t);
		}

		/* else, there was nothing to read */
		else
			/* break of this loop */
			break;
	}

	/* fill up attribute nbytes and nlink fields */
	file_attr->attr.nbytes = offset;
	file_attr->attr.nlink = 2 + nlink;

	/* update attribute times */
	iofunc_time_update (&file_attr->attr);

	/* call iofunc stat */
	error = iofunc_stat (ctp, &file_attr->attr, &msg->o);

  part_shortcut:

	/* check for an error */
	if (error)
		return _RESMGR_ERRNO (error);

	/* return iov with data and length */
	return _RESMGR_PTR (ctp, &msg->o, sizeof (msg->o));
/*}}}2*/
}

int
f3s_dir_lock (resmgr_context_t * ctp,
			  void *reserved,
			  f3s_ocb_t * ocb)
{
/*{{{2*/
	f3s_attr_t *part_attr, *file_attr;

	/* get file attributes pointer */
	file_attr = ocb->ocb.attr;

	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	/* lock file attributes */
	iofunc_attr_lock (&file_attr->attr);

	return EOK;
/*}}}2*/
}

int
f3s_dir_unlock (resmgr_context_t * ctp,
				void *reserved,
				f3s_ocb_t * ocb)
{
/*{{{2*/
	f3s_attr_t *part_attr, *file_attr;

	/* get file attributes pointer */
	file_attr = ocb->ocb.attr;

	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;

	/* unlock file attributes */
	iofunc_attr_unlock (&file_attr->attr);

	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);

	return EOK;
/*}}}2*/
}

int
f3s_file_lock (resmgr_context_t * ctp,
				void *reserved,
				f3s_ocb_t * ocb)
{
/*{{{2*/
	f3s_attr_t *file_attr;

	/* get file attributes pointer */
	file_attr = ocb->ocb.attr;

	/* The partition lock needs to be acquired only in certain cases */
	switch (ctp->msg->type) {
	case _IO_MMAP:
	case _IO_SYNC:
	case _IO_CONNECT:
	case _IO_OPENFD:
	case _IO_FDINFO:
	case _IO_CLOSE:
		/* lock partition attributes */
		iofunc_attr_lock (&file_attr->part_attr->attr);
		break;
	default:
		break;
	}

	/* lock file attributes */
	iofunc_attr_lock (&file_attr->attr);

	return EOK;
/*}}}2*/
}

int
f3s_file_unlock (resmgr_context_t * ctp,
				void *reserved,
				f3s_ocb_t * ocb)
{
/*{{{2*/
	f3s_attr_t *part_attr, *file_attr;

	/* get file attributes pointer */
	file_attr = ocb->ocb.attr;

	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;

	/* unlock file attributes */
	iofunc_attr_unlock (&file_attr->attr);

	if (part_attr->attr.lock_count > 0 &&  part_attr->attr.lock_tid == pthread_self()) {
		/* unlock partition attributes */
		iofunc_attr_unlock (&part_attr->attr);
	}

	return EOK;
/*}}}2*/
}

int
f3s_rawio_lock (resmgr_context_t * ctp,
			  void *reserved,
			  f3s_ocb_t * ocb)
{
/*{{{2*/
	f3s_attr_t *part_attr, *file_attr;

	f3s_assert(ocb != NULL);
	/* get file attributes pointer */
	file_attr = ocb->ocb.attr;

	f3s_assert(file_attr != NULL);
	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;
	
	if (part_attr != NULL)
		/* lock partition attributes */
		iofunc_attr_lock (&part_attr->attr);

	/* lock file attributes */
	iofunc_attr_lock (&file_attr->attr);

	return EOK;
/*}}}2*/
}

int
f3s_rawio_unlock (resmgr_context_t * ctp,
				void *reserved,
				f3s_ocb_t * ocb)
{
/*{{{2*/
	f3s_attr_t *part_attr, *file_attr;

	/* get file attributes pointer */
	file_attr = ocb->ocb.attr;

	/* get partition attributes pointer */
	part_attr = file_attr->part_attr;

	/* unlock file attributes */
	iofunc_attr_unlock (&file_attr->attr);

if (part_attr == NULL)
	return EOK;

	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);

	return EOK;
/*}}}2*/
}
/*}}}1
 */

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/resmgr.c $ $Rev: 580669 $" );
