/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_root_read.c
**
** Description:
**
** This file contains the root read function for the f3s resource manager for
** Neutrino
**
** Ident: $Id: root_read.c 580669 2011-09-01 15:18:50Z builder@qnx.com $
**
*/

/*
** Includes
*/

#include "f3s.h"

/*
** Function: f3s_root_read
*/

int32_t
f3s_root_read (f3s_part_t * part,
			   f3s_dirent_t * dirent,
			   char * name,
			   f3s_stat_t * stat)
{
	f3s_extptr_t	root;
	int32_t			phys;
	int32_t			read_size;
	uint32_t		text_offset;
	int				error;

	/* read fresh root extent pointer and lock the unit */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	read_size = f3s_extptr_fresh (part, part->table.root_extptr, NULL,
	                              &text_offset, NULL, &root);
	/* XXX - Shouldn't we use fail_shortcut here too? */
	if (read_size < 0) return (-1);

	/* get physical unit index */
	phys = f3s_extptr_phys (part, root);

	/* read dirent info for root */
	error = f3s_flash_read (part->array, phys, text_offset, sizeof(*dirent), (uint8_t *)dirent);
	if (error) goto fail_shortcut;

	/* check if dirent structure needs swapping */
	if (part->table.swap_flag) f3s_dirent_swap (dirent);

	/* read name for root */
	error = f3s_flash_read (part->array, phys, text_offset + sizeof(*dirent), F3S_NAME_ALIGN(dirent->namelen), (uint8_t*)name);
	if (error) goto fail_shortcut;

	/* read stat info for root */
	error = f3s_flash_read (part->array, phys,
						text_offset + sizeof (*dirent) +
						F3S_NAME_ALIGN (dirent->namelen), sizeof (*stat),
						(uint8_t *) stat);

	/* check if stat structure needs swapping */
	if (part->table.swap_flag) f3s_stat_swap (stat);

fail_shortcut:
	F3S_UNIT_LEAVE (part, phys);
	F3S_LOGI_LEAVE (part, root.logi_unit);
	if (error) {errno = error; return (-1);}
	return (sizeof(*stat));
}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/root_read.c $ $Rev: 580669 $" );
