/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_root_write.c
**
** Description:
**
** This file contains the root write function for the f3s resource manager for
** Neutrino
**
** Ident: $Id: root_write.c 580669 2011-09-01 15:18:50Z builder@qnx.com $
**
*/

/*
** Includes
*/

#include "f3s.h"

/*
** Function: f3s_root_write
*/

int f3s_root_write (f3s_part_t *part, int32_t phys, f3s_extptr_t first,
                    char *name, int16_t length, uint32_t ctime,
                    uint16_t align_pow2)
{
	f3s_dirent_t *	dirent;
	f3s_stat_t *	stat;
	f3s_map_t *		map;
	int32_t			dirent_size;
	int32_t			err_size;

	/* Allocate stack space for the dirent */
	dirent_size = sizeof(*dirent) + F3S_NAME_ALIGN (length) + sizeof(*stat);
	dirent = alloca (dirent_size);
	if (!dirent) {
		f3s_assert (dirent);
		return (ENOMEM);
	}

	/* Initialize the dirent */
	dirent->struct_size = sizeof(*dirent);
	dirent->moves       = 0;
	dirent->namelen     = min(length, F3S_NAME_MAX);
	dirent->first       = first;

	memcpy ((uint8_t *)(&dirent[1]), name, length);

	/* Initialize the stat structure */
	stat = (f3s_stat_t *)((uint8_t *)(&dirent[1]) + F3S_NAME_ALIGN(length));

	stat->struct_size = sizeof(*stat);
	stat->mode        = S_IFDIR | S_IPERMS;
	stat->uid         = 0;
	stat->gid         = 0;
	stat->ctime       = ctime;
	stat->mtime       = ctime;

	/* Swap the data structures */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) {
		f3s_dirent_swap (dirent);
		f3s_stat_swap (stat);
	}

	/* Force clear the last header bit on the boot index */
	err_size = f3s_head_status (part, &phys, F3S_BOOT_INDEX, ~F3S_EXT_LAST, 0);
	if (err_size < 0) return (errno);

	/* Write the root directory extent */
	err_size = f3s_ext_write (part, &phys, F3S_ROOT_INDEX,
	                          F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t) + sizeof(f3s_unit_logi_t) + sizeof(f3s_boot_t), F3S_SYS_ALIGN),
	                          dirent_size, (uint8_t *)dirent,
	                          ~F3S_EXT_TYPE | F3S_EXT_DIR, align_pow2, 1, 0);
	if (err_size < 0) return (errno);

	/* Update the map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];
	map->head_last  = F3S_ROOT_INDEX;
	map->free_first = F3S_ROOT_INDEX + 1;
	map->text_size += F3S_TEXT_ALIGN(part, dirent_size, F3S_SYS_ALIGN);
	map->free_size -= F3S_TEXT_ALIGN(part, dirent_size, F3S_SYS_ALIGN);

	/* Force clear the last header bit on the root index */
	err_size = f3s_head_status (part, &phys, F3S_ROOT_INDEX, ~F3S_EXT_LAST, 0);
	if (err_size < 0) return (errno);

	/* Write a dummy directory entry into the root */
	err_size = f3s_ext_write (part, &phys, F3S_FIRST_INDEX, 0, 0, NULL,
	                          ~F3S_EXT_TYPE | F3S_EXT_DIR, align_pow2, 1, 0);
	if (err_size < 0) return (errno);

	/* Update the map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];
	map->head_last  = F3S_FIRST_INDEX;
	map->free_first = F3S_FIRST_INDEX + 1;

	return (EOK);
}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/root_write.c $ $Rev: 580669 $" );
