/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

/* This function always releases the unit lock, whether it was put on the
 * spare list or not.  Otherwise we run into potential deadlock situations
 * in the future (when erasing the partition, for example).
 */
void f3s_spare_drop (f3s_part_t *part, int32_t phys, int dirty)
{
	f3s_spare_t *spare_entry;

	/* allocate spare unit entry */
	spare_entry = f3s_malloc (sizeof(*spare_entry));
	if (spare_entry != NULL)
	{
		/* put physical unit in spare list entry */
		spare_entry->phys = phys;
		spare_entry->dirty = dirty; //block will need a refresh
		F3S_UNIT_ENTER(part, phys);
		spare_entry->erase_count = part->array->map_vect[phys].erase_count;
		F3S_UNIT_LEAVE(part, phys);

		/* put entry in spare list */
		F3S_SPARE_ENTER (part);
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		f3s_list_add (&part->table.spare_list, (f3s_list_t *)spare_entry);
		F3S_SPARE_LEAVE (part);
	}
	F3S_UNIT_LEAVE(part, phys);
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/spare_drop.c $ $Rev: 580669 $" );
