/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_sync_pulse.c
**
** Description:
**
** This file contains the sync pulse function for the f3s resource manager
** for Neutrino
**
** Ident: $Id: sync_pulse.c 580669 2011-09-01 15:18:50Z builder@qnx.com $
**
*/

/*
** Includes
*/

#include "f3s.h"

/*
** Function: f3s_sync_pulse
*/

void
f3s_sync_pulse (resmgr_context_t * ctp,
				int code,
				union sigval value,
				f3s_attr_t * part_attr)
{
	int32_t error;
	f3s_share_t *file_share;

	/* lock partition attributes */
	iofunc_attr_lock (&part_attr->attr);

	if (part_attr->part->table.status & F3S_TABLE_NO_ATTACH)
	{
		iofunc_attr_unlock (&part_attr->attr);
		return;
	}
	f3s_assert((part_attr->part->table.status & F3S_TABLE_NO_ATTACH) == 0);

	F3S_TABLE_ENTER(part_attr->part);	// lock so we can look at the sync_code
	if (part_attr->part->sync_code == -1)
	{
		F3S_TABLE_LEAVE(part_attr->part);
		iofunc_attr_unlock(&part_attr->attr);
		return;
	}
	F3S_TABLE_LEAVE(part_attr->part);		// return the table lock to its original state

	/* setup share pointer at beginning of partition share list */
	file_share = (f3s_share_t *) part_attr->share_list.next;

	/* while there is a share in the list */
	while (file_share != (f3s_share_t *) & part_attr->share_list)
	{
		iofunc_attr_lock(&file_share->ocb->ocb.attr->attr);
		/* synchronize file attributes */
		error = f3s_attr_sync (file_share->ocb, file_share->ocb->ocb.attr);
		f3s_assert (!error);

		/* synchronize meta data */
		error = f3s_sync_meta (file_share->ocb, file_share->ocb->ocb.attr);
// mkisel - FAILED ASSERTION - carry on for now
#if 0
		f3s_assert (!error);
#else
		if ((error != EOK) && (f3s.verbose >= 2))
			printf("(devf  t%d::%s:%d) FAILED ASSERTION: f3s_sync_meta() returned %d, errno = %d ... carry on\n",
						pthread_self(), __func__, __LINE__, error, errno);
#endif
		iofunc_attr_unlock(&file_share->ocb->ocb.attr->attr);

		/* go to next list entry */
		file_share = (f3s_share_t *) file_share->list.next;
	}

	/* unlock partition attributes */
	iofunc_attr_unlock (&part_attr->attr);
}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/sync_pulse.c $ $Rev: 580669 $" );
