/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int f3s_table_find (f3s_part_t *part)
{
/*{{{1*/
	f3s_map_t *	map;
	uint8_t **	link_matrix;
	int32_t *	reclaim_vect;
	int32_t		curr_phys;
	//int32_t		spare_phys;
	int			readonly = 0;
	int			i,error;

	/* set initial status in table */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	part->table.status = F3S_DWORD_ERASE;

	/* init sub locks of partition table */
	f3s_lock_init(&part->table.cache_lock);
	f3s_lock_init(&part->table.alloc_lock);

	/* init spare_lock with recursive */
	f3s_lock_init(&part->table.spare_lock);

	/* intitialize total space counters */
	part->table.head_num   = 0;
	part->table.dirent_num = 0;

	part->table.align_size = 0;
	part->table.align_pow2 = 0;

	part->table.free_total  = 0;
	part->table.stale_total = 0;
	part->table.pad_total   = 0;
	part->table.resv_total  = 0;

	/* initialize spare list information */
	f3s_list_init (&part->table.spare_list);

	/* try to recover boot info extent */
	error = f3s_recover_boot (part);

	if (error)
	{
		/* log raw partition */
		f3s_log_raw (part->array->index, part->index, part->unit_num);
		return (error);
	}else if ((f3s.enum_only != -1) && (f3s.enum_only <= part->index)){
		/* Check if we want to mount the filesystem, or if we are just
		 * enumerating.  Make sure we aren't trying to mount externally.
		 */
		if (!f3s.force_mount)
			return (-1);

		if (part->unit_num == 0){
			/* this should not happen, and is a fatal error for this partition. */
			return (EBADFSYS);
		}
	}

	/* clear initial values for copy vector */
	part->table.copy_max  = 0;
	part->table.copy_vect = NULL;

	/* clear initial values for obsolete vector */
	part->table.obsol_max  = 0;
	part->table.obsol_vect = NULL;

	/* clear initial values for text buffer */
	part->table.text_size   = 0;
	part->table.text_buffer = NULL;

	/* allocate space for table vectors */
	part->table.logi_vect =
		f3s_calloc ((part->unit_num + F3S_FIRST_LOGI) * sizeof(*part->table.logi_vect));

	part->table.logi_lock_vect =
		f3s_calloc ((part->unit_num + F3S_FIRST_LOGI) * sizeof(f3s_lock_t));

	/* initialize logi_lock_vect with recursive */
	for(i = 0; i < (part->unit_num + F3S_FIRST_LOGI); i++) 
	{
		f3s_lock_init(&part->table.logi_lock_vect[i]);
	}

	part->table.sort_vect =
		f3s_calloc (F3S_UNIT_ACTIVE(part) * sizeof(f3s_sort_t));

	/* allocate space for partition vectors */
	part->table.cache_vect =
		f3s_calloc (F3S_CACHE_MAX * sizeof(f3s_cache_t));

	if (!(part->table.logi_vect) || !(part->table.sort_vect) ||
		!(part->table.cache_vect) || !(part->table.logi_lock_vect))
	{
		error = ENOMEM;
		goto error;
	}

	/* try to recover from unfinished reclaims */
	error = f3s_recover_reclaim (part);
	if (error)
	{
		if (error == EROFS) readonly = 1;
		else                goto error;
	}

	/* allocate space for reclaim vector */
	reclaim_vect = f3s_calloc ((part->unit_first + part->unit_num) * sizeof(*reclaim_vect));
	if (reclaim_vect == NULL) {error = ENOMEM; goto error;}

	/* try to recover from unfinished writes */
	error = f3s_recover_write (part, reclaim_vect);
	if (error)
	{
		if (error == EROFS) readonly = 1;
		else                goto error;
	}
	/* Due to recover_write() using the facility of head_fix() to mask
	 * any extent errors, The reclaim_vect will be empty, so skip this
	 * code
	 */

#if 0
	/* reclaim loop for unfinished writes */
	for (curr_phys = part->unit_first;
		 curr_phys < (part->unit_first + part->unit_num); curr_phys++)
	{
		F3S_UNIT_ENTER(part, curr_phys);

		/* get unit map pointer */
		f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[curr_phys].thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];

		/* set spare phys equal to curr phys just to leave it right */
		spare_phys = curr_phys;
		/* check if current unit needs reclaim */
		if (reclaim_vect[curr_phys])
		{
			error = f3s_head_repair (part, &spare_phys, 0, NULL);
			if (error)
			{
				fprintf (stderr, "(devf  t%d::%s:%d) head repair failure on phys %d\n",
							pthread_self(), __func__, __LINE__, spare_phys);

				/* This is bad... really bad... */
				readonly = 1;
				goto reclaim_done;
			}

			/* This is a new spare, so don't need to repair */
			reclaim_vect[curr_phys] = 0;

			/* This is the newly repaired unit, so don't need to repair */
			reclaim_vect[spare_phys] = 0;
		}
		F3S_UNIT_LEAVE(part, spare_phys);
	}
reclaim_done:
#endif

	/* allocate space for link matrix unit vector */
	link_matrix = f3s_calloc ((part->unit_first + part->unit_num) * sizeof(*link_matrix));
	if (link_matrix == NULL) 
	{
		error = ENOMEM;

	} else
	{
		/* allocate loop for link matrix vectors */
		for (curr_phys = part->unit_first;
			 curr_phys < (part->unit_first + part->unit_num);
			 curr_phys++)
		{
			F3S_UNIT_ENTER(part, curr_phys);

			/* get unit map pointer */
			f3s_lock_assert((part->table_lock.thread == pthread_self()));
			map = &part->array->map_vect[curr_phys];

			/* allocate space for link matrix head vector */
			link_matrix[curr_phys] =
				f3s_calloc ((map->head_last + 1) * sizeof(**link_matrix));
			if (!link_matrix[curr_phys]) error = ENOMEM;

			F3S_UNIT_LEAVE(part, curr_phys);
		}

		/* try to recover from unfinished links */
		if (!error) error = f3s_recover_link (part, link_matrix);
	}

	/* try to recover from unfinished deletes */
	if (!error) error = f3s_recover_delete (part, link_matrix);

	/* for all units in partition */
	for (curr_phys = part->unit_first;
		 curr_phys < (part->unit_first + part->unit_num); curr_phys++)
	{
		F3S_UNIT_ENTER(part, curr_phys);

		/* get unit map pointer */
		f3s_lock_assert((part->table_lock.thread == pthread_self()));
		map = &part->array->map_vect[curr_phys];

		/* free space for link matrix head vector */
		f3s_free (link_matrix[curr_phys],
		          (map->head_last + 1) * sizeof(**link_matrix));
		link_matrix[curr_phys] = NULL;
		
		F3S_UNIT_LEAVE(part, curr_phys);
	}

	/* free space for link matrix unit vector */
	f3s_free (link_matrix,
	          (part->unit_first + part->unit_num) * sizeof(*link_matrix));
	link_matrix = NULL;

	/* free space for reclaim vector */
	f3s_free (reclaim_vect,
	          (part->unit_first + part->unit_num) * sizeof(*reclaim_vect));
	reclaim_vect = NULL;

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* check if readonly in necessary */
	if (readonly) return (EROFS);
	return (error);
error:
	/* log corrupted file system */
	f3s_log_corrupt (part->array->index, part->index, part->unit_num);

	/* drain the spare unit list.	*/
	f3s_list_drain (&part->table.spare_list, sizeof(f3s_spare_t) - sizeof(f3s_list_t));

	/* free space for table vectors */
	f3s_free (part->table.logi_vect,
	          (part->unit_num + F3S_FIRST_LOGI) * sizeof(*part->table.logi_vect));
	part->table.logi_vect = NULL;
	f3s_free (part->table.sort_vect,
	          F3S_UNIT_ACTIVE(part) * sizeof(f3s_sort_t));
	part->table.sort_vect = NULL;
	f3s_free (part->table.cache_vect,
	          F3S_CACHE_MAX * sizeof(f3s_cache_t));
	part->table.cache_vect = NULL;

	return (error);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/table_find.c $ $Rev: 580669 $" );
