/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include "proto.h"

typedef struct lock_data
{	int tid, pid, rcvid;
} lock_data_t;

typedef struct list_data
{	lock_data_t data;
	struct list_data * next;
} list_data_t;

static list_data_t * head = NULL;

static int fifo_add(lock_data_t * info)
{	list_data_t * element = (list_data_t *)calloc(1, sizeof(list_data_t));

	if(element != NULL) {
		element->data.tid = info->tid;
		element->data.pid = info->pid;
		element->data.rcvid = info->rcvid;

		if(head == NULL) {
			head = element;
			return 0;
		}
		else {
			list_data_t * p;
	
			for(p = head; p->next != NULL; p = p->next)
				;

			p->next = element;
			return 0;
		}
	}

	return -1;
}

static void fifo_purge(lock_data_t * info)
{   if(head != NULL) {
        list_data_t * p = head;
        list_data_t * prev = head;

        do {
            if((info->tid == p->data.tid ) && (info->pid == p->data.pid)) {
                if(p == head) {
                    head = head->next;
                }
                else {
                    prev->next = p->next;
                }

                free(p);
                break;
            }

            prev = p;
        }
        while((p = p->next) != NULL);
    }
}

static int fifo_get(lock_data_t * info)
{	if(head != NULL) {
		list_data_t * p = head;

		head = head->next;

		info->tid = p->data.tid;
		info->pid = p->data.pid;
		info->rcvid = p->data.rcvid;

		free(p);

		return 0;
	}

	return -1;
}

static int tid = -1, pid = -1;

static int lock_resmgr(lock_data_t * info)
{	if((tid != -1) && (pid != -1))
		return EBUSY;

	tid = info->tid;
	pid = info->pid;

	return EOK;
}

static int unlock_resmgr(lock_data_t * info)
{	if((tid == -1) || (pid == -1))
		return EINVAL;

	tid = -1;
	pid = -1;

	return EOK;
}

int
_i2c_devctl(resmgr_context_t *ctp, io_devctl_t *msg, i2c_ocb_t *ocb)
{
    i2c_dev_t       *dev = ocb->hdr.attr;
    union {
        i2c_addr_t      addr;
        _Uint32t        bus_speed;
    } *idata;
    i2c_driver_info_t   *driver_info;
    int             status;
    int             nbytes;
    int             info;
	int				rr;

	status = iofunc_devctl_default(ctp, msg, &ocb->hdr);
	if (status != _RESMGR_DEFAULT)
		return status;

    switch (msg->i.dcmd) {
    case DCMD_I2C_SET_SLAVE_ADDR: 
        if (msg->i.nbytes < sizeof(i2c_addr_t))
            return EINVAL;

        idata = _DEVCTL_DATA(msg->i);
        ocb->slave = idata->addr;
        
        memset(&msg->o, 0, sizeof(msg->o));
        msg->o.ret_val = 0;
        msg->o.nbytes = 0;
        return _RESMGR_PTR(ctp, msg, sizeof(msg->o));

    case DCMD_I2C_GET_BUS_SPEED:
        if (msg->i.nbytes < sizeof(_Uint32t))
            return EINVAL;

        idata = _DEVCTL_DATA(msg->o);
        idata->bus_speed = ocb->bus_speed;
        msg->o.ret_val = 0;
        msg->o.nbytes = sizeof(_Uint32t);
        return _RESMGR_PTR(ctp, msg, sizeof(msg->o) + sizeof(_Uint32t));

    case DCMD_I2C_SET_BUS_SPEED:
        if (msg->i.nbytes < sizeof(_Uint32t))
            return EINVAL;

        idata = _DEVCTL_DATA(msg->i);
        ocb->bus_speed = idata->bus_speed;

        memset(&msg->o, 0, sizeof(msg->o));
        msg->o.ret_val = 0;
        msg->o.nbytes = 0;
        return _RESMGR_PTR(ctp, msg, sizeof(msg->o));

    case DCMD_I2C_MASTER_SEND:
    case DCMD_I2C_SEND:
        return _i2c_master_send(ctp, msg, ocb);

    case DCMD_I2C_MASTER_RECV:
    case DCMD_I2C_RECV:
        return _i2c_master_recv(ctp, msg, ocb);

    case DCMD_I2C_SENDRECV:
        return _i2c_master_sendrecv(ctp, msg, ocb);

    case DCMD_I2C_LOCK:
		if((rr = iofunc_attr_lock((iofunc_attr_t *)ocb->hdr.attr)) == EOK) {
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = ctp->rcvid;

			if(lock_resmgr(&info) == EBUSY) {
				if(fifo_add(&info) == -1) {
					iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
					return ENOMEM;
				}

				iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
				return _RESMGR_NOREPLY;
			}

			iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
		}
		return rr;

    case DCMD_I2C_UNLOCK:
		if((rr = iofunc_attr_lock((iofunc_attr_t *)ocb->hdr.attr)) == EOK) {
			int r;
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = -1;

			if((r = unlock_resmgr(&info)) != EOK) {
				iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
				return r;
			}

			if(fifo_get(&info) != -1) {
				lock_resmgr(&info);

				/* A lock should have occurred, free the client. */
				MsgReply(info.rcvid, EOK, NULL, 0);
			}

			iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
		}
       	return rr;

    case DCMD_I2C_DRIVER_INFO:
        driver_info = _DEVCTL_DATA(msg->o);
        if (-1 == dev->mfuncs.driver_info(dev->hdl, driver_info))
            return EIO;
        memset(&msg->o, 0, sizeof(msg->o));
        msg->o.ret_val = 0;
        msg->o.nbytes = sizeof(*driver_info);
        return _RESMGR_PTR(ctp, msg, sizeof(msg->o) + sizeof(*driver_info));

    default:
        status = dev->mfuncs.ctl(dev->hdl, msg->i.dcmd, _DEVCTL_DATA(msg->i), 
                         msg->i.nbytes, &nbytes, &info);
        if (status != EOK)
            return status;

        memset(&msg->o, 0, sizeof(msg->o));
        msg->o.ret_val = info;
        msg->o.nbytes = nbytes;
        return _RESMGR_PTR(ctp, msg, sizeof(msg->o) + nbytes);
    }

    return ENOSYS;
}

int _i2c_io_unblock(resmgr_context_t *ctp, io_pulse_t *msg, RESMGR_OCB_T *ocb)
{	int rr;

	if((rr = iofunc_unblock_default(ctp, msg, (iofunc_ocb_t *)ocb)) != _RESMGR_DEFAULT)
		return(rr);

	if(iofunc_attr_lock((iofunc_attr_t *)ocb->hdr.attr) == EOK) {
		if((tid == ctp->info.tid) && (pid == ctp->info.pid)) {
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = -1;

			if(unlock_resmgr(&info) == EOK) {
				if(fifo_get(&info) != -1) {
					lock_resmgr(&info);

					/* A lock should have occurred, free the client. */
					MsgReply(info.rcvid, EOK, NULL, 0);
				}
			}
		}
		else {
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = -1;

			fifo_purge(&info);
		}

		iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
	}

	return(EOK);
}

int _i2c_io_unblock_connect(resmgr_context_t *ctp, io_pulse_t *msg, RESMGR_HANDLE_T *handle, void * reserved)
{	int rr;

	if((rr = iofunc_unblock_default(ctp, msg, (iofunc_ocb_t *)handle)) != _RESMGR_DEFAULT)
		return(rr);

	if(iofunc_attr_lock((iofunc_attr_t *)((RESMGR_OCB_T *)handle)->hdr.attr) == EOK) {
		if((tid == ctp->info.tid) && (pid == ctp->info.pid)) {
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = -1;

			if(unlock_resmgr(&info) == EOK) {
				if(fifo_get(&info) != -1) {
					lock_resmgr(&info);

					/* A lock should have occurred, free the client. */
					MsgReply(info.rcvid, EOK, NULL, 0);
				}
			}
		}
		else {
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = -1;

			fifo_purge(&info);
		}

		iofunc_attr_unlock((iofunc_attr_t *)((RESMGR_OCB_T *)handle)->hdr.attr);
	}

	return(EOK);
}

int _i2c_io_close(resmgr_context_t *ctp, void *rsrv, RESMGR_OCB_T *ocb)
{   
if(iofunc_attr_lock((iofunc_attr_t *)ocb->hdr.attr) == EOK) {
		if((tid == ctp->info.tid) && (pid == ctp->info.pid)) {
			lock_data_t info;

			info.tid = ctp->info.tid;
			info.pid = ctp->info.pid;
			info.rcvid = -1;

			if(unlock_resmgr(&info) == EOK) {
				if(fifo_get(&info) != -1) {
					lock_resmgr(&info);

					/* A lock should have occurred, free the client. */
					MsgReply(info.rcvid, EOK, NULL, 0);
				}
			}
		}

		iofunc_attr_unlock((iofunc_attr_t *)ocb->hdr.attr);
	}

    return(iofunc_close_ocb_default(ctp, rsrv, (iofunc_ocb_t *)ocb));
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/i2c/master/devctl.c $ $Rev: 580669 $" );
