/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <stdio.h>
#include <errno.h>
#include "proto.h"


static void *
stub_init(int argc, char *argv[])
{
    return NULL;
}

static void
stub_fini(void *hdl)
{
}

static i2c_status_t
stub_status(void *hdl, void *buf, unsigned len, unsigned stop)
{
    return I2C_STATUS_DONE | I2C_STATUS_ERROR;
}

static int
stub_driver_info(void *hdl, i2c_driver_info_t *info)
{
    errno = ENOSYS;
    return -1;
}

static int
stub_version_info(i2c_libversion_t *version)
{
    errno = ENOSYS;
    return -1;
}

static int
stub_abort(void *hdl, int rcvid)
{
    errno = ENOSYS;
    return -1;
}

static int
stub_ctl(void *hdl, int cmd, void *msg, int msglen, int *nbytes, int *info)
{
    return ENOSYS;
}

static int
stub_set_bus_speed(void *hdl, unsigned speed, unsigned *ospeed)
{
    errno = ENOSYS;
    return -1;
}

static int
stub_set_slave_addr(void *hdl, unsigned addr, i2c_addrfmt_t fmt)
{
    errno = ENOSYS;
    return -1;
}

static void
set_default_master_funcs(i2c_master_funcs_t *funcs)
{
    int tabsize = sizeof(i2c_master_funcs_t);

    funcs->size = tabsize;

    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            version_info, stub_version_info, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            driver_info, stub_driver_info, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            init, stub_init, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            fini, stub_fini, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            recv, stub_status, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            send, stub_status, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            abort, stub_abort, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            ctl, stub_ctl, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            set_bus_speed, stub_set_bus_speed, tabsize);
    I2C_ADD_FUNC(i2c_master_funcs_t, funcs,
            set_slave_addr, stub_set_slave_addr, tabsize);
}

int
_i2c_initlib(i2c_dev_t *dev)
{
    i2c_libversion_t    version;

    set_default_master_funcs(&dev->mfuncs);
    if (-1 == i2c_master_getfuncs(&dev->mfuncs, sizeof(i2c_master_funcs_t)))
        return -1;

    if (-1 == dev->mfuncs.version_info(&version)) {
        fprintf(stderr, "Warning: No version info\n");
    }

    if ((version.major != I2CLIB_VERSION_MAJOR) ||
        (version.minor > I2CLIB_VERSION_MINOR))
    {
        fprintf(stderr,
                "Version mismatch: Library = v%d.%d, Driver = v%d.%d\n",
                I2CLIB_VERSION_MAJOR, I2CLIB_VERSION_MINOR, 
                version.major, version.minor);
        return -1;
    }
    return 0;
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/i2c/master/lib.c $ $Rev: 580669 $" );
