/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <atomic.h>
#include <sys/procmgr.h>
#include "proto.h"

static volatile unsigned done = 0;

static void
exit_signal(int signo)
{
    atomic_set(&done, 1);
    return;
}

int
_i2c_main(i2c_dev_t *dev, i2c_options_t *devinit)
{
    resmgr_connect_funcs_t  connect_funcs;
    resmgr_io_funcs_t   io_funcs;
    resmgr_attr_t       rattr;
    iofunc_funcs_t      ocb_funcs = { _IOFUNC_NFUNCS,
                                      _i2c_ocb_calloc,
                                      _i2c_ocb_free };
    iofunc_mount_t      mount = { 0, 0, 0, 0, &ocb_funcs };
    char                name[PATH_MAX];
    struct sigaction    sa;

    if (!dev)
        return -1;

    if (NULL == (dev->dpp = dispatch_create())) {
        perror("dispatch_create() failed");
        return -1;
    }

    memset(&rattr, 0, sizeof(rattr));
    rattr.nparts_max = devinit->nparts_max;
    rattr.msg_max_size = devinit->msg_max_size;

    iofunc_func_init(_RESMGR_CONNECT_NFUNCS, &connect_funcs,
                     _RESMGR_IO_NFUNCS, &io_funcs);
    io_funcs.devctl = _i2c_devctl;
	io_funcs.unblock = _i2c_io_unblock;
	io_funcs.close_ocb = _i2c_io_close;

	connect_funcs.unblock = _i2c_io_unblock_connect;

    iofunc_attr_init(&dev->hdr, S_IFCHR | 0666, NULL, NULL);
    dev->hdr.mount = &mount;

    snprintf(name, PATH_MAX, "/dev/i2c%d", devinit->unit);
    if (-1 == (dev->id = 
               resmgr_attach(dev->dpp, &rattr, name, _FTYPE_ANY, 0,
                             &connect_funcs, &io_funcs, dev))) {
        perror("resmgr_attach() failed");
        return -1;
    }

    if (NULL == (dev->ctp = resmgr_context_alloc(dev->dpp))) {
        perror("resmgr_context_alloc() failed");
        return -1;
    }

    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGTERM);
    sa.sa_handler = exit_signal;
    sa.sa_flags = 0;
    sigaction(SIGTERM, &sa, NULL);

    procmgr_daemon(0, PROCMGR_DAEMON_NOCLOSE|PROCMGR_DAEMON_NODEVNULL);

    while (!done) {
        if (dev->ctp == resmgr_block(dev->ctp))
            resmgr_handler(dev->ctp);
        else if (errno != EFAULT)
            atomic_set(&done, 1);
    }

    return 0;
}

int
main(int argc, char *argv[])
{
    int                 status;
    i2c_dev_t           dev;
    i2c_options_t       devinit;
    int                 new_argc;
    char controller_str[]="controller=xxx";
    const int nums= hwi_find_num_units(HWI_ITEM_BUS_I2C);
   
    memset(&dev, 0, sizeof(dev));
    memset(&devinit, 0, sizeof(devinit));

    devinit.controller_num = -1;
    new_argc = _i2c_options(&devinit, argc, argv);

    if(devinit.controller_num >= 0 && nums>0 )
    {
          if(devinit.controller_num < nums){
	        assert(new_argc < argc);
	     	snprintf(controller_str, sizeof(controller_str), "controller=%u", devinit.controller_num);
	        argv[new_argc++]=controller_str;
	   }
	   else 
	   {
		printf("Invalid I2C device index number\n");
		return -1;
		}
    }

    if (-1 == new_argc)
        return -1;
    argc = new_argc;
    argv[argc] = NULL;

    if (-1 == _i2c_initlib(&dev))
        return -1;


    dev.hdl = dev.mfuncs.init(argc, argv);
    if (NULL == dev.hdl) {
        return -1;
    }

    if (-1 == 
        dev.mfuncs.set_bus_speed(dev.hdl, devinit.default_bus_speed, NULL)) {
        fprintf(stderr, "Bad bus speed: %d: %s\n", devinit.default_bus_speed, 
                   strerror(errno));
        dev.mfuncs.fini(dev.hdl);
        return -1;
    }
    dev.bus_speed = devinit.default_bus_speed;

    status = _i2c_main(&dev, &devinit);

    dev.mfuncs.fini(dev.hdl);

    return status;
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/i2c/master/main.c $ $Rev: 580669 $" );
