/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <inttypes.h>
#include "proto.h"

union msgs {
    i2c_masterhdr_t     a;
    i2c_driver_info_t   b;
};

int
_i2c_options(i2c_options_t *opts, int argc, char *argv[])
{
    int         i;
    const char  *p;
    const char  *v;
    int         new_argc = 1;
    const char  *ptr;

    /* defaults */
    opts->unit = 0;
    opts->msg_max_size = 128;
    opts->nparts_max = 2;
    opts->default_bus_speed = 100000;

    for (i = 1; i < argc; ++i) {
        if ((argv[i][0] != '-') || (argv[i][1] != '-') || (argv[i][2] == '\0')) {
            ++new_argc;
            continue;
        }

        p = argv[i] + 2;
        if (*p == '\0') {
            ++new_argc;
            continue;
        }

        argv[new_argc] = argv[i + 1];

        v = p + 1;
        if (*v == '\0') {
            if (++i < argc) {
                v = argv[i];
                if (*v == '-')
                    v = NULL;
            } else
                v = NULL;
        }

        if (v)
            argv[new_argc] = argv[i + 1];

        switch (*p) {
        case 'b':
            if (!v) 
                goto missing_argument;
            opts->default_bus_speed = strtoul(v, NULL, 0);
            break;

        case 'm':
            if (!v) 
                goto missing_argument;
            opts->msg_max_size = strtoul(v, NULL, 0);
            break;

        case 'n':
            if (!v) 
                goto missing_argument;
            opts->nparts_max = strtoimax(v, NULL, 0);
            break;

        case 'u':
            if (!v) 
                goto missing_argument;
            if((ptr=strchr(v , ',')) !=NULL) {
            	opts->controller_num = strtoul(ptr+1 , NULL , 0);
            }
            opts->unit = strtoimax(v, NULL, 0);
            break;

        default:
            goto unrecognized_option;
        }
    }

    if (opts->msg_max_size < (sizeof(io_devctl_t) + sizeof(union msgs))) {
        opts->msg_max_size = 
            sizeof(io_devctl_t) + sizeof(i2c_masterhdr_t) + 4;
    }

    if (opts->nparts_max < 2) {
        opts->nparts_max = 2;
    }

    optind = 1;
    return new_argc;

unrecognized_option:
    fprintf(stderr, "%s: Unrecognized option '--%c'\n", argv[0], p[0]);
    return -1;

missing_argument:
    fprintf(stderr, "%s: Missing argument for '--%c'\n", argv[0], p[0]);
    return -1;
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/i2c/master/options.c $ $Rev: 580669 $" );

