/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include "proto.h"

int
_i2c_master_recv(resmgr_context_t *ctp, io_devctl_t *msg, i2c_ocb_t *ocb)
{
    i2c_dev_t           *dev = ocb->hdr.attr;
    union {
        i2c_masterhdr_t     i;
        i2c_recv_t          x;
    } *hdr;
    i2c_addr_t          *slave;
    void                *buf0;
    unsigned            offset;
    int                 nparts = 1;
    unsigned int        len, stop;
    i2c_status_t        mstatus;

    hdr = _DEVCTL_DATA(msg->i);

    if (msg->i.dcmd == DCMD_I2C_MASTER_RECV) {
        if (msg->i.nbytes < sizeof(hdr->i)) {
            _i2c_slogf("Incomplete receive hdr");
            return EINVAL;
        }

        offset = 0;
        len = hdr->i.len;
        stop = hdr->i.stop;
        slave = &ocb->slave;
    } else {
        if (msg->i.nbytes < sizeof(hdr->x)) {
            _i2c_slogf("Incomplete receive hdr");
            return EINVAL;
        }

        offset = sizeof(hdr->x);
        len = hdr->x.len;
        stop = hdr->x.stop;
        slave = &hdr->x.slave;
    }

    if (len <= 0) {
        memset(&msg->o, 0, sizeof(msg->o));
        return _RESMGR_PTR(ctp, msg, sizeof(msg->o) + offset);
    }

    if ((offset + len) > msg->i.nbytes) {
        _i2c_slogf("Receive buffer too short");
        return EINVAL;
    }

    if ((sizeof(msg->o) + offset + len) > ctp->msg_max_size) {
        if (len > dev->buflen) {
            dev->buflen = len;
            free(dev->buf);
            if (NULL == (dev->buf = malloc(dev->buflen))) {
                dev->buflen = 0;
                return ENOMEM;
            }
        }
        ++nparts;
        SETIOV(&ctp->iov[0], &msg->o, sizeof(msg->o) + offset);
        SETIOV(&ctp->iov[1], dev->buf, len);
        buf0 = dev->buf;
    } else {
        SETIOV(&ctp->iov[0], &msg->o, sizeof(msg->o) + offset + len);
        buf0 = _DEVCTL_DATA(msg->o) + offset;
    }

    if (dev->bus_speed != ocb->bus_speed) {
        if (-1 == dev->mfuncs.set_bus_speed(dev->hdl, ocb->bus_speed, NULL)) {
            _i2c_slogf("Bad bus speed: %d: %s", ocb->bus_speed, 
                       strerror(errno));
            return EIO;
        }
        dev->bus_speed = ocb->bus_speed;
    }

    if (-1 == dev->mfuncs.set_slave_addr(dev->hdl, slave->addr, slave->fmt)) {
        _i2c_slogf("Bad slave address: %x (%s): %s", slave->addr,
                   (slave->fmt == I2C_ADDRFMT_7BIT)? "7-bit": "10-bit",
                   strerror(errno));
        return EIO;
    }
    
    mstatus = dev->mfuncs.recv(dev->hdl, buf0, len, stop);
    if (mstatus == I2C_STATUS_DONE) {
        dev->hdr.flags |= IOFUNC_ATTR_ATIME;
        memset(&msg->o, 0, sizeof(msg->o));
        msg->o.ret_val = len;
        msg->o.nbytes = offset + len;
        return _RESMGR_NPARTS(nparts);
    }

    if (!(mstatus & I2C_STATUS_DONE)) {
        _i2c_slogf("Master receive did not terminate");
    }
    if (mstatus & I2C_STATUS_NACK) {
        _i2c_slogf("Master receive NACK");
    }
    if (mstatus & I2C_STATUS_ARBL) {
        _i2c_slogf("Master receive ARBL");
    }
    if (mstatus & I2C_STATUS_BUSY) {
        _i2c_slogf("Master receive BUSY");
    }
    return EIO;
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/i2c/master/recv.c $ $Rev: 580669 $" );
