/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include "proto.h"

int
_i2c_master_sendrecv(resmgr_context_t *ctp, io_devctl_t *msg, i2c_ocb_t *ocb)
{
    i2c_dev_t           *dev = ocb->hdr.attr;
    i2c_sendrecv_t      *hdr;
    void                *buf0, *buf1;
    i2c_status_t        mstatus;
    unsigned            maxbuflen;

    if (msg->i.nbytes < sizeof(*hdr)) {
        _i2c_slogf("Incomplete sendrecv hdr");
        return EINVAL;
    }

    hdr = _DEVCTL_DATA(msg->i);

    if ((hdr->send_len < 0) || (hdr->recv_len < 0)) {
        _i2c_slogf("Negative send or recv length");
        return EINVAL;
    }

    if (hdr->send_len < hdr->recv_len)
        maxbuflen = hdr->recv_len;
    else
        maxbuflen = hdr->send_len;

    if (maxbuflen == 0) {
        memset(&msg->o, 0, sizeof(msg->o));
        return _RESMGR_PTR(ctp, msg, sizeof(*msg) + sizeof(*hdr));
    }

    if ((sizeof(*hdr) + hdr->send_len) > msg->i.nbytes) {
        _i2c_slogf("Incomplete send data");
        return EINVAL;
    }

    if ((sizeof(*hdr) + hdr->recv_len) > msg->i.nbytes) {
        _i2c_slogf("Receive buffer too short");
        return EINVAL;
    }

    if ((sizeof(*msg) + sizeof(*hdr) + maxbuflen) > ctp->msg_max_size) {
        if (maxbuflen > dev->buflen) {
            free(dev->buf);
            if (NULL == (dev->buf = malloc(maxbuflen))) {
                dev->buflen = 0;
                return ENOMEM;
            }
            dev->buflen = maxbuflen;
        }
    }

    if ((sizeof(*msg) + sizeof(*hdr) + hdr->send_len) > ctp->msg_max_size) {
        int status;
        status = resmgr_msgread(ctp, dev->buf, hdr->send_len, 
                                sizeof(*msg) + sizeof(*hdr));
        if (status < 0)
            return errno;
        if (status < hdr->send_len)
            return EFAULT;

        buf0 = dev->buf;
    } else {
        buf0 = (void *)(hdr + 1);
    }

    if ((sizeof(*msg) + sizeof(*hdr) + hdr->recv_len) > ctp->msg_max_size) {
        buf1 = dev->buf;
    } else {
        buf1 = (void *)(hdr + 1);
    }
    SETIOV(&ctp->iov[0], msg, sizeof(*msg) + sizeof(*hdr));
    SETIOV(&ctp->iov[1], buf1, hdr->recv_len);

    if (dev->bus_speed != ocb->bus_speed) {
        if (-1 == dev->mfuncs.set_bus_speed(dev->hdl, ocb->bus_speed, NULL)) {
            _i2c_slogf("Bad bus speed: %d: %s", ocb->bus_speed, 
                       strerror(errno));
            return EIO;
        }
        dev->bus_speed = ocb->bus_speed;
    }

    if (-1 == 
        dev->mfuncs.set_slave_addr(dev->hdl, hdr->slave.addr, hdr->slave.fmt))
    {
        _i2c_slogf("Bad slave address: %x (%s): %s", hdr->slave.addr,
                   (hdr->slave.fmt == I2C_ADDRFMT_7BIT)? "7-bit": "10-bit",
                   strerror(errno));
        return EIO;
    }

    if (hdr->send_len > 0) {
        mstatus = dev->mfuncs.send(dev->hdl, buf0, hdr->send_len, 0);
        if (mstatus != I2C_STATUS_DONE) {
            if (!(mstatus & I2C_STATUS_DONE)) {
                _i2c_slogf("Master send did not terminate");
            }
            if (mstatus & I2C_STATUS_NACK) {
                _i2c_slogf("Master send NACK");
            }
            if (mstatus & I2C_STATUS_ARBL) {
                _i2c_slogf("Master send ARBL");
            }
            if (mstatus & I2C_STATUS_BUSY) {
                _i2c_slogf("Master send BUSY");
            }
            return EIO;
        }
        dev->hdr.flags |= IOFUNC_ATTR_MTIME | IOFUNC_ATTR_CTIME;
    }

    if (hdr->recv_len) {
        mstatus = dev->mfuncs.recv(dev->hdl, buf1, hdr->recv_len, hdr->stop);
        if (mstatus != I2C_STATUS_DONE) {
            if (!(mstatus & I2C_STATUS_DONE)) {
                _i2c_slogf("Master receive did not terminate");
            }
            if (mstatus & I2C_STATUS_NACK) {
                _i2c_slogf("Master receive NACK");
            }
            if (mstatus & I2C_STATUS_ARBL) {
                _i2c_slogf("Master receive ARBL");
            }
            if (mstatus & I2C_STATUS_BUSY) {
                _i2c_slogf("Master receive BUSY");
            }
            return EIO;
        }
        dev->hdr.flags |= IOFUNC_ATTR_ATIME;
    }

    memset(&msg->o, 0, sizeof(msg->o));
    msg->o.ret_val = hdr->recv_len;
    msg->o.nbytes = sizeof(*hdr) + hdr->recv_len;
    return _RESMGR_NPARTS(2);
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/i2c/master/sendrecv.c $ $Rev: 580669 $" );
