/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <sys/mman.h>
#include <hw/spi-master.h>


static inline int spi_dma_xfer(int fd, uint32_t device, void *paddr, int len)
{
	iov_t		iov[2];
	spi_msg_t	msg;

	memset((char *)&msg, 0, sizeof(msg));

	msg.msg_hdr.i.type        = _IO_MSG;
	msg.msg_hdr.i.subtype     = _SPI_IOMSG_DMAXCHANGE;
	msg.msg_hdr.i.mgrid       = _IOMGR_SPI;
	msg.msg_hdr.i.combine_len = sizeof(spi_msg_t) + sizeof(spi_dma_paddr_t);

	msg.device                = device;
	msg.xlen                  = len;

	SETIOV(iov + 0, &msg, sizeof(spi_msg_t));
	SETIOV(iov + 1, paddr, sizeof(spi_dma_paddr_t));

	return (MsgSendv(fd, iov, 2, 0, 0));
}


/*
 * DMA exchange for specific device.
 * Valid flags : SPI_DEV_LOCK / SPI_DEV_UNLOCK
 */
int spi_dma_xchange(int fd, uint32_t device, void *wbuf, void *rbuf, int len)
{
	spi_dma_paddr_t	paddr;

	paddr.rpaddr = 0;
	paddr.wpaddr = 0;

	if (wbuf) {
		if (mem_offset64(wbuf, NOFD, 1, (off64_t *) &paddr.wpaddr, 0) == -1) {
			errno = EINVAL;
			return -1;
		}

		if (rbuf) {
			if (mem_offset64(rbuf, NOFD, 1, (off64_t *) &paddr.rpaddr, 0) == -1) {
				errno = EINVAL;
				return -1;
			}
		}
	}
	else if (rbuf) {
		if (mem_offset64(rbuf, NOFD, 1, (off64_t *) &paddr.rpaddr, 0) == -1) {
			errno = EINVAL;
			return -1;
		}
	}
	else {
		errno = EINVAL;
		return -1;
	}

	return spi_dma_xfer(fd, device, &paddr, len);
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/spi-master/spi_dma_xchange.c $ $Rev: 580669 $" );
