/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <fcntl.h>
#include <errno.h>
#include <hw/spi-master.h>

/*
 * Get device information.
 * Get the current device (when SPI_DEV_DEFAULT is set) or the next device
 * information. If the device ID is SPI_DEV_NONE(0xFFFF), the first available
 * device will be selected.
 */

int spi_getdevinfo(int fd, uint32_t device, spi_devinfo_t *devinfo)
{
	io_devctl_t	msg;
	iov_t		iov[4];
	int			status;

	/*
	 * Configurations are device specific
	 */
	if (devinfo != NULL) {
		/* Stuff the message. */
		msg.i.type        = _IO_DEVCTL;
		msg.i.combine_len = sizeof msg.i;
		msg.i.dcmd        = DCMD_SPI_GET_DEVINFO;
		msg.i.nbytes      = sizeof(spi_devinfo_t) + sizeof msg.i;
		msg.i.zero        = 0;

		/* Setup data to the device. */
		SETIOV(iov + 0, &msg.i, sizeof(msg.i));
		SETIOV(iov + 1, &device, sizeof(device));

		/* Setup data from the device. */
		SETIOV(iov + 2, &msg.o, sizeof(msg.o));
		SETIOV(iov + 3, devinfo, sizeof(spi_devinfo_t));

		if ((status = MsgSendv_r(fd, iov + 0, 2, iov + 2, 2)) != EOK)
			return (status == -ENOSYS ? ENOTTY : -status);

		return (EOK);
	}

	return (EINVAL);
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/spi-master/spi_getdevinfo.c $ $Rev: 580669 $" );
