/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include <fcntl.h>
#include <errno.h>
#include <hw/spi-master.h>


/*
 * Set configuration for specific device.
 * Valid flags : SPI_DEV_DEFAULT will set the device as default device for this fd
 */

int spi_setcfg(int fd, uint32_t device, spi_cfg_t *cfg)
{
	io_devctl_t	msg;
	iov_t		iov[3];

	if (((device & SPI_DEV_ID_MASK) != SPI_DEV_ID_NONE) && (cfg != NULL)) {
		/* Stuff the message. */
		msg.i.type        = _IO_DEVCTL;
		msg.i.combine_len = sizeof msg.i + sizeof(spi_cfg_t) + sizeof(device);
		msg.i.dcmd        = DCMD_SPI_SET_CONFIG;
		msg.i.nbytes      = sizeof(spi_cfg_t) + sizeof(device);
		msg.i.zero        = 0;

		/* Setup data to the device. */
		SETIOV(iov + 0, &msg.i, sizeof msg.i);
		SETIOV(iov + 1, &device, sizeof(device));
		SETIOV(iov + 2, cfg, sizeof(spi_cfg_t));

		return (MsgSendv(fd, iov, 3, 0, 0));
	}

	return (EINVAL);
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/spi-master/spi_setcfg.c $ $Rev: 580669 $" );
