// vim:ts=4 sw=4 sts=4
/*
 * $QNXLicenseC:
 * Copyright 2011, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ioctl.h>
#include <net/if.h>
#include <net/if_media.h>
#include <net/ifdrvcom.h>
#include <netinet/in.h>
#include <errno.h>
#include <fcntl.h>
#include <ifaddrs.h>
#include <err.h>
#include <devctl.h>

#include <dpaa_exp.h>

char *rxtx_n[] = {"RX", "TX", 0};
char *port_n[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", 0};
char *port_nn[] = {"00", "01", "02", "03", "04", "05", "06", "07", "08", "09",
                   "10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
                   "20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
                   "30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
                   "40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
                   "50", "51", "52", "53", "54", "55", "56", "57", "58", "59",
                   "60", "61", "62", "63", "64", "65", "66", "67", "68", "69",
                   "70", "71", "72", "73", "74", "75", "76", "77", "78", "79",
                   "80", "81", "82", "83", "84", "85", "86", "87", "88", "89",
                   "90", "91", "92", "93", "94", "95", "96", "97", "98", "99",
		   0};

struct dpaa_counters {
    int	ioctl;
    char *module;
    char *name;
    int	counter;
	int	ninst;
	char **instname;
} dpaa_counters[] = {

    { SIOCDPAABM, "BM", "FBPR",                       BMAN_COUNTERS_FBPR },                       /* Total Free Buffer Proxy Record (FBPR) Free Pool Count in external memory */

	{ SIOCDPAABMPOOL, "BMPOOL", "content",            BMAN_POOL_COUNTERS_CONTENT, 64, port_nn},             /**< number of free buffers for a particular pool */
    { SIOCDPAABMPOOL, "BMPOOL", "sw_depletion",       BMAN_POOL_COUNTERS_SW_DEPLETION, 64, port_nn},        /**< number of times pool entered sw depletion */
    { SIOCDPAABMPOOL, "BMPOOL", "hw_depletion",       BMAN_POOL_COUNTERS_HW_DEPLETION, 64, port_nn},        /**< number of times pool entered hw depletion */
	{ SIOCDPAABMPOOL, "BMPOOL", "hdptr",              BMAN_POOL_COUNTERS_HDPTR, 64, port_nn},             /**< Head Ptr for free buffers list for a particular pool */

    { SIOCDPAAQM, "QM", "SFDR_in_use",                QMAN_COUNTERS_SFDR_IN_USE },                /* Total Single Frame Descriptor Records (SFDR) currently in use */
    { SIOCDPAAQM, "QM", "PFDR_in_use",                QMAN_COUNTERS_PFDR_IN_USE },                /* Total Packed Frame Descriptor Records (PFDR) currently in use */
    { SIOCDPAAQM, "QM", "PFDR_free_pool",             QMAN_COUNTERS_PFDR_FREE_POOL },             /* Total Packed Frame Descriptor Record (PFDR) Free Pool Count in external memory */

    { SIOCDPAAFM, "FM", "enq_total_frame",            FMAN_COUNTERS_ENQ_TOTAL_FRAME },            /* QMI total enqueued frames counter */
    { SIOCDPAAFM, "FM", "deq_total_frame",            FMAN_COUNTERS_DEQ_TOTAL_FRAME },            /* QMI total dequeued frames counter */
    { SIOCDPAAFM, "FM", "deq_0",                      FMAN_COUNTERS_DEQ_0 },                      /* QMI 0 frames from QMan counter */
    { SIOCDPAAFM, "FM", "deq_1",                      FMAN_COUNTERS_DEQ_1 },                      /* QMI 1 frames from QMan counter */
    { SIOCDPAAFM, "FM", "deq_2",                      FMAN_COUNTERS_DEQ_2 },                      /* QMI 2 frames from QMan counter */
    { SIOCDPAAFM, "FM", "deq_3",                      FMAN_COUNTERS_DEQ_3 },                      /* QMI 3 frames from QMan counter */
    { SIOCDPAAFM, "FM", "deq_from_default",           FMAN_COUNTERS_DEQ_FROM_DEFAULT },           /* QMI dequeue from default queue counter */
    { SIOCDPAAFM, "FM", "deq_from_context",           FMAN_COUNTERS_DEQ_FROM_CONTEXT },           /* QMI dequeue from FQ context counter */
    { SIOCDPAAFM, "FM", "deq_from_fd",                FMAN_COUNTERS_DEQ_FROM_FD },                /* QMI dequeue from FD command field counter */
    { SIOCDPAAFM, "FM", "deq_confirm",                FMAN_COUNTERS_DEQ_CONFIRM },                /* QMI dequeue confirm counter */
    { SIOCDPAAFM, "FM", "semaphor_entry_full_reject", FMAN_COUNTERS_SEMAPHOR_ENTRY_FULL_REJECT }, /* DMA semaphore reject due to full entry counter */
    { SIOCDPAAFM, "FM", "semaphor_queue_full_reject", FMAN_COUNTERS_SEMAPHOR_QUEUE_FULL_REJECT }, /* DMA semaphore reject due to full CAM queue counter */
    { SIOCDPAAFM, "FM", "semaphor_sync_reject",       FMAN_COUNTERS_SEMAPHOR_SYNC_REJECT },       /* DMA semaphore reject due to sync counter */

#if 0
    { SIOCDPAAFMPORT, "FMPORT", "cycle",			  FMAN_PORT_COUNTERS_CYCLE, 2, rxtx_n },               /**< BMI performance counter */
    { SIOCDPAAFMPORT, "FMPORT", "task_util",		  FMAN_PORT_COUNTERS_TASK_UTIL, 2, rxtx_n },           /**< BMI performance counter */
    { SIOCDPAAFMPORT, "FMPORT", "queue_util",		  FMAN_PORT_COUNTERS_QUEUE_UTIL, 2, rxtx_n },          /**< BMI performance counter */
    { SIOCDPAAFMPORT, "FMPORT", "dma_util",			  FMAN_PORT_COUNTERS_DMA_UTIL, 2, rxtx_n },            /**< BMI performance counter */
    { SIOCDPAAFMPORT, "FMPORT", "fifo_util",		  FMAN_PORT_COUNTERS_FIFO_UTIL, 2, rxtx_n },           /**< BMI performance counter */
#endif
    { SIOCDPAAFMPORT, "FMPORT", "rx_pause_activation",FMAN_PORT_COUNTERS_RX_PAUSE_ACTIVATION, 2, rxtx_n }, /**< BMI Rx only performance counter */
    { SIOCDPAAFMPORT, "FMPORT", "frame",			  FMAN_PORT_COUNTERS_FRAME, 2, rxtx_n },               /**< BMI statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "discard_frame",	  FMAN_PORT_COUNTERS_DISCARD_FRAME, 2, rxtx_n },       /**< BMI statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "dealloc_buf",		  FMAN_PORT_COUNTERS_DEALLOC_BUF, 2, rxtx_n },         /**< BMI deallocate buffer statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "rx_bad_frame",		  FMAN_PORT_COUNTERS_RX_BAD_FRAME, 2, rxtx_n },        /**< BMI Rx only statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "rx_large_frame",	  FMAN_PORT_COUNTERS_RX_LARGE_FRAME, 2, rxtx_n },      /**< BMI Rx only statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "rx_out_of_buffers_discard", FMAN_PORT_COUNTERS_RX_OUT_OF_BUFFERS_DISCARD, 2, rxtx_n }, /**< BMI Rx only statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "rx_filter_frame",	  FMAN_PORT_COUNTERS_RX_FILTER_FRAME, 2, rxtx_n },     /**< BMI Rx & OP only statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "rx_list_dma_err",    FMAN_PORT_COUNTERS_RX_LIST_DMA_ERR, 2, rxtx_n },     /**< BMI Rx, OP & HC only statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "wred_discard",		  FMAN_PORT_COUNTERS_WRED_DISCARD, 2, rxtx_n },        /**< BMI OP & HC only statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "length_err",		  FMAN_PORT_COUNTERS_LENGTH_ERR, 2, rxtx_n },          /**< BMI non-Rx statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "unsupprted_format",  FMAN_PORT_COUNTERS_UNSUPPRTED_FORMAT, 2, rxtx_n },   /**< BMI non-Rx statistics counter */
    { SIOCDPAAFMPORT, "FMPORT", "deq_total",		  FMAN_PORT_COUNTERS_DEQ_TOTAL, 2, rxtx_n },           /**< QMI counter */
    { SIOCDPAAFMPORT, "FMPORT", "enq_total",		  FMAN_PORT_COUNTERS_ENQ_TOTAL, 2, rxtx_n },           /**< QMI counter */
    { SIOCDPAAFMPORT, "FMPORT", "deq_from_default",   FMAN_PORT_COUNTERS_DEQ_FROM_DEFAULT, 2, rxtx_n },    /**< QMI counter */
    { SIOCDPAAFMPORT, "FMPORT", "deq_confirm",		  FMAN_PORT_COUNTERS_DEQ_CONFIRM, 2, rxtx_n },         /**< QMI counter */

    { SIOCDPAAFMPCD,  "FMPCD",  "kg_total", FMAN_PCD_KG_COUNTERS_TOTAL },                                 /**< Policer counter */

    { SIOCDPAAFMPCD,  "FMPCD",  "plcr_yellow", FMAN_PCD_PLCR_COUNTERS_YELLOW },                              /**< Policer counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "plcr_red", FMAN_PCD_PLCR_COUNTERS_RED },                                 /**< Policer counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "plcr_recolored_to_red", FMAN_PCD_PLCR_COUNTERS_RECOLORED_TO_RED },                    /**< Policer counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "plcr_recolored_to_yellow", FMAN_PCD_PLCR_COUNTERS_RECOLORED_TO_YELLOW },                 /**< Policer counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "plcr_total", FMAN_PCD_PLCR_COUNTERS_TOTAL },                               /**< Policer counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "plcr_length_mismatch", FMAN_PCD_PLCR_COUNTERS_LENGTH_MISMATCH },                     /**< Policer counter */

    { SIOCDPAAFMPCD,  "FMPCD",  "prs_parse_dispatch", FMAN_PCD_PRS_COUNTERS_PARSE_DISPATCH },                       /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_l2_parse_result_returned", FMAN_PCD_PRS_COUNTERS_L2_PARSE_RESULT_RETURNED },             /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_l3_parse_result_returned", FMAN_PCD_PRS_COUNTERS_L3_PARSE_RESULT_RETURNED },             /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_l4_parse_result_returned", FMAN_PCD_PRS_COUNTERS_L4_PARSE_RESULT_RETURNED },             /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_shim_parse_result_returned", FMAN_PCD_PRS_COUNTERS_SHIM_PARSE_RESULT_RETURNED },           /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_l2_parse_result_returned_with_err", FMAN_PCD_PRS_COUNTERS_L2_PARSE_RESULT_RETURNED_WITH_ERR },    /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_l3_parse_result_returned_with_err", FMAN_PCD_PRS_COUNTERS_L3_PARSE_RESULT_RETURNED_WITH_ERR },    /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_l4_parse_result_returned_with_err", FMAN_PCD_PRS_COUNTERS_L4_PARSE_RESULT_RETURNED_WITH_ERR },    /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_shim_parse_result_returned_with_err", FMAN_PCD_PRS_COUNTERS_SHIM_PARSE_RESULT_RETURNED_WITH_ERR },  /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_soft_prs_cycles", FMAN_PCD_PRS_COUNTERS_SOFT_PRS_CYCLES },                      /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_soft_prs_stall_cycles", FMAN_PCD_PRS_COUNTERS_SOFT_PRS_STALL_CYCLES },                /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_hard_prs_cycle_incl_stall_cycles", FMAN_PCD_PRS_COUNTERS_HARD_PRS_CYCLE_INCL_STALL_CYCLES },     /**< Parser counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_muram_read_cycles", FMAN_PCD_PRS_COUNTERS_MURAM_READ_CYCLES },                    /**< MURAM counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_muram_read_stall_cycles", FMAN_PCD_PRS_COUNTERS_MURAM_READ_STALL_CYCLES },              /**< MURAM counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_muram_write_cycles", FMAN_PCD_PRS_COUNTERS_MURAM_WRITE_CYCLES },                   /**< MURAM counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_muram_write_stall_cycles", FMAN_PCD_PRS_COUNTERS_MURAM_WRITE_STALL_CYCLES },             /**< MURAM counter */
    { SIOCDPAAFMPCD,  "FMPCD",  "prs_fpm_command_stall_cycles", FMAN_PCD_PRS_COUNTERS_FPM_COMMAND_STALL_CYCLES },             /**< FPM counter */

    { 0, 0, 0}
};

struct query_fq_p_fields {
	uint8_t	rsrv2[8];
	uint8_t	orpc;
	uint8_t	cgid;
	uint16_t fq_ctrl;
	uint16_t dest_wq;
	uint16_t ic_cred;
	uint16_t td_thresh;
	uint32_t context_b;
	uint64_t context_a;
	uint16_t oac;
	uint8_t  rsrv34[30];
} __packed;

struct query_fq_np_fields {
	uint8_t	rsrv2;
	uint8_t	state;
	uint32_t fqd_link;
	uint16_t odp_seq;
	uint16_t orp_nesn;
	uint16_t orp_ea_hseq, orp_ea_tseq;
	uint32_t orp_ea_hptr, orp_ea_tptr;
	uint32_t pfdr_hptr, pfdr_tptr;
	uint32_t rsrv32;
	uint8_t  rsrv36;
	uint8_t  is;
	uint16_t ics_surp;
	uint32_t byte_cnt;
	uint32_t frm_cnt;
	uint32_t rsrv48;
	uint16_t ra1_sfdr, ra2_sfdr;
	uint16_t rsrv56;
	uint16_t od1_sfdr, od2_sfdr, od3_sfdr;
} __packed;

struct query_wq_fields {
	uint16_t wqid;
	uint8_t	rsrv2[28];
	union {
		uint32_t wq_len[8];
		struct {
			uint32_t wq0_len;
			uint32_t wq1_len;
			uint32_t wq2_len;
			uint32_t wq3_len;
			uint32_t wq4_len;
			uint32_t wq5_len;
			uint32_t wq6_len;
			uint32_t wq7_len;
		} __packed;
	} __packed;
} __packed;

struct ifdrv dpaa_ifd;

uint32_t
dpaa_cntr_ioctl (int s, int cmd, int inst, int ctr) {
    struct {
		struct ifdrv ifd;
		uint32_t args[2];
    } dcmd;
    int error;

    dcmd.ifd = dpaa_ifd;
    dcmd.ifd.ifd_cmd = cmd;
    dcmd.args[0] = ctr;
    dcmd.args[1] = inst;
    if((error=devctl(s, SIOCGDRVSPEC, &dcmd, sizeof(dcmd), NULL)) == EOK)
		return dcmd.args[0];
    return 0;
}

int
dpaa_get_fqd (int s, int idx, struct dpaa_fqd *fqd, int np) {
	struct dpaa_fqd_req dcmd;
    int error;

    dcmd.ifd = dpaa_ifd;
    dcmd.ifd.ifd_cmd = np ? SIOCDPAAQRYFQ_NP : SIOCDPAAQRYFQ_P;
    dcmd.args[0] = idx;
    dcmd.args[1] = 0;
    if((error=devctl(s, SIOCGDRVSPEC, &dcmd, sizeof(dcmd), NULL)) == EOK)
		if(fqd) memcpy((void*)fqd, (void*)&dcmd.fqd, sizeof(dcmd.fqd));
	return error;
}

int
dpaa_get_wq (int s, int idx, struct dpaa_fqd *fqd) {
	struct dpaa_fqd_req dcmd;
    int error;

    dcmd.ifd = dpaa_ifd;
    dcmd.ifd.ifd_cmd = SIOCDPAAQRYWQ;
    dcmd.args[0] = idx;
    dcmd.args[1] = 0;
    if((error=devctl(s, SIOCGDRVSPEC, &dcmd, sizeof(dcmd), NULL)) == EOK)
		if(fqd) memcpy((void*)fqd, (void*)&dcmd.fqd, sizeof(dcmd.fqd));
	return error;
}

int		verbose=0, longlist=0;
int		memtest=0;
int	domemtest(int testno);

int main (int argc, char **argv)
{
	int			s = -1, i, to_find;
	uint32_t cntr, cntr_val;
	char *module = NULL;
	int		report_queues=0, report_work_queues=0, report_counters=0;
	int		nfq =  2048, nwq = 0x70, error;
	char			*curp;

	while ( (i = getopt (argc, argv, "cqQ:wW:vlm:")) != EOF)
	{
		switch (i)
		{
		 case 'v':
			verbose++;
			break;
		 case 'l':
			longlist++;
			break;
		 case 'c':
			report_counters = 1;
			break;
		 case 'q':
			report_queues = 1;
			break;
		 case 'Q':
			report_queues = 1;
			nfq = strtol (optarg, NULL, 0);
			break;
		 case 'w':
			report_work_queues = 1;
			break;
		 case 'W':
			report_work_queues = 1;
			nwq = strtol (optarg, NULL, 0);
			break;
		 case 'm':
			memtest = strtol (optarg, NULL, 0);
			if(memtest == 0) memtest = -1;
			break;
		 default:
			exit (1);
		}
	}
	if(report_queues==0 && report_counters==0 && report_work_queues == 0 && memtest == 0)
		report_counters = 1;

	to_find = argc - optind;

	if(memtest == -1) {
		int i;
		for(i=1; i<100; i++)
			domemtest(i);
		return 0;
	} else if(memtest)
		return domemtest(memtest);

	if (!to_find)
	{
		fprintf (stderr, "dpaainfo: must specify at least one interface.\n");
		exit (2);
	}

	if (s == -1 && (s = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		err(3, "socket");
	}	

	if(report_counters) {
		for (i = optind; i < argc; i++) {
			if ((curp = argv[i]) == NULL)
				continue;

			printf("%s: DPAA counters\n", curp);

			memcpy(dpaa_ifd.ifd_name, curp, IFNAMSIZ);

			for(cntr=0; dpaa_counters[cntr].module; cntr++) {
				int inst;

				if(module && strcmp(module, dpaa_counters[cntr].module) != 0)
					continue;

				if(dpaa_counters[cntr].ninst == 0) dpaa_counters[cntr].ninst = 1;
				if(cntr > 0 && strcmp(dpaa_counters[cntr].module, dpaa_counters[cntr-1].module) != 0)
					printf("\n");

				for(inst=0; inst < dpaa_counters[cntr].ninst; inst++) {
					cntr_val = dpaa_cntr_ioctl (s, dpaa_counters[cntr].ioctl, inst, dpaa_counters[cntr].counter);
					if(dpaa_counters[cntr].ninst > 1) {
						if(dpaa_counters[cntr].ninst <= 2 || cntr_val > 0 || inst==0)
							printf("  %-8s %26s/%-3s %d\n", dpaa_counters[cntr].module, dpaa_counters[cntr].name, dpaa_counters[cntr].instname[inst], cntr_val);
					} else
						printf("  %-8s %26s     %d\n", dpaa_counters[cntr].module, dpaa_counters[cntr].name, cntr_val);
				}
			}
			
			argv[i] = NULL;

			if (--to_find == 0)
				exit(EXIT_SUCCESS);
		}
	}

	if(report_queues) {
		curp = argv[optind];

		memcpy(dpaa_ifd.ifd_name, curp, IFNAMSIZ);

		printf("%s: Queue Manager Frame Queues (first %d):\n", curp, nfq);
		for(i=0; i<nfq; i++) {
			int j;
			struct dpaa_fqd fqd[2];
			struct query_fq_p_fields *fq_p = (struct query_fq_p_fields *)&fqd[0];
			struct query_fq_np_fields *fq_np = (struct query_fq_np_fields *)&fqd[1];

			/* Query FQ P */
			memset((void*)&fqd[0], 0, sizeof(fqd[0]));
			if((error=dpaa_get_fqd (s, i, &fqd[0], 0)) != 0) {
				if(errno == ENOENT || error == ENOENT)
					break;
				perror("dpaa_get_fqd");
				printf("%3d error = 0x%x\n", i, error);
				break;
			}

			/* Query FQ NP */
			memset((void*)&fqd[1], 0, sizeof(fqd[1]));
			if((error=dpaa_get_fqd (s, i, &fqd[1], 1)) != 0) {
				if(errno == ENOENT || error == ENOENT)
					break;
				perror("dpaa_get_fqd");
				printf("%3d error = 0x%x\n", i, error);
				break;
			}

			if(fq_np->state == 0 && !(verbose > 1))
				continue;

			printf("%3d", i);

			printf(" state %x%c%s%s bytes %6d frames %2d odp_seq %5d orp_nesn %5d",
					fq_np->state&0x07, "-RTSPA??"[fq_np->state&0x07], fq_np->state&0x10?"FE":"  ", fq_np->state&0x08?"R":" ",
					fq_np->byte_cnt, fq_np->frm_cnt, fq_np->odp_seq, fq_np->orp_nesn);
			printf(" fq_ctrl %02x dest_wq 0x%02x:%x context_a %16llx context_b %8x",
				   fq_p->fq_ctrl, fq_p->dest_wq/8, fq_p->dest_wq%8, fq_p->context_a, fq_p->context_b);
			printf(" fqd_link %d", fq_np->fqd_link);

			if(longlist) {
				printf("\n%3s  ","");
				printf(" nra %d, it %d nod %d npc %d",
					   (fq_np->ra1_sfdr>>14)&0x3, (fq_np->ra2_sfdr>>14)&0x1, (fq_np->od1_sfdr>>14)&0x3, (fq_np->od3_sfdr>>14)&0x3);
				printf(" ra1 %d ra2 %d od1 %d od2 %d od3 %d",
					   fq_np->ra1_sfdr&0x7ff, fq_np->ra2_sfdr&0x7ff, fq_np->od1_sfdr&0x7ff, fq_np->od2_sfdr&0x7ff, fq_np->od3_sfdr&0x7ff);
				printf(" orp_ea_seqs %d/%d orp_ea_ptrs %d/%d pfdr_ptrs %d/%d",
						fq_np->orp_ea_hseq, fq_np->orp_ea_tseq,
						fq_np->orp_ea_hptr, fq_np->orp_ea_tptr,
						fq_np->pfdr_hptr,   fq_np->pfdr_tptr);
			}

			printf("\n");

			if(verbose > 0) {
				printf("%3s","p");
				for(j=0; j<sizeof(struct query_fq_p_fields); j++) {
					if(j != 0 && (j%32) == 0)
						printf("%3s", "");
					printf(" %02x", ((uint8_t*)&fqd[0])[j]);
					if((j%32) == 32-1)
						putchar('\n');
				}
				if((j%32) != 0)
					putchar('\n');
			}

			if(verbose > 0) {
				printf("%3s", "np");
				for(j=0; j<sizeof(struct query_fq_np_fields); j++) {
					if(j != 0 && (j%32) == 0)
						printf("%3s", "");
					printf(" %02x", ((uint8_t*)&fqd[1])[j]);
					if((j%32) == 32-1)
						putchar('\n');
				}
				if((j%32) != 0)
					putchar('\n');
			}
		}
	}

	if(report_work_queues) {
		curp = argv[optind];

		memcpy(dpaa_ifd.ifd_name, curp, IFNAMSIZ);

		printf("%s: Queue Manager Work Queues (first %d):\n", curp, nwq);
		for(i=0; i<nwq; i++) {
			int j;
			struct dpaa_fqd fqd;
			struct query_wq_fields *wq = (struct query_wq_fields *)&fqd;

			/* Query WQ */
			memset((void*)&fqd, 0, sizeof(fqd));
			if((error=dpaa_get_wq (s, i, &fqd)) != 0) {
				if(errno == ENOENT || error == ENOENT)
					continue;
				perror("dpaa_get_wq");
				printf("0x%02x error = 0x%x\n", i, error);
				break;
			}

			printf("0x%02x", i);

			for(j=0; j<8; j++)
				printf(" wq%d %d", j, wq->wq_len[j]);

			printf("\n");

			if(verbose > 0) {
				printf("%4s","");
				for(j=0; j<sizeof(struct query_wq_fields); j++) {
					if(j != 0 && (j%32) == 0)
						printf("%4s", "");
					printf(" %02x", ((uint8_t*)&fqd)[j]);
					if((j%32) == 32-1)
						putchar('\n');
				}
				if((j%32) != 0)
					putchar('\n');
			}

		}
	}

	exit(0);
}

#include <sys/time.h>

#define CACHELINE_SIZE	64
#define PAGE_SIZE		1024

#define lwsync()    __asm__ __volatile__ ("lwsync" : : : "memory")

#define dcbt_ro(p)  __asm__ __volatile__ ("dcbt 0,%0" : : "r" (p))		/* Data Cache Block Touch           - pre-read data into cache block */
#define dcbt_rw(p)  __asm__ __volatile__ ("dcbtst 0,%0" : : "r" (p))	/* Data Cache Block Touch for Store - pre-read data for future storage */
#define dcbzl(p)    __asm__ __volatile__ ("dcbzl 0,%0" : : "r" (p))		/* Data Cache Block Zero            - set cache block to zero */
#define dcbf(p)     __asm__ __volatile__ ("dcbf 0,%0" : : "r" (p))		/* Data Cache Block Flush           - copy block to storage and mark invalid */
#define dcbst(p)    __asm__ __volatile__ ("dcbst 0,%0" : : "r" (p))		/* Data Cache Block Store           - copy block to storage and mark unmodified */


#define CACHE_PREFETCH(p)	dcbt_ro(p)
#define CACHE_ALLOC(p)		dcbzl(p)
#define CACHE_FLUSH(p)		dcbf(p)

#define OP_RD		(1)
#define OP_WR		(2)
#define OP_CPY		(OP_RD|OP_WR)

#define PREFETCH	(1)
#define ALLOC		(1)
#define RFLUSH		(1)
#define WFLUSH		(1)

#define DOMEMTEST(op,n,pre,alloc,wflush,rflush) \
	do { \
			if(pre && n > 0) CACHE_PREFETCH((uint8_t *)sp); \
			if(pre && n > 1) CACHE_PREFETCH((uint8_t *)sp+1*CACHELINE_SIZE); \
			if(pre && n > 2) CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE); \
			if(pre && n > 3) CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE); \
			if(pre && n > 4) CACHE_PREFETCH((uint8_t *)sp+4*CACHELINE_SIZE); \
			if(pre && n > 5) CACHE_PREFETCH((uint8_t *)sp+5*CACHELINE_SIZE); \
			if(pre && n > 6) CACHE_PREFETCH((uint8_t *)sp+6*CACHELINE_SIZE); \
			if(pre && n > 7) CACHE_PREFETCH((uint8_t *)sp+7*CACHELINE_SIZE); \
			for(i=0; i<sizeof(sbuf)/((n)*CACHELINE_SIZE); i++) { \
				if(pre && n > 0) CACHE_PREFETCH((uint8_t *)sp+(n+0)*CACHELINE_SIZE); \
				if(pre && n > 1) CACHE_PREFETCH((uint8_t *)sp+(n+1)*CACHELINE_SIZE); \
				if(pre && n > 2) CACHE_PREFETCH((uint8_t *)sp+(n+2)*CACHELINE_SIZE); \
				if(pre && n > 3) CACHE_PREFETCH((uint8_t *)sp+(n+3)*CACHELINE_SIZE); \
				if(pre && n > 4) CACHE_PREFETCH((uint8_t *)sp+(n+4)*CACHELINE_SIZE); \
				if(pre && n > 5) CACHE_PREFETCH((uint8_t *)sp+(n+5)*CACHELINE_SIZE); \
				if(pre && n > 6) CACHE_PREFETCH((uint8_t *)sp+(n+6)*CACHELINE_SIZE); \
				if(pre && n > 7) CACHE_PREFETCH((uint8_t *)sp+(n+7)*CACHELINE_SIZE); \
				if(alloc && n > 0) CACHE_ALLOC((uint8_t *)dp); \
				if(alloc && n > 1) CACHE_ALLOC((uint8_t *)dp+1*CACHELINE_SIZE); \
				if(alloc && n > 2) CACHE_ALLOC((uint8_t *)dp+2*CACHELINE_SIZE); \
				if(alloc && n > 3) CACHE_ALLOC((uint8_t *)dp+3*CACHELINE_SIZE); \
				if(alloc && n > 4) CACHE_ALLOC((uint8_t *)dp+4*CACHELINE_SIZE); \
				if(alloc && n > 5) CACHE_ALLOC((uint8_t *)dp+5*CACHELINE_SIZE); \
				if(alloc && n > 6) CACHE_ALLOC((uint8_t *)dp+6*CACHELINE_SIZE); \
				if(alloc && n > 7) CACHE_ALLOC((uint8_t *)dp+7*CACHELINE_SIZE); \
				for(j=0; j<(n)*CACHELINE_SIZE/sizeof(*sp); j++) { \
					if(op==OP_CPY) *dp++ = *sp++; \
					else if(op==OP_RD) sum += *sp++; \
					else if(op==OP_WR) *dp++ = -1; \
				} \
				if(rflush && n > 7) CACHE_FLUSH((uint8_t *)sp - 8*CACHELINE_SIZE); \
				if(rflush && n > 6) CACHE_FLUSH((uint8_t *)sp - 7*CACHELINE_SIZE); \
				if(rflush && n > 5) CACHE_FLUSH((uint8_t *)sp - 6*CACHELINE_SIZE); \
				if(rflush && n > 4) CACHE_FLUSH((uint8_t *)sp - 5*CACHELINE_SIZE); \
				if(rflush && n > 3) CACHE_FLUSH((uint8_t *)sp - 4*CACHELINE_SIZE); \
				if(rflush && n > 2) CACHE_FLUSH((uint8_t *)sp - 3*CACHELINE_SIZE); \
				if(rflush && n > 1) CACHE_FLUSH((uint8_t *)sp - 2*CACHELINE_SIZE); \
				if(rflush && n > 0) CACHE_FLUSH((uint8_t *)sp - 1*CACHELINE_SIZE); \
				if(wflush && n > 7) CACHE_FLUSH((uint8_t *)dp - 8*CACHELINE_SIZE); \
				if(wflush && n > 6) CACHE_FLUSH((uint8_t *)dp - 7*CACHELINE_SIZE); \
				if(wflush && n > 5) CACHE_FLUSH((uint8_t *)dp - 6*CACHELINE_SIZE); \
				if(wflush && n > 4) CACHE_FLUSH((uint8_t *)dp - 5*CACHELINE_SIZE); \
				if(wflush && n > 3) CACHE_FLUSH((uint8_t *)dp - 4*CACHELINE_SIZE); \
				if(wflush && n > 2) CACHE_FLUSH((uint8_t *)dp - 3*CACHELINE_SIZE); \
				if(wflush && n > 1) CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE); \
				if(wflush && n > 0) CACHE_FLUSH((uint8_t *)dp - 1*CACHELINE_SIZE); \
			} \
			lwsync(); \
	} while(0)

#define TESTMEMCOPY(n,pre,alloc,wflush,rflush)	DOMEMTEST(OP_CPY,n,pre,alloc,wflush,rflush)
#define TESTMEMREAD(n,pre,rflush)				DOMEMTEST(OP_RD,n,pre,0,0,rflush)
#define TESTMEMWRITE(n,alloc,wflush)			DOMEMTEST(OP_WR,n,0,alloc,wflush,0)


volatile uint32_t sbuf[1024*1024*1024/sizeof(uint32_t)], dbuf[1024*1024*1024/sizeof(uint32_t)], pbuf[4096/sizeof(uint32_t)];

int
domemtest(int testno) {
	volatile uint32_t *sp, *dp;
	int i, j, sum=0;
	struct timeval start, end;
	double delta;

	/* warm up run to allocate page mappings and TLB entries */
	sp=sbuf; dp=dbuf;
	for(i=0; i<sizeof(sbuf)/PAGE_SIZE; i++) {
		*dp = -1;
		*sp = -1;
		/* flush cache entries we just modified */
		CACHE_FLUSH((uint8_t *)sp);
		CACHE_FLUSH((uint8_t *)dp);
		sp += PAGE_SIZE/sizeof(*sp);
		dp += PAGE_SIZE/sizeof(*dp);
	}
	dp = pbuf;
	for(i=0; i<sizeof(pbuf)/PAGE_SIZE; i++) {
		*dp = -1;
		/* flush cache entry we just modified */
		CACHE_FLUSH((uint8_t *)dp);
		dp += PAGE_SIZE/sizeof(*dp);
	}
	lwsync();

	sp=sbuf; dp=dbuf;
	switch(testno) {
		case 1:
			printf("Cached memcpy: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++)
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			lwsync();
#else
			TESTMEMCOPY(1, 0, 0, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 2:
			printf("Cached memcpy with prefetch: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
#else
			TESTMEMCOPY(1, PREFETCH, 0, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 3:
			printf("Cached memcpy with prefetch and cache-alloc: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
#else
			TESTMEMCOPY(1, PREFETCH, ALLOC, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 4:
			printf("Cached memcpy with prefetch, cache-alloc and write flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMCOPY(1, PREFETCH, ALLOC, WFLUSH, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 5:
			printf("Cached memcpy with prefetch, cache-alloc, read-flush and write-flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)sp - CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMCOPY(1, PREFETCH, ALLOC, WFLUSH, RFLUSH);
#endif
			gettimeofday(&end, NULL);
			break;

		case 11:
			printf("2 Line cached memcpy: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++)
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			lwsync();
#else
			TESTMEMCOPY(2, 0, 0, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 12:
			printf("2 Line cached memcpy with prefetch: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
#else
			TESTMEMCOPY(2, PREFETCH, 0, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 13:
			printf("2 Line cached memcpy with prefetch and cache-alloc: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
#else
			TESTMEMCOPY(2, PREFETCH, ALLOC, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 14:
			printf("2 Line cached memcpy with prefetch, cache-alloc and write flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMCOPY(2, PREFETCH, ALLOC, WFLUSH, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 15:
			printf("2 Line cached memcpy with prefetch, cache-alloc, read-flush and write-flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)sp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)sp - CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMCOPY(2, PREFETCH, ALLOC, WFLUSH, RFLUSH);
#endif
			gettimeofday(&end, NULL);
			break;

		case 21:
			printf("Cached mem read: ");
			gettimeofday(&start, NULL);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++)
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					sum += *sp++;
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 22:
			printf("Cached mem read with prefetch: ");
			gettimeofday(&start, NULL);
			CACHE_PREFETCH((uint8_t *)sp);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					sum += *sp++;
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 25:
			printf("Cached mem read with prefetch and read-flush: ");
			gettimeofday(&start, NULL);
			CACHE_PREFETCH((uint8_t *)sp);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					sum += *sp++;
				CACHE_FLUSH((uint8_t *)sp - CACHELINE_SIZE);
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;

		case 31:
			printf("2 Line cached mem read: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++)
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					sum += *sp++;
			lwsync();
#else
			TESTMEMREAD(2, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 32:
			printf("2 Line cached mem read with prefetch: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					sum += *sp++;
			}
			lwsync();
#else
			TESTMEMREAD(2, PREFETCH, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 35:
			printf("2 Line cached mem read with prefetch and read-flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					sum += *sp++;
				CACHE_FLUSH((uint8_t *)sp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)sp - CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMREAD(2, PREFETCH, RFLUSH);
#endif
			gettimeofday(&end, NULL);
			break;

		case 41:
			printf("Cached mem write: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++)
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = -1;
			lwsync();
#else
			TESTMEMWRITE(1, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 42:
			printf("Cached mem write with cache-alloc: ");
			gettimeofday(&start, NULL);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_ALLOC((uint8_t *)dp);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = -1;
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 44:
			printf("Cached mem write with cache-alloc and write flush: ");
			gettimeofday(&start, NULL);
			for(i=0; i<sizeof(sbuf)/CACHELINE_SIZE; i++) {
				CACHE_ALLOC((uint8_t *)dp);
				for(j=0; j<CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = -1;
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;

		case 51:
			printf("2 Line cached mem write: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++)
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = -1;
			lwsync();
#else
			TESTMEMWRITE(2, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 53:
			printf("2 Line cached mem write with cache-alloc: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = -1;
			}
			lwsync();
#else
			TESTMEMWRITE(2, ALLOC, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 54:
			printf("2 Line cached mem write with cache-alloc and write flush: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = -1;
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMWRITE(2, ALLOC, WFLUSH);
#endif
			gettimeofday(&end, NULL);
			break;

		case 61:
			printf("2 Line cached memcpy with offset 1: ");
			dp += CACHELINE_SIZE/sizeof(*dp);
			gettimeofday(&start, NULL);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++)
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 62:
			printf("2 Line cached memcpy with prefetch with offset 1: ");
			dp += CACHELINE_SIZE/sizeof(*dp);
			gettimeofday(&start, NULL);
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 63:
			printf("2 Line cached memcpy with prefetch and cache-alloc with offset 1: ");
			dp += CACHELINE_SIZE/sizeof(*dp);
			gettimeofday(&start, NULL);
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 64:
			printf("2 Line cached memcpy with prefetch, cache-alloc and write flush with offset 1: ");
			dp += CACHELINE_SIZE/sizeof(*dp);
			gettimeofday(&start, NULL);
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;
		case 65:
			printf("2 Line cached memcpy with prefetch, cache-alloc, read-flush and write-flush with offset 1: ");
			dp += CACHELINE_SIZE/sizeof(*dp);
			gettimeofday(&start, NULL);
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(2*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+CACHELINE_SIZE);
				for(j=0; j<2*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)sp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)sp - CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - CACHELINE_SIZE);
			}
			lwsync();
			gettimeofday(&end, NULL);
			break;

		case 71:
			printf("4 Line cached memcpy: ");
			gettimeofday(&start, NULL);
#if 0
			for(i=0; i<sizeof(sbuf)/(4*CACHELINE_SIZE); i++)
				for(j=0; j<4*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			lwsync();
#else
			TESTMEMCOPY(4, 0, 0, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 72:
			printf("4 Line cached memcpy with prefetch: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+1*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(4*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+4*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+5*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+6*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+7*CACHELINE_SIZE);
				for(j=0; j<4*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
#else
			TESTMEMCOPY(4, PREFETCH, 0, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 73:
			printf("4 Line cached memcpy with prefetch and cache-alloc: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+1*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(4*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+4*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+5*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+6*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+7*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+1*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp+2*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp+3*CACHELINE_SIZE);
				for(j=0; j<4*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
			}
			lwsync();
#else
			TESTMEMCOPY(4, PREFETCH, ALLOC, 0, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 74:
			printf("4 Line cached memcpy with prefetch, cache-alloc and write flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+1*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(4*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+4*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+5*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+6*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+7*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+1*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp+2*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp+3*CACHELINE_SIZE);
				for(j=0; j<4*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)dp - 4*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 3*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 1*CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMCOPY(4, PREFETCH, ALLOC, WFLUSH, 0);
#endif
			gettimeofday(&end, NULL);
			break;
		case 75:
			printf("4 Line cached memcpy with prefetch, cache-alloc, read-flush and write-flush: ");
			gettimeofday(&start, NULL);
#if 0
			CACHE_PREFETCH((uint8_t *)sp);
			CACHE_PREFETCH((uint8_t *)sp+1*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+2*CACHELINE_SIZE);
			CACHE_PREFETCH((uint8_t *)sp+3*CACHELINE_SIZE);
			for(i=0; i<sizeof(sbuf)/(4*CACHELINE_SIZE); i++) {
				CACHE_PREFETCH((uint8_t *)sp+4*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+5*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+6*CACHELINE_SIZE);
				CACHE_PREFETCH((uint8_t *)sp+7*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp);
				CACHE_ALLOC((uint8_t *)dp+1*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp+2*CACHELINE_SIZE);
				CACHE_ALLOC((uint8_t *)dp+3*CACHELINE_SIZE);
				for(j=0; j<4*CACHELINE_SIZE/sizeof(*sp); j++)
					*dp++ = *sp++;
				CACHE_FLUSH((uint8_t *)sp - 4*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)sp - 3*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)sp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)sp - 1*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 4*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 3*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 2*CACHELINE_SIZE);
				CACHE_FLUSH((uint8_t *)dp - 1*CACHELINE_SIZE);
			}
			lwsync();
#else
			TESTMEMCOPY(4, PREFETCH, ALLOC, WFLUSH, RFLUSH);
#endif
			gettimeofday(&end, NULL);
			break;

		default:
			//printf("Error: unknown mem test number %d\n", testno);
			return 1;
	}
	lwsync();
	sum = abs(sum*2); /* make sure compiler doesn't optimize out sum */
	delta = (end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec) / 1000000.0;
	printf("%d Kbytes in %f seconds, %fMb/s\n", sizeof(sbuf)/1024, delta, sizeof(sbuf)/(delta*1024*1024));

	return 0;
}

__SRCVERSION( "$URL: http://svn.ott.qnx.com/product/branches/6.5.0/trunk/lib/io-pkt/utils/d/dpaainfo/dpaainfo.c $ $Rev: 233513 $" );
