/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          frame.c

  @Author

  @Description   Functions for frame creation
*//***************************************************************************/
/* move in .h file when ready */
#define __ERR_MODULE__  MODULE_UNKNOWN

#include "error_ext.h"
#include "endian_ext.h"
#include "string_ext.h"

#include "iphc_manip.h"
#include "frame.h"


/**************************************************************************//**
 @Description   Global Variables
*//***************************************************************************/
uint32_t crc32Table[CRC_TABLE_SIZE];     /* Table of CRC 8-bit remainders */
uint8_t  crc8Table[CRC_TABLE_SIZE];      /* 8-bit CRC table for HEC calculation */
static int32_t  framesCounter = 0;        /* Number of frames configured */
static uint8_t vlanIndex      = 0;        /* Number of VLAN headers encountered */
static uint8_t mplsIndex     = 0;         /* Number of mpls headers encountered */
static uint8_t  padIndex      = 0;        /* Number of Padding headers encountered */
static uint8_t llcIndex      = 0;         /* Number of LLC  headers encountered */
static uint8_t snapIndex     = 0;         /* Number of SNAP headers encountered */

static t_HeaderInfo pRegisteredHeaderArray[e_MAX_HEADER_TYPE_COUNT] =
{
    {
        e_HEADER_TYPE_NONE,
        0,0,0,0,0,0,0,0,0,0,0,0
    },
    {
         e_HEADER_TYPE_PAYLOAD,
         0,
         HDR_Payload_Config,
         HDR_Payload_Init,
         HDR_Payload_Clone,
         HDR_Payload_GetField,
         HDR_Payload_ConfigField,
         HDR_Payload_ConfigAuto,
         HDR_Payload_ConfigHeader,
         HDR_Payload_Randomize,
         HDR_Payload_Serialize,
         HDR_Payload_GetHeaderSize,
         HDR_Payload_GetHeader,
         HDR_Payload_GetHeaderError
    },
    {
         e_HEADER_TYPE_ETH,
         0,
         HDR_Eth_Config,
         HDR_Eth_Init,
         HDR_Eth_Clone,
         HDR_Eth_GetField,
         HDR_Eth_ConfigField,
         HDR_Eth_ConfigAuto,
         HDR_Eth_ConfigHeader,
         HDR_Eth_Randomize,
         HDR_Eth_Serialize,
         HDR_Eth_GetHeaderSize,
         HDR_Eth_GetHeader,
         HDR_Eth_GetHeaderError
    },
    {
         e_HEADER_TYPE_ETH_802_3,
         0,
         HDR_Eth_Config,
         HDR_Eth_Init,
         HDR_Eth_Clone,
         HDR_Eth_GetField,
         HDR_Eth_ConfigField,
         HDR_Eth_ConfigAuto,
         HDR_Eth_ConfigHeader,
         HDR_Eth_Randomize,
         HDR_Eth_Serialize,
         HDR_Eth_GetHeaderSize,
         HDR_Eth_GetHeader,
         HDR_Eth_GetHeaderError
    },
    {
         e_HEADER_TYPE_IEEE1588_ETH,
         0,
         HDR_Eth_Config,
         HDR_Eth_Init,
         HDR_Eth_Clone,
         HDR_Eth_GetField,
         HDR_Eth_ConfigField,
         HDR_Eth_ConfigAuto,
         HDR_Eth_ConfigHeader,
         HDR_Eth_Randomize,
         HDR_Eth_Serialize,
         HDR_Eth_GetHeaderSize,
         HDR_Eth_GetHeader,
         HDR_Eth_GetHeaderError
    },
    {
         e_HEADER_TYPE_VLAN,
         0,
         HDR_VLAN_Config,
         HDR_VLAN_Init,
         HDR_VLAN_Clone,
         HDR_VLAN_GetField,
         HDR_VLAN_ConfigField,
         HDR_VLAN_ConfigAuto,
         HDR_VLAN_ConfigHeader,
         HDR_VLAN_Randomize,
         HDR_VLAN_Serialize,
         HDR_VLAN_GetHeaderSize,
         HDR_VLAN_GetHeader,
         HDR_VLAN_GetHeaderError
    },
    {
         e_HEADER_TYPE_VTAGGED_802_3,
         0,
         HDR_VLAN_Config,
         HDR_VLAN_Init,
         HDR_VLAN_Clone,
         HDR_VLAN_GetField,
         HDR_VLAN_ConfigField,
         HDR_VLAN_ConfigAuto,
         HDR_VLAN_ConfigHeader,
         HDR_VLAN_Randomize,
         HDR_VLAN_Serialize,
         HDR_VLAN_GetHeaderSize,
         HDR_VLAN_GetHeader,
         HDR_VLAN_GetHeaderError
    },
    {
         e_HEADER_TYPE_MPLS,
         0,
         HDR_MPLS_Config,
         HDR_MPLS_Init,
         HDR_MPLS_Clone,
         HDR_MPLS_GetField,
         HDR_MPLS_ConfigField,
         HDR_MPLS_ConfigAuto,
         HDR_MPLS_ConfigHeader,
         HDR_MPLS_Randomize,
         HDR_MPLS_Serialize,
         HDR_MPLS_GetHeaderSize,
         HDR_MPLS_GetHeader,
         HDR_MPLS_GetHeaderError
    },
    {
         e_HEADER_TYPE_L2TPV2,
         0,
         HDR_L2TPv2_Config,
         HDR_L2TPv2_Init,
         HDR_L2TPv2_Clone,
         HDR_L2TPv2_GetField,
         HDR_L2TPv2_ConfigField,
         HDR_L2TPv2_ConfigAuto,
         HDR_L2TPv2_ConfigHeader,
         HDR_L2TPv2_Randomize,
         HDR_L2TPv2_Serialize,
         HDR_L2TPv2_GetHeaderSize,
         HDR_L2TPv2_GetHeader,
         HDR_L2TPv2_GetHeaderError
     },
     {
         e_HEADER_TYPE_L2TPV3,
         0,
         HDR_L2TPv3_Config,
         HDR_L2TPv3_Init,
         HDR_L2TPv3_Clone,
         HDR_L2TPv3_GetField,
         HDR_L2TPv3_ConfigField,
         HDR_L2TPv3_ConfigAuto,
         HDR_L2TPv3_ConfigHeader,
         HDR_L2TPv3_Randomize,
         HDR_L2TPv3_Serialize,
         HDR_L2TPv3_GetHeaderSize,
         HDR_L2TPv3_GetHeader,
         HDR_L2TPv3_GetHeaderError
     },
     {
         e_HEADER_TYPE_IPv4,
         0,
         HDR_IPv4_Config,
         HDR_IPv4_Init,
         HDR_IPv4_Clone,
         HDR_IPv4_GetField,
         HDR_IPv4_ConfigField,
         HDR_IPv4_ConfigAuto,
         HDR_IPv4_ConfigHeader,
         HDR_IPv4_Randomize,
         HDR_IPv4_Serialize,
         HDR_IPv4_GetHeaderSize,
         HDR_IPv4_GetHeader,
         HDR_IPv4_GetHeaderError
    },
    {
        e_HEADER_TYPE_IPv6,
        0,
        HDR_IPv6_Config,
        HDR_IPv6_Init,
        HDR_IPv6_Clone,
        HDR_IPv6_GetField,
        HDR_IPv6_ConfigField,
        HDR_IPv6_ConfigAuto,
        HDR_IPv6_ConfigHeader,
        HDR_IPv6_Randomize,
        HDR_IPv6_Serialize,
        HDR_IPv6_GetHeaderSize,
        HDR_IPv6_GetHeader,
        HDR_IPv6_GetHeaderError
    },
    {
         e_HEADER_TYPE_TCP,
         0,
         HDR_TCP_Config,
         HDR_TCP_Init,
         HDR_TCP_Clone,
         HDR_TCP_GetField,
         HDR_TCP_ConfigField,
         HDR_TCP_ConfigAuto,
         HDR_TCP_ConfigHeader,
         HDR_TCP_Randomize,
         HDR_TCP_Serialize,
         HDR_TCP_GetHeaderSize,
         HDR_TCP_GetHeader,
         HDR_TCP_GetHeaderError
    },
    {
         e_HEADER_TYPE_UDP,
         0,
         HDR_UDP_Config,
         HDR_UDP_Init,
         HDR_UDP_Clone,
         HDR_UDP_GetField,
         HDR_UDP_ConfigField,
         HDR_UDP_ConfigAuto,
         HDR_UDP_ConfigHeader,
         HDR_UDP_Randomize,
         HDR_UDP_Serialize,
         HDR_UDP_GetHeaderSize,
         HDR_UDP_GetHeader,
         HDR_UDP_GetHeaderError
    },
    {
         e_HEADER_TYPE_SCTP,
         0,
         HDR_SCTP_Config,
         HDR_SCTP_Init,
         HDR_SCTP_Clone,
         HDR_SCTP_GetField,
         HDR_SCTP_ConfigField,
         HDR_SCTP_ConfigAuto,
         HDR_SCTP_ConfigHeader,
         HDR_SCTP_Randomize,
         HDR_SCTP_Serialize,
         HDR_SCTP_GetHeaderSize,
         HDR_SCTP_GetHeader,
         HDR_SCTP_GetHeaderError
    },
    {
         e_HEADER_TYPE_SCTP_CHUNK_DATA,
         0,
         HDR_SCTP_CHUNK_DATA_Config,
         HDR_SCTP_CHUNK_DATA_Init,
         HDR_SCTP_CHUNK_DATA_Clone,
         HDR_SCTP_CHUNK_DATA_GetField,
         HDR_SCTP_CHUNK_DATA_ConfigField,
         HDR_SCTP_CHUNK_DATA_ConfigAuto,
         HDR_SCTP_CHUNK_DATA_ConfigHeader,
         HDR_SCTP_CHUNK_DATA_Randomize,
         HDR_SCTP_CHUNK_DATA_Serialize,
         HDR_SCTP_CHUNK_DATA_GetHeaderSize,
         HDR_SCTP_CHUNK_DATA_GetHeader,
         HDR_SCTP_CHUNK_DATA_GetHeaderError
    },
    {
         e_HEADER_TYPE_IEEE1588_V1_GENERAL_UDP,
         0,
         HDR_UDP_Config,
         HDR_UDP_Init,
         HDR_UDP_Clone,
         HDR_UDP_GetField,
         HDR_UDP_ConfigField,
         HDR_UDP_ConfigAuto,
         HDR_UDP_ConfigHeader,
         HDR_UDP_Randomize,
         HDR_UDP_Serialize,
         HDR_UDP_GetHeaderSize,
         HDR_UDP_GetHeader,
         HDR_UDP_GetHeaderError
    },
    {
         e_HEADER_TYPE_IEEE1588_V2_GENERAL_UDP,
         0,
         HDR_UDP_Config,
         HDR_UDP_Init,
         HDR_UDP_Clone,
         HDR_UDP_GetField,
         HDR_UDP_ConfigField,
         HDR_UDP_ConfigAuto,
         HDR_UDP_ConfigHeader,
         HDR_UDP_Randomize,
         HDR_UDP_Serialize,
         HDR_UDP_GetHeaderSize,
         HDR_UDP_GetHeader,
         HDR_UDP_GetHeaderError
    },
    {
         e_HEADER_TYPE_IEEE1588_V1_EVENT_UDP,
         0,
         HDR_UDP_Config,
         HDR_UDP_Init,
         HDR_UDP_Clone,
         HDR_UDP_GetField,
         HDR_UDP_ConfigField,
         HDR_UDP_ConfigAuto,
         HDR_UDP_ConfigHeader,
         HDR_UDP_Randomize,
         HDR_UDP_Serialize,
         HDR_UDP_GetHeaderSize,
         HDR_UDP_GetHeader,
         HDR_UDP_GetHeaderError
    },
    {
         e_HEADER_TYPE_IEEE1588_V2_EVENT_UDP,
         0,
         HDR_UDP_Config,
         HDR_UDP_Init,
         HDR_UDP_Clone,
         HDR_UDP_GetField,
         HDR_UDP_ConfigField,
         HDR_UDP_ConfigAuto,
         HDR_UDP_ConfigHeader,
         HDR_UDP_Randomize,
         HDR_UDP_Serialize,
         HDR_UDP_GetHeaderSize,
         HDR_UDP_GetHeader,
         HDR_UDP_GetHeaderError
    },
    {
         e_HEADER_TYPE_IEEE1588_V1_MESSAGE,
         0,
         HDR_IEEE1588Message_Config,
         HDR_IEEE1588Message_Init,
         HDR_IEEE1588Message_Clone,
         HDR_IEEE1588Message_GetField,
         HDR_IEEE1588Message_ConfigField,
         HDR_IEEE1588Message_ConfigAuto,
         HDR_IEEE1588Message_ConfigHeader,
         HDR_IEEE1588Message_Randomize,
         HDR_IEEE1588Message_Serialize,
         HDR_IEEE1588Message_GetHeaderSize,
         HDR_IEEE1588Message_GetHeader,
         HDR_IEEE1588Message_GetHeaderError
    },
     {
         e_HEADER_TYPE_IEEE1588_V2_MESSAGE,
         0,
         HDR_IEEE1588Message_Config,
         HDR_IEEE1588Message_Init,
         HDR_IEEE1588Message_Clone,
         HDR_IEEE1588Message_GetField,
         HDR_IEEE1588Message_ConfigField,
         HDR_IEEE1588Message_ConfigAuto,
         HDR_IEEE1588Message_ConfigHeader,
         HDR_IEEE1588Message_Randomize,
         HDR_IEEE1588Message_Serialize,
         HDR_IEEE1588Message_GetHeaderSize,
         HDR_IEEE1588Message_GetHeader,
         HDR_IEEE1588Message_GetHeaderError
    },
    {
         e_HEADER_TYPE_IPHC,
         0,
         HDR_IPHC_Config,
         HDR_IPHC_Init,
         HDR_IPHC_Clone,
         HDR_IPHC_GetField,
         HDR_IPHC_ConfigField,
         HDR_IPHC_ConfigAuto,
         HDR_IPHC_ConfigHeader,
         HDR_IPHC_Randomize,
         HDR_IPHC_Serialize,
         HDR_IPHC_GetHeaderSize,
         HDR_IPHC_GetHeader,
         HDR_IPHC_GetHeaderError
    },
    {
         e_HEADER_TYPE_PPPoE,
         0,
         HDR_PPPoE_Config,
         HDR_PPPoE_Init,
         HDR_PPPoE_Clone,
         HDR_PPPoE_GetField,
         HDR_PPPoE_ConfigField,
         HDR_PPPoE_ConfigAuto,
         HDR_PPPoE_ConfigHeader,
         HDR_PPPoE_Randomize,
         HDR_PPPoE_Serialize,
         HDR_PPPoE_GetHeaderSize,
         HDR_PPPoE_GetHeader,
         HDR_PPPoE_GetHeaderError
    },
    {
         e_HEADER_TYPE_PPP,
         0,
         HDR_PPP_Config,
         HDR_PPP_Init,
         HDR_PPP_Clone,
         HDR_PPP_GetField,
         HDR_PPP_ConfigField,
         HDR_PPP_ConfigAuto,
         HDR_PPP_ConfigHeader,
         HDR_PPP_Randomize,
         HDR_PPP_Serialize,
         HDR_PPP_GetHeaderSize,
         HDR_PPP_GetHeader,
         HDR_PPP_GetHeaderError
    },
    {
        e_HEADER_TYPE_PPP_MLMC,
        0,
        HDR_PPPMLMC_Config,
        HDR_PPPMLMC_Init,
        HDR_PPPMLMC_Clone,
        HDR_PPPMLMC_GetField,
        HDR_PPPMLMC_ConfigField,
        HDR_PPPMLMC_ConfigAuto,
        HDR_PPPMLMC_ConfigHeader,
        HDR_PPPMLMC_Randomize,
        HDR_PPPMLMC_Serialize,
        HDR_PPPMLMC_GetHeaderSize,
        HDR_PPPMLMC_GetHeader,
        HDR_PPPMLMC_GetHeaderError
    },
    {
         e_HEADER_TYPE_HDLC,
         0,
         HDR_HDLC_Config,
         HDR_HDLC_Init,
         HDR_HDLC_Clone,
         HDR_HDLC_GetField,
         HDR_HDLC_ConfigField,
         HDR_HDLC_ConfigAuto,
         HDR_HDLC_ConfigHeader,
         HDR_HDLC_Randomize,
         HDR_HDLC_Serialize,
         HDR_HDLC_GetHeaderSize,
         HDR_HDLC_GetHeader,
         HDR_HDLC_GetHeaderError
    },
    {
        e_HEADER_TYPE_LLC,
        0,
        HDR_LLC_Config,
        HDR_LLC_Init,
        HDR_LLC_Clone,
        HDR_LLC_GetField,
        HDR_LLC_ConfigField,
        HDR_LLC_ConfigAuto,
        HDR_LLC_ConfigHeader,
        HDR_LLC_Randomize,
        HDR_LLC_Serialize,
        HDR_LLC_GetHeaderSize,
        HDR_LLC_GetHeader,
        HDR_LLC_GetHeaderError
    },
    {
        e_HEADER_TYPE_NLPID,
        0,
        HDR_NLPID_Config,
        HDR_NLPID_Init,
        HDR_NLPID_Clone,
        HDR_NLPID_GetField,
        HDR_NLPID_ConfigField,
        HDR_NLPID_ConfigAuto,
        HDR_NLPID_ConfigHeader,
        HDR_NLPID_Randomize,
        HDR_NLPID_Serialize,
        HDR_NLPID_GetHeaderSize,
        HDR_NLPID_GetHeader,
        HDR_NLPID_GetHeaderError
    },
    {
        e_HEADER_TYPE_SNAP,
        0,
        HDR_SNAP_Config,
        HDR_SNAP_Init,
        HDR_SNAP_Clone,
        HDR_SNAP_GetField,
        HDR_SNAP_ConfigField,
        HDR_SNAP_ConfigAuto,
        HDR_SNAP_ConfigHeader,
        HDR_SNAP_Randomize,
        HDR_SNAP_Serialize,
        HDR_SNAP_GetHeaderSize,
        HDR_SNAP_GetHeader,
        HDR_SNAP_GetHeaderError
    },
    {
        e_HEADER_TYPE_AAL0,
        0,
        HDR_AAL0Cell_Config,
        HDR_AAL0Cell_Init,
        HDR_AAL0Cell_Clone,
        HDR_AAL0Cell_GetField,
        HDR_AAL0Cell_ConfigField,
        HDR_AAL0Cell_ConfigAuto,
        HDR_AAL0Cell_ConfigHeader,
        HDR_AAL0Cell_Randomize,
        HDR_AAL0Cell_Serialize,
        HDR_AAL0Cell_GetHeaderSize,
        HDR_AAL0Cell_GetHeader,
        HDR_AAL0Cell_GetHeaderError
    },
    {
        e_HEADER_TYPE_RAW_DATA,
        0,
        HDR_Payload_Config,
        HDR_Payload_Init,
        HDR_Payload_Clone,
        HDR_Payload_GetField,
        HDR_Payload_ConfigField,
        HDR_Payload_ConfigAuto,
        HDR_Payload_ConfigHeader,
        HDR_Payload_Randomize,
        HDR_Payload_Serialize,
        HDR_Payload_GetHeaderSize,
        HDR_Payload_GetHeader,
        HDR_Payload_GetHeaderError
    },
    {
        e_HEADER_TYPE_RTP,
        0,
        HDR_RTP_Config,
        HDR_RTP_Init,
        HDR_RTP_Clone,
        HDR_RTP_GetField,
        HDR_RTP_ConfigField,
        HDR_RTP_ConfigAuto,
        HDR_RTP_ConfigHeader,
        HDR_RTP_Randomize,
        HDR_RTP_Serialize,
        HDR_RTP_GetHeaderSize,
        HDR_RTP_GetHeader,
        HDR_RTP_GetHeaderError
    },
    {
         e_HEADER_TYPE_PWE3_ATM_MPLS_DEMUX,
         0,
        HDR_MPLS_Config,
        HDR_MPLS_Init,
        HDR_MPLS_Clone,
        HDR_MPLS_GetField,
        HDR_MPLS_ConfigField,
        HDR_MPLS_ConfigAuto,
        HDR_MPLS_ConfigHeader,
        HDR_MPLS_Randomize,
        HDR_MPLS_Serialize,
        HDR_MPLS_GetHeaderSize,
         HDR_MPLS_GetHeader,
         HDR_MPLS_GetHeaderError
    },
    {
         e_HEADER_TYPE_PWE3_ATM_ENCAP,
         0,
         HDR_PWE3Encap_Config,
         HDR_PWE3Encap_Init,
         HDR_PWE3Encap_Clone,
         HDR_PWE3Encap_GetField,
         HDR_PWE3Encap_ConfigField,
         HDR_PWE3Encap_ConfigAuto,
         HDR_PWE3Encap_ConfigHeader,
         HDR_PWE3Encap_Randomize,
         HDR_PWE3Encap_Serialize,
         HDR_PWE3Encap_GetHeaderSize,
         HDR_PWE3Encap_GetHeader,
         HDR_PWE3Encap_GetHeaderError
    },
    {
         e_HEADER_TYPE_PWE3_SAToP_MPLS_DEMUX,
         0,
        HDR_MPLS_Config,
        HDR_MPLS_Init,
        HDR_MPLS_Clone,
        HDR_MPLS_GetField,
        HDR_MPLS_ConfigField,
        HDR_MPLS_ConfigAuto,
        HDR_MPLS_ConfigHeader,
        HDR_MPLS_Randomize,
        HDR_MPLS_Serialize,
        HDR_MPLS_GetHeaderSize,
         HDR_MPLS_GetHeader,
         HDR_MPLS_GetHeaderError
    },
    {
         e_HEADER_TYPE_PWE3_SAToP_ENCAP,
         0,
         HDR_PWE3Encap_Config,
         HDR_PWE3Encap_Init,
         HDR_PWE3Encap_Clone,
         HDR_PWE3Encap_GetField,
         HDR_PWE3Encap_ConfigField,
         HDR_PWE3Encap_ConfigAuto,
         HDR_PWE3Encap_ConfigHeader,
         HDR_PWE3Encap_Randomize,
         HDR_PWE3Encap_Serialize,
         HDR_PWE3Encap_GetHeaderSize,
         HDR_PWE3Encap_GetHeader,
         HDR_PWE3Encap_GetHeaderError
    },
    {
         e_HEADER_TYPE_TUNNELED_IPv4,
         0,
         HDR_IPv4_Config,
         HDR_IPv4_Init,
         HDR_IPv4_Clone,
         HDR_IPv4_GetField,
         HDR_IPv4_ConfigField,
         HDR_IPv4_ConfigAuto,
         HDR_IPv4_ConfigHeader,
         HDR_IPv4_Randomize,
         HDR_IPv4_Serialize,
         HDR_IPv4_GetHeaderSize,
         HDR_TunnelledIPv4_GetHeader,
         HDR_IPv4_GetHeaderError
    },
    {
       e_HEADER_TYPE_TUNNELED_IPv6,
       0,
       HDR_IPv6_Config,
       HDR_IPv6_Init,
       HDR_IPv6_Clone,
       HDR_IPv6_GetField,
       HDR_IPv6_ConfigField,
       HDR_IPv6_ConfigAuto,
       HDR_IPv6_ConfigHeader,
       HDR_IPv6_Randomize,
       HDR_IPv6_Serialize,
       HDR_IPv6_GetHeaderSize,
        HDR_TunnelledIPv6_GetHeader,
        HDR_IPv6_GetHeaderError
    },
    {
         e_HEADER_TYPE_PADDING,
         0,
         HDR_Padding_Config,
         HDR_Padding_Init,
         HDR_Padding_Clone,
         HDR_Padding_GetField,
         HDR_Padding_ConfigField,
         HDR_Padding_ConfigAuto,
         HDR_Padding_ConfigHeader,
         HDR_Padding_Randomize,
         HDR_Padding_Serialize,
         HDR_Padding_GetHeaderSize,
         HDR_Padding_GetHeader,
         HDR_Padding_GetHeaderError
    }
};
/**************************************************************************//**
 @Group          InternalFrame

 @Description    Internal Frame Functions.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group          Frame

 @Description    Frame Functions

 @{
*//***************************************************************************/
t_Error INTERNAL_StartFrameFactory()
{
     /* Check the registered headers array */
     if (INTERNAL_ValidateHeaderArray(pRegisteredHeaderArray) != E_OK)
         return E_INVALID_STATE;

     /* Initialize the 32-bit crc table */
     INTERNAL_InitCRC32Table();

     /* Initialize the 8-bit CRC table */
     INTERNAL_InitCRC8Table();

     /* Register the header dependent dynamic values association*/
     INTERNAL_RegisterFieldAssoc(e_HEADER_TYPE_ETH, e_FIELD_ETH_TYPE, e_HEADER_TYPE_VLAN, 0x8100);
     INTERNAL_RegisterFieldAssoc(e_HEADER_TYPE_VLAN, e_FIELD_VLAN_TYPE, e_HEADER_TYPE_VLAN, 0x8100);

     return E_OK;
}

t_Error INTERNAL_StopFrameFactory()
{
     /* Free the header dependent dynamic values association */
     INTERNAL_FreeFieldAssocArray(e_HEADER_TYPE_PAYLOAD);
     INTERNAL_FreeFieldAssocArray(e_HEADER_TYPE_ETH);
     INTERNAL_FreeFieldAssocArray(e_HEADER_TYPE_VLAN);

     /* All clear */
     return E_OK;
}

t_Error INTERNAL_RegisterFieldAssoc(e_PgHeaderType currentHeaderID, uint32_t fieldID, e_PgHeaderType nextHeaderID, uint32_t value)
{
     t_Assoc    **p_Object = NULL, *pNewObject = NULL;
    uint64_t    ID;

     /* Validate the input fields */
     if ( (currentHeaderID < e_HEADER_TYPE_NONE) || (currentHeaderID > e_MAX_HEADER_TYPE_COUNT) ||
         (nextHeaderID < e_HEADER_TYPE_NONE) || (nextHeaderID > e_MAX_HEADER_TYPE_COUNT))
     {
         RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid header type"));
     }

     /* Get our hands on the array owned by currentHeaderID */
     p_Object = &pRegisteredHeaderArray[currentHeaderID].FieldAssoc;

     if (!*p_Object)
     {
         /* This is the first node - it will not be used to store info, but just as a list begining*/
         *p_Object = MEMALLOC(sizeof(t_Assoc));
         if (!*p_Object)
         {
             RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Could not register the field association"));
         }
         memset(*p_Object, 0, sizeof(t_Assoc));
         (*p_Object)->ID = 0;
         (*p_Object)->value = 0;
         (*p_Object)->node.p_Next = (*p_Object)->node.p_Prev = 0;
     }

     /* The lookup key is filedID+nextHeaderID */
     ID = (((uint64_t)fieldID<<32) | (uint32_t)nextHeaderID);

     /* Create the new node to be inserted */
     /* The function does not check for duplicates!!!*/
     pNewObject = MEMALLOC(sizeof(t_Assoc));
     if (!pNewObject)
     {
         RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Could not register the field association"));
     }
     memset(pNewObject, 0, sizeof(t_Assoc));
     pNewObject->node.p_Next = pNewObject->node.p_Prev = 0;
     pNewObject->ID = ID;
     pNewObject->value = value;

     /* Insert the new node in the list*/
     return INTERNAL_AddToTail(&(*p_Object)->node, &pNewObject->node);
}

uint32_t INTERNAL_RetrieveFieldAssoc(e_PgHeaderType currentHeaderID, uint32_t fieldID, e_PgHeaderType nextHeaderID)
{
     t_Assoc* p_Object = NULL;
     t_Assoc* hNextObject = NULL;
     uint64_t ID;

     /* Validate the input fields */
     if ( (currentHeaderID < e_HEADER_TYPE_NONE) || (currentHeaderID > e_MAX_HEADER_TYPE_COUNT) ||
         (nextHeaderID < e_HEADER_TYPE_NONE) || (nextHeaderID > e_MAX_HEADER_TYPE_COUNT))
     {
         REPORT_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid header type"));
         return 0;
     }

     /* Get our hands on the array owned by currentHeaderID */
     p_Object = pRegisteredHeaderArray[currentHeaderID].FieldAssoc;

     if (!p_Object)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid associtative array"));
         return 0;
     }

     /* The lookup key is filedID+nextHeaderID */
     ID = (((uint64_t)fieldID<<32) | (uint32_t)nextHeaderID);

     /* Try to find the desired value */
     while (p_Object)
     {
         if (p_Object->ID == ID)
             return p_Object->value;

         hNextObject = (t_Assoc*)INTERNAL_GetNextNode(&p_Object->node);
         p_Object = hNextObject ? (t_Assoc*)LIST_OBJECT(&hNextObject->node, t_Assoc, node) : NULL;
     }

     REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Could not find the desired association! Please check INTERNAL_StartFrameFactory() settings "));
     return 0;
}

t_Error INTERNAL_FreeFieldAssocArray(e_PgHeaderType HeaderID)
{
     t_Assoc* p_Object = NULL, *pAuxObject = NULL;
     t_Handle hNextObject = NULL;

     /* Validate the input fields */
     if ( (HeaderID < e_HEADER_TYPE_NONE) || (HeaderID > e_MAX_HEADER_TYPE_COUNT) )
     {
         RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid header type"));
     }

     /* Get our hands on the array owned by HeaderID */
     p_Object = pRegisteredHeaderArray[HeaderID].FieldAssoc;

     /* Delete every object in the list */
     while (p_Object)
     {
         /* Get the handle to the next object (if any) */
         hNextObject = (t_Assoc*)INTERNAL_GetNextNode(&p_Object->node);
         pAuxObject = hNextObject ? (t_Assoc*)LIST_OBJECT(hNextObject, t_Assoc, node) : NULL;

         /* Free the current object */
         MEMFREE(p_Object);

         /* Move on to the next one */
         p_Object = pAuxObject;
     }

     pRegisteredHeaderArray[HeaderID].FieldAssoc = NULL;

     return E_OK;
}

t_Error INTERNAL_ValidateHeaderArray(t_HeaderInfo* pArray)
{
     int i=0;

     if (pArray == NULL)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid registered header types array"));
     }

     for (i = 1; i < e_MAX_HEADER_TYPE_COUNT; i++)
     {
         if (pArray[i].HeaderID != i)
         {
             RETURN_ERROR(MAJOR, E_CONFLICT, ("The registered header types array does not match the e_PgHeaderType enumeration!"));
         }
     }

     return E_OK;
}

t_Error INTERNAL_SerializeValue(uint8_t* pMember, uint32_t memberSize, uint8_t** p_Buffer, uint32_t *sizeRemaining, uint8_t endian)
{
     UNUSED(endian);
     /* Internal use only - assume the params are valid */
     if ((*sizeRemaining -= memberSize) >= 0)
     {
     /* This should not be set on a little endian machine */
#ifndef _BIG_ENDIAN_
         /* Check if endian convertion is required */
         if ((endian == CONVERT_ENDIANNESS))
         {
             switch (memberSize)
             {
                 case 2:
                     SwapCopyUint16((uint16_t*)pMember, (uint16_t*)*p_Buffer);
                     *p_Buffer += memberSize;
                     return E_OK;
                 case 4:
                     SwapCopyUint32((uint32_t*)pMember, (uint32_t*)*p_Buffer);
                     *p_Buffer += memberSize;
                     return E_OK;
                 default:
                     /* This settings do not match: convertion is possible only on shorts and longs */
                     break;
             }
         }
 #endif
         /* Do not convert endianness */
         memcpy(*p_Buffer, pMember, memberSize);
         *p_Buffer += memberSize;
     }
     else
     {
         RETURN_ERROR(MAJOR, E_NOT_IN_RANGE, ("Serialize value failed: buffer to small"));
     }

     return E_OK;
}

t_Error INTERNAL_SerializeNextHeader(t_Handle h_Frame, uint8_t **p_Buffer, uint32_t* pSize)
{
     t_FrameStruct *p_Frame = (t_FrameStruct*)h_Frame;
     t_FrameStruct *p_NextHeader = NULL;

     if (!p_Frame || !pSize || !p_Buffer)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(h_Frame);
     if (p_NextHeader)
     {
         /* Serialize all other headers except IPHC. IPHC will be serialized only
            when all CRC, checksum (Eg. SCTP, UDP checksum) is serialized in the frame. */
         if(p_NextHeader->HeaderID == e_HEADER_TYPE_IPHC)
         {
            p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(p_NextHeader);
            if(!p_NextHeader)
            {
                /* Return if nextHeader following HEADER_TYPE_IPHC is NULL */
                return E_OK;
            }
         }

         if (pRegisteredHeaderArray[p_NextHeader->HeaderID].pfnSerializeHeader(p_NextHeader, p_Buffer, pSize) != E_OK)
             return E_NOT_IN_RANGE;
     }

     return E_OK;
}

t_Error INTERNAL_ConfigPayloadType(t_Handle h_Frame, e_PgPayloadType type, t_Handle hParams)
{
     t_FrameStruct *p_Header = (t_FrameStruct*)h_Frame;
     t_Payload *p_Payload = NULL;

     if (!p_Header)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     p_Payload = (t_Payload*)p_Header->h_HeaderStruct;
     p_Payload->payloadType = type;

     ASSERT_COND(hParams);

     switch (type)
     {
         case e_PAYLOAD_PRBS:
         {
             /* Initialize the random payload field values */
             p_Payload->polynomial = ((t_PgPayloadPrbs*)hParams)->polynomial;
             p_Payload->numBits = ((t_PgPayloadPrbs*)hParams)->numBits;
             p_Payload->initVector = ((t_PgPayloadPrbs*)hParams)->initVector;

             if (!p_Payload->initVector)
             {
                /* Take different value for every compilation */
                p_Payload->initVector = (uint32_t)
                (
                    ((__DATE__[4]-'0')<<28)|((__DATE__[5]-'0')<<24) /* Day    */
                                           |
                    ((__TIME__[0]-'0')<<20)|((__TIME__[1]-'0')<<16) /* Hour   */
                                           |
                    ((__TIME__[3]-'0')<<12)|((__TIME__[4]-'0')<< 8) /* Minute */
                                           |
                    ((__TIME__[6]-'0')<< 4)|((__TIME__[7]-'0')<< 0) /* Second */
                );
             }
         }
         break;
         case e_PAYLOAD_USER:
         {
             /* Initialize the user defined payload fields */
             return FRAME_SetHeaderField(h_Frame, h_Frame, e_FIELD_PAYLOAD_BUFFER, ((t_PgPayloadUser*)hParams)->p_Buffer, ((t_PgPayloadUser*)hParams)->size);
         }
         break;
         case e_PAYLOAD_PATTERN:
         {
             /* Initialize the payload pattern fields */
             p_Payload->startVal = ((t_PgPayloadPattern*)hParams)->startVal;
             p_Payload->stepVal = ((t_PgPayloadPattern*)hParams)->stepVal;
             p_Payload->stepCount = ((t_PgPayloadPattern*)hParams)->stepCount;
         }
         break;
         case e_PAYLOAD_FIXED_PATTERN:
         {
             p_Payload->patternSize = ((t_PgPayloadFixedPattern*)hParams)->patternSize;
             memcpy(p_Payload->patternValue,(((t_PgPayloadFixedPattern*)hParams)->patternValue),p_Payload->patternSize);
         }
         break;
         default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid payload type!"));
         }
     }

     return E_OK;
}

 /* This function returns the size of the frame delimited by the StartFrame and StopFrame */
uint32_t INTERNAL_GetFrameSize(t_Handle hStartHeader, t_Handle hStopHeader)
{
     t_FrameStruct *pCrtHeader = NULL;
     uint32_t size = 0;

     if (!hStartHeader && !hStopHeader)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("You must provide at least the start or stop header!"));
         return 0;
     }

     /* If the user did not specify the start header, use the first header in the frame*/
     if (!hStartHeader)
         hStartHeader = INTERNAL_GetFirstHeader(hStopHeader);
     /* If the user did not specify the last header, use the last header in the frame*/
     if (!hStopHeader)
         hStopHeader = INTERNAL_GetLastHeader(hStartHeader);

     pCrtHeader = (t_FrameStruct*)hStartHeader;
     while (pCrtHeader)
     {
         size += pRegisteredHeaderArray[pCrtHeader->HeaderID].pfnGetHeaderSize(pCrtHeader);
         if(pCrtHeader != (t_FrameStruct*)hStopHeader)
            pCrtHeader = INTERNAL_GetNextHeader(pCrtHeader);
         else
            break;
     }

     return size;
}

t_Error INTERNAL_Frame_Free(t_Handle h_Frame, uint8_t recurse)
{
     t_FrameStruct*  p_Frame = (t_FrameStruct*)h_Frame;
     t_Payload*      p_Payload = NULL;
     t_Handle        hNextHeader = NULL;

     t_PgPaddingHeader *p_PaddingHeader  = NULL;         /* Padding header pointer */

     if (p_Frame)
     {
         hNextHeader = INTERNAL_GetNextHeader(h_Frame);

         /* apply the recursive destruction of the rest of the frame */
         if (recurse && hNextHeader)
             INTERNAL_Frame_Free(hNextHeader, recurse);

         /* delete the current header/frame */
         if (p_Frame->h_HeaderStruct)
         {
             /*if the frame type is payload - free the buffer */
             if ((p_Frame->HeaderID == e_HEADER_TYPE_PAYLOAD) ||
             (p_Frame->HeaderID == e_HEADER_TYPE_RAW_DATA))
             {
                 p_Payload = (t_Payload*)p_Frame->h_HeaderStruct;
                 if (p_Payload->p_Data)
                {
                     MEMFREE(p_Payload->p_Data);
                    p_Payload->p_Data = NULL;
                }
             }
             /*if the frame type is padding - free the shim header */
             if (p_Frame->HeaderID == e_HEADER_TYPE_PADDING)
             {
                 p_PaddingHeader = (t_PgPaddingHeader*)p_Frame->h_HeaderStruct;
                 if (p_PaddingHeader->p_ShimHeader)
                {
                     MEMFREE(p_PaddingHeader->p_ShimHeader);
                    p_PaddingHeader->p_ShimHeader = NULL;
                }
             }
             /* if the frame type is padding - reset the padIndex counter to 0 */
             if ((p_Frame->HeaderID == e_HEADER_TYPE_PADDING))
             {
                 padIndex = 0;
             }
             /*if the frame type is LLC  - reset the llcIndex counter to 0 */
             if ((p_Frame->HeaderID == e_HEADER_TYPE_LLC))
             {
                llcIndex = 0;
             }
             /*if the frame type is SNAP - reset the snapIndex counter to 0 */
             if ((p_Frame->HeaderID == e_HEADER_TYPE_SNAP))
             {
                snapIndex = 0;
             }

             MEMFREE(p_Frame->h_HeaderStruct);
             p_Frame->h_HeaderStruct = NULL;
         }

         p_Frame->node.p_Next = p_Frame->node.p_Prev = NULL;

         /* delete the frame itself */
         MEMFREE(p_Frame);
     }

     return E_OK;
}

t_Error INTERNAL_InsertAfterNode(t_Handle parentNode, t_Handle childList)
{
     t_List *pParentNode = (t_List*)parentNode;
     t_List *pChildList = (t_List*)childList;
     t_List *pParentNextNode = NULL;
     t_List *pChildLastNode = NULL;

     if (!(parentNode && childList))
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));

     pParentNextNode = INTERNAL_GetNextNode(pParentNode);
     pChildLastNode = INTERNAL_GetLastNode(pChildList);

     /* If this is part of a bigger frame truncate it */
     if (pChildList->p_Prev)
         pChildList->p_Prev->p_Next = NULL;

     pParentNode->p_Next = pChildList;
     pChildList->p_Prev = pParentNode;
     pChildLastNode->p_Next =  pParentNextNode;
     if (pParentNextNode)
         pParentNextNode->p_Prev = pChildLastNode;

     return E_OK;
}

t_Error INTERNAL_InsertBeforeNode(t_Handle parentNode, t_Handle childList)
{
     t_List *pParentNode = (t_List*)parentNode;
     t_List *pChildList  = (t_List*)childList;
     t_List *pParentPrevNode = NULL;
     t_List *pChildLastNode  = NULL;

     if (!(parentNode && childList))
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));

     pParentPrevNode = INTERNAL_GetPrevNode(pParentNode);
     pChildLastNode = INTERNAL_GetLastNode(pChildList);

     /* If this is part of a bigger frame truncate it */
     if (pChildList->p_Prev)
         pChildList->p_Prev->p_Next = NULL;

     pParentNode->p_Prev = pChildLastNode;
     pChildLastNode->p_Next = pParentNode;
     pChildList->p_Prev = pParentPrevNode;
     if (pParentPrevNode)
         pParentPrevNode->p_Next = pChildList;

     return E_OK;
}

t_Error INTERNAL_AddToTail(t_Handle parentList, t_Handle childList)
{
     t_List *pParentList = (t_List*)parentList;
     t_List *pChildList  = (t_List*)childList;
     t_List *pParentLastNode = NULL;

     if (!(parentList && childList))
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));

     pParentLastNode = INTERNAL_GetLastNode(pParentList);
     return INTERNAL_InsertAfterNode(pParentLastNode, pChildList);
}

t_Error INTERNAL_RemoveNode(t_Handle hNode)
{
     t_List *pNode = hNode;
     t_List *pPrevNode = NULL;
     t_List *pNextNode = NULL;

     if (!hNode)
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));

     pPrevNode = INTERNAL_GetPrevNode(hNode);
     pNextNode = INTERNAL_GetNextNode(hNode);

     if (pPrevNode)
         pPrevNode->p_Next = pNextNode;
     if (pNextNode)
         pNextNode->p_Prev = pPrevNode;

     pNode->p_Prev = pNode->p_Next = NULL;

     return E_OK;
}

t_Handle INTERNAL_GetLastNode(t_Handle hNode)
{
     t_List *pList = (t_List*)hNode;

     while (pList)
     {
         if (pList->p_Next)
             pList = pList->p_Next;
         else
             return pList;
     }
     return NULL;
 }

t_Handle INTERNAL_GetFirstNode(t_Handle hNode)
{
     t_List *pList = (t_List*)hNode;

     while (pList)
     {
         if (pList->p_Prev)
             pList = pList->p_Prev;
         else
             return pList;
     }
     return NULL;
}

t_Handle INTERNAL_GetNextNode(t_Handle hNode)
{
     t_List *pList = (t_List*)hNode;
     return pList->p_Next;
}

t_Handle INTERNAL_GetPrevNode(t_Handle hNode)
{
     t_List *pList = (t_List*)hNode;
     return pList->p_Prev;
}

t_Error INTERNAL_DebugPrintChain(t_Handle hNode)
{
     t_List* pNode = (t_List*)hNode;

     if (!pNode)
     {
         XX_Print("End of chain\n");
         return E_OK;
     }

     XX_Print("Prev: %d Me: %d Next: %d\n", pNode->p_Prev, pNode, pNode->p_Next);
     return INTERNAL_DebugPrintChain(pNode->p_Next);
}

t_Handle INTERNAL_GetNextHeader(t_Handle h_Header)
{
     t_FrameStruct* p_Frame = (t_FrameStruct*)h_Header;
     return (!p_Frame || !p_Frame->node.p_Next) ? NULL : (t_Handle)LIST_OBJECT(p_Frame->node.p_Next, t_FrameStruct, node);
}

t_Handle INTERNAL_GetPrevHeader(t_Handle h_Header)
{
     t_FrameStruct* p_Frame = (t_FrameStruct*)h_Header;
     return (!p_Frame || !p_Frame->node.p_Prev) ? NULL : (t_Handle)LIST_OBJECT(p_Frame->node.p_Prev, t_FrameStruct, node);
}

t_Handle INTERNAL_GetLastHeader(t_Handle h_Header)
{
     t_FrameStruct* p_Frame = (t_FrameStruct*)h_Header;
     t_Handle       hNextFrame = INTERNAL_GetNextHeader(p_Frame);

     return hNextFrame ? INTERNAL_GetLastHeader(hNextFrame) : h_Header;
}

t_Handle INTERNAL_GetFirstHeader(t_Handle h_Frame)
{
     t_Handle       hPrevFrame = INTERNAL_GetPrevHeader(h_Frame);

     return hPrevFrame ? INTERNAL_GetFirstHeader(hPrevFrame) : h_Frame;
}

uint32_t INTERNAL_ComputeCkSum(uint8_t *p_Buffer, uint32_t size, uint32_t offset, bool b16bitFold)
{
     uint32_t  i;
     uint16_t  word16 = 0;
     uint32_t  ckSum = offset;
     uint32_t  iterations = 0;

     /* Checksum is performed only on 16bit alligned buffers*/
     iterations = size-(size%2);

     /* Accumultate the buffer values into the ckSum filed */
     for(i=0; i<iterations; i+=2)
     {
         word16 = (uint16_t)((((uint16_t)p_Buffer[i] << 8) & 0xFF00) | ((uint16_t)p_Buffer[i+1] & 0x00FF));
         ckSum += (uint32_t)word16;
         //XX_Print(" %X - %X", p_Buffer[i], p_Buffer[i+1]);
         //ckSum += (uint32_t)((p_Buffer[i]/*<<8*/& 0xFF00) | p_Buffer[i+1]);
     }

     /* Add an extra 0 as padding to the buffer - to be 16bit alligned */
     if (size%2)
     {
         word16 = (uint16_t)((((uint16_t)p_Buffer[i] << 8) & 0xFF00) | ((uint16_t)0 & 0x00FF));
         ckSum += (uint32_t)word16;
     }

     if (!b16bitFold)
         return ckSum;

     /*  Fold th 32-bit sum to 16 bits */
     while(ckSum>>16)
         ckSum = (ckSum & 0xFFFF) + (ckSum >> 16);

     return (~ckSum);
}

uint32_t INTERNAL_ComputeCRC(uint32_t crcAccum, uint8_t *p_Buffer, uint32_t size)
{
     /* Returns the CRC of a buffer defined by it's starting point and size */
     uint8_t   *tmp_data_ptr;
     uint8_t   tmp_crc[4];
     int       i;
     uint32_t  crc32;

     tmp_data_ptr = (uint8_t *)XX_Malloc(size*sizeof(uint8_t));
     if(tmp_data_ptr==NULL)
     {
         XX_Print("ERROR cannot allocate enough memory for calc_crc_wrapper\n");
         XX_Exit(1);
     }

     for(i=0;i< size;i++) tmp_data_ptr[i] = INTERNAL_ReverseBit(p_Buffer[i]);

     crc32 = INTERNAL_MakeCRC(crcAccum, tmp_data_ptr, size);

     tmp_crc[0] = (uint8_t)(crc32 & 0xff);
     tmp_crc[1] = (uint8_t)((crc32 >> 8) & 0xff);
     tmp_crc[2] = (uint8_t)((crc32 >> 16) & 0xff);
     tmp_crc[3] = (uint8_t)((crc32 >> 24) & 0xff);
     tmp_crc[0] = INTERNAL_ReverseBit(tmp_crc[0]);
     tmp_crc[1] = INTERNAL_ReverseBit(tmp_crc[1]);
     tmp_crc[2] = INTERNAL_ReverseBit(tmp_crc[2]);
     tmp_crc[3] = INTERNAL_ReverseBit(tmp_crc[3]);
     crc32 = (uint32_t)tmp_crc[3]<<24;
     crc32 |= (uint32_t)tmp_crc[2]<<16;
     crc32 |= (uint32_t)tmp_crc[1]<<8;
     crc32 |= (uint32_t)tmp_crc[0];

    XX_Free(tmp_data_ptr);
     return(crc32);
}

uint16_t INTERNAL_Compute16BitCRC(uint16_t crcAccum, uint8_t *p_Buffer, uint32_t size)
{
     uint32_t i, j;
     uint8_t  data;
     for (j=0; j<size; j++)
     {
         data = *p_Buffer++;
         for (i = 0; i < 8; i++, data >>= 1)
         {
             if ((crcAccum ^ data) & 0x1)
             {
                 crcAccum >>= 1;
                 crcAccum ^= CRC16_POLYNOMIAL;
             }
             else
             {
                 crcAccum >>= 1;
             }
         }
     }

     return crcAccum;
}


uint32_t INTERNAL_MakeCRC(uint32_t crcAccum, uint8_t *p_Buffer, uint32_t size)
{
     /* Used by INTERNAL_ComputeCRC to compute the CRC of a buffer */
     uint32_t i, j;

     for (j=0; j<size; j++)
     {
         i = ((uint32_t) (crcAccum >> 24) ^ *p_Buffer++) & 0xFF;
         crcAccum = (crcAccum << 8) ^ crc32Table[i];
     }
     crcAccum = ~crcAccum;

     return crcAccum;
}

uint8_t INTERNAL_ReverseBit(uint8_t in_byte)
{
     uint8_t out_byte = 0, tmp_byte;
     int i;

     for(i=0; i<8; i++)
     {
         tmp_byte = (uint8_t)(in_byte & 1);
         in_byte = (uint8_t)(in_byte >> 1);
         out_byte |= tmp_byte;
         if(i!=7) out_byte = (uint8_t)(out_byte<< 1);
     }
     return(out_byte);
}

t_Error INTERNAL_InitCRC32Table()
{
     uint16_t i, j;
     uint32_t crc_accum;

     for (i=0;  i<256;  i++)
     {
         crc_accum = ( (uint32_t) i << 24 );
         for ( j = 0;  j < 8;  j++ )
         {
             if ( crc_accum & 0x80000000L )
             crc_accum = (crc_accum << 1) ^ POLYNOMIAL;
             else
             crc_accum = (crc_accum << 1);
         }
         crc32Table[i] = crc_accum;
     }
     return E_OK;
}

/* Generate a table for 8-bit CRC calculation on the polynomial x^8 + x^2 + x^1 + x^0 */
t_Error INTERNAL_InitCRC8Table()
{
    uint32_t i = 0, j = 0;
    uint32_t poly = 0, c = 0;

    /* Terms of polynomial defining this crc (except x^8): */
    static uint8_t p[] = {0,1,2};

    /* Make exclusive-or pattern from polynomial (0x07) */
    for ( i = 0; i < sizeof(p) / sizeof(uint8_t); i++ )
    {
        poly |= 1L << p[i];
    }

    for ( i = 0; i < CRC_TABLE_SIZE; i++ )
    {
        c = i;
        for ( j = 0; j < 8; j++ ) {
            c = (c & 0x80) ? poly ^ (c << 1) : (c << 1);
        }
        crc8Table[i] = (uint8_t) c;
    }

    return E_OK;
}

uint8_t INTERNAL_ComputeHEC(uint8_t *p_Buffer, uint32_t size)
{
    uint8_t crc = 0;
    uint8_t *tmp_data_ptr = p_Buffer;

    while( size-- ) {
        crc = crc8Table[crc ^ *tmp_data_ptr++];
    }

    return crc;
}

void INTERNAL_ResetMultipleHeaderCounters(void)
{
    padIndex      = 0;        /* Number of Padding headers encountered */
    llcIndex      = 0;        /* Number of LLC  headers encountered */
    snapIndex     = 0;        /* Number of SNAP headers encountered */
}

/** @} */ /* end of Frame group */

/**************************************************************************//**
@Group          Header

@Description    Header functions

@{
*//***************************************************************************/

 /**************************************************************************//**
@Group          Ethernet

@Description    Ethernet functions

@{
*//***************************************************************************/
t_Handle HDR_Eth_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
     t_Ethernet       *p_Header = NULL;
     UNUSED(p_ProtocolInfo);

     /* Check the header handle */
     if (!hHeader)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
         return NULL;
     }

     p_Header = (t_Ethernet*)MEMALLOC(sizeof(t_Ethernet));
     if (!p_Header)
     {
         REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
         return NULL;
     }
     memset(p_Header, 0, sizeof(t_Ethernet));

     /* Configure the auto flags */
     FLAG_SET(p_Header->autoFlags, e_FIELD_ETH_LENGTH | e_FIELD_ETH_TYPE | e_FIELD_ETH_FINAL_CKSUM);

     /* Initialize the user configured Ethernet header fields in the header */
     memcpy(&p_Header->data, hHeader, sizeof(t_PgEthernetHeader));

     if((p_Header->data.validFields & e_FIELD_ETH_USE_FINAL_CKSUM) &&
     !(p_Header->data.bUseFinalCkSum) && p_Header->data.finalCheckSum)
     {
         REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Invalid FCS configuration"));
         return NULL;
     }

     return (t_Handle)p_Header;
}

t_Handle HDR_Eth_Clone(t_Handle h_Object)
{
     t_Ethernet* p_Object = (t_Ethernet*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
     t_Ethernet* p_Header = (t_Ethernet*)MEMALLOC(sizeof(t_Ethernet));

     if (!p_Header)
     {
         REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
         return NULL;
     }

     memset(p_Header, 0, sizeof(t_Ethernet));

     /* Take a copy of the Ethernet header */
     memcpy(p_Header,p_Object, sizeof(t_Ethernet));

     return p_Header;
}

t_Error HDR_Eth_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
     t_Ethernet *p_Header = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

     if (!(p_Header && h_Value && size))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     /* Get the value of the field specified by fieldID */
     switch (fieldID)
     {
         case e_FIELD_ETH_DA:
             SAFE_COPY_VALUE(p_Header->data.da, h_Value, ETH_ADDR_SIZE, size);
         break;
         case e_FIELD_ETH_SA:
             SAFE_COPY_VALUE(p_Header->data.sa, h_Value, ETH_ADDR_SIZE, size);
         break;
         case e_FIELD_ETH_LENGTH:
         case e_FIELD_ETH_TYPE:
             SAFE_COPY_VALUE(&p_Header->data.typeLen, h_Value, sizeof(p_Header->data.typeLen), size);
         break;
         case e_FIELD_ETH_USE_FINAL_CKSUM:
             SAFE_COPY_VALUE(&p_Header->data.bUseFinalCkSum, h_Value, sizeof(p_Header->data.bUseFinalCkSum), size);
         break;
         case e_FIELD_ETH_FINAL_CKSUM:
             SAFE_COPY_VALUE(&p_Header->data.finalCheckSum, h_Value, sizeof(p_Header->data.finalCheckSum), size);
         break;
         case e_FIELD_ETH_PADDING:
             SAFE_COPY_VALUE(&p_Header->data.padding, h_Value, sizeof(p_Header->data.padding), size);
         break;
         case e_FIELD_ETH_NEXT_HEADER_TYPE:
             SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
         break;
         case e_FIELD_ETH_VALID_FIELDS:
             SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
         break;
         default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
         }
     }

     return E_OK;
}

t_Error HDR_Eth_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
     t_Ethernet* p_Header = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

     if (!p_Header || !h_Value)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     size = (size) ? size : (uint32_t)(-1);

     /* Configure the value of the field specified by fieldID */
     switch (fieldID)
     {
         case e_FIELD_ETH_DA:
             SAFE_COPY_VALUE(h_Value, p_Header->data.da, ETH_ADDR_SIZE, size);
         break;
         case e_FIELD_ETH_SA:
             SAFE_COPY_VALUE(h_Value, p_Header->data.sa, ETH_ADDR_SIZE, size);
         break;
         case e_FIELD_ETH_LENGTH:
         {
             SAFE_COPY_VALUE(h_Value, &p_Header->data.typeLen, sizeof(p_Header->data.typeLen), size);
             p_Header->bIsType = FALSE;
             FLAG_CLEAR(p_Header->autoFlags, fieldID);
         }
         break;
         case e_FIELD_ETH_TYPE:
         {
             SAFE_COPY_VALUE(h_Value, &p_Header->data.typeLen, sizeof(p_Header->data.typeLen), size);
             p_Header->bIsType = TRUE;
             FLAG_CLEAR(p_Header->autoFlags, fieldID);
         }
         break;
         case e_FIELD_ETH_USE_FINAL_CKSUM:
             SAFE_COPY_VALUE(h_Value, &p_Header->data.bUseFinalCkSum, sizeof(p_Header->data.bUseFinalCkSum), size);
         break;
         case e_FIELD_ETH_FINAL_CKSUM:
         {/* The final checksum is not used unless the user sets it to auto or sets its value*/
             SAFE_COPY_VALUE(h_Value, &p_Header->data.finalCheckSum, sizeof(p_Header->data.finalCheckSum), size);
             FLAG_CLEAR(p_Header->autoFlags, fieldID);
             p_Header->data.bUseFinalCkSum = TRUE;
         }
         break;
         case e_FIELD_ETH_PADDING:
             SAFE_COPY_VALUE(h_Value, &p_Header->data.padding, sizeof(p_Header->data.padding), size);
         break;
         case e_FIELD_ETH_NEXT_HEADER_TYPE:
             SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
         break;
         case e_FIELD_ETH_VALID_FIELDS:
             SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
         break;
        default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
         }
     }

     return E_OK;
}

t_Error HDR_Eth_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
     t_Ethernet* p_Header = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

     if (!p_Header)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     switch (fieldID)
     {
         case e_FIELD_ETH_LENGTH:
         case e_FIELD_ETH_TYPE:
         case e_FIELD_ETH_FINAL_CKSUM:
         break;
         default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
         }break;
     }

     if (bSetAuto)
         FLAG_SET(p_Header->autoFlags, fieldID);
     else
         FLAG_CLEAR(p_Header->autoFlags, fieldID);

     if ((fieldID==e_FIELD_ETH_FINAL_CKSUM) && bSetAuto) p_Header->data.bUseFinalCkSum = TRUE;

     return E_OK;
}

t_Error HDR_Eth_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_Ethernet       *p_Header = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgEthernetHeader *pValues = (t_PgEthernetHeader*)hValues;

     if (!p_Header || !pValues)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     if (!(fieldIDs & e_FIELD_ETH_ALL_FIELDS))
     {
         RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
     }

     /* Configure the Ethernet header fields specified by fieldIDs */
     if (fieldIDs & e_FIELD_ETH_DA)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_DA, (t_Handle)&pValues->da, sizeof(pValues->da));

     if (fieldIDs & e_FIELD_ETH_SA)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_SA, (t_Handle)&pValues->sa, sizeof(pValues->sa));

     /* If both Length and Type are set, the Type has priority (overwrites Length) */
     if (fieldIDs & e_FIELD_ETH_LENGTH)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_LENGTH, (t_Handle)&pValues->typeLen, sizeof(pValues->typeLen));

     if (fieldIDs & e_FIELD_ETH_TYPE)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_TYPE, (t_Handle)&pValues->typeLen, sizeof(pValues->typeLen));

     if(fieldIDs & e_FIELD_ETH_USE_FINAL_CKSUM)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_USE_FINAL_CKSUM, (t_Handle)&pValues->bUseFinalCkSum, sizeof(pValues->bUseFinalCkSum));

     if(fieldIDs & e_FIELD_ETH_FINAL_CKSUM)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_FINAL_CKSUM, (t_Handle)&pValues->finalCheckSum, sizeof(pValues->finalCheckSum));

     if (fieldIDs & e_FIELD_ETH_PADDING)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_PADDING, (t_Handle)&pValues->padding,sizeof(pValues->padding));

     if (fieldIDs & e_FIELD_ETH_NEXT_HEADER_TYPE)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType,sizeof(pValues->nextHeaderType));

     if (fieldIDs & e_FIELD_ETH_VALID_FIELDS)
         HDR_Eth_ConfigField(h_Header, e_FIELD_ETH_VALID_FIELDS, (t_Handle)&pValues->validFields,sizeof(pValues->validFields));


     return E_OK;

}

t_Error HDR_Eth_Randomize(t_Handle h_Header)
{
     /* Get the handle to the Ethernet header */
     t_Ethernet*   pHeaderStr = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
     uint8_t       index = 0; /* Loop index */
     t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header; /* Frame struct header handle */

     /* Generate random values to the fields not configured by the user */
     if (!(pHeaderStr->data.validFields & e_FIELD_ETH_DA))
     {
         for (index=0; index<ETH_ADDR_SIZE; index++)
             pHeaderStr->data.da[index] = (uint8_t)rand();
     }

     if (!(pHeaderStr->data.validFields & e_FIELD_ETH_SA))
     {
         for (index=0; index<ETH_ADDR_SIZE; index++)
             pHeaderStr->data.sa[index] = (uint8_t)rand();
     }

     /* If both Length and Type flags are set then the Type has priority (overwrites Length) */
     if (!(pHeaderStr->data.validFields & e_FIELD_ETH_LENGTH) &&
        !(pHeaderStr->data.validFields & e_FIELD_ETH_TYPE))
     {
        if(p_Frame->HeaderID == e_HEADER_TYPE_ETH_802_3)
            pHeaderStr->bIsType = FALSE;
        else
        {
            pHeaderStr->data.typeLen = (uint16_t)(MAX_ETH_LEN_VAL + (rand()%200));
            pHeaderStr->bIsType = TRUE;
        }
     }
     else if(pHeaderStr->data.validFields & e_FIELD_ETH_LENGTH)
     {
        pHeaderStr->bIsType = FALSE;
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_ETH_LENGTH);
     }
     else if(pHeaderStr->data.validFields & e_FIELD_ETH_TYPE)
     {
        pHeaderStr->bIsType = TRUE;
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_ETH_TYPE);
     }

     if (!(pHeaderStr->data.validFields & e_FIELD_ETH_USE_FINAL_CKSUM))
     {
         pHeaderStr->data.bUseFinalCkSum = (bool)(rand()%2);
     }

     if (pHeaderStr->data.bUseFinalCkSum && !(pHeaderStr->data.validFields & e_FIELD_ETH_FINAL_CKSUM))
     {
         pHeaderStr->data.finalCheckSum = (uint32_t)rand();
         FLAG_SET(pHeaderStr->autoFlags, e_FIELD_ETH_FINAL_CKSUM);
     }

     else if(pHeaderStr->data.validFields & e_FIELD_ETH_FINAL_CKSUM)
     {
         FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_ETH_FINAL_CKSUM);
         pHeaderStr->data.bUseFinalCkSum = TRUE;
     }

     return E_OK;
 }

t_Error HDR_Eth_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
     t_FrameStruct  *p_Frame = (t_FrameStruct*)h_Header; /* Frame handle */
     t_Ethernet     *p_Header = (t_Ethernet*)p_Frame->h_HeaderStruct; /* Ethernet header handle */
     t_FrameStruct  *p_NextHeader = NULL;     /* Next header pointer */
     uint32_t       bufferSize    = 0;        /* Holds the buffer size */
     uint8_t        *pPrevBuffer  = 0;        /* Previous buffer pointer */
     int32_t        frameSize     = 0;        /* Holds the frame size */
     uint32_t       finalCheckSum = 0;        /* Final CheckSum */
     uint8_t        padValue = 0;             /* Pad with 0 */

     if (!(p_Frame && h_Header && p_Buffer && pSize))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

      /* Get the size of the frame (if needed) */
     if (!p_Header->bIsType && FLAG_CHECK(p_Header->autoFlags, e_FIELD_ETH_LENGTH))
     {
         p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(h_Header);
         if (p_NextHeader)
             p_Header->data.typeLen = (uint16_t)FRAME_GetFrameSize(p_NextHeader, p_NextHeader);
         else
             p_Header->data.typeLen = 0;
     }

     /* Store the current buffer pointer for later */
     pPrevBuffer = *p_Buffer;
     bufferSize = *pSize;

     /* Write the values into the buffer limited by size (issue error if insufficient space) */
     if (
             (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.da), sizeof(p_Header->data.da), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
             (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.sa), sizeof(p_Header->data.sa), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
             (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.typeLen), sizeof(p_Header->data.typeLen), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
         )
     {
         return E_NOT_IN_RANGE;
     }

      /* Serialize the rest of the frame */
     if (INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize) != E_OK)
     {
         RETURN_ERROR(MAJOR, E_INVALID_STATE, (""));
     }

     /* Pad the ethernet frame if needed */
     if (p_Header->data.padding)
     {
         frameSize = FIXED_HEADER_SIZE_ETH;

         p_NextHeader = INTERNAL_GetNextHeader(h_Header);
         frameSize += (int32_t)(p_NextHeader ? INTERNAL_GetFrameSize(p_NextHeader, NULL) : 0);

         if (p_Header->data.bUseFinalCkSum)
             frameSize += ETH_FCS_SIZE;

         if (frameSize < MIN_FRAME_LEN_ETHERNET)
         {
             /* Take into account the final checksum length */
             frameSize = MIN_FRAME_LEN_ETHERNET - frameSize;
             /* Add padding */
             while (frameSize-- > 0)
             {
                 if (INTERNAL_SerializeValue((uint8_t*)(&padValue), sizeof(padValue), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
                 {
                     return E_NOT_IN_RANGE;
                 }
             }
         }
     }

     /* Add the final CRC */
     if (p_Header->data.bUseFinalCkSum)
     {
         /* Get the size of the serialized frame */
         bufferSize -= *pSize;

         /* Compute the 32-bit FCS */
         finalCheckSum = INTERNAL_ComputeCRC(0xFFFFFFFF, pPrevBuffer, bufferSize);

         /* Get the checksum - if set to auto*/
         if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_ETH_FINAL_CKSUM))
         {
             p_Header->data.finalCheckSum = finalCheckSum;
         }
         else
         {
             if(finalCheckSum != p_Header->data.finalCheckSum)
                p_Header->errorFlag = e_FCS_ERROR;
         }

         /* Serialize the checksum */
         if (INTERNAL_SerializeValue((uint8_t*)&p_Header->data.finalCheckSum, ETH_FCS_SIZE, p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
         {
             return E_NOT_IN_RANGE;
         }
     }

     return E_OK;
}

uint32_t HDR_Eth_GetHeaderSize(t_Handle h_Header)
{
    t_Ethernet*  pHeaderStr   = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct; /* Ethernet header handle */
    t_Handle     h_NextHeader = INTERNAL_GetNextHeader(h_Header); /* Next header pointer */
    uint32_t     size         = FIXED_HEADER_SIZE_ETH; /* Ethernet header size */
    int32_t frameSize = 0;                  /* Frame size */
    t_Handle h_LastHeader = NULL;
    e_PgHeaderType lastHeaderType;

    /* Account for the final 4 FCS length */
    size += (pHeaderStr->data.bUseFinalCkSum) ? ETH_FCS_SIZE : 0;

    /* Account for the ethernet padding (if requested)*/
    if (pHeaderStr->data.padding)
    {
        /* The size of this header (with FCS if enabled) */
        frameSize += (int32_t)size;

        /* Get the size of the frame starting after this ETH header */
        frameSize += (int32_t)(h_NextHeader ? INTERNAL_GetFrameSize(h_NextHeader, NULL) : 0);

        h_LastHeader = INTERNAL_GetLastHeader(h_Header);
        lastHeaderType = ((t_FrameStruct*)h_LastHeader)->HeaderID;

        if((lastHeaderType != e_HEADER_TYPE_PAYLOAD) ||
        (INTERNAL_GetFrameSize(h_LastHeader, NULL)))
        {
            /* Get the size of the required padding */
            if (frameSize < MIN_FRAME_LEN_ETHERNET)
                size += (uint32_t)(MIN_FRAME_LEN_ETHERNET - frameSize);
        }
    }

    return size;
}

t_Error HDR_Eth_Init(t_Handle h_Header)
{
    t_Ethernet *p_Header = NULL;       /* Ethernet header handle */
    t_FrameStruct *p_NextHeader = NULL,*p_Frame = NULL;    /* Next header handle */
    bool      bSessionStarted = FALSE; /* Status of PPPoE session */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Frame = (t_FrameStruct*)h_Header;
    p_Header = (t_Ethernet*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Based on the next header, initialize the type field */
    if (p_NextHeader)
    {
        if ((p_Header->bIsType) && FLAG_CHECK(p_Header->autoFlags , e_FIELD_ETH_TYPE))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_VLAN || p_NextHeader->HeaderID == e_HEADER_TYPE_VTAGGED_802_3 )
                p_Header->data.typeLen = 0x8100;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv4)
                p_Header->data.typeLen = 0x0800;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv6)
                p_Header->data.typeLen = 0x86DD;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_PPPoE)
            {
                HDR_PPPoE_GetField( p_NextHeader, e_FIELD_PPPoE_SESSION,
                                   &bSessionStarted, sizeof(bool));
                p_Header->data.typeLen=(uint16_t)(bSessionStarted ?
                                                0x8864 : 0x8863); // 0x8863 - discovery stage
                                                                  // 0x8864 - session stage
            }
            if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_ETH)
            {
                if (p_NextHeader->HeaderID == e_HEADER_TYPE_PAYLOAD)
                p_Header->data.typeLen = 0x88ff;
            }
            if(p_NextHeader->HeaderID == e_HEADER_TYPE_MPLS)
               p_Header->data.typeLen = 0x8847;
        }
    }
      /* If the next header is null or of an unknown type, leave the type field as it was */
    return E_OK;
}

t_Handle HDR_Eth_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.ethernetHeader);
}

t_Error HDR_Eth_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_Ethernet *p_Header = NULL;       /* Ethernet header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_Ethernet*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of Ethernet group */

/**************************************************************************//**
@Group          Payload

@Description    Payload functions

@{
*//***************************************************************************/
t_Handle HDR_Payload_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PgPayloadType *p_PayloadType = (t_PgPayloadType*)hHeader;
    t_Payload *p_Payload = NULL;
    UNUSED(p_ProtocolInfo);


    /* Check the payload handle */
    if (!p_PayloadType)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Payload = (t_Payload*)MEMALLOC(sizeof(t_Payload));
    if (!p_Payload)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Payload, 0, sizeof(t_Payload));

    p_Payload->p_Data = NULL;
    p_Payload->length = 0;

    p_Payload->seqNumber = 0;

    p_Payload->autoFlags = 0;
    FLAG_SET(p_Payload->autoFlags, e_FIELD_PAYLOAD_SIZE);

    /* Initialize the user configured payload fields in the payload */
    p_Payload->payloadType = p_PayloadType->type;

    if(p_Payload->payloadType == e_PAYLOAD_PATTERN)
    {
        p_Payload->startVal = p_PayloadType->value.pattern.startVal;
        p_Payload->stepVal = p_PayloadType->value.pattern.stepVal;
        p_Payload->stepCount = p_PayloadType->value.pattern.stepCount;
    }
    else if(p_Payload->payloadType == e_PAYLOAD_PRBS)
    {
        p_Payload->polynomial = p_PayloadType->value.prbs.polynomial;
        p_Payload->numBits = p_PayloadType->value.prbs.numBits;
        p_Payload->initVector = p_PayloadType->value.prbs.initVector;

        if (!p_Payload->initVector)
        {
           /* Take different value for every compilation */
           p_Payload->initVector = (uint32_t)
           (
               ((__DATE__[4]-'0')<<28)|((__DATE__[5]-'0')<<24) /* Day    */
                                      |
               ((__TIME__[0]-'0')<<20)|((__TIME__[1]-'0')<<16) /* Hour   */
                                      |
               ((__TIME__[3]-'0')<<12)|((__TIME__[4]-'0')<< 8) /* Minute */
                                      |
               ((__TIME__[6]-'0')<< 4)|((__TIME__[7]-'0')<< 0) /* Second */
           );
        }
    }
    else if(p_Payload->payloadType == e_PAYLOAD_USER)
    {
        p_Payload->length = p_PayloadType->value.user.size;

        if(p_PayloadType->value.user.p_Buffer)
        {
            p_Payload->p_Data = (uint8_t*)MEMALLOC(p_Payload->length);
            if (!p_Payload->p_Data)
            {
                MEMFREE(p_Payload);
                return NULL;
            }
            memcpy(p_Payload->p_Data, p_PayloadType->value.user.p_Buffer, p_Payload->length);
        }

        FLAG_CLEAR(p_Payload->autoFlags, e_FIELD_PAYLOAD_SIZE);
    }
    else if(p_Payload->payloadType == e_PAYLOAD_FIXED_PATTERN)
    {
        p_Payload->patternSize = p_PayloadType->value.fixedPattern.patternSize;
        memcpy(p_Payload->patternValue, p_PayloadType->value.fixedPattern.patternValue, p_Payload->patternSize);
    }

    if(p_Payload->payloadType != e_PAYLOAD_USER)
    {
        p_Payload->minFrameLen = p_PayloadType->minFrameLen;
        p_Payload->maxFrameLen = p_PayloadType->maxFrameLen;
    }

    p_Payload->nextHeaderType = p_PayloadType->nextHeaderType;

    return (t_Handle) p_Payload;
}

t_Handle HDR_Payload_Clone(t_Handle h_Object)
{
    t_Payload *p_Object = (t_Payload*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_Payload* p_Payload = (t_Payload*)MEMALLOC(sizeof(t_Payload));

    if (!p_Payload)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Payload, 0, sizeof(t_Payload));

    /* Take a copy of the payload */
    p_Payload->length = p_Object->length;
    p_Payload->maxFrameLen = p_Object->maxFrameLen;
    p_Payload->minFrameLen = p_Object->minFrameLen;
    p_Payload->autoFlags = p_Object->autoFlags;
    p_Payload->payloadType = p_Object->payloadType;
    p_Payload->startVal = p_Object->startVal;
    p_Payload->stepVal = p_Object->stepVal;
    p_Payload->stepCount = p_Object->stepCount;
    p_Payload->polynomial = p_Object->polynomial;
    p_Payload->numBits = p_Object->numBits;
    p_Payload->initVector = p_Object->initVector;
    p_Payload->patternSize = p_Object->patternSize;

    p_Payload->seqNumber = p_Object->seqNumber;
    p_Payload->nextHeaderType = p_Object->nextHeaderType;

    /* Copy the data (if any)*/
    if (p_Object->p_Data)
    {
        p_Payload->p_Data = (uint8_t*)MEMALLOC(p_Object->length);
        if (!p_Payload->p_Data)
        {
            MEMFREE(p_Payload);
            return NULL;
        }
        memcpy(p_Payload->p_Data, p_Object->p_Data, p_Object->length);
    }

    /* Copy the pattern data*/
    memcpy(p_Payload->patternValue, p_Object->patternValue, p_Object->patternSize);

    return p_Payload;
}

t_Error HDR_Payload_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_Payload *p_Header = (t_Payload*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_PAYLOAD_BUFFER:
            if (p_Header->p_Data)
            {
                if (p_Header->length < size)
                    size = p_Header->length;
            SAFE_COPY_VALUE(p_Header->p_Data, h_Value, p_Header->length, size);
            }
        break;
        case e_FIELD_PAYLOAD_SIZE:
            SAFE_COPY_VALUE(&p_Header->length, h_Value, sizeof(p_Header->length), size);
        break;
        case e_FIELD_MAX_FRM_SIZE:
            SAFE_COPY_VALUE(&p_Header->maxFrameLen, h_Value, sizeof(p_Header->maxFrameLen), size);
        break;
        case e_FIELD_MIN_FRM_SIZE:
            SAFE_COPY_VALUE(&p_Header->minFrameLen, h_Value, sizeof(p_Header->minFrameLen), size);
        break;
        case e_FIELD_PAYLOAD_TYPE:
            SAFE_COPY_VALUE(&p_Header->payloadType, h_Value, sizeof(p_Header->payloadType), size);
        break;
        case e_FIELD_PRBS_POLYNOMIAL:
            SAFE_COPY_VALUE(&p_Header->polynomial, h_Value, sizeof(p_Header->polynomial), size);
        break;
        case e_FIELD_PRBS_NUM_BITS:
            SAFE_COPY_VALUE(&p_Header->numBits, h_Value, sizeof(p_Header->numBits), size);
        break;
        case e_FIELD_PRBS_INIT_VECTOR:
            SAFE_COPY_VALUE(&p_Header->initVector, h_Value, sizeof(p_Header->initVector), size);
        break;
        case e_FIELD_FIXED_PATTERN_SIZE:
            SAFE_COPY_VALUE(&p_Header->patternSize, h_Value, sizeof(p_Header->patternSize), size);
        break;
        case e_FIELD_FIXED_PATTERN_VALUE:
            if (p_Header->patternSize < size)
                size = p_Header->patternSize;
            SAFE_COPY_VALUE(p_Header->patternValue, h_Value, p_Header->patternSize, size);
        break;
        case e_FIELD_SEQ_NUM:
            SAFE_COPY_VALUE(&p_Header->seqNumber, h_Value, sizeof(p_Header->seqNumber), size);
        break;
        case e_FIELD_PAYLOAD_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->nextHeaderType, h_Value, sizeof(p_Header->nextHeaderType), size);
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_Payload_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_Payload *p_Header = (t_Payload*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgPayloadType *pArg = NULL;

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_PAYLOAD_BUFFER:
        {
            /* Free the buffer if not null*/
            if (p_Header->p_Data)
                MEMFREE(p_Header->p_Data);
            p_Header->p_Data = NULL;
            /* Aquire a new data buffer*/
            p_Header->p_Data = MEMALLOC(size);
            if (!p_Header->p_Data)
            {
                RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Could not config the payload"));
            }
            /* Copy the provided buffer*/
            memcpy(p_Header->p_Data, h_Value, size);
            /* Set the new buffer size*/
            p_Header->length = size;
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_PAYLOAD_SIZE:
        {
            p_Header->length = *((uint32_t*)h_Value);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_FRAME_SIZE:
        {
             if ((HDR_Payload_ConfigField(h_Header, e_FIELD_MAX_FRM_SIZE, h_Value, size) != E_OK) ||
             (HDR_Payload_ConfigField(h_Header, e_FIELD_MIN_FRM_SIZE, h_Value, size) != E_OK) )
             {
                 RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Could not config the frame size"));
             }
        }
        break;
        case e_FIELD_MAX_FRM_SIZE:
            p_Header->maxFrameLen = *((uint32_t*)h_Value);
        break;
        case e_FIELD_MIN_FRM_SIZE:
            p_Header->minFrameLen = *((uint32_t*)h_Value);
        break;
        case e_FIELD_PAYLOAD_TYPE:
        {
            pArg = (t_PgPayloadType*)h_Value;
            p_Header->payloadType = (e_PgPayloadType)pArg->type;

            switch (pArg->type)
            {
                case e_PAYLOAD_PRBS:
                {
                    p_Header->polynomial = pArg->value.prbs.polynomial;
                    p_Header->numBits = pArg->value.prbs.numBits;
                    p_Header->initVector = pArg->value.prbs.initVector;
                }
                break;
                case e_PAYLOAD_USER:
                    return HDR_Payload_ConfigField(h_Header, e_FIELD_PAYLOAD_BUFFER, pArg->value.user.p_Buffer, pArg->value.user.size);
                break;
                case e_PAYLOAD_PATTERN:
                {
                    p_Header->startVal = pArg->value.pattern.startVal;
                    p_Header->stepVal = pArg->value.pattern.stepVal;
                    p_Header->stepCount = pArg->value.pattern.stepCount;
                }break;
                case e_PAYLOAD_FIXED_PATTERN:
                {
                    p_Header->patternSize = pArg->value.fixedPattern.patternSize;
                    memcpy(p_Header->patternValue, pArg->value.fixedPattern.patternValue, p_Header->patternSize);
                }break;
            }
        }
        break;
        case e_FIELD_SEQ_NUM:
            SAFE_COPY_VALUE(h_Value, &p_Header->seqNumber, sizeof(p_Header->seqNumber), size);
        break;
        case e_FIELD_PAYLOAD_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->nextHeaderType, sizeof(p_Header->nextHeaderType), size);
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_Payload_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    /* There are no auto fields in the payload */
    t_Payload *p_Header = (t_Payload*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_PAYLOAD_SIZE:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        }
        break;
    }

    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_Payload_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_Payload *p_Header = (t_Payload*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgPayloadType *pValues = (t_PgPayloadType*)hValues;

    if (!p_Header || !hValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_PAYLOAD_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the payload fields specified by fieldIDs */
    if (fieldIDs & e_FIELD_PAYLOAD_TYPE)
        HDR_Payload_ConfigField(h_Header, e_FIELD_PAYLOAD_TYPE, hValues, AUTO_FIELD_SIZE);

    if (fieldIDs & e_FIELD_MAX_FRM_SIZE)
        HDR_Payload_ConfigField(h_Header, e_FIELD_MAX_FRM_SIZE, (t_Handle)&pValues->maxFrameLen, AUTO_FIELD_SIZE);

    if (fieldIDs & e_FIELD_MIN_FRM_SIZE)
        HDR_Payload_ConfigField(h_Header, e_FIELD_MIN_FRM_SIZE, (t_Handle)&pValues->minFrameLen, AUTO_FIELD_SIZE);

    if (fieldIDs & e_FIELD_PAYLOAD_NEXT_HEADER_TYPE)
        HDR_Payload_ConfigField(h_Header, e_FIELD_PAYLOAD_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    return E_OK;
}

t_Error HDR_Payload_Randomize(t_Handle h_Header)
{
    UNUSED(h_Header);

    return E_OK;
}

t_Error HDR_Payload_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header; /* Frame handle */
    t_Payload     *p_Header = (t_Payload*)p_Frame->h_HeaderStruct; /* Payload pointer */
    uint32_t      payloadLength = 0;

    if (!(h_Header && p_Frame && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    payloadLength = (p_Header->length - sizeof(p_Header->seqNumber));

    /* Write the values into the buffer limited by size (issue error if insufficient space) */
    if ( INTERNAL_SerializeValue((uint8_t*)(p_Header->p_Data), payloadLength, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize sequence number even though it is zero */
    /*if(p_Header->seqNumber)
    {*/
        /* Write the sequence number of the frame */
        if ( INTERNAL_SerializeValue((uint8_t*)(&p_Header->seqNumber), sizeof(p_Header->seqNumber), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    /*}*/

    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_Payload_GetHeaderSize(t_Handle h_Header)
{
    t_Payload* pHeaderStr = (t_Payload*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    return pHeaderStr->length;
}

t_Error HDR_Payload_Init(t_Handle h_Header)
{
    t_Payload* p_Payload = (t_Payload*)((t_FrameStruct*)h_Header)->h_HeaderStruct;/* Payload handle */
    uint32_t hdrSize = 0;              /* Header size */
    uint32_t pos = 0;                  /* Byte position */
    int32_t newMax = 0, newMin = 0;        /* Payload size */
    bool bInitPayload = FALSE;
    t_Handle h_PrevHeader;

    uint32_t polyVal =0 ,n = 0,randomNum = 0;
    uint32_t l,maxLength = 0,numOfOnes = 0,setOne = 0,poly =0, regLength =0;

    uint32_t size = 0;
    uint32_t payloadLength = 0;

    if (FLAG_CHECK(p_Payload->autoFlags, e_FIELD_PAYLOAD_SIZE))
    {
        h_PrevHeader = INTERNAL_GetPrevHeader(h_Header);
        /* Get the size of the frame (all header until this one)*/
        hdrSize += (uint32_t)(h_PrevHeader ? INTERNAL_GetFrameSize(NULL, h_PrevHeader) : 0);

        newMax = (int32_t)(p_Payload->maxFrameLen - hdrSize);
        newMin = (int32_t)(p_Payload->minFrameLen - hdrSize);

        if (newMax <= 0)
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Cannot create payload: headers size is bigger than maximum size! Check settings (min, max frame size)"));
        }

        if (newMin <= (int)sizeof(p_Payload->seqNumber))
            newMin = sizeof(p_Payload->seqNumber) + 1;

        if (newMin == newMax)
            p_Payload->length = (uint32_t)newMin;
        else
            p_Payload->length = (uint32_t)(newMin + rand()%(newMax-newMin+1));

        if (!p_Payload->length)
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Cannot create payload: headers size is bigger than maximum size! Check settings (min, max frame size)"));
        }

        /* If the buffer in not null, just free it */
        if (p_Payload->p_Data)
            MEMFREE(p_Payload->p_Data);
        p_Payload->p_Data = NULL;

        bInitPayload = TRUE;

        FLAG_CLEAR(p_Payload->autoFlags, e_FIELD_PAYLOAD_SIZE); /* Changed */
    }
    else /* The payload has a user defined length */
    {
        if(p_Payload->payloadType == e_PAYLOAD_USER)
        {
            if (!p_Payload->p_Data)
                bInitPayload = TRUE;
            else if(p_Payload->seqNumber)
                p_Payload->length += sizeof(p_Payload->seqNumber);
        }
    }

    if (bInitPayload)
    {
        payloadLength = p_Payload->length;
        /* Realloc the buffer to be of required length */
        ASSERT_COND(payloadLength > sizeof(p_Payload->seqNumber));
        if(p_Payload->seqNumber)
            payloadLength -= sizeof(p_Payload->seqNumber);

        p_Payload->p_Data = MEMALLOC(payloadLength);

        if (!p_Payload->p_Data)
        {
            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Could not create the payload buffer"));
        }
        memset(p_Payload->p_Data, 0, payloadLength);

        /* Fill the buffer with the desired payload type (random or pattern)*/
        switch (p_Payload->payloadType)
        {
            case e_PAYLOAD_PRBS:
            {
                /* Expect the user to provide nothing else except the payload size */
                /* PRBS */
                polyVal = p_Payload->polynomial;
                n = p_Payload->numBits;
                randomNum = p_Payload->initVector;

                if (!p_Payload->polynomial)
                {
                    RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Illegal PRBS Polynomial value"));
                }
                if (!p_Payload->initVector)
                {
                    RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Illegal PRBS InitVector value"));
                }
                if (!p_Payload->numBits)
                {
                    RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Illegal PRBS numBits value"));
                }


               /* Get the maximal length of the PRBS */
                maxLength = ((uint32_t)(2 << (n-1))- 1);
                randomNum = maxLength & randomNum;

                for (l= 0;l<payloadLength; l++)
                {
                    switch(n)
                    {
                    /* Convert the n-bit binary sequence output into byte and store it as payload */

                        case PRBS_32BITS:
                        {
                            p_Payload->p_Data[pos++] = (uint8_t)(randomNum>>24 & 0xFF);
                            if(pos == payloadLength)
                                break;
                            p_Payload->p_Data[pos++] = (uint8_t)(randomNum>>16 & 0xFF);
                            if(pos == payloadLength)
                                break;
                        }
                        case PRBS_16BITS:
                        {
                            p_Payload->p_Data[pos++] = (uint8_t)(randomNum>>8 & 0xFF);
                            if(pos == payloadLength)
                                break;
                        }
                        case PRBS_8BITS:
                        {
                            p_Payload->p_Data[pos++] = (uint8_t)randomNum;
                        }
                    }

                    if (pos >= payloadLength)
                    {
                        break;
                    }
                    /* ExOR the values of randomNum according to polynomial coefficient */
                    /* caluclate numOfOnes in the PRBS register */
                    poly = randomNum & polyVal;
                    for ( regLength = 0; regLength <= (n-1); regLength++)
                    {
                        if (poly & 1)
                        {
                            numOfOnes++;
                        }
                        poly = poly >> 1;
                    }
                    /* Insert the Exor result into MSB of N-bit binary sequence variable */
                    /* if numOfOnes = odd value insert 1 else insert 0*/
                    randomNum = randomNum >> 1;
                    if (numOfOnes % 2)
                    {
                        setOne = (uint32_t)(1 << (n - 1));
                        randomNum = (randomNum |setOne);
                    }
                    numOfOnes = 0;
                }
            }

            break;
            case e_PAYLOAD_PATTERN:
            {
                /* Fill the buffer by writing incremented values*/
                /*
                EX #1: startVal = 3; stepVal = 2; stepCount = 3; payloadSize = 7;
                Buffer Values: 3 5 7 3 5 7 3
                EX #2: startVal = 10; stepVal = -1; stepCount = 4; payloadSize = 10;
                Buffer Values: 10 9 8 7 10 9 8 7 10 9
                */
                while(pos < payloadLength)
                {
                    if (pos && (pos%p_Payload->stepCount))
                        p_Payload->p_Data[pos] = (uint8_t)(p_Payload->p_Data[pos-1]+p_Payload->stepVal);
                    else
                        p_Payload->p_Data[pos] = p_Payload->startVal;
                    pos++;
                }
            }break;
            case e_PAYLOAD_USER:
            break;
            case e_PAYLOAD_FIXED_PATTERN:
            {
                while(pos < payloadLength)
                {
                    size = ((payloadLength - pos) >= p_Payload->patternSize) ? p_Payload->patternSize : (payloadLength - pos);
                    memcpy((p_Payload->p_Data + pos),p_Payload->patternValue, size);
                    pos += size;
                }
            }break;
            default:
            {
                RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid payload type"));
            }
            break;
        }
    }

    return E_OK;
}

t_Handle HDR_Payload_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->payload);
}

t_Error HDR_Payload_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}
/** @} */ /* end of Payload group */

/**************************************************************************//**
@Group          Padding

@Description    Padding functions

@{
*//***************************************************************************/
t_Handle HDR_Padding_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PgPaddingHeader *p_PaddingHeader = (t_PgPaddingHeader*)hHeader, *p_CurrPaddingHeader;
    t_PgPaddingHeader *p_Header = NULL;

    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    if(!(p_PaddingHeader->validFields & e_FIELD_PADDING_SIZE))
    {
        REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Padding Size not configured"));
        return NULL;
    }

    p_Header = (t_PgPaddingHeader*)MEMALLOC(sizeof(t_PgPaddingHeader));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_PgPaddingHeader));

    /* Initialize the user configured padding header fields in the header */
    p_CurrPaddingHeader = &p_PaddingHeader[padIndex++];
    memcpy(p_Header, p_CurrPaddingHeader, sizeof(t_PgPaddingHeader));

    if( (p_CurrPaddingHeader->validFields & e_FIELD_PADDING_SHIM_HEADER) && p_CurrPaddingHeader->p_ShimHeader )
    {
        p_Header->p_ShimHeader = (uint8_t*)MEMALLOC(p_CurrPaddingHeader->size);
        if (!p_Header->p_ShimHeader)
        {
            MEMFREE(p_Header);
            return NULL;
        }
        memcpy(p_Header->p_ShimHeader, p_CurrPaddingHeader->p_ShimHeader, p_CurrPaddingHeader->size);
    }
    else
    {
        p_Header->p_ShimHeader =  NULL;
    }

    return (t_Handle) p_Header;
}

t_Handle HDR_Padding_Clone(t_Handle h_Object)
{
   t_PgPaddingHeader  *p_Object = (t_PgPaddingHeader*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
   t_PgPaddingHeader  *p_Header = (t_PgPaddingHeader*)MEMALLOC(sizeof(t_PgPaddingHeader));

   if (!p_Header)
   {
       REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the padding Header"));
       return NULL;
   }

   memset(p_Header, 0, sizeof(t_PgPaddingHeader));

   /* Take a copy of the Padding header */
   memcpy(p_Header, p_Object, sizeof(t_PgPaddingHeader));

   /* Copy the data (if any)*/
   if (p_Object->p_ShimHeader)
   {
       p_Header->p_ShimHeader = (uint8_t*)MEMALLOC(p_Object->size);
       if (!p_Header->p_ShimHeader)
       {
           MEMFREE(p_Header);
           return NULL;
       }
       memcpy(p_Header->p_ShimHeader, p_Object->p_ShimHeader, p_Object->size);
   }

   return p_Header;
}

t_Error HDR_Padding_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PgPaddingHeader  *p_Header  = (t_PgPaddingHeader*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }
    switch (fieldID)
    {
        case e_FIELD_PADDING_VALUE:
            SAFE_COPY_VALUE(&p_Header->value, h_Value, sizeof(p_Header->value), size);
        break;
        case e_FIELD_PADDING_SIZE:
            SAFE_COPY_VALUE(&p_Header->size, h_Value, sizeof(p_Header->size) , size);
        break;
        case e_FIELD_PADDING_SHIM_HEADER:
            if(p_Header->p_ShimHeader)
            {
                if (p_Header->size < size)
                    size = p_Header->size;
                SAFE_COPY_VALUE(p_Header->p_ShimHeader, h_Value, p_Header->size, size);
            }
        break;
        case e_FIELD_PADDING_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->nextHeaderType, h_Value, sizeof(p_Header->nextHeaderType), size);
        break;
        case e_FIELD_PADDING_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->validFields, h_Value, sizeof(p_Header->validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }
   return E_OK;
}

t_Error HDR_Padding_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
   t_PgPaddingHeader *p_PaddingHeader = (t_PgPaddingHeader*)((t_FrameStruct*)h_Header)->h_HeaderStruct;


    if (!(p_PaddingHeader && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

   size = (size) ? size : (uint32_t)(-1);

   switch (fieldID)
   {
       case e_FIELD_PADDING_VALUE:
           SAFE_COPY_VALUE(h_Value, &p_PaddingHeader->value, sizeof(p_PaddingHeader->value), size);
           break;
       case e_FIELD_PADDING_SIZE:
           SAFE_COPY_VALUE(h_Value, &p_PaddingHeader->size, sizeof(p_PaddingHeader->size), size);
          break;
       case e_FIELD_PADDING_SHIM_HEADER:
            /* Free the buffer if not null*/
            if (p_PaddingHeader->p_ShimHeader)
                MEMFREE(p_PaddingHeader->p_ShimHeader);
            p_PaddingHeader->p_ShimHeader = NULL;
            /* Aquire a new data buffer*/
            p_PaddingHeader->p_ShimHeader = MEMALLOC(size);
            if (!p_PaddingHeader->p_ShimHeader)
            {
                RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Could not config the payload"));
            }
            /* Copy the provided buffer*/
            memcpy(p_PaddingHeader->p_ShimHeader, h_Value, size);
            /* Set the new buffer size*/
            p_PaddingHeader->size = (uint16_t)size;
          break;
       case e_FIELD_PADDING_NEXT_HEADER_TYPE:
           SAFE_COPY_VALUE(h_Value, &p_PaddingHeader->nextHeaderType, sizeof(p_PaddingHeader->nextHeaderType), size);
       break;

       case e_FIELD_PADDING_VALID_FIELDS:
           SAFE_COPY_VALUE(h_Value, &p_PaddingHeader->validFields, sizeof(p_PaddingHeader->validFields), size);
       break;

       default:
       {
           RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
       }
   }

   return E_OK;
}

t_Error HDR_Padding_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
   UNUSED(h_Header);
   UNUSED(fieldID);
   UNUSED(bSetAuto);
   return E_OK;
}

t_Error HDR_Padding_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
   t_PgPaddingHeader *p_Header = (t_PgPaddingHeader*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
   t_PgPaddingHeader *pValues = (t_PgPaddingHeader*)hValues;

    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_PADDING_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the Ethernet header fields specified by fieldIDs */
    if (fieldIDs & e_FIELD_PADDING_VALUE)
        HDR_Padding_ConfigField(h_Header, e_FIELD_PADDING_VALUE, (t_Handle)&pValues->value, sizeof(pValues->value));

    if (fieldIDs & e_FIELD_PADDING_SIZE)
        HDR_Padding_ConfigField(h_Header, e_FIELD_PADDING_SIZE, (t_Handle)&pValues->size, sizeof(pValues->size));

    /*TODO: TEST with Header field manipulation */
    if (fieldIDs & e_FIELD_PADDING_SHIM_HEADER)
        HDR_Padding_ConfigField(h_Header, e_FIELD_PADDING_SHIM_HEADER, (t_Handle)pValues->p_ShimHeader, pValues->size);

    if (fieldIDs & e_FIELD_PADDING_NEXT_HEADER_TYPE)
        HDR_Padding_ConfigField(h_Header, e_FIELD_PADDING_NEXT_HEADER_TYPE, (t_Handle)&p_Header->nextHeaderType,sizeof(p_Header->nextHeaderType));

    if (fieldIDs & e_FIELD_PADDING_VALID_FIELDS)
        HDR_Padding_ConfigField(h_Header, e_FIELD_PADDING_VALID_FIELDS, (t_Handle)&p_Header->validFields,sizeof(p_Header->nextHeaderType));

    return E_OK;
}

t_Error HDR_Padding_Randomize(t_Handle h_Header)
{
    UNUSED(h_Header);

    return E_OK;
}

t_Error HDR_Padding_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame     = (t_FrameStruct*)h_Header;            /* Frame handle */
    t_PgPaddingHeader  *p_Header = (t_PgPaddingHeader*)p_Frame->h_HeaderStruct; /* Padding pointer */

    uint8_t *p_PadBuffer = NULL;

    if (!(h_Header && p_Frame && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_PadBuffer = (uint8_t *)XX_Malloc(p_Header->size);
    if (!p_PadBuffer)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the buffer"));
        return E_NO_MEMORY;
    }

    if( (p_Header->validFields & e_FIELD_PADDING_SHIM_HEADER) && p_Header->p_ShimHeader)
    {
        /* Copy the user defined header */
        memcpy(p_PadBuffer, p_Header->p_ShimHeader, p_Header->size);
    }
    else
    {
       /* Fill the buffer with padding value */
       memset(p_PadBuffer, p_Header->value, p_Header->size);
    }

    /* Serialize the padding header */
    if ((INTERNAL_SerializeValue((uint8_t*)(p_PadBuffer), p_Header->size, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK))
    {
        return E_NOT_IN_RANGE;
    }

    XX_Free(p_PadBuffer);

    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_Padding_GetHeaderSize(t_Handle h_Header)
{
    t_PgPaddingHeader * pHeaderStr = (t_PgPaddingHeader*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    return pHeaderStr->size;
}

t_Error HDR_Padding_Init(t_Handle h_Header)
{

   t_PgPaddingHeader *p_Header             = NULL;         /* Padding header pointer */

   if (!h_Header)
   {
       RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
   }

    p_Header = (t_PgPaddingHeader*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

   /* Based on the size, configure the size */
     if (!(p_Header->validFields &  e_FIELD_PADDING_VALUE))
       p_Header->value = DEFAULT_PADDING_VALUE;

   return E_OK;
 }

t_Handle HDR_Padding_GetHeader(t_Handle hFrame)
{
   /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;

    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->paddingHeader);
}

t_Error HDR_Padding_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}
/** @} */ /* end of Padding group */

/**************************************************************************//**
@Group          IEEE 1588

@Description    IEEE 1588 Messages functions

@{
*//***************************************************************************/
t_Handle HDR_IEEE1588Message_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_IEEE1588Message *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return NULL;
    }

    p_Header = (t_IEEE1588Message*)MEMALLOC(sizeof(t_IEEE1588Message));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_IEEE1588Message));

    FLAG_SET(p_Header->autoFlags, e_FIELD_IEEE1588_V1_VERSION|e_FIELD_IEEE1588_V2_VERSION);

    /* Initialize the user configured IEEE 1588 Message header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(u_PgIeee1588MessageHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_IEEE1588Message_Clone(t_Handle h_Object)
{
    t_IEEE1588Message *p_Object   = (t_IEEE1588Message*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_IEEE1588Message *p_Header = (t_IEEE1588Message*)MEMALLOC(sizeof(t_IEEE1588Message));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_IEEE1588Message));

    /* Take a copy of the IEEE1588 header */
    memcpy(p_Header, p_Object, sizeof(t_IEEE1588Message));

    return p_Header;
}

t_Error HDR_IEEE1588Message_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_IEEE1588Message *p_Header   = (t_IEEE1588Message*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;         /* Frame struct header handle */
    t_PgIeee1588V1Message          *v1MessageHeader;           /* IEEE 1588 Version1 Message Header */
    t_PgIeee1588V2Message          *v2MessageHeader;           /* IEEE 1588 Version2 Message Header */
    uint8_t value8;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_MESSAGE)
    {
        v1MessageHeader = &p_Header->data.ieee1588V1MessageHeader;

        switch (fieldID)
        {
            case e_FIELD_IEEE1588_V1_VERSION:
                SAFE_COPY_VALUE(&v1MessageHeader->version, h_Value, sizeof(v1MessageHeader->version), size);
            break;
            case e_FIELD_IEEE1588_V1_VERSION_NETWORK:
                SAFE_COPY_VALUE(&v1MessageHeader->versionNetwork, h_Value, sizeof(v1MessageHeader->versionNetwork), size);
            break;
            case e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM:
                SAFE_COPY_VALUE(v1MessageHeader->subDomainNum, h_Value, sizeof(v1MessageHeader->subDomainNum), size);
            break;
            case e_FIELD_IEEE1588_V1_MESSAGE_TYPE:
                SAFE_COPY_VALUE(&v1MessageHeader->messageType, h_Value, sizeof(v1MessageHeader->messageType), size);
            break;
            case e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH:
                SAFE_COPY_VALUE(&v1MessageHeader->sourceCommTech, h_Value, sizeof(v1MessageHeader->sourceCommTech), size);
            break;
            case e_FIELD_IEEE1588_V1_SOURCE_U_UID:
                SAFE_COPY_VALUE(v1MessageHeader->sourceUuid, h_Value, sizeof(v1MessageHeader->sourceUuid), size);
            break;
            case e_FIELD_IEEE1588_V1_SOURCE_PORT_ID:
                SAFE_COPY_VALUE(&v1MessageHeader->sourcePortID, h_Value, sizeof(v1MessageHeader->sourcePortID), size);
            break;
            case e_FIELD_IEEE1588_V1_SEQUENCE_ID:
                SAFE_COPY_VALUE(&v1MessageHeader->sequenceID, h_Value, sizeof(v1MessageHeader->sequenceID), size);
            break;
            case e_FIELD_IEEE1588_V1_CONTROL:
                SAFE_COPY_VALUE(&v1MessageHeader->control, h_Value, sizeof(v1MessageHeader->control), size);
            break;
            case e_FIELD_IEEE1588_V1_FLAGS:
                SAFE_COPY_VALUE(&v1MessageHeader->flags, h_Value, sizeof(v1MessageHeader->flags), size);
            break;
            case e_FIELD_IEEE1588_V1_NEXT_HEADER_TYPE:
                SAFE_COPY_VALUE(&v1MessageHeader->nextHeaderType, h_Value, sizeof(v1MessageHeader->nextHeaderType), size);
            break;
            case e_FIELD_IEEE1588_V1_VALID_FIELDS:
                SAFE_COPY_VALUE(&v1MessageHeader->validFields, h_Value, sizeof(v1MessageHeader->validFields), size);
            break;

            default:
            {
                RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
            }
        }
    }
    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_MESSAGE)
    {
        v2MessageHeader = &p_Header->data.ieee1588V2MessageHeader;

        switch (fieldID)
        {
            case e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC:
                value8 = (uint8_t)v2MessageHeader->transportSpecific;
                SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
            break;
            case e_FIELD_IEEE1588_V2_MESSAGE_TYPE:
                value8 = (uint8_t)v2MessageHeader->messageType;
                SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
            break;
            case e_FIELD_IEEE1588_V2_VERSION:
                SAFE_COPY_VALUE(&v2MessageHeader->version, h_Value, sizeof(v2MessageHeader->version), size);
            break;
            case e_FIELD_IEEE1588_V2_MESSAGE_LENGTH:
                SAFE_COPY_VALUE(&v2MessageHeader->messageLength, h_Value, sizeof(v2MessageHeader->messageLength), size);
            break;
            case e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM:
                SAFE_COPY_VALUE(&v2MessageHeader->subDomainNum, h_Value, sizeof(v2MessageHeader->subDomainNum), size);
            break;
            case e_FIELD_IEEE1588_V2_FLAGS:
                SAFE_COPY_VALUE(&v2MessageHeader->flags, h_Value, sizeof(v2MessageHeader->flags), size);
            break;
            case e_FIELD_IEEE1588_V2_CORRECTION_FIELD:
                SAFE_COPY_VALUE(v2MessageHeader->correctionField, h_Value, sizeof(v2MessageHeader->correctionField), size);
            break;
            case e_FIELD_IEEE1588_V2_CLOCK_IDENTITY:
                SAFE_COPY_VALUE(v2MessageHeader->clockIdentity, h_Value, sizeof(v2MessageHeader->clockIdentity), size);
            break;
            case e_FIELD_IEEE1588_V2_SOURCE_PORT_ID:
                SAFE_COPY_VALUE(&v2MessageHeader->sourcePortID, h_Value, sizeof(v2MessageHeader->sourcePortID), size);
            break;
            case e_FIELD_IEEE1588_V2_SEQUENCE_ID:
                SAFE_COPY_VALUE(&v2MessageHeader->sequenceID, h_Value, sizeof(v2MessageHeader->sequenceID), size);
            break;
            case e_FIELD_IEEE1588_V2_CONTROL:
                SAFE_COPY_VALUE(&v2MessageHeader->control, h_Value, sizeof(v2MessageHeader->control), size);
            break;
            case e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD:
                SAFE_COPY_VALUE(&v2MessageHeader->logMessagePeriod, h_Value, sizeof(v2MessageHeader->logMessagePeriod), size);
            break;
            case e_FIELD_IEEE1588_V2_NEXT_HEADER_TYPE:
                SAFE_COPY_VALUE(&v2MessageHeader->nextHeaderType, h_Value, sizeof(v2MessageHeader->nextHeaderType), size);
            break;
            case e_FIELD_IEEE1588_V2_VALID_FIELDS:
                SAFE_COPY_VALUE(&v2MessageHeader->validFields, h_Value, sizeof(v2MessageHeader->validFields), size);
            break;

            default:
            {
                RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
            }
        }
    }

    return E_OK;
}

t_Error HDR_IEEE1588Message_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_IEEE1588Message *p_Header   = (t_IEEE1588Message*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;         /* Frame struct header handle */
    t_PgIeee1588V1Message          *v1MessageHeader;           /* IEEE 1588 Version1 Message Header */
    t_PgIeee1588V2Message          *v2MessageHeader;           /* IEEE 1588 Version2 Message Header */
    uint8_t value8;

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */

    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_MESSAGE)
    {
        v1MessageHeader = &p_Header->data.ieee1588V1MessageHeader;

        switch (fieldID)
        {
            case e_FIELD_IEEE1588_V1_VERSION:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->version, sizeof(v1MessageHeader->version), size);
            break;
            case e_FIELD_IEEE1588_V1_VERSION_NETWORK:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->versionNetwork, sizeof(v1MessageHeader->versionNetwork), size);
            break;
            case e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM:
                SAFE_COPY_VALUE(h_Value, v1MessageHeader->subDomainNum, sizeof(v1MessageHeader->subDomainNum), size);
            break;
            case e_FIELD_IEEE1588_V1_MESSAGE_TYPE:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->messageType, sizeof(v1MessageHeader->messageType), size);
            break;
            case e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->sourceCommTech, sizeof(v1MessageHeader->sourceCommTech), size);
            break;
            case e_FIELD_IEEE1588_V1_SOURCE_U_UID:
                SAFE_COPY_VALUE(h_Value, v1MessageHeader->sourceUuid, sizeof(v1MessageHeader->sourceUuid), size);
            break;
            case e_FIELD_IEEE1588_V1_SOURCE_PORT_ID:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->sourcePortID, sizeof(v1MessageHeader->sourcePortID), size);
            break;
            case e_FIELD_IEEE1588_V1_SEQUENCE_ID:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->sequenceID, sizeof(v1MessageHeader->sequenceID), size);
            break;
            case e_FIELD_IEEE1588_V1_CONTROL:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->control, sizeof(v1MessageHeader->control), size);
            break;
            case e_FIELD_IEEE1588_V1_FLAGS:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->flags, sizeof(v1MessageHeader->flags), size);
            break;
            case e_FIELD_IEEE1588_V1_NEXT_HEADER_TYPE:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->nextHeaderType, sizeof(v1MessageHeader->nextHeaderType), size);
            break;
            case e_FIELD_IEEE1588_V1_VALID_FIELDS:
                SAFE_COPY_VALUE(h_Value, &v1MessageHeader->validFields, sizeof(v1MessageHeader->validFields), size);
            break;

            default:
            {
                RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
            }
        }
    }
    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_MESSAGE)
    {
        v2MessageHeader = &p_Header->data.ieee1588V2MessageHeader;

        switch (fieldID)
        {
            case e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC:
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 v2MessageHeader->transportSpecific = value8;
            break;
            case e_FIELD_IEEE1588_V2_MESSAGE_TYPE:
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 v2MessageHeader->messageType = value8;
            break;
            case e_FIELD_IEEE1588_V2_VERSION:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->version, sizeof(v2MessageHeader->version), size);
            break;
            case e_FIELD_IEEE1588_V2_MESSAGE_LENGTH:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->messageLength, sizeof(v2MessageHeader->messageLength), size);
            break;
            case e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->subDomainNum, sizeof(v2MessageHeader->subDomainNum), size);
            break;
            case e_FIELD_IEEE1588_V2_FLAGS:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->flags, sizeof(v2MessageHeader->flags), size);
            break;
            case e_FIELD_IEEE1588_V2_CORRECTION_FIELD:
                SAFE_COPY_VALUE(h_Value, v2MessageHeader->correctionField, sizeof(v2MessageHeader->correctionField), size);
            break;
            case e_FIELD_IEEE1588_V2_CLOCK_IDENTITY:
                SAFE_COPY_VALUE(h_Value, v2MessageHeader->clockIdentity, sizeof(v2MessageHeader->clockIdentity), size);
            break;
            case e_FIELD_IEEE1588_V2_SOURCE_PORT_ID:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->sourcePortID, sizeof(v2MessageHeader->sourcePortID), size);
            break;
            case e_FIELD_IEEE1588_V2_SEQUENCE_ID:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->sequenceID, sizeof(v2MessageHeader->sequenceID), size);
            break;
            case e_FIELD_IEEE1588_V2_CONTROL:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->control, sizeof(v2MessageHeader->control), size);
            break;
            case e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->logMessagePeriod, sizeof(v2MessageHeader->logMessagePeriod), size);
            break;
            case e_FIELD_IEEE1588_V2_NEXT_HEADER_TYPE:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->nextHeaderType, sizeof(v2MessageHeader->nextHeaderType), size);
            break;
            case e_FIELD_IEEE1588_V2_VALID_FIELDS:
                SAFE_COPY_VALUE(h_Value, &v2MessageHeader->validFields, sizeof(v2MessageHeader->validFields), size);
            break;

            default:
            {
                RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
            }
        }
    }

    return E_OK;
}

t_Error HDR_IEEE1588Message_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_IEEE1588Message* p_Header = (t_IEEE1588Message*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
       case e_FIELD_IEEE1588_V1_VERSION:
       case e_FIELD_IEEE1588_V2_VERSION:
       break;
       default:
       {
           RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
       FLAG_SET(p_Header->autoFlags, fieldID);
    else
       FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_IEEE1588Message_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
   t_IEEE1588Message *p_Header = (t_IEEE1588Message*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
   u_PgIeee1588MessageHeader *pValues = (u_PgIeee1588MessageHeader*)hValues;
   t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;  /* Frame struct header handle */
   uint8_t value8 = 0;

    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_MESSAGE)
    {

        if (!(fieldIDs & e_FIELD_IEEE1588_V1_ALL_FIELDS))
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }

        /* Configure the IEEE 1588 V1 Message header fields specified by fieldIDs */
        if (fieldIDs & e_FIELD_IEEE1588_V1_VERSION)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_VERSION, (t_Handle)&pValues->ieee1588V1MessageHeader.version, sizeof(pValues->ieee1588V1MessageHeader.version));

        if (fieldIDs & e_FIELD_IEEE1588_V1_VERSION_NETWORK)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_VERSION_NETWORK, (t_Handle)&pValues->ieee1588V1MessageHeader.versionNetwork, sizeof(pValues->ieee1588V1MessageHeader.versionNetwork));

        if (fieldIDs & e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM, (t_Handle)&pValues->ieee1588V1MessageHeader.subDomainNum, sizeof(pValues->ieee1588V1MessageHeader.subDomainNum));

        if (fieldIDs & e_FIELD_IEEE1588_V1_MESSAGE_TYPE)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_MESSAGE_TYPE, (t_Handle)&pValues->ieee1588V1MessageHeader.messageType, sizeof(pValues->ieee1588V1MessageHeader.messageType));

        if(fieldIDs & e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH, (t_Handle)&pValues->ieee1588V1MessageHeader.sourceCommTech, sizeof(pValues->ieee1588V1MessageHeader.sourceCommTech));

        if(fieldIDs & e_FIELD_IEEE1588_V1_SOURCE_U_UID)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_SOURCE_U_UID, (t_Handle)&pValues->ieee1588V1MessageHeader.sourceUuid, sizeof(pValues->ieee1588V1MessageHeader.sourceUuid));

        if (fieldIDs & e_FIELD_IEEE1588_V1_SOURCE_PORT_ID)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_SOURCE_PORT_ID, (t_Handle)&pValues->ieee1588V1MessageHeader.sourcePortID,sizeof(pValues->ieee1588V1MessageHeader.sourcePortID));

        if(fieldIDs & e_FIELD_IEEE1588_V1_SEQUENCE_ID)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_SEQUENCE_ID, (t_Handle)&pValues->ieee1588V1MessageHeader.sequenceID, sizeof(pValues->ieee1588V1MessageHeader.sequenceID));

        if (fieldIDs & e_FIELD_IEEE1588_V1_CONTROL)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_CONTROL, (t_Handle)&pValues->ieee1588V1MessageHeader.control,sizeof(pValues->ieee1588V1MessageHeader.control));

        if (fieldIDs & e_FIELD_IEEE1588_V1_FLAGS)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_FLAGS, (t_Handle)&pValues->ieee1588V1MessageHeader.flags,sizeof(pValues->ieee1588V1MessageHeader.flags));

        if (fieldIDs & e_FIELD_IEEE1588_V1_NEXT_HEADER_TYPE)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_NEXT_HEADER_TYPE, (t_Handle)&pValues->ieee1588V1MessageHeader.nextHeaderType,sizeof(pValues->ieee1588V1MessageHeader.nextHeaderType));

        if (fieldIDs & e_FIELD_IEEE1588_V1_VALID_FIELDS)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V1_VALID_FIELDS, (t_Handle)&pValues->ieee1588V1MessageHeader.validFields,sizeof(pValues->ieee1588V1MessageHeader.validFields));

    }

    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_MESSAGE)
    {
        if (!(fieldIDs & e_FIELD_IEEE1588_V2_ALL_FIELDS))
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }

        /* Configure the IEEE 1588 V2 Message header fields specified by fieldIDs */
        if (fieldIDs & e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC)
        {
           value8 = (uint8_t)pValues->ieee1588V2MessageHeader.transportSpecific;
           HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC, (t_Handle)&value8, sizeof(value8));
        }

        if (fieldIDs & e_FIELD_IEEE1588_V2_MESSAGE_TYPE)
        {
           value8 = (uint8_t)pValues->ieee1588V2MessageHeader.messageType;
           HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_MESSAGE_TYPE, (t_Handle)&value8, sizeof(value8));
        }

        if (fieldIDs & e_FIELD_IEEE1588_V2_VERSION)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_VERSION, (t_Handle)&pValues->ieee1588V2MessageHeader.version, sizeof(pValues->ieee1588V2MessageHeader.version));

        if (fieldIDs & e_FIELD_IEEE1588_V2_MESSAGE_LENGTH)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_MESSAGE_LENGTH, (t_Handle)&pValues->ieee1588V2MessageHeader.messageLength, sizeof(pValues->ieee1588V2MessageHeader.messageLength));

        if (fieldIDs & e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM, (t_Handle)&pValues->ieee1588V2MessageHeader.subDomainNum, sizeof(pValues->ieee1588V2MessageHeader.subDomainNum));

        if(fieldIDs & e_FIELD_IEEE1588_V2_FLAGS)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_FLAGS, (t_Handle)&pValues->ieee1588V2MessageHeader.flags, sizeof(pValues->ieee1588V2MessageHeader.flags));

        if(fieldIDs & e_FIELD_IEEE1588_V2_CORRECTION_FIELD)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_CORRECTION_FIELD, (t_Handle)&pValues->ieee1588V2MessageHeader.correctionField, sizeof(pValues->ieee1588V2MessageHeader.correctionField));

        if (fieldIDs & e_FIELD_IEEE1588_V2_CLOCK_IDENTITY)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_CLOCK_IDENTITY, (t_Handle)&pValues->ieee1588V2MessageHeader.clockIdentity,sizeof(pValues->ieee1588V2MessageHeader.clockIdentity));

        if(fieldIDs & e_FIELD_IEEE1588_V2_SOURCE_PORT_ID)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_SOURCE_PORT_ID, (t_Handle)&pValues->ieee1588V2MessageHeader.sourcePortID, sizeof(pValues->ieee1588V2MessageHeader.sourcePortID));

        if (fieldIDs & e_FIELD_IEEE1588_V2_SEQUENCE_ID)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_SEQUENCE_ID, (t_Handle)&pValues->ieee1588V2MessageHeader.sequenceID,sizeof(pValues->ieee1588V2MessageHeader.sequenceID));

        if (fieldIDs & e_FIELD_IEEE1588_V2_CONTROL)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_CONTROL, (t_Handle)&pValues->ieee1588V2MessageHeader.control,sizeof(pValues->ieee1588V2MessageHeader.control));

        if (fieldIDs & e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD, (t_Handle)&pValues->ieee1588V2MessageHeader.logMessagePeriod,sizeof(pValues->ieee1588V2MessageHeader.logMessagePeriod));

        if (fieldIDs & e_FIELD_IEEE1588_V2_NEXT_HEADER_TYPE)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_NEXT_HEADER_TYPE, (t_Handle)&pValues->ieee1588V2MessageHeader.nextHeaderType,sizeof(pValues->ieee1588V2MessageHeader.nextHeaderType));

        if (fieldIDs & e_FIELD_IEEE1588_V2_VALID_FIELDS)
            HDR_IEEE1588Message_ConfigField(h_Header, e_FIELD_IEEE1588_V2_VALID_FIELDS, (t_Handle)&pValues->ieee1588V2MessageHeader.validFields,sizeof(pValues->ieee1588V2MessageHeader.validFields));

    }
    return E_OK;
}

t_Error HDR_IEEE1588Message_Randomize(t_Handle h_Header)
{
    /* Get the handle to the IEEE 1588Message header */
    t_IEEE1588Message* pHeaderStr = (t_IEEE1588Message*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t    index = 0; /* Loop index */
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;;    /* Frame struct header handle */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_MESSAGE)
    {
        /* Generate random values to the fields not configured by the user */
       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_VERSION))
       {
           pHeaderStr->data.ieee1588V1MessageHeader.version = 0x1;
           FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IEEE1588_V1_VERSION);
       }

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_VERSION_NETWORK))
           pHeaderStr->data.ieee1588V1MessageHeader.versionNetwork = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_SUB_DOMAIN_NUM))
       {
           for (index=0; index < IEEE1588_V1_SUB_DOMIAN_NUM_SIZE; index++)
             pHeaderStr->data.ieee1588V1MessageHeader.subDomainNum[index] = (uint8_t)rand();
       }

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_MESSAGE_TYPE))
           pHeaderStr->data.ieee1588V1MessageHeader.messageType = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_SOURCE_COMM_TECH))
           pHeaderStr->data.ieee1588V1MessageHeader.sourceCommTech = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_SOURCE_U_UID))
       {
           for (index=0; index < IEEE1588_V1_SOURCE_UUID_SIZE; index++)
             pHeaderStr->data.ieee1588V1MessageHeader.sourceUuid[index] = (uint8_t)rand();
       }

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_SOURCE_PORT_ID))
           pHeaderStr->data.ieee1588V1MessageHeader.sourcePortID = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_SEQUENCE_ID))
           pHeaderStr->data.ieee1588V1MessageHeader.sequenceID = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_CONTROL))
           pHeaderStr->data.ieee1588V1MessageHeader.control = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V1MessageHeader.validFields & e_FIELD_IEEE1588_V1_FLAGS))
           pHeaderStr->data.ieee1588V1MessageHeader.flags = (uint16_t)rand();
    }

    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_MESSAGE)
    {
        /* Generate random values to the fields not configured by the user */
       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_TRANSPORT_SPECIFIC))
           pHeaderStr->data.ieee1588V2MessageHeader.transportSpecific = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_MESSAGE_TYPE))
           pHeaderStr->data.ieee1588V2MessageHeader.messageType = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_VERSION))
       {
           pHeaderStr->data.ieee1588V2MessageHeader.version = 0x2;
           FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IEEE1588_V2_VERSION);
       }
       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_MESSAGE_LENGTH))
           pHeaderStr->data.ieee1588V2MessageHeader.messageLength = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_SUB_DOMAIN_NUM))
             pHeaderStr->data.ieee1588V2MessageHeader.subDomainNum = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_FLAGS))
           pHeaderStr->data.ieee1588V2MessageHeader.flags = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_CORRECTION_FIELD))
       {
           for (index=0; index < IEEE1588_V2_CORRECTION_FIELD_SIZE; index++)
             pHeaderStr->data.ieee1588V2MessageHeader.correctionField[index] = (uint8_t)rand();
       }

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_CLOCK_IDENTITY))
       {
           for (index=0; index < IEEE1588_V2_CLOCK_IDENTITY_SIZE; index++)
             pHeaderStr->data.ieee1588V2MessageHeader.clockIdentity[index] = (uint8_t)rand();
       }

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_SOURCE_PORT_ID))
           pHeaderStr->data.ieee1588V2MessageHeader.sourcePortID = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_SEQUENCE_ID))
           pHeaderStr->data.ieee1588V2MessageHeader.sequenceID = (uint16_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_CONTROL))
           pHeaderStr->data.ieee1588V2MessageHeader.control = (uint8_t)rand();

       if (!(pHeaderStr->data.ieee1588V2MessageHeader.validFields & e_FIELD_IEEE1588_V2_LOG_MESSAGE_PERIOD))
           pHeaderStr->data.ieee1588V2MessageHeader.logMessagePeriod = (uint8_t)rand();

    }
    return E_OK;
}

t_Error HDR_IEEE1588Message_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
   /* Get the handle to the IEEE 1588 Message header */
   t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
   t_IEEE1588Message* p_Header = (t_IEEE1588Message*)p_Frame->h_HeaderStruct;
   uint8_t     value8  = 0;
   uint32_t    value32 = 0;

   /* Check the handle */
   if (!(p_Frame && h_Header && p_Buffer && pSize))
   {
       RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
   }

   /* Serialize the values */
    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_MESSAGE)
    {
       if ( (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader), sizeof(uint32_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader.subDomainNum), sizeof(p_Header->data.ieee1588V1MessageHeader.subDomainNum), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader.messageType), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader.sourceUuid), sizeof(p_Header->data.ieee1588V1MessageHeader.sourceUuid), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader.sourcePortID), sizeof(uint32_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader.control), sizeof(p_Header->data.ieee1588V1MessageHeader.control), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&value8), sizeof(value8), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V1MessageHeader.flags), sizeof(p_Header->data.ieee1588V1MessageHeader.flags), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) )
            {
               return E_NOT_IN_RANGE;
            }
      }
    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_MESSAGE)
    {
       if ( (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V2MessageHeader), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V2MessageHeader.messageLength), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V2MessageHeader.subDomainNum), sizeof(uint8_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&value8), sizeof(value8), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V2MessageHeader.flags), sizeof(p_Header->data.ieee1588V2MessageHeader.flags), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.ieee1588V2MessageHeader.correctionField), sizeof(p_Header->data.ieee1588V2MessageHeader.correctionField), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&value32), sizeof(value32), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.ieee1588V2MessageHeader.clockIdentity), sizeof(p_Header->data.ieee1588V2MessageHeader.clockIdentity), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V2MessageHeader.sourcePortID), sizeof(uint32_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
            (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ieee1588V2MessageHeader.control), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK))
            {
               return E_NOT_IN_RANGE;
            }
    }
      /* Serialize the rest of the frame */
       return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_IEEE1588Message_GetHeaderSize(t_Handle h_Header)
{
   t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
   uint32_t size = 0;

   /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return 0;
    }

    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_MESSAGE)
    {
       size = FIXED_HEADER_SIZE_IEEE1588_V1_MESSAGE;
    }
    if(p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_MESSAGE)
    {
       size = FIXED_HEADER_SIZE_IEEE1588_V2_MESSAGE;
    }

    return size;
}

t_Error HDR_IEEE1588Message_Init(t_Handle h_Header)
{
    UNUSED(h_Header);

    return E_OK;
}

t_Handle HDR_IEEE1588Message_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;

    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return((t_Handle)(&p_Frame->ieee1588Header));

}

t_Error HDR_IEEE1588Message_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
   UNUSED(hHeader); UNUSED(hError);
   return E_OK;
}

/** @} */ /* end of IEEE 1588 group */

/**************************************************************************//**
@Group          VLAN

@Description    VLAN functions

@{
*//***************************************************************************/
t_Handle HDR_VLAN_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PgVlanHeader *p_VLANHeader = (t_PgVlanHeader*)hHeader;
    t_VLAN *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!p_VLANHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_VLAN*)MEMALLOC(sizeof(t_VLAN));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_VLAN));

    p_Header->bIsType = TRUE;

    FLAG_SET(p_Header->autoFlags, e_FIELD_VLAN_LENGTH | e_FIELD_VLAN_TYPE);

    /* Initialize the user configured VLAN header fields in the header */
    memcpy(&p_Header->data, &p_VLANHeader[vlanIndex++], sizeof(t_PgVlanHeader));

    if ((p_Header->data.nextHeaderType != e_HEADER_TYPE_VLAN) &&
        (p_Header->data.nextHeaderType != e_HEADER_TYPE_VTAGGED_802_3))
    {
        vlanIndex = 0;
    }

    return (t_Handle) p_Header;
}

t_Handle HDR_VLAN_Clone(t_Handle h_Object)
{
    t_VLAN *p_Object = (t_VLAN*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_VLAN* p_Header = (t_VLAN*)MEMALLOC(sizeof(t_VLAN));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_VLAN));

    /* Take a copy of the VLAN header */
    memcpy(p_Header, p_Object, sizeof(t_VLAN));

    return p_Header;
}

t_Error HDR_VLAN_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_VLAN *p_Header = (t_VLAN*)((t_FrameStruct*)h_Header)->h_HeaderStruct; /* VLAN header handle */
    uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;      /* Holds the bit fields of size between 8 and 16 bits */

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_VLAN_VPRI:
        {
            value8 = (uint8_t)p_Header->data.vPri;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_VLAN_CFI:
        {
            value8 = (uint8_t)p_Header->data.cfi;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_VLAN_VID:
        {
            value16 = (uint16_t)p_Header->data.vid;
            SAFE_COPY_VALUE(&value16, h_Value, sizeof(value16), size);
        }
        break;
        case e_FIELD_VLAN_LENGTH:
        case e_FIELD_VLAN_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.typeLen, h_Value, sizeof(p_Header->data.typeLen), size);
        break;
        case e_FIELD_VLAN_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_VLAN_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error     HDR_VLAN_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_VLAN *p_Header = (t_VLAN*)((t_FrameStruct*)h_Header)->h_HeaderStruct; /* VLAN header handle */
    uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;      /* Holds the bit fields of size between 8 and 16 bits */

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_VLAN_VPRI:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.vPri = value8;
        }
        break;
        case e_FIELD_VLAN_CFI:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.cfi = value8;
        }
        break;
        case e_FIELD_VLAN_VID:
        {
            SAFE_COPY_VALUE(h_Value, &value16, sizeof(value16), size);
            p_Header->data.vid = value16;
        }
        break;
        case e_FIELD_VLAN_LENGTH:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.typeLen, sizeof(p_Header->data.typeLen), size);
            p_Header->bIsType = FALSE;
        }break;
        case e_FIELD_VLAN_TYPE:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.typeLen, sizeof(p_Header->data.typeLen), size);
            p_Header->bIsType = TRUE;
        }break;
        case e_FIELD_VLAN_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_VLAN_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_VLAN_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_VLAN *p_Header = (t_VLAN*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_VLAN_LENGTH:
        case e_FIELD_VLAN_TYPE:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
       FLAG_SET(p_Header->autoFlags, fieldID);
    else
       FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_VLAN_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgVlanHeader* pValues = (t_PgVlanHeader*)hValues;
    uint8_t value8 = 0;
    uint16_t value16 = 0;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_VLAN_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the VLAN header fields specified by fieldIDs */
    if (fieldIDs & e_FIELD_VLAN_VPRI)
    {
       value8 = (uint8_t)pValues->vPri;
       HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_VPRI, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_VLAN_CFI)
    {
       value8 = (uint8_t)pValues->cfi;
       HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_CFI, (t_Handle)&value8, sizeof(value8));
    }

    /* If both Length and Type are set, the Type has priority (overwrites Length) */
    if (fieldIDs & e_FIELD_VLAN_VID)
    {
       value16 = (uint16_t)pValues->vid;
       HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_VID, (t_Handle)&value16, sizeof(value16));
    }

    if (fieldIDs & e_FIELD_VLAN_LENGTH)
        HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_LENGTH, (t_Handle)&pValues->typeLen, sizeof(pValues->typeLen));

    if (fieldIDs & e_FIELD_VLAN_TYPE)
        HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_TYPE, (t_Handle)&pValues->typeLen, sizeof(pValues->typeLen));

    if (fieldIDs & e_FIELD_VLAN_NEXT_HEADER_TYPE)
        HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_VLAN_VALID_FIELDS)
        HDR_VLAN_ConfigField(h_Header, e_FIELD_VLAN_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error  HDR_VLAN_Randomize(t_Handle h_Header)
{
    t_VLAN* p_Header = NULL;
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;  /* Frame struct header handle */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = ((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Generate random values for the fields not configured by the user */
    if (!(p_Header->data.validFields & e_FIELD_VLAN_VPRI))
        p_Header->data.vPri = (uint8_t)(rand()%8);

    if (!(p_Header->data.validFields & e_FIELD_VLAN_CFI))
        p_Header->data.cfi = (uint8_t)rand();

    if (!(p_Header->data.validFields & e_FIELD_VLAN_VID))
        p_Header->data.vid = (uint16_t)rand();

    /* The type has priority over length (length is overwriten)*/
    /* if both Length and Type flags are set then the Type has priority (overwrites Length) */
    if (!(p_Header->data.validFields & e_FIELD_VLAN_LENGTH) &&
    !(p_Header->data.validFields & e_FIELD_VLAN_TYPE))
    {
        if(p_Frame->HeaderID == e_HEADER_TYPE_VTAGGED_802_3)
            p_Header->bIsType = FALSE;
        else
        {
            p_Header->data.typeLen = (uint16_t)(MAX_VLAN_LEN_VAL + (rand()%200));
            p_Header->bIsType = TRUE;
        }
    }
    else if(p_Header->data.validFields & e_FIELD_VLAN_LENGTH)
    {
        p_Header->bIsType = FALSE;
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_VLAN_LENGTH);
    }
    else if(p_Header->data.validFields & e_FIELD_VLAN_TYPE)
    {
        p_Header->bIsType = TRUE;
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_VLAN_TYPE);
    }

    return E_OK;
}

t_Error HDR_VLAN_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_VLAN *p_Header = (t_VLAN*)((t_FrameStruct*)h_Header)->h_HeaderStruct; /* VLAN header pointer */
    t_Handle p_NextHeader = NULL; /* Next header handle */

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Compute the frame size if needed */
    if (!p_Header->bIsType && FLAG_CHECK(p_Header->autoFlags, e_FIELD_VLAN_LENGTH))
    {
        p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(h_Header);
        if (p_NextHeader)
            p_Header->data.typeLen = (uint16_t)FRAME_GetFrameSize(p_NextHeader, p_NextHeader);
        else
            p_Header->data.typeLen = 0;
    }

    /* Serialize the stored values */
    if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.typeLen), sizeof(p_Header->data.typeLen), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_VLAN_GetHeaderSize(t_Handle h_Header)
{
    UNUSED(h_Header);

    return (FIXED_HEADER_SIZE_VLAN);
}

t_Error  HDR_VLAN_Init(t_Handle h_Header)
{
    t_VLAN *p_Header = NULL;            /* VLAN header pointer */
    t_FrameStruct *p_NextHeader = NULL; /* Next header pointer */
    bool bSessionStarted = FALSE;       /* Status of PPPoE session */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_VLAN*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Based on the next header, configure the type */
    if (p_NextHeader)
    {
        if ((p_Header->bIsType) && FLAG_CHECK(p_Header->autoFlags , e_FIELD_VLAN_TYPE))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_VLAN || p_NextHeader->HeaderID == e_HEADER_TYPE_VTAGGED_802_3 )
                p_Header->data.typeLen = 0x8100;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv4)
                p_Header->data.typeLen = 0x0800;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv6)
                p_Header->data.typeLen = 0x86DD;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_PPPoE)
            {
                HDR_PPPoE_GetField( p_NextHeader, e_FIELD_PPPoE_SESSION,
                                    &bSessionStarted, sizeof(bool));
                p_Header->data.typeLen=(uint16_t)(bSessionStarted ?
                                                 0x8864 : 0x8863); // 0x8863 - discovery stage
                                                                   // 0x8864 - session stage
            }
            if(p_NextHeader->HeaderID == e_HEADER_TYPE_MPLS)
                p_Header->data.typeLen = 0x8847;

        }
    }

    /* if the next header is null or of an unknown type, leave the type field as it was */
    return E_OK;
}

t_Handle HDR_VLAN_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.vlanHeader);
}

t_Error HDR_VLAN_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of VLAN group */

/**************************************************************************//**
@Group          LLC

@Description    LLC functions

@{
*//***************************************************************************/
t_Handle HDR_LLC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PgLlcHeader *p_LLCHeader = (t_PgLlcHeader*)hHeader;
    t_LLC *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_LLC*)MEMALLOC(sizeof(t_LLC));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_LLC));

    FLAG_SET(p_Header->autoFlags, e_FIELD_LLC_DSAP | e_FIELD_LLC_SSAP | e_FIELD_LLC_CTRL);

    /* Initialize the user configured LLC header fields in the header */
    memcpy(&p_Header->data, &p_LLCHeader[llcIndex++], sizeof(t_PgLlcHeader));

    return (t_Handle) p_Header;

}

t_Handle HDR_LLC_Clone(t_Handle h_Object)
{
    t_LLC *p_Object = (t_LLC*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_LLC* p_Header = (t_LLC*)MEMALLOC(sizeof(t_LLC));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_LLC));

    /* Take a copy of the LLC header */
    memcpy(p_Header, p_Object, sizeof(t_LLC));

    return p_Header;
}

t_Error HDR_LLC_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_LLC *p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_LLC_DSAP:
            SAFE_COPY_VALUE(&p_Header->data.dsap, h_Value, sizeof(p_Header->data.dsap), size);
        break;
        case e_FIELD_LLC_SSAP:
            SAFE_COPY_VALUE(&p_Header->data.ssap, h_Value, sizeof(p_Header->data.ssap), size);
        break;
        case e_FIELD_LLC_CTRL:
            SAFE_COPY_VALUE(&p_Header->data.ctrl, h_Value, sizeof(p_Header->data.ctrl), size);
        break;
        case e_FIELD_LLC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_LLC_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_LLC_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_LLC *p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_LLC_DSAP:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.dsap, sizeof(p_Header->data.dsap), size);
        break;
        case e_FIELD_LLC_SSAP:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.ssap, sizeof(p_Header->data.ssap), size);
        break;
        case e_FIELD_LLC_CTRL:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.ctrl, sizeof(p_Header->data.ctrl), size);
        break;
        case e_FIELD_LLC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_LLC_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_LLC_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_LLC *p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_LLC_DSAP:
        case e_FIELD_LLC_SSAP:
        case e_FIELD_LLC_CTRL:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
       FLAG_SET(p_Header->autoFlags, fieldID);
    else
       FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_LLC_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgLlcHeader* pValues = (t_PgLlcHeader*)hValues;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_LLC_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the LLC header fields specified by fieldIDs */
    if(fieldIDs & e_FIELD_LLC_DSAP)
        HDR_LLC_ConfigField(h_Header, e_FIELD_LLC_DSAP, (t_Handle)&pValues->dsap, sizeof(pValues->dsap));

    if(fieldIDs & e_FIELD_LLC_SSAP)
        HDR_LLC_ConfigField(h_Header, e_FIELD_LLC_SSAP, (t_Handle)&pValues->ssap, sizeof(pValues->ssap));

    if(fieldIDs & e_FIELD_LLC_CTRL)
       HDR_LLC_ConfigField(h_Header, e_FIELD_LLC_CTRL,     (t_Handle)&pValues->ctrl, sizeof(pValues->ctrl));

    if (fieldIDs & e_FIELD_LLC_NEXT_HEADER_TYPE)
       HDR_LLC_ConfigField(h_Header, e_FIELD_LLC_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_LLC_VALID_FIELDS)
       HDR_LLC_ConfigField(h_Header, e_FIELD_LLC_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error HDR_LLC_Randomize(t_Handle h_Header)
{
    t_LLC *p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
    if(!(p_Header->data.validFields & e_FIELD_LLC_DSAP))
        p_Header->data.dsap = (uint8_t)(rand() % 0x100);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_LLC_DSAP);

    if(!(p_Header->data.validFields & e_FIELD_LLC_SSAP))
        p_Header->data.ssap = (uint8_t)(rand() % 0x100);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_LLC_SSAP);

    if(!(p_Header->data.validFields & e_FIELD_LLC_CTRL))
        p_Header->data.ctrl = (uint8_t)(rand() % 0x100);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_LLC_CTRL);

    return E_OK;
}

t_Error HDR_LLC_Init(t_Handle h_Header)
{
    t_LLC* p_Header = NULL;             /* LLC header pointer */
    t_FrameStruct *p_NextHeader = NULL; /* Next header pointer */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Based on the next header, configure DSAP, SSAP and CTRL */
    if (p_NextHeader)
    {
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_LLC_DSAP))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_NLPID)
                p_Header->data.dsap = 0xFE;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_SNAP)
                p_Header->data.dsap = 0xAA;
        }
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_LLC_SSAP))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_NLPID)
                p_Header->data.ssap = 0xFE;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_SNAP)
                p_Header->data.ssap = 0xAA;
        }
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_LLC_CTRL))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_NLPID)
                p_Header->data.ctrl = 0x03;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_SNAP)
                p_Header->data.ctrl = 0x03;
        }
    }

    /* If the next header is null or of an unknown type, leave the fields as they were */
    return E_OK;
}

t_Error HDR_LLC_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_LLC  *p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if(INTERNAL_SerializeValue((uint8_t*)&p_Header->data.dsap, sizeof(p_Header->data.dsap), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
       INTERNAL_SerializeValue((uint8_t*)&p_Header->data.ssap, sizeof(p_Header->data.ssap), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
       INTERNAL_SerializeValue((uint8_t*)&p_Header->data.ctrl, sizeof(p_Header->data.ctrl), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_LLC_GetHeaderSize(t_Handle h_Header)
{
    t_LLC *p_Header = (t_LLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return (uint32_t)3;
}

t_Handle HDR_LLC_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.llcHeader);
}

t_Error HDR_LLC_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of LLC group */


/**************************************************************************//**
@Group          NLPID

@Description    NLPID functions

@{
*//***************************************************************************/
t_Handle HDR_NLPID_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_NLPID *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_NLPID*)MEMALLOC(sizeof(t_NLPID));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_NLPID));

    FLAG_SET(p_Header->autoFlags, e_FIELD_NLPID_NLPID);

    /* Initialize the user configured NLPID header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgNlpidHeader));

    return (t_Handle) p_Header;

}

t_Handle HDR_NLPID_Clone(t_Handle h_Object)
{
    t_NLPID *p_Object = (t_NLPID*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_NLPID* p_Header = (t_NLPID*)MEMALLOC(sizeof(t_NLPID));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_NLPID));

    /* Take a copy of the NLPID header */
    memcpy(p_Header, p_Object, sizeof(t_NLPID));

    return p_Header;
}

t_Error HDR_NLPID_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_NLPID *p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_NLPID_NLPID:
            SAFE_COPY_VALUE(&p_Header->data.nlpid, h_Value, sizeof(p_Header->data.nlpid), size);
        break;
        case e_FIELD_NLPID_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_NLPID_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_NLPID_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_NLPID *p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_NLPID_NLPID:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nlpid, sizeof(p_Header->data.nlpid), size);
        break;
        case e_FIELD_NLPID_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_NLPID_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_NLPID_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_NLPID *p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_NLPID_NLPID:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
       FLAG_SET(p_Header->autoFlags, fieldID);
    else
       FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_NLPID_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgNlpidHeader* pValues = (t_PgNlpidHeader*)hValues;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_NLPID_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the NLPID header fields specified by fieldIDs */
    if(fieldIDs & e_FIELD_NLPID_NLPID)
        HDR_NLPID_ConfigField(h_Header, e_FIELD_NLPID_NLPID, (t_Handle)&pValues->nlpid, sizeof(pValues->nlpid));

    if (fieldIDs & e_FIELD_NLPID_NEXT_HEADER_TYPE)
        HDR_NLPID_ConfigField(h_Header, e_FIELD_NLPID_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_NLPID_VALID_FIELDS)
        HDR_NLPID_ConfigField(h_Header, e_FIELD_NLPID_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error HDR_NLPID_Randomize(t_Handle h_Header)
{
    t_NLPID *p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
    if(!(p_Header->data.validFields & e_FIELD_NLPID_NLPID))
        p_Header->data.nlpid = (uint8_t)(rand() % 0x100);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_NLPID_NLPID);

    return E_OK;
}

t_Error HDR_NLPID_Init(t_Handle h_Header)
{
    t_NLPID* p_Header = NULL;
    t_FrameStruct *p_NextHeader = NULL;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Based on the next header, configure the PID */
    if (p_NextHeader)
    {
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_NLPID_NLPID))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_PPP)
                p_Header->data.nlpid = 0xCF;
        }
    }

    return E_OK;
}

t_Error HDR_NLPID_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
   t_NLPID     *p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if(INTERNAL_SerializeValue((uint8_t*)&p_Header->data.nlpid, sizeof(p_Header->data.nlpid), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_NLPID_GetHeaderSize(t_Handle h_Header)
{
    t_NLPID *p_Header = (t_NLPID*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return (uint32_t)1;
}

t_Handle HDR_NLPID_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.nlpidHeader);
}

t_Error HDR_NLPID_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of NLPID group */

/**************************************************************************//**
@Group          SNAP

@Description    SNAP functions

@{
*//***************************************************************************/
t_Handle HDR_SNAP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PgSnapHeader *p_SNAPHeader = (t_PgSnapHeader*)hHeader;
    t_SNAP *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_SNAP*)MEMALLOC(sizeof(t_SNAP));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_SNAP));

    FLAG_SET(p_Header->autoFlags, e_FIELD_SNAP_OUI | e_FIELD_SNAP_PID);

    /* Initialize the user configured SNAP header fields in the header */
    memcpy(&p_Header->data, &p_SNAPHeader[snapIndex++], sizeof(t_PgSnapHeader));

    return (t_Handle) p_Header;

}

t_Handle HDR_SNAP_Clone(t_Handle h_Object)
{
    t_SNAP *p_Object = (t_SNAP*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_SNAP* p_Header = (t_SNAP*)MEMALLOC(sizeof(t_SNAP));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_SNAP));

    /* Take a copy of the SNAP header */
    memcpy(p_Header, p_Object, sizeof(t_SNAP));

    return p_Header;
}

t_Error HDR_SNAP_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_SNAP *p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_SNAP_OUI:
            SAFE_COPY_VALUE(&p_Header->data.oui, h_Value, sizeof(p_Header->data.oui), size);
        break;
        case e_FIELD_SNAP_PID:
            SAFE_COPY_VALUE(&p_Header->data.pid, h_Value, sizeof(p_Header->data.pid), size);
        break;
        case e_FIELD_SNAP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_SNAP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_SNAP_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_SNAP *p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_SNAP_OUI:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.oui, sizeof(p_Header->data.oui), size);
        break;
        case e_FIELD_SNAP_PID:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.pid, sizeof(p_Header->data.pid), size);
        break;
        case e_FIELD_SNAP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_SNAP_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_SNAP_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_SNAP *p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_SNAP_OUI:
        case e_FIELD_SNAP_PID:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
       FLAG_SET(p_Header->autoFlags, fieldID);
    else
       FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_SNAP_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgSnapHeader* pValues = (t_PgSnapHeader*)hValues;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_SNAP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the SNAP header fields specified by fieldIDs */
    if(fieldIDs & e_FIELD_SNAP_OUI)
        HDR_SNAP_ConfigField(h_Header, e_FIELD_SNAP_OUI, (t_Handle)&pValues->oui, sizeof(pValues->oui));

    if(fieldIDs & e_FIELD_SNAP_PID)
        HDR_SNAP_ConfigField(h_Header, e_FIELD_SNAP_PID, (t_Handle)&pValues->pid, sizeof(pValues->pid));

    if (fieldIDs & e_FIELD_SNAP_NEXT_HEADER_TYPE)
        HDR_SNAP_ConfigField(h_Header, e_FIELD_SNAP_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_SNAP_VALID_FIELDS)
        HDR_SNAP_ConfigField(h_Header, e_FIELD_SNAP_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_SNAP_Randomize(t_Handle h_Header)
{
    t_SNAP *p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
    if(!(p_Header->data.validFields & e_FIELD_SNAP_OUI))
        p_Header->data.oui = (uint32_t)(rand() % 0x1000000);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SNAP_OUI);

    if(!(p_Header->data.validFields & e_FIELD_SNAP_PID))
        p_Header->data.pid = (uint16_t)(rand() % 0x10000);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SNAP_PID);

    return E_OK;
}

t_Error HDR_SNAP_Init(t_Handle h_Header)
{
    t_SNAP* p_Header = NULL;
    t_FrameStruct *p_NextHeader = NULL;
    t_Ethernet* p_EtherHeader = NULL;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Based on the next header, configure the OUI and PID */
    if (p_NextHeader)
    {
      if (FLAG_CHECK(p_Header->autoFlags ,e_FIELD_SNAP_OUI))
      {
            /* There should be a switch here for routed protocols if more than IP is needed */
            if ((p_NextHeader->HeaderID == e_HEADER_TYPE_IPv4) || (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv6))
                p_Header->data.oui = 0x000000;
            /* There should be a switch here for bridged protocols if more than IP is needed */
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_ETH ||
                p_NextHeader->HeaderID == e_HEADER_TYPE_ETH_802_3)
                p_Header->data.oui = 0x0080C2;
      }
      if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_SNAP_PID))
      {
            /* There should be a switch here for routed protocols if more than IP is needed */
            if ((p_NextHeader->HeaderID == e_HEADER_TYPE_IPv4) || (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv6))
                p_Header->data.pid = 0x0800;
            /* There should be a switch here for bridged protocols if more than IP is needed */
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_ETH ||
                p_NextHeader->HeaderID == e_HEADER_TYPE_ETH_802_3 )
            {
                /* The PID value is 0x0001 if the ETH requires checksum */
                p_EtherHeader = p_NextHeader->h_HeaderStruct;
                p_Header->data.pid = (uint8_t)((p_EtherHeader->data.bUseFinalCkSum) ? 0x0001 : 0x0007);
            }
      }
    }


    return E_OK;
}

t_Error HDR_SNAP_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_SNAP      *p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct; /* SNAP header pointer */
    t_FrameStruct  *p_NextHeader = NULL; /* Next header pointer */
    uint16_t    tmp = 0;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if(INTERNAL_SerializeValue(((uint8_t*)&p_Header->data.oui) + 1, 3, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
       INTERNAL_SerializeValue((uint8_t*)&p_Header->data.pid, sizeof(p_Header->data.pid), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    if ( p_NextHeader->HeaderID == e_HEADER_TYPE_ETH ||
         p_NextHeader->HeaderID == e_HEADER_TYPE_ETH_802_3)
    {
        INTERNAL_SerializeValue((uint8_t*)&tmp, sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS);

    }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_SNAP_GetHeaderSize(t_Handle h_Header)
{
    t_SNAP *p_Header = (t_SNAP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_FrameStruct  *p_NextHeader = NULL;
    uint32_t   size = FIXED_HEADER_SIZE_SNAP;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    if ( p_NextHeader->HeaderID == e_HEADER_TYPE_ETH ||
         p_NextHeader->HeaderID == e_HEADER_TYPE_ETH_802_3)
    {
        size+=HEADER_SIZE_ETH_IN_SNAP;

    }

    return size;
}

t_Handle HDR_SNAP_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.snapHeader);
}

t_Error HDR_SNAP_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of SNAP group */


/**************************************************************************//**
@Group          TCP

@Description    TCP functions

@{
*//***************************************************************************/
t_Handle HDR_TCP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_TCP *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_TCP*)MEMALLOC(sizeof(t_TCP));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_TCP));

    FLAG_SET(p_Header->autoFlags, e_FIELD_TCP_OFFSET | e_FIELD_TCP_CHECKSUM);

    /* Initialize the user configured TCP header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgTcpHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_TCP_Clone(t_Handle h_Object)
{
    t_TCP *p_Object   = (t_TCP*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_TCP* p_Header = (t_TCP*)MEMALLOC(sizeof(t_TCP));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_TCP));

    /* Take a copy of the TCP header */
    memcpy(p_Header, p_Object, sizeof(t_TCP));

    return p_Header;
}

t_Error HDR_TCP_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_TCP *p_Header   = (t_TCP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_TCP_PORT_SRC:
            SAFE_COPY_VALUE(&p_Header->data.srcPort, h_Value, sizeof(p_Header->data.srcPort), size);
        break;
        case e_FIELD_TCP_PORT_DST:
            SAFE_COPY_VALUE(&p_Header->data.dstPort, h_Value, sizeof(p_Header->data.dstPort), size);
        break;
        case e_FIELD_TCP_SEQ:
            SAFE_COPY_VALUE(&p_Header->data.seq, h_Value, sizeof(p_Header->data.seq), size);
        break;
        case e_FIELD_TCP_ACK:
            SAFE_COPY_VALUE(&p_Header->data.ack, h_Value, sizeof(p_Header->data.ack), size);
        break;
        case e_FIELD_TCP_OFFSET:
        {
            value8 = (uint8_t)p_Header->data.offset;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_TCP_FLAGS:
        {
            value8 = (uint8_t)p_Header->data.flags;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_TCP_WINDOW:
            SAFE_COPY_VALUE(&p_Header->data.window, h_Value, sizeof(p_Header->data.window), size);
        break;
        case e_FIELD_TCP_CHECKSUM:
            SAFE_COPY_VALUE(&p_Header->data.checksum, h_Value, sizeof(p_Header->data.checksum), size);
        break;
        case e_FIELD_TCP_URGPTR:
            SAFE_COPY_VALUE(&p_Header->data.urgentPtr, h_Value, sizeof(p_Header->data.urgentPtr), size);
        break;
        case e_FIELD_TCP_OPTS:
            SAFE_COPY_VALUE(p_Header->data.p_Opts, h_Value, p_Header->data.optionsCount, size);
        break;
        case e_FIELD_TCP_OPTS_COUNT:
            SAFE_COPY_VALUE(&p_Header->data.optionsCount, h_Value, sizeof(p_Header->data.optionsCount), size);
        break;
        case e_FIELD_TCP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_TCP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_TCP_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_TCP *p_Header   = (t_TCP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_TCP_PORT_SRC:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.srcPort, sizeof(p_Header->data.srcPort), size);
        break;
        case e_FIELD_TCP_PORT_DST:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.dstPort, sizeof(p_Header->data.dstPort), size);
        break;
        case e_FIELD_TCP_SEQ:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.seq, sizeof(p_Header->data.seq), size);
        break;
        case e_FIELD_TCP_ACK:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.ack, sizeof(p_Header->data.ack), size);
        break;
        case e_FIELD_TCP_OFFSET:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.offset = value8;
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_TCP_FLAGS:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.flags = value8;
        }
        break;
        case e_FIELD_TCP_WINDOW:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.window, sizeof(p_Header->data.window), size);
        break;
        case e_FIELD_TCP_CHECKSUM:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.checksum, sizeof(p_Header->data.checksum), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_TCP_URGPTR:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.urgentPtr, sizeof(p_Header->data.urgentPtr), size);
        break;
        case e_FIELD_TCP_OPTS:
        {
            SAFE_COPY_VALUE(h_Value, p_Header->data.p_Opts, MAX_TCP_OPTS_BYTES, size);
            p_Header->data.optionsCount = (uint8_t)size;
            ASSERT_COND(p_Header->data.optionsCount <= MAX_TCP_OPTS_BYTES);
        }break;
        case e_FIELD_TCP_OPTS_COUNT:
        {
             p_Header->data.optionsCount = *((uint8_t*)h_Value);
             ASSERT_COND(p_Header->data.optionsCount <= MAX_TCP_OPTS_BYTES);
        }break;
        case e_FIELD_TCP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_TCP_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_TCP_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_TCP *p_Header   = (t_TCP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_TCP_OFFSET:
        case e_FIELD_TCP_CHECKSUM:
        break;
        default:
        {
           RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        }break;
    }

    if(bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_TCP_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgTcpHeader* pValues = (t_PgTcpHeader*)hValues;
    uint8_t value8 = 0; /* Holds the bit fields of size less than 8 bits */

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_TCP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the TCP header fields specified by fieldIDs */
    if(fieldIDs & e_FIELD_TCP_PORT_SRC)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_PORT_SRC, (t_Handle)&pValues->srcPort, sizeof(pValues->srcPort));

    if(fieldIDs & e_FIELD_TCP_PORT_DST)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_PORT_DST, (t_Handle)&pValues->dstPort, sizeof(pValues->dstPort));

    if(fieldIDs & e_FIELD_TCP_SEQ)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_SEQ, (t_Handle)&pValues->seq, sizeof(pValues->seq));

    if(fieldIDs & e_FIELD_TCP_ACK)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_ACK, (t_Handle)&pValues->ack, sizeof(pValues->ack));

    if(fieldIDs & e_FIELD_TCP_OFFSET)
    {
        value8 = (uint8_t)pValues->offset;
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_OFFSET, (t_Handle)&value8, sizeof(value8));
    }

    if(fieldIDs & e_FIELD_TCP_FLAGS)
    {
        value8 = (uint8_t)pValues->flags;
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_FLAGS, (t_Handle)&value8, sizeof(value8));
    }

    if(fieldIDs & e_FIELD_TCP_WINDOW)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_WINDOW, (t_Handle)&pValues->window, sizeof(pValues->window));

    if(fieldIDs & e_FIELD_TCP_CHECKSUM)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_CHECKSUM, (t_Handle)&pValues->checksum, sizeof(pValues->checksum));

    if(fieldIDs & e_FIELD_TCP_URGPTR)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_URGPTR, (t_Handle)&pValues->urgentPtr, sizeof(pValues->urgentPtr));

/* if(fieldIDs & e_FIELD_TCP_OPTS)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_OPTS, (t_Handle)pValues->p_Opts, pValues->optionsCount);*/

    if (fieldIDs & e_FIELD_TCP_NEXT_HEADER_TYPE)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_TCP_VALID_FIELDS)
        HDR_TCP_ConfigField(h_Header, e_FIELD_TCP_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_TCP_Randomize(t_Handle h_Header)
{
    t_TCP *p_Header = (t_TCP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint32_t index = 0; /* Loop index */

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
    if(!(p_Header->data.validFields & e_FIELD_TCP_PORT_SRC))
        p_Header->data.srcPort = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_PORT_DST))
        p_Header->data.dstPort = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_SEQ))
        p_Header->data.seq = (uint32_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_ACK))
        p_Header->data.ack = (uint32_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_OFFSET))
        p_Header->data.offset = (uint8_t)rand();
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_TCP_OFFSET);

    if(!(p_Header->data.validFields & e_FIELD_TCP_FLAGS))
        p_Header->data.flags = (uint8_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_WINDOW))
        p_Header->data.window = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_CHECKSUM))
        p_Header->data.checksum = (uint16_t)rand();
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_TCP_CHECKSUM);

    if(!(p_Header->data.validFields & e_FIELD_TCP_URGPTR))
        p_Header->data.urgentPtr = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_OPTS))
        for(index=0; index < MAX_TCP_OPTS_BYTES; index++)
            p_Header->data.p_Opts[index] = (uint8_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_TCP_OPTS_COUNT))
        p_Header->data.optionsCount = 0;
    /* use <<(uint8_t)(rand()%MAX_TCP_OPTS_BYTES)>> for real random */

    return E_OK;
}

t_Error HDR_TCP_Init(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return E_OK;
}

t_Error HDR_TCP_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_FrameStruct *p_PrevHeader = NULL, *p_NextHeader = NULL;
    t_PgIpv4Header* pIpHeader = NULL;
    t_TCP *p_Header = NULL;
    uint16_t temp = 0;
    uint8_t aValue8 = 0, bValue8 = 0;
    uint8_t opts = 0;
    uint8_t *pPrevBuffer = NULL;
    uint8_t *pCksumOffset = NULL;
    uint32_t bufferSize = 0, tcpSize = 0, checksum = 0;
    uint8_t pPseudoHeader[FIXED_HEADER_SIZE_PSEUDO_IP];
    uint8_t *offset = NULL;
    uint16_t ckSum = 0;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_TCP*)p_Frame->h_HeaderStruct;
    /* Get the next and prev. header */
    p_PrevHeader = INTERNAL_GetPrevHeader(h_Header);
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Get the frame length (used in the Length field and in the pseudo-header used for checksum) */
    tcpSize = HDR_TCP_GetHeaderSize(h_Header);
    if (p_NextHeader)
        tcpSize += FRAME_GetFrameSize(p_NextHeader, p_NextHeader);

    /* Store the start buffer marker and the remaining size (used below for checksum)*/
    pPrevBuffer = *p_Buffer;
    bufferSize = *pSize;

    /* If options count is not 4 multiple get the nearest higher 4 multiple (ip header must be 32 byte alligned) */
    opts = p_Header->data.optionsCount;
    while (opts && (opts % 4))
        opts++;

    /* Compute the header length if needed */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_TCP_OFFSET))
    {
        /* the header size is defined as the number of 32 bit words (minimum size is 5) */
        p_Header->data.offset = (uint8_t)(HDR_TCP_GetHeaderSize(h_Header) / (uint32_t)sizeof(uint32_t));
    }

    /* If the auto checksum is enabled set it to 0 first */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_TCP_CHECKSUM))
        p_Header->data.checksum = 0;

    /* Store the special values (flags and less than 8 bits) */
    aValue8 = (uint8_t)(p_Header->data.offset);
    aValue8 <<= OFFSET_TCP_RESERVED;
    bValue8 = (uint8_t)(p_Header->data.flags);

    /* Serialize the header members */
    if (INTERNAL_SerializeValue((uint8_t*)&p_Header->data.srcPort, sizeof(p_Header->data.srcPort), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)&p_Header->data.dstPort, sizeof(p_Header->data.dstPort), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)&p_Header->data.seq, sizeof(p_Header->data.seq), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)&p_Header->data.ack, sizeof(p_Header->data.ack), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue(&aValue8, sizeof(aValue8), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue(&bValue8, sizeof(bValue8), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)&p_Header->data.window, sizeof(p_Header->data.window), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        !(pCksumOffset = *p_Buffer) ||
        INTERNAL_SerializeValue((uint8_t*)&p_Header->data.checksum, sizeof(p_Header->data.checksum), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)&p_Header->data.urgentPtr, sizeof(p_Header->data.urgentPtr), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the options */
    if (p_Header->data.optionsCount)
    {
        /* Validate the options field (max limit)*/
        ASSERT_COND(p_Header->data.optionsCount <= MAX_TCP_OPTS_BYTES);
        /* Store the options */
        if (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.p_Opts), p_Header->data.optionsCount, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
            return E_NOT_IN_RANGE;

        /* Pad the header so it is 32 bit aligned */
        aValue8 = 0;
        while(opts-- > p_Header->data.optionsCount)
        {
            if (INTERNAL_SerializeValue(&aValue8, sizeof(aValue8), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
                return E_NOT_IN_RANGE;
        }
    }

    /* Serialize the rest of the frame */
    if (INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize) != E_OK)
    {
        return E_INVALID_STATE;
    }

    /* Compute and serialize the header checksum if needed */
    if (!p_PrevHeader || ((p_PrevHeader->HeaderID != e_HEADER_TYPE_IPv4) && (p_PrevHeader->HeaderID != e_HEADER_TYPE_IPv6)))
    {
        /* This is not a valid context, but maybe the user wanted it this way - do nothing else */
        return E_OK;
    }

    pIpHeader = p_PrevHeader->h_HeaderStruct;
                 offset = pPseudoHeader;
                 temp = (uint16_t)tcpSize;

    /* Build the pseudo-header used for computing the checksum */
    memcpy(offset, &pIpHeader->srcIp, sizeof(uint32_t));
    offset += sizeof(uint32_t);
    memcpy(offset, &pIpHeader->dstIp, sizeof(uint32_t));
    offset += sizeof(uint32_t);
    *offset = 0x00; offset++;
    *offset = 0x06; /* TCP Type - 6 decimal*/
    offset++;
    memcpy(offset, &temp, sizeof(uint16_t));

    /* Compute the partial checksum for the pseudoheader - do not fold the result to 16bit*/
    checksum =  INTERNAL_ComputeCkSum(pPseudoHeader, FIXED_HEADER_SIZE_PSEUDO_IP, 0, FALSE);

    /* Compute the checksum for the UDP header and the rest of the frame using the partial checksum from above */
    ckSum = (uint16_t)INTERNAL_ComputeCkSum(pPrevBuffer, bufferSize, checksum, TRUE);

    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_TCP_CHECKSUM))
    {
        /* Compute the checksum for the UDP header and the rest of the frame using the partial checksum from above */
        p_Header->data.checksum = ckSum;

        /* Reuse tcpSize as "remaining size" parameter in the INTERNAL_SerializeValue function */
        tcpSize = 100;
        /* Serialize the udp checksum */
        if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.checksum), sizeof(p_Header->data.checksum), &pCksumOffset, &tcpSize, CONVERT_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    }
    else
    {
       if(ckSum != p_Header->data.checksum)
           p_Header->errorFlag = e_TCP_CHECKSUM_ERROR;
    }

    return E_OK;
}

uint32_t HDR_TCP_GetHeaderSize(t_Handle h_Header)
{
    t_FrameStruct* p_Header = (t_FrameStruct*)h_Header;
    t_TCP*  pHeaderStr = NULL;
    uint8_t    opts = 0;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    pHeaderStr = (t_TCP*)p_Header->h_HeaderStruct;
    opts = pHeaderStr->data.optionsCount;

    /* if options count is not 4 multiple get the nearest higher 4 multiple (ip header must be 32 byte alligned) */
    while (opts && (opts % 4))
        opts++;

    /* redundat, but adds extra safety (opts count should not be grater than  IP_MAX_OPTIONS_COUNT)*/
    if (opts > MAX_TCP_OPTS_BYTES)
        opts = MAX_TCP_OPTS_BYTES;

    /* each option is one byte */
    return (uint32_t)(FIXED_HEADER_SIZE_TCP + opts);
}

t_Handle HDR_TCP_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l4Header.tcpHeader);
}

t_Error HDR_TCP_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_TCP *p_Header = NULL;        /* TCP header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_TCP*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of TCP group */

/**************************************************************************//**
@Group          UDP

@Description    UDP functions

@{
*//***************************************************************************/
t_Handle HDR_UDP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_UDP *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_UDP*)MEMALLOC(sizeof(t_UDP));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_UDP));

    FLAG_SET(p_Header->autoFlags, e_FIELD_UDP_PORT_SRC | e_FIELD_UDP_PORT_DST | e_FIELD_UDP_LEN | e_FIELD_UDP_CHECKSUM );

    /* Initialize the user configured UDP header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgUdpHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_UDP_Clone(t_Handle h_Object)
{
    t_UDP *p_Object   = (t_UDP*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_UDP* p_Header = (t_UDP*)MEMALLOC(sizeof(t_UDP));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_UDP));

    /* Take a copy of the UDP header */
    memcpy(p_Header, p_Object, sizeof(t_UDP));

    return p_Header;
}

t_Error HDR_UDP_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_UDP *p_Header   = (t_UDP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_UDP_PORT_SRC:
            SAFE_COPY_VALUE(&p_Header->data.srcPort, h_Value, sizeof(p_Header->data.srcPort), size);
        break;
        case e_FIELD_UDP_PORT_DST:
            SAFE_COPY_VALUE(&p_Header->data.dstPort, h_Value, sizeof(p_Header->data.dstPort), size);
        break;
        case e_FIELD_UDP_LEN:
            SAFE_COPY_VALUE(&p_Header->data.len, h_Value, sizeof(p_Header->data.len), size);
        break;
        case e_FIELD_UDP_CHECKSUM:
            SAFE_COPY_VALUE(&p_Header->data.checksum, h_Value, sizeof(p_Header->data.checksum), size);
        break;
        case e_FIELD_UDP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_UDP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_UDP_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_UDP *p_Header   = (t_UDP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_UDP_PORT_SRC:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.srcPort, sizeof(p_Header->data.srcPort), size);
        break;
        case e_FIELD_UDP_PORT_DST:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.dstPort, sizeof(p_Header->data.dstPort), size);
        break;
        case e_FIELD_UDP_LEN:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.len, sizeof(p_Header->data.len), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        break;
        case e_FIELD_UDP_CHECKSUM:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.checksum, sizeof(p_Header->data.checksum), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        break;
        case e_FIELD_UDP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_UDP_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_UDP_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_UDP *p_Header   = (t_UDP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_UDP_LEN:
        case e_FIELD_UDP_CHECKSUM:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        }break;
    }

    if(bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_UDP_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgUdpHeader* pValues = (t_PgUdpHeader*)hValues;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_UDP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the UDP header fields specified by fieldIDs */
    if(fieldIDs & e_FIELD_UDP_PORT_SRC)
        HDR_UDP_ConfigField(h_Header, e_FIELD_UDP_PORT_SRC, (t_Handle)&pValues->srcPort, sizeof(pValues->srcPort));

    if(fieldIDs & e_FIELD_UDP_PORT_DST)
        HDR_UDP_ConfigField(h_Header, e_FIELD_UDP_PORT_DST, (t_Handle)&pValues->dstPort, sizeof(pValues->dstPort));

    if(fieldIDs & e_FIELD_UDP_LEN)
        HDR_UDP_ConfigField(h_Header, e_FIELD_UDP_LEN, (t_Handle)&pValues->len, sizeof(pValues->len));

    if(fieldIDs & e_FIELD_UDP_CHECKSUM)
        HDR_UDP_ConfigField(h_Header, e_FIELD_UDP_CHECKSUM, (t_Handle)&pValues->checksum, sizeof(pValues->checksum));

    if (fieldIDs & e_FIELD_UDP_NEXT_HEADER_TYPE)
        HDR_UDP_ConfigField(h_Header, e_FIELD_UDP_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_UDP_VALID_FIELDS)
        HDR_UDP_ConfigField(h_Header, e_FIELD_UDP_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_UDP_Randomize(t_Handle h_Header)
{
    t_UDP *p_Header   = (t_UDP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
    if(!(p_Header->data.validFields & e_FIELD_UDP_PORT_SRC))
        p_Header->data.srcPort = (uint16_t)(rand() % 0x10);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_UDP_PORT_SRC);


    if(!(p_Header->data.validFields & e_FIELD_UDP_PORT_DST))
        p_Header->data.dstPort = (uint16_t)(rand() % 0x10);
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_UDP_PORT_DST);

    if(!(p_Header->data.validFields & e_FIELD_UDP_LEN))
        p_Header->data.len = (uint16_t)rand();
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_UDP_LEN);

    if(!(p_Header->data.validFields & e_FIELD_UDP_CHECKSUM))
        p_Header->data.checksum = (uint16_t)rand();
    else
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_UDP_CHECKSUM);

    return E_OK;
}

t_Error HDR_UDP_Init(t_Handle h_Header)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_UDP *p_Header = (t_UDP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;  /* UDP header handle */
    t_FrameStruct    *p_NextHeader = NULL;                                   /* Next header pointer */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_NextHeader = INTERNAL_GetNextHeader(h_Header);
     /* Based on the next header, initialize the srcPort and dstPort field */
    if (p_NextHeader)
    {
            if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_PORT_SRC) && FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_PORT_DST))
         {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_L2TPV2 || p_NextHeader->HeaderID == e_HEADER_TYPE_L2TPV3 )
            {
                p_Header->data.srcPort = 1701;
                 p_Header->data.dstPort = 1701;
            }
        }
    }

    if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_PORT_SRC) && FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_PORT_DST))
    {
          if((p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_GENERAL_UDP) || (p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_GENERAL_UDP))
        {
           p_Header->data.srcPort = 0x0140;
           p_Header->data.dstPort = 0x0140;
        }
        if((p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V1_EVENT_UDP) || (p_Frame->HeaderID == e_HEADER_TYPE_IEEE1588_V2_EVENT_UDP))
        {
           p_Header->data.srcPort = 0x013f;
           p_Header->data.dstPort = 0x013f;
        }
    }

    return E_OK;
}

t_Error HDR_UDP_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_FrameStruct *p_PrevHeader = NULL, *p_NextHeader = NULL;
    t_PgIpv4Header* pIpHeader = NULL;
    t_UDP *p_Header = (t_UDP*)h_Header;
    uint32_t udpSize = 0, bufferSize = 0, checksum = 0;
    uint8_t *pTempBuffer = NULL, *pCksumMarker = NULL;
    uint8_t pPseudoHeader[FIXED_HEADER_SIZE_PSEUDO_IP];
    uint8_t *offset = NULL;
    uint16_t ckSum = 0;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_UDP*)p_Frame->h_HeaderStruct;
    /* Get the next and prev. header */
    p_PrevHeader = INTERNAL_GetPrevHeader(h_Header);
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Store the position that marks the buffer start */
    pTempBuffer = *p_Buffer;
    bufferSize = *pSize;

    /* Get the frame length (used in the Length field and in the pseudo-header used for checksum) */
    udpSize = HDR_UDP_GetHeaderSize(h_Header);
    if (p_NextHeader)
        udpSize += FRAME_GetFrameSize(p_NextHeader, p_NextHeader);

    /* If length is set to auto get the frame length */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_LEN))
        p_Header->data.len = (uint16_t)udpSize;

    /* If set to auto, first serialize the checksum with a 0 value */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_CHECKSUM))
        p_Header->data.checksum = 0;

    /* Serialize the udp fields */
    if ((INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.srcPort), sizeof(p_Header->data.srcPort), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
        (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.dstPort), sizeof(p_Header->data.dstPort), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
        (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.len), sizeof(p_Header->data.len), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK) ||
        !(pCksumMarker = *p_Buffer) ||
        (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.checksum), sizeof(p_Header->data.checksum), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK))
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the rest of the frame */
    if(INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Get the size of the buffer used for serialization*/
    bufferSize -= *pSize;

    if (!p_PrevHeader || p_PrevHeader->HeaderID != e_HEADER_TYPE_IPv4)
    {
        /* This is not a valid context, but maybe the user wanted it this way - do nothing else */
        return E_OK;
    }

    pIpHeader = p_PrevHeader->h_HeaderStruct;
    offset = pPseudoHeader;

    /* Build the pseudo-header used for computing the checksum */
    memcpy(offset, &pIpHeader->srcIp, sizeof(uint32_t));
    offset += sizeof(uint32_t);
    memcpy(offset, &pIpHeader->dstIp, sizeof(uint32_t));
    offset += sizeof(uint32_t);
    *offset = 0x00; offset++;
    *offset = 0x11; /* UDP Type - 17 decimal*/
    offset++;
    memcpy(offset, &p_Header->data.len, sizeof(uint16_t));

    /* Compute the partial checksum for the pseudoheader -  do not fold the result to 16bit*/
    checksum =  INTERNAL_ComputeCkSum(pPseudoHeader, FIXED_HEADER_SIZE_PSEUDO_IP, 0, FALSE);

    /* Compute the checksum for the UDP header and the rest of the frame using the partial checksum from above */
    ckSum = (uint16_t)INTERNAL_ComputeCkSum(pTempBuffer, bufferSize, checksum, TRUE);

    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_UDP_CHECKSUM))
    {
        /* Compute the checksum for the UDP header and the rest of the frame using the partial checksum from above */
        p_Header->data.checksum = ckSum;

        /* Reuse udpSize as "remaining size" parameter in the INTERNAL_SerializeValue function */
        udpSize = 100;
        /* Serialize the udp checksum */
        if(INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.checksum), sizeof(p_Header->data.checksum), &pCksumMarker, &udpSize, KEEP_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    }
    else
    {
        if(ckSum != p_Header->data.checksum)
           p_Header->errorFlag = e_UDP_CHECKSUM_ERROR;
    }

    return E_OK;
}

uint32_t HDR_UDP_GetHeaderSize(t_Handle h_Header)
{
    t_UDP *p_Header   = (t_UDP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return 0;
    }

    return FIXED_HEADER_SIZE_UDP;
}

t_Handle HDR_UDP_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l4Header.udpHeader);
}

t_Error HDR_UDP_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_UDP *p_Header = NULL;        /* UDP header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_UDP*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of UDP group */

/**************************************************************************//**
@Group          SCTP

@Description    SCTP functions

@{
*//***************************************************************************/


t_Handle HDR_SCTP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_SCTP *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_SCTP*)MEMALLOC(sizeof(t_SCTP));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_SCTP));

    /*set the auto flags (default: all dynamic fields must be set to auto)*/
    FLAG_SET(p_Header->autoFlags, e_FIELD_SCTP_CKSUM);

    /* Initialize the user configured SCTP header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgSctpCommonHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_SCTP_Clone(t_Handle h_Object)
{
    t_SCTP *p_Object   = (t_SCTP*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_SCTP *p_Header   = (t_SCTP*)MEMALLOC(sizeof(t_SCTP));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_SCTP));

    /* Take a copy of the Ethernet header */
    memcpy(p_Header,p_Object, sizeof(t_SCTP));

    return p_Header;
}

t_Error HDR_SCTP_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_SCTP *p_Header   = (t_SCTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_SCTP_PORT_SRC:
            SAFE_COPY_VALUE(&p_Header->data.srcPort, h_Value, sizeof(p_Header->data.srcPort), size);
        break;
        case e_FIELD_SCTP_PORT_DST:
            SAFE_COPY_VALUE(&p_Header->data.dstPort, h_Value, sizeof(p_Header->data.dstPort), size);
        break;
        case e_FIELD_SCTP_VER_TAG:
            SAFE_COPY_VALUE(&p_Header->data.verTag, h_Value, sizeof(p_Header->data.verTag), size);
        break;
        case e_FIELD_SCTP_CKSUM:
            SAFE_COPY_VALUE(&p_Header->data.checksum, h_Value, sizeof(p_Header->data.checksum), size);
        break;
        case e_FIELD_SCTP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_SCTP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_SCTP_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_SCTP *p_Header   = (t_SCTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    switch (fieldID)
    {

        case e_FIELD_SCTP_PORT_SRC:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.srcPort, sizeof(p_Header->data.srcPort), size);
        break;
        case e_FIELD_SCTP_PORT_DST:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.dstPort, sizeof(p_Header->data.dstPort), size);
        break;
        case e_FIELD_SCTP_VER_TAG:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.verTag, sizeof(p_Header->data.verTag), size);
        break;
        case e_FIELD_SCTP_CKSUM:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.checksum, sizeof(p_Header->data.checksum), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        break;
        case e_FIELD_SCTP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_SCTP_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_SCTP_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_SCTP *p_Header   = (t_SCTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_SCTP_CKSUM:

        break;
        default:
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        break;
    }

    if (bSetAuto)
         FLAG_SET(p_Header->autoFlags, fieldID);
    else
         FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_SCTP_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgSctpCommonHeader  *pValues = (t_PgSctpCommonHeader*)hValues;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_SCTP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    if (fieldIDs & e_FIELD_SCTP_PORT_SRC)
        HDR_SCTP_ConfigField(h_Header, e_FIELD_SCTP_PORT_SRC, (t_Handle)&pValues->srcPort, sizeof(pValues->srcPort));

    if (fieldIDs & e_FIELD_SCTP_PORT_DST)
        HDR_SCTP_ConfigField(h_Header, e_FIELD_SCTP_PORT_DST, (t_Handle)&pValues->dstPort, sizeof(pValues->dstPort));

    if (fieldIDs & e_FIELD_SCTP_VER_TAG)
        HDR_SCTP_ConfigField(h_Header, e_FIELD_SCTP_VER_TAG, (t_Handle)&pValues->verTag, sizeof(pValues->verTag));

    if (fieldIDs & e_FIELD_SCTP_CKSUM)
        HDR_SCTP_ConfigField(h_Header, e_FIELD_SCTP_CKSUM, (t_Handle)&pValues->checksum, sizeof(pValues->checksum));

    if (fieldIDs & e_FIELD_SCTP_NEXT_HEADER_TYPE)
        HDR_SCTP_ConfigField(h_Header, e_FIELD_SCTP_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType,sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_SCTP_VALID_FIELDS)
        HDR_SCTP_ConfigField(h_Header, e_FIELD_SCTP_VALID_FIELDS, (t_Handle)&pValues->validFields,sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_SCTP_Randomize(t_Handle h_Header)
{
    t_SCTP        *p_Header = (t_SCTP*)(((t_FrameStruct*)h_Header)->h_HeaderStruct);

    if (!p_Header)
    {
       RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if(!(p_Header->data.validFields & e_FIELD_SCTP_PORT_SRC))
       p_Header->data.srcPort = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_SCTP_PORT_DST))
       p_Header->data.dstPort = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_SCTP_VER_TAG))
       p_Header->data.verTag = (uint32_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_SCTP_CKSUM))
       p_Header->data.checksum = (uint32_t)rand();
    else
    {
       FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SCTP_CKSUM);
    }

    return E_OK;
}

t_Error HDR_SCTP_Init(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return E_OK;
}

t_Error HDR_SCTP_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct  *p_Frame      = (t_FrameStruct*)h_Header;
    t_SCTP         *p_Header     = NULL;
    uint8_t        *pPrevBuffer  = NULL;
    uint8_t        *pCksumOffset = NULL;
    uint32_t       bufferSize    = 0;
    uint32_t       finalCheckSum = 0;        /* Final CheckSum */

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_SCTP*)p_Frame->h_HeaderStruct;

    /* change the value of the checksum if set to auto */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_SCTP_CKSUM))
    {/* set this to 0, and update it after all the values are serialized */
        p_Header->data.checksum = 0;
    }

    /* store the buffer size and buffer offset for later use (in the checksum) */
    pPrevBuffer = *p_Buffer;
    bufferSize = *pSize;

    /* serialize the values */
    if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.srcPort), sizeof(p_Header->data.srcPort), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.dstPort), sizeof(p_Header->data.dstPort), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.verTag), sizeof(p_Header->data.verTag), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        /* save the checksum offset in the buffer */
        !(pCksumOffset = *p_Buffer) ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.checksum), sizeof(p_Header->data.checksum), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK )
    {
        return E_NOT_IN_RANGE;
    }

    if(INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize)!= E_OK)
    {
       return E_NOT_IN_RANGE;
    }

    /* compute the header checksum if needed */
    /* get the size of the serialized frame */
    bufferSize -= *pSize;

    /* compute the 32-bit crc */
    finalCheckSum = INTERNAL_ComputeCRC(0xFFFFFFFF, pPrevBuffer, bufferSize);

    /* Get the checksum - if set to auto*/
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_SCTP_CKSUM))
    {
       p_Header->data.checksum = finalCheckSum;
    }
    else
    {
       if(finalCheckSum != p_Header->data.checksum)
           p_Header->errorFlag = (uint8_t)e_SCTP_CHECKSUM_ERROR;
    }

    bufferSize = sizeof(p_Header->data.checksum);

    if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.checksum), sizeof(p_Header->data.checksum), &pCksumOffset, &bufferSize, CONVERT_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    return E_OK;
}

uint32_t HDR_SCTP_GetHeaderSize(t_Handle h_Header)
{
    t_SCTP *p_Header   = (t_SCTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return 0;
    }

    return FIXED_HEADER_SIZE_SCTP;
}

t_Handle HDR_SCTP_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }
    return (t_Handle)(&p_Frame->l4Header.sctpHeader.sctpCommonHeader);
}

t_Error HDR_SCTP_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_SCTP *p_Header = NULL;       /* SCTP header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_SCTP*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of SCTP group */

/**************************************************************************//**
@Group          SCTP Chunk Data

@Description    SCTP Chunk data functions

@{
*//***************************************************************************/

t_Handle HDR_SCTP_CHUNK_DATA_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_SCTPChunkData *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_SCTPChunkData*)MEMALLOC(sizeof(t_SCTPChunkData));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_SCTPChunkData));

    FLAG_SET(p_Header->autoFlags, e_FIELD_SCTP_CHUNK_DATA_LENGTH | e_FIELD_SCTP_CHUNK_DATA_UNORDERED | e_FIELD_SCTP_CHUNK_DATA_BEGGINING | e_FIELD_SCTP_CHUNK_DATA_END);

    /* Initialize the user configured UDP header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgSctpChunkDataHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_SCTP_CHUNK_DATA_Clone(t_Handle h_Object)
{
    t_SCTPChunkData     *p_Object = (t_SCTPChunkData*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_SCTPChunkData      *p_Header = (t_SCTPChunkData*)MEMALLOC(sizeof(t_SCTPChunkData));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_SCTPChunkData));

    /* Take a copy of the Ethernet header */
    memcpy(p_Header,p_Object, sizeof(t_SCTPChunkData));

    return p_Header;
}

t_Error HDR_SCTP_CHUNK_DATA_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_SCTPChunkData  *p_Header   = (t_SCTPChunkData*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_SCTP_CHUNK_DATA_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.type, h_Value, sizeof(p_Header->data.type), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_FLAGS:
            SAFE_COPY_VALUE(&p_Header->data.flags, h_Value, sizeof(p_Header->data.flags), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_LENGTH:
            SAFE_COPY_VALUE(&p_Header->data.len, h_Value, sizeof(p_Header->data.len), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_TSN:
            SAFE_COPY_VALUE(&p_Header->data.tsn, h_Value, sizeof(p_Header->data.tsn), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_STREAM_ID:
            SAFE_COPY_VALUE(&p_Header->data.streamId, h_Value, sizeof(p_Header->data.streamId), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN:
            SAFE_COPY_VALUE(&p_Header->data.streamSeq, h_Value, sizeof(p_Header->data.streamSeq), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID:
            SAFE_COPY_VALUE(&p_Header->data.payloadPid, h_Value, sizeof(p_Header->data.payloadPid), size);
        break;
        case e_FIELD_SCTP_CHUNK_DATA_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
       case e_FIELD_SCTP_CHUNK_DATA_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
       break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided !"));
        }
    }

    return E_OK;
}

t_Error HDR_SCTP_CHUNK_DATA_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_SCTPChunkData  *p_Header = (t_SCTPChunkData*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_SCTP_CHUNK_DATA_TYPE:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.type, sizeof(p_Header->data.type), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_FLAGS:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.flags, sizeof(p_Header->data.flags), size);
            FLAG_CLEAR(p_Header->autoFlags, e_FLAG_CHUNK_UNORDERED | e_FLAG_CHUNK_BEGGINING | e_FLAG_CHUNK_END);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_UNORDERED:
            p_Header->autoFlags |= e_FLAG_CHUNK_UNORDERED;
            *(uint8_t *)(h_Value) &= (uint8_t)SCTP_UNORDERED_FLAG;
            SAFE_COPY_VALUE( h_Value, &p_Header->data.flags, sizeof(p_Header->data.flags), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_BEGGINING:
            p_Header->autoFlags |= e_FLAG_CHUNK_BEGGINING;
            *(uint8_t *)(h_Value) &= (uint8_t)SCTP_BEGINNING_FLAG;
            SAFE_COPY_VALUE( h_Value, &p_Header->data.flags, sizeof(p_Header->data.flags), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_END:
            p_Header->autoFlags |= e_FLAG_CHUNK_END;
            *(uint8_t *)(h_Value) &= (uint8_t)SCTP_END_FLAG;
            SAFE_COPY_VALUE( h_Value, &p_Header->data.flags, sizeof(p_Header->data.flags), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_LENGTH:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.len, sizeof(p_Header->data.len), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_TSN:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.tsn, sizeof(p_Header->data.tsn), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_STREAM_ID:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.streamId, sizeof(p_Header->data.streamId), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.streamSeq, sizeof(p_Header->data.streamSeq), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID:
            SAFE_COPY_VALUE( h_Value, &p_Header->data.payloadPid, sizeof(p_Header->data.payloadPid), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
            break;
        case e_FIELD_SCTP_CHUNK_DATA_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
            break;
        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_SCTP_CHUNK_DATA_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_SCTPChunkData  *p_Header = (t_SCTPChunkData*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_SCTP_CHUNK_DATA_BEGGINING:
        case e_FIELD_SCTP_CHUNK_DATA_UNORDERED:
        case e_FIELD_SCTP_CHUNK_DATA_END:
        case e_FIELD_SCTP_CHUNK_DATA_LENGTH:
        break;
        default:
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        break;
    }

    if (bSetAuto)
         FLAG_SET(p_Header->autoFlags, fieldID);
    else
         FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_SCTP_CHUNK_DATA_ConfigHeader(t_Handle h_Header, t_Handle h_Values, uint32_t fieldIDs)
{
    t_SCTPChunkData          *p_Header = (t_SCTPChunkData*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgSctpChunkDataHeader    *p_Values = (t_PgSctpChunkDataHeader*)h_Values;

    if (!p_Header || !p_Values)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_SCTP_CHUNK_DATA_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_TYPE)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_TYPE, (t_Handle)&p_Values->type, sizeof(p_Values->type));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_FLAGS)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_FLAGS, (t_Handle)&p_Values->flags, sizeof(p_Values->flags));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_LENGTH)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_LENGTH, (t_Handle)&p_Values->len, sizeof(p_Values->len));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_TSN)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_TSN, (t_Handle)&p_Values->tsn, sizeof(p_Values->tsn));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_STREAM_ID)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_STREAM_ID, (t_Handle)&p_Values->streamId, sizeof(p_Values->streamId));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN, (t_Handle)&p_Values->streamSeq, sizeof(p_Values->streamSeq));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID, (t_Handle)&p_Values->payloadPid, sizeof(p_Values->payloadPid));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_NEXT_HEADER_TYPE)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_NEXT_HEADER_TYPE, (t_Handle)&p_Values->nextHeaderType,sizeof(p_Values->nextHeaderType));

    if (fieldIDs & e_FIELD_SCTP_CHUNK_DATA_VALID_FIELDS)
        HDR_SCTP_CHUNK_DATA_ConfigField(h_Header, e_FIELD_SCTP_CHUNK_DATA_VALID_FIELDS, (t_Handle)&p_Values->validFields,sizeof(p_Values->validFields));


    return E_OK;
}

t_Error HDR_SCTP_CHUNK_DATA_Randomize(t_Handle h_Header)
{
    t_SCTPChunkData  *p_Header = (t_SCTPChunkData*)(((t_FrameStruct*)h_Header)->h_HeaderStruct);

    if (!p_Header)
    {
       RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
   if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_TYPE))
       p_Header->data.type = (uint8_t)rand();

   if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_FLAGS))
   {
           p_Header->data.flags = (uint8_t)rand();
   }
   else
   {
       FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SCTP_CHUNK_DATA_UNORDERED);
       FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SCTP_CHUNK_DATA_BEGGINING);
       FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SCTP_CHUNK_DATA_END);
   }

   if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_LENGTH))
   {
       p_Header->data.len = (uint16_t)rand();
   }
   else
   {
       FLAG_CLEAR(p_Header->autoFlags, e_FIELD_SCTP_CHUNK_DATA_LENGTH);
   }

    if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_TSN))
       p_Header->data.tsn = (uint32_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_STREAM_ID))
       p_Header->data.streamId = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_STREAM_SQN))
       p_Header->data.streamSeq = (uint16_t)rand();

    if(!(p_Header->data.validFields & e_FIELD_SCTP_CHUNK_DATA_PAYLOAD_PID))
       p_Header->data.payloadPid = (uint32_t)rand();

    return E_OK;
}

t_Error HDR_SCTP_CHUNK_DATA_Init(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return E_OK;
}

t_Error HDR_SCTP_CHUNK_DATA_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct      *p_Frame      = (t_FrameStruct*)h_Header;
    t_SCTPChunkData    *p_Header     = NULL;
    t_FrameStruct      *p_PrevHeader = NULL, *p_NextHeader = NULL, *p_NextSubFrame = NULL, *p_Temp = NULL;
    t_List             *p_TempList   = NULL;

    uint8_t  value8     = 0;
    uint16_t chunkSize  = 0;
    uint32_t bufferSize = 0;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_SCTPChunkData*)p_Frame->h_HeaderStruct;
    p_PrevHeader = INTERNAL_GetPrevHeader(h_Header);
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    bufferSize = *pSize;

    /* find the next PPP MUX subframe and separate it from the frame */
    p_NextSubFrame = FRAME_GetNextHeaderByType(h_Header, h_Header, e_HEADER_TYPE_SCTP_CHUNK_DATA);

    if (p_NextSubFrame)
    {
        p_Temp = FRAME_GetPrevHeaderByType(p_NextSubFrame, p_NextSubFrame, HEADER_TYPE_ANY);
        p_TempList = p_Temp->node.p_Next;
        p_Temp->node.p_Next = NULL;
    }

    if(p_NextHeader->HeaderID == e_HEADER_TYPE_IPHC)
    {
        p_NextHeader = INTERNAL_GetNextHeader(p_NextHeader);
    }

    /* get the size of the subframe */
    chunkSize = (uint16_t)(FRAME_GetFrameSize(p_NextHeader,p_NextHeader) + FIXED_HEADER_SIZE_SCTP_CHUNK_DATA);

    if ( chunkSize > MAX_SCTP_CHUNK_SIZE )                 /* if the subframe exceeds the maximum allowed dimension */
    {
        RETURN_ERROR(MINOR,E_NOT_IN_RANGE,(" Subframe size should be less than 65 535. "));
    }
    else
    {
            /* serialize TYPE field */
            if ( INTERNAL_SerializeValue( (uint8_t *)&p_Header->data.type,  sizeof(uint8_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
            {
                return E_NOT_IN_RANGE;
            }

            /* set CHUNK flags */

            value8 = p_Header->data.flags;

            if ( p_Header->autoFlags & e_FIELD_SCTP_CHUNK_DATA_UNORDERED )
            {
                value8 |= SCTP_UNORDERED_FLAG;
            }
            if ( p_Header->autoFlags & e_FIELD_SCTP_CHUNK_DATA_BEGGINING )
            {
                value8 |= SCTP_BEGINNING_FLAG;
            }
            if ( p_Header->autoFlags & e_FIELD_SCTP_CHUNK_DATA_END )
            {
                value8 |= SCTP_END_FLAG;
            }

             if ( INTERNAL_SerializeValue( (uint8_t*)&value8, sizeof(uint8_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
                 INTERNAL_SerializeValue( (uint8_t *)&chunkSize, sizeof(uint16_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
                 INTERNAL_SerializeValue( (uint8_t *)&p_Header->data.tsn, sizeof(uint32_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
                 INTERNAL_SerializeValue( (uint8_t *)&p_Header->data.streamId, sizeof(uint16_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
                 INTERNAL_SerializeValue( (uint8_t *)&p_Header->data.streamSeq, sizeof(uint16_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
                 INTERNAL_SerializeValue( (uint8_t *)&p_Header->data.payloadPid, sizeof(uint32_t),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK
               )
            {
                return E_NOT_IN_RANGE;
            }

            /* serialize the rest of the chunk */
            INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);


    }
    /* reconstruct the original frame */
    if (p_TempList)
        p_Temp->node.p_Next = p_TempList;

    /* serialize the rest of the frame */
    if (p_NextSubFrame)
    {
        if (pRegisteredHeaderArray[p_NextSubFrame->HeaderID].pfnSerializeHeader(p_NextSubFrame, p_Buffer, pSize) != E_OK)
            return E_NOT_IN_RANGE;
    }

    return E_OK;

}

uint32_t HDR_SCTP_CHUNK_DATA_GetHeaderSize(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return FIXED_HEADER_SIZE_SCTP_CHUNK_DATA;
}

 t_Handle HDR_SCTP_CHUNK_DATA_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l4Header.sctpHeader.sctpChunkHeader);
}

t_Error HDR_SCTP_CHUNK_DATA_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of SCTP Chunk Data group */

/**************************************************************************//**
@Group          IPHC

@Description    IPHC functions

@{
*//***************************************************************************/
t_Handle HDR_IPHC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_IPHC *p_Header = NULL;

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_IPHC*)MEMALLOC(sizeof(t_IPHC));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_IPHC));

    p_Header->autoFlags = 0;

    /* Initialize the user configured IPHC header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgIphcManip));

    if(!p_ProtocolInfo || !p_ProtocolInfo->h_TrafficIphc)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE , ("Failed to create the frame"));
        return NULL;
    }
    p_Header->h_TrafficIphc = p_ProtocolInfo->h_TrafficIphc;

    return (t_Handle) p_Header;
}


t_Handle HDR_IPHC_Clone(t_Handle h_Object)
{
    t_IPHC *p_Object   = (t_IPHC*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_IPHC* p_Header = (t_IPHC*)MEMALLOC(sizeof(t_IPHC));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_IPHC));

    /* Take a copy of the IPHC fields */
    memcpy(p_Header, p_Object, sizeof(t_IPHC));

    return p_Header;
}

t_Error HDR_IPHC_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_IPHC *p_Header   = (t_IPHC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_IPHC_CID:
            SAFE_COPY_VALUE(&p_Header->data.cidNumber, h_Value, sizeof(p_Header->data.cidNumber), size);
        break;
        case e_FIELD_IPHC_CID_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.cidType, h_Value, sizeof(p_Header->data.cidType), size);
        break;
        case e_FIELD_IPHC_HCINDEX:
            SAFE_COPY_VALUE(&p_Header->data.hcIndex, h_Value, sizeof(p_Header->data.hcIndex), size);
        break;
        case e_FIELD_IPHC_GEN:
            SAFE_COPY_VALUE(&p_Header->data.gen, h_Value, sizeof(p_Header->data.gen), size);
        break;
        case e_FIELD_IPHC_D_BIT:
            SAFE_COPY_VALUE(&p_Header->data.dBit, h_Value, sizeof(p_Header->data.dBit), size);
        break;
        case e_FIELD_IPHC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_IPHC_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_IPHC_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_IPHC *p_Header   = (t_IPHC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_IPHC_CID:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.cidNumber, sizeof(p_Header->data.cidNumber), size);
        break;
        case e_FIELD_IPHC_CID_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.cidType, sizeof(p_Header->data.cidType), size);
        break;
        case e_FIELD_IPHC_HCINDEX:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.hcIndex, sizeof(p_Header->data.hcIndex), size);
        break;
        case e_FIELD_IPHC_GEN:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.gen, sizeof(p_Header->data.gen), size);            //FLAG_CLEAR(p_Header->autoFlags, fieldID);
        break;
        case e_FIELD_IPHC_D_BIT:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.dBit, sizeof(p_Header->data.dBit), size);          //FLAG_CLEAR(p_Header->autoFlags, fieldID);
        break;
        case e_FIELD_IPHC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_IPHC_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_IPHC_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_IPHC *p_Header   = (t_IPHC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
       // case FIELD_UDP_LEN:
       // case FIELD_UDP_CKSUM:
       // break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        }break;
    }

    if(bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_IPHC_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgIphcManip* pValues = (t_PgIphcManip*)hValues;

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_IPHC_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the IPHC fields specified by fieldIDs */
    if(fieldIDs & e_FIELD_IPHC_CID)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_CID, (t_Handle)&pValues->cidNumber, sizeof(pValues->cidNumber));

    if(fieldIDs & e_FIELD_IPHC_CID_TYPE)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_CID_TYPE, (t_Handle)&pValues->cidType, sizeof(pValues->cidType));

    if(fieldIDs & e_FIELD_IPHC_HCINDEX)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_HCINDEX, (t_Handle)&pValues->hcIndex, sizeof(pValues->hcIndex));

    if(fieldIDs & e_FIELD_IPHC_GEN)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_GEN, (t_Handle)&pValues->gen, sizeof(pValues->gen));

    if(fieldIDs & e_FIELD_IPHC_D_BIT)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_D_BIT, (t_Handle)&pValues->dBit, sizeof(pValues->dBit));

    if (fieldIDs & e_FIELD_IPHC_NEXT_HEADER_TYPE)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_IPHC_VALID_FIELDS)
        HDR_IPHC_ConfigField(h_Header, e_FIELD_IPHC_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_IPHC_Randomize(t_Handle h_Header)
{
    t_IPHC *p_Header   = (t_IPHC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Generate random values for the fields not configured by the user */
    if(!(p_Header->data.validFields & e_FIELD_IPHC_CID))
        p_Header->data.cidNumber = (uint16_t)(rand() % 0x10);

    if(!(p_Header->data.validFields & e_FIELD_IPHC_CID_TYPE))
        p_Header->data.cidType = (uint16_t)(rand() % 0x10);

    if(!(p_Header->data.validFields & e_FIELD_IPHC_HCINDEX))
        p_Header->data.hcIndex = (uint8_t)(rand() % 0x10);

    if(!(p_Header->data.validFields & e_FIELD_IPHC_GEN))
        p_Header->data.gen = (uint8_t)(rand() % 0x10);

    /* D_BIT is not randomized since it has to be 0x00 by default */
    return E_OK;
}

t_Error HDR_IPHC_Init(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return E_OK;
}

t_Error    HDR_IPHC_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct  *p_Frame = (t_FrameStruct*)h_Header;
    t_IPHC         *p_Header = (t_IPHC*)p_Frame->h_HeaderStruct;
    uint8_t        headerSize = 0;
    uint8_t        remainingSize = 0;
    bool           isFH = TRUE;
    t_Error        errorVal =  E_INVALID_VALUE;
    t_FrameStruct  *p_PppoeFrame = NULL;
    uint8_t        *p_TempBuffer = NULL;
    bool           isUDP = TRUE;

    t_Handle    *p_IpHeader  = NULL;
    uint32_t    ipOffsetDiff = 0;
    uint32_t remainingFrameSize = 0;

    if (!(p_Frame && h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_IPHC*)p_Frame->h_HeaderStruct;

    /* Calculate the ip start difference from the current p_Buffer position */
    p_IpHeader = FRAME_GetPrevHeaderByType(p_Frame, p_Frame, e_HEADER_TYPE_IPv4);
    ipOffsetDiff = INTERNAL_GetFrameSize(p_IpHeader,p_Frame);

    if((*p_Buffer)[IPV4_PROTOCOL_OFFSET] != IP_HEADER_UDP_PROTOCOL)
    {
        isUDP = FALSE;
    }

    errorVal = IPHC_GoldenModel(p_Header->data.cidNumber,
                                p_Header->data.hcIndex,
                                *p_Buffer,
                                p_Header->data.gen,
                                p_Header->data.cidType,p_Header->data.dBit,&remainingSize,&headerSize,&isFH,
                                p_Header->h_TrafficIphc);

    if(errorVal != E_OK)
    {
        return errorVal;
    }

     p_PppoeFrame =FRAME_GetPrevHeaderByType(h_Header,h_Header, e_HEADER_TYPE_PPP);
     if (!isFH)
     {
           if(p_PppoeFrame)
           {
               p_TempBuffer = *p_Buffer;
               p_TempBuffer[-2] = 0x00;
               p_TempBuffer[-1] = 0x65;

           }
           *pSize += remainingSize;
           if(!isUDP)
           {
               /* Serialize Non UDP till IPHC node */
               memcpy(*p_Buffer+headerSize,*p_Buffer+FIXED_HEADER_SIZE_IP,ipOffsetDiff - FIXED_HEADER_SIZE_IP);
               *p_Buffer += ipOffsetDiff - FIXED_HEADER_SIZE_IP;
           }
           *p_Buffer += headerSize;

           /* Copy the remaining frame to the empty fields left by the compressed header */
           remainingFrameSize = FRAME_GetFrameSize(p_Frame,p_Frame);
           memcpy(*p_Buffer,*p_Buffer+remainingSize,remainingFrameSize);
     }
     else
     {
           if(p_PppoeFrame)
           {
               p_TempBuffer = *p_Buffer;
               p_TempBuffer[-2] = 0x00;
               p_TempBuffer[-1] = 0x61;

           }
           *p_Buffer += ipOffsetDiff;
     }
     p_Header->hcHdrLength = remainingSize;

    /* IPHC is Serialized after all headers are serialized. Hence
       INTERNAL_SerializeNextHeader() funtion need not be invoked */
    return E_OK;
}

uint32_t HDR_IPHC_GetHeaderSize(t_Handle h_Header)
{
    t_IPHC *p_Header   = (t_IPHC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return 0;
    }

    return p_Header->hcHdrLength;
}

t_Handle HDR_IPHC_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->iphcManip);
}

t_Error HDR_IPHC_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of IPHC group */

/**************************************************************************//**
@Group          PPPoE

@Description    PPPoE functions

@{
*//***************************************************************************/
t_Handle HDR_PPPoE_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PPPoE *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_PPPoE*)MEMALLOC(sizeof(t_PPPoE));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_PPPoE));

    /*set the auto flags (default: all dynamic fields must be set to auto)*/
    FLAG_SET(p_Header->autoFlags, e_FIELD_PPPoE_LEN);

    /* Initialize the user configured PPPoE header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgPppoeHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_PPPoE_Clone(t_Handle h_Object)
{
    t_PPPoE *p_Object   = (t_PPPoE*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_PPPoE* p_Header = (t_PPPoE*)MEMALLOC(sizeof(t_PPPoE));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_PPPoE));

    /* Take a copy of the PPPoE header */
    memcpy(p_Header, p_Object, sizeof(t_PPPoE));

    return p_Header;
}

t_Error HDR_PPPoE_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PPPoE *p_Header   = (t_PPPoE*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0; /* Holds the bit fields of size less than 8 bits */

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_PPPoE_VER:
        {
            value8 = (uint8_t)p_Header->data.ver;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_PPPoE_TYPE:
        {
            value8 = (uint8_t)p_Header->data.type;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_PPPoE_CODE:
            SAFE_COPY_VALUE(&p_Header->data.code, h_Value, sizeof(p_Header->data.code), size);
        break;
        case e_FIELD_PPPoE_SID:
            SAFE_COPY_VALUE(&p_Header->data.sid, h_Value, sizeof(p_Header->data.sid), size);
        break;
        case e_FIELD_PPPoE_LEN:
            SAFE_COPY_VALUE(&p_Header->data.len, h_Value, sizeof(p_Header->data.len), size);
        break;
        case e_FIELD_PPPoE_SESSION:
            SAFE_COPY_VALUE(&p_Header->data.sessionStarted, h_Value, sizeof(p_Header->data.sessionStarted), size);
        break;
        case e_FIELD_PPPoE_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_PPPoE_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PPPoE_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PPPoE *p_Header = (t_PPPoE*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0; /* Holds the bit fields of size less than 8 bits */

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Confugure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_PPPoE_VER:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.ver = value8;
        }
        break;
        case e_FIELD_PPPoE_LEN:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.len, sizeof(p_Header->data.len), size);
            FLAG_CLEAR(p_Header->autoFlags, e_FIELD_PPPoE_LEN);
        }
        break;
        case e_FIELD_PPPoE_TYPE:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.type = value8;
        }
        break;
        case e_FIELD_PPPoE_CODE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.code, sizeof(p_Header->data.code), size);
        break;
        case e_FIELD_PPPoE_SID:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.sid, sizeof(p_Header->data.sid), size);
        break;
        case e_FIELD_PPPoE_SESSION:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.sessionStarted, sizeof(p_Header->data.sessionStarted), size);
        break;
        case e_FIELD_PPPoE_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_PPPoE_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PPPoE_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_PPPoE *p_Header   = (t_PPPoE*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_PPPoE_LEN:

        break;

        default:
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        break;
    }

    if (bSetAuto)
         FLAG_SET(p_Header->autoFlags, fieldID);
    else
         FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_PPPoE_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PPPoE *p_Header = (t_PPPoE*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgPppoeHeader* pValues = (t_PgPppoeHeader*)hValues;
    uint8_t value8 = 0; /* Holds the bit fields of size less than 8 bits */

    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_PPPoE_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the PPPoE header fields specified by fieldIDs */
    if (fieldIDs & e_FIELD_PPPoE_VER)
    {
        value8 = (uint8_t)pValues->ver;
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_VER, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PPPoE_TYPE)
    {
        value8 = (uint8_t)pValues->type;
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_TYPE, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PPPoE_CODE)
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_CODE, (t_Handle)&pValues->code, sizeof(pValues->code));

    if (fieldIDs & e_FIELD_PPPoE_SID)
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_SID, (t_Handle)&pValues->sid, sizeof(pValues->sid));

    if (fieldIDs & e_FIELD_PPPoE_LEN)
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_LEN, (t_Handle)&pValues->len, sizeof(pValues->len));

    if (fieldIDs & e_FIELD_PPPoE_SESSION)
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_SESSION, (t_Handle)&pValues->sessionStarted, sizeof(pValues->sessionStarted));

    if (fieldIDs & e_FIELD_PPPoE_NEXT_HEADER_TYPE)
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_PPPoE_VALID_FIELDS)
        HDR_PPPoE_ConfigField(h_Header, e_FIELD_PPPoE_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_PPPoE_Randomize(t_Handle h_Header)
{
    t_PPPoE *pHeaderStr = (t_PPPoE *)(((t_FrameStruct*)h_Header)->h_HeaderStruct);

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_PPPoE_VER))
        pHeaderStr->data.ver = 0x1;

    if (!(pHeaderStr->data.validFields & e_FIELD_PPPoE_TYPE))
        pHeaderStr->data.type = 0x1;

    if (!(pHeaderStr->data.validFields & e_FIELD_PPPoE_CODE))
        pHeaderStr->data.code = 0x00;

    if (!(pHeaderStr->data.validFields & e_FIELD_PPPoE_SID))
        pHeaderStr->data.sid = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_PPPoE_LEN))
        pHeaderStr->data.len = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_PPPoE_LEN);

    if (!(pHeaderStr->data.validFields & e_FIELD_PPPoE_SESSION))
        pHeaderStr->data.sessionStarted = TRUE;

    return E_OK;
}

t_Error HDR_PPPoE_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_PPPoE* p_Header = NULL;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_PPPoE*)p_Frame->h_HeaderStruct;

    /* Compute the total length if needed */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_PPPoE_LEN))
    {
        p_Header->data.len = (uint16_t)FRAME_GetFrameSize(h_Header, h_Header);
        p_Header->data.len -= FIXED_HEADER_SIZE_PPPoE;
   //  ASSERT_COND(p_Header->data.len <= IP_MAX_FRAME_SIZE);
    }

    /* Serialize the values */
    if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data),  1,  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.code), sizeof(p_Header->data.code), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.sid),  sizeof(p_Header->data.sid),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.len),  sizeof(p_Header->data.len),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_PPPoE_GetHeaderSize(t_Handle h_Header)
{
    UNUSED(h_Header);
    return (uint32_t)(FIXED_HEADER_SIZE_PPPoE);
}

t_Error HDR_PPPoE_Init(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return E_OK;
}

t_Handle HDR_PPPoE_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.pppoeHeader);
}

t_Error HDR_PPPoE_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of PPPoE group */

/**************************************************************************//**
@Group          IPv4

@Description    IPv4 functions

@{
*//***************************************************************************/
t_Handle HDR_IPv4_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_IPv4 *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_IPv4*)MEMALLOC(sizeof(t_IPv4));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_IPv4));

    /* Set the auto flags (default: all dynamic fields must be set to auto)*/
    FLAG_SET(p_Header->autoFlags, e_FIELD_IPv4_HDR_LEN | e_FIELD_IPv4_TOTAL_LEN | e_FIELD_IPv4_PROTO | e_FIELD_IPv4_CKSUM);

    /* Initialize the user configured IPv4 header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgIpv4Header));

    return (t_Handle) p_Header;
}

t_Handle HDR_IPv4_Clone(t_Handle h_Object)
{
    t_IPv4 *p_Object   = (t_IPv4*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_IPv4* p_Header = (t_IPv4*)MEMALLOC(sizeof(t_IPv4));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_IPv4));

    /* Take a copy of the IPv4 header */
    memcpy(p_Header, p_Object, sizeof(t_IPv4));

    return p_Header;
}

t_Error     HDR_IPv4_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_IPv4 *p_Header   = (t_IPv4*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    uint8_t flag = 0;          /* Holds the IPv4 flags */
    uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;      /* Holds the bit fields of size between 8 and 16 bits */

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_IPv4_VER:
        {
            value8 = (uint8_t)p_Header->data.version;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_IPv4_HDR_LEN:
        {
            value8 = (uint8_t)p_Header->data.headerLength;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_IPv4_TOS:
            SAFE_COPY_VALUE(&p_Header->data.tos, h_Value, sizeof(p_Header->data.tos), size);
        break;
        case e_FIELD_IPv4_TOTAL_LEN:
            SAFE_COPY_VALUE(&p_Header->data.totalLength, h_Value, sizeof(p_Header->data.totalLength), size);
        break;
        case e_FIELD_IPv4_ID:
            SAFE_COPY_VALUE(&p_Header->data.id, h_Value, sizeof(p_Header->data.id), size);
        break;
        case e_FIELD_IPv4_FLAG_D:
        {
            flag = (uint8_t)((p_Header->data.flags & ~MASK_IPV4_FLAG_D) >> OFFSET_IPV4_FLAG_D);
            SAFE_COPY_VALUE(&flag, h_Value, sizeof(flag), size);
        }
        break;
        case e_FIELD_IPv4_FLAG_M:
        {
            flag = (uint8_t)((p_Header->data.flags & ~MASK_IPV4_FLAG_M) >> OFFSET_IPV4_FLAG_M);
            SAFE_COPY_VALUE(&flag, h_Value, sizeof(flag), size);
        }
        break;
        case e_FIELD_IPv4_OFFSET:
        {
            value16 = (uint16_t)p_Header->data.fragmentOffset;
            SAFE_COPY_VALUE(&value16, h_Value, sizeof(value16), size);
        }
        break;
        case e_FIELD_IPv4_TTL:
            SAFE_COPY_VALUE(&p_Header->data.ttl, h_Value, sizeof(p_Header->data.ttl), size);
        break;
        case e_FIELD_IPv4_PROTO:
            SAFE_COPY_VALUE(&p_Header->data.protocol, h_Value, sizeof(p_Header->data.protocol), size);
        break;
        case e_FIELD_IPv4_CKSUM:
            SAFE_COPY_VALUE(&p_Header->data.headerCksum, h_Value, sizeof(p_Header->data.headerCksum), size);
        break;
        case e_FIELD_IPv4_SRC_IP:
            SAFE_COPY_VALUE(&p_Header->data.srcIp, h_Value, sizeof(p_Header->data.srcIp), size);
        break;
        case e_FIELD_IPv4_DST_IP:
            SAFE_COPY_VALUE(&p_Header->data.dstIp, h_Value, sizeof(p_Header->data.dstIp), size);
        break;
        case e_FIELD_IPv4_OPTS:
            SAFE_COPY_VALUE(p_Header->data.options, h_Value, sizeof(p_Header->data.options), size);
        break;
        case e_FIELD_IPv4_OPTS_COUNT:
            SAFE_COPY_VALUE(&p_Header->data.optionsCount, h_Value, sizeof(p_Header->data.optionsCount), size);
        break;
        case e_FIELD_IPv4_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_IPv4_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_IPv4_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_IPv4 *p_Header = (t_IPv4*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;      /* Holds the bit fields of size between 8 and 16 bits */

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_IPv4_VER:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.version = value8;
        }
        break;
        case e_FIELD_IPv4_HDR_LEN:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.headerLength = value8;
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_IPv4_TOS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.tos, sizeof(p_Header->data.tos), size);
        break;
        case e_FIELD_IPv4_TOTAL_LEN:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.totalLength, sizeof(p_Header->data.totalLength), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_IPv4_ID:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.id, sizeof(p_Header->data.id), size);
        break;
        case e_FIELD_IPv4_FLAG_D:
            p_Header->data.flags = (uint8_t)( (*((uint8_t*)h_Value) << OFFSET_IPV4_FLAG_D) | (p_Header->data.flags & MASK_IPV4_FLAG_D) );
        break;
        case e_FIELD_IPv4_FLAG_M:
            p_Header->data.flags = (uint8_t)( (*((uint8_t*)h_Value) << OFFSET_IPV4_FLAG_M) | (p_Header->data.flags & MASK_IPV4_FLAG_M) );
        break;
        case e_FIELD_IPv4_OFFSET:
        {
            SAFE_COPY_VALUE(h_Value, &value16, sizeof(value16), size);
            p_Header->data.fragmentOffset = value16;
        }
        break;
        case e_FIELD_IPv4_TTL:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.ttl, sizeof(p_Header->data.ttl), size);
        break;
        case e_FIELD_IPv4_PROTO:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.protocol, sizeof(p_Header->data.protocol), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_IPv4_CKSUM:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.headerCksum, sizeof(p_Header->data.headerCksum), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_IPv4_SRC_IP:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.srcIp, sizeof(p_Header->data.srcIp), size);
        break;
        case e_FIELD_IPv4_DST_IP:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.dstIp, sizeof(p_Header->data.dstIp), size);
        break;
        case e_FIELD_IPv4_OPTS:
            SAFE_COPY_VALUE(h_Value, p_Header->data.options, IP_MAX_OPTIONS_COUNT, size);
        break;
        case e_FIELD_IPv4_OPTS_COUNT:
        {
            if (*((uint8_t*)h_Value) > IP_MAX_OPTIONS_COUNT)
                *((uint8_t*)h_Value) = IP_MAX_OPTIONS_COUNT; /*not more than 15 option bytes*/
            SAFE_COPY_VALUE(h_Value, &p_Header->data.optionsCount, sizeof(p_Header->data.optionsCount), size);
        }
        break;
        case e_FIELD_IPv4_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_IPv4_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_IPv4_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_IPv4 *p_Header = (t_IPv4*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_IPv4_HDR_LEN:
        case e_FIELD_IPv4_TOTAL_LEN:
        case e_FIELD_IPv4_CKSUM:
        case e_FIELD_IPv4_PROTO:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_IPv4_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_IPv4 *p_Header   = (t_IPv4*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgIpv4Header* pValues = (t_PgIpv4Header*)hValues;
    uint8_t value8 = 0;    /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;  /* Holds the bit fields of size between 8 and 16 bits */

    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_IPv4_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the IPv4 header fields specified by fieldIDs */
    if (fieldIDs & e_FIELD_IPv4_VER)
    {
        value8 = (uint8_t)pValues->version;
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_VER, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_IPv4_HDR_LEN)
    {
        value8 = (uint8_t)pValues->headerLength;
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_HDR_LEN, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_IPv4_TOS)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_TOS, (t_Handle)&pValues->tos, sizeof(pValues->tos));

    if (fieldIDs & e_FIELD_IPv4_TOTAL_LEN)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_TOTAL_LEN, (t_Handle)&pValues->totalLength, sizeof(pValues->totalLength));

    if (fieldIDs & e_FIELD_IPv4_ID)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_ID, (t_Handle)&pValues->id, sizeof(pValues->id));

    if (fieldIDs & e_FIELD_IPv4_FLAG_D)
    {
        value8 = (uint8_t)pValues->flags;
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_FLAG_D, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_IPv4_FLAG_M)
    {
        value8 = (uint8_t)pValues->flags;
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_FLAG_M, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_IPv4_OFFSET)
    {
        value16 = (uint16_t)pValues->fragmentOffset;
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_OFFSET, (t_Handle)&value16, sizeof(value16));
    }

    if (fieldIDs & e_FIELD_IPv4_TTL)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_TTL, (t_Handle)&pValues->ttl, sizeof(pValues->ttl));

    if (fieldIDs & e_FIELD_IPv4_PROTO)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_PROTO, (t_Handle)&pValues->protocol, sizeof(pValues->protocol));

    if (fieldIDs & e_FIELD_IPv4_CKSUM)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_CKSUM, (t_Handle)&pValues->headerCksum, sizeof(pValues->headerCksum));

    if (fieldIDs & e_FIELD_IPv4_SRC_IP)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_SRC_IP, (t_Handle)&pValues->srcIp, sizeof(pValues->srcIp));

    if (fieldIDs & e_FIELD_IPv4_DST_IP)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_DST_IP, (t_Handle)&pValues->dstIp, sizeof(pValues->dstIp));

    if (fieldIDs & e_FIELD_IPv4_OPTS)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_OPTS, (t_Handle)&pValues->options, sizeof(pValues->options));

    if (fieldIDs & e_FIELD_IPv4_OPTS_COUNT)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_OPTS_COUNT, (t_Handle)&pValues->optionsCount, sizeof(pValues->optionsCount));

    if (fieldIDs & e_FIELD_IPv4_NEXT_HEADER_TYPE)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_IPv4_VALID_FIELDS)
        HDR_IPv4_ConfigField(h_Header, e_FIELD_IPv4_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error HDR_IPv4_Randomize(t_Handle h_Header)
{
    t_IPv4 *pHeaderStr = ((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t index = 0;

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_VER))
        pHeaderStr->data.version = 4;

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_HDR_LEN))
        pHeaderStr->data.headerLength = (uint8_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IPv4_HDR_LEN);

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_TOS))
        pHeaderStr->data.tos = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_TOTAL_LEN))
        pHeaderStr->data.totalLength = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IPv4_TOTAL_LEN);

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_ID))
        pHeaderStr->data.id = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_FLAG_D) &&
    !(pHeaderStr->data.validFields & e_FIELD_IPv4_FLAG_M))
        pHeaderStr->data.flags = 0;

    /* The user should set this to something different - otherwise it does not make sence */
    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_OFFSET))
        pHeaderStr->data.fragmentOffset = 0;

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_TTL))
    {
        pHeaderStr->data.ttl = (uint8_t)rand();
        if (pHeaderStr->data.ttl < 2) pHeaderStr->data.ttl=2;
    }

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_PROTO))
        pHeaderStr->data.protocol = (uint8_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IPv4_PROTO);

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_CKSUM))
        pHeaderStr->data.headerCksum = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IPv4_CKSUM);

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_SRC_IP))
        for (index=0; index<IPv4_ADDR_SIZE; index++)
            ((uint8_t*)&pHeaderStr->data.srcIp)[index] = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_DST_IP))
        for (index=0; index<IPv4_ADDR_SIZE; index++)
            ((uint8_t*)&pHeaderStr->data.dstIp)[index] = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_OPTS))
        for (index=0; index<IP_MAX_OPTIONS_COUNT; index++)
            pHeaderStr->data.options[index] = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv4_OPTS_COUNT))
        pHeaderStr->data.optionsCount = 0;
    /* use <<(uint8_t)(rand()%IP_MAX_OPTIONS_COUNT)>> for real random */

    return E_OK;
}

t_Error HDR_IPv4_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_IPv4* p_Header = NULL;
    uint16_t value16 = 0;
    uint8_t value8 = 0;
    uint8_t opts = 0;
    uint8_t *pPrevBuffer = NULL;
    uint8_t *pCksumOffset = NULL;
    uint32_t bufferSize = 0;
    uint16_t ckSum = 0;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

     p_Header = (t_IPv4*)p_Frame->h_HeaderStruct;

    opts = p_Header->data.optionsCount;

    /* If options count is not 4 multiple get the nearest higher 4 multiple (ip header must be 32 byte alligned) */
    while (opts && (opts % 4))
        opts++;

    /* Compute the header length if needed */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv4_HDR_LEN))
    {
        /* The header size is defined as the number of 32 bit words (minimum size is 5) */
        p_Header->data.headerLength = (uint8_t)(HDR_IPv4_GetHeaderSize(h_Header) / (uint16_t)sizeof(uint32_t));
    }

    /* Compute the total length if needed */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv4_TOTAL_LEN))
    {
        p_Header->data.totalLength = (uint16_t)FRAME_GetFrameSize(h_Header, h_Header);
        ASSERT_COND(p_Header->data.totalLength <= MAX_FRAME_SIZE_IP);
    }

    /* Change the value of the checksum if set to auto */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv4_CKSUM))
    {/* set this to 0, and update it after all the values are serialized */
        p_Header->data.headerCksum = 0;
    }

    /* Store the buffer size and buffer offset for later use (in the checksum) */
    pPrevBuffer = *p_Buffer;
    bufferSize = *pSize;

    /* Store the Flags and Fragment Offset in the 16 bit value */
    value16 = (uint16_t)(p_Header->data.flags << 13);
    value16 |= p_Header->data.fragmentOffset;

    /* Serialize the values */
    if (
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data), sizeof(uint8_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.tos), sizeof(p_Header->data.tos), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.totalLength), sizeof(p_Header->data.totalLength), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.id), sizeof(p_Header->data.id), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&value16), sizeof(value16), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.ttl), sizeof(p_Header->data.ttl), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.protocol), sizeof(p_Header->data.protocol), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        !(pCksumOffset = *p_Buffer) ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.headerCksum), sizeof(p_Header->data.headerCksum), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.srcIp), sizeof(p_Header->data.srcIp), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.dstIp), sizeof(p_Header->data.dstIp), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK )
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the options */
    if (p_Header->data.optionsCount)
    {
        /* Store the options */
        if (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.options), p_Header->data.optionsCount, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
            return E_NOT_IN_RANGE;

        /* Pad the header so it is 32 bit aligned */
        value8 = 0;
        while(opts-- > p_Header->data.optionsCount)
        {
            if (INTERNAL_SerializeValue(&value8, sizeof(value8), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
                return E_NOT_IN_RANGE;
        }
    }

    /* Compute and serialize the header checksum if needed */
    /* Compute the checksum */
    bufferSize -= *pSize;
    ckSum = (uint16_t)INTERNAL_ComputeCkSum(pPrevBuffer, bufferSize, 0, TRUE);

    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv4_CKSUM))
    {
        p_Header->data.headerCksum = ckSum;
        /* Reutilize the bufferSize variable to re-serialize the checksum*/
        bufferSize = 10;
        /* Serialize the checksum */
        if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.headerCksum), sizeof(p_Header->data.headerCksum), &pCksumOffset, &bufferSize, CONVERT_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    }
    else
    {
        if(ckSum != p_Header->data.headerCksum)
            p_Header->errorFlag = e_IPv4_CHECKSUM_ERROR;
    }

    if(p_Header->data.version != IPv4_VERSION)
        p_Header->errorFlag |= e_IPv4_VERSION_ERROR;

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_IPv4_GetHeaderSize(t_Handle h_Header)
{
    t_IPv4* pHeaderStr = (t_IPv4*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
   uint8_t opts = 0;

    opts = pHeaderStr->data.optionsCount;

    /* If options count is not 4 multiple get the nearest higher 4 multiple (ip header must be 32 byte alligned) */
    while (opts && (opts % 4))
        opts++;

    /* Redundant, but adds extra safety (opts count should not be grater than  IP_MAX_OPTIONS_COUNT)*/
    if (opts > IP_MAX_OPTIONS_COUNT)
        opts = IP_MAX_OPTIONS_COUNT;

    /* Each option is one byte */
    return (uint32_t)(FIXED_HEADER_SIZE_IP + opts);
}

t_Error HDR_IPv4_Init(t_Handle h_Header)
{
    t_IPv4* p_Header = NULL;
    t_FrameStruct *p_NextHeader = NULL;

    if (!h_Header)
    {
            RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        }

    p_Header = (t_IPv4*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Check the following protocol type */
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    if (p_NextHeader)
    {
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv4_PROTO))
        {
            switch (p_NextHeader->HeaderID)
            {
                case e_HEADER_TYPE_TCP:
                p_Header->data.protocol = 6;
                    break;
                case e_HEADER_TYPE_UDP:
                case e_HEADER_TYPE_IEEE1588_V1_GENERAL_UDP:
                case e_HEADER_TYPE_IEEE1588_V2_GENERAL_UDP:
                case e_HEADER_TYPE_IEEE1588_V1_EVENT_UDP:
                case e_HEADER_TYPE_IEEE1588_V2_EVENT_UDP:
                p_Header->data.protocol = 17;
                    break;
                case e_HEADER_TYPE_TUNNELED_IPv4:
                p_Header->data.protocol = 4;
                    break;
                case e_HEADER_TYPE_TUNNELED_IPv6:
                p_Header->data.protocol = 41;
                    break;
                case e_HEADER_TYPE_SCTP:
                p_Header->data.protocol = 132;
                    break;
                default:
                    return E_INVALID_SELECTION;
            }
        }
    }

    /* If the next header is null or of an unknown type, leave the type field as it was */
    return E_OK;
}

t_Handle HDR_IPv4_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l3Header.ipv4Header);
}

t_Error HDR_IPv4_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_IPv4 *p_Header = NULL;       /* IPv4 header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_IPv4*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of IPv4 group */

/**************************************************************************//**
@Group          PPP

@Description    PPP functions

@{
*//***************************************************************************/
t_Handle HDR_PPP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PPP *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_PPP*)MEMALLOC(sizeof(t_PPP));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_PPP));

    FLAG_SET(p_Header->autoFlags, e_FIELD_PPP_TYPE);

    /* Initialize the user configured PPP header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgPppHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_PPP_Clone(t_Handle h_Object)
{
    t_PPP *p_Object = (t_PPP*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_PPP* p_Header = (t_PPP*)MEMALLOC(sizeof(t_PPP));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_PPP));

    /* Take a copy of PPP header */
    memcpy(p_Header, p_Object, sizeof(t_PPP));

    return p_Header;
}

t_Error HDR_PPP_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PPP *p_Header = (t_PPP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_PPP_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.type, h_Value, sizeof(p_Header->data.type), size);
        break;
        case e_FIELD_PPP_COMPRESSED:
            SAFE_COPY_VALUE(&p_Header->data.compressedHeader, h_Value, sizeof(p_Header->data.compressedHeader), size);
        break;
        case e_FIELD_PPP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_PPP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PPP_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PPP *p_Header = (t_PPP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_PPP_TYPE:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.type, sizeof(p_Header->data.type), size);
            FLAG_CLEAR( p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_PPP_COMPRESSED:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.compressedHeader, sizeof(p_Header->data.compressedHeader), size);
        break;
        case e_FIELD_PPP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_PPP_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PPP_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_PPP *p_Header = (t_PPP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_PPP_TYPE:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;

}

t_Error HDR_PPP_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PPP *p_Header = (t_PPP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgPppHeader* pValues = (t_PgPppHeader*)hValues;

    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_PPP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the PPP header fields specified by fieldIDs */
    if (fieldIDs & e_FIELD_PPP_TYPE)
        HDR_PPP_ConfigField(h_Header, e_FIELD_PPP_TYPE, (t_Handle)&pValues->type, sizeof(pValues->type));

    if (fieldIDs & e_FIELD_PPP_NEXT_HEADER_TYPE)
        HDR_PPP_ConfigField(h_Header, e_FIELD_PPP_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_PPP_VALID_FIELDS)
        HDR_PPP_ConfigField(h_Header, e_FIELD_PPP_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_PPP_Randomize(t_Handle h_Header)
{
    t_PPP *pHeaderStr = (t_PPP *)(((t_FrameStruct*)h_Header)->h_HeaderStruct);

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_TYPE))
        pHeaderStr->data.type = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_PPP_TYPE);

    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_COMPRESSED))
        pHeaderStr->data.compressedHeader = FALSE; /*(bool)(rand()%2)*/

    return E_OK;
}

t_Error HDR_PPP_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_PPP* p_Header = NULL;
    uint8_t    headerByte;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_PPP*)p_Frame->h_HeaderStruct;

    /* Check if header is compressed */
    if (p_Header->data.compressedHeader)
    {
        headerByte = (uint8_t)p_Header->data.type;

        /* Serialize the values */
        if (INTERNAL_SerializeValue( (uint8_t*)&headerByte,  sizeof(headerByte),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    }
    else
    {
        /* Serialize the values */
        if (INTERNAL_SerializeValue( (uint8_t*)&p_Header->data.type,  sizeof(p_Header->data.type),  p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_PPP_GetHeaderSize(t_Handle h_Header)
{
    t_PPP *pHeaderStr = (t_PPP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    return (uint32_t)((pHeaderStr->data.compressedHeader) ? COMPRESSED_PPP_HEADER_SIZE : PPP_HEADER_SIZE );
}

t_Error HDR_PPP_Init(t_Handle h_Header)
{

    t_PPP *p_Header = NULL;
    t_FrameStruct *p_NextHeader = NULL;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_PPP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    /* Based on the next header, configure the type */
    if (p_NextHeader)
    {
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_PPP_TYPE))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv4)
                p_Header->data.type = 0x0021;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv6)
                p_Header->data.type = 0x0057;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_PPP_MLMC)
                p_Header->data.type = 0x003D;
            /*
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_VLAN)
                p_Header->data.type = 0x8100;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_PPPoE)
            {
                HDR_PPPoE_GetField( p_NextHeader, e_FIELD_PPPoE_SESSION,
                                    &bSessionStarted, sizeof(bool));
                p_Header->data.type=(uint16_t)(bSessionStarted ?
                                                 0x8864 : 0x8863); // 0x8863 - discovery stage
                                                                   // 0x8864 - session stage
            }
            */
        }
    }

    /* If the next header is null or of an unknown type, leave the type field as it was */
    return E_OK;
}

t_Handle HDR_PPP_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.pppHeader);
}

t_Error HDR_PPP_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of PPP group */

/**************************************************************************//**
@Group          IPv6

@Description    IPv6 functions

@{
*//***************************************************************************/
t_Handle HDR_IPv6_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_IPv6 *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    /* Allocate memory for IPv6 header */
    p_Header = (t_IPv6*)MEMALLOC(sizeof(t_IPv6));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_IPv6));

    /* Set the auto flags */
    FLAG_SET(p_Header->autoFlags, e_FIELD_IPv6_NEXT_HEADER | e_FIELD_IPv6_PAYLOAD_LENGTH);

    /* Initialize the user configured IPv6 header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgIpv6Header));

    return (t_Handle) p_Header;
}

t_Handle HDR_IPv6_Clone(t_Handle h_Object)
{
    /* Get the handle to the IPv6 header */
    t_IPv6* p_Object = (t_IPv6*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_IPv6* p_Header = (t_IPv6*)MEMALLOC(sizeof(t_IPv6));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_IPv6));

    /* Take a copy of the IPv6 header */
    memcpy(p_Header, p_Object, sizeof(t_IPv6));

    return p_Header;
}

t_Error  HDR_IPv6_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the IPv6 header */
    t_IPv6 *p_Header   = (t_IPv6*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
     uint32_t value32 = 0;      /* Holds the bit fields of size between 16 and 32 bits */

    /* Check the handle */
    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Retrieve the field value specified by the user */
    switch (fieldID)
    {
        case e_FIELD_IPv6_VER:
        {
            value8 = (uint8_t)p_Header->data.ipVersion;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_IPv6_TRAFFIC_CLASS:
        {
            value8 = (uint8_t)p_Header->data.trafficClass;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_IPv6_FLOW_LABEL:
        {
            value32 = (uint32_t)p_Header->data.flowLabel;
            SAFE_COPY_VALUE(&value32, h_Value, sizeof(value32), size);
        }
        break;
        case e_FIELD_IPv6_PAYLOAD_LENGTH:
            SAFE_COPY_VALUE(&p_Header->data.payloadLength, h_Value, sizeof(p_Header->data.payloadLength), size);
        break;
        case e_FIELD_IPv6_NEXT_HEADER:
            SAFE_COPY_VALUE(&p_Header->data.nextHeader, h_Value, sizeof(p_Header->data.nextHeader), size);
        break;
        case e_FIELD_IPv6_HOP_LIMIT:
            SAFE_COPY_VALUE(&p_Header->data.hopLimit, h_Value, sizeof(p_Header->data.hopLimit), size);
        break;
        case e_FIELD_IPv6_SOURCE_IP:
            SAFE_COPY_VALUE(p_Header->data.srcIp, h_Value, sizeof(p_Header->data.srcIp), size);
        break;
        case e_FIELD_IPv6_DEST_IP:
            SAFE_COPY_VALUE(p_Header->data.dstIp, h_Value, sizeof(p_Header->data.dstIp), size);
        break;
        case e_FIELD_IPv6_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_IPv6_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_IPv6_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the IPv6 header */
    t_IPv6* p_Header = (t_IPv6*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
     uint32_t value32 = 0;      /* Holds the bit fields of size between 16 and 32 bits */

    /* Check the handle */
    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_IPv6_VER:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.ipVersion = value8;
        }
        break;
        case e_FIELD_IPv6_TRAFFIC_CLASS:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.trafficClass = value8;
        }
        break;
        case e_FIELD_IPv6_FLOW_LABEL:
        {
            SAFE_COPY_VALUE(h_Value, &value32, sizeof(value32), size);
            p_Header->data.flowLabel = value32;
        }
        break;
        case e_FIELD_IPv6_PAYLOAD_LENGTH:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.payloadLength, sizeof(p_Header->data.payloadLength), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_IPv6_NEXT_HEADER:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeader, sizeof(p_Header->data.nextHeader), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_IPv6_HOP_LIMIT:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.hopLimit, sizeof(p_Header->data.hopLimit), size);
        break;
        case e_FIELD_IPv6_SOURCE_IP:
            SAFE_COPY_VALUE(h_Value, p_Header->data.srcIp, sizeof(p_Header->data.srcIp), size);
        break;
        case e_FIELD_IPv6_DEST_IP:
            SAFE_COPY_VALUE(h_Value, p_Header->data.dstIp, sizeof(p_Header->data.dstIp), size);
        break;
        case e_FIELD_IPv6_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_IPv6_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_IPv6_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    /* Get the handle to the IPv6 header */
    t_IPv6* p_Header = (t_IPv6*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Check the handle */
    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_IPv6_NEXT_HEADER:
        case e_FIELD_IPv6_PAYLOAD_LENGTH:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    /* Set or clear the auto flag based on the user input */
    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_IPv6_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    /* Get the handle to the IPv6 header */
    t_IPv6* p_Header = (t_IPv6*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgIpv6Header* pValues = (t_PgIpv6Header*)hValues;
     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
     uint32_t value32 = 0;      /* Holds the bit fields of size between 16 and 32 bits */

    /* Check the handle */
    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_IPv6_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the IPv6 header based on the field ID */
    if (fieldIDs & e_FIELD_IPv6_VER)
    {
        value8 = (uint8_t)pValues->ipVersion;
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_VER, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_IPv6_TRAFFIC_CLASS)
    {
        value8 = (uint8_t)pValues->trafficClass;
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_TRAFFIC_CLASS, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_IPv6_FLOW_LABEL)
    {
        value32 = (uint32_t)pValues->flowLabel;
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_FLOW_LABEL, (t_Handle)&value32, sizeof(value32));
    }

    if (fieldIDs & e_FIELD_IPv6_PAYLOAD_LENGTH)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_PAYLOAD_LENGTH, (t_Handle)&pValues->payloadLength, sizeof(pValues->payloadLength));

    if (fieldIDs & e_FIELD_IPv6_NEXT_HEADER)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_NEXT_HEADER, (t_Handle)&pValues->nextHeader, sizeof(pValues->nextHeader));

    if (fieldIDs & e_FIELD_IPv6_HOP_LIMIT)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_HOP_LIMIT, (t_Handle)&pValues->hopLimit, sizeof(pValues->hopLimit));

    if (fieldIDs & e_FIELD_IPv6_SOURCE_IP)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_SOURCE_IP, (t_Handle)&pValues->srcIp, sizeof(pValues->srcIp));

    if (fieldIDs & e_FIELD_IPv6_DEST_IP)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_DEST_IP, (t_Handle)&pValues->dstIp, sizeof(pValues->dstIp));

    if (fieldIDs & e_FIELD_IPv6_NEXT_HEADER_TYPE)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_IPv6_VALID_FIELDS)
        HDR_IPv6_ConfigField(h_Header, e_FIELD_IPv6_NEXT_HEADER_TYPE, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_IPv6_Randomize(t_Handle h_Header)
{
    /* Get the handle to the IPv6 header */
    t_IPv6* pHeaderStr = (t_IPv6*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t index = 0; /* Loop index */

    /* Generate random values for the fields which are not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_VER))
        pHeaderStr->data.ipVersion = 6;

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_TRAFFIC_CLASS))
        pHeaderStr->data.trafficClass = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_FLOW_LABEL))
        pHeaderStr->data.flowLabel = (uint32_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_PAYLOAD_LENGTH))
        pHeaderStr->data.payloadLength = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IPv6_PAYLOAD_LENGTH);

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_NEXT_HEADER))
        pHeaderStr->data.nextHeader = (uint8_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_IPv6_NEXT_HEADER);

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_HOP_LIMIT))
        pHeaderStr->data.hopLimit = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_SOURCE_IP))
    {
        for (index=0; index<IPv6_ADDR_SIZE; index++)
            pHeaderStr->data.srcIp[index] = (uint8_t)rand();
    }

    if (!(pHeaderStr->data.validFields & e_FIELD_IPv6_DEST_IP))
    {
        for (index=0; index<IPv6_ADDR_SIZE; index++)
            pHeaderStr->data.dstIp[index] = (uint8_t)rand();
    }

    return E_OK;
}

t_Error HDR_IPv6_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    /* Get the handle to the IPv6 header */
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_FrameStruct *p_NextHeader = NULL;
    t_IPv6* p_Header = NULL;

    /* Check the handle */
    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_IPv6*)p_Frame->h_HeaderStruct;

     /* Get the size of the frame (if needed) */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv6_PAYLOAD_LENGTH))
    {
        p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(h_Header);
        if (p_NextHeader)
            p_Header->data.payloadLength = (uint16_t)FRAME_GetFrameSize(p_NextHeader, p_NextHeader);
        else
            p_Header->data.payloadLength = 0;
    }

    /* Serialize the values */
    if (INTERNAL_SerializeValue((uint8_t*)&p_Header->data, sizeof(uint32_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.payloadLength), sizeof(p_Header->data.payloadLength), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.nextHeader), sizeof(p_Header->data.nextHeader), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.hopLimit), sizeof(p_Header->data.hopLimit), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.srcIp), sizeof(p_Header->data.srcIp), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.dstIp), sizeof(p_Header->data.dstIp), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK )
    {
        return E_NOT_IN_RANGE;
    }

    if(p_Header->data.ipVersion != IPv6_VERSION)
         p_Header->errorFlag = e_IPv6_VERSION_ERROR;

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_IPv6_GetHeaderSize(t_Handle h_Header)
{
    UNUSED(h_Header);

    /* Return the IPv6 header size */
    return (FIXED_HEADER_SIZE_IPv6);
}

t_Error HDR_IPv6_Init(t_Handle h_Header)
{
    t_IPv6* p_Header = NULL;
    t_FrameStruct *p_NextHeader = NULL;

    /* Check the handle */
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the handle to the IPv6 header */
    p_Header = (t_IPv6*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Get the next header */
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    if (p_NextHeader)
    {
        /* Initialize the ID of the next header if the autoflag is true */
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_IPv6_NEXT_HEADER))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_TCP)
                p_Header->data.nextHeader = 6;

            else if (p_NextHeader->HeaderID == e_HEADER_TYPE_UDP ||
                     p_NextHeader->HeaderID == e_HEADER_TYPE_IEEE1588_V1_GENERAL_UDP ||
                     p_NextHeader->HeaderID == e_HEADER_TYPE_IEEE1588_V2_GENERAL_UDP ||
                     p_NextHeader->HeaderID == e_HEADER_TYPE_IEEE1588_V1_EVENT_UDP ||
                     p_NextHeader->HeaderID == e_HEADER_TYPE_IEEE1588_V2_EVENT_UDP )
                p_Header->data.nextHeader = 17;

            else if (p_NextHeader->HeaderID == e_HEADER_TYPE_TUNNELED_IPv4)
                p_Header->data.nextHeader = 4;

            else if (p_NextHeader->HeaderID == e_HEADER_TYPE_TUNNELED_IPv6)
                p_Header->data.nextHeader = 41;
        }
    }

    return E_OK;
}

t_Handle HDR_IPv6_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l3Header.ipv6Header);
}

t_Error HDR_IPv6_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_IPv6 *p_Header = NULL;       /* IPv6 header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_IPv6*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of IPv6 group */


/**************************************************************************//**
@Group         MPLS

@Description    MPLS functions

@{
*//***************************************************************************/

t_Handle HDR_MPLS_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PgMplsHeader *p_MPLSHeader = (t_PgMplsHeader*)hHeader;
    t_MPLS  *p_Header          = NULL;
    UNUSED(p_ProtocolInfo);

  /* Check the header handle */
     if (!hHeader)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
         return NULL;
     }

    p_Header = (t_MPLS*)MEMALLOC(sizeof(t_MPLS));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_MPLS));
    FLAG_SET(p_Header->autoFlags, e_FIELD_MPLS_STACKVALUE);

 /* Initialize the user configured MPLS header fields in the header */
    memcpy(&p_Header->data, &p_MPLSHeader[mplsIndex++], sizeof(t_PgMplsHeader));

    if((p_Header->data.nextHeaderType != e_HEADER_TYPE_MPLS) && (p_Header->data.nextHeaderType != e_HEADER_TYPE_PWE3_ATM_MPLS_DEMUX) && (p_Header->data.nextHeaderType != e_HEADER_TYPE_PWE3_SAToP_MPLS_DEMUX))
    {
        mplsIndex = 0;
    }
    return (t_Handle) p_Header;
}


t_Handle HDR_MPLS_Clone(t_Handle h_Object)
{
    /* Get the handle to the MPLS header */
    t_MPLS  *p_Object = (t_MPLS*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_MPLS  *p_Header = (t_MPLS*)MEMALLOC(sizeof(t_MPLS));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the MPLS Header"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_MPLS));
    /* Take a copy of the MPLS header */
    memcpy(p_Header, p_Object, sizeof(t_MPLS));

    return p_Header;
}


t_Error HDR_MPLS_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the value of field specified by fieldID */
    t_MPLS  *p_Header  = (t_MPLS*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint32_t value32   =0;
    uint8_t  value8    =0;

    if (!(p_Header && h_Value && size))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }
    switch (fieldID)
     {
         case e_FIELD_MPLS_LABEL:
             value32 = (uint32_t)p_Header->data.label;
             SAFE_COPY_VALUE(&value32, h_Value, sizeof(value32), size);
         break;
         case e_FIELD_MPLS_CLASSOFSERVICE:
             value8 = (uint8_t)p_Header->data.cos;
             SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
         break;
         case e_FIELD_MPLS_STACKVALUE:
             value8 = (uint8_t)p_Header->data.stackValue;
             SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
         break;
         case e_FIELD_MPLS_TIMETOLIVE:
             value8 = (uint8_t)p_Header->data.ttl;
             SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
         break;
         case e_FIELD_MPLS_NEXT_HEADER_TYPE:
             SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
         break;
         case e_FIELD_MPLS_VALID_FIELDS:
             SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
         break;

         default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
         }
     }
    return E_OK;
}


t_Error HDR_MPLS_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_MPLS  *p_Header = (t_MPLS*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint32_t value32 = 0;
    uint8_t  value8  = 0;

     if (!(p_Header && h_Value))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

    size = (size) ? size : (uint32_t)(-1);

    switch (fieldID)
    {
        case e_FIELD_MPLS_LABEL:
            SAFE_COPY_VALUE(h_Value, &value32, sizeof(value32), size);
            p_Header->data.label = value32;
            break;

        case e_FIELD_MPLS_CLASSOFSERVICE:
           SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
           p_Header->data.cos = value8;
           break;

        case e_FIELD_MPLS_STACKVALUE:
           SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
           p_Header->data.stackValue = value8;
           break;

        case e_FIELD_MPLS_TIMETOLIVE:
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.ttl = value8;
            break;

        case e_FIELD_MPLS_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;

        case e_FIELD_MPLS_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}


t_Error HDR_MPLS_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_MPLS  *p_Header = (t_MPLS*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
         case e_FIELD_MPLS_STACKVALUE:
         break;
         default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_STATE, ("The field is not dynamic"));
         }break;
    }

    if (bSetAuto)
    {
        FLAG_SET(p_Header->autoFlags, fieldID);
    }
    else
    {
        FLAG_CLEAR(p_Header->autoFlags, fieldID);
    }

    return E_OK;
}


t_Error HDR_MPLS_ConfigHeader(t_Handle h_Header, t_Handle h_Values, uint32_t t_FieldFlags)
{
    t_MPLS          *p_Header = (t_MPLS*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgMplsHeader  *p_Values = (t_PgMplsHeader*)h_Values;
    uint32_t    value32     = 0;
    uint8_t value8          = 0;

    if (!p_Header || !p_Values)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, NO_MSG);
    }

    if (!(t_FieldFlags & e_FIELD_MPLS_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    if (t_FieldFlags & e_FIELD_MPLS_LABEL)
    {
        value32 = (uint32_t)p_Header->data.label;
        HDR_MPLS_ConfigField(h_Header, e_FIELD_MPLS_LABEL, (t_Handle)&value32, sizeof(value32));
    }

    if (t_FieldFlags & e_FIELD_MPLS_CLASSOFSERVICE)
    {
        value8 = (uint8_t)p_Header->data.cos;
        HDR_MPLS_ConfigField(h_Header, e_FIELD_MPLS_CLASSOFSERVICE, (t_Handle)&value8, sizeof(value8));
    }

    if (t_FieldFlags & e_FIELD_MPLS_STACKVALUE)
    {
        value8 = (uint8_t)p_Header->data.stackValue;
        HDR_MPLS_ConfigField(h_Header, e_FIELD_MPLS_STACKVALUE, (t_Handle)&value8, sizeof(value8));
    }

    if (t_FieldFlags & e_FIELD_MPLS_TIMETOLIVE)
    {
        value8 = (uint8_t)p_Header->data.ttl;
        HDR_MPLS_ConfigField(h_Header, e_FIELD_MPLS_TIMETOLIVE, (t_Handle)&value8, sizeof(value8));
    }

    if (t_FieldFlags & e_FIELD_MPLS_NEXT_HEADER_TYPE)
    {
        HDR_MPLS_ConfigField(h_Header, e_FIELD_MPLS_NEXT_HEADER_TYPE, (t_Handle)&p_Values->nextHeaderType, sizeof(p_Values->nextHeaderType));
    }

    if (t_FieldFlags & e_FIELD_MPLS_VALID_FIELDS)
    {
        HDR_MPLS_ConfigField(h_Header, e_FIELD_MPLS_VALID_FIELDS, (t_Handle)&p_Values->validFields, sizeof(p_Values->validFields));
    }

    return E_OK;
}


t_Error HDR_MPLS_Randomize(t_Handle h_Header)
{
    t_MPLS  *p_HeaderStr;
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }
/*  Get the handle to the MPLS header */
    p_HeaderStr = (t_MPLS *)(((t_FrameStruct*)h_Header)->h_HeaderStruct);

/*  Generate random values for the fields which are not configured by the user */
    if (!(p_HeaderStr->data.validFields & e_FIELD_MPLS_LABEL))
        p_HeaderStr->data.label = (uint32_t)rand();

    if (!(p_HeaderStr->data.validFields & e_FIELD_MPLS_CLASSOFSERVICE))
        p_HeaderStr->data.cos = (uint8_t)rand();

    if (!(p_HeaderStr->data.validFields & e_FIELD_MPLS_STACKVALUE))
    {
        p_HeaderStr->data.stackValue = (uint8_t)rand();
    }
    else
    {
        FLAG_CLEAR(p_HeaderStr->autoFlags, e_FIELD_MPLS_STACKVALUE);
    }

    if (!(p_HeaderStr->data.validFields & e_FIELD_MPLS_TIMETOLIVE))
        p_HeaderStr->data.ttl = (uint8_t)rand();

 return E_OK;
}

t_Error HDR_MPLS_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *p_Size)
{
     t_FrameStruct *p_Frame           = (t_FrameStruct*)h_Header;
     t_MPLS *p_Header = NULL;;

     if (!p_Frame)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     if (*p_Size < FIXED_HEADER_SIZE_MPLS)
     {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("MPLS header size should be >= %d", FIXED_HEADER_SIZE_MPLS));
     }

     p_Header = (t_MPLS*)p_Frame->h_HeaderStruct;

    /* Serialize the MPLS fields */
    if ((INTERNAL_SerializeValue((uint8_t*)(&p_Header->data), sizeof(uint32_t), p_Buffer, p_Size, KEEP_ENDIANNESS) != E_OK) )
    {
        return E_NOT_IN_RANGE;
    }

     /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, p_Size);
}


uint32_t HDR_MPLS_GetHeaderSize(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE,(""));
    }

    return FIXED_HEADER_SIZE_MPLS;
}


t_Error HDR_MPLS_Init(t_Handle h_Header)
{
    t_MPLS *p_Header            = NULL;         /* MPLS header pointer */
    t_FrameStruct *p_NextHeader = NULL;         /* Next header pointer */

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

     p_Header = (t_MPLS*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
     p_NextHeader = INTERNAL_GetNextHeader(h_Header);


    /* Based on the next header, configure the type */

    if (p_NextHeader)
    {
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_MPLS_STACKVALUE))
        {
            if ((p_NextHeader->HeaderID == e_HEADER_TYPE_MPLS) || (p_NextHeader->HeaderID == e_HEADER_TYPE_PWE3_SAToP_MPLS_DEMUX) || (p_NextHeader->HeaderID == e_HEADER_TYPE_PWE3_ATM_MPLS_DEMUX))
                p_Header->data.stackValue = 0x0;
            else
                p_Header->data.stackValue = 0x01;
         }
    }
    return E_OK;
}

t_Handle HDR_MPLS_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }
    return (t_Handle)(&p_Frame->l2Header.mplsHeader);
}

t_Error HDR_MPLS_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);
    return E_OK;
}

/** @} */ /* end of MPLS group */

/**************************************************************************//**
@Group          L2TPv2

@Description    L2TPv2 functions

@{
*//***************************************************************************/
t_Handle HDR_L2TPv2_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_L2TPv2 *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_L2TPv2*)MEMALLOC(sizeof(t_L2TPv2));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_L2TPv2));


    /* Set the auto flags (default: all dynamic fields must be set to auto)*/
    FLAG_SET(p_Header->autoFlags, e_FIELD_L2TPv2_LBIT | e_FIELD_L2TPv2_SBIT | e_FIELD_L2TPv2_OBIT | e_FIELD_L2TPv2_LEN );

    /* Initialize the user configured L2TPv2 header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgL2tpv2Header));

    return (t_Handle) p_Header;
}

t_Handle HDR_L2TPv2_Clone(t_Handle h_Object)
{
    t_L2TPv2 *p_Object   = (t_L2TPv2*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_L2TPv2 * p_Header  = (t_L2TPv2*)MEMALLOC(sizeof(t_L2TPv2));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_L2TPv2));

    /* Take a copy of the L2TPv2 header */
    memcpy(p_Header, p_Object, sizeof(t_L2TPv2));

    return p_Header;
}

t_Error     HDR_L2TPv2_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_L2TPv2 *p_Header   = (t_L2TPv2*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

     /* Get the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_L2TPv2_TBIT:
        {
             value8 = (uint8_t)p_Header->data.type;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_L2TPv2_LBIT:
        {
             value8 = (uint8_t)p_Header->data.lengthPresent;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_L2TPv2_SBIT:
        {

            value8 = (uint8_t)p_Header->data.sequencePresent;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_L2TPv2_OBIT:
        {
            value8 = (uint8_t)p_Header->data.offsetPresent;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_L2TPv2_PBIT:
        {
            value8 = (uint8_t)p_Header->data.priority;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_L2TPv2_VER:
        {
            value8 = (uint8_t)p_Header->data.version;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_L2TPv2_LEN:
        {
            SAFE_COPY_VALUE(&p_Header->data.length, h_Value, sizeof(p_Header->data.length), size);
        }
        break;
        case e_FIELD_L2TPv2_TID:
        {
            SAFE_COPY_VALUE(&p_Header->data.tunnelId, h_Value, sizeof(p_Header->data.tunnelId), size);
        }
        break;
        case e_FIELD_L2TPv2_SID:
        {
            SAFE_COPY_VALUE(&p_Header->data.sessionId, h_Value, sizeof(p_Header->data.sessionId), size);
        }
        break;
        case e_FIELD_L2TPv2_NS:
        {
            SAFE_COPY_VALUE(&p_Header->data.sequenceNo, h_Value, sizeof(p_Header->data.sequenceNo), size);
        }
        break;
        case e_FIELD_L2TPv2_NR:
        {
            SAFE_COPY_VALUE(&p_Header->data.expectedSequenceNo, h_Value, sizeof(p_Header->data.expectedSequenceNo), size);
        }
        break;
        case e_FIELD_L2TPv2_OFFSIZE:
        {
            SAFE_COPY_VALUE(&p_Header->data.offsetSize, h_Value, sizeof(p_Header->data.offsetSize), size);
        }
        break;
        case e_FIELD_L2TPv2_NEXT_HEADER_TYPE:
         {
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
         }
        break;
        case e_FIELD_L2TPv2_VALID_FIELDS:
        {
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
         }
        break;
        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_L2TPv2_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_L2TPv2 *p_Header = (t_L2TPv2*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

     /* Configure the value of the field specified by fieldID */
    switch (fieldID)
    {
        case e_FIELD_L2TPv2_TBIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.type = value8;
        }
        break;
        case e_FIELD_L2TPv2_LBIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.lengthPresent = value8;
        }
        break;
        case e_FIELD_L2TPv2_SBIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.sequencePresent = value8;
        }
        break;
        case e_FIELD_L2TPv2_OBIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.offsetPresent = value8;
        }
        break;
        case e_FIELD_L2TPv2_PBIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.priority = value8;
        }
        break;
        case e_FIELD_L2TPv2_VER:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.version = value8;
        }
        break;
        case e_FIELD_L2TPv2_LEN:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.length, sizeof(p_Header->data.length), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_L2TPv2_TID:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.tunnelId, sizeof(p_Header->data.tunnelId), size);
        }
        break;
        case e_FIELD_L2TPv2_SID:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.sessionId, sizeof(p_Header->data.sessionId), size);
        }
        break;
        case e_FIELD_L2TPv2_NS:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.sequenceNo, sizeof(p_Header->data.sequenceNo), size);
        }
        break;
        case e_FIELD_L2TPv2_NR:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.expectedSequenceNo, sizeof(p_Header->data.expectedSequenceNo), size);
        }
        break;
        case e_FIELD_L2TPv2_OFFSIZE:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.offsetSize, sizeof(p_Header->data.offsetSize), size);
        }
        break;
        case e_FIELD_L2TPv2_NEXT_HEADER_TYPE:
         {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
         }
        break;
        case e_FIELD_L2TPv2_VALID_FIELDS:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        }
        break;
        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }
    return E_OK;
}

t_Error HDR_L2TPv2_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_L2TPv2 *p_Header = (t_L2TPv2*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
         case e_FIELD_L2TPv2_LBIT:
           case e_FIELD_L2TPv2_SBIT:
           case e_FIELD_L2TPv2_OBIT:
        case e_FIELD_L2TPv2_LEN:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_L2TPv2_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    t_PgL2tpv2Header* pValues = (t_PgL2tpv2Header*)hValues;
     uint8_t value8 = 0;    /* Holds the bit fields of size less than 8 bits */

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_L2TPv2_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }
     /* Configure the L2TPv2 header fields specified by fieldIDs */

    if (fieldIDs & e_FIELD_L2TPv2_TBIT)
    {
         value8 = (uint8_t)pValues->type;
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_TBIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_L2TPv2_LBIT)
    {
         value8 = (uint8_t)pValues->lengthPresent;
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_LBIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_L2TPv2_SBIT)
    {
        value8 = (uint8_t)pValues->sequencePresent;
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_SBIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_L2TPv2_OBIT)
    {
         value8 = (uint8_t)pValues->offsetPresent;
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_OBIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_L2TPv2_PBIT)
    {
         value8 = (uint8_t)pValues->priority;
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_PBIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_L2TPv2_VER)
    {
         value8 = (uint8_t)pValues->version;
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_VER, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_L2TPv2_LEN)
    {
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_LEN, (t_Handle)&pValues->length, sizeof(pValues->length));
    }

    if (fieldIDs & e_FIELD_L2TPv2_TID)
    {
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_TID, (t_Handle)&pValues->tunnelId, sizeof(pValues->tunnelId));
    }

    if (fieldIDs & e_FIELD_L2TPv2_SID)
    {
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_SID, (t_Handle)&pValues->sessionId, sizeof(pValues->sessionId));
    }

    if (fieldIDs & e_FIELD_L2TPv2_NS)
    {
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_NS, (t_Handle)&pValues->sequenceNo, sizeof(pValues->sequenceNo));
    }

    if (fieldIDs & e_FIELD_L2TPv2_NR)
    {
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_NR, (t_Handle)&pValues->expectedSequenceNo, sizeof(pValues->expectedSequenceNo));
    }

    if (fieldIDs & e_FIELD_L2TPv2_OFFSIZE)
    {
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_OFFSIZE, (t_Handle)&pValues->offsetSize, sizeof(pValues->offsetSize));
    }

    if (fieldIDs & e_FIELD_L2TPv2_NEXT_HEADER_TYPE)
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_L2TPv2_VALID_FIELDS)
        HDR_L2TPv2_ConfigField(h_Header, e_FIELD_L2TPv2_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error HDR_L2TPv2_Randomize(t_Handle h_Header)
{
    t_L2TPv2 *pHeaderStr = ((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_TBIT))
        pHeaderStr->data.type = (uint8_t)(rand() %2);

    /* If Control Message - ENABLE: L,S DISABLE: O,P*/
    if(pHeaderStr->data.type)
    {
        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_LBIT))
        pHeaderStr->data.lengthPresent = 1;

        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_SBIT))
        pHeaderStr->data.sequencePresent = 1;

        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_OBIT))
        pHeaderStr->data.offsetPresent = 0;

        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_PBIT))
        pHeaderStr->data.priority = 0;
    }
    else
    {
         if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_LBIT))
        pHeaderStr->data.lengthPresent = (uint8_t)(rand() %2);

        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_SBIT))
        pHeaderStr->data.sequencePresent = (uint8_t)(rand() %2);

        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_OBIT))
        pHeaderStr->data.offsetPresent = (uint8_t)(rand() %2);

        if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_PBIT))
        pHeaderStr->data.priority = (uint8_t)(rand() %2);
    }

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_VER))
        pHeaderStr->data.version = 0x02;

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_LEN))
        pHeaderStr->data.length = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_L2TPv2_LEN);

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_TID))
        pHeaderStr->data.tunnelId = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_SID))
        pHeaderStr->data.sessionId = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_NS))
        pHeaderStr->data.sequenceNo = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_NR))
        pHeaderStr->data.expectedSequenceNo = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_L2TPv2_OFFSIZE))
         pHeaderStr->data.offsetSize = (uint16_t)(rand() % (uint32_t)MAX_PADDING_LIMIT);

    return E_OK;
}

t_Error HDR_L2TPv2_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_Handle p_NextHeader = NULL;
    t_L2TPv2* p_Header = NULL;
    uint16_t padLength = 0;
    uint8_t padValue = 0;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_L2TPv2*)p_Frame->h_HeaderStruct;

    /* Serialize the values */
     if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK )
     {
        return E_NOT_IN_RANGE;
    }

     if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv2_LBIT))
     {
         /* If L = 1, then compute the header length if needed */
         if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv2_LEN))
         {
             /* Get the next header */
             p_NextHeader = INTERNAL_GetNextHeader(h_Header);

             /* Length field indicates the length of the L2TP header + L2TP Message Payload  */
             p_Header->data.length = (uint8_t)(HDR_L2TPv2_GetHeaderSize(h_Header));
             if (p_NextHeader)
             p_Header->data.length += FRAME_GetFrameSize(p_NextHeader, p_NextHeader);
         }
         /* If L = 1, then serialize length field */
         if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.length), sizeof(p_Header->data.length), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
         {
             return E_NOT_IN_RANGE;
         }
     }

     if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.tunnelId), sizeof(p_Header->data.tunnelId), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.sessionId), sizeof(p_Header->data.sessionId), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
     {
        return E_NOT_IN_RANGE;
    }

     if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv2_SBIT))
     {
          /* If S = 1, then serialize sequence and expected sequence field */
         if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.sequenceNo), sizeof(p_Header->data.sequenceNo), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
              INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.expectedSequenceNo), sizeof(p_Header->data.expectedSequenceNo), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
         {
             return E_NOT_IN_RANGE;
         }
     }

     if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv2_OBIT))
     {
        /* If O = 1, then serialize offset field */
         if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.offsetSize), sizeof(p_Header->data.offsetSize), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
        {
            return E_NOT_IN_RANGE;
        }
         /* Pad the L2TP frame if needed */
         if (p_Header->data.offsetSize)
         {
             padLength = p_Header->data.offsetSize;
             /* Add padding */
             while (padLength-- > 0)
             {
                 if (INTERNAL_SerializeValue((uint8_t*)(&padValue), sizeof(padValue), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
                 {
                     return E_NOT_IN_RANGE;
                 }
             }
         }
     }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_L2TPv2_GetHeaderSize(t_Handle h_Header)
{
   t_L2TPv2* pHeaderStr = (t_L2TPv2*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint32_t size = FIXED_HEADER_SIZE_L2TPv2;

   if(FLAG_CHECK(pHeaderStr->autoFlags, e_FIELD_L2TPv2_LBIT))
   {
        /* If L = 1, then length field exist */
       size += sizeof(pHeaderStr->data.length);
   }

    if(FLAG_CHECK(pHeaderStr->autoFlags, e_FIELD_L2TPv2_SBIT))
    {
         /* If S = 1, then sequence and expected sequence field exist */
       size += sizeof(pHeaderStr->data.sequenceNo) + sizeof(pHeaderStr->data.expectedSequenceNo);
    }

    if(FLAG_CHECK(pHeaderStr->autoFlags, e_FIELD_L2TPv2_OBIT))
    {
        /* If O = 1, then offset field exist */
          size += sizeof(pHeaderStr->data.offsetSize) + pHeaderStr->data.offsetSize;
    }

    return size;
}

t_Error HDR_L2TPv2_Init(t_Handle h_Header)
{
    t_L2TPv2* p_Header = NULL;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_L2TPv2*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if(!p_Header->data.lengthPresent)
    {
        /* If L = 1, then clear length field */
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_L2TPv2_LBIT);
    }

    if(!p_Header->data.sequencePresent)
    {
         /* If S = 1, then clear sequence and expected sequence field */
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_L2TPv2_SBIT);
    }

    if(!p_Header->data.offsetPresent)
    {
        /* If O = 1, then clear offset field */
        FLAG_CLEAR(p_Header->autoFlags, e_FIELD_L2TPv2_OBIT);
    }

    return E_OK;
}

t_Handle HDR_L2TPv2_GetHeader(t_Handle hFrame)
{
     /* Get the frame handle */
     t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2tpHeader.l2tpv2Header);
}

t_Error HDR_L2TPv2_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}


/** @} */ /* end of L2TPv2 group */

/**************************************************************************//**
@Group          L2TPv3

@Description    L2TPv3 functions

@{
*//***************************************************************************/
t_Handle HDR_L2TPv3_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_L2TPv3 *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    p_Header = (t_L2TPv3*)MEMALLOC(sizeof(t_L2TPv3));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_L2TPv3));


    /* Set the auto flags (default: all dynamic fields must be set to auto)*/
    FLAG_SET(p_Header->autoFlags, e_FIELD_L2TPv3_LBIT | e_FIELD_L2TPv3_SBIT | e_FIELD_L2TPv3_LEN );

    /* Initialize the user configured L2TPv3 header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(u_PgL2tpv3Header));

    return (t_Handle) p_Header;
}


t_Handle HDR_L2TPv3_Clone(t_Handle h_Object)
{
    t_L2TPv3 *p_Object   = (t_L2TPv3*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_L2TPv3* p_Header = (t_L2TPv3*)MEMALLOC(sizeof(t_L2TPv3));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_L2TPv3));

    /* Take a copy of the L2TPv3 header */
    memcpy(p_Header, p_Object, sizeof(t_L2TPv3));

    return p_Header;
}

t_Error     HDR_L2TPv3_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_L2TPv3 *p_Header   = (t_L2TPv3*)((t_FrameStruct*)h_Header)->h_HeaderStruct;


     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
     uint32_t value32 = 0;        /* Holds the bit fields of size between 16 and 32 bits */

    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

     if(p_Header->data.pgL2tpv3CtrlMsgHeader.type)     /* Control Message */
     {
          /* Get the value of the field specified by fieldID */
         switch (fieldID)
         {
             case e_FIELD_L2TPv3_TBIT:
             {
                  value8 = (uint8_t)p_Header->data.pgL2tpv3CtrlMsgHeader.type;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
             }
             break;
             case e_FIELD_L2TPv3_LBIT:
             {
                  value8 = (uint8_t)p_Header->data.pgL2tpv3CtrlMsgHeader.lengthPresent;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
             }
             break;
             case e_FIELD_L2TPv3_SBIT:
             {
                 value8 = (uint8_t)p_Header->data.pgL2tpv3CtrlMsgHeader.sequencePresent;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
             }
             break;
             case e_FIELD_L2TPv3_VER:
             {
                 value8 = (uint8_t)p_Header->data.pgL2tpv3CtrlMsgHeader.version;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
             }
             break;
             case e_FIELD_L2TPv3_LEN:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3CtrlMsgHeader.length, h_Value, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.length), size);
             }
             case e_FIELD_L2TPv3_CCID:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3CtrlMsgHeader.controlConnectionId, h_Value, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.controlConnectionId), size);
             }
             break;
             case e_FIELD_L2TPv3_NS:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3CtrlMsgHeader.sequenceNo, h_Value, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.sequenceNo), size);
             }
             break;
             case e_FIELD_L2TPv3_NR:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo, h_Value, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo), size);
             }
             break;
             case e_FIELD_L2TPv3_NEXT_HEADER_TYPE:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3CtrlMsgHeader.nextHeaderType, h_Value, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.nextHeaderType), size);
             }
             break;

             case e_FIELD_L2TPv3_VALID_FIELDS:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3CtrlMsgHeader.validFields, h_Value, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.validFields), size);
             }
             break;

             default:
             {
                   RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
             }
         }
     }
     else
     {
          /* Get the value of the field specified by fieldID */
         switch (fieldID)
         {
             case e_FIELD_L2TPv3_TBIT:
             {
                  value8 = (uint8_t)p_Header->data.pgL2tpv3DataMsgHeader.type;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
             }
             break;
             case e_FIELD_L2TPv3_VER:
             {
                 value8 = (uint8_t)p_Header->data.pgL2tpv3DataMsgHeader.version;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
             }
             break;
             case e_FIELD_L2TPv3_SID:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3DataMsgHeader.sessionId, h_Value, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.sessionId), size);
             }
             break;
             case e_FIELD_L2TPv3_COOKIE_SIZE:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3DataMsgHeader.cookieSize, h_Value, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.cookieSize), size);
             }
             break;
             case e_FIELD_L2TPv3_COOKIE:
             {
                 SAFE_COPY_VALUE(p_Header->data.pgL2tpv3DataMsgHeader.cookie, h_Value, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.cookie), size);
             }
             break;
             case e_FIELD_L2TPv3_L2_SUBLAYER_SBIT:
             {
                 value8 = (uint8_t)p_Header->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceValid;
                 SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);

             }
             break;
             case e_FIELD_L2TPv3_L2_SUBLAYER_NS:
             {
                 value32 = (uint32_t)p_Header->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceNo;
                 SAFE_COPY_VALUE(&value32, h_Value, sizeof(value32), size);

             }
             break;
             case e_FIELD_L2TPv3_NEXT_HEADER_TYPE:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3DataMsgHeader.nextHeaderType, h_Value, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.nextHeaderType), size);
             }
             break;

             case e_FIELD_L2TPv3_VALID_FIELDS:
             {
                 SAFE_COPY_VALUE(&p_Header->data.pgL2tpv3DataMsgHeader.validFields, h_Value, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.validFields), size);
             }
             break;

             default:
             {
                   RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
             }
         }
     }

    return E_OK;
}

t_Error HDR_L2TPv3_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_L2TPv3 *p_Header = (t_L2TPv3*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
     uint8_t value8 = 0;        /* Holds the bit fields of size less than 8 bits */
     uint32_t value32 = 0;        /* Holds the bit fields of size between 16 and 32 bits */

    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

     if(p_Header->data.pgL2tpv3CtrlMsgHeader.type)     /* Control Message */
     {
          /* Configure the value of the field specified by fieldID */
         switch (fieldID)
         {
             case e_FIELD_L2TPv3_TBIT:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3CtrlMsgHeader.type = value8;
             }
             break;
             case e_FIELD_L2TPv3_LBIT:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3CtrlMsgHeader.lengthPresent = value8;
             }
             break;
             case e_FIELD_L2TPv3_SBIT:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3CtrlMsgHeader.sequencePresent = value8;
             }
             break;
             case e_FIELD_L2TPv3_VER:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3CtrlMsgHeader.version = value8;
             }
             break;
             case e_FIELD_L2TPv3_LEN:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3CtrlMsgHeader.length, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.length), size);
                 FLAG_CLEAR(p_Header->autoFlags, fieldID);
             }
             break;
             case e_FIELD_L2TPv3_CCID:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3CtrlMsgHeader.controlConnectionId, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.controlConnectionId), size);
             }
             break;
             case e_FIELD_L2TPv3_NS:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3CtrlMsgHeader.sequenceNo, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.sequenceNo), size);
             }
             break;
             case e_FIELD_L2TPv3_NR:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo), size);
             }
             break;
             case e_FIELD_L2TPv3_NEXT_HEADER_TYPE:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3CtrlMsgHeader.nextHeaderType, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.nextHeaderType), size);
             }
             break;
             case e_FIELD_L2TPv3_VALID_FIELDS:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3CtrlMsgHeader.validFields, sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.validFields), size);
             }
             break;
             default:
             {
                   RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
             }
         }
     }
    else
    {
          /* Configure the value of the field specified by fieldID */
         switch (fieldID)
         {
             case e_FIELD_L2TPv3_TBIT:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3DataMsgHeader.type = value8;
             }
             break;
             case e_FIELD_L2TPv3_VER:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3DataMsgHeader.version = value8;
             }
             break;
             case e_FIELD_L2TPv3_SID:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3DataMsgHeader.sessionId, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.sessionId), size);
             }
             break;
             case e_FIELD_L2TPv3_COOKIE_SIZE:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3DataMsgHeader.cookieSize, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.cookieSize), size);
             }
             break;
             case e_FIELD_L2TPv3_COOKIE:
             {
                 SAFE_COPY_VALUE(h_Value, p_Header->data.pgL2tpv3DataMsgHeader.cookie, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.cookie), size);
             }
             break;
             case e_FIELD_L2TPv3_L2_SUBLAYER_SBIT:
             {
                 SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
                 p_Header->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceValid = value8;
             }
             break;
             case e_FIELD_L2TPv3_L2_SUBLAYER_NS:
             {
                 SAFE_COPY_VALUE(h_Value, &value32, sizeof(value32), size);
                 p_Header->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceNo = value32;
             }
             break;
             case e_FIELD_L2TPv3_NEXT_HEADER_TYPE:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3DataMsgHeader.nextHeaderType, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.nextHeaderType), size);
             }
             break;
             case e_FIELD_L2TPv3_VALID_FIELDS:
             {
                 SAFE_COPY_VALUE(h_Value, &p_Header->data.pgL2tpv3DataMsgHeader.validFields, sizeof(p_Header->data.pgL2tpv3DataMsgHeader.validFields), size);
             }
             break;
             default:
             {
                   RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
             }
         }
     }

    return E_OK;
}

t_Error HDR_L2TPv3_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_L2TPv3 *p_Header = (t_L2TPv3*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
       case e_FIELD_L2TPv3_LBIT:
       case e_FIELD_L2TPv3_SBIT:
       case e_FIELD_L2TPv3_LEN:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_L2TPv3_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    u_PgL2tpv3Header* pValues = (u_PgL2tpv3Header*)hValues;
     uint8_t value8 = 0;    /* Holds the bit fields of size less than 8 bits */
     uint32_t value32 = 0;    /* Holds the bit fields of size between 16 and 32 bits */

    if (!h_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_L2TPv3_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    if(pValues->pgL2tpv3CtrlMsgHeader.type) /* Control Message */
    {
          /* Configure the L2TPv3 header fields specified by fieldIDs */
         if (fieldIDs & e_FIELD_L2TPv3_TBIT)
         {
              value8 = (uint8_t)pValues->pgL2tpv3CtrlMsgHeader.type;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_TBIT, (t_Handle)&value8, sizeof(value8));
         }

         if (fieldIDs & e_FIELD_L2TPv3_LBIT)
         {
              value8 = (uint8_t)pValues->pgL2tpv3CtrlMsgHeader.lengthPresent;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_LBIT, (t_Handle)&value8, sizeof(value8));
         }

         if (fieldIDs & e_FIELD_L2TPv3_SBIT)
         {
             value8 = (uint8_t)pValues->pgL2tpv3CtrlMsgHeader.sequencePresent;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_SBIT, (t_Handle)&value8, sizeof(value8));
         }
         if (fieldIDs & e_FIELD_L2TPv3_VER)
         {
              value8 = (uint8_t)pValues->pgL2tpv3CtrlMsgHeader.version;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_VER, (t_Handle)&value8, sizeof(value8));
         }

         if (fieldIDs & e_FIELD_L2TPv3_LEN)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_LEN, (t_Handle)&pValues->pgL2tpv3CtrlMsgHeader.length, sizeof(pValues->pgL2tpv3CtrlMsgHeader.length));
         }

         if (fieldIDs & e_FIELD_L2TPv3_CCID)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_CCID, (t_Handle)&pValues->pgL2tpv3CtrlMsgHeader.controlConnectionId, sizeof(pValues->pgL2tpv3CtrlMsgHeader.controlConnectionId));
         }

         if (fieldIDs & e_FIELD_L2TPv3_NS)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_NS, (t_Handle)&pValues->pgL2tpv3CtrlMsgHeader.sequenceNo, sizeof(pValues->pgL2tpv3CtrlMsgHeader.sequenceNo));
         }

         if (fieldIDs & e_FIELD_L2TPv3_NR)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_NR, (t_Handle)&pValues->pgL2tpv3CtrlMsgHeader.expectedSequenceNo, sizeof(pValues->pgL2tpv3CtrlMsgHeader.expectedSequenceNo));
         }
         if (fieldIDs & e_FIELD_L2TPv3_NEXT_HEADER_TYPE)
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_NEXT_HEADER_TYPE, (t_Handle)&pValues->pgL2tpv3CtrlMsgHeader.nextHeaderType, sizeof(pValues->pgL2tpv3CtrlMsgHeader.nextHeaderType));

         if (fieldIDs & e_FIELD_L2TPv3_VALID_FIELDS)
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_VALID_FIELDS, (t_Handle)&pValues->pgL2tpv3CtrlMsgHeader.validFields, sizeof(pValues->pgL2tpv3CtrlMsgHeader.validFields));
    }
    else
    {
         if (fieldIDs & e_FIELD_L2TPv3_TBIT)
         {
              value8 = (uint8_t)pValues->pgL2tpv3DataMsgHeader.type;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_TBIT, (t_Handle)&value8, sizeof(value8));
         }

         if (fieldIDs & e_FIELD_L2TPv3_VER)
         {
              value8 = (uint8_t)pValues->pgL2tpv3DataMsgHeader.version;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_VER, (t_Handle)&value8, sizeof(value8));
         }

         if (fieldIDs & e_FIELD_L2TPv3_SID)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_SID, (t_Handle)&pValues->pgL2tpv3DataMsgHeader.sessionId, sizeof(pValues->pgL2tpv3DataMsgHeader.sessionId));
         }

         if (fieldIDs & e_FIELD_L2TPv3_COOKIE_SIZE)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_COOKIE_SIZE, (t_Handle)&pValues->pgL2tpv3DataMsgHeader.cookieSize, sizeof(pValues->pgL2tpv3DataMsgHeader.cookieSize));
         }

         if (fieldIDs & e_FIELD_L2TPv3_COOKIE)
         {
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_COOKIE, (t_Handle)&pValues->pgL2tpv3DataMsgHeader.cookie, sizeof(pValues->pgL2tpv3DataMsgHeader.cookie));
         }

         if (fieldIDs & e_FIELD_L2TPv3_L2_SUBLAYER_SBIT)
         {
              value8 = (uint8_t)pValues->pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceValid;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_L2_SUBLAYER_SBIT, (t_Handle)&value8, sizeof(value8));
         }

         if (fieldIDs & e_FIELD_L2TPv3_L2_SUBLAYER_NS)
         {
              value32 = (uint32_t)pValues->pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceNo;
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_L2_SUBLAYER_NS, (t_Handle)&value32, sizeof(value32));
         }

         if (fieldIDs & e_FIELD_L2TPv3_NEXT_HEADER_TYPE)
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_NEXT_HEADER_TYPE, (t_Handle)&pValues->pgL2tpv3DataMsgHeader.nextHeaderType, sizeof(pValues->pgL2tpv3DataMsgHeader.nextHeaderType));

         if (fieldIDs & e_FIELD_L2TPv3_VALID_FIELDS)
             HDR_L2TPv3_ConfigField(h_Header, e_FIELD_L2TPv3_VALID_FIELDS, (t_Handle)&pValues->pgL2tpv3DataMsgHeader.validFields, sizeof(pValues->pgL2tpv3DataMsgHeader.validFields));
    }

    return E_OK;
}

t_Error HDR_L2TPv3_Randomize(t_Handle h_Header)
{
    t_L2TPv3     *pHeaderStr = ((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t     index;

   /* Generate random values for the fields not configured by the user */
   /* Randomize type field when type is not configured in both CtrlMsg and DataMsg */
    if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_TBIT) &&
        !(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_TBIT))
    {
        pHeaderStr->data.pgL2tpv3CtrlMsgHeader.type = (uint8_t)(rand() %2);
    }

    /* If Control Message - ENABLE: L,S Bits*/
    if(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.type)
    {
        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_LBIT))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.lengthPresent = 1;                 /* As per RFC 3931 */

        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_SBIT))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.sequencePresent = 1;

        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_VER))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.version = 3;

        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_LEN))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.length = (uint16_t)rand();
        else
            FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_L2TPv3_LEN);

        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_CCID))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.controlConnectionId = (uint32_t)rand();

        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_NS))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.sequenceNo = (uint16_t)rand();

        if (!(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.validFields & e_FIELD_L2TPv3_NR))
            pHeaderStr->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo = (uint16_t)rand();
    }
    else /* Data Message */
    {
         if (!(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_VER))
            pHeaderStr->data.pgL2tpv3DataMsgHeader.version = 3;

        if (!(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_SID))
            pHeaderStr->data.pgL2tpv3DataMsgHeader.sessionId = (uint32_t)rand();

        if (!(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_COOKIE_SIZE))
            pHeaderStr->data.pgL2tpv3DataMsgHeader.cookieSize = 4;

        if (!(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_COOKIE))
        {
            for(index = 0; index < L2TP_V3_COOKIE_SIZE; index++)
            {
                pHeaderStr->data.pgL2tpv3DataMsgHeader.cookie[index] = (uint8_t)rand();
            }
        }

        if (!(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_L2_SUBLAYER_SBIT))
            pHeaderStr->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceValid = (uint8_t)(rand() %2);

        if (!(pHeaderStr->data.pgL2tpv3DataMsgHeader.validFields & e_FIELD_L2TPv3_L2_SUBLAYER_NS))
               pHeaderStr->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer.sequenceNo = (uint32_t)rand();
    }

    return E_OK;
}

t_Error HDR_L2TPv3_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_Handle p_NextHeader = NULL;
    t_L2TPv3* p_Header = NULL;
    uint8_t *p_Value8 = NULL;

    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_L2TPv3*)p_Frame->h_HeaderStruct;

   /* Serialize the values */
     if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data), sizeof(uint16_t), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK )
     {
        return E_NOT_IN_RANGE;
    }

    if(p_Header->data.pgL2tpv3CtrlMsgHeader.type)    /* Control Message */
    {
         if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv3_LBIT))
         {
             /* Compute the header length if needed */
             if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv3_LEN))
             {
                 /* Get the next header */
                 p_NextHeader = INTERNAL_GetNextHeader(h_Header);

                 /* Length field indicates the length of the L2TP header + L2TP Message Payload  */
                 p_Header->data.pgL2tpv3CtrlMsgHeader.length = (uint8_t)(HDR_L2TPv3_GetHeaderSize(h_Header));
                 if (p_NextHeader)
                 p_Header->data.pgL2tpv3CtrlMsgHeader.length += FRAME_GetFrameSize(p_NextHeader, p_NextHeader);
             }

             /* If L = 1, then serialize length field */
             if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3CtrlMsgHeader.length), sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.length), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
             {
                 return E_NOT_IN_RANGE;
             }
         }

          if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3CtrlMsgHeader.controlConnectionId), sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.controlConnectionId), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
         {
             return E_NOT_IN_RANGE;
         }

         if(FLAG_CHECK(p_Header->autoFlags, e_FIELD_L2TPv3_SBIT))
         {
              /* If S = 1, then serialize sequence and expected sequence field */
             if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3CtrlMsgHeader.sequenceNo), sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.sequenceNo), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
                  INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo), sizeof(p_Header->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
             {
                 return E_NOT_IN_RANGE;
             }
         }
    }
     else /* Data Message */
     {
         p_Value8 = p_Header->data.pgL2tpv3DataMsgHeader.cookie;
         if(p_Header->data.pgL2tpv3DataMsgHeader.cookieSize == L2TP_V3_COOKIE_SIZE/2)
         {
              /* Move the pointer to serialize the required cookie */
              p_Value8 += p_Header->data.pgL2tpv3DataMsgHeader.cookieSize;
         }

         if (INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3DataMsgHeader.res1), sizeof(p_Header->data.pgL2tpv3DataMsgHeader.res1), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
             INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3DataMsgHeader.sessionId), sizeof(p_Header->data.pgL2tpv3DataMsgHeader.sessionId), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
             INTERNAL_SerializeValue(p_Value8, p_Header->data.pgL2tpv3DataMsgHeader.cookieSize, p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK ||
             INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer), sizeof(p_Header->data.pgL2tpv3DataMsgHeader.l2SpecificSublayer), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK )
         {
             return E_NOT_IN_RANGE;
         }
     }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
 }

uint32_t HDR_L2TPv3_GetHeaderSize(t_Handle h_Header)
{
   t_L2TPv3* pHeaderStr = (t_L2TPv3*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint32_t size = 0;

   if(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.type)    /* Control Message */
    {
        size += FIXED_HEADER_SIZE_L2TPv3_CONTROL_MESSAGE;
        if(FLAG_CHECK(pHeaderStr->autoFlags, e_FIELD_L2TPv3_LBIT))
        {
            /* If L = 1, then length field exist */
            size += sizeof(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.length);
        }

        if(FLAG_CHECK(pHeaderStr->autoFlags, e_FIELD_L2TPv3_SBIT))
        {
             /* If S = 1, then sequence and expected sequence field exist */
            size += sizeof(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.sequenceNo) + sizeof(pHeaderStr->data.pgL2tpv3CtrlMsgHeader.expectedSequenceNo);
        }
     }
     else /* Data Message */
     {
        size += FIXED_HEADER_SIZE_L2TPv3_DATA_MESSAGE + pHeaderStr->data.pgL2tpv3DataMsgHeader.cookieSize;
     }

    return size;
}

t_Error HDR_L2TPv3_Init(t_Handle h_Header)
{
    t_L2TPv3* p_Header = NULL;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_L2TPv3*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if(p_Header->data.pgL2tpv3CtrlMsgHeader.type)    /* Control Message */
     {
        if(!p_Header->data.pgL2tpv3CtrlMsgHeader.lengthPresent)
        {
            /* If L = 1, then clear length field */
            FLAG_CLEAR(p_Header->autoFlags, e_FIELD_L2TPv3_LBIT);
        }

        if(!p_Header->data.pgL2tpv3CtrlMsgHeader.sequencePresent)
        {
            /* If S = 1, then clear sequence and expected sequence field */
           FLAG_CLEAR(p_Header->autoFlags, e_FIELD_L2TPv3_SBIT);
        }
     }

    return E_OK;
}

t_Handle HDR_L2TPv3_GetHeader(t_Handle hFrame)
{
     /* Get the frame handle */
     t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2tpHeader.l2tpv3Header);
}

t_Error HDR_L2TPv3_GetHeaderError(t_Handle hHeader, t_Handle hError)
{

    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}
/** @} */ /* end of L2TPv3 group */

/**************************************************************************//**
@Group          PWE3 Encapsulation

@Description    PWE3 Encapsulation functions

@
*//***************************************************************************/

t_Handle HDR_PWE3Encap_Config(t_Handle h_Header, t_ProtocolInfo *p_ProtocolInfo)
{
   t_PWE3Encap  *p_Header = NULL;
   UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!h_Header)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }
    p_Header = (t_PWE3Encap*)MEMALLOC(sizeof(t_PWE3Encap));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the PWE3 Encapsulation Header"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_PWE3Encap));

 /* Initialize the user configured pwe3Encap header fields in the header */
    memcpy(&p_Header->data, h_Header, sizeof(t_PgPwe3CtrlWordHeader));

    FLAG_SET(p_Header->autoFlags, e_FIELD_PWE3_ENCAP_LEN | e_FIELD_PWE3_ENCAP_TYPE );

    return (t_Handle) p_Header;
}

t_Handle HDR_PWE3Encap_Clone(t_Handle h_Object)
{
    t_PWE3Encap  *p_Object = (t_PWE3Encap*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_PWE3Encap  *p_Header = (t_PWE3Encap*)MEMALLOC(sizeof(t_PWE3Encap));

    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the PWE3 Encapsulation Header"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_PWE3Encap));
    memcpy(p_Header, p_Object, sizeof(t_PWE3Encap));

    return p_Header;
}

t_Error HDR_PWE3Encap_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PWE3Encap  *p_Header = (t_PWE3Encap*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint16_t value16       = 0;
    uint8_t value8         = 0;

    if (!(p_Header && h_Value && size))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }


    switch (fieldID)
    {
        case e_FIELD_PWE3_ENCAP_TYPE:
            value8 = (uint8_t)p_Header->data.type;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
            break;

        case e_FIELD_PWE3_ENCAP_FLAGS:
            value8 = (uint8_t)p_Header->data.flags;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
            break;

        case e_FIELD_PWE3_ENCAP_FRG:
            value8 = (uint8_t)p_Header->data.frg;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
            break;

        case e_FIELD_PWE3_ENCAP_LEN:
            value8 = (uint8_t)p_Header->data.length;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
            break;

        case e_FIELD_PWE3_ENCAP_SEQ_NUM:
            value16 = (uint16_t)p_Header->data.seqNum;
            SAFE_COPY_VALUE(&value16, h_Value, sizeof(value16), size);
            break;

        case e_FIELD_PWE3_ENCAP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;

        case e_FIELD_PWE3_ENCAP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PWE3Encap_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_PWE3Encap  *p_Header = (t_PWE3Encap*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint16_t value16       = 0;
    uint8_t value8         = 0;

    if (!(p_Header && h_Value && size))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

    size = (size) ? size : (uint32_t)(-1);

    switch (fieldID)
    {
        case e_FIELD_PWE3_ENCAP_TYPE:
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.type = value8;
            break;

        case e_FIELD_PWE3_ENCAP_FLAGS:
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.flags = value8;
            break;

        case e_FIELD_PWE3_ENCAP_FRG:
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.frg = value8;
            break;

        case e_FIELD_PWE3_ENCAP_LEN:
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.length = value8;
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
            break;

        case e_FIELD_PWE3_ENCAP_SEQ_NUM:
            SAFE_COPY_VALUE(h_Value, &value16, sizeof(value16), size);
            p_Header->data.seqNum = value16;
            break;

        case e_FIELD_PWE3_ENCAP_NEXT_HEADER_TYPE:
             SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
             break;

        case e_FIELD_PWE3_ENCAP_VALID_FIELDS:
             SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
             break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PWE3Encap_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_PWE3Encap  *p_Header = (t_PWE3Encap*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }
    switch (fieldID)
    {

        case e_FIELD_PWE3_ENCAP_LEN:
        case e_FIELD_PWE3_ENCAP_TYPE:
         break;
         default:
         {
             RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
         }break;
    }
    if (bSetAuto)
    {
        FLAG_SET(p_Header->autoFlags, fieldID);
    }
    else
    {
        FLAG_CLEAR(p_Header->autoFlags, fieldID);
    }

    return E_OK;
}

t_Error HDR_PWE3Encap_ConfigHeader(t_Handle h_Header, t_Handle h_Values, uint32_t fieldIDs)
{
    t_PWE3Encap      *p_Header          = (t_PWE3Encap*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgPwe3CtrlWordHeader    *p_Values     = (t_PgPwe3CtrlWordHeader*)h_Values;
    uint16_t value16        =0;
    uint8_t value8          =0;

    if (!p_Header || !p_Values)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

    if (!(fieldIDs & e_FIELD_PWE3_ENCAP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    if (fieldIDs & e_FIELD_PWE3_ENCAP_TYPE)
    {
        value8 = (uint8_t)p_Header->data.type;
        HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_PWE3_ENCAP_TYPE, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PWE3_ENCAP_FLAGS)
    {
       value8 = (uint8_t)p_Header->data.flags;
       HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_PWE3_ENCAP_FLAGS, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PWE3_ENCAP_FRG)
    {
        value8 = (uint8_t)p_Header->data.frg;
        HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_PWE3_ENCAP_FRG, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PWE3_ENCAP_LEN)
    {
        value8 = (uint8_t)p_Header->data.length;
        HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_PWE3_ENCAP_LEN, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PWE3_ENCAP_SEQ_NUM)
    {
        value16 = (uint16_t)p_Header->data.seqNum;
        HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_PWE3_ENCAP_SEQ_NUM, (t_Handle)&value16, sizeof(value16));
    }

    if (fieldIDs & e_FIELD_PWE3_ENCAP_NEXT_HEADER_TYPE)
         HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_ETH_NEXT_HEADER_TYPE, (t_Handle)&p_Values->nextHeaderType,sizeof(p_Values->nextHeaderType));

    if (fieldIDs & e_FIELD_PWE3_ENCAP_VALID_FIELDS)
         HDR_PWE3Encap_ConfigField(h_Header, e_FIELD_ETH_VALID_FIELDS, (t_Handle)&p_Values->validFields,sizeof(p_Values->validFields));

    return E_OK;
}

t_Error HDR_PWE3Encap_Randomize(t_Handle h_Header)
{
    t_PWE3Encap  *p_HeaderStr;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

/*  Get the handle to the PWE3 Control Word header */
    p_HeaderStr = (t_PWE3Encap *)(((t_FrameStruct*)h_Header)->h_HeaderStruct);

/*  Generate random values for the fields which are not configured by the user */
    if (!(p_HeaderStr->data.validFields & e_FIELD_PWE3_ENCAP_TYPE))
    {
        p_HeaderStr->data.type = 0x0;
    }
    else
    {
        FLAG_CLEAR(p_HeaderStr->autoFlags, e_FIELD_PWE3_ENCAP_TYPE);
    }


    if (!(p_HeaderStr->data.validFields &  e_FIELD_PWE3_ENCAP_FLAGS))
        p_HeaderStr->data.flags = (uint8_t)rand();

    if (!(p_HeaderStr->data.validFields & e_FIELD_PWE3_ENCAP_FRG))
        p_HeaderStr->data.frg = (uint8_t)rand();

    if (!(p_HeaderStr->data.validFields & e_FIELD_PWE3_ENCAP_LEN))
    {
        p_HeaderStr->data.length = (uint8_t)rand();
    }
    else
    {
        FLAG_CLEAR(p_HeaderStr->autoFlags, e_FIELD_PWE3_ENCAP_LEN);
    }

    if (!(p_HeaderStr->data.validFields & e_FIELD_PWE3_ENCAP_SEQ_NUM))
        p_HeaderStr->data.seqNum = (uint16_t)rand();

    return E_OK;
}


t_Error HDR_PWE3Encap_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *p_Size)
{
    t_FrameStruct    *p_Frame        = (t_FrameStruct*)h_Header;
    t_FrameStruct    *p_PrevHeader   = NULL, *p_NextHeader = NULL;
    t_PWE3Encap  *p_Header           = (t_PWE3Encap*)p_Frame->h_HeaderStruct;
    uint32_t    encaplen = 0;

    if (!p_Header)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

    p_PrevHeader = (t_FrameStruct*)INTERNAL_GetPrevHeader(h_Header);

    if ( (!p_PrevHeader) ||
         ((p_PrevHeader->HeaderID != e_HEADER_TYPE_PWE3_ATM_MPLS_DEMUX) && (p_PrevHeader->HeaderID != e_HEADER_TYPE_PWE3_SAToP_MPLS_DEMUX)) )
    {
        REPORT_ERROR(MAJOR, E_INVALID_STATE, ("PWE3 Encap MUST be over PWE3 Demux!"));
    }

     /* If length is set to auto get the frame length */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_PWE3_ENCAP_LEN))
    {
        p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(h_Header);
        if(p_NextHeader)
        {
             /* Get the Encap length and Payload length */
             if (p_NextHeader)
                 encaplen += FRAME_GetFrameSize(h_Header, h_Header);

             if(encaplen < MIN_FRAME_LEN_ETHERNET)
             {
                p_Header->data.length = (uint8_t) encaplen;
             }
             else
             {
                p_Header->data.length = 0;
             }
        }
    }
     /* Serialize the PWE3 Control Word fields */
     if ((INTERNAL_SerializeValue((uint8_t*)(&p_Header->data), sizeof(uint32_t), p_Buffer, p_Size, KEEP_ENDIANNESS) != E_OK) )
     {
        return E_NOT_IN_RANGE;
     }

     /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, p_Size);
}

uint32_t HDR_PWE3Encap_GetHeaderSize(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return FIXED_HEADER_SIZE_PWE3_ENCAP;
}

t_Error HDR_PWE3Encap_Init(t_Handle h_Header)
{
    UNUSED(h_Header);

    /* Nothing to do */

    return E_OK;
}

t_Handle HDR_PWE3Encap_GetHeader(t_Handle hFrame)
{
   /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;

    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->pwe3CwHeader);
}

t_Error HDR_PWE3Encap_GetHeaderError(t_Handle h_Header, t_Handle hError)
{
    UNUSED(hError);UNUSED(h_Header);

    return E_OK;
}




/** @} */ /* end of PWE3 Encap group */

/**************************************************************************//**
@Group          HDLC

@Description    HDLC functions

@{
*//***************************************************************************/
t_Handle HDR_HDLC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_HDLC *p_Header = NULL;  /* Check the header handle */
    UNUSED(p_ProtocolInfo);

    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    /* Allocate memory for the HDLC header */
    p_Header = (t_HDLC*)MEMALLOC(sizeof(t_HDLC));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_HDLC));

    /* Configure the auto flags */
    FLAG_SET(p_Header->autoFlags, e_FIELD_HDLC_FCS);

    /* Initialize the user configured HDLC header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgHdlcHeader));

    if((p_Header->data.validFields & e_FIELD_HDLC_USE_FCS) && !(p_Header->data.bUseFinalCkSum) && p_Header->data.finalCheckSum)
    {
        REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Invalid FCS configuration"));
        return NULL;
    }

    if((p_Header->data.finalCheckSum && !p_Header->data.finalCheckSumType) || (p_Header->data.finalCheckSumType > e_FCS_32))
    {
        REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Invalid FCS configuration"));
        return NULL;
    }

    return (t_Handle) p_Header;
}

t_Handle HDR_HDLC_Clone(t_Handle h_Object)
{
    /* Get the handle to the HDLC header */
    t_HDLC* p_Object = (t_HDLC*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_HDLC* p_Header = (t_HDLC*)MEMALLOC(sizeof(t_HDLC));

    /* Check the handle */
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_HDLC));

    /* Copy the header fields */
    memcpy(p_Header, p_Object, sizeof(t_HDLC));

    return p_Header;
}

t_Error  HDR_HDLC_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the HDLC header */
    t_HDLC      *p_Header   = (t_HDLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t     value8;

    /* Check the handle */
    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Retrieve the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_HDLC_ADDRESS:
            if (p_Header->data.addressType == e_ADDRESS_8)
            {
                value8 = (uint8_t)p_Header->data.address;
                SAFE_COPY_VALUE(&value8, h_Value, e_ADDRESS_8, size);
            }
            else
            {
                 SAFE_COPY_VALUE(&p_Header->data.address, h_Value, e_ADDRESS_16, size);
            }
        break;
        case e_FIELD_HDLC_CONTROL:
            SAFE_COPY_VALUE(&p_Header->data.control, h_Value, sizeof(p_Header->data.control), size);
        break;
        case e_FIELD_HDLC_ADDRESS_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.addressType, h_Value, sizeof(p_Header->data.addressType), size);
        break;
        case e_FIELD_HDLC_USE_FCS:
            SAFE_COPY_VALUE(&p_Header->data.bUseFinalCkSum, h_Value, sizeof(p_Header->data.bUseFinalCkSum), size);
        break;
        case e_FIELD_HDLC_FCS:
            SAFE_COPY_VALUE(&p_Header->data.finalCheckSum, h_Value, sizeof(p_Header->data.finalCheckSum), size);
        break;
        case e_FIELD_HDLC_FCS_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.finalCheckSumType, h_Value, sizeof(p_Header->data.finalCheckSumType), size);
        break;
        case e_FIELD_HDLC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_HDLC_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_HDLC_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the HDLC header */
    t_HDLC* p_Header = (t_HDLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Check the handle */
    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_HDLC_ADDRESS:
            if (p_Header->data.addressType == e_ADDRESS_8)
            {
                 SAFE_COPY_VALUE(h_Value, (uint8_t *)(&p_Header->data.address)+1, e_ADDRESS_8, size);
            }
            else
            {
                 SAFE_COPY_VALUE(h_Value,&p_Header->data.address, e_ADDRESS_16, size);
            }
        break;
        case e_FIELD_HDLC_CONTROL:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.control, sizeof(p_Header->data.control), size);
        break;
        case e_FIELD_HDLC_ADDRESS_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.addressType, sizeof(p_Header->data.addressType), size);
        break;
        case e_FIELD_HDLC_USE_FCS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.bUseFinalCkSum, sizeof(p_Header->data.bUseFinalCkSum), size);
        break;
        case e_FIELD_HDLC_FCS:
        {/* The final checksum is not used unless the user sets it to auto or sets its value*/
            SAFE_COPY_VALUE(h_Value, &p_Header->data.finalCheckSum, sizeof(p_Header->data.finalCheckSum), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
            p_Header->data.bUseFinalCkSum = TRUE;
        }
        break;
        case e_FIELD_HDLC_FCS_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.finalCheckSumType, sizeof(p_Header->data.finalCheckSumType), size);
        break;
        case e_FIELD_HDLC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_HDLC_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_HDLC_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_HDLC* p_Header = (t_HDLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_HDLC_FCS:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    if ((fieldID==e_FIELD_HDLC_FCS) && bSetAuto) p_Header->data.bUseFinalCkSum = TRUE;

    return E_OK;
}

t_Error HDR_HDLC_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    /* Get the handle to the HDLC header */
   t_HDLC           *p_Header = (t_HDLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
   t_PgHdlcHeader   *pValues = (t_PgHdlcHeader*)hValues;
   uint8_t          value8;

   /* Check the handle */
   if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_HDLC_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the HDLC header based on the field ID */
    if (fieldIDs & e_FIELD_HDLC_ADDRESS)
    {
        if (p_Header->data.addressType == e_ADDRESS_8)
        {
             value8 = (uint8_t)p_Header->data.address;
             HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_ADDRESS, (t_Handle)&value8, e_ADDRESS_8);
        }
        else
        {
             HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_ADDRESS, (t_Handle)&pValues->address, e_ADDRESS_16);
        }
    }
    if (fieldIDs & e_FIELD_HDLC_CONTROL)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_CONTROL, (t_Handle)&pValues->control, sizeof(pValues->control));

    if (fieldIDs & e_FIELD_HDLC_ADDRESS_TYPE)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_ADDRESS_TYPE, (t_Handle)&pValues->addressType, sizeof(pValues->addressType));

    if(fieldIDs & e_FIELD_HDLC_USE_FCS)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_USE_FCS, (t_Handle)&pValues->bUseFinalCkSum, sizeof(pValues->bUseFinalCkSum));

    if(fieldIDs & e_FIELD_HDLC_FCS)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_FCS, (t_Handle)&pValues->finalCheckSum, sizeof(pValues->finalCheckSum));

    if (fieldIDs & e_FIELD_HDLC_FCS_TYPE)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_FCS_TYPE, (t_Handle)&pValues->finalCheckSumType,sizeof(pValues->finalCheckSumType));

    if (fieldIDs & e_FIELD_HDLC_NEXT_HEADER_TYPE)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_HDLC_VALID_FIELDS)
        HDR_HDLC_ConfigField(h_Header, e_FIELD_HDLC_NEXT_HEADER_TYPE, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_HDLC_Randomize(t_Handle h_Header)
{
    /* Get the handle to the HDLC header */
    t_HDLC* pHeaderStr = (t_HDLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_HDLC_ADDRESS))
        pHeaderStr->data.address = 0xFF;

    if (!(pHeaderStr->data.validFields & e_FIELD_HDLC_CONTROL))
        pHeaderStr->data.control = 0x03;

    if (!(pHeaderStr->data.validFields & e_FIELD_HDLC_ADDRESS_TYPE))
        pHeaderStr->data.addressType = e_ADDRESS_8;

    if (!(pHeaderStr->data.validFields & e_FIELD_HDLC_USE_FCS))
    {
        pHeaderStr->data.bUseFinalCkSum = (bool)(rand()%2);
    }

    if (pHeaderStr->data.bUseFinalCkSum && !(pHeaderStr->data.validFields & e_FIELD_HDLC_FCS))
    {
        pHeaderStr->data.finalCheckSum = (uint32_t)rand();
        FLAG_SET(pHeaderStr->autoFlags, e_FIELD_HDLC_FCS);
    }
    else if(pHeaderStr->data.validFields & e_FIELD_HDLC_FCS)
    {
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_HDLC_FCS);
        pHeaderStr->data.bUseFinalCkSum = TRUE;
    }

    if (pHeaderStr->data.bUseFinalCkSum && !(pHeaderStr->data.validFields & e_FIELD_HDLC_FCS_TYPE))
    {
        pHeaderStr->data.finalCheckSumType = (e_PgFcsType)((rand()%2) + 1);
    }

    return E_OK;
}

t_Error HDR_HDLC_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    /* Get the handle to the HDLC header */
    t_FrameStruct   *p_Frame = (t_FrameStruct*)h_Header; /* Frame handle */
    t_HDLC          *p_Header = (t_HDLC*)p_Frame->h_HeaderStruct; /* HDLC header handle */
    uint32_t        bufferSize = 0;       /* Holds the buffer size */
    uint8_t         *pPrevBuffer = 0;     /* Previous buffer pointer */
    uint32_t        finalCheckSum = 0;    /* Final CheckSum */
    uint8_t         fcsSize = 0;          /* FCS size */
    uint8_t         fcsOffset = 0;        /* To serialize the LSB 2 bytes for 16 bit FCS */
    uint16_t        value16;
    uint8_t         value8;

    /* Check the handle */
    if (!(p_Frame && h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Store the current buffer pointer for later */
    pPrevBuffer = *p_Buffer;
    bufferSize = *pSize;

    /* Serialize the values */

    if (p_Header->data.addressType == e_ADDRESS_8)
    {
        value8 = (uint8_t)p_Header->data.address;
        if (INTERNAL_SerializeValue((uint8_t*)(&value8), e_ADDRESS_8, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.control), sizeof(p_Header->data.control), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }
    }

    else if(p_Header->data.addressType == e_ADDRESS_16)
    {
        value16 = (uint16_t)p_Header->data.address;
        if (INTERNAL_SerializeValue((uint8_t*)(&value16), sizeof(p_Header->data.address), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
        INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.control), sizeof(p_Header->data.control), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
        {
            return E_NOT_IN_RANGE;
        }

    }

     /* Serialize the rest of the frame */
    if (INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize) != E_OK)
    {
        RETURN_ERROR(MAJOR, E_INVALID_STATE, (""));
    }

    /* Add the final CRC */
    if (p_Header->data.bUseFinalCkSum)
    {
        /* Get the size of the serialized frame */
        bufferSize -= *pSize;

        if(p_Header->data.finalCheckSumType == e_FCS_16)
        {
            /* Compute 16-bit FCS */
            finalCheckSum = INTERNAL_Compute16BitCRC(0xFFFF, pPrevBuffer, bufferSize);
            fcsSize = HDLC_16_BIT_FCS_SIZE;
            fcsOffset = HDLC_16_BIT_FCS_SIZE;
        }
        else
        {
            /* Compute the 32-bit FCS */
            finalCheckSum = INTERNAL_ComputeCRC(0xFFFFFFFF, pPrevBuffer, bufferSize);
            fcsSize = HDLC_32_BIT_FCS_SIZE;
        }

        /* Get the checksum - if set to auto*/
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_HDLC_FCS))
        {
            p_Header->data.finalCheckSum = finalCheckSum;
        }
        else
        {
            if(finalCheckSum != p_Header->data.finalCheckSum)
                 p_Header->errorFlag = e_FCS_ERROR;
        }

        /* Serialize the checksum */
        if (INTERNAL_SerializeValue(((uint8_t*)&p_Header->data.finalCheckSum) + fcsOffset, fcsSize, p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
        {
             return E_NOT_IN_RANGE;
        }
    }
    return E_OK;
}

uint32_t HDR_HDLC_GetHeaderSize(t_Handle h_Header)
{
    t_HDLC      *pHeaderStr = (t_HDLC*)((t_FrameStruct*)h_Header)->h_HeaderStruct; /* HDLC header handle */
    uint32_t    size;


    if(pHeaderStr->data.addressType == e_ADDRESS_8)
        size = HDLC_8Bit_ADDRESS_SIZE;
    else
        size = HDLC_16Bit_ADDRESS_SIZE;

    /* Account for the final 4 FCS length */
    size += (pHeaderStr->data.bUseFinalCkSum) ? ((pHeaderStr->data.finalCheckSumType == e_FCS_16) ? HDLC_16_BIT_FCS_SIZE : HDLC_32_BIT_FCS_SIZE) : 0;

    /* Return the HDLC header size */
    return size;
}

t_Error HDR_HDLC_Init(t_Handle h_Header)
{
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return E_OK;
}

t_Handle HDR_HDLC_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.hdlcHeader);
}

t_Error HDR_HDLC_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_HDLC *p_Header = NULL;       /* HDLC header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_HDLC*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of HDLC group */

/**************************************************************************//**
@Group          RTP

@Description    RTP functions

@{
*//***************************************************************************/
t_Handle HDR_RTP_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_RTP *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    /* Allocate memory for the RTP header */
    p_Header = (t_RTP*)MEMALLOC(sizeof(t_RTP));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_RTP));

    /* Set the auto flags */
    FLAG_SET(p_Header->autoFlags, e_FIELD_RTP_TIME_STAMP);

    /* Initialize the user configured RTP header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgRtpHeader));

    if(p_Header->data.padding && !(p_Header->data.validFields & e_FIELD_RTP_PAD_LENGTH))
    {
        REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Invalid RTP padding configuration"));
        return NULL;
    }

    return (t_Handle) p_Header;
}

t_Handle HDR_RTP_Clone(t_Handle h_Object)
{
    /* Get the handle to the RTP header */
    t_RTP* p_Object = (t_RTP*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_RTP* p_Header = (t_RTP*)MEMALLOC(sizeof(t_RTP));

    /* Check the handle */
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_RTP));

    /* Copy the header fields */
    memcpy(p_Header, p_Object, sizeof(t_RTP));

    return p_Header;
}

t_Error  HDR_RTP_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the RTP header */
    t_RTP *p_Header   = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */

    /* Check the handle */
    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Retrieve the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_RTP_VER:
        {
            value8 = (uint8_t)p_Header->data.version;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_RTP_PADDING:
        {
            value8 = (uint8_t)p_Header->data.padding;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_RTP_EXTENSION:
        {
            value8 = (uint8_t)p_Header->data.extension;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_RTP_CSRC_COUNT:
        {
            value8 = (uint8_t)p_Header->data.csrcCount;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_RTP_MARKER:
        {
            value8 = (uint8_t)p_Header->data.marker;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_RTP_PAYLOAD_TYPE:
        {
            value8 = (uint8_t)p_Header->data.payloadType;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_RTP_SEQ_NUM:
            SAFE_COPY_VALUE(&p_Header->data.seqNum, h_Value, sizeof(p_Header->data.seqNum), size);
        break;
        case e_FIELD_RTP_TIME_STAMP:
            SAFE_COPY_VALUE(&p_Header->data.timeStamp, h_Value, sizeof(p_Header->data.timeStamp), size);
        break;
        case e_FIELD_RTP_SSRC:
            SAFE_COPY_VALUE(&p_Header->data.ssrc, h_Value, sizeof(p_Header->data.ssrc), size);
        break;
        case e_FIELD_RTP_CSRC_LIST:
            SAFE_COPY_VALUE(p_Header->data.csrcList, h_Value, sizeof(p_Header->data.csrcList), size);
        break;
        case e_FIELD_RTP_EXTN_PROFILE:
            SAFE_COPY_VALUE(&p_Header->data.extnProfile, h_Value, sizeof(p_Header->data.extnProfile), size);
        break;
        case e_FIELD_RTP_EXTN_LENGTH:
            SAFE_COPY_VALUE(&p_Header->data.extnLength, h_Value, sizeof(p_Header->data.extnLength), size);
        break;
        case e_FIELD_RTP_EXTN_HEADER:
            SAFE_COPY_VALUE(p_Header->data.extnHeader, h_Value, sizeof(p_Header->data.extnHeader), size);
        break;
        case e_FIELD_RTP_PAD_LENGTH:
            SAFE_COPY_VALUE(&p_Header->data.padLength, h_Value, sizeof(p_Header->data.padLength), size);
        break;
        case e_FIELD_RTP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_RTP_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_RTP_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the RTP header */
    t_RTP* p_Header = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */

    /* Check the handle */
    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_RTP_VER:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.version = value8;
        }
        break;
        case e_FIELD_RTP_PADDING:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.padding = value8;
        }
        break;
        case e_FIELD_RTP_EXTENSION:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.extension = value8;
        }
        break;
        case e_FIELD_RTP_CSRC_COUNT:
        {
            if (*((uint8_t*)h_Value) > MAX_RTP_SRC_ID)
                *((uint8_t*)h_Value) = MAX_RTP_SRC_ID; /*not more than 15 SRC ids*/

            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.csrcCount = value8;
        }
        break;
        case e_FIELD_RTP_MARKER:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.marker = value8;
        }
        break;
        case e_FIELD_RTP_PAYLOAD_TYPE:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.payloadType = value8;
        }
        break;
        case e_FIELD_RTP_SEQ_NUM:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.seqNum, sizeof(p_Header->data.seqNum), size);
        break;
        case e_FIELD_RTP_TIME_STAMP:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.timeStamp, sizeof(p_Header->data.timeStamp), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }break;
        case e_FIELD_RTP_SSRC:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.ssrc, sizeof(p_Header->data.ssrc), size);
        break;
        case e_FIELD_RTP_CSRC_LIST:
            SAFE_COPY_VALUE(h_Value, p_Header->data.csrcList, sizeof(p_Header->data.csrcList), size);
        break;
        case e_FIELD_RTP_EXTN_PROFILE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.extnProfile, sizeof(p_Header->data.extnProfile), size);
        break;
        case e_FIELD_RTP_EXTN_LENGTH:
        {
            if (*((uint8_t*)h_Value) > MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER)
                *((uint8_t*)h_Value) = MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER;

            SAFE_COPY_VALUE(h_Value, &p_Header->data.extnLength, sizeof(p_Header->data.extnLength), size);
        }
        break;
        case e_FIELD_RTP_EXTN_HEADER:
            SAFE_COPY_VALUE(h_Value, p_Header->data.extnHeader, sizeof(p_Header->data.extnHeader), size);
        break;
        case e_FIELD_RTP_PAD_LENGTH:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.padLength, sizeof(p_Header->data.padLength), size);
        break;
        case e_FIELD_RTP_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_RTP_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_RTP_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    /* Get the handle to the RTP header */
    t_RTP* p_Header = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Check the handle */
    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_RTP_TIME_STAMP:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    /* Set or clear the auto flags based on the user configuration */
    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_RTP_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    /* Get the handle to the RTP header */
    t_RTP* p_Header = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgRtpHeader* pValues = (t_PgRtpHeader*)hValues;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */

    /* Check the handle */
    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_RTP_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the RTP header based on the field ID */
    if (fieldIDs & e_FIELD_RTP_VER)
    {
        value8 = (uint8_t)pValues->version;
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_VER, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_RTP_PADDING)
    {
        value8 = (uint8_t)pValues->padding;
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_PADDING, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_RTP_EXTENSION)
    {
        value8 = (uint8_t)pValues->extension;
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_EXTENSION, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_RTP_CSRC_COUNT)
    {
        value8 = (uint8_t)pValues->csrcCount;
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_CSRC_COUNT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_RTP_MARKER)
    {
        value8 = (uint8_t)pValues->marker;
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_MARKER, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_RTP_PAYLOAD_TYPE)
    {
        value8 = (uint8_t)pValues->payloadType;
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_PAYLOAD_TYPE, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_RTP_SEQ_NUM)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_SEQ_NUM, (t_Handle)&pValues->seqNum, sizeof(pValues->seqNum));

    if (fieldIDs & e_FIELD_RTP_TIME_STAMP)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_TIME_STAMP, (t_Handle)&pValues->timeStamp, sizeof(pValues->timeStamp));

    if (fieldIDs & e_FIELD_RTP_SSRC)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_SSRC, (t_Handle)&pValues->ssrc, sizeof(pValues->ssrc));

    if (fieldIDs & e_FIELD_RTP_CSRC_LIST)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_CSRC_LIST, (t_Handle)&pValues->csrcList, sizeof(pValues->csrcList));

    if (fieldIDs & e_FIELD_RTP_EXTN_PROFILE)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_EXTN_PROFILE, (t_Handle)&pValues->extnProfile, sizeof(pValues->extnProfile));

    if (fieldIDs & e_FIELD_RTP_EXTN_LENGTH)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_EXTN_LENGTH, (t_Handle)&pValues->extnLength, sizeof(pValues->extnLength));

    if (fieldIDs & e_FIELD_RTP_EXTN_HEADER)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_EXTN_HEADER, (t_Handle)&pValues->extnHeader, sizeof(pValues->extnHeader));

    if (fieldIDs & e_FIELD_RTP_PAD_LENGTH)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_PAD_LENGTH, (t_Handle)&pValues->padLength, sizeof(pValues->padLength));

    if (fieldIDs & e_FIELD_RTP_NEXT_HEADER_TYPE)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_RTP_VALID_FIELDS)
        HDR_RTP_ConfigField(h_Header, e_FIELD_RTP_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));


    return E_OK;
}

t_Error HDR_RTP_Randomize(t_Handle h_Header)
{
    /* Get the handle to the RTP header */
    t_RTP* pHeaderStr = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t index = 0;

    /* Generate random values based for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_VER))
        pHeaderStr->data.version = 2;

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_PADDING))
        pHeaderStr->data.padding = 0;

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_EXTENSION))
        pHeaderStr->data.extension = 0;

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_CSRC_COUNT))
        pHeaderStr->data.csrcCount = 0;

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_MARKER))
        pHeaderStr->data.marker = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_PAYLOAD_TYPE))
        pHeaderStr->data.payloadType = ((uint8_t)rand() % MAX_RTP_PAYLOAD_TYPES);

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_SEQ_NUM))
        pHeaderStr->data.seqNum = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_TIME_STAMP))
        pHeaderStr->data.timeStamp = (uint32_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_RTP_TIME_STAMP);

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_SSRC))
        pHeaderStr->data.ssrc = (uint32_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_CSRC_LIST))
    {
        for (index=0; index<MAX_RTP_SRC_ID; index++)
            pHeaderStr->data.csrcList[index] = (uint32_t)rand();
    }

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_EXTN_PROFILE))
        pHeaderStr->data.extnProfile = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_EXTN_LENGTH))
        pHeaderStr->data.extnLength = 0;

    if (!(pHeaderStr->data.validFields & e_FIELD_RTP_EXTN_HEADER))
    {
        for (index = 0;index < MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER;index++)
            pHeaderStr->data.extnHeader[index] = (uint32_t)rand();
    }

    return E_OK;
}

t_Error HDR_RTP_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
     /* Get the handle to the RTP header */
     t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
     t_RTP* p_Header = NULL;
     uint8_t padValue = 0;
     uint8_t padLength = 0;

     /* Check the handle */
     if (!(h_Header && p_Buffer && pSize))
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     p_Header = (t_RTP*)p_Frame->h_HeaderStruct;

     /* Serialize the values */
     if (INTERNAL_SerializeValue((uint8_t*)&p_Header->data, FIXED_HEADER_SIZE_RTP, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
     {
         return E_NOT_IN_RANGE;
     }

     /* Serialize the csrc list */
     if (p_Header->data.csrcCount)
     {
         /* Store the csrc list */
         if (INTERNAL_SerializeValue((uint8_t*)(p_Header->data.csrcList), (p_Header->data.csrcCount * sizeof(uint32_t)), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
             return E_NOT_IN_RANGE;
     }

     if(p_Header->data.extension)
     {
         /* Store the extension header */
         if (INTERNAL_SerializeValue((uint8_t*)&p_Header->data.extnProfile, sizeof(p_Header->data.extnProfile), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
         INTERNAL_SerializeValue((uint8_t*)&p_Header->data.extnLength, sizeof(p_Header->data.extnLength), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
         INTERNAL_SerializeValue((uint8_t*)p_Header->data.extnHeader, (p_Header->data.extnLength * sizeof(uint32_t)), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
             return E_NOT_IN_RANGE;
     }

     if(p_Header->data.version != RTP_VERSION)
         p_Header->errorFlag = e_RTP_VERSION_ERROR;

     /* Serialize the rest of the frame */
     if (INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize) != E_OK)
     {
         RETURN_ERROR(MAJOR, E_INVALID_STATE, (""));
     }

     /* Pad the RTP frame if needed */
     if (p_Header->data.padding)
     {
         padLength = p_Header->data.padLength;
         /* Add padding */
         while (padLength-- > 0)
         {
             if(!padLength)
                 padValue = p_Header->data.padLength;

             if (INTERNAL_SerializeValue((uint8_t*)(&padValue), sizeof(padValue), p_Buffer, pSize, CONVERT_ENDIANNESS) != E_OK)
             {
                 return E_NOT_IN_RANGE;
             }
         }
     }
     return E_OK;
}

uint32_t HDR_RTP_GetHeaderSize(t_Handle h_Header)
{
    t_RTP* pHeaderStr = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t csrcCount = 0;
    uint32_t extnLength = 0;

    csrcCount = pHeaderStr->data.csrcCount;

    /* Redundant, but adds extra safety (csrc count should not be greater than MAX_RTP_SRC_ID)*/
    if (csrcCount > MAX_RTP_SRC_ID)
        csrcCount = MAX_RTP_SRC_ID;

    if(pHeaderStr->data.extension)
    {
        if(pHeaderStr->data.extnLength > MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER)
            pHeaderStr->data.extnLength = MAX_32BIT_WORDS_IN_RTP_EXTN_HEADER;
        extnLength = RTP_EXTN_HEADER_IDENTIFIER_SIZE + (pHeaderStr->data.extnLength * sizeof(uint32_t));
    }

    /* Return the RTP header size */
    /* Size of each CSRC is 4 bytes */
    return (uint32_t)(FIXED_HEADER_SIZE_RTP + (csrcCount * sizeof(uint32_t)) + extnLength + pHeaderStr->data.padLength);
}

t_Error HDR_RTP_Init(t_Handle h_Header)
{
    t_RTP* p_Header = NULL;

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the handle to the RTP header */
    p_Header = (t_RTP*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Initialize the time stamp if the auto flag is set */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_RTP_TIME_STAMP))
    {
         p_Header->data.timeStamp = 0; //Check
    }

    return E_OK;
}

t_Handle HDR_RTP_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l5Header.rtpHeader);
}

t_Error HDR_RTP_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_RTP *p_Header = NULL;        /* RTP header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_RTP*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of RTP group */

/**************************************************************************//**
@Group          ML/MC

@Description    ML/MC functions

@{
*//***************************************************************************/
t_Handle HDR_PPPMLMC_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_PPPMLMC *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    /* Allocate memory for the ML/MC header */
    p_Header = (t_PPPMLMC*)MEMALLOC(sizeof(t_PPPMLMC));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_PPPMLMC));

    /* Set the auto flags */
    FLAG_SET(p_Header->autoFlags, e_FIELD_PPP_MLMC_PID);

    /* Initialize the user configured ML/MC header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgPppMlmcHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_PPPMLMC_Clone(t_Handle h_Object)
{
    /* Get the handle to the ML/MC header */
    t_PPPMLMC* p_Object = (t_PPPMLMC*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_PPPMLMC* p_Header = (t_PPPMLMC*)MEMALLOC(sizeof(t_PPPMLMC));

    /* Check the handle */
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_PPPMLMC));

    /* Copy the header fields */
    memcpy(p_Header, p_Object, sizeof(t_PPPMLMC));

    return p_Header;
}

t_Error  HDR_PPPMLMC_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the ML/MC header */
    t_PPPMLMC *p_Header   = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */
    uint32_t value32 = 0;   /* Holds the bit fields of size between 16 and 32 bits */

    /* Check the handle */
    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Retrieve the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT:
        {
            value8 = (uint8_t)p_Header->data.beginFragmentBit;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_PPP_MLMC_END_FRAG_BIT:
        {
            value8 = (uint8_t)p_Header->data.endFragmentBit;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_PPP_MLMC_CLASS:
        {
            value8 = (uint8_t)p_Header->data.classId;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_PPP_MLMC_SEQ_NUM:
        {
            value32 = (uint32_t)p_Header->data.seqNum;
            SAFE_COPY_VALUE(&value32, h_Value, sizeof(value32), size);
        }
        break;
        case e_FIELD_PPP_MLMC_SEQ_NUM_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.seqNumType, h_Value, sizeof(p_Header->data.seqNumType), size);
        break;
        case e_FIELD_PPP_MLMC_PID:
            SAFE_COPY_VALUE(&p_Header->data.pid, h_Value, sizeof(p_Header->data.pid), size);
        break;
        case e_FIELD_PPP_MLMC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_PPP_MLMC_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PPPMLMC_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the ML/MC header */
    t_PPPMLMC* p_Header = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */
    uint32_t value32 = 0;   /* Holds the bit fields of size between 16 and 32 bits */

    /* Check the handle */
    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.beginFragmentBit = value8;
        }
        break;
        case e_FIELD_PPP_MLMC_END_FRAG_BIT:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.endFragmentBit = value8;
        }
        break;
        case e_FIELD_PPP_MLMC_CLASS:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.classId = value8;
        }
        break;
        case e_FIELD_PPP_MLMC_SEQ_NUM:
        {
            SAFE_COPY_VALUE(h_Value, &value32, sizeof(value32), size);
            p_Header->data.seqNum = value32;
        }
        break;
        case e_FIELD_PPP_MLMC_PID:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.pid, sizeof(p_Header->data.pid), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }
        break;
        case e_FIELD_PPP_MLMC_SEQ_NUM_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.seqNumType, sizeof(p_Header->data.seqNumType), size);
        break;
        case e_FIELD_PPP_MLMC_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_PPP_MLMC_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_PPPMLMC_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    /* Get the handle to the ML/MC header */
    t_PPPMLMC* p_Header = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Check the handle */
    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_PPP_MLMC_PID:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    /* Set or clear the auto flag based on the user input */
    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;

}

t_Error HDR_PPPMLMC_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    /* Get the handle to the ML/MC header */
   t_PPPMLMC* p_Header = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
   t_PgPppMlmcHeader* pValues = (t_PgPppMlmcHeader*)hValues;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */
    uint32_t value32 = 0;   /* Holds the bit fields of size between 16 and 32 bits */

   /* Check the handle */
   if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_PPP_MLMC_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the ML/MC header based on the field ID */
    if (fieldIDs & e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT)
    {
        value8 = (uint8_t)pValues->beginFragmentBit;
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PPP_MLMC_END_FRAG_BIT)
    {
        value8 = (uint8_t)pValues->endFragmentBit;
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_END_FRAG_BIT, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PPP_MLMC_CLASS)
    {
        value8 = (uint8_t)pValues->classId;
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_CLASS, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_PPP_MLMC_SEQ_NUM)
    {
        value32 = (uint32_t)pValues->seqNum;
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_SEQ_NUM, (t_Handle)&value32, sizeof(value32));
    }

    if (fieldIDs & e_FIELD_PPP_MLMC_SEQ_NUM_TYPE)
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_SEQ_NUM_TYPE, (t_Handle)&pValues->seqNumType, sizeof(pValues->seqNumType));

    if (fieldIDs & e_FIELD_PPP_MLMC_PID)
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_PID, (t_Handle)&pValues->pid, sizeof(pValues->pid));

    if (fieldIDs & e_FIELD_PPP_MLMC_NEXT_HEADER_TYPE)
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_PPP_MLMC_VALID_FIELDS)
        HDR_PPPMLMC_ConfigField(h_Header, e_FIELD_PPP_MLMC_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error HDR_PPPMLMC_Randomize(t_Handle h_Header)
{
    /* Get the handle to the ML/MC header */
    t_PPPMLMC* pHeaderStr = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_MLMC_BEGIN_FRAG_BIT))
        pHeaderStr->data.beginFragmentBit = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_MLMC_END_FRAG_BIT))
        pHeaderStr->data.endFragmentBit = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_MLMC_SEQ_NUM_TYPE))
        pHeaderStr->data.seqNumType = (e_PgMlmcSeqNumType)((uint8_t)rand() % 2);

    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_MLMC_CLASS))
    {
        if(pHeaderStr->data.seqNumType == e_SEQ_NUM_24_BIT)
            pHeaderStr->data.classId = ((uint8_t)rand() & MASK_MLMC_4_BIT_CLASS_ID);
        else if(pHeaderStr->data.seqNumType == e_SEQ_NUM_12_BIT)
            pHeaderStr->data.classId = ((uint8_t)rand() & MASK_MLMC_2_BIT_CLASS_ID);
    }

    if (!(pHeaderStr->data.validFields & e_FIELD_PPP_MLMC_SEQ_NUM))
    {
        if(pHeaderStr->data.seqNumType == e_SEQ_NUM_24_BIT)
            pHeaderStr->data.seqNum = ((uint32_t)rand() & MASK_MLMC_24_BIT_SEQ_NUM);
        else if(pHeaderStr->data.seqNumType == e_SEQ_NUM_12_BIT)
            pHeaderStr->data.seqNum = ((uint32_t)rand() & MASK_MLMC_12_BIT_SEQ_NUM);
    }

    if(!(pHeaderStr->data.validFields & e_FIELD_PPP_MLMC_PID))
        pHeaderStr->data.pid = (uint16_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_PPP_MLMC_PID);

    return E_OK;
}

t_Error HDR_PPPMLMC_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    /* Get the handle to the ML/MC header */
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_PPPMLMC* p_Header = NULL;
    uint32_t value32 = 0;
    uint16_t value16 = 0;

    /* Check the handle */
    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_PPPMLMC*)p_Frame->h_HeaderStruct;

    /* Store the begin fragment bit, end fragment bit, sequence number and class id in the 32/16 bit value */
    value32 = (uint32_t)(p_Header->data.beginFragmentBit);
    value32 = ((value32 << OFFSET_MLMC_END_FRAG_BIT) | p_Header->data.endFragmentBit);
    if(p_Header->data.seqNumType == e_SEQ_NUM_24_BIT)
    {
        value32 = ((value32 << OFFSET_MLMC_4_BIT_CLASS_ID) | p_Header->data.classId);
        value32 = ((value32 << OFFSET_MLMC_RESERVED) & (SET_MLMC_RESERVED_2_BIT));
        value32 = ((value32 << OFFSET_MLMC_24_BIT_SEQ_NUM) | p_Header->data.seqNum);
    }
    else if(p_Header->data.seqNumType == e_SEQ_NUM_12_BIT)
    {
        value16 = (uint16_t)value32;
        value16 = (uint16_t)((value16 << OFFSET_MLMC_2_BIT_CLASS_ID) | (p_Header->data.classId & MASK_MLMC_2_BIT_CLASS_ID));
        value16 = (uint16_t)((value16 << OFFSET_MLMC_12_BIT_SEQ_NUM) | (p_Header->data.seqNum & MASK_MLMC_12_BIT_SEQ_NUM));
    }

    /* Serialize the values */
    if (INTERNAL_SerializeValue((p_Header->data.seqNumType == e_SEQ_NUM_24_BIT ? (uint8_t*)&value32 : (uint8_t*)&value16), (p_Header->data.seqNumType == e_SEQ_NUM_24_BIT ? sizeof(value32) : sizeof(value16)), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK ||
    INTERNAL_SerializeValue((uint8_t*)(&p_Header->data.pid), sizeof(p_Header->data.pid), p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_PPPMLMC_GetHeaderSize(t_Handle h_Header)
{
    t_PPPMLMC* pHeaderStr = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint32_t headerSize = 0;

    /* Return the ML/MC header size */
    if(pHeaderStr->data.seqNumType == e_SEQ_NUM_24_BIT)
        headerSize = FIXED_HEADER_SIZE_PPP_MLMC_RFC1990;
    else if(pHeaderStr->data.seqNumType == e_SEQ_NUM_12_BIT)
        headerSize = FIXED_HEADER_SIZE_PPP_MLMC_RFC2686;

    return headerSize;
}

t_Error HDR_PPPMLMC_Init(t_Handle h_Header)
{
    t_PPPMLMC* p_Header = NULL;
    t_FrameStruct *p_NextHeader = NULL;

    /* Check the handle */
    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Get the handle to the header */
    p_Header = (t_PPPMLMC*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Get the next header */
    p_NextHeader = INTERNAL_GetNextHeader(h_Header);

    if (p_NextHeader)
    {
        /* Initialize the PID if the auto flag is set */
        if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_PPP_MLMC_PID))
        {
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv6)
                p_Header->data.pid = 0x0057;
            if (p_NextHeader->HeaderID == e_HEADER_TYPE_IPv4)
                p_Header->data.pid = 0x0021;
        }
    }

    return E_OK;
}

t_Handle HDR_PPPMLMC_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.pppMlmcHeader);
}

t_Error HDR_PPPMLMC_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    UNUSED(hHeader);
    UNUSED(hError);

    return E_OK;
}

/** @} */ /* end of ML/MC group */

/**************************************************************************//**
@Group          AAL0

@Description    AAL0 functions

@{
*//***************************************************************************/
t_Handle HDR_AAL0Cell_Config(t_Handle hHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_AAL0Cell *p_Header = NULL;
    UNUSED(p_ProtocolInfo);

    /* Check the header handle */
    if (!hHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid header handle"));
        return NULL;
    }

    /* Allocate memory for the AAL0 cell header */
    p_Header = (t_AAL0Cell*)MEMALLOC(sizeof(t_AAL0Cell));
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }
    memset(p_Header, 0, sizeof(t_AAL0Cell));

    /* Set the auto flags */
    FLAG_SET(p_Header->autoFlags, e_FIELD_AAL0_CELL_HEC);

    /* Initialize the user configured AAL0 cell header fields in the header */
    memcpy(&p_Header->data, hHeader, sizeof(t_PgAal0CellHeader));

    return (t_Handle) p_Header;
}

t_Handle HDR_AAL0Cell_Clone(t_Handle h_Object)
{
    /* Get the handle to the AAL0 cell header */
    t_AAL0Cell* p_Object = (t_AAL0Cell*)((t_FrameStruct*)h_Object)->h_HeaderStruct;
    t_AAL0Cell* p_Header = (t_AAL0Cell*)MEMALLOC(sizeof(t_AAL0Cell));

    /* Check the handle */
    if (!p_Header)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Header, 0, sizeof(t_AAL0Cell));

    /* Copy the header fields */
    memcpy(p_Header, p_Object, sizeof(t_AAL0Cell));

    return p_Header;
}

t_Error  HDR_AAL0Cell_GetField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the AAL0 cell header */
    t_AAL0Cell *p_Header   = (t_AAL0Cell*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;   /* Holds the bit fields of size between 8 and 16 bits */

    /* Check the handle */
    if (!(p_Header && h_Value && size))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* Retrieve the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_AAL0_CELL_GFC:
        {
            value8 = (uint8_t)p_Header->data.gfc;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_AAL0_CELL_VPI:
        {
            value8 = (uint8_t)p_Header->data.vpi;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_AAL0_CELL_VCI:
        {
            value16 = (uint16_t)p_Header->data.vci;
            SAFE_COPY_VALUE(&value16, h_Value, sizeof(value16), size);
        }
        break;
        case e_FIELD_AAL0_CELL_PTI:
        {
            value8 = (uint8_t)p_Header->data.pti;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_AAL0_CELL_CLP:
        {
            value8 = (uint8_t)p_Header->data.clp;
            SAFE_COPY_VALUE(&value8, h_Value, sizeof(value8), size);
        }
        break;
        case e_FIELD_AAL0_CELL_HEC:
            SAFE_COPY_VALUE(&p_Header->data.hec, h_Value, sizeof(p_Header->data.hec), size);
        break;
        case e_FIELD_AAL0_CELL_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(&p_Header->data.nextHeaderType, h_Value, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_AAL0_VALID_FIELDS:
            SAFE_COPY_VALUE(&p_Header->data.validFields, h_Value, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
              RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_AAL0Cell_ConfigField(t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    /* Get the handle to the AAL0 cell header */
    t_AAL0Cell* p_Header = (t_AAL0Cell*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;   /* Holds the bit fields of size between 8 and 16 bits */

    /* Check the handle */
    if (!p_Header || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    size = (size) ? size : (uint32_t)(-1);

    /* Configure the field specified by the user */
    switch (fieldID)
    {
        case e_FIELD_AAL0_CELL_GFC:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.gfc = value8;
        }
        break;
        case e_FIELD_AAL0_CELL_VPI:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.vpi = value8;
        }
        break;
        case e_FIELD_AAL0_CELL_VCI:
        {
            SAFE_COPY_VALUE(h_Value, &value16, sizeof(value16), size);
            p_Header->data.vci = value16;
        }
        break;
        case e_FIELD_AAL0_CELL_PTI:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.pti = value8;
        }
        break;
        case e_FIELD_AAL0_CELL_CLP:
        {
            SAFE_COPY_VALUE(h_Value, &value8, sizeof(value8), size);
            p_Header->data.clp = value8;
        }
        break;
        case e_FIELD_AAL0_CELL_HEC:
        {
            SAFE_COPY_VALUE(h_Value, &p_Header->data.hec, sizeof(p_Header->data.hec), size);
            FLAG_CLEAR(p_Header->autoFlags, fieldID);
        }break;
        case e_FIELD_AAL0_CELL_NEXT_HEADER_TYPE:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.nextHeaderType, sizeof(p_Header->data.nextHeaderType), size);
        break;
        case e_FIELD_AAL0_VALID_FIELDS:
            SAFE_COPY_VALUE(h_Value, &p_Header->data.validFields, sizeof(p_Header->data.validFields), size);
        break;

        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
        }
    }

    return E_OK;
}

t_Error HDR_AAL0Cell_ConfigAuto(t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    /* Get the handle to the AAL0 cell header */
    t_AAL0Cell* p_Header = (t_AAL0Cell*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Check the handle */
    if (!p_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    switch (fieldID)
    {
        case e_FIELD_AAL0_CELL_HEC:
        break;
        default:
        {
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("The field is not dynamic"));
        }break;
    }

    /* Set or clear the auto flags based on the user input */
    if (bSetAuto)
        FLAG_SET(p_Header->autoFlags, fieldID);
    else
        FLAG_CLEAR(p_Header->autoFlags, fieldID);

    return E_OK;
}

t_Error HDR_AAL0Cell_ConfigHeader(t_Handle h_Header, t_Handle hValues, uint32_t fieldIDs)
{
    /* Get the handle to the AAL0 cell header */
    t_AAL0Cell* p_Header = (t_AAL0Cell*)((t_FrameStruct*)h_Header)->h_HeaderStruct;
    t_PgAal0CellHeader* pValues = (t_PgAal0CellHeader*)hValues;
    uint8_t value8 = 0;     /* Holds the bit fields of size less than 8 bits */
    uint16_t value16 = 0;   /* Holds the bit fields of size between 8 and 16 bits */

    /* Check the handle */
    if (!p_Header || !pValues)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    if (!(fieldIDs & e_FIELD_AAL0_CELL_ALL_FIELDS))
    {
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid field types provided!"));
    }

    /* Configure the AAL0 cell header based on the field ID */
    if (fieldIDs & e_FIELD_AAL0_CELL_GFC)
    {
        value8 = (uint8_t)pValues->gfc;
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_GFC, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_AAL0_CELL_VPI)
    {
        value8 = (uint8_t)pValues->vpi;
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_VPI, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_AAL0_CELL_VCI)
    {
        value16 = (uint16_t)pValues->vci;
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_VCI, (t_Handle)&value16, sizeof(value16));
    }

    if (fieldIDs & e_FIELD_AAL0_CELL_PTI)
    {
        value8 = (uint8_t)pValues->pti;
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_PTI, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_AAL0_CELL_CLP)
    {
        value8 = (uint8_t)pValues->clp;
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_CLP, (t_Handle)&value8, sizeof(value8));
    }

    if (fieldIDs & e_FIELD_AAL0_CELL_HEC)
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_HEC, (t_Handle)&pValues->hec, sizeof(pValues->hec));

    if (fieldIDs & e_FIELD_AAL0_CELL_NEXT_HEADER_TYPE)
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_CELL_NEXT_HEADER_TYPE, (t_Handle)&pValues->nextHeaderType, sizeof(pValues->nextHeaderType));

    if (fieldIDs & e_FIELD_AAL0_VALID_FIELDS)
        HDR_AAL0Cell_ConfigField(h_Header, e_FIELD_AAL0_VALID_FIELDS, (t_Handle)&pValues->validFields, sizeof(pValues->validFields));

    return E_OK;
}

t_Error HDR_AAL0Cell_Randomize(t_Handle h_Header)
{
    /* Get the handle to the AAL0 cell header */
    t_AAL0Cell* pHeaderStr = (t_AAL0Cell*)((t_FrameStruct*)h_Header)->h_HeaderStruct;

    /* Generate random values for the fields not configured by the user */
    if (!(pHeaderStr->data.validFields & e_FIELD_AAL0_CELL_GFC))
        pHeaderStr->data.gfc = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_AAL0_CELL_VPI))
        pHeaderStr->data.vpi = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_AAL0_CELL_VCI))
        pHeaderStr->data.vci = (uint16_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_AAL0_CELL_PTI))
        pHeaderStr->data.pti = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_AAL0_CELL_CLP))
        pHeaderStr->data.clp = (uint8_t)rand();

    if (!(pHeaderStr->data.validFields & e_FIELD_AAL0_CELL_HEC))
        pHeaderStr->data.hec = (uint8_t)rand();
    else
        FLAG_CLEAR(pHeaderStr->autoFlags, e_FIELD_AAL0_CELL_HEC);

    return E_OK;
}

t_Error HDR_AAL0Cell_Serialize(t_Handle h_Header, uint8_t **p_Buffer, uint32_t *pSize)
{
    /* Get the handle to the AAL0 cell header */
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_AAL0Cell* p_Header = NULL;
    uint8_t hec = 0;

    /* Check the handle */
    if (!(h_Header && p_Buffer && pSize))
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_AAL0Cell*)p_Frame->h_HeaderStruct;

    hec = INTERNAL_ComputeHEC((uint8_t*)&p_Header->data, sizeof(uint32_t));

    /* Initialize the HEC if the auto flag is set */
    if (FLAG_CHECK(p_Header->autoFlags, e_FIELD_AAL0_CELL_HEC))
    {
        p_Header->data.hec = hec;
    }
    else
    {
        if(hec != p_Header->data.hec)
            p_Header->errorFlag = e_AAL0_HEC_ERROR;
    }

    /* Serialize the values */
    if (INTERNAL_SerializeValue((uint8_t*)&p_Header->data, FIXED_HEADER_SIZE_ATM, p_Buffer, pSize, KEEP_ENDIANNESS) != E_OK)
    {
        return E_NOT_IN_RANGE;
    }

    /* Serialize the rest of the frame */
    return INTERNAL_SerializeNextHeader(h_Header, p_Buffer, pSize);
}

uint32_t HDR_AAL0Cell_GetHeaderSize(t_Handle h_Header)
{
    UNUSED(h_Header);

    /* Return the AAL0 cell header size */
    return (FIXED_HEADER_SIZE_ATM);
}

t_Error HDR_AAL0Cell_Init(t_Handle h_Header)
{
     if (!h_Header)
     {
         RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     return E_OK;
}

t_Handle HDR_AAL0Cell_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->l2Header.aal0CellHeader);
}

t_Error HDR_AAL0Cell_GetHeaderError(t_Handle hHeader, t_Handle hError)
{
    t_AAL0Cell *p_Header = NULL;       /* AAL0 cell header handle */

    if (!hHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    p_Header = (t_AAL0Cell*)((t_FrameStruct*)hHeader)->h_HeaderStruct;

    *((uint32_t*)hError) = p_Header->errorFlag;

    return E_OK;
}

/** @} */ /* end of AAL0 group */

/**************************************************************************//**
@Group          Tunnelled IP

@Description    Tunnelled IP functions

@{
*//***************************************************************************/

t_Handle HDR_TunnelledIPv4_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->tunnelledL3Header.ipv4Header);
}

t_Handle HDR_TunnelledIPv6_GetHeader(t_Handle hFrame)
{
    /* Get the frame handle */
    t_PgFrameSetting *p_Frame = (t_PgFrameSetting*)hFrame;
    /* Check the frame handle */
    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid frame handle"));
        return NULL;
    }

    return (t_Handle)(&p_Frame->tunnelledL3Header.ipv6Header);
}

/** @} */ /* end of Tunnelled IP group */
/** @} */ /* end of Header group */
/** @} */ /* end of InternalFrame group */

/**************************************************************************//**
@Group          ExternalFrame

@Description    External frame functions

@{
*//***************************************************************************/
t_Handle FRAME_Config(t_Handle h_Params)
{
    t_FrameStruct* p_Frame = NULL;

    /* init the frame factory if not already inited*/
    if (framesCounter <= 0)
        INTERNAL_StartFrameFactory();
    framesCounter++;

    p_Frame = (t_FrameStruct*)MEMALLOC(sizeof(t_FrameStruct));
    UNUSED(h_Params);

    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
        return NULL;
    }

    memset(p_Frame, 0, sizeof(t_FrameStruct));
    p_Frame->HeaderID = e_HEADER_TYPE_NONE;
    p_Frame->h_HeaderStruct = NULL;
    p_Frame->node.p_Next = NULL;
    p_Frame->node.p_Prev = NULL;

    return (t_Handle)p_Frame;
}

t_Error FRAME_Free(t_Handle h_Frame)
{
    /* cleanup the frame factory if this is the last frame object*/
    if (--framesCounter <= 0)
        INTERNAL_StopFrameFactory();

    /* there should not be a negative number of frames */
    ASSERT_COND(framesCounter >= 0);

    return INTERNAL_Frame_Free(h_Frame, RECURSIVE);
}

t_Error FRAME_Free_Header(t_Handle h_Frame)
{
    return INTERNAL_Frame_Free(h_Frame, NON_RECURSIVE);
}

t_Error FRAME_ConfigRemoveHeader(t_Handle h_Frame, t_Handle h_Header)
{
    t_FrameStruct* p_Header = (t_FrameStruct*)h_Header;
    t_FrameStruct* p_Frame  = (t_FrameStruct*)h_Frame;
    t_FrameStruct* p_NextHeader = NULL;
   t_FrameStruct* p_NextNext = NULL;

    if( p_Header != p_Frame)
    {

        if (INTERNAL_RemoveNode(&p_Header->node) != E_OK)
        {
            RETURN_ERROR(MAJOR, E_INVALID_STATE, (""));
        }

        return INTERNAL_Frame_Free(h_Header, NON_RECURSIVE);
    }
    else
    {
           //get next header
           p_NextHeader = INTERNAL_GetNextHeader(h_Header);
           p_NextNext = p_NextHeader ? INTERNAL_GetNextHeader(p_NextHeader) : NULL;

        /* delete the current header/frame */
        if (p_Frame->h_HeaderStruct)
        {
            MEMFREE(p_Frame->h_HeaderStruct);
        }

        p_Frame->HeaderID        = e_HEADER_TYPE_NONE;
        p_Frame->h_HeaderStruct   = NULL;
        p_Frame->node.p_Next     = NULL;
        p_Frame->node.p_Prev     = NULL;

          //copy the next header at the front
        if(p_NextHeader)
        {
            *p_Frame             = *p_NextHeader;
            p_Frame->node.p_Prev = NULL;

            //remove next header "original copy" from the frame&list
            //if(INTERNAL_Frame_Free(p_NextHeader, NON_RECURSIVE) != E_OK)
            //{
            //    RETURN_ERROR(MAJOR, E_INVALID_STATE, (""));
            //}
            MEMFREE(p_NextHeader);
            p_NextHeader->HeaderID        = e_HEADER_TYPE_NONE;
           p_NextHeader->h_HeaderStruct   = NULL;
           p_NextHeader->node.p_Next     = NULL;
           p_NextHeader->node.p_Prev     = NULL;

            /* link back over the removed header */
            if(p_NextNext != NULL)
            {
                p_NextNext->node.p_Prev = &p_Frame->node;
            }
        }
        return E_OK;
    }

    return E_NOT_AVAILABLE;//it will never happen if you're lucky ;)
}

t_Error FRAME_Init(t_Handle h_Frame)
{
    t_FrameStruct* p_Frame = (t_FrameStruct*)h_Frame;
    t_Handle *p_Next = NULL;
    t_Error errCode = E_OK;

    if (!h_Frame)
    {
        RETURN_ERROR(MAJOR, E_INVALID_FRAME, (""));
    }

    /* initialize the header chain recursively */
    if (pRegisteredHeaderArray[p_Frame->HeaderID].pfnInit)
        errCode = pRegisteredHeaderArray[p_Frame->HeaderID].pfnInit(h_Frame);

    if (errCode != E_OK)
    {
        RETURN_ERROR(MAJOR, E_INVALID_STATE, (""));
    }

    p_Next = INTERNAL_GetNextHeader(p_Frame);

    return (p_Next) ? FRAME_Init(p_Next) : E_OK;
}

t_Handle FRAME_ConfigAddHeader(t_Handle h_Frame, e_PgHeaderType headerID, t_Handle h_NewHeader)
{
    t_FrameStruct* p_Frame = (t_FrameStruct*)h_Frame;
    t_FrameStruct* p_NewFrame = NULL;

    if (!p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_FRAME, (""));
    }

    if ( (headerID <= e_HEADER_TYPE_NONE) || (headerID >= e_MAX_HEADER_TYPE_COUNT) )
    {
         REPORT_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid frame type provided!"));
         return NULL;
    }

    /* Check if the frame is empty */
    if (p_Frame->HeaderID == e_HEADER_TYPE_NONE)
    {
        /*Init the frame (the first header becomes the frame)*/
        p_NewFrame = p_Frame;
    }
    else
    {
       /* Create the new header (frame) */
        p_NewFrame = (t_FrameStruct*)MEMALLOC(sizeof(t_FrameStruct));

        if (!p_NewFrame)
        {
            REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
            return NULL;
        }

        memset(p_NewFrame, 0, sizeof(t_FrameStruct));
        p_NewFrame->node.p_Next = NULL;
        p_NewFrame->node.p_Prev = NULL;

        /* Add the new header (frame) at the end of the list */
       INTERNAL_AddToTail(&p_Frame->node, &p_NewFrame->node);
    }

    /* Set the proper values */
    p_NewFrame->HeaderID = headerID;
    p_NewFrame->h_HeaderStruct = pRegisteredHeaderArray[headerID].pfnConfig(h_NewHeader,NULL);

    /* trigger the header specific randomization method */
    if (pRegisteredHeaderArray[headerID].pfnRandomizeHeader)
            pRegisteredHeaderArray[headerID].pfnRandomizeHeader(p_NewFrame);

    //INTERNAL_DebugPrintChain(&p_Frame->node);

   /* Return the new frame (header) */
    return p_NewFrame;
}

t_Handle FRAME_Configure(t_Handle h_Frame, t_Handle h_InputFrame, t_ProtocolInfo *p_ProtocolInfo)
{
    t_FrameStruct* p_Frame = (t_FrameStruct*)h_Frame;
    t_FrameStruct* p_NewFrame = NULL;
    t_PgFrameSetting *p_InputFrame = (t_PgFrameSetting*)h_InputFrame;
    e_PgHeaderType headerId = e_HEADER_TYPE_NONE;
    t_Handle        h_Header = NULL;

    /* Check the frame handle */
    if (!p_InputFrame || !p_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_FRAME, (""));
    }

    headerId = p_InputFrame->initHeaderType;
    if ( (headerId <= e_HEADER_TYPE_NONE) || (headerId >= e_MAX_HEADER_TYPE_COUNT) )
    {
         REPORT_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid frame type provided!"));
         return NULL;
    }

    while(headerId != e_HEADER_TYPE_NONE)
    {
        /* Check if the frame is empty */
        if (p_Frame->HeaderID == e_HEADER_TYPE_NONE)
        {
            /*Init the frame (the first header becomes the frame)*/
            p_NewFrame = p_Frame;
        }
        else
        {
           /* Create the new header (frame) */
            p_NewFrame = (t_FrameStruct*)MEMALLOC(sizeof(t_FrameStruct));

            if (!p_NewFrame)
            {
                REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
                return NULL;
            }

            memset(p_NewFrame, 0, sizeof(t_FrameStruct));
            p_NewFrame->node.p_Next = NULL;
            p_NewFrame->node.p_Prev = NULL;

            /* Add the new header (frame) at the end of the list */
           INTERNAL_AddToTail(&p_Frame->node, &p_NewFrame->node);
        }

        /* Set the proper values */
        p_NewFrame->HeaderID = headerId;

        h_Header = pRegisteredHeaderArray[headerId].pfnGetHeader(h_InputFrame);
        p_NewFrame->h_HeaderStruct = pRegisteredHeaderArray[headerId].pfnConfig(h_Header,p_ProtocolInfo);

        if (!p_NewFrame->h_HeaderStruct)
        {
             REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Cannot configure the frame!"));
             return NULL;
        }

        /* Trigger the header specific randomization method */
        if (pRegisteredHeaderArray[headerId].pfnRandomizeHeader)
                pRegisteredHeaderArray[headerId].pfnRandomizeHeader(p_NewFrame);

        if(pRegisteredHeaderArray[headerId].pfnGetField(p_NewFrame,NEXT_HEADER_TYPE,&headerId,sizeof(headerId)) != E_OK)
        {
             REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Cannot get the next header type!"));
             return NULL;
        }
    }

    /* Reset Multiple Header Index counters to allow User Manipulation to refer to appropriate header.

       User Manipulation (Header Insertion/Header Replace/Header Manipulation) configures the header node (FRAME_ConfigureHeader)
       through the specific HDR_XXX_Config functions.  If not reset,  the 'hHeader' passed to HDR_XXX_Config refers to the
       the already incremented Header Index and fails to perform the expected User Manipulation */
    INTERNAL_ResetMultipleHeaderCounters();

   /* Return the new frame (header) */
    return p_Frame;
}

t_Handle FRAME_ConfigInsertHeader(t_Handle h_Frame, e_InsertionPrefference flags, t_Handle h_Header, e_PgHeaderType headerID, t_Handle h_NewHeader)
{
    t_FrameStruct *p_Frame = (t_FrameStruct*)h_Header;
    t_FrameStruct* p_NewFrame = NULL;

    UNUSED(h_Frame);

    if (!h_Header)
    {
        REPORT_ERROR(MAJOR, E_INVALID_FRAME, (""));
    }

    if ( (headerID <= e_HEADER_TYPE_NONE) || (headerID >= e_MAX_HEADER_TYPE_COUNT) )
    {
        REPORT_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid frame type provided!"));
        return NULL;
    }

    /* Create the new header (frame) */
      p_NewFrame = (t_FrameStruct*)MEMALLOC(sizeof(t_FrameStruct));

      if (!p_NewFrame)
      {
          REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
          return NULL;
      }

      memset(p_NewFrame, 0, sizeof(t_FrameStruct));
      p_NewFrame->node.p_Next = NULL;
      p_NewFrame->node.p_Prev = NULL;

      switch (flags)
      {
          case INSERT_AFTER:
          {
              if (INTERNAL_InsertAfterNode(&p_Frame->node, &p_NewFrame->node) != E_OK)
              {
                  REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Failed to chain the header in the frame"));
                  return NULL;
              }
          }
          break;
          case INSERT_BEFORE:
          {
              if (INTERNAL_InsertBeforeNode(&p_Frame->node, &p_NewFrame->node) != E_OK)
              {
                  REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Failed to chain the header in the frame"));
                  return NULL;
              }
          }
          break;
      }

      /* Set the proper values */
      p_NewFrame->HeaderID = headerID;
      p_NewFrame->h_HeaderStruct = pRegisteredHeaderArray[headerID].pfnConfig(h_NewHeader,NULL);

      /* trigger the header specific randomization method */
    if (pRegisteredHeaderArray[headerID].pfnRandomizeHeader)
            pRegisteredHeaderArray[headerID].pfnRandomizeHeader(p_NewFrame);

    /* Return the new frame (header) */
    return p_NewFrame;
}

t_Error FRAME_InsertHeader(e_InsertionPrefference flags, t_Handle h_Header, t_Handle h_NewHeader)
{
    t_FrameStruct *p_Frame     = (t_FrameStruct*)h_Header;
    t_FrameStruct *p_NewFrame  = (t_FrameStruct*)h_NewHeader;

    if (!h_Header || !h_NewHeader)
    {
        REPORT_ERROR(MAJOR, E_INVALID_FRAME, (""));
    }

   switch (flags)
   {
      case INSERT_AFTER:
      {
          if (INTERNAL_InsertAfterNode(&p_Frame->node, &p_NewFrame->node) != E_OK)
          {
              RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Failed to chain the header in the frame after the specified node"));
          }
      }
      break;
      case INSERT_BEFORE:
      {
          if (INTERNAL_InsertBeforeNode(&p_Frame->node, &p_NewFrame->node) != E_OK)
          {
              RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Failed to chain the header in the frame before the specified node"));
          }
      }
      break;
   }

   return E_OK;
}

t_Handle FRAME_ConfigureHeader(e_PgHeaderType headerID, t_Handle h_InputHeader, t_ProtocolInfo *p_ProtocolInfo)
{
    t_FrameStruct* p_NewFrame = NULL;
    t_Handle       h_Header = h_InputHeader;

    if ( (headerID <= e_HEADER_TYPE_NONE) || (headerID >= e_MAX_HEADER_TYPE_COUNT) )
    {
        REPORT_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid frame type provided!"));
        return NULL;
    }

    /* Create the new header (frame) */
    p_NewFrame = (t_FrameStruct*)MEMALLOC(sizeof(t_FrameStruct));

    if (!p_NewFrame)
    {
       REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
       return NULL;
    }

    memset(p_NewFrame, 0, sizeof(t_FrameStruct));
    p_NewFrame->node.p_Next = NULL;
    p_NewFrame->node.p_Prev = NULL;

    /* Set the proper values */
    p_NewFrame->HeaderID = headerID;
    p_NewFrame->h_HeaderStruct = pRegisteredHeaderArray[headerID].pfnConfig(h_Header, p_ProtocolInfo);

    /* trigger the header specific randomization method */
    if (pRegisteredHeaderArray[headerID].pfnRandomizeHeader)
            pRegisteredHeaderArray[headerID].pfnRandomizeHeader(p_NewFrame);

    /* Return the new frame (header) */
    return p_NewFrame;

}

t_Error FRAME_SetHeaderField(t_Handle h_Frame,  t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_FrameStruct* p_Frame = (t_FrameStruct*)h_Header;
    UNUSED(h_Frame);

    if (!p_Frame || !h_Value)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    //XX_Print("Frame type: %d FieldID: %d\n", p_Frame->HeaderID, fieldID);
    //INTERNAL_DebugPrintChain(&p_Frame->node);

   return  pRegisteredHeaderArray[p_Frame->HeaderID].pfnConfigField(p_Frame, fieldID, h_Value, size);
}

t_Error FRAME_FindAndSetHeaderField(t_Handle h_Frame, e_PgHeaderType headerID, uint8_t headerIndex, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_FrameStruct *p_TargetHeader = NULL;

    p_TargetHeader = FRAME_GetHeaderByTypeAndIndex(h_Frame, headerID, headerIndex);

    if (!p_TargetHeader)
    {
       return E_INVALID_ID;
    }

    return FRAME_SetHeaderField(h_Frame, p_TargetHeader, fieldID, h_Value, size);
}

t_Handle FRAME_GetInputHeader(e_PgHeaderType headerID, t_Handle h_InputFrame)
{
   t_PgFrameSetting *p_InputFrame = (t_PgFrameSetting*)h_InputFrame;
   t_Handle         h_Header = NULL;

   /* Check the frame handle */
   if (!p_InputFrame)
   {
       REPORT_ERROR(MAJOR, E_INVALID_FRAME, (""));
   }

   if ( (headerID <= e_HEADER_TYPE_NONE) || (headerID >= e_MAX_HEADER_TYPE_COUNT) )
   {
        REPORT_ERROR(MAJOR, E_INVALID_SELECTION, ("Invalid frame type provided!"));
        return NULL;
   }

   h_Header = pRegisteredHeaderArray[headerID].pfnGetHeader(h_InputFrame);

 /* Return the new header */
   return h_Header;

}

t_Handle FRAME_GetHeaderByTypeAndIndex(t_Handle h_Frame, e_PgHeaderType headerID, uint8_t headerIndex)
{
    t_FrameStruct *p_Header = (t_FrameStruct*)h_Frame;
    t_FrameStruct *p_NextHeader = p_Header;

    if (!h_Frame || !headerIndex)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return NULL;
    }

    if (p_Header->HeaderID == headerID)
        headerIndex--;

    while (headerIndex--)
    {
        /* find the next header of type*/
        p_NextHeader = FRAME_GetNextHeaderByType(h_Frame, p_NextHeader, headerID);
        /* if no such header */
        if (!p_NextHeader)
            break;
        /*else continue the loop*/
    }

    return p_NextHeader;
}

uint32_t FRAME_GetHeaderSizeByType(t_Handle h_Frame, e_PgHeaderType headerID)
{
    t_FrameStruct *p_Header = (t_FrameStruct*)h_Frame;
    t_FrameStruct *p_TargetHeader = NULL;

    if (!p_Header)
    {
       REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
       return 0;
    }

     p_TargetHeader = FRAME_GetHeaderByTypeAndIndex(h_Frame, headerID, 1);

     /* Return the size of the specified header */
     return pRegisteredHeaderArray[headerID].pfnGetHeaderSize(p_TargetHeader);
}

t_Error FRAME_FindAndGetHeaderField(t_Handle h_Frame, e_PgHeaderType headerID, uint8_t headerIndex, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_FrameStruct *p_TargetHeader = NULL;

    p_TargetHeader = FRAME_GetHeaderByTypeAndIndex(h_Frame, headerID, headerIndex);

    if (!p_TargetHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_ID, ("There is no header with the type and index specified"));
    }

    return FRAME_GetHeaderField(h_Frame, p_TargetHeader, fieldID, h_Value, size);
}

t_Error FRAME_ConfigHeaderAuto(t_Handle h_Frame, t_Handle h_Header, uint32_t fieldID, bool bSetAuto)
{
    t_FrameStruct* p_Frame = (t_FrameStruct*)h_Header;
    UNUSED(h_Frame);

    if (!h_Header)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

   return  pRegisteredHeaderArray[p_Frame->HeaderID].pfnConfigAuto(p_Frame, fieldID, bSetAuto);
}

t_Error FRAME_CloneFrame(t_Handle h_Frame, t_Handle h_SourceFrame )
{
    t_FrameStruct *p_DstFrame = (t_FrameStruct*)h_Frame;
    t_FrameStruct *p_NextDstFrame = NULL;
    t_FrameStruct *p_NextSrcFrame = NULL;
    t_FrameStruct *p_SrcFrame = (t_FrameStruct*)h_SourceFrame;

    if (!h_SourceFrame || !h_Frame)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    /* first, free the rest of the frame (the following header chain) */
    p_NextDstFrame = INTERNAL_GetNextHeader((t_FrameStruct*)p_DstFrame);

    if (p_NextDstFrame)
        FRAME_Free(p_NextDstFrame);

    /* free the custom data structure in the frame */
    if (p_DstFrame->h_HeaderStruct)
    {
        MEMFREE(p_DstFrame->h_HeaderStruct);
        p_DstFrame->h_HeaderStruct = NULL;
    }

    /* init the frame with the new values */
      p_DstFrame->HeaderID = p_SrcFrame->HeaderID;
      p_DstFrame->h_HeaderStruct = pRegisteredHeaderArray[p_DstFrame->HeaderID].pfnClone(p_SrcFrame);

      p_NextSrcFrame = (t_FrameStruct*)INTERNAL_GetNextHeader(p_SrcFrame);

      if (p_NextSrcFrame)
      {
          /* create the next destination frame (which will be chained to the current one) */
          p_NextDstFrame = (t_FrameStruct*)MEMALLOC(sizeof(t_FrameStruct));

          if (!p_NextDstFrame)
          {
              RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Failed to create the frame"));
          }

          memset(p_NextDstFrame, 0, sizeof(t_FrameStruct));
          p_NextDstFrame->node.p_Next = NULL;
          p_NextDstFrame->node.p_Prev = NULL;
          p_NextDstFrame->h_HeaderStruct = NULL;
          p_NextDstFrame->HeaderID = e_HEADER_TYPE_NONE;

          /*proceed with the recursive cloning of the rest of the source frame*/
          if (FRAME_CloneFrame(p_NextDstFrame, p_NextSrcFrame) != E_OK)
          {
              RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Failed to clone the frame"));
          }

          /* link the NextDstFrame to the DstFrame */
          INTERNAL_InsertAfterNode(&p_DstFrame->node, &p_NextDstFrame->node);
      }

    return E_OK;
}

t_Error FRAME_ConfigHeader(t_Handle h_Frame,  t_Handle h_Header, t_Handle hParams, uint32_t fieldIDs)
{
    t_FrameStruct* p_Frame = (t_FrameStruct*)h_Header;

    UNUSED(h_Frame);

    if (!p_Frame || !hParams)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
    }

    return pRegisteredHeaderArray[p_Frame->HeaderID].pfnConfigHeader(p_Frame, hParams, fieldIDs);
}

t_Error FRAME_Serialize(t_Handle h_Frame, uint8_t **p_Buffer, uint32_t* pSize)
{
    t_FrameStruct  *p_Frame = (t_FrameStruct*)h_Frame;
    uint8_t     *pOrigBufferStart = *p_Buffer;
    uint32_t    originalSize = *pSize;
    t_Handle    h_IphcHeader = NULL;
    t_Handle    h_IpHeader = NULL;
    t_Handle    h_PrevHeader = NULL;
    uint32_t ipOffset = 0;

    if (!p_Frame || !pSize || !p_Buffer)
     {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }

     /* each type of header will be responsible for calling serialize method on the next following neighbor */
     if (pRegisteredHeaderArray[p_Frame->HeaderID].pfnSerializeHeader(p_Frame, p_Buffer, pSize) != E_OK)
            return E_NOT_IN_RANGE;

     *p_Buffer   = pOrigBufferStart;

     /* Check if HEADER_TYPE_IPHC is present. If present serialize IPHC */
     h_IphcHeader = FRAME_GetNextHeaderByType(p_Frame, p_Frame, e_HEADER_TYPE_IPHC);
     if(h_IphcHeader)
     {
         /* Get IP offset to be given to the IPHC_Serialize function */
         h_IpHeader = FRAME_GetNextHeaderByType(p_Frame, p_Frame, e_HEADER_TYPE_IPv4);
         h_PrevHeader = INTERNAL_GetPrevHeader(h_IpHeader);
         ipOffset = INTERNAL_GetFrameSize(p_Frame,h_PrevHeader);
         *p_Buffer = *p_Buffer+ipOffset;
         if (pRegisteredHeaderArray[e_HEADER_TYPE_IPHC].pfnSerializeHeader(h_IphcHeader, p_Buffer, pSize) != E_OK)
             return E_NOT_IN_RANGE;
     }

     /* by now the frame has already been recursively serialized */
     /* pSize contains the size not used in the buffer, we need to return the frame size:*/
     *pSize     = originalSize - *pSize;
     *p_Buffer   = pOrigBufferStart;

     return E_OK;
}

t_Handle FRAME_GetNextHeaderByType(t_Handle h_Frame,  t_Handle h_Header, e_PgHeaderType headerID)
{
    t_FrameStruct *p_NextHeader = NULL;

    if(!h_Frame)
        return NULL;


    if(!h_Header)//get first header
        return h_Frame;

    p_NextHeader = (t_FrameStruct*)INTERNAL_GetNextHeader(h_Header);

    if ( (headerID == HEADER_TYPE_ANY) || (p_NextHeader && p_NextHeader->HeaderID == headerID) )
        return p_NextHeader;
    else
        return p_NextHeader ? FRAME_GetNextHeaderByType(h_Frame, p_NextHeader, headerID) : NULL;
}

t_Handle FRAME_GetPrevHeaderByType(t_Handle h_Frame,  t_Handle h_Header, e_PgHeaderType headerID)
{
    t_FrameStruct *p_PrevHeader = (t_FrameStruct*)INTERNAL_GetPrevHeader(h_Header);
    UNUSED(h_Frame);

    if ( (headerID == HEADER_TYPE_ANY) || (p_PrevHeader && p_PrevHeader->HeaderID == headerID) )
        return p_PrevHeader;
    else
        return p_PrevHeader ? FRAME_GetPrevHeaderByType(h_Frame, p_PrevHeader, headerID) : NULL;
}

t_Handle FRAME_GetPrevHeader(t_Handle h_Header)
{
    t_FrameStruct *p_PrevHeader = (t_FrameStruct*)INTERNAL_GetPrevHeader(h_Header);

    return p_PrevHeader ? p_PrevHeader : NULL;
}

t_Error FRAME_GetHeaderField(t_Handle h_Frame, t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size)
{
    t_FrameStruct *p_Header = (t_FrameStruct*)h_Header;
    UNUSED(h_Frame);

    if (!p_Header)
     {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
     }
    return pRegisteredHeaderArray[p_Header->HeaderID].pfnGetField(p_Header, fieldID, h_Value, size);
}

e_PgHeaderType FRAME_GetHeaderType(t_Handle h_Frame, t_Handle h_Header)
{
    t_FrameStruct *p_Header = (t_FrameStruct*)h_Header;
    UNUSED(h_Frame);

    if (!p_Header)
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return e_HEADER_TYPE_NONE;
     }

     return p_Header->HeaderID;
}

uint32_t FRAME_GetPayLoadOffest(t_Handle h_Frame)
{
    t_Handle    h_TmpFrame = NULL;
    t_FrameStruct  *p_Frame = NULL;
    uint32_t    payloadOffset = 0;
    int32_t     sizeOfHeader = 0;
    t_Error     rc = E_OK;
    bool        isFCS = FALSE;
    uint32_t    fcsType = 0;

    h_TmpFrame = INTERNAL_GetFirstHeader(h_Frame);

    p_Frame = (t_FrameStruct*)h_TmpFrame;

   while(p_Frame->HeaderID != e_HEADER_TYPE_PAYLOAD && p_Frame->HeaderID != e_HEADER_TYPE_RAW_DATA && p_Frame->HeaderID != e_HEADER_TYPE_NONE)
   {
       sizeOfHeader = (int32_t)(pRegisteredHeaderArray[p_Frame->HeaderID].pfnGetHeaderSize(h_TmpFrame));
       if(p_Frame->HeaderID==e_HEADER_TYPE_IPHC)
       {
           sizeOfHeader = -sizeOfHeader;
       }

       if(p_Frame->HeaderID==e_HEADER_TYPE_ETH)
       {
           rc = pRegisteredHeaderArray[p_Frame->HeaderID].pfnGetField(h_TmpFrame,e_FIELD_ETH_USE_FINAL_CKSUM,&isFCS,sizeof(isFCS));
           ASSERT_COND(rc == E_OK);
           if(isFCS)
           {
               sizeOfHeader -= ETH_FCS_SIZE;
           }
       }

       if(p_Frame->HeaderID==e_HEADER_TYPE_HDLC)
       {
           rc = pRegisteredHeaderArray[p_Frame->HeaderID].pfnGetField(h_TmpFrame,e_FIELD_HDLC_USE_FCS,&isFCS,sizeof(isFCS));
           ASSERT_COND(rc == E_OK);
           if(isFCS)
           {
               rc = pRegisteredHeaderArray[p_Frame->HeaderID].pfnGetField(h_TmpFrame,e_FIELD_HDLC_FCS_TYPE,&fcsType,sizeof(fcsType));
               ASSERT_COND(rc == E_OK);

               sizeOfHeader -= ((fcsType == e_FCS_16) ? HDLC_16_BIT_FCS_SIZE : HDLC_32_BIT_FCS_SIZE);
           }
       }

       payloadOffset += sizeOfHeader;
       h_TmpFrame = (t_FrameStruct *)INTERNAL_GetNextHeader(h_TmpFrame);
       p_Frame = (t_FrameStruct*)h_TmpFrame;
   }

    return payloadOffset;
}

uint32_t FRAME_GetFrameSize(t_Handle h_Frame, t_Handle h_Header)
{
    uint32_t frameSize = 0;
    t_Handle hNextHeader = NULL;

    UNUSED(h_Frame);

    if (!h_Header)
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, (""));
        return 0;
     }

     /* get the size of the rest of the frame */
     hNextHeader = INTERNAL_GetNextHeader(h_Header);
     frameSize = hNextHeader ? FRAME_GetFrameSize(h_Frame, hNextHeader) : 0;

     /* add the size of the current header to the size of the rest of the frame and return the value (total size) */
         return frameSize + (pRegisteredHeaderArray[((t_FrameStruct*)h_Header)->HeaderID].pfnGetHeaderSize(h_Header));
}

t_Error FRAME_GetHeaderError(t_Handle h_Frame, t_Handle h_Header, t_Handle h_Error)
{
   UNUSED(h_Frame);
   if (!h_Header)
   {
      RETURN_ERROR(MAJOR, E_INVALID_HANDLE, (""));
   }

   return (pRegisteredHeaderArray[((t_FrameStruct*)h_Header)->HeaderID].pfnGetHeaderError(h_Header, h_Error));
}

/**************************************************************************************************/
t_Error FRAME_BufferInsert(uint8_t **p_Buffer, uint32_t *bufferSize, uint8_t* p_NewBuffer, uint32_t newBufferSize, uint32_t offset, bool updateLength)
{
    /* the method assumes that p_Buffer is a piece of a bigger buffer - the starting point of p_Buffer will be shifted back */
    uint8_t *p_tempBuffer = *p_Buffer - newBufferSize;
    uint16_t size = 0;

    memcpy(p_tempBuffer, *p_Buffer, offset);
    memcpy(p_tempBuffer + offset, p_NewBuffer, newBufferSize);

    *p_Buffer = p_tempBuffer;

    if(updateLength)
    {
        size = (uint16_t)*bufferSize;
        memcpy(*p_Buffer + offset + newBufferSize - sizeof(uint16_t), &size, sizeof(uint16_t));
    }

    *bufferSize   += newBufferSize;

    return E_OK;
}

t_Error FRAME_BufferReplace(uint8_t **p_Buffer, uint32_t *bufferSize, uint8_t* p_NewBuffer, uint32_t newBufferSize, uint32_t offset)
{
    /* make sure the user does not attempt to replace a bigger portion than the buffer itself */
    ASSERT_COND(*bufferSize >= newBufferSize+offset);
    UNUSED(bufferSize);

    memcpy(*p_Buffer + offset, p_NewBuffer, newBufferSize);

    return E_OK;
}

t_Error FRAME_BufferRemove(uint8_t **p_Buffer, uint32_t *bufferSize, uint32_t size, uint32_t offset)
{
    /* if a pozitive offset is present, the method will remove the bits from the begining of the buffer */
    /* if a negative offset is present, the method will remove the bits from the end of the buffer*/
    *bufferSize -= size;

    ASSERT_COND((int32_t)*bufferSize > 0);

    if ((int32_t)offset >= 0)
    {
        /* overwrite the portion of the frame that is to be removed */
        memcpy(*p_Buffer + offset, *p_Buffer + offset + size, *bufferSize-offset);
    }

    return E_OK;
}

uint32_t FRAME_Compute32BitCRC(uint32_t crcAccum, uint8_t *pBuffer, uint32_t size)
{
     return INTERNAL_ComputeCRC(crcAccum, pBuffer, size);
}

uint16_t FRAME_Compute16BitCRC(uint16_t crcAccum, uint8_t *pBuffer, uint32_t size)
{
     return INTERNAL_Compute16BitCRC(crcAccum, pBuffer, size);
}

uint8_t FRAME_ComputeHEC(uint8_t *p_Buffer, uint32_t size)
{
     return INTERNAL_ComputeHEC(p_Buffer, size);
}

/** @} */ /* end of ExternalFrame group */

